#!/bin/sh
# =====================================================================
# This file is part of CARDAMOM (R) which is jointly developed by THALES
# and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
# Copyright (C) THALES 2000-2003. All rights reserved.
# Copyright (C) THALES 2004-2005. All rights reserved.
# Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
# All rights reserved.
# 
# CARDAMOM is free software; you can redistribute it and/or modify it
# under the terms of the GNU Library General Public License as published
# by the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# 
# CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
# License for more details.
# 
# You should have received a copy of the GNU Library General Public
# License along with CARDAMOM; see the file COPYING. If not, write to the
# Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
# =====================================================================

 
my_script=`basename $0`
current_dir=`pwd` 


## Usage
usage() {
    echo "$my_script: Gather and merge Emma information from different hosts, "
    echo "and use Emma to generate a html report."
    echo
    echo "usage: $my_script -o output [-h host1,...,hostN]"
    echo
    echo "options:"
    echo "    -o <output-dir>      Name of the directory used for emma generation."
    echo "                         This directory is created if it doesn't exist."
    echo "    -h <host1,...,hostN> List of remote hosts on which gcov infos" 
    echo "                         should be gathered."
    exit 1
}


check_emma() {
    emma_classpath=`echo $CLASSPATH | sed s%:%\\\n%g | grep emma.jar` 2> /dev/null
    if [ -z "$emma_classpath" ]; then
        echo "Emma should be installed and emma.jar should be present in CLASSPATH."
        exit 1
    fi
}
 
 
check_host() {
    host_to_check=$1
    cdmw_home=$2
    h=`ssh $host_to_check uname -n`
    if [ -z "$h" ]; then
        echo "Cannot reach host $host_to_check"
        exit 1
    fi
    
    _tmp_=`ssh $host_to_check ls -al $cdmw_home 2> /dev/null`
    if [ -z "$_tmp_" ]; then
        echo "Cannot find $cdmw_home on $host_to_check"
        exit 1
    fi
} 

 

### Check current directory
if [ ! -x $my_script ]; then
    echo "This script should be started from the Build/testcoverage directory."
    exit 1
fi

cdmw_home=${current_dir%Build/testcoverage}

### Check emma
check_emma
echo "EMMA CLASSPATH: $emma_classpath"


## Parse command line arguments.
while getopts o:h: option; do
    case $option in
        o) output_dir=$OPTARG ;;
        h) remote_hosts=$OPTARG ;;
       \?) usage ;;
    esac
done

if [ "$OPTIND" -le "$#" ]
then
   shift $(($OPTIND-1))
   echo "ERROR on input parameter $1"
   usage
   exit 1
fi

if [ -z "$output_dir" ]; then
    echo "Missing -o option."
    usage
fi 
if [ ! -d $output_dir ]; then
    mkdir -p $output_dir
    if [ $? != 0 ]; then
        exit 1
    fi
fi

remote_hosts=`echo $remote_hosts | sed "s%,% %g"`

for i in $remote_hosts; do
    check_host $i $cdmw_home
done


####
echo out dir = $output_dir
echo remote_hosts = $remote_hosts
echo cdmw_home = $cdmw_home
####


# gather all meta-data first
echo " gather all meta-data first"
emma_metadata=$output_dir/metadata.em
emma_metadata_in_files=""
for i in `find $cdmw_home -name "coverage.em"`; do
    # filter-out classes from */test/* directories
    # and from */idllib/* directories
    if [ -z "`echo $i | grep /test/`" -a -z "`echo $i | grep /idllib/`" ]; then
        emma_metadata_in_files="$emma_metadata_in_files -in $i"
    fi
done
java emma merge $emma_metadata_in_files -out $emma_metadata



### gather on localhost
emma_runtime_local=$output_dir/infos_localhost.ec
echo " gather emma runtime infos on localhost"
./emma_gather_local_infos.sh -c $cdmw_home -o $emma_runtime_local


### gather on remote host
remote_gathering() {
    host=$1
    cdmw_home=$2
    output_dir=$3
    
    remote_emma=`ssh $host mktemp /tmp/${my_script}.XXXXXX`
    echo "     remote_emma=$remote_emma"
    echo " gather infos on $host"
    ssh $host "( export PATH=$PATH; cd $current_dir; ./emma_gather_local_infos.sh -c $cdmw_home -o $remote_emma)"
    echo " copy infos from $host"
    echo "     into $output_dir/infos_${host}.ec"
    scp $host:$remote_emma $output_dir/infos_${host}.ec
    
    ssh $host rm $remote_emma
}


### gather on all remote hosts
for i in $remote_hosts; do
    remote_gathering $i $cdmw_home $output_dir
done

### merge meta-data and runtime data
echo " merge meta-data and runtime infos"
emma_final=$output_dir/all_infos.es
emma_add_option="-in $emma_metadata -in $emma_runtime_local"
for i in $remote_hosts; do
    emma_add_option="$emma_add_option -in ${output_dir}/infos_${i}.ec"
done
echo "  > java emma merge $emma_add_option -out $emma_final"
java emma merge $emma_add_option -out $emma_final


### generate html report
echo " generate html report in $output_dir"
source_path=
for i in `find $cdmw_home -type d -name java`; do
    source_path="$source_path:$i/src"
done
mkdir -p $output_dir
java emma report \
    -r html,txt -Dreport.columns=name,line -Dreport.metrics=line:60 \
    -Dreport.html.out.file=$output_dir/index.html \
    -Dreport.txt.out.file=$output_dir/coverage.txt \
    -sp $source_path -in $emma_final
    


