/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO 
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2009. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#ifndef INCL_CCM_CIF_LB_CCMHOME_IMPL_HPP
#define INCL_CCM_CIF_LB_CCMHOME_IMPL_HPP

#include <map>

#include <CCMContainer/ccmcif/CCMHome_impl.hpp>
#include <LoadBalancing/idllib/CdmwLBGroupManager.stub.hpp>


namespace Cdmw {

namespace CCM {

namespace CIF {



/*
 * This class is the base class for home embedded in a Service or Session 
 * container (using a SessionContext) which creates LB replicated components.
 */
   
class LBCCMHome_impl : public CCMHome_impl
{
public:
    /**
    * Purpose:
    * <p>
    *   Constructor.
    *
    *@param rep_id       The home repository ID
    *@param comp_rep_id  The component repository ID
    *@param deactivator  The Object Deactivator responsible to 
    *                    deactivate home at its removal.
    *@param config       The config values used to specify if
    *                    the home is LB or not
    *                    
    *@exception Any CORBA Exception.
    */ 
    LBCCMHome_impl(const std::string&                         rep_id,
                   const std::string&                         comp_rep_id,
                   const Cdmw::LifeCycle::ObjectDeactivator&  deactivator,
                   CdmwLifeCycle::ObjectRegistration_ptr      object_registration,
                   const Components::ConfigValues&            config)
       throw(CORBA::SystemException);

    virtual ~LBCCMHome_impl() throw();
    
   /**
    * Purpose:
    * <p>This method is called by Container after Home creation.
    * 
    *@param ctxt Component context given by container.
    *
    *@exception Components::CCMException Any Component exception
    *@exception CORBA::SystemException Any CORBA system exception
    */
    void set_context(CdmwCcmContainer::CCM2Context_ptr ctxt)
       throw (Components::CCMException,
              CORBA::SystemException);

protected:
   /**
    * Purpose:
    * <p>
    *   This creates new component reference and add it as member of the 
    *   LB group with name specified as "LoadBalancingGroupName" ConfigValue 
    *   in config parameter.
    *
    *@param config The ConfigValues to be used for component creation.
    *@param comp_exec The component's executor.
    *@return A reference on the new component.
    *@exception Components::CreateFailure if application errors.
    *@exception Any other CORBA Exception.
    */ 
    CORBA::Object_ptr create_component_ref(
          const Components::ConfigValues& config, 
          Components::EnterpriseComponent_ptr  comp_exec = 
             Components::EnterpriseComponent::_nil())
        throw(Components::CreateFailure,
              CORBA::SystemException);

    /**
     * Purpose:
     * <p>
     *   Notify supervision that a component reference has been added to a 
     *   FT group. Code is in generated Home servant inheriting from this class.
     *
     *@param component_name The component's name
     */
    virtual void notify_add_member(const char* component_name) = 0;
        
    
private:
    // Hide copy constructor and =operator
    LBCCMHome_impl(const LBCCMHome_impl&);
    void operator=(const LBCCMHome_impl&);
    
    // get a group reference from it's name
    ::PortableGroup::ObjectGroup_ptr get_group_for_name(const char* name)
        throw(Components::CreateFailure, CORBA::SystemException);
        
    // get current LB location 
    ::PortableGroup::Location* get_current_lb_location();
              
    
    // LBGroupManager reference    
    CdmwLB::LBGroupManager_var  m_lb_grp_mgr;

    
}; // end class LBCCMHome_impl

} // End of namespace CIF

} // End of namespace CCM

} // End of namespace Cdmw

#endif // INCL_CCM_CIF_LB_CCMHOME_IMPL_HPP
