/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO 
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2009. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


// Cdmw files
#include <Foundation/orbsupport/CORBA.hpp>
#include <Foundation/orbsupport/OrbSupport.hpp>
#include <Foundation/orbsupport/ExceptionMinorCodes.hpp>
#include <Foundation/common/Assert.hpp>
#include <Foundation/logging/CdmwLogging.stub.hpp>


// Cdmw CCM files
#include <CCMContainer/ccmcif/CCMContextExt_impl.hpp>
#include <CCMContainer/ccmcommon/CCMUtils.hpp>
#include <CCMContainer/ccmcommon/CCMLoggers.hpp>

namespace 
{
    const std::string UNAVAILABLE_SERVICE_IDENTIFIERS[] = {"RootPOA", 
                                                           "POACurrent", 
                                                           "ORBPolicyManager",
                                                           "PolicyCurrent",
                                                           "PICurrent"};
    // size of the table is sizeof / 4 octets
    const int UNAVAILABLE_SERVICE_IDENTIFIERS_NB 
       = sizeof(UNAVAILABLE_SERVICE_IDENTIFIERS)/sizeof(UNAVAILABLE_SERVICE_IDENTIFIERS[0]);

       
       
    // Specific Cookie implementation to ease Service search
    class Cookie_impl : virtual public OBV_Components::Cookie,
                        virtual public CORBA::DefaultValueRefCountBase
    {
        public:
            Cookie_impl(const std::string service_id)
            {
                CORBA::OctetSeq_var cookie_value = 
                    Cdmw::CCM::Common::string_to_OctetSeq(service_id.c_str());
                cookieValue(cookie_value.in());
            }
            
            // Copy Constructor
            Cookie_impl::Cookie_impl (const Cookie_impl& right)
            {
                cookieValue(right.cookieValue());
            }

            std::string get_cookie_string() const
            {
                CORBA::String_var str = 
                    Cdmw::CCM::Common::OctetSeq_to_string(cookieValue());
                std::string std_str(str.in());
                return std_str;
            }

            virtual CORBA::ValueBase* _copy_value()
            {
                return new Cookie_impl(get_cookie_string());
            }
    };
    
       
} // end anonymous namespace


namespace Cdmw {

namespace CCM {

namespace CIF {

//
// IDL:omg.org/CdmwComponents/CCMContextExt:1.0
//  
CCMContextExt_impl::CCMContextExt_impl(Components::CCMContext_ptr ctx,
                                       CdmwLogging::LoggerFactory_ptr logger_factory)
{
    // create logger with domain for ccmcif
    m_logger = logger_factory->get_logger(CCMLoggers::CCMCIF_LOG_DOMAIN);
    
    CDMW_LOG_FUNCTION(m_logger.in(), "");
    
    m_context = CdmwCcmContainer::CCM2Context::_narrow(ctx);
    
    if (CORBA::is_nil(m_context.in()))
    {
        // This is to avoid Assertion failure in debug mode as internal ref count
        // equals 1 (CORBA C++ mapping oblige!)
        _set_ref_count(0);
    
        throw CORBA::BAD_PARAM(OrbSupport::BAD_PARAMWrongContextType, 
                               CORBA::COMPLETED_NO);
    }

}

CCMContextExt_impl::~CCMContextExt_impl()
{
    CDMW_LOG_FUNCTION(m_logger.in(), "");
}

//
// IDL:omg.org/Components/CCMContext/get_caller_principal:1.0
//
Components::Principal_ptr 
CCMContextExt_impl::get_caller_principal()
    throw(CORBA::SystemException)
{
    CDMW_LOG_FUNCTION(m_logger.in(), "");
    return m_context->get_caller_principal();
}

//
// IDL:omg.org/Components/CCMContext/get_CCM_home:1.0
//
Components::CCMHome_ptr  
CCMContextExt_impl::get_CCM_home()
    throw(CORBA::SystemException)
{
    CDMW_LOG_FUNCTION(m_logger.in(), "");
    return m_context->get_CCM_home();
}

//
// IDL:omg.org/Components/CCMContext/get_rollback_only:1.0
//
CORBA::Boolean  
CCMContextExt_impl::get_rollback_only()
    throw(Components::IllegalState,
          CORBA::SystemException)
{
    CDMW_LOG_FUNCTION(m_logger.in(), "");
    return m_context->get_rollback_only();
}

//
// IDL:omg.org/Components/CCMContext/get_user_transaction:1.0
//
Components::Transaction::UserTransaction_ptr  
CCMContextExt_impl::get_user_transaction()
    throw(Components::IllegalState,
          CORBA::SystemException)
{
    CDMW_LOG_FUNCTION(m_logger.in(), "");
    return m_context->get_user_transaction();
}

//
// IDL:omg.org/Components/CCMContext/is_caller_in_role:1.0
//
CORBA::Boolean  
CCMContextExt_impl::is_caller_in_role(const char* role)
    throw(CORBA::SystemException)
{
    CDMW_LOG_FUNCTION(m_logger.in(), "");
    return m_context->is_caller_in_role(role);
}

//
// IDL:omg.org/Components/CCMContext/set_rollback_only:1.0
//
void  
CCMContextExt_impl::set_rollback_only()
    throw(Components::IllegalState,
          CORBA::SystemException)
{
    CDMW_LOG_FUNCTION(m_logger.in(), "");
    m_context->set_rollback_only();
}


//
// IDL:omg.org/Components/CCMContext/resolve_service_reference:1.0
//
CORBA::Object_ptr 
CCMContextExt_impl::resolve_service_reference(const char* service_id) 
    throw(Components::CCMException,
          CORBA::SystemException)
{
    CDMW_LOG_FUNCTION(m_logger.in(), "service_id="<<service_id);
    
    // Check if service access is authorized
    std::string id_str = service_id;
    for (int i=0 ; i < UNAVAILABLE_SERVICE_IDENTIFIERS_NB ; i++)
    {
        if (id_str == UNAVAILABLE_SERVICE_IDENTIFIERS[i]) {
            CDMW_ERROR(m_logger.in(), 
                "Attempt to access unauthorized service: " << service_id);
            Components::CCMException ex(Components::OBJECT_NOT_FOUND);
            CDMW_LOG_FUNCTION_EXCEPTION(ex);
            throw ex;
        }
    }
    
    // try to find service in ServicesMap first
    ServicesMap::iterator service_pos = m_services_map.find(service_id);
    if (service_pos != m_services_map.end())
    {
        CDMW_DEBUG(m_logger.in(), 
            "Service " << service_id << " found in ServiceMap");
        // service found, return it
        return CORBA::Object::_duplicate(service_pos->second.in());
    }
    
    // call CCM2Context::resolve_initial_references()
    try
    {
        CDMW_DEBUG(m_logger.in(), 
            "Service " << service_id << " not found in ServiceMap. "
            << "Try to get it from CCM2Context.");
        return m_context->resolve_initial_references(service_id);
    }
    catch (const CdmwCcmContainer::CCM2Context::InvalidName& )
    {
        CDMW_ERROR(m_logger.in(), 
            "Service " << service_id << " has InvalidName !");
        Components::CCMException ex(Components::OBJECT_NOT_FOUND);
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw ex;
    }
    catch (const CORBA::SystemException& e)
    {
        CDMW_ERROR(m_logger.in(), 
            "Exception received getting service " << service_id << " : "
            << Cdmw::OrbSupport::OrbSupport::exception_to_string(e));
        CDMW_LOG_FUNCTION_EXCEPTION(e);
        throw;
    }
    
}



//
// IDL:omg.org/CdmwCcmComponents/CCMContextExt/get_service:1.0
//
CORBA::Object_ptr 
CCMContextExt_impl::get_service(const char* identifier)
    throw(CdmwComponents::CCMContextExt::ServiceUnavailable,
          CORBA::SystemException)
{
    // Deprectated !!
    // But implement it as delegation to resolve_service_reference()
    CDMW_WARN(m_logger.in(), 
              "Operation CCMContextExt::get_service() is deprecated. " <<
              "Use CCMContext::resolve_service_reference() instead.");
    
    try {
        return this->resolve_service_reference(identifier);
    } catch (const Components::CCMException& e) {
        // service not found
        throw CdmwComponents::CCMContextExt::ServiceUnavailable();
    }
}


//
// IDL:omg.org/Components/ExtensionContext/install_service_reference:1.0
//
Components::Cookie* 
CCMContextExt_impl::install_service_reference(const char* service_id, 
                                              CORBA::Object_ptr objref) 
    throw(Components::CCMException,
          CORBA::SystemException)
{
    CDMW_LOG_FUNCTION(m_logger.in(), "service_id="<<service_id);
    
    // check if service is already installed
    ServicesMap::iterator service_pos = m_services_map.find(service_id);
    if (service_pos != m_services_map.end())
    {
        CDMW_ERROR(m_logger.in(), 
            "Service " << service_id << " is already installed.");
        // service already installed
        Components::CCMException ex(Components::SERVICE_INSTALLATION_ERROR);
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw ex;
    }

    // add service to ServicesMap    
    m_services_map.insert(
        ServicesMap::value_type(service_id, 
                                CORBA::Object::_duplicate(objref)));

    // return Cookie with service_id inside
    return new Cookie_impl(service_id);
}

//
// IDL:omg.org/Components/ExtensionContext/uninstall_service_reference:1.0
//
CORBA::Object_ptr 
CCMContextExt_impl::uninstall_service_reference(Components::Cookie* ck) 
    throw(Components::CCMException,
          CORBA::SystemException)
{
    CDMW_LOG_FUNCTION(m_logger.in(), "");
    
    // extract service_id from Cookie
    Cookie_impl* cookie = dynamic_cast<Cookie_impl*>(ck);
    if (cookie == 0) {
        CDMW_ERROR(m_logger.in(), 
            "Invalid Cookie. Cannot uninstall service.");
        Components::CCMException ex(Components::SERVICE_INSTALLATION_ERROR);
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw ex;
    }
    std::string service_id = cookie->get_cookie_string();
    
    CDMW_DEBUG(m_logger.in(), 
        "Uninstall service " << service_id);
    
    // check if service is installed
    ServicesMap::iterator service_pos = m_services_map.find(service_id);
    if (service_pos == m_services_map.end())
    {
        CDMW_ERROR(m_logger.in(), 
            "Service " << service_id << " is not installed");
        // service not installed
        Components::CCMException ex(Components::SERVICE_INSTALLATION_ERROR);
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw ex;
    }

    // remove service from ServicesMap    
    CORBA::Object_var obj = 
        CORBA::Object::_duplicate(service_pos->second.in());
    m_services_map.erase(service_id);
    return obj._retn();
}
              


} // end namespace CIF
} // end namespace CCM
} // end namespace Cdmw



