/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO 
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2009. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#include <iostream>
#include <list>

// Cdmw Files
#include <Foundation/orbsupport/CORBA.hpp>
#include <Foundation/orbsupport/OrbSupport.hpp>
#include <Foundation/orbsupport/ExceptionMinorCodes.hpp>
#include <Foundation/common/Assert.hpp>
#include <Foundation/logging/FunctionLogger.hpp>

// Cdmw CCM Files
#include <CCMContainer/ccmcif/CCMHome_impl.hpp>
#include <CCMContainer/ccmcif/CCMObject_impl.hpp>
#include <CCMContainer/ccmcif/CCMUtils.hpp>
#include <CCMContainer/ccmcommon/CCMUtils.hpp>
#include <CCMContainer/ccmcommon/CCMLoggers.hpp>
#include <ccmcif/SessionContext.hpp>
#include <Foundation/common/AlreadyExistException.hpp>

#include <Foundation/idllib/CdmwDeployment.stub.hpp>


namespace Cdmw {

namespace CCM {

namespace CIF {


//
// IDL:thalesgroup.com/CdmwCcmCif/CdmwCCMHome:1.0
//
CCMHome_impl::CCMHome_impl(const std::string&                         rep_id,
                           const std::string&                         comp_rep_id,
                           const Cdmw::LifeCycle::ObjectDeactivator&  deactivator,
                           CdmwLifeCycle::ObjectRegistration_ptr      object_registration,
                           const Components::ConfigValues&            config,
                           const bool                                 config_expected)
  throw(CORBA::SystemException)
  : LifeCycleSingleObject(deactivator),
    m_context(0),
    m_oid(deactivator.get_object_id()),
    m_rep_id(rep_id),
    m_comp_rep_id(comp_rep_id),
    m_object_registration(CdmwLifeCycle::ObjectRegistration::_duplicate(object_registration)),
    m_config_expected(config_expected),
    m_shlib_handle(0)
{
    CDMW_LOG_FUNCTION(CCMLoggers::Get_ccmcif_logger(), 
        "rep_id="<<rep_id<<"  comp_rep_id="<<comp_rep_id);

   if ( (!m_config_expected) && (config.length() > 0) )
   {
       // No ConfigValues expected, but some still are present !
       CDMW_DEBUG(CCMLoggers::Get_ccmcif_logger(), 
            "CCMHome_impl is not inherited. No FT ConfigValues expected.");
 
       std::string config_names = "";
       for (CORBA::ULong i=0; i<config.length(); ++i) {
           config_names += config[i]->name();
           config_names += " ";
       }
       
       _set_ref_count(0);
       CDMW_ERROR(CCMLoggers::Get_ccmcif_logger(),
                "No ConfigValues expected in CCMHome_impl constructor."
                   << "Following were found: " << config_names);
       CORBA::BAD_PARAM ex(OrbSupport::BAD_PARAMInvalidConfigValues, 
                                CORBA::COMPLETED_NO);
       CDMW_LOG_FUNCTION_EXCEPTION(ex);
       throw ex;

   }
}

CCMHome_impl::~CCMHome_impl()
  throw()
{
    CDMW_LOG_FUNCTION(CCMLoggers::Get_ccmcif_logger(), "");
    // in case remove was not called, m_context is not deleted. Delete it.
    if (m_context != 0) {
        delete m_context;
        m_context = 0;
    }
    
    // clear map before to unload library, in order to have component
    // executors removed before their destructors are gone !
    m_components.clear();   
    
    // unload dynamic library
    if (m_shlib_handle != 0) {
        CDMW_DEBUG(CCMLoggers::Get_ccmcif_logger(), "dlclose on home library");
        Cdmw::OsSupport::OS::dlclose(m_shlib_handle);
        delete m_shlib_handle;
        m_shlib_handle = 0;
    }
}

//
// IDL:omg.org/Components/CCMHome/get_component_def:1.0
//
CORBA::IRObject_ptr
CCMHome_impl::get_component_def()
    throw(CORBA::SystemException)
{
    CDMW_LOG_FUNCTION(CCMLoggers::Get_ccmcif_logger(), "");
    check_is_removed(m_oid.in());
    CDMW_ASSERT(m_context);

    // TODO: Implementation
    throw CORBA::NO_IMPLEMENT(Cdmw::OrbSupport::NO_IMPLEMENT,
                              CORBA::COMPLETED_NO);

    CORBA::IRObject_ptr _r = CORBA::IRObject::_nil();
    return _r;
}

//
// IDL:omg.org/Components/CCMHome/get_home_def:1.0
//
CORBA::IRObject_ptr
CCMHome_impl::get_home_def()
    throw(CORBA::SystemException)
{
    CDMW_LOG_FUNCTION(CCMLoggers::Get_ccmcif_logger(), "");
    check_is_removed(m_oid.in());
    CDMW_ASSERT(m_context);

    // TODO: Implementation
    throw CORBA::NO_IMPLEMENT(Cdmw::OrbSupport::NO_IMPLEMENT,
                              CORBA::COMPLETED_NO);

    CORBA::IRObject_ptr _r = CORBA::IRObject::_nil();
    return _r;
}


//
// IDL:omg.org/Components/CCMHome/remove_component:1.0
//
void
CCMHome_impl::remove_component(Components::CCMObject_ptr comp)
    throw(Components::RemoveFailure,
          CORBA::SystemException)
{
    CDMW_LOG_FUNCTION(CCMLoggers::Get_ccmcif_logger(), "");
    check_is_removed(m_oid.in());
    CDMW_ASSERT(m_context);

    if (CORBA::is_nil(comp)) {
        CDMW_ERROR(CCMLoggers::Get_ccmcif_logger(),
            "Cannot remove nil component.");
        CORBA::BAD_PARAM ex(OrbSupport::BAD_PARAMNilObjectReference, 
                            CORBA::COMPLETED_NO);
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw ex;
    }
    
    try
    {
        Components::CCMObject_var the_comp = Components::CCMObject::_duplicate(comp);
        CORBA::String_var comp_oid_str = CORBA::string_dup("");
        try {
            comp_oid_str = m_context->get_oid_str_from_ref(the_comp.in());
CDMW_INFO(CCMLoggers::Get_ccmcif_logger(), "***** YES, the original component is found !!! : comp_oid=" << comp_oid_str.in());
        } catch (Components::BadComponentReference& ex) {
            // PCR-1522 : If Component ref is not found via get_oid_str_from_ref(),
            // try to retrieve it via _is_equivalent()
            // In case of a redirect reference to a local component, let's use this hack!
            try {
                CORBA::Object_var target = comp->_get_component();
                the_comp = Components::CCMObject::_narrow(target.in());
                CDMW_ASSERT(!CORBA::is_nil(the_comp.in()));
                comp_oid_str = m_context->get_oid_str_from_ref(the_comp.in());
CDMW_INFO(CCMLoggers::Get_ccmcif_logger(), "***** YES, the target component is found !!! : comp_oid=" << comp_oid_str.in());
            } catch (Components::BadComponentReference& ex){
                // Definitely not from here!
                CDMW_ERROR(CCMLoggers::Get_ccmcif_logger(),
                    "Components::BadComponentReference Exception raised when removing component!");
            } 
        }   
        
        // We don't test if comp_oid_str is empty or not but assume that find will return end()
        ComponentMap::iterator comp_iter = m_components.find(comp_oid_str.in());
        
        if ( comp_iter != m_components.end() ) 
        {
            try
            {
                // ask to the component its removal
                the_comp->remove();
            }
            catch (const Components::RemoveFailure& e)
            {
                CDMW_ERROR(CCMLoggers::Get_ccmcif_logger(),
                    "RemoveFailure Exception raised when removing component");
                CDMW_LOG_FUNCTION_EXCEPTION(e);
                throw;
            }
            catch (const CORBA::OBJECT_NOT_EXIST& e)
            {
                CDMW_ERROR(CCMLoggers::Get_ccmcif_logger(),
                    "This component is being removed!");
                Components::RemoveFailure ex(
                    CdmwDeployment::BAD_COMPONENT_REFERENCE);
                CDMW_LOG_FUNCTION_EXCEPTION(ex);
                throw ex;
            }
            catch (const CORBA::SystemException& e)
            {
                CDMW_ERROR(CCMLoggers::Get_ccmcif_logger(),
                    "System exception raised when removing component: " <<
                    Cdmw::OrbSupport::OrbSupport::exception_to_string(e));
                CDMW_LOG_FUNCTION_EXCEPTION(e);
                throw;
            }
        }
        else
        {
            // this component has not been created by this home
            CDMW_ERROR(CCMLoggers::Get_ccmcif_logger(),
                "This component has not been created by this home or has already been removed!");
            Components::RemoveFailure ex(CdmwDeployment::BAD_COMPONENT_REFERENCE);
            CDMW_LOG_FUNCTION_EXCEPTION(ex);
            throw ex;
        }
    }
    catch (const Components::IllegalState& )
    {
        CDMW_ERROR(CCMLoggers::Get_ccmcif_logger(),
            "IllegalState exception raised!");
        Components::RemoveFailure ex(
            CdmwDeployment::ILLEGAL_STATE);
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw ex;
    }
    catch (const Components::BadComponentReference& )
    {
        CDMW_ERROR(CCMLoggers::Get_ccmcif_logger(),
            "BadComponentReference exception raised!");
        Components::RemoveFailure ex(
            CdmwDeployment::BAD_COMPONENT_REFERENCE);
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw ex;
    }
    catch (const CORBA::SystemException& ex)
    {
        CDMW_ERROR(CCMLoggers::Get_ccmcif_logger(),
            "System exception raised when removing component: " <<
            Cdmw::OrbSupport::OrbSupport::exception_to_string(ex));
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw;
    }
}

//
// IDL:omg.org/CosLifeCycle/LifeCycleObject/remove:1.0
//
void 
CCMHome_impl::remove()
throw(CosLifeCycle::NotRemovable,
      CORBA::SystemException)
{
    CDMW_LOG_FUNCTION(CCMLoggers::Get_ccmcif_logger(), "");
    check_is_removed(m_oid.in());
    CDMW_ASSERT(m_context);

    try 
    {
        // perform a copy of components from the map first, since remove of each
        // component will automatically remove the component from the map via
        // delete_component_info() called after req_passivate().
        std::list<Components::CCMObject_var> comp_list;
        ComponentMap::iterator iter = m_components.begin();
        while (iter != m_components.end()) {
            comp_list.push_back(
                Components::CCMObject::_narrow(
                    iter->second.comp_ref.in()));
            iter++;
        }
        
        std::list<Components::CCMObject_var>::iterator comp_iter = comp_list.begin();
        while (comp_iter != comp_list.end())
        {
            try
            {
                (*comp_iter)->remove();
            } 
            catch (const Components::RemoveFailure&)
            {
                // Ignore
                CDMW_WARN(CCMLoggers::Get_ccmcif_logger(),
                    "CCMHome_impl remove: remove failure when trying to remove a component! Ignore it");
            }
            catch (const CORBA::OBJECT_NOT_EXIST&)
            {
                // Ignore
                CDMW_WARN(CCMLoggers::Get_ccmcif_logger(),
                    "CCMHome_impl remove: component is being removed! Ignore it");
            }
            catch (const CORBA::SystemException&)
            {
                // Ignore
                CDMW_WARN(CCMLoggers::Get_ccmcif_logger(),
                    "CCMHome_impl remove: System exception raised when removing component! Ignore it");
            }

                
            comp_iter++;
        }

        // clean component list
        m_components.clear();
    } 
    catch (const PortableServer::Current::NoContext & ) 
    {
        // Not called within the context of an operation invocation
        CDMW_ERROR(CCMLoggers::Get_ccmcif_logger(),
            "NoContext exception: cannot remove home.");
        CORBA::INTERNAL ex(Cdmw::OrbSupport::INTERNALNoContext,
                           CORBA::COMPLETED_NO);
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw ex;
    } 
    catch (const CORBA::SystemException & e) 
    {
        CDMW_ERROR(CCMLoggers::Get_ccmcif_logger(),
            "System exception raised when removing home: " <<
            Cdmw::OrbSupport::OrbSupport::exception_to_string(e));
        CDMW_LOG_FUNCTION_EXCEPTION(e);
        // rethrow
        throw;
    }

    LifeCycle::LifeCycleSingleObject::remove();
    
    delete m_context;
    m_context = 0;
}

//
// IDL:thalesgroup.com/CdmwLifeCycle/FactoryBase/supports:1.0
//
CORBA::Boolean
CCMHome_impl::supports(const char* repository_id)
        throw(CORBA::SystemException)
{
    CDMW_LOG_FUNCTION(CCMLoggers::Get_ccmcif_logger(), 
        "repository_id="<<repository_id);
    check_is_removed(m_oid.in());
    CDMW_ASSERT(m_context);

    CORBA::Boolean result(LifeCycle::ServantProviderBase::supports(repository_id));
    CDMW_LOG_FUNCTION_RETURN(result);
    return result;
}


//
// IDL:thalesgroup.com/CdmwLifeCycle/FactoryBase/get_supported_objects:1.0
//
CdmwLifeCycle::RepositoryIdSeq* 
CCMHome_impl::get_supported_objects()
    throw(CORBA::SystemException)
{
    CDMW_LOG_FUNCTION(CCMLoggers::Get_ccmcif_logger(), "");
    check_is_removed(m_oid.in());
    CDMW_ASSERT(m_context);

    return LifeCycle::ServantProviderBase::get_supported_objects();
}


//
// ServantProvider abstract operation implementation
//
 
/**
* Purpose:
* <p>This method provides a servant for the CORBA object with
* the specified ObjectId. It is called by the Container
* whenever a servant is needed to incarnate an object.
* 
*@param oid ObjectId of the object to be incarnated
*
*@return A servant incarnating the object 
*
*@exception PortableServer::ForwardRequest Indicates to the Life Cycle
* Framework ORB that the SevantProvider cannot handle this object 
* and requests can be handled to the returned IOR. If this exception 
* is translated/forwarded to the client-side ORB, the request invocation 
* will be transparently re-transmitted to the returnded object.
*@exception CORBA::SystemException Any CORBA system exception
*/
PortableServer::Servant        
CCMHome_impl::lookup_servant(const PortableServer::ObjectId& oid) 
    throw(PortableServer::ForwardRequest,
          CORBA::SystemException)
{
    CDMW_LOG_FUNCTION(CCMLoggers::Get_ccmcif_logger(), "");
    check_is_removed(m_oid.in());
    CDMW_ASSERT(m_context);

    CDMW_DEBUG(CCMLoggers::Get_ccmcif_logger(), 
        "Entering in lookup_servant...");

    PortableServer::ServantBase_var servant = 0;

    try  
    {
        // check if it is a component oid or a facet/consumer oid or 
        // a local push consumer oid.
        CORBA::String_var oid_str = PortableServer::ObjectId_to_string(oid);
        std::string object_id(oid_str.in());
        CCMUtils::NameVector names = CCMUtils::Read_oid(object_id);

        std::string comp_oid = names[0];
        CDMW_DEBUG(CCMLoggers::Get_ccmcif_logger(), 
            "oid=" << object_id << "  comp_oid=" << comp_oid);
        
        // check a component with this oid has been created in this home
        ComponentMap::iterator comp_pos = m_components.find(comp_oid);
        
        if (comp_pos  == m_components.end()) 
        {
            CDMW_ERROR(CCMLoggers::Get_ccmcif_logger(), 
                "Cannot find component with oid: " << comp_oid);
            CORBA::OBJECT_NOT_EXIST ex(Cdmw::OrbSupport::OBJECT_NOT_EXISTCCMObjectNotFound,
                                       CORBA::COMPLETED_NO);
            CDMW_LOG_FUNCTION_EXCEPTION(ex);
            throw ex;
        }

        ComponentMap::value_type& comp_value = *comp_pos;
        
        if (comp_value.second.comp_servant.in())
        {
            // because the call to release servant comes from POA, it may occurs that
            // the object is deactivated but release_servant not called and so the 
            // component has not been deleted from the map
            CCMObject_impl*  component 
                = dynamic_cast<CCMObject_impl*>(comp_value.second.comp_servant.in());

            if (component && component->is_removed()) 
            {
                CDMW_ERROR(CCMLoggers::Get_ccmcif_logger(), 
                    "Component found but has been removed!");
                CORBA::OBJECT_NOT_EXIST ex(Cdmw::OrbSupport::OBJECT_NOT_EXISTObjectRemoved,
                                           CORBA::COMPLETED_NO);
                CDMW_LOG_FUNCTION_EXCEPTION(ex);
                throw ex;
            }
        }

        switch (names.size())
        {
        case 1: // oid is a component oid
            {
                CDMW_DEBUG(CCMLoggers::Get_ccmcif_logger(), 
                    "oid= " << object_id << "  is a component");
                // Now we can create the component servant and store servant
                // while facet servant are part of component servant
                comp_value.second.comp_servant 
                   = create_component_servant(comp_oid, 
                                              m_context, 
                                              comp_value.second);
                CDMW_DEBUG(CCMLoggers::Get_ccmcif_logger(), 
                    "component servant created");
                CDMW_ASSERT(comp_value.second.comp_servant.in());
                
                // NOTA: comp executor is not null if an executor has been created at ref 
                // creation that is a factory operation has been used to create ref.
                
                servant = comp_value.second.comp_servant;
            }
            break;

        case 2: // oid is a facet/consumer oid
            {        
                CDMW_DEBUG(CCMLoggers::Get_ccmcif_logger(), 
                    "oid= " << object_id << "  is a facet or consumer");
                std::string name = names[1];
                CDMW_DEBUG(CCMLoggers::Get_ccmcif_logger(), "name= " << name);

                CDMW_ASSERT(comp_value.second.comp_servant.in());
                CCMObject_impl* comp = dynamic_cast<CCMObject_impl*>(comp_value.second.comp_servant.in());
                CDMW_ASSERT(comp);
                servant = comp->get_facet_servant(name);
            }
            break;
            
        case 3:  // oid is a local push consumer
            {
                CDMW_DEBUG(CCMLoggers::Get_ccmcif_logger(), 
                    "oid= " << object_id << "  is a local push consumer");
                CDMW_ASSERT(comp_value.second.comp_servant.in());
                CCMObject_impl* comp = dynamic_cast<CCMObject_impl*>(comp_value.second.comp_servant.in());
                CDMW_ASSERT(comp);

                std::string name = names[1];
                std::string ck_value = names[2];
                CDMW_DEBUG(CCMLoggers::Get_ccmcif_logger(), 
                    "name=" << name << "  ck_value=" << ck_value);
                
                servant = comp->get_local_push_consumer_servant(name, ck_value);
            }
            break;

        default:
            {
                CDMW_ERROR(CCMLoggers::Get_ccmcif_logger(),
                    "oid " << object_id << " is not valid: " << names.size() << " names !");
                CORBA::INTERNAL ex(Cdmw::OrbSupport::INTERNALLifeCycleFrameworkError,
                                   CORBA::COMPLETED_NO);
                CDMW_LOG_FUNCTION_EXCEPTION(ex);
                throw ex;
            }
            break;
        }
    } 
    catch (const std::bad_alloc & ) 
    { 
        // Failed to allocate memory
        CDMW_ERROR(CCMLoggers::Get_ccmcif_logger(),
            "lookup_servant: Failed to allocate memory!");
        CORBA::NO_MEMORY ex(Cdmw::OrbSupport::NO_MEMORY,
                            CORBA::COMPLETED_NO); 
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw ex;
    } 
    catch (const CORBA::BAD_PARAM & ) 
    { // Pb with ObjectId
        CDMW_ERROR(CCMLoggers::Get_ccmcif_logger(),
            "lookup_servant: Pb with ObjectId!");
        CORBA::OBJECT_NOT_EXIST ex(Cdmw::OrbSupport::OBJECT_NOT_EXISTInvalidObjectId,
                                   CORBA::COMPLETED_NO);
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw ex;
    } 
    catch (const Components::CreateFailure& )
    {
        CDMW_ERROR(CCMLoggers::Get_ccmcif_logger(),
            "lookup_servant: Executor create failure!");
        CORBA::OBJECT_NOT_EXIST ex(Cdmw::OrbSupport::OBJECT_NOT_EXISTExecutorCreateFailure,
                                   CORBA::COMPLETED_NO); 
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw ex;
    }
    catch (const CORBA::OBJECT_NOT_EXIST & ex)
    {
        CDMW_ERROR(CCMLoggers::Get_ccmcif_logger(),
            "lookup_servant: Object not exist!");
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw;
    }

    CDMW_DEBUG(CCMLoggers::Get_ccmcif_logger(), "End of lookup_servant...");
    return servant._retn();
}
  
  
/**
* Purpose:
* <p>This method is called by Container whenever it is done
* with the specified servant. The ServantProvider is free to handle it at
* will.
* 
*@param oid ObjectId of the object incarnated by the servant
*@param poa The POA where the object is activated
*@param servant The servant in question
*
*@exception CORBA::SystemException Any CORBA system exception
*/
void 
CCMHome_impl::release_servant(const PortableServer::ObjectId& oid,
                                   PortableServer::POA_ptr         poa,
                                   PortableServer::Servant         servant) 
    throw(CORBA::SystemException)
{
    CDMW_LOG_FUNCTION(CCMLoggers::Get_ccmcif_logger(), "");
    // CDMW_ASSERT(m_context);
    // Do not check if context is not nil while release_servant can be called after
    // remove operation on the home to release each component.

    CDMW_DEBUG(CCMLoggers::Get_ccmcif_logger(), 
        "Entering in release_servant...");

    try  
    {
        // check if it is a component oid or a facet oid
        CORBA::String_var oid_str = PortableServer::ObjectId_to_string(oid);
        std::string object_id(oid_str.in());
        CCMUtils::NameVector names = CCMUtils::Read_oid(object_id);

        std::string comp_oid = names[0];
        CDMW_DEBUG(CCMLoggers::Get_ccmcif_logger(), 
            "oid=" << object_id << "  comp_oid=" << comp_oid);

        // check a component with this oid has been created in this home
        ComponentMap::const_iterator comp_pos = m_components.find(comp_oid);
        
        if (comp_pos  == m_components.end()) 
        {
            CDMW_ERROR(CCMLoggers::Get_ccmcif_logger(),
                "Cannot find component with oid: " << comp_oid);
            CORBA::OBJECT_NOT_EXIST ex(Cdmw::OrbSupport::OBJECT_NOT_EXISTCCMObjectNotFound,
                                       CORBA::COMPLETED_NO); 
            CDMW_LOG_FUNCTION_EXCEPTION(ex);
            throw ex;
        }

        ComponentMap::value_type comp_value = *comp_pos;

        switch (names.size())
        {
        case 1: // oid is a component oid
            {
                CDMW_DEBUG(CCMLoggers::Get_ccmcif_logger(), 
                    "oid= " << object_id << "  is a component");
                // check if operation removed has been requested on component
                //CDMW_ASSERT(comp_value.second.comp_servant.in());
                //CCMObject_impl*  component = dynamic_cast<CCMObject_impl*>(comp_value.second.comp_servant.in());
                CDMW_ASSERT(servant);
                CCMObject_impl*  component = dynamic_cast<CCMObject_impl*>(servant);
                
                CDMW_ASSERT(component);
                if (component->is_removed()) 
                {
                    CDMW_DEBUG(CCMLoggers::Get_ccmcif_logger(), 
                        "Component is removed, delete it from map");
                    // if so remove component from the map and the local naming 
                    // context
                    delete_component_info(comp_oid);            
                }
                
                CDMW_DEBUG(CCMLoggers::Get_ccmcif_logger(), 
                    "Release servant");
                servant->_remove_ref();
            }
            break;

        case 2: // oid is a facet/consumer oid
            {
                CDMW_DEBUG(CCMLoggers::Get_ccmcif_logger(), 
                    "oid= " << object_id << "  is a facet or consumer");
                std::string name = names[1];
                CDMW_DEBUG(CCMLoggers::Get_ccmcif_logger(), "name= " << name);       
                // release servant
                servant->_remove_ref();
            }
            break;
            
        case 3:  // oid is a local push consumer
            {
                CDMW_DEBUG(CCMLoggers::Get_ccmcif_logger(), 
                    "oid= " << object_id << "  is a local push consumer");
                std::string name = names[1];
                std::string ck_value = names[2];
                CDMW_DEBUG(CCMLoggers::Get_ccmcif_logger(), 
                    "name=" << name << "  ck_value=" << ck_value);
                // release servant
                servant->_remove_ref();
            }
            break;

        default:
            {
                CDMW_ERROR(CCMLoggers::Get_ccmcif_logger(),
                    "oid " << object_id << " is not valid: " << names.size() << " names !");
                CORBA::INTERNAL ex(Cdmw::OrbSupport::INTERNALLifeCycleFrameworkError,
                                   CORBA::COMPLETED_NO);
                CDMW_LOG_FUNCTION_EXCEPTION(ex);
                throw ex;
            }
        }
    } 
    catch (const std::bad_alloc & ) 
    { 
        // Failed to allocate memory
        CDMW_ERROR(CCMLoggers::Get_ccmcif_logger(),
            "lookup_servant: Failed to allocate memory!");
        CORBA::NO_MEMORY ex(Cdmw::OrbSupport::NO_MEMORY,
                            CORBA::COMPLETED_NO); 
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw ex;
    } 
    catch (const CORBA::BAD_PARAM & ) 
    { 
        // Pb with ObjectId
        CDMW_ERROR(CCMLoggers::Get_ccmcif_logger(),
            "release_servant: Pb with ObjectId!");
        CORBA::OBJECT_NOT_EXIST ex(Cdmw::OrbSupport::OBJECT_NOT_EXISTInvalidObjectId,
                                   CORBA::COMPLETED_NO); 
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw ex;
    }
}

/**
* Purpose:
* <p>This method is called by Container after Home creation.
* 
*@param ctxt Component context given by container.
*
*@exception Components::CCMException Any Component exception
*@exception CORBA::SystemException Any CORBA system exception
*/
void 
CCMHome_impl::set_context(CdmwCcmContainer::CCM2Context_ptr ctxt)
throw (Components::CCMException,
       CORBA::SystemException)
{
    CDMW_LOG_FUNCTION(CCMLoggers::Get_ccmcif_logger(), "");
    check_is_removed(m_oid.in());

    // check component context is a Session2Context one
    CdmwCcmContainer::Session2Context_var s2ctx 
        = CdmwCcmContainer::Session2Context::_narrow(ctxt);
    
    if (!CORBA::is_nil(s2ctx.in()))
    {
       m_context = new SessionContext(s2ctx.in());
    }
    else
    {
        CDMW_ERROR(CCMLoggers::Get_ccmcif_logger(),
            "Context is not a Session2Context!");
        Components::CreateFailure ex(CdmwDeployment::WRONG_CONTEXT_KIND);
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw ex;
    }
}


/**
* Purpose:
* <p>This method is called by Container after Home creation.
* 
*@param handle Handle to the library loaded for this home.
* It has to be unloaded at home destruction.
*/
void CCMHome_impl::set_shlib_handle(Cdmw::OsSupport::OS::SharedObjectHandle* handle)
throw ()
{
    CDMW_LOG_FUNCTION(CCMLoggers::Get_ccmcif_logger(), "");
    m_shlib_handle = handle;
}


/**
* Purpose:
* <p>This method returns the RepositoryID of the object associated to the
* <I>oid</I>.
* 
*@param oid ObjectId of the object
*
*@return The RepositoryId of the object
*
*@exception CORBA::OBJECT_NOT_EXIST The object does not exist any more.
*/
std::string
CCMHome_impl::get_supported_interface(const PortableServer::ObjectId& oid) const
   throw (CORBA::OBJECT_NOT_EXIST)
{
    CDMW_LOG_FUNCTION(CCMLoggers::Get_ccmcif_logger(), "");
    check_is_removed(m_oid.in());
    CDMW_ASSERT(m_context);

    try  
    {
        CORBA::String_var s = PortableServer::ObjectId_to_string(oid);
        std::string name(s.in());
        ComponentMap::const_iterator serv_pos = m_components.find(name);

        if (serv_pos  == m_components.end()) {
            CDMW_ERROR(CCMLoggers::Get_ccmcif_logger(),
                "Cannot find component with oid: " << name);
            CORBA::OBJECT_NOT_EXIST ex(Cdmw::OrbSupport::OBJECT_NOT_EXISTCCMObjectNotFound,
                                       CORBA::COMPLETED_NO); 
            CDMW_LOG_FUNCTION_EXCEPTION(ex);
            throw ex;
        }
        else 
        {
            std::string result(m_comp_rep_id);
            CDMW_LOG_FUNCTION_RETURN(result);
            return result;
        }
    } 
    catch (const std::bad_alloc & ) 
    { // Failed to allocate memory
        CORBA::NO_MEMORY ex(Cdmw::OrbSupport::NO_MEMORY,
                            CORBA::COMPLETED_NO); 
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw ex;
    } 
    catch (const CORBA::BAD_PARAM & ) 
    { // Pb with ObjectId
        CORBA::OBJECT_NOT_EXIST ex(Cdmw::OrbSupport::OBJECT_NOT_EXISTInvalidObjectId,
                                   CORBA::COMPLETED_NO); 
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw ex;
    }  
}



CORBA::Object_ptr 
CCMHome_impl::create_component_ref(const Components::ConfigValues& config, 
                                   Components::EnterpriseComponent_ptr  comp_exec)
    throw(Components::CreateFailure,
          CORBA::SystemException)
{
    CDMW_LOG_FUNCTION(CCMLoggers::Get_ccmcif_logger(), "");
    check_is_removed(m_oid.in());
    CDMW_ASSERT(m_context);

    std::string comp_oid;

    CORBA::Object_var to_return = create_component_ref_i(comp_oid, comp_exec);

    // Setting realised to inject information about timeouts of receptacles.
    // That's the context of a home instance, which stores the config values.
    try
    {

        m_context->set_config_values(comp_oid, config);

        CDMW_DEBUG(CCMLoggers::Get_ccmcif_logger(), 
            "CCMHome_impl create_component_ref: length config values ="
            << config.length());

    }
    catch( Cdmw::Common::AlreadyExistException ) {

        CDMW_ERROR(CCMLoggers::Get_ccmcif_logger(), 
            "CCMHome_impl create_component_ref: config values exist already!");

        CORBA::INTERNAL ex(OrbSupport::INTERNALCCMCIFError,
                           CORBA::COMPLETED_NO);
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw ex;
    }

    register_component(config, to_return.in());
    
    // register component for recording (if Recording ConfigValue set to true)
    // NOTA: Failing to record the component for recording will only log
    //       a WARN message and not abort deployment of component.
    try {
       m_context->register_for_recording(comp_oid, to_return.in());
    }
    catch (const Components::CreateFailure& e)
    {
       CDMW_ERROR(CCMLoggers::Get_ccmcif_logger(),
                  "Error registering component '" << comp_oid << "' for recording: " <<
                  (e.reason == CdmwDeployment::WRONG_RECORDING_CONFIG_VALUES ?
                   "Wrong Recording ConfigValue" : "Recording Daemon error") );
       CDMW_WARN(CCMLoggers::Get_ccmcif_logger(),
                 "Incoming requets to component '" << comp_oid << 
                 "' will not be recorded!");
    }

    return to_return._retn();
}

void  
CCMHome_impl::register_component(const Components::ConfigValues& config,
                                 CORBA::Object_ptr comp_ref)
    throw (Components::CreateFailure,
               CORBA::SystemException)
{
    CDMW_LOG_FUNCTION(CCMLoggers::Get_ccmcif_logger(), "");
    // register component in a local naming context
    if (!CORBA::is_nil(m_object_registration.in())) 
    {
        std::string component_name;
        try
        {
            CORBA::Any_var component_name_any;
            if (Common::read_config_value(CdmwDeployment::COMPONENT_NAME, 
                                  config, 
                                  component_name_any))
            {
                const char* name;
                component_name_any >>= name;
                component_name = name;
                        
                CDMW_INFO(CCMLoggers::Get_ccmcif_logger(), 
                    "CCMHome_impl register_component with name "<< name);
                m_object_registration->register_object(comp_ref, 
                                                       component_name.c_str());
            }
            else
            {
                CDMW_WARN(CCMLoggers::Get_ccmcif_logger(), 
                    "CCMHome_impl register_component: No component name");
            }
        }
        catch (const CosNaming::NamingContext::InvalidName& e)
        {
            CDMW_ERROR(CCMLoggers::Get_ccmcif_logger(), 
                "CCMHome_impl register_component: Invalid component name");
            Components::CreateFailure ex
                (CdmwDeployment::WRONG_COMPONENT_NAME_CONFIG_VALUES);
            CDMW_LOG_FUNCTION_EXCEPTION(ex);
            throw ex;
        }
        catch (const CosNaming::NamingContext::AlreadyBound& e)
        {
            CDMW_ERROR(CCMLoggers::Get_ccmcif_logger(), 
                "CCMHome_impl register_component: component name is already bound");
             Components::CreateFailure ex
                (CdmwDeployment::WRONG_COMPONENT_NAME_CONFIG_VALUES);
            CDMW_LOG_FUNCTION_EXCEPTION(ex);
            //throw ex;
        }
        catch (const CORBA::SystemException& e)
        {
            CDMW_ERROR(CCMLoggers::Get_ccmcif_logger(), 
                "CCMHome_impl register_component: " << e);
            CORBA::INTERNAL ex(OrbSupport::INTERNALCCMCIFError,
                               CORBA::COMPLETED_NO);
            CDMW_LOG_FUNCTION_EXCEPTION(ex);
            throw ex;
        }
    }
}

CORBA::Object_ptr
CCMHome_impl::create_component_ref_i
              (std::string&                         comp_oid_str, 
               Components::EnterpriseComponent_ptr  comp_exec)
    throw (Components::CreateFailure,
           CORBA::SystemException)
{
    CDMW_LOG_FUNCTION(CCMLoggers::Get_ccmcif_logger(), "");
    ComponentInfo comp_info;
    
    try
    {
        comp_info.comp_ref = m_context->create_ref(m_comp_rep_id.c_str());

        if (CORBA::is_nil(comp_info.comp_ref.in()))
        {
            CDMW_ERROR(CCMLoggers::Get_ccmcif_logger(), 
                "Create_ref has failed!");
            CORBA::INTERNAL ex(OrbSupport::INTERNALLifeCycleFrameworkError, 
                               CORBA::COMPLETED_NO);
            CDMW_LOG_FUNCTION_EXCEPTION(ex);
            throw ex;
        }

        // set executor => executor is not nil if factory operation is used!
        comp_info.comp_exec 
           = Components::EnterpriseComponent::_duplicate(comp_exec);

        CORBA::String_var oid 
           = m_context->get_oid_str_from_ref(comp_info.comp_ref.in());
        comp_oid_str = oid.in();

        m_components.insert(ComponentMap::value_type(comp_oid_str, comp_info));
    }
    catch (const Components::IllegalState& )
    {
        CDMW_ERROR(CCMLoggers::Get_ccmcif_logger(), 
            "IllegalState exception raised!");
        Components::CreateFailure ex(CdmwDeployment::ILLEGAL_STATE);
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw ex;
    }
    catch (const Components::BadComponentReference& )
    {
        CDMW_ERROR(CCMLoggers::Get_ccmcif_logger(), 
            "BadComponentReference exception raised!");
        Components::CreateFailure ex(CdmwDeployment::BAD_COMPONENT_REFERENCE);
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw ex;
    }
    catch (const CORBA::SystemException& ex)
    {
        CDMW_ERROR(CCMLoggers::Get_ccmcif_logger(), 
            "SystemException raised: " <<
            Cdmw::OrbSupport::OrbSupport::exception_to_string(ex));
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw;
    }
        
    return comp_info.comp_ref._retn();
}

void
CCMHome_impl::delete_component_info(const std::string& comp_oid)
{
    CDMW_LOG_FUNCTION(CCMLoggers::Get_ccmcif_logger(), "");
    m_components.erase(comp_oid);
}


} // End of namespace CIF

} // End of namespace CCM

} // End of namespace Cdmw

