/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO 
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2009. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#include <Foundation/common/Assert.hpp>
#include <CCMContainer/idllib/CdmwCcmContainer.stub.hpp>
#include <CCMContainer/ccmcif/Context.hpp>
#include <Foundation/orbsupport/CORBAMessaging.hpp>
#include <CCMContainer/ccmcif/SessionContext.hpp>
#include <CCMContainer/ccmcommon/CCMLoggers.hpp>
#include <CCMContainer/ccmcommon/CCMUtils.hpp>
#include <Foundation/orbsupport/OrbSupport.hpp>
#include <Foundation/orbsupport/ExceptionMinorCodes.hpp>
#include <Foundation/idllib/CdmwDeployment.stub.hpp>
#include <CCMContainer/idllib/CdmwCcmCif.stub.hpp>
#include <Foundation/idllib/CdmwRecordingContainerManager.stub.hpp>
#include <Foundation/ossupport/SystemProperties.hpp>
#include <Foundation/commonsvcs/naming/NamingInterface.hpp>


namespace {
    // SystemProperty to use component name in service_id for recording
    const char* PROPERTY_RECORD_USING_COMPONENT_NAME =
        "Cdmw.CCMContainer.Recording.UseComponentName";
}


namespace Cdmw {

namespace CCM {

namespace CIF {

/*
 * This class is the base class for ccmcif context embedding specific CCM 
 * context.
 */
Context::Context(CdmwCcmContainer::CCM2Context_ptr ctx)
  : m_ccm_context(CdmwCcmContainer::CCM2Context::_duplicate(ctx))
{
    CDMW_LOG_FUNCTION(CCMLoggers::Get_ccmcif_logger(), "");
    CDMW_ASSERT(!CORBA::is_nil(ctx));
}

Context::~Context() 
{
    CDMW_LOG_FUNCTION(CCMLoggers::Get_ccmcif_logger(), "");
    m_ccm_context = CdmwCcmContainer::CCM2Context::_nil();
}

CORBA::Object_ptr 
Context::resolve_initial_references(const char* identifier)
  throw (CdmwCcmContainer::CCM2Context::InvalidName,
         CORBA::SystemException)
{
    CDMW_LOG_FUNCTION(CCMLoggers::Get_ccmcif_logger(), 
        "identifier="<<identifier);
    return m_ccm_context->resolve_initial_references(identifier);
}

Components::CCMHome_ptr 
Context::get_CCM_home() throw(CORBA::SystemException)
{
    CDMW_LOG_FUNCTION(CCMLoggers::Get_ccmcif_logger(), "");
    return m_ccm_context->get_CCM_home();
}
   
::CosEventChannelAdmin::EventChannel_ptr
Context::obtain_event_channel_for_emitter(::Components::PortDescription* port)
  throw (CORBA::SystemException)
{
    CDMW_LOG_FUNCTION(CCMLoggers::Get_ccmcif_logger(), "");
    return m_ccm_context->obtain_event_channel_for_emitter(port);
}

::CosEventChannelAdmin::EventChannel_ptr
Context::create_event_channel_for_publisher(::Components::PortDescription* port)
  throw (CORBA::SystemException)
{
    CDMW_LOG_FUNCTION(CCMLoggers::Get_ccmcif_logger(), "");
    return m_ccm_context->create_event_channel_for_publisher(port);
}

void Context::set_config_values( const std::string& comp_oid,
                                 const Components::ConfigValues& config )
  throw( Cdmw::Common::AlreadyExistException )
{
    CDMW_LOG_FUNCTION(CCMLoggers::Get_ccmcif_logger(), "comp_oid="<<comp_oid);
  
    ConfigValuesMap::iterator cv_itr = m_config_values.find( comp_oid );
    
    if( cv_itr != m_config_values.end() ) {
        CDMW_ERROR(CCMLoggers::Get_ccmcif_logger(),
            "Component with oid " << comp_oid << " already has ConfigValues set");
        Cdmw::Common::AlreadyExistException ex;
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw ex;
    }
  
    // create a copy of config values to be stored in the map
    m_config_values.insert( ConfigValuesMap::value_type( comp_oid,  new Components::ConfigValues(config)));
  
    CDMW_DEBUG(CCMLoggers::Get_ccmcif_logger(),
        "insertion of config values in the map of the context." << 
        " Key member of the map : oid = " << comp_oid);
}

Components::ConfigValues* Context::get_config_values( const std::string& comp_oid )
  throw( Cdmw::Common::NotFoundException )
{
    CDMW_LOG_FUNCTION(CCMLoggers::Get_ccmcif_logger(), "comp_oid="<<comp_oid);
  
    ConfigValuesMap::iterator cv_itr = m_config_values.find( comp_oid );
    Components::ConfigValues_var config_values;
  
    if( cv_itr != m_config_values.end() ){
        config_values = cv_itr->second;
        CDMW_DEBUG(CCMLoggers::Get_ccmcif_logger(), 
            "Context get_config_values : config values found.");
        return config_values._retn();
    }else{
        CDMW_ERROR(CCMLoggers::Get_ccmcif_logger(), 
            "Context get_config_values : config values not found.");
        Cdmw::Common::NotFoundException ex;
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw ex;
    }
}

CORBA::Object_ptr 
Context::set_timeout_policy ( unsigned long timeout_in_ms,
                              CORBA::Object_ptr obj )
  throw ( CORBA::SystemException, CORBA::PolicyError )
{
    CDMW_LOG_FUNCTION(CCMLoggers::Get_ccmcif_logger(), 
        "timeout_in_ms="<<timeout_in_ms);

    /* WARNING : TimeT is expressed in units of 100 nano seconds */
    TimeBase::TimeT _timeout = ((unsigned long) (timeout_in_ms * 10000));
  
    CDMW_DEBUG(CCMLoggers::Get_ccmcif_logger(), 
        "Context set_timeout_policy: timeout = " << _timeout << " ( * 100 ns)");
  
    CORBA::Any timeout_any;
    timeout_any <<= _timeout;
      
    CORBA::PolicyList timeout_policy_list (1);
    timeout_policy_list.length (1);
      
    /*
     * recover the orb reference
     */
    CdmwCcmContainer::Session2Context_var session2context = CdmwCcmContainer::Session2Context::_narrow(m_ccm_context.in());
      
    timeout_policy_list[0] = session2context->create_policy ( Messaging::RELATIVE_RT_TIMEOUT_POLICY_TYPE, timeout_any);
      
    /*
     * set policy at object level
     */
    CORBA::Object_var obj_copy =
      obj->_set_policy_overrides ( timeout_policy_list,
                                   CORBA::ADD_OVERRIDE );
      
     timeout_policy_list[0]->destroy();
  
    CDMW_DEBUG(CCMLoggers::Get_ccmcif_logger(), 
        "Context set_timeout_policy done.");
  
    return obj_copy._retn();
}

CORBA::Object_ptr
Context::apply_policy(const CORBA::Object_ptr connection,
                      const std::string& comp_oid,
                      const std::string& receptacle_name) 
  throw( CORBA::SystemException )
{
    CDMW_LOG_FUNCTION(CCMLoggers::Get_ccmcif_logger(), 
        "comp_oid="<<comp_oid<<"  receptacle_name="<<receptacle_name);

    CORBA::Object_var to_return = CORBA::Object::_duplicate( connection );
    Components::ConfigValues_var config_values;
  
    CDMW_DEBUG(CCMLoggers::Get_ccmcif_logger(), 
        "Context apply_policy: receptacle name =" << receptacle_name);
           
    /*
     * search the config values matched to the OID
     */
    try{
  
        CdmwCcmContainer::Session2Context_var session2context = 
            CdmwCcmContainer::Session2Context::_narrow(m_ccm_context.in());
        
        config_values = this->get_config_values(comp_oid);
        
        // Timeout on a receptacle in milliseconds
        unsigned long timeDurationInMs = 0; 
        
        // default timeout found in the ConfigValues sequence (in milliseconds)
        unsigned long defaultTimeout = 0;
        
        // flag which indicates whether the timeout matched to the receptacle was
        // found or not
        bool receptacle_timeout_found = false;
        
        // flag which indicates whether the default timeout matched to the recaptacle
        // was found or not
        bool default_timeout_found = false;
        
        std::string name_config_value;
        
        /*
         * search the timeout to apply on the receptacle
         */
        
        // Search in ConfigValues sequence
        for( CORBA::ULong i = 0; i < config_values->length(); i++ ){
            // std::string name_config_value = config_values[i]->name();
            name_config_value = config_values[i]->name();
            
            if( !name_config_value.compare(CdmwDeployment::USES_PORT_TIMEOUTS )){
                CdmwCcmCif::UsesPortTimeouts_var uses_port_timeouts;
                config_values[i]->value() >>= uses_port_timeouts;
                
                // Search in UsesPortTimeouts sequence
                for( CORBA::ULong j = 0; j < uses_port_timeouts->length(); j++ ){
                    std::string uses_name(uses_port_timeouts[j].uses_name.in());
                    
                    if( !uses_name.compare(receptacle_name) ){
                      timeDurationInMs = uses_port_timeouts[j].timeout;
                      receptacle_timeout_found = true;
                      CDMW_DEBUG(CCMLoggers::Get_ccmcif_logger(), 
                          "Context apply_policy: timeout of " << receptacle_name
                          << " found. " << "timeout receptacle = "
                          << timeDurationInMs << " (ms)");
                      break;
                    }else{
                      if( !uses_name.compare("") ){
                        default_timeout_found = true;
                        defaultTimeout = uses_port_timeouts[j].timeout;
                      CDMW_DEBUG(CCMLoggers::Get_ccmcif_logger(), 
                          "Context apply_policy: default timeout found. "
                          << "default_timeout = " << defaultTimeout << " (ms)");
                      }
                    }// if, UsesPortTimemout structure
                }// for, UsesPortTimeouts sequence 
                uses_port_timeouts._retn();
                break;
            }//if, ConfigValue valuetype
        }//for, ConfigValues sequence
        
        if( default_timeout_found == true && receptacle_timeout_found == false ){
            timeDurationInMs = defaultTimeout;
        }
        
        // Set the policy
        if( receptacle_timeout_found == true || default_timeout_found == true ){
        
            try{
            
                to_return = this->set_timeout_policy(timeDurationInMs, to_return.in());
            
            }
            catch(const CORBA::PolicyError& e){
                CDMW_ERROR(CCMLoggers::Get_ccmcif_logger(), 
                    "Context apply_policy : set_timeout_policy error! " 
                    << Cdmw::OrbSupport::OrbSupport::exception_to_string(e));
                CORBA::INTERNAL ex(OrbSupport::INTERNALCCMCIFError,
                                   CORBA::COMPLETED_NO);
                CDMW_LOG_FUNCTION_EXCEPTION(ex);
                throw ex;
            } 
            catch(const CORBA::SystemException& e) {
                CDMW_ERROR(CCMLoggers::Get_ccmcif_logger(), 
                    "Context apply_policy : set_timeout_policy error! " 
                    << Cdmw::OrbSupport::OrbSupport::exception_to_string(e));
                CDMW_LOG_FUNCTION_EXCEPTION(e);
                throw;
            } 
        }
        else{
          CDMW_DEBUG(CCMLoggers::Get_ccmcif_logger(), 
              "Context apply_policy: no default and specific timeout found "
              << "for the receptacle :" << receptacle_name )
        }
    }
    catch(const Cdmw::Common::NotFoundException& e)
    {
        // no config values matched to the oid
        CDMW_DEBUG(CCMLoggers::Get_ccmcif_logger(), 
            "no config values matched to the oid"
            << " in the map of the context ( oid = " << comp_oid << "). "
            << e );
    }  
  
    CDMW_DEBUG(CCMLoggers::Get_ccmcif_logger(), "Context apply_policy done.");
  
    return to_return._retn();
    
}



void Context::register_for_recording(const std::string& comp_oid,
                                     CORBA::Object_ptr obj)
    throw( Components::CreateFailure )
{
    CDMW_LOG_FUNCTION(CCMLoggers::Get_ccmcif_logger(), "comp_oid="<<comp_oid);
    
    // get ConfigValues for the component
    Components::ConfigValues_var config;
    try {
        config = this->get_config_values(comp_oid);
    }
    catch(const Cdmw::Common::NotFoundException& e)
    {
        // no config values matched to the oid
        CDMW_DEBUG(CCMLoggers::Get_ccmcif_logger(), 
            "no config values matched to the oid"
            << " in the map of the context ( oid = " << comp_oid << "). "
            << e );
            
        // nothing to do
        return;
    }  
        
    /***** DEBUG ****/
    for (CORBA::ULong i=0; i< config->length(); ++i) {
        CDMW_DEBUG(CCMLoggers::Get_ccmcif_logger(), 
            "CONFIG " << i << " : " << config[i]->name()); 
    }
    /****************/
    
    // check if Recording ConfigValue is set
    CORBA::Any_var recording_flag_any;
    if (Common::read_config_value(CdmwDeployment::RECORDING, 
                                  config.in(), 
                                  recording_flag_any.out()))
    {
        CORBA::Boolean recording_flag;
        
        if (!(recording_flag_any.in() >>= CORBA::Any::to_boolean(recording_flag))) {
            CDMW_ERROR(CCMLoggers::Get_ccmcif_logger(), 
                "Recording ConfigValue is not a CORBA::Boolean");
            Components::CreateFailure ex
                (CdmwDeployment::WRONG_RECORDING_CONFIG_VALUES);
            CDMW_LOG_FUNCTION_EXCEPTION(ex);
            throw ex;            
        }
        
        if (recording_flag) {
            CDMW_INFO(CCMLoggers::Get_ccmcif_logger(), 
                "***** Register for Recording - comp_oid = " << comp_oid);

            // get CdmwRecording::ContainerManager object
            CdmwRecording::ContainerManager_var recording_mgr;
            try {
                CORBA::Object_var o = m_ccm_context->resolve_initial_references(
                    CdmwRecording::REC_CONTAINER_MNGR_ID);
                
                recording_mgr = CdmwRecording::ContainerManager::_narrow(o.in());
                    
                if (CORBA::is_nil(recording_mgr.in())) {
                    CDMW_ERROR(CCMLoggers::Get_ccmcif_logger(), 
                        CdmwRecording::REC_CONTAINER_MNGR_ID << 
                        " object is not a CdmwRecording::ContainerManager");
                    Components::CreateFailure ex
                        (CdmwDeployment::RECORDING_DAEMON_ERROR);
                    CDMW_LOG_FUNCTION_EXCEPTION(ex);
                    throw ex;            
                }
                
            } catch (const CdmwCcmContainer::CCM2Context::InvalidName& e) {
                CDMW_ERROR(CCMLoggers::Get_ccmcif_logger(), 
                    CdmwRecording::REC_CONTAINER_MNGR_ID << 
                    " object is not registered as initial reference");
                Components::CreateFailure ex
                    (CdmwDeployment::RECORDING_DAEMON_ERROR);
                CDMW_LOG_FUNCTION_EXCEPTION(ex);
                throw ex;            
            } catch (const CORBA::SystemException& e) {
                CDMW_ERROR(CCMLoggers::Get_ccmcif_logger(),
                    "Error retrieving " <<
                    CdmwRecording::REC_CONTAINER_MNGR_ID << " object : " <<
                    Cdmw::OrbSupport::OrbSupport::exception_to_string(e));
                Components::CreateFailure ex
                    (CdmwDeployment::RECORDING_DAEMON_ERROR);
                CDMW_LOG_FUNCTION_EXCEPTION(ex);
                throw ex;            
            }
            
            // get server_id
            CORBA::String_var server_id = m_ccm_context->server_id();
            
            // if SystemProperty PROPERTY_RECORD_USING_COMPONENT_NAME is "true"
            // find component name and add it to server_id for recording
            std::string prop = Cdmw::OsSupport::SystemProperties::Get_Property(
                PROPERTY_RECORD_USING_COMPONENT_NAME, "false");
            if (prop == "true") {
                
                // get COMPONENT_NAME
                CORBA::Any_var component_name_any;
                if (Common::read_config_value(CdmwDeployment::COMPONENT_NAME, 
                                      config, 
                                      component_name_any))
                {
                    using namespace Cdmw::CommonSvcs::Naming;
                    const char* component_name;
                    component_name_any >>= component_name;
                           
                    // ConfigValue found, add component name to server_id
                    CosNaming::Name_var name =
                        NamingInterface::to_name(server_id.in());
                    name->length(name->length()+1);
                    name[name->length()-1].id = component_name;
                    
                    server_id = NamingInterface::to_string(name.in()).c_str();
                }
            }
            
            // register object for recording
            CDMW_INFO(CCMLoggers::Get_ccmcif_logger(), 
                "Register object \"" << comp_oid << 
                "\" in Recording service with service_id=" << server_id.in());
            try {
                CdmwRecording::Object_List list(1);
                list.length(1);
                list[0] = CORBA::Object::_duplicate(obj);
                
                recording_mgr->record_objects(server_id.in(), list);
                
            } catch (const CORBA::SystemException& e) {
                CDMW_ERROR(CCMLoggers::Get_ccmcif_logger(),
                    "Error registering object in Recording service : " <<
                    Cdmw::OrbSupport::OrbSupport::exception_to_string(e));
                Components::CreateFailure ex
                    (CdmwDeployment::RECORDING_DAEMON_ERROR);
                CDMW_LOG_FUNCTION_EXCEPTION(ex);
                throw ex;            
            }
            
        }
        else 
        {
            // recording_flag = false
            CDMW_DEBUG(CCMLoggers::Get_ccmcif_logger(), 
                "No recording required for this component "
                "(Recording ConfigValue set to false)");
        }
    }
    else
    {
        // Recording ConfigValue not specified
        CDMW_DEBUG(CCMLoggers::Get_ccmcif_logger(), 
            "No recording required for this component"
            "(no Recording ConfigValue specified)");
    }
}


} // namespace CIF 
} // end namespace Cdmw 
} // end namespace CCM 

