/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO 
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2009. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


// Cdmw CCM files
#include <CCMContainer/ccmcif/DefaultCCMObjectStates.hpp>
#include <CCMContainer/ccmcif/CCMObject_impl.hpp>
#include <Foundation/common/Assert.hpp>
#include <Foundation/orbsupport/OrbSupport.hpp>
#include <Foundation/logging/FunctionLogger.hpp>
#include <Foundation/logging/Logger.hpp>

namespace Cdmw {

namespace CCM {

namespace CIF {


//
// Implementation of INITIAL CCMObject's state
//

/*
 * In case of configuration_complete:
 *   - call ccm_activate
 *   - change state to FULLY_ACTIVATED
 */
const CCMObjectState* CCMObjectState_Initial::configuration_complete(CCMObject_impl* ccm_object) const
    throw (Components::CCMException, CORBA::SystemException) 
{
  std::string receptacle_name = ccm_object->check_connections();
  if (receptacle_name.empty()) {
    // call ccm_activate
    ccm_object->ccm_activate();
    
    // change state to FULLY_ACTIVATED
    return CCM_OBJECT_STATE_FULLY_ACTIVATED;
  } else
    return this;
};

/*
 * In case of remove:
 *   - call ccm_remove
 *   - change state to REMOVED
 */
const CCMObjectState* CCMObjectState_Initial::remove(CCMObject_impl* ccm_object) const
    throw (Components::CCMException, CORBA::SystemException) 
{
    // call ccm_remove
    ccm_object->ccm_remove();
    
    // change state to REMOVED
    return CCM_OBJECT_STATE_REMOVED;
};

/*
 * In case of activate:
 *   - failed assertion, since this transition 
 *     doesn't exist for default case
 */
const CCMObjectState* CCMObjectState_Initial::activate(CCMObject_impl* ccm_object) const
    throw (Components::CCMException, CORBA::SystemException) 
{
    std::cerr << "'activate' transition not authorized from " 
              << "CCM_OBJECT_STATE_INITIAL"<< std::endl;
    CDMW_ASSERT(false);

    // for compiler
    return 0;
};

/*
 * In case of deactivate:
 *   - failed assertion, since this transition 
 *     doesn't exist for default case
 */
const CCMObjectState* CCMObjectState_Initial::deactivate(CCMObject_impl* ccm_object) const
    throw (Components::CCMException, CORBA::SystemException) 
{
    std::cerr << "'deactivate' transition not authorized from " 
              << "CCM_OBJECT_STATE_INITIAL"<< std::endl;
    CDMW_ASSERT(false);

    // for compiler
    return 0;
};

/*
 * In case of operational_request:
 *   - check that all connections were made. If a connection is missing
*      raise a CORBA::TRANSIENT() system exception. 
 */
const CCMObjectState* CCMObjectState_Initial::operational_request(CCMObject_impl* ccm_object) const
    throw (CORBA::SystemException) 
{
  // Check that the connections are done
  std::string receptacle_name = ccm_object->check_connections();
  if(!receptacle_name.empty()) {
    CORBA::TRANSIENT ex(OrbSupport::TRANSIENTMissingConnection,
                        CORBA::COMPLETED_NO);
    std::cerr << "CCMObjectState_Initial::operational_request: the receptacle "
              << receptacle_name
              <<" is not connected."
              << std::endl;
    throw ex;
  }

  // ignore: just return same state
  return this;
};

//
// Implementation of FULLY_ACTIVATED CCMObject's state
//

/*
 * In case of configuration_complete:
 *   - already configured; ignore
 */
const CCMObjectState* CCMObjectState_FullyActivated::configuration_complete(CCMObject_impl* ccm_object) const
    throw (Components::CCMException, CORBA::SystemException) 
{
    // ignore: just return same state
    return this;
};

/*
 * In case of remove:
 *   - call ccm_passivate
 *   - call ccm_remove
 *   - change state to REMOVED
 */
const CCMObjectState* CCMObjectState_FullyActivated::remove(CCMObject_impl* ccm_object) const
    throw (Components::CCMException, CORBA::SystemException) 
{
    // call ccm_passivate
    ccm_object->ccm_passivate();
    
    // call ccm_remove
    ccm_object->ccm_remove();
    
    // change state to REMOVED
    return CCM_OBJECT_STATE_REMOVED;
};

/*
 * In case of activate:
 *   - failed assertion, since this transition 
 *     doesn't exist for default case
 */
const CCMObjectState* CCMObjectState_FullyActivated::activate(CCMObject_impl* ccm_object) const
    throw (Components::CCMException, CORBA::SystemException) 
{
    std::cerr << "'activate' transition not authorized from " 
              << "CCM_OBJECT_STATE_FULLY_ACTIVATED"<< std::endl;
    CDMW_ASSERT(false);

    // for compiler
    return 0;
};

/*
 * In case of deactivate:
 *   - failed assertion, since this transition 
 *     doesn't exist for default case
 */
const CCMObjectState* CCMObjectState_FullyActivated::deactivate(CCMObject_impl* ccm_object) const
    throw (Components::CCMException, CORBA::SystemException) 
{
    std::cerr << "'deactivate' transition not authorized from " 
              << "CCM_OBJECT_STATE_FULLY_ACTIVATED"<< std::endl;
    CDMW_ASSERT(false);

    // for compiler
    return 0;
};


/*
 * In case of operational_request:
 *   - Do nothing: stay in the same state.
 */
const CCMObjectState* CCMObjectState_FullyActivated::operational_request(CCMObject_impl* ccm_object) const
  throw (CORBA::SystemException) 
{
    // ignore: just return same state
    return this;
};

//
// Implementation of REMOVED CCMObject's state
//

/*
 * In case of configuration_complete:
 *   - failed assertion, since component is removed
 */
const CCMObjectState* CCMObjectState_Removed::configuration_complete(CCMObject_impl* ccm_object) const
    throw (Components::CCMException, CORBA::SystemException) 
{
    std::cerr << "'configuration_complete' transition not authorized from " 
              << "CCM_OBJECT_STATE_REMOVED"<< std::endl;
    CDMW_ASSERT(false);

    // for compiler
    return 0;
};

/*
 * In case of remove:
 *   - already removed; ignore
 */
const CCMObjectState* CCMObjectState_Removed::remove(CCMObject_impl* ccm_object) const
    throw (Components::CCMException, CORBA::SystemException) 
{
    // ignore: just return same state
    return this;
};

/*
 * In case of activate:
 *   - failed assertion, since this transition 
 *     doesn't exist for default case
 */
const CCMObjectState* CCMObjectState_Removed::activate(CCMObject_impl* ccm_object) const
    throw (Components::CCMException, CORBA::SystemException) 
{
    std::cerr << "'activate' transition not authorized from " 
              << "CCM_OBJECT_STATE_REMOVED"<< std::endl;
    CDMW_ASSERT(false);

    // for compiler
    return 0;
};

/*
 * In case of deactivate:
 *   - failed assertion, since this transition 
 *     doesn't exist for default case
 */
const CCMObjectState* CCMObjectState_Removed::deactivate(CCMObject_impl* ccm_object) const
    throw (Components::CCMException, CORBA::SystemException) 
{
    std::cerr << "'deactivate' transition not authorized from " 
              << "CCM_OBJECT_STATE_REMOVED"<< std::endl;
    CDMW_ASSERT(false);

    // for compiler
    return 0;
};

/*
 * In case of operational_request:
 *   - failed assertion, since check_removed should have raisen OBJECT_NOT_EXISTS
 */
const CCMObjectState* CCMObjectState_Removed::operational_request(CCMObject_impl* ccm_object) const
    throw (CORBA::SystemException) 
{
    CDMW_ASSERT(false);

    // for compiler
    return 0;
};


    
} // end namespace CIF
} // end namespace CCM
} // end namespace Cdmw
