/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO 
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2009. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#include <iostream>

// Cdmw Files
#include <Foundation/orbsupport/CORBA.hpp>
#include <Foundation/orbsupport/OrbSupport.hpp>
#include <Foundation/orbsupport/ExceptionMinorCodes.hpp>
#include <Foundation/common/Assert.hpp>
#include <Foundation/commonsvcs/naming/NamingInterface.hpp>
#include <Foundation/logging/FunctionLogger.hpp>
#include <FaultTolerance/idllib/FT.stub.hpp>

// Cdmw CCM Files
#include <Foundation/idllib/CdmwDeployment.stub.hpp>
#include <CCMContainer/ccmcommon/CCMUtils.hpp>
#include <CCMContainer/ccmcommon/CCMLoggers.hpp>
#include <CCMContainer/ccmcif/FTCCMHome_impl.hpp>

namespace Cdmw {

namespace CCM {

namespace CIF {


/* This class inherits from CCMHome_impl and manages FT aspects for any CCM 
 * home. This is the base class for CARDAMOM generated CCM home in an FT 
 * environement.
 */
FTCCMHome_impl::FTCCMHome_impl(
      const std::string&                         rep_id,
      const std::string&                         comp_rep_id,
      const Cdmw::LifeCycle::ObjectDeactivator&  deactivator,
      CdmwLifeCycle::ObjectRegistration_ptr      object_registration,
      const Components::ConfigValues&            config)
  throw(CORBA::SystemException)
  : CCMHome_impl(rep_id, comp_rep_id, deactivator, object_registration, config, true)
{
    CDMW_LOG_FUNCTION(CCMLoggers::Get_ccmcif_logger(), 
        "rep_id="<<rep_id<<"  comp_rep_id="<<comp_rep_id);
    // m_ft_rep_mng and m_ft_group_rep will be set in set_context()
    m_ft_rep_mng = CdmwFT::ReplicationManager::_nil();
    m_ft_group_rep = CdmwFT::Location::GroupRepository::_nil();
    
    CORBA::Any_var ft_rep_style_any;
    bool ft_config_value_found = 
       Common::read_config_value(CdmwDeployment::FAULT_TOLERANCE_REPLICATION_STYLE, 
                         config, 
                         ft_rep_style_any); 

    if (ft_config_value_found) 
    {
        CdmwDeployment::FTReplicationStyleValue ft_rep_style;
        if (ft_rep_style_any.in() >>= ft_rep_style) 
        {       
            if (ft_rep_style != CdmwDeployment::WARM_PASSIVE)  
            {
                _set_ref_count(0);
                CDMW_ERROR(CCMLoggers::Get_ccmcif_logger(), 
                    "Unsupported FT Configuration values for FT home creation");
                CORBA::BAD_PARAM ex(
                      OrbSupport::BAD_PARAMInvalidConfigValues, 
                      CORBA::COMPLETED_NO);
                CDMW_LOG_FUNCTION_EXCEPTION(ex);
                throw ex;
            }
        }
        else 
        {
            _set_ref_count(0);
            CDMW_ERROR(CCMLoggers::Get_ccmcif_logger(), 
                "Cannot read FT Configuration values for FT home creation");
            CORBA::BAD_PARAM ex(
                  OrbSupport::BAD_PARAMInvalidConfigValues, 
                  CORBA::COMPLETED_NO);
            CDMW_LOG_FUNCTION_EXCEPTION(ex);
            throw ex;
        }
    }
    else 
    {
        _set_ref_count(0);
        CDMW_ERROR(CCMLoggers::Get_ccmcif_logger(),
                  "Expected ConfigValue " 
                  << CdmwDeployment::FAULT_TOLERANCE_REPLICATION_STYLE
                  << " not found");
        CORBA::BAD_PARAM ex(
              OrbSupport::BAD_PARAMInvalidConfigValues,
              CORBA::COMPLETED_NO);
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw ex;
    }
}

FTCCMHome_impl::~FTCCMHome_impl() 
    throw()
{
    CDMW_LOG_FUNCTION(CCMLoggers::Get_ccmcif_logger(), "");
}

void 
FTCCMHome_impl::set_context(CdmwCcmContainer::CCM2Context_ptr ctxt)
throw (Components::CCMException,
       CORBA::SystemException)
{
    CDMW_LOG_FUNCTION(CCMLoggers::Get_ccmcif_logger(), "");
    // call 
    CCMHome_impl::set_context(ctxt);

    try
    {
        CORBA::Object_var obj_ref 
             = m_context->resolve_initial_references("FTCurrent");
    }
    catch (const CdmwCcmContainer::CCM2Context::InvalidName& )
    {
        // Home is FT, process should be FT
        // This pb should be detected when installing the Home. If the user use
        // CDMW Code generator, all is ok (no FT allocator) but if he modified
        // the generated code...
        // Then the container shall check the FTCurrent before creating the home.
        CDMW_ERROR(CCMLoggers::Get_ccmcif_logger(), 
            "set_context: FT Home is waiting for a FT context from a FT process");
        CORBA::BAD_PARAM ex(Cdmw::OrbSupport::BAD_PARAMWrongContextType,
                            CORBA::COMPLETED_NO);
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw ex;
    }

    try {
        // get ReplicationManager reference
        CORBA::Object_var obj_ref = 
            m_context->resolve_initial_references("ReplicationManager");
        m_ft_rep_mng = CdmwFT::ReplicationManager::_narrow(obj_ref.in());
        CDMW_ASSERT(!CORBA::is_nil(m_ft_rep_mng.in()));
    }
    catch (const CdmwCcmContainer::CCM2Context::InvalidName& )
    {
        // shouldn't happen
        CDMW_ERROR(CCMLoggers::Get_ccmcif_logger(), 
             "set_context: Cannot find ReplicationManager");
        CDMW_ASSERT(false);
    }

    try {
        // get FTGroupRepository reference
        CORBA::Object_var obj_ref = 
            m_context->resolve_initial_references("FTGroupRepository");
        m_ft_group_rep = CdmwFT::Location::GroupRepository::_narrow(obj_ref.in());
        CDMW_ASSERT(!CORBA::is_nil(m_ft_group_rep.in()));
    }
    catch (const CdmwCcmContainer::CCM2Context::InvalidName& )
    {
        // shouldn't happen
        CDMW_ERROR(CCMLoggers::Get_ccmcif_logger(), 
             "set_context: Cannot find FTGroupRepository");
        CDMW_ASSERT(false);
    }


}


CORBA::Object_ptr
FTCCMHome_impl::create_component_ref(const Components::ConfigValues& config, Components::EnterpriseComponent_ptr  comp_exec)
    throw(Components::CreateFailure,
          CORBA::SystemException)
{
    CDMW_LOG_FUNCTION(CCMLoggers::Get_ccmcif_logger(), "");
    
    if (is_removed(m_oid.in()))
    {
        CDMW_ERROR(CCMLoggers::Get_ccmcif_logger(), 
            "Home has been removed");
        CORBA::OBJECT_NOT_EXIST ex(Cdmw::OrbSupport::OBJECT_NOT_EXIST,
                                   CORBA::COMPLETED_NO); 
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw ex;
    }
    CDMW_ASSERT(m_context);

    // get FAULT_TOLERANCE_GROUP_NAME Config value
    CORBA::Any_var ft_group_name_any;
    bool ft_config_value_found = 
       Common::read_config_value(CdmwDeployment::FAULT_TOLERANCE_GROUP_NAME, 
                         config, 
                         ft_group_name_any); 
    if (! ft_config_value_found) 
    {
        CDMW_ERROR(CCMLoggers::Get_ccmcif_logger(), 
            "Missing FT Configuration values for FT component creation: "
                    << CdmwDeployment::FAULT_TOLERANCE_GROUP_NAME
                    << " is not specified");
        Components::CreateFailure ex(CdmwDeployment::MISSING_FT_CONFIG_VALUES);
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw ex;
    }
    
    
    // extract FAULT_TOLERANCE_GROUP_NAME Config value
    const char* ft_group_name;
    if (! (ft_group_name_any >>= ft_group_name))
    {
        CDMW_ERROR(CCMLoggers::Get_ccmcif_logger(), 
              "Wrong FT Configuration values for FT component creation: "
             << CdmwDeployment::FAULT_TOLERANCE_GROUP_NAME
             << " value is not a string.");
        throw Components::CreateFailure
           (CdmwDeployment::WRONG_FT_CONFIG_VALUES);
    }
    
    // get group reference for FAULT_TOLERANCE_GROUP_NAME Config value
    ::FT::ObjectGroup_var ft_group_ref = get_group_for_name(ft_group_name);

    // create component reference
    std::string comp_oid;
    CORBA::Object_var result = create_component_ref_i(comp_oid, comp_exec);

    // add reference to FT group
    try {
        ::FT::Location_var location = m_ft_group_rep->the_location();
        
        CDMW_INFO(CCMLoggers::Get_ccmcif_logger(), 
             "Add this component as member of FT group " << ft_group_name);
        
        ft_group_ref = 
            m_ft_rep_mng->add_member(ft_group_ref.in(), 
                                     location.in(), 
                                     result.in());
                                     
    }
    catch (const FT::ObjectGroupNotFound& e) {
        // Group was found in get_group_for_name, but no more exists ?
        // It could have been removed in the meanwhile...
        CDMW_ERROR(CCMLoggers::Get_ccmcif_logger(), 
              "Wrong FT Configuration values for FT component creation: "
                    << ft_group_name << " group not found!");
        throw Components::CreateFailure
           (CdmwDeployment::WRONG_FT_CONFIG_VALUES);
    }
    catch (const FT::MemberAlreadyPresent& e) {
        // Member has just been created ! this shouldn't happen
        CDMW_ERROR(CCMLoggers::Get_ccmcif_logger(), 
              "Component already present in group!!");
        CDMW_ASSERT(false);
    }
    catch (const FT::ObjectNotAdded& e) {
        CDMW_ERROR(CCMLoggers::Get_ccmcif_logger(), 
              "ReplicationManager raised FT::ObjectNotAdded "
                    << "adding component in FT group "<< ft_group_name);
        throw Components::CreateFailure
           (CdmwDeployment::REPLICATION_MANAGER_ERROR);
    }
    catch (const CORBA::SystemException& e) {
        CDMW_ERROR(CCMLoggers::Get_ccmcif_logger(), 
              "Exception adding component in FT group "<< ft_group_name
                    << Cdmw::OrbSupport::OrbSupport::exception_to_string(e));
        throw Components::CreateFailure
           (CdmwDeployment::REPLICATION_MANAGER_ERROR);
    }
    
    // get COMPONENT_NAME Config value
    CORBA::Any_var component_name_any;
    std::string s_component_name("");
    if (Common::read_config_value(CdmwDeployment::COMPONENT_NAME, 
                                  config, 
                                  component_name_any))
    {
        const char* component_name;
        component_name_any >>= component_name;
        s_component_name = component_name;
        // notify Supervision of add_member
        notify_add_member(component_name);
        
    } else {
        CDMW_WARN(CCMLoggers::Get_ccmcif_logger(), 
              "No ComponentName defined for this component. " <<
              "Cannot notify Supervision that add_member has been performed.");
    }
     
    // Setting realised to inject information about timeouts of receptacles.
    // That's the context of a home instance, which stores the config values.
    try{

        m_context->set_config_values(comp_oid, config);

        CDMW_DEBUG(CCMLoggers::Get_ccmcif_logger(), 
            "CCMHome_impl create_component_ref: length config values ="
            << config.length());

    }catch( Cdmw::Common::AlreadyExistException ) {

        CDMW_ERROR(CCMLoggers::Get_ccmcif_logger(), 
            "CCMHome_impl create_component_ref: config values exist already!");

        CORBA::INTERNAL ex(OrbSupport::INTERNALCCMCIFError,
                           CORBA::COMPLETED_NO);
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw ex;
    }

    register_component(config, result.in());
    
    // register component for recording (if Recording ConfigValue set to true)
    // NOTA: Failing to record the component for recording will only log
    //       a WARN message and not abort deployment of component.
    try {
       m_context->register_for_recording(comp_oid, result.in());
    }
    catch (const Components::CreateFailure& e)
    {
       std::string comp_identification(comp_oid);
       if (!s_component_name.empty())
          comp_identification = s_component_name;
       CDMW_ERROR(CCMLoggers::Get_ccmcif_logger(),
                  "Error registering component '" << comp_identification << 
                  "' for recording: " <<
                  (e.reason == CdmwDeployment::WRONG_RECORDING_CONFIG_VALUES ?
                   "Wrong Recording ConfigValue" : "Recording Daemon error") );
       CDMW_WARN(CCMLoggers::Get_ccmcif_logger(),
                 "Incoming requets to component '" << comp_identification << 
                 "' will not be recorded!");
    }

    m_comp_groups.insert(ComponentGroupMap::value_type(comp_oid, 
                                                       ft_group_ref));
    CDMW_DEBUG(CCMLoggers::Get_ccmcif_logger(), 
        "Add group ref in map for component " << comp_oid);

    return result._retn();
}

void 
FTCCMHome_impl::delete_component_info(const std::string& comp_oid)
{
    CDMW_LOG_FUNCTION(CCMLoggers::Get_ccmcif_logger(), 
        "comp_oid="<<comp_oid);
    m_comp_groups.erase(comp_oid);

    CCMHome_impl::delete_component_info(comp_oid);
}

CORBA::Object_ptr 
FTCCMHome_impl::find_component_group(const std::string& comp_oid)
   throw (CORBA::SystemException)
{
    CDMW_LOG_FUNCTION(CCMLoggers::Get_ccmcif_logger(), 
        "comp_oid="<<comp_oid);
    ComponentGroupMap::iterator comp_pos = m_comp_groups.find(comp_oid);
        
    if (comp_pos  == m_comp_groups.end()) 
    {
        CDMW_ERROR(CCMLoggers::Get_ccmcif_logger(), 
            "No group found for this component");
        CORBA::OBJECT_NOT_EXIST ex(Cdmw::OrbSupport::OBJECT_NOT_EXIST,
                                  CORBA::COMPLETED_NO);
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw ex;
    }

    ComponentGroupMap::value_type& comp_value = *comp_pos;
    
    return CORBA::Object::_duplicate(comp_value.second.in());
}

::FT::ObjectGroup_ptr
FTCCMHome_impl::get_group_for_name(const char* name)
    throw(Components::CreateFailure, 
          CORBA::SystemException)
{
    CDMW_LOG_FUNCTION(CCMLoggers::Get_ccmcif_logger(), 
        "name="<<name);
    try {
        CosNaming::Name_var cos_name = 
            Cdmw::CommonSvcs::Naming::NamingInterface::to_name(name);
    
        return m_ft_rep_mng->get_object_group_ref_from_name(cos_name.in());
            
    } catch (const Cdmw::CommonSvcs::Naming::InvalidNameException& e) {
        CDMW_ERROR(CCMLoggers::Get_ccmcif_logger(), 
              "Wrong FT Configuration values for FT component creation: "
              << name << " is not a valid group name!");
        throw Components::CreateFailure
           (CdmwDeployment::WRONG_FT_CONFIG_VALUES);
    } catch (const FT::ObjectGroupNotFound& e) {
        CDMW_ERROR(CCMLoggers::Get_ccmcif_logger(), 
              "Wrong FT Configuration values for FT component creation: "
              << name << " group not found!");
        throw Components::CreateFailure
           (CdmwDeployment::WRONG_FT_CONFIG_VALUES);
    }
}


} // End of namespace CIF

} // End of namespace CCM

} // End of namespace Cdmw

