/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO 
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2009. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


// Cdmw CCM files
#include <CCMContainer/ccmcif/FTCCMObjectStates.hpp>
#include <CCMContainer/ccmcif/FTCCMObject_impl.hpp>
#include <Foundation/common/Assert.hpp>


namespace Cdmw {

namespace CCM {

namespace CIF {


//
// Implementation of INITIAL FTCCMObject's state
//

/*
 * In case of configuration_complete:
 *   - change state to CCM_ACTIVATED
 */
const CCMObjectState* FTCCMObjectState_Initial::configuration_complete(CCMObject_impl* ccm_object) const
    throw (Components::CCMException, CORBA::SystemException) 
{
    // change state to CCM_ACTIVATED
    return FT_CCM_OBJECT_STATE_CCM_ACTIVATED;
};

/*
 * In case of remove:
 *   - call ccm_remove
 *   - change state to REMOVED
 */
const CCMObjectState* FTCCMObjectState_Initial::remove(CCMObject_impl* ccm_object) const
    throw (Components::CCMException, CORBA::SystemException) 
{
    // call ccm_remove
    ccm_object->ccm_remove();
    
    // change state to REMOVED
    return FT_CCM_OBJECT_STATE_REMOVED;
};

/*
 * In case of activate:
 *   - change state to FT_ACTIVATED
 */
const CCMObjectState* FTCCMObjectState_Initial::activate(CCMObject_impl* ccm_object) const
    throw (Components::CCMException, CORBA::SystemException) 
{
    // change state to FT_ACTIVATED
    return FT_CCM_OBJECT_STATE_FT_ACTIVATED;
};

/*
 * In case of deactivate:
 *   - was not activated; ignore
 */
const CCMObjectState* FTCCMObjectState_Initial::deactivate(CCMObject_impl* ccm_object) const
    throw (Components::CCMException, CORBA::SystemException) 
{
    // ignore: just return same state
    return this;
};

/*
 * In case of operational_request:
 *   - If the request is an IOGR or all the receptacle are not connected
 *    thus raise a TRANSIENT CORBA exception. Otherwise stay in the same state.
 */
const CCMObjectState* FTCCMObjectState_Initial::operational_request(CCMObject_impl* ccm_object) const
    throw (CORBA::SystemException) 
{
  // Use FTCurrent to check if request was performed via an IOGR
  FTCCMObject_impl * ftccm_object = dynamic_cast<FTCCMObject_impl*>(ccm_object);
  CDMW_ASSERT(ftccm_object != 0);
  if (ftccm_object->is_a_group_call()) {
    CORBA::TRANSIENT ex(OrbSupport::TRANSIENT,
                         CORBA::COMPLETED_NO);
    std::cerr << "FTCCMObjectState_Initial::operational_request: the request was not performed via an IOGR."
              << std::endl;
    throw ex; 
  }
  else {
    std::string receptacle_name = ccm_object->check_connections();

      // raise TRANSIENT if at least a receptacle is still to be connected
    if(!receptacle_name.empty()) {
      CORBA::TRANSIENT ex(OrbSupport::TRANSIENTMissingConnection,
                          CORBA::COMPLETED_NO);
      std::cerr << "FTCCMObjectState_Initial::operational_request: the receptacle " << receptacle_name << " is not connected."
                << std::endl;
      throw ex;
    }
  }
  // ignore: just return same state
  return this;
};

//
// Implementation of FT_ACTIVATED FTCCMObject's state
//

/*
 * In case of configuration_complete:
 *   - call ccm_activate
 *   - change state to FULLY_ACTIVATED
 */
const CCMObjectState* FTCCMObjectState_FTActivated::configuration_complete(CCMObject_impl* ccm_object) const
    throw (Components::CCMException, CORBA::SystemException) 
{
    // notify Supervision of FT activation
    FTCCMObject_impl* ft_ccm_object = 
        dynamic_cast<FTCCMObject_impl*>(ccm_object);
    CDMW_ASSERT(ft_ccm_object != 0);
    ft_ccm_object->notify_ft_activate();
    
    // call ccm_activate
    ccm_object->ccm_activate();
    
    // change state to FULLY_ACTIVATED
    return FT_CCM_OBJECT_STATE_FULLY_ACTIVATED;
};

/*
 * In case of remove:
 *   - call ccm_remove
 *   - change state to REMOVED
 */
const CCMObjectState* FTCCMObjectState_FTActivated::remove(CCMObject_impl* ccm_object) const
    throw (Components::CCMException, CORBA::SystemException) 
{
    // call ccm_remove
    ccm_object->ccm_remove();
    
    // change state to REMOVED
    return FT_CCM_OBJECT_STATE_REMOVED;
};

/*
 * In case of activate:
 *   - already activated; ignore
 */
const CCMObjectState* FTCCMObjectState_FTActivated::activate(CCMObject_impl* ccm_object) const
    throw (Components::CCMException, CORBA::SystemException) 
{
    // ignore: just return same state
    return this;
};

/*
 * In case of deactivate:
 *   - go back to INITIAL state
 */
const CCMObjectState* FTCCMObjectState_FTActivated::deactivate(CCMObject_impl* ccm_object) const
    throw (Components::CCMException, CORBA::SystemException) 
{
    // change state to INITIAL
    return FT_CCM_OBJECT_STATE_INITIAL;
};

/*
 * In case of operational_request:
 *   - If the request is an IOGR or all the receptacle are not connected
 *    thus raise a TRANSIENT CORBA exception. Otherwise stay in the same state.
 */
const CCMObjectState* FTCCMObjectState_FTActivated::operational_request(CCMObject_impl* ccm_object) const
    throw (CORBA::SystemException) 
{
  // Use FTCurrent to check if request was performed via an IOGR
  FTCCMObject_impl * ftccm_object = dynamic_cast<FTCCMObject_impl*>(ccm_object);
  CDMW_ASSERT(ftccm_object != 0);
  if (ftccm_object->is_a_group_call()) {
    // Trace not kept to not pollute the displays. And no logger found to set a log level
//     std::cerr << "FTCCMObjectState_FTActivated::operational_request: the request was not performed via an IOGR."
//               << std::endl;
    CORBA::TRANSIENT ex(OrbSupport::TRANSIENT,
                         CORBA::COMPLETED_NO);
    throw ex; 
  } 
  else {
    std::string receptacle_name = ccm_object->check_connections();

      // raise TRANSIENT if at least a receptacle is still to be connected
    if(!receptacle_name.empty()) {
      CORBA::TRANSIENT ex(OrbSupport::TRANSIENTMissingConnection,
                          CORBA::COMPLETED_NO);
      std::cerr << "FTCCMObjectState_FTActivated::operational_request: the receptacle " 
                << receptacle_name
                << " is not connected."
                << std::endl;
        
      throw ex;
    }
  }
  return this;
};


//
// Implementation of CCM_ACTIVATED FTCCMObject's state
//

/*
 * In case of configuration_complete:
 *   - already configured; ignore
 */
const CCMObjectState* FTCCMObjectState_CCMActivated::configuration_complete(CCMObject_impl* ccm_object) const
    throw (Components::CCMException, CORBA::SystemException) 
{
    // ignore: just return same state
    return this;
};

/*
 * In case of remove:
 *   - call ccm_remove
 *   - change state to REMOVED
 */
const CCMObjectState* FTCCMObjectState_CCMActivated::remove(CCMObject_impl* ccm_object) const
    throw (Components::CCMException, CORBA::SystemException) 
{
    // call ccm_remove
    ccm_object->ccm_remove();
    
    // change state to REMOVED
    return FT_CCM_OBJECT_STATE_REMOVED;
};

/*
 * In case of activate:
 *   - call ccm_activate
 *   - change state to FULLY_ACTIVATED
 */
const CCMObjectState* FTCCMObjectState_CCMActivated::activate(CCMObject_impl* ccm_object) const
    throw (Components::CCMException, CORBA::SystemException) 
{
    // notify Supervision of FT activation
    FTCCMObject_impl* ft_ccm_object = 
        dynamic_cast<FTCCMObject_impl*>(ccm_object);
    CDMW_ASSERT(ft_ccm_object != 0);
    ft_ccm_object->notify_ft_activate();
    
    // call ccm_activate
    ccm_object->ccm_activate();
    
    // change state to FULLY_ACTIVATED
    return FT_CCM_OBJECT_STATE_FULLY_ACTIVATED;
};

/*
 * In case of deactivate:
 *   - was not activated; ignore
 */
const CCMObjectState* FTCCMObjectState_CCMActivated::deactivate(CCMObject_impl* ccm_object) const
    throw (Components::CCMException, CORBA::SystemException) 
{
    // ignore: just return same state
    return this;
};

/*
 * In case of operational_request:
 *   - If the request is an IOGR or all the receptacle are not connected
 *    thus raise a TRANSIENT CORBA exception. Otherwise stay in the same state.
 */
const CCMObjectState* FTCCMObjectState_CCMActivated::operational_request(CCMObject_impl* ccm_object) const
    throw (CORBA::SystemException) 
{
  // Use FTCurrent to check if request was performed via an IOGR
  FTCCMObject_impl * ftccm_object = dynamic_cast<FTCCMObject_impl*>(ccm_object);
  CDMW_ASSERT(ftccm_object != 0);
  if (ftccm_object->is_a_group_call()) {
    CORBA::TRANSIENT ex(OrbSupport::TRANSIENT,
                        CORBA::COMPLETED_NO);
    std::cerr << "FTCCMObjectState_CCMActivated::operational_request: the request was not performed via an IOGR."
              << std::endl;
    throw ex; 
  }

  // ignore: just return same state
  return this;
    
};

//
// Implementation of FULLY_ACTIVATED FTCCMObject's state
//

/*
 * In case of configuration_complete:
 *   - already configured; ignore
 */
const CCMObjectState* FTCCMObjectState_FullyActivated::configuration_complete(CCMObject_impl* ccm_object) const
    throw (Components::CCMException, CORBA::SystemException) 
{
    // ignore: just return same state
    return this;
};

/*
 * In case of remove:
 *   - call ccm_passivate
 *   - call ccm_remove
 *   - change state to REMOVED
 */
const CCMObjectState* FTCCMObjectState_FullyActivated::remove(CCMObject_impl* ccm_object) const
    throw (Components::CCMException, CORBA::SystemException) 
{
    // notify Supervision of FT deactivation
    FTCCMObject_impl* ft_ccm_object = 
        dynamic_cast<FTCCMObject_impl*>(ccm_object);
    CDMW_ASSERT(ft_ccm_object != 0);
    
    // ECR-0412    
    // ft_ccm_object->notify_ft_deactivate();
    
    // call ccm_passivate
    ccm_object->ccm_passivate();
    
    // call ccm_remove
    ccm_object->ccm_remove();
    
    // change state to REMOVED
    return FT_CCM_OBJECT_STATE_REMOVED;
};

/*
 * In case of activate:
 *   - was already activated; ignore
 */
const CCMObjectState* FTCCMObjectState_FullyActivated::activate(CCMObject_impl* ccm_object) const
    throw (Components::CCMException, CORBA::SystemException) 
{
    // ignore: just return same state
    return this;
};

/*
 * In case of deactivate:
 *   - call ccm_passivate
 *   - go back to CCM_ACTIVATED state
 */
const CCMObjectState* FTCCMObjectState_FullyActivated::deactivate(CCMObject_impl* ccm_object) const
    throw (Components::CCMException, CORBA::SystemException) 
{
    // notify Supervision of FT deactivation
    FTCCMObject_impl* ft_ccm_object = 
        dynamic_cast<FTCCMObject_impl*>(ccm_object);
    CDMW_ASSERT(ft_ccm_object != 0);
    
    ft_ccm_object->notify_ft_deactivate();
    
    // call ccm_passivate
    ccm_object->ccm_passivate();
    
    // change state to CCM_ACTIVATED
    return FT_CCM_OBJECT_STATE_CCM_ACTIVATED;
};

/*
 * In case of operational_request:
 *   - If the request is an IOGR or all the receptacle are not connected
 *    thus raise a TRANSIENT CORBA exception. Otherwise stay in the same state.
 */
const CCMObjectState* FTCCMObjectState_FullyActivated::operational_request(CCMObject_impl* ccm_object) const
    throw (CORBA::SystemException) 
{
  // ignore: just return same state
  return this;
};


//
// Implementation of REMOVED FTCCMObject's state
//

/*
 * In case of configuration_complete:
 *   - failed assertion, since component is removed
 */
const CCMObjectState* FTCCMObjectState_Removed::configuration_complete(CCMObject_impl* ccm_object) const
    throw (Components::CCMException, CORBA::SystemException) 
{
    std::cerr << "'configuration_complete' transition not authorized from " 
              << "FT_CCM_OBJECT_STATE_REMOVED"<< std::endl;
    CDMW_ASSERT(false);
    
    // for compiler
    return 0;
};

/*
 * In case of remove:
 *   - already removed; ignore
 */
const CCMObjectState* FTCCMObjectState_Removed::remove(CCMObject_impl* ccm_object) const
    throw (Components::CCMException, CORBA::SystemException) 
{
    // ignore: just return same state
    return this;
};

/*
 * In case of activate:
 *   - FT process activated while removing ? Ignore
 */
const CCMObjectState* FTCCMObjectState_Removed::activate(CCMObject_impl* ccm_object) const
    throw (Components::CCMException, CORBA::SystemException) 
{
    // ignore: just return same state
    return this;
};

/*
 * In case of deactivate:
 *   - FT process deactivated while removing ? Ignore
 */
const CCMObjectState* FTCCMObjectState_Removed::deactivate(CCMObject_impl* ccm_object) const
    throw (Components::CCMException, CORBA::SystemException) 
{
    // ignore: just return same state
    return this;
};

/*
 * In case of operational_request:
 *   - If the request is an IOGR or all the receptacle are not connected
 *    thus raise a TRANSIENT CORBA exception. Otherwise stay in the same state.
 */
const CCMObjectState* FTCCMObjectState_Removed::operational_request(CCMObject_impl* ccm_object) const
    throw (CORBA::SystemException) 
{
  // ignore: just return same state
  return this;
};


    
} // end namespace CIF
} // end namespace CCM
} // end namespace Cdmw
