/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO 
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2009. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

#include <iostream>

// Cdmw files
#include <Foundation/orbsupport/ExceptionMinorCodes.hpp>
#include <Foundation/orbsupport/CORBA.hpp>
#include <Foundation/orbsupport/OrbSupport.hpp>
#include <Foundation/common/Assert.hpp>
#include <Foundation/logging/FunctionLogger.hpp>

// Cdmw CCM files
#include <CCMContainer/ccmcif/FTCCMObject_impl.hpp>
#include <CCMContainer/ccmcommon/CCMLoggers.hpp>


namespace Cdmw {

namespace CCM {

namespace CIF {

FTCCMObject_impl::FTCCMObject_impl(const std::string&  comp_oid,
                                   Context*            ctx,
                                   const std::string&  rep_id,
                                   CORBA::Object_ptr   comp_ref,
                                   CdmwLifeCycle::ObjectRegistration_ptr object_registration,
                                   CdmwLogging::LoggerFactory_ptr logger_factory,
                                   bool                is_monolithic)
   : CCMObject_impl(comp_oid, ctx, rep_id, comp_ref, object_registration, logger_factory, is_monolithic)
{
    // create logger with domain for ccmcif
    m_logger = logger_factory->get_logger(CCMLoggers::CCMCIF_LOG_DOMAIN);
    
    CDMW_LOG_FUNCTION(m_logger.in(), "oid="<<comp_oid<<"  rep_id="<<rep_id);
    
    try
    {
        CORBA::Object_var ft_current_obj = 
           m_context->resolve_initial_references("FTCurrent");

        m_ft_current = CdmwFT::Current::_narrow(ft_current_obj.in());
        CDMW_ASSERT(!CORBA::is_nil(m_ft_current.in()));

        CORBA::Object_var ft_group_rep_obj =
           m_context->resolve_initial_references("FTGroupRepository");
    
        m_ft_group_rep = 
           CdmwFT::Location::GroupRepository::_narrow(ft_group_rep_obj.in());
        CDMW_ASSERT(!CORBA::is_nil(m_ft_group_rep.in()));

        CORBA::Object_var ft_rep_mng_obj = 
           m_context->resolve_initial_references("ReplicationManager");

        m_ft_rep_mng = CdmwFT::ReplicationManager::_narrow(ft_rep_mng_obj.in());
        CDMW_ASSERT(!CORBA::is_nil(m_ft_rep_mng.in()));
    }
    catch (const CdmwCcmContainer::CCM2Context::InvalidName& e)
    {
        // FTCurrent/FTGroupRepository not defined, we are not in FT process,
        CDMW_FATAL(m_logger.in(), 
            "Cannot find FT objects. This is not an FT process!");
        CDMW_ASSERT(false);
    }
    catch (const CORBA::SystemException& e)
    {
        CDMW_FATAL(m_logger.in(), 
            "SystemException raised: " <<
            Cdmw::OrbSupport::OrbSupport::exception_to_string(e));
        CDMW_ASSERT(false);
    }

}
 
FTCCMObject_impl::~FTCCMObject_impl()
{
    CDMW_LOG_FUNCTION(m_logger.in(), "");
}


void FTCCMObject_impl::ft_activate()
    throw (CORBA::SystemException)
{
    // change state
    m_state = m_state->activate(this);
}


void FTCCMObject_impl::ft_deactivate()
    throw (CORBA::SystemException)
{
    // change state
    m_state = m_state->deactivate(this);
}


/**
* Purpose:
* <p>
*   return a group reference if the current request is invoked on a FT group reference.
*   Otherwise return the same reference.
* 
*@param obj_ref       The facet/consumer/component reference.
*
*@return  The reference to be exported to clients
*/ 
CORBA::Object_ptr 
FTCCMObject_impl::get_object_ref(const CORBA::Object_ptr obj_ref)
   throw (CORBA::SystemException)
{
    CDMW_LOG_FUNCTION(m_logger.in(), "");
    CORBA::Object_var result = CORBA::Object::_duplicate(obj_ref);

    ::FT::ObjectGroupId group_id = 0;
    try
    {
        CDMW_ASSERT(!CORBA::is_nil(m_ft_current.in()));
        
        // test if we are able to get group id: if OK it significates it is a FT 
        // request on a group ref. Otherwise, it will raise NoContext exception.
        group_id = m_ft_current->get_object_group_id();
    
        // No NoContext exception raised: this is a FT request on a group ref.
        // Find and return the corresponding group reference of the given object ref.
        // To do so, we use the FTGroupRepository to fing the group from the object 
        // ref instead from the group_id on which the request is done. It may be
        // different in segmented case.
        if (!CORBA::is_nil(m_ft_group_rep.in()))
        {
            result = m_ft_group_rep->get_object_group(obj_ref);
        }
    }
    catch (const CdmwFT::Current::NoContext&)
    {
        // This is not a FT request, return the object ref
        CDMW_DEBUG(m_logger.in(), 
            "Not a FT request; return object reference");
    }
    catch (const FT::MemberNotFound& )
    {
        CDMW_DEBUG(m_logger.in(), 
            "MemberNotFound; check with ReplicationManager");
            
        // Object ref is not a member of a group
        // May be the location manager has not been advised by the replication 
        // yet. We can check replication manager.
        // Currently, component are monolithic if FT. So the group id of the 
        // request is the one of the specified object ref.
        // TODO add an operation on replication manager to find the group ref 
        // from a member ref.
        try 
        {
           if (!CORBA::is_nil(m_ft_rep_mng.in())) 
           {
               result = m_ft_rep_mng->get_object_group_ref_from_gid(group_id);
           }
        }
        catch (const ::FT::ObjectGroupNotFound&)
        {
            // Shall not happen: it means the group has not been created by this 
            // replication manager.
            CDMW_ERROR(m_logger.in(), 
                "FT::ObjectGroupNotFound raised!");
            CORBA::INTERNAL ex(OrbSupport::INTERNAL, CORBA::COMPLETED_NO);
            CDMW_LOG_FUNCTION_EXCEPTION(ex);
            throw ex;
        }
        catch (const CORBA::SystemException& e)
        {
            CDMW_ERROR(m_logger.in(), 
                "CORBA::SystemException raised : " << 
                Cdmw::OrbSupport::OrbSupport::exception_to_string(e));
            CDMW_LOG_FUNCTION_EXCEPTION(e);
            throw;
        }
    }
    catch (const CORBA::SystemException& e)
    {
        CDMW_ERROR(m_logger.in(), 
            "CORBA::SystemException raised : " << 
            Cdmw::OrbSupport::OrbSupport::exception_to_string(e));
        CDMW_LOG_FUNCTION_EXCEPTION(e);
        throw;
    }

    return result._retn();
}

/**
* Purpose:
* <p>
*   return a group reference if the current request is invoked on a FT group reference.
*   Otherwise return the same reference.
* 
*@param obj_ref       The facet/consumer/component reference.
*
*@return  The reference to be exported to clients
*/ 
bool 
FTCCMObject_impl::is_a_group_call()
   throw (CORBA::SystemException)
{
  CDMW_ASSERT(!CORBA::is_nil(m_ft_current.in()));
  ::FT::ObjectGroupId group_id = 0;
  try
    {
      // test if we are able to get group id: if OK it significates it is a FT 
      // request on a group ref. Otherwise, it will raise NoContext exception.
      group_id = m_ft_current->get_object_group_id();
    }
  catch (const CdmwFT::Current::NoContext&)
    {
      // This is not a FT request
      CDMW_DEBUG(m_logger.in(), 
                 "Not a FT request; return object reference");
      return false;
    }
  return true;
}

} // end namespace CIF

} // end namespace CCM

} // end namespace Cdmw
