/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO 
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2009. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#include <iostream>

// Cdmw Files
#include <Foundation/orbsupport/CORBA.hpp>
#include <Foundation/orbsupport/OrbSupport.hpp>
#include <Foundation/orbsupport/ExceptionMinorCodes.hpp>
#include <Foundation/common/Assert.hpp>
#include <Foundation/commonsvcs/naming/NamingInterface.hpp>
#include <Foundation/ossupport/OS.hpp>
#include <SystemMngt/platforminterface/PlatformInterface.hpp>

// Cdmw CCM Files
#include <Foundation/idllib/CdmwDeployment.stub.hpp>
#include <CCMContainer/ccmcommon/CCMUtils.hpp>
#include <CCMContainer/ccmcommon/CCMLoggers.hpp>
#include <CCMContainer/ccmcif/LBCCMHome_impl.hpp>



namespace Cdmw {

namespace CCM {

namespace CIF {


/* This class inherits from CCMHome_impl and manages LB aspects for any CCM 
 * home. This is the base class for CARDAMOM generated CCM home in an LB 
 * environement.
 */
LBCCMHome_impl::LBCCMHome_impl(
      const std::string&                         rep_id,
      const std::string&                         comp_rep_id,
      const Cdmw::LifeCycle::ObjectDeactivator&  deactivator,
      CdmwLifeCycle::ObjectRegistration_ptr      object_registration,
      const Components::ConfigValues&            config)
  throw(CORBA::SystemException)
  : CCMHome_impl(rep_id, comp_rep_id, deactivator, object_registration, config, true)
{
    CDMW_LOG_FUNCTION(CCMLoggers::Get_ccmcif_logger(), "");
    // m_lb_grp_mgr will be set in set_context()
    m_lb_grp_mgr = CdmwLB::LBGroupManager::_nil();
    
/****** PCR-1378: comment this code since the LoadBalancingStrategy ConfigValue
                  is useless at this time, and provokes an error. 
                  
    CORBA::Any_var lb_strategy_any;
    bool lb_config_value_found = 
       Common::read_config_value(CdmwDeployment::LOAD_BALANCING_STRATEGY, 
                         config, 
                         lb_strategy_any); 

    if (lb_config_value_found) 
    {
        const char* lb_strategy;
        if (lb_strategy_any.in() >>= lb_strategy) 
        {
            // TODO: manage LB Strategy
        }
        else 
        {
            _set_ref_count(0);
            CDMW_ERROR(CCMLoggers::Get_ccmcif_logger(),
                  "Cannot read LB Configuration values for LB home creation");
            CORBA::BAD_PARAM ex(
                  OrbSupport::BAD_PARAMInvalidConfigValues,
                  CORBA::COMPLETED_NO);
            CDMW_LOG_FUNCTION_EXCEPTION(ex);
            throw ex;
        }
    } 
    else 
    {
        _set_ref_count(0);
        CDMW_ERROR(CCMLoggers::Get_ccmcif_logger(),
              "Expected ConfigValue " 
                    << CdmwDeployment::LOAD_BALANCING_STRATEGY << " not found");
        CORBA::BAD_PARAM ex( 
              OrbSupport::BAD_PARAMInvalidConfigValues,
              CORBA::COMPLETED_NO);
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw ex; 
    }
******/
}

LBCCMHome_impl::~LBCCMHome_impl() 
    throw()
{
    CDMW_LOG_FUNCTION(CCMLoggers::Get_ccmcif_logger(), "");
}

void 
LBCCMHome_impl::set_context(CdmwCcmContainer::CCM2Context_ptr ctxt)
throw (Components::CCMException,
       CORBA::SystemException)
{
    CDMW_LOG_FUNCTION(CCMLoggers::Get_ccmcif_logger(), "");
    // call 
    CCMHome_impl::set_context(ctxt);

    // Look for the LBGroupManager in the initial references.
    try {
        // get LBGroupManager reference
        CORBA::Object_var obj_ref = 
            m_context->resolve_initial_references("LBGroupManager");
        
        // try to narrow reference
        m_lb_grp_mgr = CdmwLB::LBGroupManager::_narrow(obj_ref.in());
    }
    catch (const CdmwCcmContainer::CCM2Context::InvalidName& )
    {
        // LBGroupManager not found.
        // It should have been specified as ORBInitRef (non-FT LBGroupManager)
        // or found from Repository during CDMW_init (FT LBGroupManager)
        CDMW_ERROR(CCMLoggers::Get_ccmcif_logger(),
              "LBGroupManager not registered as initial reference!");
        Components::CCMException ex(Components::SYSTEM_ERROR);
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw ex;
    }
    catch (const CORBA::SystemException& e)
    {
        // LBGroupManager was found, but an error occured contacting it.
        CDMW_ERROR(CCMLoggers::Get_ccmcif_logger(),
              "Exception contacting the LBGroupManager: " << e);
        Components::CCMException ex(Components::SYSTEM_ERROR);
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw ex;
    }
    
    if (CORBA::is_nil(m_lb_grp_mgr.in())) {
        CDMW_ERROR(CCMLoggers::Get_ccmcif_logger(),
              "LBGroupManager is nil !!");
        Components::CCMException ex(Components::SYSTEM_ERROR);
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw ex;
    }
    
}


CORBA::Object_ptr
LBCCMHome_impl::create_component_ref(const Components::ConfigValues& config, Components::EnterpriseComponent_ptr  comp_exec)
    throw(Components::CreateFailure,
          CORBA::SystemException)
{
    CDMW_LOG_FUNCTION(CCMLoggers::Get_ccmcif_logger(), "");
    if (is_removed(m_oid.in()))
    {
       throw CORBA::OBJECT_NOT_EXIST(Cdmw::OrbSupport::OBJECT_NOT_EXIST,
                                     CORBA::COMPLETED_NO); 
    }
    CDMW_ASSERT(m_context);

    // get LOAD_BALANCING_GROUP_NAME Config value
    CORBA::Any_var lb_group_name_any;
    if (! Common::read_config_value(CdmwDeployment::LOAD_BALANCING_GROUP_NAME, 
                            config, 
                            lb_group_name_any))
    {
        CDMW_ERROR(CCMLoggers::Get_ccmcif_logger(),
              "Missing LB Configuration values for LB component creation: "
                    << CdmwDeployment::LOAD_BALANCING_GROUP_NAME
                    << " is not specified");
        Components::CreateFailure ex(CdmwDeployment::MISSING_LB_CONFIG_VALUES);
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw ex;
    }
    
    
    // extract LOAD_BALANCING_GROUP_NAME Config value
    const char* lb_group_name;
    if (! (lb_group_name_any >>= lb_group_name))
    {
        CDMW_ERROR(CCMLoggers::Get_ccmcif_logger(),
             "Wrong LB Configuration values for LB component creation: "
                    << CdmwDeployment::LOAD_BALANCING_GROUP_NAME
                    << " value is not a string.");
        Components::CreateFailure ex(CdmwDeployment::WRONG_LB_CONFIG_VALUES);
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw ex;
    }
    
    // get group reference for LOAD_BALANCING_GROUP_NAME Config value
    ::PortableGroup::ObjectGroup_var lb_group_ref = get_group_for_name(lb_group_name);

    // create component reference
    std::string comp_oid;
    CORBA::Object_var result = create_component_ref_i(comp_oid, comp_exec);

    // add reference to LB group
    try {
        ::PortableGroup::Location_var location = get_current_lb_location();
        
        CDMW_INFO(CCMLoggers::Get_ccmcif_logger(),
             "Add this component as member of LB group " << lb_group_name);
        
        lb_group_ref = 
            m_lb_grp_mgr->add_member(lb_group_ref.in(), 
                                 location.in(), 
                                 result.in());
    }
    catch (const PortableGroup::MemberAlreadyPresent& e) {
        // Member has just been created ! this shouldn't happen
        CDMW_ERROR(CCMLoggers::Get_ccmcif_logger(),
              "Component already present in group!!");
        CDMW_ASSERT(false);
    }
    catch (const PortableGroup::ObjectNotAdded& e) {
        CDMW_ERROR(CCMLoggers::Get_ccmcif_logger(),
              "LBGroupManager raised PortableGroup::ObjectNotAdded "
                    << "adding component in LB group "<< lb_group_name);
        Components::CreateFailure ex(CdmwDeployment::LB_GROUP_MANAGER_ERROR);
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw ex;
    }
    catch (const CORBA::SystemException& e) {
        CDMW_ERROR(CCMLoggers::Get_ccmcif_logger(),
              "Exception adding component in LB group "<< lb_group_name
                    << Cdmw::OrbSupport::OrbSupport::exception_to_string(e));
        Components::CreateFailure ex(CdmwDeployment::LB_GROUP_MANAGER_ERROR);
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw ex;
    }
    
    // get COMPONENT_NAME Config value
    CORBA::Any_var component_name_any;
    std::string s_component_name("");
    if (Common::read_config_value(CdmwDeployment::COMPONENT_NAME, 
                                  config, 
                                  component_name_any))
    {
        const char* component_name;
        component_name_any >>= component_name;
        s_component_name = component_name;
        
        // notify Supervision of add_member
        notify_add_member(component_name);
        
    } else {
        CDMW_WARN(CCMLoggers::Get_ccmcif_logger(), 
              "No ComponentName defined for this component. " <<
              "Cannot notify Supervision that add_member has been performed.");
    }
         
    // Setting realised to inject information about timeouts of receptacles.
    // That's the context of a home instance, which stores the config values.
    try{

      m_context->set_config_values(comp_oid, config);

      CDMW_INFO(CCMLoggers::Get_ccmcif_logger(),
           "CCMHome_impl create_component_ref: length config values ="
                 << config.length());

    }catch( Cdmw::Common::AlreadyExistException ) {

      CDMW_ERROR(CCMLoggers::Get_ccmcif_logger(),
           "CCMHome_impl create_component_ref: config values exist "
                  << "already!");
      CORBA::INTERNAL ex(OrbSupport::INTERNALCCMCIFError,
                            CORBA::COMPLETED_NO);
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw ex;
    }
    
    register_component(config, result.in());
    
    // register component for recording (if Recording ConfigValue set to true)
    // NOTA: Failing to record the component for recording will only log
    //       a WARN message and not abort deployment of component.
    try {
       m_context->register_for_recording(comp_oid, result.in());
    }
    catch (const Components::CreateFailure& e)
    {
       std::string comp_identification(comp_oid);
       if (!s_component_name.empty())
          comp_identification = s_component_name;
       CDMW_ERROR(CCMLoggers::Get_ccmcif_logger(),
                  "Error registering component '" << comp_identification << 
                  "' for recording: " <<
                  (e.reason == CdmwDeployment::WRONG_RECORDING_CONFIG_VALUES ?
                   "Wrong Recording ConfigValue" : "Recording Daemon error") );
       CDMW_WARN(CCMLoggers::Get_ccmcif_logger(),
                 "Incoming requets to component '" << comp_identification << 
                 "' will not be recorded!");
    }
    
    return result._retn();
}


::PortableGroup::ObjectGroup_ptr
LBCCMHome_impl::get_group_for_name(const char* name)
    throw(Components::CreateFailure, 
          CORBA::SystemException)
{
    CDMW_LOG_FUNCTION(CCMLoggers::Get_ccmcif_logger(), "");
    try {
        CosNaming::Name_var cos_name = 
            Cdmw::CommonSvcs::Naming::NamingInterface::to_name(name);
    
        return m_lb_grp_mgr->get_object_group_ref_from_name(cos_name.in());
            
    } catch (const Cdmw::CommonSvcs::Naming::InvalidNameException& e) {
        CDMW_ERROR(CCMLoggers::Get_ccmcif_logger(),
              "Wrong LB Configuration values for LB component creation: "
                    << name << " is not a valid group name!");
        Components::CreateFailure ex(CdmwDeployment::WRONG_LB_CONFIG_VALUES);
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw ex;
    } catch (const PortableGroup::ObjectGroupNotFound& e) {
        // Group was found in get_group_for_name, but no more exists ?
        // It could have been removed in the meanwhile...
        CDMW_ERROR(CCMLoggers::Get_ccmcif_logger(),
              "Wrong LB Configuration values for LB component creation: "
                    << name << " group not found!");
        Components::CreateFailure ex(CdmwDeployment::WRONG_LB_CONFIG_VALUES);
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw ex;
    }
}


::PortableGroup::Location* LBCCMHome_impl::get_current_lb_location()
{
    ::PortableGroup::Location* loc = new ::PortableGroup::Location();
    loc->length(3);
    std::string hostname = Cdmw::OsSupport::OS::get_hostname();
    (*loc)[0].id = hostname.c_str();
    (*loc)[0].kind = "hostname";
    (*loc)[1].id = Cdmw::PlatformMngt::PlatformInterface::Get_application_name();
    (*loc)[1].kind = "applicationname";
    (*loc)[2].id = Cdmw::PlatformMngt::PlatformInterface::Get_process_name();
    (*loc)[2].kind = "processname";

    return loc;
}


} // End of namespace CIF

} // End of namespace CCM

} // End of namespace Cdmw

