/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO 
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2009. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


package cdmw.ccm.cif;

import cdmw.common.Assert;
import cdmw.orbsupport.ExceptionMinorCodes;
import cdmw.lifecycle.LifeCycleSingleObject;
import cdmw.ccm.common.DisposableObject;
import cdmw.ccm.common.CCMLoggers;
import static cdmw.logging.Logger.*;
import static cdmw.logging.FunctionLogger.*;
import java.lang.Throwable;
import com.thalesgroup.CdmwDeployment.WRONG_RECORDING_CONFIG_VALUES;


//
// IDL:thalesgroup.com/CdmwCcmCif/CdmwCCMHome:1.0
//
public abstract class CCMHomeImpl
    extends com.thalesgroup.CdmwCcmCif.CCMHomePOA
    implements 
        cdmw.lifecycle.ServantProviderBaseInterface,
        cdmw.ccm.container.ComponentServantProviderInterface, 
        cdmw.lifecycle.LifeCycleObjectBaseInterface,
        cdmw.ccm.common.DisposableObject {

    /**
     * Component map with key = stringified oid.
     * 
     */
    protected class ComponentInfo {
        
        /**
         * component reference
         */
        public org.omg.CORBA.Object compRef = null;
        
        /**
         * component servant
         */
        public org.omg.PortableServer.Servant compServant = null;
        
        /**
         * component executor
         */
        public org.omg.Components.EnterpriseComponent compExec = null;
        
        public ComponentInfo() {}
        
    }

    /**
     * This inner class is used to delegate protected methods
     * from LifeCycleSingleObject
     * 
     */
    protected class LifeCycleSingleObjectDelegate 
        extends LifeCycleSingleObject {

        public LifeCycleSingleObjectDelegate(
            cdmw.lifecycle.ObjectDeactivator deactivator) {
            super(deactivator);
        }

        public org.omg.PortableServer.POA _getPOA() 
            throws org.omg.PortableServer.CurrentPackage.NoContext {
            return this.getPOA();
        }
        
        public byte[] _getObjectId() 
            throws org.omg.PortableServer.CurrentPackage.NoContext {
            return this.getObjectId();
        }
        
        public void _deactivateObject() 
            throws org.omg.PortableServer.CurrentPackage.NoContext {
            this.deactivateObject();
        }
    
        public void _registerObjectAsRemoved(byte[] oid) {
            this.registerObjectAsRemoved(oid);
        }

    }
    
    /**
     * This inner class is used to delegate protected methods
     * from ServantProviderBase
     * 
     */
    protected class ServantProviderBaseDelegate
        extends cdmw.lifecycle.ServantProviderBase {
            
        public ServantProviderBaseDelegate() {}
        
        public void _cleanupObject(byte[] oid) {
            super.cleanupObject(oid);
        }
    
        // dummy method
        protected String[] getIds() { return null;}
    
        // dummy method
        protected int getNumberOfIds() { return 0;}
        
        // dummy method
        public boolean mayIncarnateMultipleObjects() { return false;}
        
        // dummy method
        public org.omg.PortableServer.Servant lookupServant(byte[] oid)
            throws org.omg.PortableServer.ForwardRequest { return null;}
        
        // dummy method
        public void releaseServant(
            byte[] oid,
            org.omg.PortableServer.POA poa,
            org.omg.PortableServer.Servant servant) {}

        // dummy method
        public String getSupportedInterface(byte[] oid)
            throws org.omg.CORBA.OBJECT_NOT_EXIST { return null;}

    }
    
    /**
     * home Context
     */
    protected Context context;
    
    // tmpContext to be removed after merge with PT delivery
    protected Context tmpContext;
    
    /**
     * home id
     */
    protected byte[] oid;
    
    /**
     * home repository id
     */
    protected String repId;
    
    /**
     * component rep id
     */
    protected String compRepId;
    
    /**
     * list of created component
     */
    protected java.util.Hashtable components = new java.util.Hashtable();
    
    /**
     * this attribute is used to delegate LifeCycleSingleObject methods
     */
    protected LifeCycleSingleObjectDelegate lifecycleDelegate;
    
    /**
     * this attribute is used to delegate ServantProviderBase methods
     */
    protected ServantProviderBaseDelegate spbDelegate;

    /** 
     * Used to register CCMObject ref in a local NamingContext.
     */
    protected com.thalesgroup.CdmwLifeCycle.ObjectRegistration objectRegistration; 
    
    /**
     * Constructor.
     * By default,  no config values are expected, and the constructor
     * will raise a CORBA::BAD_PARAM exception if any
     * config value is present.
     * 
     * @param repId repId home repository ID
     * @param compRepId
     * @param deactivator
     * @param objectRegistration
     * @param config
     *
     * @exception Any CORBA Exception.
     */
    public CCMHomeImpl(String repId, 
                       String compRepId, 
                       cdmw.lifecycle.ObjectDeactivator deactivator,
                       com.thalesgroup.CdmwLifeCycle.ObjectRegistration objectRegistration,
                       org.omg.Components.ConfigValue[] config)
    {
        this(repId, compRepId, deactivator, objectRegistration, config, false);
    }
    
    
   /**
    * Constructor.
    *
    * @param rep_id          The home repository ID
    * @param comp_rep_id     The component repository ID
    * @param deactivator     The Object Deactivator responsible to 
    *                       deactivate home at its removal.
    * @param config          The config values used to specify if
    *                       the home is FT or not
    * @param objectRegistration Used to register CCMObject ref in a naming ctx
    * @param configExpected If some config values are expected.
    *                       If no config values are expected, and the constructor
    *                       will raise a CORBA::BAD_PARAM exception if any
    *                       config value is present.
    *                    
    * @exception Any CORBA Exception.
    */ 
    public CCMHomeImpl(String repId, 
                       String compRepId, 
                       cdmw.lifecycle.ObjectDeactivator deactivator,
                       com.thalesgroup.CdmwLifeCycle.ObjectRegistration objectRegistration,
                       org.omg.Components.ConfigValue[] config,
                       boolean configExpected) 
    {
        CDMW_LOG_FUNCTION(CCMLoggers.GetCcmcifLogger(), 
            "repId="+repId+"  compRepId"+compRepId);
        this.context = null;
        this.lifecycleDelegate = 
            new LifeCycleSingleObjectDelegate(deactivator);
        this.spbDelegate = new ServantProviderBaseDelegate();
        try {
            this.oid = deactivator.getObjectId();
        } catch (org.omg.PortableServer.CurrentPackage.NoContext nc)
        {
            CDMW_ERROR(CCMLoggers.GetCcmcifLogger(), 
                "System Exception raised in the CCMHome constructor",
                new Throwable());
            nc.printStackTrace();
        }
        this.repId = repId;
        this.compRepId = compRepId;
        this.objectRegistration = objectRegistration;
        
        if ( (!configExpected) && (config.length > 0) )
        {
            // this class is not inherited by an other
            CDMW_DEBUG(CCMLoggers.GetCcmcifLogger(), 
                "CCMHome_impl is not inherited. No FT ConfigValues expected.",
                new Throwable());
            org.omg.CORBA.Any ft_rep_style_any =  
                cdmw.ccm.common.CCMUtils.ReadConfigValue(
                    com.thalesgroup.CdmwDeployment.FAULT_TOLERANCE_REPLICATION_STYLE.value,
                    config);

            if (ft_rep_style_any != null)
            {
                CDMW_ERROR(CCMLoggers.GetCcmcifLogger(), 
                    "Unexpected FT Configuration values for non FT home creation",
                    new Throwable());
                org.omg.CORBA.BAD_PARAM ex = new org.omg.CORBA.BAD_PARAM(
                    ExceptionMinorCodes.BAD_PARAMInvalidConfigValues,
                                      org.omg.CORBA.CompletionStatus.COMPLETED_NO);
                CDMW_LOG_FUNCTION_EXCEPTION(CCMLoggers.GetCcmcifLogger(), ex);
                throw ex;
            }
            else
            {
                CDMW_ERROR(CCMLoggers.GetCcmcifLogger(),
                    "Unexpected Configuration values for a standard home creation",
                    new Throwable());
                org.omg.CORBA.BAD_PARAM ex = new org.omg.CORBA.BAD_PARAM(
                    ExceptionMinorCodes.BAD_PARAMInvalidConfigValues,
                                      org.omg.CORBA.CompletionStatus.COMPLETED_NO);
                CDMW_LOG_FUNCTION_EXCEPTION(CCMLoggers.GetCcmcifLogger(), ex);
                throw ex;
            }
        }
    }
    
    /**
     * Destructor
     */
    public void dispose() 
        throws org.omg.Components.RemoveFailure {
        CDMW_LOG_FUNCTION(CCMLoggers.GetCcmcifLogger(), "");
        
        this.context = null;
        // do not clean component list while this will be done in release_servant
        //this.components.clear();
    }
    
    /**
     * Returns an object reference that supports 
     * the CORBA.ComponentIR.ComponentDef interface, 
     * describing the component type associated with the home object.
     * 
     * @return CORBA.ComponentIR.ComponentDef describing 
     * the component type associated with the home object.
     */
    //
    // IDL:omg.org/Components/CCMHome/get_component_def:1.0
    //
    //public org.omg.CORBA.IRObject get_component_def() {
    public org.omg.Components.IRObject get_component_def() {
        CDMW_LOG_FUNCTION(CCMLoggers.GetCcmcifLogger(), "");
        checkIsRemoved(oid);
        Assert.check(context != null);

        // TODO: Implementation
        throw new org.omg.CORBA.NO_IMPLEMENT(
            ExceptionMinorCodes.NO_IMPLEMENT,
            org.omg.CORBA.CompletionStatus.COMPLETED_NO);
    }

    /**
     * This returns an object reference that supports 
     * the CORBA.ComponentIR.HomeDef interface describing the home type.
     * 
     * @return CORBA.ComponentIR.HomeDef interface describing the home type.
     */
    //
    // IDL:omg.org/Components/CCMHome/get_home_def:1.0
    //
    //public org.omg.CORBA.IRObject get_home_def() {
    public org.omg.Components.IRObject get_home_def() {
        CDMW_LOG_FUNCTION(CCMLoggers.GetCcmcifLogger(), "");
        checkIsRemoved(oid);
        Assert.check(context != null);

        // TODO: Implementation
        throw new org.omg.CORBA.NO_IMPLEMENT(
            ExceptionMinorCodes.NO_IMPLEMENT,
            org.omg.CORBA.CompletionStatus.COMPLETED_NO);
    }


    /**
     * This causes the component denoted by the reference to cease to exist.
     * Subsequent invocations on the reference will cause an OBJECT_NOT_EXIST
     * system exception to be raised.
     * 
     * @param comp
     * @exception org.omg.CORBA.BAD_PARAM system exception raised 
     * if the component denoted by the parameter does not exist 
     * in the container associated with target home object.
     * @exception org.omg.Components.RemoveFailure exception raised 
     * if application errors.
     */
    //
    // IDL:omg.org/Components/CCMHome/remove_component:1.0
    //
    public void remove_component(org.omg.Components.CCMObject comp) 
        throws org.omg.Components.RemoveFailure 
    {
        CDMW_LOG_FUNCTION(CCMLoggers.GetCcmcifLogger(), "");

        checkIsRemoved(oid);
        Assert.check(this.context != null);

        if (comp == null) {
            CDMW_ERROR(CCMLoggers.GetCcmcifLogger(), 
                "Cannot remove nil component.",
                new Throwable());
            org.omg.CORBA.BAD_PARAM ex = new org.omg.CORBA.BAD_PARAM(
                ExceptionMinorCodes.BAD_PARAMNilObjectReference, 
                org.omg.CORBA.CompletionStatus.COMPLETED_NO);
            CDMW_LOG_FUNCTION_EXCEPTION(CCMLoggers.GetCcmcifLogger(), ex);
            throw ex;
        }
        
        try 
        {
            org.omg.Components.CCMObject theComp = comp;
            String compOid = "";
            
            try
            {
                compOid = context.get_oid_str_from_ref(comp);
                CDMW_DEBUG(CCMLoggers.GetCcmcifLogger(), "The component is found: compOid=" + compOid);
            } catch (org.omg.Components.BadComponentReference e) {
                // PCR-1522 : If Component ref is not found via get_oid_str_from_ref(),
                // try to retrieve it via '_get_component()' operation in case of 
                // a redirect reference to a local component.
                try {
                    org.omg.CORBA.Object target = comp._get_component();
                    theComp = org.omg.Components.CCMObjectHelper.narrow(target);
                    Assert.check(theComp != null);
                    compOid = context.get_oid_str_from_ref(theComp);
                    CDMW_DEBUG(CCMLoggers.GetCcmcifLogger(), "The target component is found: compOid=" + compOid);
                } catch (org.omg.Components.BadComponentReference ex) {
                    // Definitely not from here!
                    CDMW_ERROR(CCMLoggers.GetCcmcifLogger(),
                        "Components::BadComponentReference Exception raised when removing component!");
                }
            }

            if (components.containsKey(compOid))
            {
                try 
                {
                    // ask to the component its removal
                    theComp.remove();
                }
                catch (org.omg.Components.RemoveFailure e)
                {
                    CDMW_ERROR(CCMLoggers.GetCcmcifLogger(), 
                        "Exception raised when removing component: "+e,
                        new Throwable());
                    CDMW_LOG_FUNCTION_EXCEPTION(CCMLoggers.GetCcmcifLogger(), e);
                    throw e;
                } 
                catch (org.omg.CORBA.OBJECT_NOT_EXIST e)
                {
                    CDMW_ERROR(CCMLoggers.GetCcmcifLogger(), 
                        "This component is being removed!",
                        new Throwable());
                    org.omg.Components.RemoveFailure ex = 
                        new org.omg.Components.RemoveFailure(
                            com.thalesgroup.CdmwDeployment.BAD_COMPONENT_REFERENCE.value);
                    CDMW_LOG_FUNCTION_EXCEPTION(CCMLoggers.GetCcmcifLogger(), ex);
                    throw ex;
                } 
                catch (org.omg.CORBA.SystemException e)
                {
                    CDMW_ERROR(CCMLoggers.GetCcmcifLogger(), 
                        "System exception raised when removing component: "+e,
                        new Throwable());
                    CDMW_LOG_FUNCTION_EXCEPTION(CCMLoggers.GetCcmcifLogger(), e);
                    throw e;
                }
            }
            else 
            {
                // this component has not been created by this home
                CDMW_ERROR(CCMLoggers.GetCcmcifLogger(), 
                    "This component has not been created by this home or has already been removed! compOid="
                     + compOid);
                org.omg.Components.RemoveFailure ex =
                    new org.omg.Components.RemoveFailure(
                        com.thalesgroup.CdmwDeployment.BAD_COMPONENT_REFERENCE.value);
                CDMW_LOG_FUNCTION_EXCEPTION(CCMLoggers.GetCcmcifLogger(), ex);
                throw ex;
            }
        }  
        catch (org.omg.Components.IllegalState e)
        {
            CDMW_ERROR(CCMLoggers.GetCcmcifLogger(), 
                "IllegalState exception raised!",
                new Throwable());
            org.omg.Components.RemoveFailure ex =
                new org.omg.Components.RemoveFailure(
                    com.thalesgroup.CdmwDeployment.ILLEGAL_STATE.value);
            CDMW_LOG_FUNCTION_EXCEPTION(CCMLoggers.GetCcmcifLogger(), ex);
            throw ex;
        }
        catch (org.omg.CORBA.SystemException e)
        {
            CDMW_ERROR(CCMLoggers.GetCcmcifLogger(), 
                "System exception raised when removing component: "+e,
                new Throwable()); 
            CDMW_LOG_FUNCTION_EXCEPTION(CCMLoggers.GetCcmcifLogger(), e);
            throw e;
        }
    
    }

    /**
     * Remove instructs the object to cease to exist. The object reference 
     * for the target is no longer valid after remove successfully completes.
     * <B>NOTA:</B> Any further call to <I>Remove</I> for this object will raise
     * a org.omg.CORBA.OBJECT_NOT_EXIST system exception.
     *
     * @exception org.omg.CosLifeCycle.NotRemovable An implementation 
     * that refuses to remove itself should raise this exception
     */ 
    //
    // IDL:omg.org/CosLifeCycle/LifeCycleObject/remove:1.0
    //
    public void remove() throws org.omg.CosLifeCycle.NotRemovable {
        
        CDMW_LOG_FUNCTION(CCMLoggers.GetCcmcifLogger(), "");
        checkIsRemoved(oid);
        Assert.check(this.context != null);

        java.util.Enumeration e = this.components.elements();
        while(e.hasMoreElements()) {
            ComponentInfo compInfo = (ComponentInfo) e.nextElement();
            try {
                org.omg.Components.CCMObject ccmObj = 
                    org.omg.Components.CCMObjectHelper.narrow(compInfo.compRef);
                ccmObj.remove();
            } catch (org.omg.Components.RemoveFailure rf) {
                // Ignore
                CDMW_WARN(CCMLoggers.GetCcmcifLogger(), 
                    "CCMHomeImpl remove: error when trying to remove a component !",
                    new Throwable());
            } catch (org.omg.CORBA.OBJECT_NOT_EXIST ex) {
                // Ignore
                CDMW_WARN(CCMLoggers.GetCcmcifLogger(), 
                    "CCMHome_impl remove: component is being removed! Ignore it",
                    new Throwable());
            } catch (org.omg.CORBA.SystemException ex) {
                // Ignore
                CDMW_WARN(CCMLoggers.GetCcmcifLogger(), 
                    "CCMHome_impl remove: System exception raised when removing component! Ignore it",
                    new Throwable());
            }
        }

        // do not clean component list while this will be done in release_servant
    
        this.lifecycleDelegate.remove();
    
        // release context
        this.context = null;
    
    }

    /**
     * This operation returns whether the CDMW factory supports the 
     * creation of objects supporting the specified repository Id.
     *
     * @param repositoryId The Repository Id of the desired interface.
     * @return <B>true</B> if the factory supports/creates objects of 
     * <I>repositoryId</I> type or of derived types. Otherwise it returns
     * <B>false</B>.
     */
    //
    // IDL:thalesgroup.com/CdmwLifeCycle/FactoryBase/supports:1.0
    //
    public boolean supports(String repositoryId) {
        CDMW_LOG_FUNCTION(CCMLoggers.GetCcmcifLogger(), 
            "repositoryId="+repositoryId);
        checkIsRemoved(oid);
        Assert.check(this.context != null);
        boolean result = this.spbDelegate.supports(repositoryId);
        CDMW_LOG_FUNCTION_RETURN(CCMLoggers.GetCcmcifLogger(), result); 
        return result;
    }

    /**
     * This operation returns a sequence of CORBA Repository IDs of CDMW 
     * objects that can be created by the factory.
     *
     * @return A sequence of CORBA Repository IDs of CDMW objects that 
     * can be created by the factory.
     */
    //
    // IDL:thalesgroup.com/CdmwLifeCycle/FactoryBase/get_supported_objects:1.0
    //
    public String[] get_supported_objects() {
        CDMW_LOG_FUNCTION(CCMLoggers.GetCcmcifLogger(), "");
        checkIsRemoved(oid);
        Assert.check(this.context != null);
        return this.spbDelegate.getSupportedObjects();
    }
    
    /**
     * This operation returns a sequence of CORBA Repository IDs of CDMW 
     * objects that can be created by the factory.
     *
     * @return A sequence of CORBA Repository IDs of CDMW objects that 
     * can be created by the factory.
     */
    public String[] getSupportedObjects() {
        CDMW_LOG_FUNCTION(CCMLoggers.GetCcmcifLogger(), "");
        return get_supported_objects();
    }
    
    /**
     * This method returns the RepositoryID of the object associated 
     * to the <I>oid</I>.
     * 
     * @param oid ObjectId of the object
     * @return The RepositoryId of the object
     * @exception org.omg.CORBA.OBJECT_NOT_EXIST 
     * The object does not exist any more.
     */
    public String getSupportedInterface(byte[] oid) 
        throws org.omg.CORBA.OBJECT_NOT_EXIST 
    {
        CDMW_LOG_FUNCTION(CCMLoggers.GetCcmcifLogger(), "");
        checkIsRemoved(oid);
        Assert.check(this.context != null);

        try {
            String name = cdmw.ccm.common.CCMUtils.OidToString(oid);
            ComponentInfo compInfo = 
                (ComponentInfo) this.components.get(name);
        
            if (compInfo == null) {
                CDMW_ERROR(CCMLoggers.GetCcmcifLogger(), 
                    "No ComponentInfo for "+name,
                    new Throwable());
                org.omg.CORBA.OBJECT_NOT_EXIST ex =
                    new org.omg.CORBA.OBJECT_NOT_EXIST(
                        ExceptionMinorCodes.OBJECT_NOT_EXIST,
                        org.omg.CORBA.CompletionStatus.COMPLETED_NO);
                CDMW_LOG_FUNCTION_EXCEPTION(CCMLoggers.GetCcmcifLogger(), ex);
                throw ex;
            }
            else {
                return this.compRepId;
            }
        } catch (org.omg.CORBA.BAD_PARAM bp) {
            // Pb with ObjectId
            CDMW_ERROR(CCMLoggers.GetCcmcifLogger(), 
                "ObjectID is not valid ",
                new Throwable());
            org.omg.CORBA.OBJECT_NOT_EXIST ex = 
                new org.omg.CORBA.OBJECT_NOT_EXIST(
                    ExceptionMinorCodes.OBJECT_NOT_EXIST,
                    org.omg.CORBA.CompletionStatus.COMPLETED_NO);
            CDMW_LOG_FUNCTION_EXCEPTION(CCMLoggers.GetCcmcifLogger(), ex);
            throw ex;
        }
    }
       
    /**
     * This method provides a servant for the CORBA object with
     * the specified ObjectId. It is called by the Container
     * whenever a servant is needed to incarnate an object.
     * 
     * @param oid ObjectId of the object to be incarnated
     * @return A servant incarnating the object 
     * @exception org.omg.PortableServer.ForwardRequest Indicates 
     * to the Life Cycle Framework ORB that the SevantProvider cannot handle 
     * this object and requests can be handled to the returned IOR. 
     * If this exception is translated/forwarded to the client-side ORB, 
     * the request invocation will be transparently re-transmitted 
     * to the returned object.
     */
    public org.omg.PortableServer.Servant 
        lookupServant(byte[] oid) 
        throws org.omg.PortableServer.ForwardRequest 
    {
        CDMW_LOG_FUNCTION(CCMLoggers.GetCcmcifLogger(), "");
        checkIsRemoved(oid);
        Assert.check(this.context != null);
    
        CDMW_DEBUG(CCMLoggers.GetCcmcifLogger(), 
            "Entering in lookupServant...",
            new Throwable());
    
        org.omg.PortableServer.Servant servant = null;
    
        try {
            // check if it is a component oid or a facet/consumer oid or 
            // a local push consumer oid.
            String objectId = cdmw.ccm.common.CCMUtils.OidToString(oid);
            java.util.Vector names = CCMUtils.readOid(objectId);
    
            String compOid = (String) names.elementAt(0);
            CDMW_DEBUG(CCMLoggers.GetCcmcifLogger(), 
                "oid = " + objectId + "  compOid = " + compOid,
                new Throwable());
            
            // check a component with this oid has been created in this home
            ComponentInfo compInfo = 
                (ComponentInfo) this.components.get(compOid);
            
            if (compInfo == null) {
                CDMW_ERROR(CCMLoggers.GetCcmcifLogger(), 
                    "Cannot find component with oid: " + compOid,
                    new Throwable());
                org.omg.CORBA.OBJECT_NOT_EXIST ex = new org.omg.CORBA.OBJECT_NOT_EXIST(
                    ExceptionMinorCodes.OBJECT_NOT_EXIST,
                    org.omg.CORBA.CompletionStatus.COMPLETED_NO);
                CDMW_LOG_FUNCTION_EXCEPTION(CCMLoggers.GetCcmcifLogger(), ex);
                throw ex;
            }

            if (compInfo.compServant != null) {
                CCMObjectImpl _comp = getCCMObjectImpl(compInfo.compServant);

                if (_comp.is_removed()) {
                    // because the call to release servant comes from POA,
                    // it may occurs that the object is deactivated but
                    // release_servant not called and so the component has
                    // not been deleted from the map.
                    CDMW_ERROR(CCMLoggers.GetCcmcifLogger(), 
                        "Component found but has been removed!",
                        new Throwable());
                    org.omg.CORBA.OBJECT_NOT_EXIST ex =
                        new org.omg.CORBA.OBJECT_NOT_EXIST(
                            ExceptionMinorCodes.OBJECT_NOT_EXIST,
                            org.omg.CORBA.CompletionStatus.COMPLETED_NO);
                    CDMW_LOG_FUNCTION_EXCEPTION(CCMLoggers.GetCcmcifLogger(), ex);
                    throw ex;
                }
            }
            switch (names.size()) {
            case 1: // oid is a component oid
                CDMW_DEBUG(CCMLoggers.GetCcmcifLogger(), 
                    "oid= " + objectId + "  is a component",
                    new Throwable());
                // Now we can create the component servant and store servant
                // while facet servant are part of component servant
                compInfo.compServant = create_component_servant(
                    compOid, this.context, compInfo);
                CDMW_DEBUG(CCMLoggers.GetCcmcifLogger(), 
                    "component servant created",
                    new Throwable());
                Assert.check(compInfo.compServant != null);
                CDMW_DEBUG(CCMLoggers.GetCcmcifLogger(), 
                    "assert on component servant done",
                    new Throwable());
                
                // NOTA: comp executor is not null if an executor has been created at ref 
                // creation that is a factory operation has been used to create ref.
                
                servant = compInfo.compServant;
                break;
    
            case 2: // oid is a facet/consumer oid
                CDMW_DEBUG(CCMLoggers.GetCcmcifLogger(), 
                    "oid= " + objectId + "  is a facet or consumer",
                    new Throwable());
                String name = (String) names.elementAt(1);
                CDMW_DEBUG(CCMLoggers.GetCcmcifLogger(), 
                    "name = " + name,
                    new Throwable());

                Assert.check(compInfo.compServant != null);
                CCMObjectImpl comp = getCCMObjectImpl(compInfo.compServant);
                Assert.check(comp != null);
                
                servant = comp.get_facet_servant(name);
                break;
                
            case 3:  // oid is a local push consumer
                CDMW_DEBUG(CCMLoggers.GetCcmcifLogger(), 
                    "oid= " + objectId + "  is a local push consumer",
                    new Throwable());
                Assert.check(compInfo.compServant != null);
                comp = getCCMObjectImpl(compInfo.compServant);
                Assert.check(comp != null);
                
                name = (String) names.elementAt(1);
                String ckValue = (String) names.elementAt(2);
                CDMW_DEBUG(CCMLoggers.GetCcmcifLogger(), 
                    "name = " + name + "  ckValue = " + ckValue,
                    new Throwable());
                
                servant = comp.get_local_push_consumer_servant(name, ckValue);
                break;
    
            default:
                CDMW_ERROR(CCMLoggers.GetCcmcifLogger(), 
                    "oid " + objectId + " is not valid: " + names.size() + " names !",
                    new Throwable());
                org.omg.CORBA.INTERNAL ex = new org.omg.CORBA.INTERNAL(
                    ExceptionMinorCodes.INTERNAL,
                    org.omg.CORBA.CompletionStatus.COMPLETED_NO);
                CDMW_LOG_FUNCTION_EXCEPTION(CCMLoggers.GetCcmcifLogger(), ex);
                throw ex;
            }
            
        // Pb with ObjectId
        } catch (org.omg.CORBA.BAD_PARAM bp) {
            CDMW_ERROR(CCMLoggers.GetCcmcifLogger(), 
                "lookupServant: Pb with ObjectId!",
                new Throwable());
            org.omg.CORBA.OBJECT_NOT_EXIST ex = new org.omg.CORBA.OBJECT_NOT_EXIST(
                ExceptionMinorCodes.OBJECT_NOT_EXIST,
                org.omg.CORBA.CompletionStatus.COMPLETED_NO);
            CDMW_LOG_FUNCTION_EXCEPTION(CCMLoggers.GetCcmcifLogger(), ex);
            throw ex;
        } catch (org.omg.Components.CreateFailure cf) {
            CDMW_ERROR(CCMLoggers.GetCcmcifLogger(), 
                "lookupServant: Executor create failure!",
                new Throwable());
            org.omg.CORBA.OBJECT_NOT_EXIST ex = new org.omg.CORBA.OBJECT_NOT_EXIST(
                ExceptionMinorCodes.OBJECT_NOT_EXIST,
                org.omg.CORBA.CompletionStatus.COMPLETED_NO);
            CDMW_LOG_FUNCTION_EXCEPTION(CCMLoggers.GetCcmcifLogger(), ex);
            throw ex;
        } catch (org.omg.CORBA.OBJECT_NOT_EXIST one) {
            CDMW_ERROR(CCMLoggers.GetCcmcifLogger(), 
                "lookupServant: Object not exist!",
                new Throwable());
            CDMW_LOG_FUNCTION_EXCEPTION(CCMLoggers.GetCcmcifLogger(), one);
            throw one;
        }
    
        CDMW_DEBUG(CCMLoggers.GetCcmcifLogger(),
            "End of lookup_servant...",
            new Throwable());
        return servant;

    }
    
    /**
     * This method is called by Container whenever it is done with 
     * the specified servant. The ServantProvider is free to handle it at will.
     * 
     * @param oid ObjectId of the object incarnated by the servant
     * @param poa The POA where the object is activated
     * @param servant The servant in question
     */
    public void releaseServant(byte[] oid,
        org.omg.PortableServer.POA poa, 
                               org.omg.PortableServer.Servant servant) 
    {
        CDMW_LOG_FUNCTION(CCMLoggers.GetCcmcifLogger(), "");
        CDMW_DEBUG(CCMLoggers.GetCcmcifLogger(), 
            "Entering in releaseServant...",
            new Throwable());
    
        try {
            // check if it is a component oid or a facet oid
            String objectId = cdmw.ccm.common.CCMUtils.OidToString(oid);
            java.util.Vector names = CCMUtils.readOid(objectId);
    
            String compOid = (String) names.elementAt(0);
            CDMW_DEBUG(CCMLoggers.GetCcmcifLogger(), 
                "oid= " + objectId + "  compOid= " + compOid,
                new Throwable());
    
            switch (names.size()) {
            case 1: // oid is a component oid
                // check a component with this oid has been created in this home
                // do this only for component while the consumer and facet are released
                // after the component
                ComponentInfo compInfo = 
                    (ComponentInfo) this.components.get(compOid);
            
                if (compInfo  == null) {
                    CDMW_ERROR(CCMLoggers.GetCcmcifLogger(), 
                        "Cannot find component with oid: " + compOid,
                        new Throwable());
                    org.omg.CORBA.OBJECT_NOT_EXIST ex = 
                        new org.omg.CORBA.OBJECT_NOT_EXIST(
                            ExceptionMinorCodes.OBJECT_NOT_EXIST,
                            org.omg.CORBA.CompletionStatus.COMPLETED_NO);
                    CDMW_LOG_FUNCTION_EXCEPTION(CCMLoggers.GetCcmcifLogger(), ex);
                    throw ex;
                }
    
                CDMW_DEBUG(CCMLoggers.GetCcmcifLogger(), 
                    "oid= " + objectId + "  is a component",
                    new Throwable());
                // check if operation removed has been requested on component
                Assert.check(servant != null);
                CCMObjectImpl component = getCCMObjectImpl(servant);
                
                Assert.check(component != null);
                if (component.is_removed()) {
                    CDMW_DEBUG(CCMLoggers.GetCcmcifLogger(), 
                        "Component is removed, delete it from map",
                        new Throwable());
                    // if so remove component from the map
                    this.components.remove(compOid);            
                }
                // release servant, nothing to do in Java
                break;
    
            case 2: // oid is a facet/consumer oid
                CDMW_DEBUG(CCMLoggers.GetCcmcifLogger(), 
                    "oid= " + objectId + "  is a facet or consumer",
                    new Throwable());
                String name = (String) names.elementAt(1);
                CDMW_DEBUG(CCMLoggers.GetCcmcifLogger(), 
                    "name= " + name,
                    new Throwable());
                // release servant, nothing to do in Java
                break;
                
            case 3:  // oid is a local push consumer
                CDMW_DEBUG(CCMLoggers.GetCcmcifLogger(), 
                    "oid= " + objectId + "  is a local push consumer",
                    new Throwable());
                name = (String) names.elementAt(1);
                String ckValue = (String) names.elementAt(2);
                CDMW_DEBUG(CCMLoggers.GetCcmcifLogger(), 
                    "name= " + name + "  ckValue= " + ckValue,
                    new Throwable());
                // release servant, nothing to do in Java
                break;
    
            default:
                CDMW_ERROR(CCMLoggers.GetCcmcifLogger(), 
                    "oid " + objectId + " is not valid: " + names.size() + " names !",
                    new Throwable());
                org.omg.CORBA.INTERNAL ex = new org.omg.CORBA.INTERNAL(
                    ExceptionMinorCodes.INTERNAL,
                    org.omg.CORBA.CompletionStatus.COMPLETED_NO);
                CDMW_LOG_FUNCTION_EXCEPTION(CCMLoggers.GetCcmcifLogger(), ex);
                throw ex;
            }
        } catch (org.omg.CORBA.BAD_PARAM bp) {
            // Pb with ObjectId
            CDMW_ERROR(CCMLoggers.GetCcmcifLogger(), 
                "releaseServant: Pb with ObjectId!",
                new Throwable());
            org.omg.CORBA.OBJECT_NOT_EXIST ex = new org.omg.CORBA.OBJECT_NOT_EXIST(
                ExceptionMinorCodes.OBJECT_NOT_EXIST,
                org.omg.CORBA.CompletionStatus.COMPLETED_NO);
            CDMW_LOG_FUNCTION_EXCEPTION(CCMLoggers.GetCcmcifLogger(), ex);
            throw ex;
        }

    }

    /**
     * This method is called by Container after Home creation.
     * 
     * @param ctxt Component context given by container.
     * @exception org.omg.Components.CreateFailure Any Component exception
     */
    public void set_context(
        com.thalesgroup.CdmwCcmContainer.CCM2Context ctxt)
        throws org.omg.Components.CreateFailure 
    {
        CDMW_LOG_FUNCTION(CCMLoggers.GetCcmcifLogger(), "");
        checkIsRemoved(oid);
        
        try {
            // check component context is a Session2Context one
            com.thalesgroup.CdmwCcmContainer.Session2Context s2ctx =
                (com.thalesgroup.CdmwCcmContainer.Session2Context) ctxt;
                
            this.context = new SessionContext(s2ctx);
            
        } catch (ClassCastException e) {
            CDMW_ERROR(CCMLoggers.GetCcmcifLogger(), 
                "Context is not a Session2Context!",
                new Throwable());
            org.omg.Components.CreateFailure ex =
                new org.omg.Components.CreateFailure(
                    com.thalesgroup.CdmwDeployment.WRONG_CONTEXT_KIND.value);
            CDMW_LOG_FUNCTION_EXCEPTION(CCMLoggers.GetCcmcifLogger(), ex);
            throw ex;
        }
    }
            
    /**
     * This creates new component reference.
     * 
     * @return A reference on the new component.
     * @exception org.omg.Components.CreateFailure if application errors.
     */ 
    protected org.omg.CORBA.Object create_component_ref(
            org.omg.Components.ConfigValue[] config,
            org.omg.Components.EnterpriseComponent compExec)
        throws org.omg.Components.CreateFailure 
    {
        CDMW_LOG_FUNCTION(CCMLoggers.GetCcmcifLogger(), "");
        checkIsRemoved(oid);
        Assert.check(this.context != null);
        
        org.omg.CORBA.StringHolder comp_oid = 
            new org.omg.CORBA.StringHolder();

        org.omg.CORBA.Object to_return = 
            create_component_ref_i(comp_oid, compExec);

        // Setting realised to inject information about timeouts of receptacles.
        // That's the context of a home instance, which stores the config values.
        try {
            this.context.setConfigValues(comp_oid.value, config);
            CDMW_DEBUG(CCMLoggers.GetCcmcifLogger(), 
              "CCMHome_impl create_component_ref: length config values ="
              + config.length,
              new Throwable());
              
        } catch(cdmw.common.AlreadyExistException e) {
            CDMW_ERROR(CCMLoggers.GetCcmcifLogger(), 
              "CCMHome_impl create_component_ref: config values exist already!",
              new Throwable());
            org.omg.CORBA.INTERNAL ex = new org.omg.CORBA.INTERNAL(
                cdmw.orbsupport.ExceptionMinorCodes.INTERNALCCMCIFError,
                org.omg.CORBA.CompletionStatus.COMPLETED_NO);
            CDMW_LOG_FUNCTION_EXCEPTION(CCMLoggers.GetCcmcifLogger(), ex);
            throw ex;
        }

        register_component(config, to_return);
        
        // register component for recording (if Recording ConfigValue set to true)
        // NOTA: Failing to record the component for recording will only log
        //       a WARN message and not abort deployment of component.
        try {
           this.context.registerForRecording(comp_oid.value, to_return);
        }
        catch (org.omg.Components.CreateFailure e)
        {
           CDMW_ERROR(CCMLoggers.GetCcmcifLogger(), 
                      "Error registering component " + comp_oid + " for recording: " +
                      (e.reason == WRONG_RECORDING_CONFIG_VALUES.value ?
                       "Wrong Recording ConfigValue" : "Recording Daemon error"),
                      new Throwable());
           CDMW_WARN(CCMLoggers.GetCcmcifLogger(), 
                     "Incoming requets to component '" + comp_oid + 
                     "' will not be recorded!" ,
                     new Throwable());
        }

        return to_return;
    }

       
    /** 
     * This creates new component reference.
     *
     * @return A reference on the new component.
     * @exception org.omg.Components.CreateFailure if application errors.
     */
    protected org.omg.CORBA.Object create_component_ref(
            org.omg.Components.ConfigValue[] config) 
        throws org.omg.Components.CreateFailure 
    {
        CDMW_LOG_FUNCTION(CCMLoggers.GetCcmcifLogger(), "");
        return create_component_ref(config, null);
    }
                

    /**
     * Purpose:
     * <p>
     *   This creates new component reference. This is called by 
     *   create_component_ref
     *@param comp_oid Out parameter containing the oid of the new created 
     * component.
     *@return A reference on the new component.
     *@exception Components::CreateFailure if application errors.
     *@exception Any other CORBA Exception.
     *
     */
    protected org.omg.CORBA.Object create_component_ref_i(
            org.omg.CORBA.StringHolder comp_oid,
            org.omg.Components.EnterpriseComponent compExec)
        throws org.omg.Components.CreateFailure 
    {
        CDMW_LOG_FUNCTION(CCMLoggers.GetCcmcifLogger(), "");
        ComponentInfo compInfo = new ComponentInfo();

        try {
            compInfo.compRef = this.context.create_ref(this.compRepId);

            if (compInfo.compRef == null) {
                CDMW_ERROR(CCMLoggers.GetCcmcifLogger(), 
                    "Create_ref has failed!",
                    new Throwable());
                org.omg.CORBA.INTERNAL ex = new org.omg.CORBA.INTERNAL(
                    ExceptionMinorCodes.INTERNAL, 
                    org.omg.CORBA.CompletionStatus.COMPLETED_NO);
                CDMW_LOG_FUNCTION_EXCEPTION(CCMLoggers.GetCcmcifLogger(), ex);
                throw ex;
            }

            // set executor => executor is not nil if factory operation is used!
            compInfo.compExec = compExec;

            comp_oid.value = 
                this.context.get_oid_str_from_ref(compInfo.compRef);

            this.components.put(comp_oid.value, compInfo);

        }
        catch (org.omg.Components.IllegalState is) {
            CDMW_ERROR(CCMLoggers.GetCcmcifLogger(), 
                "IllegalState exception raised!",
                new Throwable());
            org.omg.Components.CreateFailure ex =
                new org.omg.Components.CreateFailure(
                    com.thalesgroup.CdmwDeployment.ILLEGAL_STATE.value);
            CDMW_LOG_FUNCTION_EXCEPTION(CCMLoggers.GetCcmcifLogger(), ex);
            throw ex;
        }
        catch (org.omg.Components.BadComponentReference bcr) {
            CDMW_ERROR(CCMLoggers.GetCcmcifLogger(), 
                "BadComponentReference exception raised!",
                new Throwable());
            org.omg.Components.CreateFailure ex =
                new org.omg.Components.CreateFailure(
                    com.thalesgroup.CdmwDeployment.BAD_COMPONENT_REFERENCE.value);
            CDMW_LOG_FUNCTION_EXCEPTION(CCMLoggers.GetCcmcifLogger(), ex);
            throw ex;
        }
        catch (org.omg.CORBA.SystemException se) {
            CDMW_ERROR(CCMLoggers.GetCcmcifLogger(), 
                "SystemException raised: " + se,
                new Throwable());
            CDMW_LOG_FUNCTION_EXCEPTION(CCMLoggers.GetCcmcifLogger(), se);
            throw se;
        }

        return compInfo.compRef;
     }
 
    /**
     * Purpose:
     * <p>
     *   This creates new component reference. This is called by 
     *   create_component_ref
     *@param comp_oid Out parameter containing the oid of the new created 
     * component.
     *@return A reference on the new component.
     *@exception Components::CreateFailure if application errors.
     *@exception Any other CORBA Exception.
     *
     */ 
    protected org.omg.CORBA.Object create_component_ref_i(
            org.omg.CORBA.StringHolder comp_oid)
        throws org.omg.Components.CreateFailure 
    {
        CDMW_LOG_FUNCTION(CCMLoggers.GetCcmcifLogger(), "");
        return create_component_ref_i(comp_oid, null);
    }
    
    /**
     * Purpose:
     * <p>
     *   Register the component in a naming context
     *@param config configuration values of the newly created component 
     *@param comp_oid component object id of the newly created component 
     *@param comp_ref component reference of the newly created component 
     *@return void
     *@exception Components::CreateFailure if wrong config values.
     *@exception Any other CORBA Exception.
     *
     */ 
    protected void register_component(
            org.omg.Components.ConfigValue[] config,
            org.omg.CORBA.Object componentRef)
        throws org.omg.Components.CreateFailure,
               org.omg.CORBA.SystemException
    {
        CDMW_LOG_FUNCTION(CCMLoggers.GetCcmcifLogger(), "");

        // register component in a local naming context
        if (this.objectRegistration != null) {
            java.lang.String componentName;
            try {
                org.omg.CORBA.Any componentNameAny =  
                    cdmw.ccm.common.CCMUtils.ReadConfigValue(
                        com.thalesgroup.CdmwDeployment.COMPONENT_NAME.value,
                        config);
                if (componentNameAny != null) {
                    componentName = componentNameAny.extract_string();
                    objectRegistration.register_object(componentRef,
                                                       componentName);
                    CDMW_DEBUG(CCMLoggers.GetCcmcifLogger(),
                              "CCMHome_impl register_component: name="
                              + componentName,
                              new Throwable());
                }
                else {
                    CDMW_WARN(CCMLoggers.GetCcmcifLogger(),
                              "CCMHome_impl register_component: "
                              + "No component name",
                              new Throwable());
                }
            }

            catch (org.omg.CosNaming.NamingContextPackage.InvalidName e) {
                CDMW_ERROR(CCMLoggers.GetCcmcifLogger(), 
                           "CCMHome_impl register_component: "
                           + "Invalid component name",
                           new Throwable());
                org.omg.Components.CreateFailure ex =
                    new org.omg.Components.CreateFailure
                    (com.thalesgroup.CdmwDeployment.WRONG_COMPONENT_NAME_CONFIG_VALUES.value);
                CDMW_LOG_FUNCTION_EXCEPTION(CCMLoggers.GetCcmcifLogger(), ex);
                throw ex;
            }
            catch (org.omg.CosNaming.NamingContextPackage.AlreadyBound e) {
                CDMW_ERROR(CCMLoggers.GetCcmcifLogger(),
                           "CCMHome_impl register_component: component name"
                           + " is already bound",
                           new Throwable());
                org.omg.Components.CreateFailure ex =
                    new org.omg.Components.CreateFailure
                    (com.thalesgroup.CdmwDeployment.WRONG_COMPONENT_NAME_CONFIG_VALUES.value);
                CDMW_LOG_FUNCTION_EXCEPTION(CCMLoggers.GetCcmcifLogger(), ex);
                throw ex;
            }
            catch (org.omg.CORBA.SystemException e) {
                CDMW_ERROR(CCMLoggers.GetCcmcifLogger(), 
                           "CCMHome_impl register_component: " + e,
                           new Throwable());
                org.omg.CORBA.INTERNAL ex = new org.omg.CORBA.INTERNAL(
                   cdmw.orbsupport.ExceptionMinorCodes.INTERNALCCMCIFError,
                   org.omg.CORBA.CompletionStatus.COMPLETED_NO);
                CDMW_LOG_FUNCTION_EXCEPTION(CCMLoggers.GetCcmcifLogger(), ex);
                throw ex;
            }
        }
    }

    /**
     * This creates new component servant.
     * This shall be implemented by the well typed home servant.
     *
     * @param compOid 
     * @param ctx The component context
     * @param executor The component executor
     * @return The new component servant.
     * @exception org.omg.Components.CreateFailure
     */ 
    protected abstract org.omg.PortableServer.Servant create_component_servant(
        String compOid,
        Context ctx,
        ComponentInfo compInfo)
        throws org.omg.Components.CreateFailure;

    protected abstract CCMObjectImpl 
        getCCMObjectImpl(org.omg.PortableServer.Servant servant);

    /**
     * the following methods delegate LifeCycleSingleObject methods
     * 
     */

    //
    // IDL:omg.org/CosLifeCycle/LifeCycleObject/copy:1.0
    //
    public org.omg.CosLifeCycle.LifeCycleObject copy(
        org.omg.CosLifeCycle.FactoryFinder there,
        org.omg.CosLifeCycle.NVP[] the_criteria)
        throws
            org.omg.CosLifeCycle.NoFactory,
            org.omg.CosLifeCycle.NotCopyable,
            org.omg.CosLifeCycle.InvalidCriteria,
            org.omg.CosLifeCycle.CannotMeetCriteria 
    {
        CDMW_LOG_FUNCTION(CCMLoggers.GetCcmcifLogger(), "");
        return this.lifecycleDelegate.copy(there, the_criteria);
    }
    
    //
    // IDL:omg.org/CosLifeCycle/LifeCycleObject/move:1.0
    //
    public void move(
        org.omg.CosLifeCycle.FactoryFinder there,
        org.omg.CosLifeCycle.NVP[] the_criteria)
        throws
            org.omg.CosLifeCycle.NoFactory,
            org.omg.CosLifeCycle.NotMovable,
            org.omg.CosLifeCycle.InvalidCriteria,
            org.omg.CosLifeCycle.CannotMeetCriteria 
    {
        CDMW_LOG_FUNCTION(CCMLoggers.GetCcmcifLogger(), "");
        this.lifecycleDelegate.move(there, the_criteria);
    }
    
    protected final org.omg.PortableServer.POA getPOA() 
        throws org.omg.PortableServer.CurrentPackage.NoContext 
    {
        CDMW_LOG_FUNCTION(CCMLoggers.GetCcmcifLogger(), "");
        return this.lifecycleDelegate._getPOA();
    }
    
    protected final byte[] getObjectId() 
        throws org.omg.PortableServer.CurrentPackage.NoContext 
    {
        CDMW_LOG_FUNCTION(CCMLoggers.GetCcmcifLogger(), "");
        return this.lifecycleDelegate._getObjectId();
    }
    
    protected final void deactivateObject() 
        throws org.omg.PortableServer.CurrentPackage.NoContext 
    {
        CDMW_LOG_FUNCTION(CCMLoggers.GetCcmcifLogger(), "");
        this.lifecycleDelegate._deactivateObject();
    }
    
    public boolean isRemoved(byte[] oid) 
    {
        CDMW_LOG_FUNCTION(CCMLoggers.GetCcmcifLogger(), "");
        return this.lifecycleDelegate.isRemoved(oid);
    }
    
    public void checkIsRemoved(byte[] oid) {
        if (this.lifecycleDelegate.isRemoved(oid)) {
            throw new org.omg.CORBA.OBJECT_NOT_EXIST(
                    ExceptionMinorCodes.OBJECT_NOT_EXIST,
                    org.omg.CORBA.CompletionStatus.COMPLETED_NO);
        }
    }

    protected void registerObjectAsRemoved(byte[] oid) 
    {
        CDMW_LOG_FUNCTION(CCMLoggers.GetCcmcifLogger(), "");
        this.lifecycleDelegate._registerObjectAsRemoved(oid);
    }
    
    /**
     * the following methods delegate ServantProviderBase methods
     * 
     */
    
    public void setCleanupNotifier(cdmw.lifecycle.CleanupNotifier notifier) 
    {
        CDMW_LOG_FUNCTION(CCMLoggers.GetCcmcifLogger(), "");
        this.spbDelegate.setCleanupNotifier(notifier);
    }
    
    protected void cleanupObject(byte[] oid) 
    {
        CDMW_LOG_FUNCTION(CCMLoggers.GetCcmcifLogger(), "");
        this.spbDelegate._cleanupObject(oid);
    }
    
    protected abstract String[] getIds();
    
    protected abstract int getNumberOfIds();
                                      
}

