/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO 
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2009. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


package cdmw.ccm.cif;

import cdmw.common.InternalErrorException;
import cdmw.common.AlreadyDoneException;
import cdmw.orbsupport.ExceptionMinorCodes;
import cdmw.ccm.common.DisposableObject;
import cdmw.ccm.common.CookieImpl;
import com.thalesgroup.CdmwDeployment.WRONG_RECORDING_CONFIG_VALUES;
import com.thalesgroup.CdmwDeployment.RECORDING_DAEMON_ERROR;
import com.thalesgroup.CdmwRecording.REC_CONTAINER_MNGR_ID;
import static cdmw.logging.Logger.*;
import static cdmw.logging.FunctionLogger.*;
import static cdmw.ccm.cif.DefaultCCMObjectStates.*;
import static cdmw.ccm.common.CCMSystemProperties.TYPECHECKING;
import static cdmw.ccm.common.CCMSystemProperties.TYPECHECKING_TIMEOUT;
import com.thalesgroup.CdmwCcmContainer.CCM2ContextPackage.InvalidName;

import cdmw.common.Assert;

import java.lang.System;
import java.lang.String;
import java.lang.Throwable;


/**
 * This class implements the org.omg.Components.CCMObject interface.
 * It will be inherited by all components servant implementation.
 *
 */
//
// IDL:omg.org/Components/CCMObject:1.0
//
public abstract class CCMObjectImpl 
    extends org.omg.Components.CCMObjectPOA
    implements cdmw.ccm.common.DisposableObject {
    
    public static final String PUSH_CONSUMER_ID = 
        "IDL:CosEventComm::PushConsumer:1.0";

    public static final String PSEUDO_EMITTER_CK_VALUE = "Emitter";
    
    ////////////////////////
    //                    //
    //    Attributes      //
    //                    //
    ////////////////////////

    /**
     * Component Context for CDMW component
     */
    protected Context context;

    /**
     * component rep id
     */
    protected String repId;

    /**
     * object Id of the current component
     */
    protected String componentOid;

    /**
     * state of the CCMObject
     */
    protected CCMObjectState state;
    
    /**
     * Component object reference.
     */

    protected org.omg.CORBA.Object compRef; 
    
    /**
     * Logger for logging
     */
    private com.thalesgroup.CdmwLogging.Logger logger;
    
    /** 
     * Used to (un)register CCMObject ref in a local NamingContext.
     */
    protected com.thalesgroup.CdmwLifeCycle.ObjectRegistration objectRegistration; 


    /**
     * Facet Map: find facet info by name
     */
    private final class FacetInfo {
        /**
         * facet repository id
         */
        public String repId;
        /**
         *  facet reference
         */
        public org.omg.CORBA.Object objRef;
    };

    private final class ReceptacleInfo {
        /**
         * receptacle repository id
         */
        public String repId;
        /**
         * true if receptacle is multiplex
         */
        public boolean isMultiplex;
        /**
         * connection info
         */
        public java.util.Hashtable connections = new java.util.Hashtable();
    };

    /**
     * Consumer Map
     */
    private final class ConsumerInfo {
        /**
         * consumer repository id
         */
        public String repId;
        /**
         * consumer reference
         */
        public org.omg.CORBA.Object consumer;
    };

    private final class EmitterInfo {
        /**
         * consumer repository id
         */
        public String repId;
        /**
         * event channel
         */
        public cdmw.event.EventChannelInterface channel;
        /**
         * proxy push consumer used to push event in event channel
         */
        public org.omg.CosEventComm.PushConsumer proxyConsumer;
        /**
         * proxy push supplier used to connect consumer
         */
        public org.omg.CosEventComm.PushSupplier proxySupplier;
        /**
         * component consumer reference
         */
        public org.omg.Components.EventConsumerBase ccmConsumer;
        /**
         * consumer reference implemented org.omg.CosEventComm.PushConsumer
         */
        public org.omg.CosEventComm.PushConsumer pushConsumer;
        /**
         * local consumer servant implementing org.omg.CosEventComm.PushConsumer
         */
        public org.omg.PortableServer.Servant pushConsumerServant;
    };

    private final class SubscriberInfo {
        /**
         * proxy push supplier used to connect consumer
         */
        public org.omg.CosEventComm.PushSupplier proxySupplier;
        /**
         * component consumer reference
         */
        public org.omg.Components.EventConsumerBase ccmConsumer;
        /**
         * consumer reference implementing org.omg.CosEventComm.PushConsumer
         */
        public org.omg.CosEventComm.PushConsumer pushConsumer;
        /**
         * local consumer servant implementing org.omg.CosEventComm.PushConsumer
         */
        public org.omg.PortableServer.Servant pushConsumerServant;
    };

    private final class PublisherInfo {
        /**
         * consumer repository id
         */
        public String repId;
        /**
         * event channel
         */
        public cdmw.event.EventChannelInterface channel;
        /**
         * proxy push consumer used to push event in event channel
         */
        public org.omg.CosEventComm.PushConsumer proxyConsumer;
        /**
         * connection info
         */
        public java.util.Hashtable subscriptions = new java.util.Hashtable();
    };

    /**
     *  map of component facet
     */
    private java.util.Hashtable facets = new java.util.Hashtable();

    /**
     * map of component receptacle
     */
    private java.util.Hashtable receptacles = new java.util.Hashtable();

    /**
     * map of component event sink
     */
    private java.util.Hashtable consumers = new java.util.Hashtable();

    /**
     * map of component source (emitter)
     */
    private java.util.Hashtable emitters = new java.util.Hashtable();

    /**
     * map of component source (publisher)
     */
    private java.util.Hashtable publishers = new java.util.Hashtable();
    
    /**
     * true if component has only one servant for it and its facets and consumers.
     */
    private boolean isMonolithic;

    /**
     * true if remove operation has been called
     */
    private boolean isRemoved = false;

    /**
     * True if the corresponding system property is set; default is false.
     */
    private boolean isTypeCheckingOn;

    /**
     * Set by the corresponding system property; default is 2500 ms.
     */
    protected long typecheckingTimeoutInMs;

    /**
     * Purpose:
     * <p>
     *   Constructor.
     *
     *@param compOid       The component oid
     *@param ctx            The component extended context
     *@param repId         The component repository ID
     *@param compRef       The component reference.
     *@param objectRegistration Used to (un)register CCMObject ref in a local
     * NamingContext. 
     *@param loggerFactory The LoggerFactory to be used for Logger creation
     *@param isMonolithic  True if the component, its facets and consumers 
     *                      have a single object reference.
     */
    public CCMObjectImpl(
        String compOid,
        Context ctx,
        String repId,
        org.omg.CORBA.Object compRef,
        com.thalesgroup.CdmwLifeCycle.ObjectRegistration objectRegistration,
        com.thalesgroup.CdmwLogging.LoggerFactory loggerFactory,
        boolean isMonolithic) 
    {
        // create logger with domain for ccmcif
        this.logger = loggerFactory.get_logger(
            cdmw.ccm.common.CCMLoggers.CCMCIF_LOG_DOMAIN);

        CDMW_LOG_FUNCTION(logger, 
            "compOid="+compOid+"  repId="+repId);

        cdmw.common.Assert.check(ctx != null);
        this.context = ctx;
        this.repId = repId;
        this.componentOid = compOid;
        this.state = CCM_OBJECT_STATE_INITIAL;
        this.compRef = compRef;
        this.objectRegistration = objectRegistration;
        this.isMonolithic = isMonolithic;

        // get the property TYPECHECKING
        String typeCheckingProperty = System.getProperty(TYPECHECKING, "off");
        if (typeCheckingProperty.equals("on"))
            this.isTypeCheckingOn = true;
        else
            this.isTypeCheckingOn = false;

        typecheckingTimeoutInMs = Long.parseLong(System.getProperty(TYPECHECKING_TIMEOUT, "2500"));
        CDMW_DEBUG(logger, "Typechecking timeout was set: "
                   +  typecheckingTimeoutInMs
                   + " ms", new Throwable());                 
    }

    /**
     * Purpose:
     * <p>
     *   Constructor with isMonolithic set to false.
     *
     *@param compOid       The component oid
     *@param ctx            The component extended context
     *@param repId         The component repository ID
     *@param compRef       The component reference.
     *@param objectRegistration Used to (un)register CCMObject ref in a local
     * NamingContext. 
     *@param loggerFactory The LoggerFactory to be used for Logger creation
     */
    public CCMObjectImpl(
        String compOid,
        Context ctx,
        String repId,
        org.omg.CORBA.Object compRef,
        com.thalesgroup.CdmwLifeCycle.ObjectRegistration objectRegistration,
        com.thalesgroup.CdmwLogging.LoggerFactory loggerFactory) 
    {
        this(compOid, ctx, repId, compRef, objectRegistration, loggerFactory, false);
        
        CDMW_LOG_FUNCTION(logger, "");
    }

    /**
     * Pupose:
     * <p>
     *   Destructor
     *
     */
    public void dispose() {
        CDMW_LOG_FUNCTION(logger, "");
    }

    public org.omg.CORBA.Object _get_component()
    {
        CDMW_LOG_FUNCTION(logger, "");
        return compRef._duplicate();
    }


    //
    // IDL:omg.org/Components/CCMObject/get_component_def:1.0
    //
    /**
     * Purpose:
     * <p>
     *   This operation returns an object reference that supports the 
     *   CORBA.ComponentIR.ComponentDef interface, describing the component 
     *   type associated with the home object.
     * 
     *@return CORBA.ComponentIR.ComponentDef described the component type 
     * associated with the home object.
     *@exception Any CORBA Exception.
     *
     */
    public org.omg.Components.IRObject get_component_def() {
        CDMW_LOG_FUNCTION(logger, "");
        checkIsRemoved();
        org.omg.Components.CCMHome home = this.context.get_CCM_home();
        return home.get_component_def();
    }

    //
    // IDL:omg.org/Components/CCMObject/get_ccm_home:1.0
    //
    public org.omg.Components.CCMHome get_ccm_home() {
        CDMW_LOG_FUNCTION(logger, "");
        checkIsRemoved();
        return this.context.get_CCM_home();
    }

    //
    // IDL:omg.org/Components/CCMObject/get_primary_key:1.0
    //
    public org.omg.Components.PrimaryKeyBase get_primary_key()
        throws org.omg.Components.NoKeyAvailable, org.omg.CORBA.SystemException 
    {
        CDMW_LOG_FUNCTION(logger, "");
        
        checkIsRemoved();

        // Check that there is an associated primary key.
        // If so, context is an entity context
        return this.context.get_primary_key();
        
    }

    //
    // IDL:omg.org/Components/CCMObject/configuration_complete:1.0
    //
    public void configuration_complete()
        throws org.omg.Components.InvalidConfiguration  
    {
        CDMW_LOG_FUNCTION(logger, "");
        checkIsRemoved();

        // Check receptacle are connected
        String receptacleName = checkConnections();

        if (receptacleName.length() > 0) {
            CDMW_ERROR(logger, 
                       "InvalidConfiguration: receptacle" + receptacleName +
                       " is not connected.",
                       new Throwable());
            org.omg.Components.InvalidConfiguration ex =
                new org.omg.Components.InvalidConfiguration();
            CDMW_LOG_FUNCTION_EXCEPTION(logger, ex);
            throw ex;
        }
    
    }

    /**
     * Purpose:
     * <p>
     *    Check that the receptacles are connected. 
     *
     *@return an empty string if all receptacles are connected else the name
     *        of the receptacle which is not connected.
     * 
     */
    public String checkConnections()
    {
        CDMW_LOG_FUNCTION(logger, "");

        // Check receptacle are connected
        java.util.Iterator i = this.receptacles.entrySet().iterator();
        
        while(i.hasNext()) {
            java.util.Map.Entry e = (java.util.Map.Entry) i.next();
            ReceptacleInfo rInfo = (ReceptacleInfo) e.getValue();
            
            if (rInfo.connections.size() == 0) {
                CDMW_ERROR(logger, 
                    "InvalidConfiguration: receptacle" + e.getKey().toString() +
                    " is not connected.",
                    new Throwable());
                return e.getKey().toString();
            }
        }

        return "";
    
    }

    /**
     * Purpose:
     * <p>
     *    Check that the receptacles are connected. 
     *
     *@return an empty string if all receptacles are connected else the name of the receptacle which is not connected.
     * 
     */
    public void checkValidityOfOperationalRequest()
    {
        CDMW_LOG_FUNCTION(logger, "");

        // Check that all receptacles are connected else raise a TRANSIENT
        // exception
        String receptacleName = checkConnections();
        
        if (receptacleName.length() > 0) {
            CDMW_ERROR(logger, 
                       "InvalidConfiguration: receptacle" + receptacleName +
                       " is not connected.",
                       new Throwable());
            org.omg.CORBA.TRANSIENT ex = new org.omg.CORBA.TRANSIENT(
                ExceptionMinorCodes.TRANSIENTMissingConnection, 
                org.omg.CORBA.CompletionStatus.COMPLETED_NO);
            CDMW_LOG_FUNCTION_EXCEPTION(logger, ex);
            throw ex;
        }

        this.state = this.state.operationalRequest(this);
        
    }

    //
    // IDL:omg.org/Components/CCMObject/remove:1.0
    //
    public void remove() 
       throws org.omg.Components.RemoveFailure, org.omg.CORBA.SystemException
    {
        CDMW_LOG_FUNCTION(logger, "");
        checkIsRemoved();
        
        // unregister component from recording service.
        unregister_from_recording();
        
        // unregister component from the local Naming Context.
        unregister_component();
    
        // advise executor of its remove
        CDMW_DEBUG(logger, 
            "advise_executor_of_remove_component...", 
            new Throwable());
        this.advise_executor_of_remove_component();

        if (!isMonolithic) {
            CDMW_DEBUG(logger, 
                "component not monolithic : remove facets",
                new Throwable());
            // remove all facets
            java.util.Enumeration e = this.facets.elements();

            while(e.hasMoreElements()) {
                FacetInfo fInfo = (FacetInfo) e.nextElement();

                org.omg.CosLifeCycle.LifeCycleObject facetBase 
                    = org.omg.CosLifeCycle.LifeCycleObjectHelper.narrow(fInfo.objRef);

                try {
                    facetBase.remove();
                } catch(org.omg.CosLifeCycle.NotRemovable nr) {
                    CDMW_ERROR(logger, 
                        "Error removing facet: " + nr, 
                        new Throwable());
                    org.omg.Components.RemoveFailure ex =
                        new org.omg.Components.RemoveFailure(
                        com.thalesgroup.CdmwDeployment.REMOVE_FAILURE_BASE.value);
                    CDMW_LOG_FUNCTION_EXCEPTION(logger, ex);
                    throw ex;
                }
            }
        }

        CDMW_DEBUG(logger, "clear facets...", new Throwable());
        this.facets.clear();

        if (!isMonolithic) {
            CDMW_DEBUG(logger, 
                "component not monolithic : remove consumers",
                new Throwable());
            // remove all consumers
            java.util.Enumeration e = this.consumers.elements();

            while (e.hasMoreElements()) {
                ConsumerInfo cInfo = (ConsumerInfo) e.nextElement();

                org.omg.CosLifeCycle.LifeCycleObject consumerBase 
                    = org.omg.CosLifeCycle.LifeCycleObjectHelper.narrow(cInfo.consumer);

                try {
                    consumerBase.remove();
                } catch(org.omg.CosLifeCycle.NotRemovable nr) {
                    CDMW_ERROR(logger, 
                        "Error removing consumer: " + nr, 
                        new Throwable());
                    org.omg.Components.RemoveFailure ex =
                        new org.omg.Components.RemoveFailure(
                        com.thalesgroup.CdmwDeployment.REMOVE_FAILURE_BASE.value);
                    CDMW_LOG_FUNCTION_EXCEPTION(logger, ex);
                    throw ex;
                }
            }
        }

        CDMW_DEBUG(logger, "clear consumers...", new Throwable());
        this.consumers.clear();

        // remove all publishers
        CDMW_DEBUG(logger, "remove all publisher...", new Throwable());
        java.util.Enumeration e = this.publishers.elements();

        while (e.hasMoreElements()) {
            PublisherInfo publisher = (PublisherInfo) e.nextElement();
            CDMW_DEBUG(logger, "remove all publisher... disconnect_push_consumer", new Throwable());
                
            if (publisher.proxyConsumer != null)
                publisher.proxyConsumer.disconnect_push_consumer();

            CDMW_DEBUG(logger, "remove all publisher... destroy channel", new Throwable());
            if (publisher.channel != null) {
                publisher.channel.destroy();
            }
        }

        // remove all emitters
        CDMW_DEBUG(logger, "remove all emitters...", new Throwable());
        e = this.emitters.elements();

        while (e.hasMoreElements()) {
            EmitterInfo emitter = (EmitterInfo) e.nextElement();

            if (emitter.proxyConsumer != null) {
                emitter.proxyConsumer.disconnect_push_consumer();
            }
        }

        try {
            // change state
            this.state = this.state.remove(this);
        }
        catch (org.omg.Components.CCMException ex) {
            CDMW_ERROR(logger,
                       "Error changing CCMObject state to removed:",
                       new Throwable());
            ex.printStackTrace();
            throw new org.omg.Components.RemoveFailure(
                com.thalesgroup.CdmwDeployment.REMOVE_FAILURE_BASE.value);
        }

        // register object as removed
        this.isRemoved = true;

        // deactivate servant
        CDMW_DEBUG(logger, "ask to the context req_passivate...",
                   new Throwable());
        try {
            this.context.req_passivate();
        }
        catch(org.omg.Components.PolicyMismatch pm) {
            CDMW_ERROR(logger, 
                "Error calling req_passivate: " + pm, 
                new Throwable());
            org.omg.Components.RemoveFailure ex =
                new org.omg.Components.RemoveFailure(
                    com.thalesgroup.CdmwDeployment.REMOVE_FAILURE_BASE.value);
            CDMW_LOG_FUNCTION_EXCEPTION(logger, ex);
            throw ex;
        }

    }

    //
    // IDL:omg.org/Components/CCMObject/get_all_ports:1.0
    //
    public org.omg.Components.ComponentPortDescription get_all_ports() {
        CDMW_LOG_FUNCTION(logger, "");
        checkIsRemoved();
        
        org.omg.Components.ComponentPortDescription componentPortDesc =
            new ComponentPortDescriptionImpl();
        
        componentPortDesc.facets = get_all_facets();
        componentPortDesc.receptacles = get_all_receptacles();
        componentPortDesc.consumers = get_all_consumers();
        componentPortDesc.emitters = get_all_emitters();
        componentPortDesc.publishers = get_all_publishers();
        
        return componentPortDesc;
    
    }

    //
    // IDL:omg.org/Components/Navigation/provide_facet:1.0
    //
    public org.omg.CORBA.Object provide_facet(String name)
        throws org.omg.Components.InvalidName  
    {
        CDMW_LOG_FUNCTION(logger, "name="+name);
        checkIsRemoved();

        if (name == null) {
            CDMW_ERROR(logger, "facet name is null", new Throwable());
            org.omg.Components.InvalidName ex = 
                new org.omg.Components.InvalidName();
            CDMW_LOG_FUNCTION_EXCEPTION(logger, ex);
            throw ex;
        }
    
        String[] nameList = new String[1];
        nameList[0] = name;
    
        org.omg.Components.FacetDescription[] fDesc = 
            get_named_facets(nameList);
        if (fDesc.length != 1) {
            if (fDesc.length == 0) {
                CDMW_ERROR(logger, 
                    "Facet " + name + " not found.",
                    new Throwable());
            } else {             
                CDMW_ERROR(logger, 
                    "Several facets with name " + name,
                    new Throwable());
            }
            org.omg.Components.InvalidName ex =
                new org.omg.Components.InvalidName();
            CDMW_LOG_FUNCTION_EXCEPTION(logger, ex);
            throw ex;
        }
        return fDesc[0].facet_ref;

    }

    //
    // IDL:omg.org/Components/Navigation/get_all_facets:1.0
    //
    public org.omg.Components.FacetDescription[] get_all_facets() {
        
        CDMW_LOG_FUNCTION(logger, "");
        checkIsRemoved();
        
        String[] nameList = new String[this.facets.size()];
        java.util.Set keyset = this.facets.keySet();
        java.util.Iterator iter = keyset.iterator();
        int i = 0;
    
        while (iter.hasNext()) {
            nameList[i] = (String) iter.next();
            i++;
        }
    
        try {
            return get_named_facets(nameList);
        } catch (org.omg.Components.InvalidName in) {
            CDMW_ERROR(logger, "InvalidName exception!", new Throwable());
            org.omg.CORBA.INTERNAL ex = new org.omg.CORBA.INTERNAL(
                ExceptionMinorCodes.INTERNAL, 
                org.omg.CORBA.CompletionStatus.COMPLETED_NO);
            CDMW_LOG_FUNCTION_EXCEPTION(logger, ex);
            throw ex;
        }
    }

    //
    // IDL:omg.org/Components/Navigation/get_named_facets:1.0
    //
    public org.omg.Components.FacetDescription[] get_named_facets(
        String[] names)
        throws org.omg.Components.InvalidName  
    {
        CDMW_LOG_FUNCTION(logger, "");
        checkIsRemoved();

        org.omg.Components.FacetDescription[] fDesc = 
            new org.omg.Components.FacetDescription[names.length];
        
        for (int i=0; i<names.length; i++) {
            String name = names[i];
            FacetInfo fInfo = (FacetInfo) this.facets.get(name);
    
            if (fInfo != null) {
                // receptacle_name exists
                fDesc[i] = new FacetDescriptionImpl();
    
                fDesc[i].name = name;
                fDesc[i].type_id = fInfo.repId;
                fDesc[i].facet_ref = fInfo.objRef;
            } else {
                CDMW_ERROR(logger, 
                    "Facet " + name + " not found.",
                    new Throwable());
                // name not found => error
                org.omg.Components.InvalidName ex =
                    new org.omg.Components.InvalidName();
                CDMW_LOG_FUNCTION_EXCEPTION(logger, ex);
                throw ex;
            }
        }
        
        return fDesc;

    }

    //
    // IDL:omg.org/Components/Navigation/same_component:1.0
    //
    public boolean same_component(org.omg.CORBA.Object objectRef)  {
        
        CDMW_LOG_FUNCTION(logger, "");
        checkIsRemoved();
    
        if (objectRef == null) {
            CDMW_ERROR(logger, "object_ref is null", new Throwable());
            org.omg.CORBA.BAD_PARAM ex = new org.omg.CORBA.BAD_PARAM(
                ExceptionMinorCodes.BAD_PARAMNilObjectReference, 
                org.omg.CORBA.CompletionStatus.COMPLETED_NO);
            CDMW_LOG_FUNCTION_EXCEPTION(logger, ex);
            throw ex;
        }

        // First case: object_ref is facet
        // Check object reference is equivalent with a facet of the component.
        java.util.Enumeration e = this.facets.elements();
    
        while (e.hasMoreElements()) {
            FacetInfo fInfo = (FacetInfo) e.nextElement();
    
            if (objectRef._is_equivalent(fInfo.objRef)) {
                CDMW_LOG_FUNCTION_RETURN(logger, true);
                return true;
            }
        }
    
        // Second case: object_ref is a component ref
        /*
        // Check facets of this component is equivalent with the current ones
        try {
            org.omg.Components.CCMObject componentRef 
                = org.omg.Components.CCMObjectHelper.narrow(objectRef);
    
            if (componentRef == null) {
                return false;
            }
            org.omg.Components.FacetDescription[] remoteFacets = 
                componentRef.get_all_facets();
            org.omg.Components.FacetDescription[] myFacets = get_all_facets();
    
            int nbFacet = myFacets.length;
    
            if (nbFacet != remoteFacets.length) {
                // components have not the same number of facet
                return false;
            }
    
            for (int i=0; i<nbFacet; i++) {
                org.omg.CORBA.Object objRef = myFacets[i].facet_ref;
                org.omg.CORBA.Object objRef2 = remoteFacets[i].facet_ref;
                if ( !objRef._is_equivalent(objRef2) ) {
                    return false;
                }
            }
        } catch (Exception e) {
            // exception raised
            // object ref is propably not a component ref
            return false;
        }
        */
    
        try {
            org.omg.CORBA.Object compRef = this.context.get_CCM_object();
            return compRef._is_equivalent(objectRef);
        } catch(org.omg.Components.IllegalState is) {
            CDMW_ERROR(logger, "IllegalState exception raised!", new Throwable());
            CDMW_LOG_FUNCTION_RETURN(logger, false);
            return false;
        } catch (org.omg.CORBA.SystemException se) {
            // exception raised
            CDMW_LOG_FUNCTION_RETURN(logger, false);
            return false;
        }
    
    }

    //
    // IDL:omg.org/Components/Receptacles/connect:1.0
    //
    /**
     * connects the object reference specified by the connection parameter to
     * the receptacle specified by the name parameter on the component.
     * 
     * @param name        The receptacle name.
     * @param connection  The connection object reference.
     *
     * @return a cookie if receptacle is multiplex, nil otherwise.
     * @exception org.omg.Components.InvalidName The name parameter does not specify a valid receptacle name.
     * @exception org.omg.Components.InvalidConnection The object reference in the connection parameter does
     * not support the interface declared in the receptacle's uses statement.
     * @exception org.omg.Components.AlreadyConnected The receptacle is a simplex receptacle and it is already connected.
     * @exception org.omg.Components.ExceededConnectionLimit The receptacle is a multiplex receptacle and the 
     * implementation-defined limit to the number of connections is exceeded.
     * @exception CORBA.SystemException  Any Corba system exception.
     */
    public org.omg.Components.Cookie connect(
        String name,
        org.omg.CORBA.Object connection)
        throws
            org.omg.Components.InvalidName,
            org.omg.Components.InvalidConnection,
            org.omg.Components.AlreadyConnected,
            org.omg.Components.ExceededConnectionLimit
    {
        CDMW_LOG_FUNCTION(logger, "name="+name);
        checkIsRemoved();

        org.omg.Components.Cookie cookieValue = null;

        if (name == null) {
            CDMW_ERROR(logger, "name is null", new Throwable());
            org.omg.Components.InvalidName ex = 
                new org.omg.Components.InvalidName();
            CDMW_LOG_FUNCTION_EXCEPTION(logger, ex);
            throw ex;
        }
    
        if (connection == null) {
            CDMW_ERROR(logger, "connection is null", new Throwable());
            org.omg.CORBA.BAD_PARAM ex = new org.omg.CORBA.BAD_PARAM(
                ExceptionMinorCodes.BAD_PARAMNilObjectReference, 
                org.omg.CORBA.CompletionStatus.COMPLETED_NO);
            CDMW_LOG_FUNCTION_EXCEPTION(logger, ex);
            throw ex;
        }
    
        // find receptacle
        ReceptacleInfo rInfo = (ReceptacleInfo) this.receptacles.get(name);
        
        if (rInfo != null) {
            // receptacle_name exists
    
            if (!rInfo.isMultiplex) {
                // simplex receptacle: check if a receptacle is not already connected!
                if (rInfo.connections.size() > 0) {
                    CDMW_ERROR(logger, 
                        "Receptacle " + name + " is already connected.", 
                        new Throwable());
                    org.omg.Components.AlreadyConnected ex = 
                        new org.omg.Components.AlreadyConnected();
                    CDMW_LOG_FUNCTION_EXCEPTION(logger, ex);
                    throw ex;
                }
            } else {
                // multiplex receptacle
                // No limit defined for connection : exception ExceededConnectionLimit not raised!
            }
    
            // Check connection supports the interface declared in uses statement
            if ( isTypeCheckingOn )
                checkType(rInfo.repId, connection, name);
    
            // object has the right interface, insert new connection in the list
            // Cookie is created even for simplex receptacle for homogeneity
            cookieValue = CookieImpl.createCookie();

            org.omg.CORBA.Object obj = connection._duplicate();

            // set the timeout on the receptacle if necessary
            try { 
                   obj = this.context.applyPolicy(obj, componentOid, name);
            }
            catch(org.omg.CORBA.SystemException e) { 
                CDMW_ERROR(logger, 
                    "CCMObject_impl connect: applyPolicy error!" + e,
                    new Throwable()); 
                org.omg.CORBA.INTERNAL ex = new org.omg.CORBA.INTERNAL(
                    ExceptionMinorCodes.INTERNALCCMCIFError, 
                    org.omg.CORBA.CompletionStatus.COMPLETED_NO);
                CDMW_LOG_FUNCTION_EXCEPTION(logger, ex);
                throw ex;
            }    

            rInfo.connections.put(cookieValue, obj);
    
            if (!rInfo.isMultiplex) {
                cookieValue = null;
            }
        } else {
            // no receptacle named name exist!
            CDMW_ERROR(logger, 
                "Receptacle " + name + " not found",
                new Throwable()); 
            org.omg.Components.InvalidName ex = 
                new org.omg.Components.InvalidName();
            CDMW_LOG_FUNCTION_EXCEPTION(logger, ex);
            throw ex;
        }
    
        return cookieValue;

    }

    //
    // IDL:omg.org/Components/Receptacles/disconnect:1.0
    //
    /**
     * disconnects the object reference specified by ck parameter to
     * the receptacle specified by the name parameter on the component.
     * 
     * @param name   The receptacle name.
     * @param ck     The connection cookie value.
     *
     * @return void.
     * @exception org.omg.Components.InvalidName The name parameter does not specify a valid receptacle name.
     * @exception org.omg.Components.InvalidConnection The receptacle is a multiplex receptacle and the cookie
     *           value in the ck parameter does not denote an existing connection on the receptacle.
     * @exception org.omg.Components.CookieRequired The receptacle is a multiplex receptacle and a null value
     *           is specified in the ck parameter.
     * @exception org.omg.Components.NoConnection The receptacle is a simplex receptacle there is no current connection.
     * @exception CORBA.SystemException  Any Corba system exception.
     *
     */
    public void disconnect(String name, org.omg.Components.Cookie ck)
        throws
            org.omg.Components.InvalidName,
            org.omg.Components.InvalidConnection,
            org.omg.Components.CookieRequired,
            org.omg.Components.NoConnection 
    {
        CDMW_LOG_FUNCTION(logger, "name="+name);
                
        checkIsRemoved();

        if (name == null) {
            CDMW_ERROR(logger, "name is null", new Throwable());
            org.omg.Components.InvalidName ex = 
                new org.omg.Components.InvalidName();
            CDMW_LOG_FUNCTION_EXCEPTION(logger, ex);
            throw ex;
        }
    
        // find receptacle
        ReceptacleInfo rInfo = (ReceptacleInfo) this.receptacles.get(name);
        
        if (rInfo != null) {
            // receptacleName exists
    
            if (rInfo.isMultiplex) {
                if (ck == null) {
                    CDMW_ERROR(logger, "Cookie is null", new Throwable());
                    org.omg.Components.CookieRequired ex =
                        new org.omg.Components.CookieRequired();
                    CDMW_LOG_FUNCTION_EXCEPTION(logger, ex);
                    throw ex;
                }
                
                // try to remove this connection from the list
                if (rInfo.connections.remove(ck) == null) {
                    // no connection found 
                    CDMW_ERROR(logger, 
                        "No connection corresponding to cookie", 
                        new Throwable());
                    org.omg.Components.InvalidConnection ex =
                        new org.omg.Components.InvalidConnection();
                    CDMW_LOG_FUNCTION_EXCEPTION(logger, ex);
                    throw ex;
                }
            } else {
                // simplex receptacle
                if (rInfo.connections.size() == 0) {
                    CDMW_ERROR(logger, 
                        "Receptacle " + name + " is not connected", 
                        new Throwable());
                    org.omg.Components.NoConnection ex =
                        new org.omg.Components.NoConnection();
                    CDMW_LOG_FUNCTION_EXCEPTION(logger, ex);
                    throw ex;
                }            
                
                // erase all element in map (there is only one!)
                rInfo.connections.clear();
            }
        } else {
            // no receptacle named name exist!
            CDMW_ERROR(logger, 
                "Receptacle " + name + " not found",
                new Throwable()); 
            org.omg.Components.InvalidName ex = 
                new org.omg.Components.InvalidName();
            CDMW_LOG_FUNCTION_EXCEPTION(logger, ex);
            throw ex;
        }

    }

    //
    // IDL:omg.org/Components/Receptacles/get_connections:1.0
    //
    public org.omg.Components.ConnectionDescription[] get_connections(
        String name)
        throws org.omg.Components.InvalidName 
    {
        CDMW_LOG_FUNCTION(logger, "name="+name);
        checkIsRemoved();

        if (name == null) {
            CDMW_ERROR(logger, "name is null", new Throwable());
            org.omg.Components.InvalidName ex = 
                new org.omg.Components.InvalidName();
            CDMW_LOG_FUNCTION_EXCEPTION(logger, ex);
            throw ex;
        }
        
        String[] nameList = new String[1];
        nameList[0] = name;
        
        org.omg.Components.ReceptacleDescription[] rDesc = 
            get_named_receptacles(nameList);
        return rDesc[0].connections;

    }

    //
    // IDL:omg.org/Components/Receptacles/get_all_receptacles:1.0
    //
    public org.omg.Components.ReceptacleDescription[] get_all_receptacles() {
        CDMW_LOG_FUNCTION(logger, "");
        checkIsRemoved();
    
        String[] nameList = new String[this.receptacles.size()];
        java.util.Set keyset = this.receptacles.keySet();
        java.util.Iterator iter = keyset.iterator();
        int i = 0;
    
        while (iter.hasNext()) {
            nameList[i] = (String) iter.next();
            i++;
        }
        try {
            org.omg.Components.ReceptacleDescription[] rDesc 
                = get_named_receptacles(nameList);
            return rDesc;
        } catch (org.omg.Components.InvalidName in) {
            CDMW_ERROR(logger, "InvalidName exception!", new Throwable());
            org.omg.CORBA.INTERNAL ex = new org.omg.CORBA.INTERNAL(
                ExceptionMinorCodes.INTERNAL, 
                org.omg.CORBA.CompletionStatus.COMPLETED_NO);
            CDMW_LOG_FUNCTION_EXCEPTION(logger, ex);
            throw ex;
        }
    
    }

    //
    // IDL:omg.org/Components/Receptacles/get_named_receptacles:1.0
    //
    public org.omg.Components.ReceptacleDescription[] get_named_receptacles(
        String[] names)
        throws org.omg.Components.InvalidName 
    {
        CDMW_LOG_FUNCTION(logger, "");
        checkIsRemoved();

        org.omg.Components.ReceptacleDescription[] rDesc = 
            new org.omg.Components.ReceptacleDescription[names.length];
        
        for (int i=0; i<names.length; i++) {
            String name = names[i];
            ReceptacleInfo rInfo = (ReceptacleInfo) this.receptacles.get(name);
    
            if (rInfo != null) {
                // receptacle_name exists
    
                rDesc[i] = new ReceptacleDescriptionImpl();
                rDesc[i].name = name;
                rDesc[i].type_id = rInfo.repId;
                rDesc[i].is_multiple = rInfo.isMultiplex;
    
                org.omg.Components.ConnectionDescription[] connections = 
                    new org.omg.Components.ConnectionDescription[rInfo.connections.size()];
                
                java.util.Set keySet = rInfo.connections.keySet();
                java.util.Iterator iter = keySet.iterator();
                int j = 0;

                while(iter.hasNext()) {
                    org.omg.Components.Cookie cookie = 
                        (org.omg.Components.Cookie) iter.next();
                    org.omg.CORBA.Object objectRef = 
                        (org.omg.CORBA.Object) rInfo.connections.get(cookie);
                    
                    connections[j] = new ConnectionDescriptionImpl();
                    connections[j].ck = cookie;
                    connections[j].objref = objectRef;
                    
                    j++;
                }
                rDesc[i].connections = connections;
            
            } else {
                // name not found => error
                CDMW_ERROR(logger, 
                    "Receptacle " + name + " not found", 
                    new Throwable());
                org.omg.Components.InvalidName ex = 
                    new org.omg.Components.InvalidName();
                CDMW_LOG_FUNCTION_EXCEPTION(logger, ex);
                throw ex;
            }
        }
        
        return rDesc;

    }

    //
    // IDL:omg.org/Components/Events/get_consumer:1.0
    //
    public org.omg.Components.EventConsumerBase get_consumer(String sinkName)
        throws org.omg.Components.InvalidName  
    {
        CDMW_LOG_FUNCTION(logger, "sinkName="+sinkName);    
        checkIsRemoved();

        org.omg.Components.EventConsumerBase result = null;

        if (sinkName == null) {
            CDMW_ERROR(logger, "sinkName is null", new Throwable());
            org.omg.Components.InvalidName ex = 
                new org.omg.Components.InvalidName();
            CDMW_LOG_FUNCTION_EXCEPTION(logger, ex);
            throw ex;
        }
    
        ConsumerInfo cInfo = (ConsumerInfo) this.consumers.get(sinkName);
        
        if (cInfo != null) {
            // sink exist
            return org.omg.Components.EventConsumerBaseHelper.unchecked_narrow(cInfo.consumer);
        } else {
            // no event sink defined with this name for this component
            CDMW_ERROR(logger, 
                "Consumer " + sinkName + " not found", 
                new Throwable());
            org.omg.Components.InvalidName ex = 
                new org.omg.Components.InvalidName();
            CDMW_LOG_FUNCTION_EXCEPTION(logger, ex);
            throw ex;
        }

    }

    //
    // IDL:omg.org/Components/Events/subscribe:1.0
    //
    public org.omg.Components.Cookie subscribe(
        String publisherName,
        org.omg.Components.EventConsumerBase subscriber)
        throws
            org.omg.Components.InvalidName,
            org.omg.Components.InvalidConnection,
            org.omg.Components.ExceededConnectionLimit 
    {
        CDMW_LOG_FUNCTION(logger, "publisherName="+publisherName);    
        checkIsRemoved();

        org.omg.Components.Cookie cookieValue = null;

        if (publisherName == null) {
            CDMW_ERROR(logger, "publisherName is null", new Throwable());
            org.omg.Components.InvalidName ex = 
                new org.omg.Components.InvalidName();
            CDMW_LOG_FUNCTION_EXCEPTION(logger, ex);
            throw ex;
        }
    
        if (subscriber == null) {
            CDMW_ERROR(logger, "subscriber is null", new Throwable());
            org.omg.CORBA.BAD_PARAM ex = new org.omg.CORBA.BAD_PARAM(
                ExceptionMinorCodes.BAD_PARAMNilObjectReference, 
                org.omg.CORBA.CompletionStatus.COMPLETED_NO);
            CDMW_LOG_FUNCTION_EXCEPTION(logger, ex);
            throw ex;
        }

        // find publisher
        PublisherInfo pInfo = 
            (PublisherInfo) this.publishers.get(publisherName);
        
        if (pInfo != null) {
            // publisherName exists

            // Check connection supports the interface declared in uses statement
            if ( isTypeCheckingOn )
                checkType(pInfo.repId, subscriber, publisherName);

            // check whether this consumer is not already registered;
            // in this case return the stored cookie
            java.util.Set< java.util.Map.Entry > subscribers =
                pInfo.subscriptions.entrySet();
            for (java.util.Map.Entry entry : subscribers) {
                SubscriberInfo sInfo = (SubscriberInfo) entry.getValue();
                if (subscriber._is_equivalent(sInfo.ccmConsumer)) {
                    CDMW_DEBUG(logger, "CCM_Object_impl:subscribe => "
                               + "is_equivalent true for " + publisherName 
                               + " return same cookie",
                               new java.lang.Throwable());
                    return (org.omg.Components.Cookie) entry.getKey();
                }
            }

            // object has the right interface, 
            // insert new subscription in the list
            cookieValue = CookieImpl.createCookie();
    
            // Subscribe consumer to the right event channel
            SubscriberInfo sInfo = new SubscriberInfo();
    
            sInfo.ccmConsumer = subscriber;
            
            // test if subscriber is a cdmw subscriber or not 
            // (that is implement CosEventComm.PushConsumer)
            sInfo.pushConsumer = null;
            if (isTypeCheckingOn) {
                try {
                    sInfo.pushConsumer =
                        org.omg.CosEventComm.PushConsumerHelper.narrow(
                            subscriber);
                }
                catch(org.omg.CORBA.BAD_PARAM bp1) {
                    // let sInfo.pushConsumer to null, it will be
                    // set just after.
                }
            }

            // insert subscription now because we need to find it 
            // in lookupServant when narrow operation is called!
            pInfo.subscriptions.put(cookieValue, sInfo);
            
            if (sInfo.pushConsumer == null) {
                // event_sink is not a CDMW one
                // Create my local push consumer to receive event 
                // this consumer will send to the EventConsumerBase.
                sInfo.pushConsumerServant =
                    new PushConsumerImpl(this.context, subscriber);

                String cookieStr = 
                    ((CookieImpl)cookieValue).getCookieString();
                System.out.println("cookieStr = " + cookieStr);
                byte[] pushConsumerOid = 
                CCMUtils.createOid(this.componentOid, 
                                   publisherName, 
                                   cookieStr);
                String pushConsumerOidStr =
                    cdmw.ccm.common.CCMUtils.OidToString(pushConsumerOid);

                org.omg.CORBA.Object obj =
                    context.create_ref_from_oid_str(pushConsumerOidStr, 
                                                    PUSH_CONSUMER_ID);
                try {
                    sInfo.pushConsumer = 
                        org.omg.CosEventComm.PushConsumerHelper.narrow(obj);
                } catch (org.omg.CORBA.BAD_PARAM e) {
                    // shouldn't happen, we just created obj as a
                    // PushConsumer reference !
                    cdmw.common.Assert.check(false);
                }
            }
    
            try {
                sInfo.proxySupplier 
                    = pInfo.channel.connectPushConsumer(sInfo.pushConsumer);
            }
            catch (org.omg.CORBA.SystemException se) {
                pInfo.subscriptions.remove(cookieValue);
                CDMW_ERROR(logger, 
                    "Exception raised when connecting consumer to channel: "+se,
                    new Throwable());
                CDMW_LOG_FUNCTION_EXCEPTION(logger, se);
                throw se;
            }
        }

        else {
            // no publisher named name exist!
            CDMW_ERROR(logger, 
                "Publisher " + publisherName + " not found", 
                new Throwable());
            org.omg.Components.InvalidName ex = 
                new org.omg.Components.InvalidName();
            CDMW_LOG_FUNCTION_EXCEPTION(logger, ex);
            throw ex;
        }

        return cookieValue;

    }

    //
    // IDL:omg.org/Components/Events/unsubscribe:1.0
    //
    public void unsubscribe(String publisherName,
        org.omg.Components.Cookie ck)
        throws org.omg.Components.InvalidName, 
            org.omg.Components.InvalidConnection 
    {
        CDMW_LOG_FUNCTION(logger, "publisherName="+publisherName);   
        checkIsRemoved();

        if (publisherName == null) {
            CDMW_ERROR(logger, "publisherName is null", new Throwable());
            org.omg.Components.InvalidName ex = 
                new org.omg.Components.InvalidName();
            CDMW_LOG_FUNCTION_EXCEPTION(logger, ex);
            throw ex;
        }
    
        if (ck == null) {
            CDMW_ERROR(logger, "Cookie is null", new Throwable());
            org.omg.CORBA.BAD_PARAM ex = new org.omg.CORBA.BAD_PARAM(
                ExceptionMinorCodes.BAD_PARAMNilObjectReference, 
                org.omg.CORBA.CompletionStatus.COMPLETED_NO);
            CDMW_LOG_FUNCTION_EXCEPTION(logger, ex);
            throw ex;
        }
    
        // find publisher
        PublisherInfo pInfo = 
            (PublisherInfo) this.publishers.get(publisherName);
        
        if (pInfo != null) {
            // publisher_name exists
    
            SubscriberInfo sInfo = (SubscriberInfo) pInfo.subscriptions.get(ck);
    
            if (sInfo != null) {
                // Unsubscribe consumer from the event channel
                sInfo.proxySupplier.disconnect_push_supplier();
    
                try {
                    // Call narrow with a timeout
                    com.thalesgroup.CdmwCcmCif.PushConsumer localPushConsumer
                        = narrowCdmwPushConsumer(sInfo.pushConsumer);
                    // localConsumer created, remove it!
                    localPushConsumer.remove();
                } catch(org.omg.CORBA.BAD_PARAM bp) {
                    // localConsumer not created, do nothing
                } catch(org.omg.CosLifeCycle.NotRemovable nr) {
                    CDMW_ERROR(logger, 
                        "LocalConsumer is not removable", 
                        new Throwable());
                    org.omg.Components.InvalidName ex = 
                        new org.omg.Components.InvalidName();
                    CDMW_LOG_FUNCTION_EXCEPTION(logger, ex);
                    throw ex;
                } catch (org.omg.Components.InvalidConnection ic) {
                    // Exception ignored
                } catch(org.omg.CORBA.SystemException se) {
                    // Exception ignored
                }
    
                // remove subscription
                pInfo.subscriptions.remove(ck);
            } else {
                // no connection found 
                CDMW_ERROR(logger, 
                    "Publisher " + publisherName + " is not connected", 
                    new Throwable());
                org.omg.Components.InvalidConnection ex =
                    new org.omg.Components.InvalidConnection();
                CDMW_LOG_FUNCTION_EXCEPTION(logger, ex);
                throw ex;
            }
        } else {
            // no publisher named publisher_name exist!
            CDMW_ERROR(logger, 
                "Publisher " + publisherName + " not found", 
                new Throwable());
            org.omg.Components.InvalidName ex = 
                new org.omg.Components.InvalidName();
            CDMW_LOG_FUNCTION_EXCEPTION(logger, ex);
            throw ex;
        }

    }

    //
    // IDL:omg.org/Components/Events/connect_consumer:1.0
    //
    public void connect_consumer(
        String emitterName,
        org.omg.Components.EventConsumerBase consumer)
        throws
            org.omg.Components.InvalidName,
            org.omg.Components.AlreadyConnected,
            org.omg.Components.InvalidConnection 
    {
        CDMW_LOG_FUNCTION(logger, "emitterName="+emitterName);   
        checkIsRemoved();
        if (emitterName == null) {
            CDMW_ERROR(logger, "emitterName is null", new Throwable());
            org.omg.Components.InvalidName ex = 
                new org.omg.Components.InvalidName();
            CDMW_LOG_FUNCTION_EXCEPTION(logger, ex);
            throw ex;
        }
    
        if (consumer == null) {
            CDMW_ERROR(logger, "consumer is null", new Throwable());
            org.omg.CORBA.BAD_PARAM ex = new org.omg.CORBA.BAD_PARAM(
                ExceptionMinorCodes.BAD_PARAMNilObjectReference, 
                org.omg.CORBA.CompletionStatus.COMPLETED_NO);
            CDMW_LOG_FUNCTION_EXCEPTION(logger, ex);
            throw ex;
        }

        // find emitter
        EmitterInfo eInfo = (EmitterInfo) this.emitters.get(emitterName);
        
        if (eInfo != null) {
            // emitterName exists
    
            // Check connection supports the interface declared in uses statement
            if ( isTypeCheckingOn )
                checkType(eInfo.repId, consumer, emitterName);
    
            if (eInfo.ccmConsumer == null) {
                // No consumer connected
    
                eInfo.ccmConsumer = consumer;
    
                // test if subscriber is a cdmw subscriber or not
                // (that is implement CosEventComm.PushConsumer)
                if (isTypeCheckingOn) {
                    try {
                        // Call narrow with a timeout
                        eInfo.pushConsumer =
                            narrowCosPushConsumer(
                                consumer);
                    } catch (org.omg.Components.InvalidConnection ic) {
                        // Exception ignored
                        eInfo.pushConsumer = null;
                    } catch(org.omg.CORBA.SystemException se) {
                        // Exception ignored
                        eInfo.pushConsumer = null;
                    }
                }

                if (eInfo.pushConsumer == null) {
                    // event_sink is not a CDMW one
                    // Create my local push consumer to receive event 
                    // this consumer will send to the EventConsumerBase.
                    eInfo.pushConsumerServant = 
                        new PushConsumerImpl(this.context, consumer);

                    byte[] pushConsumerOid = 
                        CCMUtils.createOid(this.componentOid, 
                                           emitterName, 
                                           PSEUDO_EMITTER_CK_VALUE);
                    String pushConsumerOidStr =
                        cdmw.ccm.common.CCMUtils.OidToString(pushConsumerOid);

                    org.omg.CORBA.Object obj = 
                        this.context.create_ref_from_oid_str(
                            pushConsumerOidStr, PUSH_CONSUMER_ID);

                    try {
                        eInfo.pushConsumer = 
                            org.omg.CosEventComm.PushConsumerHelper.narrow(obj);
                    }
                    catch (org.omg.CORBA.BAD_PARAM e) {
                        // shouldn't happen, we just created obj as a
                        // PushConsumer reference !
                        cdmw.common.Assert.check(false);
                    }
                }
                              
                try {
                    eInfo.proxySupplier =
                        eInfo.channel.connectPushConsumer(eInfo.pushConsumer);
                }
                catch (org.omg.CORBA.SystemException se) {
                    CDMW_ERROR(logger, 
                        "Exception raised when connecting consumer to channel:"+se,
                        new Throwable());
                    CDMW_LOG_FUNCTION_EXCEPTION(logger, se);
                    throw se;
                }
            }

            else if (! eInfo.ccmConsumer._is_equivalent(consumer)) {
                CDMW_ERROR(logger, 
                    "Emitter " + emitterName + " is already connected",
                    new Throwable());
                org.omg.Components.AlreadyConnected ex =
                    new org.omg.Components.AlreadyConnected();
                CDMW_LOG_FUNCTION_EXCEPTION(logger, ex);
                throw ex;
            }
        }

        else {
            // no emitter named emitterName exist!
            CDMW_ERROR(logger, 
                "Emitter " + emitterName + " not found",
                new Throwable());
            org.omg.Components.InvalidName ex =
                new org.omg.Components.InvalidName();
            CDMW_LOG_FUNCTION_EXCEPTION(logger, ex);
            throw ex;
        }
    }

    //
    // IDL:omg.org/Components/Events/disconnect_consumer:1.0
    //
    public org.omg.Components.EventConsumerBase disconnect_consumer(
        String sourceName)
        throws org.omg.Components.InvalidName, 
               org.omg.Components.NoConnection 
    {
        CDMW_LOG_FUNCTION(logger, "sourceName="+sourceName);  
        checkIsRemoved();

        org.omg.Components.EventConsumerBase result = null;

        if (sourceName == null) {
            CDMW_ERROR(logger, "sourceName is null", new Throwable());
            org.omg.Components.InvalidName ex = 
                new org.omg.Components.InvalidName();
            CDMW_LOG_FUNCTION_EXCEPTION(logger, ex);
            throw ex;
        }
    
        // find emitter
        EmitterInfo eInfo = (EmitterInfo) this.emitters.get(sourceName);
        
        if (eInfo != null) {
            // emitter_name exists
    
            if (eInfo.ccmConsumer == null) {
                CDMW_ERROR(logger, 
                    "Emitter " + sourceName + " is not connected", 
                    new Throwable());
                org.omg.Components.NoConnection ex =
                    new org.omg.Components.NoConnection();
                CDMW_LOG_FUNCTION_EXCEPTION(logger, ex);
                throw ex;
            } else {
                // disconnect consumer from event channel
                eInfo.proxySupplier.disconnect_push_supplier();
    
                try {
                    // Call narrow with a timeout
                    com.thalesgroup.CdmwCcmCif.PushConsumer localPushConsumer
                        = narrowCdmwPushConsumer(eInfo.pushConsumer);
                    // localConsumer created, remove it!
                    localPushConsumer.remove();
                } catch(org.omg.CORBA.BAD_PARAM bp) {
                    // localConsumer not created, do nothing
                } catch(org.omg.CosLifeCycle.NotRemovable nr) {
                    CDMW_ERROR(logger, 
                        "LocalConsumer is not removable", 
                        new Throwable());
                    org.omg.Components.InvalidName ex =
                        new org.omg.Components.InvalidName();
                    CDMW_LOG_FUNCTION_EXCEPTION(logger, ex);
                    throw ex;
                } catch (org.omg.Components.InvalidConnection ic) {
                    // Exception ignored
                } catch(org.omg.CORBA.SystemException se) {
                    // Exception ignored
                }
    
                // remove consumer
                result = eInfo.ccmConsumer;
                eInfo.ccmConsumer = null;
                eInfo.pushConsumer = null;
                eInfo.pushConsumerServant = null;
    
            }
        } else {
            // no emitter named emitter_name exist!
            CDMW_ERROR(logger, 
                "Emitter " + sourceName + " not found",
                new Throwable());
            org.omg.Components.InvalidName ex =
                new org.omg.Components.InvalidName();
            CDMW_LOG_FUNCTION_EXCEPTION(logger, ex);
            throw ex;
        }
    
        return result;

    }

    //
    // IDL:omg.org/Components/Events/get_all_consumers:1.0
    //
    public org.omg.Components.ConsumerDescription[] get_all_consumers() {

        CDMW_LOG_FUNCTION(logger, "");  
        checkIsRemoved();

        String[] nameList = new String[this.consumers.size()];
        java.util.Set keyset = this.consumers.keySet();
        java.util.Iterator iter = keyset.iterator();
        int i = 0;

        while (iter.hasNext()) {
            nameList[i] = (String) iter.next();
            i++;
        }
        try {
            return get_named_consumers(nameList);
        } catch (org.omg.Components.InvalidName in) {
            CDMW_ERROR(logger, "InvalidName exception!", new Throwable());
            org.omg.CORBA.INTERNAL ex = new org.omg.CORBA.INTERNAL(
                ExceptionMinorCodes.INTERNAL, 
                org.omg.CORBA.CompletionStatus.COMPLETED_NO);
            CDMW_LOG_FUNCTION_EXCEPTION(logger, ex);
            throw ex;
        }

    }

    //
    // IDL:omg.org/Components/Events/get_named_consumers:1.0
    //
    public org.omg.Components.ConsumerDescription[] get_named_consumers(
        String[] names)
        throws org.omg.Components.InvalidName 
    {
        CDMW_LOG_FUNCTION(logger, "");  
        checkIsRemoved();
        org.omg.Components.ConsumerDescription[] cDesc = 
            new org.omg.Components.ConsumerDescription[names.length];
        
        for (int i=0; i<names.length; i++) {
            String name = names[i];
            ConsumerInfo cInfo = (ConsumerInfo) this.consumers.get(name);
    
            if (cInfo != null) {
                // consumer_name exists
    
                cDesc[i] = new ConsumerDescriptionImpl();
    
                cDesc[i].name = name;
                cDesc[i].type_id = cInfo.repId;
                cDesc[i].consumer = 
                    org.omg.Components.EventConsumerBaseHelper.unchecked_narrow(
                        cInfo.consumer);
            }
            else {
                // name not found => error
                CDMW_ERROR(logger, 
                    "Consumer " + name + " not found", 
                    new Throwable());
                org.omg.Components.InvalidName ex = 
                    new org.omg.Components.InvalidName();
                CDMW_LOG_FUNCTION_EXCEPTION(logger, ex);
                throw ex;
            }
        }
        
        return cDesc;

    }

    //
    // IDL:omg.org/Components/Events/get_all_emitters:1.0
    //
    public org.omg.Components.EmitterDescription[] get_all_emitters() {
        
        CDMW_LOG_FUNCTION(logger, "");
        checkIsRemoved();
        
        String[] nameList = new String[this.emitters.size()];
        java.util.Set keyset = this.emitters.keySet();
        java.util.Iterator iter = keyset.iterator();
        int i = 0;
    
        while (iter.hasNext()) {
            nameList[i] = (String) iter.next();
            i++;
        }
        try {
            return get_named_emitters(nameList);
        } catch (org.omg.Components.InvalidName in) {
            CDMW_ERROR(logger, "InvalidName exception!", new Throwable());
            org.omg.CORBA.INTERNAL ex = new org.omg.CORBA.INTERNAL(
                ExceptionMinorCodes.INTERNAL, 
                org.omg.CORBA.CompletionStatus.COMPLETED_NO);
            CDMW_LOG_FUNCTION_EXCEPTION(logger, ex);
            throw ex;
        }
        
    }

    //
    // IDL:omg.org/Components/Events/get_named_emitters:1.0
    //
    public org.omg.Components.EmitterDescription[] get_named_emitters(
        String[] names) throws org.omg.Components.InvalidName  
    {
        CDMW_LOG_FUNCTION(logger, "");
        checkIsRemoved();

        org.omg.Components.EmitterDescription[] eDesc = 
            new org.omg.Components.EmitterDescription[names.length];
        
        for (int i=0; i<names.length; i++) {
            String name = names[i];
            EmitterInfo eInfo = (EmitterInfo) this.emitters.get(name);
    
            if (eInfo != null) {
                // emitter_name exists
    
                eDesc[i] = new EmitterDescriptionImpl();
    
                eDesc[i].name= name;
                eDesc[i].type_id = eInfo.repId;
                eDesc[i].consumer = eInfo.ccmConsumer;
            } else {
                // name not found => error
                CDMW_ERROR(logger, 
                    "Emitter " + name + " not found", 
                    new Throwable());
                org.omg.Components.InvalidName ex = 
                    new org.omg.Components.InvalidName();
                CDMW_LOG_FUNCTION_EXCEPTION(logger, ex);
                throw ex;
            }
        }
        
        return eDesc;

    }

    //
    // IDL:omg.org/Components/Events/get_all_publishers:1.0
    //
    public org.omg.Components.PublisherDescription[] get_all_publishers() {
        
        CDMW_LOG_FUNCTION(logger, "");
        checkIsRemoved();
        String[] nameList = new String[this.publishers.size()];
        java.util.Set keyset = this.publishers.keySet();
        java.util.Iterator iter = keyset.iterator();
        int i = 0;
    
        while (iter.hasNext()) {
            nameList[i] = (String) iter.next();
            i++;
        }
        try {
            return get_named_publishers(nameList);
        } catch (org.omg.Components.InvalidName in) {
            CDMW_ERROR(logger, "InvalidName exception!", new Throwable());
            org.omg.CORBA.INTERNAL ex = new org.omg.CORBA.INTERNAL(
                ExceptionMinorCodes.INTERNAL, 
                org.omg.CORBA.CompletionStatus.COMPLETED_NO);
            CDMW_LOG_FUNCTION_EXCEPTION(logger, ex);
            throw ex;
       }

    }

    //
    // IDL:omg.org/Components/Events/get_named_publishers:1.0
    //
    public org.omg.Components.PublisherDescription[] get_named_publishers(
        String[] names)
        throws org.omg.Components.InvalidName  
    {
        CDMW_LOG_FUNCTION(logger, "");
        checkIsRemoved();

        org.omg.Components.PublisherDescription[] pDesc = 
            new org.omg.Components.PublisherDescription[names.length];
    
        for (int i=0; i<names.length; i++) {
            String name = names[i];
            PublisherInfo pInfo = (PublisherInfo) this.publishers.get(name);
    
            if (pInfo != null) {
                // publisher_name exists
    
                pDesc[i] = new PublisherDescriptionImpl();
                pDesc[i].name = name;
                pDesc[i].type_id = pInfo.repId;

                org.omg.Components.SubscriberDescription[] consumers = 
                    new org.omg.Components.SubscriberDescription[pInfo.subscriptions.size()];
                java.util.Set keySet = pInfo.subscriptions.keySet();
                java.util.Iterator iter = keySet.iterator();
                int j = 0;
    
                while (iter.hasNext()) {
                    org.omg.Components.Cookie cookie = 
                        (org.omg.Components.Cookie) iter.next();
                    SubscriberInfo sInfo = 
                        (SubscriberInfo) pInfo.subscriptions.get(cookie);

                    consumers[j] = new SubscriberDescriptionImpl();
                    consumers[j].ck = cookie;
                    consumers[j].consumer = sInfo.ccmConsumer;
                    j++;
                }
                
                pDesc[i].consumers = consumers;
            } else {
                // name not found => error
                CDMW_ERROR(logger, 
                    "Publisher " + name + " not found", 
                    new Throwable());
                org.omg.Components.InvalidName ex = 
                    new org.omg.Components.InvalidName();
                CDMW_LOG_FUNCTION_EXCEPTION(logger, ex);
                throw ex;
            }
        }
        
        return pDesc;

    }



    /**
     * This checks whether the CORBA object incarnated by the current
     * object (this) has been removed (<I>remove</I> operation has been 
     * called).
     *
     * @return <B>true</B> if the object has been removed, <B>false</B> otherwise.
     */
    public boolean is_removed() {
        return this.isRemoved;
    }

    public void checkIsRemoved() throws org.omg.CORBA.OBJECT_NOT_EXIST
    {
        if(is_removed())
        {
            CDMW_ERROR(logger, "Component has been removed", new Throwable());
            throw new org.omg.CORBA.OBJECT_NOT_EXIST(
                ExceptionMinorCodes.OBJECT_NOT_EXISTObjectRemoved,
                org.omg.CORBA.CompletionStatus.COMPLETED_NO);
        }
    }



    /**
     * This returns the proxy consumer for the event source specified by
     * source_name. This operation has been created for internal use.
     *
     * @param source_name The event source name (publisher/emitter)
     *
     * @return the proxy consumer of the event service.
     */
    public org.omg.CosEventComm.PushConsumer 
        get_proxy_consumer(String sourceName) 
    {
        CDMW_LOG_FUNCTION(logger, "");
        checkIsRemoved();

        org.omg.CosEventComm.PushConsumer consumer = null;
        
        // find source name in publisher map and emitter map
        PublisherInfo pInfo = (PublisherInfo) this.publishers.get(sourceName);
        
        if (pInfo != null) {
            // publisher_name exists
            consumer = pInfo.proxyConsumer;
        } else {
            // find in emitter map
            EmitterInfo eInfo = (EmitterInfo) this.emitters.get(sourceName);
            
            if (eInfo != null) {
                // emitter_name exists
                consumer = eInfo.proxyConsumer;
            } else {
                // no emitter nor publisher exist!
                // nil is return
            }
        }
        
        return consumer;

    }

    /**
     * This returns the servant for the local push consumer specified by source_name
     * and cookie value.
     *
     * @param source_name The source name
     * @param ck_value    The cookie value
     *
     * @return the local push consumer servant.
     */
    public org.omg.PortableServer.Servant get_local_push_consumer_servant(
        String sourceName,
        String ckValue) 
    {
        CDMW_LOG_FUNCTION(logger, 
            "sourceName="+sourceName+"  ckValue="+ckValue);
        checkIsRemoved();

        if (!ckValue.equals(PSEUDO_EMITTER_CK_VALUE)) {
            // look in publisher map
            PublisherInfo pInfo = (PublisherInfo) this.publishers.get(sourceName);
            
            if (pInfo != null) {
                // sourceName is a publisher
                
                org.omg.Components.Cookie ck = new CookieImpl(ckValue);
                SubscriberInfo sInfo = (SubscriberInfo) pInfo.subscriptions.get(ck);
                
                if (sInfo != null) {
                    
                    if (sInfo.pushConsumerServant != null) {
                        return sInfo.pushConsumerServant;
                    } else {
                        // No local push consumer with this name exist in this component
                        CDMW_ERROR(logger, 
                            "Local push consumer not found: do not exist for this connection.", 
                            new Throwable());
                        org.omg.CORBA.OBJECT_NOT_EXIST ex = 
                            new org.omg.CORBA.OBJECT_NOT_EXIST(
                                ExceptionMinorCodes.OBJECT_NOT_EXIST,
                                org.omg.CORBA.CompletionStatus.COMPLETED_NO);
                        CDMW_LOG_FUNCTION_EXCEPTION(logger, ex);
                        throw ex;
                    }
                } else {
                    // No local push consumer with this name exist in this component
                    CDMW_ERROR(logger, 
                        "Local push consumer not found: no connexion exists in this component", 
                        new Throwable());
                    org.omg.CORBA.OBJECT_NOT_EXIST ex = 
                        new org.omg.CORBA.OBJECT_NOT_EXIST(
                            ExceptionMinorCodes.OBJECT_NOT_EXIST,
                            org.omg.CORBA.CompletionStatus.COMPLETED_NO);
                    CDMW_LOG_FUNCTION_EXCEPTION(logger, ex);
                    throw ex;
                }
            } else {
                // No local push consumer with this name exist in this component
                CDMW_ERROR(logger, 
                    "Local push consumer not found: no publisher exists in this component", 
                    new Throwable());
                org.omg.CORBA.OBJECT_NOT_EXIST ex =
                    new org.omg.CORBA.OBJECT_NOT_EXIST(
                        ExceptionMinorCodes.OBJECT_NOT_EXIST,
                        org.omg.CORBA.CompletionStatus.COMPLETED_NO);
                CDMW_LOG_FUNCTION_EXCEPTION(logger, ex);
                throw ex;
            }
        } else {
            // look in emitter map
            EmitterInfo eInfo = (EmitterInfo) this.emitters.get(sourceName);
            
            if (eInfo != null) {
                // sourceName is an emitter
    
                if (eInfo.pushConsumerServant != null) {
                    return eInfo.pushConsumerServant;
                } else {
                    // No local push consumer with this name exist in this component
                    CDMW_ERROR(logger, 
                        "Local push consumer not found: do not exists in this component for this emitter", 
                        new Throwable());
                    org.omg.CORBA.OBJECT_NOT_EXIST ex = 
                        new org.omg.CORBA.OBJECT_NOT_EXIST(
                            ExceptionMinorCodes.OBJECT_NOT_EXIST,
                            org.omg.CORBA.CompletionStatus.COMPLETED_NO);
                    CDMW_LOG_FUNCTION_EXCEPTION(logger, ex);
                    throw ex;
                }
            } else {
                // No local push consumer with this name exist in this component
                CDMW_ERROR(logger, 
                    "Local push consumer not found: no emitter exists in this component", 
                    new Throwable());
                org.omg.CORBA.OBJECT_NOT_EXIST ex = 
                    new org.omg.CORBA.OBJECT_NOT_EXIST(
                        ExceptionMinorCodes.OBJECT_NOT_EXIST,
                        org.omg.CORBA.CompletionStatus.COMPLETED_NO);
                CDMW_LOG_FUNCTION_EXCEPTION(logger, ex);
                throw ex;
            }
        }

    }

    /**
     * This returns the servant for the facet specified by facet_name.
     * This operation is implemented by the well typed component servant.
     *
     * @param facet_name The facet name
     *
     * @return the facet servant.
     */
    public abstract org.omg.PortableServer.Servant get_facet_servant(
        String facet_name);

        
    /**
    * In case of a session component, this operation will call 
    * it's executor's ccm_activate operation.
    *
    *@exception Components::CCMException Any CCMException thrown by the
    * executor's ccm_activate operation.
    *@exception CORBA::SystemException Any SystemException thrown by the
    * executor's ccm_activate operation.
    */ 
    public abstract void ccm_activate() 
        throws org.omg.Components.CCMException, 
               org.omg.CORBA.SystemException;
    
    /**
    * In case of a session component, this operation will call 
    * it's executor's ccm_passivate operation.
    *
    *@exception Components::CCMException Any CCMException thrown by the
    * executor's ccm_passivate operation.
    *@exception CORBA::SystemException Any SystemException thrown by the
    * executor's ccm_passivate operation.
    */ 
    public abstract void ccm_passivate() 
        throws org.omg.Components.CCMException, 
               org.omg.CORBA.SystemException;
    
    /**
    * In case of a session component, this operation will call 
    * it's executor's ccm_remove operation.
    *
    *@exception Components::CCMException Any CCMException thrown by the
    * executor's ccm_remove operation.
    *@exception CORBA::SystemException Any SystemException thrown by the
    * executor's ccm_remove operation.
    */ 
    public abstract void ccm_remove() 
        throws org.omg.Components.CCMException, 
               org.omg.CORBA.SystemException;
    
        
    //////////////////////////
    //                      //
    // Internal Operations  //
    //                      //
    //////////////////////////

    /**
     * Adds a facet in FacetMap
     * 
     * @param comp_oid       The component oid used to create the facet reference.
     * @param facet_name     The facet name.
     * @param repId         The facet repository id.
     *
     * @return  The reference to be exported to clients
     * @exception InternalErrorException
     * @exception AlreadyDoneException
     */
    protected org.omg.CORBA.Object declare_facet(
        String compOid,
        String facetName,
        String repId)
        throws AlreadyDoneException, InternalErrorException 
    {
        CDMW_LOG_FUNCTION(logger, 
            "compOid="+compOid+"  facetName="+facetName+"  repId="+repId);
        FacetInfo facetInfo = new FacetInfo();
        facetInfo.repId = repId;
        facetInfo.objRef = null;
    
        try {
            // check a facet with a same name has not been already declared
            if (!this.facets.containsKey(facetName)) {
                if (isMonolithic) {
                    facetInfo.objRef =
                        this.context.create_ref_from_oid_str(compOid, 
                                                             repId);
                } else {
                    byte[] facetOid = CCMUtils.createOid(compOid, facetName);
                    String facetOidStr = 
                        cdmw.ccm.common.CCMUtils.OidToString(facetOid);
                    facetInfo.objRef = 
                        context.create_ref_from_oid_str(facetOidStr, repId);
                }
    
                this.facets.put(facetName, facetInfo);
                
            } else {
                CDMW_ERROR(logger, 
                    "Facet " + facetName + " already declared", 
                    new Throwable());
                AlreadyDoneException ex = new AlreadyDoneException();
                CDMW_LOG_FUNCTION_EXCEPTION(logger, ex);
                throw ex;
            }
        } catch (org.omg.CORBA.SystemException se) {
            CDMW_ERROR(logger, 
                "SystemException raised: "+se, 
                new Throwable());
            se.printStackTrace();
            InternalErrorException ex = new InternalErrorException();
            CDMW_LOG_FUNCTION_EXCEPTION(logger, ex);
            throw ex;
        }
    
        // Only register facet for recording in case of segmented containers
        // since there is only one single object reference for all the component
        // features when the container is monolithic.
        // NOTA: Failing to record the component for recording will only log
        //       a WARN message and not abort deployment of component.
        if(!isMonolithic) {
           try {
              // register facet for recording (if Recording ConfigValue set to true)
              this.context.registerForRecording(compOid, facetInfo.objRef);
           }
           catch (org.omg.Components.CreateFailure e)
           {
              // Should have worked since it already worked for component itself !
              // Throw InternalErrorException
              CDMW_ERROR(logger, 
                         "Error registering facet " + facetName + " for recording: " +
                         (e.reason == WRONG_RECORDING_CONFIG_VALUES.value ?
                          "Wrong Recording ConfigValue" : "Recording Daemon error"),
                         new Throwable());
              CDMW_WARN(logger, 
                        "Incoming requets to facet '" + facetName + 
                        "' will not be recorded!" ,
                        new Throwable());
           }
        }

        return facetInfo.objRef;

    }

    /**
     * Adds a receptacle in ReceptacleMap
     * 
     * @param receptacleName  The receptacle name.
     * @param repId           The receptacle repository id.
     * @param isMultiplex     This is set to true if receptacle is multiple
     *
     * @exception AlreadyDoneException
     */
    protected void declare_receptacle(
        String receptacleName,
        String repId,
        boolean isMultiplex)
        throws AlreadyDoneException 
    {
        CDMW_LOG_FUNCTION(logger, 
            "receptacleName="+receptacleName+"  repId="+repId+
            "  isMultiplex="+isMultiplex);

        ReceptacleInfo receptacleInfo = new ReceptacleInfo();
        receptacleInfo.repId = repId;
        receptacleInfo.isMultiplex = isMultiplex;
    
        // check a receptacle with a same name has not been already declared
        ReceptacleInfo rInfo = 
            (ReceptacleInfo) this.receptacles.get(receptacleName);
        if (rInfo == null) {
            this.receptacles.put(receptacleName, receptacleInfo);
        } else {
            CDMW_ERROR(logger, 
                "Receptacle " + receptacleName + " already declared", 
                new Throwable());
            AlreadyDoneException ex = new AlreadyDoneException();
            CDMW_LOG_FUNCTION_EXCEPTION(logger, ex);
            throw ex;
        }

    }
    
    /**
     * Adds a receptacle in ReceptacleMap if receptacle is simple
     * 
     * @param receptacleName  The receptacle name.
     * @param repId           The receptacle repository id.
     *
     * @exception AlreadyDoneException
     */
    protected void declare_receptacle(
        String receptacleName,
        String repId)
        throws AlreadyDoneException {
    
        declare_receptacle(receptacleName, repId, false);
    
    }

    /**
     * Adds a consumer in ConsumerMap
     * 
     * @param compOid        The component oid used to create the consumer reference.
     * @param consumerName   The consumer name.
     * @param repId          The consumer repository id.
     *
     * @return void
     * @exception AlreadyDoneException
     */
    protected org.omg.CORBA.Object declare_consumer(
        String compOid,
        String consumerName,
        String repId)
        throws AlreadyDoneException, 
               InternalErrorException 
    {
        CDMW_LOG_FUNCTION(logger, 
            "compOid="+compOid+"  consumerName="+consumerName+"  repId="+repId);

        ConsumerInfo consumerInfo = new ConsumerInfo();
        consumerInfo.repId = repId;
        consumerInfo.consumer = null;
    
        try {
            // check a consumer with a same name has not been already declared
            ConsumerInfo cInfo = 
                (ConsumerInfo) this.consumers.get(consumerName);
            if (cInfo == null) { 
                if(isMonolithic) {
                       consumerInfo.consumer = context.create_ref_from_oid_str(compOid, repId);
                } else {
                    byte[]  consumer_oid_array = 
                        CCMUtils.createOid(compOid, consumerName);
                    String consumer_oid_str = 
                        cdmw.ccm.common.CCMUtils.OidToString(consumer_oid_array);
                    consumerInfo.consumer = 
                        context.create_ref_from_oid_str(consumer_oid_str, repId);
                 }
        
                this.consumers.put(consumerName, consumerInfo);

            } else {
                CDMW_ERROR(logger, 
                    "Consumer " + consumerName + " already declared", 
                    new Throwable());
                AlreadyDoneException ex = new AlreadyDoneException();
                CDMW_LOG_FUNCTION_EXCEPTION(logger, ex);
                throw ex;
            }
        } catch (org.omg.CORBA.SystemException se) {
            CDMW_ERROR(logger, 
                "SystemException raised: "+se, 
                new Throwable());
            se.printStackTrace();
            InternalErrorException ex = new InternalErrorException();
            CDMW_LOG_FUNCTION_EXCEPTION(logger, ex);
            throw ex;
        }

        // Only register consumer for recording in case of segmented containers
        // since there is only one single object reference for all the component
        // features when the container is monolithic.
        // NOTA: Failing to record the component for recording will only log
        //       a WARN message and not abort deployment of component.
        if(!isMonolithic) {
           try {
              // register consumer for recording (if Recording ConfigValue set to true)
              this.context.registerForRecording(compOid, consumerInfo.consumer);
           }
           catch (org.omg.Components.CreateFailure e)
           {
              CDMW_ERROR(logger, 
                         "Error registering consumer " + consumerName + " for recording: " +
                         (e.reason == WRONG_RECORDING_CONFIG_VALUES.value ?
                          "Wrong Recording ConfigValue" : "Recording Daemon error"),
                         new Throwable());
              CDMW_WARN(logger, 
                         "Incoming events to event sink '" + consumerName + 
                        "' will not be recorded!" ,
                         new Throwable());
           }
        }

        return consumerInfo.consumer;
    }

    /**
     * Adds an emitter in EmitterMap
     * 
     * @param emitterName   The emitter name.
     * @param repId         The emitter repository id.
     *
     * @return void
     * @exception AlreadyDoneException
     */
    protected void declare_emitter(String emitterName, String repId)
        throws AlreadyDoneException 
    {
        CDMW_LOG_FUNCTION(logger, 
            "emitterName="+emitterName+"  repId="+repId);
            
        checkIsRemoved();

        // check a emitter with a same name has not been already declared
        EmitterInfo eInfo = (EmitterInfo) this.emitters.get(emitterName);
        if (eInfo == null) {
            // store information concerning the emitter
            EmitterInfo emitterInfo = new EmitterInfo();
            emitterInfo.repId = repId;
            
            // create an event channel associated to this component instance and 
            // this port name
            try {
                org.omg.Components.PortDescription portDesc 
                    = new PortDescriptionImpl();
                portDesc.name = emitterName;
                portDesc.type_id = repId;
                
                org.omg.CosEventChannelAdmin.EventChannel eventChannel =
                    this.context.obtain_event_channel_for_emitter(portDesc);
                
                emitterInfo.channel = 
                    new cdmw.event.EventChannelInterface(eventChannel);
            } catch (com.thalesgroup.CdmwCcmContainer.NoChannel nc) {
                CDMW_ERROR(logger, 
                    emitterName + " event channel doesn't exist!",
                    new Throwable());
                org.omg.CORBA.INTERNAL ex = new org.omg.CORBA.INTERNAL(
                    ExceptionMinorCodes.INTERNAL, 
                    org.omg.CORBA.CompletionStatus.COMPLETED_NO);
                CDMW_LOG_FUNCTION_EXCEPTION(logger, ex);
                throw ex;
            } catch (org.omg.CORBA.SystemException se){
                CDMW_ERROR(logger, 
                    "System exception raised during event channel creation: "+se,
                    new Throwable());
                CDMW_LOG_FUNCTION_EXCEPTION(logger, se);
                throw se;
            }
            
            try {
                emitterInfo.proxyConsumer = 
                    emitterInfo.channel.getPushConsumer(null);
            } catch (org.omg.CORBA.SystemException se) {
                CDMW_ERROR(logger, 
                    "System exception raised during emitter connection to event channel: "+se,
                    new Throwable());
                CDMW_LOG_FUNCTION_EXCEPTION(logger, se);
                throw se;
            }
            
            // emitter info completed, store it in map
            this.emitters.put(emitterName, emitterInfo);
        } else {
            CDMW_ERROR(logger, 
                "Emitter " + emitterName + " already declared", 
                new Throwable());
            AlreadyDoneException ex = new AlreadyDoneException();
            CDMW_LOG_FUNCTION_EXCEPTION(logger, ex);
            throw ex;
        }

    }

    /**
     * Adds a publisher in PublisherMap
     * 
     * @param publisher_name   The publisher name.
     * @param repId           The publisher repository id.
     *
     * @return void
     * @exception AlreadyDoneException if the publisher has already been declared.
     */
    protected void declare_publisher(String publisherName, String repId)
        throws AlreadyDoneException 
    {
        CDMW_LOG_FUNCTION(logger, 
            "publisherName="+publisherName+"  repId="+repId);
            
        checkIsRemoved();

        // check a publisher with a same name has not been already declared
        PublisherInfo pInfo = 
            (PublisherInfo) this.publishers.get(publisherName);
        if (pInfo == null) {
            // store information concerning the pusblisher
            PublisherInfo publisherInfo = new PublisherInfo();
            publisherInfo.repId = repId;
            
            // create an event channel associated to this component instance and 
            // this port name
            try {
                org.omg.Components.PortDescription portDesc = 
                    new PortDescriptionImpl();
                portDesc.name = publisherName;
                portDesc.type_id = repId;
                
                org.omg.CosEventChannelAdmin.EventChannel eventChannel =
                    this.context.create_event_channel_for_publisher(portDesc);
                
                publisherInfo.channel = 
                    new cdmw.event.EventChannelInterface(eventChannel);
            } catch (org.omg.Components.CreateFailure cf) {
                CDMW_ERROR(logger, 
                    publisherName + " event channel doesn't exist!",
                    new Throwable());
                org.omg.CORBA.INTERNAL ex = new org.omg.CORBA.INTERNAL(
                    ExceptionMinorCodes.INTERNAL, 
                    org.omg.CORBA.CompletionStatus.COMPLETED_NO);
                CDMW_LOG_FUNCTION_EXCEPTION(logger, ex);
                throw ex;
            } catch (org.omg.CORBA.SystemException se) {
                CDMW_ERROR(logger, 
                    "System exception raised during event channel creation: "+se,
                    new Throwable());
                CDMW_LOG_FUNCTION_EXCEPTION(logger, se);
                throw se;
            }
            
            try {
                publisherInfo.proxyConsumer = 
                    publisherInfo.channel.getPushConsumer(null);
            } catch (org.omg.CORBA.SystemException se) {
                CDMW_ERROR(logger, 
                    "System exception raised during publisher connection to event channel: "+se,
                    new Throwable());
                CDMW_LOG_FUNCTION_EXCEPTION(logger, se);
                throw se;
            }
    
            // publisher info completed, store it in map
            this.publishers.put(publisherName, publisherInfo);
        } else {
            CDMW_ERROR(logger, 
                "Publisher " + publisherName + " already declared", 
                new Throwable());
            AlreadyDoneException ex = new AlreadyDoneException();
            CDMW_LOG_FUNCTION_EXCEPTION(logger, ex);
            throw ex;
        }

    }

    /**
     * Returns an event consumer stored in PublisherMap at ck
     * 
     * @param publisher_name   The publisher name.
     * @param ck               The cookie return at consumer subscription.
     *
     * @return a reference on the target consumer
     * @exception org.omg.Components.InvalidConnection if the connection does'nt exist
     */
    protected org.omg.Components.EventConsumerBase get_consumer(
        String publisherName,
        org.omg.Components.Cookie ck)
        throws org.omg.Components.InvalidName, org.omg.Components.InvalidConnection  
    {
        CDMW_LOG_FUNCTION(logger, "publisherName="+publisherName);

        if (ck == null) {
            CDMW_ERROR(logger, "Cookie is null", new Throwable());
            org.omg.CORBA.BAD_PARAM ex = new org.omg.CORBA.BAD_PARAM(
                ExceptionMinorCodes.BAD_PARAMNilObjectReference, 
                org.omg.CORBA.CompletionStatus.COMPLETED_NO);
            CDMW_LOG_FUNCTION_EXCEPTION(logger, ex);
            throw ex;
        }
    
        // find publisher
        org.omg.Components.EventConsumerBase result = null;
        PublisherInfo pInfo = (PublisherInfo) this.publishers.get(publisherName);
        
        if (pInfo != null) {
            // publisherName exists
            
            // find subscription
            SubscriberInfo sInfo = (SubscriberInfo) pInfo.subscriptions.get(ck);
            
            if (sInfo != null) {
                result = sInfo.ccmConsumer;
            } else {
                CDMW_ERROR(logger, "No subscriber found for Cookie", new Throwable());
                org.omg.Components.InvalidConnection ex = 
                    new org.omg.Components.InvalidConnection();
                CDMW_LOG_FUNCTION_EXCEPTION(logger, ex);
                throw ex;
            }
        } else {
            CDMW_ERROR(logger, 
                "Publisher " + publisherName + " not found", 
                new Throwable());
            org.omg.Components.InvalidName ex = 
                new org.omg.Components.InvalidName();
            CDMW_LOG_FUNCTION_EXCEPTION(logger, ex);
            throw ex;
        }
    
        return result;

    }

    /**
     * Calls ccm_passivate and ccm_remove on executor. 
     * This function shall be implemented by the well typed class.
     * 
     * @return void
     * @exception Any CORBA Exception.
     */
    protected abstract void advise_executor_of_remove_component();

    /**
     * Purpose:
     * <p>
     *   return a group reference if the current request is invoked on a FT group reference.
     *   Otherwise return the same reference.
     * 
     *@param objRef       The facet/consumer/component reference.
     *
     *@return  The reference to be exported to clients
     */
    org.omg.CORBA.Object get_object_ref(org.omg.CORBA.Object objRef)  
      throws org.omg.CORBA.SystemException
    {
        CDMW_LOG_FUNCTION(logger, "");
        org.omg.CORBA.Object result = objRef._duplicate();
        return result;
    }

    /**
     * Purpose:
     * <p>
     *   Call is_a() with a timeout. The system property TYPECHECKING_TIMEOUT
     *   defines the duration of the timeout.
     * 
     *@param repId       The repository ID.
     *@param obj          The object which has its type checked.
     *@param portName    The name of the port for the type checking.
     *@exception Components.InvalidConnection if is_a() has failed or the timeout has expired or the object doesn't exist.
     */ 
    protected
        void
        checkType(String repId, org.omg.CORBA.Object obj, String portName)
        throws org.omg.Components.InvalidConnection, org.omg.CORBA.SystemException
    {
        CDMW_LOG_FUNCTION(logger, "");
        org.omg.CORBA.Object objCopy =  null;

        // Add a timeout 
        try{
            CDMW_DEBUG(logger, "setTimeoutPolicy");
            objCopy = context.setTimeoutPolicy(typecheckingTimeoutInMs, obj);
        } catch( org.omg.CORBA.PolicyError pe) {
            CDMW_ERROR(logger, 
                       "Cannot check the connection type. Setting of the timeout failed for is_a() call. "
                       + "Port name: " + portName
                       + " - Rep ID: " + repId 
                       + ". Exception received : "
                       + pe);
            org.omg.Components.InvalidConnection ex =
                new org.omg.Components.InvalidConnection();
            CDMW_LOG_FUNCTION_EXCEPTION(logger, ex);
            throw ex;
        }
        try{
            CDMW_DEBUG(logger, "is_a"); 
            if(!objCopy._is_a(repId))
                {
                    CDMW_ERROR(logger, 
                               "Object for connection to port " + portName +
                               " is not a " + repId);
                    org.omg.Components.InvalidConnection ex =
                        new org.omg.Components.InvalidConnection();
                    CDMW_LOG_FUNCTION_EXCEPTION(logger, ex);
                    throw ex;
                }
        } catch(org.omg.CORBA.TIMEOUT t) {
            CDMW_ERROR(logger,
                       "Cannot check the connection type because the timeout has expired, " 
                       + typecheckingTimeoutInMs
                       + " ms have elapsed : " 
                       + t);
            org.omg.Components.InvalidConnection ex =
                new org.omg.Components.InvalidConnection();
            CDMW_LOG_FUNCTION_EXCEPTION(logger, ex);
            throw ex;
        } catch(org.omg.CORBA.OBJECT_NOT_EXIST ne) {
            CDMW_ERROR(logger,
                       "Cannot check the connection type. CORBA.OBJECT_NOT_EXIST exception received: " 
                       + ne);
            org.omg.Components.InvalidConnection ex =
                new org.omg.Components.InvalidConnection();
            CDMW_LOG_FUNCTION_EXCEPTION(logger, ex);
            throw ex;
        } catch(org.omg.CORBA.SystemException se) {
            CDMW_ERROR(logger,
                       "Cannot check the connection type beacuse a CORBA.SystemException was received: "
                       +se);
            org.omg.CORBA.TRANSIENT ex = new org.omg.CORBA.TRANSIENT(
                ExceptionMinorCodes.TRANSIENT, 
                org.omg.CORBA.CompletionStatus.COMPLETED_NO);
            CDMW_LOG_FUNCTION_EXCEPTION(logger, ex);
            throw ex;
        }
    }

    /**
     * Purpose:
     * <p>
     *   Call narrow with a timeout on CosEventComm.PushConsumer. The system property TYPECHECKING_TIMEOUT
     *   defines the duration of the timeout.
     * 
     *@param obj          The object which has its type checked.
     *@return  The result of narrow on CosEventComm::PushConsumer.
     *@exception Components.InvalidConnection if narrow has failed or the timeout has expired or the object doesn't exist.
     */ 
    protected
        org.omg.CosEventComm.PushConsumer
        narrowCosPushConsumer(org.omg.CORBA.Object obj)
        throws org.omg.Components.InvalidConnection, org.omg.CORBA.SystemException
    {
        CDMW_LOG_FUNCTION(logger, "");
        org.omg.CORBA.Object objCopy =  null;

        // Add a timeout 
        try{
            CDMW_DEBUG(logger, "setTimeoutPolicy");
            objCopy = context.setTimeoutPolicy(typecheckingTimeoutInMs, obj);
        } catch( org.omg.CORBA.PolicyError pe) {
            CDMW_ERROR(logger, 
                       "Cannot perform narrow with a timeout on CosEventComm.PushConsumer. The setting of the timeout has failed : "
                       + pe);
            org.omg.Components.InvalidConnection ex =
                new org.omg.Components.InvalidConnection();
            CDMW_LOG_FUNCTION_EXCEPTION(logger, ex);
            throw ex;
        }
        try{
            CDMW_DEBUG(logger, "narrow with timeout on CosEventComm.PushConsumer"); 
            org.omg.CosEventComm.PushConsumer pushConsumer = org.omg.CosEventComm.PushConsumerHelper.narrow(objCopy);

            // if obj is not a CosEventComm.PushConsumer, nil is returned
            // Therefore return nil and not a stub on CosEventComm.PushConsumer !
            if(pushConsumer != null) {
                // Don't return a stub with our timeout
                pushConsumer = org.omg.CosEventComm.PushConsumerHelper.unchecked_narrow(obj);
            }

            return pushConsumer;

        } catch(org.omg.CORBA.TIMEOUT t) {
            CDMW_ERROR(logger,
                       "Cannot perform narrow with a timeout on CosEventComm.PushConsumer because the timeout has expired, " 
                       + typecheckingTimeoutInMs
                       + " ms have elapsed : " 
                       + t);
            org.omg.Components.InvalidConnection ex =
                new org.omg.Components.InvalidConnection();
            CDMW_LOG_FUNCTION_EXCEPTION(logger, ex);
            throw ex;
        } catch(org.omg.CORBA.OBJECT_NOT_EXIST ne) {
            CDMW_ERROR(logger,
                       "Cannot perform narrow with a timeout on CosEventComm.PushConsumer. CORBA.OBJECT_NOT_EXIST exception received: " 
                       + ne);
            org.omg.Components.InvalidConnection ex =
                new org.omg.Components.InvalidConnection();
            CDMW_LOG_FUNCTION_EXCEPTION(logger, ex);
            throw ex;
        } catch(org.omg.CORBA.SystemException se) {
            CDMW_ERROR(logger,
                       "Cannot perform narrow with a timeout on CosEventComm.PushConsumer. CORBA.SystemException was received: "
                       +se);
            org.omg.CORBA.TRANSIENT ex = new org.omg.CORBA.TRANSIENT(
                ExceptionMinorCodes.TRANSIENT, 
                org.omg.CORBA.CompletionStatus.COMPLETED_NO);
            CDMW_LOG_FUNCTION_EXCEPTION(logger, ex);
            throw ex;
        }
    }

    /**
     * Purpose:
     * <p>
     *   Call narrow with a timeout on CdmwCcmCif.PushConsumer. The system property TYPECHECKING_TIMEOUT
     *   defines the duration of the timeout.
     * 
     *@param obj          The object which has its type checked.
     *@return  The result of narrow on CdmwCcmCif.PushConsumer.
     *@exception Components.InvalidConnection if narrow has failed or the timeout has expired or the object doesn't exist.
     */ 
    protected
        com.thalesgroup.CdmwCcmCif.PushConsumer
        narrowCdmwPushConsumer(org.omg.CORBA.Object obj)
        throws org.omg.Components.InvalidConnection, org.omg.CORBA.SystemException
    {
        CDMW_LOG_FUNCTION(logger, "");
        org.omg.CORBA.Object objCopy =  null;

        // Add a timeout 
        try{
            CDMW_DEBUG(logger, "setTimeoutPolicy");
            objCopy = context.setTimeoutPolicy(typecheckingTimeoutInMs, obj);
        } catch( org.omg.CORBA.PolicyError pe) {
            CDMW_ERROR(logger, 
                       "Cannot perform narrow with a timeout on CdmwCcmCif.PushConsumer. The setting of the timeout has failed : "
                       + pe);
            org.omg.Components.InvalidConnection ex =
                new org.omg.Components.InvalidConnection();
            CDMW_LOG_FUNCTION_EXCEPTION(logger, ex);
            throw ex;
        }
        try{
            CDMW_DEBUG(logger, "narrow with timeout on CdmwCcmCif.PushConsumer"); 
            com.thalesgroup.CdmwCcmCif.PushConsumer pushConsumer = com.thalesgroup.CdmwCcmCif.PushConsumerHelper.narrow(objCopy);
            
            // if obj is not a CdmwCcmCif.PushConsumer, nil is returned
            // Therefore return nil and not a stub on CdmwCcmCif.PushConsumer !
            if(pushConsumer != null) {
                // Don't return a stub with our timeout
                pushConsumer = com.thalesgroup.CdmwCcmCif.PushConsumerHelper.unchecked_narrow(obj);
            }

            return pushConsumer;

        } catch(org.omg.CORBA.TIMEOUT t) {
            CDMW_ERROR(logger,
                       "Cannot perform narrow with a timeout on CdmwCcmCif.PushConsumer because the timeout has expired, " 
                       + typecheckingTimeoutInMs
                       + " ms have elapsed : " 
                       + t);
            org.omg.Components.InvalidConnection ex =
                new org.omg.Components.InvalidConnection();
            CDMW_LOG_FUNCTION_EXCEPTION(logger, ex);
            throw ex;
        } catch(org.omg.CORBA.OBJECT_NOT_EXIST ne) {
            CDMW_ERROR(logger,
                       "Cannot perform narrow with a timeout on CdmwCcmCif.PushConsumer. CORBA.OBJECT_NOT_EXIST exception received: " 
                       + ne);
            org.omg.Components.InvalidConnection ex =
                new org.omg.Components.InvalidConnection();
            CDMW_LOG_FUNCTION_EXCEPTION(logger, ex);
            throw ex;
        } catch(org.omg.CORBA.SystemException se) {
            CDMW_ERROR(logger,
                       "Cannot perform narrow with a timeout on CdmwCcmCif.PushConsumer. CORBA.SystemException was received: "
                       +se);
            org.omg.CORBA.TRANSIENT ex = new org.omg.CORBA.TRANSIENT(
                ExceptionMinorCodes.TRANSIENT, 
                org.omg.CORBA.CompletionStatus.COMPLETED_NO);
            CDMW_LOG_FUNCTION_EXCEPTION(logger, ex);
            throw ex;
        }
    }

    /**
     * Purpose:
     * <p>
     *   unregister the component from the local naming context if registered
     * 
     *@return void
     *@exception Any CORBA Exception.
     */ 
    protected void unregister_component()
    {
        CDMW_LOG_FUNCTION(logger, "");

        // unregister component from the local naming context
        if (this.objectRegistration != null) {
            try {
                org.omg.Components.ConfigValue[] config =
                    this.context.getConfigValues(componentOid);

                // unregister component from the local naming context
                org.omg.CORBA.Any componentNameAny =
                    cdmw.ccm.common.CCMUtils.ReadConfigValue(
                        com.thalesgroup.CdmwDeployment.COMPONENT_NAME.value,
                        config);
                
                if (componentNameAny != null) {
                    java.lang.String componentName;
                    componentName = componentNameAny.extract_string();

                    this.objectRegistration.unregister_object(componentName);
                    CDMW_INFO(logger,
                              "CCMHome_impl unregister_component: name=" + componentName
                              + " - comp_oid=" + componentOid,
                              new Throwable());
                }
                else {
                    CDMW_WARN(logger,
                              "CCMHome_impl unregister_component: No component"
                              + " name defined for component comp_oid="
                              + componentOid,
                              new Throwable());
                }
            }

            catch (cdmw.common.NotFoundException e) {
                // no config values defined for this component, OK
            }
            catch (org.omg.CosNaming.NamingContextPackage.NotFound e) {
                // No local naming context?
                CDMW_WARN(logger,
                          "CCMObject_impl unregister_component: naming context"
                          + " not found for component comp_oid="
                          + componentOid,
                          new Throwable());
            }
            catch (org.omg.CosNaming.NamingContextPackage.InvalidName e) {
                CDMW_ERROR(logger,
                           "CCMObject_impl unregister_component: Invalid name "
                           + " for component comp_oid=" + componentOid,
                           new Throwable());
            }
            catch (org.omg.CORBA.SystemException e) {
                CDMW_ERROR(logger,
                           "CCMHome_impl unregister_component: "
                           + "System exception raised when unregistering "
                           + "component comp_oid=" + componentOid + ": "+ e,
                           new Throwable());
            }
        }
    }


   /**
    * Purpose:
    * <p>
    *   unregister the component from the recording service if registered
    * 
    *@return void
    *@exception Any CORBA Exception.
    */ 
   protected void unregister_from_recording()
           throws org.omg.Components.RemoveFailure
   {  
      CDMW_LOG_FUNCTION(logger, "");

        // get ConfigValues for the component
        org.omg.Components.ConfigValue[] config;
        try {
            config = this.context.getConfigValues(componentOid);
        }
        catch(cdmw.common.NotFoundException e)
        {
            // no config values matched to the oid
            CDMW_WARN(logger, 
                "No config values matched to the oid"
                + " in the map of the context ( oid = " + componentOid + "). "
                + e.getMessage(),
                new Throwable());
            // nothing to do
            return;
        }
            
        // check if Recording ConfigValue is set
        boolean recordingFlag = false;
        org.omg.CORBA.AnyHolder recordingFlagAny = new org.omg.CORBA.AnyHolder();
        if (cdmw.ccm.common.CCMUtils.ReadConfigValue(
                com.thalesgroup.CdmwDeployment.RECORDING.value, 
                config, 
                recordingFlagAny))
        {
            try {
                recordingFlag = recordingFlagAny.value.extract_boolean();
            } catch (org.omg.CORBA.BAD_OPERATION e) {
                CDMW_ERROR(logger, 
                    "Recording ConfigValue is not a CORBA::Boolean",
                    new Throwable());
                org.omg.Components.RemoveFailure ex = 
                    new org.omg.Components.RemoveFailure(
                        WRONG_RECORDING_CONFIG_VALUES.value);
                CDMW_LOG_FUNCTION_EXCEPTION(logger, ex);
                throw ex;
            }
            
        }

        if (! recordingFlag) {
            CDMW_DEBUG(logger, 
                "No recording required for this component. Recording ConfigValue"
                + " set to false. (oid = " + componentOid + "). ",
                new Throwable());
            // nothing to do
            return;
        }

        // get CdmwRecording::ContainerManager object
        com.thalesgroup.CdmwRecording.ContainerManager recordingMgr = null;
        try {
          org.omg.CORBA.Object o = 
          this.context.resolve_initial_references(
              REC_CONTAINER_MNGR_ID.value);
            
          recordingMgr = 
            com.thalesgroup.CdmwRecording.ContainerManagerHelper.narrow(o);
                
        } catch (com.thalesgroup.CdmwCcmContainer.CCM2ContextPackage.InvalidName e) {
            CDMW_WARN(logger, 
                REC_CONTAINER_MNGR_ID.value + 
                " object is not registered as initial reference",
                new Throwable());
            org.omg.Components.RemoveFailure ex = 
                new org.omg.Components.RemoveFailure(
                    RECORDING_DAEMON_ERROR.value);
            CDMW_LOG_FUNCTION_EXCEPTION(logger, ex);
            throw ex;            
        } catch (org.omg.CORBA.BAD_PARAM e) {
            CDMW_ERROR(logger, 
                REC_CONTAINER_MNGR_ID.value + 
                " object is not a CdmwRecording::ContainerManager",
                new Throwable());
            org.omg.Components.RemoveFailure ex = 
                new org.omg.Components.RemoveFailure(
                    RECORDING_DAEMON_ERROR.value);
            CDMW_LOG_FUNCTION_EXCEPTION(logger, ex);
            throw ex;            
        }
                               
        try {
            // unregister component object, facets and consumers from recording
            int listSize = 1;
            if (!isMonolithic) {
                listSize += facets.size() + consumers.size();
            }
            org.omg.CORBA.Object[] list = new org.omg.CORBA.Object[listSize];

            CDMW_INFO(logger, "Unregister a component from Recording service - comp_oid=" + componentOid);
            int nbObj = 0;
            list[nbObj++] = compRef;
                  
            if (!isMonolithic) {
                // unregister all facets if not monolithic
                CDMW_DEBUG(logger,
                    " - unregister " + facets.size() + " facets from recording - comp_oid=" + componentOid);
                java.util.Enumeration e = this.facets.elements();
                while(e.hasMoreElements()) {
                    FacetInfo fInfo = (FacetInfo) e.nextElement();
                    list[nbObj++] = fInfo.objRef;
                }

                // remove all consumers if not monolithic
                CDMW_DEBUG(logger,
                    " - unregister " + consumers.size() + " consumers from recording - comp_oid=" + componentOid);
                e = this.consumers.elements();
                while (e.hasMoreElements()) {
                    ConsumerInfo cInfo = (ConsumerInfo) e.nextElement();
                    list[nbObj++] = cInfo.consumer;
                }
            }

            recordingMgr.discard_objects(list);
                    
        } catch (org.omg.CORBA.SystemException e) {
            CDMW_ERROR(logger,
                "Error registering object in Recording service : " + e,
                new Throwable());
            org.omg.Components.RemoveFailure ex = 
                new org.omg.Components.RemoveFailure(
                    RECORDING_DAEMON_ERROR.value);
            CDMW_LOG_FUNCTION_EXCEPTION(logger, ex);
            throw ex;            
        }
          
    }

}

