/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO 
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2009. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

package cdmw.ccm.cif;

import java.util.*;

import org.omg.CORBA.Any;
import org.omg.Components.*;
import org.omg.CosEventChannelAdmin.EventChannel;

import cdmw.common.Assert;
import cdmw.orbsupport.ExceptionMinorCodes;
import cdmw.commonsvcs.naming.NamingInterface;
import cdmw.ccm.common.CCMLoggers;
import static cdmw.logging.Logger.*;
import static cdmw.logging.FunctionLogger.*;

import com.thalesgroup.CdmwCcmContainer.CCM2Context;
import com.thalesgroup.CdmwCcmContainer.NoChannel;
import com.thalesgroup.CdmwCcmContainer.Session2ContextHelper;
import com.thalesgroup.CdmwCcmContainer.CCM2ContextPackage.InvalidName;
import com.thalesgroup.CdmwCcmContainer.Session2Context;
import com.thalesgroup.CdmwCcmCif.UsesPortTimeout;
import com.thalesgroup.CdmwCcmCif.UsesPortTimeoutsHelper;
import com.thalesgroup.CdmwDeployment.WRONG_RECORDING_CONFIG_VALUES;
import com.thalesgroup.CdmwDeployment.RECORDING_DAEMON_ERROR;
import com.thalesgroup.CdmwRecording.REC_CONTAINER_MNGR_ID;

/**
 * This class is the base class for ccmcif context embedding specific CCM 
 * context.
 */
public abstract class Context {

    private static final String PROPERTY_RECORD_USING_COMPONENT_NAME =
        "Cdmw.CCMContainer.Recording.UseComponentName";
    
    
    private CCM2Context ccmContext;

    private Map<String,ConfigValue[]> configValues = 
        new HashMap<String,ConfigValue[]>(); // map of config values

    /**
     * Purpose:
     * <p> Constructor.
     *@param ctx encapsulated context. Each operation of this class are delegated
     * to this context.
     *
     */
    public Context(CCM2Context ctx) {
        CDMW_LOG_FUNCTION(CCMLoggers.GetCcmcifLogger(), "");
        this.ccmContext = ctx;
        Assert.check(ctx != null);
    }

    /**
     * Purpose:
     * <p> This operation returns the encapsulated ccm context.
     */
     public CCM2Context get_ccm_context()
     {
          CDMW_LOG_FUNCTION(CCMLoggers.GetCcmcifLogger(), "");
          return ccmContext;
     }


    //
    // IDL:thalesgroup.com/CdmwCcmContainer/CCM2Context/resolve_initial_references:1.0
    //
    /**
     * Purpose: Resolve Initial References
     *
     *@exception  InvalidName raised if the name is invalid
     *@exception  CORBA system exception 
     *@return A initial reference to a identifier 
     *
     */
    public org.omg.CORBA.Object resolve_initial_references(String identifier)
        throws InvalidName, org.omg.CORBA.SystemException 
    {
        CDMW_LOG_FUNCTION(CCMLoggers.GetCcmcifLogger(), 
            "identifier="+identifier);
        return this.ccmContext.resolve_initial_references(identifier);
    }
    
     //
    // IDL:omg.org/Components/CCM2Context/req_passivate:1.0
    //
    /**
     * The reqPassivate operation is used by the component to inform the
     * container that it wishes to be passivated when its current operation
     * completes. To be valid, the component must have a servant lifetime policy
     * of component or container. If not, the PolicyMismatch exception shall be
     * raised.
     * @throws PolicyMismatch
     */

    public abstract void req_passivate() throws org.omg.CORBA.SystemException, PolicyMismatch;

    //
    // IDL:omg.org/Components/CCMContext/get_CCM_home:1.0
    //
    /**
     * Purpose:
     * <p>The getCCMHome operation is used to obtain a reference to the home 
     * interface.
     *
     *@return A reference to the component's home
     *
     */
    public CCMHome get_CCM_home() throws org.omg.CORBA.SystemException {
        CDMW_LOG_FUNCTION(CCMLoggers.GetCcmcifLogger(), "");
        return this.ccmContext.get_CCM_home();
    }

    //
    // IDL:thalesgroup.com/CdmwCcmContainer/CCM2Context/obtain_event_channel_for_emitter:1.0
    //
    /**
     * Purpose:
     * <p>Obtain a Event Channel For Emitter 
     * 
     *@exception  CORBA system exception 
     *@return A Event Channel 
     *
     */
    public EventChannel obtain_event_channel_for_emitter(PortDescription port)
        throws org.omg.CORBA.SystemException, NoChannel 
    {
        CDMW_LOG_FUNCTION(CCMLoggers.GetCcmcifLogger(), "");
        return this.ccmContext.obtain_event_channel_for_emitter(port);
    }

    //
    // IDL:thalesgroup.com/CdmwCcmContainer/CCM2Context/create_event_channel_for_publisher:1.0
    //
    /**
     * Purpose:
     * <p>Create a Event Channel For a Publisher 
     * 
     *@exception  CORBA system exception 
     *@exception  CreateFailure raised if a event channel could not be created 
     *@return A Event Channel 
     *
     */
    public EventChannel create_event_channel_for_publisher(PortDescription port)
        throws org.omg.CORBA.SystemException, CreateFailure 
    {
        CDMW_LOG_FUNCTION(CCMLoggers.GetCcmcifLogger(), "");
        return this.ccmContext.create_event_channel_for_publisher(port);
    }


     //
    // IDL:omg.org/Components/Session2Context/create_ref:1.0
    //
    /**
     * Purpose:
     * <p>The create_ref operation is used to create a reference to be exported
     * to clients to invoke operations. The RepositoryId must match the 
     * RepositoryId of the component itself, one of its bases, one of its 
     * supported interfaces, or one of its facets. This operation shall be 
     * implemented by specialised context.
     *
     *@param repid Identifies the RepositoryId associated with the interface for
     * which a reference is being created.
     *
     *@return A newly created object reference.
     */

    public abstract org.omg.CORBA.Object create_ref(String rep_id) 
          throws org.omg.CORBA.SystemException;

    //
    // IDL:omg.org/Components/Session2Context/create_ref_from_oid:1.0
    //
    /**
     * Purpose:
     * <p>The create_ref_from_oid_str operation is used to create a reference to be
     * exported to clients which includes information provided by the component
     * which it can use on subsequent operation requests. This operation shall
     * be implemented by specialised context.
     *
     *@param oid Identifies the ObjectIdSeq to be encapsulated in the reference.
     *@param repid Identifies the RepositoryId associated with the interface for
     * which a reference is being created.
     *
     *@return A newly created object reference.
     */
     public abstract org.omg.CORBA.Object create_ref_from_oid_str( String oid, String rep_id) 
          throws org.omg.CORBA.SystemException;
 
    //
    // IDL:omg.org/Components/Session2Context/get_oid_from_ref:1.0
    //
    /**
     * Purpose:
     * <p>The get_oid_from_ref operation is used by the component to extract the
     * oid encapsulated in the reference. This operation must be called within an
     * operation invocation. This operation shall be implemented by specialised
     * context.
     *
     *@param objref Specifies the reference which contains the oid.
     *@return the oid.
     *
     *@exception Components.IllegalState Raise if the operation is not called 
     * within an operation invocation.
     *@exception Components.BadComponentReference Raised with the 
     * WRONG_CONTAINER minor code if the reference was not created by this 
     * container.
     */
    public abstract String get_oid_str_from_ref( org.omg.CORBA.Object comp_ref) 
      throws org.omg.CORBA.SystemException, 
             org.omg.Components.IllegalState,
             org.omg.Components.BadComponentReference;

    //
    // IDL:omg.org/Components/Session2Context/get_oid_from_ref:1.0
    //
    /**
     * Purpose:
     * <p>The get_primary_key operation return the primary key when component 
     * is an entity component. This operation shall be implemented by specialised
     * context.
     * *@exception Components.NoKeyAvailable Raise if the component is not an
     * entity one.
     */
    public abstract PrimaryKeyBase get_primary_key() 
      throws NoKeyAvailable, org.omg.CORBA.SystemException;


    //
    // IDL:omg.org/Components/SessionContext/get_CCM_object:1.0
    //
    /**
     * The get_CCM_object operation is used to get the reference used to invoke 
     * the component. For basic components, this will always be the component 
     * reference. For extended components, this will be a specific facet 
     * reference. If this operation is issued outside of the scope of a callback
     * operation, the IllegalState exception is returned. This operation shall 
     * be implemented by specialised context.
     *
     *@exception  IllegalState raised
     */
    public abstract org.omg.CORBA.Object get_CCM_object() 
      throws org.omg.CORBA.SystemException, org.omg.Components.IllegalState;

    /**
     * Purpose:
     * <p>The setConfigValues operation is used to store config values. 
     * The config values are added in a map. Component's OID are used as map's 
     * index and ConfigValues as map's members.
     *
     *@param compOid Specifies the component's OID.
     *@param config Specifies a reference on a sequence of config values.
     *@exception AlreadyExistException raised if the oid exists already in
     *the map.
     *
     */
    public void setConfigValues(String compOid, ConfigValue[] config)
        throws cdmw.common.AlreadyExistException 
    {
        CDMW_LOG_FUNCTION(CCMLoggers.GetCcmcifLogger(), "");
        if (this.configValues.containsKey(compOid))
            throw new cdmw.common.AlreadyExistException();

        // create a copy of config values to be stored in the map
        this.configValues.put(compOid, config);

        CDMW_DEBUG(CCMLoggers.GetCcmcifLogger(),
            "insertion of config values in the map of the context. "
            + "Key member of the map : oid = " + compOid,
            new Throwable());
    }

    /**
     * Purpose:
     * <p>The getConfigValues operation is used to obtain config values. 
     * The config values are added in a map. Component's OID are used as map's 
     * index and ConfigValues as map's members.
     *
     *@param compOid Specifies the component's OID.
     *@return Components.ConfigValues the config values store in a Map.
     *
     */
    public ConfigValue[] getConfigValues(String compOid)
        throws cdmw.common.NotFoundException 
    {
        CDMW_LOG_FUNCTION(CCMLoggers.GetCcmcifLogger(), "compOid="+compOid);
        //List<ConfigValue> configValues = new ArrayList<ConfigValue>();
        ConfigValue[] rtnConfigValues = this.configValues.get(compOid);

        if (rtnConfigValues != null) {
            CDMW_DEBUG(CCMLoggers.GetCcmcifLogger(),
                "Context get_config_values : config values found.",
                new Throwable());
            return rtnConfigValues;
        } else {
            CDMW_ERROR(CCMLoggers.GetCcmcifLogger(),
                "Context get_config_values : config values not found.",
                new Throwable());
            cdmw.common.NotFoundException ex = new cdmw.common.NotFoundException();
            CDMW_LOG_FUNCTION_EXCEPTION(CCMLoggers.GetCcmcifLogger(), ex);
            throw ex;
        }
    }

    /**
     * Purpose:
     * <p>The setTimeoutPolicy operation is used to set a policy to use the timeout
     * mecanism provided by the ORB.
     *
     *@param timeout the timeout specified for a receptacle in milliseconds
     *@param obj CORBA object which has its policy set. 
     *
     *@exception org.omg.CORBA.PolicyError raised if the policy is not set like expected.
     *
     */
    public org.omg.CORBA.Object setTimeoutPolicy(long timeoutInMs,
                                                 org.omg.CORBA.Object obj) 
        throws org.omg.CORBA.SystemException,
               org.omg.CORBA.PolicyError   
    {
        CDMW_LOG_FUNCTION(CCMLoggers.GetCcmcifLogger(), 
            "timeoutInMs="+timeoutInMs);
        /* WARNING : TimeT is expressed in units of 100 nano seconds */
        long timeout = ((long) (timeoutInMs * 10000));

        CDMW_DEBUG(CCMLoggers.GetCcmcifLogger(),
            "Context set_timeout_policy: timeout = " + timeout + " ( * 100 ns)",
            new Throwable());

        // use ORBSingleton to create Any
        org.omg.CORBA.Any timeoutAny = org.omg.CORBA.ORB.init().create_any();
        timeoutAny.insert_ulonglong(timeout);

        org.omg.CORBA.Policy[] timeoutPolicies = 
            new org.omg.CORBA.Policy[1];

        /*
         * recover the orb reference
         */
        Session2Context session2context = Session2ContextHelper.narrow(this.ccmContext);

        timeoutPolicies[0] = session2context.create_policy(
            org.omg.Messaging.RELATIVE_RT_TIMEOUT_POLICY_TYPE.value, timeoutAny);
        /*
         * set policy at object level
         */
        org.omg.CORBA.Object objCopy = obj._set_policy_override(
            timeoutPolicies, org.omg.CORBA.SetOverrideType.ADD_OVERRIDE);

        timeoutPolicies[0].destroy();

        CDMW_DEBUG(CCMLoggers.GetCcmcifLogger(),
            "Context set_timeout_policy done.",
            new Throwable());

        return objCopy;

    }

    /**
     * Purpose:
     * <p>The applyPolicy operation is used to set a timeout on a
     * receptacle if necessary
     *
     *@param connection The connection object reference.
     *@param compOid The component oid used to create the facet reference
     *@param receptacleName The name of the receptacle
     *
     */
    public org.omg.CORBA.Object applyPolicy(org.omg.CORBA.Object connection,
                                            String compOid, 
                                            String receptacleName)
        throws org.omg.CORBA.SystemException 
    {
        CDMW_LOG_FUNCTION(CCMLoggers.GetCcmcifLogger(), 
            "compOid="+compOid+"  receptacleName="+receptacleName);
        org.omg.CORBA.Object toReturn = connection;
        ConfigValue[] configValues;

        CDMW_DEBUG(CCMLoggers.GetCcmcifLogger(),
            "Context apply_policy: receptacle name =" + receptacleName,
            new Throwable());

        /*
         * search the config values matched to the OID
         */
        try {

            Session2Context session2context = 
                Session2ContextHelper.narrow(this.ccmContext);

            configValues = this.getConfigValues(compOid);
            
            // Timeout on a receptacle in milliseconds
            long timeDurationInMs = 0;

            // default timeout found in the ConfigValues sequence (in
            // milliseconds)
            long defaultTimeout = 0;

            // flag which indicates whether the timeout matched to the
            // receptacle was
            // found or not
            boolean receptacleTimeoutFound = false;

            // flag which indicates whether the default timeout matched to the
            // recaptacle
            // was found or not
            boolean defaultTimeoutFound = false;

            String nameConfigValue = null;

            /*
             * search the timeout to apply on the receptacle
             */

            // Search in ConfigValues sequence
            for (ConfigValue c : configValues) {
                nameConfigValue = c.name;

                if (nameConfigValue.equals(
                       com.thalesgroup.CdmwDeployment.USES_PORT_TIMEOUTS.value)) 
                {
                    UsesPortTimeout[] usesPortTimeouts = 
                        UsesPortTimeoutsHelper.extract(c.value);

                    // Search in UsesPortTimeouts sequence
                    for (UsesPortTimeout upt : usesPortTimeouts) {
                        String usesName = upt.uses_name;

                        if (usesName.equals(receptacleName)) {
                            timeDurationInMs = upt.timeout;
                            receptacleTimeoutFound = true;
                            CDMW_DEBUG(CCMLoggers.GetCcmcifLogger(),
                                "Context apply_policy: timeout of "
                                + receptacleName + " found. timeout receptacle = "
                                + timeDurationInMs + " (ms)",
                                new Throwable());
                            break;
                        }
                        else {
                            if (usesName.equals("")) {
                                defaultTimeoutFound = true;
                                defaultTimeout = upt.timeout;
                                CDMW_DEBUG(CCMLoggers.GetCcmcifLogger(),
                                    "Context apply_policy: default timeout found. "
                                    + "default_timeout = " + defaultTimeout
                                    + " (ms)",
                                    new Throwable());
                            }
                        } // if, UsesPortTimemout structure
                    } // for, UsesPortTimeouts sequence
                    
                    break;
                } // if, ConfigValue valuetype
            } // for, ConfigValues sequence

            if (defaultTimeoutFound == true && receptacleTimeoutFound == false) {
                timeDurationInMs = defaultTimeout;
            }

            // Set the policy
            if (receptacleTimeoutFound == true || defaultTimeoutFound == true) {

                try {
                    toReturn = this.setTimeoutPolicy(timeDurationInMs, toReturn);
                }
                catch (org.omg.CORBA.PolicyError e) {
                    CDMW_ERROR(CCMLoggers.GetCcmcifLogger(),
                        "Context apply_policy : set_timeout_policy error! " + e,
                        new Throwable());
                    org.omg.CORBA.INTERNAL ex = new org.omg.CORBA.INTERNAL(
                        ExceptionMinorCodes.INTERNALCCMCIFError,
                        org.omg.CORBA.CompletionStatus.COMPLETED_NO);
                    CDMW_LOG_FUNCTION_EXCEPTION(CCMLoggers.GetCcmcifLogger(), ex);
                    throw ex;
                }
                catch (org.omg.CORBA.SystemException e) {
                    CDMW_ERROR(CCMLoggers.GetCcmcifLogger(),
                        "Context apply_policy : set_timeout_policy error! " +e,
                        new Throwable());
                    CDMW_LOG_FUNCTION_EXCEPTION(CCMLoggers.GetCcmcifLogger(), e);
                    throw e;
                }
            }
            else {
                CDMW_DEBUG(CCMLoggers.GetCcmcifLogger(),
                    "Context apply_policy: no default and specific timeout found "
                    + "for the receptacle :" + receptacleName,
                    new Throwable());
            }
        }
        catch (cdmw.common.NotFoundException e) {
            // no config values matched to the oid
            CDMW_DEBUG(CCMLoggers.GetCcmcifLogger(),
                "no config values matched to the oid"
                + " in the map of the context ( oid = " + compOid + "). "
                + e.getMessage(),
                new Throwable());
        }

        CDMW_DEBUG(CCMLoggers.GetCcmcifLogger(),
            "Context apply_policy done.",
            new Throwable());

        return toReturn;
    }


    /**
     * Purpose:
     * <p>Register a component's object (facet, consumer or the component
     * itself) to be recorded by RecordingService.
     * Registration is performed only if the component was created
     * with appropriate Recording ConfigValue.
     *
     *@param compOid The component's oid (for ConfigValues retrieval)
     *@param obj The object to be recorded
     *
     *@exception Components::CreateFailure if an error occurs during 
     * object registration
     */
    public void registerForRecording(String compOid,
                                     org.omg.CORBA.Object obj)
        throws org.omg.Components.CreateFailure
    {
        CDMW_LOG_FUNCTION(CCMLoggers.GetCcmcifLogger(), "compOid="+compOid);
        
        // get ConfigValues for the component
        ConfigValue[] config;
        try {
            config = this.getConfigValues(compOid);
        }
        catch(cdmw.common.NotFoundException e)
        {
            // no config values matched to the oid
            CDMW_DEBUG(CCMLoggers.GetCcmcifLogger(), 
                "no config values matched to the oid"
                + " in the map of the context ( oid = " + compOid + "). "
                + e.getMessage(),
                new Throwable());
                
            // nothing to do
            return;
        }  
            
        // check if Recording ConfigValue is set
        org.omg.CORBA.AnyHolder recordingFlagAny = new org.omg.CORBA.AnyHolder();
        if (cdmw.ccm.common.CCMUtils.ReadConfigValue(
                com.thalesgroup.CdmwDeployment.RECORDING.value, 
                config, 
                recordingFlagAny))
        {
            boolean recordingFlag;
            try {
                recordingFlag = recordingFlagAny.value.extract_boolean();
            } catch (org.omg.CORBA.BAD_OPERATION e) {
                CDMW_ERROR(CCMLoggers.GetCcmcifLogger(), 
                    "Recording ConfigValue is not a CORBA::Boolean",
                    new Throwable());
                org.omg.Components.CreateFailure ex = 
                    new org.omg.Components.CreateFailure(
                        WRONG_RECORDING_CONFIG_VALUES.value);
                CDMW_LOG_FUNCTION_EXCEPTION(CCMLoggers.GetCcmcifLogger(), ex);
                throw ex;
            }
            
            if (recordingFlag) {
                CDMW_DEBUG(CCMLoggers.GetCcmcifLogger(), 
                    "Register for Recording - comp_oid = " + compOid);
                // get CdmwRecording::ContainerManager object
                com.thalesgroup.CdmwRecording.ContainerManager recordingMgr = null;
                try {
                    org.omg.CORBA.Object o = 
                        this.ccmContext.resolve_initial_references(
                            REC_CONTAINER_MNGR_ID.value);
                    
                    recordingMgr = 
                        com.thalesgroup.CdmwRecording.ContainerManagerHelper.narrow(o);
                        
                } catch (InvalidName e) {
                    CDMW_ERROR(CCMLoggers.GetCcmcifLogger(), 
                        REC_CONTAINER_MNGR_ID.value + 
                        " object is not registered as initial reference",
                        new Throwable());
                    org.omg.Components.CreateFailure ex = 
                        new org.omg.Components.CreateFailure(
                            RECORDING_DAEMON_ERROR.value);
                    CDMW_LOG_FUNCTION_EXCEPTION(CCMLoggers.GetCcmcifLogger(), ex);
                    throw ex;            
                } catch (org.omg.CORBA.BAD_PARAM e) {
                    CDMW_ERROR(CCMLoggers.GetCcmcifLogger(), 
                        REC_CONTAINER_MNGR_ID.value + 
                        " object is not a CdmwRecording::ContainerManager",
                        new Throwable());
                    org.omg.Components.CreateFailure ex = 
                        new org.omg.Components.CreateFailure(
                            RECORDING_DAEMON_ERROR.value);
                    CDMW_LOG_FUNCTION_EXCEPTION(CCMLoggers.GetCcmcifLogger(), ex);
                    throw ex;            
                }
                
                // get server_id
                String serverId = this.ccmContext.server_id();
                
                // if SystemProperty PROPERTY_RECORD_USING_COMPONENT_NAME is "true"
                // find component name and add it to server_id for recording
                if (System.getProperty(PROPERTY_RECORD_USING_COMPONENT_NAME, "false").equals("true")) {
                    
                    // get COMPONENT_NAME
                    org.omg.CORBA.Any componentNameAny =
                        cdmw.ccm.common.CCMUtils.ReadConfigValue(
                            com.thalesgroup.CdmwDeployment.COMPONENT_NAME.value,
                            config);
                    
                    if (componentNameAny != null) {
                        java.lang.String componentName;
                        componentName = componentNameAny.extract_string();
                               
                        // ConfigValue found, add component name to server_id
                        try {
                            org.omg.CosNaming.NameComponent[] name =
                                NamingInterface.toName(serverId);
                            java.util.ArrayList<org.omg.CosNaming.NameComponent> nameAsList = 
                                new java.util.ArrayList<org.omg.CosNaming.NameComponent>(java.util.Arrays.asList(name));
                            org.omg.CosNaming.NameComponent comp = 
                                new org.omg.CosNaming.NameComponent();
                            comp.id = componentName;
                            comp.kind = "";
                            nameAsList.add(comp);
                            serverId = NamingInterface.toString(nameAsList.toArray(name));
                        } catch (cdmw.commonsvcs.naming.InvalidNameException e) {
                            // shouldn't happen
                            e.printStackTrace();
                            Assert.check(false);
                        }
                    }
                }
            
                // register object for recording
                CDMW_DEBUG(CCMLoggers.GetCcmcifLogger(), 
                    "Register object in Recording service with service_id="
                    + serverId,
                    new Throwable());
                try {
                    org.omg.CORBA.Object[] list = new org.omg.CORBA.Object[1];
                    list[0] = obj;
                    
                    recordingMgr.record_objects(serverId, list);
                    
                } catch (org.omg.CORBA.SystemException e) {
                    CDMW_ERROR(CCMLoggers.GetCcmcifLogger(),
                        "Error registering object in Recording service : " + e,
                        new Throwable());
                    org.omg.Components.CreateFailure ex = 
                        new org.omg.Components.CreateFailure(
                            RECORDING_DAEMON_ERROR.value);
                    CDMW_LOG_FUNCTION_EXCEPTION(CCMLoggers.GetCcmcifLogger(), ex);
                    throw ex;            
                }
                
            }
            else 
            {
                // recording_flag = false
                CDMW_DEBUG(CCMLoggers.GetCcmcifLogger(), 
                    "No recording required for this component " +
                    "(Recording ConfigValue set to false)",
                    new Throwable());
            }
        }
        else
        {
            // Recording ConfigValue not specified
            CDMW_DEBUG(CCMLoggers.GetCcmcifLogger(), 
                "No recording required for this component" +
                "(no Recording ConfigValue specified)",
                new Throwable());
        }
    }

    
}

