/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO 
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2009. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


package cdmw.ccm.cif;

import cdmw.common.Assert;
import cdmw.orbsupport.ExceptionMinorCodes;
import cdmw.lifecycle.LifeCycleSingleObject;
import org.omg.Components.ConfigValue;
import cdmw.ccm.common.CCMLoggers;
import static cdmw.logging.Logger.*;
import static cdmw.logging.FunctionLogger.*;
import java.lang.Throwable;
import com.thalesgroup.CdmwDeployment.WRONG_RECORDING_CONFIG_VALUES;

/**
 * This class is the base class for home embedded in a Service or Session 
 * container (using a SessionContext) which creates LB replicated components.
 */
public abstract class LBCCMHomeImpl
    extends CCMHomeImpl
{

    /**
     * LBGroupManager reference
     */
    private CdmwLB.LBGroupManager lbGroupManager = null;
    

    /**
     * Constructor
     * 
     * @param repId repId home repository ID
     * @param compRepId
     * @param deactivator
     * @param objectRegistration
     * @param config
     */
    public LBCCMHomeImpl(String repId, 
                         String compRepId, 
                         cdmw.lifecycle.ObjectDeactivator deactivator,
                         com.thalesgroup.CdmwLifeCycle.ObjectRegistration objectRegistration,
                         org.omg.Components.ConfigValue[] config)
    {
        super(repId, compRepId, deactivator, objectRegistration, config, true);
        CDMW_LOG_FUNCTION(CCMLoggers.GetCcmcifLogger(), 
            "repId="+repId+"  compRepId"+compRepId);
        
/****** PCR-1378: comment this code since the LoadBalancingStrategy ConfigValue
                  is useless at this time, and provokes an error. 
                  
        org.omg.CORBA.AnyHolder lbStrategyAny = new org.omg.CORBA.AnyHolder();
        boolean lbConfigValueFound = 
           cdmw.ccm.common.CCMUtils.ReadConfigValue(
               com.thalesgroup.CdmwDeployment.LOAD_BALANCING_STRATEGY.value, 
               config, 
               lbStrategyAny); 
    
        if (lbConfigValueFound) 
        {
            try 
            {
                String lbStrategy = lbStrategyAny.value.extract_string();
                
                // TODO: manage LB Strategy
                
            } catch (org.omg.CORBA.BAD_OPERATION e) {
                System.err.println("Cannot read LB Configuration values for LB home creation");
                throw new org.omg.CORBA.BAD_PARAM(
                    ExceptionMinorCodes.BAD_PARAMInvalidConfigValues, 
                    org.omg.CORBA.CompletionStatus.COMPLETED_NO);
            }
        } 
        else 
        {
            System.err.println("Expected ConfigValue " +
                com.thalesgroup.CdmwDeployment.LOAD_BALANCING_STRATEGY.value + 
                " not found");
            throw new org.omg.CORBA.BAD_PARAM(
                ExceptionMinorCodes.BAD_PARAMInvalidConfigValues, 
                org.omg.CORBA.CompletionStatus.COMPLETED_NO);
        }
******/
    }


    /**
     * This method is called by Container after Home creation.
     * 
     * @param ctxt Component context given by container.
     * @exception org.omg.Components.CreateFailure Any Component exception
     */
    public void set_context(
        com.thalesgroup.CdmwCcmContainer.CCM2Context ctxt)
        throws org.omg.Components.CreateFailure
    {
        CDMW_LOG_FUNCTION(CCMLoggers.GetCcmcifLogger(), "");
        super.set_context(ctxt);

        // Look for the LBGroupManager in the initial references.
        try {
            // get LBGroupManager reference
            org.omg.CORBA.Object obj = 
                this.context.resolve_initial_references("LBGroupManager");

            // try to narrow reference
            this.lbGroupManager = 
                CdmwLB.LBGroupManagerHelper.narrow(obj);
        }
        catch (com.thalesgroup.CdmwCcmContainer.CCM2ContextPackage.InvalidName e)
        {
            // LBGroupManager not found.
            // It should have been specified as ORBInitRef (non-FT LBGroupManager)
            // or found from Repository during CDMW_init (FT LBGroupManager)
            CDMW_ERROR(CCMLoggers.GetCcmcifLogger(), 
              "LBGroupManager not registered as initial reference!",
              new Throwable());
            org.omg.Components.CreateFailure ex = 
            new org.omg.Components.CreateFailure(
                com.thalesgroup.CdmwDeployment.LB_GROUP_MANAGER_ERROR.value);
            CDMW_LOG_FUNCTION_EXCEPTION(CCMLoggers.GetCcmcifLogger(), ex);
            throw ex;
        }
        catch (org.omg.CORBA.SystemException e)
        {
            // LBGroupManager was found, but an error occured contacting it.
            CDMW_ERROR(CCMLoggers.GetCcmcifLogger(), 
              "Exception contacting the LBGroupManager: " + e,
              new Throwable());
            org.omg.Components.CreateFailure ex = 
            new org.omg.Components.CreateFailure(
                com.thalesgroup.CdmwDeployment.LB_GROUP_MANAGER_ERROR.value);
            CDMW_LOG_FUNCTION_EXCEPTION(CCMLoggers.GetCcmcifLogger(), ex);
            throw ex;
        }
        
        if (this.lbGroupManager == null) {
            CDMW_ERROR(CCMLoggers.GetCcmcifLogger(), 
              "LBGroupManager is nil !!",
              new Throwable());
            org.omg.Components.CreateFailure ex = 
            new org.omg.Components.CreateFailure(
                com.thalesgroup.CdmwDeployment.LB_GROUP_MANAGER_ERROR.value);
            CDMW_LOG_FUNCTION_EXCEPTION(CCMLoggers.GetCcmcifLogger(), ex);
            throw ex;
        }
    }
            
    /**
     * This creates new component reference.
     * 
     * @return A reference on the new component.
     * @exception org.omg.Components.CreateFailure if application errors.
     */ 
    protected org.omg.CORBA.Object create_component_ref(
        org.omg.Components.ConfigValue[] config,
        org.omg.Components.EnterpriseComponent compExec) 
        throws org.omg.Components.CreateFailure 
    {
        CDMW_LOG_FUNCTION(CCMLoggers.GetCcmcifLogger(), "");
        checkIsRemoved(oid);
        Assert.check(this.context != null);
        
        // get LOAD_BALANCING_GROUP_NAME Config value
        org.omg.CORBA.AnyHolder lbGroupNameAny = new org.omg.CORBA.AnyHolder();
        if (! cdmw.ccm.common.CCMUtils.ReadConfigValue(
                com.thalesgroup.CdmwDeployment.LOAD_BALANCING_GROUP_NAME.value,
                config, 
                lbGroupNameAny))
        {
            CDMW_ERROR(CCMLoggers.GetCcmcifLogger(), 
              "Missing LB Configuration values for LB component creation: "
                + com.thalesgroup.CdmwDeployment.LOAD_BALANCING_GROUP_NAME.value
                + " is not specified",
              new Throwable());
            org.omg.Components.CreateFailure ex = 
            new org.omg.Components.CreateFailure(
                com.thalesgroup.CdmwDeployment.MISSING_LB_CONFIG_VALUES.value);
            CDMW_LOG_FUNCTION_EXCEPTION(CCMLoggers.GetCcmcifLogger(), ex);
            throw ex;
        }           
        
        // extract LOAD_BALANCING_GROUP_NAME Config value
        String lbGroupName;
        try 
        {
            lbGroupName = lbGroupNameAny.value.extract_string();
            
        } catch (org.omg.CORBA.BAD_OPERATION e) {
            CDMW_ERROR(CCMLoggers.GetCcmcifLogger(), 
                "Wrong LB Configuration values for LB component creation: "
                + com.thalesgroup.CdmwDeployment.LOAD_BALANCING_GROUP_NAME.value
                + " value is not a string.",
                new Throwable());
            org.omg.Components.CreateFailure ex = 
            new org.omg.Components.CreateFailure(
                com.thalesgroup.CdmwDeployment.WRONG_LB_CONFIG_VALUES.value);
            CDMW_LOG_FUNCTION_EXCEPTION(CCMLoggers.GetCcmcifLogger(), ex);
            throw ex;
        }

        // get group reference for LOAD_BALANCING_GROUP_NAME Config value
        org.omg.CORBA.Object lbGroupRef = getGroupForName(lbGroupName);
        
        // create component reference
        org.omg.CORBA.StringHolder comp_oid = 
            new org.omg.CORBA.StringHolder();
        org.omg.CORBA.Object to_return = 
            create_component_ref_i(comp_oid, compExec);

        // add reference to LB group
        try {
            org.omg.CosNaming.NameComponent[] location = getCurrentLBLocation();
            
            CDMW_INFO(CCMLoggers.GetCcmcifLogger(),
                "Add this component as member of LB group " + lbGroupName,
                new Throwable());
            lbGroupRef = lbGroupManager.add_member(lbGroupRef, 
                                                   location, 
                                                   to_return);
        }
        catch (org.omg.PortableGroup.ObjectGroupNotFound e) {
            CDMW_ERROR(CCMLoggers.GetCcmcifLogger(), 
                "LB group " + lbGroupName + " not found",
                new Throwable());
            org.omg.Components.CreateFailure ex = 
            new org.omg.Components.CreateFailure(
                com.thalesgroup.CdmwDeployment.LB_GROUP_MANAGER_ERROR.value);
            CDMW_LOG_FUNCTION_EXCEPTION(CCMLoggers.GetCcmcifLogger(), ex);
            throw ex;
        }
        catch (org.omg.PortableGroup.MemberAlreadyPresent e) {
            // Member has just been created ! this shouldn't happen
            CDMW_ERROR(CCMLoggers.GetCcmcifLogger(), 
                "Component already present in group!!",
                new Throwable());
            cdmw.common.Assert.check(false);
        }
        catch (org.omg.PortableGroup.ObjectNotAdded e) {
            CDMW_ERROR(CCMLoggers.GetCcmcifLogger(), 
                "LBGroupManager raised PortableGroup.ObjectNotAdded "
                + "adding component in LB group " + lbGroupName,
                new Throwable());
            org.omg.Components.CreateFailure ex = 
            new org.omg.Components.CreateFailure(
                com.thalesgroup.CdmwDeployment.LB_GROUP_MANAGER_ERROR.value);
            CDMW_LOG_FUNCTION_EXCEPTION(CCMLoggers.GetCcmcifLogger(), ex);
            throw ex;
        }
        catch (org.omg.CORBA.SystemException e) {
            CDMW_ERROR(CCMLoggers.GetCcmcifLogger(), 
                "Exception adding component in LB group " + lbGroupName
                + " : " + e,
                new Throwable());
            org.omg.Components.CreateFailure ex = 
            new org.omg.Components.CreateFailure(
                com.thalesgroup.CdmwDeployment.LB_GROUP_MANAGER_ERROR.value);
            CDMW_LOG_FUNCTION_EXCEPTION(CCMLoggers.GetCcmcifLogger(), ex);
            throw ex;
        }

        // get COMPONENT_NAME Config value
        org.omg.CORBA.Any componentNameAny =
            cdmw.ccm.common.CCMUtils.ReadConfigValue(
                com.thalesgroup.CdmwDeployment.COMPONENT_NAME.value,
                config);
        
        java.lang.String componentName = null;
        if (componentNameAny != null) {
            componentName = componentNameAny.extract_string();
        
            // notify Supervision of add_member
            this.notify_add_member(componentName);
        }
        else {
            CDMW_WARN(CCMLoggers.GetCcmcifLogger(),
                "No ComponentName defined for this component. " +
                "Cannot notify Supervision that add_member has been performed.",
                new Throwable());
       }

        // Setting realised to inject information about timeouts of receptacles.
        // That's the context of a home instance, which stores the config values.
        try {
            this.context.setConfigValues(comp_oid.value, config);
            CDMW_DEBUG(CCMLoggers.GetCcmcifLogger(), 
              "CCMHome_impl create_component_ref: length config values ="
              + config.length,
              new Throwable());
              
        } catch(cdmw.common.AlreadyExistException e) {
            CDMW_ERROR(CCMLoggers.GetCcmcifLogger(), 
              "CCMHome_impl create_component_ref: config values exist already!",
              new Throwable());
            org.omg.CORBA.INTERNAL ex = new org.omg.CORBA.INTERNAL(
                cdmw.orbsupport.ExceptionMinorCodes.INTERNALCCMCIFError,
                org.omg.CORBA.CompletionStatus.COMPLETED_NO);
            CDMW_LOG_FUNCTION_EXCEPTION(CCMLoggers.GetCcmcifLogger(), ex);
            throw ex;
        }


        register_component(config, to_return);
        
        // register component for recording (if Recording ConfigValue set to true)
        // NOTA: Failing to record the component for recording will only log
        //       a WARN message and not abort deployment of component.
        try {
           this.context.registerForRecording(comp_oid.value, to_return);
        }
        catch (org.omg.Components.CreateFailure e)
        {
           java.lang.String componentIdentification = 
              new java.lang.String(comp_oid.value);
           if ((componentName != null) && (componentName.length() > 0))
              componentIdentification = componentName;
           CDMW_ERROR(CCMLoggers.GetCcmcifLogger(), 
                      "Error registering component " + componentIdentification + 
                      " for recording: " +
                      (e.reason == WRONG_RECORDING_CONFIG_VALUES.value ?
                       "Wrong Recording ConfigValue" : "Recording Daemon error"),
                      new Throwable());
           CDMW_WARN(CCMLoggers.GetCcmcifLogger(), 
                     "Incoming requets to component '" + componentIdentification + 
                     "' will not be recorded!" ,
                     new Throwable());
        }

        return to_return;
    }
    
    /**
     * Notify supervision that a component reference has been added to a 
     * LB group. Code is in generated Home servant inheriting from this class.
     *
     * @param component_name The component's name
     */
    protected abstract void notify_add_member(String component_name);
    
    
    
    /**
     * This creates new component reference.
     * 
     * @return A reference on the new component.
     * @exception org.omg.Components.CreateFailure if application errors.
     */ 
    protected org.omg.CORBA.Object create_component_ref(
            org.omg.Components.ConfigValue[] config) 
        throws org.omg.Components.CreateFailure 
    {
        CDMW_LOG_FUNCTION(CCMLoggers.GetCcmcifLogger(), "");
        return create_component_ref(config, null);
    }

    
    
    private org.omg.CORBA.Object getGroupForName(String name)
        throws org.omg.Components.CreateFailure
    {
        CDMW_LOG_FUNCTION(CCMLoggers.GetCcmcifLogger(), 
            "name=" + name);
        try {
            org.omg.CosNaming.NameComponent[] cosName = 
                cdmw.commonsvcs.naming.NamingInterface.toName(name);
                
            return lbGroupManager.get_object_group_ref_from_name(cosName);

        } 
        catch (cdmw.commonsvcs.naming.InvalidNameException e) {
            CDMW_ERROR(CCMLoggers.GetCcmcifLogger(), 
                "Wrong LB Configuration values for LB component creation: "
                + name + " is not a valid group name!",
              new Throwable());
            org.omg.Components.CreateFailure ex = 
                new org.omg.Components.CreateFailure(
                    com.thalesgroup.CdmwDeployment.WRONG_LB_CONFIG_VALUES.value);
            CDMW_LOG_FUNCTION_EXCEPTION(CCMLoggers.GetCcmcifLogger(), ex);
            throw ex;
        }
        catch (org.omg.PortableGroup.ObjectGroupNotFound e) {
            // Group was found in get_group_for_name, but no more exists ?
            // It could have been removed in the meanwhile...
            CDMW_ERROR(CCMLoggers.GetCcmcifLogger(), 
                "Wrong LB Configuration values for LB component creation: "
                + name + " group not found!",
              new Throwable());
            org.omg.Components.CreateFailure ex = 
                new org.omg.Components.CreateFailure(
                    com.thalesgroup.CdmwDeployment.WRONG_LB_CONFIG_VALUES.value);
            CDMW_LOG_FUNCTION_EXCEPTION(CCMLoggers.GetCcmcifLogger(), ex);
            throw ex;
        }
    }
    
    private org.omg.CosNaming.NameComponent[] getCurrentLBLocation()
    {
        CDMW_LOG_FUNCTION(CCMLoggers.GetCcmcifLogger(), "");
        org.omg.CosNaming.NameComponent[] loc =
            new org.omg.CosNaming.NameComponent[3];
                
        try {
            loc[0] = new org.omg.CosNaming.NameComponent();
            loc[0].id = cdmw.ossupport.OS.getHostname();
            loc[0].kind = "hostname";
            loc[1] = new org.omg.CosNaming.NameComponent();
            loc[1].id = cdmw.platformmngt.PlatformInterface.getApplicationName();
            loc[1].kind = "applicationname";
            loc[2] = new org.omg.CosNaming.NameComponent();
            loc[2].id = cdmw.platformmngt.PlatformInterface.getProcessName();
            loc[2].kind = "processname";
        } 
        catch (cdmw.common.BadOrderException e) {
            // shouldn't happen since process has been initialized at startup
            cdmw.common.Assert.check(false);
        }
        
        return loc;
    }
    
}

