/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO 
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2009. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

package cdmw.ccm.cif;

import cdmw.ccm.common.CCMLoggers;
import static cdmw.logging.Logger.*;
import static cdmw.logging.FunctionLogger.*;

import org.omg.Components.BadComponentReference;
import org.omg.Components.IllegalState;
import org.omg.Components.NoKeyAvailable;
import org.omg.Components.PolicyMismatch;
import org.omg.Components.PrimaryKeyBase;

import com.thalesgroup.CdmwCcmContainer.*;

/**
 * This class is the base class for ccmcif context embedding specific CCM
 * context.
 */
public class SessionContext extends Context {
    
    public SessionContext(Session2Context ctx) {
        super(ctx);
        CDMW_LOG_FUNCTION(CCMLoggers.GetCcmcifLogger(), "");
    }
    
 
    //
    // IDL:omg.org/Components/Session2Context/create_ref:1.0
    //
    /**
     * Purpose: <p>The createRef operation is used to create a reference to be
     * exported to clients to invoke operations. The RepositoryId must match the
     * RepositoryId of the component itself, one of its bases, one of its
     * supported interfaces, or one of its facets.
     * 
     * @param repid
     *            Identifies the RepositoryId associated with the interface for
     *            which a reference is being created.
     * 
     * @return A newly created object reference.
     */
    public org.omg.CORBA.Object create_ref(String repId) 
        throws org.omg.CORBA.SystemException 
    {
        CDMW_LOG_FUNCTION(CCMLoggers.GetCcmcifLogger(), "repId="+repId);
        Session2Context sc = this.getSessionContext();
        return sc.create_ref(repId);
    }

    //
    // IDL:omg.org/Components/Session2Context/create_ref_from_oid:1.0
    //
    /**
     * Purpose: <p>The createRefFromOidStr operation is used to create a
     * reference to be exported to clients which includes information provided
     * by the component which it can use on subsequent operation requests.
     * 
     * @param oid
     *            Identifies the stringified ObjectIdSeq to be encapsulated in
     *            the reference.
     * @param repid
     *            Identifies the RepositoryId associated with the interface for
     *            which a reference is being created.
     * 
     * @return A newly created object reference.
     **/    
    public org.omg.CORBA.Object create_ref_from_oid_str(String oid, String repId)   
       throws org.omg.CORBA.SystemException 
    {
        CDMW_LOG_FUNCTION(CCMLoggers.GetCcmcifLogger(), 
            "oid="+oid+"  repId="+repId);
        byte[] compOid = cdmw.ccm.common.CCMUtils.StringToOid(oid);
        Session2Context sc = this.getSessionContext();
        return sc.create_ref_from_oid(compOid, repId);
    }
    
    //
    // IDL:omg.org/Components/Session2Context/get_oid_from_ref:1.0
    //
    /**
     * Purpose: <p>The getOidStrFromRef operation is used by the component
     * to extract the oid encapsulated in the reference. This operation must be
     * called within an operation invocation.
     * 
     * @param objref
     *            Specifies the reference which contains the oid.
     * @return the stringified oid.
     * @throws IllegalState 
     * @throws BadComponentReference 
     * @throws  
     * 
     * @exception Components.IllegalState
     *                Raise if the operation is not called within an operation
     *                invocation.
     * @exception Components.BadComponentReference
     *                Raised with the WRONG_CONTAINER minor code if the
     *                reference was not created by this container.
     */    
    public String get_oid_str_from_ref(org.omg.CORBA.Object compRef)
       throws org.omg.CORBA.SystemException, 
              org.omg.Components.BadComponentReference, 
              org.omg.Components.IllegalState
    {
        CDMW_LOG_FUNCTION(CCMLoggers.GetCcmcifLogger(), "");
        Session2Context sc = this.getSessionContext();
        byte[] compOid = sc.get_oid_from_ref(compRef);
        String compOidStr = cdmw.ccm.common.CCMUtils.OidToString(compOid);
        
        CDMW_LOG_FUNCTION_RETURN(CCMLoggers.GetCcmcifLogger(), compOidStr);
        return compOidStr;
    }
    
    //
    // IDL:omg.org/Components/Entity2Context/get_primary_key:1.0
    //
    /**
     * Purpose: <p>The getPrimaryKey operation return the primary key when
     * component is an entity component. Because we are in a session context
     * this operation will always raise NoKeyAvailable.
     */
    public PrimaryKeyBase get_primary_key() 
        throws NoKeyAvailable, 
               org.omg.CORBA.SystemException 
    {
        CDMW_LOG_FUNCTION(CCMLoggers.GetCcmcifLogger(), "");
       // this does not exist with a session container but with an entity
       // container.
       throw new NoKeyAvailable();
    }
    
    //
    // IDL:omg.org/Components/SessionContext/get_CCM_object:1.0
    //
    /**
     * The getCCMObject operation is used to get the reference used to invoke
     * the component. For basic components, this will always be the component
     * reference. For extended components, this will be a specific facet
     * reference. If this operation is issued outside of the scope of a callback
     * operation, the IllegalState exception is returned.
     * @throws IllegalState 
     */
    public org.omg.CORBA.Object get_CCM_object()
       throws org.omg.CORBA.SystemException, 
              org.omg.Components.IllegalState
    {
        CDMW_LOG_FUNCTION(CCMLoggers.GetCcmcifLogger(), "");
        Session2Context sc = this.getSessionContext();
        return sc.get_CCM_object();
    }
    
    //
    // IDL:omg.org/Components/CCM2Context/req_passivate:1.0
    //
    /**
     * The reqPassivate operation is used by the component to inform the
     * container that it wishes to be passivated when its current operation
     * completes. To be valid, the component must have a servant lifetime policy
     * of component or container. If not, the PolicyMismatch exception shall be
     * raised.
     * @throws PolicyMismatch 
     */
    
    public void req_passivate()
       throws org.omg.CORBA.SystemException,
              org.omg.Components.PolicyMismatch 
    {
        CDMW_LOG_FUNCTION(CCMLoggers.GetCcmcifLogger(), "");
        Session2Context sc = this.getSessionContext();
        sc.req_passivate();
    }
    
    /**
     * Purpose: <p> Return the session context.
     */
    protected Session2Context getSessionContext()
        throws org.omg.CORBA.SystemException
    {
        CDMW_LOG_FUNCTION(CCMLoggers.GetCcmcifLogger(), "");
        CCM2Context context = this.get_ccm_context();
        Session2Context session2Context = Session2ContextHelper.narrow(context);

        return session2Context;
    }
}
