/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO 
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2009. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


package cdmw.ccm.container;

import cdmw.common.Assert;
import cdmw.common.BadParameterException;
import cdmw.ccm.common.CCMLoggers;
import static cdmw.logging.Logger.*;
import static cdmw.logging.FunctionLogger.*;

import cdmw.lifecycle.ObjectDeactivator;

/**
 * A template class for Home Allocators.
 *
 */
public class HomeAllocator implements HomeAllocatorBase {

    /**
     * Name of the class of the servant to instantiate
     */
    private String classname;

    /**
     * Constructor.
     *
     * @param classname Name of the class of the servant to instantiate
     */
    public HomeAllocator(String classname) {
        CDMW_LOG_FUNCTION(CCMLoggers.GetCcmcontainerLogger(), 
            "classname="+classname);
        this.classname = classname;
    }

    /**
     * A static method to get an instance of a Home Allocator
     *
     * @param classname Name of the class of the servant to instantiate
     */
    public static HomeAllocator getAllocator(String classname) {
        CDMW_LOG_FUNCTION(CCMLoggers.GetCcmcontainerLogger(), 
            "classname="+classname);
        return new HomeAllocator(classname);
    }

    /**
     * This operation is called by the container during homes installation
     * to allocate a servant for the home.
     *
     * @param homeExecutor Home executor acquired from the loaded library.
     * @param deactivator Object Deactivator for use
     * for home object deactivation.
     * @param objectRegistration reference of the object responsible to 
     * register ccm object in a local naming context.
     * @param config Home config values
     * @exception BadParameterException Wrong parameter value
     */
    public org.omg.PortableServer.Servant allocate(
        org.omg.Components.HomeExecutorBase homeExecutor,
        ObjectDeactivator deactivator,
        com.thalesgroup.CdmwLifeCycle.ObjectRegistration objectRegistration,
        org.omg.Components.ConfigValue[] config)
        throws BadParameterException 
    {
        CDMW_LOG_FUNCTION(CCMLoggers.GetCcmcontainerLogger(), "");

        try {
            // don't know how to get Class object for an Array using Class.forname
            org.omg.Components.ConfigValue[] tmp =
                new org.omg.Components.ConfigValue[0];
            
            // signature of the constructor
            Class[] parameterTypes = {
                Class.forName("org.omg.Components.HomeExecutorBase"),
                Class.forName("cdmw.lifecycle.ObjectDeactivator"),
                Class.forName("com.thalesgroup.CdmwLifeCycle.ObjectRegistration"),
                tmp.getClass(),
            };
            java.lang.reflect.Method method =
                Class.forName(this.classname).getMethod(
                    "allocate", 
                    parameterTypes);
            Object[] initArgs = {
                homeExecutor,
                deactivator,
                objectRegistration,
                config
            };
            return (org.omg.PortableServer.Servant)
                method.invoke(null, initArgs);
        } catch(ClassNotFoundException cnfe) {
            CDMW_ERROR(CCMLoggers.GetCcmcontainerLogger(),
                "Error allocating Home servant: "+cnfe.getMessage(),
                new Throwable());
            BadParameterException ex =
                new BadParameterException("the class cannot be located");
            CDMW_LOG_FUNCTION_EXCEPTION(CCMLoggers.GetCcmcontainerLogger(), ex);
            throw ex;
        } catch(NoSuchMethodException nsme) {
            CDMW_ERROR(CCMLoggers.GetCcmcontainerLogger(),
                "Error allocating Home servant: "+nsme.getMessage(),
                new Throwable());
            BadParameterException ex =
                new BadParameterException("a matching method is not found " +
                "or if the name is '<init>' or '<clinit>'");
            CDMW_LOG_FUNCTION_EXCEPTION(CCMLoggers.GetCcmcontainerLogger(), ex);
            throw ex;
        } catch(IllegalAccessException iae) {
             CDMW_ERROR(CCMLoggers.GetCcmcontainerLogger(),
                "Error allocating Home servant: "+iae.getMessage(),
                new Throwable());
           BadParameterException ex =
                new BadParameterException(
                "this Method object enforces Java language access control " + 
                "and the underlying method is inaccessible");
            CDMW_LOG_FUNCTION_EXCEPTION(CCMLoggers.GetCcmcontainerLogger(), ex);
            throw ex;
        } catch(java.lang.reflect.InvocationTargetException ite) {
            CDMW_ERROR(CCMLoggers.GetCcmcontainerLogger(),
                "Error allocating Home servant: "+ite.getMessage(),
                new Throwable());
            BadParameterException ex =
                new BadParameterException(
                "the underlying method throws an exception");
            CDMW_LOG_FUNCTION_EXCEPTION(CCMLoggers.GetCcmcontainerLogger(), ex);
            throw ex;
        } catch(LinkageError le) {
            // LinkageError raised by Class.forName()
            CDMW_ERROR(CCMLoggers.GetCcmcontainerLogger(),
                "Error allocating Home servant: "+le.getMessage(),
                new Throwable());
            BadParameterException ex =
                new BadParameterException(
                "the linkage fails. Perhaps a class is missing in a jar...");
            CDMW_LOG_FUNCTION_EXCEPTION(CCMLoggers.GetCcmcontainerLogger(), ex);
            throw ex;
        }

    }
    
    public org.omg.PortableServer.Servant getDelegate(
        org.omg.PortableServer.Servant servant) throws BadParameterException 
    {
        CDMW_LOG_FUNCTION(CCMLoggers.GetCcmcontainerLogger(), "");
        try {
            // signature of the constructor
            Class[] parameterTypes = {
                Class.forName("org.omg.PortableServer.Servant")
            };
            java.lang.reflect.Method method =
                Class.forName(this.classname).getMethod(
                    "getDelegate", 
                    parameterTypes);
            Object[] initArgs = { servant };
            Object result = method.invoke(null, initArgs);
            if (result == null) {
                CDMW_ERROR(CCMLoggers.GetCcmcontainerLogger(),
                    "Error getting delegate: the result is not of the expected type",
                    new Throwable());
                BadParameterException ex = new BadParameterException(
                    "the result is not of the expected type");
                CDMW_LOG_FUNCTION_EXCEPTION(CCMLoggers.GetCcmcontainerLogger(), ex);
                throw ex;
            }
            return (org.omg.PortableServer.Servant) result;
        } catch(ClassNotFoundException cnfe) {
            CDMW_ERROR(CCMLoggers.GetCcmcontainerLogger(),
                "Error getting delegate: "+cnfe.getMessage(),
                new Throwable());
            BadParameterException ex =
                new BadParameterException("the class cannot be located");
            CDMW_LOG_FUNCTION_EXCEPTION(CCMLoggers.GetCcmcontainerLogger(), ex);
            throw ex;
        } catch(NoSuchMethodException nsme) {
            CDMW_ERROR(CCMLoggers.GetCcmcontainerLogger(),
                "Error getting delegate: "+nsme.getMessage(),
                new Throwable());
            BadParameterException ex =
                new BadParameterException("a matching method is not found " +
                "or the name is '<init>' or '<clinit>'");
            CDMW_LOG_FUNCTION_EXCEPTION(CCMLoggers.GetCcmcontainerLogger(), ex);
            throw ex;
        } catch(IllegalAccessException iae) {
            CDMW_ERROR(CCMLoggers.GetCcmcontainerLogger(),
                "Error getting delegate: "+iae.getMessage(),
                new Throwable());
            BadParameterException ex =
                new BadParameterException(
                "this Method object enforces Java language access control " + 
                "and the underlying method is inaccessible");
            CDMW_LOG_FUNCTION_EXCEPTION(CCMLoggers.GetCcmcontainerLogger(), ex);
            throw ex;
        } catch(java.lang.reflect.InvocationTargetException ite) {
            CDMW_ERROR(CCMLoggers.GetCcmcontainerLogger(),
                "Error getting delegate: "+ite.getMessage(),
                new Throwable());
            BadParameterException ex =
                new BadParameterException(
                "the underlying method throws an exception");
            CDMW_LOG_FUNCTION_EXCEPTION(CCMLoggers.GetCcmcontainerLogger(), ex);
            throw ex;
        } catch(LinkageError le) {
            // LinkageError raised by Class.forName()
            CDMW_ERROR(CCMLoggers.GetCcmcontainerLogger(),
                "Error getting delegate: "+le.getMessage(),
                new Throwable());
            BadParameterException ex =
                new BadParameterException(
                "the linkage fails. Perhaps a problem in a jar file, for instance a class is missing...");
            CDMW_LOG_FUNCTION_EXCEPTION(CCMLoggers.GetCcmcontainerLogger(), ex);
            throw ex;
        }
        
    }

    /**
     * This operation is called whenever the Container is done
     * with the Home servant previously allocated by the current allocator.
     *
     * @param home a Home servant.
     */
    public void dispose(org.omg.PortableServer.Servant home) {
        CDMW_LOG_FUNCTION(CCMLoggers.GetCcmcontainerLogger(), "");
        Assert.check( home != null);
        cleanup(home);
    }

    /**
     * This operation perfoms actual disposal of the home servant.
     * This is an ideal place to persist state of the factory.
     *
     * @param home a Home servant.
     */
    protected void cleanup(org.omg.PortableServer.Servant home) {
        CDMW_LOG_FUNCTION(CCMLoggers.GetCcmcontainerLogger(), "");
        try {
            cdmw.ccm.common.DisposableObject obj =
                (cdmw.ccm.common.DisposableObject) this.getDelegate(home);
            obj.dispose();
        }
        catch (ClassCastException cce) {
            CDMW_ERROR(CCMLoggers.GetCcmcontainerLogger(),
                "Error when cleanup: "+cce,
                new Throwable());
        }   
        catch (org.omg.Components.RemoveFailure rf) {
            CDMW_ERROR(CCMLoggers.GetCcmcontainerLogger(),
                "Error when cleanup: "+rf,
                new Throwable());
        }
        catch (cdmw.common.BadParameterException bpe) {
            CDMW_ERROR(CCMLoggers.GetCcmcontainerLogger(),
                "Error when cleanup: "+bpe,
                new Throwable());
        }
    }

}

