/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO 
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2009. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */



#include <ccm_recording/CCM_Client_impl.hpp>
#include <ccm_recording/TestEvent_impl.hpp>

#include <Foundation/common/Assert.hpp>
#include <Foundation/ossupport/OS.hpp>
#include <Foundation/orbsupport/OrbSupport.hpp>
#include <Foundation/logging/CdmwLogging.stub.hpp>
#include <Foundation/logging/Logger.hpp>
#include <Foundation/osthreads/MutexGuard.hpp>

#include <TraceAndPerf/recordingidl/CdmwRecordingData.stub.hpp>
#include <Foundation/idllib/CdmwRecordingContainerManager.stub.hpp>

#include <sstream>



namespace TestCcmRecording
{

//
// IDL:thalesgroup.com/CcmFtTest/ClientModule/CCM_Client:1.0
//
CCM_Client_impl::CCM_Client_impl()
    : m_test_thread(0)
{
}

CCM_Client_impl::~CCM_Client_impl()
{
    std::cerr << "   ... CCM_Client_impl::~CCM_Client_impl called!" << std::endl;
}


void CCM_Client_impl::set_session_context(Components::SessionContext_ptr ctx)
    throw(CORBA::SystemException)
{
    m_session_context = ::TestCcmRecording::CCM_Client_Context::_narrow(ctx);
}
        
::TestCcmRecording::CCM_Client_Context* CCM_Client_impl::get_context()
{
    return ::TestCcmRecording::CCM_Client_Context::_duplicate(m_session_context.in());
}


//
// ccm_activate function : it actives the component (call by the container)
//
void
CCM_Client_impl::ccm_activate()
   throw(CORBA::SystemException)
{
    std::cerr << "   ... CCM_Client_impl::ccm_activate called!" << std::endl;

    try {   
        // save context in CORBATestManager for usage in test
        Cdmw::TestUtils::CORBATestManager::instance()->add(
            "client_context", 
            get_context());
    
        // save this executor in CORBATestManager for usage in test
        Cdmw::TestUtils::CORBATestManager::instance()->add(
            "this_executor", 
            dynamic_cast< Session_CCM_Client_ptr > (this));
    
        // create test runner thread
        m_test_thread = new Cdmw::TestUtils::ThreadedTestRunner(
            Cdmw::TestUtils::CORBATestManager::instance()->get_controller());
        // add test to be run
        m_test_thread->addTest( 
            CppUnit::TestFactoryRegistry::getRegistry("test_ccm_recording").makeTest());
        // start test
        m_test_thread->start();

    } catch (const CORBA::Exception& e) {
        std::cerr << "CORBA Exception thrown during test: "
                  << Cdmw::OrbSupport::OrbSupport::exception_to_string(e)
                  << std::endl;
    } catch (const std::exception& e) {
        std::cerr << "std::exception thrown during test: "
                  << e.what()
                  << std::endl;
    } catch (...) {
        std::cerr << "Unkown thrown during test !!" << std::endl;
    }
}


//
// ccm_passivate function : it makes stop the component (call by the container)
//
void
CCM_Client_impl::ccm_passivate()
   throw(CORBA::SystemException)
{
    std::cerr << "   ... CCM_Client_impl::ccm_passivate called!" << std::endl;
    
    if (m_test_thread != 0) {
        // wait test to be finished
        m_test_thread->join();
        delete m_test_thread;
        m_test_thread = 0;
    }
} 


//
// ccm_remove function : it destroys the component (call by the container)
//
void
CCM_Client_impl::ccm_remove()
   throw(CORBA::SystemException)
{
    std::cerr << "   ... CCM_Client_impl::ccm_remove called!" << std::endl;
}

//
// CdmwRecorindg::Recorder::record() implementation
//
void CCM_Client_impl::record(const CdmwRecording::GroupHeader& header,
                             const CdmwRecording::GIOP_Buffers& data)
        throw(CORBA::SystemException)
{
    CDMW_MUTEX_GUARD(m_mutex);
    
    TEST_INFO("RECORD: === " << header.num_pairs << " new Records received from " <<
              header.hostname << " at " << header.base_time);

    // Use TAO_InputCDR for unmarshalling
    CORBA::ULong len = data.length();
    const CORBA::Octet* buffer = data.get_buffer();
    size_t char_len = len * sizeof(CORBA::Octet) / sizeof(char);
    TAO_InputCDR input((const char*) buffer, char_len);
    input.reset_byte_order(ACE_static_cast (int, header.flags));
    
    // Unmarshall while TAO_InputCDR is not empty
    CdmwRecording::RequestReplyPair pair;
    int i=0;
    while (input.length() > 0) {
        
        // number of octets still to read
        size_t left = input.length();

        // Unmarshall a RequestReplyPair
        if (input >> pair) {
            TEST_INFO("RECORD: ===   - " << (++i) << ": \"" <<
                      pair.request.operation <<
                      "\" from " <<
                      pair.request.server_id);

            // save copy of RequestReplyPair in list
            // ignore _is_a requests
            if (strcmp(pair.request.operation.in(), "_is_a") != 0) {
                m_record_list.push_back(new CdmwRecording::RequestReplyPair(pair));
            }

        } else {
            // number of read octets
            size_t nb_read = left-input.length();
            TEST_INFO("RECORD: === UNMARSHALL ERROR (only " << nb_read <<
                      " octets read)");
            ACE_HEX_DUMP ((LM_INFO,
                           (const char *) input.rd_ptr() - nb_read,
                           nb_read,
                           ACE_TEXT ("read buffer")));
            TEST_INFO("   - Content of pair.request.head:");
            ACE_HEX_DUMP ((LM_INFO,
                           (const char *) &(pair.request.head),
                           12,
                           ACE_TEXT ("pair.request.head")));
            TEST_INFO("   - Content of pair.reply.head:");
            ACE_HEX_DUMP ((LM_INFO,
                           (const char *) &(pair.reply.head),
                           12,
                           ACE_TEXT ("pair.reply.head")));
            // It's useless to try to continue unmarshalling... break loop
            break;
        }
    }
}


//
// Testable do_tests() operation
//
CPPUNIT_TEST_SUITE_NAMED_REGISTRATION(test_ccm_recording, "test_ccm_recording");


test_ccm_recording::test_ccm_recording()
{
    // retrieve context object
    CORBA::Object_var obj = 
        Cdmw::TestUtils::CORBATestManager::instance()->get_object("client_context");
    m_session_context = 
        ::TestCcmRecording::CCM_Client_Context::_narrow(obj.in());
        
    obj = 
        Cdmw::TestUtils::CORBATestManager::instance()->get_object("this_executor");
    m_ccm_client = dynamic_cast< CCM_Client_impl* > (obj.in());
}

void
test_ccm_recording::do_tests()
{
    std::cerr << "   ... test_ccm_recording::do_tests called!" << std::endl;
    
    // wait previous connection requests to be recorded...
    int timescale = Cdmw::TestUtils::get_timescale();
    Cdmw::OsSupport::OS::sleep(2000);
    
    // clear m_record_list content (previous records about connections)
    m_ccm_client->m_record_list.clear();

    try {

        TEST_INFO("Send request to Server1");
        try {
            TestRecording_var test_recording =
                m_session_context->get_connection_test_server1();

            test_recording->test_operation("MESSAGE TO SERVER1");

        } catch (const CORBA::SystemException& e) {
            std::ostringstream os;
            os << "Exception calling Server1's facet: " 
               << Cdmw::OrbSupport::OrbSupport::exception_to_string(e);
            CPPUNIT_ASSERT_MESSAGE(os.str(), false);
        }

        
        TEST_INFO("Send request to Server2");
        try {
            TestRecording_var test_recording =
                m_session_context->get_connection_test_server2();
                
            test_recording->test_operation("MESSAGE TO SERVER2");
            
        } catch (const CORBA::SystemException& e) {
            std::ostringstream os;
            os << "Exception calling Server2's facet: " 
               << Cdmw::OrbSupport::OrbSupport::exception_to_string(e);
            CPPUNIT_ASSERT_MESSAGE(os.str(), false);
        }
        
        
        TEST_INFO("Send request to Server3");
        try {
            TestRecording_var test_recording =
                m_session_context->get_connection_test_server3();
                
            test_recording->test_operation("MESSAGE TO SERVER3");
            
        } catch (const CORBA::SystemException& e) {
            std::ostringstream os;
            os << "Exception calling Server3's facet: " 
               << Cdmw::OrbSupport::OrbSupport::exception_to_string(e);
            CPPUNIT_ASSERT_MESSAGE(os.str(), false);
        }
        
        
        TEST_INFO("Publish Event");
        try {
            TestEvent_var ev = new TestEvent_impl("EVENT WITH MESSAGE");
    
            m_session_context->push_test_event (ev.in());
            
        } catch (const CORBA::SystemException& e) {
            std::ostringstream os;
            os << "Exception calling Server3's facet: " 
               << Cdmw::OrbSupport::OrbSupport::exception_to_string(e);
            CPPUNIT_ASSERT_MESSAGE(os.str(), false);
        }
        
        // wait requests to be recorded
        Cdmw::OsSupport::OS::sleep(5000);
        
        TEST_INFO("Check received records:");
        
        TEST_INFO("A total of " << m_ccm_client->m_record_list.size() << 
                  " records were received:");
        for (CORBA::ULong i=0; i < m_ccm_client->m_record_list.size(); i++) {
            TEST_INFO("  - " << (i+1) << ": \"" <<
                      m_ccm_client->m_record_list[i]->request.operation <<
                      "\" from " <<
                      m_ccm_client->m_record_list[i]->request.server_id);
        }
        
        // 4 records expected:
        
        //  - "test_operation" with "MESSAGE TO SERVER1"
        CPPUNIT_ASSERT_MESSAGE(
            "Message \"test_operation\" with parameter \"MESSAGE TO SERVER1\" was not recorded",
            find_test_operation_record("Server1", "MESSAGE TO SERVER1"));
                               
        //  - "test_operation" with "MESSAGE TO SERVER3"
        CPPUNIT_ASSERT_MESSAGE(
            "Message \"test_operation\" with parameter \"MESSAGE TO SERVER3\" was not recorded",
            find_test_operation_record("Server3", "MESSAGE TO SERVER3"));
                               
        //  - "push_event" with "EVENT WITH MESSAGE"
        CPPUNIT_ASSERT_MESSAGE(
            "Message \"test_operation\" with parameter \"EVENT WITH MESSAGE\" was not recorded",
            find_push_event_record("Server1", "EVENT WITH MESSAGE"));
                               
        //  - "push_event" with "EVENT WITH MESSAGE"
        CPPUNIT_ASSERT_MESSAGE(
            "Message \"test_operation\" with parameter \"EVENT WITH MESSAGE\" was not recorded",
            find_push_event_record("Server3", "EVENT WITH MESSAGE"));
                               
        // Check if there is non-expected records
        if (! m_ccm_client->m_record_list.empty()) {
            TEST_INFO("Following records were not expected:");
            for (CORBA::ULong i=0; i < m_ccm_client->m_record_list.size(); i++) {
                TEST_INFO("  - " << (i+1) << ": \"" <<
                          m_ccm_client->m_record_list[i]->request.operation <<
                          "\" from " <<
                          m_ccm_client->m_record_list[i]->request.server_id);
            }
            
            CPPUNIT_ASSERT_MESSAGE("Some records were not expected", false);
        }
        
//(====================
        // Get a component reference to 'Server1' via 'cdmw_make_ref' tool
        std::string corbaloc_str = "";
        CORBA::String_var s = "";

        std::string HOST = Cdmw::OsSupport::OS::get_hostname();
        std::string PORT = "21869";
        std::string NAME = "DeploymentPlans/V-CMC-007-1.DeploymentPlan/Server1.Component";

        const std::string of("f.out");
        std::ostringstream args;
        args << " --kind=corbaloc --host=" << HOST << " --port=" << PORT << " --name=" << NAME << " --output-file=" << of;
        Cdmw::OsSupport::OS::create_process( "cdmw_make_ref", args.str() );

        // wait end of cdmw_make_ref command
        Cdmw::OsSupport::OS::sleep(timescale*5000);

        // import the object reference from the file
        std::ifstream is;
        is.open(of.c_str(), std::ifstream::in);
        if (is.good()) {
           is >> corbaloc_str;
           is.close();
        } else {
           std::cerr << "Error opening file: '" << of << "'" << std::endl;
           CPPUNIT_ASSERT(false);
        }

        std::cout << "***** cdmw_make_ref CORBALOC : " << corbaloc_str << std::endl;
        CORBA::Object_var o_cmr = Cdmw::TestUtils::CORBATestManager::instance()->get_ORB()->string_to_object(corbaloc_str.c_str());
        CPPUNIT_ASSERT(!CORBA::is_nil(o_cmr.in()));
//)====================

        // Server 1
        TestRecording_var test_recording_1 = 
              m_session_context->get_connection_test_server1();        
        CORBA::Object_var s1__ = test_recording_1->_get_component();
        if (CORBA::is_nil(s1__.in()))
        {
           CPPUNIT_ASSERT_MESSAGE("Failed to get a component reference using _get_component()!", false);
        }
        TestCcmRecording::Server_var server_1 = TestCcmRecording::Server::_narrow(s1__.in());
        if (CORBA::is_nil(server_1.in()))
        {
           CPPUNIT_ASSERT_MESSAGE("server_1 is not a  TestCcmRecording::Server!", false);
        }
        Components::CCMHome_var home_1 = server_1 -> get_ccm_home();
        TestCcmRecording::ServerHome_var server_home_1 = TestCcmRecording::ServerHome::_narrow(home_1.in());
        if (CORBA::is_nil(server_home_1.in()))
        {
           CPPUNIT_ASSERT_MESSAGE("home_1 is not a  TestCcmRecording::ServerHome!", false);
        }
//(====================
        s = Cdmw::TestUtils::CORBATestManager::instance()->get_ORB()->object_to_string(server_1.in());
        std::cout << "***** server_1 IOR : " << s.in() << std::endl;
        TestCcmRecording::Server_var server_1_cmr = TestCcmRecording::Server::_narrow(o_cmr.in());
        if (CORBA::is_nil(server_1_cmr.in())) {
            CPPUNIT_ASSERT_MESSAGE("server_1_cmr is not a TestCcmRecording::Server!", false);
        }
        s = Cdmw::TestUtils::CORBATestManager::instance()->get_ORB()->object_to_string(o_cmr.in());
        std::cout << "***** server_1_cmr IOR : " << s.in() << std::endl;         
//)====================

        // Remove component Server 1   
        TEST_INFO("Removing component Server1");
        try {
            server_home_1->remove_component(server_1_cmr.in());
        } catch (const CORBA::SystemException& e) {
            std::ostringstream os;
            os << "Exception removing component Server1: " 
               << Cdmw::OrbSupport::OrbSupport::exception_to_string(e);
            CPPUNIT_ASSERT_MESSAGE(os.str(), false);
        }
          
        Cdmw::OsSupport::OS::sleep(timescale*2000);

        // Server 3
        TestRecording_var test_recording_3 = 
              m_session_context->get_connection_test_server3();
        CORBA::Object_var s3__ = test_recording_3->_get_component();
        if (CORBA::is_nil(s3__.in()))
        {
           CPPUNIT_ASSERT_MESSAGE("Failed to get a component reference using _get_component()!", false);
        }
        TestCcmRecording::Server_var server_3 = TestCcmRecording::Server::_narrow(s3__.in());
        if (CORBA::is_nil(server_3.in()))
        {
           CPPUNIT_ASSERT_MESSAGE("server_3 is not a  TestCcmRecording::Server!", false);
        }
        Components::CCMHome_var home_3 = server_3 -> get_ccm_home();
        TestCcmRecording::ServerHome_var server_home_3 = TestCcmRecording::ServerHome::_narrow(home_3.in());
        if (CORBA::is_nil(server_home_3.in()))
        {
           CPPUNIT_ASSERT_MESSAGE("home_3 is not a  TestCcmRecording::ServerHome!", false);
        }

        // Remove component Server 3   
        TEST_INFO("Removing component Server3");
        try {
            server_home_3->remove_component(server_3.in());
        } catch (const CORBA::SystemException& e) {
            std::ostringstream os;
            os << "Exception removing component Server3: " 
               << Cdmw::OrbSupport::OrbSupport::exception_to_string(e);
            CPPUNIT_ASSERT_MESSAGE(os.str(), false);
        }

        // Wait for the 2 get_ccm_home requests sent by Server1 and Server3
        // and erase them from the list
        TEST_INFO("Waiting for the 2 get_ccm_home requests sent by \"Server1\" and \"Server3\"");
        do {Cdmw::OsSupport::OS::sleep(1000);} while (m_ccm_client->m_record_list.size() < 2);

        // clear m_record_list content (previous records about connections)
        TEST_INFO("Clearing the record list content");
        m_ccm_client->m_record_list.clear();
        CPPUNIT_ASSERT_MESSAGE("List of recorded messages not empty", 
          m_ccm_client->m_record_list.empty());
        
       // Send again "test_operation" requests
       TEST_INFO("Send  \"test_operation\" request to Server1 after it has been removed");
       try {
         
         test_recording_1->test_operation("MESSAGE TO SERVER1");
       
        } catch (const CORBA::SystemException& e) {
            std::ostringstream os;
            os << "Expected exception calling Server1's facet: " 
               << Cdmw::OrbSupport::OrbSupport::exception_to_string(e);
            TEST_INFO(os.str());
        }
               
        TEST_INFO("Send \"test_operation\" request to Server3 after it has been removed");
        try {
                
           test_recording_3->test_operation("MESSAGE TO SERVER3");
            
        } catch (const CORBA::SystemException& e) {
            std::ostringstream os;
            os << "Expected exception calling Server3's facet: " 
               << Cdmw::OrbSupport::OrbSupport::exception_to_string(e);
            TEST_INFO(os.str());
        }
     
        // wait requests to be recorded (if any)
        Cdmw::OsSupport::OS::sleep(5000);

        // Check if there are non-expected records
        if (! m_ccm_client->m_record_list.empty()) {
            TEST_INFO("Following not expected records were received:");
            for (CORBA::ULong i=0; i < m_ccm_client->m_record_list.size(); i++) {
                TEST_INFO("  - " << (i+1) << ": \"" <<
                     m_ccm_client->m_record_list[i]->request.operation <<
                     "\" from " <<
                     m_ccm_client->m_record_list[i]->request.server_id);
            }           
            CPPUNIT_ASSERT_MESSAGE("Some records were not expected", false);
        }
        
    } catch (const CORBA::SystemException& e) {
        std::ostringstream os;
        os << "Unexpected exception raised : \n" 
           << Cdmw::OrbSupport::OrbSupport::exception_to_string(e);
        CPPUNIT_ASSERT_MESSAGE(os.str(), false);
    } catch (const std::exception& e) {
        // catch any CPPUnit exception, print it and throw it
        TEST_INFO(e.what());
        throw e;
    }
    
    // print test summary
    Cdmw::TestUtils::CORBATestManager::instance()->summary();
}

bool
test_ccm_recording::find_test_operation_record(const char* component_name,
                                               const char* message)
{
    CCM_Client_impl::RecordList::iterator it;
    for (it = m_ccm_client->m_record_list.begin();
         it != m_ccm_client->m_record_list.end();
         it++)
    {
        // test is record is for a "test_operation"
        if (strcmp((*it)->request.operation, "test_operation") == 0) {
            
            // check if server_id ends with component_name
            std::string server_id((*it)->request.server_id);
            // get last chars of server_id with same lenght as component_name
            std::string substr = 
                server_id.substr(server_id.length() - strlen(component_name));
            if (substr == component_name) {
                
                // try to unmarshall parameters as a string
                CORBA::ULong len = (*it)->request.data.length();
                const CORBA::Octet* buffer = (*it)->request.data.get_buffer();
                size_t char_len = len * sizeof(CORBA::Octet) / sizeof(char);
                TAO_InputCDR input((const char*) buffer, char_len);
                input.reset_byte_order(ACE_static_cast (int, (*it)->request.head.flags));

                CORBA::String_var str;
                if (input >> str.out()) 
                {
                    // unmarshall OK. Test if it's expected message
                    if (strcmp(str.in(), message) == 0) {
                        // message found, remove it from list
                        TEST_INFO("Remove '" << (*it)->request.operation << "' message from the list : " << message);
                        m_ccm_client->m_record_list.erase(it);
                        return true;
                    }
                }
                else
                {
                    TEST_INFO("record for \"test_operation\" found, but parameter"
                              " is not a string!");
                }
            }
            else
            {
                TEST_INFO("record for \"test_operation\" found, but server_id \"" <<
                          server_id << "\" doesn't end with \"" << component_name << "\"");
            }
        }
    }
    
    return false;
}

bool
test_ccm_recording::find_push_event_record(const char* component_name,
                                           const char* message)
{
    CCM_Client_impl::RecordList::iterator it;
    for (it = m_ccm_client->m_record_list.begin();
         it != m_ccm_client->m_record_list.end();
         it++) 
    {
        // test is record is for a "push_event"
        if (strcmp((*it)->request.operation, "push_event") == 0) {

            // check if server_id ends with component_name
            std::string server_id((*it)->request.server_id);
            // get last chars of server_id with same lenght as component_name
            std::string substr = 
                server_id.substr(server_id.length() - strlen(component_name));
            if (substr == component_name) {
                
                // try to unmarshall parameters as a TestEvent
                CORBA::ULong len = (*it)->request.data.length();
                const CORBA::Octet* buffer = (*it)->request.data.get_buffer();
                size_t char_len = len * sizeof(CORBA::Octet) / sizeof(char);
                TAO_InputCDR input((const char*) buffer, char_len);
                input.reset_byte_order(ACE_static_cast (int, (*it)->request.head.flags));
                
                TestCcmRecording::TestEvent_var evt;
                if (input >> evt.out())
                {
                    // unmarshall OK. Test if it's expected message
                    if (strcmp(evt->message(), message) == 0) {
                        // message found, remove it from list
                        TEST_INFO("Remove '" << (*it)->request.operation << "' message from the list : " << message);
                        m_ccm_client->m_record_list.erase(it);
                        return true;
                    }
                }
                else
                {
                    TEST_INFO("record for \"push_event\" found, but parameter"
                              " is not a TestEvent!");
                }
            }
            else
            {
                TEST_INFO("record for \"push_event\" found, but server_id \"" <<
                          server_id << "\" doesn't end with \"" << component_name << "\"");
            }
        }
    }
    
    return false;
}


};  // namespace TestCcmRecording

