/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO 
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2009. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */



#include <ccm_services/CCM_Client_impl.hpp>

#include <Foundation/common/Assert.hpp>
#include <Foundation/ossupport/OS.hpp>
#include <Foundation/orbsupport/OrbSupport.hpp>
#include <Foundation/logging/CdmwLogging.stub.hpp>
#include <Foundation/logging/Logger.hpp>
#include <SystemMngt/idllib/CdmwPlatformMngtManagedElementContexts.stub.hpp>

namespace {
    
    const std::string SYSTEM_NAME = "system_deployment";
    const std::string APPLICATION_NAME = "CcmServicesApplication";
    const std::string CLIENT_PROCESS_NAME = "ClientProcess";
    const std::string SERVER_PROCESS1_NAME = "ServerProcess1";
    const std::string SERVER_PROCESS2_NAME = "ServerProcess2";
    const std::string SERVER_PROCESS3_NAME = "ServerProcess3";

    const std::string CLIENT_COMPONENT_NAME = "Client1";
    const std::string SERVER_COMPONENT1_NAME = "Server1";
    const std::string SERVER_COMPONENT2_NAME = "Server2";
    const std::string SERVER_COMPONENT3_NAME = "Server3";
  
    const std::string CLIENT_ISSUER_NAME = "ClientProcess/Client1";
    const std::string SERVER_ISSUER1_NAME = "ServerProcess1/Server1";
    const std::string SERVER_ISSUER2_NAME = "ServerProcess2/Server2";
    const std::string SERVER_ISSUER3_NAME = "ServerProcess3/Server3";
    const std::string SUPERVISION_ISSUER_NAME = "CDMW.PFMMNG";
  
    const std::string CLIENT_NOTIFICATION_MESSAGE = 
        "Notification message from Client";
    const CORBA::ULong CLIENT_USER_EVENT = 33;

    const std::string SERVER_NOTIFICATION_MESSAGE = 
        "Notification message from Server";
    const std::string SERVER_NOTIFICATION_FATAL = 
        "Fatal error from Server";
    const CORBA::ULong SERVER_USER_EVENT = 44;
    const CORBA::Long SERVER_EXIT_CODE = 66;
    

    const char* event_kind_to_string(CdmwPlatformMngt::Event* ev) {
        switch (ev->event_kind()) {
            case CdmwPlatformMngt::SYSTEM_MODE_CHANGE:
                return "SYSTEM_MODE_CHANGE";
            case CdmwPlatformMngt::SYSTEM_STATUS_CHANGE:
                return "SYSTEM_STATUS_CHANGE";
            case CdmwPlatformMngt::HOST_STATUS_CHANGE:
                return "HOST_STATUS_CHANGE";
            case CdmwPlatformMngt::APPLICATION_AGENT_DEATH:
                return "APPLICATION_AGENT_DEATH";
            case CdmwPlatformMngt::APPLICATION_MODE_CHANGE:
                return "APPLICATION_MODE_CHANGE";
            case CdmwPlatformMngt::APPLICATION_STATUS_CHANGE:
                return "APPLICATION_STATUS_CHANGE";
            case CdmwPlatformMngt::PROCESS_MESSAGE:
                return "PROCESS_MESSAGE";
            case CdmwPlatformMngt::PROCESS_STATUS_CHANGE:
                return "PROCESS_STATUS_CHANGE";
            //PCR-1257 commented code to be deleted
            /*case CdmwPlatformMngt::SYSTEM_ENTITY_STATUS_CHANGE:
                return "SYSTEM_ENTITY_STATUS_CHANGE";
            case CdmwPlatformMngt::HOST_ENTITY_STATUS_CHANGE:
                return "HOST_ENTITY_STATUS_CHANGE";
            case CdmwPlatformMngt::APPLICATION_ENTITY_STATUS_CHANGE:
                return "APPLICATION_ENTITY_STATUS_CHANGE";
            case CdmwPlatformMngt::PROCESS_ENTITY_STATUS_CHANGE:
                return "PROCESS_ENTITY_STATUS_CHANGE";*/
            case CdmwPlatformMngt::ENTITY_STATUS_CHANGE:
                return "ENTITY_STATUS_CHANGE";
            case CdmwPlatformMngt::CONFIGURATION_CHANGE:
                return "CONFIGURATION_CHANGE";
            case CdmwPlatformMngt::USER_EVENT:
                return "USER_EVENT";
            case CdmwPlatformMngt::PROPERTY_CHANGE:
                return "PROPERTY_CHANGE";
            case CdmwPlatformMngt::RESTART_REQUESTED:
                return "RESTART_REQUESTED";
            case CdmwPlatformMngt::REBOOT_REQUESTED:
                return "REBOOT_REQUESTED";
            case CdmwPlatformMngt::STOP_REQUESTED:
                return "STOP_REQUESTED";
            case CdmwPlatformMngt::COMPONENT_STATUS_CHANGE:
                return "COMPONENT_STATUS_CHANGE";
            default:
                return "unknown event kind !";
        }
    }

};

namespace TestCcmServices
{

//
// IDL:thalesgroup.com/CcmFtTest/ClientModule/CCM_Client:1.0
//
CCM_Client_impl::CCM_Client_impl()
{
}

CCM_Client_impl::~CCM_Client_impl()
{
    std::cout << "   ... CCM_Client_impl::~CCM_Client_impl called!" << std::endl;
}


void CCM_Client_impl::set_session_context(Components::SessionContext_ptr ctx)
    throw(CORBA::SystemException)
{
    m_session_context = ::TestCcmServices::CCM_Client_Context::_narrow(ctx);
}

::TestCcmServices::CCM_Client_Context* CCM_Client_impl::get_context()
{
    return ::TestCcmServices::CCM_Client_Context::_duplicate(m_session_context.in());
}

//
// ccm_activate function : it actives the component (call by the container)
//
void
CCM_Client_impl::ccm_activate()
   throw(CORBA::SystemException)
{
    std::cout << "   ... CCM_Client_impl::ccm_activate called!" << std::endl;

    // Get this component CORBA object
    // Note: this has to be performed in a CORBA request thread; thus
    //       not in test which is a specific thread.
    TestCcmServices::Client_var this_component =
            TestCcmServices::Client::_nil();
    try {
        // narrow context in SessionContext for get_CCM_objet() access
        Components::SessionContext_var ctx =
        Components::SessionContext::_narrow(m_session_context.in());

        // call SessionContext::get_CCM_objet()
        CORBA::Object_var obj =
        ctx->get_CCM_object();

        this_component =
        TestCcmServices::Client::_narrow(obj.in());

        CDMW_ASSERT(!CORBA::is_nil(this_component.in()));

    } catch (const CORBA::UserException& e) {
        std::cerr << "UserException during get_CCM_Object() :"
        << Cdmw::OrbSupport::OrbSupport::exception_to_string(e)
        << std::endl;
        CDMW_ASSERT(false);
    } catch (const CORBA::SystemException& e) {
        std::cerr << "SystemException during get_CCM_Object() :"
        << Cdmw::OrbSupport::OrbSupport::exception_to_string(e)
        << std::endl;
        CDMW_ASSERT(false);
    }

    try {
        // save context in CORBATestManager for usage in test
        Cdmw::TestUtils::CORBATestManager::instance()->add(
                "client_context",
                get_context());

        // save this component in CORBATestManager for usage in test
        Cdmw::TestUtils::CORBATestManager::instance()->add(
                "this_component",
                this_component.in());

        // save this executor in CORBATestManager for usage in test
        Cdmw::TestUtils::CORBATestManager::instance()->add(
                "this_executor",
                dynamic_cast< Session_CCM_Client_ptr> (this));

        // m_testThread.start();

        // create test runner thread
        m_test_thread = new Cdmw::TestUtils::ThreadedTestRunner(
                Cdmw::TestUtils::CORBATestManager::instance()->get_controller());
        // add test to be run
        m_test_thread->addTest(
                CppUnit::TestFactoryRegistry::getRegistry("test_ccm_services").makeTest());
        // start test
        m_test_thread->start();

    } catch (const CORBA::Exception& e) {
        std::cerr << "CORBA Exception thrown during test: "
        << Cdmw::OrbSupport::OrbSupport::exception_to_string(e)
        << std::endl;
    } catch (const std::exception& e) {
        std::cerr << "std::exception thrown during test: "
        << e.what()
        << std::endl;
    } catch (...) {
        std::cerr << "Unkown thrown during test !!" << std::endl;
    }
}

//
// ccm_passivate function : it makes stop the component (call by the container)
//
void 
CCM_Client_impl::ccm_passivate() 
    throw(CORBA::SystemException) 
{
    std::cout << "   ... CCM_Client_impl::ccm_passivate called!" << std::endl;

    if (m_test_thread != 0) {
        // wait test to be finished
        m_test_thread->join();
        delete m_test_thread;
        m_test_thread = 0;
        // print test summary
    }
}

//
// ccm_remove function : it destroys the component (call by the container)
//
void
CCM_Client_impl::ccm_remove()
   throw(CORBA::SystemException)
{
    std::cout << "   ... CCM_Client_impl::ccm_remove called!" << std::endl;
}

//
// SupervisionObserver::notify() implementation
//
void CCM_Client_impl::notify(CdmwPlatformMngt::Event* event)
        throw(CORBA::SystemException)
{
    CORBA::String_var msg = event->to_string();
    std::cout << "   ... " << event_kind_to_string(event)
            << " notification received from Supervision : \n" 
            << msg.in() << std::endl;

    // save event in event list
    event->_add_ref();
    m_event_list.push_back(event);
}

//
// return last received event
//
CdmwPlatformMngt::Event* 
CCM_Client_impl::get_last_event() 
{
    // if no event in list, return null
    if (m_event_list.size() == 0)
        return 0;

    // get last event from event list, and remove it
    CdmwPlatformMngt::Event* event = m_event_list.back();
    m_event_list.pop_back();

    return event;
}

//
// return first received event
//
CdmwPlatformMngt::Event* 
CCM_Client_impl::get_first_event() 
{
    // if no event in list, return null
    if (m_event_list.size() == 0)
        return 0;

    // get first event from event list, and remove it
    CdmwPlatformMngt::Event* event = m_event_list.front();
    m_event_list.pop_front();

    return event;
}

//
// clear received event list
//
void 
CCM_Client_impl::clear_event_list() 
{
    m_event_list.clear();
}

//
// Testable do_tests() operation
//
CPPUNIT_TEST_SUITE_NAMED_REGISTRATION(test_ccm_services, "test_ccm_services");

test_ccm_services::test_ccm_services() 
{
    // retrieve context object
    CORBA::Object_var obj = 
        Cdmw::TestUtils::CORBATestManager::instance()->get_object("client_context");
    m_session_context = 
        ::TestCcmServices::CCM_Client_Context::_narrow(obj.in());

    obj = 
        Cdmw::TestUtils::CORBATestManager::instance()->get_object("this_executor");
    m_ccm_client = dynamic_cast< CCM_Client_impl* > (obj.in());

    obj = 
        Cdmw::TestUtils::CORBATestManager::instance()->get_object("this_component");
    m_component_ref =
        TestCcmServices::Client::_narrow(obj.in());
}

void
test_ccm_services::do_tests()
{
    std::cout << "   ... test_ccm_services::do_tests called!" << std::endl;

    //
    // Test Logger service
    //
    TEST_INFO("Testing LoggerFactory QoS Service");
    try {
        // get LoggerFactory
        CORBA::Object_var obj =
        m_session_context->resolve_service_reference("LoggerFactory");
        CdmwLogging::LoggerFactory_var loggerFactory =
        CdmwLogging::LoggerFactory::_narrow(obj.in());
        // get Logger for TEST_DOMAIN
        CdmwLogging::Logger_var logger =
        loggerFactory->get_logger("TEST_DOMAIN");

        // log a message
        CDMW_INFO(logger, "Test logging from component");

        // log a warnng
        CDMW_WARN(logger, "Test warning from component");

        // log an error
        CDMW_ERROR(logger, "Test error from component");

    } catch (const Components::CCMException& e) {
        TEST_INFO("LoggerFactory no found !!");
        CPPUNIT_ASSERT(false);
    }

    // wait 2sec System to be started, since we will test system status 
    // and we expect its SYSTEM_STARTED
    TEST_INFO("Wait 5 seconds System reach SYSTEM_STARTED status");
    Cdmw::OsSupport::OS::sleep(5000);

    TEST_INFO("Testing ManagedComponentContext QoS Service");
    CORBA::String_var real_component_name;
    try {
        // get ManagedComponentContext
        CORBA::Object_var obj =
        m_session_context->resolve_service_reference("ManagedComponentContext");
        CdmwPlatformMngt::ManagedElementContexts::ComponentContext_var supContext =
        CdmwPlatformMngt::ManagedElementContexts::ComponentContext::_narrow(obj.in());
        CPPUNIT_ASSERT(!CORBA::is_nil(supContext.in()));

        TEST_INFO("   test get names");
        // get component, process, application names
        CORBA::String_var component_name = supContext->managed_element_name();
        TEST_INFO("      component_name = '"
                << (component_name.in() == 0 ? "null" : component_name.in())
                << "'");
        CPPUNIT_ASSERT(component_name.in() != 0);
        CPPUNIT_ASSERT(CLIENT_COMPONENT_NAME == component_name.in());

        CORBA::String_var process_name = supContext->process_name();
        TEST_INFO("      process_name = '"
                << (process_name.in() == 0 ? "null" : process_name.in())
                << "'");
        CPPUNIT_ASSERT(process_name.in() != 0);
        CPPUNIT_ASSERT(CLIENT_PROCESS_NAME == process_name.in());

        CORBA::String_var appli_name = supContext->application_name();
        TEST_INFO("      appli_name = '"
                << (appli_name.in() == 0 ? "null" : appli_name.in())
                << "'");
        CPPUNIT_ASSERT(appli_name.in() != 0);
        CPPUNIT_ASSERT(APPLICATION_NAME == appli_name.in());

        // get System supervision object
        TEST_INFO("   get System supervision object");
        CdmwPlatformMngt::External::System_var system = supContext->the_system();

        // register this component as System observer
        TEST_INFO("   register this component as System Observer");
        CdmwPlatformMngt::SupervisionObserver_var old_one =
        system->register_observer("TestCcmServices_Client",
                m_component_ref.in());

        // test System supervision object
        TEST_INFO("   test System status");
        CORBA::String_var status_info;
        CPPUNIT_ASSERT(system->get_status(status_info.out()) ==
                CdmwPlatformMngt::SYSTEM_STARTED);
        TEST_INFO("      status info: " << status_info.in());

        // test Application supervision object
        TEST_INFO("   test Application status");
        CdmwPlatformMngt::External::Application_var application =
        supContext->the_application();
        CPPUNIT_ASSERT(application->get_status(status_info.out()) ==
                CdmwPlatformMngt::APPLICATION_RUNNING);
        TEST_INFO("      status info: " << status_info.in());

        // test Process supervision object
        TEST_INFO("   test Process status");
        CdmwPlatformMngt::External::Process_var process =
        supContext->the_process();
        CPPUNIT_ASSERT(process->get_status(status_info.out()) ==
                CdmwPlatformMngt::PROCESS_RUNNING);
        TEST_INFO("      status info: " << status_info.in());

        // test Host supervision object
        TEST_INFO("   test Host status");
        CdmwPlatformMngt::External::Host_var host =
        supContext->the_host();
        CPPUNIT_ASSERT(host->status() == CdmwPlatformMngt::HOST_STARTED);
        TEST_INFO("      status info: HOST_STARTED");

        // test message notification to Supervision
        TEST_INFO("   notify message to supervision");
        supContext->notify_message(CdmwPlatformMngtBase::INF,
                CLIENT_NOTIFICATION_MESSAGE.c_str());
        Cdmw::OsSupport::OS::sleep(1000);

        TEST_INFO("      check that notification has been received back");
        CdmwPlatformMngt::Event_var event = m_ccm_client->get_last_event();
        CPPUNIT_ASSERT(event.in() != 0);
        CPPUNIT_ASSERT(event->event_kind() == CdmwPlatformMngt::PROCESS_MESSAGE);
        CdmwPlatformMngt::ProcessMessage* message_event =
        CdmwPlatformMngt::ProcessMessage::_downcast(event.in());
        CPPUNIT_ASSERT(CLIENT_ISSUER_NAME == message_event->issuer());
        CPPUNIT_ASSERT(CLIENT_NOTIFICATION_MESSAGE == message_event->info());

        // test user event notification to Supervision
        TEST_INFO("   notify user event to supervision");
        CORBA::Any_var any = new CORBA::Any();
        *any <<= CLIENT_USER_EVENT;
        supContext->notify_user_event(any.in());
        Cdmw::OsSupport::OS::sleep(1000);

        TEST_INFO("      check that notification has been received back");
        event = m_ccm_client->get_last_event();
        CPPUNIT_ASSERT(event.in() != 0);
        CPPUNIT_ASSERT(event->event_kind() == CdmwPlatformMngt::USER_EVENT);
        CdmwPlatformMngt::UserEvent* user_event =
        CdmwPlatformMngt::UserEvent::_downcast(event.in());
        CORBA::ULong value;
        if (user_event->data() >>= value) {
            CPPUNIT_ASSERT(value == CLIENT_USER_EVENT);
        } else {
            CPPUNIT_FAIL("UserEvent's data doesn't contain a CORBA::ULong");
        }
        CPPUNIT_ASSERT(CLIENT_ISSUER_NAME == user_event->issuer());
        CPPUNIT_ASSERT(APPLICATION_NAME == user_event->application_name());
        CPPUNIT_ASSERT(CLIENT_PROCESS_NAME == user_event->process_name());
        CPPUNIT_ASSERT(Cdmw::OsSupport::OS::get_hostname() == user_event->host_name());

        //
        // test with Server components
        //
        TEST_INFO("Test ManagedComponentContext in Server components");

        // test with Server1
        TEST_INFO(" tests on Server1");
        test_server_ft_state(true); /* expected initial FT state: PRIMARY*/
        test_server_notify(SERVER_ISSUER1_NAME,
                SERVER_PROCESS1_NAME);
        test_server_fatal_error(SERVER_ISSUER1_NAME,
                SERVER_PROCESS1_NAME,
                true); /*DEGRADED_MODE notif expected*/

        // test with Server2
        TEST_INFO(" tests on Server2");
        test_server_ft_state(false); /* expected initial FT state: BACKUP*/
        test_server_notify(SERVER_ISSUER2_NAME,
                SERVER_PROCESS2_NAME);
        sleep(10);
        test_server_fatal_error(SERVER_ISSUER2_NAME,
                SERVER_PROCESS2_NAME,
                false); /*System and Appli are already DEGRADED*/

        // test with Server3
        TEST_INFO(" tests on Server3");
        test_server_ft_state(false); /* expected initial FT state: BACKUP*/
        test_server_notify(SERVER_ISSUER3_NAME,
                SERVER_PROCESS3_NAME);
        sleep(10);
        test_server_exit(SERVER_ISSUER3_NAME,
                SERVER_PROCESS3_NAME,
                system.in(),
                false); /*System and Appli are already DEGRADED*/

        TEST_INFO("Test ended.");

    } catch (const Components::CCMException& e) {
        TEST_INFO("ManagedComponentContext no found !!");
        CPPUNIT_ASSERT(false);
    } catch (const CORBA::UserException& e) {
        TEST_INFO("UserException during ManagedComponentContext test : " <<
                Cdmw::OrbSupport::OrbSupport::exception_to_string(e));
        CPPUNIT_ASSERT(false);
    } catch (const CORBA::SystemException& e) {
        TEST_INFO("SystemException during ManagedComponentContext test : " <<
                Cdmw::OrbSupport::OrbSupport::exception_to_string(e));
        CPPUNIT_ASSERT(false);
    } catch (const std::exception& e) {
        TEST_INFO("std::exception during ManagedComponentContext test : " <<
                e.what());
        CPPUNIT_ASSERT(false);
    } catch (...) {
        TEST_INFO("unknown exception during ManagedComponentContext test !!");
        CPPUNIT_ASSERT(false);
    }

    Cdmw::TestUtils::CORBATestManager::instance()->summary();
}

void 
test_ccm_services::test_server_ft_state(bool expect_initial_ft_state_primary)
     throw (CORBA::SystemException, std::exception)
{
    // get connected server
    TestServices_var test_services =
            m_session_context->get_connection_test_services();

    // test get_initial_ft_state()
    if (expect_initial_ft_state_primary) {
        TEST_INFO("   test initial FTState was PRIMARY");
        CPPUNIT_ASSERT(test_services->get_initial_ft_state() == PRIMARY);
    } else {
        TEST_INFO("   test initial FTState was BACKUP");
        CPPUNIT_ASSERT(test_services->get_initial_ft_state() == BACKUP);
    }

    // test get_actual_ft_state()
    TEST_INFO("   test actual FTState is PRIMARY");
    CPPUNIT_ASSERT(test_services->get_actual_ft_state() == PRIMARY);
}

void 
test_ccm_services::test_server_notify(const std::string& expected_server_issuer_name,
                                      const std::string& expected_server_process_name)
   throw (CORBA::SystemException, std::exception)
{
    // get connected server
    TestServices_var test_services =
            m_session_context->get_connection_test_services();

    // test call_notify_message()
    TEST_INFO("   test notify_message from Server");
    test_services->call_notify_message(CdmwPlatformMngtBase::INF,
            SERVER_NOTIFICATION_MESSAGE.c_str());
    Cdmw::OsSupport::OS::sleep(1000);
    TEST_INFO("      check that notification has been received back");
    CdmwPlatformMngt::Event_var event = m_ccm_client->get_last_event();
    CPPUNIT_ASSERT(event.in() != 0);
    CPPUNIT_ASSERT(event->event_kind() == CdmwPlatformMngt::PROCESS_MESSAGE);
    CdmwPlatformMngt::ProcessMessage* message_event =
            CdmwPlatformMngt::ProcessMessage::_downcast(event.in());
    CPPUNIT_ASSERT(expected_server_issuer_name == message_event->issuer());
    CPPUNIT_ASSERT(SERVER_NOTIFICATION_MESSAGE == message_event->info());

    // test call_notify_user_event()
    TEST_INFO("   test notify_user_event from Server");
    CORBA::Any_var any = new CORBA::Any();
    *any <<= SERVER_USER_EVENT;
    test_services->call_notify_user_event(any.in());
    Cdmw::OsSupport::OS::sleep(1000);
    TEST_INFO("      check that notification has been received back");
    event = m_ccm_client->get_last_event();
    CPPUNIT_ASSERT(event.in() != 0);
    CPPUNIT_ASSERT(event->event_kind() == CdmwPlatformMngt::USER_EVENT);
    CdmwPlatformMngt::UserEvent* user_event =
            CdmwPlatformMngt::UserEvent::_downcast(event.in());
    CORBA::ULong value;
    if (user_event->data() >>= value) {
        CPPUNIT_ASSERT(value == SERVER_USER_EVENT);
    } else {
        CPPUNIT_FAIL("UserEvent's data doesn't contain a CORBA::ULong");
    }
    CPPUNIT_ASSERT(expected_server_issuer_name == user_event->issuer());
    CPPUNIT_ASSERT(APPLICATION_NAME == user_event->application_name());
    CPPUNIT_ASSERT(expected_server_process_name == user_event->process_name());
}

void 
test_ccm_services::test_server_fatal_error(
        const std::string& expected_server_issuer_name,
        const std::string& expected_server_process_name,
        bool expect_degraded_notif) 
   throw (CORBA::SystemException, std::exception)
{
    // get connected server
    TestServices_var test_services =
            m_session_context->get_connection_test_services();

    // clear event list
    m_ccm_client->clear_event_list();

    // test call_notify_fatal_error()
    TEST_INFO("   test notify_fatal_error from Server");
    test_services->call_notify_fatal_error(SERVER_NOTIFICATION_FATAL.c_str());
    Cdmw::OsSupport::OS::sleep(1000);
    TEST_INFO("      check that notification has been received back");
    CdmwPlatformMngt::Event_var event = m_ccm_client->get_first_event();
    CPPUNIT_ASSERT(event.in() != 0);

    // first event should be ProcessStatusChanged
    CPPUNIT_ASSERT(event->event_kind() == CdmwPlatformMngt::PROCESS_STATUS_CHANGE);
    CdmwPlatformMngt::ProcessStatusChange* fatal_event =
            CdmwPlatformMngt::ProcessStatusChange::_downcast(event.in());
    CPPUNIT_ASSERT(expected_server_issuer_name == fatal_event->issuer());
    CPPUNIT_ASSERT(SERVER_NOTIFICATION_FATAL == fatal_event->info());
    CPPUNIT_ASSERT(fatal_event->process_status() == CdmwPlatformMngt::PROCESS_FAILED_ERROR);

    if (expect_degraded_notif) {
        // we expect 2 MODE_CHANGE to DEGRADED notifications
        // and 2 STATUS_CHANGE

        //  - Test application MODE_CHANGE to DEGRADED
        TEST_INFO("      check application mode changed to DEGRADED");
        event = m_ccm_client->get_first_event();

        CPPUNIT_ASSERT(event.in() != 0);
        CPPUNIT_ASSERT(event->event_kind() == CdmwPlatformMngt::APPLICATION_MODE_CHANGE);
        CdmwPlatformMngt::ApplicationModeChange* app_mode_change =
                CdmwPlatformMngt::ApplicationModeChange::_downcast(event.in());
        CPPUNIT_ASSERT(APPLICATION_NAME == app_mode_change->application_name());
        CPPUNIT_ASSERT(app_mode_change->application_mode() == CdmwPlatformMngt::DEGRADED_MODE);

        //  - Test system MODE_CHANGE to DEGRADED
        TEST_INFO("      check system mode changed to DEGRADED");
        event = m_ccm_client->get_first_event();
        CPPUNIT_ASSERT(event.in() != 0);
        CPPUNIT_ASSERT(event->event_kind() == CdmwPlatformMngt::SYSTEM_MODE_CHANGE);
        CdmwPlatformMngt::SystemModeChange* sys_mode_change =
                CdmwPlatformMngt::SystemModeChange::_downcast(event.in());
        CPPUNIT_ASSERT(sys_mode_change->system_mode() == CdmwPlatformMngt::DEGRADED_MODE);
        //  - Test application STATUS_CHANGE to COMPONENT_FAILED_ERROR
        TEST_INFO("      check application status changed to COMPONENT_STATUS_CHANGE");
        event = m_ccm_client->get_first_event();
        CPPUNIT_ASSERT(event.in() != 0);
        CPPUNIT_ASSERT(event->event_kind() == CdmwPlatformMngt::COMPONENT_STATUS_CHANGE);
        CdmwPlatformMngt::ComponentStatusChange* component_status_change =
                CdmwPlatformMngt::ComponentStatusChange::_downcast(event.in());
        CPPUNIT_ASSERT(component_status_change->component_status() == CdmwPlatformMngt::COMPONENT_FAILED_ERROR);

        //  - Test application STATUS_CHANGE to PARTIAL_RUNNING
        TEST_INFO("      check application status changed to PARTIAL_RUNNING");
        event = m_ccm_client->get_first_event();
        CPPUNIT_ASSERT(event.in() != 0);
        CPPUNIT_ASSERT(event->event_kind() == CdmwPlatformMngt::APPLICATION_STATUS_CHANGE);
        CdmwPlatformMngt::ApplicationStatusChange* app_status_change =
                        CdmwPlatformMngt::ApplicationStatusChange::_downcast(event.in());
        CPPUNIT_ASSERT(APPLICATION_NAME == app_status_change->application_name());
        CPPUNIT_ASSERT(app_status_change->application_status() == CdmwPlatformMngt::APPLICATION_PARTIAL_RUNNING);

        //  - Test system STATUS_CHANGE to PARTIAL_STARTED
        TEST_INFO("      check system status changed to PARTIAL_STARTED");
        event = m_ccm_client->get_first_event();

        CPPUNIT_ASSERT(event.in() != 0);
        CPPUNIT_ASSERT(event->event_kind() == CdmwPlatformMngt::SYSTEM_STATUS_CHANGE);
        CdmwPlatformMngt::SystemStatusChange* sys_status_change =
                CdmwPlatformMngt::SystemStatusChange::_downcast(event.in());
        CPPUNIT_ASSERT(sys_status_change->system_status() == CdmwPlatformMngt::SYSTEM_PARTIAL_STARTED);

    } else {

        //  - Test application STATUS_CHANGE to COMPONENT_FAILED_ERROR
        TEST_INFO("      check application status changed to COMPONENT_STATUS_CHANGE");
        event = m_ccm_client->get_first_event();
        CPPUNIT_ASSERT(event.in() != 0);
        CPPUNIT_ASSERT(event->event_kind()
                == CdmwPlatformMngt::COMPONENT_STATUS_CHANGE);
        CdmwPlatformMngt::ComponentStatusChange* component_status_change =
                CdmwPlatformMngt::ComponentStatusChange::_downcast(event.in());
        CPPUNIT_ASSERT(component_status_change->component_status()
                == CdmwPlatformMngt::COMPONENT_FAILED_ERROR);
    }

    //  - Test host STATUS_CHANGE to PARTIAL_STARTED
    TEST_INFO("      check host status changed to PARTIAL_STARTED");
    event = m_ccm_client->get_first_event();
    CPPUNIT_ASSERT(event.in() != 0);
    CPPUNIT_ASSERT(event->event_kind() == CdmwPlatformMngt::HOST_STATUS_CHANGE);
    CdmwPlatformMngt::HostStatusChange* host_status_change =
            CdmwPlatformMngt::HostStatusChange::_downcast(event.in());
    CPPUNIT_ASSERT(host_status_change->host_status() == CdmwPlatformMngt::HOST_PARTIAL_STARTED);

}

void
test_ccm_services::test_server_exit(
        const std::string& expected_server_issuer_name,
        const std::string& expected_server_process_name,
        CdmwPlatformMngt::External::System_ptr system,
        bool expect_degraded_notif) 
    throw (CORBA::SystemException, std::exception) 
{
    // get connected server
    TestServices_var test_services =
            m_session_context->get_connection_test_services();

    // clear event list
    m_ccm_client->clear_event_list();

    // test call_exit()
    TEST_INFO("   test exit from Server");
    try {
        test_services->call_exit(SERVER_EXIT_CODE);
    } catch (const CORBA::TRANSIENT& e) {
        std::cout << "CORBA::TRANSIENT catch : normal"<< std::endl;
        // Expected exception, since exit() is synchronous.
        // The Server will exit during CORBA request.
    } catch (CORBA::SystemException& e) {
        std::cout << "CORBA::SystemException catch : normal"<< std::endl;
        // Expected exception, since exit() is synchronous.
        // The Server will exit during CORBA request.
        // actually we receive an exception of this kind
    }

    Cdmw::OsSupport::OS::sleep(1000);

    TEST_INFO("      check that notification has been received back");
    CdmwPlatformMngt::Event_var event = m_ccm_client->get_first_event();
    CPPUNIT_ASSERT(event.in() != 0);

    // first event should be ProcessStatusChanged
    CPPUNIT_ASSERT(event->event_kind() == CdmwPlatformMngt::PROCESS_STATUS_CHANGE);
    CdmwPlatformMngt::ProcessStatusChange* fatal_event =
            CdmwPlatformMngt::ProcessStatusChange::_downcast(event.in());
    CPPUNIT_ASSERT(SUPERVISION_ISSUER_NAME == fatal_event->issuer());
    CPPUNIT_ASSERT(fatal_event->process_status() == CdmwPlatformMngt::PROCESS_FAILED_DEATH);

    // get Supervision object for process
    std::string element_path = event->element_path();
    TEST_INFO("      get_element " << element_path);
    CdmwPlatformMngt::ManagedElement_var elt =
            system->get_element(element_path.c_str());

    CdmwPlatformMngt::External::Process_var process =
            CdmwPlatformMngt::External::Process::_narrow(elt.in());
    CdmwPlatformMngt::ProcessExitInfo_var exit_info =
            process->get_last_exit_code();
    CPPUNIT_ASSERT(exit_info->exit_event == CdmwPlatformMngt::EXITED);
    CPPUNIT_ASSERT(exit_info->event_value == SERVER_EXIT_CODE);

    if (expect_degraded_notif) {
        // we expect 2 MODE_CHANGE to DEGRADED notifications
        // and 2 STATUS_CHANGE

        //  - Test application MODE_CHANGE to DEGRADED
        TEST_INFO("      check application mode changed to DEGRADED");
        event = m_ccm_client->get_first_event();
        CPPUNIT_ASSERT(event.in() != 0);
        CPPUNIT_ASSERT(event->event_kind() == CdmwPlatformMngt::APPLICATION_MODE_CHANGE);
        CdmwPlatformMngt::ApplicationModeChange* app_mode_change =
                CdmwPlatformMngt::ApplicationModeChange::_downcast(event.in());
        CPPUNIT_ASSERT(APPLICATION_NAME == app_mode_change->application_name());
        CPPUNIT_ASSERT(app_mode_change->application_mode() == CdmwPlatformMngt::DEGRADED_MODE);

        //  - Test system MODE_CHANGE to DEGRADED
        TEST_INFO("      check system mode changed to DEGRADED");
        event = m_ccm_client->get_first_event();
        CPPUNIT_ASSERT(event.in() != 0);
        CPPUNIT_ASSERT(event->event_kind() == CdmwPlatformMngt::SYSTEM_MODE_CHANGE);
        CdmwPlatformMngt::SystemModeChange* sys_mode_change =
                CdmwPlatformMngt::SystemModeChange::_downcast(event.in());
        CPPUNIT_ASSERT(sys_mode_change->system_mode() == CdmwPlatformMngt::DEGRADED_MODE);

        //  - Test application STATUS_CHANGE to COMPONENT_FAILED_ERROR
        TEST_INFO("      check application status changed to COMPONENT_STATUS_CHANGE");
        event = m_ccm_client->get_first_event();
        CPPUNIT_ASSERT(event.in() != 0);
        CPPUNIT_ASSERT(event->event_kind()
                == CdmwPlatformMngt::COMPONENT_STATUS_CHANGE);
        CdmwPlatformMngt::ComponentStatusChange* component_status_change =
                CdmwPlatformMngt::ComponentStatusChange::_downcast(event.in());
        CPPUNIT_ASSERT(component_status_change->component_status()
                == CdmwPlatformMngt::COMPONENT_FAILED_ERROR);
        
        //  - Test application STATUS_CHANGE to PARTIAL_RUNNING
        TEST_INFO("      check application status changed to PARTIAL_RUNNING");
        event = m_ccm_client->get_first_event();
        CPPUNIT_ASSERT(event.in() != 0);
        CPPUNIT_ASSERT(event->event_kind() == CdmwPlatformMngt::APPLICATION_STATUS_CHANGE);
        CdmwPlatformMngt::ApplicationStatusChange* app_status_change =
                        CdmwPlatformMngt::ApplicationStatusChange::_downcast(event.in());
        CPPUNIT_ASSERT(APPLICATION_NAME == app_status_change->application_name());
        CPPUNIT_ASSERT(app_status_change->application_status() == CdmwPlatformMngt::APPLICATION_PARTIAL_RUNNING);

        //  - Test system STATUS_CHANGE to PARTIAL_STARTED
        TEST_INFO("      check system status changed to PARTIAL_STARTED");
        event = m_ccm_client->get_first_event();
        CPPUNIT_ASSERT(event.in() != 0);
        CPPUNIT_ASSERT(event->event_kind() == CdmwPlatformMngt::SYSTEM_STATUS_CHANGE);
        CdmwPlatformMngt::SystemStatusChange* sys_status_change =
                CdmwPlatformMngt::SystemStatusChange::_downcast(event.in());
        CPPUNIT_ASSERT(sys_status_change->system_status() == CdmwPlatformMngt::SYSTEM_PARTIAL_STARTED);

    } else {

        //  - Test application STATUS_CHANGE to COMPONENT_FAILED_ERROR
        TEST_INFO("      check application status changed to COMPONENT_STATUS_CHANGE");
        event = m_ccm_client->get_first_event();
        CPPUNIT_ASSERT(event.in() != 0);
        CPPUNIT_ASSERT(event->event_kind() == CdmwPlatformMngt::COMPONENT_STATUS_CHANGE);
        CdmwPlatformMngt::ComponentStatusChange* component_status_change =
                CdmwPlatformMngt::ComponentStatusChange::_downcast(event.in());
        CPPUNIT_ASSERT(component_status_change->component_status() == CdmwPlatformMngt::COMPONENT_FAILED_ERROR);
    }
    
    //  - Test host STATUS_CHANGE to PARTIAL_STARTED
    TEST_INFO("      check host status changed to PARTIAL_STARTED");
    event = m_ccm_client->get_first_event();
    CPPUNIT_ASSERT(event.in() != 0);
    CPPUNIT_ASSERT(event->event_kind() == CdmwPlatformMngt::HOST_STATUS_CHANGE);
    CdmwPlatformMngt::HostStatusChange* host_status_change =
            CdmwPlatformMngt::HostStatusChange::_downcast(event.in());
    CPPUNIT_ASSERT(host_status_change->host_status() == CdmwPlatformMngt::HOST_PARTIAL_STARTED);
}



}; // namespace TestCcmServices

