/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO 
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2009. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#ifndef INCL_CCM_NODE_APPLICATION_MANAGER_IMPL_HPP
#define INCL_CCM_NODE_APPLICATION_MANAGER_IMPL_HPP

// Cdmw Files
#include <Foundation/idllib/CdmwDnC.skel.hpp>
#include <Foundation/orbsupport/RefCountServantBase.hpp>

#include <DAnCE/Deployment/Deployment_DomainApplicationManagerC.h>


namespace Cdmw {

namespace CCM {

namespace NodeManager {

/**
 * Implementation of the CdmwDnC::NodeApplicationManager interface.
 */
class NodeApplicationManager_impl : 
    virtual public POA_CdmwDnC::NodeApplicationManager,
    virtual public OrbSupport::RefCountServantBase

{

public:

    /**
     * Constructor.
     *
     * @param poa POA for object creations. 
     *            Note: poa should have RETAIN and IMPLICIT_ACTIVATION policies
     *                  (RootPOA fits)
     * @param dom_app_mgr The DomainApplicationManager to be used
     * @param local_plan_name The DeploymentPlan name
     * @param plan The DeploymentPlan
     */
    NodeApplicationManager_impl(
        CORBA::ORB_ptr orb,
        PortableServer::POA_ptr poa,
        Deployment::DomainApplicationManager_ptr dom_app_mgr,
        const char* local_plan_name,
        const Deployment::DeploymentPlan& plan);
    
    /**
     * Destructor.
     */
    virtual ~NodeApplicationManager_impl() throw();

    /**
     * Call DomainApplicationManager::startLaunch()
     *
     * @return A NodeApplication object for this deployment plan
     * @exception CdmwDnC::StartError If an error occurs during start-up
     */
    virtual CdmwDnC::NodeApplication_ptr 
    start_launch()  
        throw (CORBA::SystemException, CdmwDnC::StartError);

    /**
     * Call DomainApplicationManager::destroyApplication() and deactivate 
     * NodeApplication object
     *
     * @param application The NodeApplication to be destroyed
     * @exception CdmwDnC::InvalidReference If the application reference is unkown
     * @exception CdmwDnC::StopError If an error occurs during destruction
     */
    virtual void 
    destroy_application(CdmwDnC::NodeApplication_ptr application) 
        throw(CORBA::SystemException, CdmwDnC::StopError, CdmwDnC::InvalidReference);


    /**
     * Return DomainApplicationManager object associated to this 
     * NodeApplicationManager
     *
     * @return The DomainApplicationManager object
     */
    inline Deployment::DomainApplicationManager_ptr get_domain_application_manager()
    {
        return Deployment::DomainApplicationManager::_duplicate(m_dom_app_mgr.in());
    }
    
    /**
     * Return DeploymentPlan name associated to this 
     * NodeApplicationManager
     *
     * @return The DeploymentPlan name
     */
    inline const char* get_local_plan_name()
    {
        return m_local_plan_name;
    }

private:
    NodeApplicationManager_impl();
    NodeApplicationManager_impl(const NodeApplicationManager_impl&);
    void operator=(const NodeApplicationManager_impl&);

    /**
     * Add to connections list the external references retrieved from
     * DeploymentPlan's connections
     *
     * @param connections The connections list for external references addition
     *
     * @throw CdmwDnC::StartError If an error occurs retrieving external 
     * references
     */
    void add_external_references(Deployment::Connections& connections)
        throw (CdmwDnC::StartError);
    
    CORBA::ORB_var m_orb;
    PortableServer::POA_var m_poa;
    Deployment::DomainApplicationManager_var m_dom_app_mgr;
    CORBA::String_var m_local_plan_name;
    Deployment::DeploymentPlan m_plan;
    
    
};




} // End of namespace NodeManager

} // End of namespace CCM

} // End of namespace Cdmw

#endif  // INCL_CCM_NODE_APPLICATION_MANAGER_IMPL_HPP

