/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO 
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2009. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#include <ccmnodemanager/NodeApplicationManager_impl.hpp>
#include <ccmnodemanager/NodeApplication_impl.hpp>
#include <ccmnodemanager/Logger.hpp>

#include <Foundation/orbsupport/OrbSupport.hpp>



namespace Cdmw {

namespace CCM {

namespace NodeManager {

    
NodeApplicationManager_impl::NodeApplicationManager_impl(
        CORBA::ORB_ptr orb,
        PortableServer::POA_ptr poa,
        Deployment::DomainApplicationManager_ptr dom_app_mgr,
        const char* local_plan_name,
        const Deployment::DeploymentPlan& plan)
    : m_orb(CORBA::ORB::_duplicate(orb)),
      m_poa(PortableServer::POA::_duplicate(poa)),
      m_dom_app_mgr(Deployment::DomainApplicationManager::_duplicate(dom_app_mgr)),
      m_local_plan_name(local_plan_name),
      m_plan(plan)
{
    CDMW_LOG_FUNCTION(Logger::Get_Logger(), "local_plan_name="<<local_plan_name);
}


NodeApplicationManager_impl::~NodeApplicationManager_impl() 
    throw()
{
    CDMW_LOG_FUNCTION(Logger::Get_Logger(), "");
}


//
// IDL:thalesgroup.com/CdmwDnC/NodeApplicationManager/start_launch:1.0
//
CdmwDnC::NodeApplication_ptr 
NodeApplicationManager_impl::start_launch()  
    throw (CORBA::SystemException, CdmwDnC::StartError)
{
    CDMW_LOG_FUNCTION(Logger::Get_Logger(), "");
    CDMW_DEBUG(Logger::Get_Logger(), "start_launch for " << m_local_plan_name);
    
    // call DomainApplicationManager::startLaunch()
    Deployment::DomainApplication_var domain_app;
    Deployment::Connections_var connections;
    try {
        // no overriding properties (empty sequence)
        Deployment::Properties_var properties = 
            new Deployment::Properties(0);
        
        CDMW_INFO(Logger::Get_Logger(), "Call startLaunch() for " << m_local_plan_name);
        Deployment::Application_var app = 
            m_dom_app_mgr->startLaunch(properties.in(), connections.out());
            
        if (Logger::Get_Logger()->is_debug_enabled()) {
            CDMW_DEBUG(Logger::Get_Logger(), 
                "Returned references for connections (" << connections->length() << ") :");
            for (CORBA::ULong i=0; i < connections->length(); ++i) {
                CDMW_DEBUG(Logger::Get_Logger(), 
                    "   - " << connections[i].name.in() << 
                    " (" << connections[i].endpoint.length() << " endpoints)");
            }
        }
            
        domain_app = Deployment::DomainApplication::_narrow(app.in());
        if (CORBA::is_nil(domain_app.in())) {
            std::ostringstream reason;
            reason << "Deployment::Application received after startLaunch for "
                << m_local_plan_name << " is not a Deployment::DomainApplication";
            CDMW_ERROR(Logger::Get_Logger(), reason.str());
            CdmwDnC::StartError ex;
            ex.name = "";
            ex.reason = reason.str().c_str();
            CDMW_LOG_FUNCTION_EXCEPTION(ex);
            throw ex;
        }
        
    } catch (const Deployment::InvalidProperty& e) {
        CDMW_ERROR(Logger::Get_Logger(), 
            "Deployment::InvalidProperty received during startLaunch for "
            << m_local_plan_name << " ("<<e.name<<" : "<<e.reason<<")");
        CdmwDnC::StartError ex;
        ex.name = e.name;
        ex.reason = e.reason;
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw ex;
    } catch (const Deployment::StartError& e) {
        CDMW_ERROR(Logger::Get_Logger(), 
            "Deployment::StartError received during startLaunch for "
            << m_local_plan_name << " ("<<e.name<<" : "<<e.reason<<")");
        CdmwDnC::StartError ex;
        ex.name = e.name;
        ex.reason = e.reason;
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw ex;
    } catch (const Deployment::ResourceNotAvailable& e) {
        std::ostringstream reason;
        reason << "ResourceNotAvailable for requirement " << e.name 
               << " with type " << e.resourceType 
               << " and property " << e.propertyName << " : ";
        if (strcmp(e.resourceName.in(), "") == 0) {
            reason << "no matching RequirementSatisfier found";
        } else {
            reason << "element " << e.elementName << " with resource "
                   << e.resourceName << " doesn't match"; 
        }
        CDMW_ERROR(Logger::Get_Logger(),
            "Deployment::ResourceNotAvailable received during startLaunch for "
            << m_local_plan_name << " ("<<reason.str()<<")");
        CdmwDnC::PlanError ex;
        ex.name = e.name;
        ex.reason = reason.str().c_str();
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw ex;
    } catch (const Deployment::InvalidNodeExecParameter& e) {
        CDMW_ERROR(Logger::Get_Logger(), 
            "Deployment::InvalidNodeExecParameter received during startLaunch for "
            << m_local_plan_name);
        CdmwDnC::StartError ex;
        ex.name = m_local_plan_name;
        ex.reason = "Deployment::InvalidNodeExecParameter";
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw ex;
    } catch (const Deployment::InvalidComponentExecParameter& e) {
        CDMW_ERROR(Logger::Get_Logger(), 
            "Deployment::InvalidComponentExecParameter received during startLaunch for "
            << m_local_plan_name);
        CdmwDnC::StartError ex;
        ex.name = m_local_plan_name;
        ex.reason = "Deployment::InvalidComponentExecParameter";
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw ex;
    } catch (const CORBA::SystemException& e) {
        CDMW_ERROR(Logger::Get_Logger(),
            "CORBA::SystemException received during startLaunch for "
            << m_local_plan_name << " : "
            << Cdmw::OrbSupport::OrbSupport::exception_to_string(e));
        CdmwDnC::StartError ex;
        ex.name = "Deployment::DomainApplicationManager::startLaunch()";
        ex.reason = Cdmw::OrbSupport::OrbSupport::exception_to_string(e).c_str();
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw ex;
    }
    
    // get external references from DeploymentPlan's connections
    // and add them to connections list
    this->add_external_references(connections.inout());
    
    // create NodeApplication
    CdmwDnC::NodeApplication_var node_app;
    try {
        PortableServer::ServantBase_var servant =
            new NodeApplication_impl(m_poa.in(), 
                                     domain_app.in(), 
                                     connections.in(),
                                     m_local_plan_name.in());
        CORBA::Object_var obj = m_poa->servant_to_reference(servant.in());
        node_app = CdmwDnC::NodeApplication::_narrow(obj.in());
    } catch (const PortableServer::POA::ServantNotActive& e) {
        // POA has'nt IMPLICIT_ACTIVATION
        CDMW_FATAL(Cdmw::CCM::NodeManager::Logger::Get_Logger(), 
            "Error activating CdmwDnC::NodeApplication object : " 
            << Cdmw::OrbSupport::OrbSupport::exception_to_string(e));
        CDMW_ASSERT(false);
    } catch (const PortableServer::POA::WrongPolicy& e) {
        // POA hasn't RETAIN and either UNIQUE_ID or IMPLICIT_ACTIVATION
        CDMW_FATAL(Cdmw::CCM::NodeManager::Logger::Get_Logger(), 
            "Error activating CdmwDnC::NodeApplication object : " 
            << Cdmw::OrbSupport::OrbSupport::exception_to_string(e));
        CDMW_ASSERT(false);
    } catch (const CORBA::SystemException& e) {
        CDMW_FATAL(Cdmw::CCM::NodeManager::Logger::Get_Logger(), 
            "Error activating CdmwDnC::NodeApplication object : " 
            << Cdmw::OrbSupport::OrbSupport::exception_to_string(e));
        CDMW_ASSERT(false);
    }
        
    // return NodeApplication
    return node_app._retn();
    
}


//
// IDL:thalesgroup.com/CdmwDnC/NodeApplicationManager/destroy_manager:1.0
//
void 
NodeApplicationManager_impl::destroy_application(CdmwDnC::NodeApplication_ptr application) 
    throw(CORBA::SystemException, CdmwDnC::StopError, CdmwDnC::InvalidReference)
{
    CDMW_LOG_FUNCTION(Logger::Get_Logger(), "");
    CDMW_DEBUG(Logger::Get_Logger(), "destroy_application for " << m_local_plan_name);
    
    // get DomainApplication attached to NodeApplication
    Deployment::DomainApplication_var domain_app;
    try {
        PortableServer::ServantBase_var servant =
            m_poa->reference_to_servant(application);
        NodeApplication_impl* impl = 
            dynamic_cast<NodeApplication_impl*>(servant.in());
        if (impl == 0) {
            CDMW_ERROR(Logger::Get_Logger(),
                "Specified NodeApplication is not a NodeApplication_impl");
            CdmwDnC::InvalidReference ex;
            CDMW_LOG_FUNCTION_EXCEPTION(ex);
            throw ex;
        }
        domain_app = impl->get_domain_application();
    } catch (const PortableServer::POA::ObjectNotActive& e) {
        // Object is not active
        CDMW_ERROR(Logger::Get_Logger(),
            "Specified NodeApplication is already destroyed");
        CdmwDnC::InvalidReference ex;
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw ex;
    } catch (const PortableServer::POA::WrongAdapter& e) {
        // Object not created by this POA
        CDMW_ERROR(Logger::Get_Logger(),
            "Specified NodeApplication was not created by this CdmwNodeManager");
        CdmwDnC::InvalidReference ex;
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw ex;
    } catch (const PortableServer::POA::WrongPolicy& e) {
        // POA hasn't RETAIN or USE_DEFAULT_SERVANT
        CDMW_FATAL(Cdmw::CCM::NodeManager::Logger::Get_Logger(), 
            "Error destroying CdmwDnC::NodeApplication object : " 
            << Cdmw::OrbSupport::OrbSupport::exception_to_string(e));
        CDMW_ASSERT(false);
    } catch (const CORBA::SystemException& e) {
        CDMW_ERROR(Logger::Get_Logger(),
            "Error destroying NodeApplication : "
            << Cdmw::OrbSupport::OrbSupport::exception_to_string(e));
        CdmwDnC::StopError ex;
        ex.name = "";
        ex.reason = Cdmw::OrbSupport::OrbSupport::exception_to_string(e).c_str();
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw ex;
    }
    
    // destroy DomainApplication
    try {
        m_dom_app_mgr->destroyApplication(domain_app.in());
    } catch (const Deployment::StopError& e) {
        CDMW_ERROR(Logger::Get_Logger(),
            "Deployment::StopError received during destroyApplication for "
            << m_local_plan_name << " ("<<e.name<<" : "<<e.reason<<")");
        CdmwDnC::StopError ex;
        ex.name = e.name;
        ex.reason = e.reason;
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw ex;
    } catch (const CORBA::SystemException& e) {
        CDMW_ERROR(Logger::Get_Logger(),
            "CORBA::SystemException received during destroyApplication for "
            << m_local_plan_name << " : "
            << Cdmw::OrbSupport::OrbSupport::exception_to_string(e));
        CdmwDnC::StopError ex;
        ex.name = "Deployment::DomainApplicationManager::destroyApplication()";
        ex.reason = Cdmw::OrbSupport::OrbSupport::exception_to_string(e).c_str();
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw ex;
    }
    
    // deactivate NodeApplication
    try {
        PortableServer::ObjectId_var oid =
            m_poa->reference_to_id(application);
        m_poa->deactivate_object(oid.in());
    } catch (const PortableServer::POA::ObjectNotActive& e) {
        // Object is not active
        CDMW_ERROR(Logger::Get_Logger(),
            "Specified NodeApplication is already deactivated");
        CdmwDnC::InvalidReference ex;
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw ex;
    } catch (const PortableServer::POA::WrongAdapter& e) {
        // Object not created by this POA
        CDMW_ERROR(Logger::Get_Logger(),
            "Specified NodeApplication was not created by this CdmwNodeManager");
        CdmwDnC::InvalidReference ex;
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw ex;
    } catch (const PortableServer::POA::WrongPolicy& e) {
        // POA hasn't RETAIN
        CDMW_FATAL(Cdmw::CCM::NodeManager::Logger::Get_Logger(), 
            "Error deactivating CdmwDnC::NodeApplication object : " 
            << Cdmw::OrbSupport::OrbSupport::exception_to_string(e));
        CDMW_ASSERT(false);
    } catch (const CORBA::SystemException& e) {
        CDMW_ERROR(Logger::Get_Logger(),
            "Error deactivating NodeApplication : "
            << Cdmw::OrbSupport::OrbSupport::exception_to_string(e));
        CdmwDnC::StopError ex;
        ex.name = "";
        ex.reason = Cdmw::OrbSupport::OrbSupport::exception_to_string(e).c_str();
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw ex;
    }
}

void 
NodeApplicationManager_impl::add_external_references(
        Deployment::Connections& connections)
    throw (CdmwDnC::StartError)
{
    CDMW_LOG_FUNCTION(Logger::Get_Logger(), "");
    
    // for all connections
    for (CORBA::ULong i=0; i < m_plan.connection.length(); i++)
    {
        if (m_plan.connection[i].externalReference.length() > 0
            && m_plan.connection[i].externalReference[0].provider)
        {
            CDMW_DEBUG(Logger::Get_Logger(),
                "For connection " << m_plan.connection[i].name.in() <<
                " retrieve object at location " << 
                m_plan.connection[i].externalReference[0].location);

            CORBA::Object_ptr obj;
            try {
                obj = m_orb->string_to_object(
                m_plan.connection[i].externalReference[0].location.in());
            } catch (const CORBA::Exception& e) {
                std::ostringstream reason;
                reason << "Error retrieving object at location " 
                    << m_plan.connection[i].externalReference[0].location << " : "
                    << Cdmw::OrbSupport::OrbSupport::exception_to_string(e);
                CDMW_ERROR(Logger::Get_Logger(), reason.str());
                CdmwDnC::StartError ex;
                ex.name = m_plan.connection[i].name;
                ex.reason = reason.str().c_str();
                CDMW_LOG_FUNCTION_EXCEPTION(ex);
                throw ex;
            }
            
            if (CORBA::is_nil(obj)) {
                std::ostringstream reason;
                reason << "Error retrieving object at location " 
                    << m_plan.connection[i].externalReference[0].location 
                    << " : object is nil.";
                CDMW_ERROR(Logger::Get_Logger(), reason.str());
                CdmwDnC::StartError ex;
                ex.name = m_plan.connection[i].name;
                ex.reason = reason.str().c_str();
                CDMW_LOG_FUNCTION_EXCEPTION(ex);
                throw ex;
            }
                
            CORBA::ULong indx = connections.length();
            connections.length(indx + 1);
            connections[indx].name = CORBA::string_dup(m_plan.connection[i].name.in());
            connections[indx].endpoint.length(1L);
            connections[indx].endpoint[0] = obj;
        }
    }
}



} // End of namespace NodeManager

} // End of namespace CCM

} // End of namespace Cdmw


