/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO 
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2009. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#include <ccmnodemanager/NodeApplication_impl.hpp>
#include <ccmnodemanager/Logger.hpp>

#include <Foundation/orbsupport/OrbSupport.hpp>


namespace Cdmw {

namespace CCM {

namespace NodeManager {

    
NodeApplication_impl::NodeApplication_impl(
        PortableServer::POA_ptr poa,
        Deployment::DomainApplication_ptr domain_app,
        const Deployment::Connections& connections,
        const char* local_plan_name)
    : m_poa(PortableServer::POA::_duplicate(poa)),
      m_domain_app(Deployment::DomainApplication::_duplicate(domain_app)),
      m_local_plan_name(local_plan_name)
{
    CDMW_LOG_FUNCTION(Logger::Get_Logger(), "local_plan_name="<<local_plan_name);
    // copy Connections list
    m_connections = new Deployment::Connections(connections);
}


NodeApplication_impl::~NodeApplication_impl() 
    throw()
{
    CDMW_LOG_FUNCTION(Logger::Get_Logger(), "");
}


//
// IDL:thalesgroup.com/CdmwDnC/NodeApplication/finish_launch:1.0
//
void 
NodeApplication_impl::finish_launch()  
    throw (CORBA::SystemException, CdmwDnC::StartError)
{
    CDMW_LOG_FUNCTION(Logger::Get_Logger(), "");
    CDMW_DEBUG(Logger::Get_Logger(), "finish_launch for " << m_local_plan_name);
    
    // call DomainApplication::finishLaunch()
    try {
        // don't start yet (use false as last parameter)
        CDMW_INFO(Logger::Get_Logger(), "Call finishLaunch() for " << m_local_plan_name);
        m_domain_app->finishLaunch(m_connections.in(), false);
            
    } catch (const Deployment::InvalidConnection& e) {
        CDMW_ERROR(Logger::Get_Logger(), 
            "Deployment::InvalidConnection received during finishLaunch for "
            << m_local_plan_name << " ("<<e.name<<" : "<<e.reason<<")");
        CdmwDnC::StartError ex;
        ex.name = e.name;
        ex.reason = e.reason;
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw ex;
    } catch (const Deployment::StartError& e) {
        CDMW_ERROR(Logger::Get_Logger(), 
            "Deployment::StartError received during finishLaunch for "
            << m_local_plan_name << " ("<<e.name<<" : "<<e.reason<<")");
        CdmwDnC::StartError ex;
        ex.name = e.name;
        ex.reason = e.reason;
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw ex;
    } catch (const CORBA::SystemException& e) {
        CDMW_ERROR(Logger::Get_Logger(),
            "CORBA::SystemException received during finishLaunch for "
            << m_local_plan_name << " : "
            << Cdmw::OrbSupport::OrbSupport::exception_to_string(e));
        CdmwDnC::StartError ex;
        ex.name = "Deployment::DomainApplication::finishLaunch()";
        ex.reason = Cdmw::OrbSupport::OrbSupport::exception_to_string(e).c_str();
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw ex;
    }
}


//
// IDL:thalesgroup.com/CdmwDnC/NodeApplication/start:1.0
//
void 
NodeApplication_impl::start() 
    throw (CORBA::SystemException, CdmwDnC::StartError)
{
    CDMW_LOG_FUNCTION(Logger::Get_Logger(), "");
    CDMW_DEBUG(Logger::Get_Logger(), "start for " << m_local_plan_name);
    
    // call DomainApplication::start()
    try {
        CDMW_INFO(Logger::Get_Logger(), "Call start() for " << m_local_plan_name);
        m_domain_app->start();
            
    } catch (const Deployment::StartError& e) {
        CDMW_ERROR(Logger::Get_Logger(), 
            "Deployment::StartError received during start for "
            << m_local_plan_name << " ("<<e.name<<" : "<<e.reason<<")");
        CdmwDnC::StartError ex;
        ex.name = e.name;
        ex.reason = e.reason;
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw ex;
    } catch (const CORBA::SystemException& e) {
        CDMW_ERROR(Logger::Get_Logger(),
            "CORBA::SystemException received during start for "
            << m_local_plan_name << " : "
            << Cdmw::OrbSupport::OrbSupport::exception_to_string(e));
        CdmwDnC::StartError ex;
        ex.name = "Deployment::DomainApplication::start()";
        ex.reason = Cdmw::OrbSupport::OrbSupport::exception_to_string(e).c_str();
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw ex;
    }
}


} // End of namespace NodeManager

} // End of namespace CCM

} // End of namespace Cdmw


