/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO 
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2009. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#include <ccmnodemanager/NodeManager_impl.hpp>
#include <ccmnodemanager/NodeApplicationManager_impl.hpp>
#include <ccmnodemanager/Logger.hpp>

#include <Foundation/orbsupport/OrbSupport.hpp>




namespace Cdmw {

namespace CCM {

namespace NodeManager {

    
NodeManager_impl::NodeManager_impl(CORBA::ORB_ptr orb,
                                   PortableServer::POA_ptr poa,
                                   Deployment::ExecutionManager_ptr exec_mgr,
                                   const char* plans_directory)
    throw (CORBA::SystemException)
    : m_orb(CORBA::ORB::_duplicate(orb)),
      m_poa(PortableServer::POA::_duplicate(poa)),
      m_exec_mgr(Deployment::ExecutionManager::_duplicate(exec_mgr))
{
    CDMW_LOG_FUNCTION(Logger::Get_Logger(), "plans_directory="<<plans_directory);
    try {
        CDRDeploymentPlanReader* reader = 
            new CDRDeploymentPlanReader(orb, plans_directory);
        m_plan_reader = std::auto_ptr<CDRDeploymentPlanReader>(reader);
    } catch (const CORBA::SystemException& e) {
        // destructor will be called, set refcount to 0 before, avoiding assert
        _set_ref_count(0);
        throw;
    }
}


NodeManager_impl::~NodeManager_impl() 
    throw()
{
    CDMW_LOG_FUNCTION(Logger::Get_Logger(), "");
}


//
// IDL:thalesgroup.com/CdmwDnC/NodeManager/prepare_plan:1.0
//
CdmwDnC::NodeApplicationManager_ptr 
NodeManager_impl::prepare_plan(const char* local_plan_name)  
    throw (CORBA::SystemException, CdmwDnC::StartError, CdmwDnC::PlanError)
{
    CDMW_LOG_FUNCTION(Logger::Get_Logger(), "local_plan_name="<<local_plan_name);
    
    // read plan file
    CDMW_INFO(Logger::Get_Logger(), "Read DeploymentPlan " << local_plan_name);
    Deployment::DeploymentPlan_var plan = 
        m_plan_reader->read_deployment_plan(local_plan_name);
    
    // call ExecutionManager::preparePlan()
    Deployment::DomainApplicationManager_var dom_app_mgr;
    try {
        CDMW_INFO(Logger::Get_Logger(), "Call preparePlan() for " << local_plan_name);
        Deployment::ResourceCommitmentManager_var resource_commit_mgr = 
            Deployment::ResourceCommitmentManager::_nil();
        dom_app_mgr =
            m_exec_mgr->preparePlan(plan.in(), resource_commit_mgr.in());
    } catch (const Deployment::StartError& e) {
        CDMW_ERROR(Logger::Get_Logger(),
            "Deployment::StartError received during preparePlan for "
            << local_plan_name << " (reason: "<<e.reason<<")");
        CdmwDnC::StartError ex;
        ex.name = e.name;
        ex.reason = e.reason;
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw ex;
    } catch (const Deployment::PlanError& e) {
        CDMW_ERROR(Logger::Get_Logger(),
            "Deployment::PlanError received during preparePlan for "
            << local_plan_name << " (on "<<e.name<<" : "<<e.reason<<")");
        CdmwDnC::PlanError ex;
        ex.name = e.name;
        ex.reason = e.reason;
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw ex;
    } catch (const Deployment::ResourceNotAvailable& e) {
        std::ostringstream reason;
        reason << "ResourceNotAvailable for requirement " << e.name 
               << " with type " << e.resourceType 
               << " and property " << e.propertyName << " : ";
        if (strcmp(e.resourceName.in(), "") == 0) {
            reason << "no matching RequirementSatisfier found";
        } else {
            reason << "element " << e.elementName << " with resource "
                   << e.resourceName << " doesn't match"; 
        }
        CDMW_ERROR(Logger::Get_Logger(),
            "Deployment::ResourceNotAvailable received during preparePlan for "
            << local_plan_name << " ("<<reason.str()<<")");
        CdmwDnC::PlanError ex;
        ex.name = e.name;
        ex.reason = reason.str().c_str();
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw ex;
    } catch (const CORBA::SystemException& e) {
        CDMW_ERROR(Logger::Get_Logger(),
            "CORBA::SystemException received during preparePlan for "
            << local_plan_name << " : "
            << Cdmw::OrbSupport::OrbSupport::exception_to_string(e));
        CdmwDnC::StartError ex;
        ex.name = "Deployment::ExecutionManager::preparePlan()";
        ex.reason = Cdmw::OrbSupport::OrbSupport::exception_to_string(e).c_str();
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw ex;
    }
    
    // create NodeApplicationManager
    CdmwDnC::NodeApplicationManager_var node_app_mgr;
    try {
        PortableServer::ServantBase_var servant =
            new NodeApplicationManager_impl(m_orb.in(),
                                            m_poa.in(), 
                                            dom_app_mgr.in(), 
                                            local_plan_name,
                                            plan.in());
        CORBA::Object_var obj = m_poa->servant_to_reference(servant.in());
        node_app_mgr = CdmwDnC::NodeApplicationManager::_narrow(obj.in());
    } catch (const PortableServer::POA::ServantNotActive& e) {
        // POA has'nt IMPLICIT_ACTIVATION
        CDMW_FATAL(Cdmw::CCM::NodeManager::Logger::Get_Logger(), 
            "Error activating CdmwDnC::NodeApplicationManager object : " 
            << Cdmw::OrbSupport::OrbSupport::exception_to_string(e));
        CDMW_ASSERT(false);
    } catch (const PortableServer::POA::WrongPolicy& e) {
        // POA hasn't RETAIN and either UNIQUE_ID or IMPLICIT_ACTIVATION
        CDMW_FATAL(Cdmw::CCM::NodeManager::Logger::Get_Logger(), 
            "Error activating CdmwDnC::NodeApplicationManager object : " 
            << Cdmw::OrbSupport::OrbSupport::exception_to_string(e));
        CDMW_ASSERT(false);
    } catch (const CORBA::SystemException& e) {
        CDMW_FATAL(Cdmw::CCM::NodeManager::Logger::Get_Logger(), 
            "Error activating CdmwDnC::NodeApplicationManager object : " 
            << Cdmw::OrbSupport::OrbSupport::exception_to_string(e));
        CDMW_ASSERT(false);
    }
        
    // return NodeApplicationManager
    return node_app_mgr._retn();
}


//
// IDL:thalesgroup.com/CdmwDnC/NodeManager/destroy_manager:1.0
//
void 
NodeManager_impl::destroy_manager(CdmwDnC::NodeApplicationManager_ptr appManager) 
    throw(CORBA::SystemException, CdmwDnC::StopError, CdmwDnC::InvalidReference)
{
    CDMW_LOG_FUNCTION(Logger::Get_Logger(), "");
    
    // get DomainApplicationManager attached to NodeApplicationManager
    Deployment::DomainApplicationManager_var dom_app_mgr;
    const char* local_plan_name = 0;
    try {
        PortableServer::ServantBase_var servant =
            m_poa->reference_to_servant(appManager);
        NodeApplicationManager_impl* impl = 
            dynamic_cast<NodeApplicationManager_impl*>(servant.in());
        if (impl == 0) {
            CDMW_ERROR(Logger::Get_Logger(),
                "Specified NodeApplicationManager is not a NodeApplicationManager_impl");
            CdmwDnC::InvalidReference ex;
            CDMW_LOG_FUNCTION_EXCEPTION(ex);
            throw ex;
        }
        dom_app_mgr = impl->get_domain_application_manager();
        local_plan_name = impl->get_local_plan_name();
        
    } catch (const PortableServer::POA::ObjectNotActive& e) {
        // Object is not active
        CDMW_ERROR(Logger::Get_Logger(),
            "Specified NodeApplicationManager is already destroyed");
        CdmwDnC::InvalidReference ex;
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw ex;
    } catch (const PortableServer::POA::WrongAdapter& e) {
        // Object not created by this POA
        CDMW_ERROR(Logger::Get_Logger(),
            "Specified NodeApplicationManager was not created by this CdmwNodeManager");
        CdmwDnC::InvalidReference ex;
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw ex;
    } catch (const PortableServer::POA::WrongPolicy& e) {
        // POA hasn't RETAIN or USE_DEFAULT_SERVANT
        CDMW_FATAL(Cdmw::CCM::NodeManager::Logger::Get_Logger(), 
            "Error destroying CdmwDnC::NodeApplicationManager object : " 
            << Cdmw::OrbSupport::OrbSupport::exception_to_string(e));
        CDMW_ASSERT(false);
    } catch (const CORBA::SystemException& e) {
        CDMW_ERROR(Logger::Get_Logger(),
            "Error destroying NodeApplicationManager : "
            << Cdmw::OrbSupport::OrbSupport::exception_to_string(e));
        CdmwDnC::StopError ex;
        ex.name = "";
        ex.reason = Cdmw::OrbSupport::OrbSupport::exception_to_string(e).c_str();
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw ex;
    }
    
    CDMW_DEBUG(Logger::Get_Logger(), 
        "destroy_manager for " << local_plan_name);
    
    // destroy DomainApplicationManager
    try {
        m_exec_mgr->destroyManager(dom_app_mgr.in());
    } catch (const Deployment::StopError& e) {
        CDMW_ERROR(Logger::Get_Logger(),
            "Deployment::StopError received during destroyManager for "
            << local_plan_name << " ("<<e.name<<" : "<<e.reason<<")");
        CdmwDnC::StopError ex;
        ex.name = e.name;
        ex.reason = e.reason;
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw ex;
    } catch (const CORBA::SystemException& e) {
        CDMW_ERROR(Logger::Get_Logger(),
            "CORBA::SystemException received during destroyManager for "
            << local_plan_name << " : "
            << Cdmw::OrbSupport::OrbSupport::exception_to_string(e));
        CdmwDnC::StopError ex;
        ex.name = "Deployment::ExecutionManager::destroyManager()";
        ex.reason = Cdmw::OrbSupport::OrbSupport::exception_to_string(e).c_str();
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw ex;
    }
    
    // deactivate NodeApplicationManager
    try {
        PortableServer::ObjectId_var oid =
            m_poa->reference_to_id(appManager);
        m_poa->deactivate_object(oid.in());
    } catch (const PortableServer::POA::ObjectNotActive& e) {
        // Object is not active
        CDMW_ERROR(Logger::Get_Logger(),
            "Specified NodeApplicationManager is already deactivated");
        CdmwDnC::InvalidReference ex;
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw ex;
    } catch (const PortableServer::POA::WrongAdapter& e) {
        // Object not created by this POA
        CDMW_ERROR(Logger::Get_Logger(),
            "Specified NodeApplicationManager was not created by this CdmwNodeManager");
        CdmwDnC::InvalidReference ex;
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw ex;
    } catch (const PortableServer::POA::WrongPolicy& e) {
        // POA hasn't RETAIN
        CDMW_FATAL(Cdmw::CCM::NodeManager::Logger::Get_Logger(), 
            "Error deactivating CdmwDnC::NodeApplicationManager object : " 
            << Cdmw::OrbSupport::OrbSupport::exception_to_string(e));
        CDMW_ASSERT(false);
    } catch (const CORBA::SystemException& e) {
        CDMW_ERROR(Logger::Get_Logger(),
            "Error deactivating NodeApplicationManager : "
            << Cdmw::OrbSupport::OrbSupport::exception_to_string(e));
        CdmwDnC::StopError ex;
        ex.name = "";
        ex.reason = Cdmw::OrbSupport::OrbSupport::exception_to_string(e).c_str();
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw ex;
    }    
}


} // End of namespace NodeManager

} // End of namespace CCM

} // End of namespace Cdmw


