/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO 
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2009. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

#include "Foundation/orbsupport/OrbSupport.hpp"
#include "ConfAndPlug/cdmwinit/CdmwInterface.hpp"
#include "Foundation/ossupport/OS.hpp"
#include "Foundation/ossupport/Signals.hpp"
#include "Foundation/ossupport/SignalHandler.hpp"
#include "Foundation/commonsvcs/naming/NamingInterface.hpp"

#include <ccmnodemanager/NodeManager_impl.hpp>
#include <ccmnodemanager/Logger.hpp>


namespace {
    const int SUCCESS = 0;
    const int FAILURE = 1;

    const int POA_THREAD_POOL_SIZE = 5;    
    
    // Name of ExecutionManager in NameService
    const std::string EXECUTION_MANAGER_NAME = "ExecutionManager";
    
    // Name for registration as corbaloc
    const std::string CORBALOC_NAME = "CdmwNodeManager";
    
    // Name for registration in NameService
    const std::string NS_REGISTRATION_NAME = CORBALOC_NAME;
    
    // Name for rebinding "hostname" NameContext in NameService
    const std::string HOSTNAME_NC_REBIND_NAME = "DeploymentPlans";
    
    
    const std::string HELP_OPTION    = "--help";
    const std::string CDR_DIR_OPTION = "--cdr-dir";
    
    /*
     * Print the usage.
     *
     * @param out the output stream.
     * @param programName the program name.
     */
    static void
    usage(std::ostream& out, const std::string& programName)
    {
        using std::endl;

        out << "usage: " << programName << " options...\n"
            << "options:\n"
            << "  "<<HELP_OPTION<<"               Display the help.\n"
            << "  "<<CDR_DIR_OPTION<<"=DIR   Directory of DeploymentPlans in CDR format\n"
            << std::endl;
    }
    
};



class ProcessControl : public Cdmw::CdmwInit::ProcessControl
{
public:
    
    ProcessControl(CORBA::ORB_ptr orb,
                   CosNaming::NamingContext_ptr naming,
                   const char* plans_directory)
        throw(CORBA::SystemException)
        : m_orb(CORBA::ORB::_duplicate(orb)),
          m_naming(naming),
          m_plans_directory(plans_directory)
    {
        //
        // Get POA for object activations
        // (use RootPOA)
        //
        CORBA::Object_var obj = m_orb->resolve_initial_references("RootPOA");
        m_poa = PortableServer::POA::_narrow(obj.in());
    }
    
    virtual ~ProcessControl() throw () {};
    
    virtual void on_initialise(const CdmwPlatformMngtBase::StartupKind& startup_kind)
        throw(CORBA::SystemException)
    {
        // get ExecutionManager from NameService
        Deployment::ExecutionManager_var exec_mgr;
        try {
            CORBA::Object_var obj = m_naming.resolve(EXECUTION_MANAGER_NAME);
            exec_mgr = Deployment::ExecutionManager::_narrow(obj.in());
            if (CORBA::is_nil(exec_mgr.in())) {
                CDMW_FATAL(Cdmw::CCM::NodeManager::Logger::Get_Logger(), 
                    "Object found via \"" << EXECUTION_MANAGER_NAME
                    << "\" name is not a Deployment::ExecutionManager");
                CDMW_ASSERT(false);
            }
        } catch (const CosNaming::NamingContext::NotFound& e) {
            CDMW_FATAL(Cdmw::CCM::NodeManager::Logger::Get_Logger(), 
                "No \"" << EXECUTION_MANAGER_NAME 
                << "\" object found in NameService");
            CDMW_ASSERT(false);
        } catch (const CosNaming::NamingContext::CannotProceed& e) {
            CDMW_FATAL(Cdmw::CCM::NodeManager::Logger::Get_Logger(), 
                "CannotProceed exception raised resolving \"" 
                << EXECUTION_MANAGER_NAME << "\" in NameService");
            CDMW_ASSERT(false);
        } catch (const CosNaming::NamingContext::InvalidName& e) {
            CDMW_FATAL(Cdmw::CCM::NodeManager::Logger::Get_Logger(), 
                "\"" << EXECUTION_MANAGER_NAME << "\" is an invalid name");
            CDMW_ASSERT(false);
        } catch (const CORBA::SystemException& e) {
            CDMW_FATAL(Cdmw::CCM::NodeManager::Logger::Get_Logger(), 
                "Error retrieving Object from NameService with name \"" 
                << EXECUTION_MANAGER_NAME << "\" : " 
                << Cdmw::OrbSupport::OrbSupport::exception_to_string(e));
            CDMW_ASSERT(false);
        }

        // get "hostname" NameContext from NameService (bound by ExecutionManager)
        CosNaming::NamingContext_var hostname_nc;
        std::string hostname_nc_name;
        try {
            // transform "hostname" as NameContext id
            // (cannot directly use it in resolve since it could have some '.')
            std::string hostname = Cdmw::OsSupport::OS::get_hostname();
            CosNaming::Name n(1);
            n.length(1);
            n[0].id = CORBA::string_dup(hostname.c_str());
            hostname_nc_name = m_naming.to_string(n);
            
            CORBA::Object_var obj = m_naming.resolve(hostname_nc_name);
            hostname_nc = CosNaming::NamingContext::_narrow(obj.in());
            if (CORBA::is_nil(hostname_nc.in())) {
                CDMW_FATAL(Cdmw::CCM::NodeManager::Logger::Get_Logger(), 
                    "Object found via \"" << hostname_nc_name
                    << "\" name is not a CosNaming::NamingContext");
                CDMW_ASSERT(false);
            }
            
        } catch (const Cdmw::CommonSvcs::Naming::InvalidNameException& e) {
            CDMW_FATAL(Cdmw::CCM::NodeManager::Logger::Get_Logger(), 
                "Hostname \"" << Cdmw::OsSupport::OS::get_hostname() 
                << "\" cannot be transformed as NameContext (InvalidNameException)");
            CDMW_ASSERT(false);
        } catch (const CosNaming::NamingContext::NotFound& e) {
            CDMW_FATAL(Cdmw::CCM::NodeManager::Logger::Get_Logger(), 
                "No \"" << hostname_nc_name << "\" object found in NameService");
            CDMW_ASSERT(false);
        } catch (const CosNaming::NamingContext::CannotProceed& e) {
            CDMW_FATAL(Cdmw::CCM::NodeManager::Logger::Get_Logger(), 
                "CannotProceed exception raised resolving \"" 
                << hostname_nc_name << "\" in NameService");
            CDMW_ASSERT(false);
        } catch (const CosNaming::NamingContext::InvalidName& e) {
            CDMW_FATAL(Cdmw::CCM::NodeManager::Logger::Get_Logger(), 
                "\"" << hostname_nc_name << "\" is an invalid name");
            CDMW_ASSERT(false);
        } catch (const CORBA::SystemException& e) {
            CDMW_FATAL(Cdmw::CCM::NodeManager::Logger::Get_Logger(), 
                "Error retrieving Object from NameService with name \"" 
                << hostname_nc_name << "\" : " 
                << Cdmw::OrbSupport::OrbSupport::exception_to_string(e));
            CDMW_ASSERT(false);
        }
        
        // bind "hostname" NameContext as "NameService" corbaloc
        try {
            Cdmw::OrbSupport::OrbSupport::bind_object_to_corbaloc(
                m_orb.in(),
                "NameService",
                hostname_nc.in());
        } catch (const CORBA::SystemException& e) {
            CDMW_FATAL(Cdmw::CCM::NodeManager::Logger::Get_Logger(), 
                "Error binding NameService to corbaloc : " 
                << Cdmw::OrbSupport::OrbSupport::exception_to_string(e));
            CDMW_ASSERT(false);
        }
        
        // rebind "hostname" NameContext as a non-hostname dependent 
        // NameContext in NameService (HOSTNAME_NC_REBIND_NAME)
        try {
            m_naming.rebind(HOSTNAME_NC_REBIND_NAME, hostname_nc.in(), true);
        } catch (const CosNaming::NamingContext::NotFound& e) {
            CDMW_FATAL(Cdmw::CCM::NodeManager::Logger::Get_Logger(), 
                "NotFound exception raised rebinding \"" << hostname_nc_name 
                << "\" as \"" << HOSTNAME_NC_REBIND_NAME << "\"");
            CDMW_ASSERT(false);
        } catch (const CosNaming::NamingContext::CannotProceed& e) {
            CDMW_FATAL(Cdmw::CCM::NodeManager::Logger::Get_Logger(), 
                "CannotProceed exception raised rebinding \"" << hostname_nc_name 
                << "\" as \"" << HOSTNAME_NC_REBIND_NAME << "\"");
            CDMW_ASSERT(false);
        } catch (const CosNaming::NamingContext::InvalidName& e) {
            CDMW_FATAL(Cdmw::CCM::NodeManager::Logger::Get_Logger(), 
                "\"" << HOSTNAME_NC_REBIND_NAME << "\" is an invalid name");
            CDMW_ASSERT(false);
        } catch (const CORBA::SystemException& e) {
            CDMW_FATAL(Cdmw::CCM::NodeManager::Logger::Get_Logger(), 
                "Error rebinding \"" << hostname_nc_name 
                << "\" as \"" << HOSTNAME_NC_REBIND_NAME << "\" : "
                << Cdmw::OrbSupport::OrbSupport::exception_to_string(e));
            CDMW_ASSERT(false);
        }
        
        
        // create NodeManager
        try {
            PortableServer::ServantBase_var servant =
                new Cdmw::CCM::NodeManager::NodeManager_impl(
                    m_orb.in(),
                    m_poa.in(), 
                    exec_mgr.in(), 
                    m_plans_directory);
            CORBA::Object_var obj = m_poa->servant_to_reference(servant.in());
            m_node_mgr = CdmwDnC::NodeManager::_narrow(obj.in());
        } catch (const PortableServer::POA::ServantNotActive& e) {
            // POA has'nt IMPLICIT_ACTIVATION
            CDMW_FATAL(Cdmw::CCM::NodeManager::Logger::Get_Logger(), 
                "Error activating CdmwDnC::NodeManager object : " 
                << Cdmw::OrbSupport::OrbSupport::exception_to_string(e));
            CDMW_ASSERT(false);
        } catch (const PortableServer::POA::WrongPolicy& e) {
            // POA hasn't RETAIN and either UNIQUE_ID or IMPLICIT_ACTIVATION
            CDMW_FATAL(Cdmw::CCM::NodeManager::Logger::Get_Logger(), 
                "Error activating CdmwDnC::NodeManager object : " 
                << Cdmw::OrbSupport::OrbSupport::exception_to_string(e));
            CDMW_ASSERT(false);
        } catch (const CORBA::SystemException& e) {
            CDMW_FATAL(Cdmw::CCM::NodeManager::Logger::Get_Logger(), 
                "Error activating CdmwDnC::NodeManager object : " 
                << Cdmw::OrbSupport::OrbSupport::exception_to_string(e));
            CDMW_ASSERT(false);
        }
        
        // bind NodeManager to corbaloc
        try {
            Cdmw::OrbSupport::OrbSupport::bind_object_to_corbaloc(
                m_orb.in(),
                CORBALOC_NAME,
                m_node_mgr.in());
        } catch (const CORBA::SystemException& e) {
            CDMW_FATAL(Cdmw::CCM::NodeManager::Logger::Get_Logger(), 
                "Error binding " << CORBALOC_NAME << " to corbaloc : " 
                << Cdmw::OrbSupport::OrbSupport::exception_to_string(e));
            CDMW_ASSERT(false);
        }
        
        // register NodeManager in NameService
        try {
            m_naming.rebind(NS_REGISTRATION_NAME,
                            m_node_mgr.in());
        
        } catch (const CosNaming::NamingContext::NotFound& e) {
            CDMW_FATAL(Cdmw::CCM::NodeManager::Logger::Get_Logger(), 
                "Error registering " << NS_REGISTRATION_NAME << " in NameService : " 
                << Cdmw::OrbSupport::OrbSupport::exception_to_string(e));
            CDMW_ASSERT(false);
        } catch (const CosNaming::NamingContext::CannotProceed& e) {
            CDMW_FATAL(Cdmw::CCM::NodeManager::Logger::Get_Logger(), 
                "Error registering " << NS_REGISTRATION_NAME << " in NameService : " 
                << Cdmw::OrbSupport::OrbSupport::exception_to_string(e));
            CDMW_ASSERT(false);
        } catch (const CosNaming::NamingContext::InvalidName& e) {
            CDMW_FATAL(Cdmw::CCM::NodeManager::Logger::Get_Logger(), 
                "Error registering " << NS_REGISTRATION_NAME << " in NameService : " 
                << Cdmw::OrbSupport::OrbSupport::exception_to_string(e));
            CDMW_ASSERT(false);
        } catch (const CORBA::SystemException& e) {
            CDMW_FATAL(Cdmw::CCM::NodeManager::Logger::Get_Logger(), 
                "Error registering " << NS_REGISTRATION_NAME << " in NameService : " 
                << Cdmw::OrbSupport::OrbSupport::exception_to_string(e));
            CDMW_ASSERT(false);
        }
        
        CDMW_INFO(Cdmw::CCM::NodeManager::Logger::Get_Logger(),
            CORBALOC_NAME << " on " << Cdmw::OsSupport::OS::get_hostname() 
            << " is ready");
    }
    
    virtual void on_run()
        throw(CdmwPlatformMngt::ProcessDelegate::NotReadyToRun, 
          CORBA::SystemException)
    {
        // nothing to do
    }
    
    virtual void on_stop()
        throw(CORBA::SystemException)
    {
        // deatcivate NodeManager
        try {
            PortableServer::ObjectId_var oid =
                m_poa->reference_to_id(m_node_mgr.in());
            m_poa->deactivate_object(oid.in());
        } catch (const PortableServer::POA::ObjectNotActive& e) {
            // Object is not active
            CDMW_ERROR(Cdmw::CCM::NodeManager::Logger::Get_Logger(), 
                "Error deactivating CdmwDnC::NodeManager object : " 
                << Cdmw::OrbSupport::OrbSupport::exception_to_string(e));
            // continue anyway
        } catch (const PortableServer::POA::WrongAdapter& e) {
            // Object not created by this POA
            CDMW_ERROR(Cdmw::CCM::NodeManager::Logger::Get_Logger(), 
                "Error deactivating CdmwDnC::NodeManager object : " 
                << Cdmw::OrbSupport::OrbSupport::exception_to_string(e));
            // continue anyway
        } catch (const PortableServer::POA::WrongPolicy& e) {
            // POA hasn't RETAIN
            CDMW_ERROR(Cdmw::CCM::NodeManager::Logger::Get_Logger(), 
                "Error deactivating CdmwDnC::NodeManager object : " 
                << Cdmw::OrbSupport::OrbSupport::exception_to_string(e));
            // continue anyway
        } catch (const CORBA::SystemException& e) {
            CDMW_ERROR(Cdmw::CCM::NodeManager::Logger::Get_Logger(), 
                "Error deactivating CdmwDnC::NodeManager object : " 
                << Cdmw::OrbSupport::OrbSupport::exception_to_string(e));
            // continue anyway
        }
    }
    
private:

    CORBA::ORB_var m_orb;
    PortableServer::POA_var m_poa;
    Cdmw::CommonSvcs::Naming::NamingInterface m_naming;
    
    const char* m_plans_directory;
    
    CdmwDnC::NodeManager_var m_node_mgr;
};




int
main(int argc, char* argv[])
{
    int status = SUCCESS;

    CORBA::ORB_var orb = CORBA::ORB::_nil();
    
    // parse options
    std::string option_val;
    
    // HELP_OPTION
    option_val = Cdmw::OsSupport::OS::get_option_value(argc, argv, HELP_OPTION);
    if (option_val != "no") 
    {
        usage(std::cout, argv[0]);
        return SUCCESS;
    }
    
    // CDR_DIR_OPTION
    std::string cdr_directory;
    option_val = Cdmw::OsSupport::OS::get_option_value(argc, argv, CDR_DIR_OPTION);
    if (option_val == "no") 
    {
        std::cerr << CDR_DIR_OPTION << " is required.\n" << std::endl;
        usage(std::cout, argv[0]);
        return FAILURE;
    } 
    else if (option_val == "yes")
    {
        std::cerr << CDR_DIR_OPTION << " is not set.\n" << std::endl;
        usage(std::cout, argv[0]);
        return FAILURE;
    } 
    else
    {
        cdr_directory = option_val;
    }
    

    try {
        // initialise the LogManager
        Cdmw::Logging::LogManager::Init(argc, argv);
         
        // Call CDMW_preORBInit
        Cdmw::CdmwInit::CDMW_preORBInit(argc, argv);

        // call ORB_init
        Cdmw::OrbSupport::StrategyList orb_strategies;
        orb_strategies.add_OrbThreaded();
        orb_strategies.add_PoaThreadPool(POA_THREAD_POOL_SIZE);
        orb = Cdmw::OrbSupport::OrbSupport::ORB_init(argc, argv, orb_strategies);

        //
        // Get the root POA 
        //
        CORBA::Object_var obj = orb->resolve_initial_references("RootPOA");
        PortableServer::POA_var rootPOA = PortableServer::POA::_narrow(obj.in());

        //
        // Get Naming Service
        //
        CosNaming::NamingContext_var naming;
        try {
            CORBA::Object_var obj = 
                orb->resolve_initial_references("NameService");
            naming = CosNaming::NamingContext::_narrow(obj.in());
            if (CORBA::is_nil(naming.in())) {
                throw Cdmw::InitErrorException(
                    "NameService found by resolve_initial_references is not "
                    "a CosNaming::NamingContext");
            }
            
        } catch (const CORBA::ORB::InvalidName& e) {
            throw Cdmw::InitErrorException(
                "NameService not found by resolve_initial_references");
        } catch (const CORBA::SystemException& e) {
            std::string msg = "Error retrieving NameService: ";
            msg += Cdmw::OrbSupport::OrbSupport::exception_to_string(e);
            throw Cdmw::InitErrorException(msg);
        }
        
        //
        // Run implementation (Activate the POA manager)
        //
        PortableServer::POAManager_var poaManager = rootPOA->the_POAManager();
        poaManager->activate();

        // Create ProcessControl
        Cdmw::CdmwInit::ProcessControl_var process_ctrl =
            new ProcessControl(orb.in(), naming.in(), cdr_directory.c_str());
        
        // Call CDMW_init
        Cdmw::CdmwInit::CDMW_init(orb.in(), argc, argv, process_ctrl.in());

        // Setup signal handling for ORB to be shut down
        Cdmw::OrbSupport::OrbSupport::install_orb_shutdown_signal_handlers(orb.in());

        // ORB run
        orb->run();

        Cdmw::OrbSupport::OrbSupport::uninstall_orb_shutdown_signal_handlers(orb.in());

    } catch (const Cdmw::Exception & ex) {
        CDMW_FATAL(Cdmw::CCM::NodeManager::Logger::Get_Logger(), ex.what());
        status = FAILURE;
    } catch(const CORBA::Exception& ex)  {        
        CDMW_FATAL(Cdmw::CCM::NodeManager::Logger::Get_Logger(), ex);
        status = FAILURE;
    }

    if(!CORBA::is_nil(orb.in()))
    {
        try {
            // call CDMW_cleanup
            Cdmw::CdmwInit::CDMW_cleanup(orb.in());
            
            // call ORB_cleanup
            Cdmw::OrbSupport::OrbSupport::ORB_cleanup(orb.in());
            
            // destroy ORB
            orb->destroy();
        }
        catch(const CORBA::Exception& ex)
        {           
            CDMW_FATAL(Cdmw::CCM::NodeManager::Logger::Get_Logger(), ex);
            status = FAILURE;
        }
    }        

    return status;    
}



