/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


package cdmw.cdmwinit;

import cdmw.common.Assert;
import cdmw.common.BadOrderException;
import cdmw.common.BadParameterException;

import cdmw.platformmngt.ProcessBehaviour;
import cdmw.platformmngt.PlatformInterface;

import cdmw.logging.LogManager;
import cdmw.logging.FilterMngr;
import cdmw.commonsvcs.naming.NamingInterface;

import cdmw.orbsupport.*;



class ExitThread extends Thread
{
    protected int m_exitCode;

    public ExitThread(int exitCode)
    {
        m_exitCode = exitCode;
    }

    public void run()
    {
       java.lang.System.exit(m_exitCode);
    }
}


/**
 * The default implementation of a managed process.
 *
 */
public class CdmwProcessImpl
    extends com.thalesgroup.CdmwPlatformMngt.ProcessDelegatePOA {

    /**
     * The ORB.
     */
    private org.omg.CORBA.ORB orb;

    /**
     * ORB Root POA.
     */
    private org.omg.PortableServer.POA poa;

    /**
     * CDMW Root POA.
     */
    private org.omg.PortableServer.POA cdmwRootPOA;

    /**
     * The actual control of the process.
     */
    private ProcessControl control = null;

    /**
     * Is there an exception raised during the CDMW init ?
     */
    private boolean cdmwInitExceptionRaised = false;

    /**
     * Is CDMW init done ?
     */
    private boolean cdmwInitDone = false;
    
    /**
     * The PullMonitorable object
     */
    private org.omg.FT.PullMonitorable pullMonitorable = null;

    /**
     *  Constructor.
     *  Takes ownership of the supplied ProcessControl
     */
    public CdmwProcessImpl(org.omg.CORBA.ORB orb,
        org.omg.PortableServer.POA poa,
        org.omg.PortableServer.POA cdmwRootPOA, 
        ProcessControl ctrl,
        com.thalesgroup.CdmwFT.LocalPullMonitorable pullM) 
    {
        
        Assert.check(orb != null);
        Assert.check(ctrl != null);
        this.orb = orb;
        this.poa = poa;
        this.cdmwRootPOA = cdmwRootPOA;
        this.control = ctrl;
        
        DefaultPullMonitorableImpl pm = 
            new DefaultPullMonitorableImpl(poa, ctrl, pullM);
        this.pullMonitorable = pm._this(orb);
    }

    /**
     *  Constructor.
     *  Takes ownership of the supplied ProcessControl
     */
    public CdmwProcessImpl(org.omg.CORBA.ORB orb,
        org.omg.PortableServer.POA poa,
        org.omg.PortableServer.POA cdmwRootPOA, 
        ProcessControl ctrl)
    {
        this(orb, poa, cdmwRootPOA, ctrl, null);
    }
    
    public synchronized void setCdmwInitExceptionRaised() {
        cdmwInitExceptionRaised = true;
        notify();
    }

    public synchronized void setCdmwInitDone() {
        cdmwInitDone = true;
        notify();
    }

    /**
     * Implements the
     * IDL:thalesgroup.com/CdmwPlatformMngt/ProcessDelegate/nb_steps:1.0
     * attribute
     */
    public int nb_steps() {
        Assert.check(control != null);
        return control.nbSteps();
    }

    /**
     * Implements the
     * IDL:thalesgroup.com/CdmwPlatformMngt/ProcessDelegate/nb_activity_points:1.0
     * attribute
     */
    public int nb_activity_points() {
        return 0;
    }

    /**
     * implements the
     * IDL:thalesgroup.com/CdmwPlatformMngt/ProcessDelegate/get_service:1.0
     * operation
     */
    public org.omg.CORBA.Object get_service() {
        Assert.check(control != null);
        return control.getService();
    }

    /**
     * Implements the
     * IDL:thalesgroup.com/CdmwPlatformMngt/ProcessDelegate/get_pull_monitorable:1.0
     * operation
     */
    public org.omg.FT.PullMonitorable
        get_pull_monitorable() {
        Assert.check(pullMonitorable != null);
        return pullMonitorable;
    }

    /**
     * Implements the
     * IDL:thalesgroup.com/CdmwPlatformMngt/ProcessDelegate/get_push_monitorable:1.0
     * operation
     */
    public com.thalesgroup.CdmwPlatformMngt.PushMonitorable
        get_push_monitorable() {
        return null;
    }


    /**
     * Implements the
     * IDL:thalesgroup.com/CdmwPlatformMngt/ProcessDelegate/initialise:1.0
     * operation
     */
    public void initialise(
        com.thalesgroup.CdmwPlatformMngtBase.StartupKind startupKind) {

        try {
            synchronized(this) {
                if (!(cdmwInitExceptionRaised || cdmwInitDone)) {
                    // Wait for cdmw_init end
                    wait();
                }

                if (cdmwInitExceptionRaised) {
                    PlatformInterface.notifyFatalError(
                        PlatformInterface.getProcessName(),
                            "Exception raised during CDMW_init!");
                } else {
                    if (cdmwInitDone) {
                        // CDMW POA Managers is in the discarding state.
                        // So we have to re-activate it.
                        // NOTA: We do activate the POA manager before
                        //       on_initialise() to enable nested callbacks
                        //         are needed (user receives an incomming request
                        //       while making an outgoing call).
                        this.cdmwRootPOA.the_POAManager().activate();
                        control.onInitialise(startupKind);
                    } else {
                        PlatformInterface.notifyFatalError(
                            PlatformInterface.getProcessName(),
                                "Unexpected statement reached!");
                    }
                }
            }
        } catch (org.omg.PortableServer.POAManagerPackage.AdapterInactive ai) {
        } catch(BadParameterException bpe) {
        } catch(BadOrderException boe) {
        } catch(InterruptedException ie) {}

    }

    /**
     * Implements the
     * IDL:thalesgroup.com/CdmwPlatformMngt/ProcessDelegate/next_step:1.0
     * operation
     */
    public void next_step()
        throws com.thalesgroup.CdmwPlatformMngt.ProcessDelegatePackage.InvalidStep {
        Assert.check(control != null);
        // TODO: Add CDMW specific init if necessary
        control.onNextStep();
    }

    /**
     * Implements the
     * IDL:thalesgroup.com/CdmwPlatformMngt/ProcessDelegate/run:1.0
     * operation
     */
    public void run()
        throws com.thalesgroup.CdmwPlatformMngt.ProcessDelegatePackage.NotReadyToRun {
        Assert.check(control != null);
        // TODO: Add CDMW specific stuff if necessary
        control.onRun();
    }

    /**
     * Implements the
     * IDL:thalesgroup.com/CdmwPlatformMngt/ProcessDelegate/stop:1.0
     * operation
     */
    public void stop() {
       
        Assert.check(control != null);
        cdmw.logging.Logger.CDMW_DEBUG(InitUtils.Get_Logger(),"Call onStop()");
        control.onStop();
       
/*        OrbGracefulShutdownSignalHandler sighandler = null;
       
        // ===================================================
        // Add ORB shutdown hooks
        // ===================================================
        sighandler = new OrbGracefulShutdownSignalHandler (orb);
                
        ORBSupport.InstallORBShutdownSignalHandler (orb, sighandler);
         
        ExitThread exitThread = new ExitThread(0);
    
        exitThread.start();*/
        
        orb.shutdown(false);
            
    }
    
/*
    public com.thalesgroup.CdmwPlatformMngt.CommandResponse 
        call_command(java.lang.String command_name, 
                     com.thalesgroup.CdmwPlatformMngt.CmdParameterValue[] command_args, 
                     org.omg.CORBA.StringHolder result_info) 
        throws com.thalesgroup.CdmwPlatformMngt.CommandCallBackNotFound {
        // TODO
        throw new org.omg.CORBA.NO_IMPLEMENT (cdmw.orbsupport.ExceptionMinorCodes.NO_IMPLEMENT, org.omg.CORBA.CompletionStatus.COMPLETED_NO);
    }
  */      
  
    public void activate_level(java.lang.String com, java.lang.String dom, int levelValue) {
        LogManager pLogManager = control.getLogManager();
         
        FilterMngr pFilterMngr = pLogManager.getFilterManager();
 
        if (com.compareTo("!")==0) {
            pFilterMngr.activateLevel(dom, levelValue);
        } else {
            pFilterMngr.activateLevel(com, dom, levelValue);
        }
    }

    public void deactivate_level(java.lang.String com, java.lang.String dom,int levelValue) {
        LogManager pLogManager = control.getLogManager();
         
        FilterMngr pFilterMngr = pLogManager.getFilterManager();
 
        if (com.compareTo("!")==0) {
            pFilterMngr.deactivateLevel(dom, levelValue);
        } else {
            pFilterMngr.deactivateLevel(com, dom, levelValue);
        }
    }

    /**
     * Implements the
     * IDL:thalesgroup.com/CdmwPlatformMngt/ProcessDelegate/stop:1.0
     * operation
     */
/*   public com.thalesgroup.CdmwPlatformMngt.CommandResponse 
        call_entity_command(java.lang.String entity_name, 
                            java.lang.String command_name, 
                            com.thalesgroup.CdmwPlatformMngt.CmdParameterValue[] command_args, 
                            org.omg.CORBA.StringHolder result_info) 
        throws com.thalesgroup.CdmwPlatformMngt.CommandCallBackNotFound {
        // TODO
        throw new org.omg.CORBA.NO_IMPLEMENT (cdmw.orbsupport.ExceptionMinorCodes.NO_IMPLEMENT, org.omg.CORBA.CompletionStatus.COMPLETED_NO);
    }
*/
   /**
     * Implements the
     * IDL:thalesgroup.com/CdmwPlatformMngt/ProcessDelegate/get_process_redundancy_state:1.0
     * operation
     */
      public com.thalesgroup.CdmwPlatformMngt.RedundancyEltState get_process_redundancy_state()
              throws org.omg.CORBA.SystemException
      {
        // TODO
        throw new org.omg.CORBA.NO_IMPLEMENT (cdmw.orbsupport.ExceptionMinorCodes.NO_IMPLEMENT, org.omg.CORBA.CompletionStatus.COMPLETED_NO);
    }

    /**
     * Implements the
     * IDL:thalesgroup.com/CdmwPlatformMngt/ProcessDelegate/get_component_redundancy_state:1.0
     * operation
     */
      public com.thalesgroup.CdmwPlatformMngt.RedundancyEltState get_component_redundancy_state(java.lang.String component_name) throws org.omg.CORBA.SystemException
      {
        // TODO
        throw new org.omg.CORBA.NO_IMPLEMENT(cdmw.orbsupport.ExceptionMinorCodes.NO_IMPLEMENT, org.omg.CORBA.CompletionStatus.COMPLETED_NO);
    }

    /**
    * Implements the
    * IDL:thalesgroup.com/CdmwPlatformMngt/ProcessDelegate/get_managed_component:1.0
    * operation
    */
    public  org.omg.CORBA.Object  get_managed_component(java.lang.String component_name)
    {
         
        org.omg.CORBA.Object  obj = null;
        try
        {
            obj = orb.resolve_initial_references("ManagedComponentNamingContext");
        }
        catch(org.omg.CORBA.ORBPackage.InvalidName invName)
        {
            System.out.println(invName);
        }
        org.omg.CosNaming.NamingContext ctx = org.omg.CosNaming.NamingContextHelper.narrow(obj);

        NamingInterface nc = new NamingInterface(ctx);
        try
        {
            obj = nc.resolve(component_name);
        }
        catch(Exception ex)
        {
            obj = null;
            System.out.println(ex);        
        }
        return obj;
   }
}



/**
 * A default LocalPullMonitorable object
 */
class DefaultLocalPullMonitorableImpl 
    extends org.omg.CORBA.LocalObject
    implements com.thalesgroup.CdmwFT.LocalPullMonitorable
{
    public boolean is_alive() {
        return true;
    }
};


/**
 * A default PullMonitorable object
 */
class DefaultPullMonitorableImpl
    extends org.omg.FT.PullMonitorablePOA 
{
    private org.omg.PortableServer.POA poa;
    private ProcessControl control = null;
    private com.thalesgroup.CdmwFT.LocalPullMonitorable localPullMonitorable;

    public DefaultPullMonitorableImpl(
        org.omg.PortableServer.POA poa,
        ProcessControl ctrl,
        com.thalesgroup.CdmwFT.LocalPullMonitorable lpm) 
    {
        Assert.check(poa != null);
        Assert.check(ctrl != null);
        this.poa = poa;
        this.control = ctrl;
        
        if (lpm == null) {
            this.localPullMonitorable = new DefaultLocalPullMonitorableImpl();
        } else {
            this.localPullMonitorable = lpm;
        }
    }
    
    public org.omg.PortableServer.POA _default_POA() {
        return this.poa;
    }
    
    public boolean is_alive() {
        boolean alive = localPullMonitorable.is_alive();
        
        if (alive) {
            alive = this.control.isAlive();
        }
        
        return alive;
    }
    
};


