/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

#include <sstream>

#include "Foundation/common/Locations.hpp"
#include "Foundation/common/Options.hpp"
#include "Foundation/common/String.hpp"
#include "Foundation/common/Assert.hpp"
#include "Foundation/commonsvcs/naming/NamingInterface.hpp"
#include "Foundation/ossupport/OS.hpp"
#include "Foundation/orbsupport/CORBA.hpp"
#include "Foundation/orbsupport/OrbSupport.hpp"
#include "Foundation/orbsupport/StrategyList.hpp"
#include "Foundation/orbsupport/ExceptionMinorCodes.hpp"
#include "Repository/repositoryinterface/RepositoryInterface.hpp"

#include "testrequestfollowupinit/RequestFollowUpTest.hpp"

const char* NAME_PORT = "5010";
const char* PROP_PORT = "5015";
const char* REP_PORT = "5020";
const char* REC_MGT_PORT = "59875";

const int POA_THREAD_POOL_SIZE = 5;

class SimRepoStarter {
private:
    Cdmw::OsSupport::OS::ProcessId idNaming;
    Cdmw::OsSupport::OS::ProcessId idRepository;

public:
    SimRepoStarter() :
        idNaming(0), idRepository(0){
    }

    ~SimRepoStarter() {
        assert(idNaming != 0);
        assert(idRepository != 0);
    }

    void start(int argc, char* argv[]) {
        using namespace Cdmw::OsSupport;
        using namespace Cdmw::Common;

        int timescale = Cdmw::TestUtils::get_timescale();

        //start naming service
        std::cout << "Starting Name Service on port " << NAME_PORT << "..."
                << std::endl;
        const std::string namingservice="Naming_Service";
        idNaming = OS::create_process(namingservice,
                std::string(" -ORBEndpoint iiop://localhost:") + NAME_PORT);
        OS::sleep(timescale*3000);
        if (idNaming == 0)
            throw;

        //start simulated repository
        const std::string repository = "cdmw_simulated_repository";
        std::stringstream repositoryArgs;
        repositoryArgs << " -ORBEndpoint iiop://localhost:" << REP_PORT;
        repositoryArgs << " -ORBInitRef NameService=corbaloc:iiop:localhost:"
                << NAME_PORT << "/NameService";
        repositoryArgs << " -DOM "
                << Cdmw::Common::Locations::CDMW_SERVICES_NAME_DOMAIN;
        repositoryArgs << " -DOM "
                << Cdmw::Common::Locations::CDMW_EVENT_SERVICE_NAME_DOMAIN;
        repositoryArgs << " -DOM CDMW/SERVICES/EVENT/FACTORIES";
        repositoryArgs << " -DOM CDMW/SERVICES/EVENT/EVENT_CHANNELS";
        repositoryArgs << " -DOM CDMW/SERVICES/EVENT/TYPED_EVENT_CHANNELS";
        repositoryArgs << " -ROOT "
                << Cdmw::Common::Locations::ADMIN_ROOT_CONTEXT_ID;

        std::cout << "start repository simulator" << std::endl;
        std::cout << repository.c_str() << repositoryArgs.str() << std::endl;

        idRepository = OS::create_process(repository, repositoryArgs.str());
        OS::sleep(timescale*10000);
        if (idRepository == 0)
            throw;

        std::cout << "Simrepository started!" << std::endl;

    }

    void stop() {
        assert(idRepository != 0);
        TEST_INFO("Kill The repository");
        Cdmw::OsSupport::OS::kill_process(idRepository);
        TEST_INFO("Kill The Naming service");
        assert(idNaming != 0);
        Cdmw::OsSupport::OS::kill_process(idNaming);
    }
};

void initRepository(CORBA::ORB_ptr orb) {
    using namespace Cdmw::NamingAndRepository;
    using namespace Cdmw::Common;

    //
    // Get naming service
    //
    CORBA::Object_var obj;

    try
    {
        obj = orb -> resolve_initial_references("NameService");
    }
    catch(const CORBA::ORB::InvalidName&)
    {
        std::cerr<< "Can't resolve `NameService'" << std::endl;
        throw;
    }

    if (CORBA::is_nil(obj.in())) {
        std::cerr << "`NameService' is a nil object reference" << std::endl;
        throw CORBA::TRANSIENT(Cdmw::OrbSupport::TRANSIENTCannotReachRepository,
                CORBA::COMPLETED_NO);
    }

    CosNaming::NamingContextExt_var nc;
    try
    {
        nc = CosNaming::NamingContextExt::_narrow(obj.in());
    }
    catch (...)
    {
        std::cerr << "Obj is not a NamingContextExt" << std::endl;
        throw;
    }

    if (CORBA::is_nil(nc.in())) {
        std::cerr <<"`NameService' is not a NamingContextExt object reference"
                << std::endl;
        throw CORBA::TRANSIENT(Cdmw::OrbSupport::TRANSIENTCannotReachRepository,
                CORBA::COMPLETED_NO);
    }

    const char* REPOSITORY_NAME = "CDMW.I/repository.simulated";

    // Get reference to the repository interface
    CdmwNamingAndRepository::Repository_var rep =
            CdmwNamingAndRepository::Repository::_nil();
    {
        try {
            Cdmw::CommonSvcs::Naming::NamingInterface ni(nc.in());
            CORBA::Object_var obj__ = ni.resolve(REPOSITORY_NAME);

            rep = CdmwNamingAndRepository::Repository::_narrow(obj__.in());
            if (CORBA::is_nil(rep.in()))
            {
                std::cerr << "Couldn't retrieve reference of the Repository"
                << "\n(file:" << __FILE__ << " line:" << __LINE__ << ")" << std::endl;
                throw CORBA::TRANSIENT(Cdmw::OrbSupport::TRANSIENTCannotReachRepository,
                        CORBA::COMPLETED_NO);
            }
        } catch (...) {
            std::cerr << "Couldn't retrieve reference of the Repository"
            << "\n(file:" << __FILE__ << " line:" << __LINE__ << ")" << std::endl;
            throw CORBA::TRANSIENT(Cdmw::OrbSupport::TRANSIENTCannotReachRepository,
                    CORBA::COMPLETED_NO);
        }

        try {
            RepositoryInterface::init(Locations::CDMW_SERVICES_NAME_DOMAIN, rep.in());
        }
        catch (...) {
            std::cerr << "Couldn't init Repository"
            << "\n(file:" << __FILE__ << " line:" << __LINE__ << ")" << std::endl;
            throw;
        }
    }
}

int main(int argc, char* argv[]) {
    using namespace Cdmw::OsSupport;
    using namespace Cdmw::Common;
    CORBA::ORB_var orb;

    //retrieve timescale
    int timescale = Cdmw::TestUtils::get_timescale();

    //define orb stategies
    Cdmw::OrbSupport::StrategyList orb_strategies;
    orb_strategies.add_OrbThreaded();
    orb_strategies.add_PoaThreadPool(POA_THREAD_POOL_SIZE);

    std::cout << "initialise the ORB" << std::endl;
    //intialise the orb
    std::stringstream serverArgs;
    serverArgs << argv[0];
    serverArgs << " -ORBInitRef NameService=corbaloc:iiop:localhost:"
            << NAME_PORT << "/NameService ";
    serverArgs << Cdmw::Common::Options::SYSTEM_PORT_OPTION << "=" << NAME_PORT;
    std::cout << serverArgs.str() << std::endl;
    int nbArgs = 4;
    orb = Cdmw::OrbSupport::OrbSupport::ORB_init(nbArgs,
            String::to_char_array(String::to_strings(serverArgs.str())),
            orb_strategies);

    CORBA::Object_var obj = orb->resolve_initial_references("RootPOA");
    PortableServer::POA_var rootPOA = PortableServer::POA::_narrow(obj.in());
    PortableServer::POAManager_var poaMgr = rootPOA->the_POAManager();
    poaMgr->activate();

    //start simulated repository
    SimRepoStarter simrepo;
    simrepo.start(argc, argv);

    //initialise repository
    initRepository(orb.in());

    OS::sleep(timescale*3000);

    //run tests
    std::cout << "Simrepository initialised!" << std::endl;
    Cdmw::TestUtils::CORBATestManager::instance()->init(orb, rootPOA, argc, argv);
    Cdmw::TestUtils::CORBATestManager::instance()->run_tests();
    Cdmw::TestUtils::CORBATestManager::instance()->summary();

    //stop naming and repository
    simrepo.stop();
}
