/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


// Cdmw Files 
#include "Event/eventsupport/EventChannelFactory_impl.hpp"
#include "Event/eventinterface/EventChannelInterface.hpp"
#include "eventsupport/EventChannelUtils_TAOOpenFusion.hpp"

#include <Foundation/commonsvcs/naming/NamingInterface.hpp>
#include <Repository/repositoryinterface/RepositoryInterface.hpp>
#include <Foundation/common/Locations.hpp>
#include <Foundation/common/Assert.hpp>
#include <Foundation/common/String.hpp>
#include <Foundation/orbsupport/ExceptionMinorCodes.hpp>
#include <Foundation/orbsupport/RefCountLocalObject.hpp>
#include <Foundation/orbsupport/RefCountServantBase.hpp>

// TAO files
#define TAO_HAS_TYPED_EVENT_CHANNEL
#include "orbsvcs/CosEvent/CEC_EventChannel.h"
#include "orbsvcs/CosEvent/CEC_Default_Factory.h"
#include "tao/PortableServer/ServantLocatorC.h"

#include "Event/idllib/CdmwEventAdmin.stub.hpp"

#include <Foundation/ossupport/SystemProperties.hpp>
#include <Foundation/osthreads/Thread.hpp>
#include <Foundation/logging/Logger.hpp>
#include <Foundation/osthreads/Mutex.hpp>
#include <Foundation/osthreads/MutexGuard.hpp>

namespace
{
    // Maximum number of retries on receiving CORBA::TRANSIENT exception
    // for registering objects.
    const signed long MAX_RETRIES = 1;
    
    const char* DEFAULT_EVENT_CHANNEL_FACTORY = "DefaultEventChannelFactory";
    
    // Supported interface
    const char* EVT_CHANNEL_REP_ID = "IDL:omg.org/CosEventChannelAdmin/EventChannel:1.0";

    const char* PUSH_SUPPIER_REP_ID = "IDL:omg.org/CosEventComm/PushSupplier:1.0";

   /**
    * PCR-1637: Add properties for configuring TAO's CECDispatchingThreads option
    */
   const char* const CEC_DISPATCHING_THREADS_PROPERTY_NAME =
      "Cdmw.Event.TAO.CECDispatchingThreads";
   const std::string DEFAULT_CEC_DISPATCHING_THREADS("2");

   // -- Add a logger for the EventChannel Factory
   class ECFLogger
   {
      public:
         
         /**
          * Constructor
          */
         
         ECFLogger()
         {}
         
         /**
          * Destructor
          */
         ~ECFLogger()
         {}
          
         /**
          * ECF Domain for Logs
          */
         static const char* ECF_LOG_DOMAIN;

         inline static Cdmw::Logging::Logger * GetLogger() throw ()
         {
            static Cdmw::Logging::Logger * Logger__ = 
               Cdmw::Logging::Logger::Get_Logger(ECF_LOG_DOMAIN);
            CDMW_ASSERT(Logger__ != 0);
            return Logger__;
         }
         
   };        
   const char* ECFLogger::ECF_LOG_DOMAIN = "CDMW.EVT.EventChannelFactory";

   class MultipleEventChannelDestructionNotifier : 
         public virtual POA_CosEventComm::PushSupplier,
         public virtual PortableServer::RefCountServantBase
   {
      public:
         MultipleEventChannelDestructionNotifier
         (CORBA::ORB_ptr orb,
          CdmwLifeCycle::ObjectRegistration_ptr   name_domain,
          const char* name_domain_name = "")
            throw (CORBA::SystemException)
            : m_domain(CdmwLifeCycle::ObjectRegistration::_duplicate(name_domain)),
              m_name_domain_name (name_domain_name),
              m_current(PortableServer::Current::_nil())
         {
            CDMW_ASSERT(!CORBA::is_nil(orb));
            CORBA::Object_var obj =
               orb->resolve_initial_references("POACurrent");
            m_current = PortableServer::Current::_narrow(obj.in());
         }
         
        // Override _default_POA
        virtual PortableServer::POA_ptr _default_POA() 
        {
           CDMW_ASSERT(0);
           // This is only for the compiler
           return PortableServer::POA::_nil();
        }

        void
        disconnect_push_supplier()
            throw (CORBA::SystemException)
        {
            CDMW_ASSERT(!(CORBA::is_nil(m_domain.in())));

            try 
            {
               CDMW_LOG(ECFLogger::GetLogger(),
                        Cdmw::Logging::CDMW_DEBUG_1,
                        "EventChannelDestructionNotifier disconnect order");
               // Forge event channel name
               
               PortableServer::POA_var poa = m_current->get_POA();
               PortableServer::ObjectId_var oid = m_current->get_object_id();

               // Turn the OID into a string
               CORBA::String_var oid_str;
               if (Cdmw::Event::EventChannelUtils::M_local_manager_is_used) {
                  oid_str = PortableServer::ObjectId_to_string(oid.in());
                  std::string name = m_name_domain_name + "/" + oid_str.in();
                  oid_str = name.c_str();
               } else
                  oid_str = PortableServer::ObjectId_to_string(oid.in());
               
               CDMW_DEBUG_1(ECFLogger::GetLogger(),
                            "\t[EVENT]===> Disconnecting from " << oid_str.in());

               m_domain->unregister_object(oid_str.in());
            } 
            catch (const CosNaming::NamingContext::NotFound & ) 
            {
               // This should not happen 
               CDMW_WARN(ECFLogger::GetLogger(),
                         "EventChannelDestructionNotifier : NameDomain::NotRegistered!");
            } 
            catch (const CosNaming::NamingContext::InvalidName & ) 
            {
               // This should not happen either 
               CDMW_WARN(ECFLogger::GetLogger(),
                         "EventChannelDestructionNotifier : InvalidName!");
            } 
            catch (const PortableServer::Current::NoContext & )
            {
               CDMW_ASSERT(0); // Should not happen in our context!
            }
            catch (const CORBA::SystemException& e)
            {
               CDMW_LOG(ECFLogger::GetLogger(),
                        Cdmw::Logging::CDMW_DEBUG_1,
                        "EventChannelDestructionNotifier :" << e );
            }
        }

    private:
         /**
          * domain where event channels are registered
          */
          CdmwLifeCycle::ObjectRegistration_var   m_domain;
         /**
          * channel which has been destroyed
          */
         std::string                         m_channel_name;
         /**
          * name of the domain, useful to make unregister_object() on
          */
         std::string m_name_domain_name;

         PortableServer::Current_var m_current;

    }; // End MultipleEventChannelDestructionNotifier
 

    // TAO_CEC_EventChannel is not a refcounted servant !!
    // This class is to add refcount.
    class Refcounted_TAO_CEC_EventChannel: 
         public virtual TAO_CEC_EventChannel,
         public virtual Cdmw::OrbSupport::RefCountServantBase
    {
       public:
          Refcounted_TAO_CEC_EventChannel (PortableServer::POA_ptr poa,
                                           const TAO_CEC_EventChannel_Attributes& attributes,
                                           TAO_CEC_Factory* factory = 0,
                                           int own_factory = 0)
             : TAO_CEC_EventChannel(attributes, factory, own_factory),
               m_poa(PortableServer::POA::_duplicate(poa))
          {
          };
          
          // override destructor to perform shutdown if not already performed
          virtual ~Refcounted_TAO_CEC_EventChannel()
          {    
                TAO_CEC_EventChannel::shutdown();
          };
        // Override _default_POA
        virtual PortableServer::POA_ptr _default_POA() 
        {
           return PortableServer::POA::_duplicate(m_poa.in());
        }
          
          
       private:
          PortableServer::POA_var m_poa;
    };
     
};

namespace Cdmw
{
namespace Event
{
        
/**
*Implementation notes: [if no pertinent write none ]
*<p>
* The EventChannelFactory_impl class is a user-defined factory while it doesn't 
* localy create event channel but let the Event Server doing it.
* 
*Portability issues: [if no pertinent write none ]
*<p>
*/
        
        
EventChannelFactory_impl::EventChannelFactory_impl(
     CORBA::ORB_ptr                           orb, 
     PortableServer::POA_ptr                  parent,
     CdmwLifeCycle::ObjectRegistration_ptr   name_domain,
     const std::string &                      name_domain_name,
     const std::string &                      factory_name,
     Cdmw::OrbSupport::POAThreadingPolicy &   threading_policy,
     int & argc, char** argv,
     bool use_profile_manager)
     throw(Cdmw::BadParameterException,
     Cdmw::InternalErrorException,
     CORBA::SystemException)
     : m_orb(CORBA::ORB::_duplicate(orb)), 
     m_poa(PortableServer::POA::_duplicate(parent)),
     m_domain(CdmwLifeCycle::ObjectRegistration::_duplicate(name_domain)),
     m_domain_name(name_domain_name),
     m_factory_name(factory_name),
     m_use_profile_manager(use_profile_manager)
{
    // Create policy list for simple persistence
    CORBA::PolicyList pl;
    CORBA::ULong len = pl.length();
    pl.length(len + 1);
    pl[len++] = m_poa->create_lifespan_policy(PortableServer::PERSISTENT);

    pl.length(len + 1);
    pl[len++] = m_poa->create_id_assignment_policy(PortableServer::USER_ID);

    pl.length(len + 1);
    pl[len++] = m_poa->create_implicit_activation_policy(
        PortableServer::NO_IMPLICIT_ACTIVATION);
    
    pl.length(len + 1);
    pl[len++] = m_poa->create_servant_retention_policy(PortableServer::NON_RETAIN);

    pl.length(len + 1);
    pl[len++] = 
        m_poa->create_request_processing_policy(PortableServer::USE_DEFAULT_SERVANT);

    pl.length(len + 1);
    pl[len++] = 
        m_poa->create_id_uniqueness_policy(PortableServer::MULTIPLE_ID);

    
    // Get parent POA's POA manager
    PortableServer::POAManager_var pmanager = m_poa->the_POAManager();

    // Create new POA
    std::string notifier_poa_name = factory_name + "POA";
    m_notifier_poa = m_poa->create_POA(notifier_poa_name.c_str(), pmanager.in(), pl);

    // Create and register servant locator
    PortableServer::ServantLocator_var the_locator;
    
    MultipleEventChannelDestructionNotifier * default_servant =
       new MultipleEventChannelDestructionNotifier
       (m_orb.in(),name_domain, name_domain_name.c_str());
    // set default servant
    m_notifier_poa->set_servant(default_servant);
    default_servant->_remove_ref();

    

    // Clean up
    for (CORBA::ULong i = 0; i < len; ++i)
        pl[i]->destroy();

    // Create an other new POA for TAO CEC
    // The attributes used to create eventchannel includes
    // the POA used to activate the ConsumerAdmin, SupplierAdmin, 
    // ProxyPushConsumer, ProxyPushSupplier, ProxyPullConsumer and the 
    // ProxyPullSupplier objects; these POAs must have the IMPLICIT_ACTIVATION 
    // and the SYSTEM_ID policies (as the RootPOA does).     
    len = 0;
    pl.length(len + 1);
    pl[len++] = m_poa->create_id_assignment_policy(PortableServer::SYSTEM_ID);

    pl.length(len + 1);
    pl[len++] 
        = m_poa->create_implicit_activation_policy(PortableServer::IMPLICIT_ACTIVATION);
    
    std::string internal_poa_name = factory_name + "InternalPOA";
    m_internal_poa = m_poa->create_POA(internal_poa_name.c_str(), pmanager.in(), pl);

    // Clean up
    for (CORBA::ULong i = 0; i < len; ++i)
        pl[i]->destroy();
}
        
        
EventChannelFactory_impl::~EventChannelFactory_impl()
    throw()
{

    // get all channel name
    ChannelMap::iterator c_iter = m_created_channels.begin();
    
    // Print out the number of days in June.
    while (c_iter != m_created_channels.end ())
    {
        try 
        {
           if (!Cdmw::Event::EventChannelUtils::M_local_manager_is_used)
              m_domain->unregister_object((*c_iter).first.c_str());
           else {
              std::string name = m_domain_name + "/" + (*c_iter).first;
              m_domain->unregister_object(name.c_str());
           }
        } 
        catch (const CosNaming::NamingContext::NotFound & ) 
        {
            // This could happen if channel has been destroyed! 
        } 
        catch (const CosNaming::NamingContext::InvalidName & ) 
        {
           // This should not happen  
           CDMW_WARN(ECFLogger::GetLogger(),
                     "~EventChannelFactory_impl : InvalidName!");
        } 
        catch (const CORBA::SystemException& e)
        {
           CDMW_LOG(ECFLogger::GetLogger(),
                    Cdmw::Logging::CDMW_DEBUG_1,
                    "~EventChannelFactory_impl :" << e );
        }
        catch (...)
        {
           CDMW_LOG(ECFLogger::GetLogger(),
                    Cdmw::Logging::CDMW_DEBUG_1,
                    "~EventChannelFactory_impl : unexpected exception raised");
        }

        c_iter++;
    }
    m_created_channels.clear();
}

//
// IDL:thalesgroup.com/CdmwLifeCycle/EventChannelFactory/create_event_channel:1.0
//
CosEventChannelAdmin::EventChannel_ptr
EventChannelFactory_impl::create_event_channel(const char* name)
    throw(CdmwLifeCycle::AlreadyExistant,
          CdmwLifeCycle::InvalidName,
          CdmwEvent::CreationError,
          CORBA::SystemException)
{
    CosEventChannelAdmin::EventChannel_var channel;
    CDMW_LOG(ECFLogger::GetLogger(),
             Cdmw::Logging::CDMW_DEBUG_1,
             "INFO: begin creation untyped channel named: " << name );
    
    try 
    {
        // create a new factory able to load CEC TAO options
        TAO_CEC_Default_Factory* tao_factory = new TAO_CEC_Default_Factory();

        std::string factory_properties = "";

        if (m_use_profile_manager) {
            // get information from eventchannelmanager to init factory with tao properties
            // init (argc, argv), set to 0 for the moment!
            factory_properties = EventChannelUtils::get_factory_properties(name);
        }
        

        std::string dummy_CECDispatchingThreads = 
           OsSupport::SystemProperties::Get_Property(CEC_DISPATCHING_THREADS_PROPERTY_NAME,
                                                     DEFAULT_CEC_DISPATCHING_THREADS);
        int nbthreads__ = atoi(dummy_CECDispatchingThreads.c_str());
        if (nbthreads__ <= 0) {
           CDMW_WARN(ECFLogger::GetLogger(),
                     "WARN: Invalid value for '" << CEC_DISPATCHING_THREADS_PROPERTY_NAME
                     << "' property. Using default value '" 
                     << DEFAULT_CEC_DISPATCHING_THREADS << "'");
           dummy_CECDispatchingThreads = DEFAULT_CEC_DISPATCHING_THREADS;
        }

        factory_properties += " -CECDispatching mt -CECDispatchingThreads ";
        factory_properties += dummy_CECDispatchingThreads;


        std::vector< std::string > properties =
            Common::String::to_strings(factory_properties);
        int argc = properties.size();
        char** argv = Common::String::to_char_array(properties);
        
        tao_factory->init(argc, argv);
        
        Common::String::delete_char_array(argv);

        // get internal POA while the attributes use to create eventchannel includes
        // the POA used to activate the ConsumerAdmin, SupplierAdmin, 
        // ProxyPushConsumer, ProxyPushSupplier, ProxyPullConsumer and the 
        // ProxyPullSupplier objects; these POAs must have the IMPLICIT_ACTIVATION 
        // and the SYSTEM_ID policies (as the RootPOA does). 
        TAO_CEC_EventChannel_Attributes attributes(m_internal_poa.in (),
                                                   m_internal_poa.in ());

        Refcounted_TAO_CEC_EventChannel* ec_impl 
            = new Refcounted_TAO_CEC_EventChannel(m_internal_poa.in (),
                                                  attributes, tao_factory, true);

        ec_impl->activate();

        channel = ec_impl->_this();
        ec_impl->_remove_ref();

        /*ChannelInfo channel_info;
        channel_info.channel_servant = ec_impl;
        channel_info.channel_ref = CosEventChannelAdmin::EventChannel::_nil();
        //    = CosEventChannelAdmin::EventChannel::_duplicate(channel.in());
        m_created_channels.insert(ChannelMap::value_type(std::string(name), channel_info));
        */
        ChannelMap::iterator c_iter = m_created_channels.find(std::string(name));
        if (c_iter != m_created_channels.end())
        {
            // a channel with the same name has been destroyed, replace it!
            // We are sure it has been destroyed while naming service doesn't raise already exist!
            m_created_channels.erase(c_iter);
        }
        m_created_channels.insert(ChannelMap::value_type(std::string(name), 0));

        if (CORBA::is_nil(channel.in()))
        {
           CDMW_ERROR(ECFLogger::GetLogger(), "channel is nil!" );
            CdmwEvent::CreationError ex;
            std::string reason = "Channel is nil!";
            ex.reason = CORBA::string_dup(reason.c_str());
            throw ex;
        }
    }
    catch (const CdmwEvent::CreationError&)
    {
       CDMW_ERROR(ECFLogger::GetLogger(),
                  "CdmwEvent::CreationError exception raised");
        throw;
    }
    catch (const CORBA::SystemException& e)
    {
       CDMW_ERROR(ECFLogger::GetLogger(),
                  "ERROR: System exception raised '" << e << "'");
       throw;
    }
    catch (...)
     {
        CDMW_ERROR(ECFLogger::GetLogger(),
                   "Unexpected exception raised");
        CdmwEvent::CreationError ex;
        std::string reason = "Unexpected exception raised";
        ex.reason = CORBA::string_dup(reason.c_str());
        throw ex;
     }
    
    // Convert channelname to OID
    PortableServer::ObjectId_var oid = PortableServer::string_to_ObjectId(name);

    // Make a new reference.
    CORBA::Object_var obj = m_notifier_poa->create_reference_with_id(oid.in(), PUSH_SUPPIER_REP_ID);
    CosEventComm::PushSupplier_var notifier = CosEventComm::PushSupplier::_narrow(obj.in());


    // connect notifier to channel in order to be notified when event channel is destroyed
    try 
    {
        Event::EventChannelInterface eventChannelInt(channel.in());

        CosEventComm::PushConsumer_var proxyConsumer 
            = eventChannelInt.get_pushConsumer(notifier.in());
    }
    catch (...)
    {
        // destroy channel, it will be unregistered automatically by ObjectRegistrationGuard
        channel->destroy();
        
        CdmwEvent::CreationError ex;
        std::string reason = "Impossible to connect channel destruction notifier";
        ex.reason = CORBA::string_dup(reason.c_str());
        
        throw ex;
    }
    
    // Register (name,object) within the repository. 
    EventChannelUtils::register_object(m_domain.in(), m_domain_name, 
                                       channel.in(), name, MAX_RETRIES);

    // Return the object reference
    return channel._retn();
}


//
// IDL:thalesgroup.com/CdmwLifeCycle/FactoryBase/supports:1.0
//
CORBA::Boolean 
EventChannelFactory_impl::supports(const char* repository_id)
throw(CORBA::SystemException)
{
    bool is_supported = false;
    
    if (strcmp (repository_id, EVT_CHANNEL_REP_ID) == 0)
        is_supported = true;
    
    return is_supported;
}

//
// IDL:thalesgroup.com/CdmwLifeCycle/FactoryBase/get_supported_objects:1.0
//
CdmwLifeCycle::RepositoryIdSeq* 
EventChannelFactory_impl::get_supported_objects()
throw(CORBA::SystemException)
{
    CdmwLifeCycle::RepositoryIdSeq_var repIDSeq = new CdmwLifeCycle::RepositoryIdSeq();
    
    repIDSeq->length(1);
    (*repIDSeq)[0] = CORBA::string_dup(EVT_CHANNEL_REP_ID);
    
    return repIDSeq._retn();
}


}; // End namespace Event
}; // End namespace Cdmw


