/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO 
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2009. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

#include <FaultTolerance/ftcommon/FallbackOid.hpp>
#include <sstream>
#include <Foundation/orbsupport/ExceptionMinorCodes.hpp>
#include <FaultTolerance/ftcommon/FTSystemProperties.hpp> // for OGID_KIND
#include <FaultTolerance/ftcommon/FTManagerGroups.hpp> // for OGID_MIN

#include <FaultTolerance/ftcommon/FTLogger.hpp>

#include <Foundation/logging/FunctionLogger.hpp>
#include <Foundation/commonsvcs/naming/NamingInterface.hpp>
#include <Foundation/common/Assert.hpp>

namespace
{
   bool is_group_domain_id_valid( const char* group_domain_id)
   {
      // Check if group_domain_id is well formed!
      bool result = false;
      try {
         std::string FTDomainID(group_domain_id);
         CosNaming::Name_var name = 
            Cdmw::CommonSvcs::Naming::NamingInterface::to_name(FTDomainID);
         if (name->length() == 1)
            result = true;
         
      } catch (const Cdmw::CommonSvcs::Naming::InvalidNameException & ex) {
         // ignore
      }
      return result;
   }
   //  Empty name is valid
   bool is_group_name_valid( const char* group_name )
   {
      // Check if group_domain_id is well formed!
      bool result = false;
      try {
         std::string GroupName(group_name);
         if (!GroupName.empty()) 
            CosNaming::Name_var name = 
               Cdmw::CommonSvcs::Naming::NamingInterface::to_name(GroupName);
         result = true;
         
      } catch (const Cdmw::CommonSvcs::Naming::InvalidNameException & ex) {
         // ignore
      }
      return result;
   }


   inline std::string NameComponent_to_string(const CosNaming::NameComponent & nc)
   {
      CosNaming::Name n__(1L);
      n__.length(1L);
      n__[0L] = nc;
      return Cdmw::CommonSvcs::Naming::NamingInterface::to_string(n__);
   }
   inline CosNaming::NameComponent string_to_NameComponent(const std::string & s)
   {
      CosNaming::Name_var name__ = 
         Cdmw::CommonSvcs::Naming::NamingInterface::to_name(s);
      CosNaming::NameComponent nc__(name__[0L]);
      return nc__;
   }

    
}; // End of anonymous namespace

namespace Cdmw {
namespace FT {

FallbackOid::FallbackOid(const char *                group_domain_id,
                         ::FT::ObjectGroupId         object_group_id,
                         ::FT::ObjectGroupRefVersion group_version,
                         const char *                repository_id,
                         const char*                 group_name)
    throw ()
    : m_group_domain_id(group_domain_id),
      m_object_group_id(object_group_id),
      m_group_version(group_version),
      m_repository_id(repository_id),
      m_group_name(group_name)
{
    CDMW_LOG_FUNCTION(FTLogger::GetLogger(),
                      "group_domain_id="  << group_domain_id <<
                      ",object_group_id=" << object_group_id <<
                      ",group_version="   << group_version   <<
                      ",repository_id="   << repository_id   <<
                      ",group_name="      << group_name);
    CDMW_ASSERT(!m_group_domain_id.empty());
    CDMW_ASSERT(is_group_domain_id_valid(group_domain_id));
    CDMW_ASSERT(is_group_name_valid(group_name));
    //CDMW_ASSERT((m_object_group_id != Cdmw::FT::OGID_MIN) || !m_group_name.empty());
}
FallbackOid::FallbackOid(const PortableServer::ObjectId & oid) 
    throw (CORBA::BAD_PARAM)
{
    CDMW_LOG_FUNCTION(FTLogger::GetLogger(),"");
    // First convert the id into a string
    CORBA::String_var oids = PortableServer::ObjectId_to_string (oid);
    
    // The ObjectId has the following pattern:
    // group_domain_id/[group_version.v]/[repository_id.typeid]/[object_group_id.gid]/[group_name.name]
    std::string s__(oids.in());
    std::string group_domain_s,gid_s,version_s,type_id_s, group_name;
    try {
       CosNaming::Name_var n__=Cdmw::CommonSvcs::Naming::NamingInterface::to_name(s__);
       CDMW_ASSERT(n__->length() >= 1);
       group_domain_s = NameComponent_to_string(n__[0L]);
       std::string tmp_id, tmp_kind;
       for (CORBA::ULong idx = 1L; idx <n__->length(); ++idx) {
          tmp_id = n__[idx].id.in(); 
          // .kind == VERSION_KIND | TYPEID_KIND | OGID_KIND | NAME_KIND  
          tmp_kind = n__[idx].kind.in();
          CDMW_ASSERT((tmp_kind == Cdmw::FT::VERSION_KIND) || 
                      (tmp_kind == Cdmw::FT::TYPEID_KIND) || 
                      (tmp_kind == Cdmw::FT::OGID_KIND) || 
                      (tmp_kind == Cdmw::FT::NAME_KIND));
          if (tmp_kind == Cdmw::FT::VERSION_KIND)
             version_s=tmp_id;
          else if (tmp_kind == Cdmw::FT::TYPEID_KIND)
             type_id_s=tmp_id;
          else if (tmp_kind == Cdmw::FT::OGID_KIND)
             gid_s=tmp_id;
          else if (tmp_kind == Cdmw::FT::NAME_KIND)
             group_name=tmp_id;
          else 
             CDMW_NEVER_HERE();
       }
    } catch (Cdmw::CommonSvcs::Naming::InvalidNameException & e) {
        CDMW_ERROR(FTLogger::GetLogger(),
                   "ERROR: Fallback oid badly formed!");
        CORBA::BAD_PARAM ex(OrbSupport::BAD_PARAMInvalidObjectId, CORBA::COMPLETED_NO);
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw ex;
    }
    if (group_domain_s.empty()) {
        CDMW_ERROR(FTLogger::GetLogger(),
                   "ERROR: Fallback oid badly formed!");
        CORBA::BAD_PARAM ex(OrbSupport::BAD_PARAMInvalidObjectId, CORBA::COMPLETED_NO);
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw ex;
    }
    // Check that group_name is well formed!
    if (!group_name.empty() && !is_group_name_valid(group_name.c_str())) {
       CDMW_ERROR(FTLogger::GetLogger(),
                  "ERROR: Fallback oid badly formed!");
       CORBA::BAD_PARAM ex(OrbSupport::BAD_PARAMInvalidObjectId, CORBA::COMPLETED_NO);
       CDMW_LOG_FUNCTION_EXCEPTION(ex);
       throw ex;
    }
    m_group_domain_id = group_domain_s;
    if (gid_s.empty())
       m_object_group_id = Cdmw::FT::OGID_MIN;
    else 
       m_object_group_id = ::atoll( gid_s.c_str() );
    m_group_version   = ::atol(version_s.c_str());
    m_repository_id   = type_id_s;
    m_group_name      = group_name;
    CDMW_DEBUG(FTLogger::GetLogger(),"FallbackOid : \n"
               << "\t group_domain_id=" << m_group_domain_id  << "\n"
               << "\t object_group_id=" << m_object_group_id   << "\n"
               << "\t group_version="   << m_group_version   << "\n"
               << "\t repository_id="   << m_repository_id  << "\n"
               << "\t group_name="      << m_group_name );
}

FallbackOid::FallbackOid & 
FallbackOid::operator=(const FallbackOid::FallbackOid & rhs)
    throw ()
{
    CDMW_LOG_FUNCTION(FTLogger::GetLogger(),"");
    if (this != &rhs) {
        m_group_domain_id = rhs.m_group_domain_id;
        m_object_group_id = rhs.m_object_group_id;
        m_group_version   = rhs.m_group_version;
        m_repository_id   = rhs.m_repository_id;
        m_group_name      = rhs.m_group_name;
    }
    
    return *this;
}

FallbackOid::FallbackOid(const FallbackOid::FallbackOid & rhs)
    throw ()
    : m_group_domain_id(rhs.m_group_domain_id),
      m_object_group_id(rhs.m_object_group_id),
      m_group_version(rhs.m_group_version),
      m_repository_id(rhs.m_repository_id),
      m_group_name(rhs.m_group_name)
{
    CDMW_LOG_FUNCTION(FTLogger::GetLogger(),
                      "group_domain_id="  << m_group_domain_id <<
                      ",object_group_id=" << m_object_group_id <<
                      ",group_version="   << m_group_version <<
                      ",repository_id="   << m_repository_id <<
                      ",group_name="      << m_group_name);
}
std::string FallbackOid::name()   const
{
    CDMW_LOG_FUNCTION(FTLogger::GetLogger(),"");
    std::string to_name__("");
    std::ostringstream oss;
    if (!m_group_name.empty()) {
       CosNaming::NameComponent nc;
       nc.id = CORBA::string_dup(m_group_domain_id.c_str());
       oss << NameComponent_to_string(nc) << "/" << m_group_name;
       to_name__ = oss.str();
    } else {
       CosNaming::Name n;
       n.length(1L);
       n[0L]=string_to_NameComponent(m_group_domain_id);
       if (m_object_group_id != Cdmw::FT::OGID_MIN)
       {
          n.length(2L);
          oss << m_object_group_id;
          n[1L].id=CORBA::string_dup(oss.str().c_str());
          n[1L].kind=CORBA::string_dup(Cdmw::FT::OGID_KIND.c_str());
       }
       to_name__ = Cdmw::CommonSvcs::Naming::NamingInterface::to_string(n);
    }

    CDMW_LOG_FUNCTION_RETURN(to_name__);
    return to_name__;
}
// TODO!
std::string FallbackOid::to_string() const
{
    CDMW_LOG_FUNCTION(FTLogger::GetLogger(),"");
    CORBA::ULong idx = 0L;
    CosNaming::Name n;
    n.length(idx+1);
    n[idx]=string_to_NameComponent(m_group_domain_id);
    ++idx;
    std::ostringstream oss;
    if (m_group_version != 0) {
       n.length(idx+1);
       oss.str("");
       oss << m_group_version;
       n[idx].id=CORBA::string_dup(oss.str().c_str());
       n[idx].kind=CORBA::string_dup(Cdmw::FT::VERSION_KIND.c_str());
       ++idx;
    }
    if (!m_repository_id.empty()) {
       n.length(idx+1);
       n[idx].id=CORBA::string_dup(m_repository_id.c_str());
       n[idx].kind=CORBA::string_dup(Cdmw::FT::TYPEID_KIND.c_str());
       ++idx;
    }
    if (m_object_group_id != Cdmw::FT::OGID_MIN) {
       n.length(idx+1);
       oss.str("");
       oss << m_object_group_id;
       n[idx].id=CORBA::string_dup(oss.str().c_str());
       n[idx].kind=CORBA::string_dup(Cdmw::FT::OGID_KIND.c_str());
       ++idx;
    }
    if (!m_group_name.empty()) {
       n.length(idx+1);
       n[idx].id=CORBA::string_dup(m_group_name.c_str());
       n[idx].kind=CORBA::string_dup(Cdmw::FT::NAME_KIND.c_str());
       ++idx;
    }
    std::string to_string__ = Cdmw::CommonSvcs::Naming::NamingInterface::to_string(n);
    CDMW_LOG_FUNCTION_RETURN(to_string__);
    return to_string__;
}

PortableServer::ObjectId * FallbackOid::to_ObjectId() const
{
    CDMW_LOG_FUNCTION(FTLogger::GetLogger(),"");
    std::string oid_s = this->to_string();
    PortableServer::ObjectId_var oid =
        PortableServer::string_to_ObjectId(oid_s.c_str());
    CDMW_LOG_FUNCTION_RETURN(oid_s);
    return oid._retn();
}


}; // namespace FT
}; // namespace Cdmw

