/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO 
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2009. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */



#include "ftcommon/TAOUltimateFallbackFactory.hpp"

#include <Foundation/osthreads/MutexGuard.hpp>
#include <Foundation/common/Assert.hpp>
#include <FaultTolerance/ftcommon/FTLogger.hpp>
#include <FaultTolerance/ftcommon/FTSystemProperties.hpp>
#include <Foundation/logging/FunctionLogger.hpp>
#include <sstream>


#include "tao/ORB_Core.h"
#include "tao/GIOP_Message_Version.h"
#include "tao/IIOP_Profile.h"
#include "tao/Transport_Acceptor.h"
#include "orbsvcs/orbsvcs/PortableGroup/GOA.h"

// anonymous namespace
namespace
{
   std::string ObjectId_to_string(const PortableServer::ObjectId & oid)
   {
      CORBA::String_var oids = PortableServer::ObjectId_to_string (oid);
      std::string s(oids.in());
      return s;
   }
} // End of anonymous namespace

namespace Cdmw {
namespace FT {
   
   TAOUltimateFallbackFactory::TAOUltimateFallbackFactory(CORBA::ORB_ptr orb,
                                                          const char* host,
                                                          unsigned short int port,
                                                          PortableServer::ServantLocator* locator=0)
      throw (CORBA::SystemException)
      : m_orb(CORBA::ORB::_duplicate(orb)),
        m_poa(PortableServer::POA::_nil()),
        m_host(host),
        m_port(port),
        m_poa_id(0)
   {
      CDMW_LOG_FUNCTION(FTLogger::GetLogger(),"host=" << host << ",port="<<port);
      CORBA::Object_var poa_object =
         m_orb->resolve_initial_references("RootPOA");
      
      PortableServer::POA_var root_poa =
         PortableServer::POA::_narrow (poa_object.in ());
      
      m_poa = this->create_persistent_POA(root_poa.in(),locator);
      /*
        TAO_POA* castPOA = m_poa->_tao_poa_downcast ();
        m_poa_id =  castPOA->id();
      */
      m_poa_id = m_poa->id();
   }
    
   TAO::ObjectKey *
   TAOUltimateFallbackFactory::create_object_key (const CORBA::OctetSeq & poa_id,
                                                  const PortableServer::ObjectId &id)
   {
      CDMW_LOG_FUNCTION(FTLogger::GetLogger(),"");
      // Calculate the space required for the key.
      CORBA::ULong buffer_size =
         poa_id.length () +
         id.length ();
        
      // Create the buffer for the key.
      CORBA::Octet *buffer = TAO::ObjectKey::allocbuf (buffer_size);
        
      // First copy the POA id into the key.
      ACE_OS::memcpy (&buffer[0],
                      poa_id.get_buffer (),
                      poa_id.length ());
        
      // Then copy the object id into the key.
      ACE_OS::memcpy (&buffer[poa_id.length ()],
                      id.get_buffer (),
                      id.length ());
        
      // Create the key, giving the ownership of the buffer to the
      // sequence.
      TAO::ObjectKey *key = 0;
      ACE_NEW_RETURN (key,
                      TAO::ObjectKey (buffer_size,
                                      buffer_size,
                                      buffer,
                                      1),
                      0);
        
      return key;
   }
    

   std::string 
   TAOUltimateFallbackFactory::create_object_key (const PortableServer::ObjectId &oid)
   {
      CDMW_LOG_FUNCTION(FTLogger::GetLogger(),
                        "oid=" << ObjectId_to_string(oid));  
      TAO::ObjectKey_var key =
         this->create_object_key (m_poa_id.in(),oid);

      CORBA::String_var key_str;
      TAO::ObjectKey::encode_sequence_to_string (key_str.out (), key.in());
      CDMW_DEBUG(FTLogger::GetLogger(),
                 "create_object_key (oid=" << ObjectId_to_string(oid) << ")=" << key_str.in());
      std::string s(key_str.in());
      CDMW_LOG_FUNCTION_RETURN(s);
      return s;
      
   }
    
   CORBA::Object_ptr 
   TAOUltimateFallbackFactory::create_reference_with_user_id(const PortableServer::ObjectId & oid,
                                                             const char* type_id)
   {
      CDMW_LOG_FUNCTION(FTLogger::GetLogger(),
                        "oid=" << ObjectId_to_string(oid) << ",type_id="<<type_id);
        
      TAO::ObjectKey_var key =
         this->create_object_key (m_poa_id.in(),oid);

      if (FTLogger::GetLogger()->is_debug_enabled()) {
         CORBA::String_var key_str;
         TAO::ObjectKey::encode_sequence_to_string (key_str.out (), key.in());
         CDMW_DEBUG(FTLogger::GetLogger(),
                    "create_object_key (oid=" << ObjectId_to_string(oid) << ",type_id="<<type_id
                    << ")=" << key_str.in());
      }
        
      TAO_GIOP_Message_Version GIOP_ver (TAO_DEF_GIOP_MAJOR,
                                         TAO_DEF_GIOP_MINOR);
      std::ostringstream ostr;
      ostr << m_host << ":" << m_port;
        
      TAO_IIOP_Profile *iiop_profile =new TAO_IIOP_Profile(m_host.c_str(),m_port,
                                                           key.in(),
                                                           ACE_INET_Addr (ostr.str().c_str()),
                                                           GIOP_ver,
                                                           m_orb->orb_core());
      iiop_profile->tagged_components ().set_orb_type (TAO_ORB_TYPE);
      //        iiop_profile.encode_endpoints();
        
        
      IOP::IOR ior;
      ior.type_id = type_id;
      ior.profiles.length(1);
        
      ior.profiles[0L].tag = IOP::TAG_INTERNET_IOP;
        
      TAO_OutputCDR encap (ACE_CDR::DEFAULT_BUFSIZE,
                           TAO_ENCAP_BYTE_ORDER,
                           m_orb->orb_core()->output_cdr_buffer_allocator (),
                           m_orb->orb_core()->output_cdr_dblock_allocator (),
                           m_orb->orb_core()->output_cdr_msgblock_allocator (),
                           m_orb->orb_core()->orb_params ()->cdr_memcpy_tradeoff (),
                           GIOP_ver.major,
                           GIOP_ver.minor );
        
        
      TAO_Profile & the_profile = *iiop_profile;
        
      the_profile.encodeAddressInfo(encap);
      // OCTET SEQUENCE for object key
      TAO::ObjectKey* key__ = the_profile._key();
      TAO::ObjectKey_var dummy_key = key__;
      if (key__!=0)
         encap << *key__;
      if (the_profile.version().major > 1
          || the_profile.version().minor > 0)
         the_profile.tagged_components ().encode (encap);
        
      CORBA::ULong length = static_cast<CORBA::ULong>(encap.total_length ());
        
      ior.profiles[0L].profile_data.length (length);
      CORBA::Octet *buffer =
         ior.profiles[0L].profile_data.get_buffer ();
      for (const ACE_Message_Block *i = encap.begin ();
           i != encap.end ();
           i = i->next ())
      {
         ACE_OS::memcpy (buffer, i->rd_ptr (), i->length ());
         buffer += i->length ();
      }
        
      TAO_OutputCDR iorOut (ACE_CDR::DEFAULT_BUFSIZE,
                            TAO_ENCAP_BYTE_ORDER,
                            m_orb->orb_core()->output_cdr_buffer_allocator (),
                            m_orb->orb_core()->output_cdr_dblock_allocator (),
                            m_orb->orb_core()->output_cdr_msgblock_allocator (),
                            m_orb->orb_core()->orb_params ()->cdr_memcpy_tradeoff (),
                            GIOP_ver.major,
                            GIOP_ver.minor );
      iorOut << ior;
      TAO_InputCDR iorIn (iorOut);
      CORBA::Object_ptr objref = CORBA::Object::_nil ();
      iorIn >> objref;
      iiop_profile->_decr_refcnt();
      return objref;
   }
    
   PortableServer::POA_ptr 
   TAOUltimateFallbackFactory::create_persistent_POA(PortableServer::POA_ptr root_poa,
                                                     PortableServer::ServantLocator* locator)
   { 
      CDMW_LOG_FUNCTION(FTLogger::GetLogger(),"");
      PortableServer::POAManager_var poa_manager =
         root_poa->the_POAManager ();
        
      CORBA::PolicyList policies;
      policies.length (4);
      policies[0] =
         root_poa->create_lifespan_policy(PortableServer::PERSISTENT);
      policies[1] =
         root_poa->create_id_assignment_policy (PortableServer::USER_ID);  
      // Tell the POA to use a servant manager
      policies[2] =
         root_poa->create_request_processing_policy (PortableServer::USE_SERVANT_MANAGER);
      policies[3] =
         root_poa->create_servant_retention_policy (PortableServer::NON_RETAIN);
        
        
      PortableServer::POA_var persistent_poa =
         root_poa->create_POA(Cdmw::FT::FALLBACK_POA_NAME.c_str(),
                              poa_manager.in (),
                              policies);
        
      if (locator != 0) 
         persistent_poa->set_servant_manager(locator);
        
      for (CORBA::ULong i=0L; i < policies.length();++i)
         policies[i]->destroy ();
        
      return persistent_poa._retn();
   }

}; // namespace FT
}; // namespace Cdmw

