/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO 
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2009. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#include <Foundation/ossupport/SystemProperties.hpp>
#include <Foundation/orbsupport/OrbSupport.hpp>
#include <Foundation/orbsupport/StrategyList.hpp>
#include <Foundation/ossupport/OS.hpp>
#include <Foundation/common/Options.hpp>
#include <Foundation/common/Exception.hpp>
#include <Foundation/common/Assert.hpp>
#include <Foundation/logging/LogManager.hpp>
#include <FaultTolerance/ftcommon/UltimateFallbackFactory.hpp>
#include <FaultTolerance/ftcommon/FallbackOid.hpp>
#include <Foundation/commonsvcs/naming/NamingInterface.hpp>
#include <FaultTolerance/ftcommon/FTManagerGroups.hpp> // for OGID_MIN


#include <sstream>
#include <fstream>
#include <iostream>
#include <ios>
#include <string>
#include <iomanip>
#include <cstdlib>


namespace
{

   const std::string NAME_OPTION       ="--name";  
   const std::string HOST_OPTION       ="--host";
   const std::string PORT_OPTION       ="--port";
   const std::string TYPE_ID_OPTION    ="--type-id";
   const std::string KIND_OPTION       ="--kind";
   const std::string OUTPUT_FILE_OPTION="--output-file";
   const std::string HELP_OPTION       ="--help";

   const std::string OBJECT_KEY_KIND="object-key";
   const std::string CORBALOC_KIND="corbaloc";
   const std::string IOR_KIND="ior";


   void print_option(std::ostream& os, 
                     const std::string & option,
                     const char * value,
                     const char*  description,
                     const char*  default_value = "")
   {
      std::string opt(option);
      opt += "=";
      opt += value;
      os << "\t" << std::setw(45) << std::left << opt << ": " << description;
      if (strlen(default_value) != 0) 
         os << "Default is " << default_value << ".";
      os << "\n";
   }

   void print_usage(int argc, char* argv[])
   {
      std::ostringstream ostr;
      ostr 
         << "\nUsage: " << argv[0] << " " << NAME_OPTION << "<object's fully qualified name> [OPTION] ...\n"
         << "Command line tool for creating CARDAMOM long-lived object references.\n\n"
         << "Available options:\n";
      print_option(ostr,HOST_OPTION,"<hostname|ip address>","Only needed for corbaloc or ior");
      print_option(ostr,PORT_OPTION,"<port number>","Only needed for corbaloc or ior");
      print_option(ostr,TYPE_ID_OPTION,"<the object's repository id>","Only needed for ior");
      print_option(ostr,KIND_OPTION,"corbaloc|ior|object-key","Different output kinds.",OBJECT_KEY_KIND.c_str());
      print_option(ostr,OUTPUT_FILE_OPTION,"<destination file>","File when output is to be stored");
      print_option(ostr,HELP_OPTION,"","Print this Usage.");      
      std::cout << ostr.str() << std::endl;
   }

   class OptionErrorException : public Cdmw::Exception
   {
      public:
         
         /**
          * Construct the exception object
          *
          * @parameter reason : reason of the initialisation failure
          * @parameter severity : Severity of exception
          */        
         OptionErrorException( const std::string& reason)
            : Cdmw::Exception( Cdmw::Exception::FATAL, "Initialisation Error : " + reason )
         {}
         ~OptionErrorException() throw() {}
   };
   
};

int main(int argc, char* argv[])
{
   using namespace Cdmw::OsSupport;
   using namespace Cdmw::OrbSupport;
   using namespace Cdmw::Common;
   try {
      try {
         // First initialise the CARDAMOM SystemProperties and the 
         // logging library is initialised.
         (void*) Cdmw::OsSupport::SystemProperties::Init(argc, argv);
         Cdmw::Logging::LogManager::Init(argc, argv);
         // From now on, CARDAMOM logging library is initialised.
      } catch (const std::exception & e ) {
         std::ostringstream ostr;
         ostr << "Cannot initialise CARDAMOM SystemProperties or Logging library! \n"
              << "Caught exception : " << e.what();
         throw Cdmw::InitErrorException(ostr.str());
      }
      // Get help option
      std::string help = OS::get_option_value (argc, argv, HELP_OPTION);
      if (help != "no") {
         print_usage(argc,argv);
         ::exit(EXIT_SUCCESS);
      }
      
      // Get name
      std::string name = OS::get_option_value (argc, argv, NAME_OPTION);
      if ((name == "no") || (name == "yes")) {
         throw OptionErrorException("Object name missing.");
      } else {
         // Check if name is properly formed         
         try {
            CosNaming::Name_var name__ =
               Cdmw::CommonSvcs::Naming::NamingInterface::to_name(name);            
         } catch (const Cdmw::CommonSvcs::Naming::InvalidNameException & ex) {
            throw OptionErrorException("Object name ill-formed.");
         }
      }
      // Get kind
      std::string kind = OS::get_option_value (argc, argv, KIND_OPTION);
      if (kind != "no") {  
         if ((kind != OBJECT_KEY_KIND) &&
             (kind != CORBALOC_KIND) &&
             (kind != IOR_KIND)) {
            std::ostringstream os;
            os << "Kind value '" << kind << "' is unknown.\n" 
               << "Allowed values are " 
               << OBJECT_KEY_KIND << " | " 
               << CORBALOC_KIND   << " | " 
               << IOR_KIND << ".";
            throw OptionErrorException(os.str());
         }
      } else {
         // Only the name is required!
         kind = OBJECT_KEY_KIND;
      }
      // Get Host
      std::string host = OS::get_option_value (argc, argv, HOST_OPTION);
      if (host == "yes")
         throw OptionErrorException("Wrong host value.");
      else if (host == "no")
         host = "";
      // Get Port
      std::string port = OS::get_option_value (argc, argv, PORT_OPTION);
      if (port == "yes")
         throw OptionErrorException("Wrong port value.");
      else if (port == "no")
         port = "";
      else {
         int p = atoi(port.c_str());
         if (p <= 0)
            throw OptionErrorException("Invalid port number.");
      }
      // Get Type-id
      std::string type_id = OS::get_option_value (argc, argv, TYPE_ID_OPTION);
      if (type_id == "no")
         type_id = "";
      else if (!OrbSupport::is_type_id_valid(type_id.c_str())) {
         // Check if type-id is well formed
         std::ostringstream os;
         os << "Type-id '"<<type_id<<"' is ill-formed:\n" 
            << "Use OMG IDL Format\n" 
            << "\tIDL:identifier[/identifier]...:version-number\n" 
            << "version-number contains a version number with the following format:\n" 
            << "\tmajor.minor\n";
         throw OptionErrorException(os.str());
      }
      // Get Output file
      std::string output = OS::get_option_value (argc, argv, OUTPUT_FILE_OPTION);
      if (output == "yes")
         throw OptionErrorException("Missing destination file name.");
      else if (output == "no")
         output = "";


      // Create ORB
      // Initialize the ORB
      // StrategyList orbStrategies;
//       orbStrategies.add_OrbThreaded();
//       orbStrategies.add_PoaThreadPerRequest();
      
//       CORBA::ORB_var orb = OrbSupport::ORB_init(argc, argv, orbStrategies);
      CORBA::ORB_var orb = CORBA::ORB_init(argc, argv);
      {
         // Activate the POA manager
         CORBA::Object_var obj
            = orb->resolve_initial_references( "RootPOA" );
         PortableServer::POA_var rootPOA = PortableServer::POA::_narrow( obj.in() );
         
         PortableServer::POAManager_var poaMgr = rootPOA->the_POAManager();
         poaMgr->activate();
      }


      // Create Ultimatre Fallback Factory
      Cdmw::FT::UltimateFallbackFactory * ff = 
         Cdmw::FT::UltimateFallbackFactory::Init(orb.in(),
                                                 host.c_str(),atoi(port.c_str()));
      // Create Oid from name      
      using namespace Cdmw::CommonSvcs::Naming; 
      CosNaming::Name_var name__ = NamingInterface::to_name(name);     
      std::string group_domain_id = NamingInterface::NameComponent_to_string(name__[0L]);
      std::string group_name ("");
      CORBA::ULong name_len = name__->length();
      if (name_len > 1) {
         CosNaming::Name n__(name_len - 1);
         n__.length(name_len - 1);
         
         for (CORBA::ULong i = 1L; i< name_len; i++)
         {
            n__[i-1] = name__[i];
         }
         group_name = NamingInterface::to_string(n__);
      }
      std::string ior = "";
      if (kind == IOR_KIND) {
         if (host.empty() || port.empty() || type_id.empty())
            throw OptionErrorException("host or port or type-id missing.");
         Cdmw::FT::FallbackOid foid(group_domain_id.c_str(),
                                    Cdmw::FT::OGID_MIN,
                                    0,
                                    type_id.c_str(),
                                    group_name.c_str());
         PortableServer::ObjectId_var oid = foid.to_ObjectId();
         CORBA::Object_var obj = ff->create_reference_with_user_id(oid.in(),type_id.c_str());
         CORBA::String_var s = orb->object_to_string(obj.in());
         ior = s.in();
      } else {
         Cdmw::FT::FallbackOid foid(group_domain_id.c_str(),
                                    Cdmw::FT::OGID_MIN,
                                    0,
                                    "",
                                    group_name.c_str());
         PortableServer::ObjectId_var oid = foid.to_ObjectId();
         std::string object_key = ff->create_object_key(oid.in());
         ior = object_key;
         if (kind == CORBALOC_KIND) {
            if (host.empty() || port.empty())
               throw OptionErrorException("host or port missing.");
            std::string url("corbaloc::");
            url+=host;
            url+=":";
            url+=port;
            url+="/";
            url+=object_key;
            ior = url;
         }
      }
      
      if (output.empty())
         std::cout << ior << std::endl;
      else {
         std::ofstream  out_file;
         out_file.open(output.c_str(),std::ios_base::out);
        if( !out_file )
           throw OptionErrorException("Cannot write on " + output);
        out_file<<ior;
      }
      
   } catch (const OptionErrorException & ex) {
      std::cerr << ex.what() << std::endl;
      print_usage(argc,argv);
      ::exit(EXIT_FAILURE);
   } catch (const Cdmw::InitErrorException & ex) {
      std::cerr << ex.what() << std::endl;
      print_usage(argc,argv);
      ::exit(EXIT_FAILURE);
   } catch (CORBA::Exception & ex) {
      std::cerr << "Unexpected CORBA::Exception:\n"
                << Cdmw::OrbSupport::OrbSupport::exception_to_string(ex)
                << std::endl;
      ::exit(EXIT_FAILURE);
   }
   return EXIT_SUCCESS;
}


