/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO 
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2009. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


// Cdmw Files
#include <Foundation/common/Assert.hpp> 
#include <Foundation/commonsvcs/time/Util.hpp>
#include <Foundation/orbsupport/CORBA.hpp>
#include <Foundation/orbsupport/ExceptionMinorCodes.hpp>
#include <Foundation/orbsupport/CORBAExceptionHelper.hpp>
#include <Foundation/ossupport/OS.hpp>
#include <Foundation/ossupport/SystemProperties.hpp>
#include <Foundation/commonsvcs/naming/NamingInterface.hpp>

#include <FaultTolerance/ftcommon/FTLogger.hpp>
#include <Foundation/logging/FunctionLogger.hpp>

// Local Files
#include <ftinit/FTServerRequestInterceptor_impl.hpp>
#include <FaultTolerance/ftcommon/MacroDef.hpp>
#include <idllib/FT.stub.hpp>
#include <FaultTolerance/ftcommon/FTObjectIds.hpp>
#include <FaultTolerance/ftcommon/FTSystemProperties.hpp>
#include <orbsupport/MarshallingBuffer.hpp>

#include <sstream>



#define TEST_AND_THROW(T,repid) \
   (strcmp(id,repid) == 0) \
    do { \
        CDMW_DEBUG(FTLogger::GetLogger(), #T << " exception raised in receive_request_service_contexts!\n"); \
        const T * ex;                                                  \
        exception_any.in() >>= ex;                          \
        CDMW_LOG_FUNCTION_EXCEPTION(*ex); \
        throw *ex;  \
    } while(0)




namespace 
{
    // Default hold time for client performing requests on backup but with
    // a correct group version. This is to avoid "ping-pong" between client
    // and this server interceptor which could overload cpu.
    const int DEFAULT_RECOVERY_HOLD_TIME = 100;
    
    
    const PortableInterceptor::ORBInitInfo::ObjectId FT_SERVER_REQUEST_INTERCEPTOR_NAME 
        = "FTServerRequestInterceptor";
//     const PortableInterceptor::ORBInitInfo::ObjectId FT_REPLICATION_MANAGER 
//         = "ReplicationManager";
//     const PortableInterceptor::ORBInitInfo::ObjectId FT_GROUP_REPOSITORY 
//         = "FTGroupRepository";

} // end of anonymous namespace


namespace Cdmw
{
    
namespace FT
{

namespace FTInit
{



ServerRequestInterceptor_impl::ServerRequestInterceptor_impl(
                 PortableInterceptor::ORBInitInfo_ptr info,
                 const PortableInterceptor::SlotId  ft_domain_id_slot_id,
                 const PortableInterceptor::SlotId  object_group_id_slot_id,
                 const PortableInterceptor::SlotId  object_group_ref_version_slot_id,
                 const PortableInterceptor::SlotId  client_id_slot_id,
                 const PortableInterceptor::SlotId  retention_id_slot_id,
                 const PortableInterceptor::SlotId  expiration_time_slot_id,
                 const PortableInterceptor::SlotId  is_a_ft_request_slot_id,
                 const PortableInterceptor::SlotId  exception_raised_slot_id,
                 const PortableInterceptor::SlotId  CORBA_exception_slot_id)
    : m_ft_domain_id_slot_id(ft_domain_id_slot_id),
      m_object_group_id_slot_id(object_group_id_slot_id),
      m_object_group_ref_version_slot_id(object_group_ref_version_slot_id),
      m_client_id_slot_id(client_id_slot_id),
      m_retention_id_slot_id(retention_id_slot_id),
      m_expiration_time_slot_id(expiration_time_slot_id),
      m_is_a_ft_request_slot_id(is_a_ft_request_slot_id),
      m_exception_raised_slot_id(exception_raised_slot_id),
      m_CORBA_exception_slot_id(CORBA_exception_slot_id),
      m_recorder(0)
{
    CDMW_LOG_FUNCTION(FTLogger::GetLogger(),"");
    m_processId = Cdmw::OsSupport::OS::get_processId();
    m_recorder = NULL;
    
    CDMW_DEBUG(FTLogger::GetLogger(),"m_recorder: " << m_recorder);
    //
    // Get the codec factory
    //
    IOP::CodecFactory_var factory = info->codec_factory();
    CDMW_ASSERT(!CORBA::is_nil(factory.in()));

    //
    // Create codec
    //
    IOP::Encoding how;
    how.major_version = 1;
    how.minor_version = 0;
    how.format = IOP::ENCODING_CDR_ENCAPS;

    try
    {
        m_cdrCodec = factory->create_codec(how);
    }
    catch(const IOP::CodecFactory::UnknownEncoding&)
    {
        CDMW_ASSERT(false);
    }

    CDMW_ASSERT(!CORBA::is_nil(m_cdrCodec.in()));

    try
    {
        CORBA::Object_var obj = info->resolve_initial_references(::Cdmw::FT::FT_GROUP_REPOSITORY_ObjectId);
        m_group_repository 
            = CdmwFT::Location::PrimaryBackupGroupRepository::_narrow(obj.in());
    }
    catch (const PortableInterceptor::ORBInitInfo::InvalidName& )
    {
        CDMW_FATAL(FTLogger::GetLogger(),
                   "Impossible to resolve initial reference: InvalidName raised!");
        CDMW_ASSERT(false);
    }
    catch (const CORBA::SystemException& ex)
    {
        CDMW_FATAL(FTLogger::GetLogger(),
                   "Impossible to resolve initial reference: CORBA::SystemException raised!\n" 
                   << ex );
        CDMW_ASSERT(false);
    }

    CDMW_ASSERT(!CORBA::is_nil(m_group_repository.in()));
}

char*
ServerRequestInterceptor_impl::name() 
    throw (CORBA::SystemException)
{
    CDMW_LOG_FUNCTION(FTLogger::GetLogger(),"");
    CDMW_LOG_FUNCTION_RETURN(FT_SERVER_REQUEST_INTERCEPTOR_NAME);
    return CORBA::string_dup(FT_SERVER_REQUEST_INTERCEPTOR_NAME);
}

void
ServerRequestInterceptor_impl::destroy() 
    throw (CORBA::SystemException)
{
    CDMW_LOG_FUNCTION(FTLogger::GetLogger(),"");
}

void
ServerRequestInterceptor_impl::receive_request_service_contexts(
        PortableInterceptor::ServerRequestInfo_ptr ri)
    throw (CORBA::SystemException, PortableInterceptor::ForwardRequest)
{
    CDMW_LOG_FUNCTION(FTLogger::GetLogger(),"operation="<<ri->operation());
    if (false == m_bUseFTInterceptionPoint) {
       try {     
         ::FT::FTRequestServiceContext_var          ft_request_context;
           if (check_incoming_request(ri,ft_request_context.out())) {
               // Request is valid. There is no request logging so no handling 
               // of expiration time & request logging
               if (m_recorder) {
                   CDMW_DEBUG(FTLogger::GetLogger(),"reply is not recorded");
                   
                   TimeBase::TimeT expiration_time = ft_request_context->expiration_time;
                   TimeBase::TimeT current_time = CommonSvcs::Time::Util::get_current_time();
   
                   if (expiration_time < current_time ) {
                       CDMW_INFO(FTLogger::GetLogger(),
                                 "Request has expired (expiration_time=" << expiration_time
                                 << ", now=" << current_time << "). Raising BAD_CONTEXT exception.");
                       // The operation may have already been executed so
                       // in accordance with the CORBA FT specification
                       // we raise BAD_CONTEXT
                       // NOTA: OMG Specification did not specify a minor code!
                       CORBA::BAD_CONTEXT ex(OrbSupport::BAD_CONTEXTRequestExpired, CORBA::COMPLETED_NO);
                       CDMW_LOG_FUNCTION_EXCEPTION(ex);
                       throw ex;
                   }
               }
           }
       } catch (const PortableInterceptor::ForwardRequest & ex) {
           CDMW_DEBUG(FTLogger::GetLogger(),
                      "PortableServer::ForwardRequest exception raised in receive_request_service_contexts!\n");
           CDMW_LOG_FUNCTION_EXCEPTION(ex);
           throw;
       } catch (const CORBA::SystemException& ex) {
           CDMW_DEBUG(FTLogger::GetLogger(),
                      "CORBA System exception raised in receive_request_service_contexts!\n" << ex);
           CDMW_LOG_FUNCTION_EXCEPTION(ex);
           throw;
       }
   } else {
       // cdmw_ft_interception_point has already perfomed checking ... etc.
       bool exception_raised = false;
       CORBA::Any_var any = ri->get_slot(m_exception_raised_slot_id);
       if (!(any >>= CORBA::Any::to_boolean(exception_raised))) {
          CDMW_ERROR(FTLogger::GetLogger(),
                     "ERROR: unexpected data returned by get_slot" );
          CORBA::INTERNAL ex(OrbSupport::INTERNALFaultToleranceError, CORBA::COMPLETED_NO);
          CDMW_LOG_FUNCTION_EXCEPTION(ex);
          throw ex;
       }
       if (exception_raised) {       
          CORBA::Any_var exception_any = ri->get_slot(m_CORBA_exception_slot_id);
          // First check for PortableInterceptor::ForwardRequest exception
          {
             CORBA::TypeCode_var tc = exception_any->type ();
             const char* id = tc->id();
             if (strcmp(id,"IDL:omg.org/PortableInterceptor/ForwardRequest:1.0") == 0) {
                PortableInterceptor::ForwardRequest * fwd_request =
                   OrbSupport::CORBAExceptionHelper::Extract_Exception<PortableInterceptor::ForwardRequest>
                   (exception_any.in());
                if (fwd_request) {
                   CDMW_DEBUG(FTLogger::GetLogger(), 
                              " PortableInterceptor::ForwardRequest Exception raised in receive_request_service_contexts!\n");
                   CDMW_LOG_FUNCTION_EXCEPTION(*fwd_request);
                   fwd_request->_raise();
                }
             }
          }
          // Then check for CORBA::SystemException exception
          CORBA::SystemException * system_exception = 
             OrbSupport::CORBAExceptionHelper::Extract_SystemException(exception_any.in());
          if (system_exception) {
             CDMW_DEBUG(FTLogger::GetLogger(), 
                        " CORBA System Exception raised in receive_request_service_contexts!\n"
                        << OrbSupport::OrbSupport::exception_to_string(*system_exception));
             CDMW_LOG_FUNCTION_EXCEPTION(*system_exception);
             system_exception->_raise();
          }
          CDMW_NEVER_HERE();
       }
   } // if (false == m_bUseFTInterceptionPoint)
       
}

void
ServerRequestInterceptor_impl::receive_request(PortableInterceptor::ServerRequestInfo_ptr ri)
        throw (CORBA::SystemException, PortableInterceptor::ForwardRequest)
{
    CDMW_LOG_FUNCTION(FTLogger::GetLogger(),"operation="<<ri->operation());
    // Do nothing as this is done in receive_request_service_contexts() or cdmw_ft_interception_point()
}


void
ServerRequestInterceptor_impl::send_reply(PortableInterceptor::ServerRequestInfo_ptr ri)
    throw (CORBA::SystemException)
{
    CDMW_LOG_FUNCTION(FTLogger::GetLogger(),"operation="<<ri->operation());

    if (true == m_bUseFTInterceptionPoint) {
       if (m_recorder)
       {
           ::FT::FTRequestServiceContext* context = 0; 
       
           try
           {
              context = extract_FT_REQUEST_service_context(ri,this->get_location()); // FIXME: TAO BUG Workaround: 
               // calling get_FTRequestServiceContext_from_slots(ri) causes an assertion in get_slot() 
               // get_FTRequestServiceContext_from_slots(ri); 
       
               if (context)
               { // FT Request. Log reply
                   CORBA::Any_var result = ri->result ();
                   
                   Cdmw::OrbSupport::MarshallingBuffer marshBuf;

                   marshBuf.add_any(result.in());
                   
                   Dynamic::ParameterList_var pl = ri->arguments ();
                   CORBA::ULong len = pl->length ();
   
                   for (CORBA::ULong index = 0; index != len ; ++index)
                   {
                       // Save INOUT & OUT parameters
                       if ((*pl)[index].mode == CORBA::PARAM_INOUT)
                       {
                          marshBuf.add_any((*pl)[index].argument);
                       } 
                       else if ((*pl)[index].mode == CORBA::PARAM_OUT)
                       {
                          marshBuf.add_any((*pl)[index].argument);
                       }
                   }
       
                   CORBA::OctetSeq_var reply;
                   reply = marshBuf.get_buffer();
       
                   m_recorder->record(context->client_id.in(),
                                      context->retention_id,
                                      context->expiration_time,
                                      reply.in());
                   delete context;
               }
           }
           catch (const CORBA::SystemException& ex)
           {
               CDMW_DEBUG(FTLogger::GetLogger(),
                          "ERROR: send_reply FT_REQUEST raises exception " << ex);
               delete context;
               CDMW_LOG_FUNCTION_EXCEPTION(ex);
               throw;
           }
       }
    } // if (true == m_bUseFTInterceptionPoint)
}

void
ServerRequestInterceptor_impl::send_other(PortableInterceptor::ServerRequestInfo_ptr ri)
        throw (CORBA::SystemException, PortableInterceptor::ForwardRequest)
{
    CDMW_LOG_FUNCTION(FTLogger::GetLogger(),"operation="<<ri->operation());
    // nothing to do!
}

void
ServerRequestInterceptor_impl::send_exception(PortableInterceptor::ServerRequestInfo_ptr ri)
        throw (CORBA::SystemException, PortableInterceptor::ForwardRequest)
{
    CDMW_LOG_FUNCTION(FTLogger::GetLogger(),"operation="<<ri->operation());
    // nothing to do!
}


void
ServerRequestInterceptor_impl::cdmw_ft_interception_point(PortableInterceptor::ServerRequestInfo_ptr ri,
                                                         CORBA::OctetSeq_out ocs)
    throw (CORBA::SystemException, PortableInterceptor::ForwardRequest)
{
   CDMW_LOG_FUNCTION(FTLogger::GetLogger(),"operation="<<ri->operation());
   // cdmw_ft_interception_point() does not handle exceptions correctly! 
   // PortableInterceptor::ForwardRequest exception seems to work but
   // we better handle CORBA::Exception (s) coherently!
   try {
      ::FT::FTRequestServiceContext_var ft_request_context;
      if (check_incoming_request(ri,ft_request_context.out())) {
         
         // Request is valid. Check if already logged
         if (m_recorder)
         {
            try
            {
               m_recorder->get_reply(ft_request_context->client_id.in(),
                                     ft_request_context->retention_id, ocs);
               CDMW_DEBUG(FTLogger::GetLogger(),"reply is recorded, returing previous reply.");
            }
            catch (const Cdmw::Common::NotFoundException&)
            {
               // Request is not recorded. Check expiration time
               CDMW_DEBUG(FTLogger::GetLogger(),"reply was not recorded");
               
               TimeBase::TimeT expiration_time = ft_request_context->expiration_time;
               
               TimeBase::TimeT current_time = CommonSvcs::Time::Util::get_current_time();
               if (expiration_time < current_time ) {
                  CDMW_INFO(FTLogger::GetLogger(),
                            "Request has expired (expiration_time=" << expiration_time
                            << ", now=" << current_time << "). Raising BAD_CONTEXT exception.");
                  // The operation may have already been executed so
                  // in accordance with the CORBA FT specification
                  // we raise BAD_CONTEXT
                  // NOTA: OMG Specification did not specify a minor code!
                  CORBA::BAD_CONTEXT ex(OrbSupport::BAD_CONTEXTRequestExpired, CORBA::COMPLETED_NO);
                  CDMW_LOG_FUNCTION_EXCEPTION(ex);
                  throw ex;
               }
            }
         }
      }
      // set the slot id to indicate that this request has not generated any exception!
      bool exception_raised = false;
      CORBA::Any slotData;
      slotData <<= CORBA::Any::from_boolean(exception_raised);
      ri->set_slot(m_exception_raised_slot_id, slotData);
   } catch (const CORBA::SystemException & ex) {
      CDMW_DEBUG(FTLogger::GetLogger(),
                 "CORBA::SystemException raised in cdmw_ft_interception_point!" << ex );
      // set the slot id to indicate that this request has generated an exception!
      bool exception_raised = true;
      CORBA::Any slotData, exceptionData;
      OrbSupport::CORBAExceptionHelper::Insert_SystemException(exceptionData,ex);
      ri->set_slot(m_CORBA_exception_slot_id, exceptionData);
      slotData <<= CORBA::Any::from_boolean(exception_raised);
      ri->set_slot(m_exception_raised_slot_id, slotData);
   } catch (const PortableInterceptor::ForwardRequest & ex) {
      CDMW_DEBUG(FTLogger::GetLogger(),
                 "PortableInterceptor::ForwardRequest raised in cdmw_ft_interception_point!" << ex );
      // set the slot id to indicate that this request has generated an exception!
      bool exception_raised = true;
      CORBA::Any slotData, exceptionData;
      OrbSupport::CORBAExceptionHelper::Insert_Exception(exceptionData,ex);
      ri->set_slot(m_CORBA_exception_slot_id, exceptionData);
      slotData <<= CORBA::Any::from_boolean(exception_raised);
      ri->set_slot(m_exception_raised_slot_id, slotData);
   }
}



void
ServerRequestInterceptor_impl::set_reply_recorder
(Cdmw::FT::ReplyRecording* recorder)
{
    CDMW_LOG_FUNCTION(FTLogger::GetLogger(),"");
    m_recorder = recorder;
}






/**
 * This operation extracts the (FT-specific) service contexts that may 
 * be attached to the request. It then sets appropriate PI slots
 * (from now on, FTCurrent can be used to get these info)
 *
 * @return true if this an FT request, false otherwise
 *
 * @exception TODO: describe exceptions...
 */
bool
ServerRequestInterceptor_impl::get_service_contexts(PortableInterceptor::ServerRequestInfo_ptr ri,
                                                    const char *                        location_str,
                                                    ::FT::FTRequestServiceContext_out   ft_request_context,
                                                    ::FT::TagFTGroupTaggedComponent_out ft_group_version_context)
    throw (CORBA::SystemException)
{
   CDMW_LOG_FUNCTION(FTLogger::GetLogger(),"operation="<<ri->operation()<<",location_str="<<location_str);

    ::FT::FTRequestServiceContext* FT_REQUEST_sc = extract_FT_REQUEST_service_context(ri,location_str);
    
    if (FT_REQUEST_sc) {
       //
       // Store Request Service Context in slot 
       //
       CORBA::Any slotData;
       slotData <<= FT_REQUEST_sc->client_id.in();
       ri->set_slot(m_client_id_slot_id, slotData);
       
       slotData <<= FT_REQUEST_sc->retention_id;
       ri->set_slot(m_retention_id_slot_id, slotData);
       
       slotData <<= FT_REQUEST_sc->expiration_time;
       ri->set_slot(m_expiration_time_slot_id, slotData);
       ft_request_context = FT_REQUEST_sc;
    } else {
       // set the slot id to indicate that this is not a ft_request!
       CORBA::Any slotData;
       bool is_a_ft_request = false; /* default value*/
       
       slotData <<= CORBA::Any::from_boolean(is_a_ft_request);
       ri->set_slot(m_is_a_ft_request_slot_id, slotData);
       
       return false;
    }

    ::FT::TagFTGroupTaggedComponent* FT_GROUP_VERSION_sc =
         extract_FT_GROUP_VERSION_service_context(ri,location_str);
    if (FT_GROUP_VERSION_sc) {
       //
       // Store Request Service Context in slot 
       //
       CORBA::Any slotData;
       
       slotData <<= FT_GROUP_VERSION_sc->group_domain_id.in();
       ri->set_slot(m_ft_domain_id_slot_id, slotData);
       
       slotData <<= FT_GROUP_VERSION_sc->object_group_id;
       ri->set_slot(m_object_group_id_slot_id, slotData);
       
       slotData <<= FT_GROUP_VERSION_sc->object_group_ref_version;
       ri->set_slot(m_object_group_ref_version_slot_id, slotData);
       ft_group_version_context = FT_GROUP_VERSION_sc;
    } else {
       // FT_REQUEST is present and not FT_GROUP_VERSION
       // throw a BAD_CONTEXT exception       
       CDMW_ERROR(FTLogger::GetLogger(),"ERROR: FT_GROUP_VERSION service context is missing! ");
       // OMG spec: minor code 1 = IDL context not found
       CORBA::BAD_CONTEXT ex(CORBA::OMGVMCID | 1, CORBA::COMPLETED_NO); // Std OMG minor code
       CDMW_LOG_FUNCTION_EXCEPTION(ex);
       throw ex;
    }
    // set the slot id to indicate that this is a ft_request!
    CORBA::Any slotData;
    bool is_a_ft_request = true;
    
    slotData <<= CORBA::Any::from_boolean(is_a_ft_request);
    ri->set_slot(m_is_a_ft_request_slot_id, slotData);

    return true;
}

/**
 * This operation insures server-side FT CORBA failover semantics
 */
void 
ServerRequestInterceptor_impl::validate_request(PortableInterceptor::ServerRequestInfo_ptr ri,
                                                const char *                            location_str,
                                                const ::FT::FTRequestServiceContext &   ft_request_context,
                                                const ::FT::TagFTGroupTaggedComponent & ft_group_version_context)
    throw (CORBA::SystemException, PortableInterceptor::ForwardRequest)
{
   CDMW_LOG_FUNCTION(FTLogger::GetLogger(),"operation="<<ri->operation()<<",location_str="<<location_str);
    ::FT::ObjectGroupRefVersion object_group_version = 0;
    ::FT::ObjectGroupRefVersion request_object_group_version = ft_group_version_context.object_group_ref_version;
    ::FT::ObjectGroupId object_group_id = ft_group_version_context.object_group_id;

    try
    {
        // get group_version from the GroupRepository
        object_group_version =
            m_group_repository->get_object_group_version_from_gid(object_group_id);
        CDMW_DEBUG(FTLogger::GetLogger(), 
                   " group_version from the GroupRepository = " << object_group_version
                   << ", request_object_group_version = " << request_object_group_version);
    }
    catch (const ::FT::ObjectGroupNotFound& )
    {
        CDMW_WARN(FTLogger::GetLogger(), 
                   "ObjectGoup not known yet, throw transient");
        CORBA::TRANSIENT ex(OrbSupport::TRANSIENT, CORBA::COMPLETED_NO);
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw ex;
    }
    catch (const CORBA::SystemException& ex)
    {
        CDMW_ERROR(FTLogger::GetLogger(), 
                   "ERROR: System exception raised while checking the group version!\n" << ex);
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw;
    }

    // Now compare the both version
    if (request_object_group_version == object_group_version)
    {
        // we are in a primary backup configuration
        // check if we are the primary member
       bool is_primary = false; // Initialise value to avoid compiler warnings in optimisation mode
        
        try
        {
            is_primary = m_group_repository->is_primary_from_gid(object_group_id);
        }
        catch (const ::FT::ObjectGroupNotFound &)
        {
            CDMW_FATAL(FTLogger::GetLogger(),
                       "FATAL ERROR: is_primary_from_gid raises ObjectGroupNotFound exception ");
            CDMW_ASSERT(false);
//             CORBA::INTERNAL ex(OrbSupport::INTERNALFaultToleranceError, CORBA::COMPLETED_NO);
//             CDMW_LOG_FUNCTION_EXCEPTION(ex);
//             throw ex;
        }
        catch (const CORBA::SystemException& ex)
        {
            CDMW_ERROR(FTLogger::GetLogger(),
                       "ERROR: System exception raised while checking the primary member!\n"
                       << ex );
            CDMW_LOG_FUNCTION_EXCEPTION(ex);
            throw;
        }
        
        if (is_primary)
        {
            // all is ok, let the request go up!
            CDMW_DEBUG(FTLogger::GetLogger(), 
                       "Request is on the primary, let the request go up!");
            CDMW_LOG_FUNCTION_RETURN("");
            return;
        }
        else
        {
            // Throw transient, but after a delay to avoid client to directly
            // retry request, as such a "ping-pong" could provoke cpu oveload.
            static int hold_time = 
                Cdmw::OsSupport::SystemProperties::Get_Property( 
                    Cdmw::FT::RECOVERY_HOLD_TIME_PROPERTY_NAME,
                    DEFAULT_RECOVERY_HOLD_TIME);

            if (hold_time > 0) {
                CDMW_DEBUG(FTLogger::GetLogger(), 
                           "Not the primary, hold request for " 
                           << hold_time << "ms in case of ongoing recovery");
                Cdmw::OsSupport::OS::sleep(hold_time);

                // We could have sent transient here, but an optimization is
                // to check if group has been updated during hold time.
                try {
                    object_group_version =
                        m_group_repository->get_object_group_version_from_gid(object_group_id);
                } 
                catch (const CORBA::SystemException& ex)
                {
                    CDMW_ERROR(FTLogger::GetLogger(),
                               "ERROR: System exception raised while getting group version!\n"
                               << ex );
                    CDMW_LOG_FUNCTION_EXCEPTION(ex);
                    throw;
                }
                
                if (object_group_version > request_object_group_version) {
                    // group have been updated, throw forward request permanent
                    // (bool = 1) with the local group reference
                    CDMW_DEBUG(FTLogger::GetLogger(), 
                               "Group updated during hold time, throw location forward");
                    try {
                        ::FT::ObjectGroup_var object_group =
                            m_group_repository->get_object_group_from_gid(object_group_id);
                        PortableInterceptor::ForwardRequest ex(object_group.in());
                        CDMW_LOG_FUNCTION_EXCEPTION(ex);
                        throw ex;
                    }
                    catch (const CORBA::SystemException& ex)
                    {
                        CDMW_ERROR(FTLogger::GetLogger(),
                                   "ERROR: System exception raised while getting the group reference!\n"
                                   << ex);
                        CDMW_LOG_FUNCTION_EXCEPTION(ex);
                        throw;
                    }
                    
                } else {
                    // group has not been updated, throw transient
                    CDMW_DEBUG(FTLogger::GetLogger(), 
                               "No group update during hold time, throw transient");
                    CORBA::TRANSIENT ex(OrbSupport::TRANSIENT, CORBA::COMPLETED_NO);
                    CDMW_LOG_FUNCTION_EXCEPTION(ex);
                    throw ex;
                }
                
            } else {
                // no hold time, throw transient directly
                CDMW_DEBUG(FTLogger::GetLogger(), 
                           "Not the primary, throw transient directly");
                           
                CORBA::TRANSIENT ex(OrbSupport::TRANSIENT, CORBA::COMPLETED_NO);
                CDMW_LOG_FUNCTION_EXCEPTION(ex);
                throw ex;
            }
                       
        }
    }
    else
    {
        if (request_object_group_version < object_group_version)
        {
            // request object group reference is obsolete, 
            // get the local group reference 
            try
            {
                CDMW_DEBUG(FTLogger::GetLogger(), 
                           "Request object group reference is obsolete, get the local group reference and throw a location forward");
                ::FT::ObjectGroup_var object_group =
                    m_group_repository->get_object_group_from_gid(object_group_id);
    
                // throw a forward request permanent (bool = 1) with the local group reference
                CDMW_DEBUG(FTLogger::GetLogger(), 
                           "Throw a location forward permanent");
                PortableInterceptor::ForwardRequest ex(object_group.in());
                CDMW_LOG_FUNCTION_EXCEPTION(ex);
                throw ex;
            }
            catch (const ::FT::ObjectGroupNotFound &)
            {
                CDMW_FATAL(FTLogger::GetLogger(),
                           "FATAL ERROR: get_object_group_from_gid raises ObjectGroupNotFound exception ");
                CDMW_ASSERT(false);
//                 CORBA::INTERNAL ex(OrbSupport::INTERNALFaultToleranceError, CORBA::COMPLETED_NO);
//                 CDMW_LOG_FUNCTION_EXCEPTION(ex);
//                 throw ex;
            }
            catch (const CORBA::SystemException& ex)
            {
                CDMW_ERROR(FTLogger::GetLogger(),
                           "ERROR: System exception raised while getting the group reference!\n"
                           << ex);
                CDMW_LOG_FUNCTION_EXCEPTION(ex);
                throw;
            }
        }
        else
        {
            // request_object_group_version > object_group_version
            // local object group is obsolete, check with the replication manager
            ::FT::ObjectGroupRefVersion last_object_group_version = 0; // Initialise value to avoid compiler warnings in optimisation mode
            try
            {
                CDMW_DEBUG(FTLogger::GetLogger(), 
                           "Local object group is obsolete, check with the replication manager");
                CdmwFT::ReplicationManager_var replication_manager
                    = m_group_repository->the_replication_manager();
                last_object_group_version
                    = replication_manager->get_object_group_version_from_gid(object_group_id);
            }
            catch (const ::FT::ObjectGroupNotFound &)
            {
                CDMW_FATAL(FTLogger::GetLogger(),
                           "FATAL ERROR: get_object_group_version_from_gid raises ObjectGroupNotFound exception ");
                CDMW_ASSERT(false);
//                 CORBA::INTERNAL ex(OrbSupport::INTERNALFaultToleranceError, CORBA::COMPLETED_NO);
//                 CDMW_LOG_FUNCTION_EXCEPTION(ex);
//                 throw ex;
            }
            catch (const CORBA::SystemException& ex)
            {
                CDMW_ERROR(FTLogger::GetLogger(),
                           "ERROR: System exception raised while getting the group reference version from the replication manager!\n"
                           << ex );
                CDMW_LOG_FUNCTION_EXCEPTION(ex);
                throw;
            }

            if (last_object_group_version == request_object_group_version)
            {
                // The ReplicationManager has not updated the local GroupRepository
                // We should wait this update so that the primary will be updated, if a new
                // new one is elected => throw TRANSIENT exception!
                CDMW_DEBUG(FTLogger::GetLogger(), 
                           "The ReplicationManager has not updated the local GroupRepository. "
                           << "We should wait this update so that the primary will be updated, if a new one is elected => throw TRANSIENT exception!");
                CORBA::TRANSIENT ex(OrbSupport::TRANSIENT, CORBA::COMPLETED_NO);
                CDMW_LOG_FUNCTION_EXCEPTION(ex);
                throw ex;
            }
            else
            {
                if (last_object_group_version < request_object_group_version)
                {
                    // Request object group doesn't exist in the domain
                    CORBA::INV_OBJREF ex(OrbSupport::INV_OBJREFInvalidObjectGroupRefVersion,
                                         CORBA::COMPLETED_NO);
                    CDMW_LOG_FUNCTION_EXCEPTION(ex);
                    throw ex;
                }
                else
                {
                    // last_object_group_version > request_object_group_version
                    // A more recent group reference exist for that group,
                    // advise client with a location forward perm
                    try
                    {
                        CDMW_DEBUG(FTLogger::GetLogger(), 
                                   "A more recent group reference exist for that group, advise client with a location forward perm");
                        CdmwFT::ReplicationManager_var replication_manager
                            = m_group_repository->the_replication_manager();
                        ::FT::ObjectGroup_var last_object_group =
                            replication_manager->get_object_group_ref_from_gid(object_group_id);
                        
                        CDMW_DEBUG(FTLogger::GetLogger(), 
                                   "last_object_group_version > request_object_group_version, "
                                   << "group_version from the replication manager = " << last_object_group_version);

                        // throw a forward request permanent (bool = 1) with the local group reference
                        PortableInterceptor::ForwardRequest ex(last_object_group.in());
                        CDMW_LOG_FUNCTION_EXCEPTION(ex);
                        throw ex;
                    }
                    catch (const ::FT::ObjectGroupNotFound &)
                    {
                        CDMW_FATAL(FTLogger::GetLogger(),
                                   "FATAL ERROR: get_object_group_version_from_gid raises ObjectGroupNotFound exception ");
                        CDMW_ASSERT(false);
//                         CORBA::INTERNAL ex(OrbSupport::INTERNALFaultToleranceError, CORBA::COMPLETED_NO);
//                         CDMW_LOG_FUNCTION_EXCEPTION(ex);
//                         throw ex;
                    }
                    catch (const CORBA::SystemException& ex)
                    {
                        CDMW_ERROR(FTLogger::GetLogger(),
                                   "ERROR: System exception raised while getting the group reference version from the replication manager!\n"
                                   << ex);
                        CDMW_LOG_FUNCTION_EXCEPTION(ex);
                        throw;
                    }
                }
            }
        }
    }
}


::FT::FTRequestServiceContext*
ServerRequestInterceptor_impl::get_FTRequestServiceContext_from_slots(PortableInterceptor::ServerRequestInfo_ptr ri)
    throw (CORBA::INTERNAL)
{
    CDMW_LOG_FUNCTION(FTLogger::GetLogger(),"operation="<<ri->operation());
    ::FT::FTRequestServiceContext* result = 0;
    
    // Check if this is a FT request
    bool is_a_ft_request = false;

    try
    {
        CORBA::Any_var any = ri->get_slot(m_is_a_ft_request_slot_id);

        if (!(any >>= CORBA::Any::to_boolean(is_a_ft_request)))
        {
            CDMW_ERROR(FTLogger::GetLogger(),
                       "ERROR: unexpected data returned by get_slot" );
            CORBA::INTERNAL ex(OrbSupport::INTERNALFaultToleranceError, CORBA::COMPLETED_NO);
            CDMW_LOG_FUNCTION_EXCEPTION(ex);
            throw ex;
        }

        if (is_a_ft_request)
        {
            // this is not an FT request, get the context
            const char* client_id;
            CORBA::Long retention_id;
            TimeBase::TimeT expiration_time;
            // Get client_id
            any = ri->get_slot(m_client_id_slot_id);
            
            if (!(any.in() >>= client_id))
            {
                CDMW_ERROR(FTLogger::GetLogger(),
                           "ERROR: unexpected data returned by get_slot" );
                CORBA::INTERNAL ex(OrbSupport::INTERNALFaultToleranceError, CORBA::COMPLETED_NO);
                CDMW_LOG_FUNCTION_EXCEPTION(ex);
                throw ex;
            }
            ::FT::FTRequestServiceContext_var ctxt = new ::FT::FTRequestServiceContext();
            
            ctxt->client_id = client_id;

            // Get retention_id
            any = ri->get_slot(m_retention_id_slot_id);
            
            if (!(any.in() >>= retention_id))
            {
                CDMW_ERROR(FTLogger::GetLogger(),
                           "ERROR: unexpected data returned by get_slot" );
                CORBA::INTERNAL ex(OrbSupport::INTERNALFaultToleranceError, CORBA::COMPLETED_NO);
                CDMW_LOG_FUNCTION_EXCEPTION(ex);
                throw ex;
            }
            ctxt->retention_id = retention_id;

            // Get expiration time
            any = ri->get_slot(m_expiration_time_slot_id);
            
            if (!(any.in() >>= expiration_time))
            {
                CDMW_ERROR(FTLogger::GetLogger(),
                           "ERROR: unexpected data returned by get_slot" );
                CORBA::INTERNAL ex(OrbSupport::INTERNALFaultToleranceError, CORBA::COMPLETED_NO);
                CDMW_LOG_FUNCTION_EXCEPTION(ex);
                throw ex;
            }
            ctxt->expiration_time = expiration_time;
            result = ctxt._retn();
        }
    } catch (const PortableInterceptor::InvalidSlot& ) {
        // This slot that has not been allocated
        CDMW_ERROR(FTLogger::GetLogger(),
                   "ERROR: InvalidSlot exception raised!" );
        CORBA::INTERNAL ex(OrbSupport::INTERNALFaultToleranceError, CORBA::COMPLETED_NO);
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw ex;
    }
            
    return result;
}

const char*
ServerRequestInterceptor_impl::get_location() throw ()
{
    CDMW_LOG_FUNCTION(FTLogger::GetLogger(),"");
    try {
        Cdmw::OsSupport::OS::ProcessId pid = m_processId;
//         static CORBA::String_var My_Location = m_group_repository->the_slocation();
        std::stringstream pid_stream;
        pid_stream << pid;
        static CORBA::String_var My_Location = CORBA::string_dup(pid_stream.str().c_str());
        
        CDMW_LOG_FUNCTION_RETURN(My_Location.in());
        return My_Location.in();
    }
    catch (const CORBA::BAD_INV_ORDER & )
    {
        CDMW_FATAL(FTLogger::GetLogger(), 
                   "GroupRepository object not yet initialised!");
        CDMW_ASSERT(false);
        return 0; 
    }
}



::FT::FTRequestServiceContext*
ServerRequestInterceptor_impl::extract_FT_REQUEST_service_context(PortableInterceptor::ServerRequestInfo_ptr ri,
                                                              const char * location_str)
    throw (CORBA::SystemException)
{
   CDMW_LOG_FUNCTION(FTLogger::GetLogger(),"operation="<<ri->operation()<<",location_str="<<location_str);
    ::FT::FTRequestServiceContext* result = 0;

    try
    {
        IOP::ServiceContext_var sc 
            = ri->get_request_service_context(IOP::FT_REQUEST);
        CDMW_DEBUG(FTLogger::GetLogger(), "Context_id: FT_REQUEST");

        CORBA::OctetSeq data(sc->context_data.length());
        data.length(sc->context_data.length());
        for (unsigned int i=0; i < sc->context_data.length(); i++)
        {
           data[i] = sc->context_data[i];
        }
        //memcpy(data.get_buffer(), sc->context_data.get_buffer(),
        //    sc->context_data.length());
        CORBA::Any_var any =
           m_cdrCodec->decode_value(data,
                                    ::FT::_tc_FTRequestServiceContext);

        if (any.in() >>= result)
        {
           result = new ::FT::FTRequestServiceContext(*result);
           CDMW_DEBUG(FTLogger::GetLogger(),
                      "FTRequestServiceContext: "
                      << "\n\t        Client ID: " << result->client_id.in()
                      << "\n\t        Retention ID: " << result->retention_id
                      << "\n\t        Expiration Time: " << result->expiration_time);
        }      
        else
        {
           CDMW_ERROR(FTLogger::GetLogger(),
                      "ERROR: Error occured when reading FTRequestServiceContext");
           CORBA::BAD_CONTEXT ex(OrbSupport::BAD_CONTEXTCannotReadFTRequestServiceContext, CORBA::COMPLETED_NO);
           CDMW_LOG_FUNCTION_EXCEPTION(ex);
           throw ex;
        }   
    }
    catch(const CORBA::BAD_CONTEXT&)
    {
       throw; // Just rethrow here.
    }
    catch(const CORBA::BAD_PARAM&)
    {
        CDMW_DEBUG(FTLogger::GetLogger(),"No service context FT_REQUEST for that request");
        // This is not an error while a request can come to a single reference
        // with no FT service context.
    }
    catch (const CORBA::SystemException& ex)
    {
        CDMW_ERROR(FTLogger::GetLogger(),
                   "ERROR: extract_FT_REQUEST_service_context FT_REQUEST raises exception " << ex);
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw;
    }
    catch (const CORBA::Exception& ex)
    {
        CDMW_ERROR(FTLogger::GetLogger(),
                   "ERROR: extract_FT_REQUEST_service_context FT_REQUEST raises CORBA exception " << ex);
        CORBA::BAD_CONTEXT e(OrbSupport::BAD_CONTEXTCannotReadFTRequestServiceContext, CORBA::COMPLETED_NO);
        CDMW_LOG_FUNCTION_EXCEPTION(e);
        throw e;
    }
    catch (const std::exception & ex)
    {
        CDMW_ERROR(FTLogger::GetLogger(),
                   "ERROR: extract_FT_REQUEST_service_context raises standard exception " << ex.what());
        CORBA::INTERNAL e(OrbSupport::INTERNALFaultToleranceError, CORBA::COMPLETED_NO);
        CDMW_LOG_FUNCTION_EXCEPTION(e);
        throw e;
    }

    return result;
}



::FT::TagFTGroupTaggedComponent*
ServerRequestInterceptor_impl::extract_FT_GROUP_VERSION_service_context(PortableInterceptor::ServerRequestInfo_ptr ri,
                                                                        const char * location_str)
   throw (CORBA::SystemException)
{
   CDMW_LOG_FUNCTION(FTLogger::GetLogger(),"operation="<<ri->operation()<<",location_str="<<location_str);
   ::FT::TagFTGroupTaggedComponent* result = 0;

   try
   {
      IOP::ServiceContext_var sc 
         = ri->get_request_service_context(IOP::FT_GROUP_VERSION);
      
      CDMW_DEBUG(FTLogger::GetLogger(), "Context_id: FT_GROUP_VERSION");      
      CORBA::OctetSeq data(sc->context_data.length());
      data.length(sc->context_data.length());
      for (unsigned int i=0; i < sc->context_data.length(); i++)
      {
         data[i] = sc->context_data[i];
      } 

      //
      // The following code is right but TAO doesn't follow the standard!
      // It replaces the FTGroupVersionServiceContext by the TagFTGroupTaggedComponent.
      //
      //CORBA::Any_var any = m_cdrCodec->decode_value(data, FT::_tc_FTGroupVersionServiceContext);
      //
      //const FT::FTGroupVersionServiceContext* context;
      //if (any.in() >>= context)
      //{
      //    DEBUG_DUMP(location_str, "  FTGroupVersionServiceContext: ");
      //    DEBUG_ECHO("     Object group ref version: " << context->object_group_ref_version);
      //
      CORBA::Any_var any = m_cdrCodec->decode_value(data, ::FT::_tc_TagFTGroupTaggedComponent);
      if (any.in() >>= result)
      {
         result = new ::FT::TagFTGroupTaggedComponent(*result);
         CDMW_DEBUG(FTLogger::GetLogger(),
                    "FTGroupVersionServiceContext: "
                    << "\n\t        version = " 
                    << (int)result->component_version.major << "." 
                    << (int)result->component_version.minor
                    << "\n\t        ft_domain_id = " << result->group_domain_id.in()
                    << "\n\t        object_group_id = " << result->object_group_id
                    << "\n\t        object_group_ref_version = " << result->object_group_ref_version);       
      }
      else
      {
         CDMW_ERROR(FTLogger::GetLogger(),
                    "ERROR: Error occured when reading FTGroupVersionServiceContext");
         CORBA::BAD_CONTEXT ex(OrbSupport::BAD_CONTEXTCannotReadFTGroupVersionServiceContext, CORBA::COMPLETED_NO);
         CDMW_LOG_FUNCTION_EXCEPTION(ex);
         throw ex;
      }   


    }
    catch(const CORBA::BAD_CONTEXT&)
    {
       throw; // Just rethrow here.
    }
    catch(const CORBA::BAD_PARAM&)
    {
        //DEBUG_DUMP(location_str, "No service context FT_GROUP_VERSION for that request");

        // This is not an error while a request can come to a single reference.
        // with no FT service context.
    }
    catch (const CORBA::SystemException& ex)
    {
        CDMW_ERROR(FTLogger::GetLogger(),
                   "ERROR: get_request_service_context FT_GROUP_VERSION raises exception " << ex );
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw;
    }
    catch (const CORBA::Exception& ex)
    {
        CDMW_ERROR(FTLogger::GetLogger(),
                   "ERROR: get_request_service_context FT_GROUP_VERSION raises CORBA exception " << ex);
        CORBA::BAD_CONTEXT e(OrbSupport::BAD_CONTEXTCannotReadFTGroupVersionServiceContext, CORBA::COMPLETED_NO);
        CDMW_LOG_FUNCTION_EXCEPTION(e);
        throw e;
    }
    catch (const std::exception & ex)
    {
        CDMW_ERROR(FTLogger::GetLogger(),
                   "ERROR: get_request_service_context raises standard exception " << ex.what());
        CORBA::INTERNAL e(OrbSupport::INTERNALFaultToleranceError, CORBA::COMPLETED_NO);
        CDMW_LOG_FUNCTION_EXCEPTION(e);
        throw e;
    }
    return result;
}




bool
ServerRequestInterceptor_impl::check_incoming_request(PortableInterceptor::ServerRequestInfo_ptr ri,
                                                       ::FT::FTRequestServiceContext_out   ft_request_context)
    throw (CORBA::SystemException, PortableInterceptor::ForwardRequest)
{
    CDMW_LOG_FUNCTION(FTLogger::GetLogger(),"operation="<<ri->operation());
    bool result=false;
    const char* loc = get_location();  
    try {     
        ::FT::FTRequestServiceContext_var          ft_request_context__;
        ::FT::TagFTGroupTaggedComponent_var        ft_group_version_context__;
        if (result=get_service_contexts(ri,loc,
                                        ft_request_context__.out(),
                                        ft_group_version_context__.out()))
        {
            // This an FT request. Validate it!
            validate_request(ri,loc,
                             ft_request_context__.in(),
                             ft_group_version_context__.in());
            ft_request_context=ft_request_context__._retn();
            // Request is valid. There is no request logging so no handling 
            // of expiration time & request logging
        } 
        CDMW_LOG_FUNCTION_RETURN(result);
        return result;
    } catch (const PortableInterceptor::ForwardRequest & ex) {
        CDMW_DEBUG(FTLogger::GetLogger(),
                   "PortableServer::ForwardRequest exception raised in check_incoming_request!\n");
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw;
    } catch (const CORBA::SystemException& ex) {
        CDMW_DEBUG(FTLogger::GetLogger(),
                   "CORBA System exception raised in check_incoming_request!\n" << ex);
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw;
    }
 }


} // End of namespace FTInit

} // End of namespace FT

} // End of namespace Cdmw
