/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO 
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2009. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#include <iostream>
#include <algorithm>

// Cdmw Files
#include <Foundation/common/Assert.hpp> 
#include <Foundation/orbsupport/CORBA.hpp>
#include <Foundation/commonsvcs/naming/NamingInterface.hpp>
#include <FaultTolerance/ftlocationmanager/StatefullPrimaryBackupAdmin_impl.hpp>
#include <FaultTolerance/ftcommon/MacroDef.hpp>
#include <FaultTolerance/ftstatemanager/DataStore.hpp>
#include <FaultTolerance/ftstatemanager/DataStoreContext.hpp>
#include <Foundation/commonsvcs/naming/NamingInterface.hpp>

#include <FaultTolerance/ftcommon/FTLogger.hpp>
#include <Foundation/logging/FunctionLogger.hpp>


//
// IDL:thalesgroup.com/CdmwFT:1.0
//
namespace Cdmw
{

namespace FT
{

    //
    // IDL:thalesgroup.com/CdmwFT/Location:1.0
    //
    namespace Location
    {

        //
        // IDL:thalesgroup.com/CdmwFT/Location/StatefullPrimaryBackupAdmin:1.0
        //
        StatefullPrimaryBackupAdmin_impl::StatefullPrimaryBackupAdmin_impl
        (CORBA::ORB_ptr orb,
         PortableServer::POA_ptr poa,
         const ::FT::Location& the_location,
         PrimaryBackupGroupRepository_impl*  group_repository,
         bool cold_restart,
         bool is_ftmanager)
            : PrimaryBackupAdmin_impl(orb, poa, the_location, group_repository, is_ftmanager),
              m_poa(PortableServer::POA::_duplicate(poa)),
              m_primary_backup_group_repository(group_repository),
              m_is_ftmanager(is_ftmanager),
              m_the_location(the_location)
        {
            CDMW_LOG_FUNCTION(FTLogger::GetLogger(),"cold_restart="<<cold_restart<<",is_ftmanager="<<is_ftmanager);
            m_init_type = cold_restart;
        }

        StatefullPrimaryBackupAdmin_impl::~StatefullPrimaryBackupAdmin_impl()
        {
            CDMW_LOG_FUNCTION(FTLogger::GetLogger(),"");
        }

    
        //
        // IDL:thalesgroup.com/CdmwFT/Location/StatefullPrimaryBackupAdmin/date_statefull_group_view:1.0
        //
        void
        StatefullPrimaryBackupAdmin_impl::update_statefull_group_view
        (const CdmwFT::Location::StatefullGroupInfos& statefull_group_infos)
            throw(::FT::ObjectGroupNotFound, 
                  CdmwFT::Location::UnknownDataStoreID,
                  CdmwFT::StateTransfer::InvalidDataStoreState,
                  CORBA::SystemException)
        {
            CDMW_LOG_FUNCTION(FTLogger::GetLogger(),"");
            try
            {
                update_from_group_info(statefull_group_infos);

                // for each statefullGroupInfos
                CORBA::ULong statefull_group_infos_length = statefull_group_infos.length();
                for (CORBA::ULong i = 0; i<statefull_group_infos_length; i++)
                {
                    // call update_group_view on the PrimaryBackupAdmin
                    update_group_view(statefull_group_infos[i].group_info);
                }
            }
            catch( const CORBA::Exception & ex )
            {
                CDMW_LOG_FUNCTION_EXCEPTION(ex);
                throw;
            }
        }
        
        
        //
        // IDL:thalesgroup.com/CdmwFT/Location/StatefullPrimaryBackupAdmin/update_statefull_primary:1.0
        //
        void
        StatefullPrimaryBackupAdmin_impl::update_statefull_primary
        (const CdmwFT::Location::StatefullGroupInfos&  statefull_group_infos,
         const ::FT::Location& primary_location)
            throw(::FT::ObjectGroupNotFound,
                  CdmwFT::Location::UnknownDataStoreID,
                  CdmwFT::StateTransfer::InvalidDataStoreState,
                  CORBA::SystemException)
        {
            CDMW_LOG_FUNCTION(FTLogger::GetLogger(),"");
            try
            {
                
                // check if the datastore must be activate or deactivate
                std::string new_primary = Cdmw::CommonSvcs::Naming::NamingInterface::to_string(primary_location);

                // NOTA: This operation is called once for all the groups where this location
                //       (m_location) has a member. This is called whenever a primary has been 
                //       elected.
                // 1. Set coordinator location on all datastore contexts involved.
                // 2. Call contexts.deactivate() if m_location lost primary role
                // 3. Call contexts.activate() if m_location is promoted to primary
                
                for (DataStoreContextSet::iterator iter = m_contexts.begin();
                     iter != m_contexts.end();
                     iter ++)
                {
                    CDMW_DEBUG(FTLogger::GetLogger(),
                               "Set coordinator location to '"<< new_primary << "'");
                    DataStoreContext * context__ = *iter;
                    context__->set_coordinator_location(new_primary);

                    if (new_primary != m_location && 
                        m_primary_location == m_location)
                    {
                       // realise the deactivate
                        CDMW_DEBUG(FTLogger::GetLogger(),
                                   "Deactivating DataStoreContext");
                        context__->deactivate();
                    }
                    else
                    {
                       if (m_primary_location != m_location && 
                           new_primary == m_location)
                       {
                          // realise the activate
                          CDMW_DEBUG(FTLogger::GetLogger(),
                                     "Activating DataStoreContext");
                          context__->activate();
                       }
                       else
                       {
                          // do nothing : the new primary is not for this location
                       }
                    }
                }
                update_from_group_info(statefull_group_infos, true);
                
                // for each statefullGroupInfos
                CORBA::ULong statefull_group_infos_length = statefull_group_infos.length();
                for (CORBA::ULong i = 0; i<statefull_group_infos_length; i++)
                {
                    // call update_primary on the PrimaryBackupAdmin
                    update_primary(statefull_group_infos[i].group_info, primary_location);
                }
            }
            catch(const CORBA::Exception& ex)
            {
                CDMW_LOG_FUNCTION_EXCEPTION(ex);
                throw;
            }
        }
        
            
        //
        // IDL:thalesgroup.com/CdmwFT/Location/StatefullPrimaryBackupAdmin/init_backup_data_stores:1.0
        //
        void
        StatefullPrimaryBackupAdmin_impl::init_backup_data_stores
        (const CdmwFT::StateTransfer::DataStoreInfo&
         primary_data_store_infos)
            throw(CdmwFT::Location::InitFailed,
                  CdmwFT::Location::AlreadyDone,
                  CORBA::SystemException)
        {
            CDMW_LOG_FUNCTION(FTLogger::GetLogger(),"");
            try
            {
               // NOTA: This is called for each object group containing datastores.
               //       Initialisation is now done at DataStoreContext level and not per datastore.
               // NOTA: We will keep track of initialised datastores while the FTManager is not
               //       enhanced to initialise all datastores of a process once to minimise
               //       impact on existing tests and check that datastores are properly defined
               //       at object group level.
                CdmwFT::StateTransfer::LocalDataStoreInfos 
                   primary_local_data_store_infos(primary_data_store_infos.local_data_store_infos);
                
                // For each datastore
                CORBA::ULong primary_local_data_store_infos_length =
                    primary_local_data_store_infos.length();
                CORBA::ULong total_datastores = 0;
                CORBA::ULong index = 0;

                for (CORBA::ULong i = 0; i<primary_local_data_store_infos_length; i++) 
                {
                    total_datastores += primary_local_data_store_infos[i].dsids.length();

                    for (CORBA::ULong j = 0;
                         j < primary_local_data_store_infos[i].dsids.length();
                         j++)
                    {
                        // check if all datastore are not initialised
                        DataStoreIDSet::iterator id_iter =
                            std::find(m_init_backup_datastore_id.begin(),
                                      m_init_backup_datastore_id.end(),
                                      primary_local_data_store_infos[i].dsids[j]); 

                        // if the datastore is not initialise
                        if (id_iter == m_init_backup_datastore_id.end())
                        {
                            // increase the index
                            index++;
                        }
                        // get the reference of the datastore for the dsid
                        Cdmw::FT::DataStoreBase* dataStore =
                            Cdmw::FT::DataStoreBase::Get_dataStore
                                (primary_local_data_store_infos[i].dsids[j]);
                        if (dataStore == 0)
                        {
                            std::ostringstream stream;
                            stream << MembersAdmin_impl::get_the_location()
                                   <<" datastore reference for dsid "
                                   << primary_local_data_store_infos[i].dsids[j]
                                   <<" is NULL";
                            CDMW_ERROR(FTLogger::GetLogger(),
                                       "Throw CdmwFT::Location::InitFailed : "
                                       << stream.str().c_str() );
                            CdmwFT::Location::InitFailed ex(stream.str().c_str());
                            CDMW_LOG_FUNCTION_EXCEPTION(ex);
                            throw ex;
                        }
                    }
                }

                //if all datastore must be initialise
                if (index == total_datastores)
                {
                    // For each datastore, 
                    for (CORBA::ULong i = 0;
                         i < primary_local_data_store_infos_length;
                         i++) 
                    {   
                        // get the reference of the datastore for the dsid
                        Cdmw::FT::DataStoreBase* dataStore =
                            Cdmw::FT::DataStoreBase::Get_dataStore
                            (primary_local_data_store_infos[i].dsids[0]);

                        CDMW_ASSERT(dataStore != 0);

                        DataStoreContext * context__ = &dataStore->get_context();

                        // 
                        DataStoreContextSet::iterator ctxt_iter = m_contexts.find(context__);
                        if (ctxt_iter == m_contexts.end())
                        {
                           // Context not yet initialised
                           const std::string primary_location = 
                              Cdmw::CommonSvcs::Naming::NamingInterface::to_string(
                                 primary_data_store_infos.the_location);
                           // call the method on the data store to update
                           // the state with the primary
                           context__->update_with_coordinator
                              (primary_local_data_store_infos[i].local_data_store.in(),
                               primary_location,
                               MembersAdmin_impl::get_the_location(),
                               primary_local_data_store_infos[i].dsids);
                           
                           m_contexts.insert(context__);
                        }


                        // dataStore->get_context().set_coordinator_location(primary_location);

                        for (CORBA::ULong j = 0;
                             j < primary_local_data_store_infos[i].dsids.length();
                             j++)
                        {
                            CDMW_DEBUG(FTLogger::GetLogger(),
                                       " intialised the datastore :"
                                       << primary_local_data_store_infos[i].dsids[j]);

                            // save the id of the datastore into the set
                            // of datastore initialised
                            m_init_backup_datastore_id.insert
                                (m_init_backup_datastore_id.end(),
                                 primary_local_data_store_infos[i].dsids[j]);
                        }
                        // Enable 'SLAVE' mode as DataStoreContext state is initialised as neither MASTER nor SLAVE !
                        // dataStore->get_context().deactivate();
                    }           
                } 
                else
                {
                    if (index == 0)
                    {
                        // all the datastore are already initialised
                        CDMW_ERROR(FTLogger::GetLogger(), "throw CdmwFT::Location::AlreadyDone()");
                        CdmwFT::Location::AlreadyDone ex;
                        CDMW_LOG_FUNCTION_EXCEPTION(ex);
                        throw ex;
                    } 
                    else
                    {    
                        // some datastore are initialised and some are not => insubtantial state
                        CDMW_ERROR(FTLogger::GetLogger(), "throw CdmwFT::Location::InitFailed : some datastore are initialised and some are not => insubstantial state");
                        CdmwFT::Location::InitFailed ex("some datastore are initialised and some are not => insubstantial state");
                        CDMW_LOG_FUNCTION_EXCEPTION(ex);
                        throw ex;
                    }
                }
            }            
            catch(const CdmwFT::Location::InitFailed& ex)
            {
                CDMW_LOG_FUNCTION_EXCEPTION(ex);
                throw;
            }
            catch(const CdmwFT::Location::AlreadyDone& ex)
            {
                CDMW_LOG_FUNCTION_EXCEPTION(ex);
                throw;                
            }
            catch(const Cdmw::InitErrorException&)
            {
                CdmwFT::Location::InitFailed ex("InitErrorException");
                CDMW_LOG_FUNCTION_EXCEPTION(ex);
                throw ex;
            }
            catch(const CORBA::SystemException &)
            {
                CdmwFT::Location::InitFailed ex;
                CDMW_LOG_FUNCTION_EXCEPTION(ex);
                throw ex;
            }
            

        }
        
          
        //
        // IDL:thalesgroup.com/CdmwFT/Location/StatefullPrimaryBackupAdmin/init_primary_data_stores:1.0
        //
        void 
        StatefullPrimaryBackupAdmin_impl::init_primary_data_stores
        (const CdmwFT::StateTransfer::LocalDataStoreInfos& local_datastore_infos)
            throw(CdmwFT::Location::InitFailed,
                  CdmwFT::Location::AlreadyDone,
                  CORBA::SystemException)
        {
            CDMW_LOG_FUNCTION(FTLogger::GetLogger(),"");
            try
            {
               // NOTA: This is called for each object group containing datastores.
               //       Initialisation is now done at DataStoreContext level and not per datastore.
               // NOTA: We will keep track of initialised datastores while the FTManager is not
               //       enhanced to initialise all datastores of a process once to minimise
               //       impact on existing tests and check that datastores are properly defined
               //       at object group level.

                // For each datastore
                CORBA::ULong datastore_info_length = local_datastore_infos.length();
                CORBA::ULong total_datastores = 0;
                CORBA::ULong index = 0;

                for (CORBA::ULong i = 0; i < datastore_info_length; i++) 
                {
                    total_datastores += local_datastore_infos[i].dsids.length();

                    for (CORBA::ULong j = 0;
                         j < local_datastore_infos[i].dsids.length();
                         j++)
                    {
                        // check if all datastore are not initialised
                        DataStoreIDSet::iterator id_iter =
                            std::find(m_init_primary_datastore_id.begin(),
                                      m_init_primary_datastore_id.end(),
                                      local_datastore_infos[i].dsids[j]); 

                        // if the datastore is not initialise
                        if (id_iter == m_init_primary_datastore_id.end())
                        {
                            // increase the index
                            index++;
                        }
                        // get the reference of the datastore for the dsid
                        Cdmw::FT::DataStoreBase* dataStore =
                            Cdmw::FT::DataStoreBase::Get_dataStore
                                (local_datastore_infos[i].dsids[j]);

                        if (dataStore == 0)
                        {
                            std::ostringstream stream;
                            stream << MembersAdmin_impl::get_the_location()
                                   <<" datastore reference for dsid "
                                   << local_datastore_infos[i].dsids[j]
                                   <<" is NULL";
                            CDMW_ERROR(FTLogger::GetLogger(),
                                       "Throw CdmwFT::Location::InitFailed : "
                                       << stream.str().c_str());
                            CdmwFT::Location::InitFailed ex(stream.str().c_str());
                            CDMW_LOG_FUNCTION_EXCEPTION(ex);
                            throw ex;
                        }
                    }
                }

                if (index == total_datastores)
                {
                    // For each datastore, 
                    for (CORBA::ULong i = 0; i<datastore_info_length; i++) 
                    {   
                        for (CORBA::ULong j = 0;
                             j < local_datastore_infos[i].dsids.length();
                             j++)
                        {
                            CDMW_DEBUG(FTLogger::GetLogger(),
                                       "intialise the datastore :"
                                       << local_datastore_infos[i].dsids[j]);

                            // get the reference of the datastore for the dsid
                            Cdmw::FT::DataStoreBase* dataStore =
                                Cdmw::FT::DataStoreBase::Get_dataStore
                                (local_datastore_infos[i].dsids[j]);

                            CDMW_ASSERT(dataStore != 0);

                            DataStoreContext * context__ = &dataStore->get_context(); 

                            DataStoreContextSet::iterator ctxt_iter = m_contexts.find(context__);
                            if (ctxt_iter == m_contexts.end())
                            {
                               // Init all datastores within the DataStoreContext  at once
                               DataStoreContext::DataStoreIdList idlist = context__->get_datastore_ids();
                               for (DataStoreContext::DataStoreIdList::iterator dsiter = idlist.begin();
                                    dsiter != idlist.end();
                                    ++dsiter) 
                               {
                                  DataStoreBase * ds = context__->get_datastore(*dsiter);
                                  CDMW_ASSERT(ds != 0);
                                  // call init on the datastore
                                  ds->init(m_init_type);
                               }
                               context__->activate();
                               m_contexts.insert(context__);
                            }


                        
                            // save the id of the datastore into the
                            // set of datastore initialised
                            m_init_primary_datastore_id.insert
                                (m_init_primary_datastore_id.end(),
                                 local_datastore_infos[i].dsids[j]);
                            m_primary_location =
                                MembersAdmin_impl::get_the_location();
                        }
                    }
                } 
                else
                {
                    if (index == 0)
                    {
                        // all the datastore are already initialised
                        CDMW_ERROR(FTLogger::GetLogger(), "throw CdmwFT::Location::AlreadyDone()");
                        CdmwFT::Location::AlreadyDone ex;
                        CDMW_LOG_FUNCTION_EXCEPTION(ex);
                        throw ex;
                    } 
                    else
                    {    
                        // some datastore are initialised and some are not => insubtantial state
                        CDMW_ERROR(FTLogger::GetLogger(), "throw CdmwFT::Location::InitFailed : some datastore are initialised and some are not => insubtantial state");
                        CdmwFT::Location::InitFailed ex("some datastore are initialised and some are not => insubtantial state");
                        CDMW_LOG_FUNCTION_EXCEPTION(ex);
                        throw ex;
                    }
                }
            }            
            catch(const CdmwFT::Location::InitFailed& ex)
            {
                CDMW_LOG_FUNCTION_EXCEPTION(ex);
                throw;
            }
            catch(const CdmwFT::Location::AlreadyDone& ex)
            {
                CDMW_LOG_FUNCTION_EXCEPTION(ex);
                throw;                
            }
            catch(const Cdmw::InitErrorException&)
            {
                CdmwFT::Location::InitFailed ex("InitErrorException");
                CDMW_LOG_FUNCTION_EXCEPTION(ex);
                throw ex;
            }
            catch(const CORBA::SystemException &)
            {
                CdmwFT::Location::InitFailed ex;
                CDMW_LOG_FUNCTION_EXCEPTION(ex);
                throw ex;
            }
        }
        
         
        //
        // IDL:thalesgroup.com/CdmwFT/Location/StatefullPrimaryBackupAdmin/handle_backup_insertion_failure:1.0
        //
        void StatefullPrimaryBackupAdmin_impl::handle_backup_insertion_failure
        (const ::FT::Location& failed_location)
            throw(CORBA::SystemException)
        {
            CDMW_LOG_FUNCTION(FTLogger::GetLogger(),"");
            Cdmw::FT::DataStoreBase::On_Failure(Cdmw::CommonSvcs::Naming::NamingInterface::to_string(failed_location));
        }
        

       void StatefullPrimaryBackupAdmin_impl::update_from_group_info(const CdmwFT::Location::StatefullGroupInfos&  statefull_group_infos,
                                                                     bool is_update_primary)
          throw(::FT::ObjectGroupNotFound,
                CdmwFT::Location::UnknownDataStoreID,
                CdmwFT::StateTransfer::InvalidDataStoreState,
                CORBA::SystemException)
       {       
          CDMW_LOG_FUNCTION(FTLogger::GetLogger(),"is_update_primary="<<is_update_primary);
          // clear all the map used to realise the on failure and the activate on the new data store or on the remove data store
          
                    
          AddLocationDataMap    add_location_map;
          ContextLocationSetMap context_location_set_map;
          
          // for each statefullGroupInfos
          CORBA::ULong statefull_group_infos_length = statefull_group_infos.length();
          for (CORBA::ULong i = 0; i<statefull_group_infos_length; i++)
          {
             // for each statefull_group_infos[i].data_store_infos
             CORBA::ULong data_store_infos_length = statefull_group_infos[i].data_store_infos.length();
             for (CORBA::ULong j = 0; j<data_store_infos_length; j++)
             {
                //for each statefull_group_infos[i].data_store_infos[j].local_data_store_infos
                CORBA::ULong local_data_store_infos_length = statefull_group_infos[i].data_store_infos[j].local_data_store_infos.length();
                for (CORBA::ULong k = 0; k<local_data_store_infos_length; k++)
                {
                   const CdmwFT::StateTransfer::LocalDataStoreInfo & ldsi = 
                      statefull_group_infos[i].data_store_infos[j].local_data_store_infos[k];
                   for (CORBA::ULong m = 0; m < ldsi.dsids.length(); m++)
                   {
                      CDMW_DEBUG(FTLogger::GetLogger(),
                                 "get_datastore reference for the dsid :"
                                 << ldsi.dsids[m]);
                      // check if the data store is known
                      Cdmw::FT::DataStoreBase* dataStore 
                         = Cdmw::FT::DataStoreBase::Get_dataStore(ldsi.dsids[m]);
                      if (dataStore == 0)
                      {
                         CDMW_ERROR(FTLogger::GetLogger(),"throw CdmwFT::Location::UnknownDataStoreID");
                         CdmwFT::Location::UnknownDataStoreID ex;
                         CDMW_LOG_FUNCTION_EXCEPTION(ex);
                         throw ex;
                      }
                      
                      // check into the m_nominal_location if the dsid is known
                      // if the dsid is known, the data store was initialised else the data store
                      // is not initialise and the state is in insubtantial state
                      DataStoreContext* context__ = & dataStore->get_context();
                      DataStoreContextSet::iterator ctxtiter = m_contexts.find(context__);
                      if (ctxtiter == m_contexts.end())
                      {
                         CDMW_ERROR(FTLogger::GetLogger(),
                                    " throw InvalidDataStoreState exception");
                         CdmwFT::StateTransfer::InvalidDataStoreState ex;
                         CDMW_LOG_FUNCTION_EXCEPTION(ex);
                         throw ex;
                      }
                      else 
                      {
                         std::string location = 
                            Cdmw::CommonSvcs::Naming::NamingInterface::to_string(statefull_group_infos[i].data_store_infos[j].the_location);
                         CDMW_DEBUG(FTLogger::GetLogger(),
                                    " save the location "<<location<<" into a temporally map");
                         ContextLocationSetMap::iterator tmp_ctxt_locs_iter = 
                            context_location_set_map.find(context__);
                         if (tmp_ctxt_locs_iter == context_location_set_map.end())
                         {
                            LocationSet location_set;
                            location_set.insert(location);
                            context_location_set_map.insert
                               (make_pair(context__, location_set));
                         } else {
                            (*tmp_ctxt_locs_iter).second.insert(location);
                         }
                         // Save TPC data for later in per context/location manner
                         
                         AddLocationKey key(context__,location);
                         TPCData data(ldsi.coordinator.in(),
                                      ldsi.cohort.in());
                         add_location_map[key]=data;
                      }
                   }
                }
             }
          }


          // Build remove and add location map
          //  .  add_location_map contains all <context,location>:TPCData
          //  .  context_location_set_map contains <context>:<location set> received
          //     from the FTManager (updated members)
          for (ContextLocationSetMap::iterator ctxt_locset_iter = context_location_set_map.begin();
               ctxt_locset_iter != context_location_set_map.end();
               ++ctxt_locset_iter)
          {
             LocationSet addlocationset, removelocationset;
             // create two insert_iterator
             std::insert_iterator<LocationSet > add_ins(addlocationset, addlocationset.begin());
             std::insert_iterator<LocationSet > remove_ins(removelocationset, removelocationset.begin());
             // to realise the set of the removed member, take the set of the nominal member and realise the
             // difference with the set of the updated member
             DataStoreContext* context__ = (*ctxt_locset_iter).first;

             LocationSet nominal_LocationSet = context__->get_locations();
             LocationSet& updated_LocationSet = (*ctxt_locset_iter).second;

             set_difference(nominal_LocationSet.begin(), nominal_LocationSet.end(),
                            updated_LocationSet.begin(), updated_LocationSet.end(),
                            remove_ins);

             // to realise the set of the added member, take the set of the upated member and realise the
             // difference with the set of the nominal member
             
             set_difference(updated_LocationSet.begin(), updated_LocationSet.end(),
                            nominal_LocationSet.begin(), nominal_LocationSet.end(),
                            add_ins);
             // realise a on failure for all the failed location on all the data store
             for (LocationSet::iterator iter = removelocationset.begin();
                  iter != removelocationset.end();
                  iter++)
             {
                CDMW_INFO(FTLogger::GetLogger(),
                          "Remove location => on_failure ("<<(*iter)<<")");
                context__->on_failure(*iter);
             }
             // realise the add_location on all the data store
             // For the new backup, realise add_location for all the member, except himself
             // For the old backup, realise an add_location with the new backup location
             for (LocationSet::iterator iter = addlocationset.begin();
                  iter != addlocationset.end();
                  iter++)
             {
                std::string location = (*iter);
                // if the location to add is the primary location or the local location, do not add it
                if (m_location != m_primary_location && 
                    location != m_location)
                {
                   CDMW_DEBUG(FTLogger::GetLogger(),
                              " adding location '" << location << "'");
                   try {
                      context__->add_location( location );
                      AddLocationKey key(context__,location);
                      if (!CORBA::is_nil( add_location_map[key].m_coordinator.in()) )
                      {
                         CDMW_DEBUG(FTLogger::GetLogger(),
                                    " add IIOP location '"<< location <<"'");
                         context__->add_ref_for_location
                            (location,
                             add_location_map[key].m_coordinator.in(),
                             add_location_map[key].m_cohort.in());
                      }
                   }
                   catch(const Cdmw::FT::AlreadyExistException& e)
                   {
                      CDMW_ERROR(FTLogger::GetLogger(),
                                 " update_from_group_info raises exception AlreadyExistException "
                                 << e <<" for location  "<<location);
                      CdmwFT::StateTransfer::InvalidDataStoreState ex;
                      CDMW_LOG_FUNCTION_EXCEPTION(ex);
                      throw ex;
                   }
                   catch(const Cdmw::FT::NotFoundException& e)
                   {
                      CDMW_ERROR(FTLogger::GetLogger(),
                                 " update_from_group_info raises exception NotFoundException "
                                 << e <<" for location  "<<location);
                      
                      CdmwFT::StateTransfer::InvalidDataStoreState ex;
                      CDMW_LOG_FUNCTION_EXCEPTION(ex);
                      throw ex;
                   }
                   
                }

             }
          }
       }
    } // End of namespace Location

} // End of namespace FT

} // End of namespace Cdmw

