/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO 
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2009. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#ifndef _CDMW_FT_REPLICATION_PROPERTY_MANAGER_IMPL_HPP_
#define _CDMW_FT_REPLICATION_PROPERTY_MANAGER_IMPL_HPP_

#include "Foundation/osthreads/Mutex.hpp"
#include <Foundation/orbsupport/RefCountServantBase.hpp>
#include <FaultTolerance/ftcommon/IOGRFactory.hpp>
#include "FaultTolerance/ftstatemanager/TXDataStore.hpp"
#include "Foundation/commonsvcs/datastore/StorageHome.hpp"
#include "Foundation/commonsvcs/datastore/StorageObject.hpp"
#include "ftreplicationmanager/ObjectGroup.hpp"
#include "ftreplicationmanager/Location.hpp"
#include <map>
#include <vector>
#include <FaultTolerance/ftcommon/FTManagerDSID.hpp>
#include "FT.skel.hpp"

DEFINE_VARIABLE_LENGTH_DATA_TRAITS(::FT::Properties);



namespace Cdmw {
namespace FT {
namespace ReplicationManager {


    /**
     * Actual FT::PropertyManager implementation class.
     */
    class PropertyManager_impl
        : public virtual POA_FT::PropertyManager,
          public virtual Cdmw::OrbSupport::RefCountServantBase

    {


        friend class OGDObserver;
        friend class LocInfoObserver;
        

    public:
        /**
         * Creates a property manager.
         *
         * @param ft_domain_id The fault tolerance domain id.
         */
        PropertyManager_impl(CORBA::ORB_ptr orb, 
                             PortableServer::POA_ptr poa)
            throw(CORBA::SystemException);
        
        /**
         * Destroys a property manager.
         */
        ~PropertyManager_impl(void) throw ();
        
        /**
         * IDL:omg.org/FT/PropertyManager/set_default_properties:1.0
         *
         * Sets the default properties for all object groups that are to be
         * created within the fault tolerance domain.
         *
         * @param props The properties to be set for all newly created
         *              object groups within the fault tolerance domain.
         *
         * @exception InvalidProperty if one or more of the properties in the
         *         sequence is not valid.
         * @exception UnsupportedProperty if one or more of the properties in
         *         the sequence is not supported.
         */
        virtual void
        set_default_properties(const ::FT::Properties & props)
            throw(::FT::InvalidProperty,
                  ::FT::UnsupportedProperty,
                  CORBA::SystemException);
        
        /**
         * IDL:omg.org/FT/PropertyManager/get_default_properties:1.0
         *
         * Returns the default properties for the object groups within the
         * fault tolerance domain.
         *
         * @return the default properties that have been set for the object
         *         groups.
         */
        virtual ::FT::Properties *
        get_default_properties(void)
            throw(CORBA::SystemException);
        
        /**
         * IDL:omg.org/FT/PropertyManager/remove_default_properties:1.0
         *
         * Removes the given default properties.
         *
         * @param props The properties to be removed.
         *
         * @exception InvalidProperty if one or more of the properties in the
         *         sequence is not valid.
         * @exception UnsupportedProperty if one or more of the properties in
         *         the sequence is not supported.
         */
        virtual void
        remove_default_properties(const ::FT::Properties & props)
            throw(::FT::InvalidProperty,
                  ::FT::UnsupportedProperty,
                  CORBA::SystemException);
        
        /**
         * IDL:omg.org/FT/PropertyManager/set_type_properties:1.0
         *
         * Sets the properties that override the default properties of the
         * object groups, with the given type identifier, that are created
         * in the future.
         *
         * @param type_id   The repository id for which the properties, that
         *                  are to override the existing properties, are set.
         * @param overrides The overriding properties.
         *
         * @exception InvalidProperty if one or more of the properties in the
         *         sequence is not valid.
         * @exception UnsupportedProperty if one or more of the properties in
         *         the sequence is not supported.
         */
        virtual void
        set_type_properties(const char * type_id,
                            const ::FT::Properties & overrides)
            throw(::FT::InvalidProperty,
                  ::FT::UnsupportedProperty,
                  CORBA::SystemException);
        
        /**
         * IDL:omg.org/FT/PropertyManager/get_type_properties:1.0
         *
         * Returns the properties of the object groups, with the given type
         * identifier, that are created in the future. These properties
         * include the properties determined by <code>set_type_properties()
         * </code>, as well as the default properties that are not overridden
         * by <code>set_type_properties</code>.
         *
         * @param type_id The repository id for which the properties, that
         *                are to override the existing properties, are set.
         *
         * @return the effective properties for the given type identifier.
         */
        virtual ::FT::Properties *
        get_type_properties(const char * type_id)
            throw(CORBA::SystemException);
        
        /**
         * IDL:omg.org/FT/PropertyManager/remove_type_properties:1.0
         *
         * Removes the given properties, with the given type identifier.
         *
         * @param type_id The repository id for which the given properties
         *                are to be removed.
         * @param props   The properties to be removed.
         *
         * @exception InvalidProperty if one or more of the properties in the
         *         sequence is not valid.
         * @exception UnsupportedProperty if one or more of the properties in
         *         the sequence is not supported.
         */
        virtual void
        remove_type_properties(const char * type_id,
                               const ::FT::Properties & props)
            throw(::FT::InvalidProperty,
                  ::FT::UnsupportedProperty,
                  CORBA::SystemException);

        /**
         * IDL:omg.org/FT/PropertyManager/set_properties_dynamically:1.0
         *
         * Sets the properties for the object group with the given reference
         * dynamically while the application executes. The properties given
         * as a parameter override the properties for the object when it was
         * created which, in turn, override the properties for the given type
         * which, in turn, override the default properties.
         *
         * @param object_group The reference of the object group for which
         *                     the overriding properties are set.
         * @param overrides    The overriding properties.
         *
         * @exception ObjectGroupNotFound if the object group is not found with
         *         the given reference.
         * @exception InvalidProperty if one or more of the properties in the
         *         sequence is not valid.
         * @exception UnsupportedProperty if one or more of the properties in
         *         the sequence is not supported.
         */
        virtual void
        set_properties_dynamically(::FT::ObjectGroup_ptr object_group,
                                   const ::FT::Properties & overrides)
            throw(::FT::ObjectGroupNotFound,
                  ::FT::InvalidProperty,
                  ::FT::UnsupportedProperty,
                  CORBA::SystemException);

        /**
         * IDL:omg.org/FT/PropertyManager/get_properties:1.0
         *
         * Returns the current properties of the given object group. These
         * properties include those that are set dynamically, those that are
         * set when the object group was created but are not overridden by
         * <code>set_properties_dynamically()</code>, those that are set as
         * properties of a type but are not overridden by <code>
         * create_object()</code> and <code>set_properties_dyamically()
         * </code>, and those that are set as defaults but are not overridden
         * by <code>set_type_properties()</code>, <code>create_object()
         * </code>, and <code>set_properties_dyamically()</code>.
         *
         * @param object_group The reference of the object group for which
         *                     the properties are to be returned.
         *
         * @return the set of current properties for the object group with
         *         the given reference.
         *
         * @exception ObjectGroupNotFound if the object group is not found by
         *         the Replication Manager.
         */
        virtual ::FT::Properties *
        get_properties(::FT::ObjectGroup_ptr object_group)
            throw(::FT::ObjectGroupNotFound,
                  CORBA::SystemException);
        

        
    public:
        // Maps a property name to a Property object.
        typedef std::map< std::string,
                          ::FT::Property >          PropertiesMap;
        

        // Defines a list of strings.
        typedef std::vector<std::string> StringList;

    private:
        // The ORB pseudo-object.
        CORBA::ORB_var m_orb;
        
        // The parent POA for fallback IORs.
        PortableServer::POA_var m_poa;
        
        /**
         * The mutex used for changing the functioning mode.
         */
        OsSupport::Mutex m_mutex;
        
        // The list of Fault Tolerance Properties as stated in the
        // Fault Tolerant CORBA specifications.
        StringList m_ft_properties;
        
       
        //Cdmw::FT::DataStoreContext* m_context;
        PropertiesStorageHome*    m_propertiesStorageHome;
        OGDStorageHome*           m_ogdStorageHome;

    protected:
        /**
         * Checks if the given properties are valid and supported.
         *
         * @param props       The properties to check.
         * @param type        The type of properties, type can be either
         *                    <code>DEFAULT_PROPERTIES</code> or
         *                    <code>TYPE_PROPERTIES</code> or
         *                    <code>GROUP_PROPERTIES</code>
         * @param check_value If false then bypass the value checking.
         *
         * @throw InvalidProperty if one or more of the properties in the
         *        sequence is not valid.
         * @throw UnsupportedProperty if one or more of the properties in
         *        the sequence is not supported.
         */
        void check_properties(const ::FT::Properties & props,
                              int type,
                              bool check_value = true)
            throw(::FT::InvalidProperty,
                  ::FT::UnsupportedProperty);


        /**
         * Returns the object groupid of the object group.
         *
         * @param object_group the object group reference.
         *
         * @return an ObjectGroupId
         *
         * @exception ObjectGroupNotFound if the object group with
         *   the given reference is not found 
         */
        ::FT::ObjectGroupId
        get_object_group_id_from_ref ( ::FT::ObjectGroup_ptr object_group ) const
            throw(::FT::ObjectGroupNotFound,
                  CORBA::SystemException);

    };

}; // namespace PropertyManager
}; // namespace FT
}; // namespace Cdmw

#endif // _CDMW_FT_REPLICATION_PROPERTY_MANAGER_IMPL_HPP_

