/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO 
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2009. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


//#include "ftreplicationmanager/LocationInfoDataTraits.hpp"
#include "ftreplicationmanager/PropertyManager_impl.hpp"
#include <Foundation/commonsvcs/naming/NamingInterface.hpp>
#include <Foundation/ossupport/OS.hpp>
#include <Foundation/orbsupport/OrbSupport.hpp>
#include <Foundation/orbsupport/ExceptionMinorCodes.hpp>
#include <Foundation/orbsupport/RefCountLocalObject.hpp>
#include <Foundation/osthreads/MutexGuard.hpp>
#include <Foundation/common/Assert.hpp>
#include <FaultTolerance/ftcommon/MacroDef.hpp>
#include <FaultTolerance/ftstatemanager/DataStoreContext.hpp>
#include <FaultTolerance/ftcommon/ReplyRecording.hpp>
#include <FaultTolerance/ftcommon/FTLogger.hpp>
#include "ftreplicationmanager/ReplicationManagerDefinition.hpp"
#include "ftreplicationmanager/FTUtils.hpp"
#include <Foundation/logging/FunctionLogger.hpp>

#include <algorithm>
#include <sstream>


namespace Cdmw {
namespace FT {
namespace ReplicationManager {

    /**
     * Creates a replication manager.
     *
     * @param ft_domain_id The fault tolerance domain id.
     */
    PropertyManager_impl::PropertyManager_impl(
            CORBA::ORB_ptr               orb,
            PortableServer::POA_ptr      poa)
        throw(CORBA::SystemException)
        : m_orb(CORBA::ORB::_duplicate(orb)),
          m_poa(PortableServer::POA::_duplicate(poa))
    {

        CDMW_LOG_FUNCTION(FTLogger::GetLogger(),"");
        // The list of fault tolerance properties.
        m_ft_properties.push_back("org.omg.ft.ReplicationStyle");
        m_ft_properties.push_back("org.omg.ft.MembershipStyle");
        m_ft_properties.push_back("org.omg.ft.ConsistencyStyle");
        m_ft_properties.push_back("org.omg.ft.FaultMonitoringStyle");
        m_ft_properties.push_back("org.omg.ft.FaultMonitoringGranularity");
        m_ft_properties.push_back("org.omg.ft.Factories");
        m_ft_properties.push_back("org.omg.ft.InitialNumberReplicas");
        m_ft_properties.push_back("org.omg.ft.MinimumNumberReplicas");
        m_ft_properties.push_back("org.omg.ft.FaultMonitoringInterval");
        m_ft_properties.push_back("org.omg.ft.CheckpointInterval");

        m_propertiesStorageHome = new PropertiesStorageHome(PROPERTIES_DATA_STORE);
        m_ogdStorageHome = new OGDStorageHome(OGD_DATA_STORE);
 
    }

    /**
     * Destroys a replication manager.
     */
    PropertyManager_impl::~PropertyManager_impl(void) throw ()
    {
        // CDMW_LOG_FUNCTION(FTLogger::GetLogger(),"");
    }





    /**
     * IDL:omg.org/FT/PropertyManager/set_default_properties:1.0
     *
     * Sets the default properties for all object groups that are to be created
     * within the fault tolerance domain.
     *
     * @param props The properties to be set for all newly created object groups
     *              within the fault tolerance domain.
     *
     * @exception InvalidProperty if one or more of the properties in the sequence
     *         is not valid.
     * @exception UnsupportedProperty if one or more of the properties in the
     *         sequence is not supported.
     */
    void
    PropertyManager_impl::set_default_properties(const ::FT::Properties & props)
        throw(::FT::InvalidProperty,
              ::FT::UnsupportedProperty,
              CORBA::SystemException) 
    {
        CDMW_LOG_FUNCTION(FTLogger::GetLogger(),"");
        CDMW_MUTEX_GUARD (m_mutex);

        // Sanity check.
        check_properties(props, DEFAULT_PROPERTIES);

        //store default properties
        ::FT::Properties default_properties(props);        
        try
        {
            //get the previous default properties
            PropertiesStorageObject obj = m_propertiesStorageHome->find_by_oid("default_properties");
            ::FT::Properties_var current_properties = obj.get();
            
            // copy current default properties into a map
            PropertiesMap properties;
            CORBA::ULong property_length = current_properties->length();
            for (CORBA::ULong i = 0; i < property_length; ++i) {
                properties[std::string(current_properties[i].nam[0].id)] = current_properties[i];
            }
            // overwrite the current default properties by the news
            property_length = default_properties.length();
            for (CORBA::ULong i = 0; i < property_length; ++i) {
                properties.insert(make_pair(std::string(default_properties[i].nam[0].id), default_properties[i]));
            }

            // Finally, return the result list of properties.
            PropertiesMap::size_type size = properties.size();
            ::FT::Properties result_properties;// = new ::FT::Properties(size);
            result_properties.length(size);
            CORBA::ULong i = 0;
            PropertiesMap::iterator properties_iter;
            for (properties_iter = properties.begin();
                  properties_iter != properties.end(); ++properties_iter) {                
                  result_properties[i++] = (*properties_iter).second;
             }
            
            // set the default properties
            obj.set(result_properties);
        }
        catch (const NotFoundException& )
        {            
            m_propertiesStorageHome->create("default_properties",default_properties);
        }
        
    }

    /**
     * IDL:omg.org/FT/PropertyManager/get_default_properties:1.0
     *
     * Returns the default properties for the object groups within the fault
     * tolerance domain.
     *
     * @return the default properties that have been set for the object groups.
     */
    ::FT::Properties *
    PropertyManager_impl::get_default_properties(void)
        throw(CORBA::SystemException) 
    {
        CDMW_LOG_FUNCTION(FTLogger::GetLogger(),"");
        CDMW_MUTEX_GUARD (m_mutex);
        // get the storage object for the default properties
        PropertiesStorageObject obj = m_propertiesStorageHome->find_by_oid("default_properties");
        
        // get copy
        ::FT::Properties_var default_properties = obj.get();

        // return copy
        return default_properties._retn();
    }

    /**
     * IDL:omg.org/FT/PropertyManager/remove_default_properties:1.0
     *
     * Removes the given default properties.
     *
     * @param props The properties to be removed.
     *
     * @exception InvalidProperty if one or more of the properties in the sequence
     *         is not valid.
     * @exception UnsupportedProperty if one or more of the properties in the
     *         sequence is not supported.
     */
    void
    PropertyManager_impl::remove_default_properties(const ::FT::Properties & props)
        throw(::FT::InvalidProperty,
              ::FT::UnsupportedProperty,
              CORBA::SystemException) 
    {
        CDMW_LOG_FUNCTION(FTLogger::GetLogger(),"");
        CDMW_MUTEX_GUARD (m_mutex);
        // Sanity check.
        // Properties values are not checked.
        check_properties(props, DEFAULT_PROPERTIES, false);

        //get the previous default properties
        PropertiesStorageObject obj = m_propertiesStorageHome->find_by_oid("default_properties");
        ::FT::Properties_var current_properties = obj.get();
            
        // copy current default properties into a map
        PropertiesMap properties;
        CORBA::ULong property_length = current_properties->length();
        
        for (CORBA::ULong i = 0; i < property_length; ++i) {
            properties.insert(make_pair(std::string(current_properties[i].nam[0].id), current_properties[i]));
        }
        
        // each properties into the sequence, find if is is arlready into the map of the current properties
        for (CORBA::ULong i = 0; i < props.length(); ++i)
        {
            PropertiesMap::iterator iter = properties.find(std::string(props[i].nam[0].id));
            // if the properties is find, erase this properties 
            if (iter != properties.end())
                properties.erase(iter);
            // else throw invlaid property
            else {
                ::FT::InvalidProperty ex;            
                CDMW_LOG_FUNCTION_EXCEPTION(ex);
                throw ex;
            }
        }
        
        // Finally, return the result list of properties.
        PropertiesMap::size_type size = properties.size();
        ::FT::Properties result_properties;// = new ::FT::Properties(size);
        result_properties.length(size);
        CORBA::ULong i = 0;
        PropertiesMap::iterator properties_iter;
        for (properties_iter = properties.begin();
             properties_iter != properties.end(); ++properties_iter) {                
            result_properties[i++] = (*properties_iter).second;
        }
 

        if (size == 0)
        {            
            // remove the default properties
            m_propertiesStorageHome->remove("default_properties");
        } else {
            // set the properties rest
            obj.set(result_properties);            
        }
        
    }

    /**
     * IDL:omg.org/FT/PropertyManager/set_type_properties:1.0
     *
     * Sets the properties that override the default properties of the object
     * groups, with the given type identifier, that are created in the future.
     *
     * @param type_id   The repository id for which the properties, that are to
     *                  override the existing properties, are set.
     * @param overrides The overriding properties.
     *
     * @exception InvalidProperty if one or more of the properties in the sequence
     *         is not valid.
     * @exception UnsupportedProperty if one or more of the properties in the
     *         sequence is not supported.
     */
    void
    PropertyManager_impl::set_type_properties(
            const char * type_id,
            const ::FT::Properties & overrides)
        throw(::FT::InvalidProperty,
              ::FT::UnsupportedProperty,
              CORBA::SystemException) 
    {
        CDMW_LOG_FUNCTION(FTLogger::GetLogger(),"");
        CDMW_MUTEX_GUARD (m_mutex);
        // Sanity check.
        check_properties(overrides, TYPE_PROPERTIES);

        //store default properties
        ::FT::Properties type_properties(overrides);
        try
        {
            //get the previous default properties
            PropertiesStorageObject obj = m_propertiesStorageHome->find_by_oid(type_id);
            ::FT::Properties_var current_properties = obj.get();
            
            // copy current default properties into a map
            PropertiesMap properties;
            CORBA::ULong property_length = current_properties->length();
            for (CORBA::ULong i = 0; i < property_length; ++i) {
                properties[std::string(current_properties[i].nam[0].id)] = current_properties[i];
            }
            // overwrite the current default properties by the news
            property_length = type_properties.length();
            for (CORBA::ULong i = 0; i < property_length; ++i) {
                properties.insert(make_pair(std::string(type_properties[i].nam[0].id), type_properties[i]));
            }

            // Finally, return the result list of properties.
            PropertiesMap::size_type size = properties.size();
            ::FT::Properties result_properties;// = new ::FT::Properties(size);
            result_properties.length(size);
            CORBA::ULong i = 0;
            PropertiesMap::iterator properties_iter;
            for (properties_iter = properties.begin();
                  properties_iter != properties.end(); ++properties_iter) {                
                  result_properties[i++] = (*properties_iter).second;
             }
            
            // set the default properties
            obj.set(result_properties);
        }
        catch (const NotFoundException& )
        {            
            m_propertiesStorageHome->create(type_id,type_properties);
        }
    }

    /**
     * IDL:omg.org/FT/PropertyManager/get_type_properties:1.0
     *
     * Returns the properties of the object groups, with the given type
     * identifier, that are created in the future. These properties include
     * the properties determined by <code>set_type_properties()</code>, as well
     * as the default properties that are not overridden by <code>
     * set_type_properties</code>.
     *
     * @param type_id The repository id for which the properties, that are to
     *                override the existing properties, are set.
     *
     * @return the effective properties for the given type identifier.
     */
    ::FT::Properties *
    PropertyManager_impl::get_type_properties(const char * type_id)
        throw(CORBA::SystemException) 
    {
        CDMW_LOG_FUNCTION(FTLogger::GetLogger(),"type_id="<<type_id);
        CDMW_MUTEX_GUARD (m_mutex);
        // get the storage object for the default properties
        PropertiesStorageObject obj_default_properties = m_propertiesStorageHome->find_by_oid("default_properties");
        ::FT::Properties_var default_property = obj_default_properties.get();

        // transfert the sequence into a map
        PropertiesMap default_properties;
        CORBA::ULong default_property_length = default_property->length();
        for (CORBA::ULong i = 0; i < default_property_length; ++i) {
            default_properties.insert(make_pair(std::string(default_property[i].nam[0].id), default_property[i]));  
            }
        
        // get the storage object for the type properties
        PropertiesStorageObject obj_type_properties = m_propertiesStorageHome->find_by_oid(type_id);
        ::FT::Properties_var type_property = obj_type_properties.get();

        PropertiesMap type_properties;
        CORBA::ULong type_property_length = type_property->length();
        for (CORBA::ULong i = 0; i < type_property_length; ++i) {
            type_properties.insert(make_pair(std::string(type_property[i].nam[0].id), type_property[i]));
            }
        
        PropertiesMap properties = default_properties;
        PropertiesMap::iterator type_properties_iter;
        for (type_properties_iter = type_properties.begin();
             type_properties_iter != type_properties.end();
             ++type_properties_iter) {
            properties[(*type_properties_iter).first] = (*type_properties_iter).second;
        }

        // Finally, return the result list of properties.
        PropertiesMap::size_type size = properties.size();
        ::FT::Properties_var result_properties = new ::FT::Properties(size);
        result_properties->length(size);
        CORBA::ULong i = 0;
        PropertiesMap::iterator properties_iter;
        for (properties_iter = properties.begin();
             properties_iter != properties.end(); ++properties_iter) {

            result_properties[i++] = (*properties_iter).second;
        }

        return result_properties._retn();
    }

    /**
     * IDL:omg.org/FT/PropertyManager/remove_type_properties:1.0
     *
     * Removes the given properties, with the given type identifier.
     *
     * @param type_id The repository id for which the given properties are to
     *                be removed.
     * @param props   The properties to be removed.
     *
     * @exception InvalidProperty if one or more of the properties in the sequence
     *         is not valid.
     * @exception UnsupportedProperty if one or more of the properties in the
     *         sequence is not supported.
     */
    void
    PropertyManager_impl::remove_type_properties(
            const char * type_id,
            const ::FT::Properties & props)
        throw(::FT::InvalidProperty,
              ::FT::UnsupportedProperty,
              CORBA::SystemException) 
    {
        CDMW_LOG_FUNCTION(FTLogger::GetLogger(),"type_id="<<type_id);
        CDMW_MUTEX_GUARD (m_mutex);
        // Sanity check.
        // Properties values are not checked.
        check_properties(props, TYPE_PROPERTIES, false);

        //get the previous default properties
        PropertiesStorageObject obj = m_propertiesStorageHome->find_by_oid(type_id);
        ::FT::Properties_var current_properties = obj.get();
            
        // copy current default properties into a map
        PropertiesMap properties;
        CORBA::ULong property_length = current_properties->length();
        for (CORBA::ULong i = 0; i < property_length; ++i) {
            properties.insert(make_pair(std::string(current_properties[i].nam[0].id), current_properties[i]));
        }
        
        // each properties into the sequence, find if is is arlready into the map of the current properties
        for (CORBA::ULong i = 0; i < props.length(); ++i)
        {
            PropertiesMap::iterator iter = properties.find(std::string(props[i].nam[0].id));
            // if the properties is find, erase this properties 
            if (iter != properties.end())
                properties.erase(iter);
            // else throw invlaid property
            else
                throw ::FT::InvalidProperty();            
        }
        
        // Finally, return the result list of properties.
        PropertiesMap::size_type size = properties.size();
        ::FT::Properties result_properties;// = new ::FT::Properties(size);
        result_properties.length(size);
        CORBA::ULong i = 0;
        PropertiesMap::iterator properties_iter;
        for (properties_iter = properties.begin();
             properties_iter != properties.end(); ++properties_iter) {                
            result_properties[i++] = (*properties_iter).second;
        }
 

        if (size == 0)
        {            
            // remove the default properties
            m_propertiesStorageHome->remove(type_id);
        } else {
            // set the properties rest
            obj.set(result_properties);            
        }
    }

    /**
     * IDL:omg.org/FT/PropertyManager/set_properties_dynamically:1.0
     *
     * Sets the properties for the object group with the given reference
     * dynamically while the application executes. The properties given as a
     * parameter override the properties for the object when it was created
     * which, in turn, override the properties for the given type which, in
     * turn, override the default properties.
     *
     * @param object_group The reference of the object group for which the
     *                     overriding properties are set.
     * @param overrides    The overriding properties.
     *
     * @exception ObjectGroupNotFound if the object group is not found with the
     *         given reference.
     * @exception InvalidProperty if one or more of the properties in the sequence
     *         is not valid.
     * @exception UnsupportedProperty if one or more of the properties in the
     *         sequence is not supported.
     */
    void
    PropertyManager_impl::set_properties_dynamically (
            ::FT::ObjectGroup_ptr object_group,
            const ::FT::Properties & overrides)
        throw(::FT::ObjectGroupNotFound,
              ::FT::InvalidProperty,
              ::FT::UnsupportedProperty,
              CORBA::SystemException) 
    {
        CDMW_LOG_FUNCTION(FTLogger::GetLogger(),"");
        CDMW_MUTEX_GUARD (m_mutex);
        // Sanity check.
        check_properties(overrides, DYNAMIC_PROPERTIES);

        try
        {            
            // Get all known data of the object group
            // May raise ::FT::ObjectGroupNotFound
            ::FT::ObjectGroupId object_group_id = get_object_group_id_from_ref( object_group );
            OGDStorageObject obj = m_ogdStorageHome->find_by_oid(object_group_id);        
            ::CdmwFT::ObjectGroupData_var object_group_data = obj.get();
            


            //store default properties
            ::FT::Properties dynamic_properties(overrides);
            try
            {
                ::FT::Properties current_properties = object_group_data->dynamic_properties;                
                
                // copy current default properties into a map
                PropertiesMap properties;
                CORBA::ULong property_length = current_properties.length();
                for (CORBA::ULong i = 0; i < property_length; ++i) {
                    properties.insert(make_pair(std::string(current_properties[i].nam[0].id), current_properties[i]));
                }
                // overwrite the current default properties by the news
                property_length = dynamic_properties.length();
                for (CORBA::ULong i = 0; i < property_length; ++i) {
                    properties[std::string(dynamic_properties[i].nam[0].id)] =  dynamic_properties[i];
                }

                // Finally, return the result list of properties.
                PropertiesMap::size_type size = properties.size();
                ::FT::Properties result_properties;// = new ::FT::Properties(size);
                result_properties.length(size);
                CORBA::ULong i = 0;
                PropertiesMap::iterator properties_iter;
                for (properties_iter = properties.begin();
                     properties_iter != properties.end(); ++properties_iter) {                
                    result_properties[i++] = (*properties_iter).second;
                }
            
                // set the default properties
                object_group_data->dynamic_properties=result_properties;            
                obj.set(object_group_data.in());
            }
            catch (const NotFoundException& )
            {            
           
            }

        }
        catch (const NotFoundException& e)
        {            
            ::FT::ObjectGroupNotFound ex;   
            CDMW_INFO(FTLogger::GetLogger(),
                      "set_properties_dynamically(): " 
                      << "Received " << e << " exception. "
                      <<"\nThrowing " << ex << " exception. ");
            CDMW_LOG_FUNCTION_EXCEPTION(ex);
            throw ex;         
        }

    }

    /**
     * IDL:omg.org/FT/PropertyManager/get_properties:1.0
     *
     * Returns the current properties of the given object group. These
     * properties include those that are set dynamically, those that are set
     * when the object group was created but are not overridden by <code>
     * set_properties_dynamically()</code>, those that are set as properties of
     * a type but are not overridden by <code>create_obj*ect()</code> and <code>
     * set_properties_dyamically()</code>, and those that are set as defaults
     * but are not overridden by <code>set_type_properties()</code>, <code>
     * create_object()</code>, and <code>set_properties_dyamically()</code>.
     *
     * @param object_group The reference of the object group for which the
     *                     properties are to be returned.
     *
     * @return the set of current properties for the object group with the given
     *         reference.
     *
     * @exception ObjectGroupNotFound if the object group is not found by the
     *         Replication Manager.
     */
    ::FT::Properties *
    PropertyManager_impl::get_properties(
            ::FT::ObjectGroup_ptr object_group)
        throw(::FT::ObjectGroupNotFound,
              CORBA::SystemException) 
    {
        CDMW_LOG_FUNCTION(FTLogger::GetLogger(),"");
        CDMW_MUTEX_GUARD (m_mutex);
        // Get all known data of the object group
        // May raise ::FT::ObjectGroupNotFound
        ::FT::ObjectGroupId object_group_id = get_object_group_id_from_ref( object_group );
        OGDStorageObject obj = m_ogdStorageHome->find_by_oid(object_group_id);        
        ::CdmwFT::ObjectGroupData_var object_group_data = obj.get();


        // get the storage object for the default properties
        PropertiesStorageObject obj_default_properties = m_propertiesStorageHome->find_by_oid("default_properties");
        ::FT::Properties_var default_property = obj_default_properties.get();

        // transfert the sequence into a map
        PropertiesMap default_properties;
        CORBA::ULong default_property_length = default_property->length();
        for (CORBA::ULong i = 0; i < default_property_length; ++i) {
            default_properties.insert(make_pair(std::string(default_property[i].nam[0].id), default_property[i]));
            }
        
        // get the storage object for the type properties
        PropertiesStorageObject obj_type_properties = m_propertiesStorageHome->find_by_oid(std::string(object_group_data->repository_id));
        ::FT::Properties_var type_property = obj_type_properties.get();

        PropertiesMap type_properties;
        CORBA::ULong type_property_length = type_property->length();
        for (CORBA::ULong i = 0; i < type_property_length; ++i) {
            type_properties.insert(make_pair(std::string(type_property[i].nam[0].id), type_property[i]));
            }
        
        PropertiesMap properties = default_properties;

        PropertiesMap::iterator type_properties_iter;
        for (type_properties_iter = type_properties.begin();
             type_properties_iter != type_properties.end();
             ++type_properties_iter) {
            properties[(*type_properties_iter).first] = (*type_properties_iter).second;
        }


        // get the creation properties
        ::FT::Properties creation_property = object_group_data->creation_properties;
        PropertiesMap creation_properties;
        CORBA::ULong creation_property_length = creation_property.length();
        for (CORBA::ULong i = 0; i < creation_property_length; ++i) {
            creation_properties.insert(make_pair(std::string(creation_property[i].nam[0].id),creation_property[i]));
        }
        

        PropertiesMap::iterator creation_properties_iter;
        for (creation_properties_iter = creation_properties.begin();
             creation_properties_iter != creation_properties.end();
             ++creation_properties_iter) {
            properties[(*creation_properties_iter).first] = (*creation_properties_iter).second;
        }


        // get the dynamic properties
        ::FT::Properties dynamic_property = object_group_data->dynamic_properties;
        PropertiesMap dynamic_properties;
        CORBA::ULong dynamic_property_length = dynamic_property.length();
        for (CORBA::ULong i = 0; i < dynamic_property_length; ++i) {
            dynamic_properties.insert(make_pair(std::string(dynamic_property[i].nam[0].id), dynamic_property[i]));
        }
        

        PropertiesMap::iterator dynamic_properties_iter;
        for (dynamic_properties_iter = dynamic_properties.begin();
             dynamic_properties_iter != dynamic_properties.end();
             ++dynamic_properties_iter) {
            properties[(*dynamic_properties_iter).first] =  (*dynamic_properties_iter).second;
        }

        // Finally, return the result list of properties.
        PropertiesMap::size_type size = properties.size();
        ::FT::Properties_var result_properties = new ::FT::Properties(size);
        result_properties->length(size);
        CORBA::ULong i = 0;
        PropertiesMap::iterator properties_iter;
        for (properties_iter = properties.begin();
             properties_iter != properties.end(); ++properties_iter) {

            result_properties[i++] = (*properties_iter).second;
        }

        return result_properties._retn();
    }
    

    /**
     * Returns all known data for a specific object group.
     *
     * @param object_group the object group reference.
     *
     * @return an ObjectGroupId
     *
     * @exception ObjectGroupNotFound if the object group with
     *   the given reference is not found 
     */
    
    
    ::FT::ObjectGroupId
    PropertyManager_impl::get_object_group_id_from_ref ( ::FT::ObjectGroup_ptr object_group ) const
        throw(::FT::ObjectGroupNotFound,
              CORBA::SystemException) 
    {
        CDMW_LOG_FUNCTION(FTLogger::GetLogger(),"");
        std::vector<OGDStorageObject> ogd_list;
        m_ogdStorageHome->find_if(
            std::bind2nd( is_equivalent_to< std::pair< Cdmw::OrbSupport::type_traits< ::FT::ObjectGroupId >::_var_type, 
                                                       Cdmw::OrbSupport::type_traits< ::CdmwFT::ObjectGroupData >::_var_type 
                                                     > 
                                          > (),
                          object_group),
            std::back_inserter(ogd_list));
            
        if (ogd_list.size() != 1) {
            ::FT::ObjectGroupNotFound ex;
            CDMW_LOG_FUNCTION_EXCEPTION(ex);
            throw ex;
        }
        ::FT::ObjectGroupId ogid= ogd_list[0L].get_oid();
        CDMW_LOG_FUNCTION_RETURN(ogid);
        return ogid;
    }


    /**
     * Checks if the given properties are valid and supported.
     *
     * @param props       The properties to check.
     * @param type        The type of properties, type can be either
     *                    <code>DEFAULT_PROPERTIES</code> or
     *                    <code>TYPE_PROPERTIES</code> or
     *                    <code>GROUP_PROPERTIES</code>
     * @param check_value If false then bypass the value checking.
     *
     * @throw InvalidProperty if one or more of the properties in the
     *        sequence is not valid.
     * @throw UnsupportedProperty if one or more of the properties in
     *        the sequence is not supported.
     */
    void
    PropertyManager_impl::check_properties(
            const ::FT::Properties & props,
            int type,
            bool check_value)
        throw(::FT::InvalidProperty,
              ::FT::UnsupportedProperty) 
    {
        CDMW_LOG_FUNCTION(FTLogger::GetLogger(),"type="<<type<<",check_value="<<check_value);
        CORBA::ULong props_length = props.length();
        for (CORBA::ULong iProps = 0; iProps < props_length; ++iProps) {
            // Get the property name.
            ::FT::Property property = props[iProps];
            std::string propertyName = std::string(property.nam[0].id);
            
            // Check the property name against the list of FT properties.
            StringList::iterator ft_properties_iter =
                std::find(m_ft_properties.begin(),
                          m_ft_properties.end(),
                          propertyName);

            if (ft_properties_iter != m_ft_properties.end()) {
                bool is_valid = true;
                bool is_supported = true;

                if (propertyName == "org.omg.ft.ReplicationStyle") {
                    // STATELESS = 0
                    // COLD_PASSIVE = 1
                    // WARM_PASSIVE = 2
                    // ACTIVE = 3
                    // ACTIVE_WITH_VOTING = 4

                    // (see paragraph 23.3.2 of the specifications)
                    // ReplicationStyle cannot be set dynamically.
                    if (type == DYNAMIC_PROPERTIES) {
                        is_valid = false;
                    } else if (check_value) {
                        CORBA::UShort value = 0;
                        property.val >>= value;
                        if (value > 4) {
                            is_valid = false;
                        } else {
                            // Only WARM_PASSIVE is supported right now.
                            if (value != 2) {
                                is_supported = false;
                            }
                        }
                    }
                } else if (propertyName == "org.omg.ft.MembershipStyle") {
                    // MEMB_APP_CTRL = 0
                    // MEMB_INF_CTRL = 1

                    // (see paragraph 23.3.2 of the specifications)
                    // MembershipStyle cannot be set dynamically.
                    if (type == DYNAMIC_PROPERTIES) {
                        is_valid = false;
                    } else if (check_value) {
                        CORBA::UShort value = 0;
                        property.val >>= value;
                        if (value > 1) {
                            is_valid = false;
                        } else {
                            // Only MEMB_APP_CTRL is supported at the moment.
                            if (value != 0) {
                                is_supported = false;
                            }
                        }
                    }
                } else if (propertyName == "org.omg.ft.ConsistencyStyle") {
                    // CONS_APP_CTRL = 0
                    // CONS_INF_CTRL = 1

                    // Not supported at the moment.
                    is_supported = false;

                    // (see paragraph 23.3.2 of the specifications)
                    // ConsistencyStyle cannot be set at creation time
                    // nor dynamically.
                    if ((type == CREATION_PROPERTIES) ||
                        (type == DYNAMIC_PROPERTIES)) {

                        is_valid = false;
                    }
                } else if (propertyName == "org.omg.ft.FaultMonitoringStyle") {
                    // PULL = 0
                    // PUSH = 1
                    // NOT_MONITORED = 2

                    // (see paragraph 23.3.2 of the specifications)
                    // FaultMonitoringStyle cannot be set at creation time
                    // nor dynamically.
                    if ((type == CREATION_PROPERTIES) ||
                        (type == DYNAMIC_PROPERTIES)) {
                        is_valid = false;
                    } else if (check_value) {
                        CORBA::UShort value = 0;
                        property.val >>= value;
                        if (value > 2) {
                            is_valid = false;
                        } else {
                            // Only PULL is supported right now.
                            if (value != 0) {
                                is_supported = false;
                            }
                        }
                    }
                } else if (propertyName == "org.omg.ft.FaultMonitoringGranularity") {
                    // MEMB = 0
                    // LOC = 1
                    // LOC_AND_TYPE = 2

                    is_supported = false;
                } else if (propertyName == "org.omg.ft.Factories") {
                    // Not supported at the moment.
                    is_supported = true;

                    // (see paragraph 23.3.2 of the specifications)
                    // Factories cannot be set as default properties.
                    if (type == DEFAULT_PROPERTIES) {
                        is_valid = false;
                    }
                } else if (propertyName == "org.omg.ft.InitialNumberReplicas") {
                    // (see paragraph 23.3.2 of the specifications)
                    // InitialNumberReplicas cannot be set dynamically.
                    if (type == DYNAMIC_PROPERTIES) {
                        is_valid = false;
                    } else if (check_value) {
                        CORBA::UShort value = 0;
                        property.val >>= value;

                        // This is not supported as the Factories property
                        // is not currently supported.
                        is_supported = false;
                    }
                } else if (propertyName == "org.omg.ft.MinimumNumberReplicas") {
                    if (check_value) {
                        try {
                            CORBA::UShort value = 0;
                            property.val >>= value;
                        } catch (...) {
                            is_valid = false;
                        }
                    }
                } else if (propertyName == "org.omg.ft.FaultMonitoringInterval") {
                    // Not sure whether we support this property.
                    is_supported = false;
                } else if (propertyName == "org.omg.ft.CheckpointInterval") {
                    is_supported = false;
                }

                // If this property is either invalid or unsupported, then raise
                // the appropriate exception.
                if (!is_valid) {
                    CDMW_WARN(FTLogger::GetLogger(),"Invalid Property '" << property.nam[0].id  << "'");
                    ::FT::InvalidProperty ex(property.nam, property.val);
                    CDMW_LOG_FUNCTION_EXCEPTION(ex);
                    throw ex;
                } else if (!is_supported) {
                    CDMW_WARN(FTLogger::GetLogger(),"Unsupported Property '" << property.nam[0].id  << "'");
                    ::FT::UnsupportedProperty ex(property.nam, property.val);
                    CDMW_LOG_FUNCTION_EXCEPTION(ex);
                    throw ex;
                }
            } else {
                // We couldn't check this property against the FT properties,
                // we'll consider it as invalid.
                CDMW_WARN(FTLogger::GetLogger(),"Invalid Property '" << property.nam[0].id << "'");
                ::FT::InvalidProperty ex(property.nam, property.val);
                CDMW_LOG_FUNCTION_EXCEPTION(ex);
                throw ex;
            }
        }
    }
    
}; // namespace ReplicationManager
}; // namespace FT
}; // namespace Cdmw

