/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO 
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2009. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#include <fstream>

// Cdmw Files
#include <Foundation/common/Options.hpp>
#include <Foundation/testutils/CORBATestManager.hpp>
#include <Foundation/orbsupport/CORBA.hpp>
#include <Foundation/orbsupport/RefCountLocalObject.hpp>
#include <Foundation/orbsupport/ExceptionMinorCodes.hpp>
#include <Foundation/orbsupport/OrbSupport.hpp>
#include <Foundation/orbsupport/StrategyList.hpp>
#include <Foundation/osthreads/Thread.hpp>
#include <Foundation/ossupport/OS.hpp>
#include <FaultTolerance/ftinit/FTServiceInit.hpp>

// internal cdmw init
#include <testsimultaneousaddmember/CdmwInterface.hpp>


#include <testsimultaneousaddmember/TestHello_impl.hpp>
#include <testsimultaneousaddmember/TestComput_impl.hpp>
#include <testsimultaneousaddmember/TestUtils.hpp>
#include <testsimultaneousaddmember/OrbThread.hpp>


#include <idllib/CdmwFTActivation.skel.hpp>

#include <idllib/CdmwFTReplicationManager.stub.hpp>
#include <idllib/CdmwFTGroupRepository.stub.hpp>
#include <idllib/FT.stub.hpp>

using namespace std;

namespace 
{

    const std::string FILE_PATH_OPTION2  = "--ComputFilePath";
    const std::string HOST_NAME_OPTION  = "--HostName";
    const std::string SLEEP_DURATION_OPTION  = "--SleepDuration";
    const std::string CRASH_SERVER_OPTION  = "--CrashServer";
    const std::string CHANGE_ORDER_OPTION  = "--ChangeOrder";

    const int POA_THREAD_POOL_SIZE = 5;   

    class HelloActivationHandler : virtual public CdmwFT::Location::ActivationHandler, 
                                   virtual public Cdmw::OrbSupport::RefCountLocalObject
    {
        std::string m_location;

    public:
        HelloActivationHandler(std::string location) : m_location(location) { };

        virtual void activate() throw(CORBA::SystemException)
        {
            std::ofstream os("hello.act");
            os << m_location;
            os.close();

            std::cout << m_location << " <####### HELLO ACTIVATION ######>" << std::endl;
        };


        virtual void deactivate() throw(CORBA::SystemException)
        {
            std::ofstream os("hello.dct");
            os << m_location;
            os.close();

            std::cout << m_location << " <####### HELLO DEACTIVATION ######>" << std::endl;
        };
    };

                                   
    class ComputActivationHandler : virtual public CdmwFT::Location::ActivationHandler, 
                                   virtual public Cdmw::OrbSupport::RefCountLocalObject
    {
        std::string m_location;

    public:
        ComputActivationHandler(std::string location) : m_location(location) { };

        virtual void activate() throw(CORBA::SystemException)
        {
            std::ofstream os("comput.act");
            os << m_location;
            os.close();

            std::cout << m_location << " <####### COMPUT ACTIVATION ######>" << std::endl;
        };


        virtual void deactivate() throw(CORBA::SystemException)
        {
            std::ofstream os("comput.dct");
            os << m_location;
            os.close();

            std::cout << m_location << " <####### COMPUT DEACTIVATION ######>" << std::endl;
        };
    };


} // end anonymous namespace


static void
usage(std::ostream & os, const std::string &program_name)
{
    os << "Usage:\n"
        << program_name << " options\n"
        << "\n"
        << "Options:\n"
        << "--help                                         Display this information.\n"
        << "\n"
        << "-ORBInitRef ReplicationManager=<rep_corbaloc>  ReplicationManager corbaloc\n"
        << HOST_NAME_OPTION <<"=hostname      hostname where server is launched.\n"
        << "[" << SLEEP_DURATION_OPTION <<"]=duration      sleep duration between the 2 add_member.\n"
        << "[" << CRASH_SERVER_OPTION <<"]    the server crash between the two add member.\n"
        << "[" << CHANGE_ORDER_OPTION <<"]    the order of add member call is changed.\n"
        << std:: endl;
}

                               
int main( int argc, char* argv[] )
{
    // help argument
    if (argc <=2)
    {
        usage (std::cout, argv[0]);
        return EXIT_SUCCESS;
    }

    if (argc >= 2)
    {
        if (strcmp(argv[1], "--help") == 0)
        {
            usage (std::cout, argv[0]);
            return EXIT_SUCCESS;
        }
    }

    // get host name
    std::string host_name = 
       Cdmw::OsSupport::OS::get_option_value (argc, argv, HOST_NAME_OPTION);

    // get sleep_duration
    std::string sleep_duration = 
       Cdmw::OsSupport::OS::get_option_value (argc, argv, SLEEP_DURATION_OPTION);

    if (sleep_duration == "no")
    {
        sleep_duration = "0";
    }

    // get CRASH_SERVER_OPTION 
    std::string crash_server_option = 
       Cdmw::OsSupport::OS::get_option_value (argc, argv, CRASH_SERVER_OPTION);

    // get CHANGE_ORDER_OPTION 
    std::string change_order_option = 
       Cdmw::OsSupport::OS::get_option_value (argc, argv, CHANGE_ORDER_OPTION);


    CORBA::ORB_var orb = CORBA::ORB::_nil();

    try
    {
        
        // Initialise FT service
        std::cout << "test_server: initialise FT service" << std::endl;
        Cdmw::FT::FTServiceInit::Init(argc, argv);

        // Initialises the ORB
        Cdmw::OrbSupport::StrategyList strategyList;
        strategyList.add_OrbThreaded();
        strategyList.add_PoaThreadPool(POA_THREAD_POOL_SIZE);

        std::cout << host_name 
                  << " test_server: initialises the ORB" 
                  << std::endl;
        orb= Cdmw::OrbSupport::OrbSupport::ORB_init(argc, argv, strategyList);
  
        // Get the root POA
        std::cout << host_name << " test_server: get the root POA" << std::endl;
        CORBA::Object_var obj = orb->resolve_initial_references("RootPOA");
        PortableServer::POA_var rootPOA = PortableServer::POA::_narrow(obj.in());

        std::cout << host_name << " test_server: activate the root POA" << std::endl;
        PortableServer::POAManager_var poaMgr = rootPOA->the_POAManager();
        poaMgr->activate();

        // init Cdmw
        std::cout << host_name << " test_server: init Cdmw"<<std::endl;
        Cdmw::CdmwInit::CDMW_init(orb.in(), argc, argv);


        std::cout << host_name << " test_server: create HelloInterface_impl"<<std::endl;
        // Create activation handler, it will be register in init operation of Hello
        std::cout<< host_name << " test_server: create activation handler and register it"
                 <<std::endl;
        CdmwFT::Location::ActivationHandler_var activation_handler 
            = new HelloActivationHandler(host_name);

        Cdmw::Test::HelloInterface_impl * hello_interface 
            = new Cdmw::Test::HelloInterface_impl(orb.in(), host_name, activation_handler.in());
        CdmwFT::Test::HelloInterface_var hello_interface_ref 
            =  hello_interface->_this();

        std::cout << host_name << " test_server: create ComputInterface_impl"<<std::endl;
        // Create activation handler, it will be register in init operation of Hello
        std::cout<< host_name << " test_server: create activation handler and register it"
                 <<std::endl;
        CdmwFT::Location::ActivationHandler_var comput_activation_handler 
            = new ComputActivationHandler(host_name);

        Cdmw::Test::ComputInterface_impl * comput_interface 
            = new Cdmw::Test::ComputInterface_impl(orb.in(), host_name, activation_handler.in());
        CdmwFT::Test::ComputInterface_var comput_interface_ref 
            =  comput_interface->_this();

        std::cout << host_name << " test_server: get hello_group ref"<<std::endl;
        CdmwFT::Test::HelloInterface_var hello_group 
           = Cdmw::FT::TestUtils::get_hello_ref_from_file(orb.in(), "hello_group.ior");

        std::cout << host_name << " test_server: get comput_group ref"<<std::endl;
        CdmwFT::Test::ComputInterface_var comput_group 
           = Cdmw::FT::TestUtils::get_comput_ref_from_file(orb.in(), "comput_group.ior");

        std::cout << host_name << " test_server: get replication manager ref"<<std::endl;
        obj = orb->resolve_initial_references("ReplicationManager");
        CdmwFT::ReplicationManager_var replication_manager 
           = CdmwFT::ReplicationManager::_narrow(obj.in());

        // Start the OrbThread to run the orb!
        Cdmw::Test::OrbThread orb_thread(orb.in(), host_name);
        orb_thread.start();


        // Do test
        Cdmw::TestUtils::CORBATestManager::instance()->init(orb.in(),
                                                            rootPOA.in(),
                                                            argc,argv);
        Cdmw::TestUtils::CORBATestManager::instance()->set_property("hostname", host_name);
        Cdmw::TestUtils::CORBATestManager::instance()->set_property("sleep_duration", sleep_duration);
        Cdmw::TestUtils::CORBATestManager::instance()->set_property("crash_server_option", crash_server_option);
        Cdmw::TestUtils::CORBATestManager::instance()->set_property("change_order_option", change_order_option);


        Cdmw::TestUtils::CORBATestManager::instance()->add("hello_object_group", 
                                                           hello_group.in());
        Cdmw::TestUtils::CORBATestManager::instance()->add("comput_object_group", 
                                                           comput_group.in());
        Cdmw::TestUtils::CORBATestManager::instance()->add("hello_object", 
                                                           hello_interface_ref.in());
        Cdmw::TestUtils::CORBATestManager::instance()->add("comput_object", 
                                                           comput_interface_ref.in());
        Cdmw::TestUtils::CORBATestManager::instance()->add("replication_manager", 
                                                           replication_manager.in());
        Cdmw::TestUtils::CORBATestManager::instance()->run_tests();

        int timescale = Cdmw::TestUtils::get_timescale();
        Cdmw::OsSupport::OS::sleep(timescale*10000);

        Cdmw::TestUtils::CORBATestManager::instance()->summary();

        orb_thread.shutdown();
        orb_thread.join();
    }
    catch (const CORBA::SystemException &e )
    {
        cerr <<__FILE__<<" "<<__LINE__<<" "<< e << endl;
        return EXIT_FAILURE;
    }
    catch( const CORBA::Exception &e )
    {
        cerr <<__FILE__<<" "<<__LINE__<<" "<< e._name() << endl;
        return EXIT_FAILURE;
    }
    catch( const Cdmw::Exception &e )
    {
        cerr <<__FILE__<<" "<<__LINE__<<" "<< e.what() << endl;
        return EXIT_FAILURE;
    }
    catch(...)
    {
        cerr << "Uncaught exception" << endl;
        return EXIT_FAILURE;
    }

    if (!CORBA::is_nil(orb.in()))
    {
         try
         {
             Cdmw::OrbSupport::OrbSupport::ORB_cleanup(orb.in());
             orb->destroy();
         }
         catch(const CORBA::Exception& ex)
         {
             std::cerr << ex << std::endl;
             std::cerr << " **** TEST FAILED AT LINE " << __LINE__ 
                       << " IN FILE: " << __FILE__ << std::endl;
             return EXIT_FAILURE;
         }
    }        
    
    return EXIT_SUCCESS;
}

