/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO 
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2009. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

#ifndef INCL_CDMW_FT_FTSTATEMANAGER_DATASTORE_HPP
#define INCL_CDMW_FT_FTSTATEMANAGER_DATASTORE_HPP

#include <Foundation/osthreads/ReaderLockGuard.hpp>

#include <FaultTolerance/ftstatemanager/DataStoreBase.hpp>
#include <FaultTolerance/ftstatemanager/DataStoreRecordIteratorBase.hpp>
#include <FaultTolerance/ftstatemanager/TransactionEngine.hpp>
#include <FaultTolerance/ftstatemanager/TransactionEngineCohort.hpp>
#include <FaultTolerance/ftstatemanager/TransactionEngineCoordinator.hpp>
#include <FaultTolerance/idllib/CdmwFTStateTransfer.skel.hpp>

#include <Foundation/common/Exception.hpp>
#include <Foundation/commonsvcs/datastore/DataStoreInterface.hpp>
#include <Foundation/commonsvcs/datastore/DataStoreObserver.hpp>
#include <Foundation/commonsvcs/datastore/DataStoreRegistry.hpp>
#include <Foundation/osthreads/ReaderLockGuard.hpp>
#include <Foundation/osthreads/WriterLockGuard.hpp>
#include <Foundation/osthreads/MutexGuard.hpp>
#include <Foundation/orbsupport/TypeTraits.hpp>
#include <Foundation/orbsupport/Codec.hpp>
#include <Foundation/orbsupport/Sequence.hpp>
#include <Foundation/orbsupport/OrbSupport.hpp>
#include <sstream>
#include <list>
#include <functional>

#include <FaultTolerance/ftcommon/FTLogger.hpp>


namespace Cdmw {
namespace FT {

// Forward declaration
template <class OID, class DATA, class OIDCOMPARE> 
class DataStoreInitialiser;

template <class OID, class DATA, class OIDCOMPARE> 
class DataStoreRecordIterator;

typedef Cdmw::CommonSvcs::DataStore::AlreadyExistException
    AlreadyExistException;
typedef Cdmw::CommonSvcs::DataStore::NotFoundException NotFoundException;

/**
*This template class defines a DataStore which is able to transfer
*each of its modified data to other DataStores in other backup process.
*The state transfer could use multicast or point-to-point protocol. 
*
*This is a template class where the Oid and Data should be IDL types.
*
*Features:
*<p> Thread safe. Exception safe.
*/
template <class OID, class DATA, class OIDCOMPARE=std::less<typename Cdmw::OrbSupport::type_traits<OID>::_var_type> > 
class DataStore : public DataStoreBase,
                  //                  public Timer_Event_Handler<void>,
                  public Cdmw::CommonSvcs::DataStore::DataStoreInterface<OID, DATA, OIDCOMPARE>
{
    typedef CdmwFT::Transaction::StateChangeRecord StateChangeRecord;
    typedef CdmwFT::Transaction::StateChangeRecords StateChangeRecords;
    typedef DataStoreRecordIterator<OID, DATA, OIDCOMPARE>
        DataStoreRecordIterator;
    typedef std::map<std::string, DataStoreRecordIterator*,
                     std::less<std::string> >
        RecordIteratorMap;
    typedef typename DataStoreContext::LocationSet LocationList;

public:   

    typedef typename Cdmw::OrbSupport::type_traits<OID>::_var_type     _Oid_var_type;
    typedef typename Cdmw::OrbSupport::type_traits<OID>::_in_type      _Oid_in_type;
    typedef typename Cdmw::OrbSupport::type_traits<OID>::_retn_type    _Oid_retn_type;

    typedef typename Cdmw::OrbSupport::type_traits<DATA>::_var_type     _Data_var_type;
    typedef typename Cdmw::OrbSupport::type_traits<DATA>::_in_type      _Data_in_type;
    typedef typename Cdmw::OrbSupport::type_traits<DATA>::_retn_type    _Data_retn_type;
    
    typedef Cdmw::CommonSvcs::DataStore::DataStoreObserver<OID, DATA> DataStoreObserver;
    typedef std::list<DataStoreObserver*> DataStoreObserverList;
    typedef typename DataStoreObserverList::iterator DataStoreObserverListIterator;
    typedef Cdmw::CommonSvcs::DataStore::Functor<OID, DATA> Functor;

    typedef std::map<_Oid_var_type, _Data_var_type, OIDCOMPARE> DataMap;
    typedef typename DataMap::iterator DataMapIterator;
    typedef typename DataMap::const_iterator DataMapConstIterator;
    typedef std::insert_iterator<DataMap> DataInsertIterator;
    typedef DataStoreInitialiser<OID, DATA, OIDCOMPARE> DataStoreInitialiser;


    /**
     * Constructor of a DataStore using multicast state transfer.
     *
     *@param dsid the id of the data store. this id must be unique in the system
     *@param orb the orb reference
     *@param poa the poa reference
     *
     *@exception CORBA::SystemException Any CORBA system exception that
     * may be raised by the underlying ORB.
     */ 
    DataStore(DsIdentifier            dsId,
              CORBA::ORB_ptr          orb,
              PortableServer::POA_ptr poa,
              DataStoreContext&       context,
              bool                    register_data_store = true)
        throw(AlreadyExistException, CORBA::SystemException)
        : Cdmw::FT::DataStoreBase(dsId),
          m_orb(CORBA::ORB::_duplicate(orb)),
          m_poa(PortableServer::POA::_duplicate(poa)),
          m_context(context),
          m_oid_codec(m_orb.in()),
          m_data_codec(m_orb.in()),
          m_pInitialiser(0),
          m_register_data_store(register_data_store)
    {
        if (m_register_data_store)
        {
            Cdmw::CommonSvcs::DataStore::DataStoreRegistry::
                Register_data_store(this);
        }
        m_context.register_datastore(this);
        DataStoreBase::Register_dataStore(this);
    }

    
    /**
     * Destructor.
     */
    virtual ~DataStore()
    {
        if (m_register_data_store)
        {
            try
            {
                Cdmw::CommonSvcs::DataStore::DataStoreRegistry::
                    Unregister_data_store(this->get_dsid());
            }
            catch (const NotExistException&)
            {
                CDMW_ERROR(FTLogger::GetLogger(),
                           "DataStore " << this->get_dsid() 
                           << " was not registered in DataStoreRegistry!");
            }
        }
        m_context.unregister_datastore(this->get_dsid());
        DataStoreBase::Unregister_dataStore(this->get_dsid());
    }

    inline DataStoreContext& get_context()
    {
        return m_context;
    }

    /**
     * Returns an iterator that provides state information necessary to copy
     * a DataStore.
     *
     * @param from_location is the name of the location asking the records.
     * @param coordinator the transaction cordinator.
     * @param cohort the cohort.
     *
     * @exception InvalidDataStoreState is raised if the data store is not the 
     * coordinator
     */
    virtual DataStoreRecordIteratorBase* get_record_iterator
    (const char * from_location)
        throw (CORBA::SystemException,
               CdmwFT::StateTransfer::InvalidDataStoreState);

    /**
     * Adds the given location as a new backup for the DataStore and removes
     * the RecordIterator associated with the location.
     *
     * @param location location of new backup.
     * @param coordinator the transaction cordinator.
     * @param cohort the cohort.
     */
    virtual void iterator_finished(const char * from_location)
        throw (NotFoundException);

    /**
     * Freeze timer callback method.
     */
//    int handle_timeout(const void*);

    /**
     * Registers an observer to be notified whenever there is an insertion, update, or removal
     * of a record.
     *
     *@param observer a const pointer to a DataStoreObserver object
     *@return true if the Data Store observer is registered, false if failed to register it
     * (it has already been registered, or exceeded maximum allowed number of observers ...)
     */
    bool register_observer(DataStoreObserver * observer) throw()
    {
        CDMW_WRITER_LOCK_GUARD(m_observers_mutex);
        DataStoreObserverListIterator pos;

        pos = std::find(m_observers.begin(), m_observers.end(), observer);

        if (pos == m_observers.end())
        {
            m_observers.push_back(observer);
            return true;
        }
        else
        {
            return false;
        }
    }

    /**
     * Un-registers an already registered observer. The observer will no longer be notified by the datastore.
     *
     *@param observer a const pointer to the DataStoreObserver object
     *@return true if the Data Store observer is unregistered, false if the observer was not previously registered
     */
    bool unregister_observer(DataStoreObserver * observer) throw()
    {
        CDMW_WRITER_LOCK_GUARD(m_observers_mutex);
        DataStoreObserverListIterator pos;

        pos = std::find(m_observers.begin(), m_observers.end(), observer);

        if (pos != m_observers.end())
        {
            m_observers.erase(pos);
            return true;
        }
        else
        {
            return false;
        }
    }

    /**
     * Registers an initialiser object to be notified when the coordinator datastore is initialising.
     * NOTA: Only one initialiser is allowed.
     *
     *@param initialiser a const pointer to a DataStoreInitialiser object
     *@return true if the Data Store initialiser is registered, false if failed to register it
     * (it has already been registered)
     */

    bool register_initialiser(DataStoreInitialiser * initialiser) throw()
    {
        if (!m_pInitialiser)
        {
            m_pInitialiser = initialiser;
            return true;
        }
        else
        {
            return false;
        }
    }
    
    /**
     * Un-registers an already registered initialiser. The initialiser will no longer be called by the datastore.
     *
     *@param initialiser a const pointer to the DataStoreInitialiser object
     *@return true if the Data Store initialiser is unregistered, false if the initialiser was not previously registered
     */
    bool unregister_initialiser(DataStoreInitialiser * initialiser) throw()
    {
        if (m_pInitialiser == initialiser)
        {
            m_pInitialiser = 0;
            return true;
        }
        else
        {
            return false;
        }
    }
    
    /**
     * Registers an backup observer to be notified whenever there is
     * an insertion, update, or removal of a record. In contrast to
     * the regular obsever a backup observer is called while the data
     * map mutex is held.
     *
     *@param observer a const pointer to a DataStoreObserver object
     *@return true if the Data Store observer is registered, false if
     *failed to register it (it has already been registered, or
     *exceeded maximum allowed number of observers ...)
     */
    bool register_backup_observer(DataStoreObserver * observer) throw()
    {
        CDMW_WRITER_LOCK_GUARD(m_backup_observers_mutex);
        DataStoreObserverListIterator pos;

        pos = std::find(m_backup_observers.begin(),
                        m_backup_observers.end(), observer);

        if (pos == m_backup_observers.end())
        {
            m_backup_observers.push_back(observer);
            return true;
        }
        else
        {
            return false;
        }
    }

    /**
     * Un-registers an already registered backup observer. The
     * observer will no longer be notified by the datastore.
     *
     *@param observer a const pointer to the DataStoreObserver object
     *@return true if the Data Store observer is unregistered, false
     *if the observer was not previously registered
     */
    bool unregister_backup_observer(DataStoreObserver * observer) throw()
    {
        CDMW_WRITER_LOCK_GUARD(m_backup_observers_mutex);
        DataStoreObserverListIterator pos;

        pos = std::find(m_backup_observers.begin(),
                        m_backup_observers.end(), observer);

        if (pos != m_backup_observers.end())
        {
            m_backup_observers.erase(pos);
            return true;
        }
        else
        {
            return false;
        }
    }

    /**
     * Return true if the DataStore contains a data with Object Id key
     *
     *@param oid is the Object Id key
     *@return true if the Data Store contains this data
     */
    bool contains(_Oid_in_type oid) throw();

    /**
     * Return the data with Object Id key
     *
     *@param oid is the Object Id key
     *@exception NotFound if no object found with this oid
     */
    _Data_retn_type 
    select(const _Oid_in_type oid) 
        throw (NotFoundException);

    /**
     * Remove from the DataStore the entry with the given oid key
     * Do it in the cohorts then localy (if no problem with cohorts)
     *
     * This operation will potentially block until updating replicas
     *
     *@param oid is the Object Id key
     */
    void remove(const _Oid_in_type oid) throw (NotFoundException,
                                               Cdmw::CommonSvcs::DataStore::StorageErrorException);
    
    /**
     * Remove from the local DataStore the entry with the given oid key
     *     
     *@param oid is the Object Id key
     */
    void remove_local(const _Oid_in_type oid) throw (NotFoundException);
    
    /**
     * insert in the DataStore the entry with the given oid key
     * do it in the cohorts then localy (if no problem with cohorts)
     *
     * This operation will potentially block until updating replicas
     *
     *@param oid is the Object Id key
     *@param data is the data associated with Oid key
     */
    void insert(const _Oid_in_type oid, _Data_in_type data)
        throw (AlreadyExistException,Cdmw::CommonSvcs::DataStore::StorageErrorException);

    /**
     * insert in the local DataStore the entry with the given oid key
     *
     *@param oid is the Object Id key
     *@param data is the data associated with Oid key
     */
    void insert_local(const _Oid_in_type oid, const _Data_in_type data)
        throw (AlreadyExistException);

    /**
     * update the DataStore entry with the given oid key
     * do it in the cohorts then localy (if no problem with cohorts)
     *
     * This operation will potentially block until updating replicas
     *
     *@param oid is the Object Id key
     *@param data is the data associated with Oid key
     */
    void update(const _Oid_in_type oid, const _Data_in_type data)
        throw (NotFoundException,Cdmw::CommonSvcs::DataStore::StorageErrorException);

    /**
     * update the local DataStore entry with the given oid key
     *
     *@param oid is the Object Id key
     *@param data is the data associated with Oid key
     */
    void update_local(const _Oid_in_type oid, const _Data_in_type data)
        throw (NotFoundException);

    /**
     * Give the size of the DataStore
     *
     *@return int the size
     */
    int size()
    {
        CDMW_READER_LOCK_GUARD(m_dataMapMutex);
        return m_data.size();
    }

    /**
     * Initialized the datastore in cold/hot restart.
     */
    void init(bool cold_restart) throw (Cdmw::InitErrorException);

    //  Part for link with transaction engine


    /**
     * process failure of a location
     *
     *@param location is the failed location
     */
    void on_failure(const std::string& location)
    {
        m_context.on_failure(location);
    }

    /**
     * Load state transferred from another DataStore
     *
     * @param recs sequence of change records
     */
    void load_state(const StateChangeRecords& recs)
        throw (NotFoundException, AlreadyExistException);

    /**
     * Validate a set of changes.
     *
     * @param recs sequence of state changes (inserts, updates and removes).
     */
    bool validate(const StateChangeRecords& recs);

    /**
     * Compare the oids for a set of changes.
     *
     * @result true if any oids match, false otherwise
     */
    bool oids_match(const CdmwFT::Transaction::StateChangeRecords& recs1,
                    const CdmwFT::Transaction::StateChangeRecords& recs2);

    

    /**
     * Wait for transactions in progress to complete.
     * Deprecated. Only for use by RecordIterator.
     */
    void wait_for_no_transactions_in_progress();

    inline bool is_activated()
    {
       return m_context.is_activated();
    }

      void hexdump(std::ostream & os) const;
      

protected:

    /**
     * execute function for each element of DataStore
     * NOTE: The implementation does not autorise the given function to modify the data
     *       Only Function with const parameter will compile
     *       (typical example: const std::pair<Oid, Data>& is OK)
     *
     *@param Functor f to execute
     */
    void do_for_each(Cdmw::CommonSvcs::DataStore::Functor<OID,DATA>& f);

private:

    /**
     * Applies a state changes (insert, update or remove) to the DataStore.
     *
     * @param rec a StateChangeRecord.
     */
    void apply_change(const StateChangeRecord& rec)
        throw (NotFoundException, AlreadyExistException);


    CORBA::ORB_var m_orb;
    PortableServer::POA_var m_poa;
    
    DataStoreContext& m_context;

    Cdmw::OrbSupport::Codec<OID> m_oid_codec;
    Cdmw::OrbSupport::Codec<DATA> m_data_codec; 

    PortableServer::ObjectId_var m_coordinator_id;
    PortableServer::ObjectId_var m_cohort_id;

    // The data in local memory
    DataMap m_data;

    // Current observers
    DataStoreObserverList m_observers;

    // Lock for observers list
    OsSupport::ReaderWriterLock m_observers_mutex;

    // Current backup observers
    DataStoreObserverList m_backup_observers;

    // Lock for backup observers list
    OsSupport::ReaderWriterLock m_backup_observers_mutex;

    // Initialiser
    DataStoreInitialiser* m_pInitialiser;

      
    // Mutex for accessing m_iter_map
    Cdmw::OsSupport::Mutex m_iter_map_mutex;

    // Iterator map
    RecordIteratorMap m_iter_map;

    // Mutex protection to R/W data
    // For simplicity we choose a global one for all data instances
    // (if we choosed to handle each Oid independently the lock should
    //  be global anyway when insert and remove)
    OsSupport::ReaderWriterLock m_dataMapMutex;    

    // if DataStore has to be registered in DataStoreRegistry
    bool m_register_data_store;

    /**
     * Default constructor.
     */
    DataStore();
    
    /**
     * Assignement operator is not allowed.
     */
    DataStore& operator=(const DataStore& rhs);

}; // End class DataStore

}  // End namespace FT
}  // End namespace Cdmw

#include <FaultTolerance/ftstatemanager/DataStoreInitialiser.hpp>
#include <FaultTolerance/ftstatemanager/DataStoreRecordIterator.hpp>

namespace Cdmw {
namespace FT {

//
// IMPLEMENTATION PART OF THE TEMPLATE
//


template <class OID, class DATA, class OIDCOMPARE>
DataStoreRecordIteratorBase*
DataStore<OID, DATA, OIDCOMPARE>::get_record_iterator
(const char * from_location)
   throw (CORBA::SystemException,
          CdmwFT::StateTransfer::InvalidDataStoreState)
{

    if (!this->is_activated())
    {
        // Not the primary
        throw CdmwFT::StateTransfer::InvalidDataStoreState();
    }

    CDMW_MUTEX_GUARD(m_iter_map_mutex);


    typename RecordIteratorMap::iterator pos;
    pos = m_iter_map.find(from_location);

    if (pos != m_iter_map.end ())
    {
        // Iterator already exists for this location
        throw CORBA::BAD_INV_ORDER(Cdmw::OrbSupport::BAD_INV_ORDERIteratorPresent,
                                   CORBA::COMPLETED_NO);
    }

    LocationList locations = m_context.get_locations();

    if (locations.find(from_location) != locations.end())
    {
        // DataStore already exists for this location
        throw CORBA::BAD_INV_ORDER(Cdmw::OrbSupport::BAD_INV_ORDERLocationAlreadyPresent,
                                   CORBA::COMPLETED_NO);
    }

    CDMW_READER_LOCK_GUARD(m_dataMapMutex);

    DataStoreRecordIterator* iter_impl =
       new DataStoreRecordIterator(*this, m_orb.in(), from_location,
                                   m_dataMapMutex);
    m_iter_map[from_location] = iter_impl;

    return iter_impl;
}

template <class OID, class DATA, class OIDCOMPARE>
void
DataStore<OID, DATA, OIDCOMPARE>::iterator_finished
(const char * from_location)
    throw (NotFoundException)
{
   std::string location(from_location);
   
   CDMW_MUTEX_GUARD(m_iter_map_mutex);
   typename RecordIteratorMap::iterator pos;
   pos = m_iter_map.find(location);
   if (pos != m_iter_map.end ())
   {
      delete (*pos).second;
      m_iter_map.erase(pos);
   } else {
      std::ostringstream ostr;
      ostr << "No DataStoreRecordIterator defined for location '" << location << "'";
      throw NotFoundException(ostr.str());
   }
}


template <class OID, class DATA, class OIDCOMPARE>
bool DataStore<OID, DATA, OIDCOMPARE>::contains(_Oid_in_type oid) throw ()
{
    CDMW_READER_LOCK_GUARD(m_dataMapMutex);
    return (m_data.find(oid) != m_data.end());
}



template <class OID, class DATA, class OIDCOMPARE>
typename DataStore<OID, DATA, OIDCOMPARE>::_Data_retn_type
DataStore<OID, DATA, OIDCOMPARE>::select(const _Oid_in_type oid) 
    throw (NotFoundException)
{
    CDMW_READER_LOCK_GUARD(m_dataMapMutex);
    DataMapConstIterator itData = m_data.find(oid);
    if (itData != m_data.end())
    {
        // make deep copy
        _Data_var_type result = itData->second;
        // return result
        return Cdmw::OrbSupport::type_traits<DATA>::_retn(result);
    }
    else
    {
        throw NotFoundException("The Oid does not exist");
    }
}


template <class OID, class DATA, class OIDCOMPARE>
void DataStore<OID, DATA, OIDCOMPARE>::remove(const _Oid_in_type oid) 
    throw (NotFoundException,Cdmw::CommonSvcs::DataStore::StorageErrorException)
{
    ModificationGuard mod_guard(m_context);

    if (m_context.get_locations().empty())
    {
        remove_local(oid);
    }
    else
    {
       CDMW_ASSERT(this->is_activated());

        if (this->is_activated())
        {
            CdmwFT::Transaction::StateChanges changes;

            changes.length(1);
            changes[0].dsid = this->get_dsid();
            changes[0].records.length(1);

            // remove in cohorts
            CORBA::OctetSeq_var oid_seq_var = m_oid_codec.encode(oid);
            changes[0].records[0].oid_seq = oid_seq_var;
            changes[0].records[0].change_data._d
                (CdmwFT::Transaction::DS_REMOVAL);
            
            bool doLocal = true;

            try
            {
                doLocal = m_context.request(changes);
            }
            catch (const ValidationFailureException&)
            {
                throw NotFoundException();
            }

            if (doLocal)
            {
                remove_local(oid);
            } else {
                throw Cdmw::CommonSvcs::DataStore::StorageErrorException();
            }
        }
    }
}

template <class OID, class DATA, class OIDCOMPARE>
void DataStore<OID, DATA, OIDCOMPARE>::remove_local(const _Oid_in_type oid)
    throw (NotFoundException)
{
    DataStoreObserverList observers;
    DataStoreObserverListIterator pos;

    {
        CDMW_WRITER_LOCK_GUARD(m_dataMapMutex);

        DataMapConstIterator itData = m_data.find(oid);
        if (itData != m_data.end())
        {
            m_data.erase(oid);
        }
        else
        {
            throw NotFoundException("DataStore: The Oid does not exist");
        }

        {
            CDMW_READER_LOCK_GUARD(m_backup_observers_mutex);
            observers = m_backup_observers;
        }

        for (pos = observers.begin(); pos != observers.end(); ++pos)
        {
            (*pos)->on_remove(oid);
        }
    }

    {
        CDMW_READER_LOCK_GUARD(m_observers_mutex);
        observers = m_observers;
    }

    for (pos = observers.begin(); pos != observers.end(); ++pos)
    {
        (*pos)->on_remove(oid);
    }
}

template <class OID, class DATA, class OIDCOMPARE>
void DataStore<OID, DATA, OIDCOMPARE>::insert(const _Oid_in_type oid,
                                              const _Data_in_type data) 
    throw (AlreadyExistException,Cdmw::CommonSvcs::DataStore::StorageErrorException)
{
    ModificationGuard mod_guard(m_context);

    if (m_context.get_locations().empty())
    {
        insert_local(oid, data);
    }
    else
    {
        CDMW_ASSERT(this->is_activated());

        if (this->is_activated())
        {
            CdmwFT::Transaction::StateChanges changes;

            changes.length(1);
            changes[0].dsid = this->get_dsid();
            changes[0].records.length(1);

            // insert in cohorts
            CORBA::OctetSeq_var oid_seq_var = m_oid_codec.encode(oid);
            changes[0].records[0].oid_seq = oid_seq_var;

            CORBA::OctetSeq_var data_seq_var = m_data_codec.encode(data);
            changes[0].records[0].change_data.data_seq(data_seq_var.in());
            changes[0].records[0].change_data._d
                (CdmwFT::Transaction::DS_INSERTION);

            bool doLocal = true;
    
            try
            {
                doLocal = m_context.request(changes);
            }
            catch (const ValidationFailureException&)
            {
                throw AlreadyExistException();
            }

            if (doLocal)
            {
                insert_local(oid, data);
            } else {
                throw Cdmw::CommonSvcs::DataStore::StorageErrorException();
            }
        }
    }
}

template <class OID, class DATA, class OIDCOMPARE>
void  DataStore<OID, DATA, OIDCOMPARE>::insert_local(const _Oid_in_type oid,
                                                     const _Data_in_type data) 
    throw (AlreadyExistException)
{
    DataStoreObserverList observers;
    DataStoreObserverListIterator pos;

    {
        CDMW_WRITER_LOCK_GUARD(m_dataMapMutex);

        DataMapConstIterator itData = m_data.find(oid);
        if (itData == m_data.end())
        {
            // store copy of data
            m_data[oid] = Cdmw::OrbSupport::type_traits<DATA>::_copy(data);
        }
        else
        {
            throw AlreadyExistException("DataStore: The oid already exist");
        }

        {
            CDMW_READER_LOCK_GUARD(m_backup_observers_mutex);
            observers = m_backup_observers;
        }

        for (pos = observers.begin(); pos != observers.end(); ++pos)
        {
            (*pos)->on_insert(oid, data);
        }
    }

    {
        CDMW_READER_LOCK_GUARD(m_observers_mutex);
        observers = m_observers;
    }

    for (pos = observers.begin(); pos != observers.end(); ++pos)
    {
        (*pos)->on_insert(oid, data);
    }
}

template <class OID, class DATA, class OIDCOMPARE>
void DataStore<OID, DATA, OIDCOMPARE>::update(const _Oid_in_type oid,
                                              const _Data_in_type data) 
    throw (NotFoundException,Cdmw::CommonSvcs::DataStore::StorageErrorException)
{
    ModificationGuard mod_guard(m_context);

    if (m_context.get_locations().empty())
    {
        update_local(oid, data);
    }
    else
    {
       CDMW_ASSERT(this->is_activated());

        if (this->is_activated())
        {
            CdmwFT::Transaction::StateChanges changes;

            changes.length(1);
            changes[0].dsid = this->get_dsid();
            changes[0].records.length(1);

            // update cohorts
            CORBA::OctetSeq_var oid_seq_var = m_oid_codec.encode(oid);
            changes[0].records[0].oid_seq = oid_seq_var;

            CORBA::OctetSeq_var data_seq_var = m_data_codec.encode(data);
            changes[0].records[0].change_data.data_seq(data_seq_var.in());
            changes[0].records[0].change_data._d
                (CdmwFT::Transaction::DS_UPDATE);

            bool doLocal = true;
            try
            {
                doLocal = m_context.request(changes);
            }
            catch (const ValidationFailureException&)
            {
                throw NotFoundException();
            }

            if (doLocal)
            {
                update_local(oid, data);
            } else {
                throw Cdmw::CommonSvcs::DataStore::StorageErrorException();
            }
        }
    }
}

template <class OID, class DATA, class OIDCOMPARE>
void DataStore<OID, DATA, OIDCOMPARE>::update_local(const _Oid_in_type oid,
                                                    const _Data_in_type data) 
    throw (NotFoundException)
{
    DataStoreObserverList observers;
    DataStoreObserverListIterator pos;

    {
        CDMW_WRITER_LOCK_GUARD(m_dataMapMutex);

        DataMapIterator itData = m_data.find(oid);
        if (itData != m_data.end())
        {
            // store copy of data
            itData->second = Cdmw::OrbSupport::type_traits<DATA>::_copy(data);
        }
        else
        {
            throw NotFoundException("DataStore: The Oid does not exist");
        }

        {
            CDMW_READER_LOCK_GUARD(m_backup_observers_mutex);
            observers = m_backup_observers;
        }

        for (pos = observers.begin(); pos != observers.end(); ++pos)
        {
            (*pos)->on_update(oid, data);
        }
    }

    {
        CDMW_READER_LOCK_GUARD(m_observers_mutex);
        observers = m_observers;
    }

    for (pos = observers.begin(); pos != observers.end(); ++pos)
    {
        (*pos)->on_update(oid, data);
    }
}

template <class OID, class DATA, class OIDCOMPARE>
void DataStore<OID, DATA, OIDCOMPARE>::do_for_each(Cdmw::CommonSvcs::DataStore::Functor<OID,DATA>& f)
{
    CDMW_READER_LOCK_GUARD(m_dataMapMutex);

    DataMapConstIterator itBegin = m_data.begin();
    DataMapConstIterator itEnd = m_data.end();
    // Security: we force call of std::for_each with const iterators
    Cdmw::CommonSvcs::DataStore::FunctorWrapper<OID,DATA> f__(f);
    std::for_each(itBegin, itEnd, f__);
}

template <class OID, class DATA, class OIDCOMPARE>
void DataStore<OID, DATA, OIDCOMPARE>::init(bool cold_restart) 
   throw (Cdmw::InitErrorException) 
{
    CDMW_WRITER_LOCK_GUARD(m_dataMapMutex);

    if (m_pInitialiser)
    {
        DataInsertIterator iter(m_data, m_data.begin());

        m_pInitialiser->on_init(iter, cold_restart);
    }
}

template <class OID, class DATA, class OIDCOMPARE>
void
DataStore<OID, DATA, OIDCOMPARE>::load_state(const StateChangeRecords& recs)
    throw (NotFoundException, AlreadyExistException)
{
    for (CORBA::ULong i = 0; i < recs.length(); i++)
    {
        apply_change(recs[i]);
    }
}


template <class OID, class DATA, class OIDCOMPARE>
bool
DataStore<OID, DATA, OIDCOMPARE>::validate(const StateChangeRecords& recs)
{
    CDMW_READER_LOCK_GUARD(m_dataMapMutex);
    for (CORBA::ULong i = 0; i < recs.length(); i++)
    {
        _Oid_var_type oid = Cdmw::OrbSupport::type_traits<OID>::_default_value();

        m_oid_codec.decode(recs[i].oid_seq,
                           Cdmw::OrbSupport::type_traits<OID>::_inout(oid));
        DataMapConstIterator itData = m_data.find(oid);
        bool oid_present = (itData != m_data.end());

        switch(recs[i].change_data._d())
        {
            case CdmwFT::Transaction::DS_INSERTION:
            {
                if (oid_present)
                {
                    return false;
                }
                break;
            }
            case CdmwFT::Transaction::DS_UPDATE:
            case CdmwFT::Transaction::DS_REMOVAL:
            {
                if (!oid_present)
                {
                    return false;
                }
                break;
            }
        }
    }

    return true;
}


template <class OID, class DATA, class OIDCOMPARE>
bool
DataStore<OID, DATA, OIDCOMPARE>::oids_match
(const CdmwFT::Transaction::StateChangeRecords& recs1,
 const CdmwFT::Transaction::StateChangeRecords& recs2)
{
    typedef std::set<_Oid_var_type, OIDCOMPARE> OidSet;
    OidSet set1;

    for (CORBA::ULong i = 0; i < recs1.length(); i++)
    {
        _Oid_var_type oid1 =
            Cdmw::OrbSupport::type_traits<OID>::_default_value();
        m_oid_codec.decode(recs1[i].oid_seq,
                           Cdmw::OrbSupport::type_traits<OID>::_inout(oid1));
        set1.insert(oid1);
    }

    for (CORBA::ULong j = 0; j < recs2.length(); j++)
    {
        _Oid_var_type oid2 =
            Cdmw::OrbSupport::type_traits<OID>::_default_value();
        m_oid_codec.decode(recs2[j].oid_seq,
                           Cdmw::OrbSupport::type_traits<OID>::_inout(oid2));

        if (set1.find(oid2) != set1.end())
        {
            return true;
        }
    }

    return false;
}


template <class OID, class DATA, class OIDCOMPARE>
void
DataStore<OID, DATA, OIDCOMPARE>::apply_change(const StateChangeRecord& rec)
    throw (NotFoundException, AlreadyExistException)
{
    _Oid_var_type oid = Cdmw::OrbSupport::type_traits<OID>::_default_value();

    m_oid_codec.decode(rec.oid_seq,
                       Cdmw::OrbSupport::type_traits<OID>::_inout(oid));

    switch(rec.change_data._d())
    {
        case CdmwFT::Transaction::DS_INSERTION:
        {
            _Data_var_type data =
                Cdmw::OrbSupport::type_traits<DATA>::_default_value();
            m_data_codec.decode(rec.change_data.data_seq(),
                                Cdmw::OrbSupport::type_traits<DATA>::_inout(data));
            insert_local(Cdmw::OrbSupport::type_traits<OID>::_in(oid), Cdmw::OrbSupport::type_traits<DATA>::_in(data));
            break;
        }
        case CdmwFT::Transaction::DS_UPDATE:
        {
            _Data_var_type data =
                Cdmw::OrbSupport::type_traits<DATA>::_default_value();
            m_data_codec.decode(rec.change_data.data_seq(),
                                Cdmw::OrbSupport::type_traits<DATA>::_inout(data));
            update_local(Cdmw::OrbSupport::type_traits<OID>::_in(oid), Cdmw::OrbSupport::type_traits<DATA>::_in(data));
            break;
        }
        case CdmwFT::Transaction::DS_REMOVAL:
        {
            remove_local(Cdmw::OrbSupport::type_traits<OID>::_in(oid));
            break;
        }
    }
}


template <class OID, class DATA, class OIDCOMPARE>
void DataStore<OID, DATA, OIDCOMPARE>::wait_for_no_transactions_in_progress()
{
    if ((m_context.get_locations().size() > 0)
        && this->is_activated())
    {
        m_context.wait_for_no_transactions_in_progress();
    }
}


template <class OID, class DATA, class OIDCOMPARE>
void DataStore<OID, DATA, OIDCOMPARE>::hexdump(std::ostream & os) const
{
   // Follow following pattern
   // DUMP FT DATASTORE [123] 
   // DATASTORE ID
   // OID type: (tyepeinfo, repid?)
   // TYPE type: (tyepeinfo, repid?)
   // BEGIN DS[123] 
   // TIME/DATE
   // NB RECORDS
   // STATUS 
   // BEGIN 
   // ==> RECORD [1]
   // ==> RECORD [2]
   // ==> RECORD [3]
   // ...
   // FT DATASTORE ID # dsid
   // END  DS[123] 
   CDMW_READER_LOCK_GUARD(const_cast<OsSupport::ReaderWriterLock& >(m_dataMapMutex));
   const std::size_t NB_RECORDS = m_data.size();
   os << "FT DATASTORE [" << this->get_dsid() << "] ==>\n"
      << "--------------------------------------------------------------\n"
      << "DSID       :'" << this->get_dsid() << "'\n"
      << "OID TYPE   :'" << FTLogger::Get_typename(typeid(OID)) << "'\n"
      << "DATA TYPE  :'" << FTLogger::Get_typename(typeid(DATA)) << "'\n"
      << "NB RECORDS :'" << NB_RECORDS << "'\n"
      << "--------------------------------------------------------------\n";
   
   std::size_t idx = 0;
   for (DataMapConstIterator iter = m_data.begin(); iter != m_data.end(); ++iter)
   {
      ++idx;
      os << "RECORD [" << idx << "/" << NB_RECORDS << "]\n";
      
      CORBA::OctetSeq_var oid_seq_var = 
         const_cast<Cdmw::OrbSupport::Codec<OID>& >(m_oid_codec).encode(Cdmw::OrbSupport::type_traits<OID>::_in(iter->first));
      FTLogger::HexDump(oid_seq_var.in(),
                        "OID", os );
      CORBA::OctetSeq_var data_seq_var = 
         const_cast<Cdmw::OrbSupport::Codec<DATA>& >(m_data_codec).encode(Cdmw::OrbSupport::type_traits<DATA>::_in(iter->second));
      FTLogger::HexDump(data_seq_var.in(),
                        "DATA", os );
   }
}

}  // End namespace FT
}  // End namespace Cdmw

#endif

