/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO 
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2009. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

#ifndef INCL_CDMW_FT_FTSTATEMANAGER_DATASTORECONTEXT_HPP
#define INCL_CDMW_FT_FTSTATEMANAGER_DATASTORECONTEXT_HPP

#include <FaultTolerance/idllib/CdmwFTTPCCohort.stub.hpp>
#include <FaultTolerance/idllib/CdmwFTTPCCoordinator.stub.hpp>
#include <FaultTolerance/idllib/CdmwFTStateTransfer.stub.hpp>
#include <FaultTolerance/ftstatemanager/TransactionEngineRef.hpp>
#include <FaultTolerance/ftstatemanager/ValidationFailureException.hpp>
#include <FaultTolerance/ftstatemanager/RecordIterator_impl.hpp>
#include <FaultTolerance/ftstatemanager/Scheduler.hpp>
#include <Foundation/common/AlreadyExistException.hpp>
#include <Foundation/common/NotFoundException.hpp>
#include <Foundation/ossupport/OS.hpp>
#include <Foundation/orbsupport/OrbSupport.hpp>
#include "orbsvcs/orbsvcs/PortableGroup/GOA.h"
#include <list>
#include <FaultTolerance/ftstatemanager/Timer.hpp>

namespace Cdmw {
namespace FT {

const unsigned short DEFAULT_NB_WORKER_TREADS = 4;
const CORBA::ULong DEFAULT_CONTEXT_ITERATOR_INCREMENT = 100;
const CORBA::ULong DEFAULT_CONTEXT_FREEZE_THRESHOLD = 100;
const OsSupport::OS::Timeval DEFAULT_CONTEXT_FREEZE_TIMEOUT(5L, 0L);
    
class TransactionEngineCoordinator;
class TransactionEngineCohort;
class TransactionEngine;

class DataStoreContext : public Timer_Event_Handler<void>
{
    typedef Cdmw::Common::AlreadyExistException AlreadyExistException;
    typedef Cdmw::Common::NotFoundException NotFoundException;

    typedef std::map<std::string, TransactionEngineRef>
        TransactionEngineRefMap;

    typedef std::map<std::string, RecordIterator_impl*, std::less<std::string> >
        RecordIteratorMap;

    typedef std::map<DsIdentifier, DataStoreBase*, std::less<DsIdentifier> >
        DataStoreMap;
    typedef Timer<void> Timer;
    typedef Timer::TimerId TimerId;
        
public:
    typedef std::list<DsIdentifier> DataStoreIdList;
    typedef std::set<std::string> LocationSet;
    typedef std::map<DsIdentifier, LocationSet> LocationSetMap;

    /**
     * Constructor of a DataStoreContext using multicast state transfer.
     *
     *@param orb the orb reference
     *@param poa the poa reference
     *@param port the port for the multicast address. the port and the port+1 will be 
     *            used to the transfert of data between coordinator and cohort
     *@param domain the domain for the multicast address.
     *@param ip_address the ip address for the multicast address.
     *@param max_transaction_in_progress the max number of transaction in progress 
     *@param max_transaction_done the max number of transaction done
     *@param cohort_timeout the cohort timeout (timeout between a prepared_ operation and
     * the commit)
     *@param coordinator_timeout (timeout between the prepared_ operation and a yes/no, 
     *                           and a commit/roolback operation and a end
     *@param nbthreads the number of worker threads.
     *@param iterator_increment the size of sequence to request when copying
     * state from a primary. 
     *@param freeze_threshold threshold of the count of state change records
     * remaining to transfer that below which an iterator will freeze the
     * DataStore.
     *@param freeze_timeout Freeze timeout for cancelling freeze. 
     *
     *@exception CORBA::SystemException Any CORBA system exception that
     * may be raised by the underlying ORB.
     */ 
    DataStoreContext(CORBA::ORB_ptr          orb,
                     PortableGroup::GOA_ptr  poa,
                     const std::string&      port,
                     const std::string&      domain,
                     const std::string&      ip_address,
                     int                     max_transaction_in_progress,
                     int                     max_transaction_done,
                     OsSupport::OS::Timeval  cohort_timeout,
                     OsSupport::OS::Timeval  coordinator_timeout,
                     unsigned short          nbthreads = DEFAULT_NB_WORKER_TREADS,
                     CORBA::ULong            iterator_increment = DEFAULT_CONTEXT_ITERATOR_INCREMENT,
                     CORBA::ULong            freeze_threshold = DEFAULT_CONTEXT_FREEZE_THRESHOLD,

                     OsSupport::OS::Timeval  freeze_timeout = DEFAULT_CONTEXT_FREEZE_TIMEOUT)
        throw(CORBA::SystemException);

    /**
     * Constructor of a DataStoreContext using point to point state transfer.
     *
     *@param orb the orb reference
     *@param poa the poa reference
     *@param max_transaction_in_progress the max number of transaction in progress 
     *@param max_transaction_done the max number of transaction done
     *@param cohort_timeout the cohort timeout (timeout between a prepared_ operation and
     * the commit)
     *@param coordinator_timeout (timeout between the prepared_ operation and a yes/no, 
     *                           and a commit/roolback operation and a end
     *@param nbthreads the number of worker threads.
     *@param iterator_increment the size of sequence to request when copying
     * state from a primary. 
     *@param freeze_threshold threshold of the count of state change records
     * remaining to transfer that below which an iterator will freeze the
     * DataStore.
     *@param freeze_timeout Freeze timeout for cancelling freeze. 
     *
     *@exception CORBA::SystemException Any CORBA system exception that
     * may be raised by the underlying ORB.
     */ 
    DataStoreContext(CORBA::ORB_ptr          orb,
                     PortableGroup::GOA_ptr  poa,
                     int                     max_transaction_in_progress,
                     int                     max_transaction_done,
                     OsSupport::OS::Timeval  cohort_timeout,
                     OsSupport::OS::Timeval  coordinator_timeout,
                     unsigned short          nbthreads = DEFAULT_NB_WORKER_TREADS,
                     CORBA::ULong            iterator_increment = DEFAULT_CONTEXT_ITERATOR_INCREMENT,
                     CORBA::ULong            freeze_threshold = DEFAULT_CONTEXT_FREEZE_THRESHOLD,

                     OsSupport::OS::Timeval  freeze_timeout = DEFAULT_CONTEXT_FREEZE_TIMEOUT)
        throw(CORBA::SystemException);

    /**
     * Destructor.
     */
    virtual ~DataStoreContext();

    /**
     * Register a DataStoreBase.
     *
     *@param ds the DataStoreBase to be registered
     *
     *@exception AlreadyExistException if a DataStoreBase with same ID is
     *           already registered.
     */
    void register_datastore(DataStoreBase* ds) 
        throw (AlreadyExistException);

    /**
     * Unregister a DataStoreBase.
     *
     *@param dsid the ID of the DataStoreBase to be unregistered
     *
     *@exception NotExistException if no DataStoreBase with specified ID was found
     */
    void unregister_datastore(const DsIdentifier& dsid)
        throw (NotExistException);

    /**
     * Return the DataStore with the specified identifier.
     *
     *@return the requested DataStore
     */
    DataStoreBase* get_datastore(const DsIdentifier& id);

    DataStoreIdList get_datastore_ids();
    
    /**
     * The data store becomes the one of the master
     */
    void activate();

    /**
     * The data store becomes the one of the slave
     */
    void deactivate();

    /**
     * Purpose:
     * <p> Ask the cohorts to apply a change
     *
     *@param oid is the Object Id key
     */
    bool request(const CdmwFT::Transaction::StateChanges& change)
        throw (InternalErrorException, ValidationFailureException);

      /**
       * Wait for transactions in progress to complete.
       * Deprecated. Only for use by RecordIterator.
       */
      void wait_for_no_transactions_in_progress();

      /**
       * Wait for modifications in progress to complete.
       * Deprecated. Only for use by RecordIterator.
       */
//      void wait_for_no_modifications_in_progress();
      void inc_modification_in_progress();
      void dec_modification_in_progress();

    /**
     * process failure of a location
     *
     *@param location is the failed location
     */
    void on_failure(const std::string& location);

    /**
     * Add the references to the Cohort and the Coordinator objects of another new location.
     * This operation is used in case of point-to-point protocol.
     *
     *@param location is the new location
     *@param coordinator is the reference to the Coordinator object
     *@param cohort is the reference to the Cohort object
     */
    void add_ref_for_location(const std::string& location,
                      CdmwFT::Transaction::TPCCoordinator_ptr coordinator,
                      CdmwFT::Transaction::TPCCohort_ptr cohort)
        throw (AlreadyExistException);

    /**
     * Return the reference to the local Coordinator object
     */
    CdmwFT::Transaction::TPCCoordinator_ptr get_tpc_coordinator() const;
    
    /**
     * Return the reference to the local Cohort object
     */
    CdmwFT::Transaction::TPCCohort_ptr get_tpc_cohort() const;

    /**
     * Return the references to the Coordinator and the Cohort object of a location
     *
     *@param location is the location
     *@result a structure with the references to the Coordinator and the Cohort
     */
    TransactionEngineRef
    get_ref_for_location(const std::string& location) const
        throw (NotFoundException);
    
    /**
    * set coordinator location
    *
    *@param location 
    */
    void set_coordinator_location(const std::string& location)
        throw ();

    /**
    * get coordinator location
    *
    *@param location 
    */
    std::string get_coordinator_location() const
        throw ();

    CdmwFT::Transaction::TPCCoordinator_ptr get_coordinator_ref() const;

    CdmwFT::Transaction::TPCCohort_ptr get_cohort_ref() const;
    
    inline bool is_activated() const
    {
        return m_state == DSC_STATE_MASTER;
    }
    

    /**
     * Returns an iterator that provides state information necessary to copy
     * a DataStore.
     *
     * @param from_location is the name of the location asking the records.
     * @param datastore_ids Ids of DataStores to be transferred.
     *
     * @exception InvalidDataStoreState is raised if the data store is not the 
     * coordinator
     */
    virtual ::CdmwFT::StateTransfer::RecordIterator_ptr get_record_iterator
    (const char * from_location, 
     const CdmwFT::StateTransfer::DataStoreIDs& datastore_ids,
     CdmwFT::Transaction::TPCCoordinator_ptr coordinator,
     CdmwFT::Transaction::TPCCohort_ptr cohort)
        throw (CORBA::SystemException,
               CdmwFT::StateTransfer::InvalidDataStoreState);

    /*
     * Transfer state from primary.
     *
     * @param primary_datastore_group source DataStoreGroup.
     * @param coordinator_location is the location of the coordinator (primary).
     * @param from_location is the name of the location asking the records.
     * @param datastore_ids Ids of DataStores to be transferred.
     */
    void update_with_coordinator
    (CdmwFT::StateTransfer::DataStoreGroup_ptr primary_datastore_group,
     const std::string & coordinator_location,
     const std::string & location,
     const CdmwFT::StateTransfer::DataStoreIDs& datastore_ids)
        throw (CORBA::SystemException,
               CdmwFT::StateTransfer::InvalidDataStoreState);

    /**
     * Validate a set of changes against DataStores
     *
     * @result true if changes valid, false if any change is invalid
     */
    bool validate(const CdmwFT::Transaction::StateChanges& changes);

    /**
     * Compare the oids for a set of changes.
     *
     * @result true if any oids for the same DataStore match, false otherwise
     */
    bool oids_match(const CdmwFT::Transaction::StateChanges& changes1,
                    const CdmwFT::Transaction::StateChanges& changes2);

    static void On_Failure(const std::string& location)
        throw (NotFoundException);

    /**
    * add one location (to the ack check) to all DataStores
    *
    *@param location the new location
    */
    void add_location(const std::string& location) 
        throw (AlreadyExistException);

    /**
    * set locations (to the ack check)
    *
    *@param new locations
    */
    void set_locations(const LocationSet& locations);



    void hexdump(std::ostream & os) const;

      /**
       * Stop Transaction Engine's scheduler
       *
       */
      void stop();

    /**
    * Get locations
    */
    LocationSet get_locations();

//     // Check if element is contained 
//     bool contains(const DsIdentifier& dsid, const std::string& location)
//         throw (NotFoundException);

    // The mutex used to maintain cohorts list coherency
    Cdmw::OsSupport::ReaderWriterLock m_cohorts_mutex;

      // Operations for freezing all DataStores for terminating backup insertion
      void freeze(const std::string& location);
      // Returns true if successfull, false when freeze timeout occured in the meantime!
      bool unfreeze(const std::string& location,
                    CdmwFT::Transaction::TPCCoordinator_ptr coordinator,
                    CdmwFT::Transaction::TPCCohort_ptr cohort);
      void on_freeze_timeout();
//      void freeze_check();
      /**
       * Freeze timer callback method.
       */
      int handle_timeout(const void*);
      
    
private:
    /**
     * Assignement operator is not allowed.
     */
    DataStoreContext& operator=(const DataStoreContext& rhs);

    CORBA::ORB_var m_orb;
    PortableGroup::GOA_var m_poa;

    DataStoreMap m_datastore_map;
    
    TransactionEngineCoordinator * m_pTransactionCoordinator;
    TransactionEngineCohort * m_pTransactionCohort;
    TransactionEngine * m_pTransactionEngine;

    CdmwFT::Transaction::TPCCoordinator_var m_coordinator;
    CdmwFT::Transaction::TPCCohort_var m_cohort;

    PortableServer::ObjectId_var m_coordinator_id;
    PortableServer::ObjectId_var m_cohort_id;

    // corbaloc
    std::string m_coordinator_corbaloc;
    std::string m_cohort_corbaloc;

    TransactionEngineRefMap m_transactionEngineRefMap;

    enum DataStoreContextState { DSC_STATE_NONE, 
                                 DSC_STATE_MASTER,
                                 DSC_STATE_SLAVE };
    DataStoreContextState m_state;

    /**
     * coordinator location
     */
    std::string m_coordinator_location;

    /**
     * local location
     */
    std::string m_location;

    /**
     *
     */
    bool m_is_miop;

      //LocationSetMap m_cohortsMap;

    LocationSet    m_cohortsSet;

    // Iterator map
    RecordIteratorMap m_iter_map;

    // Size of sequence used by update_with_coordinator to transfer state
    CORBA::ULong m_iterator_increment;

    // Size of change set for freezing
    CORBA::ULong m_freeze_threshold;

    // Freeze timeout for cancelling freeze
    OsSupport::OS::Timeval m_freeze_timeout;

    // Ace Scheduler for (update, remove, insert) requests 
    Scheduler::Scheduler* m_reqScheduler;

    // Count of multicast DataStoreContexts for miop group id
    static int M_context_count;

    // the number of modifications in progress (insert, update and remove)
    unsigned int m_modifications_in_progress;

    // Mutex for number of modifications in progress
    // Cdmw::OsSupport::Mutex m_modifications_mutex;
    
    // Condition variable for waiting for modifications to complete
    //Cdmw::OsSupport::Condition m_modifications_condition;

    // -------------------------------------------------------------
    // TODO: Check wether a separate mutex is necessary
    // 
    // Freeze timeout timer
    Timer m_freeze_timer;

    // Mutex for freezing
    Cdmw::OsSupport::Mutex m_freeze_mutex;

    // Condition variable for freezing and for waiting for modifications to complete
    Cdmw::OsSupport::Condition m_freeze_condition;

    // List of frozen iterator locations
    LocationSet m_freeze_locations;

      TimerId m_freeze_timer_id;
    // -------------------------------------------------------------

};
/**
 * Guard class to track number of outstanding modifications.
 * This is usefull for closing ending insertion.
 */
class ModificationGuard
  {
  public:
    ModificationGuard(DataStoreContext& dc)
      : m_context(dc)
    {
      m_context.inc_modification_in_progress();
    }
    ~ModificationGuard()
    {
      m_context.dec_modification_in_progress();
    }
  private:
    DataStoreContext& m_context;
  };
}  // End namespace FT
}  // End namespace Cdmw

#endif
