/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO 
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2009. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

#include "ftstatemanager/DataStoreContext.hpp"
#include <FaultTolerance/ftstatemanager/TransactionEngineCohort.hpp>
#include <FaultTolerance/ftstatemanager/TransactionEngineCoordinator.hpp>
#include <sstream>

#include <FaultTolerance/ftcommon/FTLogger.hpp>
#include <Foundation/ossupport/SystemProperties.hpp>
#include <Foundation/ossupport/Signals.hpp>
#include <FaultTolerance/ftcommon/FTSystemProperties.hpp>

// Anonymous namespace
namespace {
   const int BACKUP_INSERTION_EXIT_FAILURE (-1);
   const std::string DataStoreContext_PROPERTY_NAME("Cdmw.FaultTolerance.DataStoreContext");

#define STORE_DATASTORE_CONTEXT_STATE(X) \
   do { \
      std::string s("NONE"); \
      if (X == Cdmw::FT::DataStoreContext::DSC_STATE_MASTER) s = "MASTER"; \
      else if (X == Cdmw::FT::DataStoreContext::DSC_STATE_SLAVE) s = "SLAVE"; \
      Cdmw::OsSupport::SystemProperties::Set_Property( \
         DataStoreContext_PROPERTY_NAME + ".State", s); \
   } while (0)  

   void dump_changes(const CdmwFT::Transaction::StateChanges& changes,
                     std::ostream & os)
   {
      // Print number of state changes
      os << "STATE CHANGES size=" << changes.length() << "\n";
      os << "==== BEGIN ==================================================\n";
      for (CORBA::ULong i = 0; i < changes.length(); ++i) {
         
         CdmwFT::Transaction::DataStoreID dsid = changes[i].dsid;
         // Print DS Identifier
         os << "STATE CHANGE [" << i << "] DataStore ID [" << dsid << "] "
            << changes[i].records.length() << " records\n";
         // Print number of records
         
         for (CORBA::ULong j = 0; j < changes[i].records.length(); ++j) {
            os << "RECORD [" << j << "]\n";
            // Dump oid_seq changes[i].records[j].oid_seq
            
            Cdmw::FT::FTLogger::HexDump(changes[i].records[j].oid_seq,
                                        "OID", os );
            
            // Print StateChangeData changes[i].records[j].change_data
            // enum StateChangeKind { DS_INSERTION, DS_UPDATE, DS_REMOVAL };
            switch(changes[i].records[j].change_data._d()) {
               case CdmwFT::Transaction::DS_INSERTION:
                  Cdmw::FT::FTLogger::HexDump(changes[i].records[j].change_data.data_seq(),
                                              "INSERTION", os);
                  break;
               case CdmwFT::Transaction::DS_UPDATE:
                  Cdmw::FT::FTLogger::HexDump(changes[i].records[j].change_data.data_seq(),
                                              "UPDATE", os);
                  break;
               case CdmwFT::Transaction::DS_REMOVAL:
                  os << "REMOVAL\n";
            }
         }
      }
      os << "==== END   ==================================================\n";
   }

   class SIGUSER1_Handler: public Cdmw::OsSupport::SignalHandler
   {
      public:
         SIGUSER1_Handler(Cdmw::FT::DataStoreContext * dsc) 
            : m_dsc(dsc)
         {
            CDMW_ASSERT(dsc != 0);
         }
         virtual ~SIGUSER1_Handler()
         {
            m_dsc = 0;
         }
         int handle_signal(int sig) throw()
         {
            Cdmw::Common::Properties* sp = Cdmw::OsSupport::SystemProperties::Instance();
            
            std::cout << "\n==================================================================\n"
                      << "\nCARDAMOM Fault Tolerance \n"
                      << "\n==================================================================\n"
                      << "\n======================\n"
                      << "System Propeties:"
                      << "\n======================\n";
            sp->store(std::cout);
            std::cout << "\n======================\n"
                      << "HexDump of all datastores:"
                      << "\n======================\n";
            m_dsc->hexdump(std::cout);
            return 0;
         };
      private:
         Cdmw::FT::DataStoreContext * m_dsc;
   };



}; // End of anonymous namespace

namespace Cdmw {
namespace FT {

NotFoundException::NotFoundException(const std::string& exceptionInfo)
    : Exception( Exception::SAFE, exceptionInfo )
{}

AlreadyExistException::AlreadyExistException(const std::string& exceptionInfo)
    : Exception( Exception::SAFE, exceptionInfo )
{}

int DataStoreContext::M_context_count = 0;

DataStoreContext::DataStoreContext
(CORBA::ORB_ptr          orb,
 PortableGroup::GOA_ptr poa,
 const std::string&      port,
 const std::string&      domain,
 const std::string&      ip_address,
 int                     max_transaction_in_progress,
 int                     max_transaction_done,
 OsSupport::OS::Timeval  cohort_timeout,
 OsSupport::OS::Timeval  coordinator_timeout,
 unsigned short          nbthreads,
 CORBA::ULong            iterator_increment,
 CORBA::ULong            freeze_threshold,
 OsSupport::OS::Timeval  freeze_timeout)
    throw(CORBA::SystemException)
    : m_orb(CORBA::ORB::_duplicate(orb)),
      m_poa(PortableGroup::GOA::_duplicate(poa)),
      m_state(DSC_STATE_NONE),
      m_is_miop(true),
      m_iterator_increment(iterator_increment),
      m_freeze_threshold(freeze_threshold),
      m_freeze_timeout(freeze_timeout),
      m_reqScheduler(0),
      m_modifications_in_progress(0),
      m_freeze_condition(m_freeze_mutex),
      m_freeze_timer_id(-1)
{
   STORE_DATASTORE_CONTEXT_STATE(m_state);
    M_context_count++;

    int cohort_id = M_context_count * 2;
    int coordinator_id = cohort_id + 1;

    // set the port to string
    std::ostringstream buffer1;
    buffer1 << "corbaloc:miop:1.0@1.0-" << domain << "-" << cohort_id << "/"
                << ip_address << ":" << port;

    std::string cohort_corbaloc = buffer1.str();

    std::ostringstream buffer2;
    buffer2 << "corbaloc:miop:1.0@1.0-" << domain << "-" << coordinator_id
            << "/" << ip_address << ":" << port;

    std::string coordinator_corbaloc = buffer2.str();

    m_coordinator_corbaloc = coordinator_corbaloc;
    m_cohort_corbaloc = cohort_corbaloc;

    
    m_pTransactionCoordinator = 
        new TransactionEngineCoordinator(*this,
                                         m_orb.in(),
                                         m_poa.in(),
                                         cohort_corbaloc,
                                         max_transaction_in_progress,
                                         max_transaction_done,
                                         cohort_timeout,
                                         nbthreads);

    // create the object
    m_pTransactionCohort =
        new TransactionEngineCohort(*this,
                                    m_orb.in(),
                                    m_poa.in(),
                                    coordinator_corbaloc,
                                    max_transaction_in_progress,
                                    max_transaction_done,
                                    coordinator_timeout);

    m_reqScheduler = new Scheduler::Scheduler(m_orb.in(), m_pTransactionCoordinator, nbthreads);
    m_reqScheduler->start();

    m_pTransactionCoordinator->set_scheduler(m_reqScheduler);
    m_pTransactionCohort->set_scheduler(m_reqScheduler);
        
    
    // create id for the cohort
    CORBA::Object_var obj =
        m_orb->string_to_object(coordinator_corbaloc.c_str());
    m_coordinator =
        CdmwFT::Transaction::TPCCoordinator::_unchecked_narrow(obj.in());
    m_coordinator_id = m_poa->create_id_for_reference(obj.in());

    obj = m_orb->string_to_object(cohort_corbaloc.c_str());
    m_cohort = CdmwFT::Transaction::TPCCohort::_unchecked_narrow(obj.in());
    m_cohort_id = m_poa->create_id_for_reference(obj.in());

    m_poa->activate_object_with_id (m_coordinator_id.in (),m_pTransactionCoordinator);

    //        CORBA_Object_var obj = m_orb->string_to_object(m_coordinator_corbaloc.c_str());
    // TAO BUG : the disassociate_reference_with_id is not implemented
    //        m_poa->disassociate_reference_with_id(obj.in(), m_coordinator_id.in ());

    // activate the cohort
    m_poa->activate_object_with_id (m_cohort_id.in (),m_pTransactionCohort);
    m_pTransactionEngine = m_pTransactionCohort;
   
    m_pTransactionCoordinator->deactivate();
    m_pTransactionCohort->deactivate();

    Cdmw::Common::Properties* sp = Cdmw::OsSupport::SystemProperties::Instance();
    if (!sp->get_property(CDMW_FT_INSTALL_HEXDUMP_SIGUSR1_HANDLER).empty())
       Cdmw::OsSupport::Signals::Instance()->register_handler(SIGUSR1, new SIGUSER1_Handler(this));
}

DataStoreContext::DataStoreContext
(CORBA::ORB_ptr          orb,
 PortableGroup::GOA_ptr poa,
 int                     max_transaction_in_progress,
 int                     max_transaction_done,
 OsSupport::OS::Timeval  cohort_timeout,
 OsSupport::OS::Timeval  coordinator_timeout,
 unsigned short          nbthreads,
 CORBA::ULong            iterator_increment,
 CORBA::ULong            freeze_threshold,
 OsSupport::OS::Timeval  freeze_timeout)
    throw(CORBA::SystemException)
    : m_orb(CORBA::ORB::_duplicate(orb)),
      m_poa(PortableGroup::GOA::_duplicate(poa)),
      m_state(DSC_STATE_NONE),
      m_is_miop(false),
      m_iterator_increment(iterator_increment),
      m_freeze_threshold(freeze_threshold),
      m_freeze_timeout(freeze_timeout),
      m_reqScheduler(0),
      m_modifications_in_progress(0),
      m_freeze_condition(m_freeze_mutex),
      m_freeze_timer_id(-1)
{
   STORE_DATASTORE_CONTEXT_STATE(m_state);
    m_coordinator_location = DataStoreBase::Get_Location_Name();
    Cdmw::OsSupport::SystemProperties::Set_Property(
       DataStoreContext_PROPERTY_NAME + ".CoordinatorLocation",
       m_coordinator_location);
    m_location = DataStoreBase::Get_Location_Name();
    Cdmw::OsSupport::SystemProperties::Set_Property(
       DataStoreContext_PROPERTY_NAME + ".Location",
       m_location);

    // create the object
    m_pTransactionCohort =
        new TransactionEngineCohort(*this,
                                    m_orb.in(),
                                    m_poa.in(),
                                    "",
                                    max_transaction_in_progress,
                                    max_transaction_done,
                                    coordinator_timeout);
    
    m_pTransactionCoordinator = 
        new TransactionEngineCoordinator(*this,
                                         m_orb.in(),
                                         m_poa.in(),
                                         "",
                                         max_transaction_in_progress,
                                         max_transaction_done,
                                         cohort_timeout,
                                         nbthreads);

    m_reqScheduler = new Scheduler::Scheduler(m_orb.in(), m_pTransactionCoordinator, nbthreads);
    m_reqScheduler->start();

    m_pTransactionCoordinator->set_scheduler(m_reqScheduler);
    m_pTransactionCohort->set_scheduler(m_reqScheduler);

    m_coordinator = m_pTransactionCoordinator->_this();
    CdmwFT::Transaction::TPCCohort_var m_cohort_temp =
        m_pTransactionCohort->_this();

    TimeBase::TimeT relative_timeout =
        (cohort_timeout.seconds * 1000000 + cohort_timeout.microseconds) *10 ;
    CORBA::Any relative_timeout_as_any;
    relative_timeout_as_any <<= relative_timeout;
    CORBA::PolicyList policy_list;
    policy_list.length(1);
    policy_list[0] =
        m_orb->create_policy(Messaging::RELATIVE_RT_TIMEOUT_POLICY_TYPE,
                             relative_timeout_as_any);
    CORBA::Object_var object =
        m_cohort_temp->_set_policy_overrides(policy_list, CORBA::ADD_OVERRIDE);
    policy_list[0]->destroy();

    m_cohort = CdmwFT::Transaction::TPCCohort::_narrow(object.in());

    m_pTransactionEngine = m_pTransactionCohort;
   
    m_pTransactionCoordinator->deactivate();
    m_pTransactionCohort->deactivate();
    Cdmw::Common::Properties* sp = Cdmw::OsSupport::SystemProperties::Instance();
    if (!sp->get_property(CDMW_FT_INSTALL_HEXDUMP_SIGUSR1_HANDLER).empty())
       Cdmw::OsSupport::Signals::Instance()->register_handler(SIGUSR1, new SIGUSER1_Handler(this));
}

/**
 * Destructor.
 */
DataStoreContext::~DataStoreContext()
{
   // Cancel timers
   {
      CDMW_MUTEX_GUARD(m_freeze_mutex);
//       for (LocationSet::iterator pos = m_freeze_locations.begin(); pos != m_freeze_locations.end();
//            ++pos)
//       {
//          m_iter_map[*pos]->deactivate();
//          m_iter_map[*pos]->_remove_ref();
//          m_iter_map.erase(*pos);
//       }
      m_freeze_locations.clear();
      if (m_freeze_timer_id != -1) {
         m_freeze_timer.cancel_timer(m_freeze_timer_id);
         m_freeze_timer_id = -1;
      }
   }

    m_pTransactionCoordinator->_remove_ref();
    m_pTransactionCoordinator = 0;
    m_pTransactionCohort->_remove_ref();;
    m_pTransactionCohort = 0;
    m_pTransactionEngine = 0;

    if (m_reqScheduler!=0)
    {
        m_reqScheduler->stop(true);
        delete m_reqScheduler;
        m_reqScheduler = 0;
    }

}

void DataStoreContext::register_datastore(DataStoreBase* ds)
    throw (AlreadyExistException)
{
    // BEWARE: We chose not to add a mutext since this is meant to be called
    //         by generated code at startup where DataStore creation is serialised!
    DataStoreMap::iterator info_pos = m_datastore_map.find(ds->get_dsid());
    if (info_pos != m_datastore_map.end())
    {
        std::ostringstream msg;
        msg << "DataStoreContext: A DataStoreBase with " << ds->get_dsid() 
            << " as ID is already registered.";
        throw AlreadyExistException(msg.str());
    }
    else
    {
        m_datastore_map[ds->get_dsid()] = ds;
    }
}

void DataStoreContext::unregister_datastore(const DsIdentifier& dsid)
    throw (NotExistException)
{
    // BEWARE: We chose not to add a mutext since this is meant to be called
    //         by generated code at cleanup where DataStore disposal/cleanup is serialised!
    DataStoreMap::iterator info_pos = m_datastore_map.find(dsid);
    if (info_pos != m_datastore_map.end())
    {
        m_datastore_map.erase(dsid);
    }
    else
    {
        throw NotExistException();
    }
}

DataStoreBase* DataStoreContext::get_datastore(const DsIdentifier& id)
{
    CDMW_ASSERT(m_datastore_map.find(id) != m_datastore_map.end());
    return m_datastore_map[id];
}

DataStoreContext::DataStoreIdList DataStoreContext::get_datastore_ids()
{
    // BEWARE: We chose not to add a mutext since this is meant to be called
    //         after DataStore creation (at startup) and their disposal (at cleanup)
    //         DataStores are not meant to be added/removed dynamically outside of 
    //         startup and cleanup phases.
    DataStoreContext::DataStoreIdList lst;
    for (DataStoreContext::DataStoreMap::iterator it = m_datastore_map.begin(); 
         it != m_datastore_map.end(); 
         ++it) 
    {
        lst.push_back(it->first);
    }
    return lst;
}

/**
 * process failure of a location
 *
 *@param location is the failed location
 */
void DataStoreContext::on_failure(const std::string& location)
{
    // Protect m_transactionEngineRefMap & m_cohortsMap
    // from concurrent access -> allow on_failure when transactions are in progress!
    CDMW_READER_LOCK_GUARD(m_cohorts_mutex);
    TransactionEngineRefMap::iterator iter =
        m_transactionEngineRefMap.find(location);
    if (iter != m_transactionEngineRefMap.end())
        m_transactionEngineRefMap.erase(location);
    
    m_pTransactionEngine->on_failure(location);

    m_cohortsSet.erase(location);
}

/**
 * Add the references to the Cohort and the Coordinator objects of another new location.
 * This operation is used in case of point-to-point protocol.
 *
 *@param location is the new location
 *@param coordinator is the reference to the Coordinator object
 *@param cohort is the reference to the Cohort object
 */
void DataStoreContext::add_ref_for_location
(const std::string& location,
 CdmwFT::Transaction::TPCCoordinator_ptr coordinator,
 CdmwFT::Transaction::TPCCohort_ptr cohort)
    throw (AlreadyExistException)
{
    // We can change locations only when no transactions in all datastores
    CDMW_WRITER_LOCK_GUARD(m_cohorts_mutex);

    if (!m_is_miop && (location != DataStoreBase::Get_Location_Name()))
    {
        TransactionEngineRef EngineStruct;
        EngineStruct.coordinator =
            CdmwFT::Transaction::TPCCoordinator::_duplicate(coordinator);
        EngineStruct.cohort =
            CdmwFT::Transaction::TPCCohort::_duplicate(cohort);
        m_transactionEngineRefMap[location] = EngineStruct;
    }
}

/**
 * Return the reference to the local Coordinator object
 */
CdmwFT::Transaction::TPCCoordinator_ptr DataStoreContext::get_tpc_coordinator()
    const
{
    return CdmwFT::Transaction::TPCCoordinator::_duplicate(m_coordinator.in());
}

/**
 * Return the reference to the local Cohort object
 */
CdmwFT::Transaction::TPCCohort_ptr DataStoreContext::get_tpc_cohort() const
{
    return CdmwFT::Transaction::TPCCohort::_duplicate(m_cohort.in());
}

/**
 * Return the references to the Coordinator and the Cohort object of a location
 *
 *@param location is the location
 *@result a structure with the references to the Coordinator and the Cohort
 */
TransactionEngineRef
DataStoreContext::get_ref_for_location(const std::string& location) const
    throw (NotFoundException)
{
   // Called by TransactionEngineCoordinator.cpp and TransactionEngineCohort.cpp
    if(location == DataStoreBase::Get_Location_Name())
    {
        TransactionEngineRef engine;

        engine.coordinator =
            CdmwFT::Transaction::TPCCoordinator::_duplicate(m_coordinator.in());
        engine.cohort =
            CdmwFT::Transaction::TPCCohort::_duplicate(m_cohort.in ());

        return engine;
    } 
    else
    {
       // Protect access to m_transactionEngineRefMap using
       // a read lock since transactions in progress aleady acquired 
       // read locks
       CDMW_READER_LOCK_GUARD(const_cast<Cdmw::OsSupport::ReaderWriterLock&>(m_cohorts_mutex));
       TransactionEngineRefMap::const_iterator iter =
          m_transactionEngineRefMap.find(location);
       if (iter == m_transactionEngineRefMap.end())
          throw NotFoundException();
       else
       {
          return iter->second;
       }
    
    }
}

/**
* set coordinator location
*
*@param location 
*/
void DataStoreContext::set_coordinator_location(const std::string& location)
    throw ()
{
    m_coordinator_location = location;
    Cdmw::OsSupport::SystemProperties::Set_Property(
       DataStoreContext_PROPERTY_NAME + ".CoordinatorLocation",
       m_coordinator_location);
}


/**
* get coordinator location
*
*@param location 
*/
std::string DataStoreContext::get_coordinator_location() const
    throw ()
{
    return m_coordinator_location;
}


CdmwFT::Transaction::TPCCoordinator_ptr DataStoreContext::get_coordinator_ref()
    const
{
    return CdmwFT::Transaction::TPCCoordinator::_duplicate(m_coordinator.in());
}

CdmwFT::Transaction::TPCCohort_ptr DataStoreContext::get_cohort_ref() const
{
    return CdmwFT::Transaction::TPCCohort::_duplicate(m_cohort.in());
}


/**
 * The data store becomes the one of the master
 */
void DataStoreContext::activate()
{
    if (m_state == DSC_STATE_MASTER)
    {
        return;
    }

    CDMW_INFO(FTLogger::GetLogger(),"Activate DataStoreContext : set state to 'MASTER' mode");
    m_state = DSC_STATE_MASTER;
    STORE_DATASTORE_CONTEXT_STATE(m_state);

    // Start freeze timer
    m_freeze_timer.start();
    

    // TAO BUG : the disassociate_reference_with_id is not implemented
    //    m_poa->disassociate_reference_with_id(m_cohort.in(), m_cohort_id.in ());
    
    //    m_poa->associate_reference_with_id(m_coordinator.in(), m_coordinator_id.in ());
    m_pTransactionCohort->deactivate();

    m_pTransactionEngine = m_pTransactionCoordinator;
    m_pTransactionCoordinator->activate
        (m_pTransactionCohort->get_cohort_transactions(),
         m_pTransactionCohort->get_done_transactions_list());
    m_pTransactionCohort->clear_done_transactions_list();
}

/**
 * The data store becomes the one of the slave
 */
void DataStoreContext::deactivate()
{
    if (m_state == DSC_STATE_SLAVE)
    {
        return;
    }

    CDMW_INFO(FTLogger::GetLogger(),"Deactivate DataStoreContext : set state to 'SLAVE' mode");
    m_state = DSC_STATE_SLAVE;
    STORE_DATASTORE_CONTEXT_STATE(m_state);

    // TAO BUG : the disassociate_reference_with_id is not implemented
    //    m_poa->disassociate_reference_with_id(m_coordinator.in(), m_coordinator_id.in ());
    
    //    m_poa->associate_reference_with_id(m_cohort.in(), m_cohort_id.in ());

    m_pTransactionCoordinator->deactivate();
    m_pTransactionCohort->activate();

    m_pTransactionEngine = m_pTransactionCohort;
    // Stop freeze timer
    m_freeze_timer.stop();
}

bool DataStoreContext::request(const CdmwFT::Transaction::StateChanges& change)
    throw (InternalErrorException, ValidationFailureException)
{
    return m_pTransactionCoordinator->request(change);
}

void DataStoreContext::wait_for_no_transactions_in_progress()
{
    m_pTransactionCoordinator->wait_for_no_transactions_in_progress();
}
void DataStoreContext::inc_modification_in_progress()
{
   CDMW_MUTEX_GUARD(m_freeze_mutex);
   
   if (!m_freeze_locations.empty())
   {
      CDMW_DEBUG(FTLogger::GetLogger(),"About to freeze");
      
      do
      {
         m_freeze_condition.wait();
      } while (!m_freeze_locations.empty());
      
   }
   ++m_modifications_in_progress;
}
void DataStoreContext::dec_modification_in_progress()
{
  CDMW_MUTEX_GUARD(m_freeze_mutex);
  if (--m_modifications_in_progress == 0)
    {
      // Release threads blocked in
      // wait_for_no_modifications_in_progress
      m_freeze_condition.broadcast();
    }
}

::CdmwFT::StateTransfer::RecordIterator_ptr
DataStoreContext::get_record_iterator
(const char * from_location,
 const CdmwFT::StateTransfer::DataStoreIDs& datastore_ids,
 CdmwFT::Transaction::TPCCoordinator_ptr coordinator,
 CdmwFT::Transaction::TPCCohort_ptr cohort)
   throw (CORBA::SystemException,
          CdmwFT::StateTransfer::InvalidDataStoreState)
{

    if (!this->is_activated())
    {
        // Not the primary
        throw CdmwFT::StateTransfer::InvalidDataStoreState();
    }

    // Protects from concurrent creation of RecordIterators.
    CDMW_MUTEX_GUARD(m_freeze_mutex);
    RecordIteratorMap::iterator pos = m_iter_map.find(from_location);

    // NOTA: What if this is a backup restart? Is previous record iterator
    //       garbage collected?
    // . When RecordIterator reaches datastore freeze, the freeze timeout will make sure
    // the iterator is destroyed if the backup fails before calling destroy().
    // . The FTManager design expects it to call handle_backup_insertion_failure() on the 
    // the primary when the backup fails during insertion. This is NOT done in V4.2!
    // We need to handle restarts of backups that have failed during their insertion at the
    // primary! We can safely handle the fact that once insertion is comlpete, failure of 
    // the backup will be reported to the primary the normal way (by the FTManager);
    // We can thus at least handle attempts to insert a location that is already recognised as a backup!
    if (pos != m_iter_map.end ())
    {
       // NOTA: make sure m_freeze_mutex is aquired before aquiring m_cohorts_mutex
       //       to avoid deadlocks (unfreeze calls them in this order).
       {
          CDMW_WRITER_LOCK_GUARD(m_cohorts_mutex);
          LocationSet::iterator it =
             find(m_cohortsSet.begin(), m_cohortsSet.end(), from_location);
          if (it != m_cohortsSet.end())
          {
             // Iterator already exists for this location
             throw CORBA::BAD_INV_ORDER(Cdmw::OrbSupport::BAD_INV_ORDERIteratorPresent,
                                        CORBA::COMPLETED_NO);
          } 
       }
       // We assume for now that this is a backup restart following failure during
       // backup insertion. This is also usefull when the freeze timeout is rather
       // big and a backup gets restarted before previous freeze timeout expires ...
       CDMW_INFO(FTLogger::GetLogger(),"Backup Insertion <" 
                 << m_location << "> request to re-insert location '" 
                 << from_location << "' after failure.");
       // Cleanup iterator to unregister datastore observers ... etc..cs
       CDMW_LOG(FTLogger::GetLogger(), FTLogger::BACKUP_INSERTION_LEVEL, 
                "Backup Insertion <"<<m_location<<"> Deactivating iterator");
       (*pos).second->deactivate();
       (*pos).second->_remove_ref();
       m_iter_map.erase(pos);
       CDMW_LOG(FTLogger::GetLogger(), FTLogger::BACKUP_INSERTION_LEVEL, 
                "Backup Insertion <"<<m_location<<"> After erase");
       // Did it call freeze?
       LocationSet::iterator freeze_loc_pos = m_freeze_locations.find(from_location);
       if (freeze_loc_pos != m_freeze_locations.end())
       {
          m_freeze_locations.erase(*freeze_loc_pos);
       }
       if (m_freeze_locations.empty())
          m_freeze_condition.broadcast();
    }

    RecordIterator_impl* iter_impl =
       new RecordIterator_impl(*this, m_poa.in(), m_orb.in(), from_location,
                               datastore_ids, coordinator, cohort,
                               m_freeze_threshold);
    
    CdmwFT::StateTransfer::RecordIterator_var iter = iter_impl->_this();
    m_iter_map[from_location] = iter_impl;

    return iter._retn();
}

void DataStoreContext::update_with_coordinator(
    CdmwFT::StateTransfer::DataStoreGroup_ptr primary_datastore_group,
    const std::string & coordinator_location,
    const std::string & location,
    const CdmwFT::StateTransfer::DataStoreIDs& datastore_ids)
    throw (CORBA::SystemException,
           CdmwFT::StateTransfer::InvalidDataStoreState)
{
    // For safety reasons, it is advised to transfer all datastores at once; 
    // but we should make sure that this is only performed once for a backup!
    //
    // NOTA: We do not perform any protection against concurrent updates because 
    //       the FTManager, by design, does not perform concurrent call on
    //       the StatefullPrimarybackupAdmin and all datastores are already created!

   // We only rely on m_state instead of adding a local flag because upon first init
   // the DataStoreContext is deactivated (call to this->deactivate()).
   if (m_state == DSC_STATE_NONE) {
      CdmwFT::StateTransfer::DataStoreIDs all_DsIds (m_datastore_map.size());
      all_DsIds.length(m_datastore_map.size());
      CORBA::ULong idx = 0L;
      for (DataStoreContext::DataStoreMap::iterator it = m_datastore_map.begin();
           it != m_datastore_map.end();
           ++it)
      {
         all_DsIds[idx] = it->second->get_dsid();
         ++idx;
      }
      
      CdmwFT::StateTransfer::RecordIterator_var rec_iter =
         primary_datastore_group->get_record_iterator(location.c_str(),
                                                      all_DsIds,
                                                      m_coordinator.in(),
                                                      m_cohort.in());
      
      bool more;
      
      try
      {
         do
         {
            CdmwFT::Transaction::StateChanges_var changes;
            
            more = rec_iter->next_n(m_iterator_increment, changes.out());
            
            for (CORBA::ULong i = 0; i < changes->length(); i++)
            {
               DataStoreBase* ds = m_datastore_map[changes[i].dsid];
               ds->load_state(changes[i].records);
            }
            
         } while (more);
         
         this->set_coordinator_location(coordinator_location);
         // Enable 'SLAVE' mode as DataStoreContext state is initialised as neither MASTER nor SLAVE !
         this->deactivate();
         rec_iter->destroy();
         
      }
      catch (const CORBA::Exception & ex)
      {
         // NOTA: We ARE NOT attempting recovery from transient network failures!
         CDMW_FATAL(FTLogger::GetLogger(),
                    "[ERROR] Location <" << m_location
                    << " updating with coordinator "
                    << m_coordinator_location
                    << "> Aborting location! \n"
                    << "     Reason: received exception "
                    << Cdmw::OrbSupport::OrbSupport::exception_to_string(ex));
         ::exit(BACKUP_INSERTION_EXIT_FAILURE);
      }
      catch (const NotFoundException& e)
      {
         CDMW_FATAL(FTLogger::GetLogger(),
                    "[ERROR] Location <" << m_location
                    << " updating with coordinator "
                    << m_coordinator_location
                    << "> Aborting location! \n"
                    << "     Reason: received NotFoundException "
                    << e.what() );
         ::exit(BACKUP_INSERTION_EXIT_FAILURE);
      }
      catch (const AlreadyExistException& e)
      {
         CDMW_FATAL(FTLogger::GetLogger(),
                    "[ERROR] Location <" << m_location
                    << " updating with coordinator "
                    << m_coordinator_location
                    << "> Aborting location! \n"
                    << "     Reason: received AlreadyExistException "
                    << e.what() );
         ::exit(BACKUP_INSERTION_EXIT_FAILURE);
      }
   }
}
   
bool DataStoreContext::validate
(const CdmwFT::Transaction::StateChanges& changes)
{
    for (CORBA::ULong i = 0; i < changes.length(); ++i)
    {
        DsIdentifier dsid = changes[i].dsid;
        DataStoreBase *dataStore = m_datastore_map[dsid];

        if (!dataStore || !dataStore->validate(changes[i].records))
        {
           if (FTLogger::GetLogger()->is_debug_enabled())
           {
              std::ostringstream ostr;
              ostr << "CHANGES NOT VALIDATED!!\n";
              dump_changes(changes,ostr);
              FTLogger::GetLogger()->log(::Cdmw::Logging::DEBUG,
                                         ostr.str().c_str(), __FILE__, __LINE__);
           }
           return false;
        }
    }

    return true;
}

bool DataStoreContext::oids_match
(const CdmwFT::Transaction::StateChanges& changes1,
 const CdmwFT::Transaction::StateChanges& changes2)
{
    for (CORBA::ULong i = 0; i < changes1.length(); ++i)
    {
        DsIdentifier dsid1 = changes1[i].dsid;
        DataStoreBase *dataStore = m_datastore_map[dsid1];

        for (CORBA::ULong j = 0; j < changes2.length(); ++j)
        {
            DsIdentifier dsid2 = changes2[j].dsid;

            if (dsid1 == dsid2)
            {
                if (dataStore->oids_match(changes1[i].records,
                                          changes2[j].records))
                {
                    return true;
                }
            }
        }
    }

    return false;
}

/**
 * Purpose:
 * <p> add one location (to the yes/no/end check)
 *
 *@param new location
 */
void DataStoreContext::add_location(const std::string& location)
    throw (AlreadyExistException)
{

    // We can change locations only when no transactions in all datastores
    CDMW_WRITER_LOCK_GUARD(m_cohorts_mutex);
    LocationSet::iterator it =
       find(m_cohortsSet.begin(), m_cohortsSet.end(), location);
    if (it != m_cohortsSet.end())
    {
        throw AlreadyExistException("The location is already registered");
    }
    else if (location == m_location)
    {
        throw AlreadyExistException("Should be a coordinator");
    }
    else
    {
       m_cohortsSet.insert(location);
    }
}

/**
 * Purpose:
 * <p> set locations (to the ack check)
 *
 *@param new locations
 */
void DataStoreContext::set_locations(const LocationSet& locations)
{
    // We can change locations only when no transactions in all datastores
    CDMW_WRITER_LOCK_GUARD(m_cohorts_mutex);
    m_cohortsSet = locations;
    m_cohortsSet.erase(m_location);
}


DataStoreContext::LocationSet
DataStoreContext::get_locations()
{
    return m_cohortsSet;
}

void DataStoreContext::hexdump(std::ostream & os) const
{
   // NOTA: We do not use any mutex here because we assume that
   // datastores we already added when we want this dump function.
   // datastores are usualy added by generated code at startup.
   
   const std::size_t NB_DATASTORES = m_datastore_map.size();
   
   std::string state("NONE");
   if (m_state == Cdmw::FT::DataStoreContext::DSC_STATE_MASTER) state = "MASTER"; 
   else if (m_state == Cdmw::FT::DataStoreContext::DSC_STATE_SLAVE) state = "SLAVE";

   OsSupport::OS::Timeval  now = OsSupport::OS::get_time();
   // format date
   char time_buf[128];
   OsSupport::OS::get_fmt_time (time_buf, sizeof(time_buf),now.seconds,
                                "%Y-%m-%dT%H:%M:%SZ", OsSupport::OS::GMT_TIME);
   unsigned int NB_MODIFS_IN_PROGRESS = 0;
   {
      CDMW_MUTEX_GUARD(const_cast<Cdmw::OsSupport::Mutex&>(m_freeze_mutex));
      NB_MODIFS_IN_PROGRESS = m_modifications_in_progress;
   }
   os << "DUMP FT DATASTORES FOR LOCATION  '" << m_location << "' ==>\n"
      << "--------------------------------------------------------------\n"
      << "TIME     :'" << std::string(time_buf) << "'\n"
      << "STATE    :'" << state << "'\n"
      << "LOCATION :'" << m_location << "'\n"
      << "COORDINATOR LOCATION :'" << m_coordinator_location << "'\n"
      << "TRANSPORT MODE       :'" << (m_is_miop? "MIOP":"IIOP")  << "'\n";
   if (m_is_miop) {
      os << "COORDINATOR CORBALOC :'" << m_coordinator_corbaloc << "'\n";
      os << "COHORT CORBALOC      :'" << m_cohort_corbaloc << "'\n";
   }
   os << "INSERTION CHUNK SIZE       :'" << m_iterator_increment << "'\n"
      << "INSERTION FREEZE THRESHOLD :'" << m_freeze_threshold << "'\n"
      << "INSERTION FREEZE TIMEOUT   :'" << m_freeze_timeout.seconds << " sec, " 
      << m_freeze_timeout.microseconds << " µsec'\n"
      << "MODIFICATIONS IN PROGRESS  :'" << NB_MODIFS_IN_PROGRESS << "'\n"
      << "WORKER THREADS             :'" << m_reqScheduler->nb_threads() << "'\n";
   {
      CDMW_MUTEX_GUARD(const_cast<Cdmw::OsSupport::Mutex&>(m_freeze_mutex));
      os << "ON GOING INSERTIONS : '" << m_iter_map.size() << "'\n";
      for (RecordIteratorMap::const_iterator iter = m_iter_map.begin();
           iter != m_iter_map.end();
           ++iter)
         os << "\t '" << (*iter).first << "'\n ";
      os << "ON GOING INSERTIONS AT FREEZE LEVEL: '" << m_freeze_locations.size() << "'\n";
      for (LocationSet::const_iterator iter = m_freeze_locations.begin();
           iter != m_freeze_locations.end();
           ++iter)
         os << "\t '" << (*iter) << "'\n ";
   }
   {
      CDMW_READER_LOCK_GUARD(const_cast<Cdmw::OsSupport::ReaderWriterLock&>(m_cohorts_mutex));
      os << "ON GOING INSERTIONS AT FREEZE LEVEL: '" << m_cohortsSet.size() << "'\n";
      for (LocationSet::const_iterator iter = m_cohortsSet.begin();
           iter != m_cohortsSet.end();
           ++iter)
         os << "\t '" << (*iter) << "'\n ";

   }

   os << "TRANSACTION ENGINE COORDINATOR:\n";
   m_pTransactionCoordinator->hexdump(os);
   os << "TRANSACTION ENGINE COHORT:\n";
   m_pTransactionCohort->hexdump(os);
   os << "NB DATASTORES :'" << NB_DATASTORES << "'\n"
      << "--------------------------------------------------------------\n"
      << "-------------------- BEGIN -----------------------------------\n";
   std::size_t idx = 0;
   for (DataStoreMap::const_iterator iter = m_datastore_map.begin(); 
        iter != m_datastore_map.end(); ++iter)
   {
      ++idx;
      os << "DATASTORE  [" << idx << "/" << NB_DATASTORES << "]\n";
      iter->second->hexdump(os);
   }
   os  << "--------------------  END  -----------------------------------\n";
}
void DataStoreContext::stop()
{
   // Cancel timers
   {
      CDMW_MUTEX_GUARD(m_freeze_mutex);
      
      for (LocationSet::iterator pos = m_freeze_locations.begin(); pos != m_freeze_locations.end();
           ++pos)
      {
         m_iter_map[*pos]->deactivate();
         m_iter_map[*pos]->_remove_ref();
         m_iter_map.erase(*pos);
      }
      m_freeze_locations.clear();
      if (m_freeze_timer_id != -1) {
         m_freeze_timer.cancel_timer(m_freeze_timer_id);
         m_freeze_timer_id = -1;
      }
    // do we to unfreeze datastores?
//     m_freeze_condition.broadcast();
   }
   if (m_reqScheduler!=0)
   {
      m_reqScheduler->stop(true);
   }
}



// Operations for freezing all DataStores for terminating backup insertion
void DataStoreContext::freeze(const std::string& location)
{
   //    DataStoreContext::freeze() should wait for on_freeze_timeout() to complete
   //    should also add a wait for modifications to complete.
   
   CDMW_MUTEX_GUARD(m_freeze_mutex);
   CDMW_DEBUG(FTLogger::GetLogger()," freeze invoked for location <" << location << ">");
   if (m_freeze_locations.empty()) {
      // Set up a timeout
      m_freeze_timer_id = m_freeze_timer.start_timer(this, 0, m_freeze_timeout);
   }
   m_freeze_locations.insert(location);
   // Check if this call can be removed? Check if this is to be called without holding freeze_mutex.
   m_pTransactionCoordinator->wait_for_no_transactions_in_progress();
   
   while (m_modifications_in_progress > 0)
   {
      m_freeze_condition.wait();
   }   
   
}

// Returns true if successfull, false when freeze timeout occured in the meantime!
bool DataStoreContext::unfreeze(const std::string& location,
                                CdmwFT::Transaction::TPCCoordinator_ptr coordinator,
                                CdmwFT::Transaction::TPCCohort_ptr cohort)
{
   //    DataStoreContext::unfreeze() should wait for on_freeze_timeout() to complete
   //          should call add_ref_for_location() before trying to unfreeze!

   CDMW_MUTEX_GUARD(m_freeze_mutex);

   CDMW_DEBUG(FTLogger::GetLogger(),"unfreeze invoked for location <"
              << location << ">");

   // Did on_freeze_timeout() happen?
   if (m_freeze_locations.find(location) == m_freeze_locations.end())
      return false;
   // DataStores are frozen here and normally there is no transaction/update in progress
   // so we should get a deadlock when acquiring CDMW_WRITER_LOCK_GUARD(m_cohorts_mutex)
   // Use both add_location and add_ref_for_location here.
   this->add_ref_for_location(location, coordinator, cohort);
   try {
      this->add_location(location);
   } catch (const std::exception & e) {
      CDMW_FATAL(FTLogger::GetLogger(),
                 "Unfreezing '" << m_location << "' caught unexpected exception '"
                 << e.what() << "'");
      CDMW_ASSERT(0);
   }
   RecordIteratorMap::iterator pos = m_iter_map.find(location);
   if (pos != m_iter_map.end ())
   {
      pos->second->_remove_ref();
      m_iter_map.erase(pos);
   }
   
   
   m_freeze_locations.erase(location);
   
   if (m_freeze_locations.empty())
   {
      m_freeze_condition.broadcast();
   }
   return true;
}

void DataStoreContext::on_freeze_timeout()
{
   // Block all calls to RecordIterator_impl::destroy() calls while
   // this operation is running.
   // Also freeze all freeze() requests. 
   // The chalenge is:
   // 1) destroy() should normaly only be called once all transactions/modifications
   // are complete and the data stores are frozen. This also means no Transactions 
   // in progress.
   // 2) if a freeze timeout occurs before then all data stores will be 'unfreezed' again
   // and the destroy() should fail (raise a CORBA::SystemException ?) in order to abort
   // backup insertion. But this may allow transactions again before destroy() is 
   // actually called.
   // 3) freeze check should take into account any freeze_timeout() in progress to avoid
   // any side effects.
   // 4) Locations can only added when no transactions are in progress. This also means 
   // that this datastores should be frozen at that time.
   // One more thing: 
   // RecordIterator_impl::next_n() may request a freeze of all the datastores and then 
   // wait for all transactions/modifications to be completed before returning from
   // the operation (and leaving datastores frozen).
   // Let's sumarize:
   //    DataStoreContext::freeze() should wait for on_freeze_timeout() to complete
   //          should also add a wait for modifications to complete.
   //    DataStoreContext::unfreeze() should wait for on_freeze_timeout() to complete
   //          should call add_ref_for_location() before trying to unfreeze!
   //    DataStoreContext::freeze_check() is called by DataStore<>::insert, update and remove
   //          it is immediately followed for a modification_guard that increments/decrements
   //          the modification in progress counter. A possible optimisation is to couple this
   //          with the freeze_check()? ==> DataStoreContext::inc_modification_in_progress()
   //          to perform previous freeze_check!
   //          

   
   CDMW_MUTEX_GUARD(m_freeze_mutex);

   CDMW_DEBUG(FTLogger::GetLogger(),"Backup Insertion: location '"<<m_location<<"' in freeze timeout");
   for (LocationSet::iterator pos = m_freeze_locations.begin(); pos != m_freeze_locations.end();
        ++pos)
   {
      CDMW_WARN(FTLogger::GetLogger(),"Backup Insertion <" 
                << m_location<<"> Calling Timeout for location " << *pos);
      // Cleanup iterator to unregister datastore observers ... etc..cs
      CDMW_LOG(FTLogger::GetLogger(), FTLogger::BACKUP_INSERTION_LEVEL, 
               "Backup Insertion <"<<m_location<<"> Deactivating iterator");
      m_iter_map[*pos]->deactivate();
      m_iter_map[*pos]->_remove_ref();
      m_iter_map.erase(*pos);
      CDMW_LOG(FTLogger::GetLogger(), FTLogger::BACKUP_INSERTION_LEVEL, 
               "Backup Insertion <"<<m_location<<"> After erase");
      DataStoreBase::Timeout(*pos); // To help in testing!
   }

   m_freeze_timer.cancel_timer(m_freeze_timer_id);
   m_freeze_timer_id = -1;
   CDMW_DEBUG(FTLogger::GetLogger(),"Backup Insertion: location '"<<m_location<<"' Timeouts triggered");

   m_freeze_locations.clear();
   m_freeze_condition.broadcast();
}


int DataStoreContext::handle_timeout(const void*)
{
   this->on_freeze_timeout();
   return -1;
}

}  // End namespace FT
}  // End namespace Cdmw
