/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO 
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2009. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

#include "FaultTolerance/ftstatemanager/Timer.hpp"
#include <ace/Reactor.h>
#include "ace/Timer_Hash.h"
#include <FaultTolerance/ftcommon/FTLogger.hpp>
#include <Foundation/osthreads/Mutex.hpp>
#include <Foundation/osthreads/MutexGuard.hpp>
#include <Foundation/osthreads/Condition.hpp>


namespace Cdmw {
namespace FT {

    
    
    TimerBase::TimerBase()
            : m_timer_queue(new CDMW_Timer_List()),
         m_started(false),
         m_stopped(false),
         m_condition(m_mutex)
    {
    }
    
    TimerBase::~TimerBase()
        throw ()
    {
       try {
        // stop threads if not already done
        this->stop();
       } catch (const std::exception & e) {
          std::cerr << "\n@@@@@@@@@@@@@@@@@@@ UNCAUGHT EXCEPTION = " << e.what() << std::endl;
          throw;
       }
        delete m_timer_queue;
    }

    void TimerBase::stop()
    {
       //waiting for the completion of a too time-close run() call !!
       //Cdmw::OsSupport::OS::sleep(500); 
       
        if (!m_stopped)
        {
          
            // Wait the termination of the internal thread
            // outside of the mutex guard to let the thread to finish
            if ( this->get_status() != OsSupport::THREAD_CREATED )
            {   
                {
                    CDMW_MUTEX_GUARD(m_mutex);
                    m_stopped = true;
                // awake the thread, if it is sleeping
                    m_condition.broadcast();
                }
                
                this->join();
                
            } 
        }
    }
    
    int TimerBase::cancel_timer (int timer_id, const void **act)
    {
        int res = m_timer_queue->cancel(timer_id, act);
        return res;
    }
 
/*  could be use to factorize deletion processing but it is better to have
    new and delete in the same class (TransactionEngineCohort)

    int TimerBase::cancel_timer (int timer_id)
    {
        const void* p_tidToDelete;
        int res = m_timer_queue->cancel(timer_id, &p_tidToDelete);
        if (p_tidToDelete != NULL)
        {
            delete (CdmwFT::Transaction::TransactionIdentification*)p_tidToDelete;
        }
        return res;
    }
*/
    void TimerBase::run (void) throw()
    {
      
     CDMW_MUTEX_GUARD(m_mutex);
     m_stopped = false;      
       
     while(!m_stopped){
             // wait until wake by stop(), cancel_timer()
             // or timer expiration
             (void)m_condition.wait(400);
             if (!m_stopped)
             {
                m_timer_queue->expire();  
             }
     }
    }
    

}; // namespace FT
}; // namespace Cdmw
