/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO 
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2009. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

#include "ftultimatefallback/FallbackHelper.hpp"
#include <Foundation/orbsupport/ExceptionMinorCodes.hpp>
#include <Foundation/common/Assert.hpp> // for CDMW_ASSERT macro
#include <FaultTolerance/ftcommon/FallbackOid.hpp>
#include <FaultTolerance/ftcommon/FTLogger.hpp>
#include <Foundation/logging/FunctionLogger.hpp>

namespace Cdmw {
namespace FT {

namespace UltimateFallback {
    

   CORBA::Object_ptr 
   FallbackHelper::Get_taget_object(CosNaming::NamingContextExt_ptr root,
                                    const FallbackOid & foid)
      throw (CORBA::SystemException)
   {
      CDMW_LOG_FUNCTION(Cdmw::FT::FTLogger::GetLogger(),"");
      CDMW_ASSERT(!CORBA::is_nil(root));
      
      //    * NOTA: object_id, adapter_id and adapter_name are available here!
      //    1- Create a FallbackOid(object_id) in case of adapter_name == "UltimateFallbackPOA"
      //    2- get group_domain_id and perform a discover global service
      //        CORBA::Object_var obj 
      //            = Cdmw::CommonSvcs::ServiceRegistration::ServiceRegistration_impl::get_instance(
      //                  )->get_service(group_domain_id, true);
      //    3- Narrow to naming context 
      //    4- nc->resolve("group_id.gid");
      //    5- raise PortableInterceptor::ForwardRequest 
      //
      // ==> (Daemon's NC == rir("NameService")) -> resolve_str( fallback_oid.to_name()); --> // "group_domain_id/group_id.gid"
      //     ServiceRegistration::get_service(group_domain_id)
      //         -  Have a canonical domain representation in CARDAMOM
      //         -  (à la corbaloc)
      //         -  
      //         -  
      //     Be able to define a (name, group_domain_id) mappping/definition for daemons
      //     default [name = group_domain_id], [protocole = cdm service discovery]
      //     ex: FT_Domain-X = corbaloc::localhost:1234/cdmw.name-domain.FTDomainX
      // 
      
      //    * Q1: Why not use a default servant instead of a PI? 
      //    * Q2: Why not use a servant locator instead of a PI? 
      //
      //  Requirement
      //
      CDMW_DEBUG(Cdmw::FT::FTLogger::GetLogger(),
                 "Request on Ultimate Fallback object\n"
                 "oid='"<<foid.to_string()<<"'\n"
                 "name='"<<foid.name()<<"'");
      CORBA::Object_var obj = CORBA::Object::_nil();

      // Handling of resolve_str() user exceptions
      // CosNaming::NamingContext::NotFound -> CORBA::OBJECT_NOT_EXIST
      // CosNaming::NamingContext::CannotProceed --> INV_OBJREF
      // CosNaming::NamingContext::InvalidName --> INV_OBJREF
      // CORBA::SystemException --> rethrow
      try {
         obj = root->resolve_str(foid.name().c_str());
      } catch (const CosNaming::NamingContext::NotFound & ex) {
         CDMW_WARN(Cdmw::FT::FTLogger::GetLogger(),
                   "Ultimate fallback: object '" << foid.name()<<"' NotFound"
                   << ". Raising CORBA::OBJECT_NOT_EXIST");
         CORBA::OBJECT_NOT_EXIST x(OrbSupport::OBJECT_NOT_EXIST, CORBA::COMPLETED_NO);
         CDMW_LOG_FUNCTION_EXCEPTION(x);
         throw x;
      } catch (const CosNaming::NamingContext::InvalidName & ex) {
         CDMW_WARN(Cdmw::FT::FTLogger::GetLogger(),
                   "Ultimate fallback: object '" << foid.name()<<"' InvalidName"
                   << ". Raising CORBA::INV_OBJREF");
         CORBA::INV_OBJREF x(OrbSupport::INV_OBJREF, CORBA::COMPLETED_NO);
         CDMW_LOG_FUNCTION_EXCEPTION(x);
         throw x;
      } catch (const CosNaming::NamingContext::CannotProceed & ex) {
         CDMW_WARN(Cdmw::FT::FTLogger::GetLogger(),
                   "Ultimate fallback: object '" << foid.name()<<"' CannotProceed"
                   << ". Raising CORBA::INV_OBJREF");
         CORBA::INV_OBJREF x(OrbSupport::INV_OBJREF, CORBA::COMPLETED_NO);
         CDMW_LOG_FUNCTION_EXCEPTION(x);
         throw x;
      }
      catch (const CORBA::SystemException& ex)
      {
         CDMW_WARN(Cdmw::FT::FTLogger::GetLogger(),
                   "Ultimate fallback: System exception raised getting object '" << foid.name()<<"'"
                   << ". Rethrowing exception '" << ex << "'");
         CDMW_LOG_FUNCTION_EXCEPTION(ex);
         throw;
      }
      return obj._retn();
   }


}; // namespace UltimateFallback
}; // namespace FT
}; // namespace Cdmw

