/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO 
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2009. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

#ifndef INCL_CDMW_FT_LOCATION_FAULT_DETECTOR_REQUEST_TIMEOUT_LOGGER_HPP
#define INCL_CDMW_FT_LOCATION_FAULT_DETECTOR_REQUEST_TIMEOUT_LOGGER_HPP

// std Files
#include <map>

// Cdmw Files
#include <Foundation/osthreads/Mutex.hpp>
#include <Foundation/common/NotFoundException.hpp>
#include <Foundation/commonsvcs/time/TimeBase.skel.hpp>
//#include <FaultTolerance/ftstatemanager/Timer.hpp>

/**
* Root namespace for CDMW runtime.
*/

namespace Cdmw
{
    
/**
* Namespace for CDMW Fault Tolerance types and data.
*/

namespace FT
{

/**
* Namespace for CDMW Fault Tolerance location fault detector functionality
*/

namespace Watchdog
{

/**
*Purpose:
*<p>    This class is a singleton class and is reqponsible to log incoming
* date, outgoing request date and check of outgoing request timeout expiration.
*
*Features:
*<p> Thread safe
*
*/

class RequestTimeoutLogger 
{
public:
    /**
     * init a RequestTimeoutLogger.
     *
     * @param milliseconds maximum elapse time of request processing for the server.
     * (expressed in ms)
     */
    static void Init_request_timeout_logger(CORBA::ULong milliseconds);

    /**
     * return the instance of the RequestTimeoutLogger
     */
    static RequestTimeoutLogger* Get_instance() throw();
   
    /**
     * Destructor.
     */
    ~RequestTimeoutLogger() throw();

    /**
     * Reset the timeout to a new value.
     *
     * @param milliseconds maximum elapse time of request processing for the server.
     * (expressed in ms)
     */
    void set_timeout(CORBA::ULong milliseconds);

    /**
     * log the time of the request reception.
     *
     * @param request_id request unique identifier.
     */
    void entering_request(CORBA::ULong request_id);

    /**
     * Remove the request entry. This should be done when the answer is sent
     * back to the sender.
     *
     * @param request_id request unique identifier.
     */
    void leaving_request(CORBA::ULong request_id)
        throw (Cdmw::Common::NotFoundException);

    /**
     * Indicate that to process the received request, the server send a request
     * to an other server.
     *
     * @param request_id request unique identifier.
     */
    void waiting_response(CORBA::ULong request_id)
        throw (Cdmw::Common::NotFoundException);

    /**
     * The server has received the response from an other server. The waiting
     * flag is removed and the duration of the request processing is 
     * added to the first request. 
     *
     * @param request_id request unique identifier.
     * @param duration   duration of the request processing.
     */
    void remove_waiting_response(CORBA::ULong request_id)
        throw (Cdmw::Common::NotFoundException);


    /**
     * This operation check all the request processing doesn't last more than
     * expected
     *
     * @param request_id request unique identifier.
     * @param duration   duration of the request processing.
     */
    bool check_timeouts_expiration();
    


private:
    /**
     * Default Constructor
     */
    RequestTimeoutLogger();


    // Structure used to store incoming request time 
    // and outgoing request (called by the incoming request) time
    struct RequestInfo
    {
        TimeBase::TimeT  time_0; // T0 expressed in units of 100 nano seconds
        TimeBase::TimeT  waiting_response_t0; // sub request T0

        RequestInfo(TimeBase::TimeT time)
           : time_0(time),
             waiting_response_t0(0)
        {
        }
    };

    typedef std::map< CORBA::ULong, 
                      RequestInfo, 
                      std::less<CORBA::ULong> > RequestInfoMap;
    
    TimeBase::TimeT   m_timeout; // maximum duration of a request
                                 // processing (100 nano seconds)
    
    Cdmw::OsSupport::Mutex  m_mutex; // mutex to protect map access
       
    RequestInfoMap m_request_infos; // map of incoming request (T0/request_id)

    static RequestTimeoutLogger* M_instance;
    // fixing memory leak
    static int Cleanup_RequestTimeoutLogger();
};

} // End of namespace Watchdog

} // End of namespace FT

} // End of namespace Cdmw

#endif  // INCL_CDMW_FT_LOCATION_FAULT_DETECTOR_REQUEST_TIMEOUT_LOGGER_HPP

