/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO 
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2009. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

// standard files
#include <sstream>

// Cdmw Files
#include <Foundation/common/Assert.hpp> 
#include <Foundation/orbsupport/CORBA.hpp>
#include <Foundation/orbsupport/ExceptionMinorCodes.hpp>
#include <Foundation/ossupport/OS.hpp>


// Local Files
#include <ftwatchdog/FTRequestTimeoutClientRequestInterceptor_impl.hpp>
#include <FaultTolerance/ftcommon/FTLogger.hpp>
#include <Foundation/logging/FunctionLogger.hpp>
#include <ftwatchdog/FTRequestTimeoutLogger.hpp>

#include <idllib/FT.stub.hpp>




namespace 
{
    const PortableInterceptor::ORBInitInfo::ObjectId FT_CLIENT_REQUEST_INTERCEPTOR_NAME 
        = "FTRequestTimeoutClientRequestInterceptor";
    const std::string FORBIDDEN_OPERATION = "add_log";
} // end of anonymous namespace


namespace Cdmw
{
    
namespace FT
{

namespace Watchdog
{

RequestTimeoutClientRequestInterceptor_impl::RequestTimeoutClientRequestInterceptor_impl(
                 PortableInterceptor::ORBInitInfo_ptr info,
                 const PortableInterceptor::SlotId  request_id_slot_id)
   : m_request_id_slot_id(request_id_slot_id)
{
    CDMW_LOG_FUNCTION(FTLogger::GetLogger(),
                      "request_id_slot_id=" << m_request_id_slot_id);
}

char*
RequestTimeoutClientRequestInterceptor_impl::name() 
    throw (CORBA::SystemException)
{
    CDMW_LOG_FUNCTION(FTLogger::GetLogger(),"");
    CDMW_LOG_FUNCTION_RETURN(FT_CLIENT_REQUEST_INTERCEPTOR_NAME);
    return CORBA::string_dup(FT_CLIENT_REQUEST_INTERCEPTOR_NAME);
}

void
RequestTimeoutClientRequestInterceptor_impl::destroy() 
    throw (CORBA::SystemException)
{
    CDMW_LOG_FUNCTION(FTLogger::GetLogger(),"");
}

void
RequestTimeoutClientRequestInterceptor_impl::send_request(
        PortableInterceptor::ClientRequestInfo_ptr ri)
    throw (CORBA::SystemException, PortableInterceptor::ForwardRequest)
{
    if (ri->operation() == FORBIDDEN_OPERATION) 
    {
        return;
    }
    // The log below must not be recorded if the intercepted operation is add_log(). Else if Log Collector
    // is used we enter in a cyclic recording of the log!
    // This explains why the macro is called only after the condition if() 
    CDMW_LOG_FUNCTION(FTLogger::GetLogger(),
                      " for operation: " << ri->operation());
    
    try 
    {     
        // Get request_id
        CORBA::ULong request_id;
        CORBA::Any_var any = ri->get_slot(m_request_id_slot_id);
            
        if (!(any.in() >>= request_id))
        {
            CORBA::TypeCode_var tc = any->type();

            if (tc->kind() == CORBA::tk_null)
            {
                // the request invocation is done in the context of an incoming
                // request processing.
                CDMW_DEBUG(FTLogger::GetLogger(),
                           "request_id_slot is not set" );
            }
            else
            {
                CDMW_ERROR(FTLogger::GetLogger(),
                           "Unexpected type stored in request_id_slot");
                CORBA::INTERNAL ex(OrbSupport::INTERNALFaultToleranceError, 
                               CORBA::COMPLETED_NO);
                CDMW_LOG_FUNCTION_EXCEPTION(ex);
                throw ex;
            }
        }
        else
        {
            CDMW_DEBUG(FTLogger::GetLogger(), 
                       "Calling waiting_response for request_id=" << request_id);
            RequestTimeoutLogger::Get_instance()->waiting_response(
                       request_id);
        }
    } 
    catch (const PortableInterceptor::InvalidSlot& ) 
    {
        // This slot that has not been allocated
        CDMW_ERROR(FTLogger::GetLogger(),
                   "ERROR: InvalidSlot exception raised!" );
        CORBA::INTERNAL ex(OrbSupport::INTERNALFaultToleranceError, 
                           CORBA::COMPLETED_NO);
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw ex;
    }
    catch (const Cdmw::Common::NotFoundException& )
    {
        // The request id is not found 
        CORBA::INTERNAL ex(OrbSupport::INTERNALFaultToleranceError, 
                           CORBA::COMPLETED_NO);
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw ex;
    }
    catch (const CORBA::SystemException& ex) 
    {
        CDMW_DEBUG(FTLogger::GetLogger(),
                   "CORBA System exception raised in send_request!\n" << ex);
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw;
    }
}

void
RequestTimeoutClientRequestInterceptor_impl::send_poll(PortableInterceptor::ClientRequestInfo_ptr ri)
        throw (CORBA::SystemException)
{
    CDMW_LOG_FUNCTION(FTLogger::GetLogger(),
                      "for operation: " << ri->operation());
}


void
RequestTimeoutClientRequestInterceptor_impl::receive_reply(PortableInterceptor::ClientRequestInfo_ptr ri)
    throw (CORBA::SystemException)
{
    if (ri->operation() == FORBIDDEN_OPERATION) 
    {
        return;
    }
    // The log below must not be recorded if the intercepted operation is add_log(). Else if Log Collector
    // is used we enter in a cyclic recording of the log!
    // This explains why the macro is called only after the condition if() 
    CDMW_LOG_FUNCTION(FTLogger::GetLogger(),
                      " for operation: " << ri->operation());

    try 
    {     
        // Get request_id
        CORBA::ULong   request_id;
        CORBA::Any_var any = ri->get_slot(m_request_id_slot_id);
            
        if (!(any.in() >>= request_id))
        {
            CORBA::TypeCode_var tc = any->type();
            
            if (tc->kind() == CORBA::tk_null)
            {
                // the request invocation is not the result of an incoming request
                // processing.
                CDMW_DEBUG(FTLogger::GetLogger(),
                           "request_id_slot is not set" );
            }
            else
            {
                CDMW_ERROR(FTLogger::GetLogger(),
                           "Unexpected type stored in request_id_slot");
                CORBA::INTERNAL ex(OrbSupport::INTERNALFaultToleranceError, 
                               CORBA::COMPLETED_NO);
                CDMW_LOG_FUNCTION_EXCEPTION(ex);
                throw ex;
            }
        }
        else
        {
            CDMW_DEBUG(FTLogger::GetLogger(),
                       "Calling remove_waiting_response for request_id=" << request_id);
            RequestTimeoutLogger::Get_instance()->remove_waiting_response(request_id);
        }

    } 
    catch (const PortableInterceptor::InvalidSlot& ) 
    {
        // This slot that has not been allocated
        CDMW_ERROR(FTLogger::GetLogger(),
                   "ERROR: InvalidSlot exception raised!" );
        CORBA::INTERNAL ex(OrbSupport::INTERNALFaultToleranceError, CORBA::COMPLETED_NO);
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw ex;
    }
    catch (const Cdmw::Common::NotFoundException& )
    {
        // The request id is not found 
        CORBA::INTERNAL ex(OrbSupport::INTERNALFaultToleranceError, 
                           CORBA::COMPLETED_NO);
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw ex;
    }
    catch (const CORBA::SystemException& ex) 
    {
        CDMW_DEBUG(FTLogger::GetLogger(),
                   "CORBA System exception raised in send_request!\n" << ex);
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw;
    }
}


void
RequestTimeoutClientRequestInterceptor_impl::receive_exception(PortableInterceptor::ClientRequestInfo_ptr ri)
        throw (CORBA::SystemException, PortableInterceptor::ForwardRequest)
{
    if (ri->operation() == FORBIDDEN_OPERATION) 
    {
        return;
    }
    // The log below must not be recorded if the intercepted operation is add_log(). Else if Log Collector
    // is used we enter in a cyclic recording of the log!
    // This explains why the macro is called only after the condition if() 
    CDMW_LOG_FUNCTION(FTLogger::GetLogger()," for operation: " << ri->operation());

    try 
    {     
        // Get request_id
        CORBA::ULong   request_id;
        CORBA::Any_var any = ri->get_slot(m_request_id_slot_id);
            
        if (!(any.in() >>= request_id))
        {
            CORBA::TypeCode_var tc = any->type();

            if (tc->kind() == CORBA::tk_null)
            {
                // the request invocation is done in the context of an incoming
                // request processing.
                CDMW_DEBUG(FTLogger::GetLogger(),
                           "request_id_slot is not set" );
            }
            else
            {
                CDMW_ERROR(FTLogger::GetLogger(),
                           "Unexpected type stored in request_id_slot");
                CORBA::INTERNAL ex(OrbSupport::INTERNALFaultToleranceError, 
                               CORBA::COMPLETED_NO);
                CDMW_LOG_FUNCTION_EXCEPTION(ex);
                throw ex;
            }
        }
        else
        {
            CDMW_DEBUG(FTLogger::GetLogger(), 
                       "Calling remove_waiting_response for request_id=" 
                       << request_id);
            RequestTimeoutLogger::Get_instance()->remove_waiting_response(
                  request_id);
        }

    } 
    catch (const PortableInterceptor::InvalidSlot& ) 
    {
        // This slot that has not been allocated
        CDMW_ERROR(FTLogger::GetLogger(),
                   "ERROR: InvalidSlot exception raised!" );
        CORBA::INTERNAL ex(OrbSupport::INTERNALFaultToleranceError, CORBA::COMPLETED_NO);
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw ex;
    }
    catch (const Cdmw::Common::NotFoundException& )
    {
        // The request id is not found 
        CORBA::INTERNAL ex(OrbSupport::INTERNALFaultToleranceError, 
                           CORBA::COMPLETED_NO);
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw ex;
    }
    catch (const CORBA::SystemException& ex) 
    {
        CDMW_DEBUG(FTLogger::GetLogger(),
                   "CORBA System exception raised in send_request!\n" << ex);
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw;
    }
}

void
RequestTimeoutClientRequestInterceptor_impl::receive_other(PortableInterceptor::ClientRequestInfo_ptr ri)
        throw (CORBA::SystemException, PortableInterceptor::ForwardRequest)
{
    if (ri->operation() == FORBIDDEN_OPERATION) 
    {
        return;
    }
    // The log below must not be recorded if the intercepted operation is add_log(). Else if Log Collector
    // is used we enter in a cyclic recording of the log!
    // This explains why the macro is called only after the condition if() 
    CDMW_LOG_FUNCTION(FTLogger::GetLogger(),
                      " for operation: " << ri->operation());

    try 
    {     
        // Get request_id
        CORBA::ULong   request_id;
        CORBA::Any_var any = ri->get_slot(m_request_id_slot_id);
            
        if (!(any.in() >>= request_id))
        {
            CORBA::TypeCode_var tc = any->type();

            if (tc->kind() == CORBA::tk_null)
            {
                // the request invocation is done in the context of an incoming
                // request processing.
                CDMW_DEBUG(FTLogger::GetLogger(),
                           "request_id_slot is not set" );
            }
            else
            {
                CDMW_ERROR(FTLogger::GetLogger(),
                           "Unexpected type stored in request_id_slot");
                CORBA::INTERNAL ex(OrbSupport::INTERNALFaultToleranceError, 
                               CORBA::COMPLETED_NO);
                CDMW_LOG_FUNCTION_EXCEPTION(ex);
                throw ex;
            }
        }
        else
        {
            CDMW_DEBUG(FTLogger::GetLogger(), 
                       "Calling remove_waiting_response for request_id=" 
                       << request_id);
            RequestTimeoutLogger::Get_instance()->remove_waiting_response(request_id);
        }

    } 
    catch (const PortableInterceptor::InvalidSlot& ) 
    {
        // This slot that has not been allocated
        CDMW_ERROR(FTLogger::GetLogger(),
                   "ERROR: InvalidSlot exception raised!" );
        CORBA::INTERNAL ex(OrbSupport::INTERNALFaultToleranceError, CORBA::COMPLETED_NO);
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw ex;
    }
    catch (const Cdmw::Common::NotFoundException& )
    {
        // The request id is not found 
        CORBA::INTERNAL ex(OrbSupport::INTERNALFaultToleranceError, 
                           CORBA::COMPLETED_NO);
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw ex;
    }
    catch (const CORBA::SystemException& ex) 
    {
        CDMW_DEBUG(FTLogger::GetLogger(),
                   "CORBA System exception raised in send_request!\n" << ex);
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw;
    }
}

} // End of namespace Watchdog

} // End of namespace FT

} // End of namespace Cdmw
