/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO 
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2009. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


// Cdmw Files
#include <Foundation/common/Assert.hpp> 
#include <Foundation/orbsupport/CORBA.hpp>
#include <Foundation/orbsupport/ExceptionMinorCodes.hpp>
#include <Foundation/osthreads/MutexGuard.hpp>
#include <Foundation/ossupport/OS.hpp>
#include <Foundation/commonsvcs/time/Util.hpp>
#include <FaultTolerance/ftcommon/MacroDef.hpp>
#include <idllib/FT.stub.hpp>

#include <ftwatchdog/FTRequestTimeoutLogger.hpp>

#include <FaultTolerance/ftcommon/FTLogger.hpp>
#include <Foundation/logging/FunctionLogger.hpp>


namespace Cdmw
{
    
namespace FT
{

namespace Watchdog
{
RequestTimeoutLogger*  RequestTimeoutLogger::M_instance = 0;

void RequestTimeoutLogger::Init_request_timeout_logger(CORBA::ULong milliseconds)
{
    CDMW_LOG_FUNCTION(FTLogger::GetLogger(),
                      "timeout=" << milliseconds << " ms");

    CDMW_INFO(FTLogger::GetLogger(),
              "Maximum duration for processing incoming request is = "
              << milliseconds << " ms");
    
    CDMW_ASSERT(!M_instance);

    M_instance = new RequestTimeoutLogger();
    // convert timeout in TimeBase::TimeT (100 ns)
    M_instance->m_timeout = (TimeBase::TimeT) milliseconds*10000;
}

RequestTimeoutLogger* RequestTimeoutLogger::Get_instance() 
    throw() 
{
    CDMW_LOG_FUNCTION(FTLogger::GetLogger(),"");
    CDMW_ASSERT(M_instance);
    return M_instance;
}

RequestTimeoutLogger::RequestTimeoutLogger()
{
    CDMW_LOG_FUNCTION(FTLogger::GetLogger(),"");
}

//fixing memory leak
int RequestTimeoutLogger::Cleanup_RequestTimeoutLogger()
{
  if (M_instance != 0)
  {
    delete M_instance;
    M_instance = 0;
  }
  return 0;
}

RequestTimeoutLogger::~RequestTimeoutLogger() throw() 
{
    CDMW_LOG_FUNCTION(FTLogger::GetLogger(),"");
}

void RequestTimeoutLogger::set_timeout(CORBA::ULong milliseconds)
{
    CDMW_LOG_FUNCTION(FTLogger::GetLogger(),
                      "timeout=" << milliseconds << " ms");
    CDMW_MUTEX_GUARD(m_mutex);

    CDMW_INFO(FTLogger::GetLogger(),
              "Resetting Maximum duration for processing incoming request to = "
              << milliseconds << " ms");
    // convert timeout in TimeBase::TimeT (100 ns)
    m_timeout = (TimeBase::TimeT) milliseconds*10000;
}


void RequestTimeoutLogger::entering_request(CORBA::ULong request_id)
{
    CDMW_LOG_FUNCTION(FTLogger::GetLogger(),"request_id" << request_id);

    CDMW_MUTEX_GUARD(m_mutex);
    RequestInfoMap::iterator rt_iter = m_request_infos.find(request_id);

    if (rt_iter != m_request_infos.end())
    {
        // request_id already registered (impossible)
        CDMW_ASSERT(false);
    }
    else
    {
        // register T0 request
        TimeBase::TimeT current_time 
            = Cdmw::CommonSvcs::Time::Util::get_current_time();
        CDMW_DEBUG(FTLogger::GetLogger(), 
                  "Request " << request_id << 
                  " T0=" << current_time << "100 ns");
        
        RequestInfo request_info(current_time);
        m_request_infos.insert(RequestInfoMap::value_type(request_id,
                                                          request_info));
    }
}

void RequestTimeoutLogger::leaving_request(CORBA::ULong request_id)
        throw (Cdmw::Common::NotFoundException)
{
    CDMW_LOG_FUNCTION(FTLogger::GetLogger(),"request_id" << request_id);
    CDMW_MUTEX_GUARD(m_mutex);
    CDMW_DEBUG(FTLogger::GetLogger(), 
                  "Remove Request " << request_id );

    if (m_request_infos.erase(request_id) == 0)
    {
        throw Cdmw::Common::NotFoundException();
    }
}

void RequestTimeoutLogger::waiting_response(CORBA::ULong request_id)
        throw (Cdmw::Common::NotFoundException)
{
    CDMW_LOG_FUNCTION(FTLogger::GetLogger(),"request_id" << request_id);

    CDMW_DEBUG(FTLogger::GetLogger(), 
               "Add waiting flag to request " << request_id);

    CDMW_MUTEX_GUARD(m_mutex);

    RequestInfoMap::iterator rt_iter = m_request_infos.find(request_id);

    if (rt_iter == m_request_infos.end())
    {
        throw Cdmw::Common::NotFoundException();
    }
    else
    {
        RequestInfo* request_info = &((*rt_iter).second);

        if (request_info->waiting_response_t0 > 0)
        {
            CDMW_ASSERT(false);
        }
        else
        {
            request_info->waiting_response_t0  
               = Cdmw::CommonSvcs::Time::Util::get_current_time();
        }
    }
}

void RequestTimeoutLogger::remove_waiting_response(CORBA::ULong request_id)
        throw (Cdmw::Common::NotFoundException)
{
    CDMW_LOG_FUNCTION(FTLogger::GetLogger(), "request_id=" << request_id);
    CDMW_MUTEX_GUARD(m_mutex);
    RequestInfoMap::iterator rt_iter = m_request_infos.find(request_id);

    if (rt_iter == m_request_infos.end())
    {
        throw Cdmw::Common::NotFoundException();
    }
    else
    {
        RequestInfo* request_info = &((*rt_iter).second);
        
        if (request_info->waiting_response_t0 == 0)
        {
            CDMW_ASSERT(false);
        }
        else
        {
            // update request info field 
            TimeBase::TimeT current_time
                = Cdmw::CommonSvcs::Time::Util::get_current_time();

            TimeBase::TimeT duration 
                = current_time - request_info->waiting_response_t0;

            CDMW_DEBUG(FTLogger::GetLogger(), 
                       "Remove waiting flag to request " << request_id
                       << "duration was " << duration << " 100 ns");
    
            request_info->time_0 = request_info->time_0 + duration;
            request_info->waiting_response_t0 = 0;
        }
    }
}

bool RequestTimeoutLogger::check_timeouts_expiration()
{
    CDMW_LOG_FUNCTION(FTLogger::GetLogger(), "");
    CDMW_MUTEX_GUARD(m_mutex);

    RequestInfoMap::iterator rt_iter = m_request_infos.begin();

    TimeBase::TimeT current_time 
        = Cdmw::CommonSvcs::Time::Util::get_current_time();

    CDMW_DEBUG(FTLogger::GetLogger(), "check_timeouts_expiration current_time "
                                     << current_time << " 100ns");

    while (rt_iter != m_request_infos.end())
    {
        RequestInfoMap::value_type request_info_vt = *rt_iter;

        RequestInfo* request_info = &(request_info_vt.second);
                  
        TimeBase::TimeT duration = current_time - request_info->time_0;
        CDMW_DEBUG(FTLogger::GetLogger(), 
                      "duration for request " << request_info_vt.first
                      << " duration=" << duration << " 100 ns");

        if (duration > m_timeout)
        {
            CDMW_DEBUG(FTLogger::GetLogger(), 
                      "Timeout for request " << request_info_vt.first
                      << " duration=" << duration << " 100 ns");

            if (request_info->waiting_response_t0 > 0)
            {
                CDMW_DEBUG(FTLogger::GetLogger(), 
                           "Request " << request_info_vt.first
                           << " is waiting for a response");
            }
            else
            {
                return true;
            }
        }
        rt_iter++;
    }

    return false;
}


} // End of namespace Watchdog

} // End of namespace FT

} // End of namespace Cdmw
