/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO 
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2009. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


// Cdmw Files
#include <Foundation/common/Assert.hpp> 
#include <Foundation/orbsupport/CORBA.hpp>
#include <Foundation/orbsupport/ExceptionMinorCodes.hpp>
#include <Foundation/ossupport/OS.hpp>
#include <Foundation/osthreads/MutexGuard.hpp>

#include <FaultTolerance/ftcommon/FTLogger.hpp>
#include <Foundation/logging/FunctionLogger.hpp>

// Local Files
#include <ftwatchdog/FTRequestTimeoutServerRequestInterceptor_impl.hpp>
#include <ftwatchdog/FTRequestTimeoutLogger.hpp>

#include <FaultTolerance/ftcommon/MacroDef.hpp>
#include <idllib/FT.stub.hpp>

#include <sstream>


namespace 
{
    const PortableInterceptor::ORBInitInfo::ObjectId FT_SERVER_REQUEST_INTERCEPTOR_NAME 
        = "FTRequestTimeoutServerRequestInterceptor";

} // end of anonymous namespace


namespace Cdmw
{
    
namespace FT
{

namespace Watchdog
{

CORBA::ULong RequestTimeoutServerRequestInterceptor_impl::M_request_counter = 0;

RequestTimeoutServerRequestInterceptor_impl::RequestTimeoutServerRequestInterceptor_impl(
                 PortableInterceptor::ORBInitInfo_ptr info,
                 const PortableInterceptor::SlotId  request_id_slot_id)
    : m_request_id_slot_id(request_id_slot_id)
{
    CDMW_LOG_FUNCTION(FTLogger::GetLogger()," request_id_slot_id=" << m_request_id_slot_id);
}

char*
RequestTimeoutServerRequestInterceptor_impl::name() 
    throw (CORBA::SystemException)
{
    CDMW_LOG_FUNCTION(FTLogger::GetLogger(),"");
    CDMW_LOG_FUNCTION_RETURN(FT_SERVER_REQUEST_INTERCEPTOR_NAME);
    return CORBA::string_dup(FT_SERVER_REQUEST_INTERCEPTOR_NAME);
}

void
RequestTimeoutServerRequestInterceptor_impl::destroy() 
    throw (CORBA::SystemException)
{
    CDMW_LOG_FUNCTION(FTLogger::GetLogger(),"");
}


void
RequestTimeoutServerRequestInterceptor_impl::cdmw_ft_interception_point(
      PortableInterceptor::ServerRequestInfo_ptr ri,
      CORBA::OctetSeq_out ocs)
    throw (CORBA::SystemException, PortableInterceptor::ForwardRequest)
{
    CDMW_LOG_FUNCTION(FTLogger::GetLogger(),
                      " for operation: " << ri->operation());

    CORBA::ULong request_counter;
    {
        CDMW_MUTEX_GUARD(m_rc_mutex);

        // Increment here M_request_counter because we are sure the request will
        // be treated at application level.
        M_request_counter++;
        request_counter = M_request_counter;
    }
    
    CORBA::Any slotData;
    slotData <<= request_counter;

    try
    {
        CDMW_DEBUG(FTLogger::GetLogger(), 
                   "cdmw_ft_interception_point set_slot m_request_id_slot_id=" 
                   << m_request_id_slot_id);
        ri->set_slot(m_request_id_slot_id, slotData);
    }
    catch (const PortableInterceptor::InvalidSlot& ) 
    {
        // This slot that has not been allocated
        CDMW_ERROR(FTLogger::GetLogger(),
                   "ERROR: InvalidSlot exception raised!" );
        CORBA::INTERNAL ex(OrbSupport::INTERNALFaultToleranceError, 
                           CORBA::COMPLETED_NO);
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw ex;
    }

    CDMW_DEBUG(FTLogger::GetLogger(),
               "cdmw_ft_interception_point logs " << request_counter);

    // log T0 of the request treatement
    RequestTimeoutLogger::Get_instance()->entering_request(request_counter);
}



void
RequestTimeoutServerRequestInterceptor_impl::receive_request_service_contexts(
        PortableInterceptor::ServerRequestInfo_ptr ri)
    throw (CORBA::SystemException, PortableInterceptor::ForwardRequest)
{
    CDMW_LOG_FUNCTION(FTLogger::GetLogger(),
                      " for operation: " << ri->operation());


    if (false == m_bUseFTInterceptionPoint) {

       CORBA::ULong request_counter;
       {
           CDMW_MUTEX_GUARD(m_rc_mutex);

           // Increment here M_request_counter because we are sure the request will
           // be treated at application level.
           M_request_counter++;
           request_counter = M_request_counter;
       }
    
       CORBA::Any slotData;
       slotData <<= request_counter;

       try
       {
           CDMW_DEBUG(FTLogger::GetLogger(), 
                      "receive_request_service_contexts set_slot m_request_id_slot_id=" 
                      << m_request_id_slot_id);
           ri->set_slot(m_request_id_slot_id, slotData);
       }
       catch (const PortableInterceptor::InvalidSlot& ) 
       {
           // This slot that has not been allocated
           CDMW_ERROR(FTLogger::GetLogger(),
                      "ERROR: InvalidSlot exception raised!" );
           CORBA::INTERNAL ex(OrbSupport::INTERNALFaultToleranceError, 
                              CORBA::COMPLETED_NO);
           CDMW_LOG_FUNCTION_EXCEPTION(ex);
           throw ex;
       }
   
       CDMW_DEBUG(FTLogger::GetLogger(),
                  "receive_request_service_contexts logs " << request_counter);
   
       // log T0 of the request treatement
       RequestTimeoutLogger::Get_instance()->entering_request(request_counter);
   }
}

void
RequestTimeoutServerRequestInterceptor_impl::receive_request(PortableInterceptor::ServerRequestInfo_ptr ri)
        throw (CORBA::SystemException, PortableInterceptor::ForwardRequest)
{
    CDMW_LOG_FUNCTION(FTLogger::GetLogger(),
                      " for operation: " << ri->operation());
}


void
RequestTimeoutServerRequestInterceptor_impl::send_reply(PortableInterceptor::ServerRequestInfo_ptr ri)
    throw (CORBA::SystemException)
{
    CDMW_LOG_FUNCTION(FTLogger::GetLogger()," for operation: " << ri->operation());
    
    remove_current_timeout_request_log(ri);
}

void
RequestTimeoutServerRequestInterceptor_impl::send_other(PortableInterceptor::ServerRequestInfo_ptr ri)
        throw (CORBA::SystemException, PortableInterceptor::ForwardRequest)
{
    CDMW_LOG_FUNCTION(FTLogger::GetLogger(),
                      " for operation: " << ri->operation());

    remove_current_timeout_request_log(ri);
}

void
RequestTimeoutServerRequestInterceptor_impl::send_exception(PortableInterceptor::ServerRequestInfo_ptr ri)
        throw (CORBA::SystemException, PortableInterceptor::ForwardRequest)
{
    CDMW_LOG_FUNCTION(FTLogger::GetLogger(),
                      " for operation: " << ri->operation());
    
    remove_current_timeout_request_log(ri);
}

void 
RequestTimeoutServerRequestInterceptor_impl::remove_current_timeout_request_log(
      PortableInterceptor::ServerRequestInfo_ptr ri)
       throw (CORBA::SystemException)
{
    CDMW_LOG_FUNCTION(FTLogger::GetLogger(),
                      " for operation: " << ri->operation());

    try
    {
        CDMW_DEBUG(FTLogger::GetLogger(), 
                   "get_slot m_request_id_slot_id=" << m_request_id_slot_id);

        CORBA::Any_var any = ri->get_slot(m_request_id_slot_id);

        CORBA::ULong request_id;
        if (!(any.in() >>= request_id))
        {
            CDMW_ERROR(FTLogger::GetLogger(),
                       "ERROR: unexpected data returned by get_slot" );
            CORBA::INTERNAL ex(OrbSupport::INTERNALFaultToleranceError, 
                               CORBA::COMPLETED_NO);
            CDMW_LOG_FUNCTION_EXCEPTION(ex);
            throw ex;
        }

        try
        {
            RequestTimeoutLogger::Get_instance()->leaving_request(request_id);
        }
        catch (const Cdmw::Common::NotFoundException&)
        {
            CDMW_ERROR(FTLogger::GetLogger(),
                       "ERROR: NotFoundException raised when removing request"
                       << request_id);
            CORBA::INTERNAL ex(OrbSupport::INTERNALFaultToleranceError, 
                               CORBA::COMPLETED_NO);
            CDMW_LOG_FUNCTION_EXCEPTION(ex);
            throw ex;
        }
    }
    catch (const PortableInterceptor::InvalidSlot& ) 
    {
        // This slot that has not been allocated
        CDMW_ERROR(FTLogger::GetLogger(),
                   "ERROR: InvalidSlot exception raised!" );
        CORBA::INTERNAL ex(OrbSupport::INTERNALFaultToleranceError, 
                           CORBA::COMPLETED_NO);
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw ex;
    }

    try
    {
        // set_slot to empty to avoid mistake if the same thread is used
        // to send request outside an incoming request context.
        CORBA::Any slotData;
        ri->set_slot(m_request_id_slot_id, slotData);
    }
    catch (const PortableInterceptor::InvalidSlot& ) 
    {
        // This slot that has not been allocated
        CDMW_ERROR(FTLogger::GetLogger(),
                   "ERROR: InvalidSlot exception raised!" );
        CORBA::INTERNAL ex(OrbSupport::INTERNALFaultToleranceError, 
                           CORBA::COMPLETED_NO);
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw ex;
    }

    
}


} // End of namespace Watchdog

} // End of namespace FT

} // End of namespace Cdmw
