/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO 
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2009. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */
package cdmw.ft.watchdog;

import java.util.*;

import cdmw.common.*;
import cdmw.ft.*;
import cdmw.logging.*;

import static cdmw.logging.Logger.*;
import static cdmw.logging.FunctionLogger.*;

public class RequestTimeoutLogger {
    
    /** 
    * Class used to store incoming request time 
    * and outgoing request (called by the incoming request) time
    */
    private class RequestInfo {
        /**
         * T0 expressed in units of 100 nano seconds
         */
        private long time0;
        /**
         * sub request T0
         */
        private long waitingResponse_t0;
        
        public RequestInfo(long time) {
            this.waitingResponse_t0 = 0;
            this.time0 = time;
        }
    }
    
    /**
     * Singleton instance
     */
    private static RequestTimeoutLogger instance = null;
    
    /**
     * maximum duration of a request
     */
    private long timeout = 0; 
    
    /**
     * map of incoming request (T0/request_id)
     */
    private Map<Integer, RequestInfo> requestInfosMap = 
        Collections.synchronizedMap(new HashMap<Integer, RequestInfo>()); 

    /**
     * 
     * @return RequestTimeoutLogger
     */
    public static RequestTimeoutLogger getInstance() {
        CDMW_LOG_FUNCTION(FTLogger.GetLogger(), "");
        Assert.check(instance != null);
        
        return instance;
    }

    /**
     * init a RequestTimeoutLogger.
     *
     * @param milliseconds maximum elapse time of request processing for the server.
     * (expressed in ms)
     */
    public static void initRequestTimeoutLogger(int milliseconds) {
        CDMW_LOG_FUNCTION(FTLogger.GetLogger(), 
                          "timeout=" + milliseconds + " ms");
        CDMW_INFO(FTLogger.GetLogger(), 
            "Maximum duration for processing incoming request is = " + 
            milliseconds + " ms", 
            new Throwable());

        Assert.check(instance == null);
        instance = new RequestTimeoutLogger();
        
        // convert timeout in TimeBase.TimeT (100 ns)
        instance.timeout = ((long) milliseconds) * 10000;
    }

    /**
     * Constructor 
     */    
    private RequestTimeoutLogger() {
        CDMW_LOG_FUNCTION(FTLogger.GetLogger(), "");
    }

    /**
     * Reset the timeout to a new value.
     *
     * @param milliseconds maximum elapse time of request processing for the server.
     * (expressed in ms)
     */
     public void setTimeout(int milliseconds) {
        CDMW_LOG_FUNCTION(FTLogger.GetLogger(), 
            "timeout=" + milliseconds + " ms");

        CDMW_INFO(FTLogger.GetLogger(), 
            "Resetting Maximum duration for processing incoming request to = " + 
            milliseconds + " ms", 
            new Throwable());
            
        // convert timeout in TimeBase.TimeT (100 ns)
        this.timeout = ((long) milliseconds) * 10000;
    }

    /**
     * log the time of the request reception.
     *
     * @param request_id request unique identifier
     */
    public void enteringRequest(int requestId) {
        CDMW_LOG_FUNCTION(FTLogger.GetLogger(), 
            "requestId=" + requestId);

        Object rInfo = requestInfosMap.get(requestId);

        if (rInfo != null)  {
            // requestId already registered (impossible)
            Assert.check(false);
        }
        else {
            // register T0 request
            long currentTime = System.nanoTime() / 100;
            CDMW_DEBUG(FTLogger.GetLogger(), 
                       "Request " + requestId + " T0=" + currentTime + "100 ns", 
                       new Throwable());
                       
            RequestInfo requestInfo = new RequestInfo(currentTime);
            this.requestInfosMap.put(requestId, requestInfo);
        }
    }

    /**
     * Remove the request entry. This should be done when the answer is sent
     * back to the sender.
     *
     * @param request_id request unique identifier.
     */
    public void leavingRequest(int requestId) throws NotFoundException {
        CDMW_LOG_FUNCTION(FTLogger.GetLogger(), 
                          "requestId" + requestId);
        CDMW_DEBUG(FTLogger.GetLogger(), 
                   "Remove Request " + requestId, 
                   new Throwable()); 
        
        if (this.requestInfosMap.remove(requestId) == null) {
            throw new NotFoundException();
        }
    }

    /**
     * Indicate that to process the received request, the server send a request
     * to an other server.
     *
     * @param request_id request unique identifier.
     */
    public void waitingResponse(int requestId) throws NotFoundException { 
        CDMW_LOG_FUNCTION(FTLogger.GetLogger(), 
                          "requestId" + requestId);
        CDMW_DEBUG(FTLogger.GetLogger(), 
                   "Add waiting flag to request " + requestId, 
                   new Throwable()); 

        RequestInfo requestInfo = this.requestInfosMap.get(requestId);
        if (requestInfo == null) {
            throw new NotFoundException();
        }
        else {
            if (requestInfo.waitingResponse_t0 > 0) {
                Assert.check(false);
            } 
            else {
                requestInfo.waitingResponse_t0 = 
                    System.nanoTime() / 100;
            }
        }
    }

    /**
     * The server has received the response from an other server. The waiting
     * flag is removed and the duration of the request processing is 
     * added to the first request. 
     *
     * @param request_id request unique identifier.
     * @param duration   duration of the request processing.
     */
    public void removeWaitingResponse(int requestId) throws NotFoundException {
        CDMW_LOG_FUNCTION(FTLogger.GetLogger(), 
                          "requestId" + requestId);
                          
        RequestInfo requestInfo = this.requestInfosMap.get(requestId);
        if (requestInfo == null) {
            throw new NotFoundException();
        }
        else {
            if (requestInfo.waitingResponse_t0 == 0) {
                Assert.check(false);
            }
            else {
                // update request info field 
                long currentTime = System.nanoTime() / 100;
                long duration = currentTime - requestInfo.waitingResponse_t0;

                CDMW_DEBUG(FTLogger.GetLogger(), 
                           "Remove waiting flag to request " + requestId + 
                           "duration was " + duration + " 100 ns", 
                           new Throwable());

                requestInfo.time0 = requestInfo.time0 + duration;
                requestInfo.waitingResponse_t0 = 0;
            }
        }
    }

    /**
     * This operation check all the request processing doesn't last more than
     * expected
     *
     * @param request_id request unique identifier.
     * @param duration   duration of the request processing.
     */
    public boolean checkTimeoutsExpiration() {
        CDMW_LOG_FUNCTION(FTLogger.GetLogger(), "");

        long currentTime = System.nanoTime() / 100;
        
        CDMW_DEBUG(FTLogger.GetLogger(), 
                   "check_timeouts_expiration currentTime " + currentTime + " 100ns", 
                   new Throwable()); 

        synchronized (this.requestInfosMap) {
            
            for (Map.Entry<Integer,RequestInfo> entry : requestInfosMap.entrySet()) 
            {
                RequestInfo requestInfo = entry.getValue();
    
                long duration = currentTime - requestInfo.time0;
                
                CDMW_DEBUG(FTLogger.GetLogger(), 
                           "duration for request " + entry.getKey() + 
                           " duration=" + duration + " 100 ns", 
                           new Throwable());
                           
                if (duration > this.timeout) {
                    CDMW_DEBUG(FTLogger.GetLogger(),
                               "Timeout for request " + entry.getKey() + 
                               " duration=" + duration + " 100 ns", 
                               new Throwable());
    
                    if (requestInfo.waitingResponse_t0 > 0) {
                        CDMW_DEBUG(FTLogger.GetLogger(), 
                                   "Request " + entry.getKey() + 
                                   " is waiting for a response", 
                                   new Throwable());
                    }
                    else 
                    {
                        return true;
                    }
                }
            }
            
        }

        return false;
    }

} 
