/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO 
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2009. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

package cdmw.ft.watchdog;

import org.omg.CORBA.BAD_PARAM;
import org.omg.CORBA.SystemException;

import cdmw.ft.FTLogger;
import cdmw.ft.FTSystemProperties;
import cdmw.logging.LogManager;
import cdmw.orbsupport.ExceptionMinorCodes;

import static cdmw.logging.Logger.*;

/**
*Purpose:
*<p>    This class provides an init of the Fault Tolerance Framework
* capability (location fault detector part).
*
*Features:
*<p> Thread safe
*
*/
public class WatchdogServiceInit {

    /**
     * Default timeout value
     */
    private static final int CDMW_DEFAULT_WATCHDOG_TIMEOUT_VALUE = 900000; // 15 minutes";
    
    /**
     * Purpose:
     * <p> Initialises location fault detection mechanism : it creates server
     * request interceptor and client request interceptor able to log the time
     * of an incoming and outgoing request.
     *
     *@param args The list of parameter received of the command line
     *
     *@return none
     *
     *@exception CORBA.INTERNAL  The CDMW <B>INTERNAL</B> minor code indicates an
     * internal error occured within Cdmw FT library.
     *@exception CORBA.SystemException Any CORBA system exception that
     * may be raised by the underlying ORB.
     */ 
    public static void init(String[] args) throws SystemException {
        //Init the Logging library
        LogManager.Init(args);
        
        // get default request_timeout value if any 
        // (CDMW internal use for debug purpose if CDMW_init is not called)
        String defReqTimeoutStr = System.getProperty(
            FTSystemProperties.WATCHDOG_DEFAULT_TIMEOUT_PROPERTY_NAME, 
            String.valueOf(CDMW_DEFAULT_WATCHDOG_TIMEOUT_VALUE));
        System.setProperty(
            FTSystemProperties.WATCHDOG_DEFAULT_TIMEOUT_PROPERTY_NAME, 
            defReqTimeoutStr);
        
        // get request_timeout value
        String reqTimeoutStr = 
            System.getProperty(FTSystemProperties.WATCHDOG_TIMEOUT_PROPERTY_NAME,
            defReqTimeoutStr);
        // force property to be set since it will be retrieved in 
        // FTRequestTimeoutORBInitializer default constructor
        System.setProperty(
            FTSystemProperties.WATCHDOG_TIMEOUT_PROPERTY_NAME,
            reqTimeoutStr);

        try {    
            int requestTimeout = Integer.parseInt(reqTimeoutStr);
            if (requestTimeout > 0) {
                CDMW_INFO(FTLogger.GetLogger(), 
                    "register RequestTimeoutORBInitializer with request_timeout of " 
                    + requestTimeout + "ms", new Throwable()); 
    
                System.setProperty(
                    "org.omg.PortableInterceptor.ORBInitializerClass.FTRequestTimeoutORBInitializer", 
                    FTRequestTimeoutORBInitializer.class.getName());
    
            } else {
                CDMW_WARN(FTLogger.GetLogger(), 
                    "Invalid timeout value '" + requestTimeout + 
                    "'.RequestTimeoutORBInitializer will not be registered!", 
                    new Throwable());
            }
        } catch (NumberFormatException e) {
            CDMW_WARN(FTLogger.GetLogger(), 
                "Invalid timeout value '" + reqTimeoutStr + 
                "'.RequestTimeoutORBInitializer will not be registered!", 
                new Throwable());
        }
    }
    
    /**
     * Purpose:
     * <p> Resets the timeout value used in the location fault detection mechanism to a new value.
     * 
     *@param milliseconds maximum elapse time of request processing for the server.
     * (expressed in ms)
     *
     *@return none
     *
     *@exception none.
     */ 
    public static void setTimeout(int milliseconds) throws BAD_PARAM {
        CDMW_INFO(FTLogger.GetLogger(), "Setting a new timeout value of " + milliseconds + "ms", new Throwable());
        if (milliseconds > 0) {
            CDMW_INFO(FTLogger.GetLogger(), "Reseting RequestTimeoutORBInitializer with request_timeout of " + 
                                                    + milliseconds + "ms", new Throwable());
         
            RequestTimeoutLogger.getInstance().setTimeout(milliseconds);
            System.setProperty(FTSystemProperties.WATCHDOG_TIMEOUT_PROPERTY_NAME, Long.toString(milliseconds));
       } else {
          CDMW_ERROR(FTLogger.GetLogger(), "Invalid timeout value '" + milliseconds + 
                      "'. Value ignored in RequestTimeoutORBInitializer!",  new Throwable());
          
          throw new BAD_PARAM(ExceptionMinorCodes.BAD_PARAM, org.omg.CORBA.CompletionStatus.COMPLETED_NO);
       }
    }
}
