/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#ifndef CDMW_EXCEPTION_H
#define CDMW_EXCEPTION_H

#include <stdexcept>
#include <string>

namespace Cdmw
{
    namespace OsSupport
    {
        extern std::string get_fmt_current_localtime(char* formatString);
    };
};


namespace Cdmw
{

    /**
     * @brief Base class for Cdmw exceptions
     */
    class Exception : public std::exception
    {

        public:

            /**
             * @brief Define values for Exception severity
             */
            enum Severity {
                SAFE,
                FATAL,
                UNKNOWN
            };

            /**
             * Construct the exception object
             *
             * @param severity Severity of exception
             * @param info exception reason
             */
            Exception (Severity severity, std::string info);

            
            //! Copy Ctor
            
            Exception (const Exception& rhs);


           /**
            * Destructor.
            */        
            virtual ~Exception () throw();

            /**
             * Get full information about exception
             *
             * @return the exception full information : file, line , reason
             */
            virtual const char* what() const throw();

            /**
             * Get exception reason
             *
             * @return the exception reason 
             */
            const std::string& get_info() const;

            /**
             * Get file where exception has been raised
             *
             * @return the exception file 
             */
            const std::string& get_file() const;

            /**
             * Set file where exception has been raised
             *
             * @param file the exception file 
             */
            void set_file(const std::string& file);

            /**
             * Get line where exception has been raised
             *
             * @return the exception line 
             */
            int get_line() const;

            /**
             * Set line where exception has been raised
             *
             * @param line the exception line 
             */
            void set_line(int line);

            /**
             * Get time when exception has been raised
             *
             * @return the exception raise time 
             */
            const std::string& get_timestamp() const;

            /**
             * Set time when exception has been raised
             *
             * @param timestamp the exception raise time string
             */
            void set_timestamp(const std::string& timestamp);

            /**
             * Set time when exception has been raised
             *
             * @param timestamp the exception raise time in seconds since 01 January 1970
             */
            void set_timestamp(const long timestamp);

        private:
            /**
             * severity of exception
             */
            Severity m_severity;

            /**
             * reason of exception
             */
            const std::string m_info;

            /**
             * file of exception
             */
            std::string m_file;

            /**
             * line of exception
             */
            int m_line;

            /**
             * full information on exception
             */
            mutable std::string m_fullInfo;

            /**
            * Exception raise time
            */
            std::string m_timestamp;

            //! Ctor
            Exception();
    };

    /**
     * @brief Out Of Memory Exception
     */
    class OutOfMemoryException : public Exception
    {
        public:
            OutOfMemoryException();
    };


    /**
     * @brief Out Of Resource Exception.
     * 
     * Resource includes memory and persistent storage.
     */
    class OutOfResourcesException : public Exception
    {
        public:
            OutOfResourcesException();
    };


    /**
     * @brief Internal Error Exception
     */
    class InternalErrorException : public Exception
    {
        public:
            InternalErrorException(const std::string& parameter = "");
    };

    /**
     * @brief Already Done Exception
     */
    class AlreadyDoneException : public Exception
    {
        public:
            AlreadyDoneException();
    };

    /**
     * @brief Bad Order Exception
     */
    class BadOrderException : public Exception
    {
        public:
            BadOrderException()
                    : Exception( Exception::SAFE, "Bad order" )
            {}
    }

    ;

    /**
     * @brief Assertion Failed Exception
     */
    class AssertionFailedException : public Exception
    {

        public:
            AssertionFailedException();
    };


    /**
     * @brief Permission Denied Exception
     */
    class PermissionDeniedException : public Exception
    {
        public:
            PermissionDeniedException();
    };

    /**
     * @brief Bad Parameter Exception
     *
     * @param parameterName  the name of the conflicting parameter
     * @param parameterValue the value of this parameter
     */
    class BadParameterException : public Exception
    {

        public:
           /**
            * Constructor.
            * @param parameterName parameter that causes the problem
            * @param parameterValue wrong value attributed to the parameter
            */
            BadParameterException(
                const std::string& parameterName,
                const std::string& parameterValue);

            /**
             * Constructor with the given parameter.
             *
             * @param m message to be printed when exception raised
             */
            BadParameterException(const std::string& m);

            virtual ~BadParameterException() throw();
            virtual const char* what() const throw();


            const std::string m_parameterName;
            const std::string m_parameterValue;

        private:
            /** internal reason */
            std::string m_what;

    };


    /**
     * @brief Exception while initialization.
     */
    class InitErrorException : public Exception
    {
    public:
        
        /**
         * @brief Construct the exception object
         *
         * @param reason reason of the initialisation failure
         * @param severity Severity of exception
         */        
        InitErrorException( const std::string& reason,
                            Exception::Severity severity = Exception::FATAL);
        
        ~InitErrorException() throw() {}
    };


    /**
     * @brief Not Exist Exception
     */
    class NotExistException : public Exception
    {
    public:
        NotExistException();
    };

    /**
     * @brief Invalid Argument Exception
     */
    class InvalidArgumentException : public Exception
    {
    public:
        /**
         * @brief Construct the exception object
         *
         * @param argument name of the invalid argument
         */        
        InvalidArgumentException(const std::string& name);
    };

};


/**
 * @brief Exception output on std::ostream
 */
std::ostream& operator<< (std::ostream&, const Cdmw::Exception&);


#define CDMW_THROW( ExType ) do { ExType __ex; \
        __ex.set_file(__FILE__); \
        __ex.set_line(__LINE__); \
        throw __ex; } while(0)

#define CDMW_THROW1( __exType, arg_1 ) do { __exType __ex(arg_1) ; \
        __ex.set_file(__FILE__); \
        __ex.set_line(__LINE__); \
        throw __ex; }  while(0)

#define CDMW_THROW2( __exType, arg_1, arg_2 ) do { __exType __ex(arg_1, arg_2) ; \
        __ex.set_file(__FILE__); \
        __ex.set_line(__LINE__); \
        throw __ex; }  while(0)

#define CDMW_THROW3( __exType, arg_1, arg_2, arg_3 ) do { __exType __ex(arg_1, arg_2, arg_3) ; \
        __ex.set_file(__FILE__); \
        __ex.set_line(__LINE__); \
        throw __ex; }  while(0)


#endif // CDMW_EXCEPTION_H

