
/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#ifndef INCL_COMMON_PROPETIES_HPP
#define INCL_COMMON_PROPETIES_HPP

#include <Foundation/common/Exception.hpp>

#include <set>
#include <map>
#include <string>
#include <iostream>


/**
 * Root namespace for CDMW runtime.
 */

namespace Cdmw
{
    /**
     * Namespace for common types used by CDMW runtime.
     */

    namespace Common
    {

        /**
         *Purpose:
         *<p>    This class provides several helper tools
         *        for the manipulation of properties
         *
         *Features:
         *<p>    none
         *
         */
        class Properties
        {


        public:
            typedef std::map<std::string,std::string> PropertiesMap;
            typedef std::set<std::string> PropertyNames;

            /**
             * Default constructor.
             */
            Properties();

            /**
             * Copy constructor.
             */
            Properties(const Properties*);

            /**
             * Copy constructor.
             */
            Properties(const Properties&);

            Properties& operator=(const Properties&);

            /**
             * destructor
             */
            virtual ~Properties() throw();

            /**
             * Purpose:
             * <p> Set a value 
             * 
             *@param name a the name of the value
             *@param value the value             
             *
             *@return void
             *
             */
            virtual void
            set_property(const std::string& name, 
                         const std::string& value);
            
            /**
             * Purpose:
             * <p> Get a value 
             * 
             *@param name the name of the value
             *@param default_value a value provided by default
             *
             *@return the value or if not found default_value
             *
             */
            virtual std::string
            get_property(const std::string& name,
                         const std::string& default_value = "") const;
            
           
            /**
             * Purpose:
             * <p> Return a list of all the properties 
             * saved
             *@param list of all the property names
             *
             *@return void
             *
             */
            virtual void
            property_names(PropertyNames& list) const;

            /**
             * Purpose:
             * <p> Load properties from input stream. Format is \<name\>=\<value\>; comments start with
             *  '!' or '#' character.
             *
             * @param is input stream
             */
            virtual void
            load(std::istream& is)
                throw (BadParameterException);

            /**
             * Purpose:
             * <p> Writes properties in output stream. 
             * Format is \<name\>=\<value\>
             *
             * @param os output stream
             */
            virtual void
            store(std::ostream& os) const;

            /** 
             * @see load()
             */
            friend std::istream& operator>>(std::istream& is,Properties& properties)
                throw(BadParameterException);

            /** 
             * @see store()
             */
            friend std::ostream& operator<<(std::ostream& os,const Properties& properties);
            
        protected:
            /**
             * Purpose:
             * <p> Get a value 
             * 
             *@param name the name of the value
             *@param default_value a value provided by default
             *
             *@return the value or if not found default_value
             *
             */
            std::string do_get_property(const std::string& name,
                                        const std::string& default_value) const;
            
            /**
             * Purpose:
             * <p> Return a list of all the properties 
             * saved
             *@param list of all the property names
             *
             *@return void
             *
             */
            void do_property_names(PropertyNames& list) const;

            /**
             * Retrieve a property value.
             * @param env definition of properties
             * @param name property to retrieve
             * @param value found value
             *
             * @exception BadParameterException if the value is missing on the property
             * @return true if ok, false if it was not a property -DXX=YY

            */
            bool explode_cmdline_property(const std::string& env, std::string & name, std::string & value)
                throw (BadParameterException);
            
            
        private:
            PropertiesMap  m_properties_map;
            Properties* m_defaults;

            /**
             * Purpose:
             * <p> delete the default properties
             *
             */
            void delete_default_properties () throw();

            /**
             * Purpose:
             * <p> copy the default properties
             *
             *@param prop these properties are copied and become
             * the default properties. The attribut m_defaults points
             * on this copy.
             */
            void copy_default_properties ( const Properties* prop);
                                    
        }; // End class Properties
        
    } // End namespace Common
} // End namespace Cdmw



#endif // INCL_COMMON_PROPERTIES_HPP

