
/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

#include <Foundation/common/Properties.hpp>
#include <Foundation/common/Assert.hpp>
#include <vector>
#include <iterator>
#include <iostream>

namespace
{
    inline bool is_comment_starter(char c)
    {
        return ((c == '!') || (c == '#'));
    }
    inline bool is_separator(char c)
    {
        return c == '=';
    }
    inline bool not_separator(char c)
    {
        return !is_separator(c);
    }
    
}; // End of anonymous namespace
/**
* Root namespace for CDMW runtime.
*/

namespace Cdmw
{
    /**
    * Namespace for common types used by CDMW runtime.
    */

    namespace Common
    {

        Properties::Properties()
            : m_defaults(0)
        {
        }
        Properties::Properties(const Properties * prop)
            : m_defaults(0)
        {
      copy_default_properties (prop);
        }
      
          void Properties::delete_default_properties () throw() {
        if ( m_defaults != 0 ) {
          
          Properties* default_p, *default2_p;
          
          for ( default_p = m_defaults; default_p->m_defaults != 0; default_p = default2_p  ) {
        default2_p = default_p->m_defaults;
        delete default_p;
          }
          
          delete default_p;
          
        }
      }

        Properties::~Properties() throw(){
      delete_default_properties ();
        }

        Properties& Properties::operator=(const Properties& prop)
    {
      // delete the previous default properties
      delete_default_properties ();

      m_properties_map = prop.m_properties_map;

      if(prop.m_defaults == 0){
        m_defaults = 0;
      }else{
        copy_default_properties (prop.m_defaults);
      }

      return *this;
        }

        void Properties::copy_default_properties ( const Properties* prop)
    {
          // PCR-0754 analysis part remark 1.5
          if ( prop == 0 ) { 
        std::cerr << "Cdmw: failed assertion in "__FILE__
              << ":" <<  __LINE__ << std::endl; 
        Cdmw::assertion_failed(); 
          }
          
          // Pointer on the properties to copy
          const Properties* src_p = prop;

          m_defaults = new Properties();
          // Pointer on the destination of the copy
          Properties* dst_p = m_defaults;
          dst_p->m_properties_map = src_p->m_properties_map;
          dst_p->m_defaults = 0;
          
          while ( src_p->m_defaults != 0){
        dst_p->m_defaults = new Properties();
        dst_p = dst_p->m_defaults;
        dst_p->m_defaults = 0;
        dst_p->m_properties_map = src_p->m_defaults->m_properties_map;
        src_p = src_p->m_defaults;
          }
    }


        Properties::Properties(const Properties & prop)
            : m_properties_map(prop.m_properties_map)
        {
      if(prop.m_defaults == 0){
        m_defaults = 0;
      }else{
        copy_default_properties (prop.m_defaults);
      }
        }
        
        void
        Properties::set_property(const std::string& name, 
                                 const std::string& value)
        {
            m_properties_map[name] = value;
        }
        
         
        std::string
        Properties::get_property(const std::string& name,
                                 const std::string& default_value) const
        {
            return do_get_property(name,default_value);
        }
        
        std::string
        Properties::do_get_property(const std::string& name,
                                 const std::string& default_value) const
        {
            PropertiesMap::const_iterator iter = m_properties_map.find(name);
            if (iter ==  m_properties_map.end())
            {
                if (m_defaults == 0)
                {
                    return default_value;
                } else { 
                    return m_defaults->get_property(name, default_value);
                }
                
            } else {
                return iter->second;
            }
            
        }
        
        void
        Properties::property_names(PropertyNames& s) const
        {
            do_property_names(s);
        }
        void
        Properties::do_property_names(PropertyNames& s) const
        {
            PropertyNames defs;
            if (m_defaults != 0)
                m_defaults->property_names(defs);

            PropertyNames stmp;
            std::insert_iterator<PropertyNames> ins(stmp, stmp.begin());

            for (PropertiesMap::const_iterator iter = m_properties_map.begin();
                 iter != m_properties_map.end();
                 ++iter)
            {
                *ins = iter->first;
            }
            std::insert_iterator<PropertyNames> res(s, s.begin());
            std::set_union(stmp.begin(), stmp.end(),
                           defs.begin(), defs.end(),
                           res);
        }
        bool
        Properties::explode_cmdline_property(const std::string& env, std::string & name, std::string & value)
            throw (BadParameterException)
        {
            std::string arg(env);
            std::string::const_iterator i = arg.begin();
            std::string::const_iterator end = arg.end();
            // find end of ior
            std::string::const_iterator j = std::find_if(i,end,is_separator);
            if (j == end) {
          return false;
            }
            // copy the characters in [i,j)
            std::string n("");
            // copy name
            if (i != end)
                n = std::string(i,j);
            // skip '=' character
            i = std::find_if(j,end,not_separator);
            // copy value
            if (i != end) {
                name = n;
                value = std::string(i,end);
            } else {
                std::string v = "<Missing value!>";
                CDMW_THROW2( BadParameterException, n, v );
            }
            return true;
        }

        void
        Properties::load(std::istream& is)
            throw(BadParameterException)
        {
            typedef std::istream_iterator< std::vector<std::string>::value_type,
                                           char, std::char_traits<char>,ptrdiff_t> is_iter;
            std::vector<std::string> v;
            
            std::copy(is_iter(is), is_iter(), std::inserter(v, v.begin()));
            std::string name, value;
            for (unsigned int i = 0; i<v.size(); i++)
            {
                if ((v[i][0] != '#') && (v[i][0] != '!'))
                {
                    // Remove trailing comment
                    std::string::const_iterator beg = v[i].begin();
                    std::string::const_iterator end = v[i].end();
                    std::string::const_iterator j = std::find_if(beg,end,is_comment_starter);
                    
                    std::string line(v[i]);
                    // copy the characters in [beg,j)
                    if (j != end)
                        line = std::string(beg,j);
//                    std::cout<<"Import property :"<<line<<std::endl;
                    if (explode_cmdline_property(line,name,value))
            {
                      m_properties_map[name] = value;                    
            }
                }
            }
        }

        void
        Properties::store(std::ostream& os) const
        {
            typedef std::set<std::string> PropertyNames;
            PropertyNames defs;
            this->do_property_names(defs);
            
            
            for (PropertyNames::const_iterator iter = defs.begin();
                 iter != defs.end();
                 ++iter)
            {
                os << (*iter) << "=" << this->do_get_property(*iter,"") << std::endl;
            }

        }
        
        std::istream& operator>>(std::istream& is,Properties& properties)
            throw(BadParameterException)
        {
            properties.load(is);            
            return is;
        }
        
        
        std::ostream& operator<<(std::ostream& os,const Properties& properties)
        {
            properties.store(os);
            return os;
        }
        

                                    
    } // End namespace Common
} // End namespace Cdmw
