/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#ifndef INCL_CDMW_COMMNSVCS_INITUTILS_INIT_UTILS_IMPL_HPP
#define INCL_CDMW_COMMNSVCS_INITUTILS_INIT_UTILS_IMPL_HPP

#include <Foundation/common/Locations.hpp>
#include <Foundation/common/Exception.hpp>
#include <Foundation/common/NotFoundException.hpp>
#include <Foundation/osthreads/Mutex.hpp>
#include <Foundation/osthreads/MutexGuard.hpp>
#include <Foundation/logging/CdmwLoggingCollector.stub.hpp>
#include <Foundation/orbsupport/CORBA.hpp>

namespace Cdmw
{
    namespace CommonSvcs
    {

        extern std::string LOCAL_NAME_SERVICE;

        /**
         * Helper class that provides services used while initialization.
         * Singleton pattern.
         */
        class InitUtils
        {

        public:
            /**
             * Access to the singleton instance.
             */
            inline static
            InitUtils* Instance()
                throw()
            {
                CDMW_MUTEX_GUARD(M_mutex);
                if (M_instance == 0) {
                    M_instance = new InitUtils();
                }
                return M_instance;
            }

            /**
             * Registers as initial reference in the ORB object 
             * the specified Naming service.
             *
             * @param orb ORB instance
             * @param namingContext the NameService that is attached to name
             * LOCAL_NAME_SERVICE
             */
            void
            register_LocalNameService(
                CORBA::ORB_ptr orb,
                CosNaming::NamingContext_ptr namingContext
                )
                throw (CORBA::SystemException);

            /**
             * Retrieves the Naming service that is registered in the ORB
             * with the name LOCAL_NAME_SERVICE.
             *
             * @param orb ORB instance
             * @return the NameService reference, CORBA::nil if not found
             *
             * @exception NotFoundException 
             *      when the local NameService is not reachable
             * @exception BadParameterException 
             *      when the ORB that is used has registered a bad object 
             *      as local NameService
             */
            CosNaming::NamingContext_ptr
            get_LocalNameService(
                CORBA::ORB_ptr orb
                )
                throw (Cdmw::Common::NotFoundException,
                       Cdmw::BadParameterException);

            /**
             * Retrieves the Name server running at localhost to specified port.
             *
             * @param orb ORB instance
             * @param port system port where the NameServer is running
             * @return the found NamingContext object
             *
             * @exception NotFoundException 
             *      when the NameServer does not exist at given location
             * @exception BadParameterException
             *      when the reference does not locate a NameServer
             */
            CosNaming::NamingContext_ptr
            retrieve_NameService(
                CORBA::ORB_ptr orb,
                short port
                )
                throw (Cdmw::Common::NotFoundException,
                       Cdmw::BadParameterException);

            /**
             * Registers a LogCollector as logging delegate. The 
             * LogCollector must be located in the local NameService, which
             * can be retrieved either in the ORB with name LOCAL_NAME_SERVICE,
             * or with a corbaloc to localhost provided the system port is used
             * (assuming that reference is corbaloc::<port>/NameService).
             *
             * @param orb ORB instance
             * @param port system port where the NameService is running at
             * localhost
             * @param collectorName name to retrieve in the Naming service
             *
             * @exception NotFoundException 
             *      when the collector cannot be found in the NameService,
             *      or the NameService is not reachable
             * @exception BadParameterException 
             *      when a parameter is inconsistent, the NameService is
             *      invalid or the collector name is badly built
             * @exception OutOfMemoryException 
             *      the LogCollector delegate cannot be allocated
             */
            void
            activate_LogCollector(
                CORBA::ORB_ptr orb,
                short port = -1,
                const std::string collectorName = Cdmw::Common::Locations::DEFAULT_LOG_COLLECTOR_ID
                )
                throw (Cdmw::Common::NotFoundException,
                       Cdmw::BadParameterException,
                       Cdmw::OutOfMemoryException);

            /**
             * @see InitUtils#activate_LogCollector(CORBA::ORB_ptr,short,std::string)
             * 
             * The only difference with the alternate activate_LogCollector
             * is in regards to the Naming service that is here directly
             * referenced by a CORBA object.
             *
             * @param namingContext the Naming service to use
             */
            void
            activate_LogCollector(
                CosNaming::NamingContext_ptr namingContext,
                const std::string collectorName = Cdmw::Common::Locations::DEFAULT_LOG_COLLECTOR_ID
                )
                throw (Cdmw::Common::NotFoundException,
                       Cdmw::BadParameterException,
                       Cdmw::OutOfMemoryException);

            /**
             * Deactivates the existing LogCollector from the logging 
             * delegate role.
             * This method assumes the current LogCollector has been registered
             * with activate_LogCollector.
             * If no LogCollector is active, nothing is done.
             *
             * @see InitUtils#activate_LogCollector()
             */
            void
            deactivate_LogCollector();


        protected:
            /**
             * Not visible constructor.
             */
            InitUtils() {}

            /**
             * Destructor.
             */
            virtual
            ~InitUtils() throw() {}

            /**
             * Retrieves the LogCollector with specified name into the 
             * Name server.
             *
             * @param nameService reference to the NameContext object
             * @param collectorName the name used to locate the collector
             * @return the found LogCollector object
             *
             * @exception NotFoundException 
             *      when no LogCollector can be found at given location
             * @exception BadParameterException
             *      when the name does not locate a LogCollector object
             */
            CdmwLogging::LogCollector_ptr
            retrieve_LogCollector(
                CosNaming::NamingContext_ptr namingContext,
                const std::string & collectorName
                )
                throw (Cdmw::Common::NotFoundException,
                       Cdmw::BadParameterException);

        private:
            static InitUtils *M_instance; /**< Singleton object. */
            static Cdmw::OsSupport::Mutex M_mutex;
        };

    }; // end namespace CommonSvcs
}; // End namespace Cdmw

#endif // INCL_CDMW_COMMNSVCS_INITUTILS_INIT_UTILS_IMPL_HPP

