/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

#include <Foundation/commonsvcs/initutils/InitUtils.hpp>

#include <Foundation/logging/LogManager.hpp>
#include <Foundation/logging/LogManagerCollectorDelegate.hpp>
#include <Foundation/commonsvcs/naming/NamingInterface.hpp>
#include <Foundation/orbsupport/OrbSupport.hpp>
#include <Foundation/orbsupport/CORBASystemExceptionWrapper.hpp>
#include <Foundation/orbsupport/ExceptionMinorCodes.hpp>

// LV 12/03/2008
// TODO This should be moved to OsSupport, 
//      and content of OrbSupport.cpp relative to port using so well.
//      Actually a consistent port management is to be added in OsSupport.
//
#define IS_SYSTEM_PORT_VALID   (port >= 2^10 && port < 2^16)


std::string Cdmw::CommonSvcs::LOCAL_NAME_SERVICE = "LocalNameService";

// init static member
Cdmw::CommonSvcs::InitUtils* Cdmw::CommonSvcs::InitUtils::M_instance = 0;
Cdmw::OsSupport::Mutex Cdmw::CommonSvcs::InitUtils::M_mutex;

using namespace Cdmw::CommonSvcs;


void
InitUtils::register_LocalNameService(
    CORBA::ORB_ptr orb,
    CosNaming::NamingContext_ptr namingContext
    )
    throw (CORBA::SystemException)
{
    CDMW_ASSERT(!CORBA::is_nil(orb));
    CDMW_ASSERT(!CORBA::is_nil(namingContext));

    try
    {
        orb->register_initial_reference(
            LOCAL_NAME_SERVICE.c_str(), namingContext);
    }
    catch (const CORBA::ORB::InvalidName& )
    {
        CDMW_THROW2(
            Cdmw::OrbSupport::CORBASystemExceptionWrapperT<CORBA::INTERNAL>,
            CORBA::INTERNAL(Cdmw::OrbSupport::INTERNAL, CORBA::COMPLETED_NO),
            "InvalidName => Cannot register " + LOCAL_NAME_SERVICE);
    }
}

CosNaming::NamingContext_ptr
InitUtils::get_LocalNameService(
    CORBA::ORB_ptr orb
    )
    throw (Cdmw::Common::NotFoundException,
           Cdmw::BadParameterException)
{
    CDMW_ASSERT(!CORBA::is_nil(orb));

    CosNaming::NamingContext_var ns__(0);
    CORBA::Object_var obj__(0);

    try
    {
        obj__ = orb->resolve_initial_references(LOCAL_NAME_SERVICE.c_str());
        ns__  = CosNaming::NamingContext::_narrow(obj__.in());

        if (CORBA::is_nil(ns__.in()))
        {
            CDMW_THROW1(
                Cdmw::BadParameterException, 
                LOCAL_NAME_SERVICE
                + " does not denote a valid NamingContext object");
        }
    }
    catch (const CORBA::SystemException& se)
    {
        CDMW_THROW1(Cdmw::Common::NotFoundException,
                    "Could not retrieve the local NameService");
    }

    CDMW_ASSERT(ns__.in() != 0);
    return ns__._retn();
}


void
InitUtils::activate_LogCollector(
    CORBA::ORB_ptr orb,
    short port,
    const std::string collectorName
    )
    throw (Cdmw::Common::NotFoundException,
           Cdmw::BadParameterException,
           Cdmw::OutOfMemoryException)
{
    CDMW_ASSERT(!CORBA::is_nil(orb));

    // get the local NS
    CosNaming::NamingContext_var ns__;
    if (port == -1)
    {
        ns__ = M_instance->get_LocalNameService(orb);
    }
    else if (IS_SYSTEM_PORT_VALID)
    {
        ns__ = M_instance->retrieve_NameService(orb, port);
    }
    else
    {
        CDMW_THROW1(
            Cdmw::BadParameterException, 
            "the specified port does not denote a valid number");
    }

    // now register the LogCollector
    CDMW_ASSERT(ns__.in() != 0);
    M_instance->activate_LogCollector(ns__.in(), collectorName);
}

void
InitUtils::activate_LogCollector(
    const CosNaming::NamingContext_ptr nameService,
    const std::string collectorName
    )
    throw (Cdmw::Common::NotFoundException,
           Cdmw::BadParameterException,
           Cdmw::OutOfMemoryException)
{
    CDMW_ASSERT(!CORBA::is_nil(nameService));

    CdmwLogging::LogCollector_var collector__(0);
    collector__ = M_instance->retrieve_LogCollector(nameService, collectorName);
    CDMW_ASSERT(!CORBA::is_nil(collector__.in()));

    try
    {
        using namespace Cdmw::Logging;
        LogManagerCollectorDelegate * delegate__ =
            new LogManagerCollectorDelegate(collector__.in());
        LogManagerDelegate* old__ =
            LogManager::Instance()->register_alternate_delegate(delegate__);
        if (old__) { delete old__; old__ = 0x0; }
    }
    catch (const Cdmw::OutOfMemoryException& oome)
    {
        throw oome;
    }
}


void
InitUtils::deactivate_LogCollector(
    )
{
    using namespace Cdmw::Logging;
    LogManagerDelegate * delegate__ = 
        LogManager::Instance()->get_alternate_delegate();
    if (delegate__)
    {
        LogManagerCollectorDelegate * d__ = 
            dynamic_cast< LogManagerCollectorDelegate* >(delegate__);
        CDMW_ASSERT(delegate__ != 0);
        (void) LogManager::Instance()->unregister_alternate_delegate(d__);
        delete d__; d__ = 0x0;
    }
}


CosNaming::NamingContext_ptr
InitUtils::retrieve_NameService(
    CORBA::ORB_ptr orb,
    short port
    )
    throw (Cdmw::Common::NotFoundException,
           Cdmw::BadParameterException)
{
    CDMW_ASSERT(!CORBA::is_nil(orb));

    CosNaming::NamingContext_var ns__(0);
    CORBA::Object_var obj__(0);

    try
    {
        if (! IS_SYSTEM_PORT_VALID)
        {
            CDMW_THROW1(
                Cdmw::BadParameterException, 
                "the specified port does not denote a valid number");
        }

        std::ostringstream ostr;
        ostr << "corbaloc::localhost:" << port << "/NameService";
        obj__ = orb->string_to_object(ostr.str().c_str());

        ns__  = CosNaming::NamingContext::_narrow(obj__.in());
        if (CORBA::is_nil(ns__.in()))
        {
            CDMW_THROW1(
                Cdmw::BadParameterException, 
                "The reference does not denote a valid NameService ");
        }
    }
    catch (const CORBA::SystemException& se)
    {
        CDMW_THROW1(Cdmw::Common::NotFoundException,
                    "Could not retrieve the local NameService in port " + port);
    }

    CDMW_ASSERT(ns__.in() != 0);
    return ns__._retn();
}

CdmwLogging::LogCollector_ptr
InitUtils::retrieve_LogCollector(
    const CosNaming::NamingContext_ptr nameService,
    const std::string & collectorName
    )
    throw (Cdmw::Common::NotFoundException,
           Cdmw::BadParameterException)
{
    CDMW_ASSERT(!CORBA::is_nil(nameService));

    CdmwLogging::LogCollector_var collector__(0);
    CORBA::Object_var obj__(0);

    try
    {
        Cdmw::CommonSvcs::Naming::NamingInterface ni__(nameService);
        obj__ = ni__.resolve(collectorName.c_str());
        collector__ = CdmwLogging::LogCollector::_narrow(obj__.in());
        if (CORBA::is_nil(collector__.in()))
        {
            CDMW_THROW1(
                Cdmw::BadParameterException, 
                collectorName + " doesn't denote a valid LogCollector object");
        }
    }
    catch (const CosNaming::NamingContext::InvalidName& in)
    {
        CDMW_THROW1(
            Cdmw::Common::NotFoundException, 
            "Could not retrieve reference of the LogCollector, '"
            + collectorName + "' is not a valid name.");
    }
    catch (const CosNaming::NamingContext::NotFound& nf)
    {
        CDMW_THROW1(
            Cdmw::Common::NotFoundException, 
            "Could not retrieve reference of the LogCollector '"
            + collectorName + "' in the Naming service.");
    }
    catch (const CosNaming::NamingContext::CannotProceed& cp)
    {
        CDMW_THROW1(
            Cdmw::Common::NotFoundException, 
            "Could not retrieve reference of the LogCollector '"
            + collectorName + "', the Naming service raised"
            " CosNaming::NamingContext::ConnotProceed");
    }

    CDMW_ASSERT(collector__.in() != 0);
    return collector__._retn();
}
