/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


package cdmw.commonsvcs.naming.test;

import cdmw.commonsvcs.naming.NamingInterface;
import cdmw.testutils.CORBATestManager;
import cdmw.testutils.CORBATestManager.InvalidObject;

import org.omg.CORBA.ObjectHelper;
import org.omg.CosNaming.Binding;
import org.omg.CosNaming.BindingHolder;
import org.omg.CosNaming.BindingIterator;
import org.omg.CosNaming.BindingIteratorHolder;
import org.omg.CosNaming.BindingListHolder;
import org.omg.CosNaming.NamingContext;

import org.junit.BeforeClass;
import org.junit.AfterClass;
import org.junit.Before;
import org.junit.After;
import org.junit.Test;

import static org.junit.Assert.assertTrue;
import static org.junit.Assert.fail;

import org.omg.CosNaming.NamingContextPackage.NotFound;
import org.omg.CosNaming.NamingContextPackage.CannotProceed;
import org.omg.CosNaming.NamingContextPackage.InvalidName;
import org.omg.CosNaming.NamingContextPackage.AlreadyBound;
import org.omg.CosNaming.NamingContextPackage.NotEmpty;
import org.omg.CosNaming.NamingContextPackage.NotFoundReason;


public class BindTest {

    private static NamingContext namingContext;
    private static TestObject testObject;

    private NamingInterface ni0;

    @BeforeClass
    public static void initTests() throws InvalidObject{
        CORBATestManager testManager = CORBATestManager.instance();

        org.omg.CORBA.Object obj = testManager.getObject("NameService");
        namingContext = org.omg.CosNaming.NamingContextHelper.narrow(obj);
        
        obj = testManager.getObject("TestObject");
        testObject = TestObjectHelper.narrow(obj);
    }

    @AfterClass
    public static void finiTests() {
        namingContext = null;
        testObject = null;
    }

    @Before
    public void init() {
        this.ni0 = new NamingInterface(namingContext);
    }

    @After
    public void fini() throws NotFound, CannotProceed, InvalidName {
        // Clear context from bindings
        try {
            BindingListHolder listHolder = new BindingListHolder();
            BindingIteratorHolder iteratorHolder = new BindingIteratorHolder();
            namingContext.list(0, listHolder, iteratorHolder);
            BindingIterator iterator = iteratorHolder.value;
            if (iterator != null) {
                BindingHolder bindingHolder = new BindingHolder();
    
                while (iterator.next_one(bindingHolder)) {
                    Binding binding = bindingHolder.value;
                    namingContext.unbind(binding.binding_name);
                }
    
                iterator.destroy();
            }
        }
        finally {
            this.ni0 = null;
        }
    }


    @Test
    public void testBind() throws CannotProceed, InvalidName, AlreadyBound, NotFound {
        //-------------------------------------------------
        // Testing bind operation
        //-------------------------------------------------
        System.out.println("Testing NamingInterface#bind operation");

        // This test creates the following bindings in the naming service:
        //     cdmw.test/archive.3/orbacus.mk

        //
        // Try to bind an object when an intermediate context doesn't exist
        //
        String input = "cdmw.test/archive.3/orbacus.mk";
        System.out.println("Attempt to bind " + input);
        try {
            this.ni0.bind(input, testObject, false);
            fail();
        } catch (NotFound nf) {
            assertTrue( nf.why.equals(NotFoundReason.missing_node) );
        }

        // Forcing binding
        System.out.println("Forcing bind of " + input);

        this.ni0.bind(input, testObject, true);
        org.omg.CORBA.Object obj = this.ni0.resolve(input);
        assertTrue(obj!=null);

        //
        // Try to bind an object when an object is in the path
        //
        input = "cdmw.test/archive.3/orbacus.mk/bla.bla";
        System.out.println("Attempt to bind " + input);
        try {
            this.ni0.bind(input, testObject, false);
            fail();
        } catch (NotFound nf) {
            // The correct situation - do nothing intentionally
        }

        // Forcing Binding
        System.out.println("Forcing bind of " + input);
        try {
            this.ni0.bind(input, testObject, true);
            fail();
        } catch (NotFound nf) {
            // The correct situation - do nothing intentionally
        }

        //
        // Try to bind an object using the name of an existing context
        //
        input = "cdmw.test/archive.3";
        System.out.println("Attempt to bind " + input);
        try {
            this.ni0.bind(input, testObject, false);
            fail();
        } catch (AlreadyBound ab ) {
            // The correct situation - do nothing intentionally
        }

        // Forcing bind
        System.out.println("Forcing bind of " + input);
        try {
            this.ni0.bind(input, testObject, true);
            fail();
        } catch (NotFound nf) {
            assertTrue(nf.why.equals(NotFoundReason.not_object));
        }

        //
        // Try to bind an object using the name of an existing object
        //
        input = "cdmw.test/archive.3/orbacus.mk";
        System.out.println("Attempt to bind " + input);
        try {
            this.ni0.bind(input, testObject, false);
            fail();
        } catch (AlreadyBound ab) {
            // The correct situation - do nothing intentionally
        }

        // Forcing bind
        System.out.println("Forcing bind of " + input);

        this.ni0.bind(input, testObject, true);
    }

    @Test
    public void testRebind() throws CannotProceed, InvalidName, AlreadyBound, NotFound {
        //-------------------------------------------------
        // Testing rebind operation
        //-------------------------------------------------
        System.out.println("Testing NamingInterface#rebind operation");

        // This test creates the following bindings in the naming service:
        //     cdmw.test/archive.4/orbacus.mk

        //
        // Try to rebind an object when an intermediate context doesn't exist
        //
        String input = "cdmw.test/archive.4/orbacus.mk";
        System.out.println("Attempt to rebind " + input);
        try {
            this.ni0.rebind(input, testObject, false);
            fail();
        } catch(NotFound nf) {
            assertTrue(nf.why.equals(NotFoundReason.missing_node));
        }

        // Forcing binding
        System.out.println("Forcing rebind of " + input);
        
        this.ni0.rebind(input, testObject, true);
        org.omg.CORBA.Object obj = this.ni0.resolve(input);
        assertTrue(obj != null);

        //
        // Try to rebind an object when an object is in the path
        //
        input = "cdmw.test/archive.4/orbacus.mk/bla.bla";
        System.out.println("Attempt to rebind " + input);
        try {
            this.ni0.rebind(input, testObject, false);
            fail();
        } catch (NotFound nf) {
            assertTrue(nf.why.equals(NotFoundReason.not_context));
        }

        // Forcing Rebind
        System.out.println("Forcing rebind of " + input);
        try {
            this.ni0.rebind(input, testObject, true);
            fail();
        } catch (NotFound nf) {
            assertTrue(nf.why.equals(NotFoundReason.not_context));
        }

        //
        // Try to rebind an object using the name of an existing context
        //
        input = "cdmw.test/archive.4";
        System.out.println("Attempt to rebind " + input);
        try {
            this.ni0.rebind(input, testObject, false);
            fail();
        } catch (NotFound nf) {
            assertTrue(nf.why.equals(NotFoundReason.not_object));
        }

        // Forcing rebind
        System.out.println("Forcing rebind of " + input);
        try {
            this.ni0.rebind(input, testObject, true);
            fail();
        } catch (NotFound nf) {
            assertTrue(nf.why.equals(NotFoundReason.not_object));
        }

        //
        // Try to rebind an object using the name of an existing object
        //
        input = "cdmw.test/archive.4/orbacus.mk";
        System.out.println("Attempt to rebind " + input);
        
        this.ni0.rebind(input, testObject, false);

        // Forcing rebind
        System.out.println("Forcing rebind of " + input);
        
        this.ni0.rebind(input, testObject, true);
    }
}

