/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


package cdmw.commonsvcs.naming.test;

import cdmw.common.TypeMismatchException;
import cdmw.commonsvcs.naming.InvalidNameException;
import cdmw.commonsvcs.naming.NamingInterface;
import cdmw.testutils.CORBATestManager;
import cdmw.testutils.CORBATestManager.InvalidObject;

import org.omg.CORBA.ObjectHelper;
import org.omg.CORBA.SystemException;

import org.omg.CosNaming.Binding;
import org.omg.CosNaming.BindingHolder;
import org.omg.CosNaming.BindingIterator;
import org.omg.CosNaming.BindingIteratorHolder;
import org.omg.CosNaming.NamingContext;
import org.omg.CosNaming.NamingContextHelper;
import org.omg.CosNaming.NameComponent;
import org.omg.CosNaming.BindingListHolder;

import org.omg.CosNaming.NamingContextPackage.NotFound;
import org.omg.CosNaming.NamingContextPackage.CannotProceed;
import org.omg.CosNaming.NamingContextPackage.InvalidName;
import org.omg.CosNaming.NamingContextPackage.AlreadyBound;
import org.omg.CosNaming.NamingContextPackage.NotEmpty;
import org.omg.CosNaming.NamingContextPackage.NotFoundReason;

import org.junit.BeforeClass;
import org.junit.AfterClass;
import org.junit.Before;
import org.junit.After;
import org.junit.Test;

import static org.junit.Assert.assertTrue;
import static org.junit.Assert.fail;

public class DestroyTest {

    private static NamingContext namingContext;
    private static TestObject testObject;

    private NamingInterface ni0;

    @BeforeClass
    public static void initTests() throws InvalidObject {
        CORBATestManager testManager = CORBATestManager.instance();

        org.omg.CORBA.Object obj = testManager.getObject("NameService");
        namingContext = org.omg.CosNaming.NamingContextHelper.narrow(obj);
        
        obj = testManager.getObject("TestObject");
        testObject = TestObjectHelper.narrow(obj);
    }

    @AfterClass
    public static void finiTests() {
        namingContext = null;
        testObject = null;
    }

    @Before
    public void init() {
        this.ni0 = new NamingInterface(namingContext);
    }

    @After
    public void fini() throws NotFound, CannotProceed, InvalidName {
        // Clear context from bindings
        try {
            BindingListHolder listHolder = new BindingListHolder();
            BindingIteratorHolder iteratorHolder = new BindingIteratorHolder();
            namingContext.list(0, listHolder, iteratorHolder);
            BindingIterator iterator = iteratorHolder.value;
            if (iterator != null) {
                BindingHolder bindingHolder = new BindingHolder();
    
                while (iterator.next_one(bindingHolder)) {
                    Binding binding = bindingHolder.value;
                    namingContext.unbind(binding.binding_name);
                }
    
                iterator.destroy();
            }
        }
        finally {
            this.ni0 = null;
        }
    }

    @Test
    public void testDestroy()
        throws CannotProceed, InvalidName, TypeMismatchException, NotFound, NotEmpty,
            InvalidNameException, AlreadyBound {
        // This tests NamingInterface class
        //-------------------------------------------------
        // Testing destroy operation
        //-------------------------------------------------
        System.out.println("Testing NamingInterface#destroy operation");
        String base = "test.destroy/runtime";

        // Install test name tree
        if ( !this.installInitialBindings(this.ni0, base, testObject) ) {
            fail("Unable to install initial bindings.");
        }

        //
        // Additional tests to test resolveName()
        //
        System.out.println("Additional tests to test resolveName() function");
        String input = base + "/commonsvcs/naming/dummy/ada.1";
        // Testing NotFound Exception
        System.out.println("Testing NotFound Exception");
        try {
            NamingContext nc = resolveName(this.ni0, input);
            fail();
        } catch (NotFound nf) {
            // The correct situation - do nothing intentionally
        }

        // Testing TypeMismatch Exception
        input = base + "/bin/CVS/Root";
        System.out.println("Testing TypeMismatch Exception");
        try {
            NamingContext nc = resolveName(this.ni0, input);
            fail();
        } catch (TypeMismatchException tme) {
            // The correct situation - do nothing intentionally
        }

        //
        // Destroy commonsvcs/naming/ada.1 naming context
        //
        input = base + "/commonsvcs/naming/ada.1";
        System.out.println("Destroying an empty context");

        NamingContext nc = resolveName(this.ni0, input);
        NamingInterface n = new NamingInterface(nc);
        n.destroy();
        this.ni0.unbind(input);

        // Forcing destroy
        input = base + "/commonsvcs/naming/ada.2";
        System.out.println("Forcing destroy");

        nc = resolveName(ni0,input);
        n = new NamingInterface(nc);
        assertTrue(n.destroyAll());
        this.ni0.unbind(input);

        //
        // Try to destroy a context that's not empty
        //
        System.out.println("Test of destroying a context that's not empty");
        input = base;
        nc = null;
        try {
            nc = resolveName(this.ni0, input);
            n = new NamingInterface(nc);
            n.destroy();
            fail();
        } catch (NotEmpty ne) {
            // The correct situation - do nothing intentionally
        }
        
        // Forcing destroy
        System.out.println("Forcing destroy");
        
        n = new NamingInterface(nc);
        assertTrue(n.destroyAll());
            
        // Is it destroyed ?
        System.out.println("Is it destroyed ?");
        try {
            NameComponent[] name = NamingInterface.toName("toto");
            NamingContext nc_ = nc.bind_new_context(name);
            fail();
        } catch (org.omg.CORBA.OBJECT_NOT_EXIST one) {
            // the test passed - remove binding
            this.ni0.unbind(input);
        }

        //
        // Try to destroy a context that's not empty
        // with an undestroyable context
        //
        // Install test name tree
        System.out.println("Install test name tree");
        if (!this.installInitialBindings(this.ni0, base, testObject)) {
            fail();
        }

        System.out.println("Test of destroying a context that's not empty "+
            "with an undestroyable context");
        // To simulate an undestroyable context, we destroy a context without
        // removing the binding. This will generate a system exception when its
        // destruction is requested.
        input = base + "/commonsvcs/naming/ada.1";
        
        nc = resolveName(this.ni0, input);
        n = new NamingInterface(nc);
        n.destroy();
            
        // Forcing destroy
        System.out.println("Forcing destroy");
        
        nc = resolveName(this.ni0, base);

        n = new NamingInterface(nc);
        assertTrue(!n.destroyAll());

        // Is it destroyed
        System.out.println("Is it destroyed ?");
       
        n = new NamingInterface(nc);
        BindingListHolder bndList = new BindingListHolder();
        n.listContext(bndList);
        assertTrue(bndList.value.length == 1);
    }

    private boolean installInitialBindings(NamingInterface n, String base, org.omg.CORBA.Object obj) {

        boolean result = true;
        try {
            NamingInterface ni = n.bindNewContext(base,true);

            ni.bind("bin/CVS/Entries",obj,true);
            ni.bind("bin/CVS/Repository",obj,true);
            ni.bind("bin/CVS/Root",obj,true);
            ni.bind("bin/Makefile",obj,true);
            ni.bind("bin/Makefile.mak",obj,true);
            ni.bind("bin/Makefile.tmpl",obj,true);
            ni.bind("bin/mk_idl_include.sh",obj,true);
            ni.bind("bin/CdmwCreateClass",obj,true);
            ni.bind("bin/CdmwCreateCSC",obj,true);
            ni.bind("commonsvcs/naming/c++/CVS/Entries",obj,true);
            ni.bind("commonsvcs/naming/c++/CVS/Entries.Log",obj,true);
            ni.bind("commonsvcs/naming/c++/CVS/Repository",obj,true);
            ni.bind("commonsvcs/naming/c++/CVS/Root",obj,true);
            ni.bind("commonsvcs/naming/c++/incl/CVS/Entries",obj,true);
            ni.bind("commonsvcs/naming/c++/incl/CVS/Repository",obj,true);
            ni.bind("commonsvcs/naming/c++/incl/CVS/Root",obj,true);
            ni.bind("commonsvcs/naming/c++/incl/NameParser.hpp",obj,true);
            ni.bind("commonsvcs/naming/c++/incl/NameParserContext.hpp",obj,true);
            ni.bind("commonsvcs/naming/c++/incl/NameParserFSM.hpp",obj,true);
            ni.bind("commonsvcs/naming/c++/src/CSC.mk",obj,true);
            ni.bind("commonsvcs/naming/c++/src/CVS/Entries",obj,true);
            ni.bind("commonsvcs/naming/c++/src/CVS/Repository",obj,true);
            ni.bind("commonsvcs/naming/c++/src/CVS/Root",obj,true);
            ni.bind("commonsvcs/naming/c++/src/InvalidName.cpp",obj,true);
            ni.bind("commonsvcs/naming/c++/src/main.cpp",obj,true);
            ni.bind("commonsvcs/naming/c++/src/NameParser.cpp",obj,true);
            ni.bind("commonsvcs/naming/c++/src/NameParserContext.cpp",obj,true);
            ni.bind("commonsvcs/naming/c++/src/NameParserFSM.cpp",obj,true);
            ni.bind("commonsvcs/naming/c++/src/NamingInterface.cpp",obj,true);
            NamingInterface nitf = ni.bindNewContext("commonsvcs/naming/ada.1",true);
            nitf = ni.bindNewContext("commonsvcs/naming/ada.2",true);
        } catch (Exception e) {
            System.out.println("Unexpected error (could'nt install initial bindings)");
            result = false;
        }
        return result;
    }

    private NamingContext resolveName(NamingInterface ni,String s)
        throws  NotFound,
                CannotProceed,
                InvalidName,
                TypeMismatchException {

        org.omg.CORBA.Object obj = ni.resolve(s);
        NamingContext nc = null;

        try {
            nc = NamingContextHelper.narrow(obj);
        } catch(SystemException se) {
            throw new TypeMismatchException();
        }

        return nc;
    }

}

