/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


package cdmw.commonsvcs.naming.test;

import cdmw.commonsvcs.naming.InvalidNameException;
import cdmw.commonsvcs.naming.NameParser;

import org.omg.CosNaming.NameComponent;

import org.junit.Test;
import static org.junit.Assert.assertTrue;
import static org.junit.Assert.fail;


public class ToNameTest {

    @Test public void parseIdKindIdKindString() {
        System.out.println("Test name components with both ID and Kind field");

        String input = "first.1st/second.2nd";
        try {
            NameComponent[] name = NameParser.toName(input);
            assertTrue( name.length == 2 );
            assertTrue( name[0].id.equals("first") );
            assertTrue( name[0].kind.equals("1st") );
            assertTrue( name[1].id.equals("second") );
            assertTrue( name[1].kind.equals("2nd") );
        } catch(InvalidNameException ine) {
            fail(ine.getMessage());
        }
    }

    @Test public void parseIdKindIdEmptyKindString() {
        System.out.println("Test name components with missing Kind field");
        
        String input = "first.1st/second";
        try {
            NameComponent[] name = NameParser.toName(input);
            assertTrue( name.length == 2 );
            assertTrue( name[0].id.equals("first") );
            assertTrue( name[0].kind.equals("1st") );
            assertTrue( name[1].id.equals("second") );
            assertTrue( name[1].kind.length() == 0 );
        } catch(InvalidNameException ine) {
            fail(ine.getMessage());
        }
    }

    @Test public void parseIdKindEmptyIdKindString() {
        System.out.println("Test name components with missing ID field");
        
        String input = "first.1st/.2nd";
        try {
            NameComponent[] name = NameParser.toName(input);
            assertTrue( name.length == 2 );
            assertTrue( name[0].id.equals("first") );
            assertTrue( name[0].kind.equals("1st") );
            assertTrue( name[1].id.length() == 0 );
            assertTrue( name[1].kind.equals("2nd") );
        } catch(InvalidNameException ine) {
            fail(ine.getMessage());
        }
    }

    @Test public void parseIdKindEmptyIdEmptyKindIdKindString() {
        System.out.println("Test node with empty ID and Kind field");
        
        String input = "first.1st/./third.3rd";
        try {
            NameComponent[] name = NameParser.toName(input);
            assertTrue( name.length == 3 );
            assertTrue( name[0].id.equals("first") );
            assertTrue( name[0].kind.equals("1st") );
            assertTrue( name[1].id.length() == 0 );
            assertTrue( name[1].id.length() == 0 );
            assertTrue( name[2].id.equals("third") );
            assertTrue( name[2].kind.equals("3rd") );
        } catch(InvalidNameException ine) {
            fail(ine.getMessage());
        }
    }
    
    @Test public void parseStringWithEscapedChars() {
        System.out.println("Test nodes with characters that are escaped");

        String input = "first.1st\\.dot/second\\.dot.2nd/third\\/slash.3rd\n";
        try {
            NameComponent[] name = NameParser.toName(input);
            assertTrue( name.length == 3 );
            assertTrue( name[0].id.equals("first") );
            assertTrue( name[0].kind.equals("1st.dot") );
            assertTrue( name[1].id.equals("second.dot") );
            assertTrue( name[1].kind.equals("2nd") );
            assertTrue( name[2].id.equals("third/slash") );
            assertTrue( name[2].kind.equals("3rd\n") );
        } catch(InvalidNameException ine) {
            fail(ine.getMessage());
        }
    }

    @Test public void parseEmptyIdEmptyKindString() {
        System.out.println("Test empty name");

        String input = ".";
        try {
            NameComponent[] name = NameParser.toName(input);
            assertTrue( name.length == 1 );
            assertTrue( name[0].id.length() == 0 );
            assertTrue( name[0].kind.length() == 0 );
        } catch(InvalidNameException ine) {
            fail(ine.getMessage());
        }
    }

    @Test(expected=InvalidNameException.class) public void parseEmptyString()
        throws InvalidNameException {
        System.out.println("Test empty string");

        String input = "";
        NameParser.toName(input);
        fail("Error: An empty string is not a legal name.");
    }

    @Test(expected=InvalidNameException.class) public void parseIdKindIdPointString()
        throws InvalidNameException {
        System.out.println("Test ending '.'");

        String input = "first.1st/second.";
        NameParser.toName(input);
        // If a name component has a non-empty id field and an empty kind field,
        // the stringified representation consists only of the id field.
        // A trailing '.' character is not permitted.
        fail("Error: A trailing '.' character is not permitted.");
    }

    @Test(expected=InvalidNameException.class) public void parseSlashIdKindIdKindString()
        throws InvalidNameException {
        System.out.println("Test starting '/'");

        String input = "/first.1st/second.2nd";
        NameParser.toName(input);
        // The single '.' character is the only representation of a name
        // component with empty id and kind fields.
        fail("Error: A name component with a starting '/' is not permitted.");
    }

}

