/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO 
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2009. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

// standard files
#include <sstream>

// Cdmw Files
#include <Foundation/orbsupport/CORBA.hpp>
#include <Foundation/ossupport/OS.hpp>
#include <Foundation/logging/FunctionLogger.hpp>

// Local Files
#include <requestfollowup/NDCClientRequestInterceptor_impl.hpp>
#include <requestfollowup/NDCLogger.hpp>
#include <requestfollowup/NDCFactory.hpp>


namespace 
{
    const PortableInterceptor::ORBInitInfo::ObjectId NDC_CLIENT_REQUEST_INTERCEPTOR_NAME 
        = "NDCClientRequestInterceptor";
} // end of anonymous namespace


namespace Cdmw
{
namespace CommonSvcs
{
namespace RequestFollowUp
{

NDCClientRequestInterceptor_impl::NDCClientRequestInterceptor_impl(
                 PortableInterceptor::ORBInitInfo_ptr info,
                 const PortableInterceptor::SlotId&  ndc_slot_id)
   : m_ndc_slot_id(ndc_slot_id)
{
    CDMW_LOG_FUNCTION(NDC_LOGGER, "NDC_SlotId=" << m_ndc_slot_id);
}

char*
NDCClientRequestInterceptor_impl::name() 
    throw (CORBA::SystemException)
{
    CDMW_LOG_FUNCTION(NDC_LOGGER,"");
    CDMW_LOG_FUNCTION_RETURN(NDC_CLIENT_REQUEST_INTERCEPTOR_NAME);
    return CORBA::string_dup(NDC_CLIENT_REQUEST_INTERCEPTOR_NAME);
}

void
NDCClientRequestInterceptor_impl::destroy() 
    throw (CORBA::SystemException)
{
    CDMW_LOG_FUNCTION(NDC_LOGGER, "PortableInterceptor=" << name());
}

void
NDCClientRequestInterceptor_impl::send_request(
        PortableInterceptor::ClientRequestInfo_ptr ri)
    throw (CORBA::SystemException, PortableInterceptor::ForwardRequest)
{
    // Do not use request follow up mechanims for "Logger" and "Collector" operations
    if ( ! NDCLogger::Get_instance()->is_operation_to_log(ri->operation()) ) {
        return;
    }
    // The log below must not be recorded if the intercepted operation is add_log(). Else if Log Collector
    // is used we enter in a cyclic recording of the log (infinite call to add_log)!
    // This explains why the macro is called only after the condition if()
    CDMW_LOG_FUNCTION(NDC_LOGGER, "Operation=" << ri->operation());
    
    CDMW_LOG(NDC_LOGGER, Cdmw::Logging::CDMW_DEBUG_3,
             NDCLogger::Get_instance()->ip_to_string(NDC_CLIENT_SEND_REQUEST) << " (" << ri->operation() << ")" );
    log_outgoing_request(NDC_CLIENT_SEND_REQUEST, ri);

}

void
NDCClientRequestInterceptor_impl::send_poll(PortableInterceptor::ClientRequestInfo_ptr ri)
        throw (CORBA::SystemException)
{
    // Do not use request follow up mechanims for "Logger" and "Collector" operations
    if ( ! NDCLogger::Get_instance()->is_operation_to_log(ri->operation()) ) {
        return;
    }
    // The log below must not be recorded if the intercepted operation is add_log(). Else if Log Collector
    // is used we enter in a cyclic recording of the log (infinite call to add_log)!
    // This explains why the macro is called only after the condition if()
    CDMW_LOG_FUNCTION(NDC_LOGGER, "Operation=" << ri->operation());
   
    CDMW_LOG(NDC_LOGGER, Cdmw::Logging::CDMW_DEBUG_3,
             NDCLogger::Get_instance()->ip_to_string(NDC_CLIENT_SEND_POLL) << " (" << ri->operation() << ")" );
    log_outgoing_request(NDC_CLIENT_SEND_POLL, ri);

}


void
NDCClientRequestInterceptor_impl::receive_reply(PortableInterceptor::ClientRequestInfo_ptr ri)
    throw (CORBA::SystemException)
{
    // Do not use request follow up mechanims for "Logger" and "Collector" operations
    if ( ! NDCLogger::Get_instance()->is_operation_to_log(ri->operation()) ) {
        return;
    }
    // The log below must not be recorded if the intercepted operation is add_log(). Else if Log Collector
    // is used we enter in a cyclic recording of the log (infinite call to add_log)!
    // This explains why the macro is called only after the condition if()
    CDMW_LOG_FUNCTION(NDC_LOGGER, "Operation=" << ri->operation());

    // Do not use request follow up mechanims for "Logger" and "Collector" operations
    if ( ! NDCLogger::Get_instance()->is_operation_to_log(ri->operation()) ) {
        return;
    }
    
    CDMW_LOG(NDC_LOGGER, Cdmw::Logging::CDMW_DEBUG_3,
             NDCLogger::Get_instance()->ip_to_string(NDC_CLIENT_RECEIVE_REPLY) << " (" << ri->operation() << ")" );
    log_incoming_response(NDC_CLIENT_RECEIVE_REPLY, ri);

}


void
NDCClientRequestInterceptor_impl::receive_exception(PortableInterceptor::ClientRequestInfo_ptr ri)
        throw (CORBA::SystemException, PortableInterceptor::ForwardRequest)
{
    // Do not use request follow up mechanims for "Logger" and "Collector" operations
    if ( ! NDCLogger::Get_instance()->is_operation_to_log(ri->operation()) ) {
        return;
    }

    // The log below must not be recorded if the intercepted operation is add_log(). Else if Log Collector
    // is used we enter in a cyclic recording of the log (infinite call to add_log)!
    // This explains why the macro is called only after the condition if()
    CDMW_LOG_FUNCTION(NDC_LOGGER, "Operation=" << ri->operation());
    
    CDMW_LOG(NDC_LOGGER, Cdmw::Logging::CDMW_DEBUG_3,
             NDCLogger::Get_instance()->ip_to_string(NDC_CLIENT_RECEIVE_EXCEPTION) << " (" << ri->operation() << ")" );
    log_incoming_response(NDC_CLIENT_RECEIVE_EXCEPTION, ri);
}

void
NDCClientRequestInterceptor_impl::receive_other(PortableInterceptor::ClientRequestInfo_ptr ri)
        throw (CORBA::SystemException, PortableInterceptor::ForwardRequest)
{
    // Do not use request follow up mechanims for "Logger" and "Collector" operations
    if ( ! NDCLogger::Get_instance()->is_operation_to_log(ri->operation()) ) {
        return;
    }

    // The log below must not be recorded if the intercepted operation is add_log(). Else if Log Collector
    // is used we enter in a cyclic recording of the log (infinite call to add_log)!
    // This explains why the macro is called only after the condition if()
    CDMW_LOG_FUNCTION(NDC_LOGGER, "Operation=" << ri->operation());
   
    CDMW_LOG(NDC_LOGGER, Cdmw::Logging::CDMW_DEBUG_3,
             NDCLogger::Get_instance()->ip_to_string(NDC_CLIENT_RECEIVE_OTHER) << " (" << ri->operation() << ")" );
    log_incoming_response(NDC_CLIENT_RECEIVE_OTHER, ri);
}


void NDCClientRequestInterceptor_impl::log_outgoing_request(
        NDCInterceptionPoint ip,
        PortableInterceptor::ClientRequestInfo_ptr ri)
        throw ()
{
    CDMW_LOG_FUNCTION(NDC_LOGGER, "InterceptionPoint=" << NDCLogger::Get_instance()->ip_to_string(ip) << ", Operation=" << ri->operation());
    
    NDCList ndc_list;

    try {
        CDMW_LOG(NDC_LOGGER, Cdmw::Logging::CDMW_DEBUG_3, "Get NDCs list from Slot");

        CORBA::Any_var slot_data = ri->get_slot(m_ndc_slot_id);
        
        ndc_list = NDCFactory::Get_instance()->from_any(slot_data.in());
    }
    catch (const PortableInterceptor::InvalidSlot& ex) {
        // NDC slot has not been allocated
        CDMW_WARN(NDC_LOGGER, "Catch PortableInterceptor::InvalidSlot Exception - " << ex);
    }
    catch (const CORBA::Exception& ex) {
        CDMW_WARN(NDC_LOGGER, "Catch CORBA::Exception - " << ex);
    }
    catch (const std::exception& ex)  {
        CDMW_WARN(NDC_LOGGER, "Catch std::exception - " << ex.what());
    }

    // Request Follow Up  not activated for that request
    if ( (ndc_list.size() == 0) && !(NDC_LOGGER->is_enabled_for(Cdmw::Logging::REQ_FOLLOWUP)) ) {
        CDMW_LOG(NDC_LOGGER, Cdmw::Logging::CDMW_DEBUG_3, "Request Follow Up not activated for that request");
        return;
    }
    
    try {
        CDMW_LOG(NDC_LOGGER, Cdmw::Logging::CDMW_DEBUG_3, "Create a new NDC and add it into the NDCs list.");
        NDC ndc = NDCFactory::Get_instance()->create_new_ndc(ri->operation());
        ndc_list.push_back(ndc);
        
        // Add NDCs to NDC Service Context of the request
        CDMW_LOG(NDC_LOGGER, Cdmw::Logging::CDMW_DEBUG_3,
                "Set Service Context from NDCs list : " << NDCFactory::Get_instance()->to_string(ndc_list));
        IOP::ServiceContext sc;
        sc.context_id = CDMW_NDC_SERVICE_CONTEXT_ID;
        CORBA::OctetSeq_var sc_data = NDCFactory::Get_instance()->to_octetseq(ndc_list);
        sc.context_data = sc_data.in();
        ri->add_request_service_context(sc, true);
    }
    catch (const CORBA::Exception& ex) {
        CDMW_WARN(NDC_LOGGER, "Catch CORBA::Exception - " << ex);
    }
    catch (const std::exception& ex)  {
        CDMW_WARN(NDC_LOGGER, "Catch std::exception - " << ex.what());
    }

    // log the Request Follow Up context
    NDCLogger::Get_instance()->log_request(ip, ri->operation(), ndc_list);
}


void NDCClientRequestInterceptor_impl::log_incoming_response(
        NDCInterceptionPoint ip,
        PortableInterceptor::ClientRequestInfo_ptr ri)
        throw ()
{
    CDMW_LOG_FUNCTION(NDC_LOGGER, "InterceptionPoint=" << NDCLogger::Get_instance()->ip_to_string(ip) << ", Operation=" << ri->operation());
    
    NDCList ndc_list;

    try {
        //CDMW_LOG(NDC_LOGGER, Cdmw::Logging::CDMW_DEBUG_3, "Get NDCs list from Slot");
        //CORBA::Any_var slot_data = ri->get_slot(m_ndc_slot_id);
        //ndc_list = NDCFactory::Get_instance()->from_any(slot_data.in());
        CDMW_LOG(NDC_LOGGER, Cdmw::Logging::CDMW_DEBUG_3, "Get NDCs list from Service Context");
        IOP::ServiceContext_var sc = ri->get_request_service_context(CDMW_NDC_SERVICE_CONTEXT_ID);
        ndc_list = NDCFactory::Get_instance()->from_octetseq(sc->context_data);

    }
    catch(const CORBA::BAD_PARAM&) {
        CDMW_LOG(NDC_LOGGER, Cdmw::Logging::CDMW_DEBUG_3,"No NDC Service Context for that request");
        // This is not an error while a response can come to a single reference
        // with no NDC service context.
    }
    catch (const PortableInterceptor::InvalidSlot& ex)  {
        // NDC slot has not been allocated
        CDMW_WARN(NDC_LOGGER, "Catch PortableInterceptor::InvalidSlot Exception - " << ex);
    }
    catch (const CORBA::Exception& ex) {
        CDMW_WARN(NDC_LOGGER, "Catch CORBA::Exception - " << ex);
    }
    catch (const std::exception& ex)  {
        CDMW_WARN(NDC_LOGGER, "Catch std::exception - " << ex.what());
    }

    // Request Follow Up  not activated for that request
    if ( !(NDC_LOGGER->is_enabled_for(Cdmw::Logging::REQ_FOLLOWUP)) && (ndc_list.size() == 0) ) {
        CDMW_LOG(NDC_LOGGER, Cdmw::Logging::CDMW_DEBUG_3, "Request Follow Up not activated for that request");
        return;
    }
    
    if (ndc_list.size() == 0)
    {
        CDMW_LOG(NDC_LOGGER, Cdmw::Logging::CDMW_DEBUG_3, "Unexpected data returned by NDC Service Context, NDCs list is empty." );
    }

    // log the Request Follow Up context
    NDCLogger::Get_instance()->log_request(ip, ri->operation(), ndc_list);
}

} // End of namespace RequestFollowUp
} // End of namespace CommonSvcs
} // End of namespace Cdmw
