/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

#include "ace/UUID.h"

#include <Foundation/osthreads/MutexGuard.hpp>
#include "Foundation/common/Exception.hpp"

#include "requestfollowup/NDCLogger.hpp"
#include "requestfollowup/NDCFactory.hpp"


    
namespace Cdmw
{
namespace CommonSvcs
{
namespace RequestFollowUp
{

NDCFactory::NDCFactory* NDCFactory::M_instance;
Cdmw::OsSupport::Mutex  NDCFactory::M_mutex;


NDCFactory::NDCFactory()
{
    // Initialize an UUID base for creation of new NDC
    ACE_Utils::UUID_Generator uuid_gen;
    uuid_gen.init();
    ACE_Utils::UUID* uuid = uuid_gen.generateUUID();
    const ACE_CString* uuid_str = uuid->to_string();
    m_uuid_base = uuid_str->c_str();
    //JSM : memory leak 64bytes
    delete uuid;
    m_request_counter = 0;
}



void
NDCFactory::Init_NDC_factory()
{
    CDMW_LOG_FUNCTION(NDC_LOGGER, "");

    // Initialize NDCFactory singleton
    CDMW_ASSERT(!M_instance);
    M_instance = new NDCFactory();
}

//fixing memory leak
void NDCFactory::Cleanup_NDC_factory()
{
    CDMW_ASSERT(M_instance);
    delete M_instance;
    M_instance = 0;
}

NDCFactory* const
NDCFactory::Get_instance() 
{
    CDMW_ASSERT(M_instance);
    return M_instance;
}


NDC
NDCFactory::create_new_ndc(const char* operation)
{
    CDMW_LOG_FUNCTION(NDC_LOGGER, "Operation=" << operation << ", UUID=" << m_uuid_base.c_str() << ", RequestId=" << m_request_counter+1);

    CDMW_MUTEX_GUARD(M_mutex);
    m_request_counter++;
    NDC ndc(operation, m_uuid_base.c_str(), m_request_counter);
    
    CDMW_LOG_FUNCTION_RETURN("New NDC = " << ndc.to_string());
    return ndc;
}


NDCList
NDCFactory::from_string(const char* ndcs) const
{
    CDMW_LOG_FUNCTION(NDC_LOGGER, "NDCList=" << ndcs);

    NDCList ndc_list;

    std::istringstream iss(ndcs);
    std::string ndc_str;
    while ( std::getline( iss, ndc_str, '/' ) )
    {
        try {
            NDC ndc(ndc_str);
            ndc_list.push_back(ndc);
        } catch (Cdmw::BadParameterException::BadParameterException ex) {
            CDMW_WARN(NDC_LOGGER, "Invalid NDC string format: " << ex.what());
        }
    }

    CDMW_LOG_FUNCTION_RETURN("NDC list length = " << ndc_list.size());
    return ndc_list;
}

NDCList
NDCFactory::from_octetseq(const CORBA::OctetSeq& ndcs) const
{
    CDMW_LOG_FUNCTION(NDC_LOGGER, "");

    NDCList ndc_list;
    
    std::string ndcs_str;
    for (unsigned int i=0; i < ndcs.length(); i++) {
       ndcs_str += ndcs[i];
    }
    
    ndc_list = from_string(ndcs_str.c_str());
    
    return ndc_list;
}

NDCList
NDCFactory::from_any(const CORBA::Any& ndcs) const
{
    CDMW_LOG_FUNCTION(NDC_LOGGER, "");

    NDCList ndc_list;

    CORBA::TypeCode_var tc = ndcs.type();
    if (tc->kind() != CORBA::tk_null)
      {
    const char* ndcs_str;
    if ( ndcs >>= ndcs_str ) {
      ndc_list = from_string(ndcs_str);
    }
      }
    
    return ndc_list;
}


std::string
NDCFactory::to_string(const NDCList& ndcs) const
{
    CDMW_LOG_FUNCTION(NDC_LOGGER, "");

    std::string ndcs_str;
    NDCList::const_iterator iter = ndcs.begin();
    while ( iter != ndcs.end() ) {
        if (iter != ndcs.begin()) {
            ndcs_str += '/';
        }
        ndcs_str += (*iter).to_string();
        iter++;
    }
    
    CDMW_LOG_FUNCTION_RETURN("NDC list = " << ndcs_str);
    return ndcs_str;
}

CORBA::OctetSeq*
NDCFactory::to_octetseq(const NDCList& ndcs) const
{
    CDMW_LOG_FUNCTION(NDC_LOGGER, "");
    
    std::string ndcs_str = to_string(ndcs);
    
    CORBA::OctetSeq_var ndcs_os = new CORBA::OctetSeq(ndcs_str.size());
    
    ndcs_os->length(ndcs_str.size());
    for (unsigned int i=0; i < ndcs_str.size(); i++) {
       ndcs_os[i] = ndcs_str[i];
    }

    return ndcs_os._retn();
}

CORBA::Any*
NDCFactory::to_any(const NDCList& ndcs) const
{
    CDMW_LOG_FUNCTION(NDC_LOGGER, "");
    
    std::string ndcs_str = to_string(ndcs);
    
    CORBA::Any ndcs_any;
    ndcs_any <<= ndcs_str.c_str();    
    
    return new CORBA::Any(ndcs_any);
}


} // End namespace RequestFollowUp
} // End namespace CommonSvcs
} // End namespace Cdmw

