/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

package cdmw.commonsvcs.requestfollowup;

import java.util.*;
import java.util.UUID;

import org.omg.CORBA.*;
//import org.omg.IOP.*;

import cdmw.common.*;
//import cdmw.logging.*;

//import cdmw.orbsupport.ExceptionMinorCodes;
import static cdmw.logging.Logger.*;
import static cdmw.logging.FunctionLogger.*;


//
// NDCFactory class
//
public class NDCFactory
{
    private String  m_UUIDBase;
    private int m_requestCounter = 0;
    private org.omg.CORBA.ORB m_orb = null;

    private static NDCFactory M_instance = null;

    //private static Mutex M_mutex;

    /**
     * Constructor
     */       
    private NDCFactory() {
        // Initialize an UUID base for creation of new NDC
        UUID uuid = UUID.randomUUID();
        m_UUIDBase = uuid.toString();
        m_orb = org.omg.CORBA.ORB.init();
    }
    
    /**
     * Init a NDCLogger.
     * If called more than once, an assert produce a core dump
     */
    public static void initNDCFactory() {
        CDMW_LOG_FUNCTION(NDCLogger.getInstance().getLogger(), "");

        Assert.check(M_instance == null);
        M_instance = new NDCFactory();
    }


    /**
     * Accessors
     */
    public static NDCFactory getInstance() {
        CDMW_LOG_FUNCTION(NDCLogger.getInstance().getLogger(), "");
        Assert.check(M_instance != null);
        
        return M_instance;
    }

    public String getUUIDBase() {
        return m_UUIDBase;
    }

    // Used only in Test, not protected
    public int getRequestCounter() {
        return m_requestCounter;
    }


    /**
     * Creation of a new NDC.
     *
     * @param operation of current request follow up
     */
    synchronized public NDC createNewNDC(String operation) {
        CDMW_LOG_FUNCTION(NDCLogger.getInstance().getLogger(),
        "Operation=" + operation + 
        ", UUID=" + m_UUIDBase + 
        ", RequestId=" + (m_requestCounter+1));

        //CDMW_MUTEX_GUARD(M_mutex);
        m_requestCounter++;
        NDC ndc = new NDC(operation, m_UUIDBase, m_requestCounter);
        
        CDMW_LOG_FUNCTION_RETURN(NDCLogger.getInstance().getLogger(),
            ndc.toString() );
        return ndc;
    }

    public NDCList fromString(String ndcs) {
        CDMW_LOG_FUNCTION(NDCLogger.getInstance().getLogger(), "" + ndcs);

        NDCList ndcList = new NDCList();

        int pos1 = 0;
        int pos2 = 0;
        try {
            // '/' is separator between several NDC 
            while ( (pos2 = ndcs.indexOf('/', pos1))  != -1 ) {  
                NDC ndc = new NDC(ndcs.substring(pos1, pos2));
                ndcList.list.addLast(ndc);
                pos1 = pos2 + 1; // skip '/' separator
            }
            NDC ndc = new NDC(ndcs.substring(pos1));
            ndcList.list.addLast(ndc);
        }
        catch (BadParameterException ex) {
            CDMW_WARN(NDCLogger.getInstance().getLogger(),
                "Invalid NDC string format (pos=" + pos1 + "): " + ex.what(),
                new Throwable());
        }

        CDMW_LOG_FUNCTION_RETURN(NDCLogger.getInstance().getLogger(), "NDCList length=" + ndcList.list.size());
        return ndcList;
    }
    
    public String toString(NDCList ndcList) {
        CDMW_LOG_FUNCTION(NDCLogger.getInstance().getLogger(), "NDCList length=" + ndcList.list.size());
        
        String ndcsStr = "";

        ListIterator<NDC> iter = ndcList.list.listIterator(0);
        while (iter.hasNext())
        {
            NDC ndc = iter.next();
            if ( iter.previousIndex() > 0) {
                ndcsStr += '/';
            }
            ndcsStr += ndc.getOperation() + ":" + ndc.getUUID() + "-" + ndc.getRequestId();
        }
        
        CDMW_LOG_FUNCTION_RETURN(NDCLogger.getInstance().getLogger(), ndcsStr);
        return ndcsStr;
    }
    
    public NDCList fromOctetSeq(byte [] ndcs) {
        CDMW_LOG_FUNCTION(NDCLogger.getInstance().getLogger(), "");
        
        String ndcsStr = new String(ndcs);
        
        NDCList ndcList = fromString(ndcsStr);
        
        return ndcList;
     }
    
    public byte [] toOctetSeq(NDCList ndcs) {
        CDMW_LOG_FUNCTION(NDCLogger.getInstance().getLogger(), "");
        
        String ndcsStr = toString(ndcs);
        byte [] os = ndcsStr.getBytes();
        
        return os;      
    }
    
    public NDCList fromAny(Any any) {
        CDMW_LOG_FUNCTION(NDCLogger.getInstance().getLogger(), "");
        
        NDCList ndcList = new NDCList();
        
        TypeCode tc = any.type();
        if (tc.kind().value() != TCKind._tk_string) {

            if (tc.kind().value() == TCKind._tk_null) {
                CDMW_LOG(NDCLogger.getInstance().getLogger(), CDMW_DEBUG_3,
                    "NDC Slot is empty.",
                    new Throwable());
            }
            else {
                CDMW_WARN(NDCLogger.getInstance().getLogger(),
                    "Unexpected type stored in NDC Slot!",
                    new Throwable());
            }
        }
        else {
            ndcList = fromString(any.extract_string());
        }
        
        return ndcList;
    }

    public Any toAny(NDCList ndcs) {
        CDMW_LOG_FUNCTION(NDCLogger.getInstance().getLogger(), "");
        
        String ndcsStr = toString(ndcs);
        
        Any any = m_orb.create_any();
        any.insert_string(ndcsStr);
        
        return any;
    }

}

