/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO 
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2009. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */
package cdmw.commonsvcs.requestfollowup;

import java.util.*;

//import org.omg.IOP.*;

//import cdmw.common.*;
import cdmw.logging.*;

import cdmw.common.Assert;
import cdmw.orbsupport.ExceptionMinorCodes;
import static cdmw.logging.Logger.*;
import static cdmw.logging.FunctionLogger.*;

public class NDCLogger {

    /**
     * CDMW specific Service Context Id for org.omg.CORBA.ServiceContext
     */
    public static final int CDMW_NDC_SERVICE_CONTEXT_ID = 1 | ExceptionMinorCodes.CDMW_VMCID;

    
    /**
     * Used Interception Point in Request Follow Up 
     */
    public static final int NDC_SERVER_RECEIVE_REQUEST     =  1;
    public static final int NDC_SERVER_SEND_REPLY          =  2;
    public static final int NDC_SERVER_SEND_OTHER          =  3;
    public static final int NDC_SERVER_SEND_EXECEPTION     =  4;
    public static final int NDC_CLIENT_SEND_REQUEST        =  5;
    public static final int NDC_CLIENT_SEND_POLL           =  6;
    public static final int NDC_CLIENT_RECEIVE_REPLY       =  7;
    public static final int NDC_CLIENT_RECEIVE_EXCEPTION   =  8;
    public static final int NDC_CLIENT_RECEIVE_OTHER       =  9;
    
    public static final String sNDC_SERVER_RECEIVE_REQUEST     = "SERVER_RECEIVE_REQUEST";
    public static final String sNDC_SERVER_SEND_REPLY          = "SERVER_SEND_REPLY";
    public static final String sNDC_SERVER_SEND_OTHER          = "SERVER_SEND_OTHER";
    public static final String sNDC_SERVER_SEND_EXECEPTION     = "SERVER_SEND_EXECEPTION";
    public static final String sNDC_CLIENT_SEND_REQUEST        = "CLIENT_SEND_REQUEST";
    public static final String sNDC_CLIENT_SEND_POLL           = "CLIENT_SEND_POLL";
    public static final String sNDC_CLIENT_RECEIVE_REPLY       = "CLIENT_RECEIVE_REPLY";
    public static final String sNDC_CLIENT_RECEIVE_EXCEPTION   = "CLIENT_RECEIVE_EXCEPTION";
    public static final String sNDC_CLIENT_RECEIVE_OTHER       = "CLIENT_RECEIVE_OTHER";
    public static final String sNDC_UNKNOWN_IP                 = "UNKNOWN_INTERCEPTION_POINT";

    public void CDMW_REQFU(
            cdmw.logging.Logger logger, 
            String msg,
            String ndc,
            Throwable thr) {
        logger.log(cdmw.logging.Logger.REQ_FOLLOWUP, msg, ndc, thr);
    }

    String NDC_LOGGER_NAME = "CDMW.FDS.RequestFollowUp";
    
    /**
     * Singleton M_instance
     */
    private static NDCLogger M_instance = null;
    
    /**
     * maximum duration of a request
     */
    private long logFullNDCList = 0; 
    private long logNDCOperation = 0; 
    
    /**
     * Initialize operations list we don't log in Request Follow Up
     */
    private String [] notLoggedOperationList = {
        "put_formatted_messages",
        "add_log"
    };
    
    private cdmw.logging.Logger m_NDCLogger;

    public cdmw.logging.Logger getLogger() {
        return m_NDCLogger;
    }
    
   /**
     * 
     * @return NDCLogger
     */
    public static NDCLogger getInstance() {
        Assert.check(M_instance != null);
        
        return M_instance;
    }

    /**
     * init a NDCLogger.
     * if called more than once, an assert produce a core dump
     *
     * @param log_full_ndc_list request follow up log full NDCs list
     * @param log_ndc_operation request follow up log operation of the NDCs
     *
     */
    public static void initNDCLogger(int logList, int logOperation) {
        Assert.check(M_instance == null);
        M_instance = new NDCLogger();
        
        // Initialize format of the NDC when logging Request Follow Up
        M_instance.logFullNDCList = logList;
        M_instance.logNDCOperation = logOperation;
    }

    /**
     * Constructor 
     */    
    private NDCLogger() {
        m_NDCLogger = cdmw.logging.Logger.Get_Logger(NDC_LOGGER_NAME);
    }

    /**
     * This operation return string format of an Interception Point
     *
     * @param ip  int value of an Interception Point.
     * @return ipStr  string value of the Interception Point
     */
    public String ipToString(int ip) {
        
        switch (ip)
        {
        case NDC_SERVER_RECEIVE_REQUEST:
            return sNDC_SERVER_RECEIVE_REQUEST;
        case NDC_SERVER_SEND_REPLY:
            return sNDC_SERVER_SEND_REPLY;
        case NDC_SERVER_SEND_OTHER:
            return sNDC_SERVER_SEND_OTHER;
        case NDC_SERVER_SEND_EXECEPTION:
            return sNDC_SERVER_SEND_EXECEPTION;
        case NDC_CLIENT_SEND_REQUEST:
            return sNDC_CLIENT_SEND_REQUEST;
        case NDC_CLIENT_SEND_POLL:
            return sNDC_CLIENT_SEND_POLL;
        case NDC_CLIENT_RECEIVE_REPLY:
            return sNDC_CLIENT_RECEIVE_REPLY;
        case NDC_CLIENT_RECEIVE_EXCEPTION:
            return sNDC_CLIENT_RECEIVE_EXCEPTION;
        case NDC_CLIENT_RECEIVE_OTHER:
            return sNDC_CLIENT_RECEIVE_OTHER;
        }

        return sNDC_UNKNOWN_IP;
    }

    /**
     * This operation check if the operation has to be follow up
     *
     * @param operation  string operation to check.
     */
    public boolean isOperationToLog(String operation) {
        
        for ( int i=0 ; i<notLoggedOperationList.length ; i++) {
            if ( operation.equals(notLoggedOperationList[i])) {
                return false;
            }
        }

        return true;
    }

    /**
     * This operation log the Request Follow Up information
     *
     * @param operation  string operation to log.
     */
    public void logRequest(int ip, String operation, NDCList ndcList) {

        String ndcsStr = "";

        if ( ndcList.list.isEmpty() ) {
            // If no NDC in the list, log an empty NDC.
        }
        else if ( logFullNDCList == 1 && logNDCOperation == 1 ) {
            ndcsStr = NDCFactory.getInstance().toString(ndcList);
        }
        else  if ( logFullNDCList == 1 && logNDCOperation == 0 ) {
            ListIterator<NDC> iter = ndcList.list.listIterator(0);
            while (iter.hasNext())
            {
                if ( iter.hasPrevious() ) {
                    ndcsStr += '/';
                }
                NDC ndc = iter.next();
                ndcsStr += ndc.getUUID() + "-" + ndc.getRequestId();
            }
        }
        else if ( logFullNDCList == 0 && logNDCOperation == 1 ) {
            NDC ndc = ndcList.list.getFirst();
            ndcsStr = ndc.toString();
        }
        else { // if ( logFullNDCList == 0 && logNDCOperation == 0 ) {
            NDC ndc = ndcList.list.getFirst();
            ndcsStr = ndc.getUUID() + "-" + ndc.getRequestId();
        }

        //System.out.println("REQ_FOLLOWUP : " + " [" + rfuStr + "]\t" + ipToString(ip) + " (" + operation + ")" + "\n");
        CDMW_REQFU(NDCLogger.getInstance().getLogger(), ipToString(ip) + " (" + operation + ")", ndcsStr,
                new Throwable());
    }

}
