/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

#ifndef INCL_CDMW_COMMNSVCS_SERVICE_REGISTRATION_SERVICE_REGISTRATION_IMPL_HPP
#define INCL_CDMW_COMMNSVCS_SERVICE_REGISTRATION_SERVICE_REGISTRATION_IMPL_HPP


#include <Foundation/osthreads/Mutex.hpp>
#include <Foundation/osthreads/Condition.hpp>
#include <Foundation/orbsupport/CORBA.hpp>
#include <Foundation/idllib/CdmwServiceLocator.skel.hpp>

#include <iostream>
#include <map>

namespace Cdmw
{
namespace CommonSvcs
{
namespace ServiceRegistration
{
  /**
   * CARDAMOM Registration service.
   *
   * Helper interface to allow users to register services and get them back
   * from anywhere. A service must be identified by a unique name.
   *
   * @see IDL:thalesgroup.com/Foundation/CdmwServiceLocator/ServiceRegistration
   */
   class  ServiceRegistration_impl :
         public virtual POA_CdmwServiceLocator::ServiceRegistration,
         public virtual PortableServer::RefCountServantBase
   {

      public :
         /**
          * Purpose:
          * <p>
          * Initialise the ServiceRegistration singleton object.
          * @param orb Reference of the ORB pseudo object
          * @param goa Reference of the Group Object Adapter
          * @param service_locator_address 
          * @param registration_service_address 
          * @param nb_retries Number of attempts for service discovery interrogation
          * @param time_interval Time interval (milliseconds) between two consecutives 
          *        interrogations
          * @return Pointer to the singleton object
          * @exception CORBA::BAD_INV_ORDER Raise with BAD_INV_ORDERInitAlreadyDone minor
          *            code when Init() was already called
          * @exception CORBA::SystemException Any CORBA system exception
          *
          */  
         static ServiceRegistration_impl* Init(CORBA::ORB_ptr orb,
                                               PortableGroup::GOA_ptr goa,
                                               std::string service_locator_address,
                                               std::string registration_service_address,
                                               int nb_retries,
                                               int time_interval)
            throw (CORBA::SystemException);
    
         ~ServiceRegistration_impl();
         /**
          * Purpose:
          * <p>Returns a pointer to the singleton object
          * @return Pointer to the singleton object
          * @exception CORBA::BAD_INV_ORDER Raised with BAD_INV_ORDERInitNotCalled 
          *            minor code when Init() is not called before.
          */
         static ServiceRegistration_impl* Get_instance() throw (CORBA::BAD_INV_ORDER);

         /**
          * Purpose:
          * <p>
          * implements the
          * IDL:thalesgroup.com/Foundation/CdmwServiceLocator/register_service:1.0
          * operation
          * NOTA: Service name shall be compliant to OMG standard CosNaming stringified
          *       name and shall only contain one name element. A service which name 
          *       does not comply with this requirement will not be registered!
          * @param name Service name
          * @param ref Service object reference
          *
          */  
         virtual void register_service(const char * name, CORBA::Object_ptr ref)
            throw(CORBA::SystemException);
    
         /**
          * Purpose:
          * <p> register locally the service.
          * NOTA: Service name shall be compliant to OMG standard CosNaming stringified
          *       name and shall only contain one name element. Assertion will be used
          *       to check that name complies with this requirement.
          * @param name Service name
          * @param ref Service object reference
          * @param force Indicates wether to overwrite previously registered services or not.
          *        Default is not to overwrite.
          * @return <B>true</B> of registration succeeded, <B>false</B> if already registered.
          * 
          */    
         bool register_local_service(const char * name, 
                                     CORBA::Object_ptr ref, 
                                     bool force = false);

         /**
          * Purpose:
          * <p> unregister a service from local cache.
          * NOTA: Service name shall be compliant to OMG standard CosNaming stringified
          *       name and shall only contain one name element. Assertion will be used
          *       to check that name complies with this requirement.
          * @param name Service name
          * @return <B>true</B> of unregistration succeeded, <B>false</B> if not already registered.
          * 
          */    
         bool unregister_service(const char * name);

         /**
          * Purpose:
          * <p> Get the service reference from its name. This may use a discovery
          *     protocol to get the service's object reference.
          * @param name Service name
          * @param do_refresh Indicates wether to acquire new object reference via
          *        the discovery protocol or to return already cached one.
          *        Default is return already cached reference if any. 
          * @return Service object reference. Returns nil object reference if no reference
          *        is available.dd 
          */
         CORBA::Object_ptr get_service(const char * name, bool do_refresh = false);


         typedef std::map<std::string, CORBA::Object_var, std::less<std::string> > ServiceMap;
    
         /**
          * Purpose:
          * <p> Returns a copy of the underlying service map.
          * 
          */
         ServiceMap get_known_services();
    
         /**
          * Purpose:
          * <p> Returns a Root Naming Context.
          * @param force_cache_refresh States whether to use local cache or force cache refresh. 
          * Default value force cache refresh for requested bindings.
          * @return Root Naming Context that either updates the cache systematically or reuses
          *         previously cached values, if any.
          * 
          */
         inline CosNaming::NamingContextExt_ptr get_root_context(bool force_cash_refresh=false)
         {
            if (force_cash_refresh)
               return CosNaming::NamingContextExt::_duplicate(m_root_with_cache_refresh.in());
            else
               return CosNaming::NamingContextExt::_duplicate(m_root.in());
         }
    
      protected:
         // protected constructor
         ServiceRegistration_impl(CORBA::ORB_ptr orb,
                                  PortableGroup::GOA_ptr goa,
                                  std::string service_locator_address,
                                  std::string registration_service_address,
                                  int nb_retries,
                                  int time_interval)
            throw (CORBA::SystemException);
         /**
          * Purpose:
          * <p>Checks wether a service name is of a standard format.
          * NOTA: Service name shall be compliant to OMG standard CosNaming stringified
          *       name and shall only contain one name element. A service which name 
          *       does not comply with this requirement will not be registered!
          * @param s Service bame
          * @return <B>true</B> if the service name is well formed, <B>false</B> otherwise.
          * 
          */
         bool is_valid_name(const char* s) throw();

      private:
         // hide default constructor and copy constructor  
         ServiceRegistration_impl();
         ServiceRegistration_impl(const ServiceRegistration_impl&);
         void operator=(const ServiceRegistration_impl&);
    
         struct CacheData
         {
               CacheData()
                  : m_obj(CORBA::Object::_nil()),
                    m_is_refreshed(false),
                    m_is_local(false)
               {
               }
               CacheData(CORBA::Object_ptr obj, bool fresh = true, bool local = false)
                  : m_obj(CORBA::Object::_duplicate(obj)),
                    m_is_refreshed(fresh),
                    m_is_local(local)
               {
               }
               CacheData(const CacheData& cd)
                  : m_obj(CORBA::Object::_duplicate(cd.m_obj.in())),
                    m_is_refreshed(cd.m_is_refreshed),
                    m_is_local(cd.m_is_local)
               {
               }

               CacheData & operator=(const CacheData & rhs)
               {
                  if (this != &rhs) {
                     m_obj = CORBA::Object::_duplicate(rhs.m_obj.in());
                     m_is_refreshed = rhs.m_is_refreshed;
                     m_is_local = rhs.m_is_local;
                  }
                  return *this;
               }
               /**
                * Object reference for a cached service
                */
               CORBA::Object_var m_obj;
               /**
                * Boolean indicated that data has been refreshed or not
                */
               bool              m_is_refreshed;
               /**
                * Boolean indicated that data is local and will not be synchronised with other daemons
                */
               bool              m_is_local;
            
         };
         typedef std::map<std::string, CacheData, std::less<std::string> > InternalServiceMap;

         /**
          * The global services.
          */
         InternalServiceMap m_services;
    
         /**
          * The mutex protecting concurrent access to m_globalServices.
          * (associated to the receiving registration condition)
          */
         OsSupport::Mutex m_servicesMutex;
    
         /**
          * The condition for receiving the registration.
          */
         OsSupport::Condition m_servicesCondition;
    
         /**
          * The orb reference.
          */
         CORBA::ORB_var m_orb;

         /**
          * The poa reference.
          */
         PortableGroup::GOA_var m_goa;
         /**
          * The root naming context.
          */
         CosNaming::NamingContextExt_var m_root;
         /**
          * The root naming context with forcing cache refresh.
          */
         CosNaming::NamingContextExt_var m_root_with_cache_refresh;
    
         /**
          * The max number of retries to interogate the service locator.
          */
         int m_nb_retries;
    
         /**
          * The time interval between two interogations.
          */
         int m_time_interval;

         /**
          * The service locator to interrogate.
          */
         CdmwServiceLocator::ServiceLocator_var m_service_locator;
    
         /**
          * The string reference of the registration service receiving the answers..
          */
         std::string m_registration_service_address;


         /**
          * The static reference of the serviceRegistration implementation
          */
         static ServiceRegistration_impl* M_instance;
   };
    
} // End namespace ServiceRegistration
} // End namespace CommonSvcs
} // End namespace Cdmw

#endif // INCL_CDMW_COMMNSVCS_SERVICE_REGISTRATION_SERVICE_REGISTRATION_IMPL_HPP
