/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#include <Foundation/orbsupport/ExceptionMinorCodes.hpp>
#include "serviceregistration/NamingContext_impl.hpp"
#include "serviceregistration/SvcLogger.hpp"
#include <Foundation/logging/FunctionLogger.hpp>
#include <Foundation/commonsvcs/naming/NamingInterface.hpp>
#include <iostream>

namespace Cdmw
{
namespace CommonSvcs
{
namespace ServiceRegistration
{


NamingContext_impl::NamingContext_impl(ServiceRegistration_impl* serviceRegistration)
   : m_serviceRegistration(serviceRegistration)
{
   CDMW_LOG_FUNCTION(SvcLogger::GetLogger(),"");
}

NamingContext_impl::~NamingContext_impl()
{
   CDMW_LOG_FUNCTION(SvcLogger::GetLogger(),"");
}


Cdmw::CommonSvcs::Naming::BindingsMap 
NamingContext_impl::get_all_bindings() const
{
   CDMW_LOG_FUNCTION(SvcLogger::GetLogger(),"");
   return m_serviceRegistration->get_known_services();
}

CORBA::Object_ptr 
NamingContext_impl::resolve(const CosNaming::NameComponent& n)
   throw(CosNaming::NamingContext::NotFound,
         CosNaming::NamingContext::CannotProceed,
         CosNaming::NamingContext::InvalidName,
         CORBA::SystemException)
{
   CDMW_LOG_FUNCTION(SvcLogger::GetLogger(),NameComponent_to_string(n));
   CORBA::Object_var obj = CORBA::Object::_nil();
   std::string s = NameComponent_to_string(n);
   try {
      // Use Cache so no forcing of refresh
      obj = m_serviceRegistration->get_service(s.c_str(),false);
   } catch (...) {
      // Ignore
   }
   // NOTA: Service Registration does not allow registration of nil
   //       object references, so a return of a nil object reference 
   //       means that we failed to discover the service within predefined 
   //       time frame.
   //       NamingContext for Ultimate Fallbacks should raise a TRANSIENT system
   //       exception to allow rediscovery of the service again later.
   //       Since this service is for a NameService access, we interpret
   //       this as NotFound within the predefined time frame :-).
   if (CORBA::is_nil(obj.in())) {
      CosNaming::NamingContext::NotFound ex;
      CDMW_LOG_FUNCTION_EXCEPTION(ex);
      throw ex;
   }
   return obj._retn();
}


void NamingContext_impl::bind(const CosNaming::NameComponent& n,
                              CORBA::Object_ptr obj)
   throw(CosNaming::NamingContext::CannotProceed,
         CosNaming::NamingContext::InvalidName,
         CosNaming::NamingContext::AlreadyBound,
         CORBA::SystemException)
{
   CDMW_LOG_FUNCTION(SvcLogger::GetLogger(),NameComponent_to_string(n));
   if (CORBA::is_nil(obj)) {
      CORBA::BAD_PARAM ex(OrbSupport::BAD_PARAMNilObjectReference, CORBA::COMPLETED_NO);
      CDMW_LOG_FUNCTION_EXCEPTION(ex);
      throw ex;
   }
   // Be able to bind objects local objects
   // raise an AlreadyBound exception if already bound (locally!)
   bool result = false;
   std::string s = NameComponent_to_string(n);
   try {
      result = m_serviceRegistration->register_local_service(s.c_str(), 
                                                             obj,
                                                             false);
   } catch (...) {
      CosNaming::NamingContext::CannotProceed ex;
      CDMW_LOG_FUNCTION_EXCEPTION(ex);
      throw ex;
   }
   if (!result) {
      CosNaming::NamingContext::AlreadyBound ex;
      CDMW_LOG_FUNCTION_EXCEPTION(ex);
      throw ex;
   }
}
void NamingContext_impl::rebind(const CosNaming::NameComponent& n,
                                CORBA::Object_ptr obj)
   throw(CosNaming::NamingContext::CannotProceed,
         CosNaming::NamingContext::InvalidName,
         CORBA::SystemException)
{
   CDMW_LOG_FUNCTION(SvcLogger::GetLogger(),NameComponent_to_string(n));
   if (CORBA::is_nil(obj)) {
      CORBA::BAD_PARAM ex(OrbSupport::BAD_PARAMNilObjectReference, CORBA::COMPLETED_NO);
      CDMW_LOG_FUNCTION_EXCEPTION(ex);
      throw ex;
   }
   // Be able to rebind objects local objects (locally!)
   std::string s = NameComponent_to_string(n);
   try {
      (void)m_serviceRegistration->register_local_service(s.c_str(), 
                                                          obj,
                                                          true);
   } catch (...) {
      CosNaming::NamingContext::CannotProceed ex;
      CDMW_LOG_FUNCTION_EXCEPTION(ex);
      throw ex;
   }
}
void NamingContext_impl::bind_context(const CosNaming::NameComponent& n,
                                      CosNaming::NamingContext_ptr nc)
   throw(CosNaming::NamingContext::CannotProceed,
         CosNaming::NamingContext::InvalidName,
         CosNaming::NamingContext::AlreadyBound,
         CORBA::SystemException)
{
   CDMW_LOG_FUNCTION(SvcLogger::GetLogger(),NameComponent_to_string(n));
   // Be able to bind local context 
   // raise an AlreadyBound exception if already bound (locally!)
   bind(n,nc);
}

void NamingContext_impl::rebind_context(const CosNaming::NameComponent& n,
                                        CosNaming::NamingContext_ptr nc)
   throw(CosNaming::NamingContext::CannotProceed,
         CosNaming::NamingContext::InvalidName,
         CORBA::SystemException)
{
   CDMW_LOG_FUNCTION(SvcLogger::GetLogger(),NameComponent_to_string(n));
   // Be able to rebind local context 
   rebind(n,nc);
}
void NamingContext_impl::unbind(const CosNaming::NameComponent& n)
   throw(CosNaming::NamingContext::NotFound,
         CosNaming::NamingContext::CannotProceed,
         CosNaming::NamingContext::InvalidName,
         CORBA::SystemException)
{
   CDMW_LOG_FUNCTION(SvcLogger::GetLogger(),NameComponent_to_string(n));
   // Be able to unbind local object/context 
   bool result = false;
   std::string s = NameComponent_to_string(n);
   try {
      result = m_serviceRegistration->unregister_service(s.c_str());
   } catch (...) {
      CosNaming::NamingContext::CannotProceed ex;
      CDMW_LOG_FUNCTION_EXCEPTION(ex);
      throw ex;
   }
   if (!result) {
      CosNaming::NamingContext::NotFound ex;
      CDMW_LOG_FUNCTION_EXCEPTION(ex);
      throw ex;
   }
}


} // End of namespace ServiceRegistration
} // End of namespace CommonSvcs
} // End of namespace Cdmw

