/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

#ifndef INCL_COMMONSVCS_TIME_UTIL_HPP
#define INCL_COMMONSVCS_TIME_UTIL_HPP

#include <Foundation/common/Exception.hpp>
#include <Foundation/ossupport/OS.hpp>
#include <Foundation/orbsupport/CORBA.hpp>

#include <Foundation/commonsvcs/time/TimeBase.skel.hpp>

#include <limits.h>

namespace Cdmw 
{ 
namespace CommonSvcs 
{ 
  namespace Time 
  {        
    namespace Util
    {
   /**
        * Contains the possible values for NTP Status
        */
        typedef enum 
        {
    /**
          * The host is not synchronized via NTP
          */
          NOT_SYNCHRONIZED,

    /**
          * The host is  synchronized on its local clock
          */
          LOCAL_SYNCHRONIZED,

    /**
          * The host is  synchronized with an External Reference
          */
          EXTERNAL_SYNCHRONIZED,
   /**
          * Status not yet determinated 
          */
          STATUS_NOT_YET_DETERMINATED,
   /**
          * The host is  synchronized on Internal Time Server
          */          
          INTERNAL_SYNCHRONIZED,
    /**
          * Status not  determinated 
          */         
          UNKNOWN

        } NTP_Status;

        const TimeBase::TimeT naTimeT = (TimeBase::TimeT) ULONG_LONG_MAX;
        
        const Cdmw::OsSupport::OS::Timeval naTimeval (ULONG_MAX, ULONG_MAX);
        
   /**
         * Purpose:
         * <p> convert a TimeT value to a struct OsSupport::OS::Timeval
         * @param input_time is the number of 10^(-7) seconds in gregorian date.
         *
         * @return is a Timeval struct or naTimeval if error.
         *
         */
         OsSupport::OS::Timeval
         TimeT_2_Timeval (const TimeBase::TimeT& input_time);
                            

   /**
         * Purpose:
         * <p> convert a struct OsSupport::OS::Timeval to a TimeT value
         * @param input_tv is a Timeval struct.
         *
         * @return the associated output TimeT expressed as the number of 
         *         10^(-7) seconds from gregorian date or naTimeT if error
         *
         */
         TimeBase::TimeT
         Timeval_2_TimeT(const OsSupport::OS::Timeval& input_tv);

         /**
          * Specify the base of time used to compute a TimeBase::TimeT.
          *  GREGORIAN: retrieve 10^(-7) seconds from Fri 15 Oct 1582
          *  EPOCH: retrieve 10^(-7) seconds depending on platform used 
          *             ( 1/1/1970 for unix, 1/1/1601 for windows).
          */
         enum Format
         {
           GREGORIAN,
           EPOCH,
           //...
         };


   /**
         * Purpose:
         * <p> convert a TimeT to a TimeT value in another Format
         *
         * @param input_time is a TimeT struct.
         * @param from is origin Format.
         * @param to is destination Format.
         *
         * @return the proper TimeT or naTimeT if error
         *
         */
         TimeBase::TimeT
         change_base(const TimeBase::TimeT& input_time,
                     Format from, Format to);
         

   /**
         * Purpose:
         * <p> Returns the number of 10^(-7) seconds.
         *     By default the value is from gregorian day 
         *     Fri 15 Oct 1582 (at 00:00:00,000,000), 
         *        Coordinated Universal Time (UTC).
         * 
         * @return  the current date in 10^(-7) seconds
         *
         * @exception InternalErrorException
         */
         TimeBase::TimeT get_current_time()
             throw (::Cdmw::InternalErrorException);
    
   /**
         * Purpose:
         * <p> Check if the local host is synchronized via NTP
         *
         * @return  TRUE if the localhost is synchronized
         *          FALSE if the localhost is not synchronized 
         *
         * @exception InternalErrorException
         */
         bool is_synchronized()
             throw (::Cdmw::InternalErrorException) ;
   /**
         * Purpose:
         * <p> Check if the localhost is locally or externally 
         *     synchronized via NTP or if it is not synchronized.
         *     The host is externally synchronised, 
         *     if the reference clock used is provided 
         *     by an external source 
         *     The host is locally synchronised, 
         *     if the reference clock used is itself 
         *     It returns a possible value of NTP_Status 
         *
         * @param Sdegraded is the threshold limit value for degraded 
         * @param Sfailed       is the threshold  limit value for failed
         *
         * @return  
         *          NOT_SYNCHRONIZED if the localhost is not synchronized
         *          LOCAL_SYNCHRONIZED if the localhost is  synchronized on its localClock
         *          EXTERNAL_SYNCHRONIZED if the localhost is  synchronized with an External Reference
         *          INTERNAL_SYNCHRONIZED if the localhost is  synchronized with one of its peers
         *
         * @exception InternalErrorException
         */
         NTP_Status get_synchronization_status(int Sdegraded, int Sfailed)
             throw (::Cdmw::InternalErrorException);

        /**
        * @brief Conversion from TimeT to string with ISO 8601 representation
        * @param time2convert Value to be converted
        */
        std::string
        TimeT2string_ISO8601(const TimeBase::TimeT& time2convert);

        /**
        * @brief Conversion from TimeT to string, using given format string
        * @param time2convert Value to be converted
        * @param format_string Format string, according to strftime() format
        */
        std::string
        TimeT2formatted_string(const TimeBase::TimeT& time2convert, char* format_string);


    }  /* Util */

  }    /* Time */ 

}      /* CommonSvcs */ 

}      /* Cdmw */

#endif /* INCL_COMMONSVCS_TIME_UTIL_HPP */
