/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */
package cdmw.commonsvcs.time;

import org.omg.TimeBase.*;

public class Time {
        /**
         * Specify the base of time used to compute a TimeBase::TimeT.
         *  GREGORIAN: retrieve 10^(-7) seconds from Fri 15 Oct 1582
         *  EPOCH: retrieve 10^(-7) seconds depending on platform used 
         *             ( 1/1/1970 for unix, 1/1/1601 for windows).
         */
        public static final int GREGORIAN = 0;

        public static final int EPOCH = 1;

        static final cdmw.ossupport.Timeval naTimeval = new cdmw.ossupport.Timeval(
                        java.lang.Integer.MAX_VALUE, java.lang.Integer.MAX_VALUE);

        static final int naTimeT = java.lang.Integer.MAX_VALUE;

        // definition of local constants in an unnammed namespace
        // nbr of 100 nsec in 1 sec
        public static final long HUNDRED_OF_NSEC_IN_SEC = 10000000;

        // TimeVal is in sec/microsec from 1st Jan 1970 (This is different in Windows!!!)
        //
        // Time difference in 100 nanoseconds between DCE and POSIX
        // is 141427 days.  
        public static final long DELTA_POSIX_DCE = 141427 * 86400 * HUNDRED_OF_NSEC_IN_SEC;

        /**
         * Convert a TimeT value to a Timeval object
         * 
         * @param input_time is the number of 10^(-7) seconds in gregorian date.
         *
         * @return is a Timeval struct or naTimeval if error.
         *
         */
        public static cdmw.ossupport.Timeval TimeT_2_Timeval(long input_time) {
                cdmw.ossupport.Timeval output_tv = naTimeval;

                // compute from input_time the integral number of seconds since the Epoch

                // get time in 100nsec from epc
                long timet_epc = input_time - DELTA_POSIX_DCE;

                // if epoch time is valid
                if (timet_epc > 0) {
                        // get time in sec from epc
                        long time_sec = timet_epc / HUNDRED_OF_NSEC_IN_SEC;
                        long time_hund_of_nsec = timet_epc
                                        - (time_sec * HUNDRED_OF_NSEC_IN_SEC);

                        if (time_sec < (java.lang.Long.MAX_VALUE)) {
                                output_tv = new cdmw.ossupport.Timeval((int) (time_sec),
                                                (int) (time_hund_of_nsec / 10));
                        }
                }

                return output_tv;
        }

        /**
         * Convert a Timeval to a TimeT value
         * 
         * @param input_tv is a Timeval struct.
         *
         * @return the associated output TimeT expressed as the number of 
         *         10^(-7) seconds from gregorian date or naTimeT if error
         *
         */
        public static long Timeval_2_TimeT(cdmw.ossupport.Timeval input_tv) {
                long out_time = naTimeT;

                // if input Timeval is valid convert it
                if (!(input_tv == naTimeval)) {
                        out_time = DELTA_POSIX_DCE + (input_tv.getSeconds())
                                        * HUNDRED_OF_NSEC_IN_SEC + (input_tv.getMicroseconds())
                                        * 10;
                }

                return out_time;
        }

        /**
         * Convert a TimeT to a TimeT value in another Format
         *
         * @param input_time is a TimeT struct.
         * @param from is origin Format.
         * @param to is destination Format.
         *
         * @return the proper TimeT or naTimeT if error
         */
        public static long change_base(long input_time, int from, int to) {
                long out_time = input_time;

                // if conversion requested and input valid 
                if (from != to && out_time != naTimeT) {
                        if (from == EPOCH && to == GREGORIAN) {
                                out_time += DELTA_POSIX_DCE;
                        } else if (from == GREGORIAN && to == EPOCH) {
                                if (out_time >= DELTA_POSIX_DCE) {
                                        out_time -= DELTA_POSIX_DCE;
                                }
                        }
                }

                return out_time;
        }

        /**
         * Returns the number of 10^(-7) seconds.
         * By default the value is from gregorian day 
         *     Fri 15 Oct 1582 (at 00:00:00,000,000), Coordinated Universal Time (UTC).
         * 
         * @return the current date in 10^(-7) seconds
         *
         * @exception InternalErrorException
         */
        public static long get_current_time()
        //    throw (.Cdmw.InternalErrorException)
        {
                // first get TimeVal from cdmw.ossupport
                long current_time = Timeval_2_TimeT(cdmw.ossupport.OS.getTime());

                return current_time;
        }

}; // End namespace CommonSvcs

