/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#ifndef _INCL_LOGGING_FILTERMNGR_HPP_
#define _INCL_LOGGING_FILTERMNGR_HPP_

#include <list>
#include <map>
#include <stdexcept>
#include <sstream>
#include <string>

#include <Foundation/common/Exception.hpp>
#include <Foundation/common/System.hpp>
#include <Foundation/osthreads/Mutex.hpp>
#include <Foundation/osthreads/ReaderWriterLock.hpp>
#include <Foundation/logging/LogLevel.hpp>


namespace Cdmw
{

namespace Logging
{

class Filter;
class FilterMngr;



/**
 * This structure fully describes the origin of a message, and 
 * it is use to prefix each message.
 */
struct FilterData
{
    /**
     * The component name.
     */
    std::string m_component_name;
    
    /**
     * The domain name.
     */
    std::string m_domain_name;
    
    /**
     * The level mask.
     */
    LevelType m_levels;
};

// The type of container containing the filter data
typedef std::list<FilterData> FilterDataList;

// The type of container containing the filter data
typedef std::map<std::string, Filter*> FilterContainer;

// The type of container containing the component filter manager data
typedef std::map<std::string, FilterMngr*> FilterMngrContainer;



/**
 * Define a filter that will either let a message be logged or not.
 * At any time, a filter is either activated or deactivated.
 */
class Filter
{
    public:
        

        /**
         * Purpose:
         * <p>  Create a new Filter with a specified level mask.
         *
         * @param levels the specified level mask
         */
        Filter(LevelType levels);
        
        
        /**
         * Purpose:
         * <p>   Destructor.
         */
        virtual
        ~Filter(void)
            throw();


        /**
         * Purpose:
         * <p>   Get the level of the filter
         *
         * @return the level value
         */
        LevelType get_level();
        
        /**
         * Purpose:
         * <p>   Get the reference of the level
         *   
         * @return the level reference
         */
        LevelType * get_level_ref();


        /**
         * Purpose:
         * <p>   Set a default level to be set to all newly created filters
         *
         * @param level the default level
         */
        void set_default_level (LevelType level);


        /**
         * Purpose:
         * <p>   Activate a specific level of the filter
         * note : in the input level the level bits to activate are set
         *
         * @param level the level to activate
         */
        void activate_level (LevelType level);



        /**
         * Purpose:
         * <p>   Activate all levels of the filter.
         */
        void activate_all_levels (void);



        /**
         * Purpose:
         * <p>   Deactivate a specific level of the filter
         * note : in the input level the level bits to deactivate are set
         *
         * @param level the level to deactivate
         */
        void deactivate_level (LevelType level);



        /**
         * Purpose:
         * <p>   Deactivate all levels of the filter.
         */
        void deactivate_all_levels (void);
        
        
        /**
         * Purpose:
         * <p>   Check if a level is activated in the filter.
         * note : in the input level the level bits to check are set
         *
         * @param level the level to check
         *
         * @return true if all level bits to check are set or false
         */
        bool is_level_activated(LevelType level);

                     
        /**
         * Purpose:
         * <p>   Check if the filter is associated to a domain or to a component
         *
         * @return true if filter is associated to a domain
         */
        bool is_domain_filter();

    protected:
        
        /**
         * Purpose:
         * <p>   Set the filter associated to a component (filter manager)
         */
        void set_component_filter();
        
        /**
         * the level mask
         */
        LevelType m_levels;
        
        /**
         * flag indicating if filter is associated
         * to a component or to a domain
         */
        bool m_domain_filter;
        
        
        // mutex to control the access to the level of filter
        OsSupport::ReaderWriterLock m_level_mtx;       
};



/**
 * Define a filter manager which is a component filter 
 * that will either let a message be logged or not.
 *
 */
class FilterMngr : virtual public Filter
{
    // inject the names of base member into the scope of FilterMngr
    using Filter::activate_level;
    using Filter::deactivate_level;
    using Filter::activate_all_levels;
    using Filter::deactivate_all_levels;
    
    public:
        
        /**
         * Purpose:
         * <p>   Create a new Filter Manager with a specified level mask.
         *
         * @param levels the specified level mask
         */
        FilterMngr(LevelType levels);
        
        
        /**
         * Purpose:
         * <p>   Destructor.
         */
        virtual
        ~FilterMngr(void)
            throw();

            
        /**
         * Purpose:
         * <p>   Activate the level of a component/domain
         *
         * The component filter is created if not found
         *
         * @param component the name of the component
         * @param domain the name of the domain
         * @param level the level to activate
         *
         */
        void activate_level(const std::string& component,
                            const std::string& domain,
                            LevelType level);
                                
        /**
         * Purpose:
         * <p>   Activate the level of a domain
         *
         * The domain filter is created if not found
         *
         * @param domain the name of the domain
         * @param level the level to activate
         *
         */
        void activate_level(const std::string& domain,
                            LevelType level);
        
        /**
         * Purpose:
         * <p>   Activate all levels of a component/domain
         *
         * The component filter is created if not found
         *
         * @param component the name of the component
         * @param domain the name of the domain
         */
        void activate_all_levels(const std::string& component,
                                 const std::string& domain);
                            
        /**
         * Purpose:
         * <p>   Activate all levels of a domain
         *
         * The domain filter is created if not found
         *
         * @param domain the name of the domain
         */
        void activate_all_levels(const std::string& domain);
        
        /**
         * Purpose:
         * <p>   Deactivate the level of a component/domain
         *
         * The component filter is created if not found
         *
         * @param component the name of the component
         * @param domain the name of the domain
         * @param level the level to deactivate
         */
        void deactivate_level(const std::string& component,
                              const std::string& domain,
                              LevelType level);
                                                  
        /**
         * Purpose:
         * <p>   Deactivate the level of a domain
         *
         * The domain filter is created if not found
         *
         * @param domain the name of the domain
         * @param level the level to deactivate
         */
        void deactivate_level(const std::string& domain,
                              LevelType level);
        
        /**
         * Purpose:
         * <p>   Deactivate all levels of a component/domain
         *
         * The component filter is created if not found
         *
         * @param component the name of the component
         * @param domain the name of the domain
         */
        void deactivate_all_levels(const std::string& component,
                                   const std::string& domain);
        
        /**
         * Purpose:
         * <p>   Deactivate all levels of a domain
         *
         * The domain filter is created if not found
         *
         * @param domain the name of the domain
         */
        void deactivate_all_levels(const std::string& domain);
        

        /**
         * Purpose:
         * <p>   Check if a level for a component/domain is activated.
         *
         * The component filter is created if not found
         *
         * @param component the name of the component
         * @param domain the name of the domain
         * @param level the level to check
         *
         * @return true or false
         *
         */
        bool is_activated(const std::string& component,
                          const std::string& domain,
                          LevelType level);
                          
                          
        /**
         * Purpose:
         * <p>   Check if a level for a domain is activated.
         *
         * The domain filter is created if not found
         *
         * @param domain the name of the domain
         * @param level the level to check
         *
         * @return true or false
         *
         */
        bool is_activated(const std::string& domain,
                          LevelType level);
             
        
        /**
         * Purpose:
         * <p>   Return the reference of the filter for a component/domain
         *
         * The component filter is created if not found
         *
         * @param component the name of the component
         * @param domain the name of the domain
         *         
         * @return the filter reference
         *
         */
        Filter* get_filter_ref(const std::string& component,
                               const std::string& domain);
             
             
        /**
         * Purpose:
         * <p>   Return the reference of the filter for a domain
         *
         * The domain filter is created if not found
         *
         * @param domain the name of the domain
         *         
         * @return the filter reference
         */
        Filter* get_filter_ref(const std::string& domain);
        
        
        /**
         * Purpose:
         * <p>   Returns the list of Filter for this component and sub components.
         *
         * @param list of filters
         * @param component the name of component to get the filter
         *        default is the "root" component
         *
         * @exception OutOfMemoryException
         */
        void get_log_filters (FilterDataList& filter_list,
                              const std::string& component="")
             throw(OutOfMemoryException);

    private:
    
        /**
         * Purpose:
         * <p>   get the path of base component
         *
         * @param full_component_path the component full path ("comp1.comp2.comp3")
         * @param base_component the returned base component ("comp1")
         * @param sub_component_path the returned sub component path ("comp2.comp3")
         *
         * @return the number of remaining sub components
         *
         */
        unsigned int get_component_base (const std::string& full_component_path,
                                         std::string& base_component,
                                         std::string& sub_component_path);
        
        
        /**
         * Purpose:
         * <p>   Search domain in the component, if not found create it
         *
         * @param domain the name of the domain
         *
         * @return the filter
         *
         */
        Filter* search_and_create_dom (const std::string& domain);
        
        
        /**
         * Purpose:
         * <p>   Search component, if not found create it (not thread safe)
         *
         * @param component the name of the component
         *
         * @return the filter
         *
         */
        FilterMngr* search_and_create_comp_unguard (const std::string& component);
        
        /**
         * Purpose:
         * <p>   Search component and sub components
         * if component is not found it is created
         * thread safe
         *
         * @param component the full name of the component
         *
         * @return the filter
         *
         */
        FilterMngr* search_and_create_comp (const std::string& component);    
        
       
                 
        
        // The container containing the domain name and associated filter data
        // The mutex must be aquired prior any manipulations.
        FilterContainer m_domain_filters;

        // mutex to control the access to the list of filters
        OsSupport::Mutex m_domain_filters_mtx;
                
        // The container containing the sub component name and associated filter data
        // The mutex must be aquired prior any manipulations.
        FilterMngrContainer m_component_filters;

        // mutex to control the access to the list of filters
        OsSupport::Mutex m_component_filters_mtx;

};





} // End namespace Logging

} // End namespace Cdmw


#endif // _INCL_LOGGING_FILTERMNGR_HPP_
