/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#ifndef _CDMW_FDS_LOGGING_FUNCTION_LOGGER_HPP_
#define _CDMW_FDS_LOGGING_FUNCTION_LOGGER_HPP_

#include <Foundation/logging/Logger.hpp>
#include <Foundation/common/Assert.hpp>
#include <memory>

namespace Cdmw
{
   namespace Logging
   {
      class FunctionLogger
      {
         public:
            static const LevelType LEVEL = CALL_TRACE;
            
            virtual ~FunctionLogger() throw();

            /**
             * Allows to log a message before the return instruction.
             * Expected to be called just before calling return.
             *
             * @param message the message.
             * @param line the line number in the file.
             */
            virtual void log_return(const char* message, long line) = 0;

            /**
             * Allows to log a message when an exception is raised.
             * Expected to be called just before throwing the exception.
             *
             * @param message the message.
             * @param line the line number in the file.
             */
            virtual void log_exception(const char* message, long line) = 0;

            /**
             * Checks if the CALL_TRACE level has been activated.
             *
             * @return true if enabled, false if not
             */
            virtual bool is_enabled() = 0;

         protected:
            static const char * ENTERING;
            static const char * EXITING;
            static const char * RETURN_MSG_START;
            static const char * RETURN_MSG_END;
            static const char * EXCEPTION_MSG_START;
            static const char * EXCEPTION_MSG_END;

            FunctionLogger();
      };



      template <typename LoggerT>
      struct logger_type_traits
      {
            typedef LoggerT                    logger_type;
            typedef Cdmw::Logging::LevelType   level_type;
            static const level_type CALL_TRACE = FunctionLogger::LEVEL;
      };
//       template<>
//       struct logger_type_traits<Cdmw::Logging::Logger> 
//       {
//             typedef Cdmw::Logging::Logger      logger_type;
//             typedef Cdmw::Logging::LevelType   level_type;
//             static const level_type CALL_TRACE = FunctionLogger::LEVEL;
//       };
//       template<>
//       struct logger_type_traits<CdmwLogging::Logger> 
//       {
//             typedef CdmwLogging::Logger      logger_type;
//             typedef CdmwLogging::LogLevel    level_type;
//             static const level_type CALL_TRACE = Cdmw::Logging::FunctionLogger::LEVEL;
//       };
      
      template <typename  LoggerT> 
      struct logger_traits 
      {
            typedef typename logger_type_traits<LoggerT>::logger_type logger_type;
            typedef typename logger_type_traits<LoggerT>::level_type  level_type;
            static const level_type CALL_TRACE = logger_type_traits<LoggerT>::CALL_TRACE;
            
            /**
             * Check whether a log level is enabled in the specified logger.
             *
             * @param logger the concerned logger
             * @param level the log level
             *
             * @return true if enabled.
             */
            static bool is_enabled_for(logger_type* logger, level_type level)
            {
               return logger->is_enabled_for(level);
            }
            
            /**
             * Log a generic message with the specified logger.
             *
             * @param logger the concerned logger
             * @param level the message level.
             * @param message the message.
             * @param file the file from which the message is output.
             * @param line the line number in the file.
             */
            static void
            log(logger_type* logger, level_type level,
                const char* message,
                const char* file,
                long line)
            {
               logger->log(level,message,file,line);
            }
      };


       template <typename LoggerT>
       class FunctionLoggerT : public FunctionLogger
       {
          public:
             typedef typename logger_traits<LoggerT>::logger_type logger_type;
             typedef typename logger_traits<LoggerT>::level_type  level_type;
             static const level_type CALL_TRACE = logger_traits<LoggerT>::CALL_TRACE;
             
             FunctionLoggerT(logger_type * logger, const char* args,
                             const char * file, const char* function, long line)
                : m_logger(logger), m_file(file), m_function(function), m_line(line), m_message("")
             {
                CDMW_ASSERT(logger != 0);
                if (logger_traits<LoggerT>::is_enabled_for(m_logger,CALL_TRACE)) { 
                   std::ostringstream oss;
                   oss << ENTERING << m_function << "(" << args << ") ...";
                   logger_traits<LoggerT>::log(m_logger,CALL_TRACE, oss.str().c_str(), 
                                               m_file, m_line);
                }
                m_line = -1;
             }
             ~FunctionLoggerT() throw()
             {
                try {
                   if (m_logger->is_enabled_for(LEVEL)) { 
                      std::ostringstream oss;
                      oss << EXITING << m_function << "() ... " << m_message;
                      logger_traits<LoggerT>::log(m_logger,CALL_TRACE, oss.str().c_str(), 
                                                  m_file, m_line);
                   }
                } catch (...) {
                   // ignore
                }
             }
             virtual void log_return(const char* message, long line)
             {
                std::ostringstream oss;
                oss << "[" << RETURN_MSG_START << message << RETURN_MSG_END << "]";
                m_message = oss.str();
                m_line = line;
             }
             virtual void log_exception(const char* message, long line)
             {
                std::ostringstream oss;
                oss << "[" << EXCEPTION_MSG_START << message << EXCEPTION_MSG_END << "]";
                m_message = oss.str();
                m_line = line;
             }
             virtual bool is_enabled() { 
                return logger_traits<LoggerT>::is_enabled_for(m_logger,CALL_TRACE);
             }
          private:
             FunctionLoggerT();
             
             logger_type *  m_logger;
             const char*    m_file;
             const char*    m_function;
             long           m_line;
             std::string    m_message;
       };

      template <typename LoggerT>
      inline FunctionLoggerT<LoggerT> * make_FunctionLogger (LoggerT * logger, const char* args,
                                                             const char * file, 
                                                             const char* function, long line)
      {
         return new FunctionLoggerT<LoggerT>(logger, args,
                                             file, function, line);
      }


    } // end namespace Logging
} // end namespace Cdmw


#if defined __GNUC__

// TODO: Append __FUNCTION__ to variable name

#define _CDMW_MAKE_FUNCTION_LOGGER_NAME_() _CDMW_FUNCTION_LOGGER__
#define _CDMW_MAKE_FUNCTION_LOGGER_ARGS_STRING_NAME_() _CDMW_FUNCTION_LOGGER_ARGS_STRING__
#else

#define _CDMW_MAKE_FUNCTION_LOGGER_NAME_() _CDMW_FUNCTION_LOGGER__
#define _CDMW_MAKE_FUNCTION_LOGGER_ARGS_STRING_NAME_() _CDMW_FUNCTION_LOGGER_ARGS_STRING__

#endif

//#define CDMW_LOG_FUNCTION(logger)  Cdmw::Logging::FunctionLogger _CDMW_MAKE_FUNCTION_LOGGER_NAME_() (logger,__FILE__,__FUNCTION__,__LINE__)

#if defined CDMW_DISABLE_LOG_FUNCTION

#define CDMW_LOG_FUNCTION(logger, X)
#define CDMW_LOG_PRETTY_FUNCTION(logger, X)  
    
// Let's define helper macros!
#define CDMW_LOG_FUNCTION_RETURN(ret)  
#define CDMW_LOG_FUNCTION_EXCEPTION(ex)
#define CDMW_LOG_FUNCTION_RETURN_OUTBOUND(ret) 
#define CDMW_LOG_FUNCTION_EXCEPTION_INBOUND(ex) 
#define CDMW_LOG_FUNCTION_INOUTBOUND(logger, X)

#else

#define CDMW_LOG_FUNCTION(logger, X)                                    \
   std::string _CDMW_MAKE_FUNCTION_LOGGER_ARGS_STRING_NAME_()("");      \
   std::auto_ptr<Cdmw::Logging::FunctionLogger> _CDMW_MAKE_FUNCTION_LOGGER_NAME_() (0); \
   do { \
      if (logger->is_enabled_for(Cdmw::Logging::FunctionLogger::LEVEL)) { \
         std::ostringstream __function_logger_oss__;                    \
         __function_logger_oss__ << X;                                  \
         _CDMW_MAKE_FUNCTION_LOGGER_ARGS_STRING_NAME_()=__function_logger_oss__.str(); \
         _CDMW_MAKE_FUNCTION_LOGGER_NAME_().reset                       \
            (Cdmw::Logging::make_FunctionLogger(logger,_CDMW_MAKE_FUNCTION_LOGGER_ARGS_STRING_NAME_().c_str(),__FILE__,__FUNCTION__,__LINE__));                                                      \
            }                                                           \
   } while(0)

#define CDMW_LOG_FUNCTION_INOUTBOUND(logger, X)                           \
   std::string _CDMW_MAKE_FUNCTION_LOGGER_ARGS_STRING_NAME_()("");        \
   std::auto_ptr<Cdmw::Logging::FunctionLogger> _CDMW_MAKE_FUNCTION_LOGGER_NAME_() (0); \
   do {                                                                   \
      if (logger->is_enabled_for(Cdmw::Logging::FunctionLogger::LEVEL)) { \
         std::ostringstream __function_logger_oss__;                      \
         __function_logger_oss__ << X;                                    \
         _CDMW_MAKE_FUNCTION_LOGGER_ARGS_STRING_NAME_()=__function_logger_oss__.str(); \
         _CDMW_MAKE_FUNCTION_LOGGER_NAME_().reset                         \
            (Cdmw::Logging::make_FunctionLogger(logger,_CDMW_MAKE_FUNCTION_LOGGER_ARGS_STRING_NAME_().c_str(),__FILE__,__PRETTY_FUNCTION__,__LINE__));                                                 \
            }                                                             \
   } while(0)

// Let's define helper macros!
#define CDMW_LOG_FUNCTION_RETURN(ret)                                   \
   do {                                                                 \
      if ((_CDMW_MAKE_FUNCTION_LOGGER_NAME_().get()!= 0) &&             \
          _CDMW_MAKE_FUNCTION_LOGGER_NAME_()->is_enabled()) {           \
         std::ostringstream __function_logger_oss__;                    \
         __function_logger_oss__ << ret;                                \
         _CDMW_MAKE_FUNCTION_LOGGER_NAME_()->log_return(__function_logger_oss__.str().c_str(), __LINE__); \
      }                                                                 \
   } while(0)

#define CDMW_LOG_FUNCTION_RETURN_OUTBOUND(ret)                          \
   do {                                                                 \
      if ((_CDMW_MAKE_FUNCTION_LOGGER_NAME_().get()!= 0) &&             \
          _CDMW_MAKE_FUNCTION_LOGGER_NAME_()->is_enabled()) {           \
         std::ostringstream __function_logger_oss__;                    \
         __function_logger_oss__ << ": OUTBOUND PARAMETER CHECK. " << ret;                                \
         _CDMW_MAKE_FUNCTION_LOGGER_NAME_()->log_return(__function_logger_oss__.str().c_str(), __LINE__); \
      }                                                                 \
   } while(0)   
      
#define CDMW_LOG_FUNCTION_EXCEPTION(ex)                                 \
   do {                                                                 \
      if ((_CDMW_MAKE_FUNCTION_LOGGER_NAME_().get() != 0) &&            \
          _CDMW_MAKE_FUNCTION_LOGGER_NAME_()->is_enabled()) {           \
         std::ostringstream __function_logger_oss__;                    \
         __function_logger_oss__ << ex;                                 \
         _CDMW_MAKE_FUNCTION_LOGGER_NAME_()->log_exception(__function_logger_oss__.str().c_str(), __LINE__); \
      }                                                                 \
   } while(0)

#define CDMW_LOG_FUNCTION_EXCEPTION_INBOUND(ex)                         \
   do {                                                                 \
      if ((_CDMW_MAKE_FUNCTION_LOGGER_NAME_().get() != 0) &&            \
          _CDMW_MAKE_FUNCTION_LOGGER_NAME_()->is_enabled()) {           \
         std::ostringstream __function_logger_oss__;                    \
         __function_logger_oss__ << ": INBOUND PARAMETER CHECK. " << ex;                                 \
         _CDMW_MAKE_FUNCTION_LOGGER_NAME_()->log_exception(__function_logger_oss__.str().c_str(), __LINE__); \
      }                                                                 \
   } while(0)
#endif

#endif // _CDMW_FDS_LOGGING_FUNCTION_LOGGER_HPP_
