
/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#ifndef _CDMW_FDS_LOGGING_LOGLEVEL_HPP_
#define _CDMW_FDS_LOGGING_LOGLEVEL_HPP_

#include <string>
#include <ostream>
#include <sstream>
#include <iomanip>


namespace Cdmw
{
    namespace Logging
    {
        /**
         * the type of log level.
         */
        typedef unsigned long LevelType;
        
        
        /** 
         * Constant identifying all components
         */ 
        const std::string ALL_COMPONENT_NAMES = "*";


        /** 
         * Constant identifying all domains
         */
        const std::string ALL_DOMAINS = "*";


        /** 
         * Constant identifying the component separator
         */ 
        const char COMPONENT_SEP = '.';

        
        /**
         * Default log levels.
         *
         * The mask can be separated into 4 bytes:
         * - first and second are dedicated to CDMW APIs;
         * - third is let to users;
         * - fourth is reserved to CDMW internal usage.
         *
         */
        enum LogLevel
        {         
            // this level reports a fatal error from CDMW; CDMW is no more
            // able to process safely.
            FATAL = 1,
        
            // this level reports an erorr from CDMW.
            ERROR = 2,
        
            // maybe the user has made an error, warn him.
            WARN  = 4,
        
            // used for information purposes.
            INFO  = 8,
        
            // used for debug purposes.
            DEBUG = 16,
        
            /**
             * Additional debug levels.
             */
            DEBUG_1 = 0x00000020,
            DEBUG_2 = 0x00000040,
            DEBUG_3 = 0x00000080,
            DEBUG_4 = 0x00000100,
            DEBUG_5 = 0x00000200,
            DEBUG_6 = 0x00000400,
            DEBUG_7 = 0x00000800,
            DEBUG_8 = 0x00001000,
            DEBUG_9 = 0x00002000,

            //
            // here users have the possiblity to define their own levels.
            //

            /**
             * Levels internal to CDMW.
             * They are reserved to CDMW, yet users are allowed to use them;
             * but so they assume that they are aware of these levels ownership
             * and cannot be confused.
             */
            CDMW_DEBUG_1 = 0x01000000,
            CDMW_DEBUG_2 = 0x02000000,
            CDMW_DEBUG_3 = 0x04000000,
            CDMW_DEBUG_4 = 0x08000000,

            //
            // two vacancies in the mask are remaining
            //

            REQ_FOLLOWUP = 0x10000000,
            CALL_TRACE   = 0x40000000,
            ORB_TRACE    = 0x80000000

        };

        /** 
         * Constant identifying all levels set
         */
        const LevelType ALL_LEVELS_SET = 0xFFFFFFFF;

        /** 
         * Constant identifying none level set
         */
        const LevelType NONE_LEVEL = 0;

        /** 
         * Constant identifying the default level.
         */
        const LevelType DEFAULT_LEVEL  = FATAL | ERROR | WARN | INFO;


        const std::string sFATAL           = "FATAL";
        const std::string sERROR           = "ERROR";
        const std::string sWARN            = "WARN";
        const std::string sINFO            = "INFO";
        const std::string sDEBUG           = "DEBUG";
        const std::string sDEBUG_1         = "DEBUG_1";
        const std::string sDEBUG_2         = "DEBUG_2";
        const std::string sDEBUG_3         = "DEBUG_3";
        const std::string sDEBUG_4         = "DEBUG_4";
        const std::string sDEBUG_5         = "DEBUG_5";
        const std::string sDEBUG_6         = "DEBUG_6";
        const std::string sDEBUG_7         = "DEBUG_7";
        const std::string sDEBUG_8         = "DEBUG_8";
        const std::string sDEBUG_9         = "DEBUG_9";
        const std::string sCDMW_DEBUG_1    = "CDMW_DEBUG_1";
        const std::string sCDMW_DEBUG_2    = "CDMW_DEBUG_2";
        const std::string sCDMW_DEBUG_3    = "CDMW_DEBUG_3";
        const std::string sCDMW_DEBUG_4    = "CDMW_DEBUG_4";
        const std::string sREQ_FOLLOWUP    = "REQ_FOLLOWUP";
        const std::string sCALL_TRACE      = "CALL_TRACE";
        const std::string sORB_TRACE       = "ORB_TRACE";
        const std::string sALL             = "ALL";
        const std::string sDEFAULT         = "DEFAULT";
        const std::string sNONE            = "NONE";


        class LevelUtilities
        {
            
        public:
            /**
             * Return the numeric level as string.
             *
             * @param level the level.
             *
             * @return the level as string.
             */
            static std::string
            Level_To_String(Cdmw::Logging::LevelType level)
            {
                // if several types are in the level, for example ORB_TRACE + ORB_DEBUG,
                // only ORB_DEBUG is relevant
                if ((level & FATAL) != 0)              { return sFATAL;        }
                else if ((level & ERROR) != 0)         { return sERROR;        }
                else if ((level & WARN) != 0)          { return sWARN;         }
                else if ((level & INFO) != 0)          { return sINFO;         }
                else if ((level & DEBUG) != 0)         { return sDEBUG;        }
                else if ((level &  DEBUG_1) != 0)      { return sDEBUG_1;      }
                else if ((level &  DEBUG_2) != 0)      { return sDEBUG_2;      }
                else if ((level &  DEBUG_3) != 0)      { return sDEBUG_3;      }
                else if ((level &  DEBUG_4) != 0)      { return sDEBUG_4;      }
                else if ((level &  DEBUG_5) != 0)      { return sDEBUG_5;      }
                else if ((level &  DEBUG_6) != 0)      { return sDEBUG_6;      }
                else if ((level &  DEBUG_7) != 0)      { return sDEBUG_7;      }
                else if ((level &  DEBUG_8) != 0)      { return sDEBUG_8;      }
                else if ((level &  DEBUG_9) != 0)      { return sDEBUG_9;      }
                else if ((level &  CDMW_DEBUG_1) != 0) { return sCDMW_DEBUG_1; }
                else if ((level &  CDMW_DEBUG_2) != 0) { return sCDMW_DEBUG_2; }
                else if ((level &  CDMW_DEBUG_3) != 0) { return sCDMW_DEBUG_3; }
                else if ((level &  CDMW_DEBUG_4) != 0) { return sCDMW_DEBUG_4; }
                else if ((level &  REQ_FOLLOWUP) != 0) { return sREQ_FOLLOWUP; }
                else if ((level &  CALL_TRACE) != 0)   { return sCALL_TRACE;   }
                else
                {
                   std::ostringstream oss;
                   oss << std::right << std::setw(8) << std::setfill('0') << std::hex << level;
                   return oss.str();
                }
            }
            
        protected:
            LevelUtilities() {}
        };

    } // end namespace Logging
} // end namespace Cdmw


#endif // _CDMW_FDS_LOGGING_LOGLEVEL_HPP_
