/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#ifndef _CDMW_FDS_LOGGING_LOGMANAGER_HPP_
#define _CDMW_FDS_LOGGING_LOGMANAGER_HPP_


#include <string>
#include <list>

#include <Foundation/ossupport/OS.hpp>
#include <Foundation/common/Assert.hpp>
#include <Foundation/common/Exception.hpp>
#include <Foundation/common/NotFoundException.hpp>
#include <Foundation/osthreads/ReaderWriterLock.hpp>
#include <Foundation/logging/FilterMngr.hpp>
#include <Foundation/logging/Logger.hpp>
#include <Foundation/logging/FatalHandler.hpp>
#include <Foundation/logging/CdmwLoggingCollector.stub.hpp>


namespace Cdmw
{
    namespace Logging
    {
        class LogManagerDelegate;

        /**
         *
         */
        class LogManager
        {
            public:
                /**
                 * Destructor.
                 */
                virtual
                ~LogManager()
                    throw();


                /**
                 * Return the singleton instance.
                 */
                inline static
                LogManager* Instance()
                    throw() 
                {
                    CDMW_ASSERT(M_Instance != 0);
                    return M_Instance;
                }


                /**
                 * Initialise the LogManager.
                 *
                 * It creates a default delegate, a target to the logs.
                 * Command-line arguments will be used to initialise the
                 * default logger.
                 *
                 * @return the LogManager that has been initialised.
                 */
                static LogManager*
                Init(int& argc, char** argv)
                    throw(OutOfMemoryException,
                         InternalErrorException);


                /**
                 * Send a message to the logger.
                 *
                 * @param header the message header.
                 * @param msg the message.
                 *
                 * @return true if successful.
                 */
                bool
                add_log(const CdmwLogging::LogHeader& header, const char* msg, const char* ndc="")
                    throw(InternalErrorException);


                /**
                 * Register an alternate delegate.
                 *
                 * @param logmgr the alternate delegate.
                 *
                 * @return the previous registered alternate delegate.
                 */
                LogManagerDelegate*
                register_alternate_delegate(LogManagerDelegate* logmgr);


                /**
                 * Unregister the alternate delegate.
                 *
                 * @param logmgr the alternate delegate.
                 *
                 * @return true if the alternate delegate was previously registered.
                 */
                bool
                unregister_alternate_delegate(LogManagerDelegate* logmgr);


                /**
                 * Get the alternate delegate.
                 *
                 * @return the alternate delegate.
                 */
                LogManagerDelegate*
                get_alternate_delegate() const;

                /**
                 * Get the default delegate.
                 *
                 * @return the default delegate.
                 */
                inline LogManagerDelegate*
                get_default_delegate() const {
                    return m_default_delegate;
                }
                
                /**
                 * Register the FatalHandler.
                 *
                 * @param handler the FatalHandler.
                 *
                 * @return the previous registered FatalHandler.
                 */
                FatalHandler*
                register_fatal_handler(FatalHandler* handler);


                /**
                 * Unregister the FatalHandler.
                 *
                 * @param handler the FatalHandler.
                 *
                 * @return true if the FatalHandler was previously registered.
                 */
                bool
                unregister_fatal_handler(FatalHandler* handler);

                /**
                 * Get the filter manager.
                 *
                 * @return the filter manager.
                 */
                FilterMngr* get_filter_manager() const;

                /**
                 * Search the logger associated to the specified component name and
                 * domain name, create and register a new logger if not existing.
                 * 
                 * @param component The component name.
                 * @param domain The domain name.
                 *
                 * @return the associated logger
                 */
                Logger* 
                get_logger(const std::string& component,
                            const std::string& domain);

                /**
                 * Set the application name.
                 * @param application_name application name
                 */
                void
                set_application_name(const std::string& application_name);

                /**
                 * Get the application name.
                 * @return the application name
                 */
                std::string 
                get_application_name() const;

                /**
                 * Set the process name.
                 * @param process_name process name
                 */
                void 
                set_process_name(const std::string& process_name);

                /**
                 * Get the process name.
                 * @return the process name
                 */
                std::string 
                get_process_name();


            protected:

                /**
                 * Constructor.
                 */
                LogManager() throw();


                /**
                 * Constructor.
                 *
                 * @param argc number of arguments in argv.
                 * @param argv list of arguments.
                 */
                LogManager(int& argc, char** argv)
                    throw(OutOfMemoryException,
                          InternalErrorException);

                /**
                 * Fills the LogContext fields of the calling thread.
                 * @param context the out context object
                 */
                void fill_current_context(CdmwLogging::LogContext& context, const char* ndc="");


                /**
                 * Retrieve the log level value described in command line arguments.
                 * 
                 * @param argc number of arguments in argv.
                 * @param argv list of arguments.
                 * @return the level found
                 * @throws NotFoundException
                 * @throws BadParameterException
                 */
                LevelType read_level(int& argc, char** argv)
                    throw (Cdmw::Common::NotFoundException, Cdmw::BadParameterException);

                /**
                 * Retrieve the output log file described in command line arguments.
                 * 
                 * @param argc number of arguments in argv.
                 * @param argv list of arguments.
                 * @return the found file name
                 * @throws NotFoundException
                 * @throws BadParameterException
                 */
                std::string read_output_file(int& argc, char** argv)
                    throw (Cdmw::Common::NotFoundException, Cdmw::BadParameterException);
            
                /**
                 * Retrieve if the verbose mode is activated in command line arguments.
                 * 
                 * @param argc number of arguments in argv.
                 * @param argv list of arguments.
                 * @return yes if found and activated, false otherwise.
                 * @throws BadParameterException
                 */
                bool read_verbose_mode(int& argc, char** argv)
                    throw (Cdmw::BadParameterException);

                /**
                 * Converts a command line option into the corresponding log
                 * level.
                 * The level must be defined accordingly to the next grammar:
                 *     Integer   |   (Integer [+-])? String ([+-] String )*
                 *
                 * It means next rules must be respected:
                 * - either a number (corresponding to a level mask)
                 * - or a string with level names (or mask names) separated
                 *   with + or - (for activation or deactivation)
                 * - or a number followed by a + or - and level names.
                 *
                 * @param level_option the user option read in the command_line
                 * that must be respectful with the above grammar.
                 *
                 * @return the log level resulting from the combination of
                 * all level names with associated operations.
                 *
                 * @exception BadParameterException raised when the level 
                 * description is not valid
                 */
                virtual LevelType
                parse_log_level_option(const std::string& level_option)
                    throw (Cdmw::BadParameterException);

                /**
                 * Reads in the stream a log level that is written in a string.
                 * Supports all log levels enumerated in LogLevel.hpp, 
                 * and also: DEFAULT, ALL, NONE
                 *
                 * Note that the algorithm is not efficient, the comparison is 
                 * done for each existing level until the good one is found. As
                 * this is used in initialization, it is considered as 
                 * satisfactory.
                 *
                 * @param str_level the string representation of a level.
                 *
                 * @return the numerous level corresponding to the read level
                 * name.
                 *
                 * @exception BadParameterException when an unknown identifier
                 * is discovered into the stream
                 */
                virtual LevelType
                read_level(std::istringstream & str_level)
                    throw (Cdmw::InvalidArgumentException);


            private:
                            
                // the default delegate.
                LogManagerDelegate* m_default_delegate;

                // the alternate delegate supersedes the default
                // delegate if defined.
                LogManagerDelegate* m_alternate_delegate;
                
                // indicates if the default delegate must be used even though an alternate
                // delegate exists 
                bool m_is_verbose;

                // the fatal handler.
                FatalHandler* m_fatal_handler;

                // the filter manager
                FilterMngr* m_filter_manager;
                
                // list of registered loggers associated to a component name and a domain name,
                // to be deleted at destruction
                std::map<std::string, Logger*, std::less<std::string> > m_logger_list;
                
                std::string m_application_name;
                std::string m_process_name;
                Cdmw::OsSupport::OS::ProcessId m_process_id;
                std::string m_hostname;

                // mutex protecting all access to the LogManager.
                static OsSupport::ReaderWriterLock M_logMngr_mtx;
                
                // the singleton object.
                static LogManager* M_Instance;

        };
        
    } // end namespace Logging
} // end namespace Cdmw


#endif // _CDMW_FDS_LOGGING_LOGMANAGER_HPP_
