/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */



#include <Foundation/osthreads/MutexGuard.hpp>
#include <Foundation/osthreads/ReaderLockGuard.hpp>
#include <Foundation/osthreads/WriterLockGuard.hpp>

#include <Foundation/logging/FilterMngr.hpp>



namespace Cdmw
{

namespace Logging
{



/**
 * Create a new Filter with a default level
 */
Filter::Filter(LevelType levels)
{
    m_levels = levels;
    m_domain_filter = true;
}

/**
 * Destructor
 */
Filter::~Filter()
    throw()
{
    // NOOP
}

/**
 * Get the level of the filter
 */
LevelType Filter::get_level()
{
    // note : read Mutex is not used to speed up processing
    
    return m_levels;
}


/**
 * Get the reference of the level
 */
LevelType * Filter::get_level_ref()
{
    return &m_levels;
}


/**
 * Set a default level to be set to all newly created filters
 */
void Filter::set_default_level (LevelType level)
{
    CDMW_WRITER_LOCK_GUARD (m_level_mtx);
    
    m_levels = level;
}

/**
 * Activate a specific level of the filter
 * note : in the input level the level bits to activate are set
 */
void Filter::activate_level (LevelType level)
{
    CDMW_WRITER_LOCK_GUARD (m_level_mtx);
    
    m_levels |= level;
}

/**
 * Activate all levels of the filter.
 */
void Filter::activate_all_levels (void)
{
    CDMW_WRITER_LOCK_GUARD (m_level_mtx);
    
    m_levels = Logging::ALL_LEVELS_SET;
}

/**
 * Deactivate a specific level of the filter
 * note : in the input level the level bits to deactivate are set
 */
void Filter::deactivate_level (LevelType level)
{
    CDMW_WRITER_LOCK_GUARD (m_level_mtx);
    
    m_levels &= ~level;
}

/**
 * Deactivate all levels of the filter.
 */
void Filter::deactivate_all_levels (void)
{
    CDMW_WRITER_LOCK_GUARD (m_level_mtx);
    
    m_levels = 0;
}   


/**
 * Check if a level is activated in the filter.
 * note : in the input level the level bits to check are set
 */
bool Filter::is_level_activated(LevelType level)
{
    bool result = false;
    
    // note : read Mutex is not used to speed up processing
    
    LevelType activation = m_levels & level;
    
    // if result is equal to requested level, filter is activated
    if (activation == level)
    {
        result = true;
    }
    
    return result;
}

/**
 * Check if the filter is associated to a domain or to a component
 *
 */
bool Filter::is_domain_filter()
{
    return m_domain_filter;
}

/**
 * Set the filter associated to a component (or it a filter manager)
 *
 */
void Filter::set_component_filter()
{
    m_domain_filter = false;
}


/**
 * Create a new Component Filter Manager with a default level
 */
FilterMngr::FilterMngr(LevelType levels)
                : Filter (levels)
{
    set_component_filter();
}

/**
 * Destructor
 */
FilterMngr::~FilterMngr()
    throw()
{
    // for each domain in the map
    for (FilterContainer::iterator iter = m_domain_filters.begin(); 
         iter != m_domain_filters.end(); iter++)
    {
        delete iter->second;
    }
    
    // for each component in the map
    for (FilterMngrContainer::iterator iter = m_component_filters.begin(); 
         iter != m_component_filters.end(); iter++)
    {
        delete iter->second;
    }
}


/**
 * Activate the level of a component/domain
 */
void FilterMngr::activate_level (const std::string& component,
                                 const std::string& domain,
                                 LevelType level)
{  
    // first activate level of domains belonging to this component if requested
    if (component == Logging::ALL_COMPONENT_NAMES || component == "")
    { 
        // activate the level for the domain specified
        activate_level (domain, level);
    }

    // then activate level for the component(s) associated to this component
    
    // if all components associated to this component to activate
    if (component == Logging::ALL_COMPONENT_NAMES)
    {
        // activate the level for all component children
        // first lock the mutex of map
        CDMW_MUTEX_GUARD(m_component_filters_mtx);
    
        // for each component in the map
        for (FilterMngrContainer::iterator iter = m_component_filters.begin(); 
             iter != m_component_filters.end(); iter++)
        {
            // activate the level for the component
            (iter->second)->activate_level(Logging::ALL_COMPONENT_NAMES, domain, level);
        }
    }
    // else only one component to activate
    else if (component != "")
    {
        // search the specified component (create it if not found)
        FilterMngr* p_filter = search_and_create_comp (component);
        
        // activate domain level for the found component
        p_filter->activate_level (domain, level);       
    }
}


/**
 * Activate the level of a domain
 */
void FilterMngr::activate_level (const std::string& domain,
                                 LevelType level)
{   
    // first activate default level for this component 
    // if all domain are requested or no domain is requested
    if (domain == Logging::ALL_DOMAINS || domain == "")
    {
        activate_level (level);
    }
    
    // if all domains associated to component to activate
    if (domain == Logging::ALL_DOMAINS)
    {
        // first lock the mutex of map
        CDMW_MUTEX_GUARD(m_domain_filters_mtx);
    
        // for each domain in the map
        for (FilterContainer::iterator iter = m_domain_filters.begin(); 
             iter != m_domain_filters.end(); iter++)
        {
            // activate the level
            (iter->second)->activate_level (level);
        }
    }
    
    // else only one domain to activate the level
    else if (domain != "")
    {
        // search the specified domain (create it if not found)
        Filter* p_filter = search_and_create_dom (domain);
    
        // activate the level for the found domain
        p_filter->activate_level (level);
    }
}

/**
 * Activate all levels of a component/domain
 */
void FilterMngr::activate_all_levels (const std::string& component,
                                      const std::string& domain)
{  
    // first activate level of domains belonging to this component if requested
    if (component == Logging::ALL_COMPONENT_NAMES || component == "")
    { 
        // activate the default all level for this component
        activate_all_levels();
        
        // activate all levels for the domain belonging to this component
        activate_all_levels (domain);
    }

    // then activate all levels for the component(s) associated to this component
    
    // if all components associated to this component to activate
    if (component == Logging::ALL_COMPONENT_NAMES)
    {
        // activate all levels for all component children
        // first lock the mutex of map
        CDMW_MUTEX_GUARD(m_component_filters_mtx);
    
        // for each component in the map
        for (FilterMngrContainer::iterator iter = m_component_filters.begin(); 
             iter != m_component_filters.end(); iter++)
        {
            // activate all levels for the component
            (iter->second)->activate_all_levels(Logging::ALL_COMPONENT_NAMES, domain);
        }
    }
    // else only one component to activate
    else if (component != "")
    {
        // search the specified component (create it if not found)
        FilterMngr* p_filter = search_and_create_comp (component);
        
        // activate all domain levels for the found component
        p_filter->activate_all_levels (domain);       
    }
}

                            
/**
 * Activate all levels of a domain
 */
void FilterMngr::activate_all_levels(const std::string& domain)
{
    // first activate all default levels for this component 
    // if all domain are requested or no domain is requested
    if (domain == Logging::ALL_DOMAINS || domain == "")
    {
        activate_all_levels();
    }
    
    // if all domains associated to component to activate
    if (domain == Logging::ALL_DOMAINS)
    {
        // first lock the mutex of map
        CDMW_MUTEX_GUARD(m_domain_filters_mtx);
    
        // for each domain in the map
        for (FilterContainer::iterator iter = m_domain_filters.begin(); 
             iter != m_domain_filters.end(); iter++)
        {
            // activate all levels
            (iter->second)->activate_all_levels();
        }
    }

    // else only one domain to activate
    else if (domain != "")
    {
        // search the specified domain (create it if not found)
        Filter* p_filter = search_and_create_dom (domain);
    
        // activate all levels for the found domain
        p_filter->activate_all_levels();
    }
}

/**
 * Deactivate the level of a component/domain
 */
void FilterMngr::deactivate_level (const std::string& component,
                                   const std::string& domain,
                                   LevelType level)
{  
    // first deactivate level of domains belonging to this component if requested
    if (component == Logging::ALL_COMPONENT_NAMES || component == "")
    { 
        // deactivate the level for the speciifed domain 
        deactivate_level (domain, level);
    }

    // then deactivate level for the component(s) associated to this component
    
    // if all components associated to this component to deactivate
    if (component == Logging::ALL_COMPONENT_NAMES)
    {
        // deactivate the level for all component children
        // first lock the mutex of map
        CDMW_MUTEX_GUARD(m_component_filters_mtx);
    
        // for each component in the map
        for (FilterMngrContainer::iterator iter = m_component_filters.begin(); 
             iter != m_component_filters.end(); iter++)
        {
            // deactivate the level for the component
            (iter->second)->deactivate_level(Logging::ALL_COMPONENT_NAMES, domain, level);
        }
    }
    // else only one component to deactivate
    else if (component != "")
    {
        // search the specified component (create it if not found)
        FilterMngr* p_filter = search_and_create_comp (component);
        
        // deactivate domain level for the found component
        p_filter->deactivate_level (domain, level);       
    }
}

/**
 * Deactivate the level of a domain
 */
void FilterMngr::deactivate_level (const std::string& domain,
                                   LevelType level)
{
    // first set default level for this component 
    // if all domain are requested or no domain is requested
    if (domain == Logging::ALL_DOMAINS || domain == "")
    {
        deactivate_level (level);
    }
    
    // if all domains associated to component to deactivate
    if (domain == Logging::ALL_DOMAINS)
    { 
        // first lock the mutex of map
        CDMW_MUTEX_GUARD(m_domain_filters_mtx);
    
        // for each domain in the map
        for (FilterContainer::iterator iter = m_domain_filters.begin(); 
             iter != m_domain_filters.end(); iter++)
        {
            // deactivate the level
            (iter->second)->deactivate_level (level);
        }
    }

    // else only one domain to deactivate the level
    else if (domain != "")
    {
        // search the specified domain (create it if not found)
        Filter* p_filter = search_and_create_dom (domain);
    
        // deactivate the level for the found domain
        p_filter->deactivate_level (level);
    }
}

/**
 * Deactivate all levels of a component/domain
 */
void FilterMngr::deactivate_all_levels (const std::string& component,
                                        const std::string& domain)
{  
    // first deactivate level of domains belonging to this component if requested
    if (component == Logging::ALL_COMPONENT_NAMES || component == "")
    { 
        // deactivate the default all level for this component
        deactivate_all_levels();
        
        // deactivate all levels for domain belonging to this component
        deactivate_all_levels (domain);
    }

    // then deactivate all levels for the component(s) associated to this component
    
    // if all components associated to this component to deactivate
    if (component == Logging::ALL_COMPONENT_NAMES)
    {
        // deactivate all levels for all component children
        // first lock the mutex of map
        CDMW_MUTEX_GUARD(m_component_filters_mtx);
    
        // for each component in the map
        for (FilterMngrContainer::iterator iter = m_component_filters.begin(); 
             iter != m_component_filters.end(); iter++)
        {
            // deactivate all levels for the component
            (iter->second)->deactivate_all_levels(Logging::ALL_COMPONENT_NAMES, domain);
        }
    }
    // else only one component to deactivate
    else if (component != "")
    {
        // search the specified component (create it if not found)
        FilterMngr* p_filter = search_and_create_comp (component);
        
        // deactivate all domain levels for the found component
        p_filter->deactivate_all_levels (domain);       
    }
}
                            
/**
 * Deactivate all levels of a domain
 */
void FilterMngr::deactivate_all_levels(const std::string& domain)
{
    // first deactivate all default levels for this component 
    // if all domain are requested or no domain is requested
    if (domain == Logging::ALL_DOMAINS || domain == "")
    {
        deactivate_all_levels();
    }
    
    // if all domains associated to component to deactivate
    if (domain == Logging::ALL_DOMAINS)
    {
        // first lock the mutex of map
        CDMW_MUTEX_GUARD(m_domain_filters_mtx);
    
        // for each domain in the map
        for (FilterContainer::iterator iter = m_domain_filters.begin(); 
             iter != m_domain_filters.end(); iter++)
        {
            // deactivate all levels
            (iter->second)->deactivate_all_levels();
        }
    }

    // else only one domain to deactivate
    else if (domain != "")
    {
        // search the specified domain (create it if not found)
        Filter* p_filter = search_and_create_dom (domain);
    
        // deactivate all levels for the found domain
        p_filter->deactivate_all_levels();
    }
}                            

/**
 * Check if a level for a component/domain is activated 
 * (component/domain are created if not exist)
 */
bool FilterMngr::is_activated (const std::string& component,
                               const std::string& domain,
                               LevelType level)
{
    bool result;
    
    // if component is undefined get the level of the domain
    if (component == "" || component == Logging::ALL_COMPONENT_NAMES)
    {
        // Check if the level for a domain is activated
        // (or the level of this filter if domain is undefined)
        result = is_activated(domain, level);
    }
    
    // else component is defined
    else
    {
        // search / create (if not found) the specified component filter
        FilterMngr* p_filter = search_and_create_comp (component);
        
        // activate the level of domain for this component
        // domain is created if not found
        result = p_filter->is_activated (domain, level);
    }
    
    return result;
}


/**
 * Check if a level for a domain is activated (domain is created if not exist)
 */
bool FilterMngr::is_activated (const std::string& domain,
                               LevelType level)
{
    bool result;
    
    // if domain is undefined get the level of this filter
    if (domain == "" || domain == Logging::ALL_DOMAINS)
    {
        result = is_level_activated (level);
    }
    
    // else search / create (if not found) the specified domain
    // and get its level
    else
    {
        Filter* p_filter = search_and_create_dom (domain);
    
        result = p_filter->is_level_activated (level);
    }
    
    return result;
}

/**
 * Return the reference of the level for a component/domain
 */
Filter* FilterMngr::get_filter_ref (const std::string& component,
                                    const std::string& domain)
{
    Filter* p_filter = NULL;
        
    // if component undefined, get the domain filter for this filter
    if (component == "")
    {
        p_filter = get_filter_ref(domain);
    }
    
    // else component is defined, search it
    else
    {
        // search / create (if not found) the specified component filter
        FilterMngr* p_comp_filter = search_and_create_comp (component);
        
        // get the domain filter for the found component filter
        p_filter = p_comp_filter->get_filter_ref(domain);
    }

    return p_filter;
}

             
/**
 * Return the reference of the domain filter
 */
Filter* FilterMngr::get_filter_ref(const std::string& domain)
{
    Filter* p_filter = NULL;
    
    if (domain == "")
    {
        p_filter = this;
    }
    
    // else find the specified domain or create it
    else
    {
        // search the specified domain (create it if not found)
        p_filter = search_and_create_dom (domain);
    }
        
    return p_filter;
}

/**
 * get the path of base component
 */
unsigned int FilterMngr::get_component_base (
                                const std::string& full_component_path,
                                std::string& base_component,
                                std::string& sub_component_path)
{
    const char *p_char = full_component_path.c_str();
    
    // start position of the base component
    unsigned int start_pos = 0;
    
    // position of separator between base and sub component
    unsigned int end_pos = 0;
    
    // number of sub components
    unsigned int count = 0;
    
    for (unsigned int i=0 ; i < full_component_path.size() ; p_char++, i++)
    {
        // if separator is found
        if (*p_char == COMPONENT_SEP)
        {
            // if separator at beginning of string, increment the start position
            if (i == start_pos)
            {
                start_pos ++;
            }
            
            // else if not yet found, store the position of the separator
            else if (end_pos == 0)
            {
                end_pos = i;
            }
            
            // if already found, simply increment counter
            else
            {
                count++;
            }        
        }
    }
    
    // if no sub component
    if (end_pos == 0)
    {
        base_component = full_component_path.substr (start_pos);
        sub_component_path = "";
    }
    else
    {
        base_component = full_component_path.substr (start_pos, end_pos-start_pos);
        sub_component_path = full_component_path.substr (end_pos+1);
        count ++;
    }
    
    return count;
}



                                
/**
 * Search domain, if not found create it
 */
Filter* FilterMngr::search_and_create_dom (const std::string& domain)
{
    Filter* p_filter = NULL;
    
    // first lock the mutex of map
    CDMW_MUTEX_GUARD(m_domain_filters_mtx);
    
    // search domain name in map
    FilterContainer::iterator iter = m_domain_filters.find(domain);
    
    // if domain found
    if (iter != m_domain_filters.end())
    {
        p_filter = iter->second;
    }
    // if it is not found create it with level of parent filter
    else
    {
        p_filter = new Filter (get_level());
        
        // store in map the new domain filter
        m_domain_filters[domain] = p_filter;
    }
    
    return p_filter;
}

/**
 * Search component, if not found create it
 * not thread safe
 */
FilterMngr* FilterMngr::search_and_create_comp_unguard (
                                   const std::string& component)
{
    FilterMngr* p_filter = NULL;
    
    // search component name in map
    FilterMngrContainer::iterator iter = m_component_filters.find(component);
    
    // if component found
    if (iter != m_component_filters.end())
    {
        p_filter = iter->second;
    }
    // if it is not found create it with level of parent filter
    else
    {
        p_filter = new FilterMngr (get_level());
        
        // store in map the new component filter
        m_component_filters[component] = p_filter;
    }
    
    return p_filter;
}

 

/**
 * Search component
 * if component is composed of sub components, the last sub component filter is returned
 * if component is not found it is created
 * thread safe
 */
FilterMngr* FilterMngr::search_and_create_comp (const std::string& component)
{
    FilterMngr* p_final_filter = NULL;
    
    // first lock the mutex of map of this filter
    CDMW_MUTEX_GUARD(m_component_filters_mtx);
    
    std::string base_component;
    std::string sub_component_path;
        
    // get base component and sub components from component name
    get_component_base (component, base_component, sub_component_path);
        
    // search and create the base component filter
    FilterMngr* p_filter = search_and_create_comp_unguard (base_component);
    
    // if component end has been reached stop the creation
    if (sub_component_path == "")
    {
        // return this filter
        p_final_filter = p_filter;
    }
    else
    {
        // on this base component search and create the sub component
        p_final_filter = 
            p_filter->search_and_create_comp (sub_component_path);
    }

    return p_final_filter;
}


/**
 * Returns the list of Filter for this component and sub components.
 */
void FilterMngr::get_log_filters (FilterDataList& filter_list, const std::string& component)
    throw(OutOfMemoryException) 
{
    try
    {
        // get the default filter
        FilterData filter_data;
        filter_data.m_component_name = component;
        filter_data.m_domain_name = "";
        filter_data.m_levels = m_levels;
        filter_list.push_back(filter_data);
        
        // get the list of the domain belonging to this component    
        {
            // first lock the mutex of map
            CDMW_MUTEX_GUARD(m_domain_filters_mtx);
            
            // get domain from map
            for (FilterContainer::iterator iter = m_domain_filters.begin(); 
                 iter != m_domain_filters.end(); iter++)
            {
                FilterData filter_data;
                filter_data.m_component_name = component;
                filter_data.m_domain_name = iter->first;
                filter_data.m_levels = (iter->second)->get_level();
                filter_list.push_back(filter_data);
            }
        }
    
        // for each sub component get the list of filters
        {
            // first lock the mutex of component map of this filter
            CDMW_MUTEX_GUARD(m_component_filters_mtx);

            // get sub component from map
            for (FilterMngrContainer::iterator iter = m_component_filters.begin(); 
                 iter != m_component_filters.end(); iter++)
            {
                (iter->second)->get_log_filters (filter_list, iter->first); 
            }
        }
    }
    catch (std::bad_alloc&) 
    {
        CDMW_THROW(OutOfMemoryException);
    }
}


} // End namespace Logging

} // End namespace Cdmw
