/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#include <Foundation/logging/LogManager.hpp>
#include <Foundation/common/Options.hpp>
#include <Foundation/osthreads/ReaderLockGuard.hpp>
#include <Foundation/osthreads/WriterLockGuard.hpp>
#include <Foundation/ossupport/OS.hpp>

#include "logging/LogManager.hpp"
#include "logging/LogManagerDefaultDelegate.hpp"
#include "logging/Logger_impl.hpp"

namespace Cdmw
{
    namespace Logging
    {
        // initialize the static variables.
        OsSupport::ReaderWriterLock LogManager::M_logMngr_mtx;
        LogManager* LogManager::M_Instance(0);


        /**
         * Default constructor.
         */
        LogManager::LogManager()
            throw()
                : m_default_delegate(0),
                  m_alternate_delegate(0),
                  m_is_verbose(false),
                  m_fatal_handler(0),
                  m_filter_manager(0),
                  m_application_name(""),
                  m_process_name("")
        {
            m_process_id = Cdmw::OsSupport::OS::get_processId();
            m_hostname = Cdmw::OsSupport::OS::get_hostname();
        }


        /**
         * Constructor.
         *
         * @param argc number of arguments in argv.
         * @param argv list of arguments.
         */
        LogManager::LogManager(int& argc, char** argv)
            throw(OutOfMemoryException,
                  InternalErrorException)
                : m_default_delegate(0),
                  m_alternate_delegate(0),
                  m_is_verbose(false),
                  m_fatal_handler(0),
                  m_application_name("")
        {
            m_process_id = Cdmw::OsSupport::OS::get_processId();
            m_hostname = Cdmw::OsSupport::OS::get_hostname();

            // create the filter manager with default levels
            m_filter_manager = new FilterMngr(DEFAULT_LEVEL);

            if (argv != 0)
            {
                // parse log level option
                //
                try
                {
                    LevelType level = this->read_level(argc, argv);
    
                    // set the new level if parsing succeded
                    m_filter_manager->set_default_level(level);
                }
                catch (const Cdmw::Common::NotFoundException& nfe) {
                    // no level indicated, default is convenient
                }
                catch (const Cdmw::BadParameterException& bde) {
                    std::cerr << "Invalid option will be ignored: " << bde.what();
                }

                // parse log file option
                //
                std::string file_name = "";
                try
                {
                    file_name = this->read_output_file(argc, argv);
                    m_default_delegate = new LogManagerDefaultDelegate(file_name);
                }
                catch (const Cdmw::Common::NotFoundException& nfe)
                {
                    // not activated
                }
                catch (const Cdmw::BadParameterException& bde)
                {
                    std::cerr << "Invalid option will be ignored: " << bde.what();
                }

                // parse verbose mode
                //
                try
                {
                    m_is_verbose = this->read_verbose_mode(argc, argv);
                }
                catch (const Cdmw::BadParameterException& bde)
                {
                   std::cerr << "Invalid option will be ignored: " << bde.what();
                   // let to default
                }
            }

            if (m_default_delegate == 0) {
                m_default_delegate = new LogManagerDefaultDelegate();
            }
        }


        /**
         * Destructor.
         */
        LogManager::~LogManager()
            throw()
        {
            CDMW_WRITER_LOCK_GUARD(M_logMngr_mtx);

            // we are not assuming the ownership of the pointer so
            // we'll just set it to NULL.
            m_alternate_delegate = 0;

            delete m_default_delegate;
            m_default_delegate = 0;
            
            
            delete m_filter_manager;
            m_filter_manager = 0;
            
            // delete registered Loggers
            for (std::map<std::string, Logger*>::iterator it = m_logger_list.begin();
                 it != m_logger_list.end(); 
                 ++it)
            {
                delete (it->second);
            }
            m_logger_list.clear();
        }


        /**
         * Initialise the LogManager.
         */
        LogManager*
        LogManager::Init(int& argc, char** argv)
            throw(OutOfMemoryException,
                  InternalErrorException)
        {
            try 
            {
                CDMW_WRITER_LOCK_GUARD(M_logMngr_mtx);

                if (M_Instance == 0) 
                {
                    static LogManager instance__(argc, argv);
                    M_Instance = &instance__;
                }

                return M_Instance;
            } catch (const std::bad_alloc&) {
                throw OutOfMemoryException();
            }
        }


        /**
         * Send a message to the logger.
         */
        bool
        LogManager::add_log(const CdmwLogging::LogHeader& header, const char* msg, const char* ndc)
            throw(InternalErrorException)
        {
            bool result = true;

            CdmwLogging::LogContext context;
            this->fill_current_context(context, ndc);

            if (m_is_verbose || m_alternate_delegate == 0)
            {
                result &= m_default_delegate->add_log(context, header, msg);
            }
            if (m_alternate_delegate != 0) 
            {
                result &= m_alternate_delegate->add_log(context, header, msg);
            }

            // if it's a FATAL message and a FatalHandler is registered:
            // call it
            if ((header.level == Cdmw::Logging::FATAL) && m_fatal_handler) 
            {
                m_fatal_handler->on_fatal_log(header, msg);
            }

            return result;
        }


        /**
         * Register an alternate delegate.
         */
        LogManagerDelegate*
        LogManager::register_alternate_delegate(LogManagerDelegate* alt)
        {
            CDMW_WRITER_LOCK_GUARD(M_logMngr_mtx);

            LogManagerDelegate* old = m_alternate_delegate;
            m_alternate_delegate = alt;

            return old;
        }


        /**
         * Unregister the alternate delegate.
         */
        bool
        LogManager::unregister_alternate_delegate(LogManagerDelegate* alt)
        {
            CDMW_WRITER_LOCK_GUARD(M_logMngr_mtx);

            bool result = false;

            if (alt == m_alternate_delegate) {
                result = true;
                m_alternate_delegate = 0;
            }

            return result;
        }

        /**
         * Register a FatalHandler.
         */
        FatalHandler*
        LogManager::register_fatal_handler(FatalHandler* handler)
        {
            CDMW_WRITER_LOCK_GUARD(M_logMngr_mtx);

            FatalHandler* old = m_fatal_handler;
            m_fatal_handler = handler;

            return old;
        }


        /**
         * Unregister the alternate delegate.
         */
        bool
        LogManager::unregister_fatal_handler(FatalHandler* handler)
        {
            CDMW_WRITER_LOCK_GUARD(M_logMngr_mtx);

            bool result = false;

            if (handler == m_fatal_handler) {
                result = true;
                m_fatal_handler = 0;
            }

            return result;
        }


        /**
         * Get the alternate delegate.
         *
         * @return the alternate delegate.
         */
        LogManagerDelegate*
        LogManager::get_alternate_delegate() const
        {
            return m_alternate_delegate;
        }
        
        /**
         * Get the filter manager.
         *
         * @return the filter manager.
         */
        FilterMngr* LogManager::get_filter_manager() const
        {
            return m_filter_manager;
        }
        
        /**
         * Search the logger associated to the specified component name
         * and domain name, create and register a new logger if not existing.
         * This method is thread-safe.
         */
        Logger* LogManager::get_logger(
                                 const std::string& component,
                                 const std::string& domain) 
        {
            CDMW_WRITER_LOCK_GUARD(M_logMngr_mtx);

            Logger* logger_ = 0;
            std::string logger_name = component + "/" + domain;
            std::map<std::string, Logger*>::const_iterator it = 
                    m_logger_list.find(logger_name);
            if ( it != m_logger_list.end() ) {
                // Retrieve a registered logger
                logger_ = it->second;
            } else {
                 // register a new logger
                 logger_ = new Logger_impl(this, component, domain);
                 m_logger_list[logger_name]=logger_;
            }

            return logger_;
        }
        
        /**
         * Set the application name.
         */
        void LogManager::set_application_name(const std::string& application_name)
        {
            m_application_name = application_name;
        }

        /**
         * Get the application name.
         */
        std::string LogManager::get_application_name() const
        {
            return m_application_name;
        }

        /**
         * Set the process name.
         */
        void LogManager::set_process_name(const std::string& process_name)
        {
            m_process_name = process_name;
        }

        /**
         * Get the process name.
         */
        std::string LogManager::get_process_name()
        {
            return m_process_name;
        }

        /**
         * Fills the LogContext fields of the calling thread.
         */
        void LogManager::fill_current_context(CdmwLogging::LogContext& context, const char* ndc)
        {
            context.host_name = m_hostname.c_str();
            context.application_name = m_application_name.c_str();
            context.process_name = m_process_name.c_str();
            context.process_id = (int)m_process_id;
            context.ndc = ndc;
        }

        /**
         * Retrieve the log level described in command line arguments.
         */
        LevelType LogManager::read_level(int& argc, char** argv)
            throw (Cdmw::Common::NotFoundException, Cdmw::BadParameterException)
        {
            LevelType level;
            std::string level_option = 
                Cdmw::OsSupport::OS::get_option_value(argc,
                                                      argv,
                                                      Common::Options::LOG_LEVEL_OPTION);
    
             if (level_option == "no")
             {
                 throw Cdmw::Common::NotFoundException();
             }
             else if (level_option == "yes")
             {
                 throw Cdmw::BadParameterException(Common::Options::LOG_LEVEL_OPTION, "");
             }
             else
             {
                 try
                 {
                     level = this->parse_log_level_option(level_option);
                 }
                 catch (const Cdmw::InvalidArgumentException& iae)
                 {
                     throw Cdmw::BadParameterException(Common::Options::LOG_LEVEL_OPTION, level_option);
                 }
             }

             return level;
        }

        /**
         * Retrieve the output log file described in command line arguments.
         */
        std::string LogManager::read_output_file(int& argc, char** argv)
            throw (Cdmw::Common::NotFoundException, Cdmw::BadParameterException)
        {
            std::string file_option = 
                Cdmw::OsSupport::OS::get_option_value(argc, 
                                                      argv,
                                                      Common::Options::LOG_FILE_OPTION);
    
            if (file_option == "no")
            {
                throw Cdmw::Common::NotFoundException();
            }
            else if (file_option == "yes")
            {
                throw Cdmw::BadParameterException(Common::Options::LOG_FILE_OPTION, "");
            }

            return file_option;
        }

        /**
         * Retrieve if the verbose mode is activated in command line arguments.
         */
        bool LogManager::read_verbose_mode(int& argc, char** argv)
            throw (Cdmw::BadParameterException)
        {
            bool verbose;
            std::string level_option = 
                Cdmw::OsSupport::OS::get_option_value(argc,
                                                        argv,
                                                        Common::Options::LOG_VERBOSE_OPTION);
    
             if (level_option == "no")
             {
                 verbose = false;
             }
             else if (level_option == "yes")
             {
                 verbose = true;
             }
             else 
             {
                 throw Cdmw::BadParameterException(Common::Options::LOG_VERBOSE_OPTION, level_option);
             }
    
             return verbose;
        }

        /**
         * Converts a command line option into the corresponding log
         * level.
         */
        LevelType
        LogManager::parse_log_level_option(const std::string& level_option)
            throw (Cdmw::BadParameterException)
        {
           LevelType level = 0;
           std::string strLevel(level_option);
           char op = '+'; // the first level is always added!
            
           const std::string delimiters = " +-";
           std::string::size_type prevPos = 0;
           std::string::size_type lastPos = strLevel.find_first_not_of(delimiters, 0);
           std::string::size_type pos = strLevel.find_first_of(delimiters, lastPos);

           while (std::string::npos != pos || std::string::npos != lastPos)
           {
              std::istringstream istr_tmp(strLevel.substr(lastPos, pos - lastPos));
              std::string str_tmp(strLevel.substr(lastPos, pos - lastPos));

              // Process options in the following order : bin/dec/str
              LevelType new_level = 0;
              if (!str_tmp.compare(0, 2, "0x"))
              {
                 istr_tmp.seekg(2);
                 istr_tmp >> std::hex >> new_level;
              }
              else
              {
                 istr_tmp >> std::dec >> new_level;
              }
              if (istr_tmp.fail())
              {
                 istr_tmp.clear();
                 try 
                 {
                    new_level = this->read_level(istr_tmp);
                 }
                 catch (const Cdmw::InvalidArgumentException &iae) 
                 {
                    std::cerr << "throwing exception because of " << str_tmp << std::endl;
                    throw Cdmw::BadParameterException("LogManager::parse_log_level_option");
                 }
              }

              switch (op)
              {
                 case '+': // A OR B
                    level |= new_level;
                    break;
                 case '-': // (A XOR B) AND (NOT B)
                    level = (level ^ new_level) & (~new_level);
                    break;
                 default:
                    throw
                       Cdmw::BadParameterException(
                          Common::Options::LOG_LEVEL_OPTION,
                          &op
                          );
              }

              lastPos = strLevel.find_first_not_of(delimiters, pos);
              pos = strLevel.find_first_of(delimiters, lastPos);

              if (lastPos && (lastPos != std::string::npos) )
              {
                 std::string str_tmp = strLevel.substr(prevPos, lastPos - prevPos);
                 std::string::size_type op_pos = str_tmp.find_first_of("+-"); // takes care of ' ' between options
                 if (op_pos != std::string::npos )
                    op = str_tmp[op_pos];

                 prevPos = pos;
              }
           } // end while

           return level;
        }


        /**
         * Reads in the stream a log level that is written in a string shape.
         * Supports all log levels, and also: DEFAULT, ALL, NONE
         */
        LevelType
        LogManager::read_level(
            std::istringstream & str_level
            )
            throw (Cdmw::InvalidArgumentException)
        {
            std::string aux = str_level.str().substr(str_level.tellg());
            LevelType level = 0;

            if (! aux.compare(0, sALL.size(), sALL))
            {
                str_level.ignore(sALL.size());
                level = Cdmw::Logging::ALL_LEVELS_SET;
            }
            else if (! aux.compare(0, sDEFAULT.size(), sDEFAULT))
            {
                str_level.ignore(sDEFAULT.size());
                level = Cdmw::Logging::DEFAULT_LEVEL;
            }
            else if (! aux.compare(0, sNONE.size(), sNONE))
            {
                str_level.ignore(sNONE.size());
                level = Cdmw::Logging::NONE_LEVEL;
            }

            else if (! aux.compare(0, sFATAL.size(), sFATAL))
            {
                str_level.ignore(sFATAL.size());
                level = Cdmw::Logging::FATAL;
            }
            else if (! aux.compare(0, sERROR.size(), sERROR))
            {
                str_level.ignore(sERROR.size());
                level = Cdmw::Logging::ERROR;
            }
            else if (! aux.compare(0, sWARN.size(), sWARN))
            {
                str_level.ignore(sWARN.size());
                level = Cdmw::Logging::WARN;
            }
            else if (! aux.compare(0, sINFO.size(), sINFO))
            {
                str_level.ignore(sINFO.size());
                level = Cdmw::Logging::INFO;
            }

            else if (! aux.compare(0, sDEBUG_1.size(), sDEBUG_1))
            {
                str_level.ignore(sDEBUG_1.size());
                level = Cdmw::Logging::DEBUG_1;
            }
            else if (! aux.compare(0, sDEBUG_2.size(), sDEBUG_2))
            {
                str_level.ignore(sDEBUG_2.size());
                level = Cdmw::Logging::DEBUG_2;
            }
            else if (! aux.compare(0, sDEBUG_3.size(), sDEBUG_3))
            {
                str_level.ignore(sDEBUG_3.size());
                level = Cdmw::Logging::DEBUG_3;
            }
            else if (! aux.compare(0, sDEBUG_4.size(), sDEBUG_4))
            {
                str_level.ignore(sDEBUG_4.size());
                level = Cdmw::Logging::DEBUG_4;
            }
            else if (! aux.compare(0, sDEBUG_5.size(), sDEBUG_5))
            {
                str_level.ignore(sDEBUG_5.size());
                level = Cdmw::Logging::DEBUG_5;
            }
            else if (! aux.compare(0, sDEBUG_6.size(), sDEBUG_6))
            {
                str_level.ignore(sDEBUG_6.size());
                level = Cdmw::Logging::DEBUG_6;
            }
            else if (! aux.compare(0, sDEBUG_7.size(), sDEBUG_7))
            {
                str_level.ignore(sDEBUG_7.size());
                level = Cdmw::Logging::DEBUG_7;
            }
            else if (! aux.compare(0, sDEBUG_8.size(), sDEBUG_8))
            {
                str_level.ignore(sDEBUG_8.size());
                level = Cdmw::Logging::DEBUG_8;
            }
            else if (! aux.compare(0, sDEBUG_9.size(), sDEBUG_9))
            {
                str_level.ignore(sDEBUG_9.size());
                level = Cdmw::Logging::DEBUG_9;
            }
            else if (! aux.compare(0, sDEBUG.size(), sDEBUG))
            {
                str_level.ignore(sDEBUG.size());
                level = Cdmw::Logging::DEBUG;
            }

            else if (! aux.compare(0, sCDMW_DEBUG_1.size(), sCDMW_DEBUG_1))
            {
                str_level.ignore(sCDMW_DEBUG_1.size());
                level = Cdmw::Logging::CDMW_DEBUG_1;
            }
            else if (! aux.compare(0, sCDMW_DEBUG_2.size(), sCDMW_DEBUG_2))
            {
                str_level.ignore(sCDMW_DEBUG_2.size());
                level = Cdmw::Logging::CDMW_DEBUG_2;
            }
            else if (! aux.compare(0, sCDMW_DEBUG_3.size(), sCDMW_DEBUG_3))
            {
                str_level.ignore(sCDMW_DEBUG_3.size());
                level = Cdmw::Logging::CDMW_DEBUG_3;
            }
            else if (! aux.compare(0, sCDMW_DEBUG_4.size(), sCDMW_DEBUG_4))
            {
                str_level.ignore(sCDMW_DEBUG_4.size());
                level = Cdmw::Logging::CDMW_DEBUG_4;
            }

            else if (! aux.compare(0, sREQ_FOLLOWUP.size(), sREQ_FOLLOWUP))
            {
                str_level.ignore(sREQ_FOLLOWUP.size());
                level = Cdmw::Logging::REQ_FOLLOWUP;
            }
            else if (! aux.compare(0, sCALL_TRACE.size(), sCALL_TRACE))
            {
                str_level.ignore(sCALL_TRACE.size());
                level = Cdmw::Logging::CALL_TRACE;
            }
            else if (! aux.compare(0, sORB_TRACE.size(), sORB_TRACE))
            {
                str_level.ignore(sORB_TRACE.size());
                level = Cdmw::Logging::ORB_TRACE;
            }

            else
            {
                throw Cdmw::InvalidArgumentException("str_level");
            }
            return level;
        }
        
    } // end namespace Logging
} // end namespace Cdmw
