/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#include <Foundation/logging/LogLevel.hpp>
#include <Foundation/osthreads/MutexGuard.hpp>
#include <Foundation/logging/LogManager.hpp>

#include "logging/LogManagerCollectorDelegate.hpp"

#include "logging/LogManagerDefaultDelegate.hpp"

// Anonymous namespace
namespace 
{
  static Cdmw::Logging::LogManagerDelegate * M_LogManagerDelegate = 0;

  bool open_log_store(const CdmwLogging::LogContext& context,
                      const CdmwLogging::LogHeader& header) throw()
  {
    CDMW_ASSERT(M_LogManagerDelegate==0);
    bool result = false;
    try {
      std::string application_name(context.application_name.in());
      std::string process_name(context.process_name.in());
      std::string component_name(header.component_name.in());
      std::ostringstream ostr;
      if (!application_name.empty())
        ostr << application_name << "-";
      if (!process_name.empty())
        ostr << process_name << "-";
      if (application_name.empty() && process_name.empty() && !component_name.empty())
        ostr << component_name << "-";
      ostr << context.process_id << ".log";

      M_LogManagerDelegate = new Cdmw::Logging::LogManagerDefaultDelegate(ostr.str());
      result = true;
    } catch (...) {
      // ignore
    }
    return result;
  } 
  bool close_log_store() throw()
  {
    delete M_LogManagerDelegate;
    M_LogManagerDelegate = 0;
    return true;
  }
  bool store_log(const CdmwLogging::LogContext& context,
                 const CdmwLogging::LogHeader& header,
                 const char* msg)
    throw(Cdmw::InternalErrorException)
  {
    CDMW_ASSERT(M_LogManagerDelegate!=0);
    return M_LogManagerDelegate->add_log(context,
                                         header,
                                         msg);
  }
  
}; // End of anonymous namespace
namespace Cdmw
{
    namespace Logging
    {
        /**
         * Constructor.
         */
        LogManagerCollectorDelegate::LogManagerCollectorDelegate(CdmwLogging::LogCollector_ptr ref_collector)
            throw(OutOfMemoryException)
        {
           CDMW_ASSERT(!CORBA::is_nil(ref_collector));
           m_ref_collector = CdmwLogging::LogCollector::_duplicate(ref_collector);
        }

        /**
         * Destructor.
         */
        LogManagerCollectorDelegate::~LogManagerCollectorDelegate()
            throw()
        {
          close_log_store();
        }


        /**
         * Send a message to the logger.
         */
        bool
        LogManagerCollectorDelegate::add_log(const CdmwLogging::LogContext& context,
                                             const CdmwLogging::LogHeader& header,
                                             const char* msg)
            throw(InternalErrorException)
        {
            bool result = true;
            try
            {
              // NOTA: Workaround to TAO's dispatch problem!
              // 1) Store only the ORB trace in a local file, any other level is to be 
              // sent to the daemon (PCR-1963)
              // 2) Do not send TAO's logs as well to avoid deadlock in the
              // TAO's Transport_Cache_Manager as it acquire a lock!
              if (!(header.level & Cdmw::Logging::ORB_TRACE)) {
                CdmwLogging::StructuredLog log;
                log.ctx = context;
                log.header = header;
                log.message = msg;
                result = m_ref_collector->add_log(log);
              } else {
                static bool log_store_opened__ = open_log_store(context, header);
                if (log_store_opened__)
                  result = store_log(context,
                                     header,
                                     msg);
                else
                  result = false;
              }

            }
            catch (const ::CORBA::SystemException& se)
            {
                std::cerr << "Cannot log message, unexpected CORBA::SystemException raised: "
                          << se << " (minor = " << se.minor() << ")" << std::endl;
                result = false;
            }
    
            return result;
        }

    } // end namespace Logging
} // end namespace Cdmw
