/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#include <ctime>
#include <ostream>
#include <sstream>
#include <string>

#include <Foundation/logging/LogLevel.hpp>
#include <Foundation/osthreads/MutexGuard.hpp>
#include <Foundation/ossupport/OS.hpp>
#include <Foundation/logging/LogManager.hpp>

#include "logging/LogManagerDefaultDelegate.hpp"


#define BUFFER_MAX_LENGTH 128


namespace Cdmw
{
    namespace Logging
    {
        /**
         * Constructor.
         */
        LogManagerDefaultDelegate::LogManagerDefaultDelegate()
            throw(OutOfMemoryException)
            :
            m_ofstream(),
            m_postream(&std::cout)
        {
        }

        /**
         * Constructor.
         */
        LogManagerDefaultDelegate::LogManagerDefaultDelegate(std::string file_name)
            throw(OutOfMemoryException,
                  BadParameterException)
            :
            m_ofstream(),
            m_postream(&std::cout)
        {
            m_ofstream.open(file_name.c_str(), std::ios::out);

            if (! m_ofstream)
            {
                throw Cdmw::BadParameterException(
                            std::string("Couldn't create output file (")
                            + file_name + ") for logger!");
            }
            else
            {
                m_postream = &m_ofstream;
                std::cout << "CDMW Log file set to " << file_name
                          << std::endl;
            }
        }


        /**
         * Destructor.
         */
        LogManagerDefaultDelegate::~LogManagerDefaultDelegate()
            throw()
        {
            try
            {
                m_postream = &std::cout;
                m_ofstream.close();
            }
            catch (...)
            {
                // Ignore ...
            }
        }

        
        /**
         * timestamp_to_string() utility function
         */
        std::string LogManagerDefaultDelegate::timestamp_to_string(CdmwLogging::Timeval tv, const char* format) const
        {
            // format date
            char time_buf[BUFFER_MAX_LENGTH];
            Cdmw::OsSupport::OS::get_fmt_time (time_buf, sizeof(time_buf),
                                               tv.seconds,
                                               const_cast<char*>(format),
                                               Cdmw::OsSupport::OS::LOCAL_TIME);
            return std::string(time_buf);
        }


        /**
         * Replaces an empty string by "[]" and truncate to BUFFER_MAX_LENGTH if needed
         */
        void LogManagerDefaultDelegate::manage_field(char *buffer, const char *format, const char *value) const
        {
            static const char* EMPTY_FIELD = "[]";
            if (strcmp(value, "") == 0) {
                sprintf (buffer, "%s", EMPTY_FIELD);
            }
            else {
                // The following code will truncate 'value' to BUFFER_MAX_LENGTH, if it is longer than BUFFER_MAX_LENGTH
                char strTmp[BUFFER_MAX_LENGTH+1];
                if (strlen(value) > BUFFER_MAX_LENGTH) {
                    // if we truncate, we replace the last 3 characters by '...'
                    strncpy(strTmp, value, BUFFER_MAX_LENGTH-3);
                    strTmp[BUFFER_MAX_LENGTH-3] = 0;
                    strcat(strTmp, "...");
                }
                else {
                    strncpy(strTmp, value, BUFFER_MAX_LENGTH);
                }
            
                strTmp[BUFFER_MAX_LENGTH] = 0;
                sprintf (buffer, format, strTmp);
            }
        }


        /**
         * Format horizontal log header
         * @param os the stream where to build the log header
         * @param context the log context
         * @param header the log header
         */
        void LogManagerDefaultDelegate::format_log(std::ostringstream & os,
                        const CdmwLogging::LogContext& context,
                        const CdmwLogging::LogHeader& header) const
            throw() {
            char string_buf[BUFFER_MAX_LENGTH+1];

            // UTC Time
            sprintf (string_buf, "%s", timestamp_to_string(header.time_stamping,"%Y-%m-%dT%H:%M:%SZ").c_str());
            os << string_buf << " ";

            sprintf (string_buf, "%012u", header.time_stamping.seconds);
            os << string_buf << ".";
            sprintf (string_buf, "%06u", header.time_stamping.microseconds);
            os << string_buf << " ";

            // Operational Time
            manage_field(string_buf, "%s", "");
            os << string_buf << " ";

            // Hostname
            manage_field(string_buf, "%s", context.host_name.in());
            os << string_buf << " ";

            // Application name
            manage_field(string_buf, "%s", context.application_name.in());
            os << string_buf << " ";

            // Process Name
            manage_field(string_buf, "%s", context.process_name.in());
            os << string_buf << " ";

            // PID
            {
                std::ostringstream stream_buf;
                stream_buf << context.process_id;
                manage_field(string_buf, "%05.5s", stream_buf.str().c_str());
                os << string_buf << " ";
            }

            // Component name
            manage_field(string_buf, "%s", header.component_name.in());
            os << string_buf << " ";

            // User domain
            manage_field(string_buf, "%s", header.user_domain.in());
            os << string_buf << " ";

            // Log level
//             sprintf (string_buf, "%08X", header.level);
//             os << string_buf << " ";
           os <<  LevelUtilities::Level_To_String(header.level) << " ";

            // Thread id
            {
                std::ostringstream stream_buf;

                stream_buf << header.thread_id;
                manage_field(string_buf, "%05.5s", stream_buf.str().c_str());
                os << string_buf << " ";
            }

            // File name
            manage_field(string_buf, "%s", Cdmw::OsSupport::OS::basename(header.file_name.in()).c_str());
            os << string_buf << " ";

            // File line
            sprintf (string_buf, "%-5.5d", header.file_line);
            os << string_buf << " ";

            // NDC
            manage_field(string_buf, "%s", context.ndc);
            os << string_buf;
        }


        /**
         * Send a message to the logger.
         */
        bool
        LogManagerDefaultDelegate::add_log(const CdmwLogging::LogContext& context,
                                           const CdmwLogging::LogHeader& header,
                                           const char* msg)
            throw(InternalErrorException)
        {
            std::ostringstream stream;
            format_log(stream, context, header);
            stream << " ==> " << msg;

            CDMW_MUTEX_GUARD(m_mutex);            
            (*m_postream) << stream.str() << std::endl;
            return true;
        }

    } // end namespace Logging
} // end namespace Cdmw
