/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#include <logging/Logger_impl.hpp>
#include <Foundation/osthreads/Thread.hpp>
#include <Foundation/ossupport/OS.hpp>
#include <Foundation/logging/CdmwLoggingCollector.stub.hpp>


namespace Cdmw
{
    namespace Logging
    {
        /**
         * Constructor.
         */
        Logger_impl::Logger_impl(LogManager* logMngr,
                                 const std::string& component_name,
                                 const std::string& domain)
                        : m_logMngr(logMngr),
                          m_component_name(component_name),
                          m_domain(domain)
        {
            // get the reference of the filter level
            FilterMngr* filter_mngr = logMngr->get_filter_manager();
            m_filter = filter_mngr->get_filter_ref(component_name, domain);
        }
    
        /**
         * Destructor.
         */
        Logger_impl::~Logger_impl()
          throw()
        {
        }
            
        /**
         * Log a generic message.
         */
        void
        Logger_impl::log(LevelType level,
                         const char* message,
                         const char* ndc,
                         const char* file,
                         long line)
        {
            CdmwLogging::LogHeader header;

            header.user_domain = this->m_domain.c_str();
            header.component_name = this->m_component_name.c_str();
            header.level = level;
            header.thread_id = OsSupport::Thread::self();
            header.file_line = line;
            header.file_name = file;
            OsSupport::OS::Timeval time = OsSupport::OS::get_time(); 
            header.time_stamping.seconds = time.seconds;
            header.time_stamping.microseconds = time.microseconds;

            m_logMngr->add_log(header, message, ndc);
        }


        /**
         * Log a debug message.
         */
        void
        Logger_impl::debug(const char* message,
                           const char* file,
                           long line)
        {
            log(::Cdmw::Logging::DEBUG, message, "", file, line);
        }


        /**
         * Log an info message.
         */
        void
        Logger_impl::info(const char* message,
                          const char* file,
                          long line)
        {
            log(::Cdmw::Logging::INFO, message, "", file, line);
        }


        /**
         * Log a warning message.
         */
        void
        Logger_impl::warn(const char* message,
                          const char* file,
                          long line)
        {
            log(::Cdmw::Logging::WARN, message, "", file, line);
        }


        /**
         * Log an error message.
         */
        void
        Logger_impl::error(const char* message,
                           const char* file,
                           long line)
        {
            log(::Cdmw::Logging::ERROR, message, "", file, line);
        }


        /**
         * Log a fatal error message.
         */
        void
        Logger_impl::fatal(const char* message,
                           const char* file,
                           long line)
        {
            log(::Cdmw::Logging::FATAL, message, "", file, line);
        }


        /**
         * Check whether a log level is enabled.
         */
        bool
        Logger_impl::is_enabled_for(LevelType level)
        {
            return m_filter->is_level_activated(level);
        }


        /**
         * Check whether the DEBUG log level is enabled.
         */
        bool
        Logger_impl::is_debug_enabled()
        {
            return m_filter->is_level_activated(Cdmw::Logging::DEBUG);
        }


        /**
         * Check whether the INFO log level is enabled.
         */
        bool
        Logger_impl::is_info_enabled()
        {
            return m_filter->is_level_activated(Cdmw::Logging::INFO);
        }


        /**
         * Check whether the WARN log level is enabled.
         */
        bool
        Logger_impl::is_warn_enabled()
        {
            return m_filter->is_level_activated(Cdmw::Logging::WARN);
        }


        /**
         * Check whether the ERROR log level is enabled.
         */
        bool
        Logger_impl::is_error_enabled()
        {
            return m_filter->is_level_activated(Cdmw::Logging::ERROR);
        }

        /**
         * Check whether the FATAL log level is enabled.
         */
        bool
        Logger_impl::is_fatal_enabled()
        {
            return m_filter->is_level_activated(Cdmw::Logging::FATAL);
        }
        
        
    } // End namespace Logging
} // End namespace Cdmw
