/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

package cdmw.logging;
import java.util.*;
            
public class FilterMngr extends Filter
{
    private Map<String,FilterMngr> m_componentFilters = new HashMap<String,FilterMngr>();
    private Map<String,Filter> m_domainFilters = new HashMap<String,Filter>();
    private String m_baseComponent = "";
    private String m_subComponentPath = "";

    /**
     * Purpose:
     * <p>  Create a new Filter Manager with a specified level mask.
     *
     * @param levels the specified level mask
     */

    public FilterMngr(long levels) {
        super(levels);
        m_domainFilter = true;
        setComponentFilter();
    }
   
     /**
      * Purpose:
      * <p>   Activate the level of a component/domain
      *
      * The component filter is created if not found
      *
      * @param component the name of the component
      * @param domain the name of the domain
      * @param level the level to activate
      *
      */

    public synchronized void activateLevel(String component, String domain, long level) {
        if (component.equals(Logging.ALL_COMPONENT_NAMES) || component.length () == 0)
        {
            activateLevel(domain, level);
        }
 
        // if all components associated to this component to activate
        if (component.equals(Logging.ALL_COMPONENT_NAMES))
        {
            // activate the level for all component children
            // first lock the mutex of map
 
            Iterator hashMapIter = m_componentFilters.entrySet().iterator();
            // for each component in the map
            while(hashMapIter.hasNext())
            {
                Map.Entry  mapEntry = (Map.Entry) hashMapIter.next();
                FilterMngr second = (FilterMngr) mapEntry.getValue();
                // activate the level for the component
                second.activateLevel(Logging.ALL_COMPONENT_NAMES, domain, level);
            }
        } 
        // else only one component to activate
        else if (component.length () > 0)
        {
            // search the specified component (create it if not found)
            FilterMngr p_filter = searchAndCreateComp (component);

            // activate domain level for the found component
            p_filter.activateLevel (domain, level);
        }

    }

    /**
     * Purpose:
     * <p>   Activate the level of a domain
     *
     * The domain filter is created if not found
     *
     * @param domain the name of the domain
     * @param level the level to activate
     *
     */

    public synchronized void activateLevel (String domain, long level) {
         if (domain.equals(Logging.ALL_DOMAINS) || domain.length () == 0)
         {
             this.activateLevel (level);
         }

         if (domain.equals(Logging.ALL_DOMAINS))
         {
             Iterator hashMapIter = m_domainFilters.entrySet().iterator();
             // for each component in the map
             while (hashMapIter.hasNext())
             {
                 Map.Entry  mapEntry = (Map.Entry) hashMapIter.next();
                 Filter second = (Filter) mapEntry.getValue();
                 // activate the level for the component
                 second.activateLevel(level);
             }
         }

         // else only one domain to activate the level
         else if (domain.length () > 0)
         {
             // search the specified domain (create it if not found)
             Filter p_filter = searchAndCreateDom (domain);
             // activate the level for the found domain
             p_filter.activateLevel (level);
         }
    }

    /**
     * Purpose:
     * <p>   Activate all levels of a component/domain
     *
     * The component filter is created if not found
     *
     * @param component the name of the component
     * @param domain the name of the domain
     */

    public synchronized void activateAllLevels (String component, String domain) {
        // first activate level of domains belonging to this component if requested
        if (component.equals(Logging.ALL_COMPONENT_NAMES) || component.length () == 0)
        {
            // activate the default all level for this component
            activateAllLevels();
 
            // activate all levels for the domain belonging to this component
            activateAllLevels (domain);
        }
 
        // then activate all levels for the component(s) associated to this component
 
        // if all components associated to this component to activate
        if (component.equals(Logging.ALL_COMPONENT_NAMES))
        {
             // activate all levels for all component children
 
             // for each component in the map
             // activate all levels for the component
             Iterator hashMapIter = m_componentFilters.entrySet().iterator();
             // for each component in the map
             while (hashMapIter.hasNext())
             {
                 Map.Entry  mapEntry = (Map.Entry) hashMapIter.next();
                 FilterMngr second = (FilterMngr) mapEntry.getValue();
                 // activate the level for the component
                 second.activateAllLevels(Logging.ALL_COMPONENT_NAMES,domain);
             }
 
        }
        // else only one component to activate
        else if (component.length () > 0)
        {
            // search the specified component (create it if not found)
            FilterMngr p_filter = searchAndCreateComp (component);

            // activate all domain levels for the found component
            p_filter.activateAllLevels (domain);
        }
    }

    /**
     * Purpose:
     * <p>   Activate all levels of a domain
     *
     * The domain filter is created if not found
     *
     * @param domain the name of the domain
     */

    public synchronized void activateAllLevels(String domain) {
        // first activate all default levels for this component 
        // if all domain are requested or no domain is requested
        if (domain.equals(Logging.ALL_DOMAINS) || domain.length () == 0)
        {
           activateAllLevels();
        }
 
        // if all domains associated to component to activate
        if (domain.equals(Logging.ALL_DOMAINS))
        {
           // first lock the mutex of map
 
           // for each domain in the map
           Iterator hashMapIter = m_domainFilters.entrySet().iterator();
           while ( hashMapIter.hasNext())
           {
                 Map.Entry  element = (Map.Entry) hashMapIter.next();
                 Filter second = (Filter) element.getValue();
                 // activate the level for the component
                 second.activateAllLevels();
           }
                     // activate all levels
        } 
 
        // else only one domain to activate
        else if (domain.length () > 0)
        {
           // search the specified domain (create it if not found)
           Filter p_filter = searchAndCreateDom (domain);

           // activate all levels for the found domain
           p_filter.activateAllLevels();
        }
    }


    /**
     * Purpose:
     * <p>   Deactivate the level of a component/domain
     *
     * The component filter is created if not found
     *
     * @param component the name of the component
     * @param domain the name of the domain
     * @param level the level to deactivate
     */

    public synchronized void deactivateLevel(String component, String domain, long level) {
        if (component.equals(Logging.ALL_COMPONENT_NAMES) || component.length () == 0)
        {
            deactivateLevel(domain, level);
        }
 
        // if all components associated to this component to activate
        if (component.equals(Logging.ALL_COMPONENT_NAMES))
        {
            // activate the level for all component children
            // first lock the mutex of map
 
            Iterator hashMapIter = m_componentFilters.entrySet().iterator();
            // for each component in the map
            while ( hashMapIter.hasNext())
            {
                Map.Entry  element = (Map.Entry) hashMapIter.next();
                FilterMngr second = (FilterMngr) element.getValue();
                // activate the level for the component
                second.deactivateLevel(Logging.ALL_COMPONENT_NAMES, domain, level);
            }
        } 
        // else only one component to activate
        else if (component.length () > 0)
        {
            // search the specified component (create it if not found)
            FilterMngr p_filter = searchAndCreateComp (component);
            
            // activate domain level for the found component
            p_filter.deactivateLevel (domain, level);
        }

    }

    /**
     * Purpose:
     * <p>   Deactivate the level of a domain
     *
     * The domain filter is created if not found
     *
     * @param domain the name of the domain
     * @param level the level to deactivate
     */

    public synchronized void deactivateLevel (String domain, long level) {
         // if all domains associated to component to deactivate
         if (domain.equals(Logging.ALL_DOMAINS) || domain.length () == 0)
         {
             this.deactivateLevel (level);
         }

         if (domain.equals(Logging.ALL_DOMAINS))
         {
             Iterator hashMapIter = m_domainFilters.entrySet().iterator();
             // for each component in the map
             while ( hashMapIter.hasNext())
             {
                 Map.Entry  element = (Map.Entry) hashMapIter.next();
                 Object key = element.getKey();
                 Filter second = (Filter) element.getValue();
                 // deactivate the level for the component
                 second.deactivateLevel(level);
             } 
         // first deactivate default level for this component 
         // if all domain are requested or no domain is requested
         }

         // else only one domain to activate the level
         else if (domain.length () > 0)
         {
             // search the specified domain (create it if not found)
             Filter p_filter = searchAndCreateDom (domain);

             // deactivate the level for the found domain
             p_filter.deactivateLevel (level);
         }
    }

    /**
     * Purpose:
     * <p>   Deactivate all levels of a component/domain
     *
     * The component filter is created if not found
     *
     * @param component the name of the component
     * @param domain the name of the domain
     */

    public synchronized void deactivateAllLevels (String component, String domain) {
        // first deactivate level of domains belonging to this component if requested
        if (component.equals(Logging.ALL_COMPONENT_NAMES) || component.length () == 0)
        {
            // deactivate the default all level for this component
            deactivateAllLevels();
 
            // deactivate all levels for the domain belonging to this component
            deactivateAllLevels (domain);
        }
 
        // then deactivate all levels for the component(s) associated to this component
 
        // if all components associated to this component to deactivate
        if (component.equals(Logging.ALL_COMPONENT_NAMES))
        {
         // deactivate all levels for all component children
 
             // for each component in the map
             // deactivate all levels for the component
             Iterator hashMapIter = m_componentFilters.entrySet().iterator();
             // for each component in the map
             while ( hashMapIter.hasNext())
             {
                 Map.Entry  element = (Map.Entry) hashMapIter.next();
                 FilterMngr second = (FilterMngr) element.getValue();
                 // activate the level for the component
                 second.deactivateAllLevels(Logging.ALL_COMPONENT_NAMES,domain);
             }
  
        }
        // else only one component to activate
        else if (component.length () > 0)
        {
            // search the specified component (create it if not found)
            FilterMngr p_filter = searchAndCreateComp (component);
 
            // activate all domain levels for the found component
            p_filter.deactivateAllLevels (domain);
        }
    } 

    /**
     * Purpose:
     * <p>   Deactivate all levels of a domain
     *
     * The domain filter is created if not found
     *
     * @param domain the name of the domain
     */

    public synchronized void deactivateAllLevels(String domain) {
       // first deactivate all default levels for this component 
       // if all domain are requested or no domain is requested
        if (domain.equals(Logging.ALL_DOMAINS) || domain.length () == 0)
        {
           deactivateAllLevels();
        }

        // if all domains associated to component to activate
        if (domain.equals(Logging.ALL_DOMAINS))
        {
           // for each domain in the map
           Iterator hashMapIter = m_domainFilters.entrySet().iterator();
           // for each component in the map
           while ( hashMapIter.hasNext())
           { 
               Map.Entry  element = (Map.Entry) hashMapIter.next();
               Filter second = (Filter) element.getValue();
               // activate the level for the component
               second.deactivateAllLevels();
           } 
                     // activate all levels
        }

        // else only one domain to activate
        else if (domain.length () > 0)
        {
            // search the specified domain (create it if not found)
            Filter p_filter = searchAndCreateDom (domain);

            // activate all levels for the found domain
            p_filter.deactivateAllLevels();
        }
    }

    /**
     * Purpose:
     * <p>   Check if a level for a component/domain is activated.
     *
     * The component filter is created if not found
     *
     * @param component the name of the component
     * @param domain the name of the domain
     * @param level the level to check
     *
     * @return true or false
     *
     */

    public boolean isActivated (String component, String domain, long level) {
        boolean result;

        // if component is undefined get the level of the domain
        if (component.length () == 0 || component.equals(Logging.ALL_COMPONENT_NAMES))
        {
          // Check if the level for a domain is activated
          // (or the level of this filter if domain is undefined)
          result = isActivated(domain, level);
        }

        // else component is defined
        else
        {
           // search / create (if not found) the specified component filter
           FilterMngr p_filter = searchAndCreateComp (component);

           // activate the level of domain for this component
           // domain is created if not found
           result = p_filter.isActivated (domain, level);
        }

        return result;
    }

    /**
     * Purpose:
     * <p>   Check if a level for a domain is activated.
     *
     * The domain filter is created if not found
     *
     * @param domain the name of the domain
     * @param level the level to check
     *
     * @return true or false
     *
     */

    public boolean isActivated (String domain, long level) {
        boolean result;

        // if domain is undefined get the level of this filter
        if (domain.length () == 0 || domain.equals(Logging.ALL_DOMAINS))
        {
            result = isLevelActivated (level);
        }

        // else search / create (if not found) the specified domain
        // and get its level
        else
        {
            Filter p_filter = searchAndCreateDom (domain);

            result = p_filter.isLevelActivated (level);
        }

        return result;
    }

    /**
     * Purpose:
     * <p>   Return the reference of the filter for a component/domain
     *
     * The component filter is created if not found
     *
     * @param component the name of the component
     * @param domain the name of the domain
     *
     * @return the filter reference
     *
     */

    public Filter getFilterRef (String component, String domain) {
        Filter p_filter;

        // if component undefined, get the domain filter for this filter
        if (component.length () == 0)
        {
            p_filter = getFilterRef(domain);
        }

        // else component is defined, search it
        else
        {
            // search / create (if not found) the specified component filter
            FilterMngr p_compFilter = searchAndCreateComp (component);

            // get the domain filter for the found component filter
            p_filter = p_compFilter.getFilterRef(domain);
        }

        return p_filter;
    }
 
    /**
     * Purpose:
     * <p>   Return the reference of the filter for a domain
     *
     * The domain filter is created if not found
     *
     * @param domain the name of the domain
     *
     * @return the filter reference
     */

    public Filter getFilterRef(String domain) {
        Filter p_filter;

        if (domain.length () == 0)
        {
            p_filter = this;
        }

        // else find the specified domain or create it
        else
        {
            // search the specified domain (create it if not found)
            p_filter = searchAndCreateDom (domain);
        }
     
        return p_filter;
    }


    /**
     * Purpose:
     * <p>   get the path of base component
     *
     * @param full_component_path the component full path ("comp1.comp2.comp3")
     * @param base_component the returned base component ("comp1")
     * @param sub_component_path the returned sub component path ("comp2.comp3")
     *
     * @return the number of remaining sub components
     *
     */

    public long getComponentBase (String fullComponentPath, String baseComponent, String subComponentPath) {
        String p_char = fullComponentPath;

        // start position of the base component
        int startPos = 0;

        // position of separator between base and sub component
        int endPos = 0;

        // number of sub components
        int count = 0;

        for (int i=0; i < fullComponentPath.length(); i++)
        {
            // if separator is found
            if (p_char.charAt(i) == Logging.COMPONENT_SEP)
            {
                // if separator at beginning of string, increment the start position
                if (i == startPos)
                {
                    startPos ++;
                }

                // else if not yet found, store the position of the separator
                else if (endPos == 0)
                {
                    endPos = i;
                }

                // if already found, simply increment counter
               else
               {
                   count++;
               }
            }
        }

        // if no sub component
        if (endPos == 0)
        {
            m_baseComponent = fullComponentPath.substring (startPos);
            m_subComponentPath = "";
        }
        else
        {
            m_baseComponent = fullComponentPath.substring (startPos, endPos-startPos);
            m_subComponentPath = fullComponentPath.substring (endPos+1);
            count ++;
        }
    
        return count;
    }

    /**
     * Purpose:
     * <p>   Search domain in the component, if not found create it
     *
     * @param domain the name of the domain
     *
     * @return the filter
     *
     */

    public synchronized Filter searchAndCreateDom (String domain) {
        Filter p_filter;

        // search domain name in map
        
        if(m_domainFilters.containsKey(domain))
        {
            p_filter = m_domainFilters.get(domain);
        }       
        else
        {
            p_filter = new Filter (getLevel());

            // store in map the new domain filter
            m_domainFilters.put(domain,p_filter);
    
        }

        return p_filter;
    }

    /**
     * Purpose:
     * <p>   Search component, if not found create it (not thread safe)
     *
     * @param component the name of the component
     *
     * @return the filter
     *
     */

    private FilterMngr searchAndCreateCompUnguard (String component) {

        FilterMngr p_filter;

        // search domain name in map

        if(m_componentFilters.containsKey(component))
        {
            p_filter = (FilterMngr)m_componentFilters.get(component);
        } 
        else
        {
            p_filter = new FilterMngr (getLevel().getType());

            // store in map the new domain filter
            m_componentFilters.put(component,p_filter);
        }

        return p_filter;
    }

    /**
     * Purpose:
     * <p>   Search component and sub components
     * if component is not found it is created
     * thread safe
     *
     * @param component the full name of the component
     *
     * @return the filter
     *
     */

    public synchronized FilterMngr searchAndCreateComp (String component) {
        FilterMngr p_finalFilter;

        String baseComponent = "";
        String subComponentPath = "";

        // get base component and sub components from component name
        getComponentBase (component, baseComponent, subComponentPath);

        // search and create the base component filter
        FilterMngr p_filter = searchAndCreateCompUnguard (m_baseComponent);

        // if component end has been reached stop the creation
        if (m_subComponentPath.length () == 0)
        {
            // return this filter
            p_finalFilter = p_filter;
        }
        else
        {
            // on this base component search and create the sub component
            p_finalFilter = p_filter.searchAndCreateComp (m_subComponentPath);
        }

        return p_finalFilter;
    }

    /**
     * Purpose:
     * <p>   Returns the list of Filter for this component and sub components.
     *
     * @param list of filters
     * @param component the name of component to get the filter
     *        default is the "root" component
     *
     * @exception OutOfMemoryException
     */

    public synchronized void getLogFilters (FilterDataList filterList, String component) {
        try
        {
            // get the default filter
            FilterData filterData = new FilterData();
            filterData.m_componentName = component;
            filterData.m_domainName = "";
            filterData.m_levels = m_levels;
            filterList.list.addLast(filterData);

            // get the list of the domain belonging to this component    
            {
                // for each domain in the map
                Iterator hashMapIter = m_domainFilters.entrySet().iterator();
                // for each component in the map
                while ( hashMapIter.hasNext())
                {
                    Map.Entry  element = (Map.Entry) hashMapIter.next();
                    String first = (String) element.getKey();
                    Filter second = (Filter) element.getValue();
                    FilterData filterData2 = new FilterData();
                    filterData2.m_componentName = component;
                    filterData2.m_domainName = first;
                    filterData2.m_levels = second.getLevel();
                    filterList.list.addLast(filterData2);
                }

            }

            // for each sub component get the list of filters
            {
                // for each domain in the map:wq
                Iterator hashMapIter = m_componentFilters.entrySet().iterator();
                // get sub component from map
                while ( hashMapIter.hasNext())
                {
                    Map.Entry  element = (Map.Entry) hashMapIter.next();
                    String first = (String) element.getKey();
                    FilterMngr second = (FilterMngr) element.getValue();
                    second.getLogFilters (filterList, first);
                }
            }
        }
        catch (OutOfMemoryError bounded)
        {
        }
    }
}
