/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */
package cdmw.logging;

import cdmw.common.Assert;
import static cdmw.logging.Logger.*;

public class FunctionLogger {
    
    private final static String ENTERING="Entering ";
    private final static String EXITING="Exiting ";
    private final static String RETURN_MSG_START="Returning <";
    private final static String RETURN_MSG_END=">";
    private final static String EXCEPTION_MSG_START="Raising exception <";
    private final static String EXCEPTION_MSG_END=">";
    
    private static final LevelType LEVEL = new LevelType(LogLevel.CALL_TRACE);
    
    
    
    /**
     * Allows to log a message at the beginning of a function.
     * Expected to be called just when entering the method.
     *
     * @param logger the recipient logger
     * @param args message describing the function input arguments
     */
    public static void CDMW_LOG_FUNCTION(Logger logger,
                                         String args) 
    {
        if (logger.isEnabledFor(LEVEL)) {                                            
            // get calling stack trace element
            StackTraceElement stack = 
                StackTraceAccessor.getCallingStackElement();
            
            // get filename and line number
            String filename = "no_debug_info";
            int line = 0;
            if (stack.getFileName() != null) {
                filename = stack.getFileName();
                line = stack.getLineNumber();
            }
                
            logger.log(LEVEL, 
                       ENTERING + stack.getMethodName() + "(" + args + ") ...",
                       filename, line);  
        } 
    }

    /**
     * Allows to log a message at the beginning of a function.
     * Expected to be called just when entering the method.
     *
     * @param logger the recipient logger
     * @param args message describing the function input arguments
     * @param depth the index of the stack element to be logged.
     */
    public static void CDMW_LOG_FUNCTION(Logger logger,
                                         String args, int depth) 
    {
        if (logger.isEnabledFor(LEVEL)) {                                            
            // get calling stack trace element
            StackTraceElement stack = 
                StackTraceAccessor.getCallingStackElement(depth);
            
            // get filename and line number
            String filename = "no_debug_info";
            int line = 0;
            if (stack.getFileName() != null) {
                filename = stack.getFileName();
                line = stack.getLineNumber();
            }
                
            logger.log(LEVEL, 
                       ENTERING + stack.getMethodName() + "(" + args + ") ...",
                       filename, line);  
        } 
    }

    /**
     * Allows to log a message when an exception is raised.
     * Expected to be called just before throwing the exception.
     *
     * @param logger the recipient logger
     * @param exception the exception that is about to be raised
     */
    public static void CDMW_LOG_FUNCTION_EXCEPTION(Logger logger, 
                                                   Throwable exception) 
    {
        if (logger.isEnabledFor(LEVEL)) {                                            
            // get calling stack trace element
            StackTraceElement stack = 
                StackTraceAccessor.getCallingStackElement();
            
            // get filename and line number
            String filename = "no_debug_info";
            int line = 0;
            if (stack.getFileName() != null) {
                filename = stack.getFileName();
                line = stack.getLineNumber();
            }
                
            logger.log(LEVEL, 
                       EXITING + stack.getMethodName() + "() ... [" + 
                       EXCEPTION_MSG_START  + exception + EXCEPTION_MSG_END + "]", 
                       filename, line);  
        } 
    }

    /**
     * Allows to log a message when an exception is raised.
     * Expected to be called just before throwing the exception.
     *
     * @param logger the recipient logger
     * @param exception the exception that is about to be raised
     * @param depth the index of the stack element to be logged.     
     */
    public static void CDMW_LOG_FUNCTION_EXCEPTION(Logger logger, 
                                                   Throwable exception,
                                                   int depth) 
    {
        if (logger.isEnabledFor(LEVEL)) {                                            
            // get calling stack trace element
            StackTraceElement stack = 
                StackTraceAccessor.getCallingStackElement(depth);
            
            // get filename and line number
            String filename = "no_debug_info";
            int line = 0;
            if (stack.getFileName() != null) {
                filename = stack.getFileName();
                line = stack.getLineNumber();
            }
                
            logger.log(LEVEL, 
                       EXITING + stack.getMethodName() + "() ... [" + 
                       EXCEPTION_MSG_START  + exception + EXCEPTION_MSG_END + "]", 
                       filename, line);  
        } 
    }
        
    /**
     * Allows to log a message before the return instruction.
     * Expected to be called just before calling return.
     *
     * @param logger the recipient logger
     */
    public static void CDMW_LOG_FUNCTION_RETURN(Logger logger) 
    {
        if (logger.isEnabledFor(LEVEL)) {      
            // get calling stack trace element
            StackTraceElement stack = 
                StackTraceAccessor.getCallingStackElement();
            
            // get filename and line number
            String filename = "no_debug_info";
            int line = 0;
            if (stack.getFileName() != null) {
                filename = stack.getFileName();
                line = stack.getLineNumber();
            }
                
            logger.log(LEVEL, 
                       EXITING + stack.getMethodName() + "() ...", 
                       filename, line);  
        } 
    }

    /**
     * Allows to log a message before the return instruction.
     * Expected to be called just before calling return.
     *
     * @param logger the recipient logger
     * @param depth the index of the stack element to be logged.     
     */
    public static void CDMW_LOG_FUNCTION_RETURN(Logger logger, int depth) 
    {
        if (logger.isEnabledFor(LEVEL)) {      
            // get calling stack trace element
            StackTraceElement stack = 
                StackTraceAccessor.getCallingStackElement(depth);
            
            // get filename and line number
            String filename = "no_debug_info";
            int line = 0;
            if (stack.getFileName() != null) {
                filename = stack.getFileName();
                line = stack.getLineNumber();
            }
                
            logger.log(LEVEL, 
                       EXITING + stack.getMethodName() + "() ...", 
                       filename, line);  
        } 
    }

    /**
     * Allows to log a message before the return instruction.
     * Expected to be called just before calling return.
     *
     * @param logger the recipient logger
     * @param result the result of the function
     */
    public static void CDMW_LOG_FUNCTION_RETURN(Logger logger, 
                                                Object result) 
    {
        if (logger.isEnabledFor(LEVEL)) {      
            // get calling stack trace element
            StackTraceElement stack = 
                StackTraceAccessor.getCallingStackElement();
            
            // get filename and line number
            String filename = "no_debug_info";
            int line = 0;
            if (stack.getFileName() != null) {
                filename = stack.getFileName();
                line = stack.getLineNumber();
            }
                
            logger.log(LEVEL, 
                       EXITING + stack.getMethodName() + "() ... [" + 
                       RETURN_MSG_START  + result + RETURN_MSG_END + "]", 
                       filename, line);  
        } 
    }

    /**
     * Allows to log a message before the return instruction.
     * Expected to be called just before calling return.
     *
     * @param logger the recipient logger
     * @param result the result of the function
     * @param depth the index of the stack element to be logged.     
     */
    public static void CDMW_LOG_FUNCTION_RETURN(Logger logger, 
                                                Object result, int depth) 
    {
        if (logger.isEnabledFor(LEVEL)) {      
            // get calling stack trace element
            StackTraceElement stack = 
                StackTraceAccessor.getCallingStackElement(depth);
            
            // get filename and line number
            String filename = "no_debug_info";
            int line = 0;
            if (stack.getFileName() != null) {
                filename = stack.getFileName();
                line = stack.getLineNumber();
            }
                
            logger.log(LEVEL, 
                       EXITING + stack.getMethodName() + "() ... [" + 
                       RETURN_MSG_START  + result + RETURN_MSG_END + "]", 
                       filename, line);  
        } 
    }
    
    //
    // Same macros but using com.thalesgroup.CdmwLogging.Logger IDL
    //

    /**
     * Allows to log a message at the beginning of a function.
     * Expected to be called just when entering the method.
     *
     * @param logger the recipient logger
     * @param args message describing the function input arguments
     */
    public static void CDMW_LOG_FUNCTION(com.thalesgroup.CdmwLogging.Logger logger, 
                                         String args) 
    {
        if (logger.is_enabled_for((int) LEVEL.getType())) {                                            
            // get calling stack trace element
            StackTraceElement stack = 
                StackTraceAccessor.getCallingStackElement();
            
            // get filename and line number
            String filename = "no_debug_info";
            int line = 0;
            if (stack.getFileName() != null) {
                filename = stack.getFileName();
                line = stack.getLineNumber();
            }
                
            logger.log((int) LEVEL.getType(), 
                       ENTERING + stack.getMethodName() + "(" + args + ") ...",
                       filename, line);  
        } 
    }

    /**
     * Allows to log a message at the beginning of a function.
     * Expected to be called just when entering the method.
     *
     * @param logger the recipient logger
     * @param args message describing the function input arguments
     * @param depth the index of the stack element to be logged.     
     */
    public static void CDMW_LOG_FUNCTION(com.thalesgroup.CdmwLogging.Logger logger, 
                                         String args, int depth) 
    {
        if (logger.is_enabled_for((int) LEVEL.getType())) {                                            
            // get calling stack trace element
            StackTraceElement stack = 
                StackTraceAccessor.getCallingStackElement(depth);
            
            // get filename and line number
            String filename = "no_debug_info";
            int line = 0;
            if (stack.getFileName() != null) {
                filename = stack.getFileName();
                line = stack.getLineNumber();
            }
                
            logger.log((int) LEVEL.getType(), 
                       ENTERING + stack.getMethodName() + "(" + args + ") ...",
                       filename, line);  
        } 
    }
    
    /**
     * Allows to log a message when an exception is raised.
     * Expected to be called just before throwing the exception.
     *
     * @param logger the recipient logger
     * @param exception the exception that is about to be raised
     */
    public static void CDMW_LOG_FUNCTION_EXCEPTION(com.thalesgroup.CdmwLogging.Logger logger, 
                                                   Throwable exception) 
    {
        if (logger.is_enabled_for((int) LEVEL.getType())) {                                            
            // get calling stack trace element
            StackTraceElement stack = 
                StackTraceAccessor.getCallingStackElement();
            
            // get filename and line number
            String filename = "no_debug_info";
            int line = 0;
            if (stack.getFileName() != null) {
                filename = stack.getFileName();
                line = stack.getLineNumber();
            }
                
            logger.log((int) LEVEL.getType(), 
                       EXITING + stack.getMethodName() + "() ... [" + 
                       EXCEPTION_MSG_START  + exception + EXCEPTION_MSG_END + "]", 
                       filename, line);  
        } 
    }

    /**
     * Allows to log a message when an exception is raised.
     * Expected to be called just before throwing the exception.
     *
     * @param logger the recipient logger
     * @param exception the exception that is about to be raised
     * @param depth the index of the stack element to be logged.     
     */
    public static void CDMW_LOG_FUNCTION_EXCEPTION(com.thalesgroup.CdmwLogging.Logger logger, 
                                                   Throwable exception, int depth) 
    {
        if (logger.is_enabled_for((int) LEVEL.getType())) {                                            
            // get calling stack trace element
            StackTraceElement stack = 
                StackTraceAccessor.getCallingStackElement(depth);
            
            // get filename and line number
            String filename = "no_debug_info";
            int line = 0;
            if (stack.getFileName() != null) {
                filename = stack.getFileName();
                line = stack.getLineNumber();
            }
                
            logger.log((int) LEVEL.getType(), 
                       EXITING + stack.getMethodName() + "() ... [" + 
                       EXCEPTION_MSG_START  + exception + EXCEPTION_MSG_END + "]", 
                       filename, line);  
        } 
    }
    
    /**
     * Allows to log a message before the return instruction.
     * Expected to be called just before calling return.
     *
     * @param logger the recipient logger
     */
    public static void CDMW_LOG_FUNCTION_RETURN(com.thalesgroup.CdmwLogging.Logger logger) 
    {
        if (logger.is_enabled_for((int) LEVEL.getType())) {      
            // get calling stack trace element
            StackTraceElement stack = 
                StackTraceAccessor.getCallingStackElement();
            
            // get filename and line number
            String filename = "no_debug_info";
            int line = 0;
            if (stack.getFileName() != null) {
                filename = stack.getFileName();
                line = stack.getLineNumber();
            }
                
            logger.log((int) LEVEL.getType(), 
                       EXITING + stack.getMethodName() + "() ...", 
                       filename, line);  
        } 
    }

    /**
     * Allows to log a message before the return instruction.
     * Expected to be called just before calling return.
     *
     * @param logger the recipient logger
     * @param depth the index of the stack element to be logged.     
     */
    public static void CDMW_LOG_FUNCTION_RETURN(com.thalesgroup.CdmwLogging.Logger logger, int depth) 
    {
        if (logger.is_enabled_for((int) LEVEL.getType())) {      
            // get calling stack trace element
            StackTraceElement stack = 
                StackTraceAccessor.getCallingStackElement(depth);
            
            // get filename and line number
            String filename = "no_debug_info";
            int line = 0;
            if (stack.getFileName() != null) {
                filename = stack.getFileName();
                line = stack.getLineNumber();
            }
                
            logger.log((int) LEVEL.getType(), 
                       EXITING + stack.getMethodName() + "() ...", 
                       filename, line);  
        } 
    }

    /**
     * Allows to log a message before the return instruction.
     * Expected to be called just before calling return.
     *
     * @param logger the recipient logger
     * @param result the result og the function
     */
    public static void CDMW_LOG_FUNCTION_RETURN(com.thalesgroup.CdmwLogging.Logger logger, 
                                                Object result) 
    {
        if (logger.is_enabled_for((int) LEVEL.getType())) {      
            // get calling stack trace element
            StackTraceElement stack = 
                StackTraceAccessor.getCallingStackElement();
            
            // get filename and line number
            String filename = "no_debug_info";
            int line = 0;
            if (stack.getFileName() != null) {
                filename = stack.getFileName();
                line = stack.getLineNumber();
            }
                
            logger.log((int) LEVEL.getType(), 
                       EXITING + stack.getMethodName() + "() ... [" + 
                       RETURN_MSG_START  + result + RETURN_MSG_END + "]", 
                       filename, line);  
        } 
    }

    /**
     * Allows to log a message before the return instruction.
     * Expected to be called just before calling return.
     *
     * @param logger the recipient logger
     * @param result the result og the function
     * @param depth the index of the stack element to be logged.     
     */
    public static void CDMW_LOG_FUNCTION_RETURN(com.thalesgroup.CdmwLogging.Logger logger, 
                                                Object result, int depth) 
    {
        if (logger.is_enabled_for((int) LEVEL.getType())) {      
            // get calling stack trace element
            StackTraceElement stack = 
                StackTraceAccessor.getCallingStackElement(depth);
            
            // get filename and line number
            String filename = "no_debug_info";
            int line = 0;
            if (stack.getFileName() != null) {
                filename = stack.getFileName();
                line = stack.getLineNumber();
            }
                
            logger.log((int) LEVEL.getType(), 
                       EXITING + stack.getMethodName() + "() ... [" + 
                       RETURN_MSG_START  + result + RETURN_MSG_END + "]", 
                       filename, line);  
        } 
    }

}




