/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

package cdmw.logging;

import com.thalesgroup.CdmwLogging.LogCollector;
import com.thalesgroup.CdmwLogging.LogCollectorHelper;
import com.thalesgroup.CdmwLogging.LogHeader;
import com.thalesgroup.CdmwLogging.LogContext;
import com.thalesgroup.CdmwLogging.StructuredLog;

import java.lang.String;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.GregorianCalendar;


/**
 * This class enables to delegate logging to a remote or collocated LogCollector.
 */
public class LogManagerCollectorDelegate implements LogManagerDelegate
{
    private boolean m_ORBLogfileOpenAttempted = false;
    private cdmw.logging.LogManagerDelegate m_ORBLogManagerDelegate = null;
    private LogCollector refCollector;


/**
 * Try to create LogManagerDefaultDelegate for ORB Traces
 *
 * @param context where to retrieve application_name and process_name
 *        and process_id
 * @param header where to found component_name
 */
    synchronized private boolean openLogFile(LogContext context, LogHeader header) {
        if (! this.m_ORBLogfileOpenAttempted) {
            // Try only one time to create the ORBLogManagerDelegate
            this.m_ORBLogfileOpenAttempted = true;

            String logName = new String();
            try {
                // Log File Name [application_name-process_name-process_id.log] or
                //               [component_name-process_id.log]
                if (context.application_name.length() > 0 )
                    logName += context.application_name + "-";
                if (context.process_name.length() > 0 )
                    logName += context.process_name + "-";
                if (logName.length() == 0 && header.component_name.length() > 0 )
                    logName += header.component_name + "-";
                logName += context.process_id + ".log";

                this.m_ORBLogManagerDelegate =
                    new cdmw.logging.LogManagerDefaultDelegate(logName);
            } catch(cdmw.common.BadParameterException bpe) {
                System.out.println(bpe);
            }
        }

        return (this.m_ORBLogManagerDelegate != null);
    }

/**
 * Store ORB Traces
 *
 * @param context the log context.
 * @param header the log header.
 * @param msg the message.
 */
    private boolean storeLog(LogContext context, LogHeader header, String msg) {
        if((this.m_ORBLogManagerDelegate != null) || this.openLogFile(context, header)){
            // Attempt opening the log file
            return this.m_ORBLogManagerDelegate.addLog(context,header,msg);
        }

        return false;
    }

/**
 * Constructor.
 * @param refCollector reference to the remote CORBA LogCollector
 */
    public LogManagerCollectorDelegate(LogCollector refCollector) {
        cdmw.common.Assert.check(refCollector != null);
        this.refCollector = refCollector;
    }

/**
 * Send a message to the LogCollector
 * Except ORB traces written in a file
 *
 * @param context the log context.
 * @param header the log header.
 * @param msg the message.
 *
 * @return true if successful.
 */
    public boolean addLog(LogContext context, LogHeader header, String msg) {

        boolean result = true;

        if ( (header.level & LogLevel.ORB_TRACE) != 0) {
            return this.storeLog(context, header, msg);
        }

        try {
            StructuredLog log = new StructuredLog(context, header, msg);
            result = this.refCollector.add_log(log);
        }
        catch (java.lang.Exception e) {
            System.out.println("Cannot log message: " + e.getMessage());
            result = false;
        }

        return result;
    }

}
