/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

package cdmw.logging;

import java.io.*;
import java.lang.Long;
import cdmw.ossupport.*;
import cdmw.common.*;

import com.thalesgroup.CdmwLogging.LogHeader;
import com.thalesgroup.CdmwLogging.LogContext;
import com.thalesgroup.CdmwLogging.StructuredLog;

import java.security.InvalidAlgorithmParameterException;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.GregorianCalendar;
import java.util.NoSuchElementException;
import java.util.StringTokenizer;
import java.lang.String;
import java.text.NumberFormat;

/**
 * This class enables logging in standard output or file.
 */
public class LogManagerDefaultDelegate implements LogManagerDelegate
{

    /**
     * logs output; if null, standard output is used.
     */ 
    private BufferedWriter ofstream;

    /**
     * Max length of a field in a log head
     */
    private static final int BUFFER_MAX_LENGTH = 128;



    /**
     * Constructor.
     */
    public LogManagerDefaultDelegate() {
        this.ofstream = null;
    }

    /**
     * Constructor.
     * @param fileName output log file
     * @exception BadParameterException when the specified file is not writable
     */
    public LogManagerDefaultDelegate(String fileName) throws BadParameterException {
        try {
            this.ofstream = new BufferedWriter(new FileWriter(fileName, true));
            System.out.println("CDMW Log set to " + fileName);
        }
        catch (IOException ioe) {
            throw new BadParameterException("Couldn't create log file " + fileName + ": "
                    + ioe.getMessage());
        }
    }


    /**
     * timestamp_to_string() utility function
     */
    protected String timestampToString(com.thalesgroup.CdmwLogging.Timeval tv)
    {
        // write time with second precision in buffer
        SimpleDateFormat tf = new SimpleDateFormat("yyyy-MM-dd'T'HH:mm:ss'Z'");
        GregorianCalendar calendar = new GregorianCalendar();
        calendar.setTime( 
                         new Date((long) 
                                  tv.seconds * 1000
                                  + tv.microseconds / 1000));
        // add milliseconds and microseconds in buffer
        return tf.format(calendar.getTime());
    }


    /**
     * Replaces an empty string by "[]" and truncate to BUFFER_MAX_LENGTH if needed
     */
    protected String manageField(final String format, final String value)
    {
        final String EMPTY_FIELD = "[]";
        if(value.length() == 0)
            return EMPTY_FIELD;
        else {
            // The following code will truncate 'value' to BUFFER_MAX_LENGTH, if it is longer than BUFFER_MAX_LENGTH
            String tmp;
            if (value.length() > BUFFER_MAX_LENGTH)
                // if we truncate, we replace the last 3 characters by '...'
                tmp = value.substring(0,BUFFER_MAX_LENGTH-3) + "...";
            else
                tmp = value;
            
            return String.format(format,tmp);
        }
    }


    /**
     * Format horizontal log header
     * @param os the stream where to build the log header
     * @param context the log context
     * @param header the log header
     */
    protected String formatLog(final com.thalesgroup.CdmwLogging.LogContext context,
                      final com.thalesgroup.CdmwLogging.LogHeader header) {
        String buffer = new String();

        // UTC Time
        buffer += manageField("%s", timestampToString(header.time_stamping)) + " ";
        NumberFormat nf = NumberFormat.getInstance();
        nf.setGroupingUsed(false); // No grouping used for integer part of a number
        nf.setParseIntegerOnly(true); // Numbers are only parsed as integers
        nf.setMaximumIntegerDigits(12);
        nf.setMinimumIntegerDigits(12);
        buffer += manageField("%s", nf.format(header.time_stamping.seconds)) + ".";
        
        nf.setMaximumIntegerDigits(6);
        nf.setMinimumIntegerDigits(6);        
        buffer += manageField("%s",  nf.format(header.time_stamping.microseconds)) + " ";

        // Operational Time
        buffer += manageField("%s", "") + " ";

        // Hostname
        buffer += manageField("%s", context.host_name) + " ";

        // Application name
        buffer += manageField("%s", context.application_name) + " ";

        // Process Name
        buffer += manageField("%s", context.process_name) + " ";

        // PID
        nf.setMaximumIntegerDigits(5);
        nf.setMinimumIntegerDigits(5);
        buffer += manageField ("%s", nf.format(context.process_id)) + " ";

        // Component name
        buffer += manageField("%s", header.component_name) + " ";

        // User domain
        buffer += manageField("%s", header.user_domain) + " ";

        // Log Level
        buffer += manageField("%s", LogLevel.LevelToString(header.level))  + " ";

        // Thread id
        buffer += manageField("%s", nf.format(header.thread_id)) + " ";

        // File name
        buffer += manageField("%s", header.file_name) + " ";

        // File line
        buffer += manageField("%-5.5s", nf.format(header.file_line)) + " ";

        // NDC
        buffer += manageField ("%s", context.ndc);

        return buffer;
    }


    /**
     * Send a message to the logger.
     *
     * @param context the log context.
     * @param header the message header.
     * @param msg the message.
     *
     * @return true if successful.
     */
    public boolean addLog(LogContext context, LogHeader header, String msg) {
        
        boolean result = true;
        try
        {
            // context not logged
            String buffer = formatLog(context,header);
            buffer += " ==> " + msg;

            synchronized (this) {
                if (this.ofstream != null) {
                    this.ofstream.write(buffer);
                    this.ofstream.newLine();
                    this.ofstream.flush();
                }
                else {
                    System.out.println(buffer);
                }
            }
        }
        catch (IOException e) {
            System.out.println("Cannot log message: " + e.getMessage());
            result = false;
        }

        return result;
    }


    @Override
    protected void finalize() throws Throwable {
        super.finalize();
        this.ofstream.close();
    }

}
