/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

package cdmw.logging;

import java.lang.Thread;
import java.util.*;
import java.io.PrintWriter;
import java.io.StringWriter;

import com.thalesgroup.CdmwLogging.LogHeader;
import com.thalesgroup.CdmwLogging.Timeval;

public class Logger
{

    /**
     * Default LevelTypes object
     */
    public static final LevelType FATAL   = new LevelType(LogLevel.FATAL  );
    public static final LevelType ERROR   = new LevelType(LogLevel.ERROR  );
    public static final LevelType WARN    = new LevelType(LogLevel.WARN   );
    public static final LevelType INFO    = new LevelType(LogLevel.INFO   );
    public static final LevelType DEBUG   = new LevelType(LogLevel.DEBUG  );

    public static final LevelType DEBUG_1 = new LevelType(LogLevel.DEBUG_1);
    public static final LevelType DEBUG_2 = new LevelType(LogLevel.DEBUG_2);
    public static final LevelType DEBUG_3 = new LevelType(LogLevel.DEBUG_3);
    public static final LevelType DEBUG_4 = new LevelType(LogLevel.DEBUG_4);
    public static final LevelType DEBUG_5 = new LevelType(LogLevel.DEBUG_5);
    public static final LevelType DEBUG_6 = new LevelType(LogLevel.DEBUG_6);
    public static final LevelType DEBUG_7 = new LevelType(LogLevel.DEBUG_7);
    public static final LevelType DEBUG_8 = new LevelType(LogLevel.DEBUG_8);
    public static final LevelType DEBUG_9 = new LevelType(LogLevel.DEBUG_9);

    public static final LevelType CDMW_DEBUG_1 = new LevelType(LogLevel.CDMW_DEBUG_1);
    public static final LevelType CDMW_DEBUG_2 = new LevelType(LogLevel.CDMW_DEBUG_2);
    public static final LevelType CDMW_DEBUG_3 = new LevelType(LogLevel.CDMW_DEBUG_3);
    public static final LevelType CDMW_DEBUG_4 = new LevelType(LogLevel.CDMW_DEBUG_4);
    
    public static final LevelType REQ_FOLLOWUP = new LevelType(LogLevel.REQ_FOLLOWUP);
    public static final LevelType CALL_TRACE   = new LevelType(LogLevel.CALL_TRACE);
    public static final LevelType ORB_DEBUG    = new LevelType(LogLevel.ORB_TRACE);


    private Filter m_filter = null;
    private String m_domain = "";
    private String m_componentName = "";
    private LogManager m_logMngr = null;
   
    /**
     * Create a Logger.
     *
     */
    protected Logger() {
    }

    /**
     * Create a Logger .
     *
     * @param logMngr the LogManager.
     * @param componentName the name of the component.
     * @param domain the name of the domain.
     */
    protected Logger(LogManager logMngr, String componentName, String domain) {
        m_logMngr = logMngr;
        m_componentName = componentName;
        m_domain = domain;
        FilterMngr filterMngr = logMngr.getFilterManager();
        m_filter = filterMngr.getFilterRef(componentName,domain);
    }
    
    /**
     * Create a Logger.
     *
     * @param logMngr the LogManager.
     * @param domain the name of the domain.
     */
    protected Logger(LogManager logMngr, String domain) {
        m_logMngr = logMngr;
        m_componentName = "";
        m_domain = domain;
        FilterMngr filterMngr = logMngr.getFilterManager();
        m_filter = filterMngr.getFilterRef(domain);
    }


    /**
     * Log a generic message (with Nested Diagnostic Context).
     *
     * @param level the message level.
     * @param message the message.
     * @param ndc the Nested Diagnostic Context.
     * @param file the file from which the message is output.
     * @param line the line number in the file.
     */
    public void log (LevelType level, String message, String ndc, String filename, int line) {
        LogHeader header = new LogHeader();
        Thread t = Thread.currentThread();
        header.component_name = this.m_componentName;
        header.user_domain = this.m_domain;
        header.level = (int) level.getType();
        header.thread_id = t.getId();
        header.file_line = line;
        header.file_name = filename;
        cdmw.ossupport.Timeval cur_time = new cdmw.ossupport.Timeval(Calendar.getInstance().getTimeInMillis());
        header.time_stamping = new Timeval(cur_time.getSeconds(), cur_time.getMicroseconds());

        m_logMngr.addLog(header, message, ndc); 
    }
    
    /**
     * Log a generic message.
     *
     * @param level the message level.
     * @param message the message.
     * @param file the file from which the message is output.
     * @param line the line number in the file.
     */
    public void log (LevelType level, String message, String filename, int line) {
        log(level, message, "", filename, line); 
    }
    
    /**
     * Log a generic message (with Nested Diagnostic Context).
     *
     * @param level the message level.
     * @param message the message.
     * @param ndc the Nested Diagnostic Context.
     * @param thr the throwable thread.
     */
    public void log (LevelType level, String message, String ndc, Throwable thr) {
       String filename = "";
       int line = 0;
       if (thr != null) {
           StackTraceElement[] stack = thr.getStackTrace();
           if (stack[0].getFileName() != null) {
               filename = stack[0].getFileName();
               line = stack[0].getLineNumber();
          } else {
               filename = "no_debug_info";
          }
       }
       
       if(level.equals(Logger.FATAL)){
          StringWriter sw=new StringWriter();
          thr.printStackTrace(new PrintWriter(sw));
          message+="\n"+sw.toString();
       }

        log(level, message, ndc, filename, line); 

    }

    /**
     * Log a generic message (with Nested Diagnostic Context).
     *
     * @param level the message level.
     * @param message the message.
     * @param ndc the Nested Diagnostic Context.
     * @param depth the index of the stack element to be logged.
     */
    public void log (LevelType level, String message, int depth, String ndc) {
       String filename = "no_debug_info";
       int line = 0;
                                                
        // get calling stack trace element
        StackTraceElement stack = 
                StackTraceAccessor.getCallingStackElement(depth);
            
        if (stack.getFileName() != null) {
            filename = stack.getFileName();
            line = stack.getLineNumber();
        }

        log(level, message, ndc, filename, line); 

    }
    
    /**
     * Log a generic message.
     *
     * @param level the message level.
     * @param message the message.
     * @param thr the throwable thread.
     */
    public void log (LevelType level, String message, Throwable thr) {
        log(level, message, "", thr); 
    }

    /**
     * Log a generic message.
     *
     * @param level the message level.
     * @param message the message.
     * @param depth the index of the stack element to be logged.
     */
    public void log (LevelType level, String message, int depth) {
        log(level, message, depth+1,""); 
    }
    
    /**
     * Log a generic message.
     *
     * @param level the message level.
     * @param message the message.
     */
    public void log (LevelType level, String message) {
        log(level, message, 0+1,""); 
    }    


    /**
     * Find the logger associated to the specified domain name.
     *
     * NOTA: the caller must assume the ownership of the returned
     * Logger object.
     *
     * @param domain the domain name.
     *
     * @return the logger if found, a new logger otherwise.
     */
    public static Logger Get_Logger(String domain) {
        LogManager logMngr = LogManager.Instance();
        return logMngr.getLogger("", domain);
    }

    /**
     * Find the logger associated to the specified component name and
     * domain name.
     *
     * NOTA: the caller must assume the ownership of the returned
     * Logger object.
     *
     * NOTA: If the logger don't exist in the logger list, create a new instance
     *
     * @param component the component name.
     * @param domain the domain name.
     *
     * @return the logger if found, a new logger otherwise.
     */
    public static Logger Get_Logger(String componentName, String domain) {
        LogManager logMngr = LogManager.Instance();
        return logMngr.getLogger(componentName, domain);
    }

//    /**
//     * Log a debug message.
//     *
//     * @param message the message.
//     * @param file the file from which the message is output.
//     * @param line the line number in the file.
//     */
//
//    public void debug (String message, Throwable thr) {
//        log(DEBUG, message, thr);
//    }

//    /**
//     * Log a info message.
//     *
//     * @param message the message.
//     * @param file the file from which the message is output.
//     * @param line the line number in the file.
//     */
//
//    public void info (String message, Throwable thr) {
//        log(INFO, message, thr);
//    }

//    /**
//     * Log a warn message.
//     *
//     * @param message the message.
//     * @param file the file from which the message is output.
//     * @param line the line number in the file.
//     */
//
//    public void warn (String message, Throwable thr) {
//        log(WARN, message, thr);
//    }
    
//    /**
//     * Log a error message.
//     *
//     * @param message the message.
//     * @param file the file from which the message is output.
//     * @param line the line number in the file.
//     */
//
//    public void error (String message, Throwable thr) {
//        log(ERROR, message, thr);
//    }
    

//    /**
//     * Log a fatal message.
//     *
//     * @param message the message.
//     * @param file the file from which the message is output.
//     * @param line the line number in the file.
//     */
//
//    public void fatal (String message, Throwable thr) {
//        log(FATAL, message, thr);
//    }

    /**
     * Check whether a log level is enabled.
     *
     * @return true if enabled.
     */

    public boolean isEnabledFor(LevelType level) {
        return m_filter.isLevelActivated(level.getType());
    }

    
    /**
     * Check whether DEBUG level is enabled.
     *
     * @return true if enabled.
     */
    public boolean isDebugEnabled() {
        return m_filter.isLevelActivated(LogLevel.DEBUG);
    }

    /**
     * Check whether DEBUG level is enabled.
     *
     * @return true if enabled.
     */

    public boolean isInfoEnabled() {
        return m_filter.isLevelActivated(LogLevel.INFO);
    }

    /**
     * Check whether WARN level is enabled.
     *
     * @return true if enabled.
     */

    public boolean isWarnEnabled() {
        return m_filter.isLevelActivated(LogLevel.WARN);
    }

    /**
     * Check whether ERROR level is enabled.
     *
     * @return true if enabled.
     */

    public boolean isErrorEnabled() {
        return m_filter.isLevelActivated(LogLevel.ERROR);
    }

    
    /**
     * Check whether FATAL level is enabled.
     *
     * @return true if enabled.
     */

    public boolean isFatalEnabled() {
        return m_filter.isLevelActivated(LogLevel.FATAL);
    }


    /**
     * Check whether ORB_TRACE level is enabled.
     *
     * @return true if enabled.
     */

    public boolean isOrbTraceEnabled() {
        return m_filter.isLevelActivated(LogLevel.ORB_TRACE);
    }


    static public void CDMW_LOG(Logger logger, LevelType level, String msg, Throwable thr) {
        if (logger.isEnabledFor(level)) {
            logger.log(level, msg, thr);
        }
    }

    static public void CDMW_LOG(Logger logger, LevelType level, String msg, String ndc, Throwable thr) {
        if (logger.isEnabledFor(level)) {
            logger.log(level, msg, ndc, thr);
        }
    }

    static public void CDMW_LOG(Logger logger, LevelType level, String msg, String ndc, int depth) {
        if (logger.isEnabledFor(level)) {
            logger.log(level, msg, depth+1, ndc);
        }
    }
    
    static public void CDMW_LOG(Logger logger, LevelType level, String msg, int depth) {
        if (logger.isEnabledFor(level)) {
            logger.log(level, msg, depth+1);
        }
    }

    static public void CDMW_LOG(Logger logger, LevelType level, String msg) {
        if (logger.isEnabledFor(level)) {
            logger.log(level, msg, 0+1);
        }
    }
             
    static public void CDMW_FATAL(Logger logger, String msg, Throwable thr) {
        if (logger.isFatalEnabled()) {
            logger.log(FATAL, msg, thr);
        }
    }

    static public void CDMW_FATAL(Logger logger, String msg, int depth) {
        if (logger.isFatalEnabled()) {
            logger.log(FATAL, msg, depth+1);
        }
    }

    static public void CDMW_FATAL(Logger logger, String msg) {
        if (logger.isFatalEnabled()) {
            logger.log(FATAL, msg, 0+1);
        }
    }

    static public void CDMW_ERROR(Logger logger, String msg, Throwable thr) {
        if (logger.isErrorEnabled()) {
            logger.log(ERROR, msg, thr);
        }
    }
    
    static public void CDMW_ERROR(Logger logger, String msg) {
        if (logger.isErrorEnabled()) {
            logger.log(ERROR, msg,0+1);
        }
    }
    
    static public void CDMW_ERROR(Logger logger, String msg, int depth) {
        if (logger.isErrorEnabled()) {
            logger.log(ERROR, msg, depth+1);
        }
    }
            
    static public void CDMW_WARN(Logger logger, String msg, Throwable thr) {
        if (logger.isWarnEnabled()) {
            logger.log(WARN, msg, thr);
        }
    }

    static public void CDMW_WARN(Logger logger, String msg, int depth) {
        if (logger.isWarnEnabled()) {
            logger.log(WARN, msg, depth+1);
        }
    }
    
    static public void CDMW_WARN(Logger logger, String msg) {
        if (logger.isWarnEnabled()) {
            logger.log(WARN, msg, 0+1);
        }
    }    
 
    static public void CDMW_INFO(Logger logger, String msg, Throwable thr) {
        if (logger.isInfoEnabled()) {
            logger.log(INFO, msg, thr);
        }
    }

    static public void CDMW_INFO(Logger logger, String msg, int depth) {
        if (logger.isInfoEnabled()) {
            logger.log(INFO, msg, depth+1);
        }
    }

    static public void CDMW_INFO(Logger logger, String msg) {
        if (logger.isInfoEnabled()) {
            logger.log(INFO, msg, 0+1);
        }
    }

    static public void CDMW_DEBUG(Logger logger, String msg, Throwable thr) {
        if (logger.isDebugEnabled()) {
            logger.log(DEBUG, msg, thr);
        }
    }

    static public void CDMW_DEBUG(Logger logger, String msg, int depth) {
        if (logger.isDebugEnabled()) {
            logger.log(DEBUG, msg, depth+1);
        }
    }

    static public void CDMW_DEBUG(Logger logger, String msg) {
        if (logger.isDebugEnabled()) {
            logger.log(DEBUG, msg, 0+1);
        }
    }
    
    static public void CDMW_DEBUG_1(Logger logger, String msg, Throwable thr) {
        if (logger.isEnabledFor(DEBUG_1)) {
            logger.log(DEBUG_1, msg, thr);
        }
    }

    static public void CDMW_DEBUG_1(Logger logger, String msg, int depth) {
        if (logger.isEnabledFor(DEBUG_1)) {
            logger.log(DEBUG_1, msg, depth+1);
        }
    }
    
    static public void CDMW_DEBUG_1(Logger logger, String msg) {
        if (logger.isEnabledFor(DEBUG_1)) {
            logger.log(DEBUG_1, msg, 0+1);
        }
    }
            
    static public void CDMW_DEBUG_2(Logger logger, String msg, Throwable thr) {
        if (logger.isEnabledFor(DEBUG_2)) {
            logger.log(DEBUG_2, msg, thr);
        }
    }

    static public void CDMW_DEBUG_2(Logger logger, String msg, int depth) {
        if (logger.isEnabledFor(DEBUG_2)) {
            logger.log(DEBUG_2, msg, depth+1);
        }
    }
    
    static public void CDMW_DEBUG_2(Logger logger, String msg) {
        if (logger.isEnabledFor(DEBUG_2)) {
            logger.log(DEBUG_2, msg, 0+1);
        }
    }
        
    static public void CDMW_DEBUG_3(Logger logger, String msg, Throwable thr) {
        if (logger.isEnabledFor(DEBUG_3)) {
            logger.log(DEBUG_3, msg, thr);
        }
    }

    static public void CDMW_DEBUG_3(Logger logger, String msg, int depth) {
        if (logger.isEnabledFor(DEBUG_3)) {
            logger.log(DEBUG_3, msg, depth+1);
        }
    }
    
    static public void CDMW_DEBUG_3(Logger logger, String msg) {
        if (logger.isEnabledFor(DEBUG_3)) {
            logger.log(DEBUG_3, msg, 0+1);
        }
    }
        
    static public void CDMW_DEBUG_4(Logger logger, String msg, Throwable thr) {
        if (logger.isEnabledFor(DEBUG_4)) {
            logger.log(DEBUG_4, msg, thr);
        }
    }

    static public void CDMW_DEBUG_4(Logger logger, String msg, int depth) {
        if (logger.isEnabledFor(DEBUG_4)) {
            logger.log(DEBUG_4, msg, depth+1);
        }
    }
    
    static public void CDMW_DEBUG_4(Logger logger, String msg) {
        if (logger.isEnabledFor(DEBUG_4)) {
            logger.log(DEBUG_4, msg, 0+1);
        }
    }
        
   static public void CDMW_DEBUG_5(Logger logger, String msg, Throwable thr) {
        if (logger.isEnabledFor(DEBUG_5)) {
            logger.log(DEBUG_5, msg, thr);
        }
    }

    static public void CDMW_DEBUG_5(Logger logger, String msg, int depth) {
        if (logger.isEnabledFor(DEBUG_5)) {
            logger.log(DEBUG_5, msg, depth+1);
        }
    }
    
    static public void CDMW_DEBUG_5(Logger logger, String msg) {
        if (logger.isEnabledFor(DEBUG_5)) {
            logger.log(DEBUG_5, msg, 0+1);
        }
    }
        
    static public void CDMW_DEBUG_6(Logger logger, String msg, Throwable thr) {
        if (logger.isEnabledFor(DEBUG_6)) {
            logger.log(DEBUG_6, msg, thr);
        }
    }

    static public void CDMW_DEBUG_6(Logger logger, String msg, int depth) {
        if (logger.isEnabledFor(DEBUG_6)) {
            logger.log(DEBUG_6, msg, depth+1);
        }
    }
    
    static public void CDMW_DEBUG_6(Logger logger, String msg) {
        if (logger.isEnabledFor(DEBUG_6)) {
            logger.log(DEBUG_6, msg, 0+1);
        }
    }
        
    static public void CDMW_DEBUG_7(Logger logger, String msg, Throwable thr) {
        if (logger.isEnabledFor(DEBUG_7)) {
            logger.log(DEBUG_7, msg, thr);
        }
    }

    static public void CDMW_DEBUG_7(Logger logger, String msg, int depth) {
        if (logger.isEnabledFor(DEBUG_7)) {
            logger.log(DEBUG_7, msg, depth+1);
        }
    }
    
    static public void CDMW_DEBUG_7(Logger logger, String msg) {
        if (logger.isEnabledFor(DEBUG_7)) {
            logger.log(DEBUG_7, msg, 0+1);
        }
    }
        
    static public void CDMW_DEBUG_8(Logger logger, String msg, Throwable thr) {
        if (logger.isEnabledFor(DEBUG_8)) {
            logger.log(DEBUG_8, msg, thr);
        }
    }

    static public void CDMW_DEBUG_8(Logger logger, String msg, int depth) {
        if (logger.isEnabledFor(DEBUG_8)) {
            logger.log(DEBUG_8, msg, depth+1);
        }
    }
    
    static public void CDMW_DEBUG_8(Logger logger, String msg) {
        if (logger.isEnabledFor(DEBUG_8)) {
            logger.log(DEBUG_8, msg, 0+1);
        }
    }
        
    static public void CDMW_DEBUG_9(Logger logger, String msg, Throwable thr) {
        if (logger.isEnabledFor(DEBUG_9)) {
            logger.log(DEBUG_9, msg, thr);
        }
    }

    static public void CDMW_DEBUG_9(Logger logger, String msg, int depth) {
        if (logger.isEnabledFor(DEBUG_9)) {
            logger.log(DEBUG_9, msg, depth+1);
        }
    }
    
    static public void CDMW_DEBUG_9(Logger logger, String msg) {
        if (logger.isEnabledFor(DEBUG_9)) {
            logger.log(DEBUG_9, msg, 0+1);
        }
    }
    
    //
    // Same macros but using com.thalesgroup.CdmwLogging.Logger IDL
    //
    
    static public void CDMW_LOG(com.thalesgroup.CdmwLogging.Logger logger, 
                                LevelType level, 
                                String msg, 
                                Throwable thr) 
    {
        if (logger.is_enabled_for((int)level.getType())) {
            String filename = "";
            int line = 0;
            if (thr != null) {
                StackTraceElement[] stack = thr.getStackTrace();
                if (stack[0].getFileName() != null) {
                    filename = stack[0].getFileName();
                    line = stack[0].getLineNumber();
               } else {
                    filename = "no_debug_info";
               }
            }
            logger.log((int)level.getType(), msg, filename, line);
        }
    }
    
    static public void CDMW_LOG(com.thalesgroup.CdmwLogging.Logger logger, 
                                LevelType level, 
                                String msg, 
                                int depth) 
    {
        if (logger.is_enabled_for((int)level.getType())) {
            String filename = "no_debug_info";
            int line = 0;
            
            // get calling stack trace element
            StackTraceElement stack = 
                StackTraceAccessor.getCallingStackElement(depth);
            
            if (stack.getFileName() != null) {
            filename = stack.getFileName();
            line = stack.getLineNumber();
            }
            
            logger.log((int)level.getType(), msg, filename, line);
        }
    }

   static public void CDMW_LOG(com.thalesgroup.CdmwLogging.Logger logger, 
                                LevelType level, 
                                String msg) 
    {
        CDMW_LOG(logger, level, msg, 0+1);
    }        
    
    static public void CDMW_FATAL(com.thalesgroup.CdmwLogging.Logger logger, 
                                  String msg, 
                                  Throwable thr) 
    {
        CDMW_LOG(logger, FATAL, msg, thr);
    }

    static public void CDMW_FATAL(com.thalesgroup.CdmwLogging.Logger logger, 
                                  String msg, 
                                  int depth) 
    {
        CDMW_LOG(logger, FATAL, msg, depth+1);
    }

    static public void CDMW_FATAL(com.thalesgroup.CdmwLogging.Logger logger, 
                                  String msg) 
    {
        CDMW_LOG(logger, FATAL, msg,0+1);
    }
        
    static public void CDMW_ERROR(com.thalesgroup.CdmwLogging.Logger logger, 
                                  String msg, 
                                  Throwable thr) 
    {
        CDMW_LOG(logger, ERROR, msg, thr);
    }

    static public void CDMW_ERROR(com.thalesgroup.CdmwLogging.Logger logger, 
                                  String msg, 
                                  int depth) 
    {
        CDMW_LOG(logger, ERROR, msg, depth+1);
    }

    static public void CDMW_ERROR(com.thalesgroup.CdmwLogging.Logger logger, 
                                  String msg) 
    {
        CDMW_LOG(logger, ERROR, msg,0+1);
    }
    
    static public void CDMW_WARN(com.thalesgroup.CdmwLogging.Logger logger, 
                                 String msg, 
                                 Throwable thr) 
    {
        CDMW_LOG(logger, WARN, msg, thr);
    }

    static public void CDMW_WARN(com.thalesgroup.CdmwLogging.Logger logger, 
                                  String msg, 
                                  int depth) 
    {
        CDMW_LOG(logger, WARN, msg, depth+1);
    }

    static public void CDMW_WARN(com.thalesgroup.CdmwLogging.Logger logger, 
                                  String msg) 
    {
        CDMW_LOG(logger, WARN, msg,0+1);
    }
 
    static public void CDMW_INFO(com.thalesgroup.CdmwLogging.Logger logger, 
                                 String msg, 
                                 Throwable thr) 
    {
        CDMW_LOG(logger, INFO, msg, thr);
    }

    static public void CDMW_INFO(com.thalesgroup.CdmwLogging.Logger logger, 
                                  String msg, 
                                  int depth) 
    {
        CDMW_LOG(logger, INFO, msg, depth+1);
    }

    static public void CDMW_INFO(com.thalesgroup.CdmwLogging.Logger logger, 
                                  String msg) 
    {
        CDMW_LOG(logger, INFO, msg,0+1);
    }

    static public void CDMW_DEBUG(com.thalesgroup.CdmwLogging.Logger logger, 
                                  String msg, 
                                  Throwable thr) 
    {
        CDMW_LOG(logger, DEBUG, msg, thr);
    }

    static public void CDMW_DEBUG(com.thalesgroup.CdmwLogging.Logger logger, 
                                  String msg, 
                                  int depth) 
    {
        CDMW_LOG(logger, DEBUG, msg, depth+1);
    }

    static public void CDMW_DEBUG(com.thalesgroup.CdmwLogging.Logger logger, 
                                  String msg) 
    {
        CDMW_LOG(logger, DEBUG, msg,0+1);
    }
    
    static public void CDMW_DEBUG_1(com.thalesgroup.CdmwLogging.Logger logger, 
                                    String msg, 
                                    Throwable thr) 
    {
        CDMW_LOG(logger, DEBUG_1, msg, thr);
    }
    static public void CDMW_DEBUG_1(com.thalesgroup.CdmwLogging.Logger logger, 
                                  String msg, 
                                  int depth) 
    {
        CDMW_LOG(logger, DEBUG_1, msg, depth+1);
    }

    static public void CDMW_DEBUG_1(com.thalesgroup.CdmwLogging.Logger logger, 
                                  String msg) 
    {
        CDMW_LOG(logger, DEBUG_1, msg,0+1);
    }
    
    static public void CDMW_DEBUG_2(com.thalesgroup.CdmwLogging.Logger logger, 
                                    String msg, 
                                    Throwable thr) 
    {
        CDMW_LOG(logger, DEBUG_2, msg, thr);
    }

    static public void CDMW_DEBUG_2(com.thalesgroup.CdmwLogging.Logger logger, 
                                  String msg, 
                                  int depth) 
    {
        CDMW_LOG(logger, DEBUG_2, msg, depth+1);
    }

    static public void CDMW_DEBUG_2(com.thalesgroup.CdmwLogging.Logger logger, 
                                  String msg) 
    {
        CDMW_LOG(logger, DEBUG_2, msg,0+1);
    }
    
    static public void CDMW_DEBUG_3(com.thalesgroup.CdmwLogging.Logger logger, 
                                    String msg, 
                                    Throwable thr) 
    {
        CDMW_LOG(logger, DEBUG_3, msg, thr);
    }

    static public void CDMW_DEBUG_3(com.thalesgroup.CdmwLogging.Logger logger, 
                                  String msg, 
                                  int depth) 
    {
        CDMW_LOG(logger, DEBUG_3, msg, depth+1);
    }

    static public void CDMW_DEBUG_3(com.thalesgroup.CdmwLogging.Logger logger, 
                                  String msg) 
    {
        CDMW_LOG(logger, WARN, msg,0+1);
    }
    
    static public void CDMW_DEBUG_4(com.thalesgroup.CdmwLogging.Logger logger, 
                                    String msg, 
                                    Throwable thr) 
    {
        CDMW_LOG(logger, DEBUG_4, msg, thr);
    }

    static public void CDMW_DEBUG_4(com.thalesgroup.CdmwLogging.Logger logger, 
                                  String msg, 
                                  int depth) 
    {
        CDMW_LOG(logger, DEBUG_4, msg, depth+1);
    }

    static public void CDMW_DEBUG_4(com.thalesgroup.CdmwLogging.Logger logger, 
                                  String msg) 
    {
        CDMW_LOG(logger, DEBUG_4, msg,0+1);
    }
    
   static public void CDMW_DEBUG_5(com.thalesgroup.CdmwLogging.Logger logger, 
                                   String msg, 
                                   Throwable thr) 
    {
        CDMW_LOG(logger, DEBUG_5, msg, thr);
    }

    static public void CDMW_DEBUG_5(com.thalesgroup.CdmwLogging.Logger logger, 
                                  String msg, 
                                  int depth) 
    {
        CDMW_LOG(logger, DEBUG_5, msg, depth+1);
    }

    static public void CDMW_DEBUG_5(com.thalesgroup.CdmwLogging.Logger logger, 
                                  String msg) 
    {
        CDMW_LOG(logger, DEBUG_5, msg,0+1);
    }
    
    static public void CDMW_DEBUG_6(com.thalesgroup.CdmwLogging.Logger logger, 
                                    String msg, 
                                    Throwable thr) 
    {
        CDMW_LOG(logger, DEBUG_6, msg, thr);
    }

    static public void CDMW_DEBUG_6(com.thalesgroup.CdmwLogging.Logger logger, 
                                  String msg, 
                                  int depth) 
    {
        CDMW_LOG(logger, DEBUG_6, msg, depth+1);
    }

    static public void CDMW_DEBUG_6(com.thalesgroup.CdmwLogging.Logger logger, 
                                  String msg) 
    {
        CDMW_LOG(logger, DEBUG_6, msg,0+1);
    }
    
    static public void CDMW_DEBUG_7(com.thalesgroup.CdmwLogging.Logger logger, 
                                    String msg, 
                                    Throwable thr) 
    {
        CDMW_LOG(logger, DEBUG_7, msg, thr);
    }

    static public void CDMW_DEBUG_7(com.thalesgroup.CdmwLogging.Logger logger, 
                                  String msg, 
                                  int depth) 
    {
        CDMW_LOG(logger, DEBUG_7, msg, depth+1);
    }

    static public void CDMW_DEBUG_7(com.thalesgroup.CdmwLogging.Logger logger, 
                                  String msg) 
    {
        CDMW_LOG(logger, DEBUG_7, msg,0+1);
    }
    
    static public void CDMW_DEBUG_8(com.thalesgroup.CdmwLogging.Logger logger, 
                                    String msg, 
                                    Throwable thr) 
    {
        CDMW_LOG(logger, DEBUG_8, msg, thr);
    }

    static public void CDMW_DEBUG_8(com.thalesgroup.CdmwLogging.Logger logger, 
                                  String msg, 
                                  int depth) 
    {
        CDMW_LOG(logger, DEBUG_8, msg, depth+1);
    }

    static public void CDMW_DEBUG_8(com.thalesgroup.CdmwLogging.Logger logger, 
                                  String msg) 
    {
        CDMW_LOG(logger, DEBUG_8, msg,0+1);
    }
    
    static public void CDMW_DEBUG_9(com.thalesgroup.CdmwLogging.Logger logger, 
                                   String msg, 
                                   Throwable thr) 
    {
        CDMW_LOG(logger, DEBUG_9, msg, thr);
    }

    static public void CDMW_DEBUG_9(com.thalesgroup.CdmwLogging.Logger logger, 
                                  String msg, 
                                  int depth) 
    {
        CDMW_LOG(logger, DEBUG_9, msg, depth+1);
    }

    static public void CDMW_DEBUG_9(com.thalesgroup.CdmwLogging.Logger logger, 
                                  String msg) 
    {
        CDMW_LOG(logger, DEBUG_9, msg,0+1);
    }
    
}
