/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#include "Foundation/orbsupport/CORBA.hpp"
#include "Foundation/orbsupport/OrbSupport.hpp"
#include "Foundation/orbsupport/ExceptionMinorCodes.hpp"
#include "Foundation/common/Options.hpp"
#include "Foundation/ossupport/OS.hpp"
#include "Foundation/ossupport/Signals.hpp"
#include "orbsupport/ExceptionStrings.hpp"
#include "orbsupport/OrbShutdownSignalHandler.hpp"

#if CDMW_ORB_VDR == orbacus

#include "orbsupport/OrbOrbacusImpl.hpp"

#elif CDMW_ORB_VDR == tao

#include "orbsupport/OrbTaoImpl.hpp"

#endif

#include <sstream>
#include <string>

#include <Foundation/logging/LogManager.hpp>
#include <Foundation/ossupport/SystemProperties.hpp>


#define DEFINE_EXCEPTION_TO_STRING(name) \
    std::string name ## _to_string(const CORBA::name & ex) \
    throw (Cdmw::OutOfMemoryException)  \
    { \
        return system_exception_string(#name, \
                                       ex.minor(), \
                                       ex.completed(), \
                                       Cdmw::OrbSupport::ExceptionStrings::name ## _reasons); \
    }

#define CHECK_AND_RETURN_SYSTEM_EXCEPTION(name) \
    (ex_name == #name) \
    do { \
        CORBA::name * sys_ex \
        = CORBA::name ::_downcast( (CORBA::Exception*)&ex ); \
        return name ## _to_string(*sys_ex); \
    } while(0)



#if HAS_CORBA_EXCEPTION_OSTREAM_OPERATOR_DEFINITION == 0

//
// Exception output on std::ostream
//
std::ostream &operator<<(std::ostream& os, const CORBA::Exception& ex)
{
    try
    {
        os << Cdmw::OrbSupport::OrbSupport::exception_to_string(ex);
    }
    catch (...)
    {
        // ignore ...
    }

    return os;
}

#endif

namespace
{
    // VMCID (Vendor Minor Codeset ID) mask
    const CORBA::ULong VMCID_MASK = 0xfffff000;
    const CORBA::ULong MINOR_CODE_MASK = 0x00000fff;

    const long MIN_PORT_NUMBER = 0;
    const long MAX_PORT_NUMBER = 65535;

    std::string system_exception_string(const std::string & exception_name,
                                        CORBA::ULong minor_code,
                                        CORBA::CompletionStatus completion_status,
                                        const char* messages[])
    throw(Cdmw::OutOfMemoryException)
    {
        try
        {
            std::ostringstream s;
            s << "System exception `" << exception_name << "'" << "\n";
            s << "Completed: ";

            switch (completion_status)
            {

                    case CORBA::COMPLETED_YES:
                    s << "yes";
                    break;

                    case CORBA::COMPLETED_NO:
                    s << "no";
                    break;

                    case CORBA::COMPLETED_MAYBE:
                    s << "maybe";
                    break;
            }

            s << "\n";
            s << "Minor code: " << std::hex
            << (unsigned long)minor_code << std::dec << "\n";

            s << "(" << messages[minor_code & MINOR_CODE_MASK] << ")";

            return s.str();
        }
        catch (const std::bad_alloc & )
        {
            CDMW_THROW(Cdmw::OutOfMemoryException);
        }
    }

    DEFINE_EXCEPTION_TO_STRING(BAD_INV_ORDER)
    DEFINE_EXCEPTION_TO_STRING(BAD_PARAM)
    DEFINE_EXCEPTION_TO_STRING(INTERNAL)
    DEFINE_EXCEPTION_TO_STRING(NO_IMPLEMENT)
    DEFINE_EXCEPTION_TO_STRING(NO_MEMORY)
    DEFINE_EXCEPTION_TO_STRING(NO_PERMISSION)
    DEFINE_EXCEPTION_TO_STRING(NO_RESOURCES)
    DEFINE_EXCEPTION_TO_STRING(OBJECT_NOT_EXIST)
    DEFINE_EXCEPTION_TO_STRING(TRANSIENT)
    DEFINE_EXCEPTION_TO_STRING(BAD_CONTEXT)
}

; // End anonymous namespace

namespace Cdmw
{

    namespace OrbSupport
    {

        const char* CDMW_ROOT_POA_NAME = "CDMW_RootPOA";

#if CDMW_ORB_VDR == orbacus

        typedef OrbOrbacusImpl UnderlyingORB;

#elif CDMW_ORB_VDR == tao

        typedef OrbTaoImpl UnderlyingORB;

#endif

        /**
        *Implementation notes: none 
        *<p>
        *Portability issues: none
        *<p>
        */
       CORBA::ORB_ptr
       OrbSupport::ORB_init (int& argc, char** argv,
                             const char* orb_identifier,
                             const Cdmw::Common::Properties* prop)
          throw(CORBA::SystemException)
       {
          return CORBA::ORB::_nil();
       }

        //
        // Returns an initiliased ORB
        //
        CORBA::ORB_ptr
        OrbSupport::ORB_init (int& argc, char** argv,
                              const StrategyList& strategylist,
                              const char* orb_identifier)
        throw (CORBA::SystemException)
        {
            // Is CDMW Process port number specified?
            // Check Options::PROCESS_PORT_OPTION in argv for the value of the port number

            using namespace Cdmw::OsSupport;

            using namespace Cdmw::Common;
            std::string root_poa_port_option;

            try
            {
                root_poa_port_option = OS::get_option_value(argc, argv,
                                       Options::LOCALISATION_SERVICE_OPTION);
            }
            catch (const OutOfMemoryException &)
            {
                throw CORBA::NO_MEMORY(NO_MEMORY, CORBA::COMPLETED_NO);
            }
            catch (const InternalErrorException &)
            {
                throw CORBA::INTERNAL(INTERNAL, CORBA::COMPLETED_NO);
            }


            std::size_t root_poa_port_number(0);

            if (root_poa_port_option != "no")
            {
                if (root_poa_port_option != "yes")
                {
                    long port;
                    std::istringstream strPoaPort(root_poa_port_option);
                    strPoaPort >> port;

                    if (strPoaPort.fail() ||
                            !(port > MIN_PORT_NUMBER && port < MAX_PORT_NUMBER))
                    {
                        throw CORBA::BAD_PARAM(BAD_PARAMInvalidPortNumber, CORBA::COMPLETED_NO);
                    }
                    else
                        root_poa_port_number = port;
                }
                else
                {
                    throw CORBA::BAD_PARAM(BAD_PARAMInvalidPortNumber, CORBA::COMPLETED_NO);
                }
            } // else ignore.

            std::string cdmw_root_poa_port_option;

            try
            {
                cdmw_root_poa_port_option = OS::get_option_value (argc, argv,
                                             Options::PROCESS_PORT_OPTION);
            }
            catch (const OutOfMemoryException &)
            {
                throw CORBA::NO_MEMORY(NO_MEMORY, CORBA::COMPLETED_NO);
            }
            catch (const InternalErrorException &)
            {
                throw CORBA::INTERNAL(INTERNAL, CORBA::COMPLETED_NO);
            }


            std::size_t cdmw_root_poa_port_number(0);

            if (cdmw_root_poa_port_option != "no")
            {
                if (cdmw_root_poa_port_option != "yes")
                {
                    long port;
                    std::istringstream strPoaPort(cdmw_root_poa_port_option);
                    strPoaPort >> port;

                    if (strPoaPort.fail() ||
                            !(port > MIN_PORT_NUMBER && port < MAX_PORT_NUMBER))
                    {
                        throw CORBA::BAD_PARAM(BAD_PARAMInvalidPortNumber, CORBA::COMPLETED_NO);
                    }
                    else
                        cdmw_root_poa_port_number = port;
                }
                else
                {
                    throw CORBA::BAD_PARAM(BAD_PARAMInvalidPortNumber, CORBA::COMPLETED_NO);
                }
            } // else ignore.



            // Init CARDAMOM Logging library (log manager) & System Properties
        try {
            // Init the SystemProperties
            (void*) Cdmw::OsSupport::SystemProperties::Init(argc, argv);
        
            // Init the Logging library
            Cdmw::Logging::LogManager::Init(argc, argv);
        } catch (const std::exception & e ) {
             std::cerr << "Cannot initialise CARDAMOM SystemProperties or Logging library! \n"
                        << "Caught exception : " << e.what() << std::endl;

             throw CORBA::BAD_PARAM(::Cdmw::OrbSupport::BAD_PARAM,CORBA::COMPLETED_NO);
        }
        
            // get System port option, and set it as property (if present)
            std::string system_port_option = 
                OS::get_option_value(argc, argv, Options::SYSTEM_PORT_OPTION);
            if (system_port_option != "no")
            {
                if (system_port_option != "yes")
                {
                    long port;
                    std::istringstream strSystemPort(system_port_option);
                    strSystemPort >> port;

                    if (strSystemPort.fail() ||
                            !(port > MIN_PORT_NUMBER && port < MAX_PORT_NUMBER))
                    {
                        throw CORBA::BAD_PARAM(BAD_PARAMInvalidPortNumber, CORBA::COMPLETED_NO);
                    }
                    else 
                    {
                        SystemProperties::Set_Property(SYSTEM_PORT_PROPERTY_NAME, 
                                                       system_port_option);
                    }
                }
                else
                {
                    throw CORBA::BAD_PARAM(BAD_PARAMInvalidPortNumber, CORBA::COMPLETED_NO);
                }
            } // else ignore.
        
        
            return UnderlyingORB::ORB_init (argc, argv, strategylist,
                                            root_poa_port_number,
                                            cdmw_root_poa_port_number,
                                            orb_identifier);
        }


        
        void
        OrbSupport::install_orb_shutdown_signal_handlers(CORBA::ORB_ptr orb,
                                                         Cdmw::OsSupport::SignalHandler* handler)
            throw(CORBA::SystemException)
        {
            if (handler == 0) {
                // create default handler
                handler = new OrbShutdownSignalHandler(orb);
            }
            
            Cdmw::OsSupport::Signals* signals =
                Cdmw::OsSupport::Signals::Instance();
            signals->register_handler(SIGTERM, handler);
            signals->register_handler(SIGINT,  handler);
            signals->register_handler(SIGQUIT, handler);
        }

        Cdmw::OsSupport::SignalHandler* 
        OrbSupport::uninstall_orb_shutdown_signal_handlers(CORBA::ORB_ptr orb)
        {
            Cdmw::OsSupport::Signals* signals =
                Cdmw::OsSupport::Signals::Instance();
                
            Cdmw::OsSupport::SignalHandler* handler =
                signals->unregister_handler(SIGTERM);
            signals->unregister_handler(SIGINT);
            signals->unregister_handler(SIGQUIT);
            
            if (dynamic_cast<OrbShutdownSignalHandler*>(handler)) {
                // default handler was created in 
                // install_orb_shutdown_signal_handlers; delete it here.
                delete handler;
                return 0;
            } else {
                return handler;
            }
        }

        
        //
        // Create a POA, child of the specified POA, with the right
        //    policies to respect the Cdmw Strategies.
        //
        PortableServer::POA_ptr
        OrbSupport::create_POA (PortableServer::POA_ptr parentPOA,
                                const char* adapter_name,
                                PortableServer::POAManager_ptr a_POAManager,
                                const CORBA::PolicyList& policies,
                                const StrategyList& strategyList)
        throw (PortableServer::POA::AdapterAlreadyExists,
               PortableServer::POA::InvalidPolicy,
               CORBA::SystemException)
        {
            return UnderlyingORB::create_POA(parentPOA, adapter_name, a_POAManager, policies,
                                             strategyList);
        }



        //
        // Make the current thread adopted by the underlying thread
        //    system
        //
        ThreadAdoptionToken*
        OrbSupport::adopt_thread()
        throw(OutOfMemoryException, InternalErrorException)
        {
#if CDMW_ORB_VDR == orbacus

            return UnderlyingORB::adopt_thread();

#else

            return NULL;

#endif
        }



        //
        // Transform a CORBA excpetion in a human readable format
        //
        std::string
        OrbSupport::exception_to_string(const CORBA::Exception& ex )
        throw(OutOfMemoryException)
        {
            // Is it a system exception?
            const CORBA::SystemException* sys
               = CORBA::SystemException::_downcast( (CORBA::Exception*) & ex );

            if (sys)
                return system_exception_to_string(*sys);
            else
                return UnderlyingORB::exception_to_string( ex );
        }

        std::string
        OrbSupport::system_exception_to_string(const CORBA::SystemException& ex)
        throw(OutOfMemoryException)
        {
           try
           {
              CORBA::ULong minor_code = ex.minor();

              if ((minor_code & VMCID_MASK) == CDMW_VMCID)
              {
                 // This is a CDMW system exception.
                 std::string ex_name = UnderlyingORB::exception_name(ex);
                 if CHECK_AND_RETURN_SYSTEM_EXCEPTION(BAD_INV_ORDER);
                 else if CHECK_AND_RETURN_SYSTEM_EXCEPTION(BAD_PARAM);
                 else if CHECK_AND_RETURN_SYSTEM_EXCEPTION(INTERNAL);
                 else if CHECK_AND_RETURN_SYSTEM_EXCEPTION(NO_IMPLEMENT);
                 else if CHECK_AND_RETURN_SYSTEM_EXCEPTION(NO_MEMORY);
                 else if CHECK_AND_RETURN_SYSTEM_EXCEPTION(NO_PERMISSION);
                 else if CHECK_AND_RETURN_SYSTEM_EXCEPTION(NO_RESOURCES);
                 else if CHECK_AND_RETURN_SYSTEM_EXCEPTION(OBJECT_NOT_EXIST);
                 else if CHECK_AND_RETURN_SYSTEM_EXCEPTION(TRANSIENT);
                 else if CHECK_AND_RETURN_SYSTEM_EXCEPTION(BAD_CONTEXT);
                 else
                    return "";
              }
              else
                 return UnderlyingORB::exception_to_string( ex );
           }
           catch (const OutOfMemoryException & )
           {
              throw;
           }
           catch (const std::bad_alloc & )
           {
              CDMW_THROW(OutOfMemoryException);
           }
        }



        void
        OrbSupport::bind_object_to_corbaloc (CORBA::ORB_ptr orb,
                                             const std::string& corbaloc_name,
                                             CORBA::Object_ptr object)
        throw (CORBA::SystemException)
        {
            UnderlyingORB::bind_object_to_corbaloc (orb, corbaloc_name, object);
        }

        void
        OrbSupport::remove_corbaloc_binding (CORBA::ORB_ptr orb,
                                             const std::string& corbaloc_name)
        throw (CORBA::SystemException)
        {
            UnderlyingORB::remove_corbaloc_binding (orb, corbaloc_name);
        }


        void
        OrbSupport::register_initial_reference (CORBA::ORB_ptr orb,
                                                const std::string& service_name,
                                                CORBA::Object_ptr object)
        throw (CORBA::SystemException)
        {
            UnderlyingORB::register_initial_reference (orb, service_name, object);
        }


        void
        OrbSupport::ORB_cleanup(CORBA::ORB_ptr orb)
        throw(CORBA::BAD_INV_ORDER)
        {
            UnderlyingORB::ORB_cleanup(orb);
        }
        
        void                
        OrbSupport::set_endpoint_selector_factory (const std::string& endpoint_selector_factory)
        {
            UnderlyingORB::set_endpoint_selector_factory(endpoint_selector_factory);
        }
        
        
        std::string
        OrbSupport::get_repository_id_from_ior(CORBA::ORB_ptr orb, CORBA::Object_ptr object)
            throw( CORBA::SystemException )
        {
            return UnderlyingORB::get_repository_id_from_ior(orb, object);
        }
       bool
        OrbSupport::is_type_id_valid(const char * type_id)
            throw(  )
        {
           // OMG IDL Format
           // 
           //     IDL:identifier[/identifier]...:version-number 
           //  
           //  version-number contains a version number with the following format:
           //
           //     major.minor
           //
           bool result = false;
           try {
              // type_id shall be greater than 'IDL:' + ':X.Y' (8 characters)
              std::string repid(type_id);
              if (repid.length() > 8) {
                 // Check for common pitfalls 'IDL::' and 'IDL://' !
                 std::string pitfall1("IDL::"), pitfall2("IDL://");
                 if ((repid.find(pitfall1,0) == std::string::npos ) &&
                     (repid.find(pitfall2,0) == std::string::npos )) {
                    std::string prefix=repid.substr(0,4);
                    if (prefix == "IDL:") {
                       // check for version-number
                       std::string r = repid.substr(4);
                       std::string::size_type loc = r.find( ":", 0 );
                       if( loc != std::string::npos ) {
                          loc = r.find(".",loc);
                          if( loc != std::string::npos ) {
                             // We don't check major and minor for the moment!
                             result = true;
                          }
                       }
                    }
                 }
              }
           } catch (...) {
              // ignore
           }
           return result;
           
        }
        
        
        
        
        
    } // End namespace OrbSupport
} // End namespace Cdmw

