/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

#include "testlargesequence/TestLargeSequenceDriver.hpp"

#include <fstream>
#include <sstream>
#include <string>

#include "Foundation/orbsupport/StrategyList.hpp"
#include "Foundation/orbsupport/CORBA.hpp"
#include "Foundation/orbsupport/OrbSupport.hpp"
#include "Foundation/commonsvcs/time/Util.hpp"
#include "Foundation/common/System.hpp"
#include "Foundation/testutils/CORBATestManager.hpp"
#include "Foundation/ossupport/OS.hpp"

#include "testlargesequence/TestLargeSequence.stub.hpp"
#include "testlargesequence/TestLargeSequenceCommon.hpp"
#include "testlargesequence/TestLargeSequenceServerProcess.hpp"

namespace Cdmw
{
    namespace OrbSupport
    {
        CPPUNIT_TEST_SUITE_REGISTRATION( TestLargeSequenceDriver );

        TestLargeSequenceDriver::TestLargeSequenceDriver()
        {
            m_process_name = PROCESS_NAME;
        }

        TestLargeSequenceDriver::~TestLargeSequenceDriver() throw()
        {
        }

        void TestLargeSequenceDriver::do_tests()
        {
            const std::string IOR_FILE = IOR_FILE_NAME;
            
            StrategyList strategies;
            strategies.add_OrbThreaded();
            strategies.add_PoaThreadPerRequest();
    
            int argc2 = 0;
            CORBA::ORB_var orb = OrbSupport::ORB_init(argc2, NULL, strategies);
                    
            // Process creation
            TEST_INFO( "Starting a CORBA server in another process" );
            OsSupport::OS::ProcessId pid =
                OsSupport::OS::create_process( m_process_name, "--server");
            // Sleeping a while to wait the process creation
            int timescale = Cdmw::TestUtils::get_timescale();
            OsSupport::OS::sleep(timescale*3000);
            TEST_INFO( "Server created with pid: " );
            TEST_INFO( pid );
            
            // time values for performance measuring
            TimeBase::TimeT time_diff_ior, time_diff_corbaloc;
                            
            // Accessing the server via corbaloc
            try
            {
                std::stringstream server_corbaloc;
                server_corbaloc << "corbaloc::localhost:"
                    << TestLargeSequenceServerProcess::SERVER_PORT
                    << "/"
                    << TestLargeSequenceServerProcess::SERVER_CORBALOC_NAME;
                TEST_INFO( "Try to contact server via corbaloc: " )
                TEST_INFO( server_corbaloc.str() );
                CORBA::Object_var obj = orb->string_to_object( server_corbaloc.str().c_str() );
                ::test::TestLargeSequence_var server1 =
                    ::test::TestLargeSequence::_narrow( obj.in() );
                CPPUNIT_ASSERT( ! CORBA::is_nil(server1.in()) );

                do_large_sequence_test(server1, time_diff_corbaloc);
                TEST_INFO( "Time elapsed accessing via corbaloc (msec): " );
                TEST_INFO( time_diff_corbaloc/10000.0f );
               
                CPPUNIT_ASSERT(true);
            }
            catch (CORBA::Exception& e)
            {
                CPPUNIT_ASSERT(false);
                TEST_INFO("Exception: ");
                TEST_INFO(OrbSupport::exception_to_string(e));
            }
            
            // Accessing the server via IOR
            TEST_INFO( "Getting back the IOR from the file" );
            std::ifstream iorFile( IOR_FILE.c_str() );
            std::string iorStr;
            iorFile >> iorStr;
            iorFile.close();

            TEST_INFO( "Creating the server stub " );
            CORBA::Object_var obj2 = orb->string_to_object( iorStr.c_str() );
            ::test::TestLargeSequence_var server2 =
                ::test::TestLargeSequence::_narrow( obj2.in() );
            CPPUNIT_ASSERT( ! CORBA::is_nil(server2.in()) );

            do_large_sequence_test(server2, time_diff_ior);
            TEST_INFO( "Time elapsed accessing via IOR (msec): " );
            TEST_INFO( time_diff_ior/10000.0f );
             
            // compute the ratio between corbaloc and IOR access time 
            float f_time_diff_ior = static_cast<float>(time_diff_ior);
            float ratio = 0.0f;
            if (f_time_diff_ior != 0.0f) {
                 ratio = static_cast<float>(time_diff_corbaloc) / f_time_diff_ior;
                 TEST_INFO( "Access time ratio:" );
                 TEST_INFO( ratio );
                 CPPUNIT_ASSERT( MAX_RATIO >= ratio );
            }
            else {
                 TEST_INFO( "Ratio unavailable" );
                 CPPUNIT_ASSERT( false );
            }
            
            orb->shutdown(false);
            OrbSupport::OrbSupport::ORB_cleanup(orb.in());
            TEST_INFO( "Destroy ORB" );
            orb->destroy();
            
            TEST_INFO( "Killing the server" );
            OsSupport::OS::kill_process( pid );

            TEST_INFO( "Removing IOR files" );
            OsSupport::OS::unlink(IOR_FILE);
        }
        
        void TestLargeSequenceDriver::do_large_sequence_test( ::test::TestLargeSequence_var server, TimeBase::TimeT& time_diff )
        {
            TimeBase::TimeT time_start = Cdmw::CommonSvcs::Time::Util::get_current_time();
            
            // Fill a sequence of long
            ::test::TestLargeSequence::SequenceOfLong in_sol;
            in_sol.length(SEQUENCE_LENGTH);
            for (unsigned int i=0;i<in_sol.length();i++) in_sol[i] = i;
            
            ::test::TestLargeSequence::SequenceOfLong* out_sol;
            
            server->processSequences(in_sol, out_sol);

            TimeBase::TimeT time_end = Cdmw::CommonSvcs::Time::Util::get_current_time();
            time_diff = time_end - time_start;
            
            delete out_sol;
        }

    } // End namespace OrbSupport

} // End namespace Cdmw

