/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#ifndef INCL_DEFAULTSHAREDOBJECT_OS_HPP
#define INCL_DEFAULTSHAREDOBJECT_OS_HPP

#include <Foundation/common/MacroDefs.hpp>
#ifdef CDMW_POSIX
//#ifndef AIX

#include <string>
#include <dlfcn.h>
#include <Foundation/common/Exception.hpp>
#include <Foundation/osthreads/Mutex.hpp>
#include <Foundation/ossupport/SharedObjectHandle.hpp>

namespace Cdmw
{

    namespace OsSupport
    {

        class DefaultSharedObjectHandle
        {

            public :
                typedef void *OsShlibHandle;
                typedef void *FunctionHandle;

        };

        class DefaultSharedObject : public SharedObjectHandle_T<DefaultSharedObjectHandle>
        {

            public:
                /**
                * Purpose:
                * <p> Open a dynamic library that will managed by the returned
                * object.
                * @param name of the dynamic library
                * @param flag the value can be either RTLD_LAZY or RTLD_NOW: 
                * when RTLD_NOW is specified or the environment variable 
                * LD_BIND_NOW is set to a non-empty string, all undefined 
                * symbols in the library are resolved before dlopen returns; 
                * otherwise binding is lazy i.e. symbol values are first 
                * resolved when needed.
                *
                * @return the built object wrapping the dynamic library;
                * if this cannot be done, NULL is returned.  
                */
                static DefaultSharedObject* dlopen(const char* libname, int flag = RTLD_LAZY);

                /**
                * Purpose:
                * <p> Close the dynamic library
                * @return 0 if the library is closed without problem, if not 1 
                */
                int dlclose(void);

                /**
                * Purpose:
                * <p> Search a function in the dynamic library.
                *
                * @param symname symbol to lookup corresponding to the function
                * @return a handler on this function if found, else NULL
                */
                DefaultSharedObjectHandle::FunctionHandle dlsym(const char* symname);

                /**
                * Purpose:
                * <p> return a description of the error generated by dlopen,
                * dlclose and dlsym function
                * 
                * @return a string with a description of the error
                */
                static std::string dlerror(void);

                /**
                * Purpose:
                * <p> Writes the handle of the library in the specified stream.
                * @param os output stream
                */
                void print(std::ostream& os);

                //
                // Equality and inequality operation
                //

                /**
                 * Checks whether the shared library exists.
                 * @return true if a library was opened, false otherwise.
                 */
                bool
                is_null()
                {
                    return m_OS_handle == NULL;
                }

                bool
                operator==(const DefaultSharedObject & rhs) const
                {
                    return m_OS_handle == rhs.m_OS_handle;
                }

                bool
                operator!=(const DefaultSharedObject & rhs) const
                {
                    return m_OS_handle != rhs.m_OS_handle;
                }

                bool
                operator!() const
                {
                    return m_OS_handle == NULL;
                }

                operator bool() const
                {
                    return m_OS_handle != NULL;
                }

            private:
                //
                // Constructor
                //

                DefaultSharedObject();

                //
                // Handle of the library
                //

                OsShlibHandle m_OS_handle;

        };


    } // End namespace ossupport

} // End namespace Cdmw



//#endif
#endif
#endif
