/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#ifndef INCL_OSSUPPORT_SYSTEM_PROPERTIES_HPP
#define INCL_OSSUPPORT_SYSTEM_PROPERTIES_HPP


#include <Foundation/common/Properties.hpp>
#include <Foundation/osthreads/ReaderWriterLock.hpp>

namespace Cdmw
{
namespace OsSupport
{

    
/**
 *Purpose:
 *<p>
 * Handle the System Properties
 *<p>
 *Features:
 *<p>
 * Thread safe.
 *
 *@see Properties
 */
    
    class SystemProperties : public Common::Properties
    {
        
    public:


        /**
         * Purpose:
         * <p> Initialise system properties from command line argumens, properties file or environment variables.
         * The following precedence rules apply when a property is present more than once:
         * 1) Command line argument: -D<name>=<value>
         * 2) Properies file: -DCdmwPropertyFile=filename
         * 3) Environment variable: NAME=value
         * 
         *@param argc number of elements in argv ("arg cout")
         *@name argv array of character pointers to the arguments
         *@param envp environment pointer array
         *
         *@return A pointer to the SystemProperties object
         *
         */
        static Common::Properties*
        Init(int argc, char *argv[])
            throw (BadParameterException);
        
        /**
         * Purpose:
         * <p> Initialise system properties from a predefined properties set.
         * 
         *@param prop the original Properties object
         *
         *@return A pointer to the SystemProperties object
         *
         *@see Cdmw::Common::Properties
         */
        static Common::Properties*
        Init(Common::Properties* prop);
        
        /**
         * @return the instance singleton of SystemProperties.
         */
        static Common::Properties*
        Instance();

        // Base class operations with thread-safety

        /**
         * Purpose:
         * <p> Set a value. Thread-safe.
         * 
         *@param name a the name of the value
         *@param value the value             
         *
         *@return void
         *
         */
        void set_property(const std::string& name, const std::string& value);

        /**
         * Purpose:
         * <p> Get a value. Thread-safe.
         * 
         *@param name the name of the value
         *@param default_value a value provided by default
         *
         *@return the value or if not found default_value
         *
         */
        std::string get_property(const std::string& name, const std::string& default_value) const;

        /**
         * Purpose:
         * <p> Return a list of all the properties 
         * saved. Thread-safe.
         *@param names list of all the propety names
         *
         *@return void
         *
         */
        void property_names(PropertyNames& names) const;        

        /**
         * Purpose:
         * <p> Load properties from input stream. Format is \<name\>=\<value\>; comments start with
         *  '!' or '#' character.
         * Thread-safe.
         *
         * @param is input stream
         */
        void load(std::istream& is)
            throw (BadParameterException);        

        /**
         * Purpose:
         * <p> Writes properties in output stream. 
         * Format is \<name\>=\<value\>
         * Thread-safe.
         *
         * @param os output stream
         */
        void store(std::ostream& os) const;
        
        /**
         * Purpose:
         * <p> Adds a new property composed of a unique name and the value.
         * 
         *@param name a the name of the value
         *@param value the value
         *
         *@return void
         *
         */
        static void
        Set_Property(const std::string& name, 
                     const std::string& value);
        
        /**
         * Purpose:
         * <p> Adds a new property composed of a unique name and the 
         * integer value.
         * 
         *@param name a the name of the value
         *@param value the value
         *
         *@return void
         *
         */
        static void
        Set_Property(const std::string& name, 
                     int value);
        
        /**
         * Purpose:
         * <p> Get a value 
         * 
         *@param name the name of the value
         *@param default_value a value provided by default
         *
         *@return the value or if not found default_value
         *
         */
        static std::string
        Get_Property(const std::string& name, const std::string& default_value = "");
        
        /**
         * Purpose:
         * <p> Get a value 
         * 
         *@param name the name of the value
         *@param default_value a value provided by default
         *
         *@return the value or if not found default_value
         *
         */
        static int
        Get_Property(const std::string& name, int default_value);
       

        // Names of some Cdmw Properties
        static const std::string CDMW_THREADS_STACKSIZE_NAME;

        static const std::string ORB_THREADS_STACKSIZE_NAME;

        static const std::string FTSTATEMNGR_SCHED_THREADS_STACKSIZE_NAME;

        static const std::string FTSTATEMNGR_TIMER_THREAD_STACKSIZE_NAME;

        static const std::string FLUSHAREAMNGR_THREAD_STACKSIZE_NAME;
 
    protected:
        SystemProperties();
        SystemProperties(const Common::Properties*);

        /**
         * Purpose:
         * <p> Initialise system properties from command line argumens, properties file or environment variables.
         * The following precedence rules apply when a property is present more than once:
         * 1) Command line argument: -D<name>=<value>
         * 2) Properies file: -DCdmwPropertyFile=filename
         * 3) Environment variable: NAME=value
         *
         * NOTA: This operation is not protected against concurrent access!
         *
         *@param argc number of elements in argv ("arg cout")
         *@name argv array of character pointers to the arguments
         *@param envp environment pointer array
         *
         */
        void initialise(int argc, char *argv[], char *envp[])
            throw (BadParameterException);        
        
    private:
        SystemProperties(const SystemProperties&);
        SystemProperties operator=(const SystemProperties&);
        
        static Common::Properties* M_properties;
        
        static ReaderWriterLock M_properties_mutex;
        
    }; //End of SystemProperties

} // End of namespace OsSupport
} // End of namespace Cdmw

#endif

