/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#ifndef INCL_OSSUPPORT_PROCESS_MONITOR_HPP
#define INCL_OSSUPPORT_PROCESS_MONITOR_HPP


#include "Foundation/ossupport/OS.hpp"
#include "Foundation/ossupport/SignalHandler.hpp"
#include "Foundation/ossupport/ProcessCallback.hpp"
#include "Foundation/osthreads/Mutex.hpp"
#include "Foundation/common/Exception.hpp"


#include <map>
#include <iostream>
#include <vector>
#include <unistd.h>  


/**
* Root namespace for CDMW runtime.
*/

namespace Cdmw
{
    /**
    * Namespace for CDMW OsSupport
    */

    namespace OsSupport
    {


        /**
        *Purpose:
        *<p> This class is in charge to create process and to handle
        * SIGCHLD signals in order to execute ending callbacks
        *
        *Features:
        *<p> Thread safe
        *
        *@see  
        *
        */

        class ProcessMonitor: public SignalHandler
        {

            public:

                /**
                 * Purpose:
                 * <p> Constructor
                 *
                 */
                ProcessMonitor()
                throw();

                /**
                 * Purpose:
                 * <p> Destructor
                 * 
                 */
                virtual
                ~ProcessMonitor()
                throw();

                /**
                 * Purpose:
                 * <p> Register this object for SIGCHLD handling.
                 */
                void
                register_for_SIGCHLD();
                      
                /**
                 * Purpose:
                 * <p> Create a new process from the specified executable
                 *
                 * @param path path to the executable
                 * @param args arguments to pass to the process
                 * @param home directory where the process must be started
                 * @param creationCallback the callback that will be executed just after
                 * the process creation.
                 * @param endingCallback the callback that will be executed just after
                 * the process ending.
                 *
                 * @return the process id of the newly created process
                 *
                 * @exception BadParameterException
                 * @exception OutOfMemoryException
                 * @exception InternalErrorException
                 */
                OS::ProcessId
                create_process (const std::string& path, 
                                const std::string& args,
                                const std::string& home,
                                ProcessCallback* creationCallback,
                                ProcessCallback* endingCallback)
                throw (BadParameterException,
                       OutOfMemoryException,
                       InternalErrorException);
                
                
                /**
                 * Purpose:
                 * <p> Create a new process from the specified executable
                 *
                 * @param path path to the executable
                 * @param args arguments to pass to the process
                 * @param home directory where the process must be started
                 * @param env  a vector of string contaning the enviroment variables that will be added
                 * @param core_size the maximum size limit imposed on core dump creation,
                 * @param run_as_root a flag to run process with root uid and guid 
                 * @param creationCallback the callback that will be executed just after
                 * the process creation.
                 * @param endingCallback the callback that will be executed just after
                 * the process ending.
                 *
                 * @return the process id of the newly created process
                 *
                 * @exception BadParameterException
                 * @exception OutOfMemoryException
                 * @exception InternalErrorException
                 */
                OS::ProcessId
                create_process (const std::string& path, 
                                const std::string& args,
                                const std::string& home,
                                const std::vector<std::string> env,
                                const int core_size,
                                const bool run_as_root,
                                ProcessCallback* creationCallback,
                                ProcessCallback* endingCallback)
                throw (BadParameterException,
                       OutOfMemoryException,
                       InternalErrorException);      
                
                
                /**
                 * Purpose:
                 * <p> Unregister an ending callback previously registered
                 * during the creation of process with specified pid.
                 *
                 *@param pid PID of the process for which the ending 
                 * callback has to be unregistered.
                 *
                 *@return the unregistered ending callback.
                 *
                 */
                ProcessCallback*
                unregister_ending_callback(OS::ProcessId pid)
                throw();
                

                
                /**
                 * Purpose:
                 * <p> Unregister all ending callbacks previously registered
                 * during process creations.
                 *
                 */
                void
                clear_ending_callbacks()
                throw();
                       
                
                /**
                 * Purpose:
                 * <p> This method is called when a signal is received and if 
                 * this handler is registered for this signal
                 *
                 */
                int 
                handle_signal(int sig) 
                throw();

                
                /**
                * Purpose:
                * <p> Sets the attributes for created processes.
                *
                * @param uid the user id
                * @param gid the group id
                */
                void
                set_process_attributes(uid_t uid, gid_t gid)
                throw();
                
                /**
                 * Purpose:
                 * <p> Get the uid attribute
                 *
                 * @return  the uid
                 */
                uid_t 
                get_uid()
                throw();
                
                /**
                 * Purpose:
                 * <p> Get the gid attribute
                 *
                 * @return  the uid
                 */
                gid_t 
                get_gid()
                throw();
                
                
                
            private:

                /**
                 * Purpose:
                 * <p> Create a new process from the specified executable
                 *
                 * @param path path to the executable
                 * @param args arguments to pass to the process
                 * @param home directory where the process must be started
                 * @param endingCallback the callback that will be executed just after
                 * the process ending.
                 *
                 * @return the process id of the newly created process
                 *
                 * @exception BadParameterException
                 * @exception OutOfMemoryException
                 * @exception InternalErrorException
                 */
                OS::ProcessId
                create_process (const std::string& path, 
                                const std::string& args,
                                const std::string& home,
                                ProcessCallback* endingCallback)
                throw (BadParameterException,
                       OutOfMemoryException,
                       InternalErrorException);
                
                       
                /**
                * Purpose:
                 * <p> Create a new process from the specified executable
                 *
                 * @param path path to the executable
                 * @param args arguments to pass to the process
                 * @param home directory where the process must be started
                 * @param env  a vector of string containg the enviroment variable that will be set
                 * @param core_size the maximum size limit imposed on core dump creation,
                 * @param run_as_root a flag to run process with root uid and guid,
                 * 
                 * @param endingCallback the callback that will be executed just after
                 * the process ending.
                 *
                 * @return the process id of the newly created process
                 *
                 * @exception BadParameterException
                 * @exception OutOfMemoryException
                 * @exception InternalErrorException
                 */
                OS::ProcessId
                create_process (const std::string& path, 
                                const std::string& args,
                                const std::string& home,
                                const std::vector<std::string> env,
                                const int core_size,
                                const bool run_as_root,
                                ProcessCallback* endingCallback)
                throw (BadParameterException,
                       OutOfMemoryException,
                       InternalErrorException);
                       
                
                       
                /**
                * Purpose:
                * <p> replace the process image with a new image
                *
                * @param path to the executable file
                * @param args arguments to be passed to the process
                * 
                */
                void
                exec_process(const std::string& path, 
                             const std::string& args)
                throw ();

                /**
                 * Purpose:
                 * <p> replace the process image with a new image
                 *
                 * @param path to the executable file,
                 * @param args arguments to be passed to the process,
                 * @param env vector of environment variables
                 *  defined in the process scope,
                 * @param core_size  the maximum size limit 
                 *  imposed on core dump writting,
                 * 
                 */
                void
                exec_process(const std::string& path, 
                             const std::string& args,
                             const std::vector<std::string> env,
                             const int core_size)
                throw ();
                
                typedef std::map<OS::ProcessId, ProcessCallback*> CallbackMap;
                
                CallbackMap m_ending_callbacks;

                OsSupport::Mutex m_ending_callbacks_mutex;
                
                int m_uid;
                int m_gid;
                
                bool m_registered_for_SIGCHLD;
                
        }; // End class ProcessMonitor

    }  // End namespace OsSupport
}  // End namespace Cdmw



#endif // INCL_OSSUPPORT_PROCESS_MONITOR_HPP

