/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


// To get ::realpath() on sparc environment
#ifdef sun
#    define __EXTENSIONS__
#endif


#include "Foundation/ossupport/OS.hpp"
#include "ossupport/ProcessMonitor.hpp"
#include "ossupport/Which.hpp"


#include <time.h>
#include <cerrno>
#include <cstdio>
#include <iostream>
#include <vector>


#if defined (CDMW_POSIX)
#    include <sys/types.h>
#    include <sys/utsname.h>

// FIXME: gettimeofday not available under posix
#    undef _POSIX_C_SOURCE
#    include <stdlib.h>
#    include <unistd.h>
#    include <sys/time.h>
#    include <signal.h>

#elif defined(_WIN32)

#   include <windows.h>
#   include <io.h>
#   include <winsock2.h>
#   include <direct.h>
#endif

#include "Foundation/common/Assert.hpp"
#include "Foundation/common/String.hpp"
#include "Foundation/osthreads/MutexGuard.hpp"
#include "Foundation/osthreads/Barrier.hpp"
#include "Foundation/ossupport/ProcessCallback.hpp"
#include "Foundation/ossupport/SharedObjectHandle.hpp"
#include "Foundation/osthreads/Thread.hpp"

#include <csignal>


#ifdef linux
extern "C" char *realpath (const char * __name,
                                 char * __resolved);
// libC version
extern "C" char *dirname (char *__path);
// POSIX version
extern "C" char *__xpg_basename (char *__path);


#endif

namespace Cdmw
{

    namespace OsSupport
    {
        /**
        *   @param formatString Format string according to strftime(); format
        *   @return formatted current LOCAL date string
        */
        CDMW_DLL_EXPORT_1
        std::string get_fmt_current_localtime(char* formatString)
        {
            char pTimeBuffer[20];
            Cdmw::OsSupport::OS::get_fmt_time(
                pTimeBuffer,                    //Buffer to return formatted time
                20,                             //Size of Buffer
                Cdmw::OsSupport::OS::get_time().seconds, //Time to format (current time in seconds)
                formatString,            //Format string
                Cdmw::OsSupport::OS::LOCAL_TIME); //GMT or LOCAL time
    
            return std::string(pTimeBuffer);
        }

        ProcessMonitor OS::M_processMonitor;

        /**
        *Implementation notes:  none 
        *<p>
        *Portability issues:  none 
        *<p>
        */

#ifdef sun
        typedef DefaultSharedObject EffectiveSharedObject;
#elif linux
        //typedef DefaultSharedObject EffectiveSharedObject;
        typedef LinuxSharedObject EffectiveSharedObject;
#elif AIX
        typedef AIXSharedObject EffectiveSharedObject;
#elif _WIN32
        typedef Win32DllObject EffectiveSharedObject;
#endif



#if defined(_WIN32)


        //
        // Retreive from a Process ID, a win32 handle
        //
        HANDLE
        get_win32_process_handle(OS::ProcessId pid)
        throw ( PermissionDeniedException,
                BadParameterException,
                InternalErrorException
              )
        {

            // We need to get the handle to the process
            // TODO: maybe we can only require right we need ?
            HANDLE process_handle = ::OpenProcess (PROCESS_ALL_ACCESS,
                                                   FALSE,
                                                   pid);

            if (process_handle == NULL)
            {

                switch (::GetLastError())
                {

                        case ERROR_ACCESS_DENIED:
                        CDMW_THROW( PermissionDeniedException );
                        break;


                        // FIXME
                        // I don't know the error code returned, when a pid is not
                        // valid, so now I suppose it is the current problem

                        default:
                        CDMW_THROW2(BadParameterException, "processId", "No process with this id");
                        break;

                }

            }

            return process_handle;

        }

#endif


       void
       OS::get_option_values (int argc, char** argv, 
                              const std::string& option,
                              std::list<std::string>& values)
          throw (OutOfMemoryException,
                 InternalErrorException)
       {
            try
            {
                int iArg(1);


                // We scan all arguments to find the matching with option

                while (iArg < argc)
                {

                    std::string arg (argv[iArg]);

                    // We check if in the current argument we have, the option

                    if (arg.find(option) != std::string::npos)
                    {

                        std::string::size_type pos_sep = arg.find('=');

                        std::string option_value = "";

                        // We have no separator the option is only here so
                        //    we return yes, or if the equal is not followed
                        //    by something

                        if ( ( pos_sep == std::string::npos ) || (pos_sep + 1 == (arg.size()) ))
                        {

                            option_value = "yes";

                            // Else we return the value
                        }
                        else
                        {

                            option_value = arg.substr(pos_sep + 1);
                        }
                        values.push_back(option_value);

                    }

                    iArg++;
                }

            }
            catch ( const std::bad_alloc& )
            {

                CDMW_THROW( OutOfMemoryException );

            }
            catch ( ... )
            {

                CDMW_THROW( InternalErrorException );

            }

       }

        //
        // Return the of the option specified in the option parameter
        //    if the option is not on the command line "no" is returned
        //
        std::string
        OS::get_option_value(int argc, char** argv, const std::string& option)
        throw ( OutOfMemoryException,
                InternalErrorException )
        {
            std::string option_value = "no";

            try
            {
                int iArg(1);


                // We scan all arguments to find the matching with option

                while (iArg < argc)
                {

                    std::string arg (argv[iArg]);

                    // We check if in the current argument we have, the option

                    if (arg.find(option) != std::string::npos)
                    {

                        std::string::size_type pos_sep = arg.find('=');

                        // We have no separator the option is only here so
                        //    we return yes, or if the equal is not followed
                        //    by something

                        if ( ( pos_sep == std::string::npos ) || (pos_sep + 1 == (arg.size()) ))
                        {

                            option_value = "yes";

                            // Else we return the value
                        }
                        else
                        {

                            option_value = arg.substr(pos_sep + 1);
                        }

                    }

                    iArg++;
                }

            }
            catch ( const std::bad_alloc& )
            {

                CDMW_THROW( OutOfMemoryException );

            }
            catch ( ... )
            {

                CDMW_THROW( InternalErrorException );

            }

            return option_value;
        }




        //
        // Make sleep the calling thread during X milliseconds
        //
        void
        OS::sleep(unsigned int milliseconds)
        throw( InterruptedException,
               InternalErrorException)
        {
            Thread::sleep(milliseconds);
        }

        // 
        // Extended version
        //
        void
        OS::sleep(unsigned int seconds,
              unsigned int nanoseconds)
            throw (std::range_error)
        {
            timespec ts;
            ts.tv_sec  = seconds;
            ts.tv_nsec = nanoseconds;

            if (::nanosleep(&ts, 0) != 0)
                throw std::range_error(strerror(errno));
        }

        //
        // Return true if the specified file exists
        //
        bool
        OS::file_exists(const std::string& path)
        throw()
        {
            bool exists(false);
            
#ifdef _WIN32

            if ( 0 == ::_access (path.c_str(), 00))
            {
                exists = true;
            }

#else
            if ( 0 == ::access (path.c_str(), F_OK))
            {
                exists = true;
            }

#endif

            return exists;
        }




        //
        // Delete the specified file
        //
        void
        OS::unlink(const std::string& path)
        throw( PermissionDeniedException,
               InterruptedException,
               BadParameterException,
               InternalErrorException)
        {

            if ( 0 != ::unlink( path.c_str()) )
            {

#       if defined(CDMW_POSIX)

                switch (errno)
                {

                        // The file is a mount point

                        case EBUSY:
                        // No break;

                        // The file is a directory

                        case EPERM:
                        // No break;

                        // The file is being executed

                        case ETXTBSY:
                        // No break;

                        // The file system of the file is read-only

                        case EROFS:
                        // No break;

                        // The user is not allowed

                        case EACCES:

                        CDMW_THROW(PermissionDeniedException);
                        break;

                        case ENOENT:
                        CDMW_THROW2(BadParameterException, "path", "file not exists");
                        break;

                        // Interrupted by a signal

                        case EINTR:
                        CDMW_THROW(InterruptedException);
                        break;

                        // Too many symbolic link

                        case ELOOP:
                        CDMW_THROW2(BadParameterException, "path", "too many symbolic link");
                        break;

                        // Path exceeds MAX_PATH

                        case ENAMETOOLONG:
                        CDMW_THROW2(BadParameterException, "path", "path too long");
                        break;


                        case ENOLINK:
                        CDMW_THROW2(BadParameterException, "path", "file on a remote machine down");
                        break;

                        case ENOTDIR:
                        CDMW_THROW2(BadParameterException, "path", "a path component is not a directory");
                        break;

                        // The path point on an illegal adress

                        case EFAULT:
                        // No break;

                        default:
                        CDMW_THROW(InternalErrorException);
                        break;
                }

#       elif defined(_WIN32)

                switch (errno)
                {

                        // The user is not allowed

                        case EACCES:

                        CDMW_THROW(PermissionDeniedException);
                        break;

                        case ENOENT:
                        CDMW_THROW2(BadParameterException, "path", "file not exists");
                        break;

                        default:
                        CDMW_THROW(InternalErrorException);
                        break;
                }

#       endif

            }
        }


        void
        OS::chdir(const std::string& path)
        throw( BadParameterException,
               PermissionDeniedException,
               InterruptedException,
               InternalErrorException)
        {

#if defined(CDMW_POSIX)

            int status = ::chdir (path.c_str());

            if (status != 0)
            {

                switch (errno)
                {

                        case EACCES:
                        CDMW_THROW(PermissionDeniedException);
                        break;

                        case EINTR:
                        CDMW_THROW(InterruptedException);
                        break;

                        case ENAMETOOLONG:
                        // No break

                        case ELOOP:
                        CDMW_THROW2(BadParameterException, "path", "path too long");
                        break;

                        case ENOTDIR:
                        // No break

                        case ENOLINK:
                        // No break

                        case ENOENT:
                        CDMW_THROW2(BadParameterException, "path", "invalid path");
                        break;


                        case EIO:
                        // No break

                        case EFAULT:
                        // No break

                        default:
                        CDMW_THROW(InternalErrorException);
                        break;
                }

            }

#elif defined(_WIN32)
            int status = ::_chdir (path.c_str());

            if (status != 0)
            {
                CDMW_THROW2(BadParameterException, "path", "invalid path");
            }

#endif

        }

        void
        OS::mkdir(const std::string& path, unsigned short mode )
        throw( BadParameterException,
               PermissionDeniedException,
               InterruptedException,
               InternalErrorException)
        {

#if defined(CDMW_POSIX)

            int status = ::mkdir (path.c_str(), mode);

            if (status != 0)
            {

                switch (errno)
                {
                        case EROFS:
                        // No break

                        case ENOSPC:
                        // No break

                        case EACCES:
                        CDMW_THROW(PermissionDeniedException);
                        break;

                        case EEXIST:
                        // ignore
                        break;

                        case EINTR:
                        CDMW_THROW(InterruptedException);
                        break;

                        case ENAMETOOLONG:
                        // No break

                        case ELOOP:
                        CDMW_THROW2(BadParameterException, "path", "path too long");
                        break;

                        case ENOTDIR:
                        // No break

                        case ENOENT:
                        CDMW_THROW2(BadParameterException, "path", "invalid path");
                        break;

                        case EFAULT:
                        // No break

                        default:
                        CDMW_THROW(InternalErrorException);
                        break;
                }

            }

#elif defined(_WIN32)
            int status = ::_mkdir (path.c_str(), mode);

            if (status != 0)
            {
                CDMW_THROW2(BadParameterException, "path", "invalid path");
            }

#endif

        }

        
        std::string
        OS::basename(const std::string& path)
        throw( InterruptedException)
        {

#if defined(CDMW_POSIX)
            if ( ::strcmp(path.c_str(),"") == 0 ) {
                return std::string(".");
            } else {
                char* p =::strdup(path.c_str()); // use of malloc
                const char* pp = ::__xpg_basename(p);
                std::string result (pp);
                free(p); // free content allocated by malloc
                return result;
            }
#else
#   error Cdmw: not supported platform
#endif
        }

        std::string
        OS::dirname(const std::string& path)
        throw( InterruptedException)
        {

#if defined(CDMW_POSIX)
            if ( ::strcmp(path.c_str(),"") == 0 ) {
                return std::string(".");
            }
            else if (path == "//") {
                // dirname returns for "//" a value of "//", but 
                // a "/" should be returned  
                return std::string("/");
            } else {
                char* p =::strdup(path.c_str()); // use of malloc
                const char* pp = ::dirname(p);
                std::string result (pp);
                free(p); // free content allocated by malloc
                return result;
            }
#else
#   error Cdmw: not supported platform
#endif
        }

        std::string
        OS::get_absolute_path (const std::string& path)
        throw( BadParameterException,
               PermissionDeniedException,
               NotExistException,
               InternalErrorException)
        {
            
#if defined(CDMW_POSIX)

            char buffer[PATH_MAX + 1];

            // The System call doesn't guarantee the NULL terminator, so
            //  we must insert it

            if (::realpath( path.c_str(), buffer) == NULL)
            {

                switch (errno)
                {

                        case EACCES:
                        CDMW_THROW(PermissionDeniedException);
                        break;

                        case ENAMETOOLONG:
                        // No break

                        case ELOOP:
                        CDMW_THROW2(BadParameterException, "path", "path too long");
                        break;

                        case ENOTDIR:
                        // No break

                        case ENOLINK:
                        // No break

                        case ENOENT:
                        CDMW_THROW(NotExistException);
                        break;


                        case EIO:
                        // No break

                        case EFAULT:
                        // No break

                        default:
                        CDMW_THROW(InternalErrorException);
                        break;
                }


            }

            return buffer;


#elif defined(_WIN32)

            char buffer[_MAX_PATH];

            if (_fullpath( buffer, path.c_str(), _MAX_PATH) != NULL)
            {

                return buffer;

            }
            else
            {

                CDMW_THROW2(BadParameterException, "path", "invalid path");

            }

#else
#   error Cdmw: not supported platform
#endif


        }


        //
        // Wait until the completion of the specified process
        //
        OS::StatusProcess
        OS::wait_process (OS::ProcessId processId)
        throw ( BadParameterException,
                InterruptedException,
                InternalErrorException)
        {

            return 0;

            /*
            #if defined(CDMW_POSIX)
             
                int status(0);
                
                pid_t status_waitpid = ::waitpid(    processId, 
                                                     &status, 
                                                     0);
                if ( static_cast<pid_t>(-1) ==  status_waitpid ) {
                    
                    switch ( errno ) {
             
                     case ECHILD:
                      CDMW_THROW2(BadParameterException,"processId", "Process does not exist");
                   break;
              
                  case EINTR:
                      CDMW_THROW(InterruptedException);
                      break;
                  
                  // Option was not valid, impossible we get corrupted
                  case EINVAL:
                      // No break;
               
                  default:
                      CDMW_THROW(InternalErrorException);
                   break;
                  
                    };
             }
             
             return static_cast<OS::StatusProcess>(status);
                
             
             
            #elif defined(_WIN32)      
             
             
                HANDLE process_handle = get_win32_process_handle( processId );
                
                DWORD status_wait = ::WaitForSingleObject( process_handle,INFINITE );
             
                if ( status_wait != WAIT_OBJECT_0 ) {
             
                    CDMW_THROW(InternalErrorException);
             
                }
             
                DWORD exitCode;
                if ( ::GetExitCodeProcess( process_handle, &exitCode) == false ) {
             
                    CDMW_THROW(InternalErrorException);
             
                }
             
                return static_cast<OS::StatusProcess>(exitCode);
                
             
            #endif
            */
        }


#if defined(CDMW_POSIX)

        void OS::set_process_attributes(uid_t uid, gid_t gid)
        {
            M_processMonitor.set_process_attributes(uid, gid);
        }



        uid_t OS::get_uid()
        {
            return M_processMonitor.get_uid();
        }
        
        gid_t OS::get_gid()
        {
            return M_processMonitor.get_gid();
        }
        
        
        void OS::chown(const std::string& path, uid_t uid, gid_t gid)
        throw (PermissionDeniedException,
               BadParameterException,
               OutOfMemoryException,
               InternalErrorException)
        {
            int res = ::chown(path.c_str(), uid, gid);
            if (res != 0)
            {
                switch (errno)
                {
                    
                    // The system ressource are exhausted
                    
                    case EPERM:
                        // No break

                    case EACCES:
                        // No break
                        
                    case EROFS:
                        
                        CDMW_THROW(PermissionDeniedException);
                        break;

                    case ENOTDIR:
                        CDMW_THROW2(BadParameterException, "path", "File not found");
                        break;

                    default:
                        CDMW_THROW(InternalErrorException);
                        break;
                }
            }
        }
        
        

#endif


        //
        // Create a new process from the specified executable file
        //
        OS::ProcessId
        OS::create_process (const std::string& path, 
                            const std::string& args,
                            const std::string& home,
                            ProcessCallback* creationCallback,
                            ProcessCallback* endingCallback)
        throw (BadParameterException,
               OutOfMemoryException,
               InternalErrorException)
        {
            // check argument validity
            check_exec_validity (path, args);

            // register handler for SIGCHLD if not already done
            M_processMonitor.register_for_SIGCHLD();
            
            return M_processMonitor.create_process(path,
                                                   args,
                                                   home,
                                                   creationCallback,
                                                   endingCallback);
        }


        
        
        // Create a new process from the specified executable file 
        // adding the process enviroment variables contained in the vector env 
        
        
        OS::ProcessId
        OS::create_process (const std::string& path, 
                            const std::string& args,
                            const std::vector<std::string> env,
                            const std::string& home,                            
                            ProcessCallback* creationCallback,
                            ProcessCallback* endingCallback,
                            const int core_size,
                            const bool run_as_root)
        throw (BadParameterException,
               OutOfMemoryException,
               InternalErrorException)
        {
            // check argument validity
            check_exec_validity (path, args);

            // register handler for SIGCHLD if not already done
            M_processMonitor.register_for_SIGCHLD();
            return M_processMonitor.create_process(path,
                                                   args,
                                                   home,
                                                   env,
                                                   core_size,
                                                   run_as_root,
                                                   creationCallback,
                                                   endingCallback);
        }
        

        void
        OS::clear_process_ending_callbacks()
            throw()
        {
            M_processMonitor.clear_ending_callbacks();
        }
        
        
        
        //
        // Kill a process
        //
        void
        OS::kill_process (OS::ProcessId processId,
                          int signal)
        throw (BadParameterException,
               PermissionDeniedException,
               InternalErrorException)
        {
            
#if defined(CDMW_POSIX)

#ifdef CDMW_USE_GCOV
            int status_kill = ::kill (processId, SIGTERM);
#else
            int status_kill = ::kill (processId, signal);
#endif

            if ( status_kill == -1 )
            {

                switch ( errno )
                {

                        // The user has not enough right

                        case EPERM:
                        CDMW_THROW(PermissionDeniedException);
                        break;

                        // The process id does not exist

                        case ESRCH:
                        CDMW_THROW2(BadParameterException, "processId", "No process with this id");
                        break;


                        // The signale value is not valid: impossible

                        case EINVAL:
                        // No break;

                        default:
                        CDMW_THROW(InternalErrorException);
                        break;

                }
            }

#elif defined(_WIN32)

            HANDLE process_handle = get_win32_process_handle (processId);

            BOOL status_terminate = ::TerminateProcess (process_handle, -1);

            if (status_terminate == FALSE)
            {

                CDMW_THROW(InternalErrorException);

            }

#else
#    error "Cdmw: implementation not defined"
#endif


        }




        //
        // Return the id of the process
        //
        OS::ProcessId
        OS::get_processId()
        {
            OS::ProcessId pid = get_processId_i();
            return pid;
        }

        //
        // Implements retrieval of the id of the process
        //
        OS::ProcessId
        OS::get_processId_i()
        {
#if defined(CDMW_POSIX)

            pid_t pid = ::getpid();

#elif defined(_WIN32)

            DWORD pid = ::GetCurrentProcessId();


#else
#    error "Cdmw: implementation not defined"
#endif

            return pid;

        }



        //
        // Check the validity of the arguments used to create a new process
        //
        void
        OS::check_exec_validity( const std::string& path, const std::string args)
        throw ( BadParameterException )
        {

#ifdef CDMW_POSIX

            // Argument list too long

            if ( args.size() > ARG_MAX )
            {
                CDMW_THROW2(BadParameterException, "args", "argument list too long");
            }

            // File name too long
            if ( path.size() > PATH_MAX )
            {
                CDMW_THROW2(BadParameterException, "path", "file path too long");
            }


            if ( path_search(path) == 0)
            {
                CDMW_THROW2(BadParameterException, "path", "file not found, not executable or permission denied ");
            }


            // Check the existence of the file path
            if ( -1 == ::access( path.c_str(), F_OK ) )
            {
                //CDMW_THROW2(BadParameterException,"path","file not found");
            }

            // Check if we have execution rights
            if ( -1 == ::access( path.c_str(), X_OK ) )
            {
                //CDMW_THROW2(BadParameterException,"path","file is not executable or permission denied");
            }

#endif

        }



        //
        // Return the name of the current host in short notation
        //
        std::string
        OS::get_hostname()
        throw( InternalErrorException )
        {
            std::string hostname = get_hostname_i();
            return hostname;
        }

        //
        // Implements retrieval of  the name of the current host in short notation
        //
        std::string
        OS::get_hostname_i()
        throw( InternalErrorException )
        {
            
#if defined(CDMW_POSIX)

            struct utsname uts_info;
            int status = ::uname (&uts_info);

            if (status == -1)
            {
                CDMW_THROW( InternalErrorException );
            }
#if defined(CDMW_USE_CANONICAL_HOSTNAME)
            std:: string res(uts_info.nodename);
#ifdef _GNU_SOURCE
            std::string domainname(uts_info.domainname);
            if (!domainname.empty()) {
                res +='.';
                res += uts_info.domainname;
            }
#endif
            return res;
#else
            return uts_info.nodename;
#endif

#elif defined(_WIN32)

            // Request a winsock version library
            // in fact the version is not important for us
            WORD wVersionRequested;
            WSADATA wsaData;
            wVersionRequested = MAKEWORD (1, 1);
            int status_startup = ::WSAStartup (wVersionRequested, &wsaData);

            if (status_startup != 0)
            {
                CDMW_THROW( InternalErrorException );
            }

            // No defined constant
#   define MAX_HOSTNAME 1024

            char hostName[MAX_HOSTNAME];

            int status_hostname = ::gethostname (hostName, MAX_HOSTNAME);

            if (status_hostname != 0)
            {

                CDMW_THROW( InternalErrorException );

            }

            ::WSACleanup();

            return hostName;
#else
#    error "Cdmw: implementation not defined"
#endif
        }

        //
        // Return the name of the user logged in on the controlling
        // terminal of the process
        //
        std::string
        OS::get_logname()
        throw( InternalErrorException )
        {
            
#if defined(CDMW_POSIX)
           char * logname = ::getenv("LOGNAME");

           if (!logname)
           {
              CDMW_THROW( InternalErrorException );
           }

           return logname;
#else
#    error "Cdmw: implementation not defined"
#endif
        }

            

        //
        // Return the current time in sec/microsec from 1st Jan 1970
        //
        OS::Timeval
        OS::get_time()
        throw( InternalErrorException )
        {

            OS::Timeval os_timeval;

#if defined(CDMW_POSIX)


            struct timeval the_timeval;
            int status = ::gettimeofday (&the_timeval, NULL);

            if (status != 0)
            {
                CDMW_THROW(InternalErrorException);
            }

            os_timeval.seconds = the_timeval.tv_sec;
            os_timeval.microseconds = the_timeval.tv_usec;



#elif defined(_WIN32)

            // This implementation comes from the ACE Framework
            //  see ACE_Time_Value object

            FILETIME systemTime;
            ::GetSystemTimeAsFileTime (&systemTime);

            DWORDLONG unix_win_time_skew = 0x19db1ded53e8000L;

            ULARGE_INTEGER _100ns = { systemTime.dwLowDateTime,
                                      systemTime.dwHighDateTime
                                    };

            _100ns.QuadPart -= unix_win_time_skew;

            // Convert 100ns units to seconds;
            os_timeval.seconds = static_cast<long>(_100ns.QuadPart / (10000 * 1000));

            // Convert remainder to microseconds;
            os_timeval.microseconds = static_cast<long> ((_100ns.LowPart % (DWORD) (10000 * 1000)) / 10);


#else
#    error "Cdmw: implementation not defined"
#endif

            return os_timeval;
        }



        void
        OS::get_fmt_time (char *p_timeBuffer, size_t timeBufferSize,
                          unsigned long timeSec, char *p_timeFormat,
                          TimeMode mode)
        throw (InternalErrorException)
        {

            struct tm *p_today;

            struct tm today;
            time_t timer_value = timeSec;

            // according to time mode display select gmt or local

            if (mode == GMT_TIME)
            {
                p_today = ::gmtime (&timer_value);
            }
            else
            {
                p_today = ::localtime (&timer_value);
            }

            today = *p_today;

            // format time
            int status = ::strftime (p_timeBuffer, timeBufferSize, p_timeFormat, &today);

            if (status == 0)
            {
                CDMW_THROW(InternalErrorException);
            }

        }


        //
        // return a handler on the dynamic library
        //


        OS::SharedObjectHandle*
        OS::dlopen(const char *libname, int flag)
        throw()
        {
            return EffectiveSharedObject::dlopen(libname, flag);
        }


        //
        // close the dynamic library
        //

        int
        OS::dlclose(OS::SharedObjectHandle *handle)
        throw()
        {
            return handle->dlclose();
        }


        //
        // return the error title produce by dlopen, dlclose and dlsym function
        //

        std::string
        OS::dlerror(void)
        throw()
        {
            return EffectiveSharedObject::dlerror();
        }


        //
        // retrun a handler on the function
        //

        OS::FunctionHandle
        OS::dlsym(OS::SharedObjectHandle* handle, const char * symname)
        throw()
        {
            return handle->dlsym(symname);
        }


    } // End namespace ossupport

} // End namespace Cdmw

