/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#include "ossupport/ProcessMonitor.hpp"
#include "ossupport/EndingCallback.hpp"
#include "ossupport/OS.hpp"
#include "ossupport/Signals.hpp"
#include "Foundation/common/Assert.hpp"
#include "Foundation/common/String.hpp"
#include "Foundation/osthreads/MutexGuard.hpp"

#include <sstream>
#include <string>
#include <vector>
#include <errno.h>
#include <ctime>
#include <iostream>
#include <unistd.h>  
#include <sys/resource.h>


#if defined(CDMW_POSIX)
#include <sys/wait.h>
#endif


namespace Cdmw
{

    namespace OsSupport
    {

        ProcessMonitor::ProcessMonitor()
            throw()
            : m_uid(-1),
              m_gid(-1),
              m_registered_for_SIGCHLD(false)
        {
        }


        ProcessMonitor::~ProcessMonitor()
            throw()
        {
            // unregister this handler for SIGCHLD
            if (m_registered_for_SIGCHLD)
                Cdmw::OsSupport::Signals::Instance()->unregister_handler(SIGCHLD);
            clear_ending_callbacks();
        }


        void
        ProcessMonitor::register_for_SIGCHLD()
        {
            CDMW_MUTEX_GUARD(m_ending_callbacks_mutex);
            if (!m_registered_for_SIGCHLD) {
                Cdmw::OsSupport::Signals::Instance()->register_handler(SIGCHLD, this);
                m_registered_for_SIGCHLD = true;
            }
        }

        

        OS::ProcessId
        ProcessMonitor::create_process (const std::string& path, 
                                        const std::string& args,
                                        const std::string& home,
                                        ProcessCallback* creationCallback,
                                        ProcessCallback* endingCallback)
            throw (BadParameterException,
                   OutOfMemoryException,
                   InternalErrorException)
        {
            OS::ProcessId childPid = 
                create_process (path, args, home, endingCallback);
            
            if (creationCallback != 0) {
                creationCallback->execute(childPid);
            }
            
            return childPid;
        }
        

        OS::ProcessId
        ProcessMonitor::create_process (const std::string& path, 
                                        const std::string& args,
                                        const std::string& home,
                                        const std::vector<std::string> env,
                                        const int core_size,
                                        const bool run_as_root,
                                        ProcessCallback* creationCallback,
                                        ProcessCallback* endingCallback)
            throw (BadParameterException,
                   OutOfMemoryException,
                   InternalErrorException)
        {
            OS::ProcessId childPid = 
                create_process (path, args, home, env,core_size,run_as_root,endingCallback );
        
            if (creationCallback != 0) {
                creationCallback->execute(childPid);
            }
            
            return childPid;
        }
        

        
        ProcessCallback*
        ProcessMonitor::unregister_ending_callback (OS::ProcessId pid)
            throw()
        {
            CDMW_MUTEX_GUARD(m_ending_callbacks_mutex);

            CallbackMap::iterator it = 
                m_ending_callbacks.find(pid);

            if (it == m_ending_callbacks.end()) {
                return 0;
            } else {
                ProcessCallback* endingCallback = it->second;
                m_ending_callbacks.erase(pid);
                return endingCallback;
            }
        }
        
        
        void
        ProcessMonitor::clear_ending_callbacks()
            throw()
        {
            CDMW_MUTEX_GUARD(m_ending_callbacks_mutex);
            m_ending_callbacks.clear();
        }

        
        int 
        ProcessMonitor::handle_signal(int sig) 
            throw()
        {
            CDMW_ASSERT(sig == SIGCHLD);
            
#   ifdef CDMW_POSIX

            // Backup errno
            int old_errno = errno;

            // Loop while waitpid returns a pid
            // This allows to manage cases where only one SIGCHLD was
            // received despite there are several terminated child processes.
            while (1) {
                register int pid;
            int status;
                
                
                // loop on waitpid while we're interrupted (pid <=0 && EINTR)
                do {
                    errno = 0;
                    pid = ::waitpid(WAIT_ANY, &status, WNOHANG);
                
                } while (pid <= 0 && errno == EINTR);

                if (pid <= 0) {
                    // waitpid failure but not an interruption
                    // means no more stopped or terminated child processes
                    // so return
                    errno = old_errno;
                    break;
                }
                
                // display exit status informations
                std::ostringstream ostr;
                OS::Timeval time = OS::get_time();
                time_t ltime = time.seconds;
                tm now;
                ::localtime_r(&ltime, &now);
                char buffer1[100];
                strftime(buffer1, 100, "%Y/%m/%d %H:%M:%S", &now);
                ostr << buffer1 << ":" << (time.microseconds / 1000) << ":" << (time.microseconds % 1000)
                     << "  INFO Process with pid " << pid << " ended; ";
                if (WIFEXITED(status)) {
                    ostr << " It exited with status " << WEXITSTATUS(status);
                } else if (WIFSIGNALED(status)) {
                    ostr << " It didn't catch signal " << WTERMSIG(status);
                } else if (WIFSTOPPED(status)) {
                    ostr << " It was stopped with signal " << WSTOPSIG(status);
                } else {
                    ostr << " It ended for unkown reason !! ";
                }
#ifdef WCOREDUMP
                if (WCOREDUMP(status))
                    ostr << " A core was produced."; 
#endif
                std::cout << ostr.str() << std::endl;

                ProcessCallback* callback = unregister_ending_callback(pid);
                
                if (callback != 0) {
                    EndingCallback* ecb=dynamic_cast < EndingCallback * >(callback);
                    if (ecb != 0) {
                        ecb->set_exit_code(status);
                    }
                    callback->execute(pid);
                }
                    
            }
            
            errno = old_errno;
            return sig;
#   else
#       error Cdmw: Not supported platform
#   endif 
        }


        void 
        ProcessMonitor::set_process_attributes(uid_t uid, gid_t gid)
            throw()
        {
            m_uid = uid;
            m_gid = gid;
        }
        
        uid_t 
        ProcessMonitor::get_uid()
            throw()
        {
            return m_uid;
        }
        
        gid_t 
        ProcessMonitor::get_gid()
            throw()
        {
            return m_gid;
        }
                
        
        

        OS::ProcessId
        ProcessMonitor::create_process (const std::string& path, 
                                        const std::string& args,
                                        const std::string& home,
                                        ProcessCallback* endingCallback)
            throw (BadParameterException,
                   OutOfMemoryException,
                   InternalErrorException)
       {
            OS::ProcessId childPid;
#if defined(CDMW_POSIX)

            // Guard mutex to block ending callback execution
            // until create_process done.
            // This avoid SIGCHLD to be managed before 
            // endingCallback is registered
            CDMW_MUTEX_GUARD(m_ending_callbacks_mutex);

            pid_t fork_return = fork();

            switch (fork_return)
            {

                // We are in the child
                case 0:
                {
                    if (m_uid > 0 && m_gid > 0)
                    {
                        if (::setgid(m_gid) != 0)
                        {
                            ::perror("setgid error");
                            ::_exit(EXIT_FAILURE);
                        }
                
                        if (::setuid(m_uid) != 0)
                        {
                            ::perror("setuid error");
                            ::_exit(EXIT_FAILURE);
                        }
                    }
                
                    // Fork has succeeded. Unblock all signals 
                    // parent process has all its signals blocked.
                    sigset_t signal_to_block__;
                    sigfillset(&signal_to_block__);
                    // For portability, do not involve SIGSTOP & SIGKILL
                    sigdelset(&signal_to_block__,SIGKILL);
                    sigdelset(&signal_to_block__,SIGSTOP);

                    pthread_sigmask(SIG_UNBLOCK, &signal_to_block__, 0);

                    for (int i=getdtablesize()-1;i>=3;--i) close(i); /* close all descriptors */

                    std::string path_to_start = path;
                    // If a home directory has been specified, we must go into
                    if (home != "") {
                        try {
                            try {
                                // We must get the absolute path to the process
                                // from its relative path
                                path_to_start = OS::get_absolute_path (path);
                            
                            } catch (const NotExistException &ex) {
                                // if path is not found by get_absolute_path
                                // it will be found by exec_process using the PATH environment
                                // path has been checked by check_exec_validity at the beginning
                                // of create_process
                                // so we can continue
                            }
                            
                            // change the working directory
                            OS::chdir (home);

                        } catch (const Exception &ex) {
                            {
                                std::ostringstream msg;
                            
                                msg << "exception in child process : " << std::endl
                                    << ex.what() << std::endl;
                                ::perror(msg.str().c_str());
                            }
                            
                            // _exit() here is preferable to simple exit() since 
                            // we don't know what static destructors can perform 
                            // (cleaning in shared memory...) 
                            // This leads to not deleted memory in child process,
                            // since static destructors haven't been called.
                            // Thus Valgrind may report memory leaks in child
                            // process here. This is normal...
                            ::_exit(EXIT_FAILURE);
                        }
                    }
                
                    // exec the new process
                    exec_process (path_to_start, args);
                }
                break;
                
                // Error
                case static_cast<pid_t>( -1):
                {
                    switch (errno) {
                        // The system ressource are exhausted
                        case EAGAIN:
                        // No break
            
                        // There is not enough memory
                        case ENOMEM:
                        CDMW_THROW(OutOfMemoryException);
                        break;
            
                        default:
                        CDMW_THROW(InternalErrorException);
                        break;
                    }
                }
                
                // The process is created, we are in the father
                default:
                {
                    childPid = fork_return;
                    // register ending callback ASAP
                    m_ending_callbacks[childPid] = endingCallback;
                }
                break;
            }

#else
#    error "Cdmw: implementation not defined"
#endif

            return childPid;
       }


        //This method is useful to create an SO process specifying the enviroment variables
        OS::ProcessId
        ProcessMonitor::create_process (const std::string& path, 
                const std::string& args,
                const std::string& home,
                const std::vector<std::string> env,
                const int core_size,
                const bool run_as_root,
                ProcessCallback* endingCallback)
        throw (BadParameterException,
                OutOfMemoryException,
                InternalErrorException)
                {
            OS::ProcessId childPid;
#if defined(CDMW_POSIX)

            // Guard mutex to block ending callback execution
            // until create_process done.
            // This avoid SIGCHLD to be managed before 
            // endingCallback is registered
            CDMW_MUTEX_GUARD(m_ending_callbacks_mutex);

            pid_t fork_return = fork();

            switch (fork_return)
            {

            // We are in the child
            case 0:
            {
                // In case of the indicator run_as_root is "true" m_uid and m_gid are not modified
                if (m_uid > 0 && m_gid > 0 && !run_as_root)
                {
                    if (::setgid(m_gid) != 0)
                    {
                        ::perror("setgid error");
                        ::_exit(EXIT_FAILURE);
                    }

                    if (::setuid(m_uid) != 0)
                    {
                        ::perror("setuid error");
                        ::_exit(EXIT_FAILURE);
                    }
                }

                // Fork has succeeded. Unblock all signals 
                // parent process has all its signals blocked.
                sigset_t signal_to_block__;
                sigfillset(&signal_to_block__);
                // For portability, do not involve SIGSTOP & SIGKILL
                sigdelset(&signal_to_block__,SIGKILL);
                sigdelset(&signal_to_block__,SIGSTOP);

                pthread_sigmask(SIG_UNBLOCK, &signal_to_block__, 0);

                for (int i=getdtablesize()-1;i>=3;--i) close(i); /* close all descriptors */

                std::string path_to_start = path;
                // If a home directory has been specified, we must go into
                if (home != "") {
                    try {
                        try {
                            // We must get the absolute path to the process
                            // from its relative path
                            path_to_start = OS::get_absolute_path (path);

                        } catch (const NotExistException &ex) {
                            // if path is not found by get_absolute_path
                            // it will be found by exec_process using the PATH environment
                            // path has been checked by check_exec_validity at the beginning
                            // of create_process
                            // so we can continue
                        }

                        // change the working directory
                        OS::chdir (home);

                    } catch (const Exception &ex) {
                        {
                            std::ostringstream msg;

                            msg << "exception in child process : " << std::endl
                            << ex.what() << std::endl;
                            ::perror(msg.str().c_str());
                        }

                        // _exit() here is preferable to simple exit() since 
                        // we don't know what static destructors can perform 
                        // (cleaning in shared memory...) 
                        // This leads to not deleted memory in child process,
                        // since static destructors haven't been called.
                        // Thus Valgrind may report memory leaks in child
                        // process here. This is normal...
                        ::_exit(EXIT_FAILURE);
                    }
                }

                // exec the new process
                exec_process (path_to_start, args ,env,core_size);
            }
            break;

            // Error
            case static_cast<pid_t>( -1):
            {
                switch (errno) {
                // The system ressource are exhausted
                case EAGAIN:
                    // No break

                    // There is not enough memory
                case ENOMEM:
                    CDMW_THROW(OutOfMemoryException);
                    break;

                default:
                    CDMW_THROW(InternalErrorException);
                    break;
                }
            }

            // The process is created, we are in the father
            default:
            {
                childPid = fork_return;
                // register ending callback ASAP
                m_ending_callbacks[childPid] = endingCallback;
            }
            break;
            }

#else
#    error "Cdmw: implementation not defined"
#endif

            return childPid;
                }



       
        void
        ProcessMonitor::exec_process(const std::string& path, 
                                     const std::string& args)
            throw ()
        {
#if defined(CDMW_POSIX)
            // The first argument (argv[0]), must be the process path, or at least
            //    the basename of the path, so we concat arguments with the path
            //    to get it.
            std::vector<std::string> parsed_args = 
                Cdmw::Common::String::to_strings( path + " " + args);

            char** ppArgs = Cdmw::Common::String::to_char_array(parsed_args);

            if ( -1 == ::execvp (path.c_str(), ppArgs)) {
                    // Sorry we can not propogate the error to the caller, because
                    // we are already in a new process.
                    // We only display the error
                    ::perror ("Cdmw::OsSupport::OS::create_process()");
                    
                    // Be careful not exit() but _exit()
                    // See in documentation the difference
                    ::_exit (errno);
            }

#else
#    error "Cdmw: implementation not defined"
#endif

        }

        /*This exec_process function could specify the process 
        * enviroment,adding the variables contained in the vector env,
        * the maximum core dump size,
        */
        void
        ProcessMonitor::exec_process(const std::string& path, 
                                     const std::string& args,
                                     const std::vector<std::string> env,
                                     const int core_size)
            throw ()
        {
#if defined(CDMW_POSIX)
            // The first argument (argv[0]), must be the process path, or at least
            //    the basename of the path, so we concat arguments with the path
            //    to get it.
            std::vector<std::string> parsed_args = 
                Cdmw::Common::String::to_strings( path + " " + args);
            char** ppArgs = Cdmw::Common::String::to_char_array(parsed_args);

            char** new_env= Cdmw::Common::String::to_char_array(env); 
            for(unsigned int j=0;j<env.size(); j++)
            {
                int result=putenv(new_env[j]); 
                if(result!=0) ::perror("Error adding  environment varibles Cdmw::OsSupport::OS::create_process()");
            }  
                      
            /*Setting the the maximum size of a core file in bytes that  may  be
            created  by  the current process*/
            if(core_size!=-1)
            {
                struct rlimit rlim;
                rlim.rlim_cur = (unsigned int) core_size ;
                rlim.rlim_max = (unsigned int) core_size ;
                if ( setrlimit(RLIMIT_CORE, &rlim) < 0) 
                {
                    ::perror(" \n Error using setrlimit(RLIMIT_CORE) Cdmw::OsSupport::OS::create_process()");
                }
            }
            delete[] new_env;
            new_env=0;
            
            if ( -1 == ::execvp (path.c_str(), ppArgs)) 
            {
                // Sorry we can not propogate the error to the caller, because
                // we are already in a new process.
                // We only display the error
                ::perror ("Cdmw::OsSupport::OS::create_process()");
                // Be careful not exit() but _exit()
                // See in documentation the difference
                ::_exit (errno);
            }            
            
#else
#    error "Cdmw: implementation not defined"
#endif
        
        }
        
    } // End namespace OsSupport
} // End namespace Cdmw

