/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#include <Foundation/ossupport/Signals.hpp>
#include <ossupport/SigWaitThread.hpp>
#include "Foundation/osthreads/MutexGuard.hpp"
#include <Foundation/ossupport/OS.hpp> // needed for OS::get_processId() in std::cout
#include <map>
#include <csignal>
#include <iostream>

#if defined (CDMW_POSIX)

#    include <sys/wait.h>
#endif


namespace
{
    
#ifdef CDMW_USE_GCOV
    class ExitHandler : public Cdmw::OsSupport::SignalHandler
    {
    public:
        ExitHandler()
        {
        }
        
        virtual ~ExitHandler() 
        {
        }
        
        int handle_signal(int sig) throw()
        {
            
            std::cout << "WARNING: signal "<<sig<<" was trapped. Exiting..." << std::endl;
            exit(1);
            return 0;
        }
    };
#endif
}


namespace Cdmw
{
    namespace OsSupport
    {
        Signals * Signals::M_Instance = new Signals;

        // destructor
        Signals::~Signals() {}
        
        // Register a new Event_handler for the signal
        // and return the old one
        SignalHandler* Signals::register_handler(int sig, 
                                                    SignalHandler* seh)
        {
            CDMW_MUTEX_GUARD(m_mutex);
            HandlerMap::iterator iter = m_handlerMap.find(sig);
            SignalHandler* oldseh = 0;
            if (iter != m_handlerMap.end())
                oldseh = (*iter).second;
            m_handlerMap[sig] = seh;
            return oldseh;
        }
        
        // remove the Event_handler for the signal
        SignalHandler*  Signals::unregister_handler(int sig)
        {
            CDMW_MUTEX_GUARD(m_mutex);
            SignalHandler* oldseh = 0;

            HandlerMap::iterator iter = m_handlerMap.find(sig);
            if(iter != m_handlerMap.end())
            {
                oldseh = (*iter).second;
                m_handlerMap.erase(sig);
            }
            return oldseh;
        }
        
        // return the handler for the signal
        SignalHandler* Signals::get_handler(int sig)
        {
            CDMW_MUTEX_GUARD(m_mutex);
            HandlerMap::iterator iter = m_handlerMap.find(sig);
            if (iter != m_handlerMap.end())
                return (*iter).second;
            else
                return 0;
        }
        
        bool Signals::handle_signal(int sig)
            throw()
        {
            CDMW_MUTEX_GUARD(m_mutex);
            HandlerMap::iterator iter = m_handlerMap.find(sig);
            if (iter != m_handlerMap.end()) {
                (*iter).second->handle_signal(sig);
                return true;
            } else {
                return false;
            }
        }


        // return the instance of the Signals
        Signals* Signals::Instance()
        {
            return M_Instance;
        }
        
        
        
        // constructor
        Signals::Signals()
        {
           if (::getenv("CDMW_NO_SIGWAIT_THREAD") == 0) {
              // set of blocking signal
              sigset_t signal_to_block;

              sigfillset(&signal_to_block);
              // For portability, do not block SIGSTOP & SIGKILL
              sigdelset(&signal_to_block,SIGKILL);
              sigdelset(&signal_to_block,SIGSTOP);
              pthread_sigmask(SIG_BLOCK, &signal_to_block, NULL);

              // TODO : for the moment, the thread is created 
              // always. We must create a method in order to init
              // and start this thread only if it is needed and 
              // at the beginning of the main

#ifdef CDMW_USE_GCOV

              ExitHandler * exitHandler = new ExitHandler();
              register_handler(SIGTERM, exitHandler);
              register_handler(SIGINT, exitHandler);
              register_handler(SIGQUIT, exitHandler);
#endif
            

              // thread for the sigwait
              SigWaitThread* sigwait_thread = new SigWaitThread(this);
              sigwait_thread->start();
           } else {
                std::cout << "CARDAMOM [pid:"
                        << OS::get_processId() 
                        << "] INFO: CDMW_NO_SIGWAIT_THREAD environment variable set. Disabling CARDAMOM-specific signal handling to not conflict with JVM's." << std::endl;
           }
        }
        

    } // End namespace ossupport
} // End namespace Cdmw
