/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

#include <iostream>
#include <sstream>
#include <string>
#include <Foundation/ossupport/SystemProperties.hpp>
#include <Foundation/osthreads/ReaderLockGuard.hpp>
#include <Foundation/osthreads/WriterLockGuard.hpp>
#include <Foundation/ossupport/OS.hpp>
#include <fstream>


extern char** environ; /* NULL-terminated array of "NAME=VALUE" environment variables.  */

namespace
{
    // Prefix for all System properties from command line
    // ex: -D<name>=<value>
    const std::string PROP_PREFIX="-D";
    const std::string CDMW_PROPERTIES_FILE_PROP_NAME="CdmwPropertyFile";
    
}; // End of anonymous namespace

namespace Cdmw
{
namespace OsSupport
{

/**
 *Purpose:
 *<p>
 * Handle the System Propperties
 *<p>
 *Features:
 *<p>
 * Thread safe.
 *
 *@see Properties
 */
    
    // Names of some Cdmw Properties
    const std::string SystemProperties::CDMW_THREADS_STACKSIZE_NAME = "Cdmw.Threads.Stacksize";

    const std::string SystemProperties::ORB_THREADS_STACKSIZE_NAME = "Orb.Threads.Stacksize";

    const std::string SystemProperties::FTSTATEMNGR_SCHED_THREADS_STACKSIZE_NAME =
                                          "Cdmw.FT.Scheduler.Threads.Stacksize";

    const std::string SystemProperties::FTSTATEMNGR_TIMER_THREAD_STACKSIZE_NAME =
                                          "Cdmw.FT.Timer.Thread.Stacksize";

    const std::string SystemProperties::FLUSHAREAMNGR_THREAD_STACKSIZE_NAME =
                                          "Cdmw.Trace.FlushAreaMngr.Thread.Stacksize";

    Common::Properties* SystemProperties::M_properties(0);
    ReaderWriterLock SystemProperties::M_properties_mutex;

    
    SystemProperties::SystemProperties()
        : Common::Properties()
    {
    }
    
    SystemProperties::SystemProperties(const Common::Properties* prop)
        : Common::Properties(prop)
    {
    }

    // 1) Get environment variables. NOTA: Ignore those with no value!
    // 2) Any property file ? ex: -DCdmwPropertyFile=f
    // 3) Parse command line arguments
    void SystemProperties::initialise(int argc, char *argv[], char *envp[])
        throw (BadParameterException)
    {
        //  1) Get environment variables
        // valgrind : Conditional jump or move depends on uninitialised value
        std::string name (""), value ("");
        for (int i=0;envp[i];++i) {
            try {
          std::string envp_s(envp[i]);
           if (explode_cmdline_property(envp_s, name, value))
           {
                 Common::Properties::set_property(name,value);
           }
            } catch (const BadParameterException&) {
                // ignore ...
            }
        }
        // 2) Any property file ? ex: -DCdmwPropertyFile=f
        std::string properties_file_proproperty_name = PROP_PREFIX + CDMW_PROPERTIES_FILE_PROP_NAME;
        std::string properties_file = OS::get_option_value (argc, argv,  properties_file_proproperty_name);
    
        if (properties_file != "no" && properties_file != "yes")
        {
            std::ifstream in(properties_file.c_str());
            
            if (!in) {
                std::string val(properties_file);
                val += " <Could not open Properties file '" + properties_file + "'>";
                CDMW_THROW2( BadParameterException, properties_file_proproperty_name, val );
            }
            else {
                try {
                   Common::Properties::load(in);
                } catch (const BadParameterException& e) {
                    std::string v = e.m_parameterValue;
                    v += " in property file '" + properties_file + "'";
                    CDMW_THROW2( BadParameterException, e.m_parameterName, v );
                }
            }
        }
                
        // 3) Parse command line arguments

        int iArg(0);
        
        // We scan all arguments to find the matching with option
        while (iArg < argc)
        {
            std::string arg (argv[iArg]);
            // "-D<name>=<value>"
            if ((arg.length() > PROP_PREFIX.length()) &&
                (arg.substr(0,PROP_PREFIX.length()) == PROP_PREFIX)) {
                // skip PROP_PREFIX
                try {
          std::string arg_s(arg.substr(2,arg.length()-2).c_str());
            if (explode_cmdline_property(arg_s, name, value))
            {
                      Common::Properties::set_property(name,value);
            }
                } catch (const BadParameterException& e) {
        // now this exception occurs only when we have -DXX= without value
                    std::string v = e.m_parameterValue;
                    v += " in command line argument: ";
                    CDMW_THROW2( BadParameterException, e.m_parameterName, v );
                }
            }
            iArg++;
        }
    }
    
    
    Common::Properties*
    SystemProperties::Init(int argc, char *argv[])
        throw (BadParameterException)
    {
        CDMW_WRITER_LOCK_GUARD(M_properties_mutex);

        if (M_properties == 0) {
            static SystemProperties instance__;
            instance__.initialise(argc, argv, environ);
            M_properties = &instance__;
        }

        return M_properties;
    }
    
    Common::Properties*
    SystemProperties::Init(Common::Properties* prop)
    {
        CDMW_WRITER_LOCK_GUARD(M_properties_mutex);

        if (M_properties == 0) {
            static SystemProperties instance__(prop);
            M_properties = &instance__;
        }

        return M_properties;
    }


    Common::Properties*
    SystemProperties::Instance()
    {
        CDMW_WRITER_LOCK_GUARD(M_properties_mutex);

        if (M_properties == 0) {
            static SystemProperties instance__;
            M_properties = &instance__;
        }
        
        return M_properties;
    }


    void
    SystemProperties::set_property(const std::string& name, 
                                   const std::string& value)
    {
        CDMW_WRITER_LOCK_GUARD(M_properties_mutex);
        Common::Properties::set_property(name,value);
    }
    
    std::string
    SystemProperties::get_property(const std::string& name,
                                   const std::string& default_value) const
    {
        CDMW_READER_LOCK_GUARD(M_properties_mutex);
        return Common::Properties::get_property(name,default_value);
    }
    
    void
    SystemProperties::property_names(PropertyNames& pn) const
    {
        CDMW_READER_LOCK_GUARD(M_properties_mutex);
        Common::Properties::property_names(pn);
    }
    
    void
    SystemProperties::load(std::istream& is)
        throw (BadParameterException)
    {
        CDMW_WRITER_LOCK_GUARD(M_properties_mutex);
        Common::Properties::load(is);
    }
    
    void
    SystemProperties::store(std::ostream& os) const
    {
        CDMW_READER_LOCK_GUARD(M_properties_mutex);
        Common::Properties::store(os);
    }


    void
    SystemProperties::Set_Property(const std::string& name, 
                                   const std::string& value)
    {
        Common::Properties* myProperties = Instance();
        myProperties->set_property(name, value);
    }

    void
    SystemProperties::Set_Property(const std::string& name, 
                                   int value)
    {
        std::ostringstream os;
        os << value;
        SystemProperties::Set_Property(name, os.str());
    }

    std::string
    SystemProperties::Get_Property(const std::string& name,
                                   const std::string& default_value)
    {
        Common::Properties* myProperties = Instance();
        return myProperties->get_property(name, default_value);
    }

    int
    SystemProperties::Get_Property(const std::string& name,
                                   int default_value)
    {
        std::ostringstream os;
        os << default_value;
        std::string strValue = SystemProperties::Get_Property(name, os.str());

        return atoi(strValue.c_str());
    }

} // End of namespace OsSupport
} // End of namespace Cdmw


