/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


package cdmw.ossupport;

/**
 * Implementation of the recursive Mutex
 *
 * Features:
 * Thread safe, exception safe
 *
 */
public class RecursiveMutex {

    /**
     * Owner Thread of the Mutex
     */
    private Thread owner = null;

    /**
     * State of the mutex
     */
    private boolean isLocked = false;

    /**
     * Number of call of the lock method
     */
    private int count = 0;

    /**
     * Lock the Mutex. Once a thread has a lock, it
     * can re-obtain it any number of times without blocking.
     * The lock is made available to other threads when
     * as many releases as acquires have occurred.
     *
     * @exception InterruptedException
     */
    public void lock() throws InterruptedException {

        if (Thread.interrupted()) throw new InterruptedException();
        Thread caller = Thread.currentThread();
        synchronized(this) {

            if(isLocked && caller == owner) {
                count++;
                return;
            }

            try {
                while(isLocked) {
                    wait();
                }
                isLocked = true;
                owner = caller;
                count = 1;
            } catch(InterruptedException ex) {
                notify();
                throw ex;
            }

        }

    }

    /**
     * Unlock the Mutex, if other threads are waiting for
     * the mutex, one of them is allowed to acquire the mutex,
     * only if count is equal to 0.
     */
    public synchronized void unlock() {

        if(--count == 0) {
            isLocked = false;
            owner = null;
            notify();
        }

    }

}

