/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


package cdmw.ossupport.test;

import java.io.IOException;

import org.junit.AfterClass;
import org.junit.BeforeClass;
import org.junit.Test;

import static org.junit.Assert.assertTrue;
import static org.junit.Assert.fail;

import cdmw.ossupport.OS;
import cdmw.ossupport.Timeval;
import cdmw.ossupport.ThreadedProcess;
import cdmw.testutils.TestManager;


public class OSTest {
    
    private static boolean validOnly;
    private static int timescale;
    
    @BeforeClass
    public static void initTests() {
        TestManager testManager = TestManager.instance();
        
        validOnly = Boolean.parseBoolean(
            testManager.getProperty("OSS_SUPPORT_TEST_VALID_ONLY"));
        timescale = TestManager.getTimescale();

        System.out.println("===================================================");
        System.out.println("Starting test " + OSTest.class.getName());
        System.out.println("===================================================");        
    }
    
    @AfterClass
    public static void finiTests() {
        System.out.println("===================================================");
        System.out.println("Ending test " + OSTest.class.getName());
        System.out.println("===================================================");        
    }

    private class TestProcessCallback extends Thread {

        private ThreadedProcess myp;
        private int exitcode = -2;
        private int step = -1;

        public TestProcessCallback(ThreadedProcess p) {
            super();
            myp = p;
            step = 0;
        }

        public int getExitCode() {
            return exitcode;
        }

        public int getStep() {
            return step;
        }

        public void run() {
            step = 1;
            // return -1 if p null, or return exitCode of process
            try {
                exitcode = OS.waitProcess(myp);
            } catch(InterruptedException ie) {
                System.out.println(ie.getMessage());
            }
            step = 2;
        }

    }

    @Test
    public void doTests() {

        if (!validOnly) {
            // TODO: finding how to test the returned date.
            System.out.println("Trying to get the time");
            Timeval time = OS.getTime();
            System.out.println(time.toString());

            System.out.println("Number of milliseconds since the epoch: "+time.getMilliseconds());
            System.out.println("Number of seconds since the epoch: "+time.getSeconds());
            System.out.println("Number of microseconds remaining: "+time.getMicroseconds());

            System.out.println("Trying to get the hostname");
            System.out.println(OS.getHostname());
            
            System.out.println("Trying to get the process id");
            System.out.println(OS.getJavaProcessId());

            System.out.println("Creating a temporary file name");
            String filename = new String("Cdmw.tmp");

            System.out.println("Checking this file not exists");
            assertTrue(!OS.fileExists(filename));

            System.out.println("Creating a new process");
            String classname = "cdmw.ossupport.test.MainTest"
                + " --create-file=" + filename;

            ThreadedProcess p = null;
            try {
                p = OS.createJavaProcess(classname, TestManager.getDefaultProperties());
                // New process successfully created.
            } catch (java.io.IOException ioe) {
                fail("Unable to create a new process.");
            }

            System.out.println("Test of the sleep function");
            System.out.println("We sleep a while ...");
            try {
                OS.sleep(timescale * 2000);
                // Sleep successful.
            } catch(InterruptedException ie) {
                fail("Unable to sleep process: " + ie.getMessage() );
            }
            catch(NumberFormatException nfe) {
                fail("Unable to sleep process: " + nfe.getMessage() );
            }

            if ( p!=null ) {
                System.out.println("We wait for the process termination");
                int exit = 0;
                try {
                    exit = OS.waitProcess(p);
                } catch(InterruptedException ie) {
                    System.out.println(ie.getMessage());
                }

                System.out.println("Checking if the process termination is normal");
                assertTrue(exit == 0);

                System.out.println("Checking if the process has created a new file");
                assertTrue(OS.fileExists(filename));

                System.out.println("Trying to execute a non executable file");
                try {
                    p = OS.createProcess(filename);
                    OS.sleep(timescale * 2000);
                    fail("Error process created.");
                } catch(IOException ioe) {
                    // Expected behaviour
                }
                catch (InterruptedException e) {
                    fail(e.getMessage());
                }

                System.out.println("Trying to delete an existing file");
                assertTrue(OS.unlink(filename));

                System.out.println("Trying to delete an unexisting file");
                assertTrue(!OS.unlink(filename));
            }
        }

        ThreadedProcess p2 = null;
        String classname2 = "cdmw.ossupport.test.MainTest --sleep=9";
        try {
            p2 = OS.createJavaProcess(classname2, TestManager.getDefaultProperties());
            // New process successfully created.
        } catch (java.io.IOException ioe) {
            fail("Unable to create a new process.");
        }
        TestProcessCallback tpc = new TestProcessCallback (p2);
        tpc.start();
        try {
            OS.sleep(timescale * 2000);
        } catch(InterruptedException ie) {
            fail("Unable to sleep process: " + ie.getMessage() );
        }
        catch(NumberFormatException nfe) {
            fail("Unable to sleep process: " + nfe.getMessage() );
        }
        System.out.println("Check that process has been created and not exited");
        assertTrue(tpc.getStep() == 1);
        System.out.println("Perform a kill on the created process");
        OS.killProcess(p2);
        try {
            OS.sleep(timescale * 500);
        } catch(InterruptedException ie) {
            fail("Unable to sleep process: " + ie.getMessage() );
        }
        catch(NumberFormatException nfe) {
            fail("Unable to sleep process: " + nfe.getMessage() );
        }
        System.out.println("Check that process has exited");
        assertTrue(tpc.getStep() == 2);
    }       
}

