/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#ifndef INCL_OSSUPPORT_ReaderWriterRecursiveMutex_HPP
#define INCL_OSSUPPORT_ReaderWriterRecursiveMutex_HPP

#include <Foundation/osthreads/Mutex.hpp>
#include <Foundation/osthreads/Condition.hpp>
#include <Foundation/osthreads/Thread.hpp>
#include <map>

namespace Cdmw
{

    namespace OsSupport
    {


        /**
        *Purpose:
        *<p>   This class allow to protect data against concurrent
        *    access. The ReaderWriterRecursiveMutex class allows more than one
        *    thread access the data for reading, but only one thread
        *    to access data for writing
        *
        *Features:
        *<p> Thread safe, exception safe
        *
        *@see  Mutex
        *
        */

        class ReaderWriterRecursiveMutex
        {

            public:

                /**
                * Constructor
                *
                * @exception OutOfMemoryException
                * @exception InternalErrorException     
                */
                ReaderWriterRecursiveMutex()
                throw (OutOfMemoryException,
                       InternalErrorException);



                /**
                * Destructor
                *
                */
                virtual
                ~ReaderWriterRecursiveMutex()
                throw();


                /**
                * Purpose:
                * <p> Acquire the lock for reading, more than one
                * thread can acquire the lock for reading.
                *
                * @exception OutOfMemoryException
                * @exception InternalErrorException     
                *
                */
                void
                readLock()
                throw (OutOfMemoryException,
                       InternalErrorException);



                /**
                * Purpose:
                * <p> Acquire the lock for writing, only one
                * thread can access data, when the lock is acquired
                *  for writing.
                *
                * @exception OutOfMemoryException
                * @exception InternalErrorException     
                */
                void
                writeLock()
                throw (OutOfMemoryException,
                       InternalErrorException);


                /**
                * Purpose:
                * <p> Release the lock previously acquired
                *  for reading.
                *
                * @exception OutOfMemoryException
                * @exception InternalErrorException     
                */
                void
                readUnlock()
                throw (OutOfMemoryException,
                       InternalErrorException);


                /**
                * Purpose:
                * <p> Release the lock previously acquired
                *  for writing.
                *
                * @exception OutOfMemoryException
                * @exception InternalErrorException     
                */
                void
                writeUnlock()
                throw (OutOfMemoryException,
                       InternalErrorException);
               
                /**
                * Purpose:
                * <p> Return 1 if it is locked for write operation, 0 if it is unlocked for write operation 
                */
                bool
                isWriteLocked();
        
                /**
                * Purpose:
                * <p> Return 1 if it is locked for read operation, 0 if it is unlocked for read operation 
                */
                bool
                isReadLocked();


            protected:


            private:

                //
                // Copy ctor
                //
                ReaderWriterRecursiveMutex(const ReaderWriterRecursiveMutex& rhs);


                /**
                * Each reader/writer thread must acquire this mutex before 
                * changing data of the ReaderWriterRecursiveMutex.
                * This mutex is change in an atomic way by the 2 internals
                * condition : m_reader_cond, m_writer_cond.
                */
                Mutex m_rwlock_mutex;

                /**
                * This condition allows writers thread to wait until there is no
                * more reader/writer thread in the ReaderWriterRecursiveMutex. It 's 
                * the responsibility of each thread to notify the conditions if
                * threads are waiting on them. This condition unlock/lock the
                * m_rwlock_mutex, in its wait() call.
                */
                Condition m_writer_cond;


                /**
                * This condition allows readers thread to wait until there is no
                * more writer thread in the ReaderWriterRecursiveMutex. It 's 
                * the responsibility of each thread to notify the conditions if
                * threads are waiting on them. This condition unlock/lock the
                * m_rwlock_mutex, in its wait() call.
                */
                Condition m_reader_cond;

                /**
                * Contains the number of readers currently locking the
                * ReaderWriterRecursiveMutex. When this variable need to be modify,
                * the corresponding mutex must be acquired.
                */
                size_t m_nbReaders;

                /**
                * This variable must never be over 1. This state is maintain
                * for assertion purpose.
                */
                size_t m_nbWriters;
        
                /**
                * Number of waiting writer threads
                */
                int m_nb_waiting_writer;
        
                /**
                * Number of waiting reader threads
                */
                int m_nb_waiting_reader;
        
                /**
                *The vector of owner and number of related read lock
                */
                std::map<Thread::ThreadId, size_t> owner_reader;
                
                /**
                * Write owner
                */
                Thread::ThreadId owner_writer;
                
                /**
                * Number of owner recursive write
                */
                size_t owner_write_counter;
        }

        ; // End class ReaderWriterRecursiveMutex

    } // End namespace osthreads

} // End namespace Cdmw

#endif // INCL_OSSUPPORT_ReaderWriterRecursiveMutex_HPP

