/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#ifndef INCL_OSSUPPORT_RECURSIVE_MUTEXIMPL_HPP
#define INCL_OSSUPPORT_RECURSIVE_MUTEXIMPL_HPP

#include <Foundation/common/Exception.hpp>
#include <Foundation/osthreads/Thread.hpp>

#include "ace/Recursive_Thread_Mutex.h"

namespace Cdmw
{

    namespace OsSupport
    {

        class RecursiveMutexImpl;


        /**
        *Purpose:
        *<p> Implementation of a recursive mutex.
        *
        * A thread that has already acquired a recursive mutex will not be blocked if it
        * locks again this recursive mutex. This thread is responsible for unlocking the
        * recursive mutex as many times as it has locked the mutex. Otherwise, the mutex
        * will remain locked definitively.
        *
        *
        *Features:
        *<p> Thread safe, exception safe
        *
        *@see  Mutex
        *@see  Condition 
        *
        */

        class RecursiveMutexImpl
        {

            public:

                /**
                * Purpose:
                * <p> construct the mutex
                *
                *@exception OutOfMemoryException
                *@exception InternalErrorException      
                */
                RecursiveMutexImpl()
                throw (OutOfMemoryException, InternalErrorException);



                //
                // Dtor
                //
                virtual
                ~RecursiveMutexImpl()
                throw() ;


                /**
                * Purpose:
                * <p> Locks in an atomic way the Mutex.
                * If the mutex is already acquired by another thread, the thread
                * waits until it is get unlocked.
                * If the mutex is already acquired by the same thread, the lock
                * count is simply incremented.
                *
                * @exception InternalErrorException 
                */

                virtual void lock ()
                    throw (InternalErrorException);


                /**
                * Purpose:
                * <p> Decrements the lock count and unlocks the mutex when
                * the lock count equals 0.
                *
                * @exception PermissionDeniedException if the calling thread 
                * is not the owner of the mutex.
                * @exception InternalErrorException
                */
                virtual void unlock()
                throw (PermissionDeniedException, InternalErrorException);


               virtual bool is_locked()
               throw ();


            protected:


            private:

                //
                // Copy Ctor
                //
                RecursiveMutexImpl( const RecursiveMutexImpl&);


                //
                // Assignment operator
                //
                RecursiveMutexImpl&
                operator=( const RecursiveMutexImpl&);


                ACE_Recursive_Thread_Mutex m_ace_recursive_mutex;


        };


    } // End namespace osthreads
} // End namespace Cdmw



#endif //INCL_OSSUPPORT_RECURSIVE_MUTEXIMPL_HPP

