/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#include "Foundation/osthreads/ReaderWriterRecursiveMutex.hpp"
#include "Foundation/common/Assert.hpp"
#include "Foundation/osthreads/MutexGuard.hpp"

namespace Cdmw
{

    namespace OsSupport
    {

        ReaderWriterRecursiveMutex::ReaderWriterRecursiveMutex()
        throw (OutOfMemoryException,
               InternalErrorException)
                : m_rwlock_mutex(),
                m_writer_cond(m_rwlock_mutex),
                m_reader_cond(m_rwlock_mutex),
                m_nbReaders(0),
                m_nbWriters(0),
                m_nb_waiting_writer(0),
                m_nb_waiting_reader(0),
                owner_write_counter(0)

        {
            owner_writer = 0;
        }

        //
        // Dtor
        //
        ReaderWriterRecursiveMutex::~ReaderWriterRecursiveMutex()
        throw()
        {
            // Nothing to do
        }

        //
        // Acquire for read the lock
        //
        void
        ReaderWriterRecursiveMutex::readLock()
        throw (OutOfMemoryException,
               InternalErrorException)
        {
            try
            {
                CDMW_MUTEX_GUARD(m_rwlock_mutex);

                Thread::ThreadId calling_thread = Thread::self();
                std::map<Thread::ThreadId, size_t>::iterator owner_reader_iterator = owner_reader.find(calling_thread);
                if(owner_reader_iterator == owner_reader.end()){
                    owner_reader[calling_thread] = 1;
                } 
                else{
                    (owner_reader_iterator->second)++;
                }
        
                CDMW_ASSERT(m_nbReaders >= 0);
                CDMW_ASSERT(m_nbWriters == 0 || m_nbWriters == 1);
                m_nb_waiting_reader++;

                // If there is writer in the condition, we need to wait
                while (m_nbWriters > 0 && owner_writer != calling_thread)
                {
                    m_reader_cond.wait();
                }
                m_nb_waiting_reader--;
        
                // There is one more reader
                m_nbReaders++;
                CDMW_ASSERT(m_nb_waiting_writer >= 0);
                CDMW_ASSERT(m_nb_waiting_reader >= 0);
                CDMW_ASSERT(m_nbReaders >= 1);
                CDMW_ASSERT( (m_nbWriters == 0) || ((m_nbWriters == 1) && owner_writer == calling_thread));
            }
            catch (const AlreadyDoneException&)
            {

                CDMW_THROW (InternalErrorException);
            }
        }

        //
        // Acquire for write the lock
        //
        void
        ReaderWriterRecursiveMutex::writeLock()
        throw (OutOfMemoryException,
               InternalErrorException)

        {
            bool can_i_write=false;
            try
            {
                CDMW_MUTEX_GUARD(m_rwlock_mutex);   
                Thread::ThreadId calling_thread = Thread::self();
                std::map<Thread::ThreadId, size_t>::iterator owner_reader_iterator = owner_reader.find(calling_thread);
        
                //the read lock of this thread
                size_t this_thread_reader_lock=0;
        
                //if the thread took the readLock the entry is in the vector, and i check that it is the one reader
                //if it is the one reader can_i_write is set to true
                if(owner_reader_iterator != owner_reader.end()){
                    this_thread_reader_lock=owner_reader_iterator->second;
                    if(owner_reader_iterator->second ==  m_nbReaders){
                        can_i_write = true;    
                    }
                    else if (0==m_nbReaders){
                        can_i_write = true;
                    }     
                }
                CDMW_ASSERT(m_nbReaders >= 0);
                CDMW_ASSERT(m_nbWriters <= 1);
                m_nb_waiting_writer++;
        
                // To acquire exclusive access no reader writer different from calling thread must be present 
                while ( (m_nbReaders != 0 && !can_i_write) || ((m_nbWriters != 0) && (owner_writer!=calling_thread)))
                {
                    m_writer_cond.wait();
                }
                if (owner_writer!=calling_thread){
                    m_nbWriters++;
                }
                owner_writer = calling_thread;
                m_nb_waiting_writer--;
                owner_write_counter++;
                CDMW_ASSERT(m_nb_waiting_writer >= 0);
                CDMW_ASSERT(m_nb_waiting_reader >= 0);
        
                //the read lock of this thread must be equals to total read lock
                CDMW_ASSERT(m_nbReaders == this_thread_reader_lock);
                CDMW_ASSERT(m_nbWriters == 1);
            }
            catch (const AlreadyDoneException&)
            {
                CDMW_THROW (InternalErrorException);
            }
        }

        //
        // Unlock the lock previously acquire for reading
        //
        void
        ReaderWriterRecursiveMutex::readUnlock()
        throw (OutOfMemoryException,
               InternalErrorException)
        {
            try
            {
                CDMW_MUTEX_GUARD(m_rwlock_mutex);

                Thread::ThreadId calling_thread = Thread::self();
                std::map<Thread::ThreadId, size_t>::iterator owner_reader_iterator = owner_reader.find(calling_thread);
                if(owner_reader_iterator == owner_reader.end()){
                    CDMW_ASSERT(false);
                } 
                else{
                    (owner_reader_iterator->second)--;
                    if ((owner_reader_iterator->second)==0){ 
                        owner_reader.erase(owner_reader_iterator);
                    }
                }
                CDMW_ASSERT(m_nbReaders >= 1);
                CDMW_ASSERT( (m_nbWriters == 0) || ((m_nbWriters == 1) && owner_writer == calling_thread));
                m_nbReaders--;
        
                // If there is no more reader or writer, we allow writer to access
                if (m_nbReaders == 0 && m_nb_waiting_writer != 0 && m_nbWriters == 0)
                {
                    m_writer_cond.broadcast();
                }
                CDMW_ASSERT( (m_nbWriters == 0) || ((m_nbWriters == 1) && owner_writer == calling_thread));
                CDMW_ASSERT(m_nbReaders >= 0);
            }
            catch (const AlreadyDoneException&)
            {
                CDMW_THROW (InternalErrorException);
            }
        }

        //
        // Unlock the lock previously acquire for writing
        //
        void
        ReaderWriterRecursiveMutex::writeUnlock()
        throw (OutOfMemoryException,
               InternalErrorException)
        {
            try
            {
            CDMW_MUTEX_GUARD(m_rwlock_mutex);
        
                //Find the number of this thread read lock 
                Thread::ThreadId calling_thread = Thread::self();
                std::map<Thread::ThreadId, size_t>::iterator owner_reader_iterator = owner_reader.find(calling_thread);
                size_t this_thread_reader_lock=0;
                if(owner_reader_iterator != owner_reader.end()){
                    this_thread_reader_lock=owner_reader_iterator->second;
                }
                CDMW_ASSERT(m_nbWriters == 1);
                owner_write_counter--;
        
                if(owner_write_counter==0){
                    m_nbWriters--;
                    owner_writer = 0;
                    CDMW_ASSERT(m_nbWriters == 0);
                }
                if (m_nb_waiting_reader != 0)
                {
                    m_reader_cond.broadcast();
                }
                else if (m_nb_waiting_writer != 0)
                {
                    m_writer_cond.broadcast();
                }
        
                //The number of this thread read lock must be equal to total read lock 
                CDMW_ASSERT(m_nbReaders==this_thread_reader_lock);
            }
            catch (const AlreadyDoneException&)
            {

                CDMW_THROW (InternalErrorException);
            }
        }
    
        bool ReaderWriterRecursiveMutex::isWriteLocked()
        {   
            CDMW_MUTEX_GUARD(m_rwlock_mutex);
            return (m_nbWriters>0);
        }
         
        bool ReaderWriterRecursiveMutex::isReadLocked()
        {
            CDMW_MUTEX_GUARD(m_rwlock_mutex);
            return (owner_reader.size()>0);
        }
    } // End namespace osthreads
} // End namespace Cdmw

