/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#include "testosthreads/TestMutexThreadDriver.hpp"
#include "testosthreads/TestMutexThread.hpp"
#include "testosthreads/TestMutexGuardThread.hpp" 
#include "testosthreads/TestRecursiveReaderWriter.hpp"
//#include <unistd.h>

#include "Foundation/osthreads/Thread.hpp"
#include "Foundation/osthreads/Mutex.hpp"
#include "Foundation/osthreads/NullMutex.hpp"
#include "Foundation/osthreads/ReaderWriterRecursiveMutex.hpp"
#include "Foundation/osthreads/ReaderRecursiveLockGuard.hpp"
#include "Foundation/osthreads/WriterRecursiveLockGuard.hpp"

namespace Cdmw
{

    namespace OsSupport
    {

        CPPUNIT_TEST_SUITE_REGISTRATION( TestMutexThreadDriver );

        TestMutexThreadDriver::TestMutexThreadDriver()
        {
            if (Cdmw::TestUtils::TestManager::instance()->get_property ("valid") == "yes") 
                m_valid_only = true;
            else
                m_valid_only = false;
            
            if (Cdmw::TestUtils::TestManager::instance()->get_property ("test") == "Mutex" 
                || Cdmw::TestUtils::TestManager::instance()->get_property ("test") == "all") 
                m_do_test = true;
            else
                m_do_test = false;
        }

        TestMutexThreadDriver::~TestMutexThreadDriver()
        {
        }


        void
        TestMutexThreadDriver::do_test_with_NullMutex()
        {

            TEST_INFO("Creating a NullMutex");
            NullMutex mutex;

            TEST_INFO("Locking the mutex");
            mutex.lock();

            TEST_INFO("Creating a thread with the mutex as reference and a variable to be changed");
            int valueToBeChanged(0);
            TestMutexThread testMutexThread(mutex, valueToBeChanged);

            TEST_INFO("Starting the thread");
            testMutexThread.start();

            TEST_INFO("Sleeping a while to see if the thread changes the variable");
            TEST_INFO("   the thread must not be blocked by the NullMutex");
            int timescale = Cdmw::TestUtils::get_timescale();
            Thread::sleep(timescale*2000);

            CPPUNIT_ASSERT( valueToBeChanged != 0);

            TEST_INFO("Unlocking the mutex");
            mutex.unlock();

        }



        void
        TestMutexThreadDriver::do_test_with_guard()
        {

            TEST_INFO("Creating a Mutex");
            Mutex mutex;

            TEST_INFO("check if the mutex is unlocked");
            CPPUNIT_ASSERT(!mutex.is_locked());

            TEST_INFO("Locking the mutex");
            mutex.lock();

            TEST_INFO("check if the mutex is locked");
            CPPUNIT_ASSERT(mutex.is_locked());

            TEST_INFO("Creating a thread using automatic lock/unlock of a thread");
            int valueToBeChanged(0);
            TestMutexGuardThread testMutexGuardThread(mutex, valueToBeChanged);

            TEST_INFO("Starting the thread");
            testMutexGuardThread.start();

            TEST_INFO("Sleeping a while to see if the thread changes the variable");
            int timescale = Cdmw::TestUtils::get_timescale();
            Thread::sleep(timescale*2000);

            CPPUNIT_ASSERT( valueToBeChanged == 0);

            TEST_INFO("Unlocking the mutex");
            mutex.unlock();

            TEST_INFO("check if the mutex is unlocked");
            CPPUNIT_ASSERT(!mutex.is_locked());

            TEST_INFO("Sleeping a while to see if the thread changes the variable");
            Thread::sleep(timescale*2000);

            CPPUNIT_ASSERT( valueToBeChanged != 0);

            TEST_INFO("Waiting for the completion of the thread");
            testMutexGuardThread.join();


        }



        void
        TestMutexThreadDriver::do_test_without_guard()
        {

            TEST_INFO("Creating a Mutex");
            Mutex mutex;

            TEST_INFO("check if the mutex is unlocked");
            CPPUNIT_ASSERT(!mutex.is_locked());

            TEST_INFO("Locking the mutex");
            mutex.lock();

            TEST_INFO("check if the mutex is locked");
            CPPUNIT_ASSERT(mutex.is_locked());

            TEST_INFO("Creating a thread with the mutex as reference and a variable to be changed");
            int valueToBeChanged(0);
            TestMutexThread testMutexThread(mutex, valueToBeChanged);

            CPPUNIT_ASSERT(testMutexThread.get_status() == THREAD_CREATED);

            TEST_INFO("Starting the thread");
            testMutexThread.start();

            TEST_INFO("Sleeping a while to wait for the thread startup");
            int timescale = Cdmw::TestUtils::get_timescale();
            Thread::sleep(timescale*2000);
            CPPUNIT_ASSERT( valueToBeChanged == 0);

            TEST_INFO("Unlocking the mutex");
            mutex.unlock();

            TEST_INFO("check if the mutex is unlocked");
            CPPUNIT_ASSERT(!mutex.is_locked());

            TEST_INFO("Sleeping a while to see if the thread changes the variable");

            Thread::sleep(timescale*2000);
            CPPUNIT_ASSERT(testMutexThread.get_status() == THREAD_ENDED);
            CPPUNIT_ASSERT( valueToBeChanged != 0);

            TEST_INFO("Waiting for the completion of the thread");
            testMutexThread.join();
        }


        void
        TestMutexThreadDriver::do_test_recursive_mutex()
        {
            TEST_INFO("Creating a non recursive Mutex");
            Mutex mutex;

            TEST_INFO("check if the mutex is unlocked");
            CPPUNIT_ASSERT(!mutex.is_locked());

            TEST_INFO("Locking the mutex");
            mutex.lock();

            TEST_INFO("check if the mutex is locked");
            CPPUNIT_ASSERT(mutex.is_locked());

            TEST_INFO("Trying to lock twice the same mutex");

            try
            {

                mutex.lock();

            }
            catch (Cdmw::AlreadyDoneException& )
            {

                CPPUNIT_ASSERT(true);

            }
            catch (...)
            {
                CPPUNIT_ASSERT(false);
            }


            TEST_INFO("Unlocking the mutex");
            mutex.unlock();

            TEST_INFO("check if the mutex is unlocked");
            CPPUNIT_ASSERT(!mutex.is_locked());

        }

        void
        TestMutexThreadDriver::do_test_recursive_read_write_mutex()
        {
            TEST_INFO("Creating recursive reader writer Mutex");
            Cdmw::OsSupport::ReaderWriterRecursiveMutex mutex;
            
        //READ TEST    
            TEST_INFO("Test R(read)");
        TestRecursiveReaderWriter* p1 = new TestRecursiveReaderWriter(1, 2, 1, &mutex);
        if(mutex.isReadLocked()){
            CPPUNIT_ASSERT(false);
        }
        p1->start();
        sleep(1);
        if(!mutex.isReadLocked()){
            CPPUNIT_ASSERT(false);
        }
        p1->join();
        if(mutex.isReadLocked()){
            CPPUNIT_ASSERT(false);
        }
        CPPUNIT_ASSERT(true);
        
        //WRITE TEST    
        TEST_INFO("Test W (write)");
        TestRecursiveReaderWriter* p2 = new TestRecursiveReaderWriter(1, 2, 2, &mutex);
        if(mutex.isWriteLocked()){
            CPPUNIT_ASSERT(false);
        }
        p2->start();
        sleep(1);
        if(!mutex.isWriteLocked()){
            CPPUNIT_ASSERT(false);
        }
        p2->join();
        if(mutex.isWriteLocked()){
            CPPUNIT_ASSERT(false);
        }
        CPPUNIT_ASSERT(true);
        
        //READ READ TEST
        TEST_INFO("Test RR(read,read)");
        TestRecursiveReaderWriter* p3 = new TestRecursiveReaderWriter(1, 2, 3, &mutex);
        p3->start();
        sleep(5);
        if(!p3->isdone()){
            CPPUNIT_ASSERT(false);
        };
        
        //READ WRITE TEST
        TEST_INFO("Test WR(write,read)");
        TestRecursiveReaderWriter* p4 = new TestRecursiveReaderWriter(1, 2, 4, &mutex);
        p4->start();
        sleep(5);
        if(!p4->isdone()){
            CPPUNIT_ASSERT(false);
        };
        
        //WRITE READ TEST
        TEST_INFO("Test RW(read,write)");
        TestRecursiveReaderWriter* p5 = new TestRecursiveReaderWriter(1, 2, 5, &mutex);
        p5->start();
        sleep(5);
        if(!p5->isdone()){
            CPPUNIT_ASSERT(false);
        };
        
        //WRITE WRITE TEST
        TEST_INFO("Test WW(write,write)");
        TestRecursiveReaderWriter* p6 = new TestRecursiveReaderWriter(1, 2, 6, &mutex);
        p6->start();
        sleep(5);
        if(!p6->isdone()){
            CPPUNIT_ASSERT(false);
        };
        
        //T1(read)T2(Read)
        TEST_INFO("Test T1(read)T2(Read)");
        TestRecursiveReaderWriter* p7 = new TestRecursiveReaderWriter(1, 1, 7, &mutex);
        p7->start();
        TestRecursiveReaderWriter* p8 = new TestRecursiveReaderWriter(2, 1, 8, &mutex);
        p8->start();
        sleep(4);
        if(!p7->isdone() || !p8->isdone()){
            CPPUNIT_ASSERT(false);
        };
        
        //T1(read)T2(write)
        TEST_INFO("Test T1(read)T2(write)");
        TestRecursiveReaderWriter* p9 = new TestRecursiveReaderWriter(1, 1, 7, &mutex);
        p9->start();
        TestRecursiveReaderWriter* p10 = new TestRecursiveReaderWriter(2, 1, 9, &mutex);
        p10->start();
        sleep(4);
        if(!p9->isdone() || p10->isdone()){
            CPPUNIT_ASSERT(false);
        };
        sleep(2);
        if(!p9->isdone() || !p10->isdone()){
            CPPUNIT_ASSERT(false);
        };
        
        //T1(write)T2(read)
        TEST_INFO("Test T1(write)T2(read)");
        TestRecursiveReaderWriter* p11 = new TestRecursiveReaderWriter(1, 1, 10, &mutex);
        p11->start();
        TestRecursiveReaderWriter* p12 = new TestRecursiveReaderWriter(2, 1, 8, &mutex);
        p12->start();
        sleep(4);
        if(!p11->isdone() || p12->isdone()){
            CPPUNIT_ASSERT(false);
        };
        sleep(2);
        if(!p11->isdone() || !p12->isdone()){
            CPPUNIT_ASSERT(false);
        };
        
        //T1(write)T2(write)
        TEST_INFO("Test T1(write)T2(write)");
        TestRecursiveReaderWriter* p13 = new TestRecursiveReaderWriter(1, 1, 10, &mutex);
        p13->start();
        TestRecursiveReaderWriter* p14 = new TestRecursiveReaderWriter(2, 1, 9, &mutex);
        p14->start();
        sleep(4);
        if(!p13->isdone() || p14->isdone()){
            CPPUNIT_ASSERT(false);
        };
        sleep(2);
        if(!p13->isdone() || !p14->isdone()){
            CPPUNIT_ASSERT(false);
        };
        CPPUNIT_ASSERT(true);
        }
      

        void
        TestMutexThreadDriver::do_tests()
        {
            if (!m_do_test)
                return;

            // set number of requested successfull tests
//             if (m_valid_only) set_nbOfRequestedTestOK (7);
//             else set_nbOfRequestedTestOK (17);

            do_test_without_guard();
            if (!m_valid_only)
            {
              do_test_with_guard();
              do_test_with_NullMutex();
              do_test_recursive_mutex();
              do_test_recursive_read_write_mutex();
            }

        }

    } // End namespace OsSupport

} // End namespace Cdmw



