/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#include "Foundation/common/MacroDefs.hpp"

#include "testosthreads/TestThreadDriver.hpp"

#include "Foundation/osthreads/Thread.hpp"

#include "testosthreads/TestThread.hpp"

#include <sstream>

#if defined (CDMW_POSIX_THREAD)
#include <unistd.h>
#include <sched.h>
#endif

namespace Cdmw
{

    namespace OsSupport
    {


        CPPUNIT_TEST_SUITE_REGISTRATION( TestThreadDriver );

        TestThreadDriver::TestThreadDriver()
        {
            if (Cdmw::TestUtils::TestManager::instance()->get_property ("valid") == "yes") 
                m_valid_only = true;
            else
                m_valid_only = false;
            
            if (Cdmw::TestUtils::TestManager::instance()->get_property ("test") == "Thread" 
                || Cdmw::TestUtils::TestManager::instance()->get_property ("test") == "all") 
                m_do_test = true;
            else
                m_do_test = false;
        }


        TestThreadDriver::~TestThreadDriver()
        {
        }


        void
        TestThreadDriver::do_tests()
        {
            if (!m_do_test)
                return;

            int nbOfRequestedTestOK = 0;
            int nbOfRequestedTestOKMemo;

       
            // --------------------------------------------------------------------
            // Thread creation
            // --------------------------------------------------------------------
        
            nbOfRequestedTestOK += 2;

            TEST_INFO(" ");
            TEST_INFO("Creating a variable with value 0");
            int valueToBeChanged(0);
      
            TEST_INFO("Starting a new thread with a reference at this");
            TestThread testThread_1("Thread_1", 1, valueToBeChanged, false);
            Thread::ThreadId threadId1 = testThread_1.start();

            {
                std::stringstream buffer;
                buffer << "  Thread 1 Id : " << threadId1 << std::endl;
                TEST_INFO(buffer.str());
            }

            TEST_INFO("Sleeping a while to wait the thread changes the variable");
            int timescale = Cdmw::TestUtils::get_timescale();
            Thread::sleep(timescale*2000);

            CPPUNIT_ASSERT (testThread_1.get_status() == THREAD_ENDED);
            CPPUNIT_ASSERT (valueToBeChanged != 0);


            // --------------------------------------------------------------------
            // Thread join
            // --------------------------------------------------------------------
            nbOfRequestedTestOK += 1;

            TEST_INFO(" ");
            TEST_INFO("Trying to wait for the thread completion");
            testThread_1.join();
            CPPUNIT_ASSERT(true);



            // --------------------------------------------------------------------
            // Thread stack change
            // --------------------------------------------------------------------
#   if defined (CDMW_HAVE_THREAD_STACK)
        
            nbOfRequestedTestOK += 4;

            // Must be >= PTHREAD_STACK_MIN.
            // A multiple of 16384 (or is it of PTHREAD_STACK_MIN?) seems better
            const size_t STACK_SIZE = 32768;

            TEST_INFO(" ");
            TEST_INFO("Trying to change stack of an already started thread");

            try
            {
                testThread_1.set_stackSize(STACK_SIZE);
                CPPUNIT_ASSERT(false);

            }
            catch (const Cdmw::AlreadyDoneException&)
            {
                CPPUNIT_ASSERT(true);

            }
            catch ( ... )
            {
                CPPUNIT_ASSERT(false);
            }
        
            TEST_INFO("Trying to change stack of a new thread");
            TestThread testThread_2 ("Thread_2", 2, valueToBeChanged, false);

            try
            {
                testThread_2.set_stackSize(STACK_SIZE);
                CPPUNIT_ASSERT(true);

            }
            catch ( const Cdmw::AlreadyDoneException& )
            {
                std::cout << "Cdmw::AlreadyDoneException" << std::endl;
                CPPUNIT_ASSERT(false);

            }
            catch (const Cdmw::InternalErrorException & ex)
            {
                TEST_INFO("Exception " << ex.what());
                CPPUNIT_ASSERT(false);

            }
            catch (const Cdmw::Exception& ex)
            {
                TEST_INFO("Exception " << ex.what());
                CPPUNIT_ASSERT(false);

            }
            catch ( ... )
            {
                std::cout << "Other Exception" << std::endl;
                CPPUNIT_ASSERT(false);
            }

            TEST_INFO("Trying to start the stack specified thread");

            try
            {
                Thread::ThreadId threadId2 = testThread_2.start();

                std::stringstream buffer;
                buffer << "  Thread 2 Id : " << threadId2 << std::endl;
                TEST_INFO(buffer.str());

                CPPUNIT_ASSERT(true);

            }
            catch (...)
            {
                CPPUNIT_ASSERT(false);
            }

            TEST_INFO("Trying to retreive the stack of a new thread");

            try
            {
                CPPUNIT_ASSERT (STACK_SIZE == testThread_2.get_stackSize());
            }
            catch (...)
            {
                CPPUNIT_ASSERT(false);
            }


            TEST_INFO("Waiting for the completion of the thread");
            testThread_2.join();
#   endif // defined( CDMW_HAVE_THREAD_STACK )

   
            // --------------------------------------------------------------------
            // Thread priority change
            // --------------------------------------------------------------------

            // REM1: with Linux this part (this section with testThread_3) is valid only if
            // the kernel can offer set_priority (2.4.x of RHEL3 can't, 2.6.x of RHEL4 can)
            // REM2: with Linux with a kernel that offer set_priority, the priority can't be
            // changed to a value>0 without first specifying a set_scopeSchedPolicy like
            // (Thread::THR_SYSTEM_SCOPE, Thread::THR_SCHEDULE_RR/FIFO..)            

/****** PCR-0323: deactivate priority tests as set_priority is not available with Linux kernel 2.4.x ******* Reactivated during RHEL4 porting */
            Thread::Priority thread_init_priority = 0;
/* Reactivated during RHEL4 porting ****** PCR-0323 *********************************/
            Thread::Priority thread_priority = 0;
            Thread::Priority priority = 0;

            TEST_INFO(" ");
            TEST_INFO("Trying to set the priority of a new thread");

            TestThread testThread_3 ("Thread_3", 2, valueToBeChanged, true);

#   if defined (CDMW_POSIX_THREAD)

/****** PCR-0323: deactivate priority tests as set_priority is not available with Linux kernel 2.4.x ******* Reactivated during RHEL4 porting */
            nbOfRequestedTestOK += 2;

            try
            {
                thread_init_priority = testThread_3.get_priority();
                priority = thread_init_priority + 1;
#       if defined (CDMW_POSIX_SOLARIS_THREAD)

                priority = (testThread_3.get_max_priority() + testThread_3.get_min_priority() ) / 2;
#       elif defined (CDMW_POSIX_AIX_THREAD)

                priority = (testThread_3.get_max_priority() + testThread_3.get_min_priority() ) / 2;
#       elif defined (CDMW_POSIX_LINUX_THREAD)

                priority = 0;
#       endif

                testThread_3.set_priority(priority);
                CPPUNIT_ASSERT(true);

            }
            catch (const Cdmw::AlreadyDoneException&)
            {
                CPPUNIT_ASSERT(false);

            }
            catch (const Cdmw::InternalErrorException & ex)
            {
                CPPUNIT_ASSERT(false);

            }
            catch (const Cdmw::Exception& ex)
            {
                CPPUNIT_ASSERT(false);
                TEST_INFO("Exception " << ex.what() );


            }
            catch ( ... )
            {
                CPPUNIT_ASSERT(false);
            }

            TEST_INFO("Trying to retreive the priority of the thread");

            thread_priority = testThread_3.get_priority();

            {
                std::stringstream buffer;
                buffer << " Thread  Init Priority : " << thread_init_priority
                << " Thread  Priority      : " << thread_priority
                << " Request Priority      : " << priority << std::endl;
                TEST_INFO(buffer.str());
            }

            CPPUNIT_ASSERT(thread_priority == priority);
/* Reactivated during RHEL4 porting ****** PCR-0323 ***********************************/

#   endif

            // start thread

            TEST_INFO(" ");

            nbOfRequestedTestOK += 3;

            Thread::ThreadId threadId3 = testThread_3.start();

            {
                std::stringstream buffer;
                buffer << "  Thread 3 Id : " << threadId3 << std::endl;
                TEST_INFO(buffer.str());
            }

            Thread::sleep(timescale*2000);

/****** PCR-0323: deactivate priority tests as set_priority is not available with Linux kernel 2.4.x ******* Reactivated during RHEL4 porting */
            TEST_INFO("Trying to change priority of started thread");

            try
            {
                thread_init_priority = testThread_3.get_priority();
                priority = thread_init_priority + 1;

#       if defined (CDMW_POSIX_SOLARIS_THREAD)

                priority = (testThread_3.get_max_priority() + testThread_3.get_min_priority() ) / 2 + 2;
#       elif defined (CDMW_POSIX_AIX_THREAD)

                priority = (testThread_3.get_max_priority() + testThread_3.get_min_priority() ) / 2 + 2;
#       elif defined (CDMW_POSIX_LINUX_THREAD)

                priority = 0;
#       elif defined(_WIN32)

                priority = 1;
#       endif

                testThread_3.set_priority(priority);
                CPPUNIT_ASSERT(true);

            }
            catch (const ThreadNotExistException&)
            {
                CPPUNIT_ASSERT(false);

            }
            catch (const Cdmw::InternalErrorException & ex)
            {
                CPPUNIT_ASSERT(false);

            }
            catch (const Cdmw::Exception& ex)
            {
                CPPUNIT_ASSERT(false);
                TEST_INFO("Exception " << ex.what() );

            }
            catch ( ... )
            {
                CPPUNIT_ASSERT(false);
            }

            TEST_INFO("Trying to retreive the priority of the thread");

            thread_priority = testThread_3.get_priority();

            {
                std::stringstream buffer;
                buffer << " Thread  Init Priority : " << thread_init_priority
                << " Thread  Priority      : " << thread_priority
                << " Request Priority      : " << priority << std::endl;
                TEST_INFO(buffer.str());
            }

            CPPUNIT_ASSERT(thread_priority == priority);
/* Reactivated during RHEL4 porting ****** PCR-0323 *******************************/

            // stop thread
            testThread_3.stop_loop();

            TEST_INFO("Waiting for the completion of the thread");
            testThread_3.join();

            TEST_INFO("Check Thread ident get by self()");
            Thread::ThreadId threadId3_self = testThread_3.get_thr_ident();
            std::stringstream buffer;
            buffer << "  Thread 3 Id Self : " << threadId3_self << std::endl;
            TEST_INFO(buffer.str());

            CPPUNIT_ASSERT(threadId3_self == threadId3);

#   if defined (CDMW_POSIX_THREAD)


            // --------------------------------------------------------------------
            // set the contention scope and scheduling policy
            // --------------------------------------------------------------------
       
            nbOfRequestedTestOK += 1;

            // priority is set by the thread itself
            priority = 10;

            TEST_INFO(" ");
            TEST_INFO("Trying to change scope and policy of an already started thread");

            try
            {
                testThread_1.set_scopeSchedPolicy (Thread::THR_PROCESS_SCOPE, Thread::THR_SCHEDULE_FIFO);
                CPPUNIT_ASSERT(false);

            }
            catch (const Cdmw::AlreadyDoneException&)
            {
                CPPUNIT_ASSERT(true);

            }
            catch (const Cdmw::InternalErrorException & ex)
            {
                CPPUNIT_ASSERT(false);

            }
            catch ( ... )
            {
                CPPUNIT_ASSERT(false);
            }
       
            // This part is only for unit test
            if (!m_valid_only)
            {

                nbOfRequestedTestOK += 1;

                TEST_INFO("Trying to set scope and scheduling policy of a new thread");

                TEST_INFO("PROCESS SCOPE and SCHED OTHER");
                TestThread testThread_4 ("Thread_4", 5, valueToBeChanged, true);

                try
                {
                    testThread_4.set_scopeSchedPolicy (Thread::THR_PROCESS_SCOPE, Thread::THR_SCHEDULE_OTHER);
                    CPPUNIT_ASSERT(true);

                }
                catch (const Cdmw::InternalErrorException & ex)
                {
                    CPPUNIT_ASSERT(false);

                }
                catch (const Cdmw::BadParameterException&)
                {
#       if defined (CDMW_POSIX_LINUX_THREAD)
                    // PROCESS SCOPE is not implemented on Linux
                    TEST_INFO("Policy not implemented on Linux");
                    CPPUNIT_ASSERT(true);
#       else

                    CPPUNIT_ASSERT(false);
#       endif

                }
                catch (const Cdmw::Exception& ex)
                {
                    CPPUNIT_ASSERT(false);
                    TEST_INFO("Exception " << ex.what());

                }
                catch ( ... )
                {
                    CPPUNIT_ASSERT(false);
                }

#   if !defined (CDMW_POSIX_LINUX_THREAD)



                nbOfRequestedTestOKMemo = nbOfRequestedTestOK;
         
                nbOfRequestedTestOK += 4;

                TEST_INFO(" ");

                try
                {
                    // start thread
                    TEST_INFO("Trying to start the thread");
                    testThread_4.start();

                    Thread::sleep(timescale*2000);

                    TEST_INFO("Trying to retreive the priority of the thread");

                    thread_priority = testThread_4.get_priority();

                    CPPUNIT_ASSERT(thread_priority == priority);

                    TEST_INFO("Trying to retreive the scope and policy of the thread");
                    Thread::ThrScope scope;
                    Thread::ThrSchedulPolicy policy;

                    testThread_4.get_scopeSchedPolicy (scope, policy);
                    CPPUNIT_ASSERT(scope == Thread::THR_PROCESS_SCOPE);
                    CPPUNIT_ASSERT(policy == Thread::THR_SCHEDULE_OTHER);
                    // stop thread
                    testThread_4.stop_loop();

                    testThread_4.join();
                    CPPUNIT_ASSERT(true);

                }
                catch (const Cdmw::InternalErrorException & ex)
                {
                    CPPUNIT_ASSERT(false);

                }
                catch (const Cdmw::Exception& ex)
                {
                    CPPUNIT_ASSERT(false);
                    TEST_INFO("Exception " << ex.what());

                }
                catch ( ... )
                {
                    CPPUNIT_ASSERT(false);
                }

         
#   endif
                nbOfRequestedTestOK += 1;

                TEST_INFO(" ");

                TEST_INFO("PROCESS SCOPE and SCHED FIFO");

                TestThread testThread_5 ("Thread_5", 5, valueToBeChanged, true);

                try
                {
                    testThread_5.set_scopeSchedPolicy (Thread::THR_PROCESS_SCOPE, Thread::THR_SCHEDULE_FIFO);

                    CPPUNIT_ASSERT(true);

                }
                catch (const Cdmw::InternalErrorException & ex)
                {
                    CPPUNIT_ASSERT(false);

                }
                catch (const Cdmw::BadParameterException&)
                {
#       if defined (CDMW_POSIX_LINUX_THREAD)
                    // PROCESS SCOPE is not implemented on Linux
                    TEST_INFO("Policy not implemented on Linux");
                    CPPUNIT_ASSERT(true);
#       else

                    CPPUNIT_ASSERT(false);
#       endif

                }
                catch (const Cdmw::Exception& ex)
                {
                    CPPUNIT_ASSERT(false);
                    TEST_INFO("Exception " << ex.what());

                }
                catch ( ... )
                {
                    CPPUNIT_ASSERT(false);
                }

#   if !defined (CDMW_POSIX_LINUX_THREAD)
        
                nbOfRequestedTestOK += 2;

                TEST_INFO(" ");

                try
                {
                    // start thread
                    TEST_INFO("Trying to start the thread");
                    testThread_5.start();

                    Thread::sleep(timescale*2000);

                    TEST_INFO("Trying to retreive the priority of the thread");

                    thread_priority = testThread_5.get_priority();

                    CPPUNIT_ASSERT(thread_priority == priority);

                    // stop thread
                    testThread_5.stop_loop();

                    testThread_5.join();
                    CPPUNIT_ASSERT(true);


                }
                catch (const Cdmw::InternalErrorException & ex)
                {
                    CPPUNIT_ASSERT(false);

                }
                catch (const Cdmw::Exception& ex)
                {
                    CPPUNIT_ASSERT(false);
                    TEST_INFO("Exception " << ex.what());

                }
                catch ( ... )
                {
                    CPPUNIT_ASSERT(false);
                }
        
#   endif

                nbOfRequestedTestOK += 1;

                TEST_INFO(" ");

                TEST_INFO("PROCESS SCOPE and SCHED RR");

                TestThread testThread_6 ("Thread_6", 5, valueToBeChanged, true);

                try
                {
                    testThread_6.set_scopeSchedPolicy (Thread::THR_PROCESS_SCOPE, Thread::THR_SCHEDULE_RR);

                    CPPUNIT_ASSERT(true);

                }
                catch (const Cdmw::InternalErrorException & ex)
                {
                    CPPUNIT_ASSERT(false);

                }
                catch (const Cdmw::BadParameterException&)
                {
#       if defined (CDMW_POSIX_LINUX_THREAD)
                    // PROCESS SCOPE is not implemented on Linux
                    TEST_INFO("Policy not implemented on Linux");
                    CPPUNIT_ASSERT(true);
#       else

                    CPPUNIT_ASSERT(false);
#       endif

                }
                catch (const Cdmw::Exception& ex)
                {
                    CPPUNIT_ASSERT(false);
                    TEST_INFO("Exception " << ex.what());

                }
                catch ( ... )
                {
                    CPPUNIT_ASSERT(false);
                }

#   if !defined (CDMW_POSIX_LINUX_THREAD)

                nbOfRequestedTestOK += 2;

                TEST_INFO(" ");

                try
                {
                    // start thread
                    TEST_INFO("Trying to start the thread");
                    testThread_6.start();

                    Thread::sleep(timescale*2000);

                    TEST_INFO("Trying to retreive the priority of the thread");

                    thread_priority = testThread_6.get_priority();

                    CPPUNIT_ASSERT(thread_priority == priority);

                    // stop thread
                    testThread_6.stop_loop();

                    testThread_6.join();
                    CPPUNIT_ASSERT(true);

                }
                catch (const Cdmw::InternalErrorException & ex)
                {
                    CPPUNIT_ASSERT(false);

                }
                catch (const Cdmw::Exception& ex)
                {
                    CPPUNIT_ASSERT(false);
                    TEST_INFO("Exception " << ex.what());

                }
                catch ( ... )
                {
                    CPPUNIT_ASSERT(false);
                }

#   endif



            } // end of if (!m_valid_only) of test Thread 4

                nbOfRequestedTestOK += 1;

                TEST_INFO(" ");

                TEST_INFO("SYSTEM SCOPE and SCHED OTHER");

                TestThread testThread_7 ("Thread_7", 5, valueToBeChanged, true);

                try
                {
                    testThread_7.set_scopeSchedPolicy (Thread::THR_SYSTEM_SCOPE, Thread::THR_SCHEDULE_OTHER);

                    int max_priority = ::sched_get_priority_max(SCHED_OTHER);
                    int min_priority = ::sched_get_priority_min(SCHED_OTHER);
                    std::stringstream buffer;
                    buffer << "euid : " << ::geteuid()
                           << " Max Priority : " << max_priority
                           << " Min Priority : " << min_priority
                           << " Max Cdmw Priority : " << testThread_7.get_max_priority()
                           << " Min Cdmw Priority : " << testThread_7.get_min_priority()
                           << std::endl;
                    TEST_INFO(buffer.str());

                    CPPUNIT_ASSERT(true);

                }
                catch (const Cdmw::InternalErrorException & ex)
                {
                    CPPUNIT_ASSERT(false);

                }
                catch (const Cdmw::Exception& ex)
                {
                    CPPUNIT_ASSERT(false);
                    TEST_INFO("Exception " << ex.what());

                }
                catch ( ... )
                {
                    CPPUNIT_ASSERT(false);
                }


                nbOfRequestedTestOKMemo = nbOfRequestedTestOK;
                nbOfRequestedTestOK += 4;

                try
                {
                    // start thread
                    TEST_INFO("Trying to start the thread");
                    testThread_7.start();

                    Thread::sleep(timescale*2000);

                    TEST_INFO("Trying to retreive the priority of the thread");

#       if defined (CDMW_POSIX_SOLARIS_THREAD)

                    priority = 10;
#       elif defined (CDMW_POSIX_AIX_THREAD)

                    priority = 10;
#       elif defined (CDMW_POSIX_LINUX_THREAD)

                    priority = 0;
#       endif

                    thread_priority = testThread_7.get_priority();

                    CPPUNIT_ASSERT(thread_priority == priority);

                    TEST_INFO("Trying to retreive the scope and policy of the thread");
                    Thread::ThrScope scope;
                    Thread::ThrSchedulPolicy policy;

                    testThread_7.get_scopeSchedPolicy (scope, policy);
                    CPPUNIT_ASSERT(scope == Thread::THR_SYSTEM_SCOPE);
                    CPPUNIT_ASSERT(policy == Thread::THR_SCHEDULE_OTHER);

                    // stop thread
                    testThread_7.stop_loop();

                    testThread_7.join();
                    CPPUNIT_ASSERT(true);

                }
                catch (const Cdmw::InternalErrorException & ex)
                {
                    CPPUNIT_ASSERT(false);

                }
                catch (const Cdmw::Exception& ex)
                {
                    CPPUNIT_ASSERT(false);
                    TEST_INFO("Exception " << ex.what());

                }
                catch ( ... )
                {
                    CPPUNIT_ASSERT(false);
                }


            // This part is only for unit test

            // With Linux this part (the following two sections with testThread_8 and testThread_9)
            // is completly tested only if both 2 conditions are present :
            // 1) the kernel can offer set_priority (2.4.x of RHEL3 can't, 2.6.x of RHEL4 can)
            // 2) we must run test a super user
            // TIP: to run as superuser you can do the following:
            //      "su" from the user account that has compiled Cardamom
            //      then do a symbolic link ~root/.CDMW to your Cardamom user .CDMW
            //      then run the test with "runtest" build target

            if (!m_valid_only)
            {

                nbOfRequestedTestOK += 1;

                TEST_INFO(" ");
                TEST_INFO("SYSTEM SCOPE and SCHED FIFO");
                TestThread testThread_8 ("Thread_8", 10, valueToBeChanged, true);

                try
                {
            testThread_8.set_scopeSchedPolicy (Thread::THR_SYSTEM_SCOPE, Thread::THR_SCHEDULE_FIFO);

                    int max_priority = ::sched_get_priority_max(SCHED_FIFO);
                    int min_priority = ::sched_get_priority_min(SCHED_FIFO);
                    std::stringstream buffer;
                    buffer << "euid : " << ::geteuid()
                           << " Max Priority : " << max_priority
                           << " Min Priority : " << min_priority
                           << " Max Cdmw Priority : " << testThread_8.get_max_priority()
                           << " Min Cdmw Priority : " << testThread_8.get_min_priority()
                           << std::endl;
                    TEST_INFO(buffer.str());

#       if defined (CDMW_POSIX_LINUX_THREAD)

/****** PCR-0323: deactivate priority tests as set_priority is not available with Linux kernel 2.4.x ******* Reactivated during RHEL4 porting*/
                    TEST_INFO("Trying to change priority of created thread");
                    priority = 1;
                    testThread_8.set_priority(priority);
/* Reactivated during RHEL4 porting ****** PCR-0323 ***************************/

#       endif

                    CPPUNIT_ASSERT(true);

                }
                catch (const Cdmw::InternalErrorException & ex)
                {
                    CPPUNIT_ASSERT(false);

                }
                catch (const Cdmw::Exception& ex)
                {
                    CPPUNIT_ASSERT(false);
                    TEST_INFO("Exception " << ex.what());

                }
                catch ( ... )
                {
                    CPPUNIT_ASSERT(false);
                }
          
                nbOfRequestedTestOKMemo = nbOfRequestedTestOK;
        
                nbOfRequestedTestOK += 2;

                try
                {
                    // start thread
                    TEST_INFO("Trying to start the thread");
                    testThread_8.start();

                    Thread::sleep(timescale*2000);

/****** PCR-0323: deactivate priority tests as set_priority is not available with Linux kernel 2.4.x ******* Reactivated during RHEL4 porting */
                    TEST_INFO("Trying to change priority of started thread");
                    priority = testThread_8.get_priority();
                    priority ++;
                    testThread_8.set_priority(priority);

                    TEST_INFO("Trying to retreive the priority of the thread");
                    CPPUNIT_ASSERT(testThread_8.get_priority() == priority);
/* Reactivated during RHEL4 porting ****** PCR-0323 *************************/
        
                    // stop thread
                    testThread_8.stop_loop();

                    testThread_8.join();
                    CPPUNIT_ASSERT(true);

                }
                catch (const ThreadSchedulingException& ex)
                {

#   if defined (CDMW_POSIX_SOLARIS_THREAD)

                    nbOfRequestedTestOK = nbOfRequestedTestOKMemo;
                    nbOfRequestedTestOK += 1;

                    // SYSTEM SCOPE and SCHED FIFO is not permitted on Posix Solaris (must be superuser)
                    TEST_INFO("Policy not permitted on Posix Solaris as user");
                    TEST_INFO("Exception " << ex.what());
                    CPPUNIT_ASSERT(true);
#   elif defined (CDMW_POSIX_LINUX_THREAD)

                    nbOfRequestedTestOK = nbOfRequestedTestOKMemo;
                    nbOfRequestedTestOK += 1;

                    // SYSTEM SCOPE and SCHED FIFO is not permitted on Linux (must be superuser)
                    TEST_INFO("Policy not permitted on Linux as user");
                    TEST_INFO("Exception " << ex.what());
                    TEST_INFO("To run full test of this part you must su root. See comments in code");
                    TEST_INFO("The exception and the pthread_create status error are normal");
                    CPPUNIT_ASSERT(true);
#   elif defined (CDMW_POSIX_AIX_THREAD)

                    nbOfRequestedTestOK = nbOfRequestedTestOKMemo;
                    nbOfRequestedTestOK += 1;

                    // SYSTEM SCOPE and SCHED FIFO is not permitted on AIX (must be superuser)
                    TEST_INFO("Policy not permitted on AIX as user");
                    TEST_INFO("Exception " << ex.what());
                    CPPUNIT_ASSERT(true);
#   else

                    CPPUNIT_ASSERT(false);
#   endif

                }
                catch (const Cdmw::InternalErrorException & ex)
                {
                    CPPUNIT_ASSERT(false);

                }
                catch (const Cdmw::Exception& ex)
                {
                    CPPUNIT_ASSERT(false);
                    TEST_INFO("Exception " << ex.what());

                }
                catch ( ... )
                {
                    CPPUNIT_ASSERT(false);
                }
          
        
                nbOfRequestedTestOK += 1;
        
                TEST_INFO(" ");
                TEST_INFO("SYSTEM SCOPE and SCHED RR");
                TestThread testThread_9 ("Thread_9", 10, valueToBeChanged, true);

                try
                {
                    testThread_9.set_scopeSchedPolicy (Thread::THR_SYSTEM_SCOPE, Thread::THR_SCHEDULE_RR);

                    int max_priority = ::sched_get_priority_max(SCHED_RR);
                    int min_priority = ::sched_get_priority_min(SCHED_RR);
                    std::stringstream buffer;
                    buffer << "euid : " << ::geteuid()
                           << " Max Priority : " << max_priority
                           << " Min Priority : " << min_priority
                           << " Max Cdmw Priority : " << testThread_9.get_max_priority()
                           << " Min Cdmw Priority : " << testThread_9.get_min_priority()
                           << std::endl;

                    TEST_INFO(buffer.str());

#       if defined (CDMW_POSIX_LINUX_THREAD)

/****** PCR-0323: deactivate priority tests as set_priority is not available with Linux kernel 2.4.x ******* Reactivated during RHEL4 porting */
                    TEST_INFO("Trying to change priority of created thread");
                    priority = 1;
                    testThread_9.set_priority(priority);
/* Reactivated during RHEL4 porting ****** PCR-0323 ****************************/

#       endif

                    CPPUNIT_ASSERT(true);

                }
                catch (const Cdmw::InternalErrorException & ex)
                {
                    CPPUNIT_ASSERT(false);

                }
                catch (const Cdmw::Exception& ex)
                {
                    CPPUNIT_ASSERT(false);
                    TEST_INFO("Exception " << ex.what());

                }
                catch ( ... )
                {
                    CPPUNIT_ASSERT(false);
                }

                nbOfRequestedTestOKMemo = nbOfRequestedTestOK;
                nbOfRequestedTestOK += 2;

                try
                {
                    // start thread
                    TEST_INFO("Trying to start the thread");
                    testThread_9.start();

                    Thread::sleep(timescale*2000);

/****** PCR-0323: deactivate priority tests as set_priority is not available with Linux kernel 2.4.x ******* Reactivated during RHEL4 porting */
                    TEST_INFO("Trying to change priority of started thread");
                    priority = testThread_9.get_priority();
                    priority ++;
                    testThread_9.set_priority(priority);

                    TEST_INFO("Trying to retreive the priority of the thread");
                    CPPUNIT_ASSERT(testThread_9.get_priority() == priority);
/* Reactivated during RHEL4 porting ****** PCR-0323 ***************************/

                    // stop thread
                    testThread_9.stop_loop();

                    testThread_9.join();
                    CPPUNIT_ASSERT(true);

                }
                catch (const Cdmw::InternalErrorException & ex)
                {
                    CPPUNIT_ASSERT(false);

                }
                catch (const ThreadSchedulingException& ex)
                {

#   if defined (CDMW_POSIX_SOLARIS_THREAD)

                    nbOfRequestedTestOK = nbOfRequestedTestOKMemo;
                    nbOfRequestedTestOK += 1;

                    // SYSTEM SCOPE and SCHED RR is not permitted on Posix Solaris (must be superuser)
                    TEST_INFO("Policy not permitted on Posix Solaris as user");
                    TEST_INFO("Exception " << ex.what());
                    CPPUNIT_ASSERT(true);
#   elif defined (CDMW_POSIX_LINUX_THREAD)

                    nbOfRequestedTestOK = nbOfRequestedTestOKMemo;
                    nbOfRequestedTestOK += 1;

                    // SYSTEM SCOPE and SCHED RR is not permitted on Linux (must be superuser)
                    TEST_INFO("Policy not permitted on Linux as user");
                    TEST_INFO("Exception " << ex.what());
                    TEST_INFO("To run full test of this part you must su root. See comments in code");
                    TEST_INFO("The exception and the pthread_create status error are normal");
                    CPPUNIT_ASSERT(true);
#   elif defined (CDMW_POSIX_AIX_THREAD)

                    nbOfRequestedTestOK = nbOfRequestedTestOKMemo;
                    nbOfRequestedTestOK += 1;

                    // SYSTEM SCOPE and SCHED RR is not permitted on AIX (must be superuser)
                    TEST_INFO("Policy not permitted on AIX as user");
                    TEST_INFO("Exception " << ex.what());
                    CPPUNIT_ASSERT(true);
#   else

                    CPPUNIT_ASSERT(false);
#   endif

                }
                catch (const Cdmw::Exception& ex)
                {
                    CPPUNIT_ASSERT(false);
                    TEST_INFO("Exception " << ex.what());

                }
                catch ( ... )
                {
                    CPPUNIT_ASSERT(false);
                }
                
#   endif

            } // end of if (!m_valid_only) set before test Thread 8

            // set number of requested successfull tests
//             set_nbOfRequestedTestOK (nbOfRequestedTestOK);

        }

    } // End namespace OsSupport
} // End namespace Cdmw



