/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

#ifndef TEST_MANAGER_HPP
#define TEST_MANAGER_HPP

#include <cppunit/extensions/HelperMacros.h>
#include <cppunit/TestResultCollector.h>
#include <Foundation/common/Properties.hpp>
#include <Foundation/testutils/ThreadedTestRunner.hpp>
#include <iostream>
#include <string>
#include <typeinfo>
#include <stdexcept>
#include <map>

#include <iomanip>

namespace Cdmw
{
   namespace TestUtils
   {
      /**
       * Return the multiplier coefficient of time
       */
      int get_timescale();

   
     
      class CppUnitSynchronizedObject : public CPPUNIT_NS::SynchronizedObject::SynchronizationObject
      {
         public:
            CppUnitSynchronizedObject()
            {
            }
             
            virtual ~CppUnitSynchronizedObject() 
            {
            }
             
            virtual void lock() 
            {
               m_mtx.lock();
            }
             
            virtual void unlock() 
            {
               m_mtx.unlock();
            }
             
         private:
            OsSupport::Mutex  m_mtx;
      };
    
      /**
       * Class to send parameter to test case.
       */
      class TestManager
      {
         public:
            /**
             * Constructor
             */
            TestManager();

            /**
             * Destructor
             */
            virtual ~TestManager()throw() ;

            /**
             * initialisation of the TestManager
             */
            int 
            init(int     argc,
                 char** argv);

            /**
             * Start Tests of registered test suite.
             */
            int
            run_tests();
      
            /**
             * return the instance of the TestManager
             */
            static  TestManager* 
            instance();
    
            /**
             * Name registration, useful if a test has a name
             * that is not executable's name.
             */
            void
            use_registration(bool registration=true)
            {
               m_is_registration_used = registration;
            }

            /**
             * verify if named registration it is used
             */
            bool
            is_registration_used() const
            {
               return m_is_registration_used;
            }
 
            /**
             * return if the test is OK or not
             */
            bool
            was_successful() const
            {
               return m_was_successful;
            }

            /**
             * set a string
             */
            void
            set_aux_info (std::string info);
        
            /**
             * get the string previously set
             */
            std::string
            get_aux_info ();
        
            /**
             * print the test report.
             * @param os output fot report
             */ 
            void 
            summary (std::ostream& os=std::cout);

            /**
             * Load the properties from a file. Returns false if the file
             * could not be opened.
             * @param file to load
             */
            bool 
            load(const char* file);
            
            /**
             * Store all registered properties into a stream.
             * @param os output for writing properties
             * @param comment additionnal comments 
             */
            void 
            store(std::ostream& os=std::cout, std::string comment="");
            
            /**
             * Registers a property for the test. It will be acessible from
             * anywhere.
             * @param name property name
             * @param value property value
             */
            void
            set_property(std::string name, std::string value);
            
            /**
             * Get a property value previously registered.
             * @param name property name to find
             * @param default_value value returned if not found
             */
            std::string
            get_property(std::string name, std::string default_value = "") const;
           
            /**
             * Return a list of all the properties.
             */
            void 
            property_names(Common::Properties::PropertyNames&) const;

            /**
             * Return the controller 
             */
            CppUnit::TestResult* get_controller();

            /**
             * Return a human readable type name
             */
            static std::string Get_typename(const std::type_info & typeinfo);
         private:
        
            mutable Common::Properties m_properties;

            CppUnit::TestResultCollector * m_result;
            
            //test controller.
            CppUnit::TestResult* m_controller;
        
            //used within cpp unint for thread-safe execution        
            Cdmw::TestUtils::CppUnitSynchronizedObject * m_CppUnitSyncObjCtrl;
            Cdmw::TestUtils::CppUnitSynchronizedObject * m_CppUnitSyncObjRes;

            std::string m_aux_info, 
               m_base_test_program_string;

            std::vector<std::string> m_test_list;

         protected:
            bool m_is_initialized,
               m_is_registration_used,
               m_was_successful;


            /**
             * Mutex to protect state, such as m_db, mutable because m_properties is mutable
             */
            mutable OsSupport::Mutex m_mutex_state;

      };
   } // end of TestUtils namespace
} // end of Cdmw

#define TEST_INFO(X) \
do { \
   std::ostringstream test_info_ostr__; \
   std::ios_base::fmtflags oldflags__ = test_info_ostr__.flags();       \
   test_info_ostr__ << Cdmw::TestUtils::TestManager::Get_typename(typeid(*this))  \
                    << " (" << std::right << std::setw(4) << std::setfill(' ') << __LINE__; \
   test_info_ostr__.flags(oldflags__); \
   test_info_ostr__ << ") : " << X; \
   std::cout << test_info_ostr__.str() << std::endl;   \
} while(0);


#endif // TEST_MANAGER_HPP

