/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

#ifndef THREADED_TEST_RUNNER_HPP
#define THREADED_TEST_RUNNER_HPP

#include <cppunit/extensions/HelperMacros.h>
#include <cppunit/TestResultCollector.h>
#include <cppunit/TestRunner.h>
#include "Foundation/osthreads/Thread.hpp"
#include <Foundation/common/Properties.hpp>

#include <iostream>
#include <string>
#include <typeinfo>
#include <stdexcept>
#include <map>
#include <list>

namespace Cdmw
{
    namespace TestUtils
    {

        /**
         * Class for registration of CppUnit::TestRunner objects
         * in order to delete them at process end.
         */
        class TestRunnerRegistration
        {
            public:
                /**
                 * Register a new CppUnit::TestRunner
                 */
                static void RegisterTestRunner(CppUnit::TestRunner* runner)
                {
                    Instance().m_list.push_back(runner);
                }
                
            private:
                typedef std::list<CppUnit::TestRunner*> RunnerList;
            
                TestRunnerRegistration() {};
                
                /**
                 * Static singleton pattern. It will be deleted at process end.
                 */
                static TestRunnerRegistration& Instance()
                {
                    static TestRunnerRegistration instance;
                    return instance;
                }
                
                
                ~TestRunnerRegistration()
                {
                    // delete each TestRunner
                    for (RunnerList::iterator it = m_list.begin();
                         it != m_list.end();
                         it++)
                    {
                        CppUnit::TestRunner* runner = *it;
                        delete runner;
                    }
                }
                
                RunnerList m_list;
        };


        
        /**
         * Class to send parameter to test case.
         */
        class ThreadedTestRunner : public Cdmw::OsSupport::Thread
        {
        public:
            /**
             * Constructor
             */
            ThreadedTestRunner(CppUnit::TestResult *controller)
            {
                m_controller = controller;
                // m_testRunner cannot be deleted in destructor, in case
                // ThreadedTestRunner is created again (core produced by CPPUnit
                // during second run, for unkown reason)
                // Thus, register each CppUnit::TestRunner object in 
                // TestRunnerRegistration singleton to be deleted at process end
                m_testRunner = new CppUnit::TestRunner();
                TestRunnerRegistration::RegisterTestRunner(m_testRunner);
            }

            /**
             * Constructor
             */
            ~ThreadedTestRunner() throw() {};

            /**
             * addTest
             */
            void addTest( CppUnit::Test *test )
            {
                m_testRunner->addTest(test);
            }
            

        protected:
            void run() throw ()
            {
                m_testRunner->run(*m_controller);
            }
        private:
            CppUnit::TestResult *m_controller;
            CppUnit::TestRunner *m_testRunner;

        };
        
    } // end of TestUtils namespace
} // end of Cdmw


#endif // THREADED_TEST_RUNNER_HPP

