/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

package cdmw.testutils;

import java.util.*;

import org.omg.CORBA.ORB;
import org.omg.CORBA.ORBPackage.InvalidName;
import org.omg.PortableServer.POA;

/**
 * Class provides ability to run JUnit tests. It also provides basic CORBA entities (POA, ORB) for user in order
 * to have ability to create tests which use CORBA.
 *
 */
public class CORBATestManager extends TestManager {

    /**
     * Base class of user exceptions that throws within CORBATestManager
     */
    public class CORBATestManagerException extends Exception {
        public CORBATestManagerException(
            String string) {
            super(string);
        }
    }

    public class InvalidObject extends CORBATestManagerException {
        public InvalidObject() {
            super("Not valid object!");
        }
    }

    public class InvalidIOR extends CORBATestManagerException {
        public InvalidIOR() {
            super("Not valid IOR!");
        }
    }

    public class InvalidArgument extends CORBATestManagerException {
        public InvalidArgument() {
            super("Not valid argument!");
        }
    }

    private Map<String, org.omg.CORBA.Object> db = Collections.synchronizedMap(new HashMap<String, org.omg.CORBA.Object>());

    /**
     * ORB
     */
    private ORB orb;

    /**
     * Root POA
     */
    private POA rootPOA;

    /** Indicates if the manager is initialized from command line */
    protected boolean isInitialized = false;

    /**
     * @return singleton instance of CORBATestManager
     */
    public static CORBATestManager instance() {
        synchronized (TestManager.class) {
            if (CORBATestManager.instance == null) {
                CORBATestManager.instance = new CORBATestManager();
            }
            
            return (CORBATestManager) CORBATestManager.instance;
        }
    }

    /**
     * Returns ORB
     * 
     * @return ORB
     */
    public ORB getORB() throws InvalidObject {
        if (!isInitialized) {
            System.err.print("Orb is non initialized!");
            throw new InvalidObject();
        }

        return orb;
    }

    /**
     * Returns root POA
     * 
     * @return POA
     */
    public POA getPOA() throws InvalidObject {
        if (!isInitialized) {
            System.err.print("POA is non initialized!");
            throw new InvalidObject();
        }
        return rootPOA;
    }

    /**
     * Initializes the singleton instance with the current test.
     * 
     * @param testName
     *            name of a test
     * @param orb
     *            ORB
     * @param poa
     *            root POA
     */
    public void init(String testName, ORB orb, POA poa) {
        super.init(testName);
        this.orb = orb;
        this.rootPOA = poa;

        this.isInitialized = true;
    }

    /**
     * Gets a service previously registered.
     * 
     * @param name object identifier
     *
     * @return CORBA.Object for the alias
     *
     * @throws InvalidObject when the name is unknown
     */
    public org.omg.CORBA.Object getObject(String name) throws InvalidObject {
        if (!isInitialized) {
            System.err.print("Orb is non initialized!");
            throw new InvalidObject();
        }

        if (null != db.get(name)) {
            org.omg.CORBA.Object ret = db.get(name)._duplicate();
            return ret;
        }

        throw new InvalidObject();
    }

    /**
     * Registers any services used in the test in order to get them
     * back from anywhere. A service is identified by an unique name.
     *
     * @param serviceAlias service identifier
     * @param obj service to register
     *
     * @throws InvalidObject when the object is unknown by the ORB
     */
    public void add(String serviceAlias, org.omg.CORBA.Object obj)
        throws InvalidObject {
        if (!isInitialized) {
            System.err.print("Orb is non initialized!");
            throw new InvalidObject();
        }

        if (obj == null) {
            org.omg.CORBA.Object object = null;
            try {
                object = orb.resolve_initial_references(serviceAlias);
            }
            catch (InvalidName e) {
                e.printStackTrace();
            }

            if (object == null)
                throw new InvalidObject();
            db.put(serviceAlias, object._duplicate());
        }
        else {
            db.put(serviceAlias, obj._duplicate());
        }
    }
    
    /**
     * @see add(String,org.omg.CORBA.Object)
     *
     * Associates the specified name to the object null.
     *
     * @param serviceAlias
     * @throws InvalidObject
     */
    public void add(String serviceAlias) throws InvalidObject {
        this.add(serviceAlias, null);
    }
    
}
