/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

package cdmw.testutils;

import java.util.*;

import org.junit.runner.*;
import org.junit.runner.notification.*;

public class TestManager {

    public static final String CDMW_TIMESCALE = "CDMW_TIMESCALE";

    public static final String CDMW_ORB_VDR = "CDMW_ORB_VDR";

    public static final String JACORB = "jacorb";
    
    /**
     * Class provides information: total count test were run, count of failured
     * tests
     */
    public class TestRunningListener extends RunListener {
        private int totalTestCount = 0;

        private int failureTestCount = 0;

        public synchronized void testStarted(Description description) {
            this.totalTestCount++ ;
        }

        public synchronized void testFailure(Failure failure) {
            this.failureTestCount++ ;
            System.err.println("Failure in test: " + failure.getTestHeader());
            System.err.println("The message is: " + failure.getMessage());
            System.err.println("Stack trace is: " + failure.getTrace());
        }

        public synchronized int getTotalTestsCount() {
            return this.totalTestCount;
        }

        public synchronized int getFailureTestCount() {
            return this.failureTestCount;
        }
    }
    
    /**
     * Key/value storage to be used to store properties
     */
    private final Map<String, Object> storage = Collections.synchronizedMap(new HashMap<String, Object>());
    
    /**
     * Thread-specific storage of TestManager instances.
     * It is intended to be used in multithread tests.
     */
    private static final ThreadLocal<TestManager> instance_holder = new ThreadLocal<TestManager>();

    /** singleton instance of TestManager */
    protected static TestManager instance = null;

    /**
     * core class to run JUnit tests
     */
    protected JUnitCore junitRunner = new JUnitCore();

    /**
     * listener for obtaining information of test session properties
     */
    protected TestRunningListener runningListener = new TestRunningListener();

    /**
     * List of names of tests to run
     */
    protected List<Class> testsToRun = new ArrayList<Class>();
      
    /**
     * Name of the test suite
     */
    protected String testName = null;

    /**
     * Constructor
     */
    public TestManager() {
        this.junitRunner.addListener(this.runningListener);
    }

    /**
     * 
     * @return singleton instance of TestManager
     */
    public static TestManager instance() {
        synchronized (TestManager.class) {
            if (TestManager.instance == null) {
                TestManager.instance = new TestManager();
            }
            
            return TestManager.instance;
        }
    }
    
    /**
     * Put an instance of TestManager to thread-specific storage.
     * The instance need to be put to the storage in order to be able
     * to run threaded tests.
     *  
     * @param testManager Instance to be put to the storage
     */
    public static void activateLocalInstance(final TestManager testManager) {
        instance_holder.set(testManager);
    }
    
    /**
     * Get instance of TestManager from the thread-specific storage.
     */
    public static TestManager getLocalInstance() {
        return instance_holder.get();
    }

    /**
     * Return the multiplier coefficient of time
     */
    public static int getTimescale() {

        int timescale = 1;
        
        String cdmwTimescale = System.getProperty(CDMW_TIMESCALE);
        if (cdmwTimescale != null) {
            try {
                timescale = Integer.parseInt(cdmwTimescale);
            }
            catch (NumberFormatException nfe) {
                nfe.printStackTrace();
            }
        }
    
        return timescale;
    }

    /**
     * initialize the test session
     */
    public void init(String testName) {
        this.testName = testName;
    }

    /**
     * Print a summary of the tests
     */
    public void printSummary() {
        System.out.println("===================================================");
        System.out.println("Ending test " + this.testName);
        System.out.println("===================================================");
        System.out.println("Summary:");
        System.out.println("   Failed    : "
            + this.runningListener.getFailureTestCount());
        System.out.println("   Succeeded : "
            + (this.runningListener.getTotalTestsCount() - this.runningListener.getFailureTestCount()));
        System.out.println("Expected successfull tests (JAVA(TM)) :" 
            + this.runningListener.getTotalTestsCount());

        if (this.runningListener.getFailureTestCount() == 0) {
            System.out.println("   >>>> Test result is set as TESTABLE_OK <<<<");
        }
        else {
            System.out.println("   >>>> Test result is NOT OK <<<<");
        }
    }
    
    public boolean isSuccessful() {
        return this.runningListener.getFailureTestCount() == 0;
    }

    /**
     * Add test class to the list of classes to be run
     * 
     * @param testClass
     *            class of test to be run
     */
    public void addTest(Class testClass) {
        this.testsToRun.add(testClass);
    }

    /**
     * Running test session
     */
    public void runTests() {
        Class[] classes = this.testsToRun.toArray(new Class[this.testsToRun.toArray().length]);
        this.junitRunner.run(classes);
    }

    /**
     * Put <code>property</code> to the storage specific to the TestManager instance
     * 
     * @param key Key to store this object under 
     * @param property Object to be put in storage
     */
    public void setLocalProperty(final String key, final Object property) {
        this.storage.put(key, property);
    }
    
    /**
     * Retrieve a value stored under the <code>key</code> from the Testmanager
     * instance-specific storagesummary
     * 
     * @param key Key to look at the storage
     * @return Object which is stored under the <code>key</code>
     */
    public Object getLocalProperty(final String key) {
        return this.storage.get(key);
    }
    
    /**
     * Set a value
     * 
     * @param name
     *            property name
     * @param value
     *            property value
     */
    public void setProperty(String name, String value) {
        System.setProperty(name, value);
    }

    /**
     * Get a value
     * 
     * @param name
     *            property name
     * 
     * @return value of property by name
     */
    public String getProperty(String name) {
        return System.getProperty(name);
    }

    /**
     * Returns the system properties needed for the tests
     * as a Vector, each element of which follows the format name=value
     * 
     * @return Vector containing all the needed system properties.
     */
    public static Vector<String> getDefaultProperties() {       
        Vector<String> properties = new Vector<String>();

        properties.add(JACORB + "=" + System.getProperty(JACORB));       
        properties.add(CDMW_ORB_VDR + "=" + System.getProperty(CDMW_ORB_VDR));   
        properties.add(CDMW_TIMESCALE + "=" + getTimescale());
        
        return properties;
    }
    
    /**
     * Return instance fo RunningListener in order to obtain results of tests which are being run in separated threads simultaneously
     * 
     */
    public TestRunningListener getRunningListener() {
        return this.runningListener;
    }
}
