/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI.
 * Copyright (C) SELEX SISTEMI INTEGRATI 2004-2009. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#include <cstdlib>
#include <vector>

#include "lbgroupcreator/GroupConfigurationMngr.hpp"

#include "Foundation/orbsupport/CORBA.hpp"
#include "Foundation/orbsupport/ExceptionMinorCodes.hpp"
#include "Foundation/orbsupport/OrbSupport.hpp"
#include "Foundation/common/Assert.hpp"
#include <Foundation/commonsvcs/naming/NamingInterface.hpp>


#include <sstream>
#include <xercesc/dom/deprecated/DOM.hpp>
#include <xercesc/dom/deprecated/DOMParser.hpp>
#include <xercesc/util/PlatformUtils.hpp>
#include <xercesc/sax/ErrorHandler.hpp>
#include <xercesc/sax/SAXParseException.hpp>


#include <lbgroupcreator/ConfigElementParser.hpp>
#include <lbgroupcreator/GroupsParser.hpp>
#include <lbgroupcreator/ObjectGroupParser.hpp>

namespace
{
    std::string to_string(const DOMString &s)
        throw (std::bad_alloc)
    {
        char *c_str = s.transcode();
    
        std::string str(c_str);
    
        delete[] c_str;
        
        return str;
    }
} // end anonymous namespace


namespace Cdmw
{
namespace LB
{
namespace GroupCreator
{
/**
*Implementation notes: none 
*<p>
*Portability issues: none 
*<p>
*/

class  ErrorReporter : public ErrorHandler
{
public:
    // -----------------------------------------------------------------------
    //  Constructors and Destructor
    // -----------------------------------------------------------------------
    ErrorReporter()
    {
    }

    ~ErrorReporter()
    {
    } 


    // Each error reported by the parsing is translated into an exception to be
    //  catch in the main procedure
    void warning(const SAXParseException& ex) { 
        throw ex; 
    }

    void error(const SAXParseException& ex)
    {
        throw ex;
    }

    void fatalError(const SAXParseException& ex)
    {
        throw ex;

    }

    void resetErrors() { 
    }
};


GroupConfigurationMngr::GroupConfigurationMngr(CdmwLB::LBGroupManager_ptr lb_group_manager)
    throw()
   : m_lb_group_manager(CdmwLB::LBGroupManager::_duplicate(lb_group_manager))
{
   CDMW_ASSERT(!CORBA::is_nil(m_lb_group_manager.in()));
}


GroupConfigurationMngr::GroupConfigurationMngr()
    throw()
    :m_lb_group_manager(CdmwLB::LBGroupManager::_nil())
{
}


void GroupConfigurationMngr::validate(const std::string& xml_conf)
    throw(OutOfMemoryException, ConfigurationException)
{
    // Xerces initialisation
    XMLPlatformUtils::Initialize();
    
    try 
    {
    try
    {
        // We parse the group configuration file
        std::auto_ptr<DOMParser> parser(new DOMParser);
        
        parser->setValidationScheme(DOMParser::Val_Auto);
        parser->setDoNamespaces(false);
        
        ErrorReporter errorReporter;
        parser->setErrorHandler( &errorReporter );
        
        parser->parse( xml_conf.c_str() );
        
            XMLPlatformUtils::Terminate();
        }
        catch (std::bad_alloc &) 
        {
        std::cerr << "bad_alloc exception raised" << std::endl;
            CDMW_THROW(OutOfMemoryException );
        } 
        catch(const SAXParseException& ex1)
        {
            CDMW_THROW2(ConfigurationException,
                        to_string(DOMString(ex1.getMessage())), ex1.getLineNumber());
        }
        catch(const XMLException& ex2)
        {
            CDMW_THROW2(ConfigurationException,
                        to_string(DOMString(ex2.getMessage())), ex2.getSrcLine());   
        }
        catch (const DOM_DOMException& e) 
        {
             CDMW_THROW2(ConfigurationException,
                        to_string(DOMString(e.msg)), e.code);   
       }
    }
    catch( CORBA::SystemException& e )
    {
        XMLPlatformUtils::Terminate();
        throw;
    }
    catch( CORBA::Exception& e )
    {
        XMLPlatformUtils::Terminate();
        CORBA::INTERNAL ex(Cdmw::OrbSupport::INTERNAL,
                           CORBA::COMPLETED_NO);
        throw ex;
    }
}




void GroupConfigurationMngr::parse(const std::string& xml_conf, bool validate)
    throw(OutOfMemoryException, ConfigurationException, CORBA::SystemException)
{
    CDMW_ASSERT(!CORBA::is_nil(m_lb_group_manager.in()));
    try
    {
        // Xerces initialisation
        XMLPlatformUtils::Initialize();
    }
    catch(const XMLException& toCatch)
    {
       CORBA::INTERNAL ex(Cdmw::OrbSupport::INTERNAL,
                           CORBA::COMPLETED_NO);
        throw ex;
    }
        
    try 
    {
        try
        {
             // We parse the group configuration file
             std::auto_ptr<DOMParser> parser(new DOMParser);
             if (validate)
             {
                 parser->setValidationScheme(DOMParser::Val_Auto);
                 parser->setDoNamespaces(false);
             }
    
             ErrorReporter errorReporter;
             parser->setErrorHandler( &errorReporter );
    
             parser->parse( xml_conf.c_str() );
    
             DOM_Document doc = parser->getDocument();
    
             // We build a TreeWalker to skip all comment and automatically resolve
             // the entities
             DOM_TreeWalker walker = doc.createTreeWalker(doc,
                              DOM_NodeFilter::SHOW_ELEMENT |
                              DOM_NodeFilter::SHOW_ATTRIBUTE |
                              DOM_NodeFilter::SHOW_TEXT |
                              DOM_NodeFilter::SHOW_DOCUMENT_TYPE,
                              NULL, true);
    
             // The first child is the DTD definition of the XML data, we skip it
             walker.firstChild();
       
            // We obtain the "Groups" node
            walker.nextSibling();
        
            GroupsParser group(walker.getCurrentNode());
            unsigned int nb_objectgroup = group.get_objectgroup_count();
             
            std::cerr <<  "Creation of " <<nb_objectgroup<<" group(s)" << std::endl;
             
            for (unsigned int incog = 0; incog < nb_objectgroup; incog++)
            {
                 
                 // for each objectgroup
                 ObjectGroupParser objectgroup(group.get_Objectgroup(incog));
                 std::string name = objectgroup.name();
                 std::string type_id = objectgroup.type_id();
         std::string lb_policy = objectgroup.lb_policy();
         
         // set lb_policy property
         ::PortableGroup::Criteria crit;        
         crit.length(2);
         
         ::PortableGroup::Name OGName;
                 OGName.length(1);
                 OGName[0].id=CORBA::string_dup(name.c_str());
                 
         crit[0].nam.length(1);
                 crit[0].nam[0].id="cdmw.lb.Name";
                 crit[0].val <<= OGName;
        
         ::PortableGroup::Properties prop;
    
         prop.length(1);
         prop[0].nam.length(1);
         prop[0].nam[0].id="cdmw.lb.LoadBalancingPolicy";
         prop[0].val <<=  lb_policy.c_str();
         
         ::PortableGroup::GenericFactory::FactoryCreationId_var factory_creation_id;
         crit[1].nam.length(1);
         crit[1].nam[0].id="cdmw.lb.LBProperties";
         crit[1].val <<= prop;
   
         std::cerr << "Create group :"<<"\n"
                           <<"\t name             : "<<name<<"\n"
                           <<"\t type_id          : "<<type_id<<"\n"
                           <<"\t with LB Policy   : "<< lb_policy << std::endl;
        
                 
                 ::PortableGroup::ObjectGroup_var obj_group;
         
                 // create the object group 
                 try
                 {
                     obj_group =  
                        m_lb_group_manager->create_object(type_id.c_str(), 
                              crit, 
                              factory_creation_id.out() );
                 }
                 catch( CORBA::SystemException& e )
                 {
                     std::cerr << "CORBA System Exception during create_object() operation: "<< e << std::endl;
                     throw;
                 }
                 catch( CORBA::Exception& e )
                 {
             std::cerr << "CORBA User Exception during create_object() operation: " << e << std::endl;
                     throw CORBA::INTERNAL(Cdmw::OrbSupport::INTERNAL,
                                                  CORBA::COMPLETED_NO);
                 }
            }

            XMLPlatformUtils::Terminate();
        }
        catch (std::bad_alloc &) 
        {
            std::cerr << "bad_alloc exception raised" << std::endl;

            CDMW_THROW(OutOfMemoryException);
        } 
        catch(const SAXParseException& ex1)
        {
            std::cerr << "XMLException raised: "
                      << to_string(ex1.getMessage()) << std::endl ;
            CDMW_THROW2(ConfigurationException,
                        to_string(DOMString(ex1.getMessage())), ex1.getLineNumber());
        }
        catch(const XMLException& ex2)
        {
            std::cerr << "XMLException raised: "
                      << to_string(ex2.getMessage()) << std::endl;
            CDMW_THROW2(ConfigurationException,
                        to_string(DOMString(ex2.getMessage())), ex2.getSrcLine());   
        }
        catch (const DOM_DOMException& e) 
        {
        std::cerr <<"DOM_DOMException raised: "
                      << to_string(e.msg) << std::endl;
             CDMW_THROW2(ConfigurationException,
                        to_string(DOMString(e.msg)), e.code);   
       }
    }
    catch( CORBA::SystemException& e )
    {
        XMLPlatformUtils::Terminate();
        std::cerr << "CORBA System Exception: " << e << std::endl;
        throw;
    }
    catch( CORBA::Exception& e )
    {
        XMLPlatformUtils::Terminate();
        std::cerr << "CORBA Exception: " << e << std::endl;
        CORBA::INTERNAL ex(Cdmw::OrbSupport::INTERNAL,
                           CORBA::COMPLETED_NO);
        throw ex;
    }
}


GroupConfigurationMngr::~GroupConfigurationMngr()
    throw()
{
};

}  // End namespace GroupCreator
} // End namespace LB
} // End namespace Cdmw


