/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI.
 * Copyright (C) SELEX SISTEMI INTEGRATI 2004-2009. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

/**
 * @brief A Fault Tolerant Group Manager process behaviour
 *
 * @author Raffaella Panella <rpanella@amjsv.it>
 */

// Cdmw Files
#include <Foundation/orbsupport/CORBA.hpp>
#include <Foundation/ossupport/OS.hpp>
#include <SystemMngt/platforminterface/PlatformInterface.hpp>
#include <Foundation/commonsvcs/naming/NamingUtil.hpp>
#include <Repository/repositoryinterface/RepositoryInterface.hpp>

#include "LoadBalancing/lbcommon/StateTransferConfig.hpp"
#include "LoadBalancing/lbgroupmanagerft/FTGroupManagerProcessBehaviour.hpp"
#include "ConfAndPlug/cdmwinit/InitUtils.hpp"
#include "FaultTolerance/ftlocationmanager/PrimaryBackupGroupRepository_impl.hpp"
#include "FaultTolerance/ftlocationmanager/PrimaryBackupAdmin_impl.hpp"
#include "FaultTolerance/ftlocationmanager/StatefullPrimaryBackupAdmin_impl.hpp"
#include <FaultTolerance/ftstatemanager/DataStore.hpp>
#include "FaultTolerance/ftstatemanager/DataStoreContext.hpp"
#include "FaultTolerance/ftstatemanager/DataStoreGroup_impl.hpp"
#include <LoadBalancing/lbgroupmanager/TypeRepository_impl.hpp>

#include "ace/Thread.h"
#include "ace/Thread_Manager.h"
#include "ace/TP_Reactor.h"
#include "ace/Select_Reactor.h"
#include "ace/Get_Opt.h"
#include "ace/Task.h"
#include "ace/Timer_Queue.h"
#include "ace/Timer_Heap.h"
#include "ace/Timer_Wheel.h"
#include "ace/Timer_Hash.h"
#include "ace/Timer_List.h"


#include "lbcommon/config.hpp"
#include <Foundation/logging/Logger.hpp>
#include <sstream>

#include <lbgroupmanager/ServiceLocator_impl.hpp>

namespace LOG=Cdmw::Logging;


#define ECHO_HEADER() \
    "[CDMW LB Group Manager FT] (file: " << __FILE__ << ", line: " << __LINE__ << ")\n -->"

#ifndef ENABLE_LB_DEBUG_DUMP

#   define DEBUG_DUMP(comment)
#   define DEBUG_ECHO(comment)

#else

#   define DEBUG_DUMP(comment) \
do {\
    std::cerr << ECHO_HEADER() << " --> " << comment << std::endl;\
} while(0)

#   define DEBUG_ECHO(comment) \
do {\
    std::cerr << comment:\
} while(0)

#endif


namespace Cdmw {
  namespace LB {
    typedef Cdmw::FT::DataStore<PortableGroup::ObjectGroupId,
                CdmwLB::ObjectGroupData>
    FTObjectGroupDataStore;
  }
}



Cdmw::LB::FTGroupManagerProcessBehaviour::FTGroupManagerProcessBehaviour
(CORBA::ORB_ptr orb,
 PortableGroup::GOA_ptr poa,
 bool primary,
 CdmwPlatformMngt::System_ptr system,
 ConfigurationMngr& conf_mngr,
 std::string xml_group_conf)
    throw()
  : m_conf_mngr(conf_mngr),
    m_xml_group_conf(xml_group_conf)
{
    m_orb = CORBA::ORB::_duplicate( orb );
    m_poa = PortableGroup::GOA::_duplicate(poa);
    m_set_has_primary = primary;
    m_system = CdmwPlatformMngt::System::_duplicate(system);
}

Cdmw::LB::FTGroupManagerProcessBehaviour::~FTGroupManagerProcessBehaviour()
    throw()
{
}

CORBA::ULong
Cdmw::LB::FTGroupManagerProcessBehaviour::nb_steps()
    throw(CORBA::SystemException)
{
    return 1;
}


void Cdmw::LB::FTGroupManagerProcessBehaviour::on_initialise
(const CdmwPlatformMngtBase::StartupKind& )
    throw(CORBA::SystemException)
{
     // Get process name
    m_process_name =
        Cdmw::PlatformMngt::PlatformInterface::Get_process_name();
    m_application_name = Cdmw::PlatformMngt::PlatformInterface::Get_application_name();
    CDMW_DEBUG(LOG::Logger::Get_Logger(LB_GROUP_MANAGER_LOGGER),"   --------  " << m_process_name << " on_initialise --------");

    PortableServer::POAManager_var poaMgr =
        m_poa->the_POAManager();

    // create a POA with USE_SERVANT_MANAGER & NON_RETAIN policy (to use ServantLocator)
    CORBA::PolicyList policies;
    policies.length(3);
    policies[0] = m_poa->create_request_processing_policy(PortableServer::USE_SERVANT_MANAGER);
    policies[1] = m_poa->create_servant_retention_policy(PortableServer::NON_RETAIN);
    policies[2] = m_poa->create_id_assignment_policy(PortableServer::USER_ID);

    Cdmw::OrbSupport::StrategyList strategyList;
    PortableServer::POA_var myPOA =
        Cdmw::OrbSupport::OrbSupport::create_POA(m_poa.in(),
                                                 "myPOA",
                                                 poaMgr.in(),
                                                 policies,
                                                 strategyList);

    // create Executor servant (using myPOA)
    CDMW_DEBUG(LOG::Logger::Get_Logger(LB_GROUP_MANAGER_LOGGER),"       ---- Server " << m_process_name << ": create Executor servant");

    CORBA::Object_var objref;

    objref =  m_orb->resolve_initial_references("ReplicationManager");


    CdmwFT::ReplicationManager_var replication_manager = CdmwFT::ReplicationManager::_narrow(objref.in());

    long localDataStoreInfosSize = 1;
    CdmwFT::StateTransfer::LocalDataStoreInfos_var localDataStoreInfos = new CdmwFT::StateTransfer::LocalDataStoreInfos();
    localDataStoreInfos->length(localDataStoreInfosSize);

    long number_of_scheduler_thread = 5;
    long chunk_size = 200;
    long freeze_size =150;
    OsSupport::OS::Timeval freeze_timeout;
    unsigned long res_long = 500;


    freeze_timeout.seconds = res_long/1000;
    freeze_timeout.microseconds = (res_long%1000)*1000;


    Cdmw::FT::DataStoreContext* datastore_context = new Cdmw::FT::DataStoreContext( m_orb.in(),
                                            m_poa.in(),
                                            m_conf_mngr.getMaxTransactionInProgress(),
                                            m_conf_mngr.getMaxTransactionDone(),
                                            m_conf_mngr.getCohortTimeOut(),
                                            m_conf_mngr.getCoordinatorTimeOut(),
                                            number_of_scheduler_thread,
                                            chunk_size,
                                            freeze_size,
                                            freeze_timeout);

    CdmwFT::StateTransfer::LocalDataStoreInfo localDataStoreInfo ;
    CdmwFT::StateTransfer::DataStoreGroup_var objectgroupDataStore_ref =
      (new Cdmw::FT::DataStoreGroup_impl(m_poa.in(),*datastore_context))->_this();

    long dataStore_inc=0;

    localDataStoreInfo.local_data_store = CdmwFT::StateTransfer::DataStoreGroup::_duplicate(objectgroupDataStore_ref.in());



    localDataStoreInfo.coordinator =
      datastore_context->get_tpc_coordinator();
    localDataStoreInfo.cohort =
      datastore_context->get_tpc_cohort();
    localDataStoreInfo.dsids.length(localDataStoreInfosSize);
    localDataStoreInfo.dsids[0] = m_conf_mngr.getObjectGroupDataIdentifier();

    (*localDataStoreInfos)[dataStore_inc] = localDataStoreInfo;

    new Cdmw::LB::FTObjectGroupDataStore(m_conf_mngr.getObjectGroupDataIdentifier(),
                     m_orb.in(),
                     m_poa.in(),
                     *datastore_context);

    // Location information shall be obtain from supervision
    ::FT::Location loc;
    loc.length(3);

    loc[0].id = CORBA::string_dup(Cdmw::OsSupport::OS::get_hostname().c_str());
    loc[0].kind = "hostname";
    loc[1].id = m_application_name.c_str();
    loc[1].kind = "applicationname";
    loc[2].id = m_process_name.c_str();
    loc[2].kind = "processname";

    std::string location_str
      = Cdmw::CommonSvcs::Naming::NamingInterface::to_string(loc);
    CDMW_DEBUG_5(LOG::Logger::Get_Logger(LB_GROUP_MANAGER_LOGGER),"setup LocationManager for location: " << location_str );


    // create a reference for the PrimaryBackupGroupRepository
    ::Cdmw::FT::Location::PrimaryBackupGroupRepository_impl * primaryBackupGroupRepository_impl
    = ::Cdmw::FT::Location::PrimaryBackupGroupRepository_impl::Get_instance();

    ::Cdmw::FT::Location::StatefullPrimaryBackupAdmin_impl * statefull_primary_backup_admin
    = new ::Cdmw::FT::Location::StatefullPrimaryBackupAdmin_impl(m_orb.in(),
                                     m_poa.in(),
                                     loc,
                                     primaryBackupGroupRepository_impl,
                                     true);

    std::string observer_name = m_conf_mngr.getObserverName() + std::string("_") + Cdmw::OsSupport::OS::get_hostname();
    ::CdmwFT::Location::StatefullPrimaryBackupAdmin_var statefull_primary_backup_admin_ref
    = statefull_primary_backup_admin->_this();


    replication_manager->register_statefull_location(loc,
                             statefull_primary_backup_admin_ref.in(),
                                                     localDataStoreInfos.in(), false);


    // Create the TypeRepository object and register it
    CdmwLB::TypeRepository_var typeRepository =
      new Cdmw::LB::TypeRepository_impl(m_conf_mngr.getSupportedInterfaceMap());
    m_orb->register_initial_reference("LBTypeRepository", typeRepository.in());


    m_group_manager_impl =
      new Cdmw::LB::FTGroupManager_impl(m_orb.in(),
                    m_poa.in(),
                    m_conf_mngr.getDomainID(),
                    m_system.in(),
                    observer_name,
                    m_xml_group_conf,
            m_conf_mngr.getObjectGroupDataIdentifier());



 
    CdmwLB::LBGroupManager_var the_lb_group_manager = m_group_manager_impl->_this();
    // create activation handler
    CDMW_DEBUG_5(LOG::Logger::Get_Logger(LB_GROUP_MANAGER_LOGGER),"       ---- Server " << m_process_name << ": create activation handler");
    m_activation_handler =
      new Cdmw::LB::LBGroupManagerActivationHandler(m_process_name,
                            m_group_manager_impl);

    // create ServantLocator
    CDMW_DEBUG_5(LOG::Logger::Get_Logger(LB_GROUP_MANAGER_LOGGER),"       ---- Server " << m_process_name << ": create ServantLocator ");
    std::string executor_oid = m_process_name;
    m_servant_locator = new Cdmw::LB::LBGroupManagerServantLocator(executor_oid,
                                   m_activation_handler);

    // set ServantLocator to myPOA
    try {
        myPOA->set_servant_manager(m_servant_locator);
    } catch (const PortableServer::POA::WrongPolicy& e) {
        // shouldn't happen
        CDMW_ASSERT(false);
    }
        
    // create Executor object under myPOA
    CDMW_DEBUG_5(LOG::Logger::Get_Logger(LB_GROUP_MANAGER_LOGGER),"       ---- LBGroupManager (FT) " << m_process_name << ": create LBGroupManager reference ");
    PortableServer::ObjectId_var oid = PortableServer::string_to_ObjectId(executor_oid.c_str());
    CORBA::Object_var group_manager_obj = 
        myPOA->create_reference_with_id(oid, "IDL:CdmwLB/LBGroupManager:1.0");
    m_group_manager_obj = CdmwLB::LBGroupManager::_narrow(group_manager_obj.in());
    CDMW_DEBUG_5(LOG::Logger::Get_Logger(LB_GROUP_MANAGER_LOGGER),"       ---- LBGroupManager (FT) " << m_process_name << ": register_initial_reference  =>(" <<  GROUP_MANAGER_NAME <<")" );
    CDMW_DEBUG_5(LOG::Logger::Get_Logger(LB_GROUP_MANAGER_LOGGER),"       ---- LBGroupManager (FT) " << m_process_name << ": register_initial_reference  IOR =>(" 
                                                                                                   << m_orb->object_to_string(m_group_manager_obj)<<")" );
    m_orb->register_initial_reference(GROUP_MANAGER_NAME.c_str(),
                      m_group_manager_obj.in());
   

    // add Executor object as member of Executor group
    this->add_group_manager_to_group();
    
    CDMW_DEBUG(LOG::Logger::Get_Logger(LB_GROUP_MANAGER_LOGGER),"       ---- LBGroupManager (FT): added ref to group ");

    CdmwLB::LBGroupManager_var lbGroupManagerGroup =
        getLBGroupManagerGroupFromCdmwReplicationMngr();
    
    //bind the ft lbgroupmanager to the corbaloc
    OrbSupport::OrbSupport::bind_object_to_corbaloc(m_orb, GROUP_MANAGER_NAME.c_str(), m_group_manager_obj.in());

    CDMW_DEBUG_5(LOG::Logger::Get_Logger(LB_GROUP_MANAGER_LOGGER),"       ---- LBGroupManager (FT)>> register service ");

    std::string multicast_addr = m_conf_mngr.getLocalisationServiceAddress();
    if (multicast_addr != "")
    {
        CDMW_DEBUG_5(LOG::Logger::Get_Logger(LB_GROUP_MANAGER_LOGGER),"       ---- LBGroupManager (FT)>> register service ");
        CDMW_DEBUG_5(LOG::Logger::Get_Logger(LB_GROUP_MANAGER_LOGGER),"       ---- LBGroupManager (FT) >> multicast_addr  "  << multicast_addr );

        CORBA::Object_var group = 
           m_orb->string_to_object(multicast_addr.c_str());
        PortableServer::ObjectId_var  
           service_locator_id = m_poa->create_id_for_reference(group.in());

        Cdmw::LB::ServiceLocator_impl* service_locator_obj
          = new  Cdmw::LB::ServiceLocator_impl(m_orb,
                                            m_conf_mngr.getDomainID(),
                                            //the_lb_group_manager.in ());
                                            lbGroupManagerGroup.in());

        
        m_poa->activate_object_with_id (service_locator_id.in (),service_locator_obj);
        
        CDMW_DEBUG_5(LOG::Logger::Get_Logger(LB_GROUP_MANAGER_LOGGER),"LBGroupManager>> ServiceLocator_impl  defined");
    } else
        CDMW_INFO(LOG::Logger::Get_Logger(LB_GROUP_MANAGER_LOGGER),"LBGroupManager>> LocalisationService not defined");

   CosNaming::NamingContextExt_var root_nc 
       = CosNaming::NamingContextExt::_narrow(the_lb_group_manager.in());
    
    if (!CORBA::is_nil(root_nc.in()))
       OrbSupport::OrbSupport::bind_object_to_corbaloc(m_orb, "NameService", root_nc.in());
    else
        CDMW_INFO(LOG::Logger::Get_Logger(LB_GROUP_MANAGER_LOGGER),"LBGroupManager (FT) >> the_lb_group_manager context not defined  ");

        CDMW_DEBUG_5(LOG::Logger::Get_Logger(LB_GROUP_MANAGER_LOGGER),"LBGroupManager (FT) >> bind the NamingContextExt object to corbaloc");
     

    try
    {
        CORBA::Any property_any_val;
        property_any_val <<=  m_conf_mngr.getDomainID();
      
        CDMW_DEBUG(LOG::Logger::Get_Logger(LB_GROUP_MANAGER_LOGGER),"LBGroupManager (FT) >> set attribute Cdmw.LBGroupmanager.Configuration.LoadBalancingDomainID = " << m_conf_mngr.getDomainID() );
        m_system->define_property_with_mode ("Cdmw.LBGroupmanager.Configuration.LoadBalancingDomainID",
                         property_any_val,
                         CosPropertyService::normal
                         );

       m_system->set_attribute("Cdmw.LBGroupmanager.Configuration.LoadBalancingDomainID", property_any_val , CdmwPlatformMngt::USER_DEFINED);
    }
    catch( const CdmwPlatformMngt::ManagedElementNotFound& e )
    {
        CDMW_INFO(LOG::Logger::Get_Logger(LB_GROUP_MANAGER_LOGGER),"LBGroupManager (FT) >>Cdmw.LBGroupmanager.Configuration.LoadBalancingDomainID  ManagedElementNotFound ");
    }
    catch (const CdmwPlatformMngt::ReadOnlyAttribute& roa)
    {
        CDMW_INFO(LOG::Logger::Get_Logger(LB_GROUP_MANAGER_LOGGER),"LBGroupManager (FT) >> Cdmw.LBGroupmanager.Configuration.LoadBalancingDomainID ReadOnlyAttribute  ");
    }
    catch(const CosPropertyService::InvalidPropertyName &ipn )
    {
        CDMW_INFO(LOG::Logger::Get_Logger(LB_GROUP_MANAGER_LOGGER),"LBGroupManager (FT) >> Cdmw.LBGroupmanager.Configuration.LoadBalancingDomainID InvalidPropertyName  ");
    }
    catch(const CosPropertyService::ConflictingProperty& cp)
    {
        CDMW_INFO(LOG::Logger::Get_Logger(LB_GROUP_MANAGER_LOGGER),"LBGroupManager (FT) >> Cdmw.LBGroupmanager.Configuration.LoadBalancingDomainID ConflictingProperty  ");
    }
    catch(const CosPropertyService::UnsupportedTypeCode utc)
    {
        CDMW_INFO(LOG::Logger::Get_Logger(LB_GROUP_MANAGER_LOGGER),"LBGroupManager (FT) >> Cdmw.LBGroupmanager.Configuration.LoadBalancingDomainID UnsupportedTypeCode ");
    }
    catch (...)
    {
        CDMW_INFO(LOG::Logger::Get_Logger(LB_GROUP_MANAGER_LOGGER),"LBGroupManager (FT) >> Cdmw.LBGroupmanager.Configuration.LoadBalancingDomainID Exception during definition ");
    }




   // activate POAManager
    CDMW_DEBUG_5(LOG::Logger::Get_Logger(LB_GROUP_MANAGER_LOGGER),"       ---- LBGroupManager (FT) >> " << m_process_name << ": activate POAManager " );
    poaMgr->activate();
    CDMW_DEBUG_5(LOG::Logger::Get_Logger(LB_GROUP_MANAGER_LOGGER),"       ---- LBGroupManager (FT) >> " << m_process_name << " on_initialise END -------- " );
      
        
}
/**
* registers a FT IOGR of LB group Manager FT  with the ServiceLocator
*
**/
#if 0
void Cdmw::LB::FTGroupManagerProcessBehaviour::register_FT_IOGRofLBGroupManager_with_ServiceLocator()
{
     
}
#endif
    
    
void Cdmw::LB::FTGroupManagerProcessBehaviour::on_next_step()
    throw( CdmwPlatformMngt::ProcessDelegate::InvalidStep,
           CORBA::SystemException )
{

}


void Cdmw::LB::FTGroupManagerProcessBehaviour::on_run()
    throw( CdmwPlatformMngt::ProcessDelegate::NotReadyToRun,
           CORBA::SystemException )
{
    CDMW_DEBUG(LOG::Logger::Get_Logger(LB_GROUP_MANAGER_LOGGER),"       ---- LBGroupManager (FT) >> " << m_process_name << " running -------- " );
}

void Cdmw::LB::FTGroupManagerProcessBehaviour::on_stop()
    throw( CORBA::SystemException )
{    
    CDMW_DEBUG(LOG::Logger::Get_Logger(LB_GROUP_MANAGER_LOGGER),"       ---- LBGroupManager (FT) >> " << m_process_name << " stopping -------- " );
    
    m_activation_handler->deactivate();
}

    

void Cdmw::LB::FTGroupManagerProcessBehaviour::add_group_manager_to_group()
        throw(::FT::ObjectGroupNotFound,
          ::FT::MemberNotFound,
          ::FT::PrimaryNotSet,
          ::FT::BadReplicationStyle,
          ::FT::MemberAlreadyPresent,
          ::FT::ObjectNotAdded,
          ::CORBA::ORB::InvalidName,
          CORBA::SystemException)
{
    // Get the group repository
   std::ostringstream oss;
    CORBA::Object_var obj = 
        m_orb->resolve_initial_references( "FTGroupRepository" );
    m_group_repository = 
        CdmwFT::Location::PrimaryBackupGroupRepository::_narrow(obj.in());
        
    // Get this process' location
    ::FT::Location_var the_location = m_group_repository->the_location();
    m_location = 
        Cdmw::CommonSvcs::Naming::NamingInterface::to_string(the_location.in() );
    oss << "\n       ---- Server " << m_process_name << ": location is " << m_location << std::endl;
    
  
    // Get the Replication Manager
    CDMW_DEBUG(LOG::Logger::Get_Logger(LB_GROUP_MANAGER_LOGGER),"       ---- Server " << m_process_name << ": Get the Replication Manager ");
    CORBA::Object_var replication_mgr_obj
        = m_orb->resolve_initial_references("ReplicationManager");

    m_replication_mgr =
        CdmwFT::ReplicationManager::_narrow(replication_mgr_obj.in());
    
    // get Executor group reference (from ReplicationManager)
    CdmwLB::LBGroupManager_var lbGroupManagerGroup =
    //        getLBGroupManagerGroupFromCdmwRepository();
    
    //if (CORBA::is_nil(lbGroupManagerGroup.in())) {
    //    lbGroupManagerGroup =
            getLBGroupManagerGroupFromCdmwReplicationMngr();
    //}
    if (CORBA::is_nil(lbGroupManagerGroup.in())) {
            CDMW_INFO(LOG::Logger::Get_Logger(LB_GROUP_MANAGER_LOGGER),"Warning: the lbGroupManagerGroup not found in CDMW Replication manager");
            return;
    }

    
    // create Location for this member
    CDMW_DEBUG(LOG::Logger::Get_Logger(LB_GROUP_MANAGER_LOGGER),"       ---- Server " << m_process_name << ": Create Location for this member ");
    ::FT::Location loc;
    loc.length( 3 );
    loc[0].id = CORBA::string_dup(Cdmw::OsSupport::OS::get_hostname().c_str());
    loc[0].kind = "hostname";
    loc[1].id = m_application_name.c_str();
    loc[1].kind = "applicationname";
    loc[2].id = m_process_name.c_str();
    loc[2].kind = "processname";        

    // add LBGroupManager object as a new member of the LBGroupManager object group
    ::FT::ObjectGroup_var obj_group;
    try 
    {
        ::FT::Name group_manager_og_name;
        group_manager_og_name.length(1);
        group_manager_og_name[0].id=GROUP_MANAGER_OBJECT_GROUP_NAME.c_str();
        CORBA::Object_var obj = 
             m_replication_mgr->get_object_group_ref_from_name(group_manager_og_name);
        lbGroupManagerGroup = CdmwLB::LBGroupManager::_narrow(obj.in());
        obj_group = m_replication_mgr->add_member(lbGroupManagerGroup.in(), loc, m_group_manager_obj.in());
        CDMW_DEBUG(LOG::Logger::Get_Logger(LB_GROUP_MANAGER_LOGGER),"  ---- LBGroupManagetFT " << m_process_name << "            added " << m_process_name << " to object group");
    }
    catch (::FT::ObjectGroupNotFound e)
    {
       CDMW_INFO(LOG::Logger::Get_Logger(LB_GROUP_MANAGER_LOGGER),"ERROR! lbGroupManagerGroup not found in CDMW Replication manager");
    }    
    
    // Register activation handler
    CDMW_DEBUG(LOG::Logger::Get_Logger(LB_GROUP_MANAGER_LOGGER),"       ---- Server " << m_process_name << ": Register activation handler");
    // get ActivationManager
    obj = m_orb->resolve_initial_references( "FTActivationManager" );
    CdmwFT::Location::ActivationManager_var activation_manager = 
        CdmwFT::Location::ActivationManager::_narrow( obj.in() );
    
    // Register activation handler into ActivationManager
    activation_manager->register_activation_handler(
    lbGroupManagerGroup.in(), m_activation_handler );
    
    // Set LBGroupManager as primary if required
    if (m_set_has_primary) {
        CDMW_DEBUG(LOG::Logger::Get_Logger(LB_GROUP_MANAGER_LOGGER),"       ---- Server " << m_process_name << ": set " << m_process_name << " as primary");
        obj_group = m_replication_mgr->set_primary_member( obj_group.in(), loc );
    } 
        
    CdmwNamingAndRepository::Repository_var repository
        = Cdmw::NamingAndRepository::RepositoryInterface::get_repository();

    try 
    { 
       // get NameDomain to object_groups (for writing)
       CdmwNamingAndRepository::NameDomain_var objGroupsDomain = repository->resolve_name_domain (REPOSITORY_GROUP_MANAGER_GROUP_NAME.c_str());

       // register Executor ObjectGroup into repository
       CDMW_DEBUG(LOG::Logger::Get_Logger(LB_GROUP_MANAGER_LOGGER),"       ---- register object group into repository");
       try{
           objGroupsDomain->release_name(GROUP_MANAGER_OBJECT_GROUP_NAME.c_str());
       }catch(const CdmwNamingAndRepository::InvalidName& e)
       {
           std::ostringstream oss;
           oss << "Resolve " << GROUP_MANAGER_OBJECT_GROUP_NAME << " => NamingContext::InvalidName raised!" << std::endl;
           CDMW_INFO(LOG::Logger::Get_Logger(LB_GROUP_MANAGER_LOGGER),oss.str());
       }
       catch(const CdmwNamingAndRepository::NameDomain::NotRegistered& e)
       {
           std::ostringstream oss;
           oss << "Resolve " << GROUP_MANAGER_OBJECT_GROUP_NAME << " => NameDomain::NotRegistered raised!" << std::endl;
           CDMW_INFO(LOG::Logger::Get_Logger(LB_GROUP_MANAGER_LOGGER),oss.str());
       }
       CdmwNamingAndRepository::NameDomain::RegistrationId_var regId =
           objGroupsDomain->new_name (GROUP_MANAGER_OBJECT_GROUP_NAME.c_str());

       objGroupsDomain->register_object (regId.in(), obj_group.in ());
       //objGroupsDomain->register_object (regId.in(), lbGroupManagerGroup.in ());
       CDMW_INFO(LOG::Logger::Get_Logger(LB_GROUP_MANAGER_LOGGER),"       ---- register object group "<< GROUP_MANAGER_OBJECT_GROUP_NAME << " into repository");
    } 
    catch(const CdmwNamingAndRepository::NameDomain::NotRegistered& e)
    {
           std::ostringstream oss;
           oss << "resolve_name_domain  " << REPOSITORY_GROUP_MANAGER_GROUP_NAME << " => NameDomain::NotRegistered raised!" << std::endl;
           CDMW_INFO(LOG::Logger::Get_Logger(LB_GROUP_MANAGER_LOGGER),oss.str());
    }
    



    
}


/**
 * Get LBGroupManager group reference from Cardamom Repository
 * This reference was created by the group_creator process
 */
CdmwLB::LBGroupManager*
Cdmw::LB::FTGroupManagerProcessBehaviour::getLBGroupManagerGroup() 
{

     CdmwLB::LBGroupManager_var lbGroupManagerGroup = CdmwLB::LBGroupManager::_nil();
     lbGroupManagerGroup = getLBGroupManagerGroupFromCdmwRepository();
     if (CORBA::is_nil(lbGroupManagerGroup.in()))
     {
         lbGroupManagerGroup = getLBGroupManagerGroupFromCdmwReplicationMngr();
     }
     return lbGroupManagerGroup._retn();       
}    

/**
 * Get LBGroupManager group reference from Cardamom Repository
 * This reference was created by the group_creator process
 */
CdmwLB::LBGroupManager*
Cdmw::LB::FTGroupManagerProcessBehaviour::getLBGroupManagerGroupFromCdmwRepository() 
{

    // Get the Cdmw repository
    CdmwNamingAndRepository::Repository_var repository
        = Cdmw::NamingAndRepository::RepositoryInterface::get_repository();
    
    // Get NamingInterface to object_groups (for reading)
    Cdmw::CommonSvcs::Naming::NamingInterface objGroupsInterface =
        Cdmw::NamingAndRepository::RepositoryInterface::get_domain_naming_interface(REPOSITORY_GROUP_MANAGER_GROUP_NAME.c_str());

    // Get LB GroupManager Object Group
    CDMW_DEBUG(LOG::Logger::Get_Logger(LB_GROUP_MANAGER_LOGGER),"       ---- Server " << m_process_name << ": Get LB GroupManager Object Group ");
    CdmwLB::LBGroupManager_var lbGroupManagerGroup = CdmwLB::LBGroupManager::_nil();
    try {
        CORBA::Object_var obj = objGroupsInterface.resolve(GROUP_MANAGER_OBJECT_GROUP_NAME);   
        lbGroupManagerGroup = CdmwLB::LBGroupManager::_narrow(obj.in());
    }
    catch(const CosNaming::NamingContext::NotFound& e)
    {
        std::ostringstream oss;
        oss << "Resolve " << GROUP_MANAGER_OBJECT_GROUP_NAME << " => NameDomain::NotRegistered raised!" << std::endl;
        CDMW_INFO(LOG::Logger::Get_Logger(LB_GROUP_MANAGER_LOGGER),oss.str());

    }
    catch(const CosNaming::NamingContext::CannotProceed& e)
    {
        std::ostringstream oss;
    oss << "Resolve " << GROUP_MANAGER_OBJECT_GROUP_NAME << " => NamingContext::CannotProceed raised!" << std::endl ;
        CDMW_INFO(LOG::Logger::Get_Logger(LB_GROUP_MANAGER_LOGGER),oss.str());

    }
    catch(const CosNaming::NamingContext::InvalidName& e)
    {
        std::ostringstream oss;
    oss << "Resolve " << GROUP_MANAGER_OBJECT_GROUP_NAME << " => NamingContext::InvalidName raised!"<< std::endl;
        CDMW_INFO(LOG::Logger::Get_Logger(LB_GROUP_MANAGER_LOGGER),oss.str());

    }
    catch (const CORBA::SystemException& e)
    {
        std::ostringstream oss;
        oss << "Resolve " << GROUP_MANAGER_OBJECT_GROUP_NAME << " => CORBA::SystemException raised!"<< std::endl;
        CDMW_INFO(LOG::Logger::Get_Logger(LB_GROUP_MANAGER_LOGGER),oss.str());

    }

    return lbGroupManagerGroup._retn();     
    
}    

/**
 * Get FT LBGroupManager IOGR reference from Cardamom Replication Manager
 * 
 */
CdmwLB::LBGroupManager*
Cdmw::LB::FTGroupManagerProcessBehaviour::getLBGroupManagerGroupFromCdmwReplicationMngr() 
{
        CdmwLB::LBGroupManager_var lbGroupManagerGroup = CdmwLB::LBGroupManager::_nil();
        ::FT::Name group_manager_og_name;
        group_manager_og_name.length(1);
        group_manager_og_name[0].id=GROUP_MANAGER_OBJECT_GROUP_NAME.c_str();
        CDMW_DEBUG(LOG::Logger::Get_Logger(LB_GROUP_MANAGER_LOGGER),"       ---- LBGroupManagerFT " << m_process_name << ": Get LB GroupManager empty Object Group from replication manager");

        CORBA::Object_var obj = m_replication_mgr->get_object_group_ref_from_name(group_manager_og_name);
        lbGroupManagerGroup = CdmwLB::LBGroupManager::_narrow(obj.in());
        return lbGroupManagerGroup._retn();    
}

