/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI.
 * Copyright (C) SELEX SISTEMI INTEGRATI 2004-2009. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

/**
 * @brief A Fault Tolerant Group Manager implementation
 *
 * @author Raffaella Panella <rpanella@amjsv.it>
 */

#include "LoadBalancing/lbgroupmanagerft/FTGroupManager_impl.hpp"
#include <lbgroupcreator/GroupConfigurationMngr.hpp>

#include <iostream>

#include <Foundation/logging/Logger.hpp>
#include <sstream>
#include <Repository/repositoryinterface/RepositoryInterface.hpp>

#include <lbgroupmanager/ServiceLocator_impl.hpp>

#include "LoadBalancing/lbcommon/StateTransferConfig.hpp"
// include for Logging
#include <sstream>
#include "lbcommon/config.hpp"


namespace LOG=Cdmw::Logging;


#define ECHO_HEADER() \
    "[CDMW FT Group Manager] (file: " << __FILE__ << ", line: " << __LINE__ << ")\n -->"


#ifndef ENABLE_LB_DEBUG_DUMP

#   define DEBUG_DUMP(comment)
#   define DEBUG_ECHO(comment)

#else

#   define DEBUG_DUMP(comment) \
do {\
    std::cerr << ECHO_HEADER() << " --> " << comment << std::endl;\
} while(0)

#   define DEBUG_ECHO(comment) \
do {\
    std::cerr << comment:\
} while(0)

#endif

std::string FT_GROUP_MANAGER_NAME            ("LBGroupManager"    );

namespace Cdmw
{

namespace LB
{

    /**
     * Creates a group manager.
     *
     * @param lb_domain_id The load balancing domain id.
     */
//    std::string REPOSITORY_GROUP_MANAGER_GROUP_NAME   = "lb_group_manager/object_groups";
//    std::string GROUP_MANAGER_OBJECT_GROUP_NAME       = "LBGroupManagerGroup";

    FTGroupManager_impl::FTGroupManager_impl(CORBA::ORB_ptr orb,
                        PortableGroup::GOA_ptr goa,
                        const char* lb_domain_id,
                        CdmwPlatformMngt::System_ptr system,
                        std::string observer_name,
                                            std::string xml_group_conf, int ObjectGroupDataIdentifier)
        throw(CORBA::SystemException)
    :
      Cdmw::LB::FTGroupManager_impl::LBGroupManager_impl(orb, goa, lb_domain_id, xml_group_conf,ObjectGroupDataIdentifier,system ),
      m_orb(CORBA::ORB::_duplicate(orb)),
      m_xml_group_conf(xml_group_conf)
    {
        m_system = CdmwPlatformMngt::System::_duplicate(system);
        m_observer_name = observer_name;
    }

    FTGroupManager_impl::~FTGroupManager_impl()
    {
    }

void
FTGroupManager_impl::activate()
{
    std::ostringstream oss;
    oss << "\n############################################################"  
        << "\n#           Activating  FTGroupManager Replica             #" << std::endl;

    CORBA::Object_var obj = m_orb->resolve_initial_references(FT_GROUP_MANAGER_NAME.c_str());
    CdmwLB::LBGroupManager_var group_manager_ref = CdmwLB::LBGroupManager::_narrow(obj.in());
    m_system->register_observer( m_observer_name.c_str(), group_manager_ref.in() );

    // Obtain the IOGR of LBGroupManager FT 
    CORBA::Object_var replication_mgr_obj
            = m_orb->resolve_initial_references("ReplicationManager");

    CdmwFT::ReplicationManager_var replication_mgr =
        CdmwFT::ReplicationManager::_narrow(replication_mgr_obj.in());

    CdmwLB::LBGroupManager_var lbGroupManagerGroup = CdmwLB::LBGroupManager::_nil();
    ::FT::Name group_manager_og_name;
    group_manager_og_name.length(1);
    group_manager_og_name[0].id=GROUP_MANAGER_OBJECT_GROUP_NAME.c_str();

    CORBA::Object_var lbgobj = replication_mgr->get_object_group_ref_from_name(group_manager_og_name);
    CDMW_DEBUG_5(LOG::Logger::Get_Logger(FT_LB_GROUP_MANAGER_LOGGER),"Get the IOGR of LBGroupMngrs from Replication Manager -->" << m_orb->object_to_string(lbgobj));
    lbGroupManagerGroup = CdmwLB::LBGroupManager::_narrow(lbgobj.in());
    try 
    {
        updateRepositoryRef(lbGroupManagerGroup.in());    
    } catch (CORBA::SystemException &se)
    {
        CDMW_DEBUG(LOG::Logger::Get_Logger(FT_LB_GROUP_MANAGER_LOGGER),"SystemException : Trying to update the reference of LBGroupManager into Repository "); 
    }catch(...)
    {
        CDMW_DEBUG(LOG::Logger::Get_Logger(FT_LB_GROUP_MANAGER_LOGGER),"Unexpected Exception : trying to Trying to update the reference of LBGroupManager into Repository ");
    }
    
    
    oss << "\n#           Updated CDMW Repository                        #" 
        << "\n#           Complete FTGroupManager_impl Initialization    #"  
        << "\n############################################################" << std::endl;
    CDMW_DEBUG(LOG::Logger::Get_Logger(FT_LB_GROUP_MANAGER_LOGGER),oss.str()); 
         

}
void FTGroupManager_impl::updateRepositoryRef(CdmwLB::LBGroupManager_ptr group_manager_ref)
{
    CdmwNamingAndRepository::Repository_var repository
        = Cdmw::NamingAndRepository::RepositoryInterface::get_repository();
    try 
    { 
       // get NameDomain to object_groups (for writing)
        CdmwNamingAndRepository::NameDomain_var objGroupsDomain = 
           repository->resolve_name_domain (REPOSITORY_GROUP_MANAGER_GROUP_NAME.c_str());
        std::ostringstream oss;   
        try
        {
            
            objGroupsDomain->release_name(GROUP_MANAGER_OBJECT_GROUP_NAME.c_str());
        }
        catch(const CdmwNamingAndRepository::InvalidName& e)
        {
            oss << "\n>>>>>>>>>>>>>>>>>>>>>> " << GROUP_MANAGER_OBJECT_GROUP_NAME 
                     <<" is an invalid name for Repository" << std::endl;
            
        }
        catch(const CdmwNamingAndRepository::NameDomain::NotRegistered& e)
        {
           
            oss << "\n>>>>>>>>>>>>>>>>>>>>>> Name " << GROUP_MANAGER_OBJECT_GROUP_NAME 
                     << " not yet registered in the Repository : it will created new one " << std::endl;
            CDMW_INFO(LOG::Logger::Get_Logger(FT_LB_GROUP_MANAGER_LOGGER),oss.str());      
        }
        CdmwNamingAndRepository::NameDomain::RegistrationId_var regId =
           objGroupsDomain->new_name (GROUP_MANAGER_OBJECT_GROUP_NAME.c_str());

        objGroupsDomain->register_object (regId.in(),group_manager_ref);
        
    } 
    catch(const CdmwNamingAndRepository::NameDomain::NotRegistered& e)
    {
        std::ostringstream oss; 
        oss << "\n>>>>>>>>>>>>>>>>>>>>>> NameDomain" << REPOSITORY_GROUP_MANAGER_GROUP_NAME 
                     << " not yet registered " << std::endl;
    
        CDMW_ERROR(LOG::Logger::Get_Logger(FT_LB_GROUP_MANAGER_LOGGER),oss.str());     
    }  

}

}//close namespace LB

}// close nampespace Cdmw

