/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI.
 * Copyright (C) SELEX SISTEMI INTEGRATI 2004-2009. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

/**
 * @brief Implementation for Client Request Interceptor.
 *
 * @author Raffaella Panella <rpanella@amjsv.it>
 * @author Antonello Cedrone <a.cedrone@vitrociset.it>
 */

#include "lbinit/LBClientRequestInterceptor.hpp"
#include <Foundation/orbsupport/OrbSupport.hpp>
#include <Foundation/orbsupport/ExceptionMinorCodes.hpp>
#include <Foundation/orbsupport/StrategyList.hpp>
#include <iostream>
#include <LoadBalancing/lbstrategy/StrategyFactory_impl.hpp>
#include <Foundation/commonsvcs/time/Util.hpp>

#include <LoadBalancing/lbcommon/Tao_LB_IOGRFactory.hpp>
#include <LoadBalancing/lbcommon/TaoAny.hpp>
#include <LoadBalancing/lbcommon/TaoFT_CORBA.hpp>

#include "lbcommon/config.hpp"
#include <Foundation/logging/Logger.hpp>
#include <sstream>
namespace LOG=Cdmw::Logging;


TimeBase::TimeT Cdmw::LB::LBInit::ClientRequestInterceptor_impl::REQUEST_TIMEOUT = 90L * 10000000L;
namespace{

  const std::string FORBIDDEN_OPERATION = "add_log";

}

namespace Cdmw
{

namespace LB
{

namespace LBInit
{


ClientRequestInterceptor_impl::ClientRequestInterceptor_impl (CORBA::ORB_ptr orb,
                                                              const char *name,
                                                              Cdmw::LB::IOGRFactory* iogr_factory)
    : m_orb (CORBA::ORB::_duplicate(orb)),
      m_name(name),
      m_iogr_factory(iogr_factory)

{
    m_isIn = 0;
    m_lb_factory = new Cdmw::LB::StrategyFactory_impl(m_iogr_factory);
    set_LBRequestTimeout(0);
    isLBRequestPolicyTimeoutSetted = false;
    num_of_network_interfaces_on = 1;
}

void
ClientRequestInterceptor_impl::destroy ()
    throw (CORBA::SystemException)
{
    delete m_iogr_factory;
    LBCache::iterator lb_cache_pos;
    for(lb_cache_pos=m_lb_cache.begin(); lb_cache_pos!=m_lb_cache.end();++lb_cache_pos)
    {
    delete (lb_cache_pos->second);
    }
}

char *
ClientRequestInterceptor_impl::name ()
    throw (CORBA::SystemException)
{
    return CORBA::string_dup (this->m_name.in ());
}

void
ClientRequestInterceptor_impl::send_request (PortableInterceptor::ClientRequestInfo_ptr ri)
    throw(CORBA::SystemException,
          PortableInterceptor::ForwardRequest)
{

    if(ri->operation() == FORBIDDEN_OPERATION) {
      return;
    }
    


     
    // The log below must not be recorded if the intercepted operation is add_log(). Else if Log Collector
    // is used we enter in a cyclic recording of the log (infinite call to add_log)!
    // This explains why the macro is called only after the condition if()
    CDMW_DEBUG_5(LOG::Logger::Get_Logger(LB_GROUP_MANAGER_LOGGER), "    send_request()   " );
    CORBA::Boolean is_LB_request = true;
    PortableGroup::ObjectGroupRefVersion lb_iogr_version=0;
    PortableGroup::ObjectGroup_var object_group = CORBA::Object::_duplicate(ri->target());
    LBCache::iterator lb_cache_pos;
    CdmwLB::TagLBGroupTaggedComponent* lb_group_tag = 0;
    LBDataStructure* lb_data = 0;
    IOP::TaggedComponent_var  eff_component = 0;
    CdmwLB::Strategy_var lb_strategy = CdmwLB::Strategy::_nil();


    /* Take the pointer of the target iogr and put it into a cache (map) */

    size_t hash_lb_key = object_group->_hash(100000L);

    try
    {

        lb_cache_pos = m_lb_cache.find(hash_lb_key);
        if (lb_cache_pos == m_lb_cache.end())
        {
            eff_component = ri-> get_effective_component(CdmwLB::TAG_LB_GROUP);
            lb_data = new LBDataStructure();
            IOP::ServiceContext service_context;
            lb_group_tag = m_iogr_factory->get_lb_tagged_components(eff_component.in());
            lb_iogr_version = lb_group_tag->object_group_ref_version;

                std::ostringstream  oss ;
                oss <<  "  TagLBGroupTaggedComponent: "
            << "\n        version = "
            << (int)lb_group_tag->version.major << "."
            << (int)lb_group_tag->version.minor
            << "\n        lb_domain_id = " << lb_group_tag->lb_domain_id.in()
            << "\n        object_group_id = " << lb_group_tag->object_group_id
            << "\n        object_group_ref_version = " << lb_iogr_version
            << "\n        lb_policy = " << lb_group_tag->lb_policy
                <<  std::endl;
                CDMW_DEBUG(LOG::Logger::Get_Logger(LB_GROUP_MANAGER_LOGGER),oss.str());

            service_context = (*(m_iogr_factory->create_group_version_service_context(lb_iogr_version, lb_group_tag->object_group_id)));
            lb_data -> obj_group_version = lb_group_tag->object_group_ref_version;
            lb_data -> svc_cxt = service_context;
            lb_data -> isLB = 1;
            lb_data -> time_0 = Cdmw::CommonSvcs::Time::Util::get_current_time();
            lb_data -> waiting_for_response = 0 ;        
        }
        else
            if (lb_cache_pos -> second -> isLB == 0)
                is_LB_request = false;
            else
            {
                lb_data =  lb_cache_pos->second;
                if (lb_data -> waiting_for_response == 0)
                {
                    CDMW_DEBUG_3(LOG::Logger::Get_Logger(LB_GROUP_MANAGER_LOGGER),"start duration : time_0 = current_time"); 
                    lb_data -> time_0 = Cdmw::CommonSvcs::Time::Util::get_current_time();
                    lb_data -> waiting_for_response = 1; 
                }   
            }

    }
    catch(const CORBA::BAD_PARAM& )
    {
        LBDataStructure* lb_data = new LBDataStructure();
        lb_data -> strategy_ptr = CdmwLB::Strategy::_nil();
        lb_data -> isLB = 0 ;
        m_lb_cache.insert(std::make_pair(hash_lb_key,lb_data));
        is_LB_request = false;
    }
    catch (const CORBA::SystemException&)
    {
        CDMW_INFO(LOG::Logger::Get_Logger(LB_GROUP_MANAGER_LOGGER),"ERROR: Error occured when creating a service context");
        throw CORBA::INTERNAL(OrbSupport::INTERNALLoadBalancingError, CORBA::COMPLETED_NO);;
    }

    if(is_LB_request)
    {
        CDMW_MUTEX_GUARD(m_mutex); //PCR-0668
    delete lb_group_tag;
    CORBA::String_var str = ri->operation();
    if (strcmp(str.in(),"_is_a"))
    {
        IOP::ServiceContext service_context = lb_data->svc_cxt;

        ri->add_request_service_context (service_context, 1);

        if(m_isIn == 0)
        {
        try
        {
            if(lb_cache_pos == m_lb_cache.end())
            {
            lb_strategy = m_lb_factory->create_strategy(object_group.in());
            CORBA::String_var strategy_name = lb_strategy->name();
            if((!(LBConfiguration::M_lb_time_refresh == 0)) && (strcmp(strategy_name.in(), "RoundRobin")))
              std::cerr << "   LoadBalancing Strategy Selected   ==>   " << strategy_name.in()
                    << " with LB TIME REFRESH="<< LBConfiguration::M_lb_time_refresh << std::endl;
            else
              std::cerr << "   LoadBalancing Strategy Selected   ==>   " << strategy_name.in() << std::endl;
            lb_data->strategy_ptr = CdmwLB::Strategy::_duplicate(lb_strategy.in());
            m_lb_cache.insert(std::make_pair(hash_lb_key , lb_data));
            }
            else
            lb_strategy = lb_data->strategy_ptr;

        }catch (const CORBA::SystemException&)
        {
            CDMW_INFO(LOG::Logger::Get_Logger(LB_GROUP_MANAGER_LOGGER),"ERROR: Error occured when creating a strategy policy");
            throw CORBA::INTERNAL(OrbSupport::INTERNALLoadBalancingError, CORBA::COMPLETED_NO);;
        }
        CORBA::Object_var obj = lb_strategy->next_member(ri);
        m_isIn = 1;
        
        
        //--------------------- check that the LB Client Request timeout is not expired                     
        if (isLBRequestTimeoutSetted() && (lb_data->waiting_for_response !=0) )
        {
            TimeBase::TimeT duration
                     = Cdmw::CommonSvcs::Time::Util::get_current_time() - lb_data->time_0;

            lb_data->waiting_for_response =  duration;
            if  (duration > get_LBRequestTimeout() )
            {
                    CDMW_ERROR(LOG::Logger::Get_Logger(LB_GROUP_MANAGER_LOGGER), 
                               "The Timeout for this request [op = " << ri->operation() <<"] is expired, CORBA::TIMEOUT exception raised ");
                    CDMW_DEBUG(LOG::Logger::Get_Logger(LB_GROUP_MANAGER_LOGGER), 
                               "\n   Timeout = "                       << get_LBRequestTimeout() << 
                               "\n   duration = "                      << duration  << 
                               "\n   lb_data->time_0 = "               << lb_data->time_0 << 
                               "\n   lb_data->waiting_for_response = " << lb_data->waiting_for_response);
                     lb_data->waiting_for_response =  0;
                     lb_data->time_0 = 0;
                     throw CORBA::TIMEOUT(0, CORBA::COMPLETED_NO);; 
                     
            } 
        }  
        //---------------------
        CDMW_DEBUG_3(LOG::Logger::Get_Logger(LB_GROUP_MANAGER_LOGGER),
                     "IOR LB Server Replica (send request) : " << m_orb->object_to_string(obj.in()));


        num_of_network_interfaces_on = obj->_stubobj ()->base_profiles ().profile_count ();

        throw PortableInterceptor::ForwardRequest(obj.in());          
        
        
        }
        else
        m_isIn = 0;
    }

    }
    else
        delete lb_group_tag;
}


void
ClientRequestInterceptor_impl::send_poll (PortableInterceptor::ClientRequestInfo_ptr ri)
    throw (CORBA::SystemException)
{
    if(ri->operation() == FORBIDDEN_OPERATION) {
      return;
    }
    // The log below must not be recorded if the intercepted operation is add_log(). Else if Log Collector
    // is used we enter in a cyclic recording of the log (infinite call to add_log)!
    // This explains why the macro is called only after the condition if()
    CDMW_DEBUG_5(LOG::Logger::Get_Logger(LB_GROUP_MANAGER_LOGGER), "    send_poll()   " );
}

void
ClientRequestInterceptor_impl::receive_reply (PortableInterceptor::ClientRequestInfo_ptr ri)
    throw (CORBA::SystemException)
{
    if(ri->operation() == FORBIDDEN_OPERATION) {
      return;
    }
    // The log below must not be recorded if the intercepted operation is add_log(). Else if Log Collector
    // is used we enter in a cyclic recording of the log (infinite call to add_log)!
    // This explains why the macro is called only after the condition if()
    CDMW_DEBUG_3(LOG::Logger::Get_Logger(LB_GROUP_MANAGER_LOGGER), "    receive_reply()   " );
    LBCache::iterator lb_cache_pos;
    
    PortableGroup::ObjectGroup_var object_group = CORBA::Object::_duplicate(ri->target());
    size_t hash_lb_key = object_group->_hash(100000L);
    LBDataStructure* lb_data = new LBDataStructure();
    lb_cache_pos = m_lb_cache.find(hash_lb_key);
    if (lb_cache_pos == m_lb_cache.end())
    {
        CDMW_DEBUG_3(LOG::Logger::Get_Logger(LB_GROUP_MANAGER_LOGGER), "waiting_for_response not setted to 0  " );
    }
    else
    {

        lb_data =  lb_cache_pos->second;
        lb_data->time_0 = 0;
        lb_data->waiting_for_response = 0;   
    }    
}

void
ClientRequestInterceptor_impl::receive_exception (PortableInterceptor::ClientRequestInfo_ptr ri)
    throw (CORBA::SystemException,
           PortableInterceptor::ForwardRequest)
{
    if(ri->operation() == FORBIDDEN_OPERATION) {
      return;
    }

    // The log below must not be recorded if the intercepted operation is add_log(). Else if Log Collector
    // is used we enter in a cyclic recording of the log (infinite call to add_log)!
    // This explains why the macro is called only after the condition if()
    CDMW_DEBUG_3(LOG::Logger::Get_Logger(LB_GROUP_MANAGER_LOGGER), "    receive_exception()   " );
    IOP::TaggedComponent_var eff_component = 0;
    CORBA::Boolean is_LB_request = true;

    try
    {
        eff_component = ri->get_effective_component(CdmwLB::TAG_LB_GROUP);
    }
    catch(const CORBA::BAD_PARAM&)
    {
        is_LB_request = false;
    }
    if(is_LB_request)
    {
        PortableInterceptor::ReplyStatus rs = ri->reply_status ();
        if (rs != PortableInterceptor::SYSTEM_EXCEPTION)
        {
        return;
        }
        CORBA::Any_var ex = ri->received_exception ();

        TAO_OutputCDR cdr;

        if (!(ex->impl ()->marshal_value (cdr)))
        return;
        TAO_InputCDR icdr (cdr);

        CORBA::String_var rep_id;
        if (!(icdr.read_string (rep_id.out ())))
        throw CORBA::MARSHAL(OrbSupport::INTERNALLoadBalancingError, CORBA::COMPLETED_NO);
        //std::cerr << "System Exception raised: " << rep_id.in() << std::endl;
        
        set_LBPolicyTimeout(ri);

        if ((strcmp (rep_id.in (),
             "IDL:omg.org/CORBA/TRANSIENT:1.0") == 0)
        ||
        (strcmp (rep_id.in (),
             "IDL:omg.org/CORBA/COMM_FAILURE:1.0") == 0)
        ||
        (strcmp (rep_id.in (),
             "IDL:omg.org/CORBA/NO_RESPONSE:1.0") == 0)
        ||
        (strcmp (rep_id.in (),
             "IDL:omg.org/CORBA/OBJ_ADAPTER:1.0") == 0))

        {
        
             CDMW_DEBUG_3(LOG::Logger::Get_Logger(LB_GROUP_MANAGER_LOGGER), 
                     "System Exception raised: " << rep_id.in()  << " (The LB Request will be reforwarded) " );
           throw PortableInterceptor::ForwardRequest(ri->target());
        }
        else
        {
        
            CDMW_DEBUG_3(LOG::Logger::Get_Logger(LB_GROUP_MANAGER_LOGGER), 
                         "System Exception raised: " << rep_id.in()  << " (The LB Request NOT will be reforwarded) " );
            LBCache::iterator lb_cache_pos;

            PortableGroup::ObjectGroup_var object_group = CORBA::Object::_duplicate(ri->target());
            size_t hash_lb_key = object_group->_hash(100000L);
            LBDataStructure* lb_data = new LBDataStructure();
            lb_cache_pos = m_lb_cache.find(hash_lb_key);
            if (lb_cache_pos == m_lb_cache.end())
            {
                CDMW_DEBUG_3(LOG::Logger::Get_Logger(LB_GROUP_MANAGER_LOGGER), "waiting_for_response not setted to 0  " );
            }
            else
            {
                CDMW_DEBUG_3(LOG::Logger::Get_Logger(LB_GROUP_MANAGER_LOGGER), "timeout setted to 0  " ); 
                lb_data =  lb_cache_pos->second;
                lb_data->time_0 = 0;
                lb_data->waiting_for_response = 0;   
            }   
                         
            return;
        }
    }
    else
    {
        return;
    }
}


void
ClientRequestInterceptor_impl::receive_other (PortableInterceptor::ClientRequestInfo_ptr ri)
    throw (CORBA::SystemException,
           PortableInterceptor::ForwardRequest)
{
    if(ri->operation() == FORBIDDEN_OPERATION) {
      return;
    }
    // The log below must not be recorded if the intercepted operation is add_log(). Else if Log Collector
    // is used we enter in a cyclic recording of the log (infinite call to add_log)!
    // This explains why the macro is called only after the condition if()
    CDMW_DEBUG_5(LOG::Logger::Get_Logger(LB_GROUP_MANAGER_LOGGER), "    receive_other()   " );
    LBCache::iterator lb_cache_pos;
    IOP::TaggedComponent_var eff_component = 0;
    CdmwLB::TagLBGroupTaggedComponent* lb_group_tag=0;
    CORBA::Boolean is_LB_request = true;
    PortableGroup::ObjectGroupRefVersion lb_iogr_version=0;
    LBDataStructure* lb_data = 0;
    PortableGroup::ObjectGroup_var obj_grp = CORBA::Object::_duplicate(ri->target());
    size_t hash_lb_key2 = obj_grp->_hash(100000L);

    try
    {
    eff_component = ri->get_effective_component(CdmwLB::TAG_LB_GROUP);
    }
    catch (CORBA::BAD_PARAM &)
    {
    is_LB_request = false;
    }
    if(is_LB_request)
    {
//  CDMW_MUTEX_GUARD(m_mutex); //PCR-0668
        PortableGroup::ObjectGroup_var object_group = CORBA::Object::_duplicate(ri->forward_reference());

    CORBA::ULong n_profile = m_iogr_factory->profile_count(object_group.in());
    if (n_profile == num_of_network_interfaces_on)
    {
       CDMW_INFO(LOG::Logger::Get_Logger(LB_GROUP_MANAGER_LOGGER),"The num of profile in the IOGR is equal to 1 (or num of network interfaces on), it means that just fallback replica is in the group.");
       CDMW_INFO(LOG::Logger::Get_Logger(LB_GROUP_MANAGER_LOGGER),"num of profile in the IOGR " << n_profile << "   num of network interfaces on " << num_of_network_interfaces_on);
       CDMW_INFO(LOG::Logger::Get_Logger(LB_GROUP_MANAGER_LOGGER),"TRANSIENT exception will be raised ");
       throw CORBA::TRANSIENT(OrbSupport::TRANSIENT, CORBA::COMPLETED_NO);
    } 

    try
    {
        lb_data = new LBDataStructure();
        lb_cache_pos = m_lb_cache.find(hash_lb_key2);
        IOP::ServiceContext service_context;
        lb_group_tag = m_iogr_factory->decode_profile_with_group_tag(object_group.in(), 0);
        lb_iogr_version = lb_group_tag->object_group_ref_version;
        lb_data->obj_group_version = lb_iogr_version;
        lb_data->svc_cxt = (*(m_iogr_factory->create_group_version_service_context(lb_iogr_version, lb_group_tag->object_group_id)));
        lb_data->strategy_ptr =  m_lb_factory -> create_strategy (object_group.in());
        lb_data -> isLB = 1;
        lb_data->time_0 = Cdmw::CommonSvcs::Time::Util::get_current_time();
        lb_data->waiting_for_response = 1;
        if (lb_cache_pos!=m_lb_cache.end())
          m_lb_cache.erase(lb_cache_pos);

        m_lb_cache.insert(std::make_pair(hash_lb_key2 , lb_data));
    } catch (const CORBA::SystemException&)
    {
        delete lb_group_tag;
        CDMW_INFO(LOG::Logger::Get_Logger(LB_GROUP_MANAGER_LOGGER),"ERROR: Error occured when creating a strategy policy");
        throw CORBA::INTERNAL(OrbSupport::INTERNALLoadBalancingError, CORBA::COMPLETED_NO);
    }
    delete lb_group_tag;
    }
    else
        delete lb_group_tag;
}
//////////////////////////////////
void ClientRequestInterceptor_impl::set_LBPolicyTimeout(PortableInterceptor::ClientRequestInfo_ptr ri)
{
        if (!isLBRequestPolicyTimeoutSetted) 
        {
                CDMW_DEBUG_3(LOG::Logger::Get_Logger(LB_GROUP_MANAGER_LOGGER),"Set LB Client Request Timeout  "  );  
                set_LBRequestTimeout(getClientRequestTimeoutValue(ri));
        } 

}
TimeBase::TimeT ClientRequestInterceptor_impl::getClientRequestTimeoutValue(PortableInterceptor::ClientRequestInfo_ptr ri)
{
    
    TimeBase::TimeT duration_time = extractRTTPolicyValueFromRequest(ri);

    if (duration_time != 0)
    {
        isLBRequestPolicyTimeoutSetted = true;
    }
    else 
    {
        duration_time = extractFTPolicyValueFromRequest(ri);
        if (duration_time==0)
            duration_time = REQUEST_TIMEOUT;
      
    }     
    return duration_time;
}
TimeBase::TimeT ClientRequestInterceptor_impl::extractRTTPolicyValueFromRequest(PortableInterceptor::ClientRequestInfo_ptr ri)
{
    TimeBase::TimeT duration_time = 0;
    CDMW_DEBUG_3(LOG::Logger::Get_Logger(LB_GROUP_MANAGER_LOGGER),"Try to obtain RELATIVE_RT_TIMEOUT_POLICY from LB Client Request  "  );
    CORBA::PolicyType policy_type = Messaging::RELATIVE_RT_TIMEOUT_POLICY_TYPE;
    CORBA::Policy_var policy      = ri->get_request_policy (policy_type);
    if(! CORBA::is_nil(policy)) {
       Messaging::RelativeRoundtripTimeoutPolicy_var durationPolicy =
       Messaging::RelativeRoundtripTimeoutPolicy::_narrow(policy);
       CDMW_DEBUG_3(LOG::Logger::Get_Logger(LB_GROUP_MANAGER_LOGGER),"RELATIVE_RT_TIMEOUT_POLICY  value : " << durationPolicy->relative_expiry()  );
       duration_time = durationPolicy->relative_expiry() ;    
   }
   return duration_time;    
}
TimeBase::TimeT ClientRequestInterceptor_impl::extractFTPolicyValueFromRequest(PortableInterceptor::ClientRequestInfo_ptr ri)
{
    TimeBase::TimeT duration_time = 0;
    CDMW_DEBUG_3(LOG::Logger::Get_Logger(LB_GROUP_MANAGER_LOGGER),"RELATIVE_RT_TIMEOUT_POLICY  not setted "  );
    CDMW_DEBUG_3(LOG::Logger::Get_Logger(LB_GROUP_MANAGER_LOGGER),"Try to obtain FT::REQUEST_DURATION_POLICY from LB Client Request  "  );
    CORBA::Policy_var policy =
        ri->get_request_policy (::FT::REQUEST_DURATION_POLICY);
    if(! CORBA::is_nil(policy)) 
    {   
        ::FT::RequestDurationPolicy_var reqDurationPolicyFT =  ::FT::RequestDurationPolicy::_narrow (policy); 
        CDMW_DEBUG_3(LOG::Logger::Get_Logger(LB_GROUP_MANAGER_LOGGER),"FT::REQUEST_DURATION_POLICY  value : " <<  reqDurationPolicyFT->request_duration_policy_value() );
        duration_time = reqDurationPolicyFT->request_duration_policy_value();
    }   
   return duration_time;    
}


void ClientRequestInterceptor_impl::set_LBRequestTimeout(TimeBase::TimeT timeout)
{
    m_LBRequestTimeout = timeout;
}
TimeBase::TimeT ClientRequestInterceptor_impl::get_LBRequestTimeout()
{
    return m_LBRequestTimeout;
}
bool ClientRequestInterceptor_impl::isLBRequestTimeoutSetted()
{
    return 
       (get_LBRequestTimeout() != 0);
}

}//End of namespace LBInit

}//End of namespace LB

}//End of namespace Cdmw
