/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI.
 * Copyright (C) SELEX SISTEMI INTEGRATI 2004-2009. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

/**
 * @brief Implementation for Server Request Interceptor.
 *
 * @author Raffaella Panella <rpanella@amjsv.it>
 * @author Antonello Cedrone <a.cedrone@vitrociset.it>
 */

#include "lbinit/LBServerRequestInterceptor.hpp"
#include <iostream>
#include <LoadBalancing/lbstrategy/StrategyFactory_impl.hpp>
#include "LoadBalancing/lbcommon/StateTransferConfig.hpp"

#include "lbcommon/config.hpp"
#include <Foundation/logging/Logger.hpp>
#include <sstream>
namespace LOG=Cdmw::Logging;


#include <LoadBalancing/lbcommon/TaoIOGRFactory.hpp>



#define ECHO_HEADER() \
    "[CDMW LBServerRequestInterceptor] "


#ifndef ENABLE_LB_DEBUG_DUMP

#   define DEBUG_DUMP(comment)
#   define DEBUG_ECHO(comment)

#else

#   define DEBUG_DUMP(comment) \
do {\
    std::cerr << ECHO_HEADER() << " --> " << comment << std::endl;\
} while(0)

#   define DEBUG_ECHO(comment) \
do {\
    std::cerr << comment << std::endl;\
} while(0)

#endif

static size_t send_repetition = 2;
static const int TOPIC_ID = 1003;


namespace Cdmw
{
    
namespace LB
{
    
namespace LBInit
{

ServerRequestInterceptor_impl::ServerRequestInterceptor_impl (CORBA::ORB_ptr orb,
                                  PortableGroup::GOA_ptr goa,
                                                              const char *name, 
                                                              Cdmw::LB::IOGRFactory* iogr_factory)
    : m_miop_corbaloc(Cdmw::LB::LBConfiguration::miop_corbaloc),
      m_orb (CORBA::ORB::_duplicate(orb)),
      m_name(name),
      m_iogr_factory(iogr_factory),
      m_protocol_handler(orb, goa, m_miop_corbaloc, send_repetition),
      m_update_manager(Cdmw::CommonSvcs::LocalTopicUpdateManager::bind(&m_protocol_handler)),
      m_group_cache(orb, m_codec, *m_update_manager),
      m_set_command(m_group_cache),
      m_remove_command(m_group_cache),
      m_topic_update_handler(m_codec) 
{
    m_topic_update_handler.register_command(::CdmwLB::SET,
                                            &m_set_command);

    m_topic_update_handler.register_command(::CdmwLB::REMOVE,
                                            &m_remove_command);

    m_update_manager->set_topic_handler(TOPIC_ID, &m_topic_update_handler);
}

void
ServerRequestInterceptor_impl::destroy ()
    throw (CORBA::SystemException)
{
    delete m_update_manager;
    delete m_iogr_factory;
}

char *
ServerRequestInterceptor_impl::name ()
    throw (CORBA::SystemException)
{
    return CORBA::string_dup (this->m_name.in ());
}

void
ServerRequestInterceptor_impl::receive_request_service_contexts(
    PortableInterceptor::ServerRequestInfo_ptr ri)
    throw (CORBA::SystemException, PortableInterceptor::ForwardRequest)
{
    CDMW_DEBUG_3(LOG::Logger::Get_Logger(LB_GROUP_MANAGER_LOGGER), "    receive_request_service_contexts()   " );
    CORBA::Object_var obj;
    try
    {
        //
        // Get LB_GROUP_VERSION request service context
        //
        IOP::ServiceContext_var sc
            = ri->get_request_service_context(CdmwLB::LB_GROUP_VERSION);
        CdmwLB::LBGroupVersionServiceContext lb_service_ctx ;
        

        std::ostringstream oss;
        if (sc->context_id == CdmwLB::LB_GROUP_VERSION)
        {
            lb_service_ctx = m_iogr_factory->decode_group_version_service_context(sc.in());
            
            oss << "  LBGroupVersionServiceContext: "
            << "\n     Object group ref version: " << lb_service_ctx.object_group_ref_version
            << "\n     Object group Id: " << lb_service_ctx.object_group_id
            <<  std::endl;
            CDMW_DEBUG(LOG::Logger::Get_Logger(LB_GROUP_MANAGER_LOGGER),oss.str());

            CdmwLB::CacheData* cache_data = m_group_cache.get(lb_service_ctx.object_group_id);
            PortableGroup::ObjectGroupRefVersion new_version = cache_data->object_group_ref_version;
            PortableGroup::ObjectGroup_var object_group = cache_data->iogr;
            CORBA::Long n_profile = m_iogr_factory->profile_count(object_group.in());
                    
            if ((lb_service_ctx.object_group_ref_version < new_version) || (n_profile == 1))
            {
                oss << "    New Object group ref version: " << new_version << std::endl;
                CDMW_DEBUG(LOG::Logger::Get_Logger(LB_GROUP_MANAGER_LOGGER),oss.str());
                // Get the most recent object group reference.
                // Trigger LOCATION_FORWARD_PERM
                throw PortableInterceptor::ForwardRequest(object_group.in()); 
            }       
        }
        else
        {
            CDMW_DEBUG(LOG::Logger::Get_Logger(LB_GROUP_MANAGER_LOGGER),"ERROR: Bad context id for IOP::LB_GROUP_VERSION");
            throw CORBA::INTERNAL(OrbSupport::INTERNALLoadBalancingError, CORBA::COMPLETED_NO);
        }
        
    }
    catch(const CORBA::BAD_PARAM&)
    {
         CDMW_DEBUG_3(LOG::Logger::Get_Logger(LB_GROUP_MANAGER_LOGGER),"No service context LB_GROUP_VERSION for that request");
        
        // This is not an error while a request can come to a single reference.
        // with no LB service context.
    }
    catch(const ::CdmwLB::GroupRefCache::UnknownGroupReference &)
    {
         CDMW_INFO(LOG::Logger::Get_Logger(LB_GROUP_MANAGER_LOGGER),"GroupRefCache Exception: UnknownGroupReference raised");
    }
    catch(const CORBA::SystemException &)
    {
         CDMW_INFO(LOG::Logger::Get_Logger(LB_GROUP_MANAGER_LOGGER),"No Check on the object group version...");  
    }
    
}          

void
ServerRequestInterceptor_impl::receive_request(PortableInterceptor::ServerRequestInfo_ptr ri)
    throw (CORBA::SystemException, PortableInterceptor::ForwardRequest)
{
         CDMW_DEBUG_3(LOG::Logger::Get_Logger(LB_GROUP_MANAGER_LOGGER), "    receive_request()   " );
}

void
ServerRequestInterceptor_impl::send_reply(PortableInterceptor::ServerRequestInfo_ptr ri)
    throw (CORBA::SystemException)
{
     CDMW_DEBUG_3(LOG::Logger::Get_Logger(LB_GROUP_MANAGER_LOGGER), "    receive_reply()   " );
}

void
ServerRequestInterceptor_impl::send_other(PortableInterceptor::ServerRequestInfo_ptr ri)
    throw (CORBA::SystemException, PortableInterceptor::ForwardRequest)
{
     CDMW_DEBUG_3(LOG::Logger::Get_Logger(LB_GROUP_MANAGER_LOGGER), "    send_other()   " );

    // nothing to do!
}

void
ServerRequestInterceptor_impl::send_exception(PortableInterceptor::ServerRequestInfo_ptr ri)
    throw (CORBA::SystemException, PortableInterceptor::ForwardRequest)
{
     CDMW_DEBUG_3(LOG::Logger::Get_Logger(LB_GROUP_MANAGER_LOGGER), "    send_exception  " );
    // nothing to do!
}

}//End of namespace LBInit

}//End of namespace LB

}//End of namespace Cdmw
