/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


// Standard Files 
#include <iostream>
#include <string>

// Cdmw Files
#include <Foundation/common/Options.hpp>
#include <Foundation/ossupport/OS.hpp>
#include <Foundation/orbsupport/CORBA.hpp>
#include <Foundation/orbsupport/OrbSupport.hpp>
#include <Foundation/orbsupport/StrategyList.hpp>
#include <Foundation/logging/Logger.hpp>
#include <ConfAndPlug/cdmwinit/CdmwInterface.hpp>
#include <ConfAndPlug/cdmwinit/ProcessControl.hpp>
#include <FaultTolerance/ftinit/FTServiceInit.hpp>
#include <SystemMngt/platforminterface/PlatformInterface.hpp>





class MyProcessBehaviour : public Cdmw::CdmwInit::ProcessControl
{
    public:
    
        MyProcessBehaviour()
        {
            logger = Cdmw::Logging::Logger::Get_Logger("TEST_MANAGED_PROCESS");
        };
    
        virtual CORBA::ULong nb_steps()
        throw( CORBA::SystemException )
        {
            return 1;
        };

        void on_next_step()
        throw( CdmwPlatformMngt::ProcessDelegate::InvalidStep,
               CORBA::SystemException )
        {};
        
        virtual void on_initialise(
            const CdmwPlatformMngtBase::StartupKind& startup_kind )
        throw( CORBA::SystemException )
        {
            CDMW_INFO(logger, 
                      Cdmw::PlatformMngt::PlatformInterface::Get_process_name() 
                      << " ===> on_initialise called");
        };

        virtual void on_run()
        throw( CdmwPlatformMngt::ProcessDelegate::NotReadyToRun,
               CORBA::SystemException )
        {
            CDMW_INFO(logger, 
                      Cdmw::PlatformMngt::PlatformInterface::Get_process_name() 
                      << " ===> on_run called");
        };

        virtual void on_stop()
        throw( CORBA::SystemException )
        {
            CDMW_INFO(logger, 
                      Cdmw::PlatformMngt::PlatformInterface::Get_process_name() 
                      << " ===> on_stop called");
        };
        
    
    private:
        Cdmw::Logging::Logger* logger;

};





int main( int argc, char* argv[] )
{

    try
    {
        // By default use 2 threads
        int nbThreads = 2;

        // checks the orb threads option
        std::string orbThreadsOption =
            Cdmw::OsSupport::OS::get_option_value(
                argc, argv, 
                Cdmw::Common::Options::ORB_THREADS);

        if ((orbThreadsOption != "no") && (orbThreadsOption != "yes")) {
            int nb = ::atoi(orbThreadsOption.c_str());

            if (nb > 0) {
                nbThreads = nb;
            }
        }
        
        // Initialise FT service
        Cdmw::CdmwInit::CDMW_preORBInit(argc, argv);

        // Initialises the ORB
        Cdmw::OrbSupport::StrategyList strategyList;
        strategyList.add_OrbThreaded();
        strategyList.add_PoaThreadPool(nbThreads);

        CORBA::ORB_var orb = Cdmw::OrbSupport::OrbSupport::ORB_init(
            argc, argv, strategyList );

        Cdmw::Logging::Logger* logger = 
            Cdmw::Logging::Logger::Get_Logger("TEST_MANAGED_PROCESS");

        CDMW_INFO(logger, "Starting with a thread pool of " << nbThreads);

        // Get the root POA
        CORBA::Object_var obj = orb->resolve_initial_references( "RootPOA" );
        PortableServer::POA_var rootPOA
            = PortableServer::POA::_narrow( obj.in() );

        PortableServer::POAManager_var poaMgr = rootPOA->the_POAManager();
        poaMgr->activate();

        // creates the process behaviour
        std::auto_ptr<MyProcessBehaviour> pProcessBehaviour(
            new MyProcessBehaviour() );

        // init Cdmw
        CDMW_INFO(logger, "Calling CDMW_init");
        Cdmw::CdmwInit::CDMW_init(orb.in(), 
                                  argc, 
                                  argv, 
                                  pProcessBehaviour.get() );

        // Enter in the main loop
        CDMW_INFO(logger, "Calling orb->run()");
        orb->run();
        
        Cdmw::CdmwInit::CDMW_cleanup(orb.in());
        
        Cdmw::OrbSupport::OrbSupport::ORB_cleanup(orb.in());
        orb->destroy();
        
        CDMW_INFO(logger, "Process stopped");

    }
    catch( const CORBA::Exception &e )
    {
        std::cerr <<__FILE__<<" "<<__LINE__<<" "<< e._name() << std::endl;
        return EXIT_FAILURE;
    }
    catch( const Cdmw::Exception &e )
    {
        std::cerr <<__FILE__<<" "<<__LINE__<<" "<< e.what() << std::endl;
        return EXIT_FAILURE;
    }
    catch( ... )
    {
        std::cerr << "Unexpected exception" << std::endl;
        return EXIT_FAILURE;
    }

    return EXIT_SUCCESS;
}

