/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

 
// Cdmw Files
#include <Foundation/orbsupport/CORBA.hpp>
#include <Foundation/ossupport/OS.hpp>
#include <SystemMngt/platforminterface/PlatformInterface.hpp>
#include <Foundation/commonsvcs/naming/NamingUtil.hpp>
#include <Foundation/common/Locations.hpp>
#include <Repository/repositoryinterface/RepositoryInterface.hpp>
#include <FaultTolerance/ftcommon/FTLocation.hpp>
#include <FaultTolerance/ftcommon/FTObjectIds.hpp>

// StateTransferPerf Files
#include <STPSupervisionServerProcessBehaviour.hpp>
#include <STPCommon.hpp>


STPSupervisionServerProcessBehaviour::STPSupervisionServerProcessBehaviour(
        CORBA::ORB_ptr orb,
        std::string protocol,
        std::string nb_worker_threads)
{
    m_orb = CORBA::ORB::_duplicate( orb );
    m_protocol = protocol;
    m_nb_worker_threads = nb_worker_threads;
    m_stp_server_impl = NULL;
}

    
STPSupervisionServerProcessBehaviour::~STPSupervisionServerProcessBehaviour() 
    throw ()
{
    if (m_stp_server_impl != NULL)
        m_stp_server_impl->_remove_ref();
}

    
CORBA::ULong STPSupervisionServerProcessBehaviour::nb_steps()
    throw( CORBA::SystemException )
{
    return 1;
}

    
void STPSupervisionServerProcessBehaviour::on_initialise(
    const CdmwPlatformMngtBase::StartupKind& startup_kind )
throw( CORBA::SystemException )
{
    using namespace Cdmw::CommonSvcs::Naming;

    // Get application and process name
    m_application_name = Cdmw::PlatformMngt::PlatformInterface::Get_application_name();
    m_process_name     = Cdmw::PlatformMngt::PlatformInterface::Get_process_name();
    INFO(m_process_name << ": ----- Supervision Server initialise BEGIN --------");
    ECHO(m_process_name << ": application = " << m_application_name);

    try {
        // get root POA            
        CORBA::Object_var obj = m_orb->resolve_initial_references( "RootPOA" );
        PortableServer::POA_var rootPOA
            = PortableServer::POA::_narrow( obj.in() );
        PortableServer::POAManager_var poaMgr = rootPOA->the_POAManager();

        // get the admin root context
        CdmwNamingAndRepository::Repository_var repository_ref 
            = Cdmw::NamingAndRepository::RepositoryInterface::get_repository();
        CosNaming::NamingContext_var admin_root_ctx 
            = repository_ref->resolve_root_context(Cdmw::Common::Locations::ADMIN_ROOT_CONTEXT_ID);
        NamingInterface ni(admin_root_ctx);

        m_stp_server_impl = new STPSupervisionServer_impl(m_orb, rootPOA.in());
        m_stp_server_obj = m_stp_server_impl->_this();

        CORBA::Object_var obj2 = m_orb->resolve_initial_references(Cdmw::FT::FT_GROUP_REPOSITORY_ObjectId);
        CdmwFT::Location::GroupRepository_var grp_repository = CdmwFT::Location::GroupRepository::_narrow(obj2.in());
        CDMW_ASSERT(!CORBA::is_nil(grp_repository.in()));
        FT::Location_var location = grp_repository->the_location();
        ECHO(m_process_name << ": Location = " << NamingInterface::to_string(location));

        StateTransferPerf::ServerConfig config;
        config.server_name = m_process_name.c_str();
        std::string group_name = "GRP_" + m_application_name + "_" + m_protocol + "_" + m_nb_worker_threads;
        config.server_group_name = group_name.c_str();
        config.wire_protocol = (m_protocol == "MIOP") ? StateTransferPerf::MIOP : StateTransferPerf::IIOP;
        config.nb_worker_threads = atoi(m_nb_worker_threads.c_str());
        config.nb_records = StateTransferPerf::DEFAULT_NUMBER_OF_RECORDS;

        ECHO("");
        ECHO(m_process_name << ": Supervision Server configuration = ");
        ECHO("    - server_name         : " << config.server_name);
        ECHO("    - group_name          : " << config.server_group_name);
        ECHO("    - wire_protocol       : " << config.wire_protocol);
        ECHO("    - nb_worker_threads   : " << config.nb_worker_threads);
        ECHO("    - nb_records          : " << config.nb_records);
        ECHO("");

        m_stp_server_admin_impl = new STPServerAdmin_impl(m_orb, rootPOA.in(), config, location, m_stp_server_obj.in());
        m_stp_server_admin_obj = m_stp_server_admin_impl->_this();

        // bind the new context where the server object shall be registered
        CosNaming::Name name;
        name.length(5);
        name[0].id = CORBA::string_dup("StateTransferPerf");
        name[1].id = CORBA::string_dup(m_application_name.c_str());
        name[2].id = CORBA::string_dup(m_protocol.c_str());
        name[3].id = CORBA::string_dup(m_nb_worker_threads.c_str());
        name[4].id = CORBA::string_dup(Cdmw::OsSupport::OS::get_hostname().c_str());

        std::string stp_path = ni.to_string(name);

        ni.bind(stp_path, m_stp_server_admin_obj.in(), true);
        INFO(m_process_name << ": Supervision Server Path : " << stp_path);
        ECHO("");

        // activate POAManager
        ECHO(m_process_name << " : activate POAManager ");
        poaMgr->activate();
    } catch (const CORBA::SystemException& e) {
        INFO(m_process_name << " : CORBA::SystemException raised !!! " << e);
        throw;
    } catch( const CORBA::Exception &e ) {
        INFO(m_process_name << " : CORBA::Exception raised !!! " << e);
        throw;
    } catch( const Cdmw::Exception &e ) {
        INFO(m_process_name << " : Cdmw::Exception raised !!! " << e.what());
        throw;
    } catch( ... ) {
        INFO(m_process_name << " : Unknown Exception raised !!! ");
        throw;
    }
        
    ECHO(m_process_name << ": ----- Supervision Server initialise END --------");
}

    
void STPSupervisionServerProcessBehaviour::on_next_step()
    throw( CdmwPlatformMngt::ProcessDelegate::InvalidStep,
           CORBA::SystemException )
{
    ECHO(m_process_name << ": ----- Supervision Server next step --------");
}

void STPSupervisionServerProcessBehaviour::on_run()
    throw( CdmwPlatformMngt::ProcessDelegate::NotReadyToRun,
           CORBA::SystemException )
{
    INFO(m_process_name << ": ----- Supervision Server running --------");
}

void STPSupervisionServerProcessBehaviour::on_stop()
    throw( CORBA::SystemException )
{    
    INFO(m_process_name << ": ----- Supervision Server stopping --------");
}



