/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */
package cdmw.ccmautoreconnect;

import org.junit.Before;
import org.junit.Test;

import static org.junit.Assert.assertTrue;
import static org.junit.Assert.fail;

import cdmw.testutils.TestManager;
import cdmw.logging.Logger;
import static cdmw.logging.Logger.*;

import com.acme.AutoReconnectDemo.*;



public class ClientActivationThread
{
    public static final String CLIENT_EXECUTOR_IMPL
        = "cdmw.ccmautoreconnect.ClientActivationThread";
    
    private ClientExecutorImpl client;
    private CCM_Client_Context context;
    private Supported supported;
    private EventChecking eventChecking;
    private Logger m_logger;

    public ClientActivationThread ()
    {
        this.m_logger = Logger.Get_Logger("QUALIF.CCM_AUTO_RECONNECT.CLIENT");
        CDMW_INFO(m_logger, "ClientActivationThread");
    }

    @Before
    public void init() {

        CDMW_INFO(m_logger, "ClientActivationThread.init");

        TestManager testManager = TestManager.getLocalInstance();
        
        this.client
            = (ClientExecutorImpl)testManager.getLocalProperty(CLIENT_EXECUTOR_IMPL);
        this.context = null;
        this.supported = null;
        this.eventChecking = null;        
    }

    @Test
    public void doTests()
    {
        CDMW_INFO(m_logger, "ClientActivationThread.doTests");
        // Wait event to be received back
        try {
            CDMW_INFO(m_logger, "Client Thread sleeps 1 sec.");
            System.out.println("Client Thread sleeps 1 sec.");
            cdmw.ossupport.OS.sleep(1000);
        } catch (InterruptedException e) {
            // ignore
        }

        context = client.getContext();
        
        try {
            supported = context.get_connection_to_supported();
            assertTrue(supported != null);
        } catch (org.omg.CORBA.SystemException e) {
            e.printStackTrace();
            fail("Exception received: " + e);
        }
        
        try {
            eventChecking = context.get_connection_to_facet();
            assertTrue(eventChecking != null);
        } catch (org.omg.CORBA.SystemException e) {
            e.printStackTrace();
            fail("Exception received: " + e);
        }
                
        // perform test invocations. 
        // Server is present, success is expected
        performTest(true);
        
        
        CDMW_INFO(m_logger, "----------------------------------------");
        CDMW_INFO(m_logger, "Send kill event to server");

        try {
            TestEvent event = new TestEventImpl();
            event.name = "SUICIDE ME";
            context.push_event_publish(event);
            // event sending should always be succesfull

            // Wait event to be received back
            try {
                CDMW_INFO(m_logger, "Client Thread sleeps 10 sec.");
                System.out.println("Client Thread sleeps 10 sec.");
                cdmw.ossupport.OS.sleep(10000);
            } catch (InterruptedException e) {
                // ignore
            }
        } catch (org.omg.CORBA.SystemException e) {
            // event sending should always be succesfull
            fail("Exception received: " + e);
        }
        
        // perform test invocations. 
        // Server is present again, success is expected
        performTest(true);
    }
    
    
    private void performTest(boolean successExpected)
    {
        CDMW_INFO(m_logger, "Perform calls on Server component");
        if (! successExpected)
            CDMW_INFO(m_logger, "exceptions are expected");
        
        CDMW_INFO(m_logger, "Test calling supported interface");
        try {
            supported.test_supported();
            assertTrue(successExpected);
        } catch (org.omg.CORBA.SystemException e) {
            CDMW_INFO(m_logger, "Exception received: " + e);
            assertTrue(!successExpected);
        }
        
        CDMW_INFO(m_logger, "Test sending event");
        try {
            TestEvent event = new TestEventImpl();
            event.name = "TEST";
            context.push_event_publish(event);
            // event sending should always be succesfull

            // Wait event to be received back
            try {
                CDMW_INFO(m_logger, "Client Thread sleeps 2 sec.");
                System.out.println("Client Thread sleeps 2 sec.");
                cdmw.ossupport.OS.sleep(2000);
            } catch (InterruptedException e) {
                // ignore
            }
        } catch (org.omg.CORBA.SystemException e) {
            // event sending should always be succesfull
            fail("Exception received: " + e);
        }
        
        CDMW_INFO(m_logger, "Test calling facet and test if event was received");
        try {
            boolean eventReceived = eventChecking.was_event_received();
            if (successExpected) {
                assertTrue(eventReceived);
            } else {
                // if success wasn't expected, SystemException should 
                // have been received
                fail(); 
            }
        } catch (org.omg.CORBA.SystemException e) {
            CDMW_INFO(m_logger, "Exception received: " + e);
            assertTrue(!successExpected);
        }

        // Wait TestEvent to be received back
        try {
            CDMW_INFO(m_logger, "Client Thread sleeps 1 sec.");
            System.out.println("Client Thread sleeps 1 sec.");
            cdmw.ossupport.OS.sleep(1000);
        } catch (InterruptedException e) {
            // ignore
        }
        
        if (successExpected) {
            CDMW_INFO(m_logger, "Check that event has been received back");
            assertTrue(this.client.hasEventBeenReceived());
        } else {
            CDMW_INFO(m_logger, "Check that event has'nt been received back");
            assertTrue(!this.client.hasEventBeenReceived());
        }
        
        CDMW_INFO(m_logger, "Calls on Server component perfomed");
        if (! successExpected)
            CDMW_INFO(m_logger, "exceptions were expected");
    }
}
