/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO 
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2009. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */



#include <ccm_ft_publisher_crash/CCM_Client_impl.hpp>
#include <ccm_ft_publisher_crash/PublishedEvent_impl.hpp>

#include <Foundation/common/Assert.hpp>
#include <Foundation/ossupport/OS.hpp>
#include <Foundation/orbsupport/OrbSupport.hpp>
#include <Foundation/logging/CdmwLogging.stub.hpp>
#include <Foundation/logging/Logger.hpp>
#include <Foundation/osthreads/MutexGuard.hpp>

#include <sstream>


namespace {
    const std::string SERVER1_NAME = "Server1";
    const std::string SERVER2_NAME = "Server2";
}


namespace CcmFtPublisherCrash
{

//
// IDL:thalesgroup.com/CcmFtTest/ClientModule/CCM_Client:1.0
//
CCM_Client_impl::CCM_Client_impl()
{
}

CCM_Client_impl::~CCM_Client_impl()
{
    std::cerr << "   ... CCM_Client_impl::~CCM_Client_impl called!" << std::endl;
}


void CCM_Client_impl::set_session_context(Components::SessionContext_ptr ctx)
    throw(CORBA::SystemException)
{
    m_session_context = ::CcmFtPublisherCrash::CCM_Client_Context::_narrow(ctx);
}
        

//
// ccm_activate function : it actives the component (call by the container)
//
void
CCM_Client_impl::ccm_activate()
   throw(CORBA::SystemException)
{
    std::cerr << "   ... CCM_Client_impl::ccm_activate called!" << std::endl;

    // nothing to do; test will be perform in ccm_passivate.
}


//
// ccm_passivate function : it makes stop the component (call by the container)
//
void
CCM_Client_impl::ccm_passivate()
   throw(CORBA::SystemException)
{
    std::cerr << "   ... CCM_Client_impl::ccm_passivate called!" << std::endl;
    
    // perform test in passivate in order to check that all expected events
    // have been received during Client life.
    try {
        // save this executor in CORBATestManager for usage in test
        Cdmw::TestUtils::CORBATestManager::instance()->add(
            "this_executor", 
            dynamic_cast< Session_CCM_Client_ptr > (this));
    
        // create test runner thread
        Cdmw::TestUtils::ThreadedTestRunner* test_thread = 
            new Cdmw::TestUtils::ThreadedTestRunner(
                Cdmw::TestUtils::CORBATestManager::instance()->get_controller());
        // add test to be run
        test_thread->addTest( 
            CppUnit::TestFactoryRegistry::getRegistry("test_ccm_ft_publisher_crash").makeTest());
        // start test
        test_thread->start();

        // wait test to be finished
        test_thread->join();
        delete test_thread;
        
        // print test summary
        Cdmw::TestUtils::CORBATestManager::instance()->summary();
    
    } catch (const CORBA::Exception& e) {
        std::cerr << "CORBA Exception thrown during test: "
                  << Cdmw::OrbSupport::OrbSupport::exception_to_string(e)
                  << std::endl;
    } catch (const std::exception& e) {
        std::cerr << "std::exception thrown during test: "
                  << e.what()
                  << std::endl;
    } catch (...) {
        std::cerr << "Unkown thrown during test !!" << std::endl;
    }
} 


//
// ccm_remove function : it destroys the component (call by the container)
//
void
CCM_Client_impl::ccm_remove()
   throw(CORBA::SystemException)
{
    std::cerr << "   ... CCM_Client_impl::ccm_remove called!" << std::endl;
}


//
// from_publisher consumer
//
void CCM_Client_impl::push_from_publisher (CcmFtPublisherCrash::PublishedEvent * event)
    throw (CORBA::SystemException)
{
    TEST_INFO("Event received from publisher \"" << 
              event->publisher_name() << "\"");
    m_event_list.push(event);
    CORBA::add_ref(event);
}

//
// from_emitter consumer
//
void CCM_Client_impl::push_from_emitter (CcmFtPublisherCrash::EmittedEvent * event)
    throw (CORBA::SystemException)
{
    TEST_INFO("Event received from emitter \"" << 
              event->emitter_name() << "\"");
    m_event_list.push(event);
    CORBA::add_ref(event);
}



//
CPPUNIT_TEST_SUITE_NAMED_REGISTRATION(test_ccm_ft_publisher_crash, "test_ccm_ft_publisher_crash");


test_ccm_ft_publisher_crash::test_ccm_ft_publisher_crash()
{
    // retrieve executor
    CORBA::Object_var obj = 
        Cdmw::TestUtils::CORBATestManager::instance()->get_object("this_executor");
    m_ccm_client = dynamic_cast< CCM_Client_impl* > (obj.in());
}

void
test_ccm_ft_publisher_crash::do_tests()
{
    std::cerr << "   ... test_ccm_ft_publisher_crash::do_tests called!" << std::endl;
    
    TEST_INFO("Number of received events: " << m_ccm_client->m_event_list.size());
    
    // Client should have received:
    //  - PublishedEvent and EmittedEvent from Server1
    //  - PublishedEvent and EmittedEvent from Server2
    //  - PublishedEvent and EmittedEvent from Server1
    
    check_published_event(m_ccm_client->m_event_list, SERVER1_NAME);
    check_emitted_event(m_ccm_client->m_event_list, SERVER1_NAME);
    check_published_event(m_ccm_client->m_event_list, SERVER2_NAME);
    check_emitted_event(m_ccm_client->m_event_list, SERVER2_NAME);
    check_published_event(m_ccm_client->m_event_list, SERVER1_NAME);
    check_emitted_event(m_ccm_client->m_event_list, SERVER1_NAME);
    
    
}

void
test_ccm_ft_publisher_crash::check_published_event(
    CCM_Client_impl::EventList& event_list,
    const std::string& expected_name)
{
    if (event_list.size() == 0) {
        std::ostringstream os;
        os << "A PublishedEvent from " << expected_name << " was expected. "
           << "But it has not been received.";
        CPPUNIT_FAIL(os.str());
    }
    
    Components::EventBase_var event = event_list.front();
    event_list.pop();
    
    try {
        CcmFtPublisherCrash::PublishedEvent* published_event =
            dynamic_cast<CcmFtPublisherCrash::PublishedEvent*>(event.in());
        
        std::ostringstream os1;
        os1 << "A PublishedEvent from " << expected_name << " was expected. "
            << "But received event is not a PublishedEvent.";
        CPPUNIT_ASSERT_MESSAGE(
            os1.str(),
            published_event != 0);
            
        std::ostringstream os2;
        os2 << "A PublishedEvent from " << expected_name << " was expected. "
            << "But received PublishedEvent comes from " 
            << published_event->publisher_name() << ".";
        CPPUNIT_ASSERT_MESSAGE(
            os2.str(),
            expected_name == published_event->publisher_name());
        
    } catch (const CORBA::SystemException& e) {
        std::ostringstream os1;
        os1 << "A PublishedEvent from " << expected_name << " was expected. "
            << "But received event is not a PublishedEvent: " << e;
        CPPUNIT_FAIL(os1.str());
    }
}
                                                   
void
test_ccm_ft_publisher_crash::check_emitted_event(
    CCM_Client_impl::EventList& event_list,
    const std::string& expected_name)
{
    if (event_list.size() == 0) {
        std::ostringstream os;
        os << "A EmittedEvent from " << expected_name << " was expected. "
           << "But it has not been received.";
        CPPUNIT_FAIL(os.str());
    }
    
    Components::EventBase_var event = event_list.front();
    event_list.pop();
    
    try {
        CcmFtPublisherCrash::EmittedEvent* emitted_event =
            dynamic_cast<CcmFtPublisherCrash::EmittedEvent*>(event.in());
        
        std::ostringstream os1;
        os1 << "A EmittedEvent from " << expected_name << " was expected. "
            << "But received event is not a EmittedEvent.";
        CPPUNIT_ASSERT_MESSAGE(
            os1.str(),
            emitted_event != 0);
            
        std::ostringstream os2;
        os2 << "A EmittedEvent from " << expected_name << " was expected. "
            << "But received EmittedEvent comes from " 
            << emitted_event->emitter_name() << ".";
        CPPUNIT_ASSERT_MESSAGE(
            os2.str(),
            expected_name == emitted_event->emitter_name());
        
    } catch (const CORBA::SystemException& e) {
        std::ostringstream os1;
        os1 << "A EmittedEvent from " << expected_name << " was expected. "
            << "But received event is not a EmittedEvent: " << e;
        CPPUNIT_FAIL(os1.str());
    }
}
                                                   



};  // namespace CcmFtPublisherCrash

