/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX Sistemi Integrati. It is derivative work based on PERCO 
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009. 
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/
/* ===================================================================== */


#include <iostream>
#include <sstream>

#include <ccm_ft_recorder/CCM_MyRecorder_impl.hpp>
#include <ccm_ft_recorder/RecordedOperationEvent_impl.hpp>

#include <Foundation/common/Assert.hpp>
#include <Foundation/ossupport/OS.hpp>
#include <Foundation/orbsupport/OrbSupport.hpp>
#include <Foundation/osthreads/MutexGuard.hpp>

#include <SystemMngt/idllib/CdmwPlatformMngtManagedElementContexts.stub.hpp>

namespace {
    const std::string MY_RECORDER_PRIMARY("MyRecorder1");
    const std::string OPERATION_NAME("test_operation");
    const std::string GENERIC_RECORDER_OPERATION_NAME("record_data");
    const std::string PARAM_A_S_SUCIDE_VALUE("MyRecorder: kill yourself !!");
   
}

namespace TestCcmFtRecorder
{

//
// IDL:thalesgroup.com/CcmFtTest/MyRecorderModule/CCM_MyRecorder:1.0
//
CCM_MyRecorder_impl::CCM_MyRecorder_impl()
    : m_component_name("UnamedComponent")
{
    m_session_context = TestCcmFtRecorder::CCM_MyRecorder_Context::_nil();
}

CCM_MyRecorder_impl::~CCM_MyRecorder_impl()
{
    std::cout << "   ... CCM_MyRecorder_impl::~CCM_MyRecorder_impl called!" << std::endl;
}


void CCM_MyRecorder_impl::set_session_context(Components::SessionContext_ptr ctx)
    throw(CORBA::SystemException)
{
  std::cout << "   ... CCM_MyRecorder_impl::set_session_context called! (" << m_component_name << ")" <<  std::endl;
    m_session_context = ::TestCcmFtRecorder::CCM_MyRecorder_Context::_narrow(ctx);
}

::TestCcmFtRecorder::CCM_MyRecorder_Context* CCM_MyRecorder_impl::get_context()
{
    return ::TestCcmFtRecorder::CCM_MyRecorder_Context::_duplicate(m_session_context.in());
}


//
// ccm_activate function : it actives the component (call by the container)
//
void
CCM_MyRecorder_impl::ccm_activate()
   throw(CORBA::SystemException)
{
    try {
        // get ManagedComponentContext
        CORBA::Object_var obj = 
            m_session_context->resolve_service_reference("ManagedComponentContext");
        CdmwPlatformMngt::ManagedElementContexts::ComponentContext_var supContext =
            CdmwPlatformMngt::ManagedElementContexts::ComponentContext::_narrow(obj.in());
        CDMW_ASSERT(!CORBA::is_nil(supContext.in()));
     
        // get component, process, application names
        m_component_name = supContext->managed_element_name();
    } catch (const Components::CCMException& e) {
        std::cerr << "ManagedComponentContext not found !!" << std::endl;
    } catch (const CORBA::SystemException& e) {
        std::cerr << "SystemException getting ManagedComponentContext: " 
                  << e << std::endl;
    }

    std::cout << ">>>>>> " << m_component_name << " is active" << std::endl;
}


//
// ccm_passivate function : it makes stop the component (call by the container)
//
void
CCM_MyRecorder_impl::ccm_passivate()
   throw(CORBA::SystemException)
{
    std::cout << ">>>>>> " << m_component_name << " is passive" << std::endl;
} 


//
// ccm_remove function : it destroys the component (call by the container)
//
void
CCM_MyRecorder_impl::ccm_remove()
   throw(CORBA::SystemException)
{
    std::cout << ">>>>>> " << m_component_name << " is removed" << std::endl;
}



//
// CdmwRecorindg::Recorder::record() implementation
//
void CCM_MyRecorder_impl::record(const CdmwRecording::GroupHeader& header,
                                 const CdmwRecording::GIOP_Buffers& data)
        throw(CORBA::SystemException)
{
    CDMW_MUTEX_GUARD(m_mutex);
    
    std::cout << ">>> CCM_MyRecorder_impl::record() - Starting at " << get_time() << std::endl;
    
    std::cout << "*** CCM_MyRecorder_impl::record() - " << m_component_name << " : " 
              << header.num_pairs << " new Records received from " 
              << header.hostname << " at " << header.base_time << std::endl;

    // Use TAO_InputCDR for unmarshalling
    CORBA::ULong len = data.length();
    const CORBA::Octet* buffer = data.get_buffer();
    size_t char_len = len * sizeof(CORBA::Octet) / sizeof(char);
    TAO_InputCDR input((const char*) buffer, char_len);
    input.reset_byte_order(ACE_static_cast (int, header.flags));
    
    // Unmarshall while TAO_InputCDR is not empty
    CdmwRecording::RequestReplyPair pair;
    int i=0;
    while (input.length() > 0) {
        
        // number of octets still to read
        size_t left = input.length();

        // Unmarshall a RequestReplyPair
        if (input >> pair) {
            std::cout << "  - " << (++i) << ": \"" 
                      << pair.request.operation 
                      << "\" from " << pair.request.server_id << std::endl;

            if (OPERATION_NAME == pair.request.operation.in()) {
                // block 1 seconds, to let client check it's an asynchronous call
                if (i == 1) Cdmw::OsSupport::OS::sleep(1000);
                      
                // check if I'm primary and if it's a suicide order
                if (OPERATION_NAME == pair.request.operation.in() && MY_RECORDER_PRIMARY == m_component_name.in()) {
                    check_suicide_order(pair);
                }
                
                // send event with info about recorded operation to Client
                send_recorded_operation_event(pair);
            } else if (GENERIC_RECORDER_OPERATION_NAME == pair.request.operation.in()) {
                // block 0.200 seconds, to let client check it's an asynchronous call
                if (i == 1) Cdmw::OsSupport::OS::sleep(250);
                      
                // send event with info about recorded operation to Client
                send_recorded_operation_event(pair);
            } else {
                // probably a request for connections; ignore it
            }
                          
        } else {
            // number of read octets
            size_t nb_read = left-input.length();
            std::cerr << "UNMARSHALL ERROR (only " << nb_read <<
                      " octets read)" << std::endl;
            ACE_HEX_DUMP ((LM_INFO,
                           (const char *) input.rd_ptr() - nb_read,
                           nb_read,
                           ACE_TEXT ("read buffer")));
            std::cerr << "- Content of pair.request.head:" << std::endl;
            ACE_HEX_DUMP ((LM_INFO,
                           (const char *) &(pair.request.head),
                           12,
                           ACE_TEXT ("pair.request.head")));
            std::cerr << "- Content of pair.reply.head:" << std::endl;
            ACE_HEX_DUMP ((LM_INFO,
                           (const char *) &(pair.reply.head),
                           12,
                           ACE_TEXT ("pair.reply.head")));
                           
            // send event with failure to Client
            send_failure_event("ERROR Unmarshalling RequestReplyPair");
                           
            // It's useless to try to continue unmarshalling... break loop
            break;
        }
    }

    std::cout << "<<< CCM_MyRecorder_impl::record() - Stoping at " << get_time() << std::endl;
} 


void CCM_MyRecorder_impl::send_recorded_operation_event(
        const CdmwRecording::RequestReplyPair& pair)
{
    try {
        RecordedOperationEvent_var event = new RecordedOperationEvent_impl();

        event->recorder_name(m_component_name.in());
        event->operation_name(pair.request.operation.in());
        
        std::cout << "*** CCM_MyRecorder_impl::send_recorded_operation_event() - "
                  << m_component_name.in() << " : " << pair.request.operation.in() << std::endl;

        if (OPERATION_NAME == pair.request.operation.in()) {
            // try to unmarshall request parameters:
            //   - in TestStruct a
            //   - inout TestStruct b
            CORBA::ULong len = pair.request.data.length();
            const CORBA::Octet* buffer = pair.request.data.get_buffer();
            size_t char_len = len * sizeof(CORBA::Octet) / sizeof(char);
            TAO_InputCDR input1((const char*) buffer, char_len);
            input1.reset_byte_order(ACE_static_cast (int, pair.request.head.flags));

            TestStruct a;
            if (! (input1 >> a))
            {
                a.s = "ERROR Unmarshalling 'a' parameter in request";
                a.i = -1;
            }
            event->a_value(a);

            TestStruct b1;
            if (! (input1 >> b1))
            {
                b1.s = "ERROR Unmarshalling 'b' parameter in request";
                b1.i = -1;
            }
            event->b1_value(b1);

            // try to unmarshall reply parameters:
            //   - return TestStruct
            //   - inout TestStruct b
            //   - out TestStruct c
            len = pair.reply.data.length();
            buffer = pair.reply.data.get_buffer();
            char_len = len * sizeof(CORBA::Octet) / sizeof(char);
            TAO_InputCDR input2((const char*) buffer, char_len);
            input2.reset_byte_order(ACE_static_cast (int, pair.reply.head.flags));

            TestStruct ret;
            if (! (input2 >> ret))
            {
                ret.s = "ERROR Unmarshalling return parameter in reply";
                ret.i = -1;
            }
            event->ret_value(ret);

            TestStruct b2;
            if (! (input2 >> b2))
            {
                b2.s = "ERROR Unmarshalling 'b' parameter in reply";
                b2.i = -1;
            }
            event->b2_value(b2);

            TestStruct c;
            if (! (input2 >> c))
            {
                c.s = "ERROR Unmarshalling 'c' parameter in reply";
                c.i = -1;
            }
            event->c_value(c);
        }

        m_session_context->push_record_publisher(event.in());

    } catch (const CORBA::SystemException& e) {
        std::cerr  << "Exception sending event: " 
                   << Cdmw::OrbSupport::OrbSupport::exception_to_string(e)
                   << std::endl;
    }
}

void CCM_MyRecorder_impl::send_failure_event(const char* msg)
{
    try {
        RecordedOperationEvent_var event = new RecordedOperationEvent_impl();

        event->recorder_name(m_component_name.in());
        event->operation_name(msg);

        std::cout << "*** CCM_MyRecorder_impl::send_failure_event() - "
                  << m_component_name.in() << " : " << msg << std::endl;

        TestStruct_var foo = new TestStruct();
        foo->s = "foo";
        foo->i = -1;
        
        event->a_value(foo.in());
        event->b1_value(foo.in());
        event->b2_value(foo.in());
        event->c_value(foo.in());
        event->ret_value(foo.in());
        
        m_session_context->push_record_publisher(event.in());
        
    } catch (const CORBA::SystemException& e) {
        std::cerr  << "Exception sending event: " 
                   << Cdmw::OrbSupport::OrbSupport::exception_to_string(e)
                   << std::endl;
    }
}

void CCM_MyRecorder_impl::check_suicide_order(
        const CdmwRecording::RequestReplyPair& pair)
{
    try {
        // try to unmarshall 'in TestStruct a' request parameter
        CORBA::ULong len = pair.request.data.length();
        const CORBA::Octet* buffer = pair.request.data.get_buffer();
        size_t char_len = len * sizeof(CORBA::Octet) / sizeof(char);
        TAO_InputCDR input1((const char*) buffer, char_len);
        input1.reset_byte_order(ACE_static_cast (int, pair.request.head.flags));
    
    
        TestStruct a;
        if (! (input1 >> a)) {
            std::cout << __FILE__ << "(" << __LINE__ << ")" << ": " << m_component_name << ":  unmarshalling failed, received buffer: " << std::endl; 
            for(unsigned int i =0; i < len; i++) {
                printf("%.2x ", buffer[i]);
            }
            std::cout << std::endl; 
            fflush(0);

            // Well... cannot check parameter value !
            // Assume it's not a suicide command...
            return;
        }
        
        if (PARAM_A_S_SUCIDE_VALUE == a.s.in()) {
            // commit suicide !S
            std::cout << "*** CCM_MyRecorder_impl::check_suicide_order() - "
                      << m_component_name 
                      << " : SUICIDE COMMAND RECEIVED !! I kill myself..." 
                      << std::endl;
            std::cout << "<<< CCM_MyRecorder_impl::record() - Exiting at " << get_time() << std::endl;

            fflush(0);
            ::exit(1);
        } else {
//            std::cout << "received: " << a.s.in()  << std::endl;    
        }

        
    } catch (const CORBA::SystemException& e) {
        std::cerr  << "Exception checking 'a' parameter: " 
                   << Cdmw::OrbSupport::OrbSupport::exception_to_string(e)
                   << std::endl;
    }
}

std::string CCM_MyRecorder_impl::get_time()
{
    Cdmw::OsSupport::OS::Timeval time = Cdmw::OsSupport::OS::get_time();
    std::ostringstream ostr;
    ostr << time.seconds << "." << time.microseconds;
    return (ostr.str());
}

};  // namespace TestCcmFtRecorder

