/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

/**
 * @author Raffaella Panella <rpanella@amjsv.it>
 * @author Antonello Cedrone <a.cedrone@vitrociset.it>
 * @author Raffaeele Mele    <rmele@progesi.it>
 */

// Standard Files
#include <string>
#include <fstream>
#include <sstream>
#include <iostream>
#include "time.h"
#include "sys/time.h"
#include "ace/High_Res_Timer.h"

// Cdmw Files
#include <Foundation/common/Options.hpp>
#include <Foundation/common/String.hpp>
#include <Foundation/osthreads/Thread.hpp>
#include <Foundation/ossupport/OS.hpp>
#include <Foundation/orbsupport/CORBA.hpp>
#include <Foundation/orbsupport/OrbSupport.hpp>
#include <Foundation/orbsupport/StrategyList.hpp>
#include <LoadBalancing/lbinit/LBServiceInit.hpp>
#include <Repository/idllib/CdmwNamingAndRepository.stub.hpp>
#include <Repository/repositoryinterface/RepositoryInterface.hpp>
#include <Foundation/orbsupport/CosNaming.stub.hpp>
#include <Foundation/commonsvcs/naming/NamingInterface.hpp>

// Local Files
#include "test_lb_invocation/CORBAManager.hpp"
#include "test_lb_invocation/TestUtils.hpp"
#include "qualif/perftests/testperfcommon/ParseCfg.hpp"
#include "test_lb_invocation/fill_struct.hpp"

#define TEST_EOF  "end_of_test.log"

using namespace std;

using namespace Cdmw;

namespace
{
const int SLEEP_VAL = 10000;
const int DEFAULT = 1024;
const string IOR_FILE_NAME = "server_Q-340.IOR";
const string IOGR_FILE_NAME = "lb_pingable_group.IOGR";
const string CFG_CLIENT_NAME = "--configFile";
const std::string REPLICA_NAME  = "--ReplicaName";
const std::string NAMING_REPOS_URL_OPTION = "--namingRepos-url";
const int SUCCESS = 0;
const int FAILURE = 1;

  static void
  usage(std::ostream & os, const std::string &program_name)
  {
    os << "\nUsage:\n"
    << program_name
    << "Options:\n\n"
    << REPLICA_NAME << "=<NAM1>   string name for a remote Replica. \n"
    << NAMING_REPOS_URL_OPTION << "=<URL>  URL of the CARDAMOM Naming and Repository service.\n"
    << std:: endl;
  }

} // end anonymous namespace

int main( int argc, char* argv[] )
{

    try {
        remove
            ( TEST_EOF );

        OsSupport::OS::sleep( SLEEP_VAL );




        cerr << "-------------------------" << endl
        << "      Start Client" << endl
        << "-------------------------" << endl;

        for ( int n = 0; n < argc; n++ )
            cerr << "argv[" << n << "]: " << argv[ n ] << endl;

        cerr << "-------------------------" << endl;

        cout << "Reading Configuration File..." << endl;

        //read configuration file

        string configFile = OsSupport::OS::get_option_value ( argc, argv, CFG_CLIENT_NAME );

        if ( configFile == "no" ) {
            cerr << "Error: Option '--configFile' is mandatory!\n";
            exit( FAILURE );
        }

        cout << "Parsing Configuration File..." << endl;

        //  client configuration container
        utils::cfgData clientcfg;

        // read configuration file

        if ( utils::parseCfg( configFile, clientcfg ) == FAILURE ) {
            cerr << "Error: when reading " << configFile << endl;
            exit( FAILURE );
        }

        //number of Invocation
        int niter = clientcfg.niter[ 0 ];

        //delay between invocation
        int delay = 0;

        char** ORB_init_argv = 0;

        int ORB_init_argc = argc;

        string str_opt( "" );

        for ( int i = 0; i < argc; i++ )
            str_opt += string( argv[ i ] ) + " ";

        str_opt += string( "--LBInterceptorSide=Client" );

        ORB_init_argc += 1;

        ORB_init_argv = Common::String::to_char_array(
                            Common::String::to_strings( str_opt ) );

        LB::LBServiceInit::Init( ORB_init_argc, ORB_init_argv );

        // Initialise the ORB
        OrbSupport::StrategyList strategyList;

        strategyList.add_OrbThreaded();

        strategyList.add_PoaThreadPerConnection();

        CORBA::ORB_var orb = OrbSupport::OrbSupport::ORB_init( argc, argv, strategyList );

        CdmwLBPerf::PingableInterface_var pingable_group =
            LB::TestUtils::Get_pingable_ref_from_file( orb.in(), IOGR_FILE_NAME.c_str() );

      // Get replica reference from the repository

      std::string namingReposURL = Cdmw::OsSupport::OS::get_option_value (argc, argv,NAMING_REPOS_URL_OPTION);

      if (namingReposURL == "no")
        {
          std::cerr << "naming and repository url undefined" << std::endl;
          usage(std::cerr, argv[0]);
          return FAILURE;
        }

      CdmwNamingAndRepository::Repository_var repository =
        CdmwNamingAndRepository::Repository::_nil();

      CORBA::Object_var obj2 = orb->string_to_object(namingReposURL.c_str());
      if (CORBA::is_nil(obj2.in()))
        {
          std::cerr << "File : " << __FILE__ << " Line : " << __LINE__
          << "could not convert to object NamingAndRepository from string" << std::endl;
          return -1;
        }

      repository = CdmwNamingAndRepository::Repository::_narrow(obj2.in());
      if (CORBA::is_nil(repository.in()))
        {
          std::cerr << "File : " << __FILE__ << " Line : " << __LINE__
          << "could not _narrow NamingAndRepository from URL" << std::endl;
          return -1;
        }

      // ======================================================
      // Initialise the repository interface
      //     "CARDAMOM" has been chosen as the default domain name
      // ======================================================
      Cdmw::NamingAndRepository::RepositoryInterface::init ("CDMW", repository.in());

      // ======================================================
      // Get Naming Interface of PERFTEST name domain
      // ======================================================
      Cdmw::CommonSvcs::Naming::NamingInterface ni =
        Cdmw::NamingAndRepository::RepositoryInterface::get_domain_naming_interface (
          "PERFTEST/HELLO_LB");

      // ===================================================
      // Get the PingableInterface object from repository
      // ===================================================
      std::string replica_name = Cdmw::OsSupport::OS::get_option_value (argc, argv, REPLICA_NAME);
      if (replica_name == "no")
        {
          std::cerr << "Replica url undefined:" << std::endl;
          usage(std::cerr, argv[0]);
          return FAILURE;
        }

      CORBA::Object_var replica_ref = ni.resolve (replica_name);

      // get the PingableInterface proxy
      CdmwLBPerf::PingableInterface_var pingable_object =
        CdmwLBPerf::PingableInterface::_narrow(replica_ref.in());

        // creating Mockup object
        Mockup::fullFPD_var ffpd = new Mockup::fullFPD();
        ffpd->revision = 0;
        ffpd->ssr = 1;
        ffpd->expiration_time = 123456789;
        Mockup::fill_FlightPlanData(ffpd->fpd);
        Mockup::fill_Trajectory(ffpd->traj);

        // data sample container
        vector<ACE_hrtime_t> data( niter );

        vector<ACE_hrtime_t> datalb( niter );

        ACE_High_Res_Timer timer;

        timer.calibrate();

        ACE_hrtime_t measurement;

        for ( size_t i_del = 0; i_del < clientcfg.delay.size(); i_del++ ) {
            // assign current delay
            delay = clientcfg.delay[ i_del ];
            //delay of measure
            timespec t_delay;
            t_delay.tv_sec = delay / 1000;
            t_delay.tv_nsec = ( delay % 1000 ) * 1000000;

            cerr << "client_Q-340>> calling a Load Balanced server with delay "
            << delay
            << " ms.."
            << endl;

            for ( int i = 0; i < niter; i++ ) {
                timer.start();
                pingable_group->fullFPD_ping(ffpd.in());
                timer.stop();
                timer.elapsed_time( measurement );
                datalb[ i ] = measurement;

                if ( delay != 0 ) {
                    // call to nanosleep
                    nanosleep( & t_delay, NULL );
                }
            }

            cerr << "client_Q-340>> calling a single normal server with delay "
            << delay
            << " ms.."
            << endl;

            for ( int i = 0; i < niter; i++ ) {
                timer.start();
                pingable_object->fullFPD_ping(ffpd.in());
                timer.stop();
                timer.elapsed_time( measurement );
                data[ i ] = measurement;

                if ( delay != 0 ) {
                    // call to nanosleep
                    nanosleep( & t_delay, NULL );
                }
            }


            //write file
            ostringstream nameStream, nameStreamLB;

            nameStream << clientcfg.inv_type[ 0 ] << "_" << clientcfg.delay[ i_del ] << "_" << "DELTA_C.dat";

            nameStreamLB << clientcfg.inv_type[ 0 ] << "_" << clientcfg.delay[ i_del ] << "_" << "DELTA_C_LB.dat";

            string filedelta ( clientcfg.fileprefix[ 0 ] + nameStream.str() );

            string filedeltaLB ( clientcfg.fileprefix[ 0 ] + nameStreamLB.str() );

            ofstream os_performance( filedelta.c_str() );

            ofstream os_performance_lb( filedeltaLB.c_str() );

            for ( int i = 0; i < niter; ++i ) {
                os_performance_lb << datalb[ i ] << endl;
                os_performance << data[ i ] << endl;
            }

            os_performance_lb.close();
            os_performance.close();
        }
        // write file for shell script execution control
        ofstream feof ( TEST_EOF );
        feof << "EOF" << endl;
        feof.close();
        cerr << "** test execution successfull terminated: Please STOP the system!" << std::endl;


    }
    catch ( const CORBA::Exception & e ) {
        cerr << __FILE__ << " " << __LINE__ << " " << e._name() << endl;
        return -1;
    }
    catch ( const Exception & e ) {
        cerr << __FILE__ << " " << __LINE__ << " " << e.what() << endl;
        return -1;
    }
    catch ( ... ) {
        cerr << "Unexpected exception" << endl;
        return -1;
    }
}

