/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

/**
 * @author Raffaella Panella <rpanella@amjsv.it>
 * @author Antonello Cedrone <a.cedrone@vitrociset.it>
 * @author Raffaele Mele     <rmele@progesi.it>
 */

// Standard Files
#include <fstream>
#include <iostream>

// Cdmw Files
#include <Foundation/common/Options.hpp>
#include <Foundation/common/String.hpp>
#include <Foundation/orbsupport/CORBA.hpp>
#include <Foundation/orbsupport/RefCountLocalObject.hpp>
#include <Foundation/orbsupport/ExceptionMinorCodes.hpp>
#include <Foundation/orbsupport/OrbSupport.hpp>
#include <Foundation/orbsupport/StrategyList.hpp>
#include <Foundation/osthreads/Thread.hpp>
#include <Foundation/ossupport/OS.hpp>
#include <LoadBalancing/lbinit/LBServiceInit.hpp>
#include <Foundation/common/String.hpp>
#include <idllib/CdmwLBGroupManager.stub.hpp>
#include <Repository/idllib/CdmwNamingAndRepository.stub.hpp>


// Local Files
#include "test_lb_invocation/TestPingable_impl.hpp"

using namespace std;
using namespace Cdmw;

namespace
{

const string REPLICA_NAME_OPT = "--ReplicaName";
const string GM_IOR_OPTION = "--GroupMngRef";
const string NAMING_REPOS_URL_OPTION = "--namingRepos-url";
const int SUCCESS = 0;
const int FAILURE = -1;

} // end anonymous namespace

static void
usage( ostream & os, const string &program_name )
{
    os << "Usage:\n"
    << program_name << " options\n"
    << "\n"
    << "Options:\n"
    << "--help : Display this information.\n"
    << "\n"
    << "-ORBInitRef LBGroupManager = <rep_corbaloc>, the LBGroupManager corbaloc\n"
    << "[" << REPLICA_NAME_OPT << "] = name, the name of the replica and IOR file.\n"
    <<  endl;
}

int main( int argc, char* argv[] )
{
    // help argument

    if ( argc <= 2 ) {
        usage ( cerr, argv[ 0 ] );
        return SUCCESS;
    }

    if ( argc >= 2 ) {
        if ( strcmp( argv[ 1 ], "--help" ) == 0 ) {
            usage ( cerr, argv[ 0 ] );
            return SUCCESS;
        }
    }

    // get Name File to save interface interface
    string replica_name = Cdmw::OsSupport::OS::get_option_value ( argc, argv, REPLICA_NAME_OPT );

    if ( replica_name == "no" ) {
          cerr << "Error: The replica name it's undefined" << std::endl;
          usage(cerr, argv[0]);
          return FAILURE;
    }
    
    // file name to store the IOR
    string file_name = replica_name + ".IOR";

    try {

        string namingReposURL =
            Cdmw::OsSupport::OS::get_option_value ( argc, argv, NAMING_REPOS_URL_OPTION );

        if ( namingReposURL == "no" ) {
            cerr << "Error: naming and repository url undefined - " 
                      << " use " 
                      << NAMING_REPOS_URL_OPTION 
                      << "option!" 
                      << endl;
            usage( cerr, argv[ 0 ] );
            return SUCCESS;
        }

        //  add Server Interceptor 
        char** ORB_init_argv = 0;

        int ORB_init_argc = argc;

        string str_opt( "" );

        for ( int i = 0; i < argc; i++ )
            str_opt += string( argv[ i ] ) + " ";

        str_opt += string( "--LBInterceptorSide=Server" );

        ORB_init_argc += 1;

        ORB_init_argv = 
            Common::String::to_char_array( Common::String::to_strings( str_opt ) );

        LB::LBServiceInit::Init( ORB_init_argc, ORB_init_argv );

        // Initialises the ORB
        Cdmw::OrbSupport::StrategyList strategyList;

        strategyList.add_OrbThreaded();

        strategyList.add_PoaThreadPerConnection();
        
        strategyList.add_multicast();

        CORBA::ORB_var orb
            = Cdmw::OrbSupport::OrbSupport::ORB_init( argc, argv, strategyList );

        // Get the root POA
        CORBA::Object_var obj = orb->resolve_initial_references( "RootPOA" );

        PortableServer::POA_var rootPOA = PortableServer::POA::_narrow( obj.in() );

        PortableServer::POAManager_var poaMgr = rootPOA->the_POAManager();

        poaMgr->activate();

        Cdmw::PingableInterface_impl * pingable_interface
            = new Cdmw::PingableInterface_impl();

        CdmwLBPerf::PingableInterface_var pingable_interface_ref
            = pingable_interface->_this();

        // export the object reference to a file
        CORBA::String_var ref_string = orb->object_to_string( pingable_interface_ref.in() );

        ofstream os( file_name.c_str() );

        os << ref_string.in();

        os.close();

        // ===================================================
        // Get the naming and repository
        //     from the input URL
        // ===================================================
        CdmwNamingAndRepository::Repository_var repository =
            CdmwNamingAndRepository::Repository::_nil();

        CORBA::Object_var obj2 = orb->string_to_object( namingReposURL.c_str() );

        repository = CdmwNamingAndRepository::Repository::_narrow( obj2.in() );

        CdmwNamingAndRepository::NameDomain_var helloDomain =
            repository->resolve_name_domain ( "PERFTEST/HELLO_LB" );


        // ========================================================
        // Register the HelloInterface object in the HELLO_LB name domain
        // ========================================================
        // reserve the name "HelloInterface" in the domain
        CdmwNamingAndRepository::NameDomain::RegistrationId_var regId =
            helloDomain->new_name ( replica_name.c_str() );

        // register the object
        helloDomain->register_object ( regId.in(), pingable_interface_ref.in() );

        cerr << "server_Q-340>> LB Server (C++) started and registered in the Repository " << endl;

        orb->run();
    }
    catch ( const CORBA::SystemException & e ) {
        cerr << __FILE__ << " " << __LINE__ << " " << e._name() << endl;
        return FAILURE;
    }
    catch ( const CORBA::Exception & e ) {
        cerr << __FILE__ << " " << __LINE__ << " " << e._name() << endl;
        return FAILURE;
    }
    catch ( ... ) {
        cerr << "Uncaught exception" << endl;
        return FAILURE;
    }

    return SUCCESS;
}

