/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

#include "test_orb_invocation/ClientThread.hpp"

using namespace utils;

namespace PerfThread {

// ======================= Invocation Thread =======================

ClientThread::ClientThread(CORBA::ORB_ptr orb, utils::cfgData& cfg)
throw( CORBA::SystemException ) : m_orb ( CORBA::ORB::_duplicate( orb ) ),
        m_cfg(cfg),
        m_stopped(false)
{
    // =========================================
    // retrieve Tester object from Repository
    // =========================================
    // get NamingInterface to test_perf
    Cdmw::CommonSvcs::Naming::NamingInterface marshallingNamingInterface =
        Cdmw::NamingAndRepository::RepositoryInterface::get_domain_naming_interface ( "perf_test" );

    // Retrieve TestPing from NamingInterface
    std::string registration_name = "TestPing";

    try {
        CORBA::Object_var obj = marshallingNamingInterface.resolve ( registration_name.c_str() );
        m_testping = Invocation::TestPing::_narrow( obj.in() );
    }
    catch ( const CosNaming::NamingContext::NotFound & e ) {
        std::cerr << "Resolve " << registration_name
        << " => NamingContext::NotFound raised!" << std::endl;
    }
    catch ( const CosNaming::NamingContext::CannotProceed & e ) {
        std::cerr << "Resolve " << registration_name
        << " => NamingContext::CannotProceed raised!" << std::endl;
    }
    catch ( const CosNaming::NamingContext::InvalidName & e ) {
        std::cerr << "Resolve " << registration_name
        << " => NamingContext::InvalidName raised!" << std::endl;
    }
    catch ( const CORBA::SystemException & e ) {
        std::cerr << "Resolve " << registration_name
        << " => CORBA::SystemException raised!" << std::endl;
    }

    // ===================================================
    // Instantiate time reading object
    // ===================================================
    std::string measure_str = cfg.measure_type[ 0 ];

    if ( measure_str.compare( MEASURE_TH ) == 0 ) {
        m_measure = new ClockCpuTime;
    }
    else if ( measure_str.compare( MEASURE_TK ) == 0 ) {
        m_measure = new ClockCpuTick;
    }
    else // using default
    {
        m_measure = new ClockHighResTime;
    }
    m_niter = m_cfg.niter[0];
    m_fileprefix = m_cfg.fileprefix[0];
} // ClientThread::ClientThread

void
ClientThread::run() throw()
{
    long double tdelta[ m_niter ];

    for ( size_t i_inv = 0; i_inv < m_cfg.inv_type.size(); i_inv++ ) {
        //result of time measurement
        long double time1, time2;
        m_invocation_type = m_cfg.inv_type[ i_inv ];

        for ( size_t i_del = 0; i_del < m_cfg.delay.size(); i_del++ ) {
            // assign current delay
            m_delay = m_cfg.delay[ i_del ];
            //delay of measure
            timespec t_delay;
            t_delay.tv_sec = m_delay / 1000;
            t_delay.tv_nsec = ( m_delay % 1000 ) * 1000000;

            //test CR: call and return

            if ( m_invocation_type.compare( CALL_RETURN ) == 0 ) { // CALL_RETURN

                //start of Invocation test

                for ( int i = 0; i < m_niter; i++ ) {

                    //delay of measure

                    if ( m_delay != 0 ) {
                        // call to nanosleep
                        nanosleep( & t_delay, NULL );
                    }

                    time1 = m_measure->getTime();
                    m_testping->invocation_ping();
                    time2 = m_measure->getTime();
                    tdelta[ i ] = time2 - time1;
                } //end for
            } //end case Call and Return

            else if ( m_invocation_type.compare( ONE_WAY ) == 0 ) { // ONE_WAY

                static bool is_initialised = false;
                if (!is_initialised) {
                    is_initialised = true;
                    CORBA::Object_var obj =
                        m_orb->resolve_initial_references ( "ORBPolicyManager" );
                    CORBA::PolicyManager_var policy_manager =
                        CORBA::PolicyManager::_narrow ( obj.in () );

                    obj = m_orb->resolve_initial_references ( "PolicyCurrent" );
                    CORBA::PolicyCurrent_var policy_current =
                        CORBA::PolicyCurrent::_narrow ( obj.in() );

                    // Default SyncScope value (ONE_WAY).
                    static Messaging::SyncScope sync_scope = Messaging::SYNC_NONE;

                    // Set up the sync scope any.
                    CORBA::Any sync_scope_any;

                    sync_scope_any <<= sync_scope;

                    // Set the length of the policy list.
                    CORBA::PolicyList sync_scope_policy_list ( 1 );

                    sync_scope_policy_list.length ( 1 );

                    // Set up the sync scope policy.
                    sync_scope_policy_list[ 0 ] =
                        m_orb->create_policy ( Messaging::SYNC_SCOPE_POLICY_TYPE,
                                               sync_scope_any );

                    // Set the sync scope policy at the object level.
                    obj = m_testping->_set_policy_overrides ( sync_scope_policy_list,
                            CORBA::ADD_OVERRIDE );
                    // Get the new object reference with the updated policy.
                    m_testping = Invocation::TestPing::_narrow ( obj.in () );
                }

                //start of Invocation test

                for ( int i = 0; i < m_niter; i++ ) {
                    //delay of measure

                    if ( m_delay != 0 ) {
                        // call to nanosleep
                        nanosleep( & t_delay, NULL );
                    }

                    time1 = m_measure->getTime();
                    m_testping->ow_invocation_ping();
                    time2 = m_measure->getTime();
                    //time read in micro sec.
                    tdelta[ i ] = time2 - time1;
                }
            } // end ONE_WAY
            else {
                std::cerr << "** FATAL: configuration file parsing error ** " << std::endl;
                exit(-1);
            }

            //write data file
            std::ostringstream nameStream;
            nameStream << m_cfg.inv_type[i_inv] << "_" << m_cfg.delay[i_del]  << "_" << "DELTA_C.dat";
            std::string filedeltaname ( m_fileprefix + nameStream.str() );

            ofstream fd ( filedeltaname.c_str() );

            fd.precision( 6 );

            for ( int i = 0; i < m_niter; i++ ) {
                fd << tdelta[ i ] << std::endl;
            }

           fd.close();
        } // end for i_inv
    } // end for i_del
    std::cerr << "** test execution successfull terminated: Please STOP the system!" << std::endl;
} // ClientThread::run()

} // namespace PerfThread
