/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#include <exception>
#include <iostream>
#include <sstream>


#include "qualif/perftests/testperfcommon/HighResTimer.hpp"
#include "qualif/perftests/testperfcommon/HighResTime.hpp"
#include "test_repository_bind/BindReaderThread.hpp"
#include "test_repository_bind/TestHello_impl.hpp"
#include "test_repository_bind/TestRepositoryBinder.hpp"
#include <Foundation/commonsvcs/naming/NamingInterface.hpp>
#include "Repository/repositoryinterface/RepositoryInterface.hpp"
#include "Repository/idllib/CdmwNamingAndRepository.stub.hpp"
#include "SystemMngt/platforminterface/PlatformInterface.hpp"
#include "SystemMngt/platforminterface/ServiceNames.hpp"


namespace Cdmw {

CPPUNIT_TEST_SUITE_REGISTRATION( TestRepositoryBinder );
// constructor
TestRepositoryBinder::TestRepositoryBinder()
:   m_orb(CORBA::ORB::_duplicate( Cdmw::TestUtils::CORBATestManager::instance()->get_ORB())),
    m_rootPOA(PortableServer::POA::_duplicate( Cdmw::TestUtils::CORBATestManager::instance()->get_POA()))    
{
   m_bind_timeout = DEFAULT_BIND_TIMEOUT ;
       
   std::string str_timeout =
      Cdmw::TestUtils::CORBATestManager::instance()->get_property ("bind_timeout");

   if(std::strcmp(str_timeout.c_str(), ""))
   {
       try 
       {
           std::stringstream sm ;
           sm << str_timeout ;
           sm >> m_bind_timeout ;
       }
       catch(...) 
       {
           std::cout << "error in --bind-timeout=\'"<< str_timeout 
                     <<"\', using default value" << std::endl ; 
           m_bind_timeout = DEFAULT_BIND_TIMEOUT ;
       }
   }
   else std::cout << "--bind-timeout is not specified, using default value" << std::endl ; 

   std::cout <<  "bind timeout: " << m_bind_timeout << std::endl;
}

//destructor
TestRepositoryBinder::~TestRepositoryBinder()
{
}


// do_tests
void TestRepositoryBinder::do_tests()
{
    try
    {
        std::cout << "Init Repository" << std::endl;

        CORBA::Object_var obj =
        PlatformMngt::PlatformInterface::Get_service(PlatformMngt::ServiceNames::NAMING_AND_REPOSITORY_SERVICE);

        CdmwNamingAndRepository::Repository_var repository;
        repository = CdmwNamingAndRepository::Repository::_narrow(obj.in());

        CosNaming::NamingContextExt_var defaultRootCtx;
       
        TEST_INFO("Get the default root context");

        try
        {
            CosNaming::NamingContext_var obj =
            repository->resolve_root_context(CdmwNamingAndRepository::DEFAULT_ROOT_CONTEXT);

            defaultRootCtx = CosNaming::NamingContextExt::_narrow(obj.in());
            CPPUNIT_ASSERT(true);
        }
        catch(CORBA::Exception &e)
        {
            std::cout << e._name();
            CPPUNIT_ASSERT(false);
        }
        catch(...)
        {
            std::cout << "Cannot get Root Context" << std::endl;
            CPPUNIT_ASSERT(false);
            return;
        }

        Cdmw::HelloInterface1_var helloInterface;

        //Create lock mutex and Condition, to synchronize threads
        Cdmw::OsSupport::Mutex bindMutex;
        Cdmw::OsSupport::Condition bindAvailable(bindMutex);

        //Create and start the listener thread, which will be waiting for the bind availability
        TEST_INFO("Starting Bind Reader thread ...");
        BindReaderThread bindReader(&bindAvailable);
        bindReader.start();
        TEST_INFO("Bind Reader thread started.");

        //Setup and bind object reference
        Cdmw::HelloInterface1_impl* impl = new Cdmw::HelloInterface1_impl(m_orb.in(), m_rootPOA.in(), BIND_OBJECT_ID);
        PortableServer::ServantBase_var implHolder(impl); //prevents memory leaks

        helloInterface = impl->_this();
        CosNaming::Name_var name = new CosNaming::Name;
        name->length(1);
        CosNaming::NameComponent_var nc = new CosNaming::NameComponent;
        nc->id = CORBA::string_dup(BIND_OBJECT_ID);
        nc->kind = CORBA::string_dup("object");
        name->operator[](0) = nc.in();

        long double delta[N_ITERATION];
        ctools::time::HighResTimer perfTimer;

        bindMutex.lock();

        for(int i = 0; i < N_ITERATION; ++i)
        {
            perfTimer.reset();

            perfTimer.start();

            try
            {
                defaultRootCtx->bind(name.in(), helloInterface.in());
            }
            catch(CORBA::Exception &e)
            {
                std::cout << e._name();
                CPPUNIT_ASSERT(false);
            }

            //Notify bindReader's thread that bind was done.
            bindReader.set_binded();

            //wait for the bind reader's notification or timeout elapse
            Cdmw::OsSupport::Condition::WaitReturn result = bindAvailable.wait(m_bind_timeout);
            
            perfTimer.stop();

            try
            {
                defaultRootCtx->unbind(name.in());
            }
            catch(CORBA::Exception &e)
            {
                std::cout << e._name();
                CPPUNIT_ASSERT(false);
            }

            //Check whether request timed out or confirmed
            if(result == Cdmw::OsSupport::Condition::TIMEDOUT)
            {
                std::cerr << "Bind " << i << " timedout." << std::endl;
            }

            //Read the elapsed time
            delta[i] = perfTimer.getElapsedTime().toMicrosec();

            
        } //for (i < N_ITERATION)

        bindMutex.unlock();

        //terminate bind reader thread and wait for it's exit
        bindReader.terminate();
        bindReader.join();

        //write data file
        std::ofstream fd ( "Q-429_DELTA_C.dat" );

        for ( int i = 0; i < N_ITERATION; ++i ) {
            fd << delta[i] << std::endl;
        }

        fd.close();

        CPPUNIT_ASSERT(true);
    }
    
    catch(const CORBA::Exception& e )
    {
        std::cerr << e._name() << std::endl;
        CPPUNIT_ASSERT(false);
    }
    catch( const Cdmw::Exception& e )
    {
        std::cerr << e.what() << std::endl;
        CPPUNIT_ASSERT(false);
    }

}

}; // namespace Cdmw

