/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

#include "test_time_invocation/ClientProcessControl.hpp"
#include "Foundation/common/Assert.hpp"
#include "Time/clockservice/Util.hpp"

namespace TimeInvocation
{

ClientProcessControl::ClientProcessControl ( CORBA::ORB_ptr orb, utils::cfgData& cfg )
throw( CORBA::SystemException )
        : m_orb ( CORBA::ORB::_duplicate( orb ) ),
        m_applicationName ( "" ),
        m_processName ( "" ),
        m_clock( 0 ),
        m_cfg( cfg )
{
    // ===================================================
    // Instantiate time reading object
    // ===================================================
    std::string measure_str = cfg.measure_type[ 0 ];

    if ( measure_str.compare( MEASURE_TH ) == 0 ) {
        m_measure = new ClockCpuTime;
    }
    else if ( measure_str.compare( MEASURE_TK ) == 0 ) {
        m_measure = new ClockCpuTick;
    }
    else // using default
    {
        m_measure = new ClockHighResTime;
    }

    m_niter = m_cfg.niter[ 0 ];
    m_fileprefix = m_cfg.fileprefix[ 0 ];

}

ClientProcessControl::~ClientProcessControl()
throw()
{}



// process initialisation called by platformmngt
void ClientProcessControl::on_initialise ( const CdmwPlatformMngtBase::StartupKind& startup_kind )
throw( CORBA::SystemException )
{


    // ===================================================
    // Get the process name and application name
    // ===================================================
    m_processName = Cdmw::CdmwInit::InitUtils::get_cdmw_process_name();

    m_applicationName = Cdmw::CdmwInit::InitUtils::get_cdmw_application_name();

    // =================================================================
    // example of using the PlatformInterface for notifying a message
    // =================================================================
    Cdmw::PlatformMngt::PlatformInterface::Notify_message( CdmwPlatformMngtBase::INF,
            m_processName.c_str(),
            ">>>>>>>>>>>>>> Initialisation requested by supervision" );

    // ====================
    // retrieve LocalClock
    // ====================
    CORBA::Object_var obj =
        m_orb->resolve_initial_references( "LocalClock" );

    if ( CORBA::is_nil( obj.in() ) ) {
        std::cerr << "Client:>> Unable to resolve ClockService initial reference"
        << std::endl;
        exit( 1 );
    }

    m_clock = CosClockService::Clock::_narrow( obj.in() );

    if ( CORBA::is_nil( m_clock.in() ) ) {
        std::cerr << "Client:>> Unable to resolve LocalClock"
        << std::endl;
        exit( 1 );
    }

} // on_initialise()

///////////////////////////////////////////////////////////////////////////////

// process to run called by platformmngt
void ClientProcessControl::on_run()
throw( CdmwPlatformMngt::ProcessDelegate::NotReadyToRun, CORBA::SystemException )
{

    // =================================================================
    // example of using the PlatformInterface for notifying a message
    // =================================================================
    Cdmw::PlatformMngt::PlatformInterface::Notify_message( CdmwPlatformMngtBase::INF,
            m_processName.c_str(),
            ">>>>>>>>>>>>>> Run requested by supervision" );

    //-----------------------------------------------------------------------------
    // TimeBase::TimeT currentTime = 0, prevTime;
    //vectors containing correct sample + skipped sample
    long double t_res[ m_niter ];
    long double t_delta[ m_niter ];
    long double tmp_time = 0;
    timespec t_delay;

    // calculate invocation time of one call to the Time Service
    // Resolution of delta is 1 usec.

    for ( size_t i_del = 0; i_del < m_cfg.delay.size(); i_del++ ) {

        m_delay = m_cfg.delay [ i_del ];

        if ( m_delay ) {
            t_delay.tv_sec = m_delay / 1000;
            t_delay.tv_nsec = ( m_delay % 1000 ) * 1000000;
        }

        for ( size_t i_inv = 0; i_inv < m_cfg.inv_type.size(); i_inv++ ) {

            for ( int i = 0; i < m_niter; ++i ) {
                t_res[ i ] = m_measure->getTime();
                m_clock->current_time();
                tmp_time = m_measure->getTime();

                t_delta[ i ] = tmp_time - t_res[ i ];

                if ( m_delay )
                    nanosleep( & t_delay, NULL );
            }

            //-----------------------------------------------------------------------------
            // write file with delta of time invocation
            std::ostringstream nameStream;

            nameStream << m_cfg.inv_type[ i_inv ] << "_" << m_delay << "_" << "DELTA_C.dat";

            std::string filedeltaname ( m_fileprefix + nameStream.str() );

            ofstream fd ( filedeltaname.c_str() );

            fd.precision( 6 );

            for ( int i = 0; i < m_niter; i++ ) {
                fd << t_delta[ i ] << std::endl;
            }

            fd.close();
        } // i_inv
    } // i_del

    //-----------------------------------------------------------------------------

    // write file for shell script execution control
    ofstream feof ( TEST_EOF );

    feof << "EOF" << std::endl;

    feof.close();

    std::cerr << "** test execution successfull terminated: Please STOP the system!" << std::endl;
} // on_run()

///////////////////////////////////////////////////////////////////////////////

// process to stop called by platformmngt
void ClientProcessControl::on_stop()
throw( CORBA::SystemException )
{

    // =================================================================
    // example of using the PlatformInterface for notifying a message
    // =================================================================
    Cdmw::PlatformMngt::PlatformInterface::Notify_message( CdmwPlatformMngtBase::INF,
            m_processName.c_str(),
            ">>>>>>>>>>>>>> Stop requested by supervision" );
    m_orb->shutdown( true );
} // on_stop()

}

; // End namespace TimeInvocation
