/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

#include <cstdlib>
#include <string>
#include <sstream>
#include <iostream>

#include "Foundation/common/System.hpp"
#include "Foundation/common/Assert.hpp"
#include "Foundation/common/Locations.hpp"
#include "Foundation/ossupport/OS.hpp"
#include "Foundation/orbsupport/OrbSupport.hpp"
#include "Foundation/orbsupport/StrategyList.hpp"

#include "ConfAndPlug/cdmwinit/CdmwInterface.hpp"
#include "ConfAndPlug/cdmwinit/InitUtils.hpp"
#include "Repository/repositoryinterface/RepositoryInterface.hpp"
#include "Repository/idllib/CdmwNamingAndRepository.stub.hpp"
#include "Time/clockservice/CdmwCosClock.hpp"

#include "test_time_invocation/ClientProcessControl.hpp"

namespace
{
const int SUCCESS = 0;
const int FAILURE = 1;

const int POA_THREAD_POOL_SIZE = 5;
const std::string CFG_CLIENT_NAME = "--configFile";

} // End anonymous namespace

int main(int argc, char* argv[])
{
    int status = SUCCESS;
        
        std::remove( TEST_EOF );

    std::cerr   << "-------------------------" << std::endl
            << "      Start Client" << std::endl
            << "-------------------------" << std::endl;
    for (int n=0; n < argc; n++)
        std::cerr << "argv[" << n <<"]: " << argv[n] << std::endl;
    std::cerr   << "-------------------------" << std::endl;

    std::string configFile = Cdmw::OsSupport::OS::get_option_value ( argc, argv, CFG_CLIENT_NAME );

    if ( configFile == "no" ) {
        std::cerr << "Error: Option '--configFile' is mandatory!\n";
        exit( FAILURE );
    }

    std::cout << "Parsing Configuration File..." << std::endl;

    //  client configuration container
    utils::cfgData clientcfg;

    //read configuration file

    if ( utils::parseCfg( configFile, clientcfg ) == FAILURE ) {
        std::cerr << "Error: when reading " << configFile << std::endl;
        exit( FAILURE );
    }

    CORBA::ORB_var orb;                        // orb reference

    try {
        // ===================================================
        // Initialize the ORB
        // ===================================================
        Cdmw::OrbSupport::StrategyList orb_strategies;
        orb_strategies.add_OrbThreaded();
        orb_strategies.add_PoaThreadPool(POA_THREAD_POOL_SIZE);

        orb = Cdmw::OrbSupport::OrbSupport::ORB_init(argc, argv, orb_strategies);

        // ===================================================
        // Initialise Clock Service
        // ===================================================
        Cdmw::clock::CosClock::init(orb.in(),argc,argv);

        // ===================================================
        // create the process control for platform management
        // ===================================================
        Cdmw::CdmwInit::ProcessControl_var process_ctrl =
                new TimeInvocation::ClientProcessControl( orb.in(), clientcfg);

        // ===================================================
        // Call generated CdmwInit
        // ===================================================
        Cdmw::CdmwInit::CDMW_init(orb.in(), argc, argv, process_ctrl.in());


        // ===================================================
        // wait for CORBA message
        // ===================================================
        orb->run();
    }
    catch ( const Cdmw::OrbSupport::CORBASystemExceptionWrapper & ex ) {
        std::cerr << ex.what() << std::endl;
        status = FAILURE;
    }
    catch ( const Cdmw::Exception & ex ) {
        std::cerr << ex.what() << std::endl;
        status = FAILURE;
    }
    catch ( const CORBA::Exception & ex ) {
        std::cerr << ex << std::endl;
        status = FAILURE;
    }
    catch ( ... ) {
        std::cerr << "unexpected exception" << std::endl;
        status = FAILURE;
    }


    // ========================================================
    // program stopping
    // ========================================================

    // ===================================================
    // Call generated Cdmw cleanup
    // ===================================================
    Cdmw::CdmwInit::CDMW_cleanup(orb.in());

    // ===================================================
    // stop Clock Service
    // ===================================================
    Cdmw::clock::CosClock::close();

    // ===================================================
    // Call ORB cleanup
    // ===================================================
    Cdmw::OrbSupport::OrbSupport::ORB_cleanup(orb.in());

    // ===================================================
    // destroy orb
    // ===================================================

    if ( !CORBA::is_nil( orb.in() ) ) {
        try {
            orb -> destroy();
        }
        catch ( const CORBA::Exception & ex ) {
            std::cerr << ex << std::endl;
            status = FAILURE;
        }
    }

    return status;
}
