/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

#ifndef CDMW_TOKENIZER_HPP
#define CDMW_TOKENIZER_HPP

#include <string>
#include <vector>
#include <iterator>
#include <cstdlib>
#include <sstream>

namespace
{
// default token delimiters
const std::string DEF_DELIM = " ,";
}

namespace utils
{


/**
 * @brief print vector to standard output.
 * @param tokens vector to print.
 * @param T template type of vector.
 * @param showTot if true print the total number of tokens.
 */

template <class T>
void print_token( const std::vector<T>& tok, const bool showTot = false )
{
    if ( showTot )
        std::cout << "Total tokens: " << tok.size() << std::endl;

    copy( tok.begin(), tok.end(), std::ostream_iterator<T>( std::cout, ", " ) );

    std::cout << std::endl;
}

/**
 * @brief Separate strings into single token of type T.
 * @param str input string (or 'char*') in input that will be tokenized.
 * @param tokens output vector with one token for element.
 * @param delimiters list of token delimiters (e.g. "," or ";,.").
 * @return none. vector with one separated tokens.
 */
template <class T>
void Tokenizer( const std::string& str,
                std::vector<T>& tokens,
                const std::string& delimiters = DEF_DELIM)
{
    // Skip delimiters at beginning of string.
    std::string::size_type lastPos = str.find_first_not_of( delimiters, 0 );
    // Find first "non-delimiter" character.
    std::string::size_type pos = str.find_first_of( delimiters, lastPos );

    while ( std::string::npos != pos || std::string::npos != lastPos ) {
        // store extracted token
        T tokValue;
        // Found a token, and add it to a stream
        std::istringstream dataStream(str.substr( lastPos, pos - lastPos ));
        // Extract and convert token from the stream
        dataStream >> tokValue;
        // Found a token, and add it to the vector.
        tokens.push_back( tokValue );
        // Skip delimiters.  Note the "not_of"
        lastPos = str.find_first_not_of( delimiters, pos );
        // Find next "non-delimiter".
        pos = str.find_first_of( delimiters, lastPos );
    }
}

} // namespace utils

#endif // CDMW_TOKENIZER_HPP
