/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO 
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2009. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

#include <iostream>
#include <Foundation/common/Assert.hpp>
#include <Foundation/ossupport/OS.hpp>
#include <Foundation/orbsupport/OrbSupport.hpp>
#include <Foundation/logging/CdmwLogging.stub.hpp>
#include <Foundation/logging/Logger.hpp>
#include <TraceAndPerf/recordingidl/CdmwRecordingData.stub.hpp>
#include <Foundation/osthreads/MutexGuard.hpp>
#include <Foundation/testutils/CORBATestManager.hpp>
#include "recording_operations/CdmwRecordingRecorder_impl.hpp"

// static variable
bool CdmwRecording::Recorder_impl::M_test_ok = false;
Cdmw::TestUtils::ThreadedTestRunner* CdmwRecording::Recorder_impl::M_test_thread = NULL;
bool CdmwRecording::Recorder_impl::M_filter_activated = false;

namespace {

  // Max time (in seconds) spends to wait the recording and check
  // the recorded data.
  const int MAX_TIME = 40;
  
  // Maximum of operations to check
  const int MAX_RECORDING = 2;
  int nb_recorded_data_to_check = 0;

  class test_recorder;
    
  CPPUNIT_TEST_SUITE_NAMED_REGISTRATION(test_recorder, "test_recorder");

  class test_recorder : public CppUnit::TestFixture
  {
  public:
    test_recorder()
    {
    };


    ~test_recorder() {};
  
    void do_tests()
    {
      try{
        TEST_INFO("Recorder: do_tests()");

        // Check usefull for Q-414
        if (CdmwRecording::Recorder_impl::M_filter_activated == true ) {
          if(::CdmwRecording::Recorder_impl::M_test_ok == false) {
            TEST_INFO("The filter on Recording daemon was successfully activated. No requests were recorded.");
          } else {
            CPPUNIT_ASSERT(false);
          }
        } else { // check usefull for Q-411
          if(::CdmwRecording::Recorder_impl::M_test_ok == true) {
            TEST_INFO("The data were recorded successfully.");
          } else {
            CPPUNIT_ASSERT(false);
          }
        }

        // Test successful
        CPPUNIT_ASSERT( true );

        TEST_INFO("Recorder: ending do_tests()");

      }catch(const CORBA::Exception& e ) {
        std::cerr << e._name() << std::endl;
        CPPUNIT_ASSERT(false);
      }
      catch( const Cdmw::Exception& e ) {
        std::cerr << e.what() << std::endl;
        CPPUNIT_ASSERT(false);
      }
    }
    CPPUNIT_TEST_SUITE( test_recorder );
    CPPUNIT_TEST( do_tests );
    CPPUNIT_TEST_SUITE_END();
  };
}

namespace CdmwRecording
{

Recorder_impl::Recorder_impl(bool filter_activated)
{
  CdmwRecording::Recorder_impl::M_filter_activated = filter_activated;
}

Recorder_impl::~Recorder_impl()
throw()
{
}

void Recorder_impl::record(const ::CdmwRecording::GroupHeader& header,
                           const GIOP_Buffers& data)
   throw(CORBA::SystemException)
{
    CDMW_MUTEX_GUARD(m_mutex);
    TEST_INFO("" << header.num_pairs << " new Records received from " <<
              header.hostname << " at " << header.base_time);

    // Use TAO_InputCDR for unmarshalling
    CORBA::ULong len = data.length();
    const CORBA::Octet* buffer = data.get_buffer();
    size_t char_len = len * sizeof(CORBA::Octet) / sizeof(char);
    TAO_InputCDR input((const char*) buffer, char_len);
    input.reset_byte_order(ACE_static_cast (int, header.flags));
    
    // Unmarshall while TAO_InputCDR is not empty
    CdmwRecording::RequestReplyPair pair;
    int i=0;

    while (input.length() > 0) {
        
        // number of octets still to read
        size_t left = input.length();

        // Unmarshall a RequestReplyPair
        if (input >> pair) {
            TEST_INFO("  - " << (++i) << ": \"" <<
                      pair.request.operation <<
                      "\" from " <<
                      pair.request.server_id);

            if (strcmp(pair.request.operation.in(), "test_exception") == 0 ||
                strcmp(pair.request.operation.in(), "test_string") == 0 
                
                )
              {
                ::nb_recorded_data_to_check++;
              }
            else {
              // If the filter on Recording daemon is activated then
              // No other recording is expected!
              if (CdmwRecording::Recorder_impl::M_filter_activated == true) {
                TEST_INFO("---- RECORDER: Test failed. Unexpected operation was recorded!!!");
                // Wait a timeout exception in do_tests()
                ::nb_recorded_data_to_check = ::MAX_RECORDING + 1;

                // Allow to display a right CPPUnit summary
                CdmwRecording::Recorder_impl::M_filter_activated = false;
              }

            }

            // save copy of RequestReplyPair in list
            m_record_list.push_back(new CdmwRecording::RequestReplyPair(pair));
  
        } else {
            // number of read octets
            size_t nb_read = left-input.length();
            TEST_INFO("UNMARSHALL ERROR (only " << nb_read <<
                      " octets read)");
            ACE_HEX_DUMP ((LM_INFO,
                           (const char *) input.rd_ptr() - nb_read,
                           nb_read,
                           ACE_TEXT ("read buffer")));
            TEST_INFO("- Content of pair.request.head:");
            ACE_HEX_DUMP ((LM_INFO,
                           (const char *) &(pair.request.head),
                           12,
                           ACE_TEXT ("pair.request.head")));
            TEST_INFO("- Content of pair.reply.head:");
            ACE_HEX_DUMP ((LM_INFO,
                           (const char *) &(pair.reply.head),
                           12,
                           ACE_TEXT ("pair.reply.head")));
            // It's useless to try to continue unmarshalling... break loop
            break;
        }
    }

    if (::nb_recorded_data_to_check == ::MAX_RECORDING) {
      std::cout << "starting of checking of recorded data..." << std::endl;
      check_recorded_data();  
    }
    
}

void 
Recorder_impl::check_recorded_data()
{
  RecordList::iterator it;
  for (it = m_record_list.begin();
       it != m_record_list.end();
       it++)
    {
      ::CdmwRecording::Recorder_impl::M_test_ok = true;

      if (strcmp((*it)->request.operation.in(), "test_string") == 0) {
        TEST_INFO("Check of the test_string operation");

        ::nb_recorded_data_to_check--;

        // try to unmarshall parameters as a string
        CORBA::ULong len = (*it)->request.data.length();
        const CORBA::Octet* buffer = (*it)->request.data.get_buffer();
        size_t char_len = len * sizeof(CORBA::Octet) / sizeof(char);

        // TAO_InputCDR: A CDR stream for reading, i.e. for demarshalling.
        TAO_InputCDR input((const char*) buffer, char_len);
        input.reset_byte_order(ACE_static_cast (int, (*it)->request.head.flags));

        CORBA::String_var str;

        // IN paramater should be recovered in the request
        if (input >> str.inout()) 
          {
            // unmarshall OK. Test if it's expected message
            if (strcmp(str.in(), "IN") == 0) {
              TEST_INFO("---- RECORDER: the input parameter was correctly recorded in the request");
            }
          }
        else
          {
            ::CdmwRecording::Recorder_impl::M_test_ok = false;
            TEST_INFO("record for \"test_string\" found, but parameter"
                      << " is not a string!");
          }

        //INOUT parameter should be recovered in the request
        if (input >> str.inout()) 
          {
            // unmarshall OK. Test if it's expected message
            if (strcmp(str.in(), "IN") == 0) {
              TEST_INFO("---- RECORDER: the inout parameter was correctly recorded in the request");
            }
          }
        else
          { 
            ::CdmwRecording::Recorder_impl::M_test_ok = false;
            TEST_INFO("record for \"test_string\" found, but parameter"
                      << " is not a string!");
          }

        if(!(*it)->reply.status == 0) {
          ::CdmwRecording::Recorder_impl::M_test_ok = false;
          TEST_INFO("reply status is not NO_EXCEPTION, but equals to "
                    << (*it)->reply.status);
        }

        //No other parameters should be recovered in the request
        if (input >> str.inout()) 
          {
            ::CdmwRecording::Recorder_impl::M_test_ok = false;
            TEST_INFO("---- RECORDER: an other recorded data in the request"
                        << "\"" << str.in() << "\"");
          }
        else
          { 
            // No more data are waited. This case is expected
            TEST_INFO("record for \"test_string\" found, but parameter"
                      << " is not a string! This is normal, the test is OK");
          }

        if(!(*it)->reply.status == 0) {
          ::CdmwRecording::Recorder_impl::M_test_ok = false;
          TEST_INFO("reply status is not NO_EXCEPTION, but equals to "
                    << (*it)->reply.status);
        }

        /**
         * try to unmarshall inout, out parameters and return value
         */
        len = (*it)->reply.data.length();
        const CORBA::Octet* buffer_reply = (*it)->reply.data.get_buffer();
        char_len = len * sizeof(CORBA::Octet) / sizeof(char);
        TAO_InputCDR reply_parameters((const char*) buffer_reply, char_len);
        reply_parameters.reset_byte_order(ACE_static_cast (int, (*it)->reply.head.flags));

        // Recover the return value
        if (reply_parameters >> str.inout()) 
          {
            // unmarshall OK. Test if it's expected message
            if (strcmp(str.in(), "RETURN") == 0) {
              TEST_INFO("---- RECORDER: the return parameter was correctly recorded in the reply");
            } else {
              ::CdmwRecording::Recorder_impl::M_test_ok = false;
              TEST_INFO("---- RECORDER: unmarshall NOK - reply stream recorded =  ");
            }
          }
        else
          {      
            ::CdmwRecording::Recorder_impl::M_test_ok = false;
            TEST_INFO("record for \"test_string\" found, but parameter"
                      << " is not a string!");
            TEST_INFO("---- RECORDER: unmarshall NOK - reply stream recorded =  "
                      << str.in());
          }

        // Recover the inout parameter
        if (reply_parameters >> str.inout()) 
          {
            // unmarshall OK. Test if it's expected message
            if (strcmp(str.in(), "INOUT") == 0) {
              TEST_INFO("---- RECORDER: the inout parameter was correctly recorded in the reply");
            } else {
              ::CdmwRecording::Recorder_impl::M_test_ok = false;
              TEST_INFO("---- RECORDER: unmarshall NOK - reply stream recorded =  "
                        << str.in());
            }
          }
        else
          {
            ::CdmwRecording::Recorder_impl::M_test_ok = false;
            TEST_INFO("record for \"test_string\" found, but parameter"
                      << " is not a string!");
            TEST_INFO("---- RECORDER: unmarshall NOK - reply stream recorded =  "
                      << str.in());
          }

        // Recover the out parameter
        if (reply_parameters >> str.inout()) 
          {
            // unmarshall OK. Test if it's expected message
            if (strcmp(str.in(), "OUT") == 0) {
              TEST_INFO("---- RECORDER: the output parameter was correctly recorded in the reply");

            } else {
              ::CdmwRecording::Recorder_impl::M_test_ok = false;
              TEST_INFO("---- RECORDER: unmarshall NOK - reply stream recorded =  "
                        << str.in());
            }
          }
        else
          {
            ::CdmwRecording::Recorder_impl::M_test_ok = false;
            TEST_INFO("record for \"test_string\" found, but parameter"
                      << " is not a string!");
            TEST_INFO("---- RECORDER: unmarshall NOK - reply stream recorded =  "
                      << str.in());
          }
      }

      if (strcmp((*it)->request.operation.in(), "test_exception") == 0) {
        TEST_INFO("Check of the test_exception operation");
        ::nb_recorded_data_to_check--;

        if((*it)->reply.status == 1) {
          TEST_INFO("reply status is USER_EXCEPTION. it's OK");

          CORBA::ULong len = (*it)->reply.data.length();
          const CORBA::Octet* buffer = (*it)->reply.data.get_buffer();
          size_t char_len = len * sizeof(CORBA::Octet) / sizeof(char);
          TAO_InputCDR input((const char*) buffer, char_len);
          input.reset_byte_order(ACE_static_cast (int, (*it)->reply.head.flags));
          CORBA::String_var str;

          if (input >> str.inout()) 
            {
              // unmarshall OK. Test if it's expected message
              if (strcmp(str.in(), "IDL:acme.com/BasicDemo/ExceptionTestRecording:1.0") == 0) {
                TEST_INFO("---- RECORDER: RepositoryId of the exception was correctly recorded in the reply");
              } else {
                ::CdmwRecording::Recorder_impl::M_test_ok = false;
                TEST_INFO("---- RECORDER: RepositoryId of the exception wasn't correctly recorded!");
              }
              
            } else {
              ::CdmwRecording::Recorder_impl::M_test_ok = false;
              TEST_INFO("record for \"test_exception\" found, but parameter"
                        << " is not a string!");
            }

          if (input >> str.inout()) {
            ::CdmwRecording::Recorder_impl::M_test_ok = false;
            TEST_INFO("data recorded is not empty. This is not normal...");
          }else {
            // No more data are waited. This case is expected
            TEST_INFO("record for \"test_exception\" found, but parameter"
                      << " is not a string!. This is normal, the test is OK");
          }
        } else {
          ::CdmwRecording::Recorder_impl::M_test_ok = false;
          TEST_INFO("reply status is not USER_EXCEPTION, but equals to "
                    << (*it)->reply.status);
        }
      }

      if (::nb_recorded_data_to_check == 0) {
        TEST_INFO("End of the checks");
        m_record_list.erase(it);
        break;
      }
    }
}
    
void
Recorder_impl::check_test_result()
  throw(CORBA::SystemException)
{

  // create test runner thread
  M_test_thread = new Cdmw::TestUtils::ThreadedTestRunner(
                      Cdmw::TestUtils::CORBATestManager::instance()->get_controller());
  // add test to be run
  M_test_thread->addTest( 
                 CppUnit::TestFactoryRegistry::getRegistry("test_recorder").makeTest());
  // start test
  M_test_thread->start();

  if (M_test_thread != 0) {
    M_test_thread->join();
    delete M_test_thread;
    M_test_thread = 0;
    // print test summary
    Cdmw::TestUtils::CORBATestManager::instance()->summary();
  }

  sleep(5);
    
}
}; // End namespace CdmwRecording
