/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#include <iostream>
#include <iomanip>
#include <typeinfo>
#include <fstream>
#include <sys/time.h>
#include "Foundation/ossupport/OS.hpp"
#include "Foundation/orbsupport/OrbSupport.hpp"
#include "Foundation/orbsupport/StrategyList.hpp"
#include "Foundation/common/Options.hpp"
#include "Foundation/common/Exception.hpp"
#include "testcommandnotification/ExtSystemObserver_impl.hpp"
#include "Foundation/testutils/CORBATestManager.hpp"


CPPUNIT_TEST_SUITE_REGISTRATION(Cdmw::PlatformMngt::ExtSystemObserver_impl);

namespace Cdmw
{

namespace PlatformMngt
{

void ExtSystemObserver_impl::setUp ()
{
    try
    {
        //Getting the orb reference
        CORBA::ORB_var orb = CORBA::ORB::_duplicate (Cdmw::TestUtils::CORBATestManager::instance ()->get_ORB ());

        //Getting the system reference
        std::string system_corbaloc = "corbaloc::localhost:21869/CdmwPlatformMngtSupervision";
        CORBA::Object_var system_obj = orb->string_to_object (system_corbaloc.c_str());
        m_system = CdmwPlatformMngt::System::_narrow (system_obj.in ());

        // Get a reference to the root POA
        CORBA::Object_var obj = orb->resolve_initial_references("RootPOA");
        PortableServer::POA_var poa = PortableServer::POA::_narrow( obj.in() );

        // Activate the POA manager
        PortableServer::POAManager_var mgr = poa->the_POAManager();
        mgr->activate();
        
        
        CdmwPlatformMngt::ElementNames_var elems;
        CdmwPlatformMngt::ElementNamesIterator_var it;
        //CdmwPlatformMngt::System_var System = CdmwPlatformMngt::System::_narrow (m_system.in());
        CORBA::ULong nbr_of_host = m_system->get_number_of_hosts();
        m_system->get_all_host_names(nbr_of_host, elems.out(), it.out());

        //The Scenario has two different hosts
        host_name1 = CORBA::string_dup ((elems.in())[0]);
        host_name2 = CORBA::string_dup ((elems.in())[1]);


        // Register event factories
        CORBA::ValueFactoryBase_var factory, old_factory;

        factory = new RestartFactory;
        old_factory = orb->register_value_factory(
                "IDL:thalesgroup.com/CdmwPlatformMngt/Restart:1.0",
                factory.in() );
        
        factory = new StopFactory;
        old_factory = orb->register_value_factory(
                "IDL:thalesgroup.com/CdmwPlatformMngt/Stop:1.0",
                factory.in() );
        
        factory = new ShutdownFactory;
        old_factory = orb->register_value_factory(
                "IDL:thalesgroup.com/CdmwPlatformMngt/Shutdown:1.0",
                factory.in() );

        // Register the observer to the m_system
        CdmwPlatformMngt::SupervisionObserver_var observer = this->_this();

        std::string observer_name = "ExtTestObserver";
        CdmwPlatformMngt::RegistrationDepth depth=CdmwPlatformMngt::FULL_HIERARCHY;
        m_system->register_observer_ext( observer_name.c_str(), depth, observer.in() );
    }
    catch (const CORBA::SystemException& ex) 
    {
        std::cerr<< ex._name()<<std::endl;
        throw;
    }
    catch(...)
    {
    }
}



ExtSystemObserver_impl::ExtSystemObserver_impl ()
{  
    /* cold restart command will be performed first
     * 
     *    0  =>  cold_restart
     *    1  =>  host_restart
     */
    restart = false;
    
    // test not performed yet..
    hot_restart_result= cold_restart_result = stop_sys_result= stop_host_result= shutdown_host_result= false;
}

ExtSystemObserver_impl::~ExtSystemObserver_impl()
{
}

void  ExtSystemObserver_impl::notify(CdmwPlatformMngt::Event* an_event)
throw( CORBA::SystemException )
{
    try
    {      
        //CdmwCommand::Restart
        CdmwPlatformMngt::Restart* rest=dynamic_cast<CdmwPlatformMngt::Restart*>(an_event);
        if(rest !=0 )
        {           
            cerr<<"##### Command event =" <<rest->to_string()<<std::endl;
            if (CdmwPlatformMngt::COLD_RESTART == rest->restart_warmth() && !(restart))
            {

                cold_restart_result = true;
                // hot restart command will be performed s
                restart = true;
            }
            if (CdmwPlatformMngt::HOT_RESTART == rest->restart_warmth() && restart)
            {
                hot_restart_result = true;
            }
        }
        
        //CdmwCommand::Stop
        CdmwPlatformMngt::Stop* stop_cmd=dynamic_cast<CdmwPlatformMngt::Stop*>(an_event);
        if( stop_cmd !=0 )
        {
            cerr<<"##### Command event =" <<stop_cmd->to_string()<<std::endl;
            
            const char* elem_name=stop_cmd->element_name();
            if(0 == strcmp(elem_name,host_name2.c_str()))
            {
                stop_sys_result=true;
            } else if(0 == strcmp(elem_name,"S1"))
            {
                stop_host_result=true;
            }          
        }
        
        //CdmwCommand::Shutdown
        CdmwPlatformMngt::Shutdown* sh_cmd=dynamic_cast<CdmwPlatformMngt::Shutdown*>(an_event);
        if( sh_cmd !=0 )
        {
            cerr<<"##### Command event =" <<sh_cmd->to_string()<<std::endl;
            
            shutdown_host_result=true;
        
        }
        
    }
    catch (const std::exception& ex)
    {
        std::cerr << " Time observer exception  " << ex.what()<<__FILE__<<__LINE__;
    }

}

void ExtSystemObserver_impl::test_system_restart()
{
    
    // System Hot Restart SYSTEM_STARTED and APPLICATION_RUNNING
    std::cerr<< "\n===================================\n";
    TEST_INFO("System Cold Restart");
    std::cerr<< "\n===================================\n";
    Cdmw::OsSupport::OS::create_process("cdmw_platform_admin","cdmw_platform_admin --system-corbaloc=corbaloc::localhost:21869/CdmwPlatformMngtSupervision --sys-cold-restart"); 
    Cdmw::OsSupport::OS::sleep (30000);
    std::cerr<< "\n===================================\n";
    TEST_INFO("System Hot Restart");
    std::cerr<< "\n===================================\n";
    Cdmw::OsSupport::OS::create_process("cdmw_platform_admin","cdmw_platform_admin --system-corbaloc=corbaloc::localhost:21869/CdmwPlatformMngtSupervision --sys-hot-restart"); 
    Cdmw::OsSupport::OS::sleep (30000); 
    std::cerr<< "\n===================================\n";
    TEST_INFO("Host Stop");
    std::cerr<< "\n===================================\n";
    std::string command = "cdmw_platform_admin --system-corbaloc=corbaloc::localhost:21869/CdmwPlatformMngtSupervision --host-stop ";
    command.append (host_name2);
    Cdmw::OsSupport::OS::create_process("cdmw_platform_admin", command);
    Cdmw::OsSupport::OS::sleep (40000); 
    std::cerr<< "\n===================================\n";
    TEST_INFO("System Stop");
    std::cerr<< "\n===================================\n";
    Cdmw::OsSupport::OS::create_process("cdmw_platform_admin","cdmw_platform_admin --system-corbaloc=corbaloc::localhost:21869/CdmwPlatformMngtSupervision --sys-stop");
    Cdmw::OsSupport::OS::sleep (10000);  
    std::cerr<< "\n===================================\n";
    TEST_INFO("Host Shutdown");
    std::cerr<< "\n===================================\n";
    command = "cdmw_platform_admin --system-corbaloc=corbaloc::localhost:21869/CdmwPlatformMngtSupervision --host-shutdown ";
    command.append (host_name2);
    Cdmw::OsSupport::OS::create_process("cdmw_platform_admin", command);
    Cdmw::OsSupport::OS::sleep (20000); 
    // Test result
    CPPUNIT_ASSERT (cold_restart_result && hot_restart_result&&stop_sys_result&&stop_host_result&&shutdown_host_result);
}

} // PlatformMngt
} // Cdmw

