/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#include "SystemMngt/platforminterface/PlatformInterface.hpp"
#include <ConfAndPlug/cdmwinit/InitUtils.hpp>

#include "traceclient/MyProcessBehaviour.hpp"
#include "Foundation/logging/LogManager.hpp"
#include "Foundation/logging/LogLevel.hpp"
#include "TraceAndPerf/tracelibrary/Trace.hpp"

using namespace Cdmw;
using namespace Cdmw::PlatformMngt;
using namespace Cdmw::OsSupport;

/**
 * Constructor
 */
MyProcessBehaviour::MyProcessBehaviour(CORBA::ORB_ptr orb) 
   : m_orb(CORBA::ORB::_duplicate(orb)),
     m_trace_client_thread(NULL)
{
    std::cout << "MyProcessBehaviour::MyProcessBehaviour" << std::endl;
}


/**
 * Destructor.
 */
MyProcessBehaviour::~MyProcessBehaviour()
    throw()
{
    std::cout << "MyProcessBehaviour::~MyProcessBehaviour" << std::endl;

    if (m_trace_client_thread != NULL) 
    {
        this->on_stop();
    }
}


/**
 * IDL:thalesgroup.com/CdmwPlatformMngt/Process/get_service:1.0
 */
CORBA::Object_ptr
MyProcessBehaviour::get_service()
    throw(CORBA::SystemException)
{
    std::cout << "MyProcessBehaviour::get_service" << std::endl;

    PlatformInterface::Notify_message (
                          CdmwPlatformMngtBase::INF,
                          m_processName.c_str(),
                          "Service requested -> NIL returned");

    return CORBA::Object::_nil();
}


/**
 * IDL:thalesgroup.com/CdmwPlatformMngt/Process/initialise:1.0
 */
void
MyProcessBehaviour::on_initialise(
        const CdmwPlatformMngtBase::StartupKind& startup_kind)
    throw(CORBA::SystemException)
{
    std::cout << "MyProcessBehaviour::on_initialise" << std::endl;

    try
    {
        // get the name of the application
        m_applicationName = Cdmw::CdmwInit::InitUtils::get_cdmw_application_name();
       
        // get the name of the process
        m_processName = Cdmw::CdmwInit::InitUtils::get_cdmw_process_name();

        /* PCR-1961 : Added log in on_initialise that should not "block" the initialisation */
        CDMW_LOG(Cdmw::Logging::Logger::Get_Logger(m_applicationName, m_processName), 
                 Cdmw::Logging::FATAL, "A first FATAL message on_initialise()");



        PlatformInterface::Notify_message (
           CdmwPlatformMngtBase::INF,
           m_processName.c_str(),
           "Initialisation requested");
        
        // create the TraceClientThread
        m_trace_client_thread = new TraceClientThread(m_orb.in(),
                                                      m_applicationName,
                                                      m_processName);

        std::cout << "Thread built !" << std::endl;
    }
    catch(...)
    {
       std::cout << "Unexpected exception" << std::endl;
    }
}

/**
 * IDL:thalesgroup.com/CdmwPlatformMngt/ProcessDelegate/next_step:1.0
 * operation
 */
void 
MyProcessBehaviour::on_next_step()
    throw(CdmwPlatformMngt::ProcessDelegate::InvalidStep, 
          CORBA::SystemException)
{
    std::cout << "MyProcessBehaviour::on_next_step" << std::endl;
}


/**
 * IDL:thalesgroup.com/CdmwPlatformMngt/Process/run:1.0
 */
void
MyProcessBehaviour::on_run()
    throw(CdmwPlatformMngt::ProcessDelegate::NotReadyToRun,
          CORBA::SystemException)
{
    std::cout << "MyProcessBehaviour::on_run" << std::endl;

    try 
    {
       PlatformInterface::Notify_message(
          CdmwPlatformMngtBase::INF,
          m_processName.c_str(),
          "Run requested");
       
       // start the TraceClientThread
       if (m_trace_client_thread != NULL) 
       {
          m_trace_client_thread->start();
       }
    }
    catch (...)
    {
       std::cout << "Unexpected exception" << std::endl;
    }
}


/**
 * IDL:thalesgroup.com/CdmwPlatformMngt/Process/stop:1.0
 */
void
MyProcessBehaviour::on_stop()
    throw(CORBA::SystemException)
{
    std::cout << "MyProcessBehaviour::on_stop" << std::endl;

    TraceClientThread* client_thread = m_trace_client_thread;
    m_trace_client_thread = NULL;

    PlatformInterface::Notify_message(
                         CdmwPlatformMngtBase::INF,
                         m_processName.c_str(),
                         "Stop requested");

    if (client_thread != NULL) 
    {
        // get the status of the client thread
        OsSupport::ThreadStatus status = client_thread->get_status();

        // if it was requested to run or if it is already running
        // then stop it
        if ((status == OsSupport::THREAD_RUN_REQUEST) ||
            (status == OsSupport::THREAD_RUNNING))
        {
            client_thread->stop();
            client_thread->join();
        }

        delete client_thread;
        client_thread=0x0;
    }

    m_orb->shutdown(false);
}
