/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#ifndef INCL_FT_NAMINGANDREPOSITORYBEHAVIOUR_HPP 
#define INCL_FT_NAMINGANDREPOSITORYBEHAVIOUR_HPP

#include <fstream>

#include <ConfAndPlug/cdmwinit/ProcessControl.hpp>
#include <FaultTolerance/ftstatemanager/DataStore.hpp>

#include "namingandrepository/CdmwNamingAndRepositoryDatastoreEntities.stub.hpp"
#include "namingandrepository/FTDatastoreHelper.hpp"
#include "namingandrepository/Repository_impl.hpp"
#include "namingandrepository/RepositoryExt_impl.hpp"

namespace Cdmw
{
    namespace NamingAndRepository
    {

        typedef Cdmw::FT::DataStore< std::string,
                                     CdmwNamingAndRepository::StorageData > FTDataStoreType;


        class DSObserver : public FTDataStoreType::DataStoreObserver
        {
        public:
            DSObserver(CORBA::ULong dsid, const std::string & filename);
        
            virtual ~DSObserver() throw();
    
            typedef FTDataStoreType::_Oid_in_type _Oid_in_type;
            typedef FTDataStoreType::_Data_in_type _Data_in_type;
        
            /**
             * call by the datastore when the insert method is called on it.
             *
             *@param oid is the Object Id key
             *@param data is the data associated with Oid key
             */
            virtual void on_insert(const _Oid_in_type oid, const _Data_in_type data)
                throw ();
        
        
            /**
             * call by the datastore when the update method is called on it.
             *
             *@param oid is the Object Id key
             *@param data is the data associated with Oid key
             */
            virtual void on_update(const _Oid_in_type oid, const _Data_in_type data)
                throw ();
        
        
            /**
             * call by the datastore when the remove method is called on it.
             *
             *@param oid is the Object Id key
             *@param data is the data associated with Oid key
             */
            virtual void on_remove(const _Oid_in_type oid)
                throw ();
        private:
            CORBA::ULong m_dsid;
            std::ofstream m_os;
        };
    
        

        /**
         * Purpose:
         * <p>
         * The abstract process behaviour.
         */
        class FTNamingAndRepositoryBehaviour : public Cdmw::CdmwInit::ProcessControl
        {
        public:

            /**
             * Purpose:
             * <p> The constructor.
             */ 
            FTNamingAndRepositoryBehaviour(CORBA::ORB_ptr orb, 
                                           bool platformManaged, // FIXME needed?
                                           const std::string& port,
                                           const std::string& physicalId,
                                           const std::string& configurationFile, 
                                           bool validateConfigurationFile,
                                           const std::string& processName);

            /**
             * Purpose:
             * <p> The destructor.
             */ 
            virtual ~FTNamingAndRepositoryBehaviour()
            throw ();

            /**
             * Purpose:
             * <p>
             * the behaviour for the
             * IDL:thalesgroup.com/Cdmw::CdmwInit::ProcessControl/nb_steps:1.0
             * operation
             */
            virtual CORBA::ULong nb_steps()
                throw( CORBA::SystemException );

            /**
             * Purpose:
             * <p>
             * the behaviour for the
             * IDL:thalesgroup.com/Cdmw::CdmwInit::ProcessControl/on_next_step:1.0
             * operation
             */
            void on_next_step()
                throw( CdmwPlatformMngt::ProcessDelegate::InvalidStep,
                       CORBA::SystemException );

            /**
             * Purpose:
             * <p>
             * the behaviour for the
             * IDL:thalesgroup.com/Cdmw::CdmwInit::ProcessControl/get_service:1.0
             * operation
             */
            virtual CORBA::Object_ptr get_service()
                throw(CORBA::SystemException);


            /**
             * Purpose:
             * <p>
             * the behaviour for the
             * IDL:thalesgroup.com/Cdmw::CdmwInit::ProcessControl/on_initialise:1.0
             * operation
             */
            virtual void on_initialise(const CdmwPlatformMngtBase::StartupKind& startup_kind )
                throw( CORBA::SystemException );
    


            /**
             * Purpose:
             * <p>
             * the behaviour for the
             * IDL:thalesgroup.com/Cdmw::CdmwInit::ProcessControl/on_run:1.0
             * operation
             */
            virtual void on_run()
                throw(CdmwPlatformMngt::ProcessDelegate::NotReadyToRun,
                      CORBA::SystemException);

            /**
             * Purpose:
             * <p>
             * the behaviour for the
             * IDL:thalesgroup.com/Cdmw::CdmwInit::ProcessControl/on_stop:1.0
             * operation
             */
            virtual void on_stop()
                throw(CORBA::SystemException);


        private:

            /**
             * Notifies the fatal error via the platform interface or
             * output in cerr.
             */ 
            void notifyFatalError(const char* issuer, const char* message)
                throw(CORBA::SystemException);

            /**
             * The ORB.
             */
            CORBA::ORB_var m_orb;

            /**
             * Indicates whether the repository is managed by the platform management.
             */
            bool m_platformManaged;

            /**
             * The port of the repository.
             */
            std::string m_port;

            /**
             * The physical identifier of the repository.
             */
            std::string m_physicalId;

            /**
             * The configuration file of the repository.
             */
            std::string m_configurationFile;

            /**
             * Indicates whether the configuration file shall be validated.
             */
            bool m_validateConfigurationFile;

            /**
             * The name of the process.
             */
            std::string m_processName;
    
            /**
             * The repository implementation.
             */
            Repository_impl *m_repository_impl;

            /**
             * The repository-ext implementation.
             */
            RepositoryExt_impl *m_repository_ext_impl;

            /**
             * The naming and repository object;
             */
            CdmwNamingAndRepository::Repository_var m_repository;

            /**
             * The naming and repository-ext object;
             */
            CdmwNamingAndRepository::RepositoryExt_var m_repository_ext;

            /**
             * The naming and repository-ext object;
             */
            ::FT::ObjectGroupId  m_repository_ext_gid;

            /**
             * The replication manager.
             */
            ::CdmwFT::ReplicationManager_var m_replication_mgr;
    
            /**
             * The replication manager.
             */
            CdmwFT::Location::GroupRepository_var m_group_repository;

            /**
             * The location of the process (host/appl/proc)
             */
            ::FT::Location_var m_location;

            /**
             *  Yes, if on_initialised has been executed successfully
             */
            bool m_initialised;
        };

    } // End namespace NamingAndRepository
} // End namespace Cdmw

#endif // INCL_FT_NAMINGANDREPOSITORYBEHAVIOUR_HPP
