/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

#include <sstream>
#include <iostream>
#include <string>

#include <Foundation/ossupport/SystemProperties.hpp>
#include <Foundation/ossupport/OS.hpp>
#include <Foundation/orbsupport/OrbSupport.hpp>
#include <Foundation/common/Options.hpp>
#include <Foundation/common/Exception.hpp>
#include <Foundation/commonsvcs/initutils/InitUtils.hpp>
#include <SystemMngt/platforminterface/PlatformInterface.hpp>

#include "namingandrepository/NamingAndRepositoryBehaviour.hpp"
#include "namingandrepository/FTDatastoreHelper.hpp"
#include "namingandrepository/NARLogger.hpp"



namespace
{
    using namespace Cdmw::NamingAndRepository;
    using namespace Cdmw::OsSupport;
    using namespace Cdmw::OrbSupport;
    using namespace Cdmw::PlatformMngt;

    const int ORB_THREAD_POOL_SIZE = 10;
    const int REP_ORB_THREADS_STACK_SIZE = 131072;

    std::string CONFIG_OPTION = "--conf";
    std::string ID_OPTION = "--id";

    std::string TRACE_OPTION = "--trace";
    const char *TRACE_PRODUCER_DEFAULT_NAME = "RepositoryTraceProducer";

/* Part not used in UT, VT or QT : suppressed, as well as TestNamingContext class
   if there is a need for this class it can be recovered with CVS
    std::string THREADING_TEST_OPTION = "--threading-test";
    const char *THREADING_TEST_NAMING_CONTEXT = "ThreadingTestNamingContext";
    const char *THREADING_TEST_NAMING_CONTEXT_IOR = "/tmp/ThreadingTestNamingContext.IOR";
*/

};

std::string getSyntax()
{

    std::ostringstream syntax;

    syntax << std::endl << "Syntax is :" << std::endl;
    syntax << Cdmw::Common::Options::LOCALISATION_SERVICE_OPTION << "=Port";
    syntax << " " << ID_OPTION << "=PhysicalID";
    syntax << " " << CONFIG_OPTION << "=XMLConfigFile" << std::endl;
    syntax << " [" << Cdmw::Common::Options::VALIDATE_XML_OPTION << "]";
//    syntax << " [" << THREADING_TEST_OPTION << "]";
    syntax << " [" << Cdmw::Common::Options::ORB_THREADS << "=Number" << "]";
    syntax << " [" << Cdmw::Common::Options::ORB_THREADS_STACKSIZE << "=Size in KB" << "]";
    syntax << std::endl;

    return syntax.str();
}

std::string getAdditionalSyntax()
{

    std::ostringstream syntax;

    syntax << Cdmw::Common::Options::LOCALISATION_SERVICE_OPTION << " : The repository will use the specified port for the localisation service (corbaloc)" << std::endl;
//    syntax << THREADING_TEST_OPTION << " : The repository will create a special purpose naming context for threading tests" << std::endl;
    syntax << Cdmw::Common::Options::VALIDATE_XML_OPTION << " : The XML configuration file will be validated before processing." << std::endl;
    syntax << ID_OPTION << " : Specifies the mandatory physical identifier of the repository" << std::endl;
    syntax << CONFIG_OPTION << " : Specifies the mandatory XML configuration file" << std::endl << std::endl;
    syntax << std::endl;

    return syntax.str();
}

class InitialisationException : public Cdmw::Exception
{

    public:
        InitialisationException()
            : Exception( Exception::FATAL, "InitialisationException" )
        {
        }
};


/**
*/
void notifyPreInitialisationError(bool platformManaged, const char* message)
{

    if (platformManaged)
    {
        PlatformInterface::Notify_fatal_error("CDMW_NAMREP", message);
    }
    else
    {
        CDMW_ERROR(Cdmw::NamingAndRepository::NARLogger::GetLogger(),message);
    }

    CDMW_THROW(InitialisationException);

}


int main(int argc, char* argv[])
{
    // initialise SystemProperties with argc/argv at first,
    // since some properties are set before ORB_init
    // (else SystemProperties will be initialized as empty)
    SystemProperties::Init(argc, argv);
    
    long orb_thread_nb = ORB_THREAD_POOL_SIZE;
    
    // get Process Executable Name for trace processing
    std::string processName = argv[0];

    try {
       // First initialise the logging library is initialised.
       Cdmw::Logging::LogManager::Init(argc, argv);

       // Get process name from command line argument (if any).
       std::string process_name = 
          Cdmw::OsSupport::OS::get_option_value (argc, argv, Cdmw::Common::Options::PROCESS_NAME_OPTION);
       if ((process_name != "no") && (process_name != "yes")) {
          // Use default process name
          processName = process_name;
       }
       Cdmw::Logging::LogManager::Instance()->set_process_name(processName);

       // From now on, CARDAMOM logging library is initialised.
    } catch (const std::exception & e ) {
       std::cerr << "Cannot initialise Logging library! \n"
                 << "Caught exception : " << e.what() << std::endl;
        ::exit(EXIT_FAILURE);
    }

    // xml option
    bool validate = false;
    std::string validateXML_option =
    OS::get_option_value( argc, argv, Cdmw::Common::Options::VALIDATE_XML_OPTION);
    if ( validateXML_option == "yes" || validateXML_option != "no" )
    {
        validate = true;
    }

    // checks the orb threads option
    std::string orb_threads_option =
        OS::get_option_value(argc,
                             argv,
                             Cdmw::Common::Options::ORB_THREADS);

    if (orb_threads_option != "no")
    {
        if (orb_threads_option != "yes")
        {
            orb_thread_nb = ::atoi(orb_threads_option.c_str());

            if (orb_thread_nb <= 0)
            {
                CDMW_ERROR(NARLogger::GetLogger(),"Bad orb_thread_nb");
                throw CORBA::BAD_PARAM();
            }
        }
    }

    std::string orb_thread_stacksize_option =
        OS::get_option_value(argc, argv, Cdmw::Common::Options::ORB_THREADS_STACKSIZE);

    // Default value chosen for the orb thread stack size in the Repository
    std::size_t orbThreadStackSize(REP_ORB_THREADS_STACK_SIZE);

    if (orb_thread_stacksize_option != "no")
    {
        if (orb_thread_stacksize_option != "yes")
        {
            orbThreadStackSize = ::atoi(orb_thread_stacksize_option.c_str())*1024;
            if (orbThreadStackSize < 0)
            {
                CDMW_ERROR(NARLogger::GetLogger(),"Bad orbThreadStackSize value");
                throw CORBA::BAD_PARAM();
            }
        }
        else
        {
            CDMW_ERROR(NARLogger::GetLogger(),"Bad orbThreadStackSize option");
            throw CORBA::BAD_PARAM();
        }
    } // else ignore.

    SystemProperties::Set_Property(ORB_THREADS_STACKSIZE_PROPERTY_NAME, orbThreadStackSize);

    // the return code
    int ret_code = 0;

    // Initialize the ORB
    StrategyList orbStrategies;
    orbStrategies.add_OrbThreaded();
    orbStrategies.add_PoaThreadPool(orb_thread_nb);
    orbStrategies.add_multicast();

    CORBA::ORB_var orb = CORBA::ORB::_nil();

    try
    {

        orb = OrbSupport::ORB_init(argc, argv, orbStrategies);
        
        // init the traces delegation to the Daemon LogCollector
        //
        std::string daemonPort =
            Cdmw::OsSupport::SystemProperties::Get_Property(Cdmw::OrbSupport::SYSTEM_PORT_PROPERTY_NAME);

        if (!daemonPort.empty()) {
            Cdmw::CommonSvcs::InitUtils* initUtils = 
                Cdmw::CommonSvcs::InitUtils::Instance();
            try {
                initUtils->activate_LogCollector(orb,
                                                 (short) atoi(daemonPort.c_str()));
            }
            catch (const Cdmw::Exception& ex) {
                CDMW_WARN(NARLogger::GetLogger(), 
                          "Trace logs will not be sent to Trace Collector: "
                          << ex.what());
            }
        } else {
            CDMW_WARN(NARLogger::GetLogger(), 
                      "Trace logs will not be sent to Trace Collector\n"
                      << "Because no Daemon port were specified");
        }

        CORBA::Object_var obj = orb->resolve_initial_references("RootPOA");
        PortableServer::POA_var poa = PortableServer::POA::_narrow(obj.in());

        PortableServer::POAManager_var poaMgr = poa->the_POAManager();
        poaMgr->activate();

        bool platformManaged =
            PlatformInterface::Is_launched_by_PlatformManagement(argc, argv);

        if (platformManaged)
        {
            // initialise the platform interface
            PlatformInterface::Setup(orb.in(), argc, argv);
        }

        // preinitialisation : command line check up

        if (argc <= 1 )
        {
            std::string message("Missing mandatory arguments.");
            message += getSyntax();
            notifyPreInitialisationError(platformManaged, message.c_str());
        }

        std::string port_option = OS::get_option_value( argc, argv,
            Cdmw::Common::Options::LOCALISATION_SERVICE_OPTION);

        if ( port_option == "no" || port_option == "yes" )
        {
            std::string message("Port not specified.");
            message += getSyntax();
            notifyPreInitialisationError(platformManaged, message.c_str());
        }

        std::string physicalId = OS::get_option_value( argc, argv, ID_OPTION);
        if ( physicalId == "no" || physicalId == "yes" )
        {
            std::string message("Physical identifier not specified.");
            message += getSyntax();
            notifyPreInitialisationError(platformManaged, message.c_str());
        }

        std::string configurationFile = OS::get_option_value( argc, argv, CONFIG_OPTION);
        if ( configurationFile == "no" || configurationFile == "yes" )
        {
            std::string message("Configuration file not specified.");
            message += getSyntax();
            notifyPreInitialisationError(platformManaged, message.c_str());
        }

        if ( ! OS::file_exists(configurationFile) )
        {
            std::string message("Configuration file '");
            message += configurationFile;
            message += "' not found";
            notifyPreInitialisationError(platformManaged, message.c_str());
        }

        Cdmw::NamingAndRepository::DefaultDataStoreFactory::Create_data_store
            (Cdmw::NamingAndRepository::NAME_DOMAIN_DATASTORE_ID);

        Cdmw::NamingAndRepository::DefaultDataStoreFactory::Create_data_store
            (Cdmw::NamingAndRepository::NAMING_CONTEXT_DATASTORE_ID);

        // creates the process behaviour
        std::auto_ptr<NamingAndRepositoryBehaviour> pProcessBehaviour(
            new NamingAndRepositoryBehaviour(orb.in(), platformManaged,
                                             port_option, physicalId,
                                             configurationFile, validate));

        if (platformManaged)
        {
            // acknowledge the creation of the process
            PlatformInterface::Acknowledge_creation(pProcessBehaviour.get());
            pProcessBehaviour.release();
            
            // This may raise CORBA::BAD_INV_ORDER
            std::string process_name = PlatformInterface::Get_process_name();
            std::string application_name = PlatformInterface::Get_application_name();
  
            Cdmw::Logging::LogManager::Instance()->set_process_name(process_name);
            Cdmw::Logging::LogManager::Instance()->set_application_name(application_name);
            

            // Setup signal handling for ORB to be shut down
            Cdmw::OrbSupport::OrbSupport::install_orb_shutdown_signal_handlers(orb.in());

            orb->run();
            CDMW_INFO(Cdmw::NamingAndRepository::NARLogger::GetLogger(),"The ORB has exited its main loop");

            // cleanup the platform interface
            PlatformInterface::Cleanup();
        }
        else
        {
            CdmwPlatformMngtBase::StartupKind startupKind;
            startupKind.state_retrieval = CdmwPlatformMngtBase::LAST_STATE;
            startupKind.startup_mode = 0;
            startupKind.specified_state_id = CORBA::string_dup("");

            (pProcessBehaviour.get())->initialise(startupKind);

            // create a threading test special purpose naming context if required
/*
            std::string threadingTest = OS::get_option_value( argc, argv, THREADING_TEST_OPTION);
            if ( threadingTest == "yes" )
            {

                // Create a naming context for multithreading access tests
                CosNaming::NamingContextExt_var namingContext = TestNamingContext_impl::create(THREADING_TEST_NAMING_CONTEXT);

                // save this naming context's IOR in a file
                CORBA::String_var namingContextIOR = orb->object_to_string(namingContext.in());

                std::ofstream namingContextIORFile(THREADING_TEST_NAMING_CONTEXT_IOR);
                namingContextIORFile << namingContextIOR.in();
                namingContextIORFile.close();

                CDMW_INFO(Cdmw::NamingAndRepository::NARLogger::GetLogger(),"IOR of Naming Context for threading test in " << THREADING_TEST_NAMING_CONTEXT_IOR << " file");

            }
*/
            (pProcessBehaviour.get())->run();

            // Setup signal handling for ORB to be shut down
            Cdmw::OrbSupport::OrbSupport::install_orb_shutdown_signal_handlers(orb.in());

            orb->run();
            // the repository can only be killed
        }

        Cdmw::OrbSupport::OrbSupport::uninstall_orb_shutdown_signal_handlers(orb.in());

    }
    catch(const InitialisationException &e)
    {
        // already logged
        ret_code = EXIT_FAILURE;
    }
    catch(const Cdmw::Exception &e)
    {
        CDMW_ERROR(Cdmw::NamingAndRepository::NARLogger::GetLogger(),e.what());
        ret_code = EXIT_FAILURE;
    }
    catch(const CORBA::Exception &e)
    {
        CDMW_ERROR(Cdmw::NamingAndRepository::NARLogger::GetLogger(),"unexpected " << e._name() << " CORBA exception");
        ret_code = EXIT_FAILURE;
    }
    catch(...)
    {
        CDMW_ERROR(Cdmw::NamingAndRepository::NARLogger::GetLogger(),"unexpected exception");
        ret_code = EXIT_FAILURE;
    }

    if (!CORBA::is_nil(orb.in()))
    {
        Cdmw::OrbSupport::OrbSupport::ORB_cleanup(orb.in());
        // Release LogCollectorDelegate
        Cdmw::CommonSvcs::InitUtils::Instance()->deactivate_LogCollector();
        orb->destroy();
    }

    return ret_code;

}


