/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

#include <fstream>

#include <Foundation/common/Locations.hpp>
#include <Foundation/commonsvcs/naming/NamingInterface.hpp>
#include <Foundation/ossupport/SystemProperties.hpp>
#include <Foundation/orbsupport/ExceptionMinorCodes.hpp>

#include <SystemMngt/idllib/CdmwPlatformMngtDaemon.stub.hpp>
#include <SystemMngt/platforminterface/ServiceNames.hpp>

#include "namingandrepository/NamingAndRepositoryBehaviour.hpp"
#include "namingandrepository/StandardRepository_impl.hpp"
#include "namingandrepository/Configurator.hpp"
#include "namingandrepository/NARLogger.hpp"

using namespace Cdmw::PlatformMngt;

namespace Cdmw
{
    namespace NamingAndRepository
    {

        const char *CORBALOC_NAME = "CdmwRepository";
        const char *NAME_SERVICE_ID = "NameService";

        const char* REPOSITORY_BIND_TO_DAEMON_PROPERTY_NAME = "Cdmw.Repository.BindToDaemon";


        /**
         * Purpose:
         * <p> The constructor.
         */ 
        NamingAndRepositoryBehaviour::NamingAndRepositoryBehaviour(CORBA::ORB_ptr orb, bool platformManaged,
                                                                   const std::string& port,
                                                                   const std::string& physicalId,
                                                                   const std::string& configurationFile,
                                                                   bool validateConfigurationFile)
            :   m_platformManaged(platformManaged),
                m_port(port),
                m_physicalId(physicalId),
                m_configurationFile(configurationFile),
                m_validateConfigurationFile(validateConfigurationFile)
        {
            m_orb = CORBA::ORB::_duplicate(orb);
        }


        /**
         * Purpose:
         * <p> The destructor.
         */ 
        NamingAndRepositoryBehaviour::~NamingAndRepositoryBehaviour() throw ()
        {
        }


        /**
         * Purpose:
         * <p>
         * the behaviour for the
         * IDL:thalesgroup.com/CdmwPlatformMngt/ProcessDelegate/get_service:1.0
         * operation
         */
        CORBA::Object_ptr NamingAndRepositoryBehaviour::get_service() throw(CORBA::SystemException)
        {
            return CdmwNamingAndRepository::Repository::_duplicate(m_repository.in());
        }


        /**
         * Purpose:
         * <p>
         * the behaviour for the
         * IDL:thalesgroup.com/CdmwPlatformMngt/ProcessDelegate/initialise:1.0
         * operation
         */
        void NamingAndRepositoryBehaviour::initialise(const CdmwPlatformMngtBase::StartupKind& startup_kind)
            throw(CdmwPlatformMngt::ProcessDelegate::BadOrder, CORBA::SystemException)
        {

            std::string configFileErrorHeader("In file '");
            configFileErrorHeader += m_configurationFile;
            configFileErrorHeader += "' - ";

            // Creates the repository
            m_repository_impl = new StandardRepository_impl();
            m_repository = m_repository_impl->_this();

            // Initializes the repository
            if (!m_repository_impl->initialize(m_orb.in())) {
                notifyFatalError("CDMW_NAMREP", "Repository initialisation failed");
            }

            // Starts the repository
            if (!m_repository_impl->start(m_orb.in())) {
                notifyFatalError("CDMW_NAMREP", "Repository initialisation failed");
            }

            // configure the repository via the XML file
            try {
                RepositoryConfigurator config(m_repository.in(), m_validateConfigurationFile);
                config.configure(m_configurationFile);
            }
            catch(const ConfigurationException& e) {
                std::string message(configFileErrorHeader);
                message += e.what();
                
                notifyFatalError("CDMW_NAMREP", message.c_str());
            }

            // register the admin root context within the CDMW admin name domain
            // TODO : Replace with a user name domain ?
            CosNaming::NamingContext_var adminRootContext;

            try {
                adminRootContext = 
                    m_repository_impl->resolve_root_context(Cdmw::Common::Locations::ADMIN_ROOT_CONTEXT_ID);
                
                CdmwNamingAndRepository::NameDomain_var adminNameDomain =
                    m_repository_impl->resolve_name_domain(Cdmw::Common::Locations::CDMW_ADMIN_NAME_DOMAIN);
                
                adminNameDomain->register_new_object(m_physicalId.c_str(), adminRootContext.in());
            }
            catch(const CdmwNamingAndRepository::Repository::NoRootContext &) {
                std::string message(configFileErrorHeader);
                message += "AdminRootContext not found";

                notifyFatalError("CDMW_NAMREP", message.c_str());
            }
            catch(const CdmwNamingAndRepository::NoNameDomain &) {
                std::string message(configFileErrorHeader);
                message += "Name domain '";
                message += Cdmw::Common::Locations::CDMW_ADMIN_NAME_DOMAIN;
                message += "' not found";

                notifyFatalError("CDMW_NAMREP", message.c_str());
            }
            catch(const CdmwNamingAndRepository::InvalidName &) {
                std::string message("'");
                message += m_physicalId;
                message += "' is an invalid physical identifier";

                notifyFatalError("CDMW_NAMREP", message.c_str());
            }
            catch(...) {
                notifyFatalError("CDMW_NAMREP", "Something caught!!");
            }

            // Register the FactoryFinder
            CosNaming::NamingContext_var defaultRootCtx = CosNaming::NamingContext::_nil();
            try {
                defaultRootCtx =
                    m_repository_impl->resolve_root_context(CdmwNamingAndRepository::DEFAULT_ROOT_CONTEXT);
            
                CosNaming::Name_var name =
                    Cdmw::CommonSvcs::Naming::NamingInterface::to_name(CdmwNamingAndRepository::FACTORY_FINDER);
            
                defaultRootCtx->rebind(name.in(), m_repository_impl->get_factory_finder());
            } 
            catch (...) {
                notifyFatalError("CDMW_NAMREP", 
                                 "Failed to rebind factory finder reference");
            }


            if (Cdmw::OsSupport::SystemProperties::Get_Property(
                                                                REPOSITORY_BIND_TO_DAEMON_PROPERTY_NAME, "no") == "yes") {
                // bind repository into local Daemon (is system port specified)
                std::string system_port = 
                    Cdmw::OsSupport::SystemProperties::Get_Property(
                                                                    Cdmw::OrbSupport::SYSTEM_PORT_PROPERTY_NAME);
                if (system_port != "") {
                    try {
                              
                        // Get Daemon
                        std::string daemon_corbaloc = 
                            "corbaloc::localhost:";
                        daemon_corbaloc += system_port + "/";
                        daemon_corbaloc += CdmwPlatformMngt::DAEMON_NAME;
                        CORBA::Object_var obj = 
                            m_orb->string_to_object(daemon_corbaloc.c_str());
                        if (CORBA::is_nil(obj.in())) {
                            CDMW_WARN(NARLogger::GetLogger(),"Cannot register Repository in local Daemon " 
                                      << "(Daemon corbaloc is nil)");
                        } else {
                            CdmwPlatformMngt::Daemon_var daemon =
                                CdmwPlatformMngt::Daemon::_narrow (obj.in());
                            if (CORBA::is_nil(daemon.in())) {
                                CDMW_WARN(NARLogger::GetLogger(),"cannot register Repository in local Daemon " 
                                          << "(Daemon is not a Daemon!!)");
                            } else {
                                // register the proxy as daemon local service
                                daemon->register_local_service(
                                                               Cdmw::PlatformMngt::ServiceNames::NAMING_AND_REPOSITORY_SERVICE,
                                                               m_repository.in());
                                CDMW_INFO(NARLogger::GetLogger(),"Repository registered in local Daemon with name " 
                                          << Cdmw::PlatformMngt::ServiceNames::NAMING_AND_REPOSITORY_SERVICE);
                            }
                        }
                    
                        // Get Daemon's NamesService
                        std::string daemon_ns_corbaloc = 
                            "corbaloc::localhost:";
                        daemon_ns_corbaloc += system_port + "/NameService";
                        obj =  m_orb->string_to_object(daemon_ns_corbaloc.c_str());
                        if (CORBA::is_nil(obj.in())) {
                            CDMW_WARN(NARLogger::GetLogger(),"Cannot register Repository in local Daemon " 
                                      << "(Daemon corbaloc is nil)");
                        } else {
                            CosNaming::NamingContext_var daemon_ns =
                                CosNaming::NamingContext::_narrow(obj.in());
                            if (CORBA::is_nil(daemon_ns.in())) {
                                CDMW_WARN(NARLogger::GetLogger(),"Cannot register Repository in local Daemon " 
                                          << "(Daemon is not a NamingContext)");
                            } else {
                                ::Cdmw::CommonSvcs::Naming::NamingInterface ni(daemon_ns.in());
                                ni.bind(CdmwNamingAndRepository::DEFAULT_ROOT_CONTEXT, 
                                        defaultRootCtx.in(), true);
                                CDMW_INFO(NARLogger::GetLogger(),"Default Root Context bound in local Daemon with name " 
                                          << CdmwNamingAndRepository::DEFAULT_ROOT_CONTEXT);
                                   
                                ni.bind(Cdmw::Common::Locations::ADMIN_ROOT_CONTEXT_ID, 
                                        adminRootContext.in(), true);
                                CDMW_INFO(NARLogger::GetLogger(),"Admin Root Context bound in local Daemon with name " 
                                          << Cdmw::Common::Locations::ADMIN_ROOT_CONTEXT_ID);                           
                            }
                        }
                    
                    } catch (const CORBA::SystemException& e) {
                        CDMW_WARN(NARLogger::GetLogger(),"Cannot register Repository in local Daemon: " << e);
                    }
                }
                else {
                    CDMW_WARN(NARLogger::GetLogger(),"Cannot register Repository in local Daemon " 
                              << "(System port not defined)");
                }
            }

            CDMW_INFO(NARLogger::GetLogger(),"started with port : " << m_port);

            // the repository is published via corbaloc 
            Cdmw::OrbSupport::OrbSupport::bind_object_to_corbaloc(m_orb.in(),
                                                                  CORBALOC_NAME, m_repository.in());
            CDMW_INFO(NARLogger::GetLogger(),"Corbaloc name : " << CORBALOC_NAME);

            // and via an IOR file
            CORBA::String_var repositoryIOR = m_orb->object_to_string(m_repository.in());

            std::ofstream repositoryIORFile("Repository.IOR");
            repositoryIORFile << repositoryIOR.in();
            repositoryIORFile.close();
            CDMW_INFO(NARLogger::GetLogger(),"IOR in Repository.IOR file");
        
            // Bind the Default RootContext to corbaloc Id NameService
            // to be accessible by corbaname
            CosNaming::NamingContext_var default_root_ctxt = 
                m_repository->resolve_root_context (CdmwNamingAndRepository::DEFAULT_ROOT_CONTEXT);
            Cdmw::OrbSupport::OrbSupport::bind_object_to_corbaloc (m_orb.in(),
                                                                   NAME_SERVICE_ID,
                                                                   default_root_ctxt.in());
            CDMW_INFO(NARLogger::GetLogger(), CdmwNamingAndRepository::DEFAULT_ROOT_CONTEXT << "'s corbaloc name : " << NAME_SERVICE_ID);
        }


        /**
         * Purpose:
         * <p>
         * the behaviour for the
         * IDL:thalesgroup.com/CdmwPlatformMngt/ProcessDelegate/run:1.0
         * operation
         */
        void NamingAndRepositoryBehaviour::run()
            throw(CdmwPlatformMngt::ProcessDelegate::NotReadyToRun,
                  CdmwPlatformMngt::ProcessDelegate::AlreadyDone,
                  CORBA::SystemException)
        {
            CDMW_INFO(NARLogger::GetLogger(),"Run requested");
        }

        /**
         * Purpose:
         * <p>
         * the behaviour for the
         * IDL:thalesgroup.com/CdmwPlatformMngt/ProcessDelegate/stop:1.0
         * operation
         */
        void NamingAndRepositoryBehaviour::stop()
            throw(CORBA::SystemException)
        {
            CDMW_INFO(NARLogger::GetLogger(),"Stop requested");

            m_orb->shutdown(false);

        }


        /**
         * Notifies the fatal error via the platform interface or
         * output in cerr.
         */ 
        void NamingAndRepositoryBehaviour::notifyFatalError(const char* issuer, const char* message)
            throw(CORBA::SystemException)
        {

            if (m_platformManaged) {
                PlatformInterface::Notify_fatal_error(issuer, message);
            }
            else {
                CDMW_ERROR(NARLogger::GetLogger(), message);
            }
        
            throw CORBA::INTERNAL(OrbSupport::INTERNAL, CORBA::COMPLETED_NO);

        }
    } // End namespace NamingAndRepository
} // End namespace Cdmw
