/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#include "Foundation/orbsupport/ExceptionMinorCodes.hpp"
#include "Foundation/common/Assert.hpp"
#include "Foundation/common/Locations.hpp"
#include "Foundation/logging/FunctionLogger.hpp"
#include "namingandrepository/Repository_impl.hpp"
#include "namingandrepository/RootNamingContext_impl.hpp"
#include "namingandrepository/NameDomain_impl.hpp"
#include "namingandrepository/FactoryFinder_impl.hpp"
#include "namingandrepository/ORBFacility.hpp"
#include "namingandrepository/NamingUtilities.hpp"
#include "namingandrepository/NARLogger.hpp"
#include "Foundation/commonsvcs/naming/NamingInterface.hpp"

#include "TraceAndPerf/tracelibrary/CdmwTrace.hpp"


namespace Cdmw
{
namespace NamingAndRepository
{


CosNaming::NamingContext_ptr Repository_impl::resolve_root_context(const char* root_id)
        throw(CdmwNamingAndRepository::Repository::NoRootContext,
              CORBA::SystemException)
{

    CDMW_LOG_FUNCTION(NARLogger::GetLogger(), " root_id=" << root_id );

    try
    {

        try
        {

            std::string rootContextId = NamingContext_impl::ROOT_CONTEXT_PREFIX;
            rootContextId += root_id;

            CosNaming::NamingContext_ptr result = NamingContext_impl::findById(rootContextId);
            
            if (CORBA::is_nil(result))
                throw CdmwNamingAndRepository::Repository::NoRootContext();

            return result;

        }
        catch(const std::bad_alloc &)
        {
            CDMW_WARN(NARLogger::GetLogger(), "std::bad_alloc in Repository_impl resolve_root_context");

            throw CORBA::NO_RESOURCES(OrbSupport::NO_RESOURCES, CORBA::COMPLETED_NO);
        }
        catch(const AssertionFailedException &)
        {
            CDMW_WARN(NARLogger::GetLogger(), "AssertionFailedException in Repository_impl resolve_root_context");

            throw CORBA::INTERNAL(OrbSupport::INTERNAL, CORBA::COMPLETED_NO);
        }
        catch(const InternalErrorException &)
        {
            CDMW_WARN(NARLogger::GetLogger(), "InternalErrorException in Repository_impl resolve_root_context");

            throw CORBA::INTERNAL(OrbSupport::INTERNAL, CORBA::COMPLETED_NO);
        }
    }
    catch(const CdmwNamingAndRepository::Repository::NoRootContext&) {
        CDMW_DEBUG(NARLogger::GetLogger(), "Exception: CdmwNamingAndRepository::Repository::NoRootContext");

        throw;
    }
    catch(const CORBA::Exception& e)
    {
        CDMW_WARN(NARLogger::GetLogger(), "Exception: " << Cdmw::OrbSupport::OrbSupport::exception_to_string(e));
        CDMW_LOG_FUNCTION_EXCEPTION(e);

        throw;
    }

}



CdmwNamingAndRepository::NameDomain_ptr Repository_impl::resolve_name_domain(const char* domain_name)
        throw(CdmwNamingAndRepository::NoNameDomain,
              CdmwNamingAndRepository::InvalidName,
              CORBA::SystemException)
{

    CDMW_LOG_FUNCTION(NARLogger::GetLogger(), "domain_name=" << domain_name );

    try
    {


        try
        {
            CdmwNamingAndRepository::NameDomain_var nameDomain;

            CosNaming::Name_var name = Cdmw::CommonSvcs::Naming::NamingInterface::to_name(domain_name);

            // get the root name domain corresponding to the first name component
            std::string firstPartStr = NamingUtilities::stringifiedNameFirstPart(name.in());

            CdmwNamingAndRepository::NameDomain_var rootNameDomain =
                NameDomain_impl::findRootNameDomain(m_defaultRootContext.in(), firstPartStr);

            if (name->length() > 1)
            {
                // propagates the resolution
                std::string endPartStr = NamingUtilities::stringifiedNameEndPart(name.in());

                nameDomain = rootNameDomain->resolve_sub_domain(endPartStr.c_str());
            }
            else
            {
                nameDomain = rootNameDomain._retn();
            }

            return nameDomain._retn();

        }
        catch(const Cdmw::CommonSvcs::Naming::InvalidNameException &)
        {
            CDMW_WARN(NARLogger::GetLogger(), "InvalidNameException in Repository_impl resolve_name_domain");

            // invalid name
            throw CdmwNamingAndRepository::InvalidName();
        }
        catch(const std::bad_alloc &)
        {
            CDMW_WARN(NARLogger::GetLogger(), "std::bad_alloc in Repository_impl resolve_name_domain");

            throw CORBA::NO_RESOURCES(OrbSupport::NO_RESOURCES, CORBA::COMPLETED_NO);
        }
        catch(const OutOfResourcesException &)
        {
            CDMW_WARN(NARLogger::GetLogger(), "OutOfResourcesException in Repository_impl resolve_name_domain");

            throw CORBA::NO_RESOURCES(OrbSupport::NO_RESOURCES, CORBA::COMPLETED_NO);
        }
        catch(const AssertionFailedException &)
        {
            CDMW_WARN(NARLogger::GetLogger(), "AssertionFailedException in Repository_impl resolve_name_domain");

            throw CORBA::INTERNAL(OrbSupport::INTERNAL, CORBA::COMPLETED_NO);
        }
        catch(const InternalErrorException &)
        {
            CDMW_WARN(NARLogger::GetLogger(), "InternalErrorException in Repository_impl resolve_name_domain");

            throw CORBA::INTERNAL(OrbSupport::INTERNAL, CORBA::COMPLETED_NO);
        }

    }
    catch(const CORBA::Exception& e)
    {
        CDMW_WARN(NARLogger::GetLogger(), "Exception: " << e._name() << " in Repository_impl resolve_name_domain");
        CDMW_LOG_FUNCTION_EXCEPTION(e);

        throw;
    }

}


bool Repository_impl::start(CORBA::ORB_ptr orb)
{

    bool result  = false;
    bool primary = false;

    CDMW_ASSERT(!CORBA::is_nil(orb));

    PortableServer::POA_var rootPOA;

    try
    {
        // Initialize ORBFacility
        ORBFacility::initialize(orb);

        // Get the root POA
        CORBA::Object_var obj = orb->resolve_initial_references("RootPOA");
        rootPOA = PortableServer::POA::_narrow(obj.in());

        // Retrieve the default root context 
        CDMW_WARN(NARLogger::GetLogger(),"CdmwNamingAndRepository::DEFAULT_ROOT_CONTEXT [" << CdmwNamingAndRepository::DEFAULT_ROOT_CONTEXT << "]");
        CosNaming::NamingContext_var root_ctx = resolve_root_context(CdmwNamingAndRepository::DEFAULT_ROOT_CONTEXT);
        m_defaultRootContext = CosNaming::NamingContextExt::_narrow(root_ctx.in());

        result = true;
    primary = true;
    }
    catch(const CdmwNamingAndRepository::Repository::NoRootContext &)
    {
        // the default root context doesn't exist
        try
        {
            // Create the default root context
            m_defaultRootContext = 
                RootNamingContext_impl::createContext(CdmwNamingAndRepository::DEFAULT_ROOT_CONTEXT);
            result  = true;
        primary = true;
        }
    catch(const AlreadyExistsException &) {
        // handle raise-condition: On startup, id concurrently
        // stored by other process in process-group, try to
        // retrieve the id again
        CosNaming::NamingContext_var root_ctx 
        = resolve_root_context(CdmwNamingAndRepository::DEFAULT_ROOT_CONTEXT);
        m_defaultRootContext = CosNaming::NamingContextExt::_narrow(root_ctx.in());
        result  = true;
        primary = false;
    } 
        catch(...)
        {
        
        }
    }
    catch(const CORBA::Exception &)
    {
        // Error

    }
    
    if (result && primary) {
    result = false;

    try
        {
        // Retrieve the admin root context 
        CosNaming::NamingContext_var admin_ctx = 
            resolve_root_context(Cdmw::Common::Locations::ADMIN_ROOT_CONTEXT_ID);
        
        result = true;      
        }
    catch(const CdmwNamingAndRepository::Repository::NoRootContext &)
        {
        // the admin root context doesn't exist
        try
            {
            // Create the admin root context
            CosNaming::NamingContext_var admin_ctx =
                RootNamingContext_impl::createContext(Cdmw::Common::Locations::ADMIN_ROOT_CONTEXT_ID);
            result  = true;
            }
        catch(const AlreadyExistsException &) {
            // handle raise-condition: On startup, id concurrently
            // stored by other process in process-group, try to
            // retrieve the id again
            CosNaming::NamingContext_var admin_ctx 
            = resolve_root_context(Cdmw::Common::Locations::ADMIN_ROOT_CONTEXT_ID);
            result  = true;
        } 
        catch(...)
            {
            
            }
        
        }
    catch(const CORBA::Exception &)
        {
        // Error
        
        }
    }

    FactoryFinder_impl *facFinder_i = new FactoryFinder_impl(m_defaultRootContext.in());

    m_factoryFinder = facFinder_i->_this();
    facFinder_i->_remove_ref(); // drop ref count due to previous call
    
    return result;

}

CdmwLifeCycle::FactoryFinder_ptr Repository_impl::get_factory_finder()
{
    return CdmwLifeCycle::FactoryFinder::_duplicate(m_factoryFinder.in());
}

} // End of namespace NamingAndRepository
} // End of namespace Cdmw

