/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

#include "Foundation/orbsupport/ExceptionMinorCodes.hpp"
#include "namingandrepositoryproxy/AccessorProxy.hpp"
#include "Foundation/commonsvcs/naming/NamingInterface.hpp"
#include "Foundation/ossupport/OS.hpp"
#include "Foundation/common/Locations.hpp"
#include "Foundation/osthreads/MutexGuard.hpp"
#include <TraceAndPerf/idllib/CdmwTraceCommon.stub.hpp>
#include "TraceAndPerf/tracelibrary/CdmwTrace.hpp"
#include "Foundation/logging/Logger.hpp"
#include "SystemMngt/platforminterface/PlatformInterface.hpp"
#include "namingandrepositoryproxy/NARProxyLogger.hpp"
#include "Foundation/logging/FunctionLogger.hpp"
#include <fstream>
#include <string>
#include <iostream>

using namespace Cdmw::Trace;
//using namespace Cdmw::PlatformMngt;

#define COUT2(mes) std::cout << "NamingAndRepository : " << mes << std::endl;

namespace Cdmw
{
    namespace NamingAndRepository
    {

        const char *CORBALOC_NAME = "CdmwRepository";
        const char *NAME_SERVICE_ID = "NameService";

        Cdmw::OsSupport::Mutex  AccessorProxy::m_accessor_mutex;
        CdmwNamingAndRepository::RepositoryExt_var AccessorProxy::m_repository = CdmwNamingAndRepository::RepositoryExt::_nil();
        CORBA::ORB_var AccessorProxy::m_orb = CORBA::ORB::_nil();
        std::string AccessorProxy::m_port = "";
        std::string AccessorProxy::m_context_holder_name = "";

        bool AccessorProxy::initialise()
            throw (CORBA::TRANSIENT, CORBA::SystemException)
        {
            // True if all the bindings of the initialise method are performed
            // otherwise false
            static bool accessor_ok = false;

            CDMW_LOG_FUNCTION(NARProxyLogger::GetLogger(),
                              " call to the initialise method of the Accessor class." );

            if(!accessor_ok) {
            
                CDMW_MUTEX_GUARD(m_accessor_mutex);
                if(accessor_ok) return accessor_ok;

                // Get the root POA
                CORBA::Object_var obj = m_orb->resolve_initial_references("RootPOA");
                PortableServer::POA_var rootPOA = 
                    PortableServer::POA::_narrow(obj.in());

                CDMW_ASSERT(!CORBA::is_nil(rootPOA.in()));

                CosNaming::NamingContext_var admin_root
                    = CosNaming::NamingContext::_nil();   


                COUT2("started with port : " << m_port);
                // the repository is published via corbaloc 
                Cdmw::OrbSupport::OrbSupport::bind_object_to_corbaloc(m_orb.in(),
                                                                      CORBALOC_NAME, m_repository.in());
                COUT2("Corbaloc name : " << CORBALOC_NAME);

                // and via an IOR file
                CORBA::String_var repositoryIOR = m_orb->object_to_string(m_repository.in());

                std::ofstream repositoryIORFile("Repository.IOR");
                repositoryIORFile << repositoryIOR.in();
                repositoryIORFile.close();
                COUT2("IOR in Repository.IOR file");
        
                // Bind the Default RootContext to corbaloc Id NameService
                // to be accessible by corbaname
                try {
                    CosNaming::NamingContext_var default_root_ctxt = 
                        m_repository->resolve_root_context (CdmwNamingAndRepository::DEFAULT_ROOT_CONTEXT);
      
                    Cdmw::OrbSupport::OrbSupport::bind_object_to_corbaloc (m_orb.in(),
                                                                           NAME_SERVICE_ID,
                                                                           default_root_ctxt.in());
                    COUT2(CdmwNamingAndRepository::DEFAULT_ROOT_CONTEXT 
                          << "'s corbaloc name : " << NAME_SERVICE_ID);
                }
                catch (const CORBA::Exception& ex) {
                    CDMW_WARN(NARProxyLogger::GetLogger(),
                              "CDMW_PROXY_REPOSITORY: Cannot bind default context.");

                    CDMW_LOG_FUNCTION_EXCEPTION(ex);

                    throw CORBA::TRANSIENT(OrbSupport::TRANSIENT, CORBA::COMPLETED_NO);
                }

                try {
                    if(CORBA::is_nil(admin_root.in()) == true )
                        admin_root = m_repository->resolve_root_context(Cdmw::Common::Locations::ADMIN_ROOT_CONTEXT_ID);
      
                    CDMW_ASSERT(!CORBA::is_nil(admin_root.in()));
      
                }
                catch (const CORBA::Exception& ex) {
                    CDMW_WARN(NARProxyLogger::GetLogger(),
                              "CDMW_PROXY_REPOSITORY: Cannot resolve admin context.");

                    CDMW_LOG_FUNCTION_EXCEPTION(ex);

                    throw CORBA::TRANSIENT(OrbSupport::TRANSIENT, CORBA::COMPLETED_NO);
                }
                try { 
                    Cdmw::CommonSvcs::Naming::NamingInterface ni (admin_root.in());

                    ni.bind_new_context(m_context_holder_name, true);

                }
                catch (const CORBA::Exception& ex) {
                    CDMW_WARN(NARProxyLogger::GetLogger(),
                              "CDMW_PROXY_REPOSITORY: Cannot bind holder context.");

                    CDMW_LOG_FUNCTION_EXCEPTION(ex);

                    throw CORBA::TRANSIENT(OrbSupport::TRANSIENT, CORBA::COMPLETED_NO);
                }
        
                accessor_ok = true;
            }
            return accessor_ok;
    
        }

        void  AccessorProxy::set_repository (CdmwNamingAndRepository::RepositoryExt_ptr repository)
            throw()
        {
            m_repository = repository;
        }

        void AccessorProxy::set_orb (CORBA::ORB_ptr orb)
            throw ()
        {
            m_orb = orb;
        }

        void AccessorProxy::set_port (const std::string& port)
            throw ()
        {
            m_port = port;
        }

        void AccessorProxy::set_context_holder_name (const std::string& holder_name)
            throw ()
        {
            m_context_holder_name = holder_name;
        }

    } // End of namespace NamingAndRepository
} // End of namespace Cdmw
