/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#include "namingandrepositoryproxy/NamingAndRepositoryProxyBehaviour.hpp"
#include "namingandrepositoryproxy/AccessorProxy.hpp"
#include "namingandrepositoryproxy/InitUtils.hpp"

#include <SystemMngt/platforminterface/PlatformInterface.hpp>
#include <SystemMngt/idllib/CdmwPlatformMngtDaemon.stub.hpp>
#include <SystemMngt/platforminterface/ServiceNames.hpp>

#include <Foundation/orbsupport/ExceptionMinorCodes.hpp>
// #include "Foundation/orbsupport/OrbSupport.hpp"

#include <fstream>
#include <iostream>
#include <string>

#define COUT2(mes) std::cout << "NamingAndRepositoryProxy : " << mes << std::endl;

namespace Cdmw
{
    namespace NamingAndRepository
    {

        /**
         * Purpose:
         * <p> The constructor.
         */ 
        NamingAndRepositoryProxyBehaviour::NamingAndRepositoryProxyBehaviour(CORBA::ORB_ptr orb, 
                                                                             const std::string& port,
                                                                             const std::string& daemonPort,
                                                                             const std::string& processName):
            m_daemonPort(daemonPort),
            m_processName(processName)
        {
            AccessorProxy::set_orb(CORBA::ORB::_duplicate(orb));
            AccessorProxy::set_port(port);
            m_orb = CORBA::ORB::_duplicate(orb);
        }
        

        /**
         * Purpose:
         * <p> The destructor.
         */ 
        NamingAndRepositoryProxyBehaviour::~NamingAndRepositoryProxyBehaviour()
            throw ()
        {
        }
        

        /**
         * Purpose:
         * <p>
         * the behaviour for the
         * IDL:thalesgroup.com/CdmwPlatformMngt/ProcessDelegate/get_service:1.0
         * operation
         */
        CORBA::Object_ptr NamingAndRepositoryProxyBehaviour::get_service()
            throw(CORBA::SystemException)
        {
            return CdmwNamingAndRepository::Repository::_duplicate(m_repository.in());
        }


        /**
         * Purpose:
         * <p>
         * the behaviour for the
         * IDL:thalesgroup.com/CdmwPlatformMngt/ProcessDelegate/initialise:1.0
         * operation
         */
        void NamingAndRepositoryProxyBehaviour::on_initialise(const CdmwPlatformMngtBase::StartupKind& startup_kind)
            throw(CORBA::SystemException)
        {
            COUT2("Init requested");

            // Get the Replication Manager
            CORBA::Object_var replication_mgr_obj
                = m_orb->resolve_initial_references("ReplicationManager");
            
            m_replication_mgr = 
                ::CdmwFT::ReplicationManager::_narrow(replication_mgr_obj.in() );
            CDMW_ASSERT(!CORBA::is_nil(m_replication_mgr.in()));

            // Get the root POA
            CORBA::Object_var obj = m_orb->resolve_initial_references("RootPOA");
            PortableServer::POA_var rootPOA = 
                PortableServer::POA::_narrow(obj.in());
            CDMW_ASSERT(!CORBA::is_nil(rootPOA.in()));

            // Find the RepositoryExt reference
            // TODO use XML config?

            ::FT::Name TestOGDName;
            TestOGDName.length(1);
            TestOGDName[0].id=CdmwNamingAndRepository::REPOSITORY_GROUP_ID;
        
            obj = m_replication_mgr->get_object_group_ref_from_name(TestOGDName);

            CdmwNamingAndRepository::RepositoryExt_var repository =
                CdmwNamingAndRepository::RepositoryExt::_narrow(obj.in());
            CDMW_ASSERT(!CORBA::is_nil(repository.in()));

            std::string application_name =
                Cdmw::PlatformMngt::PlatformInterface::Get_application_name();
        
            try {
                COUT2("create the repository proxy");
                // create the repository proxy
                m_repository = InitUtils::create_repository_proxy(m_orb.in(),
                                                                  rootPOA.in(),
                                                                  application_name,
                                                                  m_processName,
                                                                  repository.in());

                AccessorProxy::set_repository(CdmwNamingAndRepository::RepositoryExt::_duplicate(repository.in()));

            } 
            catch (const CORBA::SystemException& ex) {
                std::cerr << "File: " << __FILE__ 
                          << ", Line: " << __LINE__
                          << ex <<std::endl;
                notifyFatalError("CDMW_REPOSITORY_PROXY", 
                                 "Cannot create the repository proxy");
            }
            catch (const CORBA::Exception & ex) {
                std::cerr << "File: " << __FILE__ 
                          << ", Line: " << __LINE__
                          << ex << std::endl;
                notifyFatalError("CDMW_REPOSITORY_PROXY", 
                                 "Cannot create the repository proxy");
            }

                              
            // get the daemon reference                    
            std::ostringstream daemonCorbaloc;
            try {
                daemonCorbaloc 
                    << "corbaloc::localhost:"
                    << m_daemonPort
                    <<  "/" << CdmwPlatformMngt::DAEMON_NAME;
                  
                CORBA::Object_var obj =
                    m_orb->string_to_object (daemonCorbaloc.str().c_str());
            
                if (CORBA::is_nil (obj.in())) {
                        std::cerr << "NamingAndRepositoryProxyBehavior : Daemon local host not found using " 
                                  << daemonCorbaloc.str().c_str()
                                  << std::endl;
                        notifyFatalError("CDMW_REPOSITORY_PROXY", 
                                         "Daemon local host not found");
                    }
        
        
                CdmwPlatformMngt::Daemon_var daemon =
                    CdmwPlatformMngt::Daemon::_narrow (obj.in());
            
                if (CORBA::is_nil (daemon.in())) {
                        std::cerr << "NamingAndRepositoryProxyBehavior : Daemon local host not found using"
                                  << daemonCorbaloc.str().c_str()
                                  << std::endl;
                        notifyFatalError("CDMW_REPOSITORY_PROXY", 
                                         "Daemon local host not found");
                    }
            
                // register the proxy as daemon local service
                daemon->register_local_service(
                                               Cdmw::PlatformMngt::ServiceNames::NAMING_AND_REPOSITORY_SERVICE,
                                               m_repository.in());
            }
            catch (const CORBA::BAD_PARAM& ex) {
                    std::cerr << "NamingAndRepositoryProxyBehavior : Bad parameter during proxy registration in daemon " 
                              << ex
                              << std::endl;
                }
            catch (const CORBA::SystemException& ex) {
                    std::cerr << "NamingAndRepositoryProxyBehavior : Daemon local host cannot be reachable using " 
                              << daemonCorbaloc.str().c_str()
                              << " " << ex
                              << std::endl;
                }
            catch (const CORBA::Exception& ex) {
                    std::cerr << "NamingAndRepositoryProxyBehavior : Daemon local host cannot be reachable using " 
                              << daemonCorbaloc.str().c_str()
                              << " " << ex
                              << std::endl;
                }
        }


        /**
         * Purpose:
         * <p>
         * the behaviour for the
         * IDL:thalesgroup.com/CdmwPlatformMngt/ProcessDelegate/run:1.0
         * operation
         */
        void NamingAndRepositoryProxyBehaviour::on_run()
            throw(CdmwPlatformMngt::ProcessDelegate::NotReadyToRun,
                  CORBA::SystemException)
        {
            COUT2("Run requested");
        }

        /**
         * Purpose:
         * <p>
         * the behaviour for the
         * IDL:thalesgroup.com/CdmwPlatformMngt/ProcessDelegate/stop:1.0
         * operation
         */
        void NamingAndRepositoryProxyBehaviour::on_stop()
            throw(CORBA::SystemException)
        {
            COUT2("Stop requested");

            // shutdown already done in cdmwinit/CdmwProcess_impl
            // m_orb->shutdown(true);

        }


        /**
         * Notifies the fatal error via the platform interface or
         * output in cerr.
         */ 
        void NamingAndRepositoryProxyBehaviour::notifyFatalError(const char* issuer, const char* message)
            throw(CORBA::SystemException)
        {

            if (m_platformManaged) { 
                Cdmw::PlatformMngt::PlatformInterface::Notify_fatal_error(issuer, message);
            }
            else {
                std::cerr << "File: " << __FILE__ 
                          << ", Line: " << __LINE__
                          << "FAILURE : " << message << std::endl;
            }
            
            throw CORBA::INTERNAL(OrbSupport::INTERNAL, CORBA::COMPLETED_NO);
            
        }

    } // End namespace NamingAndRepository
} // End namespace Cdmw
