/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#include "Foundation/orbsupport/ExceptionMinorCodes.hpp"
#include "Foundation/logging/FunctionLogger.hpp"
#include "Foundation/commonsvcs/naming/NamingInterface.hpp"
#include "Foundation/common/Locations.hpp"
#include "namingandrepositoryproxy/NamingContextProxy.hpp"
#include "namingandrepositoryproxy/NARProxyLogger.hpp"
#include "namingandrepositoryproxy/AccessorProxy.hpp"

#include "TraceAndPerf/tracelibrary/CdmwTrace.hpp"

#include "tao/Stub.h"

namespace Cdmw
{
namespace NamingAndRepository
{

/*
* The implementation of NamingContextProxy
*/

NamingContextProxy::NamingContextProxy
(CORBA::ORB_ptr orb,
 PortableServer::POA_ptr parent_poa,
 CdmwNamingAndRepository::RepositoryExt_ptr repos)
    : ProxyUtil(orb, parent_poa, "NamingContextProxyPOA"),
      m_orb(CORBA::ORB::_duplicate(orb)),
      m_repository(CdmwNamingAndRepository::RepositoryExt::_duplicate(repos))
{
    m_poa->set_servant(this);
}

NamingContextProxy::~NamingContextProxy()
{
}

void NamingContextProxy::bind(const CosNaming::Name& n,
                              CORBA::Object_ptr obj)
    throw(CosNaming::NamingContext::NotFound,
          CosNaming::NamingContext::CannotProceed,
          CosNaming::NamingContext::InvalidName,
          CosNaming::NamingContext::AlreadyBound,
          CORBA::SystemException)
{
    CDMW_LOG_FUNCTION(NARProxyLogger::GetLogger(),
                      " n=" << Cdmw::CommonSvcs::Naming::NamingInterface::to_string(n) 
                      << ", id=" << get_id() );
    try{
      AccessorProxy::initialise();
    }
    catch(const CORBA::TRANSIENT& ex)
    {
      CDMW_WARN(NARProxyLogger::GetLogger(),
        "Exception " << ex._name()
        << " in NamingContextProxy bind\n"
        << "A communication failure with the Repository "
        << "may have occured.");
      CDMW_LOG_FUNCTION_EXCEPTION(ex);
      throw;
    }
    root_context()->bind(adjust_path(n), obj);

}


void NamingContextProxy::rebind(const CosNaming::Name& n,
                                CORBA::Object_ptr obj)
    throw(CosNaming::NamingContext::NotFound,
          CosNaming::NamingContext::CannotProceed,
          CosNaming::NamingContext::InvalidName,
          CORBA::SystemException)
{
    CDMW_LOG_FUNCTION(NARProxyLogger::GetLogger(),
                      " n=" << Cdmw::CommonSvcs::Naming::NamingInterface::to_string(n) 
                      << ", id=" << get_id() );
    try{
      AccessorProxy::initialise();
    }
    catch(const CORBA::TRANSIENT& ex)
    {
      CDMW_WARN(NARProxyLogger::GetLogger(),
        "Exception " << ex._name()
        << " in NamingContextProxy rebind\n"
        << "A communication failure with the Repository "
        << "may have occured.");
      CDMW_LOG_FUNCTION_EXCEPTION(ex);
      throw;
    }
    root_context()->rebind(adjust_path(n), obj);

}


void NamingContextProxy::bind_context(const CosNaming::Name& n,
                                      CosNaming::NamingContext_ptr nc)
    throw(CosNaming::NamingContext::NotFound,
          CosNaming::NamingContext::CannotProceed,
          CosNaming::NamingContext::InvalidName,
          CosNaming::NamingContext::AlreadyBound,
          CORBA::SystemException)
{
    CDMW_LOG_FUNCTION(NARProxyLogger::GetLogger(),
                      " n=" << Cdmw::CommonSvcs::Naming::NamingInterface::to_string(n) 
                      << ", id=" << get_id() );
    try{
      AccessorProxy::initialise();
    }
    catch(const CORBA::TRANSIENT& ex)
    {
      CDMW_WARN(NARProxyLogger::GetLogger(),
        "Exception " << ex._name()
        << " in NamingContextProxy bind_context\n"
        << "A communication failure with the Repository "
        << "may have occured.");
      CDMW_LOG_FUNCTION_EXCEPTION(ex);
      throw;
    }
    root_context()->bind_context(adjust_path(n), nc);

}


void NamingContextProxy::rebind_context(const CosNaming::Name& n,
                            CosNaming::NamingContext_ptr nc)
    throw(CosNaming::NamingContext::NotFound,
          CosNaming::NamingContext::CannotProceed,
          CosNaming::NamingContext::InvalidName,
          CORBA::SystemException)
{
    CDMW_LOG_FUNCTION(NARProxyLogger::GetLogger(),
                      " n=" << Cdmw::CommonSvcs::Naming::NamingInterface::to_string(n) 
                      << ", id=" << get_id() );
    try{
      AccessorProxy::initialise();
    }
    catch(const CORBA::TRANSIENT& ex)
    {
      CDMW_WARN(NARProxyLogger::GetLogger(),
        "Exception " << ex._name()
        << " in NamingContextProxy rebind_context\n"
        << "A communication failure with the Repository "
        << "may have occured.");
      CDMW_LOG_FUNCTION_EXCEPTION(ex);
      throw;
    }
    root_context()->rebind_context(adjust_path(n), nc);

}


CORBA::Object_ptr NamingContextProxy::resolve(const CosNaming::Name& n)
    throw(CosNaming::NamingContext::NotFound,
          CosNaming::NamingContext::CannotProceed,
          CosNaming::NamingContext::InvalidName,
          CORBA::SystemException)
{
    CDMW_LOG_FUNCTION(NARProxyLogger::GetLogger(),
                      " n=" << Cdmw::CommonSvcs::Naming::NamingInterface::to_string(n) 
                      << ", id=" << get_id() );
    try{
      AccessorProxy::initialise();
    }
    catch(const CORBA::TRANSIENT& ex)
    {
      CDMW_WARN(NARProxyLogger::GetLogger(),
        "Exception " << ex._name()
        << " in NamingContextProxy resolve\n"
        << "A communication failure with the Repository "
        << "may have occured.");
      CDMW_LOG_FUNCTION_EXCEPTION(ex);
      throw;
    }
    CosNaming::Name name(adjust_path(n));

    return check_for_context(root_context()->resolve(name), name);
}


void NamingContextProxy::unbind(const CosNaming::Name& n)
    throw(CosNaming::NamingContext::NotFound,
          CosNaming::NamingContext::CannotProceed,
          CosNaming::NamingContext::InvalidName,
          CORBA::SystemException)
{
    CDMW_LOG_FUNCTION(NARProxyLogger::GetLogger(),
                      " n=" << Cdmw::CommonSvcs::Naming::NamingInterface::to_string(n) 
                      << ", id=" << get_id() );
    try{
      AccessorProxy::initialise();
    }
    catch(const CORBA::TRANSIENT& ex)
    {
      CDMW_WARN(NARProxyLogger::GetLogger(),
        "Exception " << ex._name()
        << " in NamingContextProxy unbind\n"
        << "A communication failure with the Repository "
        << "may have occured.");
      CDMW_LOG_FUNCTION_EXCEPTION(ex);
      throw;
    }
    root_context()->unbind(adjust_path(n));

}


CosNaming::NamingContext_ptr NamingContextProxy::new_context()
    throw(CORBA::SystemException)
{
    CDMW_LOG_FUNCTION(NARProxyLogger::GetLogger(), " id=" << get_id() );
    try{
      AccessorProxy::initialise();
    }
    catch(const CORBA::TRANSIENT& ex)
    {
      CDMW_WARN(NARProxyLogger::GetLogger(),
        "Exception " << ex._name()
        << " in NamingContextProxy new_context\n"
        << "A communication failure with the Repository "
        << "may have occured.");
      CDMW_LOG_FUNCTION_EXCEPTION(ex);
      throw;
    }
    try
    {
    CosNaming::NamingContext_var nc = root_context()->new_context();
    
    // Obtain id from context
    CdmwNamingAndRepository::ProxyFeatureNamingContextExt_var cdmwCtx 
    = CdmwNamingAndRepository::ProxyFeatureNamingContextExt::_narrow(nc.in());
    
    if (!CORBA::is_nil(cdmwCtx.in()))
    {
        MapEntry& entry = m_map[get_id()];
        CosNaming::Name name;

        name.length(m_base_name.length() + 1);

        for (CORBA::ULong i = 0; i < m_base_name.length(); i++)
        {
            name[i] = m_base_name[i];
        }

        name[m_base_name.length()].id = cdmwCtx->get_id();

        entry.parent->resolve_root_context
        (Common::Locations::ADMIN_ROOT_CONTEXT_ID)->bind_context(name, nc.in());

        nc = create(nc.in(), entry.parent.in(), Common::Locations::ADMIN_ROOT_CONTEXT_ID, name);
    }

    return nc._retn();
    }
    catch (const CORBA::Exception& ex)
    {
        CDMW_WARN(NARProxyLogger::GetLogger(),
                  "Exception: " << ex._name() << " in NamingContextProxy new_context" );
        CDMW_LOG_FUNCTION_EXCEPTION(ex);

        throw;
    }
    catch (...)
    {
        CDMW_WARN(NARProxyLogger::GetLogger(),
                  "Exception in NamingContextProxy new_context" );
        throw;
    }

}


CosNaming::NamingContext_ptr NamingContextProxy::bind_new_context
(const CosNaming::Name& n)
    throw(CosNaming::NamingContext::NotFound,
          CosNaming::NamingContext::AlreadyBound,
          CosNaming::NamingContext::CannotProceed,
          CosNaming::NamingContext::InvalidName,
          CORBA::SystemException)
{
    CDMW_LOG_FUNCTION(NARProxyLogger::GetLogger(), " id=" << get_id() );
    try{
      AccessorProxy::initialise();
    }
    catch(const CORBA::TRANSIENT& ex)
    {
      CDMW_WARN(NARProxyLogger::GetLogger(),
        "Exception " << ex._name()
        << " in NamingContextProxy bind_new_context\n"
        << "A communication failure with the Repository "
        << "may have occured.");
      CDMW_LOG_FUNCTION_EXCEPTION(ex);
      throw;
    }

    CosNaming::NamingContext_var nc;
    CosNaming::Name name;

    nc = root_context()->bind_new_context(adjust_path(n));

    MapEntry& entry = m_map[get_id()];
    CORBA::ULong len = entry.name.length();

    name.length(len + n.length());

    for (CORBA::ULong i = 0; i < len; i++)
    {
        name[i] = entry.name[i];
    }

    for (CORBA::ULong j = 0; j < n.length(); j++)
    {
        name[len + j] = n[j];
    }

    return create(nc.in(), entry.parent.in(), entry.id.c_str(), name);
}


void NamingContextProxy::destroy()
    throw(CosNaming::NamingContext::NotEmpty,
          CORBA::SystemException)
{
    CDMW_LOG_FUNCTION(NARProxyLogger::GetLogger(), " id=" << get_id() );

    RETRY(destroy());

    m_map.erase(get_id());
}



void NamingContextProxy::list(CORBA::ULong how_many,
                              CosNaming::BindingList_out bl,
                              CosNaming::BindingIterator_out bi)
    throw(CORBA::SystemException)
{
    CDMW_LOG_FUNCTION(NARProxyLogger::GetLogger(), " how_many=" << how_many << ", id=" << get_id() );

    RETRY(list(how_many, bl, bi))

}


CosNaming::NamingContextExt::StringName NamingContextProxy::to_string(
    const CosNaming::Name& n)
    throw(CosNaming::NamingContext::InvalidName,
          CORBA::SystemException)
{

    CDMW_LOG_FUNCTION(NARProxyLogger::GetLogger(),
                      " n=" << Cdmw::CommonSvcs::Naming::NamingInterface::to_string(n) 
                      << ", id=" << get_id() );

    // Do locally as this used by the debug trace

    try
    {

        try
        {
            std::string str;

            try
            {
                str = Cdmw::CommonSvcs::Naming::NamingInterface::to_string(n);
            }
            catch(const Cdmw::CommonSvcs::Naming::InvalidNameException &)
            {
                throw CosNaming::NamingContext::InvalidName();
            }

            char *result = CORBA::string_dup(str.c_str());

            return result;

        }
        catch(const std::bad_alloc &)
        {
            CDMW_WARN(NARProxyLogger::GetLogger(),
                      "std::bad_alloc in NamingContextProxy to_string, current id:" << get_id() );
            throw CORBA::NO_RESOURCES(OrbSupport::NO_RESOURCES, CORBA::COMPLETED_NO);
        }

    }
    catch(const CORBA::Exception& e)
    {

        CDMW_WARN(NARProxyLogger::GetLogger(),
                  "Exception: " << e._name() << " in NamingContextProxy to_string, current id:" << get_id() );
        CDMW_LOG_FUNCTION_EXCEPTION(e);

        throw;

    }
}


CosNaming::Name* NamingContextProxy::to_name(const char* sn)
    throw(CosNaming::NamingContext::InvalidName,
          CORBA::SystemException)
{
    CDMW_LOG_FUNCTION(NARProxyLogger::GetLogger(),
                      " sn=" << sn << ", id=" << get_id() );
    try{
      AccessorProxy::initialise();
    }
    catch(const CORBA::TRANSIENT& ex)
    {
      CDMW_WARN(NARProxyLogger::GetLogger(),
        "Exception " << ex._name()
        << " in NamingContextProxy to_name\n"
        << "A communication failure with the Repository "
        << "may have occured.");
      CDMW_LOG_FUNCTION_EXCEPTION(ex);
      throw;
    }
    return CosNaming::NamingContextExt::_narrow(root_context())->to_name(sn);
}


CosNaming::NamingContextExt::URLString NamingContextProxy::to_url(
        const char* addr, const char* sn)
    throw(CosNaming::NamingContextExt::InvalidAddress,
          CosNaming::NamingContext::InvalidName,
          CORBA::SystemException)
{
    CDMW_LOG_FUNCTION(NARProxyLogger::GetLogger(),
                      " addr=" << addr << ", sn=" << sn << ", id=" << get_id() );
    try{
      AccessorProxy::initialise();
    }
    catch(const CORBA::TRANSIENT& ex)
    {
      CDMW_WARN(NARProxyLogger::GetLogger(),
        "Exception " << ex._name()
        << " in NamingContextProxy to_url\n"
        << "A communication failure with the Repository "
        << "may have occured.");
      CDMW_LOG_FUNCTION_EXCEPTION(ex);
      throw;
    }
    return CosNaming::NamingContextExt::_narrow(root_context())->to_url(addr, sn);
}


CORBA::Object_ptr NamingContextProxy::resolve_str(const char* sn)
    throw(CosNaming::NamingContext::NotFound,
          CosNaming::NamingContext::CannotProceed,
          CosNaming::NamingContext::InvalidName,
          CORBA::SystemException)
{
    CDMW_LOG_FUNCTION(NARProxyLogger::GetLogger(),
                      " sn=" << sn << ", id=" << get_id() );
    try{
      AccessorProxy::initialise();
    }
    catch(const CORBA::TRANSIENT& ex)
    {
      CDMW_WARN(NARProxyLogger::GetLogger(),
        "Exception " << ex._name()
        << " in NamingContextProxy resolve_str\n"
        << "A communication failure with the Repository "
        << "may have occured.");
      CDMW_LOG_FUNCTION_EXCEPTION(ex);
      throw;
    }

    CosNaming::Name name = m_map[get_id()].name;
    CosNaming::NamingContextExt::StringName_var path;

    if (name.length() > 0)
    {
        CosNaming::NamingContextExt::StringName_var base = to_string(name);
        path = CORBA::string_alloc(strlen(base) + strlen(sn) + 1);

        strcpy(path, base);
        strcat(path, "/");
        strcat(path, sn);
    }
    else
    {
        path = sn;
    }

    return CosNaming::NamingContextExt::_narrow(root_context())->resolve_str(path);
}

CosNaming::NamingContext_ptr
NamingContextProxy::create(CosNaming::NamingContext_ptr proxied,
                           CdmwNamingAndRepository::RepositoryExt_ptr parent,
                           const char* id,
                           CosNaming::Name name)
{
    CORBA::ULong new_id = next_id();
    PortableServer::ObjectId oid;
    MapEntry& entry = m_map[new_id];
    CORBA::Object_var obj;

    entry.proxied = CosNaming::NamingContextExt::_narrow(proxied);
    entry.parent = CdmwNamingAndRepository::RepositoryExt::_duplicate(parent);
    entry.id = id;
    entry.name = name;

    oid.length(sizeof(CORBA::ULong));

    for (CORBA::ULong i = 0; i < sizeof(CORBA::ULong); i++)
    {
        oid[i] = new_id & 0xFF;
        new_id >>= 8;
    }

    obj = m_poa->create_reference_with_id
        (oid, "IDL:omg.org/CosNaming/NamingContextExt:1.0");

    return CosNaming::NamingContext::_narrow(obj.in());
}

void NamingContextProxy::set_base_name(const CosNaming::Name& base_name)
{
    m_base_name = base_name;
}

CosNaming::NamingContext_ptr NamingContextProxy::root_context()
    throw (CORBA::SystemException)
{
    CDMW_LOG_FUNCTION(NARProxyLogger::GetLogger(),
                      " id=" << get_id() );
    try{
      AccessorProxy::initialise();
    }
    catch(const CORBA::TRANSIENT& ex)
    {
      CDMW_WARN(NARProxyLogger::GetLogger(),
        "Exception " << ex._name()
        << " in NamingContextProxy resolve_str\n"
        << "A communication failure with the Repository "
        << "may have occured.");
      CDMW_LOG_FUNCTION_EXCEPTION(ex);
      throw;
    }   
    MapEntry& entry = m_map[get_id()];
    return entry.parent->resolve_root_context(entry.id.c_str());
}

CosNaming::Name NamingContextProxy::adjust_path(const CosNaming::Name& n)
{
    CosNaming::Name result = m_map[get_id()].name;
    CORBA::ULong len = result.length();

    result.length(len + n.length());

    for (CORBA::ULong i = 0; i < n.length(); i++)
    {
        result[len + i] = n[i];
    }

    return result;
}

CosNaming::NamingContextExt_ptr NamingContextProxy::get_object()
    throw (CORBA::SystemException)
{
    NamingContextMap::iterator iter = m_map.find(get_id());

    if (iter != m_map.end())
    {
        return iter->second.proxied.in();
    }
    else
    {
        // TODO use Cardamom mechanism
        throw CORBA::OBJECT_NOT_EXIST();
    }
}

CORBA::Object_ptr NamingContextProxy::check_for_context
(CORBA::Object_ptr obj,
 const CosNaming::Name& n)
{
    // get Object's RepositoryId using TAO specific operations
    //const char* 
   std::string  repId = ::Cdmw::OrbSupport::OrbSupport::get_repository_id_from_ior(m_orb.in(),obj);//obj->_stubobj()->type_id.in();
    
    // Check if Object's RepositoryId corresponds to a NamingContext
/*    if ( (strcmp(repId, "IDL:thalesgroup.com/CdmwNamingAndRepository/ProxyFeatureNamingContextExt:1.0") == 0) ||
         (strcmp(repId, "IDL:omg.org/CosNaming/NamingContextExt:1.0") == 0) ||
         (strcmp(repId, "IDL:omg.org/CosNaming/NamingContext:1.0") == 0)
        )*/
    if ( (0 == repId.compare("IDL:thalesgroup.com/CdmwNamingAndRepository/ProxyFeatureNamingContextExt:1.0") ) ||
         (0 == repId.compare("IDL:omg.org/CosNaming/NamingContextExt:1.0") ) ||
         (0 == repId.compare("IDL:omg.org/CosNaming/NamingContext:1.0") )
        )
    {
        // Object is a NamingContext. 
        // Create a corresponding NamingContextProxy and return it.
        CosNaming::NamingContextExt_var nc =
            CosNaming::NamingContextExt::_narrow(obj);
        MapEntry& entry = m_map[get_id()];

        return create(nc.in(), entry.parent.in(), entry.id.c_str(), n);
    }
    else
    {
        // Not a NamingContext, return object as it is
        return obj;
    }
}

void NamingContextProxy::reconstruct() throw (CORBA::SystemException)
{
    MapEntry& entry = m_map[get_id()];
    CosNaming::NamingContext_var root_context =
        entry.parent->resolve_root_context(entry.id.c_str());
    CORBA::Object_var obj;

    if (entry.name.length() > 0)
    {
        entry.proxied = CosNaming::NamingContextExt::_narrow
            (root_context->resolve(entry.name));
    }
    else
    {
        entry.proxied =
            CosNaming::NamingContextExt::_narrow(root_context.in());
    }
}

} // End of namespace NamingAndRepository
} // End of namespace Cdmw

