/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#include <Repository/tools/NameServiceAdmin.hpp>
#include "tools/NSUtils.hpp"
#include <Foundation/commonsvcs/naming/NamingInterface.hpp>
#include <Foundation/commonsvcs/naming/NamingUtil.hpp>
#include <Foundation/common/Assert.hpp>
#include <Foundation/ossupport/OS.hpp>
#include <Foundation/orbsupport/OrbSupport.hpp>
#include <string>
#include <cctype>
#include <algorithm>
#include <fstream>
#include <sstream>


#define ECHO_CMD if (m_verbose) os 


namespace
{
    const char* INFO_MESSAGE = "Type h or help for a list of available commands.";
    const char* PROMPT_MESSAGE = "\nNS-ADMIN>";
    const char* UNKNOWN_COMMAND_MESSAGE = "Bad command!>";

    const CORBA::Long EXIT_ORDERED(-10);
    const CORBA::Long OP_SUCCESS = 0;
    const CORBA::Long OP_FAILURE = -1;

    
    bool not_space(char c)
    { return !isspace(c);}

   
   bool 
   get_ior_name(const std::string & arg, std::ostream & os,
                std::string & name, std::string & ior) throw()
   {
      bool result = false;

      // arg contains the following:
      // ior name
      
      // ===============
      // Extract ior        
      // ===============
      std::string::const_iterator i = arg.begin();
      std::string::const_iterator end = arg.end();
      // find end of ior
      std::string::const_iterator j = std::find_if(i,end,isspace);
      // copy the characters in [i,j)
      if (i != end)
         ior = std::string(i,j);
      else {
         os << "Object's IOR and name are missing." << std::endl;
         return result;
      }
      // =================
      // Extract name
      // =================
      // skip leading whitespaces
      i = std::find_if(j,end,not_space);
      // copy arguments
      if (i != end)
         name = std::string(i,end);
      else {
         os << "Object's name is missing." << std::endl;
         return result;
      }
      return true;
   }
}; // End anonymous namespace

namespace Cdmw
{
namespace Tools
{

//----------------------------------------------------------------------------
NameServiceAdmin::NameServiceAdmin(CORBA::ORB_ptr orb, 
                                   CosNaming::NamingContext_ptr root)
    throw (CORBA::SystemException)
    : m_orb(CORBA::ORB::_duplicate(orb)),
      m_root(CosNaming::NamingContext::_duplicate(root)),
      m_initial_root(CosNaming::NamingContext::_duplicate(root)),
      m_verbose(false),
      m_force(false)
{
    CDMW_ASSERT(!CORBA::is_nil(orb));
    CDMW_ASSERT(!CORBA::is_nil(root));
}

//----------------------------------------------------------------------------
NameServiceAdmin::~NameServiceAdmin() throw()
{
}
//----------------------------------------------------------------------------
const CORBA::ULong NameServiceAdmin::nb_commands = 28;

// WARNING:
// NameServiceAdmin::commands should be sorted alphabetically (by operation name)
const NameServiceAdmin::command_def NameServiceAdmin::commands[] = 
{ 
    { "bind"             , &NameServiceAdmin::bind             },
    { "bind_context"     , &NameServiceAdmin::bind_context     },
    { "bind_new_context" , &NameServiceAdmin::bind_new_context },
    { "chroot"           , &NameServiceAdmin::chroot           },
    { "destroy"          , &NameServiceAdmin::destroy          },
    { "destroy_all"      , &NameServiceAdmin::destroy_all      },
    { "exit"             , &NameServiceAdmin::exit             },
    { "force"            , &NameServiceAdmin::toggle_force     },
    { "h"                , &NameServiceAdmin::print_help       },
    { "help"             , &NameServiceAdmin::print_help       },
    { "list"             , &NameServiceAdmin::list             },
    { "load"             , &NameServiceAdmin::load_file        },
    { "obj::get_component" , &NameServiceAdmin::obj_get_component  },
    { "obj::hash"          , &NameServiceAdmin::obj_hash           },
    { "obj::is_a"          , &NameServiceAdmin::obj_is_a           },
    { "obj::is_equivalent" , &NameServiceAdmin::obj_is_equivalent  },
    { "obj::is_nil"        , &NameServiceAdmin::obj_is_nil         },
    { "obj::non_existent"  , &NameServiceAdmin::obj_non_existent   },
    { "obj::repository_id" , &NameServiceAdmin::obj_repository_id  },
    { "ping"             , &NameServiceAdmin::obj_non_existent     },
    { "rebind"           , &NameServiceAdmin::rebind           },
    { "rebind_context"   , &NameServiceAdmin::rebind_context   },
    { "reset"            , &NameServiceAdmin::reset            },
    { "resolve"          , &NameServiceAdmin::resolve          },
    { "system"           , &NameServiceAdmin::system           },
    { "unbind"           , &NameServiceAdmin::unbind           },
    { "verbose"          , &NameServiceAdmin::toggle_verbose   },
    { "x"                , &NameServiceAdmin::exit             }
};

    
//----------------------------------------------------------------------------
CORBA::Long NameServiceAdmin::runOperation(const std::string & op,
                                          const std::string & arg, 
                                          std::ostream &      os) 
    throw()
{
    CORBA::Long l = 0;
    CORBA::Long r = (CORBA::Long)nb_commands;
    CORBA::Long m;
    CORBA::Long res;
    
    while(l < r)
        {
            m = (l + r) / 2;
            res = strcmp(commands[m].name, op.c_str());
            if(res == 0)
                return (this->*commands[m].op)(arg,os);
            if(res > 0)
                r = m;
            else
                l = m + 1;
        }
    if(op != "") unknown_command(os);
    
    return OP_FAILURE;
}
//----------------------------------------------------------------------------
void NameServiceAdmin::print_info(std::ostream& os) const throw()
{
    os << "\n\n"
       << "\t\t--------------------------------------------------\n"
       << "\t\t Cdmw Name Service administration console         \n"
       << "\t\t--------------------------------------------------\n"
       << "\n\n"
       << INFO_MESSAGE
       << "\nInfo: (verbose mode is " << (m_verbose ? "on" : "off") << ")"
       << "\n      (force mode is " << (m_force ? "on" : "off") << ")"
       << std::endl;
}
//----------------------------------------------------------------------------
void NameServiceAdmin::print_prompt(std::ostream& os) const throw()
{
    os << PROMPT_MESSAGE << std::flush;
}
//----------------------------------------------------------------------------
void NameServiceAdmin::unknown_command( std::ostream & os) const throw()
{
    os << UNKNOWN_COMMAND_MESSAGE << std::endl;
}
//----------------------------------------------------------------------------
void NameServiceAdmin::run(std::istream& is,std::ostream& os) throw()
{
    print_info(os);
    print_prompt(os);

    std::string s;
    while(std::getline(is,s)) {  
        // ===============
        // Extract command        
        // ===============
        // skip leading whitespaces
        std::string::const_iterator i = s.begin();
        std::string::const_iterator end = s.end();
        i = std::find_if(i,end,not_space);
        // find end of command
        std::string::const_iterator j = std::find_if(i,end,isspace);
        // copy the characters in [i,j)
        std::string command;
        if (i != end)
            command = std::string(i,j);
        // =================
        // Extract arguments
        // =================
        // skip leading whitespaces
        i = std::find_if(j,end,not_space);
        std::string arg;
        // copy arguments
        if (i != end)
            arg = std::string(i,end);
        
        ECHO_CMD << "@cmd = <" << command << "> @args = <" << arg << ">" << std::endl;
        if (runOperation(command,arg,os) == EXIT_ORDERED) break; 
        print_prompt(os); 
    } 
}
//----------------------------------------------------------------------------
CORBA::Long NameServiceAdmin::exit(const std::string & arg, std::ostream & os) throw()
{    
    return EXIT_ORDERED;
}
//----------------------------------------------------------------------------
CORBA::Long NameServiceAdmin::toggle_verbose(const std::string & arg, std::ostream & os) throw()
{    
    m_verbose = !m_verbose;
    os << "(verbose mode is " << (m_verbose ? "on" : "off") << ")"
       << std::endl;
    return OP_SUCCESS;
}
//----------------------------------------------------------------------------
CORBA::Long NameServiceAdmin::toggle_force(const std::string & arg, std::ostream & os) throw()
{    
    m_force = !m_force;
    os << "(force mode is " << (m_force ? "on" : "off") << ")"
       << std::endl;
    return OP_SUCCESS;
}
//----------------------------------------------------------------------------
// Available commands:
//
CORBA::Long NameServiceAdmin::print_help(const std::string & arg, std::ostream & os) throw()
{
   print_info(os);
   std::ostringstream nm_ostr, obj_ostr, extra_ostr;
   nm_ostr << "\n"
           << "Available commands:" << "\n"
           << "\n"
           << " +==================+============================================+" << "\n"
           << " | COMMAND          | DESCRIPTION                                |" << "\n"
           << " +==================+============================================+" << "\n"
           << " | x | exit         | exit ADMIN mode                            |" << "\n"
           << " +------------------+--------------------------------------------+" << "\n"
           << " | h | help [obj|extra|all]                                      |" << "\n"
           << " |                  | print available commands                   |" << "\n"
           << " +------------------+--------------------------------------------+" << "\n"
           << " | bind ior name    | Bind object whose object reference is 'ior'|" << "\n"
           << " |                  | to 'name' (fully qualified)                |" << "\n"
           << " +------------------+--------------------------------------------+" << "\n"
           << " | bind_context ior name                                         |" << "\n"
           << " |                  | Bind object whose object reference is 'ior'|" << "\n"
           << " |                  | to 'name' (fully qualified)                |" << "\n"
           << " +------------------+--------------------------------------------+" << "\n"
           << " | bind_new_context name                                         |" << "\n"
           << " |                  | This creates a new context, binds it to the|" << "\n"
           << " |                  | name supplied as an argument, and wraps it |" << "\n"
           << " |                  | in a NamingInterface object.               |" << "\n"
           << " |                  | The newly-created context is implemented by|" << "\n"
           << " |                  | the same naming server as the context in   |" << "\n"
           << " |                  | which it was bound.                        |" << "\n"
           << " +------------------+--------------------------------------------+" << "\n"
           << " | chroot name|ior  | This changes root context to 'name' naming |" << "\n"
           << " |                  | context or to specified ior.               |" << "\n"
           << " +------------------+--------------------------------------------+" << "\n"
           << " | destroy name     | This deletes internal naming context. If   |" << "\n"
           << " |                  | there are bindings denoting the destroyed  |" << "\n"
           << " |                  | context, these bindings are not removed.   |" << "\n"
           << " +------------------+--------------------------------------------+" << "\n"
           << " | destroy_all name | This deletes internal naming context and,  |" << "\n"
           << " |                  | recursively, its bindings, and child       |" << "\n"
           << " |                  | contexts. If there are bindings denoting   |" << "\n"
           << " |                  | the destroyed context, these bindings are  |" << "\n"
           << " |                  | not removed. The context is destroyed only |" << "\n"
           << " |                  | if all its internal bindings are removed   |" << "\n"
           << " |                  | and internal contexts destroyed, otherwise |" << "\n"
           << " |                  | only undestroyed contexts will remain.     |" << "\n"
           << " +------------------+--------------------------------------------+" << "\n"
           << " | load filename    | Load 'filename' batch file                 |" << "\n"
           << " +------------------+--------------------------------------------+" << "\n"
           << " | list [context] [depth]                                        |" << "\n"
           << " |                  | list content of 'context'                  |" << "\n"
           << " |                  | (default list all root context)            |" << "\n"
           << " |                  | NOTA: use '.' to specify depth for root    |" << "\n"
           << " |                  |       context.                             |" << "\n"
           << " +------------------+--------------------------------------------+" << "\n"
           << " | rebind ior name  | Rebind object whose object reference is    |" << "\n"
           << " |                  | 'ior' to 'name' (fully qualified)          |" << "\n"
           << " +------------------+--------------------------------------------+" << "\n"
           << " | rebind_context ior name                                       |" << "\n"
           << " |                  | Bind object whose object reference is 'ior'|" << "\n"
           << " |                  | to 'name' (fully qualified)                |" << "\n"
           << " +------------------+--------------------------------------------+" << "\n"
           << " | reset            | Changes root to initial naming context.    |" << "\n"
           << " +------------------+--------------------------------------------+" << "\n"
           << " | resolve name     | This retrieves the object bound to 'name'  |" << "\n"
           << " |                  | in a given context. The name must exactly  |" << "\n"
           << " |                  | match the bound name.                      |" << "\n"
           << " +------------------+--------------------------------------------+" << "\n"
           << " | unbind name      | remove 'name' binding from a context       |" << "\n"
           << " +------------------+--------------------------------------------+" << "\n"
           << " | verbose          | toggle verbose mode on/off                 |" << "\n"
           << " +------------------+--------------------------------------------+" << "\n"
           << " | force            | toggle force mode on/off                   |" << "\n"
           << " |                  | Forces creation of missing nodes if on.    |" << "\n"
           << " |                  | Used for bind, rebind, bind_context,       |" << "\n"
           << " |                  | rebind_context, rebind_context, and        |" << "\n"
           << " |                  | bind_new_context.                          |" << "\n"
           << " +------------------+--------------------------------------------+" << "\n";

   obj_ostr << "\n"
            << "Some CORBA Object operations\n"
            << " +==================+============================================+" << "\n"
            << " | COMMAND          | DESCRIPTION                                |" << "\n"
            << " +==================+============================================+" << "\n"
            << " | obj::get_component name                                       |" << "\n"
            << " |                  | If the target object reference is itself a |" << "\n"
            << " |                  | component reference (i.e., it denotes the  |" << "\n"
            << " |                  | component itself), the get_component       |" << "\n"
            << " |                  | operation returns the same reference (or   |" << "\n"
            << " |                  | another equivalent reference).             |" << "\n"
            << " |                  | If the target object reference is a facet  |" << "\n"
            << " |                  | reference the get_component operation      |" << "\n"
            << " |                  | returns an object reference for the        |" << "\n"
            << " |                  | component. If the target reference is      |" << "\n"
            << " |                  | neither a component reference nor a        |" << "\n"
            << " |                  | provided reference, get_component returns  |" << "\n"
            << " |                  | a nil reference.                           |" << "\n"
            << " |                  |                                            |" << "\n"
            << " |                  | NOT IMPLEMENTED!                           |" << "\n"
            << " +------------------+--------------------------------------------+" << "\n"
            << " | obj::hash name maximum                                        |" << "\n"
            << " |                  | The maximum parameter to the hash operation|" << "\n"
            << " |                  | specifies an upper bound on the hash value |" << "\n"
            << " |                  | returned by the ORB. The lower bound of    |" << "\n"
            << " |                  |  that value is zero.                       |" << "\n"
            << " +------------------+--------------------------------------------+" << "\n"
            << " | obj::is_a name type-id                                        |" << "\n"
            << " |                  | The type-id is a string denoting a shared  |" << "\n"
            << " |                  | type identifier (RepositoryId).            |" << "\n"
            << " |                  | The operation returns true if the object is|" << "\n"
            << " |                  | really an instance of that type, including |" << "\n"
            << " |                  | if that type is an ancestor of the         |" << "\n"
            << " |                  | 'most derived' type of that object.        |" << "\n"
            << " |                  | Determining whether an object's type is    |" << "\n"
            << " |                  | compatible with the logical_type_id may    |" << "\n"
            << " |                  | require contacting a remote ORB or         |" << "\n"
            << " |                  | interface repository.                      |" << "\n"
            << " +------------------+--------------------------------------------+" << "\n"
            << " | obj::is_equivalent name name|ior                              |" << "\n"
            << " |                  | The is_equivalent operation is used to     |" << "\n"
            << " |                  | determine if two object references are     |" << "\n"
            << " |                  | equivalent, so far as the ORB can easily   |" << "\n"
            << " |                  | determine. It returns TRUE if the target   |" << "\n"
            << " |                  | object reference is known to be equivalent |" << "\n"
            << " |                  | to the other object reference passed as its|" << "\n"
            << " |                  | parameter, and FALSE otherwise.            |" << "\n"
            << " |                  | If two object references are identical,    |" << "\n"
            << " |                  | they are equivalent. Two different object  |" << "\n"
            << " |                  | references that in fact refer to the same  |" << "\n"
            << " |                  | object are also equivalent.                |" << "\n"
            << " |                  |                                            |" << "\n" 
            << " |                  | NOT IMPLEMENTED!                           |" << "\n" 
            << " +------------------+--------------------------------------------+" << "\n"
            << " | obj::is_nil name | An object reference whose value is         |" << "\n"
            << " |                  | OBJECT_NIL denotes no object. An object    |" << "\n"
            << " |                  | reference can be tested for this value by  |" << "\n"
            << " |                  | the is_nil operation.                      |" << "\n"
            << " |                  | The object implementation is not involved  |" << "\n"
            << " |                  | in the nil test.                           |" << "\n"
            << " +------------------+--------------------------------------------+" << "\n"
            << " | obj::non_existent name                                        |" << "\n"
            << " |                  | The non_existent operation may be used to  |" << "\n"
            << " |                  | test whether an object (e.g., a proxy      |" << "\n"
            << " |                  | object) has been destroyed. It does this   |" << "\n"
            << " |                  | without invoking any application level     |" << "\n"
            << " |                  | operation on the object, and so will never |" << "\n"
            << " |                  | affect the object itself. It returns true  |" << "\n"
            << " |                  | if the ORB knows authoritatively that the  |" << "\n"
            << " |                  | object does not exist; otherwise, it       |" << "\n"
            << " |                  | returns false.                             |" << "\n"
            << " |                  | Probing for object non-existence may       |" << "\n"
            << " |                  | require contacting the ORB that implements |" << "\n"
            << " |                  | the target object. Such an attempt may fail|" << "\n"
            << " |                  | at either the local or the remote end. If  |" << "\n"
            << " |                  | nonexistent cannot make a reliable         |" << "\n"
            << " |                  | determination of object existence due to   |" << "\n"
            << " |                  | failure, it raises an exception in the     |" << "\n"
            << " |                  | calling application code. This enables the |" << "\n"
            << " |                  | application to distinguish among the true, |" << "\n"
            << " |                  | false, and indeterminate cases.            |" << "\n" 
            << " +------------------+--------------------------------------------+" << "\n"
            << " | obj::repository_id name                                       |" << "\n"
            << " |                  | Returns the repository ID of an object.    |" << "\n"
            << " |                  | The implementation of this operation must  |" << "\n"
            << " |                  | contact the ORB that implements the target |" << "\n"
            << " |                  | object.                                    |" << "\n" 
            << " +------------------+--------------------------------------------+" << "\n";
   
   extra_ostr << "\n"
              << "Some Extra operations\n"
              << " +==================+============================================+" << "\n"
              << " | COMMAND          | DESCRIPTION                                |" << "\n"
              << " +==================+============================================+" << "\n"
              << " | system shell-command [args]                                   |" << "\n"
              << " |                  | system - execute a shell command!          |" << "\n"
              << " |                  | Appends -ORBInitRef NameService=<root ctxt>|" << "\n"
              << " +------------------+--------------------------------------------+" << "\n"
              << " | ping name        | Checks existence of named object using     |" << "\n"
              << " |                  | standard Object::non_existent operation    |" << "\n"
              << " +------------------+--------------------------------------------+" << "\n";
   os << nm_ostr.str();
   if (arg == "obj" || arg == "all")
      os << obj_ostr.str();
   if (arg == "extra" || arg == "all")
      os << extra_ostr.str();
   os << std::endl;
   
   return OP_SUCCESS;
}
//----------------------------------------------------------------------------
CORBA::Long NameServiceAdmin::bind(const std::string & arg, std::ostream & os) throw()
{    
    CORBA::Long result = OP_FAILURE;  

    std::string name, ior;
    if (!get_ior_name(arg, os, name, ior))
       return result;

    try {
       Cdmw::CommonSvcs::Naming::NamingInterface ni(m_root.in());         
       typedef Cdmw::CommonSvcs::Naming::NamingUtil<CosNaming::NamingContext> Util;
       
       CORBA::Object_var obj = m_orb->string_to_object(ior.c_str());

       ni.bind(name,obj.in(),m_force);
       result = OP_SUCCESS;

    } catch (const Cdmw::CommonSvcs::Naming::InvalidNameException & e) {
        os << "Invalid Name : <" << e.what() << ">" << std::endl;
    } catch (const CORBA::Exception & e) {
        os << "CORBA Exception : \n" << e << std::endl;
    }

    return result;
}
//----------------------------------------------------------------------------
CORBA::Long NameServiceAdmin::bind_context(const std::string & arg, std::ostream & os) throw()
{    
   CORBA::Long result = OP_FAILURE;
    std::string name, ior;
    if (!get_ior_name(arg, os, name, ior))
       return result;

    try {
       Cdmw::CommonSvcs::Naming::NamingInterface ni(m_root.in());         
       typedef Cdmw::CommonSvcs::Naming::NamingUtil<CosNaming::NamingContext> Util;
       
       CORBA::Object_var obj = m_orb->string_to_object(ior.c_str());
       CosNaming::NamingContext_var nc = CosNaming::NamingContext::_narrow(obj.in());
       if (CORBA::is_nil(nc.in())) {
          os << "IOR not a naming context!" << std::endl;
          return result;
       }
       ni.bind_context(name,nc.in(),m_force);
       result = OP_SUCCESS;
       
    } catch (const Cdmw::CommonSvcs::Naming::InvalidNameException & e) {
       os << "Invalid Name : <" << e.what() << ">" << std::endl;
    } catch (const CORBA::Exception & e) {
       os << "CORBA Exception : \n" << e << std::endl;
    }
    return result;
}
//----------------------------------------------------------------------------
CORBA::Long NameServiceAdmin::bind_new_context(const std::string & arg, std::ostream & os) throw()
{    
   CORBA::Long result = OP_FAILURE;

   try {
      if (arg == "") {
         os << "Name needed!" << std::endl;
      } else {
         Cdmw::CommonSvcs::Naming::NamingInterface ni(m_root.in());
         
         typedef Cdmw::CommonSvcs::Naming::NamingUtil<CosNaming::NamingContext> Util;

         Cdmw::CommonSvcs::Naming::NamingInterface ni__ = ni.bind_new_context(arg,m_force);
         
         CosNaming::NamingContext_var nc__ = ni__.name_context();
         CORBA::String_var s = m_orb->object_to_string(nc__.in());
         os << s.in() << std::endl;
         result = OP_SUCCESS;
      }
   } catch (const Cdmw::CommonSvcs::Naming::InvalidNameException & e) {
      os << "Invalid Name : <" << e.what() << ">" << std::endl;
   } catch (const CosNaming::NamingContext::InvalidName & e) {
      os << "Invalid Name : <" << e << ">" << std::endl;
   } catch (const CORBA::Exception & e) {
      os << "CORBA Exception : \n" << e << std::endl;
   }
   return result;
}
//----------------------------------------------------------------------------
CORBA::Long NameServiceAdmin::destroy(const std::string & arg, std::ostream & os) throw()
{    
   CORBA::Long result = OP_FAILURE; 
   try {
      if (arg == "") {
         os << "Name needed!" << std::endl;
      } else {
         Cdmw::CommonSvcs::Naming::NamingInterface ni(m_root.in());
         
         typedef Cdmw::CommonSvcs::Naming::NamingUtil<CosNaming::NamingContext> Util;
         CORBA::Object_var obj = ni.resolve(arg);
         
         CosNaming::NamingContext_var nc = CosNaming::NamingContext::_narrow(obj.in());
         if (CORBA::is_nil(nc.in())) {
            os << arg << " does not denote a naming context!" << std::endl;
            return result;
         }
         Cdmw::CommonSvcs::Naming::NamingInterface ni__(nc.in());
         ni__.destroy();
         result = OP_SUCCESS;
      }
   } catch (const Cdmw::CommonSvcs::Naming::InvalidNameException & e) {
      os << "Invalid Name : <" << e.what() << ">" << std::endl;
   } catch (const CosNaming::NamingContext::InvalidName & e) {
      os << "Invalid Name : <" << e << ">" << std::endl;
   } catch (const CosNaming::NamingContext::NotEmpty &) {
      os << "Naming context not empty!" << std::endl;
   } catch (const CORBA::Exception & e) {
      os << "CORBA Exception : \n" << e << std::endl;
   }
   return result;
}
//----------------------------------------------------------------------------
CORBA::Long NameServiceAdmin::destroy_all(const std::string & arg, std::ostream & os) throw()
{        
   CORBA::Long result = OP_FAILURE; 
   try {
      if (arg == "") {
         os << "Name needed!" << std::endl;
      } else {
         Cdmw::CommonSvcs::Naming::NamingInterface ni(m_root.in());
         
         typedef Cdmw::CommonSvcs::Naming::NamingUtil<CosNaming::NamingContext> Util;
         CORBA::Object_var obj = ni.resolve(arg);
         
         CosNaming::NamingContext_var nc = CosNaming::NamingContext::_narrow(obj.in());
         if (CORBA::is_nil(nc.in())) {
            os << arg << " does not denote a naming context!" << std::endl;
            return result;
         }
         Cdmw::CommonSvcs::Naming::NamingInterface ni__(nc.in());
         ni__.destroy_all();
         result = OP_SUCCESS;
      }
   } catch (const Cdmw::CommonSvcs::Naming::InvalidNameException & e) {
      os << "Invalid Name : <" << e.what() << ">" << std::endl;
   } catch (const CosNaming::NamingContext::InvalidName & e) {
      os << "Invalid Name : <" << e << ">" << std::endl;
   } catch (const CORBA::Exception & e) {
      os << "CORBA Exception : \n" << e << std::endl;
   }
   return result;
}
//----------------------------------------------------------------------------
CORBA::Long NameServiceAdmin::list(const std::string & arg, std::ostream & os) throw()
{    
   CORBA::Long result = OP_FAILURE;

   std::string name;
   try {
      CosNaming::NamingContext_var nc = CosNaming::NamingContext::_nil();
      int depth = 0;
      
      if (arg == "") {
         nc = m_root;
      } else {
         Cdmw::CommonSvcs::Naming::NamingInterface ni(m_root.in());         
         typedef Cdmw::CommonSvcs::Naming::NamingUtil<CosNaming::NamingContext> Util;
         // Extract name and optinal depth
         // skip leading whitespaces
         std::string::const_iterator i = arg.begin();
         std::string::const_iterator end = arg.end();
         i = std::find_if(i,end,not_space);
         // find end of command
         std::string::const_iterator j = std::find_if(i,end,isspace);
         // copy the characters in [i,j)
         if (i != end)
            name = std::string(i,j);
         // any depth?
         // skip leading whitespaces
         i = std::find_if(j,end,not_space);
         std::string deptharg;
         // copy arguments
         if (i != end) {
            deptharg = std::string(i,end);
            depth = atoi(deptharg.c_str());
         }
         if (name == "." ) // This is to support depth for root context
            nc = m_root;
         else
            nc = Util::resolve_name(ni,name);
      }
      ECHO_CMD << "Listing context with depth='"<< depth<<"'" << std::endl;
      bool list_result  = NSUtils::Show_context(nc.in(), os,0,depth);
      if (list_result)
         result = OP_SUCCESS;
   } catch (const CosNaming::NamingContext::InvalidName & e) {
      os << "InvalidName <" << e << ">" << std::endl;
   } catch (const CosNaming::NamingContext::NotFound & ) {
      os << "Name '" << name << "' not found!" << std::endl;
   } catch (const CORBA::Exception & e) {
      os << "CORBA Exception : \n" << e << std::endl;
   }
   
   return result;
}
//----------------------------------------------------------------------------
CORBA::Long NameServiceAdmin::rebind(const std::string & arg, std::ostream & os) throw()
{        
   CORBA::Long result = OP_FAILURE;  
   
   std::string name, ior;
   if (!get_ior_name(arg, os, name, ior))
      return result;
   
   try {
      Cdmw::CommonSvcs::Naming::NamingInterface ni(m_root.in());         
      typedef Cdmw::CommonSvcs::Naming::NamingUtil<CosNaming::NamingContext> Util;
      
      CORBA::Object_var obj = m_orb->string_to_object(ior.c_str());
      
      ni.rebind(name,obj.in(),m_force);
      result = OP_SUCCESS;
      
   } catch (const Cdmw::CommonSvcs::Naming::InvalidNameException & e) {
      os << "Invalid Name : <" << e.what() << ">" << std::endl;
   } catch (const CORBA::Exception & e) {
      os << "CORBA Exception : \n" << e << std::endl;
   }
   return result;
}
//----------------------------------------------------------------------------
CORBA::Long NameServiceAdmin::rebind_context(const std::string & arg, std::ostream & os) throw()
{    
   CORBA::Long result = OP_FAILURE;
   std::string name, ior;
   if (!get_ior_name(arg, os, name, ior))
      return result;
   
   try {
      Cdmw::CommonSvcs::Naming::NamingInterface ni(m_root.in());         
      typedef Cdmw::CommonSvcs::Naming::NamingUtil<CosNaming::NamingContext> Util;
      
      CORBA::Object_var obj = m_orb->string_to_object(ior.c_str());
      CosNaming::NamingContext_var nc = CosNaming::NamingContext::_narrow(obj.in());
      if (CORBA::is_nil(nc.in())) {
         os << "IOR not a naming context!" << std::endl;
         return result;
      }
      ni.rebind_context(name,nc.in(),m_force);
      result = OP_SUCCESS;
      
   } catch (const Cdmw::CommonSvcs::Naming::InvalidNameException & e) {
      os << "Invalid Name : <" << e.what() << ">" << std::endl;
   } catch (const CORBA::Exception & e) {
      os << "CORBA Exception : \n" << e << std::endl;
   }
   return result;
}
//----------------------------------------------------------------------------
CORBA::Long NameServiceAdmin::resolve(const std::string & arg, std::ostream & os) throw()
{
   CORBA::Long result = OP_FAILURE; 
   try {
      if (arg == "") {
         os << "Name needed!" << std::endl;
      } else {
         Cdmw::CommonSvcs::Naming::NamingInterface ni(m_root.in());
         CORBA::Object_var obj = ni.resolve(arg);
         CORBA::String_var s = m_orb->object_to_string(obj.in());
         os << s.in() << std::endl;
         result = OP_SUCCESS;
      }
   } catch (const Cdmw::CommonSvcs::Naming::InvalidNameException & e) {
      os << "Invalid Name : <" << e.what() << ">" << std::endl;
   } catch (const CosNaming::NamingContext::InvalidName & e) {
      os << "Invalid Name : <" << e << ">" << std::endl;
   } catch (const CORBA::Exception & e) {
      os << "CORBA Exception : \n" << e << std::endl;
   }
   return result;
}
//----------------------------------------------------------------------------
CORBA::Long NameServiceAdmin::reset(const std::string & arg, std::ostream & os) throw()
{
   CORBA::Long result = OP_FAILURE; 
   ECHO_CMD << "Reseting to initial root context." << std::endl;
   m_root = m_initial_root;
   return result;
}
//----------------------------------------------------------------------------
CORBA::Long NameServiceAdmin::chroot(const std::string & arg, std::ostream & os) throw()
{    
   CORBA::Long result = OP_FAILURE;

   try {
      if (arg == "") {
         os << "Name or IOR needed!" << std::endl;
      } else {
         Cdmw::CommonSvcs::Naming::NamingInterface ni(m_root.in());         
         typedef Cdmw::CommonSvcs::Naming::NamingUtil<CosNaming::NamingContext> Util;
         // Is arg an IOR?
         try {
            CORBA::Object_var obj = m_orb->string_to_object(arg.c_str());
            CosNaming::NamingContext_var nc = CosNaming::NamingContext::_narrow(obj.in());
            if (!CORBA::is_nil(nc.in())) {
               // IOR is a naming context!" << std::endl;
               m_root = nc;
               ECHO_CMD << "Changing root context to specified ior." << std::endl;
               result = OP_SUCCESS;
            } else {
               os << "IOR not a naming context!" << std::endl;
            }
         } catch (const CORBA::Exception & ) {
            // Not a valid reference. Let's try if it is a name
            CORBA::Object_var obj = ni.resolve(arg);
            CosNaming::NamingContext_var nc = CosNaming::NamingContext::_narrow(obj.in());
            if (!CORBA::is_nil(nc.in())) {
               // name is a naming context!" << std::endl;
               m_root = nc;
               ECHO_CMD << "Changing root context to specified '"<<arg<<"'." << std::endl;
               result = OP_SUCCESS;
            } else {
               os << "Name '"<<arg<<"' is not a naming context!" << std::endl;
            }
         }
      }
   } catch (const Cdmw::CommonSvcs::Naming::InvalidNameException & e) {
      os << "Invalid Name : <" << e.what() << ">" << std::endl;
   } catch (const CORBA::Exception & e) {
      os << "CORBA Exception : \n" << e << std::endl;
   }
   return result;
}
//----------------------------------------------------------------------------
CORBA::Long NameServiceAdmin::unbind(const std::string & arg, std::ostream & os) throw()
{    
   CORBA::Long result = OP_FAILURE;    
   
   try {
      if (arg == "") {
         os << "Name needed!" << std::endl;
      } else {
         Cdmw::CommonSvcs::Naming::NamingInterface ni(m_root.in());
         
         typedef Cdmw::CommonSvcs::Naming::NamingUtil<CosNaming::NamingContext> Util;
         ni.unbind(arg);
         result = OP_SUCCESS;
      }
   } catch (const Cdmw::CommonSvcs::Naming::InvalidNameException & e) {
      os << "Invalid Name : <" << e.what() << ">" << std::endl;
   } catch (const CosNaming::NamingContext::InvalidName & e) {
      os << "Invalid Name : <" << e << ">" << std::endl;
   } catch (const CORBA::Exception & e) {
      os << "CORBA Exception : \n" << e << std::endl;
   }
   return result;
}

//----------------------------------------------------------------------------
CORBA::Long NameServiceAdmin::load_file(const std::string & arg, std::ostream & os) throw()
{    
    CORBA::Long result = OP_FAILURE;    

    try {
        ECHO_CMD << " Loading batch file \"" << arg << "\"." << std::endl;
        std::ifstream in(arg.c_str());
        
        if (!in)
            os << "Could not open file \"" << arg << "\"." << std::endl;
        else {
            NameServiceAdmin admin(m_orb.in(),m_root.in());
            admin.run(in,os);
            ECHO_CMD << "Batch file \"" << arg << "\" done." << std::endl;
            result = OP_SUCCESS;
        }
    } catch (const CORBA::SystemException & e) {
        os << "CORBA System Exception : \n" << e << std::endl;
    }

    return result;
}
//----------------------------------------------------------------------------
CORBA::Long NameServiceAdmin::obj_is_nil(const std::string & arg, std::ostream & os) throw()
{    
   CORBA::Long result = OP_FAILURE;   

   try {
      if (arg == "") {
         os << "Name needed!" << std::endl;
      } else {
         Cdmw::CommonSvcs::Naming::NamingInterface ni(m_root.in());
         
         typedef Cdmw::CommonSvcs::Naming::NamingUtil<CosNaming::NamingContext> Util;
         CORBA::Object_var obj = ni.resolve(arg);
         if (CORBA::is_nil(obj.in()))
            os << " '" << arg << "' resolves to a nil object reference!" << std::endl;
         else 
            os << " '" << arg << "' resolves to a non-nil object reference!" << std::endl;            
         result = OP_SUCCESS;
      }
   } catch (const Cdmw::CommonSvcs::Naming::InvalidNameException & e) {
      os << "Invalid Name : <" << e.what() << ">" << std::endl;
   } catch (const CosNaming::NamingContext::InvalidName & e) {
      os << "Invalid Name : <" << e << ">" << std::endl;
   } catch (const CORBA::Exception & e) {
      os << "CORBA Exception : \n" << e << std::endl;
   } 
   return result;
}
//----------------------------------------------------------------------------
CORBA::Long NameServiceAdmin::obj_non_existent(const std::string & arg, std::ostream & os) throw()
{    
   CORBA::Long result = OP_FAILURE;

   try {
      if (arg == "") {
         os << "Name needed!" << std::endl;
      } else {
         Cdmw::CommonSvcs::Naming::NamingInterface ni(m_root.in());
         
         typedef Cdmw::CommonSvcs::Naming::NamingUtil<CosNaming::NamingContext> Util;
         CORBA::Object_var obj = ni.resolve(arg);
         if (!CORBA::is_nil(obj.in())) {
            CORBA::Boolean existent__ = obj->_non_existent ();
            if (existent__) 
               os << "Object '" << arg << "' does not exist." << std::endl;
            else 
               os << "Object '" << arg << "' exists." << std::endl;
            result = OP_SUCCESS;
         } else {
            os << " '" << arg << "' resolves to a nil object reference!" << std::endl;
         }
      }
   } catch (const Cdmw::CommonSvcs::Naming::InvalidNameException & e) {
      os << "Invalid Name : <" << e.what() << ">" << std::endl;
   } catch (const CosNaming::NamingContext::InvalidName & e) {
      os << "Invalid Name : <" << e << ">" << std::endl;
   } catch (const CORBA::Exception & e) {
      os << "CORBA Exception : \n" << e << std::endl;
   }
   return result;
} 
//----------------------------------------------------------------------------
CORBA::Long NameServiceAdmin::obj_is_a(const std::string & arg, std::ostream & os) throw()
{    
   CORBA::Long result = OP_FAILURE; 
   std::string name; 
   try {
      if (arg == "") {
         os << "Missing 'name' and 'type-id' parameters!" << std::endl;
      } else {
         // Extract name and type-id
         // skip leading whitespaces
         std::string::const_iterator i = arg.begin();
         std::string::const_iterator end = arg.end();
         i = std::find_if(i,end,not_space);
         // find end of command
         std::string::const_iterator j = std::find_if(i,end,isspace);
         // copy the characters in [i,j)
         if (i != end)
            name = std::string(i,j);
         // any maximum?
         // skip leading whitespaces
         i = std::find_if(j,end,not_space);
         std::string type_id;
         // copy arguments
         if (i != end) {
            type_id = std::string(i,end);
            if (!::Cdmw::OrbSupport::OrbSupport::is_type_id_valid(type_id.c_str())) {
               os << "Incorrect type-id: Use OMG IDL Format\n"
                  << "\t  IDL:identifier[/identifier]...:version-number\n"
                  << "version-number contains a version number with the following format:\n"
                  << "\t  major.minor"
                  << std::endl;
            } else {
               Cdmw::CommonSvcs::Naming::NamingInterface ni(m_root.in());
               CORBA::Object_var obj = ni.resolve(name);
               CORBA::Boolean is_a__ = obj->_is_a (type_id.c_str());
               if ( is_a__) os << "TRUE" << std::endl;
               else os << "FALSE" << std::endl;
               result = OP_SUCCESS;
            }
         } else {
            os << "Missing 'type-id' parameter!" << std::endl;
         }
      }
   } catch (const Cdmw::CommonSvcs::Naming::InvalidNameException & e) {
      os << "Invalid Name : <" << e.what() << ">" << std::endl;
   } catch (const CosNaming::NamingContext::InvalidName & e) {
      os << "Invalid Name : <" << e << ">" << std::endl;
   } catch (const CORBA::Exception & e) {
      os << "CORBA Exception : \n" << e << std::endl;
   }
   return result;
}
//----------------------------------------------------------------------------
CORBA::Long NameServiceAdmin::obj_is_equivalent(const std::string & arg, std::ostream & os) throw()
{    
   CORBA::Long result = OP_FAILURE;    
   os << "NOT YET IMPLEMENTED! " << std::endl;
   return result;
}
//----------------------------------------------------------------------------  
CORBA::Long NameServiceAdmin::obj_hash(const std::string & arg, std::ostream & os) throw()
{    
   CORBA::Long result = OP_FAILURE; 
   std::string name; 
   try {
      if (arg == "") {
         os << "Missing 'name' and 'maximum' parameters!" << std::endl;
      } else {
         // Extract name and maximum
         // skip leading whitespaces
         std::string::const_iterator i = arg.begin();
         std::string::const_iterator end = arg.end();
         i = std::find_if(i,end,not_space);
         // find end of command
         std::string::const_iterator j = std::find_if(i,end,isspace);
         // copy the characters in [i,j)
         if (i != end)
            name = std::string(i,j);
         // any maximum?
         // skip leading whitespaces
         i = std::find_if(j,end,not_space);
         std::string maxarg;
         // copy arguments
         if (i != end) {
            maxarg = std::string(i,end);
            CORBA::ULong max = atoi(maxarg.c_str());
            Cdmw::CommonSvcs::Naming::NamingInterface ni(m_root.in());
            CORBA::Object_var obj = ni.resolve(name);
            os << obj->_hash (max) << std::endl;
            result = OP_SUCCESS;
         } else {
            os << "Missing 'maximum' parameter!" << std::endl;
         }
      }
   } catch (const Cdmw::CommonSvcs::Naming::InvalidNameException & e) {
      os << "Invalid Name : <" << e.what() << ">" << std::endl;
   } catch (const CosNaming::NamingContext::InvalidName & e) {
      os << "Invalid Name : <" << e << ">" << std::endl;
   } catch (const CORBA::Exception & e) {
      os << "CORBA Exception : \n" << e << std::endl;
   }
   return result;
}
//----------------------------------------------------------------------------  
CORBA::Long NameServiceAdmin::obj_repository_id(const std::string & arg, std::ostream & os) throw()
{    
   CORBA::Long result = OP_FAILURE; 

   try {
      if (arg == "") {
         os << "Name needed!" << std::endl;
      } else {
         Cdmw::CommonSvcs::Naming::NamingInterface ni(m_root.in());
         
         typedef Cdmw::CommonSvcs::Naming::NamingUtil<CosNaming::NamingContext> Util;
         CORBA::Object_var obj = ni.resolve(arg);
         if (!CORBA::is_nil(obj.in())) {
            CORBA::String_var repid__ = obj->_repository_id ();
            os << repid__.in() << std::endl;
            result = OP_SUCCESS;
         } else {
            os << " '" << arg << "' resolves to a nil object reference!" << std::endl;
         }
      }
   } catch (const Cdmw::CommonSvcs::Naming::InvalidNameException & e) {
      os << "Invalid Name : <" << e.what() << ">" << std::endl;
   } catch (const CosNaming::NamingContext::InvalidName & e) {
      os << "Invalid Name : <" << e << ">" << std::endl;
   } catch (const CORBA::Exception & e) {
      os << "CORBA Exception : \n" << e << std::endl;
   }
   return result;
}
//----------------------------------------------------------------------------   
CORBA::Long NameServiceAdmin::obj_get_component(const std::string & arg, std::ostream & os) throw()
{    
   CORBA::Long result = OP_FAILURE;    
   os << "NOT YET IMPLEMENTED! " << std::endl;
   return result;
}
//----------------------------------------------------------------------------
CORBA::Long NameServiceAdmin::system(const std::string & arg, std::ostream & os) throw()
{    
   CORBA::Long result = OP_FAILURE; 
   try {
      if (arg == "") {
         os << "Shell command missing!" << std::endl;
      } else {
         std::string cmd;
         // Extract command and arguments
         // skip leading whitespaces
         std::string::const_iterator i = arg.begin();
         std::string::const_iterator end = arg.end();
         i = std::find_if(i,end,not_space);
         // find end of command
         std::string::const_iterator j = std::find_if(i,end,isspace);
         // copy the characters in [i,j)
         if (i != end)
            cmd = std::string(i,j);
         // any maximum?
         // skip leading whitespaces
         i = std::find_if(j,end,not_space);
         std::string cmd_args("");
         // copy arguments
         if (i != end) 
            cmd_args = std::string(i,end);
         std::ostringstream args;
         CORBA::String_var root_s = m_orb->object_to_string(m_root.in());
         args << " -ORBInitRef NameService=" << root_s.in() << " " << cmd_args;  
    
         (void)Cdmw::OsSupport::OS::create_process (cmd, args.str());

         result = OP_SUCCESS;
      }
   } catch (const CORBA::Exception & e) {
      os << "CORBA Exception : \n" << e << std::endl;
   } catch (const std::exception & e) {
      os << "Standard exception : \n" << e.what() << std::endl;
   }
   return result;
}
//----------------------------------------------------------------------------

} // namespace Tools
} // namespace Cdmw

