/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

#include "repositoryinterface/RepositoryInterfaceTest.hpp"

#include <Foundation/common/Assert.hpp>
#include <Foundation/orbsupport/OrbSupport.hpp>
#include <Foundation/commonsvcs/naming/NamingInterface.hpp>
#include <Repository/repositoryinterface/RepositoryInterface.hpp>

 

namespace {
    
    const std::string DEFAULT_NAME_DOMAIN_INVALID = "%$.*\\'/|..,\n;";
    const std::string DEFAULT_NAME_DOMAIN_UNKNOWN = "UndefinedNameDomain";
    const std::string DEFAULT_NAME_DOMAIN         = "DefaultNameDomain";
    const std::string SUB_NAME_DOMAIN             = "SubNameDomain";
    const std::string OTHER_NAME_DOMAIN           = "AnotherNameDomain";
}


namespace Cdmw
{
namespace NamingAndRepository
{

    using namespace Cdmw::CommonSvcs::Naming;
    CPPUNIT_TEST_SUITE_REGISTRATION( RepositoryInterfaceTest );
    
    RepositoryInterfaceTest::RepositoryInterfaceTest()
        throw ()
    {
        m_orb =  CORBA::ORB::_duplicate( Cdmw::TestUtils::CORBATestManager::instance()->get_ORB());
        CORBA::Object_var obj1 = Cdmw::TestUtils::CORBATestManager::instance()->get_object("repository");
        m_repository = CdmwNamingAndRepository::Repository::_narrow(obj1.in());
    }
    
    void RepositoryInterfaceTest::destroyORB()
        throw ()
    {
        m_repository = CdmwNamingAndRepository::Repository::_nil();
        if (! CORBA::is_nil(m_orb.in())) {
            try {
                m_orb->destroy();
                m_orb = CORBA::ORB::_nil();
            } catch (const CORBA::SystemException& e) {
                std::cerr << "Unexpected exception during ORB destroy: " << e << std::endl;
            }
        }
    }
    
    
    void RepositoryInterfaceTest::do_tests()
    {
        // set number of requested successfull tests
//         set_nbOfRequestedTestOK(9);
        
        TEST_INFO("Try RepositoryInterface.init with invalid domain name");
        try {
            RepositoryInterface::init(DEFAULT_NAME_DOMAIN_INVALID, m_repository.in());
            TEST_INFO("expected InvalidName exception bas not been thrown");
            CPPUNIT_ASSERT(false);
        } catch (const CdmwNamingAndRepository::InvalidName& e) {
            TEST_INFO("expected InvalidName exception catched");
            CPPUNIT_ASSERT(true);
        } catch (const CORBA::Exception& e) {
            TEST_INFO("Unexpected exception: " << e);
            CPPUNIT_ASSERT(false);
        }
        
        TEST_INFO("Try RepositoryInterface.init with unwkown domain name");
        try {
            RepositoryInterface::init(DEFAULT_NAME_DOMAIN_UNKNOWN, m_repository.in());
            TEST_INFO("expected NoNameDomain exception bas not been thrown");
            CPPUNIT_ASSERT(false);
        } catch (const CdmwNamingAndRepository::NoNameDomain& e) {
            TEST_INFO("expected NoNameDomain exception catched");
            CPPUNIT_ASSERT(true);
        } catch (const CORBA::Exception& e) {
            TEST_INFO("Unexpected exception: " << e);
            CPPUNIT_ASSERT(false);
        }
        
        TEST_INFO("RepositoryInterface.init with valid domain name");
        try {
            RepositoryInterface::init(DEFAULT_NAME_DOMAIN, m_repository.in());
            TEST_INFO("RepositoryInterface successfully initialised");
            CPPUNIT_ASSERT(true);
        } catch (const CORBA::Exception& e) {
            TEST_INFO("Unexpected exception: " << e);
            CPPUNIT_ASSERT(false);
        }
        
        TEST_INFO("Test getRepository()");
        try {
            CdmwNamingAndRepository::Repository_var rep = 
                RepositoryInterface::get_repository();
            if (m_repository->_is_equivalent(rep.in()))
                CPPUNIT_ASSERT(true);
            else
                CPPUNIT_ASSERT(false);
        } catch (const CORBA::SystemException& e) {
            TEST_INFO("Unexpected exception: " << e);
            CPPUNIT_ASSERT(false);
        }
        

        TEST_INFO("Get NamingInterface for default name domain");
        try {
            NamingInterface ni = RepositoryInterface::get_domain_naming_interface();
            try {
                // check that ni correspond to default name domain
                CORBA::Object_var obj = ni.resolve(SUB_NAME_DOMAIN);
                CPPUNIT_ASSERT(true);
            } catch (const CosNaming::NamingContext::NotFound& e) {
                TEST_INFO("NamingInterface doesn't correspond to default name domain");
                CPPUNIT_ASSERT(false);
            }
        } catch (const CORBA::Exception& e) {
            TEST_INFO("Cannot retrieve default name domain: " << e);
            CPPUNIT_ASSERT(false);
        }


        TEST_INFO("Get NamingContext for default name domain");
        try {
            CosNaming::NamingContext_var nc = RepositoryInterface::get_domain_context();
            try {
                // check that nc correspond to default name domain
                CosNaming::Name_var name = 
                    NamingInterface::to_name(SUB_NAME_DOMAIN);
                CORBA::Object_var obj = nc->resolve(name);
                CPPUNIT_ASSERT(true);
            } catch (const CosNaming::NamingContext::NotFound& e) {
                TEST_INFO("NamingContext doesn't correspond to default name domain");
                CPPUNIT_ASSERT(false);
            }
        } catch (const CORBA::Exception& e) {
            TEST_INFO("Cannot retrieve default name domain: " << e);
            CPPUNIT_ASSERT(false);
        }


        TEST_INFO("Get default name domain");
        try {
            CdmwNamingAndRepository::NameDomain_var nd = 
                RepositoryInterface::get_domain();
            try {
                // check that nd correspond to default name domain
                CdmwNamingAndRepository::NameDomain_var subnd = 
                    nd->resolve_sub_domain(SUB_NAME_DOMAIN.c_str());
                CPPUNIT_ASSERT(true);
            } catch (const CdmwNamingAndRepository::NoNameDomain& e) {
                TEST_INFO("NameDomain doesn't correspond to default name domain");
                CPPUNIT_ASSERT(false);
            }
        } catch (const CORBA::Exception& e) {
            TEST_INFO("Cannot retrieve default name domain: " << e);
            CPPUNIT_ASSERT(false);
        }

        TEST_INFO("Try to get an unkown name domain");
        try {
            CdmwNamingAndRepository::NameDomain_var nd = 
                RepositoryInterface::get_domain(DEFAULT_NAME_DOMAIN_UNKNOWN);
            TEST_INFO("expected NoNameDomain exception has not been thrown");
            CPPUNIT_ASSERT(false);
        } catch (const CdmwNamingAndRepository::NoNameDomain& e) {
            TEST_INFO("expected NoNameDomain exception catched");
            CPPUNIT_ASSERT(true);
        } catch (const CdmwNamingAndRepository::InvalidName& e) {
            TEST_INFO("Name " << DEFAULT_NAME_DOMAIN_UNKNOWN << " is an invalid name");
            CPPUNIT_ASSERT(false);
        }
        
        TEST_INFO("Try to get another name domain");
        try {
            CdmwNamingAndRepository::NameDomain_var nd = 
                RepositoryInterface::get_domain(OTHER_NAME_DOMAIN);
            try {
                // check that nd doesn't correspond to default name domain
                CdmwNamingAndRepository::NameDomain_var subnd = 
                    nd->resolve_sub_domain(SUB_NAME_DOMAIN.c_str());
                TEST_INFO("returned NameDomain correspond to default name domain");
                CPPUNIT_ASSERT(false);
            } catch (const CdmwNamingAndRepository::NoNameDomain& e) {
                CPPUNIT_ASSERT(true);
            }
        } catch (const CORBA::SystemException& e) {
            TEST_INFO("Cannot retrieve other name domain: " << e);
            CPPUNIT_ASSERT(false);
        }
        
        
    
        // ================================================================
        // Release all static object references held by RepositoryInterface
        // ================================================================
        RepositoryInterface::finish();
        destroyORB();
        
    }
    
};  // namespace NamingAndRepository

};  // namespace Cdmw
    

