/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#include <iostream>
#include "hostmonitor/MonitorThread.hpp"
#include "idllib/CdmwPlatformMngtEntity.stub.hpp"
#include "Foundation/commonsvcs/time/Util.hpp"

using namespace Cdmw::CommonSvcs::Time::Util;

namespace Cdmw
{        // Begin namespace Cdmw

    namespace PlatformMngt
    { // Begin namespace PlatformMngt

            MonitorThread::MonitorThread( Entity_ptr host_entity,
                                          int degraded_threshold,
                                          int failed_threshold,
                                          long polling_interval,
                                          long starting_timeout )
            {
                m_host_entity = host_entity;
                m_sync_status = Cdmw::CommonSvcs::Time::Util::UNKNOWN;
                m_degraded_threshold = degraded_threshold;
                m_failed_threshold = failed_threshold;
                m_polling_interval = polling_interval;
                m_starting_timeout = starting_timeout;
                m_logger = Cdmw::Logging::Logger::Get_Logger ("CDMW.PFMHMN");
                m_stopped = false;
            };
            
            
            MonitorThread::~MonitorThread() throw () {};
            
            void MonitorThread::stop()
            {
            CDMW_MUTEX_GUARD(m_mutex);
            m_stopped = true;
            }
            
            bool MonitorThread::is_stopped()
            {
            CDMW_MUTEX_GUARD(m_mutex);
            return m_stopped;
            }
            
            void MonitorThread::set_up()
            {
                CDMW_DEBUG(m_logger, "Monitor thread set up");  
                m_host_entity -> set_status(CORBA::string_dup("UNKNOWN"), CORBA::string_dup("The synchronization status of the host cannot be determined"));
            }
            
            
            void MonitorThread::run() throw ()
            {
                CDMW_INFO(m_logger, "Monitor thread is running");   
                
                long polling_delta = 500;
                long stop_timeout = 0; 
                
                NTP_Status tmp_status = Cdmw::CommonSvcs::Time::Util::UNKNOWN;
                
                m_host_entity -> set_status("STARTING","The synchronization status of the host cannot be determined");
                
                                   

                //CDMW_INFO(m_logger, "...starting polling..."); 
            
                //Cdmw::OsSupport::OS::sleep (polling_delta);
                
                if (m_starting_timeout < polling_delta) polling_delta = m_starting_timeout;
                    
                while ( (stop_timeout <  m_starting_timeout) && (!is_stopped()) ) 
                    {
                    //CDMW_INFO(m_logger, "...starting sleeping...");   
                    Cdmw::OsSupport::OS::sleep (polling_delta);
                    stop_timeout += polling_delta;
                    }    
            
                
                
                
                bool initialisation_done = false;
                stop_timeout = 0;
                polling_delta = 500;
                
                if (m_polling_interval < polling_delta) polling_delta = m_polling_interval;
                
                while (!is_stopped())
                {
                    
                    stop_timeout = 0;
                     
                    //CDMW_INFO(m_logger, "...polling...");   
                    
                    try 
                    {
                        tmp_status = get_synchronization_status( m_degraded_threshold, m_failed_threshold);
                    
                    }
                    catch (::Cdmw::InternalErrorException& ex)
                    {
                    CDMW_FATAL(m_logger, "Unexpected exception" << ex.what()); 
                    throw;
                    }
                    
                    if ( ( tmp_status != m_sync_status ) || ( initialisation_done == false ) )
                    {
                        m_sync_status = tmp_status;
                                        
                        CDMW_INFO(m_logger, "Synchronization status has changed: " << m_sync_status );  
                        CDMW_INFO(m_logger, "Degraded threshold : " <<  m_degraded_threshold <<
                                            " " << "Failed threshold : " << m_failed_threshold);  
                        
                        switch ( m_sync_status )
                        {
                            case NOT_SYNCHRONIZED :
                            {
                            m_host_entity -> set_status("FAILED","The host is not synchronized or NTP daemon is down");
                            } 
                            break;
                            case LOCAL_SYNCHRONIZED :
                            {
                            m_host_entity -> set_status("FAILED","The host is locally synchronized");
                            }
                            break;
                            case EXTERNAL_SYNCHRONIZED :
                            {
                             m_host_entity -> set_status("OK","The host is synchronized with an external source");
                            }
                            break;
                            case INTERNAL_SYNCHRONIZED :
                            {
                             m_host_entity -> set_status("DEGRADED","The host is synchronized with one of its peers");
                            }
                            break;
                            case STATUS_NOT_YET_DETERMINATED :
                            case Cdmw::CommonSvcs::Time::Util::UNKNOWN :
                            default :
                            {
                             m_host_entity -> set_status("UNKNOWN","The synchronization status of the host cannot be determined");
                                                       
                            }
                            break;
                        }
                        
                        
                        CORBA::String_var reason = CORBA::string_dup("");
                        std::string tmp_entity_status (m_host_entity -> get_status ( reason.out() ));
                        
                        CDMW_INFO (m_logger, m_host_entity -> name () << " entity status is: "   
                                                            <<  tmp_entity_status.c_str()
                                                            << " " << reason.in() ); 
                      initialisation_done = true;                                              
                      
                    }
                       
                    
                       
                       //Cdmw::OsSupport::OS::sleep (polling_delta);
                       while ( (stop_timeout  <  m_polling_interval ) && (!is_stopped()) ) 
                       {
                       //   CDMW_INFO(m_logger, "...sleeping...");   
                       Cdmw::OsSupport::OS::sleep (polling_delta);
                       stop_timeout += polling_delta;
                       }
                }
                
            };
    
    } // End namespace PlatformMngt
} // End namespace Cdmw

