/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#include "Foundation/common/Options.hpp"
#include "Foundation/common/String.hpp"
#include "Foundation/orbsupport/CORBA.hpp"
#include "Foundation/commonsvcs/time/Util.hpp"
#include "Foundation/ossupport/OS.hpp"
#include "Foundation/osthreads/ThreadHandle.hpp"
#include "Foundation/orbsupport/OrbSupport.hpp"
#include "Foundation/orbsupport/StrategyList.hpp"
#include "Foundation/logging/Logger.hpp"

#include "SystemMngt/platforminterface/PlatformInterface.hpp"
//#include "hostmonitor/HostMonitor_impl.hpp"
//#include "hostmonitor/HostMonitor.skel.hpp"
#include "hostmonitor/ExitHandler.hpp"
#include "hostmonitor/MyProcessBehaviour.hpp"

#include <cstdlib>
#include <string>
#include <exception>
#include <sstream>
#include <iostream>
#include <fstream>


const int POA_THREAD_POOL_SIZE = 5;

namespace
{
    using namespace CdmwPlatformMngt;
    using namespace Cdmw::OsSupport;
    using namespace Cdmw::OrbSupport;
    using namespace Cdmw::PlatformMngt;

    const std::string T_DEGRADED_OPTION = "--threshold-degraded";
    const std::string T_FAILED_OPTION = "--threshold-failed"; 
    const std::string POLLING_INTERVAL_OPTION = "--polling-interval";
    const std::string STARTING_TIMEOUT_OPTION = "--starting-timeout";
    
    const int DEFAULT_T_DEGRADED = 10; 
    const int DEFAULT_T_FAILED = 5;
    const long DEFAULT_POLLING_INTERVAL = 5000;
    const long DEFAULT_STARTING_TIMEOUT = 20000;
}


void exceptionHandler(void)
{
    std::cout << "Unexpected Exception" << std::endl;
}


void exitHandler(void)
{
    std::cout << "Exit Handler ...." << std::endl;
}


static void
usage(std::ostream & os, const std::string &program_name)
{
    os << "Usage:\n"
       << program_name << " options\n"
       << "\n"
       << "Options:\n"
       << "--help                             Display this information.\n"
       << "[--threshold-degraded=<integer>]   Minimum number of servers to have DEGRADED synchronization. Default value is 10.\n"
       << "[--threshold-failed=<integer>]     Minimum number of servers to have FAILED synchronization. Default value is 5.\n"
       << "[--polling-interval=<integer>]     Interval for polling the NTP server status. Parameter is expressed in milliseconds. Default is 5000\n"
        << "[--starting-timeout=<integer>]    During this initial timeout the synchronization status of the host is STARTING. Parameter is expressed in milliseconds. Default is 20000\n"
       << std:: endl;
}
    
   
int main(int argc, char* argv[])
{  
   
    // the return code 
    int ret_code = 0;
    
    std::set_unexpected(&exceptionHandler);

    ::atexit(&exitHandler);

    StrategyList strategy;
    strategy.add_OrbThreaded();
    //strategy.add_PoaThreadPerRequest();
    strategy.add_PoaThreadPool(POA_THREAD_POOL_SIZE);

    CORBA::ORB_var orb = CORBA::ORB::_nil();


    //
    // help argument
    //
    if (argc > 1)
    {
        if(strcmp(argv[1], "--help") == 0)
        {
            usage(std::cout, argv[0]);
            return ret_code;
        }
    }
    Cdmw::Logging::LogManager::Init(argc, argv);
    
    try
    {
        
        std::string t_degraded_option = OS::get_option_value (argc, argv, T_DEGRADED_OPTION);
        int t_degraded = DEFAULT_T_DEGRADED;
        
        if (t_degraded_option == "yes")
        {
            std::cout << "bad value for " << T_DEGRADED_OPTION.c_str() << "=<integer> option\n";
            std::cout << std::endl;
            return EXIT_FAILURE;
        }
        else if (t_degraded_option != "no")
        {
            std::istringstream t_degraded_option_strm(t_degraded_option);
            t_degraded_option_strm >> t_degraded;
            if (t_degraded_option_strm.fail())
            {
                std::cout << "bad value for " << T_DEGRADED_OPTION.c_str() << "=<integer> option\n";
                std::cout << std::endl;
                return EXIT_FAILURE;
            }
        }

 
        std::string t_failed_option = OS::get_option_value (argc, argv, T_FAILED_OPTION);
        int t_failed = DEFAULT_T_FAILED;
        
        if (t_failed_option == "yes")
        {
            std::cout << "bad value for " << T_FAILED_OPTION.c_str() << "=<integer> option\n";
            std::cout << std::endl;
            return EXIT_FAILURE;
        }
        else if (t_failed_option != "no")
        {
            std::istringstream t_failed_option_strm(t_failed_option);
            t_failed_option_strm >> t_failed;
            if (t_failed_option_strm.fail())
            {
                std::cout << "bad value for " << T_FAILED_OPTION.c_str() << "=<integer> option\n";
                std::cout << std::endl;
                return EXIT_FAILURE;
            }
        }
       
       
        std::string p_interval_option = OS::get_option_value (argc, argv, POLLING_INTERVAL_OPTION);
        long p_interval = DEFAULT_POLLING_INTERVAL;
        
        if (p_interval_option == "yes")
        {
            std::cout << "bad value for " << POLLING_INTERVAL_OPTION.c_str() << "=<integer> option\n";
            std::cout << std::endl;
            return EXIT_FAILURE;
        }
        else if (p_interval_option != "no")
        {
            std::istringstream p_interval_option_strm(p_interval_option);
            p_interval_option_strm >> p_interval;
            if (p_interval_option_strm.fail())
            {
                std::cout << "bad value for " << POLLING_INTERVAL_OPTION.c_str() << "=<integer> option\n";
                std::cout << std::endl;
                return EXIT_FAILURE;
            }
        }
       
        
        std::string s_timeout_option = OS::get_option_value (argc, argv, STARTING_TIMEOUT_OPTION);
        long s_timeout = DEFAULT_STARTING_TIMEOUT;
        
        if (s_timeout_option == "yes")
        {
            std::cout << "bad value for " << STARTING_TIMEOUT_OPTION.c_str() << "=<msec> option\n";
            std::cout << std::endl;
            return EXIT_FAILURE;
        }
        else if (s_timeout_option != "no")
        {
            std::istringstream s_timeout_option_strm(s_timeout_option);
            s_timeout_option_strm >> s_timeout;
            if (s_timeout_option_strm.fail())
            {
                std::cout << "bad value for " << STARTING_TIMEOUT_OPTION.c_str() << "=<msec> option\n";
                std::cout << std::endl;
                return EXIT_FAILURE;
            }
        }
       
       
       CDMW_INFO(Cdmw::Logging::Logger::Get_Logger("CDMW.PFMHMN"), 
                 "HostMonitor starting options :\nStarting timeout = " <<
                  s_timeout << "\nPolling interval = " << 
                  p_interval << "\nThreshold failed = " << 
                  t_failed << "\nThreshold degraded = "<< t_degraded);
       
        orb =  Cdmw::OrbSupport::OrbSupport::ORB_init(argc, argv, strategy); 

        CORBA::Object_var obj = orb->resolve_initial_references("RootPOA");

        PortableServer::POA_var poa = PortableServer::POA::_narrow(obj.in());

        PortableServer::POAManager_var poaMgr = poa->the_POAManager();

        poaMgr->activate();        
   
        try
        {
            //Setup signal handling for ORB to be shut down
            ExitHandler * exitHandler = new ExitHandler(orb.in());
            Cdmw::OrbSupport::OrbSupport::install_orb_shutdown_signal_handlers(
                    orb.in(), 
                    exitHandler);
                    
            // initialise the platform interface
            PlatformInterface::Setup(orb.in(), argc, argv);
    
            // creates the process behaviour
            std::auto_ptr<MyProcessBehaviour> pMyProcess(  
                            new MyProcessBehaviour( orb.in(), t_degraded, t_failed, p_interval, s_timeout ));
            
            // acknowledge the creation of the process
            PlatformInterface::Acknowledge_creation(pMyProcess.get());
            pMyProcess.release();
            
            // start orb
            orb->run();
        
        } 
        catch(...)
        {
            // cleanup the ORB to exit completely
            orb->shutdown(false);
            Cdmw::OrbSupport::OrbSupport::ORB_cleanup(orb.in());
            orb->destroy();
            throw;
        }   

        
        // cleanup the platform interface
        PlatformInterface::Cleanup();
        Cdmw::OrbSupport::OrbSupport::ORB_cleanup(orb.in());
        orb->destroy();
        
    }
    catch (const CORBA::Exception& e)
    {  
        std::cerr << "FAILURE : CORBA exception raised :" << std::endl
            << e._name() << std::endl;
        ret_code = EXIT_FAILURE;
    }
    catch(const std::exception &e)
    {
        std::cerr << "FAILURE : Exception raised : " << std::endl
            << e.what() << std::endl;
        ret_code = EXIT_FAILURE;
    }
    catch(...)
    {
        std::cerr << "FAILURE : Unexpected exception" << std::endl;
        ret_code = EXIT_FAILURE;
    }

    return ret_code;

}


