/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#include "platformadminlibrary/ApplicationParser.hpp"
#include "platformadminlibrary/ProcessParser.hpp"
#include "platformadminlibrary/TimeoutParser.hpp"
#include "platformadminlibrary/MonitoringParser.hpp"
#include "platformadminlibrary/ServiceParser.hpp"
#include "platformadminlibrary/EntityParser.hpp"
#include "platformadminlibrary/PropertyParser.hpp"
#include "platformadminlibrary/ExitActionParser.hpp"
#include "platformadminlibrary/GraphParser.hpp"

namespace Cdmw {         // Begin namespace Cdmw
namespace PlatformAdmin { // Begin namespace PlatformAdmin

/**
 * InitStepParser class.
 */
ProcessParser::InitStepParser::InitStepParser(
    const DOM_Node& initStepParser )
: ConfigElementParser( initStepParser )
{
}

std::string ProcessParser::InitStepParser::name() const
{
    return get_attribute_value( "Name" );
}

std::string ProcessParser::get_force_manual_start() const
{
    return get_attribute_value("Force-manual-start");
}


unsigned long ProcessParser::InitStepParser::timeout() const
{
    std::string timeout = get_attribute_value( "Timeout" );

    if( timeout == "" )
        throw NotFoundException();
        
    return atol( timeout.c_str() );
}

/**
 * RunStepParser class.
 */
ProcessParser::RunStepParser::RunStepParser(
    const DOM_Node& runStepParser )
: ConfigElementParser( runStepParser )
{
}

unsigned long ProcessParser::RunStepParser::timeout() const
{
    std::string timeout = get_attribute_value( "Timeout" );
    return atol( timeout.c_str() );
}

/**
 * StopStepParser class.
 */
ProcessParser::StopStepParser::StopStepParser(
    const DOM_Node& stopStepParser )
: ConfigElementParser( stopStepParser )
{
}

unsigned long ProcessParser::StopStepParser::timeout() const
{
    std::string timeout = get_attribute_value( "Timeout" );
    return atol( timeout.c_str() );
}

/**
 * ProcessParser class.
 */
ProcessParser::ProcessParser(
    const DOM_Node& processParser )
: ConfigElementParser( processParser ),
  ServiceContainerParser( processParser ),
  EntityContainerParser( processParser ),
  CcmComponentContainerParser( processParser )
{
}

std::string ProcessParser::name() const
{
    return get_attribute_value( "Name" );
}

std::string ProcessParser::host_name() const
{
    return get_attribute_value( "Host-name" );
}

CdmwPlatformMngt::ProcessType ProcessParser::type() const
{
    std::string type = get_attribute_value( "Type" );

    if( type == "MANAGED-PROCESS")
        return CdmwPlatformMngt::MANAGED_PROCESS;
    else if( type == "SERVICE-PROVIDER")
        return CdmwPlatformMngt::SERVICE_PROVIDER;
    else if( type == "COMPONENT-SERVER")
        return CdmwPlatformMngt::COMPONENT_SERVER;
    else
        return CdmwPlatformMngt::UNMANAGED_PROCESS;
}

bool ProcessParser::is_autoended() const
{
 //   return get_attribute_value( "Autoended" ) == "TRUE";
      std::string b = get_attribute_value( "Autoended");
      if (b== "TRUE")
      return true;
      return false;
}

std::string ProcessParser::executable_path() const
{
    return get_attribute_value( "Exec-path" );
}

unsigned long ProcessParser::port() const
{
    return atol( get_attribute_value( "Port" ).c_str() );
}


unsigned long ProcessParser::initialisation_timeout()  const
{
    return atol( get_attribute_value( "Initialisation-timeout" ).c_str() );
}

std::string ProcessParser::get_process_env() const
{
    return get_attribute_value( "Env" );
}

bool ProcessParser::get_memory_resident() const
{
   std::string mem_res = get_attribute_value( "Memory-resident");
   if ( 0 == strcmp(mem_res.c_str(),"Yes"))
   {
       return true;
   }
   else  
   {
       return false;
   }
}

bool ProcessParser::get_run_as_root() const
{
   std::string run_root = get_attribute_value("Run-as-root");
   if( 0 == strcmp(run_root.c_str(),"Yes"))
   {
       return true;
   }
   else 
   {
       return false;
   }
}

int ProcessParser::get_core_size() const
{
    std::string core_size = get_attribute_value("Core-dump-size");
    if ( 0 == strcmp(core_size.c_str(),"Unlimited"))
    {
        return -1;
    }
    else
    {
        return atol( core_size.c_str() ); 
    }
}

std::string ProcessParser::arguments() const
{
    return get_attribute_value( "Args" );
}

std::string ProcessParser::filters() const
{
    return get_attribute_value( "Filters" );
    //return NULL;
}

std::string ProcessParser::run_directory() const
{
    return get_attribute_value( "Run-dir" );
}

TimeoutParser ProcessParser::get_timeout() const
{
    try
    {
        DOM_Node timeout_parser( get_child_element( "Timeout", 0 ) ); 
        return TimeoutParser( timeout_parser );
    }
    catch( const NotFoundException& e )
    {
        ApplicationParser application( get_parent_element() ); 
        return application.get_timeout();
    }
}

unsigned int ProcessParser::get_init_step_count() const
{
    return get_child_count( "Init-step" );
}

std::string ProcessParser::get_init_step_name(
    unsigned int initStepIndex ) const
{
    try
    {
        InitStepParser init_step_parser(
            get_child_element( "Init-step", initStepIndex ) );

        return init_step_parser.name();
    }
    catch( const NotFoundException& e )
    {
        return std::string( "" );
    }
}

unsigned long ProcessParser::get_init_step_timeout(
    unsigned int initStepIndex ) const
{
    try
    {
        InitStepParser init_step_parser(
            get_child_element( "Init-step", initStepIndex ) );

        return init_step_parser.timeout();
    }
    catch( const NotFoundException& e )
    {
        ApplicationParser application( get_parent_element() ); 
        return application.get_timeout().init();
    }
}

unsigned long ProcessParser::get_run_timeout() const
{
    try
    {
        RunStepParser run_step_parser(
            get_child_element( "Run-timeout", 0 ) );

        return run_step_parser.timeout();
    }
    catch( const NotFoundException& e )
    {
        ApplicationParser application( get_parent_element() ); 
        return application.get_timeout().run();
    }
}

unsigned long ProcessParser::get_stop_timeout() const
{
    try
    {
        StopStepParser stop_step_parser(
            get_child_element( "Stop-timeout", 0 ) );

        return stop_step_parser.timeout();
    }
    catch( const NotFoundException& e )
    {
        ApplicationParser application( get_parent_element() ); 
        return application.get_timeout().stop();
    }
}

ProcessMonitoringParser ProcessParser::get_monitoring() const
{
    try
    {
        DOM_Node monitoring_parser(
            get_child_element( "Proc-monitoring", 0 ) );

        return ProcessMonitoringParser( monitoring_parser );
    }
    catch (const NotFoundException&)
    {
        try
        {            
            ApplicationParser application( get_parent_element() );
            return application.get_proc_monitoring();
        }
        catch (const NotFoundException&)
        {
            // exception NotFound is returned if parent undefined
            throw;
        }
    }    
}

PropertyParser ProcessParser::get_property(unsigned int propertyIndex) const
{
    DOM_Node property_parser(get_child_element("Property", propertyIndex ));
    return PropertyParser(property_parser);
}
                                                                                                                             
unsigned int ProcessParser::get_property_count() const
{
    return get_child_count("Property");
}
                                          
unsigned int ProcessParser::get_exit_action_count() const
{
    return get_child_count("Exit-action");
}

ExitActionParser ProcessParser::get_exit_action(unsigned int exit_action_index) const
{
    DOM_Node property_parser(get_child_element("Exit-action", exit_action_index ));
    return ExitActionParser(property_parser);
}
                                                                                   
CdmwPlatformMngt::ExitActionType ProcessParser::get_exit_action(std::string event_related_exit_action) const
{
    std::string exit_action = get_attribute_value( event_related_exit_action.c_str() );  
    
    if (exit_action == "RESTART_APPLICATION")
        return CdmwPlatformMngt::RESTART_APPLICATION;
    else if (exit_action == "RESTART_PROCESS")
        return CdmwPlatformMngt::RESTART_PROCESS;
    else if (exit_action == "SHUTDOWN_NODE")
        return CdmwPlatformMngt::SHUTDOWN_NODE;
    else if (exit_action == "REBOOT_NODE")
        return CdmwPlatformMngt::REBOOT_NODE;
    else if (exit_action == "STOP_APPLICATION")
        return CdmwPlatformMngt::STOP_APPLICATION;
    else if (exit_action == "ABORT_PROCESS")
        return CdmwPlatformMngt::ABORT_PROCESS; 
    else 
        return CdmwPlatformMngt::NO_ACTION;
  
}

unsigned int ProcessParser::get_max_nbr_of_restart_attempt() const
{
    return atoi( get_attribute_value( "Max-nbr-of-restart-attempt" ).c_str() );
}

CdmwPlatformMngt::AutoRestartControl ProcessParser::get_auto_restart_control() const
{
    std::string auto_restart_control = get_attribute_value( "Auto-restart-control" );
    
    if (auto_restart_control == "USE_COUNTER")
        return CdmwPlatformMngt::USE_COUNTER;
    else if (auto_restart_control == "IGNORE_COUNTER")
        return CdmwPlatformMngt::IGNORE_COUNTER;
    else if (auto_restart_control == "IGNORE_ACTION")
        return CdmwPlatformMngt::IGNORE_ACTION;
    else 
        return CdmwPlatformMngt::USE_COUNTER;
}

std::string ProcessParser::get_default_restart_warmth() const
{
    return get_attribute_value( "Default-restart" ).c_str() ;
}


GraphParser ProcessParser::get_component_init_graph() const
{
    DOM_Node graph_Parser( get_child_element( "Init-graph", 0 ) );
    return GraphParser( graph_Parser );
}

std::string ProcessParser::get_relevant_indicator() const
{
   return get_attribute_value( "Relevant-indicator" ).c_str();
}


} // End namespace PlatformAdmin
} // End namespace Cdmw

