/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

#ifndef INCL_PLATFORMMNGT_DEPLOYMENTMANAGER_HPP
#define INCL_PLATFORMMNGT_DEPLOYMENTMANAGER_HPP


#include <vector>
#include <map>
#include <iostream>
#include <Foundation/osthreads/ReaderRecursiveLockGuard.hpp>
#include <Foundation/osthreads/WriterRecursiveLockGuard.hpp>
#include <Foundation/osthreads/ReaderWriterRecursiveMutex.hpp>
#include <SystemMngt/platformapplicationlibrary/NodeDeploymentExecutor.hpp>
#include <Foundation/idllib/CdmwDnC.stub.hpp>
#include <SystemMngt/platformapplicationlibrary/Application_impl.hpp>
#include <list>

namespace Cdmw
{
  namespace PlatformMngt 
  {
    
    /**
     *Purpose:
     *<p> Creates many DeploymentExecutors as many (hostname, deployment_plan) 
     *  couples are present in the system.
     *  Recovers The NodeManager and the NodeApplicationManger 
     */

    class DeploymentManager
    {
    
    public:

      /**
       *Purpose:
       *<p> static method to recover the singleton instance.
       */
      static DeploymentManager* get_instance();

      /**
      *Purpose:
      *<p> initialise singleton, set the orb
      */
      void init(CORBA::ORB_ptr orb);
 
     /*
      * Recover Node Managers
      */
      void set_host_names(std::vector<std::string>* host_names);

      /*
      * Set the port and the name of node manager : port/name
      */ 
      void set_node_manager_port_name (const char * nodeManagerPortName);
      
     /*
      * Recover Node Manager
      */
      void set_node_manager_host_name(std::string host_name) throw (CORBA::SystemException);   

     /*
      * Remove Node Manager
      */
      void remove_node_manager_host_name(std::string host_name);

      /**
       *Purpose:
       *<p> create a NodeDeploymentExecutor for each (host_name, deployment_plan) 
       * @param  host_name the host name of the process the deployment plan belongs
       * @param  deployment_plan the deployment plan
       */
      void set_node_deployment_info(const std::string& host_name, const std::string& deployment_plan,  Cdmw::PlatformMngt::Application_impl *application) throw(CdmwDnC::StartError, CdmwDnC::PlanError, CORBA::SystemException);

      /**
       *Purpose:
       *<p> create a NodeDeploymentExecutor for each (host_name, deployment_plan[i]) 
       * @param  host_name the host name of the process the deployment plans belong
       * @param  deployment_plans the vector of deployment plan
       */
      void set_node_deployment_infos(const std::string& host_name, const std::vector<std::string>* deployment_plans, Cdmw::PlatformMngt::Application_impl *application) throw(CdmwDnC::StartError, CdmwDnC::PlanError, CORBA::SystemException);

      /**
       *Purpose:
       *<p> remove a NodeDeploymentExecutor for each (host_name, deployment_plan) 
       * @param  host_name the host name of the process the deployment plan belongs
       * @param  deployment_plan the deployment plan
       */       
      void remove_node_deployment_info(const std::string& host_name, const std::string& deployment_plan);
    
      /**
       *Purpose:
       *<p> remove a NodeDeploymentExecutor for each (host_name, deployment_plan[i]) 
       * @param  host_name the host name of the process the deployment plans belong
       * @param  deployment_plans the vector of deployment plan
       */
      void remove_node_deployment_infos(const std::string& host_name, const std::vector<std::string>* deployment_plans);
 
      /**
       *Purpose:
       *<p> Acks the NodeDeploymentExecutor about the status change of the processes 
       * belonging to its deplomnent plan and host 
       * @param  host_name the host name of the process the deployment plan belongs
       * @param  deployment_plan the deployment plan
       */
    
      void notify(const std::string& host_name, const std::string& deployment_plan) throw (CdmwDnC::StartError);


     /**
       *Purpose:
       *<p> Acks the NodeDeploymentExecutor about the status change of the processes
       * belonging to its deplomnent plan and host
       * @param  host_names_to_deploy, the host names of the processes the deployment plan belongs
       * @param  deployment_plan the deployment plan
       */

      void notify(set<std::string> host_names_to_deploy, const std::string& deployment_plan) throw  (CdmwDnC::StartError);


      void notify_process_component_completion_event(const std::string deployment_plan, const std::string & host_name) throw (CdmwDnC::StartError);

      void notify_process_components_initialisation(const std::string deployment_plan, const std::string & host_name) throw (CdmwDnC::StartError);

      void notify_process_component_suspension_event(const std::string deployment_plan, const std::string & host_name)  throw (CdmwDnC::StartError);
 

      /**
       *Purpose:
       *<p> Constructor.
       */
      virtual ~DeploymentManager() throw();
      
      /**
       *Purpose:
       *<p> Prints the map
       */
      void print();
  

     /**
      *Purpose:
      *<p> wait all node_deployment_executors related to an application are ended
      */
      void wait_all_node_deployment_executor_ended(const std::string& application_name);
      
      
      /**
      *Purpose:
      *<p> wait all node_deployment_executors related to a process are ended
      */
      void wait_all_node_deployment_executor_ended(const std::string& application_name, const std::string& process_name);
      
      /**
      *Purpose:
      *<p> wait all node_deployment_executors related to a process are ended
      */
            
      OsSupport::ReaderWriterRecursiveMutex* get_recursive_mutex ();

    private:

      /**
       *Purpose:
       *<p> Constructor.
       */
      DeploymentManager();
 
      //check if init() has been called
      bool m_init;
   
      /**
       *Purpose:
       *<p> Acks the NodeDeploymentExecutor about the completion event of the processes
       * belonging to its deplomnent plan and host
       * @param  host_name, the host names of the processes the deployment plan belongs
       * @param  deployment_plan the deployment plan
       */
      void notify_process_component_event(const DeploymentInfo deployment_info, PROCESS_COMPONENT_EVENT event) throw (CdmwDnC::StartError);


 
      void set_node_deployment_info(const  DeploymentInfo& deployment_info, 
                                    CdmwDnC::NodeApplicationManager_ptr node_application_manager_ptr,
                                    Cdmw::PlatformMngt::Application_impl *application);
                    
      void remove_node_deployment_info(const  DeploymentInfo& deployment_info);

      /**
       *Purpose:
       *<p> triggers an action to NodeDeploymentExecutor associated to a given DeploymentInfo
       */
      void update_node_deployment_executor(const  DeploymentInfo deployment_info) throw (CdmwDnC::StartError);
  
     /**
      *Purpose:
      *<p> get the node manager from a given host name
      */
      CdmwDnC::NodeManager_ptr get_node_manager(const std::string& host_name);  

      //the map of DeploymentInfo, NodeDeploymentExecutor, key DeploymentInfo = (host_name, deployment_plan)
      typedef  std::map<DeploymentInfo, NodeDeploymentExecutor*> MapNodeDeploymentExecutor;
      
      MapNodeDeploymentExecutor m_map_deployment_executor;   
      
      // MapNodeDeploymentExecutor's iterator
      typedef MapNodeDeploymentExecutor::iterator MapNodeDeploymentExecutorIterator;
    
      //the map of NodeManagers_ptr, the key is the host name
      typedef  std::map<std::string, CdmwDnC::NodeManager_ptr> MapNodeManager;
      MapNodeManager m_map_node_manager;
    
      // MapNodeDeploymentExecutor's iterator
      typedef MapNodeManager::iterator MapNodeManagerIterator;
    
    private: 
      std::string m_node_manager_port_name;  
        
      CORBA::ORB_var  m_orb;

      OsSupport::ReaderWriterRecursiveMutex rw_mutex;
    };

  }// closing namespaces
}

#endif /*INCL_PLATFORMMNGT_DEPLOYMENTMANAGER_HPP*/
