/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#ifndef INCL_PLATFORMMNGT_APPLICATION_INIT_HPP
#define INCL_PLATFORMMNGT_APPLICATION_INIT_HPP

#include "Foundation/common/Exception.hpp"
#include "Foundation/orbsupport/OrbSupport.hpp"
#include "SystemMngt/platformlibrary/Callback.hpp"
#include "SystemMngt/platformlibrary/SynchronisableTaskFactory.hpp"
#include "SystemMngt/idllib/CdmwPlatformMngtCommon.stub.hpp"
#include "SystemMngt/idllib/CdmwPlatformMngtComponent.stub.hpp"

namespace Cdmw
{
namespace PlatformMngt
{

class ManagedProcess_impl;
class CcmComponent_impl;

/**
 * The callback for the suspension of the initialisation of an component.
 */
class InitComponentSuspensionCallback : public virtual Callback
{
    

public:

    /**
     *Purpose:
     *<p> Constructor.
     *@param  process  the process managing the components
     *@param  requested_deployment_plan
     */
    InitComponentSuspensionCallback (ManagedProcess_impl* process, const std::string& requested_deployment_plan);
        

    /**
     *Purpose:
     *<p> Destructor.
     *
     */
    ~InitComponentSuspensionCallback ();
    
    
    /**
     *Purpose:
     *<p> Callback execution. notifies the process the component
     * has been suspended.
     */
    void execute() throw();
 
    
private:

    /**
     * The process managing the component.
     */
    ManagedProcess_impl* m_process;

    std::string m_requested_deployment_plan;

};



/**
 * The callback for the completion of the initialisation of an component.
 */
class InitComponentCompletionCallback : public virtual Callback
{

public:

    /**
     *Purpose:
     *<p> Constructor.
     *@param  process  the process managing the components.
     *@param  requested_deployment_plan  
     */
    InitComponentCompletionCallback (ManagedProcess_impl* process, const std::string& requested_deployment_plan);


    /**
     *Purpose:
     *<p> Destructor.
     *
     */
    ~InitComponentCompletionCallback ();
    
    
    /**
     *Purpose:
     *<p> Callback execution. Notifies the process the component
     * has been completely initialised.
     */
    void execute() throw();
    
private:

    /**
     * The process managing the component.
     */
    ManagedProcess_impl* m_process;

    std::string m_requested_deployment_plan;

};

/**
 * The task for the initialisation of an component.
 */
class InitComponentTask : public virtual SynchronisableTask
{

public:

    /**
     *Purpose:
     *<p> Constructor.
     *
     *@param  taskId           the identifier of the component task.
     *@param  sequencer        the task sequencer
     *@param  observer         the task observer
     *@param  predecessorCount the number of predecessor tasks
     *@param  component      the component object
     *@param  startup_kind     the kind of start up
     */
    InitComponentTask (const std::string& taskId,
                         Sequencer* sequencer,
                         TaskObserver* observer,
                         size_t predecessorCount,
                         //CdmwPlatformMngt::CcmComponent_ptr component,
                         CcmComponent_impl* component,
                         const CdmwPlatformMngtBase::StartupKind& startup_kind,
                         const std::string& requested_deployment_plan)
            throw (OutOfMemoryException);


    /**
     *Purpose:
     *<p> Destructor.
     *
     */
    ~InitComponentTask () throw();
    
    
    /**
     *Purpose:
     *<p> Goes through initialisation of an component.
     *
     *@return the initialisation status of the execution.
     */
    bool execute() throw();
    
private:

    /**
     * The component.
     */
    //CdmwPlatformMngt::CcmComponent_var m_component;
    CcmComponent_impl* m_component;

    /**
     * The startup kind.
     */
    CdmwPlatformMngtBase::StartupKind_var m_startupKind;

       /**
    * The requested deployment plan the component must have to be started.
    */
    std::string m_requested_deployment_plan;
};

/**
 * The initialisation task factory.
 */
class InitComponentTaskFactory : public virtual SynchronisableTaskFactory
{


public:


    /**
     *Purpose:
     *<p> Destructor.
     *
     */
    ~InitComponentTaskFactory ();
    
    
    
    /**
     *Purpose:
     *<p> Gets a new synchronisable task factory instance.
     *
     *@param  process the process managing the component.
     *@param  startup_kind the component's startup mode.
     *@param  graphElements the dependency graph of component initialisation.
     */
    static
    SynchronisableTaskFactory* createFactory (
                     ManagedProcess_impl* process,
                     const CdmwPlatformMngtBase::StartupKind& startup_kind,
                     const CdmwPlatformMngt::GraphElements& graphElements,
                     const std::string& requested_deployment_plan)
            throw (OutOfMemoryException);

    /**
     *Purpose:
     *<p> Creates a new component synchronisable task 
     * 
     *@param taskId       the component task's name.
     *@param sequencer    the sequencer of the component's execution.
     *@param taskObserver the component's observer.
     */
    SynchronisableTask* createTask (const std::string& taskId,       
                                    Sequencer* sequencer,
                                    TaskObserver* taskObserver)
            throw (InvalidTaskIdException, OutOfMemoryException);
                   
  
  
protected:

    /**
     *Purpose:
     *<p> Constructor.
     *
     *@param  process         the process managing the component.
     *@param  startup_kind   the kind of start up
     *@param  taskDefMap     the synchro TaskDef map
     *@param  rootTasks      the root tasks list
     */
    InitComponentTaskFactory (ManagedProcess_impl* process,        
                                const CdmwPlatformMngtBase::StartupKind& startup_kind,
                                const SynchronisableTaskDefMap& taskDefMap,
                                const TaskIdList& rootTasks,
                                const std::string& requested_deployment_plan)
            throw (OutOfMemoryException);


                 
private:

    /**
     * The process managing the components.
     */
    ManagedProcess_impl* m_process;

    /**
     * The startup kind.
     */
    CdmwPlatformMngtBase::StartupKind_var m_startupKind;

   /**
    * The requested deployment plan the component must have to be started.
    */
    std::string m_requested_deployment_plan;

};

} // End namespace PlatformMngt
} // End namespace Cdmw

#endif //INCL_PLATFORMMNGT_APPLICATION_INIT_HPP

