/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#ifndef INCL_PLATFORMMNGT_PROCESS_HPP
#define INCL_PLATFORMMNGT_PROCESS_HPP

#include "Foundation/common/Exception.hpp"
#include "Foundation/orbsupport/OrbSupport.hpp"
#include "Foundation/osthreads/Mutex.hpp"
#include "Foundation/osthreads/Condition.hpp"
#include "Foundation/logging/Logger.hpp"
#include "SystemMngt/idllib/CdmwPlatformMngtBase.stub.hpp"
#include "SystemMngt/tools/SingleShotTimer.hpp"
#include "SystemMngt/idllib/CdmwPlatformMngtCommon.stub.hpp"
#include "SystemMngt/idllib/CdmwPlatformMngtEvent.stub.hpp"
#include "SystemMngt/idllib/CdmwPlatformMngtProcess.skel.hpp"
#include "SystemMngt/idllib/CdmwPlatformMngtServiceContainer.stub.hpp"
#include "SystemMngt/idllib/CdmwPlatformMngtApplicationAgent.stub.hpp"
#include "SystemMngt/idllib/CdmwPlatformMngtProcessDelegateWrapper.stub.hpp"
#include "SystemMngt/idllib/CdmwPlatformMngtElementRepository.stub.hpp"
#include "SystemMngt/platformlibrary/DeactivableServant_impl.hpp"
#include "SystemMngt/platformlibrary/ServiceDefContainer_impl.hpp"
#include "SystemMngt/platformelementlibrary/EntityContainer_impl.hpp"
#include "SystemMngt/platformapplicationlibrary/Application_impl.hpp"
#include "SystemMngt/platformelementlibrary/ManagedElement_impl.hpp"
#include "SystemMngt/platformelementlibrary/DataStoreDefinition.hpp"
#include "SystemMngt/platformvaluetypes/Restart_impl.hpp"
#include "SystemMngt/platformvaluetypes/Stop_impl.hpp"
#include "SystemMngt/tools/Filters_map.hpp"

#include <map>
#include <iostream>

namespace Cdmw
{
namespace PlatformMngt
{

// Struct to manage information on Process
// DEFAULT: defined in the scenario ADDED 
// ADDED: added at runtime, 
// DEFAULT_REMOVED: removed at runtime a process defined in the
// scenario
enum KindOfStart{DEFAULT, ADDED, DEFAULT_REMOVED};


class Process_impl;
enum ProcBehaviour { ABNORMAL, FAILED_DEATH };

struct  ProcessHostInfo {std::string process_name; Process_impl* process_impl; KindOfStart kind_of_start;};

class Application_impl;
class ServiceBroker_impl;
class SupervisionEventNotifier;
class ProcessStatusChangeFactory;
class CommandCallback;
class CcmComponent_impl;

/**
 *Purpose:
 *<p>
 */
class Process_impl : virtual public POA_CdmwPlatformMngt::Process,
                     public DeactivableServant_impl,
                     public ManagedElement_impl
{
    
public:
    
    /**
     * The type of process command to notify.
     */
    enum ProcessCommand
    {
        COLD_RESTART,
        HOT_RESTART,
        STOP
    };
    
private:

    /**
     * Indicates whether the process is autoending or not.
     */
    CORBA::Boolean m_autoending;

    /**
     * The type of the process.
     */
    CdmwPlatformMngt::ProcessType m_type;

    /**
     * The process info.
     */
    CdmwPlatformMngt::ProcessInfo_var m_process_info;

    /**
     * The process last exit info.
     */
    CdmwPlatformMngt::ProcessExitInfo m_last_exit_info;


    /**
    * Maintains the current number or restart attempt
     */
    CORBA::Long m_current_restart_attempt;

    /**
     * The flag for created_dynamically .
     */
    bool m_created_dynamically;

    /**
    * the flag of kind of start
    */

    KindOfStart m_kind_of_start;
    
    /**
    * The initialisation timeout. For unmanaged process only.
    * After this timeout is expired, the unmamaged process is set to RUNNING.
    */
    unsigned long m_initialisation_timeout;
    
    /**
     * The process pid.
     */
    CdmwPlatformMngt::ProcessPID m_pid;
    
    /**
     * The process previous pid.
     */
    CdmwPlatformMngt::ProcessPID m_previous_pid;

    /* poa used */
    PortableServer::POA_var m_poa;
    
    CdmwPlatformMngtBase::StartupKind_var m_startup_kind;
    
    //counters of components notification 
    int m_comp_primary;
    int m_comp_passive;
    int m_comp_active;

protected: 
    
    /**
     * The process host name.
     */
    CORBA::String_var m_proc_host_name;

    /**
     * The process application name.
     */
    CORBA::String_var m_proc_appli_name;
    
    /**
     * The process system name.
     */
    CORBA::String_var m_proc_syst_name;

    /**
     * The process key.
     */
    CORBA::String_var m_proc_key;
    

    /**
     * The host executing the process.
     */
    CdmwPlatformMngt::Host_var m_host;
    
    /**
     * The application managing this process.
     */
    Application_impl* m_application;

    /**
     * The process status change factory.
     */
    ProcessStatusChangeFactory* m_procStatusChangeFactory;
    
    /**
     * The process restart command factory.
     */
    RestartFactory* m_procRestartFactory;
        
    /**
     * The process stop command factory.
     */
    StopFactory* m_procStopFactory;
    
    /**
     * The command callback for process starting.
     */
    CommandCallback* m_process_start_cbk;
    
    /**
     * The acknowledgement callback for process stopping.
     */
    CommandCallback* m_process_stop_cbk;
    
    /**
     * The process starting command result.
     */
    CdmwPlatformMngt::ProcessCommandResult m_starting_result;
    
    /**
     * The process stopping command result.
     */
    CdmwPlatformMngt::ProcessCommandResult m_stopping_result;
    
    /**
     * The process command command error info.
     */
    CORBA::String_var m_creation_error_info;
    
    /**
     * The process command command error issuer.
     */
    CORBA::String_var m_creation_error_issuer;
    
    /**
     * Process datastore
     */
    ProcessDataStore* m_process_ds;
    
    /**
     * The status of the process in datastore
     */
    CdmwPlatformMngt::ProcessStatus m_status_in_ds;
    
    /**
     * The dependency graph of components used for the initialisation of the process.
     */
    CdmwPlatformMngt::GraphElements_var m_component_init_graph;
    
    /**
     * The process key.
     */
    std::string m_host_issuer;
    
    //notify process command event  
    void notifyCommandEvent(ProcessCommand proc_command, const char* issuer,
                             CORBA::Boolean emergency=false);
    //get host element path for supervision commands
    const char* get_sup_host_path ();  

public:

    /**
      * The mutex used to synchronise the initialisation phase of UNMANAGED PROCESSES
      */
    OsSupport::Mutex* m_init_mutex;



          
    /**
      * Condition on which the thread is sleeping to wait 
      * for the expiration of initialisation_timeout    
      */
      
    OsSupport::Condition* m_init_condition;
    
    /**
      * The timer used during the initialisation of UNMANAGED PROCESSES
      */
    
    SingleShotTimer* m_timer;

  

    /**
     * The life cycle of this process.
     */
    CdmwPlatformMngt::LifeCycleDef_var m_lifeCycle;

    /**
     * The monitoring info.
     */
    CdmwPlatformMngt::CompleteMonitoringInfo_var m_monitoring_info;

    /**
     * The date and the time of the last error event.
     */
    CdmwPlatformMngtBase::EventHeader m_lastErrorHeader;

    /**
     * The issuer of the last error event.
     */
    std::string m_lastErrorIssuer;

    /**
     * The counter of init steps.
     */
    unsigned int m_stepCounter;

    /**
     * The status of the last action.
     */
    CdmwPlatformMngt::RequestStatus m_lastRequestStatus;

   /**
    * The indicator of the relevant status.
    */   
    bool m_indicator_relevant_status;
    
   
    /**
     * The type of stop requested
     */
    bool m_stop_emergency;

private:

    /**
     *Purpose:
     *<p> Copy constructor is not allowed.
     */
    Process_impl(const Process_impl& rhs);

    /**
     * Purpose:
     * <p> Assignment operator is not allowed.
     */ 
    Process_impl& operator=(const Process_impl& rhs);

/**
     *Purpose:
     *<p>
     * Performs the automatic action requested when the number of executions of automatic restart actions exceeds maximum number of retries.
     */
    virtual int perform_max_nbr_reached_action ()
             throw (CdmwPlatformMngt::IncompatibleStatus,
                    CORBA::SystemException);



     
protected:

    /**
     *Purpose:
     *<p> constructor.
     */
    Process_impl(
        PortableServer::POA_ptr poa,
        CdmwPlatformMngt::ProcessDef* process_def,
        const char* process_key, const char* process_path,
        Application_impl* application,
        CdmwPlatformMngt::RWElementRepository_ptr element_repository);

 
    
    /**
     *Purpose:
     *<p> Set the process record in data store
     *
     */
    void set_record_in_datastore();
            
    /**
     *Purpose:
     *<p> Remove the process record in datastore
     *
     */
    void remove_record_in_datastore();
    
    /**
     *Purpose:
     *<p> Get the process record from data store
     *
     */
    CdmwPlatformMngt::ProcessRecord* get_record_from_datastore();
    
    /**
     *Purpose:
     *<p> Get the current state name of state machine
     *
     */
    virtual
    const char* get_state_machine_state_name() = 0;
    
    /**
     *Purpose:
     *<p> Set the current state name of state machine
     *
     */
    virtual
    void set_state_machine_state_name(const char* state_name) = 0;
    


    /**
     *Purpose:
     *<p> Gets the process's states.
     * This method must exclusivaly be used by the state machine.
     */
    virtual
    CdmwPlatformMngt::ProcessStatus getStatus()
        throw( CORBA::SystemException ) = 0;
        
    /**
     *Purpose:
     *<p> Gets the process's previous state.
     * This method must exclusivaly be used by the state machine.
     */
    virtual
    CdmwPlatformMngt::ProcessStatus getPreviousStatus()
        throw( CORBA::SystemException ) = 0;       

    /**
     *Purpose:
     *<p> set previous status to state machine
     * 
     * 
     */
    virtual  void set_state_machine_previous_state_name(const char* state_name) throw( CORBA::SystemException ) = 0;
    


public:
    virtual CdmwPlatformMngt::ProcessDelegateWrapper_ptr getProcessDelegateWrapper() = 0;

    /**
     *Purpose:
     *<p> Default destructor.
     */
    virtual
    ~Process_impl();
    
    /**
     *Purpose:
     *<p> Remove the process object
     */
    void remove();


public:

    /**
     *Purpose:
     *<p> Ends the creation of the process servant
     *
     *@exception HostNotFound if the host where the process will be run is
     * not found.
     *@exception DuplicateEntity if there is a duplicate entity name among
     * the entities of the process
     *@exception DuplicateService if there is a duplicate service name among
     * the service definitions of the process
     *@exception DuplicateStep if there is a duplicate step label among
     * the steps of the process
     */
    virtual
    void end_creation(CdmwPlatformMngt::ProcessDef* process_def)
            throw (CdmwPlatformMngt::HostNotFound,
                   CdmwPlatformMngt::DuplicateEntity,
                   CdmwPlatformMngt::DuplicateService,
                   CdmwPlatformMngt::DuplicateStep,
                   OutOfMemoryException,
               CosPropertyService::MultipleExceptions,
                   CORBA::SystemException) = 0;
                   
    /**
     *Purpose:
     *<p> Synchronise with datastore
     *
     */
    virtual
    void synchro_with_datastore();
    
     /**
     *Purpose:
     *<p> Synchronise entities with datastore
     *
     */
    virtual
    void synchro_entities_with_datastore();

     /**
     *Purpose:
     *<p> Synchronise all its components with datastore
     *
     */
    void synchro_components_with_datastore();    
    
    
    /**
     *Purpose:
     *<p> Synchronise the processing
     *
     */
    void synchro_processing();
    
    /**
     *Purpose:
     *<p> Synchronise the processing to initial state
     *
     */
    virtual
    void synchro_processing_to_initial () = 0;
    
    /**
     *Purpose:
     *<p> Synchronise the processing to initialised state
     *
     */
    virtual
    void synchro_processing_to_initialised () = 0;
    
    /**
     *Purpose:
     *<p> Synchronise the processing to running state
     *
     */
    virtual
    void synchro_processing_to_running () = 0;
    

    /**
     *Purpose:
     *<p> Called when a user application error has occured.
     *
     *@param error_info the information about the error.
     */
    virtual
    void error_event(
        const CdmwPlatformMngtBase::EventHeader& header,
        const char* issuer,
        const char* error_info ) = 0;

    /**
     *Purpose:
     *<p> Called when the process has ended its running step.
     */
    virtual
    void ending_event(
        const CdmwPlatformMngtBase::EventHeader& header ) = 0;
        
    /**
     *Purpose:
     *<p> Called when a monitoring failure event has happened.
     *
     *@exception IncompatibleType if the process is not a managed process.
     */
    virtual
    void monitoringFailure_event(
        const CdmwPlatformMngtBase::EventHeader& header )
            throw( CdmwPlatformMngt::IncompatibleType,
                   CORBA::SystemException );

    /**
     *Purpose:
     *<p> Called when a technical error has occured.
     *
     *@param error_info the information about the error.
     */
    virtual
    void invalidate_event(
        const CdmwPlatformMngtBase::EventHeader& header,
        const char* reason) = 0;
        
    
    /**
     * Purpose:
     * <p> Modifies the property of process and its components.
     * 
     * e.g. : REPLICATION_ROLE property 
     */
    void property_change_event (const char* attr_name,
            const CORBA::Any& prop_value,
            const char* component_name)
    throw (CORBA::SystemException,
           CdmwPlatformMngt::ComponentNotFound);
           
           
    int Process_impl::get_number_of_ft_components() 
        throw(CORBA::SystemException);
        
    int Process_impl::get_number_of_lb_components() 
        throw(CORBA::SystemException);
           

    /*
    *Purpose:
    *<p> set a replication role property for process 
    *    and for its components ; 
    */
    void reset_replication_role() 
    throw(CORBA::SystemException);
    
    /**
     *Purpose:
     *<p> Get the internal status from state machine.
     */
     virtual
     CdmwPlatformMngt::ProcessStatus get_internal_status() = 0;
     
     /**
     *Purpose:
     *<p> Get the status stored in datastore.
     */
     CdmwPlatformMngt::ProcessStatus get_status_in_ds();
     
     /**
     *Purpose:
     *<p> Get the startup kind stored in datastore.
     */
     CdmwPlatformMngtBase::StartupKind* get_startup_kind_from_ds();
      
     
     virtual void set_pid ( CdmwPlatformMngt::ProcessPID pid );

    /**
     *Purpose:
     *<p>
     * Implements the
     * IDL:thalesgroup.com/CdmwPlatformMngt/Process/type:1.0
     * operation
     */
    virtual
    CdmwPlatformMngt::ProcessType type()
            throw(CORBA::SystemException);
    /**
     *Purpose:
     *<p> facade to get an attribute
     *
     */

     void get_attribute(const char * attr_name, CORBA::Any_out ret_attr , CdmwPlatformMngt::Owner flag)  
        throw (CORBA::SystemException, CdmwPlatformMngt::AttributeNotFound);

    /**
     *Purpose:
     *<p> facade to set an attribute
     *
     */

     void set_attribute(const char * attr_name, const CORBA::Any & attr , CdmwPlatformMngt::Owner flag)  
        throw (CORBA::SystemException, CdmwPlatformMngt::AttributeNotFound, CdmwPlatformMngt::ReadOnlyAttribute);

   
    void set_requested_restart_warmth(CdmwPlatformMngt::RestartWarmth requested_restart_warmth);
    /*const *
     *Purpose:
     *<p>
     * Implements the
     * IDL:thalesgroup.com/CdmwPlatformMngt/Process/get_status:1.0
     * operation
     */
    virtual
    CdmwPlatformMngt::ProcessStatus get_status(
        CORBA::String_out status_info)
            throw(CORBA::SystemException) = 0;

    /*
     *Purpose:
     *<p> it returns the string of trace filters ;
     * Implements the
     * IDL:thalesgroup.com/CdmwPlatformMngt/Process/get_status:1.0
     * operation
     */
    char * get_status() throw(CORBA::SystemException) ;

    /**
     *Purpose:
     *<p>
     * Implements the
     * IDL:thalesgroup.com/CdmwPlatformMngt/Process/embedded_service:1.0
     * operation
     */
    virtual CORBA::Object_ptr embedded_service()
            throw(CORBA::SystemException);
     
    /**
     *Purpose:
     *<p>
     * Implements the
     * IDL:thalesgroup.com/CdmwPlatformMngt/Process/pid:1.0
     * operation
     */       
    virtual CdmwPlatformMngt::ProcessPID pid()
            throw(CORBA::SystemException);
            
    /**
     *Purpose:
     *<p>
     * Implements the
     * IDL:thalesgroup.com/CdmwPlatformMngt/Process/previous_pid:1.0
     * operation
     */       
    virtual CdmwPlatformMngt::ProcessPID previous_pid()        
            throw(CORBA::SystemException);
                    
    /**
     *Purpose:
     *<p>
     * Implements the
     * IDL:thalesgroup.com/CdmwPlatformMngt/Process/created_dynamically:1.0
     * operation
     */
    virtual CORBA::Boolean created_dynamically()
            throw(CORBA::SystemException);
    
   
    
    
    /**
     *Purpose:
     *<p>
     * Implements the
     * IDL:thalesgroup.com/CdmwPlatformMngt/Process/restart_attempt:1.0
     * operation
     */      
    virtual CORBA::ULong restart_attempt()
            throw(CORBA::SystemException);
    
   /**
     *Purpose:
     *<p>
     *Set the current number of restart attempt
    */
     virtual void set_restart_attempt (CORBA::Long nrestart)
              throw(CORBA::SystemException);
 
    /**
     *Purpose:
     *<p>
     * Implements the
     * IDL:thalesgroup.com/CdmwPlatformMngt/Process/start_time:1.0
     * operation
     */     
    virtual CdmwPlatformMngtBase::TimeStamp start_time()
            throw(CORBA::SystemException);
    
    /**
     *Purpose:
     *<p>
     * Implements the
     * IDL:thalesgroup.com/CdmwPlatformMngt/Process/stop_time:1.0
     * operation
     */    
    virtual CdmwPlatformMngtBase::TimeStamp stop_time()
            throw(CORBA::SystemException);
 
    /**
     *Purpose:
     *<p>
     * Implements the
     * IDL:thalesgroup.com/CdmwPlatformMngt/Process/is_auto_ending:1.0
     * operation
     */
    virtual
    CORBA::Boolean is_auto_ending()
      throw(CORBA::SystemException);
      
    /**
     *Purpose:
     *<p>
     * Implements the
     * IDL:thalesgroup.com/CdmwPlatformMngt/Process/set_auto_ending:1.0
     * operation
     */
    virtual
    void set_auto_ending(CORBA::Boolean auto_ending)
        throw (CdmwPlatformMngt::IncompatibleStatus,
               CORBA::SystemException) = 0;

    /**
     *Purpose:
     *<p>
     * Implements the
     * IDL:thalesgroup.com/CdmwPlatformMngt/Process/get_parent:1.0
     * operation
     */
    CdmwPlatformMngt::ManagedElement_ptr Process_impl::get_parent() 
        throw (CORBA::SystemException);

    /**
     *Purpose:
     *<p>
     * Implements the
     * IDL:thalesgroup.com/CdmwPlatformMngt/Process/get_host:1.0
     * operation
     */
    virtual
    CdmwPlatformMngt::Host_ptr get_host()
      throw (CORBA::SystemException);
    /**
     *Purpose:
     *<p>
     * Implements the
     * IDL:thalesgroup.com/CdmwPlatformMngt/Process/set_host:1.0
     * operation
     */
    virtual
    void set_host(const char* host_name)
            throw (CdmwPlatformMngt::HostNotFound,
                   CdmwPlatformMngt::IncompatibleStatus,
                   CORBA::SystemException) = 0;

    /**
     *Purpose:
     *<p>
     * Implements the
     * IDL:thalesgroup.com/CdmwPlatformMngt/Process/get_process_info:1.0
     * operation
     */
    virtual
    CdmwPlatformMngt::ProcessInfo* get_process_info()
      throw (CORBA::SystemException);
    /**
     *Purpose:
     *<p>
     * Implements the
     * IDL:thalesgroup.com/CdmwPlatformMngt/Process/set_process_info:1.0
     * operation
     */
    virtual
    void set_process_info(const CdmwPlatformMngt::ProcessInfo& process_info)
            throw (CdmwPlatformMngt::IncompatibleStatus,
                   CORBA::SystemException) = 0;

    /**
     *Purpose:
     *<p>
     * Implements the
     * IDL:thalesgroup.com/CdmwPlatformMngt/Process/get_process_mode:1.0
     * operation
     */
    /*
    virtual
    CdmwPlatformMngt::ProcessMode  get_process_mode()
      throw (CORBA::SystemException);
    */
    
    /**
     *Purpose:
     *<p>
     * Implements the
     * IDL:thalesgroup.com/CdmwPlatformMngt/Process/get_memory_resident:1.0
     * operation
     */
    virtual
    bool get_memory_resident()
      throw (CORBA::SystemException);

    /**
     *Purpose:
     *<p>
     * Implements the
     * IDL:thalesgroup.com/CdmwPlatformMngt/Process/set_memory_resident:1.0
     * operation
     */
    virtual void set_memory_resident( bool memory_resident)
      throw (CORBA::SystemException);

    /**
     *Purpose:
     *<p>
     * Implements the
     * IDL:thalesgroup.com/CdmwPlatformMngt/Process/get_core_dump_size:1.0
     * operation
     */
    virtual
    CORBA::Long get_core_dump_size()
      throw (CORBA::SystemException);


    /**
     *Purpose:
     *<p>
     * Implements the
     * IDL:thalesgroup.com/CdmwPlatformMngt/Process/get_life_cycle:1.0
     * operation
     */
    virtual
    CdmwPlatformMngt::LifeCycleDef* get_life_cycle()
            throw (CdmwPlatformMngt::IncompatibleType,
                   CORBA::SystemException);

    /**
     *Purpose:
     *<p>
     * Implements the
     * IDL:thalesgroup.com/CdmwPlatformMngt/Process/set_life_cycle:1.0
     * operation
     */
    virtual
    void set_life_cycle (const CdmwPlatformMngt::LifeCycleDef& life_cycle)
            throw (CdmwPlatformMngt::DuplicateStep,
                   CdmwPlatformMngt::IncompatibleType,
                   CdmwPlatformMngt::IncompatibleStatus,
                   CORBA::SystemException);

    /**
     *Purpose:
     *<p>
     * Implements the
     * IDL:thalesgroup.com/CdmwPlatformMngt/Process/get_monitoring_info:1.0
     * operation
     */
    virtual
    CdmwPlatformMngt::CompleteMonitoringInfo get_monitoring_info()
            throw (CdmwPlatformMngt::IncompatibleType,
                   CORBA::SystemException);

    /**
     *Purpose:
     *<p>
     * Implements the
     * IDL:thalesgroup.com/CdmwPlatformMngt/Process/set_monitoring_info:1.0
     * operation
     */
    virtual
    void set_monitoring_info(
        const CdmwPlatformMngt::CompleteMonitoringInfo& monitoring_info)
            throw (CdmwPlatformMngt::IncompatibleType,
                   CdmwPlatformMngt::IncompatibleStatus,
                   CORBA::SystemException);

    /**
     *Purpose:
     *<p>
     * Implements the
     * IDL:thalesgroup.com/CdmwPlatformMngt/Process/get_exit_autorestart_info:1.0
     * operation
     */
    virtual CdmwPlatformMngt::ExitAutoRestartInfo* get_exit_autorestart_info ()
             throw (CORBA::SystemException);
             
             
    /**
     *Purpose:
     *<p>
     * Implements the
     * IDL:thalesgroup.com/CdmwPlatformMngt/Process/set_exit_autorestart_info:1.0
     * operation
     */
    virtual void set_exit_autorestart_info (
        const CdmwPlatformMngt::ExitAutoRestartInfo & exit_autorestart_info)
             throw (CdmwPlatformMngt::IncompatibleStatus,
                    CORBA::SystemException);
    

    /**
     *Purpose:
     *<p>
     * Performs the automatic action requested for the process on exit or failure
     */
    virtual int perform_exit_autorestart_action (ProcBehaviour proc_behaviour)
             throw (CdmwPlatformMngt::IncompatibleStatus,
                    CORBA::SystemException);


    /**
     *Purpose:
     *<p>
     * Implements the
     * IDL:thalesgroup.com/CdmwPlatformMngt/Process/get_snapshot:1.0
     * operation
     */
    virtual
    CdmwPlatformMngt::ProcessSnapshot* get_snapshot()
      throw (CORBA::SystemException);

    /**
     *Purpose:
     *<p>
     * Implements the
     * IDL:thalesgroup.com/CdmwPlatformMngt/Process/get_run_as_root:1.0
     * operation
     */
    virtual
    bool get_run_as_root()
      throw (CORBA::SystemException);
      
    /**
     *Purpose:
     *<p>
     * Implements the
     * IDL:thalesgroup.com/CdmwPlatformMngt/Process/initialise:1.0
     * operation
     */
    virtual
    CdmwPlatformMngt::RequestStatus initialise(
        const CdmwPlatformMngtBase::StartupKind& startup_kind)
            throw (CdmwPlatformMngt::IncompatibleStatus,
                   CORBA::SystemException) = 0;

    /**
     *Purpose:
     *<p>
     * Implements the
     * IDL:thalesgroup.com/CdmwPlatformMngt/Process/next_step:1.0
     * operation
     */
    virtual
    CdmwPlatformMngt::RequestStatus next_step()
            throw (CdmwPlatformMngt::InvalidStep,
                   CdmwPlatformMngt::IncompatibleType,
                   CdmwPlatformMngt::IncompatibleStatus,
                   CORBA::SystemException);

    /**
     *Purpose:
     *<p>
     * Implements the
     * IDL:thalesgroup.com/CdmwPlatformMngt/Process/run:1.0
     * operation
     */
    virtual
    CdmwPlatformMngt::RequestStatus run()
            throw (CdmwPlatformMngt::NotReadyToRun,
                   CdmwPlatformMngt::IncompatibleType,
                   CdmwPlatformMngt::IncompatibleStatus,
                   CORBA::SystemException);

    /**
     *Purpose:
     *<p>
     * Implements the
     * IDL:thalesgroup.com/CdmwPlatformMngt/Process/stop:1.0
     * operation
     */
    virtual
    CdmwPlatformMngt::RequestStatus stop(CORBA::Boolean emergency)
        throw (CORBA::SystemException) = 0;
    
      
    /**
    * Stop the process.
    * The behaviour is the same as stop (in boolean emergency).
    * This command is notified to registered observers.
    */
    virtual  CdmwPlatformMngt::RequestStatus stop_process(CORBA::Boolean emergency)
        throw (CORBA::SystemException) = 0;
    
        
    /**
     *Purpose:
     *<p>
     * Implements the
     * IDL:thalesgroup.com/CdmwPlatformMngt/Process/cold_restart:1.0
     * operation
     */  
    virtual CdmwPlatformMngt::RequestStatus cold_restart (
        const CdmwPlatformMngtBase::StartupKind & startup_kind)
      throw (CORBA::SystemException, CdmwPlatformMngt::IncompatibleStatus);

    /**
     *Purpose:
     *<p>
     * Implements the
     * IDL:thalesgroup.com/CdmwPlatformMngt/Process/hot_restart:1.0
     * operation
     */
    virtual CdmwPlatformMngt::RequestStatus hot_restart (
        const CdmwPlatformMngtBase::StartupKind & startup_kind)
      throw (CORBA::SystemException, CdmwPlatformMngt::IncompatibleStatus);

    /**
     *Purpose:
     *<p>
     * Implements the
     * restart operation
     */
    virtual CdmwPlatformMngt::LongRequestStatus restart ( const CdmwPlatformMngt::RestartWarmth restartWarmth,
    const CdmwPlatformMngtBase::StartupKind & startup_kind, bool emergency)
    throw (CdmwPlatformMngt::IncompatibleStatus, CORBA::SystemException);      
      
    /**
     *Purpose:
     *<p>
     * Implements the
     * IDL:thalesgroup.com/CdmwPlatformMngt/EntityContainer/get_number_of_entities:1.0
     * operation
     */
    virtual
    CORBA::ULong get_number_of_entities()
        throw (CORBA::SystemException);

    /**
     *Purpose:
     *<p>
     * Implements the
     * IDL:thalesgroup.com/CdmwPlatformMngt/EntityContainer/get_all_entity_names:1.0
     * operation
     */
    virtual
    void get_all_entity_names(
        CORBA::ULong how_many,
        CdmwPlatformMngt::ElementNames_out entity_names,
        CdmwPlatformMngt::ElementNamesIterator_out rest)
            throw (CORBA::SystemException);

    /**
     *Purpose:
     *<p>
     * Implements the
     * IDL:thalesgroup.com/CdmwPlatformMngt/EntityContainer/get_entity:1.0
     * operation
     */
    virtual
    CdmwPlatformMngt::Entity_ptr get_entity(const char* entity_name)
            throw (CdmwPlatformMngt::EntityNotFound,
                   CORBA::SystemException);

    /**
     *Purpose:
     *<p>
     * Implements the
     * IDL:thalesgroup.com/CdmwPlatformMngt/EntityContainer/get_entities:1.0
     * operation
     */
    virtual
    CORBA::Boolean get_entities(
        const CdmwPlatformMngt::ElementNames& entity_names,
        CdmwPlatformMngt::Entities_out nentities)
            throw (CORBA::SystemException);

    /**
     *Purpose:
     *<p>
     * Implements the
     * IDL:thalesgroup.com/CdmwPlatformMngt/EntityContainer/get_all_entities:1.0
     * operation
     */
    virtual
    void get_all_entities(
        CORBA::ULong how_many,
        CdmwPlatformMngt::Entities_out nentities,
        CdmwPlatformMngt::EntitiesIterator_out rest)
            throw (CORBA::SystemException);

    /**
     *Purpose:
     *<p>
     * Implements the
     * IDL:thalesgroup.com/CdmwPlatformMngt/RWEntityContainer/add_entity:1.0
     * operation
     */
    virtual
    CdmwPlatformMngt::Entity_ptr add_entity(const CdmwPlatformMngt::EntityDef & entity_def)
            throw (CdmwPlatformMngt::EntityAlreadyExists,
                   CORBA::SystemException);

    /**
     *Purpose:
     *<p>
     * Implements the
     * IDL:thalesgroup.com/CdmwPlatformMngt/RWEntityContainer/remove_entity:1.0
     * operation
     */
    virtual
    void remove_entity(const char* entity_name)
            throw (CdmwPlatformMngt::EntityNotFound,
                   CORBA::SystemException);

    /**
     *Purpose:
     *<p>
     * Implements the
     * IDL:thalesgroup.com/CdmwPlatformMngtService/ServiceDefContainer/get_service_def:1.0
     * operation
     */
    virtual
    CdmwPlatformMngtService::ServiceProviderID* get_service_def(
        const char* service_name)
            throw (CdmwPlatformMngtService::ServiceNotFound,
                   CORBA::SystemException);

    /**
     *Purpose:
     *<p>
     * Implements the
     * IDL:thalesgroup.com/CdmwPlatformMngtService/RWServiceDefContainer/add_service_def:1.0
     * operation
     */
    virtual
    void add_service_def(
        const CdmwPlatformMngtService::ServiceDef& service_def)
            throw (CdmwPlatformMngtService::ServiceAlreadyExists,
                   CORBA::SystemException);

    /**
     *Purpose:
     *<p>
     * Implements the
     * IDL:thalesgroup.com/CdmwPlatformMngtService/RWServiceDefContainer/remove_service_def:1.0
     * operation
     */
    virtual
    void remove_service_def(const char* service_name)
            throw (CdmwPlatformMngtService::ServiceNotFound,
                   CORBA::SystemException);
                   
    /**
     *Purpose:
     *<p>
     * Implements the
     * IDL:thalesgroup.com/CdmwPlatformMngt/RWServiceDefContainer/remove_all_service_defs:1.0
     * operation
     */ 
    virtual              
    void remove_all_service_defs()
            throw (CORBA::SystemException);

    /**
     *Purpose:
     *<p>
     * Implements the
     * IDL:thalesgroup.com/CdmwPlatformMngt/PullMonitorable/is_alive:1.0
     * operation
     */
    void is_alive();

    /**
     *Purpose:
     *<p> Gets the application agent in charge of this process. This method
     * is called within the state machine.
     *
     *@return the application agent or a NIL reference if there is no agent
     * for the application in charge of this process.
     *
     *@exception HostNotFound if the host cannot be found.
     *@exception HostNotReachable if the host doesn't respond.
     *@exception CreationError if the application agent returned a fatal error
     * and aborts its creation.
     *@exception ApplicationAgentAlreadyExists if the application agent
     * already exists.
     *@exception InvalidInfo if agent_info contains invalid information.
     *@exception CreationTimeout if the agent has not acknowledged its
     * creation during the allowed timeout.
     *@exception AgentNotFoundException if created agent cannot be found
     *@exception CORBA::SystemException
     */
    CdmwPlatformMngt::ApplicationAgent_ptr getApplicationAgent()
        throw (CdmwPlatformMngt::HostNotFound,
               CdmwPlatformMngt::HostNotReachable,
               CdmwPlatformMngt::CreationError,
               CdmwPlatformMngt::ApplicationAgentAlreadyExists,
               CdmwPlatformMngt::InvalidInfo,
               CdmwPlatformMngt::CreationTimeout,
               AgentNotFoundException,
               CORBA::SystemException);
               
    /**
     *Purpose:
     *<p> Search from daemon the application agent in charge of this process.
     *
     *@return the application agent or a NIL reference if there is no agent
     *
     *@exception HostNotFound if the host cannot be found.
     *@exception HostNotReachable if the host doesn't respond.
     *@exception CORBA::SystemException
     */
    CdmwPlatformMngt::ApplicationAgent_ptr searchApplicationAgent()
        throw (CdmwPlatformMngt::HostNotFound,
               CdmwPlatformMngt::HostNotReachable,
               CORBA::SystemException);
  
    /**
     *Purpose:
     *<p> Actually sets the process as ending itself.
     *
     *@param auto_ending set if process can end itself.
     */    
    void setAutoending(CORBA::Boolean auto_ending);

    /**
     *Purpose:
     *<p> Actually changes the host where the process will be running.
     *
     *@param host_name the name of the host.
     *
     *@exception HostNotFound if the host where the process will be run is
     * not found.
     */
    void setHost(const char* host_name)
            throw (CdmwPlatformMngt::HostNotFound,
                   CORBA::SystemException);

    /**
     *Purpose:
     *<p> Actually changes the information needed for the process creation.
     *
     *@param process_info the information of the process.
     */
    virtual
    void setProcessInfo(
        const CdmwPlatformMngt::ProcessInfo& process_info);

    /**
     *Purpose:
     *<p> Actually changes the information about the monitoring of the process.
     *
     *@param monitoring_info the new information about the monitoring of the
     * process.
     */
    void setMonitoringInfo(
        const CdmwPlatformMngt::CompleteMonitoringInfo& monitoring_info);

    /**
     *Purpose:
     *<p> Called when an error has occured. This method is called within
     * the state machine.
     */
    void errorEvent(
        const CdmwPlatformMngtBase::EventHeader& header,
        const char* issuer,
        const char* error_info);

    /**
     *Purpose:
     *<p> Called when the process has ended its running step. This method
     * is called within the state machine.
     */
    void endingEvent(
        const CdmwPlatformMngtBase::EventHeader& header);

    /**
     *Purpose:
     *<p> Called when a technical error has occured. This method is called
     * within the state machine.
     */
    void invalidateEvent(
        const CdmwPlatformMngtBase::EventHeader& header,
        const char* reason);

    /**
     * Moves the state machine to specified state.
     */
    virtual
    void setState(const char* stateName) = 0;
        
    
    /**
     *Purpose:
     *<p> return the application managing the process
     *
     */
    Application_impl* get_application();
    
        
    /**
     *Purpose:
     *<p> return the host name where process is running
     *   !! do not desallocate !!
     *
     */
    const char* get_process_host_name();
    
    /**
     *Purpose:
     *<p> return the process key (host + process)
     *   !! do not desallocate !!
     *
     */
    const char* get_process_key();
    
    
    /**
     *Purpose:
     *<p>
     * Implements the
     * IDL:thalesgroup.com/CdmwPlatformMngt/Process/set_create_process_result:1.0
     * operator
     */
     virtual
     void set_create_process_result (
        CdmwPlatformMngt::ProcessCommandResult result,
        const char* error_info,
        const char* error_issuer,
        CdmwPlatformMngt::ProcessDelegateWrapper_ptr process_delegate_wrap)
            throw (CORBA::SystemException);

     
     /**
     *Purpose:
     *<p>
     * Implements the
     * IDL:thalesgroup.com/CdmwPlatformMngt/Process/set_initialisation_result:1.0
     * operator
     */
     virtual
     void set_initialisation_result (CdmwPlatformMngt::ProcessCommandResult result)
            throw (CORBA::SystemException);
     
     /**
     *Purpose:
     *<p>
     * Implements the
     * IDL:thalesgroup.com/CdmwPlatformMngt/Process/set_next_step_result:1.0
     * operator
     */
     virtual
     void set_next_step_result (CdmwPlatformMngt::ProcessCommandResult result,
                                CORBA::ULong step_nbr)
            throw (CORBA::SystemException);
     
     /**
     *Purpose:
     *<p>
     * Implements the
     * IDL:thalesgroup.com/CdmwPlatformMngt/Process/set_run_result:1.0
     * operator
     */
     virtual
     void set_run_result (CdmwPlatformMngt::ProcessCommandResult result)
            throw (CORBA::SystemException);
     
     /**
     *Purpose:
     *<p>
     * Implements the
     * IDL:thalesgroup.com/CdmwPlatformMngt/Process/set_stop_result:1.0
     * operator
     */
     virtual
     void set_stop_result (CdmwPlatformMngt::ProcessCommandResult result)
            throw (CORBA::SystemException);

     /**
     *Purpose:
     *<p>
     * Set the properties CdmwPlatformMngt::FORCE_MANUAL_START
     * @param force_manual_start attribute
     */
     virtual void set_force_manual_start(const bool force_manual_start)
      throw (CORBA::SystemException,CdmwPlatformMngt::AttributeNotFound ); 

     /**
     *Purpose:
     *<p>
     * Get the properties CdmwPlatformMngt::FORCE_MANUAL_START
     * @return force_manual_start attribute
     */

    virtual bool get_force_manual_start()
      throw (CORBA::SystemException, CdmwPlatformMngt::AttributeNotFound); 

     /**
     *Purpose:
     *<p>
     * Implements the
     * IDL:thalesgroup.com/CdmwPlatformMngt/Process/set_last_exit_code:1.0
     * operation
     * @param last_exit_code attribute 
     */
     virtual void set_last_exit_code(const CdmwPlatformMngt::ProcessExitInfo& last_exit_code)throw (CORBA::SystemException);

     /**
     *Purpose:
     *<p>
     * Implements the
     * IDL:thalesgroup.com/CdmwPlatformMngt/Process/get_last_exit_code:1.0
     * operation
     * @return last_exit_code attribute 
     */
     CdmwPlatformMngt::ProcessExitInfo get_last_exit_code()throw (CORBA::SystemException);

     /**
     *Purpose:
     *<p>
     * Implements the
     * IDL:thalesgroup.com/CdmwPlatformMngt/Process/set_process_arguments:1.0
     * operation
     * @param process_arguments attribute 
     */
     virtual void set_process_arguments(const char * process_arguments)throw (CORBA::SystemException);

     /**
     *Purpose:
     *<p>
     * Implements the
     * IDL:thalesgroup.com/CdmwPlatformMngt/Process/get_process_arguments:1.0
     * operation
     * @return process_arguments attribute 
     */
     virtual char * get_process_arguments()throw (CORBA::SystemException);

     /**
     *Purpose:
     *<p>
     * Implements the
     * IDL:thalesgroup.com/CdmwPlatformMngt/Process/set_indicator_relevant_status:1.0
     * operation
     * @param set_indicator_relevant_status attribute 
     */
     virtual void set_indicator_relevant_status(const bool indicator_relevant_status) throw (CORBA::SystemException);

     /**
     *Purpose:
     *<p>
     * Implements the
     * IDL:thalesgroup.com/CdmwPlatformMngt/Process/get_indicator_relevant_status:1.0
     * operation
     * @return get_indicator_relevant_status attribute 
     */
     virtual bool get_indicator_relevant_status()  throw (CORBA::SystemException);

     void set_trace_filtering(const char *trace_filtering) throw (CORBA::SystemException, CdmwPlatformMngt::ParsingException , CdmwPlatformMngt::CastException);

     /*
     *<p> internal method to apply the trace filtering ...
     *
     * @return 0 as ok ,  -1 as error
     */

     int apply_trace_filters() throw (CORBA::SystemException);
     

     /*
     *Purpose: 
     *<p> 
     * used to remove blanks at the beginning
     * and at the end of a string
     */

    void stripLeadingAndTrailingBlanks(string& StringToModify);
 
    char * get_trace_filtering() throw (CORBA::SystemException);

     /**
     *Purpose:
     *<p>
     * Implements the
     * IDL:thalesgroup.com/CdmwPlatformMngt/Process/set_init_graph:1.0
     * operation
     * @param component init graph attribute 
     */
     virtual void set_init_graph(const CdmwPlatformMngt::GraphElements& component_graph) throw (CORBA::SystemException);


     /**
     *Purpose:
     *<p>
     * Implements the
     * IDL:thalesgroup.com/CdmwPlatformMngt/Process/set_stop_graph:1.0
     * operation
     * @param component stop graph attribute 
     */
     virtual void set_stop_graph(const CdmwPlatformMngt::GraphElements& component_graph) throw (CORBA::SystemException);

     /**
      *Purpose:
      *<p>
      * Implements the
      * IDL:thalesgroup.com/CdmwPlatformMngt/ManagedElement/get_element_type:1.0
      * operation
      */
     virtual ::CdmwPlatformMngt::ElementType get_element_type()
        throw (CORBA::SystemException);

     /**
      *Purpose:
      *<p>
      * Implements the
      * IDL:thalesgroup.com/CdmwPlatformMngt/ManagedElement/get_children:1.0
      * operation
      */

     virtual CdmwPlatformMngt::Children* get_children ()
     throw (CORBA::SystemException);

    
    /**
     *Purpose:
     *<p> set the attribute created_dynamically
     */
 
//    bool created_dynamically();

    /**
     *Purpose:
     *<p> set the attribute created_dynamically
     */
    void set_created_dynamically(bool created_dynamically);

    /**
     *Purpose:
     *<p> add component to process 
     * @param component_def Component Definition
     * @exception CdmwPlatformMngt::ComponentAlreadyExists
     * @exception ComponentAlreadyExists
     * @exception ComponentServerNotFound
     * @exception DuplicateEntity
     * @exception DuplicateStep
     * @exception IncompatibleStatus
     */
    virtual CdmwPlatformMngt::CcmComponent_ptr add_component(const CdmwPlatformMngt::ComponentDef& component_def)
       throw (CORBA::SystemException, CdmwPlatformMngt::ComponentAlreadyExists, CdmwPlatformMngt::ComponentServerNotFound,
              CdmwPlatformMngt::DuplicateEntity, CosPropertyService::MultipleExceptions,
              CdmwPlatformMngt::IncompatibleStatus);

    /**
     *Purpose:
     *<p> set the kind_of_start in datastore and other ...
     */
    void set_proc_info(Cdmw::PlatformMngt::ProcessHostInfo info, char flag);


   /**
    *Purpose:
    *<p> remove component from process
    * @param component_name name of component
    * @param host_name host name of host
    * @exception IncompatibleStatus
    * @exception ComponentNotFound
    * @exception ComponentStillActive 
    */
    virtual void remove_component(const char* component_name, const char* host_name)
       throw(CORBA::SystemException, 
             CdmwPlatformMngt::ComponentNotFound, 
             CdmwPlatformMngt::ComponentStillActive, 
             CdmwPlatformMngt::IncompatibleStatus);
       
    /**
    * Returns the components of the application.
    * The first how_many components will be returned at most and the
    * rest of the components may be retrieved thanks to the iterator.
    *
    * @param how_many the number of components to return
    * @param At most the number of components requested
    * @param components_iterator the iterator to the rest of components.
    */
    virtual void get_all_components(CORBA::ULong how_many, 
                                    CdmwPlatformMngt::CcmComponents_out ncomponents, 
                                    CdmwPlatformMngt::ComponentsIterator_out rest)   
        throw(CORBA::SystemException, 
              CdmwPlatformMngt::NotComponentServer);
             
    /**
    * Returns the specified component.
    *
    * @param component_name the name of the component.
    *
    * @return the component proxy
    * @exception componentNotFound if component_name doesn't denote
    * @exception NotComponentServer if the process is not a component server
    * an existing component.
    */
    virtual CdmwPlatformMngt::CcmComponent_ptr get_component(const char* component_name)
        throw(CORBA::SystemException, 
              CdmwPlatformMngt::ComponentNotFound,
              CdmwPlatformMngt::NotComponentServer);
                                    
   /**
   * Returns the specified component.
   * @param component_name the name of the component.
   *
   * @return the component implementation
   * @exception componentNotFound if component_name doesn't denote
   * @exception NotComponentServer if the process is not a component server
   * an existing component.
   */
                
    virtual CcmComponent_impl* get_component_impl(const char* component_name)
            throw(CORBA::SystemException,
                  CdmwPlatformMngt::ComponentNotFound,
              CdmwPlatformMngt::NotComponentServer);

                     
    /**
    * It returns the number of components
    *
    * @return the number of components
    *
    */
    virtual CORBA::ULong get_number_of_components()
        throw(CORBA::SystemException, 
              CdmwPlatformMngt::NotComponentServer);
    

    /**
     * The list of components of this Process
     */
    typedef std::map <std::string, CcmComponent_impl*> ComponentsMap;


   /**
    * The vector of deployment plans that belong to this process
    */
    vector<std::string>* get_deployment_plans();
    
//    virtual void set_requested_deployment_plans (const char * deployment_plan);
    
    void stopping_components();
    
    void stop_components();
        
    void failed_error_components();

    virtual void initialise_components (const char * deployment_plan);

    virtual void set_state(const std::string& state_name) throw();

    private:
    filters_map m_filt_map;
    Cdmw::Logging::Logger* m_logger; 
    protected:
    ComponentsMap m_component_map ;
    


};





} // End namespace PlatformMngt
} // End namespace Cdmw



#endif //INCL_PLATFORMMNGT_PROCESS_HPP

