/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

#include "tao/AnyTypeCode/TypeCode.h" 
#include "Foundation/osthreads/MutexGuard.hpp"
#include "Foundation/osthreads/ReaderLockGuard.hpp"
#include "Foundation/osthreads/WriterLockGuard.hpp"
#include "Foundation/ossupport/OS.hpp"
#include "Foundation/orbsupport/ExceptionMinorCodes.hpp"
#include "Foundation/logging/Logger.hpp"
#include <Foundation/common/Locations.hpp>
#include "Foundation/commonsvcs/datastore/StorageErrorException.hpp"
#include "SystemMngt/platforminterface/ServiceNames.hpp"
#include "SystemMngt/idllib/CdmwPlatformMngtBase.stub.hpp"
#include "SystemMngt/idllib/CdmwPlatformMngtDaemon.stub.hpp"
#include "SystemMngt/idllib/CdmwPlatformMngtEvent.stub.hpp"
#include "SystemMngt/idllib/CdmwPlatformMngtSystem.stub.hpp"
#include "SystemMngt/idllib/CdmwPlatformMngtApplicationObserver.stub.hpp"
#include "SystemMngt/platformlibrary/Configuration.hpp"
#include "SystemMngt/platformlibrary/EventHeaderFactory.hpp"
#include "SystemMngt/platformlibrary/GraphChecker.hpp"
#include "SystemMngt/platformlibrary/LogMngr.hpp"
#include "SystemMngt/platformlibrary/RWServiceDefContainer_impl.hpp"
#include "SystemMngt/platformlibrary/ServiceDefContainer_impl.hpp"
#include "SystemMngt/platformlibrary/SupervisionEventNotifier.hpp"
#include "SystemMngt/platformlibrary/SynchronisableTaskDef.hpp"
#include "SystemMngt/platformlibrary/TaskFactory.hpp"
#include "SystemMngt/platformlibrary/Sequencer.hpp"
#include "SystemMngt/platformlibrary/Starter.hpp"
#include "SystemMngt/platformelementlibrary/EntityContainer_impl.hpp"
#include "SystemMngt/platformelementlibrary/RWEntityContainer_impl.hpp"
#include "SystemMngt/platformelementlibrary/EntityContainer.hpp"
#include "SystemMngt/tools/ManagedElementKey.hpp"
#include "SystemMngt/platformvaluetypes/ApplicationModeChange_impl.hpp"
#include "SystemMngt/platformvaluetypes/ApplicationStatusChange_impl.hpp"
#include "SystemMngt/platformvaluetypes/GraphElement_impl.hpp"
#include "SystemMngt/platformapplicationlibrary/Application_impl.hpp"
#include "SystemMngt/platformapplicationlibrary/ApplicationLogMessageId.hpp"
#include "Repository/repositoryinterface/RepositoryInterface.hpp"
#include "platformapplicationlibrary/CcmComponent_impl.hpp"
#include "platformapplicationlibrary/Process_impl.hpp"
#include "platformapplicationlibrary/ManagedProcess_impl.hpp"
#include "platformapplicationlibrary/UnmanagedProcess_impl.hpp"
#include "platformapplicationlibrary/ProcessObserver_impl.hpp"
#include "platformapplicationlibrary/ServiceBroker_impl.hpp"
#include "platformapplicationlibrary/ProcessesIterator_impl.hpp"
#include "platformapplicationlibrary/ProcessInit.hpp"
#include "platformapplicationlibrary/ProcessRun.hpp"
#include "platformapplicationlibrary/ProcessStop.hpp"
#include "SystemMngt/tools/ApplicationIssuerManager.hpp"
#include "SystemMngt/platformlibrary/EventNotifierManager.hpp"
#include "platformapplicationlibrary/HostProcessInitSequencer.hpp"
#include "idllib/CdmwPlatformMngtPredefinedProperties.stub.hpp"
#include "platformapplicationlibrary/DeploymentManager.hpp"
#include "SystemMngt/platformlibrary/FTGroupsIterator_impl.hpp"
#include "SystemMngt/platformlibrary/LBGroupsIterator_impl.hpp"
#include <limits>
#include <sstream>
#include "SystemMngt/platformlibrary/EventNotifierManager.hpp"
#include "platformapplicationlibrary/FTGroup_impl.hpp"
#include "platformapplicationlibrary/LBGroup_impl.hpp"

namespace Cdmw
{
    namespace PlatformMngt
    {
        
        ApplicationInitial* Application_impl::M_initial           = 0;
        ApplicationInitialising* Application_impl::M_initialising = 0;
        ApplicationInitialisationSuspended*
        Application_impl::M_initialisationSuspended           = 0;
        ApplicationInitialised* Application_impl::M_initialised   = 0;
        ApplicationRunRequest* Application_impl::M_runRequest     = 0;
        ApplicationRunning* Application_impl::M_running           = 0;
        ApplicationStopping* Application_impl::M_stopping         = 0;
        ApplicationStopped* Application_impl::M_stopped           = 0;
        ApplicationInitialisationIncomplete*
        Application_impl::M_initialisationIncomplete          = 0;
        ApplicationPartialRunning*
        Application_impl::M_partialRunning                    = 0;
        
    const string Application_impl::M_initialStateName                   = "Initial";
    const string Application_impl::M_initialisingStateName              = "Initialising";
    const string Application_impl::M_initialisationSuspendedStateName   = "InitialisationSuspended";
    const string Application_impl::M_partialRunningStateName            = "PartialRunning";
    const string Application_impl::M_initialisedStateName               = "Initialised";
    const string Application_impl::M_runRequestStateName                = "RunRequest";
    const string Application_impl::M_runningStateName                   = "Running";
    const string Application_impl::M_stoppingStateName                  = "Stopping";
    const string Application_impl::M_stoppedStateName                   = "Stopped";
    const string Application_impl::M_initialisationIncompleteStateName  = "InitialisationIncomplete";

 
       // string array used to LogMngr to print ProcessStatusChange enum
       static const std::string process_status_strings[] =
       {"PROCESS_INITIAL","PROCESS_INITIALISING","PROCESS_STEP_PERFORMED","PROCESS_INITIALISED",
        "PROCESS_RUN_REQUEST","PROCESS_RUNNING","PROCESS_STOPPING","PROCESS_STOPPED","PROCESS_ENDED","PROCESS_FAILED_ERROR",
        "PROCESS_FAILED_TIMEOUT","PROCESS_FAILED_DISCREPANCY","PROCESS_FAILED_INVALID","PROCESS_FAILED_NO_RESPONSE",
        "PROCESS_FAILED_DEATH"
       };
       
       
       static const std::string appl_status_strings[] =
       {
               "Initial","Initialising","InitialisationSuspended",
               "PartialRunning","Initialised","RunRequest",
               "Running", "Stopping", "Stopped","InitialisationIncomplete"
       };
       static const CdmwPlatformMngt::ApplicationStatus Status[]=
       { 
               CdmwPlatformMngt::APPLICATION_INITIAL,
               CdmwPlatformMngt::APPLICATION_INITIALISING,
               CdmwPlatformMngt::APPLICATION_INITIALISATION_SUSPENDED,
               CdmwPlatformMngt::APPLICATION_PARTIAL_RUNNING,
               CdmwPlatformMngt::APPLICATION_INITIALISED,
               CdmwPlatformMngt::APPLICATION_RUN_REQUEST,
               CdmwPlatformMngt::APPLICATION_RUNNING,
               CdmwPlatformMngt::APPLICATION_STOPPING,
               CdmwPlatformMngt::APPLICATION_STOPPED,
               CdmwPlatformMngt::APPLICATION_INITIALISATION_INCOMPLETE
       };

        // ----------------------------------------------------------------------
        // Application_impl class.
        // ----------------------------------------------------------------------
#ifdef CDMW_USE_FAULTTOLERANCE

    Application_impl::Application_impl (PortableServer::POA_ptr poa,
                      SupervisionEventNotifier *
                      eventNotifier,
                      CdmwPlatformMngt::
                      RWElementRepository_ptr
                      element_repository,
                      const std::
                      string & replicationManagerIOGR,
                      CdmwPlatformMngt::
                      FTConverter_ptr ftConverter,
                      OsSupport::RecursiveMutex* check_consistency_status_recursive_mutex,
                      CORBA::ORB_ptr orb)
      throw (BadParameterException, AlreadyDoneException,
         UnknownStateException,
         OutOfMemoryException):DeactivableServant_impl (poa),
      ManagedElement_impl (element_repository, poa), m_applicationDefined (false),
      m_initialisation_condition (m_mutex), m_stop_condition (m_mutex),
      m_replicationManagerIOGR (replicationManagerIOGR), m_created_dynamically(true), m_current_restart_attempt(0)

#else

 Application_impl::Application_impl (PortableServer::POA_ptr poa,
                      SupervisionEventNotifier *
                      eventNotifier,
                      CdmwPlatformMngt::
                      RWElementRepository_ptr
                      element_repository,
                      OsSupport::RecursiveMutex* check_consistency_status_recursive_mutex,
                      CORBA::ORB_ptr orb)
      throw (BadParameterException, AlreadyDoneException,
         UnknownStateException,
         OutOfMemoryException):DeactivableServant_impl (poa),
      ManagedElement_impl (element_repository, poa), m_applicationDefined (false),
      m_initialisation_condition (m_mutex), m_stop_condition (m_mutex),m_created_dynamically(true), m_current_restart_attempt(0),
      m_indicator_relevant_status(1)

#endif
        {
            m_logger = Cdmw::Logging::Logger::Get_Logger(CdmwPlatformMngtBase::PFMMNG);
            if (eventNotifier == 0)
                CDMW_THROW2 (BadParameterException, "eventNotifier", "0");
            
            m_graphElementFactory = 0;
            m_appStatusChangeFactory = 0;
            m_appModeChangeFactory = 0;
            m_appRestartFactory = 0;
            m_appStopFactory = 0;
            m_appAcknowledgeFactory = 0;
            m_stateMachine = 0;
            m_actualEntityContainer = 0;
            m_actualServiceDefContainer = 0;
            m_serviceBroker = 0;
            m_processObserver = 0;
            m_serviceDefContainer = 0;
            m_default_entitydefs = 0;

            m_check_consistency_status_recursive_mutex = check_consistency_status_recursive_mutex;
            m_deployment_info_sequence= new CdmwPlatformMngt::ProcDeploymentInfos();
            
            //Initialising the maps
            m_ftgroup_map.clear(); 
            m_lbgroup_map.clear(); 
            
           // set in Managed Element the instance of Property Event Notifier Thread
           set_property_event_notifier(EventNotifierManager::Instance()->get_property_notifier());
           set_status_event_notifier(EventNotifierManager::Instance()->get_status_notifier());
 
           try
           {
               std::auto_ptr < GraphElementFactory >
               graphElementFactory (new GraphElementFactory ());

               std::auto_ptr < ApplicationStatusChangeFactory >
               appStatusChangeFactory (new ApplicationStatusChangeFactory ());

               std::auto_ptr < ApplicationModeChangeFactory >
               appModeChangeFactory (new ApplicationModeChangeFactory ());

               std::auto_ptr <RestartFactory>
               appRestartFactory (new RestartFactory ());

               std::auto_ptr < StopFactory >
               appStopFactory (new StopFactory ());

               std::auto_ptr < AcknowledgeFactory >
               appAcknowledgeFactory (new AcknowledgeFactory ());

               std::auto_ptr < ApplicationStateMachine >
               stateMachine (new ApplicationStateMachine (this, "Application"));

               std::auto_ptr < CdmwPlatformMngt::GraphElements >
               processInitGraph (new CdmwPlatformMngt::GraphElements);

               std::auto_ptr < CdmwPlatformMngt::GraphElements >
               processStopGraph (new CdmwPlatformMngt::GraphElements);


               // create the process observer       
               m_processObserverServant = new ProcessObserver_impl (poa, this);
               m_processObserver
               =
                   dynamic_cast <
                   ProcessObserver_impl * >(m_processObserverServant.in ());


                   // Set the internal data

                   m_poa = PortableServer::POA::_duplicate (poa);
                   m_orb = CORBA::ORB::_duplicate (orb);
                   m_graphElementFactory = graphElementFactory.get ();
                   m_appStatusChangeFactory = appStatusChangeFactory.get ();
                   m_appModeChangeFactory = appModeChangeFactory.get ();
                   m_appRestartFactory = appRestartFactory.get ();
                   m_appStopFactory = appStopFactory.get ();
                   m_appAcknowledgeFactory = appAcknowledgeFactory.get ();
                   m_stateMachine = stateMachine.get ();
                   m_processInitGraph = processInitGraph.get ();
                   m_processStopGraph = processStopGraph.get ();

                   set_status_info ( CORBA::string_dup (""));
                   m_mode = CdmwPlatformMngt::NORMAL_MODE;
                   m_modeInfo = CORBA::string_dup ("");
                   m_emergency = false;
                   m_newInitGraph = true;
                   m_startup_kind = new CdmwPlatformMngtBase::StartupKind ();
                   m_eventNotifier = eventNotifier;
                   m_taskFactory = 0;
                   m_starter = 0;
                   m_initSyncTaskFactory = 0;
                   m_stopSyncTaskFactory = 0;
                   m_initSequencer = 0;
                   m_stopSequencer = 0;


                   m_init_condition_flag = false;
                   m_stop_condition_flag = false;

                   m_process_servants.clear ();

                   // Start the event handler
                   m_eventHandler.start ();


                   // Add the states to the state machine
                   m_stateMachine->addState (M_initialStateName, M_initial);
                   m_stateMachine->addState (M_initialisingStateName, M_initialising);
                   m_stateMachine->addState (M_initialisationSuspendedStateName,
                           M_initialisationSuspended);
                   m_stateMachine->addState (M_initialisedStateName, M_initialised);
                   m_stateMachine->addState (M_runRequestStateName, M_runRequest);
                   m_stateMachine->addState (M_runningStateName, M_running);
                   m_stateMachine->addState (M_stoppingStateName, M_stopping);
                   m_stateMachine->addState (M_stoppedStateName, M_stopped);
                   m_stateMachine->addState (M_initialisationIncompleteStateName,
                           M_initialisationIncomplete);
                   m_stateMachine->addState (M_partialRunningStateName, M_partialRunning);

                   try{
                       // define TECH_STATUS and TECH_STATUS_INFO properties
                       const char* init_value = "";
                       define_status_property(init_value);
                   }
                   catch(...)
                   {
                       throw  CosPropertyService::MultipleExceptions();
                   }
                   // Set the initial state
                   m_stateMachine->setState (M_initialStateName.c_str());                        

                   // Init the preemption flag (false)                   
                   init_preemption ();                                                            
        
                   m_stop_deployment_plan_timeout = CdmwPlatformMngt::DEPLOYMENT_PLANS_DEFAULT_TIMEOUT;

                   m_start_deployment_plan_timeout= CdmwPlatformMngt::DEPLOYMENT_PLANS_DEFAULT_TIMEOUT;

#ifdef CDMW_USE_FAULTTOLERANCE
                m_ftConverter =
                    CdmwPlatformMngt::FTConverter::_duplicate (ftConverter);
#endif
                
                // Release the ownership of the temporary objects
                graphElementFactory.release ();
                appStatusChangeFactory.release ();
                appModeChangeFactory.release ();
                appRestartFactory.release ();
                appStopFactory.release ();
                appAcknowledgeFactory.release ();
                stateMachine.release ();
                processInitGraph.release ();
                processStopGraph.release ();
            }
            catch (const std::bad_alloc &)
            {
                CDMW_THROW (OutOfMemoryException);
            }
            
            try
            {
                Cdmw::CommonSvcs::DataStore::DataStoreBase * ds =
                    Cdmw::CommonSvcs::DataStore::DataStoreRegistry::
                    Get_data_store (Configuration::Get_application_datastore_id ());
                m_application_ds =
                    dynamic_cast < Cdmw::PlatformMngt::ApplicationDataStore * >(ds);
            }
            catch (const NotExistException & e)
            {
                CDMW_ASSERT (false);
            }
            
    }
    
    Application_impl::~Application_impl ()
    {
        // when application is removed System has one application less 
//        (m_applicationStartData->application_observer)->notify_application_running_event();

        
        AgentHosts::iterator hostSetIt;
    
            for (hostSetIt = m_agentHosts.begin ();
            hostSetIt != m_agentHosts.end (); hostSetIt++)
            {
                HostData *p_hostData = hostSetIt->second;
                delete p_hostData;
            }
            
            m_agentHosts.clear ();
        

        destroyStarter ();
        destroyInitSequencer ();
        destroyStopSequencer ();
        
        m_eventHandler.stop ();
        
        if (m_graphElementFactory != 0)
        {
            m_graphElementFactory->_remove_ref ();
        }

    if (m_appStatusChangeFactory != 0)
        {
            m_appStatusChangeFactory->_remove_ref ();
        }
        
        if (m_appModeChangeFactory != 0)
        {
            m_appModeChangeFactory->_remove_ref ();
        }
        
        if (m_appRestartFactory != 0)
        {
           m_appRestartFactory->_remove_ref ();
        }
        
        if (m_appStopFactory != 0)
        {
            m_appStopFactory->_remove_ref ();
        }
        
        if (m_appAcknowledgeFactory != 0)
        {
            m_appAcknowledgeFactory->_remove_ref ();
        }
        
        if (m_stateMachine != 0)
        {
            delete m_stateMachine;
        }
        
        if (m_actualEntityContainer != 0)
        {
            delete m_actualEntityContainer;
        }
        
        if (m_actualServiceDefContainer != 0)
        {
            ServiceDefContainer_impl::ServiceDefContainer::
                destroy (m_actualServiceDefContainer);
        }
        
        
#ifdef CDMW_USE_FAULTTOLERANCE
        
        // unregister the process observer from the ftconverter
        m_ftConverter->unregister_process_observer (m_element_name.in ());
        
#endif
        
        
        // note : servant are deleted by the corresponding _var attributes
        //        destruction
        
        if (m_processObserver != 0)
        {
            try
            {
                m_processObserver->deactivate ();
            }
            catch (...)
            {
            }
        }
        
        if (m_serviceDefContainer != 0)
        {
            try
            {
                m_serviceDefContainer->deactivate ();
            }
            catch (...)
            {
            }
        }
        
        if (m_serviceBroker != 0)
        {
            try
            {
                m_serviceBroker->deactivate ();
            }
            catch (...)
            {
            }
        }
          
    }


OsSupport::RecursiveMutex* Application_impl::get_check_consistency_status_recursive_mutex(){
          return m_check_consistency_status_recursive_mutex;
}

    
    void Application_impl::remove()
    {
         std::string element_path = std::string (get_element_path());
         Cdmw::PlatformMngt::ApplicationIssuerManager::Instance()->detach_element(element_path);
    
        // first increment counter reference to avoid the destructor calling
        // by servant deactivation
        this->_add_ref();
    
        // deactivate the Corba object
        this->deactivate();
    
        // remove associated application record in datastore
        remove_record_in_datastore();
    
        // remove the associated managed element
        this->ManagedElement_impl::remove();
    
        // then decrement the counter reference to call destructor
        this->_remove_ref();        
    }
    
    
    
    
    
    void Application_impl::end_creation (const CdmwPlatformMngt::
        ApplicationStartingData &
        applicationStartData,
        const char *application_path)
        throw (OutOfMemoryException, CORBA::SystemException)
    {
        try
        {
            // store element path in managed element
            set_element_path (application_path,
                applicationStartData.application_name.in ());
             
            //Application element_path is set so application registration 
            // on ApplicationIssuerManager can be performed.
            std::string appl_element_path = std::string(element_path());                                      
         Cdmw::PlatformMngt::ApplicationIssuerManager::Instance()->attach_element(appl_element_path);

                
            // save starting data
            m_applicationStartData =
                new CdmwPlatformMngt::
                ApplicationStartingData (applicationStartData);
            
            // save system name
            m_system_name =
                CORBA::string_dup (m_applicationStartData->system_name.in ());
            
//             (m_applicationStartData->application_observer)->
//                    notify_application_not_running_event();
 
            // create containers                
            std::auto_ptr < PlatformMngt::EntityContainer >
                actualEntityContainer (new PlatformMngt::
                EntityContainer (m_poa.in (),
                m_element_repository.in (),
                m_element_path.in ()));
            
            m_actualServiceDefContainer =
                ServiceDefContainer_impl::ServiceDefContainer::
                createApplicationContainer (m_element_name.in ());
            
            m_rwEntityContainerServant =
                new RWEntityContainer_impl (m_poa.in (),
                actualEntityContainer.get (),
                m_element_repository.in ());
            
            m_serviceDefContainerServant =
                new ServiceDefContainer_impl (m_poa.in (),
                m_actualServiceDefContainer,
                applicationStartData.
                system_service_def_container.in ());
            
            m_rwServiceDefContainerServant =
                new RWServiceDefContainer_impl (m_poa.in (),
                m_actualServiceDefContainer,
                applicationStartData.
                system_service_def_container.in ());
            
            
            
            // create service broker
            m_serviceBrokerServant = new ServiceBroker_impl (m_poa.in (), this,
                applicationStartData.
                service_binding.
                in ());
            m_serviceBroker =
                dynamic_cast < ServiceBroker_impl * >(m_serviceBrokerServant.in ());
            
            // get system service broker object reference
            m_syst_serviceBroker =
                CdmwPlatformMngtService::ServiceBroker::
                _duplicate (applicationStartData.service_broker.in ());
            
            // get system process observer object reference
            m_syst_processObserver =
                CdmwPlatformMngt::ProcessObserver::_duplicate (applicationStartData.
                process_observer.
                in ());
            
            // Set the additional internal data with the acknowledgement data
            m_rwEntityContainer
                =
                dynamic_cast <
                RWEntityContainer_impl * >(m_rwEntityContainerServant.in ());
            
            m_serviceDefContainer
                =
                dynamic_cast <
                ServiceDefContainer_impl * >(m_serviceDefContainerServant.in ());
            
            m_rwServiceDefContainer
                =
                dynamic_cast <
                RWServiceDefContainer_impl *
                >(m_rwServiceDefContainerServant.in ());
            
            
            m_hostContainer =
                CdmwPlatformMngt::HostContainer::_duplicate (applicationStartData.
                host_container.in ());
            
//            m_eventNotifier->registerObserver (m_element_name.in (),
//              m_eventNotifier->registerObserver (m_system_name.in (),
//                m_applicationStartData->
//                supervision_observer.in ());
            
            // set element path in rwservicedef container
            m_rwServiceDefContainer->set_container_path (application_path);
            
            // Activate objects
            m_processObserver->activate ();
            m_serviceBroker->activate ();
            m_serviceDefContainer->activate ();
            
            // set internal with temporary auto pointer
            m_actualEntityContainer = actualEntityContainer.get ();
            
            
#ifdef CDMW_USE_FAULTTOLERANCE
            CdmwPlatformMngt::ProcessObserver_var processObs_var = m_processObserver->_this ();

            // register the process observer for the application
            m_ftConverter->register_process_observer (m_element_name.in (),
        processObs_var.in());
            //m_processObserver->_this ());
#endif
            
            // initialise data store of the object
            set_record_in_datastore ();
            
            // Release the ownership of the temporary objects
            actualEntityContainer.release ();
      }
      catch (const std::bad_alloc &)
      {
          if (m_actualEntityContainer != 0)
              delete m_actualEntityContainer;
          
          if (m_actualServiceDefContainer != 0)
              ServiceDefContainer_impl::ServiceDefContainer::
              destroy (m_actualServiceDefContainer);
          
          CDMW_THROW (OutOfMemoryException);
      }
    }
    
    void Application_impl::deployment_info_multimap_to_sequence( DeploymentInfoMap mmap)
    {
        DeploymentInfoMap::iterator deploy_it ;  
        CORBA::ULong seq_length=0;
        CdmwPlatformMngt::ProcDeploymentInfo p_deployment_info_struct;

        for (deploy_it=mmap.begin(); deploy_it!=mmap.end();++deploy_it)
        {
            m_deployment_info_sequence->length(seq_length+1);
            p_deployment_info_struct.deployment_plan = CORBA::string_dup((deploy_it->first).c_str());
            p_deployment_info_struct.process_name = CORBA::string_dup((deploy_it->second).c_str());
            m_deployment_info_sequence[seq_length] = p_deployment_info_struct;
            seq_length++;
        } 
    }

    
    void Application_impl::sequence_to_deployment_info_multimap(const CdmwPlatformMngt::ProcDeploymentInfos_var p_deployment_infos,DeploymentInfoMap& mmap) 
    {
        CORBA::ULong seq_len = p_deployment_infos->length();
        for(CORBA::ULong i=0; i<seq_len;i++ )
        {
            mmap.insert(DeploymentInfoMap::value_type
                    (CORBA::string_dup((p_deployment_infos[i]).deployment_plan),
                            CORBA::string_dup((p_deployment_infos[i]).process_name)));
        }
    } 
      
    void Application_impl::set_record_in_datastore ()
    {
        // test if the supervision is not fault tolerant or if it is, is it the primary?
        if (!Configuration::Is_fault_tolerant ()
            || Configuration::Is_primary ())
        {
            if (Configuration::Is_synchronising ())
            {
                CDMW_INFO(m_logger,
                "<<Application_impl::set_record_in_datastore() start >> " << m_element_name.in ());
            }
            CdmwPlatformMngt::ApplicationRecord record;
            record.status = m_stateMachine->getState ()->status ();
            record.status_info = get_status_info();
            record.mode = m_mode;
            record.mode_info = m_modeInfo;
            record.state_machine =
                CORBA::string_dup (m_stateMachine->getStateName ());
            record.startup_kind = m_startup_kind.in ();
            record.stop_emergency = m_emergency;
            record.new_graph = m_newInitGraph;
            record.process_init_graph = m_processInitGraph;
            record.process_stop_graph = m_processStopGraph;
            record.current_restart_attempt = m_current_restart_attempt;
            record.created_dynamically = m_created_dynamically;
            record.indicator_relevant_status = m_indicator_relevant_status;
            record.proc_deployment_infos = m_deployment_info_sequence;
            string ds_operation("contains");
            try
            {
                if (m_application_ds->contains (m_element_path.in ()))
                {
                    ds_operation = "update";
                    m_application_ds->update (m_element_path.in (), record);
                }
                else
                {
                    set_element_in_datastore ();
                    ds_operation = "insert";
                    m_application_ds->insert (m_element_path.in (), record);

                }
            }
            catch (const Common::AlreadyExistException&)
            {
                CDMW_WARN(m_logger,
                "record already exists exception in " 
                << ds_operation << " : " 
                << m_element_path.in () << ":"
                << (const char*) record.status);
            }
            catch (const CommonSvcs::DataStore::StorageErrorException &exc_storage)
            {
                CDMW_FATAL(m_logger,
                "record storage error exception in "
                << ds_operation <<" : " 
                << m_element_path.in() << ":"
                << (const char*) record.status << ":" 
                << exc_storage.what());
                CDMW_ASSERT (false);
            }
            catch (const Common::NotFoundException&)
            {
                CDMW_FATAL(m_logger,
                "record cannot be found in datastore in "
                << ds_operation <<" : " 
                << m_element_path.in());
                CDMW_ASSERT (false);
            }
            
            catch (...)
            {
                CDMW_FATAL(m_logger,
                "record unknown exception in "
                << ds_operation << " : " 
                << m_element_path.in() << ":"
                << (const char*) record.status);
                CDMW_ASSERT (false);
            }
            
              
            if (Configuration::Is_synchronising ())
            {
                CDMW_INFO(m_logger,
                "<<Application_impl::set_record_in_datastore() done >> " <<
                    m_element_name.in () );
            }
        }
    }
    
    
    void Application_impl::remove_record_in_datastore ()
    {
        // inhibition if backup
        if (!Configuration::Is_fault_tolerant ()
            || Configuration::Is_primary ())
        {
            try
            {
                m_application_ds->remove (m_element_path.in ());
            }
            catch (const Common::NotFoundException &)
            {
            }
            catch (const CommonSvcs::DataStore::StorageErrorException &)
            {
            }
        }
    }
    
    CdmwPlatformMngt::ApplicationRecord *
        Application_impl::get_record_from_datastore ()
    {
      CdmwPlatformMngt::ApplicationRecord * application_record = 0;

      try
      {
            application_record = m_application_ds->select (m_element_path.in ());
      }
      catch (const Common::NotFoundException &)
      {
          CDMW_FATAL(m_logger,
                     "application record cannot be found in datastore : "
                     << m_element_path.in());
          CDMW_ASSERT (false);
      }

      return application_record;
    }
    
    void Application_impl::synchro_with_datastore ()
    {
        //Cdmw::PlatformMngt::ManagedElement_impl::synchro_with_datastore ();

        CdmwPlatformMngt::ApplicationRecord_var application_record =
            get_record_from_datastore ();           
  
        set_status_info ( application_record->status_info);
        m_mode = application_record->mode;
        m_modeInfo = application_record->mode_info;

        m_stateMachine->setStateName (application_record->state_machine.in ());
        m_created_dynamically = application_record->created_dynamically;
        
        m_startup_kind =
            new CdmwPlatformMngtBase::StartupKind (application_record->
                           startup_kind);
        
        m_emergency = application_record->stop_emergency;
        m_current_restart_attempt = application_record->current_restart_attempt;
        m_newInitGraph = true;
        
        m_processInitGraph =
            new CdmwPlatformMngt::GraphElements (application_record->
            process_init_graph);
        
        m_processStopGraph =
            new CdmwPlatformMngt::GraphElements (application_record->
            process_stop_graph);
       
        m_indicator_relevant_status = application_record->indicator_relevant_status;
        *m_deployment_info_sequence = application_record->proc_deployment_infos; 
        //Converting the sequence in std::multimap
        sequence_to_deployment_info_multimap(m_deployment_info_sequence,m_deployment_info_multimap);
        synchro_entities_with_datastore();
    }

    void Application_impl::set_system_port (CdmwPlatformMngt::Port port)
     {
          m_applicationStartData->system_port=port;
     }
    
    void Application_impl::synchro_processes_with_datastore ()
    {
        Cdmw::PlatformMngt::ManagedElement_impl::synchro_with_datastore ();
        ProcessServants::iterator processIt;
        
        for (processIt = m_process_servants.begin ();
        processIt != m_process_servants.end (); processIt++)
        {
            Process_impl *process = processIt->second;
            
            // synchro the process with its datastore
            process->synchro_with_datastore ();
        
            // synchro all the components with their datastore
            process->synchro_components_with_datastore ();
        }
    }
    
     void Application_impl::synchro_groups_with_datastore()
    {
         Cdmw::PlatformMngt::ManagedElement_impl::synchro_with_datastore ();
     
         //Synchronising all the FT groups.
     for (FTGroupsMap::iterator it = m_ftgroup_map.begin(); it != m_ftgroup_map.end(); ++it)
         {
            (it->second)->synchro_with_datastore();
         }
     
        //Synchronising all the LB groups.
     for (LBGroupsMap::iterator it = m_lbgroup_map.begin();  it != m_lbgroup_map.end(); ++it)
         {
            (it->second)->synchro_with_datastore();
         }
    }
    
    void Application_impl::synchro_processing ()
    {
        // reset condition wait flags
        m_init_condition_flag = false;
        m_stop_condition_flag = false;
        
        // if status Initialising, RunRequest, Stopping restart the activity
        CdmwPlatformMngt::ApplicationStatus appStatus =
            m_stateMachine->getState ()->status ();
        
        // reset the process status according to the application status restarting
        // if application is in "initialising", processes must be set to "initial"
        // to be able to go to "initialised"
        // if application is in "run_request", processes must be set to "initialised"
        // to be able to go to "running"
        // if application is in "stopping", processes must be set to "running"
        // to be able to go to "stopping"
        
        // for each process of application
        ProcessServants::iterator processIt;
        
        for (processIt = m_process_servants.begin ();
        processIt != m_process_servants.end (); processIt++)
        {
            Process_impl *process = processIt->second;
            
            if (appStatus == CdmwPlatformMngt::APPLICATION_INITIALISING)
            {
                // synchro the process status
                process->synchro_processing_to_initial ();
            }
            else if (appStatus == CdmwPlatformMngt::APPLICATION_RUN_REQUEST)
            {
                // synchro the process status
                process->synchro_processing_to_initialised ();
            }
            else if (appStatus == CdmwPlatformMngt::APPLICATION_STOPPING)
            {
                // synchro the process status
                process->synchro_processing_to_running ();
            }
        }
        
        // if application is in run request force it to initialised
        // to restart the runset by using initialisation notification
        // (all processes are in initialised status)
        if (appStatus == CdmwPlatformMngt::APPLICATION_RUN_REQUEST)
        {
           
            appStatus = CdmwPlatformMngt::APPLICATION_INITIALISED;
            m_stateMachine->setStateName (M_initialisedStateName.c_str());
        }
        
        try
        {
            if (appStatus == CdmwPlatformMngt::APPLICATION_INITIALISING ||
                appStatus == CdmwPlatformMngt::APPLICATION_STOPPING)
            {
                // first lock mutex for condition
                CDMW_MUTEX_GUARD (m_mutex);
                
                // restart the activity of the application
                
                m_stateMachine->restartStateActivity ();
                
                
                // if initialisation is in progress
                if (appStatus == CdmwPlatformMngt::APPLICATION_INITIALISING)
                {
                    
                    // so wait the condition flag is set by broadcasting
                    while (!m_init_condition_flag)
                    {
                        m_initialisation_condition.wait ();
                    }
                    
                    // reset condition wait flag
                    m_init_condition_flag = false;
                }
                
                // if stopping is in progress
                else
                {
                    
                    // so wait the condition flag is set by broadcasting
                    while (!m_stop_condition_flag)
                    {
                        m_stop_condition.wait ();
                    }
                    
                    // reset condition wait flag
                    m_stop_condition_flag = false;
                }
                
                //std::cout <<"<<Application_impl::synchro_processing() update datastore >> "<< m_element_name.in () << std::endl;
                
                
                // update data store of the object
                set_record_in_datastore ();
                
                
            }
            else if (appStatus == CdmwPlatformMngt::APPLICATION_INITIALISED ||
                appStatus == CdmwPlatformMngt::APPLICATION_RUNNING||
                appStatus == CdmwPlatformMngt::APPLICATION_PARTIAL_RUNNING)
            {
                
                // Notify the initialisation event to the application observer
                // to restart the runset
                // If appli is running the runset has no effect on the application itself
                (m_applicationStartData->application_observer)->
                    notify_initialisation (m_element_name.in ());
                
                
                // update data store of the object
                set_record_in_datastore ();
                
            }
        }
        catch (...)
        {
            // do nothing
        }
    }
    
    
    void Application_impl::synchro_processing_of_processes ()
    {
        ProcessServants::iterator processIt;
        
        for (processIt = m_process_servants.begin ();
        processIt != m_process_servants.end (); processIt++)
        {
            Process_impl *process = processIt->second;
            
            // synchro the process with its datastore
            process->synchro_processing ();
        }
    }
    
    
    void Application_impl::createStates () throw (OutOfMemoryException)
    {
        try
        {
            std::auto_ptr < ApplicationInitial >
                initial (new ApplicationInitial ());
            std::auto_ptr < ApplicationInitialising >
                initialising (new ApplicationInitialising ());
            std::auto_ptr < ApplicationInitialisationSuspended >
                initialisationSuspended (new ApplicationInitialisationSuspended ());
            std::auto_ptr < ApplicationInitialised >
                initialised (new ApplicationInitialised ());
            std::auto_ptr < ApplicationRunRequest >
                runRequest (new ApplicationRunRequest ());
            std::auto_ptr < ApplicationRunning >
                running (new ApplicationRunning ());
            std::auto_ptr < ApplicationStopping >
                stopping (new ApplicationStopping ());
            std::auto_ptr < ApplicationStopped >
                stopped (new ApplicationStopped ());
            std::auto_ptr < ApplicationInitialisationIncomplete >
                initialisationIncomplete (new
                ApplicationInitialisationIncomplete ());
           std::auto_ptr < ApplicationPartialRunning >
                partialRunning (new 
                ApplicationPartialRunning ());


            
            if (M_initial == 0)
                M_initial = initial.get ();
            if (M_initialising == 0)
                M_initialising = initialising.get ();
            if (M_initialisationSuspended == 0)
                M_initialisationSuspended = initialisationSuspended.get ();
            if (M_initialised == 0)
                M_initialised = initialised.get ();
            if (M_runRequest == 0)
                M_runRequest = runRequest.get ();
            if (M_running == 0)
                M_running = running.get ();
            if (M_stopping == 0)
                M_stopping = stopping.get ();
            if (M_stopped == 0)
                M_stopped = stopped.get ();
            if (M_initialisationIncomplete == 0)
                M_initialisationIncomplete = initialisationIncomplete.get ();
            if (M_partialRunning == 0)
                 M_partialRunning = partialRunning.get();
            // Create the processes states
            ManagedProcess_impl::createStates ();
            UnmanagedProcess_impl::createStates ();
            
            // Release the ownership of the temporary objects
            initial.release ();
            initialising.release ();
            initialisationSuspended.release ();
            initialised.release ();
            runRequest.release ();
            running.release ();
            stopping.release ();
            stopped.release ();
            initialisationIncomplete.release ();
            partialRunning.release ();
            
        }
        catch (const std::bad_alloc &)
        {
            CDMW_THROW (OutOfMemoryException);
        }
    }
    
    void Application_impl::destroyStates ()
    {
        if (M_initial != 0)
        {
            delete M_initial;
            M_initial = 0;
        }
        
        if (M_initialising != 0)
        {
            delete M_initialising;
            M_initialising = 0;
        }
        
        if (M_initialisationSuspended != 0)
        {
            delete M_initialisationSuspended;
            M_initialisationSuspended = 0;
        }
        
        if (M_initialised != 0)
        {
            delete M_initialised;
            M_initialised = 0;
        }
        
        if (M_runRequest != 0)
        {
            delete M_runRequest;
            M_runRequest = 0;
        }
        
        if (M_running != 0)
        {
            delete M_running;
            M_running = 0;
        }
        
        if (M_stopping != 0)
        {
            delete M_stopping;
            M_stopping = 0;
        }
        
        if (M_stopped != 0)
        {
            delete M_stopped;
            M_stopped = 0;
        }
       
         
        if (M_partialRunning != 0)
        {
            delete M_partialRunning;
            M_partialRunning = 0;
        }
 
        if (M_initialisationIncomplete != 0)
        {
            delete M_initialisationIncomplete;
            M_initialisationIncomplete = 0;
        }
        
        // Destroy the processes states
        ManagedProcess_impl::destroyStates ();
        UnmanagedProcess_impl::destroyStates ();
    }
    
    // TODO: - Activer une seule fois ProcessObserver
    
    CdmwPlatformMngt::ApplicationAgent_ptr Application_impl::
        getAgent (const char *hostName) throw (CdmwPlatformMngt::HostNotFound,
        CdmwPlatformMngt::
        HostNotReachable,
        CdmwPlatformMngt::CreationError,
        CdmwPlatformMngt::
        ApplicationAgentAlreadyExists,
        CdmwPlatformMngt::InvalidInfo,
        CdmwPlatformMngt::
        CreationTimeout,
        AgentNotFoundException,
        CORBA::SystemException)
    {
        CdmwPlatformMngt::ApplicationAgent_var applicationAgent =
            CdmwPlatformMngt::ApplicationAgent::_nil ();
        
        try
        {
            CdmwPlatformMngt::Daemon_var daemon;
            
            // try to find host name in host set for the application
            bool agentFound = false;
            std::string hostName_str = hostName;
            
            {
                CDMW_READER_LOCK_GUARD (m_rwAgentHosts_lock);
                
                AgentHosts::iterator hostSetIt = m_agentHosts.find (hostName_str);
                
                if (hostSetIt != m_agentHosts.end ())
                {
                    // agent has been already created in map
                    agentFound = true;
                    
                    // get the daemon
                    daemon = (hostSetIt->second)->daemonObj;
                }
            }
            
            // If application agent not found in map, search or create a new one
            if (!agentFound)
            {
                // protect with the WRITER lock
                CDMW_WRITER_LOCK_GUARD (m_rwAgentHosts_lock);
                
                // check again if agent has not been created by another thread
                // in the meantime
                AgentHosts::iterator hostSetIt = m_agentHosts.find (hostName_str);
                
                if (hostSetIt != m_agentHosts.end ())
                {
                    // the agent has been already created by another thread in map
                    agentFound = true;
                    
                    // get the daemon
                    daemon = (hostSetIt->second)->daemonObj;
                }
                else
                {
                    // Get the daemon running on the specified host
                    daemon =
                        Configuration::Get_daemon (hostName,
                        m_applicationStartData->
                        system_port);
                    
                    // first get the agent from daemon if already exist
                    applicationAgent =
                        daemon->search_application_agent (m_element_name.in ());
                    
                    // if agent does not exist for the host
                    if (CORBA::is_nil (applicationAgent.in ()))
                    {
                        // create a new one
                        CdmwPlatformMngt::ApplicationAgentInfo_var agentInfo
                            = new CdmwPlatformMngt::ApplicationAgentInfo ();
                        
                        agentInfo->system_name = m_system_name.in ();
                        agentInfo->application_name = m_element_name.in ();
                        
#ifdef CDMW_USE_FAULTTOLERANCE
                        agentInfo->replication_manager_iogr =
                            CORBA::string_dup (m_replicationManagerIOGR.c_str ());
                        
                        agentInfo->process_observer =
                            CdmwPlatformMngt::ProcessObserver::
                            _duplicate (m_syst_processObserver.in ());
                        agentInfo->service_broker =
                            CdmwPlatformMngtService::ServiceBroker::
                            _duplicate (m_syst_serviceBroker.in ());
#else
                        agentInfo->process_observer = m_processObserver->_this ();
                        agentInfo->service_broker = m_serviceBroker->_this ();
#endif
                        
                        // request to daemon to create the agent
                        applicationAgent =
                            daemon->create_application_agent (agentInfo.in ());
                    }
                    
                    // store host name with daemon in map
            /*
                    std::auto_ptr < HostData > hostData (new HostData);
                    
                    (hostData.get ())->daemonObj = daemon;
                    
                    m_agentHosts[hostName_str] = hostData.get ();
                    hostData.release ();
            */
            
            HostData *hostData  = new HostData;
            hostData->daemonObj = daemon;
            m_agentHosts[hostName_str] = hostData;
            
                }
            }
            
            // if host name has been found in map, application agent should exist        
            if (agentFound)
            {
                // Get the application agent from the daemon
                applicationAgent =
                    daemon->search_application_agent (m_element_name.in ());
               
                if (CORBA::is_nil (applicationAgent.in ()))
                {
                    // If application agent not found by daemon, throw exception
                    CDMW_THROW (AgentNotFoundException);
                }
            }
      }
      catch (const std::bad_alloc &)
      {
          throw CORBA::NO_MEMORY (OrbSupport::NO_MEMORY, CORBA::COMPLETED_NO);
      }
      
      return applicationAgent._retn ();
    }
    
    CdmwPlatformMngt::ApplicationAgent_ptr Application_impl::
        searchAgentFromDaemon (const char *hostName) throw (CdmwPlatformMngt::
        HostNotFound,
        CdmwPlatformMngt::
        HostNotReachable,
        CORBA::
        SystemException)
    {
        CdmwPlatformMngt::ApplicationAgent_var applicationAgent =
            CdmwPlatformMngt::ApplicationAgent::_nil ();
        
        try
        {
            // Get the daemon running on the specified host
            CdmwPlatformMngt::Daemon_var daemon =
                Configuration::Get_daemon (hostName,
                m_applicationStartData->system_port);
            
            // Get the application agent from the daemon
            applicationAgent =
                daemon->search_application_agent (m_element_name.in ());
        }
        catch (const std::bad_alloc &)
        {
            throw CORBA::NO_MEMORY (OrbSupport::NO_MEMORY, CORBA::COMPLETED_NO);
        }
        
        return applicationAgent._retn ();
    }
    
    
    void Application_impl::killAllAgents ()
    {
        // Kill all agents create by daemons for application
        {
            CDMW_WRITER_LOCK_GUARD (m_rwAgentHosts_lock);
            
            bool errorSet = true;
            
            
            AgentHosts::iterator hostSetIt;
            
            for (hostSetIt = m_agentHosts.begin ();
            hostSetIt != m_agentHosts.end (); hostSetIt++)
            {
                try
                {
                    LogMngr::logMessage (INF,
                        MSG_ID_APP_START_KILL_AGENT_HOST,
                        m_element_name.in (),
                        (hostSetIt->first).c_str ());
                    
                    // Get the daemon running on the specified host                
                    CdmwPlatformMngt::Daemon_var daemon =
                        (hostSetIt->second)->daemonObj;
                    
                    // request to daemon to kill agent
                    daemon->kill_application_agent (m_element_name.in ());
                    
                    // kill is ok
                    errorSet = false;
                }
                catch (const CdmwPlatformMngt::HostNotFound & e)
                {
                    // nothing to do
                }
                catch (const CdmwPlatformMngt::HostNotReachable & e)
                {
                    // nothing to do
                }
                catch (const CdmwPlatformMngt::ApplicationAgentNotFound & e)
                {
                    // nothing to do
                }
                catch (...)
                {
                    // nothing to do
                }
                
                long messageId;
                
                if (errorSet)
                {
                    messageId = MSG_ID_APP_KILL_AGENT_FAIL;
                }
                else
                {
                    messageId = MSG_ID_APP_KILL_AGENT_OK;
                }
                
                LogMngr::logMessage (INF,
                    messageId,
                    m_element_name.in (),
                    (hostSetIt->first).c_str ());
            }
            
            // clear the map
            for (hostSetIt = m_agentHosts.begin ();
            hostSetIt != m_agentHosts.end (); hostSetIt++)
            {
                HostData *p_hostData = hostSetIt->second;
                delete p_hostData;
            }
            
            m_agentHosts.clear ();
        }
    }
    
    void Application_impl::invalidateProcesses (CdmwPlatformMngtBase::
        EventHeader header,
        const char *hostName,
        InvalidationType reason)
    {
        std::string faultyHostName = hostName;
        
        // remove invalidated host from application hosts set
        {
            CDMW_WRITER_LOCK_GUARD (m_rwAgentHosts_lock);
            
            AgentHosts::iterator hostSetIt = m_agentHosts.find (faultyHostName);
            
            if (hostSetIt != m_agentHosts.end ())
            {
                HostData *p_hostData = hostSetIt->second;
                delete p_hostData;
                m_agentHosts.erase (hostSetIt);
            }
        }
    
    //PCR-770 TBS invalidateProcesses during ManagedProcess_impl::initialise
        //CDMW_READER_RECURSIVE_MUTEX_GUARD(m_application_recursive_mutex);
    
        // Invalidate all the activ processes
        ProcessServants::iterator processIt;
        for (processIt = m_process_servants.begin ();
        processIt != m_process_servants.end (); processIt++)
        {
            Process_impl *process = processIt->second;
            
            const char *processHostName = process->get_process_host_name ();
            
            
            CORBA::String_var statusInfo;
            CdmwPlatformMngt::ProcessStatus status =
            process->get_status (statusInfo.out());
            
            
            /*MRI PCR789 CdmwPlatformMngt::ProcessStatus status
                = process->get_internal_status ();*/
            
            if ((faultyHostName == processHostName) &&
                ((status == CdmwPlatformMngt::PROCESS_INITIALISING) ||
                (status == CdmwPlatformMngt::PROCESS_STEP_PERFORMED) ||
                (status == CdmwPlatformMngt::PROCESS_INITIALISED) ||
                (status == CdmwPlatformMngt::PROCESS_RUN_REQUEST) ||
                (status == CdmwPlatformMngt::PROCESS_RUNNING) ||
                (status == CdmwPlatformMngt::PROCESS_STOPPING)))
            {
                switch (reason)
                {
                case HOST_UNAVAILABLE:
                    process->invalidate_event (header,
                        LogMngr::
                        getFormattedMessage
                        (MSG_ID_HOST_CRASHED,
                        hostName).c_str ());
                    break;
                }
                
                // Enter the degraded mode
                degradation_event ();
            }
        }
    }
    
    
    void Application_impl::notifyModeChangeEvent (CdmwPlatformMngt::
                          FunctioningMode
                          functioning_mode,
                          const char *mode_info)
    {
        try
        {
            CdmwPlatformMngtBase::EventHeader header
                = EventHeaderFactory::createHeader (CdmwPlatformMngtBase::INF);
            
            CdmwPlatformMngt::ApplicationModeChange_var event
                = m_appModeChangeFactory->create (header,
                m_system_name.in (),
                m_element_name.in (),
                functioning_mode, mode_info);
            
            CORBA::String_var eventStr = event->to_string ();
            LogMngr::logMessage (eventStr.in ());
            event->_add_ref();            
            m_eventNotifier->addEvent (event);
            // Perform the status change notification to observer registered
            // with the Api in ManagedElement
            event->_add_ref();
            get_status_event_notifier()->add_event (event);
            event->_remove_ref();

            event._retn ();
        }
        catch (const std::bad_alloc &)
        {
            throw CORBA::NO_MEMORY (OrbSupport::NO_MEMORY, CORBA::COMPLETED_NO);
        }
        catch (...)
        {
            throw CORBA::INTERNAL (OrbSupport::INTERNAL, CORBA::COMPLETED_NO);
        }
    }
    
    void Application_impl::notifyStatusChangeEvent ()
    {
        try
        {
            CdmwPlatformMngtBase::EventHeader header
                = EventHeaderFactory::createHeader (CdmwPlatformMngtBase::INF);
            
            CdmwPlatformMngt::ApplicationStatusChange_var event
                = m_appStatusChangeFactory->create (header,
                m_system_name.in (),
                m_element_name.in (),
                m_stateMachine->getState ()->
                status (), get_status_info());
            
            CORBA::String_var eventStr = event->to_string ();
            LogMngr::logMessage (eventStr.in ());


            event->_add_ref(); 
            m_eventNotifier->addEvent (event);
            // Perform the status change notification to observer registered
            // with the Api in ManagedElement
            event->_add_ref(); 

            get_status_event_notifier()->add_event (event);
            event->_remove_ref();
            event._retn ();
        }
        catch (const std::bad_alloc &)
        {
            throw CORBA::NO_MEMORY (OrbSupport::NO_MEMORY, CORBA::COMPLETED_NO);
        }
        catch (...)
        {
            throw CORBA::INTERNAL (OrbSupport::INTERNAL, CORBA::COMPLETED_NO);
        }
    }
    
    void Application_impl::setInitialisationStatus (CdmwPlatformMngt::
        LongRequestStatus status)
    {
        m_mutex.lock ();
        
        // set condition wait flag
        m_init_condition_flag = true;
        
        m_initialisation_status = status;
        m_initialisation_condition.broadcast ();
        m_mutex.unlock ();
    }
    
    
    void Application_impl::setStopStatus (CdmwPlatformMngt::
        LongRequestStatus status)
    {
        m_mutex.lock ();
        
        // set condition wait flag
        m_stop_condition_flag = true;
        
        m_stop_status = status;
        m_stop_condition.broadcast ();
        m_mutex.unlock ();
    }
    
    void Application_impl::setInitGraph (const CdmwPlatformMngt::
        GraphElements & process_graph)
        throw (OutOfMemoryException, CdmwPlatformMngt::ReferenceNotFound,
        CdmwPlatformMngt::CircularReference,
        CdmwPlatformMngt::StepOutOfBound, CdmwPlatformMngt::StepJump,
        CORBA::SystemException)
    {
        try
        {
            CDMW_WRITER_RECURSIVE_MUTEX_GUARD (m_application_recursive_mutex);
      
            // Build the map of exisiting elements
            ExistingElementMap existingElements;
            ProcessServants::iterator processIt;
            
            for (processIt = m_process_servants.begin ();
            processIt != m_process_servants.end (); processIt++)
            {
                Process_impl *process = processIt->second;
                
                unsigned long initStepCount = 1;
                
                if (process->type () != CdmwPlatformMngt::UNMANAGED_PROCESS)
                {
                    CdmwPlatformMngt::LifeCycleDef_var lifeCycleDef =
                        process->get_life_cycle ();
                    
                    initStepCount = lifeCycleDef->init_steps.length ();
                }
                
                existingElements.
                    insert (PlatformMngt::ExistingElementMap::
                    value_type (process->get_process_key (),
                    initStepCount));
                
                // if process name is unique, insert it as existing element
                // (host cannot be specified) 
                if (checkNbrOfProcessNames (process->get_element_name ()) == 1)
                {
                    existingElements.
                        insert (PlatformMngt::ExistingElementMap::
                        value_type (process->get_element_name (),
                        initStepCount));
                }
            }
            
            // Check the validity of the graph of processes
            GraphChecker processGraph (process_graph, &existingElements, false);
            
            // Actually sets the graph of processes to this application
            m_processInitGraph =
                new CdmwPlatformMngt::GraphElements (process_graph);
            
            // set flag new graph
            m_newInitGraph = true;
            
            // TODO: Notify the configuration change
            
            // TODO: Trace to the log manager
            
            // update record in datastore
            set_record_in_datastore ();

        }
        catch (const std::bad_alloc &)
        {
            throw CORBA::NO_MEMORY (OrbSupport::NO_MEMORY, CORBA::COMPLETED_NO);
        }
    }
    
    void Application_impl::setStopGraph (const CdmwPlatformMngt::
        GraphElements & process_graph)
        throw (OutOfMemoryException, CdmwPlatformMngt::ReferenceNotFound,
        CdmwPlatformMngt::CircularReference,
        CdmwPlatformMngt::StepOutOfBound, CdmwPlatformMngt::StepJump,
        CORBA::SystemException)
    {
        try
        {

            CDMW_WRITER_RECURSIVE_MUTEX_GUARD (m_application_recursive_mutex); 
         
            // Build the map of existing elements
            ExistingElementMap existingElements;
            ProcessServants::iterator processIt;
            
            for (processIt = m_process_servants.begin ();
            processIt != m_process_servants.end (); processIt++)
            {
                Process_impl *p_process = processIt->second;
                
                existingElements.
                    insert (PlatformMngt::ExistingElementMap::
                    value_type (p_process->get_process_key (), 0));
                
                // if process name is unique, insert it as existing element
                // (host cannot be specified) 
                if (checkNbrOfProcessNames (p_process->get_element_name ()) == 1)
                {
                    existingElements.
                        insert (PlatformMngt::ExistingElementMap::
                        value_type (p_process->get_element_name (), 0));
                }
            }
            
            // Check the validity of the graph of processes
            GraphChecker processGraph (process_graph, &existingElements, true);
            
            // Actually sets the graph of processes to this application
            m_processStopGraph =
                new CdmwPlatformMngt::GraphElements (process_graph);
            
            // TODO: Notify the configuration change
            
            // TODO: Trace to the log manager
            
            // update record in datastore
            set_record_in_datastore ();
   
        }
        catch (const std::bad_alloc &)
        {
            throw CORBA::NO_MEMORY (OrbSupport::NO_MEMORY, CORBA::COMPLETED_NO);
        }
    }
    
    int Application_impl::checkNbrOfProcessNames (const char *process_name)
    {
        int nbr_of_process = 0;
        
        ProcessServants::iterator processIt;
        
        for (processIt = m_process_servants.begin ();
        processIt != m_process_servants.end (); processIt++)
        {
            Process_impl *process = processIt->second;
            
            if (!strcmp (process_name, process->get_element_name ()))
            {
                nbr_of_process++;
            }
        }
        
        return nbr_of_process;
    }
    
    Process_impl* Application_impl::getProcessHostName (const std::
        string & process_name,
        std::
        string & host_name)
        throw (ProcessNameNotUnique, CdmwPlatformMngt::ProcessNotFound)
    {
        int nbr_of_process = 0;
        Process_impl *process = 0;
        
        const char *p_process_name = process_name.c_str ();
        
        host_name = "";

        CDMW_READER_RECURSIVE_MUTEX_GUARD (m_application_recursive_mutex);

        ProcessServants::iterator processIt;
        
        for (processIt = m_process_servants.begin ();
        processIt != m_process_servants.end (); processIt++)
        {
            if (!strcmp
                (p_process_name, processIt->second->get_element_name ()))
            {
                // if first process found
                if (nbr_of_process == 0)
                {
                    process = processIt->second;
                    nbr_of_process++;
                }
                else
                {
                    // throw exception if more than one process is found
                    throw ProcessNameNotUnique ();
                }
            }
        }
        
        // if process not found
        if (nbr_of_process == 0)
        {
            throw CdmwPlatformMngt::ProcessNotFound ();
        }
        
        host_name = process->get_process_host_name ();

        return process;
    }
    
    
    void Application_impl::defineApplication (const CdmwPlatformMngt::
        ApplicationDef &
        application_def)
        throw (CdmwPlatformMngt::DuplicateEntity,
        CdmwPlatformMngt::DuplicateService,
        CdmwPlatformMngt::DuplicateProcess,
        CdmwPlatformMngt::ProcessHostNotFound,
        CdmwPlatformMngt::DuplicateProcessEntity,
        CdmwPlatformMngt::DuplicateProcessService,
        CdmwPlatformMngt::DuplicateProcessStep,
        CdmwPlatformMngt::AlreadyDone,
        CdmwPlatformMngt::IncompatibleStatus,
        CosPropertyService::MultipleExceptions, CORBA::SystemException)
    {
        {
            CDMW_READER_RECURSIVE_MUTEX_GUARD (m_application_recursive_mutex);
            
            if (m_applicationDefined){
                throw CdmwPlatformMngt::AlreadyDone ();
            }

        }
        
        //TODO: add ApplicationDef integrity check
        
        // Check the validity of all the process definitions
        // regarding the context of this application
        unsigned int processCount = application_def.processes.length ();

        for (unsigned int processIndex = 0;
        processIndex < processCount; processIndex++)
        {
            CdmwPlatformMngt::ProcessDef_var process_def
                = application_def.processes[processIndex];
            
            const char *p_host_name = process_def->host_name ();
            const char *p_process_name = process_def->name ();

            try
            {
                // Check the host is known from the system
                CdmwPlatformMngt::Host_var host =
                    m_hostContainer->get_host (p_host_name);
                
                // if process is managed
                if (process_def->type () != CdmwPlatformMngt::UNMANAGED_PROCESS)
                {
                    CdmwPlatformMngt::ManagedProcessDef * managed_process_def
                        =
                        CdmwPlatformMngt::ManagedProcessDef::
                        _downcast (process_def);
                    
                    // Check there is no duplicate entity
                    CdmwPlatformMngt::EntityDefs & entities =
                        managed_process_def->entities ();
                    
                    for (unsigned int entityIndex = 0;
                    entityIndex < entities.length (); entityIndex++)
                    {
                        std::string entityName =
                            entities[entityIndex].entity_name.in ();
                        
                        for (unsigned int entityIndex2 = entityIndex + 1;
                        entityIndex2 < entities.length (); entityIndex2++)
                        {
                            std::string entityName2 =
                                entities[entityIndex2].entity_name.in ();
                            
                            if (entityName == entityName2)
                                throw CdmwPlatformMngt::
                                DuplicateProcessEntity (p_process_name,
                                p_host_name,
                                entityName.c_str ());
                        }
                    }
                    
                    // Check there is no duplicate service
                    CdmwPlatformMngtService::ServiceDefs & services
                        = managed_process_def->service_definitions ();
                    
                    for (unsigned int serviceIndex = 0;
                    serviceIndex < services.length (); serviceIndex++)
                    {
                        std::string serviceName
                            = services[serviceIndex].service_name.in ();
                        
                        for (unsigned int serviceIndex2 = serviceIndex + 1;
                        serviceIndex2 < services.length (); serviceIndex2++)
                        {
                            std::string serviceName2
                                = services[serviceIndex2].service_name.in ();
                            
                            if (serviceName == serviceName2)
                                throw CdmwPlatformMngt::
                                DuplicateProcessService (p_process_name,
                                p_host_name,
                                serviceName.c_str ());
                        }
                    }
                    
                    // Check there is no duplicate step
                    CdmwPlatformMngt::Steps & initSteps
                        = managed_process_def->life_cycle ().init_steps;
                    
                    unsigned int initStepCount = initSteps.length ();
                    for (unsigned int initStepIndex = 0;
                    initStepIndex < initStepCount; initStepIndex++)
                    {
                        std::string initStepName
                            = initSteps[initStepIndex].step_label.in ();
                        
                        for (unsigned int initStepIndex2 = initStepIndex + 1;
                        initStepIndex2 < initStepCount; initStepIndex2++)
                        {
                            std::string initStepName2
                                = initSteps[initStepIndex2].step_label.in ();
                            
                            if (initStepName == initStepName2)
                                throw CdmwPlatformMngt::
                                DuplicateProcessStep (p_process_name, p_host_name,
                                initStepName.c_str ());
                        }
                    }
                }
            }
            catch (const CdmwPlatformMngt::HostNotFound &)
            {
                throw CdmwPlatformMngt::ProcessHostNotFound (p_process_name,
                    p_host_name);
            }
        }

        m_stop_deployment_plan_timeout = application_def.stop_deploymentplan_timeout;

        m_start_deployment_plan_timeout= application_def.start_deploymentplan_timeout;

        // All has been checked
        
        // Add all the processes
        for (unsigned int processIndex = 0;
        processIndex < processCount; processIndex++)
        {
            // get process definition (ProcessDef is a value type, a var
            // is returned by the [] operator, a copy is made in a new var)         
            CdmwPlatformMngt::ProcessDef_var process_def =
                application_def.processes[processIndex];
            
            const char *p_host_name = process_def->host_name ();
            const char *p_process_name = process_def->name ();
            
            try
            {
                CdmwPlatformMngt::Process_var process
                    = addProcess (process_def.in ());
            }
            catch (const CdmwPlatformMngt::HostNotFound &)
            {
                throw CdmwPlatformMngt::ProcessHostNotFound (p_process_name,
                    p_host_name);
            }
            catch (const CdmwPlatformMngt::ProcessAlreadyExists &)
            {
                throw CdmwPlatformMngt::DuplicateProcess (p_process_name,
                    p_host_name);
            }
        }
        
        // Add all the entities
        unsigned int entityCount = application_def.entities.length ();
        
        
        // Store entity initial values into default_entities
        m_default_entitydefs = new CdmwPlatformMngt::EntityDefs ();
        m_default_entitydefs->length (entityCount);
        
        for (unsigned int entityIndex = 0;
        entityIndex < entityCount; entityIndex++)
        {
            // get entity definition (EntityDef is a structure, a reference on the
            // structure is returned by the [] operator)
            const CdmwPlatformMngt::EntityDef & entity_def =
                application_def.entities[entityIndex];
            
            // store values of the entityDef
            (*m_default_entitydefs)[entityIndex] = entity_def;
            
            try
            {
                m_rwEntityContainer->add_entity (entity_def);
            }
            catch (const CdmwPlatformMngt::EntityAlreadyExists &)
            {
                throw CdmwPlatformMngt::DuplicateEntity (entity_def.entity_name.
                    in ());
            }
            
        }
        
        // Add all the services
        
        unsigned int serviceCount =
            application_def.service_definitions.length ();
        
        for (unsigned int serviceIndex = 0;
        serviceIndex < serviceCount; serviceIndex++)
        {
            // get service definition (ServiceDef is a structure, a reference on the
            // structure is returned by the [] operator)
            const CdmwPlatformMngtService::ServiceDef & service_def =
                application_def.service_definitions[serviceIndex];
            
            try
            {
                m_rwServiceDefContainer->add_service_def (service_def);
            }
            catch (const CdmwPlatformMngtService::ServiceAlreadyExists &)
            {
                throw CdmwPlatformMngt::DuplicateService (service_def.
                    service_name.in ());
            }
        }
        
        // Add Properties to Applicartion
        unsigned int propCount = application_def.properties.length ();
        
        for (unsigned int propIndex = 0; propIndex < propCount; propIndex++)
        {
            const CdmwPlatformMngt::PropertyDef & prop_def =
                application_def.properties[propIndex];
            
            std::string propertyName (application_def.properties[propIndex].
                    property_name);
           CosPropertyService::PropertyModeType propertyMode = CosPropertyService::PropertyModeType(application_def.properties[propIndex].mode_type);
            try
            {
                define_property_with_mode (propertyName.c_str (), prop_def.initial_value, propertyMode);
            }
            catch (...)
            {
                throw CosPropertyService::MultipleExceptions ();
            }
        }
        
        // MdR define rw properties 
        try
        {
      std::ostringstream property_name;
          property_name << CdmwPlatformMngt::CDMW_PREFIX << CdmwPlatformMngt::FORCE_MANUAL_START << std::ends;
      if(!is_property_defined(property_name.str().c_str()))
          {
              CORBA::Any_var any = new CORBA::Any();
              *any <<= CORBA::Any::from_boolean(false);
              define_property_with_mode(property_name.str().c_str() , any.in(), CosPropertyService::fixed_normal);
          }

          property_name.str("");
          property_name << CdmwPlatformMngt::CDMW_PREFIX << CdmwPlatformMngt::AUTORESTART_INFO << std::ends;
          if(!is_property_defined(property_name.str().c_str()))
          {
              CdmwPlatformMngt::AutoRestartInfo auto_restart_info;
              auto_restart_info.max_nbr_of_restart_attempt = 1;
              auto_restart_info.auto_restart_control = CdmwPlatformMngt::USE_COUNTER;
              CORBA::Any_var any = new CORBA::Any();
              *any <<= auto_restart_info;
              define_property_with_mode(property_name.str().c_str() , any.in(), CosPropertyService::fixed_normal);
          }

          property_name.str("");
          property_name << CdmwPlatformMngt::CDMW_PREFIX << CdmwPlatformMngt::DEFAULT_RESTART_WARMTH << std::ends;
          if(!is_property_defined(property_name.str().c_str()))
          {
              CORBA::Any_var any = new CORBA::Any();
              *any <<= CdmwPlatformMngt::HOT_RESTART;
              define_property_with_mode(property_name.str().c_str() , any.in(), CosPropertyService::fixed_normal);
          }
        }
        catch(...)
        {
           throw  CosPropertyService::MultipleExceptions();
        }

        // MdR define system properties not user defined
        std::ostringstream property_name;
        CORBA::Any_var any_val = new CORBA::Any();
        try
        {                            
            get_attribute(CdmwPlatformMngt::DEFAULT_RESTART_WARMTH, any_val.out(), CdmwPlatformMngt::PRE_DEFINED);                                     
            property_name.str("");
            property_name << CdmwPlatformMngt::CDMW_PREFIX << CdmwPlatformMngt::REQUESTED_RESTART_WARMTH << std::ends;
            define_property_with_mode(property_name.str().c_str(), any_val.in(), CosPropertyService::fixed_normal);
        }
        catch (...)
        {
            throw CosPropertyService::MultipleExceptions ();
        }
                                    
        //set_requested_restart_warmth ( application_def.default_restart_warmth );

        set_indicator_relevant_status(application_def.indicator_relevant_status);
        // store default properties values
        store_default_properties ();
        
        m_applicationDefined = true;
    }
    
    CdmwPlatformMngt::Process_ptr Application_impl::
        addProcess (CdmwPlatformMngt::ProcessDef *
        process_def) throw (CdmwPlatformMngt::ProcessAlreadyExists,
        CdmwPlatformMngt::HostNotFound,
        CdmwPlatformMngt::DuplicateEntity,
        CdmwPlatformMngt::DuplicateService,
        CdmwPlatformMngt::DuplicateStep,
        CosPropertyService::MultipleExceptions,
        CORBA::SystemException)
    {
        // if host name is "localhost" replace by the real host name
        if (!strcmp (process_def->host_name (), "localhost"))
        {
            // get host name and set in value type
            std::string name = OsSupport::OS::get_hostname ();
            process_def->host_name (name.c_str ());
        }
        
        const char *p_host_name = process_def->host_name ();
        const char *p_process_name = process_def->name ();
        
        // build the process relative key
        std::string process_key =
            ManagedElementKey::Get_process_rel_key (p_host_name, p_process_name);
        
        {

            CDMW_READER_RECURSIVE_MUTEX_GUARD (m_application_recursive_mutex);
          
            ProcessServants::iterator processIt =
                m_process_servants.find (process_key.c_str ());
            
            if (processIt != m_process_servants.end ())
              {       
                  throw CdmwPlatformMngt::ProcessAlreadyExists ();
            }

        }
        
        // build the process path
        std::string process_path =
            ManagedElementKey::Get_process_path (m_element_path.in (),
            p_host_name, p_process_name);
        
        // Create a process 
        Process_impl *processImpl = 0;
        
        // Servant var to take pointer ownership and to release
        // ref counter from 2 to 1 after object activation
        PortableServer::ServantBase_var processServant;
        
        try
        {
            if (process_def->type () == CdmwPlatformMngt::UNMANAGED_PROCESS)
            {
                UnmanagedProcess_impl *unmanagedProcess =
                    new UnmanagedProcess_impl (m_poa.in (), process_def,
                    process_key.c_str (),
                    process_path.c_str (),
                    this,
                    m_element_repository.in ());
                
                // take pointer ownership
                processServant = unmanagedProcess;
                
                // set the end creation for process
                // in case of exception the servant will be deleted automatically
                unmanagedProcess->end_creation (process_def);
                
                // Add it to the map of process 
       
                CDMW_WRITER_RECURSIVE_MUTEX_GUARD (m_application_recursive_mutex);
                 
                m_process_servants.
                    insert (ProcessServants::
                    value_type (process_key.c_str (), unmanagedProcess));
                processImpl = unmanagedProcess;
   
            }
            else
            {
                CdmwPlatformMngt::ManagedProcessDef * managed_process_def
                    = CdmwPlatformMngt::ManagedProcessDef::_downcast (process_def);
                
                
                ManagedProcess_impl *managedProcess =
                    new ManagedProcess_impl (
            m_poa.in (), 
            managed_process_def,
                    process_key.c_str (),
                    process_path.c_str (),
                    this,
                    m_element_repository.in ());
                
                // take pointer ownership
                processServant = managedProcess;
                
                // set the end creation for process
                // in case of exception the servant will be deleted automatically
                managedProcess->end_creation (managed_process_def);
                
                // Add it to the map of process 
                CDMW_WRITER_RECURSIVE_MUTEX_GUARD (m_application_recursive_mutex);
              
                m_process_servants.
                    insert (ProcessServants::
                    value_type (process_key.c_str (), managedProcess));
                
                processImpl = managedProcess;

                // see if there are components inside the managed process ... 
                CORBA::ULong num_components = managed_process_def->components().length() ;

           // intance all necessary components
           CORBA::ULong ii;

          //call add_components del managed process
           for ( ii=0; ii<num_components ; ii++)
             {
                      managedProcess->add_component(
                       ((managed_process_def->components())[ii]) );

                      std::string deployment_plan(((managed_process_def->components())[ii]).deployment_plan);
                      std::string process_name(managedProcess->name()); 
                      DeploymentInfoMap::iterator deploy_it = m_deployment_info_multimap.find(deployment_plan);
                      // if the deployment plan has not already been set, insert into m_deployment_info_multimap 
                      if ((deploy_it==m_deployment_info_multimap.end()))
                      {
                          m_deployment_info_multimap.insert(DeploymentInfoMap::value_type(deployment_plan, process_name)); 
                          //Updating the m_deployment_info_sequence
                          deployment_info_multimap_to_sequence(m_deployment_info_multimap);
                      }
              // if the deployment plan entry has not already been set for this process
                      // insert into m_deployment_info_multimap
                      else
              {
               int processes_with_same_deployment_plan = m_deployment_info_multimap.count(deployment_plan);

               bool found = false;
                       for(int current_it=0; current_it <  processes_with_same_deployment_plan; ++current_it)
               {    
                    if (deploy_it->second != process_name)
                            {   
                    ++deploy_it;
                }
                else
                {
                    found = true;
                break;
                }
            } 
            if (!found) 
            {
                m_deployment_info_multimap.insert(DeploymentInfoMap::value_type(deployment_plan, process_name));
                //Updating the m_deployment_info_sequence
                deployment_info_multimap_to_sequence(m_deployment_info_multimap);
            }
                       }
            }
            }

        }
        catch (const OutOfMemoryException &)
        {
            throw CORBA::NO_MEMORY (OrbSupport::NO_MEMORY, CORBA::COMPLETED_NO);
        }
        catch (const std::bad_alloc &)
        {
            throw CORBA::NO_MEMORY (OrbSupport::NO_MEMORY, CORBA::COMPLETED_NO);
        }
        
        // activate process servant
        processImpl->activate ();
        CdmwPlatformMngt::Process_var process = processImpl->_this ();
        
        try
        {
            // add in repository the process object
            m_element_repository->add_element (process_path.c_str (),
                CdmwPlatformMngt::PROCESS_TYPE,
                process.in ());
        }
        catch (const CdmwPlatformMngt::ElementPathInvalid&)
        {
            throw CORBA::INTERNAL (OrbSupport::INTERNAL, CORBA::COMPLETED_NO);
        }
        catch (const CdmwPlatformMngt::ElementAlreadyExists& )
        {
              throw CdmwPlatformMngt::ProcessAlreadyExists ();
        }
    
        // set the entry in HostProcessInitSequencer, to restart process after host reboot
        /*Cdmw::PlatformMngt::ProcessHostInfo info;
        info.process_name = CORBA::string_dup (processImpl->name());
        info.process_impl  = processImpl;
        info.kind_of_start = DEFAULT; 
        if (true == processImpl->created_dynamically())
          info.kind_of_start = ADDED;
        processImpl->set_proc_info(info, 0);
        */
        //HostProcessInitSequencer::get_instance()->set_init_entry(processImpl->get_host()->name(),info);
        //HostProcessInitSequencer::get_instance()->set_run_entry(processImpl->get_host()->name(),info);
        //std::string process_name = processImpl->name();


    
        return process._retn ();
    }
    
    void Application_impl::removeProcess (const char *process_key)
        throw (CdmwPlatformMngt::ProcessNotFound,
        CdmwPlatformMngt::ProcessStillActive, CORBA::SystemException)
    {
        try
        {

            CDMW_WRITER_RECURSIVE_MUTEX_GUARD (m_application_recursive_mutex);
       
            ProcessServants::iterator processIt =
                m_process_servants.find (process_key);
            if (processIt == m_process_servants.end ())
                throw CdmwPlatformMngt::ProcessNotFound ();
            
            Process_impl *p_process_impl = processIt->second;
            
            CORBA::String_var statusInfo;
            CdmwPlatformMngt::ProcessStatus status
                = p_process_impl->get_status (statusInfo.out ());
            
            if ((status == CdmwPlatformMngt::PROCESS_INITIALISING) ||
                (status == CdmwPlatformMngt::PROCESS_STEP_PERFORMED) ||
                (status == CdmwPlatformMngt::PROCESS_INITIALISED) ||
                (status == CdmwPlatformMngt::PROCESS_RUN_REQUEST) ||
                (status == CdmwPlatformMngt::PROCESS_RUNNING) ||
                (status == CdmwPlatformMngt::PROCESS_STOPPING))
                throw CdmwPlatformMngt::ProcessStillActive ();
            
            // set the entry in HostProcessInitSequencer, to restart process after host reboot
            Cdmw::PlatformMngt::ProcessHostInfo info;
            info.process_name = CORBA::string_dup (p_process_impl->name());
            info.process_impl  = p_process_impl;
            if (true == p_process_impl->created_dynamically())
                 info.kind_of_start = DEFAULT_REMOVED;

            p_process_impl->set_proc_info(info, 1);
            //HostProcessInitSequencer::ance()->remove_init_entry(p_process_impl->get_host()->name(),info);
            //HostProcessInitSequencer::get_instance()->remove_run_entry(p_process_impl->get_host()->name(),info);

            // suppress process servant from map
            m_process_servants.erase (processIt);
            
            // remove from repository the process object
            m_element_repository->remove_element (p_process_impl->
                get_element_path ());
            
            // deactivate process servant
            p_process_impl->remove();

        }
        catch (const CdmwPlatformMngt::ElementPathInvalid&)
        {
            throw CORBA::INTERNAL (OrbSupport::INTERNAL, CORBA::COMPLETED_NO);
        }
        catch (const CdmwPlatformMngt::ManagedElementNotFound&)
        {
            throw CdmwPlatformMngt::ProcessNotFound ();
        }
    }
    
    void Application_impl::removeProcesses ()
    {
        // deactivate process object referenced in map
        ProcessServants::iterator processIt;
        
        for (processIt = m_process_servants.begin ();
        processIt != m_process_servants.end (); processIt++)
        {
            Process_impl *process = processIt->second;
            
            try
            {
                // remove from repository the process object
                m_element_repository->remove_element (process->
                    get_element_path ());
                
                
                // deactivate process servant
                process->remove();
            }
            catch (...)
            {
            }
        }
        
        m_process_servants.clear ();
    }
    
    Process_impl *Application_impl::getProcess (const char *process_key)
        throw (CdmwPlatformMngt::ProcessNotFound,
        CdmwPlatformMngt::ElementPathInvalid, CORBA::SystemException)
    {
        Process_impl *process = 0;
        
        try
        {
            // check that process key is complete (host defined)
            
            std::string process_name;
            std::string host_name;
            
            // if host name undefined in the process key
            if (ManagedElementKey::
                Check_process_rel_key (process_key, process_name))
            {
                try
                {
                    // search the host name from the process name if it is unique
                    process = getProcessHostName (process_name, host_name);
                }
                catch (const ProcessNameNotUnique &)
                {
                    throw CdmwPlatformMngt::ElementPathInvalid ();
                }
            }
            
            // else the process key is ok, search in map
            else
            {

                CDMW_READER_RECURSIVE_MUTEX_GUARD (m_application_recursive_mutex);

                ProcessServants::iterator processIt =
                    m_process_servants.find (process_key);
                if (processIt == m_process_servants.end ())
                {
                    throw CdmwPlatformMngt::ProcessNotFound ();
                }
                
                process = processIt->second;

            }
        }
        catch (...)
        {
            throw CdmwPlatformMngt::ProcessNotFound ();
        }
        
        return process;
    }
    
    
    Process_impl *Application_impl::getProcess (const char *process_name,
        const char *host_name)
        throw (CdmwPlatformMngt::ProcessNotFound, CORBA::SystemException)
    {
        Process_impl *process = 0;
        
        try
        {
            std::string process_key =
                ManagedElementKey::Get_process_rel_key (host_name, process_name);

            CDMW_READER_RECURSIVE_MUTEX_GUARD (m_application_recursive_mutex);

            ProcessServants::iterator processIt =
                m_process_servants.find (process_key);
            if (processIt == m_process_servants.end ())
            {
                throw CdmwPlatformMngt::ProcessNotFound ();
            }
            
            process = processIt->second;

        }
        catch (...)
        {
            throw CdmwPlatformMngt::ProcessNotFound ();
        }
        
        return process;
    }
    
    
    
    
    CdmwPlatformMngt::ProcessObserver_ptr Application_impl::
        get_process_observer ()throw (CORBA::SystemException)
    {
        return m_processObserver->_this ();
    }
    
    ProcessObserver_impl *Application_impl::get_process_observer_servant ()
    {
        return m_processObserver;
    }
    
    CdmwPlatformMngtService::ServiceBroker_ptr Application_impl::
        get_service_broker ()throw (CORBA::SystemException)
    {
        return m_serviceBroker->_this ();
    }
    
    
    CdmwPlatformMngt::HostContainer_ptr Application_impl::
        get_host_container ()
    {
        CdmwPlatformMngt::HostContainer_var host_container = m_hostContainer;
        
        return host_container._retn ();
    }
    
    CdmwPlatformMngt::ApplicationObserver_ptr Application_impl::
        get_application_observer ()
    {
        CdmwPlatformMngt::ApplicationObserver_var app_observer =
            m_applicationStartData->application_observer;
        
        return app_observer._retn ();
    }
    
    
    
    RWServiceDefContainer_impl *Application_impl::
        get_rwservice_defcontainer ()
    {
        return m_rwServiceDefContainer;
    }
    
    SupervisionEventNotifier *Application_impl::get_event_notifier ()
    {
        return m_eventNotifier;
    }
    
    
    EventHandler *Application_impl::get_event_handler ()
    {
        return &m_eventHandler;
    }
    
    
    
    
    void Application_impl::destroyInitSequencer ()
    {   
        // Stop and destroy the init sequencer 

        if (m_initSequencer != 0)
        {
            m_initSequencer->stop ();
            delete m_initSequencer;
            m_initSequencer = 0;

        }
        if (m_initSyncTaskFactory != 0)
        {
            delete m_initSyncTaskFactory;
            m_initSyncTaskFactory = 0;
        }
    }
    
    void Application_impl::destroyStopSequencer ()
    {

        
        // Stop and destroy the stop sequencer 
        if (m_stopSequencer != 0)
        {
            m_stopSequencer->stop ();
            
            delete m_stopSequencer;
            m_stopSequencer = 0;
        }
        
        if (m_stopSyncTaskFactory != 0)
        {
            delete m_stopSyncTaskFactory;
            m_stopSyncTaskFactory = 0;
        }
    }
    
    void Application_impl::destroyStarter ()
    {
        // Stop and destroy the run starter 
        if (m_starter != 0)
        {
            m_starter->stop ();
            
            delete m_starter;
            m_starter = 0;
        }
        
        if (m_taskFactory != 0)
        {
            delete m_taskFactory;
            m_taskFactory = 0;
        }
    }
    
    void Application_impl::degradation_event ()
    {
        bool changed = false;
        
        CDMW_MUTEX_GUARD (m_mutexMode);

        
        if (m_mode == CdmwPlatformMngt::NORMAL_MODE)
        {
            changed = true;
        }
        
        if (changed)
        {
            // Log and notify the change of mode
            CdmwPlatformMngt::FunctioningMode mode =
                CdmwPlatformMngt::DEGRADED_MODE;
            CORBA::String_var mode_info = CORBA::string_dup ("");
            
            set_mode (mode, mode_info.in ());
            
            notifyModeChangeEvent (mode, mode_info.in ());
            
            try
            {
                // Notify the degradation event to the application observers
                (m_applicationStartData->application_observer)->
                    notify_degradation (m_element_name.in ());
            }
            catch (const CORBA::Exception & e)
            {
                // TODO: What to do with the raised exception ?!!
                std::string errorName = e._name ();
                LogMngr::logMessage (INF,
                    (errorName +
                    " exception raised by app observer notification").
                    c_str ());
            }
        }
    }
    
    void Application_impl::suspension_event ()
    {

    CDMW_MUTEX_GUARD(*m_check_consistency_status_recursive_mutex);

    
    
  
        CDMW_SM_WRITER_LOCK_GUARD (*m_stateMachine);
      
        m_stateMachine->getState()->suspension_event (m_stateMachine);
        
        // update record in datastore
        set_record_in_datastore ();

    }
    
    void Application_impl::completion_event ()
    {
 
        CDMW_MUTEX_GUARD(*m_check_consistency_status_recursive_mutex);

        CDMW_SM_WRITER_LOCK_GUARD (*m_stateMachine);

        m_stateMachine->getState ()->completion_event (m_stateMachine);

        // update record in datastore
        set_record_in_datastore ();

    }
    
    void Application_impl::non_completion_event ()
    {
    
        CDMW_MUTEX_GUARD(*m_check_consistency_status_recursive_mutex);
 
        CDMW_SM_WRITER_LOCK_GUARD (*m_stateMachine);

        m_stateMachine->getState ()->non_completion_event (m_stateMachine);
        
        // update record in datastore
        set_record_in_datastore ();

    }
   

    void Application_impl::check_consistency_of_state()
    {

        if ((m_stopSequencer!=0 && !m_stopSequencer->is_completed()) || (m_initSequencer!=0 && !m_initSequencer->is_completed()) || (m_starter != 0 &&
        !m_starter->is_completed()) ){

          return;
    }

        CDMW_SM_WRITER_LOCK_GUARD (*m_stateMachine);

        std::string state=m_stateMachine->getState()->check_consistency_of_state (m_stateMachine);
        CDMW_INFO(m_logger, "Application_impl::check_consistency_of_state:  Check consistency of " << name() << " application " << std::endl);      
        
        if (state != ""){
    
            m_stateMachine->setState(state);
            CDMW_INFO(m_logger, "Application_impl::check_consistency_of_state:  Check consistency of " << name() << " application Changing status" << std::endl);      
        // update record in datastore
        set_record_in_datastore ();
        }

    }

    void Application_impl::process_status_change_event(const std::string& process_name,
                                                       const std::string& process_host_name,
                                                       CdmwPlatformMngt::ProcessStatus process_status)
    {
        
        std::string composed_process_name = process_name + "/" + process_host_name;
    
        if (!getProcess(process_name.c_str(), process_host_name.c_str())->get_indicator_relevant_status())
        {
           return;        
        }
        {

        CDMW_WRITER_RECURSIVE_MUTEX_GUARD (m_application_recursive_mutex); 
    
        // remove the old process entry from the map of processes
        ProcessesStatusInfoMap::iterator it = m_processes_status_info_map.find(composed_process_name);
        if ( it!=m_processes_status_info_map.end() ) 
            m_processes_status_info_map.erase(it);

        // Log message
        std::ostringstream log_message;
        log_message <<" process_status_change_event " <<composed_process_name<< " " << process_status_strings[process_status] <<ends; 

        LogMngr::logMessage(INF, MSG_ID_CHECK_CONSISTENCY_APPLICATION, log_message.str().c_str());

        m_processes_status_info_map.insert( ProcessesStatusInfoMap::value_type (composed_process_name, process_status));
        } 
        // call a check on Application State Machine to make the application align its state ();

        if ( 
            CdmwPlatformMngt::PROCESS_RUNNING==process_status || CdmwPlatformMngt::PROCESS_STOPPED==process_status ||   
            CdmwPlatformMngt::PROCESS_FAILED_ERROR==process_status || CdmwPlatformMngt::PROCESS_FAILED_DEATH==process_status ||
            CdmwPlatformMngt::PROCESS_FAILED_DISCREPANCY==process_status || CdmwPlatformMngt::PROCESS_FAILED_TIMEOUT==process_status ||
            CdmwPlatformMngt::PROCESS_FAILED_INVALID==process_status || CdmwPlatformMngt::PROCESS_FAILED_NO_RESPONSE==process_status)
        {
            check_consistency_of_state();
        }

    }

    

    void Application_impl::process_deleted(const std::string& process_name, const std::string& process_host_name)
    {
        std::string composed_process_name = process_name + "/" + process_host_name;

        // remove the not running process from the Map of running processes
        ProcessesStatusInfoMap::iterator it = m_processes_status_info_map.find(composed_process_name);
        if ( it != m_processes_status_info_map.end() )
            m_processes_status_info_map.erase(it);

        // call a check on System State Machine to make the application align its state
        check_consistency_of_state();
    }

    CdmwPlatformMngt::ApplicationStatus Application_impl::
        get_internal_status ()
    {

        CDMW_SM_READER_LOCK_GUARD (*m_stateMachine);

        // get status
        CdmwPlatformMngt::ApplicationStatus applicationStatus =
            m_stateMachine->getState ()->status ();

        return applicationStatus;

    }
    

    CORBA::Long Application_impl::get_max_restart_attempt ()
              throw(CORBA::SystemException)
    {
        //extracting form any ...

        // Get max_nbr_of_restart_attempt
        CORBA::Any_var any_var;
        CdmwPlatformMngt::AutoRestartInfo* ari;  

        get_attribute (CdmwPlatformMngt::AUTORESTART_INFO, any_var.out(), CdmwPlatformMngt::PRE_DEFINED);
        any_var.in() >>= ari; 

        return (ari->max_nbr_of_restart_attempt) ;

    }

    void Application_impl::set_max_restart_attempt (CORBA::Long max_nrestart) throw(CORBA::SystemException)
    {

        CdmwPlatformMngt::AutoRestartInfo autores_info ;

    //fill struct autores_info
        autores_info.max_nbr_of_restart_attempt = max_nrestart ;

        autores_info.auto_restart_control = CdmwPlatformMngt::USE_COUNTER;


        CORBA::Any_var any_var = new CORBA::Any();

        *any_var <<= autores_info ;    

        set_attribute(CdmwPlatformMngt::AUTORESTART_INFO, any_var.in(), CdmwPlatformMngt::PRE_DEFINED);

    }
 

    CORBA::Long Application_impl::restart_attempt ()throw (CORBA::
        SystemException)
    {
        return m_current_restart_attempt; 
    }
    
    void Application_impl::set_restart_attempt (CORBA::Long nrestart)
              throw(CORBA::SystemException)
    {
        m_current_restart_attempt = nrestart;
    }
  
    
    void Application_impl::set_force_manual_start( const bool force_manual_start )
      throw (CORBA::SystemException, CdmwPlatformMngt::AttributeNotFound) 
    {
        CORBA::Any_var any_var = new CORBA::Any();
        try
    {           
        *any_var <<= CORBA::Any::from_boolean(force_manual_start);    
        set_attribute_internal (CdmwPlatformMngt::FORCE_MANUAL_START, any_var.in(), CdmwPlatformMngt::PRE_DEFINED);

    }
    catch(const CdmwPlatformMngt::AttributeNotFound&) 
        {
            CDMW_FATAL(m_logger,
                       "Property not found: " << CdmwPlatformMngt::FORCE_MANUAL_START);
        }
    }

    bool Application_impl::get_force_manual_start()
      throw (CORBA::SystemException, CdmwPlatformMngt::AttributeNotFound)
    {
        CORBA::Any_var any_var = new CORBA::Any();
    CORBA::Boolean b;
    try
    {  
        get_attribute (CdmwPlatformMngt::FORCE_MANUAL_START, any_var.out(), CdmwPlatformMngt::PRE_DEFINED);
        any_var >>= CORBA::Any::to_boolean (b);
    }
        catch(const CdmwPlatformMngt::AttributeNotFound&) 
        {
            CDMW_FATAL(m_logger,
                       "Property not found: " << CdmwPlatformMngt::FORCE_MANUAL_START);
        }
        return b;
        }
        
    CdmwPlatformMngt::ApplicationStatus Application_impl::property_to_status (const char* status)
    throw (CdmwPlatformMngt::AttributeNotFound)
    {
        CdmwPlatformMngt::ApplicationStatus appl_status; 
        bool result = false;
        for ( int i=0 ; i<10; i++)
        {
            if (strcmp(status, appl_status_strings[i].c_str()) == 0)
            {
                appl_status = Status[i];
                result = true; 
                return appl_status;
            }
        }
        if (!result)
        {
            throw CdmwPlatformMngt::AttributeNotFound();
        }
        
        return appl_status;
    }
    
    CdmwPlatformMngt::ApplicationStatus Application_impl::get_status (CORBA::
        String_out
        status_info)
        throw (CORBA::SystemException)
    {
        
        CdmwPlatformMngt::ApplicationStatus appl_status;
        const char* status = "";
        try
        {
            // Get predefined attributes of process
            CORBA::Any_var any_var = new CORBA::Any();

            
            get_attribute(CdmwPlatformMngt::TECH_STATUS, any_var.out(), CdmwPlatformMngt::PRE_DEFINED);           
            any_var.in() >>=  CORBA::Any::to_string(status,0);            
            
            appl_status = property_to_status (status);             
            // get info associated to status
            status_info = CORBA::string_dup (get_status_info());

            return appl_status;
        }
        catch (const CdmwPlatformMngt::AttributeNotFound&)
        {
            throw CORBA::BAD_PARAM(OrbSupport::BAD_PARAM, CORBA::COMPLETED_NO);
        }
        catch (const std::bad_alloc&)
        {
            throw CORBA::NO_MEMORY (OrbSupport::NO_MEMORY, CORBA::COMPLETED_NO);
        }
        catch (...)
        {
            throw CORBA::BAD_PARAM(OrbSupport::BAD_PARAM, CORBA::COMPLETED_NO);
        }
    }
    
    
    
    CdmwPlatformMngt::FunctioningMode Application_impl::get_mode (CORBA::
        String_out
        mode_info)
        throw (CORBA::SystemException)
    {
        try
        {
            // get record
            CdmwPlatformMngt::ApplicationRecord_var application_record =
                get_record_from_datastore ();
            
            // get mode
            CdmwPlatformMngt::FunctioningMode functioning_mode =
                application_record->mode;
            
            // get info associated to mode
            mode_info = CORBA::string_dup ((application_record->mode_info).in ());
            
            return functioning_mode;
        }
        catch (const std::bad_alloc &)
        {
            throw CORBA::NO_MEMORY (OrbSupport::NO_MEMORY, CORBA::COMPLETED_NO);
        }
    }
    
    
    void Application_impl::set_mode (CdmwPlatformMngt::
        FunctioningMode functioning_mode,
        const char *mode_info)
    {
        m_mode = functioning_mode;
        m_modeInfo = CORBA::string_dup (mode_info);
        
        // update record in datastore
        set_record_in_datastore ();
    }
    
    
    CdmwPlatformMngt::ApplicationSnapshot *
        Application_impl::get_snapshot ()throw (CORBA::SystemException)
    {
        try
        {
    
            //MRCDMW_MUTEX_GUARD(*m_check_consistency_status_recursive_mutex);

            //MRCDMW_MUTEX_GUARD(m_application_recursive_mutex);

            // Get the snapshot of all the processes
            CdmwPlatformMngt::ProcessesSnapshot_var processesSnapshot
                = new CdmwPlatformMngt::ProcessesSnapshot ();
            
            ProcessServants::iterator processIt;
            unsigned int i;
            
            processesSnapshot->length (m_process_servants.size ());
            for (processIt = m_process_servants.begin (), i = 0;
            processIt != m_process_servants.end (); processIt++, i++)
            {
                (*processesSnapshot)[i] = *(processIt->second->get_snapshot ());
            }
            
            // Gets all the FTGroups snapshots
            
                        
              CdmwPlatformMngt::FTGroups_var ftgroups;
              CdmwPlatformMngt::FTGroupsIterator_var ftgroupsIt;
              CORBA::ULong nbr_of_ftgroups;
               
              nbr_of_ftgroups = get_number_of_ft_groups ();
               
              get_all_ft_groups (nbr_of_ftgroups,//
                   ftgroups.out (), ftgroupsIt.out ());
              
     
              CdmwPlatformMngt::FTGroupsSnapshot_var ftgroupsSnapshot
                   = new CdmwPlatformMngt::FTGroupsSnapshot ();
               
              ftgroupsSnapshot->length (nbr_of_ftgroups);
              for (i = 0; i < nbr_of_ftgroups; i++)
               {
                   (*ftgroupsSnapshot)[i] = *(ftgroups[i]->get_snapshot ());
               }
             


            // Gets all the LBGroups snapshots
            
                        
              CdmwPlatformMngt::LBGroups_var lbgroups;
              CdmwPlatformMngt::LBGroupsIterator_var lbgroupsIt;
              CORBA::ULong nbr_of_lbgroups;
               
              nbr_of_lbgroups = get_number_of_lb_groups ();
               
              get_all_lb_groups (nbr_of_lbgroups,  //
                   lbgroups.out (), lbgroupsIt.out ());
              
     
              CdmwPlatformMngt::LBGroupsSnapshot_var lbgroupsSnapshot
                   = new CdmwPlatformMngt::LBGroupsSnapshot ();
               
              lbgroupsSnapshot->length (nbr_of_lbgroups);
              for (i = 0; i < nbr_of_lbgroups; i++)
               {
                   (*lbgroupsSnapshot)[i] = *(lbgroups[i]->get_snapshot ());
               }
            
            // Get the snapshot of all entities of application
            CdmwPlatformMngt::EntitiesSnapshot_var entitiesSnapshot
                = new CdmwPlatformMngt::EntitiesSnapshot ();
            
            // Gets all the entities
            CdmwPlatformMngt::Entities_var entities;
            CdmwPlatformMngt::EntitiesIterator_var entitiesIt;
            CORBA::ULong nbr_of_entities;
            
            nbr_of_entities = get_number_of_entities ();
            
            get_all_entities (nbr_of_entities,
                entities.out (), entitiesIt.out ());
            
            entitiesSnapshot->length (nbr_of_entities);
            for (i = 0; i < nbr_of_entities; i++)
            {
                (*entitiesSnapshot)[i] = *(entities[i]->get_snapshot ());
            }
            
            //Gets all the properties
            CosPropertyService::PropertiesIterator_var propertiesIt;
            CORBA::ULong nbr_of_properties;
            CdmwPlatformMngt::PropertiesSnapshot_var propertiesSnapshot =
                new CdmwPlatformMngt::PropertiesSnapshot;
            nbr_of_properties = get_number_of_properties ();
            CosPropertyService::PropertyNames_var prop_names;
            CosPropertyService::PropertyNamesIterator_var prop_namesIt;
            get_all_property_names (nbr_of_properties, prop_names.out (),
                prop_namesIt.out ());
            propertiesSnapshot->length (nbr_of_properties-2);
            CORBA::ULong snap_prop=0;
            for (i = 0; i < nbr_of_properties; i++)
            {
                if ((strcmp (prop_names[i], "Cdmw.TechStatus")==0) ||
                        (strcmp (prop_names[i], "Cdmw.TechStatusInfo")==0))
                {
                    // do not add the TECH_STATUS and TECH_STATUS_INFO properties to the snapshot 
                }
                else
                {
                    CdmwPlatformMngt::PropertySnapshot_var propertySnapshot =
                        new CdmwPlatformMngt::PropertySnapshot ();
            
                    propertySnapshot->property_name =
                        CORBA::string_dup (prop_names[i]);
                    CORBA::Any prop_value;
                    prop_value = *(get_property_value (prop_names[i]));
                    CORBA::TypeCode_var tc;
                    tc = prop_value.type ();
                    switch (tc->kind ())
                    {
                    default:
                        break;
                    case CORBA::tk_string:
                        propertySnapshot->property_type =
                            CORBA::string_dup ("String");
                        break;
                    case CORBA::tk_short:
                        propertySnapshot->property_type = CORBA::string_dup ("Short");
                        break;
                    case CORBA::tk_float:
                        propertySnapshot->property_type = CORBA::string_dup ("Float");
                        break;
                    };
                    CosPropertyService::PropertyModeType mode =
                        get_property_mode (prop_names[i]);
                    propertySnapshot->property_value = prop_value;

                    if (mode == CosPropertyService::normal) propertySnapshot->property_mode = CORBA::string_dup ("Read/Write");
                    else if (mode == CosPropertyService::read_only) propertySnapshot->property_mode = CORBA::string_dup ("Read/Only");
                    else propertySnapshot->property_mode = CORBA::string_dup ("Pre-defined");

                    propertySnapshot->property_value = prop_value;
            (*propertiesSnapshot)[snap_prop] = *(propertySnapshot);
            
                    snap_prop++;
                }
            }
            
            
            
            // Create an application snapshot
            CORBA::String_var modeInfo;
            CORBA::String_var statusInfo;
            
            CdmwPlatformMngt::ApplicationSnapshot_var snapshot
                = new CdmwPlatformMngt::ApplicationSnapshot;
            
            // note : the copy of a snapshot sequence (= operator) 
            //        make a deeply copy with a new memory allocation
            //        for the sequence        
            
            snapshot->application_name = m_element_name.in ();
            snapshot->application_status = get_status (statusInfo.out ());
            snapshot->status_info = statusInfo;
            snapshot->application_mode = get_mode (modeInfo.out ());
            snapshot->mode_info = modeInfo;
            snapshot->element_path = element_path();
            snapshot->application_processes = processesSnapshot.in ();
            snapshot->application_entities = entitiesSnapshot.in ();
            snapshot->application_properties = propertiesSnapshot.in ();
            snapshot->application_ft_component_groups = ftgroupsSnapshot.in() ;
            snapshot->application_lb_component_groups = lbgroupsSnapshot.in() ;
 
            return snapshot._retn ();
      }
      catch (const std::bad_alloc &)
      {
          throw CORBA::NO_MEMORY (OrbSupport::NO_MEMORY, CORBA::COMPLETED_NO);
      }
    }
    
    
    /**
    *Purpose:
    *<p> facade to get an attribute
    *
    */
    
    void Application_impl::get_attribute (const char *attr_name, CORBA::Any_out ret_attr, CdmwPlatformMngt::Owner flag) 
    throw (CORBA::SystemException, CdmwPlatformMngt::AttributeNotFound)
    {
        std::string tmp1 (attr_name);
        if (flag == CdmwPlatformMngt::PRE_DEFINED)
        {
            if (tmp1 == CdmwPlatformMngt::MODE)
            {
                CdmwPlatformMngt::FunctioningMode func_mode;
                CORBA::String_var mode_info;
                CORBA::Any_var tempany = new CORBA::Any;

                func_mode = get_mode (mode_info);
                tempany.inout () <<= func_mode;

                ret_attr = tempany._retn ();
            }
            else if (tmp1 == CdmwPlatformMngt::MODE_INFO)
            {
                CdmwPlatformMngt::FunctioningMode func_mode;
                CORBA::String_var mode_info;
                CORBA::Any_var tempany = new CORBA::Any;

                func_mode = get_mode (mode_info);
                tempany.inout () <<= mode_info;

                ret_attr = tempany._retn ();
            }
            else /* Case of predefined property */
            {
                std::ostringstream property_name;
                property_name << CdmwPlatformMngt::CDMW_PREFIX << attr_name << std::ends;
                try
                {
                    CORBA::Any_var outany = get_property_value ( property_name.str().c_str() );
                    ret_attr = outany._retn();
                }
                catch (const CosPropertyService::PropertyNotFound&)
                {
                    throw CdmwPlatformMngt::AttributeNotFound ();
                }
            }
        }
        else
        {
            /*case of user defined attribute */
            try
            {
                CORBA::Any * outany = get_property_value (attr_name);
                ret_attr = outany;
            }
            catch (const CosPropertyService::PropertyNotFound&)
            {
                throw CdmwPlatformMngt::AttributeNotFound ();
            }
        }

    }
    
    
    /**
    *Purpose:
    *<p> facade to set an attribute
    *
    */    
    void Application_impl::set_attribute (const char* attr_name, const CORBA::Any& attr, CdmwPlatformMngt::Owner flag) 
        throw (CORBA::SystemException, CdmwPlatformMngt::AttributeNotFound, CdmwPlatformMngt::ReadOnlyAttribute)
    {        
        if (flag == CdmwPlatformMngt::PRE_DEFINED)
        {
            std::string tmp1 (attr_name);
            /* check if the predefined property is readonly */
            if ( (tmp1 == CdmwPlatformMngt::REQUESTED_RESTART_WARMTH) ||
                 (tmp1 == CdmwPlatformMngt::TECH_STATUS) ||
                 (tmp1 == CdmwPlatformMngt::TECH_STATUS_INFO) ) 
            {
                throw CdmwPlatformMngt::ReadOnlyAttribute(); 
            }
        }
        try
        {
            set_attribute_internal ( attr_name, attr, flag );
        }
    catch (const Cdmw::BadOrderException& ex)
        {
        throw CORBA::BAD_PARAM(OrbSupport::BAD_PARAM, CORBA::COMPLETED_NO);
        }
        catch (...) {
            throw;
        }
    }
    
    
    CORBA::ULong Application_impl::get_number_of_processes() 
        throw (CORBA::SystemException)
    {
        CORBA::ULong len = m_process_servants.size ();
        return len;
    }

    CORBA::ULong Application_impl::get_number_of_processes_with_relevant_indicator()
       throw (CORBA::SystemException)
    {     
         CORBA::ULong count = 0L; 
         ProcessServants::iterator processesIt;

         for (processesIt = m_process_servants.begin(); processesIt != m_process_servants.end(); processesIt++)
         {
             if (processesIt->second->get_indicator_relevant_status())
                 ++count;
         }
         return count; 
     }

    
    void Application_impl::set_indicator_relevant_status(const bool indicator_relevant_status) throw(CORBA::SystemException )
    {
         m_indicator_relevant_status = indicator_relevant_status;
    }

    bool Application_impl::get_indicator_relevant_status() throw(CORBA::SystemException )
    {
        return m_indicator_relevant_status;
    }
       

    CdmwPlatformMngt::Processes *
        Application_impl::get_processes (CORBA::ULong how_many,
        CdmwPlatformMngt::
        ProcessesIterator_out
        processes_iterator) throw (CORBA::
        SystemException)
    {
        try
        {

            CDMW_READER_RECURSIVE_MUTEX_GUARD(m_application_recursive_mutex);

            ProcessServants::iterator processesIt;
            
            // Get the how_many application references   
            size_t len = m_process_servants.size ();
            
            // set length of requested sequence and rest of sequence
            size_t requested_len = len;
            size_t rest_len = 0;
            
            // adjust to how_many requested       
            if (len > how_many)
            {
                requested_len = how_many;
                rest_len = len - how_many;
            }
            
            // Get the how_many process references
            unsigned int count;
            CdmwPlatformMngt::Processes_var processes
                = new CdmwPlatformMngt::Processes ();
            
            processes->length (requested_len);
            for (processesIt = m_process_servants.begin (), count = 0;
            count < requested_len; processesIt++, count++)
            {
                processes[count] = processesIt->second->_this ();
            }
            
            // Get an iterator on the rest of the process 
            std::list < CdmwPlatformMngt::Process_var > processesRemaining;
            for (; processesIt != m_process_servants.end (); processesIt++)
            {
                CdmwPlatformMngt::Process_var process
                    = processesIt->second->_this ();
                
                processesRemaining.push_back (process);
            }
            
            CdmwPlatformMngt::ProcessesIterator_var processesRemainingIt
                = ProcessesIterator_impl::create (processesRemaining);
            
            // Assign the iterator to the processes_iterator
            processes_iterator = processesRemainingIt._retn ();

            return processes._retn ();
        }
        catch (const std::bad_alloc &)
        {
            throw CORBA::NO_MEMORY (OrbSupport::NO_MEMORY, CORBA::COMPLETED_NO);
        }
    }
    
    CdmwPlatformMngt::Process_ptr Application_impl::
        get_process (const char *process_name,
        const char *host_name) throw (CdmwPlatformMngt::
        ProcessNotFound,
        CORBA::SystemException)
    {
        Process_impl *process = 0;
        
        try
        {
            // check that host is defined
            std::string host_name_buf;
            
            // if host name undefined
            if (!strcmp (host_name, ""))
            {
                // search the host name from the process name if it is unique
                process = getProcessHostName (process_name, host_name_buf);
            }
            
            // else the process key is ok, search in map
            else
            {

                CDMW_READER_RECURSIVE_MUTEX_GUARD (m_application_recursive_mutex);

                std::string process_key =
                    ManagedElementKey::Get_process_rel_key (host_name,
                    process_name);
                
                ProcessServants::iterator processIt =
                    m_process_servants.find (process_key);
                if (processIt == m_process_servants.end ())
                {
                    throw CdmwPlatformMngt::ProcessNotFound ();
                }
                
                process = processIt->second;
 
            }
            
            return process->_this ();
        }
        catch (ProcessNameNotUnique &)
        {
            throw CdmwPlatformMngt::ProcessNotFound ();
        }
        catch (...)
        {
            throw CdmwPlatformMngt::ProcessNotFound ();
        }
    }
    
    
    
    void Application_impl::define (const CdmwPlatformMngt::
        ApplicationDef & application_def)
        throw (CdmwPlatformMngt::DuplicateEntity,
        CdmwPlatformMngt::DuplicateService,
        CdmwPlatformMngt::DuplicateProcess,
        CdmwPlatformMngt::ProcessHostNotFound,
        CdmwPlatformMngt::DuplicateProcessEntity,
        CdmwPlatformMngt::DuplicateProcessService,
        CdmwPlatformMngt::DuplicateProcessStep,
        CdmwPlatformMngt::AlreadyDone,
        CdmwPlatformMngt::IncompatibleStatus,
        CosPropertyService::MultipleExceptions, CORBA::SystemException)
    {
        LogMngr::logMessage (INF,
            MSG_ID_APPLICATION_DEFINE, m_element_name.in ());

        CDMW_SM_READER_LOCK_GUARD (*m_stateMachine);

        m_stateMachine->getState ()->define (m_stateMachine, application_def);

    }
    
    CdmwPlatformMngt::Process_ptr Application_impl::
        add_process (CdmwPlatformMngt::ProcessDef *
        process_def) throw (CdmwPlatformMngt::ProcessAlreadyExists,
        CdmwPlatformMngt::HostNotFound,
        CdmwPlatformMngt::DuplicateEntity,
        CdmwPlatformMngt::DuplicateService,
        CdmwPlatformMngt::DuplicateStep,
        CdmwPlatformMngt::IncompatibleStatus,
        CosPropertyService::MultipleExceptions,
        CORBA::SystemException)
    {
        LogMngr::logMessage (INF,
            MSG_ID_PROCESS_ADD,
            process_def->name (), m_element_name.in ());
        
        CDMW_SM_READER_LOCK_GUARD (*m_stateMachine);
        
        //return m_stateMachine->getState ()->add_process (m_stateMachine,
        //    process_def);
           
        CdmwPlatformMngt::Process_ptr process_retn = m_stateMachine->getState ()->add_process (m_stateMachine, process_def);
       
        notify_configuration_change("Process Added");
        return process_retn;
    }
    
    void Application_impl::remove_process (const char *process_name,
                       const char *host_name)
                       throw (CdmwPlatformMngt::ProcessNotFound,
                       CdmwPlatformMngt::ProcessStillActive,
                       CdmwPlatformMngt::IncompatibleStatus, CORBA::SystemException)
    {
        std::string process_key =
            ManagedElementKey::Get_process_rel_key (host_name, process_name);
        
        LogMngr::logMessage (INF,
            MSG_ID_PROCESS_REMOVE,
            process_key.c_str (), m_element_name.in ());
       
        { 

        CDMW_SM_READER_LOCK_GUARD (*m_stateMachine);

        
        m_stateMachine->getState ()->remove_process (m_stateMachine,
            process_key.c_str ());

        }
         
        // remove process entry to provide application consistent status 
        process_deleted(process_name, host_name);

        notify_configuration_change("Process Removed");
        
    }
    
    void Application_impl::set_init_graph (const CdmwPlatformMngt::
                       GraphElements & process_graph)
                       throw (CdmwPlatformMngt::ReferenceNotFound,
                       CdmwPlatformMngt::CircularReference,
                       CdmwPlatformMngt::StepOutOfBound, CdmwPlatformMngt::StepJump,
                       CdmwPlatformMngt::IncompatibleStatus, CORBA::SystemException)
    {
        LogMngr::logMessage (INF,
            MSG_ID_APPLICATION_SET_INIT_GRAPH,
            m_element_name.in ());

        CDMW_SM_READER_LOCK_GUARD (*m_stateMachine);

        try
        {
            
            m_stateMachine->getState ()->set_init_graph (m_stateMachine,
                process_graph);
            
        }
        catch (OutOfMemoryException)
        {
            throw CORBA::NO_MEMORY (OrbSupport::NO_MEMORY, CORBA::COMPLETED_NO);
        } 
        
    }
    
    void Application_impl::set_stop_graph (const CdmwPlatformMngt::
                       GraphElements & process_graph)
                       throw (CdmwPlatformMngt::ReferenceNotFound,
                       CdmwPlatformMngt::CircularReference,
                       CdmwPlatformMngt::IncompatibleStatus, CORBA::SystemException)
    {
        LogMngr::logMessage (INF,
            MSG_ID_APPLICATION_SET_STOP_GRAPH,
            m_element_name.in ());
        
        CDMW_SM_READER_LOCK_GUARD (*m_stateMachine);
        try
        {
            
            m_stateMachine->getState ()->set_stop_graph (m_stateMachine,
                process_graph);
            
        }
        catch (OutOfMemoryException)
        {
            throw CORBA::NO_MEMORY (OrbSupport::NO_MEMORY, CORBA::COMPLETED_NO);
        }
    }
    
   
    void Application_impl::initialise (const CdmwPlatformMngtBase::
        StartupKind & startup_kind)
        throw (CdmwPlatformMngt::IncompatibleStatus, CORBA::SystemException)
    {
        LogMngr::logMessage (INF,
            MSG_ID_APPLICATION_INITIALISE,
            m_element_name.in ());
     
        de_activate_preemption();
  
         if (!is_preemption_active())
        
        {         
            CDMW_SM_WRITER_LOCK_GUARD (*m_stateMachine);
            
            if ( CdmwPlatformMngt::COLD_RESTART==requested_restart_warmth() )
            {
                 recover_default_properties ();
                 recover_default_entities (m_rwEntityContainer, m_default_entitydefs);
            }
            
            try
            {
                 m_stateMachine->getState ()->initialise (m_stateMachine,
                    startup_kind);
            }
            catch (...)
            {
                 // even in case of exception update record in datastore
                 set_record_in_datastore ();
            
                 // rethrow exception
                 throw;
            }
 
            // update record in datastore
            set_record_in_datastore ();
            
        }
    }
    
    
    
    CdmwPlatformMngt::LongRequestStatus Application_impl::
        initialise_and_wait (const CdmwPlatformMngtBase::
        StartupKind & startup_kind)
        throw (CdmwPlatformMngt::IncompatibleStatus, CORBA::SystemException)
    {
        CdmwPlatformMngt::ApplicationStatus appStatus;
        CdmwPlatformMngt::LongRequestStatus req_status =
            CdmwPlatformMngt::LONG_REQUEST_NOT_COMPLETED;
        bool init_req_done = false;
        
        de_activate_preemption();
        
        if (! is_preemption_active())
        {
        
        {
            // lock mutex for reading status
            CDMW_SM_READER_LOCK_GUARD (*m_stateMachine);
            
            // get the current status
            appStatus = m_stateMachine->getState ()->status ();
            
            if (appStatus == CdmwPlatformMngt::APPLICATION_INITIALISED ||
                appStatus == CdmwPlatformMngt::APPLICATION_RUN_REQUEST ||
                appStatus == CdmwPlatformMngt::APPLICATION_RUNNING ||
                appStatus == CdmwPlatformMngt::APPLICATION_PARTIAL_RUNNING)
            {
                req_status = CdmwPlatformMngt::LONG_REQUEST_COMPLETED;
                init_req_done = true;
            }
            
            else if (appStatus ==
                CdmwPlatformMngt::APPLICATION_INITIALISATION_SUSPENDED)
            {
                req_status = CdmwPlatformMngt::LONG_REQUEST_SUSPENDED;
                init_req_done = true;
            }
            
            else if (appStatus ==
                CdmwPlatformMngt::APPLICATION_INITIALISATION_INCOMPLETE)
            {
                req_status = CdmwPlatformMngt::LONG_REQUEST_NOT_COMPLETED;
                init_req_done = true;
            }
        }
        
        
        // if initialise request not done :
        // init is in progress : condition flag is not yet set
        // init is done now with error or not (previously in progress) : condition flag is set
        // or appli in Initial, Stopped state : condition flag not set                    
        if (!init_req_done)
        {
            // first lock mutex for condition
            CDMW_MUTEX_GUARD (m_mutex);

            // reset the flag
            m_init_condition_flag = false;
            
            // if current status get previously in guarded sequence
            // is not in transcient initialising (in this case initialised cannot be reached)
            // it can be only in Initial, Stopped state
            if (appStatus != CdmwPlatformMngt::APPLICATION_INITIALISING)
            {
                // start initialise
                // exception is thrown in case of incompatible status
                initialise (startup_kind);
            }
            
            CDMW_INFO(m_logger,
                "Application_impl::initialise_and_wait " << name() << " : Wait starting !" );
            
            // initialisation is in progress
            // so wait the condition flag is set by broadcasting            
            while (!m_init_condition_flag)
            {
            
            CDMW_INFO(m_logger,
                "Application_impl::initialise_and_wait " << name() << " : Wait started !" );
           
                m_initialisation_condition.wait ();
            }
            
            m_init_condition_flag = false;
            
            CDMW_INFO(m_logger,
                "Application_impl::initialise_and_wait " << name() << " : Wait finishing !" );            
 
            // get the result
            req_status = m_initialisation_status;
        }
        
        } 
                     
        return req_status;
    }
    
    
    void Application_impl::resume ()
        throw (CdmwPlatformMngt::IncompatibleStatus, CORBA::SystemException)
    {
        LogMngr::logMessage (INF,
            MSG_ID_APPLICATION_RESUME, m_element_name.in ());        

        CDMW_SM_WRITER_LOCK_GUARD (*m_stateMachine);
        
        try
        {
            m_stateMachine->getState ()->resume (m_stateMachine);
        }
        catch (...)
        {
            // even in case of exception update record in datastore
            set_record_in_datastore ();
            
            // rethrow exception
            throw;
        }
        
        // update record in datastore
        set_record_in_datastore ();

    }
    
    
    CdmwPlatformMngt::LongRequestStatus Application_impl::
        resume_and_wait ()throw (CdmwPlatformMngt::IncompatibleStatus,
        CORBA::SystemException)
    {
        CdmwPlatformMngt::ApplicationStatus appStatus;
        CdmwPlatformMngt::LongRequestStatus req_status =
            CdmwPlatformMngt::LONG_REQUEST_NOT_COMPLETED;
        bool init_req_done = false;
        
        {
            // lock mutex for reading status
            CDMW_SM_READER_LOCK_GUARD (*m_stateMachine);
            
            // get the current status
            appStatus = m_stateMachine->getState ()->status ();
            
            if (appStatus == CdmwPlatformMngt::APPLICATION_INITIALISED ||
                appStatus == CdmwPlatformMngt::APPLICATION_RUN_REQUEST ||
                appStatus == CdmwPlatformMngt::APPLICATION_RUNNING ||
                appStatus == CdmwPlatformMngt::APPLICATION_PARTIAL_RUNNING)
            {
                req_status = CdmwPlatformMngt::LONG_REQUEST_COMPLETED;
                init_req_done = true;
            }
            
            else if (appStatus ==
                CdmwPlatformMngt::APPLICATION_INITIALISATION_INCOMPLETE)
            {
                req_status = CdmwPlatformMngt::LONG_REQUEST_NOT_COMPLETED;
                init_req_done = true;
            }
        }
        
        
        // if initialise request not done :
        // initialisation is in progress : condition flag is not yet set
        // initialisation is done now with error or not (previously in progress) : flag is set
        // or application in Initial, Suspended, Stopped state 
        // (flag not set)                    
        if (!init_req_done)
        {
            // first lock mutex for condition
            CDMW_MUTEX_GUARD (m_mutex);
            
            // if current status get previously in guarded sequence
            // is not in transcient initialising (in this case initialised cannot be reached)
            // it can be only in Initial, RunRequest, Stopping, Stopped state
            if (appStatus != CdmwPlatformMngt::APPLICATION_INITIALISING)
            {
                // start resume
                // exception is thrown in case of incompatible status
                resume ();
            }
            
            // initialisation is in progress
            // so wait the condition flag is set by broadcasting
            while (!m_init_condition_flag)
            {
                m_initialisation_condition.wait ();
            }
            
            m_init_condition_flag = false;
            
            // get the result
            req_status = m_initialisation_status;
        }
        
        return req_status;
    }
    
    
    void Application_impl::acknowledge ()
        throw (CdmwPlatformMngt::IncompatibleStatus, CORBA::SystemException)
    {
        LogMngr::logMessage (INF,
            MSG_ID_APPLICATION_ACKNOWLEDGE,
            m_element_name.in ());
        
        CDMW_MUTEX_GUARD (m_mutexMode);
        
        CdmwPlatformMngt::FunctioningMode mode = CdmwPlatformMngt::NORMAL_MODE;
        CORBA::String_var mode_info = CORBA::string_dup ("");
        
        set_mode (mode, mode_info.in ());
        
        // Notify the change of mode
        notifyModeChangeEvent (mode, mode_info.in ());
            
        // Create a command event to notify
        std::string appl_path = std::string (element_path());
    
        //LC: Attention: call an issuer mutex guard before calling this method.
        std::string issuer = Cdmw::PlatformMngt::ApplicationIssuerManager::Instance()->get_command_issuer(appl_path);
    
        notifyCommandEvent(ACKNOWLEDGE, issuer.c_str());
    }
    
    void Application_impl::run ()
        throw (CdmwPlatformMngt::IncompatibleStatus, CORBA::SystemException)
    {
        de_activate_preemption();

        if (! is_preemption_active())        
        {
    
            LogMngr::logMessage (INF, MSG_ID_APPLICATION_RUN, m_element_name.in ());
            
            CDMW_MUTEX_GUARD(*m_check_consistency_status_recursive_mutex);

            CDMW_SM_WRITER_LOCK_GUARD (*m_stateMachine);

            try
            {

                m_stateMachine->getState ()->run (m_stateMachine);
 
            }
            catch (...)
            {
                // even in case of exception update record in datastore
                set_record_in_datastore ();
            
                // rethrow exception
                throw;
            }
        
            // update record in datastore
            set_record_in_datastore ();
       }
    }
    
    void Application_impl::stop (CORBA::Boolean emergency)
        throw (CORBA::SystemException)
    {

        //MR
        //set_requested_restart_warmth ( get_default_restart_warmth () );                       
        LogMngr::logMessage (INF,
            MSG_ID_APPLICATION_STOP, m_element_name.in ());           
        
        activate_preemption();
        
        CDMW_SM_WRITER_LOCK_GUARD (*m_stateMachine);
        
        try
        {
            
            m_emergency = emergency;
           
            m_stateMachine->getState ()->stop (m_stateMachine, emergency);
        }
        catch (...)
        {

        
            // even in case of exception update record in datastore
            set_record_in_datastore ();
            
            // rethrow exception
            throw;
        }
        // update record in datastore
        set_record_in_datastore ();       
    }
    
    
    CdmwPlatformMngt::LongRequestStatus Application_impl::
        stop_and_wait (CORBA::Boolean emergency) throw (CORBA::SystemException)
    {
        CdmwPlatformMngt::ApplicationStatus appStatus;
        CdmwPlatformMngt::LongRequestStatus req_status =
            CdmwPlatformMngt::LONG_REQUEST_COMPLETED;
        bool stop_req_done = false;
        
        {

            // lock mutex for reading status
            CDMW_SM_READER_LOCK_GUARD (*m_stateMachine);
        
            // get the current status
            appStatus = m_stateMachine->getState ()->status ();
            
            if (appStatus == CdmwPlatformMngt::APPLICATION_INITIAL ||
                appStatus == CdmwPlatformMngt::APPLICATION_STOPPED)
            {
                req_status = CdmwPlatformMngt::LONG_REQUEST_COMPLETED;
                stop_req_done = true;
            }
        }
        
        
        // if stop request not done :
        // stopping is in progress : condition flag is not set
        // stop is done now with error or not (previously in progress) : flag is set
        // or application in Initialising, Initialised, Suspended, 
        // RunRequest, Running state 
        // (flag not set)                    
        if (!stop_req_done)
        {
            // first lock mutex for condition
            CDMW_MUTEX_GUARD (m_mutex);
        // reset the flag
        m_stop_condition_flag = false;
            
            // if current status get previously in guarded sequence
            // is not in transcient stopping (in this case stopped cannot be reached)
            // it can be only in Initialising, Initialised, Suspended, 
            // RunRequest, Running state
            if (appStatus != CdmwPlatformMngt::APPLICATION_STOPPING)
            {
                // stop application
                // exception is thrown in case of incompatible status

                stop (emergency);
            }
            
            // stopping is in progress
            // so wait the condition flag is set by broadcasting
            while (!m_stop_condition_flag)
            {
                m_stop_condition.wait ();
            }
            
            m_stop_condition_flag = false;
            
            // get the result
            req_status = m_stop_status;
        }
        
        return req_status;
    }
    
    void Application_impl::notifyCommandEvent(ApplicationCommand app_command, 
            const char* issuer,
            CORBA::Boolean emergency)
    {
        CdmwPlatformMngt::CdmwCommand_var event;       
        CdmwPlatformMngtBase::EventHeader header
        = EventHeaderFactory::createHeader (CdmwPlatformMngtBase::INF);

        if ((COLD_RESTART == app_command) || (HOT_RESTART == app_command))
        {
            //Setting up the startupKind parameters
            CdmwPlatformMngtBase::StateRetrieval state_retrieval = CdmwPlatformMngtBase::LAST_STATE;
            unsigned long startup_mode = 0;
            std::string specified_state_id = "";
            CdmwPlatformMngtBase::StartupKind_var startupKind = new CdmwPlatformMngtBase::StartupKind();
            startupKind->state_retrieval = state_retrieval;
            startupKind->startup_mode = startup_mode;
            startupKind->specified_state_id  = CORBA::string_dup( specified_state_id.c_str() );

            CdmwPlatformMngt::RestartWarmth restart_warmth;

            if (COLD_RESTART == app_command)
            {
                restart_warmth = CdmwPlatformMngt::COLD_RESTART;
            }
            else
            {               
                restart_warmth = CdmwPlatformMngt::HOT_RESTART;
            }

            // Create the event
            event = m_appRestartFactory->create(
                    header,
                    m_system_name.in (),
                    get_element_path(),
                    m_element_name.in (),
                    issuer,
                    startupKind.in(),
                    restart_warmth);
        }

        if (STOP == app_command)
        {
            event = m_appStopFactory->create (header,
                    m_system_name.in (),
                    get_element_path(),
                    m_element_name.in (),
                    issuer,
                    emergency);    

        }

        if (ACKNOWLEDGE == app_command)
        {
            event = m_appAcknowledgeFactory->create (header,
                    m_system_name.in (),
                    get_element_path(),
                    m_element_name.in (),
                    issuer);
        }

        //Log event
        CORBA::String_var eventStr = event->to_string ();
        LogMngr::logMessage (eventStr.in ()); 

        // Perform the command notification to supervision event notifier
        event->_add_ref();            
        (EventNotifierManager::Instance()->get_sup_notifier())->addEvent (event);

        // Perform the command notification to command event notifier
        event->_add_ref(); 
        (EventNotifierManager::Instance()->get_command_notifier())->add_event(event);
        event->_remove_ref();

        event._retn ();

    }
    
    CdmwPlatformMngt::LongRequestStatus Application_impl:: stop_application_wait
    (CORBA::Boolean emergency) throw (CORBA::SystemException)
    {

        // Create a command event to notify
        std::string appl_path = std::string (element_path());

        //LC: Attention: call an issuer mutex guard before calling this method.        
        std::string issuer = Cdmw::PlatformMngt::ApplicationIssuerManager::Instance()->get_command_issuer(appl_path);

       
        notifyCommandEvent(COLD_RESTART, issuer.c_str());
        return stop_and_wait (emergency);
    }

    void Application_impl::cold_restart (const CdmwPlatformMngtBase::
        StartupKind & startup_kind)
    throw (CdmwPlatformMngt::IncompatibleStatus, CORBA::SystemException)
    {

        // TODO: reload default properties and std attributes for application and all its processes
        // TODO: Set data store

        // Create a command event to notify       
        std::string appl_path = std::string (element_path());
    
        //LC: Attention: call an issuer mutex guard before calling this method.    
        std::string issuer = Cdmw::PlatformMngt::ApplicationIssuerManager::Instance()->get_command_issuer(appl_path);
       
        notifyCommandEvent(COLD_RESTART, issuer.c_str());

        set_restart_attempt (0);
        bool emergency = false;
        try {

            if (CdmwPlatformMngt::LONG_REQUEST_COMPLETED !=
                restart ( CdmwPlatformMngt::COLD_RESTART, startup_kind, emergency ) ){
                return;     
            }
        }
        catch (...) {
            throw;
        }
        return;       
    }

    
    
    void Application_impl::hot_restart (const CdmwPlatformMngtBase::
        StartupKind & startup_kind)
    throw (CdmwPlatformMngt::IncompatibleStatus, CORBA::SystemException)
    {      
    
    // Create a command event to notify        
    std::string appl_path = std::string (element_path());
    
    //LC: Attention: call an issuer mutex guard before calling this method.        
    std::string issuer = Cdmw::PlatformMngt::ApplicationIssuerManager::Instance()->get_command_issuer(appl_path);

    
        notifyCommandEvent(HOT_RESTART, issuer.c_str());

        bool emergency = false;
        try {

            if (CdmwPlatformMngt::LONG_REQUEST_COMPLETED !=
                restart ( CdmwPlatformMngt::HOT_RESTART, startup_kind, emergency ) ){

                return;     
            }

        }
        // catch (...) {
        catch (const CORBA::SystemException& ex) {

            throw;
        }
        return;
    }
    
    void 
    Application_impl::set_requested_restart_warmth(const CdmwPlatformMngt::RestartWarmth restartWarmth){
        this->ManagedElement_impl::set_requested_restart_warmth ( restartWarmth);
        ProcessServants::iterator processIt = m_process_servants.begin();
        while ( processIt != m_process_servants.end ()) {    
            Process_impl *p_process_impl = processIt->second;
            p_process_impl->set_requested_restart_warmth( restartWarmth );
            processIt ++;
        }
    }
    
    
    CdmwPlatformMngt::LongRequestStatus Application_impl::restart ( const CdmwPlatformMngt::RestartWarmth restartWarmth,
        const CdmwPlatformMngtBase::StartupKind & startup_kind,
        bool emergency)
        throw (CdmwPlatformMngt::IncompatibleStatus, CORBA::SystemException)
        
    {       
      CdmwPlatformMngt::LongRequestStatus req_status;
      set_requested_restart_warmth ( restartWarmth );
      
      try {
        
    req_status = stop_and_wait( emergency );
    if ( CdmwPlatformMngt::LONG_REQUEST_COMPLETED != req_status) {

    return req_status;
    }

    req_status = initialise_and_wait(startup_kind);
    //restart even if application initialisation is incomplete
    if ( (CdmwPlatformMngt::LONG_REQUEST_COMPLETED != req_status) && (CdmwPlatformMngt::LONG_REQUEST_NOT_COMPLETED != req_status) ) {   

    return req_status;
    }



    run(); 
        
        }
//      catch (...) {
        catch(const CORBA::SystemException & ex){   
    throw;
        }


      //MR #######################
      /* Reset application default_restart_warmth */
      //set_requested_restart_warmth ( get_default_restart_warmth () );  
      
      /* Set processes default restart warmth */
      /*processIt = m_process_servants.begin();
      while ( processIt != m_process_servants.end ()) {    
         Process_impl* p_process_impl = processIt->second;
     if (true == p_process_impl->created_dynamically() )
         {
             if (false == p_process_impl->get_force_manual_start())
             {
                //CGi the control is moved on the initalisation phase 
             p_process_impl->initialise(startup_kind);
             p_process_impl->run();
             }
         } 
         p_process_impl -> set_requested_restart_warmth( p_process_impl -> get_default_restart_warmth () );
     processIt ++;
     }*/
                              
     return CdmwPlatformMngt::LONG_REQUEST_COMPLETED;
    
    }
 
    
    CORBA::ULong Application_impl::get_number_of_entities ()throw (CORBA::
        SystemException)
    {
        return m_rwEntityContainer->get_number_of_entities ();
    }
    
    void Application_impl::get_all_entity_names (CORBA::ULong how_many,
        CdmwPlatformMngt::
        ElementNames_out
        entity_names,
        CdmwPlatformMngt::
        ElementNamesIterator_out
        rest) throw (CORBA::
        SystemException)
    {
        m_rwEntityContainer->get_all_entity_names (how_many, entity_names,
            rest);
    }
    
    CdmwPlatformMngt::Entity_ptr Application_impl::
        get_entity (const char *entity_name) throw (CdmwPlatformMngt::
                          EntityNotFound,
                          CORBA::SystemException)
    {
        return m_rwEntityContainer->get_entity (entity_name);
    }
    
    CORBA::Boolean Application_impl::get_entities (const CdmwPlatformMngt::
        ElementNames &
        entity_names,
        CdmwPlatformMngt::
        Entities_out nentities)
        throw (CORBA::SystemException)
    {
        return m_rwEntityContainer->get_entities (entity_names, nentities);
    }
    
    void Application_impl::get_all_entities (CORBA::ULong how_many,
        CdmwPlatformMngt::
        Entities_out nentities,
        CdmwPlatformMngt::
        EntitiesIterator_out rest)
        throw (CORBA::SystemException)
    {
        m_rwEntityContainer->get_all_entities (how_many, nentities, rest);
    }
    
    
    CdmwPlatformMngt::Entity_ptr Application_impl::
        add_entity (const CdmwPlatformMngt::
        EntityDef & entity_def) throw (CdmwPlatformMngt::
        EntityAlreadyExists,
        CORBA::SystemException)
    {
        std::string entityName = entity_def.entity_name.in ();
        
        LogMngr::logMessage (INF, MSG_ID_ADD_APPLICATION_ENTITY,
            entityName.c_str (), m_element_name.in ());
        
        CdmwPlatformMngt::Entity_ptr entity_ret = m_rwEntityContainer->add_entity (entity_def);
      
        notify_configuration_change("Entity Added");
        // To-Do LogMessage 
 
        return entity_ret;
        
    }
    
    void Application_impl::remove_entity (const char *entity_name)
        throw (CdmwPlatformMngt::EntityNotFound, CORBA::SystemException)
    {
        LogMngr::logMessage (INF, MSG_ID_REMOVE_APPLICATION_ENTITY,
            entity_name, m_element_name.in ());
        
        m_rwEntityContainer->remove_entity (entity_name);
        
        notify_configuration_change("Entity Removed");
    }
    
    Entity_impl *Application_impl::getEntity (const char *entity_name)
        throw (CORBA::SystemException)
    {
        return m_actualEntityContainer->getEntity (entity_name);
    }

    void Application_impl::synchro_entities_with_datastore()
    {
        CORBA::ULong how_many = get_number_of_entities (); 
        CdmwPlatformMngt::ElementNames_var entity_names;
        CdmwPlatformMngt::ElementNamesIterator_var rest;
        get_all_entity_names (how_many, entity_names, rest);
        for (CORBA::ULong count = 0U; count< how_many; count++)
        {
           //synchro the entity with its datastore 
           getEntity(entity_names[count])->synchro_with_datastore();
        }
    }  
    
    CdmwPlatformMngtService::ServiceProviderID *
        Application_impl::
        get_service_def (const char *service_name)
        throw (CdmwPlatformMngtService::ServiceNotFound, CORBA::SystemException)
    {
        return m_rwServiceDefContainer->get_service_def (service_name);
    }
    
    void Application_impl::add_service_def (const CdmwPlatformMngtService::
        ServiceDef & service_def)
        throw (CdmwPlatformMngtService::ServiceAlreadyExists,
        CORBA::SystemException)
    {
        m_rwServiceDefContainer->add_service_def (service_def);
        
        notify_configuration_change("Service Definition Added"); 
        // TODO: Trace to the log manager
    }
    
    void Application_impl::remove_service_def (const char *service_name)
        throw (CdmwPlatformMngtService::ServiceNotFound, CORBA::SystemException)
    {
        m_rwServiceDefContainer->remove_service_def (service_name);
        
        notify_configuration_change("Service Definition Removed"); 
        // TODO: Trace to the log manager
    }
    
    void Application_impl::remove_all_service_defs ()
        throw (CORBA::SystemException)
    {
        m_rwServiceDefContainer->remove_all_service_defs ();
        
        notify_configuration_change("All Service Definitions Removed");
        
        // TODO: Trace to the log manager
    }
    
    
    void Application_impl::is_alive () throw (CORBA::SystemException)
    {
        // TODO
        throw CORBA::NO_IMPLEMENT (OrbSupport::NO_IMPLEMENT,
            CORBA::COMPLETED_NO);
    }
   
  
    CdmwPlatformMngt::SupervisionObserver_ptr Application_impl::
        register_proxy_observer (const char *observer_name,
        const char *host_name,
        CdmwPlatformMngt::
        SupervisionObserver_ptr observer)
        throw (CORBA::SystemException)
    {
        // TODO
        throw CORBA::NO_IMPLEMENT (OrbSupport::NO_IMPLEMENT,
            CORBA::COMPLETED_NO);
    }


    
    const char *Application_impl::get_system_name ()
    {
        return m_system_name.in ();
    }


    void Application_impl::on_remove_process (const char *process_name,
                                              const char *host_name)
      throw (CdmwPlatformMngt::ProcessNotFound,
             CORBA::SystemException)
    {
        std::string process_key =
            ManagedElementKey::Get_process_rel_key (host_name, process_name);

        LogMngr::logMessage (INF,
                             MSG_ID_PROCESS_REMOVE,
                             process_key.c_str (), m_element_name.in ());

        try
        {

            CDMW_WRITER_RECURSIVE_MUTEX_GUARD (m_application_recursive_mutex);

            ProcessServants::iterator processIt =
                                      m_process_servants.find (process_key);
            if (processIt == m_process_servants.end ())
                throw CdmwPlatformMngt::ProcessNotFound();

            Process_impl *p_process_impl = processIt->second;

            // suppress process servant from map
            m_process_servants.erase (processIt);

            // remove from repository the process object
            m_element_repository->remove_element (p_process_impl->
                                                         get_element_path ());

            // deactivate process servant
            p_process_impl->deactivate ();

        }
        catch (const CdmwPlatformMngt::ElementPathInvalid&)
        {
            throw CORBA::INTERNAL (OrbSupport::INTERNAL, CORBA::COMPLETED_NO);
        }
        catch (const CdmwPlatformMngt::ManagedElementNotFound&)
        {
            throw CdmwPlatformMngt::ProcessNotFound ();
        }
    }


    CdmwPlatformMngt::Children* Application_impl::get_children()
        throw (CORBA::SystemException)
        {
            CdmwPlatformMngt::Children_var children = new CdmwPlatformMngt::Children;

            CORBA::ULong size_of_entities = get_number_of_entities();
            CORBA::ULong size_of_processes = get_number_of_processes();    
            CORBA::ULong size_of_ft_groups = get_number_of_ft_groups();
            CORBA::ULong size_of_lb_groups = get_number_of_lb_groups();     
        
            CORBA::ULong begin = 0;
        CORBA::ULong end = size_of_processes;
        
            children->length(size_of_processes + size_of_entities + size_of_ft_groups + size_of_lb_groups);
        
            CdmwPlatformMngt::Processes_var         processes;
            CdmwPlatformMngt::ProcessesIterator_var process_iterator;
            processes = get_processes (size_of_processes, process_iterator.out());
        
            for (CORBA::ULong count = begin; count < end; count++)
            {
                processes[count] -> _add_ref();
                children[count] = (processes[count])._retn();
            }
        
            begin = end;
        end = begin + size_of_ft_groups;
                        
        CdmwPlatformMngt::FTGroups_var          ftgroups;
        CdmwPlatformMngt::FTGroupsIterator_var  ftgroups_iterator ;

            get_all_ft_groups(size_of_ft_groups, ftgroups.out(), ftgroups_iterator.out() );

            for (CORBA::ULong count = begin; count < end; count++)
            {
                ftgroups[count - begin]-> _add_ref();
                children[count] = (ftgroups)[count - begin]._retn();
            }
        
            begin = end;
        end = begin + size_of_lb_groups;
        
        CdmwPlatformMngt::LBGroups_var          lbgroups;
        CdmwPlatformMngt::LBGroupsIterator_var  lbgroups_iterator ;
            get_all_lb_groups(size_of_lb_groups, lbgroups.out(), lbgroups_iterator.out() );

            for (CORBA::ULong count = begin; count < end; count++)
            {
                lbgroups[count - begin ] -> _add_ref();
                children[count] = (lbgroups[count - begin])._retn();
            }
        
            begin = end;
        end = begin + size_of_entities;
        
            CdmwPlatformMngt::Entities_var entities;
            CdmwPlatformMngt::EntitiesIterator_var entities_iterator;
            get_all_entities(size_of_entities, entities.out(), entities_iterator.out() );
            
            for (CORBA::ULong count = begin; count < end; count++)
            {
                entities[count - begin] -> _add_ref();
                children[count] = (entities[count - begin])._retn();
            }
            return children._retn ();
        }

bool Application_impl::created_dynamically()
{
   return m_created_dynamically;
}

void Application_impl::set_created_dynamically(bool created_dynamically)
{
   m_created_dynamically = created_dynamically;
   ProcessServants::iterator processIt;
   for (processIt = m_process_servants.begin (); processIt != m_process_servants.end (); processIt++)
   {
       Process_impl *process = processIt->second;
       process->set_created_dynamically(created_dynamically);
   }

   set_record_in_datastore();
}


     
void Application_impl::synchro_consistent_status()
{
    ProcessServants::iterator processes_it     =  m_process_servants.begin();
    ProcessServants::iterator processes_it_end =  m_process_servants.end();

    while (processes_it != processes_it_end)
    {
        string process_name = processes_it->second->name();
        string host_name = processes_it->second->get_process_host_name();
        CdmwPlatformMngt::ProcessStatus   process_status = processes_it->second->get_internal_status();
    
        // synchronize local map of processes
        process_status_change_event(process_name, host_name, process_status);
        ++processes_it;
    }
    check_consistency_of_state(); 
}

CdmwPlatformMngt::ElementType Application_impl::get_element_type()
        throw (CORBA::SystemException)
    {
            return CdmwPlatformMngt::APPLICATION_TYPE;
    }

void Application_impl::stop_application(CORBA::Boolean emergency)
    throw (CORBA::SystemException)
{
    
    // Create a command event to notify    
    std::string appl_path = std::string (element_path());
    
    //LC: Attention: call an issuer mutex guard before calling this method.    
    std::string issuer = Cdmw::PlatformMngt::ApplicationIssuerManager::Instance()->get_command_issuer(appl_path);
    
    notifyCommandEvent(STOP, issuer.c_str());
    
    stop(emergency);
}

void Application_impl::get_all_processes(CORBA::ULong how_many, CdmwPlatformMngt::Processes_out  nprocesses,
                        CdmwPlatformMngt::ProcessesIterator_out rest)
    throw (CORBA::SystemException)
{
    // TODO
    throw CORBA::NO_IMPLEMENT (OrbSupport::NO_IMPLEMENT, CORBA::COMPLETED_NO);
}


 void Application_impl::get_all_ft_groups(CORBA::ULong how_many, 
            CdmwPlatformMngt::FTGroups_out nftgroups, 
            CdmwPlatformMngt::FTGroupsIterator_out rest) 
              throw(CORBA::SystemException )
 {
   

     // create the requested sequence of ftgroups
     CdmwPlatformMngt::FTGroups_var requested_ftgroups = 
        new CdmwPlatformMngt::FTGroups;
     
     std::list <CdmwPlatformMngt::FTGroup_var> rest_of_ftgroups;

     // get length of list                                    
     size_t len = m_ftgroup_map.size();
     
     if (len > 0)
     {
             // set length of sequence
         //requested_ftgroups->length(len);

         // initialize list iterator
         FTGroupsMap::iterator it =
                      m_ftgroup_map.begin();


         for (CORBA::ULong i = 0; i < len; i++)
         {
            // get ftgroup from container list using iterator
            // and convert to FTGroup object reference
            CdmwPlatformMngt::FTGroup_var ftgroup =  (it->second)->_this();

            // store ftgroup in sequence
            if( i < how_many)
          {
          requested_ftgroups->length(i+1);
              requested_ftgroups[i] = ftgroup._retn();
          }
            else
              rest_of_ftgroups.push_back(ftgroup) ;

            // increment iterator
            it++;
         }
     }
     
     // create the FTGroupIterator with the rest of list
       CdmwPlatformMngt::FTGroupsIterator_var ftgroups_iter =
                          FTGroupsIterator_impl::create (rest_of_ftgroups); 

       nftgroups = requested_ftgroups._retn() ;
       //nftgroups = requested_ftgroups.out() ;
       rest = ftgroups_iter._retn(); 
    
 }

 
 void Application_impl::get_all_lb_groups(CORBA::ULong how_many, 
          CdmwPlatformMngt::LBGroups_out nlbgroups, 
             CdmwPlatformMngt::LBGroupsIterator_out rest) 
               throw(CORBA::SystemException )
  {


     // create the requested sequence of lbgroups
     CdmwPlatformMngt::LBGroups_var requested_lbgroups = 
        new CdmwPlatformMngt::LBGroups;
     
     std::list <CdmwPlatformMngt::LBGroup_var> rest_of_lbgroups;

     // get length of list                                    
     size_t len = m_lbgroup_map.size();
     
     if (len > 0)
     {

         // initialize list iterator
         LBGroupsMap::iterator it =
                      m_lbgroup_map.begin();


         for (CORBA::ULong i = 0; i < len; i++)
         {
            // get lbgroup from container list using iterator
            // and convert to LBGroup object reference
            CdmwPlatformMngt::LBGroup_var lbgroup =  (it->second)->_this();

            // store lbgroup in sequence
            if( i < how_many)
              {
              requested_lbgroups->length(i+1);
              requested_lbgroups[i] = lbgroup._retn();
              }
            else
              rest_of_lbgroups.push_back(lbgroup) ;



            // increment iterator
            it++;
         }
     }

     // create the LBGroupIterator with the rest of list
       CdmwPlatformMngt::LBGroupsIterator_var lbgroups_iter =
                          LBGroupsIterator_impl::create (rest_of_lbgroups);

       nlbgroups = requested_lbgroups._retn() ;
       //nlbgroups = requested_lbgroups.out() ;
       rest = lbgroups_iter._retn();



  }
                        


CORBA::ULong Application_impl::get_number_of_running_processes() 
{
     CORBA::ULong running_processes = 0;
     ProcessesStatusInfoMap::iterator it = m_processes_status_info_map.begin();
     ProcessesStatusInfoMap::iterator it_end = m_processes_status_info_map.end();
     while (it != it_end)
     {
        if ( (CdmwPlatformMngt::PROCESS_RUNNING == it->second) || (CdmwPlatformMngt::PROCESS_INITIALISED == it->second) ||
        (CdmwPlatformMngt::PROCESS_INITIALISING == it->second) || (CdmwPlatformMngt::PROCESS_RUN_REQUEST == it->second) )
            ++running_processes;
         ++it;
     }
    return running_processes;
}



CORBA::ULong Application_impl::get_number_of_processes_with_status(CdmwPlatformMngt::ProcessStatus process_status)
{
     CORBA::ULong processes_with_status = 0;
     ProcessesStatusInfoMap::iterator it = m_processes_status_info_map.begin();
     ProcessesStatusInfoMap::iterator it_end = m_processes_status_info_map.end();
     int i=1;
     while (it != it_end)
     {
         if (process_status == it->second)
            ++processes_with_status;
         ++it;
     i++;
     }
    return processes_with_status;
}


std::vector<Cdmw::PlatformMngt::Process_impl*>* Application_impl::get_processes_with_status(CdmwPlatformMngt::ProcessStatus process_status)
{
     std::vector<Cdmw::PlatformMngt::Process_impl*>* process_vector_with_status = 
              new std::vector<Cdmw::PlatformMngt::Process_impl*>;
    
     ProcessesStatusInfoMap::iterator it = m_processes_status_info_map.begin();
     ProcessesStatusInfoMap::iterator it_end = m_processes_status_info_map.end();

     while (it != it_end)
     {
         // if the status of the process is process_status 
         if (process_status == it->second)
         {  
             // take the process name
             std::string composed_process_name = it->first;

             std::string prefix = "/";
             std::string::size_type pos = (composed_process_name).find(prefix);                                                                      

             // if prefix is found
             if (pos != std::string::npos)
             {                         
                 std::string process_name = (composed_process_name).substr(0,pos);    
                 std::string host_name = (composed_process_name).substr(pos+1);
                 // get the process implementation 
                 Cdmw::PlatformMngt::Process_impl* process_impl = getProcess(process_name.c_str(), host_name.c_str()); 
                 // push back to the vector 
                 process_vector_with_status->push_back(process_impl);
             }  
         }   
         ++it;
     }
    return process_vector_with_status;
}


bool Application_impl::has_running_processes()
{
    ProcessServants::iterator processes_it = m_process_servants.begin();
    ProcessServants::iterator processes_it_end =  m_process_servants.end();

    bool status = false;
    Process_impl *process = 0;

    while (processes_it != processes_it_end)
    {
        //Process_impl *process = processIt->second;
        process = processes_it->second;

        CdmwPlatformMngt::ProcessStatus proc_status
                = process->get_internal_status ();

        // if found a RUNNING process is found, break and return false
        if ( CdmwPlatformMngt::PROCESS_RUNNING == proc_status)
        {
           status = true;
           break;
        }
        ++processes_it; 
    }
    return status;
}

Cdmw::PlatformMngt::Process_impl *Application_impl::get_process (const std::string & process_name)
                                      throw (CdmwPlatformMngt::ProcessNotFound, CORBA::SystemException)
{
    Process_impl* process = 0;
    ProcessServants::iterator process_it;
    for (process_it = m_process_servants.begin (); process_it != m_process_servants.end (); ++process_it)
    {
       process = process_it->second;
       if (process_name == process->name())
         break;    
    }
    if (0 == process)
        throw CdmwPlatformMngt::ProcessNotFound();
     
    return process;  
}

   CdmwPlatformMngt::FTGroup*
     Application_impl::add_ftgroup(const
     CdmwPlatformMngt::FTGroupDef& ftgroup_def) 
      throw (CORBA::SystemException,
       CdmwPlatformMngt::GroupAlreadyExists, CdmwPlatformMngt::DuplicateEntity,
       CosPropertyService::MultipleExceptions, CdmwPlatformMngt::IncompatibleStatus)
    {
     
       //instance the ftgroup ...
         std::string mystr(ftgroup_def.name) ;
          
           //check if the name is already filled with
           //a component with the same name

           FTGroupsMap::iterator ftgroupIt =
                    m_ftgroup_map.find (mystr);
             
           if (ftgroupIt != m_ftgroup_map.end ())
                    throw CdmwPlatformMngt::GroupAlreadyExists();
           
           std::string system_name = get_system_name() ;
           std::string application_name = name() ;
           
           std::string ftgroup_path = ManagedElementKey::Get_ftgroup_path(
                                           system_name,
                                           application_name,
                                           mystr);

           std::string ftgroup_key =  ManagedElementKey::Get_ftgroup_rel_key(
                                            application_name,
                                            mystr);
                           
          
           //try to instance ftgroup
           FTGroup_impl *ftgroup = 
                    new  FTGroup_impl(m_poa.in (), 
                          ftgroup_def, 
                            m_element_repository.in(),
                            ftgroup_key.c_str(),
                            ftgroup_path.c_str());
  
          CdmwPlatformMngt::FTGroup_var myftgroup = ftgroup->_this ();

           // add in repository the object
           m_element_repository->add_element (ftgroup_path.c_str (),
                    CdmwPlatformMngt::FT_GROUP_TYPE,
                    myftgroup.in() ); 
           m_ftgroup_map[ mystr ] = ftgroup;
           ftgroup->set_record_in_datastore();
           ftgroup->end_creation(ftgroup_def);
           return myftgroup._retn() ;
   }

    CdmwPlatformMngt::LBGroup*
     Application_impl::add_lbgroup(
       const CdmwPlatformMngt::LBGroupDef& lbgroup_def)
      throw (CORBA::SystemException,
       CdmwPlatformMngt::GroupAlreadyExists, CdmwPlatformMngt::DuplicateEntity,
       CosPropertyService::MultipleExceptions, 
       CdmwPlatformMngt::IncompatibleStatus)
    {



       //instance the lbgroup ...
       
         std::string mystr(lbgroup_def.name) ;
          
           //check if the name is already filled with
           //a component with the same name

           LBGroupsMap::iterator lbgroupIt =
                    m_lbgroup_map.find (mystr);
             
           if (lbgroupIt != m_lbgroup_map.end ())
                    throw CdmwPlatformMngt::GroupAlreadyExists();
           
           std::string system_name = get_system_name() ;
           std::string application_name = name() ;
           
           std::string lbgroup_path = ManagedElementKey::Get_lbgroup_path(
                                           system_name,
                                           application_name,
                                           mystr);

           std::string lbgroup_key =  ManagedElementKey::Get_lbgroup_rel_key(
                                            application_name,
                                            mystr);
                           
          
           //try to instance lbgroup
           LBGroup_impl *lbgroup = 
                    new  LBGroup_impl(m_poa.in (), 
                          lbgroup_def, 
                            m_element_repository.in(),
                            lbgroup_key.c_str(),
                            lbgroup_path.c_str());
           
           CdmwPlatformMngt::LBGroup_var mylbgroup = lbgroup->_this ();

           // add in repository the object
           m_element_repository->add_element (lbgroup_path.c_str (),
                    CdmwPlatformMngt::LB_GROUP_TYPE,
                    mylbgroup.in() ); 

          
           m_lbgroup_map[ mystr ] = lbgroup;
           
           lbgroup->set_record_in_datastore();
           lbgroup->end_creation(lbgroup_def);
           return mylbgroup._retn() ;
    }
  
    Cdmw::PlatformMngt::FTGroup_impl* Application_impl::getFTGroup(const char* ftgroup_name)
       throw (CORBA::SystemException, CdmwPlatformMngt::GroupNotFound)
       {
           std::string mystr(ftgroup_name) ;

           //try printing the map
           FTGroupsMap::const_iterator it  = m_ftgroup_map.begin();
           FTGroupsMap::const_iterator end = m_ftgroup_map.end();

           FTGroupsMap::iterator ftgroupIt =
               m_ftgroup_map.find (mystr);

           //check if ftgroup does not exists
           if (ftgroupIt == m_ftgroup_map.end ())
               throw CdmwPlatformMngt::GroupNotFound();

           return  (ftgroupIt->second);

       }


    Cdmw::PlatformMngt::LBGroup_impl* Application_impl::getLBGroup(
        const char* lbgroup_name)
       throw (CORBA::SystemException, CdmwPlatformMngt::GroupNotFound)
       {
           std::string mystr(lbgroup_name) ;

           //try printing the map
           LBGroupsMap::const_iterator it  = m_lbgroup_map.begin();
           LBGroupsMap::const_iterator end = m_lbgroup_map.end();

           LBGroupsMap::iterator lbgroupIt =
               m_lbgroup_map.find (mystr);

           //check if lbgroup does not exists
           if (lbgroupIt == m_lbgroup_map.end ())
               throw CdmwPlatformMngt::GroupNotFound();

           return  (lbgroupIt->second);
        
       }


    CdmwPlatformMngt::FTGroup* Application_impl::get_ft_group(const char* ftgroup_name)
        throw (CORBA::SystemException, CdmwPlatformMngt::GroupNotFound)
    {     
        CdmwPlatformMngt::FTGroup_var ftgroup = getFTGroup(ftgroup_name)->_this()  ;
        return ftgroup._retn() ;

    }


    CdmwPlatformMngt::LBGroup* Application_impl::get_lb_group(const char *lbgroup_name)
        throw (CORBA::SystemException, CdmwPlatformMngt::GroupNotFound)
    {
         CdmwPlatformMngt::LBGroup_var lbgroup =  getLBGroup(lbgroup_name)->_this()  ;
         return lbgroup._retn() ;
    }
    

    CORBA::ULong Application_impl::get_number_of_ft_groups()
         throw (CORBA::SystemException)
    {
               CORBA::ULong len = m_ftgroup_map.size();
               return len;

    }


    CORBA::ULong Application_impl::get_number_of_lb_groups()
         throw (CORBA::SystemException)
    {
               CORBA::ULong len = m_lbgroup_map.size();
               return len;
    }


bool Application_impl::is_deployment_plan_related_to_process(const std::string & deployment_plan, const std::string & process_name){

   bool ret_value=false; 

   CDMW_READER_RECURSIVE_MUTEX_GUARD (m_application_recursive_mutex);
 
   DeploymentInfoMap::iterator deploy_it = m_deployment_info_multimap.find(deployment_plan);
   DeploymentInfoMap::iterator last_element = m_deployment_info_multimap.upper_bound(deployment_plan);
   
   if (deploy_it == m_deployment_info_multimap.end()){
    ret_value = false;
   }
   else{

       while ( deploy_it != last_element && ret_value==false)
       {
    
           std::string process_name_tmp = deploy_it->second; 
      
           if (process_name_tmp == process_name)
           {
               ret_value = true;
           }
      
           deploy_it++; 
   
       } 
   } 
   return ret_value;

}

void Application_impl::process_initialised (const std::string & process_name, const std::string& host_name) throw (CORBA::SystemException)
{

   CDMW_WRITER_RECURSIVE_MUTEX_GUARD (m_application_recursive_mutex);

   std::string message;
   message = process_name + " has changed status";
   LogMngr::logMessage (process_name.c_str());
 
   DeploymentInfoMap::iterator deploy_it = m_deployment_info_multimap.begin();
        
   //find all deployment plan associated to this process
   while ( deploy_it != m_deployment_info_multimap.end())
    {
      std::string process_name_tmp = deploy_it->second; 
      if ((process_name_tmp == process_name))
    {
      //std::string current_process_name = deploy_it->second;
      std::string deployment_plan = deploy_it->first; 

      // if the current deployment plan is associated to process_name  
      message = " Checking all processes with the same deployment plan:";
      message += deployment_plan;
      LogMngr::logMessage (message.c_str());

      //counts the number of processes with the same deployment plan
      int processes_with_same_deployment_plan = m_deployment_info_multimap.count(deployment_plan);
      DeploymentInfoMap::iterator deployment_info_map_iterator_found = m_deployment_info_multimap.find(deployment_plan);

      std::set<std::string> host_names_to_deploy;
  
      int  counter_procs = 0;
      // check if the processes with the same deployment plan has the status PROCESS_INITIALISED
      for (int i=0; i< processes_with_same_deployment_plan; ++i) 
        {
          std::string process_name_to_check = deployment_info_map_iterator_found->second;
          message = "checking:";
          message += process_name_to_check; 
          message += " to start deployment operations";
          LogMngr::logMessage (message.c_str()); 
      
      std::string composed_process_name = process_name_to_check + "/" + host_name;
          ProcessesStatusInfoMap::iterator it_process_status = m_processes_status_info_map.find(composed_process_name);
          CdmwPlatformMngt::ProcessStatus process_status_to_check = it_process_status->second;
          if (CdmwPlatformMngt::PROCESS_INITIALISED == process_status_to_check || CdmwPlatformMngt::PROCESS_RUN_REQUEST == process_status_to_check || CdmwPlatformMngt::PROCESS_RUNNING == process_status_to_check)
          { 
              message = process_name_to_check;
              message += ": process ready to deploy operations";
              LogMngr::logMessage (message.c_str());
              counter_procs++;
          host_names_to_deploy.insert(host_name);
           }
           else 
           {
               message = process_name_to_check;
               message += ": process not ready to deploy operations";
               LogMngr::logMessage (message.c_str());
           }
          ++deployment_info_map_iterator_found; 
        }

      set<std::string>::iterator deployment_plans_it_found  = m_deployment_plans.find(deployment_plan);
      set<std::string>::iterator deployment_plans_it_end  = m_deployment_plans.find(deployment_plan);
      // the set of host names sharing the same deployment_plans 

      // if the deployment plan is not in the set of the application deployment plans     
      // and all process are initialised
      if ((counter_procs == processes_with_same_deployment_plan)&&(deployment_plans_it_found == deployment_plans_it_end))
      {
          message = "All processes with the same deployment plan are INITIALISED  => start deployment operations ";
          LogMngr::logMessage (message.c_str());
          m_deployment_plans.insert(deployment_plan);
          try
          { 
              DeploymentManager::get_instance()->notify(host_names_to_deploy, deployment_plan);
          }
          catch(const CdmwDnC::StartError& start_error)
          {
            CDMW_WARN(m_logger, "  Stopping Application emergency = true: " << name());
             stop(true);
           }
          }
          else
          { 
              message = "Not All processes with same DP INITIALISED => start deployment operations ";
          }
  
       }
      deploy_it++; 
    }        
  
}

    void Application_impl::initialise_process_components(DeploymentInfo deployment_info)
    {
    
      ProcessServants::iterator process_servants_it  = m_process_servants.begin();
         ProcessServants::iterator process_servants_it_end = m_process_servants.end();
      
         while(process_servants_it!=process_servants_it_end)
         {
              Process_impl* process_impl = process_servants_it->second;
              
              std::string host_name = string (process_impl->get_process_host_name());
              if ( ( host_name == deployment_info.host_name ) &&
                     CdmwPlatformMngt::COMPONENT_SERVER == process_impl->type() )
              {
                  process_impl -> initialise_components ( (deployment_info.deployment_plan).c_str() );
              }
              
              ++process_servants_it;
          } 
      
    }

    void Application_impl::error_event_on_process_components(DeploymentInfo deployment_info)
    {
      ProcessServants::iterator process_servants_it  = m_process_servants.begin();
         ProcessServants::iterator process_servants_it_end = m_process_servants.end();

         while(process_servants_it!=process_servants_it_end)
         {
              Process_impl* process_impl = process_servants_it->second;

              std::vector<std::string>* vect_deployment_plans = process_impl->get_deployment_plans();
              bool deployment_plans_present = true;
              if (vect_deployment_plans==0)
                   deployment_plans_present = false;
              //check if the process belongs to deployment_info.host_name and has a deployment_info.deployment_plan
              if (deployment_plans_present)
              { 
                  std::vector<std::string>::iterator it = 
                      find( vect_deployment_plans->begin(), vect_deployment_plans->end(), deployment_info.deployment_plan);
                  std::string host_name = string (process_impl->get_process_host_name());
                  if ( ( host_name == deployment_info.host_name ) &&
                         CdmwPlatformMngt::COMPONENT_SERVER == process_impl->type() && (it != vect_deployment_plans->end()) )
                  {
                      CdmwPlatformMngtBase::TimeStamp timeStamp = Cdmw::PlatformMngt::EventHeaderFactory::createTimeStamp ();
                      m_processObserver->notify_error( CORBA::string_dup(name()), CORBA::string_dup((process_impl->name())),
                      CORBA::string_dup(host_name.c_str()), timeStamp,
                      CdmwPlatformMngtBase::PFMMNG, CORBA::string_dup("Error occurred during deployment operations"));
                  }
                  
          vect_deployment_plans->clear();
          delete vect_deployment_plans;
              } 
              ++process_servants_it;
         }
     
    }     

     ApplicationStateMachine* Application_impl::get_stateMachine()
{
 return m_stateMachine;
}
 
 
    bool Application_impl::get_processes_initialised() throw()
    {
         
         bool all_proc_initialised = true;
         ProcessServants::iterator process_servants_it  = m_process_servants.begin();
         ProcessServants::iterator process_servants_it_end = m_process_servants.end();

         while(process_servants_it!=process_servants_it_end)
         {  
               
              Process_impl* process_impl = process_servants_it->second;
                
              std::vector<std::string>* vect_deployment_plans = process_impl->get_deployment_plans();
              
              if (vect_deployment_plans!=0){
                   
                    CORBA::String_var status_info;

                    CdmwPlatformMngt::ProcessStatus proc_status = process_impl->get_status (status_info.out ());
         
                    if (CdmwPlatformMngt::PROCESS_INITIALISED != proc_status)
                    {
                        all_proc_initialised = false; 
                        break;
                    }
            }
              ++process_servants_it;
         }
         return all_proc_initialised;
    }

    bool Application_impl::any_process_to_deploy() throw()
    {
         bool process_found = false;

         ProcessServants::iterator process_servants_it  = m_process_servants.begin();
         ProcessServants::iterator process_servants_it_end = m_process_servants.end();

         while(process_servants_it!=process_servants_it_end)
         {

              Process_impl* process_impl = process_servants_it->second;

              std::vector<std::string>* vect_deployment_plans = process_impl->get_deployment_plans();
              if (vect_deployment_plans!=0)
              {
                  process_found = true;
                  
          vect_deployment_plans->clear();
          delete vect_deployment_plans;
          
                  break;
              } 
              ++process_servants_it;
         }
     
         return process_found;
    }

    bool Application_impl::init_sequencer_not_instantiated_or_completed()
    {
        bool ret_value=true;
    
        if (m_initSequencer!=0){
        ret_value=m_initSequencer->is_completed();
        }
    
    return ret_value;
    }
              
     bool  Application_impl::is_preemption_active ()
     {
         bool result = false;
         
         CDMW_MUTEX_GUARD (m_preemption_flag_mutex);
         
         result = m_preemption_flag;    
         
         return result;
     }
     
     bool  Application_impl::activate_preemption()
     {
         bool result = false;
         
         CDMW_MUTEX_GUARD (m_preemption_flag_mutex);

         bool old_flag_value = m_preemption_flag;
         
         try 
         {
             CORBA::String_var sys_status_info;
             CdmwPlatformMngt::System_var system = CdmwPlatformMngt::System::_narrow(get_parent());
                                                                                                                                                                                                                                                                                                                         
                                                                                                                                                                                                                                                                                                                         
             CdmwPlatformMngt::SystemStatus system_status = system->get_status(sys_status_info.out());
             
             CORBA::String_var status_info;
 
             CdmwPlatformMngt::ApplicationStatus status  = get_status (status_info.out());
             
             if (system_status == CdmwPlatformMngt::SYSTEM_STARTING
                 || system_status == CdmwPlatformMngt::SYSTEM_INITIAL
                 || status == CdmwPlatformMngt::APPLICATION_INITIAL
                 || status == CdmwPlatformMngt::APPLICATION_INITIALISING
                 || status == CdmwPlatformMngt::APPLICATION_INITIALISATION_SUSPENDED
                 || status == CdmwPlatformMngt::APPLICATION_INITIALISATION_INCOMPLETE)
             {
                 m_preemption_flag = true;
                 result = true;   
             }                         
         
         }
         catch (...)
         {
             old_flag_value = m_preemption_flag;
             CDMW_ERROR(m_logger, "  Can't activate preemption of " << name() << " application " << std::endl);
         }
    
         return result;
     }
     
     bool  Application_impl::de_activate_preemption()
     {
          bool result = false;
 
          CDMW_MUTEX_GUARD (m_preemption_flag_mutex);
          
          try 
          {
             CORBA::String_var sysStatusInfo;
                                                            
             CdmwPlatformMngt::System_var system = CdmwPlatformMngt::System::_narrow(get_parent());
                                                                                                                                                                                                                                                                                                                         
             CdmwPlatformMngt::SystemStatus system_status = system->get_status(sysStatusInfo.out());
                                      
             if (m_preemption_flag)
             {
                 if (system_status == CdmwPlatformMngt::SYSTEM_STARTING 
                       || system_status == CdmwPlatformMngt::SYSTEM_INITIAL)
                 {                  
                 } 
                 else
                 {
                    m_preemption_flag = false;
                    result = true;   
                 }
            }       
          
          }
          catch (...)
          {

              CDMW_ERROR(m_logger, "  Can't deactivate preemption of " << name() << " application " << std::endl);
              m_preemption_flag = false;
              result = true;
          }         
                                                     
          return result;
     }

    void Application_impl::init_preemption() 
    {
         CDMW_MUTEX_GUARD (m_preemption_flag_mutex);
         
         m_preemption_flag = false;         
    }

    }                // End namespace PlatformMngt
}               // End namespace Cdmw


