/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#include "Foundation/orbsupport/OrbSupport.hpp"
#include "SystemMngt/platformlibrary/GraphUtility.hpp"
#include "SystemMngt/platformlibrary/LogMngr.hpp"

#include "platformapplicationlibrary/ComponentInit.hpp"
#include "platformapplicationlibrary/ManagedProcess_impl.hpp"
#include "platformapplicationlibrary/CcmComponent_impl.hpp"

#include <sstream>

namespace Cdmw
{
namespace PlatformMngt
{

// ----------------------------------------------------------------------
// InitComponentSuspensionCallback class.
// ----------------------------------------------------------------------
InitComponentSuspensionCallback::InitComponentSuspensionCallback (
                                       ManagedProcess_impl* process,
                                       const std::string& requested_deployment_plan)
    : Callback()
{
    m_process = process;
    m_requested_deployment_plan = requested_deployment_plan;
}


InitComponentSuspensionCallback::~InitComponentSuspensionCallback ()
{
    
}


void InitComponentSuspensionCallback::execute()
  throw()
{
    m_process->suspension_event(m_requested_deployment_plan);
}

// ----------------------------------------------------------------------
// InitComponentCompletionCallback class.
// ----------------------------------------------------------------------
InitComponentCompletionCallback::InitComponentCompletionCallback (
                                       ManagedProcess_impl* process,
                                       const std::string& requested_deployment_plan)
    : Callback()
{
    m_process = process;
    m_requested_deployment_plan = requested_deployment_plan;
}


InitComponentCompletionCallback::~InitComponentCompletionCallback ()
{
}


void InitComponentCompletionCallback::execute()
  throw()
{
    m_process->completion_event(m_requested_deployment_plan);
}



// ----------------------------------------------------------------------
// InitComponentTask class.
// ----------------------------------------------------------------------
InitComponentTask::InitComponentTask (const std::string& taskId,
                                      Sequencer* sequencer,
                                      TaskObserver* observer,
                                      size_t predecessorCount,
                                      //CdmwPlatformMngt::CcmComponent_ptr component,
                                      CcmComponent_impl* component,
                                      const CdmwPlatformMngtBase::StartupKind& startup_kind,
                                      const std::string& requested_deployment_plan)
             throw (OutOfMemoryException)
             : SynchronisableTask (taskId, sequencer, observer, predecessorCount)
{
    try
    {
        //m_component = CdmwPlatformMngt::CcmComponent::_duplicate(component);
        m_component = component;
        m_startupKind = new CdmwPlatformMngtBase::StartupKind (startup_kind);
        m_requested_deployment_plan = requested_deployment_plan; 
    }
    catch (const std::bad_alloc&)
    {
        CDMW_THROW (OutOfMemoryException);
    }
}


InitComponentTask::~InitComponentTask ()
  throw()
{
}


bool InitComponentTask::execute()
  throw()
{
    bool ret = false;
    CORBA::String_var comp_name = m_component->name();
    
    try
    {
        std::string msg (comp_name.in());
    msg.append(": InitComponentTask::execute ");
    LogMngr::logMessage( msg.c_str());
        m_component->initialise (m_startupKind.in(), m_requested_deployment_plan);
        ret = true;
    }
    catch (const CdmwPlatformMngt::IncompatibleStatus &)
    {
        LogMngr::logMessage("InitComponentTask::execute CdmwPlatformMngt::IncompatibleStatus");
    }
    catch (...)
    {
        LogMngr::logMessage("InitComponentTask::execute() unexpected exception");
    }
    
    
    return ret;
}



// ----------------------------------------------------------------------
// InitComponentTaskFactory class.
// ----------------------------------------------------------------------
InitComponentTaskFactory::InitComponentTaskFactory (
                                ManagedProcess_impl* process,        
                                const CdmwPlatformMngtBase::StartupKind& startup_kind,
                                const SynchronisableTaskDefMap& taskDefMap,
                                const TaskIdList& rootTasks,
                                const std::string& requested_deployment_plan)
             throw (OutOfMemoryException)
             : SynchronisableTaskFactory (taskDefMap, rootTasks)
{
    try
    {
        m_process = process;
        m_startupKind = new CdmwPlatformMngtBase::StartupKind (startup_kind);
        m_requested_deployment_plan  = requested_deployment_plan;
    }
    catch (const std::bad_alloc&)
    {
        CDMW_THROW (OutOfMemoryException);
    }
}


InitComponentTaskFactory::~InitComponentTaskFactory ()
{
}


SynchronisableTaskFactory* InitComponentTaskFactory::createFactory (
                               ManagedProcess_impl* process,
                               const CdmwPlatformMngtBase::StartupKind& startup_kind,
                               const CdmwPlatformMngt::GraphElements& graphElements,
                               const std::string& requested_deployment_plan)
             throw (OutOfMemoryException)
{
    try
    {
        // create TaskDef map and root task list from graph
        SynchronisableTaskDefMap taskDefMap;
        TaskIdList rootTasks;

        GraphUtility::getTaskDefinitions (graphElements,           
                                          true,
                                          true,
                                          taskDefMap,
                                          rootTasks);
/*
        std::auto_ptr <InitComponentTaskFactory> syncTaskFactory (
                                 new InitComponentTaskFactory (
                                            process, startup_kind, 
                                            taskDefMap, rootTasks));
        
        // return the factory reference
        return syncTaskFactory.release();
*/
        InitComponentTaskFactory* syncTaskFactory = new  InitComponentTaskFactory (process, startup_kind, taskDefMap,
    rootTasks, requested_deployment_plan);

    return syncTaskFactory;
    }
    catch (const std::bad_alloc&)
    {
        CDMW_THROW (OutOfMemoryException);
    }
}



SynchronisableTask* InitComponentTaskFactory::createTask (
                        const std::string& taskId,
                        Sequencer* sequencer,
                        TaskObserver* taskObserver)
             throw (InvalidTaskIdException, 
                    OutOfMemoryException)    
{
    
    CORBA::String_var comp_name = CORBA::string_dup ("");
    
    try
    {
        // get synchronisable task definition
        SynchronisableTaskDef syncTaskDef = getTaskDef (taskId);
        
        // get component
        CcmComponent_impl* component =
              m_process->get_component_impl (taskId.c_str());
              
        // get component name
        comp_name = component->name();
    
    
        // create synchronisable task
        std::auto_ptr <InitComponentTask> 
        syncTask(new InitComponentTask (
                                          taskId,
                                          sequencer,
                                          taskObserver,
                                          syncTaskDef.m_nbPredecessors,
                                          //component.in(),
                                          component,
                                          m_startupKind.in(),
                                          m_requested_deployment_plan)
                                        );
         
        // return synchronisable task
        return syncTask.release();
        //}
        //return 0;
    }
    catch (const CdmwPlatformMngt::ComponentNotFound&)
    {
        CDMW_THROW1(InvalidTaskIdException, taskId);
    }
    catch (const std::bad_alloc&)
    {
        CDMW_THROW (OutOfMemoryException);
    }
    catch (...)
    {
        LogMngr::logMessage("InitComponentTaskFactory::createTask");
                            
        CDMW_THROW1( InvalidTaskIdException, taskId );
    }   
}

} // End namespace PlatformMngt
} // End namespace Cdmw
