/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

#include "platformapplicationlibrary/HostProcessInitSequencer.hpp"
#include "platformapplicationlibrary/ManagedProcess_impl.hpp"
#include "SystemMngt/platformapplicationlibrary/ApplicationLogMessageId.hpp"
#include <cstdlib>
#include <iostream>
#include <vector>
#include <algorithm>
#include <map>

using namespace std;

namespace Cdmw
{
namespace PlatformMngt
{


class CommandKind
{
private:
     virtual void execute_command(Process_impl* proc_impl) = 0;

public:
     CommandKind(){};

     CdmwPlatformMngtBase::StartupKind_var get_startup_kind();

    virtual void execute(Process_impl* proc_impl, CdmwPlatformMngt::RestartWarmth restart_warmth,
            KindOfStart kind_of_start)
    throw (CdmwPlatformMngt::IncompatibleStatus, CORBA::SystemException)
     {
        if ((DEFAULT == kind_of_start) ||
            ((ADDED == kind_of_start) && (CdmwPlatformMngt::HOT_RESTART == restart_warmth)) ||
            ((DEFAULT_REMOVED == kind_of_start) && (CdmwPlatformMngt::COLD_RESTART == restart_warmth)))
     {
         execute_command(proc_impl);
     }
        //else
        //{
     //std::cout<<"*** OUTSIDE IF OF kind_of_start in execute, kind_of_start="<<kind_of_start<<std::endl;
        //}
     };
};

CdmwPlatformMngtBase::StartupKind_var CommandKind::get_startup_kind() {
    CdmwPlatformMngtBase::StartupKind_var startupKind = new CdmwPlatformMngtBase::StartupKind();
    startupKind->state_retrieval = CdmwPlatformMngtBase::LAST_STATE;
    startupKind->startup_mode = 0;
    startupKind->specified_state_id = CORBA::string_dup("");

    return startupKind;
}

class RunCommandKind: public CommandKind
{
public:
    virtual void execute_command(Process_impl* proc_impl)
    throw (CdmwPlatformMngt::IncompatibleStatus, CORBA::SystemException)
    {
        if (0 != dynamic_cast<ManagedProcess_impl*> (proc_impl))
        {
            proc_impl->run();
        }
    }
};


class InitCommandKind: public CommandKind
{
public:
    virtual void execute_command(Process_impl* proc_impl)
        throw (CdmwPlatformMngt::IncompatibleStatus, CORBA::SystemException)
    {

        
        CdmwPlatformMngt::RequestStatus status;   
        const char* p_appliName = proc_impl->get_application()->get_element_name();
        const char* p_procKey = proc_impl->get_process_key();
        string msg("InitCommandKind, ");
        msg += p_procKey;
        msg += " ";
        msg += p_appliName;
        msg += " :";
    
        try
        {    

            //CGi Added Control on manual starting of the Process
            CORBA::Any_var any_var = new CORBA::Any();
            CORBA::Boolean force_manual_start;
            proc_impl->get_attribute(CdmwPlatformMngt::FORCE_MANUAL_START, any_var.out(), CdmwPlatformMngt::PRE_DEFINED);
            any_var >>= CORBA::Any::to_boolean (force_manual_start);
            
            if (!force_manual_start) 
            {
                //TODO add support for steps
                msg += MSG_ID_APP_PROC_INIT;
                CDMW_INFO(Cdmw::Logging::Logger::Get_Logger(CdmwPlatformMngtBase::PFMMNG), msg.c_str());
                                                        
                status = proc_impl->initialise( get_startup_kind() );
                
                //Check if this process is UNMANAGED with an initialisation_timeout != 0 
                if( CdmwPlatformMngt::REQUEST_TIMEOUT == status && CdmwPlatformMngt::UNMANAGED_PROCESS == proc_impl->type() ) 
                {
                    {                   
                        CDMW_MUTEX_GUARD(*proc_impl->m_init_mutex);                 
                        while ( proc_impl->m_lastRequestStatus != CdmwPlatformMngt::REQUEST_SUCCEED )
                        {
                            try
                            {
                                msg += "Waiting for initialisation timeout...";
                                CDMW_INFO(Cdmw::Logging::Logger::Get_Logger(CdmwPlatformMngtBase::PFMMNG), msg.c_str());
                                // Waiting for the RUNNING state after the initialisation_timeout
                                proc_impl->m_init_condition->wait();  
                            } 
                            catch (const InternalErrorException& e) 
                            {
                                // Shouldn't happen
                                CDMW_ASSERT(false);
                            }                   
                        }                   
    
                    }
                    proc_impl->m_timer->stop();
                    proc_impl->m_timer->destroy_timeout_observer();
                    delete proc_impl->m_init_condition;
                    delete proc_impl->m_init_mutex;
                    proc_impl->m_init_condition=0;              
                    proc_impl->m_init_mutex=0;
                    delete proc_impl->m_timer;
                    proc_impl->m_timer=0;
                }
    
            }
            else 
            {
                msg += MSG_ID_APP_PROC_INIT_NOT_EXECUTED;
                CDMW_INFO(Cdmw::Logging::Logger::Get_Logger(CdmwPlatformMngtBase::PFMMNG), msg.c_str());
            }
        }
        catch (const CdmwPlatformMngt::IncompatibleStatus &)
        {
                msg += MSG_ID_APP_PROC_INIT_STATUS_ERR;
                CDMW_ERROR(Cdmw::Logging::Logger::Get_Logger(CdmwPlatformMngtBase::PFMMNG), msg.c_str());
        }
        catch (const CdmwPlatformMngt::InvalidStep &)
        {
            msg += MSG_ID_APP_PROC_INIT_STEP_ERR;
            CDMW_ERROR(Cdmw::Logging::Logger::Get_Logger(CdmwPlatformMngtBase::PFMMNG), msg.c_str());
        }
        catch (const CdmwPlatformMngt::IncompatibleType &)
        {
            msg += MSG_ID_APP_PROC_INIT_TYPE_ERR;
            CDMW_ERROR(Cdmw::Logging::Logger::Get_Logger(CdmwPlatformMngtBase::PFMMNG), msg.c_str()); 
        }
        catch (const CORBA::SystemException& e)
        {
            std::ostringstream exceptionInfo;
            exceptionInfo << e;
            msg += MSG_ID_APP_PROC_INIT_SYSTEM_ERR;
            msg += exceptionInfo.str().c_str();
            CDMW_ERROR(Cdmw::Logging::Logger::Get_Logger(CdmwPlatformMngtBase::PFMMNG), msg.c_str());                       
        }
        catch(const CdmwPlatformMngt::AttributeNotFound&) 
        {
            msg += "Property not found : CdmwPlatformMngt::FORCE_MANUAL_START";
            CDMW_FATAL(Cdmw::Logging::Logger::Get_Logger(CdmwPlatformMngtBase::PFMMNG), msg.c_str());   
        }
        catch (...)
        {
            msg += MSG_ID_APP_UNEXPECTED_EXCEPTION;
            CDMW_ERROR(Cdmw::Logging::Logger::Get_Logger(CdmwPlatformMngtBase::PFMMNG), msg.c_str());   
        }
  
    }
};


void HostProcessInitSequencer::set_init_entry(const std::string& host_name, ProcessHostInfo proc_info)
{
    set_entry(host_name, proc_info, 0);
}


void HostProcessInitSequencer::set_run_entry(const std::string& host_name, ProcessHostInfo proc_info)
{
    set_entry(host_name, proc_info, 1);
}


HostProcessInitSequencer* HostProcessInitSequencer::get_instance()
{
    static HostProcessInitSequencer host_proc_host_seq;
    return &host_proc_host_seq;
}

HostProcessInitSequencer::HostProcessInitSequencer()
{
    m_stop_sequences_map = new MapStopSequences();
    m_stop_sequences_map->clear();
}

void HostProcessInitSequencer::remove_init_entry(const std::string& host_name, ProcessHostInfo proc_info)
{
    remove_entry(host_name, proc_info, 0);
}

void HostProcessInitSequencer::remove_run_entry(const std::string& host_name, ProcessHostInfo proc_info)
{
    remove_entry(host_name, proc_info, 1);
}

void HostProcessInitSequencer::set_entry(const std::string& host_name, ProcessHostInfo proc_info, int m_select)
{       
    {
    CDMW_WRITER_RECURSIVE_MUTEX_GUARD(m_process_map_recursive_mutex);

    MapIterator map_it = m_host_process_map[m_select].find(host_name);
    //check if an host_name entry is present,
    // if not create a vector
    if (map_it == m_host_process_map[m_select].end())
    {
        Cdmw::PlatformMngt::ProcessInfos* vect_host_proc = new ProcessInfos(1);
        (*vect_host_proc)[0] = proc_info;
        m_host_process_map[m_select].insert(MapHostProcess::value_type(host_name, vect_host_proc));
    }
    else //update the vector in map
    {
        // check if the entry is not present in the vector
        int size_vector = map_it->second->size();
        bool found = false;
        for (int i=0; i< size_vector; i++)
        {
            if ( (*map_it->second)[i].process_name == proc_info.process_name)
            {
                found = true;
                //substitute the entry (to update ProcessHostInfo.kind_of_start)
                (*map_it->second)[i] = proc_info;
                break;
            }
        }
        if (!found)
        {
            map_it->second -> push_back(proc_info);
            CdmwPlatformMngt::Host_var process_host = proc_info.process_impl->get_host();

            MapStopSequences::iterator stop_it = m_stop_sequences_map->find(host_name);

            if (stop_it != m_stop_sequences_map->end())
            {
                (stop_it->second).push_back(proc_info.process_name);
            }

        }
     }

     bool adding_tuple = true;

     if (m_stop_sequences_map != 0)
     {
         if (!m_stop_sequences_map->empty())
         {
             MapStopSequences::iterator stop_it = m_stop_sequences_map->find(host_name);

             if (stop_it != m_stop_sequences_map->end())
             {
                 adding_tuple = false;
             }
         }
     }
     else
     {
         m_stop_sequences_map = new MapStopSequences();
     }

     if (adding_tuple)
     {
         std::list <std::string> stop_list;
         stop_list.push_back(proc_info.process_name);
         (*m_stop_sequences_map)[host_name] = stop_list;

     }
     else
     {
         MapStopSequences::iterator stop_it = m_stop_sequences_map->find(host_name);

         std::list<std::string>::iterator process_stop_it = find((stop_it->second).begin(), (stop_it->second).end(), proc_info.process_name);

          if (process_stop_it == (stop_it->second).end())
          {
              (stop_it->second).push_back(proc_info.process_name);
          }

     }

    } //Mutex guard
}

void HostProcessInitSequencer::remove_entry(const std::string& host_name, ProcessHostInfo proc_info, int m_select)
{
    CDMW_WRITER_RECURSIVE_MUTEX_GUARD(m_process_map_recursive_mutex);

    MapIterator map_it = m_host_process_map[m_select].find(host_name);
    //check if an host_name entry is present,
    // if not create a vector
    if (map_it != m_host_process_map[m_select].end())
    {
        // check if the entry is not present in the vector
        int size_vector = map_it->second->size();
        for (int i=0; i< size_vector; i++)
        {
            if (( (*map_it->second)[i].process_name == proc_info.process_name) &&
                  ( ADDED == (*map_it->second)[i].kind_of_start))
            {
                for (int j=i;j<(size_vector-1);j++)
                {
                    (*map_it->second)[j] = (*map_it->second)[j+1];
                }
                map_it->second->resize(size_vector - 1);
            }

            if (( (*map_it->second)[i].process_name == proc_info.process_name) &&
                  ( DEFAULT == (*map_it->second)[i].kind_of_start))
            {
                (*map_it->second)[i] = proc_info;
            }

            if (map_it->second->size() == 0)
            {
                m_host_process_map[m_select].erase(host_name);
            }
        }
    }

       if(m_stop_sequences_map != 0)
       {
         if (! m_stop_sequences_map->empty())
         {
              MapStopSequences::iterator stop_it = m_stop_sequences_map->find(host_name);
              if (stop_it != m_stop_sequences_map->end())
              {

                  std::list<std::string> found_sequence = stop_it->second;
                  std::list<std::string>::iterator stop_proc_it = find((stop_it->second).begin(),
                         (stop_it->second).end(), proc_info.process_name);

                  if (stop_proc_it != (stop_it->second).end())
                  {
                      (stop_it->second).remove(*stop_proc_it);

                      if ((stop_it->second).empty())
                      {
                          m_stop_sequences_map->erase(stop_it);
                      }
                  }

              }
         }
       }
}

void HostProcessInitSequencer::initialise_host_processes(CdmwPlatformMngt::RestartWarmth restart_warmth,
        const std::string& host_name)
throw (CdmwPlatformMngt::IncompatibleStatus, CORBA::SystemException)
{
    InitCommandKind* init_cmd = new InitCommandKind;
    perform_command(init_cmd, restart_warmth, host_name,0);
};

void HostProcessInitSequencer::run_host_processes(CdmwPlatformMngt::RestartWarmth restart_warmth,
        const std::string& host_name)
throw (CdmwPlatformMngt::IncompatibleStatus, CORBA::SystemException)
{
    RunCommandKind* run_cmd = new RunCommandKind;
    perform_command(run_cmd, restart_warmth, host_name,1);
};

void HostProcessInitSequencer::perform_command(CommandKind* command_kind,
        CdmwPlatformMngt::RestartWarmth restart_warmth,const std::string& host_name, int m_select)
throw (CdmwPlatformMngt::IncompatibleStatus, CORBA::SystemException)
{
    CDMW_READER_RECURSIVE_MUTEX_GUARD(m_process_map_recursive_mutex);

    MapIterator map_it = m_host_process_map[m_select].find(host_name);
    
    //check if an host_name entry is present
    if (map_it != m_host_process_map[m_select].end()){
        int size_vector = map_it->second->size();
        for (int i=0; i< size_vector; i++)
        {
            try
            {
                (*map_it->second)[i].process_impl->set_requested_restart_warmth(restart_warmth);
                command_kind->execute((*map_it->second)[i].process_impl, restart_warmth, (*map_it->second)[i].kind_of_start);
            }
            catch(const CdmwPlatformMngt::IncompatibleStatus& ex)
            {
                CDMW_INFO(Cdmw::Logging::Logger::Get_Logger(CdmwPlatformMngtBase::PFMMNG), "IncompatibleStatus Exception");
            }
        }
    }
}

void HostProcessInitSequencer::print()
{
 CDMW_READER_RECURSIVE_MUTEX_GUARD(m_process_map_recursive_mutex);

 for (int select = 0; select<= 1; select++)
 {
// MapIterator map_it = m_host_process_map[select].begin();
    //check if an host_name entry is present,
    // if not create a vector
 std::cerr << "MAP N:" << select + 1<< std::endl;
        for (MapIterator map_it = m_host_process_map[select].begin();
             map_it != m_host_process_map[select].end(); map_it++)
  {
    std::cerr << " key =" << map_it->first << std::endl;
   int size_vector = map_it->second->size();
   for (int i=0; i< size_vector; i++)
        {
             std::cerr << "      |____ value process name "<<
             (*map_it->second)[i].process_name<< "  kind_of_start " <<(*map_it->second)[i].kind_of_start << endl;
            }
        }
    }
}

void HostProcessInitSequencer::stop_host_processes(const std::string& host_name)
{
    MapIterator map_it = 0;

    MapStopSequences::iterator stop_it = 0;

    {

    CDMW_READER_RECURSIVE_MUTEX_GUARD(m_process_map_recursive_mutex);

    CDMW_INFO(Cdmw::Logging::Logger::Get_Logger(CdmwPlatformMngtBase::PFMMNG), "in stop_host_processes " << host_name);

    map_it = m_host_process_map[0].find(host_name);

    stop_it = m_stop_sequences_map->find(host_name);

    if (map_it != m_host_process_map[0].end() 
        && stop_it != m_stop_sequences_map->end())
    {

        std::list<std::string>::iterator stop_proc_it = (stop_it->second).begin();
/*
        PLEASE: keep for debugging purposes only !
        std::list<std::string>::iterator debug_it = (stop_it->second).begin();
        for (; debug_it != (stop_it->second).end(); debug_it++)
        {
                 CDMW_INFO(Cdmw::Logging::Logger::Get_Logger(CdmwPlatformMngtBase::PFMMNG), "READING AGAIN STOP MAP: element " << 
                   *debug_it << " with key " << host_name);
        }
*/
        int size_vector = 0;

        bool process_ptr_found = false;

        for (; stop_proc_it != (stop_it->second).end(); stop_proc_it++)
        {

             process_ptr_found = false;

             size_vector = map_it->second->size();

             for (int i=size_vector-1; i>= 0 && (!process_ptr_found); i--)
             {

                if ((*map_it->second)[i].process_name == *stop_proc_it)
                {

                   process_ptr_found = true;

                   try 
                   {
                       CDMW_INFO(Cdmw::Logging::Logger::Get_Logger(CdmwPlatformMngtBase::PFMMNG), "HostProcessInitSequencer::stop_host_processes: Stopping Process " << (*map_it->second)[i].process_impl->name());
                           CdmwPlatformMngt::LongRequestStatus status = (*map_it->second)[i].process_impl->stop_and_wait(false); 

                       if( CdmwPlatformMngt::LONG_REQUEST_COMPLETED != status ) {
                              CDMW_WARN(Cdmw::Logging::Logger::Get_Logger(CdmwPlatformMngtBase::PFMMNG), 
                              "Cannot stop Process " << (*map_it->second)[i].process_impl->name() << " on host " << 
                              (*map_it->second)[i].process_impl->get_process_host_name() << " ! ");
                       }

                   }
                   catch (...)
                   {
                       CDMW_ERROR(Cdmw::Logging::Logger::Get_Logger(CdmwPlatformMngtBase::PFMMNG), 
                           "Catched Unexpected Exception in HostProcessInitSequencer::stop_host_processes ");

                       throw;
                   }

                }
            }

        }

        int stop_sequence_size = (stop_it->second).size();

        if (size_vector > stop_sequence_size)
        {

             for (int i=size_vector-1; i>= 0; i--)
             {

            std::list<std::string>::iterator stop_proc_it = find ((stop_it->second).begin(), (stop_it->second).end(),
                (*map_it->second)[i].process_name);

                if (stop_proc_it == (stop_it->second).end())
                {

                   try 
                   {
                       CDMW_INFO(Cdmw::Logging::Logger::Get_Logger(CdmwPlatformMngtBase::PFMMNG), "HostProcessInitSequencer::stop_host_processes: Stopping Process " << (*map_it->second)[i].process_impl->name());
                           CdmwPlatformMngt::LongRequestStatus status = (*map_it->second)[i].process_impl->stop_and_wait(false); 

                       if( CdmwPlatformMngt::LONG_REQUEST_COMPLETED != status ) {
                              CDMW_WARN(Cdmw::Logging::Logger::Get_Logger(CdmwPlatformMngtBase::PFMMNG), 
                              "Cannot stop Process " << (*map_it->second)[i].process_impl->name() << " on host " << 
                               (*map_it->second)[i].process_impl->get_process_host_name() << " ! ");
                       } 

                    }
                    catch (...)
                    {
                       CDMW_ERROR(Cdmw::Logging::Logger::Get_Logger(CdmwPlatformMngtBase::PFMMNG), 
                           "Catched Unexpected Exception in HostProcessInitSequencer::stop_host_processes ");

                       throw;
                    }

                }
              }
        }

    }
    else
    {
        CDMW_INFO(Cdmw::Logging::Logger::Get_Logger(CdmwPlatformMngtBase::PFMMNG), "in stop_host_processes " << host_name);

        map_it = m_host_process_map[0].find(host_name);
        int size_vector = map_it->second->size();
    
        for (int i=size_vector-1; i>= 0; i--)
        {
              try 
              {
                  CDMW_INFO(Cdmw::Logging::Logger::Get_Logger(CdmwPlatformMngtBase::PFMMNG), "HostProcessInitSequencer::stop_host_processes: Stopping Process " << (*map_it->second)[i].process_impl->name());
                  CdmwPlatformMngt::LongRequestStatus status = (*map_it->second)[i].process_impl->stop_and_wait(false); 

                  if( CdmwPlatformMngt::LONG_REQUEST_COMPLETED != status ) {

                     CDMW_WARN(Cdmw::Logging::Logger::Get_Logger(CdmwPlatformMngtBase::PFMMNG), 
                        "Cannot stop Process " << (*map_it->second)[i].process_impl->name() << " on host " << 
                         (*map_it->second)[i].process_impl->get_process_host_name() << " ! ");
                  } 

             }
              catch (...)
              {
                  CDMW_ERROR(Cdmw::Logging::Logger::Get_Logger(CdmwPlatformMngtBase::PFMMNG), 
                      "Catched Unexpected Exception in HostProcessInitSequencer::stop_host_processes ");

                  throw;
              }

       }

    }
   }

}

void HostProcessInitSequencer::set_sequencer_map (MapHostProcess process_map [])
{
     CDMW_WRITER_RECURSIVE_MUTEX_GUARD(m_process_map_recursive_mutex);

     m_host_process_map [0] = process_map [0];
     m_host_process_map [1] = process_map [1];
/*
     PLEASE: keep for debugging purposes only !

     for (MapIterator map_it = m_host_process_map[0].begin(); 
           map_it != m_host_process_map[0].end(); map_it++)
     {

         CDMW_INFO(Cdmw::Logging::Logger::Get_Logger(CdmwPlatformMngtBase::PFMMNG), "READING MAP: key " << map_it->first);

         for ( unsigned int index = 0; index < (*map_it->second).size(); index++)
         {

         CDMW_INFO(Cdmw::Logging::Logger::Get_Logger(CdmwPlatformMngtBase::PFMMNG), "READING MAP: element " << 
            (*map_it->second)[index].process_impl->name() << " with key " << map_it->first);

         }

     }
*/
}

void HostProcessInitSequencer::set_stop_sequences_map (MapStopSequences* stop_sequences_map)
{

     CDMW_WRITER_RECURSIVE_MUTEX_GUARD(m_process_map_recursive_mutex);

     if (stop_sequences_map != 0)
     {
         m_stop_sequences_map = stop_sequences_map;
     }
/*
      PLEASE: keep for debugging purposes only !

     if (!stop_sequences_map->empty())
     {
         MapStopSequences::iterator map_it;

         for (map_it = m_stop_sequences_map->begin(); 
                  map_it != m_stop_sequences_map->end(); map_it++)
         {

             CDMW_INFO(Cdmw::Logging::Logger::Get_Logger(CdmwPlatformMngtBase::PFMMNG), "READING STOP MAP: key " << map_it->first);

             std::list<std::string>::iterator names_it;

             for (names_it = (map_it->second).begin(); names_it != (map_it->second).end(); names_it++)
             {
                 CDMW_INFO(Cdmw::Logging::Logger::Get_Logger(CdmwPlatformMngtBase::PFMMNG), "READING STOP MAP: element " << 
                   *names_it << " with key " << map_it->first);
             }
        }
      }
*/
}

} // End namespace PlatformMngt
} // End namespace Cdmw

