
/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

#include "SystemMngt/platformelementlibrary/EntityContainer_impl.hpp"
#include "SystemMngt/platformelementlibrary/RWEntityContainer_impl.hpp"
#include "SystemMngt/platformelementlibrary/EntityContainer.hpp"

#include "Foundation/common/Exception.hpp"
#include "Foundation/common/Assert.hpp"
#include "Foundation/orbsupport/StrategyList.hpp"
#include "platformapplicationlibrary/LBGroup_impl.hpp"
#include "SystemMngt/platforminterface/PlatformInterface.hpp"
#include "SystemMngt/platforminterface/ServiceNames.hpp"
#include "Foundation/orbsupport/ExceptionMinorCodes.hpp"
#include "SystemMngt/platformlibrary/Configuration.hpp"
#include "SystemMngt/platformlibrary/EventHeaderFactory.hpp"
#include "SystemMngt/platformlibrary/SupervisionEventNotifier.hpp"

#include "idllib/CdmwPlatformMngtSupervisionObserver.skel.hpp"
#include "idllib/CdmwPlatformMngtPredefinedProperties.stub.hpp"
#include "Repository/repositoryinterface/RepositoryInterface.hpp"
#include <iostream>


namespace Cdmw
{
  namespace PlatformMngt
  {

    LBGroup_impl::LBGroup_impl( PortableServer::POA_ptr poa, 
                const CdmwPlatformMngt::LBGroupDef& lbgroup_def,
            CdmwPlatformMngt::RWElementRepository_ptr element_repository,
            const char *lbgroup_key,
                const char *lbgroup_path) : RedundancyGroup_impl( poa, element_repository)
        {

       set_element_path (lbgroup_path, lbgroup_def.name );
       
       //Set the group type to the redundancy group 
       set_group_type(CdmwPlatformMngt::LB_GROUP_TYPE);
    
       //Set the minimun number of replicas to the redundancy group 
       set_minimum_nbr_of_replicas(lbgroup_def.minimum_nbr_replicas);

       //Set the style to the redundancy group 
       set_balancing_style(lbgroup_def.style);
       
       set_record_in_datastore();


        try
        {
        //instance entity container ...

        std::auto_ptr <PlatformMngt::EntityContainer> actualEntityContainer (
          new PlatformMngt::EntityContainer(poa, m_element_repository.in(),
           m_element_path.in()));
         
        m_rwEntityContainerServant = 
                 new RWEntityContainer_impl ( 
                    poa, actualEntityContainer.get(),
                      m_element_repository.in());


        m_rwEntityContainer
                    = dynamic_cast <RWEntityContainer_impl*>
                       (m_rwEntityContainerServant.in());
               
               
        //set the m_rwEntityContainer to the father ...
        set_rw_entity_container( m_rwEntityContainer ) ;

        m_actualEntityContainer = actualEntityContainer.get();
        actualEntityContainer.release();

        //Add all Properties 
        unsigned int propCount = lbgroup_def.properties.length();
        for (unsigned int propIndex=0 ; propIndex < propCount ; propIndex++)
        {
            const CdmwPlatformMngt::PropertyDef& prop_def =
                (lbgroup_def.properties)[propIndex];
         
            std::string
               propertyName(((lbgroup_def.properties)[propIndex]).property_name);
            CosPropertyService::PropertyModeType propertyMode =
                CosPropertyService::PropertyModeType(((lbgroup_def.properties)[propIndex]).mode_type);                
            try
            {
                define_property_with_mode(propertyName.c_str() , 
                     prop_def.initial_value,
                     propertyMode);
            }
            catch (...)
            {
                throw   CosPropertyService::MultipleExceptions();
            }
         
        }            


      }
      catch (const std::bad_alloc&)
        {
             throw CORBA::NO_MEMORY(OrbSupport::NO_MEMORY, CORBA::COMPLETED_NO);
        }

    } 

    void LBGroup_impl::end_creation(const CdmwPlatformMngt::LBGroupDef& lbgroup_def)
    throw (CdmwPlatformMngt::DuplicateEntity,
        OutOfMemoryException,
        CORBA::SystemException)
    {
        const CdmwPlatformMngt::EntityDefs& entities = lbgroup_def.entities;
        for (unsigned int entityIndex=0; entityIndex < entities.length();
              entityIndex++)
        {
            // get entity definition (EntityDef is a structure, a reference on
            // the
            // structure is returned by the [] operator)
            const CdmwPlatformMngt::EntityDef& entity_def =
              entities[entityIndex];

            // store values of the entityDef
            //(*m_default_entitydefs)[entityIndex] = entity_def;

            try
            {
                add_entity (entity_def);
            }
            catch(const CdmwPlatformMngt::EntityAlreadyExists&)
            {
                // in this case the servant will be deleted by application
                throw CdmwPlatformMngt::DuplicateEntity
                    (entity_def.entity_name.in());
            }
        }
    }
    
    CdmwPlatformMngt::LBGroupSnapshot* LBGroup_impl::get_snapshot()  throw (CORBA::SystemException)
      {

      CdmwPlatformMngt::LBGroupSnapshot_var snapshot =
                new CdmwPlatformMngt::LBGroupSnapshot;      

      // Gets all the entities
      CdmwPlatformMngt::Entities_var entities;
      CdmwPlatformMngt::EntitiesIterator_var entitiesIt;
      CORBA::ULong nbr_of_entities;

      CdmwPlatformMngt::EntitiesSnapshot_var entitiesSnapshot
                    = new CdmwPlatformMngt::EntitiesSnapshot;
         
      nbr_of_entities = get_number_of_entities ();
      get_all_entities (nbr_of_entities,
                        entities.out (), entitiesIt.out ());
         
      entitiesSnapshot->length (nbr_of_entities);
      for (CORBA::ULong i = 0; i < nbr_of_entities; i++)
      {
         (*entitiesSnapshot)[i] = *(entities[i]->get_snapshot ());
      }


      //Gets all the properties
      CosPropertyService::PropertiesIterator_var propertiesIt;
      CORBA::ULong nbr_of_properties;
      CdmwPlatformMngt::PropertiesSnapshot_var propertiesSnapshot =
          new CdmwPlatformMngt::PropertiesSnapshot;
      
      nbr_of_properties = get_number_of_properties ();
      CosPropertyService::PropertyNames_var prop_names;
      CosPropertyService::PropertyNamesIterator_var prop_namesIt;
      get_all_property_names (nbr_of_properties, prop_names.out (),
          prop_namesIt.out ());
      propertiesSnapshot->length (nbr_of_properties-2);
      CORBA::ULong snap_prop=0;
      for (CORBA::ULong i = 0; i < nbr_of_properties; i++)
      {
          if ((strcmp (prop_names[i], "Cdmw.TechStatus")==0) ||
                  (strcmp (prop_names[i], "Cdmw.TechStatusInfo")==0))
          {
              // do not add the TECH_STATUS and TECH_STATUS_INFO properties to the snapshot 
          }
          else
          {
              CdmwPlatformMngt::PropertySnapshot_var propertySnapshot =
                  new CdmwPlatformMngt::PropertySnapshot ();
              propertySnapshot->property_name =
                  CORBA::string_dup (prop_names[i]);

              CORBA::Any prop_value;
              prop_value = *(get_property_value (prop_names[i]));
              CORBA::TypeCode_var tc;
              tc = prop_value.type ();
              switch (tc->kind ())
              {
              default:
                  break;
              case CORBA::tk_string:
                  propertySnapshot->property_type =
                      CORBA::string_dup ("String");

                  break;
              case CORBA::tk_short:
                  propertySnapshot->property_type = CORBA::string_dup ("Short");
                  break;
              case CORBA::tk_float:
                  propertySnapshot->property_type = CORBA::string_dup ("Float");
                  break;
              };

              CosPropertyService::PropertyModeType mode =
                  get_property_mode (prop_names[i]);
              propertySnapshot->property_value = prop_value;

              if (mode == CosPropertyService::normal)
                  propertySnapshot->property_mode = CORBA::string_dup ("Read/Write");
                  else
                      if (mode == CosPropertyService::read_only)
                          propertySnapshot->property_mode = CORBA::string_dup ("Read/Only");
                          else
                              propertySnapshot->property_mode = CORBA::string_dup ("Pre-defined");

                              propertySnapshot->property_value = prop_value;
                              (*propertiesSnapshot)[snap_prop] = *(propertySnapshot);
                              snap_prop++;
          }
      }


      snapshot->element_path = element_path() ;
      snapshot->lb_group_name = name() ;
      CORBA::String_var status_info;
      snapshot->lb_group_status = get_status(status_info) ;
      snapshot->lb_group_entities = entitiesSnapshot.in() ;
      snapshot->lb_group_properties = propertiesSnapshot.in ();
      
      CdmwPlatformMngt::ElementPaths_var component_element_paths = new CdmwPlatformMngt::ElementPaths();
      CORBA::ULong nbr_of_components = get_number_of_components();
      CdmwPlatformMngt::CcmComponents_var components; 
      CdmwPlatformMngt::ComponentsIterator_var componentsIt; 
      
      get_all_components (nbr_of_components, components.out (), componentsIt.out ());
      component_element_paths->length (nbr_of_components);
      for (CORBA::ULong i = 0; i < nbr_of_components; i++)
      {
        (*component_element_paths)[i] = components[i]->element_path ();
      }
      
      snapshot->component_element_paths = component_element_paths.in(); 
      
      return snapshot._retn();
      } 


    CdmwPlatformMngt::BalancingStyle LBGroup_impl::get_style()  throw (CORBA::SystemException)
    {
        return get_balancing_style();
    } 


    void LBGroup_impl::set_style(CdmwPlatformMngt::BalancingStyle)  throw (CORBA::SystemException)
      {
      throw CORBA::NO_IMPLEMENT (OrbSupport::NO_IMPLEMENT, CORBA::COMPLETED_NO);
      } 
    
    CdmwPlatformMngt::ElementType LBGroup_impl::get_element_type()
      throw (CORBA::SystemException)
        {
            return CdmwPlatformMngt::LB_GROUP_TYPE;
        }

    Entity_impl *LBGroup_impl::getEntity (const char *entity_name)
        throw (CORBA::SystemException)
    {
        return m_actualEntityContainer->getEntity (entity_name);
    }

    void LBGroup_impl::synchro_entities_with_datastore()
    {
        CORBA::ULong how_many = get_number_of_entities (); 
        CdmwPlatformMngt::ElementNames_var entity_names;
        CdmwPlatformMngt::ElementNamesIterator_var rest;
        get_all_entity_names (how_many, entity_names, rest);
        for (CORBA::ULong count = 0U; count< how_many; count++)
        {
           //synchro the entity with its datastore 
           getEntity(entity_names[count])->synchro_with_datastore();
        }
    }  
  }             // End namespace PlatformMngt
}               // End namespace Cdmw
