/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#include "platformapplicationlibrary/ManagedProcess_impl.hpp"
#include "Foundation/orbsupport/ExceptionMinorCodes.hpp"
#include "Foundation/logging/Logger.hpp"
#include "Foundation/osthreads/MutexGuard.hpp"
#include "SystemMngt/platformlibrary/LogMngr.hpp"
#include "SystemMngt/platformlibrary/SupervisionEventNotifier.hpp"
#include "SystemMngt/platformlibrary/RWServiceDefContainer_impl.hpp"
#include "SystemMngt/platformelementlibrary/RWEntityContainer_impl.hpp"
#include "SystemMngt/platformvaluetypes/ProcessStatusChange_impl.hpp"
#include "SystemMngt/platformapplicationlibrary/Application_impl.hpp"
#include "SystemMngt/platformapplicationlibrary/ApplicationLogMessageId.hpp"
#include "platformapplicationlibrary/ServiceBroker_impl.hpp"
#include "platformapplicationlibrary/HostProcessInitSequencer.hpp"
#include "SystemMngt/idllib/CdmwPlatformMngtPredefinedProperties.stub.hpp"
#include "SystemMngt/idllib/CdmwPlatformMngtSystem.stub.hpp"

#include "platformapplicationlibrary/CcmComponent_impl.hpp"
#include "platformapplicationlibrary/ComponentInit.hpp"
#include "SystemMngt/platformlibrary/Sequencer.hpp"
#include "platformapplicationlibrary/DeploymentManager.hpp"
#include "SystemMngt/platformlibrary/EventHeaderFactory.hpp"
#include "SystemMngt/tools/ProcessIssuerManager.hpp"

#include "platformapplicationlibrary/CommandCallback.hpp"

#include <set>

namespace Cdmw
{
    namespace PlatformMngt
    {
        
        ManagedProcessInitial*
            ManagedProcess_impl::M_initial = NULL;
        ManagedProcessInitialising*
            ManagedProcess_impl::M_initialising = NULL;
        ManagedProcessStepPerformed*
            ManagedProcess_impl::M_stepPerformed = NULL;
        ManagedProcessInitialised*
            ManagedProcess_impl::M_initialised = NULL;
        ManagedProcessRunRequest*
            ManagedProcess_impl::M_runRequest = NULL;
        ManagedProcessRunning*
            ManagedProcess_impl::M_running = NULL;
        ManagedProcessStopping*
            ManagedProcess_impl::M_stopping = NULL;
        ManagedProcessStopped*
            ManagedProcess_impl::M_stopped = NULL;
        ManagedProcessEnded*
            ManagedProcess_impl::M_ended = NULL;
        ManagedProcessFailedError*
            ManagedProcess_impl::M_failedError = NULL;
        ManagedProcessFailedMonitoringFailure*
            ManagedProcess_impl::M_failedMonitoringFailure = NULL;
        ManagedProcessFailedDeath*
            ManagedProcess_impl::M_failedDeath = NULL;
        ManagedProcessFailedDiscrepancy*
            ManagedProcess_impl::M_failedDiscrepancy = NULL;
        ManagedProcessFailedTimeout*
            ManagedProcess_impl::M_failedTimeout = NULL;
        ManagedProcessFailedInvalid*
            ManagedProcess_impl::M_failedInvalid = NULL;
        

        static const std::string process_status_strings[] =
        {
                "Initial","Initialising","StepPerformed","Initialised",
                "RunRequest","Running","Stopping","Stopped","Ended","FailedError",
                "FailedTimeout","FailedDiscrepancy","FailedInvalid",
                "FailedMonitoringFailure","FailedDeath"
        };
        static const CdmwPlatformMngt::ProcessStatus Status[]=
        { 
                CdmwPlatformMngt::PROCESS_INITIAL,
                CdmwPlatformMngt::PROCESS_INITIALISING,
                CdmwPlatformMngt::PROCESS_STEP_PERFORMED,
                CdmwPlatformMngt::PROCESS_INITIALISED,
                CdmwPlatformMngt::PROCESS_RUN_REQUEST,
                CdmwPlatformMngt::PROCESS_RUNNING,
                CdmwPlatformMngt::PROCESS_STOPPING,
                CdmwPlatformMngt::PROCESS_STOPPED,
                CdmwPlatformMngt::PROCESS_ENDED,
                CdmwPlatformMngt::PROCESS_FAILED_ERROR,
                CdmwPlatformMngt::PROCESS_FAILED_TIMEOUT,
                CdmwPlatformMngt::PROCESS_FAILED_DISCREPANCY,
                CdmwPlatformMngt::PROCESS_FAILED_INVALID,
                CdmwPlatformMngt::PROCESS_FAILED_NO_RESPONSE,
                CdmwPlatformMngt::PROCESS_FAILED_DEATH
        };
        
        // ----------------------------------------------------------------------
        // ManagedProcess_impl class.
        // ----------------------------------------------------------------------
        ManagedProcess_impl::ManagedProcess_impl(
            PortableServer::POA_ptr poa,
            CdmwPlatformMngt::ManagedProcessDef* process_def,
            const char* process_key, const char* process_path,
            Application_impl* application,
            CdmwPlatformMngt::RWElementRepository_ptr element_repository)
            throw(CORBA::SystemException)
            : Process_impl(poa, process_def, process_key, process_path,
            application, element_repository),
        m_stop_condition (m_mutex_condition),
            m_stop_condition_flag (false)
        {
            m_stateMachine = NULL;
            m_actualEntityContainer = NULL;
            m_actualServiceDefContainer = NULL;
            m_wasProcessDelegateWrapReseted = true;
            m_processDelegateWrap = CdmwPlatformMngt::ProcessDelegateWrapper::_nil();
            m_default_entitydefs = NULL;       
            m_init_sequencer = 0;
            m_init_sync_task_factory = 0;
            try
            {
                m_actualServiceDefContainer
                    = ServiceDefContainer_impl::ServiceDefContainer::createProcessContainer(
                    application->get_element_name(),
                    process_def->name(),
                    process_def->host_name());

                CdmwPlatformMngtService::ServiceDefContainer_var appliServiceDefContainer =
                    application->get_rwservice_defcontainer()->_this();

                m_rwServiceDefContainerServant = new RWServiceDefContainer_impl (
                    poa,
                    m_actualServiceDefContainer,
                    appliServiceDefContainer.in());

                std::auto_ptr <PlatformMngt::EntityContainer> actualEntityContainer (
                    new PlatformMngt::EntityContainer(poa, m_element_repository.in(),
                    m_element_path.in()));

                m_rwEntityContainerServant = 
                    new RWEntityContainer_impl ( 
                    poa, actualEntityContainer.get(), m_element_repository.in());

                
                
                std::auto_ptr <ManagedProcessStateMachine> stateMachine (
                    new ManagedProcessStateMachine (
                    this, "ManagedProcess_impl"));

                std::auto_ptr <CommandCallback> process_start_cbk (new CommandCallback());

                std::auto_ptr <CommandCallback> process_stop_cbk (new CommandCallback());

                std::auto_ptr < CdmwPlatformMngt::GraphElements > 
                       component_init_graph (new CdmwPlatformMngt::GraphElements);

                // Sets the process's internal data
                m_rwServiceDefContainer
                    = dynamic_cast <RWServiceDefContainer_impl*> (m_rwServiceDefContainerServant.in());

                m_rwEntityContainer
                    = dynamic_cast <RWEntityContainer_impl*> (m_rwEntityContainerServant.in());

                m_actualEntityContainer = actualEntityContainer.get();

                m_process_start_cbk = process_start_cbk.get();
                m_process_stop_cbk = process_stop_cbk.get();

                m_component_init_graph = component_init_graph.get();

                m_stateMachine = stateMachine.get();

                // Add the states to the state machine
                m_stateMachine->addState ("Initial", M_initial);
                m_stateMachine->addState ("Initialising", M_initialising);
                m_stateMachine->addState ("StepPerformed", M_stepPerformed);
                m_stateMachine->addState ("Initialised", M_initialised);
                m_stateMachine->addState ("RunRequest", M_runRequest);
                m_stateMachine->addState ("Running", M_running);
                m_stateMachine->addState ("Stopping", M_stopping);
                m_stateMachine->addState ("Stopped", M_stopped);
                m_stateMachine->addState ("Ended", M_ended);
                m_stateMachine->addState ("FailedError", M_failedError);
                m_stateMachine->addState ("FailedMonitoringFailure",
                        M_failedMonitoringFailure);
                m_stateMachine->addState ("FailedDeath", M_failedDeath);
                m_stateMachine->addState ("FailedDiscrepancy", M_failedDiscrepancy);
                m_stateMachine->addState ("FailedTimeout", M_failedTimeout);
                m_stateMachine->addState ("FailedInvalid", M_failedInvalid);

                // Release the ownership of the temporary objects
                actualEntityContainer.release();
                process_start_cbk.release();
                process_stop_cbk.release();
                stateMachine.release();
                component_init_graph.release();

                try
                {
                    // define TECH_STATUS and TECH_STATUS_INFO
                    const char* init_value = "";
                    define_status_property(init_value);

                }
                catch(...)
                {
                    throw   CosPropertyService::MultipleExceptions();
                }
                // Set the initial state
                m_stateMachine->setState ("Initial");

                // Initialise preemption flag (false)
                init_preemption();
            }
            catch (const std::bad_alloc&)
            {
                if (m_actualEntityContainer != NULL)
                {
                    delete m_actualEntityContainer;
                }

                if (m_actualServiceDefContainer != NULL)
                {
                    ServiceDefContainer_impl::ServiceDefContainer::destroy(
                        m_actualServiceDefContainer);
                }

                throw CORBA::NO_MEMORY(OrbSupport::NO_MEMORY, CORBA::COMPLETED_NO);
            }
    }

    ManagedProcess_impl::~ManagedProcess_impl()
    {
        if (m_stateMachine != NULL) 
            delete m_stateMachine;
        
        if (m_actualEntityContainer != NULL)
        {
            delete m_actualEntityContainer;
        }
        
        if (m_actualServiceDefContainer != NULL)
        {
            ServiceDefContainer_impl::ServiceDefContainer::destroy(
                m_actualServiceDefContainer);
        }

        destroyInitSequencer();
        
    if(m_process_start_cbk != NULL)
        delete m_process_start_cbk;
    
    if(m_process_stop_cbk != NULL)
        delete m_process_stop_cbk;          
        
    }

    void ManagedProcess_impl::destroyInitSequencer()
    {
        // Stop and destroy the init sequencer
        if (m_init_sequencer != 0)
        {
            m_init_sequencer->stop ();
            delete m_init_sequencer;
            m_init_sequencer = 0;

        }
        if (m_init_sync_task_factory != 0)
        {
            delete m_init_sync_task_factory;
            m_init_sync_task_factory = 0;
        }
    }

    
    CdmwPlatformMngt::ProcessDelegateWrapper_ptr
    ManagedProcess_impl::getProcessDelegateWrapper()
    {
        CDMW_MUTEX_GUARD (m_processDelegateWrapMutex);

        if (m_wasProcessDelegateWrapReseted) 
        {
            // ProcessDelegateWrapper was not searched.
            // Search ProcessDelegateWrapper
            try
            {
                // get application agent from daemon
                CdmwPlatformMngt::ApplicationAgent_var agent = searchApplicationAgent();
                
                if( CORBA::is_nil (agent.in()))
                {
                    CDMW_INFO(Cdmw::Logging::Logger::Get_Logger(CdmwPlatformMngtBase::PFMMNG),
                        "<<ManagedProcess_impl::synchro_with_datastore()>> agent not exist "
                        << m_element_name.in());
                }
                else
                {        
                    // get the ProcessDelegateWrapper
                    m_processDelegateWrap = agent->search_process_delegate_wrapper(m_element_name.in());
                    
                    if (CORBA::is_nil(m_processDelegateWrap.in()))
                    {
                        CDMW_INFO(Cdmw::Logging::Logger::Get_Logger(CdmwPlatformMngtBase::PFMMNG),
                            "<<ManagedProcess_impl::synchro_with_datastore()>> ProcessDelegateWrapper not exist "
                            << m_element_name.in());
                    }  
                }      
            }
            catch(const CdmwPlatformMngt::HostNotFound& e)
            {
                std::string error_info =
                    LogMngr::getFormattedMessage (MSG_ID_HOST_NOT_FOUND, 
                    m_element_name.in());
                
                CDMW_ERROR(Cdmw::Logging::Logger::Get_Logger(CdmwPlatformMngtBase::PFMMNG),
                    "Process synchronisation : unexpected exception to get agent\n   "
                    << error_info);
            }
            catch(const CdmwPlatformMngt::HostNotReachable& e)
            {
                std::string error_info =
                    LogMngr::getFormattedMessage (MSG_ID_HOST_UNREACHABLE, 
                    m_element_name.in());
                
                CDMW_ERROR(Cdmw::Logging::Logger::Get_Logger(CdmwPlatformMngtBase::PFMMNG),
                    "Process synchronisation : unexpected exception to get agent\n   "
                    << error_info);
                
            }
            catch(const CORBA::Exception& e)
            {
                CDMW_ERROR(Cdmw::Logging::Logger::Get_Logger(CdmwPlatformMngtBase::PFMMNG),
                    "Process synchronisation : unexpected exception to get agent for the host "
                    << m_element_name.in() << ":\n   "
                    << Cdmw::OrbSupport::OrbSupport::exception_to_string(e));
            }
            
            // Found or not, it was searched; no more search until next reset.
            // Update m_wasProcessDelegateWrapReseted flag.
            m_wasProcessDelegateWrapReseted = false;
        }

        return CdmwPlatformMngt::ProcessDelegateWrapper::_duplicate(m_processDelegateWrap.in());
    }


    void ManagedProcess_impl::setProcessDelegateWrapper(
        CdmwPlatformMngt::ProcessDelegateWrapper_ptr processDelegateWrap)
    {
        CDMW_MUTEX_GUARD (m_processDelegateWrapMutex);
        m_processDelegateWrap = 
            CdmwPlatformMngt::ProcessDelegateWrapper::_duplicate(processDelegateWrap);
        // No more search until next reset.
        // Update m_wasProcessDelegateWrapReseted flag.
        m_wasProcessDelegateWrapReseted = false;
    }
    
    void ManagedProcess_impl::resetProcessDelegateWrapper() 
    {
        CDMW_MUTEX_GUARD (m_processDelegateWrapMutex);
        m_wasProcessDelegateWrapReseted = true;
        m_processDelegateWrap = CdmwPlatformMngt::ProcessDelegateWrapper::_nil();
    }
    
    
    void ManagedProcess_impl::end_creation(
        CdmwPlatformMngt::ProcessDef* process_def)
        throw (CdmwPlatformMngt::HostNotFound,
        CdmwPlatformMngt::DuplicateEntity,
        CdmwPlatformMngt::DuplicateService,
        CdmwPlatformMngt::DuplicateStep,
        OutOfMemoryException,
        CosPropertyService::MultipleExceptions,
        CORBA::SystemException)
    {   
        CdmwPlatformMngt::ManagedProcessDef* managed_process_def = CdmwPlatformMngt::ManagedProcessDef::_downcast( process_def);
        
        CdmwPlatformMngt::CompleteMonitoringInfo_var comp_info = new CdmwPlatformMngt::CompleteMonitoringInfo( managed_process_def->monitoring_info() );


        // set element path in rwservicedef container
        m_rwServiceDefContainer->set_container_path (m_element_path.in());
        
        
        // the following initialisation call the methods of the object
        // setting the attribute changes in datastore
        // in case of exception the object will be deleted by the application
        
        // set the host for this process
        // if not found throw HostNotFound exception
        setHost(managed_process_def->host_name());

        
        // set the process info

        setProcessInfo (managed_process_def->process_info());
        
        // set the init steps of the process
        // if duplicated throw DuplicateStep exception
        setLifeCycle (managed_process_def->life_cycle());
        
        // set the monitoring info
        setMonitoringInfo (managed_process_def->monitoring_info());
        
        // initialise data store of the object
        set_record_in_datastore();
        
        
        // Add the entities of the process
        // if duplicated throw DuplicateEntity exception
        const CdmwPlatformMngt::EntityDefs& entities = managed_process_def->entities();
        
        
        unsigned int entityCount =  managed_process_def->entities().length();
        // Store entity initial values into default_entities
        m_default_entitydefs =  new CdmwPlatformMngt::EntityDefs();
        m_default_entitydefs->length(entityCount);
        
        
        
        for (unsigned int entityIndex=0; entityIndex < entities.length(); entityIndex++)
        {  
            // get entity definition (EntityDef is a structure, a reference on the
            // structure is returned by the [] operator)
            const CdmwPlatformMngt::EntityDef& entity_def = entities[entityIndex];
            
            // store values of the entityDef
            (*m_default_entitydefs)[entityIndex] = entity_def;
            
            try
            {
                add_entity (entity_def);
            }
            catch(const CdmwPlatformMngt::EntityAlreadyExists&)
            {
                // in this case the servant will be deleted by application
                throw CdmwPlatformMngt::DuplicateEntity (entity_def.entity_name.in());
            }
        }
        
        // Add the services of the process
        // if duplicated throw DuplicateService exception
        const CdmwPlatformMngtService::ServiceDefs& services =
            managed_process_def->service_definitions();
        
        for (unsigned int serviceIndex = 0; serviceIndex < services.length(); serviceIndex++)
        {
            // get service definition (ServiceDef is a structure, a reference on the
            // structure is returned by the [] operator)
            const CdmwPlatformMngtService::ServiceDef& service_def = services[serviceIndex];
            
            try
            {                          
                add_service_def (service_def);
            }
            catch (const CdmwPlatformMngtService::ServiceAlreadyExists&)
            {
                // in this case the servant will be deleted by application
                throw CdmwPlatformMngt::DuplicateService(
                    service_def.service_name.in());
            }
        }
         
        // Add the components init graph of the process
        CdmwPlatformMngt::GraphElements init_graph = managed_process_def->component_init_graph();
        m_component_init_graph = new CdmwPlatformMngt::GraphElements(init_graph);           
        //Add all Properties in ManagedProcess
        unsigned int propCount = managed_process_def->properties().length();
        for (unsigned int propIndex=0 ; propIndex < propCount ; propIndex++)
        {
            const CdmwPlatformMngt::PropertyDef& prop_def =
                managed_process_def->properties()[propIndex];
         
            std::string propertyName(managed_process_def->properties()[propIndex].property_name);
            CosPropertyService::PropertyModeType propertyMode =
                CosPropertyService::PropertyModeType(managed_process_def->properties()[propIndex].mode_type);
            
            try
            {
                define_property_with_mode(propertyName.c_str() , prop_def.initial_value, propertyMode);
            }
            catch (...)
            {
                throw   CosPropertyService::MultipleExceptions(); /* TO-DO */
            }
            
        }

        // MdR define rw properties
        try
        {
          std::ostringstream property_name;
            
          property_name << CdmwPlatformMngt::CDMW_PREFIX << CdmwPlatformMngt::TRACE_FILTERS << std::ends;
          if(!is_property_defined(property_name.str().c_str()))
          {
              CORBA::Any_var any = new CORBA::Any();
              *any <<= CORBA::string_dup("");
              define_property_with_mode(property_name.str().c_str() , any.in(), CosPropertyService::fixed_normal);
          }
        
          property_name.str("");
          property_name << CdmwPlatformMngt::CDMW_PREFIX << CdmwPlatformMngt::FORCE_MANUAL_START << std::ends;
          if(!is_property_defined(property_name.str().c_str()))
          {
              CORBA::Any_var any = new CORBA::Any();
              *any <<= CORBA::Any::from_boolean(false);
              define_property_with_mode(property_name.str().c_str() , any.in(), CosPropertyService::fixed_normal);
          }
        
          property_name.str("");
          property_name << CdmwPlatformMngt::CDMW_PREFIX << CdmwPlatformMngt::AUTORESTART_INFO << std::ends;
          if(!is_property_defined(property_name.str().c_str()))
          {
              CdmwPlatformMngt::AutoRestartInfo auto_restart_info;
              auto_restart_info.max_nbr_of_restart_attempt = 1;
              auto_restart_info.auto_restart_control = CdmwPlatformMngt::USE_COUNTER;
              CORBA::Any_var any = new CORBA::Any();
              *any <<= auto_restart_info; 
              define_property_with_mode(property_name.str().c_str() , any.in(), CosPropertyService::fixed_normal);
          }
        
          property_name.str("");
          property_name << CdmwPlatformMngt::CDMW_PREFIX << CdmwPlatformMngt::DEFAULT_RESTART_WARMTH << std::ends;
          if(!is_property_defined(property_name.str().c_str()))
          {
              CORBA::Any_var any = new CORBA::Any();
              *any <<= CdmwPlatformMngt::HOT_RESTART; 
              define_property_with_mode(property_name.str().c_str() , any.in(), CosPropertyService::fixed_normal);
          }
          
        }
        catch(...)
        {
          throw   CosPropertyService::MultipleExceptions();
        }
        
        // Set trace filters
        try {
            CORBA::Any_var any_trace_filters;
            get_attribute (CdmwPlatformMngt::TRACE_FILTERS, any_trace_filters.out(), CdmwPlatformMngt::PRE_DEFINED);
            const char* trace_filters;
            any_trace_filters >>= trace_filters;
            std::string str_trace_filters(trace_filters);
            if (!str_trace_filters.empty()) {
                set_trace_filtering ( str_trace_filters.c_str() );
            }
        }
        catch (...) {
            CDMW_FATAL(Cdmw::Logging::Logger::Get_Logger(CdmwPlatformMngtBase::PFMMNG),
                     "Managed process TRACE_FILTERS attribute not found");
            CDMW_ASSERT (false);    
        }
        
        // MdR define system properties not user defined
        std::ostringstream property_name;
        CORBA::Any_var any_val = new CORBA::Any();
        try
        {
            *any_val <<= CdmwPlatformMngt::REDSTATE_NORG;   
            property_name.str("");
            property_name << CdmwPlatformMngt::CDMW_PREFIX << CdmwPlatformMngt::REPLICATION_ROLE << std::ends;                  
            define_property_with_mode(property_name.str().c_str(), any_val.in(), CosPropertyService::fixed_normal);
            
            //CdmwPlatformMngt::RestartWarmth my_restart_warmth = CdmwPlatformMngt::HOT_RESTART;
            get_attribute(CdmwPlatformMngt::DEFAULT_RESTART_WARMTH, any_val.out(), CdmwPlatformMngt::PRE_DEFINED);                                     
            property_name.str("");
            property_name << CdmwPlatformMngt::CDMW_PREFIX << CdmwPlatformMngt::REQUESTED_RESTART_WARMTH << std::ends;
            define_property_with_mode(property_name.str().c_str(), any_val.in(), CosPropertyService::fixed_normal);
        }
        catch (...)
        {
            throw CosPropertyService::MultipleExceptions ();
        }

        set_indicator_relevant_status(managed_process_def->indicator_relevant_status());

        //store the default user defined attributes values 
        store_default_properties();
    }
    
    
    
    void ManagedProcess_impl::synchro_processing_to_initial()
    {
        m_stateMachine->setStateName("Initial");
    }
    
    
    void ManagedProcess_impl::synchro_processing_to_initialised()
    {
        m_stateMachine->setStateName("Initialised");
    }
    
    
    void ManagedProcess_impl::synchro_processing_to_running()
    {
        m_stateMachine->setStateName("Running");
    }
    
    
    
    const char* ManagedProcess_impl::get_state_machine_state_name()
    {
        return m_stateMachine->getStateName();
    }
    
    void ManagedProcess_impl::set_state_machine_state_name(const char* state_name)
    {
        m_stateMachine->setStateName(state_name);
    }
    
    void ManagedProcess_impl::set_state_machine_previous_state_name(const char* previous_state_name)  throw (CORBA::SystemException)
    {
    string prev_state(previous_state_name);

        m_stateMachine->setPreviousStateName(prev_state);
    }
    
    
    void ManagedProcess_impl::createStates()
        throw( OutOfMemoryException )
    {
        try
        {
            std::auto_ptr< ManagedProcessInitial > initial(
                new ManagedProcessInitial() );
            std::auto_ptr< ManagedProcessInitialising > initialising(
                new ManagedProcessInitialising() );
            std::auto_ptr< ManagedProcessStepPerformed > stepPerformed(
                new ManagedProcessStepPerformed() );
            std::auto_ptr< ManagedProcessInitialised > initialised(
                new ManagedProcessInitialised() );
            std::auto_ptr< ManagedProcessRunRequest > runRequest(
                new ManagedProcessRunRequest() );
            std::auto_ptr< ManagedProcessRunning > running(
                new ManagedProcessRunning() );
            std::auto_ptr< ManagedProcessStopping > stopping(
                new ManagedProcessStopping() );
            std::auto_ptr< ManagedProcessStopped > stopped(
                new ManagedProcessStopped() );
            std::auto_ptr< ManagedProcessEnded > ended(
                new ManagedProcessEnded() );
            std::auto_ptr< ManagedProcessFailedError > failedError(
                new ManagedProcessFailedError() );
            std::auto_ptr< ManagedProcessFailedMonitoringFailure > failedMonitoringFailure(
                new ManagedProcessFailedMonitoringFailure() );
            std::auto_ptr< ManagedProcessFailedDeath > failedDeath(
                new ManagedProcessFailedDeath() );
            std::auto_ptr< ManagedProcessFailedDiscrepancy > failedDiscrepancy(
                new ManagedProcessFailedDiscrepancy() );
            std::auto_ptr< ManagedProcessFailedTimeout > failedTimeout(
                new ManagedProcessFailedTimeout() );
            std::auto_ptr< ManagedProcessFailedInvalid > failedInvalid(
                new ManagedProcessFailedInvalid() );
        
            
            // If the process is a component server
            //if (CdmwPlatformMngt::COMPONENT_SERVER == this->type() )
            {
            CcmComponent_impl::createStates();
            }
            
            if( M_initial == NULL )
                M_initial = initial.release();
            if( M_initialising == NULL )
                M_initialising = initialising.release();
            if( M_stepPerformed == NULL )
                M_stepPerformed = stepPerformed.release();
            if( M_initialised == NULL )
                M_initialised = initialised.release();
            if( M_runRequest == NULL )
                M_runRequest = runRequest.release();
            if( M_running == NULL )
                M_running = running.release();
            if( M_stopping == NULL )
                M_stopping = stopping.release();
            if( M_stopped == NULL )
                M_stopped = stopped.release();
            if( M_ended == NULL )
                M_ended = ended.release();
            if( M_failedError == NULL )
                M_failedError = failedError.release();
            if( M_failedMonitoringFailure == NULL )
                M_failedMonitoringFailure = failedMonitoringFailure.release();
            if( M_failedDeath == NULL )
                M_failedDeath = failedDeath.release();
            if( M_failedDiscrepancy == NULL )
                M_failedDiscrepancy = failedDiscrepancy.release();
            if( M_failedTimeout == NULL )
                M_failedTimeout = failedTimeout.release();
            if( M_failedInvalid == NULL )
                M_failedInvalid = failedInvalid.release();
        }
        catch( const std::bad_alloc& )
        {
            CDMW_THROW( OutOfMemoryException );
        }
    }
    
    void ManagedProcess_impl::destroyStates()
    {
        if( M_initial != NULL )
        {
            delete M_initial;
            M_initial = NULL;
        }
        
        if( M_initialising != NULL )
        {
            delete M_initialising;
            M_initialising = NULL;
        }
        
        if( M_stepPerformed != NULL )
        {
            delete M_stepPerformed;
            M_stepPerformed = NULL;
        }
        
        if( M_initialised != NULL )
        {
            delete M_initialised;
            M_initialised = NULL;
        }
        
        if( M_runRequest != NULL )
        {
            delete M_runRequest;
            M_runRequest = NULL;
        }
        
        if( M_running != NULL )
        {
            delete M_running;
            M_running = NULL;
        }
        
        if( M_stopping != NULL )
        {
            delete M_stopping;
            M_stopping = NULL;
        }
        
        if( M_stopped != NULL )
        {
            delete M_stopped;
            M_stopped = NULL;
        }
        
        if( M_ended != NULL )
        {
            delete M_ended;
            M_ended = NULL;
        }
        
        if( M_failedError != NULL )
        {
            delete M_failedError;
            M_failedError = NULL;
        }
        
        if( M_failedMonitoringFailure != NULL )
        {
            delete M_failedMonitoringFailure;
            M_failedMonitoringFailure = NULL;
        }
        
        if( M_failedDeath != NULL )
        {
            delete M_failedDeath;
            M_failedDeath = NULL;
        }
        
        if( M_failedDiscrepancy != NULL )
        {
            delete M_failedDiscrepancy;
            M_failedDiscrepancy = NULL;
        }
        
        if( M_failedTimeout != NULL )
        {
            delete M_failedTimeout;
            M_failedTimeout = NULL;
        }
        
        if( M_failedInvalid != NULL )
        {
            delete M_failedInvalid;
            M_failedInvalid = NULL;
        }
    
        // If the process is a component server
         //if (CdmwPlatformMngt::COMPONENT_SERVER == this->type() )
         {
         CcmComponent_impl::destroyStates();
         }
     }
    
    
    void ManagedProcess_impl::error_event(
        const CdmwPlatformMngtBase::EventHeader& header,
        const char* issuer,
        const char* error_info)
    {

        CDMW_MUTEX_GUARD(*m_application->get_check_consistency_status_recursive_mutex());

        CDMW_SM_WRITER_LOCK_GUARD(*m_stateMachine);
        
        m_stateMachine->getState()->error_event(
            m_stateMachine,
            header,
            issuer,
            error_info);

    }



    
    void ManagedProcess_impl::ending_event(
        const CdmwPlatformMngtBase::EventHeader& header)
    {

        CDMW_MUTEX_GUARD(*m_application->get_check_consistency_status_recursive_mutex());

        CDMW_SM_WRITER_LOCK_GUARD(*m_stateMachine);
        
        m_stateMachine->getState()->ending_event(
            m_stateMachine,
            header);

    }
    
    void ManagedProcess_impl::monitoringFailure_event(
        const CdmwPlatformMngtBase::EventHeader& header)
        throw( CdmwPlatformMngt::IncompatibleType,
        CORBA::SystemException )
    {

      CDMW_MUTEX_GUARD(*m_application->get_check_consistency_status_recursive_mutex());

      CDMW_SM_WRITER_LOCK_GUARD(*m_stateMachine);
   
        m_stateMachine->getState()->monitoringFailure_event(
            m_stateMachine,
            header);
        
        // update record in datastore
        set_record_in_datastore();

    }
    
    void ManagedProcess_impl::invalidate_event(
        const CdmwPlatformMngtBase::EventHeader& header,
        const char* reason)
    {

      CDMW_MUTEX_GUARD(*m_application->get_check_consistency_status_recursive_mutex());

        CDMW_SM_WRITER_LOCK_GUARD(*m_stateMachine);

        m_stateMachine->getState()->invalidate_event(
            m_stateMachine,
            header,
            reason);

    }
    
    CdmwPlatformMngt::ProcessStatus ManagedProcess_impl::get_internal_status()
    {

       CDMW_SM_READER_LOCK_GUARD(*m_stateMachine);

        // get status
        CdmwPlatformMngt::ProcessStatus processStatus = 
            m_stateMachine->getState()->status();

        return processStatus;
    }
    
    
    CORBA::Object_ptr ManagedProcess_impl::embedded_service()
        throw( CORBA::SystemException ) 
    {
        if( type() != CdmwPlatformMngt::SERVICE_PROVIDER )
            return CORBA::Object::_nil();
        
        CdmwPlatformMngt::ProcessDelegateWrapper_var processDelegateWrap =
            getProcessDelegateWrapper();
        
        if (CORBA::is_nil (processDelegateWrap.in()))
            return CORBA::Object::_nil();
        
        return processDelegateWrap->get_service();
    }
    
    CdmwPlatformMngt::ProcessStatus ManagedProcess_impl::property_to_status (const char* status)
    throw (CdmwPlatformMngt::AttributeNotFound)
    {
        CdmwPlatformMngt::ProcessStatus proc_status; 
        bool result = false;
        for ( int i=0 ; i<15; i++)
        {
            if (strcmp(status, process_status_strings[i].c_str()) == 0)
            {
                proc_status = Status[i];
                result = true;

                return proc_status;
            }
        }
        if (!result)
        {
            throw CdmwPlatformMngt::AttributeNotFound();
        }
        
        return proc_status;
    }

    CdmwPlatformMngt::ProcessStatus ManagedProcess_impl::get_status(
            CORBA::String_out status_info )
    throw( CORBA::SystemException )    
    {
        
        CdmwPlatformMngt::ProcessStatus process_status;
        try
        {
            // Get predefined attributes of process
            CORBA::Any_var any_var = new CORBA::Any();
            const char* status = "";
            
            get_attribute(CdmwPlatformMngt::TECH_STATUS, any_var.out(), CdmwPlatformMngt::PRE_DEFINED);           
            any_var.in() >>=  CORBA::Any::to_string(status,0);            
            
            process_status = property_to_status (status);    
            
            // get info associated to status
            status_info = CORBA::string_dup (get_status_info());

            return process_status;
        }
        catch (const CdmwPlatformMngt::AttributeNotFound&)
        {
            throw CORBA::BAD_PARAM(OrbSupport::BAD_PARAM, CORBA::COMPLETED_NO);
        }
        catch (const std::bad_alloc&)
        {
            throw CORBA::NO_MEMORY (OrbSupport::NO_MEMORY, CORBA::COMPLETED_NO);
        }
        catch (...)
        {
            throw CORBA::BAD_PARAM(OrbSupport::BAD_PARAM, CORBA::COMPLETED_NO);
        }
    }
    
    void ManagedProcess_impl::set_auto_ending(CORBA::Boolean auto_ending)
        throw (CdmwPlatformMngt::IncompatibleStatus,
        CORBA::SystemException)
    {
        CDMW_SM_READER_LOCK_GUARD(*m_stateMachine);
        
        m_stateMachine->getState()->set_autoending (m_stateMachine, auto_ending);
    }
    
    void ManagedProcess_impl::set_host(
        const char* host_name)
        throw (CdmwPlatformMngt::HostNotFound,
        CdmwPlatformMngt::IncompatibleStatus,
        CORBA::SystemException)
    {
        CDMW_SM_READER_LOCK_GUARD(*m_stateMachine);
        
        m_stateMachine->getState()->set_host(m_stateMachine, host_name);
    }
    
    void ManagedProcess_impl::set_process_info(
        const CdmwPlatformMngt::ProcessInfo& process_info)
        throw (CdmwPlatformMngt::IncompatibleStatus,
        CORBA::SystemException)
    {
        CDMW_SM_READER_LOCK_GUARD(*m_stateMachine);
        
        m_stateMachine->getState()->set_process_info(
            m_stateMachine,
            process_info);
    }
    
    CdmwPlatformMngt::LifeCycleDef* ManagedProcess_impl::get_life_cycle()
        throw (CdmwPlatformMngt::IncompatibleType,
        CORBA::SystemException)
    {
        try
        {
            CdmwPlatformMngt::LifeCycleDef_var lifeCycleDef
                = new CdmwPlatformMngt::LifeCycleDef (m_lifeCycle.in());
            
            return lifeCycleDef._retn();
        }
        catch(const std::bad_alloc&)
        {
            throw CORBA::NO_MEMORY(OrbSupport::NO_MEMORY, CORBA::COMPLETED_NO);
        }
    }
    
    void ManagedProcess_impl::set_life_cycle(
        const CdmwPlatformMngt::LifeCycleDef& life_cycle)
        throw (CdmwPlatformMngt::DuplicateStep,
        CdmwPlatformMngt::IncompatibleType,
        CdmwPlatformMngt::IncompatibleStatus,
        CORBA::SystemException)
    {
        CDMW_SM_READER_LOCK_GUARD(*m_stateMachine);
        
        m_stateMachine->getState()->set_life_cycle(
            m_stateMachine,
            life_cycle);
    }
    
    CdmwPlatformMngt::CompleteMonitoringInfo ManagedProcess_impl::get_monitoring_info()
        throw (CdmwPlatformMngt::IncompatibleType,
        CORBA::SystemException)
    {
        return m_monitoring_info.in();
    }
    
    void ManagedProcess_impl::set_monitoring_info(
        const CdmwPlatformMngt::CompleteMonitoringInfo& monitoring_info)
        throw (CdmwPlatformMngt::IncompatibleType,
        CdmwPlatformMngt::IncompatibleStatus,
        CORBA::SystemException)
    {
        CDMW_SM_READER_LOCK_GUARD(*m_stateMachine);
        
        m_stateMachine->getState()->set_monitoring_info(
            m_stateMachine,
            monitoring_info);
    }
    
    CdmwPlatformMngt::RequestStatus ManagedProcess_impl::initialise(
        const CdmwPlatformMngtBase::StartupKind& startup_kind)
        throw (CdmwPlatformMngt::IncompatibleStatus,
        CORBA::SystemException)
    {
        LogMngr::logMessage(
            INF,
            MSG_ID_PROCESS_INITALISE,
            get_process_key(),
            m_application->get_element_name());

         de_activate_preemption();

         CdmwPlatformMngt::RequestStatus req_status = CdmwPlatformMngt::REQUEST_SUCCEED;

         if (! is_preemption_active())
         {

         //ADO: PCR-1964
         // Removed recorsive mutex in order to initialise in parallel "root" processes 
         //CDMW_MUTEX_GUARD(*m_application->get_check_consistency_status_recursive_mutex());

         CDMW_SM_WRITER_LOCK_GUARD(*m_stateMachine);
        
        //m_startupKind = new CdmwPlatformMngtBase::StartupKind( startup_kind );
        m_startupKind = startup_kind;        
        
        if ( CdmwPlatformMngt::COLD_RESTART==requested_restart_warmth() )
        {
            recover_default_properties ();
            recover_default_entities (m_rwEntityContainer, m_default_entitydefs);
        }
        
        try
        {
            req_status =
                m_stateMachine->getState()->initialise(
                m_stateMachine,
                startup_kind);

        }
        catch (...)
        {

            // even in case of exception update record in datastore
            set_record_in_datastore();
            
            // rethrow exception
            throw;
        }


 
        // set the entry in HostProcessInitSequencer, to restart process after host reboot
        Cdmw::PlatformMngt::ProcessHostInfo info;
        info.process_name = CORBA::string_dup (name());
        info.process_impl  = this;
        if (true == created_dynamically())
        {
            info.kind_of_start = ADDED;
            set_proc_info(info,0);
        }
        else 
            info.kind_of_start = DEFAULT;
        
        //LC: PCR-792        set_proc_info(info,0);

        //HostProcessInitSequencer::get_instance()->set_init_entry(m_host->name(),info);
        //HostProcessInitSequencer::get_instance()->set_run_entry(m_host->name(),info);  
        // update record in datastore

        set_record_in_datastore();
        }

        return req_status;
    }
    
    CdmwPlatformMngt::RequestStatus ManagedProcess_impl::next_step()
        throw( CdmwPlatformMngt::InvalidStep,
        CdmwPlatformMngt::IncompatibleType,
        CdmwPlatformMngt::IncompatibleStatus,
        CORBA::SystemException )
    {

        //ADO: PCR-1964
       // CDMW_MUTEX_GUARD(*m_application->get_check_consistency_status_recursive_mutex());


        CDMW_SM_WRITER_LOCK_GUARD(*m_stateMachine);
        
        CdmwPlatformMngt::RequestStatus req_status;
        
        try
        {
            req_status =
                m_stateMachine->getState()->next_step(m_stateMachine);
        }
        catch (...)
        {
            // even in case of exception update record in datastore
            set_record_in_datastore();
            
            // rethrow exception
            throw;
        }
        
        // update record in datastore
        set_record_in_datastore();
        return req_status;
    }
    
    CdmwPlatformMngt::RequestStatus ManagedProcess_impl::nextStep(
        unsigned int targetStep)
        throw (CdmwPlatformMngt::InvalidStep,
        CdmwPlatformMngt::IncompatibleType,
        CdmwPlatformMngt::IncompatibleStatus,
        CORBA::SystemException)
    {
        //ADO: PCR-1964
        //CDMW_MUTEX_GUARD(*m_application->get_check_consistency_status_recursive_mutex());

        CDMW_SM_WRITER_LOCK_GUARD(*m_stateMachine);
        
        if (targetStep != (m_stepCounter+1))
        {
            LogMngr::logMessage(
                ERR,
                MSG_ID_PROCESS_UNEXPECTED_STEP,
                targetStep,
                get_process_key());
            
            return CdmwPlatformMngt::REQUEST_ERROR;
        }
        
        CdmwPlatformMngt::RequestStatus req_status;
        
        try
        {
            req_status =
                m_stateMachine->getState()->next_step(m_stateMachine);
        }
        catch (...)
        {
            // even in case of exception update record in datastore
            set_record_in_datastore();
            
            // rethrow exception
            throw;
        }
        
        // update record in datastore
        set_record_in_datastore();

        return req_status;
    }
    
    CdmwPlatformMngt::RequestStatus ManagedProcess_impl::run()
        throw( CdmwPlatformMngt::NotReadyToRun,
        CdmwPlatformMngt::IncompatibleType,
        CdmwPlatformMngt::IncompatibleStatus,
        CORBA::SystemException )
    {
        LogMngr::logMessage(
            INF,
            MSG_ID_PROCESS_RUN,
            get_process_key(),
            m_application->get_element_name());
        
         CdmwPlatformMngt::RequestStatus req_status = CdmwPlatformMngt::REQUEST_SUCCEED;
         
         de_activate_preemption();

         if (! is_preemption_active())
         {
        
        CDMW_MUTEX_GUARD(*m_application->get_check_consistency_status_recursive_mutex());


        CDMW_SM_WRITER_LOCK_GUARD(*m_stateMachine);       
        
        try
        {
            // recover the default user defined attributes values and entities 
  
            req_status =
                m_stateMachine->getState()->run(m_stateMachine);

        }
        catch (...)
        {
            // even in case of exception update record in datastore
            set_record_in_datastore();
            
            // rethrow exception
            throw;
        }
        
        // update record in datastore
        set_record_in_datastore();
        }
        
        return req_status;
    }
    
    CdmwPlatformMngt::RequestStatus ManagedProcess_impl::stop(CORBA::Boolean emergency) throw ( CORBA::SystemException ) 
    {
    
        LogMngr::logMessage(
            INF,
            MSG_ID_PROCESS_STOP,
            get_process_key(),
            m_application->get_element_name());

        activate_preemption();
        //MR
        //set_requested_restart_warmth ( get_default_restart_warmth () );

        //CDMW_MUTEX_GUARD(*m_application->get_check_consistency_status_recursive_mutex());

        CDMW_SM_WRITER_LOCK_GUARD(*m_stateMachine);

        CdmwPlatformMngt::RequestStatus req_status;

        try
        {    
            req_status =
                m_stateMachine->getState()->stop(m_stateMachine, emergency);
        }
        catch (...)
        {
            // even in case of exception update record in datastore
            set_record_in_datastore();

            // rethrow exception
            throw;
        }

        // update record in datastore
        set_record_in_datastore();

        return req_status;
    }
    
    CdmwPlatformMngt::RequestStatus ManagedProcess_impl::stop_process(CORBA::Boolean emergency)
        throw (CORBA::SystemException)
    {    
        // Create a command event to notify
        std::string proc_path = std::string (element_path());
        //LC: Attention: call an issuer mutex guard before calling this method.        
        std::string issuer = Cdmw::PlatformMngt::ProcessIssuerManager::Instance()->get_command_issuer(proc_path);
        notifyCommandEvent(Process_impl::STOP, issuer.c_str());

        return stop(emergency);
    } 
    
    
       CdmwPlatformMngt::LongRequestStatus ManagedProcess_impl::
        stop_and_wait (CORBA::Boolean emergency) 
            throw ( CORBA::SystemException )
    {
        CdmwPlatformMngt::ProcessStatus procStatus;
        CdmwPlatformMngt::LongRequestStatus req_status =
            CdmwPlatformMngt::LONG_REQUEST_COMPLETED;
        bool stop_req_done = false;

    {
           // lock mutex for reading status

            CDMW_SM_READER_LOCK_GUARD (*m_stateMachine);

           // get the current status
            procStatus = m_stateMachine->getState ()->status ();
            
            if (/*procStatus == CdmwPlatformMngt::PROCESS_INITIAL ||*/
                procStatus == CdmwPlatformMngt::PROCESS_STOPPED ||
                procStatus == CdmwPlatformMngt::PROCESS_FAILED_DEATH ||
                procStatus == CdmwPlatformMngt::PROCESS_FAILED_NO_RESPONSE ||
                procStatus == CdmwPlatformMngt::PROCESS_FAILED_INVALID ||
                procStatus == CdmwPlatformMngt::PROCESS_FAILED_TIMEOUT ||
                procStatus == CdmwPlatformMngt::PROCESS_FAILED_ERROR ||
                procStatus == CdmwPlatformMngt::PROCESS_FAILED_DISCREPANCY || 
                procStatus == CdmwPlatformMngt::PROCESS_ENDED )
            {
                    req_status = CdmwPlatformMngt::LONG_REQUEST_COMPLETED;
                    stop_req_done = true;
            }

    }


        // if stop request not done :
        // stopping is in progress : condition flag is not set
        // (flag not set)
        if (!stop_req_done)
        {
        // first lock mutex for condition

            CDMW_MUTEX_GUARD (m_mutex_condition);

            m_stop_condition_flag = false;


            if (procStatus != CdmwPlatformMngt::PROCESS_STOPPING)
            {
                CdmwPlatformMngt::RequestStatus status = stop (emergency);
           
                if( CdmwPlatformMngt::REQUEST_SUCCEED != status ) {
                    return CdmwPlatformMngt::LONG_REQUEST_NOT_COMPLETED;
                }

            }

            // stopping is in progress
            // so wait the condition flag is set by broadcasting
            // only if status is not INITIAL

            if (CdmwPlatformMngt::PROCESS_INITIAL == procStatus)
            {
                m_stop_condition_flag = true;
            }

            while (!m_stop_condition_flag)
            {
                 m_stop_condition.wait ();
            }

            m_stop_condition_flag = false;

           // get the result
            req_status = m_stop_status;

    }

        return req_status;
    }

    void ManagedProcess_impl::setStopStatus (CdmwPlatformMngt::
        LongRequestStatus status)
    {
    CDMW_MUTEX_GUARD (m_mutex_condition);
        // set condition wait flag
        m_stop_condition_flag = true;
        m_stop_status = status;
        m_stop_condition.broadcast ();
    }
    
   CORBA::ULong ManagedProcess_impl::get_number_of_entities()
        throw( CORBA::SystemException )
    {
        return m_rwEntityContainer->get_number_of_entities();
    }
    
    void ManagedProcess_impl::get_all_entity_names(
        CORBA::ULong how_many,
        CdmwPlatformMngt::ElementNames_out entity_names,
        CdmwPlatformMngt::ElementNamesIterator_out rest )
        throw( CORBA::SystemException )
    {
        m_rwEntityContainer->get_all_entity_names(
            how_many, entity_names, rest );
    }
    
    CdmwPlatformMngt::Entity_ptr ManagedProcess_impl::get_entity (
        const char* entity_name)
        throw( CdmwPlatformMngt::EntityNotFound, CORBA::SystemException )
    {
        return m_rwEntityContainer->get_entity (entity_name);
    }
    
    CORBA::Boolean ManagedProcess_impl::get_entities(
        const CdmwPlatformMngt::ElementNames& entity_names,
        CdmwPlatformMngt::Entities_out nentities )
        throw( CORBA::SystemException )
    {
        return m_rwEntityContainer->get_entities(
            entity_names, nentities );
    }
    
    void ManagedProcess_impl::get_all_entities(
        CORBA::ULong how_many,
        CdmwPlatformMngt::Entities_out nentities,
        CdmwPlatformMngt::EntitiesIterator_out rest )
        throw( CORBA::SystemException )
    {
        m_rwEntityContainer->get_all_entities(
            how_many, nentities, rest );
    }
    
    CdmwPlatformMngt::Entity_ptr ManagedProcess_impl::add_entity(
        const CdmwPlatformMngt::EntityDef & entity_def)
        throw( CdmwPlatformMngt::EntityAlreadyExists, CORBA::SystemException )
    {
        std::string entityName = entity_def.entity_name.in();
        
        LogMngr::logMessage(INF, MSG_ID_ADD_PROCESS_ENTITY, 
            entityName.c_str(), m_proc_appli_name.in(), 
            m_element_name.in(), m_proc_host_name.in());
        
        CdmwPlatformMngt::Entity_ptr ent_ptr = m_rwEntityContainer->add_entity(entity_def);
        
        notify_configuration_change("Entity Added");

        return ent_ptr;   
    }
    
    void ManagedProcess_impl::remove_entity(
        const char* entity_name )
        throw( CdmwPlatformMngt::EntityNotFound, CORBA::SystemException )
    {
        LogMngr::logMessage(INF, MSG_ID_REMOVE_PROCESS_ENTITY, 
            entity_name, m_proc_appli_name.in(), 
            m_element_name.in(), m_proc_host_name.in());
        
        m_rwEntityContainer->remove_entity(entity_name);
        
        notify_configuration_change("Entity Removed");
    }
    
    
    
    Entity_impl *ManagedProcess_impl::getEntity (const char *entity_name)
        throw (CORBA::SystemException)
    {
        return m_actualEntityContainer->getEntity (entity_name);
    }

    void ManagedProcess_impl::synchro_entities_with_datastore()
    {
        CORBA::ULong how_many = get_number_of_entities (); 
        CdmwPlatformMngt::ElementNames_var entity_names;
        CdmwPlatformMngt::ElementNamesIterator_var rest;
        get_all_entity_names (how_many, entity_names, rest);
        for (CORBA::ULong count = 0U; count< how_many; count++)
        {
           //synchro the entity with its datastore 
           getEntity(entity_names[count])->synchro_with_datastore();
        }
    }  
    
    CdmwPlatformMngtService::ServiceProviderID*
        ManagedProcess_impl::get_service_def(
        const char* service_name )
        throw( CdmwPlatformMngtService::ServiceNotFound, CORBA::SystemException )
    {
        return m_rwServiceDefContainer->get_service_def( service_name );
    }
    
    void ManagedProcess_impl::add_service_def(
        const CdmwPlatformMngtService::ServiceDef& service_def )
        throw( CdmwPlatformMngtService::ServiceAlreadyExists, CORBA::SystemException )
    {
        m_rwServiceDefContainer->add_service_def( service_def );
        
        // TODO: Notify the configuration change
        
        // TODO: Trace to the log manager
    }
    
    void ManagedProcess_impl::remove_service_def(
        const char* service_name )
        throw( CdmwPlatformMngtService::ServiceNotFound, CORBA::SystemException )
    {
        m_rwServiceDefContainer->remove_service_def( service_name );
        
        // TODO: Notify the configuration change
        
        // TODO: Trace to the log manager
    }
    
    void ManagedProcess_impl::remove_all_service_defs()
        throw (CORBA::SystemException)
    {
        m_rwServiceDefContainer->remove_all_service_defs();
        
        // TODO: Notify the configuration change
        
        // TODO: Trace to the log manager
    }
    
    
    void ManagedProcess_impl::setState( const char* stateName )
    {
            m_stateMachine->setState (stateName);
     
    }

    /**
     *Purpose:
     *<p> to activate specific level with passing e level value ...
     *@param : com: component name, dom: domain name, level_value: level value.
     */
    void ManagedProcess_impl::activate_level(const char *com, const char *dom, CORBA::ULong level_value)
    {
    m_processDelegateWrap->activate_level(com, dom, level_value);
    }

    /**
     *Purpose:
     *<p> to deactivate specific level with passing e level value ...
     *@param : com: component name, dom: domain name, level_value: level value.
     */
    void ManagedProcess_impl::deactivate_level(const char *com, const char *dom, CORBA::ULong level_value)
    {
    m_processDelegateWrap->deactivate_level(com, dom, level_value);
    }
    
    CdmwPlatformMngt::ProcessStatus ManagedProcess_impl::getStatus()
        throw( CORBA::SystemException )    
    {
        return m_stateMachine->getState()->status();
    }

    CdmwPlatformMngt::ProcessStatus ManagedProcess_impl::getPreviousStatus()
        throw( CORBA::SystemException )
    {
        //return m_stateMachine->getPreviousState()->status();

    CdmwPlatformMngt::ProcessStatus prev_status =  CdmwPlatformMngt::PROCESS_INITIAL;

        string str =  m_stateMachine->getPreviousStateName();

        if (str.compare("Initial") == 0)
       prev_status = CdmwPlatformMngt::PROCESS_INITIAL;

        if (str.compare("Initialising") == 0)
       prev_status = CdmwPlatformMngt::PROCESS_INITIALISING;

        if (str.compare("StepPerformed") == 0)
       prev_status = CdmwPlatformMngt::PROCESS_STEP_PERFORMED;
        
        if (str.compare("Initialised") == 0)
       prev_status = CdmwPlatformMngt::PROCESS_INITIALISED ;

        if (str.compare("RunRequest") == 0)
       prev_status = CdmwPlatformMngt::PROCESS_RUN_REQUEST;

        if (str.compare("Running") == 0)
       prev_status = CdmwPlatformMngt::PROCESS_RUNNING;

        if (str.compare("Stopping") == 0)
       prev_status = CdmwPlatformMngt::PROCESS_STOPPING;

        if (str.compare("Stopped") == 0)
       prev_status = CdmwPlatformMngt::PROCESS_STOPPED;

        if (str.compare("Ended") == 0)
       prev_status = CdmwPlatformMngt::PROCESS_ENDED;

        if (str.compare("FailedError") == 0)
       prev_status = CdmwPlatformMngt::PROCESS_FAILED_ERROR;

        if (str.compare("FailedMonitoringFailure") == 0)
       prev_status = CdmwPlatformMngt::PROCESS_FAILED_NO_RESPONSE;

        if (str.compare("FailedDeath") == 0)
       prev_status = CdmwPlatformMngt::PROCESS_FAILED_DEATH;

        if (str.compare("FailedDiscrepancy") == 0)
       prev_status = CdmwPlatformMngt::PROCESS_FAILED_DISCREPANCY;

        if (str.compare("FailedTimeout") == 0)
       prev_status = CdmwPlatformMngt::PROCESS_FAILED_TIMEOUT;

        if (str.compare("FailedInvalid") == 0)
       prev_status = CdmwPlatformMngt::PROCESS_FAILED_INVALID;

        return prev_status;
    }
    
    void ManagedProcess_impl::setLifeCycle(
        const CdmwPlatformMngt::LifeCycleDef& life_cycle)
        throw (CdmwPlatformMngt::DuplicateStep,
        CORBA::SystemException)
    {
        try
        {
            std::set< std::string > steps;
            for (unsigned int i=0; i<life_cycle.init_steps.length(); i++)
            {
                std::string stepName = life_cycle.init_steps[i].step_label.in();
                if (steps.find(stepName) != steps.end())
                    throw CdmwPlatformMngt::DuplicateStep(stepName.c_str());
                
                steps.insert(stepName);
            }
            
            m_lifeCycle = new CdmwPlatformMngt::LifeCycleDef(life_cycle);
            
            // update record in datastore
            set_record_in_datastore();
        }
        catch(const std::bad_alloc&)
        {
            throw CORBA::NO_MEMORY(OrbSupport::NO_MEMORY, CORBA::COMPLETED_NO);
        }
    }
    
    void ManagedProcess_impl::notifyStatusChangeEvent(
        const CdmwPlatformMngtBase::EventHeader& header,
        const char* issuer )
    {
        try
        {
            // Get the status
            CdmwPlatformMngt::ProcessStatus status
                = m_stateMachine->getState()->status();
            
            // Create the event
            CdmwPlatformMngt::ProcessID_var processId
                = new CdmwPlatformMngt::ProcessID;
            processId->application_name = m_application->name();
            processId->process_name = name();
            processId->host_name = m_host->name();
            
            CORBA::String_var processHostName = m_host->name();
            
            const char* p_stepLabel = 
                (m_lifeCycle->init_steps.length() == 0) ?
                "" :
            m_lifeCycle->init_steps[m_stepCounter].step_label.in();
            
            CdmwPlatformMngt::ProcessExitInfo_var exit_info;
            
            if( (status == CdmwPlatformMngt::PROCESS_FAILED_DEATH) ||
                (status == CdmwPlatformMngt::PROCESS_FAILED_NO_RESPONSE) ||
                (status == CdmwPlatformMngt::PROCESS_FAILED_INVALID) ||
                (status == CdmwPlatformMngt::PROCESS_FAILED_DISCREPANCY) ||
                (status == CdmwPlatformMngt::PROCESS_FAILED_TIMEOUT) ||
                (status == CdmwPlatformMngt::PROCESS_FAILED_ERROR) ||
                (status == CdmwPlatformMngt::PROCESS_ENDED) ||
                (status == CdmwPlatformMngt::PROCESS_STOPPED) ){
                exit_info = new  CdmwPlatformMngt::ProcessExitInfo() ;
                exit_info->core_dumped=CdmwPlatformMngt::CORE_UNKNOWN;
                exit_info->process_pid = pid();
                exit_info->exit_event=CdmwPlatformMngt::EXITED;
                exit_info->event_value=0;
            }
            else{
                exit_info = new  CdmwPlatformMngt::ProcessExitInfo() ;
                exit_info->core_dumped=CdmwPlatformMngt::CORE_UNKNOWN;
                exit_info->process_pid = 0;
                exit_info->exit_event=CdmwPlatformMngt::EXITED;
                exit_info->event_value=0;
            }
            
            CdmwPlatformMngt::ProcessStatusChange_var event
                = m_procStatusChangeFactory->create(
                header,
                m_application->get_system_name(),
                processId.in(),
                issuer,
                status,
                m_stepCounter,
                p_stepLabel,
                get_status_info(),
                exit_info.in() );
            
            
            CORBA::String_var eventStr = event->to_string();
            LogMngr::logMessage(eventStr.in());
            
            event->_add_ref();
            m_application->get_event_notifier()->addEvent(event);
            // Perform the status change notification to observer registered
            // with the Api in ManagedElement
            event->_add_ref();
            get_status_event_notifier()->add_event (event);
            event->_remove_ref();
            event._retn ();

        }
        catch(const std::bad_alloc&)
        {
            throw CORBA::NO_MEMORY(OrbSupport::NO_MEMORY, CORBA::COMPLETED_NO);
        }
        catch(...)
        {
            throw CORBA::INTERNAL(OrbSupport::INTERNAL, CORBA::COMPLETED_NO);
        }
    }
    
    
    void ManagedProcess_impl::set_create_process_result (
        CdmwPlatformMngt::ProcessCommandResult result,
        const char* error_info,
        const char* error_issuer,
        CdmwPlatformMngt::ProcessDelegateWrapper_ptr process_delegate_wrap)
        throw (CORBA::SystemException)
    {
        // store the process delegate wrapper object
        {
            CDMW_MUTEX_GUARD (m_processDelegateWrapMutex);
            m_processDelegateWrap =
                CdmwPlatformMngt::ProcessDelegateWrapper::_duplicate (process_delegate_wrap);
            // ProcessDelegateWrap updated; no more search required
            // update m_wasProcessDelegateWrapReseted flag
            m_wasProcessDelegateWrapReseted = false;
        }
        
        m_starting_result = result;
        
        m_creation_error_info = CORBA::string_dup (error_info);
        m_creation_error_issuer = CORBA::string_dup (error_issuer);
        
        // signals the acknowledgement
        if (!m_process_start_cbk->signal())
        {
            // The acknowledgement has been performed too late
            throw CORBA::NO_PERMISSION(OrbSupport::NO_PERMISSION, CORBA::COMPLETED_NO);
        }
    }
    
    
    
    
    void ManagedProcess_impl::set_initialisation_result (
        CdmwPlatformMngt::ProcessCommandResult result)
        throw (CORBA::SystemException)
    {
        m_starting_result = result;
        
        // signals the acknowledgement
        if (!m_process_start_cbk->signal())
        {
            // The acknowledgement has been performed too late
            throw CORBA::NO_PERMISSION(OrbSupport::NO_PERMISSION, CORBA::COMPLETED_NO);
        }
    }
    
    
    void ManagedProcess_impl::set_next_step_result (
        CdmwPlatformMngt::ProcessCommandResult result,
        CORBA::ULong step_nbr)
        throw (CORBA::SystemException)
    {
        m_starting_result = result;
        m_step_nbr_done = step_nbr;
        
        // signals the acknowledgement
        if (!m_process_start_cbk->signal())
        {
            // The acknowledgement has been performed too late
            throw CORBA::NO_PERMISSION(OrbSupport::NO_PERMISSION, CORBA::COMPLETED_NO);
        }
    }
    
    
    void ManagedProcess_impl::set_run_result (
        CdmwPlatformMngt::ProcessCommandResult result)
        throw (CORBA::SystemException)
    {
        m_starting_result = result;
        
        // signals the acknowledgement
        if (!m_process_start_cbk->signal())
        {
            // The acknowledgement has been performed too late
            throw CORBA::NO_PERMISSION(OrbSupport::NO_PERMISSION, CORBA::COMPLETED_NO);
        }
    }
    
    
    void ManagedProcess_impl::set_stop_result (
        CdmwPlatformMngt::ProcessCommandResult result)
        throw (CORBA::SystemException)
    {
        m_stopping_result = result;
        
        // signals the acknowledgement
        if (!m_process_stop_cbk->signal())
        {
            // The acknowledgement has been performed too late
            throw CORBA::NO_PERMISSION(OrbSupport::NO_PERMISSION, CORBA::COMPLETED_NO);
        }
    }

    CdmwPlatformMngt::Children* ManagedProcess_impl::get_children()
        throw (CORBA::SystemException)
        {
            CdmwPlatformMngt::Children_var children = new CdmwPlatformMngt::Children;
            CORBA::ULong size_of_entities = get_number_of_entities();
            CORBA::ULong size_of_components = get_number_of_components();

            children->length(size_of_components + size_of_entities);
            
            CdmwPlatformMngt::CcmComponents_var components;
            CdmwPlatformMngt::ComponentsIterator_var components_iterator;

        get_all_components (size_of_components, components.out(), components_iterator.out());
        
            for (CORBA::ULong count = 0; count<  size_of_components; count++)
            {
                components[count] -> _add_ref();
                children[count] = (components[count])._retn();
            }

            CdmwPlatformMngt::Entities_var entities;
            CdmwPlatformMngt::EntitiesIterator_var entities_iterator;
            get_all_entities(size_of_entities, entities.out(), entities_iterator.out() );

            for (CORBA::ULong count = size_of_components; count< (size_of_components + size_of_entities); count++)
            {
                entities[count - size_of_components] -> _add_ref();
                children[count] = (entities[count - size_of_components])._retn();
            }
            return children._retn ();
        }

      void ManagedProcess_impl::initialise_components(const char * deployment_plan)
      {
      
      
          CDMW_INFO(Cdmw::Logging::Logger::Get_Logger(CdmwPlatformMngtBase::PFMMNG), "initialise_components  on " << name() << " deployment_plan " << deployment_plan);

          DeploymentManager::get_instance()->notify_process_components_initialisation(deployment_plan, m_proc_host_name.in());
 
          // Create the run starter
          std::auto_ptr< InitComponentCompletionCallback > completionCallback(
             new InitComponentCompletionCallback( this, deployment_plan) );

          std::auto_ptr< InitComponentSuspensionCallback > suspensionCallback(
             new InitComponentSuspensionCallback( this, deployment_plan ) );

           m_init_sync_task_factory = 
              InitComponentTaskFactory::createFactory(this, m_startupKind, m_component_init_graph, deployment_plan);

           m_init_sequencer = new Sequencer(m_init_sync_task_factory);

           m_init_sequencer->setSuspensionCallback(suspensionCallback.release());
           m_init_sequencer->setCompletionCallback(completionCallback.release());

           //start the sequencer  
           m_init_sequencer->start();

      }

      CdmwPlatformMngt::ProcessStatus ManagedProcess_impl::get_consistent_status()
      {
      
            CORBA::ULong size_of_components =  get_number_of_components();
            CdmwPlatformMngt::CcmComponents_var components;
            CdmwPlatformMngt::ComponentsIterator_var components_iterator;
            CdmwPlatformMngt::ProcessStatus ret = CdmwPlatformMngt::PROCESS_INITIALISED;
            
            get_all_components (size_of_components, components.out(), components_iterator.out());
            CORBA::String_var status_info;
            for (CORBA::ULong count = 0; count<  size_of_components; count++)
            {
                CdmwPlatformMngt::ComponentStatus status = components[count] -> get_status(status_info.out());
        
                if ( (CdmwPlatformMngt::COMPONENT_INITIAL != status) &&
                    (CdmwPlatformMngt::COMPONENT_INITIALISED != status) )
                {
            
                    ret = CdmwPlatformMngt::PROCESS_FAILED_ERROR;
                    break;
                }
            }
            return ret;
      }
      
      void ManagedProcess_impl::suspension_event(const std::string& deployment_plan)
      {
           std::string message = "suspension_event on ";
           message += name();
           message += " while trying to start Component Init Graph";
           LogMngr::logMessage(message.c_str());       
           //m_stateMachine->getState()->suspension_event(); 
 
           DeploymentManager::get_instance()->notify_process_component_suspension_event(deployment_plan, m_proc_host_name.in());

           CdmwPlatformMngtBase::EventHeader header
                = EventHeaderFactory::createHeader (CdmwPlatformMngtBase::INF);
           error_event(header, CdmwPlatformMngtBase::PFMMNG ,"component suspension event");
      }

      void ManagedProcess_impl::completion_event(const std::string& deployment_plan)
      {
          std::string message = "completion event on ";
          message += name();
          LogMngr::logMessage(message.c_str());
          DeploymentManager::get_instance()->notify_process_component_completion_event(deployment_plan, m_proc_host_name.in());
      }

      void ManagedProcess_impl::set_state(const std::string& state_name) throw()
      {
           CORBA::String_var statusInfo;
           CdmwPlatformMngt::ProcessStatus status = get_status (statusInfo.out());
           CDMW_SM_WRITER_LOCK_GUARD(*m_stateMachine);

           // Needed because otherwhise we could have
           // the status PROCESS_FAILURE =>  PROCESS_INITIALISED 
           if (CdmwPlatformMngt::PROCESS_INITIALISING == status)
               m_stateMachine->setState(state_name.c_str());
           set_record_in_datastore();
      }

      bool ManagedProcess_impl::is_preemption_active ()
      {
          CDMW_MUTEX_GUARD (m_preemption_mutex);

          return m_preemption_flag;
      }

      bool ManagedProcess_impl::activate_preemption ()
      {
          bool result = false;

          CDMW_MUTEX_GUARD (m_preemption_mutex);

          bool old_flag_value = m_preemption_flag;

          try
          {
             CORBA::String_var sysStatusInfo;

             CdmwPlatformMngt::System_var system = CdmwPlatformMngt::System::_narrow(m_application->get_parent());

             CdmwPlatformMngt::SystemStatus system_status = system->get_status(sysStatusInfo.out());

             CORBA::String_var applStatusInfo;

             CdmwPlatformMngt::ApplicationStatus appl_status =  m_application->get_status(applStatusInfo.out());

             CdmwPlatformMngt::HostStatus host_status = m_host->status();

             CORBA::String_var statusInfo;
             
             CdmwPlatformMngt::ProcessStatus status = get_status(statusInfo.out());
             
             if (system_status == CdmwPlatformMngt::SYSTEM_STARTING 
                   || system_status == CdmwPlatformMngt::SYSTEM_INITIAL 
                   || appl_status == CdmwPlatformMngt::APPLICATION_INITIAL
                   || appl_status == CdmwPlatformMngt::APPLICATION_INITIALISING
                   || appl_status == CdmwPlatformMngt::APPLICATION_INITIALISATION_SUSPENDED
                   || host_status == CdmwPlatformMngt::HOST_STARTING
                   || status == CdmwPlatformMngt::PROCESS_INITIAL 
                   || status == CdmwPlatformMngt::PROCESS_INITIALISING
                   || status == CdmwPlatformMngt::PROCESS_STEP_PERFORMED)
             {
                 m_preemption_flag = true;
                 result = true;   
             }

          }
          catch (...)
          {
              old_flag_value = m_preemption_flag;
              CDMW_ERROR(Cdmw::Logging::Logger::Get_Logger(CdmwPlatformMngtBase::PFMMNG), "  Can't activate preemption of " << name() << " process on host: " << m_host->name() << std::endl);
          }

          return result;
      }

      bool ManagedProcess_impl::de_activate_preemption ()
      {
          bool result = false;

          CDMW_MUTEX_GUARD (m_preemption_mutex);

          try 
          {
             CORBA::String_var sysStatusInfo;

             CdmwPlatformMngt::System_var system = CdmwPlatformMngt::System::_narrow(m_application->get_parent());

             CdmwPlatformMngt::SystemStatus system_status = system->get_status(sysStatusInfo.out());

             CORBA::String_var applStatusInfo;

             CdmwPlatformMngt::ApplicationStatus appl_status = m_application->get_status(applStatusInfo.out());

             CdmwPlatformMngt::HostStatus host_status = m_host->status();

             CORBA::String_var statusInfo;

             if (m_preemption_flag)
             {
                 if (system_status == CdmwPlatformMngt::SYSTEM_STARTING 
                       || system_status == CdmwPlatformMngt::SYSTEM_INITIAL 
                       || appl_status == CdmwPlatformMngt::APPLICATION_INITIAL
//                       || appl_status == CdmwPlatformMngt::APPLICATION_INITIALISING
                       || host_status == CdmwPlatformMngt::HOST_STARTING)
                 {

                      if (appl_status == CdmwPlatformMngt::APPLICATION_INITIAL && get_force_manual_start())
                      {
                         m_preemption_flag = false;
                         result = true;
                      }

                 }
                 else
                 {
                    m_preemption_flag = false;
                    result = true;
                 }
            }

          }
          catch (...)
          {

              CDMW_ERROR(Cdmw::Logging::Logger::Get_Logger(CdmwPlatformMngtBase::PFMMNG), "  Can't deactivate preemption of " << name() << " process on host: " << m_host->name() << std::endl);
              m_preemption_flag = false;
              result = true;
          }

          return result;
      }

      void ManagedProcess_impl::init_preemption ()
      {
         CDMW_MUTEX_GUARD (m_preemption_mutex);

         m_preemption_flag = false;
      }

    } // End namespace PlatformMngt
} // End namespace Cdmw

