/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

#include <platformapplicationlibrary/NodeDeploymentExecutor.hpp>
#include "SystemMngt/platformapplicationlibrary/DeploymentManager.hpp"
#include "SystemMngt/platformlibrary/LogMngr.hpp"
#include "Foundation/common/Exception.hpp"
#include "Foundation/orbsupport/ExceptionMinorCodes.hpp"


namespace Cdmw
{
  namespace PlatformMngt
  {
    
    NodeDeploymentExecutor::NodeDeploymentExecutor(DeploymentInfo deployment_info, 
                                                   CdmwDnC::NodeApplicationManager_ptr node_application_manager,
                                                   Cdmw::PlatformMngt::Application_impl *application)
    :m_all_completion_event_condition(m_mutex)
    ,m_deployment_ended_condition(m_mutex_to_start)
    {
        m_deployment_info= deployment_info;
        m_node_application_manager = CdmwDnC::NodeApplicationManager::_duplicate(node_application_manager);
        m_application = application;
        m_all_completion_event = 0;
        m_init = true;
        m_suspended = false;
        m_to_start = false;
        m_stop_timeout = CdmwPlatformMngt::DEPLOYMENT_PLANS_DEFAULT_TIMEOUT;
        m_init_timeout = CdmwPlatformMngt::DEPLOYMENT_PLANS_DEFAULT_TIMEOUT;
    }

    NodeDeploymentExecutor::~NodeDeploymentExecutor() throw()
    {
        CDMW_MUTEX_GUARD (m_mutex_run_stop);
      
      // Call run of Process with timeout
        try
        { 
            BoundSyncCallReturn result = EXCEPTION;
            BoundSyncCallFactory* factory = BoundSyncCallFactory::getFactory();

            // Run it within the timeout
            std::auto_ptr< DanceDestroyApplicationCall > danceDestroyApplicationCall(
                factory->createDanceDestroyApplicationCall(
                    m_node_application_manager.in(),
                    m_node_application.in(),
                    m_stop_timeout ) );

            result = danceDestroyApplicationCall->execute();            
            
            switch (result)
            {
                case EXCEPTION:
                                    
                    LogMngr::logMessage(ERR, (std::string("EXCEPTION ") + danceDestroyApplicationCall->getExceptionName()+ m_deployment_info.host_name + m_deployment_info.deployment_plan ).c_str());

                break;

                case TIMEOUT:
                    LogMngr::logMessage(WRN, (std::string("Timeout: ") + m_deployment_info.host_name +" "+ m_deployment_info.deployment_plan ).c_str());

              break;

              case SUCCEED:
  
                break;

              default:
                    CDMW_ASSERT(false);
            }
        }
        catch( const OutOfMemoryException& )
        {
            // Log the error
            LogMngr::logMessage( ERR, "Run out of memory" );
            // TODO: do an assert violation till a policy is decided
            CDMW_ASSERT( false );
        }      
        catch (const CdmwDnC::StopError & ex)
        {
            CDMW_ERROR(Cdmw::Logging::Logger::Get_Logger(CdmwPlatformMngtBase::PFMMNG), " node_application_manager->destroy_application CdmwDnC::StopError");
        }
        catch (const CdmwDnC::InvalidReference & ex)
        {
            CDMW_ERROR(Cdmw::Logging::Logger::Get_Logger(CdmwPlatformMngtBase::PFMMNG), " node_application_manager->destroy_application CdmwDnC::InvalidReference");
        }
        catch(const CORBA::SystemException& ex)
        {
            CDMW_ERROR(Cdmw::Logging::Logger::Get_Logger(CdmwPlatformMngtBase::PFMMNG), " node_application_manager->destroy_application CORBA::SystemException");
        }
        catch(...)
        {
            CDMW_ERROR(Cdmw::Logging::Logger::Get_Logger(CdmwPlatformMngtBase::PFMMNG), " ~NodeDeploymentExecutor() unknown exception ");
        }
    }

    void NodeDeploymentExecutor::stop() throw()
    {
        throw CORBA::NO_IMPLEMENT (OrbSupport::NO_IMPLEMENT, CORBA::COMPLETED_NO);
    } 

    void NodeDeploymentExecutor::run() throw()
    {
        try
        {
          execute(); 
        }
        catch(const CdmwDnC::StartError& start_error)
        {

            CDMW_ERROR(Cdmw::Logging::Logger::Get_Logger(CdmwPlatformMngtBase::PFMMNG), "CdmwDnC::StartError");
        
            //failed error on process
            if (m_application)
                m_application -> error_event_on_process_components(m_deployment_info);
        } 
       catch(const CdmwDnC::StopError& ex)
       {
            //failed error on process
            if (m_application)
                m_application -> error_event_on_process_components(m_deployment_info);
       }
       catch(const CORBA::SystemException& ex)
       {
            //failed error on process
            if (m_application)
                m_application -> error_event_on_process_components(m_deployment_info);
       }

    }

    void NodeDeploymentExecutor::execute() throw (CdmwDnC::StartError, CdmwDnC::StopError, CORBA::SystemException)
    {
      try{
      
      CDMW_MUTEX_GUARD (m_mutex_run_stop);

      try
      {
          CDMW_INFO(Cdmw::Logging::Logger::Get_Logger(CdmwPlatformMngtBase::PFMMNG), "calling start_launch on NodeApplicationManager on " << m_deployment_info.host_name << " and deployment plan:" << m_deployment_info.deployment_plan );

          m_node_application =  m_node_application_manager->start_launch();
      }
      catch(const CdmwDnC::StartError& start_error)
      {
           CDMW_ERROR(Cdmw::Logging::Logger::Get_Logger(CdmwPlatformMngtBase::PFMMNG), " start_launch:   CdmwDnC::StartError "<< m_deployment_info.deployment_plan.c_str() );  
           throw;
      }
   
      catch(const CORBA::SystemException& ex)
      {
           CDMW_FATAL(Cdmw::Logging::Logger::Get_Logger(CdmwPlatformMngtBase::PFMMNG), " start_launch: " << Cdmw::OrbSupport::OrbSupport::exception_to_string(ex).c_str());
       throw;
      }

      CDMW_INFO(Cdmw::Logging::Logger::Get_Logger(CdmwPlatformMngtBase::PFMMNG), "calling finish_launch on NodeApplication on " << m_deployment_info.host_name << " and deployment plan:" << m_deployment_info.deployment_plan);
   
      try
      {
          m_node_application->finish_launch();
      }
      catch(const CdmwDnC::StartError& start_error)
      {
          CDMW_ERROR(Cdmw::Logging::Logger::Get_Logger(CdmwPlatformMngtBase::PFMMNG), start_error._name() << " finish_launch: on"<< m_deployment_info.host_name << " and deployment plan:" <<m_deployment_info.deployment_plan.c_str() );
      }
      
      catch(const CORBA::SystemException& ex)
      {
          CDMW_ERROR(Cdmw::Logging::Logger::Get_Logger(CdmwPlatformMngtBase::PFMMNG), ex._name() << " finish_launch: on"<< m_deployment_info.host_name << " and deployment plan:" <<m_deployment_info.deployment_plan.c_str() );
      }
               
         
      CDMW_INFO(Cdmw::Logging::Logger::Get_Logger(CdmwPlatformMngtBase::PFMMNG), "  About to initialise components ");
     
      if (0 == m_application) 
      {
           CDMW_ERROR(Cdmw::Logging::Logger::Get_Logger(CdmwPlatformMngtBase::PFMMNG), " application has a NULL pointer "); 
           throw CdmwDnC::StartError();
      }
      try
      {
          CDMW_INFO(Cdmw::Logging::Logger::Get_Logger(CdmwPlatformMngtBase::PFMMNG), " initialise_process_components host_name: " << m_deployment_info.host_name << " deployment_plan: "<<m_deployment_info.deployment_plan);
          
          m_application -> initialise_process_components (m_deployment_info);

      }
      catch (...)
      {
          CDMW_ERROR(Cdmw::Logging::Logger::Get_Logger(CdmwPlatformMngtBase::PFMMNG), " exception while  initialising process components");
      }
            
      CDMW_INFO(Cdmw::Logging::Logger::Get_Logger(CdmwPlatformMngtBase::PFMMNG), "About to call start on node application on " << m_deployment_info.host_name << " and deployment_plan: "<<m_deployment_info.deployment_plan <<" waiting initialise is called on all components" );
      {
         
          CDMW_MUTEX_GUARD (m_mutex);

          while ((0 != m_all_completion_event)||(m_init))
          {
                m_all_completion_event_condition.wait();
          }

      }
      CDMW_INFO(Cdmw::Logging::Logger::Get_Logger(CdmwPlatformMngtBase::PFMMNG), "Calling start() on NodeApplication on " << m_deployment_info.host_name << " and deployment_plan: "<<m_deployment_info.deployment_plan);

      try
      {  
          if (!m_suspended){
      
      
      
              //m_node_application->start();
          
          
          
                      // Call start of node application with timeout
            try
            { 
                BoundSyncCallReturn result = EXCEPTION;
                BoundSyncCallFactory* factory = BoundSyncCallFactory::getFactory();
        
                // Initialise it within the timeout
                std::auto_ptr< NodeApplicationStartCall > nodeApplicationStartCall(
                    factory->createNodeApplicationStartCall(
                        m_node_application.in(),
                        m_init_timeout ) );
    
                result = nodeApplicationStartCall->execute();
                switch (result)
                {
                    case EXCEPTION:
            
                       // component_command_result = CdmwPlatformMngt::COMPONENT_CMD_EXCEPTION; 

                       //message="Exception in component ccm activate call: " +
                        //                m_component_name + ", " + initCall->getExceptionName();
                        //LogMngr::logMessage(ERR, message.c_str());
            CDMW_ERROR(Cdmw::Logging::Logger::Get_Logger(CdmwPlatformMngtBase::PFMMNG),  " NodeApplication start() ERROR :on " << m_deployment_info.host_name << " and deployment_plan: " <<  m_deployment_info.deployment_plan.c_str());
                       throw CdmwDnC::StartError();
             break;
    
                    case TIMEOUT:
                       // component_command_result = CdmwPlatformMngt::COMPONENT_CMD_TIMEOUT;
                      //  message="Timeout expired in component ccm activate call: " +
                        //                m_component_name;
                       // LogMngr::logMessage(ERR,  message.c_str());
               
            CDMW_ERROR(Cdmw::Logging::Logger::Get_Logger(CdmwPlatformMngtBase::PFMMNG), " NodeApplication start() TIMEOUT :on " << m_deployment_info.host_name << " and deployment_plan: " <<  m_deployment_info.deployment_plan.c_str());
                       throw CdmwDnC::StartError();
             break;
    
                    case SUCCEED:
                      //  component_command_result = CdmwPlatformMngt::COMPONENT_CMD_SUCCEED;
                       // message="Component successfully activated: " +
                                       // m_component_name;
                      //  LogMngr::logMessage(INF, message.c_str());
   
                        break;
                        
                    default:
                        CDMW_ASSERT(false);
                }
            }
            catch( const OutOfMemoryException& e )
            {
                // Log the error
                LogMngr::logMessage( ERR, "Run out of memory" );
                // TODO: do an assert violation till a policy is decided
                CDMW_ASSERT( false );
            }
          
          }
      }
      catch (const CdmwDnC::StartError& start_error)
      {
          CDMW_ERROR(Cdmw::Logging::Logger::Get_Logger(CdmwPlatformMngtBase::PFMMNG), start_error._name() << " NodeApplication start():on " << m_deployment_info.host_name << " and deployment_plan: " <<  m_deployment_info.deployment_plan.c_str());
          throw start_error;
      }
      catch(const CORBA::SystemException& ex)
      {
          CDMW_FATAL(Cdmw::Logging::Logger::Get_Logger(CdmwPlatformMngtBase::PFMMNG), ex._name() << " NodeApplication start():on " << m_deployment_info.host_name << " and deployment_plan: " <<  m_deployment_info.deployment_plan.c_str());
      }
      {
      
          CDMW_MUTEX_GUARD (m_mutex_to_start);

          m_to_start = false;
      
          m_deployment_ended_condition.broadcast();

      }
      }
      catch(...)
      {
      
          CDMW_MUTEX_GUARD (m_mutex_to_start);

          m_to_start = false;
      
          m_deployment_ended_condition.broadcast();
      
          throw;
          
      }
    }

    CdmwDnC::NodeApplicationManager_ptr NodeDeploymentExecutor::get_node_application_manager()
    {
      return CdmwDnC::NodeApplicationManager::_duplicate(m_node_application_manager.in()); 
    }


    void NodeDeploymentExecutor::increase_counter() throw()
    {
        CDMW_MUTEX_GUARD (m_mutex);
        m_all_completion_event++;
        m_init = false;
    }

    void  NodeDeploymentExecutor::decrease_counter() throw()
    {
        CDMW_MUTEX_GUARD (m_mutex);
        m_all_completion_event--;
        m_all_completion_event_condition.broadcast(); 
    }

    void NodeDeploymentExecutor::force_counter_to_zero() throw()
    {
        CDMW_MUTEX_GUARD (m_mutex);
        m_all_completion_event = 0;
        m_suspended = true;
        m_all_completion_event_condition.broadcast();
    }
    
    void NodeDeploymentExecutor::set_to_start() throw()
    {
        CDMW_MUTEX_GUARD (m_mutex_to_start);
        m_to_start = true;
    }
        
   
    void NodeDeploymentExecutor::wait_deployment(const std::string& application_name) throw()
    {
   
       if (strcmp (application_name.c_str(), m_application->get_element_name ()) == 0)
       {
       
           CDMW_MUTEX_GUARD (m_mutex_to_start);
       
           while (m_to_start){

           // m_mutex_to_start mutex is released and the thread waits in an atomic operation
               m_deployment_ended_condition.wait();

           }
       }
        
    }
    
    
       
    void NodeDeploymentExecutor::wait_deployment(const std::string& application_name, const std::string& process_name) throw()
    {
   
       if (strcmp (application_name.c_str(), m_application->get_element_name ()) == 0)
       {
       
           if (m_application->is_deployment_plan_related_to_process(m_deployment_info.deployment_plan.c_str(),process_name))
       {

               CDMW_MUTEX_GUARD (m_mutex_to_start);
 
               while (m_to_start)
           {
           
               // m_mutex_to_start mutex is released and the thread waits in an atomic operation
                   m_deployment_ended_condition.wait();

               }
           }
                
       }
    
    }
    
    void NodeDeploymentExecutor::set_start_deploy_timeout(CdmwPlatformMngt::Timeout init_tm)
    {
        m_init_timeout = init_tm;
    }

    void NodeDeploymentExecutor::set_stop_deploy_timeout(unsigned long stop_tm)
    {
        m_stop_timeout = stop_tm;
    }

    unsigned long NodeDeploymentExecutor::get_stop_deploy_timeout()
    {
       return m_stop_timeout;
    }

  }// closing namespaces
}
