
/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#include "Foundation/orbsupport/OrbSupport.hpp"
#include "Foundation/osthreads/Mutex.hpp"
#include "Foundation/osthreads/Condition.hpp"
#include "platformapplicationlibrary/ProcessInit.hpp"
#include "platformapplicationlibrary/Process_impl.hpp"
#include "platformapplicationlibrary/ManagedProcess_impl.hpp"
#include "SystemMngt/platformapplicationlibrary/Application_impl.hpp"
#include "SystemMngt/platformapplicationlibrary/ApplicationLogMessageId.hpp"
#include "SystemMngt/platformlibrary/GraphUtility.hpp"
#include "SystemMngt/platformlibrary/LogMngr.hpp"
#include "idllib/CdmwPlatformMngtPredefinedProperties.stub.hpp"

#include <sstream>


namespace Cdmw
{
namespace PlatformMngt
{

// ----------------------------------------------------------------------
// InitProcessSuspensionCallback class.
// ----------------------------------------------------------------------
InitProcessSuspensionCallback::InitProcessSuspensionCallback(
    Application_impl* application )
: Callback()
{
    m_application = application;
}

void InitProcessSuspensionCallback::execute() throw()
{
    m_application->suspension_event();
}

// ----------------------------------------------------------------------
// InitProcessCompletionCallback class.
// ----------------------------------------------------------------------
InitProcessCompletionCallback::InitProcessCompletionCallback(
    Application_impl* application )
: Callback()
{
    m_application = application;
}

void InitProcessCompletionCallback::execute() throw()
{
    // Check all the processes have been initialised
    GraphElementNames elementNames
        = GraphUtility::getElementNames( (m_application->m_processInitGraph).in() );

    GraphElementNames::iterator elementNameIt;
    for( elementNameIt =  elementNames.begin();
         elementNameIt != elementNames.end();
         elementNameIt++ )
    {
        Process_impl* process
            = m_application->getProcess( elementNameIt->c_str() );
            
        CdmwPlatformMngt::ProcessType processType
            = process->type();

        CdmwPlatformMngt::ProcessStatus processStatus
            = process->get_internal_status();
            
        CORBA::Boolean processAutoended = process->is_auto_ending();
        
        //CGi If the process is manual, it has to complete the event  
                
        CORBA::Any_var any_var = new CORBA::Any();
        CORBA::Boolean force_manual_start;
        try{
        process->get_attribute(CdmwPlatformMngt::FORCE_MANUAL_START, any_var.out(), CdmwPlatformMngt::PRE_DEFINED);
        any_var >>= CORBA::Any::to_boolean (force_manual_start);
        }catch(...) 
        { 
        CDMW_FATAL(Cdmw::Logging::Logger::Get_Logger(CdmwPlatformMngtBase::PFMMNG),
                   "Predefined property not found: " << CdmwPlatformMngt::FORCE_MANUAL_START);
        throw;
        }
        if( ((processType != CdmwPlatformMngt::UNMANAGED_PROCESS) && (!force_manual_start) &&
             (processStatus != CdmwPlatformMngt::PROCESS_INITIALISED)) ||
            ((processType == CdmwPlatformMngt::UNMANAGED_PROCESS) && (!processAutoended) && (!force_manual_start) &&
             (processStatus != CdmwPlatformMngt::PROCESS_RUNNING)) )
        {
            m_application->non_completion_event();
            return;
        }
    }

    m_application->completion_event();
}

// ----------------------------------------------------------------------
// InitProcessTask class.
// ----------------------------------------------------------------------
InitProcessTask::InitProcessTask(
    const std::string& taskId,
    Sequencer* sequencer,
    TaskObserver* observer,
    size_t predecessorCount,
    Process_impl* process,
    const CdmwPlatformMngtBase::StartupKind& startup_kind,
    unsigned int step )
throw( OutOfMemoryException )
: SynchronisableTask( taskId, sequencer, observer, predecessorCount )
{
                   
    try
    {
        m_process = process;
        m_startupKind = new CdmwPlatformMngtBase::StartupKind( startup_kind );
        m_step = step;
    }
    catch( const std::bad_alloc& )
    {
        CDMW_THROW( OutOfMemoryException );
    }
}

bool InitProcessTask::execute() throw()
{
   
    bool ret = false;
    const char* p_appliName = m_process->get_application()->get_element_name();
    const char* p_procKey = m_process->get_process_key();
           
    try
    {
        CdmwPlatformMngt::RequestStatus status;
        //CGi Added Control on manual starting of the Process
        CORBA::Any_var any_var = new CORBA::Any();
        CORBA::Boolean force_manual_start;
        m_process->get_attribute(CdmwPlatformMngt::FORCE_MANUAL_START, any_var.out(), CdmwPlatformMngt::PRE_DEFINED);
        any_var >>= CORBA::Any::to_boolean (force_manual_start);
        
        if (!force_manual_start) 
        {
            if( m_step > 0 )
            {
                LogMngr::logMessage(INF, MSG_ID_APP_PROC_NEXTSTEP,
                                    p_procKey,p_appliName);
                                
                ManagedProcess_impl* managedProcess
                    = dynamic_cast< ManagedProcess_impl* >( m_process );

                status = managedProcess->nextStep( m_step );
            }
            else
            {
                LogMngr::logMessage(INF, MSG_ID_APP_PROC_INIT,
                                    p_procKey,p_appliName);                
                status = m_process->initialise( m_startupKind.in() );
            }                       

            if( status == CdmwPlatformMngt::REQUEST_SUCCEED )
            {
                LogMngr::logMessage(INF, MSG_ID_APP_PROC_INIT_STATUS_OK,
                                    p_procKey,p_appliName);
                ret = true;             
            }
            
            //Check if this process is UNMANAGED with an initialisation_timeout != 0 
            else if( CdmwPlatformMngt::REQUEST_TIMEOUT == status && CdmwPlatformMngt::UNMANAGED_PROCESS == m_process->type() ) 
            {
                {                   
                    CDMW_MUTEX_GUARD(*m_process->m_init_mutex);                 
                    while ( m_process->m_lastRequestStatus != CdmwPlatformMngt::REQUEST_SUCCEED )
                    {
                        try
                        {
                            // Waiting for the RUNNING state after the initialisation_timeout
    
                            m_process->m_init_condition->wait();                            
                            
                        } 
                        catch (const InternalErrorException& e) 
                        {
                            // Shouldn't happen
                            CDMW_ASSERT(false);
                        }                   
                    }                   
                    ret=true;
                }
                m_process->m_timer->stop();
                m_process->m_timer->destroy_timeout_observer();
                delete m_process->m_init_condition;
                delete m_process->m_init_mutex;
                m_process->m_init_condition=0;              
                m_process->m_init_mutex=0;
                delete m_process->m_timer;
                m_process->m_timer=0;
            }
        }
        else 
        {
            LogMngr::logMessage(INF, MSG_ID_APP_PROC_INIT_NOT_EXECUTED,
                                p_procKey,p_appliName);
            ret = true;    
        }
    }
    catch (const CdmwPlatformMngt::IncompatibleStatus &)
    {
        LogMngr::logMessage(INF, MSG_ID_APP_PROC_INIT_STATUS_ERR,
                            p_procKey,p_appliName);
    }
    catch (const CdmwPlatformMngt::InvalidStep &)
    {
        LogMngr::logMessage(INF, MSG_ID_APP_PROC_INIT_STEP_ERR,
                            p_procKey,p_appliName);
    }
    catch (const CdmwPlatformMngt::IncompatibleType &)
    {
        LogMngr::logMessage(INF, MSG_ID_APP_PROC_INIT_TYPE_ERR,
                            p_procKey,p_appliName);
    }
    catch (const CORBA::SystemException& e)
    {
        std::ostringstream exceptionInfo;
        exceptionInfo << e;
        
        LogMngr::logMessage(INF, MSG_ID_APP_PROC_INIT_SYSTEM_ERR,
                            exceptionInfo.str().c_str(),
                            p_procKey,p_appliName);
    }
    catch(const CdmwPlatformMngt::AttributeNotFound&) 
    {
        CDMW_FATAL(Cdmw::Logging::Logger::Get_Logger(CdmwPlatformMngtBase::PFMMNG),
                   "Property not found: " << CdmwPlatformMngt::FORCE_MANUAL_START);
    }
    catch (...)
    {
        LogMngr::logMessage(INF, MSG_ID_APP_UNEXPECTED_EXCEPTION,
                            "InitProcessTask::execute()",
                            p_procKey,p_appliName);
    }
    return ret;
}

// ----------------------------------------------------------------------
// InitProcessPreconditionTask class.
// ----------------------------------------------------------------------

InitProcessPreconditionTask::InitProcessPreconditionTask(
        const int timeout,
        const std::string& managed_element,
        const std::string& managed_status,
        const std::string& taskId,
        Sequencer* sequencer,
        TaskObserver* observer,
        SynchronisableTaskFactory* synchTaskFactory,
        size_t predecessorCount,
        Process_impl* process,
        const CdmwPlatformMngtBase::StartupKind& startup_kind,
        unsigned int step) 
       : SynchronisablePreconditionTask(timeout,managed_element,managed_status,taskId,
                sequencer,observer,synchTaskFactory,predecessorCount,process->_this())
{
        
    try
    {
        m_process = process;
        m_startupKind = new CdmwPlatformMngtBase::StartupKind( startup_kind );
        m_step = step;
         
    }
    catch( const std::bad_alloc& )
    {
        CDMW_THROW( OutOfMemoryException );
    }
}
        
        
        

bool InitProcessPreconditionTask::execute() throw()
{
    bool ret = false;
    const char* p_appliName = m_process->get_application()->get_element_name();
    const char* p_procKey = m_process->get_process_key();

    try
    {
        CdmwPlatformMngt::RequestStatus status;
        //CGi Added Control on manual starting of the Process
        CORBA::Any_var any_var = new CORBA::Any();
        CORBA::Boolean force_manual_start;
        m_process->get_attribute(CdmwPlatformMngt::FORCE_MANUAL_START, any_var.out(), CdmwPlatformMngt::PRE_DEFINED);
        any_var >>= CORBA::Any::to_boolean (force_manual_start);
        
        if (!force_manual_start) 
        {
            if( m_step > 0 )
            {
                LogMngr::logMessage(INF, MSG_ID_APP_PROC_NEXTSTEP,
                                    p_procKey,p_appliName);
                                
                ManagedProcess_impl* managedProcess
                    = dynamic_cast< ManagedProcess_impl* >( m_process );

                status = managedProcess->nextStep( m_step );
            }
            else
            {
                LogMngr::logMessage(INF, MSG_ID_APP_PROC_INIT,
                                    p_procKey,p_appliName);                
                status = m_process->initialise( m_startupKind.in() );
            }                       

            if( status == CdmwPlatformMngt::REQUEST_SUCCEED )
            {
                LogMngr::logMessage(INF, MSG_ID_APP_PROC_INIT_STATUS_OK,
                                    p_procKey,p_appliName);
                ret = true;             
            }
            
            //Check if this process is UNMANAGED with an initialisation_timeout != 0 
            else if( CdmwPlatformMngt::REQUEST_TIMEOUT == status && CdmwPlatformMngt::UNMANAGED_PROCESS == m_process->type() ) 
            {
                {                   
                    CDMW_MUTEX_GUARD(*m_process->m_init_mutex);                 
                    while ( m_process->m_lastRequestStatus != CdmwPlatformMngt::REQUEST_SUCCEED )
                    {
                        try
                        {
                            // Waiting for the RUNNING state after the initialisation_timeout
    
                            m_process->m_init_condition->wait();                            
                            
                        } 
                        catch (const InternalErrorException& e) 
                        {
                            // Shouldn't happen
                            CDMW_ASSERT(false);
                        }                   
                    }                   
                    ret=true;
                }
                m_process->m_timer->stop();
                m_process->m_timer->destroy_timeout_observer();
                delete m_process->m_init_condition;
                delete m_process->m_init_mutex;
                m_process->m_init_condition=0;              
                m_process->m_init_mutex=0;
                delete m_process->m_timer;
                m_process->m_timer=0;
            }
        }
        else 
        {
            LogMngr::logMessage(INF, MSG_ID_APP_PROC_INIT_NOT_EXECUTED,
                                p_procKey,p_appliName);
            ret = true;    
        }
    }
    catch (const CdmwPlatformMngt::IncompatibleStatus &)
    {
        LogMngr::logMessage(INF, MSG_ID_APP_PROC_INIT_STATUS_ERR,
                            p_procKey,p_appliName);
    }
    catch (const CdmwPlatformMngt::InvalidStep &)
    {
        LogMngr::logMessage(INF, MSG_ID_APP_PROC_INIT_STEP_ERR,
                            p_procKey,p_appliName);
    }
    catch (const CdmwPlatformMngt::IncompatibleType &)
    {
        LogMngr::logMessage(INF, MSG_ID_APP_PROC_INIT_TYPE_ERR,
                            p_procKey,p_appliName);
    }
    catch (const CORBA::SystemException& e)
    {
        std::ostringstream exceptionInfo;
        exceptionInfo << e;
        
        LogMngr::logMessage(INF, MSG_ID_APP_PROC_INIT_SYSTEM_ERR,
                            exceptionInfo.str().c_str(),
                            p_procKey,p_appliName);
    }
    catch(const CdmwPlatformMngt::AttributeNotFound&) 
    {
        CDMW_FATAL(Cdmw::Logging::Logger::Get_Logger(CdmwPlatformMngtBase::PFMMNG),
                   "Property not found: " << CdmwPlatformMngt::FORCE_MANUAL_START);
    }
    catch (...)
    {
        LogMngr::logMessage(INF, MSG_ID_APP_UNEXPECTED_EXCEPTION,
                            "InitProcessTask::execute()",
                            p_procKey,p_appliName);
    }
    return ret;
}


// ----------------------------------------------------------------------
// InitProcessTaskFactory class.
// ----------------------------------------------------------------------
InitProcessTaskFactory::InitProcessTaskFactory(
    Application_impl* application,
    const CdmwPlatformMngtBase::StartupKind& startup_kind,
    const SynchronisableTaskDefMap& taskDefMap,
    const TaskIdList& rootTasks )
throw( OutOfMemoryException )
: SynchronisableTaskFactory( taskDefMap, rootTasks )
{
    try
    {
        m_application = application;
        m_startupKind = new CdmwPlatformMngtBase::StartupKind( startup_kind );
    }
    catch( const std::bad_alloc& )
    {
        CDMW_THROW( OutOfMemoryException );
    }
}

SynchronisableTaskFactory* InitProcessTaskFactory::createFactory(
    Application_impl* application,
    const CdmwPlatformMngtBase::StartupKind& startup_kind,
    const CdmwPlatformMngt::GraphElements& graphElements )
throw( OutOfMemoryException,
       CORBA::SystemException )
{
    try
    {
        SynchronisableTaskDefMap taskDefMap;
        TaskIdList rootTasks;
        
        /*
         * The last parameter set to false in the getTaskDefinitions method indicates 
         * the ignorePrecondition value.
         * It has a default true value, it is used to control the reading of the infos about
         * the precondition in the processes init-graph
         */
        GraphUtility::getTaskDefinitions(
            graphElements,
            false,
            true,
            taskDefMap,
            rootTasks,
            false); 
        
        std::auto_ptr<InitProcessTaskFactory> syncTaskFactory(
            new InitProcessTaskFactory(
                application, startup_kind, taskDefMap, rootTasks ) );

        return syncTaskFactory.release();
    }
    catch( const std::bad_alloc& )
    {
        CDMW_THROW( OutOfMemoryException );
    }
}

SynchronisableTask* InitProcessTaskFactory::createTask(
    const std::string& taskId,
    Sequencer* sequencer,
    TaskObserver* taskObserver )
throw( InvalidTaskIdException,
       OutOfMemoryException )
{
    
    const char* p_appliName = m_application->get_element_name();
    std::string processName;
    

    try
    {
        SynchronisableTaskDef syncTaskDef = getTaskDef( taskId );

        // Get the name and the step from the taskId
        
        unsigned int processStep;
        std::string temp;
        
        size_t separatorPos = taskId.find( "\\" );
        processName = taskId.substr( 0, separatorPos );
        temp=taskId.substr( separatorPos+1 );
        
        separatorPos = temp.find( "\\" );
        processStep = atol( (temp.substr(0,separatorPos)).c_str() );
        temp=temp.substr( separatorPos+1 );

        separatorPos = temp.find( "\\" );
        std::string precond_managed=temp.substr( 0,separatorPos );
        temp=temp.substr( separatorPos+1 );
        
        separatorPos = temp.find( "\\" );
        std::string precond_status=temp.substr( 0,separatorPos );
        temp=temp.substr( separatorPos+1 );
        
        separatorPos = temp.find( "\\" );
        int precond_timeout=atol(temp.substr( 0,separatorPos ).c_str());
        
        // Search for the process
        Process_impl* process
            = m_application->getProcess( processName.c_str() );
            
        if( "NONE" == precond_managed && "NONE" == precond_status && 0 == precond_timeout)
        {
         std::auto_ptr<InitProcessTask> syncTask(
            new InitProcessTask(
                taskId,
                sequencer,
                taskObserver,
                syncTaskDef.m_nbPredecessors,
                process,
                m_startupKind.in(),
                processStep ) );
        return syncTask.release();
        
        }else 
            
        {
            std::auto_ptr<InitProcessPreconditionTask> syncPrecondTask(
                        new InitProcessPreconditionTask(
                            precond_timeout,
                            precond_managed,
                            precond_status,
                            taskId,
                            sequencer,
                            taskObserver,
                            this,
                            syncTaskDef.m_nbPredecessors,
                            process,
                            m_startupKind.in(),
                            processStep ) );
            return syncPrecondTask.release();
        }
       
    }
    catch( const std::bad_alloc& )
    {
        CDMW_THROW( OutOfMemoryException );
    }
    catch( const CdmwPlatformMngt::ProcessNotFound& )
    {
        CDMW_THROW1( InvalidTaskIdException, taskId );
    }
    catch (...)
    {
        LogMngr::logMessage(INF, MSG_ID_APP_UNEXPECTED_EXCEPTION,
                            "InitProcessTaskFactory::createTask",
                            processName.c_str(),p_appliName);
                            
        CDMW_THROW1( InvalidTaskIdException, taskId );
    }
}

} // End namespace PlatformMngt
} // End namespace Cdmw

