
/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

#include "SystemMngt/platformelementlibrary/EntityContainer_impl.hpp"
#include "SystemMngt/platformelementlibrary/RWEntityContainer_impl.hpp"
#include "SystemMngt/platformelementlibrary/EntityContainer.hpp"

#include "Foundation/common/Exception.hpp"
#include "Foundation/common/Assert.hpp"
#include "Foundation/orbsupport/StrategyList.hpp"
#include "Foundation/commonsvcs/datastore/StorageErrorException.hpp"
#include "platformapplicationlibrary/RedundancyGroup_impl.hpp"
#include "SystemMngt/platforminterface/PlatformInterface.hpp"
#include "SystemMngt/platforminterface/ServiceNames.hpp"
#include "Foundation/orbsupport/ExceptionMinorCodes.hpp"
#include "SystemMngt/platformlibrary/Configuration.hpp"
#include "SystemMngt/platformlibrary/EventHeaderFactory.hpp"
#include "SystemMngt/platformlibrary/SupervisionEventNotifier.hpp"
#include "SystemMngt/platformlibrary/EventNotifierManager.hpp"

#include "idllib/CdmwPlatformMngtSupervisionObserver.skel.hpp"
#include "idllib/CdmwPlatformMngtPredefinedProperties.stub.hpp"
#include "Repository/repositoryinterface/RepositoryInterface.hpp"

#include "platformapplicationlibrary/CcmComponent_impl.hpp"
#include "SystemMngt/platformlibrary/ComponentsIterator_impl.hpp"
#include "SystemMngt/platformvaluetypes/RedundancyGroupStatusChange_impl.hpp"
#include "SystemMngt/platformlibrary/EventNotifierManager.hpp"


#include <iostream>


namespace Cdmw
{
  namespace PlatformMngt
  {
  
  static const std::string group_status_strings[] =
  {
          "Initial","FullyRedundant","DegradedRedundancy",
          "LostRedundancy","CompleteFailure"
  };

    RedundancyGroup_impl::RedundancyGroup_impl (PortableServer::POA_ptr poa,
                                                CdmwPlatformMngt::
                                                RWElementRepository_ptr
                                                element_repository):
                                                ManagedElement_impl(element_repository, poa)
    {
        m_logger = Cdmw::Logging::Logger::Get_Logger(CdmwPlatformMngtBase::PFMMNG);
            
        //Initialising the class variables
        m_group_type = CdmwPlatformMngt::FT_GROUP_TYPE;
        m_minimum_nb_of_replicas = 0;
        m_group_status =  CdmwPlatformMngt::INITIAL;
        std::string ele_path = get_element_path ();
        m_replication_style = CdmwPlatformMngt::RG_STATELESS;
        m_balancing_style =  CdmwPlatformMngt::LB_RANDOM;
        m_components= 0;
        m_component_map.clear();
        set_status_info ( CORBA::string_dup (""));
        m_redgrStatusChangeFactory= 0;
        
        CDMW_INFO(m_logger, ele_path << " TECH STATUS: INITIAL" );
            
        // set in Managed Element the instance of Property Event Notifier Thread
        set_property_event_notifier(EventNotifierManager::Instance()->get_property_notifier());
        // set in Managed Element the instance of Status Event Notifier Thread
        set_status_event_notifier(EventNotifierManager::Instance()->get_status_notifier());
        
        //add pre-defined properties ...
        //Setting up the Group size property
        std::ostringstream property_name;
        CORBA::Any_var any_val = new CORBA::Any();
        try
        {

            *any_val <<= (CORBA::Long) 0;
            property_name.str("");
            property_name << CdmwPlatformMngt::CDMW_PREFIX << CdmwPlatformMngt::GROUP_SIZE<< std::ends;                
            define_property_with_mode(property_name.str().c_str(), 
                    any_val.in(), 
                    CosPropertyService::fixed_normal);

        }
        catch(const CdmwPlatformMngt::AttributeNotFound&) 
        {
            CDMW_FATAL(m_logger,
                    "Property not found: " << CdmwPlatformMngt::GROUP_SIZE);
        }
        catch (...)
        {
            CDMW_FATAL(m_logger,
                            "Multiple Exceptions " << CdmwPlatformMngt::GROUP_SIZE);
            throw CosPropertyService::MultipleExceptions ();
        }
        
        try{
            // define TECH_STATUS and TECH_STATUS_INFO properties
            const char* init_value = group_status_strings[0].c_str();
            define_status_property(init_value);
        }
        catch(...)
        {
            throw  CosPropertyService::MultipleExceptions();
        }
        
        try
        {
            Cdmw::CommonSvcs::DataStore::DataStoreBase * ds =
                Cdmw::CommonSvcs::DataStore::DataStoreRegistry::
                    Get_data_store (Configuration::Get_redundancy_group_datastore_id ());
            m_redundancy_group_ds =
                dynamic_cast < Cdmw::PlatformMngt::RedundancyGroupDataStore * >(ds);

        }
        catch (const NotExistException & e)
        {
            CDMW_FATAL(m_logger,
                                "RedundancyDatastore Not Found");
            CDMW_ASSERT (false);
        }
        
        try 
        {
            // Factory for group status change
            std::auto_ptr < RedundancyGroupStatusChangeFactory >
            redgrStatusChangeFactory (new RedundancyGroupStatusChangeFactory ());

            m_redgrStatusChangeFactory = redgrStatusChangeFactory.get ();
            
            // Release the ownership of the temporary objects
            redgrStatusChangeFactory.release ();
        }
        catch (const std::bad_alloc&)
        {
            throw CORBA::NO_MEMORY(OrbSupport::NO_MEMORY, CORBA::COMPLETED_NO);
        }
             
    }

    
    RedundancyGroup_impl::~RedundancyGroup_impl()
    {
        if (m_redgrStatusChangeFactory != 0)
        {
            m_redgrStatusChangeFactory->_remove_ref ();
        }
    }

    CORBA::ULong RedundancyGroup_impl::get_number_of_entities () throw (CORBA::
                                                                        SystemException)
    {
    return m_rwEntityContainer->get_number_of_entities();

    }


    void RedundancyGroup_impl::get_all_entity_names (CORBA::ULong how_many,
                                                     CdmwPlatformMngt::
                                                     ElementNames_out entity_names,
                                                     CdmwPlatformMngt::
                                                     ElementNamesIterator_out rest)
      throw (CORBA::SystemException)
    {
        m_rwEntityContainer->get_all_entity_names(
            how_many, entity_names, rest );
    }


    CdmwPlatformMngt::Entity *
      RedundancyGroup_impl::get_entity (const char * entity_name) throw (  CdmwPlatformMngt::EntityNotFound, 
                                                                            CORBA::SystemException)
    {
      //std::cout << "RedundancyGroup_impl::get_entity" << std::endl;
      return m_rwEntityContainer->get_entity (entity_name);
    }

    CORBA::Boolean RedundancyGroup_impl::get_entities (const CdmwPlatformMngt::
                                                       ElementNames & entity_names,
                                                       CdmwPlatformMngt::
                                                       Entities_out nentities )
      throw (CORBA::SystemException)
    {
      return m_rwEntityContainer->get_entities(
            entity_names, nentities );
    }

    void RedundancyGroup_impl::get_all_entities (CORBA::ULong how_many,
                                                 CdmwPlatformMngt::Entities_out nentities,
                                                 CdmwPlatformMngt::
                                                 EntitiesIterator_out rest)
      throw (CORBA::SystemException)
    {
     m_rwEntityContainer->get_all_entities(
                   how_many, nentities, rest );
    }


    CdmwPlatformMngt::Entity *
      RedundancyGroup_impl::add_entity (const CdmwPlatformMngt::
                                        EntityDef & ent_def) throw (CORBA::
                                                            SystemException)
    {
    CdmwPlatformMngt::Entity_ptr ent_ptr =
            m_rwEntityContainer->add_entity(ent_def);

    std:: stringstream str;

    std::string buffer(str.str());

    CDMW_INFO(m_logger ,
                                                 buffer) 

    return ent_ptr ;

    }

    void RedundancyGroup_impl::remove_entity (const char *) throw ( CdmwPlatformMngt::EntityNotFound, CORBA::
                                                                   SystemException)
    {
      throw CORBA::NO_IMPLEMENT (OrbSupport::NO_IMPLEMENT, CORBA::COMPLETED_NO);
    }

    void RedundancyGroup_impl::get_attribute (const char *attr_name, 
                                              CORBA::Any_out ret_attr,
                                              CdmwPlatformMngt::
                                              Owner flag) throw (CORBA::SystemException,
                                              CdmwPlatformMngt::AttributeNotFound)
    {
        std::string tmp1 (attr_name);
        if ( CdmwPlatformMngt::PRE_DEFINED == flag )
        {
            // Case of predefined property
            std::ostringstream property_name;
            property_name << CdmwPlatformMngt::CDMW_PREFIX << attr_name << std::ends;
            try
            {
                CORBA::Any_var outany =
                    get_property_value ( property_name.str().c_str() );
                ret_attr = outany._retn();
            }
            catch (const CosPropertyService::PropertyNotFound&)
            {
                throw CdmwPlatformMngt::AttributeNotFound ();

            }
        }
        else
        {
            /*case of user defined attribute */
            try
            {
                CORBA::Any * outany = get_property_value (attr_name);
                ret_attr = outany;
            }
            catch (const CosPropertyService::PropertyNotFound&)
            {
                throw CdmwPlatformMngt::AttributeNotFound ();
            }
        }
    }



    void RedundancyGroup_impl::set_attribute (const char *attr_name, 
                                              const CORBA::Any& attr,
                                              CdmwPlatformMngt::Owner flag)
                                              throw (CORBA::SystemException,
                                                  CdmwPlatformMngt::AttributeNotFound,
                                                  CdmwPlatformMngt::ReadOnlyAttribute)

    {

       CORBA::Any_var myany = new CORBA::Any();
        *myany = attr; 
        
        if ( flag == CdmwPlatformMngt::PRE_DEFINED )
        {
            std::string tmp1 (attr_name);
            // check if the predefined property is readonly 
            if ((tmp1 == CdmwPlatformMngt::GROUP_SIZE)  ||
                    (tmp1 == CdmwPlatformMngt::TECH_STATUS) ||
                    (tmp1 == CdmwPlatformMngt::TECH_STATUS_INFO))
            {
                throw CdmwPlatformMngt::ReadOnlyAttribute(); 
            }
        

        }

        try
        {
            set_attribute_internal ( attr_name, myany.in(), flag ); 
        }
    catch (const Cdmw::BadOrderException& ex)
        {
        throw CORBA::BAD_PARAM(OrbSupport::BAD_PARAM, CORBA::COMPLETED_NO);
        }
    catch (...)
    {
            throw;
        }


    }



    CdmwPlatformMngt::ElementType RedundancyGroup_impl::
      get_element_type ()throw (CORBA::SystemException)
    {
      throw CORBA::NO_IMPLEMENT (OrbSupport::NO_IMPLEMENT, CORBA::COMPLETED_NO);
    }


    CdmwPlatformMngt::Children *
      RedundancyGroup_impl::get_children ()throw (CORBA::SystemException)
    {
        CdmwPlatformMngt::Children_var children = new CdmwPlatformMngt::Children;
        CORBA::ULong size_of_entities = get_number_of_entities();

    children->length(size_of_entities);

    CdmwPlatformMngt::Entities_var entities;
        CdmwPlatformMngt::EntitiesIterator_var entities_iterator;
        get_all_entities(size_of_entities, entities.out(), entities_iterator.out() );
            
        for (CORBA::ULong count = 0 ; count< size_of_entities; count++)
        {
           entities[count] -> _add_ref();
           children[count] = (entities[count])._retn();
        }
        return children._retn ();

    }

    CORBA::Long RedundancyGroup_impl::get_minimum_size ()throw (CORBA::
                                                                SystemException)
    {
      return m_minimum_nb_of_replicas;
    }


    char *RedundancyGroup_impl::group_domain_id () throw (CORBA::
                                                          SystemException)
    {
      throw CORBA::NO_IMPLEMENT (OrbSupport::NO_IMPLEMENT, CORBA::COMPLETED_NO);
    }

     
     CdmwPlatformMngt::TechnicalStatus RedundancyGroup_impl::get_status(CORBA::String_out) throw (CORBA::SystemException)
      {
         //Return the technical Status 
     CDMW_READER_LOCK_GUARD (m_rwGroupStatusLock);
     
         return m_group_status;
      }
      

    void RedundancyGroup_impl::set_minimum_size (CORBA::Long) throw (CORBA::
                                                                     SystemException)
    {
      throw CORBA::NO_IMPLEMENT (OrbSupport::NO_IMPLEMENT, CORBA::COMPLETED_NO);
    }

    void RedundancyGroup_impl::start_up () throw (CORBA::SystemException)
    {
      throw CORBA::NO_IMPLEMENT (OrbSupport::NO_IMPLEMENT, CORBA::COMPLETED_NO);
    }

    void RedundancyGroup_impl::shut_down () throw (CORBA::SystemException)
    {
      throw CORBA::NO_IMPLEMENT (OrbSupport::NO_IMPLEMENT, CORBA::COMPLETED_NO);
    }

    void RedundancyGroup_impl::insert (CdmwPlatformMngt::
                                       ManagedElement *) throw (CORBA::
                                                                SystemException)
    {
      throw CORBA::NO_IMPLEMENT (OrbSupport::NO_IMPLEMENT, CORBA::COMPLETED_NO);
    }

    void RedundancyGroup_impl::get_all_components (CORBA::ULong how_many,
                                                   CdmwPlatformMngt::
                                                   CcmComponents_out ncomponents,
                                                   CdmwPlatformMngt::
                                                   ComponentsIterator_out rest)
      throw (CORBA::SystemException)
    {
        // create the requested sequence of components
        CdmwPlatformMngt::CcmComponents_var requested_components = 
            new CdmwPlatformMngt::CcmComponents;

        std::list <CdmwPlatformMngt::CcmComponent_var> rest_of_components;

        // get length of list                                    
        size_t len = m_component_map.size();

        if (len > 0)
        {
            // set length of sequence
            //requested_components->length(len);

            // initialize list iterator
            ComponentsMap::iterator it =
                m_component_map.begin();


            for (CORBA::ULong i = 0; i < len; i++)
            {
                // get entity from container list using iterator
                // and convert to Component object reference
                CdmwPlatformMngt::CcmComponent_var component = ((it->second)->component)->_this();
                
                // store entity in sequence
                if( i < how_many)
                {
                    requested_components->length(i+1) ;
                    requested_components[i] = component._retn();
                }
                else
                    rest_of_components.push_back(component) ;

                // increment iterator
                it++;
            }
        }

        // create the EntityIterator with the rest of list
        CdmwPlatformMngt::ComponentsIterator_var components_iter =
            ComponentsIterator_impl::create (rest_of_components);
        
        ncomponents = requested_components._retn() ;
        rest = components_iter._retn();
        
    }


    CdmwPlatformMngt::CcmComponent *
      RedundancyGroup_impl::get_component (const char* component_name,
                                           const char* host_name) throw (CORBA::SystemException,
                                                                CdmwPlatformMngt::ComponentNotFound)
    {
        
        //check if the name is in the map
        std::string full_component_name=component_name;
        full_component_name.append("/") ; 
        full_component_name.append(host_name);
        
        ComponentsMap::iterator ccmcomponentIt =
            m_component_map.find (full_component_name);

        if (ccmcomponentIt == m_component_map.end ())
            throw CdmwPlatformMngt::ComponentNotFound();
        
        return (ccmcomponentIt->second->component)->_this();
        
    }

    CORBA::ULong RedundancyGroup_impl::get_number_of_components ()throw (CORBA::
                                                                         SystemException)
    {
      return m_component_map.size();
    }

    void RedundancyGroup_impl::_cxx_delete (CdmwPlatformMngt::
                                            ManagedElement *) throw (CORBA::
                                                                     SystemException)
    {
      throw CORBA::NO_IMPLEMENT (OrbSupport::NO_IMPLEMENT, CORBA::COMPLETED_NO);
    }

    void RedundancyGroup_impl::set_rw_entity_container( 
                  RWEntityContainer_impl* rwEntityContainer)
    {

    m_rwEntityContainer = rwEntityContainer ;
    }
    

    void RedundancyGroup_impl::set_group_type( const CdmwPlatformMngt::ElementType& group_type)
    {
        m_group_type= group_type;
    }
    

    void RedundancyGroup_impl::set_minimum_nbr_of_replicas( const long& minimum_nb_of_replicas)
   {
        m_minimum_nb_of_replicas= minimum_nb_of_replicas;
    }
    

    void RedundancyGroup_impl::set_replication_style( const CdmwPlatformMngt::ReplicationStyle& replication_style) 
    {
        m_replication_style= replication_style;
    }
    

    void RedundancyGroup_impl::set_balancing_style( const CdmwPlatformMngt::BalancingStyle& balancing_style) 
    {
        m_balancing_style= balancing_style;
    }
    
    
    CdmwPlatformMngt::ReplicationStyle RedundancyGroup_impl::get_replication_style()
    {
        return m_replication_style;
    }

    
    CdmwPlatformMngt::BalancingStyle RedundancyGroup_impl::get_balancing_style()
    {
        return m_balancing_style;
    }
    
    
    void RedundancyGroup_impl::set_record_in_datastore ()
    {
        // test if the supervision is not fault tolerant or if it is, is it the primary?
        if (!Configuration::Is_fault_tolerant ()
        || Configuration::Is_primary ())
        {
            if (Configuration::Is_synchronising ())
            {
                CDMW_INFO(m_logger,"<<RedundancyGroup_impl_impl::set_record_in_datastore() start >> "
                    << m_element_name.in () );
            }
            CdmwPlatformMngt::RedundancyGroupRecord record;         
            // All the parameters must be set
            record.group_type=m_group_type;
            record.minimum_nb_of_replicas=m_minimum_nb_of_replicas;
            record.group_status=m_group_status;
            record.replication_style=m_replication_style;
            record.balancing_style=m_balancing_style;
            record.components=m_components;
        
            string ds_operation("contains");
                        
            try
            {
                if (m_redundancy_group_ds->contains (m_element_path.in ()))
                {
                    ds_operation = "update";
                    m_redundancy_group_ds->update (m_element_path.in (), record);
                }
                else
                {
                    set_element_in_datastore ();
                    ds_operation = "insert";
                    m_redundancy_group_ds->insert (m_element_path.in (), record);
                }

            }
            catch (const Common::AlreadyExistException&)
            {
                CDMW_WARN(m_logger,
                "record already exists exception in " 
                << ds_operation << " : " 
                << m_element_path.in () << ":"
                << (const char*) record.group_status);
            }
            catch (const CommonSvcs::DataStore::StorageErrorException &exc_storage)
            {
                CDMW_FATAL(m_logger,
                "record storage error exception in "
                << ds_operation <<" : " 
                << m_element_path.in() << ":"
                << (const char*) record.group_status << ":" 
                << exc_storage.what());
                CDMW_ASSERT (false);
            }
            catch (const Common::NotFoundException&)
            {
                CDMW_FATAL(m_logger,
                "record cannot be found in datastore in "
                << ds_operation <<" : " 
                << m_element_path.in());
                CDMW_ASSERT (false);
            }
            catch (...)
            {
                CDMW_FATAL(m_logger,
                "record unknown exception in "
                << ds_operation << " : " 
                << m_element_path.in() << ":"
                << (const char*) record.group_status);
                CDMW_ASSERT (false);
            }
            
            if (Configuration::Is_synchronising ())
            {
                CDMW_INFO (m_logger, "<<RedundancyGroup_impl::set_record_in_datastore() done >> "
                    << m_element_name.in () );
            }
        }
    }

    void RedundancyGroup_impl::synchro_entities_with_datastore()
    {
       CDMW_INFO (m_logger, "synchro_entities_with_datastore");
    }  
    
    void RedundancyGroup_impl::synchro_with_datastore ()
    {

        //std::cout<< " RedundancyGroup_impl : RedundancyDatastore synchronization start \n";
        Cdmw::PlatformMngt::ManagedElement_impl::synchro_with_datastore ();
        CdmwPlatformMngt::RedundancyGroupRecord_var redundancy_group_record = get_record_from_datastore ();
        
        m_group_type = redundancy_group_record->group_type;
        m_minimum_nb_of_replicas = redundancy_group_record->minimum_nb_of_replicas;
        m_group_status = redundancy_group_record->group_status;
        m_replication_style = redundancy_group_record->replication_style;
        m_balancing_style = redundancy_group_record->balancing_style;
        m_components = redundancy_group_record->components;
        synchro_entities_with_datastore();
        //std::cout<< " RedundancyGroup_impl : RedundancyDatastore synchronization done \n";
    }


    void RedundancyGroup_impl::remove_record_in_datastore ()
    {
        // inhibition if backup
        if (!Configuration::Is_fault_tolerant ()
        || Configuration::Is_primary ())
        {
            try
            {
                m_redundancy_group_ds->remove (m_element_path.in ());
            }
            catch (const Common::NotFoundException &)
            {
            }
            catch (const CommonSvcs::DataStore::StorageErrorException &)
            {
            }
        }
    }


    CdmwPlatformMngt::RedundancyGroupRecord*  RedundancyGroup_impl::get_record_from_datastore ()
    {
        CdmwPlatformMngt::RedundancyGroupRecord*  redundancy_group_record = NULL;

        try
        {
            redundancy_group_record = m_redundancy_group_ds->select (m_element_path.in ());
        }
        catch (const Common::NotFoundException &)
        {   
            CDMW_FATAL(m_logger,
                    "redundancy_group record cannot be found in datastore : "
                    << m_element_path.in());
            CDMW_ASSERT (false);
        }

        return redundancy_group_record;
    }    
    
    void RedundancyGroup_impl::add_component(
            std::string full_component_name,
            Cdmw::PlatformMngt::CcmComponent_impl* component,
            const CdmwPlatformMngt::ComponentDef& ccmcomponent_def ) throw (CORBA::SystemException,CdmwPlatformMngt::ComponentAlreadyExists)
    {
        ComponentsMap::iterator ccmcomponentIt =
            m_component_map.find (full_component_name);
        
        //Inserting the component in to the map
        if (ccmcomponentIt != m_component_map.end ())
            throw CdmwPlatformMngt::ComponentAlreadyExists();
        
    ComponentData* temp = new ComponentData() ;
    temp->component= component;
    temp->status= false;
    
        m_component_map[full_component_name]=temp;
       
        // Current nunmber of components contained in the group
        CORBA::ULong component_number=m_components.length();                
  
        m_components.length(++component_number);
        //Adding the new component to the sequence;
        m_components[component_number-1]=ccmcomponent_def;
 
        set_record_in_datastore();
     }
    
    void RedundancyGroup_impl::update_component_status(const std::string& full_component_name,const char* component_status) 
    throw (CORBA::SystemException,CdmwPlatformMngt::ComponentNotFound)
    {
        //check if the name is in the map
        ComponentsMap::iterator ccmcomponentIt =
             m_component_map.find (full_component_name);
       
        if (ccmcomponentIt == m_component_map.end ())
           throw CdmwPlatformMngt::ComponentNotFound();


        if ( std::string("Initialised") == std::string(component_status) )
        {
            ccmcomponentIt->second->status = true;
        update_group_status();
        }
        else if ( std::string("Stopping") ==  std::string(component_status) ||
                  std::string("Stopped") ==   std::string(component_status) ||
                  std::string("FailedError") ==  std::string(component_status) ||
                  std::string("FailedTimeout") ==  std::string(component_status) ||
                  std::string("FailedInvalid") ==  std::string(component_status) )
        {
            ccmcomponentIt->second->status = false ;
        update_group_status();
        }
    }   
    
    
    void RedundancyGroup_impl::update_group_status()
    {
        CDMW_WRITER_LOCK_GUARD (m_rwGroupStatusLock);
    
    long count_active_component = 0 ;

        //Calculating the number of active components
        for(ComponentsMap::iterator It = m_component_map.begin();It != m_component_map.end(); ++It )
        {
            if( true == It->second->status ) count_active_component++;       
        }

        //Updating the property GROUP_SIZE with the current value
        CORBA::Any_var any_var = new CORBA::Any();
        try
        {           
            CORBA::Long size = count_active_component;
            *any_var <<= size;    
            set_attribute_internal (CdmwPlatformMngt::GROUP_SIZE, any_var.in(), CdmwPlatformMngt::PRE_DEFINED);
        }
        catch(const CdmwPlatformMngt::AttributeNotFound&) 
        {
            CDMW_FATAL(m_logger,
                    "Property not found: " << CdmwPlatformMngt::GROUP_SIZE);
        }               
        
        std::string ele_path =  get_element_path ();
        CORBA::Any_var status_value = new CORBA::Any();
        
    //Updating the Technical Status
    if(  count_active_component >= m_minimum_nb_of_replicas )
    {
        m_group_status =  CdmwPlatformMngt::FULLY_REDUNDANT;
        *status_value <<= CORBA::string_dup(group_status_strings[1].c_str());
        CDMW_INFO(m_logger, ele_path << " TECH STATUS: FULLY_REDUNDANT" );
    }
        else if(  count_active_component < m_minimum_nb_of_replicas &&  count_active_component > 1)
    {
       m_group_status =  CdmwPlatformMngt::DEGRADED_REDUNDANCY;
       *status_value <<= CORBA::string_dup(group_status_strings[2].c_str());
           CDMW_INFO(m_logger, ele_path << " TECH STATUS: DEGRADED_REDUNDANCY" );
    }
        else if(  1 == count_active_component )
    {  
        m_group_status =  CdmwPlatformMngt::LOST_REDUNDANCY;
        *status_value <<= CORBA::string_dup(group_status_strings[3].c_str());
        CDMW_INFO(m_logger, ele_path << " TECH STATUS: LOST_REDUNDANCY" );       
    }
        else if(  0 == count_active_component )
    {  
        m_group_status =  CdmwPlatformMngt::COMPLETE_FAILURE;
        *status_value <<= CORBA::string_dup(group_status_strings[4].c_str());
        CDMW_INFO(m_logger, ele_path << " TECH STATUS: COMPLETE_FAILURE" );      
    }
    
    //Updating TECH_STATUS property
    try
    {           
        set_attribute_internal(CdmwPlatformMngt::TECH_STATUS,
                status_value.in(), CdmwPlatformMngt::PRE_DEFINED);
    }
    catch(const CdmwPlatformMngt::AttributeNotFound&) 
    {
        CDMW_FATAL(m_logger,
                "Property not found: " << CdmwPlatformMngt::GROUP_SIZE);
    }
    
        set_record_in_datastore();
        
        // notify group status change
        notifyStatusChangeEvent();
        
    }
    
    
    void RedundancyGroup_impl::notifyStatusChangeEvent()
    {
        try
        {        
            // Create the event
            CdmwPlatformMngtBase::EventHeader header
                = EventHeaderFactory::createHeader (CdmwPlatformMngtBase::INF);
            
            CdmwPlatformMngt::Application_var appl = 
                CdmwPlatformMngt::Application::_narrow(get_parent()) ;
            
            CdmwPlatformMngt::RedundancyGroupStatusChange_var event
                = m_redgrStatusChangeFactory->create (header,
                appl->get_parent()->name(),
                name(),
                m_group_status,
                m_group_type,
                appl->name(),
                get_status_info());
            
            CORBA::String_var eventStr = event->to_string ();
            LogMngr::logMessage (eventStr.in ());

            // Perform the status change notification 
            // to supervision event notifier
            event->_add_ref(); 
            m_eventNotifier = EventNotifierManager::Instance()->get_sup_notifier();
            m_eventNotifier->addEvent(event);
            
            // Perform the status change notification to observer registered
            // with the Api in ManagedElement
            event->_add_ref(); 
            get_status_event_notifier()->add_event (event);
            
            event->_remove_ref();
            event._retn ();
        }
        catch (const std::bad_alloc &)
        {
            throw CORBA::NO_MEMORY (OrbSupport::NO_MEMORY, CORBA::COMPLETED_NO);
        }
        catch (...)
        {
            throw CORBA::INTERNAL (OrbSupport::INTERNAL, CORBA::COMPLETED_NO);
        }
    }

  }             // End namespace PlatformMngt
}               // End namespace Cdmw


