/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#include "platformapplicationlibrary/UnmanagedProcess_impl.hpp"
#include "Foundation/orbsupport/ExceptionMinorCodes.hpp"
#include "Foundation/osthreads/MutexGuard.hpp"
#include "SystemMngt/platformlibrary/LogMngr.hpp"
#include "SystemMngt/platformlibrary/SupervisionEventNotifier.hpp"
#include "SystemMngt/platformvaluetypes/ProcessStatusChange_impl.hpp"
#include "SystemMngt/idllib/CdmwPlatformMngtEvent.stub.hpp"
#include "SystemMngt/platformapplicationlibrary/Application_impl.hpp"
#include "SystemMngt/platformapplicationlibrary/ApplicationLogMessageId.hpp"
#include "platformapplicationlibrary/HostProcessInitSequencer.hpp"
#include "SystemMngt/idllib/CdmwPlatformMngtPredefinedProperties.stub.hpp"
#include "SystemMngt/tools/ProcessIssuerManager.hpp"
#include "SystemMngt/idllib/CdmwPlatformMngtSystem.stub.hpp"

namespace Cdmw
{
    namespace PlatformMngt
    {
        
        UnmanagedProcessInitial* UnmanagedProcess_impl::M_initial = NULL;
        UnmanagedProcessInitialising* UnmanagedProcess_impl::M_initialising = NULL;
        UnmanagedProcessRunning* UnmanagedProcess_impl::M_running = NULL;
        UnmanagedProcessStopped* UnmanagedProcess_impl::M_stopped = NULL;
        UnmanagedProcessEnded* UnmanagedProcess_impl::M_ended = NULL;
        UnmanagedProcessFailedDeath* UnmanagedProcess_impl::M_failedDeath = NULL;
        UnmanagedProcessFailedInvalid* UnmanagedProcess_impl::M_failedInvalid = NULL;
        
        
        static const std::string process_status_strings[] =
        {
                "Initial","Initialising","Running","Stopping","Stopped",
                "Ended","FailedInvalid","FailedDeath"
        };
        static const CdmwPlatformMngt::ProcessStatus Status[]=
        {
                CdmwPlatformMngt::PROCESS_INITIAL,
                CdmwPlatformMngt::PROCESS_INITIALISING,
                CdmwPlatformMngt::PROCESS_RUNNING,
                CdmwPlatformMngt::PROCESS_STOPPING,
                CdmwPlatformMngt::PROCESS_STOPPED,
                CdmwPlatformMngt::PROCESS_ENDED,
                CdmwPlatformMngt::PROCESS_FAILED_INVALID,
                CdmwPlatformMngt::PROCESS_FAILED_DEATH
        };

        
        // ----------------------------------------------------------------------
        // UnmanagedProcess_impl class.
        // ----------------------------------------------------------------------
        UnmanagedProcess_impl::UnmanagedProcess_impl(
            PortableServer::POA_ptr poa,
            CdmwPlatformMngt::ProcessDef* process_def,
            const char* process_key, const char* process_path,
            Application_impl* application,
            CdmwPlatformMngt::RWElementRepository_ptr element_repository)
            throw(CORBA::SystemException)
            : Process_impl(poa, process_def, process_key, process_path,
            application, element_repository),
            m_stop_condition (m_mutex_condition),
            m_stop_condition_flag (false)
        {
            try
            {
                std::auto_ptr< UnmanagedProcessStateMachine > stateMachine(
                    new UnmanagedProcessStateMachine(
                    this, "UnmanagedProcess_impl" ) );
                
                m_stateMachine = stateMachine.get();
                
                // Add the states to the state machine
                m_stateMachine->addState ("Initial", M_initial);
                m_stateMachine->addState ("Initialising", M_initialising);
                m_stateMachine->addState ("Running", M_running);
                m_stateMachine->addState ("Stopped", M_stopped);
                m_stateMachine->addState ("Ended", M_ended);
                m_stateMachine->addState ("FailedDeath", M_failedDeath);
                m_stateMachine->addState ("FailedInvalid", M_failedInvalid);

                try
                {
                    // define TECH_STATUS and TECH_STATUS_INFO
                    const char* init_value = "";
                    define_status_property(init_value);
                }
                catch (...)
                {
                    throw CosPropertyService::MultipleExceptions ();
                }
                // Set the initial state
                m_stateMachine->setState ("Initial");

                // Release the ownership of the temporary objects
                stateMachine.release();

                // Initialise preemption flag (false)
                init_preemption();

            }
            catch (const std::bad_alloc&)
            {
                throw CORBA::NO_MEMORY(OrbSupport::NO_MEMORY, CORBA::COMPLETED_NO);
            }
        }
        
        UnmanagedProcess_impl::~UnmanagedProcess_impl()
        {
            if (m_stateMachine != NULL)
                delete m_stateMachine;
        }
        
        void UnmanagedProcess_impl::end_creation(
            CdmwPlatformMngt::ProcessDef* process_def)
            throw (CdmwPlatformMngt::HostNotFound,
            CdmwPlatformMngt::DuplicateEntity,
            CdmwPlatformMngt::DuplicateService,
            CdmwPlatformMngt::DuplicateStep,
            OutOfMemoryException,
            CosPropertyService::MultipleExceptions,
            CORBA::SystemException)
        {
    

            // the following initialisation call the methods of the object
            // setting the attribute changes in datastore
            // in case of exception the object will be deleted by the application
            
            // set the host for this process
            // if not found throw HostNotFound exception
            setHost(process_def->host_name());
            
            // set the process info
            setProcessInfo (process_def->process_info());
            
            
            //Add all Properties in Un-ManagedProcess
            unsigned int propCount = process_def->properties().length();
            for (unsigned int propIndex=0 ; propIndex < propCount ; propIndex++)
            {
                // get service definition (ServiceDef is a structure, a reference on the
                // structure is returned by the [] operator)
                const CdmwPlatformMngt::PropertyDef& prop_def = process_def->properties()[propIndex];
                
                std::string propertyName(process_def->properties()[propIndex].property_name);
                //CosPropertyService::PropertyModeType propertyMode = (CosPropertyService::PropertyModeType) 0;
                //CdmwPlatformMngt::PropertyModeType propertyModeCdmw = process_def->properties()[propIndex].mode_type;        
                //if (propertyModeCdmw == 1) propertyMode = (CosPropertyService::PropertyModeType) 1;
                CosPropertyService::PropertyModeType propertyMode = 
                    CosPropertyService::PropertyModeType(process_def->properties()[propIndex].mode_type);            
                try
                {
                    define_property_with_mode(propertyName.c_str() , prop_def.initial_value, propertyMode);
                }
                catch (...)
                {
                    throw  CosPropertyService::MultipleExceptions(); 
                }
            }
            
        // MdR define rw properties
        try
        {
          std::ostringstream property_name;
          property_name << CdmwPlatformMngt::CDMW_PREFIX << CdmwPlatformMngt::FORCE_MANUAL_START << std::ends;
          if(!is_property_defined(property_name.str().c_str()))
          {
              CORBA::Any_var any = new CORBA::Any();
              *any <<= CORBA::Any::from_boolean(false);
              define_property_with_mode(property_name.str().c_str() , any.in(), CosPropertyService::fixed_normal);
          }

          property_name.str("");
          property_name << CdmwPlatformMngt::CDMW_PREFIX << CdmwPlatformMngt::AUTORESTART_INFO << std::ends;
          if(!is_property_defined(property_name.str().c_str()))
          {
              CdmwPlatformMngt::AutoRestartInfo auto_restart_info;
              auto_restart_info.max_nbr_of_restart_attempt = 1;
              auto_restart_info.auto_restart_control = CdmwPlatformMngt::USE_COUNTER;
              CORBA::Any_var any = new CORBA::Any();
              *any <<= auto_restart_info;
              define_property_with_mode(property_name.str().c_str() , any.in(), CosPropertyService::fixed_normal);
          }

          property_name.str("");
          property_name << CdmwPlatformMngt::CDMW_PREFIX << CdmwPlatformMngt::DEFAULT_RESTART_WARMTH << std::ends;
          if(!is_property_defined(property_name.str().c_str()))
          {
              CORBA::Any_var any = new CORBA::Any();
              *any <<= CdmwPlatformMngt::HOT_RESTART;
              define_property_with_mode(property_name.str().c_str() , any.in(), CosPropertyService::fixed_normal);
          }
        }
        catch(...)
        {
           throw  CosPropertyService::MultipleExceptions();
        }

            // MdR define system properties not user defined
            std::ostringstream property_name;
            CORBA::Any_var any_val = new CORBA::Any();
            try
            {
                *any_val <<= CdmwPlatformMngt::REDSTATE_NORG;      
                property_name.str("");
                property_name << CdmwPlatformMngt::CDMW_PREFIX << CdmwPlatformMngt::REPLICATION_ROLE << std::ends; 
                define_property_with_mode(property_name.str().c_str(), any_val.in(), CosPropertyService::fixed_normal);
                                                    
                get_attribute(CdmwPlatformMngt::DEFAULT_RESTART_WARMTH, any_val.out(), CdmwPlatformMngt::PRE_DEFINED);                                     
                property_name.str("");
                property_name << CdmwPlatformMngt::CDMW_PREFIX << CdmwPlatformMngt::REQUESTED_RESTART_WARMTH << std::ends;
                define_property_with_mode(property_name.str().c_str(), any_val.in(), CosPropertyService::fixed_normal);  
            }
            catch (...)
            {
                throw CosPropertyService::MultipleExceptions ();
            }
           

            set_indicator_relevant_status(process_def->indicator_relevant_status());
 
            // store default properties values
            store_default_properties();     
             
            // initialise data store of the object
            set_record_in_datastore();    
        }
        
        
        void UnmanagedProcess_impl::synchro_processing_to_initial()
        {
            m_stateMachine->setStateName("Initial");
        }
        
        
        void UnmanagedProcess_impl::synchro_processing_to_initialised()
        {
            // "initialised" state does not exist for unmanaged process
            // after "initialising" it is set to "running" immediateley
            m_stateMachine->setStateName("Running");
        }
        
        
        void UnmanagedProcess_impl::synchro_processing_to_running()
        {
            m_stateMachine->setStateName("Running");
        }
        
        
        const char* UnmanagedProcess_impl::get_state_machine_state_name()
        {
            return m_stateMachine->getStateName();
        }
        
        void UnmanagedProcess_impl::set_state_machine_state_name(const char* state_name)
        {
            m_stateMachine->setStateName(state_name);
        }

        void
    UnmanagedProcess_impl::set_state_machine_previous_state_name(const char* previous_state_name)  throw (CORBA::SystemException)
        {
        std::string prev_state_str( previous_state_name);
            m_stateMachine->setPreviousStateName(prev_state_str);
        }
        
        CdmwPlatformMngt::ProcessStatus UnmanagedProcess_impl::get_internal_status()
        {
            CDMW_SM_READER_LOCK_GUARD(*m_stateMachine);
            
            // get status
            CdmwPlatformMngt::ProcessStatus processStatus = 
                m_stateMachine->getState()->status();
            
            return processStatus;
        }
       
   void UnmanagedProcess_impl::setStopStatus (CdmwPlatformMngt::
        LongRequestStatus status)
    {
        CDMW_MUTEX_GUARD (m_mutex_condition);
        // set condition wait flag
        m_stop_condition_flag = true;
        m_stop_status = status;
        m_stop_condition.broadcast ();
    }

 
   CdmwPlatformMngt::ProcessStatus UnmanagedProcess_impl::property_to_status (const char* status)
   throw (CdmwPlatformMngt::AttributeNotFound)
   {
       CdmwPlatformMngt::ProcessStatus proc_status; 
       bool result = false;
       for ( int i=0 ; i<8; i++)
       {
           if (strcmp(status, process_status_strings[i].c_str()) == 0)
           {
               proc_status = Status[i];
               result = true;

               return proc_status;
           }
       }
       if (!result)
       {
           throw CdmwPlatformMngt::AttributeNotFound();
       }
       
       return proc_status;
   }

   CdmwPlatformMngt::ProcessStatus UnmanagedProcess_impl::get_status(
           CORBA::String_out status_info )
   throw( CORBA::SystemException )    
   {
       
       CdmwPlatformMngt::ProcessStatus process_status;
       const char* status = "";
       try
       {
           // Get predefined attributes of process
           CORBA::Any_var any_var = new CORBA::Any();
           
           get_attribute(CdmwPlatformMngt::TECH_STATUS, any_var.out(), CdmwPlatformMngt::PRE_DEFINED);           
           any_var.in() >>=  CORBA::Any::to_string(status,0);            
           
           process_status = property_to_status (status);    
           
           // get info associated to status
           status_info = CORBA::string_dup (get_status_info());

           return process_status;
       }
       catch (const CdmwPlatformMngt::AttributeNotFound&)
       {
           throw CORBA::BAD_PARAM(OrbSupport::BAD_PARAM, CORBA::COMPLETED_NO);
       }
       catch (const std::bad_alloc&)
       {
           throw CORBA::NO_MEMORY (OrbSupport::NO_MEMORY, CORBA::COMPLETED_NO);
       }
       catch (...)
       {
           throw CORBA::BAD_PARAM(OrbSupport::BAD_PARAM, CORBA::COMPLETED_NO);
       }
   }
        
        unsigned long UnmanagedProcess_impl::get_initialisation_timeout()
        throw (CORBA::SystemException)
        {
            try
            {
                //get the process record from datastore
                CdmwPlatformMngt::ProcessRecord_var process_record = get_record_from_datastore();
                //get the initialisation timeout
                unsigned long timeout_value=process_record->initialisation_timeout;
                return timeout_value;
            }
            catch (const std::bad_alloc&)
            {
             throw CORBA::NO_MEMORY (OrbSupport::NO_MEMORY, CORBA::COMPLETED_NO);
            }
        }
        
        void UnmanagedProcess_impl::set_auto_ending(CORBA::Boolean auto_ending)
            throw( CdmwPlatformMngt::IncompatibleStatus,
            CORBA::SystemException )
        {
            CDMW_SM_READER_LOCK_GUARD( *m_stateMachine );
            
            m_stateMachine->getState()->set_autoending (m_stateMachine, auto_ending);
        }
        
        void UnmanagedProcess_impl::set_host(
            const char* host_name )
            throw( CdmwPlatformMngt::HostNotFound,
            CdmwPlatformMngt::IncompatibleStatus,
            CORBA::SystemException )
        {
            CDMW_SM_READER_LOCK_GUARD( *m_stateMachine );
            
            m_stateMachine->getState()->set_host( m_stateMachine, host_name );
        }
        
        void UnmanagedProcess_impl::set_process_info(
            const CdmwPlatformMngt::ProcessInfo& process_info )
            throw( CdmwPlatformMngt::IncompatibleStatus,
            CORBA::SystemException )
        {
            CDMW_SM_READER_LOCK_GUARD( *m_stateMachine );
            
            m_stateMachine->getState()->set_process_info(
                m_stateMachine,
                process_info );
        }
        
        CdmwPlatformMngt::RequestStatus UnmanagedProcess_impl::initialise(
            const CdmwPlatformMngtBase::StartupKind& startup_kind)
            throw (CdmwPlatformMngt::IncompatibleStatus,
            CORBA::SystemException)
        {
            LogMngr::logMessage(
                INF,
                MSG_ID_PROCESS_INITALISE,
                get_process_key(),
                m_application->get_element_name());

            de_activate_preemption();

            CdmwPlatformMngt::RequestStatus req_status = CdmwPlatformMngt::REQUEST_SUCCEED;

            if (is_preemption_active())
            {
            }
            else
            {
                recover_default_properties ();

                CDMW_MUTEX_GUARD(*m_application->get_check_consistency_status_recursive_mutex());

                CDMW_SM_WRITER_LOCK_GUARD(*m_stateMachine);

                try
               {

                   req_status =
                       m_stateMachine->getState()->initialise(
                         m_stateMachine,
                           startup_kind);
               }
               catch (...)
               {
                   // even in case of exception update record in datastore
                   set_record_in_datastore();

                   // rethrow exception
                   throw;
               }
         
               if ( CdmwPlatformMngt::COLD_RESTART==requested_restart_warmth() )
               {
                  recover_default_properties ();
                  //recover_default_standard_attributes();
               }

               // set the entry in HostProcessInitSequencer, to restart process after host reboot
               Cdmw::PlatformMngt::ProcessHostInfo info;
               info.process_name = CORBA::string_dup ( name());
               info.process_impl  = this;
               if (true == created_dynamically())
               {
                  info.kind_of_start = ADDED;
                  set_proc_info(info, 2 );
               }
               else
                  info.kind_of_start = DEFAULT;

               //LC: PCR-792 set_proc_info(info, 2 );
               //HostProcessInitSequencer::get_instance()->set_init_entry(m_host->name(),info);

               // update record in datastore
               set_record_in_datastore();
              }
               return req_status;
        }
        
        CdmwPlatformMngt::RequestStatus UnmanagedProcess_impl::stop(
            CORBA::Boolean emergency )
            throw (CORBA::SystemException)
        {
            LogMngr::logMessage(
                INF,
                MSG_ID_PROCESS_STOP,
                get_process_key(),
                m_application->get_element_name());

            //MR
            //set_requested_restart_warmth ( get_default_restart_warmth () );

            activate_preemption();

            CDMW_MUTEX_GUARD(*m_application->get_check_consistency_status_recursive_mutex());

            CDMW_SM_WRITER_LOCK_GUARD(*m_stateMachine);

            CdmwPlatformMngt::RequestStatus req_status;

            try
            {
                req_status =
                    m_stateMachine->getState()->stop(m_stateMachine, emergency);
            }
            catch (...)
            {
                // even in case of exception update record in datastore
                set_record_in_datastore();

                // rethrow exception
                throw;
            }

            // update record in datastore
            set_record_in_datastore();

            return req_status;
        }
        
        CdmwPlatformMngt::RequestStatus UnmanagedProcess_impl::stop_process(CORBA::Boolean emergency)
    throw (CORBA::SystemException)
    {
        // Create a command event to notify
        std::string proc_path = std::string (element_path());
        //LC: Attention: call an issuer mutex guard before calling this method.        
        std::string issuer = Cdmw::PlatformMngt::ProcessIssuerManager::Instance()->get_command_issuer(proc_path);

    notifyCommandEvent(Process_impl::STOP, issuer.c_str());
        
        return stop(emergency);
    }    


    
    CdmwPlatformMngt::LongRequestStatus UnmanagedProcess_impl::
        stop_and_wait (CORBA::Boolean emergency) throw (CORBA::SystemException)
    {
        CdmwPlatformMngt::ProcessStatus procStatus;
        CdmwPlatformMngt::LongRequestStatus req_status =
            CdmwPlatformMngt::LONG_REQUEST_COMPLETED;
        bool stop_req_done = false;


        {
            // lock mutex for reading status
            CDMW_SM_READER_LOCK_GUARD (*m_stateMachine);
            // get the current status
            procStatus = m_stateMachine->getState ()->status ();

            CDMW_INFO(Cdmw::Logging::Logger::Get_Logger(CdmwPlatformMngtBase::PFMMNG), 
              " UnmanagedProcess_impl::stop_and_wait " << name() << " process on host: " << m_host->name() << std::endl);

            if (//procStatus == CdmwPlatformMngt::PROCESS_INITIAL ||
                procStatus == CdmwPlatformMngt::PROCESS_STOPPED ||
                procStatus == CdmwPlatformMngt::PROCESS_FAILED_DEATH ||
                procStatus == CdmwPlatformMngt::PROCESS_FAILED_INVALID || 
                procStatus == CdmwPlatformMngt::PROCESS_ENDED)
            {
                req_status = CdmwPlatformMngt::LONG_REQUEST_COMPLETED;
                stop_req_done = true;
            }
        }
        if (!stop_req_done)
        {


            // first lock mutex for condition
            CDMW_MUTEX_GUARD (m_mutex_condition);

            m_stop_condition_flag = false;

            if (procStatus != CdmwPlatformMngt::PROCESS_STOPPING)
            {
                // stop process
                // exception is thrown in case of incompatible status

            CdmwPlatformMngt::RequestStatus status = stop (emergency);

            if( CdmwPlatformMngt::REQUEST_SUCCEED != status ) 
            {
                return CdmwPlatformMngt::LONG_REQUEST_NOT_COMPLETED;
            }

            }
            // stopping is in progress
            // so wait the condition flag is set by broadcasting
            // only if status is not INITIAL

            if (CdmwPlatformMngt::PROCESS_INITIAL == procStatus)
            {
                m_stop_condition_flag = true;
            }

            while (!m_stop_condition_flag)
            {
                  m_stop_condition.wait ();
            }

            m_stop_condition_flag = false;
            
            // get the result
            req_status = m_stop_status;


        }
    
        return req_status;
    }

        
        void UnmanagedProcess_impl::error_event(
            const CdmwPlatformMngtBase::EventHeader& header,
            const char* issuer,
            const char* error_info )
        {
        }
        
        void UnmanagedProcess_impl::ending_event(
            const CdmwPlatformMngtBase::EventHeader& header )
        {

            CDMW_MUTEX_GUARD(*m_application->get_check_consistency_status_recursive_mutex());

            CDMW_SM_WRITER_LOCK_GUARD( *m_stateMachine );
            
            m_stateMachine->getState()->ending_event(
                m_stateMachine,
                header );

        }
        
        void UnmanagedProcess_impl::invalidate_event(
            const CdmwPlatformMngtBase::EventHeader& header,
            const char* reason )
        {
      
            CDMW_MUTEX_GUARD(*m_application->get_check_consistency_status_recursive_mutex());
       
            CDMW_SM_WRITER_LOCK_GUARD( *m_stateMachine );
            
            m_stateMachine->getState()->invalidate_event(
                m_stateMachine,
                header,
                reason );

        }
        
        void UnmanagedProcess_impl::createStates()
            throw( OutOfMemoryException )
        {
            try
            {
                std::auto_ptr< UnmanagedProcessInitial > initial(
                    new UnmanagedProcessInitial() );
                std::auto_ptr< UnmanagedProcessInitialising > initialising(
                    new UnmanagedProcessInitialising() );
                std::auto_ptr< UnmanagedProcessRunning > running(
                    new UnmanagedProcessRunning() );
                std::auto_ptr< UnmanagedProcessStopped > stopped(
                    new UnmanagedProcessStopped() );
                std::auto_ptr< UnmanagedProcessEnded > ended(
                    new UnmanagedProcessEnded() );
                std::auto_ptr< UnmanagedProcessFailedDeath > failedDeath(
                    new UnmanagedProcessFailedDeath() );
                std::auto_ptr< UnmanagedProcessFailedInvalid > failedInvalid(
                    new UnmanagedProcessFailedInvalid() );
                
                if( M_initial == NULL )
                    M_initial = initial.release();
                if( M_initialising == NULL )
                    M_initialising = initialising.release();
                if( M_running == NULL )
                    M_running = running.release();
                if( M_stopped == NULL )
                    M_stopped = stopped.release();
                if( M_ended == NULL )
                    M_ended = ended.release();
                if( M_failedDeath == NULL )
                    M_failedDeath = failedDeath.release();
                if( M_failedInvalid == NULL )
                    M_failedInvalid = failedInvalid.release();
            }
            catch( const std::bad_alloc& )
            {
                CDMW_THROW( OutOfMemoryException );
            }
        }
        
        void UnmanagedProcess_impl::destroyStates()
        {
            if( M_initial != NULL )
            {
                delete M_initial;
                M_initial = NULL;
            }
            
            if( M_initialising != NULL )
            {
                delete M_initialising;
                M_initialising = NULL;
            }
            
            if( M_running != NULL )
            {
                delete M_running;
                M_running = NULL;
            }
            
            if( M_stopped != NULL )
            {
                delete M_stopped;
                M_stopped = NULL;
            }
            
            if( M_ended != NULL )
            {
                delete M_ended;
                M_ended = NULL;
            }
            
            if( M_failedDeath != NULL )
            {
                delete M_failedDeath;
                M_failedDeath = NULL;
            }
            
            if( M_failedInvalid != NULL )
            {
                delete M_failedInvalid;
                M_failedInvalid = NULL;
            }
        }
        
        void UnmanagedProcess_impl::setState( const char* stateName )
        {
    
            m_stateMachine->setState( stateName );

            set_record_in_datastore();
        }
        
        CdmwPlatformMngt::ProcessStatus UnmanagedProcess_impl::getStatus()
            throw( CORBA::SystemException )    
        {
            return m_stateMachine->getState()->status();
        }
        
        CdmwPlatformMngt::ProcessStatus UnmanagedProcess_impl::getPreviousStatus()
            throw( CORBA::SystemException )
        {
            //return m_stateMachine->getPreviousState()->status();

        CdmwPlatformMngt::ProcessStatus prev_status = CdmwPlatformMngt::PROCESS_INITIAL;

        string str =  m_stateMachine->getPreviousStateName();

        if (str.compare("Initial") == 0)
           prev_status = CdmwPlatformMngt::PROCESS_INITIAL;

        if (str.compare("Initialising") == 0)
           prev_status = CdmwPlatformMngt::PROCESS_INITIALISING;

        if (str.compare("StepPerformed") == 0)
           prev_status = CdmwPlatformMngt::PROCESS_STEP_PERFORMED;
        
        if (str.compare("Initialised") == 0)
           prev_status = CdmwPlatformMngt::PROCESS_INITIALISED ;

        if (str.compare("RunRequest") == 0)
           prev_status = CdmwPlatformMngt::PROCESS_RUN_REQUEST;


        if (str.compare("Running") == 0)
           prev_status = CdmwPlatformMngt::PROCESS_RUNNING;

        if (str.compare("Stopping") == 0)
           prev_status = CdmwPlatformMngt::PROCESS_STOPPING;

        if (str.compare("Stopped") == 0)
           prev_status = CdmwPlatformMngt::PROCESS_STOPPED;

        if (str.compare("Ended") == 0)
           prev_status = CdmwPlatformMngt::PROCESS_ENDED;

        if (str.compare("FailedError") == 0)
           prev_status = CdmwPlatformMngt::PROCESS_FAILED_ERROR;

        if (str.compare("FailedMonitoringFailure") == 0)
           prev_status = CdmwPlatformMngt::PROCESS_FAILED_NO_RESPONSE;

        if (str.compare("FailedDeath") == 0)
           prev_status = CdmwPlatformMngt::PROCESS_FAILED_DEATH;

        if (str.compare("FailedDiscrepancy") == 0)
           prev_status = CdmwPlatformMngt::PROCESS_FAILED_DISCREPANCY;

        if (str.compare("FailedTimeout") == 0)
           prev_status = CdmwPlatformMngt::PROCESS_FAILED_TIMEOUT;

        if (str.compare("FailedInvalid") == 0)
           prev_status = CdmwPlatformMngt::PROCESS_FAILED_INVALID;

        return prev_status;
        }
        
        void UnmanagedProcess_impl::notifyStatusChangeEvent(
            const CdmwPlatformMngtBase::EventHeader& header,
            const char* issuer )
        {
            try
            {
                // Get the status
                CdmwPlatformMngt::ProcessStatus status
                    = m_stateMachine->getState()->status();
                
                // Create the event
                CdmwPlatformMngt::ProcessID_var processId
                    = new CdmwPlatformMngt::ProcessID;
                processId->application_name = m_application->name();
                processId->process_name = name();
                processId->host_name = m_host->name();
                
                CORBA::String_var processHostName = m_host->name();
                
                const char* p_stepLabel = 
                    (m_lifeCycle->init_steps.length() == 0) ?
                    "" :
                m_lifeCycle->init_steps[m_stepCounter].step_label.in(); 
                
                CdmwPlatformMngt::ProcessExitInfo_var exit_info;
                if( (status == CdmwPlatformMngt::PROCESS_FAILED_DEATH) ||
                (status == CdmwPlatformMngt::PROCESS_FAILED_NO_RESPONSE) ||
                (status == CdmwPlatformMngt::PROCESS_FAILED_INVALID) ||
                (status == CdmwPlatformMngt::PROCESS_FAILED_DISCREPANCY) ||
                (status == CdmwPlatformMngt::PROCESS_FAILED_TIMEOUT) ||
                (status == CdmwPlatformMngt::PROCESS_FAILED_ERROR) ||
                (status == CdmwPlatformMngt::PROCESS_ENDED) ||
                (status == CdmwPlatformMngt::PROCESS_STOPPED) ){
                    exit_info = new  CdmwPlatformMngt::ProcessExitInfo() ;
                    exit_info->core_dumped=CdmwPlatformMngt::CORE_UNKNOWN;
                    exit_info->process_pid = pid();
                    exit_info->exit_event=CdmwPlatformMngt::EXITED;
                    exit_info->event_value=0;
                }
                else{
                    exit_info = new  CdmwPlatformMngt::ProcessExitInfo() ;
                    exit_info->core_dumped=CdmwPlatformMngt::CORE_UNKNOWN;
                    exit_info->process_pid = 0;
                    exit_info->exit_event=CdmwPlatformMngt::EXITED;
                    exit_info->event_value=0;
                }
                
                CdmwPlatformMngt::ProcessStatusChange_var event
                    = m_procStatusChangeFactory->create(
                    header,
                    m_application->get_system_name(),
                    processId.in(),
                    issuer,
                    status,
                    m_stepCounter,
                    p_stepLabel,
                    m_status_info.in(),
                    exit_info.in() );
                
                CORBA::String_var eventStr = event->to_string();
                LogMngr::logMessage(eventStr.in());
               
                event->_add_ref(); 
                m_application->get_event_notifier()->addEvent(event);

                // Perform the status change notification to observer registered
                // with the Api in ManagedElement
                event->_add_ref();
                get_status_event_notifier()->add_event (event);
                event->_remove_ref();
                event._retn ();
            }
            catch(const std::bad_alloc&)
            {
                throw CORBA::NO_MEMORY(OrbSupport::NO_MEMORY, CORBA::COMPLETED_NO);
            }
            catch(...)
            {
                throw CORBA::INTERNAL(OrbSupport::INTERNAL, CORBA::COMPLETED_NO);
            }
        }

      bool UnmanagedProcess_impl::is_preemption_active ()
      {
          CDMW_MUTEX_GUARD (m_preemption_mutex);

          return m_preemption_flag;
      }

      bool UnmanagedProcess_impl::activate_preemption ()
      {
          bool result = false;

          CDMW_MUTEX_GUARD (m_preemption_mutex);

          bool old_flag_value = m_preemption_flag;

          try
          {
             CORBA::String_var sysStatusInfo;

             CdmwPlatformMngt::System_var system = CdmwPlatformMngt::System::_narrow(m_application->get_parent());

             CdmwPlatformMngt::SystemStatus system_status = system->get_status(sysStatusInfo.out());

             CORBA::String_var applStatusInfo;

             CdmwPlatformMngt::ApplicationStatus appl_status =  m_application->get_status(applStatusInfo.out());

             CdmwPlatformMngt::HostStatus host_status = m_host->status();

             CORBA::String_var statusInfo;
             
             CdmwPlatformMngt::ProcessStatus status = get_status(statusInfo.out());
             
             if (system_status == CdmwPlatformMngt::SYSTEM_STARTING 
                   || system_status == CdmwPlatformMngt::SYSTEM_INITIAL 
                   || appl_status == CdmwPlatformMngt::APPLICATION_INITIAL
                   || appl_status == CdmwPlatformMngt::APPLICATION_INITIALISING
                   || appl_status == CdmwPlatformMngt::APPLICATION_INITIALISATION_SUSPENDED
                   || host_status == CdmwPlatformMngt::HOST_STARTING
                   || status == CdmwPlatformMngt::PROCESS_INITIAL 
                   || status == CdmwPlatformMngt::PROCESS_INITIALISING
                   || status == CdmwPlatformMngt::PROCESS_STEP_PERFORMED)
             {
                 m_preemption_flag = true;
                 result = true;   
             }

          }
          catch (...)
          {
              old_flag_value = m_preemption_flag;
              CDMW_ERROR(Cdmw::Logging::Logger::Get_Logger(CdmwPlatformMngtBase::PFMMNG), "  Can't activate preemption of " << name() << " process on host: " << m_host->name() << std::endl);
          }

          return result;
      }

      bool UnmanagedProcess_impl::de_activate_preemption ()
      {
          bool result = false;

          CDMW_MUTEX_GUARD (m_preemption_mutex);

          try 
          {
             CORBA::String_var sysStatusInfo;

             CdmwPlatformMngt::System_var system = CdmwPlatformMngt::System::_narrow(m_application->get_parent());

             CdmwPlatformMngt::SystemStatus system_status = system->get_status(sysStatusInfo.out());

             CORBA::String_var applStatusInfo;

             CdmwPlatformMngt::ApplicationStatus appl_status = m_application->get_status(applStatusInfo.out());

             CdmwPlatformMngt::HostStatus host_status = m_host->status();

             CORBA::String_var statusInfo;

             if (m_preemption_flag)
             {
                 if (system_status == CdmwPlatformMngt::SYSTEM_STARTING 
                       || system_status == CdmwPlatformMngt::SYSTEM_INITIAL 
                       || appl_status == CdmwPlatformMngt::APPLICATION_INITIAL
//                       || appl_status == CdmwPlatformMngt::APPLICATION_INITIALISING
                       || host_status == CdmwPlatformMngt::HOST_STARTING)
                 {

                      if (appl_status == CdmwPlatformMngt::APPLICATION_INITIAL && get_force_manual_start())
                      {
                         m_preemption_flag = false;
                         result = true;
                      }

                 }
                 else
                 {
                    m_preemption_flag = false;
                    result = true;
                 }
            }

          }
          catch (...)
          {

              CDMW_ERROR(Cdmw::Logging::Logger::Get_Logger(CdmwPlatformMngtBase::PFMMNG), "  Can't deactivate preemption of " << name() << " process on host: " << m_host->name() << std::endl);
              m_preemption_flag = false;
              result = true;
          }

          return result;
      }

      void UnmanagedProcess_impl::init_preemption ()
      {
         CDMW_MUTEX_GUARD (m_preemption_mutex);

         m_preemption_flag = false;
      }

    } // End namespace PlatformMngt
} // End namespace Cdmw


