/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#include "SystemMngt/platformelementlibrary/ManagedElement_impl.hpp"

#include "SystemMngt/tools/ManagedElementKey.hpp"
#include "SystemMngt/platformlibrary/Configuration.hpp"

#include "Foundation/orbsupport/ExceptionMinorCodes.hpp"
#include "Foundation/common/Assert.hpp"
#include "Foundation/commonsvcs/datastore/StorageErrorException.hpp"

#include "SystemMngt/platformlibrary/EventHeaderFactory.hpp"

#include "platformlibrary/EventNotifierManager.hpp"

#include "SystemMngt/platformelementlibrary/EntityContainer_impl.hpp"
#include "SystemMngt/platformelementlibrary/RWEntityContainer_impl.hpp"
#include "SystemMngt/platformelementlibrary/EntityContainer.hpp"
#include "SystemMngt/idllib/CdmwPlatformMngtPredefinedProperties.stub.hpp"




namespace
{
      
CdmwPlatformMngt::PropertyEventRecord
build_record(const char* name,
                 CdmwPlatformMngt::SupervisionObserver_ptr observer_ref,
                 const CosPropertyService::PropertyNames& properties)
{
        CdmwPlatformMngt::PropertyEventRecord out;
        out.observer_name  = name;
    out.observer = CdmwPlatformMngt::SupervisionObserver::_duplicate(observer_ref);
        out.is_for_all     = false;
    const CORBA::ULong length = properties.length();
        out.property_names.length(length);
    CDMW_ASSERT( out.property_names.length() == length);
        out.property_names = properties;
        return out;
}
    
CdmwPlatformMngt::PropertyEventRecord
build_record(const char* name, CdmwPlatformMngt::SupervisionObserver_ptr observer_ref)
{
        CdmwPlatformMngt::PropertyEventRecord out;
        out.observer_name  = name;
    out.observer = CdmwPlatformMngt::SupervisionObserver::_duplicate(observer_ref);
        out.is_for_all     = true;
        out.property_names.length(0);                              
        return out;
}

CdmwPlatformMngt::EventRecordExt
build_record(const char* name, const CdmwPlatformMngt::RegistrationDepth& depth_val,
                 CdmwPlatformMngt::SupervisionObserver_ptr observer_ref)
{
        CdmwPlatformMngt::EventRecordExt out;
        out.observer_name  = name;
    out.observer = CdmwPlatformMngt::SupervisionObserver::_duplicate(observer_ref);
        out.depth          = depth_val;
        return out;
}
    
};

namespace Cdmw
{
namespace PlatformMngt
{



// ----------------------------------------------------------------------
// ManagedElement_impl class.
// ----------------------------------------------------------------------

ManagedElement_impl::ManagedElement_impl (CdmwPlatformMngt::RWElementRepository_ptr element_repository, 
        PortableServer::POA_ptr poa) 
throw () 
{
        m_logger = Cdmw::Logging::Logger::Get_Logger(CdmwPlatformMngtBase::PFMMNG);

        m_property_event_notifier=0;
        m_bownerPropertyEventNotifier=false;
            m_bownerStatusEventNotifier=false;
            m_bownerSupEventNotifier=false;
    m_element_repository = CdmwPlatformMngt::RWElementRepository::_duplicate (element_repository);
        m_property_event_infos = new CdmwPlatformMngt::PropertyEventRecords();

    m_event_ext_infos      = new CdmwPlatformMngt::EventRecordExts(); 

    try {
        Cdmw::CommonSvcs::DataStore::DataStoreBase * ds =
            Cdmw::CommonSvcs::DataStore::DataStoreRegistry::Get_data_store(
                Configuration::Get_managed_element_datastore_id ());
            m_managed_element_ds = dynamic_cast < Cdmw::PlatformMngt::ManagedElementDataStore * >(ds);
        
            std::auto_ptr < CosPropertyService::PropertyDefs > userDefDefault (new CosPropertyService::PropertyDefs);

            m_default_user_defined_attributes = userDefDefault.get ();

            userDefDefault.release ();
        
            m_poa = PortableServer::POA::_duplicate (poa);
        
            m_property_set = new Cdmw::PlatformMngt::PropertySet_impl (m_poa.in());
        
            PortableServer::ServantBase_var property_set = m_property_set;
        
            m_property_set -> activate();
            m_property_set_var = m_property_set->_this ();
       
        m_propertyChangeFactory = std::auto_ptr < PropertyChangeFactory >(new PropertyChangeFactory ());
            m_configurationChangeFactory = std::auto_ptr < ConfigurationChangeFactory >(new ConfigurationChangeFactory());
    
        }
        catch (const NotExistException & e)
        {
            CDMW_ASSERT (false);
        }
    
        m_element_path = CORBA::string_dup ("");

        m_element_name = CORBA::string_dup ("");
        
        m_status_info = CORBA::string_dup( "" );
    }
    
     
ManagedElement_impl::~ManagedElement_impl ()
{
    if(m_default_user_defined_attributes != 0)
        delete m_default_user_defined_attributes;
    
    m_default_user_defined_attributes = 0;
    
    remove_element_in_datastore ();
}

void ManagedElement_impl::remove()
{
    m_property_set->_add_ref();
    // deactivate the Corba object
    m_property_set->deactivate();
    // then decrement the counter reference to call destructor
    m_property_set->_remove_ref();

    remove_element_in_datastore();
}

void ManagedElement_impl::set_property_event_notifier(PropertyEventNotifier* property_event_notifier)
{
    // if before was owner stop the thread now
    if(m_property_event_notifier && m_bownerPropertyEventNotifier)
    {
        if(m_property_event_notifier->is_running())
        {
            m_property_event_notifier->stop();
        }
    delete m_property_event_notifier;
        m_property_event_notifier=0;
    }
    m_property_event_notifier=property_event_notifier;
    m_bownerPropertyEventNotifier=false;
}

void ManagedElement_impl::set_status_event_notifier(StatusEventNotifier* status_event_notifier)
{
    // if before was owner stop the thread now
    if(m_status_event_notifier && m_bownerStatusEventNotifier)
    {
        if(m_status_event_notifier->is_running())
        {
            m_status_event_notifier->stop();
        }
        delete m_status_event_notifier;
        m_status_event_notifier=0;
    }
    m_status_event_notifier=status_event_notifier;
    m_bownerStatusEventNotifier=false;
}


char *ManagedElement_impl::name () throw (CORBA::SystemException)
{
        CORBA::String_var name = m_element_name;

        return name._retn ();
}

char *ManagedElement_impl::element_path () throw (CORBA::SystemException)
{
        CORBA::String_var path = m_element_path;

        return path._retn ();
}


char *ManagedElement_impl::get_status_info()
{
        CORBA::String_var path = m_status_info;

        return path._retn ();
}



void ManagedElement_impl::set_status_info (const char *status_info)
{
       m_status_info = CORBA::string_dup (status_info);
}

CdmwPlatformMngt::ManagedElement_ptr ManagedElement_impl::get_parent()
throw (CORBA::SystemException)
{
        CdmwPlatformMngt::ManagedElement_var parent_element;

        try
        {
        // get parent path of the managed element
        std::string parent_path;
        std::string element_name;
        std::string element_path = m_element_path.in ();
        ManagedElementKey::Get_parent_path (element_path, element_name, parent_path);

        // get the parent element object from repository
        parent_element = m_element_repository->get_element (parent_path.c_str ());
        }
        catch (const CdmwPlatformMngt::ElementPathInvalid&)
        {
        CDMW_THROW2 (BadParameterException, "element_path", "invalid");
        }


        return parent_element._retn ();
}


void ManagedElement_impl::set_element_path (const char *element_path, const char *element_name)
{
        m_element_path = CORBA::string_dup (element_path);

        m_element_name = CORBA::string_dup (element_name);
}


void ManagedElement_impl::set_element_in_datastore ()
{
    if (!Configuration::Is_fault_tolerant() || Configuration::Is_primary ())
    {
        // set in DataStore
        CdmwPlatformMngt::ManagedElementRecord record;
        record.name = m_element_name;
                
        record.default_attributes = *m_default_user_defined_attributes;

        
        try
        {
           // adding the new UDAs in datastore
            CosPropertyService::Properties_var props;
            CORBA::ULong propCount = get_number_of_properties ();
            CosPropertyService::PropertyNames_var prop_names;
            CosPropertyService::PropertyNamesIterator_var prop_names_It;
            get_all_property_names (propCount, prop_names.out(), prop_names_It.out ());
            get_properties (prop_names, props.out ());
            CosPropertyService::PropertyDefs_var propdefs = new CosPropertyService::PropertyDefs;
            propdefs->length (propCount);
            CosPropertyService::PropertyModes_var prop_modes;
            get_property_modes (prop_names, prop_modes.out ());
            
            for (CORBA::ULong i = 0; i < propCount; i++)
            {
               propdefs[i].property_name = prop_names[i];
               propdefs[i].property_mode = prop_modes[i].property_mode;
               propdefs[i].property_value = *get_property_value (props[i].property_name);
            }        

        // set the default user defined attributes value
        //record.user_defined_attributes = propdefs;
        record.attributes = propdefs;
        }
        catch (const CosPropertyService::PropertyNotFound&)
        {
            CDMW_ERROR(m_logger, "PropertyNotFound while reading property in set_element_in_datastore" );
            //MRI TODO manage as FATAL ERROR
        }

        // set the property_event_infos in datastore record
        const CORBA::ULong length = m_property_event_infos->length(); 
        record.property_event_infos.length(length);
        for (CORBA::ULong j = 0; j < length ; ++j)
        {
            const CORBA::ULong tmp_size = m_property_event_infos[j].property_names.length();
            record.property_event_infos[j].property_names.length(tmp_size);
            if(tmp_size>0)
            {
            record.property_event_infos[j].property_names = m_property_event_infos[j].property_names;
            }                
            record.property_event_infos[j].observer_name  = m_property_event_infos[j].observer_name;
            (m_property_event_infos[j].observer)->_add_ref();
            record.property_event_infos[j].observer       =  m_property_event_infos[j].observer;
            record.property_event_infos[j].is_for_all     = m_property_event_infos[j].is_for_all;
        }
                
        //Take the Infos for the registration of ObserverExt
        const CORBA::ULong length_ext = m_event_ext_infos->length();
        record.event_ext_infos.length(length_ext);
        for (CORBA::ULong j = 0; j < length_ext ; ++j)
        {
            record.event_ext_infos[j].observer_name  = m_event_ext_infos[j].observer_name;
            (m_event_ext_infos[j].observer)->_add_ref();
            record.event_ext_infos[j].observer       =  m_event_ext_infos[j].observer;
            record.event_ext_infos[j].depth         =  m_event_ext_infos[j].depth;
        }
        string ds_operation("contains");    
        try
        {
            if (m_managed_element_ds->contains (m_element_path.in ()))
            {
                ds_operation = "update";
                m_managed_element_ds->update (m_element_path.in (), record);
            }
            else
            {
                ds_operation = "insert";
                m_managed_element_ds->insert (m_element_path.in (), record);
            }
        }
        catch (const Common::AlreadyExistException&)
        {
            CDMW_WARN(m_logger,
            "record already exists exception in " 
            << ds_operation << " : " 
            << m_element_path.in () << ":"
            << (const char*) record.name);
        }
        catch (const CommonSvcs::DataStore::StorageErrorException &exc_storage)
        {
            CDMW_FATAL(m_logger,
            "record storage error exception in "
            << ds_operation <<" : " 
            << m_element_path.in() << ":"
            << (const char*) record.name << ":" 
            << exc_storage.what());
            CDMW_ASSERT (false);
        }
        catch (const Common::NotFoundException&)
        {
            CDMW_FATAL(m_logger,
            "record cannot be found in datastore in "
            << ds_operation <<" : " 
            << m_element_path.in());
            CDMW_ASSERT (false);
        }
        catch (...)
        {
            CDMW_FATAL(m_logger,
            "record unknown exception in "
            << ds_operation << " : " 
            << m_element_path.in() << ":"
            << (const char*) record.name);
            CDMW_ASSERT (false);
        }
    }
      
}

void ManagedElement_impl::remove_element_in_datastore ()
{
    if (!Configuration::Is_fault_tolerant() || Configuration::Is_primary())
    {
        try
        {
            m_managed_element_ds->remove (m_element_path.in ());
        }
        catch (const Common::NotFoundException &)
        {
        }
        catch (const CommonSvcs::DataStore::StorageErrorException &)
        {
        }

    }
}

const char *ManagedElement_impl::get_element_name ()
{
        return m_element_name.in ();
}

const char *ManagedElement_impl::get_element_path ()
{
        return m_element_path.in ();
}

void ManagedElement_impl::define_property (const char *property_name, const CORBA::Any & property_value)
throw (CosPropertyService::InvalidPropertyName,
           CosPropertyService::ConflictingProperty,
           CosPropertyService::UnsupportedTypeCode,
           CosPropertyService::UnsupportedProperty,
           CosPropertyService::ReadOnlyProperty, CORBA::SystemException)
    {
        
    
        CDMW_WRITER_RECURSIVE_MUTEX_GUARD (m_rw_property_mutex); 
    
    // Prepare RecordInfo
        CdmwPlatformMngt::PropertyInfo p_info;
        p_info.property_name = CORBA::string_dup(property_name);
        p_info.property_new_value = property_value;

        // Prepare PropertyInfos sequence
        CdmwPlatformMngt::PropertyInfos_var p_infos = new CdmwPlatformMngt::PropertyInfos();

        // old value of property_name
        CORBA::Any old_value;
        try
        {
        // p_info.property_old_value <<= * get_property_value(property_name);
        old_value = *get_property_value(property_name);
        }
        catch (const CosPropertyService::PropertyNotFound & p_not_found)
        {
        // if no property old value for property_name is present do nothing
        }
        p_info.property_old_value = old_value;
        p_infos->length(1);
        p_infos[0L] = p_info;
        m_property_set_var->define_property (property_name, property_value);
        notify_property_change("define_property", p_infos);
        set_element_in_datastore ();

    }

void ManagedElement_impl::define_properties (const CosPropertyService::Properties & nproperties)
throw (CosPropertyService::MultipleExceptions, CORBA::SystemException)
{
        
        CDMW_WRITER_RECURSIVE_MUTEX_GUARD (m_rw_property_mutex); 
    
    // Prepare RecordInfo
        CdmwPlatformMngt::PropertyInfo p_info;
        // Prepare PropertyInfos sequence
        CdmwPlatformMngt::PropertyInfos_var p_infos = new CdmwPlatformMngt::PropertyInfos();
        // old value of property_name
        p_infos->length(0L);
        CORBA::ULong properties_size = nproperties.length();
       
        for (CORBA::ULong i=0; i< properties_size; i++)
        {
        CORBA::Any old_value;
        try
        {
            old_value = *get_property_value(nproperties[i].property_name);               
        }
        catch (const CosPropertyService::PropertyNotFound & p_not_found)
        {
            // if no property old value for current property_name is present do nothing 
            // and add a new property
        }
        p_infos->length(p_infos->length() + 1);
        p_info.property_name      = CORBA::string_dup(nproperties[i].property_name);
        p_info.property_new_value = nproperties[i].property_value;
        p_info.property_old_value = old_value;
        p_infos[i] = p_info;
        }
        try
        {
        m_property_set_var->define_properties (nproperties);
        }
        catch (...)
        {
        throw CosPropertyService::MultipleExceptions();
        }
            
        notify_property_change("define_properties", p_infos);
        set_element_in_datastore ();
}

CORBA::ULong ManagedElement_impl::get_number_of_properties()
throw (CORBA::SystemException)
{
        
        CDMW_READER_RECURSIVE_MUTEX_GUARD (m_rw_property_mutex); 
    return m_property_set_var->get_number_of_properties ();

}

void ManagedElement_impl::get_all_property_names (CORBA::ULong how_many,
        CosPropertyService::PropertyNames_out property_names,
        CosPropertyService::PropertyNamesIterator_out rest) 
throw (CORBA::SystemException)
{
    
    CDMW_READER_RECURSIVE_MUTEX_GUARD (m_rw_property_mutex);
    m_property_set_var->get_all_property_names (how_many, property_names, rest);
}

CORBA::Any* ManagedElement_impl::get_property_value (const char *property_name)
throw (CosPropertyService::PropertyNotFound,
           CosPropertyService::InvalidPropertyName, 
           CORBA::SystemException)
    {
        
    
        CDMW_READER_RECURSIVE_MUTEX_GUARD (m_rw_property_mutex);
    return m_property_set_var->get_property_value (property_name);

    }

CORBA::Boolean ManagedElement_impl::get_properties (const CosPropertyService::PropertyNames& property_names,
        CosPropertyService::Properties_out nproperties) 
throw (CORBA::SystemException)
{
         
        CDMW_READER_RECURSIVE_MUTEX_GUARD (m_rw_property_mutex);
    return m_property_set_var->get_properties (property_names, nproperties);
}

void ManagedElement_impl::get_all_properties (CORBA::ULong how_many,
        CosPropertyService::Properties_out nproperties,
        CosPropertyService::PropertiesIterator_out rest)
throw (CORBA::SystemException)
{
        
        CDMW_READER_RECURSIVE_MUTEX_GUARD (m_rw_property_mutex);
    
    m_property_set_var->get_all_properties (how_many, nproperties, rest);


}



void ManagedElement_impl::delete_property (const char *property_name)
throw (CosPropertyService::PropertyNotFound,
           CosPropertyService::InvalidPropertyName,
           CosPropertyService::FixedProperty, CORBA::SystemException)
    {
        
        CDMW_WRITER_RECURSIVE_MUTEX_GUARD (m_rw_property_mutex);
    
    // old value of property_name
        CORBA::Any old_value;
        try
        {
        old_value = *get_property_value(property_name);
        }
        catch (const CosPropertyService::PropertyNotFound & p_not_found)
        {
        // if no property old value for property_name is present do nothing
        throw p_not_found;   
        }
            
        // Prepare RecordInfo
        CdmwPlatformMngt::PropertyInfo p_info;
        p_info.property_name = CORBA::string_dup(property_name);
        const char* property_deleted = "property_deleted";
        p_info.property_new_value <<= property_deleted;

        // Prepare PropertyInfos sequence
        CdmwPlatformMngt::PropertyInfos_var p_infos = new CdmwPlatformMngt::PropertyInfos();
        p_info.property_old_value = old_value;
        p_infos->length(1);
        p_infos[0L] = p_info;
        m_property_set_var->delete_property (property_name);
        notify_property_change("delete_property", p_infos);
        set_element_in_datastore ();
    }

    
void ManagedElement_impl::delete_property_without_notify (const char *property_name)
throw (CosPropertyService::PropertyNotFound,
           CosPropertyService::InvalidPropertyName,
           CosPropertyService::FixedProperty, CORBA::SystemException)
    {
        
        CDMW_WRITER_RECURSIVE_MUTEX_GUARD (m_rw_property_mutex);
    
    // old value of property_name
        CORBA::Any old_value;
        try
        {
        old_value = *get_property_value(property_name);
        }
        catch (const CosPropertyService::PropertyNotFound & p_not_found)
        {
        // if no property old value for property_name is present do nothing
        throw p_not_found;   
        }
            
        // Prepare RecordInfo
        CdmwPlatformMngt::PropertyInfo p_info;
        p_info.property_name = CORBA::string_dup(property_name);
        const char* property_deleted = "property_deleted";
        p_info.property_new_value <<= property_deleted;

        // Prepare PropertyInfos sequence
        CdmwPlatformMngt::PropertyInfos_var p_infos = new CdmwPlatformMngt::PropertyInfos();
        p_info.property_old_value = old_value;
        p_infos->length(1);
        p_infos[0L] = p_info;
        m_property_set_var->delete_property (property_name);
        set_element_in_datastore ();
    }

    

void ManagedElement_impl::delete_properties (const CosPropertyService::PropertyNames& property_names)
throw (CosPropertyService::MultipleExceptions, CORBA::SystemException)
{
        
        CDMW_WRITER_RECURSIVE_MUTEX_GUARD (m_rw_property_mutex);
    
    CosPropertyService::PropertyNames_var           property_names_found;
        CosPropertyService::PropertyNamesIterator_var   property_iterator; 
        // get the number of properties 
        CORBA::ULong number_of_properties = get_number_of_properties ();
        get_all_property_names (number_of_properties, property_names_found.out(), property_iterator.out());
        // Prepare RecordInfo
        CdmwPlatformMngt::PropertyInfo p_info;
        // Prepare PropertyInfos sequence
        CdmwPlatformMngt::PropertyInfos_var p_infos = new CdmwPlatformMngt::PropertyInfos();
        p_infos->length(0L);
        CORBA::ULong property_name_size = property_names_found->length();
        for (CORBA::ULong i=0; i< property_name_size; i++)
        {
        CORBA::Any old_value;
        try
        {
            old_value = *get_property_value(property_names_found[i]);               
            const char* property_deleted = "property_deleted";
            p_infos->length(p_infos->length() + 1);
            p_info.property_new_value <<= property_deleted;
            p_info.property_old_value = old_value;
            p_infos[i] = p_info;
            p_info.property_name = CORBA::string_dup(property_names_found[i]);
        }
        catch (const CosPropertyService::PropertyNotFound & p_not_found)
        {
            // if no property old value for current
            // property_name is present continue the loop
        }
        }
        try
        {
        m_property_set_var->delete_properties (property_names);
        }
        catch (...)
        {
        throw CosPropertyService::MultipleExceptions();
        }
            
        notify_property_change("delete_properties", p_infos);
        set_element_in_datastore ();
}

CORBA::Boolean ManagedElement_impl::delete_all_properties()
throw (CORBA::SystemException)
{
        
        CDMW_WRITER_RECURSIVE_MUTEX_GUARD (m_rw_property_mutex);
    
    CosPropertyService::PropertyNames_var           property_names_found;
        CosPropertyService::PropertyNamesIterator_var   property_iterator; 

        // get the number of properties 
        CORBA::ULong number_of_properties = get_number_of_properties ();
        get_all_property_names (number_of_properties, property_names_found.out(), property_iterator.out());

        // Prepare RecordInfo
        CdmwPlatformMngt::PropertyInfo p_info;

        // Prepare PropertyInfos sequence
        CdmwPlatformMngt::PropertyInfos_var p_infos = new CdmwPlatformMngt::PropertyInfos();
        p_infos->length(number_of_properties-m_default_user_defined_attributes->length());
        CORBA::ULong  index_of_deleted_element = 0;
        CORBA::Boolean ret_bool = true;
        for (CORBA::ULong i=0; i< number_of_properties; i++)
        {
            bool found = false;
            //if the properties is in m_default_user_defined_attributes
            int len_seq = m_default_user_defined_attributes->length();
            for (int ii=0; ii<len_seq && !found; ii++)
            {
                if (! (strcmp (CORBA::string_dup(property_names_found[i]), (*m_default_user_defined_attributes)[ii].property_name))) 
                {
                     found = true;
                }
            }    
            
            
            if(!found){
                CORBA::Any old_value;
          
                
                try
                {
                    old_value = *get_property_value(property_names_found[i]);               
                }   
                catch (const CosPropertyService::PropertyNotFound&)
                {
                   CDMW_WARN(m_logger, "PropertyNotFound trying to delete : " << property_names_found[i]);
                   ret_bool = false;
                } 
            
            const char* property_deleted = "property_deleted";
                    p_info.property_new_value <<= property_deleted;
                    p_info.property_old_value = old_value;
                    p_info.property_name = CORBA::string_dup(property_names_found[i]);
                    p_infos[index_of_deleted_element] = p_info;
                    index_of_deleted_element++;
                    try
                    {
                        delete_property(property_names_found[i]);
                    }
                    catch(const CosPropertyService::MultipleExceptions& ex)
                    {
                        ret_bool = false; 
                    }
                    catch (const CosPropertyService::FixedProperty& ex )
                    {
                        ret_bool = false;
                    }
            }
            /* MRI
            else{
            
                try
                        {
                            delete_property_without_notify(property_names_found[i]);
                        }
                        catch(const CosPropertyService::MultipleExceptions& ex)
                        {
                            ret_bool = false; 
                        }
                        catch (const CosPropertyService::FixedProperty& ex )
                        {
                            ret_bool = false;
                        }    
            }*/
        found = false;
        }
        notify_property_change("delete_all_properties", p_infos);
        set_element_in_datastore ();
        return ret_bool;
}

CORBA::Boolean ManagedElement_impl::is_property_defined (const char *property_name)
throw (CosPropertyService::InvalidPropertyName, CORBA::SystemException)
{
        
        CDMW_READER_RECURSIVE_MUTEX_GUARD (m_rw_property_mutex);
    
    return m_property_set_var->is_property_defined (property_name);
}

void ManagedElement_impl::get_allowed_property_types (CosPropertyService::PropertyTypes_out property_types)
throw (CORBA::SystemException)
{
        
        CDMW_READER_RECURSIVE_MUTEX_GUARD (m_rw_property_mutex);
    return m_property_set_var->get_allowed_property_types (property_types);
}

void ManagedElement_impl::get_allowed_properties (CosPropertyService::PropertyDefs_out property_defs)
throw (CORBA::SystemException)
{
        
        CDMW_READER_RECURSIVE_MUTEX_GUARD (m_rw_property_mutex);
    return m_property_set_var->get_allowed_properties (property_defs);
}

void ManagedElement_impl::define_property_with_mode (const char *property_name,
                   const CORBA::Any & property_value,
       CosPropertyService:: PropertyModeType property_mode)
throw (CosPropertyService::InvalidPropertyName,
           CosPropertyService::ConflictingProperty,
           CosPropertyService::UnsupportedTypeCode,
           CosPropertyService::UnsupportedProperty,
           CosPropertyService::UnsupportedMode,
           CosPropertyService::ReadOnlyProperty, CORBA::SystemException)
    {
        
        CDMW_WRITER_RECURSIVE_MUTEX_GUARD (m_rw_property_mutex);
    
        // Prepare RecordInfo
        CdmwPlatformMngt::PropertyInfo p_info;
        p_info.property_name = CORBA::string_dup(property_name);
        p_info.property_new_value = property_value;

        // Prepare PropertyInfos sequence
        CdmwPlatformMngt::PropertyInfos_var p_infos = new CdmwPlatformMngt::PropertyInfos();

        // old value of property_name
        CORBA::Any old_value;
        try
        {
        // p_info.property_old_value <<= * get_property_value(property_name);
        old_value = *get_property_value(property_name);
        }
        catch (const CosPropertyService::PropertyNotFound & p_not_found)
        {
        // if no property old value for property_name is present do nothing
        }
        p_info.property_old_value = old_value;
        p_infos->length(1);
        p_infos[0L] = p_info;

        m_property_set_var->define_property_with_mode (property_name, property_value, property_mode);
        notify_property_change("define_property_with_mode", p_infos);
        set_element_in_datastore ();
    }

void ManagedElement_impl::define_properties_with_modes (const CosPropertyService::PropertyDefs& property_defs)
throw (CosPropertyService::MultipleExceptions, CORBA::SystemException)
{
        
        CDMW_WRITER_RECURSIVE_MUTEX_GUARD (m_rw_property_mutex);
    
        // Prepare RecordInfo
        CdmwPlatformMngt::PropertyInfo p_info;
        // Prepare PropertyInfos sequence
        CdmwPlatformMngt::PropertyInfos_var p_infos = new CdmwPlatformMngt::PropertyInfos();
        // old value of property_name
        p_infos->length(0L);
        CORBA::ULong properties_size = property_defs.length();

        for (CORBA::ULong i=0; i< properties_size; i++)
        {
        CORBA::Any old_value;
        try
        {
            old_value = *get_property_value(property_defs[i].property_name);               
        }
        catch (const CosPropertyService::PropertyNotFound & p_not_found)
        {
            // if no property old value for current property_name is present do nothing 
            // and add a new property
        }
        p_infos->length(p_infos->length() + 1);
        p_info.property_name      = CORBA::string_dup(property_defs[i].property_name);
        p_info.property_new_value = property_defs[i].property_value;
        p_info.property_old_value = old_value;
        p_infos[i] = p_info;
        }
            try
            {
        m_property_set_var->define_properties_with_modes (property_defs);
        }
        catch (...)
        {
                throw CosPropertyService::MultipleExceptions();
            }
        notify_property_change("define_properties_with_modes", p_infos);
            set_element_in_datastore ();
}
        

void ManagedElement_impl::define_status_property (const char* init_value)
throw (CosPropertyService::MultipleExceptions, CORBA::SystemException)
{
    
    CDMW_WRITER_RECURSIVE_MUTEX_GUARD (m_rw_property_mutex);    

    std::ostringstream property_name;
    
    try
    {
        property_name.str("");
        property_name << CdmwPlatformMngt::CDMW_PREFIX << CdmwPlatformMngt::TECH_STATUS << std::ends;
        if(!is_property_defined(property_name.str().c_str()))
        {
            CORBA::Any_var any = new CORBA::Any();
            *any <<= CORBA::string_dup(init_value);
            define_property_with_mode(property_name.str().c_str() , any.in(), CosPropertyService::fixed_normal);
        }
        
        property_name.str("");
        property_name << CdmwPlatformMngt::CDMW_PREFIX << CdmwPlatformMngt::TECH_STATUS_INFO << std::ends;
        if(!is_property_defined(property_name.str().c_str()))
        {
            CORBA::Any_var any = new CORBA::Any();
            *any <<= CORBA::string_dup("");
            define_property_with_mode(property_name.str().c_str() , any.in(), CosPropertyService::fixed_normal);
        }
    }
    catch(...)
    {
      throw   CosPropertyService::MultipleExceptions();
    }
}

CosPropertyService::PropertyModeType ManagedElement_impl::get_property_mode (const char *property_name)
throw (CosPropertyService::PropertyNotFound,
        CosPropertyService::InvalidPropertyName, 
        CORBA::SystemException)
        {
        
            CDMW_READER_RECURSIVE_MUTEX_GUARD (m_rw_property_mutex);
            return m_property_set_var->get_property_mode (property_name);
        }

CORBA::Boolean ManagedElement_impl::get_property_modes (const CosPropertyService::PropertyNames & property_names,
        CosPropertyService::PropertyModes_out property_modes) 
throw (CORBA::SystemException)
    {
        
        CDMW_READER_RECURSIVE_MUTEX_GUARD (m_rw_property_mutex);
    return m_property_set_var->get_property_modes (property_names, property_modes);
    }

void ManagedElement_impl::set_property_mode (const char *property_name,
        CosPropertyService::PropertyModeType property_mode)
throw (CosPropertyService::InvalidPropertyName,
           CosPropertyService::PropertyNotFound,
           CosPropertyService::UnsupportedMode, CORBA::SystemException)
        {
        
    CDMW_WRITER_RECURSIVE_MUTEX_GUARD (m_rw_property_mutex);
    
    m_property_set_var->set_property_mode (property_name, property_mode);
    set_element_in_datastore ();
        }

void ManagedElement_impl::set_property_modes (const CosPropertyService::PropertyModes& property_modes)
throw (CosPropertyService::MultipleExceptions, CORBA::SystemException)
{
    
    CDMW_WRITER_RECURSIVE_MUTEX_GUARD (m_rw_property_mutex);

            try
            {
        m_property_set_var->set_property_modes (property_modes);
            }
            catch (...)
            {
                throw CosPropertyService::MultipleExceptions();
            }
            set_element_in_datastore ();
}
        
CdmwPlatformMngt::ManagedElementRecord* ManagedElement_impl::get_managed_element_record_from_datastore()
throw (Common::NotFoundException)
{
    CdmwPlatformMngt::ManagedElementRecord * managed_element_record = NULL;
    try
    {
        managed_element_record = m_managed_element_ds->select (m_element_path.in ());
            }
            catch (const Common::NotFoundException &)
            {
                throw;
            }
            catch (...)
            {
                CDMW_ASSERT (false);
            }
            
            return managed_element_record;
}
        
void ManagedElement_impl::synchro_with_datastore ()
{
            // synchronise with system record
            try
            {
                CdmwPlatformMngt::ManagedElementRecord_var managed_element_record =
                    get_managed_element_record_from_datastore ();
      
        *m_property_event_infos =   managed_element_record->property_event_infos;//->length(); 
        CORBA::ULong size = m_property_event_infos -> length();
        //m_property_event_infos->length(size);
        //*m_property_event_infos = managed_element_record->property_event_infos;

        for (CORBA::ULong i = 0; i < size ; i++)
        {
            std::string current_observer_name ( m_property_event_infos[i].observer_name);
            CdmwPlatformMngt::SupervisionObserver_var current_observer = m_property_event_infos[i].observer;
            bool is_for_all = m_property_event_infos[i].is_for_all;
                    try{
                if (is_for_all) {
                    register_all_properties_observer(current_observer_name.c_str(), current_observer.in(), 1);
                }
                else 
            {
                CosPropertyService::PropertyNames current_property_names;
                CORBA::ULong current_prop_length =  m_property_event_infos[i].property_names.length();
                current_property_names.length(current_prop_length);
                current_property_names          =  m_property_event_infos[i].property_names;
                register_property_observer(current_observer_name.c_str(), current_observer.in(),
                               current_property_names);
            }
                    }
                    catch(...)
                    {
                    }
        }
                // update record of EventRecordExt for registering ObserverExt  
                *m_event_ext_infos =   managed_element_record->event_ext_infos;//->length();
                CORBA::ULong size_ext = m_event_ext_infos -> length();
        //m_event_ext_infos->length(size_ext);
        for (CORBA::ULong i = 0; i < size_ext ; i++)
        {
            //std::string current_observer_name ( m_event_ext_infos[i].observer_name); PCR-1952-585
            std::string current_observer_name = CORBA::string_dup( m_property_event_infos[i].observer_name);
            CdmwPlatformMngt::SupervisionObserver_var current_observer = m_event_ext_infos[i].observer;
            CdmwPlatformMngt::RegistrationDepth depth = m_event_ext_infos[i].depth;
            
            try
            {
                register_observer_ext(current_observer_name.c_str(), depth, current_observer.in());
            }
            catch(const CdmwPlatformMngt::ElementAlreadyExists& ex)
            {
                try
                {
                    unregister_observer_ext(current_observer_name.c_str());
                    register_observer_ext(current_observer_name.c_str(), depth, current_observer.in());
                }
                catch(const CdmwPlatformMngt::ObserverNotFound& ex)
                {
                    CDMW_ERROR (Cdmw::Logging::Logger::Get_Logger (CdmwPlatformMngtBase::PFMMNG),
                            "Error during synchro_with_datastore() registering observer : "
                            <<current_observer_name);
                }
                catch(const CdmwPlatformMngt::ElementAlreadyExists& ex)
                {
                    CDMW_ERROR (Cdmw::Logging::Logger::Get_Logger (CdmwPlatformMngtBase::PFMMNG),
                            "Error during synchro_with_datastore() registering observer : "
                            <<current_observer_name);
                }
            }
            catch(const CORBA::SystemException &ex)
            {
                 CDMW_ERROR (Cdmw::Logging::Logger::Get_Logger (CdmwPlatformMngtBase::PFMMNG),
                            "Error during synchro_with_datastore() registering observer : "
                            <<current_observer_name);
            }
        }
        //store default properties
    
                
    {//guard scope
    CDMW_WRITER_RECURSIVE_MUTEX_GUARD (m_rw_property_mutex);
    
        *m_default_user_defined_attributes = managed_element_record->default_attributes;

        int len_seq = (managed_element_record->attributes).length() ;

        for (int ii=0; ii<len_seq; ii++)
          { //redefine the properties ...
           try
             {
                //if it is fixed or fixed normal
                if ( (managed_element_record->attributes)[ii].property_mode == CosPropertyService::fixed_normal 
                        || (managed_element_record->attributes)[ii].property_mode == CosPropertyService::fixed_readonly )
                { 
                    if ((strcmp (((managed_element_record->attributes)[ii]).property_name, "Cdmw.TechStatus")==0) ||
                            (strcmp (((managed_element_record->attributes)[ii]).property_name, "Cdmw.TechStatusInfo")==0))
                    {
                        // do not recover the TECH_STATUS and TECH_STATUS_INFO properties from datastore
                    }
                    else
                    {
                        define_property( ((managed_element_record->attributes)[ii]).property_name, 
                                ((managed_element_record->attributes)[ii]).property_value );
                    }
                }
                else // define also the mode
                { 
                    //if (!is_property_defined( (managed_element_record->attributes)[ii].property_name))
                    //{
                    define_property_with_mode( ((managed_element_record->attributes)[ii]).property_name, 
                            ((managed_element_record->attributes)[ii]).property_value, 
                            ((managed_element_record->attributes)[ii]).property_mode);
                    //}
                }
                 }
                       catch (  CosPropertyService::ReadOnlyProperty& e)
                 {
                CDMW_ERROR (Cdmw::Logging::Logger::Get_Logger (CdmwPlatformMngtBase::PFMMNG),
                       "Error during synchro_with_datastore() on property "
                       << (*m_default_user_defined_attributes)[ii].property_name
                       << ": property is read only");
             }
                       catch (...)
                 {
                CDMW_ERROR (Cdmw::Logging::Logger::Get_Logger (CdmwPlatformMngtBase::PFMMNG),
                       "Unexpected exception during synchro_with_datastore() on property "
                       << (*m_default_user_defined_attributes)[ii].property_name);
             }


           }
        }//end guard scope
       
             }
            catch (const Common::NotFoundException &)
            {
        // Element not found, This happens when the SystemManagement is the first primary
            }
}
        
void ManagedElement_impl::register_all_properties_observer (const char *observer_name,
        CdmwPlatformMngt::SupervisionObserver_ptr observer, CORBA::Boolean deeply) 
throw (CORBA::SystemException, CdmwPlatformMngt::ElementAlreadyExists)
{
                
    CDMW_READER_RECURSIVE_MUTEX_GUARD (m_rw_property_mutex);

        PropertyChangeAspect prop_aspect;

        PropertyEventNotifier* property_event_notifier = get_property_event_notifier();

    if(!property_event_notifier) {
        throw OutOfMemoryException();
    }

        try
        {
        // define register key
        prop_aspect.set_obsname((char*)observer_name);
        prop_aspect.set_register_for_all_properties(true);
        prop_aspect.set_targetelement_path(element_path());    
        // register observer
        register_observer_to_property_event_notifier(prop_aspect,observer);
        
        // update in local m_property_event_infos 
        if ( (observer != NULL) && (!observer->_is_local()) ) 
        {
        update_properties_record(build_record(observer_name,observer));
        }
    }
        catch (const BadParameterException&)
        {
        throw CORBA::BAD_PARAM(OrbSupport::BAD_PARAM,CORBA::COMPLETED_NO);
        }
        catch (const OutOfMemoryException&)
        {
        throw CORBA::NO_MEMORY(OrbSupport::NO_MEMORY,CORBA::COMPLETED_NO);
        }
        catch (const CdmwPlatformMngt::ElementAlreadyExists&)
        {
        throw;
        }

    if (deeply)
        { 
        try 
        {
            CdmwPlatformMngt::Children_var children = 0;
            children = get_children();

            if (children!=0)
            {
                for (CORBA::ULong i=0; i < children->length(); i++)
                {
                    children[i]-> register_all_properties_observer(observer_name,
                    CdmwPlatformMngt::SupervisionObserver::_duplicate(observer), deeply);
                }
            }
        }
        catch (...)
        {
            CDMW_WARN (Cdmw::Logging::Logger::Get_Logger (CdmwPlatformMngtBase::PFMMNG),
                       "Cannot deeply register observer " << observer_name <<
                       " on " << element_path());
        }
        }
        set_element_in_datastore();
            // TODO: Trace to the log manager
}

void ManagedElement_impl::unregister_all_properties_observer (const char *observer_name)
throw (CdmwPlatformMngt::ObserverNotFound, CORBA::SystemException)
{
                
    CDMW_READER_RECURSIVE_MUTEX_GUARD (m_rw_property_mutex);
    
    PropertyChangeAspect prop_aspect;
        PropertyEventNotifier* property_event_notifier = get_property_event_notifier();

    if(!property_event_notifier) {
        throw  OutOfMemoryException();
    }

        // define register key
        prop_aspect.set_obsname((char*)observer_name);
        prop_aspect.set_register_for_all_properties(true);
        prop_aspect.set_targetelement_path(element_path());
        // register observer
        try
        {
        unregister_observer_to_property_event_notifier(prop_aspect);
        remove_properties_record(build_record(observer_name,0));
        set_element_in_datastore();
        }
        catch (const CdmwPlatformMngt::PropertyObserverNotFound& e)
        {
        throw CdmwPlatformMngt::ObserverNotFound();
        }
}
        
void ManagedElement_impl::register_property_observer (const char *observer_name,
                CdmwPlatformMngt::SupervisionObserver_ptr observer,
                const CosPropertyService::PropertyNames & property_names)
throw (CdmwPlatformMngt::PropertyInListNotFound, CdmwPlatformMngt::ElementAlreadyExists, CORBA::SystemException)
{
                
    CDMW_READER_RECURSIVE_MUTEX_GUARD (m_rw_property_mutex);
    
    PropertyChangeAspect prop_aspect;
        PropertyEventNotifier* property_event_notifier = get_property_event_notifier();
            
    if(!property_event_notifier) {
        throw  OutOfMemoryException();
    }

        try
        {
        // define register key
        prop_aspect.set_obsname((char*)observer_name);
        prop_aspect.set_register_for_all_properties(false);
        prop_aspect.set_propertyset(property_names);
        prop_aspect.set_targetelement_path(element_path());
 
        // register observer
        register_observer_to_property_event_notifier(prop_aspect,observer);    
            
        // update in local m_property_event_infos
        if ( (observer != NULL) && (!observer->_is_local()) ) 
        {
        update_properties_record(build_record(observer_name, observer, property_names));
        set_element_in_datastore();
        }
    }
        catch (const BadParameterException&)
        {
        throw CORBA::BAD_PARAM(OrbSupport::BAD_PARAM,CORBA::COMPLETED_NO);
        }
        catch (const OutOfMemoryException&)
        {
        throw CORBA::NO_MEMORY(OrbSupport::NO_MEMORY,CORBA::COMPLETED_NO);
        }
    catch (const CdmwPlatformMngt::ElementAlreadyExists&)
        {
        throw;
        }
      
}
        
void ManagedElement_impl::unregister_property_observer (const char* observer_name,
                  const CosPropertyService::PropertyNames& property_names)
throw (CdmwPlatformMngt::PropertyInListNotFound, CdmwPlatformMngt::ObserverNotFound, CORBA::SystemException)
{
                
    CDMW_READER_RECURSIVE_MUTEX_GUARD (m_rw_property_mutex);
    
    PropertyEventNotifier* property_event_notifier = get_property_event_notifier();
        PropertyChangeAspect prop_aspect;
            
    if(!property_event_notifier) {
        throw  OutOfMemoryException();
    }
            
        try
        {
        // define register key
        prop_aspect.set_obsname((char*)observer_name);
        
        // update the key
        prop_aspect.set_propertyset(property_names); 
        prop_aspect.set_register_for_all_properties(false);
        prop_aspect.set_targetelement_path(element_path());
        
        // unregister observer
        unregister_observer_to_property_event_notifier(prop_aspect);
            
        // remove entry record (observer_name,property_names) from m_property_event_infos 
        remove_properties_record(build_record(observer_name,0,property_names));
        set_element_in_datastore();
        }
    catch (const CdmwPlatformMngt::PropertyObserverNotFound&)
            {
                throw CdmwPlatformMngt::ObserverNotFound();
            }
            catch (const BadParameterException&)
            {
                throw CORBA::BAD_PARAM(OrbSupport::BAD_PARAM,CORBA::COMPLETED_NO);
            }
        catch (const OutOfMemoryException&)
        {
        throw CORBA::NO_MEMORY(OrbSupport::NO_MEMORY,CORBA::COMPLETED_NO);
        }
        catch (const CdmwPlatformMngt::ElementAlreadyExists&)
        {
        throw;
        }
      
        // TODO: Notify the configuration change
            // TODO: Trace to the log manager
}
        
void ManagedElement_impl::remove_properties_record(const CdmwPlatformMngt::PropertyEventRecord&  record_to_remove)
{
    CORBA::ULong i=0;
    CORBA::ULong length = m_property_event_infos->length();
    for(i=0; i < length; ++i)
    {
        if( strcmp(m_property_event_infos[i].observer_name, record_to_remove.observer_name) == 0) break;    
        }
        if( i < length )
        {
        if( record_to_remove.is_for_all )
        {
            for (CORBA::ULong j=i+1; j < length; ++j) {
            m_property_event_infos[j-1] = m_property_event_infos[j];
            }
            m_property_event_infos->length(length-1);
        }
        else 
        {
            m_property_event_infos[i].is_for_all = false;
            
            //Create new PropertyNames to update m_property_event_infos[i].property_names
            // following a unregister operation
            CosPropertyService::PropertyNames property_names_new;
            CORBA::ULong length_new = 0;
            property_names_new.length(length_new);
            
            CORBA::ULong size_prop_infos_names = m_property_event_infos[i].property_names.length();
            CORBA::ULong size_prop_rec_names   = record_to_remove.property_names.length();
            
            //iterate through all elements of m_property_event_infos
            for (CORBA::ULong j = 0; j < size_prop_infos_names ; ++j)
            {
            //iterate through all elements of record_to_remove.property_names
            for (CORBA::ULong k = 0; k < size_prop_rec_names ; ++k)             
            {
                // if property_names[j,k] differ go on
                    if (!strcmp(m_property_event_infos[i].property_names[j], record_to_remove.property_names[k])) {
                continue;
                    }
                length_new = property_names_new.length();
                property_names_new.length(length_new+1);
                property_names_new[length_new] = m_property_event_infos[i].property_names[j];
                }
            }
            m_property_event_infos[i].property_names.length(property_names_new.length());    
            m_property_event_infos[i].property_names = property_names_new;
        }
        }
}

void ManagedElement_impl::update_record_ext(const CdmwPlatformMngt::EventRecordExt&  record_to_update)
{
    CORBA::ULong i=0;
    CORBA::ULong length = m_event_ext_infos->length();

    // find observer name
    for(i=0; i < length; ++i)
    {
        if( !strcmp(m_event_ext_infos[i].observer_name, record_to_update.observer_name)) break;
            }
            if (i < length)
            {
                //m_event_ext_infos[i].observer_name  = record_to_update.observer_name; PCR-1952
                m_event_ext_infos[i].observer_name  = CORBA::string_dup(record_to_update.observer_name);
                m_event_ext_infos[i].observer       = record_to_update.observer;
                m_event_ext_infos[i].depth         = record_to_update.depth;
            }
            else
            {
                m_event_ext_infos->length( length+1);
                //m_event_ext_infos[length].observer_name  = record_to_update.observer_name; PCR-1952
                m_event_ext_infos[length].observer       = record_to_update.observer; 
                m_event_ext_infos[length].observer_name  = CORBA::string_dup(record_to_update.observer_name);
                m_event_ext_infos[length].depth         = record_to_update.depth;
            }
}
  
void ManagedElement_impl::update_properties_record(const CdmwPlatformMngt::PropertyEventRecord&  record_to_update)
{
    CORBA::ULong i=0;
    CORBA::ULong length = m_property_event_infos->length();

        // find observer name
    for(i=0; i < length; ++i)
    {
        if( !strcmp(m_property_event_infos[i].observer_name, record_to_update.observer_name)) break;
        }
        // if not for all properties
        if( i < length )
        {
        // update property sequence
        if( record_to_update.is_for_all )
        {
            m_property_event_infos[i].is_for_all = true;
            m_property_event_infos[i].property_names.length(0);
        }
        else
        {
            m_property_event_infos[i].is_for_all = false;
            m_property_event_infos[i].property_names = record_to_update.property_names;
        }
        }
        else
        {
        m_property_event_infos->length( length+1);
        //m_property_event_infos[length].observer_name  = record_to_update.observer_name; PCR-1952
        m_property_event_infos[length].observer_name  = CORBA::string_dup(record_to_update.observer_name);
        m_property_event_infos[length].observer       = record_to_update.observer;
        m_property_event_infos[length].is_for_all     = record_to_update.is_for_all;
        m_property_event_infos[length].property_names.length(record_to_update.property_names.length());
        m_property_event_infos[length].property_names = record_to_update.property_names;
        }
}



void ManagedElement_impl::recover_default_properties ()
{
                
    CDMW_WRITER_RECURSIVE_MUTEX_GUARD (m_rw_property_mutex);
    
    delete_all_properties ();
        int len_seq = m_default_user_defined_attributes->length();
        for (int ii=0; ii<len_seq; ii++)
        { 
        try
        {
        // if it is fixed or fixed normal
            if ( (*m_default_user_defined_attributes)[ii].property_mode == CosPropertyService::fixed_normal ||
                 (*m_default_user_defined_attributes)[ii].property_mode == CosPropertyService::fixed_readonly )
            {
                     
                     if (strcmp ("Cdmw.RequestedRestartWarmth", (*m_default_user_defined_attributes)[ii].property_name)==0 ||
                         strcmp ("Cdmw.TechStatus", (*m_default_user_defined_attributes)[ii].property_name)==0 ||
                         strcmp ("Cdmw.TechStatusInfo", (*m_default_user_defined_attributes)[ii].property_name)==0) 
                     {
                        // do not overwrite this property
                     }
                     else {
                         define_property( (*m_default_user_defined_attributes)[ii].property_name, 
                                  (*m_default_user_defined_attributes)[ii].property_value );
                     }
                     
            }
            else
            {
                //if ( !(is_property_defined( (*m_default_user_defined_attributes)[ii].property_name )))
                //{
                         define_property_with_mode( (*m_default_user_defined_attributes)[ii].property_name, 
                              (*m_default_user_defined_attributes)[ii].property_value,
                      (*m_default_user_defined_attributes)[ii].property_mode );     
                //}
            }
        }
        catch (const CosPropertyService::ReadOnlyProperty& e)
             {
            //CDMW_ERROR (Cdmw::Logging::Logger::Get_Logger (CdmwPlatformMngtBase::PFMMNG),
            //           "Error during recover_default_properties() on property "
            //           << (*m_default_user_defined_attributes)[ii].property_name
            //           << ": property is read only");
             
             //Do nothing : ReadOnlyProperty do need to be recovered
             }
                   catch(...)
             {
            CDMW_ERROR (Cdmw::Logging::Logger::Get_Logger (CdmwPlatformMngtBase::PFMMNG),
                       "Unexpected exception during recover_default_properties() on property "
                       << (*m_default_user_defined_attributes)[ii].property_name);
             }
          }
            set_element_in_datastore ();
}
        
        
void ManagedElement_impl::store_default_properties ()
{
                    
        CDMW_READER_RECURSIVE_MUTEX_GUARD (m_rw_property_mutex);

            
            try 
            {
           
            // get all properties
            CosPropertyService::Properties_var props;
            CORBA::ULong propCount = get_number_of_properties ();
            CosPropertyService::PropertyNames_var prop_names;
            CosPropertyService::PropertyNamesIterator_var prop_names_It;
            get_all_property_names (propCount, prop_names.out (), prop_names_It.out ());
            get_properties (prop_names, props.out ());
    
            //prepare the PropertyDef variable 
            CosPropertyService::PropertyDefs_var propdefs = new CosPropertyService::PropertyDefs;
            propdefs->length (propCount);
            CosPropertyService::PropertyModes_var prop_modes;
            get_property_modes (prop_names.in (), prop_modes.out ());
            
            for (CORBA::ULong i = 0; i < propCount; i++)
                {
                    propdefs[i].property_name = prop_names[i];
                    propdefs[i].property_mode = prop_modes[i].property_mode;
                    propdefs[i].property_value = *get_property_value (props[i].property_name);
               }

                    
            // store properties default values 
            m_default_user_defined_attributes = new CosPropertyService::PropertyDefs (propdefs.in ());
            m_default_user_defined_attributes->length (propCount);
            }
            catch (const CosPropertyService::PropertyNotFound&)
            {
                CDMW_ERROR(m_logger, "PropertyNotFound while storing default properties");
                //MRI TODO MANAGE AS FATAL ERROR
            }
            
            set_element_in_datastore ();
}
        


void ManagedElement_impl::register_observer_to_property_event_notifier(PropertyChangeAspect& prop_aspect,
        CdmwPlatformMngt::SupervisionObserver_ptr observer) 
throw( BadParameterException, OutOfMemoryException, CdmwPlatformMngt::ElementAlreadyExists)
{
        try
        {
        if(!m_property_event_notifier) {
            CDMW_THROW(OutOfMemoryException);
        }
        // register observer and start Notifier Thread
        if (!(m_property_event_notifier->register_observer(prop_aspect,observer))) {
            CDMW_WARN (Cdmw::Logging::Logger::Get_Logger (CdmwPlatformMngtBase::PFMMNG),
                    "Cannot register observer to property event notifier: "
                    << prop_aspect.get_obsname());
        }
        else {
            CDMW_INFO(Cdmw::Logging::Logger::Get_Logger (CdmwPlatformMngtBase::PFMMNG),
                    "Property Observer "<< prop_aspect.get_obsname() << " registered on " << name());
        }

        if(!(m_property_event_notifier->is_running()))
        {
            m_property_event_notifier->start();
        }
        }
        catch (const BadParameterException&)
        {
        CDMW_THROW2(BadParameterException,"element path name", element_path() );
        }
        catch (const OutOfMemoryException&)
        {
        throw;
        }
        catch (const CdmwPlatformMngt::ElementAlreadyExists&)
        {
        throw;
        }
}



void ManagedElement_impl::notify_property_change(const char* info, const CdmwPlatformMngt::PropertyInfos* p_infos)
{
            
    CDMW_READER_RECURSIVE_MUTEX_GUARD (m_rw_property_mutex);
    
    CdmwPlatformMngt::PropertyInfos properties = *p_infos;
    CORBA::ULong size = properties.length();
    for (CORBA::ULong i=0; i<size; i++)
    {
        //Do not perform the notification for TECH_STATUS and TECH_STATUS_INFO properties
        if ((strcmp (properties[i].property_name, "Cdmw.TechStatus")==0) ||
                (strcmp (properties[i].property_name, "Cdmw.TechStatusInfo")==0))
        {
            return;
        }
    }
    
    CdmwPlatformMngtBase::EventHeader header = EventHeaderFactory::createHeader (CdmwPlatformMngtBase::INF);
    CdmwPlatformMngt::PropertyChange_var event =
        m_propertyChangeFactory->create (header,element_path(), *p_infos, info);

        if(m_property_event_notifier) {
            m_property_event_notifier->add_prop_event (event);
        }
        event._retn ();
}

void ManagedElement_impl::notify_configuration_change(const char* info_operation)
{
    CdmwPlatformMngtBase::EventHeader header = EventHeaderFactory::createHeader (CdmwPlatformMngtBase::INF);
        CdmwPlatformMngt::ConfigurationChange_var event =
        m_configurationChangeFactory->create (header,element_path(), info_operation);
    
        m_status_event_notifier =  EventNotifierManager::Instance()->get_status_notifier(); 
    if (m_status_event_notifier) {
        m_status_event_notifier->add_event (event._retn());
    }

        event._retn ();
}


void ManagedElement_impl::unregister_observer_to_property_event_notifier(PropertyChangeAspect& prop_aspect)
throw(BadParameterException,OutOfMemoryException,CdmwPlatformMngt::PropertyObserverNotFound)
{
    string target_element_path = prop_aspect.get_targetelement_path();

        try
        {
        if(!m_property_event_notifier) {
            CDMW_THROW(OutOfMemoryException);
        }    
        m_property_event_notifier->unregister_observer(prop_aspect);
        }
        catch (const BadParameterException&)
        {
        CDMW_THROW2(BadParameterException,"element path name",target_element_path);
        }
        catch (const OutOfMemoryException&)
        {
        CDMW_THROW(OutOfMemoryException);
        }
            catch (const CdmwPlatformMngt::PropertyObserverNotFound& e)
            {
        throw;
    }
    CDMW_INFO(Cdmw::Logging::Logger::Get_Logger (CdmwPlatformMngtBase::PFMMNG),
        "Property Observer: "<< prop_aspect.get_obsname() << " unregistered from " << name());        
}

void  ManagedElement_impl::register_observer_ext(const char* observer_name,CdmwPlatformMngt::RegistrationDepth depth, CdmwPlatformMngt::SupervisionObserver_ptr observer) 
throw (CORBA::SystemException, CdmwPlatformMngt::ElementAlreadyExists)
{

    try
    {

        //Register the Observer on StatusEventNotifier
        EventNotifierManager::Instance()->get_status_notifier()->register_observer (observer_name, 
                element_path(), CdmwPlatformMngt::SupervisionObserver::_duplicate(observer));
        //Register the Observer on CommandEventNotifier        
        EventNotifierManager::Instance()->get_command_notifier()->register_observer (observer_name, 
                element_path(), CdmwPlatformMngt::SupervisionObserver::_duplicate(observer));
        //Registering the PropertyObserver always  
        // with is_for_all=false because the registration is recursive when depth=FULL_HIERARCHY       
        register_all_properties_observer(observer_name, 
                CdmwPlatformMngt::SupervisionObserver::_duplicate(observer), false);
    }
    catch (const CdmwPlatformMngt::ElementAlreadyExists& ex)
    {
       observer->_remove_ref();
        throw;
    }
    catch (const BadParameterException& ex)
    {
        CDMW_ERROR(Cdmw::Logging::Logger::Get_Logger (CdmwPlatformMngtBase::PFMMNG),
            "Status Observer "<< observer_name << " registered on " << name() << " exception: " << ex.get_info() );
        throw CORBA::BAD_PARAM(OrbSupport::BAD_PARAM, CORBA::COMPLETED_NO);
    
    }
    if (CdmwPlatformMngt::FULL_HIERARCHY == depth)
    { 
        CdmwPlatformMngt::Children_var children = 0;
        children = get_children(); 
        if (children!=0) 
        { 
            for (CORBA::ULong i=0; i < children->length(); i++)
            {   
                // Managing the ElementAlreadyExist exception
                if(CdmwPlatformMngt::HOST_TYPE==children[i]->get_element_type())
                {
                    try
                    {
                        children[i]->register_observer_ext(observer_name, depth,
                                CdmwPlatformMngt::SupervisionObserver::_duplicate(observer));
                    }
                    catch(const CdmwPlatformMngt::ElementAlreadyExists& ex)
                    {
                       /* Do nothing, see PCR-775 description.
                        * Host has its own register_observer_ext method, 
                        * all processes have been already registered.
                        */
                    }
                }
                else
                {
                    try
                    { 
                        children[i]->register_observer_ext(observer_name, depth,  
                                CdmwPlatformMngt::SupervisionObserver::_duplicate(observer));
                    }
                    catch (const CdmwPlatformMngt::ElementAlreadyExists& ex)
                    {
                        observer->_remove_ref();
                        throw ;
                    }
                }
            } //for
        }

    }  //if FULL_HIERARCHY        
    if ( (observer != NULL) && (!observer->_is_local()) ) 
    {
        update_record_ext(build_record(observer_name,depth, observer));
        set_element_in_datastore();
    }
}


void ManagedElement_impl::unregister_observer_ext(const char* observer_name)
throw (CORBA::SystemException, CdmwPlatformMngt::ObserverNotFound)
{
    int observer_not_found = 3;
    try
    {
        //Unregister the observer on StatusEventNotifier
        EventNotifierManager::Instance()->get_status_notifier()->unregister_observer (observer_name, element_path());
    }
    catch(const CdmwPlatformMngt::ObserverNotFound & ex)
    {
        --observer_not_found ;
    }
    try
    {  
        //Unregister the observer on CommandEventNotifier
        EventNotifierManager::Instance()->get_command_notifier()->unregister_observer (observer_name, element_path());
    }
    catch(const CdmwPlatformMngt::ObserverNotFound & ex)
    {
        --observer_not_found ;
    } 
    try
    {     
        //Unregister all properties observer
        unregister_all_properties_observer(observer_name);
    }
    catch(const CdmwPlatformMngt::ObserverNotFound & ex)
    {
        --observer_not_found ;
    }    
   
    if (0 == observer_not_found ) 
    {
        throw CdmwPlatformMngt::ObserverNotFound();
    }
    
    CdmwPlatformMngt::Children_var children;
    children = get_children();
    if (children!=0)
    { 
        for (CORBA::ULong i=0; i < children->length(); i++)
        {
            if(CdmwPlatformMngt::HOST_TYPE==children[i]->get_element_type())
            {
                try
                {
                    children[i]->unregister_observer_ext (observer_name);
                }
                catch(const  CdmwPlatformMngt::ObserverNotFound& ex)
                {
                    /* Do nothing,  see PCR-775 description.
                     * Host has its own register_observer_ext method, 
                     * all processes have been already registered.
                     */ 
                }

            }
            else
            {
                try
                {
                    children[i]->unregister_observer_ext (observer_name);
                }
                catch(const  CdmwPlatformMngt::ObserverNotFound& ex)
                {
                    // case depth == OBJECT _ONLY !DO NOTHING!
                    break;
                }
            }
        }
    }
}
    
   
CdmwPlatformMngt::SupervisionObserver_ptr ManagedElement_impl::register_observer(const char* observer_name,
        CdmwPlatformMngt::SupervisionObserver_ptr observer) 
throw (CORBA::SystemException)
{
    /*CDMW_ASSERT( (CORBA::is_nil(observer)) && "ManagedElement_impl::register_observer - BAD_PARAM The observer reference is NULL");

    if(CORBA::is_nil(observer))
    {
        LogMngr::logMessage (ERR,"ManagedElement_impl::register_observer - BAD_PARAM The observer reference is NULL"); 
        throw CORBA::BAD_PARAM(OrbSupport::BAD_PARAM, CORBA::COMPLETED_NO);
    } */

    try
    {    
    EventNotifierManager::Instance()->get_status_notifier()->register_observer (observer_name,
                element_path(), CdmwPlatformMngt::SupervisionObserver::_duplicate(observer));

        CDMW_INFO(Cdmw::Logging::Logger::Get_Logger (CdmwPlatformMngtBase::PFMMNG),
                "Status Observer "<< observer_name << "registered on " << name());
       return CdmwPlatformMngt::SupervisionObserver::_nil();
    }
    catch(const CdmwPlatformMngt::ElementAlreadyExists & ex)
    {
        //Getting all the observers registered for the current ManagedElement
        std::vector<ObserverInfo> obs_vect = 
            EventNotifierManager::Instance()->get_status_notifier()->get_observers(element_path());
        CdmwPlatformMngt::SupervisionObserver_ptr previous_obs;
        //Looking for the observer with <observer_name> 
       for(int i=0; i < (int)obs_vect.size() ; i++)
        {
            if( 0 == strcmp( (obs_vect[i].observer_name).c_str(),observer_name ) )
            {
                try
                {
                    previous_obs=CdmwPlatformMngt::SupervisionObserver::_duplicate((obs_vect[i]).observer_ptr);
                    //Unregistering the old observer with the same name
                    this->unregister_observer(observer_name);
                    //Trying to register again the observer
                    EventNotifierManager::Instance()->get_status_notifier()->register_observer (observer_name,
                         element_path(), CdmwPlatformMngt::SupervisionObserver::_duplicate(observer));
                    return previous_obs;
                                
                }
                catch(const CdmwPlatformMngt::ObserverNotFound& ex)
                {
                    throw CORBA::INTERNAL(OrbSupport::INTERNAL,CORBA::COMPLETED_NO)  ;
                }
                catch(const CdmwPlatformMngt::ElementAlreadyExists & ex)
                {
                    throw CORBA::INTERNAL(OrbSupport::INTERNAL,CORBA::COMPLETED_NO);
                }
                
            }
        }
       
        
        return CdmwPlatformMngt::SupervisionObserver::_nil();
    }
} 
         
CdmwPlatformMngt::SupervisionObserver_ptr ManagedElement_impl::unregister_observer(const char* observer_name)
throw (CdmwPlatformMngt::ObserverNotFound, CORBA::SystemException)
{
        CdmwPlatformMngt::SupervisionObserver_ptr ptrrev = CdmwPlatformMngt::SupervisionObserver::_nil();
        CdmwPlatformMngt::SupervisionObserver_var ptrobj = ptrrev;

        EventNotifierManager::Instance()->get_status_notifier()->unregister_observer (observer_name,
                                              element_path());
    
    CDMW_INFO(Cdmw::Logging::Logger::Get_Logger (CdmwPlatformMngtBase::PFMMNG),
            "Status Observer "<< observer_name << "unregistered from " << name());

    return ptrobj._retn();
}    


CdmwPlatformMngt::SupervisionObserver_ptr ManagedElement_impl::register_proxy_observer(
       const char* observer_name, const char* host_name, CdmwPlatformMngt::SupervisionObserver_ptr observer)
throw (CORBA::SystemException)
{
            /*
             * Not iplemented - raises NO_IMPLEMENT exception. The
             * operation is deprecated.
             */
            throw CORBA::NO_IMPLEMENT (OrbSupport::NO_IMPLEMENT, CORBA::COMPLETED_NO);
} 

void ManagedElement_impl::set_requested_restart_warmth(CdmwPlatformMngt::RestartWarmth requested_restart_warmth)
{
    CORBA::Any_var any_var = new CORBA::Any();
    *any_var <<= requested_restart_warmth;
    try {
        set_attribute_internal (CdmwPlatformMngt::REQUESTED_RESTART_WARMTH, any_var.in(), 
                CdmwPlatformMngt::PRE_DEFINED);
    }
    catch(...) {
        throw;
    }
}

CdmwPlatformMngt::RestartWarmth ManagedElement_impl::requested_restart_warmth ()
throw (CORBA::SystemException)
{
    CdmwPlatformMngt::RestartWarmth rw;
    CORBA::Any_var any_var;
    try {
        get_attribute (CdmwPlatformMngt::REQUESTED_RESTART_WARMTH, any_var.out(), CdmwPlatformMngt::PRE_DEFINED);
        any_var.in() >>= rw;
    }
    catch(const CORBA::SystemException&) { 
        throw;
    }
    catch(...) {
        throw;
    }                
    return rw;
}

void ManagedElement_impl::set_default_restart_warmth (CdmwPlatformMngt::RestartWarmth default_restart_warmth )
throw (CORBA::SystemException, CdmwPlatformMngt::IncompatibleStatus)
{

    CORBA::Any_var any_var = new CORBA::Any();
    *any_var <<= default_restart_warmth;
    try {
        set_attribute_internal (CdmwPlatformMngt::DEFAULT_RESTART_WARMTH, any_var.in(), CdmwPlatformMngt::PRE_DEFINED);
    }
    catch(...) { 
        throw;
    }

    //TODO check for incompatible status
}


CdmwPlatformMngt::RestartWarmth ManagedElement_impl::get_default_restart_warmth ()
throw(CORBA::SystemException)
{
    CdmwPlatformMngt::RestartWarmth drw;
    CORBA::Any_var any_var;
    try {
        get_attribute (CdmwPlatformMngt::DEFAULT_RESTART_WARMTH, any_var.out(), CdmwPlatformMngt::PRE_DEFINED);
        any_var.in() >>= drw;
    }
    catch(...) { 
        throw;
    }
    return drw;
}


void  ManagedElement_impl::recover_default_entities (Cdmw::PlatformMngt::RWEntityContainer_impl* rwEntityContainer,CdmwPlatformMngt::EntityDefs* default_entitydefs)
{
           
    
    m_default_entitydefs = default_entitydefs; 

    // recover default entityDef
    CORBA::ULong how_many = rwEntityContainer->get_number_of_entities ();
    CdmwPlatformMngt::ElementNames_var element_names;
    CdmwPlatformMngt::ElementNamesIterator_var element_names_iterator;
    rwEntityContainer->get_all_entity_names (how_many, element_names.out(), element_names_iterator.out());

    //delete all entities, except the default system's entity
    std::string def_sys_entity = CORBA::string_dup("CdmwSystemSynchronization");
    CORBA::ULong size = element_names->length ();
    for (CORBA::ULong count = 0; count < size; count++)
    {
        if ( 0 != def_sys_entity.compare (element_names[count]))
        {
            rwEntityContainer->remove_entity (element_names[count]);
        }
    }

    //restore the size of default entities
    element_names->length (m_default_entitydefs->length ());
    for (CORBA::ULong count = 0; count < element_names->length(); count++)
    {
        rwEntityContainer->add_entity ((*m_default_entitydefs)[count]);
    }
}

void ManagedElement_impl::set_attribute_internal (const char* attr_name, const CORBA::Any& attr, 
        CdmwPlatformMngt::Owner flag)  
throw ( CORBA::SystemException, CdmwPlatformMngt::AttributeNotFound, Cdmw::BadOrderException, CdmwPlatformMngt::ReadOnlyAttribute)
{
           
   CDMW_WRITER_RECURSIVE_MUTEX_GUARD (m_rw_property_mutex);
    
    Cdmw::Logging::Logger * m_logger =
             Cdmw::Logging::Logger::Get_Logger(CdmwPlatformMngtBase::PFMMNG);
    
    if (CdmwPlatformMngt::PRE_DEFINED == flag )
    {
        std::ostringstream property_name;
        property_name << CdmwPlatformMngt::CDMW_PREFIX << attr_name << std::ends;

        //check if the predefined property exists
        try
        {
            if (false == is_property_defined (property_name.str().c_str()) )
            {
                throw CdmwPlatformMngt::AttributeNotFound(); 
            }
        }
        catch (const CosPropertyService::InvalidPropertyName& ex) 
        {
            CDMW_ERROR(m_logger,"Exception: " <<ex );
            throw CORBA::BAD_PARAM(OrbSupport::BAD_PARAM, CORBA::COMPLETED_NO);
        }

        // Ok to set the predefined property
        try
        {
            define_property (property_name.str().c_str(), attr);
        }
        catch(const CosPropertyService::ConflictingProperty& ex)
        {
            CDMW_ERROR(m_logger,"Exception: " <<ex );
            CDMW_THROW(BadOrderException);
        }
        catch(const CosPropertyService::UnsupportedTypeCode& ex)
        {
            CDMW_ERROR(m_logger,"Exception: " <<ex );
            CDMW_THROW(BadOrderException);
        }
        catch(const CosPropertyService::UnsupportedProperty& ex)
        {
            CDMW_ERROR(m_logger,"Exception: " <<ex );
            CDMW_THROW(BadOrderException);
        }
        catch(const CosPropertyService::ReadOnlyProperty& ex)
        {
            CDMW_ERROR(m_logger,"Exception: " <<ex );
            CDMW_THROW(BadOrderException);  
        }
    catch (const CosPropertyService::InvalidPropertyName& ex) 
        {
            CDMW_ERROR(m_logger,"Exception: " <<ex );
            throw CORBA::BAD_PARAM(OrbSupport::BAD_PARAM, CORBA::COMPLETED_NO);
        }
        catch(...)
        {
            throw ;
        }
        std::string tmp1 (attr_name);

       /* if (tmp1 == CdmwPlatformMngt::DEFAULT_RESTART_WARMTH )
        {
            set_requested_restart_warmth(get_default_restart_warmth());
        }*/
    }
    else
    {
    
        //check if the userdefined property exists
        try
        {
            if (false == is_property_defined (attr_name) )
            {
                throw CdmwPlatformMngt::AttributeNotFound(); 
            } 
        }
        catch (const CosPropertyService::InvalidPropertyName& ex) 
        {
            CDMW_ERROR(m_logger,"Exception: " <<ex );
            throw CORBA::BAD_PARAM(OrbSupport::BAD_PARAM, CORBA::COMPLETED_NO);
        }
        
        // Ok to set the userdefined property
        // case of user defined attributes
        try
        {
            define_property (attr_name, attr);
        }
        catch (const CosPropertyService::ReadOnlyProperty&)
        {
            throw CdmwPlatformMngt::ReadOnlyAttribute();
        }
        catch(const CosPropertyService::ConflictingProperty& ex)
        {
            CDMW_ERROR(m_logger,"Exception: " <<ex );
            CDMW_THROW(BadOrderException);
        }
        catch(const CosPropertyService::UnsupportedTypeCode& ex)
        {
            CDMW_ERROR(m_logger,"Exception: " <<ex );
            CDMW_THROW(BadOrderException);
        }
        catch(const CosPropertyService::UnsupportedProperty& ex)
        {
            CDMW_ERROR(m_logger,"Exception: " <<ex );
            CDMW_THROW(BadOrderException);
        }
        catch(...)
        {
            throw ;
        } 
    }
}

}                // End namespace PlatformMngt
}                // End namespace Cdmw


