/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */



#include <Foundation/orbsupport/ExceptionMinorCodes.hpp>

#include <SystemMngt/idllib/CdmwPlatformMngtBase.stub.hpp>
#include <SystemMngt/idllib/CdmwPlatformMngtDaemon.stub.hpp>
#include <SystemMngt/idllib/CdmwPlatformMngtEvent.stub.hpp>
#include <SystemMngt/idllib/CdmwPlatformMngtSystem.stub.hpp>
#include <Foundation/logging/FunctionLogger.hpp>

#include "platformftproxy/Application_impl.hpp"

const char* const LOGGER_NAME = "CDMW.PFMDMN.SupProxy.Application";

namespace Cdmw
{
namespace PlatformMngt
{
namespace Proxy
{


// ----------------------------------------------------------------------
// Application_impl class.
// ----------------------------------------------------------------------
Application_impl::Application_impl(CORBA::ORB_ptr orb,
                   ProxyCollection *proxy_collection,
                   AccessorLookupService *lookup_service)
    throw (Cdmw::BadParameterException, 
       Cdmw::OutOfMemoryException)                 
    : ManagedElement_impl(orb),
      m_accessor(CdmwFTSystMngt::ApplicationAccessor::_nil()),
      m_accessor_ok(false),
      m_proxy_collection(proxy_collection),
      m_lookup_service(lookup_service)
{    
    m_proxy_collection->add_ref();
    set_logger (Get_Logger ());
    set_managed_element (this);
    set_proxy_collection (proxy_collection);
}

CdmwFTSystMngt::ManagedElementAccessor_ptr Application_impl::
        accessor_managed_element () throw (CORBA::TRANSIENT)
      {
         return this->accessor();
      }

Cdmw::Logging::Logger* Application_impl::Get_Logger () {
        static Cdmw::Logging::Logger* logger =
            Cdmw::Logging::Logger::Get_Logger(LOGGER_NAME);
        return logger;
    }
    
const CdmwFTSystMngt::ApplicationAccessor_ptr 
Application_impl::accessor() 
    throw(CORBA::TRANSIENT) 
{
    if (!m_accessor_ok) {
    
    CDMW_MUTEX_GUARD(m_accessor_mutex);
    // leaving mutex other thread might have fetched the data
    // already

    try {

        if (CORBA::is_nil(m_accessor.in())) {
        m_accessor = 
            m_lookup_service->getApplicationAccessor(); // raises AccessorUndefined
        // update the flag
        m_accessor_ok = ! CORBA::is_nil(m_accessor.in());
        }

    }
    catch (...) {
        throw CORBA::TRANSIENT(::Cdmw::OrbSupport::TRANSIENT,
                   CORBA::COMPLETED_NO);
    }
    
    // paranoid testing
    if (CORBA::is_nil(m_accessor.in())) {
        throw CORBA::TRANSIENT(::Cdmw::OrbSupport::TRANSIENT,
                   CORBA::COMPLETED_NO);
    }

    // releasing mutex here
    }

    CDMW_ASSERT(m_accessor_ok &&
        ! CORBA::is_nil(m_accessor.in()));

    return m_accessor.ptr(); // return const ptr
}


Application_impl::~Application_impl() throw ()
{
    m_proxy_collection->remove_ref();
}

      
CORBA::Long Application_impl::restart_attempt ()
throw(CORBA::SystemException)
{
    CDMW_LOG_FUNCTION(Get_Logger(), "");
    try {
        std::string target = this->getObjectId();
    
        return accessor()->get_restart_attempt(target.c_str());
    } 
    catch (const CdmwPlatformMngt::ManagedElementNotFound & ex) {
        CORBA::OBJECT_NOT_EXIST e(OrbSupport::OBJECT_NOT_EXIST, 
                                  CORBA::COMPLETED_NO);
        CDMW_LOG_FUNCTION_EXCEPTION(e);
        throw e;
    }
    catch (const CORBA::Exception & ex) {
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw;
    }
    catch (...) {
        CORBA::INTERNAL ex(Cdmw::OrbSupport::INTERNAL, 
                           CORBA::COMPLETED_MAYBE);
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw ex;
    }
}
        
CdmwPlatformMngt::ApplicationStatus Application_impl::get_status(
                   CORBA::String_out status_info)
throw(CORBA::SystemException)    
{
    CDMW_LOG_FUNCTION(Get_Logger(), "");
    try {
        std::string target = this->getObjectId();
    
        return accessor()->get_status(target.c_str(), status_info);
    } 
    catch (const CdmwPlatformMngt::ManagedElementNotFound & ex) {
        CORBA::OBJECT_NOT_EXIST e(OrbSupport::OBJECT_NOT_EXIST, 
                                  CORBA::COMPLETED_NO);
        CDMW_LOG_FUNCTION_EXCEPTION(e);
        throw e;
    }
    catch (const CORBA::Exception & ex) {
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw;
    }
    catch (...) {
        CORBA::INTERNAL ex(Cdmw::OrbSupport::INTERNAL, 
                           CORBA::COMPLETED_MAYBE);
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw ex;
    }
}

CdmwPlatformMngt::FunctioningMode Application_impl::get_mode(
    CORBA::String_out mode_info)
throw(CORBA::SystemException)
{
    CDMW_LOG_FUNCTION(Get_Logger(), "");
    try {
        std::string target = this->getObjectId();
    
        return accessor()->get_mode(target.c_str(), mode_info);
    } 
    catch (const CdmwPlatformMngt::ManagedElementNotFound & ex) {
        CORBA::OBJECT_NOT_EXIST e(OrbSupport::OBJECT_NOT_EXIST, 
                                  CORBA::COMPLETED_NO);
        CDMW_LOG_FUNCTION_EXCEPTION(e);
        throw e;
    }
    catch (const CORBA::Exception & ex) {
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw;
    }
    catch (...) {
        CORBA::INTERNAL ex(Cdmw::OrbSupport::INTERNAL, 
                           CORBA::COMPLETED_MAYBE);
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw ex;
    }
}

CdmwPlatformMngt::ApplicationSnapshot* Application_impl::get_snapshot()
throw(CORBA::SystemException)
{
    CDMW_LOG_FUNCTION(Get_Logger(), "");
    try {
        std::string target = this->getObjectId();
        
        return accessor()->get_snapshot(target.c_str());
    } 
    catch (const CdmwPlatformMngt::ManagedElementNotFound & ex) {
        CORBA::OBJECT_NOT_EXIST e(OrbSupport::OBJECT_NOT_EXIST, 
                                  CORBA::COMPLETED_NO);
        CDMW_LOG_FUNCTION_EXCEPTION(e);
        throw e;
    }
    catch (const CORBA::Exception & ex) {
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw;
    }
    catch (...) {
        CORBA::INTERNAL ex(Cdmw::OrbSupport::INTERNAL, 
                           CORBA::COMPLETED_MAYBE);
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw ex;
    }
}


CORBA::ULong Application_impl::get_number_of_processes()
throw(CORBA::SystemException)
{
    CDMW_LOG_FUNCTION(Get_Logger(), "");
    try {
    std::string target = this->getObjectId();

        return accessor()->get_number_of_processes(target.c_str());
    }
    catch (const CdmwPlatformMngt::ManagedElementNotFound & ex) {
        CORBA::OBJECT_NOT_EXIST e(OrbSupport::OBJECT_NOT_EXIST, 
                                  CORBA::COMPLETED_NO);
        CDMW_LOG_FUNCTION_EXCEPTION(e);
        throw e;
    }
    catch (const CORBA::Exception & ex) {
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw;
    }
    catch (...) {
        CORBA::INTERNAL ex(Cdmw::OrbSupport::INTERNAL, 
                           CORBA::COMPLETED_MAYBE);
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw ex;
    }
}


void Application_impl::set_indicator_relevant_status(const bool indicator_relevant_status) throw(CORBA::SystemException)
{
    try {   
        std::string target = this->getObjectId();

        accessor()->set_indicator_relevant_status(target.c_str(), indicator_relevant_status);
    }
    catch (const CdmwPlatformMngt::ManagedElementNotFound & ex) { 
        throw CORBA::OBJECT_NOT_EXIST(OrbSupport::OBJECT_NOT_EXIST, 
                                      CORBA::COMPLETED_NO);
    }
    catch (const  CORBA::SystemException & ex) { 
        throw;
    }
    catch (...) { 
        throw CORBA::INTERNAL(Cdmw::OrbSupport::INTERNAL, 
                              CORBA::COMPLETED_MAYBE);
    }
}


bool Application_impl::get_indicator_relevant_status() throw(CORBA::SystemException)
{
    try {   
        std::string target = this->getObjectId();

        return accessor()->get_indicator_relevant_status(target.c_str());
    }
    catch (const CdmwPlatformMngt::ManagedElementNotFound & ex) { 
        throw CORBA::OBJECT_NOT_EXIST(OrbSupport::OBJECT_NOT_EXIST, 
                                      CORBA::COMPLETED_NO);
    }
    catch (const  CORBA::SystemException & ex) { 
        throw;
    }
    catch (...) { 
        throw CORBA::INTERNAL(Cdmw::OrbSupport::INTERNAL, 
                              CORBA::COMPLETED_MAYBE);
    }
}



CdmwPlatformMngt::Processes* Application_impl::get_processes(
    CORBA::ULong how_many,
    CdmwPlatformMngt::ProcessesIterator_out processes_iterator)
throw(CORBA::SystemException)
{
    CDMW_LOG_FUNCTION(Get_Logger(), "");
    try { 
        std::string target = this->getObjectId();
         
    
        CdmwPlatformMngt::ElementPathsIterator_var iter;
    
        CdmwPlatformMngt::ElementPaths_var paths = 
            accessor()->get_processes(target.c_str(), 
                          how_many,
                          iter.out());
        
        CdmwPlatformMngt::ProcessesIterator_var tmp_processes_iterator = 
            this->m_proxy_collection->createProcessesIteratorRef(iter.in());
        
        CdmwPlatformMngt::Processes_var tmp_processes =
            this->m_proxy_collection->createProcessesFromSeq(paths.ptr());
    
        processes_iterator = tmp_processes_iterator._retn();
    
        return tmp_processes._retn();
    } 
    catch (const CdmwPlatformMngt::ManagedElementNotFound & ex) {
        CORBA::OBJECT_NOT_EXIST e(OrbSupport::OBJECT_NOT_EXIST, 
                                  CORBA::COMPLETED_NO);
        CDMW_LOG_FUNCTION_EXCEPTION(e);
        throw e;
    }
    catch (const CORBA::Exception & ex) {
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw;
    }
    catch (...) {
        CORBA::INTERNAL ex(Cdmw::OrbSupport::INTERNAL, 
                           CORBA::COMPLETED_MAYBE);
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw ex;
    }
}

CdmwPlatformMngt::Process_ptr Application_impl::get_process(
                const char* process_name,
                const char* host_name)
throw(CdmwPlatformMngt::ProcessNotFound,
      CORBA::SystemException)
{
     CDMW_LOG_FUNCTION(Get_Logger(), "process_name=" << process_name);
    try {  
    std::string target = this->getObjectId();
    
    CORBA::String_var proc_target = 
        accessor()->get_process (target.c_str(),
                     process_name,
                     host_name); 
    
    return this->m_proxy_collection->createProcessRef(proc_target);
    }
    catch (const CdmwPlatformMngt::ManagedElementNotFound & ex) {
        CORBA::OBJECT_NOT_EXIST e(OrbSupport::OBJECT_NOT_EXIST, 
                                  CORBA::COMPLETED_NO);
        CDMW_LOG_FUNCTION_EXCEPTION(e);
        throw e;
    }
    catch (const CORBA::Exception & ex) {
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw;
    }
    catch (...) {
        CORBA::INTERNAL ex(Cdmw::OrbSupport::INTERNAL, 
                           CORBA::COMPLETED_MAYBE);
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw ex;
    }
}

void Application_impl::define(
    const CdmwPlatformMngt::ApplicationDef& application_def)
throw(CdmwPlatformMngt::DuplicateEntity,
      CdmwPlatformMngt::DuplicateService,
      CdmwPlatformMngt::DuplicateProcess,
      CdmwPlatformMngt::ProcessHostNotFound,
      CdmwPlatformMngt::DuplicateProcessEntity,
      CdmwPlatformMngt::DuplicateProcessService,
      CdmwPlatformMngt::DuplicateProcessStep,
      CdmwPlatformMngt::AlreadyDone,
      CdmwPlatformMngt::IncompatibleStatus,
      CosPropertyService::MultipleExceptions,
      CORBA::SystemException)
{
    CDMW_LOG_FUNCTION(Get_Logger(), "");
    try {  
        std::string target = this->getObjectId();
        
        accessor()->define (target.c_str(),
                    application_def);
    } 
    catch (const CdmwPlatformMngt::ManagedElementNotFound & ex) {
        CORBA::OBJECT_NOT_EXIST e(OrbSupport::OBJECT_NOT_EXIST, 
                                  CORBA::COMPLETED_NO);
        CDMW_LOG_FUNCTION_EXCEPTION(e);
        throw e;
    }
    catch (const CORBA::Exception & ex) {
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw;
    }
    catch (...) {
        CORBA::INTERNAL ex(Cdmw::OrbSupport::INTERNAL, 
                           CORBA::COMPLETED_MAYBE);
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw ex;
    }
}
    CdmwPlatformMngt::FTGroup*
     Application_impl::add_ftgroup(const
      CdmwPlatformMngt::FTGroupDef& ftgroup_def) throw (CORBA::SystemException,
      CdmwPlatformMngt::GroupAlreadyExists, CdmwPlatformMngt::DuplicateEntity,
      CosPropertyService::MultipleExceptions, CdmwPlatformMngt::IncompatibleStatus)
       {
           CDMW_LOG_FUNCTION(Get_Logger(), "");
            try {  
                std::string target = this->getObjectId();
                
                CORBA::String_var ftgroup_target = 
                    accessor()->add_ftgroup (target.c_str(),
                            ftgroup_def);
                
                return this->m_proxy_collection->createFTGroupRef(ftgroup_target);
            } 
            catch (const CdmwPlatformMngt::ManagedElementNotFound & ex) {
                CORBA::OBJECT_NOT_EXIST e(OrbSupport::OBJECT_NOT_EXIST, 
                                          CORBA::COMPLETED_NO);
                CDMW_LOG_FUNCTION_EXCEPTION(e);
                throw e;
            }
            catch (const CORBA::Exception & ex) {
                CDMW_LOG_FUNCTION_EXCEPTION(ex);
                throw;
            }
            catch (...) {
                CORBA::INTERNAL ex(Cdmw::OrbSupport::INTERNAL, 
                                   CORBA::COMPLETED_MAYBE);
                CDMW_LOG_FUNCTION_EXCEPTION(ex);
                throw ex;
            }
        
       
       }
       

CdmwPlatformMngt::Process_ptr Application_impl::add_process(
    CdmwPlatformMngt::ProcessDef* process_def)
throw (CdmwPlatformMngt::ProcessAlreadyExists,
       CdmwPlatformMngt::HostNotFound,
       CdmwPlatformMngt::DuplicateEntity,
       CdmwPlatformMngt::DuplicateService,
       CdmwPlatformMngt::DuplicateStep,
       CdmwPlatformMngt::IncompatibleStatus,
       CosPropertyService::MultipleExceptions,
       CORBA::SystemException)
{
    CDMW_LOG_FUNCTION(Get_Logger(), "");
    try {  
        std::string target = this->getObjectId();
        
        CORBA::String_var proc_target = 
            accessor()->add_process (target.c_str(),
                         process_def);
        
        return this->m_proxy_collection->createProcessRef(proc_target);
    } 
    catch (const CdmwPlatformMngt::ManagedElementNotFound & ex) {
        CORBA::OBJECT_NOT_EXIST e(OrbSupport::OBJECT_NOT_EXIST, 
                                  CORBA::COMPLETED_NO);
        CDMW_LOG_FUNCTION_EXCEPTION(e);
        throw e;
    }
    catch (const CORBA::Exception & ex) {
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw;
    }
    catch (...) {
        CORBA::INTERNAL ex(Cdmw::OrbSupport::INTERNAL, 
                           CORBA::COMPLETED_MAYBE);
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw ex;
    }
}

void Application_impl::remove_process(const char* process_name,
                                      const char* host_name)
throw (CdmwPlatformMngt::ProcessNotFound,
       CdmwPlatformMngt::ProcessStillActive,
       CdmwPlatformMngt::IncompatibleStatus,
       CORBA::SystemException)
{
    CDMW_LOG_FUNCTION(Get_Logger(), "process_name=" << process_name);
    try {  
        std::string target = this->getObjectId();
        
        accessor()->remove_process (target.c_str(),
                        process_name,
                        host_name);
    } 
    catch (const CdmwPlatformMngt::ManagedElementNotFound & ex) {
        CORBA::OBJECT_NOT_EXIST e(OrbSupport::OBJECT_NOT_EXIST, 
                                  CORBA::COMPLETED_NO);
        CDMW_LOG_FUNCTION_EXCEPTION(e);
        throw e;
    }
    catch (const CORBA::Exception & ex) {
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw;
    }
    catch (...) {
        CORBA::INTERNAL ex(Cdmw::OrbSupport::INTERNAL, 
                           CORBA::COMPLETED_MAYBE);
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw ex;
    }
}

void Application_impl::set_init_graph(
    const CdmwPlatformMngt::GraphElements& process_graph)
throw (CdmwPlatformMngt::ReferenceNotFound,
       CdmwPlatformMngt::CircularReference,
       CdmwPlatformMngt::StepOutOfBound,
       CdmwPlatformMngt::StepJump,
       CdmwPlatformMngt::IncompatibleStatus,
       CORBA::SystemException)
{
    CDMW_LOG_FUNCTION(Get_Logger(), "");
    try {  
        std::string target = this->getObjectId();
        
        accessor()->set_init_graph (target.c_str(),
                        process_graph);
    } 
    catch (const CdmwPlatformMngt::ManagedElementNotFound & ex) {
        CORBA::OBJECT_NOT_EXIST e(OrbSupport::OBJECT_NOT_EXIST, 
                                  CORBA::COMPLETED_NO);
        CDMW_LOG_FUNCTION_EXCEPTION(e);
        throw e;
    }
    catch (const CORBA::Exception & ex) {
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw;
    }
    catch (...) {
        CORBA::INTERNAL ex(Cdmw::OrbSupport::INTERNAL, 
                           CORBA::COMPLETED_MAYBE);
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw ex;
    }
}

void Application_impl::set_stop_graph(
    const CdmwPlatformMngt::GraphElements& process_graph)
throw( CdmwPlatformMngt::ReferenceNotFound,
       CdmwPlatformMngt::CircularReference,
       CdmwPlatformMngt::IncompatibleStatus,
       CORBA::SystemException)
{
    CDMW_LOG_FUNCTION(Get_Logger(), "");
    try {  
        std::string target = this->getObjectId();
        
        accessor()->set_stop_graph (target.c_str(),
                        process_graph);
    } 
    catch (const CdmwPlatformMngt::ManagedElementNotFound & ex) {
        CORBA::OBJECT_NOT_EXIST e(OrbSupport::OBJECT_NOT_EXIST, 
                                  CORBA::COMPLETED_NO);
        CDMW_LOG_FUNCTION_EXCEPTION(e);
        throw e;
    }
    catch (const CORBA::Exception & ex) {
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw;
    }
    catch (...) {
        CORBA::INTERNAL ex(Cdmw::OrbSupport::INTERNAL, 
                           CORBA::COMPLETED_MAYBE);
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw ex;
    }
}



void Application_impl::initialise(
    const CdmwPlatformMngtBase::StartupKind& startup_kind)
throw (CdmwPlatformMngt::IncompatibleStatus,
       CORBA::SystemException)
{
    CDMW_LOG_FUNCTION(Get_Logger(), "");
    try {  
        std::string target = this->getObjectId();
        
        accessor()->initialise (target.c_str(),
                    startup_kind);
    } 
    catch (const CdmwPlatformMngt::ManagedElementNotFound & ex) {
        CORBA::OBJECT_NOT_EXIST e(OrbSupport::OBJECT_NOT_EXIST, 
                                  CORBA::COMPLETED_NO);
        CDMW_LOG_FUNCTION_EXCEPTION(e);
        throw e;
    }
    catch (const CORBA::Exception & ex) {
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw;
    }
    catch (...) {
        CORBA::INTERNAL ex(Cdmw::OrbSupport::INTERNAL, 
                           CORBA::COMPLETED_MAYBE);
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw ex;
    }
}

CdmwPlatformMngt::LongRequestStatus Application_impl::initialise_and_wait(
    const CdmwPlatformMngtBase::StartupKind& startup_kind)
throw (CdmwPlatformMngt::IncompatibleStatus,
       CORBA::SystemException )
{
    CDMW_LOG_FUNCTION(Get_Logger(), "");
    try {  
        std::string target = this->getObjectId();
        
        return accessor()->initialise_and_wait(target.c_str(),
                               startup_kind);
    } 
    catch (const CdmwPlatformMngt::ManagedElementNotFound & ex) {
        CORBA::OBJECT_NOT_EXIST e(OrbSupport::OBJECT_NOT_EXIST, 
                                  CORBA::COMPLETED_NO);
        CDMW_LOG_FUNCTION_EXCEPTION(e);
        throw e;
    }
    catch (const CORBA::Exception & ex) {
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw;
    }
    catch (...) {
        CORBA::INTERNAL ex(Cdmw::OrbSupport::INTERNAL, 
                           CORBA::COMPLETED_MAYBE);
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw ex;
    }
}

void Application_impl::resume()
throw (CdmwPlatformMngt::IncompatibleStatus,
       CORBA::SystemException)
{
    CDMW_LOG_FUNCTION(Get_Logger(), "");
    try {  
        std::string target = this->getObjectId();
        
        accessor()->resume(target.c_str());
    } 
    catch (const CdmwPlatformMngt::ManagedElementNotFound & ex) {
        CORBA::OBJECT_NOT_EXIST e(OrbSupport::OBJECT_NOT_EXIST, 
                                  CORBA::COMPLETED_NO);
        CDMW_LOG_FUNCTION_EXCEPTION(e);
        throw e;
    }
    catch (const CORBA::Exception & ex) {
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw;
    }
    catch (...) {
        CORBA::INTERNAL ex(Cdmw::OrbSupport::INTERNAL, 
                           CORBA::COMPLETED_MAYBE);
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw ex;
    }
}

CdmwPlatformMngt::LongRequestStatus Application_impl::resume_and_wait()
throw (CdmwPlatformMngt::IncompatibleStatus,
       CORBA::SystemException)
{
    CDMW_LOG_FUNCTION(Get_Logger(), "");
    try {  
        std::string target = this->getObjectId();
        
        return accessor()->resume_and_wait(target.c_str());
    } 
    catch (const CdmwPlatformMngt::ManagedElementNotFound & ex) {
        CORBA::OBJECT_NOT_EXIST e(OrbSupport::OBJECT_NOT_EXIST, 
                                  CORBA::COMPLETED_NO);
        CDMW_LOG_FUNCTION_EXCEPTION(e);
        throw e;
    }
    catch (const CORBA::Exception & ex) {
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw;
    }
    catch (...) {
        CORBA::INTERNAL ex(Cdmw::OrbSupport::INTERNAL, 
                           CORBA::COMPLETED_MAYBE);
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw ex;
    }
}

void Application_impl::acknowledge()
throw (CdmwPlatformMngt::IncompatibleStatus,
       CORBA::SystemException)
{
    CDMW_LOG_FUNCTION(Get_Logger(), "");

    //get issuer value
    std::string command_issuer (m_current->get_issuer());
    
    try {  
        std::string target = this->getObjectId();
        
        accessor()->acknowledge(target.c_str(),
                                         command_issuer.c_str());
    } 
    catch (const CdmwPlatformMngt::ManagedElementNotFound & ex) {
        CORBA::OBJECT_NOT_EXIST e(OrbSupport::OBJECT_NOT_EXIST, 
                                  CORBA::COMPLETED_NO);
        CDMW_LOG_FUNCTION_EXCEPTION(e);
        throw e;
    }
    catch (const CORBA::Exception & ex) {
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw;
    }
    catch (...) {
        CORBA::INTERNAL ex(Cdmw::OrbSupport::INTERNAL, 
                           CORBA::COMPLETED_MAYBE);
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw ex;
    }
}

void Application_impl::run()
throw (CdmwPlatformMngt::IncompatibleStatus,
       CORBA::SystemException)
{
    CDMW_LOG_FUNCTION(Get_Logger(), "");
    try {  
        std::string target = this->getObjectId();
        
        accessor()->run(target.c_str());
    } 
    catch (const CdmwPlatformMngt::ManagedElementNotFound & ex) {
        CORBA::OBJECT_NOT_EXIST e(OrbSupport::OBJECT_NOT_EXIST, 
                                  CORBA::COMPLETED_NO);
        CDMW_LOG_FUNCTION_EXCEPTION(e);
        throw e;
    }
    catch (const CORBA::Exception & ex) {
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw;
    }
    catch (...) {
        CORBA::INTERNAL ex(Cdmw::OrbSupport::INTERNAL, 
                           CORBA::COMPLETED_MAYBE);
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw ex;
    }
}

void Application_impl::stop(CORBA::Boolean emergency)
throw(CORBA::SystemException)
{
    CDMW_LOG_FUNCTION(Get_Logger(), "");
    try {  
        std::string target = this->getObjectId();
        
        accessor()->stop(target.c_str(), emergency);
    } 
    catch (const CdmwPlatformMngt::ManagedElementNotFound & ex) {
        CORBA::OBJECT_NOT_EXIST e(OrbSupport::OBJECT_NOT_EXIST, 
                                  CORBA::COMPLETED_NO);
        CDMW_LOG_FUNCTION_EXCEPTION(e);
        throw e;
    }
    catch (const CORBA::Exception & ex) {
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw;
    }
    catch (...) {
        CORBA::INTERNAL ex(Cdmw::OrbSupport::INTERNAL, 
                           CORBA::COMPLETED_MAYBE);
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw ex;
    }
}

CdmwPlatformMngt::LongRequestStatus Application_impl::stop_and_wait(
    CORBA::Boolean emergency)
throw(CORBA::SystemException)
{
    CDMW_LOG_FUNCTION(Get_Logger(), "");
    try {  
        std::string target = this->getObjectId();
        
        return accessor()->stop_and_wait(target.c_str(), emergency);
    } 
    catch (const CdmwPlatformMngt::ManagedElementNotFound & ex) {
        CORBA::OBJECT_NOT_EXIST e(OrbSupport::OBJECT_NOT_EXIST, 
                                  CORBA::COMPLETED_NO);
        CDMW_LOG_FUNCTION_EXCEPTION(e);
        throw e;
    }
    catch (const CORBA::Exception & ex) {
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw;
    }
    catch (...) {
        CORBA::INTERNAL ex(Cdmw::OrbSupport::INTERNAL, 
                           CORBA::COMPLETED_MAYBE);
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw ex;
    }
}

void Application_impl::cold_restart(
    const CdmwPlatformMngtBase::StartupKind& startup_kind)
throw(CdmwPlatformMngt::IncompatibleStatus,
      CORBA::SystemException)
{
    CDMW_LOG_FUNCTION(Get_Logger(), "");

    //get issuer value
    std::string command_issuer (m_current->get_issuer());

    try {  
        std::string target = this->getObjectId();
        
        accessor()->cold_restart(target.c_str(),
                                 startup_kind,
                                 command_issuer.c_str());
    } 
    catch (const CdmwPlatformMngt::ManagedElementNotFound & ex) {
        CORBA::OBJECT_NOT_EXIST e(OrbSupport::OBJECT_NOT_EXIST, 
                                  CORBA::COMPLETED_NO);
        CDMW_LOG_FUNCTION_EXCEPTION(e);
        throw e;
    }
    catch (const CORBA::Exception & ex) {
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw;
    }
    catch (...) {
        CORBA::INTERNAL ex(Cdmw::OrbSupport::INTERNAL, 
                           CORBA::COMPLETED_MAYBE);
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw ex;
    }
}
      
            


void Application_impl::hot_restart(
    const CdmwPlatformMngtBase::StartupKind& startup_kind)
throw(CdmwPlatformMngt::IncompatibleStatus,
      CORBA::SystemException)
{
    CDMW_LOG_FUNCTION(Get_Logger(), "");

    //get issuer value
    std::string command_issuer (m_current->get_issuer());

    try {  
        std::string target = this->getObjectId();
        
        accessor()->hot_restart(target.c_str(), 
                                startup_kind,
                                command_issuer.c_str());
    } 
    catch (const CdmwPlatformMngt::ManagedElementNotFound & ex) {
        CORBA::OBJECT_NOT_EXIST e(OrbSupport::OBJECT_NOT_EXIST, 
                                  CORBA::COMPLETED_NO);
        CDMW_LOG_FUNCTION_EXCEPTION(e);
        throw e;
    }
    catch (const CORBA::Exception & ex) {
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw;
    }
    catch (...) {
        CORBA::INTERNAL ex(Cdmw::OrbSupport::INTERNAL, 
                           CORBA::COMPLETED_MAYBE);
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw ex;
    }
}
 
 
 
 

CORBA::ULong Application_impl::get_number_of_entities()
throw(CORBA::SystemException)
{
    CDMW_LOG_FUNCTION(Get_Logger(), "");
    try {  
        std::string target = this->getObjectId();
         
        return accessor()->get_number_of_entities(target.c_str());  
    } 
    catch (const CdmwPlatformMngt::ManagedElementNotFound & ex) {
        CORBA::OBJECT_NOT_EXIST e(OrbSupport::OBJECT_NOT_EXIST, 
                                  CORBA::COMPLETED_NO);
        CDMW_LOG_FUNCTION_EXCEPTION(e);
        throw e;
    }
    catch (const CORBA::Exception & ex) {
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw;
    }
    catch (...) {
        CORBA::INTERNAL ex(Cdmw::OrbSupport::INTERNAL, 
                           CORBA::COMPLETED_MAYBE);
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw ex;
    }
}

void Application_impl::get_all_entity_names(
    CORBA::ULong how_many,
    CdmwPlatformMngt::ElementNames_out entity_names,
    CdmwPlatformMngt::ElementNamesIterator_out rest)
throw(CORBA::SystemException)
{  
    CDMW_LOG_FUNCTION(Get_Logger(), "");
    try {  
        std::string target = this->getObjectId();
          
        accessor()->get_all_entity_names(target.c_str(),
                         how_many,
                         entity_names,
                         rest); 
    } 
    catch (const CdmwPlatformMngt::ManagedElementNotFound & ex) {
        CORBA::OBJECT_NOT_EXIST e(OrbSupport::OBJECT_NOT_EXIST, 
                                  CORBA::COMPLETED_NO);
        CDMW_LOG_FUNCTION_EXCEPTION(e);
        throw e;
    }
    catch (const CORBA::Exception & ex) {
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw;
    }
    catch (...) {
        CORBA::INTERNAL ex(Cdmw::OrbSupport::INTERNAL, 
                           CORBA::COMPLETED_MAYBE);
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw ex;
    }
}

CdmwPlatformMngt::Entity_ptr Application_impl::get_entity (
    const char* entity_name)
throw (CdmwPlatformMngt::EntityNotFound,
       CORBA::SystemException)
{
    CDMW_LOG_FUNCTION(Get_Logger(), "entity_name=" << entity_name);
    try {  
        std::string target = this->getObjectId();
          
        CORBA::String_var entity_target =
            accessor()->get_entity(target.c_str(), entity_name);
        
        return this->m_proxy_collection->createEntityRef(entity_target);
    } 
    catch (const CdmwPlatformMngt::ManagedElementNotFound & ex) {
        CORBA::OBJECT_NOT_EXIST e(OrbSupport::OBJECT_NOT_EXIST, 
                                  CORBA::COMPLETED_NO);
        CDMW_LOG_FUNCTION_EXCEPTION(e);
        throw e;
    }
    catch (const CORBA::Exception & ex) {
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw;
    }
    catch (...) {
        CORBA::INTERNAL ex(Cdmw::OrbSupport::INTERNAL, 
                           CORBA::COMPLETED_MAYBE);
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw ex;
    }
}

CORBA::Boolean Application_impl::get_entities(
    const CdmwPlatformMngt::ElementNames& entity_names,
    CdmwPlatformMngt::Entities_out nentities)
throw(CORBA::SystemException)
{
    CDMW_LOG_FUNCTION(Get_Logger(), "");
    try {   
        std::string target = this->getObjectId();
          
        ::CdmwPlatformMngt::ElementPaths_var paths = 
              new ::CdmwPlatformMngt::ElementPaths();
        
        CdmwPlatformMngt::ElementNames_var tmp_entity_names;
    
        CORBA::Boolean succeed = 
            accessor()->get_entities(target.c_str(),
                         entity_names,
                         paths.out());
    
        if (succeed) {
            nentities =
            this->m_proxy_collection->createEntitiesFromSeq(paths.ptr());
        }
            
        return succeed;
    } 
    catch (const CdmwPlatformMngt::ManagedElementNotFound & ex) {
        CORBA::OBJECT_NOT_EXIST e(OrbSupport::OBJECT_NOT_EXIST, 
                                  CORBA::COMPLETED_NO);
        CDMW_LOG_FUNCTION_EXCEPTION(e);
        throw e;
    }
    catch (const CORBA::Exception & ex) {
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw;
    }
    catch (...) {
        CORBA::INTERNAL ex(Cdmw::OrbSupport::INTERNAL, 
                           CORBA::COMPLETED_MAYBE);
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw ex;
    }
}

void Application_impl::get_all_entities(
    CORBA::ULong how_many,
    CdmwPlatformMngt::Entities_out nentities,
    CdmwPlatformMngt::EntitiesIterator_out rest)
throw(CORBA::SystemException)
{  
    CDMW_LOG_FUNCTION(Get_Logger(), "");
    try {  
        std::string target = this->getObjectId();
          
        ::CdmwPlatformMngt::ElementPaths_var         paths;
        ::CdmwPlatformMngt::ElementPathsIterator_var iter;
        
        accessor()->get_all_entities(target.c_str(),
                         how_many,
                         paths.out(),
                         iter.out());
        
        CdmwPlatformMngt::Entities_var tmp_nentities = 
            this->m_proxy_collection->createEntitiesFromSeq(paths.ptr());
        
        CdmwPlatformMngt::EntitiesIterator_var tmp_rest =
            this->m_proxy_collection->createEntitiesIteratorRef(iter.in());
        
        nentities = tmp_nentities._retn();
        rest      = tmp_rest._retn();
    } 
    catch (const CdmwPlatformMngt::ManagedElementNotFound & ex) {
        CORBA::OBJECT_NOT_EXIST e(OrbSupport::OBJECT_NOT_EXIST, 
                                  CORBA::COMPLETED_NO);
        CDMW_LOG_FUNCTION_EXCEPTION(e);
        throw e;
    }
    catch (const CORBA::Exception & ex) {
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw;
    }
    catch (...) {
        CORBA::INTERNAL ex(Cdmw::OrbSupport::INTERNAL, 
                           CORBA::COMPLETED_MAYBE);
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw ex;
    }
}

    
CdmwPlatformMngt::Entity_ptr Application_impl::add_entity (
     const CdmwPlatformMngt::EntityDef & entity_def)
throw(CdmwPlatformMngt::EntityAlreadyExists,
      CORBA::SystemException)
{
    CDMW_LOG_FUNCTION(Get_Logger(), "entity_name=" << entity_def.entity_name);
    try {  
        std::string target = this->getObjectId();
    
        CORBA::String_var uid = 
            accessor()->add_entity(target.c_str(),
                       entity_def);
        
        return this->m_proxy_collection->createEntityRef(uid);
    } 
    catch (const CdmwPlatformMngt::ManagedElementNotFound & ex) {
        CORBA::OBJECT_NOT_EXIST e(OrbSupport::OBJECT_NOT_EXIST, 
                                  CORBA::COMPLETED_NO);
        CDMW_LOG_FUNCTION_EXCEPTION(e);
        throw e;
    }
    catch (const CORBA::Exception & ex) {
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw;
    }
    catch (...) {
        CORBA::INTERNAL ex(Cdmw::OrbSupport::INTERNAL, 
                           CORBA::COMPLETED_MAYBE);
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw ex;
    }
}

void Application_impl::remove_entity(
    const char* entity_name)
throw (CdmwPlatformMngt::EntityNotFound,
       CORBA::SystemException)
{  
    CDMW_LOG_FUNCTION(Get_Logger(), "entity_name=" << entity_name);
    try {  
        std::string target = this->getObjectId();
    
        accessor()->remove_entity(target.c_str(),
                      entity_name);
    } 
    catch (const CdmwPlatformMngt::ManagedElementNotFound & ex) {
        CORBA::OBJECT_NOT_EXIST e(OrbSupport::OBJECT_NOT_EXIST, 
                                  CORBA::COMPLETED_NO);
        CDMW_LOG_FUNCTION_EXCEPTION(e);
        throw e;
    }
    catch (const CORBA::Exception & ex) {
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw;
    }
    catch (...) {
        CORBA::INTERNAL ex(Cdmw::OrbSupport::INTERNAL, 
                           CORBA::COMPLETED_MAYBE);
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw ex;
    }
}

CdmwPlatformMngtService::ServiceProviderID* Application_impl::get_service_def(
    const char* service_name)
throw (CdmwPlatformMngtService::ServiceNotFound,
       CORBA::SystemException)
{
    CDMW_LOG_FUNCTION(Get_Logger(), "service_name=" << service_name);
    try {    
        std::string target = this->getObjectId();
    
        return accessor()->get_service_def(target.c_str(),
                           service_name);
    } 
    catch (const CdmwPlatformMngt::ManagedElementNotFound & ex) {
        CORBA::OBJECT_NOT_EXIST e(OrbSupport::OBJECT_NOT_EXIST, 
                                  CORBA::COMPLETED_NO);
        CDMW_LOG_FUNCTION_EXCEPTION(e);
        throw e;
    }
    catch (const CORBA::Exception & ex) {
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw;
    }
    catch (...) {
        CORBA::INTERNAL ex(Cdmw::OrbSupport::INTERNAL, 
                           CORBA::COMPLETED_MAYBE);
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw ex;
    }
}

void Application_impl::add_service_def(
    const CdmwPlatformMngtService::ServiceDef& service_def)
throw (CdmwPlatformMngtService::ServiceAlreadyExists,
       CORBA::SystemException)
{
    CDMW_LOG_FUNCTION(Get_Logger(), "service_name=" << service_def.service_name);
    try {    
        std::string target = this->getObjectId();
    
        accessor()->add_service_def(target.c_str(),
                        service_def);
    } 
    catch (const CdmwPlatformMngt::ManagedElementNotFound & ex) {
        CORBA::OBJECT_NOT_EXIST e(OrbSupport::OBJECT_NOT_EXIST, 
                                  CORBA::COMPLETED_NO);
        CDMW_LOG_FUNCTION_EXCEPTION(e);
        throw e;
    }
    catch (const CORBA::Exception & ex) {
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw;
    }
    catch (...) {
        CORBA::INTERNAL ex(Cdmw::OrbSupport::INTERNAL, 
                           CORBA::COMPLETED_MAYBE);
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw ex;
    }
}

void Application_impl::remove_service_def(
    const char* service_name)
throw (CdmwPlatformMngtService::ServiceNotFound,
       CORBA::SystemException)
{
    CDMW_LOG_FUNCTION(Get_Logger(), "service_name=" << service_name);
    try {    
        std::string target = this->getObjectId();
    
        accessor()->remove_service_def(target.c_str(),
                           service_name);
    } 
    catch (const CdmwPlatformMngt::ManagedElementNotFound & ex) {
        CORBA::OBJECT_NOT_EXIST e(OrbSupport::OBJECT_NOT_EXIST, 
                                  CORBA::COMPLETED_NO);
        CDMW_LOG_FUNCTION_EXCEPTION(e);
        throw e;
    }
    catch (const CORBA::Exception & ex) {
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw;
    }
    catch (...) {
        CORBA::INTERNAL ex(Cdmw::OrbSupport::INTERNAL, 
                           CORBA::COMPLETED_MAYBE);
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw ex;
    }
}

void 
Application_impl::remove_all_service_defs ()
    throw (CORBA::SystemException)
{ 
    CDMW_LOG_FUNCTION(Get_Logger(), "");
    try {    
        std::string target = this->getObjectId();
    
        accessor()->remove_all_service_defs(target.c_str());
    } 
    catch (const CdmwPlatformMngt::ManagedElementNotFound & ex) {
        CORBA::OBJECT_NOT_EXIST e(OrbSupport::OBJECT_NOT_EXIST, 
                                  CORBA::COMPLETED_NO);
        CDMW_LOG_FUNCTION_EXCEPTION(e);
        throw e;
    }
    catch (const CORBA::Exception & ex) {
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw;
    }
    catch (...) {
        CORBA::INTERNAL ex(Cdmw::OrbSupport::INTERNAL, 
                           CORBA::COMPLETED_MAYBE);
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw ex;
    }
}

void Application_impl::is_alive()
  throw(CORBA::SystemException)
{
    CDMW_LOG_FUNCTION(Get_Logger(), "");
    /*
     * Not implemented - raises NO_IMPLEMENT exception. The
     * operation is deprecated.
     */
    throw CORBA::NO_IMPLEMENT (OrbSupport::NO_IMPLEMENT, CORBA::COMPLETED_NO);
}


void Application_impl::stop_application(CORBA::Boolean emergency)
throw (CORBA::SystemException)
{
    CDMW_LOG_FUNCTION(Get_Logger(), "");

    std::string command_issuer = "";

    //get issuer value
    std::string issuer (m_current->get_issuer());

    try
    {
        std::string target = this->getObjectId();

        accessor()->stop_application (target.c_str(),
                                     emergency,
                                     command_issuer.c_str());
    } 
    catch (const CdmwPlatformMngt::ManagedElementNotFound & ex) {
        CORBA::OBJECT_NOT_EXIST e(OrbSupport::OBJECT_NOT_EXIST, 
                CORBA::COMPLETED_NO);
        CDMW_LOG_FUNCTION_EXCEPTION(e);
        throw e;
    }
    catch (const CORBA::Exception & ex) 
    {
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw;
    }
    catch (...) {
        CORBA::INTERNAL ex(Cdmw::OrbSupport::INTERNAL, 
                CORBA::COMPLETED_MAYBE);
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw ex;
    }
}

void Application_impl::get_all_processes(CORBA::ULong how_many, 
                    CdmwPlatformMngt::Processes_out nprocesses, 
                    CdmwPlatformMngt::ProcessesIterator_out rest)   
    throw (CORBA::SystemException)
{
    throw CORBA::NO_IMPLEMENT (OrbSupport::NO_IMPLEMENT, CORBA::COMPLETED_NO);
}

/**
   *Purpose:
   *<p>
   * Implements the
   * IDL:thalesgroup.com/CdmwPlatformMngt/Application/get_all_ft_groups:1.0
   * operation
   */
 void Application_impl::get_all_ft_groups(CORBA::ULong how_many, 
            CdmwPlatformMngt::FTGroups_out nftgroups, 
            CdmwPlatformMngt::FTGroupsIterator_out rest) throw(CORBA::SystemException )
 {
    
     
        CDMW_LOG_FUNCTION(Get_Logger(), "");
        try { 
            std::string target = this->getObjectId();
            
            CdmwPlatformMngt::ElementPaths_var paths ; 
            CdmwPlatformMngt::FTGroupsIterator_var myrest;
              
            accessor()->get_all_ft_groups(target.c_str(), 
                              how_many,
                              paths.out(),
                              myrest.out());
            
            rest = myrest._retn() ;
            
            nftgroups =
                this->m_proxy_collection->createFTGroupsFromSeq(paths.ptr());
      
        } 
        catch (const CdmwPlatformMngt::ManagedElementNotFound & ex) {
            CORBA::OBJECT_NOT_EXIST e(OrbSupport::OBJECT_NOT_EXIST, 
                                      CORBA::COMPLETED_NO);
            CDMW_LOG_FUNCTION_EXCEPTION(e);
            throw e;
        }
        catch (const CORBA::Exception & ex) {
            CDMW_LOG_FUNCTION_EXCEPTION(ex);
            throw;
        }
        catch (...) {
            CORBA::INTERNAL ex(Cdmw::OrbSupport::INTERNAL, 
                               CORBA::COMPLETED_MAYBE);
            CDMW_LOG_FUNCTION_EXCEPTION(ex);
            throw ex;
        }
     

 }

 /**
    *Purpose:
    *<p>
    * Implements the
    * IDL:thalesgroup.com/CdmwPlatformMngt/Application/get_all_lb_groups:1.0
    * operation
    */  
 void Application_impl::get_all_lb_groups(CORBA::ULong how_many, 
          CdmwPlatformMngt::LBGroups_out nlbgroups, 
             CdmwPlatformMngt::LBGroupsIterator_out rest) 
               throw(CORBA::SystemException )
  {
         
            CDMW_LOG_FUNCTION(Get_Logger(), "");
            try {
                std::string target = this->getObjectId();
         
                CdmwPlatformMngt::ElementPaths_var paths ; 
                CdmwPlatformMngt::LBGroupsIterator_var myrest; 
                          
                accessor()->get_all_lb_groups(target.c_str(), 
                                  how_many,
                                  paths.out(),
                                  myrest.out());
         
                rest = myrest._retn() ;
         
                nlbgroups =
                    this->m_proxy_collection->createLBGroupsFromSeq(paths.ptr());
         
            }
            catch (const CdmwPlatformMngt::ManagedElementNotFound & ex) { 
                    CORBA::OBJECT_NOT_EXIST e(OrbSupport::OBJECT_NOT_EXIST, 
                                                          CORBA::COMPLETED_NO);
                CDMW_LOG_FUNCTION_EXCEPTION(e);
                throw e;
            }
            catch (const CORBA::Exception & ex) { 
                CDMW_LOG_FUNCTION_EXCEPTION(ex);
                throw;
            }
            catch (...) { 
                CORBA::INTERNAL ex(Cdmw::OrbSupport::INTERNAL, 
                                   CORBA::COMPLETED_MAYBE);
                CDMW_LOG_FUNCTION_EXCEPTION(ex);
                throw ex;
            }       
                 

  }




    CdmwPlatformMngt::FTGroup_ptr Application_impl::get_ft_group(const char * ft_group_name)
        throw (CORBA::SystemException, CdmwPlatformMngt::GroupNotFound)
    {
        CDMW_LOG_FUNCTION(Get_Logger(), "ft_group_name=" << ft_group_name);

        try {  
            std::string target = this->getObjectId();
    
            CORBA::String_var ftgroup_target = 
            accessor()->get_ft_group (target.c_str(),
                     ft_group_name);
    
            return this->m_proxy_collection->createFTGroupRef(ftgroup_target);
       }
       catch (const CdmwPlatformMngt::ManagedElementNotFound & ex) {
        CORBA::OBJECT_NOT_EXIST e(OrbSupport::OBJECT_NOT_EXIST, 
                                  CORBA::COMPLETED_NO);
           CDMW_LOG_FUNCTION_EXCEPTION(e);
           throw e;
       }
       catch (const CORBA::Exception & ex) {
           CDMW_LOG_FUNCTION_EXCEPTION(ex);
           throw;
       }
       catch (...) {
           CORBA::INTERNAL ex(Cdmw::OrbSupport::INTERNAL, 
                              CORBA::COMPLETED_MAYBE);
           CDMW_LOG_FUNCTION_EXCEPTION(ex);
           throw ex;
       }
    }


    CdmwPlatformMngt::LBGroup_ptr Application_impl::get_lb_group(const char* name)
        throw (CORBA::SystemException, CdmwPlatformMngt::GroupNotFound)
    {

        CDMW_LOG_FUNCTION(Get_Logger(), "lb_group_name=" << name);
      
                 
        try {   
                std::string target = this->getObjectId();
              
                CORBA::String_var lbgroup_target = 
                accessor()->get_lb_group (target.c_str(),
                                     name);
                 
                return this->m_proxy_collection->createLBGroupRef(lbgroup_target);
       }       
       catch (const CdmwPlatformMngt::ManagedElementNotFound & ex) { 
            CORBA::OBJECT_NOT_EXIST e(OrbSupport::OBJECT_NOT_EXIST, 
                                                  CORBA::COMPLETED_NO);
           CDMW_LOG_FUNCTION_EXCEPTION(e);
           throw e;
       }
       catch (const CORBA::Exception & ex) { 
           CDMW_LOG_FUNCTION_EXCEPTION(ex);
           throw;
       }
       catch (...) { 
           CORBA::INTERNAL ex(Cdmw::OrbSupport::INTERNAL, 
                              CORBA::COMPLETED_MAYBE);
           CDMW_LOG_FUNCTION_EXCEPTION(ex);
           throw ex;
       }


    }


    CORBA::ULong Application_impl::get_number_of_ft_groups()
         throw (CORBA::SystemException)
    {
    CDMW_LOG_FUNCTION(Get_Logger(), "" );

        try {  
            std::string target = this->getObjectId();
    
            CORBA::ULong num_ftgroups = 
               accessor()->get_number_of_ft_groups(target.c_str());
    
            return num_ftgroups;
       }
       catch (const CdmwPlatformMngt::ManagedElementNotFound & ex) {
        CORBA::OBJECT_NOT_EXIST e(OrbSupport::OBJECT_NOT_EXIST, 
                                  CORBA::COMPLETED_NO);
           CDMW_LOG_FUNCTION_EXCEPTION(e);
           throw e;
       }
       catch (const CORBA::Exception & ex) {
           CDMW_LOG_FUNCTION_EXCEPTION(ex);
           throw;
       }
       catch (...) {
           CORBA::INTERNAL ex(Cdmw::OrbSupport::INTERNAL, 
                              CORBA::COMPLETED_MAYBE);
           CDMW_LOG_FUNCTION_EXCEPTION(ex);
           throw ex;
       }
    }


    CdmwPlatformMngt::LBGroup_ptr
     Application_impl::add_lbgroup(const CdmwPlatformMngt::LBGroupDef&)
      throw (CORBA::SystemException,
       CdmwPlatformMngt::GroupAlreadyExists, CdmwPlatformMngt::DuplicateEntity,
       CosPropertyService::MultipleExceptions, CdmwPlatformMngt::IncompatibleStatus)
       {
       throw CORBA::NO_IMPLEMENT (OrbSupport::NO_IMPLEMENT, CORBA::COMPLETED_NO);
       }

    CORBA::ULong Application_impl::get_number_of_lb_groups()
         throw (CORBA::SystemException)
    {

    CDMW_LOG_FUNCTION(Get_Logger(), "" );

        try {  
            std::string target = this->getObjectId();
    
            CORBA::ULong num_lbgroups = 
               accessor()->get_number_of_lb_groups(target.c_str());
    
            return num_lbgroups;
       }
       catch (const CdmwPlatformMngt::ManagedElementNotFound & ex) {
        CORBA::OBJECT_NOT_EXIST e(OrbSupport::OBJECT_NOT_EXIST, 
                                  CORBA::COMPLETED_NO);
           CDMW_LOG_FUNCTION_EXCEPTION(e);
           throw e;
       }
       catch (const CORBA::Exception & ex) {
           CDMW_LOG_FUNCTION_EXCEPTION(ex);
           throw;
       }
       catch (...) {
           CORBA::INTERNAL ex(Cdmw::OrbSupport::INTERNAL, 
                              CORBA::COMPLETED_MAYBE);
           CDMW_LOG_FUNCTION_EXCEPTION(ex);
           throw ex;
       }

    }


} // End namespace Proxy
} // End namespace PlatformMngt
} // End namespace Cdmw

