/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#include "platformftproxy/Entity_impl.hpp"

#include <Foundation/orbsupport/ExceptionMinorCodes.hpp>
#include <Foundation/logging/FunctionLogger.hpp>


const char* const LOGGER_NAME = "CDMW.PFMDMN.SupProxy.Entity";

namespace Cdmw
{
namespace PlatformMngt
{
namespace Proxy
{


// ----------------------------------------------------------------------
// Entity_impl class.
// ----------------------------------------------------------------------


Entity_impl::Entity_impl (CORBA::ORB_ptr orb,
              ProxyCollection *proxy_collection,
              AccessorLookupService *lookup_service)
    throw (Cdmw::BadParameterException, 
       Cdmw::OutOfMemoryException)                  
    : ManagedElement_impl(orb),
      m_accessor(CdmwFTSystMngt::EntityAccessor::_nil()),
      m_accessor_ok(false),
      m_proxy_collection(proxy_collection),
      m_lookup_service(lookup_service)
{
    m_proxy_collection->add_ref();
    set_logger (Get_Logger ());
    set_managed_element (this);
    set_proxy_collection (proxy_collection);
}

Cdmw::Logging::Logger* Entity_impl::Get_Logger () {
        static Cdmw::Logging::Logger* logger =
            Cdmw::Logging::Logger::Get_Logger(LOGGER_NAME);
        return logger;
    }

CdmwFTSystMngt::ManagedElementAccessor_ptr Entity_impl::
        accessor_managed_element () throw (CORBA::TRANSIENT)
      {
         return this->accessor();
      }

const CdmwFTSystMngt::EntityAccessor_ptr 
Entity_impl::accessor() 
    throw(CORBA::TRANSIENT) 
{
    if (!m_accessor_ok) {
    
    CDMW_MUTEX_GUARD(m_accessor_mutex);
    // leaving mutex other thread might have fetched the data
    // already

    try {

        if (CORBA::is_nil(m_accessor.in())) {
        m_accessor = 
            m_lookup_service->getEntityAccessor(); // raises AccessorUndefined
        // update the flag
        m_accessor_ok = ! CORBA::is_nil(m_accessor.in());
        }

    }
    catch (...) {
        throw CORBA::TRANSIENT(::Cdmw::OrbSupport::TRANSIENT,
                   CORBA::COMPLETED_NO);
    }
    
    // paranoid testing
    if (CORBA::is_nil(m_accessor.in())) {
        throw CORBA::TRANSIENT(::Cdmw::OrbSupport::TRANSIENT,
                   CORBA::COMPLETED_NO);
    }

    // releasing mutex here
    }

    CDMW_ASSERT(m_accessor_ok &&
        ! CORBA::is_nil(m_accessor.in()));
    

    return m_accessor.ptr(); // return const ptr
}


Entity_impl::~Entity_impl () throw ()
{
    m_proxy_collection->remove_ref();
}


/**
 *Purpose:
 *<p>
 * Implements the
 * IDL:thalesgroup.com/CdmwPlatformMngt/Entity/get_status:1.0
 * operation
 */
char* 
Entity_impl::get_status(CORBA::String_out entity_info)
    throw (CORBA::SystemException)
{
    CDMW_LOG_FUNCTION(Get_Logger(), "");
    try {
        std::string target = this->getObjectId();
        
        return accessor()->get_status(target.c_str(), entity_info);
    } 
    catch (const CdmwPlatformMngt::ManagedElementNotFound & ex) {
        CORBA::OBJECT_NOT_EXIST e(OrbSupport::OBJECT_NOT_EXIST, 
                                  CORBA::COMPLETED_NO);
        CDMW_LOG_FUNCTION_EXCEPTION(e);
        throw e;
    }
    catch (const CORBA::Exception & ex) {
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw;
    }
    catch (...) {
        CORBA::INTERNAL ex(Cdmw::OrbSupport::INTERNAL, 
                           CORBA::COMPLETED_MAYBE);
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw ex;
    }
}

/**
 *Purpose:
 *<p>
 * Implements the
 * IDL:thalesgroup.com/CdmwPlatformMngt/Entity/set_status:1.0
 * operation
 */
void 
Entity_impl::set_status(const char* entity_status, const char* entity_info)
    throw (CdmwPlatformMngt::EntityStatusNotAllowed, 
                CORBA::SystemException)
{
    CDMW_LOG_FUNCTION(Get_Logger(), "");
    try {
        std::string target = this->getObjectId();
        
        accessor()->set_status(target.c_str(), entity_status, entity_info);
    } 
    catch (const CdmwPlatformMngt::ManagedElementNotFound & ex) {
        CORBA::OBJECT_NOT_EXIST e(OrbSupport::OBJECT_NOT_EXIST, 
                                  CORBA::COMPLETED_NO);
        CDMW_LOG_FUNCTION_EXCEPTION(e);
        throw e;
    }
    catch (const CORBA::Exception & ex) {
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw;
    }
    catch (...) {
        CORBA::INTERNAL ex(Cdmw::OrbSupport::INTERNAL, 
                           CORBA::COMPLETED_MAYBE);
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw ex;
    }
}

    

/**
 *Purpose:
 *<p>
 * Implements the
 * IDL:thalesgroup.com/CdmwPlatformMngt/Entity/get_status_values:1.0
 * operation
 */
CdmwPlatformMngt::EntityStatusValues* 
Entity_impl::get_status_values()
    throw (CORBA::SystemException)
{
    CDMW_LOG_FUNCTION(Get_Logger(), "");
    try {
        std::string target = this->getObjectId();
        
        return accessor()->get_status_values(target.c_str());
    } 
    catch (const CdmwPlatformMngt::ManagedElementNotFound & ex) {
        CORBA::OBJECT_NOT_EXIST e(OrbSupport::OBJECT_NOT_EXIST, 
                                  CORBA::COMPLETED_NO);
        CDMW_LOG_FUNCTION_EXCEPTION(e);
        throw e;
    }
    catch (const CORBA::Exception & ex) {
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw;
    }
    catch (...) {
        CORBA::INTERNAL ex(Cdmw::OrbSupport::INTERNAL, 
                           CORBA::COMPLETED_MAYBE);
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw ex;
    }
}


/**
 *Purpose:
 *<p>
 * Implements the
 * IDL:thalesgroup.com/CdmwPlatformMngt/Entity/get_snapshot:1.0
 * operation
 */
CdmwPlatformMngt::EntitySnapshot* 
Entity_impl::get_snapshot()
    throw (CORBA::SystemException)
{
    CDMW_LOG_FUNCTION(Get_Logger(), "");
    try {
        std::string target = this->getObjectId();
        
        return accessor()->get_snapshot(target.c_str());
    } 
    catch (const CdmwPlatformMngt::ManagedElementNotFound & ex) {
        CORBA::OBJECT_NOT_EXIST e(OrbSupport::OBJECT_NOT_EXIST, 
                                  CORBA::COMPLETED_NO);
        CDMW_LOG_FUNCTION_EXCEPTION(e);
        throw e;
    }
    catch (const CORBA::Exception & ex) {
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw;
    }
    catch (...) {
        CORBA::INTERNAL ex(Cdmw::OrbSupport::INTERNAL, 
                           CORBA::COMPLETED_MAYBE);
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw ex;
    }
}
     
          
} // End namespace Proxy
} // End namespace PlatformMngt
} // End namespace Cdmw


