/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#include "platformftproxy/Host_impl.hpp"

#include <SystemMngt/platformlibrary/Configuration.hpp>
#include <SystemMngt/tools/ManagedElementKey.hpp>
#include <Foundation/orbsupport/ExceptionMinorCodes.hpp>
#include <Foundation/logging/FunctionLogger.hpp>

const char* const LOGGER_NAME = "CDMW.PFMDMN.SupProxy.Host";

namespace Cdmw
{
namespace PlatformMngt
{
namespace Proxy
{

// class Host_impl;
// class System_impl;



// ----------------------------------------------------------------------
// HostRespondingCallback class.
// ----------------------------------------------------------------------
// HostRespondingCallback::HostRespondingCallback (Host_impl* host)
//     : Callback()
// {
//     m_host = host;
// }


// HostRespondingCallback::~HostRespondingCallback ()
// {
// }


// // ----------------------------------------------------------------------
// // HostNotRespondingCallback class.
// // ----------------------------------------------------------------------
// HostNotRespondingCallback::HostNotRespondingCallback (Host_impl* host)
//     : Callback()
// {



// ----------------------------------------------------------------------
// Host_impl class.
// ----------------------------------------------------------------------


Host_impl::Host_impl (CORBA::ORB_ptr orb,
              ProxyCollection *proxy_collection,
              AccessorLookupService *lookup_service)
    throw (Cdmw::BadParameterException, 
       Cdmw::OutOfMemoryException)                  
    : ManagedElement_impl(orb),
      m_accessor(CdmwFTSystMngt::HostAccessor::_nil()),
      m_accessor_ok(false),
      m_proxy_collection(proxy_collection),
      m_lookup_service(lookup_service)
{
    m_proxy_collection->add_ref();
    set_logger (Get_Logger ());
    set_managed_element (this);
    set_proxy_collection (proxy_collection);
}

Cdmw::Logging::Logger* Host_impl::Get_Logger () {
        static Cdmw::Logging::Logger* logger =
            Cdmw::Logging::Logger::Get_Logger(LOGGER_NAME);
        return logger;
    }

CdmwPlatformMngt::HostSnapshot * Host_impl::get_snapshot () 
        throw (CORBA::SystemException)
{
    CDMW_LOG_FUNCTION(Get_Logger(), "");
    try {
        std::string target = this->getObjectId();
        
        return accessor()->get_snapshot(target.c_str());
    } 
    catch (const CdmwPlatformMngt::ManagedElementNotFound & ex) {
        CORBA::OBJECT_NOT_EXIST e(OrbSupport::OBJECT_NOT_EXIST, 
                                  CORBA::COMPLETED_NO);
        CDMW_LOG_FUNCTION_EXCEPTION(e);
        throw e;
    }
    catch (const CORBA::Exception & ex) {
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw;
    }
    catch (...) {
        CORBA::INTERNAL ex(Cdmw::OrbSupport::INTERNAL, 
                           CORBA::COMPLETED_MAYBE);
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw ex;
    }
}

CdmwFTSystMngt::ManagedElementAccessor_ptr Host_impl::
        accessor_managed_element () throw (CORBA::TRANSIENT)
      {
         return this->accessor();
      }

const CdmwFTSystMngt::HostAccessor_ptr 
Host_impl::accessor() 
    throw(CORBA::TRANSIENT) 
{
    if (!m_accessor_ok) {
    
    CDMW_MUTEX_GUARD(m_accessor_mutex);
    // leaving mutex other thread might have fetched the data
    // already

    try {

        if (CORBA::is_nil(m_accessor.in())) {
        m_accessor = 
            m_lookup_service->getHostAccessor(); // raises AccessorUndefined
        // update the flag
        m_accessor_ok = ! CORBA::is_nil(m_accessor.in());
        }

    }
    catch (...) {
        throw CORBA::TRANSIENT(::Cdmw::OrbSupport::TRANSIENT,
                   CORBA::COMPLETED_NO);
    }
    
    // paranoid testing
    if (CORBA::is_nil(m_accessor.in())) {
        throw CORBA::TRANSIENT(::Cdmw::OrbSupport::TRANSIENT,
                   CORBA::COMPLETED_NO);
    }

    // releasing mutex here
    }
    
    CDMW_ASSERT(m_accessor_ok &&
        ! CORBA::is_nil(m_accessor.in()));

    return m_accessor.ptr(); // return const ptr
}

Host_impl::~Host_impl () throw ()
{  
    m_proxy_collection->remove_ref();
}


CdmwPlatformMngt::HostStatus Host_impl::status()
throw( CORBA::SystemException )
{
    CDMW_LOG_FUNCTION(Get_Logger(), "");
    try {
        std::string target = this->getObjectId();
        
        return accessor()->get_status(target.c_str());
    } 
    catch (const CdmwPlatformMngt::ManagedElementNotFound & ex) {
        CORBA::OBJECT_NOT_EXIST e(OrbSupport::OBJECT_NOT_EXIST, 
                                  CORBA::COMPLETED_NO);
        CDMW_LOG_FUNCTION_EXCEPTION(e);
        throw e;
    }
    catch (const CORBA::Exception & ex) {
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw;
    }
    catch (...) {
        CORBA::INTERNAL ex(Cdmw::OrbSupport::INTERNAL, 
                           CORBA::COMPLETED_MAYBE);
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw ex;
    }
}


char * Host_impl::host_status_info()
throw( CORBA::SystemException )
{
    CDMW_LOG_FUNCTION(Get_Logger(), "");
    std::string target = this->getObjectId();
    
    try {
        return accessor()->get_host_status_info(target.c_str());
    } 
    catch (const CdmwPlatformMngt::ManagedElementNotFound & ex) {
        CORBA::OBJECT_NOT_EXIST e(OrbSupport::OBJECT_NOT_EXIST, 
                                  CORBA::COMPLETED_NO);
        CDMW_LOG_FUNCTION_EXCEPTION(e);
        throw e;
    }
    catch (const CORBA::Exception & ex) {
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw;
    }
    catch (...) {
        CORBA::INTERNAL ex(Cdmw::OrbSupport::INTERNAL, 
                           CORBA::COMPLETED_MAYBE);
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw ex;
    }
}


    
CORBA::Long Host_impl::restart_attempt() throw (CORBA::SystemException)
{
    CDMW_LOG_FUNCTION(Get_Logger(), "");
    try {
        std::string target = this->getObjectId();
        return accessor()->get_restart_attempt(target.c_str()) ;
    } 
    catch (const CdmwPlatformMngt::ManagedElementNotFound & ex) {
        CORBA::OBJECT_NOT_EXIST e(OrbSupport::OBJECT_NOT_EXIST, 
                                  CORBA::COMPLETED_NO);
        CDMW_LOG_FUNCTION_EXCEPTION(e);
        throw e;
    }
    catch (const CORBA::Exception & ex) {
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw;
    }
    catch (...) {
        CORBA::INTERNAL ex(Cdmw::OrbSupport::INTERNAL, 
                           CORBA::COMPLETED_MAYBE);
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw ex;
    }
}


CdmwPlatformMngt::CompleteMonitoringInfo Host_impl::get_monitoring_parameters() throw (CORBA::SystemException)
{
    CDMW_LOG_FUNCTION(Get_Logger(), "");
    try {
        std::string target = this->getObjectId();
        return accessor()->get_monitoring_parameters(target.c_str()) ;
    } 
    catch (const CdmwPlatformMngt::ManagedElementNotFound & ex) {
        CORBA::OBJECT_NOT_EXIST e(OrbSupport::OBJECT_NOT_EXIST, 
                                  CORBA::COMPLETED_NO);
        CDMW_LOG_FUNCTION_EXCEPTION(e);
        throw e;
    }
    catch (const CORBA::Exception & ex) {
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw;
    }
    catch (...) {
        CORBA::INTERNAL ex(Cdmw::OrbSupport::INTERNAL, 
                           CORBA::COMPLETED_MAYBE);
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw ex;
    }
}

void Host_impl::set_monitoring_parameters(const CdmwPlatformMngt::CompleteMonitoringInfo& monitoring_parameters) throw (CORBA::SystemException)
{
    CDMW_LOG_FUNCTION(Get_Logger(), "");
    try {
        std::string target = this->getObjectId();
        accessor()->set_monitoring_parameters(monitoring_parameters, target.c_str()) ;
    } 
    catch (const CdmwPlatformMngt::ManagedElementNotFound & ex) {
        CORBA::OBJECT_NOT_EXIST e(OrbSupport::OBJECT_NOT_EXIST, 
                                  CORBA::COMPLETED_NO);
        CDMW_LOG_FUNCTION_EXCEPTION(e);
        throw e;
    }
    catch (const CORBA::Exception & ex) {
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw;
    }
    catch (...) {
        CORBA::INTERNAL ex(Cdmw::OrbSupport::INTERNAL, 
                           CORBA::COMPLETED_MAYBE);
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw ex;
    }
}


CdmwPlatformMngt::Processes * Host_impl::get_processes (
                     CORBA::ULong how_many,
                     CdmwPlatformMngt::ProcessesIterator_out processes_iterator)
    throw (CORBA::SystemException)
{   
    CDMW_LOG_FUNCTION(Get_Logger(), "");
    try {
        std::string target = this->getObjectId();
    
        CdmwPlatformMngt::ElementPathsIterator_var iter;
        CdmwPlatformMngt::ElementPaths_var paths = 
            accessor()->get_processes(target.c_str(), 
                          how_many,
                          iter.out());
        
        processes_iterator = 
            this->m_proxy_collection->createProcessesIteratorRef(iter.ptr());
        
        return this->m_proxy_collection->createProcessesFromSeq(paths.ptr());
    } 
    catch (const CdmwPlatformMngt::ManagedElementNotFound & ex) {
        CORBA::OBJECT_NOT_EXIST e(OrbSupport::OBJECT_NOT_EXIST, 
                                  CORBA::COMPLETED_NO);
        CDMW_LOG_FUNCTION_EXCEPTION(e);
        throw e;
    }
    catch (const CORBA::Exception & ex) {
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw;
    }
    catch (...) {
        CORBA::INTERNAL ex(Cdmw::OrbSupport::INTERNAL, 
                           CORBA::COMPLETED_MAYBE);
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw ex;
    }
}

CdmwPlatformMngt::Processes * Host_impl::getprocesses () throw (CORBA::SystemException)
{
    CDMW_LOG_FUNCTION(Get_Logger(), "");
    throw CORBA::NO_IMPLEMENT (OrbSupport::NO_IMPLEMENT, CORBA::COMPLETED_NO);
}

 CdmwPlatformMngt::Process_ptr Host_impl::get_process(const char *process_name, const char *application_name ) throw (CdmwPlatformMngt::ProcessNotFound, CORBA::SystemException)
   {
    std::string target = this->getObjectId();

    CORBA::String_var proc_target = accessor()->get_process (target.c_str(), process_name, application_name); 
        
    return this->m_proxy_collection->createProcessRef(proc_target);

    //return accessor()->get_process(target.c_str(), process_name, application_name);
   }

          
          
        
void Host_impl::shutdown ()
throw (CORBA::SystemException)
{  
    CDMW_LOG_FUNCTION(Get_Logger(), "");
    //get issuer value

    std::string command_issuer (m_current->get_issuer());
    
    try {
        std::string target = this->getObjectId();
    
        accessor()->shutdown(target.c_str(),
                                      command_issuer.c_str());
    } 
    catch (const CdmwPlatformMngt::ManagedElementNotFound & ex) {
        CORBA::OBJECT_NOT_EXIST e(OrbSupport::OBJECT_NOT_EXIST, 
                                  CORBA::COMPLETED_NO);
        CDMW_LOG_FUNCTION_EXCEPTION(e);
        throw e;
    }
    catch (const CORBA::Exception & ex) {
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw;
    }
    catch (...) {
        CORBA::INTERNAL ex(Cdmw::OrbSupport::INTERNAL, 
                           CORBA::COMPLETED_MAYBE);
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw ex;
    }
}


 
void Host_impl::cold_reboot ()
throw (CORBA::SystemException)
{ 
    CDMW_LOG_FUNCTION(Get_Logger(), "");
    //get issuer value
    
    std::string command_issuer (m_current->get_issuer());

    try {
        std::string target = this->getObjectId();
    
        accessor()->cold_reboot(target.c_str(),
                                         command_issuer.c_str());
    } 
    catch (const CdmwPlatformMngt::ManagedElementNotFound & ex) {
        CORBA::OBJECT_NOT_EXIST e(OrbSupport::OBJECT_NOT_EXIST, 
                                  CORBA::COMPLETED_NO);
        CDMW_LOG_FUNCTION_EXCEPTION(e);
        throw e;
    }
    catch (const CORBA::Exception & ex) {
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw;
    }
    catch (...) {
        CORBA::INTERNAL ex(Cdmw::OrbSupport::INTERNAL, 
                           CORBA::COMPLETED_MAYBE);
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw ex;
    }
}

               
void Host_impl::hot_reboot ()
throw (CORBA::SystemException)
{
    CDMW_LOG_FUNCTION(Get_Logger(), "");
    //get issuer value

    std::string command_issuer (m_current->get_issuer());

    try {
        std::string target = this->getObjectId();
    
        accessor()->hot_reboot(target.c_str(),
                                        command_issuer.c_str());
    } 
    catch (const CdmwPlatformMngt::ManagedElementNotFound & ex) {
        CORBA::OBJECT_NOT_EXIST e(OrbSupport::OBJECT_NOT_EXIST, 
                                  CORBA::COMPLETED_NO);
        CDMW_LOG_FUNCTION_EXCEPTION(e);
        throw e;
    }
    catch (const CORBA::Exception & ex) {
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw;
    }
    catch (...) {
        CORBA::INTERNAL ex(Cdmw::OrbSupport::INTERNAL, 
                           CORBA::COMPLETED_MAYBE);
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw ex;
    }
}


CORBA::ULong Host_impl::get_number_of_entities()
throw(CORBA::SystemException)
{
    CDMW_LOG_FUNCTION(Get_Logger(), "");
    try {  
        std::string target = this->getObjectId();
         
        return accessor()->get_number_of_entities(target.c_str());  
    } 
    catch (const CdmwPlatformMngt::ManagedElementNotFound & ex) {
        CORBA::OBJECT_NOT_EXIST e(OrbSupport::OBJECT_NOT_EXIST, 
                                  CORBA::COMPLETED_NO);
        CDMW_LOG_FUNCTION_EXCEPTION(e);
        throw e;
    }
    catch (const CORBA::Exception & ex) {
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw;
    }
    catch (...) {
        CORBA::INTERNAL ex(Cdmw::OrbSupport::INTERNAL, 
                           CORBA::COMPLETED_MAYBE);
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw ex;
    }
}

void Host_impl::get_all_entity_names(
    CORBA::ULong how_many,
    CdmwPlatformMngt::ElementNames_out entity_names,
    CdmwPlatformMngt::ElementNamesIterator_out rest)
throw(CORBA::SystemException)
{  
    CDMW_LOG_FUNCTION(Get_Logger(), "");
    try {  
        std::string target = this->getObjectId();
          
        accessor()->get_all_entity_names(target.c_str(),
                         how_many,
                         entity_names,
                         rest); 
    } 
    catch (const CdmwPlatformMngt::ManagedElementNotFound & ex) {
        CORBA::OBJECT_NOT_EXIST e(OrbSupport::OBJECT_NOT_EXIST, 
                                  CORBA::COMPLETED_NO);
        CDMW_LOG_FUNCTION_EXCEPTION(e);
        throw e;
    }
    catch (const CORBA::Exception & ex) {
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw;
    }
    catch (...) {
        CORBA::INTERNAL ex(Cdmw::OrbSupport::INTERNAL, 
                           CORBA::COMPLETED_MAYBE);
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw ex;
    }
}

CdmwPlatformMngt::Entity_ptr Host_impl::get_entity (
    const char* entity_name)
throw (CdmwPlatformMngt::EntityNotFound,
       CORBA::SystemException)
{
    CDMW_LOG_FUNCTION(Get_Logger(), "entity_name=" << entity_name);
    try {  
        std::string target = this->getObjectId();
          
        CORBA::String_var entity_target =
            accessor()->get_entity(target.c_str(), entity_name);
        
        return this->m_proxy_collection->createEntityRef(entity_target);
    } 
    catch (const CdmwPlatformMngt::ManagedElementNotFound & ex) {
        CORBA::OBJECT_NOT_EXIST e(OrbSupport::OBJECT_NOT_EXIST, 
                                  CORBA::COMPLETED_NO);
        CDMW_LOG_FUNCTION_EXCEPTION(e);
        throw e;
    }
    catch (const CORBA::Exception & ex) {
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw;
    }
    catch (...) {
        CORBA::INTERNAL ex(Cdmw::OrbSupport::INTERNAL, 
                           CORBA::COMPLETED_MAYBE);
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw ex;
    }
}

CORBA::Boolean Host_impl::get_entities(
    const CdmwPlatformMngt::ElementNames& entity_names,
    CdmwPlatformMngt::Entities_out nentities)
throw(CORBA::SystemException)
{
    CDMW_LOG_FUNCTION(Get_Logger(), "");
    try {   
        std::string target = this->getObjectId();
          
        ::CdmwPlatformMngt::ElementPaths_var paths = 
              new ::CdmwPlatformMngt::ElementPaths();
        
        CdmwPlatformMngt::ElementNames_var tmp_entity_names;
    
        CORBA::Boolean succeed = 
            accessor()->get_entities(target.c_str(),
                         entity_names,
                         paths.out());
    
        if (succeed) {
            nentities =
            this->m_proxy_collection->createEntitiesFromSeq(paths.ptr());
        }
            
        return succeed;
    } 
    catch (const CdmwPlatformMngt::ManagedElementNotFound & ex) {
        CORBA::OBJECT_NOT_EXIST e(OrbSupport::OBJECT_NOT_EXIST, 
                                  CORBA::COMPLETED_NO);
        CDMW_LOG_FUNCTION_EXCEPTION(e);
        throw e;
    }
    catch (const CORBA::Exception & ex) {
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw;
    }
    catch (...) {
        CORBA::INTERNAL ex(Cdmw::OrbSupport::INTERNAL, 
                           CORBA::COMPLETED_MAYBE);
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw ex;
    }
}

void Host_impl::get_all_entities(
    CORBA::ULong how_many,
    CdmwPlatformMngt::Entities_out nentities,
    CdmwPlatformMngt::EntitiesIterator_out rest)
throw(CORBA::SystemException)
{  
    CDMW_LOG_FUNCTION(Get_Logger(), "");
    try {  
        std::string target = this->getObjectId();
          
        ::CdmwPlatformMngt::ElementPaths_var         paths;
        ::CdmwPlatformMngt::ElementPathsIterator_var iter;
        
        accessor()->get_all_entities(target.c_str(),
                         how_many,
                         paths.out(),
                         iter.out());
        
        CdmwPlatformMngt::Entities_var tmp_nentities = 
            this->m_proxy_collection->createEntitiesFromSeq(paths.ptr());
        
        CdmwPlatformMngt::EntitiesIterator_var tmp_rest =
            this->m_proxy_collection->createEntitiesIteratorRef(iter.in());
        
        nentities = tmp_nentities._retn();
        rest      = tmp_rest._retn();
    } 
    catch (const CdmwPlatformMngt::ManagedElementNotFound & ex) {
        CORBA::OBJECT_NOT_EXIST e(OrbSupport::OBJECT_NOT_EXIST, 
                                  CORBA::COMPLETED_NO);
        CDMW_LOG_FUNCTION_EXCEPTION(e);
        throw e;
    }
    catch (const CORBA::Exception & ex) {
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw;
    }
    catch (...) {
        CORBA::INTERNAL ex(Cdmw::OrbSupport::INTERNAL, 
                           CORBA::COMPLETED_MAYBE);
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw ex;
    }
}

    
CdmwPlatformMngt::Entity_ptr Host_impl::add_entity (
     const CdmwPlatformMngt::EntityDef & entity_def)
throw(CdmwPlatformMngt::EntityAlreadyExists,
      CORBA::SystemException)
{
    CDMW_LOG_FUNCTION(Get_Logger(), "entity_name=" << entity_def.entity_name);
    try {  
        std::string target = this->getObjectId();
    
        CORBA::String_var uid = 
            accessor()->add_entity(target.c_str(),
                       entity_def);
        
        return this->m_proxy_collection->createEntityRef(uid);
    } 
    catch (const CdmwPlatformMngt::ManagedElementNotFound & ex) {
        CORBA::OBJECT_NOT_EXIST e(OrbSupport::OBJECT_NOT_EXIST, 
                                  CORBA::COMPLETED_NO);
        CDMW_LOG_FUNCTION_EXCEPTION(e);
        throw e;
    }
    catch (const CORBA::Exception & ex) {
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw;
    }
    catch (...) {
        CORBA::INTERNAL ex(Cdmw::OrbSupport::INTERNAL, 
                           CORBA::COMPLETED_MAYBE);
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw ex;
    }
}

void Host_impl::remove_entity(
    const char* entity_name)
throw (CdmwPlatformMngt::EntityNotFound,
       CORBA::SystemException)
{  
    CDMW_LOG_FUNCTION(Get_Logger(), "entity_name=" << entity_name);
    try {  
        std::string target = this->getObjectId();
    
        accessor()->remove_entity(target.c_str(),
                      entity_name);
    } 
    catch (const CdmwPlatformMngt::ManagedElementNotFound & ex) {
        CORBA::OBJECT_NOT_EXIST e(OrbSupport::OBJECT_NOT_EXIST, 
                                  CORBA::COMPLETED_NO);
        CDMW_LOG_FUNCTION_EXCEPTION(e);
        throw e;
    }
    catch (const CORBA::Exception & ex) {
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw;
    }
    catch (...) {
        CORBA::INTERNAL ex(Cdmw::OrbSupport::INTERNAL, 
                           CORBA::COMPLETED_MAYBE);
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw ex;
    }
}

                     
CORBA::Boolean Host_impl::get_reboot_in_progress ()
         throw(CORBA::SystemException)
{
   std::string target = this->getObjectId();
    try{
        return accessor()->get_reboot_in_progress (target.c_str());
     }
     catch (const CORBA::SystemException & ex) {
         throw;
    }
    catch (...) {
        throw CORBA::INTERNAL(Cdmw::OrbSupport::INTERNAL,
                              CORBA::COMPLETED_MAYBE);
    }
 
}

CORBA::Boolean Host_impl::get_stop_in_progress ()
         throw(CORBA::SystemException)
{
   std::string target = this->getObjectId();
    try{
        return accessor()->get_stop_in_progress (target.c_str());
     }
     catch (const CORBA::SystemException & ex) {
         throw;
    }
    catch (...) {
        throw CORBA::INTERNAL(Cdmw::OrbSupport::INTERNAL,
                              CORBA::COMPLETED_MAYBE);
    }
 
}


void Host_impl::stop_host() throw (CORBA::SystemException)
{
   
    CDMW_LOG_FUNCTION(Get_Logger(), "");
    //get issuer value

   std::string command_issuer (m_current->get_issuer());

    try {
        std::string target = this->getObjectId();
        accessor()->stop_host(target.c_str(),
                                       command_issuer.c_str());
    } 
    catch (const CdmwPlatformMngt::ManagedElementNotFound & ex) {
        CORBA::OBJECT_NOT_EXIST e(OrbSupport::OBJECT_NOT_EXIST, 
                                  CORBA::COMPLETED_NO);
        CDMW_LOG_FUNCTION_EXCEPTION(e);
        throw e;
    }
    catch (const CORBA::Exception & ex) {
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw;
    }
    catch (...) {
        CORBA::INTERNAL ex(Cdmw::OrbSupport::INTERNAL, 
                           CORBA::COMPLETED_MAYBE);
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw ex;
    }
    
}

void Host_impl::get_all_processes(CORBA::ULong how_many, CdmwPlatformMngt::Processes_out nprocesses,
                           CdmwPlatformMngt::ProcessesIterator_out rest)    
    throw(CORBA::SystemException)
{
    throw CORBA::NO_IMPLEMENT (OrbSupport::NO_IMPLEMENT, CORBA::COMPLETED_NO);
}

CORBA::ULong Host_impl::get_number_of_processes()
    throw(CORBA::SystemException)
{
    throw CORBA::NO_IMPLEMENT (OrbSupport::NO_IMPLEMENT, CORBA::COMPLETED_NO);
}

void Host_impl::cold_restart()
    throw(CORBA::SystemException)
{   
    CDMW_LOG_FUNCTION(Get_Logger(), "");
    //get issuer value

    std::string command_issuer (m_current->get_issuer());

    try {
        std::string target = this->getObjectId();
        accessor()->cold_restart(target.c_str(),
                                          command_issuer.c_str());
    } 
    catch (const CdmwPlatformMngt::ManagedElementNotFound & ex) {
        CORBA::OBJECT_NOT_EXIST e(OrbSupport::OBJECT_NOT_EXIST, CORBA::COMPLETED_NO);
        CDMW_LOG_FUNCTION_EXCEPTION(e);
        throw e;
    }
    catch (const CORBA::Exception & ex) {
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw;
    }
    catch (...) {
        CORBA::INTERNAL ex(Cdmw::OrbSupport::INTERNAL, CORBA::COMPLETED_MAYBE);
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw ex;
    }

//    throw CORBA::NO_IMPLEMENT (OrbSupport::NO_IMPLEMENT, CORBA::COMPLETED_NO);
}
    
void Host_impl::hot_restart()    
    throw(CORBA::SystemException)
{
    
    CDMW_LOG_FUNCTION(Get_Logger(), "");
    //get issuer value
    std::string command_issuer (m_current->get_issuer());

    try {
        std::string target = this->getObjectId();
        accessor()->hot_restart(target.c_str(),
                                         command_issuer.c_str());
    } 
    catch (const CdmwPlatformMngt::ManagedElementNotFound & ex) {
        CORBA::OBJECT_NOT_EXIST e(OrbSupport::OBJECT_NOT_EXIST, CORBA::COMPLETED_NO);
        CDMW_LOG_FUNCTION_EXCEPTION(e);
        throw e;
    }
    catch (const CORBA::Exception & ex) {
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw;
    }
    catch (...) {
        CORBA::INTERNAL ex(Cdmw::OrbSupport::INTERNAL, CORBA::COMPLETED_MAYBE);
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw ex;
    }
}

void Host_impl::notify(CdmwPlatformMngt::Event*) 
    throw (CORBA::SystemException)
{
    // TODO: 
    // Do nothing...
}

          
} // End namespace Proxy
} // End namespace PlatformMngt
} // End namespace Cdmw


