/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

#include <iostream>
#include "platformftproxy/Notifier_impl.hpp"
#include <Foundation/orbsupport/ExceptionMinorCodes.hpp>
#include <Foundation/logging/FunctionLogger.hpp>

namespace {
    
    const char* const LOGGER_NAME = "CDMW.PFMDMN.ProxyPeriodic.Notifier";
    
    Cdmw::Logging::Logger* Get_Logger () {
        static Cdmw::Logging::Logger* logger =
            Cdmw::Logging::Logger::Get_Logger(LOGGER_NAME);
        return logger;
    }
}

namespace Cdmw
{ // Begin namespace Cdmw

namespace PlatformMngt
{ // Begin namespace PlatformMngt
     
namespace Proxy
{ // Begin namespace Periodic
            
namespace Periodic
{ // Begin namespace Periodic



Notifier_impl::Notifier_impl(CORBA::ORB_ptr orb, 
                             ProxyCollection *proxy_collection,
                             AccessorLookupService *lookup_service)
    throw (Cdmw::BadParameterException,
           Cdmw::OutOfMemoryException)             
    : ProxyUtil(orb),
    m_accessor(CdmwFTSystMngt::Periodic::NotifierAccessor::_nil()),
    m_accessor_ok(false),
    m_proxy_collection(proxy_collection),
    m_lookup_service(lookup_service)
{        
    m_proxy_collection->add_ref();
}

            
Notifier_impl::~Notifier_impl () throw ()
{        
    m_proxy_collection->remove_ref();
}


const CdmwFTSystMngt::Periodic::NotifierAccessor_ptr Notifier_impl::accessor() 
throw(CORBA::TRANSIENT) 
{   
    if (!m_accessor_ok) 
    {
        try 
        {
            if (CORBA::is_nil(m_accessor.in()))
            {
                m_accessor = m_lookup_service->getNotifierAccessor(); // raises AccessorUndefined
                // update the flag
                m_accessor_ok = ! CORBA::is_nil(m_accessor.in());
            }

        }
        catch (...) 
        {
            throw CORBA::TRANSIENT(::Cdmw::OrbSupport::TRANSIENT,
                                   CORBA::COMPLETED_NO);
        }
    
        // paranoid testing
        if (CORBA::is_nil(m_accessor.in())) 
        {
            throw CORBA::TRANSIENT(::Cdmw::OrbSupport::TRANSIENT,
                                   CORBA::COMPLETED_NO);
        }
    }

    CDMW_ASSERT(m_accessor_ok &&
                ! CORBA::is_nil(m_accessor.in()));
    
    return m_accessor.ptr(); // return const ptr
}


void Notifier_impl::register_observer (const char * observer_name,
                                       const CosPropertyService::PropertyNames & property_names,
                                       CdmwPlatformMngt::Periodic::Observer_ptr observer_ref, 
                                       TimeBase::TimeT period)
    throw (CORBA::SystemException,
           CdmwPlatformMngt::PropertyNameInvalid,
           CdmwPlatformMngt::ElementAlreadyExists)
{
    
    CDMW_LOG_FUNCTION(::Get_Logger(), "observer_name=" << observer_name);
    try
    {
        accessor()->register_observer (0, observer_name, property_names,
                                       observer_ref, period);                
    } 
    catch (const CdmwPlatformMngt::ElementAlreadyExists & e)
    {
        CDMW_LOG_FUNCTION_EXCEPTION(e);
        throw e;
    }
    catch(const CdmwPlatformMngt::ManagedElementNotFound& )
    {
        CdmwPlatformMngt::PropertyNameInvalid e;
        CDMW_LOG_FUNCTION_EXCEPTION(e);
        throw e;
    }
    catch(const CORBA::SystemException & e)
    {
        CDMW_LOG_FUNCTION_EXCEPTION(e);
        throw;
    }
    catch (...)
    {
        CORBA::INTERNAL e(Cdmw::OrbSupport::INTERNAL, 
                CORBA::COMPLETED_MAYBE);
        CDMW_LOG_FUNCTION_EXCEPTION(e);
        throw e;
    }               
}
    

void Notifier_impl::register_observer_group (const char * observer_name, 
                                             const CosPropertyService::PropertyNames &  property_names,
                                             const char * m_miop_group,
                                             TimeBase::TimeT period)
    throw (CORBA::SystemException, 
           CdmwPlatformMngt::PropertyNameInvalid,
           CdmwPlatformMngt::ElementAlreadyExists)
{
    
    CDMW_LOG_FUNCTION(::Get_Logger(), "observer_name=" << observer_name);
    
    try
    {
        accessor()->register_observer_group (0, observer_name, property_names,
                m_miop_group, period);                
    } 
    catch (const CdmwPlatformMngt::ElementAlreadyExists & e)
    {
        CDMW_LOG_FUNCTION_EXCEPTION(e);
        throw e;
    }
    catch(const CdmwPlatformMngt::ManagedElementNotFound& )
    {
        CdmwPlatformMngt::PropertyNameInvalid e;
        CDMW_LOG_FUNCTION_EXCEPTION(e);
        throw e;
    }
    catch(const CORBA::SystemException & e)
    {
        CDMW_LOG_FUNCTION_EXCEPTION(e);
        throw;
    }
    catch (...) 
    {
        CORBA::INTERNAL e(Cdmw::OrbSupport::INTERNAL, 
                CORBA::COMPLETED_MAYBE);
        CDMW_LOG_FUNCTION_EXCEPTION(e);
        throw e;
    }
} 


CdmwPlatformMngt::Periodic::Observer_ptr Notifier_impl::unregister_observer (const char * observer_name)
    throw (CORBA::SystemException,
           CdmwPlatformMngt::ObserverNotFound)
{   
    
    CDMW_LOG_FUNCTION(::Get_Logger(), "observer_name=" << observer_name);
    
    try
    {
        return accessor()->unregister_observer(0, observer_name);
    }
    catch (const CdmwPlatformMngt::ObserverNotFound&)
    {
        throw CdmwPlatformMngt::ObserverNotFound();
    }
    catch(const CORBA::SystemException & e)
    {
        throw ;
    }
    catch (...) 
    {
        CORBA::INTERNAL ex(Cdmw::OrbSupport::INTERNAL,
                           CORBA::COMPLETED_MAYBE);
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw ex;
    }
}
           
           
           
char * Notifier_impl::unregister_observer_group (const char * observer_name)
    throw (CORBA::SystemException,
           CdmwPlatformMngt::ObserverNotFound)
{
    CDMW_LOG_FUNCTION(::Get_Logger(), "observer_name=" << observer_name);
    try
    {
        return accessor()->unregister_observer_group(0 , observer_name);

    }
    catch (const CdmwPlatformMngt::ObserverNotFound&)
    {
        throw CdmwPlatformMngt::ObserverNotFound();
    }
    catch(const CORBA::SystemException & e)
    {
        throw ;
    }
    catch (...) 
    {
        CORBA::INTERNAL ex(Cdmw::OrbSupport::INTERNAL,
                CORBA::COMPLETED_MAYBE);
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw ex;
    }
}     
    
    
char * Notifier_impl::periodic_notifier_id()
    throw (CORBA::SystemException)
{
    CDMW_LOG_FUNCTION(::Get_Logger(), "");
    throw CORBA::NO_IMPLEMENT (OrbSupport::NO_IMPLEMENT, CORBA::COMPLETED_NO);
}
        


} // End namespace Proxy
} // End namespace Periodic
} // End namespace PlatformMngt
} // End namespace Cdmw
