/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#include "platformftproxy/Process_impl.hpp"
#include <SystemMngt/idllib/CdmwPlatformMngtBase.stub.hpp>
#include <SystemMngt/idllib/CdmwPlatformMngtDaemon.stub.hpp>
#include <SystemMngt/idllib/CdmwPlatformMngtEvent.stub.hpp>
#include <Foundation/orbsupport/ExceptionMinorCodes.hpp>
#include <Foundation/logging/FunctionLogger.hpp>

const char* const LOGGER_NAME = "CDMW.PFMDMN.SupProxy.Process";

namespace Cdmw
{
namespace PlatformMngt
{
namespace Proxy
{




Process_impl::Process_impl (CORBA::ORB_ptr orb,
                ProxyCollection *proxy_collection,
                AccessorLookupService *lookup_service)
    throw (Cdmw::BadParameterException, 
       Cdmw::OutOfMemoryException)                  
    : ManagedElement_impl(orb),
      m_accessor(CdmwFTSystMngt::ProcessAccessor::_nil()),
      m_accessor_ok(false),
      m_proxy_collection(proxy_collection),
      m_lookup_service(lookup_service)
 {
     m_proxy_collection->add_ref();
     set_logger (Get_Logger ());
     set_managed_element (this);
     set_proxy_collection (proxy_collection);
 }

Cdmw::Logging::Logger* Process_impl::Get_Logger () {
        static Cdmw::Logging::Logger* logger =
            Cdmw::Logging::Logger::Get_Logger(LOGGER_NAME);
        return logger;
    }

CdmwFTSystMngt::ManagedElementAccessor_ptr Process_impl::
        accessor_managed_element () throw (CORBA::TRANSIENT)
      {
         return this->accessor();
      }

const CdmwFTSystMngt::ProcessAccessor_ptr 
Process_impl::accessor() 
    throw(CORBA::TRANSIENT) 
{
    if (!m_accessor_ok) {
    
    CDMW_MUTEX_GUARD(m_accessor_mutex);
    // leaving mutex other thread might have fetched the data
    // already

    try {

        if (CORBA::is_nil(m_accessor.in())) {
        m_accessor = 
            m_lookup_service->getProcessAccessor(); // raises AccessorUndefined
        // update the flag
        m_accessor_ok = ! CORBA::is_nil(m_accessor.in());
        }

    }
    catch (...) {
        throw CORBA::TRANSIENT(::Cdmw::OrbSupport::TRANSIENT,
                   CORBA::COMPLETED_NO);
    }
    
    // paranoid testing
    if (CORBA::is_nil(m_accessor.in())) {
        throw CORBA::TRANSIENT(::Cdmw::OrbSupport::TRANSIENT,
                   CORBA::COMPLETED_NO);
    }

    // releasing mutex here
    }
    
    CDMW_ASSERT(m_accessor_ok &&
        ! CORBA::is_nil(m_accessor.in()));

    return m_accessor.ptr(); // return const ptr
}


Process_impl::~Process_impl() throw()
{
    m_proxy_collection->remove_ref();
}


CdmwPlatformMngt::ProcessType Process_impl::type()
throw(  CORBA::SystemException )
{
    CDMW_LOG_FUNCTION(Get_Logger(), "");
    try {  
        std::string target = this->getObjectId();
        
        return accessor()->get_type(target.c_str());
    } 
    catch (const CdmwPlatformMngt::ManagedElementNotFound & ex) {
        CORBA::OBJECT_NOT_EXIST e(OrbSupport::OBJECT_NOT_EXIST, 
                                  CORBA::COMPLETED_NO);
        CDMW_LOG_FUNCTION_EXCEPTION(e);
        throw e;
    }
    catch (const CORBA::Exception & ex) {
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw;
    }
    catch (...) {
        CORBA::INTERNAL ex(Cdmw::OrbSupport::INTERNAL, 
                           CORBA::COMPLETED_MAYBE);
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw ex;
    }
}

CORBA::Object_ptr Process_impl::embedded_service()
throw(CORBA::SystemException)
{
    CDMW_LOG_FUNCTION(Get_Logger(), "");
    try {  
        std::string target = this->getObjectId();
        
        return accessor()->get_embedded_service(target.c_str());
    } 
    catch (const CdmwPlatformMngt::ManagedElementNotFound & ex) {
        CORBA::OBJECT_NOT_EXIST e(OrbSupport::OBJECT_NOT_EXIST, 
                                  CORBA::COMPLETED_NO);
        CDMW_LOG_FUNCTION_EXCEPTION(e);
        throw e;
    }
    catch (const CORBA::Exception & ex) {
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw;
    }
    catch (...) {
        CORBA::INTERNAL ex(Cdmw::OrbSupport::INTERNAL, 
                           CORBA::COMPLETED_MAYBE);
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw ex;
    }
}


CdmwPlatformMngt::ProcessPID Process_impl::pid()
throw(CORBA::SystemException)
{
    CDMW_LOG_FUNCTION(Get_Logger(), "");
    try {  
        std::string target = this->getObjectId();
        
        return accessor()->get_pid(target.c_str());
    } 
    catch (const CdmwPlatformMngt::ManagedElementNotFound & ex) {
        CORBA::OBJECT_NOT_EXIST e(OrbSupport::OBJECT_NOT_EXIST, 
                                  CORBA::COMPLETED_NO);
        CDMW_LOG_FUNCTION_EXCEPTION(e);
        throw e;
    }
    catch (const CORBA::Exception & ex) {
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw;
    }
    catch (...) {
        CORBA::INTERNAL ex(Cdmw::OrbSupport::INTERNAL, 
                           CORBA::COMPLETED_MAYBE);
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw ex;
    }
}

                
CORBA::Boolean Process_impl::created_dynamically()
throw(CORBA::SystemException)
{
    CDMW_LOG_FUNCTION(Get_Logger(), "");
    try {  
        std::string target = this->getObjectId();
        
        return accessor()->get_created_dynamically(target.c_str());
    } 
    catch (const CdmwPlatformMngt::ManagedElementNotFound & ex) {
        CORBA::OBJECT_NOT_EXIST e(OrbSupport::OBJECT_NOT_EXIST, 
                                  CORBA::COMPLETED_NO);
        CDMW_LOG_FUNCTION_EXCEPTION(e);
        throw e;
    }
    catch (const CORBA::Exception & ex) {
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw;
    }
    catch (...) {
        CORBA::INTERNAL ex(Cdmw::OrbSupport::INTERNAL, 
                           CORBA::COMPLETED_MAYBE);
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw ex;
    }
}
 
         
CORBA::ULong Process_impl::restart_attempt()
throw(CORBA::SystemException)
{
    CDMW_LOG_FUNCTION(Get_Logger(), "");
    try {  
        std::string target = this->getObjectId();
        
        return accessor()->get_restart_attempt(target.c_str());
    } 
    catch (const CdmwPlatformMngt::ManagedElementNotFound & ex) {
        CORBA::OBJECT_NOT_EXIST e(OrbSupport::OBJECT_NOT_EXIST, 
                                  CORBA::COMPLETED_NO);
        CDMW_LOG_FUNCTION_EXCEPTION(e);
        throw e;
    }
    catch (const CORBA::Exception & ex) {
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw;
    }
    catch (...) {
        CORBA::INTERNAL ex(Cdmw::OrbSupport::INTERNAL, 
                           CORBA::COMPLETED_MAYBE);
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw ex;
    }
}


bool Process_impl::get_run_as_root()
throw(CORBA::SystemException)
{
    CDMW_LOG_FUNCTION(Get_Logger(), "");
    try {  
        std::string target = this->getObjectId();
        
        return accessor()->get_run_as_root(target.c_str());
    } 
    catch (const CdmwPlatformMngt::ManagedElementNotFound & ex) {
        CORBA::OBJECT_NOT_EXIST e(OrbSupport::OBJECT_NOT_EXIST, 
                                  CORBA::COMPLETED_NO);
        CDMW_LOG_FUNCTION_EXCEPTION(e);
        throw e;
    }
    catch (const CORBA::Exception & ex) {
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw;
    }
    catch (...) {
        CORBA::INTERNAL ex(Cdmw::OrbSupport::INTERNAL, 
                           CORBA::COMPLETED_MAYBE);
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw ex;
    }
}
    
CdmwPlatformMngtBase::TimeStamp Process_impl::start_time()
throw(CORBA::SystemException)
{
    CDMW_LOG_FUNCTION(Get_Logger(), "");
    try {  
        std::string target = this->getObjectId();
        
        return accessor()->get_start_time(target.c_str());
    } 
    catch (const CdmwPlatformMngt::ManagedElementNotFound & ex) {
        CORBA::OBJECT_NOT_EXIST e(OrbSupport::OBJECT_NOT_EXIST, 
                                  CORBA::COMPLETED_NO);
        CDMW_LOG_FUNCTION_EXCEPTION(e);
        throw e;
    }
    catch (const CORBA::Exception & ex) {
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw;
    }
    catch (...) {
        CORBA::INTERNAL ex(Cdmw::OrbSupport::INTERNAL, 
                           CORBA::COMPLETED_MAYBE);
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw ex;
    }
}
       
CdmwPlatformMngtBase::TimeStamp Process_impl::stop_time()
throw(CORBA::SystemException)
{
    CDMW_LOG_FUNCTION(Get_Logger(), "");
    try {  
        std::string target = this->getObjectId();
        
        return accessor()->get_stop_time(target.c_str());
    } 
    catch (const CdmwPlatformMngt::ManagedElementNotFound & ex) {
        CORBA::OBJECT_NOT_EXIST e(OrbSupport::OBJECT_NOT_EXIST, 
                                  CORBA::COMPLETED_NO);
        CDMW_LOG_FUNCTION_EXCEPTION(e);
        throw e;
    }
    catch (const CORBA::Exception & ex) {
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw;
    }
    catch (...) {
        CORBA::INTERNAL ex(Cdmw::OrbSupport::INTERNAL, 
                           CORBA::COMPLETED_MAYBE);
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw ex;
    }
}
    
    
CORBA::Boolean Process_impl::is_auto_ending()
throw(  CORBA::SystemException )
{  
    CDMW_LOG_FUNCTION(Get_Logger(), "");
    try {  
        std::string target = this->getObjectId();
        
        return accessor()->is_auto_ending(target.c_str());
    } 
    catch (const CdmwPlatformMngt::ManagedElementNotFound & ex) {
        CORBA::OBJECT_NOT_EXIST e(OrbSupport::OBJECT_NOT_EXIST, 
                                  CORBA::COMPLETED_NO);
        CDMW_LOG_FUNCTION_EXCEPTION(e);
        throw e;
    }
    catch (const CORBA::Exception & ex) {
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw;
    }
    catch (...) {
        CORBA::INTERNAL ex(Cdmw::OrbSupport::INTERNAL, 
                           CORBA::COMPLETED_MAYBE);
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw ex;
    }
}

CdmwPlatformMngt::Host_ptr Process_impl::get_host()
throw(  CORBA::SystemException )
{  
    CDMW_LOG_FUNCTION(Get_Logger(), "");
    try {  
        std::string target = this->getObjectId();
        
        CORBA::String_var uid =
            accessor()->get_host(target.c_str());
        
        return this->m_proxy_collection->createHostRef(uid);
    } 
    catch (const CdmwPlatformMngt::ManagedElementNotFound & ex) {
        CORBA::OBJECT_NOT_EXIST e(OrbSupport::OBJECT_NOT_EXIST, 
                                  CORBA::COMPLETED_NO);
        CDMW_LOG_FUNCTION_EXCEPTION(e);
        throw e;
    }
    catch (const CORBA::Exception & ex) {
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw;
    }
    catch (...) {
        CORBA::INTERNAL ex(Cdmw::OrbSupport::INTERNAL, 
                           CORBA::COMPLETED_MAYBE);
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw ex;
    }
}


CdmwPlatformMngt::ProcessInfo* Process_impl::get_process_info()
throw(  CORBA::SystemException )
{ 
    CDMW_LOG_FUNCTION(Get_Logger(), "");
    try {  
        std::string target = this->getObjectId();
        
        return accessor()->get_process_info(target.c_str());
    } 
    catch (const CdmwPlatformMngt::ManagedElementNotFound & ex) {
        CORBA::OBJECT_NOT_EXIST e(OrbSupport::OBJECT_NOT_EXIST, 
                                  CORBA::COMPLETED_NO);
        CDMW_LOG_FUNCTION_EXCEPTION(e);
        throw e;
    }
    catch (const CORBA::Exception & ex) {
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw;
    }
    catch (...) {
        CORBA::INTERNAL ex(Cdmw::OrbSupport::INTERNAL, 
                           CORBA::COMPLETED_MAYBE);
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw ex;
    }
}

CdmwPlatformMngt::LifeCycleDef* Process_impl::get_life_cycle()
throw( CdmwPlatformMngt::IncompatibleType,
       CORBA::SystemException )
{
    CDMW_LOG_FUNCTION(Get_Logger(), "");
    try {  
        std::string target = this->getObjectId();
        
        return accessor()->get_life_cycle(target.c_str());
    } 
    catch (const CdmwPlatformMngt::ManagedElementNotFound & ex) {
        CORBA::OBJECT_NOT_EXIST e(OrbSupport::OBJECT_NOT_EXIST, 
                                  CORBA::COMPLETED_NO);
        CDMW_LOG_FUNCTION_EXCEPTION(e);
        throw e;
    }
    catch (const CORBA::Exception & ex) {
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw;
    }
    catch (...) {
        CORBA::INTERNAL ex(Cdmw::OrbSupport::INTERNAL, 
                           CORBA::COMPLETED_MAYBE);
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw ex;
    }
}

void Process_impl::set_life_cycle(
    const CdmwPlatformMngt::LifeCycleDef& life_cycle )
throw( CdmwPlatformMngt::DuplicateStep,
       CdmwPlatformMngt::IncompatibleType,
       CdmwPlatformMngt::IncompatibleStatus,
       CORBA::SystemException )
{
    CDMW_LOG_FUNCTION(Get_Logger(), "");
    try {  
        std::string target = this->getObjectId();
        
        accessor()->set_life_cycle(target.c_str(),
                       life_cycle);
    } 
    catch (const CdmwPlatformMngt::ManagedElementNotFound & ex) {
        CORBA::OBJECT_NOT_EXIST e(OrbSupport::OBJECT_NOT_EXIST, 
                                  CORBA::COMPLETED_NO);
        CDMW_LOG_FUNCTION_EXCEPTION(e);
        throw e;
    }
    catch (const CORBA::Exception & ex) {
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw;
    }
    catch (...) {
        CORBA::INTERNAL ex(Cdmw::OrbSupport::INTERNAL, 
                           CORBA::COMPLETED_MAYBE);
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw ex;
    }
}

CdmwPlatformMngt::CompleteMonitoringInfo Process_impl::get_monitoring_info()
throw( CdmwPlatformMngt::IncompatibleType,
       CORBA::SystemException )
{
    CDMW_LOG_FUNCTION(Get_Logger(), "");
    try {  
        std::string target = this->getObjectId();
        
        return accessor()->get_monitoring_info(target.c_str());
    } 
    catch (const CdmwPlatformMngt::ManagedElementNotFound & ex) {
        CORBA::OBJECT_NOT_EXIST e(OrbSupport::OBJECT_NOT_EXIST, 
                                  CORBA::COMPLETED_NO);
        CDMW_LOG_FUNCTION_EXCEPTION(e);
        throw e;
    }
    catch (const CORBA::Exception & ex) {
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw;
    }
    catch (...) {
        CORBA::INTERNAL ex(Cdmw::OrbSupport::INTERNAL, 
                           CORBA::COMPLETED_MAYBE);
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw ex;
    }
}

void Process_impl::set_monitoring_info(
    const CdmwPlatformMngt::CompleteMonitoringInfo& monitoring_info)
throw( CdmwPlatformMngt::IncompatibleType,
       CdmwPlatformMngt::IncompatibleStatus,
       CORBA::SystemException )
{
    CDMW_LOG_FUNCTION(Get_Logger(), "");
    try {  
        std::string target = this->getObjectId();
        
        accessor()->set_monitoring_info(target.c_str(),
                        monitoring_info);
    } 
    catch (const CdmwPlatformMngt::ManagedElementNotFound & ex) {
        CORBA::OBJECT_NOT_EXIST e(OrbSupport::OBJECT_NOT_EXIST, 
                                  CORBA::COMPLETED_NO);
        CDMW_LOG_FUNCTION_EXCEPTION(e);
        throw e;
    }
    catch (const CORBA::Exception & ex) {
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw;
    }
    catch (...) {
        CORBA::INTERNAL ex(Cdmw::OrbSupport::INTERNAL, 
                           CORBA::COMPLETED_MAYBE);
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw ex;
    }
}

CdmwPlatformMngt::ExitAutoRestartInfo* Process_impl::get_exit_autorestart_info ()
throw (CORBA::SystemException)
{
    CDMW_LOG_FUNCTION(Get_Logger(), "");
    try {  
        std::string target = this->getObjectId();
        
        return accessor()->get_exit_autorestart_info(target.c_str());
    } 
    catch (const CdmwPlatformMngt::ManagedElementNotFound & ex) {
        CORBA::OBJECT_NOT_EXIST e(OrbSupport::OBJECT_NOT_EXIST, 
                                  CORBA::COMPLETED_NO);
        CDMW_LOG_FUNCTION_EXCEPTION(e);
        throw e;
    }
    catch (const CORBA::Exception & ex) {
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw;
    }
    catch (...) {
        CORBA::INTERNAL ex(Cdmw::OrbSupport::INTERNAL, 
                           CORBA::COMPLETED_MAYBE);
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw ex;
    }
}

             
             
void Process_impl::set_exit_autorestart_info (
        const CdmwPlatformMngt::ExitAutoRestartInfo & exit_autorestart_info)
throw (CdmwPlatformMngt::IncompatibleStatus,
       CORBA::SystemException)
{
    CDMW_LOG_FUNCTION(Get_Logger(), "");
    try {  
        std::string target = this->getObjectId();
        
        accessor()->set_exit_autorestart_info(target.c_str(),
                              exit_autorestart_info);
    } 
    catch (const CdmwPlatformMngt::ManagedElementNotFound & ex) {
        CORBA::OBJECT_NOT_EXIST e(OrbSupport::OBJECT_NOT_EXIST, 
                                  CORBA::COMPLETED_NO);
        CDMW_LOG_FUNCTION_EXCEPTION(e);
        throw e;
    }
    catch (const CORBA::Exception & ex) {
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw;
    }
    catch (...) {
        CORBA::INTERNAL ex(Cdmw::OrbSupport::INTERNAL, 
                           CORBA::COMPLETED_MAYBE);
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw ex;
    }
}



CdmwPlatformMngt::ProcessSnapshot* Process_impl::get_snapshot()
throw(  CORBA::SystemException )
{
    CDMW_LOG_FUNCTION(Get_Logger(), "");
    try {  
        std::string target = this->getObjectId();
        
        return accessor()->get_snapshot(target.c_str());
    } 
    catch (const CdmwPlatformMngt::ManagedElementNotFound & ex) {
        CORBA::OBJECT_NOT_EXIST e(OrbSupport::OBJECT_NOT_EXIST, 
                                  CORBA::COMPLETED_NO);
        CDMW_LOG_FUNCTION_EXCEPTION(e);
        throw e;
    }
    catch (const CORBA::Exception & ex) {
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw;
    }
    catch (...) {
        CORBA::INTERNAL ex(Cdmw::OrbSupport::INTERNAL, 
                           CORBA::COMPLETED_MAYBE);
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw ex;
    }
}

CdmwPlatformMngt::RequestStatus Process_impl::next_step()
throw( CdmwPlatformMngt::InvalidStep,
       CdmwPlatformMngt::IncompatibleType,
       CdmwPlatformMngt::IncompatibleStatus,
       CORBA::SystemException )
{
    CDMW_LOG_FUNCTION(Get_Logger(), "");
    try {  
        std::string target = this->getObjectId();
        
        return accessor()->next_step(target.c_str());
    } 
    catch (const CdmwPlatformMngt::ManagedElementNotFound & ex) {
        CORBA::OBJECT_NOT_EXIST e(OrbSupport::OBJECT_NOT_EXIST, 
                                  CORBA::COMPLETED_NO);
        CDMW_LOG_FUNCTION_EXCEPTION(e);
        throw e;
    }
    catch (const CORBA::Exception & ex) {
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw;
    }
    catch (...) {
        CORBA::INTERNAL ex(Cdmw::OrbSupport::INTERNAL, 
                           CORBA::COMPLETED_MAYBE);
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw ex;
    }
}

CdmwPlatformMngt::RequestStatus Process_impl::run()
throw( CdmwPlatformMngt::NotReadyToRun,
       CdmwPlatformMngt::IncompatibleType,
       CdmwPlatformMngt::IncompatibleStatus,
       CORBA::SystemException )
{
    CDMW_LOG_FUNCTION(Get_Logger(), "");
    try {  
        std::string target = this->getObjectId();
        
        return accessor()->run(target.c_str());
    } 
    catch (const CdmwPlatformMngt::ManagedElementNotFound & ex) {
        CORBA::OBJECT_NOT_EXIST e(OrbSupport::OBJECT_NOT_EXIST, 
                                  CORBA::COMPLETED_NO);
        CDMW_LOG_FUNCTION_EXCEPTION(e);
        throw e;
    }
    catch (const CORBA::Exception & ex) {
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw;
    }
    catch (...) {
        CORBA::INTERNAL ex(Cdmw::OrbSupport::INTERNAL, 
                           CORBA::COMPLETED_MAYBE);
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw ex;
    }
}

CdmwPlatformMngt::RequestStatus Process_impl::cold_restart (
        const CdmwPlatformMngtBase::StartupKind & startup_kind)
      throw (CORBA::SystemException,
         CdmwPlatformMngt::IncompatibleStatus)
{
    CDMW_LOG_FUNCTION(Get_Logger(), "");
    //get issuer value

    std::string command_issuer (m_current->get_issuer());
    
    try {  
        std::string target = this->getObjectId();
        
        return accessor()->cold_restart(target.c_str(), 
                                                 startup_kind,
                                                 command_issuer.c_str());
    } 
    catch (const CdmwPlatformMngt::ManagedElementNotFound & ex) {
        CORBA::OBJECT_NOT_EXIST e(OrbSupport::OBJECT_NOT_EXIST, 
                                  CORBA::COMPLETED_NO);
        CDMW_LOG_FUNCTION_EXCEPTION(e);
        throw e;
    }
    catch (const CORBA::Exception & ex) {
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw;
    }
    catch (...) {
        CORBA::INTERNAL ex(Cdmw::OrbSupport::INTERNAL, 
                           CORBA::COMPLETED_MAYBE);
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw ex;
    }
}


CdmwPlatformMngt::RequestStatus Process_impl::hot_restart (
        const CdmwPlatformMngtBase::StartupKind & startup_kind)
      throw (CORBA::SystemException,
         CdmwPlatformMngt::IncompatibleStatus)
{
    CDMW_LOG_FUNCTION(Get_Logger(), "");
    //get issuer value
    
    std::string command_issuer (m_current->get_issuer());
   
    try {  
        std::string target = this->getObjectId();
        
        return accessor()->hot_restart(target.c_str(), 
                                                startup_kind,
                                                command_issuer.c_str());
    } 
    catch (const CdmwPlatformMngt::ManagedElementNotFound & ex) {
        CORBA::OBJECT_NOT_EXIST e(OrbSupport::OBJECT_NOT_EXIST, 
                                  CORBA::COMPLETED_NO);
        CDMW_LOG_FUNCTION_EXCEPTION(e);
        throw e;
    }
    catch (const CORBA::Exception & ex) {
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw;
    }
    catch (...) {
        CORBA::INTERNAL ex(Cdmw::OrbSupport::INTERNAL, 
                           CORBA::COMPLETED_MAYBE);
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw ex;
    }
}



CORBA::ULong Process_impl::get_number_of_entities()
throw( CORBA::SystemException )
{
    CDMW_LOG_FUNCTION(Get_Logger(), "");
    try {  
        std::string target = this->getObjectId();
        
        return accessor()->get_number_of_entities(target.c_str());
    } 
    catch (const CdmwPlatformMngt::ManagedElementNotFound & ex) {
        CORBA::OBJECT_NOT_EXIST e(OrbSupport::OBJECT_NOT_EXIST, 
                                  CORBA::COMPLETED_NO);
        CDMW_LOG_FUNCTION_EXCEPTION(e);
        throw e;
    }
    catch (const CORBA::Exception & ex) {
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw;
    }
    catch (...) {
        CORBA::INTERNAL ex(Cdmw::OrbSupport::INTERNAL, 
                           CORBA::COMPLETED_MAYBE);
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw ex;
    }
}

void Process_impl::get_all_entity_names(
    CORBA::ULong how_many,
    CdmwPlatformMngt::ElementNames_out entity_names,
    CdmwPlatformMngt::ElementNamesIterator_out rest )
throw( CORBA::SystemException )
{
    CDMW_LOG_FUNCTION(Get_Logger(), "");
    try {  
        std::string target = this->getObjectId();
          
        accessor()->get_all_entity_names(target.c_str(),
                         how_many,
                         entity_names,
                         rest); 
    } 
    catch (const CdmwPlatformMngt::ManagedElementNotFound & ex) {
        CORBA::OBJECT_NOT_EXIST e(OrbSupport::OBJECT_NOT_EXIST, 
                                  CORBA::COMPLETED_NO);
        CDMW_LOG_FUNCTION_EXCEPTION(e);
        throw e;
    }
    catch (const CORBA::Exception & ex) {
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw;
    }
    catch (...) {
        CORBA::INTERNAL ex(Cdmw::OrbSupport::INTERNAL, 
                           CORBA::COMPLETED_MAYBE);
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw ex;
    }
}

CdmwPlatformMngt::Entity_ptr Process_impl::get_entity (const char* entity_name)
throw(CdmwPlatformMngt::EntityNotFound, CORBA::SystemException)
{ 
    CDMW_LOG_FUNCTION(Get_Logger(), "entity_name=" << entity_name);
    try {  
        std::string target = this->getObjectId();
          
        CORBA::String_var entity_target =
            accessor()->get_entity(target.c_str(), entity_name);
        
        return this->m_proxy_collection->createEntityRef(entity_target);
    } 
    catch (const CdmwPlatformMngt::ManagedElementNotFound & ex) {
        CORBA::OBJECT_NOT_EXIST e(OrbSupport::OBJECT_NOT_EXIST, 
                                  CORBA::COMPLETED_NO);
        CDMW_LOG_FUNCTION_EXCEPTION(e);
        throw e;
    }
    catch (const CORBA::Exception & ex) {
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw;
    }
    catch (...) {
        CORBA::INTERNAL ex(Cdmw::OrbSupport::INTERNAL, 
                           CORBA::COMPLETED_MAYBE);
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw ex;
    }
}

CORBA::Boolean Process_impl::get_entities(
    const CdmwPlatformMngt::ElementNames& entity_names,
    CdmwPlatformMngt::Entities_out nentities )
throw( CORBA::SystemException )
{
    CDMW_LOG_FUNCTION(Get_Logger(), "");
    try {   
        std::string target = this->getObjectId();
          
        ::CdmwPlatformMngt::ElementPaths_var paths = 
              new ::CdmwPlatformMngt::ElementPaths();
        
        CORBA::Boolean succeed = 
            accessor()->get_entities(target.c_str(),
                         entity_names,
                         paths.out());
        
        if (succeed) {
            nentities =
            this->m_proxy_collection->createEntitiesFromSeq(paths.ptr());
        }
            
        return succeed;
    } 
    catch (const CdmwPlatformMngt::ManagedElementNotFound & ex) {
        CORBA::OBJECT_NOT_EXIST e(OrbSupport::OBJECT_NOT_EXIST, 
                                  CORBA::COMPLETED_NO);
        CDMW_LOG_FUNCTION_EXCEPTION(e);
        throw e;
    }
    catch (const CORBA::Exception & ex) {
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw;
    }
    catch (...) {
        CORBA::INTERNAL ex(Cdmw::OrbSupport::INTERNAL, 
                           CORBA::COMPLETED_MAYBE);
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw ex;
    }
}

void Process_impl::get_all_entities(
    CORBA::ULong how_many,
    CdmwPlatformMngt::Entities_out nentities,
    CdmwPlatformMngt::EntitiesIterator_out rest )
throw( CORBA::SystemException )
{ 
    CDMW_LOG_FUNCTION(Get_Logger(), "");
    try {  
        std::string target = this->getObjectId();
          
        ::CdmwPlatformMngt::ElementPaths_var         paths;
        ::CdmwPlatformMngt::ElementPathsIterator_var iter;
        
        accessor()->get_all_entities(target.c_str(),
                         how_many,
                         paths.out(),
                         iter.out());
        
        CdmwPlatformMngt::Entities_var tmp_nentities = 
            this->m_proxy_collection->createEntitiesFromSeq(paths.ptr());
        
        CdmwPlatformMngt::EntitiesIterator_var tmp_rest =
            this->m_proxy_collection->createEntitiesIteratorRef(iter.in());
        
        nentities = tmp_nentities._retn();
        rest      = tmp_rest._retn();
    } 
    catch (const CdmwPlatformMngt::ManagedElementNotFound & ex) {
        CORBA::OBJECT_NOT_EXIST e(OrbSupport::OBJECT_NOT_EXIST, 
                                  CORBA::COMPLETED_NO);
        CDMW_LOG_FUNCTION_EXCEPTION(e);
        throw e;
    }
    catch (const CORBA::Exception & ex) {
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw;
    }
    catch (...) {
        CORBA::INTERNAL ex(Cdmw::OrbSupport::INTERNAL, 
                           CORBA::COMPLETED_MAYBE);
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw ex;
    }
}

CdmwPlatformMngt::Entity_ptr Process_impl::add_entity(
    const CdmwPlatformMngt::EntityDef & entity_def)
throw( CdmwPlatformMngt::EntityAlreadyExists, CORBA::SystemException )
{
    CDMW_LOG_FUNCTION(Get_Logger(), "entity_name=" << entity_def.entity_name);
    try {  
        std::string target = this->getObjectId();
    
        CORBA::String_var uid = 
            accessor()->add_entity(target.c_str(),
                       entity_def);
        
        return this->m_proxy_collection->createEntityRef(uid);
    } 
    catch (const CdmwPlatformMngt::ManagedElementNotFound & ex) {
        CORBA::OBJECT_NOT_EXIST e(OrbSupport::OBJECT_NOT_EXIST, 
                                  CORBA::COMPLETED_NO);
        CDMW_LOG_FUNCTION_EXCEPTION(e);
        throw e;
    }
    catch (const CORBA::Exception & ex) {
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw;
    }
    catch (...) {
        CORBA::INTERNAL ex(Cdmw::OrbSupport::INTERNAL, 
                           CORBA::COMPLETED_MAYBE);
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw ex;
    }
}

void Process_impl::remove_entity(
    const char* entity_name )
throw( CdmwPlatformMngt::EntityNotFound, CORBA::SystemException )
{ 
    CDMW_LOG_FUNCTION(Get_Logger(), "entity_name=" << entity_name);
    try {  
        std::string target = this->getObjectId();
    
        accessor()->remove_entity(target.c_str(),
                      entity_name);
    } 
    catch (const CdmwPlatformMngt::ManagedElementNotFound & ex) {
        CORBA::OBJECT_NOT_EXIST e(OrbSupport::OBJECT_NOT_EXIST, 
                                  CORBA::COMPLETED_NO);
        CDMW_LOG_FUNCTION_EXCEPTION(e);
        throw e;
    }
    catch (const CORBA::Exception & ex) {
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw;
    }
    catch (...) {
        CORBA::INTERNAL ex(Cdmw::OrbSupport::INTERNAL, 
                           CORBA::COMPLETED_MAYBE);
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw ex;
    }
}

CdmwPlatformMngtService::ServiceProviderID*
Process_impl::get_service_def(
    const char* service_name )
throw( CdmwPlatformMngtService::ServiceNotFound, 
       CORBA::SystemException )
{
    CDMW_LOG_FUNCTION(Get_Logger(), "service_name=" << service_name);
    try {    
        std::string target = this->getObjectId();
    
        return accessor()->get_service_def(target.c_str(),
                           service_name);
    } 
    catch (const CdmwPlatformMngt::ManagedElementNotFound & ex) {
        CORBA::OBJECT_NOT_EXIST e(OrbSupport::OBJECT_NOT_EXIST, 
                                  CORBA::COMPLETED_NO);
        CDMW_LOG_FUNCTION_EXCEPTION(e);
        throw e;
    }
    catch (const CORBA::Exception & ex) {
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw;
    }
    catch (...) {
        CORBA::INTERNAL ex(Cdmw::OrbSupport::INTERNAL, 
                           CORBA::COMPLETED_MAYBE);
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw ex;
    }
}

void Process_impl::add_service_def(
    const CdmwPlatformMngtService::ServiceDef& service_def )
throw( CdmwPlatformMngtService::ServiceAlreadyExists, CORBA::SystemException )
{
    CDMW_LOG_FUNCTION(Get_Logger(), "service_name=" << service_def.service_name);
    try {    
        std::string target = this->getObjectId();
    
        accessor()->add_service_def(target.c_str(),
                        service_def);
    } 
    catch (const CdmwPlatformMngt::ManagedElementNotFound & ex) {
        CORBA::OBJECT_NOT_EXIST e(OrbSupport::OBJECT_NOT_EXIST, 
                                  CORBA::COMPLETED_NO);
        CDMW_LOG_FUNCTION_EXCEPTION(e);
        throw e;
    }
    catch (const CORBA::Exception & ex) {
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw;
    }
    catch (...) {
        CORBA::INTERNAL ex(Cdmw::OrbSupport::INTERNAL, 
                           CORBA::COMPLETED_MAYBE);
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw ex;
    }
}

void 
Process_impl::remove_service_def(
    const char* service_name )
throw( CdmwPlatformMngtService::ServiceNotFound, CORBA::SystemException )
{
    CDMW_LOG_FUNCTION(Get_Logger(), "service_name=" << service_name);
    try {    
        std::string target = this->getObjectId();
    
        accessor()->remove_service_def(target.c_str(),
                           service_name);
    } 
    catch (const CdmwPlatformMngt::ManagedElementNotFound & ex) {
        CORBA::OBJECT_NOT_EXIST e(OrbSupport::OBJECT_NOT_EXIST, 
                                  CORBA::COMPLETED_NO);
        CDMW_LOG_FUNCTION_EXCEPTION(e);
        throw e;
    }
    catch (const CORBA::Exception & ex) {
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw;
    }
    catch (...) {
        CORBA::INTERNAL ex(Cdmw::OrbSupport::INTERNAL, 
                           CORBA::COMPLETED_MAYBE);
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw ex;
    }
}

void 
Process_impl::remove_all_service_defs ()
     throw (CORBA::SystemException)
{
    CDMW_LOG_FUNCTION(Get_Logger(), "");
    try {    
        std::string target = this->getObjectId();
    
        accessor()->remove_all_service_defs(target.c_str());
    } 
    catch (const CdmwPlatformMngt::ManagedElementNotFound & ex) {
        CORBA::OBJECT_NOT_EXIST e(OrbSupport::OBJECT_NOT_EXIST, 
                                  CORBA::COMPLETED_NO);
        CDMW_LOG_FUNCTION_EXCEPTION(e);
        throw e;
    }
    catch (const CORBA::Exception & ex) {
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw;
    }
    catch (...) {
        CORBA::INTERNAL ex(Cdmw::OrbSupport::INTERNAL, 
                           CORBA::COMPLETED_MAYBE);
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw ex;
    }
}


    
void Process_impl::is_alive()
  throw(CORBA::SystemException)
{
    CDMW_LOG_FUNCTION(Get_Logger(), "");
    throw CORBA::NO_IMPLEMENT (OrbSupport::NO_IMPLEMENT, CORBA::COMPLETED_NO);
}


void Process_impl::set_create_process_result (
        CdmwPlatformMngt::ProcessCommandResult result,
        const char* result_info,
        CdmwPlatformMngt::ProcessDelegateWrapper_ptr process_delegate_wrap)
            throw (CORBA::SystemException)
{ 
    CDMW_LOG_FUNCTION(Get_Logger(), "");
    throw CORBA::NO_IMPLEMENT (OrbSupport::NO_IMPLEMENT, CORBA::COMPLETED_NO);
}

   
void 
Process_impl::set_create_process_result (
     ::CdmwPlatformMngt::ProcessCommandResult result,
     const char * error_info,
     const char * error_issuer,
     ::CdmwPlatformMngt::ProcessDelegateWrapper_ptr process_delegate_wrap)
    throw (CORBA::SystemException)
{
    CDMW_LOG_FUNCTION(Get_Logger(), "");
    throw CORBA::NO_IMPLEMENT (OrbSupport::NO_IMPLEMENT, CORBA::COMPLETED_NO);
}
     

void Process_impl::set_initialisation_result (
        CdmwPlatformMngt::ProcessCommandResult result)
            throw (CORBA::SystemException)
{
    CDMW_LOG_FUNCTION(Get_Logger(), "");
    throw CORBA::NO_IMPLEMENT (OrbSupport::NO_IMPLEMENT, CORBA::COMPLETED_NO);
}
     

void Process_impl::set_next_step_result (
        CdmwPlatformMngt::ProcessCommandResult result,
        CORBA::ULong step_nbr)
            throw (CORBA::SystemException)
{
    CDMW_LOG_FUNCTION(Get_Logger(), "");
    throw CORBA::NO_IMPLEMENT (OrbSupport::NO_IMPLEMENT, CORBA::COMPLETED_NO);
}
     

void Process_impl::set_run_result (
        CdmwPlatformMngt::ProcessCommandResult result)
            throw (CORBA::SystemException)
{
    CDMW_LOG_FUNCTION(Get_Logger(), "");
    throw CORBA::NO_IMPLEMENT (OrbSupport::NO_IMPLEMENT, CORBA::COMPLETED_NO);
}
     

void Process_impl::set_stop_result (
        CdmwPlatformMngt::ProcessCommandResult result)
            throw (CORBA::SystemException)
{
    CDMW_LOG_FUNCTION(Get_Logger(), "");
    throw CORBA::NO_IMPLEMENT (OrbSupport::NO_IMPLEMENT, CORBA::COMPLETED_NO);
}

         
CdmwPlatformMngt::ProcessExitInfo Process_impl::get_last_exit_code()
      throw (CORBA::SystemException)
    {
    try {   
        std::string target = this->getObjectId();
         
        return accessor()->get_last_exit_code(target.c_str());
    }
    catch (const CdmwPlatformMngt::IncompatibleType & ex) { 
        throw;
    }
    catch (const CdmwPlatformMngt::ManagedElementNotFound & ex) { 
        throw CORBA::OBJECT_NOT_EXIST(OrbSupport::OBJECT_NOT_EXIST, 
                                      CORBA::COMPLETED_NO);
    }
    catch (const CORBA::SystemException & ex) { 
        throw;
    }
    catch (...) { 
        throw CORBA::INTERNAL(Cdmw::OrbSupport::INTERNAL, 
                              CORBA::COMPLETED_MAYBE);
    }

    }


void Process_impl::set_indicator_relevant_status( const bool indicator_relevant_status)
      throw (CORBA::SystemException)
    {
    try {   
        std::string target = this->getObjectId();
         
        accessor()->set_indicator_relevant_status(target.c_str(), indicator_relevant_status);
    }
    catch (const CdmwPlatformMngt::IncompatibleType & ex) { 
        throw;
    }
    catch (const CdmwPlatformMngt::ManagedElementNotFound & ex) { 
        throw CORBA::OBJECT_NOT_EXIST(OrbSupport::OBJECT_NOT_EXIST, 
                                      CORBA::COMPLETED_NO);
    }
    catch (const CORBA::SystemException & ex) { 
        throw;
    }
    catch (...) { 
        throw CORBA::INTERNAL(Cdmw::OrbSupport::INTERNAL, 
                              CORBA::COMPLETED_MAYBE);
    }

    }

bool  Process_impl::get_indicator_relevant_status()
      throw (CORBA::SystemException)
    {
    try {   
        std::string target = this->getObjectId();
         
        return accessor()->get_indicator_relevant_status(target.c_str());
    }
    catch (const CdmwPlatformMngt::IncompatibleType & ex) { 
        throw;
    }
    catch (const CdmwPlatformMngt::ManagedElementNotFound & ex) { 
        throw CORBA::OBJECT_NOT_EXIST(OrbSupport::OBJECT_NOT_EXIST, 
                                      CORBA::COMPLETED_NO);
    }
    catch (const CORBA::SystemException & ex) { 
        throw;
    }
    catch (...) { 
        throw CORBA::INTERNAL(Cdmw::OrbSupport::INTERNAL, 
                              CORBA::COMPLETED_MAYBE);
    }

    }

     
  
CdmwPlatformMngt::ProcessStatus
Process_impl::get_status( CORBA::String_out status_info)
    throw(CORBA::SystemException)
{
    CDMW_LOG_FUNCTION(Get_Logger(), "");
    try {     
        std::string target = this->getObjectId();
        
        return accessor()->get_status(target.c_str(), status_info);
    } 
    catch (const CdmwPlatformMngt::ManagedElementNotFound & ex) {
        CORBA::OBJECT_NOT_EXIST e(OrbSupport::OBJECT_NOT_EXIST, 
                                  CORBA::COMPLETED_NO);
        CDMW_LOG_FUNCTION_EXCEPTION(e);
        throw e;
    }
    catch (const CORBA::Exception & ex) {
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw;
    }
    catch (...) {
        CORBA::INTERNAL ex(Cdmw::OrbSupport::INTERNAL, 
                           CORBA::COMPLETED_MAYBE);
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw ex;
    }
}


///get_memory_resident
bool
Process_impl::get_memory_resident()
    throw(CORBA::SystemException)
{
    CDMW_LOG_FUNCTION(Get_Logger(), "");
    try {     
        std::string target = this->getObjectId();
        
        return accessor()->get_memory_resident(target.c_str() );
    } 
    catch (const CdmwPlatformMngt::ManagedElementNotFound & ex) {
        CORBA::OBJECT_NOT_EXIST e(OrbSupport::OBJECT_NOT_EXIST, 
                                  CORBA::COMPLETED_NO);
        CDMW_LOG_FUNCTION_EXCEPTION(e);
        throw e;
    }
    catch (const CORBA::Exception & ex) {
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw;
    }
    catch (...) {
        CORBA::INTERNAL ex(Cdmw::OrbSupport::INTERNAL, 
                           CORBA::COMPLETED_MAYBE);
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw ex;
    }
 
}

///set_memory_resident
void
Process_impl::set_memory_resident(bool memory_resident)
    throw(CORBA::SystemException)
{
    CDMW_LOG_FUNCTION(Get_Logger(), "");
    try {     
        std::string target = this->getObjectId();
        
        return accessor()->set_memory_resident(target.c_str(), memory_resident);
    } 
    catch (const CdmwPlatformMngt::ManagedElementNotFound & ex) {
        CORBA::OBJECT_NOT_EXIST e(OrbSupport::OBJECT_NOT_EXIST, 
                                  CORBA::COMPLETED_NO);
        CDMW_LOG_FUNCTION_EXCEPTION(e);
        throw e;
    }
    catch (const CORBA::Exception & ex) {
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw;
    }
    catch (...) {
        CORBA::INTERNAL ex(Cdmw::OrbSupport::INTERNAL, 
                           CORBA::COMPLETED_MAYBE);
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw ex;
    }
}

///get_core_dump_size
 CORBA::Long
Process_impl::get_core_dump_size()
    throw(CORBA::SystemException)
{
    CDMW_LOG_FUNCTION(Get_Logger(), "");
    try {     
        std::string target = this->getObjectId();
        
        return accessor()->get_core_dump_size(target.c_str() );
    } 
    catch (const CdmwPlatformMngt::ManagedElementNotFound & ex) {
        CORBA::OBJECT_NOT_EXIST e(OrbSupport::OBJECT_NOT_EXIST, 
                                  CORBA::COMPLETED_NO);
        CDMW_LOG_FUNCTION_EXCEPTION(e);
        throw e;
    }
    catch (const CORBA::Exception & ex) {
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw;
    }
    catch (...) {
        CORBA::INTERNAL ex(Cdmw::OrbSupport::INTERNAL, 
                           CORBA::COMPLETED_MAYBE);
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw ex;
    }
}


void 
Process_impl::set_auto_ending(CORBA::Boolean auto_ending)
    throw (CdmwPlatformMngt::IncompatibleStatus,
       CORBA::SystemException)
{
    CDMW_LOG_FUNCTION(Get_Logger(), "");
    try {     
        std::string target = this->getObjectId();
        
        accessor()->set_auto_ending(target.c_str(), auto_ending);
    } 
    catch (const CdmwPlatformMngt::ManagedElementNotFound & ex) {
        CORBA::OBJECT_NOT_EXIST e(OrbSupport::OBJECT_NOT_EXIST, 
                                  CORBA::COMPLETED_NO);
        CDMW_LOG_FUNCTION_EXCEPTION(e);
        throw e;
    }
    catch (const CORBA::Exception & ex) {
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw;
    }
    catch (...) {
        CORBA::INTERNAL ex(Cdmw::OrbSupport::INTERNAL, 
                           CORBA::COMPLETED_MAYBE);
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw ex;
    }
}


void 
Process_impl::set_process_info(const CdmwPlatformMngt::ProcessInfo& process_info)
    throw (CdmwPlatformMngt::IncompatibleStatus,
       CORBA::SystemException)
{
    CDMW_LOG_FUNCTION(Get_Logger(), "");
    try {     
        std::string target = this->getObjectId();
        
        accessor()->set_process_info(target.c_str(), process_info);
    } 
    catch (const CdmwPlatformMngt::ManagedElementNotFound & ex) {
        CORBA::OBJECT_NOT_EXIST e(OrbSupport::OBJECT_NOT_EXIST, 
                                  CORBA::COMPLETED_NO);
        CDMW_LOG_FUNCTION_EXCEPTION(e);
        throw e;
    }
    catch (const CORBA::Exception & ex) {
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw;
    }
    catch (...) {
        CORBA::INTERNAL ex(Cdmw::OrbSupport::INTERNAL, 
                           CORBA::COMPLETED_MAYBE);
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw ex;
    }
}


CdmwPlatformMngt::RequestStatus 
Process_impl::initialise(const CdmwPlatformMngtBase::StartupKind& startup_kind)
    throw (CdmwPlatformMngt::IncompatibleStatus,
       CORBA::SystemException)
{
    CDMW_LOG_FUNCTION(Get_Logger(), "");
    try {     
        std::string target = this->getObjectId();
        
        return accessor()->initialise(target.c_str(), startup_kind);
    } 
    catch (const CdmwPlatformMngt::ManagedElementNotFound & ex) {
        CORBA::OBJECT_NOT_EXIST e(OrbSupport::OBJECT_NOT_EXIST, 
                                  CORBA::COMPLETED_NO);
        CDMW_LOG_FUNCTION_EXCEPTION(e);
        throw e;
    }
    catch (const CORBA::Exception & ex) {
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw;
    }
    catch (...) {
        CORBA::INTERNAL ex(Cdmw::OrbSupport::INTERNAL, 
                           CORBA::COMPLETED_MAYBE);
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw ex;
    }
}


CdmwPlatformMngt::RequestStatus 
Process_impl::stop(CORBA::Boolean emergency)
    throw (CORBA::SystemException)
{
    CDMW_LOG_FUNCTION(Get_Logger(), "");
    try {     
        std::string target = this->getObjectId();
        
        return accessor()->stop(target.c_str(), emergency);
    } 
    catch (const CdmwPlatformMngt::ManagedElementNotFound & ex) {
        CORBA::OBJECT_NOT_EXIST e(OrbSupport::OBJECT_NOT_EXIST, 
                                  CORBA::COMPLETED_NO);
        CDMW_LOG_FUNCTION_EXCEPTION(e);
        throw e;
    }
    catch (const CORBA::Exception & ex) {
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw;
    }
    catch (...) {
        CORBA::INTERNAL ex(Cdmw::OrbSupport::INTERNAL, 
                           CORBA::COMPLETED_MAYBE);
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw ex;
    }
}

   
CdmwPlatformMngt::LongRequestStatus
Process_impl::stop_and_wait(CORBA::Boolean emergency)
    throw (CORBA::SystemException)
{
    CDMW_LOG_FUNCTION(Get_Logger(), "");
    try {
        std::string target = this->getObjectId();

        return accessor()->stop_and_wait(target.c_str(), emergency);
    }
    catch (const CdmwPlatformMngt::ManagedElementNotFound & ex) {
            CORBA::OBJECT_NOT_EXIST e(OrbSupport::OBJECT_NOT_EXIST,
                                                  CORBA::COMPLETED_NO);
        CDMW_LOG_FUNCTION_EXCEPTION(e);
        throw e;
    }
    catch (const CORBA::Exception & ex) {
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw;
    }
    catch (...) {
        CORBA::INTERNAL ex(Cdmw::OrbSupport::INTERNAL,
                           CORBA::COMPLETED_MAYBE);
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw ex;
    }
}


void Process_impl::set_init_graph(const CdmwPlatformMngt::GraphElements& component_graph)
    throw (CORBA::SystemException)
{
    CDMW_LOG_FUNCTION(Get_Logger(), "");
    try {
        std::string target = this->getObjectId();

        return accessor()->set_init_graph(target.c_str(), component_graph);
    }
    catch (const CdmwPlatformMngt::ManagedElementNotFound & ex) {
            CORBA::OBJECT_NOT_EXIST e(OrbSupport::OBJECT_NOT_EXIST,
                                                  CORBA::COMPLETED_NO);
        CDMW_LOG_FUNCTION_EXCEPTION(e);
        throw e;
    }
    catch (const CORBA::Exception & ex) {
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw;
    }
    catch (...) {
        CORBA::INTERNAL ex(Cdmw::OrbSupport::INTERNAL,
                           CORBA::COMPLETED_MAYBE);
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw ex;
    }
}


void Process_impl::set_stop_graph(const CdmwPlatformMngt::GraphElements& component_graph)
   throw (CORBA::SystemException)
{
    CDMW_LOG_FUNCTION(Get_Logger(), "");
    try {
        std::string target = this->getObjectId();

        return accessor()->set_stop_graph(target.c_str(), component_graph);
    }
    catch (const CdmwPlatformMngt::ManagedElementNotFound & ex) {
            CORBA::OBJECT_NOT_EXIST e(OrbSupport::OBJECT_NOT_EXIST,
                                                  CORBA::COMPLETED_NO);
        CDMW_LOG_FUNCTION_EXCEPTION(e);
        throw e;
    }
    catch (const CORBA::Exception & ex) {
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw;
    }
    catch (...) {
        CORBA::INTERNAL ex(Cdmw::OrbSupport::INTERNAL,
                           CORBA::COMPLETED_MAYBE);
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw ex;
    }
}


CdmwPlatformMngt::CcmComponent_ptr Process_impl::add_component(CdmwPlatformMngt::ComponentDef& component_def)
    throw (CORBA::SystemException, CdmwPlatformMngt::ComponentAlreadyExists, CdmwPlatformMngt::ComponentServerNotFound,
       CdmwPlatformMngt::DuplicateEntity, CosPropertyService::MultipleExceptions,
           CdmwPlatformMngt::DuplicateService, CdmwPlatformMngt::DuplicateStep,
           CdmwPlatformMngt::IncompatibleStatus)
{
    CDMW_LOG_FUNCTION(Get_Logger(), "");
    try {
        std::string target = this->getObjectId();

        return accessor()->add_component(target.c_str(), component_def);
    }
    catch (const CdmwPlatformMngt::ManagedElementNotFound & ex) {
            CORBA::OBJECT_NOT_EXIST e(OrbSupport::OBJECT_NOT_EXIST,
                                                  CORBA::COMPLETED_NO);
        CDMW_LOG_FUNCTION_EXCEPTION(e);
        throw e;
    }
    catch (const CORBA::Exception & ex) {
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw;
    }
    catch (...) {
        CORBA::INTERNAL ex(Cdmw::OrbSupport::INTERNAL,
                           CORBA::COMPLETED_MAYBE);
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw ex;
    }
}

void Process_impl::remove_component(const char* component_name, const char* host_name)
     throw(CORBA::SystemException, CdmwPlatformMngt::ComponentNotFound, CdmwPlatformMngt::ComponentStillActive, CdmwPlatformMngt::IncompatibleStatus)
{
    CDMW_LOG_FUNCTION(Get_Logger(), "");
    try {
        std::string target = this->getObjectId();

        return accessor()->remove_component(target.c_str(), component_name, host_name);
    }
    catch (const CdmwPlatformMngt::ManagedElementNotFound & ex) {
            CORBA::OBJECT_NOT_EXIST e(OrbSupport::OBJECT_NOT_EXIST,
                                                  CORBA::COMPLETED_NO);
        CDMW_LOG_FUNCTION_EXCEPTION(e);
        throw e;
    }
    catch (const CORBA::Exception & ex) {
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw;
    }
    catch (...) {
        CORBA::INTERNAL ex(Cdmw::OrbSupport::INTERNAL,
                           CORBA::COMPLETED_MAYBE);
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw ex;
    }
}

CdmwPlatformMngt::ProcessPID Process_impl::previous_pid()
    throw (CORBA::SystemException)
{

    CDMW_LOG_FUNCTION(Get_Logger(), "");
    try {  
        std::string target = this->getObjectId();
        
        return accessor()->get_previous_pid(target.c_str());
    } 
    catch (const CdmwPlatformMngt::ManagedElementNotFound & ex) {
        CORBA::OBJECT_NOT_EXIST e(OrbSupport::OBJECT_NOT_EXIST, 
                                  CORBA::COMPLETED_NO);
        CDMW_LOG_FUNCTION_EXCEPTION(e);
        throw e;
    }
    catch (const CORBA::Exception & ex) {
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw;
    }
    catch (...) {
        CORBA::INTERNAL ex(Cdmw::OrbSupport::INTERNAL, 
                           CORBA::COMPLETED_MAYBE);
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw ex;
    }

}

void Process_impl::get_all_components(CORBA::ULong how_many, 
                CdmwPlatformMngt::CcmComponents_out ncomponents, 
                CdmwPlatformMngt::ComponentsIterator_out rest)
    throw (CORBA::SystemException)
{
            CDMW_LOG_FUNCTION(Get_Logger(), "");
            try {   
                std::string target = this->getObjectId();
                        
                CdmwPlatformMngt::ElementPaths_var paths ; 
                CdmwPlatformMngt::ComponentsIterator_var myrest; 
         
                accessor()->get_all_components(target.c_str(), 
                                  how_many,
                                  paths.out(),
                                  myrest.out());
         
                rest = myrest._retn() ;
         
                ncomponents =
                    this->m_proxy_collection->createCcmComponentsFromSeq(paths.ptr());
         
            }
            catch (const CdmwPlatformMngt::ManagedElementNotFound & ex) { 
                    CORBA::OBJECT_NOT_EXIST e(OrbSupport::OBJECT_NOT_EXIST, 
                                                          CORBA::COMPLETED_NO);
                CDMW_LOG_FUNCTION_EXCEPTION(e);
                throw e;
            }
            catch (const CORBA::Exception & ex) { 
                CDMW_LOG_FUNCTION_EXCEPTION(ex);
                throw;
            }
            catch (...) { 
                CORBA::INTERNAL ex(Cdmw::OrbSupport::INTERNAL, 
                                   CORBA::COMPLETED_MAYBE);
                CDMW_LOG_FUNCTION_EXCEPTION(ex);
                throw ex;
            }
         

}


CdmwPlatformMngt::CcmComponent* Process_impl::get_component(const char* component_name)
    throw (CdmwPlatformMngt::ComponentNotFound,
            CdmwPlatformMngt::NotComponentServer,
            CORBA::SystemException)
{
    CDMW_LOG_FUNCTION(Get_Logger(), "");
    try {   
        std::string target = this->getObjectId();
        
        CORBA::String_var uid =
            accessor()->get_component(target.c_str(), component_name);
        
        return this->m_proxy_collection->createCcmComponentRef(uid);
    } 
    catch (const CdmwPlatformMngt::ManagedElementNotFound & ex) { 
            CORBA::OBJECT_NOT_EXIST e(OrbSupport::OBJECT_NOT_EXIST, 
                                                  CORBA::COMPLETED_NO);
        CDMW_LOG_FUNCTION_EXCEPTION(e);
        throw e;
    }
    catch (const CORBA::Exception & ex) { 
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw;
    }
    catch (...) { 
        CORBA::INTERNAL ex(Cdmw::OrbSupport::INTERNAL, 
                           CORBA::COMPLETED_MAYBE);
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw ex;
    }


}

CORBA::ULong Process_impl::get_number_of_components()
    throw (CORBA::SystemException)
{


    CDMW_LOG_FUNCTION(Get_Logger(), "");
    try {   
        std::string target = this->getObjectId();
        
        return(accessor()->get_number_of_components(target.c_str()));
        
    } 
    catch (const CdmwPlatformMngt::ManagedElementNotFound & ex) { 
            CORBA::OBJECT_NOT_EXIST e(OrbSupport::OBJECT_NOT_EXIST, 
                                                  CORBA::COMPLETED_NO);
        CDMW_LOG_FUNCTION_EXCEPTION(e);
        throw e;
    }
    catch (const CORBA::Exception & ex) { 
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw;
    }
    catch (...) { 
        CORBA::INTERNAL ex(Cdmw::OrbSupport::INTERNAL, 
                           CORBA::COMPLETED_MAYBE);
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw ex;
    }


}

CdmwPlatformMngt::RequestStatus Process_impl::stop_process(CORBA::Boolean emergency)   
    throw (CORBA::SystemException)
{
    CDMW_LOG_FUNCTION(Get_Logger(), "");
    //get issuer value
    std::string command_issuer (m_current->get_issuer());

    try 
    {     
        std::string target = this->getObjectId();
        
        return accessor()->stop_process(target.c_str(), 
                                                 emergency,
                                                 command_issuer.c_str());
    } 
    catch (const CdmwPlatformMngt::ManagedElementNotFound & ex) {
        CORBA::OBJECT_NOT_EXIST e(OrbSupport::OBJECT_NOT_EXIST, 
                                  CORBA::COMPLETED_NO);
        CDMW_LOG_FUNCTION_EXCEPTION(e);
        throw e;
    }
    catch (const CORBA::Exception & ex) {
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw;
    }
    catch (...) 
    {
        CORBA::INTERNAL ex(Cdmw::OrbSupport::INTERNAL, 
                           CORBA::COMPLETED_MAYBE);
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw ex;
    }  
}

CdmwPlatformMngt::CcmComponent* Process_impl::add_component(const CdmwPlatformMngt::ComponentDef& component_def)
        throw (CORBA::SystemException)
{
    throw CORBA::NO_IMPLEMENT (OrbSupport::NO_IMPLEMENT, CORBA::COMPLETED_NO);
}




} // End namespace Proxy
} // End namespace PlatformMngt
} // End namespace Cdmw

