/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#include "platformftproxy/System_impl.hpp"
#include "platformftproxy/Host_impl.hpp"
// #include "platformftproxy/EntitiesIterator_impl.hpp"

#include <Foundation/orbsupport/ExceptionMinorCodes.hpp>
#include <SystemMngt/idllib/CdmwPlatformMngtEvent.stub.hpp>
#include <Foundation/common/Assert.hpp> 
#include <Foundation/logging/FunctionLogger.hpp>
#include <Foundation/orbsupport/CORBA.hpp>
#include <SystemMngt/tools/ManagedElementKey.hpp>

const char* const LOGGER_NAME = "CDMW.PFMDMN.SupProxy.System";

namespace Cdmw
{
namespace PlatformMngt
{
namespace Proxy
{

Cdmw::Logging::Logger* System_impl::Get_Logger () {
        static Cdmw::Logging::Logger* logger =
            Cdmw::Logging::Logger::Get_Logger(LOGGER_NAME);
        return logger;
    }

const char* 
System_impl::lookupSystemTargetPath() 
    throw (CORBA::SystemException) 
{

    if (!m_system_target_path_ok) {
    CDMW_MUTEX_GUARD(m_system_target_path_mutex);
    // leaving mutex other thread might have fetched the data
    // already
    if (m_system_target_path.in()==NULL ||
        strlen(m_system_target_path.in())==0) {

        try {
        // might return NULL string to indicate error, only
        // throws CORBA::SystemException
        m_system_target_path = 
            accessor()->get_system_element_path();
        }
        catch (const CORBA::SystemException &ex) {
        // eg. the accessor can not be resolved, etc
        throw;
        }
        catch (...) {
        // eg. out of memory
        throw CORBA::INTERNAL(::Cdmw::OrbSupport::INTERNAL,
                      CORBA::COMPLETED_NO);
        }
    }
    // verify the return value and, update the flag
    m_system_target_path_ok = 
        m_system_target_path.in()!=NULL  &&
        strlen(m_system_target_path.in())>0;

    if ( ! m_system_target_path_ok) {
        // systemMngt returned invalid value
        throw CORBA::TRANSIENT(::Cdmw::OrbSupport::TRANSIENT,
                   CORBA::COMPLETED_NO);
    }

    // releasing mutex here
    }

    // m_system_target path is defined, readonly  
    return m_system_target_path.in();
}

CdmwFTSystMngt::ManagedElementAccessor_ptr System_impl::
        accessor_managed_element () throw (CORBA::TRANSIENT)
      {
         return this->accessor();
      }

const CdmwFTSystMngt::SystemAccessor_ptr 
System_impl::accessor() 
    throw(CORBA::TRANSIENT)
{
    if ( ! m_system_accessor_ok) {
    CDMW_MUTEX_GUARD(m_system_accessor_mutex);
    // leaving mutex other thread might have fetched the data
    // already
    try {

        if (CORBA::is_nil(m_system_accessor.in())) {
        m_system_accessor = 
            m_lookup_service->getSystemAccessor(); // raises AccessorUndefined
        // update the flag
        m_system_accessor_ok = 
            ( ! CORBA::is_nil(m_system_accessor.in()) );
        }
    }
    catch (...) {
        throw CORBA::TRANSIENT(::Cdmw::OrbSupport::TRANSIENT,
                   CORBA::COMPLETED_NO);
    }
    
    // verify the value returned is valid
    if (CORBA::is_nil(m_system_accessor.in())) {
        throw CORBA::TRANSIENT(::Cdmw::OrbSupport::TRANSIENT,
                   CORBA::COMPLETED_NO);
    }
    
    // releasing mutex here
    }

    CDMW_ASSERT(m_system_accessor_ok &&
        ! CORBA::is_nil(m_system_accessor.in()));

    return m_system_accessor.ptr(); // return const ptr
}
// ----------------------------------------------------------------------
// System_impl class.
// ----------------------------------------------------------------------

System_impl::System_impl (CORBA::ORB_ptr orb,
              ProxyCollection *proxy_collection,
              AccessorLookupService *lookup_service) 
    throw (Cdmw::BadParameterException,
       Cdmw::OutOfMemoryException) 
    : ManagedElement_impl(orb),
      m_proxy_collection(proxy_collection),
      m_lookup_service(lookup_service),
      m_system_accessor(CdmwFTSystMngt::SystemAccessor::_nil()),
      m_system_accessor_ok(false),
      m_system_target_path((const char*) 0),
      m_system_target_path_ok(false)
{
    m_proxy_collection->add_ref();
    set_logger (Get_Logger ());
    set_managed_element (this);
    set_proxy_collection (proxy_collection);
}

System_impl::~System_impl() throw()
{
    m_proxy_collection->remove_ref();
}

CdmwPlatformMngt::Periodic::Notifier_ptr System_impl::periodic_notifier()
    throw (CORBA::SystemException)
{
    CDMW_LOG_FUNCTION(Get_Logger(), "");
    try 
    {
        CORBA::String_var uid =
            m_system_accessor->get_periodic_notifier();    

        return this->m_proxy_collection->createPeriodicNotifierRef(uid.in());
    }
    catch (const CdmwPlatformMngt::PeriodicNotifierNotFound & ex) {
            CORBA::OBJECT_NOT_EXIST e(OrbSupport::OBJECT_NOT_EXIST,
                                                  CORBA::COMPLETED_NO);
        CDMW_LOG_FUNCTION_EXCEPTION(e);
        throw e;
    }
    catch (const CORBA::Exception & ex) {
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw;
    }
    catch (...) {
        CORBA::INTERNAL ex(Cdmw::OrbSupport::INTERNAL,
                           CORBA::COMPLETED_MAYBE);
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw ex;
    }
}


CdmwPlatformMngt::SystemStatus 
System_impl::get_status(CORBA::String_out status_info)
    throw( CORBA::SystemException )
{
    CDMW_LOG_FUNCTION(Get_Logger(), "");

    try {
        CdmwFTSystMngt::SystemAccessor_ptr acc = 
            accessor();
    
        CDMW_ASSERT( ! CORBA::is_nil(acc));
    
        return acc->get_status(status_info);
    }
    catch (const CORBA::Exception & ex) {
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw;
    }
    catch (...) {
        CORBA::INTERNAL ex(Cdmw::OrbSupport::INTERNAL, 
                           CORBA::COMPLETED_MAYBE);
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw ex;
    }
}

 CdmwPlatformMngt::FunctioningMode System_impl::get_mode(CORBA::String_out mode_info)
    throw( CORBA::SystemException )
{
    CDMW_LOG_FUNCTION(Get_Logger(), "");

    try {
        return accessor()->get_mode(mode_info);
    }
    catch (const CORBA::Exception & ex) {
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw;
    }
    catch (...) {
        CORBA::INTERNAL ex(Cdmw::OrbSupport::INTERNAL, 
                           CORBA::COMPLETED_MAYBE);
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw ex;
    }
}

CdmwPlatformMngt::SystemSnapshot* 
System_impl::get_snapshot()
    throw( CORBA::SystemException )
{
    CDMW_LOG_FUNCTION(Get_Logger(), "");
    try {
        return accessor()->get_snapshot();
    }
    catch (const CORBA::Exception & ex) {
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw;
    }
    catch (...) {
        CORBA::INTERNAL ex(Cdmw::OrbSupport::INTERNAL, 
                           CORBA::COMPLETED_MAYBE);
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw ex;
    }
}


CORBA::ULong System_impl::get_number_of_applications()
    throw( CORBA::SystemException )
{
    CDMW_LOG_FUNCTION(Get_Logger(), "");
    try {
        return accessor()->get_number_of_applications();
    }
    catch (const CORBA::Exception & ex) {
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw;
    }
    catch (...) {
        CORBA::INTERNAL ex(Cdmw::OrbSupport::INTERNAL, 
                           CORBA::COMPLETED_MAYBE);
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw ex;
    }
}


CdmwPlatformMngt::Applications* 
System_impl::get_applications 
       (CORBA::ULong how_many,
    CdmwPlatformMngt::ApplicationsIterator_out applications_iterator)
    throw( CORBA::SystemException )
{
    CDMW_LOG_FUNCTION(Get_Logger(), "");
    try {
        CdmwPlatformMngt::ElementPathsIterator_var elem_iter;
        
        CdmwPlatformMngt::ElementPaths_var paths = 
            accessor()->get_applications(how_many, elem_iter.out());
        
        applications_iterator =
            this->m_proxy_collection->createApplicationsIteratorRef(elem_iter.in());
        
        CdmwPlatformMngt::Applications_var apps_seq =
            this->m_proxy_collection->createApplicationsFromSeq(paths.ptr());
        
        return apps_seq._retn();
    }
    catch (const CORBA::Exception & ex) {
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw;
    }
    catch (...) {
        CORBA::INTERNAL ex(Cdmw::OrbSupport::INTERNAL, 
                           CORBA::COMPLETED_MAYBE);
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw ex;
    }
}





CdmwPlatformMngt::Application_ptr System_impl::get_application (
                          const char* application_name)
        throw (CdmwPlatformMngt::ApplicationNotFound,
               CORBA::SystemException)
{
    CDMW_LOG_FUNCTION(Get_Logger(), "application_name=" << application_name);
    try {
        CORBA::String_var uid = 
            accessor()->get_application(application_name);
        
        return this->m_proxy_collection->createApplicationRef(uid.in());
    }
    catch (const CORBA::Exception & ex) {
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw;
    }
    catch (...) {
        CORBA::INTERNAL ex(Cdmw::OrbSupport::INTERNAL, 
                           CORBA::COMPLETED_MAYBE);
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw ex;
    }
}


void System_impl::define(const CdmwPlatformMngt::SystemDef& system_def)
        throw (CdmwPlatformMngt::DuplicateEntity,
               CosPropertyService::MultipleExceptions,
               CdmwPlatformMngt::DuplicateService,
               CdmwPlatformMngt::DuplicateHost,
               CdmwPlatformMngt::DuplicateHostEntity,
               CdmwPlatformMngt::HostPropertyExceptions,
               CdmwPlatformMngt::AlreadyDone,
               CdmwPlatformMngt::IncompatibleStatus,
               CORBA::SystemException)
{
    CDMW_LOG_FUNCTION(Get_Logger(), "system_name=" << system_def.system_name);
    try {
        accessor()->define(system_def);
    }
    catch (const CORBA::Exception & ex) {
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw;
    }
    catch (...) {
        CORBA::INTERNAL ex(Cdmw::OrbSupport::INTERNAL, 
                           CORBA::COMPLETED_MAYBE);
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw ex;
    }
}


void System_impl::set_init_graph (const CdmwPlatformMngt::GraphElements& application_graph)                  
        throw (CdmwPlatformMngt::ReferenceNotFound,
               CdmwPlatformMngt::CircularReference,
               CdmwPlatformMngt::IncompatibleStatus,
               CORBA::SystemException)
{
    CDMW_LOG_FUNCTION(Get_Logger(), "");
    try {
        accessor()->set_init_graph(application_graph);
    }
    catch (const CORBA::Exception & ex) {
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw;
    }
    catch (...) {
        CORBA::INTERNAL ex(Cdmw::OrbSupport::INTERNAL, 
                           CORBA::COMPLETED_MAYBE);
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw ex;
    }
}


void System_impl::set_stop_graph (const CdmwPlatformMngt::GraphElements& application_graph)                  
        throw (CdmwPlatformMngt::ReferenceNotFound,
               CdmwPlatformMngt::CircularReference,
               CdmwPlatformMngt::IncompatibleStatus,
               CORBA::SystemException)
{
    CDMW_LOG_FUNCTION(Get_Logger(), "");
    try {
        accessor()->set_stop_graph(application_graph);
    }
    catch (const CORBA::Exception & ex) {
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw;
    }
    catch (...) {
        CORBA::INTERNAL ex(Cdmw::OrbSupport::INTERNAL, 
                           CORBA::COMPLETED_MAYBE);
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw ex;
    }
}



void System_impl::set_run_set(const CdmwPlatformMngt::RunSetDef& run_set_def)
        throw (CdmwPlatformMngt::DuplicateRunSetElement,
               CdmwPlatformMngt::IncompatibleStatus,
               CORBA::SystemException)
{
    CDMW_LOG_FUNCTION(Get_Logger(), "set_name" << run_set_def.set_name);
    try {
        accessor()->set_run_set(run_set_def);
    }
    catch (const CORBA::Exception & ex) {
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw;
    }
    catch (...) {
        CORBA::INTERNAL ex(Cdmw::OrbSupport::INTERNAL, 
                           CORBA::COMPLETED_MAYBE);
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw ex;
    }
}

void System_impl::remove_run_set(const char* run_set_name)
        throw (CdmwPlatformMngt::RunSetNotFound,
               CdmwPlatformMngt::IncompatibleStatus,
               CORBA::SystemException)
{
    CDMW_LOG_FUNCTION(Get_Logger(), "run_set_name=" << run_set_name);
    try {
        accessor()->remove_run_set(run_set_name);
    }
    catch (const CORBA::Exception & ex) {
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw;
    }
    catch (...) {
        CORBA::INTERNAL ex(Cdmw::OrbSupport::INTERNAL, 
                           CORBA::COMPLETED_MAYBE);
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw ex;
    }
}

void System_impl::remove_all_run_sets()
        throw (CdmwPlatformMngt::IncompatibleStatus,
               CORBA::SystemException)
{
    CDMW_LOG_FUNCTION(Get_Logger(), "");
    try {
        accessor()->remove_all_run_sets();
    }
    catch (const CORBA::Exception & ex) {
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw;
    }
    catch (...) {
        CORBA::INTERNAL ex(Cdmw::OrbSupport::INTERNAL, 
                           CORBA::COMPLETED_MAYBE);
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw ex;
    }
}



CdmwPlatformMngt::Host_ptr 
      System_impl::add_host(const CdmwPlatformMngt::HostDef& host_def)
        throw (CdmwPlatformMngt::HostAlreadyExists,
               CdmwPlatformMngt::DuplicateEntity, 
               CosPropertyService::MultipleExceptions,
               CdmwPlatformMngt::IncompatibleStatus,
               CORBA::SystemException)
{
    CDMW_LOG_FUNCTION(Get_Logger(), "host_name=" << host_def.host_name.in());
    try {
        CORBA::String_var host_id = 
            accessor()->add_host(host_def);
        
        return m_proxy_collection->createHostRef(host_id.in());
    }
    catch (const CORBA::Exception & ex) {
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw;
    }
    catch (...) {
        CORBA::INTERNAL ex(Cdmw::OrbSupport::INTERNAL, 
                           CORBA::COMPLETED_MAYBE);
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw ex;
    }
}

void System_impl::remove_host(const char* host_name)
        throw (CdmwPlatformMngt::HostNotFound,
               CdmwPlatformMngt::IncompatibleStatus,
               CORBA::SystemException)
{
    CDMW_LOG_FUNCTION(Get_Logger(), "host_name=" << host_name);
    try {
        accessor()->remove_host(host_name);
    }
    catch (const CORBA::Exception & ex) {
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw;
    }
    catch (...) {
        CORBA::INTERNAL ex(Cdmw::OrbSupport::INTERNAL, 
                           CORBA::COMPLETED_MAYBE);
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw ex;
    }
    
}                  


CdmwPlatformMngt::Application_ptr 
      System_impl::add_application(const CdmwPlatformMngt::PlatformApplicationDef& platform_application_def,
                                   const CdmwPlatformMngt::ApplicationDef& application_def)
        throw (CdmwPlatformMngt::ApplicationAlreadyExists,
               CdmwPlatformMngt::HostNotFound,
               CdmwPlatformMngt::DuplicateEntity,
               CosPropertyService::MultipleExceptions,
               CdmwPlatformMngt::DuplicateService,
               CdmwPlatformMngt::DuplicateProcess,
               CdmwPlatformMngt::ProcessHostNotFound,
               CdmwPlatformMngt::DuplicateProcessEntity,
               CdmwPlatformMngt::ProcessPropertyExceptions,
               CdmwPlatformMngt::DuplicateProcessService,
               CdmwPlatformMngt::DuplicateProcessStep,
               CdmwPlatformMngt::IncompatibleStatus,
               CORBA::SystemException)
{
    CDMW_LOG_FUNCTION(Get_Logger(), "application_name=" 
                                      << platform_application_def.application_name);
    try {
        CORBA::String_var uid = 
            accessor()->add_application(platform_application_def,
                               application_def);    
        
        return m_proxy_collection->createApplicationRef(uid.in());
    }
    catch (const CORBA::Exception & ex) {
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw;
    }
    catch (...) {
        CORBA::INTERNAL ex(Cdmw::OrbSupport::INTERNAL, 
                           CORBA::COMPLETED_MAYBE);
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw ex;
    }
}

void System_impl::remove_application(const char* application_name)
        throw (CdmwPlatformMngt::ApplicationNotFound,
               CdmwPlatformMngt::ApplicationStillActive,
               CdmwPlatformMngt::IncompatibleStatus,
               CORBA::SystemException)
{
    CDMW_LOG_FUNCTION(Get_Logger(), "application_name=" << application_name);
    try {
        accessor()->remove_application(application_name);
    }
    catch (const CORBA::Exception & ex) {
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw;
    }
    catch (...) {
        CORBA::INTERNAL ex(Cdmw::OrbSupport::INTERNAL, 
                           CORBA::COMPLETED_MAYBE);
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw ex;
    }
} 

void System_impl::start (const CdmwPlatformMngtBase::StartupKind& startup_kind)      
        throw (CdmwPlatformMngt::IncompatibleStatus,
               CORBA::SystemException)
{
    CDMW_LOG_FUNCTION(Get_Logger(), "");
    try {
        accessor()->start(startup_kind);
    }
    catch (const CORBA::Exception & ex) {
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw;
    }
    catch (...) {
        CORBA::INTERNAL ex(Cdmw::OrbSupport::INTERNAL, 
                           CORBA::COMPLETED_MAYBE);
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw ex;
    }
}
    

void System_impl::resume()
        throw (CdmwPlatformMngt::IncompatibleStatus,
               CORBA::SystemException)
{
    CDMW_LOG_FUNCTION(Get_Logger(), "");
    try {
        accessor()->resume();
    }
    catch (const CORBA::Exception & ex) {
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw;
    }
    catch (...) {
        CORBA::INTERNAL ex(Cdmw::OrbSupport::INTERNAL, 
                           CORBA::COMPLETED_MAYBE);
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw ex;
    }
}


void System_impl::acknowledge()
        throw (CdmwPlatformMngt::IncompatibleStatus,
               CORBA::SystemException)
{
    CDMW_LOG_FUNCTION(Get_Logger(), "");

    //get issuer value

    std::string command_issuer (m_current->get_issuer());

    try {
        
        accessor()->acknowledge(command_issuer.c_str());
    }
    catch (const CORBA::Exception & ex) {
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw;
    }
    catch (...) {
        CORBA::INTERNAL ex(Cdmw::OrbSupport::INTERNAL, 
                           CORBA::COMPLETED_MAYBE);
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw ex;
    }
}


void System_impl::stop (CORBA::Boolean emergency)
         throw( CORBA::SystemException )      
{
    CDMW_LOG_FUNCTION(Get_Logger(), "");
    try {
        accessor()->stop(emergency);
    }
    catch (const CORBA::Exception & ex) {
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw;
    }
    catch (...) {
        CORBA::INTERNAL ex(Cdmw::OrbSupport::INTERNAL, 
                           CORBA::COMPLETED_MAYBE);
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw ex;
    }
}


void System_impl::cold_restart (const CdmwPlatformMngtBase::StartupKind& startup_kind)      
throw (CdmwPlatformMngt::IncompatibleStatus,
       CORBA::SystemException)
{
    CDMW_LOG_FUNCTION(Get_Logger(), "");
    //get issuer value

    std::string command_issuer (m_current->get_issuer());

    try {
        accessor()->cold_restart(startup_kind, command_issuer.c_str());
    }
    catch (const CORBA::Exception & ex) {
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw;
    }
    catch (...) {
        CORBA::INTERNAL ex(Cdmw::OrbSupport::INTERNAL, 
                           CORBA::COMPLETED_MAYBE);
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw ex;
    }
}


void 
System_impl::hot_restart (const CdmwPlatformMngtBase::StartupKind& startup_kind)      
throw (CdmwPlatformMngt::IncompatibleStatus,
       CORBA::SystemException)
{
    CDMW_LOG_FUNCTION(Get_Logger(), "");
    //get issuer value

    std::string command_issuer (m_current->get_issuer());

    try {
        accessor()->hot_restart(startup_kind, command_issuer.c_str());
    }
    catch (const CORBA::Exception & ex) {
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw;
    }
    catch (...) {
        CORBA::INTERNAL ex(Cdmw::OrbSupport::INTERNAL, 
                           CORBA::COMPLETED_MAYBE);
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw ex;
    }
}


CORBA::ULong 
System_impl::get_number_of_entities()
throw( CORBA::SystemException )
{
    CDMW_LOG_FUNCTION(Get_Logger(), "");
    try {
        return accessor()->get_number_of_entities(lookupSystemTargetPath()); 
    }
    catch (const CdmwPlatformMngt::ManagedElementNotFound & ex) {
        CORBA::OBJECT_NOT_EXIST e(OrbSupport::OBJECT_NOT_EXIST, 
                                  CORBA::COMPLETED_NO);
        CDMW_LOG_FUNCTION_EXCEPTION(e);
        throw e;
    }
    catch (const CORBA::Exception & ex) {
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw;
    }
    catch (...) {
        CORBA::INTERNAL ex(Cdmw::OrbSupport::INTERNAL, 
                           CORBA::COMPLETED_MAYBE);
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw ex;
    }
}

void 
System_impl::get_all_entity_names (
                         CORBA::ULong how_many,
                         CdmwPlatformMngt::ElementNames_out entity_names,
                         CdmwPlatformMngt::ElementNamesIterator_out rest)
throw( CORBA::SystemException )
{
    CDMW_LOG_FUNCTION(Get_Logger(), "");
    try {
        accessor()->get_all_entity_names(lookupSystemTargetPath(),
                            how_many,
                            entity_names,
                            rest);
    }
    catch (const CdmwPlatformMngt::ManagedElementNotFound & ex) {
        CORBA::OBJECT_NOT_EXIST e(OrbSupport::OBJECT_NOT_EXIST, 
                                  CORBA::COMPLETED_NO);
        CDMW_LOG_FUNCTION_EXCEPTION(e);
        throw e;
    }
    catch (const CORBA::Exception & ex) {
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw;
    }
    catch (...) {
        CORBA::INTERNAL ex(Cdmw::OrbSupport::INTERNAL, 
                           CORBA::COMPLETED_MAYBE);
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw ex;
    }
}



CdmwPlatformMngt::Entity_ptr 
System_impl::get_entity (const char* entity_name)
throw (CdmwPlatformMngt::EntityNotFound,
       CORBA::SystemException)
{  
    CDMW_LOG_FUNCTION(Get_Logger(), "entity_name=" << entity_name);
    try {  
        CORBA::String_var uid = 
            accessor()->get_entity(lookupSystemTargetPath(),
                          entity_name);
        
        return m_proxy_collection->createEntityRef(uid);
    } 
    catch (const CdmwPlatformMngt::ManagedElementNotFound & ex) {
        CORBA::OBJECT_NOT_EXIST e(OrbSupport::OBJECT_NOT_EXIST, 
                                  CORBA::COMPLETED_NO);
        CDMW_LOG_FUNCTION_EXCEPTION(e);
        throw e;
    }
    catch (const CORBA::Exception & ex) {
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw;
    }
    catch (...) {
        CORBA::INTERNAL ex(Cdmw::OrbSupport::INTERNAL, 
                           CORBA::COMPLETED_MAYBE);
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw ex;
    }
}

CORBA::Boolean System_impl::get_entities (
                        const CdmwPlatformMngt::ElementNames& entity_names,
                        CdmwPlatformMngt::Entities_out nentities)
throw( CORBA::SystemException )
{
    CDMW_LOG_FUNCTION(Get_Logger(), "");
    try {  
        ::CdmwPlatformMngt::ElementPaths_var paths = 
              new ::CdmwPlatformMngt::ElementPaths();
    
        CORBA::Boolean succeed = 
            accessor()->get_entities(lookupSystemTargetPath(),
                            entity_names,
                            paths.out());
        if (succeed) {
            nentities = m_proxy_collection->createEntitiesFromSeq(paths.ptr());
        }
        
        return succeed;
    } 
    catch (const CdmwPlatformMngt::ManagedElementNotFound & ex) {
        CORBA::OBJECT_NOT_EXIST e(OrbSupport::OBJECT_NOT_EXIST, 
                                  CORBA::COMPLETED_NO);
        CDMW_LOG_FUNCTION_EXCEPTION(e);
        throw e;
    }
    catch (const CORBA::Exception & ex) {
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw;
    }
    catch (...) {
        CORBA::INTERNAL ex(Cdmw::OrbSupport::INTERNAL, 
                           CORBA::COMPLETED_MAYBE);
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw ex;
    }
}


void 
System_impl::get_all_entities (
                   CORBA::ULong how_many,
                   CdmwPlatformMngt::Entities_out nentities,
                   CdmwPlatformMngt::EntitiesIterator_out rest)
    throw( CORBA::SystemException )
{
    CDMW_LOG_FUNCTION(Get_Logger(), "");
    try {  
        ::CdmwPlatformMngt::ElementPaths_var         paths;
        ::CdmwPlatformMngt::ElementPathsIterator_var iter;
        
        accessor()->get_all_entities(lookupSystemTargetPath(),
                            how_many,
                            paths.out(),
                            iter.out());
        
        CdmwPlatformMngt::Entities_var tmp_nentities = 
            m_proxy_collection->createEntitiesFromSeq(paths.ptr());
        
        CdmwPlatformMngt::EntitiesIterator_var tmp_rest =
            m_proxy_collection->createEntitiesIteratorRef(iter.in());
        
        nentities = tmp_nentities._retn();
        rest      = tmp_rest._retn();
    } 
    catch (const CdmwPlatformMngt::ManagedElementNotFound & ex) {
        CORBA::OBJECT_NOT_EXIST e(OrbSupport::OBJECT_NOT_EXIST, 
                                  CORBA::COMPLETED_NO);
        CDMW_LOG_FUNCTION_EXCEPTION(e);
        throw e;
    }
    catch (const CORBA::Exception & ex) {
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw;
    }
    catch (...) {
        CORBA::INTERNAL ex(Cdmw::OrbSupport::INTERNAL, 
                           CORBA::COMPLETED_MAYBE);
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw ex;
    }
}


CdmwPlatformMngt::Entity_ptr 
System_impl::add_entity (const CdmwPlatformMngt::EntityDef & entity_def)
    throw(CdmwPlatformMngt::EntityAlreadyExists,
      CORBA::SystemException)
{
    CDMW_LOG_FUNCTION(Get_Logger(), "");
    try {  
        CORBA::String_var uid = 
            accessor()->add_entity(lookupSystemTargetPath(),
                          entity_def);
        
        return m_proxy_collection->createEntityRef(uid);
    } 
    catch (const CdmwPlatformMngt::ManagedElementNotFound & ex) {
        CORBA::OBJECT_NOT_EXIST e(OrbSupport::OBJECT_NOT_EXIST, 
                                  CORBA::COMPLETED_NO);
        CDMW_LOG_FUNCTION_EXCEPTION(e);
        throw e;
    }
    catch (const CORBA::Exception & ex) {
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw;
    }
    catch (...) {
        CORBA::INTERNAL ex(Cdmw::OrbSupport::INTERNAL, 
                           CORBA::COMPLETED_MAYBE);
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw ex;
    }
}


void 
System_impl::remove_entity (const char* entity_name)    
       throw (CdmwPlatformMngt::EntityNotFound,
              CORBA::SystemException)
{  
    CDMW_LOG_FUNCTION(Get_Logger(), "entity_name=" << entity_name);
    try {  
        accessor()->remove_entity(lookupSystemTargetPath(),
                         entity_name);
    } 
    catch (const CdmwPlatformMngt::ManagedElementNotFound & ex) {
        CORBA::OBJECT_NOT_EXIST e(OrbSupport::OBJECT_NOT_EXIST, 
                                  CORBA::COMPLETED_NO);
        CDMW_LOG_FUNCTION_EXCEPTION(e);
        throw e;
    }
    catch (const CORBA::Exception & ex) {
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw;
    }
    catch (...) {
        CORBA::INTERNAL ex(Cdmw::OrbSupport::INTERNAL, 
                           CORBA::COMPLETED_MAYBE);
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw ex;
    }
}



void 
System_impl::remove_all_service_defs()
    throw (CORBA::SystemException)
{ 
    CDMW_LOG_FUNCTION(Get_Logger(), "");
    try {  
        accessor()->remove_all_service_defs(lookupSystemTargetPath());
    } 
    catch (const CdmwPlatformMngt::ManagedElementNotFound & ex) {
        CORBA::OBJECT_NOT_EXIST e(OrbSupport::OBJECT_NOT_EXIST, 
                                  CORBA::COMPLETED_NO);
        CDMW_LOG_FUNCTION_EXCEPTION(e);
        throw e;
    }
    catch (const CORBA::Exception & ex) {
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw;
    }
    catch (...) {
        CORBA::INTERNAL ex(Cdmw::OrbSupport::INTERNAL, 
                           CORBA::COMPLETED_MAYBE);
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw ex;
    }
}


CdmwPlatformMngtService::ServiceProviderID* 
System_impl::get_service_def (
                        const char* service_name)
       throw (CdmwPlatformMngtService::ServiceNotFound,
              CORBA::SystemException)
{
    CDMW_LOG_FUNCTION(Get_Logger(), "service_name=" << service_name);
    try {  
        return accessor()->get_service_def (lookupSystemTargetPath(), service_name);
    } 
    catch (const CdmwPlatformMngt::ManagedElementNotFound & ex) {
        CORBA::OBJECT_NOT_EXIST e(OrbSupport::OBJECT_NOT_EXIST, 
                                  CORBA::COMPLETED_NO);
        CDMW_LOG_FUNCTION_EXCEPTION(e);
        throw e;
    }
    catch (const CORBA::Exception & ex) {
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw;
    }
    catch (...) {
        CORBA::INTERNAL ex(Cdmw::OrbSupport::INTERNAL, 
                           CORBA::COMPLETED_MAYBE);
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw ex;
    }
}

void System_impl::add_service_def (const CdmwPlatformMngtService::ServiceDef& service_def)  
       throw (CdmwPlatformMngtService::ServiceAlreadyExists,
              CORBA::SystemException)
{
    CDMW_LOG_FUNCTION(Get_Logger(), "service_name=" << service_def.service_name);
    try {  
        accessor()->add_service_def(lookupSystemTargetPath(), service_def);
    } 
    catch (const CdmwPlatformMngt::ManagedElementNotFound & ex) {
        CORBA::OBJECT_NOT_EXIST e(OrbSupport::OBJECT_NOT_EXIST, 
                                  CORBA::COMPLETED_NO);
        CDMW_LOG_FUNCTION_EXCEPTION(e);
        throw e;
    }
    catch (const CORBA::Exception & ex) {
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw;
    }
    catch (...) {
        CORBA::INTERNAL ex(Cdmw::OrbSupport::INTERNAL, 
                           CORBA::COMPLETED_MAYBE);
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw ex;
    }
}

void System_impl::remove_service_def (const char* service_name)  
       throw (CdmwPlatformMngtService::ServiceNotFound,
              CORBA::SystemException)
{
    CDMW_LOG_FUNCTION(Get_Logger(), "service_name=" << service_name);
    try {  
       accessor()->remove_service_def(lookupSystemTargetPath(), service_name);
    } 
    catch (const CdmwPlatformMngt::ManagedElementNotFound & ex) {
        CORBA::OBJECT_NOT_EXIST e(OrbSupport::OBJECT_NOT_EXIST, 
                                  CORBA::COMPLETED_NO);
        CDMW_LOG_FUNCTION_EXCEPTION(e);
        throw e;
    }
    catch (const CORBA::Exception & ex) {
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw;
    }
    catch (...) {
        CORBA::INTERNAL ex(Cdmw::OrbSupport::INTERNAL, 
                           CORBA::COMPLETED_MAYBE);
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw ex;
    }
}




CORBA::ULong System_impl::get_number_of_hosts()
throw( CORBA::SystemException )
{
    CDMW_LOG_FUNCTION(Get_Logger(), "");
    try {  
        return accessor()->get_number_of_hosts(lookupSystemTargetPath());
    } 
    catch (const CdmwPlatformMngt::ManagedElementNotFound & ex) {
        CORBA::OBJECT_NOT_EXIST e(OrbSupport::OBJECT_NOT_EXIST, 
                                  CORBA::COMPLETED_NO);
        CDMW_LOG_FUNCTION_EXCEPTION(e);
        throw e;
    }
    catch (const CORBA::Exception & ex) {
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw;
    }
    catch (...) {
        CORBA::INTERNAL ex(Cdmw::OrbSupport::INTERNAL, 
                           CORBA::COMPLETED_MAYBE);
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw ex;
    }
}



void System_impl::get_all_host_names (CORBA::ULong how_many,
                                      CdmwPlatformMngt::ElementNames_out host_names,
                                      CdmwPlatformMngt::ElementNamesIterator_out rest)
throw( CORBA::SystemException )
{
    CDMW_LOG_FUNCTION(Get_Logger(), "");
    try {  
        accessor()->get_all_host_names (lookupSystemTargetPath(),
                               how_many,
                               host_names,
                               rest);
    } 
    catch (const CdmwPlatformMngt::ManagedElementNotFound & ex) {
        CORBA::OBJECT_NOT_EXIST e(OrbSupport::OBJECT_NOT_EXIST, 
                                  CORBA::COMPLETED_NO);
        CDMW_LOG_FUNCTION_EXCEPTION(e);
        throw e;
    }
    catch (const CORBA::Exception & ex) {
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw;
    }
    catch (...) {
        CORBA::INTERNAL ex(Cdmw::OrbSupport::INTERNAL, 
                           CORBA::COMPLETED_MAYBE);
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw ex;
    }
}



CdmwPlatformMngt::Host_ptr System_impl::get_host (const char* host_name)
        throw (CdmwPlatformMngt::HostNotFound,
               CORBA::SystemException)
{
    CDMW_LOG_FUNCTION(Get_Logger(), "host_name=" << host_name);
    try {  
        CORBA::String_var uid = 
            accessor()->get_host (lookupSystemTargetPath(),
                         host_name);
        
        return m_proxy_collection->createHostRef(uid);
    } 
    catch (const CdmwPlatformMngt::ManagedElementNotFound & ex) {
        CORBA::OBJECT_NOT_EXIST e(OrbSupport::OBJECT_NOT_EXIST, 
                                  CORBA::COMPLETED_NO);
        CDMW_LOG_FUNCTION_EXCEPTION(e);
        throw e;
    }
    catch (const CORBA::Exception & ex) {
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw;
    }
    catch (...) {
        CORBA::INTERNAL ex(Cdmw::OrbSupport::INTERNAL, 
                           CORBA::COMPLETED_MAYBE);
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw ex;
    }
}



CORBA::Boolean System_impl::get_hosts (const CdmwPlatformMngt::ElementNames& host_names,
                                       CdmwPlatformMngt::Hosts_out hosts)
throw( CORBA::SystemException )
{
    CDMW_LOG_FUNCTION(Get_Logger(), "");
    try {  
        CdmwPlatformMngt::ElementPaths_var paths;
    
        CORBA::Boolean succeed =
            accessor()->get_hosts(lookupSystemTargetPath(), 
                         host_names, 
                         paths.out());
        
        if (succeed) {
            hosts = m_proxy_collection->createHostsFromSeq(paths.ptr());
        }
        
        return succeed;
    } 
    catch (const CdmwPlatformMngt::ManagedElementNotFound & ex) {
        CORBA::OBJECT_NOT_EXIST e(OrbSupport::OBJECT_NOT_EXIST, 
                                  CORBA::COMPLETED_NO);
        CDMW_LOG_FUNCTION_EXCEPTION(e);
        throw e;
    }
    catch (const CORBA::Exception & ex) {
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw;
    }
    catch (...) {
        CORBA::INTERNAL ex(Cdmw::OrbSupport::INTERNAL, 
                           CORBA::COMPLETED_MAYBE);
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw ex;
    }
}



void System_impl::get_all_hosts (CORBA::ULong how_many,
                                 CdmwPlatformMngt::Hosts_out hosts,
                                 CdmwPlatformMngt::HostsIterator_out rest)
throw( CORBA::SystemException )
{
    CDMW_LOG_FUNCTION(Get_Logger(), "");
    try {  
        CdmwPlatformMngt::ElementPaths_var paths;
        CdmwPlatformMngt::ElementPathsIterator_var iter;
    
        accessor()->get_all_hosts(lookupSystemTargetPath(), 
                      how_many,
                      paths.out(),
                      iter.out());
        
        hosts = m_proxy_collection->createHostsFromSeq(paths.ptr());
        rest  = m_proxy_collection->createHostsIteratorRef(iter.in());
    } 
    catch (const CdmwPlatformMngt::ManagedElementNotFound & ex) {
        CORBA::OBJECT_NOT_EXIST e(OrbSupport::OBJECT_NOT_EXIST, 
                                  CORBA::COMPLETED_NO);
        CDMW_LOG_FUNCTION_EXCEPTION(e);
        throw e;
    }
    catch (const CORBA::Exception & ex) {
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw;
    }
    catch (...) {
        CORBA::INTERNAL ex(Cdmw::OrbSupport::INTERNAL, 
                           CORBA::COMPLETED_MAYBE);
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw ex;
    }
}


char* System_impl::get_logger_default_filters() throw (CORBA::SystemException)
{
    try{
        return accessor()->get_logger_default_filters();
     }
     catch (const CORBA::SystemException & ex) {
         throw;
    }
    catch (...) {
        throw CORBA::INTERNAL(Cdmw::OrbSupport::INTERNAL, CORBA::COMPLETED_MAYBE);
    }   
   
}

CdmwPlatformMngt::ManagedElement* System_impl::get_element(const char* element_path)
throw (CORBA::SystemException)
{
    try
    {
        //Verifing the element path. The returned value is not used as output parameter 
        //because is a reference valid in the context of the primary supervision(not fault tolerant)
        CdmwPlatformMngt::ManagedElement_var verify_managed_element = accessor()->get_element(element_path);
    }
    catch (const CORBA::SystemException & ex) {
        throw;
    }
    catch (...) {
        throw CORBA::INTERNAL(Cdmw::OrbSupport::INTERNAL, CORBA::COMPLETED_MAYBE); 
    }
    //output parameters
    std::string system_name, 
    application_name, 
    host_name,process_name,
    component_name,
    entity_name,
    attribute_name, 
    ftgroup_name,lbgroup_name;

    CdmwPlatformMngt::ElementType element_type;
    //parsing element path
    try
    {
        element_type = 
            Cdmw::PlatformMngt::ManagedElementKey::Get_element_names(element_path,
                    system_name,
                    application_name, 
                    host_name,
                    process_name, 
                    component_name,
                    entity_name,
                    attribute_name,
                    ftgroup_name,
                    lbgroup_name);
    }
    catch (const CdmwPlatformMngt::ElementPathInvalid&)
    {
        throw CORBA::BAD_PARAM (OrbSupport::BAD_PARAM, CORBA::COMPLETED_NO);
    }

    CdmwPlatformMngt::ManagedElement_var managed_element = 
        CdmwPlatformMngt::ManagedElement::_nil();  

    //check element type creating a specific and fault tolerant referent to the element required.
    switch( element_type)
    {
    case (CdmwPlatformMngt::SYSTEM_TYPE) :
    {
        try
        {
            managed_element = m_proxy_collection->createSystemRef();
        }
        catch (...)
        {
            throw CORBA::INTERNAL(OrbSupport::INTERNAL, CORBA::COMPLETED_NO);
        }
    }    
    break;

    case (CdmwPlatformMngt::HOST_TYPE) :
    {

        try 
        {  
            managed_element = m_proxy_collection->createHostRef(element_path);
        } 
        catch (...) {
            throw CORBA::INTERNAL(OrbSupport::INTERNAL, CORBA::COMPLETED_NO);
        }       
    }
    break;

    case (CdmwPlatformMngt::APPLICATION_TYPE) :
    {
        try 
        {  
            managed_element= m_proxy_collection->createApplicationRef(element_path);
        } 
        catch (...) {
            throw CORBA::INTERNAL(OrbSupport::INTERNAL, CORBA::COMPLETED_NO);
        }       


    }
    break;

    case (CdmwPlatformMngt::FT_GROUP_TYPE) :
    {  
        try 
        {  
            managed_element= m_proxy_collection->createFTGroupRef(element_path);
        } 
        catch (...) {
            throw CORBA::INTERNAL(OrbSupport::INTERNAL, CORBA::COMPLETED_NO);
        } 


    }
    break;

    case (CdmwPlatformMngt::LB_GROUP_TYPE) :
    {  
        try 
        {  
            managed_element= m_proxy_collection->createLBGroupRef(element_path);
        } 
        catch (...) {
            throw CORBA::INTERNAL(OrbSupport::INTERNAL, CORBA::COMPLETED_NO);
        } 

    }
    break;

    case (CdmwPlatformMngt::PROCESS_TYPE) :
    {
        try 
        {  
            managed_element= m_proxy_collection->createProcessRef(element_path);
        } 
        catch (...) {
            throw CORBA::INTERNAL(OrbSupport::INTERNAL, CORBA::COMPLETED_NO);
        }    
    }
    break;          

    case (CdmwPlatformMngt::COMPONENT_TYPE) :
    {
        try 
        {  
            managed_element =m_proxy_collection->createCcmComponentRef(element_path);
        } 
        catch (...) {
            throw CORBA::INTERNAL(OrbSupport::INTERNAL, CORBA::COMPLETED_NO);
        }     
    }
    break;

    case (CdmwPlatformMngt::ENTITY_TYPE) :
    {
        try 
        {  
            managed_element =m_proxy_collection->createEntityRef(element_path);
        } 
        catch (...) {
            throw CORBA::INTERNAL(OrbSupport::INTERNAL, CORBA::COMPLETED_NO);
        }  
    }
    break;

    default :            
        throw CORBA::BAD_PARAM (OrbSupport::BAD_PARAM, CORBA::COMPLETED_NO);
        break;

    }  

    return managed_element._retn();

}

void System_impl::get_all_applications(CORBA::ULong, CdmwPlatformMngt::Applications_out,
                                       CdmwPlatformMngt::ApplicationsIterator_out)
    throw (CORBA::SystemException)
{
    throw CORBA::NO_IMPLEMENT (OrbSupport::NO_IMPLEMENT, CORBA::COMPLETED_NO);
}
    
void System_impl::stop_system(CORBA::Boolean emergency)
throw (CORBA::SystemException)
{

    CDMW_LOG_FUNCTION(Get_Logger(), "");
    //get issuer value

    std::string command_issuer (m_current->get_issuer());

    try 
    {
        accessor()->stop_system(emergency, command_issuer.c_str());
    }
    catch (const CORBA::Exception & ex) {
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw;
    }
    catch (...) 
    {
        CORBA::INTERNAL ex(Cdmw::OrbSupport::INTERNAL, 
                CORBA::COMPLETED_MAYBE);
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw ex;
    }
}
     
    
    
} // End namespace Proxy
} // End namespace PlatformMngt
} // End namespace Cdmw

