/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

#include <memory> // for std::auto_ptr
#include <string>
#include <vector>


#include "platformftsystem/CdmwInterface.hpp"
#include "platformsystemlibrary/InitUtils.hpp"
#include "platformsystemlibrary/CdmwInitConfiguration.hpp"
#include "SystemMngt/platformelementlibrary/DataStoreDefinition.hpp"
#include "SystemMngt/platformlibrary/DataStoreBaseDefinition.hpp"
#include "SystemMngt/platformlibrary/ClientIDGenerator.hpp"
#include "SystemMngt/platformlibrary/Configuration.hpp"

#include <Foundation/common/Assert.hpp>
#include <Foundation/common/Exception.hpp>
#include <Foundation/orbsupport/OrbSupport.hpp>
#include <Foundation/orbsupport/ExceptionMinorCodes.hpp>
#include <Foundation/ossupport/OS.hpp>
#include <Foundation/osthreads/Mutex.hpp>
#include <Foundation/osthreads/Condition.hpp>
#include <Foundation/commonsvcs/naming/NamingInterface.hpp>



#include <FaultTolerance/ftlocationmanager/PrimaryBackupGroupRepository_impl.hpp>
#include <FaultTolerance/ftlocationmanager/PrimaryBackupAdmin_impl.hpp>
#include <FaultTolerance/idllib/CdmwFTReplicationManager.stub.hpp>
#include <FaultTolerance/idllib/CdmwFTManager.stub.hpp>
#include <FaultTolerance/ftcommon/FTConfiguration.hpp>
#include <FaultTolerance/ftstatemanager/DataStore.hpp>
#include <FaultTolerance/ftstatemanager/DataStoreGroup_impl.hpp>
      
#include <FaultTolerance/ftlocationmanager/StatefullPrimaryBackupAdmin_impl.hpp>
#include <FaultTolerance/ftlocationmanager/LocationDataStoreTimeoutHandler_impl.hpp>

#include "SystemMngt/idllib/CdmwPlatformMngtDataStore.stub.hpp"
#include "SystemMngt/idllib/CdmwPlatformMngtDaemon.stub.hpp"

namespace {
   
    const int MAX_RETRY_ON_FTMGR = 60;
    const int SLEEP_RETRY_ON_FTMGR_MS = 1000;
    const int NB_SYSMNGT_DATASTORE = 15;
    const int SLEEP_EXIT_SUPERVISION=5000;

} // end of anonymous namespace



namespace Cdmw {

namespace PlatformMngt {

void retry_on_ft(int& nbRetries,std::string& msg,Cdmw::Logging::Logger * logger)
throw (CORBA::SystemException)
{
    if ( ++nbRetries > MAX_RETRY_ON_FTMGR) 
    {
        std::string error = msg + " unreachable, CdmwPlatformSupervision is exiting in few seconds";
        CDMW_ERROR(logger, error.c_str() ); 
        /*CDMW_THROW2(Cdmw::OrbSupport::CORBASystemExceptionWrapperT<CORBA::INTERNAL>,
                CORBA::INTERNAL(Cdmw::OrbSupport::INTERNAL, CORBA::COMPLETED_NO),
                error.c_str() );*/
        Cdmw::OsSupport::OS::sleep(SLEEP_EXIT_SUPERVISION);
        exit(1);
    }
    CDMW_WARN(logger, msg << " not yet activated "); 
    CDMW_INFO(logger, "Retry to contact "<< msg <<" ("<< (MAX_RETRY_ON_FTMGR-nbRetries) <<" remaining attempts) "); 
    Cdmw::OsSupport::OS::sleep(SLEEP_RETRY_ON_FTMGR_MS);
}

void CDMW_init(CORBA::ORB_ptr orb, 
               int & argc, 
               char** argv, 
               const char* supervision_process_name,
               const char* fault_manager_corbaloc)
   throw (CORBA::SystemException)
{

     CDMW_ASSERT(!CORBA::is_nil(orb));

     Cdmw::Logging::Logger * logger =
         Cdmw::Logging::Logger::Get_Logger(CdmwPlatformMngtBase::PFMMNG);
     CDMW_ASSERT(logger != 0);

     try {
         // Get the XML init filename. This may raise CORBA::BAD_PARAM, CORBA::NO_MEMORY,
         // or CORBA::INTERNAL    
         std::string xml_file = InitUtils::Get_xml_initialisation_file(argc, argv);
        
         // Get the XML documents validation option. This may raise CORBA::NO_MEMORY,
         // or CORBA::INTERNAL
         bool validate_xml = InitUtils::Get_xml_validation_option(argc, argv);
        
         // Parsing xml
         std::auto_ptr<CdmwInitConfiguration> xml_data(0);
         try {
            CDMW_INFO(logger, "Parsing xml initialisation file: " << xml_file);
            std::auto_ptr<CdmwInitConfiguration> init
               (CdmwInitConfiguration::ReadConfiguration(xml_file,
                                                         validate_xml));
            
            xml_data = init;
         } catch (const Cdmw::OutOfMemoryException &) {
            CDMW_THROW1( Cdmw::OrbSupport::CORBASystemExceptionWrapperT<CORBA::NO_MEMORY>,
                          CORBA::NO_MEMORY(Cdmw::OrbSupport::NO_MEMORY, 
                                           CORBA::COMPLETED_NO) );
         } catch (const ConfigurationException & ex) {
            CDMW_THROW2( Cdmw::OrbSupport::CORBASystemExceptionWrapperT<CORBA::BAD_PARAM>,
                          CORBA::BAD_PARAM(Cdmw::OrbSupport::BAD_PARAMInvalidXMLInitData,
                                           CORBA::COMPLETED_NO),
                          ex.what());
         } catch(const Cdmw::InternalErrorException & ex){
            CDMW_THROW2( Cdmw::OrbSupport::CORBASystemExceptionWrapperT<CORBA::INTERNAL>,
                          CORBA::INTERNAL(Cdmw::OrbSupport::INTERNALXMLLibraryError,
                                          CORBA::COMPLETED_NO),
                          ex.what() );
         }

         // *****************************************************************
         // *****************  CDMW root POA and servant manager ***********
         // *****************************************************************
         CDMW_DEBUG(logger, "Create CDMW root POA");
         //
         // Create the CDMW root POA. It has its own POA Manager
         // May raise a CORBA::SystemException.
         //
        
     CORBA::Object_var objinit = orb -> resolve_initial_references("RootPOA");
         PortableGroup::GOA_var CDMW_rootGOA = PortableGroup::GOA::_narrow(objinit.in());
         

         PortableServer::POAManager_var poaManager = CDMW_rootGOA -> the_POAManager();
         
         //
         // Initialize the server so that incoming requests are
         // handled from this point on
         //
         // May raise PortableServer::AdapterInactive or CORBA::SystemException
         CDMW_DEBUG(logger, "Activate POAManager");
         try {
            poaManager -> activate(); // TODO: maybe move it?
         } catch (const PortableServer::POAManager::AdapterInactive & ) {
            CDMW_THROW1( Cdmw::OrbSupport::CORBASystemExceptionWrapperT<CORBA::INTERNAL>,
                          CORBA::INTERNAL(Cdmw::OrbSupport::INTERNALORBAdapterInactive,
                                          CORBA::COMPLETED_NO) );
         } catch (const CORBA::SystemException & ex ) {
            CDMW_THROW2( Cdmw::OrbSupport::CORBASystemExceptionWrapperT<CORBA::SystemException>,
                          ex, "Could not activate the CDMW POA Manager!" );
         }


         // *****************************************************************
         // *************  setup FaultTolerance Policies   ******************
         // *****************************************************************
         CDMW_DEBUG(logger, "Setup FaultTolerance Policies");

         Cdmw::OrbSupport::OrbSupport::set_endpoint_selector_factory(::Cdmw::FT::FTConfiguration::Get_FT_endpoint_selector_factory());

         size_t duration_time;
         std::string client_id_str;
         std::string hostname = Cdmw::OsSupport::OS::get_hostname();
            
         try {
             std::string res;
             try {
                 res = xml_data->getServiceAttributeValue("fault-tolerance", "request-duration-policy"); 
                 duration_time = atoi(res.c_str());
             } catch (const Cdmw::BadParameterException& ex) {
                 CDMW_THROW2( Cdmw::OrbSupport::CORBASystemExceptionWrapperT<CORBA::BAD_PARAM>,
                               CORBA::BAD_PARAM(Cdmw::OrbSupport::BAD_PARAMInvalidXMLInitData,
                                                CORBA::COMPLETED_NO),
                               ex.what() );
             }

             try  {
                 //client_id_str = xml_data->getServiceAttributeValue("fault-tolerance", "client-id");
                 client_id_str = 
                    ClientIDGenerator::generate_supervision_client_id(hostname.c_str());
                     Cdmw::FT::FTConfiguration::Set_client_id(orb, 
                                                          client_id_str.c_str());
             } catch (const Cdmw::BadParameterException& ) {
                 // do nothing : parameter is optional
             }
         } catch (const XMLErrorException & ex) {
             CDMW_THROW2( Cdmw::OrbSupport::CORBASystemExceptionWrapperT<CORBA::BAD_PARAM>,
                          CORBA::BAD_PARAM(Cdmw::OrbSupport::BAD_PARAMInvalidXMLInitData,
                                           CORBA::COMPLETED_NO),
                          ex.what() );
         } catch (const OutOfMemoryException & ex) {
             CDMW_THROW2( Cdmw::OrbSupport::CORBASystemExceptionWrapperT<CORBA::NO_MEMORY>,
                          CORBA::NO_MEMORY(Cdmw::OrbSupport::NO_MEMORY,
                                           CORBA::COMPLETED_NO),
                          ex.what() );
         } 

         CDMW_DEBUG(logger, "   duration time: " << duration_time);
         CDMW_DEBUG(logger, "   ClientID: " << client_id_str);
         
         CORBA::Object_var obj;
         obj = orb->resolve_initial_references("ORBPolicyManager");
        
         CORBA::PolicyManager_var policy_manager 
            = CORBA::PolicyManager::_narrow(obj.in());
        

      // Create and apply an ORB-wide Routed policy
         CORBA::Any any;
         TimeBase::TimeT duration = duration_time;
         any <<= duration;
         CORBA::PolicyList policies(1);
         policies.length(1);
         policies[0] =
            orb->create_policy(::FT::REQUEST_DURATION_POLICY, any);

         CDMW_DEBUG(logger, "Set ORB policies");
         policy_manager->set_policy_overrides(policies, CORBA::ADD_OVERRIDE);
         
         policies[0]->destroy();
      

         // *****************************************************************
         // ***********  setup FaultTolerance ReplicationManager  ***********
         // *****************************************************************

         // Get the reference of the Replication Manager and register it in the ORB
         
         CdmwFT::ReplicationManager_var replication_manager = CdmwFT::ReplicationManager::_nil();
         
         try
         {
            
             // if corbaloc has been defined
             if (strcmp(fault_manager_corbaloc, "") != 0)
             {    
                 bool ftmgr_not_active = true;
                 int nbRetries = 0;
                 std::string required_elem= "FTManager";
                 CORBA::Object_var fault_manager_obj = CORBA::Object::_nil();
                 CdmwFT::FTManager_var fault_manager = CdmwFT::FTManager::_nil();;
                 CDMW_INFO(logger, "Get FTManager reference from corbaloc: " << fault_manager_corbaloc);
                 do 
                 {
                     try
                     {
                         // get the FT manager
                         fault_manager_obj = orb->string_to_object(fault_manager_corbaloc);
                         if (CORBA::is_nil (fault_manager_obj.in()))
                         {
                             retry_on_ft(nbRetries,required_elem,logger);
                         }else
                         {
                             fault_manager = ::CdmwFT::FTManager::_narrow(fault_manager_obj.in());
                             ftmgr_not_active = false;
                         }
                     }
                     catch (const CdmwFT::FTManager::NoAlreadyActivated&) 
                     {
                         retry_on_ft(nbRetries,required_elem,logger);
                     }
                     catch (const CORBA::SystemException & ex) 
                     { 
                         retry_on_ft(nbRetries,required_elem,logger);
                     }

                 }while (ftmgr_not_active);
                ftmgr_not_active = true;
                nbRetries = 0;
               required_elem= "Replication Manager";
                // get the replication manager from the FTManager
                do 
                 {
                     try 
                     {
                        CDMW_DEBUG(logger, "Get ReplicationManager from FTManager");
                         CORBA::Object_var rm_obj = fault_manager->get_replication_manager();
                         if (CORBA::is_nil (rm_obj.in()))
                         {
                             retry_on_ft(nbRetries,required_elem,logger);
                         }
                         else
                         {
                             replication_manager = CdmwFT::ReplicationManager::_narrow(rm_obj.in());
                             ftmgr_not_active = false;
                         }
                     } 
                     catch (const CdmwFT::FTManager::NoAlreadyActivated&) 
                     {
                         retry_on_ft(nbRetries,required_elem,logger);
                     }
                     catch (const CORBA::Exception & ex) 
                     {
                         retry_on_ft(nbRetries,required_elem,logger);
                     }
   
                 } while (ftmgr_not_active);
             }
             // else if the corbaloc is not defined 
             else
             {
                 CDMW_INFO(logger, "Get FTManager reference from local Daemon");
                 CdmwPlatformMngt::Daemon_var daemon = CdmwPlatformMngt::Daemon::_nil();
                 try
                 {
                     // get the daemon reference
                     daemon = 
                         Configuration::Get_daemon ("localhost", 
                                 Cdmw::PlatformMngt::Configuration::Get_daemon_port());
                 } 
                 catch (const CORBA::SystemException& ex)
                 {
                     CDMW_ERROR(logger, "Exception while contacting local Daemon:\n   " 
                             << Cdmw::OrbSupport::OrbSupport::exception_to_string(ex));
                     CDMW_THROW2 (Cdmw::OrbSupport::CORBASystemExceptionWrapperT<CORBA::SystemException>,
                             CORBA::INTERNAL(Cdmw::OrbSupport::INTERNAL, CORBA::COMPLETED_NO),
                             "Local Daemon not reachable.");
                 }
                 catch (const CORBA::Exception& ex)
                 {
                     CDMW_ERROR(logger, "Exception while contacting local Daemon:\n   " 
                             << ex);
                     CDMW_THROW2 (Cdmw::OrbSupport::CORBASystemExceptionWrapperT<CORBA::SystemException>,
                             CORBA::INTERNAL(Cdmw::OrbSupport::INTERNAL, CORBA::COMPLETED_NO),
                             "Local Daemon not reachable.");
                 }
                 bool ftmgr_not_active = true;
                 int nbRetries = 0;
                 std::string required_elem= "FTManager";
                 do 
                 {
                     try
                     {
                         CDMW_DEBUG(logger, "Discover global service ReplicationManager");
                         CORBA::Object_var rm_obj = daemon->discover_global_service ("ReplicationManager");
                         replication_manager = CdmwFT::ReplicationManager::_narrow(rm_obj.in());
                         
                         if (CORBA::is_nil (rm_obj.in()))
                         {
                             retry_on_ft(nbRetries,required_elem,logger);
                         }
                         else
                         {
                             ftmgr_not_active = false;
                         }
                     }
                     catch (const CdmwFT::FTManager::NoAlreadyActivated& ex) 
                     {
                         retry_on_ft(nbRetries,required_elem,logger);
                     }

                     catch (const CORBA::SystemException& ex)
                     {
                         retry_on_ft(nbRetries,required_elem,logger);
                     }
                    
                 }while (ftmgr_not_active);
                 
             }
             
            
             
             orb->register_initial_reference("ReplicationManager",replication_manager.in());      

         }
         catch (const CORBA::SystemException& ex)
         {
             CDMW_THROW2 (Cdmw::OrbSupport::CORBASystemExceptionWrapperT<CORBA::SystemException>,
                          ex,
                          "Cannot register ReplicationManager.");
         }
         catch (const CORBA::ORB::InvalidName& )
         {
         CDMW_THROW2( Cdmw::OrbSupport::CORBASystemExceptionWrapperT<CORBA::INTERNAL>,
                      CORBA::INTERNAL(Cdmw::OrbSupport::INTERNAL, 
                                      CORBA::COMPLETED_NO),
                      "InvalidName => Cannot register ReplicationManager.");
         }
         
         
         // *****************************************************************
         // *************  setup FaultTolerance LocationManager  ************
         // *****************************************************************
         
         // Location information shall be obtain from supervision
         ::FT::Location loc;
         loc.length(2);

         loc[0].id 
            = CORBA::string_dup(Cdmw::OsSupport::OS::get_hostname().c_str());
         loc[0].kind = "hostname";
         loc[1].id = supervision_process_name;
         loc[1].kind = "processname";
         
         CDMW_DEBUG(logger, "Setup FaultTolerance LocationManager for location: "
                    << Cdmw::CommonSvcs::Naming::NamingInterface::to_string(loc));
            
         // create a reference for the PrimaryBackupGroupRepository
         ::Cdmw::FT::Location::PrimaryBackupGroupRepository_impl * 
            primary_backup_repository =
                 ::Cdmw::FT::Location::PrimaryBackupGroupRepository_impl::Get_instance();

         // get the fault notifier reference
         CDMW_DEBUG(logger, "Get the fault notifier reference");
         ::FT::FaultNotifier_var fault_notifier = 
            replication_manager->get_fault_notifier();

         // create the DataStoreTimeoutHandler
         Cdmw::FT::LocationDataStoreTimeoutHandler* 
            locationDataStoreTimeoutHandler = 
               new Cdmw::FT::LocationDataStoreTimeoutHandler(
                     fault_notifier.in());

         // Register the timeoutHandler into the DataStoreBase
         CDMW_DEBUG(logger, "Register the timeoutHandler into the DataStoreBase");
         Cdmw::FT::DataStoreBase::Register_Timeout_Handler(
               locationDataStoreTimeoutHandler);


         // *****************************************************************
         // *************  setup DataStore                       ************
         // *****************************************************************
         CDMW_INFO(logger, "Setup DataStores");
         
         // FaultTolerance use datastore
         CORBA::ULong datastore_id;
         std::string multicast_port;
         std::string multicast_domain;
         std::string multicast_IP_address;
         int max_transaction_in_progress;
         int max_transaction_done;
         OsSupport::OS::Timeval cohort_timeout;
         OsSupport::OS::Timeval coordinator_timeout;
         OsSupport::OS::Timeval freeze_timeout;
         long number_of_scheduler_thread;
         long chunk_size;
         long freeze_size;

      
         
         // Get dataStore parameters from xml file
         try
         {
             InitUtils::Get_parameters(*(xml_data.get()),
                                       multicast_port,
                                       multicast_domain,
                                       multicast_IP_address,
                                       max_transaction_in_progress,
                                       max_transaction_done,
                                       cohort_timeout,
                                       coordinator_timeout,
                                       freeze_timeout,
                                       number_of_scheduler_thread,
                                       chunk_size,
                                       freeze_size);
         } catch (const Cdmw::BadParameterException& ex) {
              CDMW_THROW2( Cdmw::OrbSupport::CORBASystemExceptionWrapperT<CORBA::BAD_PARAM>,
                           CORBA::BAD_PARAM(Cdmw::OrbSupport::BAD_PARAMInvalidXMLInitData,
                           CORBA::COMPLETED_NO),ex.what() );
         }
          
         // create the dataStore context
         CDMW_DEBUG(logger, "Create the dataStore context");
         Cdmw::FT::DataStoreContext* context =
             new Cdmw::FT::DataStoreContext
             (orb,
              CDMW_rootGOA.in(),
              max_transaction_in_progress,
              max_transaction_done,
              cohort_timeout,
              coordinator_timeout,
              number_of_scheduler_thread,
              chunk_size,
              freeze_size,
              freeze_timeout);
         
         // create the local datastore infos     
         CORBA::ULong ds_inc = 0;
         CdmwFT::StateTransfer::LocalDataStoreInfos_var localDataStoreInfos = 
            new CdmwFT::StateTransfer::LocalDataStoreInfos();
         localDataStoreInfos->length(1);
      
      
         // create local datastore info with datastore group
         CdmwFT::StateTransfer::LocalDataStoreInfo localDataStoreInfo;
         
         CdmwFT::StateTransfer::DataStoreGroup_var
              managedelementdatastore_group_ref = 
                (new Cdmw::FT::DataStoreGroup_impl(
                        CDMW_rootGOA.in(),
                        *context))->_this();
    
         localDataStoreInfo.dsids.length(NB_SYSMNGT_DATASTORE);
         localDataStoreInfo.local_data_store = 
                CdmwFT::StateTransfer::DataStoreGroup::_duplicate(
                                      managedelementdatastore_group_ref.in());
          
         localDataStoreInfo.coordinator = context->get_tpc_coordinator();
         localDataStoreInfo.cohort = context->get_tpc_cohort();
    
         (*localDataStoreInfos)[0] = localDataStoreInfo;
          
          
          
          
         // define the DataStores used by System Management
         // The order of creation is significant for backup starting and must not be changed
         // (due to record dependancy)
         // ManagedElement, System, Host, Application, Process, Entity,
         // SystemObserverRegistration, EventHandler, EventNotifier,
         // ServiceDef, Runset
         
         //
         // define the datastore responsible to store ManagedElement
         //

         CDMW_DEBUG(logger, "Create ManagedElement Datastore");
         typedef Cdmw::FT::DataStore<std::string,
                                     CdmwPlatformMngt::ManagedElementRecord> 
                 ManagedElementFTDataStore;

         // get ManagedElement datastore parameters from xml
         try
         {

             InitUtils::Get_datastore_id_parameter
                 (*(xml_data.get()),
                  "ManagedElementDataStore",
                  NB_SYSMNGT_DATASTORE,
                  datastore_id);

         } catch (const Cdmw::BadParameterException& ex) {
              CDMW_THROW2( Cdmw::OrbSupport::CORBASystemExceptionWrapperT<CORBA::BAD_PARAM>,
                           CORBA::BAD_PARAM(Cdmw::OrbSupport::BAD_PARAMInvalidXMLInitData,
                           CORBA::COMPLETED_NO),ex.what());
         }
    
    
         // create data store (registered itself)
         new ManagedElementFTDataStore (datastore_id,
                                        orb,
                                        CDMW_rootGOA.in(),
                                        *context); 
          
         // Store datastore_id in the Configuration class to have access inside
         // the whole SystemMngt CSCI
         PlatformMngt::Configuration::Set_managed_element_datastore_id(datastore_id);

         // store datastore id in local datastore info
         (*localDataStoreInfos)[0].dsids[ds_inc] = datastore_id;
         ds_inc++;
          


         //
         // define the datastore responsible to store System
         //
         CDMW_DEBUG(logger, "Create System Datastore");
         typedef Cdmw::FT::DataStore<std::string,
                                     CdmwPlatformMngt::SystemRecord> 
                 SystemFTDataStore;
      
         // get System datastore parameters from xml
         try
         {
       InitUtils::Get_datastore_id_parameter
                 (*(xml_data.get()),
                  "SystemDataStore",
                  NB_SYSMNGT_DATASTORE,
                  datastore_id);
         } catch (const Cdmw::BadParameterException& ex) {
              CDMW_THROW2( Cdmw::OrbSupport::CORBASystemExceptionWrapperT<CORBA::BAD_PARAM>,
                           CORBA::BAD_PARAM(Cdmw::OrbSupport::BAD_PARAMInvalidXMLInitData,
                           CORBA::COMPLETED_NO),ex.what() );
         }
    
         // create data store (registered itself)
         new SystemFTDataStore (datastore_id,
                                orb,
                                CDMW_rootGOA.in(),
                                *context); 
    
         // Store datastore_id in the Configuration class to have access inside
         // the whole SystemMngt CSCI
         PlatformMngt::Configuration::Set_system_datastore_id(datastore_id);

         // store datastore id in local datastore info
         (*localDataStoreInfos)[0].dsids[ds_inc] = datastore_id;
         ds_inc++;
         
         // 
         // define the datastore responsible to store Host
         //
         CDMW_DEBUG(logger, "Create Host Datastore");
         typedef Cdmw::FT::DataStore<std::string,
                                     CdmwPlatformMngt::HostRecord> 
                 HostFTDataStore;
         
         // get Host datastore parameters from xml
         try
         {
       InitUtils::Get_datastore_id_parameter
         (*(xml_data.get()),
          "HostDataStore",
          NB_SYSMNGT_DATASTORE,
          datastore_id);
         } catch (const Cdmw::BadParameterException& ex) {
              CDMW_THROW2( Cdmw::OrbSupport::CORBASystemExceptionWrapperT<CORBA::BAD_PARAM>,
                           CORBA::BAD_PARAM(Cdmw::OrbSupport::BAD_PARAMInvalidXMLInitData,
                           CORBA::COMPLETED_NO),ex.what() );
         }
    
    
         // create data store (registered itself)
         new HostFTDataStore (datastore_id,
                              orb,
                              CDMW_rootGOA.in(),
                              *context); 
    
         // Store datastore_id in the Configuration class to have access inside
         // the whole SystemMngt CSCI
         PlatformMngt::Configuration::Set_host_datastore_id(datastore_id);
    
         // store datastore id in local datastore info
         (*localDataStoreInfos)[0].dsids[ds_inc] = datastore_id;
         ds_inc++;
          


         //
         // define the datastore responsible to store Application
         //
         CDMW_DEBUG(logger, "Create Application Datastore");
         typedef Cdmw::FT::DataStore<std::string,
                            CdmwPlatformMngt::ApplicationRecord> 
                 ApplicationFTDataStore;
                 
         // get Application datastore parameters from xml        
         try
         {
       InitUtils::Get_datastore_id_parameter
         (*(xml_data.get()),
          "ApplicationDataStore",
          NB_SYSMNGT_DATASTORE,
          datastore_id);
         } catch (const Cdmw::BadParameterException& ex) {
              CDMW_THROW2(Cdmw::OrbSupport::CORBASystemExceptionWrapperT<CORBA::BAD_PARAM>,
                          CORBA::BAD_PARAM(Cdmw::OrbSupport::BAD_PARAMInvalidXMLInitData,
                          CORBA::COMPLETED_NO),ex.what() );
         }
    
         // create data store (registered itself)
         new ApplicationFTDataStore (datastore_id,
                                     orb,
                                     CDMW_rootGOA.in(),
                                     *context); 
    
         // Store datastore_id in the Configuration class to have access inside
         // the whole SystemMngt CSCI
         PlatformMngt::Configuration::Set_application_datastore_id(datastore_id);
    
         // store datastore id in local datastore info
         (*localDataStoreInfos)[0].dsids[ds_inc] = datastore_id;
         ds_inc++;


         //
         // define the datastore responsible to store Process
         //
         CDMW_DEBUG(logger, "Create Process Datastore");
         typedef Cdmw::FT::DataStore<std::string,
                                     CdmwPlatformMngt::ProcessRecord> 
                 ProcessFTDataStore;

         // get Process datastore parameters from xml
         try
         {
       InitUtils::Get_datastore_id_parameter
         (*(xml_data.get()),
          "ProcessDataStore",
          NB_SYSMNGT_DATASTORE,
          datastore_id);
         } catch (const Cdmw::BadParameterException& ex) {
              CDMW_THROW2( Cdmw::OrbSupport::CORBASystemExceptionWrapperT<CORBA::BAD_PARAM>,
                           CORBA::BAD_PARAM(Cdmw::OrbSupport::BAD_PARAMInvalidXMLInitData,
                           CORBA::COMPLETED_NO),ex.what() );
         }
    
         // create data store (registered itself)
         new ProcessFTDataStore (datastore_id,
                                 orb,
                                 CDMW_rootGOA.in(),
                                 *context); 
    
         // Store datastore_id in the Configuration class to have access inside
         // the whole SystemMngt CSCI
         PlatformMngt::Configuration::Set_process_datastore_id(datastore_id);
    
         // store datastore id in local datastore info
         (*localDataStoreInfos)[0].dsids[ds_inc] = datastore_id;
         ds_inc++;
          
         //
         // define the datastore responsible to store Entity
         //
         CDMW_DEBUG(logger, "Create Entity Datastore");
         typedef Cdmw::FT::DataStore<std::string,
                                     CdmwPlatformMngt::EntityRecord> 
                 EntityFTDataStore;

         // get Entity datastore parameters from xml
         try
         {
       InitUtils::Get_datastore_id_parameter
         (*(xml_data.get()),
          "EntityDataStore",
          NB_SYSMNGT_DATASTORE,
          datastore_id);
         } catch (const Cdmw::BadParameterException& ex) {
              CDMW_THROW2( Cdmw::OrbSupport::CORBASystemExceptionWrapperT<CORBA::BAD_PARAM>,
                           CORBA::BAD_PARAM(Cdmw::OrbSupport::BAD_PARAMInvalidXMLInitData,
                           CORBA::COMPLETED_NO),ex.what() );
         }
    
    
         // create data store (registered itself)
         new EntityFTDataStore (datastore_id,
                                orb,
                                CDMW_rootGOA.in(),
                                *context); 
                                
         // Store datastore_id in the Configuration class to have access inside
         // the whole SystemMngt CSCI
         PlatformMngt::Configuration::Set_entity_datastore_id(datastore_id);
    
         // store datastore id in local datastore info
         (*localDataStoreInfos)[0].dsids[ds_inc] = datastore_id;
         ds_inc++;
         


         //
         // define the datastore responsible to store SystemObserverRegistration
         //
         CDMW_DEBUG(logger, "Create SystemObserverRegistration Datastore");
         typedef Cdmw::FT::DataStore<std::string,
                                     CdmwPlatformMngt::SystemObserverRegistrationRecord> 
                 SystemObserverRegistrationFTDataStore;
                 
         // get SystemObserverRegistration datastore parameters from xml
         try
         {
       InitUtils::Get_datastore_id_parameter
         (*(xml_data.get()),
          "SystemObserverRegistrationDataStore",
          NB_SYSMNGT_DATASTORE,
          datastore_id);
          } catch (const Cdmw::BadParameterException& ex) {
              CDMW_THROW2( Cdmw::OrbSupport::CORBASystemExceptionWrapperT<CORBA::BAD_PARAM>,
                           CORBA::BAD_PARAM(Cdmw::OrbSupport::BAD_PARAMInvalidXMLInitData,
                           CORBA::COMPLETED_NO),ex.what() );
          }
    
          // create data store (registered itself)
          new SystemObserverRegistrationFTDataStore (datastore_id,
                                                     orb,
                                                     CDMW_rootGOA.in(),
                                                     *context);
    
          // Store datastore_id in the Configuration class to have access inside
          // the whole SystemMngt CSCI
          PlatformMngt::Configuration::Set_system_observer_registration_datastore_id(datastore_id);
    
          // store datastore id in local datastore info
          (*localDataStoreInfos)[0].dsids[ds_inc] = datastore_id;
          ds_inc++;


          //
          // define the datastore responsible to store EventHandler
          //
          CDMW_DEBUG(logger, "Create EventReport Datastore");
          typedef Cdmw::FT::DataStore<CdmwPlatformMngtBase::EventKey,
                                      CdmwPlatformMngt::EventReportRecord>
                  EventReportFTDataStore;

          // get EventHandler datastore parameters from xml
          try
          {
       InitUtils::Get_datastore_id_parameter
         (*(xml_data.get()),
          "EventReportDataStore",
          NB_SYSMNGT_DATASTORE,
          datastore_id);
          } catch (const Cdmw::BadParameterException& ex) {
               CDMW_THROW2( Cdmw::OrbSupport::CORBASystemExceptionWrapperT<CORBA::BAD_PARAM>,
                            CORBA::BAD_PARAM(Cdmw::OrbSupport::BAD_PARAMInvalidXMLInitData,
                            CORBA::COMPLETED_NO),ex.what() );
          }
    
          // create data store (registered itself)
          new EventReportFTDataStore (datastore_id,
                                      orb,
                                      CDMW_rootGOA.in(),
                                      *context); 
    
          // Store datastore_id in the Configuration class to have access inside
          // the whole SystemMngt CSCI
          PlatformMngt::Configuration::Set_event_report_datastore_id(datastore_id);
     
          // store datastore id in local datastore info
          (*localDataStoreInfos)[0].dsids[ds_inc] = datastore_id;
          ds_inc++;

 
          //
          // define the datastore responsible to store EventNotifier
          //
         CDMW_DEBUG(logger, "Create EventNotifier Datastore");
         typedef Cdmw::FT::DataStore<CdmwPlatformMngtBase::EventKey,
                                     CdmwPlatformMngt::Event>
                 EventNotifierFTDataStore;
                 
         // get EventNotifier datastore parameters from xml
         try
         {
       InitUtils::Get_datastore_id_parameter
         (*(xml_data.get()),
          "EventNotifierDataStore",
          NB_SYSMNGT_DATASTORE,
          datastore_id);
         } catch (const Cdmw::BadParameterException& ex) {
              CDMW_THROW2( Cdmw::OrbSupport::CORBASystemExceptionWrapperT<CORBA::BAD_PARAM>,
                           CORBA::BAD_PARAM(Cdmw::OrbSupport::BAD_PARAMInvalidXMLInitData,
                           CORBA::COMPLETED_NO),ex.what() );
         }
    
    
         // create data store (registered itself)
         new EventNotifierFTDataStore (datastore_id,
                                       orb,
                                       CDMW_rootGOA.in(),
                                       *context); 
    
         // Store datastore_id in the Configuration class to have access inside
         // the whole SystemMngt CSCI
         PlatformMngt::Configuration::Set_event_notifier_datastore_id(datastore_id);
    
         // store datastore id in local datastore info
         (*localDataStoreInfos)[0].dsids[ds_inc] = datastore_id;
         ds_inc++;
     


          //
          // define the datastore responsible to store EventNotifierExt
          //
         CDMW_DEBUG(logger, "Create EventNotifierExt Datastore");
         typedef Cdmw::FT::DataStore<CdmwPlatformMngtBase::EventKey,
                                     CdmwPlatformMngt::Event>
                 EventNotifierExtFTDataStore;

         // get EventNotifier datastore parameters from xml
         try
         {

              InitUtils::Get_datastore_id_parameter
                 (*(xml_data.get()),
                  "EventNotifierExtDataStore",
                  NB_SYSMNGT_DATASTORE,
                  datastore_id);

         } catch (const Cdmw::BadParameterException& ex) {
              CDMW_THROW2( Cdmw::OrbSupport::CORBASystemExceptionWrapperT<CORBA::BAD_PARAM>,
                           CORBA::BAD_PARAM(Cdmw::OrbSupport::BAD_PARAMInvalidXMLInitData,
                           CORBA::COMPLETED_NO),ex.what() );
         }

         // create data store (registered itself)
         new EventNotifierExtFTDataStore (datastore_id,
                                          orb,
                                          CDMW_rootGOA.in(),
                                          *context);

         // Store datastore_id in the Configuration class to have access inside
         // the whole SystemMngt CSCI
         PlatformMngt::Configuration::Set_event_notifier_ext_datastore_id(datastore_id);

         // store datastore id in local datastore info
         (*localDataStoreInfos)[0].dsids[ds_inc] = datastore_id;
         ds_inc++;


 
         //
         // define the datastore responsible to store ServiceDefs
         //
         CDMW_DEBUG(logger, "Create ServiceDefs Datastore");
         typedef Cdmw::FT::DataStore<std::string,
                                     CdmwPlatformMngtService::ServiceDefs> 
                 ServiceDefsFTDataStore;
                
         // get serviceDefs datastore parameters from xml
         try
         {
       InitUtils::Get_datastore_id_parameter
         (*(xml_data.get()),
          "ServiceDefsDataStore",
          NB_SYSMNGT_DATASTORE,
          datastore_id);
         } catch (const Cdmw::BadParameterException& ex) {
              CDMW_THROW2( Cdmw::OrbSupport::CORBASystemExceptionWrapperT<CORBA::BAD_PARAM>,
                           CORBA::BAD_PARAM(Cdmw::OrbSupport::BAD_PARAMInvalidXMLInitData,
                           CORBA::COMPLETED_NO),ex.what() );
         }
    
         // create data store (registered itself)
         new ServiceDefsFTDataStore (datastore_id,
                                     orb,
                                     CDMW_rootGOA.in(),
                                     *context); 
    
         // Store datastore_id in the Configuration class to have access inside
         // the whole SystemMngt CSCI
         PlatformMngt::Configuration::Set_service_defs_datastore_id(datastore_id);
    
         // store datastore id in local datastore info
         (*localDataStoreInfos)[0].dsids[ds_inc] = datastore_id;
         ds_inc++;

         //
         // define the datastore responsible to store RunSet
         //
         CDMW_DEBUG(logger, "Create RunSet Datastore");
         typedef Cdmw::FT::DataStore<std::string,
                                     CdmwPlatformMngt::RunSet> 
                 RunSetFTDataStore;
                 
         // get Runset datastore parameters from xml
         try
         {
       InitUtils::Get_datastore_id_parameter
         (*(xml_data.get()),
          "RunSetDataStore",
          NB_SYSMNGT_DATASTORE,
          datastore_id);
         } catch (const Cdmw::BadParameterException& ex) {
              CDMW_THROW2( Cdmw::OrbSupport::CORBASystemExceptionWrapperT<CORBA::BAD_PARAM>,
                           CORBA::BAD_PARAM(Cdmw::OrbSupport::BAD_PARAMInvalidXMLInitData,
                           CORBA::COMPLETED_NO),ex.what() );
         }
    
         // create data store (registered itself)
         new RunSetFTDataStore (datastore_id,
                                orb,
                                CDMW_rootGOA.in(),
                                *context); 
    
         // Store datastore_id in the Configuration class to have access inside
         // the whole SystemMngt CSCI
         PlatformMngt::Configuration::Set_runset_datastore_id(datastore_id);
    
         // store datastore id in local datastore info
         (*localDataStoreInfos)[0].dsids[ds_inc] = datastore_id;
         ds_inc++;


         //
         // define the datastore responsible to store PeriodicObserver
         //
         CDMW_DEBUG(logger, "Create PeriodicObserver Datastore");
         typedef Cdmw::FT::DataStore<std::string,
                                     CdmwPlatformMngt::PeriodicNotifierRecord> 
                 PeriodicObserverFTDataStore;
                 
         // get PeriodicObserver datastore parameters from xml
         try
         {
             InitUtils::Get_datastore_id_parameter(*(xml_data.get()),
                     "PeriodicObserverDataStore",
                     NB_SYSMNGT_DATASTORE,
                     datastore_id);
         } 
         catch (const Cdmw::BadParameterException& ex) 
         {
             CDMW_THROW2( Cdmw::OrbSupport::CORBASystemExceptionWrapperT<CORBA::BAD_PARAM>,
                          CORBA::BAD_PARAM(Cdmw::OrbSupport::BAD_PARAMInvalidXMLInitData, CORBA::COMPLETED_NO),
                          ex.what() );
         }
    
         // create data store (registered itself)
         new PeriodicObserverFTDataStore (datastore_id,
                                orb,
                                CDMW_rootGOA.in(),
                                *context); 
    
         // Store datastore_id in the Configuration class to have access inside
         // the whole SystemMngt CSCI
         PlatformMngt::Configuration::Set_periodic_observer_datastore_id(datastore_id);
                                      
         // store datastore id in local datastore info
         (*localDataStoreInfos)[0].dsids[ds_inc] = datastore_id;
         ds_inc++;
              
         /*
          *define the datastore responsible to store RedundancyGroup
          */

         CDMW_DEBUG(logger, "Create RedundancyGroup Datastore");
         typedef Cdmw::FT::DataStore<std::string,
         CdmwPlatformMngt::RedundancyGroupRecord> 
         RedundancyGroupFTDataStore;

         // get Component datastore parameters from xml
         try
         {
             InitUtils::Get_datastore_id_parameter(*(xml_data.get()),
                     "RedundancyGroupDataStore",
                     NB_SYSMNGT_DATASTORE,
                     datastore_id);
         } 
         catch (const Cdmw::BadParameterException& ex) 
         {
             CDMW_THROW2( Cdmw::OrbSupport::CORBASystemExceptionWrapperT<CORBA::BAD_PARAM>,
                     CORBA::BAD_PARAM(Cdmw::OrbSupport::BAD_PARAMInvalidXMLInitData, CORBA::COMPLETED_NO),
                     ex.what() );
         }

         // create data store (registered itself)
         new RedundancyGroupFTDataStore (datastore_id,
                 orb,
                 CDMW_rootGOA.in(),
                 *context); 

         // Store datastore_id in the Configuration class to have access inside
         // the whole SystemMngt CSCI
         PlatformMngt::Configuration::Set_redundancy_group_datastore_id(datastore_id);

         // store datastore id in local datastore info
         (*localDataStoreInfos)[0].dsids[ds_inc] = datastore_id;
         ds_inc++;
        
         /*
          *define the datastore responsible to store Component
          */
         
         CDMW_DEBUG(logger, "Create Component Datastore");
         typedef Cdmw::FT::DataStore<std::string,
                                     CdmwPlatformMngt::ComponentRecord> 
                 ComponentFTDataStore;
                 
         // get Component datastore parameters from xml
         try
         {
             InitUtils::Get_datastore_id_parameter(*(xml_data.get()),
                     "ComponentDataStore",
                     NB_SYSMNGT_DATASTORE,
                     datastore_id);
         } 
         catch (const Cdmw::BadParameterException& ex) 
         {
             CDMW_THROW2( Cdmw::OrbSupport::CORBASystemExceptionWrapperT<CORBA::BAD_PARAM>,
                          CORBA::BAD_PARAM(Cdmw::OrbSupport::BAD_PARAMInvalidXMLInitData, CORBA::COMPLETED_NO),
                          ex.what() );
         }
    
         // create data store (registered itself)
         new ComponentFTDataStore (datastore_id,
                                orb,
                                CDMW_rootGOA.in(),
                                *context); 
    
         // Store datastore_id in the Configuration class to have access inside
         // the whole SystemMngt CSCI
         PlatformMngt::Configuration::Set_component_datastore_id(datastore_id);
                                      
         // store datastore id in local datastore info
         (*localDataStoreInfos)[0].dsids[ds_inc] = datastore_id;
         ds_inc++;
         
         // create the StatefullPrimaryBackupAdmin_impl and register it
         CDMW_DEBUG(logger, "Create StatefullPrimaryBackupAdmin");
         std::string string_arg = 
             ::Cdmw::OsSupport::OS::get_option_value( argc, argv, "--HotRestart");
             
         bool cold_restart= true;
         if (strcmp(string_arg.c_str(), "yes") == 0)
         {
             cold_restart = false;
         }
         
         ::Cdmw::FT::Location::StatefullPrimaryBackupAdmin_impl * 
             statefull_primary_backup_admin
                = new ::Cdmw::FT::Location::StatefullPrimaryBackupAdmin_impl(
                                            orb, 
                                            CDMW_rootGOA.in(), 
                                            loc, 
                                            primary_backup_repository, 
                                            cold_restart);
               
         ::CdmwFT::Location::StatefullPrimaryBackupAdmin_var statefull_primary_backup_admin_ref 
             = statefull_primary_backup_admin->_this();
      
         CDMW_DEBUG(logger, "Register StatefullPrimaryBackupAdmin");
         replication_manager->register_statefull_location(
                     loc, 
                     statefull_primary_backup_admin_ref.in(),
                     localDataStoreInfos.in(), false);
   
   
    } catch (const Cdmw::OrbSupport::CORBASystemExceptionWrapper & ex) {
        CDMW_ERROR(logger, "CORBA Exception:\n   " << ex);
        ex.raise_SystemException();
    } catch (const CORBA::SystemException & ex) {
        CDMW_ERROR(logger, "CORBA Exception:\n   " 
                   << Cdmw::OrbSupport::OrbSupport::exception_to_string(ex));
        throw;
    } catch (const Cdmw::Exception & ex) {
        CDMW_ERROR(logger, "Cdmw::Exception:\n   " 
                   << ex.what());
        throw;
    } catch (const std::exception & ex) {
        CDMW_ERROR(logger, "std::exception:\n   " 
                   << ex.what());
        throw;
    }
}
 
// CDMW Factory objects will be deactivated at ORB shutdown.
void CDMW_cleanup(CORBA::ORB_ptr orb)
     throw ()
{
    CDMW_ASSERT(!CORBA::is_nil(orb));
}

   
} // end of namespace PlatformMngt

} // end of namespace Cdmw


