/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */



#ifndef INCL_PLATFORMMNGT_STARTER_CONFIGURATIONMNGR_HPP 
#define INCL_PLATFORMMNGT_STARTER_CONFIGURATIONMNGR_HPP 

#include "common/Exception.hpp"
#include <string>
#include <vector>

namespace Cdmw
{
namespace PlatformMngt
{
class StarterConfigurationException : public Exception
{
      public:

                StarterConfigurationException(const std::string& message, size_t lineNumber)
                    : Exception( Exception::FATAL, "Invalid configuration file" ),
                      m_message(message), m_lineNumber(lineNumber)
                {
                }

                ~StarterConfigurationException() throw() {};

                const char* getMessage() const
                {
                    return m_message.c_str();
                }

                size_t getLineNumber() const
                {
                    return m_lineNumber;
                }

      private:
                std::string m_message;
                size_t m_lineNumber;

};
    
    
/**
*Purpose:
*<p> 
*
*Features:
*<p> Thread safe
*
*@see  
*
*/
class StarterConfigurationMngr
{

    public:

        /**
        * Purpose:
        * <p> Constructor. Sets default values.
        * 
        */ 
        StarterConfigurationMngr()
                throw(OutOfMemoryException);


        /**
        * Purpose:
        * <p> Initializes the configuration from an XML file.
        * 
        * @param xml_conf The XML file path
        * @param validate Validate the XML file before processing
        */ 
        void parse(const std::string& xml_conf, bool validate)
                throw(OutOfMemoryException, StarterConfigurationException);


        /**
        * Purpose:
        * <p> Destructor
        * 
        */ 
        virtual 
        ~StarterConfigurationMngr()
                throw();


        /**
        * Purpose:
        * <p> Returns the daemon host as a string.
        */ 
        const char* getDaemonHostString() const throw();

        /**
        * Purpose:
        * <p> Returns the daemon port as a string.
        */ 
        const char* getDaemonPortString() const throw();

        /**
        * Purpose:
        * <p> Returns the process name.
        */ 
        const char* getProcessName() const throw();
        
        /**
        * Purpose:
        * <p> Returns the full path of the process executable.
        */ 
        const char* getProcessExecPath() const throw();
        
        /**
        * Purpose:
        * <p> Returns the process port as a string.
        */ 
        const char* getProcessPortString() const throw();
        
        /**
        * Purpose:
        * <p> Returns the process arguments.
        */ 
        const char* getProcessArgs() const throw();
        
        /**
        * Purpose:
        * <p> Returns the process working dir.
        */ 
        const char* getProcessWorkDir() const throw();
        
        /**
        * Purpose:
        * <p> Returns the process run dir.
        */ 
        const char* getProcessRunDir() const throw();

        
    private:
        
        /**
         * The host for the daemon
         */
        std::string m_daemonHost;
    
        /**
        * The port of the daemon.
        */
        std::string m_daemonPort;

        /**
        * The process name.
        */
        std::string m_processName;

        /**
        * The full path of the process executable.
        */
        std::string m_processExecPath;

        /**
        * The port of the process.
        */
        std::string m_processPort;
        
        /**
        * The arguments of the process.
        */
        std::string m_processArgs;
        
        /**
        * The working directory of the process.
        */
        std::string m_processWorkDir;
        
        /**
        * The run directory of the process.
        */
        std::string m_processRunDir;
                


}; // End class StarterConfigurationMngr 

} // End namespace PlatformMngt
} // End namespace Cdmw
#endif // INCL_PLATFORMMNGT_STARTER_CONFIGURATIONMNGR_HPP
