/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

#ifndef INCL_PLATFORMMNGT_SUPERVISION_MANAGEDELEMENT_COMMANDEVENTNOTIFIER_HPP
#define INCL_PLATFORMMNGT_SUPERVISION_MANAGEDELEMENT_COMMANDEVENTNOTIFIER_HPP

#include <Foundation/common/Exception.hpp>
#include <Foundation/osthreads/ThreadHandle.hpp>
#include <Foundation/osthreads/Mutex.hpp>
#include <Foundation/osthreads/MutexGuard.hpp>
#include <Foundation/osthreads/RecursiveMutex.hpp>
#include <Foundation/orbsupport/CORBA.hpp>
#include <Foundation/ossupport/OS.hpp>
#include <Foundation/osthreads/Condition.hpp>
#include <Foundation/orbsupport/TypeTraits.hpp>

#include <SystemMngt/platformlibrary/MessageQueue.hpp>
#include <SystemMngt/platformlibrary/LogMngr.hpp>
#include <SystemMngt/platformlibrary/CommonLogMessageId.hpp>
#include <SystemMngt/platformlibrary/Configuration.hpp>
#include <SystemMngt/platformlibrary/DataStoreBaseDefinition.hpp>
#include <SystemMngt/idllib/CdmwPlatformMngtBase.stub.hpp>
#include <SystemMngt/platformlibrary/BoundSyncCall.hpp>
#include <SystemMngt/platformlibrary/EventNotifier.hpp>
#include <SystemMngt/idllib/CdmwPlatformMngtEvent.stub.hpp>
#include <SystemMngt/platformlibrary/PropTypes.hpp>
#include <SystemMngt/platformlibrary/PropUtils.hpp>

#include <map>
#include <string>
#include <sstream>

namespace Cdmw
{
    namespace PlatformMngt
    {


    /**
     * The supervision command event deallocator.
     */

    typedef QueueElementDeallocator<CdmwPlatformMngt::CdmwCommand> CommandEventDeallocator;

    /**
     *Purpose:
     * The notifier of Managed Element Supervision Commands.
     *<p>
     *
     *Features:
     *<p> Thread safe. Exception safe.
     */
    class CommandEventNotifier:virtual public EventNotifier <CdmwPlatformMngt::CdmwCommand, 
                                                                 CommandEventDeallocator,
                                                                 CdmwPlatformMngt::SupervisionObserver_ptr,
                                                                 CdmwPlatformMngt::SupervisionObserver >
    {

    public:

        /**
         * The allowed timeout for each notification call.
         */
        unsigned long m_callTimeout;


        // attributes
    protected:
        /**
         * The Logger pointer
         */
        Cdmw::Logging::Logger * m_logger_pointer;

        /**
         * The Orb pointer
         */
        CORBA::ORB_ptr m_the_orb;

        /**
         * Internal mutex to control access at the shared resources
         */
        OsSupport::RecursiveMutex m_recursive_mutex;

        /**
         * Indicator flag for stop command
         */
        bool m_bMustStop;

        /**
         * Indicator flag for running state
         */
        bool m_bRunning;

        /**
         * Indicator flag for event on queue
         */
        bool m_beventInQueue;

        /**
         * Event report datastore
         */
        EventNotifierDataStore *m_event_notif_ds;

        /**
         * The factory of bound synchronous calls.
         */
        BoundSyncCallFactory *m_boundSyncCallFactory;

    private:
        /**
         * primary key for event
         */
        unsigned long m_evt_primary_key;

        /**
         * current time (sec and microseconds) for the previous sent event
         */
        unsigned long m_evt_seconds;
        unsigned long m_evt_microsec;

        /**
         * index for event identification
         * incremented if 2 events have the same timestamp
         */
        unsigned long m_evt_index;

        /**
         * Supervision change event queue
         */
        MessageQueue < CdmwPlatformMngt::CdmwCommand,
               CommandEventDeallocator > m_CommandEvQueue;

        /**
         * Set of the registered observer interface
         */
        MapObsForAll m_observers_regs;
    
    public:

        /**
         * Purpose:
         * <p> Constructor.
         */
        CommandEventNotifier (unsigned long callTimeout, CORBA::ORB_ptr the_orb);

        /**
         * Purpose:
         * <p> Destructor.
         */
        virtual ~ CommandEventNotifier ()throw ();


    protected:

        virtual std::string eventToString (CdmwPlatformMngt::CdmwCommand* event);

        enum TypeNotificationCallResult
        { SUCCESS, TIMEOUT, EXCEPTION };    //used on internal SetTimeoutPolicyOn - unsed now..

        /**
         *Purpose:
         *<p> Insert the event in DataStore.
         *
         *@param event The event to be placed in the data store.
         *
         */
        virtual void insertEventInDataStore (CdmwPlatformMngt::CdmwCommand * event);

        /**
         *Purpose:
         *<p> remove the event from DataStore.
         *
         *@param event_key The key of event to remove from the data store.
         *
         */
        virtual void removeEventInDataStore (CdmwPlatformMngtBase::EventKey event_key);
    
      private:

        /**
         * Purpose:
         * <p>
         * Copy constructor is not allowed.
         */
        CommandEventNotifier (const CommandEventNotifier & rhs);


        /**
         * Purpose:
         * <p>
         * Assignment operator is not allowed.
         */
        CommandEventNotifier & operator= (const CommandEventNotifier & rhs);


        // Operations
    public:

        /**
         *Purpose:
         *<p> Insert in the managed observers set the specified observer ( key based criteria )
         *
         *@param lpszelementpath target element path
         *@param EventAspect filter criteria
         *@paramCdmwPlatformMngt::SupervisionObserver_ptr pointer to observer interface
         */
         bool  register_observer (std::string observer_name, 
                                std::string element_path,
                                CdmwPlatformMngt::SupervisionObserver_ptr observer_ptr)
                                    throw (BadParameterException,  
                                           OutOfMemoryException,
                                           CdmwPlatformMngt::ElementAlreadyExists);

        /**
         *Purpose:
         *<p> Remove from managed observers set the specified observer ( key based criteria )
         *
         *@param lpszelementpath target element path
         *@param EventAspect filter criteria
         */
         bool  unregister_observer (const std::string& observer_name, const std::string element_path)
                                     throw (BadParameterException, 
                                            OutOfMemoryException,
                                            CdmwPlatformMngt::ObserverNotFound);
         
         void   unregister_observer_for_all_element_paths (const std::string& observer_name)
                                    throw (BadParameterException, 
                                            CdmwPlatformMngt::ObserverNotFound);

        void   unregister_observer_for_all_element_paths (const std::string& observer_name, bool propagate)
                                    throw (BadParameterException, 
                                            CdmwPlatformMngt::ObserverNotFound);


        /**
         *Purpose:
         *<p> Get a vector of observers registered on a certain path
         */
        std::vector<ObserverInfo> get_observers (std::string target_path);

        /**
         *Purpose:
         *<p> Check if the set of the registered observer is empty
         */
        bool is_registered_observer_empty ();
        
        /**
         *Purpose:
         *<p> Check if the thread is in running state
         */
        bool is_running ();

        /**
         *Purpose:
         *<p> Insert the property event into internal queue
         *
         *@param type of CdmwPlatformMngt::CdmwCommand event
         *
         */
        bool add_event (CdmwPlatformMngt::CdmwCommand* event) throw (InternalErrorException);
        
        /**
         *Purpose:
         *<p> execute the thread stop
         */
        void stop () throw ();

      private:

        /**
         *Purpose:
         *<p> Main Thread procedure
         */
        virtual void run () throw ();

    protected:

        /**
         *Purpose:
         *<p> Set the event key for identification.
         *
         *@param event The event to notify.
         *
         */
        void set_event_key (CdmwPlatformMngt::CdmwCommand * event);


        /**
         *Purpose:
         *<p> Set the policy timeout on Observer method called
         *
         *@paramCdmwPlatformMngt::SupervisionObserver_ptr observer to call
         *
         */
        CORBA::Object_ptr set_timeout_policy_on (CORBA::Object_ptr obj);

        /**
         *Purpose:
         *<p> Execute notification call on registered observers
         *
         *@paramCdmwPlatformMngt::SupervisionObserver_ptr observer_ptr observer called
         *@param CdmwPlatformMngt::CdmwCommand* CommandEvent event raised
         */
        virtual BoundSyncCallReturn  do_notification_call (CdmwPlatformMngt::SupervisionObserver_ptr observer_ptr,
                                                           CdmwPlatformMngt::CdmwCommand *
                                                           CommandEvent) 
                                                                throw ();

        /**
         *Purpose:
         *<p> Creates an event notification call corresponding to the event
         * and the observer.
         *
         * @param observer The observer to notify.
         * @param event The event to notify.
         * @return the bound synchronous call.
         *
         * @exception OutOfMemoryException
         */

        BoundSyncCall *createEventNotificationCall (CdmwPlatformMngt::SupervisionObserver_ptr observer,
                                                    CdmwPlatformMngt::CdmwCommand* event)
                                                       throw (OutOfMemoryException);
        
        };                // End class CommandEventNotifier
    }                // End namespace PlatformMngt
}                // End namespace Cdmw

#endif // INCL_PLATFORMMNGT_SUPERVISION_MANAGEDELEMENT_COMMANDEVENTNOTIFIER_HPP

