/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#ifndef INCL_PLATFORMMNGT_EVENT_HEADER_FACTORY_HPP 
#define INCL_PLATFORMMNGT_EVENT_HEADER_FACTORY_HPP 


#include <Foundation/orbsupport/CORBA.hpp>
#include <Foundation/ossupport/OS.hpp>
#include <Foundation/commonsvcs/time/Util.hpp>
#include <SystemMngt/idllib/CdmwPlatformMngtBase.stub.hpp>
#include <ctime>

namespace Cdmw
{
namespace PlatformMngt
{


/**
*Purpose:
*<p> 
* The factory of event header.
*/
class EventHeaderFactory
{

    public:

        /**
        * Purpose:
        * Returns the header embedding the specified level and the timestamp
        * of the operation call.
        *
        * @param level The event level
        *
        * @return The event header
        *
        */ 
        static CdmwPlatformMngtBase::EventHeader createHeader(
            CdmwPlatformMngtBase::EventLevel level)
        {

            CdmwPlatformMngtBase::EventHeader eventHeader;

            eventHeader.time_stamp = createTimeStamp();
            eventHeader.level = level;
            
            // initialise the emission event timestamp
            // used for emission
            eventHeader.event_key.primary_key = 0;
            eventHeader.event_key.seconds = 0;
            eventHeader.event_key.microseconds = 0;
            eventHeader.event_key.counter_inx = 0;
            
            return eventHeader;            
        }

        
        /**
        * Purpose:
        * Returns the header embedding the specified level and the timestamp
        * of the operation call.
        *
        * @param time_stamp The event time stamp
        * @param level The event level
        *
        * @return The event header
        *
        */ 
        static CdmwPlatformMngtBase::EventHeader createHeader(
            const CdmwPlatformMngtBase::TimeStamp& time_stamp,
            CdmwPlatformMngtBase::EventLevel level)
        {

            CdmwPlatformMngtBase::EventHeader eventHeader;

            eventHeader.time_stamp = time_stamp;
            eventHeader.level = level;
            
            // initialise the emission event timestamp
            // used for emission
            eventHeader.event_key.primary_key = 0;
            eventHeader.event_key.seconds = 0;
            eventHeader.event_key.microseconds = 0;
            eventHeader.event_key.counter_inx = 0;
            
            return eventHeader;             
        }

        /**
        * Purpose:
        * Returns the timestamp of the operation call.
        */ 
        static CdmwPlatformMngtBase::TimeStamp createTimeStamp()
        {
            Cdmw::OsSupport::OS::Timeval time_val = 
                    Cdmw::OsSupport::OS::get_time();

            CdmwPlatformMngtBase::TimeStamp timeStamp = 
                    TimeVal_2_TimeStamp (time_val);
            
            return timeStamp;             
        }
        
        /**
        * Purpose:
        * <p> convert a struct OsSupport::OS::Timeval to 
        *     a struct CdmwPlatformMngtBase::TimeStamp
        * @param tv is a Timeval struct.
        * @return the Timestamp struct.
        */
        static
        CdmwPlatformMngtBase::TimeStamp 
        TimeVal_2_TimeStamp (const ::Cdmw::OsSupport::OS::Timeval & tv)
        {
            CdmwPlatformMngtBase::TimeStamp timeStamp;
            
            time_t ltime = tv.seconds;
            
            tm now;
            ::localtime_r(&ltime, &now);

            timeStamp.year =  now.tm_year + 1900;
            timeStamp.month = now.tm_mon + 1;
            timeStamp.day =   now.tm_mday;

            timeStamp.hour = now.tm_hour;
            timeStamp.minute = now.tm_min;
            timeStamp.second = now.tm_sec;
            timeStamp.millisecond = (tv.microseconds)/1000;
            timeStamp.microsecond = (tv.microseconds)%1000;
            
            return timeStamp;
        }
        
        /**
         * Purpose:
         * <p> convert a TimeT value to a struct CdmwPlatformMngtBase::TimeStamp
         * @param timet is the number of 10^(-7) seconds from the julian day
         *        Fri 15 Oct 1582 (at 00:00:00,000,000), Coordinated Universal Time (UTC).
         * @param time_stamp is a Timestamp struct.
         * @return the return value is 0 if all ok, -1 in case of error.
         *
         */
         static
         int TimeT_2_TimeStamp (const TimeBase::TimeT timet, 
                                CdmwPlatformMngtBase::TimeStamp& time_stamp)
         {
            int result = 0;
            
            // first convert TimeT to OS::TimeVal
            OsSupport::OS::Timeval tv =
              CommonSvcs::Time::Util::TimeT_2_Timeval (timet);

            // if Timeval value is invalid, return error
            if (tv == CommonSvcs::Time::Util::naTimeval)
            {
                result = -1;
            }
            else
            {
                // else convert to TimeStamp
                time_stamp = TimeVal_2_TimeStamp (tv);                
            }
                        
            return result;
         }


}; // End class EventHeaderFactory

} // End namespace PlatformMngt
} // End namespace Cdmw

#endif // INCL_PLATFORMMNGT_EVENT_HEADER_FACTORY_HPP

