/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#ifndef INCL_PLATFORMMNGT_SYNCHRONISABLEPRECONDITIONTASK_HPP 
#define INCL_PLATFORMMNGT_SYNCHRONISABLEPRECONDITIONTASK_HPP 

#include <SystemMngt/platformlibrary/SynchronisableTaskFactory.hpp>
#include <SystemMngt/platformlibrary/Sequencer.hpp>
#include <SystemMngt/platformlibrary/Starter.hpp>
#include <SystemMngt/tools/SingleShotTimer.hpp>
#include <SystemMngt/tools/TimeoutObserver.hpp>
#include <SystemMngt/idllib/CdmwPlatformMngtProcess.stub.hpp>
#include <SystemMngt/idllib/CdmwPlatformMngtApplication.stub.hpp>
#include <SystemMngt/idllib/CdmwPlatformMngtSystem.stub.hpp>
#include "idllib/CdmwPlatformMngtSupervisionObserver.skel.hpp"
#include "Foundation/osthreads/Condition.hpp"
#include "Foundation/osthreads/Mutex.hpp"



namespace Cdmw
{
namespace PlatformMngt
{

class SynchronisablePreconditionTaskObserver;
class EventObserver;

/**
*Purpose:
*<p> extends the Synchronisable task, making the task controlled by a precondition.  
*/

class SynchronisablePreconditionTask : public SynchronisableTask
{
friend class EventObserver;
    public:

        /**
        * Purpose:
        * <p> Constructor 
        * @param timeout           the timeout of the Timer.
        * @param managed element   the name of the managed element
        *                          ables to control the task start
        * @param status            the expected status of managed element  
        * @param taskId            the identifier of the task.
        * @param sequencer         the sequencer of the created task.
        * @param observer          the task observer
        * @param syncTaskFactory   the factory used to create the task.
        * @param predecessorCount  the number of predecessor tasks
        * @param  process          the process reference
        */ 
        SynchronisablePreconditionTask(
                                     const int timeout,
                                     const std::string& managed_element,
                                     const std::string& status,
                                     const std::string& taskId,
                                     Sequencer* sequencer,
                                     TaskObserver* observer,
                                     SynchronisableTaskFactory* synchTaskFactory,
                                     size_t predecessorCount, 
                                     CdmwPlatformMngt::Process_var process);
        /**
        * Purpose:
        * <p> Destructor.
        */ 
        virtual ~SynchronisablePreconditionTask() throw();
              
         /**
         *Purpose:
         *<p> Decrements the number of predecessors to wait.
         * Starts actually the task if there are no more predecessors
         * and if the precondition is verified or, 
         * if a timeout is defined, the timer expiration.
         */  
        virtual void startTask() throw();
        
        
        /* Purpose
         * <p>
         * Starting the thread
         */
              
        virtual void run()
          throw();
               
    private:
          
        /**
        * Purpose:
        * <p>
        * Copy constructor is not allowed.
        */ 
        SynchronisablePreconditionTask(const SynchronisablePreconditionTask& rhs);

        /**
        * Purpose:
        * <p>
        * Assignment operator is not allowed.
        */ 
        SynchronisablePreconditionTask& operator=(const SynchronisablePreconditionTask& rhs);
     public:    
        /*
         * Purpose:
         * <p>
         * Convert a string in ApplicationStatus.
         */
        CdmwPlatformMngt::ApplicationStatus string_to_status(std::string& s_string) 
            throw (CdmwPlatformMngt::IncompatibleStatus);
         
  

    private:

        /**
         * The timeout value
         */
        long  m_precond_timeout;
               
        
        /*
         * ObserverName
         */
        std::string m_observer_name;
        
        /*
         * The precondition managed element 
         */
        std::string m_precond_managed_element;
        
        /*
         * The precondition status
         */
        
        std::string m_precond_status;
        
        /**
         * The mutex used to start task
         */
        OsSupport::Mutex m_mutex;
        
        /**
         * The start variable condition.
         */
        OsSupport::Condition m_condition;
        
        bool m_condition_flag; 
        
        /*
         * The process reference
         */
        CdmwPlatformMngt::Process_var  m_process;
        
        /*
         * The application reference
         */
        CdmwPlatformMngt::Application_var m_application;
        
        /*
         * The System reference
         */
        CdmwPlatformMngt::System_var m_system;
        
        /*
         * The Host reference
         */
        
        CdmwPlatformMngt::Host_var m_host;
        /*
         * The Event Observer reference 
         */
        
        CdmwPlatformMngt::SupervisionObserver_var m_event_obs;
        
        /*
         * The reference to the precodition Application
         */
        CdmwPlatformMngt::Application_var m_appl_to_wait;
        
        /*
         * The task description (System/Appl/Proc) 
         */
        std::string m_description;
    

}; // End SynchronisablePreconditionTask


class EventObserver: virtual public POA_CdmwPlatformMngt::SupervisionObserver,
                     virtual public PortableServer::RefCountServantBase
 {
    public:
        /**
         * Purpose:
         * <p> Constructor 
         * @param managed_element   the name of the managed element
         *                          ables to control the task start
         * @param status            the expected status of managed element
         * @param mutex             the mutex used to start the task
         * @param condition         the condition variable
         */

        EventObserver (
                
                Cdmw::PlatformMngt::SynchronisablePreconditionTask* task,
                std::string& status
                );

        /*
         * Purpose:
         * <p>Destructor   
         */
        
        ~EventObserver(){ };

        /*
         * Purpose:
         * <p>Notify the event on which the observer is registered   
         */
        virtual void notify( CdmwPlatformMngt::Event* an_event )
        throw( CORBA::SystemException );
        
        /*
         * Sets true the "m_expired" variable
         */
        void expired();
    
    private:
        
        /*
         * The task pointer
         */
        Cdmw::PlatformMngt::SynchronisablePreconditionTask* m_task;
        
        /*
         * The managed element status required
         */
        std::string m_status;
        
        /*
         * Is the timeout expired?
         */
        bool m_expired;

};

}//End Namespacing
}
#endif // INCL_PLATFORMMNGT_SYNCHRONISABLEPRECONDITIONTASK_HPP





