/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#ifndef INCL_PLATFORMMNGT_TAO_BOUND_SYNC_CALL_HPP
#define INCL_PLATFORMMNGT_TAO_BOUND_SYNC_CALL_HPP

#include "SystemMngt/platformlibrary/BoundSyncCall.hpp"

namespace Cdmw
{
namespace PlatformMngt
{

/**
 *Purpose:
 *<p> Synchronous method invocation with timeout factory.
 *
 *Features:
 *<p> 
 */        
class TAOBoundSyncCallFactory : virtual public BoundSyncCallFactory
{
private:
    CORBA::ORB_var m_orb;

public:
    /**
     *Purpose:
     *<p> Constructor
     *
     *@param orb An ORB ptr. The ORB to be used.
     *
     *@return An TAO BoundSyncCall factory.
     */
    TAOBoundSyncCallFactory( CORBA::ORB_ptr orb );

    /**
     *Purpose:
     *<p> Default destructor.
     */
     virtual
     ~TAOBoundSyncCallFactory();
     
public:
    /**
     *Purpose:
     *<p> Concrete method for obtaining a bound synchronous method call. The returned
     * object provides method for the specified process to perform its initialisation 
     * procedure with the given timeout.
     *
     *@param process The recipient process of the synchronous call with timeout.
     *@param startupKind The startup kind.
     *@param timeout The time expiration.
     *
     *@return a bound synchronous call with timeout object.
     *
     *@exception OutOfMemoryException Lack of memory.
     */
    ProcessInitCall* createProcessInitCall(
        CdmwPlatformMngt::ProcessDelegate_ptr process,
        const CdmwPlatformMngtBase::StartupKind& startupKind,
        unsigned long timeout )
    throw( OutOfMemoryException );

    /**
     *Purpose:
     *<p> Concrete method for obtaining a bound synchronous method call. The returned
     * object provides method for the specified process to perform its next initialisation
     * step with the given timeout.
     *
     *@param process The recipient process of the synchronous call with timeout.
     *@param timeout The time expiration.
     *
     *@return a bound synchronous call with timeout object.
     *
     *@exception OutOfMemoryException Lack of memory.
     */
    ProcessStepCall* createProcessStepCall(
        CdmwPlatformMngt::ProcessDelegate_ptr process,
        unsigned long timeout )
    throw( OutOfMemoryException );

    /**
     *Purpose:
     *<p> Concrete method for obtaining a bound synchronous method call. The returned
     * object provides method for the specified process to perform its run method
     * with the given timeout.
     *
     *@param process The recipient process of the synchronous call with timeout.
     *@param timeout The time expiration.
     *
     *@return a bound synchronous call with timeout object.
     *
     *@exception OutOfMemoryException Lack of memory.
     */
    ProcessRunCall* createProcessRunCall(
        CdmwPlatformMngt::ProcessDelegate_ptr process,
        unsigned long timeout )
    throw( OutOfMemoryException );

    /**
     *Purpose:
     *<p> Concrete method for obtaining a bound synchronous method call. The returned
     * object provides method for the specified proces to perform its stop procedure
     * with the given timeout.
     *
     *@param process The recipient process of the synchronous call with timeout.
     *@param timeout The time expiration.
     *
     *@return a bound synchronous call with timeout object.
     *
     *@exception OutOfMemoryException Lack of memory.
     */
    ProcessStopCall* createProcessStopCall(
        CdmwPlatformMngt::ProcessDelegate_ptr process,
        unsigned long timeout )
    throw( OutOfMemoryException );

    /**
     *Purpose:
     *<p> Concrete method for obtaining a bound synchronous method call. The returned
     * object provides method for notifying (with the given timeout) the specified
     * observer that a specified event happened.
     *
     *@param observer The recipient observer of the notification.
     *@param event The event to be notified.
     *@param timeout The time expiration.
     *
     *@return a bound synchronous call with timeout object.
     *
     *@exception OutOfMemoryException Lack of memory.
     */
    EventNotificationCall* createEventNotificationCall(
        CdmwPlatformMngt::SupervisionObserver_ptr observer,
        CdmwPlatformMngt::Event* event,
        unsigned long timeout )
    throw( OutOfMemoryException );

    /**
     *Purpose:
     *<p> Concrete method for obtaining a bound synchronous method call. The returned
     * object provides method for notifying (with the given timeout) the supervision
     * that the specified monitorable object is still alive.
     *
     *@param monitorable The recipient monitorable of the synchronous call with timeout.
     *@param timeout The time expiration.
     *
     *@return a bound synchronous call with timeout object.
     *
     *@exception OutOfMemoryException Lack of memory.
     */
    PullMonitoringCall* createPullMonitoringCall(
        CdmwPlatformMngt::PullMonitorable_ptr monitorable,
        unsigned long timeout )
    throw( OutOfMemoryException );
    
    
     /**
     *Purpose:
     *<p> Concrete method for obtaining a bound synchronous method call. The returned
     * object provides method for notifying (with the given timeout) the supervision
     * that the specified monitorable object is still alive.
     *
     *@param node_app_manager The recipient node application manager of the synchronous call with timeout.
     *@param node_app The Node application.
     *@param timeout The time expiration.
     *
     *@return a bound synchronous call with timeout object.
     *
     *@exception OutOfMemoryException Lack of memory.
     */
    DanceDestroyApplicationCall* createDanceDestroyApplicationCall(
        CdmwDnC::NodeApplicationManager_ptr node_app_manager,
        CdmwDnC::NodeApplication_ptr node_app,
        unsigned long timeout )
    throw( OutOfMemoryException );
    
    
     /**
     *Purpose:
     *<p> Concrete method for obtaining a bound synchronous method call. The returned
     * object provides method for notifying (with the given timeout) the supervision
     * that the specified monitorable object is still alive.
     *
     *@param node_manager The recipient node manager of the synchronous call with timeout.
     *@param node_application_manager The node application manager.
     *@param timeout The time expiration.
     *
     *@return a bound synchronous call with timeout object.
     *
     *@exception OutOfMemoryException Lack of memory.
     */
    DanceDestroyManagerCall* createDanceDestroyManagerCall(
        CdmwDnC::NodeManager_ptr node_manager,
        CdmwDnC::NodeApplicationManager_ptr node_application_manager,
        unsigned long timeout )
    throw( OutOfMemoryException );
    

    /**
     *Purpose:
     *<p> Concrete method for obtaining a bound synchronous method call. The returned
     * object provides method for the specified component to perform its initialisation 
     * procedure with the given timeout.
     *
     *@param component The recipient component of the synchronous call with timeout.
     *@param startupKind The startup kind.
     *@param timeout The time expiration.
     *
     *@return a bound synchronous call with timeout object.
     *
     *@exception OutOfMemoryException Lack of memory.
     */
    ComponentInitCall* createComponentInitCall(
        CdmwPlatformMngt::ManagedComponent_ptr component,
        const CdmwPlatformMngtBase::StartupKind& startupKind,
        unsigned long timeout )
    throw( OutOfMemoryException );
    
    /**
     *Purpose:
     *<p> Concrete method for obtaining a bound synchronous method call. The returned
     * object provides method for the specified node application to perform its start 
     * procedure with the given timeout.
     *
     *@param node_application The recipient node_application of the synchronous call with timeout.
     *@param timeout The time expiration.
     *
     *@return a bound synchronous call with timeout object.
     *
     *@exception OutOfMemoryException Lack of memory.
     */
    NodeApplicationStartCall* createNodeApplicationStartCall(
        CdmwDnC::NodeApplication_ptr node_application,
        unsigned long timeout )
    throw( OutOfMemoryException );
    

};

/**
 *Purpose:
 *<p> Synchronous process initialisation with timeout.
 *
 *Features:
 *<p> 
 */        
class TAOProcessInitCall : virtual public ProcessInitCall
{
private:
    CORBA::ORB_var m_orb;

public:
    /**
     *Purpose:
     *<p> Constructor
     *
     *@param orb An ORB ptr. The ORB to be used.
     *@param process The recipient process of the synchronous call with timeout.
     *@param startupKind The startup kind.
     *@param timeout The time expiration.
     *
     *@exception OutOfMemoryException Lack of memory.
     */
    TAOProcessInitCall(
        CORBA::ORB_ptr orb,
        CdmwPlatformMngt::ProcessDelegate_ptr process,
        const CdmwPlatformMngtBase::StartupKind& startupKind,
        unsigned long timeout )
    throw( OutOfMemoryException );

    /**
     *Purpose:
     *<p> Default destructor.
     */
     virtual
     ~TAOProcessInitCall();
     
    /**
     *Purpose:
     *<p> Performs the process's initialisation with the given timeout.
     *
     *@return This method returns the status of the invocation.
     */
    BoundSyncCallReturn execute()
    throw();
};

/**
 *Purpose:
 *<p> Synchronous process step initialisation with timeout.
 *
 *Features:
 *<p> 
 */        
class TAOProcessStepCall : virtual public ProcessStepCall
{
private:
    CORBA::ORB_var m_orb;

public:
    /**
     *Purpose:
     *<p> Constructor
     *
     *@param orb An ORB ptr. The ORB to be used.
     *@param process The recipient process of the synchronous call with timeout.
     *@param timeout The time expiration.
     */
    TAOProcessStepCall(
        CORBA::ORB_ptr orb,
        CdmwPlatformMngt::ProcessDelegate_ptr process,
        unsigned long timeout );

    /**
     *Purpose:
     *<p> Default destructor.
     */
     virtual
     ~TAOProcessStepCall();
     
    /**
     *Purpose:
     *<p> Performs the process's next initialisation step with the given timeout.
     *
     *@return This method returns the status of the invocation.
     */
    BoundSyncCallReturn execute()
    throw();
};

/**
 *Purpose:
 *<p> Synchronous process running with timeout.
 *
 *Features:
 *<p> 
 */        
class TAOProcessRunCall : virtual public ProcessRunCall
{
private:
    CORBA::ORB_var m_orb;

public:
    /**
     *Purpose:
     *<p> Constructor
     *
     *@param orb An ORB ptr. The ORB to be used.
     *@param process The recipient process of the synchronous call with timeout.
     *@param timeout The time expiration.
     */
    TAOProcessRunCall(
        CORBA::ORB_ptr orb,
        CdmwPlatformMngt::ProcessDelegate_ptr process,
        unsigned long timeout );

    /**
     *Purpose:
     *<p> Default destructor.
     */
     virtual
     ~TAOProcessRunCall();
     
    /**
     *Purpose:
     *<p> Performs the process's run method with the given timeout.
     *
     *@return This method returns the status of the invocation.
     */
    BoundSyncCallReturn execute()
    throw();
};

/**
 *Purpose:
 *<p> Synchronous process stop with timeout.
 *
 *Features:
 *<p> 
 */        
class TAOProcessStopCall : virtual public ProcessStopCall
{
private:
    CORBA::ORB_var m_orb;

public:
    /**
     *Purpose:
     *<p> Constructor
     *
     *@param orb An ORB ptr. The ORB to be used.
     *@param process The recipient process of the synchronous call with timeout.
     *@param timeout The time expiration.
     */
    TAOProcessStopCall(
        CORBA::ORB_ptr orb,
        CdmwPlatformMngt::ProcessDelegate_ptr process,
        unsigned long timeout );

    /**
     *Purpose:
     *<p> Default destructor.
     */
     virtual
     ~TAOProcessStopCall();
     
    /**
     *Purpose:
     *<p> Performs the process's stop procedure with the given timeout.
     *
     *@return This method returns the status of the invocation.
     */
    BoundSyncCallReturn execute()
    throw();
};

/**
 *Purpose:
 *<p> Synchronous event notification with timeout.
 *
 *Features:
 *<p> 
 */        
class TAOEventNotificationCall : virtual public EventNotificationCall
{
private:
    CORBA::ORB_var m_orb;

public:
    /**
     *Purpose:
     *<p> Constructor
     *
     *@param orb An ORB ptr. The ORB to be used.
     *@param observer The recipient observer of the notification.
     *@param event The event to be notified.
     *@param timeout The time expiration.
     */
    TAOEventNotificationCall(
        CORBA::ORB_ptr orb,
        CdmwPlatformMngt::SupervisionObserver_ptr observer,
        CdmwPlatformMngt::Event* event,
        unsigned long timeout );
        
    /**
     *Purpose:
     *<p> Default destructor.
     */
     virtual
     ~TAOEventNotificationCall();
     
    /**
     *Purpose:
     *<p> Notifies (with the given timout) the observer that an event happened.
     *
     *@return This method returns the status of the invocation.
     */
    BoundSyncCallReturn execute()
    throw();
};

/**
 *Purpose:
 *<p> Synchronous pull monitoring with timeout.
 *
 *Features:
 *<p> 
 */        
class TAOPullMonitoringCall : virtual public PullMonitoringCall
{
private:
    CORBA::ORB_var m_orb;

public:
    /**
     *Purpose:
     *<p> Constructor
     *
     *@param orb An ORB ptr. The ORB to be used.
     *@param monitorable The recipient monitorable of the synchronous call with timeout.
     *@param timeout The time expiration.
     */
    TAOPullMonitoringCall(
        CORBA::ORB_ptr orb,
        CdmwPlatformMngt::PullMonitorable_ptr monitorable,
        unsigned long timeout );

    /**
     *Purpose:
     *<p> Default destructor.
     */
     virtual
     ~TAOPullMonitoringCall();
     
    /**
     *Purpose:
     *<p> Notifies the supervision that the monitorable is still alive.
     *
     *@return This method returns the status of the invocation.
     */
    BoundSyncCallReturn execute()
    throw();
};


/**
 *Purpose:
 *<p> Synchronous node application manager destroy application call with timeout.
 *
 *Features:
 *<p> 
 */        
class TAODanceDestroyApplicationCall : virtual public DanceDestroyApplicationCall
{
private:
    CORBA::ORB_var m_orb;
    CdmwDnC::NodeApplicationManager_var m_node_application_manager;
    CdmwDnC::NodeApplication_var m_node_application;

public:
    /**
     *Purpose:
     *<p> Constructor
     *
     *@param orb An ORB ptr. The ORB to be used.
     *@param node_application_manager The recipient node application manager of the synchronous call with timeout.
     *@param node_application
     *@param timeout The time expiration.
     */

    
   TAODanceDestroyApplicationCall::TAODanceDestroyApplicationCall(
    CORBA::ORB_ptr orb,
    CdmwDnC::NodeApplicationManager_ptr node_application_manager,
    CdmwDnC::NodeApplication_ptr node_application,
    unsigned long timeout );    

    /**
     *Purpose:
     *<p> Default destructor.
     */
     virtual
     ~TAODanceDestroyApplicationCall();
     
    /**
     *Purpose:
     *<p> Notifies the supervision that the monitorable is still alive.
     *
     *@return This method returns the status of the invocation.
     */
    BoundSyncCallReturn execute()
    throw();
};





/**
 *Purpose:
 *<p> Synchronous node manager destroy manager call with timeout.
 *
 *Features:
 *<p> 
 */         
class TAODanceDestroyManagerCall : virtual public DanceDestroyManagerCall
{
private:
    CORBA::ORB_var m_orb;
    CdmwDnC::NodeManager_var m_node_manager;
    CdmwDnC::NodeApplicationManager_var m_node_application_manager;

public:
   /**
     *Purpose:
     *<p> Constructor
     *
     *@param orb An ORB ptr. The ORB to be used.
     *@param node_manager The recipient node manager of the synchronous call with timeout.
     *@param node_application_manager
     *@param timeout The time expiration.
     */
    
   TAODanceDestroyManagerCall::TAODanceDestroyManagerCall(
    CORBA::ORB_ptr orb,
    CdmwDnC::NodeManager_ptr node_manager,
    CdmwDnC::NodeApplicationManager_ptr node_application_manager,
    unsigned long timeout );    

    /**
     *Purpose:
     *<p> Default destructor.
     */
     virtual
     ~TAODanceDestroyManagerCall();
     
    /**
     *Purpose:
     *<p> Destroy node manager
     *
     *@return This method returns the status of the invocation.
     */
    BoundSyncCallReturn execute()
    throw();
};



/**
 *Purpose:
 *<p> Synchronous process initialisation with timeout.
 *
 *Features:
 *<p> 
 */        
class TAOComponentInitCall : virtual public ComponentInitCall
{
private:
    CORBA::ORB_var m_orb;

public:
    /**
     *Purpose:
     *<p> Constructor
     *
     *@param orb An ORB ptr. The ORB to be used.
     *@param component The recipient component of the synchronous call with timeout.
     *@param startupKind The startup kind.
     *@param timeout The time expiration.
     *
     *@exception OutOfMemoryException Lack of memory.
     */
    TAOComponentInitCall(
        CORBA::ORB_ptr orb,
        CdmwPlatformMngt::ManagedComponent_ptr component,
        const CdmwPlatformMngtBase::StartupKind& startupKind,
        unsigned long timeout )
    throw( OutOfMemoryException );

    /**
     *Purpose:
     *<p> Default destructor.
     */
     virtual
     ~TAOComponentInitCall();
     
    /**
     *Purpose:
     *<p> Performs the process's initialisation with the given timeout.
     *
     *@return This method returns the status of the invocation.
     */
    BoundSyncCallReturn execute()
    throw();
};



/**
 *Purpose:
 *<p> Synchronous process initialisation with timeout.
 *
 *Features:
 *<p> 
 */        
class TAONodeApplicationStartCall : virtual public NodeApplicationStartCall
{
private:
    CORBA::ORB_var m_orb;

public:
    /**
     *Purpose:
     *<p> Constructor
     *
     *@param orb An ORB ptr. The ORB to be used.
     *@param node_application The recipient node application of the synchronous call with timeout..
     *@param timeout The time expiration.
     *
     *@exception OutOfMemoryException Lack of memory.
     */
    TAONodeApplicationStartCall(
        CORBA::ORB_ptr orb,
        CdmwDnC::NodeApplication_ptr node_application,
        unsigned long timeout )
    throw( OutOfMemoryException );

    /**
     *Purpose:
     *<p> Default destructor.
     */
     virtual
     ~TAONodeApplicationStartCall();
     
    /**
     *Purpose:
     *<p> Performs the process's initialisation with the given timeout.
     *
     *@return This method returns the status of the invocation.
     */
    BoundSyncCallReturn execute()
    throw();
};



} // End namespace PlatformMngt
} // End namespace Cdmw

#endif //INCL_PLATFORMMNGT_TAO_BOUND_SYNC_CALL_HPP

