/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#include "SystemMngt/platformlibrary/CommandEventNotifier.hpp"
#include "SystemMngt/platformlibrary/EventNotifierManager.hpp"
#include "Foundation/commonsvcs/datastore/StorageErrorException.hpp"
#include "SystemMngt/platformlibrary/StatusEventNotifier.hpp"
#include "SystemMngt/platformlibrary/PropertyEventNotifier.hpp"

namespace Cdmw
{
    namespace PlatformMngt
    {
       
        CommandEventNotifier::CommandEventNotifier (unsigned long callTimeout, CORBA::ORB_ptr the_orb)
        {
            m_bMustStop = false;
            m_bRunning = false;
            m_beventInQueue = false;
            m_callTimeout = callTimeout;
            m_evt_primary_key = 0;
            m_evt_seconds = 0;
            m_evt_microsec = 0;
            m_evt_index = 0;
            m_the_orb = CORBA::ORB::_duplicate (the_orb);
            m_event_notif_ds = 0;
            m_logger_pointer =
            Cdmw::Logging::Logger::Get_Logger (CdmwPlatformMngtBase::PFMMNG);
            CDMW_ASSERT (m_logger_pointer != 0);

            CDMW_DEBUG (m_logger_pointer, "Command Event Notifier Instance ");

            m_boundSyncCallFactory = BoundSyncCallFactory::getFactory ();


            // instance to datastore notifiers
            try
            {
            Cdmw::CommonSvcs::DataStore::DataStoreBase * ds =
                Cdmw::CommonSvcs::DataStore::DataStoreRegistry::
                Get_data_store (Configuration::
                        Get_event_notifier_datastore_id ());
            m_event_notif_ds =
                dynamic_cast < Cdmw::PlatformMngt::EventNotifierDataStore * >(ds);
            }
            catch (const NotExistException & e)
            {
            CDMW_ASSERT (false);
            throw;
            }
            catch (std::bad_cast & e)
            {
            CDMW_FATAL (m_logger_pointer, e.what ());
            throw;
            }
        }

        CommandEventNotifier::~CommandEventNotifier ()throw ()
        {
            CORBA::release (m_the_orb);
            stop ();
            m_CommandEvQueue.destroy ();
        }


        std::string CommandEventNotifier::eventToString (CdmwPlatformMngt::CdmwCommand*  event)
        {
            CORBA::String_var str = event->to_string ();

            std::string eventStr (str.in ());

            return eventStr;
        }


        void CommandEventNotifier::insertEventInDataStore (CdmwPlatformMngt::CdmwCommand* event)
        {
            if (Configuration::Is_primary ())
            {
            // update the datastore
            // get the event key
            CdmwPlatformMngtBase::EventHeader & event_header =
                event->header ();

            try
            {
                m_event_notif_ds->insert (event_header.event_key, event);
            }
            catch (Common::AlreadyExistException &)
            {
                CDMW_WARN (m_logger_pointer,
                       "event notif record already exists in datastore : "
                       << event_header.event_key.
                       primary_key << ":" << event_header.event_key.
                       seconds << ":" << event_header.event_key.
                       microseconds << ":" << event_header.event_key.
                       counter_inx);
            }
            catch (const CommonSvcs::DataStore::StorageErrorException &)
            {
                CDMW_FATAL(m_logger_pointer,
                       "event notif record storage error exception in insertion : "
                       << event_header.event_key.
                       primary_key << ":" << event_header.event_key.
                       seconds << ":" << event_header.event_key.
                       microseconds << ":" << event_header.event_key.
                       counter_inx);
                CDMW_ASSERT(false);
            }      
            catch (...)
            {

                CDMW_FATAL (m_logger_pointer,
                       "event notif record unexpected exception in insertion : "
                       << event_header.event_key.
                       primary_key << ":" << event_header.event_key.
                       seconds << ":" << event_header.event_key.
                       microseconds << ":" << event_header.event_key.
                       counter_inx);
                CDMW_ASSERT(false);
            }
            }
        }

        void CommandEventNotifier::removeEventInDataStore (CdmwPlatformMngtBase::EventKey event_key)
        {
            if (Configuration::Is_primary ())
            {
                try
                {
                    m_event_notif_ds->remove (event_key);
                }
                catch (Common::NotFoundException &)
                {
                }
                catch (const CommonSvcs::DataStore::StorageErrorException &)
                {
                }

            }
        }

        bool CommandEventNotifier::register_observer (std::string observer_name, std::string element_path,
                CdmwPlatformMngt::SupervisionObserver_ptr observer_ptr)
            throw (BadParameterException, OutOfMemoryException, CdmwPlatformMngt::ElementAlreadyExists)
        {

            if (m_bMustStop) {
                return false;
            }
            try
            {
                CDMW_MUTEX_GUARD (m_recursive_mutex);
    
                if (CORBA::is_nil (observer_ptr)) {
                    CDMW_THROW2 (BadParameterException, "observer_ptr", "NIL");
                }
                
                //check if the observer has already been registered
                // return false if no key has the observer_name in its value
                bool bool_ret = 
                    PropUtils::check_obs_name_in_map_for_all(m_observers_regs, observer_name, element_path);

                // if the element is present throw an Element Already Present Exception 
                if (bool_ret) {
                    throw CdmwPlatformMngt::ElementAlreadyExists();
                }

                // if the element is not present insert it into observers map
                ObserverInfo obs_info;
                obs_info.observer_name = observer_name;
                obs_info.observer_ptr = CdmwPlatformMngt::SupervisionObserver::_duplicate(observer_ptr);

                m_observers_regs.insert (MapObsForAll::value_type (element_path, obs_info));
          
                //PropUtils::print_map_for_all(m_observers_regs);
            }
            catch (const std::bad_alloc &)
            {
                CDMW_THROW (OutOfMemoryException);
            }
        
        
            return true;
        }


        bool CommandEventNotifier::unregister_observer (const std::string& observer_name, 
                                                            const std::string element_path)
                                                                throw (BadParameterException, 
                                                                       OutOfMemoryException,
                                                                       CdmwPlatformMngt::ObserverNotFound)
        {
            bool bret = false;
        MapObsForAll::iterator it_for_all;
    
            std::string target_element_path = element_path;
            CDMW_DEBUG (m_logger_pointer,
                "UnRegister the observer to Command Event Notifier");
        
            {
                CDMW_MUTEX_GUARD (m_recursive_mutex);

                // int to count the occurrencies in Multimap for Status or for All
                int observer_entries = 0;
 
                // find all observer occurrencies for the element_path
                observer_entries =    m_observers_regs.count (target_element_path);
                std::string observer_name_in_map = "";
                if (observer_entries!=0)
                { 
                    it_for_all = m_observers_regs.find (target_element_path);
                    observer_name_in_map = it_for_all->second.observer_name;
                }
                // if the observer is present in the map for all for that element path, erase the entry in the Map
                int found = 0;
                //if((it_for_all != observers_regs.end ())&& (observer_name==observer_name_in_map))
                if((it_for_all != m_observers_regs.end ()))
                {    
                    for (int j = 0; j < observer_entries; ++j)
                    {
                        if (it_for_all->second.observer_name == observer_name)
                        {
                    
                        //release the SupervisionObserver_ptr
                        //CORBA::release(it_for_all->second.observer_ptr);
                    
                        MapObsForAll::iterator tmp_it = it_for_all;
                        it_for_all++;
                
                        m_observers_regs.erase (tmp_it);
                        found++;
                        }
                        else{
                            it_for_all++;
                        }
            bret = true;
                    }
                }
                if (found==0) throw CdmwPlatformMngt::ObserverNotFound(); 
            
        }
        
            return bret;
        }
        
        std::vector<ObserverInfo> CommandEventNotifier::get_observers (std::string target_path)
        {
            std::vector<ObserverInfo> ret;
            std::pair<MapObsForAll::iterator, MapObsForAll::iterator> itp = m_observers_regs.equal_range(target_path);
            for (MapObsForAll::iterator it = itp.first; it != itp.second; ++it) {
                ret.push_back((*it).second);   
            }
            return ret;
        }
        
        bool CommandEventNotifier::is_registered_observer_empty ()
        {
            bool bret = false;
           
        CDMW_MUTEX_GUARD (m_recursive_mutex);
            bret = m_observers_regs.empty ();

            return bret;
        }

        bool CommandEventNotifier::is_running ()
        {
            return m_bRunning;
        }

        bool CommandEventNotifier::add_event (CdmwPlatformMngt::CdmwCommand *
                 event) throw (InternalErrorException)
        {   
            try
            {
            CDMW_MUTEX_GUARD (m_recursive_mutex);
            CDMW_DEBUG (m_logger_pointer, "add command event");

            if (m_bMustStop)
                return false;

            set_event_key (event);
            insertEventInDataStore (event); 
            m_CommandEvQueue.push (event);
            m_beventInQueue = true;
            return true;
            }
            catch (...)
            {
            CDMW_THROW (InternalErrorException);
            }
            return false;
        }

        void CommandEventNotifier::stop () throw ()
        {
            if (m_bMustStop)
            return;
                                    
            CDMW_MUTEX_GUARD (m_recursive_mutex);
            CDMW_DEBUG (m_logger_pointer, "Stop Command Event Notifier ");
            m_bMustStop = true;
            m_beventInQueue = false;
            
            m_CommandEvQueue.destroy();
            this->join();

        }
        

        void CommandEventNotifier::run () throw ()
        {
            std::string strace;
            std::string lpszfrm;
            //TypeNotificationCallResult result=SUCCESS;
            CdmwPlatformMngt::CdmwCommand* CommandEvent = NULL;

            // SupervisionObserver_ptr to be notified
            CdmwPlatformMngt::SupervisionObserver_ptr observer_ptr = CdmwPlatformMngt::SupervisionObserver::_nil();
        
            if (!m_bRunning)
            m_bRunning = true;

            while (m_bRunning)
            { 
            std::string selement_path = "";
            std::string selement_path_chk = "";
            try
            {
                CommandEvent = 0;
                if (m_bMustStop)
                break;
                // Get the Event
                CommandEvent = m_CommandEvQueue.pop ();
                CDMW_DEBUG (m_logger_pointer,
                    "Command Event Notifier Event on Queue !");

                // make a copy of the observers in order to release the mutex
                m_recursive_mutex.lock ();
                /*ObserverElementMap */
                MapObsForAll observers_regs (m_observers_regs);
                m_recursive_mutex.unlock ();

                // get element_path affected
                selement_path = CommandEvent->element_path();
                // find map of registered observer for this element
                int observer_entries =
                observers_regs.count (selement_path);
                MapObsForAll::iterator it_for_all =
                observers_regs.find (selement_path);

                if (it_for_all != observers_regs.end ())
                {
                for (int j = 0; j < observer_entries; ++j)
                {
                    observer_ptr = it_for_all->second.observer_ptr;    
                    
                    if (CORBA::is_nil(observer_ptr))
                    {
                        
                        CDMW_WARN ( m_logger_pointer, "Notification of observer "<< it_for_all->second.observer_name << " failed. The observer entry is nil. The observer will be removed." );
                            //unregister_observer(it_for_all->second.observer_name, selement_path);

                        unregister_observer_for_all_element_paths(it_for_all->second.observer_name, true);
               
                    }
                    else
                    {
                    
                       // Notify 
                       if (do_notification_call (observer_ptr, CommandEvent) != SUCCEED)
                       {
                            // The Observer has crashed, remove the entry
                         CDMW_WARN ( m_logger_pointer, "Notification of observer "<< it_for_all->second.observer_name << " failed. The observer entry has been removed." );
                         //       unregister_observer(it_for_all->second.observer_name, selement_path);

                         unregister_observer_for_all_element_paths(it_for_all->second.observer_name, true);

                         std::string msg ("Removing failed observer : ");
                         msg += it_for_all->second.observer_name;
                         msg += "  The observer entry has been removed.";

                         LogMngr::logMessage(INF, msg.c_str());
                         //   CORBA::release(observer_ptr);

                       }
                       
                    }
                    it_for_all++;        
                }
                }
        
    
                // remove event from datastore
                // get header
                CdmwPlatformMngtBase::EventHeader event_header =
                CommandEvent->header ();
                // request removing
                removeEventInDataStore (event_header.event_key);

                // deallocates the event
                CommandEventDeallocator::deallocate (CommandEvent);
        
                CDMW_MUTEX_GUARD (m_recursive_mutex);
                // synchronize the observer map, realease the unregistered observer
                for (MapObsForAll::iterator it_my_map = observers_regs.begin(); it_my_map != observers_regs.end(); it_my_map++)
                {
                    MapObsForAll::iterator it_map = m_observers_regs.find (it_my_map->first);
                    if (it_map == m_observers_regs.end ())
                    {
                        CORBA::release(it_my_map->second.observer_ptr);
                    }
                }
            }
            catch (const MessageQueueDestroyedException&)
            {
                m_bMustStop = true;
            }
            catch (CORBA::SystemException &)
            {
                CDMW_DEBUG (m_logger_pointer, "System Exception...");
                CommandEventDeallocator::deallocate (CommandEvent);
                throw;
        }
            catch (...)
            {
                CDMW_DEBUG (m_logger_pointer, "System Exception...");
                CommandEventDeallocator::deallocate (CommandEvent);
            }
        }
    
            m_bRunning = false;

            CDMW_DEBUG (m_logger_pointer, "Command Event Notifier stopping...");
        }


        void CommandEventNotifier::set_event_key (CdmwPlatformMngt::CdmwCommand * event)
        {

            // set the event key
            CdmwPlatformMngtBase::EventKey event_key;

            // set primary key
            event_key.primary_key = m_evt_primary_key;

            // get time val
            OsSupport::OS::Timeval time_val = OsSupport::OS::get_time ();

            event_key.seconds = static_cast < unsigned long >(time_val.seconds);
            event_key.microseconds =
            static_cast < unsigned long >(time_val.microseconds);

            bool evt_inx_incr = false;

            if (event_key.seconds < m_evt_seconds)
            {
            event_key.seconds = m_evt_seconds;
            event_key.microseconds = m_evt_microsec;
            evt_inx_incr = true;
            }

            else if (event_key.seconds == m_evt_seconds)
            {
            if (event_key.microseconds < m_evt_microsec)
            {
                event_key.microseconds = m_evt_microsec;
                evt_inx_incr = true;
            }
            else if (event_key.microseconds == m_evt_microsec)
            {
                evt_inx_incr = true;
            }
            }

            // if there is no difference between previous event time stamp
            // inrement the evt index to build an unique event key
            if (evt_inx_incr)
            {
            m_evt_index++;
            }
            else
            {
            // reset the index (time is different)
            // counter index in header is already set to 0
            m_evt_index = 0;
            }

            event_key.counter_inx = m_evt_index;

            // store event key

            // get header
            CdmwPlatformMngtBase::EventHeader & event_header = event->header ();

            // set event key in header
            event_header.event_key = event_key;

            // save the current time used for index
            m_evt_seconds = event_key.seconds;
            m_evt_microsec = event_key.microseconds;
        }

        CORBA::Object_ptr CommandEventNotifier::set_timeout_policy_on (CORBA::Object_ptr obj)
        {
            // Create the timeout policy
            CORBA::Any timeout;
            CORBA::Object_var capsule;

#if CDMW_ORB_VDR == orbacus
            CORBA::PolicyList policies (1);
            timeout <<= (CORBA::ULong) m_callTimeout;
            policies.length (1);
            policies[0] =
            m_the_orb->create_policy (OB::TIMEOUT_POLICY_ID, timeout);
            // Set the policy
            capsule =
            m_the_orb->_set_policy_overrides (policies, CORBA::ADD_OVERRIDE);

#elif CDMW_ORB_VDR == tao
            // Timeout success
            TimeBase::TimeT _timeout = ((unsigned long) (m_callTimeout * 1.0e4));
            CORBA::PolicyList policy_list;

            timeout <<= _timeout;
            // Set the policy
            policy_list.length (1);
            policy_list[0] =
            m_the_orb->
            create_policy (Messaging::RELATIVE_RT_TIMEOUT_POLICY_TYPE, timeout);

            CDMW_DEBUG (m_logger_pointer, "call obj->_set_policy_overrides");
            capsule =
            obj->_set_policy_overrides (policy_list, CORBA::SET_OVERRIDE);
#endif

            return capsule._retn ();
        }

        BoundSyncCallReturn   CommandEventNotifier::do_notification_call (
                                                                 CdmwPlatformMngt::SupervisionObserver_ptr observer_ptr,
                                                                 CdmwPlatformMngt::CdmwCommand* CommandEvent) throw ()
        {
            CDMW_DEBUG (m_logger_pointer, "DoNotificationCall");
            BoundSyncCall *notifCall =
            m_boundSyncCallFactory->createEventNotificationCall (observer_ptr,
                                         CommandEvent,
                                         m_callTimeout);
            // performs the notification via the
            // bound synchronous call
            std::string strEvent = CommandEvent->to_string();
            BoundSyncCallReturn ret = notifCall->execute ();

            delete notifCall;
            return ret;
        }



        BoundSyncCall* CommandEventNotifier::createEventNotificationCall (
                                                               CdmwPlatformMngt::SupervisionObserver_ptr   observer,
                                                               CdmwPlatformMngt::CdmwCommand*   event)
                                                                    throw (OutOfMemoryException)
        {
            BoundSyncCall *notifCall =
            m_boundSyncCallFactory->createEventNotificationCall (observer,
                                         event,
                                         m_callTimeout);
            return notifCall;
        }

        void CommandEventNotifier::unregister_observer_for_all_element_paths (const std::string& observer_name, bool propagate)
                                    throw (BadParameterException, 
                                            CdmwPlatformMngt::ObserverNotFound)
        {
            {
                  CDMW_MUTEX_GUARD (m_recursive_mutex);

                  MapObsForAll::iterator it_for_elem_paths = m_observers_regs.begin(); 

                  bool found = false;

                  while (it_for_elem_paths != m_observers_regs.end ())
                  {
                        if ( it_for_elem_paths->second.observer_name == observer_name)
                        {
                            //CORBA::release(it_for_elem_paths->second.observer_ptr);
 
                            MapObsForAll::iterator tmp_it = it_for_elem_paths;
                            it_for_elem_paths++;

                             m_observers_regs.erase (tmp_it);
                             found = true;
                        }
                        else 
                        {
                             it_for_elem_paths++;
                        }

                  }
             }
             //if (false == found) throw CdmwPlatformMngt::ObserverNotFound();

             if (propagate)
             {
                 EventNotifierManager* notifiers_manager = EventNotifierManager::Instance();

                 StatusEventNotifier* sta_notifier = notifiers_manager->get_status_notifier();

                 try
                 {
                     sta_notifier->
                         unregister_observer_for_all_element_paths (observer_name, false);
                 }
                 catch (const BadParameterException& exc_param)
                 {
                     CDMW_WARN (m_logger_pointer, " CommandEventNotifier::unregister_observer_for_all_element_paths : " 
                         << observer_name  << " Propagating unregistration to Status Notif BadParameterException caught " );
                 }
                 catch (const CdmwPlatformMngt::ObserverNotFound exc_found)
                 {
                     CDMW_WARN (m_logger_pointer," CommandEventNotifier::unregister_observer_for_all_element_paths : " 
                         << observer_name  << " Propagating unregistration to Status Notif ObserverNotFound Exception caught ");
                 }
                 catch (...)
                 {
                     CDMW_WARN (m_logger_pointer, " CommandEventNotifier::unregister_observer_for_all_element_paths : " 
                         << observer_name  << " Propagating unregistration to Status Notif Unknown Exception caught "); 
                 }

                 PropertyEventNotifier* prop_notifier = notifiers_manager->get_property_notifier();

                 try
                 {
                     prop_notifier->
                         unregister_observer_for_all_element_paths (observer_name, false);
                 }
                 catch (const BadParameterException& exc_param)
                 {
                      CDMW_WARN (m_logger_pointer, " CommandEventNotifier::unregister_observer_for_all_element_paths : " 
                          << observer_name  << " Propagating unregistration to Property Notif BadParameterException caught ");
                 }
                 catch (const CdmwPlatformMngt::ObserverNotFound& exc_found)
                 {
                      CDMW_WARN (m_logger_pointer, " CommandEventNotifier::unregister_observer_for_all_element_paths : " 
                          << observer_name  << " Propagating unregistration to Property Notif ObserverNotFound Exception caught ");
                 }
                 catch (...)
                 {
                      CDMW_WARN (m_logger_pointer, " CommandEventNotifier::unregister_observer_for_all_element_paths : " 
                          << observer_name  << " Propagating unregistration to Property Notif Unknown Exception caught "); 
                 }
             }

        }

    }                // End namespace PlatformMngt
}                // End namespace Cdmw

