/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#include <SystemMngt/platformlibrary/LogMngr.hpp>
#include <Foundation/logging/LogManager.hpp>
#include <Foundation/common/Assert.hpp>
#include <Foundation/common/Exception.hpp>
#include <Foundation/ossupport/SystemProperties.hpp>

#include <SystemMngt/idllib/CdmwPlatformMngtBase.stub.hpp>

namespace Cdmw
{
namespace PlatformMngt
{


/**
* Contains the LogMngr singleton 
*/
LogMngr* LogMngr::M_pSingleton = NULL;

LogMngr::LogMngr(int& argc, char** argv)
   throw(OutOfMemoryException, InternalErrorException)
{
   try {
      // Init the SystemProperties
      (void*) Cdmw::OsSupport::SystemProperties::Init(argc, argv);
   } catch (const BadParameterException & e) {
      CDMW_THROW1(InternalErrorException,e.what());
   }
   Cdmw::Logging::LogManager::Init(argc, argv);
}

void LogMngr::destroy()
{
    // close the log file if openned
    //close ();
    
    // delete singleton
    if (M_pSingleton != NULL)
    {
        delete M_pSingleton;
        M_pSingleton = NULL;
    }
}

void LogMngr::logMessage(const char *message) throw()
{
    // This operation is usually used to log Event.
    // retreive level and issuer if it is the same than the logger from the 
    // Event
    std::string msg_str(message);
    std::string filtered_msg;
    size_t level_pos = 0;
    size_t issuer_pos = msg_str.find(M_pSingleton->getService());
    if (issuer_pos != msg_str.npos)
        issuer_pos += strlen(M_pSingleton->getService()) + 1;
    
    if ((level_pos = msg_str.find(CdmwPlatformMngtBase::INF_STR)) != 
             msg_str.npos)
    {
        filtered_msg.append(msg_str, 0, level_pos);
        // if the event issuer is different to the logger let it in the message
        if (issuer_pos == msg_str.npos)
            issuer_pos = level_pos + strlen(CdmwPlatformMngtBase::INF_STR) + 1;
        filtered_msg.append(msg_str, issuer_pos, msg_str.npos);
        GetLogger()->info(filtered_msg.c_str(), "", -1);
    }
    else if ((level_pos = msg_str.find(CdmwPlatformMngtBase::WRN_STR)) != 
             msg_str.npos)
    {
        filtered_msg.append(msg_str, 0, level_pos);
        // if the event issuer is different to the logger let it in the message
        if (issuer_pos == msg_str.npos)
            issuer_pos = level_pos + strlen(CdmwPlatformMngtBase::WRN_STR) + 1;
        filtered_msg.append(msg_str, issuer_pos, msg_str.npos);
        GetLogger()->warn(filtered_msg.c_str(), "", -1);
    }
    else if ((level_pos = msg_str.find(CdmwPlatformMngtBase::ERR_STR)) != 
             msg_str.npos)
    {
        filtered_msg.append(msg_str, 0, level_pos);
        // if the event issuer is different to the logger let it in the message
        if (issuer_pos == msg_str.npos)
            issuer_pos = level_pos + strlen(CdmwPlatformMngtBase::FTL_STR) + 1;
        filtered_msg.append(msg_str, issuer_pos, msg_str.npos);
        GetLogger()->error(filtered_msg.c_str(), "", -1);
    }
    else if ((level_pos = msg_str.find(CdmwPlatformMngtBase::FTL_STR)) != 
             msg_str.npos)
    {
        filtered_msg.append(msg_str, 0, level_pos);
        // if the event issuer is different to the logger let it in the message
        if (issuer_pos == msg_str.npos)
            issuer_pos = level_pos + strlen(CdmwPlatformMngtBase::FTL_STR) + 1;
        filtered_msg.append(msg_str, issuer_pos, msg_str.npos);
        GetLogger()->fatal(filtered_msg.c_str(), "", -1);
    }
    else
        GetLogger()->debug(message, "", -1);
}

void LogMngr::logMessage(LogType logType, const char *message) throw()
{
    try
    {
        switch (logType)
        {
            case WRN:
                GetLogger()->warn(message, "", -1);
                break;

            case INF:
                GetLogger()->info(message, "", -1);
                break;

            case ERR:
                GetLogger()->error(message, "", -1);
                break;

            case FTL:
                GetLogger()->fatal(message, "", -1);
                break;
        }
    }
    catch(const std::bad_alloc&)
    {
        logOutOfMemory(logType, message);
    }
}

void LogMngr::logMessage(LogType logType, long messageId) throw()
{
    try
    {
        std::string message = getFormattedMessage(messageId);

        logMessage(logType, message.c_str());
    }
    catch(const std::bad_alloc&)
    {
        logOutOfMemory(logType);
    }
}


std::string LogMngr::getSafeMessage(long messageId)
{
    std::string safeMessage;
    const char *message = M_pSingleton->getMessage(messageId);

    if (message != NULL)
    {
        safeMessage = message;
    }
    else
    {
        std::ostringstream temp;
        temp << "<<< Invalid message id (" << messageId << ") >>>" << std::ends;

        safeMessage = temp.str();
    }

    return safeMessage;
}

void LogMngr::logOutOfMemory(LogType logType) throw()
{
    try
    {
        GetLogger()->fatal("<<< No memory for proper log message >>>", "", -1);
    }
    catch(...)
    {
    }
}

void LogMngr::logOutOfMemory(LogType logType, const char* message) throw()
{
    try
    {
        GetLogger()->fatal(message, "", -1);
    }
    catch(...)
    {
    }
}






} // End namespace PlatformMngt
} // End namespace Cdmw


