/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#include "SystemMngt/platformlibrary/SupervisionEventNotifier.hpp"
#include "SystemMngt/platformlibrary/LogMngr.hpp"
#include "SystemMngt/platformlibrary/Configuration.hpp"

#include "Foundation/common/Exception.hpp"
#include "Foundation/common/Assert.hpp"
#include "Foundation/commonsvcs/datastore/StorageErrorException.hpp"
#include <Foundation/orbsupport/OrbSupport.hpp>

#include <string>

namespace Cdmw
{
namespace PlatformMngt
{


SupervisionEventNotifier::SupervisionEventNotifier(unsigned long callTimeout)
    throw (InternalErrorException)
    : m_callTimeout(callTimeout)
{
    m_logger = Cdmw::Logging::Logger::Get_Logger(CdmwPlatformMngtBase::PFMMNG);
    try
    {
        m_boundSyncCallFactory = BoundSyncCallFactory::getFactory();
    }
    catch(const Cdmw::Exception&)
    {
        CDMW_THROW(InternalErrorException);
    }

     m_timeout_inserter = TimeoutInserter::Instance();

    try
    {
        Cdmw::CommonSvcs::DataStore::DataStoreBase* ds =
            Cdmw::CommonSvcs::DataStore::DataStoreRegistry::Get_data_store(
                   Configuration::Get_event_notifier_datastore_id());
        m_event_notif_ds = 
                 dynamic_cast<Cdmw::PlatformMngt::EventNotifierDataStore*>(ds);
    }
    catch (const NotExistException& e)
    {
        CDMW_ASSERT(false);
    }

    m_scheduler = new scheduler_type
     (Cdmw::OsSupport::SystemProperties::Get_Property(NOTIFICATION_COORDINATOR_NOTIFIER_NB_THREADS_PROP,
                                        NOTIFICATION_COORDINATOR_NOTIFIER_DEFAULT_NB_THREADS) );
    m_scheduler->start();
}


SupervisionEventNotifier::~SupervisionEventNotifier() throw()
{
    m_scheduler->stop(true);
    delete m_scheduler;
    m_scheduler = 0;
}

/*
SupervisionEventNotifier::SupervisionEventNotifier(const SupervisionEventNotifier& rhs)
{

}


SupervisionEventNotifier& SupervisionEventNotifier::operator=(const SupervisionEventNotifier& rhs)
{
    return *this;
}
*/

std::string SupervisionEventNotifier::eventToString(CdmwPlatformMngt::Event* event)
{
    CORBA::String_var str = event->to_string();

    std::string eventStr(str.in());

    return eventStr;
}


BoundSyncCall* SupervisionEventNotifier::createEventNotificationCall(
    CdmwPlatformMngt::SupervisionObserver_ptr observer,
    CdmwPlatformMngt::Event* event)
    throw(OutOfMemoryException)
{

    BoundSyncCall* notifCall = 
        m_boundSyncCallFactory->createEventNotificationCall(observer, event, m_callTimeout);
    return notifCall;

}


void SupervisionEventNotifier::insertEventInDataStore (CdmwPlatformMngt::Event* event)
{
    if (Configuration::Is_primary())
    {
        // update the datastore
        
        // get the event key
        CdmwPlatformMngtBase::EventHeader& event_header = event->header();
            
        try
        {            
            m_event_notif_ds->insert(event_header.event_key, event);
        }
        catch (const Common::AlreadyExistException&)
        {
            CDMW_WARN(m_logger,
                      "event notif record already exists in datastore : " 
                       << event_header.event_key.primary_key << ":"
                       << event_header.event_key.seconds << ":" 
                       << event_header.event_key.microseconds << ":" 
                       << event_header.event_key.counter_inx);
                      
            // CDMW_ASSERT(false);
        }
        catch (const CommonSvcs::DataStore::StorageErrorException &exc_storage) 
        {
            CDMW_FATAL(m_logger,
                        "event notif record Storage Error exception in insertion : " 
                        << event_header.event_key.primary_key << ":"
                        << event_header.event_key.seconds << ":" 
                        << event_header.event_key.microseconds << ":" 
                        << event_header.event_key.counter_inx
                << exc_storage.what());
                CDMW_ASSERT(false);
        }
        catch (...)
        {

            CDMW_FATAL(m_logger,
                      "event notif record unexpected exception in insertion : " 
                      << event_header.event_key.primary_key << ":"
                      << event_header.event_key.seconds << ":" 
                      << event_header.event_key.microseconds << ":" 
                      << event_header.event_key.counter_inx);
              
            CDMW_ASSERT(false);      
        }
    }
}

    
void SupervisionEventNotifier::removeEventInDataStore (CdmwPlatformMngtBase::EventKey event_key)
{
    if (Configuration::Is_primary())
    {
        try
        {
            m_event_notif_ds->remove(event_key);
        }
        catch (const Common::NotFoundException&)
        {
        }
        catch (const CommonSvcs::DataStore::StorageErrorException &)
        {
        }
    }
}

void SupervisionEventNotifier::run () throw ()
{
  bool stopped = false;
  m_owner = true;
  
  LogMngr::logMessage(INF, MSG_ID_EVENT_NOTIF_RUNNING);
  
  typedef std::map<std::string, CdmwPlatformMngt::SupervisionObserver_ptr> Observers_type;
  
  do
    {
      {
        CDMW_MUTEX_GUARD (m_recursive_mutex);
        stopped = m_stopped;
      }
      
      if (stopped)
        break;
      
      CdmwPlatformMngt::Event *event = NULL;
      
      try
        {
          event = m_eventQueue.pop();
          
          // gets the string description of the event
          std::string eventStr = eventToString(event);
          
          // make a copy of the observers in order to release the mutex
          m_recursive_mutex.lock();
          Observers_type observers(m_observers);
          m_recursive_mutex.unlock();
          
          //std::cerr << __FILE__ << __LINE__ << observers.size() << std::endl;
          int observers_number = observers.size();
                    
          if (observers_number > 0 )
          {
          NotificationCoordinator * coordinator__ = new NotificationCoordinator(observers_number);
          
          for (Observers_type::iterator it =
                 observers.begin(); it != observers.end(); ++it)
            {
              const std::string& observerName(it->first);
              CdmwPlatformMngt::SupervisionObserver_ptr observer = it->second;
              
              if (CORBA::is_nil(observer))
                {           
                  unregisterObserver(observerName);
                  
                  std::string msg ("Notification of observer : ");
                  msg += observerName;
                  msg += "failed. The observer entry is nil.";
                  LogMngr::logMessage(WRN, msg.c_str());
                } 
              else
                {
                  
                  try
                    {

                    m_scheduler->enqueue_event(observer,
                                               observerName,
                                               event,
                                               coordinator__);

                    /*                               
                      BoundSyncCall* notifCall =
                        createEventNotificationCall(observer, event);
                      
                      // performs the notification via the
                      // bound synchronous call
                      BoundSyncCallReturn ret = notifCall->execute();
                      switch (ret)
                        {
                        case SUCCEED:
                          {
                            //LogMngr::logMessage(INF, MSG_ID_EVENT_NOTIF_SUCCESS,
                            //    observerName, eventStr);
                          }
                          break;

                        case TIMEOUT:
                          {
                            LogMngr::logMessage(WRN, MSG_ID_EVENT_NOTIF_TIMEOUT,
                                                observerName, eventStr);
                          }
                          break;

                        case EXCEPTION:
                          {
                            const char *exceptionName =
                              notifCall->getExceptionName();
                            std::string marshal_exception("MARSHAL");
                            if (exceptionName != marshal_exception)
                              {
                                LogMngr::logMessage(WRN, MSG_ID_EVENT_NOTIF_EXCEPTION,
                                                    exceptionName, observerName, eventStr);
                                                     
                                unregisterObserver(observerName);
    
                                std::string msg ("Notification of observer : ");
                                msg += observerName;
                                msg += "failed. The observer entry has been removed.";
                                LogMngr::logMessage(WRN, msg.c_str());
                                         
                              } 
                            else
                              {
                                    
                                std::string msg ("Special exception received : ");
                                msg += exceptionName;
                                msg += ", from observer : ";
                                msg += observerName;
                                msg += ". Observer not removed.";
                                LogMngr::logMessage(WRN, msg.c_str());
                              }
                          }
                          break;
                        }

                      delete notifCall;*/

                    }
                  catch (const OutOfMemoryException&)
                    {
                      // due to the allocation of notifCall
                      // the event is lost for the observer
                      LogMngr::logMessage(ERR, MSG_ID_EVENT_NOTIF_NOTIF_LOST, observerName,
                                          eventStr);
                    }

                }

            }

          coordinator__->wait_for_all();

          while ( coordinator__->_are_failed_observer() )
          {
                std::string failed_obs = coordinator__->_pop_failed_observer();

                if ("" != failed_obs)
                {
                    unregisterObserver(failed_obs);

                    std::string msg ("Removing failed observer : ");
                    msg += failed_obs;
                    msg += "  The observer entry has been removed.";

                    LogMngr::logMessage(INF, msg.c_str());
                }
          }

          coordinator__->_remove_ref();

          } //end of scope for coordinator__
          
          // remove event from datastore
                    
          // get header
          CdmwPlatformMngtBase::EventHeader event_header = event->header();
                    
          // request removing
          removeEventInDataStore (event_header.event_key);
            
          // deallocates the event
          SupervisionEventDeallocator::deallocate(event);

        }
      catch (const MessageQueueDestroyedException&)
        {
          stopped = true;
        }
      catch (const std::bad_alloc&)
        {
          // due to the allocation of eventStr
          // or the allocation of observers
          // => the event is lost for all observers
          LogMngr::logMessage(ERR, MSG_ID_EVENT_NOTIF_EVENT_LOST);
          SupervisionEventDeallocator::deallocate(event);
        }
      catch ( ... )
        {
          // due to an unexpected exception
          // => the event is lost for all observers
          LogMngr::logMessage(ERR, MSG_ID_EVENT_NOTIF_UNEXP_EXCEPT);
          SupervisionEventDeallocator::deallocate(event);
        }
    }
  while (!stopped);

}

CdmwPlatformMngt::SupervisionObserver_ptr SupervisionEventNotifier::registerObserver(const std::string& observerName, 
     CdmwPlatformMngt::SupervisionObserver_ptr observer)
            throw (BadParameterException, EventNotifierStoppedException, OutOfMemoryException)
{

      if (CORBA::is_nil(observer) || (!m_timeout_inserter->is_orb_attached()))
      {
          //Observer not Registered throw an Exception
          CDMW_THROW2 (BadParameterException, "SupervisionEventNotifier::registerObserver", "Observer Not registered");
      }
      
      CORBA::Object_var capsuled_obj_var = m_timeout_inserter->capsule_timeout(observer, m_callTimeout);
      CdmwPlatformMngt::SupervisionObserver_var observer_var = CdmwPlatformMngt::SupervisionObserver::_nil();
                    try
                    {
                    observer_var = CdmwPlatformMngt::SupervisionObserver::_narrow(capsuled_obj_var.in());
                    }
                    catch (const CORBA::SystemException&)
                    {
                    std::cerr << "SupervisionObserver is NIL" << std::endl;
                          //Observer not Registered throw an Exception
                          CDMW_THROW2 (BadParameterException, "SupervisionEventNotifier::registerObserver", "Observer Not registered");

                    }

      return EventNotifier<CdmwPlatformMngt::Event, SupervisionEventDeallocator,CdmwPlatformMngt::SupervisionObserver_ptr, 
           CdmwPlatformMngt::SupervisionObserver>::registerObserver(observerName, observer_var.in());

}


} // End namespace PlatformMngt
} // End namespace Cdmw

