/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#include "SystemMngt/platformlibrary/LogMngr.hpp"
#include "SystemMngt/platformlibrary/SynchronisableTimedTask.hpp"
#include "SystemMngt/platformlibrary/SynchronisableTaskFactory.hpp"

#include <sstream>

namespace Cdmw
{
namespace PlatformMngt
{

// ----------------------------------------------------------------------
// SynchronisableTaskTemporiser class.
// ----------------------------------------------------------------------

SynchronisableTimedTask::SynchronisableTimedTask ( unsigned long timeout, const std::string& taskId,
                                                   Sequencer* sequencer, SynchronisableTaskFactory* synchTaskFactory,
                                                   TaskObserver* observer,size_t predecessorCount)
             : SynchronisableTask (taskId, sequencer, observer, predecessorCount) 
{
    //Initialising the variables
    m_timeout = timeout;
    
    //Creating the Observer
    std::auto_ptr <SynchronisableTimedTaskObserver> 
              pObserver (new SynchronisableTimedTaskObserver(taskId, sequencer, synchTaskFactory));
              
    m_observer= pObserver.release();
    
    //Creating the Timer    
    std::auto_ptr <SingleShotTimer> 
              pTimer (new SingleShotTimer(m_observer,m_timeout)); 
    
    m_timer = pTimer.release();
    
    //Starting the Timer
    m_timer->start();
}

SynchronisableTimedTask::~SynchronisableTimedTask() throw() 
{
    try
    {
    //Stopping the timer
    m_timer->stop();
    }catch(const AlreadyDoneException&)
    {
        LogMngr::logMessage(ERR,"Timer already Stopped. Should never happen!!");
    }
    
    //Deleting the observer and the Timer.
    delete m_observer;
    m_observer = 0;
    
    delete m_timer;
    m_timer = 0;
}


void SynchronisableTimedTask::startTask() throw()
{
    if (m_nbPredecessorsToWait > 1)
    {
        --m_nbPredecessorsToWait;
        LogMngr::logMessage(INF,"The Task is waiting for the others predecessors end");
    }
    else
    {
        try
        {
            //Starting the timer
            m_timer->start_timer();
        }
        catch(const AlreadyDoneException&)
        {
            LogMngr::logMessage(ERR,"Timer already Started. Should never happen!!");
        }
        
        LogMngr::logMessage(INF,"The Timer attached to the task is started");
        
        //Call the method Task::start();
        start();
    }
}


bool SynchronisableTimedTask::isExpired() throw()
{
    bool tempExp = false;
    
    try
    {
         tempExp = m_observer->expired;
         
    }catch(...)
    {
        LogMngr::logMessage(ERR,"SynchronisableTimedTask::isExpired() : error");
    }
    return tempExp;
}


// ----------------------------------------------------------------------
// SynchronisableTimedTaskObserver class.
// ----------------------------------------------------------------------

SynchronisableTimedTaskObserver::SynchronisableTimedTaskObserver (const std::string& id,
                                   Sequencer* sequencer,
                                   SynchronisableTaskFactory* syncTaskFactory)
{
    //Initialising the variables
    expired = false;
    m_taskId = id;
    m_sequencer = sequencer;
    m_taskFactory = syncTaskFactory;
            
}

void SynchronisableTimedTaskObserver::wake_up() throw ()
{
    //When the timeout expired, the successors task are started
    expired = true;
    
     try
    {
        if (m_taskFactory->hasSuccessors(m_taskId))
        {
            //Get successors
            TaskIdList successorsId = m_taskFactory->getSuccessorsId(m_taskId);
            
            LogMngr::logMessage(INF,"The Timer is expired. Starting Successors Applications");
            //Add tasks to Start
            m_sequencer->addTasksToStart(successorsId);       
        }
    }
    catch(const InvalidTaskIdException&)
    {
        LogMngr::logMessage(FTL,
                "SynchronisableTimedTaskObserver error due to invalid task id exception"); 
    }
    catch (const IncompatibleStatusException&)
    {
        LogMngr::logMessage(FTL,
                "SynchronisableTimedTaskObserver error due to IncompatibleStatusException exception"); 
    }
    catch (const OutOfMemoryException&)
    {
        LogMngr::logMessage(FTL,
                "SynchronisableTimedTaskObserver error due to OutOfMemoryException exception"); 
    }
    catch(...)
    {
        LogMngr::logMessage(FTL,
                "SynchronisableTimedTaskObserver unexpected exception"); 
    }
    
} 

} // End namespace PlatformMngt
} // End namespace Cdmw

