/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#include <iostream>
#include <sstream>
#include <string>

#include "platformperiodicnotifier/Notifier_impl.hpp"
#include "SystemMngt/tools/ManagedElementKey.hpp"
#include "Foundation/logging/FunctionLogger.hpp"
#include "Foundation/common/Exception.hpp" 
#include "Foundation/orbsupport/ExceptionMinorCodes.hpp"  
#include "Foundation/commonsvcs/datastore/StorageErrorException.hpp"

#include "SystemMngt/idllib/CdmwPlatformMngtPredefinedProperties.stub.hpp" 

namespace {
    
    const char* const LOGGER_NAME = "CDMW.PFMMNG.Periodic.Notifier";
    
    Cdmw::Logging::Logger* Get_Logger () {
        static Cdmw::Logging::Logger* logger =
            Cdmw::Logging::Logger::Get_Logger(LOGGER_NAME);
        return logger;
    }
}

namespace Cdmw
{ // Begin namespace Cdmw
    
namespace PlatformMngt
{ // Begin namespace PlatformMngt

namespace Periodic
{ // Begin namespace Periodic
    
    
/*
* Unique ID for periodic notifier
*/
std::string Notifier_impl::M_periodic_notifier_id = "CdmwPlatformMngtPeriodicNotifier";


Notifier_impl::Notifier_impl(CORBA::ORB_ptr orb,
                            CdmwPlatformMngt::System_ptr system)
throw (Cdmw::OutOfResourcesException)
{   
    //get logger instance
    m_logger_pointer =
    Cdmw::Logging::Logger::Get_Logger (CdmwPlatformMngtBase::PFMMNG);
    CDMW_ASSERT (m_logger_pointer != 0);

    CDMW_DEBUG (m_logger_pointer, "Periodic Notifier Instance ");     
    m_system = CdmwPlatformMngt::System::_duplicate(system);
    
    m_pd_observers_infos = new CdmwPlatformMngt::PeriodicObserverRecords();
    m_pd_observers_infos->length(0);
    
    m_periodic_observers_map.clear();
    //m_timer_obs = 0;
    //become on first periodic observer registration
    m_ds_defined = false;
    
    try
    {
        //create a PeriodicObserver Datastore
        Cdmw::CommonSvcs::DataStore::DataStoreBase * ds =
        Cdmw::CommonSvcs::DataStore::DataStoreRegistry::
        Get_data_store (Configuration::Get_periodic_observer_datastore_id ());

        m_periodic_observer_ds =
        dynamic_cast < Cdmw::PlatformMngt::PeriodicObserverDataStore * >(ds);

    }
    catch (const NotExistException & e)
    {
        CDMW_ASSERT (false);
    }

    try 
    {
        m_orb = CORBA::ORB::_duplicate(orb);
        
    } catch (...) 
    {
        CDMW_THROW(Cdmw::OutOfResourcesException);
    }
}

            
Notifier_impl::~Notifier_impl () throw ()
{
    m_pd_observers_infos->length(0);

    CDMW_MUTEX_GUARD (m_mutex_condition);

    MapObs::iterator it = m_periodic_observers_map.begin();

    while(it != m_periodic_observers_map.end())
    {
        try
        {
            TimerObserver* m_timer_obs = it->second;
            delete m_timer_obs;
        
        }catch (...)
        {
        } 
    it++;
    }
}


void Notifier_impl::register_observer (const char * observer_name, 
                                    const CosPropertyService::PropertyNames & property_names,
                                    CdmwPlatformMngt::Periodic::Observer_ptr observer,
                                    TimeBase::TimeT period)
    throw (CORBA::SystemException,
        CdmwPlatformMngt::ElementAlreadyExists,
        CdmwPlatformMngt::PropertyNameInvalid)
{
    CDMW_LOG_FUNCTION(::Get_Logger(), "");
    CDMW_MUTEX_GUARD (m_mutex_condition);
    /*********************************************************
    * Check first if the observer has already been registered
    * *******************************************************/
    bool bool_ret = false;
        
    // return false if no key has the observer_name in its value
    bool_ret = check_periodic_obs_name_in_map (observer_name, m_periodic_observers_map);
    
    //if the element is present throw an Element Already Present Exception 
    if (true == bool_ret)
    {
        throw CdmwPlatformMngt::ElementAlreadyExists();
    }
    
    if (CORBA::is_nil (observer))
    {
        LogMngr::logMessage(ERR,"Periodic Notifier :: Observer reference is null");
        throw CORBA::BAD_PARAM (OrbSupport::BAD_PARAM, CORBA::COMPLETED_NO);      
    } 

    CdmwPlatformMngt::Periodic::ObserverMIOPGroup_var miop_obj =
        CdmwPlatformMngt::Periodic::ObserverMIOPGroup::_nil ();
    
    /****************************
    * Creating the TimerObserver
    * **************************/
    try
    {
        // costructor could throw PropertyNameInvalid exception 
        std::auto_ptr <TimerObserver>
            pObserver (new TimerObserver(miop_obj.in(),
                                        observer,
                                        m_system.in(), 
                                        period, 
                                        property_names)); 
        
        TimerObserver* m_timer_obs = pObserver.release(); 

        try
        {
            m_timer_obs->start_timer();
            
        }catch (const Cdmw::AlreadyDoneException& )
        {
            LogMngr::logMessage(ERR,"Periodic Notifier :: Can not start timer...");
        }
        
        // if the element is not present go on
        m_periodic_observers_map.insert (MapObs::value_type (observer_name, m_timer_obs));
        //print_map(m_periodic_observers_map);  
        
        // save the single periodic observer's info into member variable 
        set_observer_info(observer_name, 
                        property_names,
                        observer,
                        period);
        
        /********************************************
        * set the record in datastore if it's primary
        * ******************************************/ 
        set_observer_record_in_datastore(); 
        
        
    }catch (const CdmwPlatformMngt::PropertyNameInvalid & ex)
    {    

        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw ex;
    }
    catch (const std::bad_alloc &)
    {
        throw CORBA::NO_MEMORY (OrbSupport::NO_MEMORY, CORBA::COMPLETED_NO);
    }
}


void Notifier_impl::define_ds()
{
    //first periodic observer registration
    //create the associated record in datastore
    if (!m_ds_defined)
    {
        m_ds_defined = true;
    }

}   

void Notifier_impl::register_observer_group (const char * observer_name, 
                                            const CosPropertyService::PropertyNames &  property_names,
                                            const char * miop_group,
                                            TimeBase::TimeT period)
    throw (CORBA::SystemException, 
        CdmwPlatformMngt::ElementAlreadyExists, 
        CdmwPlatformMngt::PropertyNameInvalid)
{
    CDMW_LOG_FUNCTION(::Get_Logger(), "");
    CDMW_MUTEX_GUARD (m_mutex_condition);
    /*********************************************************
    * Check first if the observer has already been registered
    * *******************************************************/
    bool bool_ret = false;
        
    // return false if no key has the observer_name in its value
    bool_ret = check_periodic_obs_name_in_map (observer_name, m_periodic_observers_map);

    // if the element is present throw an Element Already Present Exception 
    if (true == bool_ret)
    {
        throw CdmwPlatformMngt::ElementAlreadyExists();
    }
    
    string group_str (miop_group);
    if  (group_str.empty())
    {
       
        LogMngr::logMessage(ERR,"Periodic Notifier :: Miop Group is null");
        throw CORBA::BAD_PARAM (OrbSupport::BAD_PARAM, CORBA::COMPLETED_NO);         
    } 
    
    CdmwPlatformMngt::Periodic::Observer_var observer =
        CdmwPlatformMngt::Periodic::Observer::_nil ();

    
    /************************************
    * Create ObserverMIOPGroup reference
    * *********************************/
    CdmwPlatformMngt::Periodic::ObserverMIOPGroup_var miop_obj;
    try
    {
        miop_obj = 
            CdmwPlatformMngt::Periodic::ObserverMIOPGroup::_duplicate (create_miop_ref(miop_group));
        
    } catch (const BadParameterException & ex)
    {
        LogMngr::logMessage(ERR,"Periodic Notifier :: Cannot create miop corbaloc");
        throw CORBA::BAD_PARAM (OrbSupport::BAD_PARAM, CORBA::COMPLETED_NO);  
    }

    /****************************
    * Creating the TimerObserver
    * **************************/

    try
    {    
        std::auto_ptr <TimerObserver> 
                pObserver (new TimerObserver(miop_obj.in(),
                                            observer.in(),
                                            m_system.in(),
                                            period, 
                                            property_names));           
        TimerObserver* m_timer_obs = pObserver.release();

        try
        {
            m_timer_obs->start_timer();
            
        }catch (const Cdmw::AlreadyDoneException& )
        {
            LogMngr::logMessage(ERR,"Periodic Notifier :: Can not start timer...");
        }

        // if the element is not present go on
        m_periodic_observers_map.insert (MapObs::value_type (observer_name, m_timer_obs));
        //print_map(m_periodic_observers_map);

        
        // save the single periodic observer's info into member variable 
        set_observer_info(observer_name, 
                        property_names,
                        observer,
                        period, 
                        miop_group);
        
        /********************************************
        * set the record in datastore if it's primary
        *******************************************/ 
        set_observer_record_in_datastore(); 
    }
    catch (const CdmwPlatformMngt::PropertyNameInvalid &e)
    {    
            CDMW_LOG_FUNCTION_EXCEPTION(e);
            throw e;
    }
    catch (const std::bad_alloc &)
    {
        throw CORBA::NO_MEMORY (OrbSupport::NO_MEMORY, CORBA::COMPLETED_NO);
        
    }
} 


CdmwPlatformMngt::Periodic::ObserverMIOPGroup_ptr 
Notifier_impl::create_miop_ref (const char* miop_group)
throw (BadParameterException)
{
    std::string multicast_addr = CORBA::string_dup(miop_group);
    CORBA::Object_var miop_ref = m_orb->string_to_object(multicast_addr.c_str());

    CdmwPlatformMngt::Periodic::ObserverMIOPGroup_var miop_obj =
        CdmwPlatformMngt::Periodic::ObserverMIOPGroup::_unchecked_narrow (miop_ref.in());

    if (CORBA::is_nil (miop_obj))
    {
        CDMW_THROW2 (BadParameterException, "miop_obj", "NIL"); 
    }

    return miop_obj._retn();
}


CdmwPlatformMngt::Periodic::Observer_ptr 
Notifier_impl::unregister_observer (const char * observer_name)
throw (CORBA::SystemException, CdmwPlatformMngt::ObserverNotFound)
{
    CDMW_MUTEX_GUARD (m_mutex_condition);
    CdmwPlatformMngt::Periodic::Observer_var periodic_observer =
        CdmwPlatformMngt::Periodic::Observer::_nil ();

    MapObs::iterator it;

    //remove the observer from the map 
    try
    {
        it = m_periodic_observers_map.find(observer_name);
    }catch (...)
    {
        LogMngr::logMessage(ERR,"Notifier_impl::unregister_observer() - Unexpected exception raised");
        throw CORBA::INTERNAL(OrbSupport::INTERNAL, CORBA::COMPLETED_NO);
    } 

    if (it != m_periodic_observers_map.end())
    {
        try
        {
            TimerObserver* m_timer_obs = it->second;
            
            if (CORBA::is_nil(periodic_observer = CdmwPlatformMngt::Periodic::Observer::_duplicate (m_timer_obs->get_observer_ref())))
            {
                LogMngr::logMessage(FTL,"Periodic Notifier :: periodic observer reference invalid");
            }

            //delete the timer 
            delete m_timer_obs;
            m_timer_obs = 0;
            
            //cancel observer from map
            m_periodic_observers_map.erase(observer_name);
        }catch (...)
        {
            //LogMngr::logMessage(ERR,"Notifier_impl::unregister_observer() - Unexpected exception raised");
            throw CORBA::INTERNAL(OrbSupport::INTERNAL, CORBA::COMPLETED_NO);
        } 

    }
    else 
    {
        throw CdmwPlatformMngt::ObserverNotFound();
    }

    // remove observer in datastore
    remove_observer_record_in_datastore (observer_name);

    return periodic_observer._retn ();
}
        

void Notifier_impl::remove_observer_record_in_datastore(const char * observer_name)
{
    
    
    //get record to update
    CdmwPlatformMngt::PeriodicNotifierRecord_var pd_notifier_record = 
        get_record_from_datastore (); 
    
    *m_pd_observers_infos = pd_notifier_record->periodic_observers;    
    
    CORBA::ULong length = m_pd_observers_infos -> length();
    CORBA::ULong i=0;

    // find observer name
    for(i=0; i < length; ++i)
    {
        
        if( !strcmp(m_pd_observers_infos[i].observer_name, observer_name)) break;
    }

    if (i < length)
    {
        for (CORBA::ULong j=i+1; j < length; ++j) 
        {
            m_pd_observers_infos[j-1] = m_pd_observers_infos[j];
        }
        
        m_pd_observers_infos->length(length-1);
        pd_notifier_record->periodic_observers.length(length-1);
        
        pd_notifier_record->periodic_observers = *m_pd_observers_infos ;
        
        try
        {
            m_periodic_observer_ds->update (M_periodic_notifier_id, pd_notifier_record);
        }
        catch (const Common::NotFoundException&)
        {
            CDMW_FATAL(m_logger_pointer,
            "record cannot be found in datastore in " 
             << M_periodic_notifier_id );
            CDMW_ASSERT (false);
        }
        catch (const CommonSvcs::DataStore::StorageErrorException &exc_storage)
        {
            CDMW_FATAL(m_logger_pointer,
            "record storage error exception in "
            << M_periodic_notifier_id 
            << exc_storage.what());
            CDMW_ASSERT (false);
        }

    }
    else
    {
        LogMngr::logMessage(WRN,"Periodic Notifier :: remove observer record in datastore unsuccessfull");
    }

}


char * Notifier_impl::unregister_observer_group (const char * observer_name)
throw (CORBA::SystemException, CdmwPlatformMngt::ObserverNotFound)
{   
    CDMW_MUTEX_GUARD (m_mutex_condition);
    char* miop_group ;
    CdmwPlatformMngt::Periodic::ObserverMIOPGroup_var miop_obj =
        CdmwPlatformMngt::Periodic::ObserverMIOPGroup::_nil();    

    MapObs::iterator it;    

    //remove the observer's group from the map
    try
    {
        it = m_periodic_observers_map.find(observer_name);
    }catch (...)
    {
        //LogMngr::logMessage(ERR,"Notifier_impl::unregister_observer_group() - Unexpected exception raised");
        throw CORBA::INTERNAL(OrbSupport::INTERNAL, CORBA::COMPLETED_NO);
    }

    if (it != m_periodic_observers_map.end())
    {
        try
        {
            TimerObserver* m_timer_obs = it->second;

            miop_obj = CdmwPlatformMngt::Periodic::ObserverMIOPGroup::_duplicate (m_timer_obs->get_group_ref());          

            miop_group = m_orb->object_to_string(miop_obj);        

            if (0 == miop_group)
            {
                LogMngr::logMessage(FTL,"Periodic Notifier :: periodic observer group reference invalid");
            }

            //delete the timer 
            delete m_timer_obs;
            m_timer_obs = 0;

            m_periodic_observers_map.erase(observer_name);

        }catch (...)
        {
            //LogMngr::logMessage(ERR,"Notifier_impl::unregister_observer_group() - Unexpected exception raised");
            throw CORBA::INTERNAL(OrbSupport::INTERNAL, CORBA::COMPLETED_NO);
        }
        }
    else 
    {
        throw CdmwPlatformMngt::ObserverNotFound();
    }

    // remove observer in datastore
    remove_observer_record_in_datastore (observer_name);

    return miop_group;
}
                
            
char * Notifier_impl::periodic_notifier_id()
throw (CORBA::SystemException)
{
    CORBA::String_var tmp = CORBA::string_dup ((Notifier_impl::M_periodic_notifier_id).c_str()); 
    return tmp._retn ();
}
    
    
void Notifier_impl::set_observer_record_in_datastore ()
{
    //CDMW_MUTEX_GUARD (m_mutex_condition);
    
    // test if the supervision is not fault tolerant or if it is, is it the primary?
    if (!Configuration::Is_fault_tolerant ()
        || Configuration::Is_primary ())
    {
        if (Configuration::Is_synchronising ())
        {
            CDMW_INFO(m_logger_pointer, 
                    "<<Notifier_impl::set_record_in_datastore() start >> " );
        }
        
        //create apposite record to insert in datastore
        CdmwPlatformMngt::PeriodicNotifierRecord record ;
        CORBA::ULong len = m_pd_observers_infos->length();
        record.periodic_observers.length(len+1);
        
        record.periodic_observers = *m_pd_observers_infos;
        
        /* insert record in datastore */
        string ds_operation("contains");
        try
        {        
            if (m_periodic_observer_ds->contains (M_periodic_notifier_id))
            {
                ds_operation = "update";
                m_periodic_observer_ds->update (M_periodic_notifier_id, record);
            }
            else
            {
                ds_operation = "insert";
                m_periodic_observer_ds->insert (M_periodic_notifier_id, record);
            }
        }
        catch (const Common::AlreadyExistException&)
        {
            CDMW_WARN(m_logger_pointer,
            "record already exists exception in " 
            << ds_operation << " : " 
                << M_periodic_notifier_id );
        }
        catch (const CommonSvcs::DataStore::StorageErrorException &exc_storage)
        {
            CDMW_FATAL(m_logger_pointer,
            "record storage error exception in "
            << ds_operation <<" : " 
            << M_periodic_notifier_id 
            << exc_storage.what());
            CDMW_ASSERT (false);
        }
        catch (const Common::NotFoundException&)
        {
            CDMW_FATAL(m_logger_pointer,
            "record cannot be found in datastore in " 
             << M_periodic_notifier_id );
            CDMW_ASSERT (false);
        }
        catch (...)
        {
            CDMW_FATAL(m_logger_pointer,
            "record unknown exception in "
            << ds_operation << " : " 
            << M_periodic_notifier_id );
            CDMW_ASSERT (false);
        }
    
    }
    if (Configuration::Is_synchronising ())
    {
         CDMW_INFO(m_logger_pointer,
     "<<Notifier_impl::set_record_in_datastore() done >> ");
    }
}


void Notifier_impl::notify_registered_observers()
{
    CDMW_MUTEX_GUARD (m_mutex_condition);

    CDMW_LOG_FUNCTION(::Get_Logger(), "");

    if (m_ds_defined)
    {
        CDMW_INFO(m_logger_pointer,
                "<<Notifier_impl::notify_registered_observers() - Starting...>>");

        CORBA::ULong size = m_pd_observers_infos -> length();
        std::string observer_name("");

        for (CORBA::ULong i = 0; i < size ; i++)
        {
            /************************************
            * Create ObserverMIOPGroup reference
            * *********************************/
            CdmwPlatformMngt::Periodic::ObserverMIOPGroup_var miop_obj ;

            if (CdmwPlatformMngt::IIOP == m_pd_observers_infos[i].protocol_type)
            {
                miop_obj = CdmwPlatformMngt::Periodic::ObserverMIOPGroup::_nil ();
            }
            else 
            {
                try
                {
                    miop_obj = 
                        CdmwPlatformMngt::Periodic::ObserverMIOPGroup::_duplicate(create_miop_ref(m_pd_observers_infos[i].miop_group));
                    
                } catch (const BadParameterException & ex)
                {
                    LogMngr::logMessage(ERR,"Periodic Notifier :: can not create miop corbaloc");
                }
            }

            /**********************************************
            * Creating the TimerObserver with timeout zero
            * ********************************************/
            try
            {
                // costructor could throw PropertyNameInvalid exception 
                std::auto_ptr <TimerObserver>
                pObserver (new TimerObserver(miop_obj.in(),
                                            m_pd_observers_infos[i].observer,
                                            m_system.in(), 
                                            m_pd_observers_infos[i].period, 
                                            m_pd_observers_infos[i].property_names)); 

                TimerObserver* m_timer_obs = pObserver.release();  
                m_timer_obs->wake_up();
                
                //save the timer reference in map
                observer_name = CORBA::string_dup(m_pd_observers_infos[i].observer_name.in());
                m_periodic_observers_map.insert (MapObs::value_type (observer_name, m_timer_obs)); 
                //print_map(m_periodic_observers_map);
                
            }catch (const CdmwPlatformMngt::PropertyNameInvalid & ex)
            {    
                CDMW_LOG_FUNCTION_EXCEPTION(ex);
            }
            catch (const std::bad_alloc &)
            {
                CDMW_THROW (OutOfMemoryException);
            } 
            catch (const Cdmw::AlreadyDoneException& ex)
            {
                CDMW_LOG_FUNCTION_EXCEPTION(ex);
            }
        }
    }
    else 
    {
        LogMngr::logMessage(INF,"Periodic Notifier :: do nothing because datastore is not defined yet");
    }

}

    
bool Notifier_impl::check_periodic_obs_name_in_map (const std::string& observer_name,
                                                    MapObs & map_periodic_observer)
{
    bool bool_ret = false;
        
    MapObs::iterator it;
    it = map_periodic_observer.find(observer_name);
    unsigned int count = map_periodic_observer.count(observer_name);     
    for (unsigned int i=0; i<count; i++ )
    {     
        std::string observer_name_in_map =it->first;
        if (observer_name == observer_name_in_map)
        {
            bool_ret=true;    
        }
        it++;
    }
    return bool_ret;
}                                     


void Notifier_impl::print_map( MapObs& map_periodic_observer)
{
    std::string str = " , " ;
    std::cerr << "[ STARTMAP ]\n";
    for (MapObs::iterator it = map_periodic_observer.begin(); it!=map_periodic_observer.end();it++ )    
    {    
        std::cerr << "[ KEY ] = "<< it->first<<std::endl;
    }
    std::cerr << "[ ENDMAP ]\n";    
}


CdmwPlatformMngt::PeriodicNotifierRecord * Notifier_impl::get_record_from_datastore ()
{
CdmwPlatformMngt::PeriodicNotifierRecord * pd_notifier_record = 0;

try
{
    pd_notifier_record = m_periodic_observer_ds->select (M_periodic_notifier_id);
}
catch (const Common::NotFoundException &)
{
    CDMW_FATAL(m_logger_pointer,
                "periodic notifier record cannot be found in datastore : "
                << M_periodic_notifier_id);
    CDMW_ASSERT (false);
}

return pd_notifier_record;
}


void Notifier_impl::update_observers_infos(const CdmwPlatformMngt::PeriodicObserverRecords & pd_observers_infos)
{
    CDMW_MUTEX_GUARD (m_mutex_condition);

    CORBA::ULong len = pd_observers_infos.length();

    m_pd_observers_infos->length(len);

    for (CORBA::ULong i = 0; i < len ; i++)
    {
        m_pd_observers_infos[i].protocol_type = pd_observers_infos[i].protocol_type; 
        m_pd_observers_infos[i].miop_group = pd_observers_infos[i].miop_group;        
        m_pd_observers_infos[i].observer_name = pd_observers_infos[i].observer_name;

        m_pd_observers_infos[i].observer = 
            CdmwPlatformMngt::Periodic::Observer::_duplicate (pd_observers_infos[i].observer);

        m_pd_observers_infos[i].period = pd_observers_infos[i].period;

        CORBA::ULong size = pd_observers_infos[i].property_names.length();
        m_pd_observers_infos[i].property_names.length(size);

        for (CORBA::ULong j = 0; j < size ; j++)
        {
            m_pd_observers_infos[i].property_names[j] = 
                pd_observers_infos[i].property_names[j];
        }
    }
}


void Notifier_impl::set_observer_info(const char * observer_name, 
                                const CosPropertyService::PropertyNames & property_names,
                                CdmwPlatformMngt::Periodic::Observer_ptr observer_ptr,
                                TimeBase::TimeT period, 
                                const char * miop_group)
{
    //CDMW_MUTEX_GUARD (m_mutex_condition);

    CORBA::ULong len = m_pd_observers_infos->length();
    m_pd_observers_infos->length(len+1);      
    
    m_pd_observers_infos[len].miop_group = miop_group;
    m_pd_observers_infos[len].observer_name = observer_name;
    m_pd_observers_infos[len].observer = 
        CdmwPlatformMngt::Periodic::Observer::_duplicate (observer_ptr);
    m_pd_observers_infos[len].period = period;
    
    m_pd_observers_infos[len].protocol_type = CdmwPlatformMngt::MIOP;

    CORBA::ULong size = property_names.length();
    m_pd_observers_infos[len].property_names.length(size);

    for (CORBA::ULong j = 0; j < size ; j++)
    {
        m_pd_observers_infos[len].property_names[j] = 
            property_names[j];
    }

}

void Notifier_impl::set_observer_info(const char * observer_name, 
                                const CosPropertyService::PropertyNames & property_names,
                                CdmwPlatformMngt::Periodic::Observer_ptr observer_ptr,
                                TimeBase::TimeT period)
{
    //CDMW_MUTEX_GUARD (m_mutex_condition);

    CORBA::ULong len = m_pd_observers_infos->length();
    m_pd_observers_infos->length(len+1);      
    
    m_pd_observers_infos[len].miop_group = "";
    m_pd_observers_infos[len].observer_name = observer_name;
    m_pd_observers_infos[len].observer = 
        CdmwPlatformMngt::Periodic::Observer::_duplicate (observer_ptr);
    m_pd_observers_infos[len].period = period; 
    m_pd_observers_infos[len].protocol_type = CdmwPlatformMngt::IIOP;       

    CORBA::ULong size = property_names.length();
    m_pd_observers_infos[len].property_names.length(size);

    for (CORBA::ULong j = 0; j < size ; j++)
    {
        m_pd_observers_infos[len].property_names[j] = 
            property_names[j];
    }
}

} // End namespace Periodic
} // End namespace PlatformMngt
} // End namespace Cdmw


