/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#include <SystemMngt/platformperiodicnotifier/TimerObserver.hpp>

#include <SystemMngt/idllib/CdmwPlatformMngtSystem.stub.hpp>
#include <SystemMngt/idllib/CdmwPlatformMngtApplication.stub.hpp>
#include <SystemMngt/idllib/CdmwPlatformMngtProcess.stub.hpp>
#include <SystemMngt/idllib/CdmwPlatformMngtEntity.stub.hpp>
#include <SystemMngt/idllib/CdmwPlatformMngtLBGroup.stub.hpp>
#include <SystemMngt/idllib/CdmwPlatformMngtFTGroup.stub.hpp>
#include <SystemMngt/idllib/CdmwPlatformMngtComponent.stub.hpp> 
#include <SystemMngt/idllib/CdmwPlatformMngtPredefinedProperties.stub.hpp>

#include <Foundation/logging/FunctionLogger.hpp>
#include <SystemMngt/platformlibrary/LogMngr.hpp>
#include <Foundation/common/Exception.hpp> 
#include <Foundation/orbsupport/ExceptionMinorCodes.hpp> 

#include <iostream>

namespace {
    
    const char* const LOGGER_NAME = "CDMW.PFMDMN.Periodic.TimerObserver";
    
    Cdmw::Logging::Logger* Get_Logger () {
        static Cdmw::Logging::Logger* logger =
            Cdmw::Logging::Logger::Get_Logger(LOGGER_NAME);
        return logger;
    }
}

namespace Cdmw
{ // Begin namespace Cdmw
    
namespace PlatformMngt
{ // Begin namespace PlatformMngt
    
namespace Periodic
{ // Begin namespace Periodic
    
    
//costructor              
TimerObserver::TimerObserver(CdmwPlatformMngt::Periodic::ObserverMIOPGroup_ptr miop_obj,
                            CdmwPlatformMngt::Periodic::Observer_ptr observer, 
                            CdmwPlatformMngt::System_ptr system, 
                            TimeBase::TimeT period, 
                            const CosPropertyService::PropertyNames & property_names) 
throw(CdmwPlatformMngt::PropertyNameInvalid)    
{  
    m_timer = 0;
    //get logger instance
    m_logger_pointer =
    Cdmw::Logging::Logger::Get_Logger (CdmwPlatformMngtBase::PFMMNG);
    CDMW_LOG_FUNCTION(::Get_Logger(), "");   
    CDMW_ASSERT (m_logger_pointer != 0);
    CDMW_DEBUG (m_logger_pointer, "Timer Observer Notifier Instance ");    

    try
    {
        //return false if property names syntax is wrong
        check_property_names(property_names);
        
    } catch (const CdmwPlatformMngt::PropertyNameInvalid & e)
    {    
        CDMW_LOG_FUNCTION_EXCEPTION(e);
        throw e;
    }

    m_period = period;
    
    //Creating the Timer
    std::auto_ptr <SingleShotTimer> 
            pTimer (new SingleShotTimer(this, m_period)); 
    m_timer = pTimer.release();

    //Starting the timer
    m_timer->start();

    //inizialising local variables        
    m_miop_obj = CdmwPlatformMngt::Periodic::ObserverMIOPGroup::_duplicate(miop_obj); 
    m_system =CdmwPlatformMngt::System::_duplicate (system);
    m_observer = CdmwPlatformMngt::Periodic::Observer::_duplicate (observer);  

}

                
//destructor
TimerObserver::~TimerObserver()
{
    try
    {
        //Stopping the timer
        m_timer->stop();

    }catch(const Cdmw::InternalErrorException&)
    {}
    
    //Deleting the timer.
    delete m_timer;
    m_timer = 0;
}


void TimerObserver::start_timer() 
throw(Cdmw::AlreadyDoneException)
{
    CDMW_LOG_FUNCTION(::Get_Logger(), "");
    //restart timer 
    try
    {
        m_timer->start_timer();
        
    }catch (const Cdmw::AlreadyDoneException & ex)
    {
        LogMngr::logMessage(ERR,"Periodic Notifier :: Can not restart timer again...");
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw ex;
    }
}
        


CdmwPlatformMngt::Periodic::Observer_ptr 
TimerObserver::get_observer_ref()
{     
    CdmwPlatformMngt::Periodic::Observer_var periodic_observer = 
        CdmwPlatformMngt::Periodic::Observer::_duplicate (m_observer);

    return periodic_observer._retn (); 

}


CdmwPlatformMngt::Periodic::ObserverMIOPGroup_ptr
TimerObserver::get_group_ref()
{
    CdmwPlatformMngt::Periodic::ObserverMIOPGroup_var miop_obj = 
        CdmwPlatformMngt::Periodic::ObserverMIOPGroup::_duplicate (m_miop_obj);
    
    return miop_obj._retn ();
}
    
                
bool TimerObserver::check_property_names(const CosPropertyService::PropertyNames & property_names)
throw (CdmwPlatformMngt::PropertyNameInvalid)
{
    CDMW_LOG_FUNCTION(::Get_Logger(), "");
    
    CORBA::ULong len = property_names.length ();
    std::string prefix (CdmwPlatformMngt::CDMW_PREFIX);
    
    bool result = false;
    
    for (unsigned int i=0; i<len; ++i) 
    {
        try
        {
            //check property names syntax
            m_property_info.element_type = 
                Cdmw::PlatformMngt::ManagedElementKey::Get_element_names(CORBA::string_dup(property_names[i]),
                                                                        m_property_info.system_name,
                                                                        m_property_info.application_name, 
                                                                        m_property_info.host_name,
                                                                        m_property_info.process_name, 
                                                                        m_property_info.component_name,
                                                                        m_property_info.entity_name,
                                                                        m_property_info.attribute_name,
                                                                        m_property_info.ftgroup_name,
                                                                        m_property_info.lbgroup_name); 
            
            m_property_info.property_name = CORBA::string_dup(property_names[i]);
            
            std::string::size_type pos = (m_property_info.attribute_name).find(prefix);  
            std::string::size_type size = prefix.size();    
                                                                
            // if attributes are predefined or user-defined
            if (pos != std::string::npos)
            {                         
                m_property_info.attribute_name = (m_property_info.attribute_name).substr(pos+size);
                m_property_info.flag = CdmwPlatformMngt::PRE_DEFINED;
            }
            else
            {
                m_property_info.flag = CdmwPlatformMngt::USER_DEFINED;
            }
        }                                     
        catch (const CdmwPlatformMngt::ElementPathInvalid & ex)
        {  
            CdmwPlatformMngt::PropertyNameInvalid e;     
            CDMW_LOG_FUNCTION_EXCEPTION(e);
            throw e;
        }
        catch (...) 
        {
            CdmwPlatformMngt::PropertyNameInvalid e;
            CDMW_LOG_FUNCTION_EXCEPTION(e);
            throw e;
        }
        
        result=true;
        
        //insert each property into vector
        m_property_vector.push_back(m_property_info);
    }   
    
    return result;
}  


void TimerObserver::wake_up() throw()
{
    //idex to access to property sequence   
    CORBA::ULong len = 0;
    unsigned int delay = 0;
    bool attr_found;
    CORBA::Any_var any; 
    CosPropertyService::Properties_var properties = new CosPropertyService::Properties();   
    
    // Gets the current time before the processing
    // That time will be used to calculate the delay before setting again the single shot timer
    // That delay will be taken away from the next interval to avoid any drift
    Cdmw::OsSupport::OS::Timeval timestart, timeend;
    timestart = Cdmw::OsSupport::OS::get_time();
    
    //set value for each element of properties
    for (PropertyVector::iterator it = m_property_vector.begin(); it!=m_property_vector.end();it++ )    
    {    
        attr_found = false;

        //check element type
        switch( it->element_type)
        {
        case (CdmwPlatformMngt::SYSTEM_TYPE) :
        {

            try
            {
                m_system->get_attribute((it->attribute_name).c_str(), any.out() , it->flag);

                attr_found = true;
            }
            catch (const CdmwPlatformMngt::AttributeNotFound & e)
            {
                LogMngr::logMessage(INF,"Periodic Notifier :: AttributeNotFound for SYSTEM_TYPE");
            }
            catch (...)
            {
                LogMngr::logMessage(FTL,"Periodic Notifier :: Fatal error in get attribute on System");
            } 
        }    
        break;

        case (CdmwPlatformMngt::HOST_TYPE) :
        {

            try
            {
                CdmwPlatformMngt::Host_var host = m_system->get_host((it->host_name).c_str());

                host->get_attribute((it->attribute_name).c_str(), any.out() , it->flag);

                attr_found = true;    
            }
            catch (const CdmwPlatformMngt::HostNotFound & e)
            {
                LogMngr::logMessage(INF,"Periodic Notifier :: HostNotFound");
            }
            catch (const CdmwPlatformMngt::AttributeNotFound & e)
            {
                LogMngr::logMessage(INF,"Periodic Notifier :: AttributeNotFound for HOST_TYPE");
            }
            catch (...)
            {
                LogMngr::logMessage(FTL,"Periodic Notifier :: Fatal error in get attribute on Host");
            }                 
        }
        break;

        case (CdmwPlatformMngt::APPLICATION_TYPE) :
        {

            try
            {                   
                CdmwPlatformMngt::Application_var app = m_system->get_application((it->application_name).c_str());

                app->get_attribute((it->attribute_name).c_str(), any.out() , it->flag);

                attr_found = true;    
            }
            catch (const CdmwPlatformMngt::ApplicationNotFound & e)
            {
                LogMngr::logMessage(INF,"Periodic Notifier :: ApplicationNotFound");
            }
            catch (const CdmwPlatformMngt::AttributeNotFound & e)
            {
                LogMngr::logMessage(INF,"Periodic Notifier :: AttributeNotFound for APPLICATION_TYPE");
            }
            catch (...)
            {
                LogMngr::logMessage(FTL,"Periodic Notifier :: Fatal error in get attribute on Application");
            }   

        }
        break;

        case (CdmwPlatformMngt::FT_GROUP_TYPE) :
        {

            try
            {                   
                CdmwPlatformMngt::Application_var app = m_system->get_application((it->application_name).c_str());
                
                CdmwPlatformMngt::FTGroup_var ft_group = app->get_ft_group((it->ftgroup_name).c_str());

                ft_group->get_attribute((it->attribute_name).c_str(), any.out() , it->flag);

                attr_found = true;    
            }
            catch (const CdmwPlatformMngt::ApplicationNotFound & e)
            {
                LogMngr::logMessage(INF,"Periodic Notifier :: ApplicationNotFound");
            }
            catch (const CdmwPlatformMngt::GroupNotFound & e)
            {
                LogMngr::logMessage(INF,"Periodic Notifier :: FTGroupNotFound");
            }
            catch (const CdmwPlatformMngt::AttributeNotFound & e)
            {
                LogMngr::logMessage(INF,"Periodic Notifier :: AttributeNotFound for APPLICATION_TYPE");
            }
            catch (...)
            {
                LogMngr::logMessage(FTL,"Periodic Notifier :: Fatal error in get attribute on Application");
            }   

        }
        break;
        
        case (CdmwPlatformMngt::LB_GROUP_TYPE) :
        {

            try
            {                   
                CdmwPlatformMngt::Application_var app = m_system->get_application((it->application_name).c_str());
                
                CdmwPlatformMngt::LBGroup_var lb_group = app->get_lb_group((it->lbgroup_name).c_str());

                lb_group->get_attribute((it->attribute_name).c_str(), any.out() , it->flag);

                attr_found = true;    
            }
            catch (const CdmwPlatformMngt::ApplicationNotFound & e)
            {
                LogMngr::logMessage(INF,"Periodic Notifier :: ApplicationNotFound");
            }
            catch (const CdmwPlatformMngt::GroupNotFound & e)
            {
                LogMngr::logMessage(INF,"Periodic Notifier :: LBGroupNotFound");
            }
            catch (const CdmwPlatformMngt::AttributeNotFound & e)
            {
                LogMngr::logMessage(INF,"Periodic Notifier :: AttributeNotFound for APPLICATION_TYPE");
            }
            catch (...)
            {
                LogMngr::logMessage(FTL,"Periodic Notifier :: Fatal error in get attribute on Application");
            }   

        }
        break;

        case (CdmwPlatformMngt::PROCESS_TYPE) :
        {

            try
            {
                CdmwPlatformMngt::Application_var app = m_system->get_application((it->application_name).c_str());

                CdmwPlatformMngt::Process_var proc = app->get_process((it->process_name).c_str(), (it->host_name).c_str());

                proc->get_attribute((it->attribute_name).c_str(), any.out(), it->flag);

                attr_found = true;    
            }
            catch (const CdmwPlatformMngt::ApplicationNotFound & e)
            {
                LogMngr::logMessage(INF,"Periodic Notifier :: ApplicationNotFound");
            }
            catch (const CdmwPlatformMngt::ProcessNotFound & e)
            {
                LogMngr::logMessage(INF,"Periodic Notifier :: ProcessNotFound");
            }
            catch (const CdmwPlatformMngt::AttributeNotFound & e)
            {
                LogMngr::logMessage(INF,"Periodic Notifier :: AttributeNotFound for PROCESS_TYPE");
            }
            catch (...)
            {
                LogMngr::logMessage(FTL,"Periodic Notifier :: Fatal error in get attribute on Process");
            }       
        }
        break;          

        case (CdmwPlatformMngt::COMPONENT_TYPE) :
        {

            try
            {
                CdmwPlatformMngt::Application_var app = m_system->get_application((it->application_name).c_str());

                CdmwPlatformMngt::Process_var proc = app->get_process((it->process_name).c_str(), (it->host_name).c_str());

                CdmwPlatformMngt::CcmComponent_var comp = proc->get_component((it->component_name).c_str());

                comp->get_attribute((it->attribute_name).c_str(), any.out(), it->flag);

                attr_found = true;    
            }
            catch (const CdmwPlatformMngt::ApplicationNotFound & e)
            {
                LogMngr::logMessage(INF,"Periodic Notifier :: ApplicationNotFound");
            }
            catch (const CdmwPlatformMngt::ProcessNotFound & e)
            {
                LogMngr::logMessage(INF,"Periodic Notifier :: ProcessNotFound");
            }
            catch (const CdmwPlatformMngt::ComponentNotFound & e)
            {
                LogMngr::logMessage(INF,"Periodic Notifier :: ComponentNotFound");
            }
            catch (const CdmwPlatformMngt::AttributeNotFound & e)
            {
                LogMngr::logMessage(INF,"Periodic Notifier :: AttributeNotFound for COMPONENT_TYPE");
            }
            catch (...)
            {
                LogMngr::logMessage(FTL,"Periodic Notifier :: Fatal error in get attribute on Component");
            }       
        }
        break;

        case (CdmwPlatformMngt::ENTITY_TYPE) :
        {
            CdmwPlatformMngt::ManagedElement* managed_element = 0;

            try
            {
                if (0 == (it->host_name).empty())
                {
                    CdmwPlatformMngt::Host_var host = 
                        m_system->get_host((it->host_name).c_str());

                        if (0 == (it->application_name).empty())
                        {
                            CdmwPlatformMngt::Application_var app = 
                                m_system->get_application((it->application_name).c_str());

                                if (0 == (it->process_name).empty())
                                {
                                    CdmwPlatformMngt::Process_var proc = 
                                        app->get_process((it->process_name).c_str(), (it->host_name).c_str());

                                        if (0 == (it->component_name).empty())    
                                        {
                                            CdmwPlatformMngt::CcmComponent_var comp = proc->get_component((it->component_name).c_str());
                                            
                                            managed_element = comp->get_entity(it->entity_name.c_str());

                                        }
                                        else 
                                        {                                           
                                            managed_element = proc->get_entity(it->entity_name.c_str());
                                        }
                                }                     
                        }
                        else
                        {
                            managed_element = host->get_entity(it->entity_name.c_str());

                        }
                }else if (0 == (it->application_name).empty())              
                {                  
                    CdmwPlatformMngt::Application_var app = m_system->get_application((it->application_name).c_str());
                        
                    if (0 == (it->ftgroup_name).empty())
                    {
                        CdmwPlatformMngt::FTGroup_var ft_group = app->get_ft_group((it->ftgroup_name).c_str());
                        
                        managed_element = ft_group->get_entity(it->entity_name.c_str());
                    }                  
                    else if (0 == (it->lbgroup_name).empty())
                    {
                        CdmwPlatformMngt::LBGroup_var lb_group = app->get_lb_group((it->lbgroup_name).c_str());
                        
                        managed_element = lb_group->get_entity(it->entity_name.c_str());
                    }
                    else                  
                    {
                        managed_element = app->get_entity(it->entity_name.c_str());
                    }

                }
                else
                {
                    managed_element = m_system->get_entity(it->entity_name.c_str());
                }
            }
            catch (const CdmwPlatformMngt::HostNotFound & e)
            {
                LogMngr::logMessage(INF,"Periodic Notifier :: HostNotFound");
            }
            catch (const CdmwPlatformMngt::ApplicationNotFound & e)
            {
                LogMngr::logMessage(INF,"Periodic Notifier :: ApplicationNotFound");
            }
            catch (const CdmwPlatformMngt::ProcessNotFound & e)
            {
                LogMngr::logMessage(INF,"Periodic Notifier :: ProcessNotFound");
            }
            catch (const CdmwPlatformMngt::ComponentNotFound & e)
            {
                LogMngr::logMessage(INF,"Periodic Notifier :: ComponentNotFound");
            }
            catch (const CdmwPlatformMngt::GroupNotFound & e)
            {
                LogMngr::logMessage(INF,"Periodic Notifier :: GroupNotFound");
            }
            catch (const CdmwPlatformMngt::EntityNotFound & e)
            {
                LogMngr::logMessage(INF,"Periodic Notifier :: EntityNotFound");
            }
            catch (...)
            {
                LogMngr::logMessage(FTL,"Periodic Notifier :: Fatal error in get reference on Entity");
            }       


            if( 0 != managed_element)
            {
                try
                {
                    
                    managed_element->get_attribute((it->attribute_name).c_str(), any.out(), it->flag);
                    
                    attr_found = true;    
                }
                catch (const CdmwPlatformMngt::AttributeNotFound & e)
                {
                    LogMngr::logMessage(INF,"Periodic Notifier :: AttributeNotFound for ENTITY_TYPE");
                }
                catch (...)
                {
                    LogMngr::logMessage(FTL,"Periodic Notifier :: Fatal error in get attribute on Entity");
                }       
            }
            else
            {
                LogMngr::logMessage(ERR,"Periodic Notifier :: Invalid ManagedElement");
            }

        }
        break;

        default :            
            LogMngr::logMessage(ERR,"Periodic Notifier :: ElementType not managed");
        break;

        }

        if (attr_found)
        {
            properties->length (len+1);

            properties[len].property_name = CORBA::string_dup(it->property_name);  

            properties[len].property_value = any.in() ;

            len++; 
        }
    }

    try
    {
        if (CORBA::is_nil (m_miop_obj))
        {
            //calling single periodic observer callback             
            m_observer->execute(properties.in());   
        }else 
        {
            //calling group periodic observer callback  
            m_miop_obj->execute(properties.in());
        }
    }catch (...)
    {
        LogMngr::logMessage(ERR,"Periodic Notifier :: Exception on Periodic Observer execute() ");
    }

    //calculate the delay
    timeend = Cdmw::OsSupport::OS::get_time();
    delay = ((timeend.seconds - timestart.seconds)*(int)1E6 + timeend.microseconds - timestart.microseconds)/1000;
    
    if(delay < m_period)
    {
        m_timer->setup(m_period - delay); 
    }
    
    //restart timer 
    try
    {
        m_timer->start_timer();
        
    }catch (const Cdmw::AlreadyDoneException&)
    {
        LogMngr::logMessage(ERR,"Periodic Notifier :: Can not restart timer again...");
    }
    
}


} // End namespace Periodic
} // End namespace PlatformMngt
} // End namespace Cdmw
    
