/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#include "platformsystemlibrary/Host_impl.hpp"

#include "SystemMngt/platformapplicationlibrary/Application_impl.hpp"
#include "SystemMngt/platformapplicationlibrary/ApplicationControl_impl.hpp"
#include "Foundation/osthreads/ReaderLockGuard.hpp"
#include "Foundation/osthreads/WriterLockGuard.hpp"
#include "Foundation/orbsupport/ExceptionMinorCodes.hpp"

#include "platformsystemlibrary/CentralisedSupervisionSystem_impl.hpp"
#include "platformsystemlibrary/CentralisedSupervisionObserver_impl.hpp"
#include "platformsystemlibrary/SystemLogMessageId.hpp"

#include "SystemMngt/idllib/CdmwPlatformMngtServiceBroker.stub.hpp"

#include "SystemMngt/platformlibrary/LogMngr.hpp"
#include "SystemMngt/tools/ManagedElementKey.hpp"

#include <vector>

namespace Cdmw
{
  namespace PlatformMngt
  {



// ----------------------------------------------------------------------
// CentralisedSupervisionSystem_impl class.
// ----------------------------------------------------------------------
CentralisedSupervisionSystem_impl::CentralisedSupervisionSystem_impl (
                                   SupervisionEventNotifier* eventNotifier,
                                   CORBA::ORB_ptr orb,
                                   PortableServer::POA_ptr poa,
                                   CdmwPlatformMngt::RWElementRepository_ptr element_repository)   

          throw (BadParameterException, AlreadyDoneException,
                 UnknownStateException, AlreadyExistsException,
                 OutOfResourcesException, OutOfMemoryException)                   
          : System_impl (eventNotifier, orb, poa, element_repository)
{
    // Create the SupervisionObserver servant
    m_supervisionObserver = new CentralisedSupervisionObserver_impl (this);                          
    // create an object var to take pointer ownership
    PortableServer::ServantBase_var supervisionObserverServant = m_supervisionObserver;        
    // activate object
    m_supervisionObserverObject = m_supervisionObserver->_this();
    
    // clear application servant map
    m_application_servants.clear();

    
    eventNotifier->registerObserver("InternalCentralisedSupervisionSystemObserver",
               m_supervisionObserverObject.in ());

}



    CentralisedSupervisionSystem_impl::~CentralisedSupervisionSystem_impl()
    {
 
    }




    CdmwPlatformMngt::Application_ptr 
    CentralisedSupervisionSystem_impl::doAddApplication(
                            const CdmwPlatformMngt::PlatformApplicationDef& platform_application_def,
                            const CdmwPlatformMngt::ApplicationDef& application_def)
      throw (CdmwPlatformMngt::ApplicationAlreadyExists,
         CdmwPlatformMngt::HostNotFound,
         CdmwPlatformMngt::DuplicateEntity,
         CosPropertyService::MultipleExceptions,
         CdmwPlatformMngt::DuplicateService,
         CdmwPlatformMngt::DuplicateProcess,
         CdmwPlatformMngt::ProcessHostNotFound,
         CdmwPlatformMngt::DuplicateProcessEntity,
         CdmwPlatformMngt::ProcessPropertyExceptions,
         CdmwPlatformMngt::DuplicateProcessService,
         CdmwPlatformMngt::DuplicateProcessStep,
         CdmwPlatformMngt::IncompatibleStatus,
         CORBA::SystemException)
    {
      std::string applicationName = platform_application_def.application_name.in();
    
      {
        CDMW_READER_LOCK_GUARD (m_rwSystemLock);
           
        Applications::iterator applicationIt = m_applications.find (applicationName);
             
        if (applicationIt != m_applications.end())
      {
            throw CdmwPlatformMngt::ApplicationAlreadyExists();
      }
      }

    
      try
    {
      // Create the application servant
      // ------------------------------
        
      // build the application path        
      std::string application_path = 
        ManagedElementKey::Get_application_path (
                             m_element_name.in(), applicationName);
                                         
                                            
#ifdef CDMW_USE_FAULTTOLERANCE
   /*   Application_impl* p_application = 
        new Application_impl (m_poa.in(), m_eventNotifier,
                  m_element_repository.in(),
                  m_replicationManagerIOGR, 
                  m_FTConverterObject.in());*/

     Application_impl* p_application = 
        new Application_impl (m_poa.in(), m_eventNotifier,
                  m_element_repository.in(),
                  m_replicationManagerIOGR, 
                  m_FTConverterObject.in(), & m_check_consistency_status_recursive_mutex, m_orb.in());
#else
      Application_impl* p_application = 
        new Application_impl (m_poa.in(), m_eventNotifier,
                  m_element_repository.in(), & m_check_consistency_status_recursive_mutex, m_orb.in());
#endif      

        // set application ack data
        CdmwPlatformMngt::ApplicationStartingData_var applicationStartData =
                             new CdmwPlatformMngt::ApplicationStartingData();
        applicationStartData->system_name = m_element_name;
        applicationStartData->system_port = m_systemPort;
        applicationStartData->application_name = platform_application_def.application_name;
        applicationStartData->host_container = m_hostContainerObject;
        applicationStartData->supervision_observer = m_supervisionObserverObject;
        applicationStartData->application_observer = m_applicationObserverObject;
        applicationStartData->service_binding = m_serviceBindingObject;
        applicationStartData->system_service_def_container = m_serviceDefContainerObject;

#ifdef CDMW_USE_FAULTTOLERANCE
        
      applicationStartData->process_observer = m_FTProcessObserverGroup;
      applicationStartData->service_broker = m_FTServiceBrokerGroup;
        
#else
      applicationStartData->process_observer = 
        CdmwPlatformMngt::ProcessObserver::_nil();
      applicationStartData->service_broker = 
        CdmwPlatformMngtService::ServiceBroker::_nil();
#endif
        
               
        // process end of application creation
        p_application->end_creation (applicationStartData.in(), application_path.c_str());
        // create an object var to take pointer ownership
        PortableServer::ServantBase_var applicationServant = p_application;
        
        
      // Create the application control servant
      // --------------------------------------
      ApplicationControl_impl* p_applicationControl = 
        new ApplicationControl_impl (m_poa.in(), p_application);
                             
      // create an object var to take pointer ownership
      PortableServer::ServantBase_var applicationControlServant = p_applicationControl;
                     
#ifdef CDMW_USE_FAULTTOLERANCE

      // set global service
      // ----------------
      CdmwPlatformMngtService::ServiceBroker_var serviceBroker =
        p_application->get_service_broker();
                
                
      serviceBroker->set_global_service(
                        "ReplicationManager", m_replicationMngt.in());

#endif

      // activate objects
      // ----------------
      p_application->activate();
      CdmwPlatformMngt::Application_var  applicationObject = p_application->_this();
        
      p_applicationControl->activate();
      CdmwPlatformMngt::ApplicationControl_var  applicationControlObject = 
        p_applicationControl->_this();
        
        
      // get process observer for the application
      CdmwPlatformMngt::ProcessObserver_var processObserver =
        p_application->get_process_observer();
        
      // Add them and servants into the applications maps
      {
            CDMW_WRITER_LOCK_GUARD (m_rwSystemLock);
            ApplicationRef applicationRef (applicationObject.in(), 
                                           applicationControlObject.in(),
                                           processObserver.in());
            m_applications.insert (Applications::value_type (applicationName,
                                                             applicationRef));
            
            ApplicationServantSet app_servant_set;
            app_servant_set.application = p_application;
            app_servant_set.application_control = p_applicationControl;
                                                             
            m_application_servants.insert (ApplicationServants::value_type (
                                        applicationName,            
                                        app_servant_set));

            // application is added as INITIAL (this value is going to be overwritten)
            //application_status_change_event(applicationName, CdmwPlatformMngt::APPLICATION_INITIAL); 
      
      //add_ftgroups ...
            
      //see if there are ftgroups to add
            
      CORBA::ULong num_ftgroups = (application_def.ft_groups).length() ;      
      
     //call add_ftgroup of application ...
      for (CORBA::ULong  ii=0; ii<num_ftgroups ; ii++)
             {
          
             p_application->add_ftgroup(
                       ((application_def.ft_groups)[ii]) );
             }  

          CORBA::ULong num_lbgroups = (application_def.lb_groups).length() ;      

      //call add_lbgroup of application ...
      for (CORBA::ULong  ii=0; ii<num_lbgroups ; ii++)
             {
          
             p_application->add_lbgroup(
                       ((application_def.lb_groups)[ii]) );
             }  
      }
        
      try
        {
          // add in repository the application object
          m_element_repository->add_element (application_path.c_str(),
                         CdmwPlatformMngt::APPLICATION_TYPE,
                         applicationObject.in());
        }
      catch(const CdmwPlatformMngt::ElementPathInvalid&)
        {
          throw CORBA::INTERNAL(OrbSupport::INTERNAL, CORBA::COMPLETED_NO);
        }
      catch(const CdmwPlatformMngt::ElementAlreadyExists&)
        {
          throw CdmwPlatformMngt::ApplicationAlreadyExists();
        }
      
      try
        {
          // Call Application define to create processes
          applicationObject->define (application_def);
        }
      catch (const CdmwPlatformMngt::AlreadyDone &)
        {
          throw CORBA::INTERNAL (OrbSupport::INTERNAL, CORBA::COMPLETED_NO);
        }
        
        
      // return the application object
      return applicationObject._retn();
    
    }
      catch (const std::bad_alloc&)
    {
      throw CORBA::NO_MEMORY (OrbSupport::NO_MEMORY, CORBA::COMPLETED_NO);
    }
      catch (const AlreadyDoneException&)
    {
      throw CORBA::INTERNAL (OrbSupport::INTERNAL, CORBA::COMPLETED_NO);
    }
      catch (const InternalErrorException&)
    {
      throw CORBA::INTERNAL (OrbSupport::INTERNAL, CORBA::COMPLETED_NO);
    }
    }


    void CentralisedSupervisionSystem_impl::doRemoveApplication (const char* application_name)
      throw (CdmwPlatformMngt::ApplicationNotFound,
         CdmwPlatformMngt::ApplicationStillActive,
         CORBA::SystemException)
    {
      try
    {
      CdmwPlatformMngt::ApplicationControl_var  applicationControlObject;
        
      Applications::iterator applicationIt;
        
      {
            CDMW_READER_LOCK_GUARD (m_rwSystemLock);
        
            applicationIt = m_applications.find (application_name);
            if (applicationIt == m_applications.end())
          {
                throw CdmwPlatformMngt::ApplicationNotFound();
          }
             
            // check if application is still active
            // it can be removed only if it is inactive
            CORBA::String_var info;
            CdmwPlatformMngt::ApplicationStatus appStatus = 
          ((applicationIt->second).m_application)->get_status(info.out());
        
            if ((appStatus == CdmwPlatformMngt::APPLICATION_INITIALISING) ||
                (appStatus == CdmwPlatformMngt::APPLICATION_INITIALISATION_INCOMPLETE) ||
                (appStatus == CdmwPlatformMngt::APPLICATION_INITIALISED) ||
                (appStatus == CdmwPlatformMngt::APPLICATION_RUN_REQUEST) ||
                (appStatus == CdmwPlatformMngt::APPLICATION_RUNNING) ||
                (appStatus == CdmwPlatformMngt::APPLICATION_PARTIAL_RUNNING) ||
                (appStatus == CdmwPlatformMngt::APPLICATION_STOPPING))
          {
                throw CdmwPlatformMngt::ApplicationStillActive();
          }
            
            // get application control
            applicationControlObject = (applicationIt->second).m_applicationControl;
            
      }

      {
        CdmwPlatformMngt::Application_var  applicationObject;
            
            CDMW_WRITER_LOCK_GUARD (m_rwSystemLock);
            
            // get application
            applicationIt = m_applications.find (application_name);
            applicationObject = (applicationIt->second).m_application;
            
            // remove from repository the application object
            CORBA::String_var application_path = applicationObject->element_path();
            m_element_repository->remove_element (application_path.in());
            
            // request to control servant to destroy application
            applicationControlObject->destroy_application();
    
            // then erase object from the map : object is destroyed by orb if its
            // reference counter is 0        
           
            m_applications.erase(applicationIt);
            
            // remove the application servant from map
            ApplicationServants::iterator applicationServantIt = 
          m_application_servants.find (application_name);
                           
            if (applicationServantIt != m_application_servants.end())
          {
                m_application_servants.erase(applicationServantIt);
          }
      }          
    }
      catch(const CdmwPlatformMngt::ElementPathInvalid&)
    {
      throw CORBA::INTERNAL(OrbSupport::INTERNAL, CORBA::COMPLETED_NO);
    }
      catch(const CdmwPlatformMngt::ManagedElementNotFound&)
    {
      throw CdmwPlatformMngt::ApplicationNotFound();
    }  
      catch (const std::bad_alloc&)
    {
      throw CORBA::NO_MEMORY (OrbSupport::NO_MEMORY, CORBA::COMPLETED_NO);
    }
    }



    void CentralisedSupervisionSystem_impl::on_remove_application (
                                   const char* application_name)
      throw (CdmwPlatformMngt::ApplicationNotFound,
         CORBA::SystemException)
    {
    
      LogMngr::logMessage (INF, MSG_ID_REMOVE_APPLICATION,
               application_name, m_element_name.in ());
               
      try
    {
      CDMW_WRITER_LOCK_GUARD (m_rwSystemLock);
        
      Applications::iterator applicationIt = m_applications.find (application_name);
      if (applicationIt == m_applications.end())
        {
          throw CdmwPlatformMngt::ApplicationNotFound();
        }
             
      // get application control
      CdmwPlatformMngt::ApplicationControl_var applicationControlObject = 
        (applicationIt->second).m_applicationControl;
            
      CdmwPlatformMngt::Application_var applicationObject = 
        (applicationIt->second).m_application;
            
      // remove from repository the application object
      CORBA::String_var application_path = applicationObject->element_path();
      m_element_repository->remove_element (application_path.in());
            
      // request to control servant to destroy application
      applicationControlObject->destroy_application();
    
      // then erase object from the map : object is destroyed by orb if its
      // reference counter is 0        
           
      m_applications.erase(applicationIt);
            
      // remove the application servant from map
      ApplicationServants::iterator applicationServantIt = 
        m_application_servants.find (application_name);
                           
      if (applicationServantIt != m_application_servants.end())
        {
          m_application_servants.erase(applicationServantIt);
        }        
    }
      catch(const CdmwPlatformMngt::ElementPathInvalid&)
    {
      throw CORBA::INTERNAL(OrbSupport::INTERNAL, CORBA::COMPLETED_NO);
    }
      catch(const CdmwPlatformMngt::ManagedElementNotFound&)
    {
      throw CdmwPlatformMngt::ApplicationNotFound();
    }  
      catch (const std::bad_alloc&)
    {
      throw CORBA::NO_MEMORY (OrbSupport::NO_MEMORY, CORBA::COMPLETED_NO);
    }
    }



            
            

    void CentralisedSupervisionSystem_impl::getApplicationServants (
                                    const char* application_name,
                                    Application_impl*& application_servant,
                                    ApplicationControl_impl*& app_control_servant)
      throw (CdmwPlatformMngt::ApplicationNotFound)
    {
      application_servant = 0;
      app_control_servant = 0;
    
      try
    {       
      CDMW_READER_LOCK_GUARD(m_rwSystemLock);

      ApplicationServants::iterator applicationServantIt = 
        m_application_servants.find(application_name);
      if (applicationServantIt == m_application_servants.end())
        {
          throw CdmwPlatformMngt::ApplicationNotFound();
        }

      application_servant = (applicationServantIt->second).application;
      app_control_servant = (applicationServantIt->second).application_control;     
    }
      catch (...)
    {
      throw CdmwPlatformMngt::ApplicationNotFound();
    }
    }

    void CentralisedSupervisionSystem_impl::synchro_applications_with_datastore()
    {

      ApplicationServants::iterator applicationIt;
    
      for (applicationIt = m_application_servants.begin();
       applicationIt != m_application_servants.end();
       applicationIt++)
    {
      Application_impl* application = (applicationIt->second).application;
        
      application->set_system_port(m_systemPort);       
 
      // synchro the application
      application->synchro_with_datastore();
      
      // synchro the groups of the application
      application->synchro_groups_with_datastore();
      
      // synchro the processes of the application
      application->synchro_processes_with_datastore();
    }
    }

    void CentralisedSupervisionSystem_impl::synchro_processing_of_applications()
    {
      ApplicationServants::iterator applicationIt;
    
      for (applicationIt = m_application_servants.begin();
       applicationIt != m_application_servants.end();
       applicationIt++)
    {
      Application_impl* application = (applicationIt->second).application;
        
      // synchro the application
      application->synchro_processing();
    }
    }

    void CentralisedSupervisionSystem_impl::synchro_processing_of_processes()
    {
      ApplicationServants::iterator applicationIt;
    
      for (applicationIt = m_application_servants.begin();
       applicationIt != m_application_servants.end();
       applicationIt++)
    {
      Application_impl* application = (applicationIt->second).application;
        
      // synchro the processes of the application
      application->synchro_processing_of_processes();
    }
    }
    
    void CentralisedSupervisionSystem_impl::synchro_hosts_with_datastore()
    {
        CORBA::ULong how_many = get_number_of_hosts ();
       
        CdmwPlatformMngt::ElementNames_var host_names;
        CdmwPlatformMngt::ElementNamesIterator_var rest;

        get_all_host_names (how_many, host_names, rest);

        for (CORBA::ULong count = 0U; count< how_many; count++)
        {
           //synchro the host with its datastore 
           getHost(host_names[count])->synchro_with_datastore();
        }
   
    }
    
     
    void CentralisedSupervisionSystem_impl::synchro_entities_with_datastore()
    {
        CORBA::ULong how_many = get_number_of_entities (); 
        CdmwPlatformMngt::ElementNames_var entity_names;
        CdmwPlatformMngt::ElementNamesIterator_var rest;
        get_all_entity_names (how_many, entity_names, rest);
        for (CORBA::ULong count = 0U; count< how_many; count++)
        {
           //synchro the entity with its datastore 
           getEntity(entity_names[count])->synchro_with_datastore();
        }
    }  
     
    void CentralisedSupervisionSystem_impl::freeze_event_handlers()
    {
      // freeze the event handler of system
      get_event_handler()->freeze_handler();
    
      // freeze the event handler of each application
      ApplicationServants::iterator applicationIt;
    
      for (applicationIt = m_application_servants.begin();
       applicationIt != m_application_servants.end();
       applicationIt++)
    {
      Application_impl* application = (applicationIt->second).application;
        
      application->get_event_handler()->freeze_handler();
    }
    }

    void CentralisedSupervisionSystem_impl::unfreeze_event_handlers()
    {
      // unfreeze the event handler of system
      get_event_handler()->unfreeze_handler();
    
      // unfreeze the event handler of each application
      ApplicationServants::iterator applicationIt;
    
      for (applicationIt = m_application_servants.begin();
       applicationIt != m_application_servants.end();
       applicationIt++)
    {
      Application_impl* application = (applicationIt->second).application;
        
      application->get_event_handler()->unfreeze_handler();
    }
    }
    
    void 
    CentralisedSupervisionSystem_impl::set_requested_restart_warmth(const CdmwPlatformMngt::RestartWarmth restartWarmth){
        //std::cerr << "<<<CentralisedSupervisionSystem_impl::set_requested_restart_warmth>>>" << std::endl;
        this->ManagedElement_impl::set_requested_restart_warmth ( restartWarmth);
        ApplicationServants::iterator applicationIt = m_application_servants.begin();
        while ( applicationIt != m_application_servants.end ()) 
        {    
            Application_impl *p_application_impl = (applicationIt->second).application;
            p_application_impl->set_requested_restart_warmth( restartWarmth );
            CORBA::String_var info;
            applicationIt ++;
        }
    }
    
    CdmwPlatformMngt::LongRequestStatus 
    CentralisedSupervisionSystem_impl::restart ( const CdmwPlatformMngt::RestartWarmth restartWarmth,
                         const CdmwPlatformMngtBase::StartupKind & startup_kind,
                         bool emergency,
                         CdmwPlatformMngt::SystemStatus sysStatus )
      throw (CdmwPlatformMngt::IncompatibleStatus, CORBA::SystemException)
        
    {
      bool stop_req_done = false;
      CdmwPlatformMngt::LongRequestStatus req_status;
    
      set_requested_restart_warmth ( restartWarmth);

      //prepare the vector of manual and running applications to restart
      //std::vector <Application_impl*> vector_man_and_runn_apps(0);         
     
      if (sysStatus == CdmwPlatformMngt::SYSTEM_INITIAL ||
      sysStatus == CdmwPlatformMngt::SYSTEM_STOPPED)
    {
      req_status = CdmwPlatformMngt::LONG_REQUEST_COMPLETED;
      stop_req_done = true;
    }
   

   
      // if stop request not done :
      // stopping is in progress : condition flag is not set
      // stop is done now with error or not (previously in progress) : flag is set
      // or application in Initialising, Initialised, Suspended, 
      // RunRequest, Running state 
      // (flag not set)                    
      if (!stop_req_done)
      {
          try {
        
        // first lock mutex for condition
            CDMW_MUTEX_GUARD (m_mutex);
            m_stop_condition_flag = false;

            if (sysStatus != CdmwPlatformMngt::SYSTEM_STOPPING) {
                        
            // stop system
            // exception is thrown in case of incompatible status
            stop (emergency);
            }
            
            // stopping is in progress
            // so wait the condition flag is set by broadcasting
            while (!m_stop_condition_flag) {
            m_stop_condition.wait ();
                }
            
            m_stop_condition_flag = false;
            
            // get the result
            if ( m_stop_status != CdmwPlatformMngt::LONG_REQUEST_COMPLETED) {
            return m_stop_status;           
            }
        }
        
        catch (...) {
            throw;
        }
    }
        try {
        start ( startup_kind ); 
    
        std::vector<Application_impl*> app_impls =  get_application_impls();

        for (unsigned int i=0; i<app_impls.size(); i++)
        {                    
            if (true == app_impls[i] -> created_dynamically() )
            { 
                if (false == app_impls[i] ->get_force_manual_start())
                { 
                    //CGi the control is moved on the initalisation phase                    
                    app_impls[i] -> initialise_and_wait(startup_kind);                  
                    app_impls[i] -> run();
                }
            }
            //MR
            //app_impls[i]->set_requested_restart_warmth( app_impls[i]-> get_default_restart_warmth () );
        }
      }
        catch (...) {
            throw;
        }

      /* Reset the system default_restart_warmth */
      //MR
      //set_requested_restart_warmth ( get_default_restart_warmth () );
      return CdmwPlatformMngt::LONG_REQUEST_COMPLETED;
}

    std::vector<Application_impl*> CentralisedSupervisionSystem_impl::get_application_impls()
    {
      std::vector<Application_impl*> app_vect;
      ApplicationServants::iterator applicationIt;

      for (applicationIt = m_application_servants.begin();
       applicationIt != m_application_servants.end();
       applicationIt++)
        {
      Application_impl* application = (applicationIt->second).application;
      app_vect.push_back(application); 
        }
      return app_vect;
    }

    Application_impl* CentralisedSupervisionSystem_impl::
        get_application_impl (const char *application_name) throw (CdmwPlatformMngt::ApplicationNotFound, CORBA::SystemException)
    {
        CDMW_MUTEX_GUARD (m_mutex);
        Application_impl* application = 0;
        ApplicationServants::iterator applicationIt;

        for (applicationIt = m_application_servants.begin();
             applicationIt != m_application_servants.end();
             applicationIt++)
        {    
            if (!strcmp( (applicationIt->second).application->name(), application_name ))
            { 
                application = (applicationIt->second).application;
                break;            
            }
        }
        return application; 
    }

    CORBA::ULong CentralisedSupervisionSystem_impl::get_number_of_application_with_relevant_indicator()
       throw()
    {
         CORBA::ULong count = 0L;
         ApplicationServants::iterator applicationIt;

         for (applicationIt = m_application_servants.begin();
              applicationIt != m_application_servants.end();
              applicationIt++)
         {
             if ((applicationIt->second).application->get_indicator_relevant_status())
                 ++count;
         }
         return count;
     }


  } // End namespace PlatformMngt
} // End namespace Cdmw

