/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

#include "platformsystemlibrary/HostStates.hpp"
#include <iostream>


namespace Cdmw
{
namespace PlatformMngt
{

const std::string HostStateMachine::m_HostRespondingName = HOST_RESPONDING_NAME;
const std::string HostStateMachine::m_HostStartingName = HOST_STARTING_NAME;
const std::string HostStateMachine::m_HostStartedName = HOST_STARTED_NAME;
const std::string HostStateMachine::m_HostPartialStartedName = HOST_PARTIAL_STARTED_NAME;
const std::string HostStateMachine::m_HostStoppingName = HOST_STOPPING_NAME;
const std::string HostStateMachine::m_HostUnreachableName = HOST_UNREACHABLE_NAME;


//HostStateMachine::HostStateMachine( Host_impl* hi, const std::string& s ) 
HostStateMachine::HostStateMachine( Host_impl* hi) 
    throw (OutOfMemoryException)
    : BaseHostStateMachine(hi, HOST_STATEMACHINE_NAME)
{
    try {
        std::auto_ptr<HostResponding> responding (new HostResponding ());
        std::auto_ptr<HostStarting> starting (new HostStarting ());
        std::auto_ptr<HostStarted> started (new HostStarted ());
        std::auto_ptr<HostPartialStarted> partial_started (new HostPartialStarted ());
        std::auto_ptr<HostStopping> stopping (new HostStopping ());
        std::auto_ptr<HostUnreachable> unreachable (new HostUnreachable ());
                
        m_HostResponding = responding.get ();
        m_HostStarting = starting.get();
        m_HostStarted = started.get();
        m_HostPartialStarted = partial_started.get();
        m_HostStopping = stopping.get();
        m_HostUnreachable = unreachable.get();
        
        addState(m_HostUnreachableName, m_HostUnreachable);
        addState(m_HostRespondingName, m_HostResponding);
        addState(m_HostStartingName, m_HostStarting);
        addState(m_HostStartedName, m_HostStarted);
        addState(m_HostPartialStartedName, m_HostPartialStarted);
        addState(m_HostStoppingName, m_HostStopping);        

        setState(m_HostRespondingName);
    
        responding.release();
        starting.release();
        started.release();
        partial_started.release();
        stopping.release();
        unreachable.release();
    }
    catch(const std::bad_alloc &)
    {
        CDMW_THROW (OutOfMemoryException);
    }
}

HostStateMachine::~HostStateMachine()
{
//std::cerr << "HostStateMachine d'ctor" << std::endl;
       if (m_HostResponding != 0)
        {
            delete m_HostResponding;
            m_HostResponding = 0;
        }
        
        if (m_HostStarting != 0)
        {
            delete m_HostStarting;
            m_HostStarting = 0;
        }
        
        if (m_HostStarted != 0)
        {
            delete m_HostStarted;
            m_HostStarted = 0;
        }
        
        if (m_HostPartialStarted != 0)
        {
            delete m_HostPartialStarted;
            m_HostPartialStarted = 0;
        }
        
        if (m_HostStopping != 0)
        {
            delete m_HostStopping;
            m_HostStopping = 0;
        }
        
        if (m_HostUnreachable != 0)
        {
            delete m_HostUnreachable;
            m_HostUnreachable = 0;
        }
}

void HostStateMachine::setState(CdmwPlatformMngt::HostStatus hs)
{
    std::string new_host_state = "";
    switch(hs)
    {
    case CdmwPlatformMngt::HOST_RESPONDING :
        new_host_state = m_HostRespondingName;
        break;
    case CdmwPlatformMngt::HOST_STARTING :
        new_host_state = m_HostStartingName;
        break;
    case CdmwPlatformMngt::HOST_STARTED :
        new_host_state = m_HostStartedName;
        break;
    case CdmwPlatformMngt::HOST_PARTIAL_STARTED :
        new_host_state = m_HostPartialStartedName;
        break;
    case CdmwPlatformMngt::HOST_STOPPING :
        new_host_state = m_HostStoppingName;
        break;
    case CdmwPlatformMngt::HOST_UNREACHABLE :
        new_host_state = m_HostUnreachableName;
        break;
    default:
        return;
    }
    
    // Set the new status calling the parent function
    BaseHostStateMachine::setState(new_host_state);
}

// We have to redefine this function to make it visible outside
void HostStateMachine::setState(const std::string& stateName)
{
    BaseHostStateMachine::setState(stateName);
}

void HostState::entryAction (BaseHostStateMachine* stateMachine)
{
    if (stateMachine != NULL) {
        Host_impl* host = stateMachine->getContext();
        if (host != NULL) {
            host->notify_status_change();
        }
    }
}
    
void HostState::start(HostStateMachine* stateMachine)
    throw (CdmwPlatformMngt::IncompatibleStatus)
{
    throw CdmwPlatformMngt::IncompatibleStatus();    
}

void HostState::stop(HostStateMachine* stateMachine)
    throw (CdmwPlatformMngt::IncompatibleStatus)
{
    throw CdmwPlatformMngt::IncompatibleStatus();
}

void HostState::reinsert(HostStateMachine* stateMachine)
    throw (CdmwPlatformMngt::IncompatibleStatus)
{
    throw CdmwPlatformMngt::IncompatibleStatus();
}

void HostState::notify_daemon_status(HostStateMachine* stateMachine, DaemonStatus ds) 
    throw (CdmwPlatformMngt::IncompatibleStatus)
{
    if (stateMachine != NULL) {
        if (ds == HostState::DAEMON_NOT_RUNNING) {
            stateMachine->setState(HostStateMachine::m_HostUnreachableName);
        }
    }
}

CdmwPlatformMngt::HostStatus HostResponding::status()
{
    return CdmwPlatformMngt::HOST_RESPONDING;
}

void HostResponding::start(HostStateMachine* stateMachine)
    throw (CdmwPlatformMngt::IncompatibleStatus)
{
    if (stateMachine != NULL) {
        stateMachine->setState(HostStateMachine::m_HostStartingName);
    }
}

void HostResponding::stop(HostStateMachine* stateMachine)
    throw (CdmwPlatformMngt::IncompatibleStatus)
{
    if (stateMachine != NULL) {

        Host_impl* host = stateMachine->getContext();
        if (host != NULL) {

           host->stop_processes();
        }


    }

}

void HostResponding::notify_processes_status(HostStateMachine* stateMachine, ProcessesStatus ps)
    throw (CdmwPlatformMngt::IncompatibleStatus) 
{
    if (stateMachine != NULL) {
        if (ps == HostState::NOT_ALL_PROCESSES_RUNNING) {
            stateMachine->setState(HostStateMachine::m_HostPartialStartedName);
        }
        else if (ps == HostState::ALL_PROCESSES_RUNNING) {
            stateMachine->setState(HostStateMachine::m_HostStartedName);
        
        }
    }
}

CdmwPlatformMngt::HostStatus HostReinserting::status()
{
    return CdmwPlatformMngt::HOST_REINSERTING;
}

CdmwPlatformMngt::HostStatus HostStarting::status()
{
    return CdmwPlatformMngt::HOST_STARTING;
}

void HostStarting::doActivity (BaseHostStateMachine* stateMachine)
{
    //TODO: start all host's processes here
    if (stateMachine == NULL) {
        return;
    }
    
    Host_impl* host = stateMachine->getContext();
    if (host != NULL) {
        host->start_processes();
    }  
}
    
void HostStarting::stop(HostStateMachine* stateMachine)
    throw (CdmwPlatformMngt::IncompatibleStatus)
{
    if (stateMachine != NULL) {
        stateMachine->setState(HostStateMachine::m_HostStoppingName);
    }
}

void HostStarting::notify_processes_status(HostStateMachine* stateMachine, ProcessesStatus ps)
    throw (CdmwPlatformMngt::IncompatibleStatus) 
{
    if (stateMachine != NULL) {
        if (ps == HostState::NOT_ALL_PROCESSES_RUNNING) {
            stateMachine->setState(HostStateMachine::m_HostPartialStartedName);
        }
        else if (ps == HostState::ALL_PROCESSES_RUNNING) {
            stateMachine->setState(HostStateMachine::m_HostStartedName);
        }
    }
}

CdmwPlatformMngt::HostStatus HostStarted::status()
{
    return CdmwPlatformMngt::HOST_STARTED;
}

void HostStarted::stop(HostStateMachine* stateMachine)
    throw (CdmwPlatformMngt::IncompatibleStatus)
{
    if (stateMachine != NULL) {
        stateMachine->setState(HostStateMachine::m_HostStoppingName);
    }
}

void HostStarted::notify_processes_status(HostStateMachine* stateMachine, ProcessesStatus ps) 
    throw (CdmwPlatformMngt::IncompatibleStatus)
{
    if (stateMachine != NULL) {
        if (ps == HostState::NOT_ALL_PROCESSES_RUNNING) {
            stateMachine->setState(HostStateMachine::m_HostPartialStartedName);
        }
        else if (ps == HostState::NO_PROCESS_RUNNING ) {
            stateMachine->setState(HostStateMachine::m_HostRespondingName);
        }
    }
}

void HostStarted::doActivity(BaseHostStateMachine* stateMachine)
{
    Host_impl* host = stateMachine->getContext();
    host->set_restart_attempt (0);
    CDMW_INFO(Cdmw::Logging::Logger::Get_Logger(CdmwPlatformMngtBase::PFMMNG),
                            "<<HostStates::HostStarted::doActivity >> The number of restart attempts is set to zero ");
}

CdmwPlatformMngt::HostStatus HostPartialStarted::status()
{
    return CdmwPlatformMngt::HOST_PARTIAL_STARTED;
}

void HostPartialStarted::stop(HostStateMachine* stateMachine)
    throw (CdmwPlatformMngt::IncompatibleStatus)
{
    if (stateMachine != NULL) {
        stateMachine->setState(HostStateMachine::m_HostStoppingName);
    }
}

void HostPartialStarted::notify_processes_status(HostStateMachine* stateMachine, ProcessesStatus ps) 
    throw (CdmwPlatformMngt::IncompatibleStatus)
{
    if (stateMachine != NULL) {
        if (ps == HostState::ALL_PROCESSES_RUNNING) {
            stateMachine->setState(HostStateMachine::m_HostStartedName);
        }
        else if (ps == HostState::NO_PROCESS_RUNNING ) {
            stateMachine->setState(HostStateMachine::m_HostRespondingName);
        }
    }    
}

CdmwPlatformMngt::HostStatus HostStopping::status()
{
    return CdmwPlatformMngt::HOST_STOPPING;
}

void HostStopping::entryAction (BaseHostStateMachine* stateMachine)
{
    
     HostState::entryAction( stateMachine );
    //TODO: set the stop_in_progress flag
    if (stateMachine == NULL) return;
              
    Host_impl* host = stateMachine->getContext();   
    
    if (host != NULL)
    {
        host->set_stop_in_progress(true);
    }    
} 


void HostStopping::doActivity (BaseHostStateMachine* stateMachine)
{
    //TODO: stop all host's processes here
    if (stateMachine == NULL) return;
        
    Host_impl* host = stateMachine->getContext();
    if (host != NULL) {   
    
        host->stop_processes();
    }  

}


void HostStopping::stop(HostStateMachine* stateMachine)
    throw (CdmwPlatformMngt::IncompatibleStatus)
{
    // Do nothing, host is already stopping
    std::cerr << "HostStopping::stop method called, this command does nothing ! " << std::endl;              
}

void HostStopping::notify_processes_status(HostStateMachine* stateMachine, ProcessesStatus ps) 
    throw (CdmwPlatformMngt::IncompatibleStatus)
{
    if (stateMachine != NULL) {
        if (ps == HostState::NO_PROCESS_RUNNING) {
            stateMachine->setState(HostStateMachine::m_HostRespondingName);
        }
    }    
}

CdmwPlatformMngt::HostStatus HostUnreachable::status()
{
    return CdmwPlatformMngt::HOST_UNREACHABLE;
}

void HostUnreachable::notify_daemon_status(HostStateMachine* stateMachine, DaemonStatus ds)
    throw (CdmwPlatformMngt::IncompatibleStatus)
{
    if (stateMachine != NULL) {
        if (ds == HostState::DAEMON_RUNNING) {
            stateMachine->setState(HostStateMachine::m_HostRespondingName);
        }
    }
}

void HostUnreachable::exitAction (BaseHostStateMachine* stateMachine)
{
    //TODO: set the stop_in_progress flag
    if (stateMachine == NULL) return;            
                      
    Host_impl* host = stateMachine->getContext();
    
    if (host != NULL)
    {
        host->set_stop_in_progress(false);       
    }    
} 


/*
void HostUnreachable::stop(HostStateMachine* stateMachine)
    throw (CdmwPlatformMngt::IncompatibleStatus)
{
    // Force state machine to re-enter unreachable state to make event be sent
    if (stateMachine != NULL) {
        stateMachine->setState(HostStateMachine::m_HostUnreachableName);
    }
}
*/

}
}




