/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

#include "platformsystemlibrary/HostStatusConsistencyChecker.hpp"
#include "platformsystemlibrary/HostStates.hpp"
#include <SystemMngt/platformlibrary/LogMngr.hpp>
#include <Foundation/common/Exception.hpp>
#include <Foundation/orbsupport/ExceptionMinorCodes.hpp>
#include <Foundation/osthreads/MutexGuard.hpp>
#include <Foundation/ossupport/OS.hpp>
#include "platformsystemlibrary/Host_impl.hpp"


namespace Cdmw
{
  namespace PlatformMngt
  {
    HostStatusConsistencyChecker::HostStatusConsistencyChecker(Host_impl* attaching_host, 
        HostStateMachine* state_machine) :  m_all_processes(0), m_seq_size(0)
    {
        m_attached_host = attaching_host;
        m_state_machine = state_machine;

        m_seq_size = 0;

        initSleepingPeriod();
        initStopFlag();
    }

    HostStatusConsistencyChecker::~HostStatusConsistencyChecker() throw()
    {
        if (!getStopFlag())
        {
           stop();
        }
    }

    void HostStatusConsistencyChecker::stop() throw()
    {
        CDMW_MUTEX_GUARD(m_stop_flag_mutex);
        m_stop_flag = true;

//        throw CORBA::NO_IMPLEMENT (OrbSupport::NO_IMPLEMENT, CORBA::COMPLETED_NO);
    }

    void HostStatusConsistencyChecker::run() throw()
    {
        try
        {
          execute();
        }
        catch(...)
        {
            CDMW_ERROR(Cdmw::Logging::Logger::Get_Logger(CdmwPlatformMngtBase::PFMMNG), 
            "HostStatusConsistencyChecker::run() Unknown type caught exception " << m_attached_host->name());
            throw;
        }

    }

    void HostStatusConsistencyChecker::execute() throw ()
    {
      try
      {
/*
           CDMW_INFO(Cdmw::Logging::Logger::Get_Logger(CdmwPlatformMngtBase::PFMMNG),
                 " HostStatusConsistencyChecker::execute () " << m_attached_host->name());
*/
            CdmwPlatformMngt::ProcessesIterator_var proc_iter;
            m_seq_size = m_attached_host->get_number_of_processes();
            m_all_processes = m_attached_host->get_processes(m_seq_size, proc_iter.out());

           while (!getStopFlag())
           {
/*
                CDMW_INFO(Cdmw::Logging::Logger::Get_Logger(CdmwPlatformMngtBase::PFMMNG),
                    " HostStatusConsistencyChecker::execute () into the while " << m_attached_host->name());
*/
                OsSupport::OS::sleep (getSleepingPeriod());

                if (m_seq_size > 0)
                {
/*
                   CDMW_INFO(Cdmw::Logging::Logger::Get_Logger(CdmwPlatformMngtBase::PFMMNG),
                      " HostStatusConsistencyChecker::execute () into the while " << m_attached_host->name() << " " << m_attached_host->get_number_of_processes());
*/
                   computeStatusConsistency ();
                }
                else
                {
/*
                   CDMW_INFO(Cdmw::Logging::Logger::Get_Logger(CdmwPlatformMngtBase::PFMMNG),
                       " HostStatusConsistencyChecker::execute () into the while " << m_attached_host->name() << " Assigning sequence ");
*/
                    m_seq_size = m_attached_host->get_number_of_processes();
                    m_all_processes = m_attached_host->get_processes(m_seq_size, proc_iter.out());
                }

           }
      }
      catch(const CORBA::SystemException& ex)
      {
           CDMW_FATAL(Cdmw::Logging::Logger::Get_Logger(CdmwPlatformMngtBase::PFMMNG),
               "HostStatusConsistencyChecker::execute() CORBA::SystemException caught exception " << m_attached_host->name());
           throw;
      }
      catch (...)
      {
            CDMW_FATAL(Cdmw::Logging::Logger::Get_Logger(CdmwPlatformMngtBase::PFMMNG),
               "HostStatusConsistencyChecker::execute() Unknown type caught exception " << m_attached_host->name());

           throw;
      }
    }

    void HostStatusConsistencyChecker::computeStatusConsistency ()
       throw (CORBA::SystemException)
    {
     try
     {
/*
         CDMW_INFO(Cdmw::Logging::Logger::Get_Logger(CdmwPlatformMngtBase::PFMMNG),
                 "HostStatusConsistencyChecker::computeStatusConsistency ()" <<
                   "Compute started: " << m_attached_host->name());
*/
         // Check the status af all the processes of this host
         HostState::ProcessesStatus status = HostState::NO_PROCESS_RUNNING;
         CORBA::ULong running_processes = 0;
         CORBA::ULong not_running_processes = 0;
         CORBA::ULong ended_processes = 0;
         CORBA::ULong processes_with_relevant_indicator = 0;
         CORBA::String_var status_info;
//         {//mutex scope
//            CDMW_MUTEX_GUARD(*(m_system->get_consistency_status_recursive_mutex()));
/*
         int size = m_seq_size;
         CDMW_INFO(Cdmw::Logging::Logger::Get_Logger(CdmwPlatformMngtBase::PFMMNG),
                 "HostStatusConsistencyChecker::computeStatusConsistency () " <<
                   "Compute started: " << m_attached_host->name() << " " << size);
*/
            for (CORBA::ULong i=0; i<m_seq_size; i++)
            {
                if (CORBA::is_nil((*m_all_processes)[i]))
                {
                    CDMW_WARN(Cdmw::Logging::Logger::Get_Logger(CdmwPlatformMngtBase::PFMMNG),
                       "HostStatusConsistencyChecker::computeStatusConsistency ()" <<
                       "Compute started: " << m_attached_host->name() << " processes sequence not coherent " );
                }
                else
                {
                    CdmwPlatformMngt::ProcessStatus current_status = ((*m_all_processes)[i])->get_status(status_info.out());
/*
                    CDMW_INFO(Cdmw::Logging::Logger::Get_Logger(CdmwPlatformMngtBase::PFMMNG),
                       "HostStatusConsistencyChecker::computeStatusConsistency ()" <<
                       "Compute started: " << m_attached_host->name() << " " << (
                       (*m_all_processes)[i])->name() << " " << status_info);
*/
                    if ( ((*m_all_processes)[i])-> get_indicator_relevant_status() )
                    {
                        processes_with_relevant_indicator++;

                        if ( (current_status == CdmwPlatformMngt::PROCESS_RUNNING )
                              ||
                             (current_status == CdmwPlatformMngt::PROCESS_RUN_REQUEST )
                              ||
                             (current_status == CdmwPlatformMngt::PROCESS_INITIALISED ) )
                        {
                             running_processes++;
                        }
                        else if ( current_status == CdmwPlatformMngt::PROCESS_ENDED )
                        {
                             ended_processes++;
                        }
                        else
                        {
                             not_running_processes++;
                        }
                        if ((running_processes > 0) && (not_running_processes > 0))
                        {
                            // If we are here we already know what we need...
                            break;
                        }
                    }
                }
            }

//         }//end mutex scope
/*
         CDMW_INFO(Cdmw::Logging::Logger::Get_Logger(CdmwPlatformMngtBase::PFMMNG),
                 "HostStatusConsistencyChecker::computeStatusConsistency () " <<
                   "HostState checking: " << m_attached_host->name());
*/
         //Check if all process in in ended, n this case the host state must be RESPONDING so we set status to NO_PROCESS_RUNNING
         if (ended_processes == processes_with_relevant_indicator )
         {
             status = HostState::NO_PROCESS_RUNNING;
/*
             CDMW_INFO(Cdmw::Logging::Logger::Get_Logger(CdmwPlatformMngtBase::PFMMNG),
                 "HostStatusConsistencyChecker::computeStatusConsistency ()" <<
                   "HostState checking: HostState::NO_PROCESS_RUNNING " << m_attached_host->name());
*/
         }
         else if (running_processes == (processes_with_relevant_indicator - ended_processes) )
         {
             status = HostState::ALL_PROCESSES_RUNNING;
/*
             CDMW_INFO(Cdmw::Logging::Logger::Get_Logger(CdmwPlatformMngtBase::PFMMNG),
                 "HostStatusConsistencyChecker::computeStatusConsistency ()" <<
                   "HostState checking: HostState::ALL_PROCESSES_RUNNING " << m_attached_host->name());
*/
         }
         else if (not_running_processes == (processes_with_relevant_indicator - ended_processes))
         {
             status = HostState::NO_PROCESS_RUNNING;
/*
             CDMW_INFO(Cdmw::Logging::Logger::Get_Logger(CdmwPlatformMngtBase::PFMMNG),
                 "HostStatusConsistencyChecker::computeStatusConsistency ()" <<
                   "HostState checking: HostState::NO_PROCESS_RUNNING " << m_attached_host->name());
*/
         }
         else
         {
             status = HostState::NOT_ALL_PROCESSES_RUNNING;
/*
             CDMW_INFO(Cdmw::Logging::Logger::Get_Logger(CdmwPlatformMngtBase::PFMMNG),
                 "HostStatusConsistencyChecker::computeStatusConsistency ()" <<
                   "HostState checking: HostState::NOT_ALL_PROCESSES_RUNNING " << m_attached_host->name());
*/
         }

          // Notify the statemachine according to the state found
          try
          {
/*
             CDMW_INFO(Cdmw::Logging::Logger::Get_Logger(CdmwPlatformMngtBase::PFMMNG),
                 "HostStatusConsistencyChecker::computeStatusConsistency ()" <<
                   "HostState retrieving: " << m_attached_host->name());
*/
                HostState* hs = m_state_machine->getState();
                if (hs != 0)
                {
/*
                 CDMW_INFO(Cdmw::Logging::Logger::Get_Logger(CdmwPlatformMngtBase::PFMMNG),
                      "HostStatusConsistencyChecker::computeStatusConsistency ()" <<
                      "HostState notify_processes_status: " << m_attached_host->name());
*/
                     hs->notify_processes_status(m_state_machine, status);
                }
           }
           catch (const CdmwPlatformMngt::IncompatibleStatus& e)
           {
               CDMW_ERROR(Cdmw::Logging::Logger::Get_Logger(CdmwPlatformMngtBase::PFMMNG),
                "HostStatusConsistencyChecker::computeStatusConsistency () " << 
                    "IncompatibleStatus for host: " << m_attached_host->name());
//             throw CORBA::SystemException (OrbSupport::NO_IMPLEMENT, CORBA::COMPLETED_NO);
           }
           catch (...)
           {
               CDMW_ERROR(Cdmw::Logging::Logger::Get_Logger(CdmwPlatformMngtBase::PFMMNG),
                 "HostStatusConsistencyChecker::computeStatusConsistency () " <<
                    "Unexpected exception for host: " << m_attached_host->name());
               throw;
           }
        }
        catch (const CORBA::SystemException& )
        {
             CDMW_ERROR(Cdmw::Logging::Logger::Get_Logger(CdmwPlatformMngtBase::PFMMNG),
                 "HostStatusConsistencyChecker::computeStatusConsistency () " <<
                   "CORBA::SystemException for host: " << m_attached_host->name());

             CDMW_WARN(Cdmw::Logging::Logger::Get_Logger(CdmwPlatformMngtBase::PFMMNG),
                    "HostStatusConsistencyChecker::computeStatusConsistency () " <<
                    "Processes sequence changed, resetting parameters " << m_attached_host->name());

             CdmwPlatformMngt::ProcessesIterator_var proc_iter;
             m_seq_size = m_attached_host->get_number_of_processes();
             m_all_processes = m_attached_host->get_processes(m_seq_size, proc_iter.out());
        }
        catch (...) 
        {
             CDMW_ERROR(Cdmw::Logging::Logger::Get_Logger(CdmwPlatformMngtBase::PFMMNG),
                 "HostStatusConsistencyChecker::computeStatusConsistency () " <<
                   "Unknown Exception for host: " << m_attached_host->name());
//           throw CORBA::SystemException (OrbSupport::NO_IMPLEMENT, CORBA::COMPLETED_NO);
        }
}

     long HostStatusConsistencyChecker::get_sleeping_period()
     {
         return getSleepingPeriod();
     }

     void HostStatusConsistencyChecker::set_sleeping_period(long sleeping_period)
     {
         if (sleeping_period > 0)
         {
             CDMW_MUTEX_GUARD(m_sleeping_period_mutex);
              m_sleeping_period = sleeping_period;
         }
     }

     bool HostStatusConsistencyChecker::getStopFlag ()
     {
         CDMW_MUTEX_GUARD(m_stop_flag_mutex);
         return m_stop_flag;
     }

     void HostStatusConsistencyChecker::initStopFlag()
     {
         CDMW_MUTEX_GUARD(m_stop_flag_mutex);
         m_stop_flag = false;
     }

     long HostStatusConsistencyChecker::getSleepingPeriod()
     {
         CDMW_MUTEX_GUARD(m_sleeping_period_mutex);
         return m_sleeping_period;
     }

     void HostStatusConsistencyChecker::initSleepingPeriod()
     {
         CDMW_MUTEX_GUARD(m_sleeping_period_mutex);
          //Default period is 1500 ms
          m_sleeping_period = 1500;
     }

  }// closing namespaces
}
