
/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#include <limits>
#include <iostream>
#include <stdlib.h>
#include <sstream>
#include "platformsystemlibrary/Host_impl.hpp"
#include "platformsystemlibrary/System_impl.hpp"
#include "platformsystemlibrary/SystemLogMessageId.hpp"
#include "platformsystemlibrary/DaemonMonitor.hpp"
#include "platformsystemlibrary/HostStates.hpp"

#include "SystemMngt/platformlibrary/Configuration.hpp"
#include "SystemMngt/platformlibrary/EventHeaderFactory.hpp"
#include "SystemMngt/platformlibrary/LogMngr.hpp"
#include "SystemMngt/platformlibrary/SupervisionEventNotifier.hpp"
#include "SystemMngt/platformlibrary/EventNotifierManager.hpp"

#include "SystemMngt/platformelementlibrary/EntityContainer_impl.hpp"
#include "SystemMngt/platformelementlibrary/RWEntityContainer_impl.hpp"
#include "SystemMngt/platformelementlibrary/EntityContainer.hpp"
#include "SystemMngt/platformvaluetypes/HostStatusChange_impl.hpp"
#include "SystemMngt/platforminterface/ServiceNames.hpp"
#include "platformapplicationlibrary/ProcessesIterator_impl.hpp"
#include "platformapplicationlibrary/Application_impl.hpp"
#include "platformapplicationlibrary/Process_impl.hpp"
#include "idllib/CdmwPlatformMngtSystem.stub.hpp"
#include "idllib/CdmwPlatformMngtApplication.stub.hpp"
#include "idllib/CdmwPlatformMngtProcess.stub.hpp"
#include "idllib/CdmwPlatformMngtPredefinedProperties.stub.hpp"
#include "SystemMngt/tools/ManagedElementKey.hpp"
#include <Foundation/common/Locations.hpp>
#include "Foundation/orbsupport/ExceptionMinorCodes.hpp"
#include "Foundation/osthreads/MutexGuard.hpp"
#include "Foundation/commonsvcs/datastore/StorageErrorException.hpp"
#include "Repository/repositoryinterface/RepositoryInterface.hpp"
#include "SystemMngt/idllib/CdmwPlatformMngtDaemonController.skel.hpp"
#include "SystemMngt/tools/HostIssuerManager.hpp"
#include "platformsystemlibrary/HostStatusConsistencyChecker.hpp"

using namespace std;

namespace Cdmw
{
namespace PlatformMngt
{


class Host_impl;
class System_impl;

// ----------------------------------------------------------------------
// HostRespondingCallback class.
// ----------------------------------------------------------------------
HostRespondingCallback::HostRespondingCallback (Host_impl* host)
    : Callback(), m_host(host)
{
}

HostRespondingCallback::~HostRespondingCallback ()
{
}

void HostRespondingCallback::execute()
    throw()
{
    if (m_host != NULL)
    {
        if (m_host->m_state_machine != NULL)
        {
            m_host->m_state_machine->getState()
                ->notify_daemon_status(m_host->m_state_machine, HostState::DAEMON_RUNNING);
        }
    }
}

// ----------------------------------------------------------------------
// HostNotRespondingCallback class.
// ----------------------------------------------------------------------
HostNotRespondingCallback::HostNotRespondingCallback (Host_impl* host)
    : Callback(), m_host(host)
{
}

HostNotRespondingCallback::~HostNotRespondingCallback ()
{
}

void HostNotRespondingCallback::execute()
    throw()
{
    if (m_host != NULL)
    {
        if (m_host->m_state_machine != NULL)
        {
            m_host->m_state_machine->getState()
                ->notify_daemon_status(m_host->m_state_machine, HostState::DAEMON_NOT_RUNNING);
        }
    }
}


// ----------------------------------------------------------------------
// Host_impl class.
// ----------------------------------------------------------------------

static const std::string host_status_strings[] =
{
        "Responding",
        "Reinserting",
        "Starting",
        "Started",
        "PartialStarted",
        "Stopping",
        "Unreachable"
};
static const CdmwPlatformMngt::HostStatus Status[]=
{ 
        CdmwPlatformMngt::HOST_RESPONDING,
        CdmwPlatformMngt::HOST_REINSERTING,
        CdmwPlatformMngt::HOST_STARTING,
        CdmwPlatformMngt::HOST_STARTED,
        CdmwPlatformMngt::HOST_PARTIAL_STARTED,
        CdmwPlatformMngt::HOST_STOPPING,
        CdmwPlatformMngt::HOST_UNREACHABLE
};

#ifdef CDMW_USE_FAULTTOLERANCE
Host_impl::Host_impl (PortableServer::POA_ptr poa,
        const CdmwPlatformMngt::HostDef& host_def,
        const char* host_path,
        System_impl* system,
        CdmwPlatformMngt::RWElementRepository_ptr element_repository,
        CdmwFT::FaultManagement::FaultDetector_ptr faultDetector)
#else
Host_impl::Host_impl (PortableServer::POA_ptr poa,
        const CdmwPlatformMngt::HostDef& host_def,
        const char* host_path,
        System_impl* system,
        CdmwPlatformMngt::RWElementRepository_ptr element_repository)
#endif

throw (BadParameterException,
       OutOfMemoryException,
       CosPropertyService::MultipleExceptions)

: DeactivableServant_impl (poa),
  ManagedElement_impl (element_repository, poa),
  m_state_machine(NULL)

{
    m_actualEntityContainer = NULL;
    m_reboot_in_progress = false;
    m_stop_in_progress = false;
    m_default_entitydefs = NULL;
    set_status_info ( CORBA::string_dup (""));
    m_current_host_command = NO_COMMAND;
    m_stop_host_script = "cdmw_remote_host_stop.sh";
    m_start_host_script = "cdmw_remote_host_start.sh";
    m_reboot_host_script = "cdmw_remote_host_reboot.sh";
    m_shutdown_host_script = "cdmw_remote_host_shutdown.sh";

    try
    {
        if (system == NULL)
        {
            CDMW_THROW2 (BadParameterException, "system", "NULL");
        }

        //Enable PropertyEvent features
        set_property_event_notifier(EventNotifierManager::Instance()->get_property_notifier());
        //Enable StatusChangeEvent features
        set_status_event_notifier(EventNotifierManager::Instance()->get_status_notifier());

        // Set the internal data
        m_system = system;

        // Set event notifier and register host as observer
        m_event_notifier = m_system->get_event_notifier();

        m_monitor = NULL;
        
        m_hostRestartFactory = 0;
        m_hostStopFactory = 0;
        m_hostRebootFactory = 0;
        m_hostShutdownFactory = 0;
        
        try
        {
            Cdmw::CommonSvcs::DataStore::DataStoreBase* ds =
                Cdmw::CommonSvcs::DataStore::DataStoreRegistry::Get_data_store(
                        Configuration::Get_host_datastore_id());
            m_host_ds = dynamic_cast<Cdmw::PlatformMngt::HostDataStore*>(ds);
        }
        catch (const NotExistException& e)
        {
            CDMW_ASSERT(false);
        }

        // store element path in managed element
        set_element_path (host_path, host_def.host_name.in());

        // create entity container
        std::auto_ptr <PlatformMngt::EntityContainer> actualEntityContainer (
                new PlatformMngt::EntityContainer(poa, m_element_repository.in(),
                        m_element_path.in()));
        m_rwEntityContainerServant = new RWEntityContainer_impl (
                poa, actualEntityContainer.get(), m_element_repository.in());
        m_rwEntityContainer = dynamic_cast <RWEntityContainer_impl*> (m_rwEntityContainerServant.in());

        // Complete the internal data
        m_actualEntityContainer = actualEntityContainer.get();

        // initialise data store of the object
        create_record_in_datastore(host_def);

        std::auto_ptr <HostStatusChangeFactory> host_status_change_factory(
            new HostStatusChangeFactory());
        m_host_status_change_factory = host_status_change_factory.release();

        try{
            // define TECH_STATUS and TECH_STATUS_INFO properties
            const char* init_value = "";
            define_status_property(init_value);
        }
        catch(...)
        {
            throw  CosPropertyService::MultipleExceptions();
        }
        // Create the state machine, this could throw OutOfMemoryException
        std::auto_ptr <HostStateMachine> state_machine(new HostStateMachine(this));
        m_state_machine = state_machine.release();

        m_host_status_checker = new HostStatusConsistencyChecker (this, m_state_machine);

        m_host_status_checker->set_sleeping_period(m_system->get_host_status_check_period());

        //create the command's factory
        std::auto_ptr <RestartFactory> hostRestartFactory (new RestartFactory ());
        m_hostRestartFactory= hostRestartFactory.release();
        
        std::auto_ptr < StopFactory > hostStopFactory (new StopFactory ());
        m_hostStopFactory= hostStopFactory.release();
        
        std::auto_ptr < ShutdownFactory > hostShutdownFactory (new ShutdownFactory ());
        m_hostShutdownFactory= hostShutdownFactory.release();
        
        std::auto_ptr < RebootFactory > hostRebootFactory (new RebootFactory ());
        m_hostRebootFactory= hostRebootFactory.release();
        
        // Check daemon status
        HostState::DaemonStatus daemon_status = HostState::DAEMON_NOT_RUNNING;
        try
        {
            CdmwPlatformMngt::Daemon_var daemon =
                Configuration::Get_daemon (get_element_name(), m_system->systemPort());
            if (!CORBA::is_nil(daemon.in()))
            {
                daemon_status = HostState::DAEMON_RUNNING;
            } 
        }
        catch (const CdmwPlatformMngt::HostNotFound&)
        {
        }
        catch (const CdmwPlatformMngt::HostNotReachable&)
        {
        }

        init_preemption();
           
        std::string element_path = std::string(get_element_path());
        Cdmw::PlatformMngt::HostIssuerManager::Instance()->attach_element(element_path);

        // Update datastore
//        {
//            // lock for concurent access
//            CDMW_MUTEX_GUARD(m_mutex_host_record);
//            CdmwPlatformMngt::HostRecord_var hr = get_record_from_datastore();
//            hr->status = m_state_machine->getState()->status();
//            update_record_in_datastore(hr.in());
//        }


#ifdef CDMW_USE_FAULTTOLERANCE
        m_faultDetector = CdmwFT::FaultManagement::FaultDetector::_duplicate(faultDetector);

        m_monitoringID = 0;
#else
        // create Daemon PullMonitor
        std::auto_ptr <DaemonMonitor>
        monitor(new DaemonMonitor(
                m_system,
                m_element_name.in(),
                host_def.daemon_monitoring_info.monitoring_interval,
                host_def.daemon_monitoring_info.monitoring_timeout));

        // create Callbacks
        std::auto_ptr <HostRespondingCallback>  responseCallback (
                new HostRespondingCallback (this));

        std::auto_ptr <HostNotRespondingCallback>  noResponseCallback (
                new HostNotRespondingCallback (this));

        m_monitor = monitor.release();

        // set callbacks in monitor
        m_monitor->setResponseCallback (responseCallback.release());
        m_monitor->setNoResponseCallback (noResponseCallback.release());
#endif

        // Release the ownership of the temporary objects
        actualEntityContainer.release();

                //init. the m_current_restart_attempt
        m_current_restart_attempt=0;

                //Add the Properties on Host
                unsigned int propCount = host_def.properties.length();
                for (unsigned int propIndex=0 ; propIndex < propCount ; propIndex++)
                {
                    const CdmwPlatformMngt::PropertyDef& prop_def = host_def.properties[propIndex];
                    std::string propertyName(host_def.properties[propIndex].property_name);
                    //CosPropertyService::PropertyModeType propertyMode = CosPropertyService::PropertyModeType(0);
                    //CdmwPlatformMngt::PropertyModeType propertyModeCdmw = host_def.properties[propIndex].mode_type;
                    //if (propertyModeCdmw == 1) propertyMode = CosPropertyService::PropertyModeType(1);
                    CosPropertyService::PropertyModeType propertyMode =
                        CosPropertyService::PropertyModeType(host_def.properties[propIndex].mode_type);
                    try
                    {
                        define_property_with_mode(propertyName.c_str() , prop_def.initial_value, propertyMode);
                    }
                    catch (...)
                    {
                        throw  CosPropertyService::MultipleExceptions();
                    }
                }

        // MdR define rw properties
        try
        {
            std::ostringstream property_name;

            property_name << CdmwPlatformMngt::CDMW_PREFIX << CdmwPlatformMngt::AUTORESTART_INFO << std::ends;
            if(!is_property_defined(property_name.str().c_str()))
            {
                CdmwPlatformMngt::AutoRestartInfo auto_restart_info;
                auto_restart_info.max_nbr_of_restart_attempt = 1;
                auto_restart_info.auto_restart_control = CdmwPlatformMngt::USE_COUNTER;
                CORBA::Any_var any = new CORBA::Any();
                *any <<= auto_restart_info;
                define_property_with_mode(property_name.str().c_str() , any.in(), CosPropertyService::fixed_normal);
            }
            property_name.str("");
            property_name << CdmwPlatformMngt::CDMW_PREFIX << CdmwPlatformMngt::DEFAULT_RESTART_WARMTH << std::ends;
            if(!is_property_defined(property_name.str().c_str()))
            {
                CORBA::Any_var any = new CORBA::Any();
                *any <<= CdmwPlatformMngt::HOT_RESTART;
                define_property_with_mode(property_name.str().c_str() , any.in(), CosPropertyService::fixed_normal);
            }
        }
        catch(...)
        {
            throw  CosPropertyService::MultipleExceptions();
        }


        CORBA::Any_var any_val = new CORBA::Any();

        /*
         * Get the System property GLOBAL_SUPERVISION_HOST
         * to initialise the Host property GLOBAL_SUPERVISION
         */

        CORBA::Any_var system_property_any = new CORBA::Any();
        const char* host_supervisor = 0;
        char* name = this->name();
        std::ostringstream property_name;
        try
        {
            try
            {
                m_system->get_attribute (CdmwPlatformMngt::GLOBAL_SUPERVISION_HOST,
                        system_property_any.out(),
                        CdmwPlatformMngt::PRE_DEFINED);
                system_property_any.in() >>= (host_supervisor);
            }
            catch(...)
            {
                CDMW_ERROR(Cdmw::Logging::Logger::Get_Logger(CdmwPlatformMngtBase::PFMMNG),
                "Cannot Proceed reading the property GLOBAL_SUPERVISION_HOST");
            }
            //Verify if the Host defined has the supervision running on it.
            if( 0 == strcmp(name,host_supervisor) )
            {
                *any_val <<= CORBA::Any::from_boolean(true);
            }
            else
            {
                *any_val <<= CORBA::Any::from_boolean(false);
            }
            // MdR define system properties not user defined
            property_name.str("");
            property_name << CdmwPlatformMngt::CDMW_PREFIX
            << CdmwPlatformMngt::GLOBAL_SUPERVISION << std::ends;
            define_property_with_mode(property_name.str().c_str(),
                    any_val.in(),
                    CosPropertyService::fixed_normal);

            get_attribute(CdmwPlatformMngt::DEFAULT_RESTART_WARMTH, any_val.out(), CdmwPlatformMngt::PRE_DEFINED);
            property_name.str("");
            property_name << CdmwPlatformMngt::CDMW_PREFIX << CdmwPlatformMngt::REQUESTED_RESTART_WARMTH << std::ends;
            define_property_with_mode(property_name.str().c_str(), any_val.in(), CosPropertyService::fixed_normal);
        }
        catch (...)
        {
            throw CosPropertyService::MultipleExceptions ();
        }
    }
    catch (const std::bad_alloc&)
    {
        if (m_actualEntityContainer != NULL)
            delete m_actualEntityContainer;

        CDMW_THROW (OutOfMemoryException);
    }
    catch (const CORBA::SystemException&)
    {
        CDMW_FATAL(Cdmw::Logging::Logger::Get_Logger(CdmwPlatformMngtBase::PFMMNG),
                            "CORBA::SystemException");
        CDMW_ASSERT(false);
    }
}



Host_impl::~Host_impl ()
{
    if (m_state_machine != NULL)
    {
         delete m_state_machine;
     m_state_machine =0 ;
    }

#ifndef CDMW_USE_FAULTTOLERANCE
    if (m_monitor != NULL)
    {
        // terminate pull monitor thread if not stopped
        m_monitor->stop();

        delete m_monitor;
    }
#endif

    if (m_actualEntityContainer != NULL)
    {
        delete m_actualEntityContainer;
    }

    if (m_host_status_checker != 0)
    {

        Cdmw::OsSupport::ThreadStatus t_status = get_consistency_checker_status();

        m_host_status_checker->stop();

        switch (t_status)
        {
             case Cdmw::OsSupport::THREAD_RUN_REQUEST:
             case Cdmw::OsSupport::THREAD_RUNNING:
                  m_host_status_checker->join();
                  break;
             case Cdmw::OsSupport::THREAD_ENDED:
             case Cdmw::OsSupport::THREAD_CREATED:

             default : break;
        }

        delete m_host_status_checker;
        m_host_status_checker = 0;
    }

}


void Host_impl::remove()
{
     std::string element_path = std::string (get_element_path());
     Cdmw::PlatformMngt::HostIssuerManager::Instance()->detach_element(element_path);

    // first increment counter reference to avoid the destructor calling
    // by servant deactivation
    this->_add_ref();

    // deactivate the Corba object
    this->deactivate();
    // remove associated host record in datastore
    remove_record_in_datastore();

    // remove the associated managed element
    this->ManagedElement_impl::remove();

    // then decrement the counter reference to call destructor
    this->_remove_ref();
}


void Host_impl::end_creation(const CdmwPlatformMngt::HostDef& host_def)
throw (CdmwPlatformMngt::DuplicateEntity,
        OutOfMemoryException,
        CORBA::SystemException)
{
    //store the default properties
    store_default_properties();

    // Add the entities of the process
    // if duplicated throw DuplicateEntity exception
    unsigned int entityCount = host_def.entities.length();

    // Store entity initial values into default_entities
    m_default_entitydefs = new CdmwPlatformMngt::EntityDefs ();
    m_default_entitydefs->length (entityCount);

    for (unsigned int entityIndex=0;
    entityIndex < entityCount;
    entityIndex++)
    {
        // get entity definition (EntityDef is a structure, a reference on the
        // structure is returned by the [] operator)
        const CdmwPlatformMngt::EntityDef& entity_def = host_def.entities[entityIndex];

        try
        {
            m_rwEntityContainer->add_entity(entity_def);
            // store values of the entityDef
            (*m_default_entitydefs)[entityIndex] = entity_def;

        }
        catch(const CdmwPlatformMngt::EntityAlreadyExists&)
        {
            // in this case the servant will be deleted by application
            throw CdmwPlatformMngt::DuplicateEntity (entity_def.entity_name.in());
        }
    }


}

void Host_impl::start_consistency_checker()
{
        m_host_status_checker->start();
}

void Host_impl::create_record_in_datastore(const CdmwPlatformMngt::HostDef& host_def)
throw (OutOfMemoryException)
{


    // test if the supervision is not fault tolerant or if it is, is it the primary?
    if (!Configuration::Is_fault_tolerant() || Configuration::Is_primary())
    {
        // set managed element
        set_element_in_datastore();

        CdmwPlatformMngt::HostRecord record;
//        record.status = CdmwPlatformMngt::HOST_UNREACHABLE;
        record.status = CdmwPlatformMngt::HOST_RESPONDING;
        //record.status = m_state_machine->getState()->status();
        //MR record.platform_application_exe = host_def.platform_application_exe;
        record.daemon_monitoring_info = host_def.daemon_monitoring_info;
        record.current_restart_attempt = m_current_restart_attempt;
        record.reboot_in_progress = false ;
        record.stop_in_progress = false ;
        record.monitoring_parameters = host_def.monitoring_parameters;
    record.state_machine = CORBA::string_dup (host_status_strings[0].c_str());

        try
        {
            m_host_ds->insert(m_element_path.in(), record);
        }
        catch (const Common::AlreadyExistException&)
        {
            string tmp ("host record already exists in datastore : ");
            tmp += m_element_path.in();
            LogMngr::logMessage(WRN, tmp.c_str() );
        }
        catch (const CommonSvcs::DataStore::StorageErrorException &exc_storage)
        {
            
            string tmp ("host record storage error exception in insertion : ");
            tmp += m_element_path.in();
            LogMngr::logMessage(FTL, tmp.c_str() );
            CDMW_ASSERT(false);
        }
        catch (...)
        {
        
            string tmp ("host record unexpected exception in insertion : ");
            tmp += m_element_path.in();
            LogMngr::logMessage(FTL, tmp.c_str() );
            CDMW_ASSERT(false);
        }
    }
}

void Host_impl::remove_record_in_datastore()
{
    // inhibition if backup
    if (!Configuration::Is_fault_tolerant() || Configuration::Is_primary())
    {
        try
        {
            m_host_ds->remove(m_element_path.in());
        }
        catch (const Common::NotFoundException&)
        {
        }
        catch (const CommonSvcs::DataStore::StorageErrorException &)
        {
        }
    }
}

CdmwPlatformMngt::HostRecord* Host_impl::get_record_from_datastore()
{
    CdmwPlatformMngt::HostRecord* host_record = NULL;

    try
    {
        host_record = m_host_ds->select(m_element_path.in());
    }
    catch (const Common::NotFoundException&)
    {
        string tmp ("Host record cannot be found in datastore: ");
        tmp += m_element_path.in();
        LogMngr::logMessage(FTL, tmp.c_str() );
        CDMW_ASSERT(false);
    }

    return host_record;
}



void Host_impl::update_record_in_datastore(const CdmwPlatformMngt::HostRecord& host_record)
{
    try
    {
        m_host_ds->update(m_element_path.in(), host_record);
    }
    catch (const Common::NotFoundException&)
    {
        string tmp ("Host record cannot be found in datastore: ");
        tmp += m_element_path.in();
        LogMngr::logMessage(FTL, tmp.c_str() );
        CDMW_ASSERT(false);
    }
    catch (const CommonSvcs::DataStore::StorageErrorException&)
    {
        string tmp ("Host record storage error exception: ");
        tmp += m_element_path.in();
        LogMngr::logMessage(FTL, tmp.c_str() );
        CDMW_ASSERT(false);
    }
}


CdmwPlatformMngt::HostStatus Host_impl::property_to_status (const char* status)
 throw (CdmwPlatformMngt::AttributeNotFound)
 {
     CdmwPlatformMngt::HostStatus host_status; 
     bool result = false;
     for ( int i=0 ; i<7; i++)
     {
         if (strcmp(status, host_status_strings[i].c_str()) == 0)
         {
             host_status = Status[i];
             result = true; 
             return host_status;
         }
     }
     if (!result)
     {
         throw CdmwPlatformMngt::AttributeNotFound();
     }
     
     return host_status;
 }


CdmwPlatformMngt::HostStatus Host_impl::status()
    throw( CORBA::SystemException )
{    
    CdmwPlatformMngt::HostStatus host_status;
    const char* status = "";
    try
    {
        // Get predefined attributes of process
        CORBA::Any_var any_var = new CORBA::Any();

        get_attribute(CdmwPlatformMngt::TECH_STATUS, any_var.out(), CdmwPlatformMngt::PRE_DEFINED);           
        any_var.in() >>=  CORBA::Any::to_string(status,0);            

        host_status = property_to_status (status);             

        return host_status;
    }
    catch (const CdmwPlatformMngt::AttributeNotFound&)
    {
        throw CORBA::BAD_PARAM(OrbSupport::BAD_PARAM, CORBA::COMPLETED_NO);
    }
    catch (const std::bad_alloc&)
    {
        throw CORBA::NO_MEMORY (OrbSupport::NO_MEMORY, CORBA::COMPLETED_NO);
    }
    catch (...)
    {
        throw CORBA::BAD_PARAM(OrbSupport::BAD_PARAM, CORBA::COMPLETED_NO);
    }
}

char * Host_impl::host_status_info()
    throw( CORBA::SystemException )
{
    CdmwPlatformMngt::HostRecord_var host_record = get_record_from_datastore();

    return ( host_record->host_status_info)._retn() ;
}

CdmwPlatformMngt::ManagedElement_ptr Host_impl::get_parent()
    throw (CORBA::SystemException)
{
    return m_system->_this() ;
}

/**
 *Purpose:
 *<p> facade to get an attribute
 *
 */
void Host_impl::get_attribute(const char * attr_name, CORBA::Any_out ret_attr, CdmwPlatformMngt::Owner flag  )
    throw (CORBA::SystemException, CdmwPlatformMngt::AttributeNotFound)
{
    std::string tmp1(attr_name);

    if (CdmwPlatformMngt::PRE_DEFINED == flag )
    {
        // Case of predefined property

        std::ostringstream property_name;
        property_name << CdmwPlatformMngt::CDMW_PREFIX << attr_name << std::ends;
        try
        {
            CORBA::Any_var outany = get_property_value ( property_name.str().c_str() );
            ret_attr = outany._retn();
        }
        catch (const CosPropertyService::PropertyNotFound&)
        {
            throw CdmwPlatformMngt::AttributeNotFound ();
        }
    }
    //case of user defined attribute
    else
    {
        try
        {
            CORBA::Any* outany = get_property_value( attr_name );
            //CORBA::Any_var tempany = new CORBA::Any;
            ret_attr = outany;
        }
        catch (const CosPropertyService::PropertyNotFound& )
        {
            throw CdmwPlatformMngt::AttributeNotFound();
        }
    }
}

/**
 *Purpose:
 *<p> facade to set an attribute
 *
 */

void Host_impl::set_attribute(const char* attr_name, const CORBA::Any& attr, CdmwPlatformMngt::Owner flag  )
    throw (CORBA::SystemException, CdmwPlatformMngt::AttributeNotFound, CdmwPlatformMngt::ReadOnlyAttribute)
{

    if (CdmwPlatformMngt::PRE_DEFINED == flag )
    {
        std::string tmp1 (attr_name);
        // check if the predefined property is readonly
        if ( (CdmwPlatformMngt::REQUESTED_RESTART_WARMTH == tmp1) ||
                (CdmwPlatformMngt::GLOBAL_SUPERVISION == tmp1) ||
                (tmp1 == CdmwPlatformMngt::TECH_STATUS) ||
                (tmp1 == CdmwPlatformMngt::TECH_STATUS_INFO))
        {
            throw CdmwPlatformMngt::ReadOnlyAttribute();
        }

        if (tmp1 == CdmwPlatformMngt::TRACE_FILTERS)
        {
            const char * tmp2;
            attr >>= tmp2;

            try
            {
                set_trace_filtering (tmp2);
                return;
            }
            catch (const CdmwPlatformMngt::CastException& e)
            {
                //in this case is not necessary to trow this exception
                //throw CdmwPlatformMngt::AttributeNotFound();
                return;
            }
            catch (const CdmwPlatformMngt::ParsingException& e)
            {
                   throw CORBA::BAD_PARAM (OrbSupport::BAD_PARAM, CORBA::COMPLETED_NO);
            }
        }
    }
    try
    {
        set_attribute_internal ( attr_name, attr, flag );
    }
    catch (const Cdmw::BadOrderException& ex)
    {
        throw CORBA::BAD_PARAM(OrbSupport::BAD_PARAM, CORBA::COMPLETED_NO);
    }
    catch (...)
    {
        throw;
    }

}


CdmwPlatformMngt::Processes* Host_impl::get_processes ( CORBA::ULong how_many,
        CdmwPlatformMngt::ProcessesIterator_out processes_iterator)
    throw (CORBA::SystemException)
{
    CORBA::ULong how_many_apps = m_system->get_number_of_applications();
    CdmwPlatformMngt::ApplicationsIterator_var app_iter;
    CdmwPlatformMngt::Applications_var apps = m_system->get_applications(how_many_apps, app_iter.out()) ;
    CdmwPlatformMngt::Processes* seq_processes_out = new CdmwPlatformMngt::Processes();
    std::list<CdmwPlatformMngt::Process_var> process_remaining;

    CdmwPlatformMngt::Host_var my_host_var = this->_this() ;
    
    //use a cycle with the single get_application
    CORBA::ULong k=0;
    for (CORBA::ULong i=0; i<how_many_apps; i++)
    {
        CORBA::ULong how_many_processes = (*apps)[i]->get_number_of_processes();
        CdmwPlatformMngt::ProcessesIterator_var process_iter;
        CdmwPlatformMngt::Processes_var processes =
            (*apps)[i]->get_processes( how_many_processes, process_iter.out()) ;

        for (CORBA::ULong j=0; j<how_many_processes; j++)
        {
            //use the get_host to know if it is our desired host
        CdmwPlatformMngt::Host_var actual_host = ((*processes)[j])->get_host();

            if ( actual_host->_is_equivalent(my_host_var.in()) )
            {
                if ( k < how_many )
                {
                    seq_processes_out->length(k+1);
                    (*seq_processes_out)[k] =  (*processes)[j] ;
                }
                else
                {
                    // put the processes to the iterator out ...
                    process_remaining.push_back ( (*processes)[j] );
                }
                k++;
            }
        }
    }

    CdmwPlatformMngt::ProcessesIterator_var process_remaining_iter =
        ProcessesIterator_impl::create(process_remaining);
    processes_iterator = process_remaining_iter._retn() ;

    return (seq_processes_out) ;
}


CdmwPlatformMngt::CompleteMonitoringInfo Host_impl::get_monitoring_parameters()
    throw (CORBA::SystemException)
{
    CdmwPlatformMngt::HostRecord_var host_record = get_record_from_datastore();

    return host_record->monitoring_parameters;
}

void Host_impl::set_monitoring_parameters(const CdmwPlatformMngt::CompleteMonitoringInfo& monitoring_parameters)
    throw (CORBA::SystemException)
{

    CdmwPlatformMngt::HostRecord_var host_record;
    CdmwPlatformMngt::CompleteMonitoringInfo current_monitor_par;

    // lock for concurent access
    CDMW_MUTEX_GUARD(m_mutex_host_record);

        // get current status
        host_record = get_record_from_datastore();
        current_monitor_par = host_record->monitoring_parameters;

        // if current_monitor_info changes
        if ( (monitoring_parameters.monitoring_model !=
    current_monitor_par.monitoring_model) ||
    (monitoring_parameters.pull_monitoring_interval !=
    current_monitor_par.pull_monitoring_interval) ||
    (monitoring_parameters.pull_monitoring_timeout !=
    current_monitor_par.pull_monitoring_timeout) ||
    (monitoring_parameters.push_monitoring_interval !=
    current_monitor_par.push_monitoring_interval) ||
    (monitoring_parameters.push_monitoring_timeout !=
    current_monitor_par.push_monitoring_timeout) )
        {
            // store new status
            host_record->monitoring_parameters = monitoring_parameters;
            update_record_in_datastore(host_record.in());
        }

}



CdmwPlatformMngt::Process_ptr Host_impl::get_process(const char* process_name, const char *application_name )
    throw (CdmwPlatformMngt::ProcessNotFound, CORBA::SystemException)
{
    CdmwPlatformMngt::Application_var app = m_system->get_application(application_name);
    CdmwPlatformMngt::Process_ptr myprocess = app->get_process( process_name, "" );
    CdmwPlatformMngt::Host_var actual_host = myprocess->get_host() ;
    CdmwPlatformMngt::Host_var my_host_var = this->_this() ;

    if ( !(actual_host->_is_equivalent(my_host_var.in())) )
    {
        throw CdmwPlatformMngt::ProcessNotFound();
    }

    return CdmwPlatformMngt::Process::_duplicate(myprocess) ;
}

//internal method to have the current number of restart
//attempt
CORBA::Long Host_impl::restart_attempt()
    throw (CORBA::SystemException)
{
    return m_current_restart_attempt;
}

//internal method to have the current number of restart attempt
void Host_impl::set_restart_attempt(CORBA::Long current_restart_attempt)
    throw (CORBA::SystemException)
{
    m_current_restart_attempt = current_restart_attempt;
    CdmwPlatformMngt::HostRecord_var host_rec = get_record_from_datastore() ;
    host_rec->current_restart_attempt = m_current_restart_attempt;

            update_record_in_datastore( host_rec.in() );
}

//internal method to have the current number of restart
//attempt sequenc. number
CORBA::Long Host_impl::get_max_restart_attempt() throw (CORBA::SystemException)
{
    // Get max_nbr_of_restart_attempt
    CORBA::Any_var any_var;
    CdmwPlatformMngt::AutoRestartInfo* ari;
    get_attribute (CdmwPlatformMngt::AUTORESTART_INFO, any_var.out(), CdmwPlatformMngt::PRE_DEFINED);
    any_var.in() >>= ari;
    return (ari->max_nbr_of_restart_attempt) ;
}


void Host_impl::set_trace_filtering(const char *trace_filtering)
    throw (CORBA::SystemException, CdmwPlatformMngt::ParsingException , CdmwPlatformMngt::CastException )
{
    CORBA::ULong seq_size = get_number_of_processes();
    CdmwPlatformMngt::ProcessesIterator_var proc_iter;
    CdmwPlatformMngt::Processes_var my_processes_from_host =
        get_processes( seq_size, proc_iter.out()) ;

    //analyse all the processes ...
    //seq_size = my_processes_from_host->length();

//    try
//    {
        for (CORBA::ULong ii=0; ii<seq_size; ii++)
        {
            //see if the process is managed ...
            if ( my_processes_from_host[ii]->type() != CdmwPlatformMngt::UNMANAGED_PROCESS )
            {
        /*

                // Get max_nbr_of_restart_attempt
            CORBA::Any_var any_var;
            CdmwPlatformMngt::AutoRestartInfo* ari;
            get_attribute (CdmwPlatformMngt::AUTORESTART_INFO, any_var.out(), CdmwPlatformMngt::PRE_DEFINED);
            any_var.in() >>= ari;
            return (ari->max_nbr_of_restart_attempt) ;
        }


void Host_impl::set_trace_filtering(const char *trace_filtering) throw (CORBA::SystemException, CdmwPlatformMngt::ParsingException , CdmwPlatformMngt::CastException )
        {

        //see all the processes in the host
        CdmwPlatformMngt::Processes_var my_processes_from_host;
        CdmwPlatformMngt::ProcessesIterator_var proc_iter;

        numeric_limits < CORBA::ULong > big_number;
        CORBA::ULong max_ulong, seq_size , ii ;
        max_ulong = big_number.max ();

        my_processes_from_host =  get_processes( max_ulong, proc_iter) ;

        //analyse all the processes ...
        seq_size = my_processes_from_host->length();


        try
        {

        for (ii=0; ii<seq_size; ii++)
           {
               //see if the process is managed ...
               if ( my_processes_from_host[ii]->type() != CdmwPlatformMngt::UNMANAGED_PROCESS )
              {
          */
              //set the trace filters...
              CORBA::Any * myany = new (CORBA::Any);
              *myany <<= trace_filtering;
              my_processes_from_host[ii]->set_attribute (CdmwPlatformMngt::TRACE_FILTERS, *myany, CdmwPlatformMngt::PRE_DEFINED);
              }
           }

        /*
        }
catch(const CdmwPlatformMngt::ParsingException& ex)
          {
          throw CdmwPlatformMngt::ParsingException(ex.message);
          }
        catch(const CdmwPlatformMngt::CastException& ex)
          {
          throw CdmwPlatformMngt::CastException( ex.message) ;
          }
*/
}

void Host_impl::shutdown ()
throw (CORBA::SystemException)
{
    // Create a command event to notify
    std::string element_path = std::string(get_element_path());
    //LC: Attention: call an issuer mutex guard before calling this method.    
    std::string issuer = Cdmw::PlatformMngt::HostIssuerManager::Instance()->get_command_issuer(element_path);

    notifyCommandEvent(SHUTDOWN, issuer.c_str());
    
    if (set_current_host_command(SHUTDOWN))
    {
        if (m_state_machine->getState()->status() == CdmwPlatformMngt::HOST_RESPONDING)
    {
            CdmwPlatformMngt::Daemon_var daemon =
                Configuration::Get_daemon (get_element_name(), m_system->systemPort());
            execute_host_command("HOST_SHUTDOWN");
    }
    else
    {
        try
            {
                m_state_machine->getState()->stop(m_state_machine);
            }
            catch (const CdmwPlatformMngt::IncompatibleStatus& ex)
            {
                set_current_host_command(NO_COMMAND); // This should always return true
            }
    }
    }
}


void Host_impl::cold_reboot ()
throw (CORBA::SystemException)
{
    // Create a command event to notify
    std::string element_path = std::string(get_element_path());
     //LC: Attention: call an issuer mutex guard before calling this method.    
    std::string issuer = Cdmw::PlatformMngt::HostIssuerManager::Instance()->get_command_issuer(element_path);

    notifyCommandEvent(COLD_REBOOT, issuer.c_str());
    
    reboot(false);
}


void Host_impl::hot_reboot ()
throw (CORBA::SystemException)
{
    // Create a command event to notify
    std::string element_path = std::string(get_element_path());
    //LC: Attention: call an issuer mutex guard before calling this method.    
    std::string issuer = Cdmw::PlatformMngt::HostIssuerManager::Instance()->get_command_issuer(element_path);

    notifyCommandEvent(HOT_REBOOT, issuer.c_str());
    reboot(true);
}

void Host_impl::reboot (bool is_hot_reboot)
throw (CORBA::SystemException)
{
    HostCommand hc = is_hot_reboot ? HOT_REBOOT : COLD_REBOOT;

    if (set_current_host_command(hc))
    {
        set_reboot_in_progress(true);
        if (is_hot_reboot)
        {
            set_requested_restart_warmth (CdmwPlatformMngt::HOT_RESTART);
            CDMW_INFO(Cdmw::Logging::Logger::Get_Logger(CdmwPlatformMngtBase::PFMMNG),
                    "Hot rebooting : " << m_element_path.in());
        }
        else
        {
            set_requested_restart_warmth (CdmwPlatformMngt::COLD_RESTART);
            CDMW_INFO(Cdmw::Logging::Logger::Get_Logger(CdmwPlatformMngtBase::PFMMNG),
                    "Cold rebooting : " << m_element_path.in());
        }

        try
        {
            if (!(m_state_machine->getState()->status() == CdmwPlatformMngt::HOST_UNREACHABLE ||
            m_state_machine->getState()->status() == CdmwPlatformMngt::HOST_RESPONDING
            ))
            {
                m_state_machine->getState()->stop(m_state_machine);
            }
            else
            {
                // In this case we don't know if the host is up. We could try to reboot it
                // but cannot leave the HOT_REBOOT command on because we don't know if the host
                // will ever reboot
                execute_host_command("HOST_REBOOT");
                set_current_host_command(NO_COMMAND);
            }
        }
        catch (const CdmwPlatformMngt::IncompatibleStatus& ex)
        {
            set_current_host_command(NO_COMMAND); // This should always return true
        }
    }
}

void Host_impl::stop_processes()
{
    HostProcessInitSequencer::get_instance()->stop_host_processes(get_element_name());
    /*
    CORBA::ULong how_many = 1L;
    CdmwPlatformMngt::ProcessesIterator_var processes_iterator;
    CdmwPlatformMngt::Processes_var processes = get_processes ( how_many, processes_iterator.out());
    CdmwPlatformMngt::Process_var first_process = processes[(CORBA::ULong)0];
    CdmwPlatformMngt::Process_var other_process;

    CDMW_DEBUG(Cdmw::Logging::Logger::Get_Logger(CdmwPlatformMngtBase::PFMMNG),
            "Stopping process: " << first_process->name() << " on host: "
             << m_element_path.in());

    first_process->stop_and_wait(false);

    while ( (processes_iterator -> next_one(other_process.out() )) )
    {
    CDMW_DEBUG(Cdmw::Logging::Logger::Get_Logger(CdmwPlatformMngtBase::PFMMNG),
                "Stopping process: " << other_process->name() << " on host: "
                << m_element_path.in());
        other_process ->stop_and_wait(false);
    }
    */
}

void Host_impl::start_processes()
{
    CORBA::ULong how_many = 1L;
    CdmwPlatformMngt::ProcessesIterator_var processes_iterator;

     de_activate_preemption();

    //Set default startup kind for processes to be restarted
    CdmwPlatformMngtBase::StateRetrieval state_retrieval = CdmwPlatformMngtBase::LAST_STATE;
    unsigned long startup_mode = 0;
    string specified_state_id = "";
    CdmwPlatformMngtBase::StartupKind_var startupKind = new CdmwPlatformMngtBase::StartupKind();
    startupKind->state_retrieval = state_retrieval;
    startupKind->startup_mode = startup_mode;
    startupKind->specified_state_id = CORBA::string_dup( specified_state_id.c_str() );

    //start processes
    CdmwPlatformMngt::Processes_var processes = get_processes ( how_many, processes_iterator.out());
    CdmwPlatformMngt::Process_var first_process = processes[(CORBA::ULong)0];
    CdmwPlatformMngt::Process_var other_process;

       //Cold restart
       if (CdmwPlatformMngt::COLD_RESTART == requested_restart_warmth() ){
                    HostProcessInitSequencer::get_instance()->initialise_host_processes(CdmwPlatformMngt::COLD_RESTART, name());
                    HostProcessInitSequencer::get_instance()->run_host_processes(CdmwPlatformMngt::COLD_RESTART,name());
            }
        //Hot restart
    else
    {
        HostProcessInitSequencer::get_instance()->initialise_host_processes(CdmwPlatformMngt::HOT_RESTART, name());
        HostProcessInitSequencer::get_instance()->run_host_processes(CdmwPlatformMngt::HOT_RESTART, name());
    }
    //MR
    //set_requested_restart_warmth (get_default_restart_warmth());
    CDMW_INFO(Cdmw::Logging::Logger::Get_Logger(CdmwPlatformMngtBase::PFMMNG),
            "Reboot is finished for remote host : " << m_element_path.in());
}

void Host_impl::setStatus (CdmwPlatformMngt::HostStatus status, CdmwPlatformMngtBase::TimeStamp time_stamp)
{
//    if (m_state_machine == NULL)
//    {
//        CDMW_WARN(Cdmw::Logging::Logger::Get_Logger(CdmwPlatformMngtBase::PFMMNG),
//                  "Someone tried to set host status before statemachine was created" );
//        return;
//    }

    // get current status
    CdmwPlatformMngt::HostRecord_var host_record = get_record_from_datastore();
    CdmwPlatformMngt::HostStatus current_status = host_record->status;

    // if status has been changed
    if (status != current_status)
    {
        m_state_machine->setState(status);
    }
}

void Host_impl::start()
{
    // start pull monitor thread
#ifdef CDMW_USE_FAULTTOLERANCE
    ::FT::Location location;
    location.length(1);
    location[0].kind = "hostname";
    location[0].id = get_element_name();

    ::FT::FaultMonitoringIntervalAndTimeoutValue time;
    time.monitoring_interval=0;
    time.timeout=0;

    m_monitoringID = m_faultDetector->start_monitoring(::FT::PullMonitorable::_nil(),
            location,
            time);
#else
    m_monitor->start();
#endif
}



void Host_impl::stop() throw (CORBA::SystemException)
{
    // inhibition if backup
    if (!Configuration::Is_fault_tolerant() || Configuration::Is_primary())
    {
        // terminate pull monitor thread

#ifdef CDMW_USE_FAULTTOLERANCE
        m_faultDetector->stop_monitoring(m_monitoringID);
#else
        m_monitor->stop();
#endif

    }
}


CORBA::ULong Host_impl::get_number_of_entities()
throw(CORBA::SystemException)
{
    return m_rwEntityContainer->get_number_of_entities();
}

void Host_impl::get_all_entity_names(
        CORBA::ULong how_many,
        CdmwPlatformMngt::ElementNames_out entity_names,
        CdmwPlatformMngt::ElementNamesIterator_out rest)
throw(CORBA::SystemException)
{
    m_rwEntityContainer->get_all_entity_names(
            how_many, entity_names, rest);
}

CdmwPlatformMngt::Entity_ptr Host_impl::get_entity (
        const char* entity_name)
throw (CdmwPlatformMngt::EntityNotFound,
        CORBA::SystemException)
{
    return m_rwEntityContainer->get_entity(entity_name);
}

CORBA::Boolean Host_impl::get_entities(
        const CdmwPlatformMngt::ElementNames& entity_names,
        CdmwPlatformMngt::Entities_out nentities)
throw(CORBA::SystemException)
{
    return m_rwEntityContainer->get_entities(
            entity_names, nentities);
}

void Host_impl::get_all_entities(
        CORBA::ULong how_many,
        CdmwPlatformMngt::Entities_out nentities,
        CdmwPlatformMngt::EntitiesIterator_out rest)
throw(CORBA::SystemException)
{
    m_rwEntityContainer->get_all_entities(
            how_many, nentities, rest);
}


CdmwPlatformMngt::Entity_ptr Host_impl::add_entity (
        const CdmwPlatformMngt::EntityDef & entity_def)
throw(CdmwPlatformMngt::EntityAlreadyExists,
        CORBA::SystemException)
{
    std::string entityName = entity_def.entity_name.in();

    LogMngr::logMessage(INF, MSG_ID_ADD_HOST_ENTITY,
            entityName.c_str(), m_element_name.in());

    CdmwPlatformMngt::Entity_ptr ent_ret = m_rwEntityContainer->add_entity(entity_def);

    notify_configuration_change("Entity Added");
    return ent_ret;
}

void Host_impl::remove_entity(
        const char* entity_name)
throw (CdmwPlatformMngt::EntityNotFound,
        CORBA::SystemException)
{
    LogMngr::logMessage(INF, MSG_ID_REMOVE_SYSTEM_ENTITY,
            entity_name, m_element_name.in());

    m_rwEntityContainer->remove_entity(entity_name);

    notify_configuration_change("Entity Removed");
}

Entity_impl *Host_impl::getEntity (const char *entity_name)
throw (CORBA::SystemException)
{
    return m_actualEntityContainer->getEntity (entity_name);
}

void Host_impl::synchro_entities_with_datastore()
{
    CORBA::ULong how_many = get_number_of_entities (); 
    CdmwPlatformMngt::ElementNames_var entity_names;
    CdmwPlatformMngt::ElementNamesIterator_var rest;
    get_all_entity_names (how_many, entity_names, rest);
    for (CORBA::ULong count = 0U; count< how_many; count++)
    {
       //synchro the entity with its datastore 
       getEntity(entity_names[count])->synchro_with_datastore();
    }
}  
  
CdmwPlatformMngt::ElementType Host_impl::get_element_type()
throw (CORBA::SystemException)
{
    return CdmwPlatformMngt::HOST_TYPE;
}

CdmwPlatformMngt::Children* Host_impl::get_children()
throw (CORBA::SystemException)
{
    CdmwPlatformMngt::Children_var children = new CdmwPlatformMngt::Children;
    CORBA::ULong size_of_entities = get_number_of_entities();
    children->length(size_of_entities);

    CdmwPlatformMngt::Entities_var entities;
    CdmwPlatformMngt::EntitiesIterator_var entities_iterator;
    get_all_entities(size_of_entities, entities.out(), entities_iterator.out() );

    for (CORBA::ULong count = 0; count<size_of_entities ; count++)
    {
        entities[count]->_add_ref();
        children[count] = (entities[count])._retn();
    }
    return children._retn ();
}

void  Host_impl::register_observer_ext(const char* observer_name,
        CdmwPlatformMngt::RegistrationDepth depth,
        CdmwPlatformMngt::SupervisionObserver_ptr observer)  throw (CORBA::SystemException,
                CdmwPlatformMngt::ElementAlreadyExists)
{
    ManagedElement_impl::register_observer_ext(observer_name, depth, observer);
    if (depth == CdmwPlatformMngt::FULL_HIERARCHY)
    {
        CdmwPlatformMngt::ProcessesIterator_var proc_iter;
        CORBA::ULong size = get_number_of_processes();
        CdmwPlatformMngt::Processes_var processes_from_host = get_processes (size, proc_iter.out());
        for (CORBA::ULong i=0; i<size; i++)
        {
            processes_from_host[i]->register_observer_ext(observer_name, depth,
                CdmwPlatformMngt::SupervisionObserver::_duplicate(observer));
        }
    }
}

void Host_impl::unregister_observer_ext(const char* observer_name)
    throw (CORBA::SystemException, CdmwPlatformMngt::ObserverNotFound)
{
    ManagedElement_impl::unregister_observer_ext(observer_name);
    CdmwPlatformMngt::ProcessesIterator_var proc_iter;
    CORBA::ULong size = get_number_of_processes();
    CdmwPlatformMngt::Processes_var processes_from_host = get_processes (size, proc_iter.out());
    for (CORBA::ULong i=0; i<size; i++)
    {
        try
        {
            processes_from_host[i]->unregister_observer_ext(observer_name);
        }
        catch (const CdmwPlatformMngt::ObserverNotFound& ex)
        {
            //PCR-775       
            //Not throwable exception in case of OBJECT_ONLY registration depth. 
            break;
    }
    }
}

CORBA::Boolean Host_impl::get_reboot_in_progress() throw (CORBA::SystemException)
{
    CDMW_MUTEX_GUARD(m_mutexRebootInProgress);

    return m_reboot_in_progress;
}



void  Host_impl::set_reboot_in_progress(CORBA::Boolean reboot_in_progress)
{
    CDMW_MUTEX_GUARD(m_mutexRebootInProgress);
    m_reboot_in_progress = reboot_in_progress;

    //update the datastore
    CdmwPlatformMngt::HostRecord_var host_rec = get_record_from_datastore() ;
    host_rec->reboot_in_progress = m_reboot_in_progress;
    update_record_in_datastore( host_rec.in() );
}

CORBA::Boolean Host_impl::get_stop_in_progress()  throw (CORBA::SystemException)
{
    CDMW_MUTEX_GUARD(m_mutexStopInProgress);

    return m_stop_in_progress;
}


void  Host_impl::set_stop_in_progress(CORBA::Boolean stop_in_progress)
{
    CDMW_MUTEX_GUARD(m_mutexStopInProgress);
    m_stop_in_progress = stop_in_progress;

    //update the datastore
    CdmwPlatformMngt::HostRecord_var host_rec = get_record_from_datastore() ;
    host_rec->stop_in_progress = m_stop_in_progress;
    update_record_in_datastore( host_rec.in() );
}


void Host_impl::synchro_with_datastore ()
{
    Cdmw::PlatformMngt::ManagedElement_impl::synchro_with_datastore ();
    synchro_entities_with_datastore();
    //read the host datastore
    CdmwPlatformMngt::HostRecord_var host_rec = get_record_from_datastore() ;
    m_current_restart_attempt = host_rec->current_restart_attempt;
    m_reboot_in_progress = host_rec->reboot_in_progress;
    m_stop_in_progress = host_rec->stop_in_progress;
    m_state_machine->setStateName (host_rec->state_machine.in ());
}


void Host_impl::get_all_processes(CORBA::ULong how_many, CdmwPlatformMngt::Processes_out nprocesses,
        CdmwPlatformMngt::ProcessesIterator_out rest)
    throw (CORBA::SystemException)
{
    // TODO
    throw CORBA::NO_IMPLEMENT (OrbSupport::NO_IMPLEMENT, CORBA::COMPLETED_NO);
}

CORBA::ULong Host_impl::get_number_of_processes()
    throw (CORBA::SystemException)
{
    // Get system applications
    CORBA::ULong how_many_apps = m_system->get_number_of_applications();
    CdmwPlatformMngt::ApplicationsIterator_var app_iter;
    CdmwPlatformMngt::Applications_var apps = m_system->get_applications(how_many_apps, app_iter.out()) ;

    //use a cycle with the single get_application
    CdmwPlatformMngt::Host_var my_host_var = this->_this() ;
    CORBA::ULong number_of_processes = 0;
    for (CORBA::ULong i=0; i<how_many_apps; i++)
    {
        CORBA::ULong how_many_processes = (*apps)[i]->get_number_of_processes();
        CdmwPlatformMngt::ProcessesIterator_var process_iter;
        CdmwPlatformMngt::Processes_var processes =
            (*apps)[i]->get_processes( how_many_processes, process_iter.out()) ;
        for (CORBA::ULong j=0; j<how_many_processes; j++)
        {
            //use the get_host to know if it is our desired host
        CdmwPlatformMngt::Host_var actual_host = ((*processes)[j])->get_host();

            if ( actual_host->_is_equivalent(my_host_var.in() ) )
            {
                number_of_processes++;
            }
        }
    }

    return number_of_processes;
}

void Host_impl::notifyCommandEvent(HostCommand host_command, 
        const char* issuer,
        CORBA::Boolean emergency)
{
    CdmwPlatformMngt::CdmwCommand_var event;       
    CdmwPlatformMngtBase::EventHeader header
    = EventHeaderFactory::createHeader (CdmwPlatformMngtBase::INF);

    if ((COLD_RESTART == host_command) || (HOT_RESTART == host_command) ||
            (COLD_REBOOT == host_command) || (HOT_REBOOT == host_command))
    {
        //Setting up the startupKind parameters
        CdmwPlatformMngtBase::StateRetrieval state_retrieval = CdmwPlatformMngtBase::LAST_STATE;
        unsigned long startup_mode = 0;
        std::string specified_state_id = "";
        CdmwPlatformMngtBase::StartupKind_var startupKind = new CdmwPlatformMngtBase::StartupKind();
        startupKind->state_retrieval = state_retrieval;
        startupKind->startup_mode = startup_mode;
        startupKind->specified_state_id  = CORBA::string_dup( specified_state_id.c_str() );

        CdmwPlatformMngt::RestartWarmth restart_warmth;


        if ((COLD_RESTART == host_command)|| (COLD_REBOOT == host_command))
        {
            restart_warmth = CdmwPlatformMngt::COLD_RESTART;
        }
        else if ((HOT_RESTART == host_command) || (HOT_REBOOT == host_command))
        {               
            restart_warmth = CdmwPlatformMngt::HOT_RESTART;
        }

        if ((COLD_RESTART == host_command) || (HOT_RESTART == host_command))
        {
            // Create the event
            event = m_hostRestartFactory->create(
                    header,
                    m_system->get_element_name(),
                    get_element_path(),
                    m_element_name.in(),
                    issuer,
                    startupKind.in(),
                    restart_warmth);
        }

        if ((COLD_REBOOT == host_command) || (HOT_REBOOT == host_command))
        {
            event = m_hostRebootFactory->create(
                    header,
                    m_system->get_element_name(),
                    get_element_path(),
                    m_element_name.in (),
                    issuer,
                    startupKind.in(),
                    restart_warmth);
        }
    }

    if (STOP == host_command)
    {
        event = m_hostStopFactory->create (header,
                m_system->get_element_name(),
                get_element_path(),
                m_element_name.in (),
                issuer,
                emergency);    

    }

    if (SHUTDOWN == host_command)
    {
        event = m_hostShutdownFactory->create (header,
                m_system->get_element_name(),
                get_element_path(),
                m_element_name.in (),
                issuer);
    }

    //Log event
    CORBA::String_var eventStr = event->to_string ();
    LogMngr::logMessage (eventStr.in ()); 

    // Perform the command notification to supervision event notifier
    event->_add_ref();            
    (EventNotifierManager::Instance()->get_sup_notifier())->addEvent (event);

    // Perform the command notification to command event notifier
    event->_add_ref(); 
    (EventNotifierManager::Instance()->get_command_notifier())->add_event(event);
    event->_remove_ref();

    event._retn ();
}

void Host_impl::cold_restart()
    throw (CORBA::SystemException)
{
    
    if(m_system->m_systemStarted)
    {
        // Create a command event to notify
        std::string element_path = std::string(get_element_path());
        //LC: Attention: call an issuer mutex guard before calling this method.        
        std::string issuer = Cdmw::PlatformMngt::HostIssuerManager::Instance()->get_command_issuer(element_path);

        notifyCommandEvent(COLD_RESTART, issuer.c_str());
        
        if (set_current_host_command(COLD_RESTART))
        {
            set_reboot_in_progress(true);
            set_requested_restart_warmth (CdmwPlatformMngt::COLD_RESTART);
            try
            {
                if (m_state_machine->getState()->status() != CdmwPlatformMngt::HOST_UNREACHABLE)
                {
                    m_state_machine->getState()->stop(m_state_machine);
                }
                else
                {
                    execute_host_command("HOST_START");
                }
            }
            catch (const CdmwPlatformMngt::IncompatibleStatus& ex)
            {
                set_current_host_command(NO_COMMAND); // This should always return true
            }
        }

    }
    else CDMW_ERROR(Cdmw::Logging::Logger::Get_Logger(CdmwPlatformMngtBase::PFMMNG),
    "Error, i can't restart the system because it was't started");

}

void Host_impl::hot_restart()
    throw (CORBA::SystemException)
{
    
if(m_system->m_systemStarted)
{
    // Create a command event to notify
    std::string element_path = std::string(get_element_path());
    //LC: Attention: call an issuer mutex guard before calling this method.    
    std::string issuer = Cdmw::PlatformMngt::HostIssuerManager::Instance()->get_command_issuer(element_path);

    notifyCommandEvent(HOT_RESTART, issuer.c_str());
    
    if (set_current_host_command(HOT_RESTART))
    {
        set_reboot_in_progress(true);
        set_requested_restart_warmth (CdmwPlatformMngt::HOT_RESTART);
        try
        {
            if (m_state_machine->getState()->status() != CdmwPlatformMngt::HOST_UNREACHABLE)
            {
                m_state_machine->getState()->stop(m_state_machine);
            }
            else
            {
                execute_host_command("HOST_START");
            }
        }
        catch (const CdmwPlatformMngt::IncompatibleStatus& ex)
        {
            set_current_host_command(NO_COMMAND); // This should always return true
        }
    }
}
else CDMW_ERROR(Cdmw::Logging::Logger::Get_Logger(CdmwPlatformMngtBase::PFMMNG),
                    "Error, i can't restart the system becuase it was't started");
}

void Host_impl::stop_host()
    throw (CORBA::SystemException)
{

    // Create a command event to notify
    std::string element_path = std::string(get_element_path());
    //LC: Attention: call an issuer mutex guard before calling this method.    
    std::string issuer = Cdmw::PlatformMngt::HostIssuerManager::Instance()->get_command_issuer(element_path);

    notifyCommandEvent(STOP, issuer.c_str());
    
        //MR
        //set_requested_restart_warmth ( get_default_restart_warmth () );

    if (set_current_host_command(STOP))
    {
        activate_preemption();
        
        try
        {
            m_state_machine->getState()->stop(m_state_machine);
           
        }
        catch (const CdmwPlatformMngt::IncompatibleStatus& ex)
        {
            set_current_host_command(NO_COMMAND); // This should always return true
        } 
        catch (...)
        {
        
            CDMW_ERROR(Cdmw::Logging::Logger::Get_Logger(CdmwPlatformMngtBase::PFMMNG)," Unknown Exception ");            
        }
        
    }
}

void Host_impl::notify_status_change()
{
    // statemachine could call this function during its creation
    // in this case m_state_machine is NULL
    if (m_state_machine == NULL)
    {
        return;
    }
    try
    {
        HostState* host_state = m_state_machine->getState();
        CdmwPlatformMngt::HostStatus host_status = host_state->status();

        // Update datastore
        CdmwPlatformMngt::HostRecord_var hr = get_record_from_datastore();
        hr->status = host_status;
        hr->state_machine = CORBA::string_dup (m_state_machine->getStateName ());
        update_record_in_datastore(hr.in());

        // create header

    CdmwPlatformMngtBase::EventLevel level = CdmwPlatformMngtBase::INF;

    if ( (CdmwPlatformMngt::HOST_UNREACHABLE == host_status) || (CdmwPlatformMngt::HOST_RESPONDING == host_status) )
        {
        level = CdmwPlatformMngtBase::WRN;
        }

    CdmwPlatformMngtBase::EventHeader header =
            EventHeaderFactory::createHeader (level);

        // create event value type using its factory
        CdmwPlatformMngt::HostStatusChange_var event =
            m_host_status_change_factory->create(header, m_system->get_element_name(),
                 m_element_name.in(),  host_status);

        // Trace to the log manager (must be done before addEvent)
        CORBA::String_var event_str = event->to_string();
        LogMngr::logMessage(event_str.in());
        event->_add_ref();

        // Perform the status change notification
        m_event_notifier->addEvent(event);

        // Perform the status change notification to observer registered with the Api in ManagedElement
        //PCR-809: Uncommented the following event dispatch
        event->_add_ref();
        get_status_event_notifier()->add_event(event);
        event->_remove_ref();

       if (host_status == CdmwPlatformMngt::HOST_UNREACHABLE)
        {
            // update applications of system with unavailable host name
            m_system->updateApplications (get_element_name());
        }

        // release ownership if no error
        event._retn();

    }
    catch (const std::bad_alloc &)
    {
        throw CORBA::NO_MEMORY (OrbSupport::NO_MEMORY, CORBA::COMPLETED_NO);
    }
    catch (...)
    {
        throw CORBA::INTERNAL (OrbSupport::INTERNAL, CORBA::COMPLETED_NO);
    }
}

void Host_impl::notify (CdmwPlatformMngt::Event* an_event)
    throw (CORBA::SystemException)
{

    /*std::string this_hostname = get_element_name();
    if (an_event->event_kind() == CdmwPlatformMngt::PROCESS_STATUS_CHANGE )
    {
        CdmwPlatformMngt::ProcessStatusChange* event = CdmwPlatformMngt::ProcessStatusChange::_downcast( an_event );
        std::string hostname = (const char*)(event->process_id().host_name);
        std::string appname = (const char*)(event->process_id().application_name);
        std::string process_name = (const char*)event->process_id().process_name;

        try
        {
            // Check if the process belongs to this host

            if (hostname == this_hostname)
            {
             // Check the status af all the processes of this host

                HostState::ProcessesStatus status = HostState::NO_PROCESS_RUNNING;
                CORBA::ULong running_processes = 0;
                CORBA::ULong not_running_processes = 0;
                CORBA::ULong ended_processes = 0;
                CORBA::ULong processes_with_relevant_indicator = 0;

                {//mutex scope
                CDMW_MUTEX_GUARD(m_mutex_host_record);
                CDMW_MUTEX_GUARD(*(m_system->get_consistency_status_recursive_mutex()));
                CORBA::ULong seq_size = get_number_of_processes();
                CdmwPlatformMngt::ProcessesIterator_var proc_iter;
                CdmwPlatformMngt::Processes_var all_processes =
                    get_processes(seq_size, proc_iter.out());


                for (CORBA::ULong i=0; i<seq_size; i++)
                {

                    if ( ((*all_processes)[i])-> get_indicator_relevant_status() )
                    {

                        processes_with_relevant_indicator++;
                        CORBA::String_var status_info;
                        if ( ((*all_processes)[i])->get_status(status_info.out()) == CdmwPlatformMngt::PROCESS_RUNNING
                              ||
                             ((*all_processes)[i])->get_status(status_info.out()) == CdmwPlatformMngt::PROCESS_RUN_REQUEST
                              ||
                             ((*all_processes)[i])->get_status(status_info.out()) == CdmwPlatformMngt::PROCESS_INITIALISED )
                        {
                            running_processes++;
                        }
                        else if ( ((*all_processes)[i])->get_status(status_info.out()) == CdmwPlatformMngt::PROCESS_ENDED )
                        {
                            ended_processes++;
                        }
                        else
                        {
                            not_running_processes++;
                        }
                        if ((running_processes > 0) && (not_running_processes > 0))
                        {
                            // If we are here we already know what we need...
                            break;
                        }
                    }
                }

                }//end mutex scope

        //Check if all process in in ended, n this case the host state must be RESPONDING so we set status to NO_PROCESS_RUNNING
        if (ended_processes == processes_with_relevant_indicator )
                {
                    status = HostState::NO_PROCESS_RUNNING;
                }
                else if (running_processes == (processes_with_relevant_indicator - ended_processes) )
                {
                    status = HostState::ALL_PROCESSES_RUNNING;
                }
                else if (not_running_processes == (processes_with_relevant_indicator - ended_processes))
                {
                    status = HostState::NO_PROCESS_RUNNING;
                }
                else
                {
                    status = HostState::NOT_ALL_PROCESSES_RUNNING;
                }
                // Notify the statemachine according to the state found
                try
                {
                    HostState* hs = m_state_machine->getState();
                    if (hs != NULL) {
                        hs->notify_processes_status(m_state_machine, status);
                    }
                }
                catch (const CdmwPlatformMngt::IncompatibleStatus& e)
                {
                CDMW_ERROR(Cdmw::Logging::Logger::Get_Logger(CdmwPlatformMngtBase::PFMMNG),
                "IncompatibleStatus for host: " << m_element_path.in());

                }
                catch (...) {
                CDMW_ERROR(Cdmw::Logging::Logger::Get_Logger(CdmwPlatformMngtBase::PFMMNG),
                "Unexpected exception for host: " << m_element_path.in());
                }
            }
        }
        catch (const CORBA::SystemException& ) {

             CDMW_ERROR(Cdmw::Logging::Logger::Get_Logger(CdmwPlatformMngtBase::PFMMNG),
                "CORBA::SystemException for host: " << m_element_path.in());
       }
        catch (...) {
        }
    }
    // We implement here the command logic
    else if (an_event->event_kind() == CdmwPlatformMngt::HOST_STATUS_CHANGE )
    {*/
       CdmwPlatformMngt::HostStatusChange* event = CdmwPlatformMngt::HostStatusChange::_downcast( an_event );
       /*std::string hostname = (const char*)(event->host_name());
      if (hostname == this_hostname)
            {*/


        switch (event->host_status())
        {
        case CdmwPlatformMngt::HOST_RESPONDING:
            switch (m_current_host_command)
            {
            case SHUTDOWN:
                 try
                 {
                     execute_host_command("HOST_SHUTDOWN");
                 }
                 catch (const CORBA::Exception & ex)
                 {
                       CDMW_ERROR(Cdmw::Logging::Logger::Get_Logger(CdmwPlatformMngtBase::PFMMNG),
                                  "Exception: " <<ex );
                      throw;
                 }

                 break;

            case STOP:
                try
                {
                    execute_host_command("HOST_STOP");
                }
                catch (BadParameterException& ex)
                {
                }
                catch (OutOfMemoryException)
                {
                }
                catch (InternalErrorException)
                {
                }
                catch (...)
                {
                    throw CORBA::NO_PERMISSION(OrbSupport::NO_PERMISSION, CORBA::COMPLETED_NO);
                }
                set_current_host_command(NO_COMMAND);

                break;

            case COLD_RESTART:
            case HOT_RESTART:
                // Reboot the host
                if (get_reboot_in_progress())
                {
                    execute_host_command("HOST_STOP");
                    set_reboot_in_progress(false);
                }
                else
                {
                if ( CdmwPlatformMngt::COLD_RESTART==requested_restart_warmth() )
                    {
                     CDMW_INFO(Cdmw::Logging::Logger::Get_Logger(CdmwPlatformMngtBase::PFMMNG),
                         "recover default proprietes");
                 recover_default_properties ();
                }
                    // Launch all the processes
                    start_processes();
                    set_current_host_command(NO_COMMAND);
                }
                break;

            case COLD_REBOOT:
            case HOT_REBOOT:
                if (get_reboot_in_progress())
                {
                    // Reboot the host
                    execute_host_command("HOST_REBOOT");
                    set_reboot_in_progress(false);
                }
                else
                {  
                if ( CdmwPlatformMngt::COLD_RESTART==requested_restart_warmth() )
                    {
                     CDMW_INFO(Cdmw::Logging::Logger::Get_Logger(CdmwPlatformMngtBase::PFMMNG),
                         "recover default proprietes");
                 recover_default_properties ();
                }
            // Launch all the processes
                    start_processes();
                    set_current_host_command(NO_COMMAND);
                }

                break;
            case NO_COMMAND:
           {

           std::string HSU = host_status_strings[6];//"Unreachable"
           if(!HSU.compare(m_state_machine->getPreviousStateName())){
               if ( CdmwPlatformMngt::COLD_RESTART==requested_restart_warmth() )
               {
                    CDMW_INFO(Cdmw::Logging::Logger::Get_Logger(CdmwPlatformMngtBase::PFMMNG),
                    "recover default proprietes");
                    recover_default_properties ();
               }
               if(Configuration::Get_restart_process_on_responding_option()){
                    start_processes();
               }       
           }
           }

           break;
            default:
                // Do nothing.
                break;
            }
            break;

        case CdmwPlatformMngt::HOST_UNREACHABLE:
            switch (m_current_host_command)
            {
            case SHUTDOWN:
                set_current_host_command(NO_COMMAND);
                break;

            case STOP:
                 CDMW_INFO(Cdmw::Logging::Logger::Get_Logger(CdmwPlatformMngtBase::PFMMNG),
                    "Host completely stopped: " << m_element_path.in());
                break;

            case COLD_RESTART:
            case HOT_RESTART:;
                execute_host_command("HOST_START");
                break;

            default:
                // Do nothing...
                break;
            }
            break;
        default:
            // Do nothing...
            break;

        }
    //}
    //}
}

bool Host_impl::set_current_host_command(HostCommand hc)
{
    // If we want to set NO_COMMAND we can do it in any case
    // If we want to set any other command we can do it only if
    // the current command is NO_COMMAND
    CDMW_MUTEX_GUARD(m_command_mutex);
    if ((hc == NO_COMMAND) || (m_current_host_command == NO_COMMAND))
    {
        m_current_host_command = hc;
        return true;
    }
    else
    {
        return false;
    }
}

Host_impl::HostCommand Host_impl::get_current_host_command()
{
    return m_current_host_command;
}




void Host_impl::start_remote_host(std::string hostname)
throw (CORBA::NO_PERMISSION)
{
    try
    {
        std::string args = m_start_host_script + " " + hostname + " &";
        OsSupport::OS::create_process("/bin/sh" , args , "", NULL, NULL);
    }
    catch (...)
    {
        // the deployment of the daemon doesn't allow the operation
        //LogMngr::logMessage("cdmw_host_reboot.sh failed");
        throw CORBA::NO_PERMISSION(OrbSupport::NO_PERMISSION, CORBA::COMPLETED_NO);
    }
}

void Host_impl::execute_shell_command(std::string command)
    throw (CORBA::NO_PERMISSION)
{
    try
    {
        OsSupport::OS::create_process("/bin/sh" , command , "", NULL, NULL);
    }
    catch (...)
    {
        throw CORBA::NO_PERMISSION(OrbSupport::NO_PERMISSION, CORBA::COMPLETED_NO);
    }
}


//
// return host snapshot
//
CdmwPlatformMngt::HostSnapshot* Host_impl::get_snapshot()
                                throw (CORBA::SystemException)
{
        try
        {

            CdmwPlatformMngt::HostSnapshot_var snapshot =
                   new CdmwPlatformMngt::HostSnapshot;

                snapshot->host_name = name();
                snapshot->host_status = status();
                snapshot->element_path = element_path();
                // Get the snapshot of all entities of the host
                CdmwPlatformMngt::EntitiesSnapshot_var entitiesSnapshot =
                   new CdmwPlatformMngt::EntitiesSnapshot();

                // Gets all the entities of the host
                CdmwPlatformMngt::Entities_var entities;
                CdmwPlatformMngt::EntitiesIterator_var entitiesIt;
                CORBA::ULong nbr_of_entities;

                nbr_of_entities = get_number_of_entities();

                get_all_entities(nbr_of_entities,                              entities.out(), entitiesIt.out());

                entitiesSnapshot->length(nbr_of_entities);
                for (unsigned int j = 0; j < nbr_of_entities; j++)
                    {
                    (*entitiesSnapshot)[j] = *(entities[j]->get_snapshot());                        
                    }

                CosPropertyService::PropertiesIterator_var propertiesIt;
                CORBA::ULong nbr_of_properties;
                CdmwPlatformMngt::PropertiesSnapshot_var propertiesSnapshot = new CdmwPlatformMngt::PropertiesSnapshot;
                nbr_of_properties = get_number_of_properties();
                CosPropertyService::PropertyNames_var prop_names;
                CosPropertyService::PropertyNamesIterator_var prop_namesIt;
                get_all_property_names(nbr_of_properties, prop_names.out(), prop_namesIt.out());
                propertiesSnapshot->length(nbr_of_properties-2);
                CORBA::ULong snap_prop=0;
                for (unsigned int j = 0; j < nbr_of_properties ; j++)
                {
                    if ((strcmp (prop_names[j], "Cdmw.TechStatus")==0) ||
                            (strcmp (prop_names[j], "Cdmw.TechStatusInfo")==0))
                    {
                        // do not add the TECH_STATUS and TECH_STATUS_INFO properties to the snapshot 
                    }
                    else
                    {
            CdmwPlatformMngt::PropertySnapshot_var propertySnapshot = new CdmwPlatformMngt::PropertySnapshot();
                        propertySnapshot -> property_name = CORBA::string_dup(prop_names[j]);
                        CORBA::Any prop_value;
                        prop_value = *(get_property_value(prop_names[j]));
                        CORBA::TypeCode_var tc;
                        tc = prop_value.type();
                        switch (tc->kind())
                        {
                        default:
                            break;
                        case CORBA::tk_string:
                            propertySnapshot -> property_type = CORBA::string_dup("String");
                            break;
                        case CORBA::tk_short:
                            propertySnapshot -> property_type = CORBA::string_dup("Short");
                            break;
                        case CORBA::tk_float:
                            propertySnapshot -> property_type = CORBA::string_dup("Float");
                            break;
                        };

                        CosPropertyService::PropertyModeType mode = get_property_mode(prop_names[j]);
                        propertySnapshot->property_value = prop_value;

                        if (mode == CosPropertyService::normal) propertySnapshot->property_mode = CORBA::string_dup ("Read/Write");
                        else if (mode == CosPropertyService::read_only) propertySnapshot->property_mode = CORBA::string_dup ("Read/Only");
                        else propertySnapshot->property_mode = CORBA::string_dup ("Pre-defined");

                        propertySnapshot->property_value = prop_value;
            (*propertiesSnapshot)[snap_prop] = *(propertySnapshot);
                        
                        snap_prop++;
                    }
                }
            snapshot->host_entities = entitiesSnapshot.in();
            snapshot->host_properties = propertiesSnapshot.in();

            CdmwPlatformMngt::CompleteMonitoringInfo_var monitoring_info = get_monitoring_parameters();

            snapshot->monitoring_parameters = monitoring_info.in();

        return snapshot._retn();
    }

    catch(const std::bad_alloc &)
    {
          throw CORBA::NO_MEMORY (OrbSupport::NO_MEMORY, CORBA::COMPLETED_NO);
    }
}

void Host_impl::execute_host_command(std::string command)
    throw (CORBA::NO_PERMISSION)
{
    try
    {
       std::ostringstream ostr;
       ostr << "corbaloc::" << get_element_name() << ":" << Configuration::Get_DaemonController_port_number() << "/daemon_controller";
       std::string index = ostr.str();
       CDMW_INFO(Cdmw::Logging::Logger::Get_Logger(CdmwPlatformMngtBase::PFMMNG), "CORBALOC : " << index);
       CdmwPlatformMngt::DaemonController_var iniitdm =  m_system->getDaemonController(index);
/*
       int number_of_retries = 1;

       while (number_of_retries <= 10 && CORBA::is_nil(iniitdm.in())){
            
            Cdmw::OsSupport::OS::sleep (2000);
            
            CDMW_INFO(Cdmw::Logging::Logger::Get_Logger(CdmwPlatformMngtBase::PFMMNG), "Trying to get DaemonController reference - number of retries "<<number_of_retries);
            iniitdm =  m_system->getDaemonController(index);
            number_of_retries ++;
       }
*/
       if (!CORBA::is_nil(iniitdm.in()))
       {
           CDMW_INFO(Cdmw::Logging::Logger::Get_Logger(CdmwPlatformMngtBase::PFMMNG), "DaemonController found");
           if (command=="HOST_START"){
                iniitdm->notify(5,m_system->systemPort());
           }
           else if (command=="HOST_STOP"){
                iniitdm->notify(2,m_system->systemPort());
           }
           else if (command=="HOST_REBOOT"){
                iniitdm->notify(3,m_system->systemPort());
           }
           else if (command=="HOST_SHUTDOWN"){
                iniitdm->notify(4,m_system->systemPort());
           }
           else CDMW_WARN(Cdmw::Logging::Logger::Get_Logger(CdmwPlatformMngtBase::PFMMNG),
                       "Command not found");
           }
        else {
            CDMW_INFO(Cdmw::Logging::Logger::Get_Logger(CdmwPlatformMngtBase::PFMMNG), "DaemonController not found");
            execute_host_command_without_daemoncontroller(command);
        }
        
    }
    //if i dont find the daemonmonitor i use the shell command
    catch (...)
    {
         execute_host_command_without_daemoncontroller(command);
    }
}

void Host_impl::execute_host_command_without_daemoncontroller(std::string command)
    throw (CORBA::NO_PERMISSION)
{        
    if (command=="HOST_START"){
        CDMW_INFO(Cdmw::Logging::Logger::Get_Logger(CdmwPlatformMngtBase::PFMMNG), "Host start without daemon controller");
        OsSupport::OS::create_process(m_start_host_script , get_element_name() , "", NULL, NULL);
    }
    else if (command=="HOST_STOP"){
        CDMW_INFO(Cdmw::Logging::Logger::Get_Logger(CdmwPlatformMngtBase::PFMMNG), "Host stop without daemon controller");
        OsSupport::OS::create_process(m_stop_host_script , get_element_name() , "", NULL, NULL);
    }
    else if (command=="HOST_REBOOT"){
        CDMW_INFO(Cdmw::Logging::Logger::Get_Logger(CdmwPlatformMngtBase::PFMMNG), "Host reboot without daemon controller");
        OsSupport::OS::create_process(m_reboot_host_script , get_element_name() , "", NULL, NULL);
    }
    else if (command=="HOST_SHUTDOWN"){
        CDMW_INFO(Cdmw::Logging::Logger::Get_Logger(CdmwPlatformMngtBase::PFMMNG), "Host shutdown without daemon controller");    
        OsSupport::OS::create_process(m_shutdown_host_script , get_element_name() , "", NULL, NULL);
    }
}

void Host_impl::init_preemption ()
{
    CDMW_MUTEX_GUARD (m_preemption_flag_mutex);
    
    m_preemption_flag = false;
}

bool Host_impl::is_preemption_active ()
{
    CDMW_MUTEX_GUARD (m_preemption_flag_mutex);
    
    return m_preemption_flag;
}    
            
bool Host_impl::activate_preemption ()
{
   bool result = false;
   
   CDMW_MUTEX_GUARD (m_preemption_flag_mutex);

   bool old_flag_value = m_preemption_flag;
         
   try 
   {
         CORBA::String_var statusInfo;

         CdmwPlatformMngt::SystemStatus system_status = m_system->get_status(statusInfo.out());
         CdmwPlatformMngt::HostStatus host_status = status();
                
         if (system_status == CdmwPlatformMngt::SYSTEM_STARTING
              || system_status == CdmwPlatformMngt::SYSTEM_INITIAL
              || host_status == CdmwPlatformMngt::HOST_STARTING
              || host_status == CdmwPlatformMngt::HOST_REINSERTING)
         {
             m_preemption_flag = true;
             result = true;   
         }                         
         
    }
    catch (...)
    {
         old_flag_value = m_preemption_flag;
         CDMW_ERROR(Cdmw::Logging::Logger::Get_Logger(CdmwPlatformMngtBase::PFMMNG), "  Can't activate preemption of " << name() << " host " << std::endl);
    }

   
   return result;
}   
    
bool Host_impl::de_activate_preemption() 
{
    bool result = false;
 
    CDMW_MUTEX_GUARD (m_preemption_flag_mutex);
          
    try 
    {
        CORBA::String_var sysStatusInfo;

         CdmwPlatformMngt::SystemStatus system_status = m_system->get_status(sysStatusInfo.out());
                                      
        if (m_preemption_flag)
        {
            if (system_status == CdmwPlatformMngt::SYSTEM_STARTING 
                  || system_status == CdmwPlatformMngt::SYSTEM_INITIAL)
            {                  
            } 
            else
            {
               m_preemption_flag = false;
               result = true;   
            }
       }       
          
    }
    catch (...)
    {

         CDMW_ERROR(Cdmw::Logging::Logger::Get_Logger(CdmwPlatformMngtBase::PFMMNG), 
              "  Can't deactivate preemption of " << name() << " host " << std::endl);
         m_preemption_flag = false;
         result = true;
    }         

    return result;
}

Cdmw::OsSupport::ThreadStatus Host_impl::get_consistency_checker_status()
{
    return m_host_status_checker->get_status();
}

} // End namespace PlatformMngt
} // End namespace Cdmw


