/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#include "Foundation/orbsupport/OrbSupport.hpp"

#include "SystemMngt/platformlibrary/CommonLogMessageId.hpp"
#include "SystemMngt/platformapplicationlibrary/ApplicationLogMessageId.hpp"
#include "SystemMngt/idllib/CdmwPlatformMngtBase.stub.hpp"

#include "platformsystemlibrary/SystemLogMngr.hpp"
#include "platformsystemlibrary/SystemLogMessageId.hpp"
#include "SystemMngt/tools/AdminLogMessageId.hpp"
#include "SystemMngt/tools/MessageMapManager.hpp"


namespace Cdmw
{
namespace PlatformMngt
{

static const char* messages[] =
{    
    "Invalid options",
    "Specified port %s is invalid",
    "Port not specified",
    "Cannot start supervision : port %s is already used",
    "Specified event notifier timeout %s is invalid",
    "Event notifier timeout not specified",
    "Specified creation timeout %s is invalid",
    "Creation timeout not specified",
    "Log file not specified",   
    "Supervision started on port %s",
    "Reference published with \"corbaloc::%s:%s/%s\"",
    "Log written in file \"%s\"",
    "RunSetManager runs out of memory",
    "Define System \"%s\"",
    "Add Application \"%s\" in system \"%s\"",
    "Remove Application \"%s\" in system \"%s\"",
    "Set Init Graph for system \"%s\"",
    "Set Stop Graph for system \"%s\"",
    "Set Run Set for system \"%s\"",
    "Remove Run Set for system \"%s\"",
    "Remove All Run Set for system \"%s\"",
    "Add Host \"%s\" for system \"%s\"",
    "Remove Host \"%s\" for system \"%s\"",
    "Start system \"%s\"",
    "Resume system \"%s\"",
    "Acknowledge system \"%s\"",
    "Stop system \"%s\"",
    "Start Application Init Sequencer",
    "Resume Application Init Sequencer",
    "Start Application Stop Sequencer",
    "Start RunSet \"%s\"",
    "Register Supervision Observer \"%s\" for system \"%s\"", 
    "Unregister Supervision Observer \"%s\" for system \"%s\"",
    "Initialisation by sequencer of application \"%s\"",
    "Status OK for initialising application \"%s\"",
    "Incompatible status for initialising application \"%s\"",
    "System exception '%s' \n   during initialisation of application \"%s\"",
    "Start processing by starter of application \"%s\"",
    "Status OK for starting application \"%s\"",
    "Incompatible status for starting application \"%s\"",
    "System exception '%s' \n   during starting of application \"%s\"",
    "Stop processing by sequencer of application \"%s\"",
    "Status OK for stopping application \"%s\"",
    "Incompatible status for stopping application \"%s\"",
    "System exception '%s' \n   during stopping of application \"%s\"",
    "System Unexpected Exception \"%s\" for application \"%s\"",
    "Supervision Process Name not specified",
    "PullMonitoring Registration Callback IOR not specified",
    "Add Entity \"%s\" for system \"%s\"",
    "Remove Entity \"%s\" for system \"%s\"",
    "Add Entity \"%s\" for host \"%s\"",
    "Remove Entity \"%s\" for host \"%s\"",
    "Specified connection timeout %s is invalid",
    "Connection timeout not specified",
    "Connection timeout is specified: %s ms",
    "Signal %d trapped. Platform Supervision exiting...",
    "Error while shuting ORB down in Supervision exit_handler:\n %s",
    "########## Supervision exits.",
    "Check of consistent status for system \"%s\"",
    "Initialisation of application \"%s\" not executed. Force-manual-start = MANUAL.",
    "Host status check period set to \"%s\" msec"
};

const char** SystemLogMessages::M_systemMessages = messages;



void SystemLogMngr::initialise(int& argc, char** argv)
     throw (OutOfMemoryException, InternalErrorException)
{
    // if init not already done
    if (M_pSingleton == NULL)
    {
        try 
        {
            // create the concrete Log Manager singleton
            M_pSingleton = new SystemLogMngr (argc, argv);
        }
        catch (const std::bad_alloc&)
        {
          CDMW_THROW (OutOfMemoryException);
        }
    }
}


SystemLogMngr::SystemLogMngr(int& argc, char** argv)
        throw(OutOfMemoryException, InternalErrorException)
   : LogMngr (argc, argv)
{
}
            
             
SystemLogMngr::~SystemLogMngr()
        throw()
{
}


const char* SystemLogMngr::getService() const
{
    return CdmwPlatformMngtBase::PFMMNG;
}


//
// Get the specified text message associated to message ident
// This method is static and belongs to the logMngr class
// It is defined specifically for this system process
// It includes the common, application and system messages
//
//  @param messageId    the ident of message.
//
//  @return  the text of message.
//    
const char* SystemLogMngr::getMessage(long messageId) const
{
    const char *ret_val = 0;
    // if application message
    if ((messageId >= 0)  && (messageId < MSG_ID_APPLICATION_END))
    {
        ret_val = ApplicationLogMessages::M_applicationMessages [messageId];
    }
    // if system message
    else if ((messageId >= MSG_ID_SYSTEM_START) &&
             (messageId < MSG_ID_SYSTEM_END))
    {
         ret_val =  SystemLogMessages::M_systemMessages [messageId - MSG_ID_SYSTEM_START];
    }
    // if common message
    else if ((messageId >= MSG_ID_COMMON_START) &&
             (messageId < MSG_ID_COMMON_END))
    {
         ret_val =  CommonLogMessages::M_commonMessages [messageId - MSG_ID_COMMON_START];
    }
    
    // if common message
    else if ((messageId >= PlatformAdmin::MSG_ID_ADMIN_COMMAND_USAGE) &&
             (messageId < PlatformAdmin::MSG_ID_ADMIN_END))
             
    {
        std::string error_message = PlatformAdmin::MessageMapManager::instance()->find(messageId);
  
        if(error_message.empty())
        error_message = "Not Valid messageId";

         ret_val =  error_message.c_str();
        
    }
     
    return ret_val;
}


} // End namespace PlatformMngt
} // End namespace Cdmw

