/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#include "tao/AnyTypeCode/TypeCode.h"
#include "platformsystemlibrary/System_impl.hpp"
#include "Foundation/osthreads/MutexGuard.hpp"

#include "platformsystemlibrary/ConsistentSynchroSolver.hpp"

#include "platformsystemlibrary/ServiceBinding_impl.hpp"
#include "platformsystemlibrary/HostContainer_impl.hpp"
#include "platformsystemlibrary/HostContainer.hpp"
#include "platformsystemlibrary/ApplicationObserver_impl.hpp"
#include "platformsystemlibrary/ApplicationIterator_impl.hpp"

#include "platformsystemlibrary/RunSetManager.hpp"
#include "platformsystemlibrary/SystemLogMessageId.hpp"

#include "platformsystemlibrary/HostTrees.hpp"
#include "SystemMngt/tools/SystemIssuerManager.hpp"

#include "SystemMngt/platformlibrary/Configuration.hpp"
#include "SystemMngt/platformlibrary/RWServiceDefContainer_impl.hpp"
#include "SystemMngt/platformlibrary/SupervisionEventNotifier.hpp"
#include "SystemMngt/platformlibrary/GraphChecker.hpp"
#include "SystemMngt/platformlibrary/GraphUtility.hpp"
#include "SystemMngt/platformlibrary/LogMngr.hpp"
#include "SystemMngt/platformlibrary/TaskFactory.hpp"
#include "SystemMngt/platformlibrary/Sequencer.hpp"
#include "SystemMngt/platformlibrary/EventHeaderFactory.hpp"
#include "SystemMngt/tools/ManagedElementKey.hpp"
#include "SystemMngt/platformlibrary/EventNotifierManager.hpp"

#include "SystemMngt/platformelementlibrary/RWEntityContainer_impl.hpp"

#include "Foundation/commonsvcs/datastore/StorageErrorException.hpp"

#include "SystemMngt/platformvaluetypes/GraphElement_impl.hpp"
#include "SystemMngt/platformvaluetypes/SystemStatusChange_impl.hpp"
#include "SystemMngt/platformvaluetypes/SystemModeChange_impl.hpp"
#include "SystemMngt/platformvaluetypes/HostStatusChange_impl.hpp"

#include "Foundation/osthreads/MutexGuard.hpp"
#include "Foundation/osthreads/ReaderLockGuard.hpp"
#include "Foundation/osthreads/WriterLockGuard.hpp"
#include "Foundation/orbsupport/ExceptionMinorCodes.hpp"
#include "Foundation/logging/Logger.hpp"
#include "Foundation/logging/LogLevel.hpp"
#include "Foundation/logging/FunctionLogger.hpp"

#include "SystemMngt/idllib/CdmwPlatformMngtEvent.stub.hpp"
#include "SystemMngt/idllib/CdmwPlatformMngtPredefinedProperties.stub.hpp"

#include "platformsystemlibrary/ApplicationInit.hpp"
#include "platformsystemlibrary/ApplicationRun.hpp"
#include "platformsystemlibrary/ApplicationStop.hpp"
#include "platformapplicationlibrary/Application_impl.hpp"

#include <string>


namespace
{
    
    const int MAX_RETRY_ON_FTMGR = 4;
    const int SLEEP_RETRY_ON_FTMGR_MS = 1000;
}




namespace Cdmw
{
    namespace PlatformMngt
    {
        
// set static members
        
        SystemInitial *System_impl::M_initial = 0;
        SystemStarting *System_impl::M_starting = 0;
        SystemStartupSuspended *System_impl::M_startupSuspended = 0;
        SystemStarted *System_impl::M_started = 0;
        SystemStopping *System_impl::M_stopping = 0;
        SystemStopped *System_impl::M_stopped = 0;
        SystemPartialStarted *System_impl::M_partialStarted = 0;
        
        const std::string System_impl::M_initialStateName = "Initial";
        const std::string System_impl::M_startingStateName = "Starting";
        const std::string System_impl::M_startupSuspendStateName =
            "StartupSuspend";
        const std::string System_impl::M_startedStateName = "Started";
        const std::string System_impl::M_stoppingStateName = "Stopping";
        const std::string System_impl::M_stoppedStateName = "Stopped";
        const std::string System_impl::M_partialStartedStateName = "PartialStarted";


       // string array used to LogMngr to print ApplicationStatusChange enum
       static const std::string application_status_strings[] =
       {"APPLICATION_INITIAL", "APPLICATION_INITIALISING", "APPLICATION_INITIALISATION_SUSPENDED",
        "APPLICATION_INITIALISATION_INCOMPLETE", "APPLICATION_INITIALISED", "APPLICATION_RUN_REQUEST",
        "APPLICATION_RUNNING", "APPLICATION_STOPPING","APPLICATION_STOPPED","APPLICATION_PARTIAL_RUNNING"
       };
       
       static const std::string sys_status_strings[] =
       {
               "Initial","Starting","StartupSuspend","Started","Stopping",
               "Stopped","PartialStarted"
       };
       static const CdmwPlatformMngt::SystemStatus Status[]=
       {
               CdmwPlatformMngt::SYSTEM_INITIAL,
               CdmwPlatformMngt::SYSTEM_STARTING,
               CdmwPlatformMngt::SYSTEM_STARTUP_SUSPENDED,
               CdmwPlatformMngt::SYSTEM_STARTED,
               CdmwPlatformMngt::SYSTEM_STOPPING,
               CdmwPlatformMngt::SYSTEM_STOPPED,
               CdmwPlatformMngt::SYSTEM_PARTIAL_STARTED
       };
//
// Dumper of supervision observer from datastore
// operator ()
//
        void ObserverDumper::operator () (const PairType & elt)
        {
            // for the extracted record from datastore
            // register the specified supervision observer
            try
            {
                if (!strcmp ((elt.second)->host_name, ""))
                {
                    m_system->register_observer (elt.first.c_str (),
                        (elt.second)->observer.in ());
                }
                else
                {
                    m_system->register_proxy_observer (elt.first.c_str (),
                        (elt.second)->host_name,
                        (elt.second)->observer.in ());
                }
            }
            catch (...)
            {
                // nothing to do
            }
        }
        
// ----------------------------------------------------------------------
// System_impl class.
// ----------------------------------------------------------------------

System_impl::System_impl (SupervisionEventNotifier * eventNotifier, CORBA::ORB_ptr orb, 
        PortableServer::POA_ptr poa, CdmwPlatformMngt::RWElementRepository_ptr element_repository) 
    
    throw (BadParameterException, AlreadyDoneException, UnknownStateException, AlreadyExistsException, 
        OutOfResourcesException, OutOfMemoryException)
    
    : ManagedElement_impl(element_repository, poa), m_stop_condition (m_mutex), m_stop_condition_flag (false)
    
    {
        if (eventNotifier == NULL)
        {
            CDMW_THROW2 (BadParameterException, "eventNotifier", "0");
        }

        if (CORBA::is_nil (poa))
        {
            CDMW_THROW2 (BadParameterException, "poa", "nil");
        }

      try
      {
    Cdmw::CommonSvcs::DataStore::DataStoreBase * ds1 =
      Cdmw::CommonSvcs::DataStore::DataStoreRegistry::
      Get_data_store (Configuration::Get_system_datastore_id ());
    m_system_ds =
      dynamic_cast < Cdmw::PlatformMngt::SystemDataStore * >(ds1);

    Cdmw::CommonSvcs::DataStore::DataStoreBase * ds2 =
      Cdmw::CommonSvcs::DataStore::DataStoreRegistry::
      Get_data_store (Configuration::
              Get_system_observer_registration_datastore_id ());
    m_system_observer_ds =
      dynamic_cast <
      Cdmw::PlatformMngt::SystemObserverRegistrationDataStore * >(ds2);
      }
      catch (const NotExistException & e)
      {
    CDMW_ASSERT (false);
      }

      // Set the internal data
      m_eventNotifier = eventNotifier;
      m_orb = CORBA::ORB::_duplicate (orb);
       
      m_poa = PortableServer::POA::_duplicate (poa);


      m_systemDefined = false;
      m_systemStarted = false;

      set_status_info ( CORBA::string_dup (""));
      m_mode = CdmwPlatformMngt::NORMAL_MODE;
      m_modeInfo = CORBA::string_dup ("");
      m_emergency = false;
      m_newGraph = true;
      m_startup_kind = new CdmwPlatformMngtBase::StartupKind ();

      m_actualServiceDefContainer = 0;
      m_actualEntityContainer = 0;

      m_initSyncTaskFactory = 0;
      m_initSequencer = 0;

      m_stopSyncTaskFactory = 0;
      m_stopSequencer = 0;

      m_stateMachine = 0;
      m_rwServiceDefContainer = 0;
      m_rwEntityContainer = 0;
      m_actualHostContainer = 0;
      m_runSetManager = 0;

      m_sysStatusChangeFactory = 0;
      m_sysModeChangeFactory = 0;
      m_hostStatusChangeFactory = 0;
      m_sysRestartFactory = 0;
      m_sysStopFactory = 0;
      m_sysAcknowledgeFactory = 0;
      
      m_default_entitydefs = 0;
      
      m_consistent_synchro_solver = 0;
      
      try
      {
    // create an empty init application graph
    std::auto_ptr < CdmwPlatformMngt::GraphElements >
      applicationInitGraph (new CdmwPlatformMngt::GraphElements);

    // create an empty stop application graph
    std::auto_ptr < CdmwPlatformMngt::GraphElements >
      applicationStopGraph (new CdmwPlatformMngt::GraphElements);

    // create an empty stop emergency graph
    std::auto_ptr < CdmwPlatformMngt::GraphElements >
      stopGraph (new CdmwPlatformMngt::GraphElements);



    // create local objects        
    std::auto_ptr < SystemStateMachine >
      stateMachine (new SystemStateMachine (this, "System"));

    std::auto_ptr < PlatformMngt::HostContainer >
      actualHostContainer (new PlatformMngt::
                   HostContainer (this, poa, element_repository));


    std::auto_ptr < RunSetManager >
      runSetManager (new RunSetManager (this));


    std::auto_ptr < SystemStatusChangeFactory >
      sysStatusChangeFactory (new SystemStatusChangeFactory ());
    std::auto_ptr < SystemModeChangeFactory >
      sysModeChangeFactory (new SystemModeChangeFactory ());
    std::auto_ptr < HostStatusChangeFactory >
      hostStatusChangeFactory (new HostStatusChangeFactory ());
    std::auto_ptr < RestartFactory >
        sysRestartFactory (new RestartFactory ());
    
    std::auto_ptr < StopFactory >
        sysStopFactory (new StopFactory ());
    
    std::auto_ptr < AcknowledgeFactory >
        sysAcknowledgeFactory (new AcknowledgeFactory ());
    
    // create the actual system entity container with empty parent name         
    m_actualEntityContainer =
      new PlatformMngt::EntityContainer (m_poa.in (),
                         m_element_repository.in (), "");

    // create the rw system entity container                                      
    m_rwEntityContainer =
      new RWEntityContainer_impl (m_poa.in (), m_actualEntityContainer,
                      m_element_repository.in ());
    // create an object var to take pointer ownership
    PortableServer::ServantBase_var rwEntityContainer =
      m_rwEntityContainer;


    // create the actual system service def container
    m_actualServiceDefContainer =
      ServiceDefContainer_impl::ServiceDefContainer::
      createSystemContainer ();

    // create the rw system service def container
    m_rwServiceDefContainer =
      new RWServiceDefContainer_impl (m_poa.in (),
                      m_actualServiceDefContainer);
    // create an object var to take pointer ownership
    PortableServer::ServantBase_var rwServiceDefContainer =
      m_rwServiceDefContainer;

    // create consistent synchro solver
    std::auto_ptr < ConsistentSynchroSolver >
      consistentSynchroSolver (new ConsistentSynchroSolver (this));
      
    // Create Object servants

    // Create the ServiceDefContainer servant
    m_serviceDefContainer =
      new ServiceDefContainer_impl (m_poa.in (),
                    m_actualServiceDefContainer);
    // create an object var to take pointer ownership
    PortableServer::ServantBase_var serviceDefContainerServant =
      m_serviceDefContainer;
    // activate object
    m_serviceDefContainer->activate ();
    m_serviceDefContainerObject = m_serviceDefContainer->_this ();


    // Create the ServiceBinding servant
    m_serviceBinding = new ServiceBinding_impl (this);
    // create an object var to take pointer ownership
    PortableServer::ServantBase_var serviceBindingServant =
      m_serviceBinding;
    // activate object
    m_serviceBindingObject = m_serviceBinding->_this ();


    // Create the HostContainer servant
    m_hostContainer =
      new HostContainer_impl (m_poa.in (), actualHostContainer.get (),
                  element_repository);
    // create an object var to take pointer ownership
    PortableServer::ServantBase_var hostContainerServant =
      m_hostContainer;
    // activate object
    m_hostContainerObject = m_hostContainer->_this ();


    // Create the ApplicationObserver servant
    m_applicationObserver = new ApplicationObserver_impl (this);
    // create an object var to take pointer ownership
    PortableServer::ServantBase_var applicationObserverServant =
      m_applicationObserver;
    // activate object
    m_applicationObserverObject = m_applicationObserver->_this ();


    // Set the internal data   
    m_applicationInitGraph = applicationInitGraph.get ();
    m_applicationStopGraph = applicationStopGraph.get ();
    m_stopGraph = stopGraph.get ();
    m_stateMachine = stateMachine.get ();
    m_actualHostContainer = actualHostContainer.get ();
    m_runSetManager = runSetManager.get ();
    m_sysStatusChangeFactory = sysStatusChangeFactory.get ();
    m_sysModeChangeFactory = sysModeChangeFactory.get ();
    m_hostStatusChangeFactory = hostStatusChangeFactory.get ();
    m_sysRestartFactory = sysRestartFactory.get ();
    m_sysStopFactory = sysStopFactory.get ();
    m_sysAcknowledgeFactory = sysAcknowledgeFactory.get ();
    m_consistent_synchro_solver = consistentSynchroSolver.get();

    //Period set to 1500 milliseconds by default
    m_host_status_check_period = 1500;

    // Add the states to the state machine
    m_stateMachine->addState (M_initialStateName, M_initial);
    m_stateMachine->addState (M_startingStateName, M_starting);
    m_stateMachine->addState (M_startupSuspendStateName,
                  M_startupSuspended);
    m_stateMachine->addState (M_startedStateName, M_started);
    m_stateMachine->addState (M_stoppingStateName, M_stopping);
    m_stateMachine->addState (M_stoppedStateName, M_stopped);
        m_stateMachine->addState (M_partialStartedStateName, M_partialStarted);

        try
        {
            // define TECH_STATUS and TECH_STATUS_INFO properties
            const char* initial_value = "";
            define_status_property(initial_value);
        }
        catch (...)
        {
            throw CosPropertyService::MultipleExceptions ();
        }        
    // Set the initial state
    m_stateMachine->setState (M_initialStateName);


    // start event handler
    m_eventHandler.start ();


    // start RunSetManager thread
    m_runSetManager->start ();

        //Enable PropertyEvent features
        set_property_event_notifier(EventNotifierManager::Instance()->get_property_notifier());
        //Enable StatusEvent features
        set_status_event_notifier(EventNotifierManager::Instance()->get_status_notifier());
  
    // Release the temporary objects
    applicationInitGraph.release ();
    applicationStopGraph.release ();
    stopGraph.release ();
    stateMachine.release ();
    actualHostContainer.release ();
    runSetManager.release ();
    sysStatusChangeFactory.release ();
    sysModeChangeFactory.release ();
    hostStatusChangeFactory.release ();
    sysRestartFactory.release ();
    sysStopFactory.release ();
    sysAcknowledgeFactory.release ();
    consistentSynchroSolver.release ();
    

    // release servant var for local object (to avoid automatic desallocation)
    rwEntityContainer._retn ();
    rwServiceDefContainer._retn ();
      }
      catch (const std::bad_alloc &)
      {
          // destroy containers
          try
          {
              ServiceDefContainer_impl::ServiceDefContainer::
                  destroy (m_actualServiceDefContainer);
          }
          catch (...)
          {
          }
          
          CDMW_THROW (OutOfMemoryException);
      }
      
      // MdR define system properties not user defined
        std::string init_value="NO_PRIMARY_SUPERVISION_DEFINED_YET";
        std::ostringstream property_name;
        CORBA::Any_var property_any_val = new CORBA::Any();
        try
        {                   
            *property_any_val <<= init_value.c_str();
            property_name.str("");
            property_name << CdmwPlatformMngt::CDMW_PREFIX 
                          << CdmwPlatformMngt::GLOBAL_SUPERVISION_HOST << std::ends;
            define_property_with_mode(property_name.str().c_str(), 
                                      property_any_val.in(), 
                                      CosPropertyService::fixed_normal);
        }
        catch (...)
        {
            throw CosPropertyService::MultipleExceptions ();
        }
        
        
        
    }
    
    
    
    System_impl::~System_impl ()
    {
        // stop event handler
        m_eventHandler.stop ();
        
        // destroy init and stop sequencers
        destroyInitSequencer ();
        destroyStopSequencer ();
        
        // destroy RunSet manager
        destroyRunSetManager ();
        
        // destroy containers
        try
        {
            ServiceDefContainer_impl::ServiceDefContainer::
                destroy (m_actualServiceDefContainer);
        }
        catch (...)
        {
        }
        
        
        if (m_stateMachine != 0)
        {
            delete m_stateMachine;
        }
        
        if (m_rwServiceDefContainer != 0)
        {
            // remove reference before deletion
            m_rwServiceDefContainer->_remove_ref ();
        }
        
        if (m_actualEntityContainer != 0)
        {
            delete m_actualEntityContainer;
        }
        
        if (m_rwEntityContainer != 0)
        {
            // remove reference before deletion
            m_rwEntityContainer->_remove_ref ();
        }
        
        if (m_actualHostContainer != 0)
        {
            delete m_actualHostContainer;
        }
        
        if (m_sysStatusChangeFactory != 0)
        {
            m_sysStatusChangeFactory->_remove_ref ();
        }
        
        if (m_sysModeChangeFactory != 0)
        {
            m_sysModeChangeFactory->_remove_ref ();
        }
        
        if (m_hostStatusChangeFactory != 0)
        {
            m_hostStatusChangeFactory->_remove_ref ();
        }
        
        if (m_sysRestartFactory != 0)
        {
            m_sysRestartFactory->_remove_ref ();
        }
        
        if (m_sysStopFactory != 0)
        {
            m_sysStopFactory->_remove_ref ();
        }
        
        if (m_sysAcknowledgeFactory != 0)
        {
            m_sysAcknowledgeFactory->_remove_ref ();
        }
        
        if (m_consistent_synchro_solver != 0)
        {
            delete m_consistent_synchro_solver;
        }
    }
   

#ifdef CDMW_USE_FAULTTOLERANCE   
    
    ConsistentSynchroSolver* System_impl::get_consistent_synchro_solver ()
    {
    return m_consistent_synchro_solver;
    }
    
    
    /**
    *Purpose:
    *<p> to set periodic notifier.
    *@param periodic_notifier  the periodic notifier reference.
    */
    void System_impl::set_periodic_notifier( CdmwPlatformMngt::Periodic::Notifier_ptr periodic_notifier)
    {
        m_periodicNotifierObject = CdmwPlatformMngt::Periodic::Notifier::_duplicate(periodic_notifier);  
    }

    
    /**
     *Purpose:
     *<p>
     * Implements the
     * IDL:thalesgroup.com/CdmwPlatformMngt/Periodic/Notifier:1.0
     * operation
     */
    CdmwPlatformMngt::Periodic::Notifier_ptr System_impl::periodic_notifier() throw (CORBA::SystemException)
    {
        CdmwPlatformMngt::Periodic::Notifier_var periodic_notifier = m_periodicNotifierObject;
        
        return periodic_notifier._retn ();
    }
    
#endif
   
#ifndef CDMW_USE_FAULTTOLERANCE 
    /**
     *Purpose:
     *<p>
     * Implements the
     * IDL:thalesgroup.com/CdmwPlatformMngt/Periodic/Notifier:1.0
     * operation
     */
    CdmwPlatformMngt::Periodic::Notifier_ptr System_impl::periodic_notifier() throw (CORBA::SystemException)
    {    
        // not implement for default configuration
        throw CORBA::NO_IMPLEMENT (OrbSupport::NO_IMPLEMENT,
            CORBA::COMPLETED_NO);
    }   
#endif
    
    
    void System_impl::remove()
    {
        Cdmw::PlatformMngt::SystemIssuerManager::Instance()->detach_element();
        // remove associated process record in datastore
        remove_system_record_in_datastore();
        
        // remove the associated managed element
        this->ManagedElement_impl::remove();    
    }
    
    
    void System_impl::destroyInitSequencer ()
    {
        // Stop and destroy the init sequencer 
        if (m_initSequencer != 0)
        {
            m_initSequencer->stop ();
            
            delete m_initSequencer;
            m_initSequencer = 0;
        }
        
        if (m_initSyncTaskFactory != 0)
        {
            delete m_initSyncTaskFactory;
            m_initSyncTaskFactory = 0;
        }
    }
    
    void System_impl::destroyStopSequencer ()
    {
        // Stop and destroy the stop sequencer 
        if (m_stopSequencer != 0)
        {
            m_stopSequencer->stop ();
            
            delete m_stopSequencer;
            m_stopSequencer = 0;
        }
        
        if (m_stopSyncTaskFactory != 0)
        {
            delete m_stopSyncTaskFactory;
            m_stopSyncTaskFactory = 0;
        }
    }
    
    
    void System_impl::destroyRunSetManager ()
    {
        if (m_runSetManager != 0)
        {
            // stop RunSetManager starters
            m_runSetManager->stopAllRunSets ();
            
            // stop thread
            m_runSetManager->stop ();
            
            delete m_runSetManager;
            m_runSetManager = 0;
        }
    }
    
    
    void System_impl::createStates () throw (OutOfMemoryException)
    {
        try
        {
            std::auto_ptr < SystemInitial > initial (new SystemInitial ());
            std::auto_ptr < SystemStarting > starting (new SystemStarting ());
            std::auto_ptr < SystemStartupSuspended >
                startupSuspended (new SystemStartupSuspended ());
            std::auto_ptr < SystemStarted > started (new SystemStarted ());
            std::auto_ptr < SystemStopping > stopping (new SystemStopping ());
            std::auto_ptr < SystemStopped > stopped (new SystemStopped ());
            std::auto_ptr < SystemPartialStarted > partialStarted (new SystemPartialStarted ());
            
            
            
            if (M_initial == 0)
            {
                M_initial = initial.get ();
            }
            if (M_starting == 0)
            {
                M_starting = starting.get ();
            }
            if (M_startupSuspended == 0)
            {
                M_startupSuspended = startupSuspended.get ();
            }
            if (M_started == 0)
            {
                M_started = started.get ();
            }
            if (M_stopping == 0)
            {
                M_stopping = stopping.get ();
            }
            if (M_stopped == 0)
            {
                M_stopped = stopped.get ();
            }
            if (M_partialStarted == 0)
            {
                M_partialStarted = partialStarted.get ();
            }
            
            initial.release ();
            starting.release ();
            startupSuspended.release ();
            started.release ();
            stopping.release ();
            stopped.release ();
            partialStarted.release ();
        }
        catch (const std::bad_alloc &)
        {
            CDMW_THROW (OutOfMemoryException);
        }
    }
    
    void System_impl::destroyStates ()
    {
        if (M_initial != 0)
        {
            delete M_initial;
            M_initial = 0;
        }
        
        if (M_starting != 0)
        {
            delete M_starting;
            M_starting = 0;
        }
        
        if (M_startupSuspended != 0)
        {
            delete M_startupSuspended;
            M_startupSuspended = 0;
        }
        
        if (M_started != 0)
        {
            delete M_started;
            M_started = 0;
        }
        
        if (M_stopping != 0)
        {
            delete M_stopping;
            M_stopping = 0;
        }
        
        if (M_stopped != 0)
        {
            delete M_stopped;
            M_stopped = 0;
        }

        if (M_partialStarted != 0)
        {
            delete M_partialStarted;
            M_partialStarted = 0;
        }
        
    }
    
    
    HostContainer_impl *System_impl::get_host_container_impl ()
    {
        return m_hostContainer;
    }
    
    PlatformMngt::HostContainer * System_impl::get_actual_host_container ()
    {
        return m_actualHostContainer;
    }
    
    const CdmwPlatformMngtService::ServiceBinding_ptr System_impl::
        get_service_binding ()
    {
        return m_serviceBindingObject.in ();
    }
    
    
    
    #ifdef CDMW_USE_FAULTTOLERANCE
    
    void System_impl::set_replication_mngr_iogr (const std::
        string &
        replication_mngr_iogr)
    {
        m_replicationManagerIOGR = replication_mngr_iogr;
    }
    
    void System_impl::set_replication_mngr (CdmwFT::
        ReplicationManager_ptr
        replication_mngr)
    {
        m_replicationMngt =
            CdmwFT::ReplicationManager::_duplicate (replication_mngr);
    }
    
    void System_impl::set_ft_converter_object (CdmwPlatformMngt::
        FTConverter_ptr
        ft_converter_obj)
    {
        m_FTConverterObject =
            CdmwPlatformMngt::FTConverter::_duplicate (ft_converter_obj);
    }
    
    void System_impl::set_ft_process_observer_group (CdmwPlatformMngt::
        ProcessObserver_ptr
        ft_process_observer_group)
    {
        m_FTProcessObserverGroup =
            CdmwPlatformMngt::ProcessObserver::
            _duplicate (ft_process_observer_group);
    }
    
    void System_impl::set_ft_service_broker_group (CdmwPlatformMngtService::
        ServiceBroker_ptr
        ft_service_broker_group)
    {
        m_FTServiceBrokerGroup =
            CdmwPlatformMngtService::ServiceBroker::
            _duplicate (ft_service_broker_group);
    }
    
    
    #endif
    
    void System_impl::notifyStatusChangeEvent ()
    {
        try
        {
            // Note : lock must be set by state machine caller
            
            // create header
            CdmwPlatformMngtBase::EventHeader header =
                EventHeaderFactory::createHeader (CdmwPlatformMngtBase::INF);
            
            // get status
            CdmwPlatformMngt::SystemStatus systemStatus =
                m_stateMachine->getState ()->status ();
            
            // create event value type using its factory        
            CdmwPlatformMngt::SystemStatusChange_var event =
                m_sysStatusChangeFactory->create (header, m_element_name.in (),
                        systemStatus, get_status_info());
            
            // Trace to the log manager (must be done before addEvent)
            CORBA::String_var eventStr = event->to_string ();
            LogMngr::logMessage (eventStr.in ());
            event->_add_ref();
            // Perform the status change notification
            m_eventNotifier->addEvent (event);

            // Perform the status change notification to observer registered
            // with the Api in ManagedElement
            event->_add_ref();
            get_status_event_notifier()->add_event (event);
            event->_remove_ref();

            // release ownership if no error
            event._retn ();
            
        }
        catch (const std::bad_alloc &)
        {
            throw CORBA::NO_MEMORY (OrbSupport::NO_MEMORY, CORBA::COMPLETED_NO);
        }
        catch (...)
        {
            throw CORBA::INTERNAL (OrbSupport::INTERNAL, CORBA::COMPLETED_NO);
        }
    }
    
    void System_impl::notifyModeChangeEvent (CdmwPlatformMngt::
        FunctioningMode functioning_mode,
        const char *mode_info)
    {
        try
        {
            // Note : lock on mode must be set by caller
            
            // create header
            CdmwPlatformMngtBase::EventHeader header =
                EventHeaderFactory::createHeader (CdmwPlatformMngtBase::INF);
            
            // create event value type using its factory
            CdmwPlatformMngt::SystemModeChange_var event =
                m_sysModeChangeFactory->create (header, m_element_name.in (),
                functioning_mode, mode_info);
            
            // Trace to the log manager (must be done before addEvent)
            CORBA::String_var eventStr = event->to_string ();
            LogMngr::logMessage (eventStr.in ());

            event->_add_ref();
            // Perform the status change notification
            m_eventNotifier->addEvent (event);

            // Perform the status change notification to observer registered
            // with the Api in ManagedElement
            event->_add_ref();
            get_status_event_notifier()->add_event (event);
            event->_remove_ref();
 
            // release owner ship if no error
            event._retn ();
            
        }
        catch (const std::bad_alloc &)
        {
            throw CORBA::NO_MEMORY (OrbSupport::NO_MEMORY, CORBA::COMPLETED_NO);
        }
        catch (...)
        {
            throw CORBA::INTERNAL (OrbSupport::INTERNAL, CORBA::COMPLETED_NO);
        }
    }
    
    
    void System_impl::notifyHostStatusChangeEvent (const char *host_name,
        CdmwPlatformMngt::
        HostStatus host_status,
        CdmwPlatformMngtBase::
        TimeStamp time_stamp)
    {
        try
        {
            // create header
            CdmwPlatformMngtBase::EventHeader header =
                EventHeaderFactory::createHeader (time_stamp,
                CdmwPlatformMngtBase::WRN);
            
            // create event value type using its factory
            CdmwPlatformMngt::HostStatusChange_var event =
                m_hostStatusChangeFactory->create (header, m_element_name.in (),
                host_name, host_status);
            
            // Trace to the log manager (must be done before addEvent)
            CORBA::String_var eventStr = event->to_string ();
            LogMngr::logMessage (eventStr.in ());
            
            // Perform the status change notification
            m_eventNotifier->addEvent (event);
            
            // release owner ship if no error
            event._retn ();
            
        }
        catch (const std::bad_alloc &)
        {
            throw CORBA::NO_MEMORY (OrbSupport::NO_MEMORY, CORBA::COMPLETED_NO);
        }
        catch (...)
        {
            throw CORBA::INTERNAL (OrbSupport::INTERNAL, CORBA::COMPLETED_NO);
        }
    }
    
    void System_impl::defineSystem (const CdmwPlatformMngt::SystemDef& system_def)
        throw (CdmwPlatformMngt::DuplicateEntity,
                CosPropertyService::MultipleExceptions,
                CdmwPlatformMngt::DuplicateService,
                CdmwPlatformMngt::DuplicateHost,
                CdmwPlatformMngt::DuplicateHostEntity,
                CdmwPlatformMngt::HostPropertyExceptions,
                CdmwPlatformMngt::AlreadyDone, CORBA::SystemException)
    {
        //CDMW_WRITER_LOCK_GUARD (m_rwSystemLock);

       if (!m_systemDefined)
        {
            // if a system observer is registered *before* the system is defined
            // we must unregister it and register it again later using the new system path
            std::vector<ObserverInfo> obs_vec = 
                m_status_event_notifier->get_observers(""); // Undefined system has an empty name
            std::vector<ObserverInfo>::iterator it;
            for (it = obs_vec.begin(); it != obs_vec.end(); ++it) {
                try {
                    unregister_observer_ext((*it).observer_name.c_str());
                }
                catch (const CdmwPlatformMngt::ObserverNotFound& ex)
                {
                    CDMW_ERROR(Cdmw::Logging::Logger::Get_Logger(CdmwPlatformMngtBase::PFMMNG),
                       " error unregistering observer " << (*it).observer_name << 
                       " that was previously registered on an undefined system" );
                }
                catch (const CORBA::SystemException& ex)
                {
                    CDMW_ERROR(Cdmw::Logging::Logger::Get_Logger(CdmwPlatformMngtBase::PFMMNG),
                       " error re-registering observer " << (*it).observer_name << 
                       " that was previously registered on an undefined system" );
                    throw;
                }
            }
           // build the system path        
            std::string system_path = ManagedElementKey::Get_system_path (system_def.system_name.in ());
            
            // store element path in managed element
            set_element_path (system_path.c_str(), system_def.system_name.in());
            Cdmw::PlatformMngt::SystemIssuerManager::Instance()->attach_element(get_element_path());
                  
            // now we register again all the pre-existing observers
            for (it = obs_vec.begin(); it != obs_vec.end(); ++it) {
                try {
                    register_observer_ext((*it).observer_name.c_str(),CdmwPlatformMngt::OBJECT_ONLY, (*it).observer_ptr);
                }
                catch (const CdmwPlatformMngt::ElementAlreadyExists& ex)
                {
                    CDMW_ERROR(Cdmw::Logging::Logger::Get_Logger(CdmwPlatformMngtBase::PFMMNG),
                       " error registering observer " << (*it).observer_name << 
                       " that was previously registered on an undefined system" );
                }
                catch (const CORBA::SystemException& ex)
                {
                    CDMW_ERROR(Cdmw::Logging::Logger::Get_Logger(CdmwPlatformMngtBase::PFMMNG),
                       " error re-registering observer " << (*it).observer_name << 
                       " that was previously registered on an undefined system" );
                    throw;
                }
            }   
                
            CDMW_WRITER_LOCK_GUARD (m_rwSystemLock);
           try
            {
                // add in repository the system object
                m_element_repository->add_element (system_path.c_str(), CdmwPlatformMngt::SYSTEM_TYPE, this->_this());
            }
            catch (...)
            {
                throw CORBA::INTERNAL (OrbSupport::INTERNAL, CORBA::COMPLETED_NO);
            }
            
            // set element path in rwservicedef container
            m_rwServiceDefContainer->set_container_path(system_path.c_str ());
            
            // get system port
            m_systemPort = system_def.system_port;
            
            // Store entity initial values into default_entities
            m_default_entitydefs = new CdmwPlatformMngt::EntityDefs ();
            unsigned int entityCount = system_def.entities.length ();
            m_default_entitydefs->length (entityCount);
            // initialise data store of the object
            set_system_record_in_datastore ();
            // register the parent path in entity container            
            m_actualEntityContainer->setParentPath (m_element_path.in ());
            
            // Add all the entities
            entityCount = system_def.entities.length ();

            for (unsigned int entityIndex = 0; entityIndex < entityCount; entityIndex++)
            {
                // get entity definition (EntityDef is a structure, a reference on the
                // structure is returned by the [] operator)
                const CdmwPlatformMngt::EntityDef& entity_def = system_def.entities[entityIndex];
                
                // store values of the entityDef
                (*m_default_entitydefs)[entityIndex] = entity_def;
                
                try
                {
                    m_rwEntityContainer->add_entity (entity_def);
                }
                catch (const CdmwPlatformMngt::EntityAlreadyExists &)
                {
                    throw CdmwPlatformMngt::DuplicateEntity (entity_def.entity_name.in ());
                }
            }

            //Create predefined entity for system synchronization status
            //Status is updated by the CentralisedSupervsionObserver
            CdmwPlatformMngt::EntityDef_var synchro_entity_def =
                    new CdmwPlatformMngt::EntityDef();
    
            // Get the entity's name
             synchro_entity_def->entity_name = CORBA::string_dup("CdmwSystemSynchronization");
             synchro_entity_def->status_values.length(0);
             synchro_entity_def->properties.length(0);
        
            try
            {
                m_system_synchro_entity = m_rwEntityContainer->add_entity(synchro_entity_def.in());
            }
            catch (const CdmwPlatformMngt::EntityAlreadyExists &)
            {
                 throw CdmwPlatformMngt::DuplicateEntity (synchro_entity_def->entity_name.in ());
            }
  
             //set_system_record_in_datastore ();           
   
                     
            // Add all the services
            unsigned int serviceCount = system_def.service_definitions.length ();
            
            for (unsigned int serviceIndex = 0; serviceIndex < serviceCount; serviceIndex++)
            {
                // get service definition (ServiceDef is a structure, a reference on the
                // structure is returned by the [] operator)
                const CdmwPlatformMngtService::ServiceDef& service_def =
                    system_def.service_definitions[serviceIndex];
                
                try
                {
                    m_rwServiceDefContainer->add_service_def (service_def);
                }
                catch (const CdmwPlatformMngtService::ServiceAlreadyExists &)
                {
                    throw CdmwPlatformMngt::DuplicateService (service_def.service_name.in ());
                }
            }

            // add the hosts
            unsigned int hostCount = system_def.hosts.length ();
            
            // Check the validity of all the host definitions
            for (unsigned int hostIndex = 0; hostIndex < hostCount; hostIndex++)
            {
                // get host definition (HostDef is a structure, a reference on the
                // structure is returned by the [] operator)
                const CdmwPlatformMngt::HostDef& host_def = system_def.hosts[hostIndex];
                
                // Check there is no duplicate entity            
                for (unsigned int entityIndex = 0; entityIndex < host_def.entities.length(); entityIndex++)
                {
                    std::string entityName = host_def.entities[entityIndex].entity_name.in ();
                    
                    for (unsigned int entityIndex2 = entityIndex + 1; 
                            entityIndex2 < host_def.entities.length (); entityIndex2++)
                    {
                        std::string entityName2 = host_def.entities[entityIndex2].entity_name.in ();
                        if (entityName == entityName2)
                        {
                            throw CdmwPlatformMngt::DuplicateHostEntity (host_def.host_name.in(), entityName.c_str());
                        }
                    }
                }
            }

            // All has been checked - add hosts
            for (unsigned int hostIndex = 0; hostIndex < hostCount; hostIndex++)
            {
                // get host definition (HostDef is a structure, a reference on the
                // structure is returned by the [] operator)
                const CdmwPlatformMngt::HostDef& host_def = system_def.hosts[hostIndex];

                try
                {
                    // add host in container
                    CdmwPlatformMngt::Host_var host = addHost (host_def);
                }
                catch (const CdmwPlatformMngt::HostAlreadyExists &)
                {
                    throw CdmwPlatformMngt::DuplicateHost (host_def.host_name.in());
                }
            }
            
            //add the properties to System
            unsigned int propCount = system_def.properties.length ();
            
            for (unsigned int propIndex = 0; propIndex < propCount; propIndex++)
            {
                const CdmwPlatformMngt::PropertyDef & prop_def = system_def.properties[propIndex];
            
                std::string propertyName (system_def.properties[propIndex].property_name);
                CosPropertyService::PropertyModeType propertyMode =
                    CosPropertyService::PropertyModeType(system_def.properties[propIndex].mode_type);
                CORBA::Any any_val;
                try
                {
                    define_property_with_mode (propertyName.c_str (), prop_def.initial_value, propertyMode);
                }
                catch (...)
                {
                    throw CosPropertyService::MultipleExceptions ();
                }                
            }

            // MdR define rw properties
            try
            {
                std::ostringstream property_name;
                property_name << CdmwPlatformMngt::CDMW_PREFIX << CdmwPlatformMngt::DEFAULT_RESTART_WARMTH << std::ends;
                if(!is_property_defined(property_name.str().c_str()))
                {
                    CORBA::Any_var any = new CORBA::Any();
                    *any <<= CdmwPlatformMngt::HOT_RESTART;
                    define_property_with_mode(property_name.str().c_str(), any.in(), CosPropertyService::fixed_normal);
                }
            }
            catch(...)
            {
               throw CosPropertyService::MultipleExceptions ();
            }

        // MdR define system properties not user defined
        std::ostringstream property_name;
        CORBA::Any_var any_val = new CORBA::Any();
        try
        {
         
            get_attribute(CdmwPlatformMngt::DEFAULT_RESTART_WARMTH, any_val.out(), CdmwPlatformMngt::PRE_DEFINED);                                     
            property_name.str("");
            property_name << CdmwPlatformMngt::CDMW_PREFIX << CdmwPlatformMngt::REQUESTED_RESTART_WARMTH << std::ends;
            define_property_with_mode(property_name.str().c_str(), any_val.in(), CosPropertyService::fixed_normal);
        }
        catch (...)
        {
                throw CosPropertyService::MultipleExceptions ();
        }

        // store default properties values
        store_default_properties ();

         // set system is defined
         m_systemDefined = true;

        }
        else
        {
            throw CdmwPlatformMngt::AlreadyDone ();
        }
    }
    
    void System_impl::set_system_record_in_datastore ()
    {
        // test if the supervision is not fault tolerant or if it is, is it the primary?
        if (!Configuration::Is_fault_tolerant ()
            || Configuration::Is_primary ())
        {
            CdmwPlatformMngt::SystemRecord record;
            record.system_port = m_systemPort;
            record.status = m_stateMachine->getState ()->status ();
            record.status_info = get_status_info();
            record.mode = m_mode;
            record.mode_info = m_modeInfo;
            record.state_machine =
                CORBA::string_dup (m_stateMachine->getStateName ());
            record.startup_kind = m_startup_kind.in ();
            record.stop_emergency = m_emergency;
            record.new_graph = m_newGraph;
            record.application_init_graph = m_applicationInitGraph;
            record.application_stop_graph = m_applicationStopGraph;
            record.stop_graph = m_stopGraph;
            record.configuration.restart_process_on_responding = Configuration::Get_restart_process_on_responding_option();

            std::string ds_operation("contains");
                    
            try
            {
                if (m_system_ds->contains (m_element_path.in ()))
                {
                    ds_operation = "update";
                    m_system_ds->update (m_element_path.in (), record);
                }
                else
                {
                    set_element_in_datastore ();
                    ds_operation = "insert";
                    m_system_ds->insert (m_element_path.in (), record);
                }
            }
            catch (const Common::AlreadyExistException &)
            {
                string tmp ("system record already exists in datastore : ");
                tmp += m_element_path.in ();
                tmp += " operation : ";
                tmp += ds_operation;
                tmp += ":";
                tmp += record.status;
                LogMngr::logMessage(WRN, tmp.c_str() );
            }
            catch (const CommonSvcs::DataStore::StorageErrorException &exc_storage)
            {
                string tmp ("system record storage error exception in insertion : ");
                tmp += m_element_path.in ();
                tmp += " operation : ";
                tmp += ds_operation;
                tmp += ":";
                tmp += record.status;
                LogMngr::logMessage(FTL, tmp.c_str() );
                CDMW_ASSERT(false);
            }
            catch (const Common::NotFoundException&)
            {
                string tmp ("record cannot be found in datastore in ");
                tmp += m_element_path.in ();
                tmp += " operation : ";
                tmp += ds_operation;
                LogMngr::logMessage(FTL, tmp.c_str() ); 
                CDMW_ASSERT (false);
            }
            catch (...)
            {
                string tmp ("system record unexpected exception in insertion :");
                tmp += m_element_path.in ();
                tmp += " operation : ";
                tmp += ds_operation;
                tmp += ":";
                tmp += record.status;
                LogMngr::logMessage(FTL, tmp.c_str() );
                CDMW_ASSERT(false);
            }
        }
    }
    
    
    
    
    
    void System_impl::remove_system_record_in_datastore ()
    {
        // inhibition if backup
        if (!Configuration::Is_fault_tolerant ()
            || Configuration::Is_primary ())
        {
            try
            {
                m_system_ds->remove (m_element_path.in ());
            }
            catch (const Common::NotFoundException &)
            {
            }
            catch (const CommonSvcs::DataStore::StorageErrorException &)
            {
            }
        }
    }
    
    
    CdmwPlatformMngt::SystemRecord* System_impl::get_system_record_from_datastore()
        throw (Common::NotFoundException)
    {
        CdmwPlatformMngt::SystemRecord* system_record = NULL;

      try
      {
    system_record = m_system_ds->select (m_element_path.in ());
      }
      catch (const Common::NotFoundException &)
      {
        
            LogMngr::logMessage(ERR, "System record cannot be found in datastore");
            throw;
      }
      catch (...)
      {
        LogMngr::logMessage(ERR, "Unexpected Exception raised: system record cannot be found in datastore : " );
        CDMW_ASSERT (false);
      }

        return system_record;
    }
    
    
    void System_impl::
        set_observer_record_in_datastore (const char *observer_name,
        const char *host_name,
        CdmwPlatformMngt::
        SupervisionObserver_ptr observer)
    {
        if (Configuration::Is_primary ())
        {
            CdmwPlatformMngt::SystemObserverRegistrationRecord record;
            record.host_name = CORBA::string_dup (host_name);
            record.observer =
            CdmwPlatformMngt::SupervisionObserver::_duplicate (observer);
            std::string ds_operation("contains");
            try
            {
                if (m_system_observer_ds->contains (observer_name))
                {
                    ds_operation = "update";
                    m_system_observer_ds->update (observer_name, record);
                }
                else
                {
                    ds_operation = "insert";
                    m_system_observer_ds->insert (observer_name, record);
                }
            }
            
            catch (const Common::AlreadyExistException &)
            {
                string tmp ("system record already exists in datastore : ");
                tmp += m_element_path.in ();
                tmp += " operation : ";
                tmp += ds_operation;
                tmp += ":";
                tmp += observer_name;
                tmp += record.host_name;
                LogMngr::logMessage(WRN, tmp.c_str() );
            }
            catch (const CommonSvcs::DataStore::StorageErrorException &exc_storage)
            {
                string tmp ("system record storage error exception in insertion : ");
                tmp += m_element_path.in ();
                tmp += " operation : ";
                tmp += ds_operation;
                tmp += ":";
                tmp += observer_name;
                tmp += record.host_name;
                LogMngr::logMessage(FTL, tmp.c_str() );
                CDMW_ASSERT(false);
            }
            catch (const Common::NotFoundException&)
            {
                string tmp ("record cannot be found in datastore in ");
                tmp += m_element_path.in ();
                tmp += " operation : ";
                tmp += ds_operation;
                LogMngr::logMessage(FTL, tmp.c_str() ); 
                CDMW_ASSERT (false);
            }
            catch (...)
            {
                string tmp ("system record unexpected exception in insertion :");
                tmp += m_element_path.in ();
                tmp += " operation : ";
                tmp += ds_operation;
                tmp += ":";
                tmp += observer_name;
                tmp += record.host_name;
                LogMngr::logMessage(FTL, tmp.c_str() );
                CDMW_ASSERT(false);
            }
        }
    }
    
    
    void System_impl::
        remove_observer_record_in_datastore (const char *observer_name)
    {
        // inhibition if backup
        if (Configuration::Is_primary ())
        {
            try
            {
                m_system_observer_ds->remove (observer_name);
            }
            catch (const Common::NotFoundException &)
            {
            }
            catch (const CommonSvcs::DataStore::StorageErrorException &)
            {
            }
        }
    }
    
    
    void System_impl::synchro_with_datastore ()
    {
        // synchronise with system record
        try
        {
            if (m_systemDefined){   

                Cdmw::PlatformMngt::ManagedElement_impl::synchro_with_datastore ();
  
                CdmwPlatformMngt::SystemRecord_var system_record = get_system_record_from_datastore ();    
            
                set_status_info ( system_record->status_info);
                m_mode = system_record->mode;
                m_modeInfo = system_record->mode_info;
            
                m_stateMachine->setStateName (system_record->state_machine.in ());
                m_startup_kind = new CdmwPlatformMngtBase::StartupKind (system_record->startup_kind);
            
                m_emergency = system_record->stop_emergency;
                m_newGraph = true;
            
                m_applicationInitGraph = new CdmwPlatformMngt::GraphElements (system_record->
                application_init_graph);
            
                m_applicationStopGraph = new CdmwPlatformMngt::GraphElements (system_record->
                application_stop_graph);
            
                m_stopGraph = new CdmwPlatformMngt::GraphElements (system_record->stop_graph);

                // synchronise with the system observer records
                // to register the reference of observers
                m_system_observer_ds->for_each (ObserverDumper (this));
         Cdmw::PlatformMngt::Configuration::Set_restart_process_on_responding_option(system_record->configuration.restart_process_on_responding); 
                
            }
            else { CDMW_INFO(Cdmw::Logging::Logger::Get_Logger(CdmwPlatformMngtBase::PFMMNG),
                  "<<System_impl::synchro_with_datastore() skipped because system undefined>>");
            }
        }
        catch (const Common::NotFoundException &)
        {
            // Element not found, This append when the SystemManagement is the first primary
        }
    }
    
    void System_impl::synchro_processing ()
    {
        // if status Starting, Stopping restart the activity
        CdmwPlatformMngt::SystemStatus systemStatus =
            m_stateMachine->getState ()->status ();
        
        if (systemStatus == CdmwPlatformMngt::SYSTEM_STARTING ||
            systemStatus == CdmwPlatformMngt::SYSTEM_STOPPING)
        {
            
            m_stateMachine->restartStateActivity ();
            
            // update data store of the object
            set_system_record_in_datastore ();
        }
    }
    
    
    void System_impl::setInitGraph (const CdmwPlatformMngt::
                    GraphElements & application_graph)
                    throw (OutOfMemoryException, CdmwPlatformMngt::ReferenceNotFound,
                    CdmwPlatformMngt::CircularReference, CORBA::SystemException)
    {
        try
        {
            CDMW_WRITER_LOCK_GUARD (m_rwSystemLock);
            
            // Check the validity of the init graph of applications
            
            // fill existing element map with application name (step is not used)
            ExistingElementMap existingElementMap;
            Applications::iterator applicationIt;
            
            {
                for (applicationIt = m_applications.begin ();
                applicationIt != m_applications.end (); applicationIt++)
                {
                    // insert in map the application name
                    existingElementMap.
                        insert (PlatformMngt::ExistingElementMap::
                        value_type (applicationIt->first, 0));
                }
            }
            // set the application dynamic creation to false
            std::vector<Application_impl*> vect_app = get_application_impls();
            for (unsigned int i=0; i< vect_app.size(); i++)
            {
                vect_app[i]->set_created_dynamically(false);
            }
 
            GraphChecker applicationGraph (application_graph, &existingElementMap,
                true);
            
            // Actually sets the graph of applications to this system
            // (the previous init graph is deleted by var mechanism)
            m_applicationInitGraph =
                new CdmwPlatformMngt::GraphElements (application_graph);
            
            // set flag new graph
            m_newGraph = true;
            
            // TODO: Notify the configuration change
            
            // TODO: Trace to the log manager
            
            // update record in datastore
            set_system_record_in_datastore ();
        }
        catch (const CdmwPlatformMngt::StepOutOfBound &)
        {
            // nothing to do
        }
        catch (const CdmwPlatformMngt::StepJump &)
        {
            // nothing to do
        }
        catch (const std::bad_alloc &)
        {
            throw CORBA::NO_MEMORY (OrbSupport::NO_MEMORY, CORBA::COMPLETED_NO);
        }
    }
    
    
    void System_impl::setStopGraph (const CdmwPlatformMngt::
                    GraphElements & application_graph)
                    throw (OutOfMemoryException, CdmwPlatformMngt::ReferenceNotFound,
                    CdmwPlatformMngt::CircularReference, CORBA::SystemException)
    {
        try
        {
            CDMW_WRITER_LOCK_GUARD (m_rwSystemLock);
            
            // Check the validity of the stop graph of applications
            
            // fill existing element map with application name (step is not used)
            ExistingElementMap existingElementMap;
            Applications::iterator applicationIt;
            
            {
                for (applicationIt = m_applications.begin ();
                applicationIt != m_applications.end (); applicationIt++)
                {
                    // insert in map the application name
                    existingElementMap.
                        insert (PlatformMngt::ExistingElementMap::
                        value_type (applicationIt->first, 0));
                    
                }
            }
            
            GraphChecker applicationGraph (application_graph, &existingElementMap,
                true);
            
            // Actually sets the graph of applications to this system
            // (the previous stop graph is deleted by var mechanism)
            m_applicationStopGraph =
                new CdmwPlatformMngt::GraphElements (application_graph);
            
            // TODO: Notify the configuration change
            
            // TODO: Trace to the log manager
            
            // update record in datastore
            set_system_record_in_datastore ();
        }
        catch (const CdmwPlatformMngt::StepOutOfBound &)
        {
            // nothing to do
        }
        catch (const CdmwPlatformMngt::StepJump &)
        {
            // nothing to do
        }
        catch (const std::bad_alloc &)
        {
            throw CORBA::NO_MEMORY (OrbSupport::NO_MEMORY, CORBA::COMPLETED_NO);
        }
    }
    
    
    void System_impl::setRunSet (const CdmwPlatformMngt::
        RunSetDef & run_set_def)
        throw (CdmwPlatformMngt::DuplicateRunSetElement, CORBA::SystemException)
    {
        try
        {
            m_runSetManager->setRunSet (run_set_def);
        }
        catch (const Cdmw::OutOfResourcesException &)
        {
            throw CORBA::NO_MEMORY (OrbSupport::NO_MEMORY, CORBA::COMPLETED_NO);
        }
    }
    
    
    void System_impl::removeRunSet (const char *run_set_name)
        throw (CdmwPlatformMngt::RunSetNotFound, CORBA::SystemException)
    {
        m_runSetManager->removeRunSet (run_set_name);
    }
    
    
    void System_impl::removeAllRunSets ()
    {
        m_runSetManager->removeAllRunSets ();
    }
    
    
    CdmwPlatformMngt::Host_ptr
        System_impl::addHost (const CdmwPlatformMngt::HostDef & host_def)
        throw (CdmwPlatformMngt::HostAlreadyExists,
        CdmwPlatformMngt::DuplicateEntity,
        CosPropertyService::MultipleExceptions, CORBA::SystemException)
    {
        // get host name
        std::string host_name = host_def.host_name.in();

        // set the host name in DeploymentManager to recover the NodeManager on that host
        //DeploymentManager::get_instance()->set_node_manager_host_name(host_name);

        // add host in container
        return m_actualHostContainer->addHost (host_def);
    }
    
    
    void System_impl::removeHost (const char *host_name)
        throw (CdmwPlatformMngt::HostNotFound, CORBA::SystemException)
    {
        // remove host from container
        m_actualHostContainer->removeHost (host_name);
    }
    
    Host_impl *System_impl::getHost (const char *host_name)
        throw (CdmwPlatformMngt::HostNotFound, CORBA::SystemException)
    {
        return m_actualHostContainer->getHost (host_name);
    }
    
    
    
    CdmwPlatformMngt::Application_ptr
    System_impl::addApplication (const CdmwPlatformMngt::PlatformApplicationDef& platform_application_def,
        const CdmwPlatformMngt::ApplicationDef& application_def)
    throw (CdmwPlatformMngt::ApplicationAlreadyExists,
            CdmwPlatformMngt::HostNotFound,
            CdmwPlatformMngt::DuplicateEntity,
            CosPropertyService::MultipleExceptions,
            CdmwPlatformMngt::DuplicateService,
            CdmwPlatformMngt::DuplicateProcess,
            CdmwPlatformMngt::ProcessHostNotFound,
            CdmwPlatformMngt::DuplicateProcessEntity,
            CdmwPlatformMngt::ProcessPropertyExceptions,
            CdmwPlatformMngt::DuplicateProcessService,
            CdmwPlatformMngt::DuplicateProcessStep,
            CdmwPlatformMngt::IncompatibleStatus, 
            CORBA::SystemException)
    {
        // add application according to centralised/distributed supervision
        return doAddApplication (platform_application_def, application_def);
    }
    
    
    void System_impl::removeApplication (const char *application_name)
        throw (CdmwPlatformMngt::ApplicationNotFound,
        CdmwPlatformMngt::ApplicationStillActive, CORBA::SystemException)
    {
        // remove application according to centralised/distributed supervision
        doRemoveApplication (application_name);
    }
    
    
    
    
    void System_impl::updateApplications (const char *host_name) throw ()
    {
        Applications::iterator applicationIt;
        
        CDMW_READER_LOCK_GUARD (m_rwSystemLock);
        
        // update each application with faulty host name
        for (applicationIt = m_applications.begin ();
        applicationIt != m_applications.end (); applicationIt++)
        {
            try
            {
                ((applicationIt->second).m_applicationControl)->
                    update_application (host_name);
            }
            catch (...)
            {
            }
        }
    }
    
    
    
    CdmwPlatformMngt::ApplicationControl_ptr System_impl::
        get_application_control (const char *application_name)
        throw (CdmwPlatformMngt::ApplicationNotFound)
    {
        CDMW_READER_LOCK_GUARD (m_rwSystemLock);
        
        // search application in map
        Applications::iterator applicationIt =
            m_applications.find (application_name);
        
        // if application not found
        if (applicationIt == m_applications.end ())
        {
            throw CdmwPlatformMngt::ApplicationNotFound ();
        }
        
        // return a duplicated object reference
        // (duplicate is implicit in var copy)
        CdmwPlatformMngt::ApplicationControl_var application =
            (applicationIt->second).m_applicationControl;
        
        return application._retn ();
    }
    
    CdmwPlatformMngt::ProcessObserver_ptr System_impl::
        get_process_observer (const char *application_name)
        throw (CdmwPlatformMngt::ApplicationNotFound)
    {
        CDMW_READER_LOCK_GUARD (m_rwSystemLock);
        
        // search application in map
        Applications::iterator applicationIt =
            m_applications.find (application_name);
        
        // if application not found
        if (applicationIt == m_applications.end ())
        {
            throw CdmwPlatformMngt::ApplicationNotFound ();
        }
        
        // return a duplicated object reference
        // (duplicate is implicit in var copy)
        CdmwPlatformMngt::ProcessObserver_var process_observer =
            (applicationIt->second).m_processObserver;
        
        return process_observer._retn ();
    }
    
    
    void System_impl::degradation_event ()
    {
        CDMW_MUTEX_GUARD (m_mutexMode);
        
        CdmwPlatformMngt::FunctioningMode mode =
            CdmwPlatformMngt::DEGRADED_MODE;
        CORBA::String_var mode_info = CORBA::string_dup ("");
        
        set_mode (mode, mode_info.in ());
        
        // Notify the change of mode
        notifyModeChangeEvent (mode, mode_info.in ());
    }
    
    
    void System_impl::suspension_event ()
    {
        CDMW_SM_WRITER_LOCK_GUARD (*m_stateMachine);
        
        // send event to state machine for transition
        m_stateMachine->getState ()->suspension_event (m_stateMachine);
        
        // update record in datastore
        set_system_record_in_datastore ();
    }
    
    void System_impl::completion_event ()
    {
        CDMW_SM_WRITER_LOCK_GUARD (*m_stateMachine);
        
        // send event to state machine for transition
        m_stateMachine->getState ()->completion_event (m_stateMachine);
        
        // update record in datastore
        set_system_record_in_datastore ();
    }
    
    void System_impl::check_consistency_of_state()
    {

if ((m_stopSequencer!=0 && !m_stopSequencer->is_completed()) || (m_initSequencer!=0 && !m_initSequencer->is_completed()) ){

      return;
}

CORBA::String_var statusInfo;
CdmwPlatformMngt::SystemStatus system_status;
try
{
    system_status=get_status(statusInfo.out());
}
catch (const CORBA::SystemException& e)
{
    LogMngr::logMessage(ERR, "CORBA::SystemException raised: system get_status  " );
}
if (
CdmwPlatformMngt::SYSTEM_PARTIAL_STARTED==system_status || CdmwPlatformMngt::SYSTEM_STOPPED==system_status ||CdmwPlatformMngt::SYSTEM_STARTED==system_status ){ 

         CDMW_SM_WRITER_LOCK_GUARD (*m_stateMachine);
         m_stateMachine->getState ()->check_consistency_of_state (m_stateMachine);
         // update record in datastore
         set_system_record_in_datastore ();
}
    }

    void System_impl::application_status_change_event(const std::string& application_name, 
                                                      CdmwPlatformMngt::ApplicationStatus application_status)
    {    
        CDMW_WRITER_LOCK_GUARD (m_rwSystemLock);
        if (0 == get_application_impl(application_name.c_str())) 
           throw; //never here
        //Application_impl* application_impl = get_application_impl(application_name.c_str());
        if (!get_application_impl(application_name.c_str())->get_indicator_relevant_status()) // replace with just one return
        {    
           return;// replace with no return
        }

        // insert or update the running application in the Set of running applications 
        ApplicationsStatusInfo::iterator it = m_applications_status_info.find(application_name);

        ApplicationsStatusInfo::iterator it_end = m_applications_status_info.end();

        CdmwPlatformMngt::ApplicationStatus old_status = CdmwPlatformMngt::APPLICATION_STOPPED;

        // erase the previous element         
        if (it != it_end)
        {
            old_status = it->second;
            m_applications_status_info.erase(it);
        }
 
        std::ostringstream log_message;
        log_message <<" application_status_change_event " << application_name<< " " << application_status_strings[application_status]<<ends;
        LogMngr::logMessage(INF, MSG_ID_CONSISTENCY_OF_STATUS, log_message.str().c_str());

        // insert the updated element
        m_applications_status_info.insert( ApplicationsStatusInfo::value_type (application_name, application_status));        
                 
        CdmwPlatformMngt::ApplicationStatus current_status = application_status;
                                
        if (check_preemptive_application_status_change (old_status, current_status))
        {
           if (old_status != CdmwPlatformMngt::APPLICATION_RUN_REQUEST)
           {        
               if (m_runSetManager->isInRunSet(application_name))
               {            
                   m_runSetManager->addAvoidRunApplication(application_name);
               }
           }
           
        }

        // call a check on System State Machine to make the application align its state
        if ( CdmwPlatformMngt::APPLICATION_PARTIAL_RUNNING==application_status || CdmwPlatformMngt::APPLICATION_STOPPED==application_status || 
                CdmwPlatformMngt::APPLICATION_RUNNING==application_status ){   
                check_consistency_of_state();
        }
    }

    void System_impl::application_deleted(const std::string& application_name)
    {
        // remove the not running application from the Set of running applications
        ApplicationsStatusInfo::iterator it = m_applications_status_info.find(application_name);
        if ( it!=m_applications_status_info.end() ) 
            m_applications_status_info.erase(it);

        // call a check on System State Machine to make the application align its state
        check_consistency_of_state();
    }

    
    CdmwPlatformMngt::Port System_impl::systemPort ()
    {
        return m_systemPort;
    }
    
    CdmwPlatformMngt::SystemStatus System_impl::get_internal_status ()
    {
        CDMW_SM_READER_LOCK_GUARD (*m_stateMachine);
        
        // get status
        CdmwPlatformMngt::SystemStatus systemStatus =
            m_stateMachine->getState ()->status ();
        
        return systemStatus;
    }
    
    CdmwPlatformMngt::SystemStatus System_impl::property_to_status (const char* status)
    throw (CdmwPlatformMngt::AttributeNotFound)
    {
        CdmwPlatformMngt::SystemStatus sys_status;
        
        bool result = false;
        for ( int i=0 ; i<7; i++)
        {            
            if (strcmp(status, sys_status_strings[i].c_str()) == 0)
            {
                sys_status = Status[i];
                result = true;

                return sys_status;
            }
        }
        if (!result)
        {
            throw CdmwPlatformMngt::AttributeNotFound();
        }
        
        return sys_status;
    }
    
    
    CdmwPlatformMngt::SystemStatus System_impl::get_status (CORBA::
        String_out
        status_info)
    throw (CORBA::SystemException)
    {
        try
        {
            //CDMW_SM_READER_LOCK_GUARD (*m_stateMachine);

            CdmwPlatformMngt::SystemStatus system_status;

            const char* status = "";
            try
            {
                // Get predefined attributes of process
                CORBA::Any_var any_var = new CORBA::Any();           
                get_attribute(CdmwPlatformMngt::TECH_STATUS, any_var.out(), CdmwPlatformMngt::PRE_DEFINED); 
                any_var.in() >>=  CORBA::Any::to_string(status,0);            

        system_status = property_to_status (status);    

        // get info associated to status
    //status_info = CORBA::string_dup (get_status_info());
    char *tmp = get_status_info();
    status_info = CORBA::string_dup (tmp);
    
    //free(tmp);
    delete [] tmp;

        return system_status;
            }
            catch (const CdmwPlatformMngt::AttributeNotFound&)
            {
                throw CORBA::BAD_PARAM(OrbSupport::BAD_PARAM, CORBA::COMPLETED_NO);
            }
            catch (const std::bad_alloc&)
            {
                throw CORBA::NO_MEMORY (OrbSupport::NO_MEMORY, CORBA::COMPLETED_NO);
            }
            catch (...)
            {
                throw CORBA::BAD_PARAM(OrbSupport::BAD_PARAM, CORBA::COMPLETED_NO);
            }

            return system_status;
        }
        catch (const std::bad_alloc &)
        {
            throw CORBA::NO_MEMORY (OrbSupport::NO_MEMORY, CORBA::COMPLETED_NO);
        }
    }

    /**
     *Purpose:
     *<p> facade to get an attribute
     *
     */

    void System_impl::get_attribute (const char *attr_name, CORBA::Any_out ret_attr, CdmwPlatformMngt::Owner flag) 
        throw (CORBA::SystemException, CdmwPlatformMngt::AttributeNotFound)
    {
        std::string tmp1 (attr_name);
        /*active the right methods */
    
        if (flag == CdmwPlatformMngt::PRE_DEFINED)
        { /*case of stardard attribute */
            if (tmp1 == CdmwPlatformMngt::MODE)
            {
                CdmwPlatformMngt::FunctioningMode func_mode;
                CORBA::String_var mode_info;
                CORBA::Any_var tempany = new CORBA::Any;
                        
                func_mode = get_mode (mode_info);
                tempany.inout () <<= func_mode;
                ret_attr = tempany._retn ();
            }
            else if (tmp1 == CdmwPlatformMngt::MODE_INFO)
            {
                CdmwPlatformMngt::FunctioningMode func_mode;
                CORBA::String_var mode_info;
                CORBA::Any_var tempany = new CORBA::Any;
                func_mode = get_mode (mode_info);
                tempany.inout () <<= mode_info;
                ret_attr = tempany._retn ();
            }
            else {
                // Case of predefined property
                std::ostringstream property_name;
                property_name << CdmwPlatformMngt::CDMW_PREFIX << attr_name << std::ends;
                try
                {
                    CORBA::Any_var outany = get_property_value ( property_name.str().c_str() );
                    ret_attr = outany._retn();
                }
                catch (const CosPropertyService::PropertyNotFound&)
                {
                    throw CdmwPlatformMngt::AttributeNotFound ();
                }
            }    
        }
        else
        {/*case of user defined attribute */
            try
            {
                CORBA::Any * outany = get_property_value (attr_name);
                CORBA::Any_var tempany = new CORBA::Any;
                ret_attr = outany;
            }
            catch (const CosPropertyService::PropertyNotFound&)
            {
                throw CdmwPlatformMngt::AttributeNotFound ();
            }
        }
    }

    /**
     *Purpose:
     *<p> facade to set an attribute
     *
     */

    void System_impl::set_attribute (const char *attr_name, const CORBA::Any & attr, CdmwPlatformMngt::Owner flag) 
        throw (CORBA::SystemException, CdmwPlatformMngt::AttributeNotFound , CdmwPlatformMngt::ReadOnlyAttribute )
    {
        if (CdmwPlatformMngt::PRE_DEFINED == flag )
        {           

            std::string tmp1 (attr_name);
            /* check if the predefined property is readonly */
            if ( (CdmwPlatformMngt::REQUESTED_RESTART_WARMTH == tmp1) || 
                 (CdmwPlatformMngt::GLOBAL_SUPERVISION_HOST == tmp1) ||
                 (tmp1 == CdmwPlatformMngt::TECH_STATUS) ||
                 (tmp1 == CdmwPlatformMngt::TECH_STATUS_INFO))
            {
                throw CdmwPlatformMngt::ReadOnlyAttribute(); 
            }
        }

        try
        {
        set_attribute_internal ( attr_name, attr, flag );
        }
    catch (const Cdmw::BadOrderException& ex)
        {
        throw CORBA::BAD_PARAM(OrbSupport::BAD_PARAM, CORBA::COMPLETED_NO);
        }
        catch (...)
    {
        throw;
        }

    }


    /**
     *Purpose:
     *<p> to get master_host
     *
     */

    char* System_impl::get_master_host () throw (CORBA::SystemException)
    {
      CDMW_READER_LOCK_GUARD(m_rwSystemLock);

      // get record
      CdmwPlatformMngt::SystemRecord_var system_record = get_system_record_from_datastore();

      // Use GLOBAL_SUPERVISION_HOST property
      //return (system_record->master_host)._retn() ;
        
      return NULL; //TODO: to be removed
   };


    CdmwPlatformMngt::FunctioningMode System_impl::get_mode (CORBA::String_out mode_info)
                                 throw (CORBA::SystemException)
        {
            try
            {
                CdmwPlatformMngt::FunctioningMode functioning_mode;
                
                if (!m_systemDefined)
                {
                    mode_info = CORBA::string_dup ("");
                    functioning_mode = CdmwPlatformMngt::NORMAL_MODE;
                }
                
                else
                {
                    // get record
                    CdmwPlatformMngt::SystemRecord_var system_record =
                        get_system_record_from_datastore ();
                    
                    // get mode
                    functioning_mode = system_record->mode;
                    
                    // get info associated to mode
                    mode_info = CORBA::string_dup ((system_record->mode_info).in ());
                }
                
                return functioning_mode;
            }
            catch (const std::bad_alloc &)
            {
                throw CORBA::NO_MEMORY (OrbSupport::NO_MEMORY, CORBA::COMPLETED_NO);
            }
        }
        
        
        void System_impl::set_mode (CdmwPlatformMngt::
            FunctioningMode functioning_mode,
            const char *mode_info)
        {
            m_mode = functioning_mode;
            m_modeInfo = CORBA::string_dup (mode_info);
            
            set_system_record_in_datastore ();
        }
        
        
        CdmwPlatformMngt::SystemSnapshot *
            System_impl::get_snapshot ()throw (CORBA::SystemException)
        {
            // Create a system snapshot
            try
            {

                 //MRCDMW_MUTEX_GUARD(m_check_consistency_status_recursive_mutex);

                 //MRCDMW_READER_LOCK_GUARD (m_rwSystemLock);
                
                // Get the snapshot of all hosts
                CdmwPlatformMngt::HostsSnapshot_var hostsSnapshot =
                    m_actualHostContainer->getSnapshot (); 
                
                // Get the snapshot of all applications
                CdmwPlatformMngt::ApplicationsSnapshot_var applicationsSnapshot
                    = new CdmwPlatformMngt::ApplicationsSnapshot ();
                
                applicationsSnapshot->length (m_applications.size ());
                Applications::iterator applicationIt;
                unsigned int i;
                
                for (applicationIt = m_applications.begin (), i = 0;
                applicationIt != m_applications.end (); applicationIt++, i++)
                {
                    (*applicationsSnapshot)[i] =
                        *(((applicationIt->second).m_application)->get_snapshot ());
              
                }
                
                // Get the snapshot of all entities of system
                CdmwPlatformMngt::EntitiesSnapshot_var entitiesSnapshot
                    = new CdmwPlatformMngt::EntitiesSnapshot ();
                
                // Gets all the entities
                CdmwPlatformMngt::Entities_var entities;
                CdmwPlatformMngt::EntitiesIterator_var entitiesIt;
                CORBA::ULong nbr_of_entities;
                
                nbr_of_entities = get_number_of_entities ();
                
                get_all_entities (nbr_of_entities,
                    entities.out (), entitiesIt.out ());
                
                entitiesSnapshot->length (nbr_of_entities);
                for (i = 0; i < nbr_of_entities; i++)
                {
                    (*entitiesSnapshot)[i] = *(entities[i]->get_snapshot ());
                }
                
                //Gets all the properties
                CosPropertyService::PropertiesIterator_var propertiesIt;
                CORBA::ULong nbr_of_properties;
                CdmwPlatformMngt::PropertiesSnapshot_var propertiesSnapshot =
                    new CdmwPlatformMngt::PropertiesSnapshot;
                nbr_of_properties = get_number_of_properties ();
                CosPropertyService::PropertyNames_var prop_names;
                CosPropertyService::PropertyNamesIterator_var prop_namesIt;
                get_all_property_names (nbr_of_properties, prop_names.out (),
                    prop_namesIt.out ());
                propertiesSnapshot->length (nbr_of_properties-2);
                CORBA::ULong snap_prop=0;
                for (i = 0; i < nbr_of_properties; i++)
                {
                    if ((strcmp (prop_names[i], "Cdmw.TechStatus")==0) ||
                            (strcmp (prop_names[i], "Cdmw.TechStatusInfo")==0))
                    {
                        // do not add the TECH_STATUS and TECH_STATUS_INFO properties to the snapshot 
                    }
                    else
                    {
                        CdmwPlatformMngt::PropertySnapshot_var propertySnapshot =
                            new CdmwPlatformMngt::PropertySnapshot ();
                        propertySnapshot->property_name =
                            CORBA::string_dup (prop_names[i]);
                        CORBA::Any prop_value;
                        prop_value = *(get_property_value (prop_names[i]));
                        CORBA::TypeCode_var tc;
                        tc = prop_value.type ();
                        switch (tc->kind ())
                        {
                        default:
                            break;
                        case CORBA::tk_string:
                            propertySnapshot->property_type =
                                CORBA::string_dup ("String");
                            break;
                        case CORBA::tk_short:
                            propertySnapshot->property_type = CORBA::string_dup ("Short");
                            break;
                        case CORBA::tk_float:
                            propertySnapshot->property_type = CORBA::string_dup ("Float");
                            break;
                        };
                        CosPropertyService::PropertyModeType mode =
                            get_property_mode (prop_names[i]);
                        propertySnapshot->property_value = prop_value;
                        if (mode == CosPropertyService::normal) propertySnapshot->property_mode = CORBA::string_dup ("Read/Write");
                        else if (mode == CosPropertyService::read_only) propertySnapshot->property_mode = CORBA::string_dup ("Read/Only");
                        else propertySnapshot->property_mode = CORBA::string_dup ("Pre-defined");
                        propertySnapshot->property_value = prop_value;
            (*propertiesSnapshot)[snap_prop] = *(propertySnapshot);
                        snap_prop++;
                    }
                }
                // Create a system snapshot
                CdmwPlatformMngt::SystemSnapshot_var snapshot =
                    new CdmwPlatformMngt::SystemSnapshot;
                
                // note : the copy of a snapshot sequence (= operator) 
                //        make a deeply copy with a new memory allocation
                //        for the sequence 
                snapshot->system_name = name();                
                snapshot->system_status = get_status ((snapshot->status_info).out ());
                snapshot->system_mode = get_mode ((snapshot->mode_info).out ());
                snapshot->element_path = element_path();
                snapshot->system_hosts = hostsSnapshot.in ();
                snapshot->system_applications = applicationsSnapshot.in ();
                snapshot->system_entities = entitiesSnapshot.in ();
                snapshot->system_properties = propertiesSnapshot.in ();
            
                return snapshot._retn ();
      }
      catch (const std::bad_alloc &)
      {
          throw CORBA::NO_MEMORY (OrbSupport::NO_MEMORY, CORBA::COMPLETED_NO);
      }
      catch (const Cdmw::OutOfResourcesException &)
      {
          throw CORBA::NO_MEMORY (OrbSupport::NO_MEMORY, CORBA::COMPLETED_NO);
      }
      catch (const CORBA::SystemException &)
      {
          throw;
      }
    }
    
    
    CORBA::ULong System_impl::get_number_of_applications()
        throw (CORBA::SystemException)
    {        
        CORBA::ULong num_apps = m_applications.size();
        return num_apps;
    }
    
    CdmwPlatformMngt::Applications* System_impl::get_applications (CORBA::ULong how_many,
            CdmwPlatformMngt::ApplicationsIterator_out applications_iterator) 
        throw (CORBA::SystemException)
    {
        try
        {
            CDMW_READER_LOCK_GUARD (m_rwSystemLock);
            
            Applications::iterator applicationIt;
            
            // create sequence
            CdmwPlatformMngt::Applications_var applications =
                new CdmwPlatformMngt::Applications ();
            
            // Get the how_many application references   
            size_t len = m_applications.size ();            
            
            // set length of requested sequence and rest of sequence
            size_t requested_len = len;
            size_t rest_len = 0;
            
            // adjust to how_many requested       
            if (len > how_many)
            {
                requested_len = how_many;
                rest_len = len - how_many;
            }
            
            // set sequence length 
            applications->length (requested_len);
            
            // store requested application reference objects         
            unsigned int count;
            for (applicationIt = m_applications.begin (), count = 0;
            count < requested_len; applicationIt++, count++)
            {
                // Store application reference in sequence (duplicate is processed 
                // during var copy)
                applications[count] = (applicationIt->second).m_application;
            }
            
            // Get an iterator on the rest of the applications
            std::list < CdmwPlatformMngt::Application_var > applicationsRemaining;
            for (; applicationIt != m_applications.end (); applicationIt++)
            {
                // Store remaining application reference in list (duplicate is processed 
                // during var copy)
                
                applicationsRemaining.push_back ((applicationIt->second).
                    m_application);
            }
            
            CdmwPlatformMngt::ApplicationsIterator_var applicationsRemainingIt =
                ApplicationsIterator_impl::create (applicationsRemaining);
            
            // Assign the iterator to the processes_iterator
            applications_iterator = applicationsRemainingIt._retn ();
            
            return applications._retn ();
        }
        catch (const std::bad_alloc &)
        {
            throw CORBA::NO_MEMORY (OrbSupport::NO_MEMORY, CORBA::COMPLETED_NO);
        }
    }
    
    
    
    CdmwPlatformMngt::Application_ptr System_impl::
        get_application (const char *application_name) throw (CdmwPlatformMngt::
        ApplicationNotFound,
        CORBA::
        SystemException)
    {
        CDMW_READER_LOCK_GUARD (m_rwSystemLock);
        
        // search application in map
        Applications::iterator applicationIt =
            m_applications.find (application_name);
        
        // if application not found
        if (applicationIt == m_applications.end ())
        {
            throw CdmwPlatformMngt::ApplicationNotFound ();
        }
        
        // return a duplicated object reference
        // (duplicate is implicit in var copy)
        CdmwPlatformMngt::Application_var application =
            (applicationIt->second).m_application;
        return application._retn ();
    }
    
    
    CdmwPlatformMngt::SupervisionObserver_ptr
        System_impl::register_observer (const char *observer_name,
        CdmwPlatformMngt::SupervisionObserver_ptr observer)
        throw (CORBA::SystemException)
    {
        /*CDMW_ASSERT( (CORBA::is_nil(observer))  && "System_impl::register_observer - BAD_PARAM The observer reference is NULL");
        if(CORBA::is_nil(observer))
        {
            LogMngr::logMessage (ERR,"System_impl::register_observer - BAD_PARAM The observer reference is NULL"); 
            throw CORBA::BAD_PARAM(OrbSupport::BAD_PARAM, CORBA::COMPLETED_NO);
        }*/
        
        try
        {
            LogMngr::logMessage (INF, MSG_ID_SYS_REG_OBSERVER,
                observer_name, m_element_name.in ());
              
              CdmwPlatformMngt::SupervisionObserver_var previous_observer =
                  m_eventNotifier->registerObserver (observer_name, observer);
 
            // store observer in datastore
            set_observer_record_in_datastore (observer_name, "", observer);
  
             return previous_observer._retn();

        }
        catch (const BadParameterException &)
        {
            throw CORBA::BAD_PARAM (OrbSupport::BAD_PARAM, CORBA::COMPLETED_NO);
        }
        catch (const EventNotifierStoppedException &)
        {
            throw CORBA::INTERNAL (OrbSupport::INTERNAL, CORBA::COMPLETED_NO);
        }
        catch (const OutOfMemoryException &)
        {
            throw CORBA::NO_MEMORY (OrbSupport::NO_MEMORY, CORBA::COMPLETED_NO);
        }
    }
    
    CdmwPlatformMngt::SupervisionObserver_ptr
        System_impl::register_proxy_observer (const char *observer_name,
        const char *host_name,
        CdmwPlatformMngt::
        SupervisionObserver_ptr observer)
        throw (CORBA::SystemException)
    {
        // TODO
        throw CORBA::NO_IMPLEMENT (OrbSupport::NO_IMPLEMENT,
            CORBA::COMPLETED_NO);
    }
    
    CdmwPlatformMngt::SupervisionObserver_ptr
        System_impl::unregister_observer (const char *observer_name)
        throw (CdmwPlatformMngt::ObserverNotFound, CORBA::SystemException)
    {
        try
        {
            LogMngr::logMessage (INF, MSG_ID_SYS_UNREG_OBSERVER,
                observer_name, m_element_name.in ());
            
            CdmwPlatformMngt::SupervisionObserver_var observer =
             m_eventNotifier->unregisterObserver (observer_name);
            
            // remove observer in datastore
            remove_observer_record_in_datastore (observer_name);
          
            return observer._retn ();
        }
        catch (const EventNotifierStoppedException &)
        {
            throw CORBA::INTERNAL (OrbSupport::INTERNAL, CORBA::COMPLETED_NO);
        }
    }
    
    
    void System_impl::define (const CdmwPlatformMngt::SystemDef & system_def)
        throw (CdmwPlatformMngt::DuplicateEntity,
        CosPropertyService::MultipleExceptions,
        CdmwPlatformMngt::DuplicateService,
        CdmwPlatformMngt::DuplicateHost,
        CdmwPlatformMngt::DuplicateHostEntity,
        CdmwPlatformMngt::HostPropertyExceptions,
        CdmwPlatformMngt::AlreadyDone,
        CdmwPlatformMngt::IncompatibleStatus, CORBA::SystemException)
    {
        LogMngr::logMessage (INF, MSG_ID_DEFINE_SYSTEM,
            system_def.system_name.in ());

        CDMW_SM_READER_LOCK_GUARD (*m_stateMachine);

        m_stateMachine->getState ()->define (m_stateMachine, system_def);
    }
    
    
    void System_impl::set_init_graph (const CdmwPlatformMngt::
        GraphElements & application_graph)
        throw (CdmwPlatformMngt::ReferenceNotFound,
        CdmwPlatformMngt::CircularReference,
        CdmwPlatformMngt::IncompatibleStatus, CORBA::SystemException)
    {
        LogMngr::logMessage (INF, MSG_ID_SET_INITGRAPH, m_element_name.in ());
        
        CDMW_SM_READER_LOCK_GUARD (*m_stateMachine);
        
        m_stateMachine->getState ()->set_init_graph (m_stateMachine,
            application_graph);
    }
    
    
    void System_impl::set_stop_graph (const CdmwPlatformMngt::
        GraphElements & application_graph)
        throw (CdmwPlatformMngt::ReferenceNotFound,
        CdmwPlatformMngt::CircularReference,
        CdmwPlatformMngt::IncompatibleStatus, CORBA::SystemException)
    {
        LogMngr::logMessage (INF, MSG_ID_SET_STOPGRAPH, m_element_name.in ());
        
        CDMW_SM_READER_LOCK_GUARD (*m_stateMachine);
        
        m_stateMachine->getState ()->set_stop_graph (m_stateMachine,
            application_graph);
    }
    
    
    
    void System_impl::set_run_set (const CdmwPlatformMngt::
        RunSetDef & run_set_def)
        throw (CdmwPlatformMngt::DuplicateRunSetElement,
        CdmwPlatformMngt::IncompatibleStatus, CORBA::SystemException)
    {
        LogMngr::logMessage (INF, MSG_ID_SET_RUNSET, m_element_name.in ());
        
        CDMW_SM_READER_LOCK_GUARD (*m_stateMachine);
        
        m_stateMachine->getState ()->set_run_set (m_stateMachine, run_set_def);
    }
    
    void System_impl::remove_run_set (const char *run_set_name)
        throw (CdmwPlatformMngt::RunSetNotFound,
        CdmwPlatformMngt::IncompatibleStatus, CORBA::SystemException)
    {
        LogMngr::logMessage (INF, MSG_ID_REMOVE_RUNSET, m_element_name.in ());
        
        CDMW_SM_READER_LOCK_GUARD (*m_stateMachine);
        
        m_stateMachine->getState ()->remove_run_set (m_stateMachine,
            run_set_name);
    }
    
    void System_impl::remove_all_run_sets ()
        throw (CdmwPlatformMngt::IncompatibleStatus, CORBA::SystemException)
    {
        LogMngr::logMessage (INF, MSG_ID_REMOVE_ALL_RUNSET,
            m_element_name.in ());
        
        CDMW_SM_READER_LOCK_GUARD (*m_stateMachine);
        
        m_stateMachine->getState ()->remove_all_run_sets (m_stateMachine);
    }
    
    CdmwPlatformMngt::Host_ptr
        System_impl::add_host (const CdmwPlatformMngt::HostDef & host_def)
        throw (CdmwPlatformMngt::HostAlreadyExists,
        CdmwPlatformMngt::DuplicateEntity,
        CosPropertyService::MultipleExceptions,
        CdmwPlatformMngt::IncompatibleStatus, CORBA::SystemException)
    {
        std::string hostName = host_def.host_name.in ();
        
        LogMngr::logMessage (INF, MSG_ID_ADD_HOST, hostName.c_str (),
            m_element_name.in ());
        
        CDMW_SM_READER_LOCK_GUARD (*m_stateMachine);
        
        CdmwPlatformMngt::Host_ptr host_ptr = m_stateMachine->getState ()->add_host (m_stateMachine, host_def);

        notify_configuration_change("Host Added");
        return host_ptr; 
    }
    
    void System_impl::remove_host (const char *host_name)
        throw (CdmwPlatformMngt::HostNotFound,
        CdmwPlatformMngt::IncompatibleStatus, CORBA::SystemException)
    {
        LogMngr::logMessage (INF, MSG_ID_REMOVE_HOST, host_name,
            m_element_name.in ());
        
        CDMW_SM_READER_LOCK_GUARD (*m_stateMachine);
        
        m_stateMachine->getState ()->remove_host (m_stateMachine, host_name);
        notify_configuration_change("Host Removed");
    }
    
    
    CdmwPlatformMngt::Application_ptr
        System_impl::add_application (const CdmwPlatformMngt::
                    PlatformApplicationDef &
                    platform_application_def,
                    const CdmwPlatformMngt::
                    ApplicationDef & application_def)
                    throw (CdmwPlatformMngt::ApplicationAlreadyExists,
                    CdmwPlatformMngt::HostNotFound,
                    CdmwPlatformMngt::DuplicateEntity,
                    CosPropertyService::MultipleExceptions,
                    CdmwPlatformMngt::DuplicateService,
                    CdmwPlatformMngt::DuplicateProcess,
                    CdmwPlatformMngt::ProcessHostNotFound,
                    CdmwPlatformMngt::DuplicateProcessEntity,
                    CdmwPlatformMngt::ProcessPropertyExceptions,
                    CdmwPlatformMngt::DuplicateProcessService,
                    CdmwPlatformMngt::DuplicateProcessStep,
                    CdmwPlatformMngt::IncompatibleStatus, CORBA::SystemException)
    {
        std::string applicationName =
            platform_application_def.application_name.in ();
        LogMngr::logMessage (INF, MSG_ID_ADD_APPLICATION,
            applicationName.c_str (), m_element_name.in ());
        
        bool systemDefined = false;
        {
            CDMW_READER_LOCK_GUARD (m_rwSystemLock);
            systemDefined = m_systemDefined;
        }
        
        // check if system is defined
        if (systemDefined)
        {
            CDMW_SM_READER_LOCK_GUARD (*m_stateMachine);
            
        CdmwPlatformMngt::Application_ptr app_ret = m_stateMachine->getState ()->add_application (m_stateMachine,
                platform_application_def,
                application_def);
            notify_configuration_change("Application Added");
            return app_ret; 
        }
        else
        {
            throw CdmwPlatformMngt::IncompatibleStatus ();
        }
    }
    
    void System_impl::remove_application (const char *application_name)
        throw (CdmwPlatformMngt::ApplicationNotFound,
        CdmwPlatformMngt::ApplicationStillActive,
        CdmwPlatformMngt::IncompatibleStatus, CORBA::SystemException)
    {
        LogMngr::logMessage (INF, MSG_ID_REMOVE_APPLICATION,
            application_name, m_element_name.in ());

        {        
        CDMW_SM_READER_LOCK_GUARD (*m_stateMachine);
        
        m_stateMachine->getState ()->remove_application (m_stateMachine,
            application_name); 
        }
 
        // application is removed from not running
        application_deleted(application_name);

        notify_configuration_change("Application Removed");
    }
    
    void System_impl::start (const CdmwPlatformMngtBase::
                 StartupKind & startup_kind)
                 throw (CdmwPlatformMngt::IncompatibleStatus, CORBA::SystemException)
    {
        LogMngr::logMessage (INF, MSG_ID_SYS_START, m_element_name.in ());
        
        // check if system is defined
        bool systemDefined = false;
        {
            CDMW_READER_LOCK_GUARD (m_rwSystemLock);
            systemDefined = m_systemDefined;
        }
        
        if (systemDefined)
        {
            HostTrees::getInstance()->composeMaps (this);
        
            CDMW_SM_WRITER_LOCK_GUARD (*m_stateMachine);
                        
            try
            {
                // recover the default user defined attributes values and entities
                if ( CdmwPlatformMngt::COLD_RESTART==requested_restart_warmth() )
                {
                    recover_default_properties ();
                    recover_default_entities (m_rwEntityContainer, m_default_entitydefs);
                    // recover_default_standard_attributes();
                }
                m_stateMachine->getState ()->start (m_stateMachine, startup_kind);
            }
            catch (...)
            {
                // even in case of exception update record in datastore
                set_system_record_in_datastore ();
                
                // rethrow exception
                throw;
            }

            // update record in datastore
            set_system_record_in_datastore ();
            m_systemStarted = true;

            //MRI start all the host consistency checkers
            CORBA::ULong how_many = get_number_of_hosts ();

            CdmwPlatformMngt::ElementNames_var host_names;
            CdmwPlatformMngt::ElementNamesIterator_var rest;

            get_all_host_names (how_many, host_names, rest);

            for (CORBA::ULong count = 0U; count< how_many; count++)
            {
                    Cdmw::OsSupport::ThreadStatus t_status = getHost(host_names[count])->get_consistency_checker_status();

                    switch (t_status)
                    {
                        case Cdmw::OsSupport::THREAD_RUN_REQUEST:
                             break;
                        case Cdmw::OsSupport::THREAD_RUNNING:
                             break;

                        case Cdmw::OsSupport::THREAD_ENDED:
                        case Cdmw::OsSupport::THREAD_CREATED:
                             getHost(host_names[count])->start_consistency_checker();
                             break;

                        default : break;
                    } 

            }

        }
        else
        {
            throw CdmwPlatformMngt::IncompatibleStatus ();
        }
    }
    
    
    void System_impl::resume ()
        throw (CdmwPlatformMngt::IncompatibleStatus, CORBA::SystemException)
    {
        LogMngr::logMessage (INF, MSG_ID_SYS_RESUME, m_element_name.in ());
        
        CDMW_SM_WRITER_LOCK_GUARD (*m_stateMachine);
        
        try
        {
            m_stateMachine->getState ()->resume (m_stateMachine);
        }
        catch (...)
        {
            // even in case of exception update record in datastore
            set_system_record_in_datastore ();
            
            // rethrow exception
            throw;
        }
        
        // update record in datastore
        set_system_record_in_datastore ();
    }
    
    
    void System_impl::acknowledge ()
        throw (CdmwPlatformMngt::IncompatibleStatus, CORBA::SystemException)
    {
        LogMngr::logMessage (INF, MSG_ID_SYS_ACK, m_element_name.in ());
        
        // Create a command event to notify
        //LC: Attention: call an issuer mutex guard before calling this method.        
        std::string issuer = Cdmw::PlatformMngt::SystemIssuerManager::Instance()->get_command_issuer();

        notifyCommandEvent(ACKNOWLEDGE, issuer.c_str());
        
        CDMW_MUTEX_GUARD (m_mutexMode);
        
        CdmwPlatformMngt::FunctioningMode mode = CdmwPlatformMngt::NORMAL_MODE;
        CORBA::String_var mode_info = CORBA::string_dup ("");
        
        set_mode (mode, mode_info.in ());
        
        // Notify the change of mode
        notifyModeChangeEvent (mode, mode_info.in ());
    }
    
    
    void System_impl::stop (CORBA::Boolean emergency)
        throw (CORBA::SystemException)
    {
        //MR
        //set_requested_restart_warmth ( get_default_restart_warmth () );
    
        try
        {
            LogMngr::logMessage (INF, MSG_ID_SYS_STOP, m_element_name.in ());
            
            // create graph element factory
#if CDMW_ORB_VDR == tao
            GraphElementFactory factory;
#else
            CORBA::ValueFactoryBase_var factory = new GraphElementFactory ();
#endif
            
            CDMW_SM_WRITER_LOCK_GUARD (*m_stateMachine);
            
            // emergency stop
            if (emergency)
            {
                // empty the actual emergency stop graph
                m_stopGraph->length (0);
                
                // set reader lock for application protection
                CDMW_READER_LOCK_GUARD (m_rwSystemLock);
                
                // Add all the active applications as roots of the stop graph          
                Applications::iterator applicationIt;
                
                for (applicationIt = m_applications.begin ();
                applicationIt != m_applications.end (); applicationIt++)
                {
                    CdmwPlatformMngt::ApplicationControl_var application_control =
                        (applicationIt->second).m_applicationControl;
                    
                    CdmwPlatformMngt::Application_var application =
                        (applicationIt->second).m_application;
                    
                    // get application status
                    
                    /***
                    CORBA::String_var info;
                    CdmwPlatformMngt::ApplicationStatus appStatus = 
                    application->get_status(info.out());
                    ***/
                    
                    CdmwPlatformMngt::ApplicationStatus appStatus =
                        application_control->get_internal_status ();
                    
                    if ((appStatus == CdmwPlatformMngt::APPLICATION_INITIALISING)
                        || (appStatus ==
                        CdmwPlatformMngt::APPLICATION_INITIALISED)
                        || (appStatus ==
                        CdmwPlatformMngt::
                        APPLICATION_INITIALISATION_INCOMPLETE)
                        || (appStatus ==
                        CdmwPlatformMngt::
                        APPLICATION_INITIALISATION_SUSPENDED)
                        || (appStatus ==
                        CdmwPlatformMngt::APPLICATION_RUN_REQUEST)
                        || (appStatus == 
                        CdmwPlatformMngt::APPLICATION_RUNNING)
                        || (appStatus == 
                        CdmwPlatformMngt::APPLICATION_PARTIAL_RUNNING))
                    {
                        // create graph element using factory
                        CORBA::String_var appliName = application->name ();
#if CDMW_ORB_VDR == tao
                        CdmwPlatformMngt::GraphElement_var element =
                            factory.create (appliName.in (), 0, 0);
#else
                        CdmwPlatformMngt::GraphElement_var element =
                            dynamic_cast < GraphElementFactory * >(factory.in ())->
                            create (appliName.in (), 0);
#endif
                        
                        // insert new element in graph
                        int graphlen = m_stopGraph->length ();
                        m_stopGraph->length (graphlen + 1);
                        m_stopGraph->operator[](graphlen) = element;
                    }
                }
            }
            
            else
            {
                // Get the list of all element of the stop graph
                GraphElementNames elementNames =
                    GraphUtility::getElementNames (m_applicationStopGraph.in ());
                
                
                // create a new stop graph copy of the original
                m_stopGraph = m_applicationStopGraph;
                
                // set reader lock for application protection
                CDMW_READER_LOCK_GUARD (m_rwSystemLock);
                
                // Add all the active applications as roots of the stop graph
                // if not present in the stop graph.
                Applications::iterator applicationIt;
                
                for (applicationIt = m_applications.begin ();
                applicationIt != m_applications.end (); applicationIt++)
                {
                    CdmwPlatformMngt::ApplicationControl_var application_control =
                        (applicationIt->second).m_applicationControl;
                    
                    CdmwPlatformMngt::Application_var application =
                        (applicationIt->second).m_application;
                    
                    // if application not found in the stop graph  
                    if (elementNames.find (applicationIt->first) ==
                        elementNames.end ())
                    {
                        // get application status
                        
                        /***
                        CORBA::String_var info;
                        CdmwPlatformMngt::ApplicationStatus appStatus = 
                        application->get_status(info.out());
                        ***/
                        
                        CdmwPlatformMngt::ApplicationStatus appStatus =
                            application_control->get_internal_status ();
                        
                        // if application candidate to be stopping        
                        if ((appStatus ==
                            CdmwPlatformMngt::APPLICATION_INITIALISING)
                            || (appStatus ==
                            CdmwPlatformMngt::APPLICATION_INITIALISED)
                            || (appStatus ==
                            CdmwPlatformMngt::
                            APPLICATION_INITIALISATION_INCOMPLETE)
                            || (appStatus ==
                            CdmwPlatformMngt::
                            APPLICATION_INITIALISATION_SUSPENDED)
                            || (appStatus ==
                            CdmwPlatformMngt::APPLICATION_RUN_REQUEST)
                            || (appStatus ==
                            CdmwPlatformMngt::APPLICATION_RUNNING)
                            || (appStatus ==
                            CdmwPlatformMngt::APPLICATION_PARTIAL_RUNNING))
                        {
                            // create graph element from application name using factory
                            CORBA::String_var appliName = application->name ();
#if CDMW_ORB_VDR == tao
                            CdmwPlatformMngt::GraphElement_var element =
                                factory.create (appliName.in (), 0, 0);
#else
                            CdmwPlatformMngt::GraphElement_var element =
                                dynamic_cast <
                                GraphElementFactory *
                                >(factory.in ())->create (appliName.in (), 0);
#endif
                            
                            // insert new application element in graph
                            int graphlen = m_stopGraph->length ();
                            m_stopGraph->length (graphlen + 1);
                            m_stopGraph->operator[](graphlen) = element;
                        }
                    }
                }
            }
            
            // request stop      
            try
            {
                m_emergency = emergency;
                m_stateMachine->getState ()->stop (m_stateMachine, emergency);
            }
            catch (...)
            {
                // even in case of exception update record in datastore
                set_system_record_in_datastore ();
                
                // rethrow exception
                throw;
            }
            
            // update record in datastore
            set_system_record_in_datastore ();
      }
      catch (const std::bad_alloc &)
      {
          throw CORBA::NO_MEMORY (OrbSupport::NO_MEMORY, CORBA::COMPLETED_NO);
      }
    }
    
    void System_impl::notifyCommandEvent(SystemCommand sys_command, 
            const char* issuer,
            CORBA::Boolean emergency)
    {

        CdmwPlatformMngt::CdmwCommand_var event;       
        CdmwPlatformMngtBase::EventHeader header
        = EventHeaderFactory::createHeader (CdmwPlatformMngtBase::INF);

        if ((COLD_RESTART == sys_command) || (HOT_RESTART == sys_command))
        {
            //Setting up the startupKind parameters
            CdmwPlatformMngtBase::StateRetrieval state_retrieval = CdmwPlatformMngtBase::LAST_STATE;
            unsigned long startup_mode = 0;
            std::string specified_state_id = "";
            CdmwPlatformMngtBase::StartupKind_var startupKind = new CdmwPlatformMngtBase::StartupKind();
            startupKind->state_retrieval = state_retrieval;
            startupKind->startup_mode = startup_mode;
            startupKind->specified_state_id  = CORBA::string_dup( specified_state_id.c_str() );

            CdmwPlatformMngt::RestartWarmth restart_warmth;

            if (COLD_RESTART == sys_command)
            {
                restart_warmth = CdmwPlatformMngt::COLD_RESTART; 
            }
            else
            {               
                restart_warmth = CdmwPlatformMngt::HOT_RESTART;
            }

            // Create the event
            event = m_sysRestartFactory->create (header,
                    name(),
                    get_element_path(),
                    name(),
                    issuer,
                    startupKind,
                    restart_warmth);
        }

        if (STOP == sys_command)
        {
            event = m_sysStopFactory->create (header,
                    name(),
                    get_element_path(),
                    name(),
                    issuer,
                    emergency);    
        }

        if (ACKNOWLEDGE == sys_command)
        {
            event = m_sysAcknowledgeFactory->create (header,
                    name(),
                    get_element_path(),
                    name(),
                    issuer);
        }

        //Log event
        CORBA::String_var eventStr = event->to_string ();
        LogMngr::logMessage (eventStr.in ()); 

        // Perform the command notification to supervision event notifier
        event->_add_ref(); 
        (EventNotifierManager::Instance()->get_sup_notifier())->addEvent (event);

        // Perform the command notification to command event notifier
        event->_add_ref(); 
        (EventNotifierManager::Instance()->get_command_notifier())->add_event(event);
        event->_remove_ref();

        event._retn ();

    }
    
    
    void System_impl::cold_restart (const CdmwPlatformMngtBase::
                    StartupKind & startup_kind)
                    throw (CdmwPlatformMngt::IncompatibleStatus, CORBA::SystemException)
    {
        
        // Create a command event to notify
        //LC: Attention: call an issuer mutex guard before calling this method.        
        std::string issuer = Cdmw::PlatformMngt::SystemIssuerManager::Instance()->get_command_issuer();

        notifyCommandEvent(COLD_RESTART, issuer.c_str());
        
        bool emergency = false;
        CdmwPlatformMngt::SystemStatus sysStatus;

     {
           // lock mutex for reading status
           CDMW_SM_READER_LOCK_GUARD (*m_stateMachine);
            
            // get the current status
            sysStatus = m_stateMachine->getState ()->status ();
         }
          
         try 
         {
            if ( CdmwPlatformMngt::LONG_REQUEST_COMPLETED != restart ( CdmwPlatformMngt::COLD_RESTART, startup_kind, emergency, sysStatus ) ){
       return;   
     }
      }
      catch (...) {
      throw;
      }
   }
    
    
    void System_impl::hot_restart (const CdmwPlatformMngtBase::
        StartupKind & startup_kind)
        throw (CdmwPlatformMngt::IncompatibleStatus, CORBA::SystemException)
    {

        // Create a command event to notify
        //LC: Attention: call an issuer mutex guard before calling this method.        
        std::string issuer = Cdmw::PlatformMngt::SystemIssuerManager::Instance()->get_command_issuer();
        
        notifyCommandEvent(HOT_RESTART, issuer.c_str());
        
        bool emergency = false;
        CdmwPlatformMngt::SystemStatus sysStatus;

        {
            // lock mutex for reading status
            CDMW_SM_READER_LOCK_GUARD (*m_stateMachine);
            
            // get the current status
            sysStatus = m_stateMachine->getState ()->status ();
           
            }
      
      try {

      if (CdmwPlatformMngt::LONG_REQUEST_COMPLETED !=
         restart ( CdmwPlatformMngt::HOT_RESTART, startup_kind, emergency, sysStatus ) ){
     return;     
     }
      }
      catch (...) {
      throw;
      }
      
    }
      

    void System_impl::setStopStatus (CdmwPlatformMngt::
        LongRequestStatus status)
    {
        CDMW_MUTEX_GUARD (m_mutex);
        // set condition wait flag
        m_stop_condition_flag = true;
        m_stop_status = status;
        
        m_stop_condition.broadcast ();
    }
    
    CORBA::ULong System_impl::get_number_of_entities ()throw (CORBA::
        SystemException)
    {
        return m_rwEntityContainer->get_number_of_entities ();
    }
    
    void System_impl::get_all_entity_names (CORBA::ULong how_many,
        CdmwPlatformMngt::
        ElementNames_out entity_names,
        CdmwPlatformMngt::
        ElementNamesIterator_out rest)
        throw (CORBA::SystemException)
    {
        
        m_rwEntityContainer->get_all_entity_names (how_many, entity_names,
            rest);
        
    }
    
    
    CdmwPlatformMngt::Entity_ptr System_impl::
        get_entity (const char *entity_name) throw (CdmwPlatformMngt::
                          EntityNotFound,
                          CORBA::SystemException)
    {
        return m_rwEntityContainer->get_entity (entity_name);
    }
    
    CORBA::Boolean System_impl::get_entities (const CdmwPlatformMngt::
        ElementNames & entity_names,
        CdmwPlatformMngt::
        Entities_out nentities)
        throw (CORBA::SystemException)
    {
        
        return m_rwEntityContainer->get_entities (entity_names, nentities);
        
    }
    
    void System_impl::get_all_entities (CORBA::ULong how_many,
        CdmwPlatformMngt::
        Entities_out nentities,
        CdmwPlatformMngt::
        EntitiesIterator_out rest)
        throw (CORBA::SystemException)
    {
        
        m_rwEntityContainer->get_all_entities (how_many, nentities, rest);
        
    }
    
    CdmwPlatformMngt::Entity_ptr System_impl::
        add_entity (const CdmwPlatformMngt::
        EntityDef & entity_def) throw (CdmwPlatformMngt::
        EntityAlreadyExists,
        CORBA::SystemException)
    {
        std::string entityName = entity_def.entity_name.in ();
        
        LogMngr::logMessage (INF, MSG_ID_ADD_SYSTEM_ENTITY,
            entityName.c_str (), m_element_name.in ());
        
        CdmwPlatformMngt::Entity_ptr entity_ptr = m_rwEntityContainer->add_entity (entity_def);
        
        notify_configuration_change("Entity Added");
        return entity_ptr; 
    }
    
    
    void System_impl::remove_entity (const char *entity_name)
        throw (CdmwPlatformMngt::EntityNotFound, CORBA::SystemException)
    {
        LogMngr::logMessage (INF, MSG_ID_REMOVE_SYSTEM_ENTITY,
            entity_name, m_element_name.in ());
        
        m_rwEntityContainer->remove_entity (entity_name);
        
        notify_configuration_change("Entity Removed");
    }
    
    Entity_impl *System_impl::getEntity (const char *entity_name)
    throw (CORBA::SystemException)
    {
       return m_actualEntityContainer->getEntity (entity_name);
    }

    
    CdmwPlatformMngtService::ServiceProviderID *
        System_impl::
        get_service_def (const char *service_name)
        throw (CdmwPlatformMngtService::ServiceNotFound, CORBA::SystemException)
    {
        return m_rwServiceDefContainer->get_service_def (service_name);
    }
    
    void System_impl::add_service_def (const CdmwPlatformMngtService::
        ServiceDef & service_def)
        throw (CdmwPlatformMngtService::ServiceAlreadyExists,
        CORBA::SystemException)
    {
        m_rwServiceDefContainer->add_service_def (service_def);
        
        notify_configuration_change("Addition of Service Definition");
 
        // TODO: Trace to the log manager
    }
    
    void System_impl::remove_service_def (const char *service_name)
        throw (CdmwPlatformMngtService::ServiceNotFound, CORBA::SystemException)
    {
        m_rwServiceDefContainer->remove_service_def (service_name);
        
        notify_configuration_change("Remotion of Service Definition");
 
        // TODO: Trace to the log manager
    }
    
    void System_impl::remove_all_service_defs ()
        throw (CORBA::SystemException)
    {
        m_rwServiceDefContainer->remove_all_service_defs ();
        
        // TODO: Notify the configuration change
        
        // TODO: Trace to the log manager
    }
    
    
    CORBA::ULong System_impl::get_number_of_hosts ()throw (CORBA::
        SystemException)
    {
        return m_hostContainer->get_number_of_hosts ();
    }
    
    
    
    void System_impl::get_all_host_names (CORBA::ULong how_many,
        CdmwPlatformMngt::
        ElementNames_out host_names,
        CdmwPlatformMngt::
        ElementNamesIterator_out rest)
        throw (CORBA::SystemException)
    {
        m_hostContainer->get_all_host_names (how_many, host_names, rest);
    }
    
    
    
    CdmwPlatformMngt::Host_ptr System_impl::get_host (const char *host_name)
        throw (CdmwPlatformMngt::HostNotFound, CORBA::SystemException)
    {
        return m_hostContainer->get_host (host_name);
    }
    
    
    
    CORBA::Boolean System_impl::get_hosts (const CdmwPlatformMngt::
                       ElementNames & host_names,
                       CdmwPlatformMngt::
                       Hosts_out hosts) throw (CORBA::
                       SystemException)
    {
        return m_hostContainer->get_hosts (host_names, hosts);
    }
    
    
    
    void System_impl::get_all_hosts (CORBA::ULong how_many,
        CdmwPlatformMngt::Hosts_out hosts,
        CdmwPlatformMngt::HostsIterator_out rest)
        throw (CORBA::SystemException)
    {
        m_hostContainer->get_all_hosts (how_many, hosts, rest);
    }
    
    
    
    RunSetManager *System_impl::get_runset_manager ()
    {
        return m_runSetManager;
    }
    
    
    EventHandler *System_impl::get_event_handler ()
    {
        return &m_eventHandler;
    }
    
    SupervisionEventNotifier *System_impl::get_event_notifier ()
    {
        return m_eventNotifier;
    }
    
    CdmwPlatformMngt::ElementType System_impl::get_element_type()
        throw (CORBA::SystemException)
        {
            return CdmwPlatformMngt::SYSTEM_TYPE;
        }
    
    CdmwPlatformMngt::Children* System_impl::get_children()
        throw (CORBA::SystemException)
        {
            CdmwPlatformMngt::Children_var children = new CdmwPlatformMngt::Children;
            CORBA::ULong size_of_entities = get_number_of_entities();
            CORBA::ULong size_of_hosts = get_number_of_hosts();
            CORBA::ULong size_of_applications = get_number_of_applications();
            //set the lentgh of the returned sequence     
            children->length(size_of_applications + size_of_entities + size_of_hosts);
               
            CdmwPlatformMngt::Entities_var      entities;
            CdmwPlatformMngt::Applications_var  applications;
            CdmwPlatformMngt::Hosts_var         hosts;

            CdmwPlatformMngt::EntitiesIterator_var      entities_iterator;
            CdmwPlatformMngt::HostsIterator_var         hosts_iterator;
            CdmwPlatformMngt::ApplicationsIterator_var  applications_iterator;
    
            applications = get_applications (size_of_applications, applications_iterator.out());

            // store the applications in the children sequence returned 
            for (CORBA::ULong count = 0; count<size_of_applications; count++)
            {
                applications[count] -> _add_ref();
                children[count] = (applications[count])._retn();
            }
    
            get_all_hosts (size_of_hosts, hosts.out(), hosts_iterator.out());
            // store the hosts in the children sequence returned 
            for (CORBA::ULong count = size_of_applications; count<(size_of_applications + size_of_hosts); count++)
            {
                hosts[count - size_of_applications] -> _add_ref();
                children[count] = (hosts[count - size_of_applications])._retn();
            }
            
            get_all_entities(size_of_entities, entities.out(), entities_iterator.out() );
            // store the entities in the children sequence returned 
            for (CORBA::ULong count = (size_of_applications + size_of_hosts); 
                count< (size_of_applications + size_of_entities + size_of_hosts); count++)
            {
                entities[count - (size_of_applications + size_of_hosts)]->_add_ref();
                children[count] = (entities[count - (size_of_applications + size_of_hosts)])._retn();
            }

            return children._retn ();
        }

     char* System_impl::get_logger_default_filters()
         throw (CORBA::SystemException)
     {

         string default_filters ="";
         if (Cdmw::Logging::DEFAULT_LEVEL & Cdmw::Logging::FATAL) 
            { 
            default_filters += "FATAL ";
            }
         if (Cdmw::Logging::DEFAULT_LEVEL & Cdmw::Logging::ERROR)
            { 
            default_filters += "ERROR ";  
            }
         if (Cdmw::Logging::DEFAULT_LEVEL & Cdmw::Logging::WARN) 
            {
            default_filters += "WARN ";
            }
         if (Cdmw::Logging::DEFAULT_LEVEL & Cdmw::Logging::INFO) 
            {
            default_filters += "INFO";
            }
         if (Cdmw::Logging::DEFAULT_LEVEL & Cdmw::Logging::DEBUG)
            {
            default_filters += "DEBUG";
            }
                   
         return  CORBA::string_dup(default_filters.c_str());
     }   
    
     
    CdmwPlatformMngt::ManagedElement* System_impl::get_element(const char* element_path)     
         throw (CORBA::SystemException)
    {
    CDMW_LOG_FUNCTION_INOUTBOUND (Cdmw::Logging::Logger::Get_Logger(CdmwPlatformMngtBase::PFMMNG),"");  
        //check if string is empty 
        string elem_path (element_path);
    
    //FLI: testelemnavigability problem
        //CDMW_ASSERT( elem_path.empty() && "System_impl::get_element() - Element Path is null");
        
    if (elem_path.empty())
        {
            LogMngr::logMessage(ERR,"System_impl::get_element() - Element Path is null");
            CDMW_LOG_FUNCTION_EXCEPTION_INBOUND("CORBA::BAD_PARAM");
            throw CORBA::BAD_PARAM (OrbSupport::BAD_PARAM, CORBA::COMPLETED_NO);
        }
            
        //output parameters
        string system_name, 
               application_name, 
               host_name,process_name,
               component_name,
               entity_name,
               attribute_name, 
               ftgroup_name,lbgroup_name;

        CdmwPlatformMngt::ElementType element_type;
        
        //parsing element path
        try
        {
            element_type = 
                Cdmw::PlatformMngt::ManagedElementKey::Get_element_names(element_path,
                                                                        system_name,
                                                                        application_name, 
                                                                        host_name,
                                                                        process_name, 
                                                                        component_name,
                                                                        entity_name,
                                                                        attribute_name,
                                                                        ftgroup_name,
                                                                        lbgroup_name);
        }
        catch (const CdmwPlatformMngt::ElementPathInvalid&)
        {
            LogMngr::logMessage(ERR,"System_impl::get_element() - ElementPathInvalid ");  
            CDMW_LOG_FUNCTION_EXCEPTION_INBOUND("CORBA::BAD_PARAM");
            throw CORBA::BAD_PARAM (OrbSupport::BAD_PARAM, CORBA::COMPLETED_NO);
        }

        CdmwPlatformMngt::ManagedElement_var managed_element = 
            CdmwPlatformMngt::ManagedElement::_nil();  

        //check element type
        switch( element_type)
        {
            case (CdmwPlatformMngt::SYSTEM_TYPE) :
            {
                if(strcmp(system_name.c_str(), name() )) {
                    LogMngr::logMessage(FTL,"System_impl::get_element() - SystemNotFound ");
                    throw CORBA::BAD_PARAM(OrbSupport::BAD_PARAM, CORBA::COMPLETED_NO);
                }
                try
                {
                    managed_element = CdmwPlatformMngt::ManagedElement::_narrow(this->_this());
                }
                catch (...)
                {
                    LogMngr::logMessage(FTL,"System_impl::get_element() - Fatal error ");
                    throw CORBA::INTERNAL(OrbSupport::INTERNAL, CORBA::COMPLETED_NO);
                }
            }    
            break;
                    
            case (CdmwPlatformMngt::HOST_TYPE) :
            {
                try
                {
                    managed_element = this->get_host((host_name).c_str());
                }
                catch (const CdmwPlatformMngt::HostNotFound & e)
                {
                    LogMngr::logMessage(INF,"System_impl::get_element() - HostNotFound");
                    throw CORBA::BAD_PARAM (OrbSupport::BAD_PARAM, CORBA::COMPLETED_NO);
                }
                catch (...)
                {
                    LogMngr::logMessage(FTL,"System_impl::get_element() - Fatal error in get attribute on Host");
                    throw CORBA::INTERNAL(OrbSupport::INTERNAL, CORBA::COMPLETED_NO);
                }                 
            }
            break;
    
            case (CdmwPlatformMngt::APPLICATION_TYPE) :
            {
                try
                {                   
                    managed_element = this->get_application((application_name).c_str());  
                }
                catch (const CdmwPlatformMngt::ApplicationNotFound & e)
                {
                    LogMngr::logMessage(INF,"System_impl::get_element() - ApplicationNotFound");
                    throw CORBA::BAD_PARAM (OrbSupport::BAD_PARAM, CORBA::COMPLETED_NO);
                }
                catch (...)
                {
                    LogMngr::logMessage(FTL,"System_impl::get_element() - Fatal error in get attribute on Application");
                    throw CORBA::INTERNAL(OrbSupport::INTERNAL, CORBA::COMPLETED_NO);
                }   
    
            }
            break;
    
            case (CdmwPlatformMngt::FT_GROUP_TYPE) :
            {   
                try
                {                   
                    CdmwPlatformMngt::Application_var app = this->get_application((application_name).c_str());
    
                    managed_element = app->get_ft_group((ftgroup_name).c_str());
                }
                catch (const CdmwPlatformMngt::ApplicationNotFound & e)
                {
                    LogMngr::logMessage(INF,"System_impl::get_element() - ApplicationNotFound");
                    throw CORBA::BAD_PARAM (OrbSupport::BAD_PARAM, CORBA::COMPLETED_NO);
                }
                catch (const CdmwPlatformMngt::GroupNotFound & e)
                {
                    LogMngr::logMessage(INF,"System_impl::get_element() - FTGroupNotFound");
                    throw CORBA::BAD_PARAM (OrbSupport::BAD_PARAM, CORBA::COMPLETED_NO);
                }
                catch (...)
                {
                    LogMngr::logMessage(FTL,"System_impl::get_element() - Fatal error in get attribute on Application");
                    throw CORBA::INTERNAL(OrbSupport::INTERNAL, CORBA::COMPLETED_NO);
                }   
    
            }
            break;
            
            case (CdmwPlatformMngt::LB_GROUP_TYPE) :
            {  
                try
                {                   
                    CdmwPlatformMngt::Application_var app = this->get_application((application_name).c_str());
    
                    managed_element = app->get_lb_group((lbgroup_name).c_str());   
                }
                catch (const CdmwPlatformMngt::ApplicationNotFound & e)
                {
                    LogMngr::logMessage(INF,"System_impl::get_element() - ApplicationNotFound");
                    throw CORBA::BAD_PARAM (OrbSupport::BAD_PARAM, CORBA::COMPLETED_NO);
                }
                catch (const CdmwPlatformMngt::GroupNotFound & e)
                {
                    LogMngr::logMessage(INF,"System_impl::get_element() - LBGroupNotFound");
                    throw CORBA::BAD_PARAM (OrbSupport::BAD_PARAM, CORBA::COMPLETED_NO);
                }
                catch (...)
                {
                    LogMngr::logMessage(FTL,"System_impl::get_element() - Fatal error in get attribute on Application");
                    throw CORBA::INTERNAL(OrbSupport::INTERNAL, CORBA::COMPLETED_NO);
                }   
    
            }
            break;
    
            case (CdmwPlatformMngt::PROCESS_TYPE) :
            {
                try
                {
                    CdmwPlatformMngt::Application_var app = this->get_application((application_name).c_str());
    
                    managed_element = app->get_process((process_name).c_str(), (host_name).c_str()); 
                }
                catch (const CdmwPlatformMngt::ApplicationNotFound & e)
                {
                    LogMngr::logMessage(INF,"System_impl::get_element() - ApplicationNotFound");
                    throw CORBA::BAD_PARAM (OrbSupport::BAD_PARAM, CORBA::COMPLETED_NO);
                }
                catch (const CdmwPlatformMngt::ProcessNotFound & e)
                {
                    LogMngr::logMessage(INF,"System_impl::get_element() - ProcessNotFound");
                    throw CORBA::BAD_PARAM (OrbSupport::BAD_PARAM, CORBA::COMPLETED_NO);
                }
                catch (...)
                {
                    LogMngr::logMessage(FTL,"System_impl::get_element() - Fatal error in get attribute on Process");
                    throw CORBA::INTERNAL(OrbSupport::INTERNAL, CORBA::COMPLETED_NO);
                }       
            }
            break;          
    
            case (CdmwPlatformMngt::COMPONENT_TYPE) :
            {
                try
                {
                    CdmwPlatformMngt::Application_var app = this->get_application((application_name).c_str());
    
                    CdmwPlatformMngt::Process_var proc = app->get_process((process_name).c_str(), (host_name).c_str());
    
                    managed_element = proc->get_component((component_name).c_str()); 
                }
                catch (const CdmwPlatformMngt::ApplicationNotFound & e)
                {
                    LogMngr::logMessage(INF,"System_impl::get_element() - ApplicationNotFound");
                    throw CORBA::BAD_PARAM (OrbSupport::BAD_PARAM, CORBA::COMPLETED_NO);
                }
                catch (const CdmwPlatformMngt::ProcessNotFound & e)
                {
                    LogMngr::logMessage(INF,"System_impl::get_element() - ProcessNotFound");
                    throw CORBA::BAD_PARAM (OrbSupport::BAD_PARAM, CORBA::COMPLETED_NO);
                }
                catch (const CdmwPlatformMngt::ComponentNotFound & e)
                {
                    LogMngr::logMessage(INF,"System_impl::get_element() - ComponentNotFound");
                    throw CORBA::BAD_PARAM (OrbSupport::BAD_PARAM, CORBA::COMPLETED_NO);
                }
                catch (...)
                {
                    LogMngr::logMessage(FTL,"System_impl::get_element() - Fatal error in get attribute on Component");
                    throw CORBA::INTERNAL(OrbSupport::INTERNAL, CORBA::COMPLETED_NO);
                }       
            }
            break;
    
            case (CdmwPlatformMngt::ENTITY_TYPE) :
            {
                try
                {       
                    if (0 == host_name.empty())
                    {                       

                        CdmwPlatformMngt::Host_var host = this->get_host((host_name).c_str());
                            
                        if (0 == application_name.empty())
                        {
                            CdmwPlatformMngt::Application_var app = this->get_application((application_name).c_str());
    
                            if (0 == process_name.empty())
                            {
                                CdmwPlatformMngt::Process_var proc = app->get_process((process_name).c_str(),(host_name).c_str());
    
                                if (0 == component_name.empty())    
                                {
                                    CdmwPlatformMngt::CcmComponent_var comp = proc->get_component((component_name).c_str());
                                    
                                    managed_element = comp->get_entity(entity_name.c_str());

                                }
                                else 
                                {
                                     managed_element = proc->get_entity(entity_name.c_str());
                                }
                            }
                        }
                        else
                        {                           
                            managed_element = host->get_entity(entity_name.c_str());
                        }
                    
                    }else if (0 == application_name.empty())              
                    {
                        CdmwPlatformMngt::Application_var app = this->get_application((application_name).c_str());
                        
                        if (0 == ftgroup_name.empty())
                        {
                            CdmwPlatformMngt::FTGroup_var ft_group = app->get_ft_group((ftgroup_name).c_str());
                            
                            managed_element = ft_group->get_entity(entity_name.c_str());
                        }                  
                        else if (0 == lbgroup_name.empty())
                        {
                            CdmwPlatformMngt::LBGroup_var lb_group = app->get_lb_group((lbgroup_name).c_str());
                            
                            managed_element = lb_group->get_entity(entity_name.c_str());
                        }
                        else                  
                        {         
                            managed_element = app->get_entity(entity_name.c_str());
                        }
                    }
                    else
                    {
                        managed_element = this->get_entity(entity_name.c_str());
                    }
                }
                catch (const CdmwPlatformMngt::HostNotFound & e)
                {
                    LogMngr::logMessage(INF,"System_impl::get_element() - HostNotFound");   
                    throw CORBA::BAD_PARAM (OrbSupport::BAD_PARAM, CORBA::COMPLETED_NO);  
                }
                catch (const CdmwPlatformMngt::ApplicationNotFound & e)
                {
                    LogMngr::logMessage(INF,"System_impl::get_element() - ApplicationNotFound");
                    throw CORBA::BAD_PARAM (OrbSupport::BAD_PARAM, CORBA::COMPLETED_NO); 
                }
                catch (const CdmwPlatformMngt::ProcessNotFound & e)
                {
                    LogMngr::logMessage(INF,"System_impl::get_element() - ProcessNotFound");
                    throw CORBA::BAD_PARAM (OrbSupport::BAD_PARAM, CORBA::COMPLETED_NO);    
                }
                catch (const CdmwPlatformMngt::ComponentNotFound & e)
                {
                    LogMngr::logMessage(INF,"System_impl::get_element() - ComponentNotFound");
                    throw CORBA::BAD_PARAM (OrbSupport::BAD_PARAM, CORBA::COMPLETED_NO);    
                }
                catch (const CdmwPlatformMngt::GroupNotFound & e)
                {
                    LogMngr::logMessage(INF,"System_impl::get_element() - FT/LB GroupNotFound");
                    throw CORBA::BAD_PARAM (OrbSupport::BAD_PARAM, CORBA::COMPLETED_NO);   
                }
                catch (const CdmwPlatformMngt::EntityNotFound & e)
                {
                    LogMngr::logMessage(INF,"System_impl::get_element() - EntityNotFound");
                    throw CORBA::BAD_PARAM (OrbSupport::BAD_PARAM, CORBA::COMPLETED_NO);
                }
                catch (...)
                {
                    LogMngr::logMessage(FTL,"System_impl::get_element() - Fatal error in get reference on Entity");
                    throw CORBA::INTERNAL(OrbSupport::INTERNAL, CORBA::COMPLETED_NO);
                }       
            }
            break;
    
            default :            
                LogMngr::logMessage(ERR,"System_impl::get_element() - ElementType not managed"); 
                throw CORBA::BAD_PARAM (OrbSupport::BAD_PARAM, CORBA::COMPLETED_NO);
            break;

        }  

        // CDMW_ASSERT( CORBA::is_nil(managed_element.in()) && "System_impl::get_element() returns a null pointer");
                if (CORBA::is_nil(managed_element.in()))
                {
                    LogMngr::logMessage(ERR,"System_impl::get_element() returns a null pointer");
                    CDMW_LOG_FUNCTION_RETURN_OUTBOUND(0);
                    throw CORBA::INTERNAL (Cdmw::OrbSupport::INTERNAL,
                              CORBA::COMPLETED_MAYBE);
                }
        return managed_element._retn();
     }   
     
     void System_impl::get_all_applications(CORBA::ULong how_many, 
                               CdmwPlatformMngt::Applications_out napplications, 
                               CdmwPlatformMngt::ApplicationsIterator_out rest)
         throw (CORBA::SystemException)
     {
         // TODO
         throw CORBA::NO_IMPLEMENT (OrbSupport::NO_IMPLEMENT, CORBA::COMPLETED_NO);
     }   
     
     void System_impl::stop_system(CORBA::Boolean emergency) 
        throw (CORBA::SystemException)
     {
         // Create a command event to notify
         //LC: Attention: call an issuer mutex guard before calling this method.         
         std::string issuer = Cdmw::PlatformMngt::SystemIssuerManager::Instance()->get_command_issuer();

         notifyCommandEvent(STOP, issuer.c_str());
         
         stop(emergency);
     }   
     
     CORBA::ULong System_impl::get_number_of_running_applications()
     {
         CORBA::ULong running_applications = 0;
         ApplicationsStatusInfo::iterator it = m_applications_status_info.begin();
         ApplicationsStatusInfo::iterator it_end = m_applications_status_info.end();  
         while (it != it_end)
         { 
              if ( (CdmwPlatformMngt::APPLICATION_RUNNING == it->second) || (CdmwPlatformMngt::APPLICATION_INITIALISED == it->second) || (CdmwPlatformMngt::APPLICATION_INITIALISING == it->second) || (CdmwPlatformMngt::APPLICATION_RUN_REQUEST == it->second))
                  ++running_applications;
              ++it;
         }
         return running_applications;
     }    



    CORBA::ULong System_impl::get_number_of_applications_with_status(CdmwPlatformMngt::ApplicationStatus application_status)
    {
         CORBA::ULong applications_with_status = 0;
         ApplicationsStatusInfo::iterator it = m_applications_status_info.begin();
         ApplicationsStatusInfo::iterator it_end = m_applications_status_info.end();

         while (it != it_end)
         {
              if (application_status == it->second)
                  ++applications_with_status;
              ++it;
         }
         return applications_with_status;
    }


     bool System_impl::has_partial_running_applications()
     {
         bool ret_value = false;

         std::vector<Application_impl*> vect_app = get_application_impls();
         for (unsigned int i=0; i< vect_app.size(); i++)
         {
             Application_impl* application = vect_app[i];
             std::string current_application_name = application->name();
             
             ApplicationsStatusInfo::iterator it = m_applications_status_info.find(current_application_name);
             ApplicationsStatusInfo::iterator it_end = m_applications_status_info.end();
             if (( it!= it_end )&&(CdmwPlatformMngt::APPLICATION_PARTIAL_RUNNING==it->second)) 
             {
                ret_value = true;
                break;
             }
         }
         return ret_value;
      }


    void System_impl::synchro_consistent_status()
    {
         std::vector<Application_impl*> vect_app = get_application_impls();

         for (unsigned int i=0; i< vect_app.size(); i++)
         {
             Application_impl* application = vect_app[i];

             // synchro_consistent_status for application
             application->synchro_consistent_status();
             std::string current_application_name = application->name();
             CdmwPlatformMngt::ApplicationStatus app_status =  application->get_internal_status ();
             // synchronize the local map of applications
             application_status_change_event(current_application_name, app_status);
         }          
         check_consistency_of_state();
    }

    Application_impl* System_impl::get_application_impl (const char *application_name) 
        throw (CdmwPlatformMngt::ApplicationNotFound, CORBA::SystemException)
    {
        // Implementation is provided in CentralisedSupervisionSystem 
        throw CORBA::NO_IMPLEMENT (OrbSupport::NO_IMPLEMENT,
            CORBA::COMPLETED_NO);
    }

    CORBA::ULong System_impl::get_number_of_application_with_relevant_indicator()
       throw ()
    {
        // Implementation is provided in CentralisedSupervisionSystem
        throw CORBA::NO_IMPLEMENT (OrbSupport::NO_IMPLEMENT,
            CORBA::COMPLETED_NO);
    }
    
    CdmwPlatformMngt::DaemonController_var System_impl::getDaemonController(std::string corbaloc){ 
         CdmwPlatformMngt::DaemonController_var iniitdm;
         try {
             CORBA::Object_var obv = m_orb->string_to_object(corbaloc.c_str()); 
             iniitdm = CdmwPlatformMngt::DaemonController::_narrow(obv.in());
         }
         catch(...) {
            std::cerr << "Invalid Daemon Control URL" << std::endl;
         }
         
         return iniitdm;
    }
    
     OsSupport::RecursiveMutex* System_impl::get_consistency_status_recursive_mutex() 
    {
    return &m_check_consistency_status_recursive_mutex;    
    }

    bool System_impl::check_preemptive_application_status_change (CdmwPlatformMngt::ApplicationStatus old_st, CdmwPlatformMngt::ApplicationStatus new_st)
    {
        bool result = false;        
                                
        if (old_st != CdmwPlatformMngt::APPLICATION_STOPPED)
        {
            if (old_st == CdmwPlatformMngt::APPLICATION_INITIAL
                || old_st == CdmwPlatformMngt::APPLICATION_INITIALISING
                || old_st == CdmwPlatformMngt::APPLICATION_INITIALISED
                || old_st == CdmwPlatformMngt::APPLICATION_RUN_REQUEST)
            {
               if (new_st == CdmwPlatformMngt::APPLICATION_INITIAL
                   || new_st == CdmwPlatformMngt::APPLICATION_STOPPING
                   || new_st == CdmwPlatformMngt::APPLICATION_STOPPED)
               {
                   result = true;
               }
            }
        }
        else 
        {
          CDMW_INFO(Cdmw::Logging::Logger::Get_Logger(CdmwPlatformMngtBase::PFMMNG),
                  "System_impl::check_preemptive_application_status_change APPLICATION_STOPPED old status ");
        
          if (new_st == CdmwPlatformMngt::APPLICATION_INITIAL)
          {
          CDMW_INFO(Cdmw::Logging::Logger::Get_Logger(CdmwPlatformMngtBase::PFMMNG),
                  "System_impl::check_preemptive_application_status_change APPLICATION_STOPPED old status and APPLICATION_INITIAL new one");

              result = true;
          }
        }
        
        return result;
    }

     void System_impl::set_host_status_check_period(long check_period)
     {
         if (check_period > 0)
         {
             m_host_status_check_period = check_period;
         }
     }

     long System_impl::get_host_status_check_period()
     {
          return m_host_status_check_period;
     }

  }             // End namespace PlatformMngt
}               // End namespace Cdmw
