/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

#include "plugin/PluginConfigurationMngr.hpp"

#include <cstdlib>
#include <sstream>
#include <string>
#include <iostream>

#include <xercesc/dom/deprecated/DOM.hpp>
#include <xercesc/dom/deprecated/DOMParser.hpp>
#include <xercesc/util/PlatformUtils.hpp>
#include <xercesc/sax/ErrorHandler.hpp>
#include <xercesc/sax/SAXParseException.hpp>

#include "common/Assert.hpp"
#include "ossupport/OS.hpp"
#include "orbsupport/CORBA.hpp"
#include "SystemMngt/idllib/CdmwPlatformMngtCommon.stub.hpp"

using namespace xercesc;

namespace Cdmw
{
namespace PlatformMngt
{

bool PluginConfigurationMngr::m_initialised = false;

class  ErrorReporter : public ErrorHandler
{
public:
    ErrorReporter()
    {
    }

    ~ErrorReporter()
    {
    } 

    void warning(const SAXParseException& ex) { 
        throw ex; 
    }

    void error(const SAXParseException& ex)
    {
        throw ex;
    }

    void fatalError(const SAXParseException& ex)
    {
        throw ex;
    }

    void resetErrors() 
    { 
    }
};

//
// Returns the value associated to an Element node
//  
static inline PluginInfo get_value(const DOM_Node& node)
{
   // Get attributes 'name' and 'args'
   // search the specified attribute
   DOM_NamedNodeMap attributes = node.getAttributes();
   std::string name("");
   DOM_Node nameAttrib = attributes.getNamedItem("name");
   if (!nameAttrib.isNull()) {
      // attribute found
      DOMString s = nameAttrib.getNodeValue();
      char * c_str = s.transcode();
      name = c_str;
      delete [] c_str;
   }
   // Get args attribute
   std::string args("");
   DOM_Node argsAttrib = attributes.getNamedItem("args");
   if (!argsAttrib.isNull()) {
      // attribute found
      DOMString s = argsAttrib.getNodeValue();
      char * c_str = s.transcode();
      args = c_str;
      delete [] c_str;
   }
   // Get plugin path
   DOMString s = node.getFirstChild().getNodeValue();
   char* c_str = s.transcode();
   std::string path(c_str);
   
   // trim the string
   path.erase(path.find_last_not_of("\t\n ")+1);
   path.erase(0, path.find_first_not_of("\t\n "));
    
   delete [] c_str;
   return PluginInfo(name,args,path);
}


// std::string get_value(const DOM_Node& node)
// {
//     DOMString s = node.getFirstChild().getNodeValue();
//     char* c_str = s.transcode();
//     std::string str(c_str);
    
//     // trim the string
//     str.erase(str.find_last_not_of("\t\n ")+1);
//     str.erase(0, str.find_first_not_of("\t\n "));
    
//     delete [] c_str;
//     return str;
// }


PluginConfigurationMngr::PluginConfigurationMngr()
{
}


void PluginConfigurationMngr::parse(const std::string& xml_conf, bool validate)
    throw(OutOfMemoryException, PluginConfigurationException)
{
    if (xml_conf == "") {
        CDMW_THROW2( PluginConfigurationException, "Plugin configuration file name is empty!", 0 );
    }
    
    if( !Cdmw::OsSupport::OS::file_exists(xml_conf.c_str()) ) {
        CDMW_THROW2( PluginConfigurationException, std::string( "file \"" ) + xml_conf + "\" not found", 0 );    
    }

    try 
    {   
        DOMParser* parser = NULL;

        // Xerces initialisation must occur only once
        XMLPlatformUtils::Initialize();
        m_initialised = true;
        
        // We parse the configuration file
        parser = new DOMParser();
        if (validate)
        {
            parser->setValidationScheme(DOMParser::Val_Always);
            parser->setDoNamespaces(false);
        }

        ErrorReporter errorReporter;
        parser->setErrorHandler( &errorReporter );
        parser->parse( xml_conf.c_str() );
        DOM_Document doc = parser->getDocument();
        
        // We build a TreeWalker to skip all comment and automatically resolve
        // the entities
        DOM_TreeWalker walker = doc.createTreeWalker(doc,
            DOM_NodeFilter::SHOW_ELEMENT |
            DOM_NodeFilter::SHOW_ATTRIBUTE |
            DOM_NodeFilter::SHOW_TEXT |
            DOM_NodeFilter::SHOW_DOCUMENT_TYPE,
            NULL, true);

        // The first child is the DTD definition of the XML data, we skip it
        walker.firstChild();
      
        // We obtain the "CdmwPlatformMngtDaemon" node
        walker.nextSibling();

        // Each element configuration is under the root node,
        // so we get the first one
        DOM_Node node = walker.firstChild();

        // We can iter on each configuration element
        m_pluginInfos.clear();
        while (node != NULL) 
        {
            if (node.getNodeName().equals("Plugin-location"))
            {
                m_pluginInfos.push_back(get_value(node));
            }

            // Next Element to be processed
            node = walker.nextSibling();
        }

        parser->reset();
        delete parser;
    }
    catch(const SAXParseException& ex)
    {
        DOMString s = ex.getMessage();
        char* c_str = s.transcode();
        std::string msg(c_str);
        delete [] c_str;
        CDMW_THROW2(PluginConfigurationException, msg, ex.getLineNumber());
    }
    catch(const XMLException& ex)
    {
        DOMString s = ex.getMessage();
        char* c_str = s.transcode();
        std::string msg(c_str);
        delete [] c_str;
        CDMW_THROW2(PluginConfigurationException, msg, ex.getSrcLine());
    }
    catch ( std::bad_alloc& )
    {
        CDMW_THROW(OutOfMemoryException);
    }
}

PluginConfigurationMngr::~PluginConfigurationMngr()
{
}

std::vector<PluginInfo> PluginConfigurationMngr::getPluginInfos() 
{
    return m_pluginInfos;
}      

} // End namespace PlatformMngt
} // End namespace Cdmw

