/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

#include "plugin/PluginManager.hpp"

#include "plugin/PluginConfigurationMngr.hpp"
#include <string>
#include <vector>
#include <iostream>
#include <sstream>

namespace Cdmw
{
namespace PlatformMngt
{

PluginManager::PluginManager(CdmwPlatformMngt::System_ptr system,
                             std::string system_port)
   : m_status(INITIAL), m_alreadysetup(false),
     m_system_port(system_port)
{
    m_system = CdmwPlatformMngt::System::_duplicate(system);
}

PluginManager::~PluginManager()
{
}

bool PluginManager::set_up_all(std::string conf_filename)
{
    if ( m_status != INITIAL ) return false;
    
    bool ret = true;
    m_err_desc.clear();
    try
    {
        pcm.parse(conf_filename, true);
        std::vector<PluginInfo> infos = pcm.getPluginInfos();
        m_plugins.reserve(infos.size());
        std::vector<PluginInfo>::iterator i;
        
        // Creates a PluginThread for each path found. 
        // returns true only if all setUp() returns true
        for (i=infos.begin(); i!=infos.end(); i++) {
            PluginThread* pt = new PluginThread(m_system.in(), 
                                                (*i).m_path,
                                                (*i).m_name,
                                                (*i).m_args,
                                                m_system_port); 

            if (pt->set_up()) {
                m_plugins.push_back(pt);
            }
            else {
                m_err_desc.push_back("Error loading plugin \"" + (*i).m_path + "\": file doesn't exist or it's not a valid plugin. " + (pt->get_error_description()).c_str() );
               
                delete pt;
                ret = false;
            }
        }
    }
    catch(const Cdmw::PlatformMngt::PluginConfigurationException& e )
    {
        std::ostringstream oss;
        oss << "Error parsing \"" << conf_filename << "\" at line: " << e.getLineNumber() 
            << " error: " << e.getMessage() << std::ends;
        m_err_desc.push_back(oss.str());
        return false;
    }
    catch(const Cdmw::OutOfMemoryException& e )
    {
        m_err_desc.push_back("Out of memory during plugin loading");
        return false;
    }    
    catch( ... )
    {
        m_err_desc.push_back("Unexpected exception during plugin loading");
        return false;
    }
    
    m_alreadysetup = true;
    m_status = SETUPCALLED;
    return ret;
}

bool PluginManager::run_all()
{
    if ( m_status != SETUPCALLED ) return false;
    
    bool ret = true;
    m_err_desc.clear();
    std::vector<PluginThread*>::iterator i;
    for (i=m_plugins.begin(); i!=m_plugins.end(); i++) {
        PluginThread* pt = static_cast<PluginThread*>(*i);
        try {
            pt->start();
        }
        catch ( ... ) 
        {
            ret = false;
            m_err_desc.push_back("Error during \"" + pt->get_plugin_path() + "\" plugin run()");
            if (!pt->tear_down()) {
                m_err_desc.push_back("Error during \"" + pt->get_plugin_path() + "\" plugin tear_down()");
            }        
        }
    }
    
    m_status = RUNCALLED;
    return ret;
}

bool PluginManager::tear_down_all()
{
    if ( !(m_status == RUNCALLED || m_status == SETUPCALLED) ) return false;
    
    std::vector<PluginThread*>::iterator i;
    bool ret = true;
    for (i=m_plugins.begin(); i!=m_plugins.end(); i++) {
        PluginThread* pt = static_cast<PluginThread*>(*i);
        if (!pt->tear_down()) {
            ret = false;
            m_err_desc.push_back("Error during \"" + pt->get_plugin_path() + "\" plugin tear_down()");
        }
    }  
    
    m_status = TEARDOWNCALLED;
    return ret;
}

std::vector<std::string> PluginManager::get_error_descriptions()
{
    return m_err_desc;
}

}
}


