/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

#include "testplugin/TestPlugin.hpp"

#include "plugin/PluginConfigurationMngr.hpp"
#include "plugin/Plugin.hpp"
#include "plugin/PluginThread.hpp"
#include "plugin/PluginManager.hpp"

#include <vector>
#include <algorithm>
#include <string>

CPPUNIT_TEST_SUITE_REGISTRATION( TestPlugin ) ;

class FakeSystem : public CdmwPlatformMngt::System
{
public:
    int id;
}; 

TestPlugin::TestPlugin(  ) 
{
}

TestPlugin::~TestPlugin()
{
}

void TestPlugin::test_xmlparser()
{
    TEST_INFO( "Testing xml configuration file parser" );
    
    try
    {
       using namespace Cdmw::PlatformMngt;
        TEST_INFO( "Parse a conf file with one plugin path and get its name" );
        PluginConfigurationMngr pcm;
        pcm.parse("PluginConf_01.xml", true);
        std::vector<PluginInfo> infos = pcm.getPluginInfos();
        CPPUNIT_ASSERT( (infos.size() == 1) );
        std::cerr << "String: " << infos[0].m_path << std::endl;
        //CPPUNIT_ASSERT( !(paths[0].compare("plugin.so")) );
        
        TEST_INFO( "Parse a conf file with zero plugin paths" );
        pcm.parse("PluginConf_02.xml", true);
        infos = pcm.getPluginInfos();
        CPPUNIT_ASSERT((infos.size() == 0));

        TEST_INFO( "Parse a conf file with 3 plugin path and get their name" );
        pcm.parse("PluginConf_03.xml", true);
        infos = pcm.getPluginInfos();
        CPPUNIT_ASSERT((infos.size() == 3));
        CPPUNIT_ASSERT(!(infos[0].m_path.compare("plugin1.so")));
        CPPUNIT_ASSERT(!(infos[1].m_path.compare("plugin2.so")));
        CPPUNIT_ASSERT(!(infos[2].m_path.compare("plugin3.so")));        
    }
    catch(const Cdmw::PlatformMngt::PluginConfigurationException& e )
    {
        std::cerr << "Line: " << e.getLineNumber() << " Error: " << e.getMessage() << std::endl;
        CPPUNIT_ASSERT(false);
    }
    catch( const Cdmw::Exception& e )
    {
        std::cerr << e.what() << std::endl;
        CPPUNIT_ASSERT(false);
    }
    catch( const std::exception& e )
    {
        std::cerr << "std::exception: " << e.what() << std::endl;
        CPPUNIT_ASSERT(false);
    }
    catch( ... )
    {
        std::cerr << "Unexpected exception" << std::endl;
        CPPUNIT_ASSERT(false);
    }    
}

void TestPlugin::test_pluginthread()
{
    TEST_INFO( "Testing plugin handling as a thread" );
    
    TEST_INFO( "Testing plugin thread with not-existing library" );
    Cdmw::PlatformMngt::PluginThread pt1(NULL, "not-existing");
    CPPUNIT_ASSERT( (!(pt1.set_up())) );
    
    try {
        TEST_INFO( "Testing the invocation of plugin member functions" );
        
        // Load the library in common with the plugin 
        void *handle;
        std::vector<int> (*GetSetUpCallers)();
        std::vector<int> (*GetRunCallers)();
        std::vector<int> (*GetTearDownCallers)();
        CdmwPlatformMngt::System_ptr (*GetSystemPointer)();
        void (*Reset)();
        
        
        char *error;
        handle = dlopen ("libcdmwMyPluginTester.so", RTLD_LAZY|RTLD_GLOBAL);
        if (!handle) {
            std::cerr << "Error loading dynamic library: " << dlerror() << std::endl;
            CPPUNIT_ASSERT( false );
        }
            
        dlerror();    // Clear any existing error 
        *(void **) (&GetSetUpCallers) = dlsym(handle, "GetSetUpCallers");
        *(void **) (&GetRunCallers) = dlsym(handle, "GetRunCallers");
        *(void **) (&GetTearDownCallers) = dlsym(handle, "GetTearDownCallers");
        *(void **) (&GetSystemPointer) = dlsym(handle, "GetSystemPointer");
        *(void **) (&Reset) = dlsym(handle, "Reset");
        
        if ((error = dlerror()) != NULL)  {
            std::cerr << "Error loading functions: " << dlerror() << std::endl;
            CPPUNIT_ASSERT( false );
        }
        
        // Reset all counters
        (*Reset)();
        
        FakeSystem* sys_pointer = new FakeSystem();
        sys_pointer->id = 23;
        Cdmw::PlatformMngt::PluginThread pt2(sys_pointer, "libcdmwMyPlugin.so");
                
        std::string path = pt2.get_plugin_path();
        std::cerr << "Plugin path: " << path << std::endl;         
        CPPUNIT_ASSERT(path == "libcdmwMyPlugin.so");
        
        // Check that Plugin functions have not been called yet by PluginThread
        std::vector<int> callers; 
        callers = (*GetSetUpCallers)();
        CPPUNIT_ASSERT( callers.size() == 0 );
        callers = (*GetRunCallers)();
        CPPUNIT_ASSERT( callers.size() == 0 );
        callers = (*GetTearDownCallers)();
        CPPUNIT_ASSERT( callers.size() == 0 );
        sys_pointer = dynamic_cast<FakeSystem*>((*GetSystemPointer)());
        CPPUNIT_ASSERT( sys_pointer == NULL );
        
        // Now call functions of PluginThread..
        CPPUNIT_ASSERT( pt2.set_up() );  
        callers = (*GetSetUpCallers)();
        CPPUNIT_ASSERT( ((find(callers.begin(), callers.end(), 1)) != (callers.end())) );  
        
        sys_pointer = dynamic_cast<FakeSystem*>((*GetSystemPointer)());
        CPPUNIT_ASSERT( (sys_pointer->id) == 23 );
               
        pt2.start();
        sleep(2); // sleep a while to wait for the thread completion
        callers = (*GetRunCallers)();
        CPPUNIT_ASSERT( ((find(callers.begin(), callers.end(), 1)) != (callers.end())) );  

        CPPUNIT_ASSERT( pt2.tear_down() );
        callers = (*GetTearDownCallers)();
        CPPUNIT_ASSERT( ((find(callers.begin(), callers.end(), 1)) != (callers.end())) );  

        dlclose(handle);
    }
    catch ( std::exception& e )
    {
        std::cerr << "std::exception: " << e.what() << std::endl;
        CPPUNIT_ASSERT( false );    
    }
}

void TestPlugin::test_pluginmanager()
{
    TEST_INFO( "Testing plugin manager with 0 plugins" );
    Cdmw::PlatformMngt::PluginManager pm3(NULL);
    CPPUNIT_ASSERT(pm3.set_up_all("PluginConf_02.xml"));
    
    TEST_INFO( "Testing a double call to set_up_all()" );
    CPPUNIT_ASSERT(!pm3.set_up_all("PluginConf_02.xml"));
    CPPUNIT_ASSERT(pm3.run_all());
    CPPUNIT_ASSERT(!pm3.run_all());
    CPPUNIT_ASSERT(pm3.tear_down_all());
    CPPUNIT_ASSERT(!pm3.tear_down_all());

    TEST_INFO( "Testing plugin manager with a not-existing library" );
    Cdmw::PlatformMngt::PluginManager pm(NULL);
    CPPUNIT_ASSERT(!pm.set_up_all("Not-Existing"));

    TEST_INFO( "Testing plugin manager with a void string" );
    Cdmw::PlatformMngt::PluginManager pm1(NULL);
    CPPUNIT_ASSERT(!pm1.set_up_all(""));
    
    TEST_INFO( "Testing plugin manager with a not-valid conf file" );
    Cdmw::PlatformMngt::PluginManager pm2(NULL);
    CPPUNIT_ASSERT(!pm2.set_up_all("PluginConf_04.xml"));
    std::vector<std::string> errors = pm2.get_error_descriptions();
    std::cerr << errors[0] << std::endl;
    CPPUNIT_ASSERT(errors.size() == 1);
}

void TestPlugin::test_pluginmanager2()
{
    // Load the library in common with the plugin 
    void *handle;
    
    std::vector<int> (*GetSetUpCallers)();
    std::vector<int> (*GetRunCallers)();
    std::vector<int> (*GetTearDownCallers)();
    void (*Reset)();
        
    char *error;
    handle = dlopen ("libcdmwMyPluginTester.so", RTLD_LAZY|RTLD_GLOBAL);
    if (!handle) {
        std::cerr << "Error loading dynamic library: " << dlerror() << std::endl;
        CPPUNIT_ASSERT( false );
    }
        
    dlerror();    // Clear any existing error 
    *(void **) (&GetSetUpCallers) = dlsym(handle, "GetSetUpCallers");
    *(void **) (&GetRunCallers) = dlsym(handle, "GetRunCallers");
    *(void **) (&GetTearDownCallers) = dlsym(handle, "GetTearDownCallers");
    *(void **) (&Reset) = dlsym(handle, "Reset");
        
    if ((error = dlerror()) != NULL)  {
        std::cerr << "Error loading functions: " << dlerror() << std::endl;
        CPPUNIT_ASSERT( false );
    }
    
    // Reset all counters
    (*Reset)();
        
    TEST_INFO( "Testing plugin manager with 2 valid plugins" );
    Cdmw::PlatformMngt::PluginManager pm3(NULL);
    
    // Check that Plugin functions have not been called yet by PluginManager
    std::vector<int> callers; 
    callers = (*GetSetUpCallers)();
    CPPUNIT_ASSERT( callers.size() == 0 );
    callers = (*GetRunCallers)();
    CPPUNIT_ASSERT( callers.size() == 0 );
    callers = (*GetTearDownCallers)();
    CPPUNIT_ASSERT( callers.size() == 0 );
      
    CPPUNIT_ASSERT(pm3.set_up_all("PluginConf_05.xml"));
    // set_up and get_object should have been called
    callers = (*GetSetUpCallers)();
    std::cerr << "callers.size(): " << callers.size() << std::endl;
    CPPUNIT_ASSERT( callers.size() == 2 );
    CPPUNIT_ASSERT( ((find(callers.begin(), callers.end(), 1)) != (callers.end())) );   // id == 1
    CPPUNIT_ASSERT( ((find(callers.begin(), callers.end(), 2)) != (callers.end())) );   // id == 2
    
    CPPUNIT_ASSERT(pm3.run_all());
    sleep(2); // sleep a while to wait for the thread completion
    // run() should have been called
    callers = (*GetRunCallers)();
    CPPUNIT_ASSERT( callers.size() == 2 );
    CPPUNIT_ASSERT( ((find(callers.begin(), callers.end(), 1)) != (callers.end())) );   // id == 1
    CPPUNIT_ASSERT( ((find(callers.begin(), callers.end(), 2)) != (callers.end())) );   // id == 2
    
    CPPUNIT_ASSERT(pm3.tear_down_all());
    // tear_down() should have been called
    callers = (*GetTearDownCallers)();
    CPPUNIT_ASSERT( callers.size() == 2 );
    CPPUNIT_ASSERT( ((find(callers.begin(), callers.end(), 1)) != (callers.end())) );   // id == 1
    CPPUNIT_ASSERT( ((find(callers.begin(), callers.end(), 2)) != (callers.end())) );   // id == 2
    
    // Reset all counters
    (*Reset)();
    
    
    TEST_INFO( "Testing plugin manager with some valid and not-valid plugins" );
    Cdmw::PlatformMngt::PluginManager pm4(NULL);
    CPPUNIT_ASSERT(!(pm4.set_up_all("PluginConf_06.xml")));
    
    std::vector<std::string> errors = pm4.get_error_descriptions();
    CPPUNIT_ASSERT( errors.size() == 2 );
    std::cerr << errors[0] << std::endl;
    std::cerr << errors[1] << std::endl;
    CPPUNIT_ASSERT( errors[0] == "Error loading plugin \"not-existing\": file doesn't exist or it's not a valid plugin. " );

    if ( (std::string::npos != errors[1].find("file doesn't exist or it's not a valid plugin", 0) ) &&  (std::string::npos != errors[1].find ("undefined symbol: cdmw_dll_create", 0) ) ) CPPUNIT_ASSERT(true);
    else CPPUNIT_ASSERT(false);
            
    // set_up should have been called
    callers = (*GetSetUpCallers)();
    std::cerr << "callers.size(): " << callers.size() << std::endl;
    CPPUNIT_ASSERT( callers.size() == 2 );
    CPPUNIT_ASSERT( ((find(callers.begin(), callers.end(), 1)) != (callers.end())) );   // id == 1
    CPPUNIT_ASSERT( ((find(callers.begin(), callers.end(), 2)) != (callers.end())) );   // id == 2
    
    CPPUNIT_ASSERT(pm4.run_all());
    sleep(2); // sleep a while to wait for the thread completion
    // run() should have been called
    callers = (*GetRunCallers)();
    CPPUNIT_ASSERT( callers.size() == 2 );
    CPPUNIT_ASSERT( ((find(callers.begin(), callers.end(), 1)) != (callers.end())) );   // id == 1
    CPPUNIT_ASSERT( ((find(callers.begin(), callers.end(), 2)) != (callers.end())) );   // id == 2
    
    CPPUNIT_ASSERT(pm4.tear_down_all());
    // tear_down() should have been called
    callers = (*GetTearDownCallers)();
    CPPUNIT_ASSERT( callers.size() == 2 );
    CPPUNIT_ASSERT( ((find(callers.begin(), callers.end(), 1)) != (callers.end())) );   // id == 1
    CPPUNIT_ASSERT( ((find(callers.begin(), callers.end(), 2)) != (callers.end())) );   // id == 2
    
    // Reset all counters
    (*Reset)();
    
    
    //CPPUNIT_ASSERT(false);    
}

