/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

// private include files
#include "SystemMngt/tools/BaseIssuerManager.hpp"

// include files
#include "Foundation/common/Exception.hpp"
#include <Foundation/common/Assert.hpp>
#include <Foundation/common/MacroDefs.hpp>
#include "Foundation/osthreads/ReaderLockGuard.hpp"
#include "Foundation/osthreads/WriterLockGuard.hpp"
#include "Foundation/osthreads/MutexGuard.hpp"

#include <Foundation/logging/Logger.hpp>
#include <Foundation/logging/FunctionLogger.hpp>

#include <Foundation/logging/LogManager.hpp>

#include <SystemMngt/idllib/CdmwPlatformMngtBase.stub.hpp>

#include <memory>
#include <algorithm>

namespace Cdmw {
namespace PlatformMngt 
{

/**
*Implementation notes: 
*<p>
*Portability issues:  none 
*<p>
*/

//
// Ctor
//    
BaseIssuerManager::BaseIssuerManager ()    
{
     m_mutexes_map.clear();
     m_issuers_map.clear();
}

//
// Dtor
//
BaseIssuerManager::~BaseIssuerManager()
    throw()
{
     {
         CDMW_WRITER_LOCK_GUARD(m_mutexes_lock);

         std::map<std::string, Cdmw::OsSupport::Mutex*>::iterator element_it = m_mutexes_map.begin();

         for (; element_it != m_mutexes_map.end(); element_it++)
         {
             delete (element_it->second);
         }

         m_mutexes_map.clear();

      }
      
      {
          CDMW_WRITER_LOCK_GUARD(m_issuers_lock);

          m_issuers_map.clear();
      }

}


//
// Accessor Singleton method
//
Cdmw::PlatformMngt::BaseIssuerManager* BaseIssuerManager::Instance()
{

    static Cdmw::PlatformMngt::BaseIssuerManager base_issuer_manager;
    return &base_issuer_manager;

}

bool BaseIssuerManager::attach_element (const std::string& attaching_element_path)
    throw(CORBA::SystemException, Cdmw::Exception)
{
   bool result = false;
   bool mutex_insertion = false;
   bool issuer_insertion = false;

   try 
   {
     
      {
          CDMW_WRITER_LOCK_GUARD(m_mutexes_lock);
         
          std::map<std::string, Cdmw::OsSupport::Mutex*>::iterator element_found = m_mutexes_map.find(attaching_element_path);
         
          if (element_found == m_mutexes_map.end())
          {         

              Cdmw::OsSupport::Mutex* mutex = new Cdmw::OsSupport::Mutex();

              m_mutexes_map.insert( std::pair<std::string, Cdmw::OsSupport::Mutex*>(
                     attaching_element_path, mutex));

              mutex_insertion = true;
          }
      }
      
      {
          CDMW_WRITER_LOCK_GUARD(m_issuers_lock);
         
          std::map<std::string, std::string>::iterator element_found = m_issuers_map.find(attaching_element_path);
         
          if (element_found == m_issuers_map.end())
          {         
              m_issuers_map[attaching_element_path] = "";
              issuer_insertion = true;
          }      
      }      
   
      result = mutex_insertion && issuer_insertion;
   
   }
   catch (...)
   {
        CDMW_ERROR(Cdmw::Logging::Logger::Get_Logger(CdmwPlatformMngtBase::PFMMNG),
             "BaseIssuerManager::attach_element execution failed, unknown exception thrown ");

        throw  Cdmw::Exception (Cdmw::Exception::FATAL, 
            "BaseIssuerManager::attach_element unknown exception");

   }

   return result;
}
    
bool BaseIssuerManager::detach_element (const std::string& detaching_element_path)
     throw(CORBA::SystemException, Cdmw::Exception)
{
    bool result = false;
    bool mutex_deletion = false;
    bool issuer_deletion = false;
    
   try 
   {    
     
      {
          CDMW_WRITER_LOCK_GUARD(m_mutexes_lock);
         
          std::map<std::string, Cdmw::OsSupport::Mutex*>::iterator element_found = m_mutexes_map.find(detaching_element_path);
         
          if (element_found != m_mutexes_map.end())
          {         
              m_mutexes_map.erase(detaching_element_path);
              mutex_deletion = true;
          }
      }
      
      {
          CDMW_WRITER_LOCK_GUARD(m_issuers_lock);
         
          std::map<std::string, std::string>::iterator element_found = m_issuers_map.find(detaching_element_path);
         
          if (element_found != m_issuers_map.end())
          {         
              m_issuers_map.erase(detaching_element_path); 
              issuer_deletion = true;
          }      
      }      
   
      result = mutex_deletion && issuer_deletion;
           
   }
   catch (...)
   {
        CDMW_ERROR(Cdmw::Logging::Logger::Get_Logger(CdmwPlatformMngtBase::PFMMNG),
             "BaseIssuerManager::detach_element execution failed, unknown exception thrown ");

        throw  Cdmw::Exception (Cdmw::Exception::FATAL, 
            "BaseIssuerManager::detach_element unknown exception");

   }
        
    return result;

}
    
Cdmw::OsSupport::Mutex* BaseIssuerManager::get_element_issuer_mutex (const std::string& element_seed)
    throw(CORBA::SystemException, Cdmw::Exception)
{
   try 
   {             
      CDMW_READER_LOCK_GUARD(m_mutexes_lock);         
      
      std::map<std::string, Cdmw::OsSupport::Mutex*>::iterator element_found_it = m_mutexes_map.find(element_seed);                 
               
      if (!(element_found_it == m_mutexes_map.end()))
      {         
//          CDMW_INFO(Cdmw::Logging::Logger::Get_Logger(CdmwPlatformMngtBase::PFMMNG), " Mutex Found " << element_seed);
          CDMW_DEBUG(Cdmw::Logging::Logger::Get_Logger(CdmwPlatformMngtBase::PFMMNG),
             "ProcessIssuerManager::get_process_issuer_mutex called for "+element_seed);
      
      
          return (element_found_it->second);
      }
      else
      {
          //Throws an exception      
//          CDMW_INFO(Cdmw::Logging::Logger::Get_Logger(CdmwPlatformMngtBase::PFMMNG), " MutexNotFound " << element_seed);
                    
          Cdmw::Exception exc (Cdmw::Exception::FATAL, "Element not found in BaseIssuerManager");
          throw exc;
      }
   
   }
   catch (...)
   {
        CDMW_ERROR(Cdmw::Logging::Logger::Get_Logger(CdmwPlatformMngtBase::PFMMNG),
             "BaseIssuerManager::get_element_issuer_mutex execution failed, unknown exception thrown ");

        throw  Cdmw::Exception (Cdmw::Exception::FATAL, 
            "BaseIssuerManager::get_element_issuer_mutex unknown exception");

   }

}
//LC: Please ! Guard the issuer mutex first !
std::string BaseIssuerManager::get_command_issuer (const std::string& getting_issuer_seed)
     throw(CORBA::SystemException, Cdmw::Exception)
{

      bool locked = check_mutex_locked (getting_issuer_seed);

      if (!locked)
      {
         Cdmw::Exception exc (Cdmw::Exception::UNKNOWN, 
             "BaseIssuerManager mutex not locked !"+getting_issuer_seed);
         throw exc;
      }

      std::string returning = "";

      CDMW_READER_LOCK_GUARD(m_issuers_lock);
         
      std::map<std::string, std::string>::iterator element_found_it = m_issuers_map.find(getting_issuer_seed);
         
      if (element_found_it != m_issuers_map.end())
      {         
          returning = element_found_it->second;
      }
          
      return returning;
}

void BaseIssuerManager::set_issuer (const std::string& setting_issuer_seed, 
              const std::string& issuer_value)
    throw(CORBA::SystemException, Cdmw::Exception)
{     
      bool locked = check_mutex_locked (setting_issuer_seed);

      if (!locked)
      {
         Cdmw::Exception exc (Cdmw::Exception::UNKNOWN, 
             "BaseIssuerManager mutex not locked !"+setting_issuer_seed);
         throw exc;
      }

      CDMW_WRITER_LOCK_GUARD(m_issuers_lock);
         
      std::map<std::string, std::string>::iterator element_found_it = m_issuers_map.find(setting_issuer_seed);
         
      if (element_found_it != m_issuers_map.end())
      {         
          m_issuers_map[setting_issuer_seed] = issuer_value;
      }
          
}

bool BaseIssuerManager::check_mutex_locked (const std::string& searching_path)
    throw(CORBA::SystemException, Cdmw::Exception)
{
     bool locked = false;

    if (get_element_issuer_mutex(searching_path)->is_locked())
    {
        locked = true;
    }

     return locked;
}

} // End namespace PlatformMngt 
}  // End namespace Cdmw

