/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

// private include files
#include "SystemMngt/tools/SystemIssuerManager.hpp"

// include files

#include "Foundation/common/Assert.hpp"
#include "Foundation/osthreads/ReaderLockGuard.hpp"
#include "Foundation/osthreads/WriterLockGuard.hpp"
#include "Foundation/osthreads/MutexGuard.hpp"

#include <Foundation/logging/Logger.hpp>
#include <Foundation/logging/FunctionLogger.hpp>

#include <SystemMngt/idllib/CdmwPlatformMngtBase.stub.hpp>

#include <memory>
#include <algorithm>

using Cdmw::Logging::Logger;

namespace Cdmw {
namespace PlatformMngt 
{

/**
*Implementation notes: 
*<p>
*Portability issues:  none 
*<p>
*/

//
// Ctor
//    
SystemIssuerManager::SystemIssuerManager ()
{
    m_system_mutex = new Cdmw::OsSupport::Mutex();
}



//
// Dtor
//
SystemIssuerManager::~SystemIssuerManager()
    throw()
{
    delete m_system_mutex;
}


//
// Change the timeout value
//
Cdmw::PlatformMngt::SystemIssuerManager* SystemIssuerManager::Instance()
{
    static Cdmw::PlatformMngt::SystemIssuerManager SystemIssuer_manager;
    return &SystemIssuer_manager;
}


bool SystemIssuerManager::attach_element (const std::string& attaching_element_path)
    throw (Cdmw::Exception)
{
   bool result = false;

   try 
   {   
      {
          CDMW_WRITER_LOCK_GUARD(m_mutexes_lock);

          m_system_path = attaching_element_path;
      }

      {
          CDMW_WRITER_LOCK_GUARD(m_issuers_lock);
          m_issuer = "";
      
      }
      result = true;
   
   }
   catch (...)
   {
        CDMW_ERROR(Cdmw::Logging::Logger::Get_Logger(CdmwPlatformMngtBase::PFMMNG),
             "SystemIssuerManager::attach_element execution failed, unknown exception thrown ");

        throw  Cdmw::Exception (Cdmw::Exception::FATAL, 
            "SystemIssuerManager::attach_element execution failed, unknown exception thrown ");
   }

   return result;
}
    
bool SystemIssuerManager::detach_element ()
    throw (Cdmw::Exception)
{
    bool result = true;
    
    {
       CDMW_WRITER_LOCK_GUARD(m_mutexes_lock);

       m_system_path = "";
    }

    {
        CDMW_WRITER_LOCK_GUARD(m_issuers_lock);
        m_issuer = "";

    }

    return result;

}

Cdmw::OsSupport::Mutex* SystemIssuerManager::get_system_issuer_mutex ()
     throw (Cdmw::Exception)
{
   CDMW_READER_LOCK_GUARD(m_mutexes_lock);

//   CDMW_INFO(Cdmw::Logging::Logger::Get_Logger(CdmwPlatformMngtBase::PFMMNG),
//             "SystemIssuerManager::get_system_issuer_mutex");

   CDMW_DEBUG(Cdmw::Logging::Logger::Get_Logger(CdmwPlatformMngtBase::PFMMNG),
             "SystemIssuerManager::get_system_issuer_mutex");

   return m_system_mutex;
}

void SystemIssuerManager::set_issuer (const std::string& issuer_value)
     throw (Cdmw::Exception)
{
    bool locked = check_mutex_locked();

   CDMW_INFO(Cdmw::Logging::Logger::Get_Logger(CdmwPlatformMngtBase::PFMMNG),
             "SystemIssuerManager::set_issuer");

    if (!locked)
    {
       Cdmw::Exception exc (Cdmw::Exception::UNKNOWN, "SystemIssuerManager mutex not locked !");
       throw exc;
    }

    CDMW_WRITER_LOCK_GUARD(m_issuers_lock);

    m_issuer = issuer_value;
}

//LC: Please ! Guard the issuer mutex first !        
std::string SystemIssuerManager::get_command_issuer ()
    throw (Cdmw::Exception)
{
    bool locked = check_mutex_locked();

    if (!locked)
    {
       Cdmw::Exception exc (Cdmw::Exception::UNKNOWN, "SystemIssuerManager mutex not locked !");
       throw exc;    
    }

   CDMW_READER_LOCK_GUARD(m_issuers_lock);
   
   return m_issuer;
}

bool SystemIssuerManager::check_mutex_locked ()
     throw (Cdmw::Exception)
{
    bool locked = false;

    if (get_system_issuer_mutex()->is_locked())
    {
        locked = true;
    }

    return locked;
}

bool SystemIssuerManager::detach_element (const std::string& detaching_element_path)
                     throw(CORBA::SystemException, Cdmw::Exception)
{
    return detach_element();
}

Cdmw::OsSupport::Mutex* SystemIssuerManager::get_element_issuer_mutex (const std::string& element_seed)
            throw(CORBA::SystemException, Cdmw::Exception)
{
    return get_system_issuer_mutex();
}


} // End namespace PlatformMngt 
}  // End namespace Cdmw
