/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

#include "tools/TimeoutInserter.hpp"

#include <Foundation/common/MacroDefs.hpp>
#include "Foundation/osthreads/ReaderLockGuard.hpp"
#include "Foundation/osthreads/WriterLockGuard.hpp"
#include "Foundation/osthreads/MutexGuard.hpp"

#include <Foundation/logging/Logger.hpp>
#include <SystemMngt/idllib/CdmwPlatformMngtBase.stub.hpp>

#include <iostream>

using Cdmw::Logging::Logger;

namespace Cdmw
{
    namespace PlatformMngt
    {

     TimeoutInserter::TimeoutInserter ()
     {
         m_orb = CORBA::ORB::_nil();

         CDMW_WRITER_LOCK_GUARD(m_orb_attached_lock);
         m_is_orb_attached = false;
     }

     TimeoutInserter::~TimeoutInserter()
     {
         CORBA::release(m_orb);
     }

     //
     // Accessor Singleton method
     //
     Cdmw::PlatformMngt::TimeoutInserter* TimeoutInserter::Instance()
     {

         static Cdmw::PlatformMngt::TimeoutInserter timeout_inserter;
         return &timeout_inserter;

      }

      CORBA::Object_ptr TimeoutInserter::capsule_timeout (CORBA::Object_ptr obj,
                         unsigned long callTimeout_millisec)
      {

           if (is_orb_attached())
           {

               CORBA::PolicyList policy_list(1);
               policy_list.length (1);

               CORBA::ULong i = 0L;
               // Create the timeout policy
               if (callTimeout_millisec != 0) 
               {
                   CORBA::Any timeout;
                   // Timeout success
                   //Set the timeout value as a TimeBase::TimeT (100 nanoseconds units)
                   TimeBase::TimeT _timeout = ((unsigned long) (callTimeout_millisec * 1.0e4));
                   timeout <<= _timeout;

                   policy_list[i] = m_orb->create_policy( Messaging::RELATIVE_RT_TIMEOUT_POLICY_TYPE,
                                                timeout );

                   CORBA::Object_var capsule = CORBA::Object::_nil();
                   try
                   {
                       capsule = obj->_set_policy_overrides( policy_list, CORBA::SET_OVERRIDE );

                   }
                   catch (const CORBA::NO_IMPLEMENT&)
                   {
                       CDMW_DEBUG(Cdmw::Logging::Logger::Get_Logger(CdmwPlatformMngtBase::PFMMNG),
                           "Warning: no timeout set for CORBA object");
                       capsule = obj;
                   }
                   catch (const CORBA::Exception&)
                   {
                       CDMW_DEBUG(Cdmw::Logging::Logger::Get_Logger(CdmwPlatformMngtBase::PFMMNG),
                           "Warning: invalid CORBA object to set timeout");
                   }

                   if (CORBA::is_nil(capsule))
                   {
                       CDMW_DEBUG(Cdmw::Logging::Logger::Get_Logger(CdmwPlatformMngtBase::PFMMNG),
                           "Capsule Object nill");
                   }

                   return capsule._retn();
               }
          }
          return obj;
      }

      void TimeoutInserter::attachOrb(CORBA::ORB_ptr attachingOrb)
      {
          m_orb = CORBA::ORB::_duplicate(attachingOrb);

          CDMW_WRITER_LOCK_GUARD(m_orb_attached_lock);

          m_is_orb_attached = true;
      }

      CORBA::Object_ptr  TimeoutInserter::capsule_timeout(CORBA::ORB_ptr orb, 
                             CORBA::Object_ptr obj,
                         unsigned long callTimeout_millisec)
     {
      CORBA::PolicyList policy_list(1);
      policy_list.length (1);

      CORBA::ULong i = 0L;
      // Create the timeout policy
      if (callTimeout_millisec != 0) {
          CORBA::Any timeout;
          // Timeout success
          //Set the timeout value as a TimeBase::TimeT (100 nanoseconds units)
          TimeBase::TimeT _timeout = ((TimeBase::TimeT) (callTimeout_millisec * 10000));
          timeout <<= _timeout;

          policy_list[i] = orb->create_policy( Messaging::RELATIVE_RT_TIMEOUT_POLICY_TYPE,
                                               timeout );

          CORBA::Object_var capsule = CORBA::Object::_nil();
          try
          {
              capsule = obj->_set_policy_overrides( policy_list, CORBA::SET_OVERRIDE );

          }
          catch (const CORBA::NO_IMPLEMENT&)
          {
              CDMW_DEBUG(Cdmw::Logging::Logger::Get_Logger(CdmwPlatformMngtBase::PFMMNG),
                  "Warning: no timeout set for CORBA object");
              capsule = obj;
          }
          catch (const CORBA::Exception&)
          {
              CDMW_DEBUG(Cdmw::Logging::Logger::Get_Logger(CdmwPlatformMngtBase::PFMMNG),
                  "Warning: invalid CORBA object to set timeout");

          }
          if (CORBA::is_nil(capsule.in()))
          {
              CDMW_DEBUG(Cdmw::Logging::Logger::Get_Logger(CdmwPlatformMngtBase::PFMMNG),
                  "Capsule Object nill");
          }

          return capsule._retn();

      }

         return obj;
     }

     bool TimeoutInserter::is_orb_attached()
     {
         CDMW_READER_LOCK_GUARD(m_orb_attached_lock);
         return m_is_orb_attached;
     }

   } //End PlatformMngt Namespace
};//End Cdmw Namespace
