/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#include <iostream>
#include <iomanip>
#include <typeinfo>
#include <fstream>


#include "Foundation/ossupport/OS.hpp"
#include "Foundation/orbsupport/OrbSupport.hpp"
#include "Foundation/orbsupport/StrategyList.hpp"
#include "Foundation/common/Options.hpp"
#include "Foundation/common/Exception.hpp"

#include "SystemMngt/platformvaluetypes/ProcessStatusChange_impl.hpp"
#include "SystemMngt/platformvaluetypes/EntityStatusChange_impl.hpp"

#include "idllib/CdmwPlatformMngtSystem.stub.hpp"
#include "test_Q-022/ProcObserver_impl.hpp"



namespace Cdmw
  {

  namespace PlatformMngt
    {
    
    ProcObserver_impl::ProcObserver_impl () {
    
        m_sys_changes_queue = new std::deque<Sys_event>;
        m_appl_changes_queue = new std::deque<Appl_event>;
        m_proc_changes_queue = new std::deque<Proc_event>;
    }
    
    ProcObserver_impl::~ProcObserver_impl () {
         
         if (m_sys_changes_queue != 0) {
         
             sys_changes_queue_flush ();
             
             delete m_sys_changes_queue;
         }

         if (m_appl_changes_queue != 0) {

             app_changes_queue_flush ();
    
             delete m_appl_changes_queue;
         }

         if (m_proc_changes_queue != 0) {

             pro_changes_queue_flush ();
    
             delete m_proc_changes_queue;
         }

    }
    void  ProcObserver_impl::notify(CdmwPlatformMngt::Event* an_event)
        throw( CORBA::SystemException ) 
    {
       
        try 
        {

            if (an_event != 0) 
            {
              
                const char* element_path = an_event->element_path();

                CdmwPlatformMngt::ProcessStatusChange* prc_chg = 
                    dynamic_cast<CdmwPlatformMngt::ProcessStatusChange*> (an_event);
         
                if ((prc_chg!=0))
                {                                                
                    m_pro_mutex.lock();
                  
                    Proc_event proc_event;

                    proc_event.proc_status = prc_chg->process_status();

                    proc_event.proc_path = element_path;

                    m_proc_changes_queue->push_back(proc_event);
                  
                    m_pro_mutex.unlock();
                            
                }
                
                CdmwPlatformMngt::ApplicationStatusChange* app_chg =
                    dynamic_cast<CdmwPlatformMngt::ApplicationStatusChange*> (an_event);

                if (app_chg!=0)
                {                                                
                    m_app_mutex.lock();

                    Appl_event appl_event;
                  
                    appl_event.appl_status= app_chg->application_status();

                    appl_event.appl_path = element_path;

                    m_appl_changes_queue->push_back(appl_event);                          

                    m_app_mutex.unlock();

                }


                CdmwPlatformMngt::SystemStatusChange* sys_chg = 
                    dynamic_cast<CdmwPlatformMngt::SystemStatusChange*> (an_event);
 
                if ( (sys_chg!=0))
                {                                               
                    m_sys_mutex.lock();

                    Sys_event sys_event;
                  
                    sys_event.sys_status= sys_chg->system_status();
                    
                    sys_event.sys_path= element_path;

                    m_sys_changes_queue->push_back(sys_event);
               
                    m_sys_mutex.unlock();

                }                            
            }
           
        } 
        catch (const std::exception& ex) 
        {

              std::cerr << " Process observer exception  " << ex.what();
        }
    }
    
    bool ProcObserver_impl::check_sys_stati_sequence(std::string system_name,         
             std::vector<CdmwPlatformMngt::SystemStatus> sys_stati) throw( CORBA::SystemException ) {

             std::string system_path = system_name+".sys";                      
                            
             bool result = false;
             unsigned int stati_matches = 0;    
             unsigned int counter = 0;

             try {                                    
             
                  m_sys_mutex.lock();
       
                  if (!m_sys_changes_queue->empty ()) {

                       Sys_event sys_event = m_sys_changes_queue->front();                       

                       while (counter < m_sys_changes_queue->size() && !result) {

                           sys_event = m_sys_changes_queue->at(counter);
                           counter++;
                                                                                                     
                           if (system_path.compare(sys_event.sys_path) == 0) {
                                       
                                if (sys_event.sys_status == sys_stati.at(stati_matches)) {

                                       stati_matches++;                                           
                                 }
                            }
                                                                                                            
                         if (stati_matches == sys_stati.size()) {
                               result = true;
                         }
                                                                               
              }                             
                  } else {
                      TEST_INFO("\n Warning: Empty event queue ");
                  }
                  
 
                  m_sys_mutex.unlock();
        
             } catch (const std::exception& ex) {
                
                  if (m_sys_mutex.is_locked()){
                    
                         m_sys_mutex.unlock();
                  }

                  std::cerr << "\n Process observer exception  " << ex.what();
             }               

             return result;
    }
        
    bool ProcObserver_impl::check_appl_stati_sequence(std::string system_name, 
             std::string application_name, std::vector<CdmwPlatformMngt::ApplicationStatus> appl_stati) throw( CORBA::SystemException ) {

            std::string application_path = system_name+".sys/"+application_name+".appl";

            TEST_INFO("Application path: "+application_path);

            bool result = false;
            unsigned int stati_matches = 0;             
            unsigned int counter = 0;

            try {                                    
             
                 m_app_mutex.lock();
       
                if (!m_appl_changes_queue->empty ()) {

                        Appl_event appl_event = m_appl_changes_queue->front();
                        
                        std::cerr << "Application queue size: " << m_appl_changes_queue->size() << "\n";

                        while (counter < m_appl_changes_queue->size() && !result) {

                           appl_event = m_appl_changes_queue->at(counter);   
                           counter++;           

                         std::cerr << "Current application path " << appl_event.appl_path << "\n";
                                                   
                           if (application_path.compare(appl_event.appl_path) == 0) {
                                                                   
                                 if (appl_event.appl_status == 
                                       appl_stati.at(stati_matches)) {
                                       stati_matches++;                                            
                                 }                                      
                            }
                              
                           if (stati_matches == appl_stati.size()) {
                                    result = true;
                           }
                                       
               }                             
                 } else {
                    TEST_INFO("\n Warning: Empty event queue ");
                 }
                   
                 m_app_mutex.unlock();
                 
             } catch (const std::exception& ex) {
                    
                    if (m_app_mutex.is_locked()){
                    
                        m_app_mutex.unlock();
                    }

                    std::cerr << "\n Process observer exception  " << ex.what();
             }

             return result;
    }

    bool ProcObserver_impl::check_proc_stati_sequence(std::string system_name, std::string application_name, std::string host_name, std::string process_name, std::vector<CdmwPlatformMngt::ProcessStatus> proc_stati) 
             throw( CORBA::SystemException ) {

             std::string process_path = system_name+".sys/"+application_name+".appl/"
                 +host_name+".host/"+process_name+".proc";

            TEST_INFO("Process path: "+process_path);

            bool result = false;
            unsigned int stati_matches = 0; 
            unsigned int counter = 0;

            try {                                    
             
                  m_pro_mutex.lock();
       
                  if (!m_proc_changes_queue->empty ()) {

                     Proc_event proc_event = m_proc_changes_queue->front();       

                     std::cerr << "Process queue size: " << m_proc_changes_queue->size() << "\n";
  
                     while (counter < m_proc_changes_queue->size() && !result) {

                         proc_event = m_proc_changes_queue->at(counter);
                         counter++;

                         std::cerr << "Current process path " << proc_event.proc_path << "\n";

                         if (process_path.compare(proc_event.proc_path) == 0) {  
                                                                     
                                if (proc_event.proc_status == proc_stati.at(stati_matches)) {
                                        stati_matches++;
                              }
                                     
                         }    
                              
                       if (stati_matches == proc_stati.size()) {
                                 result = true;
                       }
                                                             
               }                             
                } else {
                    TEST_INFO("\n Warning: Empty event queue ");
                }
                   
                m_pro_mutex.unlock();

             } catch (const std::exception& ex) {
                    
                    if (m_pro_mutex.is_locked()){
                    
                        m_pro_mutex.unlock();
                    }

                    std::cerr << "\n Process observer exception  " << ex.what();
             }               

             return result;
    }   
     
    void ProcObserver_impl::all_queue_flush () throw( CORBA::SystemException ){

             sys_changes_queue_flush ();
             app_changes_queue_flush ();
             pro_changes_queue_flush ();
             
    }
    void ProcObserver_impl::sys_changes_queue_flush () 
            throw( CORBA::SystemException )   {            
     
            try {

               m_sys_mutex.lock();
             
               if (!m_sys_changes_queue->empty ()) {
         
                    for (unsigned int index = 0; index < m_sys_changes_queue->size(); index++) {
             
                        m_sys_changes_queue->pop_front();
                    }
               }
                   
               m_sys_mutex.unlock();

            } catch (const std::exception& ex) {

                 if (m_sys_mutex.is_locked()){
                     m_sys_mutex.unlock();
                 }
 
                 std::cerr << " Process observer exception  " << ex.what();
            }
    }

    void ProcObserver_impl::app_changes_queue_flush () throw( CORBA::SystemException ){

              try {

               m_app_mutex.lock();
             
               if (!m_appl_changes_queue->empty ()) {
         
                    for (unsigned int index = 0; index < m_appl_changes_queue->size(); index++) {
             
                        m_appl_changes_queue->pop_front();
                    }
               }
                   
               m_app_mutex.unlock();

            } catch (const std::exception& ex) {

                 if (m_app_mutex.is_locked()){
                     m_app_mutex.unlock();
                 }
 
                 std::cerr << " Process observer exception  " << ex.what();
            }
    }

    void ProcObserver_impl::pro_changes_queue_flush () throw( CORBA::SystemException ) {
             
             try {

               m_pro_mutex.lock();
             
               if (!m_proc_changes_queue->empty ()) {
         
                    for (unsigned int index = 0; index < m_proc_changes_queue->size(); index++) {
             
                        m_proc_changes_queue->pop_front();
                    }
               }
                   
               m_pro_mutex.unlock();

            } catch (const std::exception& ex) {

                 if (m_pro_mutex.is_locked()){
                     m_pro_mutex.unlock();
                 }
 
                 std::cerr << " Process observer exception  " << ex.what();
            }
    }
    

 
  } // PlatformMngt
} // Cdmw

