/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

/**
 * @author Carlo Giovine  <cgiovine@sesm.it>
 */

#include <iostream>
#include <fstream>
#include <limits>

#include "test_force_manual_start_proc/TestObjs.hpp"
#include "test_force_manual_start_proc/ProcObserver_impl.hpp"

#include "Foundation/common/Options.hpp"
#include "Foundation/testutils/CORBATestManager.hpp"
#include "Foundation/orbsupport/OrbSupport.hpp"
#include "Foundation/orbsupport/StrategyList.hpp"
#include "Foundation/ossupport/OS.hpp"

#include "Foundation/common/Exception.hpp"
#include "Foundation/common/System.hpp"
#include "Foundation/common/Assert.hpp"
#include "Foundation/orbsupport/StrategyList.hpp"
#include "SystemMngt/platformlibrary/Iterator.hpp"

#include "SystemMngt/platformvaluetypes/ProcessStatusChange_impl.hpp"
#include "SystemMngt/platformvaluetypes/PropertyChange_impl.hpp"
#include "SystemMngt/platformvaluetypes/ConfigurationChange_impl.hpp"
#include "SystemMngt/idllib/CdmwPlatformMngtCommon.stub.hpp"

#include "SystemMngt/platformvaluetypes/EntityStatusChange_impl.hpp"

#include "idllib/CdmwPlatformMngtSystem.stub.hpp"

#include "idllib/CdmwPlatformMngtPredefinedProperties.stub.hpp"

using namespace Cdmw;

using namespace Common;

using namespace PlatformMngt;

using namespace CdmwPlatformMngt;

CPPUNIT_TEST_SUITE_REGISTRATION (TestObjs);

namespace Cdmw
{
  namespace PlatformMngt
  {
    
    TestObjs::TestObjs()
    {
    }

    void TestObjs::setUp ()
    {
          try
          {
          //Getting system var
          std::string system_corbaloc = "corbaloc::localhost:21869/CdmwPlatformMngtSupervision";
          orb = CORBA::ORB::_duplicate (Cdmw::TestUtils::CORBATestManager::instance ()->get_ORB ());
          CORBA::Object_var system_obj = orb->string_to_object (system_corbaloc.c_str());
          system = CdmwPlatformMngt::System::_narrow (system_obj.in ());
          
          //Setting up the startupKind parameters
          state_retrieval = CdmwPlatformMngtBase::LAST_STATE;
          startup_mode = 0;
          specified_state_id = "";
          startupKind = new CdmwPlatformMngtBase::StartupKind();
          startupKind->state_retrieval = state_retrieval;
          startupKind->startup_mode = startup_mode;
          startupKind->specified_state_id  = CORBA::string_dup( specified_state_id.c_str() );
          
          //Setting up the timeouts
          long_timeout  = 4 ; 
          short_timeout = 1 ;
           
          }
          catch (const CORBA::SystemException& ex) 
          {
              std::cerr<< ex._name()<<std::endl;
              throw;
          }
          catch(...)
          {
          }
    }

    void TestObjs::test_system_start() 
    {
        bool check =  true ;
        
        try {
            
            CdmwPlatformMngt::SystemSnapshot_var sys_snap = system->get_snapshot();
            TEST_INFO("Snapshot of the system ... "); 
            std::cout<< snapShotToString(sys_snap) <<std::endl;
            
            // If system is not started the assert is immediately declared false
            check = sys_status_control(system.in());
            CPPUNIT_ASSERT (check) ;
        
            nbApps=system->get_number_of_applications(); 
            /*These two "for" cycles are used to control the correct start phase of every application/processes */
            apps = system->get_applications(nbApps,appIter.out());
            TEST_INFO("Controlling if the state of application/process is correct ...");
              
            for (CORBA::ULong app_count = 0; app_count <nbApps; app_count++)
            {
                CPPUNIT_ASSERT( app_status_control( (apps[app_count].in()) , APPLICATION_INITIAL ));
                nbProcs = apps[app_count]->get_number_of_processes();
                procs   = apps[app_count]->get_processes(nbProcs, procIter.out());
                for (CORBA::ULong proc_count = 0; proc_count<nbProcs; proc_count++)
                {
                    CPPUNIT_ASSERT (proc_status_control( ( procs[proc_count].in()) , PROCESS_INITIAL ));   
                }
            }
        }
        catch (...)
        {
            CPPUNIT_ASSERT (false);
            throw;
        }  
        
    }
    
    void TestObjs::test_add_process () 
    {
        const char* indent ="";
        
        try
        {
            std::cout<<" "<<std::endl;
            TEST_INFO("Adding the application defined in to V-SMG-043-1-2.xml ...");
            std::string adminArgs("--system-corbaloc=corbaloc::localhost:21869/CdmwPlatformMngtSupervision --sys-add-application V-SMG-043-1-2.xml");
            OsSupport::OS::create_process("cdmw_platform_admin", adminArgs); 
            sleep(long_timeout);
            
            TEST_INFO("Doing an hot restart of the application ..");
            
            nbApps = system->get_number_of_applications();
            apps   = system->get_applications(nbApps,appIter.out());
            apps[nbApps-1]->hot_restart(startupKind.in());
            sleep(long_timeout);
            
            ApplicationSnapshot_var appl_snap = apps[nbApps-1]-> get_snapshot();

            TEST_INFO("Verifying the process state is still initial ...");
            std::cout<< snapShotToString(appl_snap, indent) <<std::endl;
            
            nbProcs = apps[nbApps-1]->get_number_of_processes();
            procs   = apps[nbApps-1]->get_processes(nbProcs, procIter.out());
            for (CORBA::ULong proc_count = 0; proc_count<nbProcs; proc_count++)
                {
                    CPPUNIT_ASSERT (proc_status_control( ( procs[proc_count].in()) , PROCESS_INITIAL ));   
                }
        }
        catch (const CORBA::SystemException& e)
        {
            std::cout<<e<<std::endl;
            CPPUNIT_ASSERT (false);
            throw;
        }  
   }
    
    void TestObjs::test_start_process_manually ()
    {
        bool check = true ;
        
        try 
        {
            std::cout<<""<<std::endl;
            TEST_INFO("Doing a hot restart of the system .. ");
            system -> hot_restart(startupKind.in());
            sleep (long_timeout);
            SystemSnapshot_var sys_snap = system->get_snapshot();
            TEST_INFO("Snapshot of the system  "); 
            std::cout<< snapShotToString(sys_snap) <<std::endl;

            // If system is not started the assert is immediately declared false
            check = sys_status_control(system.in());
            CPPUNIT_ASSERT (check);
            
            nbApps = system->get_number_of_applications();
            apps   = system->get_applications(nbApps,appIter.out());
            TEST_INFO("Controlling for each process with force manual start set to manual, if after an explicit command of START, it's correctly started .. ");
            for (CORBA::ULong app_count = 0; app_count<nbApps; app_count++)
            {
                nbProcs = apps[app_count]->get_number_of_processes();
                procs   = apps[app_count]->get_processes(nbProcs, procIter.out());
                for (CORBA::ULong proc_count = 0; proc_count<nbProcs; proc_count++)
                {
                    ProcessType processType = procs[proc_count]->type();
                    std::cerr << " Starting the Process ..."<< procs[proc_count]->name()<< std::endl;
                    if ( get_force_manual_start(procs[proc_count].in()) )
                    {
                        if (MANAGED_PROCESS == processType ) 
                        {
                            procs[proc_count]->initialise(startupKind.in());
                            procs[proc_count]->run();
                            sleep (short_timeout);
                        }
                        else if (UNMANAGED_PROCESS == processType )
                        {
                            procs[proc_count]->initialise(startupKind.in());
                            sleep (short_timeout);
                        }
                        CPPUNIT_ASSERT( proc_status_control( procs[proc_count].in() , PROCESS_RUNNING ));
                    }
                }
            }
        }
        catch (const CORBA::SystemException& e)
        {
            std::cout<<e<<std::endl;
            CPPUNIT_ASSERT (false);
            throw;
        }  
        
    }
      
    void TestObjs::test_restart_process ()
    {
        bool check = true ;
        
        try 
        {
            // If system is not started the assert is immediately declared false
            check = sys_status_control(system.in());
            CPPUNIT_ASSERT (check);
            
            nbApps = system->get_number_of_applications();
            apps   = system->get_applications(nbApps,appIter.out());
            TEST_INFO("Controlling for each process with force manual start set to manual, if after an explicit command of RESTART, it's correctly restarted  .. ");
            for (CORBA::ULong app_count = 0; app_count<nbApps; app_count++)
            {
                nbProcs = apps[app_count]->get_number_of_processes();
                procs   = apps[app_count]->get_processes(nbProcs, procIter.out());
                for (CORBA::ULong proc_count = 0; proc_count<nbProcs; proc_count++)
                {
                    std::cerr << " Restarting the Process ..."<< procs[proc_count]->name()<< std::endl;
                    procs[proc_count]->hot_restart(startupKind.in());
                    sleep (short_timeout);    
                    CPPUNIT_ASSERT( proc_status_control( procs[proc_count].in() , PROCESS_RUNNING ));   
                }
            }
            SystemSnapshot_var sys_snap = system->get_snapshot();
            TEST_INFO("Snapshot of the system  "); 
            std::cout<< snapShotToString(sys_snap) <<std::endl;
        }
        catch (const CORBA::SystemException& e)
        {
            std::cout<<e<<std::endl;
            CPPUNIT_ASSERT (false);
            throw;
        }
    }
    
    void TestObjs::test_automatic_action ()
    {
        bool check = true ;
        int end_timeout = 10;
        
        try 
        {
            std::cout<<" "<<std::endl;

            TEST_INFO("Adding the application defined in to V-SMG-043-1-3.xml ...");
            std::string adminArgs("--system-corbaloc=corbaloc::localhost:21869/CdmwPlatformMngtSupervision --sys-add-application V-SMG-043-1-3.xml");
            OsSupport::OS::create_process("cdmw_platform_admin", adminArgs);
            sleep(long_timeout);
                        
            check = sys_status_control(system.in());
            CPPUNIT_ASSERT (check);
                        
            nbApps = system->get_number_of_applications();
            apps   = system->get_applications(nbApps,appIter.out());
            
            CPPUNIT_ASSERT( app_status_control( apps[nbApps-1].in() , APPLICATION_INITIAL ));
            
            std::cout<<" Application  "<<apps[nbApps-1]->name()<<" added  ... "<<std::endl;
                  
            // Get a reference to the root POA
            CORBA::Object_var obj = orb->resolve_initial_references("RootPOA");
            PortableServer::POA_var poa = PortableServer::POA::_narrow( obj.in() );
            
            // Activate the POA manager
            PortableServer::POAManager_var mgr = poa->the_POAManager();
            mgr->activate();

            // Register event factories
            CORBA::ValueFactoryBase_var factory, old_factory;

            factory = new ProcessMessageFactory;
            old_factory = orb->register_value_factory(
                              "IDL:thalesgroup.com/CdmwPlatformMngt/ProcessMessage:1.0",
                              factory.in() );

            factory = new ProcessStatusChangeFactory;
            old_factory = orb->register_value_factory(
                              "IDL:thalesgroup.com/CdmwPlatformMngt/ProcessStatusChange:1.0",
                              factory.in() );
                  
            factory = new EntityStatusChangeFactory;
            old_factory = orb->register_value_factory(
                              "IDL:thalesgroup.com/CdmwPlatformMngt/EntityStatusChange:1.0",
                              factory.in() );
            factory = new PropertyChangeFactory;
            old_factory = orb->register_value_factory(
                    "IDL:thalesgroup.com/CdmwPlatformMngt/PropertyChange:1.0",
                    factory.in() );

            factory = new ConfigurationChangeFactory;
            old_factory = orb->register_value_factory(
                    "IDL:thalesgroup.com/CdmwPlatformMngt/ConfigurationChange:1.0",
                    factory.in() );


            nbApps = system->get_number_of_applications();
            apps   = system->get_applications(nbApps,appIter.out());
            
            nbProcs = apps[nbApps-1]->get_number_of_processes();
            procs   = apps[nbApps-1]->get_processes(nbProcs, procIter.out());
                                    
            for (CORBA::ULong proc_count = 0; proc_count<nbProcs; proc_count++)
            {
                ProcObserver_impl prcOb;
                CdmwPlatformMngt::SupervisionObserver_var observer = prcOb._this();
                CdmwPlatformMngt::RegistrationDepth depth=FULL_HIERARCHY;
                std::string observer_name = "prcOb_";
                observer_name.append(procs[proc_count]->name());
                
                procs[proc_count]->register_observer_ext (observer_name.c_str(),depth, observer.in ());
                std::cerr << " Observer registered on Process "<< procs[proc_count]->name() << " with depth = " << depth << std::endl;
                ProcessType processType = procs[proc_count]->type();
                if (MANAGED_PROCESS == processType ) 
                {
                    std::cerr << " Starting of "<< procs[proc_count]->name()<< std::endl;
                    procs[proc_count]->initialise(startupKind.in());
                    procs[proc_count]->run();
                }
                else if (UNMANAGED_PROCESS == processType )
                {
                    procs[proc_count]->initialise(startupKind.in());
                }
                int count = 0;
                while (!prcOb.is_failed() )
                {
                    std::cerr << " Waiting for the Failed_death notification of "<< procs[proc_count]->name()<< std::endl;
                    if ( (end_timeout+15) == count) {CPPUNIT_ASSERT(false);}
                    sleep (1);
                    count++;
                }
                count = 0;
                while (count != end_timeout)
                {
                    count++;
                    std::cerr << "Restart automatic-action disabled by force_manual_start tag, waiting for the Restart notification of "<< procs[proc_count]->name()<< std::endl;
                    
                    if(prcOb.is_restarted())
                    {
                        CPPUNIT_ASSERT(false); 
                        std::cerr<<"\n Process wrongly RESTARTED after the RESTART automatic_action ";
                        break;
                    }
                    if (end_timeout == count) 
                    { 
                        CPPUNIT_ASSERT(true);
                        std::cerr<<"\n Process "<< procs[proc_count]->name() 
                        <<", with Force-manual-start=Manual correctly remains Failed_death after the RESTART automatic_action "<< std::endl;
                        std::string adminArgs("--system-corbaloc=corbaloc::localhost:21869/CdmwPlatformMngtSupervision --proc-snapshot ");
                        adminArgs.append(" ");
                        adminArgs.append( apps[nbApps-1]->name());
                        adminArgs.append(" ");
                        adminArgs.append(procs[proc_count]->name());
                        adminArgs.append(" ");
                        adminArgs.append(Cdmw::OsSupport::OS::get_hostname());
                        OsSupport::OS::create_process("cdmw_platform_admin", adminArgs);
                    }
                    sleep (2);
                   
                }
                try
                {
                    procs[proc_count]->unregister_observer_ext (observer_name.c_str());
                }catch(const CdmwPlatformMngt::ObserverNotFound & ex)
                {
                   std::cerr<< " \n CdmwPlatformMngt::ObserverNotFound exception..."<< ex; 
                }
                catch(...)
                {
                    std::cerr << " WARNING Exception thrown during unregistrer_observer_ext";
                }
                std::cerr << " Observer unregisted on Process " << procs[proc_count]->name() <<std::endl;
                
                
                
            }
            CPPUNIT_ASSERT(true);            
            
        }
        catch (const CORBA::SystemException& e)
        {
            std::cout<<e<<std::endl;
            CPPUNIT_ASSERT (false);
            throw;
        }  
    }
    
    bool TestObjs::sys_status_control (const CdmwPlatformMngt::System_ptr ptr) throw ()
        {
            try 
            {
            SystemStatus status = ptr->get_status(statusInfo.out());
            if ( (CdmwPlatformMngt::SYSTEM_STARTED ==status) || (CdmwPlatformMngt::SYSTEM_PARTIAL_STARTED ==status) ) {return true;}
            else {return false; }
            }catch (...){ throw;}
        }

    bool TestObjs::app_status_control (const CdmwPlatformMngt::Application_ptr ptr, const CdmwPlatformMngt::ApplicationStatus& status) throw ()
        {
            try
            {
            ApplicationStatus curr_status = ptr->get_status (statusInfo.out());
            
            bool app_force_manual_start = get_force_manual_start(ptr);
            
            if ((APPLICATION_RUNNING ==curr_status) && (!app_force_manual_start)) {return true;}
            else if ((APPLICATION_PARTIAL_RUNNING ==curr_status) && (!app_force_manual_start)) {return true;}
            else if ((status ==curr_status) && (app_force_manual_start)) {return true;}
            else  {return false;}
            }
            catch (...){ throw;}
        }
        
    bool TestObjs::proc_status_control (const CdmwPlatformMngt::Process_ptr ptr, const CdmwPlatformMngt::ProcessStatus& status) throw ()
        {
            try
            {
            ProcessStatus curr_status = ptr->get_status (statusInfo.out());
             
            bool proc_force_manual_start = get_force_manual_start (ptr);
            
            bool app_force_manual_start  = get_force_manual_start (Application::_narrow(ptr->get_parent()));
            
            if ((PROCESS_RUNNING ==curr_status) && (!proc_force_manual_start)) {return true;}
            else if ((status == curr_status) && (proc_force_manual_start)) {return true;}
            else if ((status == curr_status) && (app_force_manual_start)) {return true;}
            else  {return false;}
            }
            catch (...){ throw;}
        }
    
    bool TestObjs::get_force_manual_start (const CdmwPlatformMngt::Application_ptr ptr) throw ()
    {
        CORBA::Any_var any_var = new CORBA::Any();
        CORBA::Boolean force_manual_start;
        ptr->get_attribute(FORCE_MANUAL_START, any_var.out(), PRE_DEFINED);
        any_var >>= CORBA::Any::to_boolean (force_manual_start);
        return force_manual_start;
    }
        
    bool TestObjs::get_force_manual_start (const CdmwPlatformMngt::Process_ptr ptr) throw ()
    {
        CORBA::Any_var any_var = new CORBA::Any();
        CORBA::Boolean force_manual_start;
        ptr->get_attribute(FORCE_MANUAL_START, any_var.out(), PRE_DEFINED);
        any_var >>= CORBA::Any::to_boolean (force_manual_start);
        return force_manual_start;
    }
    
}/* End namespace PlatformMngt */
}/* End namespace Cdmw */





