/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#include <iostream>
#include <fstream>
#include <limits>

#include "test_force_manual_start_sys_app/TestObjs.hpp"

#include "Foundation/common/Options.hpp"
#include "Foundation/testutils/CORBATestManager.hpp"
#include "Foundation/orbsupport/OrbSupport.hpp"
#include "Foundation/ossupport/OS.hpp"

#include "Foundation/common/Exception.hpp"
#include "Foundation/common/System.hpp"
#include "Foundation/common/Assert.hpp"
#include "Foundation/orbsupport/StrategyList.hpp"
#include <SystemMngt/platformlibrary/Iterator.hpp>

#include "idllib/CdmwPlatformMngtSystem.stub.hpp"

#include "idllib/CdmwPlatformMngtPredefinedProperties.stub.hpp"


using namespace Cdmw;

using namespace Common;

using namespace PlatformMngt;

using namespace CdmwPlatformMngt;

CPPUNIT_TEST_SUITE_REGISTRATION (TestObjs);

namespace Cdmw
{
  namespace PlatformMngt
  {
    
    TestObjs::TestObjs()
    {
    }

    void TestObjs::setUp ()
    {
          try
          {
          //Getting system var
          std::string system_corbaloc = "corbaloc::localhost:21869/CdmwPlatformMngtSupervision";
          CORBA::ORB_var orb = CORBA::ORB::_duplicate (Cdmw::TestUtils::CORBATestManager::instance ()->get_ORB ());
          CORBA::Object_var system_obj = orb->string_to_object (system_corbaloc.c_str());
          system = CdmwPlatformMngt::System::_narrow (system_obj.in ());
          
          //Setting up the startupKind parameters
          state_retrieval = CdmwPlatformMngtBase::LAST_STATE;
          startup_mode = 0;
          specified_state_id = "";
          startupKind = new CdmwPlatformMngtBase::StartupKind();
          startupKind->state_retrieval = state_retrieval;
          startupKind->startup_mode = startup_mode;
          startupKind->specified_state_id  = CORBA::string_dup( specified_state_id.c_str() );
          
          //Setting up the timeouts
          long_timeout  = 4 ; 
          short_timeout = 1 ;
           
          }
          catch (const CORBA::SystemException& ex) 
          {
              std::cerr<< ex._name()<<std::endl;
              throw;
          }
          catch(...)
          {
          }
    }

    void TestObjs::test_system_start() 
    {
        bool check =  true ;
        
        try {
            
            CdmwPlatformMngt::SystemSnapshot_var sys_snap = system->get_snapshot();
            TEST_INFO("Snapshot of the system ... "); 
            std::cout<< snapShotToString(sys_snap) <<std::endl;
            
            check = sys_status_control(system.in());
            CPPUNIT_ASSERT (check) ;
        
            nbApps=system->get_number_of_applications(); 
            /*These two "for" cycles are used to control the correct start phase of every application/processes */
            apps = system->get_applications(nbApps,appIter.out());
            TEST_INFO("Controlling if the state of application/process is correct ...");
              
            for (CORBA::ULong app_count = 0; app_count <nbApps; app_count++)
            {
                CPPUNIT_ASSERT( app_status_control( (apps[app_count].in()) , APPLICATION_INITIAL ));
                nbProcs = apps[app_count]->get_number_of_processes();
                procs   = apps[app_count]->get_processes(nbProcs, procIter.out());
                for (CORBA::ULong proc_count = 0; proc_count<nbProcs; proc_count++)
                {
                    CPPUNIT_ASSERT (proc_status_control( ( procs[proc_count].in()) , PROCESS_INITIAL ));   
                }
            }
        }
        catch (...)
        {
            CPPUNIT_ASSERT (false);
            throw;
        }  
        
    }
    
    void TestObjs::test_system_restart() 
    {
        bool check = true;
        
        try {
            std::cout<<" "<<std::endl;
            TEST_INFO("Doing a cold restart of the system .. ");
            system -> cold_restart(startupKind.in());
            sleep (long_timeout);
            
            // If system is not started the assert is immediately declared false
            check = sys_status_control(system.in() );
            CPPUNIT_ASSERT (check);
            
            nbApps=system->get_number_of_applications(); 
            apps = system->get_applications(nbApps,appIter.out());

            TEST_INFO("Controlling if, after a cold restart,  the state of application/process is correct ... ");
              
            for (CORBA::ULong app_count = 0; app_count <nbApps; app_count++)
            {
                std::cerr << " Controlling the Applications "<< apps[app_count]->name()<< std::endl;
                CPPUNIT_ASSERT( app_status_control( (apps[app_count].in()) , APPLICATION_INITIAL ));
                nbProcs = apps[app_count]->get_number_of_processes();
                procs   = apps[app_count]->get_processes(nbProcs, procIter.out());
                for (CORBA::ULong proc_count = 0; proc_count<nbProcs; proc_count++)
                {
                    std::cerr << " Controlling the Process "<< procs[proc_count]->name()<< std::endl;
                    CPPUNIT_ASSERT (proc_status_control( ( procs[proc_count].in()) , PROCESS_INITIAL ));   
                }
            }
        }
        catch (const CORBA::SystemException& e)
        {
            std::cout<<e<<std::endl;
            CPPUNIT_ASSERT (false);
            throw;
        }  
    }
    
    void TestObjs::test_add_application () 
    {
        const char* indent ="";
        
        try
        {
            std::cout<<" "<<std::endl;
            TEST_INFO("Adding the application defined in to V-SMG-042-1-2.xml ...");
            std::string adminArgs("--system-corbaloc=corbaloc::localhost:21869/CdmwPlatformMngtSupervision --sys-add-application V-SMG-042-1-2.xml");
            OsSupport::OS::create_process("cdmw_platform_admin", adminArgs); 
            sleep(long_timeout);
            
            TEST_INFO("Doing an hot restart of the system ..");
            system->hot_restart(startupKind.in());
            sleep(long_timeout);
            
            nbApps = system->get_number_of_applications();
            apps   = system->get_applications(nbApps,appIter.out());
            ApplicationSnapshot_var appl_snap = apps[nbApps-1]-> get_snapshot();

            TEST_INFO("Verifying the application state is still initial ...");
            std::cout<< snapShotToString(appl_snap, indent) <<std::endl;
            CPPUNIT_ASSERT( app_status_control( apps[nbApps-1].in() , APPLICATION_INITIAL ));
        }
        catch (const CORBA::SystemException& e)
        {
            std::cout<<e<<std::endl;
            CPPUNIT_ASSERT (false);
            throw;
        }  
   }
   
    void TestObjs::test_start_application_manually ()
    {
        bool check = true ;
        
        try 
        {
            std::cout<<" "<<std::endl;
            TEST_INFO("Doing an Hot restart of the system .. ");
            system -> hot_restart(startupKind.in());
            sleep (long_timeout);
            
            // If system is not started the assert is immediately declared false
            check = sys_status_control(system.in());
            CPPUNIT_ASSERT (check);
            
            nbApps = system->get_number_of_applications();
            apps   = system->get_applications(nbApps,appIter.out());
            TEST_INFO("Controlling for each application, if after an explicit command of START, it's correctly started  .. ");
            for (CORBA::ULong app_count = 0; app_count<nbApps; app_count++)
            {
                if ( get_force_manual_start(apps[app_count].in())  )
                {
                    apps[app_count]->initialise(startupKind.in());
                    sleep (short_timeout);
                    apps[app_count]->run();
                    sleep (short_timeout);
                    std::cerr << " Controlling the Applications "<< apps[app_count]->name()<< std::endl;
                    CPPUNIT_ASSERT( app_status_control( apps[app_count].in() , APPLICATION_PARTIAL_RUNNING ) ||
                                    app_status_control( apps[app_count].in() , APPLICATION_RUNNING ));
                    nbProcs = apps[app_count]->get_number_of_processes();
                    procs   = apps[app_count]->get_processes(nbProcs, procIter.out());
                    for (CORBA::ULong proc_count = 0; proc_count<nbProcs; proc_count++)
                    {
                        std::cerr << " Controlling the Process "<< procs[proc_count]->name()<< std::endl;
                        CPPUNIT_ASSERT( proc_status_control( procs[proc_count].in() , PROCESS_INITIAL ));   
                    }
                }
            }
        }
        catch (const CORBA::SystemException& e)
        {
            std::cout<<e<<std::endl;
            CPPUNIT_ASSERT (false);
            throw;
        }  
    }   
    
    void TestObjs::test_restart_application ()
    {
        bool check = true ;
                   
        try 
        {
            std::cout<<" "<<std::endl;
            TEST_INFO("Doing an Hot restart of the system .. ");
            system -> hot_restart(startupKind.in());
            sleep (long_timeout);
            
            check = sys_status_control(system.in());
            CPPUNIT_ASSERT (check);
            
            nbApps = system->get_number_of_applications(); 
            apps   = system->get_applications(nbApps,appIter.out());
            TEST_INFO("Controlling for each application, if after an explicit command of RESTART, it's correctly restarted  .. ");
            for (CORBA::ULong app_count = 0; app_count<nbApps; app_count++)
            {
                apps[app_count]->hot_restart(startupKind.in());
                sleep (short_timeout);    
                
                std::cerr << " Controlling the Applications "<< apps[app_count]->name()<< std::endl;
                CPPUNIT_ASSERT( app_status_control( apps[app_count].in() , APPLICATION_PARTIAL_RUNNING ) ||
                                app_status_control( apps[app_count].in() , APPLICATION_RUNNING ));
                
                nbProcs = apps[app_count]->get_number_of_processes();
                procs   = apps[app_count]->get_processes(nbProcs, procIter.out());
                for (CORBA::ULong proc_count = 0; proc_count<nbProcs; proc_count++)
                {
                    std::cerr << " Controlling the Process "<< procs[proc_count]->name()<< std::endl;
                    CPPUNIT_ASSERT( proc_status_control( procs[proc_count].in() , PROCESS_INITIAL ));   
                }
            }
            SystemSnapshot_var sys_snap = system->get_snapshot();
            TEST_INFO("System Snapshot"); 
            std::cout<< snapShotToString(sys_snap) <<std::endl;
        }
        catch (const CORBA::SystemException& e)
        {
            std::cout<<e<<std::endl;
            CPPUNIT_ASSERT (false);
            throw;
        }  
    }
    
    bool TestObjs::sys_status_control (const CdmwPlatformMngt::System_ptr ptr) throw ()
        {
            try 
            {
            SystemStatus status = ptr->get_status(statusInfo.out());
            if ( (CdmwPlatformMngt::SYSTEM_STARTED ==status) || (CdmwPlatformMngt::SYSTEM_PARTIAL_STARTED ==status) ) {return true;}
            else {return false; }
            }catch (...){ throw;}
        }

    bool TestObjs::app_status_control (const CdmwPlatformMngt::Application_ptr ptr, const CdmwPlatformMngt::ApplicationStatus& status) throw ()
        {
            try
            {
            ApplicationStatus curr_status = ptr->get_status (statusInfo.out());
            
            bool app_force_manual_start = get_force_manual_start(ptr);
            
            if ((APPLICATION_RUNNING ==curr_status) && (!app_force_manual_start)) {return true;}
            else if ((APPLICATION_PARTIAL_RUNNING ==curr_status) && (!app_force_manual_start)) {return true;}
            else if ((status ==curr_status) && (app_force_manual_start)) {return true;}
            else  {return false;}
            }
            catch (...){ throw;}
        }
        
    bool TestObjs::proc_status_control (const CdmwPlatformMngt::Process_ptr ptr, const CdmwPlatformMngt::ProcessStatus& status) throw ()
        {
            try
            {
            ProcessStatus curr_status = ptr->get_status (statusInfo.out());
             
            bool proc_force_manual_start = get_force_manual_start (ptr);
            
            bool app_force_manual_start  = get_force_manual_start (Application::_narrow(ptr->get_parent()));
            
            if ((PROCESS_RUNNING ==curr_status) && (!proc_force_manual_start)) {return true;}
            else if ((status == curr_status) && (proc_force_manual_start)) {return true;}
            else if ((status == curr_status) && (app_force_manual_start)) {return true;}
            else  {return false;}
            }
            catch (...){ throw;}
        }
    
    bool TestObjs::get_force_manual_start (const CdmwPlatformMngt::Application_ptr ptr) throw ()
    {
        CORBA::Any_var any_var = new CORBA::Any();
        CORBA::Boolean force_manual_start;
        ptr->get_attribute(FORCE_MANUAL_START, any_var.out(), PRE_DEFINED);
        any_var >>= CORBA::Any::to_boolean (force_manual_start);
        return force_manual_start;
    }
        
    bool TestObjs::get_force_manual_start (const CdmwPlatformMngt::Process_ptr ptr) throw ()
    {
        CORBA::Any_var any_var = new CORBA::Any();
        CORBA::Boolean force_manual_start;
        ptr->get_attribute(FORCE_MANUAL_START, any_var.out(), PRE_DEFINED);
        any_var >>= CORBA::Any::to_boolean (force_manual_start);
        return force_manual_start;
    }

}/* End namespace PlatformMngt */
}/* End namespace Cdmw */





