/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#include "Foundation/common/Options.hpp"
#include "Foundation/common/String.hpp"
#include "Foundation/orbsupport/CORBA.hpp"

#include "Foundation/ossupport/OS.hpp"
#include "Foundation/osthreads/ThreadHandle.hpp"
#include "Foundation/orbsupport/OrbSupport.hpp"
#include "Foundation/orbsupport/StrategyList.hpp"
#include "SystemMngt/platforminterface/PlatformInterface.hpp"

#include "test_max_restart_app_proc/MyProcessBehaviour.hpp"

#include <cstdlib>
#include <string>
#include <exception>
#include <sstream>
#include <iostream>
#include <fstream>


const int POA_THREAD_POOL_SIZE = 5;

namespace
{
    using namespace CdmwPlatformMngt;
    using namespace Cdmw::OsSupport;
    using namespace Cdmw::OrbSupport;
    using namespace Cdmw::PlatformMngt;

    std::string applicationName;    
    std::string processName;
    
    const int HIDDEN_PARAM_MANAGED = 3; // program name, CdmwCallbackIor, CdmwLocalisationService
    const int HIDDEN_PARAM_UNMANAGED = 2; // program name, CdmwLocalisationService
    const int time_before_main_thread_dies = 3000; 
    
    const std::string METHOD_NAME_OPTION = "--method";
    const std::string ACTION_NAME_OPTION = "--action";
    const std::string PARAMETER_OPTION = "--parameter";
    
    const std::string METHOD_INITIALISE = "initialise";
    const std::string METHOD_RUN = "run";
    const std::string METHOD_STOP = "stop";
    const std::string METHOD_ISALIVE = "isalive";
    const std::string METHOD_NBSTEPS = "nbsteps";
    const std::string METHOD_MAIN = "main";
        
    const std::string ACTION_TIMEOUT = "timeout";
    const std::string ACTION_EXIT = "exit";
        
}


void exceptionHandler(void)
{
    std::cout << "Unexpected Exception : Test failed" << std::endl;
}


void exitHandler(void)
{
    std::cout << "Process '" << processName << "' exited" << std::endl;
}

static void usage(std::ostream & os, const std::string &program_name)
{
    os << "Usage:\n"
       << program_name << " " << METHOD_NAME_OPTION <<"=<METHOD> [" 
                              << ACTION_NAME_OPTION <<"=<ACTION>] " 
                              << PARAMETER_OPTION <<"=<PARAMETER>" << std::endl
       << "\t<METHOD>    can be one of [" << METHOD_INITIALISE << ", " << METHOD_RUN << ", " 
                                     << METHOD_STOP << ", " << METHOD_ISALIVE << ", "
                                     << METHOD_NBSTEPS << ", " << METHOD_MAIN << "]" << std::endl
       << "\t<ACTION>    can be one of [" << ACTION_TIMEOUT << ", " << ACTION_EXIT << "]" << std::endl 
       << "\t<PARAMETER> is a positive integer" << std::endl
       << "\tThe <ACTION> option is required for every <METHOD> different from "  
                                     << METHOD_NBSTEPS << std::endl
       << "\tThe only <ACTION> available for " << METHOD_MAIN << " is " << ACTION_EXIT << std::endl
       << "\tWhen launching this process as UNMANAGED all arguments other than" << std::endl 
            << "\t\t"
            << METHOD_NAME_OPTION << "=" << METHOD_MAIN << " " 
            << ACTION_NAME_OPTION << "=" << ACTION_EXIT << " "
            << PARAMETER_OPTION << "=<PARAMETER>" << std::endl
       << "\twill be ignored" << std::endl;                         
}
    
int main(int argc, char* argv[])
{  
    int ret_code = 0;
    bool exit_in_main = false;

    // Print arguments
    std::cout << "** Arguments: " << argc << std::endl;
    for (int i=0; i< argc; i++)
    {
        std::cout << "[" << i << "]: " << argv[i] << std::endl;
    }
    //*/
    
    int HIDDEN_PARAM = HIDDEN_PARAM_UNMANAGED;
    bool platformManaged = PlatformInterface::Is_launched_by_PlatformManagement(argc, argv);
    if (platformManaged) {
        HIDDEN_PARAM = HIDDEN_PARAM_MANAGED;    
    }         
    //*/
    if (platformManaged) {
        std::cout << "** Process launched as: MANAGED" << std::endl;    
    }         
    else {
        std::cout << "** Process launched as: UNMANAGED" << std::endl;
    }
    //*/
    const int MIN_PARAM = 2 + HIDDEN_PARAM;
    const int MAX_PARAM = 3 + HIDDEN_PARAM;
    
    // If number of parameters is not in the range [MIN_PARAM,MAX_PARAM] and it's not zero, 
    // show usage and exit
    if (!((argc >= MIN_PARAM) && (argc <= MAX_PARAM)) && (argc != HIDDEN_PARAM)) {
        usage(std::cout, argv[0]);
        return ret_code;    
    } 
    
    std::string method_name = OS::get_option_value (argc, argv, METHOD_NAME_OPTION);
    std::string action_name = OS::get_option_value (argc, argv, ACTION_NAME_OPTION);
    std::string parameter_str = OS::get_option_value (argc, argv, PARAMETER_OPTION);

   
    std::cout << "** method_name: " << method_name << std::endl 
              << "** action_name: " << action_name << std::endl 
              << "** parameter_str: " << parameter_str << std::endl;
        

TestAutomaticActions::Method opt_method = TestAutomaticActions::NOMETHOD;
    TestAutomaticActions::Action opt_action = TestAutomaticActions::NOACTION;
    TestAutomaticActions::Parameter opt_parameter = (TestAutomaticActions::Parameter) 0;

    // Get parameter value
    if (parameter_str != "no" && parameter_str != "yes") {
        std::istringstream parameter_stream(parameter_str);
        unsigned int parameter_int;
        parameter_stream >> parameter_int;
        if (parameter_stream.fail()) {
            usage(std::cout, argv[0]);
            return ret_code; 
        }
        opt_parameter = (TestAutomaticActions::Parameter) parameter_int;
    }


            
    if (method_name == METHOD_MAIN && action_name == ACTION_EXIT) {
        if (!platformManaged) {
            usleep(time_before_main_thread_dies * 1000);
            exit((unsigned int) opt_parameter);
        }
        exit_in_main = true;    
    }   
    else {            
      

        // Get method name option
        if (method_name != "no" && method_name != "yes") {
            if (method_name == METHOD_INITIALISE) opt_method = TestAutomaticActions::INITIALISE;
            else if (method_name == METHOD_RUN) opt_method = TestAutomaticActions::RUN;
            else if (method_name == METHOD_STOP) opt_method = TestAutomaticActions::STOP;
            else if (method_name == METHOD_ISALIVE) opt_method = TestAutomaticActions::ISALIVE;
            else if (method_name == METHOD_NBSTEPS) opt_method = TestAutomaticActions::NBSTEPS;
        }
        


        // Get action name option
        if (action_name != "no" && action_name != "yes") {
            if (action_name == ACTION_TIMEOUT) opt_action = TestAutomaticActions::TIMEOUT;
            else if (action_name == ACTION_EXIT) opt_action = TestAutomaticActions::EXIT;
        }
    }
    
    try
    {

        std::set_unexpected(&exceptionHandler);

        ::atexit(&exitHandler);

        StrategyList strategy;
        strategy.add_OrbThreaded();
        strategy.add_PoaThreadPool(POA_THREAD_POOL_SIZE);

        CORBA::ORB_var orb =  Cdmw::OrbSupport::OrbSupport::ORB_init(argc, argv, strategy); 
    
        CORBA::Object_var obj = orb->resolve_initial_references("RootPOA");

        PortableServer::POA_var poa = PortableServer::POA::_narrow(obj.in());

        PortableServer::POAManager_var poaMgr = poa->the_POAManager();
    
        poaMgr->activate();

        try
        {
            if (platformManaged)
            {
                // initialise the platform interface
                PlatformInterface::Setup(orb.in(), argc, argv);
            }
            
            // creates the process behaviour
            std::auto_ptr<MyProcessBehaviour> pMyProcess(new MyProcessBehaviour());
            usleep(1);

            
            // Set process action according to command line arguments
            if (!pMyProcess->SetAction(opt_method, opt_action, opt_parameter)) {
                usage(std::cout, argv[0]);
                throw;
            } 
                        
            if (platformManaged) {
                // acknowledge the creation of the process
                PlatformInterface::Acknowledge_creation(pMyProcess.get());
                pMyProcess.release();

                // sample usage of the platform interface                    
                std::string applicationName = PlatformInterface::Get_application_name();
                std::string processName = PlatformInterface::Get_process_name();
                OS::ProcessId processId = OS::get_processId();
                
                std::cout << "** Application name : "
                            << applicationName.c_str() << std::endl;

                std::cout << "** Process name : " 
                            << processName.c_str() 
                            << " Process id : "
                            << processId << std::endl;
            }

            if (exit_in_main) {
                usleep(time_before_main_thread_dies * 1000);
                exit((unsigned int) opt_parameter);
            }                       
   
            // start orb
            orb->run();
            
            if (platformManaged)
            {
                // cleanup the platform interface
                PlatformInterface::Cleanup();
            }
            
        }
            
        catch(...)
        {
            // cleanup the ORB to exit completely
            orb->shutdown(false);
            Cdmw::OrbSupport::OrbSupport::ORB_cleanup(orb.in());
            orb->destroy();
            throw;
        }

        Cdmw::OrbSupport::OrbSupport::ORB_cleanup(orb.in());
        orb->destroy();
        
    }
    catch (const CORBA::Exception& e)
    {  
        std::cerr << "FAILURE : CORBA exception raised :" << std::endl
                  << e._name() << std::endl;
        ret_code = EXIT_FAILURE;
    }
    catch(const std::exception &e)
    {
        std::cerr << "FAILURE : Exception raised : " << std::endl
                  << e.what() << std::endl;
        ret_code = EXIT_FAILURE;
    }
    catch(...)
    {
        std::cerr << "FAILURE : Unexpected exception" << std::endl;
        ret_code = EXIT_FAILURE;
    }

    return ret_code;

}


