/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

/*
* @author Ganna Domnina <gdomnina@sesm.it>
*/

#include <fstream>
#include <iostream>
#include <string>
#include <sstream>


#include "test_periodic_notification_reception/TestPeriodicNotification.hpp"
#include "Foundation/orbsupport/OrbSupport.hpp"

#include "Foundation/ossupport/OS.hpp"
#include "Foundation/testutils/CORBATestManager.hpp"
#include "Foundation/osthreads/Thread.hpp"

#include <Foundation/orbsupport/StrategyList.hpp>

namespace
{
    using namespace CdmwPlatformMngt;
    using namespace Cdmw::OsSupport;
    using namespace Cdmw::OrbSupport;
    using namespace Cdmw::PlatformMngt;
    
    const std::string USERDEF_PROPERTY = "--userdef_num";
    const std::string PERIOD = "--period";
    const std::string KILL_SUP = "--kill_sup";
}


/*********************************
* Orb Worker
*********************************/

int returnError = -1;



class ORBWorker:public Cdmw::OsSupport::Thread
{

public:
    ORBWorker (CORBA::ORB_ptr orb);
    
    virtual ~ORBWorker ()
    throw ();
    
    virtual void run ()
    throw ();
    
private:
    CORBA::ORB_var
    m_ORB;
};

ORBWorker::ORBWorker (CORBA::ORB_ptr orb):m_ORB (CORBA::ORB::_duplicate (orb))
{
}


ORBWorker::~ORBWorker ()throw ()
{
try
{
    //m_ORB->shutdown (0);
    m_ORB->destroy ();
    this->join ();
}
catch (const CORBA::SystemException & e)
{
    std::cout << ":>> CORBA system exception caught: " << e._info ().
    rep () << std::endl;
}
catch (const std::exception & e)
{
    std::cout << ":>> std exception caught: " << e.what () << std::endl;
}
catch (...)
{
    std::cout << ":>> unexpected exception caught" << std::endl;
}
}



void
ORBWorker::run ()
throw ()
{
try
{
    this->m_ORB->run ();
}
catch (const CORBA::SystemException & e)
{
    std::cout << ":>> CORBA system exception caught: " << e._info ().
    rep () << std::endl;
}
catch (...)
{
    std::cout << ":>> unexpected exception caught" << std::endl;
}
}

static void usage(std::ostream & os, const std::string &program_name)
{
    os << "Usage:\n"
    << "  " << USERDEF_PROPERTY <<"=<NUMBER>" 
    << "  " << PERIOD <<"=<VALUE(ms)>" 
    << "  " << KILL_SUP <<"=<VALUE(ms)>" 
    << "\n  The options are mandatory :  "
    << "\n  The number of user-defined property is used to define and delete properties at run-time"
    << "\n  The period of notification (in milliseconds). Used to register the periodic observer." 
    << "\n  The time (in milliseconds) to kill the primary supervision. Used to verify the change of supervision mastership"
    << "\n  This time has to be between [period, period * (userdef_num *2 +1)] in order to be verify before test is finished." << std::endl;
}

int main( int argc, char* argv[])
{
    using namespace Cdmw;
    using namespace Cdmw::OsSupport;
    using namespace Cdmw::PlatformMngt;

    const int MIN_PARAM = 4;
    
    // If number of parameters is not less than MIN_PARAM 
    // show usage and exit
    if (!(argc >= MIN_PARAM))
    {
        usage(std::cout, argv[0]);
        return 0;
    }
    
    std::string userdef_num = Cdmw::OsSupport::OS::get_option_value( argc, argv, USERDEF_PROPERTY);
    std::istringstream param1(userdef_num);
    int num;
    param1>>num;
    
    std::string period = Cdmw::OsSupport::OS::get_option_value( argc, argv, PERIOD);
    std::istringstream param2(period);
    int pd;
    param2>>pd;
    
    std::string kill_sup = Cdmw::OsSupport::OS::get_option_value( argc, argv, KILL_SUP);
    std::istringstream param3(kill_sup);
    int sup;
    param3>>sup;
    
    if (pd==0)
    {
        std::cout<<" The value of period must to be bigger then zero"<<std::endl;
        usage(std::cout, argv[0]);
        return 0;
    }

    if (0 != sup)
    {
        if ((sup > (pd * (num *2 +1)) || (sup < pd)))
        {
            std::cout<<" The value of sup_kill must to be between ["<<pd<<" , "<< pd * (num *2 +1)<<"]"<<std::endl;
            usage(std::cout, argv[0]);
            return 0;
        }
    }


    /*
    ===================================================
    Initialize the ORB
    ===================================================
    */
    Cdmw::OrbSupport::StrategyList orb_strategies;
    orb_strategies.add_OrbThreaded ();
    //
    orb_strategies.add_PoaThreadPerConnection ();
    
#ifdef CDMW_USE_TEST_MIOP
    orb_strategies.add_multicast();
#endif
    
    CORBA::ORB_var orb;

    orb = Cdmw::OrbSupport::OrbSupport::ORB_init (argc, argv, orb_strategies);

    Cdmw::TestUtils::CORBATestManager::instance()->set_property(USERDEF_PROPERTY, userdef_num);
    Cdmw::TestUtils::CORBATestManager::instance()->set_property(PERIOD, period);
    Cdmw::TestUtils::CORBATestManager::instance()->set_property(KILL_SUP, kill_sup);

    CORBA::Object_var obj = orb->resolve_initial_references ("RootPOA");
    PortableServer::POA_var poa = PortableServer::POA::_narrow (obj.in ());

    PortableServer::POAManager_var poaMgr = poa->the_POAManager ();
    poaMgr->activate ();

    Cdmw::TestUtils::CORBATestManager::instance ()->init (orb.in (), poa.in (),argc, argv);

    // create an ORBWorker object.
    ORBWorker orb_worker (orb.in ());
    orb_worker.start ();

    // wait for the ORBWorker thread to run the ORB.
    Cdmw::OsSupport::OS::sleep (1, 0);

    Cdmw::TestUtils::CORBATestManager::instance ()->run_tests ();
    Cdmw::TestUtils::CORBATestManager::instance ()->summary ();

    return (Cdmw::TestUtils::CORBATestManager::instance ()->was_successful ());
}


