/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

/**
 * @author Carlo Giovine  <cgiovine@sesm.it>
 */

#include <iostream>
#include <fstream>
#include <limits>
#include <string>

#include "test_timeout_applications/TestTimeoutApplication.hpp"

#include "Foundation/common/Options.hpp"
#include "Foundation/testutils/CORBATestManager.hpp"
#include "Foundation/orbsupport/OrbSupport.hpp"
#include "Foundation/orbsupport/StrategyList.hpp"
#include "Foundation/ossupport/OS.hpp"

#include "Foundation/common/Exception.hpp"
#include "Foundation/common/System.hpp"
#include "Foundation/common/Assert.hpp"
#include "Foundation/orbsupport/StrategyList.hpp"
#include "SystemMngt/platformlibrary/Iterator.hpp"

#include "SystemMngt/platformvaluetypes/SystemModeChange_impl.hpp"
#include "SystemMngt/platformvaluetypes/SystemStatusChange_impl.hpp"
#include "SystemMngt/platformvaluetypes/HostStatusChange_impl.hpp"
#include "SystemMngt/platformvaluetypes/ApplicationAgentDeath_impl.hpp"
#include "SystemMngt/platformvaluetypes/ApplicationModeChange_impl.hpp"
#include "SystemMngt/platformvaluetypes/ApplicationStatusChange_impl.hpp"
#include "SystemMngt/platformvaluetypes/ProcessStatusChange_impl.hpp"
#include "SystemMngt/platformvaluetypes/EntityStatusChange_impl.hpp"
#include "SystemMngt/platformvaluetypes/PropertyChange_impl.hpp"
#include "SystemMngt/platformvaluetypes/UserEvent_impl.hpp"
#include "idllib/CdmwPlatformMngtSystem.stub.hpp"


using namespace Cdmw;

using namespace Common;

using namespace PlatformMngt;

using namespace CdmwPlatformMngt;

CPPUNIT_TEST_SUITE_REGISTRATION (TestTimeoutApplication);

namespace Cdmw
{
  namespace PlatformMngt
  {
    unsigned long getMin(const unsigned long a , const unsigned long b)
    {
        if ( (a < b) && (a !=0)) return a;
        else return b;
    }
    
    TestTimeoutApplication::TestTimeoutApplication()
    {
    }

    void TestTimeoutApplication::setUp ()
    {
          try
          {
          //Getting system var
          std::string system_corbaloc = "corbaloc::localhost:21869/CdmwPlatformMngtSupervision";
          orb = CORBA::ORB::_duplicate (Cdmw::TestUtils::CORBATestManager::instance ()->get_ORB ());
          CORBA::Object_var system_obj = orb->string_to_object (system_corbaloc.c_str());
          system = CdmwPlatformMngt::System::_narrow (system_obj.in ());
          
          //Setting up the startupKind parameters
          state_retrieval = CdmwPlatformMngtBase::LAST_STATE;
          startup_mode = 0;
          specified_state_id = "";
          startupKind = new CdmwPlatformMngtBase::StartupKind();
          startupKind->state_retrieval = state_retrieval;
          startupKind->startup_mode = startup_mode;
          startupKind->specified_state_id  = CORBA::string_dup( specified_state_id.c_str() );
          
          //Initialising the variables..
          obName = "SystemObserver";
          deviation = 0;
          sleeping = 500;
          
          }
          catch (const CORBA::SystemException& ex) 
          {
              std::cerr<< ex._name()<<std::endl;
              throw;
          }
          catch(...)
          {
          }
    }

    void TestTimeoutApplication::test_start_application ()
    {
        try 
        {
            std::cout<<" "<<std::endl;
            TEST_INFO("Test Starting...");
            
            //Getting the application vector.
            nbApps = system->get_number_of_applications();
            apps   = system->get_applications(nbApps,appIter.out());
            
            //Parsing all the parameter passed to the main
            parser(applicationFields);

            //Creating and Registering the observer
            SysObserver_impl* sysOb = register_observer();
            
            
            TEST_INFO("Doing a cold  restart of the system ... ");
            //Rebooting the system and Waiting for the complention of the initialisation of the system...
            std::cout<<"Waiting for the initialisation of the system ... "<<std::endl;
            
            system -> cold_restart(startupKind.in());
            
            //Waiting for the system started notification
            int count = 0;
            while (!sysOb->isStarted())
            {
                    if (sleeping == count) {CPPUNIT_ASSERT(false);}
                    sleep (1);
                    count++;
            }
            //If the system doesn't start by the sleeping time, the test fails

            std::cout<<std::endl;
            TEST_INFO("Controlling if the applications are started with the right delay ... ");
            
            for (CORBA::ULong count = 0; count<nbApps; count++)
            {
                std::string appName = apps[count]->name();
                //the delays between the task start.
                unsigned long delay = 0;
                unsigned long temp = 0;
                std::string lastPredecessor;
                std::list<std::string> Predecessors = applicationFields[appName].Predecessors; 
                
                //if it doesn't have predeccessors, all this controls are useless
                if (0 != Predecessors.size()) 
                {
                    for (std::list<std::string>::iterator it=Predecessors.begin(); it!=Predecessors.end(); ++it)
                    {
                        //Getting the real Sleep ... 
                        OsSupport::OS::Timeval t1 = sysOb->getInformation((*it),"initialising");
                        OsSupport::OS::Timeval t2 = sysOb->getInformation((*it),"initialised");
                        unsigned long realSleep = (t2.seconds - t1.seconds)*1000000 + t2.microseconds - t1.microseconds;
                        
                        //Getting the min between timeout and sleep of the predecessors
                        temp = getMin((applicationFields[(*it)].timeout*1000), realSleep);
                        
                        //Saving the max in the delay variable
                        if (temp >= delay)
                        { 
                            delay = temp;
                            lastPredecessor = *it;
                        }
                    }
                    //Getting the initialising timestamp of currApp and its last predecessor
                    OsSupport::OS::Timeval currApp      = sysOb->getInformation(appName,"initialising");
                    OsSupport::OS::Timeval predecessor  = sysOb->getInformation(lastPredecessor,"initialising");
                    
                    std::cout<<"<-------------------------------->"<<std::endl;
                    std::cout<<"Current application  : "<<appName<<" request initialisation at "<< currApp.seconds 
                                    <<", usec "<<currApp.microseconds<<std::endl;

                    std::cout<<"Its last predecessor : "<<lastPredecessor<<" request initialisation at "<< predecessor.seconds 
                                    <<", usec "<<predecessor.microseconds<<std::endl;
                                    
                    //Calculating the difference between currApp and its last predecessor
                    unsigned long diff = (1000000*(currApp.seconds - predecessor.seconds)) 
                                         + (currApp.microseconds - predecessor.microseconds);
                    //Printing the value              
                    std::cout<<"\nMisured Time < expectedDelay + deviation ? "<<std::endl;
                    std::cout<<diff<<" us < "<<delay<<" us + "<<deviation<<" us ? "<<std::endl;
                    
                    //Controlling the assert
                    CPPUNIT_ASSERT((diff < (delay + deviation)));
                    std::cout<<": OK "<<std::endl;

                }                             
            }

            CPPUNIT_ASSERT(true);
            //Destroying and unregistering the observer
            unregister_observer(sysOb);
            std::cout<<"\n"<<std::endl;
        }
        catch (const CORBA::SystemException& e)
        {
            std::cout<<e<<std::endl;
            CPPUNIT_ASSERT (false);
            throw;
        }
    }
    
    SysObserver_impl* TestTimeoutApplication::register_observer () throw () 
    {
        // Get a reference to the root POA
        CORBA::Object_var obj = orb->resolve_initial_references("RootPOA");
        PortableServer::POA_var poa = PortableServer::POA::_narrow( obj.in() );
        
        // Activate the POA manager
        PortableServer::POAManager_var mgr = poa->the_POAManager();
        mgr->activate();

        // Register event factories
        CORBA::ValueFactoryBase_var factory, old_factory;

         factory = new SystemModeChangeFactory;
        old_factory = orb->register_value_factory(
                          "IDL:thalesgroup.com/CdmwPlatformMngt/SystemModeChange:1.0",
                          factory.in() );

        factory = new SystemStatusChangeFactory;
        old_factory = orb->register_value_factory(
                          "IDL:thalesgroup.com/CdmwPlatformMngt/SystemStatusChange:1.0",
                          factory.in() );

        factory = new HostStatusChangeFactory;
        old_factory = orb->register_value_factory(
                          "IDL:thalesgroup.com/CdmwPlatformMngt/HostStatusChange:1.0",
                          factory.in() );

        factory = new ApplicationAgentDeathFactory;
        old_factory = orb->register_value_factory(
                          "IDL:thalesgroup.com/CdmwPlatformMngt/ApplicationAgentDeath:1.0",
                          factory.in() );

        factory = new ApplicationModeChangeFactory;
        old_factory = orb->register_value_factory(
                          "IDL:thalesgroup.com/CdmwPlatformMngt/ApplicationModeChange:1.0",
                          factory.in() );

        factory = new ApplicationStatusChangeFactory;
        old_factory = orb->register_value_factory(
                          "IDL:thalesgroup.com/CdmwPlatformMngt/ApplicationStatusChange:1.0",
                          factory.in() );

        factory = new ProcessMessageFactory;
        old_factory = orb->register_value_factory(
                          "IDL:thalesgroup.com/CdmwPlatformMngt/ProcessMessage:1.0",
                          factory.in() );

        factory = new ProcessStatusChangeFactory;
        old_factory = orb->register_value_factory(
                          "IDL:thalesgroup.com/CdmwPlatformMngt/ProcessStatusChange:1.0",
                          factory.in() );
        
        factory = new EntityStatusChangeFactory;
        old_factory = orb->register_value_factory(
                          "IDL:thalesgroup.com/CdmwPlatformMngt/EntityStatusChange:1.0",
                          factory.in() );

        factory = new UserEventFactory;
        old_factory = orb->register_value_factory(
                          "IDL:thalesgroup.com/CdmwPlatformMngt/UserEvent:1.0",
                          factory.in() );

        factory = new PropertyChangeFactory;
        old_factory = orb->register_value_factory(
                "IDL:thalesgroup.com/CdmwPlatformMngt/PropertyChange:1.0",
                factory.in() );
        
        TEST_INFO("Registering an observer to the system... ");
        
        //devi passargli le applicazioni per creare la mappa
        SysObserver_impl* sysOb= new SysObserver_impl(apps.in(), nbApps);
        
        observer = sysOb->_this();
        CdmwPlatformMngt::RegistrationDepth depth=FULL_HIERARCHY;
        
        //Registering the observer
        system->register_observer_ext (obName.c_str(),
                                      depth, 
                                      observer.in ());
                                            
        std::cerr << "Observer registered on System "<< " with depth = " << depth << std::endl;
        
        return sysOb;
    }
        
    void TestTimeoutApplication::unregister_observer (SysObserver_impl* sysOb) throw () 
    {
        //Unregistering the observers
        system->unregister_observer_ext (obName.c_str());
        std::cerr << "Unregistering " << obName <<std::endl;
    }  

    void TestTimeoutApplication::parser(std::map <std::string, Fields >& applicationFields)
    {
        TEST_INFO("Getting the deviation value ");
        if (1)
        {
            std::string temp ;
            temp = Cdmw::TestUtils::CORBATestManager::instance()->get_property("--deviation");
            deviation = atol( (temp.c_str() ))*1000;
            std::cout<<"The Error deviation set for this test is "<<deviation<<" us" <<std::endl;
        }
            
        /* The test takes as parameter the applications and Their initialisation timeouts.
         * These parameters are taken using the properties.
         * It must count on the number of all applications,
         * because at most the Timeouts are as many as the applications
         */
        TEST_INFO("Getting the application timeout values and its successors ...");
        //defining a container of all timeouts.  
        
        for (CORBA::ULong count = 0; count<nbApps; count++)
        {
            std::string appName = apps[count]->name();

            //Getting the Timeout value to the applications.
            std::string appTimeout    = appName+"_timeout";
            std::string appSuccessors = appName+"_successor";
            unsigned long Timeout = atol((Cdmw::TestUtils::CORBATestManager::instance()->get_property(appTimeout)).c_str()) ;
            
            //updating the map
            applicationFields[appName].timeout=Timeout;
            
            //Getting Successors
            int count = 0;
            std::string temp;
            
            do  
            {
               temp = Cdmw::TestUtils::CORBATestManager::instance()->get_property(appSuccessors+((char)(count+48)));
               //updating the map                 
               if ("" != temp) applicationFields[temp].Predecessors.push_back(appName);
               count++;
            }while  ( "" != temp );
            
            //Printing results
            std::cout<<appName<<": Timeout = "<<Timeout<<" ms"<<" Predecessors = ";
                          
            for (list<std::string>::iterator it=applicationFields[appName].Predecessors.begin(); 
                 it!=applicationFields[appName].Predecessors.end(); ++it)
            {
                 std::cout<<*it<<" ";
            }
            std::cout<<std::endl;

        }
    }
    
    
    
}/* End namespace PlatformMngt */
}/* End namespace Cdmw */





