/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO 
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2009. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */



#include <testabortprocess/CCM_Client_impl.hpp>

#include <Foundation/common/Assert.hpp>
#include <Foundation/ossupport/OS.hpp>
#include <Foundation/orbsupport/OrbSupport.hpp>
#include <Foundation/logging/CdmwLogging.stub.hpp>
#include <Foundation/logging/Logger.hpp>
#include <Foundation/osthreads/MutexGuard.hpp>
#include <SystemMngt/idllib/CdmwPlatformMngtManagedElementContexts.stub.hpp>

#include <SystemMngt/idllib/CdmwPlatformMngtSystem.stub.hpp>


namespace TestAbnormalBehaviour
{
    const std::string SYSTEM_NAME = "S1";
    const std::string APPLICATION_NAME = "CcmTestAbortProcessApplication";
    const std::string CLIENT_PROCESS_NAME = "TestAbortProcessClient1";
    const std::string SERVER_PROCESS1_NAME = "TestAbortProcessServer1";
    const std::string CLIENT_COMPONENT_NAME = "Client1";
    const std::string SERVER_COMPONENT1_NAME = "Server1";
    const std::string CLIENT_ISSUER_NAME = "TestAbortProcessClient1/Client1";
    const std::string SERVER_ISSUER1_NAME = "TestAbortProcessServer1/Server1";
    const std::string SUPERVISION_ISSUER_NAME = "CDMW.PFMMNG";
    const std::string CLIENT_NOTIFICATION_MESSAGE = "Notification message from Client";
    const CORBA::ULong CLIENT_USER_EVENT = 33;
    const std::string SERVER_NOTIFICATION_MESSAGE = "Notification message from Server";
    const std::string SERVER_NOTIFICATION_FATAL = "Fatal error from Server";
    const CORBA::ULong SERVER_USER_EVENT = 44;
    const CORBA::Long SERVER_EXIT_CODE = 10;


  const char* event_kind_to_string(CdmwPlatformMngt::Event* ev) {
        switch (ev->event_kind()) {
            case CdmwPlatformMngt::SYSTEM_MODE_CHANGE:
                return "SYSTEM_MODE_CHANGE";
            case CdmwPlatformMngt::SYSTEM_STATUS_CHANGE:
                return "SYSTEM_STATUS_CHANGE";
            case CdmwPlatformMngt::HOST_STATUS_CHANGE:
                return "HOST_STATUS_CHANGE";
            case CdmwPlatformMngt::APPLICATION_AGENT_DEATH:
                return "APPLICATION_AGENT_DEATH";
            case CdmwPlatformMngt::APPLICATION_MODE_CHANGE:
                return "APPLICATION_MODE_CHANGE";
            case CdmwPlatformMngt::APPLICATION_STATUS_CHANGE:
                return "APPLICATION_STATUS_CHANGE";
            case CdmwPlatformMngt::PROCESS_MESSAGE:
                return "PROCESS_MESSAGE";
            case CdmwPlatformMngt::PROCESS_STATUS_CHANGE:
                return "PROCESS_STATUS_CHANGE";

            case CdmwPlatformMngt::ENTITY_STATUS_CHANGE:
                return "ENTITY_STATUS_CHANGE";
            case CdmwPlatformMngt::CONFIGURATION_CHANGE:
                return "CONFIGURATION_CHANGE";
            case CdmwPlatformMngt::USER_EVENT:
                return "USER_EVENT";
            case CdmwPlatformMngt::PROPERTY_CHANGE:
                return "PROPERTY_CHANGE";
            case CdmwPlatformMngt::RESTART_REQUESTED:
                return "RESTART_REQUESTED";
            case CdmwPlatformMngt::REBOOT_REQUESTED:
                return "REBOOT_REQUESTED";
            case CdmwPlatformMngt::STOP_REQUESTED:
                return "STOP_REQUESTED";
            default:
                return "unknown event kind !";
        }
    }



//
// IDL:thalesgroup.com/CcmFtTest/ClientModule/CCM_Client:1.0
//
CCM_Client_impl::CCM_Client_impl()
{
}

CCM_Client_impl::~CCM_Client_impl()
{
    std::cerr << "   ... CCM_Client_impl::~CCM_Client_impl called!" << std::endl;
}


void CCM_Client_impl::set_session_context(Components::SessionContext_ptr ctx)
    throw(CORBA::SystemException)
{
    m_session_context = ::TestAbnormalBehaviour::CCM_Client_Context::_narrow(ctx);
}
        
::TestAbnormalBehaviour::CCM_Client_Context* CCM_Client_impl::get_context()
{
    return ::TestAbnormalBehaviour::CCM_Client_Context::_duplicate(m_session_context.in());
}


//
// ccm_activate function : it actives the component (call by the container)
//
void
CCM_Client_impl::ccm_activate()
   throw(CORBA::SystemException)
{
    std::cerr << "   ... CCM_Client_impl::ccm_activate called!" << std::endl;

    // Get this component CORBA object
    // Note: this has to be performed in a CORBA request thread; thus
    //       not in test which is a specific thread.
    TestAbnormalBehaviour::Client_var this_component =
        TestAbnormalBehaviour::Client::_nil();
    try {
        // narrow context in SessionContext for get_CCM_object() access
        Components::SessionContext_var ctx =
            Components::SessionContext::_narrow(m_session_context.in());
        
        // call SessionContext::get_CCM_object()
        CORBA::Object_var obj =
            ctx->get_CCM_object();
            
        this_component =
            TestAbnormalBehaviour::Client::_narrow(obj.in());
            
        CDMW_ASSERT(!CORBA::is_nil(this_component.in()));
            
    } catch (const CORBA::UserException& e) {
        std::cerr << "UserException during get_CCM_Object() :" 
                  << Cdmw::OrbSupport::OrbSupport::exception_to_string(e)
                  << std::endl;
        CDMW_ASSERT(false);
    } catch (const CORBA::SystemException& e) {
        std::cerr << "SystemException during get_CCM_Object() :" 
                  << Cdmw::OrbSupport::OrbSupport::exception_to_string(e)
                  << std::endl;
        CDMW_ASSERT(false);
    }


    try {   
        // save context in CORBATestManager for usage in test
        Cdmw::TestUtils::CORBATestManager::instance()->add(
            "client_context", 
            get_context());
    
        // save this component in CORBATestManager for usage in test
        Cdmw::TestUtils::CORBATestManager::instance()->add(
            "this_component", 
            this_component.in());
    
        // save this executor in CORBATestManager for usage in test
        Cdmw::TestUtils::CORBATestManager::instance()->add(
            "this_executor", 
            dynamic_cast< Session_CCM_Client_ptr > (this));
    
        // create test runner thread
        m_test_thread = new Cdmw::TestUtils::ThreadedTestRunner(
            Cdmw::TestUtils::CORBATestManager::instance()->get_controller());
        // add test to be run
        m_test_thread->addTest( 
            CppUnit::TestFactoryRegistry::getRegistry("test_abort_process").makeTest());
        // start test
        m_test_thread->start();

    } catch (const CORBA::Exception& e) {
        std::cerr << "CORBA Exception exception during test: "
                  << Cdmw::OrbSupport::OrbSupport::exception_to_string(e)
                  << std::endl;
    } catch (const std::exception& e) {
        std::cerr << "std::exception exception during test: "
                  << e.what()
                  << std::endl;
    } catch (...) {
        std::cerr << "unknown exception during test !!" << std::endl;
    }
}


//
// ccm_passivate function : it makes stop the component (call by the container)
//
void
CCM_Client_impl::ccm_passivate()
   throw(CORBA::SystemException)
{
    std::cerr << "   ... CCM_Client_impl::ccm_passivate called!" << std::endl;
    
    if (m_test_thread != 0) {
        // wait test to be finished
        m_test_thread->join();
        delete m_test_thread;
        m_test_thread = 0;
        // print test summary
        Cdmw::TestUtils::CORBATestManager::instance()->summary();
    }

} 


//
// ccm_remove function : it destroys the component (call by the container)
//
void
CCM_Client_impl::ccm_remove()
   throw(CORBA::SystemException)
{
    std::cerr << "   ... CCM_Client_impl::ccm_remove called!" << std::endl;
}


//
// Testable do_tests() operation
//
CPPUNIT_TEST_SUITE_NAMED_REGISTRATION(test_abort_process, "test_abort_process");


test_abort_process::test_abort_process()
{
    // retrieve context object
    CORBA::Object_var obj = 
        Cdmw::TestUtils::CORBATestManager::instance()->get_object("client_context");
    m_session_context = 
        ::TestAbnormalBehaviour::CCM_Client_Context::_narrow(obj.in());
        
    obj = 
        Cdmw::TestUtils::CORBATestManager::instance()->get_object("this_executor");
    m_ccm_client = dynamic_cast< CCM_Client_impl* > (obj.in());
    
    obj = 
        Cdmw::TestUtils::CORBATestManager::instance()->get_object("this_component");
    m_component_ref =
        TestAbnormalBehaviour::Client::_narrow(obj.in());
}

void
test_abort_process::do_tests()
{
    std::cerr << "   ... test_abort_process::do_tests called!" << std::endl;

    try {

        TestAbortProcess_var test_abort_process =
            m_session_context->get_connection_test_server();

       Cdmw::OsSupport::OS::sleep(3000);
        
       TEST_INFO("Testing ManagedComponentContext");

       CORBA::String_var real_component_name;
       try {
        // get ManagedComponentContext
        CORBA::Object_var obj = 
            m_session_context->resolve_service_reference("ManagedComponentContext");
        CdmwPlatformMngt::ManagedElementContexts::ComponentContext_var component_context =
            CdmwPlatformMngt::ManagedElementContexts::ComponentContext::_narrow(obj.in());
        CPPUNIT_ASSERT(!CORBA::is_nil(component_context.in()));
     
        TEST_INFO("check supervised object names");
        // get component, process, application names
        CORBA::String_var component_name = component_context->managed_element_name();
        TEST_INFO(" component_name = '" 
                  << (component_name.in() == 0 ? "null" : component_name.in())
                  << "'");
        CPPUNIT_ASSERT(component_name.in() != 0);
        CPPUNIT_ASSERT(CLIENT_COMPONENT_NAME == component_name.in());        
        
        CORBA::String_var process_name = component_context->process_name();
        TEST_INFO("      process_name = '" 
                  << (process_name.in() == 0 ? "null" : process_name.in())
                  << "'");
        CPPUNIT_ASSERT(process_name.in() != 0);
        CPPUNIT_ASSERT(CLIENT_PROCESS_NAME == process_name.in());        

        CORBA::String_var appli_name = component_context->application_name();
        TEST_INFO("      appli_name = '" 
                 << (appli_name.in() == 0 ? "null" : appli_name.in())
                 << "'");
        CPPUNIT_ASSERT(appli_name.in() != 0);
        CPPUNIT_ASSERT(APPLICATION_NAME == appli_name.in());        

        // get System supervision object
        TEST_INFO("   get System supervision object");
        CdmwPlatformMngt::External::System_var system = component_context->the_system();
        
        CdmwPlatformMngt::System_var system2 = CdmwPlatformMngt::System::_narrow(component_context->the_system());

        // register this component as System observer
        TEST_INFO("   register this component as System Observer");
        CdmwPlatformMngt::SupervisionObserver_var old_one =
            system->register_observer("TestAbnormalBehaviour_Client", 
                                      m_component_ref.in());
        
        // test System supervision object
        TEST_INFO("   test System status");
        CORBA::String_var status_info;
        CPPUNIT_ASSERT(system->get_status(status_info.out()) == 
                       CdmwPlatformMngt::SYSTEM_STARTED);
                                      
        // test Application supervision object
        TEST_INFO("   test Application status");
        CdmwPlatformMngt::External::Application_var application =
            component_context->the_application();
        CPPUNIT_ASSERT(application->get_status(status_info.out()) == 
                       CdmwPlatformMngt::APPLICATION_RUNNING);
                                      
        // test Process supervision object
        TEST_INFO("   test Process status");
        CdmwPlatformMngt::External::Process_var process =
            component_context->the_process();
        CPPUNIT_ASSERT(process->get_status(status_info.out()) == 
                       CdmwPlatformMngt::PROCESS_RUNNING);
                                      
        // test Host supervision object
        TEST_INFO("   test Host status");
        CdmwPlatformMngt::External::Host_var host =
            component_context->the_host();
        TEST_INFO(host->status());
        CPPUNIT_ASSERT(host->status() == CdmwPlatformMngt::HOST_STARTED);

        // test message notification to Supervision
        TEST_INFO("   notify message to supervision");
        component_context->notify_message(CdmwPlatformMngtBase::INF,
                                   CLIENT_NOTIFICATION_MESSAGE.c_str());
        Cdmw::OsSupport::OS::sleep(5000);
                                   
        TEST_INFO("      check that notification has been received back");
       
        CdmwPlatformMngt::Event_var event = m_ccm_client->get_last_event();
        CPPUNIT_ASSERT(event.in() != 0);
        CPPUNIT_ASSERT(event->event_kind() == CdmwPlatformMngt::PROCESS_MESSAGE);
        CdmwPlatformMngt::ProcessMessage* message_event =
            CdmwPlatformMngt::ProcessMessage::_downcast(event.in());
        CPPUNIT_ASSERT(CLIENT_ISSUER_NAME == message_event->issuer());
        CPPUNIT_ASSERT(CLIENT_NOTIFICATION_MESSAGE == message_event->info());  
  
        // test user event notification to Supervision
        TEST_INFO("   notify user event to supervision");
        CORBA::Any_var any = new CORBA::Any();
        *any <<= CLIENT_USER_EVENT;
        component_context->notify_user_event(any.in());
        Cdmw::OsSupport::OS::sleep(1000);
                                   
        TEST_INFO("      check that notification has been received back");
        event = m_ccm_client->get_last_event();
        CPPUNIT_ASSERT(event.in() != 0);
        CPPUNIT_ASSERT(event->event_kind() == CdmwPlatformMngt::USER_EVENT);
        CdmwPlatformMngt::UserEvent* user_event =
            CdmwPlatformMngt::UserEvent::_downcast(event.in());
        CORBA::ULong value;
        if (user_event->data() >>= value) {
            CPPUNIT_ASSERT(value == CLIENT_USER_EVENT);
        } else {
            CPPUNIT_FAIL("UserEvent's data doesn't contain a CORBA::ULong");
        }
    
        CPPUNIT_ASSERT(CLIENT_ISSUER_NAME == user_event->issuer());
        CPPUNIT_ASSERT(APPLICATION_NAME == user_event->application_name());
        CPPUNIT_ASSERT(CLIENT_PROCESS_NAME == user_event->process_name());
        CPPUNIT_ASSERT(Cdmw::OsSupport::OS::get_hostname()  == user_event->host_name());

        Cdmw::OsSupport::OS::sleep(1000);

        CdmwPlatformMngt::ElementNames_var elems;
        CdmwPlatformMngt::ElementNamesIterator_var it;

        // get number of hosts
        CORBA::ULong nbr_of_host = system2->get_number_of_hosts();
       
       // get the name of hosts
       system2->get_all_host_names(nbr_of_host, elems.out(), it.out());

       const char* host_name2 = elems[0U];

       std::string admin_args =  APPLICATION_NAME + " " + SERVER_PROCESS1_NAME + " " + host_name2;
     
       Cdmw::OsSupport::OS::sleep(3000); 
       TEST_INFO("test abort process - platform_admin.sh --proc-stop " + admin_args + " EMERGENCY");
       Cdmw::OsSupport::OS::create_process("platform_admin.sh", "--proc-stop " + admin_args + " EMERGENCY");      

       } catch (const CORBA::SystemException& e) {
            TEST_INFO("Exception calling Server1's facet: " << 
                      Cdmw::OrbSupport::OrbSupport::exception_to_string(e));
            CPPUNIT_ASSERT(false);

      }
    } catch (...) {
        std::cerr << "unknown exception during test " << std::endl;
    }

}

    
void CCM_Client_impl::notify(CdmwPlatformMngt::Event* event)
        throw(CORBA::SystemException)
{

    CORBA::String_var msg = event->to_string();
    std::cout << "   ... " << event_kind_to_string(event)
              << " notification received from Supervision : \n"
              << msg.in() << std::endl;

    // save event in event queue
    event->_add_ref();
    m_event_queue.push(event);

}



//
// return first unconsumed event
//
CdmwPlatformMngt::Event*
CCM_Client_impl::get_last_event()
{
    // if no event in queue, return null
    if (m_event_queue.size() == 0)
        return 0;

    // get first event from event queue
    CdmwPlatformMngt::Event* event = m_event_queue.front();
    m_event_queue.pop();

    return event;
}


};  // namespace TestAbnormalBehaviour

