/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO 
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2009. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#include "Foundation/orbsupport/OrbSupport.hpp"
#include "Foundation/ossupport/OS.hpp"
#include "ConfAndPlug/cdmwinit/CdmwInterface.hpp"
#include "Foundation/testutils/CORBATestManager.hpp"

#include "FaultTolerance/idllib/FT.stub.hpp"
#include "FaultTolerance/ftcommon/FactoryInfos.hpp"

#include "SystemMngt/platformvaluetypes/SystemModeChange_impl.hpp"
#include "SystemMngt/platformvaluetypes/SystemStatusChange_impl.hpp"
#include "SystemMngt/platformvaluetypes/HostStatusChange_impl.hpp"
#include "SystemMngt/platformvaluetypes/ApplicationAgentDeath_impl.hpp"
#include "SystemMngt/platformvaluetypes/ApplicationModeChange_impl.hpp"
#include "SystemMngt/platformvaluetypes/ApplicationStatusChange_impl.hpp"
#include "SystemMngt/platformvaluetypes/ProcessStatusChange_impl.hpp"
#include "SystemMngt/platformvaluetypes/EntityStatusChange_impl.hpp"
#include "SystemMngt/platformvaluetypes/PropertyChange_impl.hpp"
#include "SystemMngt/platformvaluetypes/UserEvent_impl.hpp"



#include <iostream>

namespace {
    const int SUCCESS = 0;
    const int FAILURE = 1;

    const int POA_THREAD_POOL_SIZE = 5;    

    const char* SERVER_GROUP_NAME="server_group";
    const char* SERVER_REPOSITORY_ID="IDL:thalesgroup.com/Cdmw_TestAbnormalBehaviourServer:1.0";
    const char* APPLICATION_NAME="CcmTestAbortProcessApplication";
    const char* SERVER1_NAME="TestAbortProcessProcessServer1";
    const char* SERVER2_NAME="TestAbortProcessProcessServer2";
    const char* SERVER3_NAME="TestAbortProcessProcessServer3";

    
}; // End anonymous namespace



void
usage(std::ostream & os, const std::string &program_name)
{
    os << "Usage:\n"
       << program_name << " options\n"
       << "\n"
       << "Options:\n"
       << "--help              Display this information.\n"
       << "--host1=<hostname>  hostname for server1.\n"
       << "--host2=<hostname>  hostname for server2.\n"
       << "--host3=<hostname>  hostname for server3.\n"
       << std:: endl;
}


int create_server_ft_group(int argc, char* argv[], CORBA::ORB_ptr orb)
{
    int i = 0;
    while(i<argc)
    {
             std::cerr << argv[i]<<std::endl;
             i++;
    } 

    std::cout << "Create " << SERVER_GROUP_NAME << " FT group..." << std::endl;
    //
    // get arguments --host1 --host2 --host3
    //
    std::string host1 = Cdmw::OsSupport::OS::get_option_value (argc, argv, "host1");
    if ((host1 == "no") || (host1=="yes")) {
        std::cerr << "host1=<hostname> option must be defined"
                  << std::endl;
        usage(std::cerr, argv[0]);
        return FAILURE;
    }
    std::string host2 = Cdmw::OsSupport::OS::get_option_value (argc, argv, "host2");
    if ((host2 == "no") || (host2=="yes")) {
        std::cerr << "host2=<hostname> option must be defined"
                  << std::endl;
        usage(std::cerr, argv[0]);
        return FAILURE;
    }
    std::string host3 = Cdmw::OsSupport::OS::get_option_value (argc, argv, "host3");
    if ((host3 == "no") || (host3=="yes")) {
        std::cerr << "host3=<hostname> option must be defined"
                  << std::endl;
        usage(std::cerr, argv[0]);
        return FAILURE;
    }
    

    // Get the Replication Manager
    CORBA::Object_var replication_mgr_obj
        = orb->resolve_initial_references("ReplicationManager");
    
    FT::ReplicationManager_var replication_mgr = 
        FT::ReplicationManager::_narrow(replication_mgr_obj.in());

    // create FT Properties
    ::FT::Properties prop;
    prop.length( 4 );
    prop[0].nam.length( 1 );
    prop[0].nam[0].id = "org.omg.ft.MinimumNumberReplicas";
    prop[0].val <<= (CORBA::UShort) 1;
    
    prop[1].nam.length( 1 );
    prop[1].nam[0].id = "org.omg.ft.ReplicationStyle";
    prop[1].val <<= (CORBA::UShort) ::FT::WARM_PASSIVE; //(CORBA::UShort) 2;
    
    prop[2].nam.length( 1 );
    prop[2].nam[0].id = "org.omg.ft.MembershipStyle";
    prop[2].val <<= (CORBA::UShort) ::FT::MEMB_APP_CTRL; // (CORBA::UShort) 0;
    
    Cdmw::FT::FactoryInfos factoryInfos;
    factoryInfos.add_ftLocation(host1, APPLICATION_NAME, SERVER1_NAME);
    factoryInfos.add_ftLocation(host2, APPLICATION_NAME, SERVER2_NAME);
    factoryInfos.add_ftLocation(host3, APPLICATION_NAME, SERVER3_NAME);    
    
    prop[3].nam.length( 1 );
    prop[3].nam[0].id = "org.omg.ft.Factories";
    prop[3].val <<= factoryInfos.get_factoryInfos();
    
    ::FT::Criteria ft_criteria;
    ft_criteria.length(2);
    ft_criteria[0].nam.length(1);
    ft_criteria[0].nam[0].id="org.omg.ft.FTProperties";
    ft_criteria[0].val <<= prop;

    ::FT::Name OGName;
    OGName.length(1);
    OGName[0].id=CORBA::string_dup(SERVER_GROUP_NAME);
    
    ft_criteria[1].nam.length(1);
    ft_criteria[1].nam[0].id="com.thalesgroup.cdmw.ft.Name";
    ft_criteria[1].val <<= OGName;
    
    // create the component's IOGR
    try {
        ::FT::GenericFactory::FactoryCreationId_var component_creation_id;
        replication_mgr->create_object(SERVER_REPOSITORY_ID, 
                                       ft_criteria, 
                                       component_creation_id.out());
    } catch(const CORBA::SystemException& e ) {
        std::cerr <<__FILE__<<" "<<__LINE__<<" "<< e << std::endl;
        throw;
    } catch (const CORBA::Exception& e ) {
        std::cerr <<__FILE__<<" "<<__LINE__<<" "<< e << std::endl;
        throw CORBA::INTERNAL();
    } 
        
    std::cout << "FT group " << SERVER_GROUP_NAME << " created." << std::endl;
    return SUCCESS;
}


void register_supervision_events_factories(CORBA::ORB_ptr orb)
{
    using namespace Cdmw::PlatformMngt;
    // Register event factories
    CORBA::ValueFactoryBase_var factory, old_factory;
    
    factory = new SystemModeChangeFactory;
    old_factory = orb->register_value_factory(
                      "IDL:thalesgroup.com/CdmwPlatformMngt/SystemModeChange:1.0",
                      factory.in() );
    
    factory = new SystemStatusChangeFactory;
    old_factory = orb->register_value_factory(
                      "IDL:thalesgroup.com/CdmwPlatformMngt/SystemStatusChange:1.0",
                      factory.in() );
    
    factory = new HostStatusChangeFactory;
    old_factory = orb->register_value_factory(
                      "IDL:thalesgroup.com/CdmwPlatformMngt/HostStatusChange:1.0",
                      factory.in() );
    
    factory = new ApplicationAgentDeathFactory;
    old_factory = orb->register_value_factory(
                      "IDL:thalesgroup.com/CdmwPlatformMngt/ApplicationAgentDeath:1.0",
                      factory.in() );
    
    factory = new ApplicationModeChangeFactory;
    old_factory = orb->register_value_factory(
                      "IDL:thalesgroup.com/CdmwPlatformMngt/ApplicationModeChange:1.0",
                      factory.in() );
    
    factory = new ApplicationStatusChangeFactory;
    old_factory = orb->register_value_factory(
                      "IDL:thalesgroup.com/CdmwPlatformMngt/ApplicationStatusChange:1.0",
                      factory.in() );
    
    factory = new ProcessMessageFactory;
    old_factory = orb->register_value_factory(
                      "IDL:thalesgroup.com/CdmwPlatformMngt/ProcessMessage:1.0",
                      factory.in() );
    
    factory = new ProcessStatusChangeFactory;
    old_factory = orb->register_value_factory(
                      "IDL:thalesgroup.com/CdmwPlatformMngt/ProcessStatusChange:1.0",
                      factory.in() );
    
    factory = new EntityStatusChangeFactory;
        old_factory = orb->register_value_factory(
                          "IDL:thalesgroup.com/CdmwPlatformMngt/EntityStatusChange:1.0",
                          factory.in() );
    
    factory = new UserEventFactory;
    old_factory = orb->register_value_factory(
                      "IDL:thalesgroup.com/CdmwPlatformMngt/UserEvent:1.0",
                      factory.in() );
    
    factory = new PropertyChangeFactory;
    old_factory = orb->register_value_factory(
            "IDL:thalesgroup.com/CdmwPlatformMngt/PropertyChange:1.0",
            factory.in() );
    }


int main(int argc, char* argv[])
{    
    int status = SUCCESS;

    //
    // help argument
    //
    if(strcmp(argv[1], "--help") == 0) {
        usage(std::cout, argv[0]);
        return SUCCESS;
    }
    
    CORBA::ORB_var orb = CORBA::ORB::_nil();

    try {
        // Call CDMW_preORBInit
        Cdmw::CdmwInit::CDMW_preORBInit(argc, argv);

        // call ORB_init
        Cdmw::OrbSupport::StrategyList orb_strategies;
        orb_strategies.add_OrbThreaded();
        orb_strategies.add_PoaThreadPool(POA_THREAD_POOL_SIZE);
        orb = Cdmw::OrbSupport::OrbSupport::ORB_init(argc, argv, orb_strategies);

        //
        // Get the root POA 
        //
        CORBA::Object_var obj = orb -> resolve_initial_references("RootPOA");
        PortableServer::POA_var rootPOA = PortableServer::POA::_narrow(obj.in());

        //
        // Run implementation (Activate the POA manager)
        //
        PortableServer::POAManager_var poaManager = rootPOA -> the_POAManager();
        poaManager -> activate();

        
        // Call CDMW_init
        Cdmw::CdmwInit::CDMW_init(orb.in(), argc, argv);

        // create group for FT servers
        status = create_server_ft_group(argc, argv, orb.in());

        if (status == SUCCESS) {
            // register ValueType factories for Supervision events,
            // since the client component will receive such events
            register_supervision_events_factories(orb.in());
            
            // init CORBATestManager
            Cdmw::TestUtils::CORBATestManager::instance()->init(orb.in(),
                                                                rootPOA.in(),
                                                                argc,
                                                                argv);
            
            // Setup signal handling for ORB to be shut down
            Cdmw::OrbSupport::OrbSupport::install_orb_shutdown_signal_handlers(orb.in());
    
            // ORB run
            orb->run();
            Cdmw::OrbSupport::OrbSupport::uninstall_orb_shutdown_signal_handlers(orb.in());
        }
        
    } catch (const Cdmw::Exception & ex) {
        std::cerr << ex.what() << std::endl;
        status = FAILURE;
    } catch(const CORBA::Exception& ex)  {        
        std::cerr << ex << std::endl;
        status = FAILURE;
    }

    if(!CORBA::is_nil(orb.in()))
    {
        try {
            // call CDMW_cleanup
            Cdmw::CdmwInit::CDMW_cleanup(orb.in());
            
            // call ORB_cleanup
            Cdmw::OrbSupport::OrbSupport::ORB_cleanup(orb.in());
            
            // destroy ORB
            orb->destroy();
        }
        catch(const CORBA::Exception& ex)
        {           
            std::cerr << ex << std::endl;
            status = FAILURE;
        }
    }        
    return status;    
}


