/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI.
 * Copyright (C) SELEX SISTEMI INTEGRATI 2004-2009. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

#ifndef CARDAMOM_CLOCK_CONTROLLER_IMPL_HPP
#define CARDAMOM_CLOCK_CONTROLLER_IMPL_HPP

#include "Foundation/orbsupport/RefCountServantBase.hpp"

#include <Time/idllib/CdmwTime.skel.hpp>
#include <Time/idllib/CdmwClockState.stub.hpp>
#include "clockservice/config.hpp"
#include "clockservice/TaoTimer.hpp"
#include "clockservice/ClockStatus.hpp"

#include "ace/Reactor.h"
#include "ace/Synch.h"
#include "ace/Event_Handler.h"
#include "clockservice/TimerAtomicCount.hpp"


namespace Cdmw
{
    namespace clock
    {
        namespace policy
        {
            class ControllerUpdate_impl;
        } /* policy */
    } /* clock */
} /* Cdmw */

namespace Cdmw
{
    namespace clock
    {
        class IClock;
        class Executor_impl;
        class ControllerEnforceIntervalPolicy;
    } /* clock */
} /* Cdmw */

namespace Cdmw
{
    namespace clock
    {

        /**
         * @brief This class implements the basic Controller functionalities
         * described in the EVoT specification.  
         *
         * Allows control of periodic 
         * execution after the appropriate object has been registered with 
         * the clock.
         *
         * @author Angelo Corsaro <acorsaro@selex-si.com>
         * @author Fabrizio Morciano<fmorciano@selex-si.com>
         * @author Marco Macellari <mmacellari@progesi.it>
         */
        class CDMW_API ControllerEx_impl 
            : public virtual POA_Cdmw::clock::ControllerEx,
              public virtual Cdmw::clock::ControllerEx,
              public virtual Cdmw::OrbSupport::RefCountServantBase
        {
            private:

                /**
                 * This class implements an <code>CDMW_Event_Handler</code>
                 * and takes care of notifying the registered <code>Periodic</code>
                 * instance once the period ellapses.
                 */ 
                class TimeoutHandler : public CDMW_Event_Handler
                {
                    public:

                        enum {INF_EXEC_LIMIT = 0};

                    public:

                        TimeoutHandler(ControllerEx_impl*                                 controller,
                                       CosClockService::PeriodicExecution::Periodic_ptr periodic,
                                       CDMW_Recursive_Thread_Mutex&                      mutex);

                        virtual ~TimeoutHandler();

                    public:

                        virtual int handle_timeout(const CDMW_Time_Value&, const void* act = 0);

                    public:

                        long&             executionLimit();
                        const long&       executionLimit() const;
                        TimerAtomicCount& executions();

                    public:

                        inline void set_periodic(CosClockService::PeriodicExecution::Periodic_ptr on)
                        {
                            periodic_ = CosClockService::PeriodicExecution::Periodic::_duplicate(on);
                        }

                        inline CosClockService::PeriodicExecution::Periodic_ptr get_periodic()
                        {
                            return periodic_.in();
                        }

                    private:

                        ControllerEx_impl*                                 controller_;
                        CosClockService::PeriodicExecution::Periodic_var periodic_;
                        TimerAtomicCount                                 executions_;
                        long                                             executionsLimit_;
                        CDMW_Recursive_Thread_Mutex&                      controller_mutex_;
                        CDMW_Recursive_Thread_Mutex                       timeout_handler_mutex_;
                };

            public:

                // Friend classes declarations.
                friend class ControllerEnforceIntervalPolicy;
                friend class policy::ControllerUpdate_impl;
                friend class TimeoutHandler;

            public:

                /**
                 * Constructor: creates a <code>ControllerEx_impl</code> with an
                 * ssociated <code>Periodic</code> and <code>Reactor</code>.
                 *
                 * @param clock The clock used by this controller to measure time.
                 *
                 * @param periodic The <code>Periodic</code> instance that will
                 *         be notified when the programmed time interval expires.
                 *
                 * @param reactor The reactor that will be used to register timeout 
                 *         handlers.
                 */
                ControllerEx_impl(Cdmw::clock::Executor_impl&                      clock,
                                CosClockService::PeriodicExecution::Periodic_ptr periodic,
                                CDMW_Reactor*                                     reactor);

                /**
                 * Destructor: release resources associated with the instance.
                 */
               virtual ~ControllerEx_impl();


               // ----- CORBA control methods. -----

               /**
                * Initiates periodic execution with a specified period for a 
                * specified count of executions. Specifying an execution limit 
                * of 0 is interpreted as an unbounded number of executions. 
                *
                * @param period the period after which this controller will
                *        schedule the execution of the associated 
                *        <code>Periodic</code>
                *
                * @param offset this parameter may be used to delay the start 
                *        of the first execution. 
                *
                * @param execution_limit the maximum number of periodic executions
                *        that have to be scheduled. an execution limit 
                *        of 0 is interpreted as an unbounded number of executions.
                *
                * @param params This parameter will be passed to each invocation.
                */
                virtual void start(TimeBase::TimeT   period, 
                                   TimeBase::TimeT   offset,
                                   CORBA::ULong      execution_limit,
                                   const CORBA::Any& params)
                    throw (CORBA::SystemException);

                /**
                 * Identical to the start operation except that the 
                 * <code>at_time</code> parameter specifies an absolute 
                 * time for the start of the first execution.
                 *
                 * @param  period the period after which this controller will
                 *         schedule the execution of the associated 
                 *         <code>Periodic</code>
                 *
                 * @param at_time specifies an absolute time for the start 
                 *        of the first execution.
                 *
                 * @param execution_limit the maximum number of periodic executions
                 *        that have to be scheduled. an execution limit 
                 *        of 0 is interpreted as an unbounded number of executions.
                 *
                 * @param params This parameter will be passed to each invocation.
                 */
                virtual void start_at(TimeBase::TimeT   period,
                                      TimeBase::TimeT   at_time,
                                      CORBA::ULong      execution_limit,
                                      const CORBA::Any& params)
                    throw (CORBA::SystemException,
                           CosClockService::PeriodicExecution::Controller::TimePast);

                /**
                 * Pauses periodic execution.
                 */ 
                virtual void pause()
                    throw (CORBA::SystemException);

                /**
                 * Resumes periodic execution.
                 */
                virtual void resume()
                    throw (CORBA::SystemException);

                /**
                 * Resumes periodic execution at a particular time.
                 *
                 * @param at_time time at which the execution has to
                 *        be resumed.
                 *
                 * @throw CosClockService::PeriodicExecution::Controller::TimePast
                 */
                virtual void resume_at(TimeBase::TimeT at_time)
                    throw (CORBA::SystemException,
                           CosClockService::PeriodicExecution::Controller::TimePast);

                /**
                 * Terminates periodic execution.
                 */
                virtual void terminate()
                    throw (CORBA::SystemException);


                // ----- CORBA getter methods. -----

                /**
                 * Get the update policy.
                 */
                virtual  Cdmw::clock::policy::ControllerUpdateID get_update_policy ()
                    throw (CORBA::SystemException);

                /**
                 * Get the controller time origin.
                 */
                virtual TimeBase::TimeT get_controller_time_origin ()
                    throw (CORBA::SystemException);

                /**
                 * Reports the number of executions that have already been initiated.
                 */
                virtual CORBA::ULong executions()
                    throw (CORBA::SystemException);


                // ----- CORBA setter methods. -----

                /**
                 * Override the update policy.
                 */
                virtual void set_update_policy (::Cdmw::clock::policy::ControllerUpdateID id )
                    throw (CORBA::SystemException,
                           ::Cdmw::clock::policy::ControllerUpdateAdmin::Unavailable);

                /**
                 * Override the controller time origin.
                 */
                virtual void set_controller_time_origin (TimeBase::TimeT time_origin) 
                    throw (CORBA::SystemException);

        virtual TimeBase::TimeT get_period()
                    throw (CORBA::SystemException);

        virtual TimeBase::TimeT get_next_expiration_time()
                    throw (CORBA::SystemException);

                // ----- non-CORBA getter methods. -----

                Cdmw::clock::Executor_impl*                      retrieve_executor();
                Cdmw::clock::Status                              get_status();
                CORBA::Any                                       get_parameter();
                CORBA::ULong                                     execution_limit();
                CosClockService::PeriodicExecution::Periodic_ptr periodic();

            private:

                // Compute next triggering virtual time from now.
                TimeBase::TimeT compute_next_trigger_offset();

                // verify that rate is good and return it
                double checked_rate() throw (CORBA::BAD_PARAM);

                // verify that offset is good and return an absolute
                // offset good for positive and negative time controller
                TimeBase::TimeT checked_offset(TimeBase::TimeT offset);

                // compute the correct period for a Controller with a rate != 1
                void update_period(TimeBase::TimeT period);

                // compute the correct offset for a Controller with a rate != 1
                void update_offset(TimeBase::TimeT offset);

                /**
                 * Perform timer reactor thread scheduling.
                 */ 
                void scheduleTimerLockFree(TimeBase::TimeT period, 
                                           TimeBase::TimeT offset); 

                /**
                 * These methods cancel the timer without actually changing its state.
                 */
                void cancelTimer();
                void cancelTimerLockFree();

                /**
                 * Thess methods freeze the timer without actually changing its state.
                 */
                void freezeTimer();
                void freezeTimerLockFree();

                // -- Disable Copy Ctor and Assignment Operator --
                ControllerEx_impl(ControllerEx_impl&);
                ControllerEx_impl& operator=(const ControllerEx_impl&);

            public:

                // Methods added to allow for FT state transfer.
                virtual void notify_status_lock_free();
                virtual void refresh_data_lock_free(Cdmw::clock::ControllerData& data);
                void         restore_object_lock_free(const Cdmw::clock::ControllerData& data);

            private:

                // Attributes of the controller
                CDMW_Recursive_Thread_Mutex              m_mutex;
                Cdmw::clock::policy::ControllerUpdateID m_ID;
                Cdmw::clock::Executor_impl&             m_clock;
                CDMW_Reactor*                            m_reactor;
                TimeoutHandler                          m_handler;
                long                                    m_timer_key;

                Cdmw::clock::Status                     m_status;
                TimeBase::TimeT                         m_controller_time_origin;
                TimeBase::TimeT                         m_real_period; 
                CORBA::Any                              m_any;

                CDMW_Time_Value                          m_period;
                CDMW_Time_Value                          m_delay;
        };
    } /* clock */
} /* Cdmw */

#endif /* CARDAMOM_CLOCK_CONTROLLER_IMPL_HPP */
