/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI.
 * Copyright (C) SELEX SISTEMI INTEGRATI 2004-2009. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

#ifdef WIN32
#pragma warning( disable : 4786 )
#endif

#include <algorithm>
#include <iostream>
#include <fstream>
#include <sstream>
#include <string>
#include <vector>

#include <Time/clockservice/CdmwCosClock.hpp>
#include <Time/idllib/CosClockService.stub.hpp>

#include <clockservice/ControlledClock_impl.hpp>
#include "clockservice/Macro.hpp"
#include <orbsvcs/orbsvcs/Property/CosPropertyService_i.h>

#ifndef NOT_FEDERATION
# include <clockservice/FederatedControlledClock.hpp>
#endif

#include "clockservice/ClockCatalog_impl.hpp"
#include "clockservice/Clock_impl.hpp"
#include "clockservice/ClockNtp.hpp"
#include "clockservice/UTCTimeService_impl.hpp"
#include "clockservice/UTC_impl.hpp"
#include "clockservice/Executor_impl.hpp"
#include "clockservice/ControlledExecutor_impl.hpp"
#include "clockservice/ControllerUpdateAdmin_impl.hpp"
#include "clockservice/ControllerCancelAllPolicy.hpp"
#include "clockservice/ControllerNoActionPolicy.hpp"
#include "clockservice/ControllerEnforceIntervalPolicy.hpp"
#include "clockservice/ControllerEnforceDeadlinePolicy.hpp"
#include "clockservice/TaoTimer.hpp"

#include <Foundation/common/Assert.hpp>
#include <Foundation/osthreads/Mutex.hpp>

#ifndef NOT_LOCALISATION
# include <Foundation/common/Options.hpp>
# include <Foundation/common/String.hpp>
# include <Foundation/orbsupport/OrbSupport.hpp>
# include <Foundation/ossupport/OS.hpp>
#endif

#include "Foundation/commonsvcs/naming/NamingInterface.hpp"
#include "Foundation/ossupport/SystemProperties.hpp"
#include "Repository/idllib/CdmwNamingAndRepository.stub.hpp"
#include "Repository/repositoryinterface/RepositoryInterface.hpp"
// add CDMW traces
#include <Foundation/logging/Logger.hpp>
#include <Foundation/logging/LogManager.hpp>


using namespace std;

using namespace CosClockService;

using namespace Cdmw::clock;

using namespace Cdmw;



// Thread used for JNI implementation

class CppClockThread : public OsSupport::Thread
{

public:

    CppClockThread ( CORBA::ORB_ptr orb )
    {
        m_orb = CORBA::ORB::_duplicate ( orb );
    }

    ~CppClockThread() throw()
    {
        try
        {
            m_orb->shutdown ( 1 );
            m_orb->destroy();
            this->join();

        }
        catch ( const CORBA::SystemException& e )
        {
            std::cout << "CppClockThread:>> CORBA system exception caught: " << e._info().rep() << std::endl;

        }
        catch ( const std::exception& e )
        {
            std::cout << "CppClockThread:>> std exception caught: " << e.what() << std::endl;

        }
        catch ( ... )
        {
            std::cout << "CppClockThread:>> unexpected exception caught" << std::endl;
        }

    }

    void
    run() throw()
    {
        this->m_orb->run();
    }


private:

    CORBA::ORB_var m_orb;
};




namespace
{
std::string HELP_ME_OPTION ( "--help" );
std::string TIMER_WHEEL_SPOKES_OPTION ( "--CdmwTimerWheelSpokes" );
std::string TIMER_WHEEL_RESOLUTION_OPTION ( "--CdmwTimerWheelResolution" );
std::string REACTOR_THREADS_OPTION ( "--CdmwReactorThreads" );
std::string VIRTUAL_CLOCK_NUM_OPTION ( "--CdmwVirtualClocks" );
std::string MESSAGE_RESEND_OPTION ( "--CdmwMessageResend" );
std::string CLOCK_CATALOG_IOR_OPTION ( "--CdmwClockCatalogIORFile=<filename>" );
std::string LOCAL_CLOCK_IOR_OPTION ( "--CdmwLocalClockIORFile=<filename>" );
std::string FEDERATED_CTRL_CLOCK_OPTION ( "--CdmwFederatedCtrlClock" );
std::string NAR_SERVICE_OPTION ( "--CdmwRepositoryURL" );
std::string NAMING_INIT_REF ( "NameService=" );

std::string clock_catalog_file_name ( "ClockCatalog.ior" );
std::string policycontrollerupdateadmin_file_name
( "PolicyControllerUpdateAdmin.ior" );
std::string local_clock_file_name ( "LocalClock.ior" );
std::string CLOCK_CATALOG_NAME ( "ClockService" );
std::string LOCAL_CLOCK_NAME ( "LocalClock" );
std::string POLICY_CONTROLLER_UPDATE_ADMIN
( "PolicyControllerUpdateAdmin" );

std::string CLOCK_SERVICE_PORT ( "4883" );

std::string JNI_CLOCK_SERVICE_PORT ( "60883" );

std::string TOPIC_UPDATE_MANAGER_CORBALOC
( "corbaloc:miop:1.0@1.0-TopicUpdateManager-1/225.1.1.225:15555" );
std::string FEDERATION_MULTICAST_PROPERTY
( "Cdmw.TMG.FEDERATION_ADDRESS" );

int federation_id = 0;
int send_repetitions = 2;
bool federation_off = false;
size_t reactor_threads = 10;
size_t timer_wheel_spokes = 8192;
size_t timer_wheel_resolution = 1000; // 1ms
int max_virtual_clock = 10;
std::string nar_service_url ( "no" ); // URL to Naming Service
bool isNamingObjectID = false; // check for NameService ObjectID
CORBA::ORB_var jni_orb = CORBA::ORB::_nil(); // orb used for JNI calls
static bool close_performed = false;
static bool clockInitialised = true;



//--------------------------------------------------------------------------

template <class T>
bool write_obj_to_ior ( CORBA::ORB_ptr orb,
                        T& ob,
                        const std::string& file_name )
{
    std::ofstream out_file ( file_name.c_str() );

    if ( !out_file )
        return true;

    CORBA::String_var
    str = orb->object_to_string ( ob.in() );

    out_file << str.in();

    out_file.close();

    return false;
}

//-----------------------------------------------------------------------------


void print_help ( char* appname )
{
    std::cerr << "USAGE: \n "
    << appname << "\n"
    << " [--CdmwRepositoryURL=IOR|corbaloc|corbaname|file]\n"
    << " [--CdmwReactorThreads=<threads num> (default 10)]\n"
    << " [--CdmwOrbThreads==<threads num> (default 10)]\n"
    << " [--CdmwTimerWheelSpokes==<spokes> (default 8192)]\n"
    << " [--CdmwTimerWheelResolution=<resolution in usec> (default 1ms)]\n"
    << " [--CdmwVirtualClocks=<number of virtual clocks> (default=10)]\n"
    << " [--CdmwMessageResend=<number of resend> (default 2)]\n"
    << " [--CdmwClockCatalogIORFile=<filename> (default ClockCatalog.ior)]\n"
    << " [--CdmwLocalClockIORFile=<filename> (default LocalClock.ior)]\n"
    << " [--CdmwFederatedCtrlClock=on|off (default is on)\n"
    << " [--help]\n"
    << std::endl;
}

bool check_arg ( int ac, char* av[], const char *what )
{
    std::ostringstream os;

    for ( int n = 0; n < ac; n++ )
        os << av[n] << " ";

    std::string s1 = os.str();

// We check if in the current argument we have, the option
    if ( s1.find ( what ) == std::string::npos )
        return false;
    else
        return true;
}

//-----------------------------------------------------------------------------
void parse_args ( int argc, char* argv[] )
{
#ifndef WIN32
// -- HELP_ME_OPTION
    std::string arg_value =
        Cdmw::OsSupport::OS::get_option_value ( argc,
                                                argv,
                                                HELP_ME_OPTION );

    if ( arg_value == "yes" )
    {
        print_help ( argv[0] );
        clockInitialised = false;
        exit ( -1 );
    }

// -- TIMER_WHEEL_SPOKES_OPTION
    arg_value =
        Cdmw::OsSupport::OS::get_option_value ( argc,
                                                argv,
                                                TIMER_WHEEL_SPOKES_OPTION );

    if ( arg_value != "no" )
        timer_wheel_spokes =
            atoi ( arg_value.c_str() );

// -- TIMER_WHEEL_RESOLUTION_OPTION
    arg_value =
        Cdmw::OsSupport::OS::get_option_value ( argc,
                                                argv,
                                                TIMER_WHEEL_RESOLUTION_OPTION );

    if ( arg_value != "no" )
        timer_wheel_resolution =
            atoi ( arg_value.c_str() );

// -- REACTOR_THREADS_OPTION
    arg_value =
        Cdmw::OsSupport::OS::get_option_value ( argc,
                                                argv,
                                                REACTOR_THREADS_OPTION );

    if ( arg_value != "no" )
        reactor_threads =
            atoi ( arg_value.c_str() );

// -- VIRTUAL_CLOCK_NUM_OPTION
    arg_value =
        Cdmw::OsSupport::OS::get_option_value ( argc,
                                                argv,
                                                VIRTUAL_CLOCK_NUM_OPTION );

    if ( arg_value != "no" )
        max_virtual_clock =
            atoi ( arg_value.c_str() );

// -- MESSAGE_RESEND_OPTION
    arg_value =
        Cdmw::OsSupport::OS::get_option_value ( argc,
                                                argv,
                                                MESSAGE_RESEND_OPTION );

    if ( arg_value != "no" )
        send_repetitions =
            atoi ( arg_value.c_str() );


// -- CLOCK_CATALOG_IOR_OPTION
    arg_value =
        Cdmw::OsSupport::OS::get_option_value ( argc,
                                                argv,
                                                MESSAGE_RESEND_OPTION );

    if ( arg_value != "no" )
        clock_catalog_file_name = arg_value;

// -- LOCAL_CLOCK_IOR_OPTION
    arg_value =
        Cdmw::OsSupport::OS::get_option_value ( argc,
                                                argv,
                                                MESSAGE_RESEND_OPTION );

    if ( arg_value != "no" )
        local_clock_file_name = arg_value;

// -- NAR_SERVICE_OPTION
    arg_value =
        Cdmw::OsSupport::OS::get_option_value ( argc,
                                                argv,
                                                NAR_SERVICE_OPTION );

    if ( arg_value != "no" )
        nar_service_url = arg_value;

    // check if NameService ObjectID is defined in command line args
    isNamingObjectID = check_arg ( argc,
                                   argv,
                                   NAMING_INIT_REF.c_str() );

#ifndef NOT_FEDERATION
// -- FEDERATED_CTRL_CLOCK_OPTION
    arg_value =
        Cdmw::OsSupport::OS::get_option_value ( argc,
                                                argv,
                                                FEDERATED_CTRL_CLOCK_OPTION );

    if ( arg_value == "off" )
        federation_off = true;

#endif
#ifndef NOT_LOCALISATION
// -- LOCALISATION_PORT_OPTION
    arg_value =
        Cdmw::OsSupport::OS::get_option_value ( argc,
                                                argv,
                                                Cdmw::Common::Options::LOCALISATION_SERVICE_OPTION );

    if ( arg_value != "no" )
        CLOCK_SERVICE_PORT = arg_value;

#endif
// -- JNI_LOCALISATION_PORT_OPTION
    arg_value =
        Cdmw::OsSupport::OS::get_option_value ( argc,
                                                argv,
                                                "--CdmwClockServicePort" );

    if ( arg_value != "no" )
    {
        JNI_CLOCK_SERVICE_PORT = arg_value;
        //std::ostringstream oss;
        // oss << "Clock Service will start on PORT = " << JNI_CLOCK_SERVICE_PORT;
        // CDMW_INFO(Cdmw::Logging::Logger::Get_Logger(CLOCKSERVICE_LOGGER), oss.str());
    }

#endif
}

///////////////////////////////////////////////////////////////////////////

class Reactor_Worker : public CDMW_Task_Base
{

    CDMW_Reactor *m_reactor;

public:

    Reactor_Worker ( CDMW_Reactor *ref )
            : m_reactor ( ref )
    {
    }

    Reactor_Worker::~Reactor_Worker()
    {
    }

    int
    Reactor_Worker::svc ( void )
    {
        int result = m_reactor->run_reactor_event_loop();

        if ( result == -1 )
        {
            std::cerr << "ClockService:>> Error handling events" << std::endl;
            CDMW_ERROR ( Cdmw::Logging::Logger::Get_Logger ( CLOCKSERVICE_LOGGER ), "ERROR: Reactor Error handling events!" );
            return 1;
        }

        return 0;
    }
};

} // end of unnamed namespace

///////////////////////////////////////////////////////////////////////////////

struct CosClock_impl
{
    PortableGroup::GOA_var m_goa;
    CORBA::ORB_var m_orb;

    std::auto_ptr<Reactor_Worker> m_reactor_worker;
    std::auto_ptr<CDMW_Reactor> m_reactor;
    std::auto_ptr<CDMW_TP_Reactor> m_tp_reactor;

    CosClock_impl() :
            m_goa ( 0 ),
            m_orb ( 0 )
    {
    }

    ~CosClock_impl()
    {
        if ( ( clockInitialised ) && ( !close_performed ) )
            CosClock::close();

    }
};

///////////////////////////////////////////////////////////////////////////////

static
std::auto_ptr<CosClock_impl> s_PIMPL ( 0 );

///////////////////////////////////////////////////////////////////////////////

bool
CosClock::init ( CORBA::ORB_ptr orb,
                 int argc,
                 char *argv[] )
{
    // initialize the log manager
    Cdmw::Logging::LogManager::Init ( argc, argv );

    try
    {


        try
        {
            ( void* ) Cdmw::OsSupport::SystemProperties::Init ( argc, argv );

        }
        catch ( const BadParameterException & e )
        {
            CDMW_ERROR ( Cdmw::Logging::Logger::Get_Logger ( CLOCKSERVICE_LOGGER ), "Error during SystemProperties Init" );

        }

        if ( s_PIMPL.get() )
            return true;

        s_PIMPL = std::auto_ptr<CosClock_impl> ( new CosClock_impl ) ;

        // just a ref
        CosClock_impl &pimpl_ref = * ( s_PIMPL.get() );

        parse_args ( argc, argv );

        // reference to orb
        pimpl_ref.m_orb = CORBA::ORB::_duplicate ( orb );

        try
        {
            CORBA::Object_var obj ;
            // reference to POA

            try
            {
                obj =
                    pimpl_ref.m_orb->resolve_initial_references ( "RootPOA" );

            }
            catch ( const CORBA::SystemException& e )
            {
                ostringstream oss;
                oss << "\n----------------------------------------------------------------\n"
                << "ERROR : SystemException retriving the reference to root POA\n\n"
                << "----------------------------------------------------------------\n"
                << "WARNING:\n"
                << "  Use the options --CdmwClockServicePort=<...>\n"
                << "  if there is another instance of ClockService on the same host, and\n"
                << "  -DCdmw.TMG.FEDERATION_ADDRESS=<...>\n"
                << "  if the federation has been activated\n"
                << "-----------------------------------------------------------------\n";

                CDMW_ERROR ( Cdmw::Logging::Logger::Get_Logger ( CLOCKSERVICE_LOGGER ),
                             oss.str() );
                clockInitialised = false;
                exit ( -1 );
            }

            pimpl_ref.m_goa = PortableGroup::GOA::_narrow ( obj.in () );


            if ( CORBA::is_nil ( pimpl_ref.m_goa.in () ) )
            {
                std::cerr << "ClockService:>> Panic -- nil RootPOA" << std::endl;
                CDMW_ERROR ( Cdmw::Logging::Logger::Get_Logger ( CLOCKSERVICE_LOGGER ),
                             "ERROR: nil RootPOA!" );
                clockInitialised = false;
                exit ( -1 );
            }

            PortableServer::POAManager_var mgr =

                pimpl_ref.m_goa->the_POAManager ();

            mgr->activate ();

        }
        catch ( const CORBA::SystemException& e )
        {
            clockInitialised = false;
            CDMW_ERROR ( Cdmw::Logging::Logger::Get_Logger ( CLOCKSERVICE_LOGGER ),
                         "error retrive the reference to PortableGroup::GOA" );
            exit ( -1 );
        }

///////////////////////////////////////////////////////////////////////////////
// -- Create Thread Pool Reactor


        CDMW_INFO ( Cdmw::Logging::Logger::Get_Logger ( CLOCKSERVICE_LOGGER ), "Create Thread Pool Reactor" );

        pimpl_ref.m_tp_reactor = std::auto_ptr<CDMW_TP_Reactor> ( new CDMW_TP_Reactor );

        pimpl_ref.m_reactor =
            std::auto_ptr<CDMW_Reactor> ( new CDMW_Reactor ( pimpl_ref.m_tp_reactor.get() ) );

        CDMW_Timer_Wheel* m_timer_queue = new CDMW_Timer_Wheel ( timer_wheel_spokes, timer_wheel_resolution );

        pimpl_ref.m_reactor->timer_queue ( m_timer_queue );

        // replace reactor

        int retval;

        pimpl_ref.m_reactor_worker =
            std::auto_ptr<Reactor_Worker> ( new Reactor_Worker ( pimpl_ref.m_reactor.get() ) );

        retval =
            pimpl_ref.m_reactor_worker->activate ( THR_NEW_LWP | THR_JOINABLE, reactor_threads );

        if ( retval != 0 )
        {
            std::cerr << "ClockService:>> Cannot activate the Thread Pool\n" << std::endl;
            CDMW_ERROR ( Cdmw::Logging::Logger::Get_Logger ( CLOCKSERVICE_LOGGER ),
                         "ERROR: Cannot activate the Thread Pool!" );
            return false;
        } 
        clockInitialised = true;

///////////////////////////////////////////////////////////////////////////////
// Registration for ControllerUpdateAdmin

        // registration
        Cdmw::clock::policy::ControllerUpdateAdmin_impl* ref2controllerupdatepolicyadmin
        = Cdmw::clock::policy::ControllerUpdateAdmin_impl::instance();
     

        Cdmw::clock::policy::ControllerUpdateAdmin_var admin_var
        = ref2controllerupdatepolicyadmin->_this();

        // now memory is under POA control

        Cdmw::clock::ControllerCancelAllPolicy*
        cancelAllPolicy = new Cdmw::clock::ControllerCancelAllPolicy();

        Cdmw::clock::policy::ControllerUpdate_ptr
        ca_policy = cancelAllPolicy->_this();

        admin_var->_cxx_register ( Cdmw::clock::policy::CANCEL_ALL,
                                   ca_policy );

        Cdmw::clock::ControllerEnforceIntervalPolicy*
        enforceIntervalPolicy = new Cdmw::clock::ControllerEnforceIntervalPolicy();

        Cdmw::clock::policy::ControllerUpdate_ptr
        ei_policy = enforceIntervalPolicy->_this();

        admin_var->_cxx_register ( Cdmw::clock::policy::ENFORCE_INTERVAL,
                                   ei_policy );

        Cdmw::clock::ControllerEnforceDeadlinePolicy*
        enforceDeadlinePolicy = new Cdmw::clock::ControllerEnforceDeadlinePolicy();

        Cdmw::clock::policy::ControllerUpdate_ptr
        ed_policy = enforceDeadlinePolicy->_this();

        admin_var->_cxx_register ( Cdmw::clock::policy::ENFORCE_DEADLINE,
                                   ed_policy );

        Cdmw::clock::ControllerNoActionPolicy*
        emptyPolicy = new Cdmw::clock::ControllerNoActionPolicy();

        Cdmw::clock::policy::ControllerUpdate_ptr
        empty_policy = emptyPolicy->_this();

        admin_var->_cxx_register ( Cdmw::clock::policy::NO_ACTION,
                                   empty_policy );


///////////////////////////////////////////////////////////////////////
// write ControllerUpdateAdmin ior
#ifndef NOT_LOCALISATION
        try
        {
            OrbSupport::OrbSupport::bind_object_to_corbaloc ( pimpl_ref.m_orb.in(),
                    POLICY_CONTROLLER_UPDATE_ADMIN,
                    admin_var.in() );
            pimpl_ref.m_orb->register_initial_reference ( POLICY_CONTROLLER_UPDATE_ADMIN.c_str(),
                    admin_var.in() );

        }
        catch ( const CORBA::BAD_PARAM& e )
        {
            std::ostringstream oss;
            oss << "ORB Exception : Already bound >> " << e;
            std::cerr << oss << std::endl;
            CDMW_ERROR ( Cdmw::Logging::Logger::Get_Logger ( CLOCKSERVICE_LOGGER ), oss.str() );
            exit ( -1 );
        }
        catch ( const CORBA::SystemException& e )
        {
            std::ostringstream oss;
            oss << "ORB Exception : >> " << e;
            std::cerr << oss << std::endl;
            CDMW_ERROR ( Cdmw::Logging::Logger::Get_Logger ( CLOCKSERVICE_LOGGER ), oss.str() );
            exit ( -1 );

        }

#else
        pimpl_ref.m_orb->register_initial_reference ( POLICY_CONTROLLER_UPDATE_ADMIN.c_str(),
                admin_var.in() );

        // write clockcatalog ior
        write_obj_to_ior ( pimpl_ref.m_orb.in(),
                           admin_var,
                           policycontrollerupdateadmin_file_name );

#endif
///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
// Registration for ClockCatalog

        // ClockCatalog registration
        ClockCatalog_impl* ref2clockcatalog = new ClockCatalog_impl;

        ClockCatalog_var clockcatalog_var = ref2clockcatalog->_this();

        // now memory is under POA control

        // UTC Registration
        UtcTimeService_impl* ref2utctimeservice = new UtcTimeService_impl;

        UtcTimeService_var utctimeservice_var = ref2utctimeservice->_this();

        // register a valid clock
        ClockCatalog::ClockEntry utcEntry;

        utcEntry.subject = UtcTimeService::_duplicate ( utctimeservice_var.in() );

        utcEntry.name = CORBA::string_dup ( "UTCTimeService" );

        ref2clockcatalog->_cxx_register ( utcEntry );


////////////////////////////////////////////////////////////////////////
// -- Add Controlled Clocks to Clock Catalog --

        string controlledclock ( "ControlledClock" );

        for ( int i = 0;
                i < max_virtual_clock;
                ++i )
        {
            //pimpl_ref.m_controlled_clocks.push_back(new ControlledClock_impl);
            ControlledClock_impl< ::Cdmw::OsSupport::Mutex >* ref2controlledclock =
                new ControlledClock_impl< ::Cdmw::OsSupport::Mutex >;
            ControlledClockEx_var controlledClock = ref2controlledclock->_this();

            string name ( controlledclock );
            ostringstream oss;

            if ( i > 0 )
            {
                oss << i;
                name += oss.str();
            }

            if ( i == 0 )
            {
                CDMW_ASSERT ( name == controlledclock );

            }
            else
            {
                CDMW_ASSERT ( name != controlledclock );
            }

            ClockCatalog::ClockEntry controlledClockEntry;

            controlledClockEntry.subject = ControlledClockEx::_duplicate ( controlledClock.in() );
            controlledClockEntry.name = CORBA::string_dup ( name.c_str() );

            ref2clockcatalog->_cxx_register ( controlledClockEntry );
        }

#ifndef NOT_FEDERATION
////////////////////////////////////////////////////////////////////////
// -- Add Federated Controlled Clocks to Clock Catalog --

        ControlledClockEx_var federatedControlledClock;

        ControlledExecutor_var federatedControlledExecutor;

        if ( !federation_off )
        {
            string federatedCtrlClockName ( "FederatedControlledClock" );
            string federatedCtrlExecutorName ( "FederatedControlledExecutor" );

            try
            {
                // get TOPIC_UPDATE_MANAGER_CORBALOC systemProperty

                std::string TOPIC_UPDATE_MANAGER_CORBALOC_prop =
                    Cdmw::OsSupport::SystemProperties::Get_Property ( FEDERATION_MULTICAST_PROPERTY );

                if ( TOPIC_UPDATE_MANAGER_CORBALOC_prop == "" )
                    TOPIC_UPDATE_MANAGER_CORBALOC_prop = TOPIC_UPDATE_MANAGER_CORBALOC;

                std::cerr << "multicast address for federated clocks  "
                << TOPIC_UPDATE_MANAGER_CORBALOC_prop << std::endl;

                CDMW_INFO ( Cdmw::Logging::Logger::Get_Logger ( CLOCKSERVICE_LOGGER ),
                            "multicast address for federated clocks " );

                CDMW_INFO ( Cdmw::Logging::Logger::Get_Logger ( CLOCKSERVICE_LOGGER ),
                            TOPIC_UPDATE_MANAGER_CORBALOC_prop );

                FederatedControlledClockSvc::init ( pimpl_ref.m_orb.in(),
                                                    pimpl_ref.m_goa.in(),
                                                    TOPIC_UPDATE_MANAGER_CORBALOC_prop,
                                                    send_repetitions );

                CDMW_INFO ( Cdmw::Logging::Logger::Get_Logger ( CLOCKSERVICE_LOGGER ),
                            "init for federated clocks" );

                FederatedControlledClock_t* ref2federatedControlledClock =
                    new FederatedControlledClock_t ( federation_id );

                FederatedControlledExecutor_t* ref2federatedControlledExecutor =
                    new FederatedControlledExecutor_t ( federation_id + 1,
                                                        pimpl_ref.m_reactor.get() );

                federatedControlledClock =
                    ref2federatedControlledClock->_this();

                federatedControlledExecutor =
                    ref2federatedControlledExecutor->_this();

                ClockCatalog::ClockEntry federatedControlledClockEntry;

                ClockCatalog::ClockEntry federatedControlledExecutorEntry;

                federatedControlledClockEntry.subject =
                    ControlledClockEx::_duplicate ( federatedControlledClock.in() );

                federatedControlledExecutorEntry.subject =
                    ControlledExecutor::_duplicate ( federatedControlledExecutor.in() );

                federatedControlledClockEntry.name =
                    CORBA::string_dup ( federatedCtrlClockName.c_str() );

                federatedControlledExecutorEntry.name =
                    CORBA::string_dup ( federatedCtrlExecutorName.c_str() );

                ref2clockcatalog->_cxx_register ( federatedControlledClockEntry );

                ref2clockcatalog->_cxx_register ( federatedControlledExecutorEntry );

            }
            catch ( const CORBA::SystemException& e )
            {
                stringstream oss;
                oss << "WARNING: Raised Exception -- " << e;
                CDMW_WARN ( Cdmw::Logging::Logger::Get_Logger ( CLOCKSERVICE_LOGGER ),
                            "WARNING: Unable to init Controlled Clock Federation!" );
                CDMW_WARN ( Cdmw::Logging::Logger::Get_Logger ( CLOCKSERVICE_LOGGER ),
                            oss );
            }
        }

#endif
///////////////////////////////////////////////////////////////////////
// -- Add "ClockService" Executor Clocks to Clock Catalog --

        try
        {
            string executorName ( "LocalExecutor" );
            Executor_impl* ref2executor = new Executor_impl ( pimpl_ref.m_reactor.get() );

            PeriodicExecution::Executor_var
            executorClock = ref2executor->_this();

            ClockCatalog::ClockEntry executorClockEntry;
            executorClockEntry.subject =
                PeriodicExecution::Executor::_duplicate ( executorClock.in() );
            executorClockEntry.name = CORBA::string_dup ( executorName.c_str() );

            ref2clockcatalog->_cxx_register ( executorClockEntry );

        }
        catch ( const CORBA::SystemException& e )
        {
            CDMW_INFO ( Cdmw::Logging::Logger::Get_Logger ( CLOCKSERVICE_LOGGER ),
                        "WARNING: Exception during registering of Executor Clocks to Clock Catalog" );
            CDMW_INFO ( Cdmw::Logging::Logger::Get_Logger ( CLOCKSERVICE_LOGGER ), e._name() );
        }

///////////////////////////////////////////////////////////////////////
// -- Add ControlledExecutor Clocks to Clock Catalog --

        string controlledExecutorName ( "ControlledExecutor" );

        ControlledExecutor_impl<CDMW_Recursive_Thread_Mutex, CDMW_Guard>* ref2controlledexecutor =
            new ControlledExecutor_impl<CDMW_Recursive_Thread_Mutex, CDMW_Guard> ( pimpl_ref.m_reactor.get() );

        ControlledExecutor_var
        controlledExecutorClock = ref2controlledexecutor->_this();

        ClockCatalog::ClockEntry controlledExecutorClockEntry;

        controlledExecutorClockEntry.subject =
            ControlledExecutor::_duplicate ( controlledExecutorClock.in() );

        controlledExecutorClockEntry.name =
            CORBA::string_dup ( controlledExecutorName.c_str() );

        ref2clockcatalog->_cxx_register ( controlledExecutorClockEntry );

///////////////////////////////////////////////////////////////////////
// -- Add "MasterClock" (a system clock on an host where the NTP
// daemon is running) to Clock Catalog --
        if ( Cdmw::clock::util::check_ntp_syncronization() )
        {
            string masterClockName ( "MasterClock" );

            try
            {
                ref2clockcatalog->get_entry ( masterClockName.c_str() );

            }
            catch ( CosClockService::ClockCatalog::UnknownEntry& )
            {
                Clock_impl *ref2masterclock = new Clock_impl;
                CosClockService::Clock_var masterclock_var = ref2masterclock->_this();

                pimpl_ref.m_orb->register_initial_reference ( "MasterClock",
                        masterclock_var.in() );
                ClockCatalog::ClockEntry masterClockEntry;
                masterClockEntry.subject =
                    Clock::_duplicate ( masterclock_var.in() );
                masterClockEntry.name = CORBA::string_dup ( masterClockName.c_str() );

                ref2clockcatalog->_cxx_register ( masterClockEntry );
            }
        }

///////////////////////////////////////////////////////////////////////
// write clockcatalog ior
#ifndef NOT_LOCALISATION

        try
        {
            OrbSupport::OrbSupport::bind_object_to_corbaloc ( pimpl_ref.m_orb.in(),
                    CLOCK_CATALOG_NAME,
                    clockcatalog_var.in() );
            pimpl_ref.m_orb->register_initial_reference ( CLOCK_CATALOG_NAME.c_str(),
                    clockcatalog_var.in() );

        }
        catch ( const CORBA::BAD_PARAM& e )
        {
            ostringstream oss;
            oss << "ERROR: ORB Exception : Already bound " << e << " at line " << __LINE__;
            CDMW_ERROR ( Cdmw::Logging::Logger::Get_Logger ( CLOCKSERVICE_LOGGER ), oss.str() );
            exit ( -1 );


        }
        catch ( const CORBA::SystemException& e )
        {
            ostringstream oss;
            oss << "ERROR: ORB Exception : " << e << " at line " << __LINE__;
            CDMW_ERROR ( Cdmw::Logging::Logger::Get_Logger ( CLOCKSERVICE_LOGGER ), oss.str() );
            exit ( -1 );
        }

#else
        pimpl_ref.m_orb->register_initial_reference ( CLOCK_CATALOG_NAME.c_str(),
                clockcatalog_var.in() );

        // write clockcatalog ior
        write_obj_to_ior ( pimpl_ref.m_orb.in(),
                           clockcatalog_var,
                           clock_catalog_file_name );

#endif

///////////////////////////////////////////////////////////////////////
// -- Local Clock --
        Clock_impl *ref2clock = new Clock_impl;

        CosClockService::Clock_var clock_var = ref2clock->_this();

#ifndef NOT_LOCALISATION
        try
        {
            OrbSupport::OrbSupport::bind_object_to_corbaloc ( pimpl_ref.m_orb.in(),
                    LOCAL_CLOCK_NAME,
                    clock_var.in() );

            pimpl_ref.m_orb->register_initial_reference ( LOCAL_CLOCK_NAME.c_str(),
                    clock_var.in() );

        }
        catch ( const CORBA::BAD_PARAM& e )
        {
            ostringstream oss;
            oss << "ERROR: ORB Exception : Already bound " << e << " at line " << __LINE__;
            CDMW_ERROR ( Cdmw::Logging::Logger::Get_Logger ( CLOCKSERVICE_LOGGER ), oss.str() );
            exit ( -1 );

        }
        catch ( const CORBA::SystemException& e )
        {
            ostringstream oss;
            oss << "ERROR: ORB Exception : " << e << " at line " << __LINE__;
            CDMW_ERROR ( Cdmw::Logging::Logger::Get_Logger ( CLOCKSERVICE_LOGGER ), oss.str() );
            exit ( -1 );
        }

#else
        pimpl_ref.m_orb->register_initial_reference ( LOCAL_CLOCK_NAME.c_str(),
                clock_var.in() );

        // write localclock ior
        write_obj_to_ior ( pimpl_ref.m_orb.in(),
                           clock_var,
                           local_clock_file_name );

#endif

///////////////////////////////////////////////////////////////////////
// retrieve NamingAndRepository Service to register Object Reference
        CORBA::Object_var tmpObj = CORBA::Object::_nil();

        CosNaming::NamingContextExt_var rootCtx = CosNaming::NamingContextExt::_nil();

        bool isValidNAR = false;

        // retrieve Naming Service reference
        if ( nar_service_url != "no" )   // Try to retrieve Repository Service
        {
            try
            {
                tmpObj = orb->string_to_object ( nar_service_url.c_str() );
                CdmwNamingAndRepository::Repository_var repository
                = CdmwNamingAndRepository::Repository::_narrow ( tmpObj.in() );

                if ( CORBA::is_nil ( repository.in() ) )
                {
                    CDMW_ERROR ( Cdmw::Logging::Logger::Get_Logger ( CLOCKSERVICE_LOGGER ),
                                 "Unable to resolve Repository Service" );
                    isValidNAR = false;
                    //exit(-1);
                }

                CosNaming::NamingContext_var temp_rootCtx =

                    repository->resolve_root_context ( CdmwNamingAndRepository::DEFAULT_ROOT_CONTEXT );
                rootCtx = CosNaming::NamingContextExt::_narrow ( temp_rootCtx.in() );
                isValidNAR = true;

            }
            catch ( const CORBA::Exception& e )
            {
                ostringstream oss;
                oss << "ERROR: ORB Exception : " << e << " at line " << __LINE__;
                CDMW_ERROR ( Cdmw::Logging::Logger::Get_Logger ( CLOCKSERVICE_LOGGER ), oss.str() );
                isValidNAR = false;
                //exit(-1);
            }

        }
        else if ( isNamingObjectID )   // Try to retrieve Naming Service from "NameService" Initial Reference
        {
            try
            {
                tmpObj = orb->resolve_initial_references ( "NameService" );

                if ( CORBA::is_nil ( tmpObj.in() ) )
                {
                    CDMW_ERROR ( Cdmw::Logging::Logger::Get_Logger ( CLOCKSERVICE_LOGGER ),
                                 "Unable to resolve NameService initial reference" );
                    isValidNAR = false;
                    //exit(-1);
                }

                rootCtx
                = CosNaming::NamingContextExt::_narrow ( tmpObj.in() );

                isValidNAR = true;

            }
            catch ( const CORBA::Exception& e )
            {
                ostringstream oss;
                oss << "ERROR: ORB Exception : " << e << " at line " << __LINE__;
                CDMW_ERROR ( Cdmw::Logging::Logger::Get_Logger ( CLOCKSERVICE_LOGGER ), oss.str() );
                isValidNAR = false;
                //exit(-1);
            }
        }

        // Bind Object References into Naming Services

        if ( isValidNAR )
        {
            CosNaming::Name_var name = new CosNaming::Name;
            CosNaming::NameComponent_var nc = new CosNaming::NameComponent;
            CosNaming::Name_var name2 = new CosNaming::Name;
            CosNaming::NameComponent_var nc2 = new CosNaming::NameComponent;

            // LocalClock
            name->length ( 1 );
            nc->id = CORBA::string_dup ( LOCAL_CLOCK_NAME.c_str() );
            nc->kind = CORBA::string_dup ( "" );
            name->operator[] ( 0 ) = nc.in();
            // ClockService
            name2->length ( 1 );
            nc2->id = CORBA::string_dup ( CLOCK_CATALOG_NAME.c_str() );
            nc2->kind = CORBA::string_dup ( "" );
            name2->operator[] ( 0 ) = nc2.in();

            try
            {
                rootCtx->bind ( name.in(), clock_var.in() );
                rootCtx->bind ( name2.in(), clockcatalog_var.in() );

            }
            catch ( CosNaming::NamingContext::AlreadyBound &e )
            {
                ostringstream oss;
                oss << "EVoT Identifiers are already bounded into Nameservice" << " at line " << __LINE__;
                CDMW_INFO ( Cdmw::Logging::Logger::Get_Logger ( CLOCKSERVICE_LOGGER ), oss.str() );

            }
            catch ( const CORBA::Exception& e )
            {
                ostringstream oss;
                oss << "ERROR: ORB Exception : " << e << " at line " << __LINE__;
                CDMW_ERROR ( Cdmw::Logging::Logger::Get_Logger ( CLOCKSERVICE_LOGGER ), oss.str() );

            }
        }

///////////////////////////////////////////////////////////////////////
// Create PropertySetDef object and register.

        TAO_PropertySetDef* m_propset_servant = new TAO_PropertySetDef();

        CosPropertyService::PropertySetDef_var propset_var = m_propset_servant->_this();

        // Create PropertySet factory and then register.
        TAO_PropertySetFactory* m_propset_factory_servant = new TAO_PropertySetFactory();

        CosPropertyService::PropertySetFactory_var propset_factory = m_propset_factory_servant->_this();

///////////////////////////////////////////////////////////////////////
// TAO registration for valuetype differs from standard
        TAO_OBV_REGISTER_FACTORY ( UTC_impl_init, UTC );

        if ( retval != 0 )
        {
            std::cerr << "ClockService:>> Cannot activate ORB Thread Pool\n" << std::endl;
            CDMW_ERROR ( Cdmw::Logging::Logger::Get_Logger ( CLOCKSERVICE_LOGGER ), "ERROR: Cannot activate ORB Thread Pool!" );
            return false;
        }

    }
    catch ( const CORBA::SystemException& e )
    {
        ostringstream oss;
        oss << "ERROR: ORB Exception : " << Cdmw::OrbSupport::OrbSupport::exception_to_string(e)  << " at line " << __LINE__;
        CDMW_ERROR ( Cdmw::Logging::Logger::Get_Logger ( CLOCKSERVICE_LOGGER ), oss.str() );
        CDMW_ERROR ( Cdmw::Logging::Logger::Get_Logger ( CLOCKSERVICE_LOGGER ), "Error during clock initialisation" );
        CDMW_ERROR ( Cdmw::Logging::Logger::Get_Logger ( CLOCKSERVICE_LOGGER ), "WARNING : If there is another instance of clock service on this machine" );
        CDMW_ERROR ( Cdmw::Logging::Logger::Get_Logger ( CLOCKSERVICE_LOGGER ), "use the option --CdmwClockServicePort=<value>" );
        std::cerr << Cdmw::OrbSupport::OrbSupport::exception_to_string(e)  << std::endl; 
        exit ( -1 );
    }

    return true;
}

///////////////////////////////////////////////////////////////////////////////

void
CosClock::close()
{

    if ( close_performed )
        return;

    if ( !s_PIMPL.get() )
        return;

    // just a ref
    CosClock_impl &pimpl_ref = * ( s_PIMPL.get() );

    pimpl_ref.m_reactor->end_reactor_event_loop();

    pimpl_ref.m_reactor_worker->wait();

    // release resources
    pimpl_ref.m_reactor->close();

    pimpl_ref.m_reactor.reset();

    pimpl_ref.m_tp_reactor.reset();

    pimpl_ref.m_reactor_worker.reset();

    close_performed = true;
}

///////////////////////////////////////////////////////////////////////////////

PortableGroup::GOA_ptr
CosClock::get_GOA()
{
    if ( s_PIMPL.get() != 0 )
    {
        PortableGroup::GOA_var returned_GOA = PortableGroup::GOA::_duplicate ( s_PIMPL->m_goa.in() );
        return returned_GOA._retn();
    }

    return 0;
}


///////////////////////////////////////////////////////////////////////////////
// functions used by JNI Clock //
///////////////////////////////////////////////////////////////////////////////


///////////////////////////////////////////////////////////////////////////////
void CosClock::initCPPClockService ( int argc, char **argv )
{
    std::vector< std::string > IOR_vector;
    parse_args ( argc, argv );


    // initialises the ORB.
    CORBA::ORB_var orb;
    const int POA_THREAD_POOL_SIZE=10;

    OrbSupport::StrategyList strategyList;
    strategyList.add_OrbThreaded();
    strategyList.add_PoaThreadPool(POA_THREAD_POOL_SIZE);
    //strategyList.add_PoaThreadPerConnection(); // CCR126
    strategyList.add_multicast();

    char** ORB_init_argv = 0;
    int ORB_init_argc = argc;
    std::string str_opt ( "" );

    for ( int i = 0; i < argc; i++ )
    {
        str_opt += std::string ( argv[i] ) + " ";
    //std::cerr << "initCPPClockSErvice : argv = " << argv[i] << std::endl;
    }

    str_opt += std::string ( "--CdmwLocalisationService=" + JNI_CLOCK_SERVICE_PORT );

    ORB_init_argc += 1;
    ORB_init_argv = Cdmw::Common::String::to_char_array (
                        Cdmw::Common::String::to_strings ( str_opt ) );

    orb = Cdmw::OrbSupport::OrbSupport::ORB_init ( ORB_init_argc, ORB_init_argv, strategyList );

    jni_orb = orb;

    // initializion for CosClockServices.

    if ( CosClock::init ( orb.in(), ORB_init_argc, ORB_init_argv ) == false )
    {
    //std::cerr << ":>> can not init clock service!" << std::endl;
        CDMW_ERROR ( Cdmw::Logging::Logger::Get_Logger ( CLOCKSERVICE_LOGGER ), "ERROR:>> can not init clock service!" );
    }

    PortableServer::POAManager_var poa_manager;

    // Get the root POA
    CORBA::Object_var robj = orb->resolve_initial_references ( "RootPOA" );
    PortableServer::POA_var rootPOA = PortableServer::POA::_narrow ( robj.in() );
    poa_manager = rootPOA->the_POAManager();
    poa_manager->activate();

    // Start the CppClockThread
    CppClockThread* clockThread = new CppClockThread ( orb.in() );
    clockThread->start();

    // wait for ORB run
    Cdmw::OsSupport::OS::sleep ( 2, 0 );
    // free the memory allocated by String::to_char_array

    for ( int iArgc = 0; iArgc < ORB_init_argc; ++iArgc )
    {
        delete[] ORB_init_argv[iArgc];
        ORB_init_argv[iArgc] = 0;
    }

    delete[] ORB_init_argv;
    ORB_init_argv = 0;
}

///////////////////////////////////////////////////////////////////////////////

std::vector< std::string > CosClock::setIORsClockService()
{
    std::vector< std::string > IOR_vector;

    CosClockService::ClockCatalog_var clockCatalog = NULL;
    // set the IORs for ClockObjects
    CORBA::Object_var obj = NULL;
    std::string IORClockObj;

// LOCAL_CLOCK_ID 0

    try
    {
        obj = jni_orb->resolve_initial_references ( "LocalClock" );
        CosClockService::Clock_var localClock = CosClockService::Clock::_narrow ( obj );
        IORClockObj = jni_orb->object_to_string ( localClock );
        IOR_vector.push_back ( IORClockObj );

    }
    catch ( CORBA::ORB::InvalidName in )
    {
        CDMW_WARN ( Cdmw::Logging::Logger::Get_Logger ( CLOCKSERVICE_LOGGER ), "WARNING : There is not reference for LocalClock" );
        IOR_vector.push_back ( "NOT PRESENT" );
    }

// CLOCK_CATALOG_ID 1

    try
    {
        obj = jni_orb->resolve_initial_references ( "ClockService" );
        clockCatalog = CosClockService::ClockCatalog::_narrow ( obj );
        IORClockObj = jni_orb->object_to_string ( clockCatalog );
        IOR_vector.push_back ( IORClockObj );

    }
    catch ( CORBA::ORB::InvalidName in )
    {
        CDMW_WARN ( Cdmw::Logging::Logger::Get_Logger ( CLOCKSERVICE_LOGGER ), "WARNING : There is not reference for ClockService" );
        IOR_vector.push_back ( "NOT PRESENT" );
    }

// MASTER_CLOCK_ID 2

    try
    {
        obj = jni_orb->resolve_initial_references ( "MasterClock" );
        CosClockService::Clock_var masterClock = CosClockService::Clock::_narrow ( obj );
        IORClockObj = jni_orb->object_to_string ( masterClock );
        IOR_vector.push_back ( IORClockObj );

    }
    catch ( CORBA::ORB::InvalidName in )
    {
        CDMW_WARN ( Cdmw::Logging::Logger::Get_Logger ( CLOCKSERVICE_LOGGER ), "WARNING : There is not reference for MasterClock" );
        IOR_vector.push_back ( "NOT PRESENT" );
    }

// POLICY_CONTROLLER_UPDATE_ADMIN_ID 3

    try
    {
        obj = jni_orb->resolve_initial_references ( "PolicyControllerUpdateAdmin" );
        Cdmw::clock::policy::ControllerUpdateAdmin_var policyAdmin = Cdmw::clock::policy::ControllerUpdateAdmin::_narrow ( obj );
        IORClockObj = jni_orb->object_to_string ( policyAdmin );
        IOR_vector.push_back ( IORClockObj );

    }
    catch ( CORBA::ORB::InvalidName in )
    {
        CDMW_WARN ( Cdmw::Logging::Logger::Get_Logger ( CLOCKSERVICE_LOGGER ), "WARNING : There is not reference for PolicyControllerUpdateAdmin" );
        IOR_vector.push_back ( "NOT PRESENT" );
    }


//EXECUTOR_ID 4

    if ( clockCatalog != NULL )
    {
        try
        {
            ClockCatalog::ClockEntry_var
            entry = clockCatalog->get_entry ( "LocalExecutor" );
            CosClockService::Clock_var m_clock = entry->subject;

            CosClockService::PeriodicExecution::Executor_var executorX;
            executorX = CosClockService::PeriodicExecution::Executor::_narrow ( m_clock.in() );

            IORClockObj = jni_orb->object_to_string ( executorX );
            IOR_vector.push_back ( IORClockObj );

        }
        catch ( CORBA::ORB::InvalidName in )
        {
            CDMW_WARN ( Cdmw::Logging::Logger::Get_Logger ( CLOCKSERVICE_LOGGER ), "WARNING : There is not reference for LocalExecutor" );
            IOR_vector.push_back ( "NOT PRESENT" );
        }


//FEDERATEDCONTROLLEDCLOCK_ID 5

        try
        {
            ClockCatalog::ClockEntry_var
            entry = clockCatalog->get_entry ( "FederatedControlledClock" );
            ControlledClockEx_var fedClock = ControlledClockEx::_narrow ( ( entry->subject ).in() );

            IORClockObj = jni_orb->object_to_string ( fedClock );
            IOR_vector.push_back ( IORClockObj );

        }
        catch ( ::CosClockService::ClockCatalog::UnknownEntry ue )
        {
            CDMW_WARN ( Cdmw::Logging::Logger::Get_Logger ( CLOCKSERVICE_LOGGER ), "WARNING : There is not reference for FederatedControlledClock" );
            IOR_vector.push_back ( "NOT PRESENT" );
        }

//FEDERATEDCONTROLLEDEXECUTOR_ID 6

        try
        {
            ClockCatalog::ClockEntry_var
            entry = clockCatalog->get_entry ( "FederatedControlledExecutor" );
            ControlledExecutor_var fedexecutor = ControlledExecutor::_narrow ( ( entry->subject ).in() );
            IORClockObj = jni_orb->object_to_string ( fedexecutor );
            IOR_vector.push_back ( IORClockObj );

        }
        catch ( ::CosClockService::ClockCatalog::UnknownEntry ue )
        {
            CDMW_WARN ( Cdmw::Logging::Logger::Get_Logger ( CLOCKSERVICE_LOGGER ), "WARNING : There is not reference for FederatedControlledExecutor" );
            IOR_vector.push_back ( "NOT PRESENT" );
        }
    }

//    Cdmw::OsSupport::OS::sleep ( 5, 0 );

    // TO DO : implement CosClock::close() for JNI call
    // CosClock::close();
    return IOR_vector;
}

///////////////////////////////////////////////////////////////////////////////
// End Of File
///////////////////////////////////////////////////////////////////////////////
