/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI.
 * Copyright (C) SELEX SISTEMI INTEGRATI 2004-2009. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

#include "clockservice/ControllerEnforceDeadlinePolicy.hpp"
#include "clockservice/ControllerEx_impl.hpp"
#include "Time/clockservice/Util.hpp"
#include <Foundation/orbsupport/ExceptionMinorCodes.hpp>
#include <Foundation/common/Assert.hpp>

using namespace Cdmw::clock::util;

////////////////////////////////////////////////////////////////////////////////

namespace 
{
}

////////////////////////////////////////////////////////////////////////////////

Cdmw::clock::ControllerEnforceDeadlinePolicy::ControllerEnforceDeadlinePolicy ()
{
}

////////////////////////////////////////////////////////////////////////////////

Cdmw::clock::ControllerEnforceDeadlinePolicy::~ControllerEnforceDeadlinePolicy ()
{
}

////////////////////////////////////////////////////////////////////////////////

void 
Cdmw::clock::ControllerEnforceDeadlinePolicy::schedule(::Cdmw::clock::ControllerEx_ptr controller_ptr)
{
    ControllerEx_impl* controller = 
             dynamic_cast<ControllerEx_impl*>(controller_ptr);
    CDMW_ASSERT(controller);
    if(!controller)
    {
        // Add logger info
        return;
    }

    /* *
     * check controlled status: 
     * if is PAUSED then NOTHING TO DO
     * re-scheduling of timers will be done by resume of
     * controlled clock
     * */
    
    if (controller->get_status() == Cdmw::clock::STARTED)  
    {
        if(controller->executions() ==
           controller->execution_limit() )
            return; // timer has completed the executions

    controller->pause();  

        TimeBase::TimeT resume_at_time = controller->get_next_expiration_time();

        try 
        {
            controller->resume_at(resume_at_time);     
        } 
        catch ( CosClockService::PeriodicExecution::Controller::TimePast& e) 
        {
            std::cerr << e << std::endl;    
            throw;
        } catch ( CORBA::SystemException& ex)
        {
            std::cerr << ex << std::endl;   
        }
    }
}

////////////////////////////////////////////////////////////////////////////////

/**
 * @fn void Cdmw::clock::ControllerEnforceDeadlinePolicy::on_set (::Cdmw::clock::ControllerEx_ptr controller)
 * 
 * \f$(x_0,y_0)\f$: is the origin of controlled clock.
 * \f$(x_s,y_s)\f$: when a set operation was performed.
 * \f$(x_n,y_n)\f$: net time base.
 * In this case \f$ x_n = x_s \f$ are the same values.
 * \f$(x_c,y_c)\f$: start of controlled clock.
 * \f$(x_a,y_a)\f$: deadline, before set.
 * \f$(x_{a1},y_{a1})\f$: new deadline, before set.
 * In this case \f$ y_a = y_{a1} \f$ are the same values.
 * 
 * Time equation is:
 * \f$ y_s - y_0 = \alpha(x_s-x_0) \f$.
 * then we can write the system:
 * 
 * \f{eqnarray*}
 * \cases{ 
 * y_a-y_s = \alpha(x_a - x_s) \cr
 * y_{a1}-y_n = \alpha(x_{a1} - x_n) \cr
 * }
 * \f}
 * 
 * But,
 * \f{eqnarray*}
 * \cases{ 
 * y_{a1} = y_a \cr
 * x_n = x_s) \cr
 * }
 * \f}
 * 
 * Then we can write:
 * 
 * \f{eqnarray*}
 * \cases{ 
 * y_a-y_s = \alpha(x_a - x_s) \cr
 * y_a-y_n = \alpha(x_{a1} - x_n) \cr
 * }
 * \f}
 * 
 * and solving:
 * 
 * \f$ x_{a1} = x_a - \frac{y_n-y_s}{\alpha} \f$.
 * where  \f$ x_{a1} \f$ is the new time of deadline in realtime.
 */
void 
Cdmw::clock::ControllerEnforceDeadlinePolicy::on_set (
    ::Cdmw::clock::ControllerEx_ptr controller)
  throw (CORBA::SystemException)
{
     if(!this->isValid(controller))
        return;
    this->schedule(controller);
}

////////////////////////////////////////////////////////////////////////////////

void 
Cdmw::clock::ControllerEnforceDeadlinePolicy::on_set_rate (
    ::Cdmw::clock::ControllerEx_ptr controller)
  throw (CORBA::SystemException)
{
     if(!this->isValid(controller))
        return;
    this->schedule(controller);
}

////////////////////////////////////////////////////////////////////////////////

void 
Cdmw::clock::ControllerEnforceDeadlinePolicy::on_pause (
    ::Cdmw::clock::ControllerEx_ptr controller)
  throw (CORBA::SystemException)
{
     if(!this->isValid(controller))
        return;
    controller->pause();
}

////////////////////////////////////////////////////////////////////////////////

void 
Cdmw::clock::ControllerEnforceDeadlinePolicy::on_terminate (
    ::Cdmw::clock::ControllerEx_ptr controller)
  throw (CORBA::SystemException)
{
     if(!this->isValid(controller))
        return;
    controller->terminate();
}

////////////////////////////////////////////////////////////////////////////////

void 
Cdmw::clock::ControllerEnforceDeadlinePolicy::on_resume(
    ::Cdmw::clock::ControllerEx_ptr controller)
  throw (CORBA::SystemException)
{
     if(!this->isValid(controller))
        return;
    controller->resume();    
}

////////////////////////////////////////////////////////////////////////////////
// End Of File
////////////////////////////////////////////////////////////////////////////////
