/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI.
 * Copyright (C) SELEX SISTEMI INTEGRATI 2004-2009. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


package cdmw.clock;

import cdmw.ossupport.OS;
import cdmw.ossupport.ThreadedProcess;
import org.omg.CORBA.ORB;
import org.omg.CosClockService.ClockCatalog;
import org.omg.CosClockService.ClockCatalogHelper;
import org.omg.CosClockService.TimeSpanValueFactory;
import org.omg.CosClockService.UTCValueFactory;

/**
 * @brief implementation for ClockService Java Wrapper
 *
 * @author Francesca Annunziata <fannunziata@selex-si.com>
 * @author Raffaele Mele <rmele@progesi.it>
 */

public class ClockServiceJavaWrapper {
    private static final String LOCALISATION_SERVICE = " --CdmwLocalisationService=";
    private static final String LOCALCLOCK_CORBALOC = " -ORBInitRef LocalClock=corbaloc::localhost:";
    private static final String LOC_SERV_PORT = "4883";
    private static final String LOCALCLOCK = "LocalClock";
    private static final String CLOCKSERVICE_CORBALOC = " -ORBInitRef ClockService=corbaloc::localhost:";
    private static final String CLOCKSERVICE = "ClockService";
    public static final int MAX_START_RETRY = 10;

    private static cdmw.ossupport.ThreadedProcess proc = null;

    public static void startClockService(ORB orb, String[] args)
    {

        try
        {
            ((org.omg.CORBA_2_3.ORB)orb).register_value_factory("IDL:omg.org/CosClockService/UTC:1.0",
                    new UTCValueFactoryImpl());
            ((org.omg.CORBA_2_3.ORB)orb).register_value_factory("IDL:omg.org/CosClockService/TimeSpan:1.0",
                    new TimeSpanValueFactoryImpl());

            String clockServiceArgs = arrayToString(args, " ");
            clockServiceArgs = clockServiceArgs + "--server";
            proc = OS.createProcess("cdmw_clockservice_standalone " + clockServiceArgs);
            boolean isStarted = false;
            int startRetry = 0;
            while (!isStarted && (startRetry < MAX_START_RETRY))
            {
                try
                {
                    startRetry++;
                    org.omg.CORBA.Object obj  = orb.resolve_initial_references(CLOCKSERVICE);
                    ClockCatalog catalog = ClockCatalogHelper.narrow(obj);
                    if (catalog != null)
                        isStarted = true;
                    else
                         OS.sleep(500); // wait before check again
                }
                catch (org.omg.CORBA.TRANSIENT ex)
                {
                    // ClockService wasn't started yet!
                }
                catch (org.omg.CORBA.ORBPackage.InvalidName ex)
                {
                    // ClockService wasn't started yet!
                    ex.printStackTrace();
                }
                catch (java.lang.InterruptedException ex)
    {
                    ex.printStackTrace();
                }

            }
            if (!isStarted)
                System.err.println("ClockService Wrapper:>> Panic -- extern process don't start.");
        }
        catch (java.io.IOException ex)
        {
            ex.printStackTrace();
        }
    }
    
    public static void startClockService(ORB orb)
    {
        try
        {
            ((org.omg.CORBA_2_3.ORB)orb).register_value_factory("IDL:omg.org/CosClockService/UTC:1.0",
                    new UTCValueFactoryImpl());
            ((org.omg.CORBA_2_3.ORB)orb).register_value_factory("IDL:omg.org/CosClockService/TimeSpan:1.0",
                    new TimeSpanValueFactoryImpl());

            String clockServiceArgs = "--server"            +
                                      LOCALISATION_SERVICE  +
                                      LOC_SERV_PORT         +
                                      LOCALCLOCK_CORBALOC   +
                                      LOC_SERV_PORT + "/"   +
                                      LOCALCLOCK            +
                                      CLOCKSERVICE_CORBALOC +
                                      LOC_SERV_PORT + "/"   +
                                      CLOCKSERVICE;
            proc = OS.createProcess("cdmw_clockservice_standalone " + clockServiceArgs);
            boolean isStarted = false;
            int startRetry = 0;
            while (!isStarted && (startRetry < MAX_START_RETRY))
            {
                try
                {
                    startRetry++;
                    org.omg.CORBA.Object obj  = orb.resolve_initial_references(CLOCKSERVICE);
                    ClockCatalog catalog = ClockCatalogHelper.narrow(obj);
                    if (catalog != null)
                        isStarted = true;
                  else
                        OS.sleep(500); // wait 500ms
                }
                catch (org.omg.CORBA.TRANSIENT ex)
                {
                    // ClockService isn't up!!
                }
                catch (org.omg.CORBA.ORBPackage.InvalidName ex)
                {
                    ex.printStackTrace();
                }
                catch (java.lang.InterruptedException ex)
                {
                    ex.printStackTrace();
                }
            }
            if (!isStarted)
                System.err.println("ClockService Wrapper:>> Panic -- extern process don't start.");
        }
        catch (java.io.IOException ex)
        {
            ex.printStackTrace();
        }
    }
    public static void stopClockService ()
    {
        if (proc != null) proc.killProcess();
    }


    /**
     * Convert an array of String to a plain String,
     *
     * @param strArr strings array
     * @param separator  plain string separator
     *
     */
    private static String arrayToString(String[] strArr, String separator) 
    {
        StringBuffer result = new StringBuffer();
        if (strArr.length > 0) {
            result.append(strArr[0]);
            for (int i=1; i<strArr.length; i++) {
                result.append(separator);
                result.append(strArr[i]);
            }
        }
        return result.toString();
    }
}
