/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI.
 * Copyright (C) SELEX SISTEMI INTEGRATI 2004-2009. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

package cdmw.clock;
import cdmw.ossupport.Timeval;
import java.util.Calendar;

public class Time
{
   
    /**
     * The number of 100 ns units per microseconds
     */
    private static final long DECIMAL_PRECISION = 10L;
    
    /**
     * The number of days between DCE time base
     * (15 October 1582 00:00:00) and POSIX time base
     * (1 January 1970 00:00:00). 141427 days.
     */
    private static final long DAYS_DCE_TO_POSIX = 141427L;

    /**
     * DAYS_DCE_TO_POSIX days, USEC_IN_A_DAY microseconds per day,
     * DECIMAL_PRECISION units of 100 ns per second.
     */
    private static final long TIME_BASE =
    DAYS_DCE_TO_POSIX * Macro.USEC_IN_A_DAY * DECIMAL_PRECISION;

    /**
     * @brief function TimeT2timeval converts a TimeT to a struct timeval
     * @param input_time is the number of 10^(-7) seconds from the julian day
     *        Fri 15 Oct 1582 (at 00:00:00,000,000), Coordinated Universal Time (UTC).
     * @return is a timeval struct (used inside Linux, file sys/time.h).
     */
    public static cdmw.ossupport.Timeval TimeT2timeval(long input_time)
    {
        int time_s  = 0;
        int time_us = 0;
        if ( input_time > TIME_BASE )
        {
            long time_tmp = (long)( input_time - TIME_BASE );
            time_s  = (int)(time_tmp / Macro.TIMET_IN_A_SEC); // number of seconds         
            time_us = (int) ( (time_tmp % Macro.TIMET_IN_A_SEC) / Macro.TIMET_IN_A_USEC) ; // number of microseconds
        return new Timeval(time_s, time_us);
    }
        else{
            Calendar calendar = Calendar.getInstance ();
            calendar.set( 1970, // 1970
                             0, // January
                             1, // 1st
                             0, // 0 h
                             0, // 0 m
                             0);// 0 s
            calendar.set(Calendar.MILLISECOND, 0);// 0 ms
            return new Timeval(calendar.getTimeInMillis());
        } 
    }

    /**
     * @brief function timeval2TimeT converts a timeval input struct in a TimeT
     * @param tv is the timeval input struct containing all detailed time informations
     * @return is the associated output TimeT expressed as the number of 10^(-7) seconds
     *        from the julian day Fri 15 Oct 1582 (at 00:00:00,000,000), Coordinated Universal Time (UTC).
     */

    public static long timeval2TimeT(cdmw.ossupport.Timeval  time)
    {
        return (time.getMicroseconds() + (time.getMilliseconds() * 1000)) * DECIMAL_PRECISION + TIME_BASE;
    }

    public static org.omg.TimeBase.UtcT createUtcT(long utcTime)
    {
        java.util.Calendar cal = java.util.Calendar.getInstance();
        int offset =
            cal.get( java.util.Calendar.ZONE_OFFSET );  // zone difference
           
    
        long inaccuracy = 10000L;
        short tdf = ( short ) ( offset / 60000 );       //  this placement from GMT(OMG spec)
                                                        //  millis to minutes
  
        long inacchi = inaccuracy >> 32;
        return new org.omg.TimeBase.UtcT(
                       utcTime,
                       ( int ) ( inaccuracy & 0xffffffffL ),
                       ( short ) ( inacchi & 0xffffL ),
                       tdf );
    }

    public static String toString(org.omg.TimeBase.UtcT utc_time)
    {
        if ( utc_time.time < TIME_BASE )
        {
            return "Time before 1 January 1970 00:00:00";
        }
        // Gets the default TimeZone for this host
        java.util.TimeZone time_zone = java.util.TimeZone.getDefault();
        
        time_zone.setRawOffset( utc_time.tdf * 60000 );  // minutes to millis
        
        java.text.SimpleDateFormat date = new java.text.SimpleDateFormat(  "yyyy-MM-dd HH:mm:ss.SSZ"  );
        date.setTimeZone(time_zone);
        return date.format( new java.util.Date(  TimeT2timeval(utc_time.time).getMilliseconds() ) );
    }

}
