/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI.
 * Copyright (C) SELEX SISTEMI INTEGRATI 2004-2009. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

package cdmw.clock;

import org.omg.CosClockService.ControlledClock;
import Cdmw.clock.*;


// CdmwTimeval
import cdmw.ossupport.Timeval;

//Calendar
import java.util.Calendar;

public class Util
{
    /**
     * real2controlled : allow to exress the time of System Clock (real) in time
     * basis of the virtual clock (controlled).
     *
     *  Assuming that time equation is:    y - y0 = rate * ( x - x0 )
     *  where :
     *   (x0, y0)  : virtual time coordinate
     *   (x , y )  : real time coordinate
     *   rate      : rate of virtual clock
     *  the method returns the value of virtual time corresponding to
     *  real clock time value passed by input
     *  based on following formula : y = rate * (x - x0) + y0
     *
     * @param  virtualclock : controlled clock
     * @param  x            : clock time value to convert
     * @return y
     *
     **/

    public static org.omg.CORBA.LongHolder  real2controlled (ControlledClock virtualclock, long x)
    {

        // assuming that time equation is:
        //
        //    y - y0 = rate * ( x - x0 )
        //
        long  out = 0;

        ControlledClockEx virtualclockEx = ControlledClockExHelper.narrow(virtualclock);

        org.omg.CORBA.LongHolder  x0 = new org.omg.CORBA.LongHolder();
        org.omg.CORBA.LongHolder  y0 = new org.omg.CORBA.LongHolder();

        try
        {
           virtualclockEx.get_time_origins (x0, y0);

        if (x0.value > x)
           throw new org.omg.CosClockService.TimeUnavailable();


        double  rate = virtualclockEx.get_rate();
            //abs_rate = fabs(rate);

        long delta = x - x0.value;


        if(rate>=0.0)
            out =  (long)(rate * delta) + y0.value;
        else
        {
            out =  (long) (rate * delta);
            if( out > y0.value )
                throw new org.omg.CosClockService.TimeUnavailable();
            out = y0.value - out;
        }
        }
        catch (org.omg.CosClockService.ControlledClockPackage.NotSupported e)
        {
        }
        catch (org.omg.CosClockService.TimeUnavailable tu)
        {
        }
        return new org.omg.CORBA.LongHolder(out);

    }


    /**
     * controlled2real : allow to express the time of virtual Clock (controlled) in time
     * basis of the system clock (real).
     *
     *  Assuming that time equation is:    y - y0 = rate * ( x - x0 )
     *  where :
     *   (x0, y0)  : virtual time coordinate
     *   (x , y )  : real time coordinate
     *   rate      : rate of virtual clock
     *  the method returns the value of real time corresponding to
     *  virtual clock time value passed by input
     *  based on following formula : x = (y - y0)/(rate + x0)
     *
     * @param  virtualclock : controlled clock
     * @param  y            : virtual clock time value to convert
     * @return x
     *
     **/

    public static long controlled2real(ControlledClock virtualclock, long y)
    {
        // assuming that time equation is:
        //
        //    y - y0 = rate * ( x - x0 )
        //

        long out = 0;
        try
        {
        Cdmw.clock.ControlledClockEx
            controlled = ControlledClockExHelper.narrow(virtualclock);


        org.omg.CORBA.LongHolder  x0 = new org.omg.CORBA.LongHolder();
        org.omg.CORBA.LongHolder  y0 = new org.omg.CORBA.LongHolder();


        controlled.get_time_origins (x0, y0);

        double rate = controlled.get_rate();

        if (rate == 0.0)
              throw new org.omg.CosClockService.TimeUnavailable();



        long delta = 0;

        if (rate >= 0.0)
        {
          if ( y < y0.value )
            throw new org.omg.CosClockService.TimeUnavailable();
          else
            delta = y - y0.value;
        }
        else
        {
          if ( y > y0.value )
            throw new org.omg.CosClockService.TimeUnavailable();
          else
            delta = y0.value - y;
        }
         out = (long) (delta / rate) + x0.value;
        }
            catch (org.omg.CosClockService.ControlledClockPackage.NotSupported e)
            {
            }
            catch (org.omg.CosClockService.TimeUnavailable tu)
            {
            }

        return out;
    }

    /**
     * @brief function cdmwTime2timeval pack time value from year,month,day,hour,minute,second,millisecond
     * @param input_time is input date in cdmwTime format (year,month,day,hour,minute,second,millisecond)
     * @return return value is a Timeval object (millisecond from the Epoch)
     */
    public static cdmw.ossupport.Timeval cdmwTime2timeval(cdmwTime input_time)
    {
        Calendar calendar = Calendar.getInstance ();
        calendar.set( input_time.tm_year+1900,
                      input_time.tm_mon,
                      input_time.tm_mday,
                      input_time.tm_hour,
                      input_time.tm_min,
                      input_time.tm_sec);
        calendar.set(Calendar.MILLISECOND, input_time.tm_msec);
        return new Timeval(calendar.getTimeInMillis());

        //throw new org.omg.CORBA.NO_IMPLEMENT();
    }

    /**
     * @brief function timeval2cdmwTime unpack time value to year,month,day,hour,minute,second,millisecond
     * @param input_time is a Timeval Object (millisecond from the Epoch)
     * @return return value is a cdmwTime object (year,month,day,hour,minute,second,millisecond)
     */

    public static cdmwTime timeval2cdmwTime(cdmw.ossupport.Timeval input_time)
    {

        Calendar output_date;
        output_date = Calendar.getInstance();
        output_date.setTimeInMillis(input_time.getMilliseconds());

        cdmwTime cdmwTimeObj = new cdmwTime();
        cdmwTimeObj.tm_year  =  output_date.get(Calendar.YEAR)-1900;
        cdmwTimeObj.tm_mon   =  output_date.get(Calendar.MONTH);
        cdmwTimeObj.tm_mday  =  output_date.get(Calendar.DAY_OF_MONTH);
        cdmwTimeObj.tm_hour  =  output_date.get(Calendar.HOUR_OF_DAY) ;
        cdmwTimeObj.tm_min   =  output_date.get(Calendar.MINUTE);
        cdmwTimeObj.tm_sec   =  output_date.get(Calendar.SECOND);
        cdmwTimeObj.tm_wday  =  output_date.get(Calendar.DAY_OF_WEEK);
        cdmwTimeObj.tm_yday  =  output_date.get(Calendar.DAY_OF_YEAR);
        cdmwTimeObj.tm_isdst =  output_date.get(Calendar.DST_OFFSET);
        cdmwTimeObj.tm_msec  =  output_date.get(Calendar.MILLISECOND);
        cdmwTimeObj.tm_usec  =  input_time.getMicroseconds();
        return cdmwTimeObj;

        //throw new org.omg.CORBA.NO_IMPLEMENT();
    }
    /**
     * @brief function add : adds s two time value
     * @param : tv1 and tv2
     * @return new timeval corresponding to the sum of tv1 with tv2
     */

    public static cdmw.ossupport.Timeval add(cdmw.ossupport.Timeval tv1, cdmw.ossupport.Timeval tv2)
    {
       return new Timeval(tv1.getMilliseconds()+tv2.getMilliseconds())  ;
    }
    /**
     * @brief function subtract : subtracts two time value
     * @param : tv1 and tv2
     * @return new timeval corresponding to the subtraction of tv1 with tv2
     */
    public static cdmw.ossupport.Timeval subtract(cdmw.ossupport.Timeval tv1, cdmw.ossupport.Timeval tv2)
    {
        if (compare(tv1,tv2) >0) // tv1>tv2
           return new Timeval(tv1.getMilliseconds()-tv2.getMilliseconds())  ;
        else
           return new Timeval(tv2.getMilliseconds()-tv1.getMilliseconds())  ;
    }
    /**
     * @brief function  : it compares two time value
     * @param : tv1 and tv2
     * @return the value 0 if the tv1 is equal to tv2;
     *         a value less than 0    (<0)  if tv1 is smaller than tv2 (tv1<tv2);
     *         a value greater than 0 (>0)  if tv1 is grater than tv2  (tv1>tv2);
     */
    public static int compare(cdmw.ossupport.Timeval tv1, cdmw.ossupport.Timeval tv2)
    {
       // java.util.Date compareTo returns the value 0 if the argument Date (tv2) is equal to this Date (tv1);
       // a value less than 0 if this Date is before the Date argument;       (<0 if tv1<tv2)
       // and a value greater than 0 if this Date is after the Date argument. (>0 if tv1>tv2)
        return tv1.toDate().compareTo(tv2.toDate());
    }

    /**
     * @brief function TimeT2cdmwTime: it convert TimeT value to cdmwTime class
     * @param : value
     * @return cdmwTime class representing the TimeT value
     */
    public static cdmwTime TimeT2cdmwTime (long value)
    {
        return timeval2cdmwTime(Time.TimeT2timeval(value));
    }

    /**
     * @brief function cdmwTime2TimeT: it convert cdmwTime class to TimeT value
     * @param : cdmw_tm
     * @return TimeT value representing the cdmwTime class
     * @throws org.omg.CosClockService.TimeUnavailable, when the conversion is impossible
     */
    public static long cdmwTime2TimeT(cdmwTime cdmw_tm) throws org.omg.CosClockService.TimeUnavailable
    {
        if (!check_cdmwTime(cdmw_tm))
            throw new org.omg.CosClockService.TimeUnavailable("cdmwTime contains wrong value(s)");
        return Time.timeval2TimeT(cdmwTime2timeval(cdmw_tm));

    }

    /**
     * @brief function check_cdmwTime: it check cdmwTime class values
     * @param : to_check, cdmwTime class to check
     * @return cdmwTime true if values are all ok, false instead.
     */
    public static boolean check_cdmwTime(cdmwTime to_check)
    {
        boolean isOk;

        isOk =
            cdmwTimeUtils.check_range(to_check.tm_sec, 0, 61)   && // It can be up to 61 to allow for leap seconds.
            cdmwTimeUtils.check_range(to_check.tm_min, 0, 59)   &&
            cdmwTimeUtils.check_range(to_check.tm_hour, 0, 23)  &&
            cdmwTimeUtils.check_range(to_check.tm_mday, 1, 31)  &&
            cdmwTimeUtils.check_range(to_check.tm_mon, 0, 11)   &&
            (to_check.tm_year >= -318)            && // 1900-318 = 1582
            cdmwTimeUtils.check_range(to_check.tm_wday, 0, 6)   &&
            cdmwTimeUtils.check_range(to_check.tm_yday, 0, 365) &&
            cdmwTimeUtils.is_positive(to_check.tm_msec )        &&
            cdmwTimeUtils.is_positive(to_check.tm_usec );

        if ( isOk && (to_check.tm_year == -318) ) // check special case for 15-oct-1582
        {
            // The date will be ok if it's greather or equal than 15-oct-1582
            isOk = (to_check.tm_mon >= 9) && (to_check.tm_mday >= 15);
        }

        return isOk;
    }
}

/**
 * @brief class cdmwTimeUtils: internal utility class
 */

class cdmwTimeUtils
{
    // check range of value
    public static boolean check_range (int val, int from, int to)
    {
      return ((val >= from) && (val <= to));
    }

    // check positive value
    public static boolean is_positive (int val)
    {
      return (val >= 0);
    }

    // check positive value
    public static boolean is_strict_positive (int val)
    {
      return (val > 0);
    }
}



