/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI.
 * Copyright (C) SELEX SISTEMI INTEGRATI 2004-2009. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

/**
 * @brief implementation for ClockCatalogTest
 *
 * @author Raffaele Mele <rmele@progesi.it>
 */

package cdmw.clock.test;

import java.io.IOException;
import org.omg.CORBA.ORB;
import org.omg.CORBA.ORB;
import org.omg.CORBA.ORBPackage.InvalidName;
import org.omg.CosClockService.*;

import org.omg.CosClockService.ClockCatalogPackage.ClockEntriesHelper;
import org.omg.CosClockService.ClockCatalogPackage.ClockEntry;
import org.omg.CosClockService.ClockCatalogPackage.ClockEntryHolder;
import org.omg.CosClockService.ClockCatalogPackage.UnknownEntry;
import org.omg.CosClockService.ControlledClockHelper;

import cdmw.clock.*;
import cdmw.orbsupport.ORBSupport;
import cdmw.orbsupport.ORBUtils;
import cdmw.ossupport.OS;
import cdmw.ossupport.ThreadedProcess;
import cdmw.testutils.Testable;

///////////////////////////////////////////////////////////////////////////////

public class ClockCatalogTest extends Testable
{
    private ORB m_orb;
    private ClockCatalog m_catalog;
    ControlledClock m_theClock;

    private int m_entries_count = 0;
    private final int NUMOFTESTS = 2;

///////////////////////////////////////////////////////////////////////////////

    private int get_entries_count()
    {
        if ( ( m_catalog == null ) || ( m_orb == null ) ) {
            System.out.println( "nil reference!" );
            return 0;
        }

        ClockEntry[] clockEntries = m_catalog.available_entries();
        return ( clockEntries.length );
    }

///////////////////////////////////////////////////////////////////////////////

    public ClockCatalogTest ( ORB m_orb, ClockCatalog ctg )
    {
        this.name = "ClockCatalogTest";
        this.out = System.out;
        this.m_orb = m_orb;
        this.m_catalog = ctg;
        this.m_entries_count = get_entries_count();
        ClockEntry entry;
        try {
            
            entry = m_catalog.get_entry("ControlledClock");
            m_theClock = org.omg.CosClockService.ControlledClockHelper.narrow(entry.subject);
        }
        catch ( UnknownEntry ex ) 
        {
            System.out.println( "ClockCatalog::UnknownEntry" );
        }
    }

///////////////////////////////////////////////////////////////////////////////

    public boolean available_entries()
    {
        return ( m_entries_count > 0 );
    }

///////////////////////////////////////////////////////////////////////////////

    public boolean entry_management()
    {
        if ( ( m_catalog == null ) || ( m_orb == null ) ) {
            System.out.println( "nil reference!" );
            return Macro.TEST_NOK;
        }

        // variable to test exception raising
        boolean exc = false;

        ClockEntry entry;
        int i,
        length = m_entries_count;

        // entries to add
        final int NEW_ENTRIES = 1000;

        final int MAX_ENTRIES = NEW_ENTRIES + length;

        String oss;

        // add some entry
        for ( i = length; i < MAX_ENTRIES ; ++i ) {
            oss = "Clock n." + i;
            entry = new ClockEntry(m_theClock, oss);
            m_catalog.register( entry );
        }

        // check number of new entry
        if ( get_entries_count() != MAX_ENTRIES )
            return Macro.TEST_NOK;

        ClockEntry entryRef;

        // try to recover available clock
        i = MAX_ENTRIES;

        while ( --i >= length ) {
            oss = "Clock n." + i;

            try {
                entryRef = m_catalog.get_entry( oss );
            }
            catch ( UnknownEntry ex ) {
                System.out.println( "ClockCatalog::UnknownEntry" );
                return Macro.TEST_NOK;
            }
        }

        // remove some entry from the catalog
        for ( i = length; i < MAX_ENTRIES; i += 2 ) {
            oss = "Clock n." + i;

            try {
                m_catalog.delete_entry( oss );
            }
            catch ( UnknownEntry ex ) {
                System.out.println( "ClockCatalog::UnknownEntry" );
                return Macro.TEST_NOK;
            }
        }

        if ( get_entries_count() != ( NEW_ENTRIES >> 1 ) + length )
            return Macro.TEST_NOK;

        // Search for all voices in the catalog
        i = MAX_ENTRIES;

        while ( --i >= length ) {
            exc = false;
            oss = "Clock n." + i;

            try {
                entryRef = m_catalog.get_entry( oss );
            }
            catch ( UnknownEntry ex ) {
                exc = true;
                 // odd elements weren't deleted before
                if ( (( i + length ) & 1) == 1 ) {
                    System.out.println( "ClockCatalog::UnknownEntry" );
                    return Macro.TEST_NOK;
                }
            }
            if (  ( ( i + length ) & 1 ) == 0) { // even element were deleted before
                if ( exc != true ) {
                    return Macro.TEST_NOK;
                }
            }
        }

        // clean entries for new test
        i = MAX_ENTRIES;

        while ( --i >= length ) {
            oss = "Clock n." + i;

            try {
                m_catalog.delete_entry( oss );
            }
            catch ( UnknownEntry ex ) {}

        }

        if ( get_entries_count() != length ) {
            System.out.println( "ClockCatalog::UnknownEntry" );
            return Macro.TEST_NOK;
        }
    return Macro.TEST_OK;
    }

///////////////////////////////////////////////////////////////////////////////

    public void doTests()
    {
        setNbOfRequestedTestOK( NUMOFTESTS );

        check( available_entries() );
        check( entry_management() );
    }
}

