/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI.
 * Copyright (C) SELEX SISTEMI INTEGRATI 2004-2009. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

/**
 * @brief
 *
 * @author Francesca Annunziata <fannunziata@selex-si.com>
 * @author Raffaele Mele <rmele@progesi.it>
 * @author Luisa De Franco <ldefranco@progesi.it>
 */

package cdmw.clock.test;

import cdmw.clock.*;
import org.omg.CORBA.*;
import cdmw.ossupport.OS;
import Cdmw.clock.ClockObserver;
import Cdmw.clock.ControlledClockEx;
import Cdmw.clock.ControlledClockExHelper;
import Cdmw.clock.ControlledClockExPackage.AlreadyPresentEntry;
import cdmw.testutils.Testable;
import org.omg.CosClockService.*;
import org.omg.CosClockService.ClockCatalogPackage.ClockEntriesHelper;
import org.omg.CosClockService.ClockCatalogPackage.ClockEntry;
import org.omg.CosClockService.ClockCatalogPackage.ClockEntryHolder;
import org.omg.CosClockService.ClockCatalogPackage.UnknownEntry;
import org.omg.CosClockService.ControlledClockPackage.NotSupported;

import java.lang.Math;

public class ControlledClockTest extends Testable
{
 
    private final int NUMOFTESTS = 10;
    private final int rate = 10;
    private final long TIMET_IN_A_SEC = 10000000L;
    private final long EPSILON = TIMET_IN_A_SEC/10;
    
    private final long SEC_IN_TIME_UNITS = 10000000; 
    private final long CDMW_TOL = 1000000;
    private final int  MAX_EXEC = 10;
    private final int WAIT_SEC = 1;
    private final int RATE_2 = 3;

    private ControlledClockEx m_controlledClock;
    private Clock m_clock;
    private ClockCatalog m_ctg;
    private boolean test_result;
    private ORB orb;


///////////////////////////////////////////////////////////////////////////////

    public ControlledClockTest ( ORB orb, ClockCatalog ctg )
    {
        name = "ControlledClockTest";
        out = System.out;
        m_ctg = ctg;
        this.orb=orb;

        try
        {
            org.omg.CORBA.Object obj = orb.resolve_initial_references("LocalClock");
            m_clock = org.omg.CosClockService.ClockHelper.narrow(obj);
            if (m_clock == null)
            {
                System.out.println("LocalClock: nil reference.");
                System.exit(1);
            }
            ClockEntry entry = m_ctg.get_entry("ControlledClock");
            m_controlledClock = ControlledClockExHelper.narrow(entry.subject);
            if (m_controlledClock == null)
            {
                System.out.println("ControlledClock: nil reference.");
                System.exit(1);
            }
        }
        catch ( UnknownEntry ex )
        {
            System.out.println( "ClockCatalog::UnknownEntry" );
        }
        catch ( org.omg.CORBA.ORBPackage.InvalidName ex )
        {
            ex.printStackTrace();
        }

    }

///////////////////////////////////////////////////////////////////////////////

    // compare two numbers with tolerance (default CARDAMOM tolerance)
    // tolerance of 100 microsec
    private int is_equal_w_tol(long time_1, long time_2)
    {
        long tol = CDMW_TOL;
        long diff;
        diff= delta(time_1, time_2);
        if (diff <= tol)
            return 1; // Ok
        return 0; // Too difference
    }

    private int is_equal_w_tol(long time_1, long time_2, long tol)
    {
        long diff;
        diff= delta(time_1, time_2);
        if (diff <= tol)
            return 1; // Ok
        return 0; // Too difference
    }
    

    public boolean set_test(long to)
    {
        long first_time;
        long new_time;
        

        try
        {
            // check for null reference
            if (m_controlledClock == null)
            {
                System.out.println("ERROR: null reference (m_controlledClock)");
                return Macro.TEST_NOK;
            }
            
            for (int n=-MAX_EXEC; n<= MAX_EXEC; n++)
            {
                first_time = m_controlledClock.current_time();
                int tmp = 100 * (n > 0 ? n : -n);
                new_time = tmp*SEC_IN_TIME_UNITS;

                if (n < 0) // remove n*100 seconds
                    m_controlledClock.set(first_time - new_time); 
                else // add n*100 seconds
                    m_controlledClock.set(first_time + new_time); 

                new_time = m_controlledClock.current_time();

                if (n < 0) // virtual time is LOWER than real time
                {  
                    if (new_time >= first_time)
                         return Macro.TEST_NOK;       
                }
                else if (n == 0) // virtual time follow the real time
                {   
                    if (is_equal_w_tol(new_time,first_time)!=1)
                        return Macro.TEST_NOK;
                }
                else // virtual time is GREATHER than real time
                {  
                    if (new_time <= first_time)
                        return Macro.TEST_NOK;       
                }
                // test if virtual time is in correct interval
                if (n < 0)
                {  
                    if ((is_equal_w_tol(new_time, first_time , 
                             (100 * (-n) * SEC_IN_TIME_UNITS)) + CDMW_TOL) < 1 )
                         return Macro.TEST_NOK;       
                } 
                else if (n==0)
                {  
                    if (is_equal_w_tol(new_time, first_time) != 1)
                        return Macro.TEST_NOK;       
                }    
                else
                {  
                    if ((is_equal_w_tol(new_time, first_time ,
                          (100 * n * SEC_IN_TIME_UNITS)) + CDMW_TOL) < 1 )
                        return Macro.TEST_NOK;       
                }    
                // reset virtual time modify
                m_controlledClock.set(m_controlledClock.current_time() + 
                100 * n * SEC_IN_TIME_UNITS);
             }

        }
        catch (org.omg.CosClockService.ControlledClockPackage.NotSupported e)
        {
            e.printStackTrace();
            return Macro.TEST_NOK;
        }
        catch (org.omg.CosClockService.TimeUnavailable ni)
        {
            System.out.println("Exception:" + ni);
            return Macro.TEST_NOK;
        }
        return Macro.TEST_OK; 
    }


///////////////////////////////////////////////////////////////////////////////

    public boolean set_and_get_rate_test()
    {
        try
        {
             // check for null reference
            if (m_controlledClock == null)
            {
                System.out.println("ERROR: null reference (m_controlledClock)");
                return Macro.TEST_NOK;
            }
      
            float  rate;
            long aTime = Long.MAX_VALUE>>10;
    
            m_controlledClock.set(aTime);
            for(rate=-10; rate<=10.0; rate+=1.0)
            {   
                m_controlledClock.set_rate(rate);
                double tmp = m_controlledClock.get_rate();
                if ( tmp == rate )
                    test_result = Macro.TEST_OK;
                else
                    test_result = Macro.TEST_NOK;
             }                    

        }
        catch (org.omg.CosClockService.ControlledClockPackage.NotSupported e)
        {
            e.printStackTrace();
            return Macro.TEST_NOK;
        }
        return test_result;
    }

///////////////////////////////////////////////////////////////////////////////

    public boolean pause_test()
    {
        try
        {
            // check for null reference
            if (m_controlledClock == null)
            {
                System.out.println("ERROR: null reference (m_controlledClock)");
                return Macro.TEST_NOK;
            }
    
            cdmw.clock.test.ClockObserverImpl my_observer1 = new cdmw.clock.test.ClockObserverImpl();
            cdmw.clock.test.ClockObserverImpl my_observer2 = new cdmw.clock.test.ClockObserverImpl();
            Cdmw.clock.ClockObserver observer1 = my_observer1._this(orb);
            Cdmw.clock.ClockObserver observer2 = my_observer2._this(orb);
           
            org.omg.CORBA.Any aparam1, aparam2;
            aparam1 = orb.create_any();
            aparam2 = orb.create_any(); 

            m_controlledClock.attach(Cdmw.clock.PAUSE.value, observer1, aparam1);
            m_controlledClock.attach(Cdmw.clock.PAUSE.value | Cdmw.clock.RESUME.value, observer2, aparam2);

            m_controlledClock.pause();
    
            if (my_observer1.getProcessedEvents() != Cdmw.clock.PAUSE.value)
                return Macro.TEST_NOK;
         
            if(my_observer2.getProcessedEvents() != Cdmw.clock.PAUSE.value)
                return Macro.TEST_NOK;

            m_controlledClock.detach(Cdmw.clock.ALL.value, observer1);    
            m_controlledClock.detach(Cdmw.clock.ALL.value, observer2);    
            
        }
        catch (org.omg.CosClockService.ControlledClockPackage.NotSupported e)
        {
            e.printStackTrace();
            return Macro.TEST_NOK;
        }
        catch (Cdmw.clock.ControlledClockExPackage.AlreadyPresentEntry e)
        {
            System.out.println("Exception: " + e);
            return Macro.TEST_NOK;
        }
        
        return Macro.TEST_OK;
    }

///////////////////////////////////////////////////////////////////////////////

    public boolean resume_test()
    {
        try
        {
            // check for null reference
            if (m_controlledClock == null)
            {
                System.out.println("ERROR: null reference (m_controlledClock)");
                return Macro.TEST_NOK;
            }
       
            cdmw.clock.test.ClockObserverImpl my_observer1 = new cdmw.clock.test.ClockObserverImpl();
            cdmw.clock.test.ClockObserverImpl my_observer2 = new cdmw.clock.test.ClockObserverImpl();
            Cdmw.clock.ClockObserver observer1    = my_observer1._this(orb);
            Cdmw.clock.ClockObserver observer2    = my_observer2._this(orb);
            
            org.omg.CORBA.Any aparam1;
            org.omg.CORBA.Any aparam2;
            aparam1 = orb.create_any();
            aparam2 = orb.create_any();
            
            m_controlledClock.attach(Cdmw.clock.RESUME.value, (ClockObserver) observer1, aparam1);
            m_controlledClock.attach(Cdmw.clock.PAUSE.value | Cdmw.clock.RESUME.value, (ClockObserver) observer2, aparam2);

            m_controlledClock.resume();

            if(my_observer1.getProcessedEvents() != Cdmw.clock.RESUME.value)
                return Macro.TEST_NOK;
         
            if(my_observer2.getProcessedEvents() != Cdmw.clock.RESUME.value)
                return Macro.TEST_NOK;
            
            m_controlledClock.detach(Cdmw.clock.ALL.value, (ClockObserver) observer1);
            m_controlledClock.detach(Cdmw.clock.ALL.value, (ClockObserver) observer2);
         
        }
        catch (org.omg.CosClockService.ControlledClockPackage.NotSupported e)
        {
            e.printStackTrace();
            return Macro.TEST_NOK;
        }
        catch (AlreadyPresentEntry e)
        {
            e.printStackTrace();
            return Macro.TEST_NOK;
        }
        return Macro.TEST_OK;
    }

///////////////////////////////////////////////////////////////////////////////

    public boolean terminate_test()
    {
        try
        {
            // check for null reference
            if (m_controlledClock == null)
            {
                System.out.println("ERROR: null reference (m_controlledClock)");
                return Macro.TEST_NOK;
            }
            m_controlledClock.terminate();
         
            cdmw.clock.test.ClockObserverImpl my_observer1 = new cdmw.clock.test.ClockObserverImpl();
            cdmw.clock.test.ClockObserverImpl my_observer2 = new cdmw.clock.test.ClockObserverImpl();
            Cdmw.clock.ClockObserver observer1    = my_observer1._this(orb);
            Cdmw.clock.ClockObserver observer2    = my_observer2._this(orb);
            
            org.omg.CORBA.Any aparam1;
            org.omg.CORBA.Any aparam2;
            aparam1 = orb.create_any();
            aparam2 = orb.create_any();

            m_controlledClock.attach(Cdmw.clock.TERMINATE.value, observer1, aparam1);
            m_controlledClock.attach(Cdmw.clock.PAUSE.value | Cdmw.clock.RESUME.value | Cdmw.clock.TERMINATE.value, observer2, aparam2);

            long previousTime = 0;
            m_controlledClock.set(m_clock.current_time()); // reset modify
            m_controlledClock.set_rate(1); // reset virtual time rate
            
            cdmw.ossupport.OS.sleep(1);
                        
            m_controlledClock.terminate();
            previousTime = m_controlledClock.current_time();
            for (int n=0; n < MAX_EXEC; ++n)
            {
               OS.sleep(WAIT_SEC);
               long actualTime = m_controlledClock.current_time();
               if( (actualTime - previousTime) == 0);
            }

            if(my_observer1.getProcessedEvents() != Cdmw.clock.TERMINATE.value)
                return Macro.TEST_NOK;
            
            if(my_observer2.getProcessedEvents() != Cdmw.clock.TERMINATE.value)
                return Macro.TEST_NOK;
         
            m_controlledClock.detach(Cdmw.clock.ALL.value, observer1);
            m_controlledClock.detach(Cdmw.clock.ALL.value, observer2);

        }
        catch (org.omg.CosClockService.ControlledClockPackage.NotSupported e)
        {
            e.printStackTrace();
            return Macro.TEST_NOK;
        }
        catch (AlreadyPresentEntry e)
        {
            e.printStackTrace();
            return Macro.TEST_NOK;
        }
        catch (TimeUnavailable e)
        {
            e.printStackTrace();
            return Macro.TEST_NOK;
        }
        catch (InterruptedException e)
        {
             e.printStackTrace();
             return Macro.TEST_NOK;
        }
        return Macro.TEST_OK;
    }


///////////////////////////////////////////////////////////////////////////////

    public boolean attach_test()
    {
        //end
        
        cdmw.clock.test.ClockObserverImpl my_observer = new cdmw.clock.test.ClockObserverImpl();
        Cdmw.clock.ClockObserver observer    = my_observer._this(orb);
         
        org.omg.CORBA.Any aparam;
        aparam = orb.create_any();
        try
        {
            m_controlledClock.attach(Cdmw.clock.SET_TIME.value, observer, aparam);
        }
        catch (NotSupported e)
        {
            e.printStackTrace();
            return Macro.TEST_NOK;
        }
        catch (AlreadyPresentEntry e)
        {
            e.printStackTrace();
            return Macro.TEST_NOK;
        }
        
        try
        {
            try
            {
                m_controlledClock.attach(Cdmw.clock.SET_RATE.value, observer, aparam);
            }
            catch (AlreadyPresentEntry e)
            {
                e.printStackTrace();
                return Macro.TEST_NOK;
            }
        }
        catch (org.omg.CosClockService.ControlledClockPackage.NotSupported ns)
        {
            try
            {
                m_controlledClock.detach(Cdmw.clock.ALL.value, observer);
            }
            catch (NotSupported e)
            {
                e.printStackTrace();
                return Macro.TEST_NOK;
            }
            return Macro.TEST_OK;
        }
        return Macro.TEST_NOK;
    }
        
 ///////////////////////////////////////////////////////////////////////////////

    public boolean detach_test() 
    {
        cdmw.clock.test.ClockObserverImpl my_observer1 = new cdmw.clock.test.ClockObserverImpl();
        cdmw.clock.test.ClockObserverImpl my_observer2 = new cdmw.clock.test.ClockObserverImpl();
        Cdmw.clock.ClockObserver observer1    = my_observer1._this(orb);
        Cdmw.clock.ClockObserver observer2    = my_observer2._this(orb);
        
        org.omg.CORBA.Any aparam;
        aparam = orb.create_any();

        try
        {
            m_controlledClock.attach(Cdmw.clock.SET_TIME.value, observer1, aparam);
        }
        catch (NotSupported e1)
        {
             e1.printStackTrace();
             return Macro.TEST_NOK;
        }
        catch (Cdmw.clock.ControlledClockExPackage.AlreadyPresentEntry ap)
        {
            System.out.println("Exception:" + ap);
            return Macro.TEST_NOK;
        }
        try
        {
             m_controlledClock.detach(Cdmw.clock.SET_TIME.value, observer2);
        }
        catch (org.omg.CosClockService.ControlledClockPackage.NotSupported e)
        {
            try
            {
                m_controlledClock.detach(Cdmw.clock.ALL.value, observer1);
            }
            catch (NotSupported e1)
            {
                e1.printStackTrace();
                return Macro.TEST_NOK;
            }
            return Macro.TEST_OK;
        }
    
        return Macro.TEST_NOK;
    }

///////////////////////////////////////////////////////////////////////////////

    public boolean get_time_origins_test()
    {
        org.omg.CORBA.LongHolder x0 = new org.omg.CORBA.LongHolder();
        org.omg.CORBA.LongHolder y0 = new org.omg.CORBA.LongHolder();
        final long tolerance = 35000; // tolerance of 3.5 ms
        
        try
        {
           long value;
           long prev;
           long t0;
           boolean assertion1, assertion2;
           
           value = m_clock.current_time();

           // just to avoid wrong exception
           m_controlledClock.set(value);
           m_controlledClock.set_rate(1);
    
           
           prev = m_controlledClock.current_time();
           
           t0 = 0;
           m_controlledClock.set(t0);
           m_controlledClock.get_time_origins(x0,y0);
                          
           assertion1 = ((value <= x0.value)  && ((x0.value - value) < tolerance));
           assertion2 = (y0.value == t0);

            if (assertion1 && assertion2)
                return Macro.TEST_OK; 
            else
                return Macro.TEST_NOK;
            
        }
        catch (org.omg.CosClockService.ControlledClockPackage.NotSupported e)
        {
            e.printStackTrace();
            return Macro.TEST_NOK;
        }
        catch (org.omg.CosClockService.TimeUnavailable ni)
        {
            System.out.println("Exception:" + ni);
            return Macro.TEST_NOK;
        }
        
    }

///////////////////////////////////////////////////////////////////////////////

    public boolean real2controlled_test()
    {
        org.omg.CORBA.LongHolder virt_time = new org.omg.CORBA.LongHolder();
        org.omg.CORBA.LongHolder realTime0 = new org.omg.CORBA.LongHolder();
        org.omg.CORBA.LongHolder virtualTime0 = new org.omg.CORBA.LongHolder();
        org.omg.CORBA.LongHolder time_val = new org.omg.CORBA.LongHolder();

        // check for null reference
        if (m_controlledClock == null)
        {
            System.out.println("ERROR: null reference (m_controlledClock)");
            return Macro.TEST_NOK;
        }
        try {
            m_controlledClock.set_rate(rate);
            m_controlledClock.set(m_clock.current_time());

            // wait one second to retrieve a new time
            OS.sleep(1000);

            // retrieve actual virtual time: V(t0+1s)
            virt_time.value = m_controlledClock.current_time();

            // retrieve to,V(t0) informations
            m_controlledClock.get_time_origins(realTime0,
                                               virtualTime0);
            // compute reference time
            //long
            time_val.value = realTime0.value + TIMET_IN_A_SEC;

            // retrieve one second in the future
            org.omg.CORBA.LongHolder
            computed_time = Util.real2controlled(m_controlledClock, time_val.value);

            long
            delta_diff = delta(virtualTime0.value,
                               computed_time.value);

            test_result = check_doubles_equal(delta_diff/rate,
                                TIMET_IN_A_SEC,
                                EPSILON);
            //System.out.println("real2controlled: first test_result = " + test_result);
            if ( true == test_result )
            {
               System.out.println("Test NOK: first check_double_equal > EPSILON");
               return (Macro.TEST_NOK);
            }   
            // the same test in the past
            m_controlledClock.set_rate(-rate);
            m_controlledClock.set(m_clock.current_time());
            OS.sleep(1000);
            virt_time.value = m_controlledClock.current_time();

            m_controlledClock.get_time_origins(realTime0,
                                               virtualTime0);
            time_val.value = realTime0.value + TIMET_IN_A_SEC;

            // retrieve one second in the past
            computed_time = Util.real2controlled(m_controlledClock,
                                                 time_val.value);

            delta_diff = delta(virtualTime0.value,
                               computed_time.value);

            test_result = check_doubles_equal(delta_diff/rate,
                                TIMET_IN_A_SEC,
                                EPSILON);
            if ( true == test_result)
            {
               System.out.println("Test NOK: second check_double_equal > EPSILON");
               return (Macro.TEST_NOK);
            }        
        }
        catch (org.omg.CosClockService.ControlledClockPackage.NotSupported ex)
        {
            ex.printStackTrace();
            System.out.println("Exception:" + ex);
            return (Macro.TEST_NOK);
        }
        catch (org.omg.CosClockService.TimeUnavailable ex)
        {
            ex.printStackTrace();
            System.out.println("Exception:" + ex);
            return (Macro.TEST_NOK);
        }
        catch (java.lang.InterruptedException ex)
        {
            ex.printStackTrace();
            System.out.println("Exception:" + ex);
            return (Macro.TEST_NOK);
        }
        return Macro.TEST_OK;
    }

///////////////////////////////////////////////////////////////////////////////

    public boolean controlled2real_test()
    {
        org.omg.CORBA.LongHolder virt_time = new org.omg.CORBA.LongHolder();
        org.omg.CORBA.LongHolder realTime0 = new org.omg.CORBA.LongHolder();
        org.omg.CORBA.LongHolder virtualTime0 = new org.omg.CORBA.LongHolder();
        org.omg.CORBA.LongHolder time_val = new org.omg.CORBA.LongHolder();

        // check for null reference
        if (m_controlledClock == null)
        {
            System.out.println("ERROR: null reference (m_controlledClock)");
            return Macro.TEST_NOK;
        }

        try {

            ClockEntry entry = m_ctg.get_entry("ControlledClock");
            ControlledClockEx extControlledClock = 
                ControlledClockExHelper.narrow(entry.subject);
            if (extControlledClock == null)
            {
                System.out.println("ControlledClock: nil reference.");
                return Macro.TEST_NOK;
            }

            extControlledClock.set_rate(rate);
            extControlledClock.set(m_clock.current_time());

            // wait one second to retrieve a new time
            OS.sleep(1000);

            // retrieve actual virtual time: V(t0+1s)
            virt_time.value = extControlledClock.current_time();

            // retrieve to,V(t0) informations
            extControlledClock.get_time_origins (realTime0,
                                                 virtualTime0);
            // compute current time
            time_val.value = realTime0.value + TIMET_IN_A_SEC;

            // retrieve one second in the future
            long computed_time = Util.controlled2real(extControlledClock, virt_time.value);
            long delta_diff = delta(realTime0.value, computed_time);

            // here time is real
            test_result = check_doubles_equal(delta_diff, TIMET_IN_A_SEC, EPSILON);
            if ( true == test_result )
            {
               //System.out.println("Test NOK: first check_double_equal > EPSILON");
               return (Macro.TEST_NOK);
            }   
            
        }
        catch (org.omg.CosClockService.ControlledClockPackage.NotSupported ex)
        {
            ex.printStackTrace();
            return (Macro.TEST_NOK);
        }
        catch (org.omg.CosClockService.TimeUnavailable ex)
        {
            ex.printStackTrace();
            return (Macro.TEST_NOK);
        }
        catch (org.omg.CosClockService.ClockCatalogPackage.UnknownEntry ex)
        {
            ex.printStackTrace();
            return (Macro.TEST_NOK);
        }
        catch (java.lang.InterruptedException ex)
        {
            ex.printStackTrace();
            return (Macro.TEST_NOK);
        }
        return Macro.TEST_OK;
    } 

///////////////////////////////////////////////////////////////////////////////

    private long
    delta(long num1,
          long num2)
    {
        if (num1 > num2)
            return num1-num2;
        return num2-num1;
    }

///////////////////////////////////////////////////////////////////////////////

    private int
    delta_sign(long num1,
               long num2)
    {
        if (num1 >= num2)
            return 1;
        return -1;
    }

///////////////////////////////////////////////////////////////////////////////

    private
    boolean check_doubles_equal ( long expected, long actual, long delta)
    {
        return Math.abs( expected - actual ) > delta;
    }

///////////////////////////////////////////////////////////////////////////////

    private void dummy (String name)
    {
        System.out.println("Warning (NO_IMPLEMENT): " + name +
                           " is avaliable only at API level");
        throw new org.omg.CORBA.NO_IMPLEMENT();
    }

///////////////////////////////////////////////////////////////////////////////

    public void doTests()
    {
        setNbOfRequestedTestOK(NUMOFTESTS);

        long to = 1;
        check(set_test(to));
        check(attach_test());
        check(detach_test());
        check(set_and_get_rate_test());
        check(pause_test());
        check(resume_test());
        check(get_time_origins_test());
        check(real2controlled_test());
        check(controlled2real_test());
        check(terminate_test());

    }
}

///////////////////////////////////////////////////////////////////////////////
