/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI.
 * Copyright (C) SELEX SISTEMI INTEGRATI 2004-2009. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */
package cdmw.clock.test;

import  cdmw.clock.*;
import cdmw.orbsupport.ORBSupport;
import cdmw.ossupport.OS;
import org.omg.CORBA.ORB;
import cdmw.testutils.Testable;

import org.omg.TimeBase.UtcT;
import org.omg.TimeBase.IntervalT;
import org.omg.CosClockService.UtcTimeService;
import org.omg.CosClockService.UTC;
import org.omg.CosClockService.UTCValueFactory;
import org.omg.CosClockService.TimeSpan;
import org.omg.CosClockService.TimeSpanValueFactory;
import org.omg.CosClockService.*;

/**
* @file TimeSpanTest.java
* @brief unit tests for EVoT TimeSpan valuetype.
* 
* @author Raffaele Mele <rmele@progesi.it>
*/

public class TimeSpanTest extends Testable
{
    private UTCValueFactory utcFactory;
    private TimeSpanValueFactory tspanFactory;

    public TimeSpanTest ()
    {
        this.name = "TimeSpanTest";
        this.out = System.out;
        utcFactory = new UTCValueFactoryImpl();
        tspanFactory = new TimeSpanValueFactoryImpl();
    }

    public void doTests()
    {
        setNbOfRequestedTestOK(7);
        check(init());
        check(compose());
        check(time_interval());
        check(spans());
        check(overlaps());
        check(lower_bound());
        check(upper_bound());
    }

    public
    boolean init()
    {
        boolean testOk = true;
        if( tspanFactory == null )
            testOk = false;
        // prepare an intervalT
        IntervalT from = new IntervalT();
        from.lower_bound = 2000;
        from.upper_bound = 4000;
        // construct the TimeSpan
        TimeSpan tspan = tspanFactory.init(from);
        // test valuetype construction
        if (( tspan.lower_bound != 2000) ||
                ( tspan.upper_bound != 4000))
            testOk = false;

        // prepare another intervalT
        from.lower_bound = 1234;
        from.upper_bound = 4321;
        // construct the TimeSpan
        TimeSpan tspan1 = tspanFactory.init(from);
        // test valuetype construction
        if (( tspan1.lower_bound != 1234) ||
                ( tspan1.upper_bound != 4321))
            testOk = false;
        //if almost one test fail
        if (testOk == false)
            return Macro.TEST_NOK;
        return Macro.TEST_OK;
    }


    public
    boolean compose()
    {
        boolean testOk = true;
        TimeSpan tspan = tspanFactory.compose(3000,9000);
        if (( tspan.lower_bound != 3000) ||
                ( tspan.upper_bound != 9000))
            testOk = false;
        TimeSpan tspan1 = tspanFactory.compose(5678,9876);
        if (( tspan1.lower_bound != 5678) ||
                ( tspan1.upper_bound != 9876))
            testOk = false;
        //if almost one test fail
        if (testOk == false)
            return Macro.TEST_NOK;
        return Macro.TEST_OK;
    }



    public
    boolean time_interval ()
    {
        boolean testOk = true;
        if ( tspanFactory == null )
            testOk = false;
        // prepare an intervalT
        IntervalT from = new IntervalT();
        from.lower_bound = 2000;
        from.upper_bound = 4000;
        // construct the TimeSpan
        TimeSpan tspan = tspanFactory.init(from);
        // get an IntervalT from valuetype
        IntervalT to;
        to = tspan.time_interval();
        // test valuetype construction
        if (( to.lower_bound != from.lower_bound) ||
                ( to.upper_bound != from.upper_bound))
            testOk = false;
        //if almost one test fail
        if (testOk == false)
            return Macro.TEST_NOK;
        return Macro.TEST_OK;

    }



    public
    boolean spans ()
    {
        boolean testOk = true;
        if (( tspanFactory == null ) ||
                ( utcFactory == null ))
            testOk = false;
        // create an useful UTC
        UTC utc1 = utcFactory.compose (2000,100,(short)0,(short)0);
        // TimeSpan out parameter
        TimeSpanHolder tspan_out = new TimeSpanHolder();

        // test for OverlapType.OTContainer
        //   +-----+  :utc
        //  +-------+ :tspan
        TimeSpan tspan1 =
            tspanFactory.compose(utc1.time - utc1.inaccuracy() - 1,
                                 utc1.time + utc1.inaccuracy() + 1);
        if ((tspan1.spans(utc1,tspan_out)!=OverlapType.OTContainer) ||
            // test TimeSpan out parameter
            ((tspan_out.value.lower_bound!=utc1.interval().lower_bound) &&
             (tspan_out.value.upper_bound!=utc1.interval().upper_bound) ))
            testOk = false;

        // test for OverlapType.OTContained
        //  +-----+ :utc
        //   +---+  :tspan
        TimeSpan tspan2 =
            tspanFactory.compose(utc1.time - utc1.inaccuracy() + 1,
                                 utc1.time + utc1.inaccuracy() - 1);
        if ((tspan2.spans(utc1,tspan_out)!=OverlapType.OTContained) ||
                // test TimeSpan out parameter
                ( (tspan_out.value.lower_bound!=tspan2.lower_bound) &&
                  (tspan_out.value.upper_bound!=tspan2.upper_bound) ))
            testOk = false;

        // test for OverlapType.OTOverlap
        //  +-----+   :utc
        //       +--+ :tspan
        TimeSpan tspan3 =
            tspanFactory.compose(utc1.time + utc1.inaccuracy() - 10,
                                 utc1.time + utc1.inaccuracy() + 100);
        if ((tspan3.spans(utc1,tspan_out)!=OverlapType.OTOverlap) ||
                // test TimeSpan out parameter
                ((tspan_out.value.lower_bound!=tspan3.lower_bound) &&
                 (tspan_out.value.upper_bound!=utc1.interval().upper_bound) ))
            testOk = false;

        // test for OverlapType.OTNoOverlap
        //  +-----+      :utc
        //          +--+ :tspan
        TimeSpan tspan4 =
            tspanFactory.compose(utc1.time + utc1.inaccuracy() + 10,
                                 utc1.time + utc1.inaccuracy() + 100);
        if ((tspan4.spans(utc1,tspan_out)!=OverlapType.OTNoOverlap) ||
                // test TimeSpan out parameter
                ((tspan_out.value.lower_bound!=utc1.interval().upper_bound) &&
                 (tspan_out.value.upper_bound!=tspan4.lower_bound) ))
            testOk = false;
        //if almost one test fail
        if (testOk == false)
            return Macro.TEST_NOK;
        return Macro.TEST_OK;
    }



    public
    boolean overlaps ()
    {
        boolean testOk = true;
        //create TimeSpan for overlap test
        TimeSpan tspan1 = tspanFactory.compose(3,9);
        TimeSpan tspan2 = tspanFactory.compose(5,8);
        TimeSpanHolder tspan_out = new TimeSpanHolder();

        // test for OverlapType.OTContainer
        //  +-----+     :tspan1
        //    +--+      :tspan2
        OverlapType ret_type =
            tspan1.overlaps(tspan2,tspan_out );
        if (( ret_type != OverlapType.OTContainer ) ||
                ( tspan_out.value.lower_bound != 5 &&
                  tspan_out.value.upper_bound != 8 ))
            testOk = false;

        // test TimeSpan out parameter
        if ( (tspan_out.value.lower_bound!=tspan2.lower_bound) &&
                (tspan_out.value.upper_bound!=tspan2.upper_bound) )
            testOk = false;

        // test for OverlapType.OTContained
        //  +-----+     :tspan1
        //    +--+      :tspan2
        if ( (tspan2.overlaps(tspan1,tspan_out) != OverlapType.OTContained) ||
                // test TimeSpan out parameter
                ((tspan_out.value.lower_bound!=tspan2.lower_bound) &&
                 (tspan_out.value.upper_bound!=tspan2.upper_bound)) )
            testOk = false;

        // test for OverlapType.OTOverlap
        //  +-----+     :tspan1
        //      +---+   :tspan3
        TimeSpan tspan3 = tspanFactory.compose(7,11);
        if ( (tspan1.overlaps(tspan3,tspan_out) != OverlapType.OTOverlap) ||
                (tspan3.overlaps(tspan1,tspan_out) != OverlapType.OTOverlap) ||
                // test TimeSpan out parameter
                ((tspan_out.value.lower_bound!=tspan3.lower_bound) &&
                 (tspan_out.value.upper_bound!=tspan1.upper_bound)) )
            testOk = false;

        // test for OverlapType.OTNoOverlap
        //  +-----+      :tspan1
        //          +--+ :tspan4
        TimeSpan tspan4 = tspanFactory.compose(13,16);
        if ( (tspan1.overlaps(tspan4,tspan_out) != OverlapType.OTNoOverlap) ||
                (tspan4.overlaps(tspan1,tspan_out) != OverlapType.OTNoOverlap) ||
                // test TimeSpan out parameter
                ((tspan_out.value.lower_bound!=tspan1.upper_bound) &&
                 tspan_out.value.upper_bound!=tspan4.lower_bound) )
            testOk = false;

        // test for the same interval
        //  +-----+ :tspan1
        //  +-----+ :tspan5
        TimeSpan tspan5 = tspanFactory.compose(3,9);
        if ( (tspan1.overlaps(tspan5,tspan_out) != OverlapType.OTContainer) ||
                // test TimeSpan out parameter
                ((tspan_out.value.lower_bound!=tspan1.lower_bound) &&
                 (tspan_out.value.upper_bound!=tspan1.upper_bound) ) ||
                (tspan5.overlaps(tspan1,tspan_out) != OverlapType.OTContainer) ||
                // test TimeSpan out parameter
                ((tspan_out.value.lower_bound!=tspan5.lower_bound) &&
                 (tspan_out.value.upper_bound!=tspan5.upper_bound) ))
            testOk = false;
        //if almost one test fail
        if (testOk == false)
            return Macro.TEST_NOK;
        return Macro.TEST_OK;
    }



    public
    boolean time ()
    {
        boolean testOk = true;
        if( tspanFactory == null )
            testOk = false;
        TimeSpan tspan = tspanFactory.compose(1000,9000);
        UTC utc = tspan.time();

        long intervMidp = (tspan.lower_bound + tspan.upper_bound)/2L;

        long intervInacc = intervMidp - tspan.lower_bound;

        if ((utc.inaccuracy() != intervInacc) ||
                (utc.time != intervMidp) )
            testOk = false;
        //if almost one test fail
        if (testOk == false)
            return Macro.TEST_NOK;
        return Macro.TEST_OK;
    }



    public
    boolean upper_bound ()
    {
        boolean testOk = true;
        TimeSpan tspan = tspanFactory.compose(3000,9000);
        if ( tspan.upper_bound != 9000)
            testOk = false;
        TimeSpan tspan1 = tspanFactory.compose(5678,9876);
        if ( tspan1.upper_bound != 9876)
            testOk = false;
        if (testOk == false)
            return Macro.TEST_NOK;
        return Macro.TEST_OK;
    }



    public
    boolean lower_bound ()
    {
        boolean testOk = true;
        TimeSpan tspan = tspanFactory.compose(3000,9000);
        if ( tspan.lower_bound != 3000)
            testOk = false;
        TimeSpan tspan1 = tspanFactory.compose(5678,9876);
        if ( tspan1.lower_bound != 5678)
            testOk = false;
        if (testOk == false)
            return Macro.TEST_NOK;
        return Macro.TEST_OK;

    }

}
