/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI.
 * Copyright (C) SELEX SISTEMI INTEGRATI 2004-2009. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

/**
 * @brief Definition of class Engine.
 * @author Marco Macellari <mmacellari@progesi.it>
 */

#ifndef ENGINE_HPP
#define ENGINE_HPP

#include "Time/idllib/CdmwTime.stub.hpp"
#include "Time/idllib/CosClockService.stub.hpp"
#include "PathsArchive.hpp"

class Engine
{
    public:

        /**
         * @brief Base class for execption from extended asserters.
         */
        class Exception {};

        /**
         * @brief The controlled executor reference must be valid.
         * This exception class is thrown when the controlled executor reference is not valid.
         */
        class InvalidExecutor : public Exception {};

        /**
         * @brief The tolerance on initial virtual coordinate must be positive.
         * This exception class is thrown when the tolerance on initial virtual coordinate is non-positive.
         */
        class InvalidEpsilonVirt : public Exception {};
                                                                                                                                                            
        /**
         * @brief The tolerance on initial executor rate must be positive.
         * This exception class is thrown when the tolerance on initial executor rate is non-positive.
         */
        class InvalidEpsilonRate : public Exception {};
                                                                                                                                                            
        /**
         * @brief The real period of the controlled clock must be positive.
         * This exception class is thrown when the real period of the controlled clock is non-positive.
         */
        class InvalidPeriod : public Exception {};

        /**
         * @brief The execution limit of the controller must be non-negative.
         * This exception class is thrown when the execution limit of the controller is negative.
         */
        class InvalidExecutionLimit : public Exception {};

        /**
         * @brief The following rules on the path must hold:
         * 1. path.size() >= 2
         * 2. path[0].x == 0.0             (test start real    time is 0.0)
         * 3. path[0].y == 0.0             (test start virtual time is 0.0)
         * 4. path[1].x == path[1].y > 0.0 (test start ratio is 1.0)
         * 5. path[i].y >= 0.0             (for i in [0, path.size() - 1])
         * 6. path[i + 1].x >= path[i].x   (for i in [0, path.size() - 2])
         * This exception class is thrown when these rules are not satisfied.
         */
        class InvalidPath : public Exception {};

        /**
         * @brief The executor current time and rate must be compatible with those
                  of the initial point. This exception class is thrown when the
                  executor current time and rate are not compatible with those of
                  the initial point.
         */
        class InvalidExecutorInitialization : public Exception {};
                                                                                                                                                            
        /**
         * @brief Constructor of class for extended engines.
         * @param executor        is the controlled executor to be tested.
         * @param epsilon_virt    is the tolerance on initial virtual coordinate.
         * @param epsilon_rate    is the tolerance on initial executor rate.
         * @param period          is the real period of the controller.
         * @param execution_limit is the execution limit on the number of do_work invocations.
         * @param path            is the path.
         * @throw InvalidExecutor       if the executor reference is not valid.
         * @throw InvalidEpsilonVirt    if the tolerance on initial virtual coordinate is non-positive.
         * @throw InvalidEpsilonRate    if the tolerance on initial executor rate is non-positive.
         * @throw InvalidPeriod         if the real period of the controller is non-positive.
         * @throw InvalidExecutionLimit if the execution limit of the controller is negative.
         * @throw InvalidPath           if the path is not valid.
         */
        Engine(Cdmw::clock::ControlledExecutor_ptr executor,
               long double                         epsilon_virt,
               long double                         epsilon_rate,
               long double                         period,
               unsigned long                       execution_limit,
               const Path_t&                       path) throw (InvalidExecutor,
                                                                InvalidEpsilonVirt,
                                                                InvalidEpsilonRate,
                                                                InvalidPeriod,
                                                                InvalidExecutionLimit,
                                                                InvalidPath);

        /**
         * @brief Destructor of class for extended engines.
         */
        ~Engine()
        {
            CORBA::release(executor_);
        }

        /**
         * @brief Print all relevant actions detected according to the given policy.
         * @param 'can_set_paused_timer' is true if and only if, compatibly with the
                  real-to-virtual time function defined through the path passed to the
                  constructor, a 'set' or 'set_rate' command can be issued to a paused
                  timer, (before the issuing of command 'resume').
         * @param 'set_rate_only_when_it_changes' is true if and only if, compatibly
                  with the real-to-virtual time function defined through the path passed
                  to the constructor, a 'set_rate' command is issued only when the current
                  rate is different from the rate to be set.
         * @throw InvalidExecutorInitialization if the executor current time and rate
                  are not compatible with those of the initial point.
         */
        void print_relevant_actions(bool can_set_paused_timer          = false,
                                    bool set_rate_only_when_it_changes = false) throw (InvalidExecutorInitialization);

        /**
         * @brief Runs the engine to simulate a test.
         * @param 'can_set_paused_timer' is true if and only if, compatibly with the
                  real-to-virtual time function defined through the path passed to the
                  constructor, a 'set' or 'set_rate' command can be issued to a paused
                  timer, (before the issuing of command 'resume').
         * @param 'set_rate_only_when_it_changes' is true if and only if, compatibly
                  with the real-to-virtual time function defined through the path passed
                  to the constructor, a 'set_rate' command is issued only when the current
                  rate is different from the rate to be set.
         * @throw InvalidExecutorInitialization if the executor current time and rate
                  are not compatible with those of the initial point.
         */
        void run(bool can_set_paused_timer          = false,
                 bool set_rate_only_when_it_changes = false) throw (InvalidExecutorInitialization);

    private:

        Cdmw::clock::ControlledExecutor_ptr executor_; // The controlled executor.

        const long double   epsilon_virt_;    // The tolerance on initial virtual coordinate.
        const long double   epsilon_rate_;    // The tolerance on initial executor rate.
        const long double   period_;          // The real period of the controller.
        const unsigned long execution_limit_; // The execution limit on the number of do_work invocations.
        const Path_t&       path_;            // The path.
        const int           num_points_;      // The number of points in the path.
};

#endif // ENGINE_HPP
