/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI.
 * Copyright (C) SELEX SISTEMI INTEGRATI 2004-2009. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */
                                                                                                                                                            
/**
 * @brief Definition of class PathsArchive.
 * @author Marco Macellari <mmacellari@progesi.it>
 */

#ifndef PATHS_ARCHIVE_HPP
#define PATHS_ARCHIVE_HPP

#include <string>
#include <vector>
#include <map>

typedef struct _point
        {
            long double x;
            long double y;
            _point(long double cx, long double cy) : x(cx), y(cy) {}
        } Point_t;
typedef std::vector<Point_t> Path_t;

class PathsArchive
{
    public:

        /**
         * @brief static method to get the only path archive instance.
         * @return the reference to the only path archive instance.
         */
        static PathsArchive& instance();

        /**
         * @brief destructor of the path archive.
         */
        ~PathsArchive();

        /**
         * @brief method for path archive initialization.
         *        Every call to this method made after the first call does nothing.
         */
        void initialize();

        /**
         * @brief method to get a path of a specified name from the path archive.
         *        If a path with the specified name does not exist, a null pointer is returned.
         * @param path_name the name of the path.
         * @return a pointer to the requested path or null if the requested path is not in the archive.
         */
        Path_t* get_path(const std::string& path_name);

    private:

        /**
         * @brief method to add a path of a specified name to the path archive.
         *        If a path with the specified name already exists, the new path replaces the old path.
         * @param path_name the name of the path.
         * @param path      the path.
         */
        void add_path(const std::string& path_name, const Path_t& path);

        /**
         * @brief method to remove a path of a specified name to the path archive.
         *        If a path with the specified name does not exist, nothing is done.
         * @param path_name the name of the path.
         */
        void remove_path(const std::string& path_name);

        /**
         * @brief method to add all paths needed to execute the controller policy tests.
         */
        void add_all_paths();

        /**
         * @brief method to add all paths needed to execute the controller tests for policy CANCEL_ALL.
         */
        void add_all_cancel_all_policy_test_paths();

        /**
         * @brief method to add all paths needed to execute the controller tests for policy ENFORCE_INTERVAL.
         */
        void add_all_enforce_interval_policy_test_paths();

        /**
         * @brief method to add all paths needed to execute the controller tests for policy ENFORCE_DEADLINE.
         */
        void add_all_enforce_deadline_policy_test_paths();

        /**
         * @brief constructor of the path archive.
         */
        PathsArchive();

        /**
         * @brief copy constructor of the path archive.
         */
        PathsArchive(const PathsArchive&);

        /**
         * @brief assignment operator.
         */
        PathsArchive& operator=(const PathsArchive&);

        typedef std::map<std::string, Path_t> PathsMap_t;

        bool       initialized;
        PathsMap_t paths;
};

#endif // PATHS_ARCHIVE_HPP

