/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI.
 * Copyright (C) SELEX SISTEMI INTEGRATI 2004-2009. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

/**
 * @brief _implementation for ControllerCancelAllPolicyTest with cppUnit library.
 *
 * @author Francesca Annunziata<fannunziata@selex-si.com>, 
 * @author Fabrizio Morciano <fmorciano@selex-si.com>
 * @author Marco Macellari <mmacellari@progesi.it>
 */

#include "testpolicies/ControllerCancelAllPolicyTest.hpp"
#include "clockservice/ControllerCancelAllPolicy.hpp"
#include "clockservice/ControllerUpdateAdmin_impl.hpp"
#include "clockservice/ControlledExecutor_impl.hpp"

#include "Foundation/ossupport/OS.hpp"


#include <string>
#include <iostream>
#include <sstream>

#include "Time/clockservice/test/testcommon/TimeTestManager.hpp"
#include "Time/clockservice/test/testcommon/MyPeriodicExecution.hpp"
#include "Time/clockservice/Util.hpp"
#include <Foundation/common/Assert.hpp>

using namespace CosClockService;
using namespace CosPropertyService;
using namespace Cdmw::clock;
using namespace Cdmw::clock::util;
using namespace CosClockService::PeriodicExecution;
using namespace std;

///////////////////////////////////////////////////////////////////////////////

CPPUNIT_TEST_SUITE_REGISTRATION(ControllerCancelAllPolicyTest);
CPPUNIT_TEST_SUITE_NAMED_REGISTRATION(ControllerCancelAllPolicyTest, "testcancelallpolicy");

///////////////////////////////////////////////////////////////////////////////

#include "ace/TP_Reactor.h"

namespace
{
    class do_set : public controller_functor
    {
        public:
            executor_command get_executor_command()
            {
                return controller_functor::set;
            }

            void operator()(Cdmw::clock::ControlledExecutor_ptr executor)
            {
                executor->set(0);
            }
    };

    class do_set_rate : public controller_functor
    {
        public:
            executor_command get_executor_command()
            {
                return controller_functor::set_rate;
            }

            void operator()(Cdmw::clock::ControlledExecutor_ptr executor)
            {
                executor->set_rate(1.0);
            }
    };

    class do_pause : public controller_functor
    {
        public:
            executor_command get_executor_command()
            {
                return controller_functor::pause;
            }

            void operator()(Cdmw::clock::ControlledExecutor_ptr executor)
            {
                executor->pause();
            }
    };

    class do_resume : public controller_functor
    {
        public:
            executor_command get_executor_command()
            {
                return controller_functor::resume;
            }

            void operator()(Cdmw::clock::ControlledExecutor_ptr executor)
            {
                executor->resume();
            }
    };

    class do_terminate : public controller_functor
    {
        public:
            executor_command get_executor_command()
            {
                return controller_functor::terminate;
            }

            void operator()(Cdmw::clock::ControlledExecutor_ptr executor)
            {
                executor->terminate();
            }
    };

    ///////////////////////////////////////////////////////////////////////////////

    class compare_for_increment : public assertion_functor
    {
        public:
            compare_for_increment(unsigned int increment, unsigned int tolerance)
              : m_increment(increment),
                m_tolerance(tolerance)
            {
            }

            void set_increment(unsigned int increment)
            {
                m_increment = increment;
            }

            void set_tolerance(unsigned int tolerance)
            {
                m_tolerance = tolerance;
            }

            unsigned int get_increment() const
            {
                return m_increment;
            }

            unsigned int get_tolerance() const
            {
                return m_tolerance;
            }

            bool operator()(const std::vector<unsigned int>& v1, const std::vector<unsigned int>& v2)
            {
                if (v1.size() != v2.size()) return false;
                for (size_t i = 0; i < v1.size(); ++i)
                    if (((int) m_increment - (int) m_tolerance) > ((int) v2[i] - (int) v1[i]) ||
                        ((int) m_increment + (int) m_tolerance) < ((int) v2[i] - (int) v1[i])) return false;
                return true;
            }

        private:
            unsigned int m_increment;
            unsigned int m_tolerance;
    };

    ///////////////////////////////////////////////////////////////////////////////
    
    class SimpleExecution : public POA_CosClockService::PeriodicExecution::Periodic
    {
        public:
            SimpleExecution()
            {
            }

            ~SimpleExecution()
            {
            }

            CORBA::Boolean do_work(const CORBA::Any&)
              throw (CORBA::SystemException)
            {
               return true;
            }
    };
}

///////////////////////////////////////////////////////////////////////////////

ControllerCancelAllPolicyTest::ControllerCancelAllPolicyTest()
{  
}

///////////////////////////////////////////////////////////////////////////////

void
ControllerCancelAllPolicyTest::setUp()
{
    try
    {
        Cdmw::TestUtils::CORBATestManager::instance()->add("ClockService");
        m_obj = Cdmw::TestUtils::CORBATestManager::instance()->get_object("ClockService");
        CPPUNIT_ASSERT(!CORBA::is_nil(m_obj.in()));

        ClockCatalog_var clock_catalog = CosClockService::ClockCatalog::_narrow(m_obj.in());
        CPPUNIT_ASSERT(!CORBA::is_nil(clock_catalog.in()));

        ClockCatalog::ClockEntry* entry = clock_catalog->get_entry("ControlledExecutor");
        CPPUNIT_ASSERT(entry);

        m_controlledClock = ControlledClock::_narrow(entry->subject.in());
        CPPUNIT_ASSERT(!CORBA::is_nil(m_controlledClock.in()));
    }
    catch (Cdmw::TestUtils::CORBATestManager::InvalidObject &)
    {
        CPPUNIT_FAIL ("Cdmw::TestUtils::CORBATestManager::InvalidObject");
    }
    catch ( ... )
    {
        CPPUNIT_FAIL ("Unknown exception!");
    }

    try
    {
        m_alias_name = "PolicyControllerUpdateAdmin";
        Cdmw::TestUtils::CORBATestManager::instance()->add(m_alias_name);
        m_obj = Cdmw::TestUtils::CORBATestManager::instance()->get_object(m_alias_name);
        CPPUNIT_ASSERT(!CORBA::is_nil(m_obj.in()));

        m_admin = Cdmw::clock::policy::ControllerUpdateAdmin::_narrow(m_obj.in());
        CPPUNIT_ASSERT(!CORBA::is_nil(m_admin.in()));
    }
    catch(Cdmw::TestUtils::CORBATestManager::InvalidObject &)
    {
        CPPUNIT_FAIL ("Cdmw::TestUtils::CORBATestManager::InvalidObject");
    }
    catch ( ... )
    {
        CPPUNIT_FAIL ("Unknown exception!");
    }
}

///////////////////////////////////////////////////////////////////////////////

void  
ControllerCancelAllPolicyTest::tearDown()
{
}

///////////////////////////////////////////////////////////////////////////////

void
ControllerCancelAllPolicyTest::on_set()
{
    ControlledExecutor_var executor;
    executor = ControlledExecutor::_narrow(m_controlledClock.in());

    ControllerFunctorManager cfm(executor.in(), true);
    cfm.add_controller_functor(new do_set());
    this->checkPolicyForMethods(cfm);
}

///////////////////////////////////////////////////////////////////////////////

void
ControllerCancelAllPolicyTest::on_set_rate()
{
    ControlledExecutor_var executor;
    executor = ControlledExecutor::_narrow(m_controlledClock.in());

    ControllerFunctorManager cfm(executor.in(), true);
    cfm.add_controller_functor(new do_set_rate());
    this->checkPolicyForMethods(cfm);
}

///////////////////////////////////////////////////////////////////////////////

/*
void
ControllerCancelAllPolicyTest::on_pause()
{
    ControlledExecutor_var executor;
    executor = ControlledExecutor::_narrow(m_controlledClock.in());

    ControllerFunctorManager cfm(executor.in(), true);
    cfm.add_controller_functor(new do_pause());
    this->checkPolicyForMethods(cfm);
}
*/

///////////////////////////////////////////////////////////////////////////////

void
ControllerCancelAllPolicyTest::on_pause_resume()
{
    ControlledExecutor_var executor;
    executor = ControlledExecutor::_narrow(m_controlledClock.in());

    ControllerFunctorManager cfm(executor.in(), true);
    cfm.add_controller_functor(new do_pause());
    cfm.add_controller_functor(new do_resume());
    this->checkPolicyForMethods(cfm);
}

///////////////////////////////////////////////////////////////////////////////

void
ControllerCancelAllPolicyTest::on_terminate()
{
    ControlledExecutor_var executor;
    executor = ControlledExecutor::_narrow(m_controlledClock.in());

    ControllerFunctorManager cfm(executor.in(), true);
    cfm.add_controller_functor(new do_terminate());
    this->checkPolicyForMethods(cfm);
}

///////////////////////////////////////////////////////////////////////////////

void
ControllerCancelAllPolicyTest::checkPolicyForMethods(const ControllerFunctorManager& cfm)
{
    const size_t controller_size        = 3;   // number of controllers per executor
    const double controller_real_period = 0.1; // real period of the controller in seconds
    const double tolerance_in_seconds   = 0.1; // tolerance in seconds on the time interval between the two registrations
    const int    sleep_time_between_registrations_sec = 1; // time in seconds between the two registrations
    const int    sleep_time_between_commands_sec      = 1; // time in seconds between two subsequent commands

    vector<POA_CosClockService::PeriodicExecution::Periodic*>  periodics;
    vector<Periodic_var>                                       periodic_vars;
    vector<CosClockService::PeriodicExecution::Controller_var> controller_vars;
    vector<unsigned int>                                       executions1;
    vector<unsigned int>                                       executions2;

    ControlledExecutor_var executor = cfm.get_executor();

    // create periodics and controllers
    CORBA::Any      params;
    TimeBase::TimeT period = seconds2TimeT(controller_real_period);
    for (size_t j = 0; j < controller_size; ++j)
    {
        periodics.push_back(new SimpleExecution);
        periodic_vars.push_back(periodics.back()->_this());

        controller_vars.push_back(executor->enable_periodic_execution(periodic_vars.back().in()));
        controller_vars.back()->start(period, 0, 0, params);
    }

    // apply executor commands
    for (size_t i = 0; i < cfm.get_functors_size(); ++i)
    {
        Cdmw::OsSupport::OS::sleep(sleep_time_between_commands_sec, 0); // wait time between two subsequent commands
        cfm.get_functor(i)->operator()(executor.in());
    }

    // store executions for the 1-st time
    for (size_t i = 0; i < controller_size; ++i)
        executions1.push_back(controller_vars[i]->executions());

    // wait a while between the two registrations
    Cdmw::OsSupport::OS::sleep(sleep_time_between_registrations_sec, 0);

    // store executions for the 2-nd time
    for (size_t i = 0; i < controller_size; ++i)
        executions2.push_back(controller_vars[i]->executions());

    // use executor commands to check for a final block in the controllers executions
    bool blocking_cmd_found = false;
    for (size_t i = 0; (blocking_cmd_found == false) && (i < cfm.get_functors_size()); ++i)
    {
        controller_functor::executor_command exe_cmd = cfm.get_functor(i)->get_executor_command();
        if (exe_cmd == controller_functor::set      ||
            exe_cmd == controller_functor::set_rate ||
            exe_cmd == controller_functor::terminate)
            blocking_cmd_found = true;
    }
    if ((blocking_cmd_found == false) && (cfm.get_functors_size() > 0))
    {
        size_t last_idx = cfm.get_functors_size() - 1;
        controller_functor::executor_command last_exe_cmd = cfm.get_functor(last_idx)->get_executor_command();
        if (last_exe_cmd == controller_functor::pause) blocking_cmd_found = true;
    }

    // use assertion functor to check for correctness
    unsigned int increment = 0, tolerance = 0;
    if (blocking_cmd_found == false)
    {
        increment = (unsigned int) std::floor(((double) sleep_time_between_registrations_sec) / controller_real_period);
        tolerance = (unsigned int) std::floor(tolerance_in_seconds / controller_real_period);
    }
    compare_for_increment cfi(increment, tolerance);
    bool assertion = cfi.operator()(executions1, executions2);

    /* ----- DEBUG CODE [BEGIN] ----- *
    for (size_t i = 0; i < controller_size; ++i)
    {
        std::cerr << "DEBUG: ControllerCancelAllPolicyTest::checkPolicyForMethods: executions1" << '[' << i << "]: " << executions1[i] << std::endl;
        std::cerr << "DEBUG: ControllerCancelAllPolicyTest::checkPolicyForMethods: executions2" << '[' << i << "]: " << executions2[i] << std::endl;
    }
     * ----- DEBUG CODE [ END ] ----- */

    // terminate all controllers
    for (size_t i = 0; i < controller_vars.size(); ++i)
        controller_vars[i]->terminate();

    // communicate the final check result
    CPPUNIT_ASSERT(assertion);
}

///////////////////////////////////////////////////////////////////////////////
// End Of File
///////////////////////////////////////////////////////////////////////////////
