/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI.
 * Copyright (C) SELEX SISTEMI INTEGRATI 2004-2009. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

/**
 * @brief Implementation for ControllerEnforceIntervalPolicyTest with cppUnit library.
 *
 * @author Francesca Annunziata<fannunziata@selex-si.com>, 
 * @author Fabrizio Morciano <fmorciano@selex-si.com>
 * @author Marco Macellari <mmacellari@progesi.it>
 */


#include "testpolicies/ControllerEnforceIntervalPolicyTest.hpp"
#include "clockservice/ControllerCancelAllPolicy.hpp"
#include "clockservice/ControllerUpdateAdmin_impl.hpp"

#include "Foundation/ossupport/OS.hpp"

#include <string>
#include <iostream>
#include <sstream>
#include <vector>

#include "Time/clockservice/test/testcommon/TimeTestManager.hpp"
#include "Time/clockservice/test/testcommon/MyPeriodicExecution.hpp"
#include "Time/clockservice/test/testcommon/anExecution.hpp"
#include "Time/clockservice/Util.hpp"

#include "testpolicies/EnforceIntervalPolicyAsserter.hpp"
#include "testpolicies/EnforceDeadlinePolicyAsserter.hpp"
#include "testpolicies/Engine.hpp"
#include "testpolicies/PathsArchive.hpp"

using namespace CosClockService;
using namespace CosPropertyService;
using namespace Cdmw::clock;
using namespace Cdmw::clock::util;
using namespace CosClockService::PeriodicExecution;
using namespace std;

///////////////////////////////////////////////////////////////////////////////

CPPUNIT_TEST_SUITE_REGISTRATION(ControllerEnforceIntervalPolicyTest);
CPPUNIT_TEST_SUITE_NAMED_REGISTRATION(ControllerEnforceIntervalPolicyTest, "testenforceintervalpolicy");

///////////////////////////////////////////////////////////////////////////////
                                                                                                                                                            
class SimpleExecution : public POA_CosClockService::PeriodicExecution::Periodic
{
    public:
        SimpleExecution()
        {
        }

        ~SimpleExecution()
        {
        }

        CORBA::Boolean do_work(const CORBA::Any&)
          throw (CORBA::SystemException)
        {
            return true;
        }
};

///////////////////////////////////////////////////////////////////////////////

ControllerEnforceIntervalPolicyTest::ControllerEnforceIntervalPolicyTest()
{  
}

///////////////////////////////////////////////////////////////////////////////

void  
ControllerEnforceIntervalPolicyTest::setUp()
{
    try
    {
        Cdmw::TestUtils::CORBATestManager::instance()->add("ClockService");
        m_obj = Cdmw::TestUtils::CORBATestManager::instance()->get_object("ClockService");
        CPPUNIT_ASSERT(!CORBA::is_nil(m_obj.in()));

        ClockCatalog_var clock_catalog = CosClockService::ClockCatalog::_narrow(m_obj.in());
        CPPUNIT_ASSERT(!CORBA::is_nil(clock_catalog.in()));

        ClockCatalog::ClockEntry* entry = clock_catalog->get_entry("ControlledExecutor");
        CPPUNIT_ASSERT(entry);

        m_controlledClock = ControlledClock::_narrow(entry->subject.in());
        CPPUNIT_ASSERT(!CORBA::is_nil(m_controlledClock.in()));

        m_controlledClock->set(Cdmw::clock::compute_current_time());
        m_controlledClock->set_rate(1.0);
    }
    catch (Cdmw::TestUtils::CORBATestManager::InvalidObject &)
    {
        CPPUNIT_FAIL ("Cdmw::TestUtils::CORBATestManager::InvalidObject");
    }
    catch ( ... )
    {
        CPPUNIT_FAIL ("Unknown exception!");
    }
        
    try
    {
        m_alias_name = "PolicyControllerUpdateAdmin";
        Cdmw::TestUtils::CORBATestManager::instance()->add(m_alias_name);
        m_obj = Cdmw::TestUtils::CORBATestManager::instance()->get_object(m_alias_name);
        CPPUNIT_ASSERT(!CORBA::is_nil(m_obj.in()));

        m_admin = Cdmw::clock::policy::ControllerUpdateAdmin::_narrow(m_obj.in());
        CPPUNIT_ASSERT(!CORBA::is_nil(m_admin.in()));
    }
    catch(Cdmw::TestUtils::CORBATestManager::InvalidObject&)
    {
        CPPUNIT_FAIL ("Cdmw::TestUtils::CORBATestManager::InvalidObject");
    }
    catch ( ... )
    {
        CPPUNIT_FAIL ("Unknown exception!");
    }

    try
    {
        PathsArchive::instance().initialize();
    }
    catch ( ... )
    {
        CPPUNIT_FAIL ("Unknown exception!");
    }
}

///////////////////////////////////////////////////////////////////////////////

void  
ControllerEnforceIntervalPolicyTest::tearDown()
{
}

///////////////////////////////////////////////////////////////////////////////

void  
ControllerEnforceIntervalPolicyTest::check_policy()
{
    // check if the policy is not registered by PolicyAdmin
    m_admin->get_policy(Cdmw::clock::policy::ENFORCE_INTERVAL);
}

///////////////////////////////////////////////////////////////////////////////

void
ControllerEnforceIntervalPolicyTest::PCR_0450_on_set_rate()
{
    Asserter* asserter = 0;
    Path_t*   path     = 0;

    try
    {
        // Get the path used for the test.
        path = PathsArchive::instance().get_path(std::string("ControllerEnforceIntervalPolicyTest::PCR_0450_on_set_rate"));

        // If the path is in the archive, then do the test.
        if (path != 0)
        {
            // Define the asserter and engine parameters.
            const long double   time_origin       = 0.0;
            const long double   period            = 4.0;
            const unsigned long execution_limit   = 0;
        // Automatic tests have not misure performaces
            // in order to avoid assertion when the machine is too busy
            // the tolerance range has been increase
            const long double   real_epsilon      = 2.0; // 0.1;
            const long double   virt_epsilon      = 2.0; //0.1;
            const long double   init_virt_epsilon = 0.05;
            const long double   init_rate_epsilon = 0.05;

            // Get a controlled executor reference.
            ControlledExecutor_var executor = ControlledExecutor::_narrow(m_controlledClock.in());

            // Set the enforce interval policy.
            executor->set_controller_update_policy(Cdmw::clock::policy::ENFORCE_INTERVAL);

            // Create the extended asserter.
            asserter = new EnforceIntervalPolicyAsserter(time_origin,
                                                         period,
                                                         execution_limit,
                                                         real_epsilon,
                                                         virt_epsilon,
                                                         *path);

            // Get a periodic reference.
            anExecution<Cdmw::clock::ControlledExecutor,
                        Cdmw::clock::ControlledExecutor_var,
                        Cdmw::clock::ControlledExecutor_ptr>* servant =
                new anExecution<Cdmw::clock::ControlledExecutor,
                                Cdmw::clock::ControlledExecutor_var,
                                Cdmw::clock::ControlledExecutor_ptr>(executor.in(), (anExecutionCallback*) asserter);
            CosClockService::PeriodicExecution::Periodic_var periodic = servant->_this();

            // Get a controller reference.
            CosClockService::PeriodicExecution::Controller_var controller =
                executor->enable_periodic_execution(periodic.in());

            // Create the extended engine.
            Engine engine(executor.in(),
                          init_virt_epsilon,
                          init_rate_epsilon,
                          period,
                          execution_limit,
                          *path);

            CORBA::Any param;

            // Reset executor
            executor->set(0);
            executor->set_rate(1.0);

            // Start the controller
            controller->start(Cdmw::clock::util::seconds2TimeT(period), 0, execution_limit, param);

            // Run the test.
            engine.run();

            // Terminate the controller.
            controller->terminate();
        }
    }
    catch (...)
    {
        // No exception should be thrown: release resources and exit with failure code.
        delete asserter;
        CPPUNIT_FAIL("unexpected exception");
    }

    // Final check: the following two conditions must be verified:
    // 1. check for do_work() invocations that failed.
    // 2. check for do_work() invocations that erroneously were not done.
    bool assertion = (path                                        &&
                      asserter                                    &&
                     (asserter->get_num_failed_executions() == 0) &&
                     (asserter->get_num_remaining_executions() == 0));
    delete asserter;
    CPPUNIT_ASSERT(assertion);
}

///////////////////////////////////////////////////////////////////////////////

void
ControllerEnforceIntervalPolicyTest::PCR_0450_on_set()
{
    Asserter* asserter = 0;
    Path_t*   path     = 0;

    try
    {
        // Get the path used for the test.
        path = PathsArchive::instance().get_path(std::string("ControllerEnforceIntervalPolicyTest::PCR_0450_on_set"));
                                                                                                                                                            
        // If the path is in the archive, then do the test.
        if (path != 0)
        {
            // Define the asserter and engine parameters.
            const long double   time_origin       = 0.0;
            const long double   period            = 4.0;
            const unsigned long execution_limit   = 0;
            // Automatic tests have not misure performaces
            // in order to avoid assertion when the machine is too busy
            // the tolerance range has been increase
            const long double   real_epsilon      = 2.0; // 0.1;
            const long double   virt_epsilon      = 2.0; //0.1;
        
            const long double   init_virt_epsilon = 0.05;
            const long double   init_rate_epsilon = 0.05;

            // Get a controlled executor reference.
            ControlledExecutor_var executor = ControlledExecutor::_narrow(m_controlledClock.in());

            // Set the enforce interval policy.
            executor->set_controller_update_policy(Cdmw::clock::policy::ENFORCE_INTERVAL);

            // Create the extended asserter.
            asserter = new EnforceIntervalPolicyAsserter(time_origin,
                                                         period,
                                                         execution_limit,
                                                         real_epsilon,
                                                         virt_epsilon,
                                                         *path);

            // Get a periodic reference.
            anExecution<Cdmw::clock::ControlledExecutor,
                        Cdmw::clock::ControlledExecutor_var,
                        Cdmw::clock::ControlledExecutor_ptr>* servant =
                new anExecution<Cdmw::clock::ControlledExecutor,
                                Cdmw::clock::ControlledExecutor_var,
                                Cdmw::clock::ControlledExecutor_ptr>(executor.in(), (anExecutionCallback*) asserter);
            CosClockService::PeriodicExecution::Periodic_var periodic = servant->_this();

            // Get a controller reference.
            CosClockService::PeriodicExecution::Controller_var controller =
                executor->enable_periodic_execution(periodic.in());

            // Create the extended engine.
            Engine engine(executor.in(),
                          init_virt_epsilon,
                          init_rate_epsilon,
                          period,
                          execution_limit,
                          *path);

            CORBA::Any param;

            // Reset executor
            executor->set(0);
            executor->set_rate(1.0);

            // Start the controller
            controller->start(Cdmw::clock::util::seconds2TimeT(period), 0, execution_limit, param);

            // Run the test.
            engine.run();

            // Terminate the controller.
            controller->terminate();
        }
    }
    catch (...)
    {
        // No exception should be thrown: release resources and exit with failure code.
        delete asserter;
        CPPUNIT_FAIL("unexpected exception");
    }

    // Final check: the following two conditions must be verified:
    // 1. check for do_work() invocations that failed.
    // 2. check for do_work() invocations that erroneously were not done.
    bool assertion = (path                                        &&
                      asserter                                    &&
                     (asserter->get_num_failed_executions() == 0) &&
                     (asserter->get_num_remaining_executions() == 0));
    delete asserter;
    CPPUNIT_ASSERT(assertion);
}

///////////////////////////////////////////////////////////////////////////////

void
ControllerEnforceIntervalPolicyTest::PCR_0450_on_pause_resume()
{
    Asserter* asserter = 0;
    Path_t*   path     = 0;

    try
    {
        // Get the path used for the test.
        path = PathsArchive::instance().get_path(std::string("ControllerEnforceIntervalPolicyTest::PCR_0450_on_pause_resume"));
                                                                                                                                                            
        // If the path is in the archive, then do the test.
        if (path != 0)
        {
            // Define the asserter and engine parameters.
            const long double   time_origin       = 0.0;
            const long double   period            = 4.0;
            const unsigned long execution_limit   = 0;
            // Automatic tests have not misure performaces
            // in order to avoid assertion when the machine is too busy
            // the tolerance range has been increase
            const long double   real_epsilon      = 2.0; // 0.1;
            const long double   virt_epsilon      = 2.0; //0.1;

        const long double   init_virt_epsilon = 0.05;
            const long double   init_rate_epsilon = 0.05;

            // Get a controlled executor reference.
            ControlledExecutor_var executor = ControlledExecutor::_narrow(m_controlledClock.in());

            // Set the enforce interval policy.
            executor->set_controller_update_policy(Cdmw::clock::policy::ENFORCE_INTERVAL);

            // Create the extended asserter.
            asserter = new EnforceIntervalPolicyAsserter(time_origin,
                                                         period,
                                                         execution_limit,
                                                         real_epsilon,
                                                         virt_epsilon,
                                                         *path);

            // Get a periodic reference.
            anExecution<Cdmw::clock::ControlledExecutor,
                        Cdmw::clock::ControlledExecutor_var,
                        Cdmw::clock::ControlledExecutor_ptr>* servant =
                new anExecution<Cdmw::clock::ControlledExecutor,
                                Cdmw::clock::ControlledExecutor_var,
                                Cdmw::clock::ControlledExecutor_ptr>(executor.in(), (anExecutionCallback*) asserter);
            CosClockService::PeriodicExecution::Periodic_var periodic = servant->_this();

            // Get a controller reference.
            CosClockService::PeriodicExecution::Controller_var controller =
                executor->enable_periodic_execution(periodic.in());

            // Create the extended engine.
            Engine engine(executor.in(),
                          init_virt_epsilon,
                          init_rate_epsilon,
                          period,
                          execution_limit,
                          *path);

            CORBA::Any param;

            // Reset executor
            executor->set(0);
            executor->set_rate(1.0);

            // Start the controller
            controller->start(Cdmw::clock::util::seconds2TimeT(period), 0, execution_limit, param);

            // Run the test.
            engine.run();

            // Terminate the controller.
            controller->terminate();
        }
    }
    catch (...)
    {
        // No exception should be thrown: release resources and exit with failure code.
        delete asserter;
        CPPUNIT_FAIL("unexpected exception");
    }

    // Final check: the following two conditions must be verified:
    // 1. check for do_work() invocations that failed.
    // 2. check for do_work() invocations that erroneously were not done.
    bool assertion = (path                                        &&
                      asserter                                    &&
                     (asserter->get_num_failed_executions() == 0) &&
                     (asserter->get_num_remaining_executions() == 0));
    delete asserter;
    CPPUNIT_ASSERT(assertion);
}

///////////////////////////////////////////////////////////////////////////////
/*
void
ControllerEnforceIntervalPolicyTest::PCR_0450_on_mixed_controls()
{
    ControlledExecutor_var executor = ControlledExecutor::_narrow(m_controlledClock.in());
    executor->set_controller_update_policy(Cdmw::clock::policy::ENFORCE_INTERVAL);
    executor->set(0);
    executor->set_rate(1.0);

    const int ref_period = 4, delta = ref_period / 2;
    myExtendedAsserter::Point_t points_array[9] = { {  0.00, 0.00 },
                                                    {  1.00, 3.00 },
                                                    {  2.00, 3.00 },
                                                    {  3.00, 2.00 },
                                                    {  6.00, 1.00 },
                                                    {  7.00, 4.00 },
                                                    {  8.00, 4.00 },
                                                    {  9.00, 5.00 },
                                                    { 10.00, 6.00 } };
    myExtendedAsserter anAsserter(ref_period, 9, points_array);
    anExecution<Cdmw::clock::ControlledExecutor,
                Cdmw::clock::ControlledExecutor_var,
                Cdmw::clock::ControlledExecutor_ptr>* periodic =
        new anExecution<Cdmw::clock::ControlledExecutor,
                        Cdmw::clock::ControlledExecutor_var,
                        Cdmw::clock::ControlledExecutor_ptr>(executor.in(), &anAsserter);
    Periodic_var periodic_var = periodic->_this();

    CosClockService::PeriodicExecution::Controller_var controller_var =
        executor->enable_periodic_execution(periodic_var.in());

    TimeBase::TimeT period = Cdmw::clock::util::seconds2TimeT(ref_period);
    CORBA::Any params;

    controller_var->start(period, 0, 0, params);
    TimeBase::TimeT time_set;

    Cdmw::OsSupport::OS::sleep(delta);
    time_set = Cdmw::clock::util::seconds2TimeT(2.5 * ref_period);
    executor->set(time_set);
    std::cerr << "\nSet time to " << Cdmw::clock::util::TimeT2seconds(time_set) << std::endl;

    Cdmw::OsSupport::OS::sleep(ref_period);
    executor->set_rate(-1.0);
    std::cerr << "\nSet rate to -1.0" << std::endl;

    Cdmw::OsSupport::OS::sleep(2 * ref_period);
    executor->pause();
    std::cerr << "\nPaused executor" << std::endl;

    Cdmw::OsSupport::OS::sleep(2 * ref_period);
    executor->resume();
    std::cerr << "\nResumed executor" << std::endl;

    Cdmw::OsSupport::OS::sleep(ref_period);
    time_set = Cdmw::clock::util::seconds2TimeT(4.5 * ref_period);
    executor->set(time_set);
    std::cerr << "\nSet time to " << Cdmw::clock::util::TimeT2seconds(time_set) << std::endl;

    Cdmw::OsSupport::OS::sleep(ref_period);
    executor->set_rate(1.0);
    std::cerr << "\nSet rate to 1.0" << std::endl;

    Cdmw::OsSupport::OS::sleep(3 * ref_period);
    controller_var->terminate();

    CPPUNIT_ASSERT(anAsserter.get_num_execs() == anAsserter.get_num_max_execs());
}
*/
///////////////////////////////////////////////////////////////////////////////

void
ControllerEnforceIntervalPolicyTest::on_set_time_and_rate_while_in_pause()
{
    Asserter* asserter = 0;
    Path_t*   path     = 0;

    try
    {
        // Get the path used for the test.
        path = PathsArchive::instance().get_path(std::string("ControllerEnforceIntervalPolicyTest::on_set_time_and_rate_while_in_pause"));
                                                                                                                                                            
        // If the path is in the archive, then do the test.
        if (path != 0)
        {
            // Define the asserter and engine parameters.
            const long double   time_origin       = 0.0;
            const long double   period            = 4.0;
            const unsigned long execution_limit   = 0;
            // Automatic tests have not misure performaces
            // in order to avoid assertion when the machine is too busy
            // the tolerance range has been increase
            const long double   real_epsilon      = 2.0; // 0.1;
            const long double   virt_epsilon      = 2.0; //0.1;
        
            const long double   init_virt_epsilon = 0.05;
            const long double   init_rate_epsilon = 0.05;

            // Get a controlled executor reference.
            ControlledExecutor_var executor = ControlledExecutor::_narrow(m_controlledClock.in());

            // Set the enforce interval policy.
            executor->set_controller_update_policy(Cdmw::clock::policy::ENFORCE_INTERVAL);

            // Create the extended asserter.
            asserter = new EnforceIntervalPolicyAsserter(time_origin,
                                                         period,
                                                         execution_limit,
                                                         real_epsilon,
                                                         virt_epsilon,
                                                         *path);

            // Get a periodic reference.
            anExecution<Cdmw::clock::ControlledExecutor,
                        Cdmw::clock::ControlledExecutor_var,
                        Cdmw::clock::ControlledExecutor_ptr>* servant =
                new anExecution<Cdmw::clock::ControlledExecutor,
                                Cdmw::clock::ControlledExecutor_var,
                                Cdmw::clock::ControlledExecutor_ptr>(executor.in(), (anExecutionCallback*) asserter);
            CosClockService::PeriodicExecution::Periodic_var periodic = servant->_this();

            // Get a controller reference.
            CosClockService::PeriodicExecution::Controller_var controller =
                executor->enable_periodic_execution(periodic.in());

            // Create the extended engine.
            Engine engine(executor.in(),
                          init_virt_epsilon,
                          init_rate_epsilon,
                          period,
                          execution_limit,
                          *path);

            CORBA::Any param;

            // Reset executor
            executor->set(0);
            executor->set_rate(1.0);

            // Start the controller
            controller->start(Cdmw::clock::util::seconds2TimeT(period), 0, execution_limit, param);

            // Run the test. Note the 'true' argument specifying the possibility to set a paused timer.
            engine.run(true);

            // Terminate the controller.
            controller->terminate();
        }
    }
    catch (...)
    {
        // No exception should be thrown: release resources and exit with failure code.
        delete asserter;
        CPPUNIT_FAIL("unexpected exception");
    }

    // Final check: the following two conditions must be verified:
    // 1. check for do_work() invocations that failed.
    // 2. check for do_work() invocations that erroneously were not done.
    bool assertion = (path                                        &&
                      asserter                                    &&
                     (asserter->get_num_failed_executions() == 0) &&
                     (asserter->get_num_remaining_executions() == 0));
    delete asserter;
    CPPUNIT_ASSERT(assertion);
}

///////////////////////////////////////////////////////////////////////////////

void
ControllerEnforceIntervalPolicyTest::more_controllers_with_different_policies_at_the_same_time()
{
    bool      assertion1 = false;
    bool      assertion2 = false;
    Asserter* asserter1  = 0;
    Asserter* asserter2  = 0;
    Path_t*   path       = 0;

    try
    {
        const int controller_size = 3;
        std::vector<POA_CosClockService::PeriodicExecution::Periodic*>  periodics;
        std::vector<Periodic_var>                                       periodic_vars;
        std::vector<CosClockService::PeriodicExecution::Controller_var> controller_vars;
        std::vector<unsigned int>                                       executions;

        // Get the path used for the test
        path = PathsArchive::instance().
               get_path(std::string("ControllerEnforceIntervalPolicyTest::more_controllers_with_different_policies_at_the_same_time"));

        // If the path is in the archive, then do the test.
        if (path != 0)
        {
            // Define the asserter and engine parameters.
            const long double   time_origin     = 0.0;
            const long double   period          = 4.0;
            const unsigned long execution_limit = 0;
            // Automatic tests have not misure performaces
            // in order to avoid assertion when the machine is too busy
            // the tolerance range has been increase
            const long double   real_epsilon      = 2.0; // 0.1;
            const long double   virt_epsilon      = 2.0; //0.1;

            // Get a controlled executor reference.
            ControlledExecutor_var executor = ControlledExecutor::_narrow(m_controlledClock.in());

            // Create the extended asserter.
            asserter1 = new EnforceIntervalPolicyAsserter(time_origin,
                                                          period,
                                                          execution_limit,
                                                          real_epsilon,
                                                          virt_epsilon,
                                                          *path);

            // Get a periodic reference.
            anExecution<Cdmw::clock::ControlledExecutor,
                        Cdmw::clock::ControlledExecutor_var,
                        Cdmw::clock::ControlledExecutor_ptr>* servant1 =
                new anExecution<Cdmw::clock::ControlledExecutor,
                                Cdmw::clock::ControlledExecutor_var,
                                Cdmw::clock::ControlledExecutor_ptr>(executor.in(), (anExecutionCallback*) asserter1);
            CosClockService::PeriodicExecution::Periodic_var periodic1 = servant1->_this();

            // Get a controller reference.
            CosClockService::PeriodicExecution::Controller_var controller1 =
                executor->enable_periodic_execution_with_policy(periodic1.in(), Cdmw::clock::policy::ENFORCE_INTERVAL);

            // Create the extended asserter.
            asserter2 = new EnforceDeadlinePolicyAsserter(time_origin,
                                                          period,
                                                          execution_limit,
                                                          real_epsilon,
                                                          virt_epsilon,
                                                          *path);

            // Get a periodic reference.
            anExecution<Cdmw::clock::ControlledExecutor,
                        Cdmw::clock::ControlledExecutor_var,
                        Cdmw::clock::ControlledExecutor_ptr>* servant2 =
                new anExecution<Cdmw::clock::ControlledExecutor,
                                Cdmw::clock::ControlledExecutor_var,
                                Cdmw::clock::ControlledExecutor_ptr>(executor.in(), (anExecutionCallback*) asserter2);
            CosClockService::PeriodicExecution::Periodic_var periodic2 = servant2->_this();

            // Get a controller reference.
            CosClockService::PeriodicExecution::Controller_var controller2 =
                executor->enable_periodic_execution_with_policy(periodic2.in(), Cdmw::clock::policy::ENFORCE_DEADLINE);

            CORBA::Any param;

            // Get periodic references
            for (int i = 0; i < controller_size; ++i)
            {
                periodics.push_back(new SimpleExecution);
                periodic_vars.push_back(periodics.back()->_this());
            }

            // Reset executor
            executor->set(0);
            executor->set_rate(1.0);

            // Start the controller (enforce interval policy)
            controller1->start(Cdmw::clock::util::seconds2TimeT(period), 0, execution_limit, param);

            // Start the controller (enforce deadline policy)
            controller2->start(Cdmw::clock::util::seconds2TimeT(period), 0, execution_limit, param);

            // Start the controllers (cancel all policy)
            for (int i = 0; i < controller_size; ++i)
            {
                controller_vars.push_back(executor->enable_periodic_execution_with_policy(periodic_vars.back().in(), Cdmw::clock::policy::CANCEL_ALL));
                controller_vars.back()->start(Cdmw::clock::util::seconds2TimeT(period), 0, execution_limit, param);
            }

            // drive the executor
            Cdmw::OsSupport::OS::sleep(6, 0);
            executor->set_rate(0.5);
            Cdmw::OsSupport::OS::sleep(12, 0);
            executor->set_rate(2.0);
            Cdmw::OsSupport::OS::sleep(6, 0);

            // Terminate the controllers.
            controller1->terminate();
            controller2->terminate();

            for (int i = 0; i < controller_size; ++i)
                executions.push_back(controller_vars[i]->executions());

            Cdmw::OsSupport::OS::sleep(1, 0);

            assertion1 = true;
            for (int i = 0; (i < controller_size) && assertion1; ++i)
            {
                assertion1 = (controller_vars[i]->executions() == 2);
            }
        }
    }
    catch (...)
    {
        // No exception should be thrown: release resources and exit with failure code.
        delete asserter1;
        delete asserter2;
        CPPUNIT_FAIL("unexpected exception");
    }
                                                                                                                                                             
    // Final check: the following two conditions must be verified:
    // 1. check for do_work() invocations that failed.
    // 2. check for do_work() invocations that erroneously were not done.
    assertion2 = (path                                            &&
                  asserter1                                       &&
                  asserter2                                       &&
                 (asserter1->get_num_failed_executions() == 0)    &&
                 (asserter2->get_num_failed_executions() == 0)    &&
                 (asserter1->get_num_remaining_executions() == 0) &&
                 (asserter2->get_num_remaining_executions() == 0));
    delete asserter1;
    delete asserter2;
    CPPUNIT_ASSERT(assertion1 && assertion2);
}

///////////////////////////////////////////////////////////////////////////////

void
ControllerEnforceIntervalPolicyTest::PCR_0450_on_terminate()
{
    Asserter* asserter = 0;
    Path_t*   path     = 0;

    try
    {
        // Get the path used for the test.
        path = PathsArchive::instance().get_path(std::string("ControllerEnforceIntervalPolicyTest::PCR_0450_on_terminate"));

        // If the path is in the archive, then do the test.
        if (path != 0)
        {
            // Define the asserter and engine parameters.
            const long double   time_origin     = 0.0;
            const long double   period          = 4.0;
            const unsigned long execution_limit = 0;
            // Automatic tests have not misure performaces
            // in order to avoid assertion when the machine is too busy
            // the tolerance range has been increase
            const long double   real_epsilon      = 2.0; // 0.1;
            const long double   virt_epsilon      = 2.0; //0.1;

            // Get a controlled executor reference.
            ControlledExecutor_var executor = ControlledExecutor::_narrow(m_controlledClock.in());

            // Set the enforce interval policy.
            executor->set_controller_update_policy(Cdmw::clock::policy::ENFORCE_INTERVAL);

            // Create the extended asserter.
            asserter = new EnforceIntervalPolicyAsserter(time_origin,
                                                         period,
                                                         execution_limit,
                                                         real_epsilon,
                                                         virt_epsilon,
                                                         *path);

            // Get a periodic reference.
            anExecution<Cdmw::clock::ControlledExecutor,
                        Cdmw::clock::ControlledExecutor_var,
                        Cdmw::clock::ControlledExecutor_ptr>* servant =
                new anExecution<Cdmw::clock::ControlledExecutor,
                                Cdmw::clock::ControlledExecutor_var,
                                Cdmw::clock::ControlledExecutor_ptr>(executor.in(), (anExecutionCallback*) asserter);
            CosClockService::PeriodicExecution::Periodic_var periodic = servant->_this();

            // Get a controller reference.
            CosClockService::PeriodicExecution::Controller_var controller =
                executor->enable_periodic_execution(periodic.in());

            CORBA::Any param;

            // Initialize the executor.
            executor->set(0);
            executor->set_rate(1.0);

            // Start the controller
            controller->start(Cdmw::clock::util::seconds2TimeT(period), 0, execution_limit, param);

            // Terminate the executor after 10 seconds from controller start.
            int int_period = (int) period;
            Cdmw::OsSupport::OS::sleep(2 * int_period + int_period / 2, 0); // 10 seconds sleep
            executor->terminate();

            // Wait for a while, just to make sure that no other invocations of do_work are made.
            Cdmw::OsSupport::OS::sleep(2 * int_period , 0); // 8 seconds sleep
            //controller->terminate(); // NB: not needed: executor->terminate() already did the work...
        }
    }
    catch (...)
    {
        // No exception should be thrown: release resources and exit with failure code.
        delete asserter;
        CPPUNIT_FAIL("unexpected exception");
    }

    // Final check: the following two conditions must be verified:
    // 1. check for do_work() invocations that failed.
    // 2. check for do_work() invocations that erroneously were not done.
    bool assertion = (path                                        &&
                      asserter                                    &&
                     (asserter->get_num_failed_executions() == 0) &&
                     (asserter->get_num_remaining_executions() == 0));
    delete asserter;
    CPPUNIT_ASSERT(assertion);
}

///////////////////////////////////////////////////////////////////////////////
// End Of File
///////////////////////////////////////////////////////////////////////////////
