/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI.
 * Copyright (C) SELEX SISTEMI INTEGRATI 2004-2009. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

/**
 * @brief Implementation of class Engine.
 * @author Marco Macellari <mmacellari@progesi.it>
 */


#include <cmath>
#include <ctime>
#include <cmath>
#include <iomanip>
#include <iostream>
#include "testpolicies/Engine.hpp"
#include "Time/clockservice/Util.hpp"
#include "Foundation/ossupport/OS.hpp"


const double sec2nsec = 1000000000.0;

Engine::Engine(Cdmw::clock::ControlledExecutor_ptr executor,
               long double                         epsilon_virt,
               long double                         epsilon_rate,
               long double                         period,
               unsigned long                       execution_limit,
               const Path_t&                       path) throw (InvalidExecutor,
                                                                InvalidEpsilonVirt,
                                                                InvalidEpsilonRate,
                                                                InvalidPeriod,
                                                                InvalidExecutionLimit,
                                                                InvalidPath)
    : epsilon_virt_   (epsilon_virt),
      epsilon_rate_   (epsilon_rate),
      period_         (period),
      execution_limit_(execution_limit),
      path_           (path),
      num_points_     (path.size())
{
    if (CORBA::is_nil(executor))  throw InvalidExecutor();
    if (epsilon_virt    <=   0.0) throw InvalidEpsilonVirt();
    if (epsilon_rate    <=   0.0) throw InvalidEpsilonRate();
    if (period          <=   0.0) throw InvalidPeriod();
    if (execution_limit <      0) throw InvalidExecutionLimit();
    if (path.size() <          2) throw InvalidPath();
    if (path[0].x   !=       0.0) throw InvalidPath();
    if (path[0].y   !=       0.0) throw InvalidPath();
    if (path[1].x   != path[1].y) throw InvalidPath();
    if (path[1].y   ==       0.0) throw InvalidPath();
    for (int i = 0; i < num_points_; i++)
        if (path[i].y < 0.0)
            throw InvalidPath();
    for (int i = 0; i < num_points_ - 1; i++)
        if (path[i + 1].x < path[i].x)
            throw InvalidPath();

    executor_ = Cdmw::clock::ControlledExecutor::_duplicate(executor);
}

void Engine::print_relevant_actions(bool can_set_paused_timer, bool set_rate_only_when_it_changes) throw (InvalidExecutorInitialization)
{
    try
    {
        // check the executor intialization is valid
        double initial_virtual_coordinate = Cdmw::clock::util::TimeT2seconds(executor_->current_time());
        double initial_executor_rate      = (double) executor_->get_rate();
        if (initial_virtual_coordinate >= epsilon_virt_) throw InvalidExecutorInitialization();
        if ((1.0 - epsilon_rate_ >= initial_executor_rate) ||
            (1.0 + epsilon_rate_ <= initial_executor_rate)) throw InvalidExecutorInitialization();

        std::cerr << "List of relevant actions:" << std::endl;

        // Drive the executor.
        bool in_pause = false;
        double curr_executor_rate = initial_executor_rate;
        for (int i = 0; i < num_points_ - 1; i++)
        {
            long double dx = path_[i + 1].x - path_[i].x;
            long double dy = path_[i + 1].y - path_[i].y;

            if (dx == 0.0)
            {
                if (can_set_paused_timer == true)
                {
                    std::cerr << "executor_->set(" << Cdmw::clock::util::seconds2TimeT(path_[i + 1].y) << ')'
                              << " [" << path_[i + 1].y << " sec]" << std::endl;
                    if (in_pause == true) std::cerr << "executor_->resume()" << std::endl;
                }
                else
                {
                    if (in_pause == true) std::cerr << "executor_->resume()" << std::endl;
                    std::cerr << "executor_->set(" << Cdmw::clock::util::seconds2TimeT(path_[i + 1].y) << ')'
                              << " [" << path_[i + 1].y << " sec]" << std::endl;
                }
                in_pause = false;
            }
            else if (dy == 0.0)
            {
                if (in_pause == false) std::cerr << "executor_->pause()" << std::endl;
                unsigned long secs  = (unsigned long) ::floorl(dx);
                unsigned long nsecs = (unsigned long) ::floorl((dx - secs) * sec2nsec);
                std::cerr << "Cdmw::OsSupport::OS::sleep(" << secs << ", " << nsecs << ')'
                          << " [" << dx << " secs]" << std::endl;
                in_pause = true;
            }
            else
            {
                if (can_set_paused_timer == true)
                {
                    if ((set_rate_only_when_it_changes == false) || (curr_executor_rate != (dy / dx)))
                        std::cerr << "executor_->set_rate(" << (dy / dx) << ')' << std::endl;
                    if (in_pause == true) std::cerr << "executor_->resume()" << std::endl;
                }
                else
                {
                    if (in_pause == true) std::cerr << "executor_->resume()" << std::endl;
                    if ((set_rate_only_when_it_changes == false) || (curr_executor_rate != (dy / dx)))
                        std::cerr << "executor_->set_rate(" << (dy / dx) << ')' << std::endl;
                }
                unsigned long secs  = (unsigned long) ::floorl(dx);
                unsigned long nsecs = (unsigned long) ::floorl((dx - secs) * sec2nsec);
                std::cerr << "Cdmw::OsSupport::OS::sleep(" << secs << ", " << nsecs << ')'
                          << " [" << dx << " secs]" << std::endl;
                curr_executor_rate = (dy / dx);
                in_pause = false;
            }
        }
    }
    catch (InvalidExecutorInitialization&)
    {
        throw;
    }
    catch (...)
    {
        // An exception is unexpected.
        std::cerr << "Unexpected exception: engine halted." << std::endl;
    }
}

void Engine::run(bool can_set_paused_timer, bool set_rate_only_when_it_changes) throw (InvalidExecutorInitialization)
{
    try
    {
        // check the executor intialization is valid
        double initial_virtual_coordinate = Cdmw::clock::util::TimeT2seconds(executor_->current_time());
        double initial_executor_rate      = (double) executor_->get_rate();
        if (initial_virtual_coordinate >= epsilon_virt_) throw InvalidExecutorInitialization();
        if ((1.0 - epsilon_rate_ >= initial_executor_rate) ||
            (1.0 + epsilon_rate_ <= initial_executor_rate)) throw InvalidExecutorInitialization();

        // Drive the executor
        bool in_pause = false;
        double curr_executor_rate = initial_executor_rate;
        for (int i = 0; i < num_points_ - 1; i++)
        {
            long double dx = path_[i + 1].x - path_[i].x;
            long double dy = path_[i + 1].y - path_[i].y;

            if (dx == 0.0)
            {
                if (can_set_paused_timer == true)
                {
                    executor_->set(Cdmw::clock::util::seconds2TimeT(path_[i + 1].y));
                    if (in_pause == true) executor_->resume();
                }
                else
                {
                    if (in_pause == true) executor_->resume();
                    executor_->set(Cdmw::clock::util::seconds2TimeT(path_[i + 1].y));
                }
                in_pause = false;
            }
            else if (dy == 0.0)
            {
                if (in_pause == false) executor_->pause();
                unsigned long secs  = (unsigned long) ::floorl(dx);
                unsigned long nsecs = (unsigned long) ::floorl((dx - secs) * sec2nsec);
                Cdmw::OsSupport::OS::sleep((unsigned int) secs, (unsigned int) nsecs);
                in_pause = true;
            }
            else
            {
                if (can_set_paused_timer == true)
                {
                    if ((set_rate_only_when_it_changes == false) || (curr_executor_rate != (dy / dx))) executor_->set_rate(dy / dx);
                    if (in_pause == true) executor_->resume();
                }
                else
                {
                    if (in_pause == true) executor_->resume();
                    if ((set_rate_only_when_it_changes == false) || (curr_executor_rate != (dy / dx))) executor_->set_rate(dy / dx);
                }
                unsigned long secs  = (unsigned long) ::floorl(dx);
                unsigned long nsecs = (unsigned long) ::floorl((dx - secs) * sec2nsec);
                Cdmw::OsSupport::OS::sleep((unsigned int) secs, (unsigned int) nsecs);
                curr_executor_rate = (dy / dx);
                in_pause = false;
            }
        }
    }
    catch (InvalidExecutorInitialization&)
    {
        throw;
    }
    catch (...)
    {
        // An exception is unexpected.
        std::cerr << "Unexpected exception: engine halted." << std::endl;
    }
}

