#!/bin/bash

# ----- Function get_remote_env
# Parameters: $1 : remote host name.
#             $2 : environment variable name.
# Return: the value of environment variable $2 on remote host $1.
function get_remote_env()
{
    echo `ssh $1 env | grep "^$2=" | cut -d'=' -f2`
}

# ----- Function get_remote_PIDs
# Parameters: $1 : remote host name
#             $2 : remote user name
#             $3 : full path to remote executable
function get_remote_PIDs()
{
    echo `ssh $1 ps -ef --cols 1024 | grep -w $2 | grep $3 | grep -v grep | tr -s ' ' | cut -d' ' -f2`
}

# ----- Handle inpur parameters
SCRIPT=`basename $0`
if [[ $# -eq 0 ]] ; then
    echo "usage: $SCRIPT <test session> [<test session> ...]"
    echo "where: <test session> is one of the following test session values:"
    echo "       - valgrind-3.1.0.memo-check for the execution of valgrind-3.1.0 with memory check"
    echo "       - valgrind-2.4.1.memo-check for the execution of valgrind-2.4.1 with memory check"
    echo "       - valgrind-2.4.1.race-check for the execution of valgrind-2.4.1 with race condition check"
    echo "       - simple-test               for the execution of simple tests"
    echo "notes: required local environment variables are:"
    echo "       - TESTS_REPORTS_SCRIPTS_ROOT pointing to the local absolute path for test reports logs."
    echo "       - TESTS_REPORTS_LOGS_ROOT    pointing to the local absolute path for test reports logs."
    exit 1
fi
SESSIONS="$*"
for session in $SESSIONS ; do
    if [[ ("$session" != "valgrind-3.1.0.memo-check") && ("$session" != "valgrind-2.4.1.memo-check") &&
          ("$session" != "valgrind-2.4.1.race-check") && ("$session" != "simple-test") ]] ; then
        echo "$SCRIPT: invalid session test specification -- $session: aborting"
        exit 1
    fi
done

# ----- Include global macros
source $TESTS_REPORTS_SCRIPTS_ROOT/tests_reports-global_macros.bash

# ----- Global variables for process management
TESTS_PROGRAMS="testcancelallpolicy testclockservice testcontrolledexecutor testenforcedeadlinepolicy testenforceintervalpolicy test_Q_100 test_Q_101"
TESTS_REPORTS_REMOTE_SERVER_SCRIPTS_ROOT=`get_remote_env $TMG_SERVER TESTS_REPORTS_SCRIPTS_ROOT`
TESTS_REPORTS_REMOTE_CLIENT_SCRIPTS_ROOT=`get_remote_env $TMG_CLIENT TESTS_REPORTS_SCRIPTS_ROOT`
VALGRIND_3_1_0_REMOTE_SERVER_BIN_ROOT=`get_remote_env $TMG_SERVER VALGRIND_3_1_0_BIN_ROOT`
VALGRIND_3_1_0_REMOTE_CLIENT_BIN_ROOT=`get_remote_env $TMG_CLIENT VALGRIND_3_1_0_BIN_ROOT`
VALGRIND_2_4_1_REMOTE_SERVER_BIN_ROOT=`get_remote_env $TMG_SERVER VALGRIND_2_4_1_BIN_ROOT`
VALGRIND_2_4_1_REMOTE_CLIENT_BIN_ROOT=`get_remote_env $TMG_CLIENT VALGRIND_2_4_1_BIN_ROOT`
SIMPLE_TEST_REMOTE_SERVER_INSTALL_ROOT=`get_remote_env $TMG_SERVER CDMW_HOME`
SIMPLE_TEST_REMOTE_CLIENT_INSTALL_ROOT=`get_remote_env $TMG_CLIENT CDMW_HOME`
REMOTE_SERVER_PIDS= # gets updated later on
REMOTE_CLIENT_PIDS= # gets updated later on
LOCAL_TERM_PIDS=    # gets updated later on

# ----- Preliminary check for missing valgrind-3.1.0 and valgrind-2.4.1 client and/or server processes
FOUND_PROCESSES=NO
# Check for missing valgrind-3.1.0 server processes
if [[ "`get_remote_PIDs $TMG_SERVER $USER $VALGRIND_3_1_0_REMOTE_SERVER_BIN_ROOT/valgrind`" != "" ]] ; then
    echo "$SCRIPT: valgrind-3.1.0 server processes already running on host $TMG_SERVER"
    FOUND_PROCESSES=YES
fi
# Check for missing valgrind-3.1.0 client processes
if [[ "`get_remote_PIDs $TMG_CLIENT $USER $VALGRIND_3_1_0_REMOTE_CLIENT_BIN_ROOT/valgrind`" != "" ]] ; then
    echo "$SCRIPT: valgrind-3.1.0 client processes already running on host $TMG_CLIENT"
    FOUND_PROCESSES=YES
fi
# Check for missing valgrind-2.4.1 server processes
if [[ "`get_remote_PIDs $TMG_SERVER $USER $VALGRIND_2_4_1_REMOTE_SERVER_BIN_ROOT/valgrind`" != "" ]] ; then
    echo "$SCRIPT: valgrind-2.4.1 server processes already running on host $TMG_SERVER"
    FOUND_PROCESSES=YES
fi
# Check for missing valgrind-2.4.1 client processes
if [[ "`get_remote_PIDs $TMG_CLIENT $USER $VALGRIND_2_4_1_REMOTE_CLIENT_BIN_ROOT/valgrind`" != "" ]] ; then
    echo "$SCRIPT: valgrind-2.4.1 client processes already running on host $TMG_CLIENT"
    FOUND_PROCESSES=YES
fi
# Check for missing simple-tests server processes
for test in $TEST_PROGRAMS ; do
    if [[ "`get_remote_PIDs $TMG_SERVER $USER $SIMPLE_TEST_REMOTE_SERVER_INSTALL_ROOT/bin/$test`" != "" ]] ; then
        echo "$SCRIPT: simple tests server processes already running on host $TMG_SERVER"
        FOUND_PROCESSES=YES
    fi
done
# Check for missing simple-tests client processes
for test in $TEST_PROGRAMS ; do
    if [[ "`get_remote_PIDs $TMG_CLIENT $USER $SIMPLE_TEST_REMOTE_CLIENT_INSTALL_ROOT/bin/$test`" != "" ]] ; then
        echo "$SCRIPT: simple tests client processes already running on host $TMG_CLIENT"
        FOUND_PROCESSES=YES
    fi
done
# Check for missing xterm processes on local host
if [[ "`get_remote_PIDs localhost $USER $TERM`" != "" ]] ; then
    echo "$SCRIPT: $TERM processes already running on local host"
    FOUND_PROCESSES=YES
fi
if [[ "$FOUND_PROCESSES" == "YES" ]] ; then
    echo "$SCRIPT: found already running processes: aborting"
    exit 1
fi

# ---------- For each specified session, do logs gathering
for session in $SESSIONS ; do
    echo ">>>>> Gather log files for test session $session"


    # ----- Set global variables indicating the test session kind
    if [[ "$session" == "valgrind-3.1.0.memo-check" ]] ; then
        CLIENT_PROCESS_TEST_KIND="valgrind-3.1.0 memo-check client-process"
        SERVER_PROCESS_TEST_KIND="valgrind-3.1.0 memo-check server-process"
        SINGLE_PROCESS_TEST_KIND="valgrind-3.1.0 memo-check single-process"
    fi
    if [[ "$session" == "valgrind-2.4.1.memo-check" ]] ; then
        CLIENT_PROCESS_TEST_KIND="valgrind-2.4.1 memo-check client-process"
        SERVER_PROCESS_TEST_KIND="valgrind-2.4.1 memo-check server-process"
        SINGLE_PROCESS_TEST_KIND="valgrind-2.4.1 memo-check single-process"
    fi
    if [[ "$session" == "valgrind-2.4.1.race-check" ]] ; then
        CLIENT_PROCESS_TEST_KIND="valgrind-2.4.1 race-check client-process"
        SERVER_PROCESS_TEST_KIND="valgrind-2.4.1 race-check server-process"
        SINGLE_PROCESS_TEST_KIND="valgrind-2.4.1 race-check single-process"
    fi
    if [[ "$session" == "simple-test" ]] ; then
        CLIENT_PROCESS_TEST_KIND="simple-test client-process"
        SERVER_PROCESS_TEST_KIND="simple-test server-process"
        SINGLE_PROCESS_TEST_KIND="simple-test single-process"
    fi

    # ----- Log all tests in single-process mode
    echo ">>>>>>>>>> Log all tests in single-process mode ($SINGLE_PROCESS_TEST_KIND)"
    for test in $TESTS_PROGRAMS ; do
        echo ">>>>>>>>>>>>>>> TEST: $test: launching test in single-process mode ($SINGLE_PROCESS_TEST_KIND) on host $TMG_SERVER ..."
        $TERM -ls -sb -sl 1000 -geometry 120X20+000+000 -display $DISPLAY -title "$USER@$TMG_SERVER: $SINGLE_PROCESS_TEST_KIND test-$test" -e ssh $TMG_SERVER $TESTS_REPORTS_REMOTE_SERVER_SCRIPTS_ROOT/$session.single-process.bash $test
        if [[ $? -ne 0 ]] ; then
            echo "$SCRIPT: could not complete operation: aborting"
            exit 1
        fi
        sleep 10
    done

    # ----- Log all tests in client-server mode
    echo ">>>>>>>>>> Log all tests in client-server mode ($CLIENT_PROCESS_TEST_KIND and $SERVER_PROCESS_TEST_KIND)"
    for test in $TESTS_PROGRAMS ; do
        echo ">>>>>>>>>>>>>>> TEST: $test: launching test in server-process mode ($SERVER_PROCESS_TEST_KIND) on host $TMG_SERVER ..."
        $TERM -ls -sb -sl 1000 -geometry 120X20+000+000 -display $DISPLAY -title "$USER@$TMG_SERVER: $SERVER_PROCESS_TEST_KIND test-$test" -e ssh $TMG_SERVER $TESTS_REPORTS_REMOTE_SERVER_SCRIPTS_ROOT/$session.server-process.bash $test &
        sleep 10
        if [[ "$session" == "valgrind-3.1.0.memo-check" ]] ; then
            REMOTE_SERVER_PIDS="`get_remote_PIDs $TMG_SERVER $USER $VALGRIND_3_1_0_REMOTE_SERVER_BIN_ROOT/valgrind`"
        fi
        if [[ "$session" == "valgrind-2.4.1.memo-check" ]] ; then
            REMOTE_SERVER_PIDS="`get_remote_PIDs $TMG_SERVER $USER $VALGRIND_2_4_1_REMOTE_SERVER_BIN_ROOT/valgrind`"
        fi
        if [[ "$session" == "valgrind-2.4.1.race-check" ]] ; then
            REMOTE_SERVER_PIDS="`get_remote_PIDs $TMG_SERVER $USER $VALGRIND_2_4_1_REMOTE_SERVER_BIN_ROOT/valgrind`"
        fi
        if [[ "$session" == "simple-test" ]] ; then
            REMOTE_SERVER_PIDS="`get_remote_PIDs $TMG_SERVER $USER $SIMPLE_TEST_REMOTE_SERVER_INSTALL_ROOT/bin/$test`"
        fi
        if [[ `echo $REMOTE_SERVER_PIDS | wc -w` -ne 1 ]] ; then
            echo "$SCRIPT: could not complete operation: aborting"
            exit 1
        fi
        echo ">>>>>>>>>>>>>>> TEST: $test: launching test in client-process mode ($CLIENT_PROCESS_TEST_KIND) on host $TMG_CLIENT ..."
        $TERM -ls -sb -sl 1000 -geometry 120X20+000+290 -display $DISPLAY -title "$USER@$TMG_CLIENT: $CLIENT_PROCESS_TEST_KIND test-$test" -e ssh $TMG_CLIENT $TESTS_REPORTS_REMOTE_CLIENT_SCRIPTS_ROOT/$session.client-process.bash $test
        if [[ $? -ne 0 ]] ; then
            echo "$SCRIPT: could not complete operation: aborting"
            ssh $TMG_SERVER kill $REMOTE_SERVER_PIDS # kill server process
            exit 1
        fi
        echo ">>>>>>>>>>>>>>> TEST: $test: killing server process on host $TMG_SERVER ..."
        ssh $TMG_SERVER kill $REMOTE_SERVER_PIDS # kill server process
        if [[ $? -ne 0 ]] ; then
            echo "$SCRIPT: could not complete operation: aborting"
            exit 1
        fi
        sleep 10
    done

done

# ----- Copy all log and out files from client and server to local host
echo ">>>>> Copy log and output files from client and server to local host"
if [[ "$HOSTNAME" != "$TMG_SERVER" ]] ; then
    /usr/bin/scp $USER@$TMG_SERVER:$TESTS_REPORTS_LOGS_ROOT/* $TESTS_REPORTS_LOGS_ROOT
    if [[ $? -ne 0 ]] ; then
        echo "$SCRIPT: could not complete operation: aborting"
        exit 1
    fi
fi
if [[ "$HOSTNAME" != "$TMG_CLIENT" ]] ; then
    /usr/bin/scp $USER@$TMG_CLIENT:$TESTS_REPORTS_LOGS_ROOT/* $TESTS_REPORTS_LOGS_ROOT
    if [[ $? -ne 0 ]] ; then
        echo "$SCRIPT: could not complete operation: aborting"
        exit 1
    fi
fi

# ----- End logging script
echo ">>>>> End of log file registration!"
exit 0

