/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI.
 * Copyright (C) SELEX SISTEMI INTEGRATI 2004-2009. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

#include <string>
#include <iostream>
#include <stdio.h>
#include "Time/idllib/CosClockService.stub.hpp"
#include "Time/clockservice/Util.hpp"
#include "Time/clockservice/Time.hpp"
#include <cppunit/extensions/HelperMacros.h>
#include <sys/time.h>
#include "date_timeval/TestClass.hpp"
#include "Foundation/testutils/CORBATestManager.hpp"

using namespace CosClockService;
using namespace CosPropertyService;
CPPUNIT_TEST_SUITE_REGISTRATION(TestClass);
CPPUNIT_TEST_SUITE_NAMED_REGISTRATION(TestClass, "testpack_unpacktimeval");

TestClass::TestClass()
{
}

// ==================================================================================
    /**
     * @brief convert a string representation of time to a time cdme_tm structure
     * @param date   string which contains the date
     * @param format character string that consists of field descriptors and text characters
     * @param tm     structure where will be stored the date
     */


// ==================================================================================
void 
TestClass::string2cdmw_tm(std::string date,const char* format, cdmw_tm& t) 
{
   char monthstr[3];
   char week_day[3];
   char * months[] = {"Jan", "Feb", "Mar", "Apr", "May", "Jun", "Jul", "Aug", "Sep", "Oct", "Nov", "Dec" };
   char* day_week [] = { "Sun", "Mon", "Tue", "Wed", "Thu", "Fri", "Sat" };

   sscanf(date.c_str(),format,
             week_day,
             monthstr,
             &t.tm_mday,
             &t.tm_hour,
             &t.tm_min,
             &t.tm_sec,
             &t.tm_msec,
             &t.tm_usec,
             &t.tm_year);

   t.tm_year = t.tm_year -1900;
   for (int month = 0; month < 12; month++) {
      if (strncasecmp(monthstr, months[month], 3) == 0) {
         t.tm_mon = month;
         break; }
   }
   for (int day = 0; day < 7; day++) {
      if (strncasecmp(week_day, day_week[day], 3) == 0) {
         t.tm_wday = day;
         break; }
   }

}


void
TestClass::setUp()
{
    try
    {
        Cdmw::TestUtils::CORBATestManager::instance ()->add ("ClockService");
        m_obj = Cdmw::TestUtils::CORBATestManager::instance ()->get_object ("ClockService");
        ClockCatalog_var m_clockcatalog = CosClockService::ClockCatalog::_narrow(m_obj.in());
        CPPUNIT_ASSERT(!CORBA::is_nil(m_clockcatalog.in()));

        ClockCatalog::ClockEntry_var entry = m_clockcatalog->get_entry("FederatedControlledExecutor");
        m_controlledClock = ControlledClock::_narrow(entry->subject.in());
        CPPUNIT_ASSERT(!CORBA::is_nil(m_controlledClock.in()));
    }
    catch (Cdmw::TestUtils::CORBATestManager::InvalidObject &)
    {
        CPPUNIT_FAIL ("Cdmw::TestUtils::CORBATestManager::InvalidObject");
    }
    catch ( ... )
    {
        CPPUNIT_FAIL ("Unknown exception!");
    }

}

///////////////////////////////////////////////////////////////////////////////

void
TestClass::tearDown()
{
}

///////////////////////////////////////////////////////////////////////////////

void
TestClass::testpacktime()
{
   struct cdmw_tm tm;
   struct timeval tv;
   TimeBase::TimeT t_time,current_time;
   char input_date[256];
   char check_date[256];
   std::string date;
   try 
   {
      current_time = m_controlledClock->current_time();
      CPPUNIT_ASSERT( Cdmw::clock::util::TimeT2timeval(current_time, tv)  == 0 );
      Cdmw::clock::util::asctime_r(tv, (char*)input_date);
      //std::cerr << "\ncurrent date (Greenwich Time): " << input_date  << std::endl;
      string2cdmw_tm(input_date,"%3s %3s %2d %02d:%02d:%02d,%03d,%03d %d\n",tm);

      // pack a time value from date(cdmw_tm)
      // 1) cdmw_tm to TimeT
      CPPUNIT_ASSERT(Cdmw::clock::util::cdmw_tm2TimeT(tm, t_time) == 0 );
      // 2) TimeT to time_val
      CPPUNIT_ASSERT ( Cdmw::clock::util::TimeT2timeval(t_time, tv)  == 0 );

      // check operations
      Cdmw::clock::util::asctime_r(tv, (char*)check_date);
      //std::cerr << "date corresponding to timeval obtained from pack operation: " << check_date  << std::endl;

      CPPUNIT_ASSERT (strcmp (check_date,input_date) == 0);
   } 
   catch ( ... )
   {
        CPPUNIT_FAIL ("Unknown exception!");
   }
}

   /**
    * @brief Given a time val corresponding to a date
    *  the test checks that unpacked value obtained using
    *  TMG operations is correct
    *  (timeval to cdmw_tm)
    */

void
TestClass::testunpacktime()
{
    /*
        Given a time val corresponding to a date
        the test checks that unpacked value obtained using
        TMG operations is correct
        (timeval to cdmw_tm)
    */


    char current_date[256];
    char comp_str[256];
    struct cdmw_tm tm;
    struct timeval tv;
    TimeBase::TimeT out_time;

    Cdmw::clock::util::TimeT2timeval(m_controlledClock->current_time(), tv);
    Cdmw::clock::util::asctime_r(tv, current_date);
    std::cout << "\ncurrent date (Greenwich Time): " << current_date << std::endl;

    std::cout << "Check of unpacking operations for following time value: "  << std::endl;
    std::cout << "tv_sec     = "  << tv.tv_sec << std::endl;
    std::cout << "tv_usec    = "  << tv.tv_usec << std::endl;
    std::cout << "corresponding to date: " << current_date << std::endl;

    // UNPACKING :
    // 1) convert timeval to TimeT
    Cdmw::clock::util::timeval2TimeT(tv, out_time);
    // 2) convert TimeT to cdmw_tm
    Cdmw::clock::util::TimeT2cdmw_tm(out_time,tm);
    Cdmw::clock::util::asctime_r(tm, comp_str);
    CPPUNIT_ASSERT ( strcmp(comp_str, current_date) == 0) ;
}

void TestClass::testpackerror ()
{
    struct cdmw_tm tm;
    struct timeval tv_1,tv_2;
    TimeBase::TimeT t_time;
    char date1[256];
    char date2[256];
    char date0[256];

    // set tm to date : Thu Jan 1 1580 0h: 0m: 0s

    std::memset(&tm, 0, sizeof(cdmw_tm));

    tm.tm_year =  -320; // 1900 - 320 = year 1580
    tm.tm_usec =  0;
    tm.tm_msec =  0;
    tm.tm_sec  =  0;
    tm.tm_min  =  0;
    tm.tm_hour =  0;
    tm.tm_mday =  1;
    tm.tm_mon  =  0;
    tm.tm_wday =  0;

//-----------------------------------------------------------------------------

    int res =  Cdmw::clock::util::cdmw_tm2TimeT(tm, t_time);
    Cdmw::clock::util::asctime_r(tm, (char*)date0);
    std::cout << "\n\nCase Error 1 : input date for pack operation : " << date0 << std::endl;
 
    if (res != 0)
    {
        std::cout << "\n=================== ERROR 1:  out of range of TimeT ========" << std::endl;
        std::cout << " Clock Service cannot manage date  before 15-OCT-1582 " << std::endl;
        std::cout <<   "============================================================" << std::endl;
    }
    CPPUNIT_ASSERT(res != 0);

//-----------------------------------------------------------------------------

    tm.tm_year =  10; // 1910
    Cdmw::clock::util::asctime_r(tm, (char*)date0);
    std::cout << "\n\nCase Error 2 : input date for pack operation : " << date0 << std::endl;
    // ------------ pack time -----------------------------------------
    Cdmw::clock::util::cdmw_tm2TimeT(tm, t_time);
    res = Cdmw::clock::util::TimeT2timeval(t_time, tv_1);
    // ----------------------------------------------------------------
    if (res != 0) 
    {
        std::cout << "\n=============== ERROR 2: input date before 1970 =============" << std::endl;
        std::cout << " The operation for packing and unpacking should be\n used for dates following 1st January 1970" << std::endl;
        std::cout <<  "===============================================================" << std::endl;
    }
    CPPUNIT_ASSERT(res != 0);

//-----------------------------------------------------------------------------

    tm.tm_year =  95;
    Cdmw::clock::util::asctime_r(tm, (char*)date0);
    // ------------ pack time -----------------------------------------
    CPPUNIT_ASSERT(Cdmw::clock::util::cdmw_tm2TimeT(tm, t_time) == 0 );
    CPPUNIT_ASSERT(Cdmw::clock::util::TimeT2timeval(t_time, tv_1) == 0);
    // ----------------------------------------------------------------
    Cdmw::clock::util::asctime_r(tv_1, (char*)date1);
    Cdmw::clock::util::TimeT2timeval(117023957673654460ULL, tv_2);
    Cdmw::clock::util::asctime_r(tv_2, (char*)date2);
    std::cout << "\n\nCase Error 3 : input date for pack operation : " << date0 ;
    std::cout << "compared with : " << date2 << std::endl;

    if ( tv_1 != tv_2 ) 
    {
        std::cout << "======== ERROR 3: pack time return unexpected result  ========" << std::endl;
        std::cout << "You are trying to compare different timevalue"  
                  << "\nFrom packing time operation results that input date correspond to " << date1 
                  << "and you are trying to compare with date " << date2 
                  << "please check the value of input date " << std::endl;
        std::cout << "===============================================================" << std::endl;
    }
    CPPUNIT_ASSERT ( tv_1 != tv_2 );

//-----------------------------------------------------------------------------

    // boundary val (correct)
    tm.tm_year =  70; // 1970
    Cdmw::clock::util::asctime_r(tm, (char*)date0);
    std::cout << "\n\nCase 4 : correct boundary value date for pack operation: " << date0;
    // ------------ pack time -----------------------------------------
    Cdmw::clock::util::cdmw_tm2TimeT(tm, t_time);
    res = Cdmw::clock::util::TimeT2timeval(t_time, tv_1);
    // ----------------------------------------------------------------
    std::cout << (res == 0 ? "OK" : "ERROR") << std::endl;
    CPPUNIT_ASSERT(res == 0);

//-----------------------------------------------------------------------------

// boundary val (incorrect)
//     tm.tm_year =  69;
//     tm.tm_usec =  0;
//     tm.tm_msec =  999;
//     tm.tm_sec  =  59;
//     tm.tm_min  =  59;
//     tm.tm_hour =  23 (also try 29)
//     tm.tm_mday =  31;
//     tm.tm_mon  =  12;
//     tm.tm_wday =  3;

    // boundary val (incorrect)
    tm.tm_year =  69;
    tm.tm_usec =  0;
    tm.tm_msec =  999;
    tm.tm_sec  =  59;
    tm.tm_min  =  59;
    tm.tm_hour =  23;
    tm.tm_mday =  31;
    tm.tm_mon  =  11;
    tm.tm_wday =  3;
    
    Cdmw::clock::util::asctime_r(tm, (char*)date0);
    std::cout << "\n\nCase Error 5 : incorrect boundary value date for unpack operation : " << date0;
    // ------------ pack time -----------------------------------------
    Cdmw::clock::util::cdmw_tm2TimeT(tm, t_time);
    res = Cdmw::clock::util::TimeT2timeval(t_time, tv_1);
    // ----------------------------------------------------------------
    if (res != 0) 
    {
        std::cout << "======== ERROR 5: unpack time return unexpected result  ========" << std::endl;
        std::cout << "ClockService unpack time routine cannot manage date  before 1st January 1970" << std::endl;
        std::cout << "===============================================================" << std::endl;
    }
    CPPUNIT_ASSERT(res != 0);
}
