/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */
#ifndef CDMW_RECORDING_OBJECTMAP_HPP
#define CDMW_RECORDING_OBJECTMAP_HPP

#include <string.h>

#ifdef __GNUC__
#include <ext/hash_map>
#else
#include <map>
#endif

#include <Foundation/orbsupport/CORBA.hpp>
#include <TraceAndPerf/recordinglibrary/NetworkSniffer.hpp>
#include <TraceAndPerf/recordinglibrary/PacketStore.hpp>
#include <TraceAndPerf/recordinglibrary/InterfaceFilter.hpp>

namespace Cdmw
{
   namespace Recording
   {
      /**
       *Purpose: Store the objects registered by the containers for
       * recording purpose. Provides the methods for interface filtering
       * ObjectMap class is not thread-safe
       * (it is up to the user to provide protection for concurrent updates)
       *<p> .
       *
       */
      class ObjectMap
      {
         public :
            typedef void (*Display)(string const &, int, int);

            ObjectMap (InterfaceFilter & filter, NetworkSniffer & sniffer)
                  throw();

            virtual ~ObjectMap ()
                  throw() ;


            /**
             *Purpose: For each reference stored, check if its
             * interface matches one of the filtered ones 
             *<p> .
             *
             */
            virtual bool checkInterfaces()
                  throw() ;

            /**
             *Purpose: Adds an object to the map
             *<p> .
             *
             */
            virtual void addObject (const char * server_id, CORBA::ORB * orb, CORBA::Object_var & ref) 
                  throw() ;

            /**
             *Purpose: Removes an object from the map
             *<p> .
             *
             */
            virtual void removeObject (CORBA::ORB * orb, CORBA::Object_var & ref) 
                  throw() ;

            /**
             *Purpose: checks if an object key must have its operations recorded
             *<p> .
             *
             */
            virtual bool checkKey (Message * msg) 
                  throw() ;

            /**
             *Purpose: return the size of the map (the number of registered objects)
             *<p> .
             *
             */
            virtual int size () 
                  throw() ;

            /**
             *Purpose:
             *<p> map iterator.
             *
            */
            virtual void ObjectMap::mapLoop (Display print)
              throw() ;

         private :
            bool
            explode_operation_line(const std::string& line, std::string & opname, bool & flag);
            /**
             * Body 
             */

            InterfaceFilter & m_intf_filter ;
            NetworkSniffer & m_sniffer ;

            struct OctetArray
            {
                  int length ;
                  octet * bytes ;
            } ;

            // Indicates if a repositoryId is to be recorded
            struct RIdInfo
            {
                  RIdInfo () : toRecord(true) {}
                  bool toRecord ;
            } ;

            struct ObjectKeyInfo
            {
                  ObjectKeyInfo () : server_id(0), rep_id(0), info(0) {}

                  char * server_id ;
                  char * rep_id ;
                  RIdInfo * info ;
            } ;


#ifdef __GNUC__
            // For performance reasons, it is highly preferable to use
            // a hash_map instead of a map. As it may not be available
            // on all environments, a map alternative is added.
            struct eqstr
            {
               bool operator()(const std::string s1, const std::string s2) const
               {
                  return s1 == s2;
               }
            };

            static inline std::size_t hash_string (const char * str)
            {
                    unsigned long h = 0;
                    for (const char * each = str ; *each != 0; each++)
                    {
                            h = 5*h + *each;
                    }
                    return size_t(h);
            }

            struct hash_std_string
            {
                    std::size_t operator()(const std::string str) const
                                    { return hash_string(str.c_str()); }
            };

            typedef __gnu_cxx::hash_map <std::string, RIdInfo *, hash_std_string, eqstr> RIdMap ;

            static inline std::size_t hash_octets (const OctetArray octets)
            {
               unsigned long h = 0;
               for (int each = octets.length - 1; each >= 0; each--)
               {
                  h = 5*h + octets.bytes[each];
               }
               return size_t(h);
            }

            struct hash
            {
               std::size_t operator()(const OctetArray octets) const
                     { return hash_octets(octets); }
            };

            struct eqoctets
            {
               bool operator()(const OctetArray s1, const OctetArray s2) const
               {
                  if (s1.length != s2.length)
                  {
                     return false ;
                  }

                  for (int each = s1.length - 1; each >= 0; each--)
                  {
                     if (s1.bytes[each] != s2.bytes[each])
                     {
                        return false ;
                     }
                  }

                  return true;
               }
            };

            typedef __gnu_cxx::hash_map <const OctetArray, ObjectKeyInfo, hash, eqoctets> KeyMap ;
#else
#warning "hash_map may-be not available, check STL extensions"
            struct ltstr
            {
               bool operator()(const char* s1, const char* s2) const
               {
                  return strcmp(s1, s2) < 0;
               }
            };

        // CDO-070330-PRS-1-567 : #62
        //typedef std::map <char*, RIdInfo *, ltstr> RIdMap ;
        typedef std::map <std::string, RIdInfo *, ltstr> RIdMap ;

            struct ltoctets
            {
               bool operator()(const OctetArray s1, const OctetArray s2) const
               {
                  if (s1.length < s2.length)
                  {
                     return true ;
                  }

                  if (s1.length > s2.length)
                  {
                     return false ;
                  }

                  for (int each = s1.length - 1; each >= 0; each--)
                  {
                     if (s1.bytes[each] < s2.bytes[each])
                     {
                        return true ;
                     }

                     if (s1.bytes[each] > s2.bytes[each])
                     {
                        return false ;
                     }
                  }

                  return false;
               }
            };

            typedef std::map <const OctetArray, RIdInfo *, ltoctets> KeyMap ;
#endif

            // This map is indexed by the repositoryId 
            RIdMap m_objectRIds ;

            // This map is indexed by the object key 
            KeyMap m_keys ;
      } ;
   }
}

#endif
