/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

#ifndef CDMW_RECORDING_PACKETSTORE_HPP
#define CDMW_RECORDING_PACKETSTORE_HPP

#include <netinet/in.h>
#include <Foundation/osthreads/Mutex.hpp>
#include <Foundation/osthreads/MutexGuard.hpp>
#include <Foundation/osthreads/Condition.hpp>

namespace Cdmw
{
   namespace Recording
   {

      typedef int Milliseconds ;
      /**
       *Purpose: Time to wait (in milliseconds) before declaring a request orphan
       *<p> .
       *
       */
      unsigned long get_Orphan_Request_Timeout() ;

      /**
       *Purpose: Describes a connection
       *<p> .
       *
       */
      struct Connection
      {
         in_addr_t src_addr ;
         in_addr_t dst_addr ;
         unsigned short src_port ;
         unsigned short dst_port ;
      };

      inline bool operator==(Connection left, Connection right)
      {
         return  (left.src_addr == right.src_addr) &&
         (left.dst_addr == right.dst_addr) &&
         (left.src_port == right.src_port) &&
         (left.dst_port == right.dst_port) ;
      }


      /**
       *Purpose: Tells the packet direction
       *<p> .
       *
       */
      enum PacketDirection { Incoming, Outgoing } ;
      typedef unsigned char octet ;

      /**
       *Purpose: determine if the endian byte indicates the need to do
       * byte swapping
       *<p> .
       *
       */
      bool needSwap (const octet endian) ;

      /**
       *Purpose: describes a network packet
       *<p> .
       *
       */
      struct Packet
      {
         Connection channel ;
         PacketDirection direction ;
         timeval date ;
         int length ;
         const octet * data ;
      };

      struct Version {
         octet major;
         octet minor;
      };

      struct GIOP_Header
      {
         char magic[4] ;
         Version version ;
         octet flags ;
         octet message_type;
         int length ;
      };

      /**
       *Purpose: describes a GIOP message
       *<p> .
       *
       */
      struct Message
      {
         Message * next ; // for chaining purposes
         Message * fragments ; // for chaining purposes
         Connection channel ;
         GIOP_Header header ;
         timeval date ;
         bool need_swap ;
         octet * data ;
         unsigned int request_id ;
         int objectKey_Offset ;
         int objectKey_Length ;
         int operation_Offset ;
         const char * server_id ;
         const char * intf_rid ;
         bool has_missing_bytes;

         size_t GiopSize () const;
      };


      /**
       *Purpose: Stores the packets and builds GIOP messages from them
       *<p> .
       *
       */
      class PacketStore
      {
         char magic[4] ;
         public :
            PacketStore ()
                  throw();

            virtual ~PacketStore ()
                  throw() ;

            /**
             *Purpose: Store a packet and tries to build a GIOP message
             * from it
             *<p> .
             *
             */
            virtual void pushPacket (Packet & packet)
                  throw() ;

            /**
             *Purpose: returns the next available GIOP message
             *<p> .
             *
             */
            virtual Message * popMessage ()
                  throw() ;

            /**
             *Purpose: returns a used message to the free list
             *<p> .
             *
             */
             virtual void  PacketStore::stop () throw() ;

             /**
             *Purpose: returns a used message to the free list
             *<p> .
             *
             */
            virtual void freeMessage (Message *& message)
                  throw() ;

            /**
             *Purpose: Clean the packet storage area and unlock popMessage
             *<p> .
             *
             */
            virtual void finalize ()
                  throw() ;

            /**
             *Purpose: Allow to reuse the store after a finalize()
             *<p> .
             *
             */
            virtual void reset ()
                  throw() ;

         private :
            /**
             * Body
             */

           struct ConnectionDescriptor
           {
             ConnectionDescriptor * next ; // for chaining purposes
             Connection channel ;
             Message * pending ;
             int bytes_got ;
           } ;

            virtual void deleteMessage(Message *& message) ;

            virtual Message * appendFragment(ConnectionDescriptor * desc, bool remove) ;

            /**
             * Purpose: push msg to the m_messages queue
             */
            void  pushMessage(Message* msg);

            OsSupport::Mutex m_mutex ;
            OsSupport::Condition m_flowControl ;

            ConnectionDescriptor * m_connections ;

            Message * m_messages_head ;
            Message * m_messages_tail ;
            Message * m_fragmented_head ;
            Message * m_fragmented_tail ;
            Message * m_freeList ;
            bool m_stopped ;
            bool m_finalized ;
     } ;

   }
}

#endif
