/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

#include <Foundation/logging/LogManager.hpp>
#include <Foundation/logging/Logger.hpp>
#include <Foundation/logging/FunctionLogger.hpp>
#include "Foundation/orbsupport/OrbSupport.hpp"
#include "Foundation/ossupport/OS.hpp"
#include "Foundation/osthreads/Thread.hpp"
#include "TraceAndPerf/recordinglibrary/Recording_Default_Parameters.hpp"
#include "TraceAndPerf/recordinglibrary/BufferSender.hpp"
#include <Foundation/orbsupport/CORBAMessaging.hpp>

namespace Cdmw
{
    namespace Recording
    {
        Cdmw::Logging::Logger* buffersender_logger;
        class BufferSender::Worker : public virtual OsSupport::Thread
        {

            void set_timeout_policy_on(CORBA::ORB_ptr orb, CdmwRecording::Recorder_ptr& obj,
                                       unsigned long m_callTimeout_millisec) {
                // Create the timeout policy
                CORBA::Any timeout;
                CORBA::Object_var capsule;
                
                // Timeout success 
                //Set the timeout value as a TimeBase::TimeT (100 nanoseconds units)
                TimeBase::TimeT _timeout = ((unsigned long) (m_callTimeout_millisec * 1.0e4));
                CORBA::PolicyList policy_list;

                timeout <<= _timeout;
                // Set the policy
                policy_list.length( 2 );
                policy_list[0] = orb->create_policy( Messaging::RELATIVE_RT_TIMEOUT_POLICY_TYPE,
                                                     timeout );

                // SYNC_WITH_TARGET makes the call the record() operation synchronous, this enables FT
                // and prevents records from being lost   
                Messaging::SyncScope sync_scope = Messaging::SYNC_WITH_TARGET;
                CORBA::Any sync_scope_any;
                sync_scope_any <<= sync_scope;
                policy_list[1] = m_orb->create_policy (Messaging::SYNC_SCOPE_POLICY_TYPE,
                                                       sync_scope_any);

                capsule = obj->_set_policy_overrides( policy_list, CORBA::SET_OVERRIDE );

                // release inout value before to re-affect
                CORBA::release(obj);
                obj = CdmwRecording::Recorder::_unchecked_narrow(capsule.in());
            }

        public :
            Worker (CoalescenceManager * manager, const char * url, OsSupport::Mutex & mutex,CORBA::ORB_ptr orb) throw()
                : OsSupport::Thread()
                , m_polling(DEFAULT_POLLING_PERIOD_MILLISEC)
                , m_num_buffers(0)
                , m_max_buffers(DEFAULT_NUMBER_MAX_OF_BUFFERS)
                , m_num_retries(0)
                , m_max_retries(-1) // -1 : infinite
                , m_mutex(mutex)
                , m_control(mutex)
                , m_manager(manager)
                , m_recorder_url(url)
                , m_orb(CORBA::ORB::_duplicate(orb))
                , m_recorder()
                , m_buffers_head(0)
                , m_buffers_tail(0)
                , m_terminated(false)
                , m_id(0)
                , m_corba_timeout_ms(DEFAULT_CORBA_TIMEOUT_MILLISEC) {
                if (url != 0)
                {
                    try
                    {
                        CORBA::Object_var obj = m_orb->string_to_object(url) ;
                        if (!CORBA::is_nil(obj.in()))
                        {
                            m_recorder = CdmwRecording::Recorder::_unchecked_narrow (obj.in()) ;

                            if (!CORBA::is_nil(m_recorder.in())) // JM 12/09/2007
                            {
                                set_timeout_policy_on (m_orb.in(), m_recorder.inout(), m_corba_timeout_ms);
                                CDMW_LOG(buffersender_logger, 
                                         MSG_BUFFERSENDER_LOG_LEVEL, 
                                         "=== BufferSender.Worker(0) : Recording server available");
                            }
                        }
                    }
                    catch (...)
                    {
                        CDMW_LOG(buffersender_logger, 
                                 MSG_BUFFERSENDER_LOG_LEVEL, 
                                 "=== BufferSender.Worker(1) : Recording server not available");
                    }                 
                }
                m_id = start();
            }

            virtual ~Worker() throw() {
                terminate () ;
            }

            /**
             *Purpose: Set the recorder reference
             *<p> .
             *
             */
            virtual void setRecorder(const char * url)
                throw() {
                CDMW_LOG(buffersender_logger, 
                         CYCLIC_MSG_BUFFERSENDER_LOG_LEVEL, 
                         "=== BufferSender.setRecorder(0) : taking mutex");
                CDMW_MUTEX_GUARD(m_mutex) ;
                CDMW_LOG(buffersender_logger, 
                         CYCLIC_MSG_BUFFERSENDER_LOG_LEVEL, 
                         "=== BufferSender.setRecorder(1) : mutex taken");
                m_recorder_url = url ;
                if (url == 0)
                {
                    m_recorder = (CdmwRecording::Recorder *)0 ;
                }
                else
                {
                    try
                    {
                        CORBA::Object_var obj = m_orb->string_to_object(url) ;
                        if (!CORBA::is_nil(obj.in()))
                        {
                            m_recorder = CdmwRecording::Recorder::_unchecked_narrow (obj.in()) ;
                            if (!CORBA::is_nil(m_recorder.in())) // JM 12/09/2007
                                //                     if (! CORBA::is_nil(m_recorder))
                            {
                                set_timeout_policy_on (m_orb.in(), m_recorder.inout(), m_corba_timeout_ms);
                                CDMW_LOG(buffersender_logger, 
                                         MSG_BUFFERSENDER_LOG_LEVEL, 
                                         "=== BufferSender.setRecorder(2) : Recording server available");
                            }
                            else
                            {
                                CDMW_LOG(buffersender_logger, 
                                         MSG_BUFFERSENDER_LOG_LEVEL, 
                                         "=== BufferSender.setRecorder(3) : Recording server not available");
                            }

                        }
                    }
                    catch (...)
                    {
                        CDMW_LOG(buffersender_logger, 
                                 MSG_BUFFERSENDER_LOG_LEVEL, 
                                 "=== BufferSender.setRecorder(4) : Recording server not available");
                    }
                }
            }

            virtual void run() throw() {
                bool recording_server_available = false;
                while (! m_terminated)
                {
                    // Wait until recording server is ready
                    while (! m_terminated)
                    {
                        try
                        {
                            if (m_recorder_url.c_str() != 0)
                            {
                                CDMW_LOG(buffersender_logger, 
                                         CYCLIC_MSG_BUFFERSENDER_LOG_LEVEL, 
                                         "=== BufferSender.run(0) : taking mutex");
                                CDMW_MUTEX_GUARD(m_mutex) ;
                                CDMW_LOG(buffersender_logger, 
                                         CYCLIC_MSG_BUFFERSENDER_LOG_LEVEL, 
                                         "=== BufferSender.run (1) : mutex taken");
                                try
                                {
                                    CORBA::Object_var obj = m_orb->string_to_object(m_recorder_url.c_str()) ;
                                    if (!CORBA::is_nil(obj.in()))
                                    {
                                        m_recorder = CdmwRecording::Recorder::_unchecked_narrow (obj.in()) ;
                                        set_timeout_policy_on (m_orb.in(), m_recorder.inout(), m_corba_timeout_ms);
                                        if ((!CORBA::is_nil(m_recorder.in())) && !(m_recorder->_non_existent())) // JM 12/09/2007
                                        {
                                            recording_server_available = true;
                                            CDMW_LOG(buffersender_logger, 
                                                     MSG_BUFFERSENDER_LOG_LEVEL, 
                                                     "=== BufferSender.run (3): Recording server available");
                                            break;
                                        }
                                        else
                                        {
                                            if (recording_server_available)
                                            {
                                                recording_server_available = false;
                                                CDMW_LOG(buffersender_logger, 
                                                         MSG_BUFFERSENDER_LOG_LEVEL, 
                                                         "***  BufferSender.run (4): Recording server not available");
                                            }
                                        }
                                    }
                                    else
                                    {
                                        CDMW_LOG(buffersender_logger, 
                                                 MSG_BUFFERSENDER_LOG_LEVEL, 
                                                 "*** Recording Server (5) : Nil reference");
                                    }
                                }
                                catch (const CORBA::Exception& ex)
                                {
                                    if (recording_server_available)
                                    {
                                        recording_server_available = false;
                                        CDMW_LOG(buffersender_logger, 
                                                 MSG_BUFFERSENDER_LOG_LEVEL, 
                                                 "*** BufferSender.run (6) : Recording server not available");
                                    }
                                }
                            }
                        }
                        catch (...)
                        {
                            CDMW_LOG(buffersender_logger, 
                                     MSG_BUFFERSENDER_LOG_LEVEL, 
                                     "*** BufferSender.run (7) : Recording server not available");
                        }
                        Cdmw::OsSupport::OS::sleep (m_polling) ;
                        CDMW_LOG(buffersender_logger, 
                                 CYCLIC_MSG_BUFFERSENDER_LOG_LEVEL, 
                                 "!!! BufferSender.run (8): ... trying to connect...");
                        if (m_max_retries >= 0)
                        {
                            if (m_num_retries < m_max_retries)
                            {
                                m_num_retries++;
                            }
                            else
                            {
                                CDMW_LOG(buffersender_logger, 
                                         MSG_BUFFERSENDER_LOG_LEVEL, 
                                         "*** number max of retries "<< m_max_retries << " exceeded, terminating ...");
                                terminate () ;
                            }
                        }
                    }

                    while (! m_terminated)
                    {
                        Buffer * buffer ;

                        // Wait for a buffer to be ready
                        {
                            CDMW_LOG(buffersender_logger, 
                                     CYCLIC_MSG_BUFFERSENDER_LOG_LEVEL, 
                                     "===  BufferSender.run (10) : taking mutex");
                            CDMW_MUTEX_GUARD(m_mutex) ;
                            CDMW_LOG(buffersender_logger, 
                                     CYCLIC_MSG_BUFFERSENDER_LOG_LEVEL, 
                                     "===  BufferSender.run (11) : mutex taken");
                            while ((m_buffers_head == 0) && ! m_terminated)
                            {
                                CDMW_LOG(buffersender_logger, 
                                         MSG_BUFFERSENDER_LOG_LEVEL, 
                                         "===  BufferSender.run : waiting ... (12)");
                                m_control.wait() ;
                            }

                            if (m_buffers_head == 0)
                            {
                                break ;
                            }

                            // Unlink it
                            buffer = m_buffers_head ;
                            m_buffers_head = buffer->next ;
                            buffer->next = 0 ;
                            if (m_buffers_head == 0)
                            {
                                m_buffers_tail = 0 ;
                            }
                            m_num_buffers-- ; // CDO-070330-PRS-1-567 : #19
                        }

                        try
                        {
                            CDMW_LOG(buffersender_logger, 
                                     CYCLIC_MSG_BUFFERSENDER_LOG_LEVEL, 
                                     "===  BufferSender.run (13) : sending message");
                            CdmwRecording::GIOP_Buffers giop_buffer (buffer->size, buffer->size, buffer->data, 0) ;
                            m_recorder->record (buffer->header, giop_buffer) ;
                            CDMW_LOG(buffersender_logger, 
                                     CYCLIC_MSG_BUFFERSENDER_LOG_LEVEL, 
                                     "===  BufferSender.run (14) : sending message sent");
                        }
                        catch (...)
                        {
                            CDMW_LOG(buffersender_logger, 
                                     CYCLIC_MSG_BUFFERSENDER_LOG_LEVEL, 
                                     "***  BufferSender.run : catch exception (15) : taking mutex");
                            CDMW_MUTEX_GUARD(m_mutex) ;
                            CDMW_LOG(buffersender_logger, 
                                     CYCLIC_MSG_BUFFERSENDER_LOG_LEVEL, 
                                     "***  BufferSender.run : catch exception (16) : mutex taken");
                            if (m_num_buffers < m_max_buffers)
                            {
                                buffer->next = m_buffers_head ;
                                m_buffers_head = buffer ;
                                buffer = 0 ;
                                if (m_buffers_tail == 0)
                                {
                                    m_buffers_tail = m_buffers_head ;
                                }
                                m_num_buffers++ ; // CDO-070330-PRS-1-567 : #19
                            }
                            break ;
                        }
                        if (buffer)
                        {
                            CDMW_LOG(buffersender_logger, 
                                     CYCLIC_MSG_BUFFERSENDER_LOG_LEVEL, 
                                     "!!! BufferSender.run : freeing buffer (19)");
                            m_manager->free_buffer (buffer) ;
                            // m_num_buffers-- ; // CDO-070330-PRS-1-567 : #19
                            CDMW_LOG(buffersender_logger, 
                                     CYCLIC_MSG_BUFFERSENDER_LOG_LEVEL, 
                                     "!!! BufferSender.run : buffer freed (20)");
                        }
                    }
                }
            }

            /**
             *Purpose: Set the maximum buffer number
             *<p> .
             *
             */
            virtual void SetMaxBufferNumber (size_t size) {
                m_max_buffers = (int) size;
            }

            /**
             *Purpose: Set the maximum buffer number
             *<p> .
             *
             */
            virtual size_t GetMaxBufferNumber () {
                return (int) m_max_buffers;
            }

            /**
             *Purpose: Set the retry period
             *<p> .
             *
             */
            virtual void SetRetryPeriod (Milliseconds delay) {
                m_polling = delay;
            }


            /**
             *Purpose: Get the retry period
             *<p> .
             *
             */
            virtual Milliseconds GetRetryPeriod () {
                return m_polling;
            }

            /**
             *Purpose: Set the maximum number of connection
             * with the recording server retries
             *<p> .
             *
             */
            virtual void SetRetryNumber (long number) {
                m_max_retries = number;
            }

            /**
             *Purpose: Set the Corba Relative Roundtrip Timeout
             *<p> .
             *
             */
            virtual void SetCorbaTimeout (unsigned long timeout) {
                m_corba_timeout_ms = timeout;
            }
            /**
             *Purpose: Post the buffer for sending
             *<p> .
             *
             */

            virtual void sendBuffer(Buffer *& buffer)
                throw() {
                if (buffer == 0)
                {
                    // (1) protection : if the same buffer is sent
                    //     more than once
                    return ;
                }
          
                CDMW_LOG(buffersender_logger, 
                         CYCLIC_MSG_BUFFERSENDER_LOG_LEVEL, 
                         "=== Buffersender.sendBuffer (0) : taking mutex");
                CDMW_MUTEX_GUARD(m_mutex) ;
                CDMW_LOG(buffersender_logger, 
                         CYCLIC_MSG_BUFFERSENDER_LOG_LEVEL, 
                         "=== Buffersender.sendBuffer (1) : mutex taken");
                if (m_buffers_tail == 0)
                {
                    m_buffers_head = buffer ;
                }
                else
                {
                    m_buffers_tail->next = buffer ;
                }
                m_buffers_tail = buffer ;
                m_num_buffers++;

                while (m_num_buffers > m_max_buffers)
                {
                    // Remove first message
                    buffer = m_buffers_head ;
                    m_buffers_head = buffer->next ;
                    buffer->next = 0 ;
                    CDMW_LOG(buffersender_logger, 
                             MSG_BUFFERSENDER_LOG_LEVEL, 
                             "!!! Buffersender.sendBuffer (2) : max buffers number reached, freeing oldest buffers");
                    m_manager->free_buffer (buffer, true) ;
                    m_num_buffers--;
                    CDMW_LOG(buffersender_logger, 
                             MSG_BUFFERSENDER_LOG_LEVEL, 
                             "!!! Buffersender.sendBuffer (3) : buffer freed\n");
                }

                CDMW_LOG(buffersender_logger, 
                         CYCLIC_MSG_BUFFERSENDER_LOG_LEVEL, 
                         "=== Buffersender.sendBuffer (4) : broadcast");
                m_control.broadcast() ;
                // protection : if the same buffer is sent
                // more than once : it is set to 0
                // if it is sent again it will not be chained
                // cf. (1) above
                buffer = 0 ;
            }            

            virtual void terminate () throw() {
                CDMW_LOG(buffersender_logger, 
                         CYCLIC_MSG_BUFFERSENDER_LOG_LEVEL, 
                         "terminate (0) : taking mutex");
                CDMW_MUTEX_GUARD(m_mutex) ;
                CDMW_LOG(buffersender_logger, 
                         CYCLIC_MSG_BUFFERSENDER_LOG_LEVEL, 
                         "terminate (1) : mutex taken, broadcast ...");
                m_terminated = true ;
                m_control.broadcast() ;
            }

        private :
            Milliseconds m_polling ;
            int m_num_buffers ;
            int m_max_buffers ;
            int m_num_retries ;
            int m_max_retries ;
            OsSupport::Mutex & m_mutex ;
            OsSupport::Condition m_control ;
            CoalescenceManager * m_manager ;
            std::string m_recorder_url ;
            CORBA::ORB_var m_orb;
            CdmwRecording::Recorder_var m_recorder ;
            Buffer * m_buffers_head ;
            Buffer * m_buffers_tail ;
            bool m_terminated ;
            OsSupport::Thread::ThreadId m_id ;
            unsigned long m_corba_timeout_ms;
        } ;

        BufferSender::BufferSender()
            throw()
            : m_mutex()
            , m_recordingURL("")
            , m_worker(0)
        {
            buffersender_logger = Cdmw::Logging::Logger::Get_Logger("CDMW.TRC.RecordingDaemon");
        }

        BufferSender::~BufferSender()
            throw()
        {
            if (m_worker)
            {
                finalize();
            }
        }


        /**
         *Purpose: setter and getter for the URL of the recording server
         *<p> .
         *
         */
        void BufferSender::recordingURL(const char * url)
            throw()
        {
            if ( url == 0)
            {
                m_recordingURL = CORBA::string_dup ("") ; 
            } else {
                m_recordingURL = CORBA::string_dup (url) ;
            }
            if (m_worker)
            {
                m_worker->setRecorder(m_recordingURL.c_str()) ;
            }
        }

        char * BufferSender::recordingURL()
            throw()
        {
            if (m_recordingURL.size() > 0)
                return CORBA::string_dup (m_recordingURL.c_str()) ;
            else
                return 0;
        }

        /**
         *Purpose: Make the plugin ready to work
         *<p> .
         *virtual
         */
        void BufferSender::initialize(CoalescenceManager * manager, CORBA::ORB_ptr orb)
            throw()
        {
            if ((m_worker) || (manager == 0))
            {
                return;
            }
            m_worker = new Worker (manager, m_recordingURL.c_str(), m_mutex, orb) ;
        }

        /**
         *Purpose: Finishes all processing
         *<p> .
         *
         */
        void BufferSender::finalize()
            throw()
        {
            if (m_worker)
            {
                m_worker ->terminate() ;
                m_worker ->join() ;
                delete m_worker ;
                m_worker = 0;
            }
        }

        /**
         *Purpose: Set the maximum buffer number
         *<p> .
         *
         */
        void BufferSender::SetMaxBufferNumber (size_t size)
        {
            if (m_worker)
            {
                m_worker -> SetMaxBufferNumber (size);
            }
        }

        /**
         *Purpose: Set the Corba Relative Roundtrip Timeout
         *<p> .
         *
         */
        void BufferSender::SetCorbaTimeout (unsigned long timeout)
        {
            if (m_worker)
            {
                m_worker -> SetCorbaTimeout (timeout);
            }
        }

        /**
         *Purpose: Get the maximum buffer number
         *<p> .
         *
         */
        size_t BufferSender::GetMaxBufferNumber ()
        {
            if (m_worker)
            {
                return m_worker -> GetMaxBufferNumber ();
            }
            return 0;
        }
        /**
         *Purpose: Set the retry period
         *<p> .
         *
         */
        void BufferSender::SetRetryPeriod (Milliseconds delay)
        {
            if (m_worker)
            {
                m_worker -> SetRetryPeriod  (delay);
            }
        }

        /**
         *Purpose: Get the retry period
         *<p> .
         *
         */
        Milliseconds BufferSender::GetRetryPeriod ()
        {
            if (m_worker)
            {
                return m_worker -> GetRetryPeriod ();
            }
            return 0 ;
        }

        /**
         *Purpose: Set the maximum number of connection
         * with the recording server retries
         *<p> .
         *
         */
        void BufferSender::SetRetryNumber (long number)
        {
            if (m_worker)
            {
                m_worker -> SetRetryNumber (number);
            }
        }

        /**
         *Purpose: Sends the buffer now
         *<p> .
         *
         */
        void BufferSender::sendBuffer(Buffer *& buffer)
            throw()
        {
            if (m_worker)
            {
                m_worker->sendBuffer(buffer) ;
            }
        }

    }
}
