/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */
#include <iostream>
#include <fstream>

#include <Foundation/logging/LogManager.hpp>
#include <Foundation/logging/Logger.hpp>
#include <Foundation/logging/FunctionLogger.hpp>
#include "Foundation/ossupport/OS.hpp"
#include "Foundation/osthreads/Thread.hpp"
#include "Foundation/commonsvcs/time/Util.hpp"
#include "TraceAndPerf/recordinglibrary/Recording_Default_Parameters.hpp"
#include "TraceAndPerf/recordinglibrary/CoalescenceManager.hpp"
#include "TraceAndPerf/recordinglibrary/Swapping.hpp"


namespace {

   //const long int IDLE_TIMEOUT = 3600000;
   //const int MSG_COALESCENCEMANAGER_LOG_LEVEL = 20;
   //const int CYCLIC_MSG_COALESCENCEMANAGER_LOG_LEVEL = 21;
   //const int DEFAULT_BUFFER_MAX_SIZE = 8*1024;
   //const int DEFAULT_SENDING_TIMEOUT = 100;

    // Definition of a GIOP_BufferReplyComponent
    // which should be empty for empty_reply:
    //     struct GIOP_BufferReplyComponent {
    //        GIOP_Header head;
    //        unsigned long request_id;
    //        short date;
    //        boolean endian;
    //        unsigned long status ;
    //        ServiceContextList service_contexts;
    //        sequence <octet> data;
    //     };
    const Cdmw::Recording::octet EMPTY_REPLY[] = {
      'G','I','O','P', 1, 2, 1, 1, 0, 0, 0, 0,  // GIOP_Header
      0, 0, 0, 0,                               // request_id
      0, 0,                                     // date
      0,                                        // endian
      0,                                        // padding to 4 octets
      0, 0, 0, 0,                               // status
      0, 0, 0, 0,                               // empty service_contexts
      0, 0, 0, 0                                // empty data
    };

    const size_t EMPTY_REPLY_SIZE =
        sizeof(EMPTY_REPLY) / sizeof(Cdmw::Recording::octet);

    // TODO: to put in common in a .hpp with the same in PacketStore.cpp
    const short ENDIAN_CHECKER = 0x0001 ;
    const Cdmw::Recording::octet * ENDIAN_BYTE =
        (const Cdmw::Recording::octet *) &ENDIAN_CHECKER ;
    const Cdmw::Recording::octet NATIVE_ENDIAN = *ENDIAN_BYTE ;
};


namespace Cdmw
{
   namespace Recording
   {
      Cdmw::Logging::Logger* coalescencemanager_logger;

      Milliseconds operator-(const timeval& t1, const timeval& t2)
      {
         Milliseconds msec = (t1.tv_sec - t2.tv_sec) * 1000 + (t1.tv_usec - t2.tv_usec) / 1000 ;
         return msec ;
      }


      class CoalescenceManager::Sequencer : public virtual OsSupport::Thread
      {
         public :
            Sequencer (Milliseconds timeout, OsSupport::Mutex & mutex, CoalescenceManager & manager) throw()
                  : OsSupport::Thread()
                  , m_timeOut(timeout)
                  , m_mutex(mutex)
                  , m_control(mutex)
                  , m_manager(manager)
                  , m_do_wakeup(false)
                  , m_is_wakeup(false)
                  , m_terminated(true)
            {
              /* CDO-070330-PRS-1-567 : #26
              start() ;
              CDMW_MUTEX_GUARD(m_mutex) ;
              while (m_terminated)
              {
                m_control.wait() ;
        }
              */
            }

            virtual ~Sequencer () throw()
            {
               terminate () ;
            }

            // CDO-070330-PRS-1-567 : #26
            virtual void start_now() throw()
        {
              start() ;
              CDMW_MUTEX_GUARD(m_mutex) ;
              while (m_terminated)
              {
                m_control.wait() ;
              }
        }

            virtual void run() throw()
        {
               Milliseconds timeOut = IDLE_TIMEOUT ;

               {
                 CDMW_MUTEX_GUARD(m_mutex) ;
                 m_terminated = false ;
                 m_control.broadcast() ;
               }

               while (! m_terminated)
               {
                  bool do_flush = false ;

                  {
                    CDMW_MUTEX_GUARD(m_mutex) ;

                    if (m_terminated)
                    {
                      break ;
                    }
                    else if (m_do_wakeup)
                    {
              CDMW_LOG(coalescencemanager_logger,
                   CYCLIC_MSG_COALESCENCEMANAGER_LOG_LEVEL,
                   "=== Coalescence.run (1)");
                      m_do_wakeup = false ;
                      m_is_wakeup = true ;
                      timeOut = m_timeOut ;
                    }
                    else
                    {
              CDMW_LOG(coalescencemanager_logger,
                   CYCLIC_MSG_COALESCENCEMANAGER_LOG_LEVEL,
                   "===  Coalescence.run (2)");
                      m_control.wait(timeOut) ;
                      timeOut = IDLE_TIMEOUT ;
                      do_flush = m_is_wakeup && ! m_do_wakeup ;
                      m_is_wakeup = false ;
                    }
                  }
                  if (do_flush)
                  {
                    //if (coales_dbg)
            CDMW_LOG(coalescencemanager_logger,
                 MSG_COALESCENCEMANAGER_LOG_LEVEL,
                 "===  Coalescence.run -- FLUSH TIMEOUT --(3)");
                    m_manager.flush() ;
                  }
              }
            }

            virtual void set_timeOut(Milliseconds timeout) throw()
            {
               CDMW_MUTEX_GUARD(m_mutex) ;
               m_timeOut = timeout ;
            }

            virtual Milliseconds get_timeOut() throw()
            {
              return m_timeOut;
            }

            virtual void wakeup() throw()
            {

               // No guard as already done in caller (CAR106)
               if (m_do_wakeup || m_is_wakeup)
               {
                 return;
               }
               m_do_wakeup = true ;
               m_control.broadcast() ;
            }

            virtual void terminate () throw()
            {
               CDMW_MUTEX_GUARD(m_mutex) ;
               m_terminated = true ;
               m_control.broadcast() ;
            }

         private :
            Milliseconds m_timeOut ;
            OsSupport::Mutex & m_mutex ;
            OsSupport::Condition m_control ;
            CoalescenceManager & m_manager ;
            bool m_do_wakeup ;
            bool m_is_wakeup ;
            bool m_terminated ;
      } ;


      CoalescenceManager::CoalescenceManager (BufferSender & sender)
            throw()
            : m_mutex()
            , m_sequencer(0)
            , m_maxSize(DEFAULT_BUFFER_MAX_SIZE)
            , m_timeOut(DEFAULT_SENDING_TIMEOUT)
            , m_sender(sender)
            , m_free_buffers(0)
            , m_current_buffer(0)
      {
        m_hostname = Cdmw::OsSupport::OS::get_hostname();
        coalescencemanager_logger = Cdmw::Logging::Logger::Get_Logger("CDMW.TRC.RecordingDaemon");
      }

      CoalescenceManager::~CoalescenceManager ()
          throw()
      {
        if (m_sequencer)
        {
          m_sequencer->terminate() ;
          m_sequencer ->join() ;
          delete m_sequencer ;
          m_sequencer = 0;
        }
      }

      void CoalescenceManager::initialize ()
            throw()
      {
         if (m_sequencer)
         {
           return;
         }
         m_sequencer = new Sequencer(m_timeOut, m_mutex, *this) ;
     m_sequencer->start_now();  // CDO-070330-PRS-1-567 : #26
      }

      void CoalescenceManager::finalize ()
            throw()
      {
        if (m_sequencer)
        {
          m_sequencer->terminate() ;
          m_sequencer ->join() ;
          delete m_sequencer ;
          m_sequencer = 0;
        }
      }

      /**
       *Purpose: Set the maximum buffer size
       *<p> .
       *
       */
      void CoalescenceManager::MaxSize (size_t size)
      {
        m_maxSize = size ;
      }

      /**
       *Purpose: Set the timeout
       *<p> .
       *
       */
      void CoalescenceManager::TimeOut (Milliseconds delay)
      {
        m_timeOut = delay ;
        if (m_sequencer)
        {
          m_sequencer -> set_timeOut (m_timeOut);
        }
      }


      /**
       *Purpose: Get the timeout
       *<p> .
       *
       */
      Milliseconds CoalescenceManager::TimeOut ()
      {
        if (m_sequencer)
        {
          return m_sequencer -> get_timeOut ();
        }
        return m_timeOut ;
      }


      /**
       *Purpose: Write the request/reply pair in the sending
       * buffer
       *<p> .
       *
       */
      void CoalescenceManager::WriteOperation(Message * request, Message * reply)
            throw()
      {
    CDMW_LOG(coalescencemanager_logger,
         CYCLIC_MSG_COALESCENCEMANAGER_LOG_LEVEL,
         "=== Coalescence.WriteOperation (0)");
         if (request == 0)
         {
           return;
         }

         // compute future size of GIOP_BufferRequestComponent structure
         size_t size = request->GiopSize () ;

         // compute size of alignment to 4 before reply structure
         size += ((-size) & 3);

         if (reply == 0)
         {
           // add size of an empty GIOP_BufferReplyComponent structure
           size += EMPTY_REPLY_SIZE ;
         }
         else
         {
           // compute future size of GIOP_BufferReplyComponent structure
           size += reply->GiopSize () ;
         }

         // If there is no room in the buffer, flush it
         if ((m_current_buffer != 0) && (size + m_current_buffer->size > m_maxSize))
         {
       CDMW_LOG(coalescencemanager_logger,
            MSG_COALESCENCEMANAGER_LOG_LEVEL,
            "=== Coalescence.WriteOperation (1) -- FLUSH (MAX_BUFFER_SIZE)--");
           flush() ;
         }

     CDMW_LOG(coalescencemanager_logger,
          CYCLIC_MSG_COALESCENCEMANAGER_LOG_LEVEL,
          "=== Coalescence.WriteOperation (2) : taking mutex");
         CDMW_MUTEX_GUARD(m_mutex) ;
     CDMW_LOG(coalescencemanager_logger,
          CYCLIC_MSG_COALESCENCEMANAGER_LOG_LEVEL,
          "=== Coalescence.WriteOperation (3) : mutex taken");

         if (m_current_buffer == 0)
         {
            // If there is no current buffer

            if (m_free_buffers != 0)
            {
               // if there is some free buffers, pick the first one
               m_current_buffer = m_free_buffers ;
               m_free_buffers = m_free_buffers->next ;
               if (size > m_current_buffer->maxSize)
               {
                  delete[] m_current_buffer->data ;
                  m_current_buffer->maxSize = size ;
                  m_current_buffer->data = new octet[size] ;
               }
            }
            else
            {
               // if there is no more free buffer, create a new one for current
               m_current_buffer = new Buffer ;
               m_current_buffer->maxSize = (size > m_maxSize)?size:m_maxSize ;
               m_current_buffer->data = new octet[m_current_buffer->maxSize] ;
               m_current_buffer->header.hostname = CORBA::string_dup (m_hostname.c_str()) ;
               m_current_buffer->header.flags = NATIVE_ENDIAN;
            }

            // Initialize the buffer
            m_current_buffer->next = 0 ;
            m_current_buffer->date = request->date ;
            OsSupport::OS::Timeval time (request->date) ;
            m_current_buffer->header.base_time = Cdmw::CommonSvcs::Time::Util::Timeval_2_TimeT(time) ;
            m_current_buffer->header.num_pairs = 0 ;
            m_current_buffer->size = 0 ;

            // Arm the timeout
            m_sequencer->wakeup() ;
         }
         else
         {
             // align m_current_buffer content to 4 before adding new RequestReplyPair
             m_current_buffer->size += (-m_current_buffer->size) & 3;
         }

         // increase count of RequestReplyPair in current buffer
         m_current_buffer->header.num_pairs++ ;

         // set src_ptr to Request data buffer
         octet * src_ptr = request->data ;
         // set dst_ptr to free space in current buffer's data (just after current size)
         octet * dst_ptr = m_current_buffer->data + m_current_buffer->size ;

         // Update the used buffer size
         m_current_buffer->size += size ;

         // memorize address of the GIOP_BufferRequestComponent.head.message_size
         // for update after strucure is created
         // (+8 because head is in 1st position, and message_size is at 8 in GIOP_Header)
         size_t * header_size = (size_t *) (dst_ptr + 8) ;

         // Copy the MessageHeader_1_2 as GIOP_Header
         // (but size => only 8 firts octets)
         memcpy (dst_ptr, src_ptr, 8) ;
         // jump over GIOP_Header (12 octets)
         dst_ptr += 12 ;

         // GIOP_BufferRequestComponent.server_id
         size = strlen (request->server_id) + 1 ;
         *(size_t *)dst_ptr = size ;
         dst_ptr += 4 ;
         memcpy (dst_ptr, request->server_id, size) ;
         dst_ptr += size + ((-size) & 3) ; //align to 4

         // GIOP_BufferRequestComponent.request_id
         *(unsigned int *)dst_ptr = request->request_id ;
         dst_ptr += 4 ;

         // GIOP_BufferRequestComponent.date increment
         *(unsigned short *) dst_ptr = request->date - m_current_buffer->date ;
         dst_ptr += 2 ;

         // GIOP_BufferRequestComponent.response_flags
         // (src_ptr+16 to jump over MessageHeader_1_2 + RequestHeader_1_2.request_id)
         *dst_ptr = * (src_ptr + 16) ;
         dst_ptr ++ ;

         // GIOP_BufferRequestComponent.params_endian
         // (src_ptr+6 to jump to MessageHeader_1_2.flags)
         *dst_ptr = * (src_ptr + 6) ;
         dst_ptr ++ ;

         // GIOP_BufferRequestComponent.interface_name
         size = strlen (request->intf_rid) + 1 ;
         *(size_t *)dst_ptr = size ;
         dst_ptr += 4 ;
         memcpy (dst_ptr, request->intf_rid, size) ;
         dst_ptr += size + ((-size) & 3) ; //align to 4

         // GIOP_BufferRequestComponent.operation
         // move to offset of RequestHeader_1_2.operation
         src_ptr += request->operation_Offset ;
         size = strlen ((const char *)src_ptr) + 1 ;
         *(size_t *)dst_ptr = size ;
         dst_ptr += 4 ;
         memcpy (dst_ptr, src_ptr, size) ;
         dst_ptr += size + ((-size) & 3) ; //align destination to 4
         src_ptr += size + ((-size) & 3) ; //align source to 4

         // GIOP_BufferRequestComponent.service_contexts
         int num ;
         octet quad[4] ;

         Swapping::Swap4 (quad, src_ptr, request->need_swap) ;
         // read number of Service Contexts, and write it as sequence length (4 octets)
         num = *(int *) quad ;
         *(int *)dst_ptr = num ;
         src_ptr += 4 ;
         dst_ptr += 4 ;

         for (; num > 0; num --)
         {
            // Service context tag
            Swapping::Swap4 (quad, src_ptr, request->need_swap) ;
            // write ServiceContext.ServiceId
            *(int *)dst_ptr = *(int *) quad ;
            src_ptr += 4 ;
            dst_ptr += 4 ;

            // Service context size
            Swapping::Swap4 (quad, src_ptr, request->need_swap) ;
            // write ServiceContext.context_data sequence length
            size = *(size_t *) quad ;
            *(size_t *)dst_ptr = size ;
            src_ptr += 4 ;
            dst_ptr += 4 ;
            // copy ServiceContext.context_data buffer
            memcpy(dst_ptr, src_ptr, size) ;
            dst_ptr += size + ((-size) & 3) ; //align destination to 4
            src_ptr += size + ((-size) & 3) ; //align source to 4
         }

         // Align src_ptr to 8, since after Request Header, the Request Body
         // is aligned to 8 (see GIOP spec: chapter Request Body)
         size = src_ptr - request->data ;
         src_ptr += (-size) & 7 ;
         size += (-size) & 7 ;

         // check if Request Body length is not 0
         // (size = what have been read)
         size_t * params_size = 0;
         if ((unsigned int)request->header.length + 12 > size)
         {
           // compute size if Request Body (what hasn't been read)
           size = request->header.length + 12 - size ;

           // write Request Body buffer length
           size_t * params_size = (size_t *)dst_ptr ;
           *params_size = size ;
           dst_ptr += 4 ;
           // copy Request Body buffer
           memcpy(dst_ptr, src_ptr, size) ;

           dst_ptr += size + ((-size) & 3) ; //align to 4
         }
         else
         {
           // no parameters, write 0 as sequence<octet> length
           params_size = (size_t *)dst_ptr ;
           *params_size = 0 ;
           dst_ptr += 4 ;
         }

         // write the message size in GIOP_BufferRequestComponent.head.message_size
         size = dst_ptr - (octet *)header_size - 4 ;
         *header_size = size ;

     CDMW_LOG(coalescencemanager_logger,
          CYCLIC_MSG_COALESCENCEMANAGER_LOG_LEVEL,
          "=== Coalescence.WriteOperation Now the reply (3)");
         // Now the reply
         //--------------
         if (reply == 0)
         {
           // Copy an empty reply
           memcpy (dst_ptr, EMPTY_REPLY, EMPTY_REPLY_SIZE) ;

           if (m_current_buffer->size > m_maxSize)
           { // Can happen for big messages
         CDMW_LOG(coalescencemanager_logger,
              MSG_COALESCENCEMANAGER_LOG_LEVEL,
              "=== Coalescence.WriteOperation Big Message (4)");
             m_sender.sendBuffer(m_current_buffer) ;
             m_current_buffer = 0 ;
           }
           return;
         }

         // set src_ptr to start of Reply data buffer
         src_ptr = reply->data ;

         // memorize address of the GIOP_BufferRequestComponent.head.message_size
         // for update after strucure is created
         // (+8 because head is in 1st position, and message_size is at 8 in GIOP_Header)
         header_size = (size_t *) (dst_ptr + 8) ;

         // Copy the MessageHeader_1_2 as GIOP_Header
         // (but size => only 8 firts octets)
         memcpy (dst_ptr, src_ptr, 8) ;
         // jump over GIOP_Header (12 octets)
         dst_ptr += 12 ;

         // GIOP_BufferReplyComponent.request_id
         *(unsigned int *)dst_ptr = reply->request_id ;
         dst_ptr += 4 ;

         // GIOP_BufferReplyComponent.date increment
         *(unsigned short *) dst_ptr = reply->date - m_current_buffer->date ;
         dst_ptr += 2 ; // jump over short

         // GIOP_BufferReplyComponent.params_endian
         // (src_ptr+6 to jump to MessageHeader_1_2.flags)
         *dst_ptr = * (src_ptr + 6) ;

         dst_ptr += 2 ; // aligne to 4 (jump overt endian(1 octet) + 1 octet)

         // GIOP_BufferReplyComponent.status
         // (src_ptr+16 to jump over MessageHeader_1_2 + ReplyHeader_1_2.request_id)
         *(unsigned int *)dst_ptr = *(unsigned int *) (src_ptr + 16) ;
         dst_ptr += 4 ;

         // GIOP_BufferRequestComponent.service_contexts
         // src_ptr jump over MessageHeader_1_2 + ReplyHeader_1_2.request_id + ReplyHeader_1_2.status
         src_ptr += 20 ;
         Swapping::Swap4 (quad, src_ptr, reply->need_swap) ;
         // read number of Service Contexts, and write it as sequence length (4 octets)
         num = *(int *) quad ;
         *(int *)dst_ptr = num ;
         src_ptr += 4 ;
         dst_ptr += 4 ;

         for (; num > 0; num --)
         {
            // Service context tag
            Swapping::Swap4 (quad, src_ptr, reply->need_swap) ;
            // write ServiceContext.ServiceId
            *(int *)dst_ptr = *(int *) quad ;
            src_ptr += 4 ;
            dst_ptr += 4 ;
            // Service context size
            Swapping::Swap4 (quad, src_ptr, reply->need_swap) ;
            // write ServiceContext.context_data sequence length
            size = *(size_t *) quad ;
            *(size_t *)dst_ptr = size ;
            src_ptr += 4 ;
            dst_ptr += 4 ;
            // copy ServiceContext.context_data buffer
            memcpy(dst_ptr, src_ptr, size) ;
            dst_ptr += size + ((-size) & 3) ; //align to 4
            src_ptr += size + ((-size) & 3) ; //align to 4
         }

         // Align src_ptr to 8, since after Reply Header, the Reply Body
         // is aligned to 8 (see GIOP spec: chapter Reply Body)
         size = src_ptr - reply->data ;
         src_ptr += (-size) & 7 ;
         size += (-size) & 7 ;

         // check if Reply Body length is not 0
         // (size = what have been read)
         if ((unsigned int)reply->header.length + 12 > size)
         {
           // compute size if Reply Body (what hasn't been read)
           size = reply->header.length + 12 - size ;

           // write Reply Body buffer length
           params_size = (size_t *)dst_ptr ;
           *params_size = size ;
           dst_ptr += 4 ;
           // copy Reply Body buffer
           memcpy(dst_ptr, src_ptr, size) ;

           dst_ptr += size + ((-size) & 3) ; //align to 4
         }
         else
         {
           // no parameters, write 0 as sequence<octet> length
           params_size = (size_t *)dst_ptr ;
           *params_size = 0 ;
           dst_ptr += 4 ;
         }

         // write the message size in GIOP_BufferRequestComponent.head.message_size
         size = dst_ptr - (octet *)header_size - 4 ;
         *header_size = size ;

         if (m_current_buffer->size > m_maxSize)
         { // Can happen for big messages
       CDMW_LOG(coalescencemanager_logger,
            MSG_COALESCENCEMANAGER_LOG_LEVEL,
            "=== Coalescence.WriteOperation Big Message (5)");
           m_sender.sendBuffer(m_current_buffer) ;
           m_current_buffer = 0 ;
         }
      }

      /**
       *Purpose: Sends the buffer now
       *<p> .
       *
       */
      void CoalescenceManager::flush()
            throw()
      {
         CDMW_MUTEX_GUARD(m_mutex) ;

         if (m_current_buffer == 0)
         {
            return ;
         }

         m_sender.sendBuffer(m_current_buffer) ;
         m_current_buffer = 0 ;
      }

      /**
       *Purpose: Return a used buffer to the free list
       *<p> .
       *
       */
     void CoalescenceManager::free_buffer(Buffer *& buffer, bool already_mutexed)
            throw()
      {
        if (!already_mutexed)
      {
    CDMW_LOG(coalescencemanager_logger,
         CYCLIC_MSG_COALESCENCEMANAGER_LOG_LEVEL,
         "=== CoalescenceManager::free_buffer(1) : taking coalescence mutex");
        CDMW_MUTEX_GUARD(m_mutex) ;
    CDMW_LOG(coalescencemanager_logger,
         CYCLIC_MSG_COALESCENCEMANAGER_LOG_LEVEL,
         "=== CoalescenceManager::free_buffer(2) : coalescence mutex taken, freeing buffer");
      }
         if (buffer != 0)
         {
           buffer->next = m_free_buffers ;
           m_free_buffers = buffer ;
           buffer = 0 ;
         }
    CDMW_LOG(coalescencemanager_logger,
         CYCLIC_MSG_COALESCENCEMANAGER_LOG_LEVEL,
         "=== CoalescenceManager::free_buffer(3) : freeing buffer freed");
      }

   }
}
