/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

#include <errno.h>

#include "Foundation/ossupport/SystemProperties.hpp"
#include "Foundation/orbsupport/CORBA.hpp"

#include <Foundation/logging/LogManager.hpp>
#include <Foundation/logging/Logger.hpp>
#include <Foundation/logging/FunctionLogger.hpp>

#include "TraceAndPerf/recordinglibrary/Recording_Default_Parameters.hpp"
#include "TraceAndPerf/recordinglibrary/DebugPlugin.hpp"
#include "TraceAndPerf/recordinglibrary/Swapping.hpp"
#include "TraceAndPerf/recordingidl/CdmwRecordingData.stub.hpp"
#include "TraceAndPerf/recordingidl/CdmwRecordingDebug.stub.hpp"

namespace Cdmw
{
   namespace Recording
   {
      Cdmw::Logging::Logger* debugplugin_logger;
      const octet NULL_REPLY[] = {'G','I','O','P', 1, 2, 1, 1, 0, 0, 0, 0};
      const std::string DEPRECATED_MAX_DEBUG_FILE_SIZE_NAME ("Max_Debug_File_Size") ;
       const std::string MAX_DEBUG_FILE_SIZE_NAME ("Cdmw.Recording.MaxDebugFileSize") ;
 
      /**
       *Purpose: Store request/reply pairs in a file
       *<p> .
       *
       */

      DebugPlugin::DebugPlugin ()
            throw() : Plugin (), m_fd(-1), m_file_size(0), m_file_name(0)
      {
          debugplugin_logger = Cdmw::Logging::Logger::Get_Logger("CDMW.TRC.RecordingDaemon");

          int Max_Debug_File_Size = Cdmw::OsSupport::SystemProperties::Get_Property
              (DEPRECATED_MAX_DEBUG_FILE_SIZE_NAME, -1) ;
          if( Max_Debug_File_Size > -1 )
          {
              CDMW_WARN(debugplugin_logger, "Prefer the use of ["
                        << MAX_DEBUG_FILE_SIZE_NAME << "], instead of ["
                        << DEPRECATED_MAX_DEBUG_FILE_SIZE_NAME << "]");
              Cdmw::OsSupport::SystemProperties::Set_Property(
                  MAX_DEBUG_FILE_SIZE_NAME, Max_Debug_File_Size);
          }          
      }

      DebugPlugin::~DebugPlugin ()
            throw()
      {
         finalize () ;
      }


      /**
       *Purpose: Set the debug destination file
       *<p> .  
       *
       */
      bool DebugPlugin::Destination (const char * destination)
            throw()
      {
         if (destination != 0 && m_file_name != 0 &&
                                    strcmp(m_file_name, destination) == 0)
         {
               return true ;
          }

          if (m_file_name != 0)
         {
            ::close (m_fd) ;
            m_fd = - 1 ;

            // JM : reset m_file_name
            CORBA::string_free (m_file_name) ;
            m_file_name = 0 ;
            m_file_size = 0 ;
         }

         if (destination == 0)
         {
            return false ;
         }

         std::string dest = destination;

         m_fd = ::open(destination, O_WRONLY | O_CREAT | O_EXCL | O_TRUNC, S_IRUSR|S_IRGRP|S_IROTH) ;
         while (m_fd == -1)
         {
           if (EEXIST == errno)
           {
              dest += '+' ;
              m_fd = ::open(dest.c_str(), O_WRONLY | O_CREAT | O_EXCL | O_TRUNC, S_IRUSR|S_IRGRP|S_IROTH) ;
           }
           else
           {
              return false ;
           }
         }

         m_file_name = CORBA::string_dup (dest.c_str()) ;
         return true ;
      }

      /**
       *Purpose: Get the debug destination file
       *<p>    : caller shall call CORBA::string_free().
       *
       */
      char * DebugPlugin::Destination ()
            throw()
      {
         if (m_file_name == 0)
         {
            return 0 ;
         }

         return CORBA::string_dup (m_file_name) ;
      }

      /**
       *Purpose: Store the request-reply pair given as argument
       *<p> .
       *
       */
      void DebugPlugin::recordOperation (Message * request, Message * reply)
            throw()
      {
          CDMW_LOG(debugplugin_logger, 
                   CYCLIC_DEBUGPLUGIN_LOG_LEVEL, 
                   "=== recordoperation(0) " << m_file_name);
          // JM Thales : 12/07/07
          static int Max_Debug_File_Size = Cdmw::OsSupport::SystemProperties::Get_Property
              (MAX_DEBUG_FILE_SIZE_NAME, DEFAULT_MAX_DEBUG_FILE_SIZE);

          if (m_file_name == 0)
          {
              return ;
          }
          
     CDMW_LOG(debugplugin_logger, 
          CYCLIC_DEBUGPLUGIN_LOG_LEVEL, 
          "=== recordoperation(1)");
         if (request == 0)
         {
            return ;
         }

         if (request->header.message_type == CdmwRecording::Request)
         {
           CDMW_LOG(debugplugin_logger, 
                    CYCLIC_DEBUGPLUGIN_LOG_LEVEL, 
                    "=== recordoperation(2)");
           // Calculate the resulting file_size
           int req_size = request->GiopSize () + 8 + 12 ; // Addition of date size (8) and connection channel size (12)
           req_size += ((-req_size) & 3) ;
           m_file_size += req_size ;
           int rep_size = 0;
           if (reply != 0)
           {
             rep_size = reply->GiopSize () + 8 ; // addition of date size (8)
             m_file_size += rep_size ;
           }
           else
           {
             m_file_size += 12 ;
           }
           m_file_size += ((-m_file_size) & 3) ;

           if (m_file_size > Max_Debug_File_Size)
           {
             CDMW_ERROR(Cdmw::Logging::Logger::Get_Logger("CDMW.TRC.RecordingDaemon"),
                        "Max_Debug_File_Size raised !!!\n") ;
             // Output file full
             CORBA::string_free (m_file_name) ;
             m_file_name = 0 ;
             m_file_size = 0 ;
             return ;
           }
           CDMW_LOG(debugplugin_logger, 
                    CYCLIC_DEBUGPLUGIN_LOG_LEVEL, 
                    "=== recordoperation(3)");

           octet quad[4] ;
           // Substraction of header size (12)
           int size = req_size - 12 ;
           size += ((-size) & 3) ;

           octet * src_ptr = request->data ;

           // Write the GIOP header (but size)
           ::write (m_fd, src_ptr, 8) ;

           // Write size
           ::write (m_fd, &size, 4) ;

           // connection channel
           ::write (m_fd, &request->channel, 12) ;

           // server_id
           size = strlen (request->server_id) + 1 ;
           ::write (m_fd, &size, 4) ;
           ::write (m_fd, request->server_id, size) ;
           *(unsigned int *)quad = 0 ;
           ::write (m_fd, quad, ((-size) & 3)) ; //align to 4

           // request_id
           ::write (m_fd, &request->request_id, 4) ;

           // date
           ::write (m_fd, &request->date, 8) ;

           // response_flags
           ::write (m_fd, src_ptr + 16, 1) ;

           // params_endian
           ::write (m_fd, src_ptr + 6, 1) ;

           *(unsigned int *)quad = 0 ;
           ::write (m_fd, quad, 2) ; //align to 4

           // interface_name
           size = strlen (request->intf_rid) + 1 ;
           ::write (m_fd, &size, 4) ;
           ::write (m_fd, request->intf_rid, size) ;
           *(unsigned int *)quad = 0 ;
           ::write (m_fd, quad, ((-size) & 3)) ; //align to 4

           // operation
           src_ptr += request->operation_Offset ;
           size = strlen ((const char *)src_ptr) + 1 ;
           ::write (m_fd, &size, 4) ;
           ::write (m_fd, src_ptr, size) ;
           *(unsigned int *)quad = 0 ;
           ::write (m_fd, quad, ((-size) & 3)) ; //align to 4
           src_ptr += size + ((-size) & 3) ; //align to 4

           // service contexts
           int num ;

           Swapping::Swap4 (quad, src_ptr, request->need_swap) ;
           num = *(int *) quad ;
           ::write (m_fd, quad, 4) ;
           src_ptr += 4 ;

           for (; num > 0; num --)
           {
             // Service context tag
             Swapping::Swap4 (quad, src_ptr, request->need_swap) ;
             ::write (m_fd, quad, 4) ;
             src_ptr += 4 ;

             // Service context size
             Swapping::Swap4 (quad, src_ptr, request->need_swap) ;
             size = *(size_t *) quad ;
             ::write (m_fd, quad, 4) ;
             src_ptr += 4 ;
             // Service context data
             ::write (m_fd, src_ptr, size) ;
             *(unsigned int *)quad = 0 ;
             ::write (m_fd, quad, ((-size) & 3)) ; //align to 4
             src_ptr += size + ((-size) & 3) ; //align to 4
           }

           // Align src_ptr to 8
           size = src_ptr - request->data ;
           src_ptr += (-size) & 7 ;
           long long pad = 0 ;
           ::write (m_fd, &pad, (-size) & 3) ; // Alignment to 4 only
           size += (-size) & 7 ;
           // size is the total size,
           // header.length is without Giop Header
           size = request->header.length + 12 - size ;

           // JM
           if (size > 0)
           {
             // operation parameters
             ::write (m_fd, &size, 4) ;
             ::write (m_fd, src_ptr, size) ;

             // Now add the fragments

             *(unsigned int *)quad = 0 ;
             ::write (m_fd, quad, ((-size) & 3)) ; //align to 4
           }
           else
           {
             // no parameters
             size = 0;
             ::write (m_fd, &size, 4) ;
           }


           size = rep_size - 12 ; // substraction of header (12)
           // Now the reply
           //--------------
           if (reply ==0)
           {
             ::write (m_fd, NULL_REPLY, 12);
             return ;
           }
           CDMW_LOG(debugplugin_logger, 
                    CYCLIC_DEBUGPLUGIN_LOG_LEVEL, 
                    "=== recordoperation(4)");         
           src_ptr = reply->data ;

           // Write the GIOP header (but size)
           ::write (m_fd, src_ptr, 8) ;

           // Write size
           ::write (m_fd, &size, 4) ;

           // request_id
           ::write (m_fd, &reply->request_id, 4) ;

           // date
           ::write (m_fd, &reply->date, 8) ;

           // params_endian
           ::write (m_fd, src_ptr + 6, 1) ;
           *(unsigned int *)quad = 0 ;
           ::write (m_fd, quad, 3) ; //align to 4

           // status
           ::write (m_fd, src_ptr + 16, 4) ;

           // service contexts
           src_ptr += 20 ;
           Swapping::Swap4 (quad, src_ptr, reply->need_swap) ;
           num = *(int *) quad ;
           ::write (m_fd, quad, 4) ;
           src_ptr += 4 ;

           for (; num > 0; num --)
           {
             // Service context tag
             Swapping::Swap4 (quad, src_ptr, reply->need_swap) ;
             ::write (m_fd, quad, 4) ;
             src_ptr += 4 ;

             // Service context size
             Swapping::Swap4 (quad, src_ptr, reply->need_swap) ;
             size = *(size_t *) quad ;
             ::write (m_fd, quad, 4) ;
             src_ptr += 4 ;
             // Service context data
             ::write (m_fd, src_ptr, size) ;
             *(unsigned int *)quad = 0 ;
             ::write (m_fd, quad, ((-size) & 3)) ; //align to 4
             src_ptr += size + ((-size) & 3) ; //align to 4
           }

           // Align src_ptr to 8
           size = src_ptr - reply->data ;
           src_ptr += (-size) & 7 ;
           pad = 0 ;
           ::write (m_fd, &pad, (-size) & 3) ;  //align to 4 only
           size += (-size) & 7 ;
           // size is the total size,
           // header.length is without Giop Header
           size = reply->header.length + 12 - size ;

           // JM
           if (size > 0)
           {
             // operation parameters
             ::write (m_fd, &size, 4) ;
             ::write (m_fd, src_ptr, size) ;

             // Now add the fragments

             *(unsigned int *)quad = 0 ;
             ::write (m_fd, quad, ((-size) & 3)) ; //align to 4
           }
           else
           {
             // no parameters
             size = 0;
             ::write (m_fd, &size, 4) ;
           }
         }
         else if (request->header.message_type == CdmwRecording::LocateRequest)
         {
           // Addition of date size (8) and connection channel size (12)
           int req_size = request->header.length + 12 + 8 + 12 ;
           req_size += ((-req_size) & 3) ;
           m_file_size += req_size ;
           int rep_size = 0;
           if (reply != 0)
           {
             rep_size = reply->header.length + 12 + 8 ; // addition of date size (8)
             m_file_size += rep_size ;
           }
           else
           {
             m_file_size += 12 ;
           }
           m_file_size += ((-m_file_size) & 3) ;

           if (m_file_size > Max_Debug_File_Size)
           {
             CDMW_ERROR(Cdmw::Logging::Logger::Get_Logger("CDMW.TRC.RecordingDaemon"),
                        "Max_Debug_File_Size raised !!!\n") ;
             // Output file full
             CORBA::string_free (m_file_name) ;
             m_file_name = 0 ;
             m_file_size = 0 ;
             return ;
           }
           CDMW_LOG(debugplugin_logger, 
                    CYCLIC_DEBUGPLUGIN_LOG_LEVEL, 
                    "=== recordoperation(5)");

           octet quad[4] ;
           // Substraction of header size (12)
           int size = req_size - 12 ;
           size += ((-size) & 3) ;

           // Write the GIOP header (but size)
           ::write (m_fd, request->data, 8) ;

           // Write size
           ::write (m_fd, &size, 4) ;

           // connection channel
           ::write (m_fd, &request->channel, 12) ;

           // date
           ::write (m_fd, &request->date, 8) ;

           // Locate request as a whole
           ::write (m_fd, request->data + 12, request->header.length) ;

           *(unsigned int *)quad = 0 ;
           ::write (m_fd, quad, ((-size) & 3)) ; //align to 4

           // Now the reply
           //--------------
           if (reply == 0)
           {
             ::write (m_fd, NULL_REPLY, 12);
           }
           else
           {
             size = rep_size - 12 ; // substraction of header (12)

             CDMW_LOG(debugplugin_logger, 
                      CYCLIC_DEBUGPLUGIN_LOG_LEVEL, 
                      "=== recordoperation(6)");         

             // Write the GIOP header (but size)
             ::write (m_fd, reply->data, 8) ;

             // Write size
             ::write (m_fd, &size, 4) ;

             // date
             ::write (m_fd, &reply->date, 8) ;

             // Locate reply as a whole
             ::write (m_fd, reply->data + 12, reply->header.length) ;

             *(unsigned int *)quad = 0 ;
             ::write (m_fd, quad, ((-size) & 3)) ; //align to 4
           }
         }

         CDMW_LOG(debugplugin_logger, 
          CYCLIC_DEBUGPLUGIN_LOG_LEVEL, 
          "=== recordoperation(end)");
      }

      /**
       *Purpose: initialize the plugin
       *<p> .
       *
       */
      void DebugPlugin::initialize (CORBA::ORB_ptr orb)
            throw()
      {
         // Nothing to do
      }

      /**
       *Purpose: flush the contents of the plugin
       *<p> .
       *
       */
      void DebugPlugin::finalize ()
            throw()
      {
         if (m_file_name != 0)
         {
            ::close (m_fd) ;
            CORBA::string_free (m_file_name) ;
            m_file_name = 0 ;
            m_fd = - 1 ;
         }
      }

   }
}
