/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#include <fstream>
#include <iostream>

#include <Foundation/logging/LogManager.hpp>
#include <Foundation/logging/Logger.hpp>
#include "TraceAndPerf/recordinglibrary/Recording_Default_Parameters.hpp"
#include "TraceAndPerf/recordinglibrary/InterfaceFilter.hpp"

namespace Cdmw
{
   namespace Recording
   {

      Cdmw::Logging::Logger* interfacefilter_logger;
      const char C_SEPARATOR = ' ' ;
      const short LENGTH_MAX = 1000 ;
      InterfaceFilter::InterfaceFilter (const char * filename) throw()
      {
         interfacefilter_logger = Cdmw::Logging::Logger::Get_Logger("CDMW.TRC.RecordingDaemon");
         std::ifstream a_file ( filename );
         char str[LENGTH_MAX];

         if ( !a_file.is_open() ) {
            // The file could not be opened
            m_interface_list.clear();
            CDMW_INFO(interfacefilter_logger, 
                      "InterfaceFilter : No interface list defined - ALL interfaces will be recorded");
         }
         else {
            std::string my_str;
            while (!a_file.eof())
            {
               //Read one string from the file and add it to the
               //interface list
               a_file.getline(str, LENGTH_MAX);
               my_str = str;
               CDMW_LOG(interfacefilter_logger, 
                        INTERFACE_FILTER_LOG_LEVEL,
                        "=== Adding interface : " << my_str);
               addInterface (my_str);
            }
            a_file.close();
            finalizeMap ();
         }   
      }

      InterfaceFilter::~InterfaceFilter () throw()
      {
         m_interface_list.clear();
         m_children_list.clear();
      }

      /**
       *Purpose: 
       *<p> checks "if an object in the Object Map
       * implements one of the stored interfaces then flag
       * it to be recorded".
       *
       */
      bool InterfaceFilter::checkInterface (const char * object_id)
          throw()
      {
        // if interface list is empty, all interfaces will be recorded
        if (m_interface_list.empty())
        {
           CDMW_LOG(interfacefilter_logger, 
                    CYCLIC_INTERFACE_FILTER_LOG_LEVEL,
                    "=== Interfaces list empty : record everything");
           return true;
       }
        
        std::string rep_id = object_id;
        if (m_interface_list.find(rep_id) == m_interface_list.end())
        {
          // interface not found
           CDMW_LOG(interfacefilter_logger, 
                    CYCLIC_INTERFACE_FILTER_LOG_LEVEL,
                    "=== Interface : " << object_id << " not found");
          return false;
        }
        InterfaceFilter::Flag_t flag = m_interface_list[rep_id];
        CDMW_LOG(interfacefilter_logger, 
                 CYCLIC_INTERFACE_FILTER_LOG_LEVEL,
                 "=== For interface : " << rep_id << " / recording ownflag=" << flag.ownflag << ", counter=" << flag.counter);
        return (flag.counter > 0);
      }

      /**
       *Purpose:
       *<p> suppress blanks at the head and at
       * the tail of a string
       *
       */
      inline std::string suppr_head_tail_blanks (std::string & str)
      {
         std::string l_str = str;
         int each = 0;
         int pos1 = 0;
         int str_len = str.length();
         if (str_len == 0)
           return str;
         while ((each < str_len) && ((str[each] == ' ') || (str[each] == TAB_CHARACTER)))
         {
            each ++;
         }
         pos1 = each;
         each = str_len;
         while ((each > pos1) && ((str[each - 1] == ' ') || (str[each - 1] == TAB_CHARACTER)))
         {
            each --;
         }
         l_str = str.substr(pos1, each - pos1);
         return l_str;        
      }

      /**
       *Purpose:
       *<p> put a child only once in the children list
       *
       */
      void InterfaceFilter::addChild (std::string const & parent, std::string const & child)
      {
        if ((parent != "TRUE") && (parent != "FALSE"))
        {
          bool found = false;
          for (size_t i = 0; i < m_children_list[parent].size(); i++)
          {
            if (m_children_list[parent][i] == child)
            {
              found = true;
              break;
            }
          }
          if (!found)
          {
            // parent_str not in children list yet
            m_children_list[parent].push_back(child);
          }
        }
        
      }
      
      /**
       *Purpose:
       *<p> put the rep_id as a child of its direct ancestor, and
       *return a flag struct
       *
       */
      bool InterfaceFilter::get_flag (std::string const & rep_id, std::string const & str)
      {
         std::string l_str = str;        
         unsigned int pos = l_str.find(C_SEPARATOR, 0);

         //  put the current interface in the children list
         // of each of its parents
         CDMW_LOG(interfacefilter_logger, 
                  CYCLIC_INTERFACE_FILTER_LOG_LEVEL,
                  "=== get_flag, rep_id :" << rep_id << " / parents :" << str );
         while (pos != std::string::npos)
         {
           std::string parent_str = l_str.substr(0, pos);
           addChild (parent_str, rep_id);
           
           int len = l_str.length() - pos;
           l_str = l_str.substr(pos + 1, len);
           l_str = suppr_head_tail_blanks (l_str);
           pos = l_str.find(C_SEPARATOR, 0);
         }
         addChild (l_str, rep_id);
         return (l_str != "FALSE");
      }

      /**
       *Purpose:
       *<p> recursively increment children activation counter
       *
       */
      void InterfaceFilter::incr_activation_counter (std::string & intf0, std::string & intf)
      {
        size_t nb_children = (m_children_list[intf]).size();
         for (size_t i = 0; i < nb_children; i++)
        {
          std::string child_str = m_children_list[intf][i];
          m_interface_list [child_str].counter++;
          // treat cycle
          CDMW_LOG(interfacefilter_logger, 
                   INTERFACE_FILTER_LOG_LEVEL,
                   "=== child_str / intf0 (parent) =" << child_str << " /" << intf0);
          if (child_str == intf0)
          {
            CDMW_LOG(interfacefilter_logger, 
                     INTERFACE_FILTER_LOG_LEVEL,
                     "!!! cyclic inheritance ");
            return;
          }
            incr_activation_counter (intf0, child_str);
        }        
      }
      
      /**
       *Purpose:
       *<p> recursively decrement children activation counter
       *
       */
      void InterfaceFilter::decr_activation_counter (std::string & intf0, std::string & intf)
      {
        size_t nb_children = (m_children_list[intf]).size();
        for (size_t i = 0; i < nb_children; i++)
        {
          std::string child_str = m_children_list[intf][i];
          m_interface_list [child_str].counter--;
          // treat cycle
          CDMW_LOG(interfacefilter_logger, 
                   INTERFACE_FILTER_LOG_LEVEL,
                   "=== child_str / intf0 (parent)=" << child_str << " /" << intf0);          
          if (child_str == intf0)
          {
            CDMW_LOG(interfacefilter_logger, 
                     INTERFACE_FILTER_LOG_LEVEL,
                     "!!! cyclic inheritance ");
            return;
          }
          decr_activation_counter (intf0, child_str);
        }        
      }

      /**
       *Purpose:
       *<p> selects for recording the requested interface of the interface store.
       *
       */
      InterfaceFilter::Status InterfaceFilter::selectInterface
            (std::string & intf)
            throw() 
      {
         //unsigned int pos;
         //std::string sub_str;
         std::string key_str;
         //std::string parent_str;

         key_str = suppr_head_tail_blanks(intf);
         if (key_str.length() == 0)
           return InterfaceFilter::INTF_INVALID;
 
         if (m_interface_list[key_str].ownflag)
         {
           CDMW_LOG(interfacefilter_logger, 
                    INTERFACE_FILTER_LOG_LEVEL,
                    "!!! " << key_str << " already selected");
           return InterfaceFilter::INTF_ALREADY_ACTIVE;
         }
         // modify own flag in m_interface_list and increment
         // its own activation counter
         m_interface_list[key_str].ownflag = true;
         m_interface_list[key_str].counter++;

         return InterfaceFilter::INTF_ADDED;
      }

      /**
       *Purpose:
       *<p> selects for recording the requested interfaces of the interface store.
       *
       */
      InterfaceFilter::Status InterfaceFilter::selectInterfaces
            (CdmwRecording::String_List & list)
            throw() 
      {
         std::string intf;
         int len = list.length();         
         for (int each = 0 ; each < len ; each ++)
         {
            intf = list[each];
            selectInterface (intf);
         }
         return INTF_OK;
      }

      /**
       *Purpose:
       *<p> removes the requested interface from the interface store.
       *
       */
      InterfaceFilter::Status InterfaceFilter::removeInterface
          (std::string & intf)
          throw() 
      {
        std::string key_str = suppr_head_tail_blanks(intf);
        if (key_str.length() == 0)
        {
          return InterfaceFilter::INTF_INVALID;
        }

        // Interface List : find if intf exists (Unit_Test : 010)
        // because if we access to m_interface_list[key_str] with
        // a keystr which has not been added before, m_interface_list.size
        // will be incremented => Unit_Test 010 failed !!!
        map<std::string, Flag_t>::iterator myIntfIterator;
        for(myIntfIterator = m_interface_list.begin(); 
            myIntfIterator != m_interface_list.end();
            myIntfIterator++)
        {
          if (key_str == myIntfIterator->first)
          {
            if (!m_interface_list[key_str].ownflag)
            {
              CDMW_LOG(interfacefilter_logger, 
                       INTERFACE_FILTER_LOG_LEVEL,
                       "!!! " << key_str << " already removed");
              return InterfaceFilter::INTF_ALREADY_REMOVED;
            }
            // modify own flag in m_interface_list and increment
            // its own activation counter
            m_interface_list[key_str].ownflag = false;
            m_interface_list[key_str].counter--;

            // decrement children activation counter
            decr_activation_counter (key_str, key_str);
            return InterfaceFilter::INTF_REMOVED;
          };
        }
        CDMW_LOG(interfacefilter_logger, 
                 INTERFACE_FILTER_LOG_LEVEL,
                 "!!! " << key_str << " has never been selected");
        return InterfaceFilter::INTF_NEVER_SELECTED;
      }

     /**
       *Purpose:
       *<p> removes the requested interfaces from the interface store.
       *
       */
      InterfaceFilter::Status InterfaceFilter::removeInterfaces
            (CdmwRecording::String_List & list)
            throw() 
      {
         std::string intf;
         int len = list.length();

         for (int each = 0 ; each < len ; each ++)
         {
            intf = list[each];
            removeInterface(intf);
         }
         return INTF_OK;
      }

      /**
       *Purpose:
       *<p> clear the interface store.
       *
       */
      InterfaceFilter::Status InterfaceFilter::removeAllInterfaces ()
          throw() 
      {
        map<std::string, Flag_t>::iterator myIntfMapIterator;
        std::string key_str;

        for(myIntfMapIterator = m_interface_list.begin(); 
            myIntfMapIterator != m_interface_list.end();
            myIntfMapIterator++)
        {
          key_str = myIntfMapIterator->first;
          removeInterface(key_str);
        }
        return INTF_OK;
      }

      /**
       *Purpose:
       *<p> adds the requested interface to the initial interface store.
       *
       */
      InterfaceFilter::Status InterfaceFilter::addInterface
          (std::string & intf)
          throw() 
      {
        unsigned int pos;
        std::string sub_str;
        std::string key_str;
        std::string parent_str;

        std::string l_str = suppr_head_tail_blanks(intf);
        if (l_str.length() == 0)
          return InterfaceFilter::INTF_INVALID;
        pos = l_str.find(C_SEPARATOR, 0);

        if (pos != std::string::npos)
        {
          // the interface
          sub_str = l_str.substr(0, pos);
          key_str = sub_str;
          // direct parents of the interface
          sub_str = l_str.substr(pos + 1, l_str.length());
          parent_str = suppr_head_tail_blanks(sub_str);
         
          // put current interface as a child of each of its direct
          // parents, and initialises its own flag from configuration
          // file - default own flag = true (if not specified)
          if (get_flag (key_str, parent_str))
          {
            selectInterface (key_str);
          }
          else
          {
            // reset flags
            m_interface_list[key_str].ownflag = false;
            // m_interface_list[key_str].counter = 0;
          }
        }
        else
        {
          // no parent
          selectInterface (l_str);
        }

        return InterfaceFilter::INTF_ADDED;
      }


      /**
       *Purpose:
       *<p> adds the interface to the interface store.
       *
       */
      InterfaceFilter::Status InterfaceFilter::addInterfaces (CdmwRecording::String_List & list)
          throw()
      {
        std::string intf;
        int len = list.length();

        for (int each = 0 ; each < len ; each ++)
        {
          intf = list[each];
          addInterface(intf);
        }
        finalizeMap ();
        return INTF_OK;
      }

      /**
       *Purpose:
       *<p> map iterator : finalize interface lists
       *
       */
      void InterfaceFilter::finalizeMap ()
          throw()
      {

        // Interface List
        map<std::string, Flag_t>::iterator myIntfIterator;
        std::string key_str;
        InterfaceFilter::Flag_t flag;

        for(myIntfIterator = m_interface_list.begin(); 
            myIntfIterator != m_interface_list.end();
            myIntfIterator++)
        {
          key_str = myIntfIterator->first;
          flag = myIntfIterator->second;
          if (flag.counter > 0)
          {
            // recursively increment children activation counter
            incr_activation_counter(key_str, key_str);
          }
        }
      }

      /**
       *Purpose:
       *<p> return the interface map size
       *
       */
      int InterfaceFilter::list_size()
      {
  return m_interface_list.size();
      }      

      /**
       *Purpose:
       *<p> map iterator.
       *
       */
      void InterfaceFilter::mapLoop (Display print, Display_flag print_flag)
            throw()
      {

        // Interface List
        map<std::string, Flag_t>::iterator myIntfIterator;
        std::string key_str;
        InterfaceFilter::Flag_t flag;

        for(myIntfIterator = m_interface_list.begin(); 
            myIntfIterator != m_interface_list.end();
            myIntfIterator++)
        {
          key_str = myIntfIterator->first;
          flag = myIntfIterator->second;
          print_flag (key_str, flag);
        }
        // Children list
         map<std::string, Intf_vector_t>::iterator myIntfMapIterator;
         InterfaceFilter::Intf_vector_t children;

         for(myIntfMapIterator = m_children_list.begin(); 
             myIntfMapIterator != m_children_list.end();
             myIntfMapIterator++)
         {
            key_str = myIntfMapIterator->first;
            children = myIntfMapIterator->second;
            for (size_t i = 0; i < children.size(); i++)
            {
               print (key_str, children[i], i);
            }
         }
      }
   }
} 
