/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#include <stdio.h>
#include <iostream>

#include <Foundation/logging/LogManager.hpp>
#include <Foundation/logging/Logger.hpp>
#include "Foundation/common/Exception.hpp"
#include <Foundation/logging/FunctionLogger.hpp>
#include "TraceAndPerf/recordingidl/CdmwRecordingData.stub.hpp"
#include "TraceAndPerf/recordingidl/CdmwRecordingDebug.stub.hpp"
#include "TraceAndPerf/recordinglibrary/MessageIdentifier.hpp"
#include "TraceAndPerf/recordinglibrary/Swapping.hpp"
#include "TraceAndPerf/recordinglibrary/Recording_Default_Parameters.hpp"

namespace Cdmw
{
  namespace Recording
  {
     Cdmw::Logging::Logger* msg_identifier_logger;
     /**
     *Purpose:
     *<p> This local class is the workhorse of the message processing
     */
    class MessageIdentifier::Worker : public virtual OsSupport::Thread
    {
      public :
        Worker (MessageIdentifier & msgIdentifier) throw(
                OutOfMemoryException,
                InternalErrorException,
                BadParameterException)
            : OsSupport::Thread()
            , m_id(0)
            , m_control(msgIdentifier.m_mutex)
            , m_msgIdentifier(msgIdentifier)
        {
        }

        virtual ~Worker () throw()
        {
        }

        virtual Status init() throw()
        {
          m_id = start() ;

          return OK ;
        }

        virtual void run() throw()
        {
          while (! m_msgIdentifier.m_terminated)
          {
            bool do_message = false ;
            {
          CDMW_LOG(msg_identifier_logger,
                       CYCLIC_MSG_IDENTIFIER_LOG_LEVEL,
                       "=== MessageIdentifier.run(0)");
              CDMW_MUTEX_GUARD(m_msgIdentifier.m_mutex) ;
          CDMW_LOG(msg_identifier_logger,
                       CYCLIC_MSG_IDENTIFIER_LOG_LEVEL,
                       "=== MessageIdentifier.run(1)");

              if (m_msgIdentifier.m_terminated)
              {
                break ;
              }
              else if (m_msgIdentifier.m_started)
              {
                if (m_msgIdentifier.m_stopped)
                {
                  m_msgIdentifier.m_stopped = false ;
                  m_control.broadcast() ;
                }
                do_message = true ;
              }
              else
              {
                if (m_msgIdentifier.m_stopped)
                {
                  m_control.wait() ;
                }
                else
                {
                  m_msgIdentifier.m_stopped = true ;
                  m_control.broadcast() ;
                }
              }
            }
            if (do_message)
            {
              handle_message() ;
            }
          }
        }

        /**
         * Purpose: pop the messages from the packet Store and
         * treat them
         *
         */
        virtual void handle_message() throw()
        {
          Cdmw::Recording::Message * message ;

      CDMW_LOG(msg_identifier_logger,
                   CYCLIC_MSG_IDENTIFIER_LOG_LEVEL,
                   "=== MessageIdentifier.handle_message (0)");
          // Pop the message from the packet stored
          message = m_msgIdentifier.m_packet_store.popMessage () ;

          if (message == 0)
          {
           CDMW_LOG(msg_identifier_logger,
                     CYCLIC_MSG_IDENTIFIER_LOG_LEVEL,
                     "=== MessageIdentifier.handle_message (orphan)(1)");
            m_msgIdentifier.m_request_pairing.checkOrphanRequests();
            return ;
          }

          if (message->has_missing_bytes)
          {
              CDMW_ERROR(msg_identifier_logger,
                       "GIOP Message has missing bytes, skip it");
              m_msgIdentifier.m_packet_store.freeMessage(message) ;
              return ;
          }

          // REQ-SRS-TRC-311 : check giop version is >= giop 1.2
          int GIOP_version = message->header.version.major * 256 + message->header.version.minor ;
          if (GIOP_version < GIOP_1_2)
          {
             CDMW_WARN(msg_identifier_logger,
                        "!!! GIOP version (<1.2) :" << message->header.version.major << '.' << message->header.version.minor );
             m_msgIdentifier.m_packet_store.freeMessage (message) ;
             return;
          }

          CDMW_LOG(msg_identifier_logger,
                   CYCLIC_MSG_IDENTIFIER_LOG_LEVEL,
                   "=== MessageIdentifier.handle_message (checkmessagetype)(2)");
          // if oneway request storeRequest directly sends the message
          if (message->header.message_type == CdmwRecording::Request)
          {
            // check if an object key must have its operations recorded
            if (! m_msgIdentifier.m_object_map.checkKey(message))
             {
            CDMW_LOG(msg_identifier_logger,
                         CYCLIC_MSG_IDENTIFIER_LOG_LEVEL,
                         "*** MessageIdentifier.handle_message : Object not registered (3)");
                m_msgIdentifier.m_packet_store.freeMessage (message) ;
                return;
             }
             // check if the operation in the message is to be recorded
             std::string operation = (char*)(message->data + message->operation_Offset);
             if (! m_msgIdentifier.m_operation_map.checkOperation(operation))
             {
            CDMW_LOG(msg_identifier_logger,
                         CYCLIC_MSG_IDENTIFIER_LOG_LEVEL,
                         "*** MessageIdentifier.handle_message : Operation not " <<operation<<" registered (4)");
                m_msgIdentifier.m_packet_store.freeMessage (message) ;
                return;
             }

         CDMW_LOG(msg_identifier_logger,
                      CYCLIC_MSG_IDENTIFIER_LOG_LEVEL,
                      "=== MessageIdentifier.handle_message : storing request (5)");
             m_msgIdentifier.m_request_pairing.storeRequest(message);
          }
          else if (message->header.message_type == CdmwRecording::Reply)
          {
         CDMW_LOG(msg_identifier_logger,
                      CYCLIC_MSG_IDENTIFIER_LOG_LEVEL,
                      "=== MessageIdentifier.handle_message : treating a reply (6)");
             // Set the request id
             octet quad [4] ;
             Swapping::Swap4 (quad, message->data + 12, message->need_swap) ;
             message->request_id = *(unsigned int *) quad ;
             m_msgIdentifier.m_request_pairing.checkReply(message);
          }
          else if (message->header.message_type == CdmwRecording::Fragment)
          {
        CDMW_LOG(msg_identifier_logger,
                     CYCLIC_MSG_IDENTIFIER_LOG_LEVEL,
                     "!!! MessageIdentifier.handle_message : treating a fragment (7)");
            // CDMW_INFO(Cdmw::Logging::Logger::Get_Logger("CDMW.TRC.RecordingDaemon"), "Fragment detected");
            // Set the request id
            octet quad [4] ;
            Swapping::Swap4 (quad, message->data + 12, message->need_swap) ;
            message->request_id = *(unsigned int *) quad ;
            m_msgIdentifier.m_request_pairing.storeFragment(message);
          }
          else if (message->header.message_type == CdmwRecording::LocateRequest)
          {
            CDMW_LOG(msg_identifier_logger,
                     CYCLIC_MSG_IDENTIFIER_LOG_LEVEL,
                     "=== MessageIdentifier.handle_message : treating a LocateRequest (8)");
            // Set the request id
            octet quad [4] ;
            Swapping::Swap4 (quad, message->data + 12, message->need_swap) ;
            message->request_id = *(unsigned int *) quad ;
            m_msgIdentifier.m_request_pairing.storeRequest(message);
          }
          else if (message->header.message_type == CdmwRecording::LocateReply)
          {
            CDMW_LOG(msg_identifier_logger,
                     CYCLIC_MSG_IDENTIFIER_LOG_LEVEL,
                     "=== MessageIdentifier.handle_message : treating a LocateReply (9)");
            // Set the request id
            octet quad [4] ;
            Swapping::Swap4 (quad, message->data + 12, message->need_swap) ;
            message->request_id = *(unsigned int *) quad ;
            m_msgIdentifier.m_request_pairing.checkReply(message);
          }
          else
          {
        CDMW_LOG(msg_identifier_logger,
                     CYCLIC_MSG_IDENTIFIER_LOG_LEVEL,
                      "!!! MessageIdentifier.handle_message : Service message detected (10)");
          }
        }

        // Worker members
        OsSupport::Thread::ThreadId m_id ;
        OsSupport::Condition m_control ;

      private :
        MessageIdentifier & m_msgIdentifier ;
    } ;// end class local worker


    // Constructor
    MessageIdentifier::MessageIdentifier (PacketStore & store,
                                          ObjectMap & object_map,
                                          InterfaceFilter & interface_filter,
                                          OperationMap & operation_map,
                                          RequestPairing & pairing) throw()
        : m_mutex()
        , m_started (false)
        , m_stopped (true)
        , m_terminated (false)
        , m_packet_store(store)
        , m_object_map(object_map)
        , m_interface_filter(interface_filter)
        , m_operation_map(operation_map)
        , m_request_pairing(pairing)
        , m_worker (0)
    {
       msg_identifier_logger = Cdmw::Logging::Logger::Get_Logger("CDMW.TRC.RecordingDaemon");
    }

    // Destructor
    MessageIdentifier::~MessageIdentifier () throw()
    {
      if (! m_terminated)
      {
        terminate() ;
      }

    }

    MessageIdentifier::Status MessageIdentifier::start () throw()
    {
      if ((m_worker == 0) || (m_started))
      {
        return INVALID ;
      }

      CDMW_MUTEX_GUARD(m_mutex) ;

      // Wake up the worker
      m_started = true ;
      m_worker ->m_control.broadcast() ;

      // Wait for the worker to start working
      while ( m_stopped )
      {
        m_worker ->m_control.wait() ;
      }
      return OK ;
    }

    MessageIdentifier::Status MessageIdentifier::stop () throw()
    {
      if  ((m_worker == 0) || (m_stopped))
      {
        return INVALID ;
      }

      m_packet_store.stop() ;

      CDMW_MUTEX_GUARD(m_mutex) ;

      m_started = false ;

      m_worker ->m_control.broadcast() ;

      // Wait for the worker to stop working
      while (! m_stopped )
      {
        m_worker ->m_control.wait() ;
      }
      return OK ;
    }


    MessageIdentifier::Status MessageIdentifier::terminate () throw()
    {
      if  ((m_worker == 0) || (m_terminated))
      {
        return INVALID ;
      }
      m_packet_store.stop() ;

      {
        CDMW_MUTEX_GUARD(m_mutex) ;

        m_started = false ;
        m_terminated = true ;

        m_worker ->m_control.broadcast() ;
      }

      m_worker ->join() ;
      delete m_worker ;
      m_worker = 0 ;
      return OK ;
    }


    MessageIdentifier::Status MessageIdentifier::run () throw()
    {
      if (m_worker != 0)
      {
        return INVALID;
      }
      m_worker = new Worker(*this) ;
      return m_worker->init() ;
    }
  }
}

