/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

#include <Foundation/logging/LogManager.hpp>
#include <Foundation/logging/Logger.hpp>
#include "Foundation/orbsupport/OrbSupport.hpp"
#include "TraceAndPerf/recordinglibrary/ObjectMap.hpp"
#include "TraceAndPerf/recordinglibrary/Swapping.hpp"
#include "TraceAndPerf/recordinglibrary/Recording_Default_Parameters.hpp"

namespace
{
        static const char * NOT_YET_KNOWN = "Not yet known";
        static const char * CORBA_OBJECT = "IDL:omg.org/CORBA/Object:1.0";
}

namespace Cdmw
{
   namespace Recording
   {

      Cdmw::Logging::Logger* objectmap_logger;
      ObjectMap::ObjectMap (InterfaceFilter & filter, NetworkSniffer & sniffer) throw()
            : m_intf_filter (filter)
            , m_sniffer (sniffer)
      {
        objectmap_logger = Cdmw::Logging::Logger::Get_Logger("CDMW.TRC.RecordingDaemon");
      }


      ObjectMap::~ObjectMap () throw()
      {
         KeyMap::iterator key_iter = m_keys.begin() ;

         // Iterate the key map
         for (; key_iter != m_keys.end(); key_iter++)
         {
            // Nullify its contents
            CORBA::string_free( (*key_iter).second.server_id );
            (*key_iter).second.server_id = 0 ;
            (*key_iter).second.rep_id = 0 ;
            (*key_iter).second.info = 0 ;
         }
         // Clear it
         m_keys.clear() ;

         RIdMap::iterator rid_iter = m_objectRIds.begin() ;
         // Iterate the repositoryId map
         for (; rid_iter != m_objectRIds.end(); rid_iter++)
         {
            // Delete its contents
            delete (*rid_iter).second ;
            (*rid_iter).second = 0 ;
         }

         // Clear it
         m_objectRIds.clear() ;
      }

      bool ObjectMap::checkInterfaces() throw()
      {
         // Iterate the repositoryId map
         RIdMap::iterator iter = m_objectRIds.begin() ;
         // Iterate the repositoryId map
         for (; iter != m_objectRIds.end(); iter++)
         {
            // Check each interface and update its recording info
            (*iter).second->toRecord = m_intf_filter.checkInterface((*iter).first.c_str()) ;
            CDMW_LOG(objectmap_logger, 
                     CYCLIC_OBJECTMAP_FILTER_LOG_LEVEL,
                     "=== ObjectMap::checkInterfaces map_size= " << m_objectRIds.size() << " / Key = " << iter->first
                     << " / toRecord=" << iter->second->toRecord);
         }

         return true ;
      }

      static octet HEXA_VALUE(char c) 
      {
         if (c <='9')  return  c - '0';
         if (c <='F')  return  c - 'A' + 10;
         else          return  c - 'a' + 10;
      };

      void ObjectMap::addObject (const char * server_id, CORBA::ORB * orb, CORBA::Object_var & ref) throw()
      {
         RIdInfo * info ;

         // CDO-070330-PRS-1-567 : #61
         // Get the repositoryId of the interface
         CORBA::String_var id = CORBA::string_dup (OrbSupport::OrbSupport::get_repository_id_from_ior
                               (orb, ref.in()).c_str()) ;
         CDMW_LOG(objectmap_logger, 
                  CYCLIC_OBJECTMAP_FILTER_LOG_LEVEL, "=== ObjectMap::addObject : Key = " << id);
         
         // Get its recording info
         RIdMap::iterator iter = m_objectRIds.find(id.in()) ;

         
         // If null, create a new one and update it
         if (iter == m_objectRIds.end())
         {
            info = new RIdInfo() ;
            info-> toRecord = m_intf_filter.checkInterface(id) ;
            m_objectRIds[id.in()] = info ;
            CDMW_LOG(objectmap_logger, 
                     CYCLIC_OBJECTMAP_FILTER_LOG_LEVEL,
                     "=== ObjectMap size=" << m_objectRIds.size() << " / Key=" << id
                     << " / toRecord=" << info->toRecord);
         }
         else
         {
            id = CORBA::string_dup ((*iter).first.c_str()) ;
            //id = (*iter).first ;
            info = (*iter).second ;
            CDMW_LOG(objectmap_logger, 
                     CYCLIC_OBJECTMAP_FILTER_LOG_LEVEL,
                     "=== ObjectMap size=" << m_objectRIds.size() << " / Key=" << id
                     << " already in the map / toRecord=" << info->toRecord);
         }

         // Get the stringified object reference
         char * ref_string = orb->object_to_string (ref.in()) ;

         // Transform to bytes
         octet * ior = new octet[strlen (ref_string) / 2 - 2] ;  // ignore prefix IDL:
         int index = 0 ;
         for (size_t each = 12 ; each < strlen (ref_string) ; each += 2)
         {
            ior [index++] = HEXA_VALUE(ref_string[each]) * 16 + HEXA_VALUE(ref_string[each + 1]) ;
         }
         int Max = index ;

         CORBA::string_free (ref_string) ;
         ref_string = 0 ;

         // Skip repository id
         index = 0;
         int size = *(int *) ior ;
         index = 4 + size ;

         // Align to 4
         index += (-index) & 3 ;

         // Skip number of profiles
         index += 4 ;

         // Walk through the profiles to get their object key
         while (index < Max)
         {
            // Profile Tag
            int tag = *(int *) (ior + index) ;
            index += 4 ;

            size = *(int *) (ior + index) ;
            index += 4 ;
            int next = index + size ;
            

            if (tag == 0) // IIOP profile
            {
               bool swap = needSwap (ior[index]) ;
               octet quad [4] ;

               // Skip endian, version and padding
               index += 4 ;

               // Skip host
               Swapping::Swap4 (quad, ior + index, swap) ;
               size = *(int *) quad ;
               index += 4 + size ;

               // Align to 2 and get port
               index += (-index) & 1;
               unsigned short port ;
               Swapping::Swap2 ((octet *) &port, ior + index, swap) ;
               m_sniffer.addPort (port) ;

               // Skip port
               index += 2 ;

               // Align to 4
               index += (-index) & 3 ;

               // Here is the object key
               Swapping::Swap4 (quad, ior + index, swap) ;
               size = *(int *) quad ;
               index += 4 ;
               OctetArray obj_key ;
               obj_key.bytes = new octet [size] ;
               obj_key.length = size ;
               memcpy (obj_key.bytes, ior + index, size) ;

               ObjectKeyInfo objk_info ;
               objk_info.server_id = CORBA::string_dup (server_id) ;
               objk_info.rep_id = CORBA::string_dup (id) ;
               objk_info.info = info ;
               // REQ-SRS-TRC-310
               // Associate each object key with the recording info in the map
               m_keys [obj_key] = objk_info ;
            }
            else
            // REQ-SRS-TRC-310
            {
               // Ignore non IIOP profile as we cannot have a request
               // using it as we only sniff IIOP connections
            }

            index = next ;

            // Align to 4
            index += (-index) & 3 ;
         }

         delete [] ior ;         
      }

      void ObjectMap::removeObject (CORBA::ORB * orb, CORBA::Object_var & ref) throw()
      {
         char * id = 0 ;

         // Get the stringified object reference
         char * ref_string = orb->object_to_string (ref.in()) ;

         CDMW_LOG(objectmap_logger, 
                  OBJECTMAP_FILTER_LOG_LEVEL, "=== ObjectMap : ref_string = " << ref_string);

         // Transform to bytes
         octet * ior = new octet[strlen (ref_string) / 2 - 2] ;  // ignore prefix IDL:
         int index = 0 ;
         for (size_t each = 12 ; each < strlen (ref_string) ; each += 2)
         {
            ior [index++] = HEXA_VALUE(ref_string[each]) * 16 + HEXA_VALUE(ref_string[each + 1]) ;
         }
         int Max = index ;

         CORBA::string_free (ref_string) ;
         ref_string = 0 ;

         // Skip repository id
         index = 0;
         int size = *(int *) ior ;
         index = 4 + size ;

         // Align to 4
         index += (-index) & 3 ;

         // Skip number of profiles
         index += 4 ;

         // Walk through the profiles to get their object key
         while (index < Max)
         {
            // Profile Tag
            int tag = *(int *) (ior + index) ;
            index += 4 ;

            size = *(int *) (ior + index) ;
            index += 4 ;
            int next = index + size ;


            if (tag == 0) // IIOP profile
            {
               bool swap = needSwap (ior[index]) ;
               octet quad [4] ;

               // Skip endian, version and padding
               index += 4 ;

               // Skip host
               Swapping::Swap4 (quad, ior + index, swap) ;
               size = *(int *) quad ;
               index += 4 + size ;

               // Align to 2 and get port (PCR-1523)
               index += (-index) & 1;
               unsigned short port ;
               Swapping::Swap2 ((octet *) &port, ior + index, swap) ;

               // Skip port
               index += 2 ;

               // Align to 4
               index += (-index) & 3 ;

               // Here is the object key
               Swapping::Swap4 (quad, ior + index, swap) ;
               size = *(int *) quad ;
               index += 4 ;
               OctetArray obj_key ;
               obj_key.bytes = new octet [size] ;
               obj_key.length = size ;
               memcpy (obj_key.bytes, ior + index, size) ;

               ObjectKeyInfo objk_info = m_keys [obj_key] ;
               CORBA::string_free (objk_info.server_id) ;
               objk_info.server_id = 0 ;
               id = objk_info.rep_id ;
               CDMW_LOG(objectmap_logger, 
                        OBJECTMAP_FILTER_LOG_LEVEL, "=== ObjectMap : id  = " << id);

               objk_info.rep_id = 0 ;

               // PCR-1523 : only remove port if object not already removed
               KeyMap::iterator iter = m_keys.find(obj_key) ;
               // If null, ignore it
               if (iter != m_keys.end() && (*iter).second.server_id )
               {
                  m_keys.erase (obj_key);
                  m_sniffer.removePort (port) ;
               }
            }
            else
            {
               // Ignore non IIOP profile as we cannot have a request
               // using it as we only sniff IIOP connections
            }

            CORBA::string_free (id) ;

            index = next ;

            // Align to 4
            index += (-index) & 3 ;
         }

         delete [] ior ;         

      }

      bool ObjectMap::checkKey (Message * msg) throw()
      {
         int index = 12 ;
         unsigned int size ;
         volatile unsigned int quad_num ;
         octet * quad = (octet *) &quad_num ;

         // Get the object key
         int GIOP_version = msg->data [4] * 256 + msg->data [5] ;

         if (GIOP_version < GIOP_1_2)
         {
           // REQ-SRS-TRC-311
           return false ;
         }
         else
         {
            // GIOP 1.2 and above
            // Get request serial
            Swapping::Swap4 (quad, msg->data + index, msg->need_swap) ;
            msg->request_id = quad_num ;
            // Skip request serial and response flags
            index += 8 ;

            // Check addressing
            short addressing = msg->data [index] + msg->data [index + 1];
            index += 2 ;

            // Align to 4
            index += (-index) & 3 ;

            if (addressing == 0)
            {
               // Set object key offset
               Swapping::Swap4 (quad, msg->data + index, msg->need_swap) ;
               size = quad_num ;
               index += 4 ;
               msg->objectKey_Offset = index ;
               msg->objectKey_Length = size ;

               // Skip object key
               index += size + ((-size) & 3) ;
            }
            else if (addressing == 1)
            {
               // Profile

               // Skip tag
               index += 4 ;

               Swapping::Swap4 (quad, msg->data + index, msg->need_swap) ;
               size = quad_num ;
               unsigned int next = index + 4 + size ;

               // Skip size, endian and version
               index += 8 ;

               // Skip host
               Swapping::Swap4 (quad, msg->data + index, msg->need_swap) ;
               size = quad_num ;
               index += 4 + size + ((-size) & 1) ;

               // Skip TCP port
               index += 2 ;

               // Align to 4
               index += (-index) & 3 ;

               // Set object key offset
               Swapping::Swap4 (quad, msg->data + index, msg->need_swap) ;
               size = quad_num ;
               index += 4 ;
               msg->objectKey_Offset = index ;
               msg->objectKey_Length = size ;

               index = next ;

               // Align to 4
               index += (-index) & 3 ;
            }
            else if (addressing == 2)
            {
               // Full IOR
               Swapping::Swap4 (quad, msg->data + index, msg->need_swap) ;
               unsigned int number = quad_num ;
              
               // Skip number of profiles
               index += 4 ;

               bool found = false ;
               // Walk through the profiles to find an IIOP object key
               for (unsigned int each = number ; each > 0 ; each --)
               {
                  // Profile Tag
                  int tag = *(int *) (msg->data + index) ;
                  index += 4 ;

                  Swapping::Swap4 (quad, msg->data + index, msg->need_swap) ;
                  size = quad_num ;
                  index += 4 ;
                  int next = index + size ;

                  if ((! found) && (tag == 0)) // IIOP profile
                  {
                     // Skip endian, version and padding
                     index += 4 ;

                     // Skip host
                     Swapping::Swap4 (quad, msg->data + index, msg->need_swap) ;
                     size = quad_num ;
                     index += 4 + size ;

                     // Align to 2 and skip port
                     index += ((-index) & 1) + 2 ;

                     // Align to 4
                     index += (-index) & 3 ;

                     // Here is the object key
                     // Set object key offset
                     Swapping::Swap4 (quad, msg->data + index, msg->need_swap) ;
                     size = quad_num ;
                     index += 4 ;
                     msg->objectKey_Offset = index ;
                     msg->objectKey_Length = size ;
                  }

                  index = next ;

                  // Align to 4
                  index += (-index) & 3 ;
               }
            }
         }

         // Set object operation offset
         msg->operation_Offset = index + 4 ;

         if (m_keys.size() == 0)
         {
            // No object yet, record everything
            msg->server_id = NOT_YET_KNOWN ;
            msg->intf_rid = CORBA_OBJECT ;
            CDMW_LOG(objectmap_logger, 
                     CYCLIC_OBJECTMAP_FILTER_LOG_LEVEL,
                     "!!! ObjectMap is empty : record everything");

            return true ;
         }

         // Get its recording info
         OctetArray obj_key ;
         obj_key.bytes = msg->data + msg->objectKey_Offset ;
         obj_key.length = msg->objectKey_Length ;
         // REQ-SRS-TRC-310
         KeyMap::iterator iter = m_keys.find(obj_key) ;

         // If null, reject it
         if (iter == m_keys.end())
         {
            return false ;
         }

         // If known, set the server_id and the intf_rid and return its
         // info contents
         msg->server_id = (*iter).second.server_id ;
         msg->intf_rid = (*iter).second.rep_id ;

         return (*iter).second.info->toRecord ;
      }

      int ObjectMap:: size () throw()
      {
         return m_keys.size();
      }
      /**
       *Purpose:
       *<p> map iterator.
       *
       */
      void ObjectMap::mapLoop (Display print)
            throw()
      {
         std::string id;
         int i = 0;
         int map_size = m_objectRIds.size();
         RIdMap::iterator iter = m_objectRIds.begin() ;
         // Iterate the repositoryId map
         for (; iter != m_objectRIds.end(); iter++)
         {
             i++;
             id = (*iter).first;
             if (i <= map_size)
             {
                print (id, i, map_size);
             }
             else
             {
                CDMW_ERROR(objectmap_logger,
                           "*** ObjectMap : error in map index " << i << " > " << map_size << " (map size)");
                break ;
             }
         }
      }
   }
}

