/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#include <fstream>
#include <iostream>

#include <vector>
#include <iterator>

#include <cctype> // for toupper
#include <locale> // for isalpha, isdigit
#include <string>
#include <algorithm>


#include <Foundation/logging/LogManager.hpp>
#include <Foundation/logging/Logger.hpp>
#include "TraceAndPerf/recordinglibrary/OperationMap.hpp"
#include "TraceAndPerf/recordinglibrary/Recording_Default_Parameters.hpp"

namespace
{
//    const char C_SEPARATOR = ' ';
   const unsigned short LENGTH_MAX = 1000;

   inline bool is_action_starter(char c)
   {
      return (c == '@');
   }

   inline bool is_comment_starter(char c)
   {
      return ((c == '!') || (c == '#') || (c == ';'));
   }

   inline bool is_space(char c)
   {
      return ((c == ' ') || (c == TAB_CHARACTER));
   }
   bool not_space(char c)
   { return !is_space(c);}

   inline bool is_separator(char c)
   {
      return c == '=';
   }
   inline bool not_separator(char c)
   {
      return !is_separator(c);
   }
   

   inline void to_uppercase(std::string & s) 
   {
      std::transform(s.begin(), s.end(), s.begin(), 
                     toupper);
   }
   inline bool is_underscore(char c)
   {
      return c == '_';
   }


   /**
    * Checks whether the operation name is a valid IDL Identifier
    * OMG document:
    *    Common Object Request Broker Architecture: Core Specification
    *    March 2004, Version 3.0.3 - Editorial changes
    *    formal/04-03-12
    * Section §3.2.3 Identifiers
    *   "An identifier is an arbitrarily long sequence of ASCII alphabetic, digit, 
    *   and underscore (_) characters. The first character must be an ASCII alphabetic 
    *   character. All characters are significant."
    * 
    */
   bool check_IDL_Identifier(const std::string & opname, std::string & error_msg)
   {
      int lg = opname.length();
      if (lg == 0) {
         error_msg = "Empty operation name";
         return false;
      } 
      if (!isalpha(opname[0])) {
         error_msg = "The first character must be an ASCII alphabetic character.";
         return false;
      }

      int idx = 1;
      while ((idx < lg) && (isalpha(opname[idx]) || isdigit(opname[idx]) || is_underscore(opname[idx])))
      {
         idx++;
      }
      if (idx != lg)  {
         error_msg = "Invalid character.  A valid operation name is an arbitrarily long sequence of ASCII alphabetic, "
            "digit, and underscore (_) characters. The first character must be an ASCII alphabetic character.";
         return false;
      }
      return true;

   }
}

namespace Cdmw
{
   namespace Recording
   {


      /**
       *Purpose:
       *<p> suppress blanks at the head and at
       * the tail of a string
       *
       */
      inline std::string suppr_head_tail_blanks (std::string & str)
      {
        std::string l_str = str;
        int each = 0;
        int pos1 = 0;
        int str_len = str.length();
        while ((each < str_len) && ((str[each] == ' ') || (str[each] == TAB_CHARACTER)))
        {
          each ++;
        }
        pos1 = each;
        each = str_len;
        while ((each > pos1) && ((str[each - 1] == ' ') || (str[each - 1] == TAB_CHARACTER)) )
        {
          each --;
        }
        l_str = str.substr(pos1, each - pos1);
        return l_str;        
      }
   
      inline bool trim(std::string & str)
      {
         std::string s = suppr_head_tail_blanks(str);
         str = s;
         return !str.empty();
      }


      Cdmw::Logging::Logger* operationmap_logger;


      bool
      OperationMap::explode_action_line(const std::string& line, std::string & name, std::string & value,
                                        const char* filename, size_t ln)
      {
         // skip first char '@'
         std::string arg = line.substr(1);
         std::string::const_iterator i = arg.begin();
         std::string::const_iterator end = arg.end();
         // find end of ior
         std::string::const_iterator j = std::find_if(i,end,is_separator);
         if (j == end) {
            return false;
         }
         // copy the characters in [i,j)
         std::string n;
         // copy name
         if (i != end)
            n = std::string(i,j);
         // skip '=' character
         i = std::find_if(j,end,not_separator);
         // copy value
         if (i != end) {
            name = n;
            value = std::string(i,end);
            trim(name);
            trim(value);
         } else {
            CDMW_ERROR(operationmap_logger,  
                      filename << ":" << ln << " Missing value for command '" <<
                      n << "'! Ignoring instruction ' " << line << "'.");
         }
         return true;
      }

      bool
      OperationMap::explode_operation_line(const std::string& line, 
                                           std::string &      opname,
                                           const char* filename, size_t ln)
      {
         bool result = false;
         // ===================================================
         // skip leading whitespaces
         std::string::const_iterator i = line.begin();
         std::string::const_iterator end = line.end();
         i = std::find_if(i,end,not_space);
         // find end of command
         std::string::const_iterator j = std::find_if(i,end,is_space);
         // copy the characters in [i,j)
         if (i != end) {
            opname = std::string(i,j);
            result = true;
            i = std::find_if(j,end,not_space);
            if (i != end) {
               CDMW_ERROR(operationmap_logger, 
                         filename << ":" << ln 
                         << " Invalid characters after operation '" << opname <<"'. Ignoring '"
                         << std::string(i,end) << "'.");
            }

         }
         std::string error_msg("");
         result = check_IDL_Identifier(opname, error_msg);
         if (!result) {
            CDMW_ERROR(operationmap_logger, 
                      filename << ":" << ln 
                      << " Invalid operation name '" << opname <<"'. " << error_msg);
            
         }
         return result;
      }

      OperationMap::OperationMap (const char * filename) throw()
         : m_filtering_mode(FILTERING_MODE_MATCH)
      {
         operationmap_logger = Cdmw::Logging::Logger::Get_Logger("CDMW.TRC.RecordingDaemon");
         std::ifstream a_file ( filename );

         if ( !a_file.is_open() ) {
            // The file could not be opened
            m_operation_list.clear();
            CDMW_ERROR(operationmap_logger, 
                      "Operations file [" << filename << "] could not be opened ");
         }
         else {
            //Read one string from the file and add it to the
            //operation list
            // Here is the supported format:
            //   @filter=[match]|exclude ; as a first instruction (default is match)
            //   operation-name
            // NOTA:
            // A line starting with '!', '#', or ';' is considered a comment and will
            // be ignored.
            bool action_defined = false;
            std::string s;
            size_t ln = 0; // Line number
            while(std::getline(a_file,s)) {
               ++ln;
               // Skip leading and trailing whitespaces
               if ( trim(s) && (!is_comment_starter(s[0])) )
               {
                  // Remove trailing comment
                  std::string::const_iterator beg = s.begin();
                  std::string::const_iterator end = s.end();
                  std::string::const_iterator j   = std::find_if(beg,end,is_comment_starter);
                  
                  std::string line(s);
                  // copy the characters in [beg,j)
                  if (j != end)
                     line = std::string(beg,j);
                  // Is this an action command?
                  // @filter=[match]|exclude
                  if (is_action_starter(line[0])) {
                     std::string action("");
                     std::string value("");
                     if (explode_action_line(line, action, value, filename, ln)) {
                        CDMW_DEBUG(operationmap_logger, 
                                   filename << ":" << ln << " Command '" << line << "' found in file '" 
                                   << filename << "'.");
                        trim(action); 
                        trim(value);
                        std::string action__(action);
                        std::string value__(value);
                        to_uppercase(action);
                        to_uppercase(value);
                        // FILTER = [MATCH]|EXCLUDE
                        if (action == "FILTER") {
                           if (action_defined) {
                              CDMW_WARN(operationmap_logger, 
                                        filename << ":" << ln 
                                        << " Filering action already defined! Ignoring line '" << line << "'.");
                           } else {
                              FilteringMode fmode = FILTERING_MODE_MATCH;
                              if (value.empty() || value == "MATCH")
                                 fmode = FILTERING_MODE_MATCH;
                              else if (value == "EXCLUDE")
                                 fmode = FILTERING_MODE_EXCLUDE;
                              else {
                                 CDMW_ERROR(operationmap_logger, 
                                           filename << ":" << ln 
                                           << " Wrong filering action '" << value__ << "'. Assuming '@action=match'");
                              }

                              if (fmode == FILTERING_MODE_MATCH) {
                                 CDMW_INFO(operationmap_logger, 
                                           "Filering action '" << value__ << "' in use for operations. Only operations "
                                           << "from '" << filename << "' will be recorded.");
                              } else {
                                 CDMW_INFO(operationmap_logger, 
                                           "Filering action '" << value__ << "' in use for operations. operations "
                                           << "from '" << filename << "' will not be recorded.");
                                 m_filtering_mode = FILTERING_MODE_EXCLUDE;
                              }
                              action_defined = true;
                              
                           } 
                        } else {
                           CDMW_ERROR(operationmap_logger,  
                                     filename << ":" << ln 
                                     << " Unknown filtering action! Ignoring '" << line << "'.");
                        }
                     }
                  } else {
                     if (!action_defined) {
                        CDMW_INFO(operationmap_logger, 
                                  "No filering action is defined. Assuming 'MATCH' in use for operations. Only operations "
                                  << "from '" << filename << "' will be recorded.");
                        action_defined = true;
                     }
                     std::string opname("");
                     if (explode_operation_line(line, opname, filename, ln)) {
                        Status status__ = addOperation (opname);
                        if (status__ == OPERATION_ALREADY_EXISTS) {
                           CDMW_WARN(operationmap_logger,  
                                     filename << ":" << ln 
                                     << " Operation '" << line << "' already defined in file '" 
                                     << filename << "'!");
                        } else if (status__ == OPERATION_ADDED) {
                           CDMW_DEBUG(operationmap_logger,  
                                     filename << ":" << ln 
                                     << " Added operation '" << line << "' from file '" 
                                     << filename << "'.");
                        } else if (status__ == OPERATION_NOP) {
                           CDMW_DEBUG(operationmap_logger,  
                                     filename << ":" << ln 
                                     << " Operation '" << line << "' from file '" 
                                     << filename << "' not added!");
                        }
                     }
                  }
                }
            } 
            a_file.close();
         }   
      }

      OperationMap::~OperationMap () throw()
      {
         m_operation_list.clear();
      }

      /**
       *Purpose: 
       *<p> checks "if an object in the Object Map
       * implements one of the stored operations then flag
       * it to be recorded".
       *
       */
      bool OperationMap::checkOperation (std::string & operation)
            throw()
      {
         // if operation list is empty, all operations will be recorded
         if (m_operation_list.empty())
         {
            return true;
         }
         

//          std::string l_str = suppr_head_tail_blanks(operation);
//          if (l_str.length() == 0)
//          {
//            return false;
//          }     
         for(unsigned int each=0; each<m_operation_list.size(); each++)
         {
           if (operation == m_operation_list[each])
              return (m_filtering_mode == FILTERING_MODE_MATCH);
         }
        return (m_filtering_mode != FILTERING_MODE_MATCH);
      }

      /**
       *Purpose:
       *<p> adds the requested operation to the operations store.
       *
       */
      OperationMap::Status OperationMap::addOperation (std::string & operation)
            throw()
      {
         std::string l_str = suppr_head_tail_blanks(operation);
         if (l_str.length() == 0)
         {
           return OperationMap::OPERATION_NOP;
         }           
         std::vector<std::string>::iterator each;
         for(each = m_operation_list.begin(); 
             each != m_operation_list.end();
             each++)
         {
            if (l_str == *each)
           {
             return OperationMap::OPERATION_ALREADY_EXISTS;
           }
         }
         m_operation_list.push_back(l_str);
         return OperationMap::OPERATION_ADDED;
      }

      /**
       *Purpose:
       *<p> adds the requested operation to the operation store.
       *
       */
      OperationMap::Status OperationMap::addOperations
            (CdmwRecording::String_List & list)
            throw() 
      {
         std::string operation;
         int len = list.length();
         for (int each = 0 ; each < len ; each ++)
         {
            operation = list[each];
            addOperation(operation);
         }
         return OPERATION_OK;
      }

      /**
       *Purpose:
       *<p> removes the requested interface from the operations store.
       *
       */
      OperationMap::Status OperationMap::removeOperation (std::string & operation)
          throw()
      {
        std::string l_str = suppr_head_tail_blanks(operation);
        std::vector<std::string>::iterator each;
        if (l_str.length() == 0)
        {
          return OperationMap::OPERATION_NOP;
        }           
        for(each = m_operation_list.begin(); 
            each != m_operation_list.end();
            each++)
        {
           if (l_str == *each)
          {
            m_operation_list.erase(each);
            return OperationMap::OPERATION_REMOVED;
          }
        }
        return OperationMap::OPERATION_NOT_FOUND;
      }

      /**
       *Purpose:
       *<p> removes the requested operations from the operation store.
       *
       */
      OperationMap::Status OperationMap::removeOperations
            (CdmwRecording::String_List & list)
            throw() 
      {
         std::string operation;
         int len = list.length();

         for (int each = 0 ; each < len ; each ++)
         {
            operation = list[each];
            removeOperation(operation);
         }
         return OPERATION_OK;
      }

      /**
       *Purpose:
       *<p> clear the operation store.
       *
       */
      OperationMap::Status OperationMap::removeAllOperations ()
            throw() 
      {
         m_operation_list.clear();
         return OPERATION_OK;
      }

      /**
       *Purpose:
       *<p> return the operation map size
       *
       */
       int OperationMap::list_size()
       {
          return m_operation_list.size();
       }

      /**
       *Purpose:
       *<p> map iterator.
       *
       */
      void OperationMap::mapLoop (display print)
          throw()
      {

        for (size_t i = 0; i < m_operation_list.size(); i++)
        {
          print (m_operation_list[i], i);
        }
      }

   }
} 
