/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

#include <Foundation/logging/LogManager.hpp>
#include <Foundation/logging/Logger.hpp>
#include <Foundation/logging/FunctionLogger.hpp>
#include "Foundation/ossupport/SystemProperties.hpp"
#include "TraceAndPerf/recordinglibrary/PacketStore.hpp"
#include "TraceAndPerf/recordingidl/CdmwRecordingData.stub.hpp"
#include "TraceAndPerf/recordingidl/CdmwRecordingDebug.stub.hpp"
#include "TraceAndPerf/recordinglibrary/Swapping.hpp"
#include "TraceAndPerf/recordinglibrary/Recording_Default_Parameters.hpp"

namespace
{
        const std::string ORPHAN_REQUEST_TIMEOUT_NAME ("Orphan_Request_Timeout") ;

        const char GIOP_MAGIC [4] = {'G', 'I', 'O', 'P'} ;
        const int * GIOP_MAGIC_INT = (const int *) GIOP_MAGIC ;

        const short ENDIAN_CHECKER = 0x0001 ;
        const Cdmw::Recording::octet * ENDIAN_BYTE   = (const Cdmw::Recording::octet *) &ENDIAN_CHECKER ;
        const Cdmw::Recording::octet NATIVE_ENDIAN = *ENDIAN_BYTE ;
}

namespace Cdmw
{
   namespace Recording
   {

      Cdmw::Logging::Logger* packetstore_logger;

      // JM Thales : 12/07/07
      /*
       *         Orphaned requests are those requests with no
       *         reply within pre-defined time frame. This timeout
       *         must be set as big as possible depending
       *         on your application
      */
      unsigned long Orphan_Request_Timeout_millisec = DEFAULT_ORPHAN_REQUEST_TIMEOUT_MILLISEC;

      bool needSwap (const octet endian) { return (((endian) & (1)) != NATIVE_ENDIAN); }

      unsigned long get_Orphan_Request_Timeout()
      {
         // JM Thales : 12/07/07
         static unsigned long Orphan_Request_Timeout_Millisec =
         Cdmw::OsSupport::SystemProperties::Get_Property
                                      (ORPHAN_REQUEST_TIMEOUT_NAME, DEFAULT_ORPHAN_REQUEST_TIMEOUT_MILLISEC) ;
         return Orphan_Request_Timeout_Millisec;
      }

      size_t Message::GiopSize () const
      {
         size_t size = 12 ; // GIOP header
         octet * src_ptr = data ;
         octet quad[4] ;

         if (header.message_type == CdmwRecording::Request)
         {
            // Server Id
            size += 4 + strlen (server_id) + 1 ;
            size += ((-size) & 3) ; //align to 4
            size += 4 ; // request_id
            size += 4 ; // date increment + response_flags + params_endian
            // interface_name
            size += 4 + strlen (intf_rid) + 1 ;
            size += ((-size) & 3) ; //align to 4
            // operation
            src_ptr += operation_Offset ;
            size_t op_len = strlen ((const char *)src_ptr) + 1 ;
            size += 4 + op_len ;
            size += ((-size) & 3) ; //align to 4
            src_ptr += op_len + ((-op_len) & 3) ;

            // service contexts
            int num ;
            Swapping::Swap4 (quad, src_ptr, need_swap) ;
            num = *(int *) quad ;
            src_ptr += 4 ;
            size += 4 ;

            for (; num > 0; num --)
            {
               // Service context tag
               src_ptr += 4 ;
               size += 4 ;
               // Service context size
               Swapping::Swap4 (quad, src_ptr, need_swap) ;

               // Service context data
               size += 4 + *(size_t *) quad ;
               src_ptr += 4 + *(size_t *) quad ;

               size += ((-size) & 3) ; //align to 4
               src_ptr += ((-*(size_t *) quad) & 3) ; //align to 4
            }
         }
         else
         {
            size += 4 ; // request_id
            size += 4 ; // date increment + params_endian + padding(1)
            size += 4 ; // status
            src_ptr += 20 ;

            // service contexts
            int num ;
            Swapping::Swap4 (quad, src_ptr, need_swap) ;
            num = *(int *) quad ;
            src_ptr += 4 ;
            size += 4 ;

            for (; num > 0; num --)
            {
               // Service context tag
               src_ptr += 4 ;
               size += 4 ;
               // Service context size
               Swapping::Swap4 (quad, src_ptr, need_swap) ;

               // Service context data
               size += 4 + *(size_t *) quad ;
               src_ptr += 4 + *(size_t *) quad ;

               size += ((-size) & 3) ; //align to 4
               src_ptr += ((-*(size_t *) quad) & 3) ; //align to 4
            }
         }

         // Align src_ptr to 8
         int header_size = src_ptr - data ;
         src_ptr += (-header_size) & 7 ;
         header_size += (-header_size) & 7 ;

         // Add parameters size
         if (header.length + 12 - header_size > 0)
         {
           size += 4 + header.length + 12 - header_size ;
         }
         else
         {
           // no parameters, just add the parameter area size
           size += 4 ;
         }

         return size ;
      }

      PacketStore::PacketStore () throw()
            : m_mutex()
            , m_flowControl(m_mutex)
            , m_connections(0)
            , m_messages_head(0)
            , m_messages_tail(0)
            , m_freeList(0)
            , m_stopped (false)
            , m_finalized (false)
      {
          packetstore_logger = Cdmw::Logging::Logger::Get_Logger("CDMW.TRC.RecordingDaemon");
      }

      PacketStore::~PacketStore () throw()
      {
         // Cleanup connections, messages
         finalize () ;

         // and freeList
         for (Message * tmp = m_freeList ; tmp!= 0; tmp = m_freeList)
         {
            m_freeList = tmp->next ;
            if (tmp-> data)
               delete [] tmp->data;
            tmp->data = 0;
            delete tmp ;
         }
         m_freeList = 0 ;

      }

      void  PacketStore::pushMessage(Message* msg)
      {
          if (m_messages_head == 0)
          {
            m_messages_head = msg;
          }
          else
          {
            m_messages_tail->next = msg;
          }
          m_messages_tail = msg ;
      }

      void PacketStore::pushPacket (Packet & packet) throw()
      {
         if (m_finalized)
         {
            return ;
         }

         // 05-05-08
         CDMW_MUTEX_GUARD(m_mutex) ;

         // There is at most one pending GIOP message per connection
         // First finds the connection descriptor
         ConnectionDescriptor * desc = m_connections ;

         for (; desc != 0; desc = desc->next)
         {
            if ((desc->channel.src_addr == packet.channel.src_addr) &&
                (desc->channel.dst_addr == packet.channel.dst_addr) &&
                (desc->channel.src_port == packet.channel.src_port) &&
                (desc->channel.dst_port == packet.channel.dst_port) )
            {
               // Found
               break ;
            }
         }

         if (desc == 0)
         {
            // New connection
            desc = new ConnectionDescriptor ;
            desc->next = m_connections ;
            m_connections = desc ;
            desc->channel = packet.channel ;
            desc->pending = 0 ;
            desc->bytes_got = 0 ;
         }

         // There could be several GIOP messages in a packet
         int index = 0 ;
         while (index < packet.length)
         {
            int num_to_copy = 0;
            // REQ-SRS-TRC-311
            // If the head of the packet is "GIOP", starts a new GIOP message
            if (*GIOP_MAGIC_INT == *(int *) (packet.data + index) && desc->pending == 0)
            {
                    // Get a new message header
                    if (m_freeList == 0)
                    {
                        desc->pending = new Message ;
                    }
                    else
                    {
                        desc->pending = m_freeList ;
                        m_freeList = m_freeList->next ;
                    }

                    // Initialize the new message
                    desc->pending->next = 0 ;
                    desc->pending->fragments = 0 ;
                    desc->pending->channel = packet.channel ;
                    desc->pending->date = packet.date ;
                    desc->pending->header = *(GIOP_Header *) (packet.data + index);
                    desc->pending->need_swap = needSwap (desc->pending->header.flags);
                    desc->pending->has_missing_bytes = false;
                    // Swap the length in the header if required
                    Swapping::Swap4 ((octet *) &desc->pending->header.length, desc->pending->need_swap) ;

                    // Copy the bytes
                    num_to_copy = desc->pending->header.length + 12 ;
                    desc->pending->data = new octet [num_to_copy] ;

                    if (num_to_copy > packet.length - index)
                    {
                        num_to_copy = packet.length - index ;
                    }

                    desc->bytes_got = num_to_copy ;
                    memcpy (desc->pending->data, packet.data + index, num_to_copy) ;
            }
            else if (desc->pending == 0)
            {
              // not a valid packet : maybe should we look for a "GIOP"
              // in the packet? (TBD)
                //no use to keep this packet in the connections list since we have no information on it
                if (m_connections == desc)
                    m_connections = desc->next;
                else
                {
                    for (ConnectionDescriptor * desc_iter = m_connections; 0 != desc_iter;
                        desc_iter = desc_iter->next)
                    {
                        if (desc_iter->next == desc)
                        {
                            desc_iter->next = desc->next;
                            break;
                        }
                    }
                }
                delete desc;
                return ;
            }
            else
            {

               // Tries to complete the GIOP message
               num_to_copy = desc->pending->header.length + 12 - desc->bytes_got ;
               if (num_to_copy > packet.length)
               {
                  num_to_copy = packet.length ;
               }

               memcpy (desc->pending->data + desc->bytes_got, packet.data, num_to_copy) ;
               desc->bytes_got += num_to_copy ;
            }

            index += num_to_copy ;

            if (desc->pending->has_missing_bytes)
            {
                // The GIOP message  has missing bytes!  chain it and let the MessageIdentifier bin it
                // TODO: Check whether we need to free Message from fragment list !
                //  - we see two potential problems here:
                //      1)  we may lost an inner fragment and store an incomplete message
                //      2) if we loose last fragment, a message will stay in the fragment list for ever.
                //CDMW_MUTEX_GUARD(m_mutex) ;
                pushMessage(desc->pending);
                m_flowControl.broadcast() ;
                desc->pending = 0 ;
                desc->bytes_got = 0 ;
            }
            else if (desc->bytes_got == desc->pending->header.length + 12)
            {
               // The GIOP message is complete, chain it to the list of
               // complete messages
//05-05-08               CDMW_MUTEX_GUARD(m_mutex) ;
               if ((desc->pending->header.flags & 2) == 2)
               {
                 if (desc->pending->header.message_type == CdmwRecording::Fragment)
                 {
                   (void*)appendFragment(desc, false) ;
                 }
                 else
                 {
                   // Store in the fragmented message queue
                     pushMessage(desc->pending);
                 }
               }
               else
               {
                 if (desc->pending->header.message_type == CdmwRecording::Fragment)
                 {
                   // Last fragment of a message
                   Message * msg = appendFragment(desc, true) ;

                   // Remove from fragmented list

                   // Move to complete message list
                   pushMessage(msg);
                   // Tell the listener
//                   m_flowControl.broadcast() ;
                 }
                 else
                 {
                   pushMessage(desc->pending);
                 }
                 m_flowControl.broadcast() ;
               }
               desc->pending = 0 ;
               desc-> bytes_got = 0 ;
            }
         }
      }

      void  PacketStore::stop () throw()
      {
        CDMW_MUTEX_GUARD(m_mutex) ;
        m_stopped = true ;
        m_flowControl.broadcast() ;
      }

      Message * PacketStore::popMessage () throw()
      {
         Message * message ;
         // JM Thales : 12/07/07
         static unsigned long Orphan_Request_Timeout_Millisec = get_Orphan_Request_Timeout ();

         // Wait for a message to be ready
         CDMW_MUTEX_GUARD(m_mutex) ;
         if (m_stopped)
         {
           return 0 ;
         }
         while ((m_messages_head == 0) && (! m_stopped) && ! m_finalized)
         {
            OsSupport::Condition::WaitReturn statusWait = m_flowControl.wait(Orphan_Request_Timeout_Millisec);
            if (statusWait == Cdmw::OsSupport::Condition::TIMEDOUT)
            {
               break ;
            }
         }

         if (m_messages_head == 0)
         {
            return 0 ;
         }

         // Unlink it
         message = m_messages_head ;
         m_messages_head = message->next ;
         message->next = 0 ;
         if (m_messages_head == 0)
         {
            m_messages_tail = 0 ;
         }

         return message ;
      }

      void PacketStore::freeMessage (Message *& message) throw()
      {
         if (message == 0)
         {
            return ;
         }

         CDMW_MUTEX_GUARD(m_mutex) ;
         deleteMessage (message) ;
      }

      void PacketStore::finalize () throw()
      {
         CDMW_MUTEX_GUARD(m_mutex) ;

         // Clear the connections
         ConnectionDescriptor * desc ;
         for (desc = m_connections ; desc != 0; desc = m_connections)
         {
            m_connections = desc->next ;
            if (desc->pending != 0)
            {
               delete [] desc->pending->data ;
               desc->pending->data = 0 ;
               desc->pending->next = m_freeList ;
               m_freeList = desc->pending ;
               desc->pending = 0 ;
            }
            delete desc ;
         }
         m_connections = 0 ;

         // Clear the messages
         for (Message * tmp = m_messages_head ; tmp!= 0; tmp = m_messages_head)
         {
            m_messages_head = tmp->next ;
            delete [] tmp->data ;
            tmp->data = 0 ;
            tmp->next = m_freeList ;
            m_freeList = tmp ;
         }
         m_messages_tail = 0 ;

         m_stopped = true ;
         m_finalized = true ;
         m_flowControl.broadcast() ;
      }

      void PacketStore::reset () throw()
      {
         CDMW_MUTEX_GUARD(m_mutex) ;
         m_stopped = false ;
         m_finalized = false ;
      }

      void PacketStore::deleteMessage(Message *& message)
      {
        delete [] message->data ;
        message->data = 0 ;
        // Valgrind : leak detected
        // CORBA::string_free(message->server_id._retn());
        // CORBA::string_free(message->intf_rid._retn());
        message->next = m_freeList ;
        m_freeList = message ;
        message = 0 ;
      }

      Message * PacketStore::appendFragment(ConnectionDescriptor * desc, bool remove)
      {
        // Find corresponding message
        Message * msg ;
        Message * prev = 0 ;
        for (msg = m_fragmented_head ; msg != 0; msg = msg->next)
        {
          if (msg->request_id == desc->pending->request_id)
          {
            break ;
          }
          prev = msg ;
        }

        if (msg != 0)
        {
          // Merge the data
          int total_len = msg->header.length + desc->pending->header.length ;
          // - Augment the buffer size
          octet * data = new octet [total_len + 12] ;

          // - copy the bytes of the first part of the message
          memcpy (data, msg->data, msg->header.length + 12) ;
          // - append the bytes of the fragment (but not the header)
          memcpy (data + msg->header.length + 12, desc->pending->data + 12, desc->pending->header.length) ;
          delete [] msg->data ;
          msg->data = data ;

          // - update the header
          msg->header.length = total_len ;
          *(int *) (msg->data + 8) = total_len ;
          // Swap the length in the buffer if required
          Swapping::Swap4 ((octet *) msg->data + 8, msg->need_swap) ;
        }

        // Free the message
        deleteMessage (desc->pending) ;
        delete [] desc->pending->data ;
        desc->pending->data = 0 ;
        desc->bytes_got = 0 ;

        if (remove)
        {
          // Remove from the fragmented list
          if (prev == 0)
          {
            m_fragmented_head = msg->next ;
          }
          else
          {
            prev->next = msg->next ;
          }

          if (m_fragmented_head == 0)
          {
            m_fragmented_tail = 0 ;
          }
          msg->next = 0 ;
        }
        return msg ;
      }
   }
}
