/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

#include "Foundation/orbsupport/CORBA.hpp"
#include "TraceAndPerf/recordingidl/CdmwRecordingData.stub.hpp"
#include "TraceAndPerf/recordinglibrary/RecordingPlugin.hpp"
#include "TraceAndPerf/recordinglibrary/CoalescenceManager.hpp"

namespace Cdmw
{
   namespace Recording
   {
      const octet LOCATION_FORWARD = 3 ;

      RecordingPlugin::RecordingPlugin ()
            throw() : Plugin(), m_manager (0), m_sender(0)
      {
      }
      
      RecordingPlugin::~RecordingPlugin ()
            throw()
      {
         delete m_manager ;
         m_manager = 0;
         delete m_sender ;
         m_sender = 0;
      }

      /**
       *Purpose: Set the recording destination URL
       *<p> .
       *
       */
      bool RecordingPlugin::Destination (const char * destination)
            throw()
      {
        if (m_sender)
        {
         m_sender->recordingURL (destination) ;
         return true ;
        }
        return false;
      }

      /**
       *Purpose: Get the recording destination URL
       *<p> .
       *
       */
      char * RecordingPlugin::Destination ()
            throw()
      {
        if (m_sender)
        {
         return m_sender->recordingURL () ;
        }
        return 0;
      }

      /**
       *Purpose: Process the request-reply pair given as argument
       *<p> .
       *
       */
      void RecordingPlugin::recordOperation (Message * request, Message * reply)
            throw()
      {
          // V-TRC-023 : only sends Request and Reply (Service messages are filtered)
          if ( request != 0 && request->header.message_type == CdmwRecording::Request)
          {
            // Ignore also orb operations (PCR 1030)
            const char * op = (const char *) (request->data + request->operation_Offset) ;
            if (op [0] != '_' || (strncmp (op, "_set_", 5) == 0) || (strncmp (op, "_get_", 5) == 0))
            {
              // Ignore also replies with LOCATION_FORWARD (PCR 1030)
              if ((reply == 0) || ((reply->data[16] + reply->data[19]) < LOCATION_FORWARD))
              {
                m_manager->WriteOperation(request, reply) ;
              }
            }
          }
      }

      /**
       *Purpose: initialize the plugin
       *<p> .
       *
       */
      void RecordingPlugin::initialize (CORBA::ORB_ptr orb)
          throw()
      {
        m_sender = new BufferSender;
        m_manager = new CoalescenceManager (*m_sender) ;
        m_manager->initialize();
        m_sender->initialize(m_manager, orb);
       }
      
      /**
       *Purpose: Set the maximum buffer number
       *<p> .
       *
       */
      void RecordingPlugin::SetMaxBufferNumber (size_t size)
      {
        if (m_sender)
        {
          m_sender -> SetMaxBufferNumber (size);
        }
      }

      /**
       *Purpose: Get the maximum buffer number
       *<p> .
       *
       */
      size_t RecordingPlugin::GetMaxBufferNumber ()
      {
        if (m_sender)
        {
          return m_sender -> GetMaxBufferNumber ();
        }
        return 0;
      }
      /**
       *Purpose: Set the retry period
       *<p> .
       *
       */
      void RecordingPlugin::SetRetryPeriod (milliseconds delay)
      {
        if (m_sender)
        {
          m_sender -> SetRetryPeriod  (delay);
        }
      }

      /**
       *Purpose: Get the retry period
       *<p> .
       *
       */
      milliseconds RecordingPlugin::GetRetryPeriod ()
      {
        if (m_sender)
        {
          return m_sender -> GetRetryPeriod ();
        }
        return 0 ;
      }

      /**
       *Purpose: Set the max number of retries
       *<p> .
       *
       */
      void RecordingPlugin::SetRetryNumber (long number)
      {
        if (m_sender)
        {
          m_sender -> SetRetryNumber  (number);
        }
      }
      
      /**
       *Purpose: Set the Corba Relative Roundtrip Timeout
       *<p> .
       *
      */
      
      void RecordingPlugin::SetCorbaTimeout (unsigned long timeout)
      {
        if (m_sender)
        {
          m_sender -> SetCorbaTimeout (timeout);
        }
      }

      /**
       *Purpose: Set the maximum buffer size
       *<p> .
       *
       */
      void RecordingPlugin::MaxSize (size_t size)
      {
        if (m_manager)
        {
          m_manager -> MaxSize (size);
        }
      }

      /**
       *Purpose: Get the maximum buffer size
       *<p> .
       *
       */
      size_t RecordingPlugin::MaxSize ()
      {
        if (m_manager)
        {
          return m_manager -> MaxSize ();
        }
        return 0;
      }
      
      /**
       *Purpose: Set the timeout
       *<p> .
       *
       */
      void RecordingPlugin::TimeOut (milliseconds delay)
      {
        if (m_manager)
        {
          m_manager -> TimeOut (delay);
        }
      }

      /**
       *Purpose: Get the timeout
       *<p> .
       *
       */
      milliseconds RecordingPlugin::TimeOut ()
      {
        if (m_manager)
        {
          return m_manager -> TimeOut ();
        }
        return 0 ;
      }
          

      /**
       *Purpose: flush the contents of the plugin
       *<p> .
       *
       */
      void RecordingPlugin::finalize ()
            throw()
      {
        if (m_manager)
        {
         m_manager->finalize () ;
         m_manager = 0;
        }
        if (m_sender)
        {
         m_sender->finalize () ;
         m_sender = 0;
        }
      }

   }
}
