/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

#include <Foundation/logging/LogManager.hpp>
#include <Foundation/logging/Logger.hpp>
#include <Foundation/logging/FunctionLogger.hpp>
#include "Foundation/ossupport/OS.hpp"
#include "TraceAndPerf/recordinglibrary/RequestPairing.hpp"
#include "TraceAndPerf/recordinglibrary/Recording_Default_Parameters.hpp"

namespace Cdmw
{
   namespace Recording
   {

      Cdmw::Logging::Logger* requestpairing_logger;
      RequestPairing::RequestPairing (PacketStore & store)
            throw()
            : m_plugins(0)
            , m_request_queue(0)
            , m_free_queue(0)
            , m_store(store)
      {
    requestpairing_logger = Cdmw::Logging::Logger::Get_Logger("CDMW.TRC.RecordingDaemon");
      }

      RequestPairing::~RequestPairing ()
            throw()
      {
         // Cleanup the plugin and request lists
      }

      /**
       *Purpose: add a recording plugin to the plugin list
       *<p> .
       *
       */
      void RequestPairing::addPlugin (Plugin * plugin)
            throw()
      {
         // Check if not already in the list
         for ( Plugin * check = m_plugins ; check != 0; check = check->next)
         {
            if (check == plugin)
            {
               return ;
            }
         }

         // Add it to the plugin list
         plugin->next = m_plugins ;
         m_plugins = plugin ;
      }

      /**
       *Purpose: Sets the recording destination URL for a plugin
       *<p> .
       *
       */
      void RequestPairing::setDestination (PluginType type, const char * destination)
            throw()
      {
         // Find a plugin of the given type
         for ( Plugin * check = m_plugins ; check != 0; check = check->next)
         {
            if (check->type() == type)
            {
               check->Destination (destination) ;
               return ;
            }
         }
      }

      /**
       *Purpose: Store a request until the corresponding reply is
       * received
       *<p> .
       *
       */
      void RequestPairing::storeRequest (Message * request)
            throw()
      {
         // If oneway, sends directly to the plugins
         if ((request->data [16] & 1) == 0)
         {
            for ( Plugin * check = m_plugins ; check != 0; check = check->next)
            {
               check->recordOperation (request, 0) ;
            }

            // returns the request to the store
            m_store.freeMessage(request) ;
            return ;
         }

     CDMW_LOG(requestpairing_logger, 
          CYCLIC_MSG_REQUESTPAIRING_LOG_LEVEL, 
          "=== RequestPairing.storeRequest twoway (0)");
         // Find the appropriate queue
         RequestQueue * queue ;

         for (queue = m_request_queue ; queue != 0 ; queue = queue->next )
         {
            if (queue->request_channel == request->channel)
            {
               break ;
            }
         }
         
         // If not found add a new one
         if (queue == 0)
         {
            if (m_free_queue == 0)
            {
               queue = new RequestQueue ;
            }
            else
            {
               queue = m_free_queue ;
               m_free_queue = m_free_queue->next ;
            }

            queue->next = m_request_queue ;
            m_request_queue = queue ;
            queue->request_channel = request->channel ;
            queue->reply_channel.src_addr = request->channel.dst_addr ;
            queue->reply_channel.dst_addr = request->channel.src_addr ;
            queue->reply_channel.src_port = request->channel.dst_port ;
            queue->reply_channel.dst_port = request->channel.src_port ;
            queue->requests = request ;
            request->next = 0 ;
         }
         else // Chain the request
         {
            request->next = queue->requests ;
            queue->requests = request ;
         }
      }

      void RequestPairing::storeFragment (Message * fragment)
            throw()
      {
         bool is_Request_Fragment = true ;

         // Find the appropriate queue
         RequestQueue * prev_queue = 0 ;
         RequestQueue * queue ;
         for (queue = m_request_queue ; queue != 0 ; queue = queue->next )
         {
            if (queue->reply_channel == fragment->channel)
            {
               is_Request_Fragment = false ;
               break ;
            }
            if (queue->request_channel == fragment->channel)
            {
               break ;
            }
            prev_queue = queue ;
         }

         if (queue == 0)
         {
            // Not found, drop the fragment
            m_store.freeMessage(fragment) ;
            return ;
         }

         // Find the corresponding request
         Message * prev = 0 ;
         Message * request ;
         for (request = queue->requests ; request != 0; request = request->next)
         {
            if (request->request_id == fragment->request_id)
            {
               break ;
            }

            prev = request ;
         }

         if (request == 0)
         {
            // Not found, drop the fragment
            m_store.freeMessage(fragment) ;
            return ;
         }

         // Chain the fragment
      }

      /**
       *Purpose: Check if a reply is to be recorded, and if so send
       * it and its request to the plugins
       *<p> .
       *
       */
      void RequestPairing::checkReply (Message * reply)
            throw()
      {
     CDMW_LOG(requestpairing_logger, 
          CYCLIC_MSG_REQUESTPAIRING_LOG_LEVEL, 
          "=== RequestPairing.checkReply twoway (0)");
         // Find the appropriate queue
         RequestQueue * prev_queue = 0 ;
         RequestQueue * queue ;
         for (queue = m_request_queue ; queue != 0 ; queue = queue->next )
         {
            if (queue->reply_channel == reply->channel)
            {
               break ;
            }
            prev_queue = queue ;
         }

     CDMW_LOG(requestpairing_logger, 
          CYCLIC_MSG_REQUESTPAIRING_LOG_LEVEL, 
          "=== RequestPairing.checkReply twoway (1)");
         if (queue == 0)
         {
            // Not found, drop the reply
            m_store.freeMessage(reply) ;
            return ;
         }

     CDMW_LOG(requestpairing_logger, 
          CYCLIC_MSG_REQUESTPAIRING_LOG_LEVEL, 
          "=== RequestPairing.checkReply twoway (2)");
         // Find the corresponding request
         Message * prev = 0 ;
         Message * request ;
         for (request = queue->requests ; request != 0; request = request->next)
         {
            if (request->request_id == reply->request_id)
            {
               break ;
            }

            prev = request ;
         }

     CDMW_LOG(requestpairing_logger, 
          CYCLIC_MSG_REQUESTPAIRING_LOG_LEVEL, 
          "=== RequestPairing.checkReply twoway (3)");
         if (request == 0)
         {
            // Not found, drop the reply
            m_store.freeMessage(reply) ;
            return ;
         }

     CDMW_LOG(requestpairing_logger, 
          CYCLIC_MSG_REQUESTPAIRING_LOG_LEVEL, 
          "=== RequestPairing.checkReply twoway (4)");
         // Unlink the request
         if (prev == 0)
         {
            queue->requests = request->next ;

            if (queue->requests == 0)
            {
               // Empty queue, put in free list
               if (prev_queue == 0)
               {
                  RequestQueue * tmp = m_request_queue ;
                  m_request_queue = queue->next ;
                  queue->next = m_free_queue ;
                  m_free_queue = tmp ;
               }
               else
               {
                  prev_queue->next = queue->next ;
               queue->next = m_free_queue ;
               m_free_queue = queue ;
            }
         }
         }
         else
         {
            prev->next = request->next ;
         }

         request->next = 0 ;

         // Send to the plugins
     CDMW_LOG(requestpairing_logger, 
          CYCLIC_MSG_REQUESTPAIRING_LOG_LEVEL, 
          "=== RequestPairing.checkReply twoway (5)");
         for ( Plugin * check = m_plugins ; check != 0; check = check->next)
         {
       CDMW_LOG(requestpairing_logger, 
            CYCLIC_MSG_REQUESTPAIRING_LOG_LEVEL, 
            "=== RequestPairing.checkReply call recordOperation (6)");
            check->recordOperation (request, reply) ;
         }

         // returns the message to the store
         m_store.freeMessage(request) ;
         m_store.freeMessage(reply) ;

         // Check for orphan requests
     CDMW_LOG(requestpairing_logger, 
          CYCLIC_MSG_REQUESTPAIRING_LOG_LEVEL, 
          "=== RequestPairing.checkReply call checkOrphanRequests (7)");
         checkOrphanRequests () ;
      }

      /**
       *Purpose: Check if some requests are not orphan
       *         Orphaned requests are those requests with no 
       *         reply within pre-defined time frame. This timeout
       *         must be set as big as possible depending
       *         on your application
       *
       */
      void RequestPairing::checkOrphanRequests ()
            throw()
      {
         if (m_request_queue == 0)
         {
           return;
         }
       
         Message * prev = 0 ;
         Message * request ;
         Cdmw::OsSupport::OS::Timeval orphan_date = Cdmw::OsSupport::OS::get_time() ;
         orphan_date.microseconds -= (get_Orphan_Request_Timeout() * 1000) % 1000000 ;
         if (orphan_date.microseconds < 0)
         {
            orphan_date.microseconds += 1000000 ;
            orphan_date.seconds -- ;
         }
         orphan_date.seconds -= get_Orphan_Request_Timeout() / 1000 ;
         

         for (request = m_request_queue->requests ; request != 0; request = request->next)
         {
            Cdmw::OsSupport::OS::Timeval request_date(request->date) ;
            if (request_date < orphan_date)
            {
               // Orphan request
               // Unlink it
               if (prev == 0)
               {
                  m_request_queue->requests = 0 ;
               }
               else
               {
                  prev->next = 0 ;
               }

               // Send it (and the request following) to the plugins
               for (; request != 0;)
               {
                  prev = request->next ;
                  request->next = 0 ;

                  // Send to the plugins
                  for ( Plugin * check = m_plugins ; check != 0; check = check->next)
                  {
                     check->recordOperation (request, 0) ;
                  }

                  // returns the message to the store
                  m_store.freeMessage(request) ;

                  request = prev ;
               }

               return ;
            }

            prev = request ;
         }
      }

      /**
       *Purpose: clear the contents: plugins and requests
       *<p> .
       *
       */
      void RequestPairing::finalize ()
            throw()
      {
         RequestQueue * queue ;
         RequestQueue * next_queue ;
         for (queue = m_request_queue ; queue != 0 ; queue = next_queue )
         {
            next_queue = queue->next ;
            Message * request ;
            Message * next_request ;
            for (request = queue->requests ; request != 0; request = next_request)
            {
               next_request = request->next ;

               request->next = 0 ;
               m_store.freeMessage(request) ;
            }

            queue->requests = 0 ;
            queue->next = m_free_queue ;
            m_free_queue = queue ;
         }
         m_request_queue = 0 ;

         for (queue = m_free_queue ; queue != 0 ; queue = next_queue )
         {
            next_queue = queue->next ;
            delete queue;
         }
         m_free_queue = 0;

         // Finalize and clear the plugin list
         Plugin * next = 0 ;
         for (Plugin * cur = m_plugins ; cur != 0; cur = next)
         {
            next = cur->next ;
            cur->next = 0 ;
            try
            {
               cur->finalize() ;
            }
            catch (...) {}
         }
         m_plugins = 0 ;
      }

   }
}
