/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

#include <sstream>

#include <stdio.h>
#include <stdlib.h>
#include <Foundation/logging/LogManager.hpp>
#include <Foundation/logging/Logger.hpp>
#include <Foundation/logging/FunctionLogger.hpp>
#include <Foundation/ossupport/OS.hpp>
#include "Foundation/osthreads/Thread.hpp"
#include <Foundation/common/Options.hpp>
#include "TraceAndPerf/recordingtools/Command.hpp"

namespace
{
        const std::string IFR_URL                    = "--ifr_url";    
        const std::string CLI_OPT                    = "--cli";    
        const std::string NOCLI_OPT                  = "--nocli";    
        const std::string FLAGS_OPT                  = "--flags";
}

namespace Cdmw
{
   namespace Recording
   {

     Cdmw::Logging::Logger* command_logger;
     void Usage() ;
     void Help() ;
     /**
      * Get the options
      */
     int get_debugger_options (int argc, char* argv[],
                              Cdmw::Recording::DebuggerOptions &options)

     {
       command_logger = Cdmw::Logging::Logger::Get_Logger("CDMW.TRC.RecordingDaemon");
       // check the number of arguments.
       if (argc == 1) 
       {
         // ifr_url is a mandatory argument
         Usage();
         return EXIT;
       }

       // --help
       if (strcmp(argv[1], "--help") == 0) 
       {
         Help();
         return EXIT;
       }

       std::string option_val;

       // --configuration_file
       option_val = Cdmw::OsSupport::OS::get_option_value(argc, argv, IFR_URL);
       if (option_val == "no") 
       {
         CDMW_ERROR(command_logger, "*** Error : " << IFR_URL << " =NAME mandatory argument must be defined");
         Usage();
         return FAILURE;
       }
       else if (option_val == "yes") 
       {
         CDMW_ERROR(command_logger, "*** Error : " << IFR_URL << " =NAME option must be defined");
         Usage();
         return FAILURE;
       }
       options.ifr_url = option_val;

       // --interactive mode
       options.interactive = true; // default
       option_val = Cdmw::OsSupport::OS::get_option_value (argc, argv, NOCLI_OPT);
       if (option_val == "no")
       {
         options.interactive = true; // default
       }
       else 
       {
         options.interactive = false;
       }

       option_val = Cdmw::OsSupport::OS::get_option_value (argc, argv, FLAGS_OPT);
       if (option_val == "no")
       {
         options.flags = 0; // default to 0
       }
       else 
       {
         std::istringstream num_value(option_val);
         num_value >> options.flags;
         if (num_value.fail()) 
         {
           CDMW_ERROR(command_logger, "*** Error : bad value for the " << FLAGS_OPT << " =NUM option");
           return FAILURE;
         }
         if ((options.flags < 0) || (options.flags > 2))
         {
           Usage();
           return EXIT;
         }
       }

       return SUCCESS;
     }

     /**
       *Purpose: Display the debug files produced by the daemon
       *
       *<p> .
       *
       */
      class Command::Worker : public virtual OsSupport::Thread
      {
         public :
            Worker (Command & master)
                  : m_master (master)
                  , m_id(0)
            {
            }

            ~Worker () throw ()
            {
            }

            void start_now() throw ()
            {
               m_id = start() ;
            }

            void run() throw ()
            {
               while (m_master.nextCommand ())
               {
               }
               std::cout << "Quitting" <<std::endl;
               m_master.Quit();
            }

            Command & m_master ;
            OsSupport::Thread::ThreadId m_id ;
      } ;

      Command::Command(unsigned int flags, const char * ifr_url)
            throw()
            : m_ifr_store (ifr_url)
            , m_decoder ()
            , m_display (m_ifr_store, m_decoder)
            , m_worker (0)
            , m_input_file (0)
            , m_output_file (0)
            , m_prompt (CORBA::string_dup("-> "))
            , m_flags (flags)
            , m_run(false)
            , m_terminated(false)
            , m_mutex()
            , m_control(m_mutex)
            , m_error_already_displayed (false)
      {
         Set_Flags(flags) ;
      }

      Command::~Command()
            throw()
      {
         if (m_input_file != 0)
         {
            CORBA::string_free (m_input_file) ;
         }
         if (m_output_file != 0)
         {
            CORBA::string_free (m_output_file) ;
         }

         CORBA::string_free (m_prompt) ;
      }

     
      /**
       *Purpose: Handle the operator or gui commands
       * A "false" result indicates that exit is requested
       *<p> .
       *
       */
      bool Command::nextCommand ()
            throw()
      {
         char command [100] ;
         char * line = (char *) ::malloc(256) ;
         size_t size = 256 ;

         printf ("%s", m_prompt) ;
         fflush(stdout) ;
         
         ssize_t len = ::getline (&line, &size, stdin) ;
         if (len < 0)
           {
             std::cout<<"!!! Do not launch this program in background"<<std::endl;
             return false;
           }
            
         // std::cout << "len=" << len << std::endl;      
         if (len > 0)
         {
            command[0] = '\0';
            int res = sscanf (line, "%s", (char *)command) ;

            if (strcmp (command, "help") == 0)
            {
               Help() ;
            }
            else if (strcmp (command, "quit") == 0)
            {
               return false ;
            }
            else if (strcmp (command, "prompt") == 0)
            {
               char prompt [200] ;
               res = sscanf (line," %s %s", (char *) command, (char *) prompt) ;
               CORBA::string_free (m_prompt) ;
               m_prompt = CORBA::string_dup(prompt) ;
            }
            else if (strcmp (command, "input") == 0)
            {
               char file [200] ;
               file[0] = '\0';
               res = sscanf (line," %s %s", (char *) command, (char *) file) ;
               Set_Input(file) ;
            }
            else if (strcmp (command, "output") == 0)
            {
               char file [200] ;
               file[0] = '\0';
               res = sscanf (line," %s %s", (char *) command, (char *) file) ;
               if (! Set_Output(file))
               {
                  printf ("Cannot use file %s as output, keeping %s\n", file, m_output_file) ;
               }
               else
               {
                  CORBA::string_free (m_output_file) ;
                  m_output_file = CORBA::string_dup(file) ;
               }
            }
            else if (strcmp (command, "flags") == 0)
            {
               unsigned int flags ;
               res = sscanf (line," %s %i", (char *) command, &flags) ;
               Set_Flags(flags) ;
            }
            else
            {
               Help() ;
            }
        }

         return true ;
      }

      /**
       *Purpose: Do the requested decoding
       *<p> .
       *
       */
      void Command::Decode (bool interactive)
            throw()
      {
         if (interactive)
         {
            m_worker = new Worker(*this) ;
            m_worker->start_now();

            while (! m_terminated)
            {
               // Wait for a new file
               {
                  CDMW_MUTEX_GUARD(m_mutex) ;
                  //JM
                  //while (!(m_run || m_terminated))
                  while (!m_run)
                  {
                     m_control.wait() ;
                  }
               }

               // JM
               if (!m_terminated)
               {
                 if (! m_display.Display_File (m_input_file))
                 {
                   if (! m_error_already_displayed)
                   {
                      printf ("Cannot decode %s\n", m_input_file) ;
                      m_error_already_displayed = true;
                   }
                }
                 else if (m_display.Is_Stopped())
                 {
                   Stop();
                   printf ("%s", m_prompt) ;
                   fflush(stdout) ;
                 }
               }
            }
            std::cout << "Decoder()" << std::endl;
            delete m_worker ;
            m_worker = 0 ;

         }
         else
         {
            m_display.Display_File ("-") ;
         }
      }

      void Usage()
      {
         printf ("\n\nUsage: debug_decoder [options] --ifr_url=NAME\n") ;
         printf ("--ifr_url=NAME mandatory arguments indicates the url of the Interface Repository to use\n") ;
         printf ("options:\n") ;
         printf ("   '[--cli]'                   : interactive mode (default)\n") ;
         printf ("   '[--nocli]'                 : non interactive (filter) mode\n") ;
         printf ("                                 'cli' stands for 'command line interpreter'\n") ;
         printf ("   '[--flags=NUM]' : indicate the display flags (bitmask default 0)\n") ;
         printf ("    1 indicates headers only, 2 indicates forever reading\n\n") ;
      }

      void Help()
      {
         printf ("commands:\n") ;
         printf ("   'help                   : display this text\n") ;
         printf ("   'quit                   : terminate the decoding and exit\n") ;
         printf ("   'prompt <prompt value>  : indicate the prompt string to use (default '-> ')\n") ;
         printf ("   'input <input_file>'    : indicate the input file to decode\n") ;
         printf ("   'output <output_file>'  : indicate the output file where to put the result\n") ;
         printf ("   'flags <display_flags>' : indicate the display flags (bitmask)\n") ;
         printf ("    1 indicates headers only, 2 indicates forever reading\n") ;
         printf ("input, output and flags without parameter display the current value\n") ;
      }

      // JM
      void Command::Stop()
      {
        CDMW_MUTEX_GUARD(m_mutex) ;
        if (m_input_file != 0)
        {
          CORBA::string_free (m_input_file) ;
        }
        if (m_output_file != 0)
        {
          CORBA::string_free (m_output_file) ;
        }
        m_run = false;
        //
        m_control.broadcast() ;
      }
      
      void Command::Set_Input(const char * name) 
      {
         CDMW_MUTEX_GUARD(m_mutex) ;
         m_input_file = CORBA::string_dup (name) ;
         // JM
         m_error_already_displayed = false;
         m_run = true;
         //
         m_control.broadcast() ;
      }

      bool Command::Set_Output(const char * name)
      {
         return m_display.Set_Output (name) ;
      }

      void Command::Set_Flags(unsigned int flags)
      {
         m_display.Set_Flags (flags) ;
      }

      void Command::Quit()
      {
        {
          CDMW_MUTEX_GUARD(m_mutex) ;
          m_run = true;
          m_terminated = true ;
          m_display.Stop_Display () ;
          m_control.broadcast() ;
        }
      }

   }
}

