/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#include <clocale>
#include <fstream>
#include <iostream>
#include <sstream>
#include <string>

#include <Foundation/logging/LogManager.hpp>
#include <Foundation/logging/Logger.hpp>
#include <Foundation/logging/FunctionLogger.hpp>
#include <Foundation/common/Exception.hpp>
#include <Foundation/common/Locations.hpp>
#include <Foundation/common/Options.hpp>
#include <Foundation/common/System.hpp>
#include <Foundation/common/String.hpp>
#include <Foundation/commonsvcs/initutils/InitUtils.hpp>
#include <Foundation/ossupport/OS.hpp>
#include <Foundation/ossupport/SystemProperties.hpp>
#include <Foundation/orbsupport/CORBA.hpp>
#include <Foundation/orbsupport/ExceptionMinorCodes.hpp>
#include <Foundation/orbsupport/OrbSupport.hpp>
#include <Foundation/orbsupport/StrategyList.hpp>
#ifdef CDMW_USE_FAULTTOLERANCE
#include <FaultTolerance/ftinit/FTServiceInit.hpp>
#include <FaultTolerance/idllib/FT.stub.hpp>
#endif
#include <Foundation/commonsvcs/naming/NamingInterface.hpp>

//#include <Foundation/idllib/CdmwRecordingContainerManager.skel.hpp>
#include <TraceAndPerf/recordingidl/CdmwRecordingDaemonControl.skel.hpp>
#include "recordingtools/DaemonInit.hpp"
#include "recordingtools/CdmwRecordingContainerManager_impl.hpp"
#include "recordingtools/CdmwRecordingDaemonControl_impl.hpp"


// definition of local constants in an unnammed namespace
namespace 
{   
    Cdmw::Logging::Logger* daemoninit_logger;
    using std::string;

    const string INTERFACES_FILE_OPT        = "--interfaces_file";    
    const string OPERATIONS_FILE_OPT        = "--operations_file";    
    const string MAX_SIZE_OPT               = "--max_buffer_size";    
    const string TIMEOUT_OPT                = "--timeout";    
    const string CORBA_TIMEOUT_OPT          = "--corba_timeout";    
    const string RETRY_DELAY_OPT            = "--retry_delay";    
    const string MAX_BUFFER_OPT             = "--max_buffer"; 
    const string RECORDER_URL_OPTION        = "--recorder_url";
    const string DEBUG_FILE_OPTION          = "--debug_file" ; 
    const string BIND_TO_NS_OPTION          = "--bind_to_ns" ; 
    const string REBIND_TO_NS_OPTION        = "--rebind_to_ns" ;                  

    const string LOC_REC_DAEMON_CONTAINER_MGR_CORBALOC_ID      = "RecordingDaemonContainerMgr";
    const string LOC_REC_DAEMON_CONTROL_CORBALOC_ID            = "RecordingDaemonController";
    const unsigned short POA_THREAD_POOL_SIZE = 5;


    /**
     * Print the usage.
     *
     * @param out the output stream.
     * @param programName the program name.
     */
    static void
    usage(std::ostream& out, const string& programName)
    {
        using std::endl;
        using Cdmw::Common::Options;

        out << "usage: " << programName << " options...\n"
            << "options:\n"
            << "  --help                             " << "Display the help.\n"
            << "  [--interfaces_file=NAME]           " << "Name of the interfaces file.\n"
            << "  [--operations_file=NAME]           " << "Name of the operations file.\n"
            << "  [--max_size=NUM]                   " << "max buffer size\n"
            << "  [--timeout=NUM]                    " << "timeout (max time before data are sent to recorder)\n"
            << "  [--corba_timeout=NUM]              " << "CORBA Relative roundtrip timeout (with recorder)\n"
            << "  [--retry_delay=NUM]                " << "retry delay (reconnection loop parameter used after corba_timeout has expired\n"
            << "  [--recorder_url=URLs]              " << "recorder corbaloc URLs. Activates recorder pluggin\n"
            << "  [--debug_file=NAME]                " << "file name to store debugger output. Activates debug pluggin\n"

            << "  [--bind_to_ns]                     " << "bind RecordingDaemonContainerMgr and RecordingDaemonController in name server.\n"
            << "  [--rebind_to_ns]                   " << "rebind RecordingDaemonContainerMgr and RecordingDaemonController in name server.\n"
            << "                                     " << "Use -ORBInitRef NameService=<<name service ior>> to define Name Server.\n"
            << endl;
    }

    // Default value for ORB connection timeout
    std::string CONNECTION_TIMEOUT_PROPERTY_VALUE="300"; // 300 ms
    
    // Default value for FT::REQUEST_DURATION_POLICY
    std::string FT_REQUEST_DURATION_PROPERTY_NAME="Cdmw.TraceAndPerf.Recording.RequestDurationPolicy";
    TimeBase::TimeT DEFAULT_FT_REQUEST_DURATION_PROPERTY_VALUE=2000000; // 200 ms
    

} // End namespace anonymous

namespace Cdmw
{

namespace Trace
{
    namespace DaemonInit
    {
        using namespace Cdmw::OrbSupport;
        using namespace Cdmw::OsSupport;
        using namespace Cdmw::Common;
        
        /**
         * Start the client thread in interactive mode and start the ORB.
         *
         * @param orb the ORB.
         * @param interactive_mode flag to set to true if interactive mode is wanted.
         */
        int run_processing (CORBA::ORB_ptr orb, bool interactive_mode)
        {
            int status = SUCCESS;

            orb->run();

            return status;
        }
        
        /**
         * Create the process agent
         *
         * @param orb the ORB.
         * @param root_POA the root poa.
         *
         */
        int create_process_agent (CORBA::ORB_ptr orb,
                                  PortableServer::POA_ptr root_POA,
                                  DaemonInit::DaemonOptions &options)
        {
            int status = FAILURE;
            
            try
            {
                CdmwRecording::ContainerManager_var agent_ref =
                    CdmwRecording::ContainerManager::_nil();
                CdmwRecording::DaemonControl_var control_ref =
                    CdmwRecording::DaemonControl::_nil();
                CosNaming::NamingContextExt_var naming_ctx =
                    CosNaming::NamingContextExt::_nil();

                // create the DaemonControl Servant.
                CdmwRecording::DaemonControl_impl* p_process_control_servant =
                           new CdmwRecording::DaemonControl_impl(orb, root_POA, options.interfaces_file, options.operations_file);
                PortableServer::ServantBase_var control_servant_var = p_process_control_servant;
                if (p_process_control_servant->start() != CdmwRecording::Started)
                  {
                      CDMW_WARN(daemoninit_logger, "!!! Warning : insufficient privilege, network sniffing not allowed");                      
                  };

                // get the ContainerManager Servant
                CdmwRecording::ContainerManager_impl* p_process_agent_servant =
                  p_process_control_servant->get_ContainerManager();
                PortableServer::ServantBase_var servant_var = p_process_agent_servant;

                // activate ContainerManager servant on POA.
                PortableServer::ObjectId_var oid =
                root_POA->activate_object(p_process_agent_servant);
                CORBA::Object_var object = root_POA->id_to_reference(oid.in());
                agent_ref = CdmwRecording::ContainerManager::_narrow(object.in());


                // activate DaemonControl servant on POA.
                oid = root_POA->activate_object(p_process_control_servant);
                object = root_POA->id_to_reference(oid.in());
                control_ref = CdmwRecording::DaemonControl::_narrow(object.in());

                // bind the ContainerManager to a corbaloc name.
                Cdmw::OrbSupport::OrbSupport::bind_object_to_corbaloc(
                        orb, LOC_REC_DAEMON_CONTAINER_MGR_CORBALOC_ID, agent_ref.in()); 
                Cdmw::OrbSupport::OrbSupport::bind_object_to_corbaloc(
                        orb, LOC_REC_DAEMON_CONTROL_CORBALOC_ID, control_ref.in());

                if (options.recorder_url != "")
                {
                    // white space or coma separated list of URLs
                    std::vector< std::string > urls_vector = Cdmw::Common::String::to_strings( options.recorder_url.c_str(), " ," );
                    if (urls_vector.size() > CdmwRecording::REC_DAEMON_MAX_RECORDERS)
                    {
                        CDMW_ERROR(daemoninit_logger, "*** Error : Process initialisation: too many recorders declared");
                        return status;
                    }
                    for ( size_t which = 0; which < urls_vector.size(); ++which)
                    {
                       p_process_control_servant->setRecorderUrl (which, urls_vector[which].c_str());
                    }
                }

                if (options.debug_file != "")
                {
                   p_process_control_servant->debug_file (options.debug_file.c_str());
                }
                // (re)bind to NS
                if (options.bind_to_ns || options.rebind_to_ns ) {
                   // Get Name Server reference
                   CORBA::Object_var obj = 
                      orb->resolve_initial_references("NameService");  
                   if(CORBA::is_nil(obj.in()))
                   {
                      CDMW_ERROR(daemoninit_logger, "*** Error : Process initialisation: `NameService' is a nil object reference");
                      return status;
                   }

                   naming_ctx = CosNaming::NamingContextExt::_narrow(obj.in());
                   if(CORBA::is_nil(naming_ctx.in()))
                   {
                      CDMW_ERROR(daemoninit_logger, "*** Error : Process initialisation: `NameService' is not a NamingContextExt object");
                      return status;
                   }
                   ::Cdmw::CommonSvcs::Naming::NamingInterface ni(naming_ctx.in());
                   bool force=false;
                   if (options.rebind_to_ns)
                      force = true;
                   ni.bind(LOC_REC_DAEMON_CONTAINER_MGR_CORBALOC_ID, agent_ref.in(), force);
                   ni.bind(LOC_REC_DAEMON_CONTROL_CORBALOC_ID, control_ref.in(), force);

                   // init the traces delegation to the Daemon LogCollector
                   //
                   Cdmw::CommonSvcs::InitUtils* initUtils =
                       Cdmw::CommonSvcs::InitUtils::Instance();
                   try
                   {
                       initUtils->activate_LogCollector(naming_ctx.in());
                   }
                   catch (const Cdmw::Exception& ex)
                   {
                       CDMW_WARN(daemoninit_logger,
                                 "Trace logs will not be sent to Trace Collector:"
                                 << ex.what());
                   }
                }

                status = SUCCESS;                   
            }
            catch (const std::bad_alloc&) {
                CDMW_ERROR(daemoninit_logger, "*** Error : Process initialisation: bad allocation exception");
            }
            catch (const Cdmw::Exception& ex) {
                CDMW_ERROR(daemoninit_logger, "*** Error : " << ex.what());
            }
            catch (const CORBA::SystemException& ex) {
                CDMW_ERROR(daemoninit_logger, "*** Error : " << ex);
            }
            catch(const CORBA::Exception& ex) {
                CDMW_ERROR(daemoninit_logger, "*** Error :" << ex);
            }
            return status;
        }

        /**
         * Get the options
         */
        int get_recdaemon_options (int argc, char* argv[],
                                   DaemonOptions & options,
                                   int &new_argc, char** &new_argv,
                                   const std::string& def_process_port)
        {
            daemoninit_logger = Cdmw::Logging::Logger::Get_Logger("CDMW.TRC.RecordingDaemon");
            // check the number of arguments.
            if (argc == 1) 
            {
                // No mandatory arguments
                return SUCCESS;
            }

            // --help
            if (strcmp(argv[1], "--help") == 0) 
            {
                usage(std::cout, argv[0]);
                return EXIT;
            }
            
            std::string option_val;

            // --interfaces_file
            option_val = OS::get_option_value(argc, argv, INTERFACES_FILE_OPT);
            if (option_val == "no") 
            {
                    option_val = "";
            }
            else if (option_val == "yes") 
            {
                    CDMW_ERROR(daemoninit_logger, "*** Error : " << INTERFACES_FILE_OPT << "=NAME option must be defined");
                    usage(std::cout, argv[0]);
                    return FAILURE;
            }
            options.interfaces_file = option_val;
            
            // --operations_file
            option_val = OS::get_option_value(argc, argv, OPERATIONS_FILE_OPT);
            if (option_val == "no") 
            {
                option_val = "";
            }
            else if (option_val == "yes") 
            {
                CDMW_ERROR(daemoninit_logger, "*** Error : " << OPERATIONS_FILE_OPT << "=NAME option must be defined");
                usage(std::cout, argv[0]);
                return FAILURE;
            }
            options.operations_file = option_val;


            // --max_size
            option_val = OS::get_option_value (argc, argv, MAX_SIZE_OPT);
            if (option_val == "no")
            {
                options.max_size = 8192; // default to 8192
            }
            else 
            {
                std::istringstream num_value(option_val);
                num_value >> options.max_size;
                if (num_value.fail()) 
                {
                    CDMW_ERROR(daemoninit_logger, "*** Error : bad value for the " << MAX_SIZE_OPT << "=NUM option");
                    return FAILURE;
                }
            }

            option_val = OS::get_option_value (argc, argv, TIMEOUT_OPT);
            if (option_val == "no")
            {
                options.timeout = 5; // default to 5
            }
            else 
            {
                std::istringstream num_value(option_val);
                num_value >> options.timeout;
                if (num_value.fail()) 
                {
                    CDMW_ERROR(daemoninit_logger, "*** Error : bad value for the " << TIMEOUT_OPT << "=NUM option");
                    return FAILURE;
                }
            }

            option_val = OS::get_option_value (argc, argv, CORBA_TIMEOUT_OPT);
            if (option_val == "no")
            {
                options.corba_timeout = 100; // default to 5
            }
            else 
            {
                std::istringstream num_value(option_val);
                num_value >> options.corba_timeout;
                if (num_value.fail()) 
                {
                    CDMW_ERROR(daemoninit_logger, "*** Error : bad value for the " << TIMEOUT_OPT << "=NUM option");
                    return FAILURE;
                }
            }
            
            option_val = OS::get_option_value (argc, argv, RETRY_DELAY_OPT);
            if (option_val == "no")
            {
                options.recorder_retry_delay = 5; // default to 5
            }
            else 
            {
                std::istringstream num_value(option_val);
                num_value >> options.recorder_retry_delay;
                if (num_value.fail()) 
                {
                    CDMW_ERROR(daemoninit_logger, "*** Error : bad value for the " << RETRY_DELAY_OPT << "=NUM option");
                    return FAILURE;
                }
            }

            option_val = OS::get_option_value (argc, argv, MAX_BUFFER_OPT);
            if (option_val == "no")
            {
                options.max_size = 10; // default to 10
            }
            else 
            {
                std::istringstream num_value(option_val);
                num_value >> options.max_size;
                if (num_value.fail()) 
                {
                    CDMW_ERROR(daemoninit_logger, "*** Error : bad value for the " << MAX_BUFFER_OPT << "=NUM option");
                    return FAILURE;
                }
            }

            // process port option
            options.process_port = def_process_port;
            option_val = OS::get_option_value(argc, argv, Options::PROCESS_PORT_OPTION);
            if (option_val == "no") 
            {
                options.process_port = def_process_port;
            }
            else
            {
                std::istringstream port_strm(option_val);
                if (port_strm.fail()) 
                {
                    CDMW_ERROR(daemoninit_logger, "*** Error : bad value for the "
                              << Options::PROCESS_PORT_OPTION
                              << "=PORT option");
                     return FAILURE;
                }                
                options.process_port = option_val;
            }

            // recorder url option
            option_val = OS::get_option_value(argc, argv, RECORDER_URL_OPTION);
            if (option_val == "no") 
            {
                option_val = "";
            }
            else if (option_val == "yes")
            {
                    CDMW_ERROR(daemoninit_logger, "*** Error : " << RECORDER_URL_OPTION
                              << "=URL option must be defined");
                    usage(std::cout, argv[0]);
                    return FAILURE;
            }
            options.recorder_url = option_val;

            // debug file option
            option_val = OS::get_option_value(argc, argv, DEBUG_FILE_OPTION);
            if (option_val == "no") 
            {
                option_val = "";
            }
            else if (option_val == "yes")
            {
                     CDMW_ERROR(daemoninit_logger, "*** Error : " << DEBUG_FILE_OPTION
                              << "=FILE option must be defined");
                     usage(std::cout, argv[0]);
                     return FAILURE;
            }            
            options.debug_file = option_val;
            
            // bind to ns option
            option_val = OS::get_option_value(argc, argv, BIND_TO_NS_OPTION);
            if (option_val == "yes")
               options.bind_to_ns=true;
            else 
               options.bind_to_ns=false;
            // rebind to ns option
            option_val = OS::get_option_value(argc, argv, REBIND_TO_NS_OPTION);
            if (option_val == "yes")
               options.rebind_to_ns=true;
            else 
               options.rebind_to_ns=false;
            return SUCCESS;
        }
        
        
        /**
         * Initialise the orb and PlatformInterface
         *
         */
        void init_processing (
             int argc, char* argv[],
             bool platform_managed,
             CORBA::ORB*& p_orb,
             PortableServer::POA_out p_root_POA )
        {
            CORBA::ORB_var orb;
                    
#ifdef CDMW_USE_FAULTTOLERANCE
            // Initialise FT service
            Cdmw::FT::FTServiceInit::Init( argc, argv, true );
#endif

            // Allow specification of a value for the ORB connection timeout from
            // command-line arguments or a property file
            std::string dummy_prop = 
                Cdmw::OsSupport::SystemProperties::Get_Property(
                    Cdmw::OrbSupport::CDMW_ORB_CONNECTION_TIMEOUT_PROPERTY_NAME);
            if (dummy_prop.empty())
                Cdmw::OsSupport::SystemProperties::Set_Property(
                    Cdmw::OrbSupport::CDMW_ORB_CONNECTION_TIMEOUT_PROPERTY_NAME,
                    CONNECTION_TIMEOUT_PROPERTY_VALUE);

            // initialize the ORB.
            StrategyList orb_strategies;
            orb_strategies.add_OrbThreaded();
            orb_strategies.add_PoaThreadPool(POA_THREAD_POOL_SIZE);

            orb = Cdmw::OrbSupport::OrbSupport::ORB_init(argc, argv, orb_strategies);

#ifdef CDMW_USE_FAULTTOLERANCE
            // Set Request Duration policy
            CORBA::Object_var obj_pm = 
                orb->resolve_initial_references("ORBPolicyManager");            
            CORBA::PolicyManager_var policy_manager
                = CORBA::PolicyManager::_narrow(obj_pm.in());
            
            // Create and apply an ORB-wide Routed policy
            CORBA::Any any;
            TimeBase::TimeT duration =
                Cdmw::OsSupport::SystemProperties::Get_Property(
                    FT_REQUEST_DURATION_PROPERTY_NAME,
                    0);
            if (duration == 0) {
                duration = DEFAULT_FT_REQUEST_DURATION_PROPERTY_VALUE;
            }
            CDMW_INFO(daemoninit_logger,
                      "FT Request duration policy:" << duration << "");
            
            any <<= duration;
            CORBA::PolicyList policies(1);
            policies.length(1);
            policies[0] =
                orb->create_policy(::FT::REQUEST_DURATION_POLICY, any);
            policy_manager->set_policy_overrides(policies, CORBA::ADD_OVERRIDE);
            policies[0]->destroy();
#endif

            // get a reference to the root POA.
            CORBA::Object_var root_obj = orb->resolve_initial_references("RootPOA");
            PortableServer::POA_var root_POA =
                PortableServer::POA::_narrow(root_obj.in());

            // activate the POA Manager.
            PortableServer::POAManager_var POA_mngr = root_POA->the_POAManager();
            POA_mngr->activate();

/*
            // creates the process behaviour
            std::auto_ptr<MyProcessBehaviour> pProcessBehaviour(new MyProcessBehaviour( orb.in() ));

            // init Cdmw
            std::cout << " faulttoleraceclient: init Cdmw"<<std::endl;
            Cdmw::CdmwInit::CDMW_init(orb.in(), argc, argv,  pProcessBehaviour.get());
*/
            // return parameters
            p_orb = orb._retn();
            p_root_POA = root_POA._retn();
        }
    }


}; // End namespace Trace
}; // End namespace Cdmw
