/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

#include <stdio.h>
#include <iostream>

#include <Foundation/logging/LogManager.hpp>
#include <Foundation/logging/Logger.hpp>
#include <Foundation/logging/FunctionLogger.hpp>

#include "Foundation/ossupport/OS.hpp"
#include "Foundation/osthreads/Thread.hpp"

#include "TraceAndPerf/recordingtools/Display.hpp"
#include "TraceAndPerf/recordinglibrary/Swapping.hpp"
#include "TraceAndPerf/recordingidl/CdmwRecordingDebug.stub.hpp"

namespace
{
   static const char GIOP_MAGIC [4] = {'G', 'I', 'O', 'P'} ;
   static const int  GIOP_MAGIC_INT = *(const int *) GIOP_MAGIC ;
   const short INPUT_WAIT            = 1000 ;
   const short NO_EXCEPTION          = 0 ;
   const short USER_EXCEPTION        = 1 ;
   const short SYSTEM_EXCEPTION      = 2 ;
   const short LOCATION_FORWARD      = 3 ;
   const short LOCATION_FORWARD_PERM = 4 ;
   const short NEEDS_ADDRESSING_MOD  = 5 ;

}

namespace Cdmw
{
   namespace Recording
   {
       Cdmw::Logging::Logger* display_logger;

       /**
       *Purpose: Indicates that the output is limited to headers only
       *<p> .
       *
       */
      const unsigned int Display::M_Headers_Only     = 1 ;

      /**
       *Purpose: Indicates that the reading must continue even if
       * the end of the file is reached, in order to display
       * additionnal operations to come
       *<p> .
       *
       */
      const unsigned int Display::M_For_Ever_Reading = 2 ;

      const unsigned int DEFAULT_BUFFER_LENGTH = 65436;
      
      Display::Display(InterfaceMap & ifr_store,
                       Decoder & decoder)
            throw()
            : m_ifr_map(ifr_store)
            , m_decoder(decoder)
            , m_in_file(0)
            , m_out_file(stdout)
            , m_flags(0)
            , m_buffer_length(DEFAULT_BUFFER_LENGTH)
            , m_buffer(new octet[m_buffer_length])
            , m_stopped(true)
      {
        display_logger = Cdmw::Logging::Logger::Get_Logger("CDMW.TRC.RecordingDaemon");
        if (m_ifr_map.Headers_Only())
        {
          Set_Flags (Display::M_Headers_Only);
        }
        m_in_file_name = 0;

        //Set_Output ("/tmp/cdmw_decoder_output.log");
      }

      Display::~Display()
            throw()
      {
         Set_Output ("-") ;
         delete [] m_buffer ;
         m_buffer = 0;
      }

      /**
       *Purpose: Display the full contents of the given file
       *<p> .
       *
       */
      bool Display::Display_File (const char * file_name)
            throw()
      {
         FILE * in_file ;
         
         if (file_name == 0)
         {
            return false;
         }
         
         if ( strcmp (file_name, "-") != 0)
         {
            in_file = fopen (file_name, "r") ;

            if (in_file == 0)
            {
               // The file cannot be opened
               // Ignore the request to change
               // But tell the caller of the failure
               return false ;
            }
         }

         if (m_in_file == stdin)
         {
            if ((file_name != 0) && (strcmp (file_name, "-") != 0))
            {
               m_in_file = 0 ;
            }
         }
         else
         {
            if (m_in_file_name != 0)
            {
               if (strcmp (file_name, m_in_file_name) != 0)
               {
                  CORBA::string_free (m_in_file_name) ;
                  m_in_file_name = 0 ;
                  ::fclose (m_in_file) ;
                  m_in_file = 0 ;
               }
            }
         }

         if (file_name == 0)
         {
            m_in_file = 0 ;
            return false ;
         }

         if (m_in_file == 0)
         {
            if (strcmp (file_name, "-") == 0)
            {
               m_in_file = stdin ;
            }
            else
            {
               m_in_file = in_file ;
            }
            m_in_file_name = CORBA::string_dup (file_name) ;
         }

         m_stopped = false ;
         while ((!m_stopped) && Display_Next_Operation ())
         {
            CDMW_INFO(display_logger, "--- next operation :");
         }
         //JM
         if (m_in_file_name != 0)
         {
             CORBA::string_free (m_in_file_name) ;
             m_in_file_name = 0 ;
             ::fclose (m_in_file) ;
             m_in_file = 0 ;
         }
         
         m_stopped = true ;
         return true ;
      }

      /**
       *Purpose: Control the details of the display
       *<p> .
       *
       */
      void Display::Set_Flags (unsigned int flags)
            throw()
      {
         m_flags = flags ;
         m_decoder.SetConfiguration (flags) ;
      }

      /**
       *Purpose: File where to do the output
       *<p> .
       *
       */
      bool Display::Set_Output (const char * file)
            throw()
      {
         if (file == 0)
         {
            return false ;
         }

         ::fflush (m_out_file) ;

         FILE * out_file ;
         if (strcmp (file, "-") != 0)
         {
            out_file = fopen (file, "w") ;

            if (out_file == 0)
            {
               // The file cannot be opened
               // Ignore the request to change
               // But tell the caller of the failure
               return false ;
            }
         }

         if (m_out_file == stdout)
         {
            if (strcmp (file, "-") == 0)
            {
               return true ;
            }
         }
         else
         {
            if (strcmp (file, m_out_file_name) == 0)
            {
               return true ;
            }
            ::fclose (m_out_file) ;
            CORBA::string_free (m_out_file_name) ;
            m_out_file_name = 0 ;
         }

         if (strcmp (file, "-") == 0)
         {
            m_out_file = stdout ;
         }
         else
         {
            m_out_file = out_file ;
            m_out_file_name = CORBA::string_dup (file) ;
         }

         return true ;
      }

      /**
       *Purpose: Stop the display processing
       *<p> .
       *
       */
      bool Display::Is_Stopped ()
          throw()
      {
        return m_stopped;
      }
      
      /**
       *Purpose: Stop the display processing
       *<p> .
       *
       */
      void Display::Stop_Display ()
            throw()
      {
         m_stopped = true ;
      }

      /**
       *Purpose:  Display the header of a reply message
       *<p> .
       *
       */
      int Display::Display_Reply_Info (Message & reply)
            throw()
      {
         int index = 12 ; // Skip header

         // request_id
         index += (-index) & 3 ; //align to 4
         reply.request_id = * (unsigned int *) &reply.data[index] ;
         index += 4 ;

         // date
         reply.date = * (timeval *) &reply.data[index] ;
         index += 8 ;

         // params_endian
         reply.need_swap = needSwap (reply.data[index]) ;
         index ++ ;

         // status
         index += (-index) & 3 ; //align to 4
         index += 4 ; // Skip status

         time_t ltime = reply.date.tv_sec;
         char buffer1[100], buffer2[100];

         // use time structure to build a customized time string.
         tm now;
         ::localtime_r(&ltime, &now);

         // write time with second precision in buffer
         strftime(buffer1, 100, "%Y/%m/%d %H:%M:%S", &now);

         // add milliseconds and microseconds in buffer
         sprintf(buffer2,
                 "%.3ld:%.3ld ",
                 (reply.date.tv_usec) / 1000,
                 (reply.date.tv_usec) % 1000
                );

         // Display the header
         //unsigned long int date = (unsigned long int) reply.date.tv_sec * 1000000 + reply.date.tv_usec ;
         fprintf (m_out_file, "reply_to %d at %s:%s\n", reply.request_id, buffer1, buffer2) ;
//        fprintf (m_out_file, "reply_to %d at %d%03d\n",
//        reply.request_id, reply.date.tv_sec, reply.date.tv_usec/1000)
//        ;

         // service contexts
         index += (-index) & 3 ; //align to 4
         int number = * (int *) &reply.data[index] ;
         index += 4 ;

         fprintf (m_out_file, "%d service context tags:", number) ;

         for (int each = number ; each > 0; each --)
         {
            index += (-index) & 3 ; //align to 4
            unsigned int tag = * (unsigned int *) &reply.data[index] ;
            fprintf (m_out_file, " %x", tag) ;
            index += 4 ;
            int len = * (int *) &reply.data[index] ;
            index += 4 + len ;
         }

         fprintf (m_out_file, "\n") ;

         return index ;
      }

      /**
       *Purpose: Display the header of the operation
       *<p> .
       *
       */
      int Display::Display_Operation_Info (Message & op)
            throw()
      {
              
         int index = 12 ; // Skip header

         // Get Channel
         op.channel = * (Connection *) &op.data[index] ;
         index += 12 ;

         // server_id
         int len = * (int *) &op.data[index] ;
         index += 4 ;
         op.server_id = (const char *) &op.data[index] ;
         index += len ;

         // request_id
         index += (-index) & 3 ; //align to 4
         op.request_id = * (unsigned int *) &op.data[index] ;
         index += 4 ;

         // date
         op.date = * (timeval *) &op.data[index] ;
         index += 8 ;

         // response_flags
         index ++ ;

         // params_endian
         op.need_swap = needSwap (op.data[index]) ;
         index ++ ;

         // interface_name
         index += (-index) & 3 ; //align to 4
         len = * (int *) &op.data[index] ;
         index += 4 ;
         op.intf_rid = (const char *) &op.data[index] ;
         index += len ;

         // operation
         index += (-index) & 3 ; //align to 4
         len = * (int *) &op.data[index] ;
         index += 4 ;
         op.operation_Offset = index ;
         const char * operation = (const char *) &op.data[index] ;
         index += len ;

         // Display the header
         // unsigned long int date = (unsigned long int) op.date.tv_sec * 1000000 + op.date.tv_usec ;
         // fprintf (m_out_file, "reply_to %d at %d%03d\n", reply.request_id, reply.date.tv_sec, reply.date.tv_usec/1000) ;
         octet * ip_src = (octet *) &op.channel.src_addr ;
         octet * ip_dst = (octet *) &op.channel.dst_addr ;
         
         time_t ltime = op.date.tv_sec;
         char buffer1[100], buffer2[100];

         // use time structure to build a customized time string.
         tm now;
         ::localtime_r(&ltime, &now);

         // write time with second precision in buffer
         strftime(buffer1, 100, "%Y/%m/%d %H:%M:%S", &now);
         
         // add milliseconds and microseconds in buffer
         sprintf(buffer2,
                 "%.3ld:%.3ld ",
                 (op.date.tv_usec) / 1000,
                 (op.date.tv_usec) % 1000
                );
         
         fprintf (m_out_file, "request %d at %s:%s from %d.%d.%d.%d %d to %d.%d.%d.%d %d\n",
                  op.request_id, buffer1, buffer2, ip_src[0], ip_src[1], ip_src[2], ip_src[3], op.channel.src_port,
                  ip_dst[0], ip_dst[1], ip_dst[2], ip_dst[3], op.channel.dst_port) ;
         
//         fprintf (m_out_file, "request %d at %d%03d from %d.%d.%d.%d %d to %d.%d.%d.%d %d\n",
//                  op.request_id, op.date.tv_sec, op.date.tv_usec/1000, ip_src[0], ip_src[1], ip_src[2], ip_src[3], op.channel.src_port,
//                  ip_dst[0], ip_dst[1], ip_dst[2], ip_dst[3],
//                  op.channel.dst_port) ;

         fprintf (m_out_file, "server_id %s interface %s operation %s\n", op.server_id, op.intf_rid, operation) ;

         // service contexts
         index += (-index) & 3 ; //align to 4
         int number = * (int *) &op.data[index] ;
         index += 4 ;

         fprintf (m_out_file, "%d service context tags:", number) ;

         for (int each = number ; each > 0; each --)
         {
            index += (-index) & 3 ; //align to 4
            unsigned int tag = * (unsigned int *) &op.data[index] ;
            fprintf (m_out_file, " %x", tag) ;
            index += 4 ;
            len = * (int *) &op.data[index] ;
            index += 4 + len ;
         }

         fprintf (m_out_file, "\n") ;

         return index ;
      }

      /**
       *Purpose:  Display a LocateRequest message
       *<p> .
       *
       */
      void Display::Display_Locate_Request (Message & request)
          throw()
      {
        int index = 12 ; // Skip header

        // Get Channel
        request.channel = * (Connection *) &request.data[index] ;
        index += 12 ;

        octet * ip_src = (octet *) &request.channel.src_addr ;
        octet * ip_dst = (octet *) &request.channel.dst_addr ;

        // date
        request.date = * (timeval *) &request.data[index] ;
        index += 8 ;

        time_t ltime = request.date.tv_sec;
        char buffer1[100], buffer2[100];

        // use time structure to build a customized time string.
        tm now;
        ::localtime_r(&ltime, &now);

        // write time with second precision in buffer
        strftime(buffer1, 100, "%Y/%m/%d %H:%M:%S", &now);

        // add milliseconds and microseconds in buffer
        sprintf(buffer2,
                "%.3ld:%.3ld ",
                (request.date.tv_usec) / 1000,
                (request.date.tv_usec) % 1000
               );

        // request_id
        request.request_id = * (unsigned int *) &request.data[index] ;
        index += 4 ;

        // Display the header
        fprintf (m_out_file, "LocateRequest %d at %s:%s from %d.%d.%d.%d %d to %d.%d.%d.%d %d\n", request.request_id,
                 buffer1, buffer2, ip_src[0], ip_src[1], ip_src[2], ip_src[3], request.channel.src_port,
                 ip_dst[0], ip_dst[1], ip_dst[2], ip_dst[3], request.channel.dst_port) ;

        // Display the target
        // Not that much meaningful
      }

      /**
       *Purpose:  Display a LocateReply message
       *<p> .
       *
       */
      void Display::Display_Locate_Reply (Message & reply)
          throw()
      {
        int index = 12 ; // Skip header

        // date
        reply.date = * (timeval *) &reply.data[index] ;
        index += 8 ;

        time_t ltime = reply.date.tv_sec;
        char buffer1[100], buffer2[100];

        // use time structure to build a customized time string.
        tm now;
        ::localtime_r(&ltime, &now);

        // write time with second precision in buffer
        strftime(buffer1, 100, "%Y/%m/%d %H:%M:%S", &now);

        // add milliseconds and microseconds in buffer
        sprintf(buffer2,
                "%.3ld:%.3ld ",
                (reply.date.tv_usec) / 1000,
                (reply.date.tv_usec) % 1000
               );

        // request_id
        reply.request_id = * (unsigned int *) &reply.data[index] ;
        index += 4 ;

        // Display the header
        fprintf (m_out_file, "LocateReply %d at %s:%s\n", reply.request_id, buffer1, buffer2) ;

        // Display the status
        int status = reply.data[index] + reply.data[index + 3] ;
        index += 4 ;

        switch (status)
        {
          case 0:
            fprintf (m_out_file, "Object unknown\n") ;
            break ;
          case 1:
            fprintf (m_out_file, "Object here\n") ;
            break ;
          case 2:
            fprintf (m_out_file, "Object forward\n") ;
            break ;
          case 3:
            fprintf (m_out_file, "Object forward permanent\n") ;
            break ;
          case 4:
            fprintf (m_out_file, "Locate exception\n") ;
            break ;
          case 5:
            fprintf (m_out_file, "Locate needs addressing mode\n") ;
            break ;
        }
      }

      /**
       *Purpose: Display the next operation from the file
       * It returns false if no more operations are available and the
       * flag M_For_Ever_Reading is not set
       *<p> .
       *
       */
      bool Display::Display_Next_Operation ()
            throw()
      {
                 
         // Get operation request header
         GIOP_Header header ;

         int number = ::fread (&header, sizeof(GIOP_Header),  1, m_in_file) ;
       
         if ((* (int *) header.magic) != GIOP_MAGIC_INT)
         {
            fprintf (m_out_file, "\n###End_Of_File###\n") ;
            ::fflush (m_out_file) ;
            return false ;
         }

         while (number != 1)
         {
            if ((m_flags & M_For_Ever_Reading) == 0)
            {
              fprintf (m_out_file, "\n###End_Of_File###\n") ;
              ::fflush (m_out_file) ;
              return false ;
            }

            Cdmw::OsSupport::OS::sleep (INPUT_WAIT) ;
            number = ::fread (&header, sizeof(GIOP_Header), 1, m_in_file) ;
            if ((* (int *) header.magic) != GIOP_MAGIC_INT)
            {
               fprintf (m_out_file, "\n###End_Of_File###\n") ;
               ::fflush (m_out_file) ;
               return false ;
            }
         }

         // align the length
         header.length += (-header.length) & 3 ;
         // Adjust m_buffer if needed
         if (m_buffer_length < header.length + 12)
         {
            delete [] m_buffer ;
            m_buffer_length = header.length + 1024 ;  // take some margin
            m_buffer = new octet[m_buffer_length] ;
         }

         // Get operation request bytes
         memcpy (m_buffer, &header, sizeof(GIOP_Header)) ;
         int length = ::fread (m_buffer + sizeof(GIOP_Header), 1, header.length, m_in_file) ;
         while (length != header.length)
         {
            if ((m_flags & M_For_Ever_Reading) == 0)
            {
               fprintf (m_out_file, "\n###End_Of_File###\n") ;
               ::fflush (m_out_file) ;
               return false ;
            }

            Cdmw::OsSupport::OS::sleep (INPUT_WAIT) ;
            length += ::fread (m_buffer + sizeof(GIOP_Header) + length, 1, header.length - length, m_in_file) ;
         }

         // Build the request message
         Message msg ;
         msg.next = 0 ;
         msg.fragments = 0 ;
         msg.header = header ;
         msg.data = m_buffer ;

         int offset = 0 ;

         CORBA::OperationDescription_var op_desc ;

         if (header.message_type == CdmwRecording::Request)
         {
           // Display operation details
           offset = Display_Operation_Info (msg) ;

           // parameters if requested
           fprintf (m_out_file, "Parameters {") ;
           if ((m_flags & M_Headers_Only) == 0)
           {
             // Get operation definition
             op_desc = m_ifr_map.getOperationDef(msg.intf_rid, (const char*) (msg.data + msg.operation_Offset)) ;

             if (op_desc == 0)
             {
               fprintf (m_out_file, " unknown operation") ;
             }
             else
             {
               if (op_desc->parameters.length() == 0)
               {
                 fprintf (m_out_file, " none ") ;
               }
               else
               {
                 offset += ((-offset) & 3) + 4 ;
                 octet * params = msg.data + offset ;
                 offset = 0 ;
                 for (unsigned int each = 0; each < op_desc->parameters.length() ; each ++)
                 {
                   CORBA::ParameterDescription& param = op_desc->parameters[each] ;
                   if ((param.mode == CORBA::PARAM_IN) || (param.mode == CORBA::PARAM_INOUT))
                   {
                     fprintf (m_out_file, "%s => ", param.name.in()) ;
                     m_decoder.Decode_Parameter (params, offset, param.type.in(), msg.need_swap) ;
                   }
                 }

                 if (offset == 0)
                 {
                   fprintf (m_out_file, " none ") ;
                 }
               }
             }

             fprintf (m_out_file, " }\n") ;
           }
           else
           {
             fprintf (m_out_file, " not displayed }\n") ;
           }
         }
         else // LocateRequest
         {
           msg.need_swap = needSwap (header.flags);
           Display_Locate_Request (msg) ;
         }


         // Get operation reply header
         number = ::fread (&header, sizeof(GIOP_Header), 1, m_in_file) ;
         if ((* (int *) header.magic) != GIOP_MAGIC_INT)
         {
            fprintf (m_out_file, "\n###End_Of_File###\n") ;
            ::fflush (m_out_file) ;
            return false ;
         }

         while (number != 1)
         {
            if ((m_flags & M_For_Ever_Reading) == 0)
            {
               fprintf (m_out_file, "\n###End_Of_File###\n") ;
               ::fflush (m_out_file) ;
               return false ;
            }

            Cdmw::OsSupport::OS::sleep (INPUT_WAIT) ;
            number = ::fread (&header, sizeof(GIOP_Header), 1, m_in_file) ;
            if ((* (int *) header.magic) != GIOP_MAGIC_INT)
            {
              fprintf (m_out_file, "\n###End_Of_File###\n") ;
              ::fflush (m_out_file) ;
               return false ;
            }
         }

         if (header.length == 0)
         {
            fprintf (m_out_file, "\n") ;
            ::fflush (m_out_file) ;
            return true;
         }
         
         // align the length
         header.length += (-header.length) & 3 ;
         // Adjust m_buffer if needed
         if (m_buffer_length < header.length + 12)
         {
            delete [] m_buffer ;
            m_buffer_length = header.length + 1024 ;  // take some margin
            m_buffer = new octet[m_buffer_length] ;
         }

         // Get operation reply bytes
         memcpy (m_buffer, &header, sizeof(GIOP_Header)) ;
         length = ::fread (m_buffer + sizeof(GIOP_Header), 1, header.length, m_in_file) ;
         while (length != header.length)
         {
            if ((m_flags & M_For_Ever_Reading) == 0)
            {
               fprintf (m_out_file, "\n###End_Of_File###\n") ;
               ::fflush (m_out_file) ;
               return false ;
            }

            Cdmw::OsSupport::OS::sleep (INPUT_WAIT) ;
            length += ::fread (m_buffer + sizeof(GIOP_Header) + length, 1, header.length - length, m_in_file) ;
         }

         msg.header = header ;
         msg.data = m_buffer ;

         if (header.message_type == CdmwRecording::Reply)
         {
           // Display reply header
           offset = Display_Reply_Info (msg) ;
           int status = * (int *) (m_buffer + 28) ;

           // Display reply parameters if requested
           fprintf (m_out_file, "Parameters {") ;
           offset += ((-offset) & 3) + 4 ;
           octet * params = msg.data + offset ;
           offset = 0 ;

           if ((m_flags & M_Headers_Only) == 0)
           {
             if (op_desc == 0)
             {
               fprintf (m_out_file, " unknown operation") ;
             }
             else if (status == NO_EXCEPTION)
             {
               if (op_desc->parameters.length() == 0)
               {
                 fprintf (m_out_file, " none ") ;
               }
               else
               {
                 fprintf (m_out_file, "returns => ") ;
                 m_decoder.Decode_Parameter (params, offset, op_desc->result.in(), msg.need_swap) ;
                 for (unsigned int each = 0; each < op_desc->parameters.length() ; each ++)
                 {
                   CORBA::ParameterDescription& param = op_desc->parameters[each] ;
                   if ((param.mode == CORBA::PARAM_OUT) || (param.mode == CORBA::PARAM_INOUT))
                   {
                     fprintf (m_out_file, "%s => ", param.name.in()) ;
                     m_decoder.Decode_Parameter (params, offset, param.type.in(), msg.need_swap) ;
                   }
                 }
               }
             }
             else if (status == USER_EXCEPTION)
             {
               for (unsigned int each = 0; each < op_desc->exceptions.length() ; each ++)
               {
                 CORBA::ExceptionDescription& exc = op_desc->exceptions[each] ;
                 if (strcmp ((const char*) params + 4, exc.id.in()) == 0)
                 {
                   m_decoder.Decode_Parameter (params, offset, exc.type.in(), msg.need_swap) ;
                   break ;
                 }
               }
             }
             else if (status == SYSTEM_EXCEPTION)
             {
               m_decoder.Decode_System_Exception (params, offset, msg.need_swap) ;
             }
             else
             {
               fprintf (m_out_file, " forwarding") ;
             }

             fprintf (m_out_file, " }\n") ;
           }
           else
           {
             fprintf (m_out_file, " not displayed }\n") ;
           }
         }
         else // LocateReply
         {
           msg.need_swap = needSwap (header.flags);
           Display_Locate_Reply (msg) ;
         }

         fprintf (m_out_file, "\n") ;
         ::fflush (m_out_file) ;

         return true ;
      }

   }
}

