/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

#include <Foundation/logging/LogManager.hpp>
#include <Foundation/logging/Logger.hpp>
#include <Foundation/logging/FunctionLogger.hpp>
#include "TraceAndPerf/recordingtools/InterfaceMap.hpp"


namespace Cdmw
{
   namespace Recording
   {
      Cdmw::Logging::Logger* interfacemap_logger;
      InterfaceMap::InterfaceMap(const char * ifr_url)
            throw()
            : m_orb()
            , m_ifr()
            , m_operations()
      {
         int argc = 1 ;
         char* argv [1] ;
         argv [0] = "operation_decoder" ;

         // initialize the ORB.
         Cdmw::OrbSupport::StrategyList orb_strategies;
         orb_strategies.add_OrbThreaded();
         m_orb = Cdmw::OrbSupport::OrbSupport::ORB_init(argc, argv, orb_strategies) ;

         interfacemap_logger = Cdmw::Logging::Logger::Get_Logger("CDMW.TRC.RecordingDaemon");
         try
         {
           CORBA::Object_var obj = m_orb->string_to_object(ifr_url) ;
           m_ifr = CORBA::Repository::_unchecked_narrow (obj.in()) ;
         }
         catch (...)
         {
           CDMW_INFO(interfacemap_logger, "!!! IFR Repository not found");
           m_ifr = 0;
         }
      }

      InterfaceMap::~InterfaceMap()
            throw()
      {
      }

      /**
       *Purpose: return Headers_Only mode if the interface repository is bad
       *<p> .
       *
       */
      bool InterfaceMap::Headers_Only()
      {
        return (m_ifr == 0);
      }

      /**
       *Purpose: Get the operation description from the cache
       * querying the Interface Repository if needed
       *<p> .
       *
       */
      CORBA::OperationDescription * InterfaceMap::getOperationDef(const char * interface, const char * operation)
            throw()
      {
         Operation_Name op_name ;
         CORBA::OperationDescription * op_desc = 0 ;

         if (interface == 0 || operation == 0)
         {
           return 0;
         }
         
         op_name.interface = CORBA::string_dup (interface) ;
         op_name.operation = CORBA::string_dup (operation) ;

         OperationMap::iterator iter = m_operations.find(op_name) ;

         // If not 0, return it
         if (iter != m_operations.end())
         {
            return (*iter).second ;
         }

         // Else try to get it from IFR
         CORBA::InterfaceDef_var intf ;
         try
         {
           CORBA::Object_var obj = m_ifr->lookup_id (interface) ;
           intf = CORBA::InterfaceDef::_unchecked_narrow (obj.in()) ;
         }
         catch (...)
         {
           CDMW_INFO(interfacemap_logger, "!!! interface " << interface << " not found");
           return 0;
         }

         // JM
         if (intf == 0)
         {
           return 0;
         }
         
         if (operation[0] == '_')
         { // attribute or intrinsic
            if ((operation[1] == 's') || (operation[1] == 'g'))
            { // attribute
 
               CORBA::ContainedSeq_var list = intf->lookup_name (operation + 5, 1, CORBA::dk_Attribute, false) ;
               if (list->length() < 1)
               {
                 // attribute not found
                 return 0 ;
               }
               CORBA::AttributeDef_var attr = CORBA::AttributeDef::_unchecked_narrow (list.in()[0]) ;
               if (!attr)
               {
                 return 0 ;
               }
               op_desc = new CORBA::OperationDescription() ;
               if (operation[1] == 's')
               { // attribute setter
                  op_desc->parameters.length(1) ;
                  op_desc->parameters[0].name = CORBA::string_dup ("to_value") ;
                  op_desc->parameters[0].type = attr->type() ;
                  op_desc->parameters[0].mode = CORBA::PARAM_IN ;
               }
               else
               { // attribute getter
                  op_desc->result = attr->type() ;
               }
               op_desc->name = CORBA::string_dup (operation) ;
               m_operations[op_name] = op_desc ;
            }
            else
            {  // intrinsic operation
               return 0 ;
            }
         }
         else
         {
            CORBA::ContainedSeq_var list = intf->lookup_name (operation, 1, CORBA::dk_Operation, false) ;
            if (list->length() < 1)
            {
              // operation not found
              return 0 ;
            }
            CORBA::OperationDef_var op = CORBA::OperationDef::_unchecked_narrow (list.in()[0]) ;
            if (!op)
            {
              return 0 ;
            }
            op_desc = new CORBA::OperationDescription() ;
            CORBA::ParDescriptionSeq_var params = op->params() ;
            op_desc->parameters.length (params->length()) ;
            for (int each = op_desc->parameters.length() - 1 ; each >= 0 ; each--)
            {
              op_desc->parameters[each].name = params.in()[each].name ;
              op_desc->parameters[each].type = params.in()[each].type ;
              op_desc->parameters[each].mode = params.in()[each].mode ;
            }
            CORBA::ExceptionDefSeq_var excepts = op->exceptions() ;
            op_desc->exceptions.length (excepts->length()) ;
            for (int each = op_desc->exceptions.length() - 1 ; each >= 0 ; each--)
            {
               op_desc->exceptions[each].id = excepts.in()[each]->id() ;
               op_desc->exceptions[each].type = excepts.in()[each]->type() ;
            }

            op_desc->result = op->result() ;

            op_desc->name = CORBA::string_dup (operation) ;
            m_operations[op_name] = op_desc ;
         }

         return op_desc ;
      }

   }
}
