/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

#include <clocale>
#include <fstream>
#include <iostream>
#include <sstream>
#include <string>
#include <vector>

#include "Foundation/ossupport/Signals.hpp"
#include "Foundation/ossupport/SignalHandler.hpp"

#include <Foundation/common/Assert.hpp>
#include <Foundation/common/Exception.hpp>
#include <Foundation/common/Locations.hpp>
#include <Foundation/common/Options.hpp>
#include <Foundation/common/System.hpp>
#include <Foundation/commonsvcs/initutils/InitUtils.hpp>
#include <Foundation/ossupport/OS.hpp>
#include <Foundation/ossupport/SystemProperties.hpp>
#include <Foundation/orbsupport/CORBA.hpp>
#include <Foundation/orbsupport/ExceptionMinorCodes.hpp>
#include <Foundation/orbsupport/OrbSupport.hpp>
#include <Foundation/orbsupport/StrategyList.hpp>
#include <Foundation/logging/LogManager.hpp>
#include <Foundation/logging/Logger.hpp>

#include <SystemMngt/platforminterface/PlatformInterface.hpp>
#include <Repository/repositoryinterface/RepositoryInterface.hpp>

//#include <ConfAndPlug/cdmwinit/ProcessControl.hpp>
#include <ConfAndPlug/cdmwinit/CdmwInterface.hpp>

#include "recordingtools/CdmwRecordingGenericRecorder_impl.hpp"
//#include "recordingtools/DaemonInit.hpp"
#include "recordingtools/CdmwRecordingContainerManager_impl.hpp"
//#include "recordingtools/CdmwRecordingDaemonControl_impl.hpp"

namespace
{
  
    const std::string GENERIC_RECORDER_NAME_OPT = "--CdmwGenericRecorderName";    
    const int POA_THREAD_POOL_SIZE = 10;    

    // default port number used by the generic recording daemon
    const std::string DFLT_SERVICE_PORT = "59886";
        
    // ORB reference
    CORBA::ORB_var orb = CORBA::ORB::_nil();
    Cdmw::Logging::Logger * logger = 0;
    
    const unsigned short SUCCESS = 0;
    const unsigned short FAILURE = 1;
    const unsigned short EXIT = 2;   

    // Print the usage.
    static void usage(std::ostream& out, const std::string& programName)
    {
        out << "usage: " << programName << " options...\n"
            << "options:\n"
            << "  --help                             " << "Display the help.\n"
            << "  [" << GENERIC_RECORDER_NAME_OPT << "]   " << "Name of the Generic Recorder.\n"
            << std::endl;
    }

}; // anonymous namespace

using namespace Cdmw;
using namespace Cdmw::OsSupport;
using namespace Cdmw::OrbSupport;
//using namespace Cdmw::Trace;

//
// end of processing
//
void end_recording_daemon(void)
{
    //nothing to do
}


//
// main module
//
int main(int argc, char* argv[])
{
    Cdmw::OsSupport::SystemProperties::Init (argc, argv);

    Cdmw::Logging::LogManager::Init(argc, argv);
    logger = Cdmw::Logging::Logger::Get_Logger("CDMW.TRC.GenericRecording");
    
    // Get the options
    std::string option_val;

    if ( (argc == 1) && (strcmp(argv[1], "--help"))  == 0) 
    {
        usage(std::cout, argv[0]);
        return EXIT;
    }
            
    option_val = OS::get_option_value(argc, argv, GENERIC_RECORDER_NAME_OPT);
    if (option_val == "no")  {
            option_val = "";
    }
    else if (option_val == "yes")  {
            CDMW_ERROR(logger, "***** Error : " << GENERIC_RECORDER_NAME_OPT << "=NAME option must be defined");
            usage(std::cout, argv[0]);
            return FAILURE;
    }
    std::string generic_recorder_name = option_val;

    int status = SUCCESS;
    
    try 
    {
        // Initialise FT service
        Cdmw::CdmwInit::CDMW_preORBInit(argc, argv);

        // Get current hostname
        std::string host_name = OS::get_hostname();
    
        std::ostringstream oss; 
        oss << "GenericRecorder: 'main_generic_recording_daemon' starting on  '"<< host_name << "' with args:" << std::endl;
        for (int i=1; i<argc; ++i) {
          oss << "   -> argv[" << i << "] \"" << argv[i] << "\"" << std::endl;
        }
        CDMW_LOG(logger, Cdmw::Logging::CDMW_DEBUG_3, oss);
        
        CDMW_INFO(logger, "***** CDMW Recording Generic Recorder Daemon STARTING on << '"
                          << host_name << "' : " << generic_recorder_name << " ...");
        
        Cdmw::OrbSupport::StrategyList strategyList;
        strategyList.add_OrbThreaded(); // Mandatory in OrbSupport::ORB_init
        strategyList.add_PoaThreadPool(POA_THREAD_POOL_SIZE);

        // Initialises the ORB
        CORBA::ORB_var orb = Cdmw::OrbSupport::OrbSupport::ORB_init(argc, argv, strategyList, host_name.c_str());

        // Get the root POA
        CORBA::Object_var obj = orb->resolve_initial_references("RootPOA");
        PortableServer::POA_var rootPOA = PortableServer::POA::_narrow(obj.in());
        PortableServer::POAManager_var poaMgr = rootPOA->the_POAManager();
        poaMgr->activate();

        // Init Cdmw
        Cdmw::CdmwInit::CDMW_init(orb.in(), argc, argv);
        CDMW_LOG(logger, Cdmw::Logging::CDMW_DEBUG_3, 
                 "GenericRecorder: CDMW_init() done");

        // Get current application and process name
        std::string app_name  = Cdmw::PlatformMngt::PlatformInterface::Get_application_name();
        std::string proc_name = Cdmw::PlatformMngt::PlatformInterface::Get_process_name();
    
        // Initialise default Generic Recorder name : <proc_name>@<app_name>.recorder
        if (generic_recorder_name == "") {
            generic_recorder_name = proc_name + "@";
            generic_recorder_name +=  app_name + ".recorder";

        }
        CDMW_LOG(logger, Cdmw::Logging::CDMW_DEBUG_3, 
                 "GenericRecorder: generic_recorder_name = " << generic_recorder_name);
            
        // Activate the Generic Recorder servant
        CdmwRecording::GenericRecorder_impl * generic_recorder
                = new CdmwRecording::GenericRecorder_impl(rootPOA, orb.in(), logger);
        CdmwRecording::GenericRecorder_var generic_recorder_ref
                = generic_recorder->_this();
        generic_recorder->_remove_ref();
        
        // Get the Recording Daemon Container Manager
        //std::cout << "GenericRecorder: retrieve CdmwRecording::ContainerManager" << std::endl;
        CORBA::Object_var o_contMgr = orb->resolve_initial_references(CdmwRecording::REC_CONTAINER_MNGR_ID);
        CdmwRecording::ContainerManager_var contMgr = CdmwRecording::ContainerManager::_narrow(o_contMgr.in());
        if(CORBA::is_nil(contMgr.in())) {
            CDMW_ERROR(logger, "***** Error: could not find RecordingDaemonContainerMgr"); 
            ::exit(FAILURE);
        }

        //std::cout << "GenericRecorder: resolve_initial_references(\"LocalNameService\")" << std::endl;
        CORBA::Object_var obj2 =  orb->resolve_initial_references("LocalNameService");  
        if(CORBA::is_nil(obj2.in())) {
           CDMW_ERROR(logger, "*** Error : `LocalNameService' is a nil object reference");
            ::exit(FAILURE);
        }
        //std::cout << "GenericRecorder: CosNaming::NamingContextExt::_narrow(obj2)" << std::endl;
        CosNaming::NamingContextExt_var naming_ctx = CosNaming::NamingContextExt::_narrow(obj2.in());
        if(CORBA::is_nil(naming_ctx.in())) {
           CDMW_ERROR(logger, "*** Error : `LocalNameService' is not a NamingContextExt object");
            ::exit(FAILURE);
        }

        //std::cout << "GenericRecorder: get Cdmw::CommonSvcs::Naming::NamingInterface" << std::endl;
        Cdmw::CommonSvcs::Naming::NamingInterface ni(naming_ctx.in());

// ==============
        // Get the Recording Daemon Controller
        //std::cout << "GenericRecorder: retrieve CdmwRecording::DaemonControl" << std::endl;

        CORBA::Object_var o_daemonCtrl;
        try {
           o_daemonCtrl = ni.resolve(CdmwRecording::REC_DAEMON_CTRL_ID);
           if (CORBA::is_nil(o_daemonCtrl.in())) {
              CDMW_ERROR(logger, "Nil Object Reference for '" << CdmwRecording::REC_DAEMON_CTRL_ID << "'.");
            ::exit(FAILURE);
           }
        } catch (const CosNaming::NamingContext::NotFound& nf) {
           CDMW_ERROR(logger, "Could not retrieve reference of the Recording Daemon."
                     << "'" << CdmwRecording::REC_DAEMON_CTRL_ID << "' not found in the naming.");
            ::exit(FAILURE);
        }
            
        CdmwRecording::DaemonControl_var daemonCtrl = CdmwRecording::DaemonControl::_narrow(o_daemonCtrl);
        if(CORBA::is_nil(daemonCtrl.in())) {
            CDMW_ERROR(logger, "***** Error: could not find RecordingDaemonDaemonControl"); 
            ::exit(FAILURE);
        }

        for( unsigned int i=0 ; i< CdmwRecording::REC_DAEMON_MAX_RECORDERS; i++)
        {
           CORBA::String_var url_str ;
           if ( daemonCtrl->state()->recorders_urls[i] != NULL)
           {
              url_str = CORBA::String_var(daemonCtrl->state()->recorders_urls[i]);
              std::cout << "---> recording Server URL ["<< i <<"] = " << url_str << std::endl;
           }
           else std::cout << "---> recording Server URL ["<< i <<"] = 0 " << std::endl;
        }
// ==============
        
        // Register a list of Object to be recorded
        CdmwRecording::Object_List Obj_List(1);
        Obj_List.length(1);
        Obj_List[0] = CORBA::Object::_duplicate(generic_recorder_ref.in());

        // Server location is 'host/appname/procname'
        std::ostringstream server_id;
        server_id << host_name << "/" << app_name << "/" << proc_name;

        CDMW_LOG(logger, Cdmw::Logging::CDMW_DEBUG_3, 
                 "GenericRecorder: Register Generic Recorder daemon for recording - server_id = "
                 << server_id.str());
        contMgr->record_objects (server_id.str().c_str(), Obj_List); 

        // Bind Generic Recorder daemon to NS
        bool force=true;
        CDMW_LOG(logger, Cdmw::Logging::CDMW_DEBUG_3, 
                 "GenericRecorder: Bind Generic Recorder daemon '" << generic_recorder_name << "' to NamingService");
        ni.rebind(generic_recorder_name, generic_recorder_ref.in(), force);

        // init the traces delegation to the Daemon LogCollector
//         std::cout << "GenericRecorder: activate LogCollector" << std::endl;
//         Cdmw::CommonSvcs::InitUtils* initUtils =
//             Cdmw::CommonSvcs::InitUtils::Instance();
//         try
//         {
//             initUtils->activate_LogCollector(naming_ctx.in());
//         }
//         catch (const Cdmw::Exception& ex) {
//             CDMW_WARN(logger,"Trace logs will not be sent to Trace Collector: " << ex.what());
//         }

        // Setup signal handling for ORB to be shut down
        Cdmw::OrbSupport::OrbSupport::install_orb_shutdown_signal_handlers(orb.in());
        CDMW_LOG(logger, Cdmw::Logging::CDMW_DEBUG_3, 
                 "GenericRecorder: ORB is running...");
        orb->run();
        contMgr->discard_objects(Obj_List);
        
        // end of process
        CDMW_INFO(logger, "***** CDMW Recording Generic Recorder Daemon STOPPING on << '"
                          << host_name << "' : " << generic_recorder_name << " ...");
    }
    catch (const Cdmw::Exception& ex) {
        CDMW_ERROR(logger, "***** Error - " << ex.what());
        status = FAILURE;
    }
    catch(const CORBA::Exception& ex) {
        CDMW_ERROR(logger, "***** Error - " << ex );
        status = FAILURE;
    }

    // end of corba processing
    if(!CORBA::is_nil(orb.in())) 
    {
        try 
        {
            Cdmw::OrbSupport::OrbSupport::ORB_cleanup(orb.in());
            orb->destroy();
            Cdmw::OrbSupport::OrbSupport::uninstall_orb_shutdown_signal_handlers(orb.in());
        }
        catch(const CORBA::Exception& ex) {
            CDMW_ERROR(logger, "*** Error : " << ex );
            status = FAILURE;
        }
    }

    return status;
}
