/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

package cdmw.recording.gui;

import java.io.IOException;
import java.net.URL;
import java.util.Vector;

import javax.xml.parsers.ParserConfigurationException;
import javax.xml.parsers.SAXParser;
import javax.xml.parsers.SAXParserFactory;

import org.xml.sax.Attributes;
import org.xml.sax.SAXException;
import org.xml.sax.SAXParseException;
import org.xml.sax.helpers.DefaultHandler;

public class PortParser extends DefaultHandler
{
   /**
    * The xml file we need to parse.
    */
   private URL file;

   /**
    * Boolean flag. True when we've found the port list.
    */
   private boolean portListFound = false;
   
   /**
    * Boolean flag. True when we've found an port.
    */
   private boolean portFound = false;

   /**
    * Vector of ports.
    */
   Vector<Short> values = new Vector<Short> ();
  
   /**
    * Constructor.
    * @param filename The file.
    */
   public PortParser (URL file)
   {
      this.file = file;
   }
   
   /**
    * Parse method.
    * @return true if parse was successful.
    */
   public boolean parse ()
      throws SAXParseException, SAXException, ParserConfigurationException, IOException
      
   {   
      boolean result = false;
      
      /* Enable validation of the xml. */
      SAXParserFactory factory = SAXParserFactory.newInstance ();
      factory.setValidating (true);

      /* Parse the input */
      try
      {
         SAXParser saxParser = factory.newSAXParser ();
         saxParser.parse (file.openStream (), this);
         result = true;
      }
      catch (SAXParseException spe)
      {
         /* Error generated by the parser. */
         System.out.println ("Error parsing xml : line " + spe.getLineNumber () 
                             + ", uri " + spe.getSystemId ());
         System.out.println (spe.getMessage ());

         /* Use the contained exception, if any. */
         Exception x = spe;
         if (spe.getException () != null)
         {
            x = spe.getException ();
         }
         x.printStackTrace ();
         
         throw spe;
      }
      catch (SAXException sxe)
      {
         /* Error generated by this application (or a parser-initialization error). */
         Exception x = sxe;
         if (sxe.getException () != null)
         {
            x = sxe.getException ();
         }
         x.printStackTrace ();

         throw sxe;
      }
      catch (ParserConfigurationException pce)
      {
         /* Parser with specified options can't be built. */
         pce.printStackTrace ();
         
         throw pce;
      }
      catch (IOException ioe)
      {
         /* I/O error. */
         ioe.printStackTrace ();
         
         throw ioe;
      }

      return result;
   }

   /**
    * Return the values we have parsed.
    * @return The values
    */
   public Short[] getPorts ()
   {
      Short[] ports = new Short[values.size ()];
      values.copyInto (ports);
      return ports;
   }
   
   // ===========================================================
   // SAX DocumentHandler methods
   // ===========================================================

   /**
    * This method will be called when the end of the document is detected.
    */
   public void endDocument () throws SAXException
   {
      if (values.size () == 0)
      {
         throw new SAXException ("No port values found.");
      }
   }

   /**
    * This method is called when an element is detected.
    * @param namespaceURI The URI
    * @param lName Local name
    * @param qName Qualified name
    * @param attrs Attributes
    * @throws SAXException
    */
   public void startElement (String namespaceURI,
                             String lName,
                             String qName,
                             Attributes attrs) 
      throws SAXException
   {
      String eName = lName; 
      
      if ("".equals (eName))
      {
         /* Namespace aware = false. */
         eName = qName; 
      }
      
      if (eName.equals ("PortList"))
      {
         /* Do nothing for the port list except set a flag and continue. */
         portListFound = true;
      }
      else if (eName.equals ("Port") && portListFound == true)
      {
         portFound = true;
      }
   }

   /**
    * This method is called when the end of an element is reached.
    * @param namespaceURI The URI
    * @param sName Simple name
    * @param qName Qualified name
    * @throws SAXEception
    */
   public void endElement (String namespaceURI, 
                           String sName,
                           String qName)
      throws SAXException
   {
      String eName = sName; 
      
      if ("".equals (eName))
      {
         /* Namespace aware = false. */
         eName = qName; 
      }
      
      if (eName.equals ("Port"))
      {
         portFound = false;
      }
   }
   
   public void characters(char buf[], int offset, int len)
    throws SAXException
   {
      if (portFound == true)
      {
         String port = new String(buf, offset, len);
         try
         {
            values.add (new Short (port));
         }
         catch (NumberFormatException nex)
         {
            System.out.println ("NumberFormatException reading the port xml file : " + nex);
            System.exit (1);
         }
      }
   }

   // ===========================================================
   // SAX ErrorHandler methods
   // ===========================================================

   /**
    * This method is called when an error is detected during validation.
    * We treat validation errors as fatal and throw them on.
    * @param e The exception raised due to the error
    * @throws SAXParseException
    */
   public void error (SAXParseException e) 
      throws SAXParseException
   {
      throw e;
   }

   /**
    * This method is called when a warning is detected during validation.
    * @param e The exception raised due to the error
    * @throws SAXParseException
    */
   public void warning (SAXParseException e) throws SAXParseException
   {
      System.out.println ("Warning while parsing xml: line " + e.getLineNumber ()
                          + ", uri " + e.getSystemId ());
      System.out.println (e.getMessage ());
   }
   
   public static void main (String args[])
   {
      try
      {
         PortParser pp = new PortParser (new URL (args[0]));
         pp.parse ();
      }
      catch (Exception ex)
      {
         System.out.println (ex);
      }
   }
}
