/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */
package cdmw.recording.gui;

import java.text.Collator;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.Comparator;
import java.util.List;
import javax.swing.AbstractListModel;
import javax.swing.ListModel;
import javax.swing.event.ListDataEvent;
import javax.swing.event.ListDataListener;

/**
 * SortedListModel decorates an unsorted ListModel to provide a sorted model.
 * You can create a SortedListModel from models you already have. Place the
 * SortedListModel into a JList, for example, to provide a sorted view of your
 * underlying model.
 */
public class SortedListModel extends AbstractListModel
{
   /**
    * Generated SerialVersionUID.
    */
   private static final long serialVersionUID = 4223092961427098149L;

   /**
    * The list structure that we use to store the data.
    */
   private List<SortedListEntry> sortedModel;

   /**
    * The underlying model.
    */
   private ListModel unsortedModel;

   /**
    * The comparator.
    */
   private Comparator<Object> comparator;

   /**
    * The current sort order we're using.
    */
   private SortOrder sortOrder;

   /**
    * The sort order enumeration.
    */
   public enum SortOrder
   {
      UNORDERED, ASCENDING, DESCENDING;
   }

   /**
    * Private default constructor.
    */
   private SortedListModel ()
   {
   }

   /**
    * Create a SortedListModel from an existing model using a default text
    * comparator for the default Locale. Sort in ascending order.
    * 
    * @param <code>model</code> the underlying, unsorted ListModel.
    */
   public SortedListModel (ListModel model)
   {
      this (model, SortOrder.ASCENDING, null);
   }

   /**
    * Create a SortedListModel from an existing model using a specific
    * comparator and sort order. Use a default text comparator.
    * 
    * @param <code>model</code> the underlying, unsorted ListModel.
    * @param <code>sortOrder</code> that should be used.
    */
   public SortedListModel (ListModel model, SortOrder sortOrder)
   {
      this (model, sortOrder, null);
   }

   /**
    * Create a SortedListModel from an existing model. Sort the model in the
    * specified sort order using the given comparator.
    * 
    * @param <code>model</code> the underlying, unsorted ListModel.
    * @param <code>sortOrder</code> that should be used.
    * @param <code>comp</code> the comparator to use.
    * 
    */
   public SortedListModel (ListModel model, SortOrder sortOrder, Comparator<Object> comp)
   {
      unsortedModel = model;
      
      unsortedModel.addListDataListener (new ListDataListener ()
      {
         public void intervalAdded (ListDataEvent e)
         {
            unsortedIntervalAdded (e);
         }

         public void intervalRemoved (ListDataEvent e)
         {
            unsortedIntervalRemoved (e);
         }

         public void contentsChanged (ListDataEvent e)
         {
            unsortedContentsChanged (e);
         }
      });
      
      this.sortOrder = sortOrder;
      
      if (comp != null)
      {
         comparator = comp;
      }
      else
      {
         comparator = Collator.getInstance ();
      }

      int size = model.getSize ();
      sortedModel = new ArrayList<SortedListEntry> (size);
      for (int x = 0; x < size; x++)
      {
         SortedListEntry entry = new SortedListEntry (x);
         int insertionPoint = findInsertionPoint (entry);
         sortedModel.add (insertionPoint, entry);
      }
   }

   /**
    * Retrieve the sorted entry from the original model.
    * 
    * @param <code>index</code> index of an entry in the sorted model.
    * @return element in the original model to which our entry points.
    */
   public Object getElementAt (int index) throws IndexOutOfBoundsException
   {
      int modelIndex = toUnsortedModelIndex (index);
      Object element = unsortedModel.getElementAt (modelIndex);
      return element;
   }

   /**
    * Retrieve the size of the underlying model.
    * 
    * @return size of the model.
    */
   public int getSize ()
   {
      int size = sortedModel.size ();
      return size;
   }

   /**
    * Convert sorted model index to an unsorted model index.
    * 
    * @param <code>index</code> an index in the sorted model.
    * @return modelIndex an index in the unsorted model.
    * 
    */
   public int toUnsortedModelIndex (int index) throws IndexOutOfBoundsException
   {
      int modelIndex = -1;
      SortedListEntry entry = sortedModel.get (index);
      modelIndex = entry.getIndex ();
      return modelIndex;
   }

   /**
    * Convert an array of sorted model indices to their unsorted model indices.
    * Sort the resulting set of indices.
    * 
    * @param <code>sortedSelectedIndices</code> indices of selected elements in 
    * the sorted model or sorted view.
    * @return unsortedSelectedIndices selected indices in the unsorted model.
    */
   public int[] toUnsortedModelIndices (int[] sortedSelectedIndices)
   {
      int[] unsortedSelectedIndices = new int[sortedSelectedIndices.length];
      int x = 0;
      
      /* 
       * This loop sets sortedIndex to each of the values of the sortedSelectedIndices
       * array in turn and builds the list of unsortedSelectedIndices one at a time.
       */
      for (int sortedIndex : sortedSelectedIndices)
      {  
         unsortedSelectedIndices[x++] = toUnsortedModelIndex (sortedIndex);
      }
 
      /* Sort the array of indices before returning. */
      Arrays.sort (unsortedSelectedIndices);
      return unsortedSelectedIndices;
   }

   /**
    * Convert an unsorted model index to a sorted model index.
    * 
    * @param <code>unsortedIndex</code> an element index in the unsorted model.
    * @return sortedIndex an element index in the sorted model.
    */
   public int toSortedModelIndex (int unsortedIndex)
   {
      int sortedIndex = -1;
      int x = -1;

      /*
       * This loop sets entry to each of the values in the sortedModel list and
       * works through them one at a time until the one with the correct original
       * (i.e. unsorted) index is found so that the sorted index can be returned.
       */
      for (SortedListEntry entry : sortedModel)
      {
         x++;
         if (entry.getIndex () == unsortedIndex)
         {
            sortedIndex = x;
            break;
         }
      }
      return sortedIndex;
   }

   /**
    * Convert an array of unsorted model selection indices to indices in the
    * sorted model. Sort the model indices from low to high to duplicate JList's
    * getSelectedIndices method.
    * 
    * @param <code>unsortedModelIndices</code>
    * @return an array of selected indices in the sorted model.
    */
   public int[] toSortedModelIndices (int[] unsortedModelIndices)
   {
      int[] sortedModelIndices = new int[unsortedModelIndices.length];
      int x = 0;

      /* 
       * This loop sets unsortedIndex to each of the values of the unsortedModelIndices
       * array in turn and builds the list of sortedSelectedIndices one at a time.
       */
      for (int unsortedIndex : unsortedModelIndices)
      {
         sortedModelIndices[x++] = toSortedModelIndex (unsortedIndex);
      }
      Arrays.sort (sortedModelIndices);
      return sortedModelIndices;
   }

   /**
    * Reset the model data.
    */
   private void resetModelData ()
   {
      int index = 0;
      
      /*
       * This loop works through each entry in the sortedModel list 
       * incrementing their indices by one throughout the collection.
       */
      for (SortedListEntry entry : sortedModel)
      {
         entry.setIndex (index++);
      }
   }

   /**
    * Set a new comparator to be used in sorting.
    * 
    * @param <code>comp</code> the comparator.
    */
   public void setComparator (Comparator<Object> comp)
   {
      if (comp == null)
      {
         sortOrder = SortOrder.UNORDERED;
         comparator = Collator.getInstance ();
         resetModelData ();
      }
      else
      {
         comparator = comp;
         Collections.sort (sortedModel);
      }
      fireContentsChanged (ListDataEvent.CONTENTS_CHANGED, 0,
                           sortedModel.size () - 1);
   }

   /**
    * Change the sort order of the model at runtime.
    * 
    * @param <code>sortOrder</code> the new sort order.
    */
   public void setSortOrder (SortOrder sortOrder)
   {
      if (this.sortOrder != sortOrder)
      {
         this.sortOrder = sortOrder;
         if (sortOrder == SortOrder.UNORDERED)
         {
            resetModelData ();
         }
         else
         {
            Collections.sort (sortedModel);
         }
         fireContentsChanged (ListDataEvent.CONTENTS_CHANGED, 0,
                              sortedModel.size () - 1);
      }
   }

   /**
    * Update the sorted model whenever new items are added to the
    * original/decorated model.
    * 
    */
   private void unsortedIntervalAdded (ListDataEvent e)
   {
      int begin = e.getIndex0 ();
      int end = e.getIndex1 ();
      int nElementsAdded = end - begin + 1;

      /*
       * Items in the decorated model have shifted in flight. Increment our
       * model pointers into the decorated model. We must increment indices that
       * intersect with the insertion point in the decorated model.
       */
      for (SortedListEntry entry : sortedModel)
      {
         int index = entry.getIndex ();
         /* 
          * If our model points to a model index >= to where
          * new model entries are added, we must bump up their index. 
          */
         if (index >= begin)
         {
            entry.setIndex (index + nElementsAdded);
         }
      }

      /* 
       * Now add the new items from the decorated model. 
       */
      for (int x = begin; x <= end; x++)
      {
         SortedListEntry newEntry = new SortedListEntry (x);
         int insertionPoint = findInsertionPoint (newEntry);
         sortedModel.add (insertionPoint, newEntry);
         fireIntervalAdded (ListDataEvent.INTERVAL_ADDED, insertionPoint,
                            insertionPoint);
      }
   }

   /**
    * Update this model when items are removed from the original/decorated
    * model. Also, let our listeners know that we've removed items.
    * 
    * @param <code>e</code> The event.
    */
   private void unsortedIntervalRemoved (ListDataEvent e)
   {
      int begin = e.getIndex0 ();
      int end = e.getIndex1 ();
      int nElementsRemoved = end - begin + 1;

      /*
       * Move from end to beginning of our sorted model, updating element
       * indices into the decorated model or removing elements as necessary.
       */
      int sortedSize = sortedModel.size ();
      boolean[] bElementRemoved = new boolean[sortedSize];
      for (int x = sortedSize - 1; x >= 0; x--)
      {
         SortedListEntry entry = sortedModel.get (x);
         int index = entry.getIndex ();
         if (index > end)
         {
            entry.setIndex (index - nElementsRemoved);
         }
         else if (index >= begin)
         {
            sortedModel.remove (x);
            bElementRemoved[x] = true;
         }
      }
      
      /*
       * Let listeners know that we've removed items.
       */
      for (int x = bElementRemoved.length - 1; x >= 0; x--)
      {
         if (bElementRemoved[x])
         {
            fireIntervalRemoved (ListDataEvent.INTERVAL_REMOVED, x, x);
         }
      }
   }

   /**
    * Resort the sorted model if there are changes in the original unsorted
    * model. Let any listeners know about changes. Since I don't track specific
    * changes, sort everywhere and redisplay all items.
    * 
    * @param <code>e</code> The event.
    */
   private void unsortedContentsChanged (ListDataEvent e)
   {
      Collections.sort (sortedModel);
      fireContentsChanged (ListDataEvent.CONTENTS_CHANGED, 0,
                           sortedModel.size () - 1);
   }

   /**
    * Internal helper method to find the insertion point for a new entry in the
    * sorted model.
    * 
    * @param <code>entry</code> The entry.
    */
   private int findInsertionPoint (SortedListEntry entry)
   {
      int insertionPoint = sortedModel.size ();
      if (sortOrder != SortOrder.UNORDERED)
      {
         insertionPoint = Collections.binarySearch ((List<SortedListEntry>) sortedModel, entry);
         if (insertionPoint < 0)
         {
            insertionPoint = -(insertionPoint + 1);
         }
      }
      return insertionPoint;
   }

   /**
    * The inner class that is a sorted list entry.
    */
   class SortedListEntry implements Comparable<SortedListEntry>
   {
      /**
       * Default constructor.
       */
      private SortedListEntry ()
      {
      }

      /**
       * Construct a new sorted list entry with a specific index.
       * 
       * @param <code>index</code> the index.
       */
      public SortedListEntry (int index)
      {
         this.index = index;
      }

      /**
       * Get the index of this entry.
       * 
       * @return the index.
       */
      public int getIndex ()
      {
         return index;
      }

      /**
       * Set the index for this entry.
       * 
       * @param <code>index</code> the index.
       */
      public void setIndex (int index)
      {
         this.index = index;
      }

      /**
       * Compare this object to the given object.
       * 
       * @param <code>sle</code> The object.
       */
      public int compareTo (SortedListEntry sle)
      {
         /* 
          * Retrieve the element that this entry points to
          * in the original model. 
          */
         Object thisElement = unsortedModel.getElementAt (index);
         SortedListEntry thatEntry = sle;
         
         /* 
          * Retrieve the element that thatEntry points to in the original
          * model. 
          */
         Object thatElement = unsortedModel.getElementAt (thatEntry.getIndex ());
         if (comparator instanceof Collator)
         {
            thisElement = thisElement.toString ();
            thatElement = thatElement.toString ();
         }

         /* 
          * Compare the base model's elements using the provided comparator. 
          */
         int comparison = comparator.compare (thisElement, thatElement);
         
         /* 
          * Convert to descending order as necessary. 
          */
         if (sortOrder == SortOrder.DESCENDING)
         {
            comparison = -comparison;
         }
         return comparison;
      }
      
      /**
       * Our index.
       */
      private int index;
   }
}
