/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */
package cdmw.recording.tools;

import com.thalesgroup.CdmwRecording.GenericRecorder;
import com.thalesgroup.CdmwRecording.NamedValue;
import org.omg.CORBA.*;
import org.omg.TimeBase.TimeTHelper;
import cdmw.commonsvcs.time.Time;

/**
 * Wrapper class 
 */
public class ApplicationRecorder {

      // Normalised context field names
      private static final String m_SenderName_Field    = "Recording.Sender";
      private static final String m_InterfaceName_Field = "Recording.Interface";
      private static final String m_TimeStampName_Field = "Recording.TimeStamp";

      private NamedValue m_Sender_NamedValue = null;
      private NamedValue m_Interface_NamedValue = null;

      private GenericRecorder m_GenericRecorder = null;
      private NamedValue[] m_DefaultContext = null;

      private ORB m_Orb = null;

      /**
       * Class Constructor for use to cache sender and source/interface information
       * Object reference of local generic recorder can be obtained through
       * the local name service by resolving the name used for starting the generic
       * recorder.
       * Ex:
       *    Object obj = sessionContext.resolve_service_reference("LocalNameService");
       *    NamingContextExt nc = NamingContextExtHelper.narrow(ob);
       *    obj = nc.resolve_str("<<MY APP RECORDER>>");
       *    GenericRecorder appRecorder = GenericRecorderHelper.narrow(obj);
       *
       * @param Object reference of the local GenericRecorder object.
       * @param sender Identifier of the sender (element path ... etc.)
       * @param source Identifier of the origin of the records (name of the external 
       * interface ... etc.)
       * @param defaultContext Default context for use when no context is specified.
       * NOTA: Sender/Interface/TimeStamp are still added without checking for duplicates
       * with defaultContext.
       */
      public ApplicationRecorder (GenericRecorder appRecorder,
                                  String sender, 
                                  String source,
                                  NamedValue[] defaultContext)
      {
         this.m_Orb = org.omg.CORBA.ORB.init();
         this.m_GenericRecorder = appRecorder;

         // Sender NamedValue element
         this.m_Sender_NamedValue = new NamedValue();
         this.m_Sender_NamedValue.name = this.m_SenderName_Field;
         this.m_Sender_NamedValue.value = m_Orb.create_any();
         this.m_Sender_NamedValue.value.insert_string(sender);
         // Interface NamedValue element
         this.m_Interface_NamedValue = new NamedValue();
         this.m_Interface_NamedValue.name = this.m_InterfaceName_Field;
         this.m_Interface_NamedValue.value = m_Orb.create_any();
         this.m_Interface_NamedValue.value.insert_string(source);
         
         this.m_DefaultContext = defaultContext;
      }
      /**
       * NOTA: There is no check that context already contains sender, interface 
       * and/or timestamp
       * @param context Context information for the application record
       * @param data Data to be recorded.
       */
      public void Record(NamedValue[] context,
                         NamedValue[] data)
      {
         //System.out.println("ApplicationRecorder::Record(context,data)...");
         int contextLength = 0;
         if (context != null)
            contextLength += context.length;
         NamedValue[] the_context = new NamedValue[3+contextLength];
         // Sender NamedValue element
         the_context[0] = this.m_Sender_NamedValue;
         // Interface NamedValue element
         the_context[1] = this.m_Interface_NamedValue;
         // Timestamp NamedValue element
         the_context[2] = new NamedValue();
         Any timeStamp = m_Orb.create_any();
         TimeTHelper.insert (timeStamp, Time.get_current_time());
         the_context[2].name = this.m_TimeStampName_Field;
         the_context[2].value = timeStamp;
         if (contextLength != 0)
            System.arraycopy(context, 0, the_context, 3, contextLength);

         this.m_GenericRecorder.record_data (the_context,
                                             data);
      }
      /**
       * Record data using shared context.
       *
       * @param data Data to be recorded.
       */
      public void Record(NamedValue[] data)
      {
         //System.out.println("ApplicationRecorder::Record(data)...");
         int contextLength = 0;
         if (this.m_DefaultContext != null)
            contextLength += this.m_DefaultContext.length;

         NamedValue[] the_context = new NamedValue[3+contextLength];
         // Sender NamedValue element
         the_context[0] = this.m_Sender_NamedValue;
         // Interface NamedValue element
         the_context[1] = this.m_Interface_NamedValue;
         // Timestamp NamedValue element
         the_context[2] = new NamedValue();
         Any timeStamp = m_Orb.create_any();
         TimeTHelper.insert (timeStamp, Time.get_current_time());
         the_context[2].name = this.m_TimeStampName_Field;
         the_context[2].value = timeStamp;
         if (contextLength != 0) {
            System.arraycopy(this.m_DefaultContext, 0, the_context, 3, contextLength);
         }

         //System.out.println("ApplicationRecorder::Record : Call to record_data()");
         this.m_GenericRecorder.record_data (the_context, data);
      }
}
