/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

#ifndef CDMW_TRACE_BUFFERSENDER_HPP
#define CDMW_TRACE_BUFFERSENDER_HPP

#include <Foundation/osthreads/MutexGuard.hpp>
#include "tracecollector/LogCircularBuffer.hpp"

namespace Cdmw
{
   namespace Trace
   {

     // A forward declaration
     class LogCircularBuffer;

      /**
       * Purpose:
       * <p> Handle the emission of the buffers to
       * the Local Trace Collector.
       *
       */
      class BufferSender
      {
         public :
            /**
             * Purpose: 
             * <p> Constructor
             *
             * @param circular_buffer_manager a pointer on the
             * manager of the circular buffer. This pointer allows
             * to the Buffer Sender to recover the logs from the circular
             * buffer.
             *
             * @param period_milliseconds the period P in milliseconds.
             * Each P times the logs are sent to the Trace Collector
             * if the collector is reachable.
             *
             * @param max_logs_to_send the maximum number of logs to
             * send at each period. This parameter should avoid to lock 
             * the access to the circular buffer during a too long time.
             *
             * @param chunk_size the maximum size in bytes of a sequence
             * to send to the Local Trace Collector. This parameter
             * should avoid to flood the ORB buffers during a sending 
             * to the Local Trace Collector.
             *
             * @param control Condition object used to wake up
             * BufferSenderThread to send the logs to Local Trace
             * Collector.
             *
             * @param mutex the mutex used to created the Condition object
             * and used by BufferSender to protect itself against
             * multithread accesses
             */        
            BufferSender(LogCircularBuffer* circular_buffer_manager,
                         size_t period_milliseconds,
                         size_t max_logs_to_send,
                         size_t chunk_size,
                         // A pointer in parameter and not a reference
                         // because the copy constructor is private in
                         // the Condition class
                         OsSupport::Condition* control,
                         OsSupport::Mutex& mutex)
                  throw() ;

            /**
             * Purpose: 
             * <p> Destructor
             *
             */
             virtual ~BufferSender()
                  throw() ;


            /**
             * Purpose: 
             * <p> Register the Local Trace Collector reference.
             *
             * @param ref_local_trace_collector the CORBA reference of
             * the Local Trace Collector.
             */
            void register_trace_collector(
                                       CdmwTrace::LocalTraceCollector_ptr
                                       ref_local_trace_collector);

            /**
             * Purpose:
             * <p> Method called by the Local Trace Collector
             * to unregister its reference from the Log Collector
             *
             */
             void unregister_trace_collector()
             throw() ;

            /**
             * enumeration of the thread status
             */ 
             typedef enum LTRC_Status
               {
                 // Buffer Sender Thread can reach the Local Trace
                 // Collector
                 LTRC_REACHABLE,
                 // Buffer Sender Thread cannot reach the Local Trace
                 //  Collector
                 LTRC_UNREACHABLE,
                 // Local Trace Collector is stopped
                 LTRC_STOPPED,
                 // The state of the Local Trace Collector is unknown
                 LTRC_UNKNOWN,
                 // Stop the excetion of the thread
                 STOP_RUNNING
               } LTRC_Status; // New type used to describe the state
                              // of the BufferSenderThread

            /**
             * Purpose: 
             * <p> flush the circular buffer: the logs are 
             * recovered from the circular buffer and are ready to be
             * sent to the Local Trace Collector.
             *
             */
             LTRC_Status BufferSender::flush()
               throw();

          inline LTRC_Status get_LTRC_Status() const {
              CDMW_MUTEX_GUARD(m_mutex) ;
              return m_status;
          }

         private :

            /**
             * Purpose: 
             * <p> Get the logs from the circular buffer.
             *
             */
            void get_logs_from_circular_buffer()
                  throw() ;
        
            /**
             * The status of the BufferSenderThread.
             */
            LTRC_Status m_status;

            /**
             * Smart pointer of the Local Trace Collector.
             */
            CdmwTrace::LocalTraceCollector_var m_local_trace_collector ;

            /**
             * Pointer on the circular buffer.
             * Don't delete this object.
             * It is managed by LogCollector_impl object
             */
            LogCircularBuffer* m_circular_buffer_manager;

            // A forward declaration
            class BufferSenderThread;

            // Allow to BufferSenderThread a free access to all data
            // members of BufferSender
            friend class BufferSenderThread;

            /**
             * A pointer on the BufferSenderThread
             */
            BufferSenderThread * m_thread;   
           
            /**
             * Period in milliseconds to send the logs to Local Trace
             * Collector
             */
            size_t m_transfer_period;

            /**
             * Max number of logs which can be sent to the Local
             * Trace Collector at each sending
             */
            size_t m_max_logs_to_send;

            /**
             * The maximum size in bytes of a sequence to send to the 
             * Local Trace Collector
             */
            size_t m_chunk_size;

            /**
             * The mutex of BufferSender
             */
            OsSupport::Mutex& m_mutex;

             /**
              * A pointer on Condition object  used to wake up BufferSenderThread
              * to send the logs to Local Trace Collector
              */  
            OsSupport::Condition* m_control;

             /**
              * The sequence which will be send to the Local Trace Collector
              */
            CdmwLogging::StructuredLogSeq_var m_log_seq;

             /**
              * Index of m_log_seq member.
              * @see m_log_seq 
              */
            size_t m_seq_index;

             /**
              * Size of m_log_seq
              * This value allows to know how many space is available
              * in the sequence to send.
              * @see m_log_seq
              */
            size_t m_log_seq_size;

      } ;
   }
}
#endif
