/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#ifndef INCL_TRACE_COLLECTORMNGR_HPP 
#define INCL_TRACE_COLLECTORMNGR_HPP 

#include <string>

#include <TraceAndPerf/tracelibrary/Message.hpp>

#include <Foundation/orbsupport/CORBA.hpp>
#include <Foundation/orbsupport/ExceptionMinorCodes.hpp>

#include <Foundation/osthreads/Barrier.hpp>
#include <Foundation/ossupport/OS.hpp>
#include <Foundation/osthreads/Thread.hpp>

#include <Foundation/logging/LogLevel.hpp>
#include <Foundation/logging/FilterMngr.hpp>

#include <TraceAndPerf/idllib/CdmwTraceCollector.stub.hpp>
#include <TraceAndPerf/idllib/CdmwTraceTraceProducer.skel.hpp>

#include <Time/clockservice/CdmwCosClock.hpp>
#include <Foundation/logging/Logger.hpp>


/**
* Root namespace for CDMW runtime.
*/
namespace Cdmw
{
/**
* Namespace for CDMW Trace types and data.
*/
namespace Trace
{

class TraceFileMngr;
class TraceCircularBuffer;
class TraceCollector_impl;

/**
*Purpose:
*<p>    This class output the content of trace messages to disk
*
*Features:
*<p> Thread safe, exception safe
*
*@see  TraceCircularBuffer
*@see  TraceFile
*
*/
class CollectorFlusher : public OsSupport::Thread
{

    public:

        /**
        * Purpose:
        * <p> Constructor
        * 
        *@param barrier the barrier is used by the internal thread
        *       to notify its creator it is ready to operate
        *@param p_circular_buffer CircularBuffer in charge of bufferize the trace messages
        *@param trace_file_name contains generic name for trace file
        *@param trace_file_bkp_nbr contains number of backup of trace files
        *@param max_trace_file_nbr contains max number of trace files
        *@param max_trace_message_nbr contains max number of message in a trace file
        *@param trace_format contains format of trace (Horizontal or Vertical)
        *@param time_mode contains gmt or local mode for display time
        *
        *@exception OutOfMemoryException
        *@exception AssertionFailedException
        *@exception InternalErrorException
        *
        */ 
        CollectorFlusher (OsSupport::Barrier& barrier,
                          TraceCircularBuffer *p_circular_buffer,
                          const std::string& trace_file_base_name, 
                          short trace_file_bkp_nbr,
                          short max_trace_file_nbr, 
                          int max_trace_message_nbr,
                          Message::TraceFormat trace_format,
                          OsSupport::OS::TimeMode time_mode)
                throw (OutOfMemoryException,
                       AssertionFailedException,
                       InternalErrorException,
                       BadParameterException);


        /**
        * Purpose:
        * <p> Destructor
        * 
        */ 
        virtual 
        ~CollectorFlusher()
                throw();




        /**
        * Purpose:
        * <p> Ask to the internal thread to stop its job, the thread
        * will exit only when all trace messages will have been flushed
        * to disk.
        * When this function return the internal thread have stopped.
        *
        *@exception InternalErrorException
        *
        */ 
        void stop_thread()
            throw (InternalErrorException);


        
        /**
        * Purpose:
        * <p> print error message in trace file
        * 
        *@param errMesg contains the message to print
        */ 
        void print_error_message (const std::string& err_mesg)
            throw();


    protected:


        /**
        * Purpose:
        * <p> Call by the underlying thread system in a new thread
        * 
        */ 
        virtual
        void run()
            throw();


    private:

        //
        // Copy ctor
        //
        CollectorFlusher (const CollectorFlusher& rhs)
                throw();




        //
        // Assignement operator
        //
        CollectorFlusher& operator=(const CollectorFlusher& rhs)
                 throw();


        /**
        * At each iteration the thread test this variable, to
        * see if it must stop it's processing. If true
        * the thread terminates.
        */
        bool m_thread_must_terminate;


        
        /**
        * This barrier is used to be synchronised with the CollectorMngr.
        * By signalling this barrier we are able to notify it, when the
        * thread is operational.
        */
        OsSupport::Barrier& m_barrier;           

        /**
        * Hold circular buffer
        */
        TraceCircularBuffer*  m_circular_buffer;

        /**
        * Hold Trace File manager
        */
        TraceFileMngr*  m_trace_file_mngr;

}; // End class CollectorFlusher 





/**
*Purpose:
*<p>    This class is used as a singleton to bufferize message comming from any TraceProducer
*       before to be write into trace file by CollectorFlusher class.
*       The CollectorMngr is in charge of:
*           - initialising the Collector Manager used to bufferize comming trace message
*           - initialising the Collector Flusher, this object is a thread
*             which flushes the trace message to trace file
*           - wrapped the allocation done by the Collector
*           - wrapped the decision to trace or not a Domain/Level
*             (done by the FilterMngr)
*           - initialised the CORBA object in charge of handling client
*             requests (TraceCollector).
*
*Features:
*<p> Thread safe
*
*@see  TraceCollector_impl
*@see  CollectorFlusher
*@see  FilterMngr
*@see  TraceCircularBuffer
*
*/
class CollectorMngr
{

    friend class CollectorFlusher;

    public:

        /**
        * Purpose:
        * <p> Destructor
        * 
        */ 
        virtual 
        ~CollectorMngr()
            throw();


        
        /**
        * Purpose:
        * <p> Returns the CollectorMngr singleton
        * 
        *@return CollectorMngr singleton
        *
        *@exception
        */ 
        static
        CollectorMngr& Instance()
            throw ();



        /**
        * Purpose:
        * <p> Free all allocated memory and stop the trace collector
        * 
        *@exception InternalErrorException
        */ 
        static
        void Cleanup()
            throw (InternalErrorException);


        /**
        * Purpose:
        * <p> Writes the specified message for flushing in trace file
        * 
        *@param messages_hdr contains the main header of trace message from collector servant
        *@param fmt_message contains the formatted trace message from collector servant
        *@return true if successfull
        *
        *@exception OutOfMemoryException        
        */ 
        virtual bool write_message (const CdmwTrace::MessagesHeader& messages_hdr,
                                    const CdmwTrace::FormattedMessage& fmt_message)
                throw (OutOfMemoryException) = 0;



        /**
        * Purpose:
        * <p> returns true if the specified level has been noted
        * to be traced
        * 
        *@param domain the Domain from where the level status must be returned
        *@param level the level to be interrogated
        *
        *@return true if the level is to be traced
        */ 
        static
        bool Is_to_be_traced (const std::string& domain, Logging::LevelType level);


        /**
         * Check if the specified level is activated.
         *
         * @param component_name the component to which the domain is associated
         * @param domain look for the specified level in this domain
         * @param level the level
         *
         * @return true or false
         */
        // ECR-0123
        static bool
        Is_to_be_traced(const std::string& component_name,
                        const std::string& domain,
                        Logging::LevelType level);
                        
        
        /**
         * Get the trace collector reference.
         *
         * @return the collector object
         */                
        CdmwTrace::Collector_ptr get_trace_collector();

        
        /**
         * Get Logger for ccmcommon
         *
         * @return the Logger to be used in ccmcommon
         */
        static Cdmw::Logging::Logger * Get_Logger() throw () 
        {
            static Cdmw::Logging::Logger * Logger__ = 
                Cdmw::Logging::Logger::Get_Logger("CDMW.TRC.TraceCollector");
            CDMW_ASSERT(Logger__ != 0);
            return Logger__;
        }
        
        /**
        * Purpose:
        * <p> Returns a pointer to the operational (federated controlled) clock
        *     used in LocalTraceCollector_impl::put_formatted_messages()
        *@param none
        *@throws a CORBA::NO_IMPLEMENT exception for the base class
        */
        virtual Cdmw::clock::ControlledClockEx_ptr get_operational_clock() const throw (CORBA::NO_IMPLEMENT) {
            throw CORBA::NO_IMPLEMENT(OrbSupport::NO_IMPLEMENT, CORBA::COMPLETED_NO);
        }

    protected:
    
        /**
         * Constructor.
         *
         * @param trace_POA POA used for the trace servant.
         * @param component_name the component to which the domain is associated
         * @param domain the domain where the level must be activated
         * @param level the level to be activated
         * @param traceFormat contains format of trace (Horizontal or Vertical)
         * @param queue_strategy strategy to follow when the circular buffer is
         *                      full (whether to discard new messages or
         *                      to remove the oldest)
         * @param queue_size size of queue in circular buffer in Kbytes
         * @param string_size size of buffer used for string data in KBytes
         * @param trace_file_name generic name for trace file
         * @param trace_file_bkp_nbr contains number of backup of trace files
         * @param max_trace_file_nbr max number of trace files
         * @param max_trace_message_nbr max number of messages in a log file
         * @param time_mode GMT or local mode for the time
         *
         * @exception OutOfMemoryException
         * @exception BadParameterException
         * @exception InternalErrorException
         */
        CollectorMngr(PortableServer::POA_ptr trace_POA,
                      const std::string& component_name,
                      const std::string& domain,
                      Logging::LevelType level,
                      Message::TraceFormat trace_format,
                      Message::CircularMode queue_strategy,
                      size_t queue_size,
                      size_t string_size,
                      const std::string& trace_file_name,
                      short trace_file_bkp_nbr,
                      short max_trace_file_nbr,
                      int max_trace_message_nbr,
                      OsSupport::OS::TimeMode time_mode)
            throw(OutOfMemoryException,
                  BadParameterException,
                  InternalErrorException);
                  
                  
    
        /**
        * This variable is set to true, since the init() function has been called.
        * The cleanup() function reset this variable to false.
        */
        static  
        bool M_init_done;

        /**
        * The reference to the singleton
        */
        static
        CollectorMngr* M_singleton;
        
       /**
        * Contains the reference handling request of the outside world:
        * change of levels, ...
        */
        CdmwTrace::Collector_var m_trace_collector;
                
        /**
        * Hold all Domain/Level which are to be traced or not
        */
        Logging::FilterMngr*  m_filter_mngr;
        
        /**
        * Contains the object in charge of flushing data to the trace file
        */
        std::auto_ptr<CollectorFlusher> m_collector_flusher;
        
        /**
        * Hold circular buffer
        */
        TraceCircularBuffer*  m_circular_buffer;
        
    private:
    
        /**
        * Purpose:
        * <p>  Copy constructor
        * 
        */ 
        CollectorMngr (const CollectorMngr& rhs)
                throw();


        /**
        * Purpose:
        * <p> Assignement operator
        * 
        */ 
        CollectorMngr& operator=(const CollectorMngr& rhs)
                throw();

        /**
        * This barrier is used to be synchronised with the CollectorFlusher
        * thread.
        */
        OsSupport::Barrier m_barrier;

        
}; // End class CollectorMngr 

} // End namespace Trace
} // End namespace Cdmw
#endif // INCL_TRACE_COLLECTORMNGR_HPP

