/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#ifndef INCL_TRACE_GLOBAL_COLLECTORMNGR_HPP 
#define INCL_TRACE_GLOBAL_COLLECTORMNGR_HPP 

#include <string>

#include "tracecollector/CollectorMngr.hpp"
#include "tracecollector/TraceCollector_impl.hpp"


/**
* Root namespace for CDMW runtime.
*/
namespace Cdmw
{
/**
* Namespace for CDMW Trace types and data.
*/
namespace Trace
{


/**
*Purpose:
*<p>    This class is used as a singleton to bufferize message comming from any local Trace Collectors
*       before to be write into trace file by CollectorFlusher class.
*       The GlobalCollectorMngr is in charge of:
*           - initialising the Collector Manager used to bufferize comming trace message
*           - initialising the Collector Flusher, this object is a thread
*             which flushes the trace message to trace file
*           - wrapped the allocation done by the Collector
*           - wrapped the decision to trace or not a Domain/Level
*             (done by the FilterMngr)
*           - initialised the CORBA object in charge of handling client
*             requests (TraceCollector).
*
*Features:
*<p> Thread safe
*
*/
class GlobalCollectorMngr : public CollectorMngr
{

    public:
    
        /**
        * Purpose:
        * <p> Destructor
        * 
        */ 
        virtual 
        ~GlobalCollectorMngr()
            throw();



        /**
         * Initialize the Trace collector.
         * This method must be called prior to any others.
         * The Trace collector must be terminated by calling cleanup().
         *
         * @param trace_POA POA used for the trace servant.
         * @param component_name the component to which the domain is associated
         * @param domain the domain where the level must be activated
         * @param level the level to be activated
         * @param traceFormat contains format of trace (Horizontal or Vertical)
         * @param queue_strategy strategy to follow when the circular buffer is
         *                      full (whether to discard new messages or
         *                      to remove the oldest)
         * @param queue_size size of queue in circular buffer in Kbytes
         * @param string_size size of buffer used for string data in KBytes
         * @param trace_file_name generic name for trace file
         * @param trace_file_bkp_nbr contains number of backup of trace files
         * @param max_trace_file_nbr max number of trace files
         * @param max_trace_message_nbr max number of messages in a log file
         * @param time_mode GMT or local mode for the time
         * @param miop_POA POA use for activate servant for MIOP (can't be trace_POA : WrongPolicy)
         * @param miopGlobalCollOid miop id of the global collector
         *
         * @return the pointer to the Collector servant object
         *
         * @exception OutOfMemoryException
         * @exception BadParameterException
         * @exception InternalErrorException
         */
        static CdmwTrace::Collector_ptr
        Init(PortableServer::POA_ptr trace_POA,
             const std::string& component_name,
             const std::string& domain,
             Logging::LevelType level,
             Message::TraceFormat trace_format,
             Message::CircularMode queue_strategy,
             size_t queue_size,
             size_t string_size,
             const std::string& trace_file_name,
             short trace_file_bkp_nbr,
             short max_trace_file_nbr,
             int max_trace_message_nbr,
             OsSupport::OS::TimeMode time_mode,
             PortableServer::POA_ptr miop_POA,
             PortableServer::ObjectId miopGlobalCollOid)
            throw(OutOfMemoryException,
                  BadParameterException,
                  InternalErrorException);
                  
                  
        /**
        * Purpose:
        * <p> Writes the specified message for flushing in trace file
        * 
        *@param messages_hdr contains the main header of trace message from collector servant
        *@param fmt_message contains the formatted trace message from collector servant
        *@return true if successfull
        *
        *@exception OutOfMemoryException        
        */ 
        virtual bool write_message (const CdmwTrace::MessagesHeader& messages_hdr,
                                    const CdmwTrace::FormattedMessage& fmt_message)
                throw (OutOfMemoryException);    


    protected:
    

    private:
    
    

        /**
         * Constructor.
         *
         * @param trace_POA POA used for the trace servant.
         * @param component_name the component to which the domain is associated
         * @param domain the domain where the level must be activated
         * @param level the level to be activated
         * @param traceFormat contains format of trace (Horizontal or Vertical)
         * @param queue_strategy strategy to follow when the circular buffer is
         *                      full (whether to discard new messages or
         *                      to remove the oldest)
         * @param queue_size size of queue in circular buffer in Kbytes
         * @param string_size size of buffer used for string data in KBytes
         * @param trace_file_name generic name for trace file
         * @param trace_file_bkp_nbr contains number of backup of trace files
         * @param max_trace_file_nbr max number of trace files
         * @param max_trace_message_nbr max number of messages in a log file
         * @param time_mode GMT or local mode for the time
         *
         * @exception OutOfMemoryException
         * @exception BadParameterException
         * @exception InternalErrorException
         */
        GlobalCollectorMngr (PortableServer::POA_ptr trace_POA,
                             const std::string& component_name,
                             const std::string& domain,
                             Logging::LevelType level,
                             Message::TraceFormat trace_format,
                             Message::CircularMode queue_strategy,
                             size_t queue_size,
                             size_t string_size,
                             const std::string& trace_file_name,
                             short trace_file_bkp_nbr,
                             short max_trace_file_nbr,
                             int max_trace_message_nbr,
                             OsSupport::OS::TimeMode time_mode)
            throw(OutOfMemoryException,
                  BadParameterException,
                  InternalErrorException);


        /**
        * Purpose:
        * <p>  Copy constructor
        * 
        */ 
        GlobalCollectorMngr (const GlobalCollectorMngr& rhs)
                throw();


        /**
        * Purpose:
        * <p> Assignement operator
        * 
        */ 
        GlobalCollectorMngr& operator=(const GlobalCollectorMngr& rhs)
                throw();
             
        // to keep a reference of miop collector id
        static PortableServer::ObjectId M_miopGlobalCollOid;   

        // POA used for miop (can't be trace POA : WrongPolicy) At this time it is root POA
        static PortableServer::POA_var M_miopPOA;

}; // End class GlobalCollectorMngr 

} // End namespace Trace
} // End namespace Cdmw
#endif // INCL_TRACE_GLOBAL_COLLECTORMNGR_HPP

