/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#ifndef CDMW_TRACE_COLLECTOR_INITUTILS_LOG_COLLECTOR_HPP
#define CDMW_TRACE_COLLECTOR_INITUTILS_LOG_COLLECTOR_HPP

#include "tracecollector/LogCollector_impl.hpp"
#include "tracecollector/InitUtils.hpp"
#include <string>

namespace Cdmw
{
namespace Trace
{

  /**
   * Purpose:
   *<p> A static class allowing to create/destroy the Log Collector
   */
  class InitUtilsLogCollector: virtual public InitUtils
  {
  public:

    /**
     * Purpose:
     * <p> Create, initialize the Log Collector and register it as a Root
     * Context inside of the CDMW daemon. After this operation the Log
     * Collector can be used: add_log() can be called on the Log
     * Collector.
     *
     * @param orb the ORB.
     * @param poa the POA.
     * @param daemon the platform daemon
     *
     * @param system_port the port of the CDMW daemon.
     *
     * @param period the period P. The logs will be sent to the Trace
     * Collector each P milliseconds. 
     *
     * @param circular_buffer_size the size of the circular buffer.
     *
     * @param max_logs_to_sent the maximum number of logs to send at
     * each period. This parameter should avoid to lock the access to
     * the circular buffer during a too long time.
     * 
     * @param chunk_size the maximum size in bytes of a sequence to send
     * to the Local Trace Collector. This parameter should avoid to
     * flood the ORB buffers during a sending to the Local Trace
     * Collector.
     * 
     * @exception OutOfMemoryException Not enough memory on the localhost.
     * @exception BadParameterException Bad parameters were set in
     * argument of this method.
     * @exception CosNaming::NamingContext::CannotProceed Binding on
     * the NameService context of the CDMW daemon was given up.
     *
     * @see Unregister_log_collector
     */
    static void Create_log_collector (CORBA::ORB_ptr orb,
                                      PortableServer::POA_ptr poa,
                                      const std::string& system_port,
                                      size_t period,
                                      size_t circular_buffer_size, 
                                      size_t max_logs_to_send,
                                      size_t chunk_size
                                      )
      throw (OutOfMemoryException,
             BadParameterException,
             CosNaming::NamingContext::CannotProceed);

    /**
     * Purpose:
     * <p> Unregister the Log Collector from the CDMW daemon.
     * This method must be called only when the Log Collector becomes
     * useless.
     * For instance if the CDMW daemon is stopped and 
     * Create_log_collector() was called before.
     *
     * @param orb The ORB.
     *
     * @exception SystemException The Log Collector wasn't unbind from
     * the CDMW daemon
     *
     * @see Create_log_collector
     */
    static void Unregister_log_collector (CORBA::ORB_ptr orb)
      throw (CORBA::SystemException);

  private:

    /**
     * Purpose:
     * <p> Constructor
     */
    InitUtilsLogCollector()
    throw():
    InitUtils(){}

    /**
     * Purpose:
     * <p> Destructor
     */
    virtual ~InitUtilsLogCollector()
    throw() {}

  };
}
}

#endif // CDMW_TRACE_COLLECTOR_INITUTILS_LOG_COLLECTOR_HPP
