/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#ifndef INCL_TRACE_LOCAL_COLLECTORMNGR_HPP 
#define INCL_TRACE_LOCAL_COLLECTORMNGR_HPP 

#include <string>

#include "tracecollector/CollectorMngr.hpp"
#include "tracecollector/LocalTraceCollector_impl.hpp"
#include "tracecollector/CollectorInit.hpp"


/**
* Root namespace for CDMW runtime.
*/
namespace Cdmw
{
/**
* Namespace for CDMW Trace types and data.
*/
namespace Trace
{


/**
*Purpose:
*<p>    This class is used as a singleton to bufferize message comming from any Trace Producers
*       before to be write into trace file by CollectorFlusher class.
*       The LocalCollectorMngr is in charge of:
*           - initialising the Collector Manager used to bufferize comming trace message
*           - initialising the Collector Flusher, this object is a thread
*             which flushes the trace message to trace file
*           - wrapped the allocation done by the Collector
*           - wrapped the decision to trace or not a Domain/Level
*             (done by the FilterMngr)
*           - initialised the CORBA object in charge of handling client
*             requests (TraceCollector).
*
*Features:
*<p> Thread safe
*
*/
class LocalCollectorMngr : public CollectorMngr
{

    public:
    
        /**
        * Purpose:
        * <p> Destructor
        * 
        */ 
        virtual 
        ~LocalCollectorMngr()
            throw();



        /**
         * Initialize the Trace collector.
         * This method must be called prior to any others.
         * The Trace collector must be terminated by calling cleanup().
         *
         * @param orb (used for serialisation)
         * @param trace_POA POA used for the trace servant.
         * @param global_collectors the list of global collectors
         * @param options options of trace collector initialisation
         *   the options used at the level or below are :  
         *    - component_name the component to which the domain is associated
         *    - domain_name the domain where the level must be activated
         *    - level_value the level to be activated
         *    - trace_format contains format of trace (Horizontal or Vertical)
         *    - queue_strategy strategy to follow when the circular buffer is
         *      full (whether to discard new messages or to remove the oldest)
         *    - queue_size size of queue in circular buffer in Kbytes
         *    - queue_str_size size of buffer used for string data in KBytes
         *    - trace_file_name generic name for trace file
         *    - trace_file_bkp_num contains number of backup of trace files
         *    - trace_file_num max number of trace files
         *    - trace_msg_num max number of messages in a log file
         *    - size_flush_area flush area size of the trace collector buffer 
         *      between local and global collector
         * @param time_mode GMT or local mode for the time
         * @param miop_collectors reference to miop collectors
         *
         * @return the pointer to the Collector servant object
         *
         * @exception OutOfMemoryException
         * @exception BadParameterException
         * @exception InternalErrorException
         */
        static CdmwTrace::Collector_ptr
        Init(CORBA::ORB_ptr orb,
         PortableServer::POA_ptr trace_POA,
             LocalTraceCollector_impl::CollectorDataList& global_collectors,
             CollectorOptions::Options& options,
             OsSupport::OS::TimeMode time_mode,
             CdmwTrace::Collector_ptr miop_collectors)

            throw(OutOfMemoryException,
                  BadParameterException,
                  InternalErrorException);
                  
                  
        /**
        * Purpose:
        * <p> Writes the specified message for flushing in trace file
        * 
        *@param messages_hdr contains the main header of trace message from collector servant
        *@param fmt_message contains the formatted trace message from collector servant
        *@return true if successfull
        *
        *@exception OutOfMemoryException        
        */ 
        virtual bool write_message (const CdmwTrace::MessagesHeader& messages_hdr,
                                    const CdmwTrace::FormattedMessage& fmt_message)
                throw (OutOfMemoryException);    

        virtual Cdmw::clock::ControlledClockEx_ptr get_operational_clock() const throw (CORBA::NO_IMPLEMENT) {
            return Cdmw::clock::ControlledClockEx::_duplicate(m_operational_clock.in());
        }

    protected:
    

    private:
    
    

        /**
         * Constructor.
         *
         * @param orb (used for serialisation)
         * @param trace_POA POA used for the trace servant.
         * @param global_collectors the list of global collectors
         * @param options options of trace collector initialisation
         *   the options used at the level or below are :  
         *    - domain_name the domain where the level must be activated
         *    - level_value the level to be activated
         *    - trace_format contains format of trace (Horizontal or Vertical)
         *    - queue_strategy strategy to follow when the circular buffer is
         *      full (whether to discard new messages or to remove the oldest)
         *    - queue_size size of queue in circular buffer in Kbytes
         *    - queue_str_size size of buffer used for string data in KBytes
         *    - trace_file_name generic name for trace file
         *    - trace_file_bkp_num contains number of backup of trace files
         *    - trace_file_num max number of trace files
         *    - trace_msg_num max number of messages in a log file
         *    - size_flush_area flush area size of the trace collector buffer 
         *      between local and global collector
         * @param time_mode GMT or local mode for the time
         * @param miop_collectors reference to miop collectors
         *
         * @exception OutOfMemoryException
         * @exception BadParameterException
         * @exception InternalErrorException
         */
        LocalCollectorMngr (CORBA::ORB_ptr orb,
                            PortableServer::POA_ptr trace_POA,
                            LocalTraceCollector_impl::CollectorDataList& global_collectors,
                            CollectorOptions::Options& options,
                            OsSupport::OS::TimeMode time_mode,
                CdmwTrace::Collector_ptr miop_collectors)
            throw(OutOfMemoryException,
                  BadParameterException,
                  InternalErrorException);


        /**
        * Purpose:
        * <p>  Copy constructor
        * 
        */ 
        LocalCollectorMngr (const LocalCollectorMngr& rhs)
                throw();


        /**
        * Purpose:
        * <p> Assignement operator
        * 
        */ 
        LocalCollectorMngr& operator=(const LocalCollectorMngr& rhs)
                throw();


        /**
        * Represents the Federated Controlled Clock
        */
        Cdmw::clock::ControlledClockEx_var m_operational_clock;

}; // End class LocalCollectorMngr 

} // End namespace Trace
} // End namespace Cdmw
#endif // INCL_TRACE_LOCAL_COLLECTORMNGR_HPP

