/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#ifndef INCL_TRACE_LOGCIRCULARBUFFER_HPP 
#define INCL_TRACE_LOGCIRCULARBUFFER_HPP 

#include <memory>

#include <Foundation/common/Exception.hpp>

#include <Foundation/ossupport/OS.hpp>
#include <Foundation/osthreads/Condition.hpp>

#include <Foundation/orbsupport/CORBA.hpp>
#include <Foundation/logging/CdmwLoggingCollector.stub.hpp>
#include "tracecollector/CdmwTraceLoggingCollector.stub.hpp"
#include "tracecollector/BufferSender.hpp"
#include "tracecollector/CircularBuffer.hpp"

/**
* Root namespace for CDMW runtime.
*/
namespace Cdmw
{
/**
* Namespace for CDMW Trace types and data.
*/
namespace Trace
{

  class BufferSender;

/**
* Purpose:
* <p>    This class is used to store the logs in a circular buffer.
*       It is a specialized class of CircularBuffer class
*
* Features:
* <p> Thread safe
*
*/

class LogCircularBuffer : public CircularBuffer
{
    
public:

    /**
    * Purpose:
    * <p> Constructor
    * 
    * @param bufferCircularSize contains size of the circular buffer
    * in Bytes
    *
    * @param period the period P. The logs are sent to the Trace
    * Collector each P milliseconds.
    *
    * @param maxLogsToSent the maximum number of logs to send at each
    * period. This parameter should avoid to lock the access to the
    * circular buffer during a too long time.
    *
    * @param chunkSize the maximum size in bytes of a sequence to send
    * to the  Local Trace Collector. This parameter should avoid to
    * flood the ORB buffers during a sending to the Local Trace Collector.
    *
    * @param bufferSender the Buffer Sender object created by this
    * constructor.
    *
    * @exception BadParameterException
    * @exception OutOfMemoryException
    */ 
    LogCircularBuffer (size_t bufferCircularSize, size_t period,
                       size_t maxLogsToSent,
                       size_t chunkSize,
                       Cdmw::Trace::BufferSender*& bufferSender)
            throw(Cdmw::BadParameterException);

    /**
    * Purpose:
    * <p> Destructor
    * 
    * @param
    * @exception
    */ 
    virtual
    ~LogCircularBuffer ()
            throw ();

    /**
    * Purpose:
    * <p> Returns the next log from circular buffer,
    *     if the circular buffer is full, the buffer is flushed
    *     by the BufferSender which sends the logs to a Trace
    *     Collector if it is reachable or in running state.
    *
    * @param log the IDL structure stored in circular buffer
    * @param log_size the size of the log returned in output
    * parameter (log parameter)
    *
    * @param allowed_log_size Size which can be read by the
    * BufferSender. This size depends on the space available
    * in the sequence which must be sent by the BufferSender.
    *
    * @return true if no error
    *
    * @exception OutOfMemoryException
    *
    */ 
    bool getNextLog(CdmwLogging::StructuredLog& log,
                    size_t& log_size,
                    size_t allowed_log_size)
            throw(Cdmw::OutOfMemoryException);

    /**
    * Purpose:
    * <p> request to write a log in the circular buffer
    * 
    *@param log the log to store
    *@return true if successfull
    *
    */ 
    bool writeLog (const CdmwLogging::StructuredLog& log)
      throw();

protected:

    

private:



    /**
    * This structure is used to store a log in the circular buffer 
    */
    struct BufferLogMessage
    {
      BufferLogMessage()
        : m_messageSize(0),
          m_bodyMessageSize(0),
          m_szHostName(0),
          m_szApplicationName(0),
          m_szProcessName(0),
          m_pid(0),
          m_szNdc(0),
          m_szComponentName(0),
          m_szUserDomain(0),
          m_level(0),
          m_tid(0),
          m_fileLine(0),
          m_szFileName(0)
      {
      }

      /**
      * The total size of message 
      * (body, application name, process name, host name, file name,
      * user domain, etc)
      */
      size_t m_messageSize;
      
      /**
      * The size of body message
      */
      size_t m_bodyMessageSize;

      /**
      * Size of host name where the trace has been produced
      */
      size_t m_szHostName;

      /**
      * The size of application name which has produced the trace
      */
      size_t m_szApplicationName;
      
      /**
      * The size of process name which has produced the trace
      */
      size_t m_szProcessName;
                                                            
      /**
      * The process which has produced the trace.
      */
      OsSupport::OS::ProcessId m_pid;
                                                            
      /**
       * Size of Nested Diagnostic Control, used to set up a log tracing.
       */
      size_t m_szNdc;      
      
      /**
       * The component name.
       */
      // ECR-0123
      size_t m_szComponentName;

      /**
      * Owner domain
      */
      size_t m_szUserDomain;

      /**
      * Level value.
      */
      ::CORBA::ULong m_level;

      /**
      * The thread which has created the message.
      */
      OsSupport::Thread::ThreadId m_tid;
        
      /**
      * The line where the message has been produced.
      */
      int m_fileLine;
            
      /**
      * The name size of file, origin of trace
      */
      size_t m_szFileName;

      /**
      * The time when the trace has been produced.
      */
      OsSupport::OS::Timeval m_timestamping;
    };
    
    
    /**
    * Purpose:
    * <p>  Copy constructor
    * 
    */ 
    LogCircularBuffer (const LogCircularBuffer& rhs)
            throw();

    /**
    * Purpose:
    * <p>  Assignement operator
    * 
    */ 
    LogCircularBuffer& operator=(const LogCircularBuffer& rhs)
            throw();

    /**
    * Purpose:
    * <p> Reallocation a buffer cache if it is too small
    * 
    * @param buffer_cache the buffer cache which is resized.
    * @param stringSz contains new size of memory for string buffer
    *
    */ 
    bool realloc_buffer_cache (char*& buffer_cache,
                               size_t stringSz)
            throw();

    /**
    * Purpose:
    * <p> Request to read a log.
    * 
    * @param log the log returned and filled after reading
    * in the circular buffer.
    *
    * @param sizeLog the size of the log which is returned
    * @param allowed_log_size the size which can be read by the
    * BufferSender. This size depends on the space available
    * in the sequence which must be sent by the BufferSender.
    *
    * @return true if no error
    *
    * @exception OutOfMemoryException
    *
    */ 
    bool readLog (CdmwLogging::StructuredLog& log, size_t& sizeLog,
                  size_t allowed_log_size)
            throw(Cdmw::OutOfMemoryException);


    /**
    * Purpose:
    * <p> request to remove the older log from the circular buffer
    *
    *@param size_new_log Size of the new log which must be stored
    *@return true if no error
    */ 
    bool removeOlderLog (size_t size_new_log)
            throw();

    /**
    * Buffer Sender pointer
    */
    BufferSender* m_buffer_sender;

    /**
     * The maximum size of a log which is equals to
     * the littlest value between bufferCircularSize
     * and chunkSize parameters from the constructor.
     * @see LogCircularBuffer
     */
    size_t m_max_log_size;

    /**
     * Condition object used to wake up BufferSender
     * to send the logs to Local Trace Collector
     */
    OsSupport::Condition m_control;            
  
}; // End class LogCircularBuffer 

}; // End namespace Trace
}; // End namespace Cdmw
#endif // INCL_LOGCIRCULARBUFFER_HPP

