/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#ifndef CDMW_TRACE_LOG_COLLECTOR_HPP
#define CDMW_TRACE_LOG_COLLECTOR_HPP

#include <SystemMngt/idllib/CdmwPlatformMngtDaemon.stub.hpp>

#include "tracecollector/CdmwTraceLoggingCollector.skel.hpp"
#include "tracecollector/BufferSender.hpp"
#include <Foundation/orbsupport/RefCountServantBase.hpp>

namespace Cdmw
{
namespace Trace
{
class LogCollector_impl: virtual public POA_CdmwTrace::LogCollector,
                           virtual public Cdmw::OrbSupport::RefCountServantBase
{

public:

    /**
     * Purpose:
     * <p> Add a new log in the Log Collector.
     * @return false is returned when the log treatment cannot be correctly processed.
     */
    ::CORBA::Boolean add_log (const ::CdmwLogging::StructuredLog & log)
      throw (::CORBA::SystemException);
    
    /**
     *Purpose:
     *<p> Register the Trace Collector reference in the Log Collector.
     *
     *@param ref_trace_collector the CORBA reference of the Trace
     *Collector.
     */
    virtual void register_trace_collector(CdmwTrace::LocalTraceCollector_ptr ref_trace_collector)
      throw(CORBA::SystemException) ;

    /**
     *Purpose:
     *<p> Method called by the Trace Collector to unregister the
     * reference of Trace collector from the Log Collector
     *
     */
    virtual void unregister_trace_collector()
      throw(CORBA::SystemException) ;
    

    /**
     * Purpose:
     * <p> Constructor
     */
    LogCollector_impl()
      throw();

    /**
     * Purpose:
     * <p> Destructor
     */
    virtual ~LogCollector_impl()
      throw();

    /**
     * Purpose:
     * <p> Initialize the Log Collector library
     * This function must be called prior to any other calls to the library.
     *
     * @param period the period P. The logs will be sent to the Trace Collector each P milliseconds.
     *
     * @param circular_buffer_size the size of the circular buffer.
     *
     * @param max_logs_to_send the maximum number of logs to
     * send at each period. This parameter should avoid to lock 
     * the access to the circular buffer during a too long time.
     *
     * @param chunk_size the maximum size in bytes of a sequence
     * to send to the Local Trace Collector. This parameter
     * should avoid to flood the ORB buffers during a sending 
     * to the Local Trace Collector.
     *
     * @param poa the POA
     *
     * @param daemon the platform daemon
     *
     * @exception OutOfMemoryException Not enough memory.
     * @exception BadParameterException Bad parameters were set in argument of this method.
     */
     void init(size_t period, size_t circular_buffer_size,
               size_t max_logs_to_send,
               size_t chunk_size,
               PortableServer::POA_ptr poa,
               CdmwPlatformMngt::Daemon_ptr daemon)
      throw(OutOfMemoryException,
            BadParameterException);

    /**
     * Purpose:
     * <p>
     * Redefines the PortableServer::ServantBase method
     * to return the correct POA.
     *
     * @return The m_POA member
     */
    virtual PortableServer::POA_ptr _default_POA();
    
  private:

    /**
     * Ponter on the BufferSender
     * This object must not be deleted.
     * It is managed by the LogCircularBuffer object
     */
    BufferSender* m_buffer_sender;

    /**
    * LogCircularBuffer pointer
    */
    LogCircularBuffer* m_log_circular_buffer;

    /**
    * The POA
    */
    PortableServer::POA_var m_poa;

    /**
     * The platform daemon
     */
    CdmwPlatformMngt::Daemon_var m_daemon;   
    
    /**
     * MASK for forwarding log to supervision
     */
    unsigned long m_forwarding_mask;
};
}
}

#endif // CDMW_TRACE_LOG_COLLECTOR_HPP
