#ifndef INCL_TRACE_TRACEPRINTER_HPP
#define INCL_TRACE_TRACEPRINTER_HPP

#include <string>
#include <iostream>
#include <Foundation/ossupport/OS.hpp>

#include "tracecollector/TraceCircularBuffer.hpp"

namespace Cdmw
{
/**
 * Namespace for CDMW Trace types and data.
 */
   namespace Trace
   {
      /**
      *Purpose:
      *<p>    This is the base class to manage the printing of the messages 
      *       issued from the Trace Producers.  The functions are pure virtual,
      *       meaning that the class only represents a formatter interface.
      *       The three following implementations are currently available :
      *             - HorizontalTracePrinter
      *             - VerticalTracePrinter
      *             - XMLTracePrinter
      */
      class TracePrinter
      {
         protected:
            /**
            * Purpose:
            * <p> Default public Constructor
            * 
            *@param time_mode Can be either Cdmw::OsSupport::OS::LOCAL_TIME or
            *                 Cdmw::OsSupport::OS::GMT_TIME
            *@exception
            */           
            TracePrinter(OsSupport::OS::TimeMode time_mode) throw()
               : m_time_mode(time_mode) {}

            // utility function used by inherited classes
            std::string timestamp_to_string(OsSupport::OS::Timeval tv, const char* format) const;
         
         public:
            // The following constants are used by the inherited classes to format 
            // their printing homogeneously
            static const char* END_OF_TRACE_FILE;
            static const char* TRACE_HEADER1;
            static const char* TRACE_HEADER2;
            static const char* TRACE_HEADER3;
            static const char* TRACE_ERR_MESG1;
            static const char* TRACE_ERR_MESG2;
          
            /**
            * Purpose:
            * <p> Destructor
            * 
            *@param
            *@exception
            */
            virtual ~TracePrinter() throw() {}

            /**
            * Purpose:
            * <p> prints the body of the message
            * 
            *@param os std::ostream to write to
            *@param Data Data containing the information to read from,
            *            like the message's header, the timestamp, etc.
            *@exception
            */
            virtual void print_message(std::ostream & os, const TraceCircularBuffer::TraceMessageRaw & Data) const
               throw() = 0;
            /**
            * Purpose:
            * <p> Prints an error message
            * 
            *@param os std::ostream to write to
            *@param errMesg Error message to write
            *@exception
            */
            virtual void print_error_message (std::ostream & os, const std::string& errMesg) const
               throw() = 0;
            /**
            * Purpose:
            * <p> Prints the header of the message in the circular file referenced by file_number
            * 
            *@param os std::ostream to write to
            *@param file_number Index of the circular file to write to
            *@exception
            */            
            virtual void print_header (std::ostream & os, int file_number) const
               throw() = 0;
            /**
            * Purpose:
            * <p> Prints the end of the trace
            * 
            *@param os std::ostream to write to
            *@exception
            */
            virtual void print_end_of_trace (std::ostream & os) const
               throw() = 0;
         private:
            /**
            * Purpose:
            * <p> Constructor
            * 
            *@param
            *@exception
            */           
            TracePrinter() throw() {}
            /**
             * Hold time mode display gmt or local
             */
            OsSupport::OS::TimeMode m_time_mode;
      };
   
   }
}
#endif /*INCL_TRACE_TRACEPRINTER_HPP*/
