/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

#include <Foundation/logging/LogManager.hpp>
#include <Foundation/logging/Logger.hpp>
#include <Foundation/logging/FunctionLogger.hpp>
#include <Foundation/orbsupport/OrbSupport.hpp>
#include <Foundation/ossupport/OS.hpp>
#include <Foundation/osthreads/Thread.hpp>
#include <Foundation/orbsupport/CORBAMessaging.hpp>
#include <Foundation/logging/CdmwLoggingCollector.stub.hpp>
#include <Foundation/osthreads/Condition.hpp>
#include "tracecollector/BufferSender.hpp"
#include "tracecollector/LogCircularBuffer.hpp"
#include "tracecollector/LogCollectorMacroDefs.hpp"

namespace Cdmw
{
    namespace Trace
    {
        /**
         * Purpose:
         * <p> This class implements the thread of BufferSender
         */
        class BufferSender::BufferSenderThread : public OsSupport::Thread
        {
        public :
            /**
             * Purpose:
             * <p> Constructor
             *
             * @param buffer_sender the pointer on BufferSender
             */
            BufferSenderThread(BufferSender* buffer_sender) throw()
                : OsSupport::Thread(),
                  m_buffer_sender(buffer_sender),
                  m_stop_called(false)
            {
            }

            /**
             * Purpose:
             * <p> Destructor
             */
            virtual ~BufferSenderThread() throw()
            {
            }

            /**
             * Purpose: 
             * <p>  Send the logs to the Local Trace Collector.
             *
             */
            void send_logs_to_trace_collector()
                throw()
            {
                // CDMW_DEBUG_LOG_COLLECTOR("");
                try {
                    // Send all the logs to the Local Trace Collector
                    if(m_buffer_sender->m_log_seq->length() != 0
                       && !CORBA::is_nil(m_buffer_sender->m_local_trace_collector.in())) {
                        m_buffer_sender->m_local_trace_collector->add_log_sequence(m_buffer_sender->m_log_seq.in());
                        // Reset the index of the sequence
                        m_buffer_sender->m_seq_index = 0;
                        // Reset the length of the sequence
                        m_buffer_sender->m_log_seq->length(0);
                        m_buffer_sender->m_log_seq_size = 0;
                        CDMW_DEBUG_LOG_COLLECTOR("Sending OK");
                    }

                    // if an exception is raised, the logs of the sequence are kept
                    // as long as the number of logs in the sequence doesn't reach
                    // the max size of the sequence (m_max_logs_to_send)
                } catch (CORBA::TRANSIENT) {
                    CDMW_MUTEX_GUARD(m_buffer_sender->m_mutex) ;
                    m_buffer_sender->m_status = LTRC_UNREACHABLE;
                
                } catch (CORBA::COMM_FAILURE) {
                    CDMW_MUTEX_GUARD(m_buffer_sender->m_mutex) ;
                    m_buffer_sender->m_status = LTRC_UNREACHABLE;

                } catch (...) {
                    CDMW_MUTEX_GUARD(m_buffer_sender->m_mutex) ;
                    CDMW_WARN_LOG_COLLECTOR("Unexpected exception: The Log Collector tried to send "
                                            << "a sequence of logs but an exception was raised. "
                                            << "This exception is neither CORBA::COMM_FAILURE nor CORBA::TRANSIENT.");
                    m_buffer_sender->m_status = LTRC_UNREACHABLE;
                }
            }

            /**
             * Purpose:
             * <p> Stop the thread
             */
            void stop () {
                CDMW_MUTEX_GUARD(m_buffer_sender->m_mutex) ;
                m_stop_called = true;
                m_buffer_sender->m_control->broadcast();
            }

            /**
             * Purpose:
             * <p> The main processing of the thread.
             */
            virtual void run() throw()
            {
                BufferSender::LTRC_Status ltrc_status;
              
                while (!m_stop_called) {

                    {
                        CDMW_MUTEX_GUARD(m_buffer_sender->m_mutex) ;
                        m_buffer_sender->m_control->wait(m_buffer_sender->m_transfer_period);
                        if (m_stop_called) break;
                        ltrc_status = m_buffer_sender->m_status;
                    }

                    switch ( ltrc_status ) {
                    case LTRC_REACHABLE:
                        // Get the logs from LogCircularBuffer and send them
                        m_buffer_sender->get_logs_from_circular_buffer();
                        send_logs_to_trace_collector();
                        break;
                    case LTRC_STOPPED:
                    case LTRC_UNKNOWN:
                    case LTRC_UNREACHABLE:
                        try {
                            // Check that the trace collector is alive
                            if (! CORBA::is_nil(m_buffer_sender->m_local_trace_collector.in())) {
                                if (m_buffer_sender->m_local_trace_collector->_non_existent() == false) {
                                    // The Trace Collector is now reachable
                                    CDMW_MUTEX_GUARD(m_buffer_sender->m_mutex) ;
                                    m_buffer_sender->m_status = LTRC_REACHABLE;
                                } 
                            }
                            // It appeared that TAO can raise an exception
                            // rather return true to _non_existent call.
                            // This explains this catch
                        }catch (::CORBA::TRANSIENT) {
                            CDMW_DEBUG_LOG_COLLECTOR("Local Trace Collector unreachable");
                        }
                        break;
                    default:
                        break;
                    }
                
                }// while
            }// run

        private :
            BufferSender* m_buffer_sender;
            bool m_stop_called;
        } ;

        //
        // Constructor
        //
        BufferSender::BufferSender(LogCircularBuffer*
                                   circular_buffer_manager,
                                   size_t period_milliseconds,
                                   size_t max_logs_to_send,
                                   size_t chunk_size,
                                   OsSupport::Condition* control,
                                   OsSupport::Mutex& mutex
                                   ) throw()
            : m_status (LTRC_UNKNOWN),
              m_circular_buffer_manager(circular_buffer_manager),
              m_transfer_period(period_milliseconds),
              m_max_logs_to_send(max_logs_to_send),
              m_chunk_size (chunk_size),
              m_mutex(mutex),
              m_control(control),
              m_seq_index(0),
              m_log_seq_size(0)
        {
            m_log_seq = new CdmwLogging::StructuredLogSeq(m_max_logs_to_send);
            m_log_seq->length(0);
            m_thread = new BufferSenderThread (this);
            m_thread->start();
        }

        //
        // Destructor
        //
        BufferSender::~BufferSender()
            throw()
        {
            if (m_thread != 0)
                {
                    m_thread->stop();
                    m_thread ->join() ;
                    delete m_thread ;
                    m_thread = 0x0;
                }
        }

        //
        // Get the logs from the circular buffer.
        //
        void BufferSender::get_logs_from_circular_buffer()
            throw()
        {
            CDMW_DEBUG_LOG_COLLECTOR("");
            // Know if the sequence to send can be filled, ie the chunk size
            // is not reached and the circular buffer is not empty
            bool go_on_flushing = true;
         
            // Size in bytes of the log which is get from the circular buffer
            size_t log_size = 0;

            CDMW_DEBUG_LOG_COLLECTOR("m_seq_index=" << m_seq_index
                                     << " m_log_seq_size=" << m_log_seq_size
                                     << " m_max_logs_to_send=" << m_max_logs_to_send
                                     << " m_chunk_size=" << m_chunk_size);

            // Get the logs
            while( go_on_flushing 
                   && m_seq_index < m_max_logs_to_send
                   && m_log_seq_size <= m_chunk_size) {

                // Get a log
                try {
                    // Size of the log which can be taken
                    size_t allowed_log_size = m_chunk_size - m_log_seq_size;

                    if (allowed_log_size > 0) {
                        go_on_flushing =
                            m_circular_buffer_manager->getNextLog(
                                                                  // (m_log_seq.inout())[m_seq_index],
                                                                  m_log_seq[m_seq_index],
                                                                  log_size,
                                                                  allowed_log_size);
                    }

                } catch (Cdmw::OutOfMemoryException&) {
                    CDMW_ERROR_LOG_COLLECTOR("OutOfMemoryException raised during getting logs from the circular buffer");
                    return;
                }

                if (go_on_flushing) {
                    m_log_seq_size += log_size;
                    m_seq_index++;
                    m_log_seq->length(m_seq_index);
                }
            }
        }      

        //
        // Set the Trace Collector reference in the BufferSender
        //
        void BufferSender::register_trace_collector(CdmwTrace::LocalTraceCollector_ptr ref_local_trace_collector)
        {
            CDMW_DEBUG_LOG_COLLECTOR("");
            CDMW_MUTEX_GUARD(m_mutex) ;
            m_local_trace_collector = 
                CdmwTrace::LocalTraceCollector::_duplicate(ref_local_trace_collector);
            m_status = LTRC_REACHABLE;
        } 
    
        //
        // Unregister the Trace Collector reference from the BufferSender
        //
        void BufferSender::unregister_trace_collector()
            throw()
        {
            CDMW_MUTEX_GUARD(m_mutex) ;
            m_local_trace_collector =  CdmwTrace::LocalTraceCollector::_nil();
            m_status = LTRC_STOPPED ;
        }  

    }
}
