/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#include <clocale>
#include <fstream>
#include <iostream>
#include <sstream>
#include <string>

#include <Foundation/common/Exception.hpp>
#include <Foundation/common/Locations.hpp>
#include <Foundation/common/Options.hpp>
#include <Foundation/common/System.hpp>
#include <Foundation/ossupport/OS.hpp>
#include <Foundation/orbsupport/CORBA.hpp>
#include <Foundation/orbsupport/ExceptionMinorCodes.hpp>
#include <Foundation/orbsupport/OrbSupport.hpp>
#include <Foundation/orbsupport/StrategyList.hpp>

#include <SystemMngt/platforminterface/ProcessBehaviour.hpp>
#include <SystemMngt/platforminterface/PlatformInterface.hpp>
#include <SystemMngt/platforminterface/ServiceNames.hpp>
#include <Repository/repositoryinterface/RepositoryInterface.hpp>

#include "tracecollector/ClientThread.hpp"
#include "tracecollector/ProcessAgent_impl.hpp"
#include "tracecollector/CollectorInit.hpp"
#include "tracecollector/CollectorOptions.hpp"



// definition of local constants in an unnammed namespace
namespace 
{   
   /**
    * Print the usage.
    *
    * @param global true if started as global collector
    * @param out the output stream.
    * @param programName the program name.
    */
   static void
   usage(bool global, std::ostream& out, const std::string& programName)
   {
      using std::endl;
      using Cdmw::Common::Options;

      out << "usage: " << programName << " options..." << endl << endl
          << "options:" << endl
          << "  --help                             " << "Display the help." << endl 
          << "  " << Cdmw::Trace::CollectorOptions::INTERACTIVE_OPT 
                         << "                      " << "Start in interactive mode." << endl << endl
          << "  --collector-name=NAME              " << "Name of the trace collector." << endl
          << "  [--trace-file-name=NAME]           " << "Name of the trace file." << endl
          << "  [--component-name=NAME]            " << "Filter on this component name." << endl
          << "  [--domain-name=NAME]               " << "Filter on this domain." << endl
          << "  [--level=NUM]                      " << "Filter on this filter." << endl
          << "  [--trace-format=<H|V|X>]           " << "Format of a trace: Horizontal, Vertical (default) or XML." << endl
          << "  [--CdmwLocalisationService=PORT]   " << "Service port of the trace collector." << endl
          << "  [--CdmwProcessPort=PORT]           " << "Process port of the trace collector." << endl
          << "  [" << Options::CALLBACK_IOR_OPTION << "=IOR]"
                                   << "            " << "IOR of the ProcessCallback object." << endl
          << "  [--naming-repos-url=URL]           " << "URL of the CDMW Naming and Repository service." << endl
          << "  [--queue-strategy=NAME]            " << "Strategy \"drop_latest\" or \"remove_oldest\"." << endl
          << "  [--queue-size=NUM]                 " << "Size of the queue in KBytes." << endl
          << "  [--queue-str-size=NUM]             " << "Size of the string queue in KBytes." << endl
          << "  [--trace-file-bak-num=NUM]         " << "Number of backup(s) of trace files." << endl
          << "  [--trace-file-num=NUM]             " << "Number of circular trace files." << endl
          << "  [--trace-msg-num=NUM]              " << "Number of messages in the circular trace files." << endl
          << "  [--flush-area-sz=NUM]              " << "Size in bytes of the flush area in the buffer for IIOP local collector." << endl
          << "  [--flush-area-nb=NUM]              " << "Number of flush areas in the buffer for IIOP local collector." << endl
          << "  [--miop-corbaloc=MIOP_ADDRESS]     " << "MIOP corbaloc, use corbaloc:miop.. syntax." << endl
          << "  [" << Cdmw::Trace::CollectorOptions::OPERATIONAL_CLOCK_NAME << "=NAME]" << endl
          << "                                     " << "Name of the Federated Controlled Clock used to get the operational time." << endl; 
      
         
      // only for local trace collector   
      if (!global)
      {
         out << "  [--trace-producer-path <Path 1> <Path 2> ...]" << endl
             << "                         " << "Path of Trace Producers " << endl
             << "                         " << "(Path is defined as host/application/process)." << endl
             << endl;
      }
  
   }


   /**
    * Default process behaviour.
    */
   class MyProcessBehaviour: public Cdmw::PlatformMngt::ProcessBehaviour
   {
      public:
         MyProcessBehaviour(CORBA::ORB_ptr orb)
            throw(CORBA::SystemException)
            : m_orb(CORBA::ORB::_duplicate(orb))
         {
         }


         ~MyProcessBehaviour()
            throw()
         {
         }


         virtual void
         initialise(const CdmwPlatformMngtBase::StartupKind& startup_kind)
            throw(CORBA::SystemException)
         {
            // Do nothing!
         }


         virtual void
         run()
            throw(CdmwPlatformMngt::ProcessDelegate::NotReadyToRun,
                  CORBA::SystemException)
         {
            // Do nothing!
         }


         virtual void
         stop()
            throw(CORBA::SystemException)
         {
            m_orb->shutdown(false);
         }


         virtual bool
         is_alive()
            throw(CORBA::SystemException)
         {
            return true;
         }


      private:
         CORBA::ORB_var m_orb;
   };

} // End namespace anonymous

namespace Cdmw
{

   namespace Trace
   {
      namespace CollectorInit
      {
         using namespace Cdmw::OrbSupport;
         using namespace Cdmw::OsSupport;
         using namespace Cdmw::Common;
         using namespace Cdmw::PlatformMngt;
        
        
         /**
          * Start the client thread in interactive mode and start the ORB.
          *
          * @param orb the ORB.
          * @param interactive_mode flag to set to true if interactive mode is wanted.
          */
         int run_processing (CORBA::ORB_ptr orb, bool interactive_mode)
         {
            int status = SUCCESS;

            // if interactive mode not set.
            if (!interactive_mode) 
            {
               // just run the ORB.
               orb->run();
            }
            else 
            {
               // in interactive mode, create a new client thread.
               ClientThread client(orb, std::cin, std::cout);

               // start the client and run the ORB.
               client.start();
               orb->run();

               // wait for the completion of the client thread
               // once the CORBA loop has stopped.
               client.join();
            }
            
            return status;
         }
        
         /**
          * Create the process agent
          *
          * @param orb the ORB.
          * @param root_POA the root poa.
          *
          */
         void create_process_agent (
            bool global,
            CORBA::ORB_ptr orb,
            PortableServer::POA_ptr root_POA)
         {
            try
            {
               TraceCollector::ProcessAgent_var agent_ref =
                  TraceCollector::ProcessAgent::_nil();
                            
               // create the ProcessAgent Servant.
               ProcessAgent_impl* p_process_agent_servant =
                  new ProcessAgent_impl(orb, root_POA);
               PortableServer::ServantBase_var servant_var = p_process_agent_servant;

               // activate servant on POA.
               PortableServer::ObjectId_var oid =
                  root_POA->activate_object(p_process_agent_servant);
               CORBA::Object_var object = root_POA->id_to_reference(oid.in());
               agent_ref = TraceCollector::ProcessAgent::_narrow(object.in());
            
               // agent corbaloc id
               std::string corbaloc_id;
                
               if (global)
               {
                  corbaloc_id = CollectorOptions::GLOB_AGENT_CORBALOC_ID;
               }
               else
               {
                  corbaloc_id = CollectorOptions::LOC_AGENT_CORBALOC_ID;
               }
                
               // bind the ProcessAgent to a corbaloc name.
               Cdmw::OrbSupport::OrbSupport::bind_object_to_corbaloc(
                  orb, corbaloc_id, agent_ref.in());                    
            }
            catch (const std::bad_alloc&) {
               std::cerr << "Process initialisation: bad allocation exception"
                         << std::endl;
            }
            catch (const Cdmw::Exception& ex) {
               std::cerr << ex.what() << std::endl;
            }
            catch (const CORBA::SystemException& ex) {
               std::cerr << ex << std::endl;
            }
            catch(const CORBA::Exception& ex) {
               std::cerr << ex << std::endl;
            }
         }



         /**
          * Get the options
          */
         int get_collector_options (bool global,
                                    int argc, char* argv[],
                                    CollectorOptions::Options & options,
                                    int &new_argc, char** &new_argv,
                                    const std::string& def_service_port,
                                    const std::string& def_process_port)
         {
            // check the number of arguments.
            if (argc == 1) 
            {
               usage(global, std::cout, argv[0]);
               return FAILURE;
            }

            // --help
            if (strcmp(argv[1], "--help") == 0) 
            {
               usage(global, std::cout, argv[0]);
               return EXIT;
            }

            std::string option_val;

            // --interactive
            option_val = OS::get_option_value(argc, argv, CollectorOptions::INTERACTIVE_OPT);
            options.interactive_mode = false;
            if (option_val != "no") 
            {
               options.interactive_mode = true;
            }

            // get the miop corbaloc used for communication between local&global
            option_val = OS::get_option_value(argc, argv, CollectorOptions::COLLECTOR_MIOP_CORBALOC);
            if (option_val == "yes") 
            {
               std::cout <<
                  "If MIOP used, the corbaloc adress must be specified with: "
                         << CollectorOptions::COLLECTOR_MIOP_CORBALOC << "=<the_corbaloc_adress>"
                         << std::endl;
               usage(global, std::cout, argv[0]);
               return FAILURE;
            }
            if (option_val == "no") 
            {
               std::cout << "IIOP will be used between local&global collector"
                         << std::endl;
            }
            else
            {
               std::cout <<
                  "MIOP will be used between local&global collector with " <<
                  option_val << std::endl;
               options.miop_corbaloc = option_val;
            }

            // get the trace collector name.
            option_val = OS::get_option_value(argc, argv, CollectorOptions::COLLECTOR_NAME_OPT);
            // Collector will stop & display that trace collector name
            // must be specified if we are in one of the 3 following cases:
            // 1) no value for this option was detected
            //    and this is a local trace collector
            // 2) no value for this option was detected and collector will use IIOP
            // 3) option value is specified, but without value..
            if ( ((option_val == "no") &&
                  (!global || (options.miop_corbaloc == "")) ) ||
                 option_val == "yes") 
            {
               std::cout << CollectorOptions::COLLECTOR_NAME_OPT << "=NAME option must be defined"
                         << std::endl;
               usage(global, std::cout, argv[0]);
               return FAILURE;
            }
            if (option_val != "no")
            {
               options.collector_name = option_val;
            }

            // get the trace file name.
            option_val = OS::get_option_value(argc, argv, CollectorOptions::TRACE_FILE_NAME_OPT);
            if (option_val == "no") 
            {
               option_val = "";
            }
            else if (option_val == "yes") 
            {
               std::cout << CollectorOptions::TRACE_FILE_NAME_OPT << "=NAME option must be defined"
                         << std::endl;
               usage(global, std::cout, argv[0]);
               return FAILURE;
            }
            
            options.trace_file_name = option_val;

            // get the name of the component to filter on.
            option_val = OS::get_option_value(argc, argv, CollectorOptions::COMPONENT_NAME_OPT);
            if (option_val == "no") 
            {
               option_val = Logging::ALL_COMPONENT_NAMES;
            }
            else if (option_val == "yes") 
            {
               std::cout << "missing value for the "
                         << CollectorOptions::COMPONENT_NAME_OPT << " option" << std::endl;
               usage(global, std::cout, argv[0]);
               return FAILURE;
            }
            
            options.component_name = option_val;

            // get the name of the domain to filter on.
            option_val = OS::get_option_value(argc, argv, CollectorOptions::DOMAIN_NAME_OPT);
            if (option_val == "no") 
            {
               option_val = Logging::ALL_DOMAINS;
            }
            else if (option_val == "yes") 
            {
               std::cout << CollectorOptions::DOMAIN_NAME_OPT << "=NAME option must be defined"
                         << std::endl;
               usage(global, std::cout, argv[0]);
               return FAILURE;
            }

            options.domain_name = option_val;
            
            // get the level to filter on.
            option_val = OS::get_option_value(argc, argv, CollectorOptions::LEVEL_OPT);
            if (option_val == "no") 
            {
               options.level_value = Logging::ALL_LEVELS_SET;
            }
            else 
            {
               std::istringstream level_option_strm(option_val);
               if (! option_val.compare(0, 2, "0x")) // hexadecimal
               {
                  level_option_strm.seekg(2);
                  level_option_strm >> std::hex >> options.level_value;
               }
               else // decimal
               {
                  level_option_strm >> std::dec >> options.level_value;
               }

               if (level_option_strm.fail()) 
               {
                  std::cout << "bad value for the " << CollectorOptions::LEVEL_OPT << "=NUM option"
                            << std::endl;
                  return FAILURE;
               }
            }

            // get the format of a trace (default: vertical).
            options.trace_format = Message::V_FORMAT;
            option_val = OS::get_option_value(argc, argv, CollectorOptions::TRACE_FORMAT_OPT);
            if (option_val != "no") 
            {
               if (option_val == CollectorOptions::TRACE_FORMAT_VAL_V) 
               {
                  options.trace_format = Message::V_FORMAT;
               }
               else if (option_val == CollectorOptions::TRACE_FORMAT_VAL_H) 
               {
                  options.trace_format = Message::H_FORMAT;
               }
               else if (option_val == CollectorOptions::TRACE_FORMAT_VAL_X) 
               {
                  options.trace_format = Message::X_FORMAT;
               }
               else 
               {
                  std::cout << "bad value for the "
                            << CollectorOptions::TRACE_FORMAT_OPT << " option" << std::endl
                            << "the possible values are : "
                            << CollectorOptions::TRACE_FORMAT_VAL_V << " or " 
                            << CollectorOptions::TRACE_FORMAT_VAL_H << " or "
                            << CollectorOptions::TRACE_FORMAT_VAL_X << std::endl;
                  return FAILURE;
               }
            }

            // get the trace collector port numbers
            // (these values are passed to ORB_init via ORBsupport::ORB_init).

            // service port option
            option_val = OS::get_option_value(argc, argv,
                                              Options::LOCALISATION_SERVICE_OPTION);
            if (option_val != "no") 
            {
               int port_nbr;

               std::istringstream port_strm(option_val);
               port_strm >> port_nbr;

               if (port_strm.fail()) 
               {
                  std::cout << "bad value for the "
                            << Options::LOCALISATION_SERVICE_OPTION
                            << "=PORT option" << std::endl;
                  return FAILURE;
               }
            }
            
            // store the service port (="no" if default port to use)
            options.service_port = option_val;

            // process port option
            option_val = OS::get_option_value(argc, argv,
                                              Options::PROCESS_PORT_OPTION);
            if (option_val != "no") 
            {
               int port_nbr;

               std::istringstream port_strm(option_val);
               port_strm >> port_nbr;

               if (port_strm.fail()) 
               {
                  std::cout << "bad value for the "
                            << Options::PROCESS_PORT_OPTION
                            << "=PORT option" << std::endl;
                  return FAILURE;
               }                
            }
            
            // store the process port (="no" if default port to use)
            options.process_port = option_val;
            
            // rebuilt the process argument with the default ports if required
            if (options.service_port == "no" || options.process_port == "no") 
            {
               int arg_inx = argc;
                
               try 
               {
                  // allocate enough place to contain each argument string
                  // plus the port option arguments
                  // and a NULL string to indicate the end.
                  new_argv = new char*[argc + 3];

                  for (int i = 0; i < argc; ++i) 
                  {
                     new_argv[i] = argv[i];
                  }

                  new_argv[argc] = NULL;
                  new_argv[argc + 1] = NULL;
                  new_argv[argc + 2] = NULL;

                  // insert the default ports in the argument list if necessary.            
                  // (if a port is defined, it is set already in the argument list)
                  if (options.service_port == "no") 
                  {
                     std::string service_port_option = Options::LOCALISATION_SERVICE_OPTION;
                     service_port_option += "=" + def_service_port;
                     new_argv[arg_inx] = new char[service_port_option.size() + 1];
                     ::strcpy(new_argv[arg_inx], service_port_option.c_str());
                     arg_inx++;
                     options.service_port = def_service_port;
                  }

                  if (options.process_port == "no") 
                  {
                     std::string process_port_option = Options::PROCESS_PORT_OPTION;
                     process_port_option += "=" + def_process_port;
                     new_argv[arg_inx] = new char[process_port_option.size() + 1];
                     ::strcpy(new_argv[arg_inx], process_port_option.c_str());
                     arg_inx++;
                     options.process_port = def_process_port;
                  }
               }
               catch (const std::bad_alloc&) 
               {
                  std::cout << "bad allocation for new arguments" << std::endl;
                  return FAILURE;
               }

               // set number of arguments
               new_argc = arg_inx;
            }


            // get the IOR of the Naming & Repository service.

            // we are either launched by the PlatformManagement and in which case
            // we can query for the reference,
            // or the reference is directly passed as command line argument.
            
            options.platform_managed =
               PlatformInterface::Is_launched_by_PlatformManagement(argc, argv);

            // get the Naming & Repository URL.
            option_val = OS::get_option_value (argc, argv, CollectorOptions::NAMING_REPOS_URL_OPT);
            if (option_val == "no") 
            {
               option_val = "";
            }

            if (!options.platform_managed && option_val == "") 
            {
               std::cout << "the " << Options::CALLBACK_IOR_OPTION
                         << "=IOR option or the " << CollectorOptions::NAMING_REPOS_URL_OPT
                         << "=URL option must be defined" << std::endl;
               usage(global, std::cout, argv[0]);
               return FAILURE;
            }
            
            options.naming_repos_url = option_val;

            // get the queue strategy (default: remove_oldest)
            options.queue_strategy = Message::REMOVE_OLDEST;
            option_val = OS::get_option_value (argc, argv, CollectorOptions::TRACE_Q_STRATEGY_OPT);
            if (option_val != "no") 
            {
               if (option_val == CollectorOptions::TRACE_Q_STRATEGY_VAL_DL)
               {
                  options.queue_strategy = Message::DROP_LATEST;
               }
               else if (option_val == CollectorOptions::TRACE_Q_STRATEGY_VAL_RO) 
               {
                  options.queue_strategy = Message::REMOVE_OLDEST;
               }
               else 
               {
                  std::cout << "bad value for the " << CollectorOptions::TRACE_Q_STRATEGY_OPT
                            << " option" << std::endl
                            << "the possible values are: "
                            << CollectorOptions::TRACE_Q_STRATEGY_VAL_DL << " or "
                            << CollectorOptions::TRACE_Q_STRATEGY_VAL_RO << std::endl;
                  return FAILURE;
               }
            }

            // get the queue size.
            option_val = OS::get_option_value (argc, argv, CollectorOptions::TRACE_Q_SIZE_OPT);
            if (option_val == "no")
            {
               options.queue_size = 1000; // default to 1000 Kb.
            }
            else 
            {
               std::istringstream queue_size_option_strm(option_val);
               queue_size_option_strm >> options.queue_size;
               if (queue_size_option_strm.fail()) 
               {
                  std::cout << "bad value for the "
                            << CollectorOptions::TRACE_Q_SIZE_OPT << "=NUM option"
                            << std::endl;
                  return FAILURE;
               }
            }

            // get the queue string size.
            option_val = OS::get_option_value (argc, argv, CollectorOptions::TRACE_Q_STRING_SIZE_OPT);
            if (option_val == "no") 
            {
               options.queue_str_size = 100; // default to 100 Kb.
            }
            else 
            {
               std::istringstream queue_str_size_option_strm(option_val);
               queue_str_size_option_strm >> options.queue_str_size;
               if (queue_str_size_option_strm.fail()) 
               {
                  std::cout << "bad value for the "
                            << CollectorOptions::TRACE_Q_STRING_SIZE_OPT << "=NUM option"
                            << std::endl;
                  return FAILURE;
               }
            }

            // get the number of backups.
            option_val = OS::get_option_value (argc, argv, CollectorOptions::TRACE_FILE_BAK_NUM_OPT);
            if (option_val == "no") 
            {
               options.trace_file_bak_num = 2; // default to 2 backup files.
            }
            else 
            {
               std::istringstream trace_file_bak_num_option_strm(option_val);
               trace_file_bak_num_option_strm >> options.trace_file_bak_num;
               if (trace_file_bak_num_option_strm.fail()) 
               {
                  std::cout << "bad value for the "
                            << CollectorOptions::TRACE_FILE_BAK_NUM_OPT << "=NUM option"
                            << std::endl;
                  return FAILURE;
               }
            }

            // get the number of circular files.
            option_val = OS::get_option_value (argc, argv, CollectorOptions::TRACE_FILE_NUM_OPT);
            if (option_val == "no") 
            {
               options.trace_file_num = 2; // default to 2 files.
            }
            else 
            {
               std::istringstream trace_file_num_option_strm(option_val);
               trace_file_num_option_strm >> options.trace_file_num;
               if (trace_file_num_option_strm.fail()) 
               {
                  std::cout << "bad value for the "
                            << CollectorOptions::TRACE_FILE_NUM_OPT << "=NUM option"
                            << std::endl;
                  return FAILURE;
               }
            }

            // get the number of messages to store in a file.
            option_val = OS::get_option_value (argc, argv, CollectorOptions::TRACE_MSG_NUM_OPT);
            if (option_val == "no") 
            {
               options.trace_msg_num = 50; // default to 50 messages per file.
            }
            else 
            {
               std::istringstream trace_msg_num_option_strm(option_val);
               trace_msg_num_option_strm >> options.trace_msg_num;
               if (trace_msg_num_option_strm.fail()) 
               {
                  std::cout << "bad value for the "
                            << CollectorOptions::TRACE_MSG_NUM_OPT << "=NUM option"
                            << std::endl;
                  return FAILURE;
               }
            }
            
            options.trace_producer_inx = 0;
            
            if (global == false)
            {
               // for local collector :
               // skip all arguments before the TRACE_PRODUCER_PATH_OPT, after this option
               // there are only path in the repository to find the TraceProducer objects

               int producer_inx = 1;
               while (producer_inx < argc && argv[producer_inx]!= CollectorOptions::TRACE_PRODUCER_PATH_OPT) 
               {
                  producer_inx++;
               }
    
               // marker argument is found, the next argument must be a path in the Repository
               if (producer_inx != argc) 
               {
                  producer_inx++;
               }
            
               options.trace_producer_inx = producer_inx;
            }
            
            // get the nb of flushing area
            option_val = OS::get_option_value (argc, argv, CollectorOptions::FLUSH_AREA_NB);
            if (option_val == "no")
            {
               options.nb_flush_area = 2; // default to 2
            }
            else 
            {
               if (options.miop_corbaloc != "")
               {
                  std::cout << 
                      "Flush area options can't be specified if MIOP is used (no flush area with MIOP)"
                      << std::endl;
                  return FAILURE;
               }

               std::istringstream flush_area_nb_option_strm(option_val);
               flush_area_nb_option_strm >> options.nb_flush_area;
               if (flush_area_nb_option_strm.fail()) 
               {
                  std::cout << "bad value for the "
                            << CollectorOptions::FLUSH_AREA_NB << "=NUM option"
                            << std::endl;
                  return FAILURE;
               }
               std::cout << "TraceCollector: value for the "
                            << CollectorOptions::FLUSH_AREA_NB << "=" << options.nb_flush_area
                            << std::endl;
            }

            // get the flush area size.
            option_val = OS::get_option_value (argc, argv, CollectorOptions::FLUSH_AREA_SIZE);
            if (option_val == "no")
            {
               options.size_flush_area = 1000*1024; // default to 1000 Kb.
            }
            else 
            {
               if (options.miop_corbaloc != "")
               {
                  std::cout << 
                      "Flush area options can't be specified if MIOP is used (no flush area with MIOP)"
                      << std::endl;
                  return FAILURE;
               }

               std::istringstream flush_area_size_option_strm(option_val);
               flush_area_size_option_strm >> options.size_flush_area;
               if (flush_area_size_option_strm.fail()) 
               {
                  std::cout << "bad value for the "
                            << CollectorOptions::FLUSH_AREA_SIZE << "=NUM option"
                            << std::endl;
                  return FAILURE;
               }
               std::cout << "TraceCollector: value for the "
                            << CollectorOptions::FLUSH_AREA_SIZE << "=" << options.size_flush_area
                            << std::endl;
            }
            
            // get the nb of messages for each area
            option_val = OS::get_option_value (argc, argv, CollectorOptions::AREA_NB_MESSAGES);
            if (option_val == "no")
            {
               options.area_nb_messages = 150; // default to 150
            }
            else 
            {
               if (options.miop_corbaloc != "")
               {
                  std::cout << 
                      "Flush area options can't be specified if MIOP is used (no flush area with MIOP)"
                      << std::endl;
                  return FAILURE;
               }

               std::istringstream area_nb_messages_option_strm(option_val);
               area_nb_messages_option_strm >> options.area_nb_messages;
               if (area_nb_messages_option_strm.fail()) 
               {
                  std::cout << "bad value for the "
                            << CollectorOptions::AREA_NB_MESSAGES << "=NUM option"
                            << std::endl;
                  return FAILURE;
               }
               std::cout << "TraceCollector: value for the "
                            << CollectorOptions::AREA_NB_MESSAGES << "=" << options.area_nb_messages
                            << std::endl;
            }

            // reads the value of the operational clock name, if exists
            option_val = OS::get_option_value (argc, argv, CollectorOptions::OPERATIONAL_CLOCK_NAME);
            if (option_val == "no")
            {
               options.operational_clock_name.clear();
            }
            else
            {
               options.operational_clock_name = option_val;            
            }
            
            return SUCCESS;
         }
        
        
         /**
          * Initialise the orb and PlatformInterface
          *
          */
         void init_processing (
            int argc, char* argv[],
            bool platform_managed,
            std::string& naming_repos_url,
            CORBA::ORB_out p_orb,
            PortableGroup::GOA_out p_root_GOA,
            CdmwNamingAndRepository::Repository_out p_repository_ref,
            bool use_miop, bool use_federated_controlled_clock)
         {
            CORBA::ORB_var orb;
            
            // initialize the ORB.
            StrategyList orb_strategies;
            orb_strategies.add_OrbThreaded();
            orb_strategies.add_PoaThreadPool(CollectorOptions::POA_THREAD_POOL_SIZE);
            if (use_miop || use_federated_controlled_clock) {
               orb_strategies.add_multicast();
            }

            orb = Cdmw::OrbSupport::OrbSupport::ORB_init(argc, argv, orb_strategies);

            // get a reference to the root POA.
            CORBA::Object_var root_obj = orb->resolve_initial_references("RootPOA");
            PortableGroup::GOA_var root_GOA =
               PortableGroup::GOA::_narrow(root_obj.in());

            if (!use_miop)
            {
               // Add SYNC_WITH_TARGET policy on the orb 
               // so that the oneway operations will behave like two ways operations
               // (if needed, this policy could be set only on all global collectors
               //  objects, instead to be put on the orb)
               static Messaging::SyncScope sync_scope = Messaging::SYNC_WITH_TARGET;

               CORBA::Object_var pm_obj = orb->resolve_initial_references("ORBPolicyManager");

               CORBA::PolicyManager_var policy_manager = CORBA::PolicyManager::_narrow(pm_obj.in());

               // Set up the sync scope any.
               CORBA::Any sync_scope_any;
               sync_scope_any <<= sync_scope;

               // Set the length of the policy list.
               CORBA::PolicyList sync_scope_policy_list (1);
               sync_scope_policy_list.length (1);
      
               // Set up the sync scope policy.
               sync_scope_policy_list[0] =
                  orb->create_policy (Messaging::SYNC_SCOPE_POLICY_TYPE,
                                      sync_scope_any);
               policy_manager->set_policy_overrides
                  (sync_scope_policy_list, CORBA::ADD_OVERRIDE);
               sync_scope_policy_list[0]->destroy();
               // end of section defining SYNC_WITH_TARGET 
            }

            // activate the POA Manager.
            PortableServer::POAManager_var POA_mngr = root_GOA->the_POAManager();
            POA_mngr->activate();

            CdmwNamingAndRepository::Repository_var repository_ref =
               CdmwNamingAndRepository::Repository::_nil();

            // if run from the PlatformManagement then init the PlatformInterface.            
            if (platform_managed) 
            {
               // initialise the platform interface.
               PlatformInterface::Setup(orb.in(), argc, argv);

               // create a process behaviour and
               // acknowledge the creation of the process.
               PlatformInterface::Acknowledge_creation (
                  new MyProcessBehaviour(orb.in()));

               // get the repository service.
               try 
               {
                  CORBA::Object_var obj =
                     PlatformInterface::Get_service (
                        ServiceNames::NAMING_AND_REPOSITORY_SERVICE);

                  repository_ref =
                     CdmwNamingAndRepository::Repository::_narrow(obj.in());
               }
               catch (const CdmwPlatformMngtService::ServiceNotFound&) {
                  throw CORBA::BAD_PARAM(BAD_PARAMRepositoryNotFound,
                                         CORBA::COMPLETED_NO);
               }
               catch (const CdmwPlatformMngtService::ServiceUnavailable&) {
                  throw CORBA::BAD_PARAM(BAD_PARAMCannotGetRepositoryReference,
                                         CORBA::COMPLETED_NO);
               }
               catch (const CORBA::SystemException& ex) {
                  throw;
               }
               catch (const Cdmw::BadOrderException& ex) {
                  throw;
               }
               catch (const Cdmw::BadParameterException& ex) {
                  throw;
               }
            }
            
            // else not running from platformmngt
            else 
            {
               // get the repository from url.
               try 
               {
                  CORBA::Object_var obj =
                     orb->string_to_object(naming_repos_url.c_str());
                  repository_ref =
                     CdmwNamingAndRepository::Repository::_narrow(obj.in());
               }
               catch (...) 
               {
                  std::cerr << "invalid URL for the Naming & Repository "
                            << "service." << std::endl;
                  throw;
               }
            }

            // if repository reference undefined
            if (CORBA::is_nil(repository_ref.in())) 
            {
               throw CORBA::BAD_PARAM(BAD_PARAMInvalidRepositoryReference,
                                      CORBA::COMPLETED_NO);
            }
        
            // return parameters
            p_orb = orb._retn();
            p_root_GOA = root_GOA._retn();
            p_repository_ref = repository_ref._retn();        
         }
      }

   }; // End namespace Trace
}; // End namespace Cdmw

