/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#include <iostream>
#include <sstream>

#include <Foundation/common/Exception.hpp>
#include <Foundation/common/Assert.hpp>
#include <Foundation/common/System.hpp>
#include <Foundation/osthreads/ThreadHandle.hpp>
#include <Foundation/osthreads/MutexGuard.hpp>

#include "tracecollector/CollectorMngr.hpp"
#include "tracecollector/TraceCollector_impl.hpp"
#include "tracecollector/TraceCircularBuffer.hpp"
#include "tracecollector/TraceFileMngr.hpp"



namespace Cdmw
{

namespace Trace
{

/**
 * Purpose:
 * <p> CollectorFlusher Constructor
 * 
 */ 
CollectorFlusher::CollectorFlusher (
                          OsSupport::Barrier& barrier,
                          TraceCircularBuffer *p_circular_buffer,
                          const std::string& trace_file_base_name, 
                          short trace_file_bkp_nbr,
                          short max_trace_file_nbr, 
                          int max_trace_message_nbr,
                          Message::TraceFormat trace_format,
                          OsSupport::OS::TimeMode time_mode)
                throw (OutOfMemoryException,
                       AssertionFailedException,
                       InternalErrorException,
                       BadParameterException)
                       
                : OsSupport::Thread (),
                  m_thread_must_terminate (false),
                  m_barrier (barrier),
                  m_circular_buffer (p_circular_buffer),
                  m_trace_file_mngr (NULL)
{
    try
    {
        // get base name for trace file
        std::string trace_file_name = trace_file_base_name;
                
        // if backup of trace files is requested and file name is defined
        if (trace_file_bkp_nbr > 1 && !trace_file_base_name.empty())
        {
            // build name of backup info
            std::string backUpInfoFileName = trace_file_base_name;
            backUpInfoFileName += "_bkpinfo";
                    
            short current_bkp;
                    
            // open the file for read backup info
            std::ifstream bkp_file_in_str (backUpInfoFileName.c_str(), std::ios::in);
                    
            // if file can be opened
            if (bkp_file_in_str)
            {
                // get current backup from info file
                bkp_file_in_str >> current_bkp;
                      
                // if current backup not retrieve (empty file)
                // set the 1st backup
                if (!bkp_file_in_str.good())
                {
                    current_bkp = 1;
                }
                      
                // else increment current backup
                else
                {
                    current_bkp ++;
                        
                    if (current_bkp > trace_file_bkp_nbr)
                    {
                        current_bkp = 1;
                    }                       
                }
                      
                // close the file
                bkp_file_in_str.close();
            }
            else
            {
                current_bkp = 1;
            }
                    
                    
            // open the file for write or create file if does not exist
            std::ofstream bkp_file_out_str (backUpInfoFileName.c_str(), std::ios::out);

            // if file cannot be opened
            if (!bkp_file_out_str)
            {
                CDMW_THROW2(BadParameterException, "Backup Info File cannot be opened",
                            backUpInfoFileName.c_str());
            }
            else
            {
                // save current backup nbr in file  
                bkp_file_out_str << current_bkp << std::endl; 
            }
                    
            // close the file
            bkp_file_out_str.close();
                    
            // update the file name base with the backup nbr
            std::ostringstream name_str;
            name_str << trace_file_base_name << "_" << current_bkp;
                        
            trace_file_name = name_str.str();                    
        }
                
        // create trace file manager
        m_trace_file_mngr = new TraceFileMngr (trace_file_name, 
                                               max_trace_file_nbr, 
                                               max_trace_message_nbr,
                                               trace_format,
                                               time_mode);
    }
    catch (const std::bad_alloc&) 
    {
        CDMW_THROW (OutOfMemoryException);
    }
            
};


/**
 * Purpose:
 * CollectorFlusher Destructor
 * 
 */ 
CollectorFlusher::~CollectorFlusher()
    throw()
{
    // delete trace file manager
    if (m_trace_file_mngr != NULL)
    {
        delete m_trace_file_mngr;
        m_trace_file_mngr = NULL;
    }
};




/**
 * Purpose:
 * <p> Ask to the internal thread to stop its job, the thread
 * will exit only when all trace messages will have been flushed
 * to disk.
 * When this function return the internal thread have stopped.
 *
 *@exception InternalErrorException
 *
*/ 
void CollectorFlusher::stop_thread()
    throw (InternalErrorException)
{
    // require the thread to terminate
    m_thread_must_terminate = true;

    // stop the buffer reading -> awake the thread, if it is sleeping
    m_circular_buffer->stopReading();

    try 
    {
        // wait the termination of our internal thread
        this->join();

        // No explanation, how is it possible : corrupted
    } 
    catch (const OsSupport::DeadlockException&)
    {
        CDMW_THROW (InternalErrorException);
    }
};


        
/**
 * Purpose:
 * <p> print error message in trace file
 * 
 *@param errMesg contains the message to print
 */ 
void CollectorFlusher::print_error_message (const std::string& err_mesg)
    throw()
{
    m_trace_file_mngr->print_error_message (err_mesg);
};



/**
 * Purpose:
 * <p> Call by the underlying thread system in a new thread
 * 
 */ 
void CollectorFlusher::run()
    throw()
{
    try
    {
        // We notify the CollectorMngr, we are ready
        m_barrier.wait();

        // message header data buffer
        TraceCircularBuffer::TraceMessageRaw message_data;

        // flag to indicate id data read
        bool data_read;
        bool loop = true;

        // Infinite loop
        // Check if someone ask the thread to terminate
        while (loop) 
        {
            // read trace message
            // if no message available thread is set to wait by CircularBuffer
            bool success = 
                  m_circular_buffer->getNextMessage(message_data, data_read);

            // if no error and data has been read, print message
            if (success == true && data_read == true)
            {
                m_trace_file_mngr->print_message (&message_data);
            }

            // else if error or thread to terminate, break the loop
            else if (success == false || m_thread_must_terminate == true)
            {
               loop = false;
            }
        }
      
        // print end of trace message in the current trace file
        m_trace_file_mngr->print_end_of_trace();
    }
    // catch any exception and assert it
    catch (...)
    {
        CDMW_ASSERT(false);
    }
};


       
/**
* This variable is set to true, when the CollectorMngr::Init() function
* has been called.
*/
bool CollectorMngr::M_init_done = false;

/**
* Contains the CollectorMngr singleton initialised by the 
* CollectorMngr::Init() function.
*/
CollectorMngr* CollectorMngr::M_singleton = NULL;

/**
 * CollectorMngr Constructor.
 *
 * Note:
 * m_barrier (counter set to 2)
 * The CollectorMngr constructor and the CollectorFlusher thread have to reach
 * their respective waiting point to continue their processing.
 */
CollectorMngr::CollectorMngr(
        PortableServer::POA_ptr trace_POA,
        const std::string& component_name,
        const std::string& domain,
        Logging::LevelType level,
        Message::TraceFormat trace_format,
        Message::CircularMode queue_strategy,
        size_t queue_size,
        size_t string_size,
        const std::string& trace_file_name,
        short trace_file_bkp_nbr,
        short max_trace_file_nbr,
        int max_trace_message_nbr,
        OsSupport::OS::TimeMode time_mode)
    throw(OutOfMemoryException,
          BadParameterException,
          InternalErrorException)
        : m_filter_mngr(NULL),
          m_collector_flusher(NULL),
          m_circular_buffer(NULL),
          m_barrier(2)
{

    try 
    {
        // set sizes from KBytes to Bytes
        queue_size *= 1024;
        string_size *= 1024;

        // We create the object in charge of determining, if a specific level/domain must be traced
        m_filter_mngr = new Logging::FilterMngr(level);

        // create a new filter
        m_filter_mngr->activate_level(component_name, domain, level);

        // We create the object in charge of circular buffer
        m_circular_buffer = new TraceCircularBuffer(queue_size, string_size, queue_strategy);

        // We create the thread in charge of flushing Trace Message as a background task

#       ifdef _MSC_VER
        // The auto_ptr implementation shipped with visual is not compliant
        //    and doesn't provide reset(), but it broken as it used an assignement
        //    operator with const auto_ptr&, we use this deviation

        // We create the thread in charge of communicating with collectors
        m_collector_flusher = 
                std::auto_ptr <CollectorFlusher> (new CollectorFlusher(m_barrier, 
                                                                       m_circular_buffer,
                                                                       trace_file_name,
                                                                       trace_file_bkp_nbr,
                                                                       max_trace_file_nbr, 
                                                                       max_trace_message_nbr,
                                                                       trace_format,
                                                                       time_mode));
#        else
        m_collector_flusher.reset (new CollectorFlusher (m_barrier,m_circular_buffer,
                                                          trace_file_name,
                                                          trace_file_bkp_nbr,
                                                          max_trace_file_nbr,
                                                          max_trace_message_nbr,
                                                          trace_format,
                                                          time_mode));
#        endif

        try
        {
            // Ok, we start the background thread in charge of fluhsing data
            m_collector_flusher->start();

            // Before returning, we wait it declares itself as operational
            m_barrier.wait();
        }
        // We have not choose to assign specific right to the thread
        //  we are corrupted
        catch (const OsSupport::ThreadSchedulingException&)
        {
            CDMW_THROW (InternalErrorException);
        }
    } 
    catch (const std::bad_alloc&) 
    {
        CDMW_THROW (OutOfMemoryException);
    }
}


//
// Free memory and terminate the Trace service
//
void CollectorMngr::Cleanup()
    throw (InternalErrorException)
{
    // The library is not initialised
    CDMW_ASSERT (M_init_done == true);

    // We stop the thread in charge of flushing data
    M_singleton->m_collector_flusher->stop_thread();

    M_init_done = false;
    delete M_singleton;

    M_singleton = NULL;
}



//
// CollectorMngr Dtor
// 
CollectorMngr::~CollectorMngr()
    throw()
{
    if (m_filter_mngr != NULL)
    {
        delete m_filter_mngr;
        m_filter_mngr = NULL;
    }

    if (m_circular_buffer != NULL)
    {
        delete m_circular_buffer;
        m_circular_buffer = NULL;
    }
     
    // The user has not called the Cleanup function
    CDMW_ASSERT (M_init_done == false);
}




CollectorMngr& CollectorMngr::Instance()
    throw ()
{
    // The user has not called the Init() function
    CDMW_ASSERT (M_init_done == true);
    
    return *M_singleton ;
}



//
// returns true if the specified level is to be traced
//
bool CollectorMngr::Is_to_be_traced (const std::string& domain, Logging::LevelType level)
{
    return Is_to_be_traced("", domain, level);
}


/**
 * Check if the specified level is activated.
 */
bool CollectorMngr::Is_to_be_traced (const std::string& component_name,
                                     const std::string& domain,
                                     Logging::LevelType level)
{
    // the user has not called CollectorMngr::Init() before
    CDMW_ASSERT(M_init_done == true);

    return M_singleton->m_filter_mngr->is_activated(component_name, domain, level);
}


CdmwTrace::Collector_ptr CollectorMngr::get_trace_collector()
{
    // duplicate is implicit
    CdmwTrace::Collector_var collector = m_trace_collector;
    
    return collector._retn();
}

}; // End namespace Trace
}; // End namespace Cdmw

