/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#include <iostream>
#include <sstream>

#include "tracecollector/GlobalCollectorMngr.hpp"
#include "tracecollector/TraceCircularBuffer.hpp"


namespace Cdmw
{

namespace Trace
{

PortableServer::ObjectId GlobalCollectorMngr::M_miopGlobalCollOid;

PortableServer::POA_var GlobalCollectorMngr::M_miopPOA;

/**
 * Constructor.
 *
 */
GlobalCollectorMngr::GlobalCollectorMngr(
        PortableServer::POA_ptr trace_POA,
        const std::string& component_name,
        const std::string& domain,
        Logging::LevelType level,
        Message::TraceFormat trace_format,
        Message::CircularMode queue_strategy,
        size_t queue_size,
        size_t string_size,
        const std::string& trace_file_name,
        short trace_file_bkp_nbr,
        short max_trace_file_nbr,
        int max_trace_message_nbr,
        OsSupport::OS::TimeMode time_mode)
    throw(OutOfMemoryException,
          BadParameterException,
          InternalErrorException)
        : CollectorMngr(trace_POA, 
                        component_name, 
                        domain, 
                        level,
                        trace_format, 
                        queue_strategy, 
                        queue_size,
                        string_size, 
                        trace_file_name, 
                        trace_file_bkp_nbr,
                        max_trace_file_nbr, 
                        max_trace_message_nbr, 
                        time_mode)
{
    // We must create the Servant to handle registration request (ref count is incremented)
    TraceCollector_impl* p_global_collector_servant =
                             new TraceCollector_impl (this, m_filter_mngr);

    // create an object var to take pointer ownership
    // (ref count will be decremented when var will be destroyed at the method end)
    PortableServer::ServantBase_var servant_var = p_global_collector_servant;

    // create an object id
    PortableServer::ObjectId_var oid =
                  PortableServer::string_to_ObjectId("TraceCollector");
    try
    {
        // activate servant on POA (ref count is incremented)
        trace_POA->activate_object_with_id(oid.in(), p_global_collector_servant);
        CORBA::Object_var object = trace_POA->id_to_reference(oid.in());
        m_trace_collector = CdmwTrace::Collector::_narrow(object.in());
    }
    catch (const PortableServer::POA::WrongPolicy &)
    {
        CDMW_THROW2(Cdmw::BadParameterException, "Wrong POA Policy for activate_object", "NULL");
    }
    catch (const PortableServer::POA::ObjectAlreadyActive & )
    {
        CDMW_THROW2(BadParameterException, "Trace Collector Object already active", "NULL"); 
    } 
    catch (const PortableServer::POA::ServantAlreadyActive & )
    {
        CDMW_THROW2(BadParameterException, "Trace Collector Servant already active", "NULL");  
    }
    catch (const CORBA::Exception& ex)
    {
        std::cerr << "Trace Collector servant activation error" << "\n";
        std::cerr << "CORBA Exception : \n" << ex << std::endl;
        CDMW_ASSERT(false);
    }

    if ( M_miopGlobalCollOid.length() != 0 )
    {
      M_miopPOA->activate_object_with_id(M_miopGlobalCollOid, p_global_collector_servant);
    }    
}




/**
 * Initialize the Trace collector.
 */
CdmwTrace::Collector_ptr
GlobalCollectorMngr::Init(
        PortableServer::POA_ptr trace_POA,
        const std::string& component_name,
        const std::string& domain,
        Logging::LevelType level,
        Message::TraceFormat trace_format,
        Message::CircularMode queue_strategy,
        size_t queue_size,
        size_t string_size,
        const std::string& trace_file_name,
        short trace_file_bkp_nbr,
        short max_trace_file_nbr,
        int max_trace_message_nbr,
        OsSupport::OS::TimeMode time_mode,
        PortableServer::POA_ptr miop_POA,
        PortableServer::ObjectId miopGlobalCollOid)
    throw(OutOfMemoryException,
          BadParameterException,
          InternalErrorException)
{
    // The user try to initialise twice the collector
    CDMW_ASSERT (M_init_done == false);

    M_miopGlobalCollOid = miopGlobalCollOid;
    M_miopPOA = PortableServer::POA::_duplicate(miop_POA);

    try 
    {
        M_singleton = new GlobalCollectorMngr (trace_POA,
                                               component_name,
                                               domain,
                                               level,
                                               trace_format,
                                               queue_strategy,
                                               queue_size,
                                               string_size,
                                               trace_file_name,
                                               trace_file_bkp_nbr,
                                               max_trace_file_nbr,
                                               max_trace_message_nbr,
                                               time_mode);
    }
    catch (const std::bad_alloc&) 
    {
        CDMW_THROW (OutOfMemoryException);
    }

    M_init_done = true;
    
    // We return the reference on the object managing communication with
    //  collector, it will be registered in the repository by the caller
    return M_singleton->get_trace_collector();
}



//
// Dtor
// 
GlobalCollectorMngr::~GlobalCollectorMngr()
    throw()
{
}


/**
 * Write a message in the circular buffer (used by global collector).
 */
bool GlobalCollectorMngr::write_message(
        const CdmwTrace::MessagesHeader& messages_hdr,
        const CdmwTrace::FormattedMessage& fmt_message)
    throw(OutOfMemoryException)
{
    bool ret = true;

    CDMW_ASSERT(m_collector_flusher.get() != NULL);
    CDMW_ASSERT(m_circular_buffer != NULL);

    // write out the message only if the message passes at least one filter.
    std::string domain = fmt_message.header.the_level.the_domain.in();
    std::string component_name =
        fmt_message.header.the_level.the_component_name.in();

    if (CollectorMngr::Is_to_be_traced (component_name,
                                        domain,
                                        fmt_message.header.the_level.the_value))
    {
        // if not filtered, write message in circular buffer.
        // collector flusher is signaled to flush message on disk.
        TraceCircularBuffer::WriteStatus write_status;
        ret = m_circular_buffer->writeMessage (messages_hdr,
                                               fmt_message,
                                               write_status);

        if (write_status == TraceCircularBuffer::STS_DROP_LATEST) 
        {
            m_collector_flusher->print_error_message(
                "Queue is full : latest trace message has been dropped");
        }
        else if (write_status == TraceCircularBuffer::STS_REMOVE_OLDEST) 
        {
            m_collector_flusher->print_error_message(
                "Queue is full : oldest trace message has been removed");
        }
    }

    return ret;
}


}; // End namespace Trace
}; // End namespace Cdmw

