/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

#include <sstream>
#include "tracecollector/HorizontalTracePrinter.hpp"

#define BUFFER_MAX_LENGTH 128

namespace Cdmw
{
   namespace Trace
   {   
      //
      // Replaces an empty string by "[]" and truncate to BUFFER_MAX_LENGTH if needed
      //
      inline void HorizontalTracePrinter::manage_if_empty_field(char *buffer, const char *format, const char *value) const
      {
         static const char* EMPTY_FIELD = "[]";
         if (strcmp(value, "") == 0)
            sprintf (buffer, "%s", EMPTY_FIELD);
         else
         {
            // The following code will truncate 'value' to BUFFER_MAX_LENGTH, if it is longer than BUFFER_MAX_LENGTH
            char strTmp[BUFFER_MAX_LENGTH+1];
            if (strlen(value) > BUFFER_MAX_LENGTH)
            {
               // if we truncate, we replace the last 3 characters by '...'
               strncpy(strTmp, value, BUFFER_MAX_LENGTH-3);
               strTmp[BUFFER_MAX_LENGTH-3] = 0;
               strcat(strTmp, "...");
            }
            else
            {
               strncpy(strTmp, value, BUFFER_MAX_LENGTH);
            }

            strTmp[BUFFER_MAX_LENGTH] = 0;
            sprintf (buffer, format, strTmp);
         }
      }
   
      //
      // prints horizontal message
      //
      void HorizontalTracePrinter::print_message(std::ostream & os, const TraceCircularBuffer::TraceMessageRaw & Data) const
         throw()
      {
         char string_buf[BUFFER_MAX_LENGTH+1];
     
         // UTC Time
         sprintf (string_buf, "%s", timestamp_to_string(Data.m_messageHeader.m_timestamping,"%Y-%m-%dT%H:%M:%SZ").c_str());
         os << string_buf << " ";            
         sprintf (string_buf, "%012lu", Data.m_messageHeader.m_timestamping.seconds);
         os << string_buf << ".";
         sprintf (string_buf, "%06lu", Data.m_messageHeader.m_timestamping.microseconds);
         os << string_buf << " ";
     
         // Operational Time
         if (Data.m_messageHeader.m_operational_timestamping.seconds)
         {
            sprintf (string_buf, "%s", timestamp_to_string(Data.m_messageHeader.m_operational_timestamping,"%Y-%m-%dT%H:%M:%SZ").c_str());
            os << string_buf << " ";
            sprintf (string_buf, "%012lu", Data.m_messageHeader.m_operational_timestamping.seconds);
            os << string_buf << ".";
            sprintf (string_buf, "%-6lu", Data.m_messageHeader.m_operational_timestamping.microseconds);
            os << string_buf << " ";
         }
         else 
            manage_if_empty_field(string_buf, "%s", "");
     
         manage_if_empty_field(string_buf, "%s", Data.m_mainHeader.m_pHostName);
         os << string_buf << " ";
     
         manage_if_empty_field (string_buf, "%s", Data.m_mainHeader.m_pApplicationName);
         os << string_buf << " ";
     
         manage_if_empty_field (string_buf, "%s", Data.m_mainHeader.m_pProcessName);
         os << string_buf << " ";
     
         {
            std::ostringstream stream_buf;
       
            stream_buf << Data.m_mainHeader.m_pid;
            manage_if_empty_field (string_buf, "%-5.5s", stream_buf.str().c_str());
            os << string_buf << " ";
         }

         // ECR-0123
         manage_if_empty_field(string_buf, "%s", Data.m_messageHeader.m_pComponentName);
         os << string_buf << " ";

         manage_if_empty_field (string_buf, "%s", Data.m_messageHeader.m_pUserDomain);
         os << string_buf << " ";
     
         sprintf (string_buf, "%-8lX", Data.m_messageHeader.m_userLevel);
         os << string_buf << " ";
     
         {
            std::ostringstream stream_buf;
           
            stream_buf << Data.m_messageHeader.m_tid;
            manage_if_empty_field (string_buf, "%-5.5s", stream_buf.str().c_str());
            os << string_buf << " ";
         }
     
         // extract the file name from the full path
         std::string file_name = Data.m_messageHeader.m_pFileName;
         int inx = file_name.rfind ('/');
         if (inx < 0 || (unsigned) inx > file_name.size())
         {
            inx = 0;
         }
         else
         {
            inx ++;
         }
     
         manage_if_empty_field (string_buf, "%s", file_name.substr(inx).c_str());
         os << string_buf << " ";
     
         sprintf (string_buf, "%-5.5d", Data.m_messageHeader.m_fileLine);
         os << string_buf << " ";

         manage_if_empty_field (string_buf, "%s", Data.m_ndc);
         os << string_buf << " ";
     
         os << "==>";
         os << Data.m_pBodyMessage << std::endl;
      }
      
      //
      // prints horizontal error message
      //
      void HorizontalTracePrinter::print_error_message (std::ostream & os, const std::string& errMesg) const
         throw()
      {
         os << TracePrinter::TRACE_ERR_MESG2 << std::endl;
         os << errMesg << std::endl;
         os << TracePrinter::TRACE_ERR_MESG1 << std::endl;
         os << std::endl;
      }

      //
      // prints horizontal header
      //        
      void HorizontalTracePrinter::print_header (std::ostream & os, int file_number) const
         throw()
      {
         os << TracePrinter::TRACE_HEADER1 << std::endl;

         os << TracePrinter::TRACE_HEADER2;
         if (file_number  > 0)
         {
            os << file_number << TracePrinter::TRACE_HEADER3;
         }
         os << std::endl;

         os << TracePrinter::TRACE_HEADER1 << std::endl;
         os << std::endl;
      }
     
      //
      // prints enf of trace
      //        
      void HorizontalTracePrinter::print_end_of_trace (std::ostream & os) const
         throw()
      {
         os << TracePrinter::END_OF_TRACE_FILE << std::endl;
      }
   
   }
}

