
/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

#include <Foundation/common/Locations.hpp>
#include <Foundation/common/Exception.hpp>

#include "tracecollector/InitUtilsLogCollector.hpp"
#include "tracecollector/LogCollectorMacroDefs.hpp"

namespace Cdmw
{

namespace Trace
{

void InitUtilsLogCollector::Create_log_collector ( CORBA::ORB_ptr orb,
                                                   PortableServer::POA_ptr poa,
                                                   const std::string& system_port,
                                                   size_t period,
                                                   size_t circular_buffer_size,
                                                   size_t max_logs_to_send,
                                                   size_t chunk_size
                                                   )
throw (OutOfMemoryException,
       BadParameterException,
       CosNaming::NamingContext::CannotProceed)
{
  CDMW_DEBUG_LOG_COLLECTOR("");

  // Create and initialize the Log Collector
  Cdmw::Trace::LogCollector_impl* log_collector = new Cdmw::Trace::LogCollector_impl();

  CdmwPlatformMngt::Daemon_var daemon = CdmwPlatformMngt::Daemon::_nil();
  try
  {
     // Get the platform daemon
     CDMW_DEBUG_LOG_COLLECTOR("Get the platform daemon...");
     
     std::ostringstream daemon_corbaloc;  
     daemon_corbaloc 
        << "corbaloc::localhost:" << system_port
        <<  "/" << CdmwPlatformMngt::DAEMON_NAME;
     CDMW_DEBUG_LOG_COLLECTOR("with corbaloc [" 
                              << daemon_corbaloc.str().c_str() <<"]");
     
     CDMW_DEBUG_LOG_COLLECTOR("Resolve reference...");
     CORBA::Object_var obj = orb->string_to_object(daemon_corbaloc.str().c_str());
     
     CDMW_DEBUG_LOG_COLLECTOR("Narrowing...");
     daemon = CdmwPlatformMngt::Daemon::_narrow(obj.in());
  }
  catch(...)
  {
     CDMW_WARN_LOG_COLLECTOR(__FUNCTION__ << " Unable to retrieve the Platform daemon reference !");
  }
         
         
  // Initialize the Log Collector
  try {
    log_collector->init(period, circular_buffer_size,
                        max_logs_to_send, chunk_size, poa, daemon.in());
  } catch (OutOfMemoryException& ofm) {
    throw ofm;
  } catch (BadParameterException bpe) {
    throw bpe;
  }


  // Register the Log Collector inside of the CDMW Daemon
  CdmwTrace::LogCollector_var log_collector_obj = log_collector->_this();
  log_collector->_remove_ref();
  if (CORBA::is_nil(log_collector_obj.in())) {
    CDMW_WARN_LOG_COLLECTOR("Log Collector is nil");
    return ;
  }
  Create_nameservice_url (system_port);
  Cdmw::CommonSvcs::Naming::NamingInterface* ni = Get_naming_interface_from_daemon(orb);
  try {
    CDMW_DEBUG_LOG_COLLECTOR("corbaloc of the CDMW daemon = " << InitUtils::M_daemon_naming_context_url);
    ni->rebind(Cdmw::Common::Locations::DEFAULT_LOG_COLLECTOR_ID, log_collector_obj.in(), false);
    CDMW_DEBUG_LOG_COLLECTOR("rebind");
  } catch (CosNaming::NamingContext::NotFound& nf) {
    CDMW_ERROR_LOG_COLLECTOR("The Context of the Log Collector was not found: " << nf);
    CDMW_THROW1(Cdmw::BadParameterException, 
                "the system_port argument could be wrong.");
  } catch (CosNaming::NamingContext::InvalidName& in) {
    CDMW_ERROR_LOG_COLLECTOR("The name of the Log Collector is invalid: " << in);
    CDMW_THROW1(Cdmw::BadParameterException, 
                "the system_port argument could be wrong.");
  } catch (CosNaming::NamingContext::CannotProceed& cpe) {
    CDMW_ERROR_LOG_COLLECTOR("Unsuccessful rebind of "
                             << Cdmw::Common::Locations::DEFAULT_LOG_COLLECTOR_ID
                             << " context in " << M_daemon_naming_context_url
                             << " : " << cpe);
    throw cpe;
  }
  delete ni;
}

void InitUtilsLogCollector::Unregister_log_collector (CORBA::ORB_ptr orb)
throw (CORBA::SystemException)
{
  CDMW_DEBUG_LOG_COLLECTOR("");
  Cdmw::CommonSvcs::Naming::NamingInterface* ni = Get_naming_interface_from_daemon(orb);
  // Unregister the Log Collector from the CDMW daemon
  if (ni != NULL) {
    ni->unbind(Cdmw::Common::Locations::DEFAULT_LOG_COLLECTOR_ID);
    delete ni;
  }
}

} // End namespace Trace
} // End namespace Cdmw
