/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

#include <Foundation/ossupport/OS.hpp>
#include <Foundation/commonsvcs/time/Util.hpp>
#include <Foundation/orbsupport/ExceptionMinorCodes.hpp>

#include <TraceAndPerf/tracelibrary/FlushAreaMngr.hpp>
#include <Time/clockservice/Util.hpp>

#include "tracecollector/LocalCollectorMngr.hpp"
#include "tracecollector/LocalTraceCollector_impl.hpp"
#include "tracecollector/InitUtils.hpp"
#include <Foundation/common/Locations.hpp>
#include "tracecollector/CdmwTraceLoggingCollector.stub.hpp"

namespace Cdmw
{
    namespace Trace
    {
/**
 * Constructor.
 */
        LocalTraceCollector_impl::LocalTraceCollector_impl (CollectorMngr* p_collector_mngr,
                                                            Logging::FilterMngr* p_filter_mngr,
                                                            PortableServer::POA_ptr poa,
                                                            CORBA::ORB_ptr orb)
            throw()
            : TraceCollector_impl (p_collector_mngr,
                                   p_filter_mngr),
              m_poa (PortableServer::POA::_duplicate(poa)),
              m_orb (CORBA::ORB::_duplicate(orb))
        {
            m_miop_collectors = CdmwTrace::Collector::_nil();
            m_codec = 0;
        }


/**
 * Destructor.
 */
        LocalTraceCollector_impl::~LocalTraceCollector_impl()
            throw()
        {
            if (m_codec != 0) {
                delete m_codec;
                m_codec = 0;
            }
        }

      
        void LocalTraceCollector_impl::add_operational_time(CdmwTrace::MessageHeader &message_header)
            throw(CORBA::SystemException)
        {
            // First sets a value of zero to the operational time
            message_header.the_operational_time.seconds = 0L;
            message_header.the_operational_time.microseconds = 0L;            

            // Then checks whether an operational clock was passed as a command line argument.
            // If so, we get the operational clock from the local collector and we use it to 
            // retrieve the operational time...
            Cdmw::clock::ControlledClockEx_var clock_var = Cdmw::clock::ControlledClockEx::_nil();
            try
            {
                clock_var = m_collector_mngr->get_operational_clock();
            }
            catch (const CORBA::NO_IMPLEMENT &)
            {
                CDMW_ERROR(LocalCollectorMngr::Get_Logger(), "get_operational_clock() raised CORBA::NO_IMPLEMENT");
                throw CORBA::INTERNAL(OrbSupport::INTERNAL, CORBA::COMPLETED_NO);    
            }
          
            if (!CORBA::is_nil(clock_var))
            {
                try
                {
                    // First converts time_stamping values to TimeT
                    Cdmw::OsSupport::OS::Timeval time_val;
                    time_val.seconds = message_header.time_stamping.seconds;
                    time_val.microseconds = message_header.time_stamping.microseconds;
                    CDMW_DEBUG(LocalCollectorMngr::Get_Logger(), "time_stamping.seconds=" << time_val.seconds);
                    CDMW_DEBUG(LocalCollectorMngr::Get_Logger(), "time_stamping.microseconds=" << time_val.microseconds);

                    // then, retrieves the operational time
                    TimeBase::TimeT trace_time = Cdmw::CommonSvcs::Time::Util::Timeval_2_TimeT(time_val);
                    TimeBase::TimeT operational_time = Cdmw::clock::util::real2controlled(clock_var, trace_time);
                  
                    // Finally, converts TimeT back to Timeval...
                    time_val = Cdmw::CommonSvcs::Time::Util::TimeT_2_Timeval(operational_time);
                    // and copies the expected data into the parameter
                    message_header.the_operational_time.seconds = time_val.seconds;
                    message_header.the_operational_time.microseconds = time_val.microseconds;
                    CDMW_DEBUG(LocalCollectorMngr::Get_Logger(), "the_operational_time.seconds="
                               << message_header.the_operational_time.seconds);
                    CDMW_DEBUG(LocalCollectorMngr::Get_Logger(), "the_operational_time.microseconds="
                               << message_header.the_operational_time.microseconds);
                }
                catch (const CosClockService::TimeUnavailable &)
                {
                    CDMW_WARN(LocalCollectorMngr::Get_Logger(), "OperationalTime is unavailable, traces will not contain operational time");
                }
            }

        }

        PortableServer::POA_ptr LocalTraceCollector_impl::_default_POA()
        {
            return PortableServer::POA::_duplicate(m_poa.in());
        }
/**
 * Write messages.
 */
        void
        LocalTraceCollector_impl::put_formatted_messages(
            const CdmwTrace::MessagesHeader& the_messages_header,
            const CdmwTrace::FormattedMessageSeq& the_messages)
            throw(CORBA::SystemException)
        {
            try 
            {
                unsigned int inx_transf_message = 0;
                // filter the messages to send to the global trace collector
                CdmwTrace::FormattedMessageSeq_var messages_to_transfer =
                    new CdmwTrace::FormattedMessageSeq(the_messages.length());
             
                CdmwTrace::FormattedMessage fmt_message_copy;
                for (size_t i_msg = 0; i_msg < the_messages.length(); ++i_msg) 
                {
                    // memset(&fmt_message_copy, 0, sizeof(fmt_message_copy));
                    fmt_message_copy = the_messages[i_msg];
                    // Retrieves and adds the operational time
                    add_operational_time(fmt_message_copy.header);
                
                    // writes the message locally.
                    m_collector_mngr->write_message (the_messages_header, fmt_message_copy);

                    // write out in the sequence the message only if it passes at least one filter
                    if (CollectorMngr::Is_to_be_traced(fmt_message_copy.header.the_level.the_component_name.in(),
                                                       fmt_message_copy.header.the_level.the_domain.in(),
                                                       fmt_message_copy.header.the_level.the_value) )
                    {
                        inx_transf_message++;
                        messages_to_transfer->length(inx_transf_message);
                        messages_to_transfer[inx_transf_message-1] = fmt_message_copy;
                    }
                }
        
                // transfer messages if any
                if (messages_to_transfer->length() > 0)
                {
                    if ( CORBA::is_nil(m_miop_collectors.in()) )
                        //if ( m_miop_collectors.in() == CdmwTrace::Collector::_nil() )
                    {
                        // constructs a message header
                        CdmwLogging::LogHeader header;
                        header.file_line = __LINE__;
                        header.file_name = __FILE__;
                        header.level = Cdmw::Logging::INFO;
                        header.user_domain = "LOCALCOLLECTOR";
                        header.component_name = "";
                        header.thread_id = OsSupport::Thread::self();
                        OsSupport::OS::Timeval time = OsSupport::OS::get_time();
                        header.time_stamping.seconds = time.seconds;
                        header.time_stamping.microseconds = time.microseconds;

                        CdmwTrace::MessagesWithHeader_var collectorMsg =
                            new CdmwTrace::MessagesWithHeader();
                        collectorMsg->header = the_messages_header;
                        collectorMsg->message_sequence = messages_to_transfer;
                        // encode data
                        CORBA::OctetSeq_var encodedData = m_codec->encode(collectorMsg.in());

                        // add serialised message in the buffer
                        FlushAreaMngr& flushAreaMngr = FlushAreaMngr::instance();
                        if (! flushAreaMngr.add_serialized_message (header, encodedData.in()) )
                        {
                            std::cerr <<
                                "Warning: message was not added in LocalTraceCollector buffer"
                                      << std::endl;
                        }

                        /* For debug
                           std::cout << "Local Trace collector, pid is:" << FlushAreaMngr::instance().getProcessId() << std::endl;
                           std::cout << "LocalCollector put_formatted_messages, size:" << encodedData->length() << " seq length was:" << messages_to_transfer->length() << std::endl;
                        */

                    }
                    else
                    {

                        //std::cout << "Send MIOP put_formatted_messages, seq length is:" << messages_to_transfer->length() << std::endl;
                        try {
                            // It is not mandatory to use bufferization for miop
                            // as it is "acceptable" to loose messages in miop
                            // advantage is to not transfer this part and parameters in tracelibrary
                            m_miop_collectors.in()->put_formatted_messages(
                                the_messages_header,
                                messages_to_transfer.in());
                        }
                        catch (const CORBA::Exception& ex)
                        {
                            std::cerr << "Exception raised in Local Collector put_formated_message "
                                      << Cdmw::OrbSupport::OrbSupport::exception_to_string(ex) << std::endl;
                            throw;
                        }

                    } 
                }
            }
            catch(const OutOfMemoryException&) 
            {
                throw CORBA::NO_RESOURCES(OrbSupport::NO_RESOURCES,
                                          CORBA::COMPLETED_NO);
            }
        }

/**
 * Write messages.
 *
 * @param the_logs sequence of logs
 *
 * @exception CORBA::SystemException.
 */
        void
        LocalTraceCollector_impl::add_log_sequence(const CdmwLogging::StructuredLogSeq& the_logs)
            throw(CORBA::SystemException)
        {
            int sequence_size = 0;

            sequence_size = the_logs.length();

            CdmwTrace::MessagesHeader the_messages_header;
            CdmwTrace::FormattedMessageSeq the_messages;
            the_messages.length(1);
  
            for (int i = 0; i < sequence_size; i++) {

              convert (the_logs[i],
                       the_messages_header,
                       the_messages[0]);

              put_formatted_messages (the_messages_header,
                                      the_messages);

            }

        }

/**
 * Convert StructuredLog into MessagesHeader + FormattedMessageSeq
 */
        void LocalTraceCollector_impl::convert (const CdmwLogging::StructuredLog& log, CdmwTrace::MessagesHeader& header,
                                                CdmwTrace::FormattedMessage& message)
            throw(){
            header.the_process_id = log.ctx.process_id;
            header.the_application_name = CORBA::string_dup(log.ctx.application_name.in());
            header.the_process_name = CORBA::string_dup(log.ctx.process_name.in());
            header.the_host_id = 0;
            header.the_host_name = CORBA::string_dup(log.ctx.host_name.in());

            message.header.the_level.the_component_name =
                   CORBA::string_dup(log.header.component_name.in());
            message.header.the_level.the_domain = CORBA::string_dup(log.header.user_domain.in());
            message.header.the_level.the_value = log.header.level;

            // activation attribute doesn't exist in StructuredLog
            // We don't activate the filter
            message.header.the_level.activation = false;
            message.header.the_thread_id = log.header.thread_id;
            message.header.the_file_line = log.header.file_line;
            message.header.the_file_name =
                   CORBA::string_dup(log.header.file_name.in());
            message.header.time_stamping = log.header.time_stamping;
            message.message =
                   CORBA::string_dup(log.message.in());
            message.ndc = CORBA::string_dup(log.ctx.ndc.in());
        }

/**
 * Set the global trace collectors.
 */
        void
        LocalTraceCollector_impl::set_global_collectors (
            CollectorDataList& collector_list,
            CORBA::ORB_ptr orb
            )
        {
            m_global_collectors = collector_list;
            m_codec = new Cdmw::OrbSupport::Codec<CdmwTrace::MessagesWithHeader> (orb);
        }

/**
 * Set the global trace collectors miop reference
 */
        void
        LocalTraceCollector_impl::set_miop_collectors (CdmwTrace::Collector_ptr miop_collectors)
        {
            m_miop_collectors = CdmwTrace::Collector::_duplicate(miop_collectors);
        }


    } // namespace Trace
} // namespace Cdmw

