/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#include <cstring>

#include "tracecollector/LogCircularBuffer.hpp"
#include <Foundation/osthreads/MutexGuard.hpp>
#include "tracecollector/LogCollectorMacroDefs.hpp"

namespace Cdmw
{
namespace Trace
{

const static int REALLOC_SIZE = 100; 

//
// Constructor
//
  LogCircularBuffer::LogCircularBuffer (size_t bufferCircularSize,
                                        size_t period, 
                                        size_t maxLogsToSent,
                                        size_t chunkSize,
                                        BufferSender*& bufferSender)
        throw(Cdmw::BadParameterException)
      : CircularBuffer (),
        m_control(getLock())
{
    CDMW_DEBUG_LOG_COLLECTOR("LogCircularBuffer::LogCircularBuffer...");

    // create the circular buffer
    create (bufferCircularSize);

    m_buffer_sender = new BufferSender(this, period, maxLogsToSent,
                                       chunkSize, &m_control, getLock());
    bufferSender = m_buffer_sender; 

    if (bufferCircularSize < chunkSize) {
      m_max_log_size = bufferCircularSize;
    } else {
      m_max_log_size = chunkSize;
    }
}


//
// Destructor
//
LogCircularBuffer::~LogCircularBuffer ()
        throw()
{
  delete m_buffer_sender;
  m_buffer_sender = 0;
}

//
// Realloc string buffer cache
//
bool LogCircularBuffer::realloc_buffer_cache (char *& buffer_cache,
                                                    size_t new_size)
        throw()
{
    try
    {
      // if invalid string buffer size
      if (new_size == 0 )
      {
        return false;           // ----> return if invalid size
      }

      if (buffer_cache != 0)
      {
        delete[] buffer_cache;
        buffer_cache = 0x0;
      }

      // create new string buffers
      buffer_cache = new char[new_size];

    }
    catch (const std::bad_alloc &)
    {
      CDMW_WARN_LOG_COLLECTOR ("Out of memory");
      return false;
    }

    return true;
}




//
// Return the next log to read
//
bool LogCircularBuffer::getNextLog (CdmwLogging::StructuredLog& log,
                                    size_t& log_size,
                                    size_t allowed_log_size)
        throw(Cdmw::OutOfMemoryException)
{
  bool success = false;
  
   try
    {

      // read message from circular buffer
      success = readLog (log, log_size, allowed_log_size);

      CDMW_DEBUG_LOG_COLLECTOR("log size=" << log_size
                               << " allowed_log_size="
                               << allowed_log_size);

    }

    catch (Cdmw::OutOfMemoryException& ex)
    {
      CDMW_ERROR_LOG_COLLECTOR(ex);
      success = false;
    }

    // catch any exception from Mutex
    catch (...)
    {
      CDMW_ERROR_LOG_COLLECTOR("Unexpected exception raised!");
      return false;
    }

    return success;
}

//
// Read message from circular buffer
//
// as method uses local trace message string buffer,
// this method is not designed to be called by more than one thread
//
bool LogCircularBuffer::readLog (CdmwLogging::StructuredLog& log,
                                 size_t& returnedLogSize,
                                 size_t allowedLogsize)
        throw (Cdmw::OutOfMemoryException)
{


    // read the length of the read frame 
    size_t count = 0;

    {
      CDMW_MUTEX_GUARD (getLock());
      
      if (getReadCount(count) == false
          || count < sizeof (struct BufferLogMessage)
          || count > allowedLogsize)
        {
          CDMW_DEBUG_LOG_COLLECTOR("count=" << count  
                                   << " ;sizeof BufferLogMessage ="
                                   << sizeof (struct BufferLogMessage));
          returnedLogSize = 0;
          
          // there is no header to read or the Buffer Sender has not enough
          // space in its sequence to send it.
          return false;
        }
    }

    // Size of string read buffer.
    // The default value is set to 100 bytes but this size can be
    // recompute if necessary.
    size_t  stringReadBufSz = 100;

    // pointer on the buffer cache used to read from the circular buffer
    char*   pStringReadBuf;   

    try
      {
        pStringReadBuf = new char[stringReadBufSz];
      }
    catch (const std::bad_alloc &)
    {
      CDMW_DEBUG_LOG_COLLECTOR("Out of memory");
      return false;
    }

    // flag to indicate message has been read
    bool messageIsRead = false;

    // then read trace header
    BufferLogMessage mesgFromBuffer;
      
    // read trace header message from circular buffer
    {
      CDMW_MUTEX_GUARD (getLock());
      // first get the initial read position
      size_t initial_read_pos = getReadPos ();
      
      bool success = read (&mesgFromBuffer,sizeof (struct BufferLogMessage));

      // from header get size of string buffer and read it  
      if (success)
        {
          // check there is enough space for string else realloc
          if (mesgFromBuffer.m_messageSize > stringReadBufSz)
            {
              bool status__;
              status__ = realloc_buffer_cache(pStringReadBuf,
                                              mesgFromBuffer.m_messageSize + REALLOC_SIZE);
              if (!status__) {
                return false;
              }
            }

          // read string buffer
          success = read (pStringReadBuf,mesgFromBuffer.m_messageSize);
    
          if (success)
            {
              CDMW_DEBUG_LOG_COLLECTOR("reading in the circular buffer successful");
              // read is successful
              messageIsRead = true;        
            }            
          else
            {
              CDMW_DEBUG_LOG_COLLECTOR("reading in the circular buffer failed");
              // else read is not successfull restore the initial read position
              // to restore the trace header
              setReadPos (initial_read_pos);
            }
        } else {
        CDMW_DEBUG_LOG_COLLECTOR("reading of the log header failed");
      }
    } // The lock can be freed

    // if message has been read
    if (messageIsRead)
    {
      // copy data in the trace message raw
      returnedLogSize = mesgFromBuffer.m_messageSize;
      log.ctx.process_id = mesgFromBuffer.m_pid;
      log.header.level = mesgFromBuffer.m_level;
      log.header.thread_id = mesgFromBuffer.m_tid;
      log.header.time_stamping.seconds = mesgFromBuffer.m_timestamping.seconds;
      log.header.time_stamping.microseconds = mesgFromBuffer.m_timestamping.microseconds;
      log.header.file_line = mesgFromBuffer.m_fileLine;

      // initialize buffer pointer
      char *pBuf = pStringReadBuf; 

      // set host name
      log.ctx.host_name = CORBA::string_dup(pBuf);
      CDMW_DEBUG_LOG_COLLECTOR(log.ctx.host_name.in());
      pBuf += mesgFromBuffer.m_szHostName;

      // set application name
      log.ctx.application_name = CORBA::string_dup(pBuf); 
      pBuf += mesgFromBuffer.m_szApplicationName;
      
      // set process name
      log.ctx.process_name = CORBA::string_dup(pBuf);
      pBuf += mesgFromBuffer.m_szProcessName;

      // set Nested Diagnostic Control
      //CCR71
      log.ctx.ndc.inout() = CORBA::string_dup(pBuf);
      pBuf += mesgFromBuffer.m_szNdc;

      // set the component name
      // ECR-0123
      log.header.component_name = CORBA::string_dup(pBuf);
      pBuf += mesgFromBuffer.m_szComponentName;

       // set user domain
       //CCR71
      log.header.user_domain.inout() = CORBA::string_dup(pBuf);
      pBuf += mesgFromBuffer.m_szUserDomain;

      // set file name
      log.header.file_name = CORBA::string_dup(pBuf);
      pBuf += mesgFromBuffer.m_szFileName;

      // set message body
      log.message = CORBA::string_dup(pBuf);
    }

    if (pStringReadBuf != 0)
      {
        delete[] pStringReadBuf;
        pStringReadBuf = 0x0;
      }

    return messageIsRead;
}

//
// Flush message from circular buffer
//
bool LogCircularBuffer::removeOlderLog (size_t size_new_log)
        throw ()
{
   // flush message header buffer
   struct BufferLogMessage flushMessage;

   // read the length of the read frame 
   size_t count = 0;

   if (getReadCount(count) == false || count < sizeof (struct BufferLogMessage))
    {
      // there is no header to flush or the circular buffer was already
      // freed by an other thread
      return false;          
    }

    // read trace header message from circular buffer
    
    // first get the initial read position
    size_t initial_read_pos = getReadPos ();
              
    // then read trace header
    bool success = read (&flushMessage,sizeof (struct BufferLogMessage));

    // from header get size of string buffer and flush it  
    if (success)
    {
      success = flush (flushMessage.m_messageSize);

      // if read is not successfull restore the initial read position
      // to restore the trace header
      if (success == false)
      {
        setReadPos (initial_read_pos);
      }
    }
     
    return success;
}



//
// Write message into circular buffer
//
bool LogCircularBuffer::writeLog (const CdmwLogging::StructuredLog& log)
  throw()    
{
    CDMW_DEBUG_LOG_COLLECTOR("");
    bool success = false;

    // write message header buffer
    struct BufferLogMessage writeMessage;

    // size of string write buffer.
    // Default value of buffer equals to 10000 bytes
    size_t  stringWriteBufSz = 10;

    // pointer on the buffer cache used to write in the circular buffer
    char*   pStringWriteBuf = 0x0;

    try
      {
        pStringWriteBuf = new char[stringWriteBufSz];
      }
    catch (const std::bad_alloc &)
    {
      CDMW_DEBUG_LOG_COLLECTOR("Out of memory");
      return false;
    }

    try
    {
      //
      // transfer the log in a local structure
      //
      
      // first get the total size for string
      writeMessage.m_szHostName = strlen(log.ctx.host_name.in()) + 1;
      writeMessage.m_messageSize = writeMessage.m_szHostName;

      writeMessage.m_szApplicationName = strlen(log.ctx.application_name.in()) + 1;
      writeMessage.m_messageSize += writeMessage.m_szApplicationName;
 
      writeMessage.m_szProcessName = strlen(log.ctx.process_name.in()) + 1;
      writeMessage.m_messageSize += writeMessage.m_szProcessName;

      writeMessage.m_szNdc = strlen(log.ctx.ndc.in()) + 1;
      writeMessage.m_messageSize += writeMessage.m_szNdc;

      writeMessage.m_szComponentName = strlen(log.header.component_name.in()) + 1;
      writeMessage.m_messageSize += writeMessage.m_szComponentName;
     
      writeMessage.m_szUserDomain = strlen(log.header.user_domain.in()) + 1;
      writeMessage.m_messageSize += writeMessage.m_szUserDomain;

      writeMessage.m_szFileName = strlen(log.header.file_name.in()) + 1;
      writeMessage.m_messageSize += writeMessage.m_szFileName;

      writeMessage.m_bodyMessageSize = strlen(log.message.in()) + 1;
      writeMessage.m_messageSize += writeMessage.m_bodyMessageSize;

      // Log too big
      bool is_too_big_log = false;

      if (writeMessage.m_messageSize > m_max_log_size) {
        is_too_big_log = true;
      }

      //
      // The log is too big. Try to truncate the message of the log.
      //
      if (is_too_big_log) {
        if (writeMessage.m_messageSize - writeMessage.m_bodyMessageSize
            < m_max_log_size) {
          // Reset the length of the log message
          writeMessage.m_messageSize -= writeMessage.m_bodyMessageSize;

          // Compute the new length of the log message. Even if the log is big,
          // don't fill whole circular buffer, forsee enough space in the circular 
          // buffer for the next logs and don't delete too many older messages.
          // This explains the division per 5.
          writeMessage.m_bodyMessageSize = (size_t) /*explicit cast*/
            (m_max_log_size - writeMessage.m_messageSize) / 5;

          // The message of the log should be truncated
          writeMessage.m_messageSize += writeMessage.m_bodyMessageSize;
        } else {
          std::string warn_msg = "Even without take into account the length";
          warn_msg += " of the log message, the length of whole log";
          warn_msg += " stays too big. The Log Collector";
          warn_msg += " doesn't store this log.";

          CDMW_WARN_LOG_COLLECTOR(warn_msg);

          if (pStringWriteBuf != 0x0)
            {
              delete[] pStringWriteBuf;
              pStringWriteBuf = 0x0;
            }         
          return false;
        }
      }

      // check there is enough space for string else realloc
      if (writeMessage.m_messageSize > stringWriteBufSz)
      {
        stringWriteBufSz = writeMessage.m_messageSize + REALLOC_SIZE;
        bool status__ = realloc_buffer_cache (pStringWriteBuf, stringWriteBufSz);
        if (!status__) {
          return false; // Out of memory
        }
      }

      // initialize buffer pointer
      char *pBuf = pStringWriteBuf; 
      
      // set host name
      memcpy(pBuf, log.ctx.host_name.in(), writeMessage.m_szHostName);
      pBuf += writeMessage.m_szHostName;

      // set application name
      memcpy(pBuf, log.ctx.application_name.in(),  writeMessage.m_szApplicationName);
      pBuf += writeMessage.m_szApplicationName;
      
      // set process name
      memcpy(pBuf, log.ctx.process_name.in(), writeMessage.m_szProcessName);
      pBuf += writeMessage.m_szProcessName;

      // set Nested Diagnostic Control
      memcpy(pBuf, log.ctx.ndc.in(),writeMessage.m_szNdc); 
      pBuf += writeMessage.m_szNdc;

      // set the component name
      // ECR-0123
      memcpy(pBuf, log.header.component_name.in(), writeMessage.m_szComponentName);
      pBuf += writeMessage.m_szComponentName;

       // set user domain
      memcpy(pBuf, log.header.user_domain.in(), writeMessage.m_szUserDomain);
      pBuf += writeMessage.m_szUserDomain;

      // set file name
      memcpy(pBuf, log.header.file_name.in(), writeMessage.m_szFileName);
      pBuf += writeMessage.m_szFileName;

      // set message body
      memcpy(pBuf, log.message.in(), writeMessage.m_bodyMessageSize);
      
      // set all other fields
      writeMessage.m_pid = log.ctx.process_id;
      writeMessage.m_tid = log.header.thread_id;
      writeMessage.m_fileLine = log.header.file_line;
      writeMessage.m_level = log.header.level;
      writeMessage.m_timestamping.seconds = 
                                log.header.time_stamping.seconds;
      writeMessage.m_timestamping.microseconds = 
                                log.header.time_stamping.microseconds;



      // get total message length
      size_t message_length = sizeof (struct BufferLogMessage) + writeMessage.m_messageSize;

      // check if there enough space in circular buffer
      size_t count = 0;
      bool bufferFull = true;
      // At the end of writeLog(), a signal can be sent to BufferSender
      // to indicate that the buffer is almost full
      bool signalToSend = false;
      CDMW_MUTEX_GUARD (getLock());

      // while the buffer is full
      while (bufferFull)
      {
        // if circular buffer is full
        if (getWriteCount(count) == false || count < message_length)
        {
          if (signalToSend == false) {
            // Indicate to BufferSender that the circular buffer is almost full
            signalToSend = true;
          }
          // Delete the older log
          bool flush_err = removeOlderLog(message_length);
          if (flush_err == false)
            {
              break;
            }
        } else {
          // Enough space in the buffer. Then no need to free older logs
          bufferFull = false;
        }
      }
   
      // first get the initial write position
      size_t initial_write_pos = getWritePos ();
      
      // then write trace header
      success = write (&writeMessage,sizeof (struct BufferLogMessage));
      
      // if successfull
      if (success)
        {
          //
          // write string buffer
          //
          success = write (pStringWriteBuf,writeMessage.m_messageSize);
          
          // if write is not successfull restore the initial write position
          // to drop the trace header
          if (success == false)
              {
                CDMW_DEBUG_LOG_COLLECTOR("writing in the circular buffer failed");
                setWritePos (initial_write_pos);
              } else {
            CDMW_DEBUG_LOG_COLLECTOR("writing in the circular buffer successful");
            // The log was too big
            if (is_too_big_log) {
              // The writing in the circular buffer is successful and the log message
              // is truncated
                CDMW_WARN_LOG_COLLECTOR("The message of the log is truncated.");
            }
            // The circular buffer is almost full. Ask to the bufferSender
            // to free the circular buffer
            if (signalToSend)  m_control.broadcast();
          }
        }
      
    }
    catch (Cdmw::OutOfMemoryException& ex)
    {
      if (pStringWriteBuf != 0x0)
        {
          delete[] pStringWriteBuf;
          pStringWriteBuf = 0x0;
        }
      return false;
    }
    // catch any exception from Mutex
    catch (...)
    {
      if (pStringWriteBuf != 0x0)
        {
          delete[] pStringWriteBuf;
          pStringWriteBuf = 0x0;
        }
      CDMW_ERROR_LOG_COLLECTOR("Unexpected exception raised!");
      return false;
    }

    if (pStringWriteBuf != 0x0)
    {
       delete[] pStringWriteBuf;
       pStringWriteBuf = 0x0;
    }
    return success;
}

}; // End namespace Trace
}; // End namespace Cdmw


