
/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

#include "tracecollector/LogCollectorPlugin.hpp"
#include "tracecollector/InitUtilsLogCollector.hpp"
#include "tracecollector/LogCollectorMacroDefs.hpp"
#include <Foundation/orbsupport/OrbSupport.hpp>
#include <Foundation/orbsupport/StrategyList.hpp>
#include <Foundation/ossupport/SystemProperties.hpp> 
#include <Foundation/common/String.hpp>
#include <Foundation/common/Options.hpp>
#include <sstream>
#include <ios>
#include <iomanip>
#include <cstdlib>
#include <vector>
// Anonymous namespace
namespace {
  // Default values
  const std::size_t DEFAULT_THREAD_POOL_SIZE_nb = 10;
  /**
   * - period the period P. The logs will be sent to the Trace Collector 
   * each P milliseconds.
   *
   * - circular_buffer_size the size of the circular buffer.
   *
   * - max_logs_to_send the maximum number of logs to
   * send at each period. This parameter should avoid to lock 
   * the access to the circular buffer during a too long time.
   *
   * - chunk_size the maximum size in bytes of a sequence
   * to send to the Local Trace Collector. This parameter
   * should avoid to flood the ORB buffers during a sending 
   * to the Local Trace Collector.
   *
   */
  const std::size_t DEFAULT_PERIOD_ms = 500;
  const std::size_t DEFAULT_CIRCULAR_BUFFER_SIZE_bytes = 51200; // 50Kb?
  const std::size_t DEFAULT_MAX_LOGS_TO_SEND_nb = 100;
  const std::size_t DEFAULT_CHUNCK_SIZE_bytes = 51200;
  
  const std::size_t MAX_PORT_NUMBER = 65535;
  
  // Command line options for using values other than the default ones
  const std::string THREADPOOL_SIZE_OPTION      = "--threadpool-size"; 
  const std::string PERIOD_OPTION               = "--period";
  const std::string CIRCULAR_BUFFER_SIZE_OPTION = "--circular-buffer-size";
  const std::string MAX_LOGS_TO_SEND_OPTION     = "--max-logs-to-send";
  const std::string CHUNK_SIZE_OPTION           = "--chunk-size";
  const std::string SYSTEM_PORT_OPTION          = Cdmw::Common::Options::SYSTEM_PORT_OPTION;
  const std::string SHOW_DATA_OPTION            = "--show-data";
  const std::string SHOW_USAGE_OPTION           = "--show-usage"; 
  
  
  std::size_t get_option_value(int argc, char* argv[], const std::string & opt_name, 
                       std::size_t default_value,
                       std::size_t max_value = 0)
  {
    std::size_t result = default_value;
    try {
      std::string option_value = Cdmw::OsSupport::OS::get_option_value (argc, argv, opt_name);
      if (!option_value.empty()) {
        int number = atoi(option_value.c_str());
        if (number > 0) {
          std::size_t nb__ = static_cast<std::size_t>(number);
          if ( (max_value == 0) ||
               ((max_value > 0) && (nb__ < max_value)))
            result = nb__;
        }
      }
    } catch (...) {
      // ignore! return default value
    }
    return result;
  } 

  void print_option(std::ostream& os, 
                    const std::string & option,
                    const char * value,
                    const char*  description,
                    std::size_t  default_value = 0)
  {
    std::string opt(option);
    opt += "=";
    opt += value;
    os << "\t" << std::setw(45) << std::left << opt << ": " << description;
    if (default_value != 0) 
      os << "Default is " << default_value << ".";
    os << "\n";
  }
  void print_usage()
  {
    std::ostringstream ostr;
    ostr 
      << "\nLogCollector Usage: [OPTION] ...\n"
      << "CARDAMOM Log Collector Plugin.\n\n"
      << "Available options:\n";
    print_option(ostr,THREADPOOL_SIZE_OPTION,"<num threads>","Size of the thread pool for servicing log requests", DEFAULT_THREAD_POOL_SIZE_nb);
    print_option(ostr,PERIOD_OPTION,"<milliseconds> ","The logs will be sent to the Trace Collector each P milliseconds. ",DEFAULT_PERIOD_ms);
    print_option(ostr,CIRCULAR_BUFFER_SIZE_OPTION,"<bytes>","The size of the circular buffer",DEFAULT_CIRCULAR_BUFFER_SIZE_bytes);
    print_option(ostr,MAX_LOGS_TO_SEND_OPTION,"<number>",
                 "The maximum number of logs to send at each period. This parameter should avoid to lock the access to the circular buffer during a too long time",DEFAULT_MAX_LOGS_TO_SEND_nb);
    print_option(ostr,CHUNK_SIZE_OPTION,"<bytes>","The maximum size in bytes of a sequence to send to the Local Trace Collector. This parameter should avoid to flood the ORB buffers during a sending to the Local Trace Collector ",DEFAULT_CHUNCK_SIZE_bytes);
    print_option(ostr,SYSTEM_PORT_OPTION,"<port number>","System Port number. ");
    print_option(ostr,SHOW_DATA_OPTION,"","Print initialisation data");
    print_option(ostr,SHOW_USAGE_OPTION,"","Print this message");
    std::cout << ostr.str() << std::endl;
  }


}; // End of anonymous namespace
namespace Cdmw
{

namespace Trace
{
   LogCollectorPlugin::LogCollectorPlugin()
      : m_orb(CORBA::ORB::_nil()),
        m_rootPOA(PortableServer::POA::_nil()),
        m_orb_threadpool_size(DEFAULT_THREAD_POOL_SIZE_nb),
        m_period(DEFAULT_PERIOD_ms),
        m_circular_buffer_size(DEFAULT_CIRCULAR_BUFFER_SIZE_bytes),
        m_max_logs_to_send(DEFAULT_MAX_LOGS_TO_SEND_nb),
        m_chunk_size(DEFAULT_CHUNCK_SIZE_bytes)
   {
      CDMW_DEBUG_LOG_COLLECTOR(__FUNCTION__); 
   }

   LogCollectorPlugin::~LogCollectorPlugin() throw()
   {
      CDMW_DEBUG_LOG_COLLECTOR(__FUNCTION__);
   }
    
   void LogCollectorPlugin::set_up()
   {
      CDMW_DEBUG_LOG_COLLECTOR(__FUNCTION__);
      try
      {
         // Get args?
         std::string name__ = this->get_name();
         if (name__.empty())
            name__="LogCollectorPlugin";
         std::ostringstream ostr;
         ostr << name__ << " " << this->get_args();
         std::vector< std::string > vect__ =
            Cdmw::Common::String::to_strings(ostr.str());
         char** argv__ = Cdmw::Common::String::to_char_array(vect__);
         int argc__(vect__.size());

         // Get option values
         bool show_init = false;
         // SHOW_DATA_OPTION
         {
           std::string option_value = Cdmw::OsSupport::OS::get_option_value (argc__, argv__, SHOW_DATA_OPTION);
           if (option_value == "yes")
             show_init = true;
         }
         bool show_usage = false;
         // SHOW_USAGE_OPTION
         {
           std::string option_value = Cdmw::OsSupport::OS::get_option_value (argc__, argv__, SHOW_USAGE_OPTION);
           if (option_value == "yes")
             show_usage = true;
         }
         // THREADPOOL_SIZE_OPTION
         m_orb_threadpool_size = get_option_value(argc__, argv__, 
                                                  THREADPOOL_SIZE_OPTION,
                                                  DEFAULT_THREAD_POOL_SIZE_nb);
         // PERIOD_OPTION
         m_period = get_option_value(argc__, argv__, 
                                     PERIOD_OPTION, 
                                     DEFAULT_PERIOD_ms);
         // CIRCULAR_BUFFER_SIZE_OPTION
         m_circular_buffer_size = get_option_value(argc__, argv__, 
                                                   CIRCULAR_BUFFER_SIZE_OPTION, 
                                                   DEFAULT_CIRCULAR_BUFFER_SIZE_bytes);
         // MAX_LOGS_TO_SEND_OPTION
         m_max_logs_to_send = get_option_value(argc__, argv__, 
                                               MAX_LOGS_TO_SEND_OPTION, 
                                               DEFAULT_MAX_LOGS_TO_SEND_nb);
         // CHUNK_SIZE_OPTION
         m_chunk_size = get_option_value(argc__, argv__, 
                                         CHUNK_SIZE_OPTION, 
                                         DEFAULT_CHUNCK_SIZE_bytes);

         Cdmw::OrbSupport::StrategyList strategyList;
         strategyList.add_OrbThreaded();
         strategyList.add_PoaThreadPool( m_orb_threadpool_size );

         
         std::string orb_id("ORB_");
         orb_id += name__;
         m_orb = Cdmw::OrbSupport::OrbSupport::ORB_init(argc__, argv__, 
                                                        strategyList,
                                                        orb_id.c_str());
         std::string system_port = this->get_system_port();
         // Get System port
         {
           std::size_t sp = get_option_value(argc__, argv__, 
                                             SYSTEM_PORT_OPTION,
                                             0,
                                             MAX_PORT_NUMBER);
           if (sp > 0) {
             std::ostringstream ostr;
             ostr << sp;
             system_port = ostr.str();
           }
         }
         if (system_port.empty())
            system_port =
               Cdmw::OsSupport::SystemProperties::Get_Property(Cdmw::OrbSupport::SYSTEM_PORT_PROPERTY_NAME);
         // Show Usage 
         if (show_usage)
           print_usage();
         if (show_init) {
           std::cout << "LogCollector name='"<< name__<< "' args='" << this->get_args() << "' \n" 
                     << "\t using following values:\n"
                     << "\t orb_threadpool_size  ='" << m_orb_threadpool_size << "' ['" << DEFAULT_THREAD_POOL_SIZE_nb << "']\n"
                     << "\t period               ='" << m_period << "' ['" << PERIOD_OPTION << "']\n"
                     << "\t circular_buffer_size ='" << m_circular_buffer_size << "' ['" << DEFAULT_CIRCULAR_BUFFER_SIZE_bytes << "']\n"
                     << "\t max_logs_to_send     ='" << m_max_logs_to_send << "' ['" << DEFAULT_MAX_LOGS_TO_SEND_nb << "']\n"
                     << "\t chunk_size           ='" << m_chunk_size << "' ['" << DEFAULT_CHUNCK_SIZE_bytes << "']\n"
                     << "\t system_port          ='" << system_port << "' ['" << "']\n"
                     << std::endl;
         }
         CDMW_ASSERT(!system_port.empty() && " SystemPort missing for the LogCollector plugin!");

         CORBA::Object_var obj = m_orb->resolve_initial_references( "RootPOA" );
         m_rootPOA = PortableServer::POA::_narrow( obj.in() );

         // TODO: prefer a non-static operation instead!
         InitUtilsLogCollector::Create_log_collector ( m_orb.in(),
                                                       m_rootPOA.in(),
                                                       system_port,
                                                       m_period,
                                                       m_circular_buffer_size,
                                                       m_max_logs_to_send,
                                                       m_chunk_size );
         
         Cdmw::Common::String::delete_char_array(argv__);
      } catch (...) {
         CDMW_WARN_LOG_COLLECTOR(__FUNCTION__);
      }

   }
   void LogCollectorPlugin::run()
   {
      CDMW_DEBUG_LOG_COLLECTOR(__FUNCTION__);
      if (!CORBA::is_nil(m_orb.in())) {
         PortableServer::POAManager_var poaMgr = m_rootPOA->the_POAManager();
         poaMgr->activate();
         m_orb->run();
      }
   }
   void LogCollectorPlugin::tear_down()
   {
      CDMW_DEBUG_LOG_COLLECTOR(__FUNCTION__);
      if (!CORBA::is_nil(m_orb.in())) {
         try {
//            There is no need to unbind the log collector since the daemon is stopping when tear_down is called
//            InitUtilsLogCollector::Unregister_log_collector (m_orb.in());
            m_orb->shutdown(false);
            Cdmw::OrbSupport::OrbSupport::ORB_cleanup(m_orb.in());
            m_orb -> destroy();
         }
         catch(const CORBA::Exception& ex)
         {
            CDMW_ERROR_LOG_COLLECTOR("["<<__FILE__ << "]:[" <<__LINE__<<"] "
                                     << "Unexpected CORBA::Exception:\n"
                                     << Cdmw::OrbSupport::OrbSupport::exception_to_string(ex));
         }
      }
   }

} // End namespace Trace
} // End namespace Cdmw
