/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX Sistemi Integrati. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2007.
 * All rights reserved.
 *
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

#include <Foundation/common/Assert.hpp>
#include <Foundation/ossupport/OS.hpp>
#include <Foundation/logging/LogLevel.hpp>
#include <Foundation/logging/LogManager.hpp>
#include <Foundation/logging/LogManagerDelegate.hpp>

#include <SystemMngt/idllib/CdmwPlatformMngtApplicationAgent.stub.hpp>
#include <SystemMngt/idllib/CdmwPlatformMngtProcessCallback.stub.hpp>
#include <SystemMngt/idllib/CdmwPlatformMngtProcessDelegateWrapper.stub.hpp>
#include <SystemMngt/platformlibrary/EventHeaderFactory.hpp>

#include "tracecollector/LogCollector_impl.hpp"
#include "tracecollector/LogCircularBuffer.hpp"
#include "tracecollector/BufferSender.hpp"
#include "tracecollector/LogCollectorMacroDefs.hpp"

namespace
{
   /**
     * Default mask for forwarding log to supervision
     * Filter on FTL or ERR messages
     */
    const unsigned long FTL_ERR_MASK=0x00000003;
}

namespace Cdmw
{

namespace Trace
{


::CORBA::Boolean LogCollector_impl::add_log (const CdmwLogging::StructuredLog& log)
  throw (::CORBA::SystemException)
{
   CDMW_DEBUG_LOG_COLLECTOR("");

   try
   {
       // While the Local Trace Collector is not reachable, the new logs are
       // automatically redirected to the Default Log Manager Delegate
       // to keep a local trace of the new log (on screen or on file depending on
       // the configuration of the log manager)
       if(this->m_buffer_sender->get_LTRC_Status() != Cdmw::Trace::BufferSender::LTRC_REACHABLE) {
           Cdmw::Logging::LogManagerDelegate* p_defDelegate = 
               Cdmw::Logging::LogManager::Instance()->get_default_delegate();
           if ( p_defDelegate != 0x0 )
               p_defDelegate->add_log(log.ctx, log.header, log.message.in());
       }

      // Forward message to platform daemon if level observes the mask
      if(log.header.level & m_forwarding_mask)
      {
         if (!CORBA::is_nil(m_daemon.in()))
         {
            CORBA::String_var app_name = CORBA::string_dup(log.ctx.application_name.in());
             
            CdmwPlatformMngt::ApplicationAgent_var app_agent = 
               m_daemon->search_application_agent(app_name.in());
             
            if (!CORBA::is_nil(app_agent.in()))
            {
               CdmwPlatformMngt::ProcessDelegateWrapper_var process_delegate_wrapper =
                  app_agent->search_process_delegate_wrapper(log.ctx.process_name.in());
               if (!CORBA::is_nil(process_delegate_wrapper.in()))
               {
                  CdmwPlatformMngt::ProcessMessageBroker_var msg_broker = 
                     process_delegate_wrapper->get_process_message_broker();
                
                  if ( ! CORBA::is_nil(msg_broker.in()))
                  {
                     // Create header
                     // Create time stamp
                     Cdmw::OsSupport::OS::Timeval 
                        time_val(log.header.time_stamping.seconds,
                                 log.header.time_stamping.microseconds);
                     CdmwPlatformMngtBase::TimeStamp time_stamp = 
                        Cdmw::PlatformMngt::EventHeaderFactory::TimeVal_2_TimeStamp(
                           time_val);

                     CdmwPlatformMngtBase::EventHeader_var event_header;
                     std::stringstream msg_stream;
                     if(log.header.level & CdmwLogging::FATAL)
                     {
                        event_header =
                           Cdmw::PlatformMngt::EventHeaderFactory::createHeader(
                              time_stamp, 
                              CdmwPlatformMngtBase::FTL);
                     }
                     else if(log.header.level & CdmwLogging::ERROR)
                     {
                        event_header =
                           Cdmw::PlatformMngt::EventHeaderFactory::createHeader(
                              time_stamp, 
                              CdmwPlatformMngtBase::ERR);
                     }
                     else if(log.header.level & CdmwLogging::WARN)
                     {
                        event_header =
                           Cdmw::PlatformMngt::EventHeaderFactory::createHeader(
                              time_stamp, 
                              CdmwPlatformMngtBase::WRN);
                     }
                     else if(log.header.level & CdmwLogging::INFO)
                     {
                        event_header =
                           Cdmw::PlatformMngt::EventHeaderFactory::createHeader(
                              time_stamp, 
                              CdmwPlatformMngtBase::INF);
                     }
                     else
                     {
                        // Debug message can be too important in number
                        // and they provoke a block in the log
                        // so they are ignored !
/*
  event_header =
  Cdmw::PlatformMngt::EventHeaderFactory::createHeader(
  time_stamp, 
  CdmwPlatformMngtBase::INF);

  // No predefined level for debug information
  // Prefix each message with its log level in 
  // an INFO message
  msg_stream 
  << "[" 
  << Cdmw::Logging::LevelUtilities::Level_To_String(
  log.header.level) << "] ";
*/
                     }

                     // TODO: Define how to use process_name and component_name in message:
                     //       Both in issuer ? add component_name in message ?
                     //       Temporarly solution: issuer = process_name/component_name
                     std::stringstream issuer_stream;
                     issuer_stream << log.ctx.process_name.in();

                     if (log.header.component_name.in() != 0x0 
                         && strlen(log.header.component_name.in())>0)
                        issuer_stream << "/" << log.header.component_name.in();

                     msg_stream << log.message.in();
                     msg_broker->notify_message(event_header.in(),
                                                issuer_stream.str().c_str(),
                                                msg_stream.str().c_str());
                  }
               }
            }
         }
      }
   }
    catch(const CORBA::SystemException& se )
    {
       // after a call to m_daemon.search_application_agent
       // or after a call to app_agent->get_process_message_broker
       // or after a call to msg_broker->notify_message
       // Platform daemon is busy, do nothing
       std::cerr << "An error appeared !" << std::endl;
    }
    catch(...)
    {
       // after a call to m_daemon.search_application_agent
       // or after a call to app_agent->get_process_message_broker
       // or after a call to msg_broker->notify_message
       // Platform daemon is busy, do nothing
       std::cerr << "An error appeared !" << std::endl;
    }

    bool status = m_log_circular_buffer->writeLog(log);
    return status;
}

void LogCollector_impl::register_trace_collector (CdmwTrace::LocalTraceCollector_ptr ref_trace_collector)
  throw (::CORBA::SystemException)
{
    CDMW_DEBUG_LOG_COLLECTOR("");
    m_buffer_sender->register_trace_collector(ref_trace_collector);
}

void LogCollector_impl::unregister_trace_collector ()
  throw (::CORBA::SystemException)
{
    CDMW_DEBUG_LOG_COLLECTOR("");
    m_buffer_sender->unregister_trace_collector();
}

PortableServer::POA_ptr LogCollector_impl::_default_POA()
{
    CDMW_DEBUG_LOG_COLLECTOR("");
    return PortableServer::POA::_duplicate(m_poa);
}

void LogCollector_impl::init(size_t period, 
                             size_t circular_buffer_size,
                             size_t max_logs_to_send,
                             size_t chunk_size,
                             PortableServer::POA_ptr poa,
                             CdmwPlatformMngt::Daemon_ptr daemon)
  throw (OutOfMemoryException,
         BadParameterException)
{
    CDMW_DEBUG_LOG_COLLECTOR("");
    m_poa = PortableServer::POA::_duplicate(poa);
    m_daemon = CdmwPlatformMngt::Daemon::_duplicate(daemon);

    try {
      m_log_circular_buffer = new LogCircularBuffer(circular_buffer_size, period,
                                                    max_logs_to_send, chunk_size,
                                                    m_buffer_sender);
    } catch (OutOfMemoryException& ofm) {
      CDMW_ERROR_LOG_COLLECTOR("No enough memory to create the circular buffer of the Log Collector" << ofm);
      throw ofm;
    } catch (BadParameterException bpe) {
      CDMW_ERROR_LOG_COLLECTOR("A bad parameter was set to create the circular buffer of the Log Collecctor" << bpe);
      throw bpe;
    }
}

LogCollector_impl::LogCollector_impl()
   throw()
    : m_buffer_sender(0x0),
      m_log_circular_buffer(0x0),
      m_forwarding_mask(FTL_ERR_MASK)
{
    CDMW_DEBUG_LOG_COLLECTOR("");
}

LogCollector_impl::~LogCollector_impl()
  throw ()
{
    CDMW_DEBUG_LOG_COLLECTOR("");
    if(m_log_circular_buffer != 0x0)
       delete m_log_circular_buffer;
    m_log_circular_buffer = 0x0;

    // Buffer Sender deleted by its owner : LogCircularBuffer
    m_buffer_sender = 0x0;
}

} // End namespace Trace
} // End namespace Cdmw
