/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

#include <sstream>
#include <iomanip>

#include <Foundation/common/Assert.hpp>
#include <Foundation/osthreads/MutexGuard.hpp>

#include "tracecollector/TraceFileMngr.hpp"
#include "tracecollector/HorizontalTracePrinter.hpp"
#include "tracecollector/VerticalTracePrinter.hpp"
#include "tracecollector/XMLTracePrinter.hpp"




namespace Cdmw
{
   namespace Trace
   {

/**
 *Implementation notes: 
 *<p>
 *Portability issues:  none 
 *<p>
 */

//
// Ctor
// 

      TraceFileMngr::TraceFileMngr (const std::string& file_name, 
                                    short max_file_nbr, int max_message_nbr,
                                    Message::TraceFormat trace_format,
                                    OsSupport::OS::TimeMode time_mode)
         throw(OutOfMemoryException,
               BadParameterException)
         : CircularFile (file_name, max_file_nbr, max_message_nbr),
           m_trace_format (trace_format),
           m_printer(NULL)
      {
         // if vertical format
         if (m_trace_format == Message::V_FORMAT)
            m_printer = new VerticalTracePrinter(time_mode);
         else if (m_trace_format == Message::H_FORMAT)
            m_printer = new HorizontalTracePrinter(time_mode);
         else // XML format
            m_printer = new XMLTracePrinter(time_mode);
      }


//
// Dtor
// 

      TraceFileMngr::~TraceFileMngr()
         throw()
      {
         delete m_printer;
         m_printer = NULL;    
      }


// return the circular trace file to write a new log message
// if a new circular file is opened due to max number of messages
// in previous file, end message and header are written in

      std::ostream* TraceFileMngr::determine_trace_file ()
         throw(BadParameterException)
      {
         // get current trace circular file stream
         std::ostream* pTraceFileStream = getCurrentCircularFileToWrite();

         // if circular file defined (not std::cout) and no stream error
         if (getCircularFileStream () != NULL && pTraceFileStream != NULL)
         {
            // If current trace file is full of messages, write end message
            if (getMessageNumber() == getMaxMessageNumber())
            {
               // Write end message
               m_printer->print_end_of_trace(*pTraceFileStream);
            }
         }
    
         // determine the circular file to write :
         // the current or a new trace circular file stream if max messages reached
         pTraceFileStream = determineCircularFileToWrite ();

         // if circular file defined (not std::cout)
         if (getCircularFileStream () != NULL)
         {
            // if no stream error
            if (pTraceFileStream != NULL)
            {
               // If current trace file is empty, write header message
               if (getMessageNumber() == 1)
               {
                  // Write a header in the log file
                  m_printer->print_header (*pTraceFileStream, getCircularFileNumber());
               }
            }
            else
            {
               CDMW_THROW2(BadParameterException,"Trace Log File cannot be opened",
                           getCircularFileName().c_str());
            }
         }
  
         return pTraceFileStream;
      }

//
// Print Error message in trace file
// 
      void TraceFileMngr::print_error_message (const std::string& errMesg)
         throw()
      {
         try
         {
            // set mutex for trace file stream updating
            CDMW_MUTEX_GUARD (m_lock);
  
            // determine the openned trace file stream
            std::ostream* p_os = determine_trace_file();

            // if no stream error
            if (p_os != NULL) 
            {
               // print error message
               m_printer->print_error_message (*p_os,errMesg);
            }
         }

         // catch any exception from Mutex and assert it
         catch (...)
         {
            CDMW_ASSERT(false);
         }
      }



//
// Format and Print Trace message
// 
      void TraceFileMngr::print_message (TraceCircularBuffer::TraceMessageRaw* pData)
         throw()
      {
    
         try
         {      
            // set mutex for trace file stream updating
            CDMW_MUTEX_GUARD (m_lock);    
            // determine the current openned trace file stream
            std::ostream* p_os = determine_trace_file();
       
            // if no stream error
            if (p_os != NULL) 
               m_printer->print_message(*p_os,*pData);
         }

         // catch any exception and assert it
         catch (...)
         {
            CDMW_ASSERT(false);
         }

      }

//
// Print end of trace file message
// 
      void TraceFileMngr::print_end_of_trace () const
         throw()
      {
         // Write end message in current log file

         // if circular file defined (not std::cout)
         // write the end of trace file
         if (getCircularFileStream () != NULL)
         {
            // get openned trace file stream
            std::ostream* pTraceFileStream = getCurrentCircularFileToWrite();

            if (pTraceFileStream != NULL)
            {
               // if no open error write end message
               m_printer->print_end_of_trace (*pTraceFileStream);
            }
         }
      }

   }; // End namespace Trace
}; // End namespace Cdmw

