/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

#include <Foundation/logging/LogLevel.hpp>
#include <sstream>
#include <iomanip>

#include "tracecollector/XMLTracePrinter.hpp"

namespace Cdmw
{
   namespace Trace
   {
   
      const std::string XMLTracePrinter::XML_END_OF_TRACE_FILE  = "_______________ END OF TRACE FILE ________________";
      const std::string XMLTracePrinter::XML_TRACE_HEADER1      = "xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx";
      const std::string XMLTracePrinter::XML_TRACE_HEADER2      = "                    LOG FILE  ";
      const std::string XMLTracePrinter::XML_TRACE_HEADER3      = "  (Cont)";
      const std::string XMLTracePrinter::XML_TRACE_ERR_MESG1    = "__________________________________________________";
      const std::string XMLTracePrinter::XML_TRACE_ERR_MESG2    = "__________ Error Message from Trace ______________";

      //
      // prints xml chainsaw message
      //         
      void XMLTracePrinter::print_message(std::ostream & os, const TraceCircularBuffer::TraceMessageRaw & Data) const
         throw()
      {
         static unsigned long idx = 0;
         // NOTA: Timestamp is to be stored in milliseconds. FIXME: use m_time_mode
         unsigned long long timestamp = static_cast<unsigned long long>(Data.m_messageHeader.m_timestamping.seconds) * 1000L;
         timestamp = timestamp + (static_cast<unsigned long long>(Data.m_messageHeader.m_timestamping.microseconds) / 1000L);
         ++idx;
         // NOTA: Convention for Logge names
         //       <application name>.<process name>.<component name>.<trace domain>
         os << "\n"
            << "<!-- Date: " << timestamp_to_string(Data.m_messageHeader.m_timestamping,"%a, %d %b %Y %H:%M:%S %z" /* "%d %m %Y %H:%M:%S" */ ) << " -->\n";
            // Operational Time
            if (Data.m_messageHeader.m_operational_timestamping.seconds)
            {
               os << "<!-- Operational Date: " << timestamp_to_string(Data.m_messageHeader.m_operational_timestamping,"%a, %d %b %Y %H:%M:%S %z" /* "%d %m %Y %H:%M:%S" */ ) << " -->\n";
            }            
            os << "<log4j:event logger=\"" << Data.m_mainHeader.m_pApplicationName << "."
            << Data.m_mainHeader.m_pProcessName << "." << Data.m_messageHeader.m_pComponentName << "."
            << Data.m_messageHeader.m_pUserDomain << "\""
            << " timestamp=\"" << timestamp << "\"";
            os << " sequenceNumber=\"" << idx <<  "\""
            << " level=\"" << Logging::LevelUtilities::Level_To_String(Data.m_messageHeader.m_userLevel) << "\""
            << " thread=\"" << Data.m_messageHeader.m_tid << "\">\n"
            << "<log4j:message><![CDATA[" << Data.m_pBodyMessage << "]]></log4j:message>\n"
            << "<log4j:NDC><![CDATA[" << Data.m_ndc  << "]]></log4j:NDC>\n"
            << "<log4j:locationInfo class=\"\" method=\"\""
            << " file=\"" << Data.m_messageHeader.m_pFileName << "\""
            << " line=\"" << Data.m_messageHeader.m_fileLine<<  "\" />\n"
            << "<log4j:properties>\n";
            // Operational Time
            if (Data.m_messageHeader.m_operational_timestamping.seconds)
            {
               timestamp = static_cast<unsigned long long>(Data.m_messageHeader.m_operational_timestamping.seconds) * 1000L;
               timestamp = timestamp + (static_cast<unsigned long long>(Data.m_messageHeader.m_operational_timestamping.microseconds) / 1000L);
               os << "\t<log4j:data name=\"OperationalTime\" value=\"" << timestamp << "\"/>\n";
            }            
            os << "\t<log4j:data name=\"ApplicationName\" value=\"" << Data.m_mainHeader.m_pApplicationName  << "\"/>\n"
            << "\t<log4j:data name=\"ComponentName\"   value=\"" << Data.m_messageHeader.m_pComponentName << "\"/>\n"
            << "\t<log4j:data name=\"ProcessID\"       value=\"" << Data.m_mainHeader.m_pid               << "\"/>\n"
            << "\t<log4j:data name=\"ProcessName\"     value=\"" << Data.m_mainHeader.m_pProcessName      << "\"/>\n"
            << "\t<log4j:data name=\"TraceDomain\"     value=\"" << Data.m_messageHeader.m_pUserDomain    << "\"/>\n"
            << "\t<log4j:data name=\"TraceLevel\"      value=\"" << "0x" << std::setw(8) << std::setfill('0') 
            << std::hex << Data.m_messageHeader.m_userLevel << std::dec << "\"/>\n"
            << "\t<log4j:data name=\"hostname\"        value=\"" << Data.m_mainHeader.m_pHostName << "\"/>\n"
            << "</log4j:properties>\n"
            << "</log4j:event>"
            << std::endl;
      }

      //
      // prints xml chainsaw error message
      //         
      void XMLTracePrinter::print_error_message (std::ostream & os, const std::string& errMesg) const
         throw()
      {
         os << "<!-- \n" 
            << XML_TRACE_ERR_MESG2 << "\n"
            << errMesg << "\n"
            << XML_TRACE_ERR_MESG1 << "\n"
            << " --> " << std::endl;
      }

      //
      // prints xml chainsaw header
      //         
      void XMLTracePrinter::print_header (std::ostream & os, int file_number) const
         throw()
      {
         os << "<!-- \n" 
            << XML_TRACE_HEADER1 << "\n"
            << XML_TRACE_HEADER2;
         if (file_number  > 0)
         {
            os << file_number << XML_TRACE_HEADER3;
         }
         os << "\n"
            << XML_TRACE_HEADER1 << "\n"
            << " -->\n" /* Chainsaw does not accept this 
                           << "<log4j:eventSet>" */ << std::endl;
      }
     
      //
      // prints enf of trace
      //
      void XMLTracePrinter::print_end_of_trace (std::ostream & os) const
         throw()
      {
         os << /* Chainsaw does not accept this "</log4j:eventSet>\n"
                  << */ "<!-- \n" << XML_END_OF_TRACE_FILE << "\n -->" << std::endl;
      }   
   
   
   }
}
