/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#include <clocale>
#include <fstream>
#include <iostream>
#include <sstream>
#include <string>

#include <Foundation/common/Exception.hpp>
#include <Foundation/common/Locations.hpp>
#include <Foundation/common/Options.hpp>
#include <Foundation/common/System.hpp>
#include <Foundation/ossupport/OS.hpp>
#include <Foundation/orbsupport/CORBA.hpp>
#include <Foundation/orbsupport/ExceptionMinorCodes.hpp>
#include <Foundation/orbsupport/OrbSupport.hpp>
#include <Foundation/orbsupport/StrategyList.hpp>

#include <SystemMngt/platforminterface/PlatformInterface.hpp>
#include <Repository/repositoryinterface/RepositoryInterface.hpp>


#include "tracecollector/CollectorInit.hpp"
#include "tracecollector/GlobalCollectorMngr.hpp"



namespace
{
    using std::string;

    // default ports used by the global collector.
    const string DFLT_SERVICE_PORT = "52345";
    const string DFLT_PROCESS_PORT = "52356";

} // anonymous namespace


using namespace Cdmw;
using namespace Cdmw::OsSupport;
using namespace Cdmw::OrbSupport;
using namespace Cdmw::Trace;
using namespace Cdmw::PlatformMngt;




/**
 * Global Collector initialization:
 *
 * - create the POA for the collector servant under RootPOA.
 * - create the GlobalCollectorMngr object:
 *       + the collector servant is created and activated on its POA.
 *       + the flushing data background thread is started.
 * - export the collector servant to the IOR file.
 * - activate the POA Manager.
 * - retrieve the Naming & Repository service.
 * - register the collector servant in Naming & Repository.
 */
int
init_global_collector (CORBA::ORB_ptr orb,
                       PortableGroup::GOA_ptr root_GOA,
                       CollectorOptions::Options& options,
                       CdmwNamingAndRepository::Repository_ptr p_repository_ref,
                       int argc, char* argv[])
{
    int status = FAILURE;

    using Cdmw::Common::Locations;


    CdmwTrace::Collector_var collector_ref = CdmwTrace::Collector::_nil();

    try 
    {
        // create a new POA.
        // the trace collector framework uses the following POA policies.
        //
        // Lifespan policy            = PERSISTENT
        // Id Assignment policy       = USER_ID
        // Id Uniqueness policy       = UNIQUE_ID
        // Implicit Activation policy = NO_IMPLICIT_ACTIVATION
        // Request Processing policy  = USE_ACTIVE_OBJECT_MAP_ONLY;
        // Servant Retention policy   = RETAIN;

        CORBA::PolicyList policies;
        policies.length(6);

        policies[0] =
            root_GOA->create_id_assignment_policy(PortableServer::USER_ID);
        policies[1] =
            root_GOA->create_lifespan_policy(PortableServer::PERSISTENT);
        policies[2] =
            root_GOA->create_servant_retention_policy(PortableServer::RETAIN);

        policies[3] =
            root_GOA->create_id_uniqueness_policy(PortableServer::UNIQUE_ID);

        policies[4] =
            root_GOA->create_request_processing_policy(
                    PortableServer::USE_ACTIVE_OBJECT_MAP_ONLY);
        policies[5] =
            root_GOA->create_implicit_activation_policy(
                    PortableServer::NO_IMPLICIT_ACTIVATION);

        Cdmw::OrbSupport::StrategyList poa_strategies;

        PortableServer::POAManager_var poa_manager = root_GOA->the_POAManager();
        PortableServer::POA_var collectors_POA = PortableServer::POA::_nil();

        collectors_POA = Cdmw::OrbSupport::OrbSupport::create_POA (
                                                 root_GOA,
                                                 options.collector_name.c_str(),
                                                 poa_manager.in(),
                                                 policies,
                                                 poa_strategies);

        PortableServer::ObjectId_var p_miopGlobalCollOid = PortableServer::ObjectId(0);
        if ( options.miop_corbaloc != "" )
        {
            // create a group reference for collectors
            CORBA::String_var corbaStrColl =
              CORBA::string_dup(options.miop_corbaloc.c_str());
            try 
            {
                CORBA::Object_var groupColl =
                  orb->string_to_object(corbaStrColl.in());
                //Create an object id for the servant and associate it
                // with the group reference
                // TO SEE : using collectors_POA raise InvalidPolicy:
                // build another specific POA ?
                //p_miopGlobalCollOid =
                //  collectors_POA.in()->create_id_for_reference(groupColl.in());
                p_miopGlobalCollOid =
                  root_GOA->create_id_for_reference(groupColl.in());
            }
            catch (const CORBA::BAD_PARAM&) {
                std::cerr << "Verify your MIOP adress" << std::endl;
                std::cerr << "Syntax is of kind:" <<
                  " corbaloc:miop:1.0@1.0-aDomain-AAAA/BBB.CCC.DDD.EEE:FFFF"
                  << std::endl;
                std::cerr <<
                  "( Where BBB.CCC.DDD.EEE must be 239.255.0.0 to 239.255.255.255,"
                  << std::endl;
                std::cerr <<
                "  The range 224.255.0.0 to 238.255.255.255 should also be valid )"
                  << std::endl;
                throw;
            }
        }

        // create and initialize the global collector manager.
        collector_ref = GlobalCollectorMngr::Init (collectors_POA.in(),
                                                   options.component_name,
                                                   options.domain_name,
                                                   options.level_value,
                                                   options.trace_format,
                                                   options.queue_strategy,
                                                   options.queue_size,
                                                   options.queue_str_size,
                                                   options.trace_file_name,
                                                   options.trace_file_bak_num,
                                                   options.trace_file_num,
                                                   options.trace_msg_num,
                                                   OS::LOCAL_TIME,
                                                   root_GOA,
                                                   //collectors_POA.in(),
                                                   p_miopGlobalCollOid.in() );

        // export the object reference to a file.
        CORBA::String_var ref_string = orb->object_to_string(collector_ref.in());
        std::ofstream os("globalTraceCollector.ior");
        os << ref_string.in();
        os.close();

        
        status = SUCCESS;
       
        if ( options.collector_name != "")
        // global collector can have no name if miop set
        {
            CdmwNamingAndRepository::NameDomain_var collector_domain;

            // register the collector object in the Naming & repository
            // under DefaultRootContext (using name domain).
            // trace collector location:
            // CDMW/SERVICES/TRACE/COLLECTORS/collectorName

            try 
            {
                if (status == SUCCESS) 
                {
                    status = FAILURE;

                    // get the collector domain ("CDMW/SERVICES/TRACE/COLLECTORS").
                    std::string
                      collector_domain_str(Locations::CDMW_SERVICES_NAME_DOMAIN);
                    collector_domain_str += "/";
                    collector_domain_str +=
                        Locations::TRACE_COLLECTORS_NAME_DOMAIN;

                    collector_domain =
                      p_repository_ref->resolve_name_domain
                                          (collector_domain_str.c_str());

                    status = SUCCESS;
                }
            }
            catch (const CdmwNamingAndRepository::NoNameDomain&) {
                std::cerr << "the trace collector name domain does not exist"
                          << std::endl;
            }
            catch (const CdmwNamingAndRepository::InvalidName&) {
                std::cerr << "the trace collector name domain is ill-formed"
                          << std::endl;
            }

            try 
            {
                if (status == SUCCESS) 
                {
                    status = FAILURE;

                    // register the object in the trace collector domain.
                    collector_domain->register_new_object
                                        ( options.collector_name.c_str(),
                                          collector_ref.in());

                    status = SUCCESS;
                }
            }
            catch (const CdmwNamingAndRepository::InvalidName&) {
                std::cerr << "the trace collector name is ill-formed"
                          << std::endl;
            }
            catch (const CdmwNamingAndRepository::NameDomain::AlreadyExists&) {
                std::cerr << "the collector object already registered in "
                          << "the repository" << std::endl;
                // the process must continue in this case.
                status = SUCCESS;
            }
            catch (const CORBA::BAD_PARAM&) {
                std::cerr << "attempt to register a nil object" << std::endl;
            }

        } // options.collector_name != ""

        // create the process agent to allow process stopping
        CollectorInit::create_process_agent (true, orb, root_GOA);
    }
    catch (const std::bad_alloc&) {
        std::cerr << "Process initialisation: bad allocation exception"
                  << std::endl;
    }
    catch (const Cdmw::Exception& ex) {
        std::cerr << ex.what() << std::endl;
    }
    catch (const CORBA::SystemException& ex) {
        std::cerr << "CORBA::SystemException" << ex << std::endl;
    }
    catch(const CORBA::Exception& ex) {
        std::cerr << "CORBA::Exception:" << ex << std::endl;
    }

    return status;
}


/**
 * Process termination.
 */
void
end_global_collector(void)
{
    GlobalCollectorMngr::Cleanup();
}


/**
 * Unexpected exception handler.
 */
void
exception_handler(void)
{
    std::cout << "Global Trace Collector: UNEXPECTED EXCEPTION RAISED"
              << std::endl;
}


/**
 * Main.
 */
int
main(int argc, char* argv[])
{
    ::setlocale(LC_ALL, "C");

#ifndef _MSC_VER
    // in VC++ contrary to the clause 18.6 of the C++ standard,
    // set_unexpected() is not defined within the std namespace.
    std::set_unexpected(&exception_handler);
#else
    set_unexpected(&exception_handler);
#endif


    using Cdmw::OsSupport::OS;
    using namespace Cdmw::Common;
    
    // get the collector options
    CollectorOptions::Options options;
    bool arg_rebuilt = false;
    char** new_argv = argv;
    int new_argc = argc;
    int arg_ini_inx = argc;

    int result = CollectorInit::get_collector_options (true, argc, argv, options,
                                                       new_argc, new_argv,
                                                       DFLT_SERVICE_PORT,
                                                       DFLT_PROCESS_PORT);
    if (result == FAILURE)
    {
        return FAILURE;
    }
    else if (result == EXIT)
    {
        return SUCCESS;
    }
    
    // if new arguments has been added                                                   
    if (new_argc > argc)
    {
        arg_rebuilt = true;
    }
    


    std::cout << "Global Trace Collector " << options.collector_name.c_str() 
              << " is listening on port " << options.service_port << "\n"; 
    std::cout << "   (" << Cdmw::Common::Options::PROCESS_PORT_OPTION
              << " is set to " << options.process_port << ")" << "\n";
    std::cout << std::endl;


    int status = SUCCESS;

    CORBA::ORB_var orb;

    try 
    {
        PortableGroup::GOA_var root_GOA;        
        CdmwNamingAndRepository::Repository_var repository_ref =
                CdmwNamingAndRepository::Repository::_nil();
        
        bool use_miop=true;
        if (options.miop_corbaloc == "") { use_miop = false; }

        // init the orb and platforminterface
        // (exception is raised in case of error)
        CollectorInit::init_processing (
                      new_argc, new_argv,
                      options.platform_managed,
                      options.naming_repos_url,
                      orb.out(), root_GOA.out(),
                      repository_ref.out(),
                      use_miop);
        
        // init the collector.
        status = init_global_collector (orb.in(),
                                        root_GOA.in(),
                                        options,
                                        repository_ref.in(),
                                        argc, argv);

        if (status == SUCCESS) 
        {
            status = CollectorInit::run_processing (orb.in(), options.interactive_mode);
        }


        // stop the process.
        end_global_collector();

        if (options.platform_managed) 
        {
            // cleanup the platform interface
            PlatformInterface::Cleanup();
        }

    }
    catch (const Cdmw::Exception& ex) {
        std::cerr << ex.what() << std::endl;
        status = FAILURE;
    }
    catch(const CORBA::Exception& ex) {
        std::cerr << ex << std::endl;
        status = FAILURE;
    }
    catch (...) {
        std::cerr << "unknown exception in main" << std::endl;
        status = FAILURE;
    }

    // end of corba processing

    if(!CORBA::is_nil(orb.in())) 
    {
        try 
        {
            Cdmw::OrbSupport::OrbSupport::ORB_cleanup(orb.in());
            orb->destroy();
        }
        catch(const CORBA::Exception& ex) 
        {
            std::cerr << ex << std::endl;
            status = FAILURE;
        }
    }

    // if program arguments table has been rebuilt.
    if (arg_rebuilt) 
    {
        // release any allocated memory
        for (int i=arg_ini_inx ; i < new_argc; ++i) 
        {
            delete[] new_argv[i];
        }

        delete[] new_argv;
    }

    return status;
}

