/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#include <cstdlib>
#include <fstream>
#include <sstream>
#include <string>

#include <Foundation/common/Assert.hpp>
#include <Foundation/common/Locations.hpp>
#include <Foundation/common/Options.hpp>
#include <Foundation/common/String.hpp>
#include <Foundation/common/System.hpp>
#include <Foundation/ossupport/OS.hpp>
#include <Foundation/orbsupport/OrbSupport.hpp>
#include <Foundation/orbsupport/StrategyList.hpp>
#include <Foundation/orbsupport/ExceptionMinorCodes.hpp>
#include <Foundation/testutils/CORBATestManager.hpp>
#include <Foundation/commonsvcs/naming/NamingInterface.hpp>
#include "testlogcollector/TestLogCollector.hpp"

namespace
{
    using std::string;

    const string DEF_NAME_SERVICE_PORT       = "5011";
    const string DEF_REPOSITORY_PORT         = "5021";
    const string DEF_TRC_COLLECTOR_SERV_PORT = "5031";
    const string DEF_TRC_COLLECTOR_CDMW_PORT = "5032";

    const string NAMING_SERVICE_PORT_OPTION  = "--NameServicePort";
    const string REPOSITORY_PORT_OPTION      = "--RepositoryPort";
    const string COLLECTOR_CDMW_PORT_OPTION  = "--CollectorCdmwPort";
    const string COLLECTOR_SERV_PORT_OPTION  = "--CollectorServicePort";

    const string COLLECTOR_NAME_OPTION       = "--collector-name";

    const string NAMING_REPOS_URL_OPTION     = "--naming-repos-url";

    const string COLLECTOR_NAME              = "collector_1";

    const int SUCCESS                        = 0;
    const int FAILURE                        = 1;

    const int POA_THREAD_POOL_SIZE           = 5;
}

void exceptionHandler(void)
{
    std::cout << "Trace Test: UNEXPECTED EXCEPTION HANDLER" << std::endl;
}


int main(int argc, char* argv[])
{
#ifndef _MSC_VER
    std::set_unexpected(&exceptionHandler);
    // VC++ contrary to the clause 18.6 of the C++ standard,
    // set_unexpected, is not inside the std:: namespace
#else
    set_unexpected(&exceptionHandler);
#endif

//     // Displays arguments used to launch this process
//     if(argc > 0) {
//         std::cout << "Arguments : " << argc-1 << std::endl;
//         for ( int index=1 ; index < argc ; index++ )
//             std::cout << "[" << index << "] :" << argv[index] << std::endl;
//     }

    using namespace Cdmw::OsSupport;
    using namespace Cdmw::Common;

    // launch Nameservice and SimRepository for tests
    OS::ProcessId idNaming, idRepository; 
    // launch TraceCollector and TraceLibrary for tests
    OS::ProcessId idLogCollector;

    // ORB arguments
    char** ORB_init_argv = 0;
    int ORB_init_argc(0);

    // get the Name Service port.
    std::string nameServicePort =
        OS::get_option_value(argc, argv, NAMING_SERVICE_PORT_OPTION);
    if (nameServicePort == "no") 
    {
        std::cout << "No Name Service Port, use default value" << std::endl;
        nameServicePort = DEF_NAME_SERVICE_PORT;
    } 

    // get the Repository port.
    std::string repositoryPort =
        OS::get_option_value (argc, argv, REPOSITORY_PORT_OPTION);
    if (repositoryPort == "no") 
    {
        std::cout << "No Repository Port, use default value" << std::endl;
        repositoryPort = DEF_REPOSITORY_PORT;
    } 

    // get the TraceCollector Service port.
    std::string collectorServPort =
        OS::get_option_value(argc, argv, COLLECTOR_SERV_PORT_OPTION);
    if (collectorServPort == "no") 
    {
        std::cout << "No Trace Collector Service Port, use default value" << std::endl;
        collectorServPort = DEF_TRC_COLLECTOR_SERV_PORT;
    } 

    // get the TraceCollector Cdmw port.
    std::string collectorCdmwPort =
        OS::get_option_value(argc, argv, COLLECTOR_CDMW_PORT_OPTION);
    if (collectorCdmwPort == "no") 
    {
        std::cout << "No Trace Collector Cdmw Port, use default value" << std::endl;
        collectorCdmwPort = DEF_TRC_COLLECTOR_CDMW_PORT;
    }
    
    // get Trace Collector Name
    std::string collectorName =
        OS::get_option_value(argc, argv, COLLECTOR_NAME_OPTION);
    if (collectorName == "no" || collectorName == "yes") 
    {
        std::cout << "No Trace Collector Name, use default value" << std::endl;
        collectorName = COLLECTOR_NAME;
    }
    
    // Starting the Name Service : "nameserv -OAport <port>"
    std::cout << "Start Name Service" << std::endl;
    std::string namingservice = "Naming_Service";
    std::string namingservice_options = "-ORBEndpoint iiop://localhost:" + nameServicePort;
    std::cout << namingservice << " " << namingservice_options << std::endl;
    
    idNaming = OS::create_process(namingservice, namingservice_options);

    int timescale = Cdmw::TestUtils::get_timescale();
    int delay = timescale * 5000 /* 10000 */;
    std::cout << "Wait " << delay / 1000 << " sec for Name Service." << std::endl;
    OS::sleep(delay);
    
    // Starting the Repository Simulator :
    // "cdmw_simulated_repository -OAport <port> 
    //  -ORBInitRef NameService=corbaloc::localhost:<port>/NameService
    //  -DOM CDMW/SERVICES -DOM CDMW/ADMIN -DOM CDMW/ADMIN/TRACE/COLLECTORS
    //  -ROOT AdminRootContext"
    std::string repository = "cdmw_simulated_repository";
    std::string namingCorbaloc =
        " -ORBInitRef NameService=corbaloc:iiop:localhost:" +
        nameServicePort + "/NameService";
    
    std::string repositoryArgs ("");

    repositoryArgs += " -ORBEndpoint iiop://localhost:";
    repositoryArgs += repositoryPort;
    repositoryArgs += namingCorbaloc;
    repositoryArgs += " -DOM ";
    repositoryArgs += Cdmw::Common::Locations::CDMW_SERVICES_NAME_DOMAIN;
    repositoryArgs += " -DOM ";
    repositoryArgs += Cdmw::Common::Locations::CDMW_SERVICES_NAME_DOMAIN;
    repositoryArgs += "/";
    repositoryArgs += Cdmw::Common::Locations::TRACE_COLLECTORS_NAME_DOMAIN;
    repositoryArgs += " -ROOT ";
    repositoryArgs += Cdmw::Common::Locations::ADMIN_ROOT_CONTEXT_ID;
        
    std::cout << "Start Repository Simulator" << std::endl
              << repository << repositoryArgs << std::endl;

    idRepository = OS::create_process(repository, repositoryArgs);

    delay = timescale * 5000 /* 15000 */;
    std::cout << "Wait " << delay / 1000 << " sec for Simulated Repository." << std::endl;
    OS::sleep(delay);
        
    // set repository url
    std::string repositoryCorbaloc =
        " " + NAMING_REPOS_URL_OPTION + "=corbaloc::localhost:" +
        repositoryPort + "/SimRepository";

    // build ORB arguments with process name and naming corbaloc
    std::string strArgv = argv[0] + namingCorbaloc;
    ORB_init_argv = String::to_char_array(String::to_strings(strArgv));
    ORB_init_argc = 3;
    
    int status = FAILURE;
    CORBA::ORB_var orb;
    PortableServer::POA_var rootPOA;
    try 
    {
        // Initialize the ORB
        Cdmw::OrbSupport::StrategyList orb_strategies;
        orb_strategies.add_OrbThreaded();
        orb_strategies.add_PoaThreadPool(POA_THREAD_POOL_SIZE);

        orb = Cdmw::OrbSupport::OrbSupport::ORB_init(ORB_init_argc,
                                                     ORB_init_argv, 
                                                     orb_strategies);
        
        // Get the root POA
        CORBA::Object_var obj = orb->resolve_initial_references( "RootPOA" );
        rootPOA
            = PortableServer::POA::_narrow( obj.in() );

        // activate the POA Manager.
        PortableServer::POAManager_var POA_mngr = rootPOA->the_POAManager();
        POA_mngr->activate();

        Cdmw::TestUtils::CORBATestManager::instance()->init(orb.in(),rootPOA.in(),argc,argv);
        status = SUCCESS;
    }
    catch (const Cdmw::Exception& ex) 
    {
      std::cerr << ex.what() << std::endl;
    } 
    catch(const CORBA::Exception& ex) 
    {        
      std::cerr << ex << std::endl;
    }


    Cdmw::TestUtils::CORBATestManager::instance()->set_property("system_port",
                                                        DEF_NAME_SERVICE_PORT );

    std::cout << "Starts the Log Collector process." << std::endl;

    // Log generated by the logcollectorprocess will be saved in LCP.log
    // If you modify this file name, update it in TestLogCollector (because it is hard coded)
    idLogCollector =
        OS::create_process("logcollectorprocess", "--CdmwLogFile=LCP.log");

    delay = timescale * 5000 /* 10000 */;
    std::cout << "Wait " << delay / 1000 << " sec for Log Collector Process." << std::endl;
    OS::sleep(delay);

    // Define parameters used to launch the L_TRC
    // Used by the logcollectorprocess
    std::string loc_service_opt = Options::LOCALISATION_SERVICE_OPTION;
    std::string proc_port_opt = Options::PROCESS_PORT_OPTION;
    std::string traceCollectorArgs ("");
    traceCollectorArgs += " " + COLLECTOR_NAME_OPTION + "=";
    traceCollectorArgs += collectorName;
    traceCollectorArgs += " " + loc_service_opt + "=";
    traceCollectorArgs += collectorServPort;
    traceCollectorArgs += " " + proc_port_opt + "=";
    traceCollectorArgs += collectorCdmwPort;
    traceCollectorArgs += repositoryCorbaloc;
    traceCollectorArgs += " ";
    traceCollectorArgs += Cdmw::Common::Options::SYSTEM_PORT_OPTION;
    traceCollectorArgs += "=" + DEF_NAME_SERVICE_PORT;

    Cdmw::TestUtils::CORBATestManager::instance()->set_property("LTRC_args",traceCollectorArgs);

    // Run the test
    Cdmw::TestUtils::CORBATestManager::instance()->run_tests();
    // Print the result
    Cdmw::TestUtils::CORBATestManager::instance()->summary();


    std::cout << "stop all processes" << std::endl;
    delay = timescale * 5000 /* 10000 */;
    std::cout << "Wait " << delay / 1000 << " sec before stopping all processes." << std::endl;
    OS::sleep(delay);
    

    // kill the Log Collector
    try 
    {
        OS::kill_process(idLogCollector);
    }
    catch(const std::exception & ex) 
    {
        std::cerr << "Cannot kill Log Collector process.:" << std::endl;
        status = FAILURE;
    }

    // then kill the Repository Simulator and Name Server
    try 
    {
        OS::kill_process(idRepository);
    }
    catch(const std::exception & ex) 
    {
        std::cerr << "Cannot kill Repository process.:" << std::endl;
        status = FAILURE;
    }
    try 
    {
        OS::kill_process(idNaming);
    }
    catch(const std::exception & ex) 
    {
        std::cerr << "Cannot kill Name Server process.:" << std::endl;
        status = FAILURE;
    }

    // free memory allocated by String::to_char_array
    for (int iArgc = 0; iArgc < ORB_init_argc; ++iArgc) 
    {
        delete[] ORB_init_argv[iArgc];
    }
    delete[] ORB_init_argv;

    // end of corba processing
    if (!CORBA::is_nil(orb.in())) 
    {
        try 
        {
            orb->shutdown(false);

            Cdmw::OrbSupport::OrbSupport::ORB_cleanup(orb.in());

            orb->destroy();
            std::cerr << std::endl;
        }
        catch(const CORBA::Exception& ex) 
        {
            std::cerr << ex << std::endl;
            status = FAILURE;
        }
    }

    return status;
}
