/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#include <cstdlib>
#include <fstream>
#include <sstream>
#include <string>

#include <Foundation/common/Assert.hpp>
#include <Foundation/common/Locations.hpp>
#include <Foundation/common/Options.hpp>
#include <Foundation/common/String.hpp>
#include <Foundation/common/System.hpp>
#include <Foundation/ossupport/OS.hpp>
#include <Foundation/orbsupport/OrbSupport.hpp>
#include <Foundation/orbsupport/StrategyList.hpp>
#include <Foundation/orbsupport/ExceptionMinorCodes.hpp>
#include <Foundation/commonsvcs/naming/NamingInterface.hpp>
#include <Foundation/logging/LogManager.hpp>
#include "tracecollector/InitUtilsLogCollector.hpp"
#include "testlogcollector/SIM_Collector.hpp"
#include <Repository/repositoryinterface/RepositoryInterface.hpp>

namespace
{
    using std::string;

    const string DEF_NAME_SERVICE_PORT       = "5011";
    const string DEF_REPOSITORY_PORT         = "5021";
    const string GLOBAL_COLLECTOR_NAME       = "global_collector_1";

    const int SUCCESS                        = 0;
    const int FAILURE                        = 1;

    const int POA_THREAD_POOL_SIZE           = 5;
}

void exceptionHandler(void)
{
    std::cout << "Trace Test: UNEXPECTED EXCEPTION HANDLER" << std::endl;
}


int main(int argc, char* argv[])
{
#ifndef _MSC_VER
    std::set_unexpected(&exceptionHandler);
    // VC++ contrary to the clause 18.6 of the C++ standard,
    // set_unexpected, is not inside the std:: namespace
#else
    set_unexpected(&exceptionHandler);
#endif

//     // Displays arguments used to launch this process
//     if(argc > 0) {
//         std::cout << "Arguments : " << argc-1 << std::endl;
//         for ( int index=1 ; index < argc ; index++ )
//             std::cout << "[" << index << "] :" << argv[index] << std::endl;
//     }

    // Used to set the Log File name
    Cdmw::Logging::LogManager::Init(argc, argv);

    using namespace Cdmw::OsSupport;
    using namespace Cdmw::Common;

    // ORB arguments
    char** ORB_init_argv = 0;
    int ORB_init_argc(0);

    // get the Name Service port.
    std::string nameServicePort = DEF_NAME_SERVICE_PORT;
    
    int status = FAILURE;
    CORBA::ORB_var orb;
    PortableServer::POA_var rootPOA;
    try 
    {
        // Initialize the ORB
        Cdmw::OrbSupport::StrategyList orb_strategies;
        orb_strategies.add_OrbThreaded();
        orb_strategies.add_PoaThreadPool(POA_THREAD_POOL_SIZE);

        orb = Cdmw::OrbSupport::OrbSupport::ORB_init(ORB_init_argc,
                                                     ORB_init_argv, 
                                                     orb_strategies);
        
        // Get the root POA
        CORBA::Object_var obj = orb->resolve_initial_references( "RootPOA" );
        rootPOA = PortableServer::POA::_narrow( obj.in() );

        // activate the POA Manager.
        PortableServer::POAManager_var POA_mngr = rootPOA->the_POAManager();
        POA_mngr->activate();
        status = SUCCESS;
    }
    catch (const Cdmw::Exception& ex) 
    {
      std::cerr << ex.what() << std::endl;
    } 
    catch(const CORBA::Exception& ex) 
    {        
      std::cerr << ex << std::endl;
    }

    // Create a simulator of Global Trace Collector before starting the Local
    // Trace Collector
    Cdmw::Trace::SIM_Collector* pSim_CollectorServant = new Cdmw::Trace::SIM_Collector;
    PortableServer::ServantBase_var servant_var = pSim_CollectorServant;
    CdmwTrace::Collector_var collector = pSim_CollectorServant->_this();

    string sim_repository_corbaloc= "corbaloc:iiop:localhost:" +
        DEF_REPOSITORY_PORT + "/SimRepository";

    CORBA::Object_var obj = orb->string_to_object(sim_repository_corbaloc.c_str()) ;
    CdmwNamingAndRepository::Repository_var repository_ref = CdmwNamingAndRepository::Repository::_narrow(obj.in());

    std::string collector_domain_str(Locations::CDMW_SERVICES_NAME_DOMAIN);
    collector_domain_str += "/";
    collector_domain_str += Locations::TRACE_COLLECTORS_NAME_DOMAIN;
    
    CdmwNamingAndRepository::NameDomain_var collector_domain =
        repository_ref->resolve_name_domain(collector_domain_str.c_str());
    collector_domain->register_new_object(GLOBAL_COLLECTOR_NAME.c_str(),
                                          collector.in());

    // Start the Log Collector
    Cdmw::Trace::InitUtilsLogCollector::Create_log_collector(
                                                orb.in(),
                                                rootPOA.in(),          
                                                nameServicePort, // SystemPort
                                                100,// Period in milliseconds
                                                500,// Circular buffer size
                                                10, // Size sequence sent
                                                    // to LTRC
                                                1000 // chunk size
                                                );

    orb->run();

    // free memory allocated by String::to_char_array
    for (int iArgc = 0; iArgc < ORB_init_argc; ++iArgc) 
    {
        delete[] ORB_init_argv[iArgc];
    }
    delete[] ORB_init_argv;

    // end of corba processing
    if (!CORBA::is_nil(orb.in())) 
    {
        try 
        {
            orb->shutdown(false);

            Cdmw::OrbSupport::OrbSupport::ORB_cleanup(orb.in());

            orb->destroy();
            std::cerr << std::endl;
        }
        catch(const CORBA::Exception& ex) 
        {
            std::cerr << ex << std::endl;
            status = FAILURE;
        }
    }

    return status;
}
