/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#include <sstream>
#include <string>

#include "tao/Messaging/Messaging.h"

#include <Foundation/common/Assert.hpp>
#include <Foundation/common/Exception.hpp>
#include <Foundation/common/Options.hpp>
#include <Foundation/common/Locations.hpp>
#include <Foundation/commonsvcs/naming/NamingInterface.hpp>
#include <Foundation/commonsvcs/naming/NamingUtil.hpp>
#include <Foundation/logging/LogManager.hpp> // ECR-0169
#include <Foundation/ossupport/OS.hpp>
#include <Foundation/orbsupport/OrbSupport.hpp>
#include <Foundation/orbsupport/ExceptionMinorCodes.hpp>

#include <Repository/repositoryinterface/RepositoryInterface.hpp>

#include <TraceAndPerf/tracelibrary/InitUtils.hpp>
#include <TraceAndPerf/tracelibrary/Trace.hpp>

#include "tracelibrary/LogManagerTraceDelegate.hpp" // ECR-0169


namespace
{
    // mnemonic name prefix for the trace collectors.
    const std::string COLLECTOR_INI_MNEMONAME = "collector_ini_";
}; // anonymous namespace


namespace Cdmw
{
namespace Trace
{


/**
 * Initialize the trace library.
 */
void
InitUtils::init_trace_library(
        CORBA::ORB_ptr orb,
        PortableServer::POA_ptr trace_POA,
        const std::string& application_name,
        const std::string& process_name,
        size_t flushing_time,
        size_t msg_threshold,
        size_t nb_FlushArea,
        size_t size_FlushArea,
        const std::vector<std::string>& collector_name_list,
        Logging::LevelType max_log_level // ECR-0169
        )
    throw(Cdmw::OrbSupport::CORBASystemExceptionWrapper)
{
    using Cdmw::Common::Locations;
    using Cdmw::OrbSupport::CORBASystemExceptionWrapperT;
    using Cdmw::NamingAndRepository::RepositoryInterface;
    using Cdmw::Trace::FlushAreaMngr;

    using namespace Cdmw::CommonSvcs::Naming;
    using namespace Cdmw::OrbSupport;
    using namespace Cdmw::OsSupport;

    // the initial list of collectors.
    FlushAreaMngr::CollectorDataList collectorList;

    // get the "AdminRootContext" root context.
    CosNaming::NamingContext_var adminRootCtx =
        CosNaming::NamingContext::_nil();

    try 
    {
        CdmwNamingAndRepository::Repository_var rep =
            RepositoryInterface::get_repository();

        adminRootCtx =
            rep->resolve_root_context(Locations::ADMIN_ROOT_CONTEXT_ID);
    }
    catch (const CdmwNamingAndRepository::Repository::NoRootContext&) 
    {
        std::string s(Locations::ADMIN_ROOT_CONTEXT_ID);
        s += " is not a root context.";

        CDMW_THROW2(
            CORBASystemExceptionWrapperT<CORBA::INTERNAL>,
            CORBA::INTERNAL(INTERNALCdmwRepositoryError,
                            CORBA::COMPLETED_NO),
            s);
    }
    catch (const CORBA::SystemException& ex) 
    {
        CDMW_THROW1(
            CORBASystemExceptionWrapperT<CORBA::SystemException>,
            ex);
    }

    // get the registered trace collector
    try 
    {
        // ECR-0145: there is only one local collector per node.
        //
        // the pattern is:
        // <hostname>/SERVICES/TRACE/COLLECTORS/<collector_name>
        //
        // where collector_name is either the first name in collector_name_list
        // or "LocalCollector" if collector_name_list is empty.
        const std::string collectorName((collector_name_list.size() > 0)
                                            ? collector_name_list[0]
                                            : "LocalCollector");

        // set the collector name context
        CosNaming::Name collectorName_;
        collectorName_.length(5);
        collectorName_[0].id = CORBA::string_dup(OS::get_hostname().c_str());
        collectorName_[1].id = CORBA::string_dup("SERVICES");
        collectorName_[2].id = CORBA::string_dup("TRACE");
        collectorName_[3].id = CORBA::string_dup("COLLECTORS");
        collectorName_[4].id = CORBA::string_dup(collectorName.c_str());

        NamingInterface ni_collector(adminRootCtx.in());

        bool collector_found = true;

        CdmwTrace::Collector_var collector;

        try 
        {
            // search the collector under admin root context
            CORBA::Object_var obj = ni_collector.resolve(
                                       ni_collector.to_string(collectorName_));
            collector = CdmwTrace::Collector::_narrow(obj.in());
        }
        catch (const CosNaming::NamingContext::NotFound&)
        {
            // ignore if collector not registered in repository
            collector_found = false;

            std::cerr << "The local trace collector "
                      << ni_collector.to_string(collectorName_)
                      << " could not be found in the repository."
                      << std::endl;
        }

        if (collector_found)
        {
            // create collector mnemonic name
            std::ostringstream mnemo_name_str;

            mnemo_name_str << COLLECTOR_INI_MNEMONAME << 0;

            // create collector data structure
            CdmwTrace::TraceProducer::CollectorData collector_data;
            collector_data.the_collectorMnemoName =
                       CORBA::string_dup(mnemo_name_str.str().c_str());
            collector_data.the_collectorObjName =
                       CORBA::string_dup(collectorName.c_str());

            /* Section to add SYNC_WITH_TARGET on the collector object */
            // the oneway operations (put_formated_message) will behave
            // like two ways operations
            static Messaging::SyncScope sync_scope = Messaging::SYNC_WITH_TARGET;

            CORBA::Object_var obj =
              orb->resolve_initial_references("ORBPolicyManager");

            CORBA::PolicyManager_var policy_manager
                = CORBA::PolicyManager::_narrow(obj.in());

            // Set up the sync scope any.
            CORBA::Any sync_scope_any;
            sync_scope_any <<= sync_scope;

            // Set the length of the policy list.
            CORBA::PolicyList sync_scope_policy_list (1);
            sync_scope_policy_list.length (1);
      
            // Set up the sync scope policy.
            sync_scope_policy_list[0] =
                orb->create_policy (Messaging::SYNC_SCOPE_POLICY_TYPE,
                                    sync_scope_any);

            // Set the sync scope policy at the object level.
            obj = collector->_set_policy_overrides ( sync_scope_policy_list,
                                                     CORBA::ADD_OVERRIDE );
            // Get the new object reference with the updated policy.
            collector = CdmwTrace::Collector::_narrow ( obj.in () );

            sync_scope_policy_list[0]->destroy();
            /* End of Section to add SYNC_WITH_TARGET */

            // (collector is a _var, _duplicate is implicit)
            collector_data.the_collector = collector;

            // insert collector data in list
            collectorList.insert(collectorList.end(), collector_data);
        }
    }
    catch (const std::bad_alloc& ex) 
    {
        CDMW_THROW2(
            CORBASystemExceptionWrapperT<CORBA::NO_MEMORY>,
            CORBA::NO_MEMORY(Cdmw::OrbSupport::NO_MEMORY,
                             CORBA::COMPLETED_NO),
            ex.what());
    }
    catch (const CdmwNamingAndRepository::NoNameDomain& ex) 
    {
        CDMW_THROW1(
            CORBASystemExceptionWrapperT<CORBA::INTERNAL>,
            CORBA::INTERNAL(INTERNALCdmwRepositoryError,
                            CORBA::COMPLETED_NO));
    }
    catch (const CdmwNamingAndRepository::InvalidName& ex) 
    {
        CDMW_THROW2(
            CORBASystemExceptionWrapperT<CORBA::INTERNAL>,
            CORBA::INTERNAL(INTERNALInvalidTraceCollectorsLocation,
                            CORBA::COMPLETED_NO),
            ex.reason.in());
    }
    catch (const Cdmw::CommonSvcs::Naming::InvalidNameException& ex) 
    {
        CDMW_THROW2(
            CORBASystemExceptionWrapperT<CORBA::INTERNAL>,
            CORBA::INTERNAL(INTERNALInvalidTraceCollectorsLocation,
                            CORBA::COMPLETED_NO),
            ex.what());
    }
    catch (const CosNaming::NamingContext::InvalidName& ex) 
    {
        CDMW_THROW1(
            CORBASystemExceptionWrapperT<CORBA::INTERNAL>,
            CORBA::INTERNAL(INTERNALInvalidTraceCollectorsLocation,
                            CORBA::COMPLETED_NO));
    }
    catch (const CosNaming::NamingContext::CannotProceed& ex) 
    {
        CDMW_THROW1(
            CORBASystemExceptionWrapperT<CORBA::INTERNAL>,
            CORBA::INTERNAL(INTERNALInvalidTraceCollectorsLocation,
                            CORBA::COMPLETED_NO));
    }
    catch (const CORBA::SystemException& ex) 
    {
        CDMW_THROW1(
            CORBASystemExceptionWrapperT<CORBA::SystemException>,
            ex);
    }
    
    
    // create the trace producer
    CdmwTrace::TraceProducer_var trace_producer =
                                  CdmwTrace::TraceProducer::_nil();

    try 
    {
        trace_producer = FlushAreaMngr::init (trace_POA,
                                              collectorList,
                                              application_name,
                                              process_name,
                                              flushing_time,
                                              nb_FlushArea,
                                              size_FlushArea,
                                              FlushAreaMngr::DEFAULT_AREA_NB_MESSAGE,
                                              msg_threshold,
                                              orb);
    }
    catch (const Cdmw::OutOfMemoryException& ex) 
    {
        CDMW_THROW2(
            CORBASystemExceptionWrapperT<CORBA::NO_MEMORY>,
            CORBA::NO_MEMORY(NO_MEMORY,
                             CORBA::COMPLETED_NO),
            ex.what());
    }
    catch (const Cdmw::BadParameterException& ex) 
    {
        CDMW_THROW2(
            CORBASystemExceptionWrapperT<CORBA::BAD_PARAM>,
            CORBA::BAD_PARAM(BAD_PARAM,
                             CORBA::COMPLETED_NO),
            ex.what());
    }
    catch (const Cdmw::InternalErrorException& ex) 
    {
        CDMW_THROW2(
            CORBASystemExceptionWrapperT<CORBA::INTERNAL>,
            CORBA::INTERNAL(INTERNAL,
                            CORBA::COMPLETED_NO),
            ex.what());
    }


    // Register TraceProducer in repository
    // the pattern is:
    // <hostname>/<application_name>/<process name>/TRACE/TraceProducer
        
    std::string trace_producer_name;

    // set the name context
    CosNaming::Name producerName_;
    producerName_.length(5);
    producerName_[0].id = CORBA::string_dup(OS::get_hostname().c_str());
    producerName_[1].id = CORBA::string_dup(application_name.c_str());
    producerName_[2].id = CORBA::string_dup(process_name.c_str());
    producerName_[3].id = CORBA::string_dup("TRACE");
    producerName_[4].id = CORBA::string_dup("TraceProducer");

    // Get "AdminRootContext" root context and then
    // register TraceProducer object

    try 
    {
        NamingInterface ni(adminRootCtx.in());

        trace_producer_name = ni.to_string(producerName_);

        // if already registered, force its registration.
        ni.bind(trace_producer_name, trace_producer.in(), true);
    }
    catch (const CosNaming::NamingContext::NotFound&) 
    {
        // should not happen!
        CDMW_THROW1(
            CORBASystemExceptionWrapperT<CORBA::BAD_PARAM>,
            CORBA::BAD_PARAM(BAD_PARAMInvalidTraceProducerLocation,
                             CORBA::COMPLETED_NO));
    }
    catch (const CosNaming::NamingContext::CannotProceed&) 
    {
        // should not happen!
        CDMW_THROW1(
            CORBASystemExceptionWrapperT<CORBA::INTERNAL>,
            CORBA::INTERNAL(INTERNALCdmwRepositoryError,
                            CORBA::COMPLETED_NO));
    }
    catch (const CosNaming::NamingContext::InvalidName& ex) 
    {
        std::string s(trace_producer_name);
        s += " is not valid.";
        CDMW_THROW2(
            CORBASystemExceptionWrapperT<CORBA::BAD_PARAM>,
            CORBA::BAD_PARAM(BAD_PARAMInvalidTraceProducerName,
                             CORBA::COMPLETED_NO),
            s);
    }
    catch (const CosNaming::NamingContext::AlreadyBound&) 
    {
        // should not happen because force is set to true!
        std::string s(trace_producer_name);
        s += " is already bound.";
        CDMW_THROW2(
            CORBASystemExceptionWrapperT<CORBA::INTERNAL>,
            CORBA::INTERNAL(INTERNALCdmwRepositoryError,
                            CORBA::COMPLETED_NO),
            s);
    }
    catch (const CORBA::SystemException& ex) 
    {
        CDMW_THROW1(
            CORBASystemExceptionWrapperT<CORBA::SystemException>,
            ex);
    }
}


}; // namespace Trace
}; // namespace Cdmw

