/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#include "TraceAndPerf/tracelibrary/Message.hpp"

// To get memcpy
#include <cstring>
#include <cstdlib>

#include "Foundation/common/Assert.hpp"
#include "tracelibrary/FlushArea.hpp"


namespace Cdmw
{
namespace Trace
{

/**
*Implementation notes:  none 
*<p>
*Portability issues:  none 
*<p>
*/


//
// Ctor
//    
Message::Message()
    throw()
    : m_pBuffer (NULL),
      m_szTotalBuffer(0),
      m_pBodyBuffer (NULL),
      m_szBodyBuffer(0)
{
    m_messageHeaderRaw.m_tid = 0;
    m_messageHeaderRaw.m_fileLine = 0;
    m_messageHeaderRaw.m_pFileName = NULL;
    m_messageHeaderRaw.m_szFileName = 0;
    m_messageHeaderRaw.m_pComponentName = NULL; // ECR-0123
    m_messageHeaderRaw.m_szComponentName = 0; // ECR-0123
    m_messageHeaderRaw.m_pUserDomain = NULL;
    m_messageHeaderRaw.m_szUserDomain = 0;
    m_messageHeaderRaw.m_userLevel = 0;
    m_messageHeaderRaw.m_timestamping.seconds = 0;
    m_messageHeaderRaw.m_timestamping.microseconds = 0;
}



//
// Copy Ctor
//  Remember the memory is not allocated by new, so
//  the memory is not copied or reallocated, we use
//  the same zone
//
Message::Message (const Message& rhs)
    throw()
    : m_pBuffer (rhs.m_pBuffer),
      m_szTotalBuffer(rhs.m_szTotalBuffer),
      m_pBodyBuffer (rhs.m_pBodyBuffer),
      m_szBodyBuffer(rhs.m_szBodyBuffer)
{
    m_messageHeaderRaw = rhs.m_messageHeaderRaw;
}




//
// Assignement operator
//
Message&
Message::operator=(const Message& rhs)
    throw()
{
    m_pBuffer = rhs.m_pBuffer;
    m_szTotalBuffer = rhs.m_szTotalBuffer;
    m_pBodyBuffer = rhs.m_pBodyBuffer;
    m_szBodyBuffer = rhs.m_szBodyBuffer;
    m_messageHeaderRaw = rhs.m_messageHeaderRaw;

    return *this;
}



//
// Dtor
//
Message::~Message()
    throw()
{
    // Nothing to do
}

//
// Get size of message header to allocate in buffer area
//
size_t Message::getMessageHeaderStringSize (const CdmwLogging::LogHeader& header)
{
    size_t hdrStrSize = std::string(header.file_name).length() + 1;
    hdrStrSize += strlen(header.component_name) + 1; // ECR-0123
    hdrStrSize += strlen(header.user_domain) + 1;

    return hdrStrSize;
}


/**
    * Compare 2 message headers.
    * Note: the timestamp is ignored.
    *
    * @param hdr1 the first message header
    * @param hdr2 the second message header
    */
bool
Message::headercmp(const CdmwLogging::LogHeader& hdr1,
                   const CdmwLogging::LogHeader& hdr2)
{
    return ((hdr1.thread_id == hdr2.thread_id) &&
            (hdr1.file_line == hdr2.file_line) &&
            (! strcmp(hdr1.file_name, hdr2.file_name)) &&
            (! strcmp(hdr1.component_name, hdr2.component_name)) && // ECR-0123
            (! strcmp(hdr1.user_domain, hdr2.user_domain)) &&
            (hdr1.level == hdr2.level));
}


//
// Store the message given by the octet sequence, with the header in internal data
//
void Message::set_message (const CdmwLogging::LogHeader& header,
                           const CORBA::OctetSeq& msgBody)
    throw (AssertionFailedException)
{
    // init current pointer on buffer area and current allocated size
    char*  pCurrentBuffer = m_pBuffer;
    size_t szCurrentBuffer = m_szTotalBuffer;
    size_t szParameter;

    // store thread id and file line
    m_messageHeaderRaw.m_tid = header.thread_id;
    m_messageHeaderRaw.m_fileLine = header.file_line;

    // check that the size of the buffer is enough for file name
    // store file name in buffer area, its pointer and its size in header
    szParameter = strlen(header.file_name) + 1;
    CDMW_ASSERT (szParameter < szCurrentBuffer);
    m_messageHeaderRaw.m_pFileName = pCurrentBuffer;
    ::strcpy (m_messageHeaderRaw.m_pFileName, header.file_name);
    m_messageHeaderRaw.m_szFileName = szParameter;
    
    // increment current pointer on buffer area and decrement current allocated size
    pCurrentBuffer += szParameter;
    szCurrentBuffer -= szParameter;

    // ECR-0123
    // add the component name to the raw message header
    szParameter = strlen(header.component_name) + 1;
    CDMW_ASSERT(szParameter < szCurrentBuffer);
    m_messageHeaderRaw.m_pComponentName = pCurrentBuffer;
    ::strcpy(m_messageHeaderRaw.m_pComponentName, header.component_name);
    m_messageHeaderRaw.m_szComponentName = szParameter;

    // increment current pointer on buffer area and decrement current allocated size
    pCurrentBuffer += szParameter;
    szCurrentBuffer -= szParameter;

    // check that the size of the buffer is enough for user domain
    // store user domain in buffer area, its pointer and its size in header
    szParameter = strlen(header.user_domain) + 1;
    CDMW_ASSERT (szParameter < szCurrentBuffer);
    m_messageHeaderRaw.m_pUserDomain = pCurrentBuffer;
    ::strcpy (m_messageHeaderRaw.m_pUserDomain, header.user_domain);
    m_messageHeaderRaw.m_szUserDomain = szParameter;

    // increment current pointer on buffer area and decrement current allocated size
    pCurrentBuffer += szParameter;
    szCurrentBuffer -= szParameter;
    
    // store user level, time stamping
    m_messageHeaderRaw.m_userLevel = header.level;
    m_messageHeaderRaw.m_timestamping.seconds = header.time_stamping.seconds;
    m_messageHeaderRaw.m_timestamping.microseconds = header.time_stamping.microseconds;

    // check that the size of the buffer is enough for body message
    // store body message in buffer area, its pointer and its size in message
    szParameter = msgBody.length();
    CDMW_ASSERT (szParameter < (szCurrentBuffer+1));
    m_pBodyBuffer = pCurrentBuffer;
    //::memcpy (m_pBodyBuffer, msgBody, szParameter); too raw i think..
    //std::cout << "set message via chars:";
    for (size_t i = 0; i < szParameter; ++i)
    {
      m_pBodyBuffer[i] = (char)msgBody[i];
      //std::cout << m_pBodyBuffer[i];
    }
    //std::cout << std::endl;
    //std::cout << "set message via char*:" << m_pBodyBuffer << std::endl;
    m_szBodyBuffer = szParameter;
}

//
// Store the message with the header in internal data
//
void Message::set_message (const CdmwLogging::LogHeader& header,
                           const std::string& str)
    throw (AssertionFailedException)
{
    size_t msgLen = str.length();
    CORBA::OctetSeq_var msgBody =  new CORBA::OctetSeq();
    msgBody->length(msgLen+1);
    for (size_t idxMsg = 0; idxMsg < msgLen; ++idxMsg)
    {
      msgBody[idxMsg] = (CORBA::Octet)str.at(idxMsg);
    }
    msgBody[msgLen] = '\0';

    set_message (header, msgBody.in());
}




//
// Returns the internal buffer coming from a FlushArea
//
const char* Message::get_buffer() const
    throw ()
{
    return  m_pBuffer;  
}   


//
// Returns the body message buffer coming from a FlushArea
//
const char* Message::get_bodyBuffer() const
    throw ()
{
    return  m_pBodyBuffer;  
}   

//
// Returns the size of the body message buffer coming from a FlushArea
//
size_t Message::get_bodyBufferSize() const
    throw ()
{
    return  m_szBodyBuffer; 
}   

//
// Returns the internal header defining the origin of the message
//
const Message::MessageHeaderRaw& Message::get_header() const
    throw ()
{
    return  m_messageHeaderRaw; 
}   




//
// A FlushArea uses this method to assign raw memory to this message
//
void Message::set_buffer (char* pBuffer, size_t szBuffer)
    throw ()
{
    m_pBuffer = pBuffer;    
    m_pBuffer[0] = '\0';
    m_szTotalBuffer = szBuffer;
}   



} // End namespace Trace
} // End namespace Cdmw

