/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#include "Foundation/common/Options.hpp"
#include "Foundation/common/String.hpp"
#include "Foundation/common/Exception.hpp"
#include "Foundation/common/Assert.hpp"
#include "Foundation/common/Locations.hpp"

#include "Foundation/orbsupport/CORBA.hpp"

#include "Foundation/ossupport/OS.hpp"
#include "Foundation/osthreads/ThreadHandle.hpp"
#include "Foundation/orbsupport/OrbSupport.hpp"
#include "Foundation/orbsupport/StrategyList.hpp"
#include "Foundation/logging/LogManager.hpp"

#include <ConfAndPlug/cdmwinit/CdmwInterface.hpp>
#include <ConfAndPlug/cdmwinit/InitUtils.hpp>

#include <Time/clockservice/CdmwCosClock.hpp>
#include <Foundation/commonsvcs/time/Util.hpp>

/* This is a main for a validation test : in validation conditions
   we expect to be compiled with CDMW_TRACE_LEVEL 6 if nothing else
   is stated (like in V-TRC-007)
#if defined(CDMW_TRACE_LEVEL)

#if CDMW_TRACE_LEVEL < 6
#undef CDMW_TRACE_LEVEL
#define CDMW_TRACE_LEVEL 6
#endif

#else
#define CDMW_TRACE_LEVEL 6
#endif
*/


#include "Foundation/logging/Logger.hpp"

#include "SystemMngt/platforminterface/PlatformInterface.hpp"
#include "SystemMngt/platforminterface/ServiceNames.hpp"

#include "Repository/repositoryinterface/RepositoryInterface.hpp"

#include "TraceAndPerf/tracelibrary/InitUtils.hpp"
#include "TraceAndPerf/tracelibrary/Trace.hpp"

#include "TraceAndPerf/performancelibrary/InitUtils.hpp"
#include "TraceAndPerf/performancelibrary/PerformanceMngr.hpp"

#include "traceclient/MyProcessBehaviour.hpp"

#include <cstdlib>
#include <string>
#include <exception>
#include <sstream>
#include <iostream>
#include <fstream>




const int POA_THREAD_POOL_SIZE = 5;

namespace
{
    using namespace Cdmw::OsSupport;
    using namespace Cdmw::OrbSupport;
    using namespace Cdmw::PlatformMngt;

//    const std::string TRACE_MSG_OPTION = "--trace-msg";
    const std::string TRACE_NBR_OPTION = "--trace-nbr";
    const std::string TRACE_REPEAT_OPTION = "--trace-repeat";
    const std::string TRACE_MULTI_DOMAIN_OPTION = "--trace-multi_domain";
    const std::string TRACE_SEQ1_OPTION = "--trace-seq1";
    const std::string TRACE_SEQ2_OPTION = "--trace-seq2";
//    const std::string COLLECTOR_NAME_OPTION = "--collector-name";
    
    const std::string PERF_CALL_OPTION = "--perf-call";
    const std::string INIT_FEDERATED_CLOCK = "--init-federated-clock";
//    const std::string PERF_ACTIVATION_OPTION = "--perf-activate";
//    const std::string PERF_LOG_FILE_OPTION = "--perf-log-file";
    
       
    
    const int SUCCESS = 0;
    const int FAILURE = 1;
    
    std::string process_name;
    
}


void exceptionHandler(void)
{
    std::cout << "Unexpected Exception : Test failed" << std::endl;
}


void exitHandler(void)
{
    std::cout << "Process '" << process_name << "' exited" << std::endl;
}


static void
usage(std::ostream & os, const std::string &program_name)
{
    os << "Usage:\n"
       << program_name << " options\n"
       << "\n"
       << "Options:\n"
       << "--help                             Display this information.\n"
//       << "[--trace-msg=<message>]            Message to trace.\n"
       << "[--trace-nbr=<number>]             Number of messages to trace.\n"
       << "[--trace-repeat]                   generate repeated message. \n"
       << "[--trace-multi_domain]             generate multi domain messages. \n"
       << "[--trace-seq1=<number>]            sequence nbr to deactivate 1st set of messages. \n"
       << "[--trace-seq2=<number>]            sequence nbr to activate 2nd set of messages. \n"
       << "[--init-federated-clock=<number>]  Number of seconds representing the operational time to be set. \n"
//       << "[--collector-name=<name>]          Name of trace collector.\n"
       << "[--perf-call]                      Performance probe calling.\n"
//       << "[--perf-activate]                  Performance measure activation.\n"
//       << "[--perf-log-file=<file>]           Performance log file. \n"
       << std:: endl;
}
    
std::string timestamp_to_string(Cdmw::OsSupport::OS::Timeval tv, const char* format)
{
   // format date
   char time_buf[128];
   Cdmw::OsSupport::OS::get_fmt_time (time_buf, sizeof(time_buf), 
                                tv.seconds,
                                const_cast<char*>(format), Cdmw::OsSupport::OS::LOCAL_TIME);
   
   return std::string(time_buf);
}

void init_controlled_clock(CORBA::ORB_ptr a_orb, unsigned long long num_sec)
{
     CDMW_ASSERT(!CORBA::is_nil(a_orb) );

     CORBA::Object_var obj = a_orb->resolve_initial_references("ClockService");
     if (CORBA::is_nil(obj.in()))
     {
         CDMW_THROW2(Cdmw::BadParameterException, "Unable to resolve ClockService", "NULL");
     }
    
     CosClockService::ClockCatalog_var clock_catalog = 
             CosClockService::ClockCatalog::_narrow(obj.in());
     if (CORBA::is_nil(clock_catalog.in()))
     {
         CDMW_THROW2(Cdmw::BadParameterException, "Unable to resolve ClockCatalog", "NULL");
     }
     
     char clock_name[] = "FederatedControlledClock";
     CosClockService::ClockCatalog::ClockEntry_var catalog_entry = clock_catalog->get_entry(clock_name);         
     Cdmw::clock::ControlledClockEx_ptr ctrl_clock = Cdmw::clock::ControlledClockEx::_narrow(catalog_entry->subject.in());
     if (CORBA::is_nil(ctrl_clock))
     {
         CDMW_THROW2(Cdmw::BadParameterException, "Unable to narrow the clock's name", "NULL");
     }
    
     Cdmw::OsSupport::OS::Timeval time_val = Cdmw::OsSupport::OS::get_time();
     std::cout << "-> Setting \"" << clock_name << "\" at " << timestamp_to_string(time_val,"%a, %d %b %Y %H:%M:%S %z") << std::endl;      
     time_val.seconds = num_sec;
     time_val.microseconds = 0L;
    
     TimeBase::TimeT to = Cdmw::CommonSvcs::Time::Util::Timeval_2_TimeT(time_val);
     ctrl_clock->set_rate(10.0);
     ctrl_clock->set(to);    
    
     std::cout << "... to " << timestamp_to_string(time_val,"%a, %d %b %Y %H:%M:%S %z") << std::endl;      
     std::cout << "~+~+~+~+~+~ time_val.seconds=" << time_val.seconds 
               << ", ratio=" << ctrl_clock->get_rate() << " ~+~+~+~+~+~" << std::endl;      
}

int main(int argc, char* argv[])
{
   std::string application_name;    
    
    
   // the return code 
   int ret_code = 0;

   std::set_unexpected(&exceptionHandler);

   ::atexit(&exitHandler);

   CORBA::ORB_var orb;

   //
   // help argument
   //
   if (argc > 1)
   {
      if(strcmp(argv[1], "--help") == 0)
       {
         usage(std::cout, argv[0]);
         return ret_code;
      }
   }
    
   using Cdmw::Logging::Logger;
   Logger* logger = NULL;
   try
   {
      // initialize the ORB.
      Cdmw::OrbSupport::StrategyList orb_strategies;
      orb_strategies.add_OrbThreaded();
      orb_strategies.add_PoaThreadPool(POA_THREAD_POOL_SIZE);

      // Federated Controlled Clock Management (1)
      std::string federatedControlledClockInitialization = OS::get_option_value (argc, argv, INIT_FEDERATED_CLOCK);
      if (federatedControlledClockInitialization != "no")
      {
          orb_strategies.add_multicast();
          std::cout << "~+~+~ multicast added to orb_strategies." << std::endl;
      }

      orb = Cdmw::OrbSupport::OrbSupport::ORB_init(argc, argv, orb_strategies);

      // get the root POA.
      CORBA::Object_var obj = orb -> resolve_initial_references("RootPOA");
      PortableServer::POA_var rootPOA = PortableServer::POA::_narrow(obj.in());

      // activate the root POA manager.
      PortableServer::POAManager_var poaManager = rootPOA -> the_POAManager();
      poaManager -> activate();

      std::cout << "~+-~+-~+-~+-~+-~ Making sure that the TraceCollector is fully initialized : Sleeping for 4 sec..." << std::endl;
      Cdmw::OsSupport::OS::sleep (4000);      
      std::cout << "~+-~+-~+-~+-~+-~ done." << std::endl;

      logger = Logger::Get_Logger("TEST", "TEST");

      // Federated Controlled Clock Management (2)
      if (federatedControlledClockInitialization != "no")
      {
          unsigned long long num_sec=atol(federatedControlledClockInitialization.c_str());
          Cdmw::clock::CosClock::init(orb.in(), argc, argv);
          init_controlled_clock(orb.in(), num_sec);
          std::cout << "~+~+~ clock service successfully initialized." << std::endl;
      }

      //
      // get number of Trace Messages
      //
      long traceCount = -1;
        
      std::string traceCountOption = OS::get_option_value (argc, argv, TRACE_NBR_OPTION);
      if (traceCountOption == "yes")
      {
         std::cout << "bad value for " << TRACE_NBR_OPTION.c_str() << "=<number> option\n";
         std::cout << std::endl;
         return FAILURE;
      }
      else if (traceCountOption != "no")
      {
          std::istringstream traceCount_strm(traceCountOption);
         traceCount_strm >> traceCount;
         if (traceCount_strm.fail()) 
         {
            std::cout << "bad value for " << TRACE_NBR_OPTION.c_str() << "=<number> option\n";
            std::cout << std::endl;
            return FAILURE;
         }
      }
        
      //
      // get trace repeat mode
      //
      bool traceRepeatMode = false;
        
      std::string traceRepeatModeOption = OS::get_option_value (argc, argv, TRACE_REPEAT_OPTION);
      if (traceRepeatModeOption == "yes")
      {
         traceRepeatMode = true;
      }
      
      //
      // get Performance probe call option
      //
      bool perf_probe_call = false;
      std::string perf_probe_call_option = OS::get_option_value (argc, argv, PERF_CALL_OPTION);
      if (perf_probe_call_option == "yes")
      {
         perf_probe_call = true;
      }
      //
      // get trace multi domain mode
      //
      bool traceMultiDomainMode = false;
        
      std::string traceMultiDomainModeOption = OS::get_option_value (argc, argv, TRACE_MULTI_DOMAIN_OPTION);
      if (traceMultiDomainModeOption == "yes")
      {
         traceMultiDomainMode = true;
      }

      //
      // get trace sequence counter 1
      //
      long traceCountSeq1 = -1;
        
      std::string traceCountSeq1Option = OS::get_option_value (argc, argv, TRACE_SEQ1_OPTION);
      if (traceCountSeq1Option == "yes")
      {
         std::cout << "bad value for " << TRACE_SEQ1_OPTION.c_str() << "=<number> option\n";
         std::cout << std::endl;
         return FAILURE;
      }
      else if (traceCountSeq1Option != "no")
      {
          std::istringstream traceCount_strm(traceCountSeq1Option);
         traceCount_strm >> traceCountSeq1;
         if (traceCount_strm.fail()) 
         {
            std::cout << "bad value for " << TRACE_SEQ1_OPTION.c_str() << "=<number> option\n";
            std::cout << std::endl;
            return FAILURE;
         }
      }
        
      //
      // get trace sequence counter 2
      //
      long traceCountSeq2 = -1;
        
      std::string traceCountSeq2Option = OS::get_option_value (argc, argv, TRACE_SEQ2_OPTION);
      if (traceCountSeq2Option == "yes")
      {
         std::cout << "bad value for " << TRACE_SEQ2_OPTION.c_str() << "=<number> option\n";
         std::cout << std::endl;
         return FAILURE;
      }
      else if (traceCountSeq2Option != "no")
      {
          std::istringstream traceCount_strm(traceCountSeq2Option);
         traceCount_strm >> traceCountSeq2;
         if (traceCount_strm.fail()) 
         {
            std::cout << "bad value for " << TRACE_SEQ2_OPTION.c_str() << "=<number> option\n";
            std::cout << std::endl;
            return FAILURE;
         }
      }

      std::string fooStr("");

      // creates the process behaviour
      MyProcessBehaviour * pProcBehaviour = new MyProcessBehaviour( orb.in(), 
                                                                    fooStr, 
                                                                    traceCount,
                                                                    traceCountSeq1,
                                                                    traceCountSeq2,
                                                                    traceRepeatMode,
                                                                    traceMultiDomainMode,
                                                                    perf_probe_call,
                                                                    logger);
 

      // call generated CDMW_init().
      Cdmw::CdmwInit::CDMW_init(orb.in(), argc, argv, pProcBehaviour);

      // register a null fatal handler to avoid the process to be killed by the supervision
      // following a call to a CDMW_FATAL message. Do not delete the return handler since
      // it will be deleted by CDMW_cleanup().
      (void*) Cdmw::Logging::LogManager::Instance()->register_fatal_handler(0);

      // get the process name and application name.
      process_name =
            Cdmw::CdmwInit::InitUtils::get_cdmw_process_name();
      application_name =
            Cdmw::CdmwInit::InitUtils::get_cdmw_application_name();


      OS::ProcessId processId = OS::get_processId();

      std::cout << "Application name : "
                << application_name.c_str() << std::endl;

      std::cout << "Process name : " 
                << process_name.c_str() 
                << " Process id : "
                << processId << std::endl;


        

      Cdmw::Performance::PerformanceMngr::deactivate();

      std::string baseMsg = "process <" + application_name + "/" + process_name + ">" ;
        
      // ===================================================
      // trace a message (don't use CDMW_TRACE in main)
      // ===================================================
            
      std::cout << "program started and waiting CORBA messages" << std::endl;
            
            
      std::string msg = baseMsg + "started and waiting CORBA messages"; 
            
                                      
      CDMW_LOG (logger, Cdmw::Logging::INFO, msg.c_str());
        
      // start orb
      orb->run();            
   }
            
   catch (const Cdmw::Exception & ex) 
   {
      std::cerr << ex.what() << std::endl;
      ret_code = FAILURE;
   }
   catch (const CORBA::Exception& e)
   {  
      std::cerr << "FAILURE : CORBA exception raised :" << std::endl
                << e._name() << std::endl;
      ret_code = FAILURE;
   }
   catch(const std::exception &e)
   {
      std::cerr << "FAILURE : Exception raised : " << std::endl
                << e.what() << std::endl;
      ret_code = FAILURE;
   }
 

   if(!CORBA::is_nil(orb.in()))
   {
      try 
      {
         Cdmw::CdmwInit::CDMW_cleanup(orb.in());
         Cdmw::OrbSupport::OrbSupport::ORB_cleanup(orb.in());
         orb->destroy();
      }
      catch(const CORBA::Exception& ex)
      {           
         std::cerr << ex << std::endl;
         ret_code = FAILURE;
      } 
   }
        
   return ret_code;
}


