/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO 
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2009. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


package cdmw.ccm.cif;

import static cdmw.logging.Logger.*;
import static cdmw.logging.FunctionLogger.*;


/**
 * This class implement the org.omg.Components.CCMContext interface.
 * It will be inherited by all components context implementation.
 *
 */
public class CCMContextExtImpl
    extends org.omg.CORBA.LocalObject
    implements com.thalesgroup.CdmwComponents.CCMContextExt {    
    
    /*
     * List of forbidden service identifiers
     */
    private static String[] UNAVAILABLE_SERVICE_IDENTIFIERS = {"RootPOA", 
                                                               "POACurrent", 
                                                               "ORBPolicyManager",
                                                               "PolicyCurrent",
                                                               "PICurrent"};

    /*
     * logger to be used in CCMContext
     */
    protected com.thalesgroup.CdmwLogging.Logger logger;
                                                               
    /*
     * container context
     */
    private org.omg.Components.CCMContext context;
    
    /*
     * Services map
     */
    private java.util.Map<String, org.omg.CORBA.Object> servicesMap;
     

    public CCMContextExtImpl(org.omg.Components.CCMContext ctx,
                             com.thalesgroup.CdmwLogging.LoggerFactory loggerFactory) 
    {
        // create logger with domain for ccmcif
        this.logger = loggerFactory.get_logger(
            cdmw.ccm.common.CCMLoggers.CCMCIF_LOG_DOMAIN);
            
        CDMW_LOG_FUNCTION(logger, "");
            
        this.context = ctx;
        this.servicesMap = new java.util.HashMap<String, org.omg.CORBA.Object>();
    }

    //
    // IDL:omg.org/Components/CCMContext/get_caller_principal:1.0
    //
    public org.omg.SecurityLevel2.Credentials get_caller_principal() 
    {
        CDMW_LOG_FUNCTION(logger, "");
        return this.context.get_caller_principal();
    }

    //
    // IDL:omg.org/Components/CCMContext/get_CCM_home:1.0
    //
    public org.omg.Components.CCMHome get_CCM_home() 
    {
        CDMW_LOG_FUNCTION(logger, "");
        return this.context.get_CCM_home();
    }

    //
    // IDL:omg.org/Components/CCMContext/get_rollback_only:1.0
    //
    public boolean get_rollback_only() 
        throws org.omg.Components.IllegalState 
    {
        CDMW_LOG_FUNCTION(logger, "");
        return this.context.get_rollback_only();
    }

    //
    // IDL:omg.org/Components/CCMContext/get_user_transaction:1.0
    //
    public org.omg.Components.Transaction.UserTransaction 
    get_user_transaction() 
        throws org.omg.Components.IllegalState 
    {
        CDMW_LOG_FUNCTION(logger, "");
        return this.context.get_user_transaction();
    }

    //
    // IDL:omg.org/Components/CCMContext/is_caller_in_role:1.0
    //
    public boolean is_caller_in_role(String role) 
    {
        CDMW_LOG_FUNCTION(logger, "role="+role);
        return this.context.is_caller_in_role(role);
    }

    //
    // IDL:omg.org/Components/CCMContext/set_rollback_only:1.0
    //
    public void set_rollback_only() 
        throws org.omg.Components.IllegalState 
    {
        CDMW_LOG_FUNCTION(logger, "");
        this.context.set_rollback_only();
    }

    
    //
    // IDL:omg.org/Components/CCMContext/resolve_service_reference:1.0
    //
    public org.omg.CORBA.Object resolve_service_reference(String service_id) 
        throws org.omg.Components.CCMException
    {
        CDMW_LOG_FUNCTION(logger, "service_id="+service_id);
        // Check if service access is authorized
        for (int i=0; i<UNAVAILABLE_SERVICE_IDENTIFIERS.length ; i++) {
            if (UNAVAILABLE_SERVICE_IDENTIFIERS[i].equals(service_id)) {
                CDMW_ERROR(logger,
                    "Attempt to access unauthorized service: " + service_id,
                    new Throwable());
                org.omg.Components.CCMException ex = new org.omg.Components.CCMException(
                    org.omg.Components.CCMExceptionReason.OBJECT_NOT_FOUND);
                CDMW_LOG_FUNCTION_EXCEPTION(logger, ex);
                throw ex;
            }
        }
        
        // try to find service in ServicesMap first
        synchronized(servicesMap) {
            org.omg.CORBA.Object obj = servicesMap.get(service_id);
            if (obj != null) {
                CDMW_DEBUG(logger,
                    "Service " + service_id + " found in ServiceMap",
                    new Throwable());
                return obj;
            }
        }
        
        // call CCM2Context::resolve_initial_references()
        try {
            CDMW_DEBUG(logger,
                "Service " + service_id + " not found in ServiceMap. " +
                "Try to get it from CCM2Context.",
                new Throwable());
            com.thalesgroup.CdmwCcmContainer.CCM2Context ctx 
                = com.thalesgroup.CdmwCcmContainer.CCM2ContextHelper.narrow(this.context);
            return ctx.resolve_initial_references(service_id);
            
        } catch (com.thalesgroup.CdmwCcmContainer.CCM2ContextPackage.InvalidName in) {
            CDMW_ERROR(logger,
                "Service " + service_id + " has InvalidName !",
                new Throwable());
            org.omg.Components.CCMException ex = new org.omg.Components.CCMException(
                org.omg.Components.CCMExceptionReason.OBJECT_NOT_FOUND);
            CDMW_LOG_FUNCTION_EXCEPTION(logger, ex);
            throw ex;
        } catch (org.omg.CORBA.SystemException e) {
            CDMW_ERROR(logger,
                "Exception received getting service " + service_id + " : " + e,
                new Throwable());
            CDMW_LOG_FUNCTION_EXCEPTION(logger, e);
            throw e;
        }
    }

    
    //
    // IDL:thalesgroup.com/CdmwComponents/CCMContextExt/get_service
    //
    public org.omg.CORBA.Object get_service(String identifier)
        throws com.thalesgroup.CdmwComponents.CCMContextExtPackage.ServiceUnavailable 
    {
        // Deprectated !!
        // But implement it as delegation to resolve_service_reference()
        CDMW_WARN(logger, 
              "Operation CCMContextExt::get_service() is deprecated. " +
              "Use CCMContext::resolve_service_reference() instead.",
              new Throwable());
    
        try {
            return this.resolve_service_reference(identifier);
        } catch (org.omg.Components.CCMException e) {
            // service not found
            throw new com.thalesgroup.CdmwComponents.CCMContextExtPackage.ServiceUnavailable();
        }
    }
    
    
    // Specific Cookie implementation to ease Service search
    private class CookieImpl extends org.omg.Components.Cookie 
    {
        public CookieImpl(String cookieValue) {
            this.cookieValue = cookieValue.getBytes();
        }
        
        public String getCookieString() {
            return new String(this.cookieValue);
        }
    }
    
    //
    // IDL:omg.org/Components/ExtensionContext/install_service_reference:1.0
    //
    public org.omg.Components.Cookie 
    install_service_reference(String service_id, org.omg.CORBA.Object objref) 
        throws org.omg.Components.CCMException
    {
        CDMW_LOG_FUNCTION(logger, "service_id="+service_id);
        synchronized(servicesMap) {
            // check if service is already installed
            if (servicesMap.containsKey(service_id))
            {
                // service already installed
                CDMW_ERROR(logger, 
                    "Service " + service_id + " is already installed.",
                    new Throwable());
               org.omg.Components.CCMException ex = new org.omg.Components.CCMException(
                    org.omg.Components.CCMExceptionReason.SERVICE_INSTALLATION_ERROR);
                CDMW_LOG_FUNCTION_EXCEPTION(logger, ex);
                throw ex;
            }
        
            // add service to ServicesMap    
            servicesMap.put(service_id, objref);
            
            // return Cookie with service_id inside
            return new CookieImpl(service_id);
        }
    }
    
    //
    // IDL:omg.org/Components/ExtensionContext/uninstall_service_reference:1.0
    //
    public org.omg.CORBA.Object
    uninstall_service_reference(org.omg.Components.Cookie ck) 
        throws org.omg.Components.CCMException
    {
        CDMW_LOG_FUNCTION(logger, "");
        synchronized(servicesMap) {
            // extract service_id from Cookie
            String service_id = ((CookieImpl) ck).getCookieString();
            
            // check if service is installed
            if (!servicesMap.containsKey(service_id))
            {
                // service not installed
                CDMW_ERROR(logger, 
                    "Service " + service_id + " is not installed",
                    new Throwable());
                org.omg.Components.CCMException ex = new org.omg.Components.CCMException(
                    org.omg.Components.CCMExceptionReason.SERVICE_INSTALLATION_ERROR);
                CDMW_LOG_FUNCTION_EXCEPTION(logger, ex);
                throw ex;
            }
        
            // remove service from ServicesMap    
            return servicesMap.remove(service_id);
        }
    }
        
    
}

