/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO 
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2009. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


package cdmw.ccm.container;

import com.thalesgroup.CdmwCcmContainer.ValueTypeRegistration;

import org.omg.CORBA.portable.ValueFactory;
import org.omg.Components.Deployment.ImplEntryPointNotFound;
import cdmw.ccm.common.CCMLoggers;
import static cdmw.logging.Logger.*;
import static cdmw.logging.FunctionLogger.*;

import java.net.URLClassLoader;
import java.net.URL;

/**
 * Implementation of the Session2Context internal interface for use with
 * Session container type.
 */
public class ValueTypeInstallerImpl
    extends org.omg.CORBA.LocalObject
    implements com.thalesgroup.CdmwCcmContainer.ValueTypeInstaller {

    private final class RegistrationInfo {

        public ValueTypeRegistration registration;
        public boolean release;

        public RegistrationInfo(ValueTypeRegistration reg, boolean release) {
            this.registration = reg;
            this.release = release;
        }

    };

    private java.util.HashMap registrations = new java.util.HashMap();

    private org.omg.CORBA_2_3.ORB orb;

    /**
     * Constructor.
     *
     * @param orb A reference to the ORB pseudo-object
     */
    public ValueTypeInstallerImpl(org.omg.CORBA.ORB orb) {
        CDMW_LOG_FUNCTION(CCMLoggers.GetCcmcontainerLogger(), "");
        this.orb = (org.omg.CORBA_2_3.ORB) orb;
    }

    //
    // IDL:thalesgroup.com/CdmwCcmContainer/ValueTypeInstaller/install_valuetype_factory:1.0
    //
    public synchronized ValueTypeRegistration install_valuetype_factory(
        String repid,
        String location,
        String entrypoint) 
    {
        CDMW_LOG_FUNCTION(CCMLoggers.GetCcmcontainerLogger(), 
            "repid="+repid+"  location="+location+"  entrypoint="+entrypoint);

        CDMW_DEBUG(CCMLoggers.GetCcmcontainerLogger(), 
            "Install Valuetype factory for " + repid +
            " from " + location + " using " + entrypoint,
            new Throwable());
            
        ValueTypeRegistration reg = null;
        RegistrationInfo regInfo = (RegistrationInfo) registrations.get(repid);
        if (regInfo != null) {
            // Already registered through the installer.
            // Return existing registration object.
            reg = regInfo.registration;
            
            CDMW_DEBUG(CCMLoggers.GetCcmcontainerLogger(), 
                "Valuetype factory for " + repid + " already registered. " +
                " Returning existing handle",
                new Throwable());
            
        } else {
            // Register the valuetype factory within the ORB.
            ValueFactory oldValuetypeFac = this.orb.lookup_value_factory(repid);

            if (oldValuetypeFac == null) {

                // Was not already registered outside of the installer
                // Leave previous valuetype factory and create a registration object
                // with no unregistration if registered outside of the installer.
                java.lang.reflect.Method factoryMethod = null;
                String classname = "";
                try {
                    // get the classname and the method name
                    int ind = entrypoint.lastIndexOf('.');
                    if (ind == -1) {
                        // entrypoint is not a method name
                        CDMW_ERROR(CCMLoggers.GetCcmcontainerLogger(), 
                            "Invalid entrypoint for valuetypefactory " +
                            repid + " : " + entrypoint,
                            new Throwable());
                        org.omg.CORBA.BAD_PARAM ex = new org.omg.CORBA.BAD_PARAM(
                            "Invalid entrypoint for valuetypefactory " +
                            repid + " : " + entrypoint);
                        CDMW_LOG_FUNCTION_EXCEPTION(CCMLoggers.GetCcmcontainerLogger(), ex);
                        throw ex;
                    }
                    classname = entrypoint.substring(0, ind);
                    String methodname = entrypoint.substring(ind + 1);
                    // load the class
                    Class factory = loadClass(classname, location);
                    // get the method object
                    java.lang.reflect.Method[] methods = factory.getMethods();
                    for(int i=0; i<methods.length; i++) {
                        if (methods[i].getName().equals(methodname)) {
                            factoryMethod = methods[i];
                            break;
                        }
                    }
                } catch(IndexOutOfBoundsException ioobe) {
                    CDMW_ERROR(CCMLoggers.GetCcmcontainerLogger(), 
                        "Entrypoint operation "+entrypoint+
                        " not found in class "+classname,
                        new Throwable());
                    org.omg.CORBA.BAD_PARAM ex = 
                        new org.omg.CORBA.BAD_PARAM(ioobe.toString());
                    CDMW_LOG_FUNCTION_EXCEPTION(CCMLoggers.GetCcmcontainerLogger(), ex);
                    throw ex;
                } catch(LinkageError le) {
                    CDMW_ERROR(CCMLoggers.GetCcmcontainerLogger(), 
                        "LinkageError loading entry point "+entrypoint+" : "+
                        le.getMessage(),
                        new Throwable());
                    le.printStackTrace();
                    org.omg.CORBA.BAD_PARAM ex =
                        new org.omg.CORBA.BAD_PARAM(le.toString());
                    CDMW_LOG_FUNCTION_EXCEPTION(CCMLoggers.GetCcmcontainerLogger(), ex);
                    throw ex;
                } catch(ClassNotFoundException cnfe) {
                    CDMW_ERROR(CCMLoggers.GetCcmcontainerLogger(), 
                        "Class "+classname+" for entry point "+
                        entrypoint+" not found",
                        new Throwable());
                    org.omg.CORBA.BAD_PARAM ex =
                        new org.omg.CORBA.BAD_PARAM(cnfe.toString());
                    CDMW_LOG_FUNCTION_EXCEPTION(CCMLoggers.GetCcmcontainerLogger(), ex);
                    throw ex;
                }

                // Get Valuetype Factory
                ValueFactory valuetypeFac = null;
                try {
                    valuetypeFac =
                        (ValueFactory) factoryMethod.invoke(null, (java.lang.Object[]) null);
                } catch(IllegalAccessException iae) {
                    CDMW_ERROR(CCMLoggers.GetCcmcontainerLogger(), 
                        "Cannot access to entry point '" + entrypoint + "' :"+
                        iae.getMessage(),
                        new Throwable());
                    org.omg.CORBA.BAD_PARAM ex =
                        new org.omg.CORBA.BAD_PARAM(iae.toString());
                    CDMW_LOG_FUNCTION_EXCEPTION(CCMLoggers.GetCcmcontainerLogger(), ex);
                    throw ex;
                } catch(IllegalArgumentException iae) {
                    CDMW_ERROR(CCMLoggers.GetCcmcontainerLogger(), 
                        "Entry point '" + entrypoint + 
                        "' shouldn't have any arguments",
                        new Throwable());
                    org.omg.CORBA.BAD_PARAM ex =
                        new org.omg.CORBA.BAD_PARAM(iae.toString());
                    CDMW_LOG_FUNCTION_EXCEPTION(CCMLoggers.GetCcmcontainerLogger(), ex);
                    throw ex;
                } catch(java.lang.reflect.InvocationTargetException ite) {
                    CDMW_ERROR(CCMLoggers.GetCcmcontainerLogger(), 
                        "Exception invoking entry point '" + entrypoint + 
                        "' : "+ite.getMessage(),
                        new Throwable());
                    ite.printStackTrace();
                    org.omg.CORBA.BAD_PARAM ex =
                        new org.omg.CORBA.BAD_PARAM(ite.toString());
                    CDMW_LOG_FUNCTION_EXCEPTION(CCMLoggers.GetCcmcontainerLogger(), ex);
                    throw ex;
                }

                if (valuetypeFac == null) {
                    CDMW_ERROR(CCMLoggers.GetCcmcontainerLogger(), 
                        "Valuetype Factory created by '" + entrypoint + 
                        "' is null",
                        new Throwable());
                    org.omg.CORBA.BAD_PARAM ex = new org.omg.CORBA.BAD_PARAM(
                        com.thalesgroup.CdmwDeployment.UNSUPPORTED_COMPONENT_TYPE.value,
                        org.omg.CORBA.CompletionStatus.COMPLETED_NO);
                    CDMW_LOG_FUNCTION_EXCEPTION(CCMLoggers.GetCcmcontainerLogger(), ex);
                    throw ex;
                }

                this.orb.register_value_factory(repid, valuetypeFac);
            }
            RegistrationInfo info = new RegistrationInfo(
                new ValueTypeRegistrationImpl(this, repid), true);
            reg = info.registration;
            registrations.put(repid, info);
        }
        return reg;

    }

    public synchronized boolean unregister_valuetype_factory(String repid) {
        CDMW_LOG_FUNCTION(CCMLoggers.GetCcmcontainerLogger(), 
            "repid="+repid);

        boolean found = false;
        RegistrationInfo regInfo = (RegistrationInfo) registrations.get(repid);
        if (regInfo != null) {
            if (regInfo.release) {
                this.orb.unregister_value_factory(repid);
            }
            registrations.remove(repid);
            found = true;
        }
        return found;

    }
    
    /**
     * Loads the class with the given name. If the classname is not found 
     * in the current classpath, then the location param is used to try 
     * to load it from the specified jar file.
     * 
     * @param classname The name of the class to load
     * @param location Optional pathname to an external jar file
     * @return The resulting Class object
     * @exception ClassNotFoundException if the classname is not found
     */
    private synchronized Class loadClass(String classname, String location) 
        throws ClassNotFoundException 
    {
        CDMW_LOG_FUNCTION(CCMLoggers.GetCcmcontainerLogger(), 
            "classname="+classname+"  location="+location);
        // try to load the jar at the specified location
        CDMW_INFO(CCMLoggers.GetCcmcontainerLogger(), 
           "Try to load " + classname + " from jar file " + location,
            new Throwable());
        URL[] urls = new URL[1];
        try {
            urls[0] = new URL(new URL("file://"), location);
            URLClassLoader classLoader = URLClassLoader.newInstance(urls);
            
            return classLoader.loadClass(classname);
            
        } catch (java.net.MalformedURLException mue) {
            CDMW_WARN(CCMLoggers.GetCcmcontainerLogger(), 
               "MalformedURLException: " + mue.getMessage(),
                new Throwable());
        
        } catch(ClassNotFoundException cnfe) {
            CDMW_WARN(CCMLoggers.GetCcmcontainerLogger(), 
               "Class " + classname + " not found in jar file " + location,
                new Throwable());
        }
         
        try {
            // try to find the class in the current classpath
            CDMW_INFO(CCMLoggers.GetCcmcontainerLogger(), 
               "Try to load " + classname + " from CLASSPATH",
                new Throwable());
            return  Class.forName(classname);
            
        } catch(ClassNotFoundException cnfe) {
            CDMW_ERROR(CCMLoggers.GetCcmcontainerLogger(), 
               "Class " + classname + " not found, neither in " + location +
               " neither in CLASSPATH",
                new Throwable());
            throw cnfe;
        }
    }

}

