/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


package cdmw.cdmwinit;

import cdmw.common.Assert;
import cdmw.common.Options;
import cdmw.common.Locations;
import cdmw.common.BadOrderException;
import cdmw.common.BadParameterException;
import cdmw.common.InternalErrorException;

import cdmw.ossupport.OS;
import cdmw.orbsupport.CORBASystemExceptionWrapper;
import cdmw.orbsupport.ORBSupport;
import cdmw.orbsupport.ORBSupport;
import cdmw.orbsupport.PolicyList;
import cdmw.orbsupport.StrategyList;
import cdmw.orbsupport.ExceptionMinorCodes;
import cdmw.orbsupport.ExceptionStrings;

import cdmw.lifecycle.ThreadPoolPolicy;
import cdmw.commonsvcs.naming.NamingInterface;
import cdmw.namingandrepository.RepositoryInterface;

import cdmw.platformmngt.ServiceNames;
import cdmw.platformmngt.PlatformInterface;
//import com.thalesgroup.CdmwPlatformMngt.InitUtils;
//import cdmw.platformmngt.*;

import com.thalesgroup.CdmwPlatformMngt.ProcessCallback;
import com.thalesgroup.CdmwPlatformMngt.ProcessCallbackHelper;
import com.thalesgroup.CdmwNamingAndRepository.Repository;
import com.thalesgroup.CdmwNamingAndRepository.RepositoryHelper;

import org.omg.CORBA.Policy;
import org.omg.CORBA.CompletionStatus;

import org.omg.PortableServer.POA;
import org.omg.PortableServer.Servant;


/**
 * This class provides helper functions to be used by the CDMW generated
 * CdmwInterface file.
 *
 * @see CdmwInterface
 *
 */
public class InitUtils {

    /**
     * Name of the POA root of CDMW LifeCycle framework
     */
    public final static String FACTORIES_POA_NAME = "FactoriesPOA";
    
    /**
     * Default Thread pool size of the CDMW factories POA (above)
     */
    public final static int FACTORIES_POA_THREAD_POOL_SIZE = 5;

    /**
     * Default Thread pool size of the CDMW root POA
     */
    public final static int CDMW_ROOT_POA_THREAD_POOL_SIZE = 5;
    
    /**
     * CDMW.INIT logger
     */
    private static cdmw.logging.Logger CdmwInitLogger = null;

    /**
     * CDMW Factories Servant Activator
     */
    static class CDMWInitServantActivator 
        extends org.omg.CORBA.LocalObject
        implements org.omg.PortableServer.ServantActivator {

        /**
         * This method should not be called.
         *
         * @param oid
         * @param poa
         */
        public Servant incarnate(byte[] oid, POA poa)
            throws org.omg.PortableServer.ForwardRequest {
                // Should not happen (at this stage!)
                Assert.check(false);
                return null; // TODO: Maybe raise org.omg.CORBA.TRANSIENT()
        }

        /**
         * The main purpose of this operation is to provide a hook
         * that will be called by the ORB during its shutdown phase
         * to etherealize CDMW factories objects.
         *
         * @param oid
         * @param poa
         * @param servant
         * @param cleanup_in_progress
         * @param remaining_activations
         */
        public void
        etherealize(byte[] oid,
                    POA poa,
                    Servant servant,
                    boolean cleanup_in_progress,
                    boolean remaining_activations) {
            // nothing to do in Java
        }
    }

    
    /**
     * Returns the Logger for CDMW.INIT domain
     *
     * @return the logger
     */
    public static synchronized cdmw.logging.Logger Get_Logger()
    {
        if (CdmwInitLogger == null) { 
            CdmwInitLogger = cdmw.logging.Logger.Get_Logger("CDMW.INIT");
        }
        return CdmwInitLogger;
    }
    
    
    /**
     * desc
     * 
     * @param orb  A reference to the ORB pseudo-object
     * @param args Arguments passed to the program
     * @param proc The process object the platform management has to manage
     * @exception CORBASystemExceptionWrapper
     */ 
    public static void initPlatformInterface(org.omg.CORBA.ORB orb, 
        String[] args, com.thalesgroup.CdmwPlatformMngt.ProcessDelegate proc) 
        throws CORBASystemExceptionWrapper {
        
        // if System port is not defined as an option, try to get it from args
        if (System.getProperty(ORBSupport.SYSTEM_PORT_PROPERTY_NAME) == null) 
        {
            // get System port option, and set it as property (if present)
            String systemPortOption = OS.getOptionValue(args,
                Options.SYSTEM_PORT_OPTION);
            int systemPortNumber = 0;
            int minor;
            if ( !systemPortOption.equals("no") ) {
                minor = ExceptionMinorCodes.BAD_PARAMInvalidPortNumber;
                if ( !systemPortOption.equals("yes") ) {
                    try {
                        int port =
                            Integer.valueOf(systemPortOption).intValue();
                        if ( port > ORBSupport.MIN_PORT_NUMBER && 
                             port < ORBSupport.MAX_PORT_NUMBER ) 
                        {
                            System.setProperty(ORBSupport.SYSTEM_PORT_PROPERTY_NAME,
                                               systemPortOption);
                        } else {
                            minor = ExceptionMinorCodes.BAD_PARAMInvalidPortNumber;
                            throw new CORBASystemExceptionWrapper(
                                new org.omg.CORBA.BAD_PARAM(
                                    ExceptionStrings.BAD_PARAM_reasons[minor],
                                    minor, CompletionStatus.COMPLETED_NO),
                                "invalid port number");
                        }
                    }
                    catch(NumberFormatException nfe) {
                        minor = ExceptionMinorCodes.BAD_PARAMInvalidPortNumber;
                        throw new CORBASystemExceptionWrapper(
                            new org.omg.CORBA.BAD_PARAM(
                                ExceptionStrings.BAD_PARAM_reasons[minor],
                                minor, CompletionStatus.COMPLETED_NO),
                            "invalid port number");
                    }
                } else {
                    minor = ExceptionMinorCodes.BAD_PARAMInvalidPortNumber;
                    throw new CORBASystemExceptionWrapper(
                        new org.omg.CORBA.BAD_PARAM(
                            ExceptionStrings.BAD_PARAM_reasons[minor],
                            minor, CompletionStatus.COMPLETED_NO),
                        "invalid port number");
                }
            } // else ignore
        }
        
        try {
            PlatformInterface.setup(orb, args);
        
            if (proc == null) {
                throw new CORBASystemExceptionWrapper(
                    new org.omg.CORBA.BAD_PARAM(
                        ExceptionMinorCodes.BAD_PARAMNilObjectReference, 
                        CompletionStatus.COMPLETED_NO),
                    "nil Process object reference");
            }
        
        cdmw.platformmngt.InitUtils.registerValuetypeFactories(orb);
                        
            // Acknowledge the creation of the process
            PlatformInterface.acknowledgeCreation(proc);
        } catch (BadOrderException boe) {
           cdmw.logging.Logger.CDMW_WARN(Get_Logger(), 
                                         "cdmw.common.BadOrderException received:" +
                                         boe.what(),
                                         new Throwable());
            throw new CORBASystemExceptionWrapper(
                new org.omg.CORBA.BAD_INV_ORDER(
                    ExceptionMinorCodes.BAD_INV_ORDERInitAlreadyDone, 
                    CompletionStatus.COMPLETED_NO),
                boe.what());
        } catch (BadParameterException bpe) {
           cdmw.logging.Logger.CDMW_WARN(Get_Logger(), 
                                         "cdmw.common.BadParameterException received:" +
                                         bpe.what(),
                                         new Throwable());
            throw new CORBASystemExceptionWrapper(
                new org.omg.CORBA.BAD_PARAM(
                    ExceptionMinorCodes.BAD_PARAMProcessCallbackReferenceMissing, 
                    CompletionStatus.COMPLETED_NO),
                bpe.what());
        } catch(org.omg.CORBA.SystemException se) {
           cdmw.logging.Logger.CDMW_WARN(Get_Logger(), 
                                         "SystemException received:" +
                                         se.getMessage(),
                                         new Throwable());
           throw new CORBASystemExceptionWrapper(se, se.getMessage());
        }
        
    }
    
    /**
     * Retrieves the CDMW ProcessCallback's object reference from
     * Program command line arguments.
     *
     * @param orb  A reference to the ORB pseudo-object
     * @param args Arguments passed to the program
     * @return A reference to the CDMW ProcessCallback
     * @exception CORBASystemExceptionWrapper A wrapped CORBA system exception
     * The following CDMW minor codes detail the reason:
     * <UL><LI><B>BAD_PARAM</B> when the Process Callback's reference 
     * is missing from the command line arguments; and
     * <LI><B>BAD_PARAM</B> when the input IOR is nil 
     * or does not denote an Process Callback.</UL>
     */
    public static ProcessCallback
        getCdmwProcessCallback(org.omg.CORBA.ORB orb, String[] args)
        throws CORBASystemExceptionWrapper {

        // Get Reference of the Process Callback
        String agentIor
            = OS.getOptionValue(args,Options.CALLBACK_IOR_OPTION);
        ProcessCallback processCallback = null;

        if ( !agentIor.equals("no") && !agentIor.equals("yes") ) {
            try {
                org.omg.CORBA.Object obj = orb.string_to_object(agentIor);
                processCallback = ProcessCallbackHelper.narrow(obj);
            } catch (Exception e) {
               cdmw.logging.Logger.CDMW_WARN(Get_Logger(), 
                                             "Cannot retrieve processCallbackIOR: " +
                                             cdmw.orbsupport.ORBSupport.exceptionToString(e),
                                             new Throwable());
                // Cannot proceed with no agent Ior
                throw new CORBASystemExceptionWrapper(
                    new org.omg.CORBA.BAD_PARAM(
                        ExceptionMinorCodes.BAD_PARAMProcessCallbackReferenceMissing,
                        CompletionStatus.COMPLETED_NO),
                    e.getMessage());
            }
        } else {
           // Cannot proceed with no processCallback IOR
           cdmw.logging.Logger.CDMW_WARN(Get_Logger(), 
                                         "Cannot proceed with no processCallback IOR",
                                         new Throwable());
           throw new CORBASystemExceptionWrapper(
              new org.omg.CORBA.BAD_PARAM(
                 ExceptionMinorCodes.BAD_PARAMProcessCallbackReferenceMissing,
                 CompletionStatus.COMPLETED_NO),
              "Cannot proceed with no processCallback IOR");
        }
        if (processCallback==null) {
           cdmw.logging.Logger.CDMW_WARN(Get_Logger(), 
                                         "Nil processCallback object reference",
                                         new Throwable());
           throw new CORBASystemExceptionWrapper(
              new org.omg.CORBA.BAD_PARAM(
                 ExceptionMinorCodes.BAD_PARAMNilObjectReference,
                 CompletionStatus.COMPLETED_NO),
              "nil processCallback object reference");
        }
        return processCallback;

    }
    
    /**
     * Returns the reference of the RootPOA pseudo object.
     *
     * @param orb A reference to the ORB pseudo-object
     * @return A reference to the RootPOA pseudo object
     * @exception CORBASystemExceptionWrapper Any wrapped CORBA system exception
     */
    public static POA getRootPOA(org.omg.CORBA.ORB orb) 
        throws CORBASystemExceptionWrapper {

        Assert.check(orb!=null);
        POA root = null;

        try {
            // Resolve the Root POA
            org.omg.CORBA.Object obj
                = orb.resolve_initial_references("RootPOA");
            root = org.omg.PortableServer.POAHelper.narrow(obj);
        } catch (org.omg.CORBA.ORBPackage.InvalidName in) {
            // Raised by resolve_initial_references("RootPOA")
            // shouldn't happen
           cdmw.logging.Logger.CDMW_WARN(Get_Logger(), 
                                         "Cannot get root POA! Exception received: " +
                                         cdmw.orbsupport.ORBSupport.exceptionToString(in),
                                         new Throwable());
           throw new CORBASystemExceptionWrapper(
              new org.omg.CORBA.INTERNAL(
                 ExceptionMinorCodes.INTERNAL,
                 CompletionStatus.COMPLETED_NO),
              in.getMessage());
        } catch(org.omg.CORBA.SystemException se) {
           cdmw.logging.Logger.CDMW_WARN(Get_Logger(), 
                                         "Cannot get root POA! SystemException received: " +
                                         cdmw.orbsupport.ORBSupport.exceptionToString(se),
                                         new Throwable());
           throw new CORBASystemExceptionWrapper(se, se.getMessage());
        }

        return root;

    }

    /**
     * Returns the reference of the CDMW upper most POA, parent of all
     * CDMW-specific POAs.
     *
     * @param orb  A reference to the ORB pseudo-object
     * @return A reference to the CDMW RootPOA pseudo object
     * @exception CORBASystemExceptionWrapper Any wrapped CORBA system exception
     */
    public static POA createCdmwRootPOA(org.omg.CORBA.ORB orb) 
        throws CORBASystemExceptionWrapper {

        //
        // Create the CDMW root POA with its own POA manager.
        // Use Default POA policies:
        //
        // Lifespan policy            = TRANSIENT
        // Id Assignment policy       = SYSTEM_ID
        // Id Uniqueness policy       = UNIQUE_ID
        // Implicit Activation policy = IMPLICIT_ACTIVATION
        // Request Processing policy  = USE_ACTIVE_OBJECT_MAP_ONLY
        // Servant Retention policy   = RETAIN;
        // Thread Policy              = ORB_CTRL_MODEL;
        //

        // Resolve the Root POA. May raise a CORBA::SystemException.
        POA root = getRootPOA(orb);

        PolicyList policies = new PolicyList();
        policies.insert(root.create_id_assignment_policy(
            org.omg.PortableServer.IdAssignmentPolicyValue.SYSTEM_ID));
        policies.insert(root.create_lifespan_policy(
            org.omg.PortableServer.LifespanPolicyValue.TRANSIENT));
        policies.insert(root.create_servant_retention_policy(
            org.omg.PortableServer.ServantRetentionPolicyValue.RETAIN));
        policies.insert(root.create_id_uniqueness_policy(
            org.omg.PortableServer.IdUniquenessPolicyValue.UNIQUE_ID));
        policies.insert(root.create_request_processing_policy(
            org.omg.PortableServer.RequestProcessingPolicyValue.
                USE_ACTIVE_OBJECT_MAP_ONLY));
        policies.insert(root.create_implicit_activation_policy(
            org.omg.PortableServer.ImplicitActivationPolicyValue.
                IMPLICIT_ACTIVATION));
        policies.insert(root.create_thread_policy(
            org.omg.PortableServer.ThreadPolicyValue.ORB_CTRL_MODEL));

        StrategyList poaStrategies = new StrategyList();
        ThreadPoolPolicy threading
            = new ThreadPoolPolicy(CDMW_ROOT_POA_THREAD_POOL_SIZE);
        try {
            threading.appendToStrategyList(poaStrategies);
        } catch(BadParameterException bpe) {
            throw new CORBASystemExceptionWrapper(
                new org.omg.CORBA.BAD_PARAM(
                    ExceptionMinorCodes.BAD_PARAM,
                    CompletionStatus.COMPLETED_NO),
                bpe.what());
        }

        POA cdmwRootPOA = null;
        try {
            cdmwRootPOA = ORBSupport.createPOA(root, 
                ORBSupport.CDMW_ROOT_POA_NAME, null, policies, poaStrategies);
        } catch (org.omg.PortableServer.POAPackage.AdapterAlreadyExists aae) {
           cdmw.logging.Logger.CDMW_WARN(Get_Logger(), 
                                         "POA::AdapterAlreadyExists received: " +
                                         cdmw.orbsupport.ORBSupport.exceptionToString(aae),
                                         new Throwable());
           throw new CORBASystemExceptionWrapper(
              new org.omg.CORBA.BAD_PARAM(
                 ExceptionMinorCodes.BAD_PARAMCdmwRootPOAExists,
                 CompletionStatus.COMPLETED_NO),
              aae.getMessage());
        } catch (org.omg.PortableServer.POAPackage.InvalidPolicy ip) {
           cdmw.logging.Logger.CDMW_WARN(Get_Logger(), 
                                         "POA::InvalidPolicy received: " +
                                         cdmw.orbsupport.ORBSupport.exceptionToString(ip),
                                         new Throwable());
            throw new CORBASystemExceptionWrapper(
                new org.omg.CORBA.INTERNAL(
                    ExceptionMinorCodes.INTERNAL,
                    CompletionStatus.COMPLETED_NO),
                ip.getMessage());
        } catch(org.omg.CORBA.SystemException se) {
           cdmw.logging.Logger.CDMW_WARN(Get_Logger(), 
                                         "CORBA::SystemException received: " +
                                         cdmw.orbsupport.ORBSupport.exceptionToString(se),
                                         new Throwable());
            throw new CORBASystemExceptionWrapper(se, se.getMessage());
        }

        return cdmwRootPOA;

    }

    /**
     * Gets the CDMW Naming and Repository's object reference from
     * the CDMW Application Agent.
     *
     * @return A reference to the CDMW Naming and Repository
     * @exception CORBASystemExceptionWrapper
     * The following CDMW minor codes details tha reason:
     * <UL><LI><B>BAD_PARAMRepositoryNotFound</B> when there is 
     * no CDMW Naming and Repository registered for the current process 
     * within the Process Callback;
     * <LI><B>BAD_PARAMUnknownProcessName</B> when the process name
     * is not known by the Process Callback;
     * <LI><B>BAD_PARAMCannotGetRepositoryReference</B> when an error occurred 
     * during retrieval of the object reference from the Process Callback; and
     * <LI><B>BAD_PARAMInvalidRepositoryReference</B> when the object reference
     * received from the Application Agent is either nil or does not denote 
     * an CDMW Naming and Repository service.</UL>
     */
    public static Repository getCdmwRepository() 
        throws CORBASystemExceptionWrapper {

        // Retreive Repository object reference from the application agent
        Repository rep = null;
        try {
            // May throw ServiceNotFound, ProcessNotFound
            org.omg.CORBA.Object obj = PlatformInterface.getService(
                ServiceNames.NAMING_AND_REPOSITORY_SERVICE);
            rep = RepositoryHelper.narrow(obj);
        } catch(BadOrderException boe) {
           cdmw.logging.Logger.CDMW_WARN(Get_Logger(), 
                                         "cdmw.common.BadOrderException received:" +
                                         boe.what(),
                                         new Throwable());
           throw new CORBASystemExceptionWrapper(
              new org.omg.CORBA.BAD_INV_ORDER(
                 ExceptionMinorCodes.BAD_INV_ORDERInitNotCalled,
                 CompletionStatus.COMPLETED_NO),
              boe.what());
        } catch(BadParameterException bpe) {
           cdmw.logging.Logger.CDMW_WARN(Get_Logger(), 
                                         "cdmw.common.BadParameterException received:" +
                                         bpe.what(),
                                         new Throwable());
            throw new CORBASystemExceptionWrapper(
                new org.omg.CORBA.BAD_PARAM(
                    ExceptionMinorCodes.BAD_PARAM,
                    CompletionStatus.COMPLETED_NO),
                bpe.what());
        } catch (com.thalesgroup.CdmwPlatformMngtService.ServiceNotFound snf) {
           cdmw.logging.Logger.CDMW_WARN(Get_Logger(), 
                                         "com.thalesgroup.CdmwPlatformMngtService.ServiceNotFound received:" +
                                         cdmw.orbsupport.ORBSupport.exceptionToString(snf),
                                         new Throwable());
            // Cannot Proceed with no CDMW Repository Ior
            throw new CORBASystemExceptionWrapper(
                new org.omg.CORBA.BAD_PARAM(
                    ExceptionMinorCodes.BAD_PARAMRepositoryNotFound,
                    CompletionStatus.COMPLETED_NO),
                snf.getMessage());
        } catch (com.thalesgroup.CdmwPlatformMngtService.ServiceUnavailable su) {
           cdmw.logging.Logger.CDMW_WARN(Get_Logger(), 
                                         "com.thalesgroup.CdmwPlatformMngtService.ServiceUnavailable received:" +
                                         cdmw.orbsupport.ORBSupport.exceptionToString(su),
                                         new Throwable());
            // Cannot Proceed with no CDMW Repository Ior
            throw new CORBASystemExceptionWrapper(
                new org.omg.CORBA.BAD_PARAM(
                    ExceptionMinorCodes.BAD_PARAMCannotGetRepositoryReference,
                    CompletionStatus.COMPLETED_NO),
                su.getMessage());
        } catch (org.omg.CORBA.SystemException se) {
           cdmw.logging.Logger.CDMW_WARN(Get_Logger(), 
                                         "SystemException received:" +
                                         cdmw.orbsupport.ORBSupport.exceptionToString(se),
                                         new Throwable());
           // Cannot Proceed with no CDMW Repository Ior
            throw new CORBASystemExceptionWrapper(
                new org.omg.CORBA.BAD_PARAM(
                    ExceptionMinorCodes.BAD_PARAMCannotGetRepositoryReference,
                    CompletionStatus.COMPLETED_NO),
                "Received: " + se.toString());
        }
        if (rep==null) {
           cdmw.logging.Logger.CDMW_WARN(Get_Logger(), 
                                         "Repository is NIL!",
                                         new Throwable());
           throw new CORBASystemExceptionWrapper(
              new org.omg.CORBA.BAD_PARAM(
                 ExceptionMinorCodes.BAD_PARAMInvalidRepositoryReference,
                 CompletionStatus.COMPLETED_NO),
              "nil repository object reference");
        }
        return rep;

    }

    /**
     * Retrieves the CDMW Process name from Program command line arguments.
     *
     * @param args The arguments passed to the program
     * @return The name of current CDMW Process
     * @exception CORBASystemExceptionWrapper
     */
    public static String getCdmwProcessName()
        throws CORBASystemExceptionWrapper {

        try {
            // Get the Process Name
            return PlatformInterface.getProcessName();
        } catch (BadOrderException boe) {
            // Cannot Proceed with no process name
            throw new CORBASystemExceptionWrapper(
                new org.omg.CORBA.BAD_INV_ORDER(
                    ExceptionMinorCodes.BAD_INV_ORDERAckNotDone,
                    CompletionStatus.COMPLETED_NO),
                "Cannot get the CDMW process name :"+ boe.what());
        }

    }
    
    /**
     * Retrieves the CDMW Appcalition name from the PlatformInterface.
     *
     * @return The name of current CDMW Application
     * @exception CORBASystemExceptionWrapper
     */ 
    public static String getCdmwApplicationName()
        throws CORBASystemExceptionWrapper {

        try {
            // Get the Application Name
            return PlatformInterface.getApplicationName();
        } catch (BadOrderException boe) {
            // Cannot Proceed with no application name
            throw new CORBASystemExceptionWrapper(
                new org.omg.CORBA.BAD_INV_ORDER(
                    ExceptionMinorCodes.BAD_INV_ORDERAckNotDone,
                    CompletionStatus.COMPLETED_NO),
                "Cannot get the CDMW application name :" + boe.what());
        }

    }

    /**
     * Gets service reference from the CDMW Application Agent.
     *
     * @param serviceName The name of the service
     * @return A service reference
     * @exception CORBASystemExceptionWrapper
     * The following CDMW minor codes details tha reason:
     * <UL><LI><B>BAD_PARAMInvalidServiceName</B> when there is 
     * no service is registered with such serviceName  
     * within the Process Callback;
     */
    public static org.omg.CORBA.Object getService(String serviceName) 
        throws CORBASystemExceptionWrapper 
    {
        cdmw.logging.Logger.CDMW_DEBUG(Get_Logger(), 
            getCdmwProcessName() + 
            ": calls InitUtils::get_service(" + serviceName + ")",
            new Throwable());
    
        try {
            org.omg.CORBA.Object service = 
                PlatformInterface.getService(serviceName);
            return service;            
        } catch(BadOrderException boe) {
           String reason = "Cannot get the CDMW service : " + serviceName +
              " " + boe.what();
           cdmw.logging.Logger.CDMW_WARN(Get_Logger(), 
                                         reason,
                                         new Throwable());
           throw new CORBASystemExceptionWrapper(
              new org.omg.CORBA.BAD_INV_ORDER(
                 ExceptionMinorCodes.BAD_INV_ORDERAckNotDone,
                 CompletionStatus.COMPLETED_NO),
              reason);
        } catch(BadParameterException bpe) {
           String reason = "Cannot get the CDMW service : " + serviceName +
              " " + bpe.what();
           cdmw.logging.Logger.CDMW_WARN(Get_Logger(), 
                                         reason,
                                         new Throwable());
           throw new CORBASystemExceptionWrapper(
              new org.omg.CORBA.BAD_PARAM(
                 ExceptionMinorCodes.BAD_PARAMInvalidServiceName,
                 CompletionStatus.COMPLETED_NO),
              reason);
        } catch (com.thalesgroup.CdmwPlatformMngtService.ServiceNotFound snf) {
           String reason = "Cannot get the CDMW service : " + serviceName + " => Service not found!";

           cdmw.logging.Logger.CDMW_WARN(Get_Logger(), 
                                         getCdmwProcessName() + reason,
                                         new Throwable());
           // Cannot Proceed with no CDMW Repository Ior
           throw new CORBASystemExceptionWrapper(
              new org.omg.CORBA.BAD_PARAM(
                 ExceptionMinorCodes.BAD_PARAMInvalidServiceName,
                 CompletionStatus.COMPLETED_NO),
              reason + snf.getMessage());
        } catch (com.thalesgroup.CdmwPlatformMngtService.ServiceUnavailable su) {
           String reason = "Cannot get the CDMW service : " + serviceName + " => Service not available!";
           cdmw.logging.Logger.CDMW_WARN(Get_Logger(), 
                                         getCdmwProcessName() + reason,
                                         new Throwable());
            // Cannot Proceed with no CDMW Repository Ior
            throw new CORBASystemExceptionWrapper(
                new org.omg.CORBA.BAD_PARAM(
                    ExceptionMinorCodes.BAD_PARAMInvalidServiceName,
                    CompletionStatus.COMPLETED_NO),
                reason + su.getMessage());
        } catch (org.omg.CORBA.SystemException se) {
           String reason = "Cannot get the CDMW service : " + serviceName
              + " " + cdmw.orbsupport.ORBSupport.exceptionToString(se);
           cdmw.logging.Logger.CDMW_WARN(Get_Logger(), 
                                         getCdmwProcessName() + " " + reason,
                                         new Throwable());
           // Cannot Proceed with no CDMW Repository Ior
           throw new CORBASystemExceptionWrapper(
              new org.omg.CORBA.INTERNAL(
                 ExceptionMinorCodes.INTERNAL,
                 CompletionStatus.COMPLETED_NO),
              reason );
        }
    }

    /**
     * Retrieves the CDMW Process XML initialisation file name
     * from Program command line arguments.
     *
     * @param args The arguments passed to the program
     * @return The name of current CDMW Process XML initialisation file name
     * @exception CORBASystemExceptionWrapper Invalid parameter.
     * The following CDMW minor codes details tha reason:
     * <B>BAD_PARAMProcessXMLFileNameMissing</B> when the CDMW Process XML 
     * initialisation file name is missing from the command line arguments
     */
    public static String getXmlInitialisationFile(String[] args)
        throws CORBASystemExceptionWrapper {

        // Get the XML file Name
        String filename = OS.getOptionValue(args,
            Options.PROCESS_XML_FILE_OPTION);

        if ( filename.equals("no") || filename.equals("yes") ) {
            // Cannot proceed with no file name
            throw new CORBASystemExceptionWrapper(
                new org.omg.CORBA.BAD_PARAM(
                    ExceptionMinorCodes.BAD_PARAMProcessXMLFileNameMissing,
                    CompletionStatus.COMPLETED_NO),
                "Cannot proceed with no file name");
        }

        return filename;

    }

    /**
     * Retrieves the CDMW Process XML initialisation file validation
     * option. It instructs the XML parser whether to validate
     * the XML document or not.
     * This will return <B>true</B> when XML document is validated whenever
     * VALIDATE_XML_OPTION appears in the argument list.
     *
     * @param args The arguments passed to the program
     * @return <B>true</B> if the XML document has to be validated
     * by the XML Parser, <B>false</B> otherwise.
     */
    public static boolean getXmlValidationOption(String[] args) {

        // XML document is validated whenever VALIDATE_XML_OPTION appears 
        // in the argument list.
        boolean validate = false;

        // Get the XML file Name
        String validateXmlOption =
            OS.getOptionValue( args, Options.VALIDATE_XML_OPTION);

        if ( validateXmlOption.equals("yes")
            || !validateXmlOption.equals("no") ) {
            validate = true;
        }

        return validate;

    }



    /**
     * Creates the CDMW LifeCycleFramework POA.
     *
     * @param parent  A reference to the Parent POA
     * @return A reference to the LifeCycleFramework Portable Object Adapter
     * @exception CORBASystemExceptionWrapper
     * @exception CORBA::BAD_PARAM Invalid parameter.
     * The following CDMW minor codes details the reason:
     * <UL><LI><B>BAD_PARAMLifeCycleFrameworkPOAExists</B> indicates that a POA
     * with the same name as the LifeCycle POA already exists.
     * <LI><B>INTERNALLifeCycleFrameworkError</B> indicates that the LifeCycle 
     * framework has choosen a wrong policy for its POA.
     * </UL>
     */
    public static POA createCdmwLifeCyclePOA(POA parent)
        throws CORBASystemExceptionWrapper {

        Assert.check(parent!=null);

        POA factoriesPOA = null;

        // Create CDMW Factories POA (persistent POA)
        try {

            org.omg.PortableServer.POAManager poaManager 
                = parent.the_POAManager();

            PolicyList policies = new PolicyList();
            policies.insert(parent.create_id_assignment_policy(
                org.omg.PortableServer.IdAssignmentPolicyValue.USER_ID));
            policies.insert(parent.create_lifespan_policy(
                org.omg.PortableServer.LifespanPolicyValue.PERSISTENT));
            policies.insert(parent.create_servant_retention_policy(
                org.omg.PortableServer.ServantRetentionPolicyValue.RETAIN));
            policies.insert(parent.create_id_uniqueness_policy(
                org.omg.PortableServer.IdUniquenessPolicyValue.UNIQUE_ID));
            policies.insert(parent.create_request_processing_policy(
                org.omg.PortableServer.RequestProcessingPolicyValue.
                    USE_SERVANT_MANAGER));
            policies.insert(parent.create_implicit_activation_policy(
                org.omg.PortableServer.ImplicitActivationPolicyValue.
                    NO_IMPLICIT_ACTIVATION));

            StrategyList poaStrategies = new StrategyList();
            ThreadPoolPolicy threading
                = new ThreadPoolPolicy(FACTORIES_POA_THREAD_POOL_SIZE);
            try {
                threading.appendToStrategyList(poaStrategies);
            } catch(BadParameterException bpe) {
                throw new CORBASystemExceptionWrapper(
                    new org.omg.CORBA.BAD_PARAM(
                        ExceptionMinorCodes.BAD_PARAM,
                        CompletionStatus.COMPLETED_NO),
                    bpe.what());
            }

            factoriesPOA = ORBSupport.createPOA(
                parent,FACTORIES_POA_NAME,poaManager,policies,poaStrategies);
            //
            // Create and install servant activator
            //
            /*org.omg.PortableServer.ServantActivator theActivator 
                = (org.omg.PortableServer.ServantActivator) new CDMWInitServantActivator();
                */
            factoriesPOA.set_servant_manager(new CDMWInitServantActivator());

        } catch (org.omg.PortableServer.POAPackage.AdapterAlreadyExists aae) {
            // Raised by create_POA
            throw new CORBASystemExceptionWrapper(
                new org.omg.CORBA.BAD_PARAM(
                    ExceptionMinorCodes.BAD_PARAMLifeCycleFrameworkPOAExists,
                    CompletionStatus.COMPLETED_NO),
                aae.getMessage());
        } catch (org.omg.PortableServer.POAPackage.InvalidPolicy ip) {
            // Raised by create_POA
            java.lang.System.out.println(ip + " index="+ ip.index+"  message="+ip.getMessage());
            java.lang.System.out.println("Printing Stack Trave");
            ip.printStackTrace(); 
            throw new CORBASystemExceptionWrapper(
                new org.omg.CORBA.INTERNAL(
                    ExceptionMinorCodes.INTERNALLifeCycleFrameworkError,
                    CompletionStatus.COMPLETED_NO),
                ip.getMessage());
        } catch (org.omg.PortableServer.POAPackage.WrongPolicy wp) {
            // Raised by set_servant_manager
            throw new CORBASystemExceptionWrapper(
                new org.omg.CORBA.INTERNAL(
                    ExceptionMinorCodes.INTERNALLifeCycleFrameworkError,
                    CompletionStatus.COMPLETED_NO),
                wp.getMessage());
        } catch(org.omg.CORBA.SystemException se) {
            throw new CORBASystemExceptionWrapper(se, se.getMessage());
        }

        return factoriesPOA;

    }

}

