/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO 
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2009. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

package cdmw.ft.watchdog;

import org.omg.PortableInterceptor.ClientRequestInfo;
import org.omg.PortableInterceptor.ForwardRequest;
import org.omg.CORBA.*;

import cdmw.ft.FTLogger;
import cdmw.logging.FunctionLogger;
import cdmw.orbsupport.ExceptionMinorCodes;
import static cdmw.logging.FunctionLogger.*;
import static cdmw.logging.Logger.*;

/**
 * Purpose: <p> This class implements the ClientRequestInterceptor class.
 * 
 * Features: <p> Thread safe
 * 
 */
public class FTRequestTimeoutClientRequestInterceptorImpl extends
    org.omg.CORBA.LocalObject implements
    org.omg.PortableInterceptor.ClientRequestInterceptor {
    // current object request id slot id
    private int requestIdSlotId;

    private static final String FT_CLIENT_REQUEST_INTERCEPTOR_NAME = "FTRequestTimeoutClientRequestInterceptor";
    private final String FORBIDDEN_OPERATION = "add_log";

    /**
     * Purpose: <p> Constructor
     * 
     * @param info
     *            ORB init information
     * @param requestIdSlotId
     *            slot id used to store the date of the outgoing request.
     * 
     */
    public FTRequestTimeoutClientRequestInterceptorImpl(
        org.omg.PortableInterceptor.ORBInitInfo info, int requestIdSlotId) {
        this.requestIdSlotId = requestIdSlotId;
        CDMW_LOG_FUNCTION(FTLogger.GetLogger(), 
            " request_id_slot_id=" + requestIdSlotId);
    }

    /**
     * Purpose: <p> When an exception occurs, this interception point is called.
     * It allows an Interceptor to query the exception's information before it
     * is raised to the client.
     * 
     * @param ri
     *            Client request information
     */
    public void receive_exception(ClientRequestInfo ri) throws ForwardRequest,
        SystemException {
        if (ri.operation().compareTo(FORBIDDEN_OPERATION) == 0)
        {
            return;
        }
        // The log below must not be recorded if the intercepted operation is add_log(). Else if Log Collector
        // is used we enter in a cyclic recording of the log!
        // This explains why the macro is called only after the condition if() 
        CDMW_LOG_FUNCTION(FTLogger.GetLogger(),
            " for operation: " + ri.operation());

        try {
            // Get request_id
            int requestId = 0;
            Any any = ri.get_slot(requestIdSlotId);

            TypeCode tc = any.type();

            if (tc.kind().value() != TCKind._tk_ulong) {

                if (tc.kind().value() == TCKind._tk_null) {
                    // the request invocation is done in the context of an
                    // incoming
                    // request processing.
                    CDMW_DEBUG(FTLogger.GetLogger(),
                        "request_id_slot is not set", new Throwable());
                }
                else {
                    CDMW_ERROR(FTLogger.GetLogger(),
                        "Unexpected type stored in request_id_slot",
                        new Throwable());
                    org.omg.CORBA.INTERNAL ex = new org.omg.CORBA.INTERNAL(
                        ExceptionMinorCodes.INTERNALFaultToleranceError,
                        org.omg.CORBA.CompletionStatus.COMPLETED_NO);
                    CDMW_LOG_FUNCTION_EXCEPTION(FTLogger.GetLogger(), ex);
                    throw ex;
                }
            }
            else {
                requestId = any.extract_ulong();
                CDMW_DEBUG(FTLogger.GetLogger(),
                    "Calling remove_waiting_response for request_id="
                        + requestId, new Throwable());
                RequestTimeoutLogger.getInstance().removeWaitingResponse(
                    requestId);
            }

        }
        catch (org.omg.PortableInterceptor.InvalidSlot e) {
            // This slot that has not been allocated
            CDMW_ERROR(FTLogger.GetLogger(),
                "ERROR: InvalidSlot exception raised!", new Throwable());
            org.omg.CORBA.INTERNAL ex = new org.omg.CORBA.INTERNAL(
                ExceptionMinorCodes.INTERNALFaultToleranceError,
                org.omg.CORBA.CompletionStatus.COMPLETED_NO);
            CDMW_LOG_FUNCTION_EXCEPTION(FTLogger.GetLogger(), ex);
            throw ex;
        }
        catch (cdmw.common.NotFoundException e) {
            // The request id is not found
            org.omg.CORBA.INTERNAL ex = new org.omg.CORBA.INTERNAL(
                ExceptionMinorCodes.INTERNALFaultToleranceError,
                org.omg.CORBA.CompletionStatus.COMPLETED_NO);
            CDMW_LOG_FUNCTION_EXCEPTION(FTLogger.GetLogger(), ex);
            throw ex;
        }
        catch (SystemException ex) {
            CDMW_DEBUG(FTLogger.GetLogger(),
                "CORBA System exception raised in receive_exception!\n"
                    + ex.getMessage(), new Throwable());
            CDMW_LOG_FUNCTION_EXCEPTION(FTLogger.GetLogger(), ex);
            throw ex;
        }
    }

    /**
     * Purpose: <p> This interception point allows an Interceptor to query the
     * information available when a request results in something other than a
     * normal reply or an exception. For example, a request could result in a
     * retry (for example, a GIOP Reply with a LOCATION_FORWARD status was
     * received); or on asynchronous calls, the reply does not immediately
     * follow the request, but control shall return to the client and an ending
     * interception point shall be called.
     * 
     * @param ri
     *            Client request information
     */
    public void receive_other(ClientRequestInfo ri) throws ForwardRequest,
        SystemException {
        if (ri.operation().compareTo(FORBIDDEN_OPERATION) == 0)
        {
            return;
        }
        // The log below must not be recorded if the intercepted operation is add_log(). Else if Log Collector
        // is used we enter in a cyclic recording of the log!
        // This explains why the macro is called only after the condition if() 
        CDMW_LOG_FUNCTION(FTLogger.GetLogger(),
            " for operation: " + ri.operation());

        try {
            // Get request_id
            int requestId;
            Any any = ri.get_slot(requestIdSlotId);

            TypeCode tc = any.type();

            if (tc.kind().value() != TCKind._tk_ulong) {

                if (tc.kind().value() == TCKind._tk_null) {
                    // the request invocation is done in the context of an
                    // incoming request processing.
                    CDMW_DEBUG(FTLogger.GetLogger(),
                        "request_id_slot is not set", new Throwable());
                }
                else {
                    CDMW_ERROR(FTLogger.GetLogger(),
                        "Unexpected type stored in request_id_slot",
                        new Throwable());
                    org.omg.CORBA.INTERNAL ex = new org.omg.CORBA.INTERNAL(
                        ExceptionMinorCodes.INTERNALFaultToleranceError,
                        org.omg.CORBA.CompletionStatus.COMPLETED_NO);
                    CDMW_LOG_FUNCTION_EXCEPTION(FTLogger.GetLogger(), ex);
                    throw ex;
                }
            }
            else {
                requestId = any.extract_ulong();
                CDMW_DEBUG(FTLogger.GetLogger(),
                    "Calling remove_waiting_response for request_id="
                        + requestId, new Throwable());
                RequestTimeoutLogger.getInstance().removeWaitingResponse(
                    requestId);
            }

        }
        catch (org.omg.PortableInterceptor.InvalidSlot e) {
            // This slot that has not been allocated
            CDMW_ERROR(FTLogger.GetLogger(),
                "ERROR: InvalidSlot exception raised!", new Throwable());
            org.omg.CORBA.INTERNAL ex = new org.omg.CORBA.INTERNAL(
                ExceptionMinorCodes.INTERNALFaultToleranceError,
                org.omg.CORBA.CompletionStatus.COMPLETED_NO);
            CDMW_LOG_FUNCTION_EXCEPTION(FTLogger.GetLogger(), ex);
            throw ex;
        }
        catch (cdmw.common.NotFoundException e) {
            // The request id is not found
            org.omg.CORBA.INTERNAL ex = new org.omg.CORBA.INTERNAL(
                ExceptionMinorCodes.INTERNALFaultToleranceError,
                org.omg.CORBA.CompletionStatus.COMPLETED_NO);
            CDMW_LOG_FUNCTION_EXCEPTION(FTLogger.GetLogger(), ex);
            throw ex;
        }
        catch (SystemException ex) {
            CDMW_DEBUG(FTLogger.GetLogger(),
                "CORBA System exception raised in receive_other!\n"
                    + ex.getMessage(), new Throwable());
            CDMW_LOG_FUNCTION_EXCEPTION(FTLogger.GetLogger(), ex);
            throw ex;
        }
    }

    /**
     * Purpose: <p> This interception point allows an Interceptor to query the
     * information on a reply after it is returned from the server and before
     * control is returned to the client. Here the date of the outgoing request
     * is reset and T0 of incoming request updated to remove the duration of the
     * outgoing request.
     * 
     * @param ri
     *            Client request information
     */
    public void receive_reply(ClientRequestInfo ri)
        throws org.omg.CORBA.SystemException {
        if (ri.operation().compareTo(FORBIDDEN_OPERATION) == 0)
        {
            return;
        }
        // The log below must not be recorded if the intercepted operation is add_log(). Else if Log Collector
        // is used we enter in a cyclic recording of the log!
        // This explains why the macro is called only after the condition if() 
        CDMW_LOG_FUNCTION(FTLogger.GetLogger(),
            " for operation: " + ri.operation());

        try {
            // Get request_id
            int requestId;
            Any slotData = ri.get_slot(this.requestIdSlotId);

            TypeCode tc = slotData.type();

            if (tc.kind().value() != TCKind._tk_ulong) {

                if (tc.kind().value() == TCKind._tk_null) {
                    // the request invocation is not the result of an incoming
                    // request
                    // processing.
                    CDMW_DEBUG(FTLogger.GetLogger(),
                        "request_id_slot is not set", new Throwable());
                }
                else {
                    CDMW_ERROR(FTLogger.GetLogger(),
                        "Unexpected type stored in request_id_slot",
                        new Throwable());
                    org.omg.CORBA.INTERNAL ex = new org.omg.CORBA.INTERNAL(
                        ExceptionMinorCodes.INTERNALFaultToleranceError,
                        org.omg.CORBA.CompletionStatus.COMPLETED_NO);
                    CDMW_LOG_FUNCTION_EXCEPTION(FTLogger.GetLogger(), ex);
                    throw ex;
                }
            }
            else {
                requestId = slotData.extract_ulong();
                CDMW_DEBUG(FTLogger.GetLogger(),
                    "Calling remove_waiting_response for request_id="
                        + requestId, new Throwable());
                RequestTimeoutLogger.getInstance().removeWaitingResponse(
                    requestId);
            }

        }
        catch (org.omg.PortableInterceptor.InvalidSlot e) {
            // This slot that has not been allocated
            CDMW_ERROR(FTLogger.GetLogger(),
                "ERROR: InvalidSlot exception raised!", new Throwable());
            org.omg.CORBA.INTERNAL ex = new org.omg.CORBA.INTERNAL(
                ExceptionMinorCodes.INTERNALFaultToleranceError,
                org.omg.CORBA.CompletionStatus.COMPLETED_NO);
            CDMW_LOG_FUNCTION_EXCEPTION(FTLogger.GetLogger(), ex);
            throw ex;
        }
        catch (cdmw.common.NotFoundException e) {
            // The request id is not found
            org.omg.CORBA.INTERNAL ex = new org.omg.CORBA.INTERNAL(
                ExceptionMinorCodes.INTERNALFaultToleranceError,
                org.omg.CORBA.CompletionStatus.COMPLETED_NO);
            CDMW_LOG_FUNCTION_EXCEPTION(FTLogger.GetLogger(), ex);
            throw ex;
        }
        catch (SystemException ex) {
            CDMW_DEBUG(FTLogger.GetLogger(),
                "CORBA System exception raised in receive_reply!\n"
                    + ex.getMessage(), new Throwable());
            CDMW_LOG_FUNCTION_EXCEPTION(FTLogger.GetLogger(), ex);
            throw ex;
        }

    }

    /**
     * Purpose: <p> This interception point allows an Interceptor to query
     * information during a Time-Independent Invocation (TII) polling get reply
     * sequence.
     * 
     * @param ri
     *            Client request information
     * 
     */
    public void send_poll(ClientRequestInfo ri)
        throws org.omg.CORBA.SystemException {
        if (ri.operation().compareTo(FORBIDDEN_OPERATION) == 0)
        {
            return;
        }
        // The log below must not be recorded if the intercepted operation is add_log(). Else if Log Collector
        // is used we enter in a cyclic recording of the log!
        // This explains why the macro is called only after the condition if()
        CDMW_LOG_FUNCTION(FTLogger.GetLogger(), 
            " for operation: " + ri.operation());
    }

    /**
     * Purpose: <p> This interception point allows an Interceptor to query
     * request information and modify the service context before the request is
     * sent to the server. Here, the date of the outgoing request is stored
     * here.
     * 
     * @param ri
     *            Client request information
     * 
     */
    public void send_request(ClientRequestInfo ri) throws ForwardRequest,
        org.omg.CORBA.SystemException {
        if ( ri.operation().compareTo(FORBIDDEN_OPERATION) == 0)
        {
            return;
        }
        // The log below must not be recorded if the intercepted operation is add_log(). Else if Log Collector
        // is used we enter in a cyclic recording of the log!
        // This explains why the macro is called only after the condition if()
        CDMW_LOG_FUNCTION(FTLogger.GetLogger(),
            " for operation: " + ri.operation());

        try {
            // Get requestId
            int requestId;
            Any slotData = ri.get_slot(this.requestIdSlotId);

            TypeCode tc = slotData.type();

            if (tc.kind().value() != TCKind._tk_ulong) {

                if (tc.kind().value() == TCKind._tk_null) {
                    // the request invocation is done in the context of an
                    // incoming
                    // request processing.
                    CDMW_DEBUG(FTLogger.GetLogger(),
                        "request_id_slot is not set", new Throwable());
                }
                else {
                    CDMW_ERROR(FTLogger.GetLogger(), "Unexpected type stored in request_id_slot",new Throwable());
                    org.omg.CORBA.INTERNAL ex = new org.omg.CORBA.INTERNAL(
                        ExceptionMinorCodes.INTERNALFaultToleranceError,
                        org.omg.CORBA.CompletionStatus.COMPLETED_NO);
                    CDMW_LOG_FUNCTION_EXCEPTION(FTLogger.GetLogger(), ex);
                    throw ex;
                }
            }
            else {
                requestId = slotData.extract_ulong();
                CDMW_DEBUG(FTLogger.GetLogger(),
                    "Calling waiting_response for request_id=" + requestId,
                    new Throwable());
                RequestTimeoutLogger.getInstance().waitingResponse(requestId);
            }
        }
        catch (org.omg.PortableInterceptor.InvalidSlot e) {
            // This slot that has not been allocated
            CDMW_ERROR(FTLogger.GetLogger(),
                "ERROR: InvalidSlot exception raised!", new Throwable());
            org.omg.CORBA.INTERNAL ex = new org.omg.CORBA.INTERNAL(
                ExceptionMinorCodes.INTERNALFaultToleranceError,
                org.omg.CORBA.CompletionStatus.COMPLETED_NO);
            CDMW_LOG_FUNCTION_EXCEPTION(FTLogger.GetLogger(), ex);
            throw ex;
        }
        catch (cdmw.common.NotFoundException e) {
            // The request id is not found
            org.omg.CORBA.INTERNAL ex = new org.omg.CORBA.INTERNAL(
                ExceptionMinorCodes.INTERNALFaultToleranceError,
                org.omg.CORBA.CompletionStatus.COMPLETED_NO);
            CDMW_LOG_FUNCTION_EXCEPTION(FTLogger.GetLogger(), ex);
            throw ex;
        }
        catch (SystemException ex) {
            CDMW_DEBUG(FTLogger.GetLogger(),
                "CORBA System exception raised in send_request!\n"
                    + ex.getMessage(), new Throwable());
            CDMW_LOG_FUNCTION_EXCEPTION(FTLogger.GetLogger(), ex);
            throw ex;
        }
    }

    /**
     * Purpose: <p> Destroy the interceptor
     * 
     */
    public void destroy() throws org.omg.CORBA.SystemException {
        CDMW_LOG_FUNCTION(FTLogger.GetLogger(), "");
    }

    /**
     * Purpose: <p> Return the name of the interceptor
     * 
     */
    public String name() throws org.omg.CORBA.SystemException {
        CDMW_LOG_FUNCTION(FTLogger.GetLogger(), "");
        CDMW_LOG_FUNCTION_RETURN(FTLogger.GetLogger(),
            FT_CLIENT_REQUEST_INTERCEPTOR_NAME);
        return FT_CLIENT_REQUEST_INTERCEPTOR_NAME;
    }
}
