/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO 
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2009. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

package cdmw.ft.watchdog;

import org.omg.CORBA.SystemException;
import org.omg.CORBA.TCKind;
import org.omg.CORBA.TypeCode;
import org.omg.PortableInterceptor.*;

import cdmw.ft.*;
import cdmw.orbsupport.*;
import static cdmw.logging.Logger.*;
import static cdmw.logging.FunctionLogger.*;

/**
 * Purpose: <p> This class implements the ServerRequestInterceptor class.
 * 
 * Features: <p> Thread safe
 * 
 */
public class FTRequestTimeoutServerRequestInterceptorImpl extends
    org.omg.CORBA.LocalObject implements
    org.omg.PortableInterceptor.ServerRequestInterceptor {

    private static final String FT_SERVER_REQUEST_INTERCEPTOR_NAME = "FTRequestTimeoutServerRequestInterceptor";

    // This counter is used to give a single id for each incoming request
    // This is what is store in m_request_id_slot_id
    private static int requestCounter = 0;

    // slot id used to store request id
    private int requestIdSlotId;

    private org.omg.CORBA.ORB m_orb = null;

    /**
     * Purpose: <p> Constructor
     * 
     * @param info
     *            ORB init information
     * @param request_id_slot_id
     *            slot id used to store request id of the incoming request.
     * 
     */
    public FTRequestTimeoutServerRequestInterceptorImpl(
        org.omg.PortableInterceptor.ORBInitInfo info, int requestIdSlotId) {
        m_orb = org.omg.CORBA.ORB.init();
        this.requestIdSlotId = requestIdSlotId;
        CDMW_LOG_FUNCTION(FTLogger.GetLogger(), " request_id_slot_id=" + requestIdSlotId);
    }

    /**
     * Purpose: <p> This interception point allows an Interceptor to query
     * request information after all the information, including operation
     * parameters, are available.
     * 
     * @param ri
     *            Server request information
     */
    public void receive_request(ServerRequestInfo arg0) throws ForwardRequest,
        SystemException {
        CDMW_LOG_FUNCTION(FTLogger.GetLogger(), " for operation: " + arg0.operation());
    }

    /**
     * Purpose: <p> At this interception point, Interceptors must get their
     * service context information from the incoming request and transfer it to
     * PortableInterceptor.Current's slots.
     * 
     * @param ri
     *            Server request information
     */
    public void receive_request_service_contexts(
        ServerRequestInfo reqInfo) throws ForwardRequest, SystemException {

        CDMW_LOG_FUNCTION(FTLogger.GetLogger(), " for operation: " + reqInfo.operation());

        int requestCounter = 0;
        // Increment here M_request_counter because we are sure the request will
        // be treated at application level.
        synchronized(this)
        {
            FTRequestTimeoutServerRequestInterceptorImpl.requestCounter++;
            requestCounter = FTRequestTimeoutServerRequestInterceptorImpl.requestCounter;
        }
        org.omg.CORBA.Any slotData = null;
        try {
            slotData = reqInfo.get_slot(this.requestIdSlotId);
            slotData.insert_ulong(requestCounter);

            CDMW_DEBUG(FTLogger.GetLogger(),
                "receive_request_service_contexts set_slot m_request_id_slot_id="
                    + this.requestIdSlotId, new Throwable());
            reqInfo.set_slot(this.requestIdSlotId, slotData);
        }
        catch (org.omg.PortableInterceptor.InvalidSlot e) {
            // This slot that has not been allocated
            CDMW_ERROR(FTLogger.GetLogger(),
                "ERROR: InvalidSlot exception raised!", new Throwable());
            org.omg.CORBA.INTERNAL ex = new org.omg.CORBA.INTERNAL(
                ExceptionMinorCodes.INTERNALFaultToleranceError,
                org.omg.CORBA.CompletionStatus.COMPLETED_NO);
            CDMW_LOG_FUNCTION_EXCEPTION(FTLogger.GetLogger(), ex);
            throw ex;
        }

        CDMW_DEBUG(FTLogger.GetLogger(), "receive_request_service_contexts logs " + requestCounter,
            new Throwable());

        // log T0 of the request treatement
        RequestTimeoutLogger.getInstance().enteringRequest(requestCounter);

    }

    /**
     * Purpose: <p> When an exception occurs, this interception point is called.
     * It allows an Interceptor to query the exception information and modify
     * the reply service context before the exception is raised to the client.
     * 
     * @param ri
     *            Server request information
     */
    public void send_exception(ServerRequestInfo arg0) throws ForwardRequest,
        SystemException {
        CDMW_LOG_FUNCTION(FTLogger.GetLogger(),
            " for operation: " + arg0.operation());
        this.removeCurrentTimeoutRequestLog(arg0);
    }

    /**
     * Purpose: <p> This interception point allows an Interceptor to query the
     * information available when a request results in something other than a
     * normal reply or an exception. A request could result in a retry (for
     * example, a GIOP Reply with a LOCATION_FORWARD status was received).
     * 
     * @param ri
     *            Server request information
     */
    public void send_other(ServerRequestInfo reqInfo) throws ForwardRequest,
        SystemException {
        CDMW_LOG_FUNCTION(FTLogger.GetLogger(), " for operation: " + reqInfo.operation());
        this.removeCurrentTimeoutRequestLog(reqInfo);
    }

    /**
     * Purpose: <p> This interception point allows an Interceptor to query reply
     * information and modify the reply service context after the target
     * operation has been invoked and before the reply is returned to the
     * client.
     * 
     * @param ri
     *            Server request information
     */
    public void send_reply(ServerRequestInfo reqInfo) {
        CDMW_LOG_FUNCTION(FTLogger.GetLogger(),  " for operation: " + reqInfo.operation());
        this.removeCurrentTimeoutRequestLog(reqInfo);
    }

    private void removeCurrentTimeoutRequestLog(ServerRequestInfo reqInfo)
        throws SystemException {
        CDMW_LOG_FUNCTION(FTLogger.GetLogger(), " for operation: " + reqInfo.operation());

        try {
            CDMW_DEBUG(FTLogger.GetLogger(), "get_slot m_request_id_slot_id=" 
                + this.requestIdSlotId, new Throwable());
            org.omg.CORBA.Any any = reqInfo.get_slot(requestIdSlotId);

            TypeCode tc = any.type();

            int requestId = 0;
            if ((tc.kind().value() != TCKind._tk_ulong)) {
                CDMW_ERROR(FTLogger.GetLogger(),
                    "ERROR: unexpected data returned by get_slot",
                    new Throwable());
                org.omg.CORBA.INTERNAL ex = new org.omg.CORBA.INTERNAL(
                    ExceptionMinorCodes.INTERNALFaultToleranceError,
                    org.omg.CORBA.CompletionStatus.COMPLETED_NO);
                CDMW_LOG_FUNCTION_EXCEPTION(FTLogger.GetLogger(), ex);
                throw ex;
            }

            requestId = any.extract_ulong();
            
            try {
                RequestTimeoutLogger.getInstance().leavingRequest(requestId);
            }
            catch (cdmw.common.NotFoundException e) {
                CDMW_ERROR(FTLogger.GetLogger(),
                    "ERROR: NotFoundException raised when removing request"
                        + requestId, new Throwable());
                org.omg.CORBA.INTERNAL ex = new org.omg.CORBA.INTERNAL(
                    ExceptionMinorCodes.INTERNALFaultToleranceError,
                    org.omg.CORBA.CompletionStatus.COMPLETED_NO);
                CDMW_LOG_FUNCTION_EXCEPTION(FTLogger.GetLogger(), ex);
                throw ex;
            }
        }
        catch (org.omg.PortableInterceptor.InvalidSlot e) {
            // This slot that has not been allocated
            CDMW_ERROR(FTLogger.GetLogger(),
                "ERROR: InvalidSlot exception raised!", new Throwable());
            org.omg.CORBA.INTERNAL ex = new org.omg.CORBA.INTERNAL(
                ExceptionMinorCodes.INTERNALFaultToleranceError,
                org.omg.CORBA.CompletionStatus.COMPLETED_NO);
            CDMW_LOG_FUNCTION_EXCEPTION(FTLogger.GetLogger(), ex);
            throw ex;
        }

        try {
            // set_slot to empty to avoid mistake if the same thread is used
            // to send request outside an incoming request context.
            org.omg.CORBA.Any slotData = m_orb.create_any();
            reqInfo.set_slot(this.requestIdSlotId, slotData);
        }
        catch (org.omg.PortableInterceptor.InvalidSlot e) {
            // This slot that has not been allocated
            CDMW_ERROR(FTLogger.GetLogger(),
                "ERROR: InvalidSlot exception raised!", new Throwable());
            org.omg.CORBA.INTERNAL ex = new org.omg.CORBA.INTERNAL(
                ExceptionMinorCodes.INTERNALFaultToleranceError,
                org.omg.CORBA.CompletionStatus.COMPLETED_NO);
            CDMW_LOG_FUNCTION_EXCEPTION(FTLogger.GetLogger(), ex);
            throw ex;
        }

    }

    /**
     * Purpose: <p> Destroy the interceptor
     * 
     */
    public void destroy() {
        CDMW_LOG_FUNCTION(FTLogger.GetLogger(), "");
    }

    /**
     * Purpose: <p> Return the name of the interceptor
     * 
     */
    public String name() {
        CDMW_LOG_FUNCTION(FTLogger.GetLogger(), "");
        CDMW_LOG_FUNCTION_RETURN(FTLogger.GetLogger(),
            FT_SERVER_REQUEST_INTERCEPTOR_NAME);

        return FT_SERVER_REQUEST_INTERCEPTOR_NAME;
    }
}
