/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO 
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2009. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

package cdmw.commonsvcs.requestfollowup;

import org.omg.PortableInterceptor.ClientRequestInfo;
//import org.omg.PortableInterceptor.ClientRequestInterceptor;
import org.omg.PortableInterceptor.ForwardRequest;
import org.omg.CORBA.*;
import org.omg.IOP.ServiceContext;

//import  cdmw.logging.LogLevel;
//import cdmw.logging.FunctionLogger;
//import cdmw.orbsupport.ExceptionMinorCodes;
import static cdmw.logging.FunctionLogger.*;
import static cdmw.logging.Logger.*;

/**
 * Purpose: <p> This class implements the ClientRequestInterceptor class.
 * 
 * Features: <p> Thread safe
 * 
 */
public class NDCClientRequestInterceptorImpl extends
    org.omg.CORBA.LocalObject implements
    org.omg.PortableInterceptor.ClientRequestInterceptor {
    
    // current request slot id
    private int m_ndcSlotId = 0;

    private static final String NDC_CLIENT_REQUEST_INTERCEPTOR_NAME = "NDCClientRequestInterceptor";

    /**
     * Purpose: <p> Constructor
     * 
     * @param info
     *            ORB init information
     * @param ndcSlotId
     *            slot id used to store the date of the outgoing request.
     * 
     */
    public NDCClientRequestInterceptorImpl(
        org.omg.PortableInterceptor.ORBInitInfo info, int ndcSlotId) {
        this.m_ndcSlotId = ndcSlotId;
        CDMW_LOG_FUNCTION(NDCLogger.getInstance().getLogger(), "NDCSlotId=" + m_ndcSlotId);
    }

    /**
     * Purpose: <p> When an exception occurs, this interception point is called.
     * It allows an Interceptor to query the exception's information before it
     * is raised to the client.
     * 
     * @param ri
     *            Client request information
     */
    public void receive_exception(ClientRequestInfo ri) throws ForwardRequest,
        SystemException {
        if ( ! NDCLogger.getInstance().isOperationToLog(ri.operation()) ) {
            return;
        }

        // The log below must not be recorded if the intercepted operation is add_log(). Else if Log Collector
        // is used we enter in a cyclic recording of the log (infinite call to add_log)!
        // This explains why the macro is called only after the condition if()
        CDMW_LOG_FUNCTION(NDCLogger.getInstance().getLogger(),
            "Operation=" + ri.operation());

        
        CDMW_LOG(NDCLogger.getInstance().getLogger(), CDMW_DEBUG_3,
            NDCLogger.getInstance().ipToString(NDCLogger.NDC_CLIENT_RECEIVE_EXCEPTION) + " (" + ri.operation() + ")",
            new Throwable());
        logIncomingResponse(NDCLogger.NDC_CLIENT_RECEIVE_EXCEPTION, ri);
    }
        

    /**
     * Purpose: <p> This interception point allows an Interceptor to query the
     * information available when a request results in something other than a
     * normal reply or an exception. For example, a request could result in a
     * retry (for example, a GIOP Reply with a LOCATION_FORWARD status was
     * received); or on asynchronous calls, the reply does not immediately
     * follow the request, but control shall return to the client and an ending
     * interception point shall be called.
     * 
     * @param ri
     *            Client request information
     */
    public void receive_other(ClientRequestInfo ri) throws ForwardRequest,
        SystemException {


        if ( ! NDCLogger.getInstance().isOperationToLog(ri.operation()) ) {
            return;
        }

        // The log below must not be recorded if the intercepted operation is add_log(). Else if Log Collector
        // is used we enter in a cyclic recording of the log (infinite call to add_log)!
        // This explains why the macro is called only after the condition if()
        CDMW_LOG_FUNCTION(NDCLogger.getInstance().getLogger(),
            "Operation=" + ri.operation());
        
        CDMW_LOG(NDCLogger.getInstance().getLogger(), CDMW_DEBUG_3,
            NDCLogger.getInstance().ipToString(NDCLogger.NDC_CLIENT_RECEIVE_OTHER) + " (" + ri.operation() + ")",
            new Throwable());
        logIncomingResponse(NDCLogger.NDC_CLIENT_RECEIVE_OTHER, ri);
    }
        
 
    /**
     * Purpose: <p> This interception point allows an Interceptor to query the
     * information on a reply after it is returned from the server and before
     * control is returned to the client. Here the date of the outgoing request
     * is reset and T0 of incoming request updated to remove the duration of the
     * outgoing request.
     * 
     * @param ri
     *            Client request information
     */
    public void receive_reply(ClientRequestInfo ri)
        throws org.omg.CORBA.SystemException {
        if ( ! NDCLogger.getInstance().isOperationToLog(ri.operation()) ) {
            return;
        }

        // The log below must not be recorded if the intercepted operation is add_log(). Else if Log Collector
        // is used we enter in a cyclic recording of the log (infinite call to add_log)!
        // This explains why the macro is called only after the condition if()
        CDMW_LOG_FUNCTION(NDCLogger.getInstance().getLogger(),
            "Operation=" + ri.operation());
        
       
        CDMW_LOG(NDCLogger.getInstance().getLogger(), CDMW_DEBUG_3,
            NDCLogger.getInstance().ipToString(NDCLogger.NDC_CLIENT_RECEIVE_REPLY) + " (" + ri.operation() + ")",
            new Throwable());
        logIncomingResponse(NDCLogger.NDC_CLIENT_RECEIVE_REPLY, ri);
    }
    
    
    /**
     * Purpose: <p> This interception point allows an Interceptor to query
     * information during a Time-Independent Invocation (TII) polling get reply
     * sequence.
     * 
     * @param ri
     *            Client request information
     * 
     */
    public void send_poll(ClientRequestInfo ri)
        throws org.omg.CORBA.SystemException {
            
        if ( ! NDCLogger.getInstance().isOperationToLog(ri.operation()) ) {
            return;
        }

        // The log below must not be recorded if the intercepted operation is add_log(). Else if Log Collector
        // is used we enter in a cyclic recording of the log (infinite call to add_log)!
        // This explains why the macro is called only after the condition if()
        CDMW_LOG_FUNCTION(NDCLogger.getInstance().getLogger(),
                "Operation=" + ri.operation());
        
        CDMW_LOG(NDCLogger.getInstance().getLogger(), CDMW_DEBUG_3,
            NDCLogger.getInstance().ipToString(NDCLogger.NDC_CLIENT_SEND_POLL) + " (" + ri.operation() + ")",
            new Throwable());
        logOutgoingRequest(NDCLogger.NDC_CLIENT_SEND_POLL, ri);
    }

    /**
     * Purpose: <p> This interception point allows an Interceptor to query
     * request information and modify the service context before the request is
     * sent to the server. Here, the date of the outgoing request is stored
     * here.
     * 
     * @param ri
     *            Client request information
     * 
     */
    public void send_request(ClientRequestInfo ri) throws ForwardRequest,
        org.omg.CORBA.SystemException {
        
        if ( ! NDCLogger.getInstance().isOperationToLog(ri.operation()) ) {
            return;
        }

        // The log below must not be recorded if the intercepted operation is add_log(). Else if Log Collector
        // is used we enter in a cyclic recording of the log (infinite call to add_log)!
        // This explains why the macro is called only after the condition if()
        CDMW_LOG_FUNCTION(NDCLogger.getInstance().getLogger(),
            "Operation=" + ri.operation());
        
        
        CDMW_LOG(NDCLogger.getInstance().getLogger(), CDMW_DEBUG_3,
            NDCLogger.getInstance().ipToString(NDCLogger.NDC_CLIENT_SEND_REQUEST) + " (" + ri.operation() + ")",
            new Throwable());
        logOutgoingRequest(NDCLogger.NDC_CLIENT_SEND_REQUEST, ri);
    }


    public void logOutgoingRequest(int ip, ClientRequestInfo ri) {
        CDMW_LOG_FUNCTION(NDCLogger.getInstance().getLogger(),
            "InterceptionPoint=" + NDCLogger.getInstance().ipToString(ip) +
            ", Operation=" + ri.operation());

        NDCList ndcList = new NDCList();
        
        try {
            // Get NDC Slot data
            CDMW_LOG(NDCLogger.getInstance().getLogger(), CDMW_DEBUG_3,
                        "Get NDCs list from Slot", new Throwable());
            Any any = ri.get_slot(m_ndcSlotId);

            ndcList = NDCFactory.getInstance().fromAny(any);
        }
        catch (org.omg.PortableInterceptor.InvalidSlot ex) {
            CDMW_WARN(NDCLogger.getInstance().getLogger(),
                "Catch PortableInterceptor.InvalidSlot exception - " + ex, new Throwable());
        }
        //  catch (UserException ex) {
        //      CDMW_WARN(NDCLogger.getInstance().getLogger(),
        //              "Catch CORBA.UserException - " + ex, new Throwable());
        //  }
        catch (SystemException ex) {
            CDMW_WARN(NDCLogger.getInstance().getLogger(),
                "Catch CORBA.SystemException - " + ex, new Throwable());
        }

        // Request Follow Up  not activated for that request
        if ( (ndcList.list.size() == 0) && !(NDCLogger.getInstance().getLogger().isEnabledFor(REQ_FOLLOWUP)) ) {
            CDMW_LOG(NDCLogger.getInstance().getLogger(), CDMW_DEBUG_3,
                "Request Follow Up not activated for that request",
                new Throwable());
            return;
        }

        try {
            CDMW_LOG(NDCLogger.getInstance().getLogger(), CDMW_DEBUG_3,
                "Create a new NDC and add it into the NDCs list.",
                new Throwable());
            NDC ndc = NDCFactory.getInstance().createNewNDC(ri.operation());
            ndcList.list.addLast(ndc);


            // Add NDCs to NDC Service Context of the request
            CDMW_LOG(NDCLogger.getInstance().getLogger(), CDMW_DEBUG_3,
                    "Set Service Context from NDCs list : " + NDCFactory.getInstance().toString(ndcList),
                    new Throwable());
            byte [] data = NDCFactory.getInstance().toOctetSeq(ndcList);
            ServiceContext context =  new ServiceContext(NDCLogger.CDMW_NDC_SERVICE_CONTEXT_ID, data);
            ri.add_request_service_context(context, true);
        }
        catch (SystemException ex) {
            CDMW_WARN(NDCLogger.getInstance().getLogger(),
                "Catch CORBA.SystemException - " + ex, new Throwable());
        }

        // log the Request Follow Up context
        NDCLogger.getInstance().logRequest(ip, ri.operation(), ndcList);

    }



    public void logIncomingResponse(int ip, ClientRequestInfo ri) {
        CDMW_LOG_FUNCTION(NDCLogger.getInstance().getLogger(),
            "InterceptionPoint=" + NDCLogger.getInstance().ipToString(ip) +
            ", Operation=" + ri.operation());

        NDCList ndcList = new NDCList();
        
        try {
            //CDMW_LOG(NDCLogger.getInstance().getLogger(), CDMW_DEBUG_3,
            //            "Get NDC slot: m_ndcSlotId=" +  m_ndcSlotId, new Throwable());
            //Any any = ri.get_slot(m_ndcSlotId);
            //ndcList = NDCFactory.getInstance().fromAny(any);
            // Get NDC Service Context data
            CDMW_LOG(NDCLogger.getInstance().getLogger(), CDMW_DEBUG_3,
                "Get NDCs list from Service Context",
                new Throwable());
            org.omg.IOP.ServiceContext sc =
                ri.get_request_service_context(NDCLogger.CDMW_NDC_SERVICE_CONTEXT_ID);
            byte [] data = sc.context_data;

            ndcList = NDCFactory.getInstance().fromOctetSeq(data);
        }
        catch (BAD_PARAM ex) {
            // This is not an error while a request can come to a single reference
            // with no NDC service context.
            CDMW_LOG(NDCLogger.getInstance().getLogger(), CDMW_DEBUG_3,
                "No NDC Service Context for that request",
                new Throwable());
        }
//        catch (org.omg.PortableInterceptor.InvalidSlot ex) {
//                CDMW_WARN(NDCLogger.getInstance().getLogger(),
//                    "Catch PortableInterceptor.InvalidSlot exception - " + ex, new Throwable());
//        }
//        catch (UserException ex) {
//            CDMW_WARN(NDCLogger.getInstance().getLogger(),
//                "Catch CORBA.UserException - " + ex, new Throwable());
//        }
        catch (SystemException ex) {
            CDMW_WARN(NDCLogger.getInstance().getLogger(),
                    "Catch CORBA.SystemException - " + ex, new Throwable());
        }

        // Request Follow Up  not activated for that request
        if ( (ndcList.list.size() == 0) && !(NDCLogger.getInstance().getLogger().isEnabledFor(REQ_FOLLOWUP)) ) {
            CDMW_LOG(NDCLogger.getInstance().getLogger(), CDMW_DEBUG_3,
                "Request Follow Up not activated for that request",
                new Throwable());
            return;
        }

        if ( ndcList.list.size() == 0 ) {
            CDMW_LOG(NDCLogger.getInstance().getLogger(), CDMW_DEBUG_3,
                "Unexpected data returned by NDC Service Context, NDCs list is empty.",
                new Throwable());
        }

        NDCLogger.getInstance().logRequest(ip, ri.operation(), ndcList);
        
    }


    /**
     * Purpose: <p> Destroy the interceptor
     * 
     */
    public void destroy() throws org.omg.CORBA.SystemException {
        CDMW_LOG_FUNCTION(NDCLogger.getInstance().getLogger(), "");
    }

    /**
     * Purpose: <p> Return the name of the interceptor
     * 
     */
    public String name() throws org.omg.CORBA.SystemException {
        CDMW_LOG_FUNCTION(NDCLogger.getInstance().getLogger(), "");
        CDMW_LOG_FUNCTION_RETURN(NDCLogger.getInstance().getLogger(),
            NDC_CLIENT_REQUEST_INTERCEPTOR_NAME);
        return NDC_CLIENT_REQUEST_INTERCEPTOR_NAME;
    }
}
