/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO 
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2009. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */

package cdmw.commonsvcs.requestfollowup;

import org.omg.CORBA.SystemException;
//import org.omg.CORBA.TCKind;
//import org.omg.CORBA.TypeCode;
import org.omg.CORBA.Any;
import org.omg.PortableInterceptor.ServerRequestInfo;
//import org.omg.PortableInterceptor.ServerRequestInterceptor;
import org.omg.PortableInterceptor.ForwardRequest;
import org.omg.IOP.ServiceContext;
import org.omg.CORBA.BAD_PARAM;

//import cdmw.orbsupport.*;
//import cdmw.logging.FunctionLogger;
//import cdmw.logging.LogLevel;
import static cdmw.logging.Logger.*;
import static cdmw.logging.FunctionLogger.*;

/**
 * Purpose: <p> This class implements the ServerRequestInterceptor class.
 * 
 * Features: <p> Thread safe
 * 
 */
public class NDCServerRequestInterceptorImpl extends
    org.omg.CORBA.LocalObject implements
    org.omg.PortableInterceptor.ServerRequestInterceptor {
    
    // current request slot id
    private int m_ndcSlotId = 0;

    private static final String NDC_SERVER_REQUEST_INTERCEPTOR_NAME = "NDCServerRequestInterceptor";

    /**
     * Purpose: <p> Constructor
     * 
     * @param info
     *            ORB init information
     * @param ndcSlotId
     *            slot id used to store the date of the outgoing request.
     * 
     */
    public NDCServerRequestInterceptorImpl(
        org.omg.PortableInterceptor.ORBInitInfo info, int ndcSlotId) {
        this.m_ndcSlotId = ndcSlotId;
        CDMW_LOG_FUNCTION(NDCLogger.getInstance().getLogger(), 
            "NDCSlotId=" + m_ndcSlotId);
    }


    /**
     * Purpose: <p> Return the name of the interceptor
     * 
     */
    public String name() throws org.omg.CORBA.SystemException {
        CDMW_LOG_FUNCTION(NDCLogger.getInstance().getLogger(), "");
        CDMW_LOG_FUNCTION_RETURN(NDCLogger.getInstance().getLogger(),
            NDC_SERVER_REQUEST_INTERCEPTOR_NAME);
        return NDC_SERVER_REQUEST_INTERCEPTOR_NAME;
    }

    /**
     * Purpose: <p> Destroy the interceptor
     * 
     */
    public void destroy() throws org.omg.CORBA.SystemException {
        CDMW_LOG_FUNCTION(NDCLogger.getInstance().getLogger(),
            "PortableInterceptor=" + name());
    }


    /**
     * Purpose: <p> At this interception point, Interceptors must get their
     * service context information from the incoming request and transfer it to
     * PortableInterceptor.Current's slots.
     * 
     * @param ri
     *            Server request information
     */
    public void receive_request_service_contexts(ServerRequestInfo ri)
        throws ForwardRequest, SystemException {
        CDMW_LOG_FUNCTION(NDCLogger.getInstance().getLogger(),
            "Operation=" + ri.operation());

        if ( ! NDCLogger.getInstance().isOperationToLog(ri.operation()) ) {
            return;
        }
        
        CDMW_LOG(NDCLogger.getInstance().getLogger(), CDMW_DEBUG_3,
            NDCLogger.getInstance().ipToString(NDCLogger.NDC_SERVER_RECEIVE_REQUEST) + " (" + ri.operation() + ")",
            new Throwable());
        logIncomingRequest(NDCLogger.NDC_SERVER_RECEIVE_REQUEST, ri);
    }

    /**
     * Purpose: <p> This interception point allows an Interceptor to query
     * request information after all the information, including operation
     * parameters, are available.
     * 
     * @param ri
     *            Server request information
     */
    public void receive_request(ServerRequestInfo ri)
        throws ForwardRequest, SystemException {
        CDMW_LOG_FUNCTION(NDCLogger.getInstance().getLogger(),
            "Operation=" + ri.operation());
    }


    /**
     * Purpose: <p> This interception point allows an Interceptor to query reply
     * information and modify the reply service context after the target
     * operation has been invoked and before the reply is returned to the
     * Server.
     * 
     * @param ri
     *            Server request information
     */
    public void send_reply(ServerRequestInfo ri) {
        CDMW_LOG_FUNCTION(NDCLogger.getInstance().getLogger(),
            "Operation=" + ri.operation());

        if ( ! NDCLogger.getInstance().isOperationToLog(ri.operation()) ) {
            return;
        }
        
        CDMW_LOG(NDCLogger.getInstance().getLogger(), CDMW_DEBUG_3,
            NDCLogger.getInstance().ipToString(NDCLogger.NDC_SERVER_SEND_REPLY) + " (" + ri.operation() + ")",
            new Throwable());
        logOutgoingRequest(NDCLogger.NDC_SERVER_SEND_REPLY, ri);
    }

    /**
     * Purpose: <p> This interception point allows an Interceptor to query the
     * information available when a request results in something other than a
     * normal reply or an exception. A request could result in a retry (for
     * example, a GIOP Reply with a LOCATION_FORWARD status was received).
     * 
     * @param ri
     *            Server request information
     */
    public void send_other(ServerRequestInfo ri)
        throws ForwardRequest, SystemException {
        CDMW_LOG_FUNCTION(NDCLogger.getInstance().getLogger(),
            "Operation=" + ri.operation());

        if ( ! NDCLogger.getInstance().isOperationToLog(ri.operation()) ) {
            return;
        }
        
        CDMW_LOG(NDCLogger.getInstance().getLogger(), CDMW_DEBUG_3,
            NDCLogger.getInstance().ipToString(NDCLogger.NDC_SERVER_SEND_OTHER) + " (" + ri.operation() + ")",
            new Throwable());
        logOutgoingRequest(NDCLogger.NDC_SERVER_SEND_OTHER, ri);
    }

    /**
     * Purpose: <p> When an exception occurs, this interception point is called.
     * It allows an Interceptor to query the exception information and modify
     * the reply service context before the exception is raised to the Server.
     * 
     * @param ri
     *            Server request information
     */
    public void send_exception(ServerRequestInfo ri)
        throws ForwardRequest, SystemException {
        CDMW_LOG_FUNCTION(NDCLogger.getInstance().getLogger(),
                "Operation=" + ri.operation());

        if ( ! NDCLogger.getInstance().isOperationToLog(ri.operation()) ) {
            return;
        }
        
        CDMW_LOG(NDCLogger.getInstance().getLogger(), CDMW_DEBUG_3,
            NDCLogger.getInstance().ipToString(NDCLogger.NDC_SERVER_SEND_EXECEPTION) + " (" + ri.operation() + ")",
            new Throwable());
        logOutgoingRequest(NDCLogger.NDC_SERVER_SEND_EXECEPTION, ri);
    }


    public void logIncomingRequest(int ip, ServerRequestInfo ri) {
        CDMW_LOG_FUNCTION(NDCLogger.getInstance().getLogger(),
                "InterceptionPoint=" + NDCLogger.getInstance().ipToString(ip) +
                ", Operation=" + ri.operation());

        NDCList ndcList = new NDCList();
        
        try {
            CDMW_LOG(NDCLogger.getInstance().getLogger(), CDMW_DEBUG_3,
                    "Get NDCs list from Service Context",
                    new Throwable());
            ServiceContext context1 =
                ri.get_request_service_context(NDCLogger.CDMW_NDC_SERVICE_CONTEXT_ID);
            byte [] data1 = context1.context_data;
            ndcList = NDCFactory.getInstance().fromOctetSeq(data1);
        }
        catch (BAD_PARAM ex) {
            // This is not an error while a request can come to a single reference
            // with no NDC service context.
            CDMW_LOG(NDCLogger.getInstance().getLogger(), CDMW_DEBUG_3,
                    "No NDC Service Context for that request",
                    new Throwable());
        }
        catch (SystemException ex) {
            CDMW_WARN(NDCLogger.getInstance().getLogger(),
                    "Catch CORBA.SystemException - " + ex, new Throwable());
        }

        // Request Follow Up  not activated for that request
        if ( (ndcList.list.size() == 0) && !(NDCLogger.getInstance().getLogger().isEnabledFor(REQ_FOLLOWUP)) ) {
            CDMW_LOG(NDCLogger.getInstance().getLogger(), CDMW_DEBUG_3,
                    "Request Follow Up not activated for that request",
                    new Throwable());
            return;
        }
    
        try {
            // If the NDC Service Context is empty, the request come from a client 
            // with no Request Follow Up service activated. 
            // We create a new NDC to follow up the request, as the service is 
            // activated on this server. 
            if (ndcList.list.size() == 0) {
                CDMW_LOG(NDCLogger.getInstance().getLogger(), CDMW_DEBUG_3,
                        "Create a new NDC and add it into the NDCs list.",
                        new Throwable());
                NDC ndc = NDCFactory.getInstance().createNewNDC(ri.operation());
                ndcList.list.addLast(ndc);

                // Add NDCs to NDC Service Context of the request
                CDMW_LOG(NDCLogger.getInstance().getLogger(), CDMW_DEBUG_3,
                        "Set Service Context from NDCs list : " + NDCFactory.getInstance().toString(ndcList),
                        new Throwable());
                byte [] data2 = NDCFactory.getInstance().toOctetSeq(ndcList);
                ServiceContext context2 = 
                    new ServiceContext(NDCLogger.CDMW_NDC_SERVICE_CONTEXT_ID, data2);
                ri.add_reply_service_context(context2, true);
            }
            
            // Set NDC Slot Id
            CDMW_LOG(NDCLogger.getInstance().getLogger(), CDMW_DEBUG_3,
                    "Set Slot from NDCs list : " + NDCFactory.getInstance().toString(ndcList),
                    new Throwable());
            Any any = NDCFactory.getInstance().toAny(ndcList);
            ri.set_slot(m_ndcSlotId, any);
        }
        catch (org.omg.PortableInterceptor.InvalidSlot ex) {
            CDMW_WARN(NDCLogger.getInstance().getLogger(),
                   "Catch PortableInterceptor.InvalidSlot exception - " + ex, new Throwable());
        }
        catch (SystemException ex) {
            CDMW_WARN(NDCLogger.getInstance().getLogger(),
                    "Catch CORBA.SystemException - " + ex, new Throwable());
        }


        // log the Request Follow Up context
        NDCLogger.getInstance().logRequest(ip, ri.operation(), ndcList);
        
    }



    public void logOutgoingRequest(int ip, ServerRequestInfo ri) {
        CDMW_LOG_FUNCTION(NDCLogger.getInstance().getLogger(),
            "InterceptionPoint=" + NDCLogger.getInstance().ipToString(ip) +
            ", Operation=" + ri.operation());

        NDCList ndcList = new NDCList();
        
        try {
            CDMW_LOG(NDCLogger.getInstance().getLogger(), CDMW_DEBUG_3,
                        "Get NDCs list from NDC Slot", new Throwable());
            Any any = ri.get_slot(m_ndcSlotId);
            ndcList = NDCFactory.getInstance().fromAny(any);

//            CDMW_LOG(NDCLogger.getInstance().getLogger(), CDMW_DEBUG_3,
//                    "Get NDCs list from Service Context",
//                    new Throwable());
//            org.omg.IOP.ServiceContext sc =
//                ri.get_request_service_context(NDCLogger.CDMW_NDC_SERVICE_CONTEXT_ID);
//            byte [] data = sc.context_data;
//            ndcList = NDCFactory.getInstance().fromOctetSeq(data);
        }
        catch (org.omg.PortableInterceptor.InvalidSlot ex) {
                CDMW_WARN(NDCLogger.getInstance().getLogger(),
                    "Catch PortableInterceptor.InvalidSlot exception - " + ex, new Throwable());
        }
    //  catch (UserException ex) {
    //      CDMW_WARN(NDCLogger.getInstance().getLogger(),
    //              "Catch CORBA.UserException - " + ex, new Throwable());
    //  }
        catch (SystemException ex) {
            CDMW_WARN(NDCLogger.getInstance().getLogger(),
                    "Catch CORBA.SystemException - " + ex, new Throwable());
        }

        // Request Follow Up  not activated for that request
        if ( (ndcList.list.size() == 0) && !(NDCLogger.getInstance().getLogger().isEnabledFor(REQ_FOLLOWUP)) ) {
            CDMW_LOG(NDCLogger.getInstance().getLogger(), CDMW_DEBUG_3,
                "Request Follow Up not activated for that request",
                new Throwable());
            return;
        }

        if ( ndcList.list.size() == 0 ) {
            CDMW_LOG(NDCLogger.getInstance().getLogger(), CDMW_DEBUG_3,
                "Unexpected data returned by NDC Slot, NDCs list is empty.",
                new Throwable());
        }

    // log the Request Follow Up context
    NDCLogger.getInstance().logRequest(ip, ri.operation(), ndcList);
        
    }
}






