/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI. It is derivative work based on PERCO
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2009.
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#ifndef ___TAO_DEBUG_H__
#define ___TAO_DEBUG_H__


// The purpose of this class is to debug refcounters in TAO objects,
// in order to detect memory leaks.
// By default, this class is not compiled. 
// If you want to use it, use -D_USE_TAO_DEBUG_ as compilation flag 
// for orbsupport compilation and for your code
#ifdef _USE_TAO_DEBUG_



#include <tao/corba.h>
#include <tao/ORB_Core.h>
#include <tao/LocalObject.h>
#include <tao/PortableServer/PortableServer.h>
#include <tao/PortableServer/Servant_Base.h>
#include <tao/Valuetype/ValueBase.h>

#include <Foundation/orbsupport/RefCountServantBase.hpp>
#include <Foundation/orbsupport/RefCountLocalObject.hpp>

#include <map>
#include <sstream>


namespace Cdmw
{
namespace OrbSupport
{


    
/*
 * Class allowing to register various refcounted objects, and
 * to get refcount value of each object at any time.
 * As this class is a static singleton pattern, the singleton is destroyed
 * at process end. So at process end the refcounts of all registered objects
 * are printed. If a counter is not 0, there is probably a memory leak.
 */
class TAO_DEBUG 
{

    public:
    
        /*
         * return Singleton object
         */
        static TAO_DEBUG& Instance()
        {
            return M_instance;
        }
    
        /*
         * Add ORB and associated objects to be checked.
         * @param orb origin ORB instance
         */
        void set_ORB(CORBA::ORB_ptr orb);
        
        /*
         * Register a CORBA::Object to be checked.
         * If it's a refcounted local object, it will be casted.
         * Fails if the name is already used.
         *
         * @param name unique identifier set the object
         * @param obj new registered object
         */
        void register_object(const char* name, 
                             CORBA::Object_ptr obj);

        /*
         * Register a Cardamom refcounted local object
         */
        void register_cdmw_refcount_localobj(const char* name, 
                                             RefCountLocalObject* localobj);

        /*
         * Register a TAO refcounted local object
         */
        void register_tao_refcount_localobj(const char* name, 
                                            TAO_Local_RefCounted_Object* localobj);

        /*
         * Register a Cardamom refcounted servant
         */
        void register_cdmw_refcount_servant(const char* name, 
                                            RefCountServantBase* servant);
                                       
        /*
         * Register a TAO refcounted servant
         */
        void register_tao_refcount_servant(const char* name, 
                                           PortableServer::RefCountServantBase* servant);
                                       
        /*
         * Register a valuetype factory
         */
        void register_value_factory(const char* name,
                                    CORBA::ValueFactoryBase* value_factory);
                                    
        /*
         * Register a valuetype
         */
        void register_valuetype(const char* name,
                                CORBA::DefaultValueRefCountBase* valuetype);
                                           
        /*
         * Return refcount for a registered object
         */
        CORBA::ULong get_refcount(const char* name);
                                        
        /*
         * Print refcount for a registered object
         */
        void print_info(const char* name, const char* extra_msg = 0);

        /*
         * Print refcount for all registered object
         */
        void print_all_infos(const char* extra_msg = 0);
    
        
    protected:

        /*
         * Protected constructor (singleton pattern)
         */
        TAO_DEBUG();
        
        /*
         * Protected destructor (singleton pattern)
         */
        virtual ~TAO_DEBUG();
    
    

    private:
        
        /*
         * Structure for a refcounted object
         *   base_ptr is the object address (casted in appropriate refcounted type)
         *   offset is the offset to be used to get refcount.
         */
        struct RefcountPtr {
            void* base_ptr;
            int   offset;
        };
        typedef std::map<std::string, RefcountPtr> RefcountPtrMap;
        
        /*
         * get refcount reading at object address + offset
         */
        CORBA::ULong get_refcount(const RefcountPtr& ptr);
    
        /*
         * Static singleton instance
         */
        static TAO_DEBUG M_instance;
        
        /*
         * Objetcs registration map
         */
        RefcountPtrMap m_refcountptr_map;
        
        
};


/*
 * Help macro for direct singleton instance access
 */
#define TAO_DEBUG_INSTANCE Cdmw::OrbSupport::TAO_DEBUG::Instance()

/*
 * Help macro to print the refcount value of a registred object.
 * Calling file and line are also printed, plus a user defined message.
 */
#define TAO_DEBUG_PRINT(X, Y) \
    do { \
        std::ostringstream __msg__; \
        __msg__ << __FILE__ << " " << __LINE__ << ": " << Y; \
        Cdmw::OrbSupport::TAO_DEBUG::Instance().print_info(X, __msg__.str().c_str()); \
    } while(0)

/*
 * Help macro to print the refcounts values of all registred objects.
 * Calling file and line are also printed, plus a user defined message.
 */
#define TAO_DEBUG_PRINT_ALL(X) \
    do { \
        std::ostringstream __msg__; \
        __msg__ << __FILE__ << " " << __LINE__ << ": " << X; \
        Cdmw::OrbSupport::TAO_DEBUG::Instance().print_all_infos(__msg__.str().c_str()); \
    } while(0)


}
}

#endif  // _USE_TAO_DEBUG_

#endif



